@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,8 @@
 // This file declares path keys for the base module.  These can be used with
 // the PathService to access various special directories and files.
 
-#include "base/basictypes.h"
+#include "build/build_config.h"
+
 #if defined(OS_WIN)
 #include "base/base_paths_win.h"
 #elif defined(OS_MACOSX)
@@ -18,7 +19,7 @@
 
 namespace base {
 
-enum {
+enum BasePathKey {
   PATH_START = 0,
 
   DIR_CURRENT,  // current directory
@@ -39,6 +40,9 @@ enum {
                 // This is $XDG_CACHE_HOME on Linux and
                 // ~/Library/Caches on Mac.
 #endif
+#if defined(OS_ANDROID)
+  DIR_ANDROID_APP_DATA,  // Directory where to put android app's data.
+#endif
 
   PATH_END
 };
