@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,19 +8,20 @@
 
 #include <string>
 
-#include "base/basictypes.h"
+#include "base/base_export.h"
+#include "build/build_config.h"
 
 namespace base {
 
 namespace env_vars {
 
 #if defined(OS_POSIX)
-extern const char kHome[];
+BASE_EXPORT extern const char kHome[];
 #endif
 
 }  // namespace env_vars
 
-class Environment {
+class BASE_EXPORT Environment {
  public:
   virtual ~Environment();
 
