@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,6 +17,7 @@
 //
 // * The encoding need not be specified on POSIX systems, although some
 //   POSIX-compliant systems do specify an encoding.  Mac OS X uses UTF-8.
+//   Chrome OS also uses UTF-8.
 //   Linux does not specify an encoding, but in practice, the locale's
 //   character set may be used.
 //
@@ -100,13 +101,16 @@
 #define BASE_FILE_PATH_H_
 #pragma once
 
+#include <stddef.h>
 #include <string>
 #include <vector>
 
-#include "base/basictypes.h"
+#include "base/base_export.h"
 #include "base/compiler_specific.h"
 #include "base/hash_tables.h"
+#include "base/string16.h"
 #include "base/string_piece.h"  // For implicit conversions.
+#include "build/build_config.h"
 
 // Windows-style drive letter support and pathname separator characters can be
 // enabled and disabled independently, to aid testing.  These #defines are
@@ -121,7 +125,7 @@ class Pickle;
 
 // An abstraction to isolate users from the differences between native
 // pathnames on different platforms.
-class FilePath {
+class BASE_EXPORT FilePath {
  public:
 #if defined(OS_POSIX)
   // On most platforms, native pathnames are char arrays, and the encoding
@@ -202,13 +206,13 @@ class FilePath {
   // only contains one component, returns a FilePath identifying
   // kCurrentDirectory.  If this object already refers to the root directory,
   // returns a FilePath identifying the root directory.
-  FilePath DirName() const;
+  FilePath DirName() const WARN_UNUSED_RESULT;
 
   // Returns a FilePath corresponding to the last path component of this
   // object, either a file or a directory.  If this object already refers to
   // the root directory, returns a FilePath identifying the root directory;
   // this is the only situation in which BaseName will return an absolute path.
-  FilePath BaseName() const;
+  FilePath BaseName() const WARN_UNUSED_RESULT;
 
   // Returns ".jpg" for path "C:\pics\jojo.jpg", or an empty string if
   // the file has no extension.  If non-empty, Extension() will always start
@@ -223,7 +227,7 @@ class FilePath {
   // Returns "C:\pics\jojo" for path "C:\pics\jojo.jpg"
   // NOTE: this is slightly different from the similar file_util implementation
   // which returned simply 'jojo'.
-  FilePath RemoveExtension() const;
+  FilePath RemoveExtension() const WARN_UNUSED_RESULT;
 
   // Inserts |suffix| after the file name portion of |path| but before the
   // extension.  Returns "" if BaseName() == "." or "..".
@@ -232,14 +236,17 @@ class FilePath {
   // path == "jojo.jpg"         suffix == " (1)", returns "jojo (1).jpg"
   // path == "C:\pics\jojo"     suffix == " (1)", returns "C:\pics\jojo (1)"
   // path == "C:\pics.old\jojo" suffix == " (1)", returns "C:\pics.old\jojo (1)"
-  FilePath InsertBeforeExtension(const StringType& suffix) const;
-  FilePath InsertBeforeExtensionASCII(const base::StringPiece& suffix) const;
+  FilePath InsertBeforeExtension(
+      const StringType& suffix) const WARN_UNUSED_RESULT;
+  FilePath InsertBeforeExtensionASCII(
+      const base::StringPiece& suffix) const WARN_UNUSED_RESULT;
 
   // Replaces the extension of |file_name| with |extension|.  If |file_name|
   // does not have an extension, them |extension| is added.  If |extension| is
   // empty, then the extension is removed from |file_name|.
   // Returns "" if BaseName() == "." or "..".
-  FilePath ReplaceExtension(const StringType& extension) const;
+  FilePath ReplaceExtension(
+      const StringType& extension) const WARN_UNUSED_RESULT;
 
   // Returns true if the file path matches the specified extension. The test is
   // case insensitive. Don't forget the leading period if appropriate.
@@ -271,16 +278,42 @@ class FilePath {
 
   // Returns a copy of this FilePath that does not end with a trailing
   // separator.
-  FilePath StripTrailingSeparators() const;
+  FilePath StripTrailingSeparators() const WARN_UNUSED_RESULT;
 
   // Returns true if this FilePath contains any attempt to reference a parent
   // directory (i.e. has a path component that is ".."
   bool ReferencesParent() const;
 
+  // Return a Unicode human-readable version of this path.
+  // Warning: you can *not*, in general, go from a display name back to a real
+  // path.  Only use this when displaying paths to users, not just when you
+  // want to stuff a string16 into some other API.
+  string16 LossyDisplayName() const;
+
+  // Return the path as ASCII, or the empty string if the path is not ASCII.
+  // This should only be used for cases where the FilePath is representing a
+  // known-ASCII filename.
+  std::string MaybeAsASCII() const;
+
+  // Return the path as UTF-8.
+  //
+  // This function is *unsafe* as there is no way to tell what encoding is
+  // used in file names on POSIX systems other than Mac and Chrome OS,
+  // although UTF-8 is practically used everywhere these days. To mitigate
+  // the encoding issue, this function internally calls
+  // SysNativeMBToWide() on POSIX systems other than Mac and Chrome OS,
+  // per assumption that the current locale's encoding is used in file
+  // names, but this isn't a perfect solution.
+  //
+  // Once it becomes safe to to stop caring about non-UTF-8 file names,
+  // the SysNativeMBToWide() hack will be removed from the code, along
+  // with "Unsafe" in the function name.
+  std::string AsUTF8Unsafe() const;
+
   // Older Chromium code assumes that paths are always wstrings.
-  // These functions convert wstrings to/from FilePaths, and are
+  // This function converts wstrings to FilePaths, and is
   // useful to smooth porting that old code to the FilePath API.
-  // They have "Hack" in their names so people feel bad about using them.
+  // It has "Hack" its name so people feel bad about using it.
   // http://code.google.com/p/chromium/issues/detail?id=24672
   //
   // If you are trying to be a good citizen and remove these, ask yourself:
@@ -290,14 +323,20 @@ class FilePath {
   //   OS-native string format.
   // - Am I using well-known file names, like "config.ini"?  Then use the
   //   ASCII functions (we require paths to always be supersets of ASCII).
+  // - Am I displaying a string to the user in some UI?  Then use the
+  //   LossyDisplayName() function, but keep in mind that you can't
+  //   ever use the result of that again as a path.
   static FilePath FromWStringHack(const std::wstring& wstring);
-  std::wstring ToWStringHack() const;
 
-  // Static helper method to write a StringType to a pickle.
-  static void WriteStringTypeToPickle(Pickle* pickle,
-                                      const FilePath::StringType& path);
-  static bool ReadStringTypeFromPickle(Pickle* pickle, void** iter,
-                                       FilePath::StringType* path);
+  // Returns a FilePath object from a path name in UTF-8. This function
+  // should only be used for cases where you are sure that the input
+  // string is UTF-8.
+  //
+  // Like AsUTF8Unsafe(), this function is unsafe. This function
+  // internally calls SysWideToNativeMB() on POSIX systems other than Mac
+  // and Chrome OS, to mitigate the encoding issue. See the comment at
+  // AsUTF8Unsafe() for details.
+  static FilePath FromUTF8Unsafe(const std::string& utf8);
 
   void WriteToPickle(Pickle* pickle);
   bool ReadFromPickle(Pickle* pickle, void** iter);
@@ -367,25 +406,24 @@ class FilePath {
 
 // Provide a hash function so that hash_sets and maps can contain FilePath
 // objects.
+namespace BASE_HASH_NAMESPACE {
 #if defined(COMPILER_GCC)
-namespace __gnu_cxx {
 
 template<>
 struct hash<FilePath> {
-  std::size_t operator()(const FilePath& f) const {
+  size_t operator()(const FilePath& f) const {
     return hash<FilePath::StringType>()(f.value());
   }
 };
 
-}  // namespace __gnu_cxx
 #elif defined(COMPILER_MSVC)
-namespace stdext {
 
 inline size_t hash_value(const FilePath& f) {
   return hash_value(f.value());
 }
 
-}  // namespace stdext
 #endif  // COMPILER
 
+}  // namespace BASE_HASH_NAMESPACE
+
 #endif  // BASE_FILE_PATH_H_
