@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -47,15 +47,6 @@ bool EnsureEndsWithSeparator(FilePath* path) {
   return true;
 }
 
-FilePath::StringType GetFileExtensionFromPath(const FilePath& path) {
-  FilePath::StringType file_name = path.BaseName().value();
-  const FilePath::StringType::size_type last_dot =
-      file_name.rfind(kExtensionSeparator);
-  return FilePath::StringType(last_dot == FilePath::StringType::npos ?
-                              FILE_PATH_LITERAL("") :
-                              file_name, last_dot+1);
-}
-
 void InsertBeforeExtension(FilePath* path, const FilePath::StringType& suffix) {
   FilePath::StringType& value =
       const_cast<FilePath::StringType&>(path->value());
@@ -169,7 +160,7 @@ bool ReadFileToString(const FilePath& path, std::string* contents) {
 
 bool IsDirectoryEmpty(const FilePath& dir_path) {
   FileEnumerator files(dir_path, false,
-      static_cast<FileEnumerator::FILE_TYPE>(
+      static_cast<FileEnumerator::FileType>(
           FileEnumerator::FILES | FileEnumerator::DIRECTORIES));
   if (files.Next().value().empty())
     return true;
@@ -341,7 +332,7 @@ bool MemoryMappedFile::Initialize(base::PlatformFile file) {
   return true;
 }
 
-bool MemoryMappedFile::IsValid() {
+bool MemoryMappedFile::IsValid() const {
   return data_ != NULL;
 }
 
@@ -351,7 +342,7 @@ bool MemoryMappedFile::MapFileToMemory(const FilePath& file_name) {
       NULL, NULL);
 
   if (file_ == base::kInvalidPlatformFileValue) {
-    LOG(ERROR) << "Couldn't open " << file_name.value();
+    DLOG(ERROR) << "Couldn't open " << file_name.value();
     return false;
   }
 
@@ -382,9 +373,10 @@ bool Delete(const std::wstring& path, bool recursive) {
   return Delete(FilePath::FromWStringHack(path), recursive);
 }
 std::wstring GetFileExtensionFromPath(const std::wstring& path) {
-  FilePath::StringType extension =
-      GetFileExtensionFromPath(FilePath::FromWStringHack(path));
-  return extension;
+  std::wstring file_name = FilePath(path).BaseName().value();
+  const std::wstring::size_type last_dot = file_name.rfind(kExtensionSeparator);
+  return std::wstring(last_dot == std::wstring::npos ? L""
+                                                     : file_name, last_dot + 1);
 }
 FILE* OpenFile(const std::wstring& filename, const char* mode) {
   return OpenFile(FilePath::FromWStringHack(filename), mode);
