@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,15 +8,19 @@
 
 #include "base/file_path.h"
 #include "base/logging.h"
-#include "base/mac/mac_util.h"
+#include "base/mac/bundle_locations.h"
+#include "base/mac/foundation_util.h"
 #include "base/sys_string_conversions.h"
 
-FileVersionInfoMac::FileVersionInfoMac(NSBundle *bundle) : bundle_(bundle) {
+FileVersionInfoMac::FileVersionInfoMac(NSBundle *bundle)
+    : bundle_([bundle retain]) {
 }
 
+FileVersionInfoMac::~FileVersionInfoMac() {}
+
 // static
 FileVersionInfo* FileVersionInfo::CreateFileVersionInfoForCurrentModule() {
-  return CreateFileVersionInfo(base::mac::MainAppBundlePath());
+  return CreateFileVersionInfo(base::mac::FrameworkBundlePath());
 }
 
 // static
