@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,10 +17,10 @@
 
 #include "base/command_line.h"
 #include "base/file_util.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/singleton.h"
 #include "base/path_service.h"
 #include "base/process_util.h"
-#include "base/singleton.h"
-#include "base/scoped_ptr.h"
 #include "base/string_util.h"
 #include "base/synchronization/lock.h"
 
@@ -62,7 +62,7 @@ class LinuxDistroHelper {
   // Indicate the check finished, move to STATE_CHECK_FINISHED.
   void CheckFinished() {
     base::AutoLock scoped_lock(lock_);
-    DCHECK(state_ == STATE_CHECK_STARTED);
+    DCHECK_EQ(STATE_CHECK_STARTED, state_);
     state_ = STATE_CHECK_FINISHED;
   }
 
@@ -73,7 +73,7 @@ class LinuxDistroHelper {
 #endif  // if defined(OS_LINUX)
 
 // expected prefix of the target of the /proc/self/fd/%d link for a socket
-static const char kSocketLinkPrefix[] = "socket:[";
+const char kSocketLinkPrefix[] = "socket:[";
 
 // Parse a symlink in /proc/pid/fd/$x and return the inode number of the
 // socket.
@@ -88,7 +88,7 @@ bool ProcPathGetInode(ino_t* inode_out, const char* path, bool log = false) {
   const ssize_t n = readlink(path, buf, sizeof(buf) - 1);
   if (n == -1) {
     if (log) {
-      LOG(WARNING) << "Failed to read the inode number for a socket from /proc"
+      DLOG(WARNING) << "Failed to read the inode number for a socket from /proc"
                       "(" << errno << ")";
     }
     return false;
@@ -97,8 +97,8 @@ bool ProcPathGetInode(ino_t* inode_out, const char* path, bool log = false) {
 
   if (memcmp(kSocketLinkPrefix, buf, sizeof(kSocketLinkPrefix) - 1)) {
     if (log) {
-      LOG(WARNING) << "The descriptor passed from the crashing process wasn't a"
-                      " UNIX domain socket.";
+      DLOG(WARNING) << "The descriptor passed from the crashing process wasn't "
+                      " a UNIX domain socket.";
     }
     return false;
   }
@@ -111,8 +111,8 @@ bool ProcPathGetInode(ino_t* inode_out, const char* path, bool log = false) {
 
   if (inode_ul == ULLONG_MAX) {
     if (log) {
-      LOG(WARNING) << "Failed to parse a socket's inode number: the number was "
-                      "too large. Please report this bug: " << buf;
+      DLOG(WARNING) << "Failed to parse a socket's inode number: the number "
+                       "was too large. Please report this bug: " << buf;
     }
     return false;
   }
@@ -131,7 +131,9 @@ static const int kDistroSize = 128 + 1;
 // We use this static string to hold the Linux distro info. If we
 // crash, the crash handler code will send this in the crash dump.
 char g_linux_distro[kDistroSize] =
-#if defined(OS_CHROMEOS)
+#if defined(OS_CHROMEOS) && defined(USE_AURA)
+    "CrOS Aura";
+#elif defined(OS_CHROMEOS)
     "CrOS";
 #else  // if defined(OS_LINUX)
     "Unknown";
@@ -154,9 +156,9 @@ std::string GetLinuxDistro() {
     base::GetAppOutput(CommandLine(argv), &output);
     if (output.length() > 0) {
       // lsb_release -d should return: Description:<tab>Distro Info
-      static const std::string field = "Description:\t";
-      if (output.compare(0, field.length(), field) == 0) {
-        SetLinuxDistro(output.substr(field.length()));
+      const char field[] = "Description:\t";
+      if (output.compare(0, strlen(field), field) == 0) {
+        SetLinuxDistro(output.substr(strlen(field)));
       }
     }
     distro_state_singleton->CheckFinished();
@@ -171,6 +173,7 @@ std::string GetLinuxDistro() {
   }
 #else
   NOTIMPLEMENTED();
+  return "Unknown";
 #endif
 }
 
@@ -200,7 +203,7 @@ bool FindProcessHoldingSocket(pid_t* pid_out, ino_t socket_inode) {
 
   DIR* proc = opendir("/proc");
   if (!proc) {
-    LOG(WARNING) << "Cannot open /proc";
+    DLOG(WARNING) << "Cannot open /proc";
     return false;
   }
 
@@ -252,12 +255,17 @@ bool FindProcessHoldingSocket(pid_t* pid_out, ino_t socket_inode) {
   return already_found;
 }
 
-pid_t FindThreadIDWithSyscall(pid_t pid, const std::string& expected_data) {
+pid_t FindThreadIDWithSyscall(pid_t pid, const std::string& expected_data,
+                              bool* syscall_supported) {
   char buf[256];
   snprintf(buf, sizeof(buf), "/proc/%d/task", pid);
+
+  if (syscall_supported != NULL)
+    *syscall_supported = false;
+
   DIR* task = opendir(buf);
   if (!task) {
-    LOG(WARNING) << "Cannot open " << buf;
+    DLOG(WARNING) << "Cannot open " << buf;
     return -1;
   }
 
@@ -280,6 +288,8 @@ pid_t FindThreadIDWithSyscall(pid_t pid, const std::string& expected_data) {
     int fd = open(buf, O_RDONLY);
     if (fd < 0)
       continue;
+    if (syscall_supported != NULL)
+      *syscall_supported = true;
     bool read_ret =
         file_util::ReadFromFD(fd, syscall_data.get(), expected_data.length());
     close(fd);
