@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,13 +14,11 @@ typedef HANDLE MutexHandle;
 // Windows doesn't define STDERR_FILENO.  Define it here.
 #define STDERR_FILENO 2
 #elif defined(OS_MACOSX)
-#include <CoreFoundation/CoreFoundation.h>
 #include <mach/mach.h>
 #include <mach/mach_time.h>
 #include <mach-o/dyld.h>
 #elif defined(OS_POSIX)
 #if defined(OS_NACL)
-#include <sys/nacl_syscalls.h>
 #include <sys/time.h> // timespec doesn't seem to be in <time.h>
 #else
 #include <sys/syscall.h>
@@ -40,10 +38,11 @@ typedef FILE* FileHandle;
 typedef pthread_mutex_t* MutexHandle;
 #endif
 
+#include <algorithm>
+#include <cstring>
 #include <ctime>
 #include <iomanip>
-#include <cstring>
-#include <algorithm>
+#include <ostream>
 
 #include "base/base_switches.h"
 #include "base/command_line.h"
@@ -52,20 +51,25 @@ typedef pthread_mutex_t* MutexHandle;
 #include "base/eintr_wrapper.h"
 #include "base/string_piece.h"
 #include "base/synchronization/lock_impl.h"
+#include "base/threading/platform_thread.h"
 #include "base/utf_string_conversions.h"
 #include "base/vlog.h"
 #if defined(OS_POSIX)
 #include "base/safe_strerror_posix.h"
 #endif
-#if defined(OS_MACOSX)
-#include "base/mac/scoped_cftyperef.h"
-#include "base/sys_string_conversions.h"
+
+#if defined(OS_ANDROID)
+#include <android/log.h>
 #endif
 
 namespace logging {
 
 DcheckState g_dcheck_state = DISABLE_DCHECK_FOR_NON_OFFICIAL_RELEASE_BUILDS;
+
+namespace {
+
 VlogInfo* g_vlog_info = NULL;
+VlogInfo* g_vlog_info_prev = NULL;
 
 const char* const log_severity_names[LOG_NUM_SEVERITIES] = {
   "INFO", "WARNING", "ERROR", "ERROR_REPORT", "FATAL" };
@@ -126,21 +130,6 @@ int32 CurrentProcessId() {
 #endif
 }
 
-int32 CurrentThreadId() {
-#if defined(OS_WIN)
-  return GetCurrentThreadId();
-#elif defined(OS_MACOSX)
-  return mach_thread_self();
-#elif defined(OS_LINUX)
-  return syscall(__NR_gettid);
-#elif defined(OS_FREEBSD)
-  // TODO(BSD): find a better thread ID
-  return reinterpret_cast<int64>(pthread_self());
-#elif defined(OS_NACL)
-  return pthread_self();
-#endif
-}
-
 uint64 TickCount() {
 #if defined(OS_WIN)
   return GetTickCount();
@@ -350,6 +339,9 @@ bool InitializeLogFileHandle() {
   return true;
 }
 
+}  // namespace
+
+
 bool BaseInitLoggingImpl(const PathChar* new_log_file,
                          LoggingDestination logging_dest,
                          LogLockingState lock_log,
@@ -357,12 +349,17 @@ bool BaseInitLoggingImpl(const PathChar* new_log_file,
                          DcheckState dcheck_state) {
   CommandLine* command_line = CommandLine::ForCurrentProcess();
   g_dcheck_state = dcheck_state;
-  delete g_vlog_info;
-  g_vlog_info = NULL;
+
   // Don't bother initializing g_vlog_info unless we use one of the
   // vlog switches.
   if (command_line->HasSwitch(switches::kV) ||
       command_line->HasSwitch(switches::kVModule)) {
+    // NOTE: If g_vlog_info has already been initialized, it might be in use
+    // by another thread. Don't delete the old VLogInfo, just create a second
+    // one. We keep track of both to avoid memory leak warnings.
+    CHECK(!g_vlog_info_prev);
+    g_vlog_info_prev = g_vlog_info;
+
     g_vlog_info =
         new VlogInfo(command_line->GetSwitchValueASCII(switches::kV),
                      command_line->GetSwitchValueASCII(switches::kVModule),
@@ -410,8 +407,11 @@ int GetVlogVerbosity() {
 
 int GetVlogLevelHelper(const char* file, size_t N) {
   DCHECK_GT(N, 0U);
-  return g_vlog_info ?
-      g_vlog_info->GetVlogLevel(base::StringPiece(file, N - 1)) :
+  // Note: g_vlog_info may change on a different thread during startup
+  // (but will always be valid or NULL).
+  VlogInfo* vlog_info = g_vlog_info;
+  return vlog_info ?
+      vlog_info->GetVlogLevel(base::StringPiece(file, N - 1)) :
       GetVlogVerbosity();
 }
 
@@ -503,12 +503,6 @@ void DisplayDebugMessageInDialog(const std::string& str) {
     MessageBoxW(NULL, &cmdline[0], L"Fatal error",
                 MB_OK | MB_ICONHAND | MB_TOPMOST);
   }
-#elif defined(OS_MACOSX)
-  base::mac::ScopedCFTypeRef<CFStringRef> message(
-      base::SysUTF8ToCFStringRef(str));
-  CFUserNotificationDisplayNotice(0, kCFUserNotificationStopAlertLevel,
-                                  NULL, NULL, NULL, CFSTR("Fatal Error"),
-                                  message, NULL);
 #else
   // We intentionally don't implement a dialog on other platforms.
   // You can just look at stderr.
@@ -540,21 +534,25 @@ LogMessage::LogMessage(const char* file, int line, LogSeverity severity)
   Init(file, line);
 }
 
-LogMessage::LogMessage(const char* file, int line, const CheckOpString& result)
+LogMessage::LogMessage(const char* file, int line, std::string* result)
     : severity_(LOG_FATAL), file_(file), line_(line) {
   Init(file, line);
-  stream_ << "Check failed: " << (*result.str_);
+  stream_ << "Check failed: " << *result;
+  delete result;
 }
 
 LogMessage::LogMessage(const char* file, int line, LogSeverity severity,
-                       const CheckOpString& result)
+                       std::string* result)
     : severity_(severity), file_(file), line_(line) {
   Init(file, line);
-  stream_ << "Check failed: " << (*result.str_);
+  stream_ << "Check failed: " << *result;
+  delete result;
 }
 
 LogMessage::~LogMessage() {
-#ifndef NDEBUG
+  // TODO(port): enable stacktrace generation on LOG_FATAL once backtrace are
+  // working in Android.
+#if  !defined(NDEBUG) && !defined(OS_ANDROID)
   if (severity_ == LOG_FATAL) {
     // Include a stack trace on a fatal.
     base::debug::StackTrace trace;
@@ -576,6 +574,24 @@ LogMessage::~LogMessage() {
       logging_destination == LOG_TO_BOTH_FILE_AND_SYSTEM_DEBUG_LOG) {
 #if defined(OS_WIN)
     OutputDebugStringA(str_newline.c_str());
+#elif defined(OS_ANDROID)
+    android_LogPriority priority = ANDROID_LOG_UNKNOWN;
+    switch (severity_) {
+      case LOG_INFO:
+        priority = ANDROID_LOG_INFO;
+        break;
+      case LOG_WARNING:
+        priority = ANDROID_LOG_WARN;
+        break;
+      case LOG_ERROR:
+      case LOG_ERROR_REPORT:
+        priority = ANDROID_LOG_ERROR;
+        break;
+      case LOG_FATAL:
+        priority = ANDROID_LOG_FATAL;
+        break;
+    }
+    __android_log_write(priority, "chromium", str_newline.c_str());
 #endif
     fprintf(stderr, "%s", str_newline.c_str());
     fflush(stderr);
@@ -659,7 +675,7 @@ void LogMessage::Init(const char* file, int line) {
   if (log_process_id)
     stream_ << CurrentProcessId() << ':';
   if (log_thread_id)
-    stream_ << CurrentThreadId() << ':';
+    stream_ << base::PlatformThread::CurrentId() << ':';
   if (log_timestamp) {
     time_t t = time(NULL);
     struct tm local_time = {0};
@@ -819,6 +835,9 @@ void RawLog(int level, const char* message) {
     base::debug::BreakDebugger();
 }
 
+// This was defined at the beginning of this file.
+#undef write
+
 }  // namespace logging
 
 std::ostream& operator<<(std::ostream& out, const wchar_t* wstr) {
