@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,48 +6,54 @@
 #define BASE_MESSAGE_LOOP_PROXY_IMPL_H_
 #pragma once
 
+#include "base/base_export.h"
 #include "base/message_loop.h"
 #include "base/message_loop_proxy.h"
 #include "base/synchronization/lock.h"
 
 namespace base {
 
-// A stock implementation of MessageLoopProxy that takes in a MessageLoop
-// and keeps track of its lifetime using the MessageLoop DestructionObserver.
-// For now a MessageLoopProxyImpl can only be created for the current thread.
-class MessageLoopProxyImpl : public MessageLoopProxy,
-                             public MessageLoop::DestructionObserver {
+// A stock implementation of MessageLoopProxy that is created and managed by a
+// MessageLoop. For now a MessageLoopProxyImpl can only be created as part of a
+// MessageLoop.
+class BASE_EXPORT MessageLoopProxyImpl
+    : public MessageLoopProxy {
  public:
   virtual ~MessageLoopProxyImpl();
 
   // MessageLoopProxy implementation
   virtual bool PostTask(const tracked_objects::Location& from_here,
-                        Task* task);
+                        const base::Closure& task) OVERRIDE;
   virtual bool PostDelayedTask(const tracked_objects::Location& from_here,
-                               Task* task, int64 delay_ms);
+                               const base::Closure& task,
+                               int64 delay_ms) OVERRIDE;
   virtual bool PostNonNestableTask(const tracked_objects::Location& from_here,
-                                   Task* task);
+                                   const base::Closure& task) OVERRIDE;
   virtual bool PostNonNestableDelayedTask(
       const tracked_objects::Location& from_here,
-      Task* task,
-      int64 delay_ms);
-  virtual bool BelongsToCurrentThread();
-
-  // MessageLoop::DestructionObserver implementation
-  virtual void WillDestroyCurrentMessageLoop();
+      const base::Closure& task,
+      int64 delay_ms) OVERRIDE;
+  virtual bool BelongsToCurrentThread() OVERRIDE;
 
  protected:
   // Override OnDestruct so that we can delete the object on the target message
   // loop if it still exists.
-  virtual void OnDestruct() const;
+  virtual void OnDestruct() const OVERRIDE;
 
  private:
   MessageLoopProxyImpl();
-  bool PostTaskHelper(const tracked_objects::Location& from_here,
-                      Task* task, int64 delay_ms, bool nestable);
 
-  // For the factory method to work
-  friend class MessageLoopProxy;
+  // Called directly by MessageLoop::~MessageLoop.
+  virtual void WillDestroyCurrentMessageLoop();
+
+
+  bool PostTaskHelper(const tracked_objects::Location& from_here,
+                      const base::Closure& task,
+                      int64 delay_ms,
+                      bool nestable);
+
+  // Allow the messageLoop to create a MessageLoopProxyImpl.
+  friend class ::MessageLoop;
 
   // The lock that protects access to target_message_loop_.
   mutable base::Lock message_loop_lock_;
@@ -59,4 +65,3 @@ class MessageLoopProxyImpl : public MessageLoopProxy,
 }  // namespace base
 
 #endif  // BASE_MESSAGE_LOOP_PROXY_IMPL_H_
-
