@@ -1,20 +1,26 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/message_pump_glib.h"
 
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include <algorithm>
 #include <vector>
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
+#include "base/callback.h"
+#include "base/memory/ref_counted.h"
 #include "base/message_loop.h"
-#include "base/ref_counted.h"
 #include "base/threading/thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+#if defined(TOOLKIT_USES_GTK)
+#include <gtk/gtk.h>
+#endif
+
 namespace {
 
 // This class injects dummy "events" into the GLib loop. When "handled" these
@@ -54,24 +60,24 @@ class EventInjector {
     Event event = events_[0];
     events_.erase(events_.begin());
     ++processed_events_;
-    if (event.task) {
-      event.task->Run();
-      delete event.task;
-    }
+    if (!event.callback.is_null())
+      event.callback.Run();
+    else if (!event.task.is_null())
+      event.task.Run();
   }
 
-  // Adds an event to the queue. When "handled", executes |task|.
+  // Adds an event to the queue. When "handled", executes |callback|.
   // delay_ms is relative to the last event if any, or to Now() otherwise.
-  void AddEvent(int delay_ms, Task* task) {
-    base::Time last_time;
-    if (!events_.empty()) {
-      last_time = (events_.end()-1)->time;
-    } else {
-      last_time = base::Time::NowFromSystemTime();
-    }
-    base::Time future = last_time + base::TimeDelta::FromMilliseconds(delay_ms);
-    EventInjector::Event event = { future, task };
-    events_.push_back(event);
+  void AddEvent(int delay_ms, const base::Closure& callback) {
+    AddEventHelper(delay_ms, callback, base::Closure());
+  }
+
+  void AddDummyEvent(int delay_ms) {
+    AddEventHelper(delay_ms, base::Closure(), base::Closure());
+  }
+
+  void AddEventAsTask(int delay_ms, const base::Closure& task) {
+    AddEventHelper(delay_ms, base::Closure(), task);
   }
 
   void Reset() {
@@ -84,13 +90,27 @@ class EventInjector {
  private:
   struct Event {
     base::Time time;
-    Task* task;
+    base::Closure callback;
+    base::Closure task;
   };
 
   struct Source : public GSource {
     EventInjector* injector;
   };
 
+  void AddEventHelper(
+      int delay_ms, const base::Closure& callback, const base::Closure& task) {
+    base::Time last_time;
+    if (!events_.empty())
+      last_time = (events_.end()-1)->time;
+    else
+      last_time = base::Time::NowFromSystemTime();
+
+    base::Time future = last_time + base::TimeDelta::FromMilliseconds(delay_ms);
+    EventInjector::Event event = {future, callback, task};
+    events_.push_back(event);
+  }
+
   static gboolean Prepare(GSource* source, gint* timeout_ms) {
     *timeout_ms = static_cast<Source*>(source)->injector->HandlePrepare();
     return FALSE;
@@ -121,10 +141,6 @@ GSourceFuncs EventInjector::SourceFuncs = {
   NULL
 };
 
-// Does nothing. This function can be called from a task.
-void DoNothing() {
-}
-
 void IncrementInt(int *value) {
   ++*value;
 }
@@ -134,19 +150,9 @@ void ExpectProcessedEvents(EventInjector* injector, int count) {
   EXPECT_EQ(injector->processed_events(), count);
 }
 
-// Quits the current message loop.
-void QuitMessageLoop() {
-  MessageLoop::current()->Quit();
-}
-
-// Returns a new task that quits the main loop.
-Task* NewQuitTask() {
-  return NewRunnableFunction(QuitMessageLoop);
-}
-
 // Posts a task on the current message loop.
 void PostMessageLoopTask(const tracked_objects::Location& from_here,
-                         Task* task) {
+                         const base::Closure& task) {
   MessageLoop::current()->PostTask(from_here, task);
 }
 
@@ -178,20 +184,17 @@ class MessagePumpGLibTest : public testing::Test {
 
 }  // namespace
 
-// EventInjector is expected to always live longer than the runnable methods.
-DISABLE_RUNNABLE_METHOD_REFCOUNT(EventInjector);
-
 TEST_F(MessagePumpGLibTest, TestQuit) {
   // Checks that Quit works and that the basic infrastructure is working.
 
   // Quit from a task
-  loop()->PostTask(FROM_HERE, NewQuitTask());
+  loop()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
   loop()->Run();
   EXPECT_EQ(0, injector()->processed_events());
 
   injector()->Reset();
   // Quit from an event
-  injector()->AddEvent(0, NewQuitTask());
+  injector()->AddEvent(0, MessageLoop::QuitClosure());
   loop()->Run();
   EXPECT_EQ(1, injector()->processed_events());
 }
@@ -204,23 +207,25 @@ TEST_F(MessagePumpGLibTest, TestEventTaskInterleave) {
   // If changes cause this test to fail, it is reasonable to change it, but
   // TestWorkWhileWaitingForEvents and TestEventsWhileWaitingForWork have to be
   // changed accordingly, otherwise they can become flaky.
-  injector()->AddEvent(0, NewRunnableFunction(DoNothing));
-  Task* check_task = NewRunnableFunction(ExpectProcessedEvents, injector(), 2);
-  Task* posted_task = NewRunnableFunction(PostMessageLoopTask,
-                                          FROM_HERE, check_task);
-  injector()->AddEvent(0, posted_task);
-  injector()->AddEvent(0, NewRunnableFunction(DoNothing));
-  injector()->AddEvent(0, NewQuitTask());
+  injector()->AddEventAsTask(0, base::Bind(&base::DoNothing));
+  base::Closure check_task =
+      base::Bind(&ExpectProcessedEvents, base::Unretained(injector()), 2);
+  base::Closure posted_task =
+      base::Bind(&PostMessageLoopTask, FROM_HERE, check_task);
+  injector()->AddEventAsTask(0, posted_task);
+  injector()->AddEventAsTask(0, base::Bind(&base::DoNothing));
+  injector()->AddEvent(0, MessageLoop::QuitClosure());
   loop()->Run();
   EXPECT_EQ(4, injector()->processed_events());
 
   injector()->Reset();
-  injector()->AddEvent(0, NewRunnableFunction(DoNothing));
-  check_task = NewRunnableFunction(ExpectProcessedEvents, injector(), 2);
-  posted_task = NewRunnableFunction(PostMessageLoopTask, FROM_HERE, check_task);
-  injector()->AddEvent(0, posted_task);
-  injector()->AddEvent(10, NewRunnableFunction(DoNothing));
-  injector()->AddEvent(0, NewQuitTask());
+  injector()->AddEventAsTask(0, base::Bind(&base::DoNothing));
+  check_task =
+      base::Bind(&ExpectProcessedEvents, base::Unretained(injector()), 2);
+  posted_task = base::Bind(&PostMessageLoopTask, FROM_HERE, check_task);
+  injector()->AddEventAsTask(0, posted_task);
+  injector()->AddEventAsTask(10, base::Bind(&base::DoNothing));
+  injector()->AddEvent(0, MessageLoop::QuitClosure());
   loop()->Run();
   EXPECT_EQ(4, injector()->processed_events());
 }
@@ -230,13 +235,14 @@ TEST_F(MessagePumpGLibTest, TestWorkWhileWaitingForEvents) {
   // Tests that we process tasks while waiting for new events.
   // The event queue is empty at first.
   for (int i = 0; i < 10; ++i) {
-    loop()->PostTask(FROM_HERE, NewRunnableFunction(IncrementInt, &task_count));
+    loop()->PostTask(FROM_HERE, base::Bind(&IncrementInt, &task_count));
   }
   // After all the previous tasks have executed, enqueue an event that will
   // quit.
   loop()->PostTask(
-      FROM_HERE, NewRunnableMethod(injector(), &EventInjector::AddEvent,
-                                   0, NewQuitTask()));
+      FROM_HERE,
+      base::Bind(&EventInjector::AddEvent, base::Unretained(injector()), 0,
+                 MessageLoop::QuitClosure()));
   loop()->Run();
   ASSERT_EQ(10, task_count);
   EXPECT_EQ(1, injector()->processed_events());
@@ -246,15 +252,16 @@ TEST_F(MessagePumpGLibTest, TestWorkWhileWaitingForEvents) {
   task_count = 0;
   for (int i = 0; i < 10; ++i) {
     loop()->PostDelayedTask(
-        FROM_HERE, NewRunnableFunction(IncrementInt, &task_count), 10*i);
+        FROM_HERE, base::Bind(&IncrementInt, &task_count), 10*i);
   }
   // After all the previous tasks have executed, enqueue an event that will
   // quit.
   // This relies on the fact that delayed tasks are executed in delay order.
   // That is verified in message_loop_unittest.cc.
   loop()->PostDelayedTask(
-      FROM_HERE, NewRunnableMethod(injector(), &EventInjector::AddEvent,
-                                   10, NewQuitTask()), 150);
+      FROM_HERE,
+      base::Bind(&EventInjector::AddEvent, base::Unretained(injector()), 10,
+                 MessageLoop::QuitClosure()), 150);
   loop()->Run();
   ASSERT_EQ(10, task_count);
   EXPECT_EQ(1, injector()->processed_events());
@@ -264,18 +271,19 @@ TEST_F(MessagePumpGLibTest, TestEventsWhileWaitingForWork) {
   // Tests that we process events while waiting for work.
   // The event queue is empty at first.
   for (int i = 0; i < 10; ++i) {
-    injector()->AddEvent(0, NULL);
+    injector()->AddDummyEvent(0);
   }
   // After all the events have been processed, post a task that will check that
   // the events have been processed (note: the task executes after the event
   // that posted it has been handled, so we expect 11 at that point).
-  Task* check_task = NewRunnableFunction(ExpectProcessedEvents, injector(), 11);
-  Task* posted_task = NewRunnableFunction(PostMessageLoopTask,
-                                          FROM_HERE, check_task);
-  injector()->AddEvent(10, posted_task);
+  base::Closure check_task =
+      base::Bind(&ExpectProcessedEvents, base::Unretained(injector()), 11);
+  base::Closure posted_task =
+      base::Bind(&PostMessageLoopTask, FROM_HERE, check_task);
+  injector()->AddEventAsTask(10, posted_task);
 
   // And then quit (relies on the condition tested by TestEventTaskInterleave).
-  injector()->AddEvent(10, NewQuitTask());
+  injector()->AddEvent(10, MessageLoop::QuitClosure());
   loop()->Run();
 
   EXPECT_EQ(12, injector()->processed_events());
@@ -302,7 +310,7 @@ class ConcurrentHelper : public base::RefCounted<ConcurrentHelper>  {
         MessageLoop::current()->Quit();
     } else {
       MessageLoop::current()->PostTask(
-          FROM_HERE, NewRunnableMethod(this, &ConcurrentHelper::FromTask));
+          FROM_HERE, base::Bind(&ConcurrentHelper::FromTask, this));
     }
   }
 
@@ -313,8 +321,8 @@ class ConcurrentHelper : public base::RefCounted<ConcurrentHelper>  {
     if (task_count_ == 0 && event_count_ == 0) {
         MessageLoop::current()->Quit();
     } else {
-      injector_->AddEvent(
-          0, NewRunnableMethod(this, &ConcurrentHelper::FromEvent));
+      injector_->AddEventAsTask(
+          0, base::Bind(&ConcurrentHelper::FromEvent, this));
     }
   }
 
@@ -346,16 +354,16 @@ TEST_F(MessagePumpGLibTest, TestConcurrentEventPostedTask) {
 
   // Add 2 events to the queue to make sure it is always full (when we remove
   // the event before processing it).
-  injector()->AddEvent(
-      0, NewRunnableMethod(helper.get(), &ConcurrentHelper::FromEvent));
-  injector()->AddEvent(
-      0, NewRunnableMethod(helper.get(), &ConcurrentHelper::FromEvent));
+  injector()->AddEventAsTask(
+      0, base::Bind(&ConcurrentHelper::FromEvent, helper.get()));
+  injector()->AddEventAsTask(
+      0, base::Bind(&ConcurrentHelper::FromEvent, helper.get()));
 
   // Similarly post 2 tasks.
   loop()->PostTask(
-      FROM_HERE, NewRunnableMethod(helper.get(), &ConcurrentHelper::FromTask));
+      FROM_HERE, base::Bind(&ConcurrentHelper::FromTask, helper.get()));
   loop()->PostTask(
-      FROM_HERE, NewRunnableMethod(helper.get(), &ConcurrentHelper::FromTask));
+      FROM_HERE, base::Bind(&ConcurrentHelper::FromTask, helper.get()));
 
   loop()->Run();
   EXPECT_EQ(0, helper->event_count());
@@ -366,14 +374,14 @@ namespace {
 
 void AddEventsAndDrainGLib(EventInjector* injector) {
   // Add a couple of dummy events
-  injector->AddEvent(0, NULL);
-  injector->AddEvent(0, NULL);
+  injector->AddDummyEvent(0);
+  injector->AddDummyEvent(0);
   // Then add an event that will quit the main loop.
-  injector->AddEvent(0, NewQuitTask());
+  injector->AddEvent(0, MessageLoop::QuitClosure());
 
   // Post a couple of dummy tasks
-  MessageLoop::current()->PostTask(FROM_HERE, NewRunnableFunction(DoNothing));
-  MessageLoop::current()->PostTask(FROM_HERE, NewRunnableFunction(DoNothing));
+  MessageLoop::current()->PostTask(FROM_HERE, base::Bind(&base::DoNothing));
+  MessageLoop::current()->PostTask(FROM_HERE, base::Bind(&base::DoNothing));
 
   // Drain the events
   while (g_main_context_pending(NULL)) {
@@ -386,7 +394,8 @@ void AddEventsAndDrainGLib(EventInjector* injector) {
 TEST_F(MessagePumpGLibTest, TestDrainingGLib) {
   // Tests that draining events using GLib works.
   loop()->PostTask(
-      FROM_HERE, NewRunnableFunction(AddEventsAndDrainGLib, injector()));
+      FROM_HERE,
+      base::Bind(&AddEventsAndDrainGLib, base::Unretained(injector())));
   loop()->Run();
 
   EXPECT_EQ(3, injector()->processed_events());
@@ -395,33 +404,38 @@ TEST_F(MessagePumpGLibTest, TestDrainingGLib) {
 
 namespace {
 
+#if defined(TOOLKIT_USES_GTK)
 void AddEventsAndDrainGtk(EventInjector* injector) {
   // Add a couple of dummy events
-  injector->AddEvent(0, NULL);
-  injector->AddEvent(0, NULL);
+  injector->AddDummyEvent(0);
+  injector->AddDummyEvent(0);
   // Then add an event that will quit the main loop.
-  injector->AddEvent(0, NewQuitTask());
+  injector->AddEvent(0, MessageLoop::QuitClosure());
 
   // Post a couple of dummy tasks
-  MessageLoop::current()->PostTask(FROM_HERE, NewRunnableFunction(DoNothing));
-  MessageLoop::current()->PostTask(FROM_HERE, NewRunnableFunction(DoNothing));
+  MessageLoop::current()->PostTask(FROM_HERE, base::Bind(&base::DoNothing));
+  MessageLoop::current()->PostTask(FROM_HERE, base::Bind(&base::DoNothing));
 
   // Drain the events
   while (gtk_events_pending()) {
     gtk_main_iteration();
   }
 }
+#endif
 
 }  // namespace
 
+#if defined(TOOLKIT_USES_GTK)
 TEST_F(MessagePumpGLibTest, TestDrainingGtk) {
   // Tests that draining events using Gtk works.
   loop()->PostTask(
-      FROM_HERE, NewRunnableFunction(AddEventsAndDrainGtk, injector()));
+      FROM_HERE,
+      base::Bind(&AddEventsAndDrainGtk, base::Unretained(injector())));
   loop()->Run();
 
   EXPECT_EQ(3, injector()->processed_events());
 }
+#endif
 
 namespace {
 
@@ -436,10 +450,16 @@ class GLibLoopRunner : public base::RefCounted<GLibLoopRunner> {
     }
   }
 
-  void RunGtk() {
+  void RunLoop() {
+#if defined(TOOLKIT_USES_GTK)
     while (!quit_) {
       gtk_main_iteration();
     }
+#else
+    while (!quit_) {
+      g_main_context_iteration(NULL, TRUE);
+    }
+#endif
   }
 
   void Quit() {
@@ -465,21 +485,21 @@ void TestGLibLoopInternal(EventInjector* injector) {
 
   int task_count = 0;
   // Add a couple of dummy events
-  injector->AddEvent(0, NULL);
-  injector->AddEvent(0, NULL);
+  injector->AddDummyEvent(0);
+  injector->AddDummyEvent(0);
   // Post a couple of dummy tasks
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableFunction(IncrementInt, &task_count));
+      FROM_HERE, base::Bind(&IncrementInt, &task_count));
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableFunction(IncrementInt, &task_count));
+      FROM_HERE, base::Bind(&IncrementInt, &task_count));
   // Delayed events
-  injector->AddEvent(10, NULL);
-  injector->AddEvent(10, NULL);
+  injector->AddDummyEvent(10);
+  injector->AddDummyEvent(10);
   // Delayed work
   MessageLoop::current()->PostDelayedTask(
-      FROM_HERE, NewRunnableFunction(IncrementInt, &task_count), 30);
+      FROM_HERE, base::Bind(&IncrementInt, &task_count), 30);
   MessageLoop::current()->PostDelayedTask(
-      FROM_HERE, NewRunnableMethod(runner.get(), &GLibLoopRunner::Quit), 40);
+      FROM_HERE, base::Bind(&GLibLoopRunner::Quit, runner.get()), 40);
 
   // Run a nested, straight GLib message loop.
   runner->RunGLib();
@@ -496,24 +516,24 @@ void TestGtkLoopInternal(EventInjector* injector) {
 
   int task_count = 0;
   // Add a couple of dummy events
-  injector->AddEvent(0, NULL);
-  injector->AddEvent(0, NULL);
+  injector->AddDummyEvent(0);
+  injector->AddDummyEvent(0);
   // Post a couple of dummy tasks
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableFunction(IncrementInt, &task_count));
+      FROM_HERE, base::Bind(&IncrementInt, &task_count));
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableFunction(IncrementInt, &task_count));
+      FROM_HERE, base::Bind(&IncrementInt, &task_count));
   // Delayed events
-  injector->AddEvent(10, NULL);
-  injector->AddEvent(10, NULL);
+  injector->AddDummyEvent(10);
+  injector->AddDummyEvent(10);
   // Delayed work
   MessageLoop::current()->PostDelayedTask(
-      FROM_HERE, NewRunnableFunction(IncrementInt, &task_count), 30);
+      FROM_HERE, base::Bind(&IncrementInt, &task_count), 30);
   MessageLoop::current()->PostDelayedTask(
-      FROM_HERE, NewRunnableMethod(runner.get(), &GLibLoopRunner::Quit), 40);
+      FROM_HERE, base::Bind(&GLibLoopRunner::Quit, runner.get()), 40);
 
   // Run a nested, straight Gtk message loop.
-  runner->RunGtk();
+  runner->RunLoop();
 
   ASSERT_EQ(3, task_count);
   EXPECT_EQ(4, injector->processed_events());
@@ -523,21 +543,23 @@ void TestGtkLoopInternal(EventInjector* injector) {
 }  // namespace
 
 TEST_F(MessagePumpGLibTest, TestGLibLoop) {
-  // Tests that events and posted tasks are correctly exectuted if the message
+  // Tests that events and posted tasks are correctly executed if the message
   // loop is not run by MessageLoop::Run() but by a straight GLib loop.
   // Note that in this case we don't make strong guarantees about niceness
   // between events and posted tasks.
-  loop()->PostTask(FROM_HERE,
-                   NewRunnableFunction(TestGLibLoopInternal, injector()));
+  loop()->PostTask(
+      FROM_HERE,
+      base::Bind(&TestGLibLoopInternal, base::Unretained(injector())));
   loop()->Run();
 }
 
 TEST_F(MessagePumpGLibTest, TestGtkLoop) {
-  // Tests that events and posted tasks are correctly exectuted if the message
+  // Tests that events and posted tasks are correctly executed if the message
   // loop is not run by MessageLoop::Run() but by a straight Gtk loop.
   // Note that in this case we don't make strong guarantees about niceness
   // between events and posted tasks.
-  loop()->PostTask(FROM_HERE,
-                   NewRunnableFunction(TestGtkLoopInternal, injector()));
+  loop()->PostTask(
+      FROM_HERE,
+      base::Bind(&TestGtkLoopInternal, base::Unretained(injector())));
   loop()->Run();
 }
