@@ -1,13 +1,11 @@
-// Copyright (c) 2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "base/message_pump_mac.h"
+#import "base/message_pump_mac.h"
 
 #import <AppKit/AppKit.h>
 #import <Foundation/Foundation.h>
-#include <IOKit/IOMessage.h>
-#include <IOKit/pwr_mgt/IOPMLib.h>
 
 #include <limits>
 
@@ -22,6 +20,10 @@ void NoOp(void* info) {
 const CFTimeInterval kCFTimeIntervalMax =
     std::numeric_limits<CFTimeInterval>::max();
 
+// Set to true if MessagePumpMac::Create() is called before NSApp is
+// initialized.  Only accessed from the main thread.
+bool not_using_crapp = false;
+
 }  // namespace
 
 namespace base {
@@ -51,7 +53,6 @@ MessagePumpCFRunLoopBase::MessagePumpCFRunLoopBase()
       run_nesting_level_(0),
       deepest_nesting_level_(0),
       delegateless_work_(false),
-      delegateless_delayed_work_(false),
       delegateless_idle_work_(false) {
   run_loop_ = CFRunLoopGetCurrent();
   CFRetain(run_loop_);
@@ -78,15 +79,9 @@ MessagePumpCFRunLoopBase::MessagePumpCFRunLoopBase()
                                        &source_context);
   CFRunLoopAddSource(run_loop_, work_source_, kCFRunLoopCommonModes);
 
-  source_context.perform = RunDelayedWorkSource;
-  delayed_work_source_ = CFRunLoopSourceCreate(NULL,  // allocator
-                                               2,     // priority
-                                               &source_context);
-  CFRunLoopAddSource(run_loop_, delayed_work_source_, kCFRunLoopCommonModes);
-
   source_context.perform = RunIdleWorkSource;
   idle_work_source_ = CFRunLoopSourceCreate(NULL,  // allocator
-                                            3,     // priority
+                                            2,     // priority
                                             &source_context);
   CFRunLoopAddSource(run_loop_, idle_work_source_, kCFRunLoopCommonModes);
 
@@ -123,33 +118,12 @@ MessagePumpCFRunLoopBase::MessagePumpCFRunLoopBase()
                                                  EnterExitObserver,
                                                  &observer_context);
   CFRunLoopAddObserver(run_loop_, enter_exit_observer_, kCFRunLoopCommonModes);
-
-  root_power_domain_ = IORegisterForSystemPower(this,
-                                                &power_notification_port_,
-                                                PowerStateNotification,
-                                                &power_notification_object_);
-  if (root_power_domain_ != MACH_PORT_NULL) {
-    CFRunLoopAddSource(
-        run_loop_,
-        IONotificationPortGetRunLoopSource(power_notification_port_),
-        kCFRunLoopCommonModes);
-  }
 }
 
 // Ideally called on the run loop thread.  If other run loops were running
 // lower on the run loop thread's stack when this object was created, the
 // same number of run loops must be running when this object is destroyed.
 MessagePumpCFRunLoopBase::~MessagePumpCFRunLoopBase() {
-  if (root_power_domain_ != MACH_PORT_NULL) {
-    CFRunLoopRemoveSource(
-        run_loop_,
-        IONotificationPortGetRunLoopSource(power_notification_port_),
-        kCFRunLoopCommonModes);
-    IODeregisterForSystemPower(&power_notification_object_);
-    IOServiceClose(root_power_domain_);
-    IONotificationPortDestroy(power_notification_port_);
-  }
-
   CFRunLoopRemoveObserver(run_loop_, enter_exit_observer_,
                           kCFRunLoopCommonModes);
   CFRelease(enter_exit_observer_);
@@ -169,9 +143,6 @@ MessagePumpCFRunLoopBase::~MessagePumpCFRunLoopBase() {
   CFRunLoopRemoveSource(run_loop_, idle_work_source_, kCFRunLoopCommonModes);
   CFRelease(idle_work_source_);
 
-  CFRunLoopRemoveSource(run_loop_, delayed_work_source_, kCFRunLoopCommonModes);
-  CFRelease(delayed_work_source_);
-
   CFRunLoopRemoveSource(run_loop_, work_source_, kCFRunLoopCommonModes);
   CFRelease(work_source_);
 
@@ -199,10 +170,6 @@ void MessagePumpCFRunLoopBase::Run(Delegate* delegate) {
       CFRunLoopSourceSignal(work_source_);
       delegateless_work_ = false;
     }
-    if (delegateless_delayed_work_) {
-      CFRunLoopSourceSignal(delayed_work_source_);
-      delegateless_delayed_work_ = false;
-    }
     if (delegateless_idle_work_) {
       CFRunLoopSourceSignal(idle_work_source_);
       delegateless_idle_work_ = false;
@@ -260,11 +227,10 @@ void MessagePumpCFRunLoopBase::RunDelayedWorkTimer(CFRunLoopTimerRef timer,
   self->delayed_work_fire_time_ = kCFTimeIntervalMax;
 
   // CFRunLoopTimers fire outside of the priority scheme for CFRunLoopSources.
-  // In order to establish the proper priority where delegate_->DoDelayedWork
-  // can only be called if delegate_->DoWork returns false, the timer used
-  // to schedule delayed work must signal a CFRunLoopSource set at a lower
-  // priority than the one used for delegate_->DoWork.
-  CFRunLoopSourceSignal(self->delayed_work_source_);
+  // In order to establish the proper priority in which work and delayed work
+  // are processed one for one, the timer used to schedule delayed work must
+  // signal a CFRunLoopSource used to dispatch both work and delayed work.
+  CFRunLoopSourceSignal(self->work_source_);
 }
 
 // Called from the run loop.
@@ -291,58 +257,39 @@ bool MessagePumpCFRunLoopBase::RunWork() {
   // released promptly even in the absence of UI events.
   MessagePumpScopedAutoreleasePool autorelease_pool(this);
 
-  // Call DoWork once, and if something was done, arrange to come back here
-  // again as long as the loop is still running.
+  // Call DoWork and DoDelayedWork once, and if something was done, arrange to
+  // come back here again as long as the loop is still running.
   bool did_work = delegate_->DoWork();
-  if (did_work) {
-    CFRunLoopSourceSignal(work_source_);
-  }
-
-  return did_work;
-}
-
-// Called from the run loop.
-// static
-void MessagePumpCFRunLoopBase::RunDelayedWorkSource(void* info) {
-  MessagePumpCFRunLoopBase* self = static_cast<MessagePumpCFRunLoopBase*>(info);
-  self->RunDelayedWork();
-}
-
-// Called by MessagePumpCFRunLoopBase::RunDelayedWorkSource.
-bool MessagePumpCFRunLoopBase::RunDelayedWork() {
-  if (!delegate_) {
-    // This point can be reached with a NULL delegate_ if Run is not on the
-    // stack but foreign code is spinning the CFRunLoop.  Arrange to come back
-    // here when a delegate is available.
-    delegateless_delayed_work_ = true;
-    return false;
-  }
-
-  // The NSApplication-based run loop only drains the autorelease pool at each
-  // UI event (NSEvent).  The autorelease pool is not drained for each
-  // CFRunLoopSource target that's run.  Use a local pool for any autoreleased
-  // objects if the app is not currently handling a UI event to ensure they're
-  // released promptly even in the absence of UI events.
-  MessagePumpScopedAutoreleasePool autorelease_pool(this);
+  bool resignal_work_source = did_work;
 
   TimeTicks next_time;
   delegate_->DoDelayedWork(&next_time);
-
-  bool more_work = !next_time.is_null();
-  if (more_work) {
-    TimeDelta delay = next_time - TimeTicks::Now();
-    if (delay > TimeDelta()) {
-      // There's more delayed work to be done in the future.
-      ScheduleDelayedWork(next_time);
-    } else {
-      // There's more delayed work to be done, and its time is in the past.
-      // Arrange to come back here directly as long as the loop is still
-      // running.
-      CFRunLoopSourceSignal(delayed_work_source_);
+  if (!did_work) {
+    // Determine whether there's more delayed work, and if so, if it needs to
+    // be done at some point in the future or if it's already time to do it.
+    // Only do these checks if did_work is false. If did_work is true, this
+    // function, and therefore any additional delayed work, will get another
+    // chance to run before the loop goes to sleep.
+    bool more_delayed_work = !next_time.is_null();
+    if (more_delayed_work) {
+      TimeDelta delay = next_time - TimeTicks::Now();
+      if (delay > TimeDelta()) {
+        // There's more delayed work to be done in the future.
+        ScheduleDelayedWork(next_time);
+      } else {
+        // There's more delayed work to be done, and its time is in the past.
+        // Arrange to come back here directly as long as the loop is still
+        // running.
+        resignal_work_source = true;
+      }
     }
   }
 
-  return more_work;
+  if (resignal_work_source) {
+    CFRunLoopSourceSignal(work_source_);
+  }
+
+  return resignal_work_source;
 }
 
 // Called from the run loop.
@@ -398,19 +345,12 @@ bool MessagePumpCFRunLoopBase::RunNestingDeferredWork() {
 
   // Immediately try work in priority order.
   if (!RunWork()) {
-    if (!RunDelayedWork()) {
-      if (!RunIdleWork()) {
-        return false;
-      }
-    } else {
-      // There was no work, and delayed work was done.  Arrange for the loop
-      // to try non-nestable idle work on a subsequent pass.
-      CFRunLoopSourceSignal(idle_work_source_);
+    if (!RunIdleWork()) {
+      return false;
     }
   } else {
-    // Work was done.  Arrange for the loop to try non-nestable delayed and
-    // idle work on a subsequent pass.
-    CFRunLoopSourceSignal(delayed_work_source_);
+    // Work was done.  Arrange for the loop to try non-nestable idle work on
+    // a subsequent pass.
     CFRunLoopSourceSignal(idle_work_source_);
   }
 
@@ -506,73 +446,6 @@ void MessagePumpCFRunLoopBase::EnterExitObserver(CFRunLoopObserverRef observer,
   self->EnterExitRunLoop(activity);
 }
 
-// Called from the run loop.
-// static
-void MessagePumpCFRunLoopBase::PowerStateNotification(void* info,
-                                                      io_service_t service,
-                                                      uint32_t message_type,
-                                                      void* message_argument) {
-  // CFRunLoopTimer (NSTimer) is scheduled in terms of CFAbsoluteTime, which
-  // measures the number of seconds since 2001-01-01 00:00:00.0 Z.  It is
-  // implemented in terms of kernel ticks, as in mach_absolute_time.  While an
-  // offset and scale factor can be applied to convert between the two time
-  // bases at any time after boot, the kernel clock stops while the system is
-  // asleep, altering the offset.  (The offset will also change when the
-  // real-time clock is adjusted.)  CFRunLoopTimers are not readjusted to take
-  // this into account when the system wakes up, so any timers that were
-  // pending while the system was asleep will be delayed by the sleep
-  // duration.
-  //
-  // The MessagePump interface assumes that scheduled delayed work will be
-  // performed at the time ScheduleDelayedWork was asked to perform it.  The
-  // delay caused by the CFRunLoopTimer not firing at the appropriate time
-  // results in a stall of queued delayed work when the system wakes up.
-  // With this limitation, scheduled work would not be performed until
-  // (system wake time + scheduled work time - system sleep time), while it
-  // would be expected to be performed at (scheduled work time).
-  //
-  // To work around this problem, when the system wakes up from sleep, if a
-  // delayed work timer is pending, it is rescheduled to fire at the original
-  // time that it was scheduled to fire.
-  //
-  // This mechanism is not resilient if the real-time clock does not maintain
-  // stable time while the system is sleeping, but it matches the behavior of
-  // the various other MessagePump implementations, and MessageLoop seems to
-  // be limited in the same way.
-  //
-  // References
-  //  - Chris Kane, "NSTimer and deep sleep," cocoa-dev@lists.apple.com,
-  //    http://lists.apple.com/archives/Cocoa-dev/2002/May/msg01547.html
-  //  - Apple Technical Q&A QA1340, "Registering and unregistering for sleep
-  //    and wake notifications,"
-  //    http://developer.apple.com/mac/library/qa/qa2004/qa1340.html
-  //  - Core Foundation source code, CF-550/CFRunLoop.c and CF-550/CFDate.c,
-  //    http://www.opensource.apple.com/
-
-  MessagePumpCFRunLoopBase* self = static_cast<MessagePumpCFRunLoopBase*>(info);
-
-  switch (message_type) {
-    case kIOMessageSystemWillPowerOn:
-      if (self->delayed_work_fire_time_ != kCFTimeIntervalMax) {
-        CFRunLoopTimerSetNextFireDate(self->delayed_work_timer_,
-                                      self->delayed_work_fire_time_);
-      }
-      break;
-
-    case kIOMessageSystemWillSleep:
-    case kIOMessageCanSystemSleep:
-      // The system will wait for 30 seconds before entering sleep if neither
-      // IOAllowPowerChange nor IOCancelPowerChange are called.  That would be
-      // pretty antisocial.
-      IOAllowPowerChange(self->root_power_domain_,
-                         reinterpret_cast<long>(message_argument));
-      break;
-
-    default:
-      break;
-  }
-}
-
 // Called by MessagePumpCFRunLoopBase::EnterExitRunLoop.  The default
 // implementation is a no-op.
 void MessagePumpCFRunLoopBase::EnterExitRunLoop(CFRunLoopActivity activity) {
@@ -672,6 +545,13 @@ MessagePumpNSApplication::MessagePumpNSApplication()
 void MessagePumpNSApplication::DoRun(Delegate* delegate) {
   bool last_running_own_loop_ = running_own_loop_;
 
+  // NSApp must be initialized by calling:
+  // [{some class which implements CrAppProtocol} sharedApplication]
+  // Most likely candidates are CrApplication or BrowserCrApplication.
+  // These can be initialized from C++ code by calling
+  // RegisterCrApp() or RegisterBrowserCrApp().
+  CHECK(NSApp);
+
   if (![NSApp isRunning]) {
     running_own_loop_ = false;
     // NSApplication manages autorelease pools itself when run this way.
@@ -697,7 +577,7 @@ void MessagePumpNSApplication::DoRun(Delegate* delegate) {
 
 void MessagePumpNSApplication::Quit() {
   if (!running_own_loop_) {
-    [NSApp stop:nil];
+    [[NSApplication sharedApplication] stop:nil];
   } else {
     keep_running_ = false;
   }
@@ -715,6 +595,9 @@ void MessagePumpNSApplication::Quit() {
            atStart:NO];
 }
 
+MessagePumpCrApplication::MessagePumpCrApplication() {
+}
+
 // Prevents an autorelease pool from being created if the app is in the midst of
 // handling a UI event because various parts of AppKit depend on objects that
 // are created while handling a UI event to be autoreleased in the event loop.
@@ -746,22 +629,50 @@ void MessagePumpNSApplication::Quit() {
 // CrApplication is responsible for setting handlingSendEvent to true just
 // before it sends the event through the event handling mechanism, and
 // returning it to its previous value once the event has been sent.
-NSAutoreleasePool* MessagePumpNSApplication::CreateAutoreleasePool() {
-  NSAutoreleasePool* pool = nil;
-  DCHECK([NSApp conformsToProtocol:@protocol(CrAppProtocol)]);
-  if (![NSApp isHandlingSendEvent]) {
-    pool = MessagePumpCFRunLoopBase::CreateAutoreleasePool();
-  }
-  return pool;
+NSAutoreleasePool* MessagePumpCrApplication::CreateAutoreleasePool() {
+  if (MessagePumpMac::IsHandlingSendEvent())
+    return nil;
+  return MessagePumpNSApplication::CreateAutoreleasePool();
 }
 
 // static
 MessagePump* MessagePumpMac::Create() {
   if ([NSThread isMainThread]) {
+    if ([NSApp conformsToProtocol:@protocol(CrAppProtocol)])
+      return new MessagePumpCrApplication;
+
+    // The main-thread MessagePump implementations REQUIRE an NSApp.
+    // Executables which have specific requirements for their
+    // NSApplication subclass should initialize appropriately before
+    // creating an event loop.
+    [NSApplication sharedApplication];
+    not_using_crapp = true;
     return new MessagePumpNSApplication;
   }
 
   return new MessagePumpNSRunLoop;
 }
 
+// static
+bool MessagePumpMac::UsingCrApp() {
+  DCHECK([NSThread isMainThread]);
+
+  // If NSApp is still not initialized, then the subclass used cannot
+  // be determined.
+  DCHECK(NSApp);
+
+  // The pump was created using MessagePumpNSApplication.
+  if (not_using_crapp)
+    return false;
+
+  return [NSApp conformsToProtocol:@protocol(CrAppProtocol)];
+}
+
+// static
+bool MessagePumpMac::IsHandlingSendEvent() {
+  DCHECK([NSApp conformsToProtocol:@protocol(CrAppProtocol)]);
+  NSObject<CrAppProtocol>* app = static_cast<NSObject<CrAppProtocol>*>(NSApp);
+  return [app isHandlingSendEvent];
+}
+
 }  // namespace base
