@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -23,6 +23,8 @@ namespace base {
   bool PathProviderWin(int key, FilePath* result);
 #elif defined(OS_MACOSX)
   bool PathProviderMac(int key, FilePath* result);
+#elif defined(OS_ANDROID)
+  bool PathProviderAndroid(int key, FilePath* result);
 #elif defined(OS_POSIX)
   bool PathProviderPosix(int key, FilePath* result);
 #endif
@@ -44,7 +46,7 @@ struct Provider {
   bool is_static;
 };
 
-static Provider base_provider = {
+Provider base_provider = {
   base::PathProvider,
   NULL,
 #ifndef NDEBUG
@@ -55,7 +57,7 @@ static Provider base_provider = {
 };
 
 #if defined(OS_WIN)
-static Provider base_provider_win = {
+Provider base_provider_win = {
   base::PathProviderWin,
   &base_provider,
 #ifndef NDEBUG
@@ -67,7 +69,7 @@ static Provider base_provider_win = {
 #endif
 
 #if defined(OS_MACOSX)
-static Provider base_provider_mac = {
+Provider base_provider_mac = {
   base::PathProviderMac,
   &base_provider,
 #ifndef NDEBUG
@@ -78,8 +80,20 @@ static Provider base_provider_mac = {
 };
 #endif
 
-#if defined(OS_POSIX) && !defined(OS_MACOSX)
-static Provider base_provider_posix = {
+#if defined(OS_ANDROID)
+Provider base_provider_android = {
+  base::PathProviderAndroid,
+  &base_provider,
+#ifndef NDEBUG
+  0,
+  0,
+#endif
+  true
+};
+#endif
+
+#if defined(OS_POSIX) && !defined(OS_MACOSX) && !defined(OS_ANDROID)
+Provider base_provider_posix = {
   base::PathProviderPosix,
   &base_provider,
 #ifndef NDEBUG
@@ -102,6 +116,8 @@ struct PathData {
     providers = &base_provider_win;
 #elif defined(OS_MACOSX)
     providers = &base_provider_mac;
+#elif defined(OS_ANDROID)
+    providers = &base_provider_android;
 #elif defined(OS_POSIX)
     providers = &base_provider_posix;
 #endif
@@ -118,7 +134,7 @@ struct PathData {
   }
 };
 
-static base::LazyInstance<PathData> g_path_data(base::LINKER_INITIALIZED);
+static base::LazyInstance<PathData> g_path_data = LAZY_INSTANCE_INITIALIZER;
 
 static PathData* GetPathData() {
   return g_path_data.Pointer();
@@ -146,7 +162,7 @@ bool PathService::GetFromOverrides(int key, FilePath* result) {
   PathData* path_data = GetPathData();
   base::AutoLock scoped_lock(path_data->lock);
 
-  // check for an overriden version.
+  // check for an overridden version.
   PathMap::const_iterator it = path_data->overrides.find(key);
   if (it != path_data->overrides.end()) {
     *result = it->second;
@@ -171,7 +187,7 @@ bool PathService::Get(int key, FilePath* result) {
   PathData* path_data = GetPathData();
   DCHECK(path_data);
   DCHECK(result);
-  DCHECK(key >= base::DIR_CURRENT);
+  DCHECK_GE(key, base::DIR_CURRENT);
 
   // special case the current directory because it can never be cached
   if (key == base::DIR_CURRENT)
@@ -208,19 +224,19 @@ bool PathService::Get(int key, FilePath* result) {
 bool PathService::Override(int key, const FilePath& path) {
   PathData* path_data = GetPathData();
   DCHECK(path_data);
-  DCHECK(key > base::DIR_CURRENT) << "invalid path key";
+  DCHECK_GT(key, base::DIR_CURRENT) << "invalid path key";
 
   FilePath file_path = path;
 
   // Make sure the directory exists. We need to do this before we translate
   // this to the absolute path because on POSIX, AbsolutePath fails if called
-  // on a non-existant path.
+  // on a non-existent path.
   if (!file_util::PathExists(file_path) &&
       !file_util::CreateDirectory(file_path))
     return false;
 
   // We need to have an absolute path, as extensions and plugins don't like
-  // relative paths, and will glady crash the browser in CHECK()s if they get a
+  // relative paths, and will gladly crash the browser in CHECK()s if they get a
   // relative path.
   if (!file_util::AbsolutePath(&file_path))
     return false;
@@ -241,7 +257,7 @@ void PathService::RegisterProvider(ProviderFunc func, int key_start,
                                    int key_end) {
   PathData* path_data = GetPathData();
   DCHECK(path_data);
-  DCHECK(key_end > key_start);
+  DCHECK_GT(key_end, key_start);
 
   base::AutoLock scoped_lock(path_data->lock);
 
