@@ -8,6 +8,7 @@
 
 #include <string>
 
+#include "base/base_export.h"
 #include "base/basictypes.h"
 #include "base/gtest_prod_util.h"
 #include "base/logging.h"
@@ -30,7 +31,7 @@
 // space is controlled by the header_size parameter passed to the Pickle
 // constructor.
 //
-class Pickle {
+class BASE_EXPORT Pickle {
  public:
   // Initialize a Pickle object using the default header size.
   Pickle();
@@ -49,14 +50,18 @@ class Pickle {
   // Initializes a Pickle as a deep copy of another Pickle.
   Pickle(const Pickle& other);
 
+  // Note: There are no virtual methods in this class.  This destructor is
+  // virtual as an element of defensive coding.  Other classes have derived from
+  // this class, and there is a *chance* that they will cast into this base
+  // class before destruction.  At least one such class does have a virtual
+  // destructor, suggesting at least some need to call more derived destructors.
   virtual ~Pickle();
 
   // Performs a deep copy.
   Pickle& operator=(const Pickle& other);
 
   // Returns the size of the Pickle's data.
-  int size() const { return static_cast<int>(header_size_ +
-                                             header_->payload_size); }
+  size_t size() const { return header_size_ + header_->payload_size; }
 
   // Returns the data for this Pickle.
   const void* data() const { return header_; }
@@ -69,6 +74,7 @@ class Pickle {
   bool ReadInt(void** iter, int* result) const;
   bool ReadLong(void** iter, long* result) const;
   bool ReadSize(void** iter, size_t* result) const;
+  bool ReadUInt16(void** iter, uint16* result) const;
   bool ReadUInt32(void** iter, uint32* result) const;
   bool ReadInt64(void** iter, int64* result) const;
   bool ReadUInt64(void** iter, uint64* result) const;
@@ -98,6 +104,9 @@ class Pickle {
   bool WriteSize(size_t value) {
     return WriteBytes(&value, sizeof(value));
   }
+  bool WriteUInt16(uint16 value) {
+    return WriteBytes(&value, sizeof(value));
+  }
   bool WriteUInt32(uint32 value) {
     return WriteBytes(&value, sizeof(value));
   }
@@ -144,12 +153,12 @@ class Pickle {
   // to the Pickle constructor.
   template <class T>
   T* headerT() {
-    DCHECK(sizeof(T) == header_size_);
+    DCHECK_EQ(header_size_, sizeof(T));
     return static_cast<T*>(header_);
   }
   template <class T>
   const T* headerT() const {
-    DCHECK(sizeof(T) == header_size_);
+    DCHECK_EQ(header_size_, sizeof(T));
     return static_cast<const T*>(header_);
   }
 
