@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,24 +7,31 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
 #include "base/eintr_wrapper.h"
 #include "base/file_path.h"
 #include "base/logging.h"
+#include "base/threading/thread_restrictions.h"
 #include "base/utf_string_conversions.h"
 
+#if defined(OS_ANDROID)
+#include "base/os_compat_android.h"
+#endif
+
 namespace base {
 
-#if defined(OS_OPENBSD) || defined(OS_FREEBSD) || \
-    (defined(OS_MACOSX) && \
-     MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5)
+#if defined(OS_BSD) || (defined(OS_MACOSX) && \
+    MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5)
 typedef struct stat stat_wrapper_t;
 static int CallFstat(int fd, stat_wrapper_t *sb) {
+  base::ThreadRestrictions::AssertIOAllowed();
   return fstat(fd, sb);
 }
 #else
 typedef struct stat64 stat_wrapper_t;
 static int CallFstat(int fd, stat_wrapper_t *sb) {
+  base::ThreadRestrictions::AssertIOAllowed();
   return fstat64(fd, sb);
 }
 #endif
@@ -32,15 +39,26 @@ static int CallFstat(int fd, stat_wrapper_t *sb) {
 // TODO(erikkay): does it make sense to support PLATFORM_FILE_EXCLUSIVE_* here?
 PlatformFile CreatePlatformFile(const FilePath& name, int flags,
                                 bool* created, PlatformFileError* error_code) {
+  base::ThreadRestrictions::AssertIOAllowed();
+
   int open_flags = 0;
   if (flags & PLATFORM_FILE_CREATE)
     open_flags = O_CREAT | O_EXCL;
 
+  if (created)
+    *created = false;
+
   if (flags & PLATFORM_FILE_CREATE_ALWAYS) {
     DCHECK(!open_flags);
     open_flags = O_CREAT | O_TRUNC;
   }
 
+  if (flags & PLATFORM_FILE_OPEN_TRUNCATED) {
+    DCHECK(!open_flags);
+    DCHECK(flags & PLATFORM_FILE_WRITE);
+    open_flags = O_TRUNC;
+  }
+
   if (!open_flags && !(flags & PLATFORM_FILE_OPEN) &&
       !(flags & PLATFORM_FILE_OPEN_ALWAYS)) {
     NOTREACHED();
@@ -60,26 +78,25 @@ PlatformFile CreatePlatformFile(const FilePath& name, int flags,
     NOTREACHED();
   }
 
-  if (flags & PLATFORM_FILE_TRUNCATE) {
-    DCHECK(flags & PLATFORM_FILE_WRITE);
-    open_flags |= O_TRUNC;
-  }
+  COMPILE_ASSERT(O_RDONLY == 0, O_RDONLY_must_equal_zero);
 
-  DCHECK(O_RDONLY == 0);
+  int mode = S_IRUSR | S_IWUSR;
+#if defined(OS_CHROMEOS)
+  mode |= S_IRGRP | S_IROTH;
+#endif
 
-  int descriptor = open(name.value().c_str(), open_flags, S_IRUSR | S_IWUSR);
+  int descriptor =
+      HANDLE_EINTR(open(name.value().c_str(), open_flags, mode));
 
   if (flags & PLATFORM_FILE_OPEN_ALWAYS) {
-    if (descriptor > 0) {
-      if (created)
-        *created = false;
-    } else {
+    if (descriptor <= 0) {
       open_flags |= O_CREAT;
       if (flags & PLATFORM_FILE_EXCLUSIVE_READ ||
           flags & PLATFORM_FILE_EXCLUSIVE_WRITE) {
         open_flags |= O_EXCL;   // together with O_CREAT implies O_NOFOLLOW
       }
-      descriptor = open(name.value().c_str(), open_flags, S_IRUSR | S_IWUSR);
+      descriptor = HANDLE_EINTR(
+          open(name.value().c_str(), open_flags, mode));
       if (created && descriptor > 0)
         *created = true;
     }
@@ -134,17 +151,33 @@ PlatformFile CreatePlatformFile(const FilePath& name, int flags,
   return descriptor;
 }
 
-PlatformFile CreatePlatformFile(const std::wstring& name, int flags,
-                                bool* created) {
-  return CreatePlatformFile(FilePath::FromWStringHack(name), flags,
-                            created, NULL);
-}
-
 bool ClosePlatformFile(PlatformFile file) {
-  return !close(file);
+  base::ThreadRestrictions::AssertIOAllowed();
+  return !HANDLE_EINTR(close(file));
 }
 
 int ReadPlatformFile(PlatformFile file, int64 offset, char* data, int size) {
+  base::ThreadRestrictions::AssertIOAllowed();
+  if (file < 0 || size < 0)
+    return -1;
+
+  int bytes_read = 0;
+  int rv;
+  do {
+    rv = HANDLE_EINTR(pread(file, data + bytes_read,
+                            size - bytes_read, offset + bytes_read));
+    if (rv <= 0)
+      break;
+
+    bytes_read += rv;
+  } while (bytes_read < size);
+
+  return bytes_read ? bytes_read : rv;
+}
+
+int ReadPlatformFileNoBestEffort(PlatformFile file, int64 offset,
+                                 char* data, int size) {
+  base::ThreadRestrictions::AssertIOAllowed();
   if (file < 0)
     return -1;
 
@@ -153,22 +186,37 @@ int ReadPlatformFile(PlatformFile file, int64 offset, char* data, int size) {
 
 int WritePlatformFile(PlatformFile file, int64 offset,
                       const char* data, int size) {
-  if (file < 0)
+  base::ThreadRestrictions::AssertIOAllowed();
+  if (file < 0 || size < 0)
     return -1;
 
-  return HANDLE_EINTR(pwrite(file, data, size, offset));
+  int bytes_written = 0;
+  int rv;
+  do {
+    rv = HANDLE_EINTR(pwrite(file, data + bytes_written,
+                             size - bytes_written, offset + bytes_written));
+    if (rv <= 0)
+      break;
+
+    bytes_written += rv;
+  } while (bytes_written < size);
+
+  return bytes_written ? bytes_written : rv;
 }
 
 bool TruncatePlatformFile(PlatformFile file, int64 length) {
+  base::ThreadRestrictions::AssertIOAllowed();
   return ((file >= 0) && !HANDLE_EINTR(ftruncate(file, length)));
 }
 
 bool FlushPlatformFile(PlatformFile file) {
-  return !fsync(file);
+  base::ThreadRestrictions::AssertIOAllowed();
+  return !HANDLE_EINTR(fsync(file));
 }
 
 bool TouchPlatformFile(PlatformFile file, const base::Time& last_access_time,
                        const base::Time& last_modified_time) {
+  base::ThreadRestrictions::AssertIOAllowed();
   if (file < 0)
     return false;
 
