@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,6 +17,8 @@ PlatformFile CreatePlatformFile(const FilePath& name,
   base::ThreadRestrictions::AssertIOAllowed();
 
   DWORD disposition = 0;
+  if (created)
+    *created = false;
 
   if (flags & PLATFORM_FILE_OPEN)
     disposition = OPEN_EXISTING;
@@ -36,7 +38,7 @@ PlatformFile CreatePlatformFile(const FilePath& name,
     disposition = CREATE_ALWAYS;
   }
 
-  if (flags & PLATFORM_FILE_TRUNCATE) {
+  if (flags & PLATFORM_FILE_OPEN_TRUNCATED) {
     DCHECK(!disposition);
     DCHECK(flags & PLATFORM_FILE_WRITE);
     disposition = TRUNCATE_EXISTING;
@@ -56,6 +58,8 @@ PlatformFile CreatePlatformFile(const FilePath& name,
   DWORD sharing = (flags & PLATFORM_FILE_EXCLUSIVE_READ) ? 0 : FILE_SHARE_READ;
   if (!(flags & PLATFORM_FILE_EXCLUSIVE_WRITE))
     sharing |= FILE_SHARE_WRITE;
+  if (flags & PLATFORM_FILE_SHARE_DELETE)
+    sharing |= FILE_SHARE_DELETE;
 
   DWORD create_flags = 0;
   if (flags & PLATFORM_FILE_ASYNC)
@@ -104,12 +108,6 @@ PlatformFile CreatePlatformFile(const FilePath& name,
   return file;
 }
 
-PlatformFile CreatePlatformFile(const std::wstring& name, int flags,
-                                bool* created) {
-  return CreatePlatformFile(FilePath::FromWStringHack(name), flags,
-                            created, NULL);
-}
-
 bool ClosePlatformFile(PlatformFile file) {
   base::ThreadRestrictions::AssertIOAllowed();
   return (CloseHandle(file) != 0);
@@ -136,6 +134,11 @@ int ReadPlatformFile(PlatformFile file, int64 offset, char* data, int size) {
   return -1;
 }
 
+int ReadPlatformFileNoBestEffort(PlatformFile file, int64 offset,
+                                 char* data, int size) {
+  return ReadPlatformFile(file, offset, data, size);
+}
+
 int WritePlatformFile(PlatformFile file, int64 offset,
                       const char* data, int size) {
   base::ThreadRestrictions::AssertIOAllowed();
@@ -213,7 +216,7 @@ bool GetPlatformFileInfo(PlatformFile file, PlatformFileInfo* info) {
   size.LowPart = file_info.nFileSizeLow;
   info->size = size.QuadPart;
   info->is_directory =
-      file_info.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY != 0;
+      (file_info.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
   info->is_symbolic_link = false; // Windows doesn't have symbolic links.
   info->last_modified = base::Time::FromFileTime(file_info.ftLastWriteTime);
   info->last_accessed = base::Time::FromFileTime(file_info.ftLastAccessTime);
@@ -221,4 +224,4 @@ bool GetPlatformFileInfo(PlatformFile file, PlatformFileInfo* info) {
   return true;
 }
 
-}  // namespace disk_cache
+}  // namespace base
