@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -35,25 +35,20 @@ static void SplitStringT(const STR& str,
   }
 }
 
-void SplitString(const std::wstring& str,
-                 wchar_t c,
-                 std::vector<std::wstring>* r) {
-  SplitStringT(str, c, true, r);
-}
-
-#if !defined(WCHAR_T_IS_UTF16)
 void SplitString(const string16& str,
                  char16 c,
                  std::vector<string16>* r) {
   DCHECK(CBU16_IS_SINGLE(c));
   SplitStringT(str, c, true, r);
 }
-#endif
 
 void SplitString(const std::string& str,
                  char c,
                  std::vector<std::string>* r) {
-  DCHECK(c >= 0 && c < 0x7F);
+#if CHAR_MIN < 0
+  DCHECK(c >= 0);
+#endif
+  DCHECK(c < 0x7F);
   SplitStringT(str, c, true, r);
 }
 
@@ -164,7 +159,10 @@ void SplitStringDontTrim(const std::string& str,
                          char c,
                          std::vector<std::string>* r) {
   DCHECK(IsStringUTF8(str));
-  DCHECK(c >= 0 && c < 0x7F);
+#if CHAR_MIN < 0
+  DCHECK(c >= 0);
+#endif
+  DCHECK(c < 0x7F);
   SplitStringT(str, c, false, r);
 }
 
@@ -208,17 +206,10 @@ void SplitStringAlongWhitespaceT(const STR& str, std::vector<STR>* result) {
   }
 }
 
-void SplitStringAlongWhitespace(const std::wstring& str,
-                                std::vector<std::wstring>* result) {
-  SplitStringAlongWhitespaceT(str, result);
-}
-
-#if !defined(WCHAR_T_IS_UTF16)
 void SplitStringAlongWhitespace(const string16& str,
                                 std::vector<string16>* result) {
   SplitStringAlongWhitespaceT(str, result);
 }
-#endif
 
 void SplitStringAlongWhitespace(const std::string& str,
                                 std::vector<std::string>* result) {
