@@ -9,7 +9,7 @@
 #include <vector>
 
 #include "base/logging.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/spin_wait.h"
 #include "base/synchronization/condition_variable.h"
 #include "base/synchronization/lock.h"
@@ -661,7 +661,7 @@ void WorkQueue::SpinUntilAllThreadsAreWaiting() {
       if (waiting_thread_count_ == thread_count_)
         break;
     }
-    PlatformThread::Sleep(30);
+    PlatformThread::Sleep(TimeDelta::FromMilliseconds(30));
   }
 }
 
@@ -672,7 +672,7 @@ void WorkQueue::SpinUntilTaskCountLessThan(int task_count) {
       if (task_count_ < task_count)
         break;
     }
-    PlatformThread::Sleep(30);
+    PlatformThread::Sleep(TimeDelta::FromMilliseconds(30));
   }
 }
 
