@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,8 +14,8 @@ class PerfTestSuite : public TestSuite {
  public:
   PerfTestSuite(int argc, char** argv);
 
-  virtual void Initialize();
-  virtual void Shutdown();
+  virtual void Initialize() OVERRIDE;
+  virtual void Shutdown() OVERRIDE;
 };
 
 }  // namespace base
