@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,15 @@
 #define BASE_THREADING_WORKER_POOL_H_
 #pragma once
 
-#include "base/tracked.h"
+#include "base/base_export.h"
+#include "base/callback_forward.h"
 
 class Task;
 
+namespace tracked_objects {
+class Location;
+}  // namespace tracked_objects
+
 namespace base {
 
 // This is a facility that runs tasks that don't require a specific thread or
@@ -20,14 +25,22 @@ namespace base {
 // inside the pool must be extremely careful about other objects they access
 // (MessageLoops, Singletons, etc). During shutdown these object may no longer
 // exist.
-class WorkerPool {
+class BASE_EXPORT WorkerPool {
  public:
   // This function posts |task| to run on a worker thread.  |task_is_slow|
   // should be used for tasks that will take a long time to execute.  Returns
   // false if |task| could not be posted to a worker thread.  Regardless of
   // return value, ownership of |task| is transferred to the worker pool.
   static bool PostTask(const tracked_objects::Location& from_here,
-                       Task* task, bool task_is_slow);
+                       const base::Closure& task, bool task_is_slow);
+
+  // Just like MessageLoopProxy::PostTaskAndReply, except the destination
+  // for |task| is a worker thread and you can specify |task_is_slow| just
+  // like you can for PostTask above.
+  static bool PostTaskAndReply(const tracked_objects::Location& from_here,
+                               const Closure& task,
+                               const Closure& reply,
+                               bool task_is_slow);
 };
 
 }  // namespace base
