@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -43,7 +43,7 @@
 #include "base/basictypes.h"
 #include "base/logging.h"
 #include "base/cpu.h"
-#include "base/singleton.h"
+#include "base/memory/singleton.h"
 #include "base/synchronization/lock.h"
 
 using base::Time;
@@ -62,7 +62,7 @@ int64 FileTimeToMicroseconds(const FILETIME& ft) {
 }
 
 void MicrosecondsToFileTime(int64 us, FILETIME* ft) {
-  DCHECK(us >= 0) << "Time is less than 0, negative values are not "
+  DCHECK_GE(us, 0LL) << "Time is less than 0, negative values are not "
       "representable in FILETIME";
 
   // Multiply by 10 to convert milliseconds to 100-nanoseconds. Bit_cast will
@@ -99,6 +99,7 @@ void InitializeClock() {
 const int64 Time::kTimeTToMicrosecondsOffset = GG_INT64_C(11644473600000000);
 
 bool Time::high_resolution_timer_enabled_ = false;
+int Time::high_resolution_timer_activated_ = 0;
 
 // static
 Time Time::Now() {
@@ -162,21 +163,35 @@ void Time::EnableHighResolutionTimer(bool enable) {
 }
 
 // static
-bool Time::ActivateHighResolutionTimer(bool activate) {
-  if (!high_resolution_timer_enabled_)
+bool Time::ActivateHighResolutionTimer(bool activating) {
+  if (!high_resolution_timer_enabled_ && activating)
     return false;
 
   // Using anything other than 1ms makes timers granular
   // to that interval.
   const int kMinTimerIntervalMs = 1;
   MMRESULT result;
-  if (activate)
+  if (activating) {
     result = timeBeginPeriod(kMinTimerIntervalMs);
-  else
+    high_resolution_timer_activated_++;
+  } else {
     result = timeEndPeriod(kMinTimerIntervalMs);
+    high_resolution_timer_activated_--;
+  }
   return result == TIMERR_NOERROR;
 }
 
+// static
+bool Time::IsHighResolutionTimerInUse() {
+  // Note:  we should track the high_resolution_timer_activated_ value
+  // under a lock if we want it to be accurate in a system with multiple
+  // message loops.  We don't do that - because we don't want to take the
+  // expense of a lock for this.  We *only* track this value so that unit
+  // tests can see if the high resolution timer is on or off.
+  return high_resolution_timer_enabled_ &&
+      high_resolution_timer_activated_ > 0;
+}
+
 // static
 Time Time::FromExploded(bool is_local, const Exploded& exploded) {
   // Create the system struct representing our exploded time. It will either be
@@ -191,40 +206,54 @@ Time Time::FromExploded(bool is_local, const Exploded& exploded) {
   st.wSecond = exploded.second;
   st.wMilliseconds = exploded.millisecond;
 
-  // Convert to FILETIME.
   FILETIME ft;
-  if (!SystemTimeToFileTime(&st, &ft)) {
-    NOTREACHED() << "Unable to convert time";
-    return Time(0);
-  }
-
+  bool success = true;
   // Ensure that it's in UTC.
   if (is_local) {
-    FILETIME utc_ft;
-    LocalFileTimeToFileTime(&ft, &utc_ft);
-    return Time(FileTimeToMicroseconds(utc_ft));
+    SYSTEMTIME utc_st;
+    success = TzSpecificLocalTimeToSystemTime(NULL, &st, &utc_st) &&
+              SystemTimeToFileTime(&utc_st, &ft);
+  } else {
+    success = !!SystemTimeToFileTime(&st, &ft);
+  }
+
+  if (!success) {
+    NOTREACHED() << "Unable to convert time";
+    return Time(0);
   }
   return Time(FileTimeToMicroseconds(ft));
 }
 
 void Time::Explode(bool is_local, Exploded* exploded) const {
+  if (us_ < 0LL) {
+    // We are not able to convert it to FILETIME.
+    ZeroMemory(exploded, sizeof(*exploded));
+    return;
+  }
+
   // FILETIME in UTC.
   FILETIME utc_ft;
   MicrosecondsToFileTime(us_, &utc_ft);
 
   // FILETIME in local time if necessary.
-  BOOL success = TRUE;
-  FILETIME ft;
-  if (is_local)
-    success = FileTimeToLocalFileTime(&utc_ft, &ft);
-  else
-    ft = utc_ft;
-
+  bool success = true;
   // FILETIME in SYSTEMTIME (exploded).
   SYSTEMTIME st;
-  if (!success || !FileTimeToSystemTime(&ft, &st)) {
+  if (is_local) {
+    SYSTEMTIME utc_st;
+    // We don't use FileTimeToLocalFileTime here, since it uses the current
+    // settings for the time zone and daylight saving time. Therefore, if it is
+    // daylight saving time, it will take daylight saving time into account,
+    // even if the time you are converting is in standard time.
+    success = FileTimeToSystemTime(&utc_ft, &utc_st) &&
+              SystemTimeToTzSpecificLocalTime(NULL, &utc_st, &st);
+  } else {
+    success = !!FileTimeToSystemTime(&utc_ft, &st);
+  }
+
+  if (!success) {
     NOTREACHED() << "Unable to convert time, don't know why";
-    ZeroMemory(exploded, sizeof(exploded));
+    ZeroMemory(exploded, sizeof(*exploded));
     return;
   }
 
@@ -334,7 +363,12 @@ class HighResNowSingleton {
     if (!IsUsingHighResClock())
       return 0;
 
-    return abs((UnreliableNow() - ReliableNow()) - skew_);
+    // The static_cast<long> is needed as a hint to VS 2008 to tell it
+    // which version of abs() to use. Other compilers don't seem to
+    // need it, including VS 2010, but to keep code identical we use it
+    // everywhere.
+    // TODO(joi): Remove the hint if/when we no longer support VS 2008.
+    return abs(static_cast<long>((UnreliableNow() - ReliableNow()) - skew_));
   }
 
  private:
