@@ -1,9 +1,11 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/timer.h"
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/message_loop.h"
 
 namespace base {
@@ -21,8 +23,9 @@ void BaseTimer_Helper::InitiateDelayedTask(TimerTask* timer_task) {
   delayed_task_ = timer_task;
   delayed_task_->timer_ = this;
   MessageLoop::current()->PostDelayedTask(
-      FROM_HERE, timer_task,
-      static_cast<int>(timer_task->delay_.InMillisecondsRoundedUp()));
+      timer_task->posted_from_,
+      base::Bind(&TimerTask::Run, base::Owned(timer_task)),
+      timer_task->delay_);
 }
 
 }  // namespace base
