@@ -27,30 +27,38 @@
 #include <string>
 #include <vector>
 
+#include "base/base_export.h"
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "base/string16.h"
-#include "build/build_config.h"
 
-class Value;
-class FundamentalValue;
-class StringValue;
+// This file declares "using base::Value", etc. at the bottom, so that
+// current code can use these classes without the base namespace. In
+// new code, please always use base::Value, etc. or add your own
+// "using" declaration.
+// http://crbug.com/88666
+namespace base {
+
 class BinaryValue;
 class DictionaryValue;
+class FundamentalValue;
 class ListValue;
+class StringValue;
+class Value;
 
 typedef std::vector<Value*> ValueVector;
 typedef std::map<std::string, Value*> ValueMap;
 
-// The Value class is the base class for Values.  A Value can be
-// instantiated via the Create*Value() factory methods, or by directly
-// creating instances of the subclasses.
-class Value {
+// The Value class is the base class for Values. A Value can be instantiated
+// via the Create*Value() factory methods, or by directly creating instances of
+// the subclasses.
+class BASE_EXPORT Value {
  public:
-  enum ValueType {
+  enum Type {
     TYPE_NULL = 0,
     TYPE_BOOLEAN,
     TYPE_INTEGER,
-    TYPE_REAL,
+    TYPE_DOUBLE,
     TYPE_STRING,
     TYPE_BINARY,
     TYPE_DICTIONARY,
@@ -65,23 +73,19 @@ class Value {
   static Value* CreateNullValue();
   static FundamentalValue* CreateBooleanValue(bool in_value);
   static FundamentalValue* CreateIntegerValue(int in_value);
-  static FundamentalValue* CreateRealValue(double in_value);
+  static FundamentalValue* CreateDoubleValue(double in_value);
   static StringValue* CreateStringValue(const std::string& in_value);
   static StringValue* CreateStringValue(const string16& in_value);
 
-  // This one can return NULL if the input isn't valid.  If the return value
-  // is non-null, the new object has taken ownership of the buffer pointer.
-  static BinaryValue* CreateBinaryValue(char* buffer, size_t size);
-
   // Returns the type of the value stored by the current Value object.
   // Each type will be implemented by only one subclass of Value, so it's
-  // safe to use the ValueType to determine whether you can cast from
+  // safe to use the Type to determine whether you can cast from
   // Value* to (Implementing Class)*.  Also, a Value object never changes
   // its type after construction.
-  ValueType GetType() const { return type_; }
+  Type GetType() const { return type_; }
 
   // Returns true if the current object represents a given type.
-  bool IsType(ValueType type) const { return type == type_; }
+  bool IsType(Type type) const { return type == type_; }
 
   // These methods allow the convenient retrieval of settings.
   // If the current setting object can be converted into the given type,
@@ -89,10 +93,13 @@ class Value {
   // returned;  otherwise, false is returned and |out_value| is unchanged.
   virtual bool GetAsBoolean(bool* out_value) const;
   virtual bool GetAsInteger(int* out_value) const;
-  virtual bool GetAsReal(double* out_value) const;
+  virtual bool GetAsDouble(double* out_value) const;
   virtual bool GetAsString(std::string* out_value) const;
   virtual bool GetAsString(string16* out_value) const;
   virtual bool GetAsList(ListValue** out_value);
+  virtual bool GetAsList(const ListValue** out_value) const;
+  virtual bool GetAsDictionary(DictionaryValue** out_value);
+  virtual bool GetAsDictionary(const DictionaryValue** out_value) const;
 
   // This creates a deep copy of the entire Value tree, and returns a pointer
   // to the copy.  The caller gets ownership of the copy, of course.
@@ -111,42 +118,42 @@ class Value {
  protected:
   // This isn't safe for end-users (they should use the Create*Value()
   // static methods above), but it's useful for subclasses.
-  explicit Value(ValueType type);
+  explicit Value(Type type);
 
  private:
   Value();
 
-  ValueType type_;
+  Type type_;
 
   DISALLOW_COPY_AND_ASSIGN(Value);
 };
 
 // FundamentalValue represents the simple fundamental types of values.
-class FundamentalValue : public Value {
+class BASE_EXPORT FundamentalValue : public Value {
  public:
   explicit FundamentalValue(bool in_value);
   explicit FundamentalValue(int in_value);
   explicit FundamentalValue(double in_value);
   virtual ~FundamentalValue();
 
-  // Subclassed methods
-  virtual bool GetAsBoolean(bool* out_value) const;
-  virtual bool GetAsInteger(int* out_value) const;
-  virtual bool GetAsReal(double* out_value) const;
-  virtual FundamentalValue* DeepCopy() const;
-  virtual bool Equals(const Value* other) const;
+  // Overridden from Value:
+  virtual bool GetAsBoolean(bool* out_value) const OVERRIDE;
+  virtual bool GetAsInteger(int* out_value) const OVERRIDE;
+  virtual bool GetAsDouble(double* out_value) const OVERRIDE;
+  virtual FundamentalValue* DeepCopy() const OVERRIDE;
+  virtual bool Equals(const Value* other) const OVERRIDE;
 
  private:
   union {
     bool boolean_value_;
     int integer_value_;
-    double real_value_;
+    double double_value_;
   };
 
   DISALLOW_COPY_AND_ASSIGN(FundamentalValue);
 };
 
-class StringValue : public Value {
+class BASE_EXPORT StringValue : public Value {
  public:
   // Initializes a StringValue with a UTF-8 narrow character string.
   explicit StringValue(const std::string& in_value);
@@ -156,11 +163,11 @@ class StringValue : public Value {
 
   virtual ~StringValue();
 
-  // Subclassed methods
-  virtual bool GetAsString(std::string* out_value) const;
-  virtual bool GetAsString(string16* out_value) const;
-  virtual StringValue* DeepCopy() const;
-  virtual bool Equals(const Value* other) const;
+  // Overridden from Value:
+  virtual bool GetAsString(std::string* out_value) const OVERRIDE;
+  virtual bool GetAsString(string16* out_value) const OVERRIDE;
+  virtual StringValue* DeepCopy() const OVERRIDE;
+  virtual bool Equals(const Value* other) const OVERRIDE;
 
  private:
   std::string value_;
@@ -168,7 +175,7 @@ class StringValue : public Value {
   DISALLOW_COPY_AND_ASSIGN(StringValue);
 };
 
-class BinaryValue: public Value {
+class BASE_EXPORT BinaryValue: public Value {
  public:
   virtual ~BinaryValue();
 
@@ -188,8 +195,8 @@ class BinaryValue: public Value {
   const char* GetBuffer() const { return buffer_; }
 
   // Overridden from Value:
-  virtual BinaryValue* DeepCopy() const;
-  virtual bool Equals(const Value* other) const;
+  virtual BinaryValue* DeepCopy() const OVERRIDE;
+  virtual bool Equals(const Value* other) const OVERRIDE;
 
  private:
   // Constructor is private so that only objects with valid buffer pointers
@@ -205,11 +212,16 @@ class BinaryValue: public Value {
 // DictionaryValue provides a key-value dictionary with (optional) "path"
 // parsing for recursive access; see the comment at the top of the file. Keys
 // are |std::string|s and should be UTF-8 encoded.
-class DictionaryValue : public Value {
+class BASE_EXPORT DictionaryValue : public Value {
  public:
   DictionaryValue();
   virtual ~DictionaryValue();
 
+  // Overridden from Value:
+  virtual bool GetAsDictionary(DictionaryValue** out_value) OVERRIDE;
+  virtual bool GetAsDictionary(
+      const DictionaryValue** out_value) const OVERRIDE;
+
   // Returns true if the current dictionary has a value for the given key.
   bool HasKey(const std::string& key) const;
 
@@ -237,7 +249,7 @@ class DictionaryValue : public Value {
   // value at that path, even if it has a different type.
   void SetBoolean(const std::string& path, bool in_value);
   void SetInteger(const std::string& path, int in_value);
-  void SetReal(const std::string& path, double in_value);
+  void SetDouble(const std::string& path, double in_value);
   void SetString(const std::string& path, const std::string& in_value);
   void SetString(const std::string& path, const string16& in_value);
 
@@ -259,7 +271,7 @@ class DictionaryValue : public Value {
   // the end of the path can be returned in the form specified.
   bool GetBoolean(const std::string& path, bool* out_value) const;
   bool GetInteger(const std::string& path, int* out_value) const;
-  bool GetReal(const std::string& path, double* out_value) const;
+  bool GetDouble(const std::string& path, double* out_value) const;
   bool GetString(const std::string& path, std::string* out_value) const;
   bool GetString(const std::string& path, string16* out_value) const;
   bool GetStringASCII(const std::string& path, std::string* out_value) const;
@@ -274,7 +286,7 @@ class DictionaryValue : public Value {
                                Value** out_value) const;
   bool GetIntegerWithoutPathExpansion(const std::string& key,
                                       int* out_value) const;
-  bool GetRealWithoutPathExpansion(const std::string& key,
+  bool GetDoubleWithoutPathExpansion(const std::string& key,
                                    double* out_value) const;
   bool GetStringWithoutPathExpansion(const std::string& key,
                                      std::string* out_value) const;
@@ -307,6 +319,11 @@ class DictionaryValue : public Value {
   // replaced.
   void MergeDictionary(const DictionaryValue* dictionary);
 
+  // Swaps contents with the |other| dictionary.
+  void Swap(DictionaryValue* other) {
+    dictionary_.swap(other->dictionary_);
+  }
+
   // This class provides an iterator for the keys in the dictionary.
   // It can't be used to modify the dictionary.
   //
@@ -332,9 +349,27 @@ class DictionaryValue : public Value {
   key_iterator begin_keys() const { return key_iterator(dictionary_.begin()); }
   key_iterator end_keys() const { return key_iterator(dictionary_.end()); }
 
+  // This class provides an iterator over both keys and values in the
+  // dictionary.  It can't be used to modify the dictionary.
+  class Iterator {
+   public:
+    explicit Iterator(const DictionaryValue& target)
+        : target_(target), it_(target.dictionary_.begin()) {}
+
+    bool HasNext() const { return it_ != target_.dictionary_.end(); }
+    void Advance() { ++it_; }
+
+    const std::string& key() const { return it_->first; }
+    const Value& value() const { return *it_->second; }
+
+   private:
+    const DictionaryValue& target_;
+    ValueMap::const_iterator it_;
+  };
+
   // Overridden from Value:
-  virtual DictionaryValue* DeepCopy() const;
-  virtual bool Equals(const Value* other) const;
+  virtual DictionaryValue* DeepCopy() const OVERRIDE;
+  virtual bool Equals(const Value* other) const OVERRIDE;
 
  private:
   ValueMap dictionary_;
@@ -343,13 +378,13 @@ class DictionaryValue : public Value {
 };
 
 // This type of Value represents a list of other Value values.
-class ListValue : public Value {
+class BASE_EXPORT ListValue : public Value {
  public:
   typedef ValueVector::iterator iterator;
   typedef ValueVector::const_iterator const_iterator;
 
   ListValue();
-  ~ListValue();
+  virtual ~ListValue();
 
   // Clears the contents of this ListValue
   void Clear();
@@ -377,7 +412,7 @@ class ListValue : public Value {
   // in the specified form.
   bool GetBoolean(size_t index, bool* out_value) const;
   bool GetInteger(size_t index, int* out_value) const;
-  bool GetReal(size_t index, double* out_value) const;
+  bool GetDouble(size_t index, double* out_value) const;
   bool GetString(size_t index, std::string* out_value) const;
   bool GetString(size_t index, string16* out_value) const;
   bool GetBinary(size_t index, BinaryValue** out_value) const;
@@ -392,36 +427,44 @@ class ListValue : public Value {
   bool Remove(size_t index, Value** out_value);
 
   // Removes the first instance of |value| found in the list, if any, and
-  // deletes it.  Returns the index that it was located at (-1 for not present).
-  int Remove(const Value& value);
+  // deletes it. |index| is the location where |value| was found. Returns false
+  // if not found.
+  bool Remove(const Value& value, size_t* index);
 
   // Appends a Value to the end of the list.
   void Append(Value* in_value);
 
-  // Appends a Value if it's not already present.
-  // Returns true if successful, or false if the value was already present.
+  // Appends a Value if it's not already present. Takes ownership of the
+  // |in_value|. Returns true if successful, or false if the value was already
+  // present. If the value was already present the |in_value| is deleted.
   bool AppendIfNotPresent(Value* in_value);
 
   // Insert a Value at index.
   // Returns true if successful, or false if the index was out of range.
   bool Insert(size_t index, Value* in_value);
 
+  // Searches for the first instance of |value| in the list using the Equals
+  // method of the Value type.
+  // Returns a const_iterator to the found item or to end() if none exists.
+  const_iterator Find(const Value& value) const;
+
   // Swaps contents with the |other| list.
   void Swap(ListValue* other) {
     list_.swap(other->list_);
   }
 
-  // Iteration
-  ListValue::iterator begin() { return list_.begin(); }
-  ListValue::iterator end() { return list_.end(); }
+  // Iteration.
+  iterator begin() { return list_.begin(); }
+  iterator end() { return list_.end(); }
 
-  ListValue::const_iterator begin() const { return list_.begin(); }
-  ListValue::const_iterator end() const { return list_.end(); }
+  const_iterator begin() const { return list_.begin(); }
+  const_iterator end() const { return list_.end(); }
 
   // Overridden from Value:
-  virtual bool GetAsList(ListValue** out_value);
-  virtual ListValue* DeepCopy() const;
-  virtual bool Equals(const Value* other) const;
+  virtual bool GetAsList(ListValue** out_value) OVERRIDE;
+  virtual bool GetAsList(const ListValue** out_value) const OVERRIDE;
+  virtual ListValue* DeepCopy() const OVERRIDE;
+  virtual bool Equals(const Value* other) const OVERRIDE;
 
  private:
   ValueVector list_;
@@ -431,7 +474,7 @@ class ListValue : public Value {
 
 // This interface is implemented by classes that know how to serialize and
 // deserialize Value objects.
-class ValueSerializer {
+class BASE_EXPORT ValueSerializer {
  public:
   virtual ~ValueSerializer();
 
@@ -446,4 +489,12 @@ class ValueSerializer {
   virtual Value* Deserialize(int* error_code, std::string* error_str) = 0;
 };
 
+}  // namespace base
+
+// http://crbug.com/88666
+using base::DictionaryValue;
+using base::ListValue;
+using base::StringValue;
+using base::Value;
+
 #endif  // BASE_VALUES_H_
