@@ -1,19 +1,18 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <limits>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string16.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-class ValuesTest : public testing::Test {
-};
+namespace base {
 
-TEST_F(ValuesTest, Basic) {
+TEST(ValuesTest, Basic) {
   // Test basic dictionary getting/setting
   DictionaryValue settings;
   std::string homepage = "http://google.com";
@@ -56,11 +55,11 @@ TEST_F(ValuesTest, Basic) {
   ASSERT_EQ(std::string("http://froogle.com"), bookmark_url);
 }
 
-TEST_F(ValuesTest, List) {
+TEST(ValuesTest, List) {
   scoped_ptr<ListValue> mixed_list(new ListValue());
   mixed_list->Set(0, Value::CreateBooleanValue(true));
   mixed_list->Set(1, Value::CreateIntegerValue(42));
-  mixed_list->Set(2, Value::CreateRealValue(88.8));
+  mixed_list->Set(2, Value::CreateDoubleValue(88.8));
   mixed_list->Set(3, Value::CreateStringValue("foo"));
   ASSERT_EQ(4u, mixed_list->GetSize());
 
@@ -74,10 +73,12 @@ TEST_F(ValuesTest, List) {
 
   ASSERT_FALSE(mixed_list->GetInteger(0, &int_value));
   ASSERT_EQ(0, int_value);
-  ASSERT_FALSE(mixed_list->GetReal(1, &double_value));
-  ASSERT_EQ(0.0, double_value);
+  ASSERT_FALSE(mixed_list->GetBoolean(1, &bool_value));
+  ASSERT_FALSE(bool_value);
   ASSERT_FALSE(mixed_list->GetString(2, &string_value));
   ASSERT_EQ("", string_value);
+  ASSERT_FALSE(mixed_list->GetInteger(2, &int_value));
+  ASSERT_EQ(0, int_value);
   ASSERT_FALSE(mixed_list->GetBoolean(3, &bool_value));
   ASSERT_FALSE(bool_value);
 
@@ -85,13 +86,25 @@ TEST_F(ValuesTest, List) {
   ASSERT_TRUE(bool_value);
   ASSERT_TRUE(mixed_list->GetInteger(1, &int_value));
   ASSERT_EQ(42, int_value);
-  ASSERT_TRUE(mixed_list->GetReal(2, &double_value));
+  // implicit conversion from Integer to Double should be possible.
+  ASSERT_TRUE(mixed_list->GetDouble(1, &double_value));
+  ASSERT_EQ(42, double_value);
+  ASSERT_TRUE(mixed_list->GetDouble(2, &double_value));
   ASSERT_EQ(88.8, double_value);
   ASSERT_TRUE(mixed_list->GetString(3, &string_value));
   ASSERT_EQ("foo", string_value);
+
+  // Try searching in the mixed list.
+  scoped_ptr<Value> sought_value(Value::CreateIntegerValue(42));
+  scoped_ptr<Value> not_found_value(Value::CreateBooleanValue(false));
+
+  ASSERT_NE(mixed_list->end(), mixed_list->Find(*sought_value));
+  ASSERT_TRUE((*mixed_list->Find(*sought_value))->GetAsInteger(&int_value));
+  ASSERT_EQ(42, int_value);
+  ASSERT_EQ(mixed_list->end(), mixed_list->Find(*not_found_value));
 }
 
-TEST_F(ValuesTest, BinaryValue) {
+TEST(ValuesTest, BinaryValue) {
   char* buffer = NULL;
   // Passing a null buffer pointer doesn't yield a BinaryValue
   scoped_ptr<BinaryValue> binary(BinaryValue::Create(buffer, 0));
@@ -124,7 +137,7 @@ TEST_F(ValuesTest, BinaryValue) {
   ASSERT_EQ(0, memcmp(stack_buffer, binary->GetBuffer(), binary->GetSize()));
 }
 
-TEST_F(ValuesTest, StringValue) {
+TEST(ValuesTest, StringValue) {
   // Test overloaded CreateStringValue.
   scoped_ptr<Value> narrow_value(Value::CreateStringValue("narrow"));
   ASSERT_TRUE(narrow_value.get());
@@ -170,7 +183,7 @@ class DeletionTestValue : public Value {
   bool* deletion_flag_;
 };
 
-TEST_F(ValuesTest, ListDeletion) {
+TEST(ValuesTest, ListDeletion) {
   bool deletion_flag = true;
 
   {
@@ -197,7 +210,7 @@ TEST_F(ValuesTest, ListDeletion) {
   }
 }
 
-TEST_F(ValuesTest, ListRemoval) {
+TEST(ValuesTest, ListRemoval) {
   bool deletion_flag = true;
   Value* removed_item = NULL;
 
@@ -232,13 +245,15 @@ TEST_F(ValuesTest, ListRemoval) {
     DeletionTestValue* value = new DeletionTestValue(&deletion_flag);
     list.Append(value);
     EXPECT_FALSE(deletion_flag);
-    EXPECT_EQ(0, list.Remove(*value));
+    size_t index = 0;
+    list.Remove(*value, &index);
+    EXPECT_EQ(0U, index);
     EXPECT_TRUE(deletion_flag);
     EXPECT_EQ(0U, list.GetSize());
   }
 }
 
-TEST_F(ValuesTest, DictionaryDeletion) {
+TEST(ValuesTest, DictionaryDeletion) {
   std::string key = "test";
   bool deletion_flag = true;
 
@@ -266,7 +281,7 @@ TEST_F(ValuesTest, DictionaryDeletion) {
   }
 }
 
-TEST_F(ValuesTest, DictionaryRemoval) {
+TEST(ValuesTest, DictionaryRemoval) {
   std::string key = "test";
   bool deletion_flag = true;
   Value* removed_item = NULL;
@@ -297,7 +312,7 @@ TEST_F(ValuesTest, DictionaryRemoval) {
   }
 }
 
-TEST_F(ValuesTest, DictionaryWithoutPathExpansion) {
+TEST(ValuesTest, DictionaryWithoutPathExpansion) {
   DictionaryValue dict;
   dict.Set("this.is.expanded", Value::CreateNullValue());
   dict.SetWithoutPathExpansion("this.isnt.expanded", Value::CreateNullValue());
@@ -319,7 +334,7 @@ TEST_F(ValuesTest, DictionaryWithoutPathExpansion) {
   EXPECT_EQ(Value::TYPE_NULL, value4->GetType());
 }
 
-TEST_F(ValuesTest, DeepCopy) {
+TEST(ValuesTest, DeepCopy) {
   DictionaryValue original_dict;
   Value* original_null = Value::CreateNullValue();
   original_dict.Set("null", original_null);
@@ -327,8 +342,8 @@ TEST_F(ValuesTest, DeepCopy) {
   original_dict.Set("bool", original_bool);
   FundamentalValue* original_int = Value::CreateIntegerValue(42);
   original_dict.Set("int", original_int);
-  FundamentalValue* original_real = Value::CreateRealValue(3.14);
-  original_dict.Set("real", original_real);
+  FundamentalValue* original_double = Value::CreateDoubleValue(3.14);
+  original_dict.Set("double", original_double);
   StringValue* original_string = Value::CreateStringValue("hello");
   original_dict.Set("string", original_string);
   StringValue* original_string16 =
@@ -337,7 +352,7 @@ TEST_F(ValuesTest, DeepCopy) {
 
   char* original_buffer = new char[42];
   memset(original_buffer, '!', 42);
-  BinaryValue* original_binary = Value::CreateBinaryValue(original_buffer, 42);
+  BinaryValue* original_binary = BinaryValue::Create(original_buffer, 42);
   original_dict.Set("binary", original_binary);
 
   ListValue* original_list = new ListValue();
@@ -347,6 +362,10 @@ TEST_F(ValuesTest, DeepCopy) {
   original_list->Append(original_list_element_1);
   original_dict.Set("list", original_list);
 
+  DictionaryValue* original_nested_dictionary = new DictionaryValue();
+  original_nested_dictionary->Set("key", Value::CreateStringValue("value"));
+  original_dict.Set("dictionary", original_nested_dictionary);
+
   scoped_ptr<DictionaryValue> copy_dict(original_dict.DeepCopy());
   ASSERT_TRUE(copy_dict.get());
   ASSERT_NE(copy_dict.get(), &original_dict);
@@ -375,14 +394,14 @@ TEST_F(ValuesTest, DeepCopy) {
   ASSERT_TRUE(copy_int->GetAsInteger(&copy_int_value));
   ASSERT_EQ(42, copy_int_value);
 
-  Value* copy_real = NULL;
-  ASSERT_TRUE(copy_dict->Get("real", &copy_real));
-  ASSERT_TRUE(copy_real);
-  ASSERT_NE(copy_real, original_real);
-  ASSERT_TRUE(copy_real->IsType(Value::TYPE_REAL));
-  double copy_real_value = 0;
-  ASSERT_TRUE(copy_real->GetAsReal(&copy_real_value));
-  ASSERT_EQ(3.14, copy_real_value);
+  Value* copy_double = NULL;
+  ASSERT_TRUE(copy_dict->Get("double", &copy_double));
+  ASSERT_TRUE(copy_double);
+  ASSERT_NE(copy_double, original_double);
+  ASSERT_TRUE(copy_double->IsType(Value::TYPE_DOUBLE));
+  double copy_double_value = 0;
+  ASSERT_TRUE(copy_double->GetAsDouble(&copy_double_value));
+  ASSERT_EQ(3.14, copy_double_value);
 
   Value* copy_string = NULL;
   ASSERT_TRUE(copy_dict->Get("string", &copy_string));
@@ -424,7 +443,9 @@ TEST_F(ValuesTest, DeepCopy) {
   ASSERT_TRUE(copy_value);
   ASSERT_NE(copy_value, original_list);
   ASSERT_TRUE(copy_value->IsType(Value::TYPE_LIST));
-  ListValue* copy_list = static_cast<ListValue*>(copy_value);
+  ListValue* copy_list = NULL;
+  ASSERT_TRUE(copy_value->GetAsList(&copy_list));
+  ASSERT_TRUE(copy_list);
   ASSERT_EQ(2U, copy_list->GetSize());
 
   Value* copy_list_element_0;
@@ -442,9 +463,19 @@ TEST_F(ValuesTest, DeepCopy) {
   int copy_list_element_1_value;
   ASSERT_TRUE(copy_list_element_1->GetAsInteger(&copy_list_element_1_value));
   ASSERT_EQ(1, copy_list_element_1_value);
+
+  copy_value = NULL;
+  ASSERT_TRUE(copy_dict->Get("dictionary", &copy_value));
+  ASSERT_TRUE(copy_value);
+  ASSERT_NE(copy_value, original_nested_dictionary);
+  ASSERT_TRUE(copy_value->IsType(Value::TYPE_DICTIONARY));
+  DictionaryValue* copy_nested_dictionary = NULL;
+  ASSERT_TRUE(copy_value->GetAsDictionary(&copy_nested_dictionary));
+  ASSERT_TRUE(copy_nested_dictionary);
+  EXPECT_TRUE(copy_nested_dictionary->HasKey("key"));
 }
 
-TEST_F(ValuesTest, Equals) {
+TEST(ValuesTest, Equals) {
   Value* null1 = Value::CreateNullValue();
   Value* null2 = Value::CreateNullValue();
   EXPECT_NE(null1, null2);
@@ -459,7 +490,7 @@ TEST_F(ValuesTest, Equals) {
   DictionaryValue dv;
   dv.SetBoolean("a", false);
   dv.SetInteger("b", 2);
-  dv.SetReal("c", 2.5);
+  dv.SetDouble("c", 2.5);
   dv.SetString("d1", "string");
   dv.SetString("d2", ASCIIToUTF16("http://google.com"));
   dv.Set("e", Value::CreateNullValue());
@@ -488,7 +519,7 @@ TEST_F(ValuesTest, Equals) {
   EXPECT_FALSE(dv.Equals(copy.get()));
 }
 
-TEST_F(ValuesTest, StaticEquals) {
+TEST(ValuesTest, StaticEquals) {
   scoped_ptr<Value> null1(Value::CreateNullValue());
   scoped_ptr<Value> null2(Value::CreateNullValue());
   EXPECT_TRUE(Value::Equals(null1.get(), null2.get()));
@@ -511,7 +542,7 @@ TEST_F(ValuesTest, StaticEquals) {
   EXPECT_FALSE(Value::Equals(NULL, null1.get()));
 }
 
-TEST_F(ValuesTest, DeepCopyCovariantReturnTypes) {
+TEST(ValuesTest, DeepCopyCovariantReturnTypes) {
   DictionaryValue original_dict;
   Value* original_null = Value::CreateNullValue();
   original_dict.Set("null", original_null);
@@ -519,8 +550,8 @@ TEST_F(ValuesTest, DeepCopyCovariantReturnTypes) {
   original_dict.Set("bool", original_bool);
   FundamentalValue* original_int = Value::CreateIntegerValue(42);
   original_dict.Set("int", original_int);
-  FundamentalValue* original_real = Value::CreateRealValue(3.14);
-  original_dict.Set("real", original_real);
+  FundamentalValue* original_double = Value::CreateDoubleValue(3.14);
+  original_dict.Set("double", original_double);
   StringValue* original_string = Value::CreateStringValue("hello");
   original_dict.Set("string", original_string);
   StringValue* original_string16 =
@@ -529,7 +560,7 @@ TEST_F(ValuesTest, DeepCopyCovariantReturnTypes) {
 
   char* original_buffer = new char[42];
   memset(original_buffer, '!', 42);
-  BinaryValue* original_binary = Value::CreateBinaryValue(original_buffer, 42);
+  BinaryValue* original_binary = BinaryValue::Create(original_buffer, 42);
   original_dict.Set("binary", original_binary);
 
   ListValue* original_list = new ListValue();
@@ -542,7 +573,7 @@ TEST_F(ValuesTest, DeepCopyCovariantReturnTypes) {
   Value* original_dict_value = &original_dict;
   Value* original_bool_value = original_bool;
   Value* original_int_value = original_int;
-  Value* original_real_value = original_real;
+  Value* original_double_value = original_double;
   Value* original_string_value = original_string;
   Value* original_string16_value = original_string16;
   Value* original_binary_value = original_binary;
@@ -551,7 +582,7 @@ TEST_F(ValuesTest, DeepCopyCovariantReturnTypes) {
   scoped_ptr<Value> copy_dict_value(original_dict_value->DeepCopy());
   scoped_ptr<Value> copy_bool_value(original_bool_value->DeepCopy());
   scoped_ptr<Value> copy_int_value(original_int_value->DeepCopy());
-  scoped_ptr<Value> copy_real_value(original_real_value->DeepCopy());
+  scoped_ptr<Value> copy_double_value(original_double_value->DeepCopy());
   scoped_ptr<Value> copy_string_value(original_string_value->DeepCopy());
   scoped_ptr<Value> copy_string16_value(original_string16_value->DeepCopy());
   scoped_ptr<Value> copy_binary_value(original_binary_value->DeepCopy());
@@ -560,14 +591,14 @@ TEST_F(ValuesTest, DeepCopyCovariantReturnTypes) {
   EXPECT_TRUE(original_dict_value->Equals(copy_dict_value.get()));
   EXPECT_TRUE(original_bool_value->Equals(copy_bool_value.get()));
   EXPECT_TRUE(original_int_value->Equals(copy_int_value.get()));
-  EXPECT_TRUE(original_real_value->Equals(copy_real_value.get()));
+  EXPECT_TRUE(original_double_value->Equals(copy_double_value.get()));
   EXPECT_TRUE(original_string_value->Equals(copy_string_value.get()));
   EXPECT_TRUE(original_string16_value->Equals(copy_string16_value.get()));
   EXPECT_TRUE(original_binary_value->Equals(copy_binary_value.get()));
   EXPECT_TRUE(original_list_value->Equals(copy_list_value.get()));
 }
 
-TEST_F(ValuesTest, RemoveEmptyChildren) {
+TEST(ValuesTest, RemoveEmptyChildren) {
   scoped_ptr<DictionaryValue> root(new DictionaryValue);
   // Remove empty lists and dictionaries.
   root->Set("empty_dict", new DictionaryValue);
@@ -642,7 +673,7 @@ TEST_F(ValuesTest, RemoveEmptyChildren) {
   }
 }
 
-TEST_F(ValuesTest, MergeDictionary) {
+TEST(ValuesTest, MergeDictionary) {
   scoped_ptr<DictionaryValue> base(new DictionaryValue);
   base->SetString("base_key", "base_key_value_base");
   base->SetString("collide_key", "collide_key_value_base");
@@ -686,3 +717,42 @@ TEST_F(ValuesTest, MergeDictionary) {
   EXPECT_TRUE(res_sub_dict->GetString("sub_merge_key", &sub_merge_key_value));
   EXPECT_EQ("sub_merge_key_value_merge", sub_merge_key_value); // Merged in.
 }
+
+TEST(ValuesTest, DictionaryIterator) {
+  DictionaryValue dict;
+  for (DictionaryValue::Iterator it(dict); it.HasNext(); it.Advance()) {
+    ADD_FAILURE();
+  }
+
+  StringValue value1("value1");
+  dict.Set("key1", value1.DeepCopy());
+  bool seen1 = false;
+  for (DictionaryValue::Iterator it(dict); it.HasNext(); it.Advance()) {
+    EXPECT_FALSE(seen1);
+    EXPECT_EQ("key1", it.key());
+    EXPECT_TRUE(value1.Equals(&it.value()));
+    seen1 = true;
+  }
+  EXPECT_TRUE(seen1);
+
+  StringValue value2("value2");
+  dict.Set("key2", value2.DeepCopy());
+  bool seen2 = seen1 = false;
+  for (DictionaryValue::Iterator it(dict); it.HasNext(); it.Advance()) {
+    if (it.key() == "key1") {
+      EXPECT_FALSE(seen1);
+      EXPECT_TRUE(value1.Equals(&it.value()));
+      seen1 = true;
+    } else if (it.key() == "key2") {
+      EXPECT_FALSE(seen2);
+      EXPECT_TRUE(value2.Equals(&it.value()));
+      seen2 = true;
+    } else {
+      ADD_FAILURE();
+    }
+  }
+  EXPECT_TRUE(seen1);
+  EXPECT_TRUE(seen2);
+}
+
+}  // namespace base
