@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,6 +9,7 @@
 #include <string>
 #include <vector>
 
+#include "base/base_export.h"
 #include "base/basictypes.h"
 
 namespace base {
@@ -18,12 +19,14 @@ namespace i18n {
 // Adds to |languages| the list of user preferred UI languages from MUI, if
 // available, falling-back on the user default UI language otherwise.  Returns
 // true if at least one language is added.
-bool GetUserPreferredUILanguageList(std::vector<std::wstring>* languages);
+BASE_EXPORT bool GetUserPreferredUILanguageList(
+    std::vector<std::wstring>* languages);
 
 // Adds to |languages| the list of thread, process, user, and system preferred
 // UI languages from MUI, if available, falling-back on the user default UI
 // language otherwise.  Returns true if at least one language is added.
-bool GetThreadPreferredUILanguageList(std::vector<std::wstring>* languages);
+BASE_EXPORT bool GetThreadPreferredUILanguageList(
+    std::vector<std::wstring>* languages);
 
 }  // namespace i18n
 }  // namespace win
