@@ -73,19 +73,16 @@
 #include <stdio.h>
 #include <sys/mman.h>
 #include <sys/prctl.h>
+#include <sys/syscall.h>
+#include <sys/wait.h>
+#include <unistd.h>
+
 #if !defined(__ANDROID__)
 #include <sys/signal.h>
-#endif
-#include <sys/syscall.h>
-#if !defined(__ANDROID__)
 #include <sys/ucontext.h>
 #include <sys/user.h>
-#endif
-#include <sys/wait.h>
-#if !defined(__ANDROID__)
 #include <ucontext.h>
 #endif
-#include <unistd.h>
 
 #include <algorithm>
 #include <utility>
@@ -99,6 +96,8 @@
 #include "common/linux/eintr_wrapper.h"
 #include "third_party/lss/linux_syscall_support.h"
 
+#include "linux/sched.h"
+
 #ifndef PR_SET_PTRACER
 #define PR_SET_PTRACER 0x59616d61
 #endif
@@ -329,8 +328,13 @@ bool ExceptionHandler::HandleSignal(int sig, siginfo_t* info, void* uc) {
   if (filter_ && !filter_(callback_context_))
     return false;
 
-  // Allow ourselves to be dumped.
-  sys_prctl(PR_SET_DUMPABLE, 1);
+  // Allow ourselves to be dumped if the signal is trusted.
+  bool signal_trusted = info->si_code > 0;
+  bool signal_pid_trusted = info->si_code == SI_USER ||
+      info->si_code == SI_TKILL;
+  if (signal_trusted || (signal_pid_trusted && info->si_pid == getpid())) {
+    sys_prctl(PR_SET_DUMPABLE, 1);
+  }
   CrashContext context;
   memcpy(&context.siginfo, info, sizeof(siginfo_t));
   memcpy(&context.context, uc, sizeof(struct ucontext));
@@ -393,7 +397,7 @@ bool ExceptionHandler::GenerateDump(CrashContext *context) {
       &thread_arg, NULL, NULL, NULL);
   int r, status;
   // Allow the child to ptrace us
-  prctl(PR_SET_PTRACER, child, 0, 0, 0);
+  sys_prctl(PR_SET_PTRACER, child);
   SendContinueSignalToChild();
   do {
     r = sys_waitpid(child, &status, __WALL);
@@ -496,7 +500,8 @@ void ExceptionHandler::AddMappingInfo(const std::string& name,
   info.start_addr = start_address;
   info.size = mapping_size;
   info.offset = file_offset;
-  strncpy(info.name, name.c_str(), std::min(name.size(), sizeof(info)));
+  strncpy(info.name, name.c_str(), sizeof(info.name) - 1);
+  info.name[sizeof(info.name) - 1] = '\0';
 
   MappingEntry mapping;
   mapping.first = info;
