@@ -79,8 +79,13 @@ bool PDBSourceLineWriter::Open(const wstring &file, FileFormat format) {
 
   CComPtr<IDiaDataSource> data_source;
   if (FAILED(data_source.CoCreateInstance(CLSID_DiaSource))) {
-    fprintf(stderr, "CoCreateInstance CLSID_DiaSource failed "
-            "(msdia80.dll unregistered?)\n");
+    const int kGuidSize = 64;
+    wchar_t classid[kGuidSize] = {0};
+    StringFromGUID2(CLSID_DiaSource, classid, kGuidSize);
+    // vc80 uses bce36434-2c24-499e-bf49-8bd99b0eeb68.
+    // vc90 uses 4C41678E-887B-4365-A09E-925D28DB33C2.
+    fprintf(stderr, "CoCreateInstance CLSID_DiaSource %S failed "
+            "(msdia*.dll unregistered?)\n", classid);
     return false;
   }
 
@@ -799,10 +804,8 @@ bool PDBSourceLineWriter::WriteMap(FILE *map_file) {
   output_ = map_file;
 
   bool ret = PrintPDBInfo();
-  // TODO(ted): This is currently disabled to allow Breakpad users to update
-  // processing infrastructure.
   // This is not a critical piece of the symbol file.
-  // PrintPEInfo();
+  PrintPEInfo();
   ret = ret &&
     PrintSourceFiles() && 
     PrintFunctions() &&
