@@ -386,6 +386,11 @@ bool MinidumpProcessor::GetOSInfo(Minidump *dump, SystemInfo *info) {
       break;
     }
 
+    case MD_OS_IOS: {
+      info->os = "iOS";
+      break;
+    }
+
     case MD_OS_LINUX: {
       info->os = "Linux";
       break;
@@ -451,7 +456,8 @@ string MinidumpProcessor::GetCrashReason(Minidump *dump, u_int64_t *address) {
     return reason;
 
   switch (raw_system_info->platform_id) {
-    case MD_OS_MAC_OS_X: {
+    case MD_OS_MAC_OS_X:
+    case MD_OS_IOS: {
       char flags_string[11];
       snprintf(flags_string, sizeof(flags_string), "0x%08x", exception_flags);
       switch (exception_code) {
