@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -32,13 +32,17 @@
 #include "breakpad/src/common/linux/linux_libc_support.h"
 #include "breakpad/src/common/memory.h"
 #include "chrome/common/child_process_logging.h"
-#include "chrome/common/chrome_descriptors.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/chrome_version_info_posix.h"
 #include "chrome/common/env_vars.h"
+#include "content/common/chrome_descriptors.h"
 #include "seccompsandbox/linux_syscall_support.h"
 
+#ifndef PR_SET_PTRACER
+#define PR_SET_PTRACER 0x59616d61
+#endif
+
 // Some versions of gcc are prone to warn about unused return values. In cases
 // where we either a) know the call cannot fail, or b) there is nothing we
 // can do when a call fails, we mark the return code as ignored. This avoids
@@ -50,6 +54,7 @@ static const char kUploadURL[] =
 
 static bool is_crash_reporter_enabled = false;
 static uint64_t process_start_time = 0;
+static char* crash_log_path = NULL;
 
 // Writes the value |v| as 16 hex characters to the memory pointed at by
 // |output|.
@@ -98,12 +103,196 @@ static unsigned my_uint64_len(uint64_t i) {
 
 // uint64_t version of my_itos() from
 // breakpad/src/common/linux/linux_libc_support.h. Convert a non-negative
-// integer to a string.
+// integer to a string (not null-terminated).
 static void my_uint64tos(char* output, uint64_t i, unsigned i_len) {
   for (unsigned index = i_len; index; --index, i /= 10)
     output[index - 1] = '0' + (i % 10);
 }
 
+namespace {
+
+// MIME substrings.
+static const char g_rn[] = "\r\n";
+static const char g_form_data_msg[] = "Content-Disposition: form-data; name=\"";
+static const char g_quote_msg[] = "\"";
+static const char g_dashdash_msg[] = "--";
+static const char g_dump_msg[] = "upload_file_minidump\"; filename=\"dump\"";
+static const char g_content_type_msg[] =
+    "Content-Type: application/octet-stream";
+
+// MimeWriter manages an iovec for writing MIMEs to a file.
+class MimeWriter {
+ public:
+  static const int kIovCapacity = 30;
+  static const size_t kMaxCrashChunkSize = 64;
+
+  MimeWriter(int fd, const char* const mime_boundary);
+  ~MimeWriter();
+
+  // Append boundary.
+  void AddBoundary();
+
+  // Append end of file boundary.
+  void AddEnd();
+
+  // Append key/value pair with specified sizes.
+  void AddPairData(const char* msg_type,
+                   size_t msg_type_size,
+                   const char* msg_data,
+                   size_t msg_data_size);
+
+  // Append key/value pair.
+  void AddPairString(const char* msg_type,
+                     const char* msg_data) {
+    AddPairData(msg_type, my_strlen(msg_type), msg_data, my_strlen(msg_data));
+  }
+
+  // Append key/value pair, splitting value into chunks no larger than
+  // |chunk_size|. |chunk_size| cannot be greater than |kMaxCrashChunkSize|.
+  // The msg_type string will have a counter suffix to distinguish each chunk.
+  void AddPairDataInChunks(const char* msg_type,
+                           size_t msg_type_size,
+                           const char* msg_data,
+                           size_t msg_data_size,
+                           size_t chunk_size,
+                           bool strip_trailing_spaces);
+
+  // Add binary file dump. Currently this is only done once, so the name is
+  // fixed.
+  void AddFileDump(uint8_t* file_data,
+                   size_t file_size);
+
+  // Flush any pending iovecs to the output file.
+  void Flush() {
+    IGNORE_RET(sys_writev(fd_, iov_, iov_index_));
+    iov_index_ = 0;
+  }
+
+ private:
+  void AddItem(const void* base, size_t size);
+  // Minor performance trade-off for easier-to-maintain code.
+  void AddString(const char* str) {
+    AddItem(str, my_strlen(str));
+  }
+  void AddItemWithoutTrailingSpaces(const void* base, size_t size);
+
+  struct kernel_iovec iov_[kIovCapacity];
+  int iov_index_;
+
+  // Output file descriptor.
+  int fd_;
+
+  const char* const mime_boundary_;
+
+  DISALLOW_COPY_AND_ASSIGN(MimeWriter);
+};
+
+MimeWriter::MimeWriter(int fd, const char* const mime_boundary)
+    : iov_index_(0),
+      fd_(fd),
+      mime_boundary_(mime_boundary) {
+}
+
+MimeWriter::~MimeWriter() {
+}
+
+void MimeWriter::AddBoundary() {
+  AddString(mime_boundary_);
+  AddString(g_rn);
+}
+
+void MimeWriter::AddEnd() {
+  AddString(mime_boundary_);
+  AddString(g_dashdash_msg);
+  AddString(g_rn);
+}
+
+void MimeWriter::AddPairData(const char* msg_type,
+                             size_t msg_type_size,
+                             const char* msg_data,
+                             size_t msg_data_size) {
+  AddString(g_form_data_msg);
+  AddItem(msg_type, msg_type_size);
+  AddString(g_quote_msg);
+  AddString(g_rn);
+  AddString(g_rn);
+  AddItem(msg_data, msg_data_size);
+  AddString(g_rn);
+}
+
+void MimeWriter::AddPairDataInChunks(const char* msg_type,
+                                     size_t msg_type_size,
+                                     const char* msg_data,
+                                     size_t msg_data_size,
+                                     size_t chunk_size,
+                                     bool strip_trailing_spaces) {
+  if (chunk_size > kMaxCrashChunkSize)
+    return;
+
+  unsigned i = 0;
+  size_t done = 0, msg_length = msg_data_size;
+
+  while (msg_length) {
+    char num[16];
+    const unsigned num_len = my_int_len(++i);
+    my_itos(num, i, num_len);
+
+    size_t chunk_len = std::min(chunk_size, msg_length);
+
+    AddString(g_form_data_msg);
+    AddItem(msg_type, msg_type_size);
+    AddItem(num, num_len);
+    AddString(g_quote_msg);
+    AddString(g_rn);
+    AddString(g_rn);
+    if (strip_trailing_spaces) {
+      AddItemWithoutTrailingSpaces(msg_data + done, chunk_len);
+    } else {
+      AddItem(msg_data + done, chunk_len);
+    }
+    AddString(g_rn);
+    AddBoundary();
+    Flush();
+
+    done += chunk_len;
+    msg_length -= chunk_len;
+  }
+}
+
+void MimeWriter::AddFileDump(uint8_t* file_data,
+                             size_t file_size) {
+  AddString(g_form_data_msg);
+  AddString(g_dump_msg);
+  AddString(g_rn);
+  AddString(g_content_type_msg);
+  AddString(g_rn);
+  AddString(g_rn);
+  AddItem(file_data, file_size);
+  AddString(g_rn);
+}
+
+void MimeWriter::AddItem(const void* base, size_t size) {
+  // Check if the iovec is full and needs to be flushed to output file.
+  if (iov_index_ == kIovCapacity) {
+    Flush();
+  }
+  iov_[iov_index_].iov_base = const_cast<void*>(base);
+  iov_[iov_index_].iov_len = size;
+  ++iov_index_;
+}
+
+void MimeWriter::AddItemWithoutTrailingSpaces(const void* base, size_t size) {
+  while (size > 0) {
+    const char* c = static_cast<const char*>(base) + size - 1;
+    if (*c != ' ')
+      break;
+    size--;
+  }
+  AddItem(base, size);
+}
+
+}  // namespace
+
 pid_t HandleCrashDump(const BreakpadInfo& info) {
   // WARNING: this code runs in a compromised context. It may not call into
   // libc nor allocate memory normally.
@@ -189,135 +378,96 @@ pid_t HandleCrashDump(const BreakpadInfo& info) {
   mime_boundary[28 + 16] = 0;
   IGNORE_RET(sys_close(ufd));
 
-  static const char version_msg[] = PRODUCT_VERSION;
-
   // The MIME block looks like this:
-  //   BOUNDARY \r\n (0, 1)
-  //   Content-Disposition: form-data; name="prod" \r\n \r\n (2..6)
-  //   Chrome_Linux \r\n (7, 8)
-  //   BOUNDARY \r\n (9, 10)
-  //   Content-Disposition: form-data; name="ver" \r\n \r\n (11..15)
-  //   1.2.3.4 \r\n (16, 17)
-  //   BOUNDARY \r\n (18, 19)
-  //   Content-Disposition: form-data; name="guid" \r\n \r\n (20..24)
-  //   1.2.3.4 \r\n (25, 26)
-  //   BOUNDARY \r\n (27, 28)
+  //   BOUNDARY \r\n
+  //   Content-Disposition: form-data; name="prod" \r\n \r\n
+  //   Chrome_Linux \r\n
+  //   BOUNDARY \r\n
+  //   Content-Disposition: form-data; name="ver" \r\n \r\n
+  //   1.2.3.4 \r\n
+  //   BOUNDARY \r\n
+  //   Content-Disposition: form-data; name="guid" \r\n \r\n
+  //   1.2.3.4 \r\n
+  //   BOUNDARY \r\n
   //
   //   zero or one:
-  //   Content-Disposition: form-data; name="ptime" \r\n \r\n (0..4)
-  //   abcdef \r\n (5, 6)
-  //   BOUNDARY \r\n (7, 8)
+  //   Content-Disposition: form-data; name="ptime" \r\n \r\n
+  //   abcdef \r\n
+  //   BOUNDARY \r\n
   //
   //   zero or one:
-  //   Content-Disposition: form-data; name="ptype" \r\n \r\n (0..4)
-  //   abcdef \r\n (5, 6)
-  //   BOUNDARY \r\n (7, 8)
+  //   Content-Disposition: form-data; name="ptype" \r\n \r\n
+  //   abcdef \r\n
+  //   BOUNDARY \r\n
+  //
+  //   zero or more gpu entries:
+  //   Content-Disposition: form-data; name="gpu-xxxxx" \r\n \r\n
+  //   <gpu-xxxxx> \r\n
+  //   BOUNDARY \r\n
   //
   //   zero or one:
-  //   Content-Disposition: form-data; name="lsb-release" \r\n \r\n (0..4)
-  //   abcdef \r\n (5, 6)
-  //   BOUNDARY \r\n (7, 8)
+  //   Content-Disposition: form-data; name="lsb-release" \r\n \r\n
+  //   abcdef \r\n
+  //   BOUNDARY \r\n
   //
   //   zero or more:
-  //   Content-Disposition: form-data; name="url-chunk-1" \r\n \r\n (0..5)
-  //   abcdef \r\n (6, 7)
-  //   BOUNDARY \r\n (8, 9)
+  //   Content-Disposition: form-data; name="url-chunk-1" \r\n \r\n
+  //   abcdef \r\n
+  //   BOUNDARY \r\n
   //
-  //   Content-Disposition: form-data; name="dump"; filename="dump" \r\n (0,1,2)
-  //   Content-Type: application/octet-stream \r\n \r\n (3,4,5)
-  //   <dump contents> (6)
-  //   \r\n BOUNDARY -- \r\n (7,8,9,10)
+  //   zero or one:
+  //   Content-Disposition: form-data; name="channel" \r\n \r\n
+  //   beta \r\n
+  //   BOUNDARY \r\n
+  //
+  //   zero or one:
+  //   Content-Disposition: form-data; name="num-views" \r\n \r\n
+  //   3 \r\n
+  //   BOUNDARY \r\n
+  //
+  //   zero or one:
+  //   Content-Disposition: form-data; name="num-extensions" \r\n \r\n
+  //   5 \r\n
+  //   BOUNDARY \r\n
+  //
+  //   zero to 10:
+  //   Content-Disposition: form-data; name="extension-1" \r\n \r\n
+  //   abcdefghijklmnopqrstuvwxyzabcdef \r\n
+  //   BOUNDARY \r\n
+  //
+  //   zero or one:
+  //   Content-Disposition: form-data; name="num-switches" \r\n \r\n
+  //   5 \r\n
+  //   BOUNDARY \r\n
+  //
+  //   zero to 15:
+  //   Content-Disposition: form-data; name="switch-1" \r\n \r\n
+  //   --foo \r\n
+  //   BOUNDARY \r\n
+  //
+  //   Content-Disposition: form-data; name="dump"; filename="dump" \r\n
+  //   Content-Type: application/octet-stream \r\n \r\n
+  //   <dump contents>
+  //   \r\n BOUNDARY -- \r\n
 
-  static const char rn[] = {'\r', '\n'};
-  static const char form_data_msg[] = "Content-Disposition: form-data; name=\"";
-  static const char prod_msg[] = "prod";
-  static const char quote_msg[] = {'"'};
+  MimeWriter writer(fd, mime_boundary);
+  {
 #if defined(OS_CHROMEOS)
-  static const char chrome_product_msg[] = "Chrome_ChromeOS";
+    static const char chrome_product_msg[] = "Chrome_ChromeOS";
 #else  // OS_LINUX
-  static const char chrome_product_msg[] = "Chrome_Linux";
+    static const char chrome_product_msg[] = "Chrome_Linux";
 #endif
-  static const char ver_msg[] = "ver";
-  static const char guid_msg[] = "guid";
-  static const char dashdash_msg[] = {'-', '-'};
-  static const char dump_msg[] = "upload_file_minidump\"; filename=\"dump\"";
-  static const char content_type_msg[] =
-      "Content-Type: application/octet-stream";
-  static const char url_chunk_msg[] = "url-chunk-";
-  static const char process_time_msg[] = "ptime";
-  static const char process_type_msg[] = "ptype";
-  static const char distro_msg[] = "lsb-release";
+    static const char version_msg[] = PRODUCT_VERSION;
 
-  struct kernel_iovec iov[29];
-  iov[0].iov_base = mime_boundary;
-  iov[0].iov_len = sizeof(mime_boundary) - 1;
-  iov[1].iov_base = const_cast<char*>(rn);
-  iov[1].iov_len = sizeof(rn);
-
-  iov[2].iov_base = const_cast<char*>(form_data_msg);
-  iov[2].iov_len = sizeof(form_data_msg) - 1;
-  iov[3].iov_base = const_cast<char*>(prod_msg);
-  iov[3].iov_len = sizeof(prod_msg) - 1;
-  iov[4].iov_base = const_cast<char*>(quote_msg);
-  iov[4].iov_len = sizeof(quote_msg);
-  iov[5].iov_base = const_cast<char*>(rn);
-  iov[5].iov_len = sizeof(rn);
-  iov[6].iov_base = const_cast<char*>(rn);
-  iov[6].iov_len = sizeof(rn);
-
-  iov[7].iov_base = const_cast<char*>(chrome_product_msg);
-  iov[7].iov_len = sizeof(chrome_product_msg) - 1;
-  iov[8].iov_base = const_cast<char*>(rn);
-  iov[8].iov_len = sizeof(rn);
-
-  iov[9].iov_base = mime_boundary;
-  iov[9].iov_len = sizeof(mime_boundary) - 1;
-  iov[10].iov_base = const_cast<char*>(rn);
-  iov[10].iov_len = sizeof(rn);
-
-  iov[11].iov_base = const_cast<char*>(form_data_msg);
-  iov[11].iov_len = sizeof(form_data_msg) - 1;
-  iov[12].iov_base = const_cast<char*>(ver_msg);
-  iov[12].iov_len = sizeof(ver_msg) - 1;
-  iov[13].iov_base = const_cast<char*>(quote_msg);
-  iov[13].iov_len = sizeof(quote_msg);
-  iov[14].iov_base = const_cast<char*>(rn);
-  iov[14].iov_len = sizeof(rn);
-  iov[15].iov_base = const_cast<char*>(rn);
-  iov[15].iov_len = sizeof(rn);
-
-  iov[16].iov_base = const_cast<char*>(version_msg);
-  iov[16].iov_len = sizeof(version_msg) - 1;
-  iov[17].iov_base = const_cast<char*>(rn);
-  iov[17].iov_len = sizeof(rn);
-
-  iov[18].iov_base = mime_boundary;
-  iov[18].iov_len = sizeof(mime_boundary) - 1;
-  iov[19].iov_base = const_cast<char*>(rn);
-  iov[19].iov_len = sizeof(rn);
-
-  iov[20].iov_base = const_cast<char*>(form_data_msg);
-  iov[20].iov_len = sizeof(form_data_msg) - 1;
-  iov[21].iov_base = const_cast<char*>(guid_msg);
-  iov[21].iov_len = sizeof(guid_msg) - 1;
-  iov[22].iov_base = const_cast<char*>(quote_msg);
-  iov[22].iov_len = sizeof(quote_msg);
-  iov[23].iov_base = const_cast<char*>(rn);
-  iov[23].iov_len = sizeof(rn);
-  iov[24].iov_base = const_cast<char*>(rn);
-  iov[24].iov_len = sizeof(rn);
-
-  iov[25].iov_base = const_cast<char*>(info.guid);
-  iov[25].iov_len = info.guid_length;
-  iov[26].iov_base = const_cast<char*>(rn);
-  iov[26].iov_len = sizeof(rn);
-
-  iov[27].iov_base = mime_boundary;
-  iov[27].iov_len = sizeof(mime_boundary) - 1;
-  iov[28].iov_base = const_cast<char*>(rn);
-  iov[28].iov_len = sizeof(rn);
-
-  IGNORE_RET(sys_writev(fd, iov, 29));
+    writer.AddBoundary();
+    writer.AddPairString("prod", chrome_product_msg);
+    writer.AddBoundary();
+    writer.AddPairString("ver", version_msg);
+    writer.AddBoundary();
+    writer.AddPairString("guid", info.guid);
+    writer.AddBoundary();
+    writer.Flush();
+  }
 
   if (info.process_start_time > 0) {
     struct kernel_timeval tv;
@@ -329,150 +479,114 @@ pid_t HandleCrashDump(const BreakpadInfo& info) {
         const unsigned time_len = my_uint64_len(time);
         my_uint64tos(time_str, time, time_len);
 
-        iov[0].iov_base = const_cast<char*>(form_data_msg);
-        iov[0].iov_len = sizeof(form_data_msg) - 1;
-        iov[1].iov_base = const_cast<char*>(process_time_msg);
-        iov[1].iov_len = sizeof(process_time_msg) - 1;
-        iov[2].iov_base = const_cast<char*>(quote_msg);
-        iov[2].iov_len = sizeof(quote_msg);
-        iov[3].iov_base = const_cast<char*>(rn);
-        iov[3].iov_len = sizeof(rn);
-        iov[4].iov_base = const_cast<char*>(rn);
-        iov[4].iov_len = sizeof(rn);
-
-        iov[5].iov_base = const_cast<char*>(time_str);
-        iov[5].iov_len = time_len;
-        iov[6].iov_base = const_cast<char*>(rn);
-        iov[6].iov_len = sizeof(rn);
-        iov[7].iov_base = mime_boundary;
-        iov[7].iov_len = sizeof(mime_boundary) - 1;
-        iov[8].iov_base = const_cast<char*>(rn);
-        iov[8].iov_len = sizeof(rn);
-
-        IGNORE_RET(sys_writev(fd, iov, 9));
+        static const char process_time_msg[] = "ptime";
+        writer.AddPairData(process_time_msg, sizeof(process_time_msg) - 1,
+                           time_str, time_len);
+        writer.AddBoundary();
+        writer.Flush();
       }
     }
   }
 
   if (info.process_type_length) {
-    iov[0].iov_base = const_cast<char*>(form_data_msg);
-    iov[0].iov_len = sizeof(form_data_msg) - 1;
-    iov[1].iov_base = const_cast<char*>(process_type_msg);
-    iov[1].iov_len = sizeof(process_type_msg) - 1;
-    iov[2].iov_base = const_cast<char*>(quote_msg);
-    iov[2].iov_len = sizeof(quote_msg);
-    iov[3].iov_base = const_cast<char*>(rn);
-    iov[3].iov_len = sizeof(rn);
-    iov[4].iov_base = const_cast<char*>(rn);
-    iov[4].iov_len = sizeof(rn);
+    writer.AddPairString("ptype", info.process_type);
+    writer.AddBoundary();
+    writer.Flush();
+  }
 
-    iov[5].iov_base = const_cast<char*>(info.process_type);
-    iov[5].iov_len = info.process_type_length;
-    iov[6].iov_base = const_cast<char*>(rn);
-    iov[6].iov_len = sizeof(rn);
-    iov[7].iov_base = mime_boundary;
-    iov[7].iov_len = sizeof(mime_boundary) - 1;
-    iov[8].iov_base = const_cast<char*>(rn);
-    iov[8].iov_len = sizeof(rn);
+  // If GPU info is known, send it.
+  unsigned gpu_vendor_len = my_strlen(child_process_logging::g_gpu_vendor_id);
+  if (gpu_vendor_len) {
+    static const char vendor_msg[] = "gpu-venid";
+    static const char device_msg[] = "gpu-devid";
+    static const char driver_msg[] = "gpu-driver";
+    static const char psver_msg[] = "gpu-psver";
+    static const char vsver_msg[] = "gpu-vsver";
 
-    IGNORE_RET(sys_writev(fd, iov, 9));
+    writer.AddPairString(vendor_msg, child_process_logging::g_gpu_vendor_id);
+    writer.AddBoundary();
+    writer.AddPairString(device_msg, child_process_logging::g_gpu_device_id);
+    writer.AddBoundary();
+    writer.AddPairString(driver_msg, child_process_logging::g_gpu_driver_ver);
+    writer.AddBoundary();
+    writer.AddPairString(psver_msg, child_process_logging::g_gpu_ps_ver);
+    writer.AddBoundary();
+    writer.AddPairString(vsver_msg, child_process_logging::g_gpu_vs_ver);
+    writer.AddBoundary();
+    writer.Flush();
   }
 
   if (info.distro_length) {
-    iov[0].iov_base = const_cast<char*>(form_data_msg);
-    iov[0].iov_len = sizeof(form_data_msg) - 1;
-    iov[1].iov_base = const_cast<char*>(distro_msg);
-    iov[1].iov_len = sizeof(distro_msg) - 1;
-    iov[2].iov_base = const_cast<char*>(quote_msg);
-    iov[2].iov_len = sizeof(quote_msg);
-    iov[3].iov_base = const_cast<char*>(rn);
-    iov[3].iov_len = sizeof(rn);
-    iov[4].iov_base = const_cast<char*>(rn);
-    iov[4].iov_len = sizeof(rn);
-
-    iov[5].iov_base = const_cast<char*>(info.distro);
-    iov[5].iov_len = info.distro_length;
-    iov[6].iov_base = const_cast<char*>(rn);
-    iov[6].iov_len = sizeof(rn);
-    iov[7].iov_base = mime_boundary;
-    iov[7].iov_len = sizeof(mime_boundary) - 1;
-    iov[8].iov_base = const_cast<char*>(rn);
-    iov[8].iov_len = sizeof(rn);
-
-    IGNORE_RET(sys_writev(fd, iov, 9));
+    static const char distro_msg[] = "lsb-release";
+    writer.AddPairString(distro_msg, info.distro);
+    writer.AddBoundary();
+    writer.Flush();
   }
 
-  // For rendererers and plugins.
+  // For renderers and plugins.
   if (info.crash_url_length) {
-    unsigned i = 0, done = 0, crash_url_length = info.crash_url_length;
-    static const unsigned kMaxCrashChunkSize = 64;
-    static const unsigned kMaxUrlLength = 8 * kMaxCrashChunkSize;
-    if (crash_url_length > kMaxUrlLength)
-      crash_url_length = kMaxUrlLength;
-
-    while (crash_url_length) {
-      char num[16];
-      const unsigned num_len = my_int_len(++i);
-      my_itos(num, i, num_len);
-
-      iov[0].iov_base = const_cast<char*>(form_data_msg);
-      iov[0].iov_len = sizeof(form_data_msg) - 1;
-      iov[1].iov_base = const_cast<char*>(url_chunk_msg);
-      iov[1].iov_len = sizeof(url_chunk_msg) - 1;
-      iov[2].iov_base = num;
-      iov[2].iov_len = num_len;
-      iov[3].iov_base = const_cast<char*>(quote_msg);
-      iov[3].iov_len = sizeof(quote_msg);
-      iov[4].iov_base = const_cast<char*>(rn);
-      iov[4].iov_len = sizeof(rn);
-      iov[5].iov_base = const_cast<char*>(rn);
-      iov[5].iov_len = sizeof(rn);
-
-      const unsigned len = crash_url_length > kMaxCrashChunkSize ?
-                           kMaxCrashChunkSize : crash_url_length;
-      iov[6].iov_base = const_cast<char*>(info.crash_url + done);
-      iov[6].iov_len = len;
-      iov[7].iov_base = const_cast<char*>(rn);
-      iov[7].iov_len = sizeof(rn);
-      iov[8].iov_base = mime_boundary;
-      iov[8].iov_len = sizeof(mime_boundary) - 1;
-      iov[9].iov_base = const_cast<char*>(rn);
-      iov[9].iov_len = sizeof(rn);
-
-      IGNORE_RET(sys_writev(fd, iov, 10));
-
-      done += len;
-      crash_url_length -= len;
-    }
+    static const char url_chunk_msg[] = "url-chunk-";
+    static const unsigned kMaxUrlLength = 8 * MimeWriter::kMaxCrashChunkSize;
+    writer.AddPairDataInChunks(url_chunk_msg, sizeof(url_chunk_msg) - 1,
+        info.crash_url, std::min(info.crash_url_length, kMaxUrlLength),
+        MimeWriter::kMaxCrashChunkSize, false /* Don't strip whitespaces. */);
   }
 
-  iov[0].iov_base = const_cast<char*>(form_data_msg);
-  iov[0].iov_len = sizeof(form_data_msg) - 1;
-  iov[1].iov_base = const_cast<char*>(dump_msg);
-  iov[1].iov_len = sizeof(dump_msg) - 1;
-  iov[2].iov_base = const_cast<char*>(rn);
-  iov[2].iov_len = sizeof(rn);
+  if (my_strlen(child_process_logging::g_channel)) {
+    writer.AddPairString("channel", child_process_logging::g_channel);
+    writer.AddBoundary();
+    writer.Flush();
+  }
 
-  iov[3].iov_base = const_cast<char*>(content_type_msg);
-  iov[3].iov_len = sizeof(content_type_msg) - 1;
-  iov[4].iov_base = const_cast<char*>(rn);
-  iov[4].iov_len = sizeof(rn);
-  iov[5].iov_base = const_cast<char*>(rn);
-  iov[5].iov_len = sizeof(rn);
+  if (my_strlen(child_process_logging::g_num_views)) {
+    writer.AddPairString("num-views", child_process_logging::g_num_views);
+    writer.AddBoundary();
+    writer.Flush();
+  }
 
-  iov[6].iov_base = dump_data;
-  iov[6].iov_len = st.st_size;
+  if (my_strlen(child_process_logging::g_num_extensions)) {
+    writer.AddPairString("num-extensions",
+                         child_process_logging::g_num_extensions);
+    writer.AddBoundary();
+    writer.Flush();
+  }
 
-  iov[7].iov_base = const_cast<char*>(rn);
-  iov[7].iov_len = sizeof(rn);
-  iov[8].iov_base = mime_boundary;
-  iov[8].iov_len = sizeof(mime_boundary) - 1;
-  iov[9].iov_base = const_cast<char*>(dashdash_msg);
-  iov[9].iov_len = sizeof(dashdash_msg);
-  iov[10].iov_base = const_cast<char*>(rn);
-  iov[10].iov_len = sizeof(rn);
+  unsigned extension_ids_len =
+      my_strlen(child_process_logging::g_extension_ids);
+  if (extension_ids_len) {
+    static const char extension_msg[] = "extension-";
+    static const unsigned kMaxExtensionsLen =
+        kMaxReportedActiveExtensions * child_process_logging::kExtensionLen;
+    writer.AddPairDataInChunks(extension_msg, sizeof(extension_msg) - 1,
+        child_process_logging::g_extension_ids,
+        std::min(extension_ids_len, kMaxExtensionsLen),
+        child_process_logging::kExtensionLen,
+        false /* Don't strip whitespace. */);
+  }
 
-  IGNORE_RET(sys_writev(fd, iov, 11));
+  if (my_strlen(child_process_logging::g_num_switches)) {
+    writer.AddPairString("num-switches",
+                         child_process_logging::g_num_switches);
+    writer.AddBoundary();
+    writer.Flush();
+  }
+
+  unsigned switches_len =
+      my_strlen(child_process_logging::g_switches);
+  if (switches_len) {
+    static const char switch_msg[] = "switch-";
+    static const unsigned kMaxSwitchLen =
+        kMaxSwitches * child_process_logging::kSwitchLen;
+    writer.AddPairDataInChunks(switch_msg, sizeof(switch_msg) - 1,
+        child_process_logging::g_switches,
+        std::min(switches_len, kMaxSwitchLen),
+        child_process_logging::kSwitchLen,
+        true /* Strip whitespace since switches are padded to kSwitchLen. */);
+  }
+
+  writer.AddFileDump(dump_data, st.st_size);
+  writer.AddEnd();
+  writer.Flush();
 
   IGNORE_RET(sys_close(fd));
 
@@ -541,11 +655,31 @@ pid_t HandleCrashDump(const BreakpadInfo& info) {
       const int len = HANDLE_EINTR(sys_read(fds[0], id_buf,
                                    sizeof(id_buf) - 1));
       if (len > 0) {
+        // Write crash dump id to stderr.
         id_buf[len] = 0;
         static const char msg[] = "\nCrash dump id: ";
         sys_write(2, msg, sizeof(msg) - 1);
         sys_write(2, id_buf, my_strlen(id_buf));
         sys_write(2, "\n", 1);
+
+        // Write crash dump id to crash log as: seconds_since_epoch,crash_id
+        struct kernel_timeval tv;
+        if (crash_log_path && !sys_gettimeofday(&tv, NULL)) {
+          uint64_t time = kernel_timeval_to_ms(&tv) / 1000;
+          char time_str[21];
+          const unsigned time_len = my_uint64_len(time);
+          my_uint64tos(time_str, time, time_len);
+
+          int log_fd = sys_open(crash_log_path, O_CREAT | O_WRONLY | O_APPEND,
+                                0600);
+          if (log_fd > 0) {
+            sys_write(log_fd, time_str, time_len);
+            sys_write(log_fd, ",", 1);
+            sys_write(log_fd, id_buf, my_strlen(id_buf));
+            sys_write(log_fd, "\n", 1);
+            IGNORE_RET(sys_close(log_fd));
+          }
+        }
       }
       IGNORE_RET(sys_unlink(info.filename));
       IGNORE_RET(sys_unlink(temp_file));
@@ -590,7 +724,7 @@ static bool CrashDone(const char* dump_path,
   google_breakpad::PageAllocator allocator;
   const unsigned dump_path_len = my_strlen(dump_path);
   const unsigned minidump_id_len = my_strlen(minidump_id);
-  char *const path = reinterpret_cast<char*>(allocator.Alloc(
+  char* const path = reinterpret_cast<char*>(allocator.Alloc(
       dump_path_len + 1 /* '/' */ + minidump_id_len +
       4 /* ".dmp" */ + 1 /* NUL */));
   memcpy(path, dump_path, dump_path_len);
@@ -634,22 +768,34 @@ static bool CrashDoneUpload(const char* dump_path,
 
 void EnableCrashDumping(const bool unattended) {
   is_crash_reporter_enabled = true;
+
+  FilePath tmp_path("/tmp");
+  PathService::Get(base::DIR_TEMP, &tmp_path);
+
+  FilePath dumps_path(tmp_path);
+  if (PathService::Get(chrome::DIR_CRASH_DUMPS, &dumps_path)) {
+    FilePath logfile = dumps_path.AppendASCII("uploads.log");
+    std::string logfile_str = logfile.value();
+    const size_t crash_log_path_len = logfile_str.size() + 1;
+    crash_log_path = new char[crash_log_path_len];
+    strncpy(crash_log_path, logfile_str.c_str(), crash_log_path_len);
+  }
+
   if (unattended) {
-    FilePath dumps_path("/tmp");
-    PathService::Get(chrome::DIR_CRASH_DUMPS, &dumps_path);
     new google_breakpad::ExceptionHandler(dumps_path.value().c_str(), NULL,
                                           CrashDoneNoUpload, NULL,
                                           true /* install handlers */);
   } else {
-    new google_breakpad::ExceptionHandler("/tmp", NULL, CrashDoneUpload, NULL,
+    new google_breakpad::ExceptionHandler(tmp_path.value().c_str(), NULL,
+                                          CrashDoneUpload, NULL,
                                           true /* install handlers */);
   }
 }
 
-// Currently Non-Browser = Renderer, Plugins, Native Client and Gpu
-static bool
-NonBrowserCrashHandler(const void* crash_context, size_t crash_context_size,
-                       void* context) {
+// Non-Browser = Extension, Gpu, Plugins, Ppapi and Renderer
+static bool NonBrowserCrashHandler(const void* crash_context,
+                                   size_t crash_context_size,
+                                   void* context) {
   const int fd = reinterpret_cast<intptr_t>(context);
   int fds[2] = { -1, -1 };
   if (sys_socketpair(AF_UNIX, SOCK_STREAM, 0, fds) < 0) {
@@ -657,6 +803,19 @@ NonBrowserCrashHandler(const void* crash_context, size_t crash_context_size,
     sys_write(2, msg, sizeof(msg)-1);
     return false;
   }
+
+  // On kernels with ptrace protection, e.g. Ubuntu 10.10+, the browser cannot
+  // ptrace this crashing process and crash dumping will fail. When using the
+  // SUID sandbox, this crashing process is likely to be in its own PID
+  // namespace, and thus there is no way to permit only the browser process to
+  // ptrace it.
+  // The workaround is to allow all processes to ptrace this process if we
+  // reach this point, by passing -1 as the allowed PID. However, support for
+  // passing -1 as the PID won't reach kernels until around the Ubuntu 12.04
+  // timeframe.
+  sys_prctl(PR_SET_PTRACER, -1);
+
+  // Start constructing the message to send to the browser.
   char guid[kGuidSize + 1] = {0};
   char crash_url[kMaxActiveURLSize + 1] = {0};
   char distro[kDistroSize + 1] = {0};
@@ -677,7 +836,9 @@ NonBrowserCrashHandler(const void* crash_context, size_t crash_context_size,
                             // browser to convert namespace tids.
 
   // The length of the control message:
-  static const unsigned kControlMsgSize = CMSG_SPACE(2*sizeof(int));
+  static const unsigned kControlMsgSize = sizeof(fds);
+  static const unsigned kControlMsgSpaceSize = CMSG_SPACE(kControlMsgSize);
+  static const unsigned kControlMsgLenSize = CMSG_LEN(kControlMsgSize);
 
   const size_t kIovSize = 7;
   struct kernel_msghdr msg;
@@ -700,15 +861,15 @@ NonBrowserCrashHandler(const void* crash_context, size_t crash_context_size,
 
   msg.msg_iov = iov;
   msg.msg_iovlen = kIovSize;
-  char cmsg[kControlMsgSize];
-  my_memset(cmsg, 0, kControlMsgSize);
+  char cmsg[kControlMsgSpaceSize];
+  my_memset(cmsg, 0, kControlMsgSpaceSize);
   msg.msg_control = cmsg;
   msg.msg_controllen = sizeof(cmsg);
 
   struct cmsghdr *hdr = CMSG_FIRSTHDR(&msg);
   hdr->cmsg_level = SOL_SOCKET;
   hdr->cmsg_type = SCM_RIGHTS;
-  hdr->cmsg_len = CMSG_LEN(2*sizeof(int));
+  hdr->cmsg_len = kControlMsgLenSize;
   ((int*) CMSG_DATA(hdr))[0] = fds[0];
   ((int*) CMSG_DATA(hdr))[1] = fds[1];
 
@@ -734,21 +895,24 @@ void EnableNonBrowserCrashDumping() {
   // We deliberately leak this object.
   google_breakpad::ExceptionHandler* handler =
       new google_breakpad::ExceptionHandler("" /* unused */, NULL, NULL,
-                                            (void*) fd, true);
+                                            reinterpret_cast<void*>(fd), true);
   handler->set_crash_handler(NonBrowserCrashHandler);
 }
 
 void InitCrashReporter() {
   // Determine the process type and take appropriate action.
   const CommandLine& parsed_command_line = *CommandLine::ForCurrentProcess();
+  if (parsed_command_line.HasSwitch(switches::kDisableBreakpad))
+    return;
+
   const std::string process_type =
       parsed_command_line.GetSwitchValueASCII(switches::kProcessType);
   if (process_type.empty()) {
     EnableCrashDumping(getenv(env_vars::kHeadless) != NULL);
   } else if (process_type == switches::kRendererProcess ||
              process_type == switches::kPluginProcess ||
+             process_type == switches::kPpapiPluginProcess ||
              process_type == switches::kZygoteProcess ||
-             process_type == switches::kNaClLoaderProcess ||
              process_type == switches::kGpuProcess) {
     // We might be chrooted in a zygote or renderer process so we cannot call
     // GetCollectStatsConsent because that needs access the the user's home
