@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,65 +10,58 @@
 #include "base/environment.h"
 #include "base/file_util.h"
 #include "base/logging.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/rand_util.h"  // For PreRead experiment.
+#include "base/sha1.h"  // For PreRead experiment.
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "base/version.h"
+#include "base/win/registry.h"
 #include "chrome/app/breakpad_win.h"
 #include "chrome/app/client_util.h"
 #include "chrome/common/chrome_constants.h"
+#include "chrome/common/chrome_result_codes.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/result_codes.h"
 #include "chrome/installer/util/browser_distribution.h"
+#include "chrome/installer/util/channel_info.h"
 #include "chrome/installer/util/install_util.h"
 #include "chrome/installer/util/google_update_constants.h"
+#include "chrome/installer/util/google_update_settings.h"
 #include "chrome/installer/util/util_constants.h"
 
 namespace {
 // The entry point signature of chrome.dll.
-typedef int (*DLL_MAIN)(HINSTANCE, sandbox::SandboxInterfaceInfo*, wchar_t*);
+typedef int (*DLL_MAIN)(HINSTANCE, sandbox::SandboxInterfaceInfo*);
 
 typedef void (*RelaunchChromeBrowserWithNewCommandLineIfNeededFunc)();
 
-// Not generic, we only handle strings up to 128 chars.
-bool ReadRegistryStr(HKEY key, const wchar_t* name, std::wstring* value) {
-  BYTE out[128 * sizeof(wchar_t)];
-  DWORD size = sizeof(out);
-  DWORD type = 0;
-  if (ERROR_SUCCESS != ::RegQueryValueExW(key, name, NULL, &type, out, &size))
-    return false;
-  if (type != REG_SZ)
-    return false;
-  value->assign(reinterpret_cast<wchar_t*>(out));
-
-  return true;
-}
-
 // Gets chrome version according to the load path. |exe_path| must be the
 // backslash terminated directory of the current chrome.exe.
 bool GetVersion(const wchar_t* exe_path, const wchar_t* key_path,
                 std::wstring* version) {
   HKEY reg_root = InstallUtil::IsPerUserInstall(exe_path) ? HKEY_CURRENT_USER :
                                                             HKEY_LOCAL_MACHINE;
-  HKEY key;
-  if (::RegOpenKeyEx(reg_root, key_path, 0, KEY_READ, &key) != ERROR_SUCCESS)
-    return false;
+  bool success = false;
 
-  // If 'new_chrome.exe' is present it means chrome was auto-updated while
-  // running. We need to consult the opv value so we can load the old dll.
-  // TODO(cpu) : This is solving the same problem as the environment variable
-  // so one of them will eventually be deprecated.
-  std::wstring new_chrome_exe(exe_path);
-  new_chrome_exe.append(installer::kChromeNewExe);
-  if (::PathFileExistsW(new_chrome_exe.c_str()) &&
-      ReadRegistryStr(key, google_update::kRegOldVersionField, version)) {
-    ::RegCloseKey(key);
-    return true;
+  base::win::RegKey key(reg_root, key_path, KEY_QUERY_VALUE);
+  if (key.Valid()) {
+    // If 'new_chrome.exe' is present it means chrome was auto-updated while
+    // running. We need to consult the opv value so we can load the old dll.
+    // TODO(cpu) : This is solving the same problem as the environment variable
+    // so one of them will eventually be deprecated.
+    std::wstring new_chrome_exe(exe_path);
+    new_chrome_exe.append(installer::kChromeNewExe);
+    if (::PathFileExistsW(new_chrome_exe.c_str()) &&
+        key.ReadValue(google_update::kRegOldVersionField,
+                      version) == ERROR_SUCCESS) {
+      success = true;
+    } else {
+      success = (key.ReadValue(google_update::kRegVersionField,
+                               version) == ERROR_SUCCESS);
+    }
   }
 
-  bool ret = ReadRegistryStr(key, google_update::kRegVersionField, version);
-  ::RegCloseKey(key);
-  return ret;
+  return success;
 }
 
 // Gets the path of the current exe with a trailing backslash.
@@ -92,109 +85,173 @@ bool EnvQueryStr(const wchar_t* key_name, std::wstring* value) {
   return true;
 }
 
+#if defined(OS_WIN)
+#if defined(GOOGLE_CHROME_BUILD)
+// These constants are used by the PreRead experiment.
+const wchar_t kPreReadRegistryValue[] = L"PreReadExperimentGroup";
+const int kPreReadExpiryYear = 2012;
+const int kPreReadExpiryMonth = 7;
+const int kPreReadExpiryDay = 1;
+
+bool PreReadShouldRun() {
+  base::Time::Exploded exploded = { 0 };
+  exploded.year = kPreReadExpiryYear;
+  exploded.month = kPreReadExpiryMonth;
+  exploded.day_of_month = kPreReadExpiryDay;
+
+  base::Time expiration_time = base::Time::FromLocalExploded(exploded);
+
+  // Get the build time. This code is copied from
+  // base::FieldTrial::GetBuildTime. We can't use MetricsLogBase::GetBuildTime
+  // because that's in seconds since Unix epoch, which base::Time can't use.
+  base::Time build_time;
+  const char* kDateTime = __DATE__ " " __TIME__;
+  bool result = base::Time::FromString(kDateTime, &build_time);
+  DCHECK(result);
+
+  // If the experiment is expired, don't run it.
+  if (build_time > expiration_time)
+    return false;
+
+  // The experiment should only run on canary and dev.
+  const string16 kChannel(GoogleUpdateSettings::GetChromeChannel(
+      GoogleUpdateSettings::IsSystemInstall()));
+  return kChannel == installer::kChromeChannelCanary ||
+      kChannel == installer::kChromeChannelDev;
+}
+
+// Checks to see if the experiment is running. If so, either tosses a coin
+// and persists it, or gets the already persistent coin-toss value. Returns
+// the coin-toss via |pre_read|. Returns true if the experiment is running and
+// pre_read has been written, false otherwise. |pre_read| is only written to
+// when this function returns true. |key| must be open with read-write access,
+// and be valid.
+bool GetPreReadExperimentGroup(DWORD* pre_read) {
+  DCHECK(pre_read != NULL);
+
+  // Experiment expired, or running on wrong channel?
+  if (!PreReadShouldRun())
+    return false;
+
+  // Get the MetricsId of the installation. This is only set if the user has
+  // opted in to reporting. Doing things this way ensures that we only enable
+  // the experiment if its results are actually going to be reported.
+  std::wstring metrics_id;
+  if (!GoogleUpdateSettings::GetMetricsId(&metrics_id) || metrics_id.empty())
+    return false;
+
+  // We use the same technique as FieldTrial::HashClientId.
+  unsigned char sha1_hash[base::kSHA1Length];
+  base::SHA1HashBytes(
+      reinterpret_cast<const unsigned char*>(metrics_id.c_str()),
+      metrics_id.size() * sizeof(metrics_id[0]),
+      sha1_hash);
+  COMPILE_ASSERT(sizeof(uint64) < sizeof(sha1_hash), need_more_data);
+  uint64* bits = reinterpret_cast<uint64*>(&sha1_hash[0]);
+  double rand_unit = base::BitsToOpenEndedUnitInterval(*bits);
+  DWORD coin_toss = rand_unit > 0.5 ? 1 : 0;
+
+  *pre_read = coin_toss;
+
+  return true;
+}
+#endif  // if defined(GOOGLE_CHROME_BUILD)
+#endif  // if defined(OS_WIN)
+
 // Expects that |dir| has a trailing backslash. |dir| is modified so it
 // contains the full path that was tried. Caller must check for the return
 // value not being null to determine if this path contains a valid dll.
 HMODULE LoadChromeWithDirectory(std::wstring* dir) {
   ::SetCurrentDirectoryW(dir->c_str());
   const CommandLine& cmd_line = *CommandLine::ForCurrentProcess();
-#ifdef _WIN64
-  if ((cmd_line.GetSwitchValueASCII(switches::kProcessType) ==
-      switches::kNaClBrokerProcess) ||
-      (cmd_line.GetSwitchValueASCII(switches::kProcessType) ==
-      switches::kNaClLoaderProcess)) {
-    // Load the 64-bit DLL when running in a 64-bit process.
-    dir->append(installer::kChromeNaCl64Dll);
-  } else {
-    // Only NaCl broker and loader can be launched as Win64 processes.
-    NOTREACHED();
-    return NULL;
-  }
-#else
   dir->append(installer::kChromeDll);
-#endif
 
+#ifndef WIN_DISABLE_PREREAD
 #ifdef NDEBUG
   // Experimental pre-reading optimization
-  // The idea is to pre read significant portion of chrome.dll in advance
+  // The idea is to pre-read a significant portion of chrome.dll in advance
   // so that subsequent hard page faults are avoided.
+  //
+  // Pre-read may be disabled at compile time by defining WIN_DISABLE_PREREAD,
+  // but by default it is enabled in release builds. The ability to disable it
+  // is useful for evaluating competing optimization techniques.
   if (!cmd_line.HasSwitch(switches::kProcessType)) {
     // The kernel brings in 8 pages for the code section at a time and 4 pages
     // for other sections. We can skip over these pages to avoid a soft page
     // fault which may not occur during code execution. However skipping 4K at
     // a time still has better performance over 32K and 16K according to data.
-    // TODO(ananta)
-    // Investigate this and tune.
+    // TODO(ananta): Investigate this and tune.
     const size_t kStepSize = 4 * 1024;
 
     DWORD pre_read_size = 0;
     DWORD pre_read_step_size = kStepSize;
     DWORD pre_read = 1;
 
-    HKEY key = NULL;
-    if (::RegOpenKeyEx(HKEY_CURRENT_USER, L"Software\\Google\\ChromeFrame",
-                       0, KEY_QUERY_VALUE, &key) == ERROR_SUCCESS) {
-      DWORD unused = sizeof(pre_read_size);
-      RegQueryValueEx(key, L"PreReadSize", NULL, NULL,
-                      reinterpret_cast<LPBYTE>(&pre_read_size), &unused);
-      RegQueryValueEx(key, L"PreReadStepSize", NULL, NULL,
-                      reinterpret_cast<LPBYTE>(&pre_read_step_size), &unused);
-      RegQueryValueEx(key, L"PreRead", NULL, NULL,
-                      reinterpret_cast<LPBYTE>(&pre_read), &unused);
-      RegCloseKey(key);
-      key = NULL;
+    // TODO(chrisha): This path should not be ChromeFrame specific, and it
+    //     should not be hard-coded with 'Google' in the path. Rather, it should
+    //     use the product name.
+    base::win::RegKey key(HKEY_CURRENT_USER, L"Software\\Google\\ChromeFrame",
+                          KEY_QUERY_VALUE);
+    if (key.Valid()) {
+      key.ReadValueDW(L"PreReadSize", &pre_read_size);
+      key.ReadValueDW(L"PreReadStepSize", &pre_read_step_size);
+      key.ReadValueDW(L"PreRead", &pre_read);
+      key.Close();
     }
+
+#if defined(OS_WIN)
+#if defined(GOOGLE_CHROME_BUILD)
+    // The PreRead experiment is unable to use the standard FieldTrial
+    // mechanism as pre-reading happens in chrome.exe prior to loading
+    // chrome.dll. As such, we use a custom approach. If the experiment is
+    // running (not expired, and we're running a version of chrome from an
+    // appropriate channel) then we look to the registry for the BreakPad/UMA
+    // metricsid. We use this to seed a coin-toss, which is then communicated
+    // to chrome.dll via an environment variable, which indicates to chrome.dll
+    // that the experiment is running, causing it to report sub-histogram
+    // results.
+
+    // If the experiment is running, indicate it to chrome.dll via an
+    // environment variable.
+    if (GetPreReadExperimentGroup(&pre_read)) {
+      DCHECK(pre_read == 0 || pre_read == 1);
+      scoped_ptr<base::Environment> env(base::Environment::Create());
+      env->SetVar(chrome::kPreReadEnvironmentVariable,
+                  pre_read ? "1" : "0");
+    }
+#endif  // if defined(GOOGLE_CHROME_BUILD)
+#endif  // if defined(OS_WIN)
+
     if (pre_read) {
-      TRACE_EVENT_BEGIN("PreReadImage", 0, "");
+      TRACE_EVENT_BEGIN_ETW("PreReadImage", 0, "");
       file_util::PreReadImage(dir->c_str(), pre_read_size, pre_read_step_size);
-      TRACE_EVENT_END("PreReadImage", 0, "");
+      TRACE_EVENT_END_ETW("PreReadImage", 0, "");
     }
   }
 #endif  // NDEBUG
+#endif  // WIN_DISABLE_PREREAD
 
   return ::LoadLibraryExW(dir->c_str(), NULL,
                           LOAD_WITH_ALTERED_SEARCH_PATH);
 }
 
-// Set did_run "dr" in omaha's client state for this product.
-bool SetDidRunState(const wchar_t* guid, const wchar_t* value) {
-  std::wstring key_path(google_update::kRegPathClientState);
-  key_path.append(L"\\").append(guid);
-  HKEY reg_key;
-  if (::RegCreateKeyExW(HKEY_CURRENT_USER, key_path.c_str(), 0, NULL,
-                        REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL,
-                        &reg_key, NULL) == ERROR_SUCCESS) {
-    // Note that the length here must be in bytes and must account for the
-    // terminating null char.
-    ::RegSetValueExW(reg_key, google_update::kRegDidRunField, 0, REG_SZ,
-                     reinterpret_cast<const BYTE *>(value),
-                     (::lstrlenW(value) + 1) * sizeof(wchar_t));
-    ::RegCloseKey(reg_key);
-    return true;
-  }
-  return false;
+void RecordDidRun(const std::wstring& dll_path) {
+  bool system_level = !InstallUtil::IsPerUserInstall(dll_path.c_str());
+  GoogleUpdateSettings::UpdateDidRunState(true, system_level);
 }
 
-bool RecordDidRun(const wchar_t* guid) {
-  return SetDidRunState(guid, L"1");
+void ClearDidRun(const std::wstring& dll_path) {
+  bool system_level = !InstallUtil::IsPerUserInstall(dll_path.c_str());
+  GoogleUpdateSettings::UpdateDidRunState(false, system_level);
 }
 
-bool ClearDidRun(const wchar_t* guid) {
-  return SetDidRunState(guid, L"0");
-}
-
-}
+}  // namespace
 //=============================================================================
 
 MainDllLoader::MainDllLoader() : dll_(NULL) {
 }
 
 MainDllLoader::~MainDllLoader() {
-#ifdef PURIFY
-  // We should never unload the dll. There is only risk and no gain from
-  // doing so. The singleton dtors have been already run by AtExitManager.
-  ::FreeLibrary(dll_);
-#endif
 }
 
 // Loading chrome is an interesting affair. First we try loading from the
@@ -249,8 +306,13 @@ HMODULE MainDllLoader::Load(std::wstring* out_version, std::wstring* out_file) {
     *out_file = dir;
     *out_version = version_string;
     out_file->append(*out_version).append(L"\\");
-    return LoadChromeWithDirectory(out_file);
+    dll = LoadChromeWithDirectory(out_file);
+    if (!dll) {
+      LOG(ERROR) << "Failed to load Chrome DLL from " << out_file;
+    }
+    return dll;
   } else {
+    LOG(ERROR) << "Could not get Chrome DLL version.";
     return NULL;
   }
 }
@@ -264,21 +326,21 @@ int MainDllLoader::Launch(HINSTANCE instance,
   std::wstring file;
   dll_ = Load(&version, &file);
   if (!dll_)
-    return ResultCodes::MISSING_DATA;
+    return chrome::RESULT_CODE_MISSING_DATA;
 
   scoped_ptr<base::Environment> env(base::Environment::Create());
   env->SetVar(chrome::kChromeVersionEnvVar, WideToUTF8(version));
 
   InitCrashReporterWithDllPath(file);
-  OnBeforeLaunch();
+  OnBeforeLaunch(file);
 
   DLL_MAIN entry_point =
       reinterpret_cast<DLL_MAIN>(::GetProcAddress(dll_, "ChromeMain"));
   if (!entry_point)
-    return ResultCodes::BAD_PROCESS_TYPE;
+    return chrome::RESULT_CODE_BAD_PROCESS_TYPE;
 
-  int rc = entry_point(instance, sbox_info, ::GetCommandLineW());
-  return OnBeforeExit(rc);
+  int rc = entry_point(instance, sbox_info);
+  return OnBeforeExit(rc, file);
 }
 
 void MainDllLoader::RelaunchChromeBrowserWithNewCommandLineIfNeeded() {
@@ -305,18 +367,16 @@ class ChromeDllLoader : public MainDllLoader {
     return key;
   }
 
-  virtual void OnBeforeLaunch() {
-    BrowserDistribution* dist = BrowserDistribution::GetDistribution();
-    RecordDidRun(dist->GetAppGuid().c_str());
+  virtual void OnBeforeLaunch(const std::wstring& dll_path) {
+    RecordDidRun(dll_path);
   }
 
-  virtual int OnBeforeExit(int return_code) {
+  virtual int OnBeforeExit(int return_code, const std::wstring& dll_path) {
     // NORMAL_EXIT_CANCEL is used for experiments when the user cancels
     // so we need to reset the did_run signal so omaha does not count
     // this run as active usage.
-    if (ResultCodes::NORMAL_EXIT_CANCEL == return_code) {
-      BrowserDistribution* dist = BrowserDistribution::GetDistribution();
-      ClearDidRun(dist->GetAppGuid().c_str());
+    if (chrome::RESULT_CODE_NORMAL_EXIT_CANCEL == return_code) {
+      ClearDidRun(dll_path);
     }
     return return_code;
   }
