@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,27 +10,56 @@
 #include <set>
 
 #include "base/command_line.h"
-#include "base/singleton.h"
+#include "base/memory/singleton.h"
 #include "base/string_number_conversions.h"
+#include "base/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/metrics/user_metrics.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/prefs/scoped_user_pref_update.h"
+#include "chrome/common/chrome_content_client.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/browser/user_metrics.h"
 #include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/gfx/gl/gl_switches.h"
+
+#if defined(USE_AURA)
+#include "ash/ash_switches.h"
+#endif
+
+using content::UserMetricsAction;
 
 namespace about_flags {
 
 // Macros to simplify specifying the type.
-#define SINGLE_VALUE_TYPE(command_line) Experiment::SINGLE_VALUE, \
-    command_line, NULL, 0
-#define MULTI_VALUE_TYPE(choices) Experiment::MULTI_VALUE, "", choices, \
-    arraysize(choices)
+#define SINGLE_VALUE_TYPE_AND_VALUE(command_line_switch, switch_value) \
+    Experiment::SINGLE_VALUE, command_line_switch, switch_value, NULL, 0
+#define SINGLE_VALUE_TYPE(command_line_switch) \
+    SINGLE_VALUE_TYPE_AND_VALUE(command_line_switch, "")
+#define MULTI_VALUE_TYPE(choices) \
+    Experiment::MULTI_VALUE, "", "", choices, arraysize(choices)
 
 namespace {
 
-const unsigned kOsAll = kOsMac | kOsWin | kOsLinux | kOsCrOS;
+const unsigned kOsAll = kOsMac | kOsWin | kOsLinux | kOsCrOS | kOsAndroid;
+
+// Adds a |StringValue| to |list| for each platform where |bitmask| indicates
+// whether the experiment is available on that platform.
+void AddOsStrings(unsigned bitmask, ListValue* list) {
+  struct {
+    unsigned bit;
+    const char* const name;
+  } kBitsToOs[] = {
+    {kOsMac, "Mac"},
+    {kOsWin, "Windows"},
+    {kOsLinux, "Linux"},
+    {kOsCrOS, "Chrome OS"},
+  };
+  for (size_t i = 0; i < ARRAYSIZE_UNSAFE(kBitsToOs); ++i)
+    if (bitmask & kBitsToOs[i].bit)
+      list->Append(new StringValue(kBitsToOs[i].name));
+}
 
 // Names for former Chrome OS Labs experiments, shared with prefs migration
 // code.
@@ -38,15 +67,34 @@ const char kMediaPlayerExperimentName[] = "media-player";
 const char kAdvancedFileSystemExperimentName[] = "advanced-file-system";
 const char kVerticalTabsExperimentName[] = "vertical-tabs";
 
-// If adding a new choice, add it to the end of the list.
-const Experiment::Choice kInstantChoices[] = {
-  { IDS_FLAGS_INSTANT_TYPE_VERBATIM, switches::kEnableVerbatimInstant },
-  { IDS_FLAGS_INSTANT_TYPE_PREDICTIVE, switches::kEnablePredictiveInstant },
-  { IDS_FLAGS_INSTANT_TYPE_PREDICTIVE_NO_AUTO_COMPLETE,
-    switches::kEnablePredictiveNoAutoCompleteInstant
-  },
+const Experiment::Choice kPrerenderFromOmniboxChoices[] = {
+  { IDS_FLAGS_PRERENDER_FROM_OMNIBOX_AUTOMATIC, "", "" },
+  { IDS_FLAGS_PRERENDER_FROM_OMNIBOX_ENABLED, switches::kPrerenderFromOmnibox,
+    switches::kPrerenderFromOmniboxSwitchValueEnabled },
+  { IDS_FLAGS_PRERENDER_FROM_OMNIBOX_DISABLED, switches::kPrerenderFromOmnibox,
+    switches::kPrerenderFromOmniboxSwitchValueDisabled }
 };
 
+const Experiment::Choice kOmniboxAggressiveHistoryURLChoices[] = {
+  { IDS_FLAGS_OMNIBOX_AGGRESSIVE_HISTORY_URL_SCORING_AUTOMATIC, "", "" },
+  { IDS_FLAGS_OMNIBOX_AGGRESSIVE_HISTORY_URL_SCORING_ENABLED,
+    switches::kOmniboxAggressiveHistoryURL,
+    switches::kOmniboxAggressiveHistoryURLEnabled },
+  { IDS_FLAGS_OMNIBOX_AGGRESSIVE_HISTORY_URL_SCORING_DISABLED,
+    switches::kOmniboxAggressiveHistoryURL,
+    switches::kOmniboxAggressiveHistoryURLDisabled }
+};
+
+#if defined(USE_AURA)
+const Experiment::Choice kAuraWindowModeChoices[] = {
+  { IDS_FLAGS_AURA_WINDOW_MODE_AUTOMATIC, "", "" },
+  { IDS_FLAGS_AURA_WINDOW_MODE_NORMAL,
+      ash::switches::kAuraWindowMode, ash::switches::kAuraWindowModeNormal },
+  { IDS_FLAGS_AURA_WINDOW_MODE_COMPACT,
+      ash::switches::kAuraWindowMode, ash::switches::kAuraWindowModeCompact }
+};
+#endif
+
 // RECORDING USER METRICS FOR FLAGS:
 // -----------------------------------------------------------------------------
 // The first line of the experiment is the internal name. If you'd like to
@@ -71,9 +119,10 @@ const Experiment::Choice kInstantChoices[] = {
 // distinct types of experiments:
 // . SINGLE_VALUE: experiment is either on or off. Use the SINGLE_VALUE_TYPE
 //   macro for this type supplying the command line to the macro.
-// . MULTI_VALUE: if enabled the command line of the selected choice is enabled.
-//   To specify this type of experiment use the macro MULTI_VALUE_TYPE supplying
-//   it the array of choices.
+// . MULTI_VALUE: a list of choices, the first of which should correspond to a
+//   deactivated state for this lab (i.e. no command line option).  To specify
+//   this type of experiment use the macro MULTI_VALUE_TYPE supplying it the
+//   array of choices.
 // See the documentation of Experiment for details on the fields.
 //
 // When adding a new choice, add it to the end of the list.
@@ -90,51 +139,6 @@ const Experiment kExperiments[] = {
     SINGLE_VALUE_TYPE("")
 #endif
   },
-  {
-    kMediaPlayerExperimentName,
-    IDS_FLAGS_MEDIA_PLAYER_NAME,
-    IDS_FLAGS_MEDIA_PLAYER_DESCRIPTION,
-    kOsCrOS,
-#if defined(OS_CHROMEOS)
-    // The switch exists only on Chrome OS.
-    SINGLE_VALUE_TYPE(switches::kEnableMediaPlayer)
-#else
-    SINGLE_VALUE_TYPE("")
-#endif
-  },
-  {
-    kAdvancedFileSystemExperimentName,
-    IDS_FLAGS_ADVANCED_FS_NAME,
-    IDS_FLAGS_ADVANCED_FS_DESCRIPTION,
-    kOsCrOS,
-#if defined(OS_CHROMEOS)
-    // The switch exists only on Chrome OS.
-    SINGLE_VALUE_TYPE(switches::kEnableAdvancedFileSystem)
-#else
-    SINGLE_VALUE_TYPE("")
-#endif
-  },
-  {
-    "vertical-tabs",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_SIDE_TABS_NAME,
-    IDS_FLAGS_SIDE_TABS_DESCRIPTION,
-    kOsWin | kOsCrOS,
-    SINGLE_VALUE_TYPE(switches::kEnableVerticalTabs)
-  },
-  {
-    "remoting",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_REMOTING_NAME,
-    IDS_FLAGS_REMOTING_DESCRIPTION,
-    kOsAll,
-    SINGLE_VALUE_TYPE(switches::kEnableRemoting)
-  },
-  {
-    "xss-auditor",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_XSS_AUDITOR_NAME,
-    IDS_FLAGS_XSS_AUDITOR_DESCRIPTION,
-    kOsAll,
-    SINGLE_VALUE_TYPE(switches::kEnableXSSAuditor)
-  },
   {
     "conflicting-modules-check",  // FLAGS:RECORD_UMA
     IDS_FLAGS_CONFLICTS_CHECK_NAME,
@@ -144,16 +148,17 @@ const Experiment kExperiments[] = {
   },
   {
     "cloud-print-proxy",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_CLOUD_PRINT_PROXY_NAME,
-    IDS_FLAGS_CLOUD_PRINT_PROXY_DESCRIPTION,
-#if defined(GOOGLE_CHROME_BUILD)
-    // For a Chrome build, we know we have a PDF plug-in, and so we'll
-    // enable by platform as we get things working.
-    0,
-#else
-    // Otherwise, where we know it could be working if a viable PDF
-    // plug-in could be supplied, we'll keep the lab enabled.
+    IDS_FLAGS_CLOUD_PRINT_CONNECTOR_NAME,
+    IDS_FLAGS_CLOUD_PRINT_CONNECTOR_DESCRIPTION,
+    // For a Chrome build, we know we have a PDF plug-in on Windows, so it's
+    // fully enabled.
+    // Otherwise, where we know Windows could be working if a viable PDF
+    // plug-in could be supplied, we'll keep the lab enabled. Mac and Linux
+    // always have PDF rasterization available, so no flag needed there.
+#if !defined(GOOGLE_CHROME_BUILD)
     kOsWin,
+#else
+    0,
 #endif
     SINGLE_VALUE_TYPE(switches::kEnableCloudPrintProxy)
   },
@@ -165,28 +170,66 @@ const Experiment kExperiments[] = {
     SINGLE_VALUE_TYPE(switches::kEnableCrxlessWebApps)
   },
   {
-    "gpu-compositing",
-     IDS_FLAGS_ACCELERATED_COMPOSITING_NAME,
-     IDS_FLAGS_ACCELERATED_COMPOSITING_DESCRIPTION,
-     kOsAll,
-     SINGLE_VALUE_TYPE(switches::kEnableAcceleratedLayers)
+    "ignore-gpu-blacklist",
+    IDS_FLAGS_IGNORE_GPU_BLACKLIST_NAME,
+    IDS_FLAGS_IGNORE_GPU_BLACKLIST_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kIgnoreGpuBlacklist)
   },
   {
-    "gpu-canvas-2d",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_ACCELERATED_CANVAS_2D_NAME,
-    IDS_FLAGS_ACCELERATED_CANVAS_2D_DESCRIPTION,
-    kOsWin | kOsLinux | kOsCrOS,
-    SINGLE_VALUE_TYPE(switches::kEnableAccelerated2dCanvas)
+    "force-compositing-mode-2",
+    IDS_FLAGS_FORCE_COMPOSITING_MODE_NAME,
+    IDS_FLAGS_FORCE_COMPOSITING_MODE_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kForceCompositingMode)
   },
-  // FIXME(scheib): Add Flags entry for WebGL,
-  // or pull it and the strings in generated_resources.grd by Dec 2010
-  // {
-  //   "webgl",
-  //   IDS_FLAGS_WEBGL_NAME,
-  //   IDS_FLAGS_WEBGL_DESCRIPTION,
-  //   kOsAll,
-  //   SINGLE_VALUE_TYPE(switches::kDisableExperimentalWebGL)
-  // }
+  {
+    "composited-layer-borders",
+    IDS_FLAGS_COMPOSITED_LAYER_BORDERS,
+    IDS_FLAGS_COMPOSITED_LAYER_BORDERS_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kShowCompositedLayerBorders)
+  },
+  {
+    "show-fps-counter",
+    IDS_FLAGS_SHOW_FPS_COUNTER,
+    IDS_FLAGS_SHOW_FPS_COUNTER_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kShowFPSCounter)
+  },
+  {
+    "accelerated-filters",
+    IDS_FLAGS_ACCELERATED_FILTERS,
+    IDS_FLAGS_ACCELERATED_FILTERS_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableAcceleratedFilters)
+  },
+  {
+    "disable-gpu-vsync",
+    IDS_FLAGS_DISABLE_GPU_VSYNC_NAME,
+    IDS_FLAGS_DISABLE_GPU_VSYNC_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kDisableGpuVsync)
+  },
+  {
+    "disable-webgl",
+    IDS_FLAGS_DISABLE_WEBGL_NAME,
+    IDS_FLAGS_DISABLE_WEBGL_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kDisableExperimentalWebGL)
+  },
+
+#if defined(GOOGLE_CHROME_BUILD)
+  // TODO(thestig) Remove this for bug 107600.
+  {
+    "disable-print-preview",  // FLAGS:RECORD_UMA
+    IDS_FLAGS_DISABLE_PRINT_PREVIEW_NAME,
+    IDS_FLAGS_DISABLE_PRINT_PREVIEW_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kDisablePrintPreview)
+  },
+#else
+  // For Chromium builds where users may not have the PDF plugin.
   {
     "print-preview",  // FLAGS:RECORD_UMA
     IDS_FLAGS_PRINT_PREVIEW_NAME,
@@ -194,6 +237,9 @@ const Experiment kExperiments[] = {
     kOsAll,
     SINGLE_VALUE_TYPE(switches::kEnablePrintPreview)
   },
+#endif
+
+  // TODO(dspringer): When NaCl is on by default, remove this flag entry.
   {
     "enable-nacl",  // FLAGS:RECORD_UMA
     IDS_FLAGS_ENABLE_NACL_NAME,
@@ -201,27 +247,6 @@ const Experiment kExperiments[] = {
     kOsAll,
     SINGLE_VALUE_TYPE(switches::kEnableNaCl)
   },
-  {
-    "dns-server",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_DNS_SERVER_NAME,
-    IDS_FLAGS_DNS_SERVER_DESCRIPTION,
-    kOsLinux,
-    SINGLE_VALUE_TYPE(switches::kDnsServer)
-  },
-  {
-    "page-prerender",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_PAGE_PRERENDER_NAME,
-    IDS_FLAGS_PAGE_PRERENDER_DESCRIPTION,
-    kOsAll,
-    SINGLE_VALUE_TYPE(switches::kEnablePagePrerender)
-  },
-  {
-    "confirm-to-quit",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_CONFIRM_TO_QUIT_NAME,
-    IDS_FLAGS_CONFIRM_TO_QUIT_DESCRIPTION,
-    kOsMac,
-    SINGLE_VALUE_TYPE(switches::kEnableConfirmToQuit)
-  },
   {
     "extension-apis",  // FLAGS:RECORD_UMA
     IDS_FLAGS_EXPERIMENTAL_EXTENSION_APIS_NAME,
@@ -230,11 +255,11 @@ const Experiment kExperiments[] = {
     SINGLE_VALUE_TYPE(switches::kEnableExperimentalExtensionApis)
   },
   {
-    "click-to-play",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_CLICK_TO_PLAY_NAME,
-    IDS_FLAGS_CLICK_TO_PLAY_DESCRIPTION,
+    "apps-new-install-bubble",
+    IDS_FLAGS_APPS_NEW_INSTALL_BUBBLE_NAME,
+    IDS_FLAGS_APPS_NEW_INSTALL_BUBBLE_DESCRIPTION,
     kOsAll,
-    SINGLE_VALUE_TYPE(switches::kEnableClickToPlay)
+    SINGLE_VALUE_TYPE(switches::kAppsNewInstallBubble)
   },
   {
     "disable-hyperlink-auditing",
@@ -250,27 +275,6 @@ const Experiment kExperiments[] = {
     kOsMac | kOsWin | kOsLinux,  // Currently does nothing on CrOS.
     SINGLE_VALUE_TYPE(switches::kExperimentalLocationFeatures)
   },
-  {
-    "instant-type",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_INSTANT_TYPE_NAME,
-    IDS_FLAGS_INSTANT_TYPE_DESCRIPTION,
-    kOsWin,
-    MULTI_VALUE_TYPE(kInstantChoices)
-  },
-  {
-    "instant-autocomplete-immediately",  // FLAGS:RECORD_UMA
-    IDS_FLAGS_INSTANT_AUTOCOMPLETE_IMMEDIATELY_NAME,
-    IDS_FLAGS_INSTANT_AUTOCOMPLETE_IMMEDIATELY_DESCRIPTION,
-    kOsWin | kOsLinux,
-    SINGLE_VALUE_TYPE(switches::kInstantAutocompleteImmediately)
-  },
-  {
-    "block-reading-third-party-cookies",
-    IDS_FLAGS_BLOCK_ALL_THIRD_PARTY_COOKIES_NAME,
-    IDS_FLAGS_BLOCK_ALL_THIRD_PARTY_COOKIES_DESCRIPTION,
-    kOsAll,
-    SINGLE_VALUE_TYPE(switches::kBlockReadingThirdPartyCookies)
-  },
   {
     "disable-interactive-form-validation",
     IDS_FLAGS_DISABLE_INTERACTIVE_FORM_VALIDATION_NAME,
@@ -279,11 +283,269 @@ const Experiment kExperiments[] = {
     SINGLE_VALUE_TYPE(switches::kDisableInteractiveFormValidation)
   },
   {
-    "webaudio",
-    IDS_FLAGS_WEBAUDIO_NAME,
-    IDS_FLAGS_WEBAUDIO_DESCRIPTION,
-    kOsMac,  // TODO(crogers): add windows and linux when FFT is ready.
-    SINGLE_VALUE_TYPE(switches::kEnableWebAudio)
+    "focus-existing-tab-on-open",  // FLAGS:RECORD_UMA
+    IDS_FLAGS_FOCUS_EXISTING_TAB_ON_OPEN_NAME,
+    IDS_FLAGS_FOCUS_EXISTING_TAB_ON_OPEN_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kFocusExistingTabOnOpen)
+  },
+  {
+    "tab-groups-context-menu",
+    IDS_FLAGS_TAB_GROUPS_CONTEXT_MENU_NAME,
+    IDS_FLAGS_TAB_GROUPS_CONTEXT_MENU_DESCRIPTION,
+    kOsWin,
+    SINGLE_VALUE_TYPE(switches::kEnableTabGroupsContextMenu)
+  },
+  {
+    "preload-instant-search",
+    IDS_FLAGS_PRELOAD_INSTANT_SEARCH_NAME,
+    IDS_FLAGS_PRELOAD_INSTANT_SEARCH_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kPreloadInstantSearch)
+  },
+  {
+    "static-ip-config",
+    IDS_FLAGS_STATIC_IP_CONFIG_NAME,
+    IDS_FLAGS_STATIC_IP_CONFIG_DESCRIPTION,
+    kOsCrOS,
+#if defined(OS_CHROMEOS)
+    // This switch exists only on Chrome OS.
+    SINGLE_VALUE_TYPE(switches::kEnableStaticIPConfig)
+#else
+    SINGLE_VALUE_TYPE("")
+#endif
+  },
+  {
+    "show-autofill-type-predictions",
+    IDS_FLAGS_SHOW_AUTOFILL_TYPE_PREDICTIONS_NAME,
+    IDS_FLAGS_SHOW_AUTOFILL_TYPE_PREDICTIONS_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kShowAutofillTypePredictions)
+  },
+  {
+    "sync-tabs",
+    IDS_FLAGS_SYNC_TABS_NAME,
+    IDS_FLAGS_SYNC_TABS_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableSyncTabs)
+  },
+  {
+    "sync-app-notifications",
+    IDS_FLAGS_SYNC_APP_NOTIFICATIONS_NAME,
+    IDS_FLAGS_SYNC_APP_NOTIFICATIONS_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kDisableSyncAppNotifications)
+  },
+  {
+    "enable-smooth-scrolling",  // FLAGS:RECORD_UMA
+    IDS_FLAGS_ENABLE_SMOOTH_SCROLLING_NAME,
+    IDS_FLAGS_ENABLE_SMOOTH_SCROLLING_DESCRIPTION,
+    // Can't expose the switch unless the code is compiled in.
+    // On by default for the Mac (different implementation in WebKit).
+    kOsWin | kOsLinux | kOsCrOS,
+    SINGLE_VALUE_TYPE(switches::kEnableSmoothScrolling)
+  },
+  {
+    "prerender-from-omnibox",  // FLAGS:RECORD_UMA
+    IDS_FLAGS_PRERENDER_FROM_OMNIBOX_NAME,
+    IDS_FLAGS_PRERENDER_FROM_OMNIBOX_DESCRIPTION,
+    kOsAll,
+    MULTI_VALUE_TYPE(kPrerenderFromOmniboxChoices)
+  },
+  {
+    "omnibox-aggressive-with-history-url",
+    IDS_FLAGS_OMNIBOX_AGGRESSIVE_HISTORY_URL_SCORING_NAME,
+    IDS_FLAGS_OMNIBOX_AGGRESSIVE_HISTORY_URL_SCORING_DESCRIPTION,
+    kOsAll,
+    MULTI_VALUE_TYPE(kOmniboxAggressiveHistoryURLChoices)
+  },
+  {
+    "enable-panels",
+    IDS_FLAGS_ENABLE_PANELS_NAME,
+    IDS_FLAGS_ENABLE_PANELS_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnablePanels)
+  },
+  {
+    "disable-shortcuts-provider",
+    IDS_FLAGS_DISABLE_SHORTCUTS_PROVIDER,
+    IDS_FLAGS_DISABLE_SHORTCUTS_PROVIDER_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kDisableShortcutsProvider)
+  },
+#if defined(OS_CHROMEOS)
+  {
+    "enable-bluetooth",
+    IDS_FLAGS_ENABLE_BLUETOOTH_NAME,
+    IDS_FLAGS_ENABLE_BLUETOOTH_DESCRIPTION,
+    kOsCrOS,
+    SINGLE_VALUE_TYPE(switches::kEnableBluetooth)
+  },
+#endif
+  {
+    "memory-widget",
+    IDS_FLAGS_MEMORY_WIDGET_NAME,
+    IDS_FLAGS_MEMORY_WIDGET_DESCRIPTION,
+    kOsCrOS,
+#if defined(OS_CHROMEOS)
+    // This switch exists only on Chrome OS.
+    SINGLE_VALUE_TYPE(switches::kMemoryWidget)
+#else
+    SINGLE_VALUE_TYPE("")
+#endif
+  },
+  {
+    "downloads-new-ui",  // FLAGS:RECORD_UMA
+    IDS_FLAGS_DOWNLOADS_NEW_UI_NAME,
+    IDS_FLAGS_DOWNLOADS_NEW_UI_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kDownloadsNewUI)
+  },
+  {
+    "enable-autologin",
+    IDS_FLAGS_ENABLE_AUTOLOGIN_NAME,
+    IDS_FLAGS_ENABLE_AUTOLOGIN_DESCRIPTION,
+    kOsMac | kOsWin | kOsLinux,
+    SINGLE_VALUE_TYPE(switches::kEnableAutologin)
+  },
+  {
+    "use-more-webui",
+    IDS_FLAGS_USE_MORE_WEBUI_NAME,
+    IDS_FLAGS_USE_MORE_WEBUI_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kUseMoreWebUI)
+  },
+  {
+    "enable-http-pipelining",
+    IDS_FLAGS_ENABLE_HTTP_PIPELINING_NAME,
+    IDS_FLAGS_ENABLE_HTTP_PIPELINING_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableHttpPipelining)
+  },
+  {
+    "enable-video-track",
+    IDS_FLAGS_ENABLE_VIDEO_TRACK_NAME,
+    IDS_FLAGS_ENABLE_VIDEO_TRACK_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableVideoTrack)
+  },
+  {
+    "enable-media-source",
+    IDS_FLAGS_ENABLE_MEDIA_SOURCE_NAME,
+    IDS_FLAGS_ENABLE_MEDIA_SOURCE_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableMediaSource)
+  },
+  {
+    "enable-pointer-lock",
+    IDS_FLAGS_ENABLE_POINTER_LOCK_NAME,
+    IDS_FLAGS_ENABLE_POINTER_LOCK_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnablePointerLock)
+  },
+#if defined(USE_AURA)
+  {
+    "aura-workspace-manager",
+    IDS_FLAGS_AURA_WORKSPACE_MANAGER_NAME,
+    IDS_FLAGS_AURA_WORKSPACE_MANAGER_DESCRIPTION,
+    kOsWin | kOsLinux | kOsCrOS,
+    SINGLE_VALUE_TYPE(ash::switches::kAuraWorkspaceManager)
+  },
+  {
+    "aura-translucent-frames",
+    IDS_FLAGS_AURA_TRANSLUCENT_FRAMES_NAME,
+    IDS_FLAGS_AURA_TRANSLUCENT_FRAMES_DESCRIPTION,
+    kOsWin | kOsLinux | kOsCrOS,
+    SINGLE_VALUE_TYPE(ash::switches::kAuraTranslucentFrames)
+  },
+  {
+    "aura-google-dialog-frames",
+    IDS_FLAGS_AURA_GOOGLE_DIALOG_FRAMES_NAME,
+    IDS_FLAGS_AURA_GOOGLE_DIALOG_FRAMES_DESCRIPTION,
+    kOsWin | kOsLinux | kOsCrOS,
+    SINGLE_VALUE_TYPE(ash::switches::kAuraGoogleDialogFrames)
+  },
+  // TODO(jamescook): Enable this for all ChromeOS builds when we're sure
+  // Aura laptop mode performance and feature set match traditional non-Aura
+  // builds.
+  {
+    "aura-window-mode",
+    IDS_FLAGS_AURA_WINDOW_MODE_NAME,
+    IDS_FLAGS_AURA_WINDOW_MODE_DESCRIPTION,
+    kOsWin | kOsLinux | kOsCrOS,
+    MULTI_VALUE_TYPE(kAuraWindowModeChoices)
+  },
+#endif  // defined(USE_AURA)
+  {
+    "enable-gamepad",
+    IDS_FLAGS_ENABLE_GAMEPAD_NAME,
+    IDS_FLAGS_ENABLE_GAMEPAD_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableGamepad)
+  },
+  {
+    "per-tile-painting",
+    IDS_FLAGS_PER_TILE_PAINTING_NAME,
+    IDS_FLAGS_PER_TILE_PAINTING_DESCRIPTION,
+#if defined(USE_SKIA)
+    kOsMac | kOsLinux | kOsCrOS,
+#else
+    0,
+#endif
+    SINGLE_VALUE_TYPE(switches::kEnablePerTilePainting)
+  },
+  {
+    "enable-javascript-harmony",
+    IDS_FLAGS_ENABLE_JAVASCRIPT_HARMONY_NAME,
+    IDS_FLAGS_ENABLE_JAVASCRIPT_HARMONY_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE_AND_VALUE(switches::kJavaScriptFlags, "--harmony")
+  },
+  {
+    "enable-tab-browser-dragging",
+    IDS_FLAGS_ENABLE_TAB_BROWSER_DRAGGING_NAME,
+    IDS_FLAGS_ENABLE_TAB_BROWSER_DRAGGING_DESCRIPTION,
+    kOsWin,
+    SINGLE_VALUE_TYPE(switches::kTabBrowserDragging)
+  },
+  {
+    "enable-restore-session-state",
+    IDS_FLAGS_ENABLE_RESTORE_SESSION_STATE_NAME,
+    IDS_FLAGS_ENABLE_RESTORE_SESSION_STATE_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableRestoreSessionState)
+  },
+  {
+    "disable-software-rasterizer",
+    IDS_FLAGS_DISABLE_SOFTWARE_RASTERIZER_NAME,
+    IDS_FLAGS_DISABLE_SOFTWARE_RASTERIZER_DESCRIPTION,
+#if defined(ENABLE_SWIFTSHADER)
+    kOsAll,
+#else
+    0,
+#endif
+    SINGLE_VALUE_TYPE(switches::kDisableSoftwareRasterizer)
+  },
+  {
+    "enable-media-stream",
+    IDS_FLAGS_MEDIA_STREAM_NAME,
+    IDS_FLAGS_MEDIA_STREAM_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableMediaStream)
+  },
+  {
+    "enable-uber-page",
+    IDS_FLAGS_ENABLE_UBER_PAGE_NAME,
+    IDS_FLAGS_ENABLE_UBER_PAGE_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableUberPage)
+  },
+  {
+    "enable-shadow-dom",
+    IDS_FLAGS_SHADOW_DOM_NAME,
+    IDS_FLAGS_SHADOW_DOM_DESCRIPTION,
+    kOsAll,
+    SINGLE_VALUE_TYPE(switches::kEnableShadowDOM)
   },
 };
 
@@ -309,7 +571,7 @@ class FlagsState {
 
  private:
   bool needs_restart_;
-  std::set<std::string> flags_switches_;
+  std::map<std::string, std::string> flags_switches_;
 
   DISALLOW_COPY_AND_ASSIGN(FlagsState);
 };
@@ -337,10 +599,8 @@ void GetEnabledFlags(const PrefService* prefs, std::set<std::string>* result) {
 // Takes a set of enabled lab experiments
 void SetEnabledFlags(
     PrefService* prefs, const std::set<std::string>& enabled_experiments) {
-  ListValue* experiments_list = prefs->GetMutableList(
-      prefs::kEnabledLabsExperiments);
-  if (!experiments_list)
-    return;
+  ListPrefUpdate update(prefs, prefs::kEnabledLabsExperiments);
+  ListValue* experiments_list = update.Get();
 
   experiments_list->Clear();
   for (std::set<std::string>::const_iterator it = enabled_experiments.begin();
@@ -352,8 +612,8 @@ void SetEnabledFlags(
 
 // Returns the name used in prefs for the choice at the specified index.
 std::string NameForChoice(const Experiment& e, int index) {
-  DCHECK(e.type == Experiment::MULTI_VALUE);
-  DCHECK(index < e.num_choices);
+  DCHECK_EQ(Experiment::MULTI_VALUE, e.type);
+  DCHECK_LT(index, e.num_choices);
   return std::string(e.internal_name) + about_flags::testing::kMultiSeparator +
       base::IntToString(index);
 }
@@ -363,19 +623,41 @@ void AddInternalName(const Experiment& e, std::set<std::string>* names) {
   if (e.type == Experiment::SINGLE_VALUE) {
     names->insert(e.internal_name);
   } else {
-    DCHECK(e.type == Experiment::MULTI_VALUE);
+    DCHECK_EQ(Experiment::MULTI_VALUE, e.type);
     for (int i = 0; i < e.num_choices; ++i)
       names->insert(NameForChoice(e, i));
   }
 }
 
+// Confirms that an experiment is valid, used in a DCHECK in
+// SanitizeList below.
+bool ValidateExperiment(const Experiment& e) {
+  switch (e.type) {
+    case Experiment::SINGLE_VALUE:
+      DCHECK_EQ(0, e.num_choices);
+      DCHECK(!e.choices);
+      break;
+    case Experiment::MULTI_VALUE:
+      DCHECK_GT(e.num_choices, 0);
+      DCHECK(e.choices);
+      DCHECK(e.choices[0].command_line_switch);
+      DCHECK_EQ('\0', e.choices[0].command_line_switch[0]);
+      break;
+    default:
+      NOTREACHED();
+  }
+  return true;
+}
+
 // Removes all experiments from prefs::kEnabledLabsExperiments that are
 // unknown, to prevent this list to become very long as experiments are added
 // and removed.
 void SanitizeList(PrefService* prefs) {
   std::set<std::string> known_experiments;
-  for (size_t i = 0; i < num_experiments; ++i)
+  for (size_t i = 0; i < num_experiments; ++i) {
+    DCHECK(ValidateExperiment(experiments[i]));
     AddInternalName(experiments[i], &known_experiments);
+  }
 
   std::set<std::string> enabled_experiments;
   GetEnabledFlags(prefs, &enabled_experiments);
@@ -421,11 +703,9 @@ void GetSanitizedEnabledFlagsForCurrentPlatform(
 }
 
 // Returns the Value representing the choice data in the specified experiment.
-// If one of the choices is enabled |is_one_selected| is set to true.
 Value* CreateChoiceData(const Experiment& experiment,
-                        const std::set<std::string>& enabled_experiments,
-                        bool* is_one_selected) {
-  DCHECK(experiment.type == Experiment::MULTI_VALUE);
+                        const std::set<std::string>& enabled_experiments) {
+  DCHECK_EQ(Experiment::MULTI_VALUE, experiment.type);
   ListValue* result = new ListValue;
   for (int i = 0; i < experiment.num_choices; ++i) {
     const Experiment::Choice& choice = experiment.choices[i];
@@ -434,10 +714,7 @@ Value* CreateChoiceData(const Experiment& experiment,
     value->SetString("description",
                      l10n_util::GetStringUTF16(choice.description_id));
     value->SetString("internal_name", name);
-    bool is_selected = enabled_experiments.count(name) > 0;
-    if (is_selected)
-      *is_one_selected = true;
-    value->SetBoolean("selected", is_selected);
+    value->SetBoolean("selected", enabled_experiments.count(name) > 0);
     result->Append(value);
   }
   return result;
@@ -458,8 +735,6 @@ ListValue* GetFlagsExperimentsData(PrefService* prefs) {
   ListValue* experiments_data = new ListValue();
   for (size_t i = 0; i < num_experiments; ++i) {
     const Experiment& experiment = experiments[i];
-    if (!(experiment.supported_platforms & current_platform))
-      continue;
 
     DictionaryValue* data = new DictionaryValue();
     data->SetString("internal_name", experiment.internal_name);
@@ -468,15 +743,36 @@ ListValue* GetFlagsExperimentsData(PrefService* prefs) {
     data->SetString("description",
                     l10n_util::GetStringUTF16(
                         experiment.visible_description_id));
+    bool supported = !!(experiment.supported_platforms & current_platform);
+#if defined(USE_AURA) && defined(OS_CHROMEOS)
+    // Some Chrome OS devices currently require Aura compact window mode, so
+    // don't offer a choice of mode.
+    // TODO(jamescook): Remove after Aura supports normal mode on all devices,
+    // likely around M19.
+    if (experiment.visible_name_id == IDS_FLAGS_AURA_WINDOW_MODE_NAME &&
+        CommandLine::ForCurrentProcess()->
+            HasSwitch(ash::switches::kAuraForceCompactWindowMode))
+      supported = false;
+#endif
+    data->SetBoolean("supported", supported);
 
-    bool enabled = enabled_experiments.count(experiment.internal_name) > 0;
+    ListValue* supported_platforms = new ListValue();
+    AddOsStrings(experiment.supported_platforms, supported_platforms);
+    data->Set("supported_platforms", supported_platforms);
 
-    if (experiment.type == Experiment::MULTI_VALUE) {
-      data->Set("choices", CreateChoiceData(experiment, enabled_experiments,
-                                            &enabled));
+    switch (experiment.type) {
+      case Experiment::SINGLE_VALUE:
+        data->SetBoolean(
+            "enabled",
+            enabled_experiments.count(experiment.internal_name) > 0);
+        break;
+      case Experiment::MULTI_VALUE:
+        data->Set("choices", CreateChoiceData(experiment, enabled_experiments));
+        break;
+      default:
+        NOTREACHED();
     }
 
-    data->SetBoolean("enabled", enabled);
     experiments_data->Append(data);
   }
   return experiments_data;
@@ -503,8 +799,10 @@ int GetCurrentPlatform() {
   return kOsWin;
 #elif defined(OS_CHROMEOS)  // Needs to be before the OS_LINUX check.
   return kOsCrOS;
-#elif defined(OS_LINUX)
+#elif defined(OS_LINUX) || defined(OS_OPENBSD)
   return kOsLinux;
+#elif defined(OS_ANDROID)
+  return kOsAndroid;
 #else
 #error Unknown platform
 #endif
@@ -517,13 +815,13 @@ void RecordUMAStatistics(const PrefService* prefs) {
        ++it) {
     std::string action("AboutFlags_");
     action += *it;
-    UserMetrics::RecordComputedAction(action);
+    content::RecordComputedAction(action);
   }
   // Since flag metrics are recorded every startup, add a tick so that the
   // stats can be made meaningful.
   if (flags.size())
-    UserMetrics::RecordAction(UserMetricsAction("AboutFlags_StartupTick"));
-  UserMetrics::RecordAction(UserMetricsAction("StartupTick"));
+    content::RecordAction(UserMetricsAction("AboutFlags_StartupTick"));
+  content::RecordAction(UserMetricsAction("StartupTick"));
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -540,36 +838,50 @@ void FlagsState::ConvertFlagsToSwitches(
 
   GetSanitizedEnabledFlagsForCurrentPlatform(prefs, &enabled_experiments);
 
-  typedef std::map<std::string, std::string> NameToSwitchMap;
-  NameToSwitchMap name_to_switch_map;
+  typedef std::map<std::string, std::pair<std::string, std::string> >
+      NameToSwitchAndValueMap;
+  NameToSwitchAndValueMap name_to_switch_map;
   for (size_t i = 0; i < num_experiments; ++i) {
     const Experiment& e = experiments[i];
     if (e.type == Experiment::SINGLE_VALUE) {
-      name_to_switch_map[e.internal_name] = e.command_line;
+      name_to_switch_map[e.internal_name] =
+          std::pair<std::string, std::string>(e.command_line_switch,
+                                              e.command_line_value);
     } else {
       for (int j = 0; j < e.num_choices; ++j)
-        name_to_switch_map[NameForChoice(e, j)] = e.choices[j].command_line;
+        name_to_switch_map[NameForChoice(e, j)] =
+            std::pair<std::string, std::string>(
+                e.choices[j].command_line_switch,
+                e.choices[j].command_line_value);
     }
   }
 
   command_line->AppendSwitch(switches::kFlagSwitchesBegin);
-  flags_switches_.insert(switches::kFlagSwitchesBegin);
+  flags_switches_.insert(
+      std::pair<std::string, std::string>(switches::kFlagSwitchesBegin,
+                                          std::string()));
   for (std::set<std::string>::iterator it = enabled_experiments.begin();
        it != enabled_experiments.end();
        ++it) {
     const std::string& experiment_name = *it;
-    NameToSwitchMap::const_iterator name_to_switch_it =
+    NameToSwitchAndValueMap::const_iterator name_to_switch_it =
         name_to_switch_map.find(experiment_name);
     if (name_to_switch_it == name_to_switch_map.end()) {
       NOTREACHED();
       continue;
     }
 
-    command_line->AppendSwitch(name_to_switch_it->second);
-    flags_switches_.insert(name_to_switch_it->second);
+    const std::pair<std::string, std::string>&
+        switch_and_value_pair = name_to_switch_it->second;
+
+    command_line->AppendSwitchASCII(switch_and_value_pair.first,
+                                    switch_and_value_pair.second);
+    flags_switches_[switch_and_value_pair.first] = switch_and_value_pair.second;
   }
   command_line->AppendSwitch(switches::kFlagSwitchesEnd);
-  flags_switches_.insert(switches::kFlagSwitchesEnd);
+  flags_switches_.insert(
+      std::pair<std::string, std::string>(switches::kFlagSwitchesEnd,
+                                          std::string()));
 }
 
 bool FlagsState::IsRestartNeededToCommitChanges() {
@@ -586,13 +898,16 @@ void FlagsState::SetExperimentEnabled(
     // We're being asked to enable a multi-choice experiment. Disable the
     // currently selected choice.
     DCHECK_NE(at_index, 0u);
-    SetExperimentEnabled(prefs, internal_name.substr(0, at_index), false);
+    const std::string experiment_name = internal_name.substr(0, at_index);
+    SetExperimentEnabled(prefs, experiment_name, false);
 
-    // And enable the new choice.
-    std::set<std::string> enabled_experiments;
-    GetSanitizedEnabledFlags(prefs, &enabled_experiments);
-    enabled_experiments.insert(internal_name);
-    SetEnabledFlags(prefs, enabled_experiments);
+    // And enable the new choice, if it is not the default first choice.
+    if (internal_name != experiment_name + "@0") {
+      std::set<std::string> enabled_experiments;
+      GetSanitizedEnabledFlags(prefs, &enabled_experiments);
+      enabled_experiments.insert(internal_name);
+      SetEnabledFlags(prefs, enabled_experiments);
+    }
     return;
   }
 
@@ -636,10 +951,9 @@ void FlagsState::SetExperimentEnabled(
 
 void FlagsState::RemoveFlagsSwitches(
     std::map<std::string, CommandLine::StringType>* switch_list) {
-  for (std::set<std::string>::const_iterator it = flags_switches_.begin();
-       it != flags_switches_.end();
-       ++it) {
-    switch_list->erase(*it);
+  for (std::map<std::string, std::string>::const_iterator
+           it = flags_switches_.begin(); it != flags_switches_.end(); ++it) {
+    switch_list->erase(it->first);
   }
 }
 
@@ -648,7 +962,7 @@ void FlagsState::reset() {
   flags_switches_.clear();
 }
 
-} // namespace
+}  // namespace
 
 namespace testing {
 
