@@ -1,40 +1,38 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <oleacc.h>
 
-#include "base/scoped_comptr_win.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_window.h"
+#include "base/win/scoped_comptr.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/view_ids.h"
-#include "chrome/browser/ui/views/bookmark_bar_view.h"
+#include "chrome/browser/ui/views/bookmarks/bookmark_bar_view.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
 #include "chrome/browser/ui/views/toolbar_view.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
+#include "ui/base/accessibility/accessibility_types.h"
 #include "ui/base/l10n/l10n_util.h"
-#include "views/accessibility/view_accessibility.h"
-#include "views/widget/root_view.h"
-#include "views/widget/widget_win.h"
-#include "views/window/window.h"
+#include "ui/base/win/atl_module.h"
+#include "ui/views/accessibility/native_view_accessibility_win.h"
+#include "ui/views/widget/widget.h"
 
 namespace {
 
 VARIANT id_self = {VT_I4, CHILDID_SELF};
 
-// Dummy class to force creation of ATL module, needed by COM to instantiate
-// ViewAccessibility.
-class TestAtlModule : public CAtlDllModuleT< TestAtlModule > {};
-TestAtlModule test_atl_module_;
+}  // namespace
 
 class BrowserViewsAccessibilityTest : public InProcessBrowserTest {
  public:
   BrowserViewsAccessibilityTest() {
+    ui::win::CreateATLModuleIfNeeded();
     ::CoInitialize(NULL);
   }
 
@@ -54,8 +52,7 @@ class BrowserViewsAccessibilityTest : public InProcessBrowserTest {
 
   // Retrieve an instance of BrowserView
   BrowserView* GetBrowserView() {
-    return BrowserView::GetBrowserViewForNativeWindow(
-               browser()->window()->GetNativeHandle());
+    return BrowserView::GetBrowserViewForBrowser(browser());
   }
 
   // Retrieves and initializes an instance of ToolbarView.
@@ -83,8 +80,7 @@ class BrowserViewsAccessibilityTest : public InProcessBrowserTest {
                                    int32 role) {
     ASSERT_TRUE(NULL != view);
 
-    TestAccessibilityInfo(
-        ViewAccessibility::GetAccessibleForView(view), name, role);
+    TestAccessibilityInfo(view->GetNativeViewAccessible(), name, role);
   }
 
 
@@ -122,7 +118,7 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
   ASSERT_TRUE(NULL != hwnd);
 
   // Get accessibility object.
-  ScopedComPtr<IAccessible> acc_obj;
+  base::win::ScopedComPtr<IAccessible> acc_obj;
   HRESULT hr = ::AccessibleObjectFromWindow(hwnd, OBJID_WINDOW, IID_IAccessible,
                                             reinterpret_cast<void**>(&acc_obj));
   ASSERT_EQ(S_OK, hr);
@@ -137,9 +133,11 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
 
 // Retrieve accessibility object for non client view and verify accessibility
 // info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestNonClientViewAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
+                       FLAKY_TestNonClientViewAccObj) {
   views::View* non_client_view =
-  GetBrowserView()->GetWindow()->GetNonClientView();
+  GetBrowserView()->GetWidget()->non_client_view();
 
   TestViewAccessibilityObject(non_client_view,
       UTF16ToWide(l10n_util::GetStringUTF16(IDS_PRODUCT_NAME)),
@@ -148,10 +146,10 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestNonClientViewAccObj) {
 
 // Retrieve accessibility object for browser root view and verify
 // accessibility info.
+// http://crbug.com/104132
 IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
-                       TestBrowserRootViewAccObj) {
-  views::View* browser_root_view =
-      GetBrowserView()->frame()->GetFrameView()->GetRootView();
+                       FLAKY_TestBrowserRootViewAccObj) {
+  views::View* browser_root_view = GetBrowserView()->frame()->GetRootView();
 
   TestViewAccessibilityObject(
       browser_root_view,
@@ -160,7 +158,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
 }
 
 // Retrieve accessibility object for browser view and verify accessibility info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestBrowserViewAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
+                       FLAKY_TestBrowserViewAccObj) {
   // Verify root view MSAA name and role.
   TestViewAccessibilityObject(
       GetBrowserView(),
@@ -169,7 +169,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestBrowserViewAccObj) {
 }
 
 // Retrieve accessibility object for toolbar view and verify accessibility info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestToolbarViewAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
+                       FLAKY_TestToolbarViewAccObj) {
   // Verify toolbar MSAA name and role.
   TestViewAccessibilityObject(
       GetToolbarView(),
@@ -178,7 +180,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestToolbarViewAccObj) {
 }
 
 // Retrieve accessibility object for Back button and verify accessibility info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestBackButtonAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
+                       FLAKY_TestBackButtonAccObj) {
   // Verify Back button MSAA name and role.
   TestViewAccessibilityObject(
       GetToolbarView()->GetViewByID(VIEW_ID_BACK_BUTTON),
@@ -188,7 +192,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestBackButtonAccObj) {
 
 // Retrieve accessibility object for Forward button and verify accessibility
 // info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestForwardButtonAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
+                       FLAKY_TestForwardButtonAccObj) {
   // Verify Forward button MSAA name and role.
   TestViewAccessibilityObject(
       GetToolbarView()->GetViewByID(VIEW_ID_FORWARD_BUTTON),
@@ -198,7 +204,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestForwardButtonAccObj) {
 
 // Retrieve accessibility object for Reload button and verify accessibility
 // info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestReloadButtonAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
+                       FLAKY_TestReloadButtonAccObj) {
   // Verify Reload button MSAA name and role.
   TestViewAccessibilityObject(
       GetToolbarView()->GetViewByID(VIEW_ID_RELOAD_BUTTON),
@@ -207,7 +215,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestReloadButtonAccObj) {
 }
 
 // Retrieve accessibility object for Home button and verify accessibility info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestHomeButtonAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
+                       FLAKY_TestHomeButtonAccObj) {
   // Verify Home button MSAA name and role.
   TestViewAccessibilityObject(
       GetToolbarView()->GetViewByID(VIEW_ID_HOME_BUTTON),
@@ -216,8 +226,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestHomeButtonAccObj) {
 }
 
 // Retrieve accessibility object for Star button and verify accessibility info.
+// http://crbug.com/104132
 IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
-                       TestStarButtonAccObj) {
+                       FLAKY_TestStarButtonAccObj) {
   // Verify Star button MSAA name and role.
   TestViewAccessibilityObject(
       GetToolbarView()->GetViewByID(VIEW_ID_STAR_BUTTON),
@@ -227,7 +238,8 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
 
 // Retrieve accessibility object for App menu button and verify accessibility
 // info.
-IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestAppMenuAccObj) {
+// http://crbug.com/104132
+IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, FLAKY_TestAppMenuAccObj) {
   // Verify App menu button MSAA name and role.
   TestViewAccessibilityObject(
       GetToolbarView()->GetViewByID(VIEW_ID_APP_MENU),
@@ -235,8 +247,9 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest, TestAppMenuAccObj) {
       ROLE_SYSTEM_BUTTONMENU);
 }
 
+// http://crbug.com/104132
 IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
-                       TestBookmarkBarViewAccObj) {
+                       FLAKY_TestBookmarkBarViewAccObj) {
   TestViewAccessibilityObject(
       GetBookmarkBarView(),
       UTF16ToWide(l10n_util::GetStringUTF16(IDS_ACCNAME_BOOKMARKS)),
@@ -246,17 +259,18 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
 IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
                        TestAboutChromeViewAccObj) {
   //  Firstly, test that the WindowDelegate got updated.
-  views::Window* aboutChromeWindow = GetBrowserView()->ShowAboutChromeDialog();
+  views::Widget* about_chrome_window =
+      GetBrowserView()->DoShowAboutChromeDialog();
   EXPECT_STREQ(
-      aboutChromeWindow->GetDelegate()->GetWindowTitle().c_str(),
+      about_chrome_window->widget_delegate()->GetWindowTitle().c_str(),
       UTF16ToWide(l10n_util::GetStringUTF16(IDS_ABOUT_CHROME_TITLE)).c_str());
-  EXPECT_EQ(aboutChromeWindow->GetDelegate()->accessible_role(),
-            AccessibilityTypes::ROLE_DIALOG);
+  EXPECT_EQ(about_chrome_window->widget_delegate()->GetAccessibleWindowRole(),
+            ui::AccessibilityTypes::ROLE_DIALOG);
 
   // Also test the accessibility object directly.
   IAccessible* acc_obj = NULL;
   HRESULT hr =
-    ::AccessibleObjectFromWindow(aboutChromeWindow->GetNativeWindow(),
+    ::AccessibleObjectFromWindow(about_chrome_window->GetNativeWindow(),
                                  OBJID_CLIENT,
                                  IID_IAccessible,
                                  reinterpret_cast<void**>(&acc_obj));
@@ -270,4 +284,3 @@ IN_PROC_BROWSER_TEST_F(BrowserViewsAccessibilityTest,
 
   acc_obj->Release();
 }
-}  // Namespace.
