@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,14 +11,19 @@
 #include <list>
 
 #include "chrome/browser/tabs/tab_strip_model_observer.h"
-#include "gfx/insets.h"
+#include "ui/gfx/insets.h"
+
+class AeroPeekWindow;
+class SkBitmap;
+class TabContentsWrapper;
+
+namespace content {
+class WebContents;
+}
 
 namespace gfx {
 class Size;
 }
-class AeroPeekWindow;
-class SkBitmap;
-class TabContents;
 
 // A class which defines interfaces called from AeroPeekWindow.
 // This class is used for dispatching an event received by a thumbnail window
@@ -110,14 +115,11 @@ class AeroPeekManager : public TabStripModelObserver,
   virtual void TabInsertedAt(TabContentsWrapper* contents,
                              int index,
                              bool foreground);
-  virtual void TabClosingAt(TabStripModel* tab_strip_model,
-                            TabContentsWrapper* contents,
-                            int index);
   virtual void TabDetachedAt(TabContentsWrapper* contents, int index);
-  virtual void TabSelectedAt(TabContentsWrapper* old_contents,
-                             TabContentsWrapper* new_contents,
-                             int index,
-                             bool user_gesture);
+  virtual void ActiveTabChanged(TabContentsWrapper* old_contents,
+                                TabContentsWrapper* new_contents,
+                                int index,
+                                bool user_gesture);
   virtual void TabMoved(TabContentsWrapper* contents,
                         int from_index,
                         int to_index,
@@ -125,6 +127,10 @@ class AeroPeekManager : public TabStripModelObserver,
   virtual void TabChangedAt(TabContentsWrapper* contents,
                             int index,
                             TabChangeType change_type);
+  virtual void TabReplacedAt(TabStripModel* tab_strip_model,
+                             TabContentsWrapper* old_contents,
+                             TabContentsWrapper* new_contents,
+                             int index);
 
   // Overriden from TabThumbnailWindowDelegate:
   virtual void CloseTab(int tab_id);
@@ -138,10 +144,19 @@ class AeroPeekManager : public TabStripModelObserver,
   // Tab ID.
   void DeleteAeroPeekWindow(int tab_id);
 
+  // If there is an AeroPeekWindow associated with |tab| it is removed and
+  // deleted.
+  void DeleteAeroPeekWindowForTab(TabContentsWrapper* tab);
+
   // Retrieves the AeroPeekWindow object associated with the specified
   // Tab ID.
   AeroPeekWindow* GetAeroPeekWindow(int tab_id) const;
 
+  // If an AeroPeekWindow hasn't been created for |tab| yet, one is created.
+  // |foreground| is true if the tab is selected.
+  void CreateAeroPeekWindowIfNecessary(TabContentsWrapper* tab,
+                                       bool foreground);
+
   // Returns a rectangle that fits into the destination rectangle and keeps
   // the pixel-aspect ratio of the source one.
   // (This function currently uses the longer-fit algorithm as IE8 does.)
@@ -149,17 +164,17 @@ class AeroPeekManager : public TabStripModelObserver,
                            const gfx::Size& source,
                            gfx::Size* output) const;
 
-  // Returns the TabContents object associated with the specified Tab ID only
+  // Returns the WebContents object associated with the specified Tab ID only
   // if it is alive.
   // Since Windows cannot send AeroPeek events directly to Chrome windows, we
   // use a place-holder window to receive AeroPeek events. So, when Windows
   // sends an AeroPeek event, the corresponding tab (and TabContents) may have
   // been deleted by Chrome. To prevent us from accessing deleted TabContents,
   // we need to check if the tab is still alive.
-  TabContents* GetTabContents(int tab_id) const;
+  content::WebContents* GetWebContents(int tab_id) const;
 
-  // Returns the tab ID from the specified TabContents.
-  int GetTabID(TabContents* contents) const;
+  // Returns the tab ID from the specified TabContentsWrapper.
+  int GetTabID(TabContentsWrapper* contents) const;
 
  private:
   // The parent window of the place-holder windows used by AeroPeek.
