@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,16 +10,16 @@
 #include <vector>
 
 #import "base/mac/cocoa_protocols.h"
-#include "base/scoped_nsobject.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_nsobject.h"
+#include "base/memory/scoped_ptr.h"
 
 @class AboutWindowController;
 class BookmarkMenuBridge;
 class CommandUpdater;
 class GURL;
 class HistoryMenuBridge;
-@class PreferencesWindowController;
 class Profile;
+@class ProfileMenuController;
 
 // The application controller object, created by loading the MainMenu nib.
 // This handles things like responding to menus when there are no windows
@@ -27,14 +27,23 @@ class Profile;
 @interface AppController : NSObject<NSUserInterfaceValidations,
                                     NSApplicationDelegate> {
  @private
+  // Manages the state of the command menu items.
   scoped_ptr<CommandUpdater> menuState_;
+
+  // The profile last used by a Browser. It is this profile that was used to
+  // build the user-data specific main menu items.
+  Profile* lastProfile_;
+
   // Management of the bookmark menu which spans across all windows
   // (and Browser*s).
   scoped_ptr<BookmarkMenuBridge> bookmarkMenuBridge_;
   scoped_ptr<HistoryMenuBridge> historyMenuBridge_;
-  PreferencesWindowController* prefsController_;  // Weak.
   AboutWindowController* aboutController_;  // Weak.
 
+  // The profile menu, which appears right before the Help menu. It is only
+  // available when multiple profiles is enabled.
+  scoped_nsobject<ProfileMenuController> profileMenuController_;
+
   // If we're told to open URLs (in particular, via |-application:openFiles:| by
   // Launch Services) before we've launched the browser, we queue them up in
   // |startupUrls_| so that they can go in the first browser window/tab.
@@ -54,12 +63,15 @@ class Profile;
 
   // Outlet for the tabpose menu item so we can hide it.
   IBOutlet NSMenuItem* tabposeMenuItem_;
+
+  // Indicates wheter an NSPopover is currently being shown.
+  BOOL hasPopover_;
 }
 
 @property(readonly, nonatomic) BOOL startupComplete;
+@property(readonly, nonatomic) Profile* lastProfile;
 
 - (void)didEndMainMessageLoop;
-- (Profile*)defaultProfile;
 
 // Try to close all browser windows, and if that succeeds then quit.
 - (BOOL)tryToTerminateApplication:(NSApplication*)app;
@@ -82,6 +94,9 @@ class Profile;
 // Owner" (NSAppliation) for a Branded About Box
 - (IBAction)orderFrontStandardAboutPanel:(id)sender;
 
+// Toggles the "Confirm to Quit" preference.
+- (IBAction)toggleConfirmToQuit:(id)sender;
+
 // Delegate method to return the dock menu.
 - (NSMenu*)applicationDockMenu:(NSApplication*)sender;
 
@@ -91,6 +106,8 @@ class Profile;
 // Clear the list of startup URLs.
 - (void)clearStartupUrls;
 
+- (BookmarkMenuBridge*)bookmarkMenuBridge;
+
 @end
 
 #endif
