@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,12 +6,16 @@
 #define CHROME_BROWSER_AUTOCOMPLETE_AUTOCOMPLETE_H_
 #pragma once
 
+#include <map>
 #include <string>
 #include <vector>
 
 #include "base/logging.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/string16.h"
+#include "base/time.h"
 #include "base/timer.h"
+#include "chrome/browser/sessions/session_id.h"
 #include "googleurl/src/gurl.h"
 #include "googleurl/src/url_parse.h"
 
@@ -41,17 +45,21 @@
 // UNKNOWN input type:
 // --------------------------------------------------------------------|-----
 // Keyword (non-substituting or in keyword UI mode, exact match)       | 1500
-// HistoryURL (exact or inline autocomplete match)                     | 1400
+// Extension App (exact match)                                         | 1425
+// HistoryURL (good exact or inline autocomplete matches, some inexact)| 1410++
+// HistoryURL (intranet url never visited match, some inexact matches) | 1400++
 // Search Primary Provider (past query in history within 2 days)       | 1399**
 // Search Primary Provider (what you typed)                            | 1300
-// HistoryURL (what you typed)                                         | 1200
+// HistoryURL (what you typed, some inexact matches)                   | 1200++
+// Extension App (inexact match)                                       | 1175*~
 // Keyword (substituting, exact match)                                 | 1100
 // Search Primary Provider (past query in history older than 2 days)   | 1050--
 // HistoryContents (any match in title of starred page)                | 1000++
-// HistoryURL (inexact match)                                          |  900++
+// HistoryURL (some inexact matches)                                   |  900++
 // Search Primary Provider (navigational suggestion)                   |  800++
 // HistoryContents (any match in title of nonstarred page)             |  700++
 // Search Primary Provider (suggestion)                                |  600++
+// Built-in                                                            |  575++
 // HistoryContents (any match in body of starred page)                 |  550++
 // HistoryContents (any match in body of nonstarred page)              |  500++
 // Keyword (inexact match)                                             |  450
@@ -63,17 +71,21 @@
 // REQUESTED_URL input type:
 // --------------------------------------------------------------------|-----
 // Keyword (non-substituting or in keyword UI mode, exact match)       | 1500
-// HistoryURL (exact or inline autocomplete match)                     | 1400
+// Extension App (exact match)                                         | 1425
+// HistoryURL (good exact or inline autocomplete matches, some inexact)| 1410++
+// HistoryURL (intranet url never visited match, some inexact matches) | 1400++
 // Search Primary Provider (past query in history within 2 days)       | 1399**
-// HistoryURL (what you typed)                                         | 1200
+// HistoryURL (what you typed, some inexact matches)                   | 1200++
+// Extension App (inexact match)                                       | 1175*~
 // Search Primary Provider (what you typed)                            | 1150
 // Keyword (substituting, exact match)                                 | 1100
 // Search Primary Provider (past query in history older than 2 days)   | 1050--
 // HistoryContents (any match in title of starred page)                | 1000++
-// HistoryURL (inexact match)                                          |  900++
+// HistoryURL (some inexact matches)                                   |  900++
 // Search Primary Provider (navigational suggestion)                   |  800++
 // HistoryContents (any match in title of nonstarred page)             |  700++
 // Search Primary Provider (suggestion)                                |  600++
+// Built-in                                                            |  575++
 // HistoryContents (any match in body of starred page)                 |  550++
 // HistoryContents (any match in body of nonstarred page)              |  500++
 // Keyword (inexact match)                                             |  450
@@ -85,14 +97,18 @@
 // URL input type:
 // --------------------------------------------------------------------|-----
 // Keyword (non-substituting or in keyword UI mode, exact match)       | 1500
-// HistoryURL (exact or inline autocomplete match)                     | 1400
-// HistoryURL (what you typed)                                         | 1200
+// Extension App (exact match)                                         | 1425
+// HistoryURL (good exact or inline autocomplete matches, some inexact)| 1410++
+// HistoryURL (intranet url never visited match, some inexact matches) | 1400++
+// HistoryURL (what you typed, some inexact matches)                   | 1200++
+// Extension App (inexact match)                                       | 1175*~
 // Keyword (substituting, exact match)                                 | 1100
-// HistoryURL (inexact match)                                          |  900++
+// HistoryURL (some inexact matches)                                   |  900++
 // Search Primary Provider (what you typed)                            |  850
 // Search Primary Provider (navigational suggestion)                   |  800++
 // Search Primary Provider (past query in history)                     |  750--
 // Keyword (inexact match)                                             |  700
+// Built-in                                                            |  575++
 // Search Primary Provider (suggestion)                                |  300++
 // Search Secondary Provider (what you typed)                          |  250
 // Search Secondary Provider (past query in history)                   |  200--
@@ -101,11 +117,13 @@
 //
 // QUERY input type:
 // --------------------------------------------------------------------|-----
+// Search Primary or Secondary (past query in history within 2 days)   | 1599**
 // Keyword (non-substituting or in keyword UI mode, exact match)       | 1500
 // Keyword (substituting, exact match)                                 | 1450
-// HistoryURL (exact or inline autocomplete match)                     | 1400
+// Extension App (exact match)                                         | 1425
 // Search Primary Provider (past query in history within 2 days)       | 1399**
 // Search Primary Provider (what you typed)                            | 1300
+// Extension App (inexact match)                                       | 1175*~
 // Search Primary Provider (past query in history older than 2 days)   | 1050--
 // HistoryContents (any match in title of starred page)                | 1000++
 // HistoryURL (inexact match)                                          |  900++
@@ -122,8 +140,10 @@
 //
 // FORCED_QUERY input type:
 // --------------------------------------------------------------------|-----
+// Extension App (exact match on title only, not url)                  | 1425
 // Search Primary Provider (past query in history within 2 days)       | 1399**
 // Search Primary Provider (what you typed)                            | 1300
+// Extension App (inexact match on title only, not url)                | 1175*~
 // Search Primary Provider (past query in history older than 2 days)   | 1050--
 // HistoryContents (any match in title of starred page)                | 1000++
 // Search Primary Provider (navigational suggestion)                   |  800++
@@ -144,15 +164,19 @@
 // ++: a series of matches with relevance from n up to (n + max_matches).
 // --: relevance score falls off over time (discounted 50 points @ 15 minutes,
 //     450 points @ two weeks)
-// --: relevance score falls off over two days (discounted 99 points after two
+// **: relevance score falls off over two days (discounted 99 points after two
 //     days).
+// *~: Partial matches get a score on a sliding scale from about 575-1125 based
+//     on how many times the URL for the Extension App has been typed and how
+//     many of the letters match.
 
+class AutocompleteController;
+class AutocompleteControllerDelegate;
 class AutocompleteInput;
 struct AutocompleteMatch;
 class AutocompleteProvider;
 class AutocompleteResult;
-class AutocompleteController;
-class HistoryContentsProvider;
+class KeywordProvider;
 class Profile;
 class SearchProvider;
 class TemplateURL;
@@ -179,17 +203,34 @@ class AutocompleteInput {
     FORCED_QUERY,   // Input forced to be a query by an initial '?'
   };
 
+  // Enumeration of the possible match query types. Callers who only need some
+  // of the matches for a particular input can get answers more quickly by
+  // specifying that upfront.
+  enum MatchesRequested {
+    // Only the best match in the whole result set matters.  Providers should at
+    // most return synchronously-available matches, and if possible do even less
+    // work, so that it's safe to ask for these repeatedly in the course of one
+    // higher-level "synchronous" query.
+    BEST_MATCH,
+
+    // Only synchronous matches should be returned.
+    SYNCHRONOUS_MATCHES,
+
+    // All matches should be fetched.
+    ALL_MATCHES,
+  };
+
   AutocompleteInput();
-  AutocompleteInput(const std::wstring& text,
-                    const std::wstring& desired_tld,
+  AutocompleteInput(const string16& text,
+                    const string16& desired_tld,
                     bool prevent_inline_autocomplete,
                     bool prefer_keyword,
                     bool allow_exact_keyword_match,
-                    bool synchronous_only);
+                    MatchesRequested matches_requested);
   ~AutocompleteInput();
 
   // If type is |FORCED_QUERY| and |text| starts with '?', it is removed.
-  static void RemoveForcedQueryStringIfNecessary(Type type, std::wstring* text);
+  static void RemoveForcedQueryStringIfNecessary(Type type, string16* text);
 
   // Converts |type| to a string representation.  Used in logging.
   static std::string TypeToString(Type type);
@@ -199,18 +240,18 @@ class AutocompleteInput {
   // it is non-NULL. The scheme is stored in |scheme| if it is non-NULL. The
   // canonicalized URL is stored in |canonicalized_url|; however, this URL is
   // not guaranteed to be valid, especially if the parsed type is, e.g., QUERY.
-  static Type Parse(const std::wstring& text,
-                    const std::wstring& desired_tld,
+  static Type Parse(const string16& text,
+                    const string16& desired_tld,
                     url_parse::Parsed* parts,
-                    std::wstring* scheme,
+                    string16* scheme,
                     GURL* canonicalized_url);
 
   // Parses |text| and fill |scheme| and |host| by the positions of them.
   // The results are almost as same as the result of Parse(), but if the scheme
   // is view-source, this function returns the positions of scheme and host
   // in the URL qualified by "view-source:" prefix.
-  static void ParseForEmphasizeComponents(const std::wstring& text,
-                                          const std::wstring& desired_tld,
+  static void ParseForEmphasizeComponents(const string16& text,
+                                          const string16& desired_tld,
                                           url_parse::Component* scheme,
                                           url_parse::Component* host);
 
@@ -220,23 +261,26 @@ class AutocompleteInput {
   // function with the URL and its formatted string, and it will return a
   // formatted string with the same meaning as the original URL (i.e. it will
   // re-append a slash if necessary).
-  static std::wstring FormattedStringWithEquivalentMeaning(
+  static string16 FormattedStringWithEquivalentMeaning(
       const GURL& url,
-      const std::wstring& formatted_url);
+      const string16& formatted_url);
+
+  // Returns the number of non-empty components in |parts| besides the host.
+  static int NumNonHostComponents(const url_parse::Parsed& parts);
 
   // User-provided text to be completed.
-  const std::wstring& text() const { return text_; }
+  const string16& text() const { return text_; }
 
   // Use of this setter is risky, since no other internal state is updated
-  // besides |text_|.  Only callers who know that they're not changing the
-  // type/scheme/etc. should use this.
-  void set_text(const std::wstring& text) { text_ = text; }
+  // besides |text_| and |parts_|.  Only callers who know that they're not
+  // changing the type/scheme/etc. should use this.
+  void UpdateText(const string16& text, const url_parse::Parsed& parts);
 
   // User's desired TLD, if one is not already present in the text to
   // autocomplete.  When this is non-empty, it also implies that "www." should
   // be prepended to the domain where possible.  This should not have a leading
   // '.' (use "com" instead of ".com").
-  const std::wstring& desired_tld() const { return desired_tld_; }
+  const string16& desired_tld() const { return desired_tld_; }
 
   // The type of input supplied.
   Type type() const { return type_; }
@@ -246,7 +290,7 @@ class AutocompleteInput {
 
   // The scheme parsed from the provided text; only meaningful when type_ is
   // URL.
-  const std::wstring& scheme() const { return scheme_; }
+  const string16& scheme() const { return scheme_; }
 
   // The input as an URL to navigate to, if possible.
   const GURL& canonicalized_url() const { return canonicalized_url_; }
@@ -256,13 +300,6 @@ class AutocompleteInput {
     return prevent_inline_autocomplete_;
   }
 
-  // Returns the value of |prevent_inline_autocomplete| supplied to the
-  // constructor. This differs from the value returned by
-  // |prevent_inline_autocomplete()| if the input contained trailing whitespace.
-  bool initial_prevent_inline_autocomplete() const {
-    return initial_prevent_inline_autocomplete_;
-  }
-
   // Returns whether, given an input string consisting solely of a substituting
   // keyword, we should score it like a non-substituting keyword.
   bool prefer_keyword() const { return prefer_keyword_; }
@@ -272,11 +309,8 @@ class AutocompleteInput {
   // keyword search, even if the input is "<keyword> <search string>".
   bool allow_exact_keyword_match() const { return allow_exact_keyword_match_; }
 
-  // Returns whether providers should avoid scheduling asynchronous work.  If
-  // this is true, providers should stop after returning all the
-  // synchronously-available matches.  This also means any in-progress
-  // asynchronous work should be canceled, so no later callbacks are fired.
-  bool synchronous_only() const { return synchronous_only_; }
+  // See description of enum for details.
+  MatchesRequested matches_requested() const { return matches_requested_; }
 
   // operator==() by another name.
   bool Equals(const AutocompleteInput& other) const;
@@ -285,17 +319,16 @@ class AutocompleteInput {
   void Clear();
 
  private:
-  std::wstring text_;
-  std::wstring desired_tld_;
+  string16 text_;
+  string16 desired_tld_;
   Type type_;
   url_parse::Parsed parts_;
-  std::wstring scheme_;
+  string16 scheme_;
   GURL canonicalized_url_;
-  bool initial_prevent_inline_autocomplete_;
   bool prevent_inline_autocomplete_;
   bool prefer_keyword_;
   bool allow_exact_keyword_match_;
-  bool synchronous_only_;
+  MatchesRequested matches_requested_;
 };
 
 // AutocompleteProvider -------------------------------------------------------
@@ -332,11 +365,6 @@ class AutocompleteProvider
                        Profile* profile,
                        const char* name);
 
-  // Invoked when the profile changes.
-  // NOTE: Do not access any previous Profile* at this point as it may have
-  // already been deleted.
-  void SetProfile(Profile* profile);
-
   // Called to start an autocomplete query.  The provider is responsible for
   // tracking its matches for this query and whether it is done processing the
   // query.  When new matches are available or the provider finishes, it
@@ -377,6 +405,9 @@ class AutocompleteProvider
   // NOTE: Remember to call OnProviderUpdate() if matches_ is updated.
   virtual void DeleteMatch(const AutocompleteMatch& match);
 
+#ifdef UNIT_TEST
+  void set_listener(ACProviderListener* listener) { listener_ = listener; }
+#endif
   // A suggested upper bound for how many matches a provider should return.
   // TODO(pkasting): http://b/1111299 , http://b/933133 This should go away once
   // we have good relevance heuristics; the controller should handle all
@@ -389,7 +420,7 @@ class AutocompleteProvider
   virtual ~AutocompleteProvider();
 
   // Returns whether |input| begins "http:" or "view-source:http:".
-  static bool HasHTTPScheme(const std::wstring& input);
+  static bool HasHTTPScheme(const string16& input);
 
   // Updates the starred state of each of the matches in matches_ from the
   // profile's bookmark bar model.
@@ -398,9 +429,9 @@ class AutocompleteProvider
   // A convenience function to call net::FormatUrl() with the current set of
   // "Accept Languages" when check_accept_lang is true.  Otherwise, it's called
   // with an empty list.
-  std::wstring StringForURLDisplay(const GURL& url,
-                                   bool check_accept_lang,
-                                   bool trim_http) const;
+  string16 StringForURLDisplay(const GURL& url,
+                               bool check_accept_lang,
+                               bool trim_http) const;
 
   // The profile associated with the AutocompleteProvider.  Reference is not
   // owned by us.
@@ -460,12 +491,24 @@ class AutocompleteResult {
     bool is_history_what_you_typed_match;
   };
 
+  // Max number of matches we'll show from the various providers.
+  static const size_t kMaxMatches;
+
+  // The lowest score a match can have and still potentially become the default
+  // match for the result set.
+  static const int kLowestDefaultScore;
+
   AutocompleteResult();
   ~AutocompleteResult();
 
   // operator=() by another name.
   void CopyFrom(const AutocompleteResult& rhs);
 
+  // Copies matches from |old_matches| to provide a consistant result set. See
+  // comments in code for specifics.
+  void CopyOldMatches(const AutocompleteInput& input,
+                      const AutocompleteResult& old_matches);
+
   // Adds a single match. The match is inserted at the appropriate position
   // based on relevancy and display order. This is ONLY for use after
   // SortAndCull() has been invoked, and preserves default_match_.
@@ -479,6 +522,9 @@ class AutocompleteResult {
   // and updates the alternate nav URL.
   void SortAndCull(const AutocompleteInput& input);
 
+  // Returns true if at least one match was copied from the last result.
+  bool HasCopiedMatches() const;
+
   // Vector-style accessors/operators.
   size_t size() const;
   bool empty() const;
@@ -499,15 +545,37 @@ class AutocompleteResult {
   // Clears the matches for this result set.
   void Reset();
 
+  void Swap(AutocompleteResult* other);
+
 #ifndef NDEBUG
   // Does a data integrity check on this result.
   void Validate() const;
 #endif
 
-  // Max number of matches we'll show from the various providers.
-  static const size_t kMaxMatches;
-
  private:
+  typedef std::map<AutocompleteProvider*, ACMatches> ProviderToMatches;
+
+#if defined(OS_ANDROID)
+  // iterator::difference_type is not defined in the STL that we compile with on
+  // Android.
+  typedef int matches_difference_type;
+#else
+  typedef ACMatches::iterator::difference_type matches_difference_type;
+#endif
+
+  // Populates |provider_to_matches| from |matches_|.
+  void BuildProviderToMatches(ProviderToMatches* provider_to_matches) const;
+
+  // Returns true if |matches| contains a match with the same destination as
+  // |match|.
+  static bool HasMatchByDestination(const AutocompleteMatch& match,
+                                    const ACMatches& matches);
+
+  // Copies matches into this result. |old_matches| gives the matches from the
+  // last result, and |new_matches| the results from this result.
+  void MergeMatchesByProvider(const ACMatches& old_matches,
+                              const ACMatches& new_matches);
+
   ACMatches matches_;
 
   const_iterator default_match_;
@@ -537,23 +605,21 @@ class AutocompleteController : public ACProviderListener {
   // second to set the providers to some known testing providers.  The default
   // providers will be overridden and the controller will take ownership of the
   // providers, Release()ing them on destruction.
-  explicit AutocompleteController(Profile* profile);
+  AutocompleteController(Profile* profile,
+                         AutocompleteControllerDelegate* delegate);
 #ifdef UNIT_TEST
-  explicit AutocompleteController(const ACProviders& providers)
-      : providers_(providers),
+  AutocompleteController(const ACProviders& providers, Profile* profile)
+      : delegate_(NULL),
+        providers_(providers),
+        keyword_provider_(NULL),
         search_provider_(NULL),
-        updated_latest_result_(false),
-        delay_interval_has_passed_(false),
-        have_committed_during_this_query_(false),
-        done_(true) {
+        done_(true),
+        in_start_(false),
+        profile_(profile) {
   }
 #endif
   ~AutocompleteController();
 
-  // Invoked when the profile changes. This forwards the call down to all
-  // the AutocompleteProviders.
-  void SetProfile(Profile* profile);
-
   // Starts an autocomplete query, which continues until all providers are
   // done or the query is Stop()ed.  It is safe to Start() a new query without
   // Stop()ing the previous one.
@@ -575,23 +641,21 @@ class AutocompleteController : public ACProviderListener {
   // in a page and telling the browser to search for it or navigate to it. This
   // parameter only applies to substituting keywords.
 
-  // If |synchronous_only| is true, the controller asks the providers to only
-  // return matches which are synchronously available, which should mean that
-  // all providers will be done immediately.
+  // If |matches_requested| is BEST_MATCH or SYNCHRONOUS_MATCHES the controller
+  // asks the providers to only return matches which are synchronously
+  // available, which should mean that all providers will be done immediately.
   //
-  // The controller will fire
-  // AUTOCOMPLETE_CONTROLLER_SYNCHRONOUS_MATCHES_AVAILABLE from inside this
-  // call, and unless the query is stopped, will fire at least one (and perhaps
-  // more) AUTOCOMPLETE_CONTROLLER_RESULT_UPDATED later as more matches come in
-  // (even if the query completes synchronously).  Listeners should use the
-  // result set provided in the accompanying Details object to update
-  // themselves.
-  void Start(const std::wstring& text,
-             const std::wstring& desired_tld,
+  // The controller calls AutocompleteControllerDelegate::OnResultChanged() from
+  // inside this call at least once. If matches are available later on that
+  // result in changing the result set the delegate is notified again. When the
+  // controller is done the notification AUTOCOMPLETE_CONTROLLER_RESULT_READY is
+  // sent.
+  void Start(const string16& text,
+             const string16& desired_tld,
              bool prevent_inline_autocomplete,
              bool prefer_keyword,
              bool allow_exact_keyword_match,
-             bool synchronous_only);
+             AutocompleteInput::MatchesRequested matches_requested);
 
   // Cancels the current query, ensuring there will be no future notifications
   // fired.  If new matches have come in since the most recent notification was
@@ -605,47 +669,52 @@ class AutocompleteController : public ACProviderListener {
   // no query is running.
   void DeleteMatch(const AutocompleteMatch& match);
 
-  // Commits the results for the current query if they've never been committed.
-  // This is used by the popup to ensure it's not showing an out-of-date query.
-  void CommitIfQueryHasNeverBeenCommitted();
+  // Removes any entries that were copied from the last result. This is used by
+  // the popup to ensure it's not showing an out-of-date query.
+  void ExpireCopiedEntries();
 
+#ifdef UNIT_TEST
+  void set_search_provider(SearchProvider* provider) {
+    search_provider_ = provider;
+  }
+#endif
   SearchProvider* search_provider() const { return search_provider_; }
 
   // Getters
   const AutocompleteInput& input() const { return input_; }
   const AutocompleteResult& result() const { return result_; }
-  // This next is temporary and should go away when
-  // AutocompletePopup::InfoForCurrentSelection() moves to the controller.
-  const AutocompleteResult& latest_result() const { return latest_result_; }
-  bool done() const { return done_ && !update_delay_timer_.IsRunning(); }
+  bool done() const { return done_; }
 
   // From AutocompleteProvider::Listener
   virtual void OnProviderUpdate(bool updated_matches);
 
  private:
-  // Updates |latest_result_| and |done_| to reflect the current provider state.
-  // Resets timers and fires notifications as necessary.  |is_synchronous_pass|
-  // is true only when Start() is calling this to get the synchronous result.
-  void UpdateLatestResult(bool is_synchronous_pass);
+  // Updates |result_| to reflect the current provider state.  Resets timers and
+  // fires notifications as necessary.  |is_synchronous_pass| is true only when
+  // Start() is calling this to get the synchronous result.
+  void UpdateResult(bool is_synchronous_pass);
 
-  // Callback for when |max_delay_timer_| fires; this notes that the delay
-  // interval has passed and commits the result, if it's changed.
-  void DelayTimerFired();
+  // For each group of contiguous matches from the same TemplateURL, show the
+  // provider name as a description on the first match in the group.
+  void UpdateKeywordDescriptions(AutocompleteResult* result);
 
-  // Copies |latest_result_| to |result_| and notifies observers of updates.
-  // |notify_default_match| should normally be true; if it's false, we don't
-  // send an AUTOCOMPLETE_CONTROLLER_DEFAULT_MATCH_UPDATED notification.  This
-  // is a hack to avoid updating the edit with out-of-date data.
-  // TODO(pkasting): Don't hardcode assumptions about who listens to these
-  // notificiations.
-  void CommitResult(bool notify_default_match);
+  // Calls AutocompleteControllerDelegate::OnResultChanged() and if done sends
+  // AUTOCOMPLETE_CONTROLLER_RESULT_READY.
+  void NotifyChanged(bool notify_default_match);
 
   // Updates |done_| to be accurate with respect to current providers' statuses.
   void CheckIfDone();
 
+  // Starts the expire timer.
+  void StartExpireTimer();
+
+  AutocompleteControllerDelegate* delegate_;
+
   // A list of all providers.
   ACProviders providers_;
 
+  KeywordProvider* keyword_provider_;
+
   SearchProvider* search_provider_;
 
   // Input passed to Start.
@@ -654,43 +723,18 @@ class AutocompleteController : public ACProviderListener {
   // Data from the autocomplete query.
   AutocompleteResult result_;
 
-  // The latest result available from the autocomplete providers.  This may be
-  // different than |result_| if we've gotten matches from our providers that we
-  // haven't yet shown the user.  If there aren't yet as many matches as in
-  // |result|, we'll wait to display these in hopes of minimizing flicker in GUI
-  // observers.
-  AutocompleteResult latest_result_;
-
-  // True if |latest_result_| has been updated since it was last committed to
-  // |result_|.  Used to determine whether we need to commit any changes.
-  bool updated_latest_result_;
-
-  // True when it's been at least one interval of the delay timer since we
-  // committed any updates.  This is used to allow a new update to be committed
-  // immediately.
-  //
-  // NOTE: This can never be true when |have_committed_during_this_query_| is
-  // false (except transiently while processing the timer firing).
-  bool delay_interval_has_passed_;
-
-  // True when we've committed a result set at least once during this query.
-  // When this is false, we commit immediately when |done_| is set, since there
-  // are no more updates to come and thus no possible flicker due to committing
-  // immediately.
-  //
-  // NOTE: This can never be false when |delay_interval_has_passed_| is true
-  // (except transiently while processing the timer firing).
-  bool have_committed_during_this_query_;
+  // Timer used to remove any matches copied from the last result. When run
+  // invokes |ExpireCopiedEntries|.
+  base::OneShotTimer<AutocompleteController> expire_timer_;
 
   // True if a query is not currently running.
   bool done_;
 
-  // Timer that tracks how long it's been since the last time we sent our
-  // observers a new result set.  This is used both to enforce a lower bound on
-  // the delay between most commits (to reduce flicker), and ensure that updates
-  // eventually get committed no matter what delays occur between them or how
-  // fast or continuously the user is typing.
-  base::RepeatingTimer<AutocompleteController> update_delay_timer_;
+  // Are we in Start()? This is used to avoid updating |result_| and sending
+  // notifications until Start() has been invoked on all providers.
+  bool in_start_;
+
+  Profile* profile_;
 
   DISALLOW_COPY_AND_ASSIGN(AutocompleteController);
 };
@@ -700,23 +744,32 @@ class AutocompleteController : public ACProviderListener {
 // The data to log (via the metrics service) when the user selects an item
 // from the omnibox popup.
 struct AutocompleteLog {
-  AutocompleteLog(std::wstring text,
-                  AutocompleteInput::Type input_type,
-                  size_t selected_index,
-                  size_t inline_autocompleted_length,
-                  const AutocompleteResult& result)
-      : text(text),
-        input_type(input_type),
-        selected_index(selected_index),
-        inline_autocompleted_length(inline_autocompleted_length),
-        result(result) {
-  }
+  AutocompleteLog(
+      const string16& text,
+      AutocompleteInput::Type input_type,
+      size_t selected_index,
+      SessionID::id_type tab_id,
+      base::TimeDelta elapsed_time_since_user_first_modified_omnibox,
+      size_t inline_autocompleted_length,
+      const AutocompleteResult& result);
   // The user's input text in the omnibox.
-  std::wstring text;
+  string16 text;
   // The detected type of the user's input.
   AutocompleteInput::Type input_type;
   // Selected index (if selected) or -1 (AutocompletePopupModel::kNoMatch).
   size_t selected_index;
+  // ID of the tab the selected autocomplete suggestion was opened in.
+  // Set to -1 if we haven't yet determined the destination tab.
+  SessionID::id_type tab_id;
+  // The amount of time since the user first began modifying the text
+  // in the omnibox.  If at some point after modifying the text, the
+  // user reverts the modifications (thus seeing the current web
+  // page's URL again), then writes in the omnibox again, this time
+  // delta should be computed starting from the second series of
+  // modifications.  If we somehow skipped the logic to record
+  // the time the user began typing (this should only happen in
+  // unit tests), this elapsed time is set to -1 milliseconds.
+  base::TimeDelta elapsed_time_since_user_first_modified_omnibox;
   // Inline autocompleted length (if displayed).
   size_t inline_autocompleted_length;
   // Result set.
