@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,23 +6,27 @@
 #define CHROME_BROWSER_AUTOCOMPLETE_AUTOCOMPLETE_EDIT_H_
 #pragma once
 
+#include "base/memory/scoped_ptr.h"
 #include "base/string16.h"
+#include "base/time.h"
+#include "chrome/browser/autocomplete/autocomplete_controller_delegate.h"
 #include "chrome/browser/autocomplete/autocomplete_match.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/page_transition_types.h"
-#include "gfx/native_widget_types.h"
+#include "chrome/common/instant_types.h"
+#include "content/public/common/page_transition_types.h"
 #include "googleurl/src/gurl.h"
+#include "ui/gfx/native_widget_types.h"
 #include "webkit/glue/window_open_disposition.h"
 
-class AutocompletePopupModel;
-class Profile;
-class SkBitmap;
-
+class AutocompleteController;
 class AutocompleteEditController;
 class AutocompleteEditModel;
-class AutocompleteEditView;
+class AutocompletePopupModel;
 class AutocompleteResult;
+class InstantController;
+class OmniboxView;
+class Profile;
+class SkBitmap;
+class TabContentsWrapper;
 
 namespace gfx {
 class Rect;
@@ -35,30 +39,6 @@ class Rect;
 // Embedders of an AutocompleteEdit widget must implement this class.
 class AutocompleteEditController {
  public:
-  // Sent when the autocomplete popup is about to close.
-  virtual void OnAutocompleteWillClosePopup() = 0;
-
-  // Sent when the edit is losing focus. |view_gaining_focus| is the view
-  // gaining focus and may be null.
-  virtual void OnAutocompleteLosingFocus(
-      gfx::NativeView view_gaining_focus) = 0;
-
-  // Sent prior to OnAutoCompleteAccept and before the model has been reverted.
-  // This is only invoked if the popup is closed before invoking
-  // OnAutoCompleteAccept.
-  virtual void OnAutocompleteWillAccept() = 0;
-
-  // Commits the suggested text. |typed_text| is the current text showing in the
-  // autocomplete. Returns true if the text was committed.
-  virtual bool OnCommitSuggestedText(const std::wstring& typed_text) = 0;
-
-  // Accepts the currently showing instant preview, if any, and returns true.
-  // Returns false if there is no instant preview showing.
-  virtual bool AcceptCurrentInstantPreview() = 0;
-
-  // Invoked when the popup is going to change its bounds to |bounds|.
-  virtual void OnPopupBoundsChanged(const gfx::Rect& bounds) = 0;
-
   // When the user presses enter or selects a line with the mouse, this
   // function will get called synchronously with the url to open and
   // disposition and transition to use when opening it.
@@ -68,7 +48,7 @@ class AutocompleteEditController {
   // AutocompleteResult::GetAlternateNavURL().
   virtual void OnAutocompleteAccept(const GURL& url,
                                     WindowOpenDisposition disposition,
-                                    PageTransition::Type transition,
+                                    content::PageTransition transition,
                                     const GURL& alternate_nav_url) = 0;
 
   // Called when anything has changed that might affect the layout or contents
@@ -76,7 +56,7 @@ class AutocompleteEditController {
   // status of any keyword- or hint-related state.
   virtual void OnChanged() = 0;
 
-  // Called when the selection of the AutocompleteEditView changes.
+  // Called when the selection of the OmniboxView changes.
   virtual void OnSelectionBoundsChanged() = 0;
 
   // Called whenever the user starts or stops an input session (typing,
@@ -91,43 +71,54 @@ class AutocompleteEditController {
   virtual void OnSetFocus() = 0;
 
   // Returns the favicon of the current page.
-  virtual SkBitmap GetFavIcon() const = 0;
+  virtual SkBitmap GetFavicon() const = 0;
 
   // Returns the title of the current page.
-  virtual std::wstring GetTitle() const = 0;
+  virtual string16 GetTitle() const = 0;
+
+  // Returns the InstantController, or NULL if instant is not enabled.
+  virtual InstantController* GetInstant() = 0;
+
+  // Returns the TabContentsWrapper of the currently selected tab.
+  virtual TabContentsWrapper* GetTabContentsWrapper() const = 0;
 
  protected:
   virtual ~AutocompleteEditController();
 };
 
-class AutocompleteEditModel : public NotificationObserver {
+class AutocompleteEditModel : public AutocompleteControllerDelegate {
  public:
   struct State {
     State(bool user_input_in_progress,
-          const std::wstring& user_text,
-          const std::wstring& keyword,
+          const string16& user_text,
+          const string16& keyword,
           bool is_keyword_hint);
     ~State();
 
     bool user_input_in_progress;
-    const std::wstring user_text;
-    const std::wstring keyword;
+    const string16 user_text;
+    const string16 keyword;
     const bool is_keyword_hint;
   };
 
-  AutocompleteEditModel(AutocompleteEditView* view,
+  AutocompleteEditModel(OmniboxView* view,
                         AutocompleteEditController* controller,
                         Profile* profile);
-  ~AutocompleteEditModel();
+  virtual ~AutocompleteEditModel();
 
-  void SetPopupModel(AutocompletePopupModel* popup_model);
+  AutocompleteController* autocomplete_controller() const {
+    return autocomplete_controller_.get();
+  }
+
+  void set_popup_model(AutocompletePopupModel* popup_model) {
+    popup_ = popup_model;
+  }
 
   // TODO: The edit and popup should be siblings owned by the LocationBarView,
   // making this accessor unnecessary.
   AutocompletePopupModel* popup_model() const { return popup_; }
 
-  // Invoked when the profile has changed.
-  void SetProfile(Profile* profile);
+  AutocompleteEditController* controller() const { return controller_; }
 
   Profile* profile() const { return profile_; }
 
@@ -144,7 +135,7 @@ class AutocompleteEditModel : public NotificationObserver {
 
   // Called when the user wants to export the entire current text as a URL.
   // Sets the url, and if known, the title and favicon.
-  void GetDataForURLExport(GURL* url, std::wstring* title, SkBitmap* favicon);
+  void GetDataForURLExport(GURL* url, string16* title, SkBitmap* favicon);
 
   // Returns true if a verbatim query should be used for instant. A verbatim
   // query is forced in certain situations, such as pressing delete at the end
@@ -155,7 +146,7 @@ class AutocompleteEditModel : public NotificationObserver {
   // desired TLD is the TLD the user desires to add to the end of the current
   // input, if any, based on their control key state and any other actions
   // they've taken.
-  std::wstring GetDesiredTLD() const;
+  string16 GetDesiredTLD() const;
 
   // Returns true if the current edit contents will be treated as a
   // URL/navigation, as opposed to a search.
@@ -172,7 +163,7 @@ class AutocompleteEditModel : public NotificationObserver {
   // is set to true and |url| set to the url to write.
   void AdjustTextForCopy(int sel_min,
                          bool is_all_selected,
-                         std::wstring* text,
+                         string16* text,
                          GURL* url,
                          bool* write_url);
 
@@ -185,14 +176,39 @@ class AutocompleteEditModel : public NotificationObserver {
   // Updates permanent_text_ to |new_permanent_text|.  Returns true if this
   // change should be immediately user-visible, because either the user is not
   // editing or the edit does not have focus.
-  bool UpdatePermanentText(const std::wstring& new_permanent_text);
+  bool UpdatePermanentText(const string16& new_permanent_text);
+
+  // Returns the URL corresponding to the permanent text.
+  GURL PermanentURL();
 
   // Sets the user_text_ to |text|.  Only the View should call this.
-  void SetUserText(const std::wstring& text);
+  void SetUserText(const string16& text);
 
   // Calls through to SearchProvider::FinalizeInstantQuery.
-  void FinalizeInstantQuery(const std::wstring& input_text,
-                            const std::wstring& suggest_text);
+  // If |skip_inline_autocomplete| is true then the |suggest_text| will be
+  // turned into final text instead of inline autocomplete suggest.
+  void FinalizeInstantQuery(const string16& input_text,
+                            const string16& suggest_text,
+                            bool skip_inline_autocomplete);
+
+  // Sets the suggestion text.
+  void SetSuggestedText(const string16& text,
+                        InstantCompleteBehavior behavior);
+
+  // Commits the suggested text. If |skip_inline_autocomplete| is true then the
+  // suggested text will be committed as final text as if it's inputted by the
+  // user, rather than as inline autocomplete suggest.
+  // Returns true if the text was committed.
+  // TODO: can the return type be void?
+  bool CommitSuggestedText(bool skip_inline_autocomplete);
+
+  // Accepts the currently showing instant preview, if any, and returns true.
+  // Returns false if there is no instant preview showing.
+  bool AcceptCurrentInstantPreview();
+
+  // Invoked any time the text may have changed in the edit. Updates instant and
+  // notifies the controller.
+  void OnChanged();
 
   // Reverts the edit model back to its unedited state (permanent text showing,
   // no user input in progress).
@@ -202,22 +218,22 @@ class AutocompleteEditModel : public NotificationObserver {
   void StartAutocomplete(bool has_selected_text,
                          bool prevent_inline_autocomplete) const;
 
+  // Closes the popup and cancels any pending asynchronous queries.
+  void StopAutocomplete();
+
   // Determines whether the user can "paste and go", given the specified text.
   // This also updates the internal paste-and-go-related state variables as
   // appropriate so that the controller doesn't need to be repeatedly queried
   // for the same text in every clipboard-related function.
-  bool CanPasteAndGo(const std::wstring& text) const;
+  bool CanPasteAndGo(const string16& text) const;
 
   // Navigates to the destination last supplied to CanPasteAndGo.
   void PasteAndGo();
 
-  // Returns the url set by way of CanPasteAndGo.
-  const GURL& paste_and_go_url() const { return paste_and_go_url_; }
-
   // Returns true if this is a paste-and-search rather than paste-and-go (or
   // nothing).
   bool is_paste_and_search() const {
-    return (paste_and_go_transition_ != PageTransition::TYPED);
+    return (paste_and_go_match_.transition != content::PAGE_TRANSITION_TYPED);
   }
 
   // Asks the browser to load the popup's currently selected item, using the
@@ -229,18 +245,17 @@ class AutocompleteEditModel : public NotificationObserver {
                    bool for_drop);
 
   // Asks the browser to load the item at |index|, with the given properties.
-  void OpenURL(const GURL& url,
-               WindowOpenDisposition disposition,
-               PageTransition::Type transition,
-               const GURL& alternate_nav_url,
-               size_t index,
-               const std::wstring& keyword);
+  void OpenMatch(const AutocompleteMatch& match,
+                 WindowOpenDisposition disposition,
+                 const GURL& alternate_nav_url,
+                 size_t index,
+                 const string16& keyword);
 
   bool has_focus() const { return has_focus_; }
 
   // Accessors for keyword-related state (see comments on keyword_ and
   // is_keyword_hint_).
-  const std::wstring& keyword() const { return keyword_; }
+  const string16& keyword() const { return keyword_; }
   bool is_keyword_hint() const { return is_keyword_hint_; }
 
   // Accepts the current keyword hint as a keyword. It always returns true for
@@ -249,13 +264,7 @@ class AutocompleteEditModel : public NotificationObserver {
 
   // Clears the current keyword.  |visible_text| is the (non-keyword) text
   // currently visible in the edit.
-  void ClearKeyword(const std::wstring& visible_text);
-
-  // Returns true if a query to an autocomplete provider is currently
-  // in progress.  This logic should in the future live in
-  // AutocompleteController but resides here for now.  This method is used by
-  // AutomationProvider::AutocompleteEditIsQueryInProgress.
-  bool query_in_progress() const;
+  void ClearKeyword(const string16& visible_text);
 
   // Returns the current autocomplete result.  This logic should in the future
   // live in AutocompleteController but resides here for now.  This method is
@@ -266,6 +275,9 @@ class AutocompleteEditModel : public NotificationObserver {
   // control key is down (at the time we're gaining focus).
   void OnSetFocus(bool control_down);
 
+  // Sent before |OnKillFocus| and before the popup is closed.
+  void OnWillKillFocus(gfx::NativeView view_gaining_focus);
+
   // Called when the view is losing focus.  Resets some state.
   void OnKillFocus();
 
@@ -280,6 +292,9 @@ class AutocompleteEditModel : public NotificationObserver {
   // Called when the user pastes in text.
   void on_paste() { paste_state_ = PASTING; }
 
+  // Returns true if pasting is in progress.
+  bool is_pasting() const { return paste_state_ == PASTING; }
+
   // Called when the user presses up or down.  |count| is a repeat count,
   // negative for moving up, positive for moving down.
   void OnUpOrDownKeyPressed(int count);
@@ -297,20 +312,20 @@ class AutocompleteEditModel : public NotificationObserver {
   //     or the currently selected keyword if |is_keyword_hint| is false (see
   //     comments on keyword_ and is_keyword_hint_).
   void OnPopupDataChanged(
-      const std::wstring& text,
+      const string16& text,
       GURL* destination_for_temporary_text_change,
-      const std::wstring& keyword,
+      const string16& keyword,
       bool is_keyword_hint);
 
-  // Called by the AutocompleteEditView after something changes, with details
-  // about what state changes occured.  Updates internal state, updates the
-  // popup if necessary, and returns true if any significant changes occurred.
+  // Called by the OmniboxView after something changes, with details about what
+  // state changes occured.  Updates internal state, updates the popup if
+  // necessary, and returns true if any significant changes occurred.
   // If |allow_keyword_ui_change| is false then the change should not affect
   // keyword ui state, even if the text matches a keyword exactly. This value
-  // may be false when:
-  // 1) The insert caret is not at the end of the edit box
-  // 2) The user is composing a text with an IME
-  bool OnAfterPossibleChange(const std::wstring& new_text,
+  // may be false when the user is composing a text with an IME.
+  bool OnAfterPossibleChange(const string16& new_text,
+                             size_t selection_start,
+                             size_t selection_end,
                              bool selection_differs,
                              bool text_differs,
                              bool just_deleted_text,
@@ -319,6 +334,12 @@ class AutocompleteEditModel : public NotificationObserver {
   // Invoked when the popup is going to change its bounds to |bounds|.
   void PopupBoundsChangedTo(const gfx::Rect& bounds);
 
+#if defined(UNIT_TEST)
+  InstantCompleteBehavior instant_complete_behavior() const {
+    return instant_complete_behavior_;
+  }
+#endif
+
  private:
   enum PasteState {
     NONE,           // Most recent edit was not a paste.
@@ -345,13 +366,17 @@ class AutocompleteEditModel : public NotificationObserver {
                           // he intended to hit "ctrl-enter".
   };
 
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // AutocompleteControllerDelegate:
+  virtual void OnResultChanged(bool default_match_changed) OVERRIDE;
+
+  // Returns true if a query to an autocomplete provider is currently
+  // in progress.  This logic should in the future live in
+  // AutocompleteController but resides here for now.  This method is used by
+  // AutomationProvider::AutocompleteEditIsQueryInProgress.
+  bool query_in_progress() const;
 
   // Called whenever user_text_ should change.
-  void InternalSetUserText(const std::wstring& text);
+  void InternalSetUserText(const string16& text);
 
   // Returns true if a keyword is selected.
   bool KeywordIsSelected() const;
@@ -359,45 +384,73 @@ class AutocompleteEditModel : public NotificationObserver {
   // Conversion between user text and display text. User text is the text the
   // user has input. Display text is the text being shown in the edit. The
   // two are different if a keyword is selected.
-  std::wstring DisplayTextFromUserText(const std::wstring& text) const;
-  std::wstring UserTextFromDisplayText(const std::wstring& text) const;
+  string16 DisplayTextFromUserText(const string16& text) const;
+  string16 UserTextFromDisplayText(const string16& text) const;
+
+  // Copies the selected match into |match|.  If an update is in progress,
+  // "selected" means "default in the latest matches".  If there are no matches,
+  // does not update |match|.
+  //
+  // If |alternate_nav_url| is non-NULL, it will be set to the alternate
+  // navigation URL for |url| if one exists, or left unchanged otherwise.  See
+  // comments on AutocompleteResult::GetAlternateNavURL().
+  //
+  // TODO(pkasting): When manually_selected_match_ moves to the controller, this
+  // can move too.
+  void InfoForCurrentSelection(AutocompleteMatch* match,
+                               GURL* alternate_nav_url) const;
 
   // Returns the default match for the current text, as well as the alternate
   // nav URL, if |alternate_nav_url| is non-NULL and there is such a URL.
   void GetInfoForCurrentText(AutocompleteMatch* match,
                              GURL* alternate_nav_url) const;
 
-  // Returns true if |text| (which is display text in the current context)
-  // parses as a URL, and in that case sets |url| to the calculated URL.
-  // Subtle note: This ignores the desired_tld_ (unlike GetDataForURLExport()
-  // and CurrentTextIsURL()).  The view needs this because it calls this
-  // function during copy handling, when the control key is down to trigger the
-  // copy.
-  bool GetURLForText(const std::wstring& text, GURL* url) const;
+  // Reverts the edit box from a temporary text back to the original user text.
+  // If |revert_popup| is true then the popup will be reverted as well.
+  void RevertTemporaryText(bool revert_popup);
 
   // Accepts current keyword if the user only typed a space at the end of
   // |new_user_text| comparing to the |old_user_text|.
   // Returns true if the current keyword is accepted.
-  bool MaybeAcceptKeywordBySpace(const std::wstring& old_user_text,
-                                 const std::wstring& new_user_text);
+  bool MaybeAcceptKeywordBySpace(const string16& old_user_text,
+                                 const string16& new_user_text);
+
+  // Checks if |allow_exact_keyword_match_| should be set to true according to
+  // the old and new user text and the current caret position. It does not take
+  // other factors into account, e.g. if the view is ready to change the keyword
+  // ui or not. This is only for the case of inserting a space character in the
+  // middle of the text. See the comment of |allow_exact_keyword_match_| below.
+  bool ShouldAllowExactKeywordMatch(const string16& old_user_text,
+                                    const string16& new_user_text,
+                                    size_t caret_position);
+
+  // Tries to start an instant preview for |match|. Returns true if instant
+  // processed the match.
+  bool DoInstant(const AutocompleteMatch& match, string16* suggested_text);
+
+  // Starts a prerender for the given |match|.
+  void DoPrerender(const AutocompleteMatch& match);
+
+  // Starts a DNS prefetch for the given |match|.
+  void DoPreconnect(const AutocompleteMatch& match);
 
   // Checks if a given character is a valid space character for accepting
   // keyword.
   static bool IsSpaceCharForAcceptingKeyword(wchar_t c);
 
-  AutocompleteEditView* view_;
+  scoped_ptr<AutocompleteController> autocomplete_controller_;
+
+  OmniboxView* view_;
 
   AutocompletePopupModel* popup_;
 
   AutocompleteEditController* controller_;
 
-  NotificationRegistrar registrar_;
-
   // Whether the edit has focus.
   bool has_focus_;
 
   // The URL of the currently displayed page.
-  std::wstring permanent_text_;
+  string16 permanent_text_;
 
   // This flag is true when the user has modified the contents of the edit, but
   // not yet accepted them.  We use this to determine when we need to save
@@ -408,7 +461,11 @@ class AutocompleteEditModel : public NotificationObserver {
 
   // The text that the user has entered.  This does not include inline
   // autocomplete text that has not yet been accepted.
-  std::wstring user_text_;
+  string16 user_text_;
+
+  // We keep track of when the user began modifying the omnibox text.
+  // This should be valid whenever user_input_in_progress_ is true.
+  base::TimeTicks time_user_first_modified_omnibox_;
 
   // When the user closes the popup, we need to remember the URL for their
   // desired choice, so that if they hit enter without reopening the popup we
@@ -425,7 +482,7 @@ class AutocompleteEditModel : public NotificationObserver {
   // simply ask the popup for the desired URL directly.  As a result, the
   // contents of this variable only need to be updated when the popup is closed
   // but user_input_in_progress_ is not being cleared.
-  std::wstring url_for_remembered_user_selection_;
+  string16 url_for_remembered_user_selection_;
 
   // Inline autocomplete is allowed if the user has not just deleted text, and
   // no temporary text is showing.  In this case, inline_autocomplete_text_ is
@@ -435,7 +492,7 @@ class AutocompleteEditModel : public NotificationObserver {
   // text (actions that close the popup should either accept the text, convert
   // it to a normal selection, or change the edit entirely).
   bool just_deleted_text_;
-  std::wstring inline_autocomplete_text_;
+  string16 inline_autocomplete_text_;
 
   // Used by OnPopupDataChanged to keep track of whether there is currently a
   // temporary text.
@@ -468,7 +525,7 @@ class AutocompleteEditModel : public NotificationObserver {
   // selected keyword, or just some input text that looks like a keyword (so we
   // can show a hint to press <tab>).  This is the keyword in either case;
   // is_keyword_hint_ (below) distinguishes the two cases.
-  std::wstring keyword_;
+  string16 keyword_;
 
   // True if the keyword associated with this match is merely a hint, i.e. the
   // user hasn't actually selected a keyword yet.  When this is true, we can use
@@ -476,12 +533,34 @@ class AutocompleteEditModel : public NotificationObserver {
   bool is_keyword_hint_;
 
   // Paste And Go-related state.  See CanPasteAndGo().
-  mutable GURL paste_and_go_url_;
-  mutable PageTransition::Type paste_and_go_transition_;
+  mutable AutocompleteMatch paste_and_go_match_;
   mutable GURL paste_and_go_alternate_nav_url_;
 
   Profile* profile_;
 
+  // This is needed as prior to accepting the current text the model is
+  // reverted, which triggers resetting instant. We don't want to update instant
+  // in this case, so we use the flag to determine if this is happening.
+  bool in_revert_;
+
+  // Indicates if the upcoming autocomplete search is allowed to be treated as
+  // an exact keyword match. If it's true then keyword mode will be triggered
+  // automatically if the input is "<keyword> <search string>". We only allow
+  // such trigger when:
+  // 1.A single space character is added at the end of a keyword, such as:
+  //   (assume "foo" is a keyword, | is the input caret)
+  //   foo| -> foo |
+  //   foo[bar] -> foo |  ([bar] indicates a selected text "bar")
+  // 2.A single space character is inserted after a keyword when the caret is
+  //   not at the end of the line, such as:
+  //   foo|bar -> foo |bar
+  //
+  // It has no effect if a keyword is already selected.
+  bool allow_exact_keyword_match_;
+
+  // Last value of InstantCompleteBehavior supplied to |SetSuggestedText|.
+  InstantCompleteBehavior instant_complete_behavior_;
+
   DISALLOW_COPY_AND_ASSIGN(AutocompleteEditModel);
 };
 
