@@ -13,67 +13,36 @@
 #include "chrome/browser/autocomplete/autocomplete_edit.h"
 #include "chrome/browser/autocomplete/autocomplete_match.h"
 #include "chrome/browser/autocomplete/autocomplete_popup_view.h"
-#include "chrome/browser/autocomplete/search_provider.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search_engines/template_url.h"
-#include "chrome/browser/search_engines/template_url_model.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_source.h"
-#include "gfx/rect.h"
+#include "chrome/browser/search_engines/template_url_service.h"
+#include "chrome/browser/search_engines/template_url_service_factory.h"
+#include "ui/gfx/rect.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 // AutocompletePopupModel
 
 AutocompletePopupModel::AutocompletePopupModel(
     AutocompletePopupView* popup_view,
-    AutocompleteEditModel* edit_model,
-    Profile* profile)
+    AutocompleteEditModel* edit_model)
     : view_(popup_view),
       edit_model_(edit_model),
-      controller_(new AutocompleteController(profile)),
-      profile_(profile),
       hovered_line_(kNoMatch),
       selected_line_(kNoMatch) {
-  registrar_.Add(this, NotificationType::AUTOCOMPLETE_CONTROLLER_RESULT_UPDATED,
-                 Source<AutocompleteController>(controller_.get()));
+  edit_model->set_popup_model(this);
 }
 
 AutocompletePopupModel::~AutocompletePopupModel() {
 }
 
-void AutocompletePopupModel::SetProfile(Profile* profile) {
-  DCHECK(profile);
-  profile_ = profile;
-  controller_->SetProfile(profile);
-}
-
-void AutocompletePopupModel::StartAutocomplete(
-    const std::wstring& text,
-    const std::wstring& desired_tld,
-    bool prevent_inline_autocomplete,
-    bool prefer_keyword,
-    bool allow_exact_keyword_match) {
-  // The user is interacting with the edit, so stop tracking hover.
-  SetHoveredLine(kNoMatch);
-
-  manually_selected_match_.Clear();
-
-  controller_->Start(text, desired_tld, prevent_inline_autocomplete,
-                     prefer_keyword, allow_exact_keyword_match, false);
-}
-
-void AutocompletePopupModel::StopAutocomplete() {
-  controller_->Stop(true);
-}
-
 bool AutocompletePopupModel::IsOpen() const {
   return view_->IsOpen();
 }
 
 void AutocompletePopupModel::SetHoveredLine(size_t line) {
   const bool is_disabling = (line == kNoMatch);
-  DCHECK(is_disabling || (line < controller_->result().size()));
+  DCHECK(is_disabling || (line < result().size()));
 
   if (line == hovered_line_)
     return;  // Nothing to do
@@ -90,18 +59,14 @@ void AutocompletePopupModel::SetHoveredLine(size_t line) {
 }
 
 void AutocompletePopupModel::SetSelectedLine(size_t line,
-                                             bool reset_to_default) {
-  // We should at least be dealing with the results of the current query.  Note
-  // that even if |line| was valid on entry, this may make it invalid.  We clamp
-  // it below.
-  controller_->CommitIfQueryHasNeverBeenCommitted();
-
-  const AutocompleteResult& result = controller_->result();
+                                             bool reset_to_default,
+                                             bool force) {
+  const AutocompleteResult& result = this->result();
   if (result.empty())
     return;
 
   // Cancel the query so the matches don't change on the user.
-  controller_->Stop(false);
+  autocomplete_controller()->Stop(false);
 
   line = std::min(line, result.size() - 1);
   const AutocompleteMatch& match = result.match_at(line);
@@ -115,7 +80,7 @@ void AutocompletePopupModel::SetSelectedLine(size_t line,
         match.is_history_what_you_typed_match;
   }
 
-  if (line == selected_line_)
+  if (line == selected_line_ && !force)
     return;  // Nothing else to do.
 
   // We need to update |selected_line_| before calling OnPopupDataChanged(), so
@@ -134,11 +99,11 @@ void AutocompletePopupModel::SetSelectedLine(size_t line,
   // Update the edit with the new data for this match.
   // TODO(pkasting): If |selected_line_| moves to the controller, this can be
   // eliminated and just become a call to the observer on the edit.
-  std::wstring keyword;
+  string16 keyword;
   const bool is_keyword_hint = GetKeywordForMatch(match, &keyword);
   if (reset_to_default) {
-    std::wstring inline_autocomplete_text;
-    if ((match.inline_autocomplete_offset != std::wstring::npos) &&
+    string16 inline_autocomplete_text;
+    if ((match.inline_autocomplete_offset != string16::npos) &&
         (match.inline_autocomplete_offset < match.fill_into_edit.length())) {
       inline_autocomplete_text =
           match.fill_into_edit.substr(match.inline_autocomplete_offset);
@@ -156,107 +121,67 @@ void AutocompletePopupModel::SetSelectedLine(size_t line,
 }
 
 void AutocompletePopupModel::ResetToDefaultMatch() {
-  const AutocompleteResult& result = controller_->result();
+  const AutocompleteResult& result = this->result();
   CHECK(!result.empty());
-  SetSelectedLine(result.default_match() - result.begin(), true);
+  SetSelectedLine(result.default_match() - result.begin(), true, false);
   view_->OnDragCanceled();
 }
 
-void AutocompletePopupModel::InfoForCurrentSelection(
-    AutocompleteMatch* match,
-    GURL* alternate_nav_url) const {
-  DCHECK(match != NULL);
-  const AutocompleteResult* result;
-  if (!controller_->done()) {
-    // NOTE: Using latest_result() is important here since not only could it
-    // contain newer results than result() for the current query, it could even
-    // refer to an entirely different query (e.g. if the user is typing rapidly
-    // and the controller is purposefully delaying updates to avoid flicker).
-    result = &controller_->latest_result();
-    // It's technically possible for |result| to be empty if no provider returns
-    // a synchronous result but the query has not completed synchronously;
-    // pratically, however, that should never actually happen.
-    if (result->empty())
-      return;
-    // The user cannot have manually selected a match, or the query would have
-    // stopped.  So the default match must be the desired selection.
-    *match = *result->default_match();
-  } else {
-    CHECK(IsOpen());
-    // The query isn't running, so the standard result set can't possibly be out
-    // of date.
-    //
-    // NOTE: In practice, it should actually be safe to use
-    // controller_->latest_result() here too, since the controller keeps that
-    // up-to-date.  However we generally try to avoid referring to that.
-    result = &controller_->result();
-    // If there are no results, the popup should be closed (so we should have
-    // failed the CHECK above), and URLsForDefaultMatch() should have been
-    // called instead.
-    CHECK(!result->empty());
-    CHECK(selected_line_ < result->size());
-    *match = result->match_at(selected_line_);
-  }
-  if (alternate_nav_url && manually_selected_match_.empty())
-    *alternate_nav_url = result->alternate_nav_url();
-}
-
 bool AutocompletePopupModel::GetKeywordForMatch(const AutocompleteMatch& match,
-                                                std::wstring* keyword) const {
+                                                string16* keyword) const {
   // Assume we have no keyword until we find otherwise.
   keyword->clear();
 
-  // If the current match is a keyword, return that as the selected keyword.
-  if (TemplateURL::SupportsReplacement(match.template_url)) {
-    keyword->assign(UTF16ToWideHack(match.template_url->keyword()));
+  if (match.template_url &&
+      TemplateURL::SupportsReplacement(match.template_url) &&
+      match.transition == content::PAGE_TRANSITION_KEYWORD) {
+    // The current match is a keyword, return that as the selected keyword.
+    keyword->assign(match.template_url->keyword());
     return false;
   }
 
   // See if the current match's fill_into_edit corresponds to a keyword.
-  if (!profile_->GetTemplateURLModel())
-    return false;
-  profile_->GetTemplateURLModel()->Load();
-  const string16 keyword_hint(TemplateURLModel::CleanUserInputKeyword(
-      WideToUTF16Hack(match.fill_into_edit)));
+  return GetKeywordForText(match.fill_into_edit, keyword);
+}
+
+bool AutocompletePopupModel::GetKeywordForText(const string16& text,
+                                               string16* keyword) const {
+  // Creates keyword_hint first in case |keyword| is a pointer to |text|.
+  const string16 keyword_hint(TemplateURLService::CleanUserInputKeyword(text));
+
+  // Assume we have no keyword until we find otherwise.
+  keyword->clear();
+
   if (keyword_hint.empty())
     return false;
+  Profile* profile = edit_model_->profile();
+  TemplateURLService* url_service =
+      TemplateURLServiceFactory::GetForProfile(profile);
+  if (!url_service)
+    return false;
+  url_service->Load();
 
   // Don't provide a hint if this keyword doesn't support replacement.
   const TemplateURL* const template_url =
-      profile_->GetTemplateURLModel()->GetTemplateURLForKeyword(keyword_hint);
+      url_service->GetTemplateURLForKeyword(keyword_hint);
   if (!TemplateURL::SupportsReplacement(template_url))
     return false;
 
   // Don't provide a hint for inactive/disabled extension keywords.
   if (template_url->IsExtensionKeyword()) {
-    const Extension* extension = profile_->GetExtensionService()->
+    const Extension* extension = profile->GetExtensionService()->
         GetExtensionById(template_url->GetExtensionId(), false);
-    if (!extension ||
-        (profile_->IsOffTheRecord() &&
-         !profile_->GetExtensionService()->IsIncognitoEnabled(extension)))
+    if (!extension || (profile->IsOffTheRecord() &&
+        !profile->GetExtensionService()->IsIncognitoEnabled(extension->id())))
       return false;
   }
 
-  keyword->assign(UTF16ToWideHack(keyword_hint));
+  keyword->assign(keyword_hint);
   return true;
 }
 
-void AutocompletePopupModel::FinalizeInstantQuery(
-    const std::wstring& input_text,
-    const std::wstring& suggest_text) {
-  if (IsOpen()) {
-    SearchProvider* search_provider = controller_->search_provider();
-    search_provider->FinalizeInstantQuery(input_text, suggest_text);
-  }
-}
-
-AutocompleteLog* AutocompletePopupModel::GetAutocompleteLog() {
-  return new AutocompleteLog(controller_->input().text(),
-      controller_->input().type(), selected_line_, 0, controller_->result());
-}
-
 void AutocompletePopupModel::Move(int count) {
-  const AutocompleteResult& result = controller_->result();
+  const AutocompleteResult& result = this->result();
   if (result.empty())
     return;
 
@@ -267,7 +192,7 @@ void AutocompletePopupModel::Move(int count) {
   // Clamp the new line to [0, result_.count() - 1].
   const size_t new_line = selected_line_ + count;
   SetSelectedLine(((count < 0) && (new_line >= selected_line_)) ? 0 : new_line,
-                  false);
+                  false, false);
 }
 
 void AutocompletePopupModel::TryDeletingCurrentItem() {
@@ -278,54 +203,51 @@ void AutocompletePopupModel::TryDeletingCurrentItem() {
     return;
 
   // Cancel the query so the matches don't change on the user.
-  controller_->Stop(false);
+  autocomplete_controller()->Stop(false);
 
-  const AutocompleteMatch& match =
-      controller_->result().match_at(selected_line_);
+  const AutocompleteMatch& match = result().match_at(selected_line_);
   if (match.deletable) {
     const size_t selected_line = selected_line_;
-    controller_->DeleteMatch(match);  // This may synchronously notify us that
-                                      // the results have changed.
-    const AutocompleteResult& result = controller_->result();
-    if (!result.empty()) {
+    const bool was_temporary_text = !manually_selected_match_.empty();
+
+    // This will synchronously notify both the edit and us that the results
+    // have changed, causing both to revert to the default match.
+    autocomplete_controller()->DeleteMatch(match);
+    const AutocompleteResult& result = this->result();
+    if (!result.empty() &&
+        (was_temporary_text || selected_line != selected_line_)) {
       // Move the selection to the next choice after the deleted one.
       // SetSelectedLine() will clamp to take care of the case where we deleted
       // the last item.
       // TODO(pkasting): Eventually the controller should take care of this
       // before notifying us, reducing flicker.  At that point the check for
       // deletability can move there too.
-      SetSelectedLine(selected_line, false);
+      SetSelectedLine(selected_line, false, true);
     }
   }
 }
 
-void AutocompletePopupModel::Observe(NotificationType type,
-                                     const NotificationSource& source,
-                                     const NotificationDetails& details) {
-  DCHECK_EQ(NotificationType::AUTOCOMPLETE_CONTROLLER_RESULT_UPDATED,
-            type.value);
-
-  const AutocompleteResult* result =
-      Details<const AutocompleteResult>(details).ptr();
-  selected_line_ = result->default_match() == result->end() ?
-      kNoMatch : static_cast<size_t>(result->default_match() - result->begin());
-  // There had better not be a nonempty result set with no default match.
-  CHECK((selected_line_ != kNoMatch) || result->empty());
-  // If we're going to trim the window size to no longer include the hovered
-  // line, turn hover off.  Practically, this shouldn't happen, but it
-  // doesn't hurt to be defensive.
-  if ((hovered_line_ != kNoMatch) && (result->size() <= hovered_line_))
-    SetHoveredLine(kNoMatch);
-
-  view_->UpdatePopupAppearance();
-  edit_model_->PopupBoundsChangedTo(view_->GetTargetBounds());
-}
-
-const SkBitmap* AutocompletePopupModel::GetSpecialIconForMatch(
+const SkBitmap* AutocompletePopupModel::GetIconIfExtensionMatch(
     const AutocompleteMatch& match) const {
   if (!match.template_url || !match.template_url->IsExtensionKeyword())
     return NULL;
 
-  return &profile_->GetExtensionService()->GetOmniboxPopupIcon(
+  return &edit_model_->profile()->GetExtensionService()->GetOmniboxPopupIcon(
       match.template_url->GetExtensionId());
 }
+
+void AutocompletePopupModel::OnResultChanged() {
+  const AutocompleteResult& result = this->result();
+  selected_line_ = result.default_match() == result.end() ?
+      kNoMatch : static_cast<size_t>(result.default_match() - result.begin());
+  // There had better not be a nonempty result set with no default match.
+  CHECK((selected_line_ != kNoMatch) || result.empty());
+  manually_selected_match_.Clear();
+  // If we're going to trim the window size to no longer include the hovered
+  // line, turn hover off.  Practically, this shouldn't happen, but it
+  // doesn't hurt to be defensive.
+  if ((hovered_line_ != kNoMatch) && (result.size() <= hovered_line_))
+    SetHoveredLine(kNoMatch);
+
+  view_->UpdatePopupAppearance();
+}
