@@ -1,23 +1,24 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/autocomplete/history_contents_provider.h"
 
-#include "base/callback.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/metrics/histogram.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/autocomplete/autocomplete_match.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/bookmarks/bookmark_utils.h"
-#include "chrome/browser/history/query_parser.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/url_constants.h"
 #include "googleurl/src/url_util.h"
 #include "net/base/net_util.h"
 
 using base::TimeTicks;
+using history::HistoryDatabase;
 
 namespace {
 
@@ -25,40 +26,34 @@ namespace {
 // time it will take.
 const int kDaysToSearch = 30;
 
-// When processing the results from the history query, this structure points to
-// a single result. It allows the results to be sorted and processed without
-// modifying the larger and slower results structure.
-struct MatchReference {
-  MatchReference(const history::URLResult* result, int relevance)
-      : result(result),
-        relevance(relevance) {
-  }
+}  // end namespace
 
-  const history::URLResult* result;
-  int relevance;  // Score of relevance computed by CalculateRelevance.
-};
-
-// This is a > operator for MatchReference.
-bool CompareMatchRelevance(const MatchReference& a, const MatchReference& b) {
-  if (a.relevance != b.relevance)
-    return a.relevance > b.relevance;
-
-  // Want results in reverse-chronological order all else being equal.
-  return a.result->last_visit() > b.result->last_visit();
+HistoryContentsProvider::MatchReference::MatchReference(
+    const history::URLResult* result,
+    int relevance)
+    : result(result),
+      relevance(relevance) {
 }
 
-}  // namespace
-
-using history::HistoryDatabase;
+// static
+bool HistoryContentsProvider::MatchReference::CompareRelevance(
+    const HistoryContentsProvider::MatchReference& lhs,
+    const HistoryContentsProvider::MatchReference& rhs) {
+  if (lhs.relevance != rhs.relevance)
+    return lhs.relevance > rhs.relevance;
+  return lhs.result->last_visit() > rhs.result->last_visit();
+}
 
 HistoryContentsProvider::HistoryContentsProvider(ACProviderListener* listener,
-                                                 Profile* profile)
+                                                 Profile* profile,
+                                                 bool body_only)
     : HistoryProvider(listener, profile, "HistoryContents"),
       star_title_count_(0),
       star_contents_count_(0),
       title_count_(0),
       contents_count_(0),
       input_type_(AutocompleteInput::INVALID),
+      body_only_(body_only),
       trim_http_(false),
       have_results_(false) {
 }
@@ -82,7 +77,13 @@ void HistoryContentsProvider::Start(const AutocompleteInput& input,
   if ((input.type() == AutocompleteInput::URL) ||
       (((input.type() == AutocompleteInput::REQUESTED_URL) ||
         (input.type() == AutocompleteInput::UNKNOWN)) &&
-       (input.text().find('.') != std::wstring::npos))) {
+       (input.text().find('.') != string16::npos))) {
+    Stop();
+    return;
+  }
+
+  if (input.matches_requested() == AutocompleteInput::BEST_MATCH) {
+    // None of our results are applicable for best match.
     Stop();
     return;
   }
@@ -105,7 +106,7 @@ void HistoryContentsProvider::Start(const AutocompleteInput& input,
     // allowed to keep running it, do so, and when it finishes, its results will
     // get marked up for this new input.  In synchronous_only mode, cancel the
     // history query.
-    if (input.synchronous_only()) {
+    if (input.matches_requested() != AutocompleteInput::ALL_MATCHES) {
       done_ = true;
       request_consumer_.CancelAllRequests();
     }
@@ -113,7 +114,7 @@ void HistoryContentsProvider::Start(const AutocompleteInput& input,
     return;
   }
 
-  if (results_.size() != 0) {
+  if (!results_.empty()) {
     // Clear the results. We swap in an empty one as the easy way to clear it.
     history::QueryResults empty_results;
     results_.Swap(&empty_results);
@@ -125,18 +126,20 @@ void HistoryContentsProvider::Start(const AutocompleteInput& input,
   // Convert the bookmark results.
   ConvertResults();
 
-  if (!input.synchronous_only()) {
+  if (input.matches_requested() == AutocompleteInput::ALL_MATCHES) {
     HistoryService* history =
         profile_->GetHistoryService(Profile::EXPLICIT_ACCESS);
     if (history) {
       done_ = false;
 
       history::QueryOptions options;
+      options.body_only = body_only_;
       options.SetRecentDayRange(kDaysToSearch);
       options.max_count = kMaxMatches;
-      history->QueryHistory(WideToUTF16(input.text()), options,
+      history->QueryHistory(input.text(), options,
           &request_consumer_,
-          NewCallback(this, &HistoryContentsProvider::QueryComplete));
+          base::Bind(&HistoryContentsProvider::QueryComplete,
+                     base::Unretained(this)));
     }
   }
 }
@@ -186,25 +189,25 @@ void HistoryContentsProvider::ConvertResults() {
   // Get the top matches and add them.
   size_t max_for_provider = std::min(kMaxMatches, result_refs.size());
   std::partial_sort(result_refs.begin(), result_refs.begin() + max_for_provider,
-                    result_refs.end(), &CompareMatchRelevance);
+                    result_refs.end(), &MatchReference::CompareRelevance);
   matches_.clear();
   for (size_t i = 0; i < max_for_provider; i++) {
-    matches_.push_back(ResultToMatch(*result_refs[i].result,
-                                     result_refs[i].relevance));
+    AutocompleteMatch match(ResultToMatch(result_refs[i]));
+    matches_.push_back(match);
   }
 }
 
-static bool MatchInTitle(const history::URLResult& result) {
-  return !result.title_match_positions().empty();
+// TODO(mrossetti): Remove MatchInTitle once body_only_ becomes permanent.
+bool HistoryContentsProvider::MatchInTitle(const history::URLResult& result) {
+  return !body_only_ && !result.title_match_positions().empty();
 }
 
 AutocompleteMatch HistoryContentsProvider::ResultToMatch(
-    const history::URLResult& result,
-    int score) {
-  // TODO(sky): if matched title highlight matching words in title.
-  // Also show star in popup.
-  AutocompleteMatch match(this, score, true, MatchInTitle(result) ?
-      AutocompleteMatch::HISTORY_TITLE : AutocompleteMatch::HISTORY_BODY);
+    const MatchReference& match_reference) {
+  const history::URLResult& result = *match_reference.result;
+  AutocompleteMatch match(this, match_reference.relevance, true,
+      MatchInTitle(result) ?
+          AutocompleteMatch::HISTORY_TITLE : AutocompleteMatch::HISTORY_BODY);
   match.contents = StringForURLDisplay(result.url(), true, trim_http_);
   match.fill_into_edit =
       AutocompleteInput::FormattedStringWithEquivalentMeaning(result.url(),
@@ -212,7 +215,7 @@ AutocompleteMatch HistoryContentsProvider::ResultToMatch(
   match.destination_url = result.url();
   match.contents_class.push_back(
       ACMatchClassification(0, ACMatchClassification::URL));
-  match.description = UTF16ToWide(result.title());
+  match.description = result.title();
   match.starred =
       (profile_->GetBookmarkModel() &&
        profile_->GetBookmarkModel()->IsBookmarked(result.url()));
@@ -240,7 +243,7 @@ void HistoryContentsProvider::ClassifyDescription(
       offset = i->second;
     }
   }
-  if (offset != result.title().size()) {
+  if (offset != result.title().length()) {
     match->description_class.push_back(
         ACMatchClassification(offset, ACMatchClassification::NONE));
   }
@@ -265,7 +268,7 @@ void HistoryContentsProvider::QueryBookmarks(const AutocompleteInput& input) {
 
   TimeTicks start_time = TimeTicks::Now();
   std::vector<bookmark_utils::TitleMatch> matches;
-  bookmark_model->GetBookmarksWithTitlesMatching(WideToUTF16Hack(input.text()),
+  bookmark_model->GetBookmarksWithTitlesMatching(input.text(),
                                                  kMaxMatches, &matches);
   for (size_t i = 0; i < matches.size(); ++i)
     AddBookmarkTitleMatchToResults(matches[i]);
@@ -275,7 +278,7 @@ void HistoryContentsProvider::QueryBookmarks(const AutocompleteInput& input) {
 
 void HistoryContentsProvider::AddBookmarkTitleMatchToResults(
     const bookmark_utils::TitleMatch& match) {
-  history::URLResult url_result(match.node->GetURL(), match.match_positions);
+  history::URLResult url_result(match.node->url(), match.match_positions);
   url_result.set_title(match.node->GetTitle());
   results_.AppendURLBySwapping(&url_result);
 }
