@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,28 +8,23 @@
 
 #include <string>
 
+#include "chrome/browser/autocomplete/autocomplete_match.h"
 #include "chrome/browser/autocomplete/history_provider.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/history/in_memory_url_index.h"
 
 class Profile;
-
-namespace history {
-class HistoryBackend;
-}  // namespace history
+class TermMatches;
 
 // This class is an autocomplete provider (a pseudo-internal component of
 // the history system) which quickly (and synchronously) provides matching
 // results from recently or frequently visited sites in the profile's
 // history.
-//
-// TODO(mrossetti): Review to see if the following applies since we're not
-// using the database during the autocomplete pass.
 class HistoryQuickProvider : public HistoryProvider {
  public:
   HistoryQuickProvider(ACProviderListener* listener, Profile* profile);
 
-  ~HistoryQuickProvider();
+  virtual ~HistoryQuickProvider();
 
   // AutocompleteProvider. |minimal_changes| is ignored since there
   // is no asynch completion performed.
@@ -38,42 +33,60 @@ class HistoryQuickProvider : public HistoryProvider {
 
   virtual void DeleteMatch(const AutocompleteMatch& match) OVERRIDE;
 
-  // Performs the autocomplete matching and scoring.
-  void DoAutocomplete();
+  // Disable this provider. For unit testing purposes only. This is required
+  // because this provider is closely associated with the HistoryURLProvider
+  // and in order to properly test the latter the HistoryQuickProvider must
+  // be disabled.
+  // TODO(mrossetti): Eliminate this once the HUP has been refactored.
+  static void set_disabled(bool disabled) { disabled_ = disabled; }
 
  private:
   friend class HistoryQuickProviderTest;
+  FRIEND_TEST_ALL_PREFIXES(HistoryQuickProviderTest, Spans);
+  FRIEND_TEST_ALL_PREFIXES(HistoryQuickProviderTest, Relevance);
 
+  // Performs the autocomplete matching and scoring.
+  void DoAutocomplete();
+
+  // Creates an AutocompleteMatch from |history_match|. |max_match_score| gives
+  // the maximum possible score for the match.
   AutocompleteMatch QuickMatchToACMatch(
       const history::ScoredHistoryMatch& history_match,
-      MatchType match_type,
-      size_t match_number);
+      bool prevent_inline_autocomplete,
+      int* max_match_score);
 
-  // Breaks a string down into individual words and return as a vector with
-  // the individual words in their original order.
-  static history::InMemoryURLIndex::String16Vector WordVectorFromString16(
-      const string16& uni_string);
-
-  // Determines the relevance for some input, given its type and which match it
-  // is.  If |match_type| is NORMAL, |match_number| is a number
-  // [0, kMaxSuggestions) indicating the relevance of the match (higher == more
-  // relevant).  For other values of |match_type|, |match_number| is ignored.
-  static int CalculateRelevance(int raw_score,
-                                AutocompleteInput::Type input_type,
-                                MatchType match_type,
-                                size_t match_number);
+  // Determines the relevance score of |history_match|. The maximum allowed
+  // score for the match is passed in |max_match_score|. The |max_match_score|
+  // is always set to the resulting score minus 1 whenever the match's score
+  // has to be limited or is <= to |max_match_score|. This function should be
+  // called in a loop with each match in decreasing order of raw score.
+  static int CalculateRelevance(
+      const history::ScoredHistoryMatch& history_match,
+      int* max_match_score);
 
   // Returns the index that should be used for history lookups.
   history::InMemoryURLIndex* GetIndex();
 
+  // Fill and return an ACMatchClassifications structure given the term
+  // matches (|matches|) to highlight where terms were found.
+  static ACMatchClassifications SpansFromTermMatch(
+      const history::TermMatches& matches,
+      size_t text_length,
+      bool is_url);
+
   // Only for use in unittests.  Takes ownership of |index|.
-  void SetIndexForTesting(history::InMemoryURLIndex* index);
+  void set_index(history::InMemoryURLIndex* index) {
+    index_for_testing_.reset(index);
+  }
+
   AutocompleteInput autocomplete_input_;
-  bool trim_http_;
   std::string languages_;
 
   // Only used for testing.
   scoped_ptr<history::InMemoryURLIndex> index_for_testing_;
+
+  // This provider is disabled when true.
+  static bool disabled_;
 };
 
 #endif  // CHROME_BROWSER_AUTOCOMPLETE_HISTORY_QUICK_PROVIDER_H_
