@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -6,7 +6,7 @@
 // is responsible for remembering/suggesting user "search keyword queries"
 // (e.g.  "imdb Godzilla") and then fixing them up into valid URLs.  An
 // instance of it gets created and managed by the autocomplete controller.
-// KeywordProvider uses a TemplateURLModel to find the set of keywords.
+// KeywordProvider uses a TemplateURLService to find the set of keywords.
 //
 // For more information on the autocomplete system in general, including how
 // the autocomplete controller and autocomplete providers work, see
@@ -19,12 +19,12 @@
 #include <string>
 
 #include "chrome/browser/autocomplete/autocomplete.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 class Profile;
 class TemplateURL;
-class TemplateURLModel;
+class TemplateURLService;
 
 // Autocomplete provider for keyword input.
 //
@@ -49,19 +49,19 @@ class TemplateURLModel;
 // but no search terms, the suggested result is shown greyed out, with
 // "<enter term(s)>" as the substituted input, and does nothing when selected.
 class KeywordProvider : public AutocompleteProvider,
-                        public NotificationObserver {
+                        public content::NotificationObserver {
  public:
   KeywordProvider(ACProviderListener* listener, Profile* profile);
   // For testing.
-  KeywordProvider(ACProviderListener* listener, TemplateURLModel* model);
+  KeywordProvider(ACProviderListener* listener, TemplateURLService* model);
 
   // Returns the replacement string from the user input. The replacement
   // string is the portion of the input that does not contain the keyword.
   // For example, the replacement string for "b blah" is blah.
   // If |trim_leading_whitespace| is true then leading whitespace in
   // replacement string will be trimmed.
-  static std::wstring SplitReplacementStringFromInput(
-      const std::wstring& input,
+  static string16 SplitReplacementStringFromInput(
+      const string16& input,
       bool trim_leading_whitespace);
 
   // Returns the matching substituting keyword for |input|, or NULL if there
@@ -69,11 +69,12 @@ class KeywordProvider : public AutocompleteProvider,
   static const TemplateURL* GetSubstitutingTemplateURLForInput(
       Profile* profile,
       const AutocompleteInput& input,
-      std::wstring* remaining_input);
+      string16* remaining_input);
 
   // AutocompleteProvider
-  virtual void Start(const AutocompleteInput& input, bool minimal_changes);
-  virtual void Stop();
+  virtual void Start(const AutocompleteInput& input,
+                     bool minimal_changes) OVERRIDE;
+  virtual void Stop() OVERRIDE;
 
  private:
   class ScopedEndExtensionKeywordMode;
@@ -85,26 +86,27 @@ class KeywordProvider : public AutocompleteProvider,
   // after the keyword are placed in |remaining_input|. Returns true if |input|
   // is valid and has a keyword. This makes use of SplitKeywordFromInput to
   // extract the keyword and remaining string, and uses
-  // TemplateURLModel::CleanUserInputKeyword to remove unnecessary characters.
+  // TemplateURLService::CleanUserInputKeyword to remove unnecessary characters.
   // In general use this instead of SplitKeywordFromInput.
   // Leading whitespace in |*remaining_input| will be trimmed.
   static bool ExtractKeywordFromInput(const AutocompleteInput& input,
-                                      std::wstring* keyword,
-                                      std::wstring* remaining_input);
+                                      string16* keyword,
+                                      string16* remaining_input);
 
   // Extracts the next whitespace-delimited token from input and returns it.
   // Sets |remaining_input| to everything after the first token (skipping over
   // the first intervening whitespace).
   // If |trim_leading_whitespace| is true then leading whitespace in
   // |*remaining_input| will be trimmed.
-  static std::wstring SplitKeywordFromInput(const std::wstring& input,
-                                            bool trim_leading_whitespace,
-                                            std::wstring* remaining_input);
+  static string16 SplitKeywordFromInput(const string16& input,
+                                        bool trim_leading_whitespace,
+                                        string16* remaining_input);
 
   // Fills in the "destination_url" and "contents" fields of |match| with the
   // provided user input and keyword data.
   static void FillInURLAndContents(
-      const std::wstring& remaining_input,
+      Profile* profile,
+      const string16& remaining_input,
       const TemplateURL* element,
       AutocompleteMatch* match);
 
@@ -122,24 +124,24 @@ class KeywordProvider : public AutocompleteProvider,
   // Creates a fully marked-up AutocompleteMatch from the user's input.
   // If |relevance| is negative, calculate a relevance based on heuristics.
   AutocompleteMatch CreateAutocompleteMatch(
-      TemplateURLModel* model,
-      const std::wstring& keyword,
+      TemplateURLService* model,
+      const string16& keyword,
       const AutocompleteInput& input,
       size_t prefix_length,
-      const std::wstring& remaining_input,
+      const string16& remaining_input,
       int relevance);
 
   void EnterExtensionKeywordMode(const std::string& extension_id);
   void MaybeEndExtensionKeywordMode();
 
-  // NotificationObserver interface.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver interface.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Model for the keywords.  This is only non-null when testing, otherwise the
-  // TemplateURLModel from the Profile is used.
-  TemplateURLModel* model_;
+  // TemplateURLService from the Profile is used.
+  TemplateURLService* model_;
 
   // Identifies the current input state. This is incremented each time the
   // autocomplete edit's input changes in any way. It is used to tell whether
@@ -158,7 +160,7 @@ class KeywordProvider : public AutocompleteProvider,
   // the URL bar while the autocomplete popup is open.
   std::string current_keyword_extension_id_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(KeywordProvider);
 };
