@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -20,16 +20,19 @@
 #include <string>
 #include <vector>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/autocomplete/autocomplete.h"
 #include "chrome/browser/autocomplete/autocomplete_match.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/search_engines/template_url.h"
 #include "chrome/browser/search_engines/template_url_id.h"
-#include "chrome/common/net/url_fetcher.h"
+#include "content/public/common/url_fetcher_delegate.h"
 
 class Profile;
+
+namespace base {
 class Value;
+}
 
 // Autocomplete provider for searches and suggestions from a search engine.
 //
@@ -42,7 +45,7 @@ class Value;
 // comes back, the provider creates and returns matches for the best
 // suggestions.
 class SearchProvider : public AutocompleteProvider,
-                       public URLFetcher::Delegate {
+                       public content::URLFetcherDelegate {
  public:
   SearchProvider(ACProviderListener* listener, Profile* profile);
 
@@ -59,21 +62,16 @@ class SearchProvider : public AutocompleteProvider,
   //
   // This method also marks the search provider as no longer needing to wait for
   // the instant result.
-  void FinalizeInstantQuery(const std::wstring& input_text,
-                            const std::wstring& suggest_text);
+  void FinalizeInstantQuery(const string16& input_text,
+                            const string16& suggest_text);
 
   // AutocompleteProvider
   virtual void Start(const AutocompleteInput& input,
-                     bool minimal_changes);
-  virtual void Stop();
+                     bool minimal_changes) OVERRIDE;
+  virtual void Stop() OVERRIDE;
 
-  // URLFetcher::Delegate
-  virtual void OnURLFetchComplete(const URLFetcher* source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  // content::URLFetcherDelegate
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
   // ID used in creating URLFetcher for default provider's suggest results.
   static const int kDefaultProviderURLFetcherID;
@@ -82,7 +80,7 @@ class SearchProvider : public AutocompleteProvider,
   static const int kKeywordProviderURLFetcherID;
 
  private:
-  ~SearchProvider();
+  virtual ~SearchProvider();
 
   // Manages the providers (TemplateURLs) used by SearchProvider. Two providers
   // may be used:
@@ -152,7 +150,7 @@ class SearchProvider : public AutocompleteProvider,
   };
 
   struct NavigationResult {
-    NavigationResult(const GURL& url, const std::wstring& site_name)
+    NavigationResult(const GURL& url, const string16& site_name)
         : url(url),
           site_name(site_name) {
     }
@@ -161,13 +159,17 @@ class SearchProvider : public AutocompleteProvider,
     GURL url;
 
     // Name for the site.
-    std::wstring site_name;
+    string16 site_name;
   };
 
-  typedef std::vector<std::wstring> SuggestResults;
+  typedef std::vector<string16> SuggestResults;
   typedef std::vector<NavigationResult> NavigationResults;
   typedef std::vector<history::KeywordSearchTermVisit> HistoryResults;
-  typedef std::map<std::wstring, AutocompleteMatch> MatchMap;
+  typedef std::map<string16, AutocompleteMatch> MatchMap;
+  typedef std::pair<string16, int> ScoredTerm;
+  typedef std::vector<ScoredTerm> ScoredTerms;
+
+  class CompareScoredTerms;
 
   // Called when timer_ expires.
   void Run();
@@ -192,15 +194,15 @@ class SearchProvider : public AutocompleteProvider,
 
   // Creates a URLFetcher requesting suggest results for the specified
   // TemplateURL. Ownership of the returned URLFetchet passes to the caller.
-  URLFetcher* CreateSuggestFetcher(int id,
-                                   const TemplateURL& provider,
-                                   const std::wstring& text);
+  content::URLFetcher* CreateSuggestFetcher(int id,
+                                            const TemplateURL& provider,
+                                            const string16& text);
 
   // Parses the results from the Suggest server and stores up to kMaxMatches of
   // them in server_results_.  Returns whether parsing succeeded.
-  bool ParseSuggestResults(Value* root_val,
+  bool ParseSuggestResults(base::Value* root_val,
                            bool is_keyword,
-                           const std::wstring& input_text,
+                           const string16& input_text,
                            SuggestResults* suggest_results);
 
   // Converts the parsed server results in server_results_ to a set of
@@ -221,6 +223,13 @@ class SearchProvider : public AutocompleteProvider,
                               int did_not_accept_suggestion,
                               MatchMap* map);
 
+  // Calculates relevance scores for all |results|.
+  ScoredTerms ScoreHistoryTerms(const HistoryResults& results,
+                                bool base_prevent_inline_autocomplete,
+                                bool input_multiple_words,
+                                const string16& input_text,
+                                bool is_keyword);
+
   // Adds a match for each result in |suggest_results| to |map|. |is_keyword|
   // indicates whether the results correspond to the keyword provider or default
   // provider.
@@ -232,10 +241,13 @@ class SearchProvider : public AutocompleteProvider,
   // Determines the relevance for a particular match.  We use different scoring
   // algorithms for the different types of matches.
   int CalculateRelevanceForWhatYouTyped() const;
-  // |time| is the time at which this query was last seen. |is_keyword| is true
-  // if the search is from the keyword provider.
+  // |time| is the time at which this query was last seen.  |is_keyword|
+  // indicates whether the results correspond to the keyword provider or default
+  // provider. |prevent_inline_autocomplete| is true if we should not inline
+  // autocomplete this query.
   int CalculateRelevanceForHistory(const base::Time& time,
-                                   bool is_keyword) const;
+                                   bool is_keyword,
+                                   bool prevent_inline_autocomplete) const;
   // |result_number| is the index of the suggestion in the result set from the
   // server; the best suggestion is suggestion number 0.  |is_keyword| is true
   // if the search is from the keyword provider.
@@ -251,8 +263,8 @@ class SearchProvider : public AutocompleteProvider,
   // Creates an AutocompleteMatch for "Search <engine> for |query_string|" with
   // the supplied relevance.  Adds this match to |map|; if such a match already
   // exists, whichever one has lower relevance is eliminated.
-  void AddMatchToMap(const std::wstring& query_string,
-                     const std::wstring& input_text,
+  void AddMatchToMap(const string16& query_string,
+                     const string16& input_text,
                      int relevance,
                      AutocompleteMatch::Type type,
                      int accepted_suggestion,
@@ -268,9 +280,6 @@ class SearchProvider : public AutocompleteProvider,
   // Updates the value of |done_| from the internal state.
   void UpdateDone();
 
-  // Updates the description/description_class of the first search match.
-  void UpdateFirstSearchMatchDescription();
-
   // Should we query for suggest results immediately? This is normally false,
   // but may be set to true during testing.
   static bool query_suggest_immediately_;
@@ -282,7 +291,7 @@ class SearchProvider : public AutocompleteProvider,
   AutocompleteInput input_;
 
   // Input text when searching against the keyword provider.
-  std::wstring keyword_input_text_;
+  string16 keyword_input_text_;
 
   // Searches in the user's history that begin with the input text.
   HistoryResults keyword_history_results_;
@@ -297,11 +306,11 @@ class SearchProvider : public AutocompleteProvider,
   base::OneShotTimer<SearchProvider> timer_;
 
   // The fetcher that retrieves suggest results for the keyword from the server.
-  scoped_ptr<URLFetcher> keyword_fetcher_;
+  scoped_ptr<content::URLFetcher> keyword_fetcher_;
 
   // The fetcher that retrieves suggest results for the default engine from the
   // server.
-  scoped_ptr<URLFetcher> default_fetcher_;
+  scoped_ptr<content::URLFetcher> default_fetcher_;
 
   // Suggestions returned by the Suggest server for the input text.
   SuggestResults keyword_suggest_results_;
@@ -318,7 +327,7 @@ class SearchProvider : public AutocompleteProvider,
   bool instant_finalized_;
 
   // The |suggest_text| parameter passed to FinalizeInstantQuery.
-  std::wstring default_provider_suggest_text_;
+  string16 default_provider_suggest_text_;
 
   DISALLOW_COPY_AND_ASSIGN(SearchProvider);
 };
