@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,16 +9,20 @@
 #include "base/string16.h"
 #include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/autofill/autofill_external_delegate.h"
 #include "chrome/browser/autofill/credit_card.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/common/autofill_messages.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/common/render_messages.h"
-#include "webkit/glue/form_data.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/web_contents.h"
+#include "webkit/forms/form_data.h"
 
-using webkit_glue::FormData;
+using base::StringPiece16;
+using content::WebContents;
+using webkit::forms::FormData;
+using webkit::forms::FormField;
 
 namespace {
 
@@ -27,60 +31,92 @@ namespace {
 const int kMaxAutocompleteMenuItems = 6;
 
 // The separator characters for SSNs.
-const string16 kSSNSeparators = ASCIIToUTF16(" -");
+const char16 kSSNSeparators[] = {' ', '-', 0};
 
 bool IsSSN(const string16& text) {
   string16 number_string;
-  RemoveChars(text, kSSNSeparators.c_str(), &number_string);
+  RemoveChars(text, kSSNSeparators, &number_string);
 
   // A SSN is of the form AAA-GG-SSSS (A = area number, G = group number, S =
   // serial number). The validation we do here is simply checking if the area,
-  // group, and serial numbers are valid. It is possible to check if the group
-  // number is valid for the given area, but that data changes all the time.
+  // group, and serial numbers are valid.
   //
-  // See: http://www.socialsecurity.gov/history/ssn/geocard.html
-  //      http://www.socialsecurity.gov/employer/stateweb.htm
-  //      http://www.socialsecurity.gov/employer/ssnvhighgroup.htm
+  // Historically, the area number was assigned per state, with the group number
+  // ascending in an alternating even/odd sequence. With that scheme it was
+  // possible to check for validity by referencing a table that had the highest
+  // group number assigned for a given area number. (This was something that
+  // Chromium never did though, because the "high group" values were constantly
+  // changing.)
+  //
+  // However, starting on 25 June 2011 the SSA began issuing SSNs randomly from
+  // all areas and groups. Group numbers and serial numbers of zero remain
+  // invalid, and areas 000, 666, and 900-999 remain invalid.
+  //
+  // References for current practices:
+  //   http://www.socialsecurity.gov/employer/randomization.html
+  //   http://www.socialsecurity.gov/employer/randomizationfaqs.html
+  //
+  // References for historic practices:
+  //   http://www.socialsecurity.gov/history/ssn/geocard.html
+  //   http://www.socialsecurity.gov/employer/stateweb.htm
+  //   http://www.socialsecurity.gov/employer/ssnvhighgroup.htm
+
   if (number_string.length() != 9 || !IsStringASCII(number_string))
     return false;
 
   int area;
-  if (!base::StringToInt(number_string.begin(),
-                         number_string.begin() + 3,
-                         &area))
+  if (!base::StringToInt(StringPiece16(number_string.begin(),
+                                       number_string.begin() + 3),
+                         &area)) {
     return false;
+  }
   if (area < 1 ||
       area == 666 ||
-      (area > 733 && area < 750) ||
-      area > 772)
+      area >= 900) {
     return false;
+  }
 
   int group;
-  if (!base::StringToInt(number_string.begin() + 3,
-                         number_string.begin() + 5,
-                         &group) || group == 0)
+  if (!base::StringToInt(StringPiece16(number_string.begin() + 3,
+                                       number_string.begin() + 5),
+                         &group)
+      || group == 0) {
     return false;
+  }
 
   int serial;
-  if (!base::StringToInt(number_string.begin() + 5,
-                         number_string.begin() + 9,
-                         &serial) || serial == 0)
+  if (!base::StringToInt(StringPiece16(number_string.begin() + 5,
+                                       number_string.begin() + 9),
+                         &serial)
+      || serial == 0) {
     return false;
+  }
 
   return true;
 }
 
+bool IsTextField(const FormField& field) {
+  return
+      field.form_control_type == ASCIIToUTF16("text") ||
+      field.form_control_type == ASCIIToUTF16("search") ||
+      field.form_control_type == ASCIIToUTF16("tel") ||
+      field.form_control_type == ASCIIToUTF16("url") ||
+      field.form_control_type == ASCIIToUTF16("email") ||
+      field.form_control_type == ASCIIToUTF16("text");
+}
+
 }  // namespace
 
 AutocompleteHistoryManager::AutocompleteHistoryManager(
-    TabContents* tab_contents)
-    : tab_contents_(tab_contents),
+    WebContents* web_contents)
+    : content::WebContentsObserver(web_contents),
       pending_query_handle_(0),
-      query_id_(0) {
-  profile_ = tab_contents_->profile();
+      query_id_(0),
+      external_delegate_(NULL) {
+  profile_ = Profile::FromBrowserContext(web_contents->GetBrowserContext());
   // May be NULL in unit tests.
   web_data_service_ = profile_->GetWebDataService(Profile::EXPLICIT_ACCESS);
-  autofill_enabled_.Init(prefs::kAutoFillEnabled, profile_->GetPrefs(), NULL);
+  autofill_enabled_.Init(prefs::kAutofillEnabled, profile_->GetPrefs(), NULL);
 }
 
 AutocompleteHistoryManager::~AutocompleteHistoryManager() {
@@ -91,7 +127,7 @@ bool AutocompleteHistoryManager::OnMessageReceived(
     const IPC::Message& message) {
   bool handled = true;
   IPC_BEGIN_MESSAGE_MAP(AutocompleteHistoryManager, message)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_RemoveAutocompleteEntry,
+    IPC_MESSAGE_HANDLER(AutofillHostMsg_RemoveAutocompleteEntry,
                         OnRemoveAutocompleteEntry)
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
@@ -115,16 +151,17 @@ void AutocompleteHistoryManager::OnFormSubmitted(const FormData& form) {
   //  - text field
   //  - value is not a credit card number
   //  - value is not a SSN
-  std::vector<webkit_glue::FormField> values;
-  for (std::vector<webkit_glue::FormField>::const_iterator iter =
+  std::vector<FormField> values;
+  for (std::vector<FormField>::const_iterator iter =
            form.fields.begin();
        iter != form.fields.end(); ++iter) {
-    if (!iter->value().empty() &&
-        !iter->name().empty() &&
-        iter->form_control_type() == ASCIIToUTF16("text") &&
-        !CreditCard::IsCreditCardNumber(iter->value()) &&
-        !IsSSN(iter->value()))
+    if (!iter->value.empty() &&
+        !iter->name.empty() &&
+        IsTextField(*iter) &&
+        !CreditCard::IsValidCreditCardNumber(iter->value) &&
+        !IsSSN(iter->value)) {
       values.push_back(*iter);
+    }
   }
 
   if (!values.empty() && web_data_service_.get())
@@ -183,21 +220,30 @@ void AutocompleteHistoryManager::OnWebDataServiceRequestDone(
     return;
   }
 
-  DCHECK(result->GetType() == AUTOFILL_VALUE_RESULT);
+  DCHECK_EQ(AUTOFILL_VALUE_RESULT, result->GetType());
   const WDResult<std::vector<string16> >* autofill_result =
       static_cast<const WDResult<std::vector<string16> >*>(result);
   std::vector<string16> suggestions = autofill_result->GetValue();
   SendSuggestions(&suggestions);
 }
 
+void AutocompleteHistoryManager::SetExternalDelegate(
+    AutofillExternalDelegate* delegate) {
+  external_delegate_ = delegate;
+}
+
 AutocompleteHistoryManager::AutocompleteHistoryManager(
-    Profile* profile, WebDataService* wds) : tab_contents_(NULL),
-                                             profile_(profile),
-                                             web_data_service_(wds),
-                                             pending_query_handle_(0),
-                                             query_id_(0) {
+    WebContents* web_contents,
+    Profile* profile,
+    WebDataService* wds)
+    : content::WebContentsObserver(web_contents),
+      profile_(profile),
+      web_data_service_(wds),
+      pending_query_handle_(0),
+      query_id_(0),
+      external_delegate_(NULL) {
   autofill_enabled_.Init(
-      prefs::kAutoFillEnabled, profile_->GetPrefs(), NULL);
+      prefs::kAutofillEnabled, profile_->GetPrefs(), NULL);
 }
 
 void AutocompleteHistoryManager::CancelPendingQuery() {
@@ -212,7 +258,7 @@ void AutocompleteHistoryManager::CancelPendingQuery() {
 void AutocompleteHistoryManager::SendSuggestions(
     const std::vector<string16>* suggestions) {
   if (suggestions) {
-    // Combine AutoFill and Autocomplete values into values and labels.
+    // Combine Autofill and Autocomplete values into values and labels.
     for (size_t i = 0; i < suggestions->size(); ++i) {
       bool unique = true;
       for (size_t j = 0; j < autofill_values_.size(); ++j) {
@@ -232,14 +278,20 @@ void AutocompleteHistoryManager::SendSuggestions(
     }
   }
 
-  RenderViewHost* host = tab_contents_->render_view_host();
-  if (host) {
-    host->Send(new ViewMsg_AutoFillSuggestionsReturned(host->routing_id(),
-                                                       query_id_,
-                                                       autofill_values_,
-                                                       autofill_labels_,
-                                                       autofill_icons_,
-                                                       autofill_unique_ids_));
+  if (external_delegate_) {
+    external_delegate_->OnSuggestionsReturned(
+        query_id_,
+        autofill_values_,
+        autofill_labels_,
+        autofill_icons_,
+        autofill_unique_ids_);
+  } else {
+    Send(new AutofillMsg_SuggestionsReturned(routing_id(),
+                                             query_id_,
+                                             autofill_values_,
+                                             autofill_labels_,
+                                             autofill_icons_,
+                                             autofill_unique_ids_));
   }
 
   query_id_ = 0;
