@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,15 +6,17 @@
 #define CHROME_BROWSER_AUTOFILL_AUTOFILL_COMMON_TEST_H_
 #pragma once
 
-class AutoFillProfile;
+class AutofillProfile;
 class CreditCard;
 class Profile;
 
-namespace webkit_glue {
-class FormField;
-}  // namespace webkit_glue
+namespace webkit {
+namespace forms {
+struct FormField;
+}
+}
 
-// Common utilities shared amongst AutoFill tests.
+// Common utilities shared amongst Autofill tests.
 namespace autofill_test {
 
 // Provides a quick way to populate a FormField with c-strings.
@@ -22,30 +24,30 @@ void CreateTestFormField(const char* label,
                          const char* name,
                          const char* value,
                          const char* type,
-                         webkit_glue::FormField* field);
+                         webkit::forms::FormField* field);
 
-// A unit testing utility that is common to a number of the AutoFill unit
+// A unit testing utility that is common to a number of the Autofill unit
 // tests.  |SetProfileInfo| provides a quick way to populate a profile with
 // c-strings.
-void SetProfileInfo(AutoFillProfile* profile,
-    const char* label, const char* first_name, const char* middle_name,
+void SetProfileInfo(AutofillProfile* profile,
+    const char* first_name, const char* middle_name,
     const char* last_name, const char* email, const char* company,
     const char* address1, const char* address2, const char* city,
     const char* state, const char* zipcode, const char* country,
-    const char* phone, const char* fax);
+    const char* phone);
 
-void SetProfileInfoWithGuid(AutoFillProfile* profile,
+void SetProfileInfoWithGuid(AutofillProfile* profile,
     const char* guid, const char* first_name, const char* middle_name,
     const char* last_name, const char* email, const char* company,
     const char* address1, const char* address2, const char* city,
     const char* state, const char* zipcode, const char* country,
-    const char* phone, const char* fax);
+    const char* phone);
 
-// A unit testing utility that is common to a number of the AutoFill unit
+// A unit testing utility that is common to a number of the Autofill unit
 // tests.  |SetCreditCardInfo| provides a quick way to populate a credit card
 // with c-strings.
 void SetCreditCardInfo(CreditCard* credit_card,
-    const char* label, const char* name_on_card, const char* card_number,
+    const char* name_on_card, const char* card_number,
     const char* expiration_month, const char* expiration_year);
 
 // TODO(isherman): We should do this automatically for all tests, not manually
