@@ -6,101 +6,83 @@
 #define CHROME_BROWSER_AUTOFILL_AUTOFILL_DOWNLOAD_H_
 #pragma once
 
+#include <stddef.h>
 #include <list>
 #include <map>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "base/scoped_vector.h"
+#include "base/compiler_specific.h"
+#include "base/gtest_prod_util.h"
 #include "base/time.h"
-#include "chrome/browser/autofill/autofill_profile.h"
-#include "chrome/browser/autofill/field_types.h"
-#include "chrome/browser/autofill/form_structure.h"
-#include "chrome/common/net/url_fetcher.h"
+#include "chrome/browser/autofill/autofill_type.h"
+#include "content/public/common/url_fetcher_delegate.h"
 
-class AutoFillMetrics;
+class AutofillMetrics;
+class FormStructure;
 class Profile;
 
-// Handles getting and updating AutoFill heuristics.
-class AutoFillDownloadManager : public URLFetcher::Delegate {
+// Handles getting and updating Autofill heuristics.
+class AutofillDownloadManager : public content::URLFetcherDelegate {
  public:
-  enum AutoFillRequestType {
+  enum AutofillRequestType {
     REQUEST_QUERY,
     REQUEST_UPLOAD,
   };
 
-  // An interface used to notify clients of AutoFillDownloadManager.
-  // Notifications are *not* guaranteed to be called.
+  // An interface used to notify clients of AutofillDownloadManager.
   class Observer {
    public:
-    // Called when field types are successfully received from the server.
-    // |heuristic_xml| - server response.
-    virtual void OnLoadedAutoFillHeuristics(
-        const std::string& heuristic_xml) = 0;
+    // Called when field type predictions are successfully received from the
+    // server.  |response_xml| contains the server response.
+    virtual void OnLoadedServerPredictions(const std::string& response_xml) = 0;
+
+    // These notifications are used to help with testing.
     // Called when heuristic either successfully considered for upload and
     // not send or uploaded.
-    // |form_signature| - the signature of the requesting form.
-    virtual void OnUploadedAutoFillHeuristics(
-        const std::string& form_signature) = 0;
+    virtual void OnUploadedPossibleFieldTypes() {}
     // Called when there was an error during the request.
     // |form_signature| - the signature of the requesting form.
     // |request_type| - type of request that failed.
     // |http_error| - HTTP error code.
-    virtual void OnHeuristicsRequestError(const std::string& form_signature,
-                                          AutoFillRequestType request_type,
-                                          int http_error) = 0;
+    virtual void OnServerRequestError(const std::string& form_signature,
+                                      AutofillRequestType request_type,
+                                      int http_error) {}
+
    protected:
     virtual ~Observer() {}
   };
 
-  // |profile| can be NULL in unit-tests only.
-  explicit AutoFillDownloadManager(Profile* profile);
-  virtual ~AutoFillDownloadManager();
-
   // |observer| - observer to notify on successful completion or error.
-  void SetObserver(AutoFillDownloadManager::Observer *observer);
+  AutofillDownloadManager(Profile* profile, Observer* observer);
+  virtual ~AutofillDownloadManager();
 
-  // Starts a query request to AutoFill servers. The observer is called with the
+  // Starts a query request to Autofill servers. The observer is called with the
   // list of the fields of all requested forms.
   // |forms| - array of forms aggregated in this request.
-  bool StartQueryRequest(const ScopedVector<FormStructure>& forms,
-                         const AutoFillMetrics& metric_logger);
+  bool StartQueryRequest(const std::vector<FormStructure*>& forms,
+                         const AutofillMetrics& metric_logger);
 
-  // Start upload request if necessary. The probability of request going
-  // over the wire are GetPositiveUploadRate() if it was matched by
-  // AutoFill, GetNegativeUploadRate() otherwise. Observer will be called
-  // even if there was no actual trip over the wire.
-  // |form| - form sent in this request.
-  // |form_was_matched| - true if form was matched by the AutoFill.
-  bool StartUploadRequest(const FormStructure& form, bool form_was_matched);
-
-  // Cancels pending request.
-  // |form_signature| - signature of the form being cancelled. Warning:
-  // for query request if more than one form sent in the request, all other
-  // forms will be cancelled as well.
-  // |request_type| - type of the request.
-  bool CancelRequest(const std::string& form_signature,
-                     AutoFillRequestType request_type);
-
-  // Probability of the form upload. Between 0 (no upload) and 1 (upload all).
-  // GetPositiveUploadRate() is for matched forms,
-  // GetNegativeUploadRate() for non matched.
-  double GetPositiveUploadRate() const;
-  double GetNegativeUploadRate() const;
-  // These functions called very rarely outside of theunit-tests. With current
-  // percentages, they would be called once per 100 auto-fillable forms filled
-  // and submitted by user. The order of magnitude would remain similar in the
-  // future.
-  void SetPositiveUploadRate(double rate);
-  void SetNegativeUploadRate(double rate);
+  // Starts an upload request for the given |form|, unless throttled by the
+  // server. The probability of the request going over the wire is
+  // GetPositiveUploadRate() if |form_was_autofilled| is true, or
+  // GetNegativeUploadRate() otherwise. The observer will be called even if
+  // there was no actual trip over the wire.
+  // |available_field_types| should contain the types for which we have data
+  // stored on the local client.
+  bool StartUploadRequest(const FormStructure& form,
+                          bool form_was_autofilled,
+                          const FieldTypeSet& available_field_types);
 
  private:
-  friend class AutoFillDownloadTestHelper;  // unit-test.
+  friend class AutofillDownloadTest;
+  FRIEND_TEST_ALL_PREFIXES(AutofillDownloadTest, QueryAndUploadTest);
 
   struct FormRequestData;
   typedef std::list<std::pair<std::string, std::string> > QueryRequestCache;
 
-  // Initiates request to AutoFill servers to download/upload heuristics.
+  // Initiates request to Autofill servers to download/upload heuristics.
   // |form_xml| - form structure XML to upload/download.
   // |request_data| - form signature hash(es) and indicator if it was a query.
   // |request_data.query| - if true the data is queried and observer notified
@@ -128,22 +110,29 @@ class AutoFillDownloadManager : public URLFetcher::Delegate {
   std::string GetCombinedSignature(
       const std::vector<std::string>& forms_in_query) const;
 
-  // URLFetcher::Delegate implementation:
-  virtual void OnURLFetchComplete(const URLFetcher* source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  // content::URLFetcherDelegate implementation:
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
-  // Profile for preference storage.
-  Profile* profile_;
+  // Probability of the form upload. Between 0 (no upload) and 1 (upload all).
+  // GetPositiveUploadRate() is for matched forms,
+  // GetNegativeUploadRate() for non-matched.
+  double GetPositiveUploadRate() const;
+  double GetNegativeUploadRate() const;
+  void SetPositiveUploadRate(double rate);
+  void SetNegativeUploadRate(double rate);
+
+  // Profile for preference storage.  The pointer value is const, so this can
+  // only be set in the constructor.  Must not be null.
+  Profile* const profile_;  // WEAK
+  // The observer to notify when server predictions are successfully received.
+  // The pointer value is const, so this can only be set in the constructor.
+  // Must not be null.
+  AutofillDownloadManager::Observer* const observer_;  // WEAK
 
   // For each requested form for both query and upload we create a separate
   // request and save its info. As url fetcher is identified by its address
   // we use a map between fetchers and info.
-  std::map<URLFetcher*, FormRequestData> url_fetchers_;
-  AutoFillDownloadManager::Observer *observer_;
+  std::map<content::URLFetcher*, FormRequestData> url_fetchers_;
 
   // Cached QUERY requests.
   QueryRequestCache cached_forms_;
@@ -162,8 +151,6 @@ class AutoFillDownloadManager : public URLFetcher::Delegate {
 
   // Needed for unit-test.
   int fetcher_id_for_unittest_;
-  bool is_testing_;
 };
 
 #endif  // CHROME_BROWSER_AUTOFILL_AUTOFILL_DOWNLOAD_H_
-
