@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -25,22 +25,24 @@ static std::string Hash32Bit(const std::string& str) {
 
 }  // namespace
 
-AutoFillField::AutoFillField()
+AutofillField::AutofillField()
     : server_type_(NO_SERVER_DATA),
-      heuristic_type_(UNKNOWN_TYPE) {
+      heuristic_type_(UNKNOWN_TYPE),
+      phone_part_(IGNORED) {
 }
 
-AutoFillField::AutoFillField(const webkit_glue::FormField& field,
+AutofillField::AutofillField(const webkit::forms::FormField& field,
                              const string16& unique_name)
-    : webkit_glue::FormField(field),
+    : webkit::forms::FormField(field),
       unique_name_(unique_name),
       server_type_(NO_SERVER_DATA),
-      heuristic_type_(UNKNOWN_TYPE) {
+      heuristic_type_(UNKNOWN_TYPE),
+      phone_part_(IGNORED) {
 }
 
-AutoFillField::~AutoFillField() {}
+AutofillField::~AutofillField() {}
 
-void AutoFillField::set_heuristic_type(const AutoFillFieldType& type) {
+void AutofillField::set_heuristic_type(AutofillFieldType type) {
   if (type >= 0 && type < MAX_VALID_FIELD_TYPE) {
     heuristic_type_ = type;
   } else {
@@ -51,24 +53,32 @@ void AutoFillField::set_heuristic_type(const AutoFillFieldType& type) {
   }
 }
 
-AutoFillFieldType AutoFillField::type() const {
+void AutofillField::set_server_type(AutofillFieldType type) {
+  // Chrome no longer supports fax numbers, but the server still does.
+  if (type >= PHONE_FAX_NUMBER && type <= PHONE_FAX_WHOLE_NUMBER)
+    return;
+
+  server_type_ = type;
+}
+
+AutofillFieldType AutofillField::type() const {
   if (server_type_ != NO_SERVER_DATA)
     return server_type_;
 
   return heuristic_type_;
 }
 
-bool AutoFillField::IsEmpty() const {
-  return value().empty();
+bool AutofillField::IsEmpty() const {
+  return value.empty();
 }
 
-std::string AutoFillField::FieldSignature() const {
-  std::string field_name = UTF16ToUTF8(name());
-  std::string type = UTF16ToUTF8(form_control_type());
+std::string AutofillField::FieldSignature() const {
+  std::string field_name = UTF16ToUTF8(name);
+  std::string type = UTF16ToUTF8(form_control_type);
   std::string field_string = field_name + "&" + type;
   return Hash32Bit(field_string);
 }
 
-bool AutoFillField::IsFieldFillable() const {
+bool AutofillField::IsFieldFillable() const {
   return type() != UNKNOWN_TYPE;
 }
