@@ -1,24 +1,35 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/autofill/autofill_ie_toolbar_import_win.h"
 
+#include <stddef.h>
+#include <map>
+#include <string>
+#include <vector>
+
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
+#include "base/logging.h"
 #include "base/string16.h"
 #include "base/win/registry.h"
 #include "chrome/browser/autofill/autofill_profile.h"
 #include "chrome/browser/autofill/credit_card.h"
 #include "chrome/browser/autofill/crypto/rc4_decryptor.h"
 #include "chrome/browser/autofill/field_types.h"
+#include "chrome/browser/autofill/form_group.h"
 #include "chrome/browser/autofill/personal_data_manager.h"
+#include "chrome/browser/autofill/personal_data_manager_observer.h"
+#include "chrome/browser/autofill/phone_number.h"
+#include "chrome/browser/autofill/phone_number_i18n.h"
 #include "chrome/browser/sync/util/data_encryption.h"
 
 using base::win::RegKey;
 
 // Forward declaration. This function is not in unnamed namespace as it
 // is referenced in the unittest.
-bool ImportCurrentUserProfiles(std::vector<AutoFillProfile>* profiles,
+bool ImportCurrentUserProfiles(std::vector<AutofillProfile>* profiles,
                                std::vector<CreditCard>* credit_cards);
 namespace {
 
@@ -30,7 +41,7 @@ const wchar_t* const kPasswordHashValue = L"password_hash";
 const wchar_t* const kSaltValue = L"salt";
 
 // This is RC4 decryption for Toolbar credit card data. This is necessary
-// because it is not standard, so Crypto api cannot be used.
+// because it is not standard, so Crypto API cannot be used.
 std::wstring DecryptCCNumber(const std::wstring& data) {
   const wchar_t* kEmptyKey =
     L"\x3605\xCEE5\xCE49\x44F7\xCF4E\xF6CC\x604B\xFCBE\xC70A\x08FD";
@@ -78,7 +89,7 @@ string16 ReadAndDecryptValue(RegKey* key, const wchar_t* value_name) {
 }
 
 struct {
-  AutoFillFieldType field_type;
+  AutofillFieldType field_type;
   const wchar_t *reg_value_name;
 } profile_reg_values[] = {
   { NAME_FIRST,                    L"name_first" },
@@ -90,9 +101,6 @@ struct {
   { PHONE_HOME_NUMBER,             L"phone_home_number" },
   { PHONE_HOME_CITY_CODE,          L"phone_home_city_code" },
   { PHONE_HOME_COUNTRY_CODE,       L"phone_home_country_code" },
-  { PHONE_FAX_NUMBER,              L"phone_fax_number" },
-  { PHONE_FAX_CITY_CODE,           L"phone_fax_city_code" },
-  { PHONE_FAX_COUNTRY_CODE,        L"phone_fax_country_code" },
   { ADDRESS_HOME_LINE1,            L"address_home_line1" },
   { ADDRESS_HOME_LINE2,            L"address_home_line2" },
   { ADDRESS_HOME_CITY,             L"address_home_city" },
@@ -113,7 +121,7 @@ struct {
   // We do not import verification code.
 };
 
-typedef std::map<std::wstring, AutoFillFieldType> RegToFieldMap;
+typedef std::map<std::wstring, AutofillFieldType> RegToFieldMap;
 
 bool ImportSingleProfile(FormGroup* profile,
                          RegKey* key,
@@ -124,9 +132,12 @@ bool ImportSingleProfile(FormGroup* profile,
 
   bool has_non_empty_fields = false;
 
-  for (uint32 value_index = 0; value_index < key->ValueCount(); ++value_index) {
+  // Phones need to be rebuilt.
+  PhoneNumber::PhoneCombineHelper phone;
+
+  for (uint32 i = 0; i < key->GetValueCount(); ++i) {
     std::wstring value_name;
-    if (key->ReadName(value_index, &value_name) != ERROR_SUCCESS)
+    if (key->GetValueNameAt(i, &value_name) != ERROR_SUCCESS)
       continue;
     RegToFieldMap::const_iterator it = reg_to_field.find(value_name);
     if (it == reg_to_field.end())
@@ -134,20 +145,30 @@ bool ImportSingleProfile(FormGroup* profile,
     string16 field_value = ReadAndDecryptValue(key, value_name.c_str());
     if (!field_value.empty()) {
       has_non_empty_fields = true;
-      if (it->second == CREDIT_CARD_NUMBER) {
+      if (it->second == CREDIT_CARD_NUMBER)
         field_value = DecryptCCNumber(field_value);
-      }
-      profile->SetInfo(AutoFillType(it->second), field_value);
+
+      // We need to store phone data in |phone| before building the whole number
+      // at the end. The rest of the fields are set "as is".
+      if (!phone.SetInfo(it->second, field_value))
+        profile->SetInfo(it->second, field_value);
     }
   }
+  // Now re-construct the phones if needed.
+  string16 constructed_number;
+  if (!phone.IsEmpty() &&
+      phone.ParseNumber(std::string("US"), &constructed_number)) {
+    profile->SetCanonicalizedInfo(PHONE_HOME_WHOLE_NUMBER, constructed_number);
+  }
+
   return has_non_empty_fields;
 }
 
 // Imports profiles from the IE toolbar and stores them. Asynchronous
 // if PersonalDataManager has not been loaded yet. Deletes itself on completion.
-class AutoFillImporter : public PersonalDataManager::Observer {
+class AutofillImporter : public PersonalDataManagerObserver {
  public:
-  explicit AutoFillImporter(PersonalDataManager* personal_data_manager)
+  explicit AutofillImporter(PersonalDataManager* personal_data_manager)
     : personal_data_manager_(personal_data_manager) {
       personal_data_manager_->SetObserver(this);
   }
@@ -158,35 +179,39 @@ class AutoFillImporter : public PersonalDataManager::Observer {
       return false;
     }
     if (personal_data_manager_->IsDataLoaded())
-      OnPersonalDataLoaded();
+      OnPersonalDataChanged();
     return true;
   }
 
-  // PersonalDataManager::Observer methods:
-  virtual void OnPersonalDataLoaded() {
-    if (!profiles_.empty())
-      personal_data_manager_->SetProfiles(&profiles_);
-    if (!credit_cards_.empty())
-      personal_data_manager_->SetCreditCards(&credit_cards_);
+  // PersonalDataManagerObserver:
+  virtual void OnPersonalDataChanged() OVERRIDE {
+    for (std::vector<AutofillProfile>::const_iterator iter = profiles_.begin();
+         iter != profiles_.end(); ++iter) {
+      personal_data_manager_->AddProfile(*iter);
+    }
+    for (std::vector<CreditCard>::const_iterator iter = credit_cards_.begin();
+         iter != credit_cards_.end(); ++iter) {
+      personal_data_manager_->AddCreditCard(*iter);
+    }
     delete this;
   }
 
  private:
-  ~AutoFillImporter() {
+  ~AutofillImporter() {
     personal_data_manager_->RemoveObserver(this);
   }
 
   PersonalDataManager* personal_data_manager_;
-  std::vector<AutoFillProfile> profiles_;
+  std::vector<AutofillProfile> profiles_;
   std::vector<CreditCard> credit_cards_;
 };
 
 }  // namespace
 
-// Imports AutoFill profiles and credit cards from IE Toolbar if present and not
+// Imports Autofill profiles and credit cards from IE Toolbar if present and not
 // password protected. Returns true if data is successfully retrieved. False if
 // there is no data, data is password protected or error occurred.
-bool ImportCurrentUserProfiles(std::vector<AutoFillProfile>* profiles,
+bool ImportCurrentUserProfiles(std::vector<AutofillProfile>* profiles,
                                std::vector<CreditCard>* credit_cards) {
   DCHECK(profiles);
   DCHECK(credit_cards);
@@ -205,18 +230,9 @@ bool ImportCurrentUserProfiles(std::vector<AutoFillProfile>* profiles,
     key_name.append(L"\\");
     key_name.append(iterator_profiles.Name());
     RegKey key(HKEY_CURRENT_USER, key_name.c_str(), KEY_READ);
-    AutoFillProfile profile;
+    AutofillProfile profile;
     if (ImportSingleProfile(&profile, &key, reg_to_field)) {
       // Combine phones into whole phone #.
-      string16 phone;
-      phone = profile.GetFieldText(AutoFillType(PHONE_HOME_COUNTRY_CODE));
-      phone.append(profile.GetFieldText(AutoFillType(PHONE_HOME_CITY_CODE)));
-      phone.append(profile.GetFieldText(AutoFillType(PHONE_HOME_NUMBER)));
-      profile.SetInfo(AutoFillType(PHONE_HOME_WHOLE_NUMBER), phone);
-      phone = profile.GetFieldText(AutoFillType(PHONE_FAX_COUNTRY_CODE));
-      phone.append(profile.GetFieldText(AutoFillType(PHONE_FAX_CITY_CODE)));
-      phone.append(profile.GetFieldText(AutoFillType(PHONE_FAX_NUMBER)));
-      profile.SetInfo(AutoFillType(PHONE_FAX_WHOLE_NUMBER), phone);
       profiles->push_back(profile);
     }
   }
@@ -239,8 +255,7 @@ bool ImportCurrentUserProfiles(std::vector<AutoFillProfile>* profiles,
       RegKey key(HKEY_CURRENT_USER, key_name.c_str(), KEY_READ);
       CreditCard credit_card;
       if (ImportSingleProfile(&credit_card, &key, reg_to_field)) {
-        string16 cc_number = credit_card.GetFieldText(
-            AutoFillType(CREDIT_CARD_NUMBER));
+        string16 cc_number = credit_card.GetInfo(CREDIT_CARD_NUMBER);
         if (!cc_number.empty())
           credit_cards->push_back(credit_card);
       }
@@ -253,7 +268,7 @@ bool ImportAutofillDataWin(PersonalDataManager* pdm) {
   // In incognito mode we do not have PDM - and we should not import anything.
   if (!pdm)
     return false;
-  AutoFillImporter *importer = new AutoFillImporter(pdm);
+  AutofillImporter *importer = new AutofillImporter(pdm);
   // importer will self delete.
   return importer->ImportProfiles();
 }
