@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -16,7 +16,7 @@
 using base::win::RegKey;
 
 // Defined in autofill_ie_toolbar_import_win.cc. Not exposed in the header file.
-bool ImportCurrentUserProfiles(std::vector<AutoFillProfile>* profiles,
+bool ImportCurrentUserProfiles(std::vector<AutofillProfile>* profiles,
                                std::vector<CreditCard>* credit_cards);
 
 namespace {
@@ -44,7 +44,7 @@ ValueDescription profile1[] = {
   { L"email", L"jdoe@test.com" },
   { L"company_name", L"Testcompany" },
   { L"phone_home_number", L"555-5555" },
-  { L"phone_home_city_code", L"444" },
+  { L"phone_home_city_code", L"650" },
   { L"phone_home_country_code", L"1" },
 };
 
@@ -53,9 +53,6 @@ ValueDescription profile2[] = {
   { L"name_last", L"Doe" },
   { L"email", L"janedoe@test.com" },
   { L"company_name", L"Testcompany" },
-  { L"phone_fax_number", L"555-6666" },
-  { L"phone_fax_city_code", L"777" },
-  { L"phone_fax_country_code", L"2" },
 };
 
 ValueDescription credit_card[] = {
@@ -140,7 +137,7 @@ void AutofillIeToolbarImportTest::TearDown() {
   key.DeleteKey(L"");
 }
 
-TEST_F(AutofillIeToolbarImportTest, TestAutoFillImport) {
+TEST_F(AutofillIeToolbarImportTest, TestAutofillImport) {
   RegKey profile_key;
   profile_key.Create(HKEY_CURRENT_USER, kProfileKey, KEY_ALL_ACCESS);
   EXPECT_TRUE(profile_key.Valid());
@@ -158,57 +155,33 @@ TEST_F(AutofillIeToolbarImportTest, TestAutoFillImport) {
   profile_key.Close();
   cc_key.Close();
 
-  std::vector<AutoFillProfile> profiles;
+  std::vector<AutofillProfile> profiles;
   std::vector<CreditCard> credit_cards;
   EXPECT_TRUE(ImportCurrentUserProfiles(&profiles, &credit_cards));
-  ASSERT_EQ(profiles.size(), 2);
+  ASSERT_EQ(2U, profiles.size());
   // The profiles are read in reverse order.
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(NAME_FIRST)),
-            profile1[0].value);
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(NAME_MIDDLE)),
-            profile1[1].value);
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(NAME_LAST)),
-            profile1[2].value);
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(EMAIL_ADDRESS)),
-            profile1[3].value);
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(COMPANY_NAME)),
-            profile1[4].value);
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(PHONE_HOME_COUNTRY_CODE)),
-            profile1[7].value);
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(PHONE_HOME_CITY_CODE)),
-            profile1[6].value);
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(PHONE_HOME_NUMBER)),
-            L"5555555");
-  EXPECT_EQ(profiles[1].GetFieldText(AutoFillType(PHONE_HOME_WHOLE_NUMBER)),
-            L"14445555555");
+  EXPECT_EQ(profile1[0].value, profiles[1].GetInfo(NAME_FIRST));
+  EXPECT_EQ(profile1[1].value, profiles[1].GetInfo(NAME_MIDDLE));
+  EXPECT_EQ(profile1[2].value, profiles[1].GetInfo(NAME_LAST));
+  EXPECT_EQ(profile1[3].value, profiles[1].GetInfo(EMAIL_ADDRESS));
+  EXPECT_EQ(profile1[4].value, profiles[1].GetInfo(COMPANY_NAME));
+  EXPECT_EQ(profile1[7].value, profiles[1].GetInfo(PHONE_HOME_COUNTRY_CODE));
+  EXPECT_EQ(profile1[6].value, profiles[1].GetInfo(PHONE_HOME_CITY_CODE));
+  EXPECT_EQ(L"5555555", profiles[1].GetInfo(PHONE_HOME_NUMBER));
+  EXPECT_EQ(L"+1 650-555-5555", profiles[1].GetInfo(PHONE_HOME_WHOLE_NUMBER));
 
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(NAME_FIRST)),
-            profile2[0].value);
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(NAME_LAST)),
-            profile2[1].value);
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(EMAIL_ADDRESS)),
-            profile2[2].value);
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(COMPANY_NAME)),
-            profile2[3].value);
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(PHONE_FAX_COUNTRY_CODE)),
-            profile2[6].value);
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(PHONE_FAX_CITY_CODE)),
-            profile2[5].value);
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(PHONE_FAX_NUMBER)),
-            L"5556666");
-  EXPECT_EQ(profiles[0].GetFieldText(AutoFillType(PHONE_FAX_WHOLE_NUMBER)),
-            L"27775556666");
+  EXPECT_EQ(profile2[0].value, profiles[0].GetInfo(NAME_FIRST));
+  EXPECT_EQ(profile2[1].value, profiles[0].GetInfo(NAME_LAST));
+  EXPECT_EQ(profile2[2].value, profiles[0].GetInfo(EMAIL_ADDRESS));
+  EXPECT_EQ(profile2[3].value, profiles[0].GetInfo(COMPANY_NAME));
 
-  ASSERT_EQ(credit_cards.size(), 1);
-  EXPECT_EQ(credit_cards[0].GetFieldText(AutoFillType(CREDIT_CARD_NAME)),
-            credit_card[0].value);
-  EXPECT_EQ(credit_cards[0].GetFieldText(AutoFillType(CREDIT_CARD_NUMBER)),
-            L"4111111111111111");
-  EXPECT_EQ(credit_cards[0].GetFieldText(AutoFillType(CREDIT_CARD_EXP_MONTH)),
-            credit_card[2].value);
-  EXPECT_EQ(credit_cards[0].GetFieldText(
-            AutoFillType(CREDIT_CARD_EXP_4_DIGIT_YEAR)),
-            credit_card[3].value);
+  ASSERT_EQ(1U, credit_cards.size());
+  EXPECT_EQ(credit_card[0].value, credit_cards[0].GetInfo(CREDIT_CARD_NAME));
+  EXPECT_EQ(L"4111111111111111", credit_cards[0].GetInfo(CREDIT_CARD_NUMBER));
+  EXPECT_EQ(credit_card[2].value,
+            credit_cards[0].GetInfo(CREDIT_CARD_EXP_MONTH));
+  EXPECT_EQ(credit_card[3].value,
+            credit_cards[0].GetInfo(CREDIT_CARD_EXP_4_DIGIT_YEAR));
 
   // Mock password encrypted cc.
   cc_key.Open(HKEY_CURRENT_USER, kCreditCardKey, KEY_ALL_ACCESS);
@@ -221,8 +194,8 @@ TEST_F(AutofillIeToolbarImportTest, TestAutoFillImport) {
   credit_cards.clear();
   EXPECT_TRUE(ImportCurrentUserProfiles(&profiles, &credit_cards));
   // Profiles are not protected.
-  EXPECT_EQ(profiles.size(), 2);
+  EXPECT_EQ(2U, profiles.size());
   // Credit cards are.
-  EXPECT_EQ(credit_cards.size(), 0);
+  EXPECT_EQ(0U, credit_cards.size());
 }
 
