@@ -1,27 +1,36 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/autofill/credit_card.h"
 
+#include <stddef.h>
+
+#include <ostream>
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/string_util.h"
-#include "base/string_number_conversions.h"
+#include "base/logging.h"
 #include "base/string16.h"
+#include "base/string_number_conversions.h"
+#include "base/string_split.h"
+#include "base/string_util.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/autofill/autofill_country.h"
+#include "chrome/browser/autofill/autofill_regexes.h"
 #include "chrome/browser/autofill/autofill_type.h"
 #include "chrome/browser/autofill/field_types.h"
 #include "chrome/common/guid.h"
 #include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "unicode/dtfmtsym.h"
+#include "unicode/uloc.h"
 
 namespace {
 
-const char* kCreditCardObfuscationString = "************";
+const char16 kCreditCardObfuscationSymbol = '*';
 
-const AutoFillFieldType kAutoFillCreditCardTypes[] = {
+const AutofillFieldType kAutofillCreditCardTypes[] = {
   CREDIT_CARD_NAME,
   CREDIT_CARD_NUMBER,
   CREDIT_CARD_TYPE,
@@ -29,9 +38,21 @@ const AutoFillFieldType kAutoFillCreditCardTypes[] = {
   CREDIT_CARD_EXP_4_DIGIT_YEAR,
 };
 
-const int kAutoFillCreditCardLength = arraysize(kAutoFillCreditCardTypes);
+const int kAutofillCreditCardLength = arraysize(kAutofillCreditCardTypes);
+
+// Returns a version of |number| that has any separator characters removed.
+const string16 StripSeparators(const string16& number) {
+  const char16 kSeparators[] = {'-', ' ', '\0'};
+  string16 stripped;
+  RemoveChars(number, kSeparators, &stripped);
+  return stripped;
+}
 
 std::string GetCreditCardType(const string16& number) {
+  // Don't check for a specific type if this is not a credit card number.
+  if (!CreditCard::IsValidCreditCardNumber(number))
+    return kGenericCard;
+
   // Credit card number specifications taken from:
   // http://en.wikipedia.org/wiki/Credit_card_numbers and
   // http://www.beachnet.com/~hstiles/cardtype.html
@@ -111,26 +132,78 @@ std::string GetCreditCardType(const string16& number) {
   return kGenericCard;
 }
 
-// Return a version of |number| that has had any non-digit values removed.
-const string16 RemoveNonAsciiDigits(const string16& number) {
-  string16 stripped;
-  for (size_t i = 0; i < number.size(); ++i) {
-    if (IsAsciiDigit(number[i]))
-      stripped.append(1, number[i]);
+bool ConvertYear(const string16& year, int* num) {
+  // If the |year| is empty, clear the stored value.
+  if (year.empty()) {
+    *num = 0;
+    return true;
   }
-  return stripped;
+
+  // Try parsing the |year| as a number.
+  if (base::StringToInt(year, num))
+    return true;
+
+  *num = 0;
+  return false;
+}
+
+bool ConvertMonth(const string16& month, int* num) {
+  // If the |month| is empty, clear the stored value.
+  if (month.empty()) {
+    *num = 0;
+    return true;
+  }
+
+  // Try parsing the |month| as a number.
+  if (base::StringToInt(month, num))
+    return true;
+
+  // Try parsing the |month| as a named month, e.g. "January" or "Jan".
+  string16 lowercased_month = StringToLowerASCII(month);
+
+  UErrorCode status = U_ZERO_ERROR;
+  icu::Locale locale(AutofillCountry::ApplicationLocale().c_str());
+  icu::DateFormatSymbols date_format_symbols(locale, status);
+  DCHECK(status == U_ZERO_ERROR || status == U_USING_FALLBACK_WARNING ||
+         status == U_USING_DEFAULT_WARNING);
+
+  int32_t num_months;
+  const icu::UnicodeString* months = date_format_symbols.getMonths(num_months);
+  for (int32_t i = 0; i < num_months; ++i) {
+    const string16 icu_month = string16(months[i].getBuffer(),
+                                        months[i].length());
+    if (lowercased_month == StringToLowerASCII(icu_month)) {
+      *num = i + 1;  // Adjust from 0-indexed to 1-indexed.
+      return true;
+    }
+  }
+
+  months = date_format_symbols.getShortMonths(num_months);
+  for (int32_t i = 0; i < num_months; ++i) {
+    const string16 icu_month = string16(months[i].getBuffer(),
+                                        months[i].length());
+    if (lowercased_month == StringToLowerASCII(icu_month)) {
+      *num = i + 1;  // Adjust from 0-indexed to 1-indexed.
+      return true;
+    }
+  }
+
+  *num = 0;
+  return false;
 }
 
 }  // namespace
 
 CreditCard::CreditCard(const std::string& guid)
-    : expiration_month_(0),
+    : type_(kGenericCard),
+      expiration_month_(0),
       expiration_year_(0),
       guid_(guid) {
 }
 
 CreditCard::CreditCard()
-    : expiration_month_(0),
+    : type_(kGenericCard),
+      expiration_month_(0),
       expiration_year_(0),
       guid_(guid::GenerateGUID()) {
 }
@@ -141,86 +214,20 @@ CreditCard::CreditCard(const CreditCard& credit_card) : FormGroup() {
 
 CreditCard::~CreditCard() {}
 
-FormGroup* CreditCard::Clone() const {
-  return new CreditCard(*this);
+void CreditCard::GetSupportedTypes(FieldTypeSet* supported_types) const {
+  supported_types->insert(CREDIT_CARD_NAME);
+  supported_types->insert(CREDIT_CARD_NUMBER);
+  supported_types->insert(CREDIT_CARD_EXP_MONTH);
+  supported_types->insert(CREDIT_CARD_EXP_2_DIGIT_YEAR);
+  supported_types->insert(CREDIT_CARD_EXP_4_DIGIT_YEAR);
+  supported_types->insert(CREDIT_CARD_EXP_DATE_2_DIGIT_YEAR);
+  supported_types->insert(CREDIT_CARD_EXP_DATE_4_DIGIT_YEAR);
 }
 
-void CreditCard::GetPossibleFieldTypes(const string16& text,
-                                       FieldTypeSet* possible_types) const {
-  if (IsNameOnCard(text))
-    possible_types->insert(CREDIT_CARD_NAME);
-
-  if (IsCreditCardNumber(text))
-    possible_types->insert(CREDIT_CARD_NUMBER);
-
-  if (IsExpirationMonth(text))
-    possible_types->insert(CREDIT_CARD_EXP_MONTH);
-
-  if (Is2DigitExpirationYear(text))
-    possible_types->insert(CREDIT_CARD_EXP_2_DIGIT_YEAR);
-
-  if (Is4DigitExpirationYear(text))
-    possible_types->insert(CREDIT_CARD_EXP_4_DIGIT_YEAR);
-}
-
-void CreditCard::GetAvailableFieldTypes(FieldTypeSet* available_types) const {
-  DCHECK(available_types);
-
-  if (!name_on_card().empty())
-    available_types->insert(CREDIT_CARD_NAME);
-
-  if (!ExpirationMonthAsString().empty())
-    available_types->insert(CREDIT_CARD_EXP_MONTH);
-
-  if (!Expiration2DigitYearAsString().empty())
-    available_types->insert(CREDIT_CARD_EXP_2_DIGIT_YEAR);
-
-  if (!Expiration4DigitYearAsString().empty())
-    available_types->insert(CREDIT_CARD_EXP_4_DIGIT_YEAR);
-
-  if (!number().empty())
-    available_types->insert(CREDIT_CARD_NUMBER);
-}
-
-void CreditCard::FindInfoMatches(const AutoFillType& type,
-                                 const string16& info,
-                                 std::vector<string16>* matched_text) const {
-  DCHECK(matched_text);
-
-  string16 match;
-  switch (type.field_type()) {
-    case CREDIT_CARD_NUMBER: {
-      // Because the credit card number is encrypted and we are not able to do
-      // comparisons with it we will say that any field that is known to be a
-      // credit card number field will match all credit card numbers.
-      string16 text = GetPreviewText(AutoFillType(CREDIT_CARD_NUMBER));
-      if (!text.empty())
-        matched_text->push_back(text);
-      break;
-    }
-
-    case CREDIT_CARD_VERIFICATION_CODE:
-      NOTREACHED();
-      break;
-
-    case UNKNOWN_TYPE:
-      for (int i = 0; i < kAutoFillCreditCardLength; ++i) {
-        if (FindInfoMatchesHelper(kAutoFillCreditCardTypes[i], info, &match))
-          matched_text->push_back(match);
-      }
-      break;
-
-    default:
-      if (FindInfoMatchesHelper(type.field_type(), info, &match))
-        matched_text->push_back(match);
-      break;
-  }
-}
-
-string16 CreditCard::GetFieldText(const AutoFillType& type) const {
-  switch (type.field_type()) {
+string16 CreditCard::GetInfo(AutofillFieldType type) const {
+  switch (type) {
     case CREDIT_CARD_NAME:
-      return name_on_card();
+      return name_on_card_;
 
     case CREDIT_CARD_EXP_MONTH:
       return ExpirationMonthAsString();
@@ -252,7 +259,7 @@ string16 CreditCard::GetFieldText(const AutoFillType& type) const {
       return string16();
 
     case CREDIT_CARD_NUMBER:
-      return number();
+      return number_;
 
     case CREDIT_CARD_VERIFICATION_CODE:
       NOTREACHED();
@@ -264,24 +271,10 @@ string16 CreditCard::GetFieldText(const AutoFillType& type) const {
   }
 }
 
-string16 CreditCard::GetPreviewText(const AutoFillType& type) const {
-  switch (type.field_type()) {
-    case CREDIT_CARD_NUMBER:
-      return last_four_digits();
-
-    case CREDIT_CARD_VERIFICATION_CODE:
-      NOTREACHED();
-      return string16();
-
-    default:
-      return GetFieldText(type);
-  }
-}
-
-void CreditCard::SetInfo(const AutoFillType& type, const string16& value) {
-  switch (type.field_type()) {
+void CreditCard::SetInfo(AutofillFieldType type, const string16& value) {
+  switch (type) {
     case CREDIT_CARD_NAME:
-      set_name_on_card(value);
+      name_on_card_ = value;
       break;
 
     case CREDIT_CARD_EXP_MONTH:
@@ -296,74 +289,123 @@ void CreditCard::SetInfo(const AutoFillType& type, const string16& value) {
       SetExpirationYearFromString(value);
       break;
 
+    case CREDIT_CARD_EXP_DATE_2_DIGIT_YEAR:
+      // This is a read-only attribute.
+      break;
+
+    case CREDIT_CARD_EXP_DATE_4_DIGIT_YEAR:
+      // This is a read-only attribute.
+      break;
+
     case CREDIT_CARD_TYPE:
       // We determine the type based on the number.
       break;
 
     case CREDIT_CARD_NUMBER: {
-      if (StartsWith(value, ASCIIToUTF16(kCreditCardObfuscationString), true)) {
-        // this is an obfuscated string. Do not change the real value.
-        break;
-      }
-      set_number(value);
-      set_type(ASCIIToUTF16(GetCreditCardType(number())));
-      // Update last four digits as well.
-      if (value.length() > 4)
-        set_last_four_digits(value.substr(value.length() - 4));
-      else
-        set_last_four_digits(string16());
-    }  break;
+      // Don't change the real value if the input is an obfuscated string.
+      if (value.size() > 0 && value[0] != kCreditCardObfuscationSymbol)
+        SetNumber(value);
+      break;
+    }
 
     case CREDIT_CARD_VERIFICATION_CODE:
       NOTREACHED();
       break;
 
     default:
-      DLOG(ERROR) << "Attempting to set unknown info-type "
-                  << type.field_type();
+      NOTREACHED() << "Attempting to set unknown info-type " << type;
       break;
   }
 }
 
+string16 CreditCard::GetCanonicalizedInfo(AutofillFieldType type) const {
+  if (type == CREDIT_CARD_NUMBER)
+    return StripSeparators(number_);
+
+  return GetInfo(type);
+}
+
+bool CreditCard::SetCanonicalizedInfo(AutofillFieldType type,
+                                      const string16& value) {
+  if (type == CREDIT_CARD_NUMBER)
+    SetInfo(type, StripSeparators(value));
+  else
+    SetInfo(type, value);
+
+  return true;
+}
+
+void CreditCard::GetMatchingTypes(const string16& text,
+                                  FieldTypeSet* matching_types) const {
+  FormGroup::GetMatchingTypes(text, matching_types);
+
+  string16 card_number = GetCanonicalizedInfo(CREDIT_CARD_NUMBER);
+  if (!card_number.empty() && StripSeparators(text) == card_number)
+    matching_types->insert(CREDIT_CARD_NUMBER);
+
+  int month;
+  if (ConvertMonth(text, &month) && month != 0 && month == expiration_month_)
+    matching_types->insert(CREDIT_CARD_EXP_MONTH);
+}
+
 const string16 CreditCard::Label() const {
-  return label_;
-}
-
-string16 CreditCard::ObfuscatedNumber() const {
+  string16 label;
   if (number().empty())
-    return string16();  // No CC number, means empty preview.
-  string16 result(ASCIIToUTF16(kCreditCardObfuscationString));
-  result.append(last_four_digits());
+    return name_on_card_;  // No CC number, return name only.
 
-  return result;
-}
-
-string16 CreditCard::PreviewSummary() const {
-  string16 preview;
-  if (number().empty())
-    return preview;  // No CC number, means empty preview.
   string16 obfuscated_cc_number = ObfuscatedNumber();
-  if (!expiration_month() || !expiration_year())
+  if (!expiration_month_ || !expiration_year_)
     return obfuscated_cc_number;  // No expiration date set.
+
   // TODO(georgey): Internationalize date.
   string16 formatted_date(ExpirationMonthAsString());
   formatted_date.append(ASCIIToUTF16("/"));
   formatted_date.append(Expiration4DigitYearAsString());
 
-  preview = l10n_util::GetStringFUTF16(
-      IDS_CREDIT_CARD_NUMBER_PREVIEW_FORMAT,
-      obfuscated_cc_number,
-      formatted_date);
-  return preview;
+  label = l10n_util::GetStringFUTF16(IDS_CREDIT_CARD_NUMBER_PREVIEW_FORMAT,
+                                     obfuscated_cc_number,
+                                     formatted_date);
+  return label;
+}
+
+void CreditCard::SetInfoForMonthInputType(const string16& value) {
+  // Check if |text| is "yyyy-mm" format first, and check normal month format.
+  if (!autofill::MatchesPattern(value, UTF8ToUTF16("^[0-9]{4}-[0-9]{1,2}$")))
+    return;
+
+  std::vector<string16> year_month;
+  base::SplitString(value, L'-', &year_month);
+  DCHECK_EQ((int)year_month.size(), 2);
+  int num = 0;
+  bool converted = false;
+  converted = base::StringToInt(year_month[0], &num);
+  DCHECK(converted);
+  SetExpirationYear(num);
+  converted = base::StringToInt(year_month[1], &num);
+  DCHECK(converted);
+  SetExpirationMonth(num);
+}
+
+string16 CreditCard::ObfuscatedNumber() const {
+  // If the number is shorter than four digits, there's no need to obfuscate it.
+  if (number_.size() < 4)
+    return number_;
+
+  string16 number = StripSeparators(number_);
+  string16 result(number.size() - 4, kCreditCardObfuscationSymbol);
+  result.append(LastFourDigits());
+
+  return result;
 }
 
 string16 CreditCard::LastFourDigits() const {
   static const size_t kNumLastDigits = 4;
 
-  if (number().size() < kNumLastDigits)
+  string16 number = StripSeparators(number_);
+  if (number.size() < kNumLastDigits)
     return string16();
 
-  return number().substr(number().size() - kNumLastDigits, kNumLastDigits);
+  return number.substr(number.size() - kNumLastDigits, kNumLastDigits);
 }
 
 void CreditCard::operator=(const CreditCard& credit_card) {
@@ -373,24 +415,42 @@ void CreditCard::operator=(const CreditCard& credit_card) {
   number_ = credit_card.number_;
   name_on_card_ = credit_card.name_on_card_;
   type_ = credit_card.type_;
-  last_four_digits_ = credit_card.last_four_digits_;
   expiration_month_ = credit_card.expiration_month_;
   expiration_year_ = credit_card.expiration_year_;
-  label_ = credit_card.label_;
   guid_ = credit_card.guid_;
 }
 
+bool CreditCard::UpdateFromImportedCard(const CreditCard& imported_card) {
+  if (this->GetCanonicalizedInfo(CREDIT_CARD_NUMBER) !=
+          imported_card.GetCanonicalizedInfo(CREDIT_CARD_NUMBER)) {
+    return false;
+  }
+
+  // Note that the card number is intentionally not updated, so as to preserve
+  // any formatting (i.e. separator characters).  Since the card number is not
+  // updated, there is no reason to update the card type, either.
+  if (!imported_card.name_on_card_.empty())
+    name_on_card_ = imported_card.name_on_card_;
+
+  // The expiration date for |imported_card| should always be set.
+  DCHECK(imported_card.expiration_month_ && imported_card.expiration_year_);
+  expiration_month_ = imported_card.expiration_month_;
+  expiration_year_ = imported_card.expiration_year_;
+
+  return true;
+}
+
 int CreditCard::Compare(const CreditCard& credit_card) const {
   // The following CreditCard field types are the only types we store in the
   // WebDB so far, so we're only concerned with matching these types in the
   // credit card.
-  const AutoFillFieldType types[] = { CREDIT_CARD_NAME,
+  const AutofillFieldType types[] = { CREDIT_CARD_NAME,
                                       CREDIT_CARD_NUMBER,
                                       CREDIT_CARD_EXP_MONTH,
                                       CREDIT_CARD_EXP_4_DIGIT_YEAR };
   for (size_t index = 0; index < arraysize(types); ++index) {
-    int comparison = GetFieldText(AutoFillType(types[index])).compare(
-        credit_card.GetFieldText(AutoFillType(types[index])));
+    int comparison = GetInfo(types[index]).compare(
+        credit_card.GetInfo(types[index]));
     if (comparison != 0)
       return comparison;
   }
@@ -399,7 +459,7 @@ int CreditCard::Compare(const CreditCard& credit_card) const {
 }
 
 bool CreditCard::operator==(const CreditCard& credit_card) const {
-  if (label_ != credit_card.label_ || guid_ != credit_card.guid_)
+  if (guid_ != credit_card.guid_)
     return false;
 
   return Compare(credit_card) == 0;
@@ -409,14 +469,22 @@ bool CreditCard::operator!=(const CreditCard& credit_card) const {
   return !operator==(credit_card);
 }
 
-// Use the Luhn formula to validate the number.
 // static
-bool CreditCard::IsCreditCardNumber(const string16& text) {
-  string16 number = RemoveNonAsciiDigits(text);
+bool CreditCard::IsValidCreditCardNumber(const string16& text) {
+  string16 number = StripSeparators(text);
 
-  if (number.empty())
+  // Credit card numbers are at most 19 digits in length [1]. 12 digits seems to
+  // be a fairly safe lower-bound [2].
+  // [1] http://www.merriampark.com/anatomycc.htm
+  // [2] http://en.wikipedia.org/wiki/Bank_card_number
+  const size_t kMinCreditCardDigits = 12;
+  const size_t kMaxCreditCardDigits = 19;
+  if (number.size() < kMinCreditCardDigits ||
+      number.size() > kMaxCreditCardDigits)
     return false;
 
+  // Use the Luhn formula [3] to validate the number.
+  // [3] http://en.wikipedia.org/wiki/Luhn_algorithm
   int sum = 0;
   bool odd = false;
   string16::reverse_iterator iter;
@@ -439,10 +507,17 @@ bool CreditCard::IsCreditCardNumber(const string16& text) {
 
 bool CreditCard::IsEmpty() const {
   FieldTypeSet types;
-  GetAvailableFieldTypes(&types);
+  GetNonEmptyTypes(&types);
   return types.empty();
 }
 
+bool CreditCard::IsComplete() const {
+  return
+      IsValidCreditCardNumber(number_) &&
+      expiration_month_ != 0 &&
+      expiration_year_ != 0;
+}
+
 string16 CreditCard::ExpirationMonthAsString() const {
   if (expiration_month_ == 0)
     return string16();
@@ -472,32 +547,33 @@ string16 CreditCard::Expiration2DigitYearAsString() const {
 
 void CreditCard::SetExpirationMonthFromString(const string16& text) {
   int month;
-  if (!ConvertDate(text, &month))
+  if (!ConvertMonth(text, &month))
     return;
 
-  set_expiration_month(month);
+  SetExpirationMonth(month);
 }
 
 void CreditCard::SetExpirationYearFromString(const string16& text) {
   int year;
-  if (!ConvertDate(text, &year))
+  if (!ConvertYear(text, &year))
     return;
 
-  set_expiration_year(year);
+  SetExpirationYear(year);
 }
 
-void CreditCard::set_number(const string16& number) {
-  number_ = RemoveNonAsciiDigits(number);
+void CreditCard::SetNumber(const string16& number) {
+  number_ = number;
+  type_ = GetCreditCardType(StripSeparators(number_));
 }
 
-void CreditCard::set_expiration_month(int expiration_month) {
+void CreditCard::SetExpirationMonth(int expiration_month) {
   if (expiration_month < 0 || expiration_month > 12)
     return;
 
   expiration_month_ = expiration_month;
 }
 
-void CreditCard::set_expiration_year(int expiration_year) {
+void CreditCard::SetExpirationYear(int expiration_year) {
   if (expiration_year != 0 &&
       (expiration_year < 2006 || expiration_year > 10000)) {
     return;
@@ -506,112 +582,6 @@ void CreditCard::set_expiration_year(int expiration_year) {
   expiration_year_ = expiration_year;
 }
 
-bool CreditCard::FindInfoMatchesHelper(const AutoFillFieldType& field_type,
-                                       const string16& info,
-                                       string16* match) const {
-  DCHECK(match);
-
-  match->clear();
-  switch (field_type) {
-    case CREDIT_CARD_NAME: {
-      if (StartsWith(name_on_card(), info, false))
-        *match = name_on_card();
-      break;
-    }
-
-    case CREDIT_CARD_EXP_MONTH: {
-      string16 exp_month(ExpirationMonthAsString());
-      if (StartsWith(exp_month, info, true))
-        *match = exp_month;
-      break;
-    }
-
-    case CREDIT_CARD_EXP_2_DIGIT_YEAR: {
-      string16 exp_year(Expiration2DigitYearAsString());
-      if (StartsWith(exp_year, info, true))
-        *match = exp_year;
-      break;
-    }
-
-    case CREDIT_CARD_EXP_4_DIGIT_YEAR: {
-      string16 exp_year(Expiration4DigitYearAsString());
-      if (StartsWith(exp_year, info, true))
-        *match = exp_year;
-      break;
-    }
-
-    case CREDIT_CARD_EXP_DATE_2_DIGIT_YEAR: {
-      string16 exp_date(ExpirationMonthAsString() + ASCIIToUTF16("/") +
-                        Expiration2DigitYearAsString());
-      if (StartsWith(exp_date, info, true))
-        *match = exp_date;
-      break;
-    }
-
-    case CREDIT_CARD_EXP_DATE_4_DIGIT_YEAR: {
-      string16 exp_date(ExpirationMonthAsString() + ASCIIToUTF16("/") +
-                        Expiration4DigitYearAsString());
-      if (StartsWith(exp_date, info, true))
-        *match = exp_date;
-      break;
-    }
-
-    case CREDIT_CARD_TYPE:
-      // We don't handle this case.
-      break;
-
-    case CREDIT_CARD_VERIFICATION_CODE:
-      NOTREACHED();
-      break;
-
-    default:
-      break;
-  }
-  return !match->empty();
-}
-
-bool CreditCard::IsNameOnCard(const string16& text) const {
-  return StringToLowerASCII(text) == StringToLowerASCII(name_on_card_);
-}
-
-bool CreditCard::IsExpirationMonth(const string16& text) const {
-  int month;
-  if (!base::StringToInt(text, &month))
-    return false;
-
-  return expiration_month_ == month;
-}
-
-bool CreditCard::Is2DigitExpirationYear(const string16& text) const {
-  int year;
-  if (!base::StringToInt(text, &year))
-    return false;
-
-  return year < 100 && (expiration_year_ % 100) == year;
-}
-
-bool CreditCard::Is4DigitExpirationYear(const string16& text) const {
-  int year;
-  if (!base::StringToInt(text, &year))
-    return false;
-
-  return expiration_year_ == year;
-}
-
-bool CreditCard::ConvertDate(const string16& date, int* num) const {
-  if (!date.empty()) {
-    bool converted = base::StringToInt(date, num);
-    DCHECK(converted);
-    if (!converted)
-      return false;
-  } else {
-    // Clear the value.
-    *num = 0;
-  }
-
-  return true;
-}
-
 // So we can compare CreditCards with EXPECT_EQ().
 std::ostream& operator<<(std::ostream& os, const CreditCard& credit_card) {
   return os
@@ -619,22 +589,20 @@ std::ostream& operator<<(std::ostream& os, const CreditCard& credit_card) {
       << " "
       << credit_card.guid()
       << " "
-      << UTF16ToUTF8(credit_card.GetFieldText(AutoFillType(CREDIT_CARD_NAME)))
+      << UTF16ToUTF8(credit_card.GetInfo(CREDIT_CARD_NAME))
       << " "
-      << UTF16ToUTF8(credit_card.GetFieldText(AutoFillType(CREDIT_CARD_TYPE)))
+      << UTF16ToUTF8(credit_card.GetInfo(CREDIT_CARD_TYPE))
       << " "
-      << UTF16ToUTF8(credit_card.GetFieldText(AutoFillType(CREDIT_CARD_NUMBER)))
+      << UTF16ToUTF8(credit_card.GetInfo(CREDIT_CARD_NUMBER))
       << " "
-      << UTF16ToUTF8(credit_card.GetFieldText(
-             AutoFillType(CREDIT_CARD_EXP_MONTH)))
+      << UTF16ToUTF8(credit_card.GetInfo(CREDIT_CARD_EXP_MONTH))
       << " "
-      << UTF16ToUTF8(credit_card.GetFieldText(
-             AutoFillType(CREDIT_CARD_EXP_4_DIGIT_YEAR)));
+      << UTF16ToUTF8(credit_card.GetInfo(CREDIT_CARD_EXP_4_DIGIT_YEAR));
 }
 
-// These values must match the values in WebKitClientImpl in webkit/glue. We
-// send these strings to WK, which then asks WebKitClientImpl to load the image
-// data.
+// These values must match the values in WebKitPlatformSupportImpl in
+// webkit/glue. We send these strings to WebKit, which then asks
+// WebKitPlatformSupportImpl to load the image data.
 const char* const kAmericanExpressCard = "americanExpressCC";
 const char* const kDinersCard = "dinersCC";
 const char* const kDiscoverCard = "discoverCC";
