@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,9 +6,12 @@
 #define CHROME_BROWSER_AUTOFILL_CREDIT_CARD_H_
 #pragma once
 
+#include <iosfwd>
+#include <string>
 #include <vector>
 
 #include "base/string16.h"
+#include "chrome/browser/autofill/field_types.h"
 #include "chrome/browser/autofill/form_group.h"
 
 // A form group that stores credit card information.
@@ -22,26 +25,26 @@ class CreditCard : public FormGroup {
   virtual ~CreditCard();
 
   // FormGroup implementation:
-  virtual FormGroup* Clone() const;
-  virtual void GetPossibleFieldTypes(const string16& text,
-                                     FieldTypeSet* possible_types) const;
-  virtual void GetAvailableFieldTypes(FieldTypeSet* available_types) const;
-  virtual void FindInfoMatches(const AutoFillType& type,
-                               const string16& info,
-                               std::vector<string16>* matched_text) const;
-  virtual string16 GetFieldText(const AutoFillType& type) const;
-  virtual string16 GetPreviewText(const AutoFillType& type) const;
-  virtual void SetInfo(const AutoFillType& type, const string16& value);
-  virtual const string16 Label() const;
+  virtual string16 GetInfo(AutofillFieldType type) const OVERRIDE;
+  virtual void SetInfo(AutofillFieldType type, const string16& value) OVERRIDE;
+  virtual string16 GetCanonicalizedInfo(AutofillFieldType type) const OVERRIDE;
+  virtual bool SetCanonicalizedInfo(AutofillFieldType type,
+                                    const string16& value) OVERRIDE;
+  virtual void GetMatchingTypes(const string16& text,
+                                FieldTypeSet* matching_types) const OVERRIDE;
+
+  // Credit card preview summary, for example: ******1234, Exp: 01/2020
+  const string16 Label() const;
+
+  // Special method to set value for HTML5 month input type.
+  void SetInfoForMonthInputType(const string16& value);
 
   // The number altered for display, for example: ******1234
   string16 ObfuscatedNumber() const;
-  // Credit card preview summary, for example: ******1234, Exp: 01/2020
-  string16 PreviewSummary() const;
   // The last four digits of the credit card number.
   string16 LastFourDigits() const;
 
-  const string16& type() const { return type_; }
+  const std::string& type() const { return type_; }
 
   // The guid is the primary identifier for |CreditCard| objects.
   const std::string guid() const { return guid_; }
@@ -50,6 +53,12 @@ class CreditCard : public FormGroup {
   // For use in STL containers.
   void operator=(const CreditCard& credit_card);
 
+  // If the card numbers for |this| and |imported_card| match, overwrites |this|
+  // card's data with the data in |credit_card| and returns true.  Otherwise,
+  // returns false.
+  bool UpdateFromImportedCard(const CreditCard& imported_card)
+      WARN_UNUSED_RESULT;
+
   // Comparison for Sync.  Returns 0 if the credit card is the same as |this|,
   // or < 0, or > 0 if it is different.  The implied ordering can be used for
   // culling duplicates.  The ordering is based on collation order of the
@@ -61,19 +70,24 @@ class CreditCard : public FormGroup {
   // Used by tests.
   bool operator==(const CreditCard& credit_card) const;
   bool operator!=(const CreditCard& credit_card) const;
-  void set_label(const string16& label) { label_ = label; }
 
-  // Returns true if |value| is a credit card number.  Uses the Luhn formula to
-  // validate the number.
-  static bool IsCreditCardNumber(const string16& text);
+  // Returns true if |text| looks like a valid credit card number.
+  // Uses the Luhn formula to validate the number.
+  static bool IsValidCreditCardNumber(const string16& text);
 
   // Returns true if there are no values (field types) set.
   bool IsEmpty() const;
 
+  // Returns true if all field types have valid values set.
+  bool IsComplete() const;
+
   // Returns the credit card number.
   const string16& number() const { return number_; }
 
  private:
+  // FormGroup:
+  virtual void GetSupportedTypes(FieldTypeSet* supported_types) const OVERRIDE;
+
   // The month and year are zero if not present.
   int Expiration4DigitYear() const { return expiration_year_; }
   int Expiration2DigitYear() const { return expiration_year_ % 100; }
@@ -87,64 +101,22 @@ class CreditCard : public FormGroup {
   // Sets |expiration_year_| to the integer conversion of |text|.
   void SetExpirationYearFromString(const string16& text);
 
-  const string16& name_on_card() const { return name_on_card_; }
-  const string16& last_four_digits() const { return last_four_digits_; }
-  int expiration_month() const { return expiration_month_; }
-  int expiration_year() const { return expiration_year_; }
-
-  void set_number(const string16& number);
-  void set_name_on_card(const string16& name_on_card) {
-    name_on_card_ = name_on_card;
-  }
-  void set_type(const string16& type) { type_ = type; }
-  void set_last_four_digits(const string16& last_four_digits) {
-    last_four_digits_ = last_four_digits;
-  }
+  // Sets |number_| to |number| and computes the appropriate card |type_|.
+  void SetNumber(const string16& number);
 
   // These setters verify that the month and year are within appropriate
   // ranges.
-  void set_expiration_month(int expiration_month);
-  void set_expiration_year(int expiration_year);
-
-  // A helper function for FindInfoMatches that only handles matching the info
-  // with the requested field type.
-  bool FindInfoMatchesHelper(const AutoFillFieldType& field_type,
-                             const string16& info,
-                             string16* match) const;
-
-  // Returns true if |text| matches the name on the card.  The comparison is
-  // case-insensitive.
-  bool IsNameOnCard(const string16& text) const;
-
-  // Returns true if |text| matches the expiration month of the card.
-  bool IsExpirationMonth(const string16& text) const;
-
-  // Returns true if the integer value of |text| matches the 2-digit expiration
-  // year.
-  bool Is2DigitExpirationYear(const string16& text) const;
-
-  // Returns true if the integer value of |text| matches the 4-digit expiration
-  // year.
-  bool Is4DigitExpirationYear(const string16& text) const;
-
-  // Converts |date| to an integer form.  Returns true if the conversion
-  // succeeded.
-  bool ConvertDate(const string16& date, int* num) const;
+  void SetExpirationMonth(int expiration_month);
+  void SetExpirationYear(int expiration_year);
 
   string16 number_;  // The credit card number.
   string16 name_on_card_;  // The cardholder's name.
-  string16 type_;  // The type of the card.
-
-  // Stores the last four digits of the credit card number.
-  string16 last_four_digits_;
+  std::string type_;  // The type of the card.
 
   // These members are zero if not present.
   int expiration_month_;
   int expiration_year_;
 
-  // This is the display name of the card set by the user, e.g., Amazon Visa.
-  string16 label_;
-
   // The guid of this credit card.
   std::string guid_;
 };
