@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,106 +9,87 @@
 #include <set>
 #include <vector>
 
+#include "base/basictypes.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/scoped_vector.h"
 #include "base/observer_list.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/scoped_vector.h"
 #include "base/string16.h"
-#include "chrome/browser/autofill/autofill_dialog.h"
 #include "chrome/browser/autofill/autofill_profile.h"
 #include "chrome/browser/autofill/credit_card.h"
 #include "chrome/browser/autofill/field_types.h"
+#include "chrome/browser/profiles/profile_keyed_service.h"
+#include "chrome/browser/sync/profile_sync_service_observer.h"
 #include "chrome/browser/webdata/web_data_service.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
-class AutoFillManager;
+class AutofillMetrics;
 class FormStructure;
+class PersonalDataManagerObserver;
 class Profile;
 
-// Handles loading and saving AutoFill profile information to the web database.
+namespace autofill_helper {
+void SetProfiles(int, std::vector<AutofillProfile>*);
+}
+
+// Handles loading and saving Autofill profile information to the web database.
 // This class also stores the profiles loaded from the database for use during
-// AutoFill.
+// Autofill.
 class PersonalDataManager
     : public WebDataServiceConsumer,
-      public AutoFillDialogObserver,
-      public base::RefCountedThreadSafe<PersonalDataManager> {
+      public ProfileSyncServiceObserver,
+      public ProfileKeyedService,
+      public content::NotificationObserver {
  public:
-  // An interface the PersonalDataManager uses to notify its clients (observers)
-  // when it has finished loading personal data from the web database.  Register
-  // the observer via PersonalDataManager::SetObserver.
-  class Observer {
-   public:
-    // Notifies the observer that the PersonalDataManager has finished loading.
-    // TODO: OnPersonalDataLoaded should be nuked in favor of only
-    // OnPersonalDataChanged.
-    virtual void OnPersonalDataLoaded() = 0;
-
-    // Notifies the observer that the PersonalDataManager changed in some way.
-    virtual void OnPersonalDataChanged() {}
-
-   protected:
-    virtual ~Observer() {}
-  };
-
-  // WebDataServiceConsumer implementation:
-  virtual void OnWebDataServiceRequestDone(WebDataService::Handle h,
-                                           const WDTypedResult* result);
-
-  // AutoFillDialogObserver implementation:
-  virtual void OnAutoFillDialogApply(std::vector<AutoFillProfile>* profiles,
-                                     std::vector<CreditCard>* credit_cards);
+  // WebDataServiceConsumer:
+  virtual void OnWebDataServiceRequestDone(
+      WebDataService::Handle h,
+      const WDTypedResult* result) OVERRIDE;
 
   // Sets the listener to be notified of PersonalDataManager events.
-  virtual void SetObserver(PersonalDataManager::Observer* observer);
+  virtual void SetObserver(PersonalDataManagerObserver* observer);
 
   // Removes |observer| as the observer of this PersonalDataManager.
-  virtual void RemoveObserver(PersonalDataManager::Observer* observer);
+  virtual void RemoveObserver(PersonalDataManagerObserver* observer);
 
-  // If AutoFill is able to determine the field types of a significant number of
-  // field types that contain information in the FormStructures a profile will
-  // be created with all of the information from recognized fields. Returns
-  // whether a profile was created.
-  bool ImportFormData(const std::vector<const FormStructure*>& form_structures);
+  // ProfileSyncServiceObserver:
+  virtual void OnStateChanged() OVERRIDE;
 
-  // Gets |imported_profile_| and |imported_credit_card_| and returns their
-  // values in |profile| and |credit_card| parameters respectively.  One or
-  // both may return NULL.  The objects returned are owned by the
-  // PersonalDataManager, so should be considered weak references by caller.
-  // TODO(dhollowa) Now that we aren't immediately saving the imported form
-  // data, we should store the profile and CC in the AFM instead of the PDM.
-  void GetImportedFormData(AutoFillProfile** profile, CreditCard** credit_card);
+  // ProfileKeyedService:
+  // Cancels any pending requests to WebDataService and stops listening for Sync
+  // notifications.
+  virtual void Shutdown() OVERRIDE;
+
+  // content::NotificationObserver:
+  // Observes "batch" changes made by Sync and refreshes data from the
+  // WebDataService in response.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
+
+  // Scans the given |form| for importable Autofill data. If the form includes
+  // sufficient address data, it is immediately imported. If the form includes
+  // sufficient credit card data, it is stored into |credit_card|, so that we
+  // can prompt the user whether to save this data.
+  // Returns |true| if sufficient address or credit card data was found.
+  bool ImportFormData(const FormStructure& form,
+                      const CreditCard** credit_card);
 
   // Saves a credit card value detected in |ImportedFormData|.
-  void SaveImportedCreditCard();
-
-  // Sets |web_profiles_| to the contents of |profiles| and updates the web
-  // database by adding, updating and removing profiles.
-  //
-  // The relationship between this and Refresh is subtle.
-  // A call to |SetProfiles| could include out-of-date data that may conflict
-  // if we didn't refresh-to-latest before an AutoFill window was opened for
-  // editing. |SetProfiles| is implemented to make a "best effort" to apply the
-  // changes, but in extremely rare edge cases it is possible not all of the
-  // updates in |profiles| make it to the DB.  This is why SetProfiles will
-  // invoke Refresh after finishing, to ensure we get into a
-  // consistent state.  See Refresh for details.
-  void SetProfiles(std::vector<AutoFillProfile>* profiles);
-
-  // Sets |credit_cards_| to the contents of |credit_cards| and updates the web
-  // database by adding, updating and removing credit cards.
-  void SetCreditCards(std::vector<CreditCard>* credit_cards);
+  virtual void SaveImportedCreditCard(const CreditCard& imported_credit_card);
 
   // Adds |profile| to the web database.
-  void AddProfile(const AutoFillProfile& profile);
+  void AddProfile(const AutofillProfile& profile);
 
   // Updates |profile| which already exists in the web database.
-  void UpdateProfile(const AutoFillProfile& profile);
+  void UpdateProfile(const AutofillProfile& profile);
 
   // Removes the profile represented by |guid|.
   void RemoveProfile(const std::string& guid);
 
   // Returns the profile with the specified |guid|, or NULL if there is no
   // profile with the specified |guid|.
-  AutoFillProfile* GetProfileByGUID(const std::string& guid);
+  AutofillProfile* GetProfileByGUID(const std::string& guid);
 
   // Adds |credit_card| to the web database.
   void AddCreditCard(const CreditCard& credit_card);
@@ -123,10 +104,8 @@ class PersonalDataManager
   // no credit card with the specified |guid|.
   CreditCard* GetCreditCardByGUID(const std::string& guid);
 
-  // Gets the possible field types for the given text, determined by matching
-  // the text with all known personal information and returning matching types.
-  void GetPossibleFieldTypes(const string16& text,
-                             FieldTypeSet* possible_types);
+  // Gets the field types availabe in the stored address and credit card data.
+  void GetNonEmptyTypes(FieldTypeSet* non_empty_types) const;
 
   // Returns true if the credit card information is stored with a password.
   bool HasPassword();
@@ -138,16 +117,9 @@ class PersonalDataManager
   // lifetime is until the web database is updated with new profile and credit
   // card information, respectively.  |profiles()| returns both web and
   // auxiliary profiles.  |web_profiles()| returns only web profiles.
-  const std::vector<AutoFillProfile*>& profiles();
-  virtual const std::vector<AutoFillProfile*>& web_profiles();
-  virtual const std::vector<CreditCard*>& credit_cards();
-
-  // Creates a profile labeled |label|.
-  // This must be called on the DB thread with the expectation that the
-  // returned form will be synchronously persisted to the WebDatabase.  See
-  // Refresh and SetProfiles for details.
-  AutoFillProfile* CreateNewEmptyAutoFillProfileForDBThread(
-      const string16& label);
+  const std::vector<AutofillProfile*>& profiles() const;
+  virtual const std::vector<AutofillProfile*>& web_profiles() const;
+  virtual const std::vector<CreditCard*>& credit_cards() const;
 
   // Re-loads profiles and credit cards from the WebDatabase asynchronously.
   // In the general case, this is a no-op and will re-create the same
@@ -159,28 +131,55 @@ class PersonalDataManager
   // Also see SetProfile for more details.
   virtual void Refresh();
 
-  // Kicks off asynchronous loading of profiles and credit cards.
-  void Init(Profile* profile);
+  // Checks suitability of |profile| for adding to the user's set of profiles.
+  static bool IsValidLearnableProfile(const AutofillProfile& profile);
+
+  // Merges |profile| into one of the |existing_profiles| if possible; otherwise
+  // appends |profile| to the end of that list. Fills |merged_profiles| with the
+  // result.
+  static bool MergeProfile(
+      const AutofillProfile& profile,
+      const std::vector<AutofillProfile*>& existing_profiles,
+      std::vector<AutofillProfile>* merged_profiles);
 
  protected:
-  // Make sure that only Profile and certain tests can create an instance of
+  // Only PersonalDataManagerFactory and certain tests can create instances of
   // PersonalDataManager.
-  friend class base::RefCountedThreadSafe<PersonalDataManager>;
+  FRIEND_TEST_ALL_PREFIXES(AutofillMetricsTest, FirstMiddleLast);
+  FRIEND_TEST_ALL_PREFIXES(AutofillMetricsTest, AutofillIsEnabledAtStartup);
+  FRIEND_TEST_ALL_PREFIXES(PersonalDataManagerTest,
+                           AggregateExistingAuxiliaryProfile);
+  friend class PersonalDataManagerFactory;
   friend class PersonalDataManagerTest;
-  friend class ProfileImpl;
+  friend class scoped_ptr<PersonalDataManager>;
   friend class ProfileSyncServiceAutofillTest;
+  friend class TestingAutomationProvider;
+  friend void autofill_helper::SetProfiles(int, std::vector<AutofillProfile>*);
 
   PersonalDataManager();
-  ~PersonalDataManager();
+  virtual ~PersonalDataManager();
 
-  // Returns the profile of the tab contents.
-  Profile* profile();
+  // Sets |web_profiles_| to the contents of |profiles| and updates the web
+  // database by adding, updating and removing profiles.
+  // The relationship between this and Refresh is subtle.
+  // A call to |SetProfiles| could include out-of-date data that may conflict
+  // if we didn't refresh-to-latest before an Autofill window was opened for
+  // editing. |SetProfiles| is implemented to make a "best effort" to apply the
+  // changes, but in extremely rare edge cases it is possible not all of the
+  // updates in |profiles| make it to the DB.  This is why SetProfiles will
+  // invoke Refresh after finishing, to ensure we get into a
+  // consistent state.  See Refresh for details.
+  void SetProfiles(std::vector<AutofillProfile>* profiles);
+
+  // Sets |credit_cards_| to the contents of |credit_cards| and updates the web
+  // database by adding, updating and removing credit cards.
+  void SetCreditCards(std::vector<CreditCard>* credit_cards);
 
   // Loads the saved profiles from the web database.
   virtual void LoadProfiles();
 
   // Loads the auxiliary profiles.  Currently Mac only.
-  void LoadAuxiliaryProfiles();
+  void LoadAuxiliaryProfiles() const;
 
   // Loads the saved credit cards from the web database.
   virtual void LoadCreditCards();
@@ -199,14 +198,22 @@ class PersonalDataManager
   // query handle.
   void CancelPendingQuery(WebDataService::Handle* handle);
 
-  // Ensures that all profile labels are unique by appending an increasing digit
-  // to the end of non-unique labels.
-  // TODO(jhawkins): Create a new interface for labeled entities and turn these
-  // two methods into one.
-  void SetUniqueCreditCardLabels(std::vector<CreditCard>* credit_cards);
+  // Saves |imported_profile| to the WebDB if it exists.
+  virtual void SaveImportedProfile(const AutofillProfile& imported_profile);
 
-  // Saves |imported_profile_| to the WebDB if it exists.
-  void SaveImportedProfile();
+  // Notifies Sync about data migration if necessary.
+  void EmptyMigrationTrash();
+
+  // The first time this is called, logs an UMA metrics for the number of
+  // profiles the user has. On subsequent calls, does nothing.
+  void LogProfileCount() const;
+
+  // Returns the value of the AutofillEnabled pref.
+  virtual bool IsAutofillEnabled() const;
+
+  // For tests.
+  const AutofillMetrics* metric_logger() const;
+  void set_metric_logger(const AutofillMetrics* metric_logger);
 
   // The profile hosting this PersonalDataManager.
   Profile* profile_;
@@ -215,28 +222,18 @@ class PersonalDataManager
   bool is_data_loaded_;
 
   // The loaded web profiles.
-  ScopedVector<AutoFillProfile> web_profiles_;
+  ScopedVector<AutofillProfile> web_profiles_;
 
   // Auxiliary profiles.
-  ScopedVector<AutoFillProfile> auxiliary_profiles_;
+  mutable ScopedVector<AutofillProfile> auxiliary_profiles_;
 
   // Storage for combined web and auxiliary profiles.  Contents are weak
   // references.  Lifetime managed by |web_profiles_| and |auxiliary_profiles_|.
-  std::vector<AutoFillProfile*> profiles_;
+  mutable std::vector<AutofillProfile*> profiles_;
 
   // The loaded credit cards.
   ScopedVector<CreditCard> credit_cards_;
 
-  // The profile that is imported from a web form by ImportFormData.
-  scoped_ptr<AutoFillProfile> imported_profile_;
-
-  // The credit card that is imported from a web form by ImportFormData.
-  scoped_ptr<CreditCard> imported_credit_card_;
-
-  // The hash of the password used to store the credit card.  This is empty if
-  // no password exists.
-  string16 password_hash_;
-
   // When the manager makes a request from WebDataService, the database
   // is queried on another thread, we record the query handle until we
   // get called back.  We store handles for both profile and credit card queries
@@ -245,7 +242,20 @@ class PersonalDataManager
   WebDataService::Handle pending_creditcards_query_;
 
   // The observers.
-  ObserverList<Observer> observers_;
+  ObserverList<PersonalDataManagerObserver> observers_;
+
+ private:
+  // Kicks off asynchronous loading of profiles and credit cards.
+  void Init(Profile* profile);
+
+  // For logging UMA metrics. Overridden by metrics tests.
+  scoped_ptr<const AutofillMetrics> metric_logger_;
+
+  // Whether we have already logged the number of profiles this session.
+  mutable bool has_logged_profile_count_;
+
+  // Manages registration lifetime for NotificationObserver implementation.
+  content::NotificationRegistrar notification_registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(PersonalDataManager);
 };
