@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,29 +12,48 @@
 #define CHROME_BROWSER_AUTOMATION_AUTOMATION_PROVIDER_H_
 #pragma once
 
+#include <list>
 #include <map>
 #include <string>
 #include <vector>
 
 #include "base/basictypes.h"
 #include "base/compiler_specific.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
+#include "base/message_loop_helpers.h"
 #include "base/observer_list.h"
-#include "base/scoped_ptr.h"
 #include "base/string16.h"
 #include "chrome/browser/autofill/field_types.h"
 #include "chrome/browser/cancelable_request.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
 #include "chrome/common/automation_constants.h"
 #include "chrome/common/content_settings.h"
-#include "chrome/common/notification_observer.h"
-#include "ipc/ipc_message.h"
+#include "content/browser/trace_controller.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/notification_observer.h"
 #include "ipc/ipc_channel.h"
-#if defined(OS_WIN)
-#include "gfx/native_widget_types.h"
-#include "views/event.h"
-#endif  // defined(OS_WIN)
 
-class PopupMenuWaiter;
+#if defined(OS_WIN) && !defined(USE_AURA)
+#include "ui/gfx/native_widget_types.h"
+#include "ui/views/events/event.h"
+#endif  // defined(OS_WIN) && !defined(USE_AURA)
+
+class AutomationBrowserTracker;
+class AutomationExtensionTracker;
+class AutomationResourceMessageFilter;
+class AutomationTabTracker;
+class AutomationWindowTracker;
+class Browser;
+class Extension;
+class ExtensionTestResultNotificationObserver;
+class ExternalTabContainer;
+class FilePath;
+class InitialLoadObserver;
+class LoginHandler;
+class MetricEventDurationObserver;
+class NavigationControllerRestoredObserver;
+class Profile;
+class RenderViewHost;
 class TabContents;
 struct AutomationMsg_Find_Params;
 struct Reposition_Params;
@@ -44,40 +63,30 @@ namespace IPC {
 class ChannelProxy;
 }
 
-class AutoFillProfile;
-class AutomationAutocompleteEditTracker;
-class AutomationBrowserTracker;
-class AutomationExtensionTracker;
-class AutomationResourceMessageFilter;
-class AutomationTabTracker;
-class AutomationWindowTracker;
-class Browser;
-class CreditCard;
-class DictionaryValue;
-class DownloadItem;
-class Extension;
-class ExtensionPortContainer;
-class ExtensionTestResultNotificationObserver;
-class ExternalTabContainer;
-class FilePath;
-class InitialLoadObserver;
-class ListValue;
-class LoginHandler;
-class MetricEventDurationObserver;
+namespace content {
 class NavigationController;
-class NavigationControllerRestoredObserver;
-class Profile;
-class RenderViewHost;
-class TabContents;
-struct AutocompleteMatchData;
+}
+
+namespace base {
+class DictionaryValue;
+}
+
+namespace content {
+class DownloadItem;
+class WebContents;
+}
 
 namespace gfx {
 class Point;
 }
 
-class AutomationProvider : public base::RefCounted<AutomationProvider>,
-                           public IPC::Channel::Listener,
-                           public IPC::Message::Sender {
+class AutomationProvider
+    : public IPC::Channel::Listener,
+      public IPC::Message::Sender,
+      public base::SupportsWeakPtr<AutomationProvider>,
+      public base::RefCountedThreadSafe<
+          AutomationProvider, content::BrowserThread::DeleteOnUIThread>,
+      public TraceSubscriber {
  public:
   explicit AutomationProvider(Profile* profile);
 
@@ -92,41 +101,28 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   // for more information about kPrimaryIPCChannel.
   bool InitializeChannel(const std::string& channel_id) WARN_UNUSED_RESULT;
 
+  virtual IPC::Channel::Mode GetChannelMode(bool use_named_interface);
+
   // Sets the number of tabs that we expect; when this number of tabs has
   // loaded, an AutomationMsg_InitialLoadsComplete message is sent.
   void SetExpectedTabCount(size_t expected_tabs);
 
   // Called when the inital set of tabs has finished loading.
   // Call SetExpectedTabCount(0) to set this to true immediately.
-  void OnInitialLoadsComplete();
+  void OnInitialTabLoadsComplete();
 
-  // Add a listener for navigation status notification. Currently only
-  // navigation completion is observed; when the |number_of_navigations|
-  // complete, the completed_response object is sent; if the server requires
-  // authentication, we instead send the auth_needed_response object.  A pointer
-  // to the added navigation observer is returned. This object should NOT be
-  // deleted and should be released by calling the corresponding
-  // RemoveNavigationStatusListener method.
-  NotificationObserver* AddNavigationStatusListener(
-      NavigationController* tab, IPC::Message* reply_message,
-      int number_of_navigations, bool include_current_navigation);
+  // Called when the ChromeOS network library has finished its first update.
+  void OnNetworkLibraryInit();
 
-  void RemoveNavigationStatusListener(NotificationObserver* obs);
-
-  // Add an observer for the TabStrip. Currently only Tab append is observed. A
-  // navigation listener is created on successful notification of tab append. A
-  // pointer to the added navigation observer is returned. This object should
-  // NOT be deleted and should be released by calling the corresponding
-  // RemoveTabStripObserver method.
-  NotificationObserver* AddTabStripObserver(Browser* parent,
-                                            IPC::Message* reply_message);
-  void RemoveTabStripObserver(NotificationObserver* obs);
+  // Called when the chromeos WebUI login is ready.
+  void OnLoginWebuiReady();
 
   // Get the index of a particular NavigationController object
   // in the given parent window.  This method uses
   // TabStrip::GetIndexForNavigationController to get the index.
-  int GetIndexForNavigationController(const NavigationController* controller,
-                                      const Browser* parent) const;
+  int GetIndexForNavigationController(
+      const content::NavigationController* controller,
+      const Browser* parent) const;
 
   // Add or remove a non-owning reference to a tab's LoginHandler.  This is for
   // when a login prompt is shown for HTTP/FTP authentication.
@@ -134,22 +130,17 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   // Eventually we'll probably want ways to interact with the ChromeView of the
   // login window in a generic manner, such that it can be used for anything,
   // not just logins.
-  void AddLoginHandler(NavigationController* tab, LoginHandler* handler);
-  void RemoveLoginHandler(NavigationController* tab);
+  void AddLoginHandler(content::NavigationController* tab,
+                       LoginHandler* handler);
+  void RemoveLoginHandler(content::NavigationController* tab);
 
-  // Add an extension port container.
-  // Takes ownership of the container.
-  void AddPortContainer(ExtensionPortContainer* port);
-  // Remove and delete the port container.
-  void RemovePortContainer(ExtensionPortContainer* port);
-  // Get the port container for the given port id.
-  ExtensionPortContainer* GetPortContainer(int port_id) const;
+  // IPC::Channel::Sender implementation.
+  virtual bool Send(IPC::Message* msg) OVERRIDE;
 
-  // IPC implementations
-  virtual bool Send(IPC::Message* msg);
-  virtual void OnChannelConnected(int pid);
-  virtual bool OnMessageReceived(const IPC::Message& msg);
-  virtual void OnChannelError();
+  // IPC::Channel::Listener implementation.
+  virtual void OnChannelConnected(int pid) OVERRIDE;
+  virtual bool OnMessageReceived(const IPC::Message& msg) OVERRIDE;
+  virtual void OnChannelError() OVERRIDE;
 
   IPC::Message* reply_message_release() {
     IPC::Message* reply_message = reply_message_;
@@ -162,28 +153,32 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   // the handle is simply returned.
   int AddExtension(const Extension* extension);
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   // Adds the external tab passed in to the tab tracker.
   bool AddExternalTab(ExternalTabContainer* external_tab);
 #endif
 
   // Get the DictionaryValue equivalent for a download item. Caller owns the
   // DictionaryValue.
-  DictionaryValue* GetDictionaryFromDownloadItem(const DownloadItem* download);
+  base::DictionaryValue* GetDictionaryFromDownloadItem(
+      const content::DownloadItem* download);
 
  protected:
-  friend class base::RefCounted<AutomationProvider>;
+  friend struct content::BrowserThread::DeleteOnThread<
+      content::BrowserThread::UI>;
+  friend class base::DeleteHelper<AutomationProvider>;
   virtual ~AutomationProvider();
 
   // Helper function to find the browser window that contains a given
   // NavigationController and activate that tab.
   // Returns the Browser if found.
-  Browser* FindAndActivateTab(NavigationController* contents);
+  Browser* FindAndActivateTab(content::NavigationController* contents);
 
-  // Convert a tab handle into a TabContents. If |tab| is non-NULL a pointer
+  // Convert a tab handle into a WebContents. If |tab| is non-NULL a pointer
   // to the tab is also returned. Returns NULL in case of failure or if the tab
-  // is not of the TabContents type.
-  TabContents* GetTabContentsForHandle(int handle, NavigationController** tab);
+  // is not of the WebContents type.
+  content::WebContents* GetWebContentsForHandle(
+      int handle, content::NavigationController** tab);
 
   // Returns the protocol version which typically is the module version.
   virtual std::string GetProtocolVersion();
@@ -192,7 +187,10 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   // Returns NULL on failure.
   RenderViewHost* GetViewForTab(int tab_handle);
 
-  scoped_ptr<AutomationAutocompleteEditTracker> autocomplete_edit_tracker_;
+  // Called on IPC message deserialization failure. Prints an error message
+  // and closes the IPC channel.
+  void OnMessageDeserializationFailure();
+
   scoped_ptr<AutomationBrowserTracker> browser_tracker_;
   scoped_ptr<InitialLoadObserver> initial_load_observer_;
   scoped_ptr<MetricEventDurationObserver> metric_event_duration_observer_;
@@ -200,10 +198,8 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   scoped_ptr<AutomationTabTracker> tab_tracker_;
   scoped_ptr<AutomationWindowTracker> window_tracker_;
 
-  typedef ObserverList<NotificationObserver> NotificationObserverList;
-  NotificationObserverList notification_observer_list_;
-
-  typedef std::map<NavigationController*, LoginHandler*> LoginHandlerMap;
+  typedef std::map<content::NavigationController*, LoginHandler*>
+      LoginHandlerMap;
   LoginHandlerMap login_handler_map_;
 
   Profile* profile_;
@@ -218,7 +214,7 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
 
   // Sends a find request for a given query.
   void SendFindRequest(
-      TabContents* tab_contents,
+      content::WebContents* web_contents,
       bool with_json,
       const string16& search_string,
       bool forward,
@@ -229,12 +225,28 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   scoped_refptr<AutomationResourceMessageFilter>
       automation_resource_message_filter_;
 
+  // True iff we should open a new automation IPC channel if it closes.
+  bool reinitialize_on_channel_error_;
+
  private:
-  void OnUnhandledMessage();
+  // Storage for EndTracing() to resume operations after a callback.
+  struct TracingData {
+    std::list<std::string> trace_output;
+    scoped_ptr<IPC::Message> reply_message;
+  };
+
+  // TraceSubscriber:
+  virtual void OnEndTracingComplete() OVERRIDE;
+  virtual void OnTraceDataCollected(const std::string& trace_fragment) OVERRIDE;
+
+  void OnUnhandledMessage(const IPC::Message& message);
+
+  // Clear and reinitialize the automation IPC channel.
+  bool ReinitializeChannel();
 
   // IPC Message callbacks.
   void WindowSimulateDrag(int handle,
-                          std::vector<gfx::Point> drag_path,
+                          const std::vector<gfx::Point>& drag_path,
                           int flags,
                           bool press_escape_en_route,
                           IPC::Message* reply_message);
@@ -260,19 +272,15 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   // for information on the arguments.
   void JavaScriptStressTestControl(int handle, int cmd, int param);
 
-  void InstallExtension(const FilePath& crx_path,
-                        IPC::Message* reply_message);
-
-  void LoadExpandedExtension(const FilePath& extension_dir,
-                             IPC::Message* reply_message);
-
-  void GetEnabledExtensions(std::vector<FilePath>* result);
+  void BeginTracing(const std::string& categories, bool* success);
+  void EndTracing(IPC::Message* reply_message);
+  void GetTracingOutput(std::string* chunk, bool* success);
 
   void WaitForExtensionTestResult(IPC::Message* reply_message);
 
-  void InstallExtensionAndGetHandle(const FilePath& crx_path,
-                                    bool with_ui,
-                                    IPC::Message* reply_message);
+  void InstallExtension(const FilePath& extension_path,
+                        bool with_ui,
+                        IPC::Message* reply_message);
 
   void UninstallExtension(int extension_handle,
                           bool* success);
@@ -307,11 +315,6 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
                         const std::string& encoding_name,
                         bool* success);
 
-  // Enables extension automation (for e.g. UITests).
-  void SetEnableExtensionAutomation(
-      int tab_handle,
-      const std::vector<std::string>& functions_enabled);
-
   // Selects all contents on the page.
   void SelectAll(int tab_handle);
 
@@ -324,14 +327,6 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   void StopAsync(int tab_handle);
   void SaveAsAsync(int tab_handle);
 
-#if defined(OS_CHROMEOS)
-  // Logs in through the Chrome OS Login Wizard with given |username| and
-  // password.  Returns true via |reply_message| on success.
-  void LoginWithUserAndPass(const std::string& username,
-                            const std::string& password,
-                            IPC::Message* reply_message);
-#endif
-
   // Returns the extension for the given handle. Returns NULL if there is
   // no extension for the handle.
   const Extension* GetExtension(int extension_handle);
@@ -347,7 +342,7 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   // Method called by the popup menu tracker when a popup menu is opened.
   void NotifyPopupMenuOpened();
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   // The functions in this block are for use with external tabs, so they are
   // Windows only.
 
@@ -390,12 +385,6 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
                                  const std::string& origin,
                                  const std::string& target);
 
-  // Determine if the message from the external host represents a browser
-  // event, and if so dispatch it.
-  bool InterceptBrowserEventMessageFromExternalHost(const std::string& message,
-                                                    const std::string& origin,
-                                                    const std::string& target);
-
   void OnBrowserMoved(int handle);
 
   void OnRunUnloadHandlers(int handle, IPC::Message* reply_message);
@@ -403,25 +392,33 @@ class AutomationProvider : public base::RefCounted<AutomationProvider>,
   void OnSetZoomLevel(int handle, int zoom_level);
 
   ExternalTabContainer* GetExternalTabForHandle(int handle);
-#endif  // defined(OS_WIN)
-
-  typedef std::map<int, ExtensionPortContainer*> PortContainerMap;
+#endif  // defined(OS_WIN) && !defined(USE_AURA)
 
   scoped_ptr<IPC::ChannelProxy> channel_;
-  scoped_ptr<NotificationObserver> new_tab_ui_load_observer_;
-  scoped_ptr<NotificationObserver> find_in_page_observer_;
-  scoped_ptr<NotificationObserver> dom_operation_observer_;
-  scoped_ptr<NotificationObserver> dom_inspector_observer_;
+  scoped_ptr<content::NotificationObserver> new_tab_ui_load_observer_;
+  scoped_ptr<content::NotificationObserver> find_in_page_observer_;
   scoped_ptr<ExtensionTestResultNotificationObserver>
       extension_test_result_observer_;
   scoped_ptr<AutomationExtensionTracker> extension_tracker_;
-  PortContainerMap port_containers_;
 
   // True iff connected to an AutomationProxy.
   bool is_connected_;
 
   // True iff browser finished loading initial set of tabs.
-  bool initial_loads_complete_;
+  bool initial_tab_loads_complete_;
+
+  // True iff the Chrome OS network library finished initialization.
+  bool network_library_initialized_;
+
+  // True iff ChromeOS webui login ui is ready.
+  bool login_webui_ready_;
+
+  // ID of automation channel.
+  std::string channel_id_;
+
+  // Trace data that has been collected but not flushed to the automation
+  // client.
+  TracingData tracing_data_;
 
   DISALLOW_COPY_AND_ASSIGN(AutomationProvider);
 };
