@@ -4,50 +4,46 @@
 
 #include "chrome/browser/automation/automation_provider.h"
 
+#include "base/bind.h"
+#include "base/callback.h"
 #include "base/debug/trace_event.h"
 #include "base/json/json_reader.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/automation/automation_browser_tracker.h"
-#include "chrome/browser/automation/automation_extension_function.h"
 #include "chrome/browser/automation/automation_tab_tracker.h"
 #include "chrome/browser/automation/automation_window_tracker.h"
-#include "chrome/browser/automation/extension_automation_constants.h"
-#include "chrome/browser/automation/extension_port_container.h"
 #include "chrome/browser/automation/ui_controls.h"
-#include "chrome/browser/browser_window.h"
-#include "chrome/browser/extensions/extension_event_router.h"
 #include "chrome/browser/external_tab_container_win.h"
+#include "chrome/browser/printing/print_view_manager.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/ui/views/bookmark_bar_view.h"
+#include "chrome/browser/sessions/restore_tab_helper.h"
+#include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/browser/ui/views/bookmarks/bookmark_bar_view.h"
 #include "chrome/common/automation_messages.h"
-#include "chrome/common/page_zoom.h"
+#include "chrome/common/render_messages.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/common/page_zoom.h"
 #include "ui/base/keycodes/keyboard_codes.h"
-#include "views/focus/accelerator_handler.h"
-#include "views/widget/root_view.h"
-#include "views/widget/widget_win.h"
-#include "views/window/window.h"
+#include "ui/views/focus/accelerator_handler.h"
+#include "ui/views/widget/root_view.h"
 
-// This task just adds another task to the event queue.  This is useful if
-// you want to ensure that any tasks added to the event queue after this one
-// have already been processed by the time |task| is run.
-class InvokeTaskLaterTask : public Task {
- public:
-  explicit InvokeTaskLaterTask(Task* task) : task_(task) {}
-  virtual ~InvokeTaskLaterTask() {}
+using content::NavigationController;
+using content::WebContents;
 
-  virtual void Run() {
-    MessageLoop::current()->PostTask(FROM_HERE, task_);
-  }
+namespace {
 
- private:
-  Task* task_;
+// This callback just adds another callback to the event queue. This is useful
+// if you want to ensure that any callbacks added to the event queue after this
+// one have already been processed by the time |callback| is run.
+void InvokeCallbackLater(const base::Closure& callback) {
+  MessageLoop::current()->PostTask(FROM_HERE, callback);
+}
 
-  DISALLOW_COPY_AND_ASSIGN(InvokeTaskLaterTask);
-};
-
-static void MoveMouse(const POINT& point) {
+void MoveMouse(const POINT& point) {
   SetCursorPos(point.x, point.y);
 
   // Now, make sure that GetMessagePos returns the values we just set by
@@ -74,84 +70,25 @@ BOOL CALLBACK EnumThreadWndProc(HWND hwnd, LPARAM l_param) {
   return TRUE;
 }
 
-// This task enqueues a mouse event on the event loop, so that the view
-// that it's being sent to can do the requisite post-processing.
-class MouseEventTask : public Task {
- public:
-  MouseEventTask(views::View* view,
-                 views::Event::EventType type,
-                 const gfx::Point& point,
-                 int flags)
-      : view_(view), type_(type), point_(point), flags_(flags) {}
-  virtual ~MouseEventTask() {}
+// This callback sends a WindowDragResponse message with the appropriate routing
+// ID to the automation proxy.  This is implemented as a task so that we know
+// that the mouse events (and any tasks that they spawn on the message loop)
+// have been processed by the time this is sent.
+void WindowDragResponseCallback(AutomationProvider* provider,
+                                IPC::Message* reply_message) {
+  DCHECK(reply_message != NULL);
+  AutomationMsg_WindowDrag::WriteReplyParams(reply_message, true);
+  provider->Send(reply_message);
+}
 
-  virtual void Run() {
-    views::MouseEvent event(type_, point_.x(), point_.y(), flags_);
-    // We need to set the cursor position before we process the event because
-    // some code (tab dragging, for instance) queries the actual cursor location
-    // rather than the location of the mouse event. Note that the reason why
-    // the drag code moved away from using mouse event locations was because
-    // our conversion to screen location doesn't work well with multiple
-    // monitors, so this only works reliably in a single monitor setup.
-    gfx::Point screen_location(point_.x(), point_.y());
-    view_->ConvertPointToScreen(view_, &screen_location);
-    MoveMouse(screen_location.ToPOINT());
-    switch (type_) {
-      case views::Event::ET_MOUSE_PRESSED:
-        view_->OnMousePressed(event);
-        break;
+}  // namespace
 
-      case views::Event::ET_MOUSE_DRAGGED:
-        view_->OnMouseDragged(event);
-        break;
-
-      case views::Event::ET_MOUSE_RELEASED:
-        view_->OnMouseReleased(event, false);
-        break;
-
-      default:
-        NOTREACHED();
-    }
-  }
-
- private:
-  views::View* view_;
-  views::Event::EventType type_;
-  gfx::Point point_;
-  int flags_;
-
-  DISALLOW_COPY_AND_ASSIGN(MouseEventTask);
-};
-
-// This task sends a WindowDragResponse message with the appropriate
-// routing ID to the automation proxy.  This is implemented as a task so that
-// we know that the mouse events (and any tasks that they spawn on the message
-// loop) have been processed by the time this is sent.
-class WindowDragResponseTask : public Task {
- public:
-  WindowDragResponseTask(AutomationProvider* provider,
-                         IPC::Message* reply_message)
-      : provider_(provider), reply_message_(reply_message) {}
-  virtual ~WindowDragResponseTask() {}
-
-  virtual void Run() {
-    DCHECK(reply_message_ != NULL);
-    AutomationMsg_WindowDrag::WriteReplyParams(reply_message_, true);
-    provider_->Send(reply_message_);
-  }
-
- private:
-  AutomationProvider* provider_;
-  IPC::Message* reply_message_;
-
-  DISALLOW_COPY_AND_ASSIGN(WindowDragResponseTask);
-};
-
-void AutomationProvider::WindowSimulateDrag(int handle,
-                                            std::vector<gfx::Point> drag_path,
-                                            int flags,
-                                            bool press_escape_en_route,
-                                            IPC::Message* reply_message) {
+void AutomationProvider::WindowSimulateDrag(
+    int handle,
+    const std::vector<gfx::Point>& drag_path,
+    int flags,
+    bool press_escape_en_route,
+    IPC::Message* reply_message) {
   if (browser_tracker_->ContainsHandle(handle) && (drag_path.size() > 1)) {
     gfx::NativeWindow window =
         browser_tracker_->GetResource(handle)->window()->GetNativeHandle();
@@ -159,21 +96,21 @@ void AutomationProvider::WindowSimulateDrag(int handle,
     UINT down_message = 0;
     UINT up_message = 0;
     WPARAM wparam_flags = 0;
-    if (flags & views::Event::EF_SHIFT_DOWN)
+    if (flags & ui::EF_SHIFT_DOWN)
       wparam_flags |= MK_SHIFT;
-    if (flags & views::Event::EF_CONTROL_DOWN)
+    if (flags & ui::EF_CONTROL_DOWN)
       wparam_flags |= MK_CONTROL;
-    if (flags & views::Event::EF_LEFT_BUTTON_DOWN) {
+    if (flags & ui::EF_LEFT_MOUSE_BUTTON) {
       wparam_flags |= MK_LBUTTON;
       down_message = WM_LBUTTONDOWN;
       up_message = WM_LBUTTONUP;
     }
-    if (flags & views::Event::EF_MIDDLE_BUTTON_DOWN) {
+    if (flags & ui::EF_MIDDLE_MOUSE_BUTTON) {
       wparam_flags |= MK_MBUTTON;
       down_message = WM_MBUTTONDOWN;
       up_message = WM_MBUTTONUP;
     }
-    if (flags & views::Event::EF_RIGHT_BUTTON_DOWN) {
+    if (flags & ui::EF_RIGHT_MOUSE_BUTTON) {
       wparam_flags |= MK_RBUTTON;
       down_message = WM_LBUTTONDOWN;
       up_message = WM_LBUTTONUP;
@@ -204,25 +141,27 @@ void AutomationProvider::WindowSimulateDrag(int handle,
       // TODO(phajdan.jr): make this use ui_test_utils::SendKeyPressSync.
       ui_controls::SendKeyPressNotifyWhenDone(
           window, ui::VKEY_ESCAPE,
-          ((flags & views::Event::EF_CONTROL_DOWN) ==
-           views::Event::EF_CONTROL_DOWN),
-          ((flags & views::Event::EF_SHIFT_DOWN) ==
-           views::Event::EF_SHIFT_DOWN),
-          ((flags & views::Event::EF_ALT_DOWN) == views::Event::EF_ALT_DOWN),
+          ((flags & ui::EF_CONTROL_DOWN) ==
+           ui::EF_CONTROL_DOWN),
+          ((flags & ui::EF_SHIFT_DOWN) ==
+           ui::EF_SHIFT_DOWN),
+          ((flags & ui::EF_ALT_DOWN) == ui::EF_ALT_DOWN),
           false,
-          new MessageLoop::QuitTask());
+          MessageLoop::QuitClosure());
       MessageLoopForUI* loop = MessageLoopForUI::current();
       bool did_allow_task_nesting = loop->NestableTasksAllowed();
       loop->SetNestableTasksAllowed(true);
       views::AcceleratorHandler handler;
-      loop->Run(&handler);
+      loop->RunWithDispatcher(&handler);
       loop->SetNestableTasksAllowed(did_allow_task_nesting);
     }
     SendMessage(top_level_hwnd, up_message, wparam_flags,
                 MAKELPARAM(end.x, end.y));
 
-    MessageLoop::current()->PostTask(FROM_HERE, new InvokeTaskLaterTask(
-        new WindowDragResponseTask(this, reply_message)));
+    MessageLoop::current()->PostTask(
+        FROM_HERE, base::Bind(
+            &InvokeCallbackLater,
+            base::Bind(&WindowDragResponseCallback, this, reply_message)));
   } else {
     AutomationMsg_WindowDrag::WriteReplyParams(reply_message, false);
     Send(reply_message);
@@ -233,7 +172,7 @@ void AutomationProvider::CreateExternalTab(
     const ExternalTabSettings& settings,
     gfx::NativeWindow* tab_container_window, gfx::NativeWindow* tab_window,
     int* tab_handle, int* session_id) {
-  TRACE_EVENT_BEGIN("AutomationProvider::CreateExternalTab", 0, "");
+  TRACE_EVENT_BEGIN_ETW("AutomationProvider::CreateExternalTab", 0, "");
 
   *tab_handle = 0;
   *tab_container_window = NULL;
@@ -242,7 +181,7 @@ void AutomationProvider::CreateExternalTab(
   scoped_refptr<ExternalTabContainer> external_tab_container =
       new ExternalTabContainer(this, automation_resource_message_filter_);
 
-  Profile* profile = settings.is_off_the_record ?
+  Profile* profile = settings.is_incognito ?
       profile_->GetOffTheRecordProfile() : profile_;
 
   // When the ExternalTabContainer window is created we grab a reference on it
@@ -254,24 +193,25 @@ void AutomationProvider::CreateExternalTab(
       settings.route_all_top_level_navigations);
 
   if (AddExternalTab(external_tab_container)) {
-    TabContents* tab_contents = external_tab_container->tab_contents();
+    WebContents* web_contents = external_tab_container->web_contents();
     *tab_handle = external_tab_container->tab_handle();
     *tab_container_window = external_tab_container->GetNativeView();
-    *tab_window = tab_contents->GetNativeView();
-    *session_id = tab_contents->controller().session_id().id();
+    *tab_window = web_contents->GetNativeView();
+    *session_id = external_tab_container->tab_contents_wrapper()->
+        restore_tab_helper()->session_id().id();
   } else {
     external_tab_container->Uninitialize();
   }
 
-  TRACE_EVENT_END("AutomationProvider::CreateExternalTab", 0, "");
+  TRACE_EVENT_END_ETW("AutomationProvider::CreateExternalTab", 0, "");
 }
 
 bool AutomationProvider::AddExternalTab(ExternalTabContainer* external_tab) {
   DCHECK(external_tab != NULL);
 
-  TabContents* tab_contents = external_tab->tab_contents();
-  if (tab_contents) {
-    int tab_handle = tab_tracker_->Add(&tab_contents->controller());
+  WebContents* web_contents = external_tab->web_contents();
+  if (web_contents) {
+    int tab_handle = tab_tracker_->Add(&web_contents->GetController());
     external_tab->SetTabHandle(tab_handle);
     return true;
   }
@@ -299,19 +239,20 @@ void AutomationProvider::SetInitialFocus(const IPC::Message& message,
 }
 
 void AutomationProvider::PrintAsync(int tab_handle) {
-  NavigationController* tab = NULL;
-  TabContents* tab_contents = GetTabContentsForHandle(tab_handle, &tab);
-  if (tab_contents) {
-    if (tab_contents->PrintNow())
-      return;
-  }
+  WebContents* web_contents = GetWebContentsForHandle(tab_handle, NULL);
+  if (!web_contents)
+    return;
+
+  TabContentsWrapper* wrapper =
+      TabContentsWrapper::GetCurrentWrapperForContents(web_contents);
+  wrapper->print_view_manager()->PrintNow();
 }
 
 ExternalTabContainer* AutomationProvider::GetExternalTabForHandle(int handle) {
   if (tab_tracker_->ContainsHandle(handle)) {
     NavigationController* tab = tab_tracker_->GetResource(handle);
     return ExternalTabContainer::GetContainerForTab(
-        tab->tab_contents()->GetNativeView());
+        tab->GetWebContents()->GetNativeView());
   }
 
   return NULL;
@@ -348,21 +289,22 @@ void AutomationProvider::OnTabReposition(
 
 void AutomationProvider::OnForwardContextMenuCommandToChrome(int tab_handle,
                                                              int command) {
-  if (tab_tracker_->ContainsHandle(tab_handle)) {
-    NavigationController* tab = tab_tracker_->GetResource(tab_handle);
-    if (!tab) {
-      NOTREACHED();
-      return;
-    }
+  if (!tab_tracker_->ContainsHandle(tab_handle))
+    return;
 
-    TabContents* tab_contents = tab->tab_contents();
-    if (!tab_contents || !tab_contents->delegate()) {
-      NOTREACHED();
-      return;
-    }
-
-    tab_contents->delegate()->ExecuteContextMenuCommand(command);
+  NavigationController* tab = tab_tracker_->GetResource(tab_handle);
+  if (!tab) {
+    NOTREACHED();
+    return;
   }
+
+  WebContents* web_contents = tab->GetWebContents();
+  if (!web_contents || !web_contents->GetDelegate()) {
+    NOTREACHED();
+    return;
+  }
+
+  web_contents->GetDelegate()->ExecuteContextMenuCommand(command);
 }
 
 void AutomationProvider::ConnectExternalTab(
@@ -373,7 +315,7 @@ void AutomationProvider::ConnectExternalTab(
     gfx::NativeWindow* tab_window,
     int* tab_handle,
     int* session_id) {
-  TRACE_EVENT_BEGIN("AutomationProvider::ConnectExternalTab", 0, "");
+  TRACE_EVENT_BEGIN_ETW("AutomationProvider::ConnectExternalTab", 0, "");
 
   *tab_handle = 0;
   *tab_container_window = NULL;
@@ -391,38 +333,22 @@ void AutomationProvider::ConnectExternalTab(
     external_tab_container->Reinitialize(this,
                                          automation_resource_message_filter_,
                                          parent_window);
-    TabContents* tab_contents = external_tab_container->tab_contents();
+    WebContents* tab_contents = external_tab_container->web_contents();
     *tab_handle = external_tab_container->tab_handle();
     *tab_container_window = external_tab_container->GetNativeView();
     *tab_window = tab_contents->GetNativeView();
-    *session_id = tab_contents->controller().session_id().id();
+    *session_id = external_tab_container->tab_contents_wrapper()->
+        restore_tab_helper()->session_id().id();
   } else {
     external_tab_container->Uninitialize();
   }
 
-  TRACE_EVENT_END("AutomationProvider::ConnectExternalTab", 0, "");
-}
-
-void AutomationProvider::SetEnableExtensionAutomation(
-    int tab_handle,
-    const std::vector<std::string>& functions_enabled) {
-  ExternalTabContainer* external_tab = GetExternalTabForHandle(tab_handle);
-  if (external_tab) {
-    external_tab->SetEnableExtensionAutomation(functions_enabled);
-  } else {
-    // Tab must exist, and must be an external tab so that its
-    // delegate has an on-empty
-    // implementation of ForwardMessageToExternalHost.
-    DLOG(WARNING) <<
-      "SetEnableExtensionAutomation called with invalid tab handle.";
-  }
+  TRACE_EVENT_END_ETW("AutomationProvider::ConnectExternalTab", 0, "");
 }
 
 void AutomationProvider::OnBrowserMoved(int tab_handle) {
   ExternalTabContainer* external_tab = GetExternalTabForHandle(tab_handle);
-  if (external_tab) {
-    external_tab->WindowMoved();
-  } else {
+  if (!external_tab) {
     DLOG(WARNING) <<
       "AutomationProvider::OnBrowserMoved called with invalid tab handle.";
   }
@@ -436,67 +362,8 @@ void AutomationProvider::OnMessageFromExternalHost(int handle,
   if (!view_host)
     return;
 
-  if (AutomationExtensionFunction::InterceptMessageFromExternalHost(
-          view_host, message, origin, target)) {
-    // Message was diverted.
-    return;
-  }
-
-  if (ExtensionPortContainer::InterceptMessageFromExternalHost(
-          message, origin, target, this, view_host, handle)) {
-    // Message was diverted.
-    return;
-  }
-
-  if (InterceptBrowserEventMessageFromExternalHost(message, origin, target)) {
-    // Message was diverted.
-    return;
-  }
-
-  view_host->ForwardMessageFromExternalHost(message, origin, target);
-}
-
-bool AutomationProvider::InterceptBrowserEventMessageFromExternalHost(
-      const std::string& message, const std::string& origin,
-      const std::string& target) {
-  if (target !=
-      extension_automation_constants::kAutomationBrowserEventRequestTarget)
-    return false;
-
-  if (origin != extension_automation_constants::kAutomationOrigin) {
-    LOG(WARNING) << "Wrong origin on automation browser event " << origin;
-    return false;
-  }
-
-  // The message is a JSON-encoded array with two elements, both strings. The
-  // first is the name of the event to dispatch.  The second is a JSON-encoding
-  // of the arguments specific to that event.
-  scoped_ptr<Value> message_value(base::JSONReader::Read(message, false));
-  if (!message_value.get() || !message_value->IsType(Value::TYPE_LIST)) {
-    LOG(WARNING) << "Invalid browser event specified through automation";
-    return false;
-  }
-
-  const ListValue* args = static_cast<const ListValue*>(message_value.get());
-
-  std::string event_name;
-  if (!args->GetString(0, &event_name)) {
-    LOG(WARNING) << "No browser event name specified through automation";
-    return false;
-  }
-
-  std::string json_args;
-  if (!args->GetString(1, &json_args)) {
-    LOG(WARNING) << "No browser event args specified through automation";
-    return false;
-  }
-
-  if (profile()->GetExtensionEventRouter()) {
-    profile()->GetExtensionEventRouter()->DispatchEventToRenderers(
-        event_name, json_args, profile(), GURL());
-  }
-
-  return true;
+  view_host->Send(new ChromeViewMsg_HandleMessageFromExternalHost(
+      view_host->routing_id(), message, origin, target));
 }
 
 void AutomationProvider::NavigateInExternalTab(
@@ -506,7 +373,10 @@ void AutomationProvider::NavigateInExternalTab(
 
   if (tab_tracker_->ContainsHandle(handle)) {
     NavigationController* tab = tab_tracker_->GetResource(handle);
-    tab->LoadURL(url, referrer, PageTransition::TYPED);
+    tab->LoadURL(
+        url,
+        content::Referrer(referrer, WebKit::WebReferrerPolicyDefault),
+        content::PAGE_TRANSITION_TYPED, std::string());
     *status = AUTOMATION_MSG_NAVIGATION_SUCCESS;
   }
 }
@@ -534,9 +404,10 @@ void AutomationProvider::OnRunUnloadHandlers(
 void AutomationProvider::OnSetZoomLevel(int handle, int zoom_level) {
   if (tab_tracker_->ContainsHandle(handle)) {
     NavigationController* tab = tab_tracker_->GetResource(handle);
-    if (tab->tab_contents() && tab->tab_contents()->render_view_host()) {
-      tab->tab_contents()->render_view_host()->Zoom(
-          static_cast<PageZoom::Function>(zoom_level));
+    if (tab->GetWebContents() && tab->GetWebContents()->GetRenderViewHost()) {
+      RenderViewHost* host = tab->GetWebContents()->GetRenderViewHost();
+      content::PageZoom zoom = static_cast<content::PageZoom>(zoom_level);
+      host->Zoom(zoom);
     }
   }
 }
