@@ -1,87 +1,40 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/automation/automation_resource_message_filter.h"
 
-#include "base/path_service.h"
+#include "base/bind.h"
 #include "base/metrics/histogram.h"
-#include "base/stl_util-inl.h"
+#include "base/path_service.h"
+#include "base/stl_util.h"
 #include "chrome/browser/automation/url_request_automation_job.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/net/url_request_failed_dns_job.h"
-#include "chrome/browser/net/url_request_mock_http_job.h"
+#include "chrome/browser/content_settings/tab_specific_content_settings.h"
 #include "chrome/browser/net/url_request_mock_util.h"
-#include "chrome/browser/net/url_request_slow_download_job.h"
-#include "chrome/browser/net/url_request_slow_http_job.h"
-#include "chrome/browser/renderer_host/render_message_filter.h"
 #include "chrome/common/automation_messages.h"
 #include "chrome/common/chrome_paths.h"
+#include "chrome/common/render_messages.h"
+#include "content/public/browser/browser_message_filter.h"
+#include "content/public/browser/browser_thread.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/net_errors.h"
+#include "net/url_request/url_request_context.h"
 #include "net/url_request/url_request_filter.h"
 
+using content::BrowserMessageFilter;
+using content::BrowserThread;
+
 base::LazyInstance<AutomationResourceMessageFilter::RenderViewMap>
-    AutomationResourceMessageFilter::filtered_render_views_(
-        base::LINKER_INITIALIZED);
+    AutomationResourceMessageFilter::filtered_render_views_ =
+        LAZY_INSTANCE_INITIALIZER;
 
 base::LazyInstance<AutomationResourceMessageFilter::CompletionCallbackMap>
-    AutomationResourceMessageFilter::completion_callback_map_(
-        base::LINKER_INITIALIZED);
+    AutomationResourceMessageFilter::completion_callback_map_ =
+        LAZY_INSTANCE_INITIALIZER;
 
 int AutomationResourceMessageFilter::unique_request_id_ = 1;
 int AutomationResourceMessageFilter::next_completion_callback_id_ = 0;
 
-// CookieStore specialization to enable fetching and setting cookies over the
-// automation channel. This cookie store is transient i.e. it maintains cookies
-// for the duration of the current request to set or get cookies from the
-// renderer.
-class AutomationCookieStore : public net::CookieStore {
- public:
-  AutomationCookieStore(AutomationResourceMessageFilter* automation_client,
-                        int tab_handle)
-      : automation_client_(automation_client),
-        tab_handle_(tab_handle) {
-  }
-
-  virtual ~AutomationCookieStore() {
-    DVLOG(1) << "In " << __FUNCTION__;
-  }
-
-  // CookieStore implementation.
-  virtual bool SetCookieWithOptions(const GURL& url,
-                                    const std::string& cookie_line,
-                                    const net::CookieOptions& options) {
-    // The cookie_string_ is available only once, i.e. once it is read by
-    // it is invalidated.
-    cookie_string_ = cookie_line;
-    return true;
-  }
-
-  virtual std::string GetCookiesWithOptions(const GURL& url,
-                                            const net::CookieOptions& options) {
-    return cookie_string_;
-  }
-
-  virtual void DeleteCookie(const GURL& url,
-                            const std::string& cookie_name) {
-    NOTREACHED() << "Should not get called for an automation profile";
-  }
-
-  virtual net::CookieMonster* GetCookieMonster() {
-    NOTREACHED() << "Should not get called for an automation profile";
-    return NULL;
-  }
-
- protected:
-  scoped_refptr<AutomationResourceMessageFilter> automation_client_;
-  int tab_handle_;
-  std::string cookie_string_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(AutomationCookieStore);
-};
-
 AutomationResourceMessageFilter::AutomationDetails::AutomationDetails()
     : tab_handle(0),
       ref_count(1),
@@ -99,8 +52,11 @@ AutomationResourceMessageFilter::AutomationDetails::AutomationDetails(
 AutomationResourceMessageFilter::AutomationDetails::~AutomationDetails() {}
 
 struct AutomationResourceMessageFilter::CookieCompletionInfo {
-  net::CompletionCallback* completion_callback;
-  scoped_refptr<net::CookieStore> cookie_store;
+  scoped_refptr<BrowserMessageFilter> filter;
+  scoped_refptr<net::URLRequestContext> context;
+  int render_process_id;
+  IPC::Message* reply_msg;
+  scoped_refptr<AutomationResourceMessageFilter> automation_message_filter;
 };
 
 AutomationResourceMessageFilter::AutomationResourceMessageFilter()
@@ -112,8 +68,7 @@ AutomationResourceMessageFilter::AutomationResourceMessageFilter()
 
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableFunction(
-          URLRequestAutomationJob::EnsureProtocolFactoryRegistered));
+      base::Bind(&URLRequestAutomationJob::EnsureProtocolFactoryRegistered));
 }
 
 AutomationResourceMessageFilter::~AutomationResourceMessageFilter() {
@@ -121,7 +76,7 @@ AutomationResourceMessageFilter::~AutomationResourceMessageFilter() {
 
 // Called on the IPC thread:
 void AutomationResourceMessageFilter::OnFilterAdded(IPC::Channel* channel) {
-  DCHECK(channel_ == NULL);
+  DCHECK(!channel_);
   channel_ = channel;
 }
 
@@ -149,6 +104,18 @@ void AutomationResourceMessageFilter::OnChannelClosing() {
       index++;
     }
   }
+
+  CompletionCallbackMap::iterator callback_index =
+      completion_callback_map_.Get().begin();
+  while (callback_index != completion_callback_map_.Get().end()) {
+    const CookieCompletionInfo& cookie_completion_info =
+        (*callback_index).second;
+    if (cookie_completion_info.automation_message_filter.get() == this) {
+      completion_callback_map_.Get().erase(callback_index++);
+    } else {
+      callback_index++;
+    }
+  }
 }
 
 // Called on the IPC thread:
@@ -174,7 +141,10 @@ bool AutomationResourceMessageFilter::OnMessageReceived(
   }
 
   bool handled = true;
-  IPC_BEGIN_MESSAGE_MAP(AutomationResourceMessageFilter, message)
+  bool deserialize_success = true;
+  IPC_BEGIN_MESSAGE_MAP_EX(AutomationResourceMessageFilter,
+                           message,
+                           deserialize_success)
     IPC_MESSAGE_HANDLER(AutomationMsg_SetFilteredInet,
                         OnSetFilteredInet)
     IPC_MESSAGE_HANDLER(AutomationMsg_GetFilteredInetHitCount,
@@ -184,7 +154,13 @@ bool AutomationResourceMessageFilter::OnMessageReceived(
     IPC_MESSAGE_HANDLER(AutomationMsg_GetCookiesHostResponse,
                         OnGetCookiesHostResponse)
     IPC_MESSAGE_UNHANDLED(handled = false)
-  IPC_END_MESSAGE_MAP()
+  IPC_END_MESSAGE_MAP_EX()
+
+  if (!deserialize_success) {
+    LOG(ERROR) << "Failed to deserialize IPC message. "
+               << "Closing the automation channel.";
+    channel_->Close();
+  }
 
   return handled;
 }
@@ -250,13 +226,9 @@ bool AutomationResourceMessageFilter::RegisterRenderView(
 
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableFunction(
-          AutomationResourceMessageFilter::RegisterRenderViewInIOThread,
-          renderer_pid,
-          renderer_id,
-          tab_handle,
-          make_scoped_refptr(filter),
-          pending_view));
+      base::Bind(&AutomationResourceMessageFilter::RegisterRenderViewInIOThread,
+                 renderer_pid, renderer_id, tab_handle,
+                 make_scoped_refptr(filter), pending_view));
   return true;
 }
 
@@ -264,8 +236,8 @@ void AutomationResourceMessageFilter::UnRegisterRenderView(
     int renderer_pid, int renderer_id) {
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableFunction(
-          AutomationResourceMessageFilter::UnRegisterRenderViewInIOThread,
+      base::Bind(
+          &AutomationResourceMessageFilter::UnRegisterRenderViewInIOThread,
           renderer_pid, renderer_id));
 }
 
@@ -279,12 +251,9 @@ bool AutomationResourceMessageFilter::ResumePendingRenderView(
 
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableFunction(
-          AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread,
-          renderer_pid,
-          renderer_id,
-          tab_handle,
-          make_scoped_refptr(filter)));
+      base::Bind(
+          &AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread,
+          renderer_pid, renderer_id, tab_handle, make_scoped_refptr(filter)));
   return true;
 }
 
@@ -294,20 +263,30 @@ void AutomationResourceMessageFilter::RegisterRenderViewInIOThread(
     bool pending_view) {
   RendererId renderer_key(renderer_pid, renderer_id);
 
-  scoped_refptr<net::CookieStore> cookie_store(
-      new AutomationCookieStore(filter, tab_handle));
-
   RenderViewMap::iterator automation_details_iter(
       filtered_render_views_.Get().find(renderer_key));
-  if (automation_details_iter != filtered_render_views_.Get().end()) {
-    DCHECK(automation_details_iter->second.ref_count > 0);
+  // We need to match the renderer key and the AutomationResourceMessageFilter
+  // instances. If the filter instances are different it means that a new
+  // automation channel (External host process) was created for this tab.
+  if (automation_details_iter != filtered_render_views_.Get().end() &&
+      automation_details_iter->second.filter == filter) {
+    DCHECK_GT(automation_details_iter->second.ref_count, 0);
     automation_details_iter->second.ref_count++;
+    // The tab handle and the pending status may have changed:-
+    // 1.A external tab container is being destroyed and a new one is being
+    //   created.
+    // 2.The external tab container being destroyed receives a RVH created
+    //   notification for the new RVH created to host the newly created tab.
+    //   In this case the tab handle in the AutomationDetails structure would
+    //   be invalid as it points to a destroyed tab.
+    // We need to replace the handle of the external tab being destroyed with
+    // the new one that is being created."
+    automation_details_iter->second.tab_handle = tab_handle;
+    automation_details_iter->second.is_pending_render_view = pending_view;
   } else {
     filtered_render_views_.Get()[renderer_key] =
         AutomationDetails(tab_handle, filter, pending_view);
   }
-
-  filtered_render_views_.Get()[renderer_key].set_cookie_store(cookie_store);
 }
 
 // static
@@ -318,7 +297,8 @@ void AutomationResourceMessageFilter::UnRegisterRenderViewInIOThread(
                                                    renderer_id)));
 
   if (automation_details_iter == filtered_render_views_.Get().end()) {
-    VLOG(1) << "UnRegisterRenderViewInIOThread: already unregistered";
+    // This is called for all RenderViewHosts, so it's fine if we don't find a
+    // match.
     return;
   }
 
@@ -330,7 +310,7 @@ void AutomationResourceMessageFilter::UnRegisterRenderViewInIOThread(
 }
 
 // static
-bool AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread(
+void AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread(
     int renderer_pid, int renderer_id, int tab_handle,
     AutomationResourceMessageFilter* filter) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
@@ -340,19 +320,12 @@ bool AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread(
   RenderViewMap::iterator automation_details_iter(
       filtered_render_views_.Get().find(renderer_key));
 
-  if (automation_details_iter == filtered_render_views_.Get().end()) {
-    NOTREACHED() << "Failed to find pending view for renderer pid:"
-                 << renderer_pid
-                 << ", render view id:"
-                 << renderer_id;
-    return false;
-  }
+  DCHECK(automation_details_iter != filtered_render_views_.Get().end())
+      << "Failed to find pending view for renderer pid:"
+      << renderer_pid << ", render view id:" << renderer_id;
 
   DCHECK(automation_details_iter->second.is_pending_render_view);
 
-  scoped_refptr<net::CookieStore> cookie_store(
-      new AutomationCookieStore(filter, tab_handle));
-
   AutomationResourceMessageFilter* old_filter =
       automation_details_iter->second.filter;
   DCHECK(old_filter != NULL);
@@ -360,10 +333,7 @@ bool AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread(
   filtered_render_views_.Get()[renderer_key] =
       AutomationDetails(tab_handle, filter, false);
 
-  filtered_render_views_.Get()[renderer_key].set_cookie_store(cookie_store);
-
   ResumeJobsForPendingView(tab_handle, old_filter, filter);
-  return true;
 }
 
 bool AutomationResourceMessageFilter::LookupRegisteredRenderView(
@@ -427,30 +397,37 @@ void AutomationResourceMessageFilter::OnRecordHistograms(
   }
 }
 
-bool AutomationResourceMessageFilter::GetCookiesForUrl(
-    const GURL& url, net::CompletionCallback* callback) {
-  GetCookiesCompletion* get_cookies_callback =
-      static_cast<GetCookiesCompletion*>(callback);
+bool AutomationResourceMessageFilter::ShouldFilterCookieMessages(
+    int render_process_id, int render_view_id) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
+  RendererId renderer_key(render_process_id, render_view_id);
+  RenderViewMap::iterator automation_details_iter(
+      filtered_render_views_.Get().find(renderer_key));
+  return automation_details_iter != filtered_render_views_.Get().end();
+}
 
-  RendererId renderer_key(get_cookies_callback->render_process_id(),
-      get_cookies_callback->render_view_id());
+void AutomationResourceMessageFilter::GetCookiesForUrl(
+    BrowserMessageFilter* filter, net::URLRequestContext* context,
+    int render_process_id, IPC::Message* reply_msg, const GURL& url) {
+
+  RendererId renderer_key(render_process_id, reply_msg->routing_id());
 
   RenderViewMap::iterator automation_details_iter(
-        filtered_render_views_.Get().find(renderer_key));
-
-  if (automation_details_iter == filtered_render_views_.Get().end()) {
-    return false;
-  }
+      filtered_render_views_.Get().find(renderer_key));
 
+  DCHECK(automation_details_iter != filtered_render_views_.Get().end());
   DCHECK(automation_details_iter->second.filter != NULL);
-  DCHECK(automation_details_iter->second.cookie_store_.get() != NULL);
 
   int completion_callback_id = GetNextCompletionCallbackId();
   DCHECK(!ContainsKey(completion_callback_map_.Get(), completion_callback_id));
 
   CookieCompletionInfo cookie_info;
-  cookie_info.completion_callback = callback;
-  cookie_info.cookie_store = automation_details_iter->second.cookie_store_;
+  cookie_info.filter = filter;
+  cookie_info.context = context;
+  cookie_info.render_process_id = render_process_id;
+  cookie_info.reply_msg = reply_msg;
+  cookie_info.automation_message_filter =
+      automation_details_iter->second.filter;
 
   completion_callback_map_.Get()[completion_callback_id] = cookie_info;
 
@@ -462,7 +439,6 @@ bool AutomationResourceMessageFilter::GetCookiesForUrl(
             automation_details_iter->second.tab_handle, url,
             completion_callback_id));
   }
-  return true;
 }
 
 void AutomationResourceMessageFilter::OnGetCookiesHostResponse(
@@ -470,61 +446,28 @@ void AutomationResourceMessageFilter::OnGetCookiesHostResponse(
     int cookie_id) {
   CompletionCallbackMap::iterator index =
       completion_callback_map_.Get().find(cookie_id);
-  if (index != completion_callback_map_.Get().end()) {
-    net::CompletionCallback* callback = index->second.completion_callback;
-
-    scoped_refptr<net::CookieStore> cookie_store = index->second.cookie_store;
-
-    DCHECK(callback != NULL);
-    DCHECK(cookie_store.get() != NULL);
-
-    completion_callback_map_.Get().erase(index);
-
-    OnGetCookiesHostResponseInternal(tab_handle, success, url, cookies,
-                                     callback, cookie_store.get());
-  } else {
+  if (index == completion_callback_map_.Get().end()) {
     NOTREACHED() << "Received invalid completion callback id:"
                  << cookie_id;
+    return;
   }
+
+  ChromeViewHostMsg_GetCookies::WriteReplyParams(index->second.reply_msg,
+                                                 cookies);
+  index->second.filter->Send(index->second.reply_msg);
+
+  completion_callback_map_.Get().erase(index);
 }
 
-void AutomationResourceMessageFilter::OnGetCookiesHostResponseInternal(
-    int tab_handle, bool success, const GURL& url, const std::string& cookies,
-  net::CompletionCallback* callback, net::CookieStore* cookie_store) {
-  DCHECK(callback);
-  DCHECK(cookie_store);
-
-  GetCookiesCompletion* get_cookies_callback =
-      static_cast<GetCookiesCompletion*>(callback);
-
-  get_cookies_callback->set_cookie_store(cookie_store);
-
-  // Set the cookie in the cookie store so that the callback can read it.
-  cookie_store->SetCookieWithOptions(url, cookies, net::CookieOptions());
-
-  Tuple1<int> params;
-  params.a = success ? net::OK : net::ERR_ACCESS_DENIED;
-  callback->RunWithParams(params);
-
-  // The cookie for this URL is only valid until it is read by the callback.
-  cookie_store->SetCookieWithOptions(url, "", net::CookieOptions());
-}
-
-bool AutomationResourceMessageFilter::SetCookiesForUrl(
-    const GURL& url, const std::string& cookie_line,
-    net::CompletionCallback* callback) {
-  SetCookieCompletion* set_cookies_callback =
-      static_cast<SetCookieCompletion*>(callback);
-
+void AutomationResourceMessageFilter::SetCookiesForUrl(
+    int render_process_id,
+    int render_view_id,
+    const GURL& url,
+    const std::string& cookie_line) {
   RenderViewMap::iterator automation_details_iter(
-        filtered_render_views_.Get().find(RendererId(
-            set_cookies_callback->render_process_id(),
-            set_cookies_callback->render_view_id())));
-
-  if (automation_details_iter == filtered_render_views_.Get().end()) {
-    return false;
-  }
-
+      filtered_render_views_.Get().find(RendererId(
+          render_process_id, render_view_id)));
+  DCHECK(automation_details_iter != filtered_render_views_.Get().end());
   DCHECK(automation_details_iter->second.filter != NULL);
 
   if (automation_details_iter->second.filter) {
@@ -532,8 +475,6 @@ bool AutomationResourceMessageFilter::SetCookiesForUrl(
         new AutomationMsg_SetCookieAsync(
             automation_details_iter->second.tab_handle, url, cookie_line));
   }
-
-  return true;
 }
 
 // static
@@ -545,15 +486,14 @@ void AutomationResourceMessageFilter::ResumeJobsForPendingView(
   DCHECK(new_filter != NULL);
 
   RequestMap pending_requests = old_filter->pending_request_map_;
+  old_filter->pending_request_map_.clear();
 
-  for (RequestMap::iterator index = old_filter->pending_request_map_.begin();
-          index != old_filter->pending_request_map_.end(); index++) {
-    scoped_refptr<URLRequestAutomationJob> job = (*index).second;
+  for (RequestMap::iterator index = pending_requests.begin();
+       index != pending_requests.end(); index++) {
+    URLRequestAutomationJob* job = (*index).second;
     DCHECK_EQ(job->message_filter(), old_filter);
     DCHECK(job->is_pending());
     // StartPendingJob will register the job with the new filter.
     job->StartPendingJob(tab_handle, new_filter);
   }
-
-  old_filter->pending_request_map_.clear();
 }
