@@ -1,8 +1,9 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/automation/chrome_frame_automation_provider.h"
+#include "chrome/browser/browser_process.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/common/automation_messages.h"
@@ -10,7 +11,17 @@
 #include "ipc/ipc_channel.h"
 
 ChromeFrameAutomationProvider::ChromeFrameAutomationProvider(Profile* profile)
-    : AutomationProvider(profile) {}
+    : AutomationProvider(profile) {
+  DCHECK(g_browser_process);
+  if (g_browser_process)
+    g_browser_process->AddRefModule();
+}
+
+ChromeFrameAutomationProvider::~ChromeFrameAutomationProvider() {
+  DCHECK(g_browser_process);
+  if (g_browser_process)
+    g_browser_process->ReleaseModule();
+}
 
 bool ChromeFrameAutomationProvider::OnMessageReceived(
     const IPC::Message& message) {
@@ -37,16 +48,14 @@ bool ChromeFrameAutomationProvider::IsValidMessage(uint32 type) {
 #if defined(OS_WIN)
     case AutomationMsg_BrowserMove::ID:
     case AutomationMsg_ProcessUnhandledAccelerator::ID:
-    case AutomationMsg_TabReposition::ID:
     case AutomationMsg_ForwardContextMenuCommandToChrome::ID:
 #endif  // defined(OS_WIN)
+#if defined(OS_WIN)
+    case AutomationMsg_TabReposition::ID:
+#endif
     case AutomationMsg_NavigateInExternalTab::ID:
     case AutomationMsg_NavigateExternalTabAtIndex::ID:
     case AutomationMsg_Find::ID:
-    case AutomationMsg_InstallExtension::ID:
-    case AutomationMsg_LoadExpandedExtension::ID:
-    case AutomationMsg_GetEnabledExtensions::ID:
-    case AutomationMsg_SetEnableExtensionAutomation::ID:
     case AutomationMsg_SetInitialFocus::ID:
     case AutomationMsg_SetPageFontSize::ID:
     case AutomationMsg_SetProxyConfig::ID:
@@ -77,4 +86,3 @@ bool ChromeFrameAutomationProvider::IsValidMessage(uint32 type) {
 
   return is_valid_message;
 }
-
