@@ -4,25 +4,28 @@
 
 #include "chrome/browser/automation/url_request_automation_job.h"
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/message_loop.h"
 #include "base/time.h"
 #include "chrome/browser/automation/automation_resource_message_filter.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/renderer_host/resource_dispatcher_host.h"
-#include "chrome/browser/renderer_host/resource_dispatcher_host_request_info.h"
 #include "chrome/common/automation_messages.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/renderer_host/resource_dispatcher_host.h"
+#include "content/browser/renderer_host/resource_dispatcher_host_request_info.h"
+#include "content/public/browser/browser_thread.h"
 #include "net/base/cookie_monster.h"
+#include "net/base/host_port_pair.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
-#include "net/http/http_response_headers.h"
 #include "net/http/http_request_headers.h"
+#include "net/http/http_response_headers.h"
 #include "net/http/http_util.h"
 #include "net/url_request/url_request_context.h"
 
 using base::Time;
 using base::TimeDelta;
+using content::BrowserThread;
 
 // The list of filtered headers that are removed from requests sent via
 // StartAsync(). These must be lower case.
@@ -32,6 +35,7 @@ static const char* const kFilteredHeaderStrings[] = {
   "expect",
   "max-forwards",
   "proxy-authorization",
+  "referer",
   "te",
   "upgrade",
   "via"
@@ -59,7 +63,7 @@ URLRequestAutomationJob::URLRequestAutomationJob(
       redirect_status_(0),
       request_id_(request_id),
       is_pending_(is_pending),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {
   DVLOG(1) << "URLRequestAutomationJob create. Count: " << ++instance_count_;
   DCHECK(message_filter_ != NULL);
 
@@ -74,20 +78,18 @@ URLRequestAutomationJob::~URLRequestAutomationJob() {
   Cleanup();
 }
 
-bool URLRequestAutomationJob::EnsureProtocolFactoryRegistered() {
+void URLRequestAutomationJob::EnsureProtocolFactoryRegistered() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
 
   if (!is_protocol_factory_registered_) {
     old_http_factory_ =
-        net::URLRequest::RegisterProtocolFactory(
+        net::URLRequest::Deprecated::RegisterProtocolFactory(
             "http", &URLRequestAutomationJob::Factory);
     old_https_factory_ =
-        net::URLRequest::RegisterProtocolFactory(
+        net::URLRequest::Deprecated::RegisterProtocolFactory(
             "https", &URLRequestAutomationJob::Factory);
     is_protocol_factory_registered_ = true;
   }
-
-  return true;
 }
 
 net::URLRequestJob* URLRequestAutomationJob::Factory(
@@ -98,18 +100,11 @@ net::URLRequestJob* URLRequestAutomationJob::Factory(
 
   // Returning null here just means that the built-in handler will be used.
   if (scheme_is_http || scheme_is_https) {
-    ResourceDispatcherHostRequestInfo* request_info = NULL;
-    if (request->GetUserData(NULL))
-      request_info = ResourceDispatcherHost::InfoForRequest(request);
+    ResourceDispatcherHostRequestInfo* request_info =
+        ResourceDispatcherHost::InfoForRequest(request);
     if (request_info) {
       int child_id = request_info->child_id();
       int route_id = request_info->route_id();
-
-      if (request_info->process_type() == ChildProcessInfo::PLUGIN_PROCESS) {
-        child_id = request_info->host_renderer_id();
-        route_id = request_info->host_render_view_id();
-      }
-
       AutomationResourceMessageFilter::AutomationDetails details;
       if (AutomationResourceMessageFilter::LookupRegisteredRenderView(
               child_id, route_id, &details)) {
@@ -135,8 +130,8 @@ void URLRequestAutomationJob::Start() {
     // callbacks happen as they would for network requests.
     MessageLoop::current()->PostTask(
         FROM_HERE,
-        method_factory_.NewRunnableMethod(
-            &URLRequestAutomationJob::StartAsync));
+        base::Bind(&URLRequestAutomationJob::StartAsync,
+                   weak_factory_.GetWeakPtr()));
   } else {
     // If this is a pending job, then register it immediately with the message
     // filter so it can be serviced later when we receive a request from the
@@ -174,8 +169,8 @@ bool URLRequestAutomationJob::ReadRawData(
   } else {
     MessageLoop::current()->PostTask(
         FROM_HERE,
-        method_factory_.NewRunnableMethod(
-            &URLRequestAutomationJob::NotifyJobCompletionTask));
+        base::Bind(&URLRequestAutomationJob::NotifyJobCompletionTask,
+                   weak_factory_.GetWeakPtr()));
   }
   return false;
 }
@@ -234,6 +229,24 @@ bool URLRequestAutomationJob::IsRedirectResponse(
   return true;
 }
 
+uint64 URLRequestAutomationJob::GetUploadProgress() const {
+  if (request_ && request_->status().is_success()) {
+    // We don't support incremental progress notifications in ChromeFrame. When
+    // we receive a response for the POST request from Chromeframe, it means
+    // that the upload is fully complete.
+    ResourceDispatcherHostRequestInfo* request_info =
+        ResourceDispatcherHost::InfoForRequest(request_);
+    if (request_info) {
+      return request_info->upload_size();
+    }
+  }
+  return 0;
+}
+
+net::HostPortPair URLRequestAutomationJob::GetSocketAddress() const {
+  return socket_address_;
+}
+
 bool URLRequestAutomationJob::MayFilterMessage(const IPC::Message& message,
                                                int* request_id) {
   switch (message.type()) {
@@ -258,11 +271,18 @@ void URLRequestAutomationJob::OnMessage(const IPC::Message& message) {
     return;
   }
 
-  IPC_BEGIN_MESSAGE_MAP(URLRequestAutomationJob, message)
+  bool deserialize_success = false;
+  IPC_BEGIN_MESSAGE_MAP_EX(URLRequestAutomationJob,
+                           message,
+                           deserialize_success)
     IPC_MESSAGE_HANDLER(AutomationMsg_RequestStarted, OnRequestStarted)
     IPC_MESSAGE_HANDLER(AutomationMsg_RequestData, OnDataAvailable)
     IPC_MESSAGE_HANDLER(AutomationMsg_RequestEnd, OnRequestEnd)
-  IPC_END_MESSAGE_MAP()
+  IPC_END_MESSAGE_MAP_EX()
+
+  if (!deserialize_success) {
+    LOG(ERROR) << "Failed to deserialize IPC message.";
+  }
 }
 
 void URLRequestAutomationJob::OnRequestStarted(
@@ -282,6 +302,7 @@ void URLRequestAutomationJob::OnRequestStarted(
         net::HttpUtil::AssembleRawHeaders(response.headers.data(),
                                           response.headers.size()));
   }
+  socket_address_ = response.socket_address;
   NotifyHeadersComplete();
 }
 
@@ -324,8 +345,8 @@ void URLRequestAutomationJob::OnRequestEnd(
   // so we don't.  We could possibly call OnSSLCertificateError with a NULL
   // certificate, but I'm not sure if all implementations expect it.
   // if (status.status() == net::URLRequestStatus::FAILED &&
-  //    net::IsCertificateError(status.os_error()) && request_->delegate()) {
-  //  request_->delegate()->OnSSLCertificateError(request_, status.os_error());
+  //    net::IsCertificateError(status.error()) && request_->delegate()) {
+  //  request_->delegate()->OnSSLCertificateError(request_, status.error());
   // }
 
   DisconnectFromMessageFilter();
@@ -337,22 +358,22 @@ void URLRequestAutomationJob::OnRequestEnd(
     // 1. We failed to connect to the server, in which case we did not receive
     //    a valid response.
     // 2. In response to a read request.
-    if (!has_response_started() || pending_buf_) {
+    if (!has_response_started()) {
+      NotifyStartError(status);
+    } else if (pending_buf_) {
+      pending_buf_ = NULL;
+      pending_buf_size_ = 0;
       NotifyDone(status);
+      NotifyReadComplete(0);
     } else {
       // Wait for the http stack to issue a Read request where we will notify
       // that the job has completed.
       request_status_ = status;
-      return;
     }
   }
-
-  // Reset any pending reads.
-  if (pending_buf_) {
-    pending_buf_ = NULL;
-    pending_buf_size_ = 0;
-    NotifyReadComplete(0);
-  }
+  // Note
+  // The job could have been destroyed above. Please don't attempt to access
+  // member variables here.
 }
 
 void URLRequestAutomationJob::Cleanup() {
@@ -362,7 +383,7 @@ void URLRequestAutomationJob::Cleanup() {
   id_ = 0;
   tab_ = 0;
 
-  DCHECK(message_filter_ == NULL);
+  DCHECK(!message_filter_);
   DisconnectFromMessageFilter();
 
   pending_buf_ = NULL;
@@ -433,14 +454,14 @@ void URLRequestAutomationJob::StartAsync() {
   }
 
   // Ask automation to start this request.
-  AutomationURLRequest automation_request(
-      request_->url().spec(),
-      request_->method(),
-      referrer.spec(),
-      new_request_headers.ToString(),
-      request_->get_upload(),
-      resource_type,
-      request_->load_flags());
+  AutomationURLRequest automation_request;
+  automation_request.url = request_->url().spec();
+  automation_request.method = request_->method();
+  automation_request.referrer = referrer.spec();
+  automation_request.extra_request_headers = new_request_headers.ToString();
+  automation_request.upload_data =request_->get_upload();
+  automation_request.resource_type = resource_type;
+  automation_request.load_flags = request_->load_flags();
 
   DCHECK(message_filter_);
   message_filter_->Send(new AutomationMsg_RequestStart(
