@@ -7,7 +7,7 @@
 #define CHROME_BROWSER_AUTOMATION_URL_REQUEST_AUTOMATION_JOB_H_
 #pragma once
 
-#include "base/task.h"
+#include "base/memory/weak_ptr.h"
 #include "chrome/common/ref_counted_util.h"
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_job.h"
@@ -18,6 +18,7 @@ struct AutomationURLResponse;
 namespace net {
 class HttpResponseHeaders;
 class HttpResponseInfo;
+class HostPortPair;
 }
 
 namespace IPC {
@@ -33,7 +34,7 @@ class URLRequestAutomationJob : public net::URLRequestJob {
                           bool is_pending);
 
   // Register our factory for HTTP/HTTPs requests.
-  static bool EnsureProtocolFactoryRegistered();
+  static void EnsureProtocolFactoryRegistered();
 
   static net::URLRequest::ProtocolFactory Factory;
 
@@ -45,6 +46,8 @@ class URLRequestAutomationJob : public net::URLRequestJob {
   virtual void GetResponseInfo(net::HttpResponseInfo* info);
   virtual int GetResponseCode() const;
   virtual bool IsRedirectResponse(GURL* location, int* http_status_code);
+  virtual uint64 GetUploadProgress() const;
+  virtual net::HostPortPair GetSocketAddress() const;
 
   // Peek and process automation messages for URL requests.
   static bool MayFilterMessage(const IPC::Message& message, int* request_id);
@@ -122,7 +125,10 @@ class URLRequestAutomationJob : public net::URLRequestJob {
   // stack when we receive a Read request for a completed job.
   net::URLRequestStatus request_status_;
 
-  ScopedRunnableMethodFactory<URLRequestAutomationJob> method_factory_;
+  // Contains the ip address and port of the destination host.
+  net::HostPortPair socket_address_;
+
+  base::WeakPtrFactory<URLRequestAutomationJob> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(URLRequestAutomationJob);
 };
