@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,6 +19,8 @@ using base::Time;
 const char* BookmarkCodec::kRootsKey = "roots";
 const char* BookmarkCodec::kRootFolderNameKey = "bookmark_bar";
 const char* BookmarkCodec::kOtherBookmarkFolderNameKey = "other";
+// The value is left as 'synced' for historical reasons.
+const char* BookmarkCodec::kMobileBookmarkFolderNameKey = "synced";
 const char* BookmarkCodec::kVersionKey = "version";
 const char* BookmarkCodec::kChecksumKey = "checksum";
 const char* BookmarkCodec::kIdKey = "id";
@@ -43,16 +45,20 @@ BookmarkCodec::BookmarkCodec()
 BookmarkCodec::~BookmarkCodec() {}
 
 Value* BookmarkCodec::Encode(BookmarkModel* model) {
-  return Encode(model->GetBookmarkBarNode(), model->other_node());
+  return Encode(model->bookmark_bar_node(),
+                model->other_node(),
+                model->mobile_node());
 }
 
 Value* BookmarkCodec::Encode(const BookmarkNode* bookmark_bar_node,
-                             const BookmarkNode* other_folder_node) {
+                             const BookmarkNode* other_folder_node,
+                             const BookmarkNode* mobile_folder_node) {
   ids_reassigned_ = false;
   InitializeChecksum();
   DictionaryValue* roots = new DictionaryValue();
   roots->Set(kRootFolderNameKey, EncodeNode(bookmark_bar_node));
   roots->Set(kOtherBookmarkFolderNameKey, EncodeNode(other_folder_node));
+  roots->Set(kMobileBookmarkFolderNameKey, EncodeNode(mobile_folder_node));
 
   DictionaryValue* main = new DictionaryValue();
   main->SetInteger(kVersionKey, kCurrentVersion);
@@ -67,6 +73,7 @@ Value* BookmarkCodec::Encode(const BookmarkNode* bookmark_bar_node,
 
 bool BookmarkCodec::Decode(BookmarkNode* bb_node,
                            BookmarkNode* other_folder_node,
+                           BookmarkNode* mobile_folder_node,
                            int64* max_id,
                            const Value& value) {
   ids_.clear();
@@ -75,12 +82,13 @@ bool BookmarkCodec::Decode(BookmarkNode* bb_node,
   maximum_id_ = 0;
   stored_checksum_.clear();
   InitializeChecksum();
-  bool success = DecodeHelper(bb_node, other_folder_node, value);
+  bool success = DecodeHelper(bb_node, other_folder_node, mobile_folder_node,
+                              value);
   FinalizeChecksum();
   // If either the checksums differ or some IDs were missing/not unique,
   // reassign IDs.
   if (!ids_valid_ || computed_checksum() != stored_checksum())
-    ReassignIDs(bb_node, other_folder_node);
+    ReassignIDs(bb_node, other_folder_node, mobile_folder_node);
   *max_id = maximum_id_ + 1;
   return success;
 }
@@ -93,21 +101,21 @@ Value* BookmarkCodec::EncodeNode(const BookmarkNode* node) {
   value->SetString(kNameKey, title);
   value->SetString(kDateAddedKey,
                    base::Int64ToString(node->date_added().ToInternalValue()));
-  if (node->type() == BookmarkNode::URL) {
+  if (node->is_url()) {
     value->SetString(kTypeKey, kTypeURL);
-    std::string url = node->GetURL().possibly_invalid_spec();
+    std::string url = node->url().possibly_invalid_spec();
     value->SetString(kURLKey, url);
     UpdateChecksumWithUrlNode(id, title, url);
   } else {
     value->SetString(kTypeKey, kTypeFolder);
     value->SetString(kDateModifiedKey,
-                     base::Int64ToString(node->date_group_modified().
+                     base::Int64ToString(node->date_folder_modified().
                                    ToInternalValue()));
     UpdateChecksumWithFolderNode(id, title);
 
     ListValue* child_values = new ListValue();
     value->Set(kChildrenKey, child_values);
-    for (int i = 0; i < node->GetChildCount(); ++i)
+    for (int i = 0; i < node->child_count(); ++i)
       child_values->Append(EncodeNode(node->GetChild(i)));
   }
   return value;
@@ -115,6 +123,7 @@ Value* BookmarkCodec::EncodeNode(const BookmarkNode* node) {
 
 bool BookmarkCodec::DecodeHelper(BookmarkNode* bb_node,
                                  BookmarkNode* other_folder_node,
+                                 BookmarkNode* mobile_folder_node,
                                  const Value& value) {
   if (value.GetType() != Value::TYPE_DICTIONARY)
     return false;  // Unexpected type.
@@ -147,21 +156,44 @@ bool BookmarkCodec::DecodeHelper(BookmarkNode* bb_node,
   if (!roots_d_value->Get(kRootFolderNameKey, &root_folder_value) ||
       root_folder_value->GetType() != Value::TYPE_DICTIONARY ||
       !roots_d_value->Get(kOtherBookmarkFolderNameKey, &other_folder_value) ||
-      other_folder_value->GetType() != Value::TYPE_DICTIONARY)
-    return false;  // Invalid type for root folder and/or other folder.
-
+      other_folder_value->GetType() != Value::TYPE_DICTIONARY) {
+    return false;  // Invalid type for root folder and/or other
+                   // folder.
+  }
   DecodeNode(*static_cast<DictionaryValue*>(root_folder_value), NULL,
              bb_node);
   DecodeNode(*static_cast<DictionaryValue*>(other_folder_value), NULL,
              other_folder_node);
+
+  // Fail silently if we can't deserialize mobile bookmarks. We can't require
+  // them to exist in order to be backwards-compatible with older versions of
+  // chrome.
+  Value* mobile_folder_value;
+  if (roots_d_value->Get(kMobileBookmarkFolderNameKey, &mobile_folder_value) &&
+      mobile_folder_value->GetType() == Value::TYPE_DICTIONARY) {
+    DecodeNode(*static_cast<DictionaryValue*>(mobile_folder_value), NULL,
+               mobile_folder_node);
+  } else {
+    // If we didn't find the mobile folder, we're almost guaranteed to have a
+    // duplicate id when we add the mobile folder. Consequently, if we don't
+    // intend to reassign ids in the future (ids_valid_ is still true), then at
+    // least reassign the mobile bookmarks to avoid it colliding with anything
+    // else.
+    if (ids_valid_)
+      ReassignIDsHelper(mobile_folder_node);
+  }
+
   // Need to reset the type as decoding resets the type to FOLDER. Similarly
   // we need to reset the title as the title is persisted and restored from
   // the file.
   bb_node->set_type(BookmarkNode::BOOKMARK_BAR);
   other_folder_node->set_type(BookmarkNode::OTHER_NODE);
-  bb_node->SetTitle(l10n_util::GetStringUTF16(IDS_BOOMARK_BAR_FOLDER_NAME));
+  mobile_folder_node->set_type(BookmarkNode::MOBILE);
+  bb_node->SetTitle(l10n_util::GetStringUTF16(IDS_BOOKMARK_BAR_FOLDER_NAME));
   other_folder_node->SetTitle(
-      l10n_util::GetStringUTF16(IDS_BOOMARK_BAR_OTHER_FOLDER_NAME));
+      l10n_util::GetStringUTF16(IDS_BOOKMARK_BAR_OTHER_FOLDER_NAME));
+  mobile_folder_node->SetTitle(
+        l10n_util::GetStringUTF16(IDS_BOOKMARK_BAR_MOBILE_FOLDER_NAME));
 
   return true;
 }
@@ -247,7 +279,7 @@ bool BookmarkCodec::DecodeNode(const DictionaryValue& value,
       return false;  // Node invalid.
 
     if (parent)
-      parent->Add(parent->GetChildCount(), node);
+      parent->Add(node, parent->child_count());
     node->set_type(BookmarkNode::URL);
     UpdateChecksumWithUrlNode(id_string, title, url_string);
   } else {
@@ -272,10 +304,10 @@ bool BookmarkCodec::DecodeNode(const DictionaryValue& value,
     node->set_type(BookmarkNode::FOLDER);
     int64 internal_time;
     base::StringToInt64(last_modified_date, &internal_time);
-    node->set_date_group_modified(Time::FromInternalValue(internal_time));
+    node->set_date_folder_modified(Time::FromInternalValue(internal_time));
 
     if (parent)
-      parent->Add(parent->GetChildCount(), node);
+      parent->Add(node, parent->child_count());
 
     UpdateChecksumWithFolderNode(id_string, title);
 
@@ -289,26 +321,31 @@ bool BookmarkCodec::DecodeNode(const DictionaryValue& value,
 }
 
 void BookmarkCodec::ReassignIDs(BookmarkNode* bb_node,
-                                BookmarkNode* other_node) {
+                                BookmarkNode* other_node,
+                                BookmarkNode* mobile_node) {
   maximum_id_ = 0;
   ReassignIDsHelper(bb_node);
   ReassignIDsHelper(other_node);
+  ReassignIDsHelper(mobile_node);
   ids_reassigned_ = true;
 }
 
 void BookmarkCodec::ReassignIDsHelper(BookmarkNode* node) {
   DCHECK(node);
   node->set_id(++maximum_id_);
-  for (int i = 0; i < node->GetChildCount(); ++i)
+  for (int i = 0; i < node->child_count(); ++i)
     ReassignIDsHelper(node->GetChild(i));
 }
 
 void BookmarkCodec::UpdateChecksum(const std::string& str) {
-  MD5Update(&md5_context_, str.data(), str.length() * sizeof(char));
+  base::MD5Update(&md5_context_, str);
 }
 
 void BookmarkCodec::UpdateChecksum(const string16& str) {
-  MD5Update(&md5_context_, str.data(), str.length() * sizeof(char16));
+  base::MD5Update(&md5_context_,
+                  base::StringPiece(
+                      reinterpret_cast<const char*>(str.data()),
+                      str.length() * sizeof(str[0])));
 }
 
 void BookmarkCodec::UpdateChecksumWithUrlNode(const std::string& id,
@@ -329,11 +366,11 @@ void BookmarkCodec::UpdateChecksumWithFolderNode(const std::string& id,
 }
 
 void BookmarkCodec::InitializeChecksum() {
-  MD5Init(&md5_context_);
+  base::MD5Init(&md5_context_);
 }
 
 void BookmarkCodec::FinalizeChecksum() {
-  MD5Digest digest;
-  MD5Final(&digest, &md5_context_);
-  computed_checksum_ = MD5DigestToBase16(digest);
+  base::MD5Digest digest;
+  base::MD5Final(&digest, &md5_context_);
+  computed_checksum_ = base::MD5DigestToBase16(digest);
 }
