@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,19 +7,22 @@
 #include "base/compiler_specific.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/bookmarks/bookmark_editor.h"
-#include "chrome/browser/bookmarks/bookmark_folder_editor_controller.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/bookmarks/bookmark_utils.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/metrics/user_metrics.h"
+#include "chrome/browser/prefs/incognito_mode_prefs.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/page_navigator.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/browser/page_navigator.h"
+#include "content/public/browser/user_metrics.h"
 #include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util.h"
 
+using content::PageNavigator;
+using content::UserMetricsAction;
+
 BookmarkContextMenuController::BookmarkContextMenuController(
     gfx::NativeWindow parent_window,
     BookmarkContextMenuControllerDelegate* delegate,
@@ -50,17 +53,17 @@ BookmarkContextMenuController::~BookmarkContextMenuController() {
 void BookmarkContextMenuController::BuildMenu() {
   if (selection_.size() == 1 && selection_[0]->is_url()) {
     AddItem(IDC_BOOKMARK_BAR_OPEN_ALL,
-            IDS_BOOMARK_BAR_OPEN_IN_NEW_TAB);
+            IDS_BOOKMARK_BAR_OPEN_IN_NEW_TAB);
     AddItem(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW,
-            IDS_BOOMARK_BAR_OPEN_IN_NEW_WINDOW);
+            IDS_BOOKMARK_BAR_OPEN_IN_NEW_WINDOW);
     AddItem(IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO,
-            IDS_BOOMARK_BAR_OPEN_INCOGNITO);
+            IDS_BOOKMARK_BAR_OPEN_INCOGNITO);
   } else {
-    AddItem(IDC_BOOKMARK_BAR_OPEN_ALL, IDS_BOOMARK_BAR_OPEN_ALL);
+    AddItem(IDC_BOOKMARK_BAR_OPEN_ALL, IDS_BOOKMARK_BAR_OPEN_ALL);
     AddItem(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW,
-        IDS_BOOMARK_BAR_OPEN_ALL_NEW_WINDOW);
+        IDS_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW);
     AddItem(IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO,
-        IDS_BOOMARK_BAR_OPEN_ALL_INCOGNITO);
+        IDS_BOOKMARK_BAR_OPEN_ALL_INCOGNITO);
   }
 
   AddSeparator();
@@ -79,12 +82,12 @@ void BookmarkContextMenuController::BuildMenu() {
   AddItem(IDC_BOOKMARK_BAR_REMOVE, IDS_BOOKMARK_BAR_REMOVE);
 
   AddSeparator();
-  AddItem(IDC_BOOKMARK_BAR_ADD_NEW_BOOKMARK, IDS_BOOMARK_BAR_ADD_NEW_BOOKMARK);
-  AddItem(IDC_BOOKMARK_BAR_NEW_FOLDER, IDS_BOOMARK_BAR_NEW_FOLDER);
+  AddItem(IDC_BOOKMARK_BAR_ADD_NEW_BOOKMARK, IDS_BOOKMARK_BAR_ADD_NEW_BOOKMARK);
+  AddItem(IDC_BOOKMARK_BAR_NEW_FOLDER, IDS_BOOKMARK_BAR_NEW_FOLDER);
 
   AddSeparator();
   AddItem(IDC_BOOKMARK_MANAGER, IDS_BOOKMARK_MANAGER);
-  AddCheckboxItem(IDC_BOOKMARK_BAR_ALWAYS_SHOW, IDS_BOOMARK_BAR_ALWAYS_SHOW);
+  AddCheckboxItem(IDC_BOOKMARK_BAR_ALWAYS_SHOW, IDS_SHOW_BOOKMARK_BAR);
 }
 
 void BookmarkContextMenuController::AddItem(int id, int localization_id) {
@@ -111,19 +114,16 @@ void BookmarkContextMenuController::ExecuteCommand(int id) {
       WindowOpenDisposition initial_disposition;
       if (id == IDC_BOOKMARK_BAR_OPEN_ALL) {
         initial_disposition = NEW_FOREGROUND_TAB;
-        UserMetrics::RecordAction(
-            UserMetricsAction("BookmarkBar_ContextMenu_OpenAll"),
-            profile_);
+        content::RecordAction(
+            UserMetricsAction("BookmarkBar_ContextMenu_OpenAll"));
       } else if (id == IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW) {
         initial_disposition = NEW_WINDOW;
-        UserMetrics::RecordAction(
-                UserMetricsAction("BookmarkBar_ContextMenu_OpenAllInNewWindow"),
-                profile_);
+        content::RecordAction(
+            UserMetricsAction("BookmarkBar_ContextMenu_OpenAllInNewWindow"));
       } else {
         initial_disposition = OFF_THE_RECORD;
-        UserMetrics::RecordAction(
-            UserMetricsAction("BookmarkBar_ContextMenu_OpenAllIncognito"),
-            profile_);
+        content::RecordAction(
+            UserMetricsAction("BookmarkBar_ContextMenu_OpenAllIncognito"));
       }
       bookmark_utils::OpenAll(parent_window_, profile_, navigator_, selection_,
                               initial_disposition);
@@ -132,61 +132,61 @@ void BookmarkContextMenuController::ExecuteCommand(int id) {
 
     case IDC_BOOKMARK_BAR_RENAME_FOLDER:
     case IDC_BOOKMARK_BAR_EDIT:
-      UserMetrics::RecordAction(
-          UserMetricsAction("BookmarkBar_ContextMenu_Edit"),
-          profile_);
+      content::RecordAction(
+          UserMetricsAction("BookmarkBar_ContextMenu_Edit"));
 
       if (selection_.size() != 1) {
         NOTREACHED();
         break;
       }
 
-      if (selection_[0]->is_url()) {
-        BookmarkEditor::Show(parent_window_, profile_, parent_,
-                             BookmarkEditor::EditDetails(selection_[0]),
-                             BookmarkEditor::SHOW_TREE);
-      } else {
-        BookmarkFolderEditorController::Show(profile_, parent_window_,
-            selection_[0], -1,
-            BookmarkFolderEditorController::EXISTING_BOOKMARK);
-      }
+      BookmarkEditor::Show(
+          parent_window_,
+          profile_,
+          BookmarkEditor::EditDetails::EditNode(selection_[0]),
+          BookmarkEditor::SHOW_TREE);
       break;
 
     case IDC_BOOKMARK_BAR_REMOVE: {
-      UserMetrics::RecordAction(
-          UserMetricsAction("BookmarkBar_ContextMenu_Remove"),
-          profile_);
+      content::RecordAction(
+          UserMetricsAction("BookmarkBar_ContextMenu_Remove"));
 
       for (size_t i = 0; i < selection_.size(); ++i) {
-        model_->Remove(selection_[i]->GetParent(),
-                      selection_[i]->GetParent()->IndexOfChild(selection_[i]));
+        int index = selection_[i]->parent()->GetIndexOf(selection_[i]);
+        if (index > -1)
+          model_->Remove(selection_[i]->parent(), index);
       }
       selection_.clear();
       break;
     }
 
     case IDC_BOOKMARK_BAR_ADD_NEW_BOOKMARK: {
-      UserMetrics::RecordAction(
-          UserMetricsAction("BookmarkBar_ContextMenu_Add"),
-          profile_);
+      content::RecordAction(
+          UserMetricsAction("BookmarkBar_ContextMenu_Add"));
 
-      // TODO: this should honor the index from GetParentForNewNodes.
+      int index;
+      const BookmarkNode* parent =
+          bookmark_utils::GetParentForNewNodes(parent_, selection_, &index);
       BookmarkEditor::Show(
-          parent_window_, profile_,
-          bookmark_utils::GetParentForNewNodes(parent_, selection_, NULL),
-          BookmarkEditor::EditDetails(), BookmarkEditor::SHOW_TREE);
+          parent_window_,
+          profile_,
+          BookmarkEditor::EditDetails::AddNodeInFolder(parent, index),
+          BookmarkEditor::SHOW_TREE);
       break;
     }
 
     case IDC_BOOKMARK_BAR_NEW_FOLDER: {
-      UserMetrics::RecordAction(
-          UserMetricsAction("BookmarkBar_ContextMenu_NewFolder"),
-          profile_);
+      content::RecordAction(
+          UserMetricsAction("BookmarkBar_ContextMenu_NewFolder"));
+
       int index;
       const BookmarkNode* parent =
           bookmark_utils::GetParentForNewNodes(parent_, selection_, &index);
-      BookmarkFolderEditorController::Show(profile_, parent_window_, parent,
-          index, BookmarkFolderEditorController::NEW_BOOKMARK);
+      BookmarkEditor::Show(
+          parent_window_,
+          profile_,
+          BookmarkEditor::EditDetails::AddFolder(parent, index),
+          BookmarkEditor::SHOW_TREE);
       break;
     }
 
@@ -194,17 +194,22 @@ void BookmarkContextMenuController::ExecuteCommand(int id) {
       bookmark_utils::ToggleWhenVisible(profile_);
       break;
 
-    case IDC_BOOKMARK_MANAGER:
-      UserMetrics::RecordAction(UserMetricsAction("ShowBookmarkManager"),
-                                profile_);
-      {
-        Browser* browser = BrowserList::GetLastActiveWithProfile(profile_);
-        if (browser)
-          browser->OpenBookmarkManager();
-        else
-          NOTREACHED();
-      }
+    case IDC_BOOKMARK_MANAGER: {
+      content::RecordAction(UserMetricsAction("ShowBookmarkManager"));
+      Browser* browser = BrowserList::GetLastActiveWithProfile(profile_);
+      if (!browser) NOTREACHED();
+
+      if (selection_.size() != 1)
+        browser->OpenBookmarkManager();
+      else if (selection_[0]->is_folder())
+        browser->OpenBookmarkManagerForNode(selection_[0]->id());
+      else if (parent_)
+        browser->OpenBookmarkManagerForNode(parent_->id());
+      else
+        browser->OpenBookmarkManager();
       break;
+    }
+
 
     case IDC_CUT:
       bookmark_utils::CopyToClipboard(model_, selection_, true);
@@ -241,39 +246,54 @@ bool BookmarkContextMenuController::IsCommandIdChecked(int command_id) const {
 bool BookmarkContextMenuController::IsCommandIdEnabled(int command_id) const {
   bool is_root_node =
       (selection_.size() == 1 &&
-       selection_[0]->GetParent() == model_->root_node());
+       selection_[0]->parent() == model_->root_node());
+  bool can_edit =
+      profile_->GetPrefs()->GetBoolean(prefs::kEditBookmarksEnabled);
+  IncognitoModePrefs::Availability incognito_avail =
+      IncognitoModePrefs::GetAvailability(profile_->GetPrefs());
   switch (command_id) {
     case IDC_BOOKMARK_BAR_OPEN_INCOGNITO:
-      return !profile_->IsOffTheRecord();
+      return !profile_->IsOffTheRecord() &&
+             incognito_avail != IncognitoModePrefs::DISABLED;
 
     case IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO:
-      return HasURLs() && !profile_->IsOffTheRecord();
+      return HasURLs() &&
+             !profile_->IsOffTheRecord() &&
+             incognito_avail != IncognitoModePrefs::DISABLED;
 
     case IDC_BOOKMARK_BAR_OPEN_ALL:
-    case IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW:
       return HasURLs();
+    case IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW:
+      return HasURLs() &&
+             incognito_avail != IncognitoModePrefs::FORCED;
 
     case IDC_BOOKMARK_BAR_RENAME_FOLDER:
     case IDC_BOOKMARK_BAR_EDIT:
-      return selection_.size() == 1 && !is_root_node;
+      return selection_.size() == 1 && !is_root_node && can_edit;
 
     case IDC_BOOKMARK_BAR_REMOVE:
-      return !selection_.empty() && !is_root_node;
+      return !selection_.empty() && !is_root_node && can_edit;
 
     case IDC_BOOKMARK_BAR_NEW_FOLDER:
     case IDC_BOOKMARK_BAR_ADD_NEW_BOOKMARK:
-      return bookmark_utils::GetParentForNewNodes(
+      return can_edit && bookmark_utils::GetParentForNewNodes(
           parent_, selection_, NULL) != NULL;
 
+    case IDC_BOOKMARK_BAR_ALWAYS_SHOW:
+      return !profile_->GetPrefs()->IsManagedPreference(
+          prefs::kShowBookmarkBar);
+
     case IDC_COPY:
     case IDC_CUT:
-      return selection_.size() > 0 && !is_root_node;
+      return !selection_.empty() && !is_root_node &&
+             (command_id == IDC_COPY || can_edit);
 
     case IDC_PASTE:
       // Paste to selection from the Bookmark Bar, to parent_ everywhere else
-      return (!selection_.empty() &&
-              bookmark_utils::CanPasteFromClipboard(selection_[0])) ||
-             bookmark_utils::CanPasteFromClipboard(parent_);
+      return can_edit &&
+             ((!selection_.empty() &&
+               bookmark_utils::CanPasteFromClipboard(selection_[0])) ||
+              bookmark_utils::CanPasteFromClipboard(parent_));
   }
   return true;
 }
