@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,12 +10,15 @@
 
 #include "base/basictypes.h"
 #include "chrome/browser/bookmarks/base_bookmark_model_observer.h"
-#include "gfx/native_widget_types.h"
 #include "ui/base/models/simple_menu_model.h"
+#include "ui/gfx/native_widget_types.h"
 
-class PageNavigator;
 class Profile;
 
+namespace content {
+class PageNavigator;
+}
+
 // An interface implemented by an object that performs actions on the actual
 // menu for the controller.
 class BookmarkContextMenuControllerDelegate {
@@ -47,7 +50,7 @@ class BookmarkContextMenuController : public BaseBookmarkModelObserver,
       gfx::NativeWindow parent_window,
       BookmarkContextMenuControllerDelegate* delegate,
       Profile* profile,
-      PageNavigator* navigator,
+      content::PageNavigator* navigator,
       const BookmarkNode* parent,
       const std::vector<const BookmarkNode*>& selection);
   virtual ~BookmarkContextMenuController();
@@ -57,15 +60,16 @@ class BookmarkContextMenuController : public BaseBookmarkModelObserver,
   ui::SimpleMenuModel* menu_model() const { return menu_model_.get(); }
 
   // ui::SimpleMenuModel::Delegate implementation:
-  virtual bool IsCommandIdChecked(int command_id) const;
-  virtual bool IsCommandIdEnabled(int command_id) const;
-  virtual bool GetAcceleratorForCommandId(int command_id,
-                                          ui::Accelerator* accelerator);
-  virtual void ExecuteCommand(int command_id);
+  virtual bool IsCommandIdChecked(int command_id) const OVERRIDE;
+  virtual bool IsCommandIdEnabled(int command_id) const OVERRIDE;
+  virtual bool GetAcceleratorForCommandId(
+      int command_id,
+      ui::Accelerator* accelerator) OVERRIDE;
+  virtual void ExecuteCommand(int command_id) OVERRIDE;
 
   // Accessors:
   Profile* profile() const { return profile_; }
-  PageNavigator* navigator() const { return navigator_; }
+  content::PageNavigator* navigator() const { return navigator_; }
 
  private:
   // Adds a IDC_* style command to the menu with a localized string.
@@ -77,7 +81,7 @@ class BookmarkContextMenuController : public BaseBookmarkModelObserver,
 
   // Overridden from BaseBookmarkModelObserver:
   // Any change to the model results in closing the menu.
-  virtual void BookmarkModelChanged();
+  virtual void BookmarkModelChanged() OVERRIDE;
 
   // Returns true if selection_ has at least one bookmark of type url.
   bool HasURLs() const;
@@ -85,7 +89,7 @@ class BookmarkContextMenuController : public BaseBookmarkModelObserver,
   gfx::NativeWindow parent_window_;
   BookmarkContextMenuControllerDelegate* delegate_;
   Profile* profile_;
-  PageNavigator* navigator_;
+  content::PageNavigator* navigator_;
   const BookmarkNode* parent_;
   std::vector<const BookmarkNode*> selection_;
   BookmarkModel* model_;
