@@ -4,31 +4,34 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/bookmarks/bookmark_context_menu_controller.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/bookmarks/bookmark_utils.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/page_navigator.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/public/browser/page_navigator.h"
+#include "content/test/test_browser_thread.h"
 #include "grit/generated_resources.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 #if defined(OS_WIN)
-#include "chrome/browser/ui/views/bookmark_bar_view.h"
+#include "chrome/browser/ui/views/bookmarks/bookmark_bar_view.h"
 #endif
 
+using content::BrowserThread;
+using content::OpenURLParams;
+using content::PageNavigator;
+using content::WebContents;
+
 // PageNavigator implementation that records the URL.
 class TestingPageNavigator : public PageNavigator {
  public:
-  virtual void OpenURL(const GURL& url,
-                       const GURL& referrer,
-                       WindowOpenDisposition disposition,
-                       PageTransition::Type transition) {
-    urls_.push_back(url);
+  virtual WebContents* OpenURL(const OpenURLParams& params) OVERRIDE {
+    urls_.push_back(params.url);
+    return NULL;
   }
 
   std::vector<GURL> urls_;
@@ -48,7 +51,6 @@ class BookmarkContextMenuControllerTest : public testing::Test {
 #endif
 
     profile_.reset(new TestingProfile());
-    profile_->set_has_history_service(true);
     profile_->CreateBookmarkModel(true);
     profile_->BlockUntilBookmarkModelLoaded();
 
@@ -62,14 +64,14 @@ class BookmarkContextMenuControllerTest : public testing::Test {
     BookmarkBarView::testing_ = false;
 #endif
 
-    // Flush the message loop to make Purify happy.
+    // Flush the message loop to make application verifiers happy.
     message_loop_.RunAllPending();
   }
 
  protected:
   MessageLoopForUI message_loop_;
-  BrowserThread ui_thread_;
-  BrowserThread file_thread_;
+  content::TestBrowserThread ui_thread_;
+  content::TestBrowserThread file_thread_;
   scoped_ptr<TestingProfile> profile_;
   BookmarkModel* model_;
   TestingPageNavigator navigator_;
@@ -86,19 +88,16 @@ class BookmarkContextMenuControllerTest : public testing::Test {
   // F4
   //   f4a
   void AddTestData() {
+    const BookmarkNode* bb_node = model_->bookmark_bar_node();
     std::string test_base = "file:///c:/tmp/";
-
-    model_->AddURL(model_->GetBookmarkBarNode(), 0, ASCIIToUTF16("a"),
-                   GURL(test_base + "a"));
-    const BookmarkNode* f1 =
-        model_->AddGroup(model_->GetBookmarkBarNode(), 1, ASCIIToUTF16("F1"));
+    model_->AddURL(bb_node, 0, ASCIIToUTF16("a"), GURL(test_base + "a"));
+    const BookmarkNode* f1 = model_->AddFolder(bb_node, 1, ASCIIToUTF16("F1"));
     model_->AddURL(f1, 0, ASCIIToUTF16("f1a"), GURL(test_base + "f1a"));
-    const BookmarkNode* f11 = model_->AddGroup(f1, 1, ASCIIToUTF16("F11"));
+    const BookmarkNode* f11 = model_->AddFolder(f1, 1, ASCIIToUTF16("F11"));
     model_->AddURL(f11, 0, ASCIIToUTF16("f11a"), GURL(test_base + "f11a"));
-    model_->AddGroup(model_->GetBookmarkBarNode(), 2, ASCIIToUTF16("F2"));
-    model_->AddGroup(model_->GetBookmarkBarNode(), 3, ASCIIToUTF16("F3"));
-    const BookmarkNode* f4 =
-        model_->AddGroup(model_->GetBookmarkBarNode(), 4, ASCIIToUTF16("F4"));
+    model_->AddFolder(bb_node, 2, ASCIIToUTF16("F2"));
+    model_->AddFolder(bb_node, 3, ASCIIToUTF16("F3"));
+    const BookmarkNode* f4 = model_->AddFolder(bb_node, 4, ASCIIToUTF16("F4"));
     model_->AddURL(f4, 0, ASCIIToUTF16("f4a"), GURL(test_base + "f4a"));
   }
 };
@@ -106,10 +105,10 @@ class BookmarkContextMenuControllerTest : public testing::Test {
 // Tests Deleting from the menu.
 TEST_F(BookmarkContextMenuControllerTest, DeleteURL) {
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(0));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(0));
   BookmarkContextMenuController controller(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes);
-  GURL url = model_->GetBookmarkBarNode()->GetChild(0)->GetURL();
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes);
+  GURL url = model_->bookmark_bar_node()->GetChild(0)->url();
   ASSERT_TRUE(controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_REMOVE));
   // Delete the URL.
   controller.ExecuteCommand(IDC_BOOKMARK_BAR_REMOVE);
@@ -119,15 +118,13 @@ TEST_F(BookmarkContextMenuControllerTest, DeleteURL) {
 
 // Tests open all on a folder with a couple of bookmarks.
 TEST_F(BookmarkContextMenuControllerTest, OpenAll) {
-  const BookmarkNode* folder = model_->GetBookmarkBarNode()->GetChild(1);
+  const BookmarkNode* folder = model_->bookmark_bar_node()->GetChild(1);
   bookmark_utils::OpenAll(
       NULL, profile_.get(), &navigator_, folder, NEW_FOREGROUND_TAB);
 
-  // Should have navigated to F1's children.
-  ASSERT_EQ(static_cast<size_t>(2), navigator_.urls_.size());
-  ASSERT_TRUE(folder->GetChild(0)->GetURL() == navigator_.urls_[0]);
-  ASSERT_TRUE(folder->GetChild(1)->GetChild(0)->GetURL() ==
-              navigator_.urls_[1]);
+  // Should have navigated to F1's child, but not F11's child.
+  ASSERT_EQ(static_cast<size_t>(1), navigator_.urls_.size());
+  ASSERT_TRUE(folder->GetChild(0)->url() == navigator_.urls_[0]);
 }
 
 // Tests the enabled state of the menus when supplied an empty vector.
@@ -151,9 +148,9 @@ TEST_F(BookmarkContextMenuControllerTest, EmptyNodes) {
 // url.
 TEST_F(BookmarkContextMenuControllerTest, SingleURL) {
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(0));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(0));
   BookmarkContextMenuController controller(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes);
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes);
   EXPECT_TRUE(controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL));
   EXPECT_TRUE(
       controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW));
@@ -170,10 +167,10 @@ TEST_F(BookmarkContextMenuControllerTest, SingleURL) {
 // urls.
 TEST_F(BookmarkContextMenuControllerTest, MultipleURLs) {
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(0));
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(1)->GetChild(0));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(0));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(1)->GetChild(0));
   BookmarkContextMenuController controller(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes);
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes);
   EXPECT_TRUE(controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL));
   EXPECT_TRUE(
       controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW));
@@ -190,9 +187,9 @@ TEST_F(BookmarkContextMenuControllerTest, MultipleURLs) {
 // folder.
 TEST_F(BookmarkContextMenuControllerTest, SingleFolder) {
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(2));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(2));
   BookmarkContextMenuController controller(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes);
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes);
   EXPECT_FALSE(controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL));
   EXPECT_FALSE(
       controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW));
@@ -209,10 +206,10 @@ TEST_F(BookmarkContextMenuControllerTest, SingleFolder) {
 // folders, all of which are empty.
 TEST_F(BookmarkContextMenuControllerTest, MultipleEmptyFolders) {
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(2));
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(3));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(2));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(3));
   BookmarkContextMenuController controller(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes);
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes);
   EXPECT_FALSE(controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL));
   EXPECT_FALSE(
       controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW));
@@ -229,10 +226,10 @@ TEST_F(BookmarkContextMenuControllerTest, MultipleEmptyFolders) {
 // folders, some of which contain URLs.
 TEST_F(BookmarkContextMenuControllerTest, MultipleFoldersWithURLs) {
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(3));
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(4));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(3));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(4));
   BookmarkContextMenuController controller(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes);
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes);
   EXPECT_TRUE(controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL));
   EXPECT_TRUE(
       controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW));
@@ -248,10 +245,10 @@ TEST_F(BookmarkContextMenuControllerTest, MultipleFoldersWithURLs) {
 // Tests the enabled state of open incognito.
 TEST_F(BookmarkContextMenuControllerTest, DisableIncognito) {
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(0));
+  nodes.push_back(model_->bookmark_bar_node()->GetChild(0));
   BookmarkContextMenuController controller(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes);
-  profile_->set_off_the_record(true);
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes);
+  profile_->set_incognito(true);
   EXPECT_FALSE(controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_INCOGNITO));
   EXPECT_FALSE(
       controller.IsCommandIdEnabled(IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO));
@@ -286,11 +283,12 @@ TEST_F(BookmarkContextMenuControllerTest, EmptyNodesNullParent) {
 }
 
 TEST_F(BookmarkContextMenuControllerTest, CutCopyPasteNode) {
+  const BookmarkNode* bb_node = model_->bookmark_bar_node();
   std::vector<const BookmarkNode*> nodes;
-  nodes.push_back(model_->GetBookmarkBarNode()->GetChild(0));
+  nodes.push_back(bb_node->GetChild(0));
   scoped_ptr<BookmarkContextMenuController> controller(
       new BookmarkContextMenuController(
-          NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes));
+          NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes));
   EXPECT_TRUE(controller->IsCommandIdEnabled(IDC_COPY));
   EXPECT_TRUE(controller->IsCommandIdEnabled(IDC_CUT));
 
@@ -298,20 +296,19 @@ TEST_F(BookmarkContextMenuControllerTest, CutCopyPasteNode) {
   controller->ExecuteCommand(IDC_COPY);
 
   controller.reset(new BookmarkContextMenuController(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes));
-  int old_count = model_->GetBookmarkBarNode()->GetChildCount();
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes));
+  int old_count = bb_node->child_count();
   controller->ExecuteCommand(IDC_PASTE);
 
-  ASSERT_TRUE(model_->GetBookmarkBarNode()->GetChild(1)->is_url());
-  ASSERT_EQ(old_count + 1, model_->GetBookmarkBarNode()->GetChildCount());
-  ASSERT_EQ(model_->GetBookmarkBarNode()->GetChild(0)->GetURL(),
-            model_->GetBookmarkBarNode()->GetChild(1)->GetURL());
+  ASSERT_TRUE(bb_node->GetChild(1)->is_url());
+  ASSERT_EQ(old_count + 1, bb_node->child_count());
+  ASSERT_EQ(bb_node->GetChild(0)->url(), bb_node->GetChild(1)->url());
 
   controller.reset(new BookmarkContextMenuController(
-      NULL, NULL, profile_.get(), NULL, nodes[0]->GetParent(), nodes));
+      NULL, NULL, profile_.get(), NULL, nodes[0]->parent(), nodes));
   // Cut the URL.
   controller->ExecuteCommand(IDC_CUT);
-  ASSERT_TRUE(model_->GetBookmarkBarNode()->GetChild(0)->is_url());
-  ASSERT_TRUE(model_->GetBookmarkBarNode()->GetChild(1)->is_folder());
-  ASSERT_EQ(old_count, model_->GetBookmarkBarNode()->GetChildCount());
+  ASSERT_TRUE(bb_node->GetChild(0)->is_url());
+  ASSERT_TRUE(bb_node->GetChild(1)->is_folder());
+  ASSERT_EQ(old_count, bb_node->child_count());
 }
