@@ -1,26 +1,29 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "testing/gtest/include/gtest/gtest.h"
 
 #include "base/file_util.h"
+#include "base/i18n/time_formatting.h"
 #include "base/message_loop.h"
 #include "base/path_service.h"
+#include "base/scoped_temp_dir.h"
 #include "base/string16.h"
 #include "base/string_util.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
-#include "base/i18n/time_formatting.h"
 #include "chrome/browser/bookmarks/bookmark_html_writer.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/importer/firefox2_importer.h"
-#include "chrome/test/testing_profile.h"
-#include "gfx/codec/png_codec.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/test/test_browser_thread.h"
 #include "grit/generated_resources.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/gfx/codec/png_codec.h"
+
+using content::BrowserThread;
 
 namespace {
 
@@ -42,25 +45,19 @@ void MakeTestSkBitmap(int w, int h, SkBitmap* bmp) {
 class BookmarkHTMLWriterTest : public testing::Test {
  protected:
   virtual void SetUp() {
-    ASSERT_TRUE(PathService::Get(base::DIR_TEMP, &path_));
-    path_ = path_.AppendASCII("bookmarks.html");
-    file_util::Delete(path_, true);
-  }
-
-  virtual void TearDown() {
-    if (!path_.empty())
-      file_util::Delete(path_, true);
+    testing::Test::SetUp();
+    ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
+    path_ = temp_dir_.path().AppendASCII("bookmarks.html");
   }
 
   // Converts a BookmarkEntry to a string suitable for assertion testing.
-  string16 BookmarkEntryToString(
-      const ProfileWriter::BookmarkEntry& entry) {
+  string16 BookmarkEntryToString(const ProfileWriter::BookmarkEntry& entry) {
     string16 result;
     result.append(ASCIIToUTF16("on_toolbar="));
     if (entry.in_toolbar)
-      result.append(ASCIIToUTF16("false"));
-    else
       result.append(ASCIIToUTF16("true"));
+    else
+      result.append(ASCIIToUTF16("false"));
 
     result.append(ASCIIToUTF16(" url=") + UTF8ToUTF16(entry.url.spec()));
 
@@ -68,11 +65,11 @@ class BookmarkHTMLWriterTest : public testing::Test {
     for (size_t i = 0; i < entry.path.size(); ++i) {
       if (i != 0)
         result.append(ASCIIToUTF16("/"));
-      result.append(WideToUTF16Hack(entry.path[i]));
+      result.append(entry.path[i]);
     }
 
     result.append(ASCIIToUTF16(" title="));
-    result.append(WideToUTF16Hack(entry.title));
+    result.append(entry.title);
 
     result.append(ASCIIToUTF16(" time="));
     result.append(base::TimeFormatFriendlyDateAndTime(entry.creation_time));
@@ -90,18 +87,15 @@ class BookmarkHTMLWriterTest : public testing::Test {
     ProfileWriter::BookmarkEntry entry;
     entry.in_toolbar = on_toolbar;
     entry.url = url;
-    // The first path element should always be 'x', as that is what we passed
-    // to the importer.
-    entry.path.push_back(L"x");
     if (!f1.empty()) {
-      entry.path.push_back(UTF16ToWideHack(f1));
+      entry.path.push_back(f1);
       if (!f2.empty()) {
-        entry.path.push_back(UTF16ToWideHack(f2));
+        entry.path.push_back(f2);
         if (!f3.empty())
-          entry.path.push_back(UTF16ToWideHack(f3));
+          entry.path.push_back(f3);
       }
     }
-    entry.title = UTF16ToWideHack(title);
+    entry.title = title;
     entry.creation_time = creation_time;
     return BookmarkEntryToString(entry);
   }
@@ -119,6 +113,7 @@ class BookmarkHTMLWriterTest : public testing::Test {
               BookmarkEntryToString(entry));
   }
 
+  ScopedTempDir temp_dir_;
   FilePath path_;
 };
 
@@ -142,8 +137,9 @@ class BookmarksObserver : public BookmarksExportObserver {
 // way of bookmark_html_writer, then using the importer to read it back in.
 TEST_F(BookmarkHTMLWriterTest, Test) {
   MessageLoop message_loop;
-  BrowserThread fake_ui_thread(BrowserThread::UI, &message_loop);
-  BrowserThread fake_file_thread(BrowserThread::FILE, &message_loop);
+  content::TestBrowserThread fake_ui_thread(BrowserThread::UI, &message_loop);
+  content::TestBrowserThread fake_file_thread(BrowserThread::FILE,
+                                              &message_loop);
 
   TestingProfile profile;
   profile.CreateHistoryService(true, false);
@@ -173,6 +169,8 @@ TEST_F(BookmarkHTMLWriterTest, Test) {
   //   F3
   //     F4
   //       url1
+  // Mobile
+  //   url1
   string16 f1_title = ASCIIToUTF16("F\"&;<1\"");
   string16 f2_title = ASCIIToUTF16("F2");
   string16 f3_title = ASCIIToUTF16("F 3");
@@ -190,27 +188,27 @@ TEST_F(BookmarkHTMLWriterTest, Test) {
   base::Time t2(t1 + base::TimeDelta::FromHours(1));
   base::Time t3(t1 + base::TimeDelta::FromHours(1));
   base::Time t4(t1 + base::TimeDelta::FromHours(1));
-  const BookmarkNode* f1 = model->AddGroup(
-      model->GetBookmarkBarNode(), 0, f1_title);
+  const BookmarkNode* f1 = model->AddFolder(
+      model->bookmark_bar_node(), 0, f1_title);
   model->AddURLWithCreationTime(f1, 0, url1_title, url1, t1);
   profile.GetHistoryService(Profile::EXPLICIT_ACCESS)->AddPage(url1,
       history::SOURCE_BROWSED);
   profile.GetFaviconService(Profile::EXPLICIT_ACCESS)->SetFavicon(url1,
-                                                                  url1_favicon,
-                                                                  icon_data);
+      url1_favicon, icon_data, history::FAVICON);
   message_loop.RunAllPending();
-  const BookmarkNode* f2 = model->AddGroup(f1, 1, f2_title);
+  const BookmarkNode* f2 = model->AddFolder(f1, 1, f2_title);
   model->AddURLWithCreationTime(f2, 0, url2_title, url2, t2);
-  model->AddURLWithCreationTime(model->GetBookmarkBarNode(),
+  model->AddURLWithCreationTime(model->bookmark_bar_node(),
                                 1, url3_title, url3, t3);
 
   model->AddURLWithCreationTime(model->other_node(), 0, url1_title, url1, t1);
   model->AddURLWithCreationTime(model->other_node(), 1, url2_title, url2, t2);
-  const BookmarkNode* f3 = model->AddGroup(model->other_node(), 2, f3_title);
-  const BookmarkNode* f4 = model->AddGroup(f3, 0, f4_title);
+  const BookmarkNode* f3 = model->AddFolder(model->other_node(), 2, f3_title);
+  const BookmarkNode* f4 = model->AddFolder(f3, 0, f4_title);
   model->AddURLWithCreationTime(f4, 0, url1_title, url1, t1);
-  model->AddURLWithCreationTime(model->GetBookmarkBarNode(), 2, url4_title,
+  model->AddURLWithCreationTime(model->bookmark_bar_node(), 2, url4_title,
                                 url4, t4);
+  model->AddURLWithCreationTime(model->mobile_node(), 0, url1_title, url1, t1);
 
   // Write to a temp file.
   BookmarksObserver observer(&message_loop);
@@ -220,19 +218,21 @@ TEST_F(BookmarkHTMLWriterTest, Test) {
   // Clear favicon so that it would be read from file.
   std::vector<unsigned char> empty_data;
   profile.GetFaviconService(Profile::EXPLICIT_ACCESS)->SetFavicon(url1,
-                                                                  url1_favicon,
-                                                                  empty_data);
+      url1_favicon, empty_data, history::FAVICON);
   message_loop.RunAllPending();
 
   // Read the bookmarks back in.
   std::vector<ProfileWriter::BookmarkEntry> parsed_bookmarks;
-  std::vector<history::ImportedFavIconUsage> favicons;
-  Firefox2Importer::ImportBookmarksFile(path_, std::set<GURL>(),
-                                        false, L"x", NULL, &parsed_bookmarks,
-                                        NULL, &favicons);
+  std::vector<history::ImportedFaviconUsage> favicons;
+  Firefox2Importer::ImportBookmarksFile(path_,
+                                        std::set<GURL>(),
+                                        NULL,
+                                        &parsed_bookmarks,
+                                        NULL,
+                                        &favicons);
 
-  // Check loaded favicon (url1 is represents by 3 separate bookmarks).
-  EXPECT_EQ(3U, favicons.size());
+  // Check loaded favicon (url1 is represented by 4 separate bookmarks).
+  EXPECT_EQ(4U, favicons.size());
   for (size_t i = 0; i < favicons.size(); i++) {
     if (url1_favicon == favicons[i].favicon_url) {
       EXPECT_EQ(1U, favicons[i].urls.size());
@@ -244,17 +244,17 @@ TEST_F(BookmarkHTMLWriterTest, Test) {
   }
 
   // Verify we got back what we wrote.
-  ASSERT_EQ(7U, parsed_bookmarks.size());
+  ASSERT_EQ(8U, parsed_bookmarks.size());
   // Windows and ChromeOS builds use Sentence case.
   string16 bookmark_folder_name =
-      l10n_util::GetStringUTF16(IDS_BOOMARK_BAR_FOLDER_NAME);
-  AssertBookmarkEntryEquals(parsed_bookmarks[0], false, url1, url1_title, t1,
+      l10n_util::GetStringUTF16(IDS_BOOKMARK_BAR_FOLDER_NAME);
+  AssertBookmarkEntryEquals(parsed_bookmarks[0], true, url1, url1_title, t1,
                             bookmark_folder_name, f1_title, string16());
-  AssertBookmarkEntryEquals(parsed_bookmarks[1], false, url2, url2_title, t2,
+  AssertBookmarkEntryEquals(parsed_bookmarks[1], true, url2, url2_title, t2,
                             bookmark_folder_name, f1_title, f2_title);
-  AssertBookmarkEntryEquals(parsed_bookmarks[2], false, url3, url3_title, t3,
+  AssertBookmarkEntryEquals(parsed_bookmarks[2], true, url3, url3_title, t3,
                             bookmark_folder_name, string16(), string16());
-  AssertBookmarkEntryEquals(parsed_bookmarks[3], false, url4, url4_title, t4,
+  AssertBookmarkEntryEquals(parsed_bookmarks[3], true, url4, url4_title, t4,
                             bookmark_folder_name, string16(), string16());
   AssertBookmarkEntryEquals(parsed_bookmarks[4], false, url1, url1_title, t1,
                             string16(), string16(), string16());
@@ -262,4 +262,6 @@ TEST_F(BookmarkHTMLWriterTest, Test) {
                             string16(), string16(), string16());
   AssertBookmarkEntryEquals(parsed_bookmarks[6], false, url1, url1_title, t1,
                             f3_title, f4_title, string16());
+  AssertBookmarkEntryEquals(parsed_bookmarks[7], false, url1, url1_title, t1,
+                            string16(), string16(), string16());
 }
