@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,7 @@
 #include <iterator>
 #include <list>
 
+#include "base/i18n/case_conversion.h"
 #include "base/string16.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/bookmarks/bookmark_utils.h"
@@ -95,7 +96,7 @@ void BookmarkIndex::GetBookmarksWithTitlesMatching(
   // matches and so this shouldn't be performance critical.
   QueryParser parser;
   ScopedVector<QueryNode> query_nodes;
-  parser.ParseQuery(query, &query_nodes.get());
+  parser.ParseQueryNodes(query, &query_nodes.get());
 
   // The highest typed counts should be at the beginning of the results vector
   // so that the best matches will always be included in the results. The loop
@@ -131,7 +132,7 @@ void BookmarkIndex::ExtractBookmarkNodePairs(
        i != match.nodes_end(); ++i) {
     history::URLRow url;
     if (url_db)
-      url_db->GetRowForURL((*i)->GetURL(), &url);
+      url_db->GetRowForURL((*i)->url(), &url);
     NodeTypedCountPair pair(*i, url.typed_count());
     node_typed_counts->push_back(pair);
   }
@@ -242,8 +243,9 @@ std::vector<string16> BookmarkIndex::ExtractQueryWords(const string16& query) {
   if (query.empty())
     return std::vector<string16>();
   QueryParser parser;
-  // TODO: use ICU normalization.
-  parser.ExtractQueryWords(l10n_util::ToLower(query), &terms);
+  // TODO(brettw): use ICU normalization:
+  // http://userguide.icu-project.org/transforms/normalization
+  parser.ParseQueryWords(base::i18n::ToLower(query), &terms);
   return terms;
 }
 
