@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,15 +7,15 @@
 #pragma once
 
 #include "base/file_path.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/bookmarks/bookmark_index.h"
 #include "chrome/common/important_file_writer.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 class BookmarkModel;
-class BookmarkNode;
+class BookmarkPermanentNode;
 class Profile;
 
 // BookmarkLoadDetails is used by BookmarkStorage when loading bookmarks.
@@ -28,16 +28,25 @@ class Profile;
 // problems.
 class BookmarkLoadDetails {
  public:
-  BookmarkLoadDetails(BookmarkNode* bb_node,
-                      BookmarkNode* other_folder_node,
+  BookmarkLoadDetails(BookmarkPermanentNode* bb_node,
+                      BookmarkPermanentNode* other_folder_node,
+                      BookmarkPermanentNode* mobile_folder_node,
                       BookmarkIndex* index,
                       int64 max_id);
   ~BookmarkLoadDetails();
 
-  BookmarkNode* bb_node() { return bb_node_.get(); }
-  BookmarkNode* release_bb_node() { return bb_node_.release(); }
-  BookmarkNode* other_folder_node() { return other_folder_node_.get(); }
-  BookmarkNode* release_other_folder_node() {
+  BookmarkPermanentNode* bb_node() { return bb_node_.get(); }
+  BookmarkPermanentNode* release_bb_node() { return bb_node_.release(); }
+  BookmarkPermanentNode* mobile_folder_node() {
+    return mobile_folder_node_.get();
+  }
+  BookmarkPermanentNode* release_mobile_folder_node() {
+    return mobile_folder_node_.release();
+  }
+  BookmarkPermanentNode* other_folder_node() {
+    return other_folder_node_.get();
+  }
+  BookmarkPermanentNode* release_other_folder_node() {
     return other_folder_node_.release();
   }
   BookmarkIndex* index() { return index_.get(); }
@@ -59,13 +68,17 @@ class BookmarkLoadDetails {
   }
   const std::string& stored_checksum() const { return stored_checksum_; }
 
-  // Whether ids were reassigned.
+  // Whether ids were reassigned. IDs are reassigned during decoding if the
+  // checksum of the file doesn't match, some IDs are missing or not
+  // unique. Basically, if the user modified the bookmarks directly we'll
+  // reassign the ids to ensure they are unique.
   void set_ids_reassigned(bool value) { ids_reassigned_ = value; }
   bool ids_reassigned() const { return ids_reassigned_; }
 
  private:
-  scoped_ptr<BookmarkNode> bb_node_;
-  scoped_ptr<BookmarkNode> other_folder_node_;
+  scoped_ptr<BookmarkPermanentNode> bb_node_;
+  scoped_ptr<BookmarkPermanentNode> other_folder_node_;
+  scoped_ptr<BookmarkPermanentNode> mobile_folder_node_;
   scoped_ptr<BookmarkIndex> index_;
   int64 max_id_;
   std::string computed_checksum_;
@@ -80,7 +93,7 @@ class BookmarkLoadDetails {
 // as notifying the BookmarkStorage every time the model changes.
 //
 // Internally BookmarkStorage uses BookmarkCodec to do the actual read/write.
-class BookmarkStorage : public NotificationObserver,
+class BookmarkStorage : public content::NotificationObserver,
                         public ImportantFileWriter::DataSerializer,
                         public base::RefCountedThreadSafe<BookmarkStorage> {
  public:
@@ -98,21 +111,19 @@ class BookmarkStorage : public NotificationObserver,
   // a pending save, it is saved immediately.
   void BookmarkModelDeleted();
 
-  // ImportantFileWriter::DataSerializer
-  virtual bool SerializeData(std::string* output);
+  // Callback from backend with the results of the bookmark file. This may be
+  // called multiple times, with different paths. This happens when we migrate
+  // bookmark data from database.
+  void OnLoadFinished(bool file_exists,
+                      const FilePath& path);
+
+  // ImportantFileWriter::DataSerializer implementation.
+  virtual bool SerializeData(std::string* output) OVERRIDE;
 
  private:
   friend class base::RefCountedThreadSafe<BookmarkStorage>;
 
-  ~BookmarkStorage();
-
-  class LoadTask;
-
-  // Callback from backend with the results of the bookmark file.
-  // This may be called multiple times, with different paths. This happens when
-  // we migrate bookmark data from database.
-  void OnLoadFinished(bool file_exists,
-                      const FilePath& path);
+  virtual ~BookmarkStorage();
 
   // Loads bookmark data from |file| and notifies the model when finished.
   void DoLoadBookmarks(const FilePath& file);
@@ -128,10 +139,10 @@ class BookmarkStorage : public NotificationObserver,
   // the temporary file, and notifies the model.
   void FinishHistoryMigration();
 
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Serializes the data and schedules save using ImportantFileWriter.
   // Returns true on successful serialization.
@@ -147,7 +158,7 @@ class BookmarkStorage : public NotificationObserver,
   ImportantFileWriter writer_;
 
   // Helper to ensure that we unregister from notifications on destruction.
-  NotificationRegistrar notification_registrar_;
+  content::NotificationRegistrar notification_registrar_;
 
   // Path to temporary file created during migrating bookmarks from history.
   const FilePath tmp_history_path_;
