@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,45 +8,53 @@
 
 #include "base/basictypes.h"
 #include "base/file_path.h"
+#include "base/i18n/case_conversion.h"
+#include "base/i18n/string_search.h"
+#include "base/metrics/histogram.h"
 #include "base/string16.h"
 #include "base/string_number_conversions.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/bookmarks/bookmark_node_data.h"
-#if defined(OS_MACOSX)
-#include "chrome/browser/bookmarks/bookmark_pasteboard_helper_mac.h"
-#endif
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_window.h"
 #include "chrome/browser/history/query_parser.h"
-#include "chrome/browser/platform_util.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/page_navigator.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/simple_message_box.h"
 #include "chrome/browser/ui/browser.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
 #include "chrome/common/pref_names.h"
-#include "grit/app_strings.h"
+#include "content/public/browser/page_navigator.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
+#include "grit/ui_strings.h"
 #include "net/base/net_util.h"
 #include "ui/base/dragdrop/drag_drop_types.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/models/tree_node_iterator.h"
-#include "views/event.h"
+
+#if defined(OS_MACOSX)
+#include "chrome/browser/bookmarks/bookmark_pasteboard_helper_mac.h"
+#endif
 
 #if defined(TOOLKIT_VIEWS)
 #include "ui/base/dragdrop/os_exchange_data.h"
-#include "views/drag_utils.h"
-#include "views/widget/root_view.h"
-#include "views/widget/widget.h"
-#elif defined(TOOLKIT_GTK)
+#include "ui/views/drag_utils.h"
+#include "ui/views/events/event.h"
+#include "ui/views/widget/native_widget.h"
+#include "ui/views/widget/widget.h"
+#endif
+
+#if defined(TOOLKIT_GTK)
 #include "chrome/browser/ui/gtk/custom_drag.h"
 #endif
 
 using base::Time;
+using content::OpenURLParams;
+using content::PageNavigator;
+using content::WebContents;
 
 namespace {
 
@@ -66,18 +74,16 @@ class NewBrowserPageNavigator : public PageNavigator {
 
   Browser* browser() const { return browser_; }
 
-  virtual void OpenURL(const GURL& url,
-                       const GURL& referrer,
-                       WindowOpenDisposition disposition,
-                       PageTransition::Type transition) {
+  virtual WebContents* OpenURL(const OpenURLParams& params) OVERRIDE {
     if (!browser_) {
-      Profile* profile = (disposition == OFF_THE_RECORD) ?
+      Profile* profile = (params.disposition == OFF_THE_RECORD) ?
           profile_->GetOffTheRecordProfile() : profile_;
       browser_ = Browser::Create(profile);
-      // Always open the first tab in the foreground.
-      disposition = NEW_FOREGROUND_TAB;
     }
-    browser_->OpenURL(url, referrer, NEW_FOREGROUND_TAB, transition);
+
+    OpenURLParams forward_params = params;
+    forward_params.disposition = NEW_FOREGROUND_TAB;
+    return browser_->OpenURL(forward_params);
   }
 
  private:
@@ -94,36 +100,47 @@ void CloneBookmarkNodeImpl(BookmarkModel* model,
   if (element.is_url) {
     model->AddURL(parent, index_to_add_at, element.title, element.url);
   } else {
-    const BookmarkNode* new_folder = model->AddGroup(parent,
-                                                     index_to_add_at,
-                                                     element.title);
+    const BookmarkNode* new_folder = model->AddFolder(parent,
+                                                      index_to_add_at,
+                                                      element.title);
     for (int i = 0; i < static_cast<int>(element.children.size()); ++i)
       CloneBookmarkNodeImpl(model, element.children[i], new_folder, i);
   }
 }
 
-// Returns the number of descendants of node that are of type url.
-int DescendantURLCount(const BookmarkNode* node) {
+// Returns the number of children of |node| that are of type url.
+int ChildURLCount(const BookmarkNode* node) {
   int result = 0;
-  for (int i = 0; i < node->GetChildCount(); ++i) {
+  for (int i = 0; i < node->child_count(); ++i) {
     const BookmarkNode* child = node->GetChild(i);
     if (child->is_url())
       result++;
-    else
-      result += DescendantURLCount(child);
   }
   return result;
 }
 
-// Implementation of OpenAll. Opens all nodes of type URL and recurses for
-// groups. |navigator| is the PageNavigator used to open URLs. After the first
-// url is opened |opened_url| is set to true and |navigator| is set to the
-// PageNavigator of the last active tab. This is done to handle a window
-// disposition of new window, in which case we want subsequent tabs to open in
-// that window.
+// Returns the total number of descendants nodes.
+int ChildURLCountTotal(const BookmarkNode* node) {
+  int result = 0;
+  for (int i = 0; i < node->child_count(); ++i) {
+    const BookmarkNode* child = node->GetChild(i);
+    result++;
+    if (child->is_folder())
+      result += ChildURLCountTotal(child);
+  }
+  return result;
+}
+
+// Implementation of OpenAll. Opens all nodes of type URL and any children of
+// |node| that are of type URL. |navigator| is the PageNavigator used to open
+// URLs. After the first url is opened |opened_url| is set to true and
+// |navigator| is set to the PageNavigator of the last active tab. This is done
+// to handle a window disposition of new window, in which case we want
+// subsequent tabs to open in that window.
 void OpenAllImpl(const BookmarkNode* node,
                  WindowOpenDisposition initial_disposition,
                  PageNavigator** navigator,
+                 Profile* profile,
                  bool* opened_url) {
   if (node->is_url()) {
     WindowOpenDisposition disposition;
@@ -131,47 +148,50 @@ void OpenAllImpl(const BookmarkNode* node,
       disposition = NEW_BACKGROUND_TAB;
     else
       disposition = initial_disposition;
-    (*navigator)->OpenURL(node->GetURL(), GURL(), disposition,
-                          PageTransition::AUTO_BOOKMARK);
+    (*navigator)->OpenURL(OpenURLParams(node->url(), content::Referrer(),
+                          disposition, content::PAGE_TRANSITION_AUTO_BOOKMARK,
+                          false));
     if (!*opened_url) {
       *opened_url = true;
       // We opened the first URL which may have opened a new window or clobbered
       // the current page, reset the navigator just to be sure.
-      Browser* new_browser = BrowserList::GetLastActive();
+      Browser* new_browser = BrowserList::GetLastActiveWithProfile(profile);
       if (new_browser) {
-        TabContents* current_tab = new_browser->GetSelectedTabContents();
+        WebContents* current_tab = new_browser->GetSelectedWebContents();
         DCHECK(new_browser && current_tab);
         if (new_browser && current_tab)
           *navigator = current_tab;
       }  // else, new_browser == NULL, which happens during testing.
     }
   } else {
-    // Group, recurse through children.
-    for (int i = 0; i < node->GetChildCount(); ++i) {
-      OpenAllImpl(node->GetChild(i), initial_disposition, navigator,
-                  opened_url);
+    // For folders only open direct children.
+    for (int i = 0; i < node->child_count(); ++i) {
+      const BookmarkNode* child_node = node->GetChild(i);
+      if (child_node->is_url())
+        OpenAllImpl(child_node, initial_disposition, navigator, profile,
+                    opened_url);
     }
   }
 }
 
 bool ShouldOpenAll(gfx::NativeWindow parent,
                    const std::vector<const BookmarkNode*>& nodes) {
-  int descendant_count = 0;
+  int child_count = 0;
   for (size_t i = 0; i < nodes.size(); ++i)
-    descendant_count += DescendantURLCount(nodes[i]);
-  if (descendant_count < bookmark_utils::num_urls_before_prompting)
+    child_count += ChildURLCount(nodes[i]);
+  if (child_count < bookmark_utils::num_urls_before_prompting)
     return true;
 
   string16 message = l10n_util::GetStringFUTF16(
       IDS_BOOKMARK_BAR_SHOULD_OPEN_ALL,
-      base::IntToString16(descendant_count));
+      base::IntToString16(child_count));
   string16 title = l10n_util::GetStringUTF16(IDS_PRODUCT_NAME);
-  return platform_util::SimpleYesNoBox(parent, title, message);
+  return browser::ShowYesNoBox(parent, title, message);
 }
 
 // Comparison function that compares based on date modified of the two nodes.
 bool MoreRecentlyModified(const BookmarkNode* n1, const BookmarkNode* n2) {
-  return n1->date_group_modified() > n2->date_group_modified();
+  return n1->date_folder_modified() > n2->date_folder_modified();
 }
 
 // Returns true if |text| contains each string in |words|. This is used when
@@ -179,7 +199,7 @@ bool MoreRecentlyModified(const BookmarkNode* n1, const BookmarkNode* n2) {
 bool DoesBookmarkTextContainWords(const string16& text,
                                   const std::vector<string16>& words) {
   for (size_t i = 0; i < words.size(); ++i) {
-    if (text.find(words[i]) == string16::npos)
+    if (!base::i18n::StringSearchIgnoringCaseAndAccents(words[i], text))
       return false;
   }
   return true;
@@ -191,13 +211,11 @@ bool DoesBookmarkContainWords(const BookmarkNode* node,
                               const std::vector<string16>& words,
                               const std::string& languages) {
   return
-      DoesBookmarkTextContainWords(
-          l10n_util::ToLower(node->GetTitle()), words) ||
-      DoesBookmarkTextContainWords(
-          l10n_util::ToLower(UTF8ToUTF16(node->GetURL().spec())), words) ||
-      DoesBookmarkTextContainWords(l10n_util::ToLower(
-          net::FormatUrl(node->GetURL(), languages, net::kFormatUrlOmitNothing,
-                         UnescapeRule::NORMAL, NULL, NULL, NULL)), words);
+      DoesBookmarkTextContainWords(node->GetTitle(), words) ||
+      DoesBookmarkTextContainWords(UTF8ToUTF16(node->url().spec()), words) ||
+      DoesBookmarkTextContainWords(net::FormatUrl(
+          node->url(), languages, net::kFormatUrlOmitNothing,
+          net::UnescapeRule::NORMAL, NULL, NULL, NULL), words);
 }
 
 }  // namespace
@@ -219,14 +237,17 @@ int PreferredDropOperation(int source_operations, int operations) {
   return ui::DragDropTypes::DRAG_NONE;
 }
 
-int BookmarkDragOperation(const BookmarkNode* node) {
+int BookmarkDragOperation(Profile* profile, const BookmarkNode* node) {
+  int move = ui::DragDropTypes::DRAG_MOVE;
+  if (!profile->GetPrefs()->GetBoolean(prefs::kEditBookmarksEnabled))
+    move = 0;
   if (node->is_url()) {
-    return ui::DragDropTypes::DRAG_COPY | ui::DragDropTypes::DRAG_MOVE |
-           ui::DragDropTypes::DRAG_LINK;
+    return ui::DragDropTypes::DRAG_COPY | ui::DragDropTypes::DRAG_LINK | move;
   }
-  return ui::DragDropTypes::DRAG_COPY | ui::DragDropTypes::DRAG_MOVE;
+  return ui::DragDropTypes::DRAG_COPY | move;
 }
 
+#if defined(TOOLKIT_VIEWS)
 int BookmarkDropOperation(Profile* profile,
                           const views::DropTargetEvent& event,
                           const BookmarkNodeData& data,
@@ -244,9 +265,10 @@ int BookmarkDropOperation(Profile* profile,
     return ui::DragDropTypes::DRAG_MOVE;
   }
   // User is dragging from another app, copy.
-  return PreferredDropOperation(event.GetSourceOperations(),
+  return PreferredDropOperation(event.source_operations(),
       ui::DragDropTypes::DRAG_COPY | ui::DragDropTypes::DRAG_LINK);
 }
+#endif  // defined(TOOLKIT_VIEWS)
 
 int PerformBookmarkDrop(Profile* profile,
                         const BookmarkNodeData& data,
@@ -260,13 +282,13 @@ int PerformBookmarkDrop(Profile* profile,
       // Drag from same profile. Move nodes.
       for (size_t i = 0; i < dragged_nodes.size(); ++i) {
         model->Move(dragged_nodes[i], parent_node, index);
-        index = parent_node->IndexOfChild(dragged_nodes[i]) + 1;
+        index = parent_node->GetIndexOf(dragged_nodes[i]) + 1;
       }
       return ui::DragDropTypes::DRAG_MOVE;
     }
     return ui::DragDropTypes::DRAG_NONE;
   }
-  // Dropping a group from different profile. Always accept.
+  // Dropping a folder from different profile. Always accept.
   bookmark_utils::CloneBookmarkNode(model, data.elements, parent_node, index);
   return ui::DragDropTypes::DRAG_COPY;
 }
@@ -289,8 +311,8 @@ bool IsValidDropLocation(Profile* profile,
       // Don't allow the drop if the user is attempting to drop on one of the
       // nodes being dragged.
       const BookmarkNode* node = nodes[i];
-      int node_index = (drop_parent == node->GetParent()) ?
-          drop_parent->IndexOfChild(nodes[i]) : -1;
+      int node_index = (drop_parent == node->parent()) ?
+          drop_parent->GetIndexOf(nodes[i]) : -1;
       if (node_index != -1 && (index == node_index || index == node_index + 1))
         return false;
 
@@ -329,16 +351,16 @@ void DragBookmarks(Profile* profile,
   BookmarkNodeData drag_data(nodes);
   drag_data.Write(profile, &data);
 
-  views::RootView* root_view =
-      views::Widget::GetWidgetFromNativeView(view)->GetRootView();
-
   // Allow nested message loop so we get DnD events as we drag this around.
   bool was_nested = MessageLoop::current()->IsNested();
   MessageLoop::current()->SetNestableTasksAllowed(true);
 
-  root_view->StartDragForViewFromMouseEvent(NULL, data,
-      ui::DragDropTypes::DRAG_COPY | ui::DragDropTypes::DRAG_MOVE |
-      ui::DragDropTypes::DRAG_LINK);
+  views::Widget* widget = views::Widget::GetWidgetForNativeView(view);
+  if (widget) {
+    widget->RunShellDrag(NULL, data,
+        ui::DragDropTypes::DRAG_COPY | ui::DragDropTypes::DRAG_MOVE |
+        ui::DragDropTypes::DRAG_LINK);
+  }
 
   MessageLoop::current()->SetNestableTasksAllowed(was_nested);
 #elif defined(OS_MACOSX)
@@ -362,22 +384,25 @@ void OpenAll(gfx::NativeWindow parent,
 
   NewBrowserPageNavigator navigator_impl(profile);
   if (!navigator) {
-    Browser* browser =
-        BrowserList::FindBrowserWithType(profile, Browser::TYPE_NORMAL, false);
-    if (!browser || !browser->GetSelectedTabContents()) {
+    Browser* browser = BrowserList::FindTabbedBrowser(profile, false);
+    if (!browser || !browser->GetSelectedWebContents()) {
       navigator = &navigator_impl;
     } else {
       if (initial_disposition != NEW_WINDOW &&
           initial_disposition != OFF_THE_RECORD) {
         browser->window()->Activate();
       }
-      navigator = browser->GetSelectedTabContents();
+      navigator = browser->GetSelectedWebContents();
     }
   }
 
   bool opened_url = false;
+  if (initial_disposition == OFF_THE_RECORD)
+    profile = profile->GetOffTheRecordProfile();
+
   for (size_t i = 0; i < nodes.size(); ++i)
-    OpenAllImpl(nodes[i], initial_disposition, &navigator, &opened_url);
+    OpenAllImpl(nodes[i], initial_disposition, &navigator, profile,
+                &opened_url);
 }
 
 void OpenAll(gfx::NativeWindow parent,
@@ -400,8 +425,9 @@ void CopyToClipboard(BookmarkModel* model,
 
   if (remove_nodes) {
     for (size_t i = 0; i < nodes.size(); ++i) {
-      model->Remove(nodes[i]->GetParent(),
-                    nodes[i]->GetParent()->IndexOfChild(nodes[i]));
+      int index = nodes[i]->parent()->GetIndexOf(nodes[i]);
+      if (index > -1)
+        model->Remove(nodes[i]->parent(), index);
     }
   }
 }
@@ -417,7 +443,7 @@ void PasteFromClipboard(BookmarkModel* model,
     return;
 
   if (index == -1)
-    index = parent->GetChildCount();
+    index = parent->child_count();
   bookmark_utils::CloneBookmarkNode(
       model, bookmark_data.elements, parent, index);
 }
@@ -430,21 +456,27 @@ bool CanPasteFromClipboard(const BookmarkNode* node) {
 
 string16 GetNameForURL(const GURL& url) {
   if (url.is_valid()) {
-    return WideToUTF16(net::GetSuggestedFilename(
-        url, std::string(), std::string(), FilePath()).ToWStringHack());
+    return net::GetSuggestedFilename(url, "", "", "", "", std::string());
   } else {
     return l10n_util::GetStringUTF16(IDS_APP_UNTITLED_SHORTCUT_FILE_NAME);
   }
 }
 
-std::vector<const BookmarkNode*> GetMostRecentlyModifiedGroups(
+// This is used with a tree iterator to skip subtrees which are not visible.
+static bool PruneInvisibleFolders(const BookmarkNode* node) {
+  return !node->IsVisible();
+}
+
+std::vector<const BookmarkNode*> GetMostRecentlyModifiedFolders(
     BookmarkModel* model,
     size_t max_count) {
   std::vector<const BookmarkNode*> nodes;
-  ui::TreeNodeIterator<const BookmarkNode> iterator(model->root_node());
+  ui::TreeNodeIterator<const BookmarkNode>
+      iterator(model->root_node(), PruneInvisibleFolders);
+
   while (iterator.has_next()) {
     const BookmarkNode* parent = iterator.Next();
-    if (parent->is_folder() && parent->date_group_modified() > base::Time()) {
+    if (parent->is_folder() && parent->date_folder_modified() > base::Time()) {
       if (max_count == 0) {
         nodes.push_back(parent);
       } else {
@@ -462,15 +494,19 @@ std::vector<const BookmarkNode*> GetMostRecentlyModifiedGroups(
   }
 
   if (nodes.size() < max_count) {
-    // Add the bookmark bar and other nodes if there is space.
-    if (find(nodes.begin(), nodes.end(), model->GetBookmarkBarNode()) ==
-        nodes.end()) {
-      nodes.push_back(model->GetBookmarkBarNode());
-    }
+    // Add the permanent nodes if there is space. The permanent nodes are the
+    // only children of the root_node.
+    const BookmarkNode* root_node = model->root_node();
 
-    if (nodes.size() < max_count &&
-        find(nodes.begin(), nodes.end(), model->other_node()) == nodes.end()) {
-      nodes.push_back(model->other_node());
+    for (int i = 0; i < root_node->child_count(); ++i) {
+      const BookmarkNode* node = root_node->GetChild(i);
+      if (node->IsVisible() &&
+          std::find(nodes.begin(), nodes.end(), node) == nodes.end()) {
+        nodes.push_back(node);
+
+        if (nodes.size() == max_count)
+          break;
+      }
     }
   }
   return nodes;
@@ -495,6 +531,12 @@ void GetMostRecentlyAddedEntries(BookmarkModel* model,
   }
 }
 
+TitleMatch::TitleMatch()
+    : node(NULL) {
+}
+
+TitleMatch::~TitleMatch() {}
+
 bool MoreRecentlyAdded(const BookmarkNode* n1, const BookmarkNode* n2) {
   return n1->date_added() > n2->date_added();
 }
@@ -506,7 +548,7 @@ void GetBookmarksContainingText(BookmarkModel* model,
                                 std::vector<const BookmarkNode*>* nodes) {
   std::vector<string16> words;
   QueryParser parser;
-  parser.ExtractQueryWords(l10n_util::ToLower(text), &words);
+  parser.ParseQueryWords(base::i18n::ToLower(text), &words);
   if (words.empty())
     return;
 
@@ -526,7 +568,7 @@ bool DoesBookmarkContainText(const BookmarkNode* node,
                              const std::string& languages) {
   std::vector<string16> words;
   QueryParser parser;
-  parser.ExtractQueryWords(l10n_util::ToLower(text), &words);
+  parser.ParseQueryWords(base::i18n::ToLower(text), &words);
   if (words.empty())
     return false;
 
@@ -538,14 +580,14 @@ static const BookmarkNode* CreateNewNode(BookmarkModel* model,
     const string16& new_title, const GURL& new_url) {
   const BookmarkNode* node;
   if (details.type == BookmarkEditor::EditDetails::NEW_URL) {
-    node = model->AddURL(parent, parent->GetChildCount(), new_title, new_url);
+    node = model->AddURL(parent, parent->child_count(), new_title, new_url);
   } else if (details.type == BookmarkEditor::EditDetails::NEW_FOLDER) {
-    node = model->AddGroup(parent, parent->GetChildCount(), new_title);
+    node = model->AddFolder(parent, parent->child_count(), new_title);
     for (size_t i = 0; i < details.urls.size(); ++i) {
-      model->AddURL(node, node->GetChildCount(), details.urls[i].second,
+      model->AddURL(node, node->child_count(), details.urls[i].second,
                     details.urls[i].first);
     }
-    model->SetDateGroupModified(parent, Time::Now());
+    model->SetDateFolderModified(parent, Time::Now());
   } else {
     NOTREACHED();
     return NULL;
@@ -554,7 +596,7 @@ static const BookmarkNode* CreateNewNode(BookmarkModel* model,
   return node;
 }
 
-const BookmarkNode* ApplyEditsWithNoGroupChange(BookmarkModel* model,
+const BookmarkNode* ApplyEditsWithNoFolderChange(BookmarkModel* model,
     const BookmarkNode* parent, const BookmarkEditor::EditDetails& details,
     const string16& new_title, const GURL& new_url) {
   if (details.type == BookmarkEditor::EditDetails::NEW_URL ||
@@ -572,7 +614,7 @@ const BookmarkNode* ApplyEditsWithNoGroupChange(BookmarkModel* model,
   return node;
 }
 
-const BookmarkNode* ApplyEditsWithPossibleGroupChange(BookmarkModel* model,
+const BookmarkNode* ApplyEditsWithPossibleFolderChange(BookmarkModel* model,
     const BookmarkNode* new_parent, const BookmarkEditor::EditDetails& details,
     const string16& new_title, const GURL& new_url) {
   if (details.type == BookmarkEditor::EditDetails::NEW_URL ||
@@ -583,8 +625,8 @@ const BookmarkNode* ApplyEditsWithPossibleGroupChange(BookmarkModel* model,
   const BookmarkNode* node = details.existing_node;
   DCHECK(node);
 
-  if (new_parent != node->GetParent())
-    model->Move(node, new_parent, new_parent->GetChildCount());
+  if (new_parent != node->parent())
+    model->Move(node, new_parent, new_parent->child_count());
   if (node->is_url())
     model->SetURL(node, new_url);
   model->SetTitle(node, new_title);
@@ -599,32 +641,40 @@ void ToggleWhenVisible(Profile* profile) {
 
   // The user changed when the bookmark bar is shown, update the preferences.
   prefs->SetBoolean(prefs::kShowBookmarkBar, always_show);
-  prefs->ScheduleSavePersistentPrefs();
-
-  // And notify the notification service.
-  Source<Profile> source(profile);
-  NotificationService::current()->Notify(
-      NotificationType::BOOKMARK_BAR_VISIBILITY_PREF_CHANGED,
-      source,
-      NotificationService::NoDetails());
 }
 
 void RegisterUserPrefs(PrefService* prefs) {
-  prefs->RegisterBooleanPref(prefs::kShowBookmarkBar, false);
+  prefs->RegisterBooleanPref(prefs::kShowBookmarkBar,
+                             false,
+                             PrefService::SYNCABLE_PREF);
+  prefs->RegisterBooleanPref(prefs::kEditBookmarksEnabled,
+                             true,
+                             PrefService::UNSYNCABLE_PREF);
 }
 
-void GetURLAndTitleToBookmark(TabContents* tab_contents,
+void GetURLAndTitleToBookmark(WebContents* web_contents,
                               GURL* url,
                               string16* title) {
-  *url = tab_contents->GetURL();
-  *title = tab_contents->GetTitle();
+  *url = web_contents->GetURL();
+  *title = web_contents->GetTitle();
 }
 
-void GetURLsForOpenTabs(Browser* browser,
+void GetURLAndTitleToBookmarkFromCurrentTab(Profile* profile,
+                                            GURL* url,
+                                            string16* title) {
+  Browser* browser = BrowserList::GetLastActiveWithProfile(profile);
+  WebContents* web_contents = browser ? browser->GetSelectedWebContents()
+                                        : NULL;
+  if (web_contents)
+    GetURLAndTitleToBookmark(web_contents, url, title);
+}
+
+void GetURLsForOpenTabs(
+    Browser* browser,
     std::vector<std::pair<GURL, string16> >* urls) {
   for (int i = 0; i < browser->tab_count(); ++i) {
     std::pair<GURL, string16> entry;
-    GetURLAndTitleToBookmark(browser->GetTabContentsAt(i), &(entry.first),
+    GetURLAndTitleToBookmark(browser->GetWebContentsAt(i), &(entry.first),
                              &(entry.second));
     urls->push_back(entry);
   }
@@ -641,14 +691,14 @@ const BookmarkNode* GetParentForNewNodes(
 
   if (index) {
     if (selection.size() == 1 && selection[0]->is_url()) {
-      *index = real_parent->IndexOfChild(selection[0]) + 1;
+      *index = real_parent->GetIndexOf(selection[0]) + 1;
       if (*index == 0) {
         // Node doesn't exist in parent, add to end.
         NOTREACHED();
-        *index = real_parent->GetChildCount();
+        *index = real_parent->child_count();
       }
     } else {
-      *index = real_parent->GetChildCount();
+      *index = real_parent->child_count();
     }
   }
 
@@ -661,11 +711,69 @@ bool NodeHasURLs(const BookmarkNode* node) {
   if (node->is_url())
     return true;
 
-  for (int i = 0; i < node->GetChildCount(); ++i) {
+  for (int i = 0; i < node->child_count(); ++i) {
     if (NodeHasURLs(node->GetChild(i)))
       return true;
   }
   return false;
 }
 
+bool ConfirmDeleteBookmarkNode(const BookmarkNode* node,
+                               gfx::NativeWindow window) {
+  DCHECK(node && node->is_folder() && !node->empty());
+  return browser::ShowYesNoBox(window,
+      l10n_util::GetStringUTF16(IDS_DELETE),
+      l10n_util::GetStringFUTF16Int(IDS_BOOKMARK_EDITOR_CONFIRM_DELETE,
+                                    ChildURLCountTotal(node)));
+}
+
+void DeleteBookmarkFolders(BookmarkModel* model,
+                           const std::vector<int64>& ids) {
+  // Remove the folders that were removed. This has to be done after all the
+  // other changes have been committed.
+  for (std::vector<int64>::const_iterator iter = ids.begin();
+       iter != ids.end();
+       ++iter) {
+    const BookmarkNode* node = model->GetNodeByID(*iter);
+    if (!node)
+      continue;
+    const BookmarkNode* parent = node->parent();
+    model->Remove(parent, parent->GetIndexOf(node));
+  }
+}
+
+void AddIfNotBookmarked(BookmarkModel* model,
+                        const GURL& url,
+                        const string16& title) {
+  std::vector<const BookmarkNode*> bookmarks;
+  model->GetNodesByURL(url, &bookmarks);
+  if (!bookmarks.empty())
+    return;  // Nothing to do, a bookmark with that url already exists.
+
+  const BookmarkNode* parent = model->GetParentForNewNodes();
+  model->AddURL(parent, parent->child_count(), title, url);
+}
+
+void RemoveAllBookmarks(BookmarkModel* model, const GURL& url) {
+  std::vector<const BookmarkNode*> bookmarks;
+  model->GetNodesByURL(url, &bookmarks);
+
+  // Remove all the bookmarks.
+  for (size_t i = 0; i < bookmarks.size(); ++i) {
+    const BookmarkNode* node = bookmarks[i];
+    int index = node->parent()->GetIndexOf(node);
+    if (index > -1)
+      model->Remove(node->parent(), index);
+  }
+}
+
+void RecordBookmarkLaunch(BookmarkLaunchLocation location) {
+#if defined(OS_WIN)
+  // TODO(estade): do this on other platforms too. For now it's compiled out
+  // so that stats from platforms for which this is incompletely implemented
+  // don't mix in with Windows, where it should be implemented exhaustively.
+  UMA_HISTOGRAM_ENUMERATION("Bookmarks.LaunchLocation", location, LAUNCH_LIMIT);
+#endif
+}
+
 }  // namespace bookmark_utils
