@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,20 +10,24 @@
 #include <vector>
 
 #include "base/string16.h"
-#include "chrome/browser/bookmarks/bookmark_node_data.h"
 #include "chrome/browser/bookmarks/bookmark_editor.h"
+#include "chrome/browser/bookmarks/bookmark_node_data.h"
 #include "chrome/browser/history/snippet.h"
-#include "gfx/native_widget_types.h"
+#include "ui/gfx/native_widget_types.h"
 #include "webkit/glue/window_open_disposition.h"
 
 class BookmarkModel;
 class BookmarkNode;
 class Browser;
-class PageNavigator;
 class PrefService;
 class Profile;
 class TabContents;
 
+namespace content {
+class PageNavigator;
+class WebContents;
+}
+
 namespace views {
 class DropTargetEvent;
 }
@@ -38,7 +42,7 @@ namespace bookmark_utils {
 int PreferredDropOperation(int source_operations, int operations);
 
 // Returns the drag operations for the specified node.
-int BookmarkDragOperation(const BookmarkNode* node);
+int BookmarkDragOperation(Profile* profile, const BookmarkNode* node);
 
 // Returns the preferred drop operation on a bookmark menu/bar.
 // |parent| is the parent node the drop is to occur on and |index| the index the
@@ -73,26 +77,27 @@ void CloneBookmarkNode(BookmarkModel* model,
                        const BookmarkNode* parent,
                        int index_to_add_at);
 
-// Begins dragging a group of bookmarks.
+// Begins dragging a folder of bookmarks.
 void DragBookmarks(Profile* profile,
                    const std::vector<const BookmarkNode*>& nodes,
                    gfx::NativeView view);
 
-// Recursively opens all bookmarks. |initial_disposition| dictates how the
-// first URL is opened, all subsequent URLs are opened as background tabs.
-// |navigator| is used to open the URLs. If |navigator| is NULL the last
-// tabbed browser with the profile |profile| is used. If there is no browser
-// with the specified profile a new one is created.
+// Opens all the bookmarks in |nodes| that are of type url and all the child
+// bookmarks that are of type url for folders in |nodes|. |initial_disposition|
+// dictates how the first URL is opened, all subsequent URLs are opened as
+// background tabs.  |navigator| is used to open the URLs. If |navigator| is
+// NULL the last tabbed browser with the profile |profile| is used. If there is
+// no browser with the specified profile a new one is created.
 void OpenAll(gfx::NativeWindow parent,
              Profile* profile,
-             PageNavigator* navigator,
+             content::PageNavigator* navigator,
              const std::vector<const BookmarkNode*>& nodes,
              WindowOpenDisposition initial_disposition);
 
-// Convenience for opening a single BookmarkNode.
+// Convenience for |OpenAll| with a single BookmarkNode.
 void OpenAll(gfx::NativeWindow parent,
              Profile* profile,
-             PageNavigator* navigator,
+             content::PageNavigator* navigator,
              const BookmarkNode* node,
              WindowOpenDisposition initial_disposition);
 
@@ -118,11 +123,11 @@ bool CanPasteFromClipboard(const BookmarkNode* node);
 string16 GetNameForURL(const GURL& url);
 
 // Returns a vector containing up to |max_count| of the most recently modified
-// groups. This never returns an empty vector.
-std::vector<const BookmarkNode*> GetMostRecentlyModifiedGroups(
+// folders. This never returns an empty vector.
+std::vector<const BookmarkNode*> GetMostRecentlyModifiedFolders(
     BookmarkModel* model, size_t max_count);
 
-// Returns the most recently added bookmarks. This does not return groups,
+// Returns the most recently added bookmarks. This does not return folders,
 // only nodes of type url.
 void GetMostRecentlyAddedEntries(BookmarkModel* model,
                                  size_t count,
@@ -131,6 +136,9 @@ void GetMostRecentlyAddedEntries(BookmarkModel* model,
 // Used by GetBookmarksMatchingText to return a matching node and the location
 // of the match in the title.
 struct TitleMatch {
+  TitleMatch();
+  ~TitleMatch();
+
   const BookmarkNode* node;
 
   // Location of the matching words in the title of the node.
@@ -159,7 +167,7 @@ bool DoesBookmarkContainText(const BookmarkNode* node,
 // done regarding moving from one folder to another).  If a new node is
 // explicitly being added, returns a pointer to the new node that was created.
 // Otherwise the return value is identically |node|.
-const BookmarkNode* ApplyEditsWithNoGroupChange(
+const BookmarkNode* ApplyEditsWithNoFolderChange(
     BookmarkModel* model,
     const BookmarkNode* parent,
     const BookmarkEditor::EditDetails& details,
@@ -170,7 +178,7 @@ const BookmarkNode* ApplyEditsWithNoGroupChange(
 // changed and the node will need to be removed and reinserted.  If a new node
 // is explicitly being added, returns a pointer to the new node that was
 // created.  Otherwise the return value is identically |node|.
-const BookmarkNode* ApplyEditsWithPossibleGroupChange(
+const BookmarkNode* ApplyEditsWithPossibleFolderChange(
     BookmarkModel* model,
     const BookmarkNode* new_parent,
     const BookmarkEditor::EditDetails& details,
@@ -184,11 +192,17 @@ void ToggleWhenVisible(Profile* profile);
 // Register user preferences for BookmarksBar.
 void RegisterUserPrefs(PrefService* prefs);
 
-// Fills in the URL and title for a bookmark of |tab_contents|.
-void GetURLAndTitleToBookmark(TabContents* tab_contents,
+// Fills in the URL and title for a bookmark of |web_contents|.
+void GetURLAndTitleToBookmark(content::WebContents* web_contents,
                               GURL* url,
                               string16* title);
 
+// Fills in the URL and title for a bookmark from the current tab of the
+// specified profile.
+void GetURLAndTitleToBookmarkFromCurrentTab(Profile* profile,
+                                            GURL* url,
+                                            string16* title);
+
 // Returns, by reference in |urls|, the url and title pairs for each open
 // tab in browser.
 void GetURLsForOpenTabs(Browser* browser,
@@ -207,6 +221,21 @@ const BookmarkNode* GetParentForNewNodes(
 // of type URL.
 bool NodeHasURLs(const BookmarkNode* node);
 
+// Ask the user before deleting a non-empty bookmark folder.
+bool ConfirmDeleteBookmarkNode(const BookmarkNode* node,
+                               gfx::NativeWindow window);
+
+// Deletes the bookmark folders for the given list of |ids|.
+void DeleteBookmarkFolders(BookmarkModel* model, const std::vector<int64>& ids);
+
+// If there are no bookmarks for url, a bookmark is created.
+void AddIfNotBookmarked(BookmarkModel* model,
+                        const GURL& url,
+                        const string16& title);
+
+// Removes all bookmarks for the given |url|.
+void RemoveAllBookmarks(BookmarkModel* model, const GURL& url);
+
 // Number of bookmarks we'll open before prompting the user to see if they
 // really want to open all.
 //
@@ -214,6 +243,31 @@ bool NodeHasURLs(const BookmarkNode* node);
 // value.
 extern int num_urls_before_prompting;
 
+// This enum is used for the Bookmarks.LaunchLocation histogram.
+enum BookmarkLaunchLocation {
+  LAUNCH_NONE,
+  LAUNCH_ATTACHED_BAR = 0,
+  LAUNCH_DETACHED_BAR,
+  // These two are kind of sub-categories of the bookmark bar. Generally
+  // a launch from a context menu or subfolder could be classified in one of
+  // the other two bar buckets, but doing so is difficult because the menus
+  // don't know of their greater place in Chrome.
+  LAUNCH_BAR_SUBFOLDER,
+  LAUNCH_CONTEXT_MENU,
+
+  // Bookmarks menu within wrench menu.
+  LAUNCH_WRENCH_MENU,
+  // Bookmark manager.
+  LAUNCH_MANAGER,
+  // Autocomplete suggestion.
+  LAUNCH_OMNIBOX,
+
+  LAUNCH_LIMIT  // Keep this last.
+};
+
+// Records the launch of a bookmark for UMA purposes.
+void RecordBookmarkLaunch(BookmarkLaunchLocation location);
+
 }  // namespace bookmark_utils
 
 #endif  // CHROME_BROWSER_BOOKMARKS_BOOKMARK_UTILS_H_
