@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,20 +8,21 @@
 
 #include <vector>
 
-#include "app/combobox_model.h"
 #include "base/string16.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
+#include "ui/base/models/combobox_model.h"
 
 // Model for the combobox showing the list of folders to choose from. The
 // list always contains the bookmark bar, other node and parent. The list
 // also contains an extra item that shows the text 'Choose another folder...'.
-class RecentlyUsedFoldersComboModel : public ComboboxModel {
+class RecentlyUsedFoldersComboModel : public ui::ComboboxModel {
  public:
   RecentlyUsedFoldersComboModel(BookmarkModel* model, const BookmarkNode* node);
+  virtual ~RecentlyUsedFoldersComboModel();
 
-  // Overridden from ComboboxModel:
-  virtual int GetItemCount();
-  virtual string16 GetItemAt(int index);
+  // Overridden from ui::ComboboxModel:
+  virtual int GetItemCount() OVERRIDE;
+  virtual string16 GetItemAt(int index) OVERRIDE;
 
   // Returns the node at the specified |index|.
   const BookmarkNode* GetNodeAt(int index);
