@@ -1,17 +1,17 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 #include <string>
 
 #include "base/file_util.h"
 #include "base/scoped_temp_dir.h"
-#include "chrome/browser/net/url_request_mock_http_job.h"
-#include "chrome/browser/download/save_package.h"
 #include "chrome/common/pref_names.h"
+#include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "chrome/test/ui/ui_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "content/browser/net/url_request_mock_http_job.h"
 
 static const FilePath::CharType* kTestDir = FILE_PATH_LITERAL("encoding_tests");
 
@@ -52,6 +52,10 @@ class BrowserEncodingTest : public UITest {
 // 2. Add more files with multiple encoding name variants for each canonical
 // encoding name). Webkit layout tests cover some, but testing in the UI test is
 // also necessary.
+
+#if defined(OS_MACOSX)
+#define TestEncodingAliasMapping FLAKY_TestEncodingAliasMapping
+#endif
 TEST_F(BrowserEncodingTest, TestEncodingAliasMapping) {
   struct EncodingTestData {
     const char* file_name;
@@ -79,7 +83,8 @@ TEST_F(BrowserEncodingTest, TestEncodingAliasMapping) {
     { "UTF-8.html", "UTF-8" },
     { "UTF-16LE.html", "UTF-16LE" },
     { "windows-874.html", "windows-874" },
-    { "windows-949.html", "windows-949" },
+    // http://crbug.com/95963
+    // { "windows-949.html", "windows-949" },
     { "windows-1250.html", "windows-1250" },
     { "windows-1251.html", "windows-1251" },
     { "windows-1252.html", "windows-1252" },
@@ -105,7 +110,7 @@ TEST_F(BrowserEncodingTest, TestEncodingAliasMapping) {
 
     std::string encoding;
     EXPECT_TRUE(tab_proxy->GetPageCurrentEncoding(&encoding));
-    EXPECT_EQ(encoding, kEncodingTestDatas[i].encoding_name);
+    EXPECT_EQ(kEncodingTestDatas[i].encoding_name, encoding);
   }
 }
 
@@ -146,7 +151,7 @@ TEST_F(BrowserEncodingTest, FLAKY_TestOverrideEncoding) {
   // name to save sub resources in it. Although this test file does not have
   // sub resources, but the directory name is still required.
   EXPECT_TRUE(tab_proxy->SavePage(full_file_name, temp_sub_resource_dir_,
-                                  SavePackage::SAVE_AS_COMPLETE_HTML));
+                                  content::SAVE_PAGE_TYPE_AS_COMPLETE_HTML));
   scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(browser.get());
   EXPECT_TRUE(WaitForDownloadShelfVisible(browser.get()));
@@ -243,7 +248,7 @@ TEST_F(BrowserEncodingTest, MAYBE_TestEncodingAutoDetect) {
   // Set the default charset to one of encodings not supported by the current
   // auto-detector (Please refer to the above comments) to make sure we
   // incorrectly decode the page. Now we use ISO-8859-4.
-  ASSERT_TRUE(browser->SetStringPreference(prefs::kDefaultCharset,
+  ASSERT_TRUE(browser->SetStringPreference(prefs::kGlobalDefaultCharset,
                                            "ISO-8859-4"));
   scoped_refptr<TabProxy> tab(GetActiveTab());
   ASSERT_TRUE(tab.get());
@@ -291,7 +296,7 @@ TEST_F(BrowserEncodingTest, MAYBE_TestEncodingAutoDetect) {
     expected_result_file_name = expected_result_file_name.AppendASCII(
         kTestDatas[i].expected_result);
     EXPECT_TRUE(tab->SavePage(full_saved_file_name, temp_sub_resource_dir_,
-                              SavePackage::SAVE_AS_COMPLETE_HTML));
+                              content::SAVE_PAGE_TYPE_AS_COMPLETE_HTML));
     EXPECT_TRUE(WaitForDownloadShelfVisible(browser.get()));
     CheckFile(full_saved_file_name, expected_result_file_name, true);
   }
