@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,23 +7,26 @@
 #include "base/basictypes.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
-#include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/dom_operation_notification_details.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/renderer_host/render_widget_host_view.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/tab_contents/tab_contents_view.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_paths.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_service.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/renderer_host/render_widget_host_view.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/browser/web_contents_view.h"
 #include "net/test/test_server.h"
 #include "ui/base/keycodes/keyboard_codes.h"
 
+using content::NavigationController;
+
 namespace {
 
 const char kTestingPage[] = "files/keyevents_test.html";
@@ -81,12 +84,12 @@ const wchar_t* GetBoolString(bool value) {
 }
 
 // A class to help wait for the finish of a key event test.
-class TestFinishObserver : public NotificationObserver {
+class TestFinishObserver : public content::NotificationObserver {
  public:
   explicit TestFinishObserver(RenderViewHost* render_view_host)
       : finished_(false), waiting_(false) {
-    registrar_.Add(this, NotificationType::DOM_OPERATION_RESPONSE,
-                   Source<RenderViewHost>(render_view_host));
+    registrar_.Add(this, chrome::NOTIFICATION_DOM_OPERATION_RESPONSE,
+                   content::Source<RenderViewHost>(render_view_host));
   }
 
   bool WaitForFinish() {
@@ -98,11 +101,11 @@ class TestFinishObserver : public NotificationObserver {
     return finished_;
   }
 
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details) {
-    DCHECK(type == NotificationType::DOM_OPERATION_RESPONSE);
-    Details<DomOperationNotificationDetails> dom_op_details(details);
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) {
+    DCHECK(type == chrome::NOTIFICATION_DOM_OPERATION_RESPONSE);
+    content::Details<DomOperationNotificationDetails> dom_op_details(details);
     // We might receive responses for other script execution, but we only
     // care about the test finished message.
     if (dom_op_details->json() == "\"FINISHED\"") {
@@ -115,7 +118,7 @@ class TestFinishObserver : public NotificationObserver {
  private:
   bool finished_;
   bool waiting_;
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(TestFinishObserver);
 };
@@ -142,9 +145,9 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     ASSERT_LT(tab_index, browser()->tab_count());
     bool actual;
     ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-        browser()->GetTabContentsAt(tab_index)->render_view_host(),
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
         L"",
-        StringPrintf(kSuppressEventJS, type, GetBoolString(!suppress)),
+        base::StringPrintf(kSuppressEventJS, type, GetBoolString(!suppress)),
         &actual));
     ASSERT_EQ(!suppress, actual);
   }
@@ -168,7 +171,7 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
   void GetResultLength(int tab_index, int* length) {
     ASSERT_LT(tab_index, browser()->tab_count());
     ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractInt(
-        browser()->GetTabContentsAt(tab_index)->render_view_host(),
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
         L"", kGetResultLengthJS, length));
   }
 
@@ -180,8 +183,8 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     for (int i = 0; i < actual_length; ++i) {
       std::string actual;
       ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractString(
-          browser()->GetTabContentsAt(tab_index)->render_view_host(),
-          L"", StringPrintf(kGetResultJS, i), &actual));
+          browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
+          L"", base::StringPrintf(kGetResultJS, i), &actual));
 
       // If more events were received than expected, then the additional events
       // must be keyup events.
@@ -196,7 +199,7 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     ASSERT_LT(tab_index, browser()->tab_count());
     std::string actual;
     ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractString(
-        browser()->GetTabContentsAt(tab_index)->render_view_host(),
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
         L"", kGetFocusedElementJS, &actual));
     ASSERT_EQ(WideToUTF8(focused), actual);
   }
@@ -205,9 +208,9 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     ASSERT_LT(tab_index, browser()->tab_count());
     bool actual;
     ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-        browser()->GetTabContentsAt(tab_index)->render_view_host(),
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
         L"",
-        StringPrintf(kSetFocusedElementJS, focused),
+        base::StringPrintf(kSetFocusedElementJS, focused),
         &actual));
     ASSERT_TRUE(actual);
   }
@@ -217,9 +220,9 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     ASSERT_LT(tab_index, browser()->tab_count());
     std::string actual;
     ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractString(
-        browser()->GetTabContentsAt(tab_index)->render_view_host(),
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
         L"",
-        StringPrintf(kGetTextBoxValueJS, id),
+        base::StringPrintf(kGetTextBoxValueJS, id),
         &actual));
     ASSERT_EQ(WideToUTF8(value), actual);
   }
@@ -229,9 +232,9 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     ASSERT_LT(tab_index, browser()->tab_count());
     std::string actual;
     ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractString(
-        browser()->GetTabContentsAt(tab_index)->render_view_host(),
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
         L"",
-        StringPrintf(kSetTextBoxValueJS, id, value),
+        base::StringPrintf(kSetTextBoxValueJS, id, value),
         &actual));
     ASSERT_EQ(WideToUTF8(value), actual);
   }
@@ -240,14 +243,14 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     ASSERT_LT(tab_index, browser()->tab_count());
     bool actual;
     ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-        browser()->GetTabContentsAt(tab_index)->render_view_host(),
-        L"", StringPrintf(kStartTestJS, result_length), &actual));
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost(),
+        L"", base::StringPrintf(kStartTestJS, result_length), &actual));
     ASSERT_TRUE(actual);
   }
 
   void TestKeyEvent(int tab_index, const KeyEventTestData& test) {
     ASSERT_LT(tab_index, browser()->tab_count());
-    ASSERT_EQ(tab_index, browser()->selected_index());
+    ASSERT_EQ(tab_index, browser()->active_index());
 
     // Inform our testing web page that we are about to start testing a key
     // event.
@@ -260,7 +263,7 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
     // because the test finished message might be arrived before returning
     // from the SendKeyPressSync() method.
     TestFinishObserver finish_observer(
-        browser()->GetTabContentsAt(tab_index)->render_view_host());
+        browser()->GetWebContentsAt(tab_index)->GetRenderViewHost());
 
     ASSERT_TRUE(ui_test_utils::SendKeyPressSync(
         browser(), test.key, test.ctrl, test.shift, test.alt, test.command));
@@ -270,7 +273,7 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
   }
 
   std::string GetTestDataDescription(const KeyEventTestData& data) {
-    std::string desc = StringPrintf(
+    std::string desc = base::StringPrintf(
         " VKEY:0x%02x, ctrl:%d, shift:%d, alt:%d, command:%d\n"
         " Suppress: keydown:%d, keypress:%d, keyup:%d, textInput:%d\n"
         " Expected results(%d):\n",
@@ -286,7 +289,13 @@ class BrowserKeyEventsTest : public InProcessBrowserTest {
   }
 };
 
+#if defined(OS_MACOSX)
+// http://crbug.com/81451
+IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, DISABLED_NormalKeyEvents) {
+#else
 IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, NormalKeyEvents) {
+#endif
+
   static const KeyEventTestData kTestNoInput[] = {
     // a
     { ui::VKEY_A, false, false, false, false,
@@ -355,7 +364,7 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, NormalKeyEvents) {
   ASSERT_NO_FATAL_FAILURE(ClickOnView(VIEW_ID_TAB_CONTAINER));
   ASSERT_TRUE(IsViewFocused(VIEW_ID_TAB_CONTAINER_FOCUS_VIEW));
 
-  int tab_index = browser()->selected_index();
+  int tab_index = browser()->active_index();
   for (size_t i = 0; i < arraysize(kTestNoInput); ++i) {
     EXPECT_NO_FATAL_FAILURE(TestKeyEvent(tab_index, kTestNoInput[i]))
         << "kTestNoInput[" << i << "] failed:\n"
@@ -440,7 +449,7 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, CtrlKeyEvents) {
   ASSERT_NO_FATAL_FAILURE(ClickOnView(VIEW_ID_TAB_CONTAINER));
   ASSERT_TRUE(IsViewFocused(VIEW_ID_TAB_CONTAINER_FOCUS_VIEW));
 
-  int tab_index = browser()->selected_index();
+  int tab_index = browser()->active_index();
   // Press Ctrl+F, which will make the Find box open and request focus.
   EXPECT_NO_FATAL_FAILURE(TestKeyEvent(tab_index, kTestCtrlF));
   EXPECT_TRUE(IsViewFocused(VIEW_ID_FIND_IN_PAGE_TEXT_FIELD));
@@ -459,7 +468,8 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, CtrlKeyEvents) {
   EXPECT_NO_FATAL_FAILURE(TestKeyEvent(tab_index, kTestCtrlEnter));
 }
 #elif defined(OS_MACOSX)
-IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, CommandKeyEvents) {
+// http://crbug.com/81451
+IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, FLAKY_CommandKeyEvents) {
   static const KeyEventTestData kTestCmdF = {
     ui::VKEY_F, false, false, false, true,
     false, false, false, false, 2,
@@ -485,7 +495,7 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, CommandKeyEvents) {
   ASSERT_NO_FATAL_FAILURE(ClickOnView(VIEW_ID_TAB_CONTAINER));
   ASSERT_TRUE(IsViewFocused(VIEW_ID_TAB_CONTAINER_FOCUS_VIEW));
 
-  int tab_index = browser()->selected_index();
+  int tab_index = browser()->active_index();
   // Press Cmd+F, which will make the Find box open and request focus.
   EXPECT_NO_FATAL_FAILURE(TestKeyEvent(tab_index, kTestCmdF));
   EXPECT_TRUE(IsViewFocused(VIEW_ID_FIND_IN_PAGE_TEXT_FIELD));
@@ -501,7 +511,12 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, CommandKeyEvents) {
 }
 #endif
 
+#if defined(OS_MACOSX)
+// http://crbug.com/81451 for mac
+IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, DISABLED_AccessKeys) {
+#else
 IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, AccessKeys) {
+#endif
 #if defined(OS_MACOSX)
   // On Mac, access keys use ctrl+alt modifiers.
   static const KeyEventTestData kTestAccessA = {
@@ -582,7 +597,7 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, AccessKeys) {
   ASSERT_NO_FATAL_FAILURE(ClickOnView(VIEW_ID_TAB_CONTAINER));
   ASSERT_TRUE(IsViewFocused(VIEW_ID_TAB_CONTAINER_FOCUS_VIEW));
 
-  int tab_index = browser()->selected_index();
+  int tab_index = browser()->active_index();
   // Make sure no element is focused.
   EXPECT_NO_FATAL_FAILURE(CheckFocusedElement(tab_index, L""));
   // Alt+A should focus the element with accesskey = "A".
@@ -601,8 +616,11 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, AccessKeys) {
   // TODO(isherman): This is an experimental change to help diagnose
   // http://crbug.com/55713
   ui_test_utils::RunAllPendingInMessageLoop();
-
+#if defined(USE_AURA)
+  EXPECT_TRUE(IsViewFocused(VIEW_ID_OMNIBOX));
+#else
   EXPECT_TRUE(IsViewFocused(VIEW_ID_LOCATION_BAR));
+#endif
   // No element should be focused, as Alt+D was handled by the browser.
   EXPECT_NO_FATAL_FAILURE(CheckFocusedElement(tab_index, L""));
 
@@ -636,8 +654,13 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, AccessKeys) {
 #endif
 }
 
-// Disabled, http://crbug.com/69475.
-IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, DISABLED_ReservedAccelerators) {
+// Flaky, http://crbug.com/69475.
+#if defined(OS_LINUX)
+#define MAYBE_ReservedAccelerators FLAKY_ReservedAccelerators
+#else
+#define MAYBE_ReservedAccelerators ReservedAccelerators
+#endif
+IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, MAYBE_ReservedAccelerators) {
   ASSERT_TRUE(test_server()->Start());
 
   ASSERT_TRUE(ui_test_utils::BringBrowserWindowToFront(browser()));
@@ -662,22 +685,19 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, DISABLED_ReservedAccelerators) {
   };
 
   ui_test_utils::WindowedNotificationObserver wait_for_new_tab(
-      NotificationType::TAB_PARENTED,
-      NotificationService::AllSources());
+      content::NOTIFICATION_TAB_PARENTED,
+      content::NotificationService::AllSources());
 
   // Press Ctrl/Cmd+T, which will open a new tab. It cannot be suppressed.
   EXPECT_NO_FATAL_FAILURE(TestKeyEvent(0, kTestCtrlOrCmdT));
-
-  ASSERT_NO_FATAL_FAILURE(
-      wait_for_new_tab.WaitFor(Source<NavigationController>(
-      &browser()->GetTabContentsAt(1)->controller())));
+  wait_for_new_tab.Wait();
 
   int result_length;
   ASSERT_NO_FATAL_FAILURE(GetResultLength(0, &result_length));
   EXPECT_EQ(1, result_length);
 
   EXPECT_EQ(2, browser()->tab_count());
-  ASSERT_EQ(1, browser()->selected_index());
+  ASSERT_EQ(1, browser()->active_index());
 
   // Because of issue http://crbug.com/65375, switching back to the first tab
   // may cause the focus to be grabbed by omnibox. So instead, we load our
@@ -692,8 +712,9 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, DISABLED_ReservedAccelerators) {
   ASSERT_NO_FATAL_FAILURE(SuppressAllEvents(1, true));
 
   ui_test_utils::WindowedNotificationObserver wait_for_tab_closed(
-      NotificationType::TAB_CLOSED, Source<NavigationController>(
-          &browser()->GetTabContentsAt(1)->controller()));
+      content::NOTIFICATION_TAB_CLOSED,
+      content::Source<NavigationController>(
+          &browser()->GetWebContentsAt(1)->GetController()));
 
   // Press Ctrl/Cmd+W, which will close the tab.
 #if defined(OS_MACOSX)
@@ -747,7 +768,7 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, EditorKeyBindings) {
   ASSERT_NO_FATAL_FAILURE(ClickOnView(VIEW_ID_TAB_CONTAINER));
   ASSERT_TRUE(IsViewFocused(VIEW_ID_TAB_CONTAINER_FOCUS_VIEW));
 
-  int tab_index = browser()->selected_index();
+  int tab_index = browser()->active_index();
   ASSERT_NO_FATAL_FAILURE(SetFocusedElement(tab_index, L"A"));
   ASSERT_NO_FATAL_FAILURE(SetTextBoxValue(tab_index, L"A", L"Hello"));
   // Move the caret to the beginning of the line.
@@ -784,7 +805,7 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, PageUpDownKeys) {
   ASSERT_NO_FATAL_FAILURE(ClickOnView(VIEW_ID_TAB_CONTAINER));
   ASSERT_TRUE(IsViewFocused(VIEW_ID_TAB_CONTAINER_FOCUS_VIEW));
 
-  int tab_index = browser()->selected_index();
+  int tab_index = browser()->active_index();
   ASSERT_NO_FATAL_FAILURE(SetFocusedElement(tab_index, L"A"));
   EXPECT_NO_FATAL_FAILURE(TestKeyEvent(tab_index, kTestPageUp));
   EXPECT_NO_FATAL_FAILURE(TestKeyEvent(tab_index, kTestPageDown));
@@ -825,7 +846,7 @@ IN_PROC_BROWSER_TEST_F(BrowserKeyEventsTest, FocusMenuBarByAltKey) {
   ASSERT_NO_FATAL_FAILURE(ClickOnView(VIEW_ID_TAB_CONTAINER));
   ASSERT_TRUE(IsViewFocused(VIEW_ID_TAB_CONTAINER_FOCUS_VIEW));
 
-  int tab_index = browser()->selected_index();
+  int tab_index = browser()->active_index();
   // Press and release Alt key to focus wrench menu button.
   EXPECT_NO_FATAL_FAILURE(TestKeyEvent(tab_index, kTestAltKey));
   EXPECT_TRUE(IsViewFocused(VIEW_ID_APP_MENU));
