@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,42 +15,61 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/memory/ref_counted.h"
 #include "ipc/ipc_message.h"
 
+class AudioManager;
 class AutomationProviderList;
-
-namespace safe_browsing {
-class ClientSideDetectionService;
-}
-
-class DevToolsManager;
+class BackgroundModeManager;
+class ChromeNetLog;
+class CRLSetFetcher;
+class ComponentUpdateService;
 class DownloadRequestLimiter;
 class DownloadStatusUpdater;
+class ExtensionEventRouterForwarder;
 class GoogleURLTracker;
 class IconManager;
 class IntranetRedirectDetector;
 class IOThread;
 class MetricsService;
+class MHTMLGenerationManager;
 class NotificationUIManager;
 class PrefService;
+class Profile;
 class ProfileManager;
 class ResourceDispatcherHost;
-class SidebarManager;
+class SafeBrowsingService;
+class StatusTray;
 class TabCloseableStateWatcher;
 class ThumbnailGenerator;
+class WatchDogThread;
 
-namespace base {
-class Thread;
-class WaitableEvent;
+#if defined(OS_CHROMEOS)
+namespace browser {
+class OomPriorityManager;
+}
+#endif  // defined(OS_CHROMEOS)
+
+namespace net {
+class URLRequestContextGetter;
+}
+
+namespace prerender {
+class PrerenderTracker;
 }
 
 namespace printing {
+class BackgroundPrintingManager;
 class PrintJobManager;
 class PrintPreviewTabController;
 }
 
 namespace policy {
-class ConfigurationPolicyProviderKeeper;
+class BrowserPolicyConnector;
+}
+
+namespace safe_browsing {
+class ClientSideDetectionService;
 }
 
 namespace ui {
@@ -64,6 +83,9 @@ class BrowserProcess {
   BrowserProcess();
   virtual ~BrowserProcess();
 
+  // Called when the ResourceDispatcherHost object is created by content.
+  virtual void ResourceDispatcherHostCreated() = 0;
+
   // Invoked when the user is logging out/shutting down. When logging off we may
   // not have enough time to do a normal shutdown. This method is invoked prior
   // to normal shutdown and saves any state that must be saved before we are
@@ -71,59 +93,49 @@ class BrowserProcess {
   virtual void EndSession() = 0;
 
   // Services: any of these getters may return NULL
-  virtual ResourceDispatcherHost* resource_dispatcher_host() = 0;
-
   virtual MetricsService* metrics_service() = 0;
   virtual ProfileManager* profile_manager() = 0;
   virtual PrefService* local_state() = 0;
-  virtual DevToolsManager* devtools_manager() = 0;
-  virtual SidebarManager* sidebar_manager() = 0;
   virtual ui::Clipboard* clipboard() = 0;
+  virtual net::URLRequestContextGetter* system_request_context() = 0;
+
+#if defined(OS_CHROMEOS)
+  // Returns the out-of-memory priority manager.
+  virtual browser::OomPriorityManager* oom_priority_manager() = 0;
+#endif  // defined(OS_CHROMEOS)
+
+  virtual ExtensionEventRouterForwarder*
+      extension_event_router_forwarder() = 0;
 
   // Returns the manager for desktop notifications.
   virtual NotificationUIManager* notification_ui_manager() = 0;
 
-  // Returns the thread that we perform I/O coordination on (network requests,
-  // communication with renderers, etc.
-  // NOTE: You should ONLY use this to pass to IPC or other objects which must
-  // need a MessageLoop*.  If you just want to post a task, use
-  // BrowserThread::PostTask (or other variants) as they take care of checking
-  // that a thread is still alive, race conditions, lifetime differences etc.
-  // If you still must use this check the return value for NULL.
+  // Returns the state object for the thread that we perform I/O
+  // coordination on (network requests, communication with renderers,
+  // etc.
+  //
+  // Can be NULL close to startup and shutdown.
+  //
+  // NOTE: If you want to post a task to the IO thread, use
+  // BrowserThread::PostTask (or other variants).
   virtual IOThread* io_thread() = 0;
 
-  // Returns the thread that we perform random file operations on. For code
-  // that wants to do I/O operations (not network requests or even file: URL
-  // requests), this is the thread to use to avoid blocking the UI thread.
-  // It might be nicer to have a thread pool for this kind of thing.
-  virtual base::Thread* file_thread() = 0;
+  // Returns the thread that is used for health check of all browser threads.
+  virtual WatchDogThread* watchdog_thread() = 0;
 
-  // Returns the thread that is used for database operations such as the web
-  // database. History has its own thread since it has much higher traffic.
-  virtual base::Thread* db_thread() = 0;
-
-  // Returns the thread that is used for background cache operations.
-  virtual base::Thread* cache_thread() = 0;
-
-#if defined(USE_X11)
-  // Returns the thread that is used to process UI requests in cases where
-  // we can't route the request to the UI thread. Note that this thread
-  // should only be used by the IO thread and this method is only safe to call
-  // from the UI thread so, if you've ended up here, something has gone wrong.
-  // This method is only included for uniformity.
-  virtual base::Thread* background_x11_thread() = 0;
-#endif
-
-  virtual policy::ConfigurationPolicyProviderKeeper*
-      configuration_policy_provider_keeper() = 0;
+  virtual policy::BrowserPolicyConnector* browser_policy_connector() = 0;
 
   virtual IconManager* icon_manager() = 0;
 
   virtual ThumbnailGenerator* GetThumbnailGenerator() = 0;
 
-  virtual AutomationProviderList* InitAutomationProviderList() = 0;
+  virtual AutomationProviderList* GetAutomationProviderList() = 0;
 
-  virtual void InitDebuggerWrapper(int port, bool useHttp) = 0;
+  virtual void InitDevToolsHttpProtocolHandler(
+      Profile* profile,
+      const std::string& ip,
+      int port,
+      const std::string& frontend_url) = 0;
 
   virtual unsigned int AddRefModule() = 0;
   virtual unsigned int ReleaseModule() = 0;
@@ -133,6 +145,8 @@ class BrowserProcess {
   virtual printing::PrintJobManager* print_job_manager() = 0;
   virtual printing::PrintPreviewTabController*
       print_preview_tab_controller() = 0;
+  virtual printing::BackgroundPrintingManager*
+      background_printing_manager() = 0;
 
   virtual GoogleURLTracker* google_url_tracker() = 0;
   virtual IntranetRedirectDetector* intranet_redirect_detector() = 0;
@@ -141,28 +155,31 @@ class BrowserProcess {
   virtual const std::string& GetApplicationLocale() = 0;
   virtual void SetApplicationLocale(const std::string& locale) = 0;
 
-  DownloadRequestLimiter* download_request_limiter();
   virtual DownloadStatusUpdater* download_status_updater() = 0;
-
-  // Returns an event that is signaled when the browser shutdown.
-  virtual base::WaitableEvent* shutdown_event() = 0;
-
-  // Returns a reference to the user-data-dir based profiles vector.
-  std::vector<std::wstring>& user_data_dir_profiles() {
-    return user_data_dir_profiles_;
-  }
+  virtual DownloadRequestLimiter* download_request_limiter() = 0;
 
   // Returns the object that watches for changes in the closeable state of tab.
   virtual TabCloseableStateWatcher* tab_closeable_state_watcher() = 0;
 
+  // Returns the object that manages background applications.
+  virtual BackgroundModeManager* background_mode_manager() = 0;
+
+  // Returns the StatusTray, which provides an API for displaying status icons
+  // in the system status tray. Returns NULL if status icons are not supported
+  // on this platform (or this is a unit test).
+  virtual StatusTray* status_tray() = 0;
+
+  // Returns the SafeBrowsing service.
+  virtual SafeBrowsingService* safe_browsing_service() = 0;
+
   // Returns an object which handles communication with the SafeBrowsing
   // client-side detection servers.
   virtual safe_browsing::ClientSideDetectionService*
       safe_browsing_detection_service() = 0;
 
-  // Trigger an asynchronous check to see if we have the inspector's files on
-  // disk.
-  virtual void CheckForInspectorFiles() = 0;
+  // Returns the state of the disable plugin finder policy. Callable only on
+  // the IO thread.
+  virtual bool plugin_finder_disabled() const = 0;
 
 #if (defined(OS_WIN) || defined(OS_LINUX)) && !defined(OS_CHROMEOS)
   // This will start a timer that, if Chrome is in persistent mode, will check
@@ -175,33 +192,19 @@ class BrowserProcess {
   virtual void StartAutoupdateTimer() = 0;
 #endif
 
-  // Return true iff we found the inspector files on disk. It's possible to
-  // call this function before we have a definite answer from the disk. In that
-  // case, we default to returning true.
-  virtual bool have_inspector_files() const = 0;
+  virtual ChromeNetLog* net_log() = 0;
 
-#if defined(IPC_MESSAGE_LOG_ENABLED)
-  // Enable or disable IPC logging for the browser, all processes
-  // derived from ChildProcess (plugin etc), and all
-  // renderers.
-  virtual void SetIPCLoggingEnabled(bool enable) = 0;
-#endif
+  virtual prerender::PrerenderTracker* prerender_tracker() = 0;
 
-  const std::string& plugin_data_remover_mime_type() const {
-    return plugin_data_remover_mime_type_;
-  }
+  virtual MHTMLGenerationManager* mhtml_generation_manager() = 0;
 
-  void set_plugin_data_remover_mime_type(const std::string& mime_type) {
-    plugin_data_remover_mime_type_ = mime_type;
-  }
+  virtual ComponentUpdateService* component_updater() = 0;
+
+  virtual CRLSetFetcher* crl_set_fetcher() = 0;
+
+  virtual AudioManager* audio_manager() = 0;
 
  private:
-  // User-data-dir based profiles.
-  std::vector<std::wstring> user_data_dir_profiles_;
-
-  // Used for testing plugin data removal at shutdown.
-  std::string plugin_data_remover_mime_type_;
-
   DISALLOW_COPY_AND_ASSIGN(BrowserProcess);
 };
 
