@@ -1,11 +1,13 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/browser_shutdown.h"
 
+#include <map>
 #include <string>
 
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
@@ -13,53 +15,56 @@
 #include "base/path_service.h"
 #include "base/process_util.h"
 #include "base/string_number_conversions.h"
-#include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/time.h"
 #include "build/build_config.h"
 #include "chrome/browser/about_flags.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/dom_ui/chrome_url_data_manager.h"
-#include "chrome/browser/first_run/first_run.h"
+#include "chrome/browser/first_run/upgrade_util.h"
 #include "chrome/browser/jankometer.h"
 #include "chrome/browser/metrics/metrics_service.h"
-#include "chrome/browser/plugin_process_host.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile_manager.h"
-#include "chrome/browser/renderer_host/render_process_host.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/renderer_host/render_widget_host.h"
-#include "chrome/browser/service/service_process_control_manager.h"
+#include "chrome/browser/service/service_process_control.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/webui/chrome_url_data_manager.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/common/chrome_plugin_lib.h"
 #include "chrome/common/switch_utils.h"
-#include "net/predictor_api.h"
+#include "content/browser/plugin_process_host.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/renderer_host/render_widget_host.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/render_process_host.h"
 #include "ui/base/resource/resource_bundle.h"
 
 #if defined(OS_WIN)
+#include "chrome/browser/browser_util_win.h"
+#include "chrome/browser/first_run/upgrade_util_win.h"
 #include "chrome/browser/rlz/rlz.h"
 #endif
 
 #if defined(OS_CHROMEOS)
 #include "chrome/browser/chromeos/boot_times_loader.h"
 #include "chrome/browser/chromeos/cros/cros_library.h"
-#include "chrome/browser/chromeos/cros/login_library.h"
-#include "chrome/browser/chromeos/system_key_event_listener.h"
 #endif
 
 using base::Time;
 using base::TimeDelta;
+using content::BrowserThread;
 
 namespace browser_shutdown {
 
 // Whether the browser is trying to quit (e.g., Quit chosen from menu).
 bool g_trying_to_quit = false;
 
-Time shutdown_started_;
+// Whether the browser should quit without closing browsers.
+bool g_shutting_down_without_closing_browsers = false;
+
+Time* shutdown_started_ = NULL;
 ShutdownType shutdown_type_ = NOT_VALID;
 int shutdown_num_processes_;
 int shutdown_num_processes_slow_;
@@ -87,14 +92,16 @@ void OnShutdownStarting(ShutdownType type) {
   // since we can't safely count the number of plugin processes from this
   // thread, and we'd really like to avoid anything which might add further
   // delays to shutdown time.
-  shutdown_started_ = Time::Now();
+  DCHECK(!shutdown_started_);
+  shutdown_started_ = new Time(Time::Now());
 
   // Call FastShutdown on all of the RenderProcessHosts.  This will be
   // a no-op in some cases, so we still need to go through the normal
   // shutdown path for the ones that didn't exit here.
   shutdown_num_processes_ = 0;
   shutdown_num_processes_slow_ = 0;
-  for (RenderProcessHost::iterator i(RenderProcessHost::AllHostsIterator());
+  for (content::RenderProcessHost::iterator i(
+          content::RenderProcessHost::AllHostsIterator());
        !i.IsAtEnd(); i.Advance()) {
     ++shutdown_num_processes_;
     if (!i.GetCurrentValue()->FastShutdownIfPossible())
@@ -108,7 +115,7 @@ FilePath GetShutdownMsPath() {
   return shutdown_ms_file.AppendASCII(kShutdownMsFile);
 }
 
-void Shutdown() {
+bool ShutdownPreThreadsStop() {
 #if defined(OS_CHROMEOS)
   chromeos::BootTimesLoader::Get()->AddLogoutTimeMarker(
       "BrowserShutdownStarted", false);
@@ -119,39 +126,17 @@ void Shutdown() {
   // shutdown.
   base::ThreadRestrictions::SetIOAllowed(true);
 
-  // Unload plugins. This needs to happen on the IO thread.
-  BrowserThread::PostTask(
-        BrowserThread::IO, FROM_HERE,
-        NewRunnableFunction(&ChromePluginLib::UnloadAllPlugins));
-
-  // Shutdown all IPC channels to service processes.
-  ServiceProcessControlManager::GetInstance()->Shutdown();
-
-#if defined(OS_CHROMEOS)
-  // The system key event listener needs to be shut down earlier than when
-  // Singletons are finally destroyed in AtExitManager.
-  chromeos::SystemKeyEventListener::GetInstance()->Stop();
-#endif
+  // Shutdown the IPC channel to the service processes.
+  ServiceProcessControl::GetInstance()->Disconnect();
 
   // WARNING: During logoff/shutdown (WM_ENDSESSION) we may not have enough
   // time to get here. If you have something that *must* happen on end session,
   // consider putting it in BrowserProcessImpl::EndSession.
-  DCHECK(g_browser_process);
-
-  // Notifies we are going away.
-  g_browser_process->shutdown_event()->Signal();
-
   PrefService* prefs = g_browser_process->local_state();
-  ProfileManager* profile_manager = g_browser_process->profile_manager();
-  PrefService* user_prefs = profile_manager->GetDefaultProfile()->GetPrefs();
-
-  chrome_browser_net::SavePredictorStateForNextStartupAndTrim(user_prefs);
 
   MetricsService* metrics = g_browser_process->metrics_service();
-  if (metrics) {
-    metrics->RecordCleanShutdown();
+  if (metrics)
     metrics->RecordCompletedSessionEnd();
-  }
 
   if (shutdown_type_ > NOT_VALID && shutdown_num_processes_ > 0) {
     // Record the shutdown info so that we can put it into a histogram at next
@@ -170,7 +155,7 @@ void Shutdown() {
     prefs->ClearPref(prefs::kRestartLastSessionOnShutdown);
   }
 
-  prefs->SavePersistentPrefs();
+  prefs->CommitPendingWrite();
 
 #if defined(OS_WIN) && defined(GOOGLE_CHROME_BUILD)
   // Cleanup any statics created by RLZ. Must be done before NotificationService
@@ -178,10 +163,19 @@ void Shutdown() {
   RLZTracker::CleanupRlz();
 #endif
 
+  return restart_last_session;
+}
+
+void ShutdownPostThreadsStop(bool restart_last_session) {
   // The jank'o'meter requires that the browser process has been destroyed
   // before calling UninstallJankometer().
   delete g_browser_process;
   g_browser_process = NULL;
+
+  // crbug.com/95079 - This needs to happen after the browser process object
+  // goes away.
+  ProfileManager::NukeDeletedProfilesFromDisk();
+
 #if defined(OS_CHROMEOS)
   chromeos::BootTimesLoader::Get()->AddLogoutTimeMarker("BrowserDeleted",
                                                         true);
@@ -194,9 +188,9 @@ void Shutdown() {
     ResourceBundle::CleanupSharedInstance();
 
 #if defined(OS_WIN)
-  if (!Upgrade::IsBrowserAlreadyRunning() &&
+  if (!browser_util::IsBrowserAlreadyRunning() &&
       shutdown_type_ != browser_shutdown::END_SESSION) {
-    Upgrade::SwapNewChromeExeIfPresent();
+    upgrade_util::SwapNewChromeExeIfPresent();
   }
 #endif
 
@@ -225,19 +219,7 @@ void Shutdown() {
       else
         new_cl->AppendSwitch(i->first);
     }
-    // Ensure restore last session is set.
-    if (!new_cl->HasSwitch(switches::kRestoreLastSession))
-      new_cl->AppendSwitch(switches::kRestoreLastSession);
-
-#if defined(OS_WIN) || defined(OS_LINUX)
-    Upgrade::RelaunchChromeBrowser(*new_cl.get());
-#endif  // defined(OS_WIN) || defined(OS_LINUX)
-
-#if defined(OS_MACOSX)
-    new_cl->AppendSwitch(switches::kActivateOnLaunch);
-    base::LaunchApp(*new_cl.get(), false, false, NULL);
-#endif  // defined(OS_MACOSX)
-
+    upgrade_util::RelaunchChromeBrowser(*new_cl.get());
 #else
     NOTIMPLEMENTED();
 #endif  // !defined(OS_CHROMEOS)
@@ -247,7 +229,7 @@ void Shutdown() {
     // Measure total shutdown time as late in the process as possible
     // and then write it to a file to be read at startup.
     // We can't use prefs since all services are shutdown at this point.
-    TimeDelta shutdown_delta = Time::Now() - shutdown_started_;
+    TimeDelta shutdown_delta = Time::Now() - *shutdown_started_;
     std::string shutdown_ms =
         base::Int64ToString(shutdown_delta.InMilliseconds());
     int len = static_cast<int>(shutdown_ms.length()) + 1;
@@ -255,13 +237,11 @@ void Shutdown() {
     file_util::WriteFile(shutdown_ms_file, shutdown_ms.c_str(), len);
   }
 
-  UnregisterURLRequestChromeJob();
-
 #if defined(OS_CHROMEOS)
-  if (chromeos::CrosLibrary::Get()->EnsureLoaded()) {
-    chromeos::CrosLibrary::Get()->GetLoginLibrary()->StopSession("");
-  }
+  BrowserList::NotifyAndTerminate(false);
 #endif
+
+  ChromeURLDataManager::DeleteDataSources();
 }
 
 void ReadLastShutdownFile(
@@ -285,14 +265,14 @@ void ReadLastShutdownFile(
   std::string time;
   std::string time_per;
   if (type == WINDOW_CLOSE) {
-    time = StringPrintf(time_fmt, "window_close");
-    time_per = StringPrintf(time_per_fmt, "window_close");
+    time = base::StringPrintf(time_fmt, "window_close");
+    time_per = base::StringPrintf(time_per_fmt, "window_close");
   } else if (type == BROWSER_EXIT) {
-    time = StringPrintf(time_fmt, "browser_exit");
-    time_per = StringPrintf(time_per_fmt, "browser_exit");
+    time = base::StringPrintf(time_fmt, "browser_exit");
+    time_per = base::StringPrintf(time_per_fmt, "browser_exit");
   } else if (type == END_SESSION) {
-    time = StringPrintf(time_fmt, "end_session");
-    time_per = StringPrintf(time_per_fmt, "end_session");
+    time = base::StringPrintf(time_fmt, "end_session");
+    time_per = base::StringPrintf(time_per_fmt, "end_session");
   } else {
     NOTREACHED();
   }
@@ -323,8 +303,7 @@ void ReadLastShutdownInfo() {
   // Read and delete the file on the file thread.
   BrowserThread::PostTask(
       BrowserThread::FILE, FROM_HERE,
-      NewRunnableFunction(
-          &ReadLastShutdownFile, type, num_procs, num_procs_slow));
+      base::Bind(&ReadLastShutdownFile, type, num_procs, num_procs_slow));
 }
 
 void SetTryingToQuit(bool quitting) {
@@ -336,11 +315,11 @@ bool IsTryingToQuit() {
 }
 
 bool ShuttingDownWithoutClosingBrowsers() {
-#if defined(USE_X11)
-  if (GetShutdownType() == browser_shutdown::END_SESSION)
-    return true;
-#endif
-  return false;
+  return g_shutting_down_without_closing_browsers;
+}
+
+void SetShuttingDownWithoutClosingBrowsers(bool without_close) {
+  g_shutting_down_without_closing_browsers = without_close;
 }
 
 }  // namespace browser_shutdown
