@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,10 @@
 #define CHROME_BROWSER_BROWSING_DATA_APPCACHE_HELPER_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
-#include "base/task.h"
-#include "chrome/browser/appcache/chrome_appcache_service.h"
-#include "chrome/common/net/url_request_context_getter.h"
+#include "base/callback.h"
+#include "base/memory/scoped_ptr.h"
+#include "content/browser/appcache/chrome_appcache_service.h"
+#include "net/base/completion_callback.h"
 #include "googleurl/src/gurl.h"
 
 class Profile;
@@ -21,7 +21,7 @@ class BrowsingDataAppCacheHelper
  public:
   explicit BrowsingDataAppCacheHelper(Profile* profile);
 
-  virtual void StartFetching(Callback0::Type* completion_callback);
+  virtual void StartFetching(const base::Closure& completion_callback);
   virtual void CancelNotification();
   virtual void DeleteAppCacheGroup(const GURL& manifest_url);
 
@@ -34,17 +34,15 @@ class BrowsingDataAppCacheHelper
   friend class base::RefCountedThreadSafe<BrowsingDataAppCacheHelper>;
   virtual ~BrowsingDataAppCacheHelper();
 
-  scoped_ptr<Callback0::Type> completion_callback_;
+  base::Closure completion_callback_;
   scoped_refptr<appcache::AppCacheInfoCollection> info_collection_;
 
  private:
   void OnFetchComplete(int rv);
-  ChromeAppCacheService* GetAppCacheService();
 
-  scoped_refptr<URLRequestContextGetter> request_context_getter_;
   bool is_fetching_;
-  scoped_refptr<net::CancelableCompletionCallback<BrowsingDataAppCacheHelper> >
-      appcache_info_callback_;
+  scoped_refptr<ChromeAppCacheService> appcache_service_;
+  net::CancelableCompletionCallback appcache_info_callback_;
 
   DISALLOW_COPY_AND_ASSIGN(BrowsingDataAppCacheHelper);
 };
@@ -56,6 +54,11 @@ class CannedBrowsingDataAppCacheHelper : public BrowsingDataAppCacheHelper {
  public:
   explicit CannedBrowsingDataAppCacheHelper(Profile* profile);
 
+  // Return a copy of the appcache helper. Only one consumer can use the
+  // StartFetching method at a time, so we need to create a copy of the helper
+  // every time we instantiate a cookies tree model for it.
+  CannedBrowsingDataAppCacheHelper* Clone();
+
   // Add an appcache to the set of canned caches that is returned by this
   // helper.
   void AddAppCache(const GURL& manifest_url);
@@ -67,12 +70,14 @@ class CannedBrowsingDataAppCacheHelper : public BrowsingDataAppCacheHelper {
   bool empty() const;
 
   // BrowsingDataAppCacheHelper methods.
-  virtual void StartFetching(Callback0::Type* completion_callback);
-  virtual void CancelNotification() {}
+  virtual void StartFetching(const base::Closure& completion_callback) OVERRIDE;
+  virtual void CancelNotification() OVERRIDE {}
 
  private:
   virtual ~CannedBrowsingDataAppCacheHelper();
 
+  Profile* profile_;
+
   DISALLOW_COPY_AND_ASSIGN(CannedBrowsingDataAppCacheHelper);
 };
 
