@@ -4,8 +4,10 @@
 
 #include "chrome/browser/browsing_data_appcache_helper.h"
 
-#include "base/stl_util-inl.h"
-#include "chrome/test/testing_profile.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
+#include "base/stl_util.h"
+#include "chrome/test/base/testing_profile.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace {
@@ -27,7 +29,9 @@ class TestCompletionCallback {
 
 }  // namespace
 
-TEST(CannedBrowsingDataAppCacheHelperTest, SetInfo) {
+typedef testing::Test CannedBrowsingDataAppCacheHelperTest;
+
+TEST_F(CannedBrowsingDataAppCacheHelperTest, SetInfo) {
   TestingProfile profile;
 
   GURL manifest1("http://example1.com/manifest.xml");
@@ -41,8 +45,8 @@ TEST(CannedBrowsingDataAppCacheHelperTest, SetInfo) {
   helper->AddAppCache(manifest3);
 
   TestCompletionCallback callback;
-  helper->StartFetching(
-      NewCallback(&callback, &TestCompletionCallback::callback));
+  helper->StartFetching(base::Bind(&TestCompletionCallback::callback,
+                                   base::Unretained(&callback)));
   ASSERT_TRUE(callback.have_result());
 
   std::map<GURL, appcache::AppCacheInfoVector>& collection =
@@ -62,7 +66,7 @@ TEST(CannedBrowsingDataAppCacheHelperTest, SetInfo) {
   EXPECT_TRUE(ContainsKey(manifest_results, manifest3));
 }
 
-TEST(CannedBrowsingDataAppCacheHelperTest, Unique) {
+TEST_F(CannedBrowsingDataAppCacheHelperTest, Unique) {
   TestingProfile profile;
 
   GURL manifest("http://example.com/manifest.xml");
@@ -73,8 +77,8 @@ TEST(CannedBrowsingDataAppCacheHelperTest, Unique) {
   helper->AddAppCache(manifest);
 
   TestCompletionCallback callback;
-  helper->StartFetching(
-      NewCallback(&callback, &TestCompletionCallback::callback));
+  helper->StartFetching(base::Bind(&TestCompletionCallback::callback,
+                                   base::Unretained(&callback)));
   ASSERT_TRUE(callback.have_result());
 
   std::map<GURL, appcache::AppCacheInfoVector>& collection =
@@ -86,7 +90,7 @@ TEST(CannedBrowsingDataAppCacheHelperTest, Unique) {
   EXPECT_EQ(manifest, collection[manifest.GetOrigin()].at(0).manifest_url);
 }
 
-TEST(CannedBrowsingDataAppCacheHelperTest, Empty) {
+TEST_F(CannedBrowsingDataAppCacheHelperTest, Empty) {
   TestingProfile profile;
 
   GURL manifest("http://example.com/manifest.xml");
