@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,12 +6,15 @@
 #define CHROME_BROWSER_BROWSING_DATA_INDEXED_DB_HELPER_H_
 #pragma once
 
+#include <list>
 #include <string>
-#include <vector>
 
 #include "base/callback.h"
+#include "base/compiler_specific.h"
 #include "base/file_path.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/synchronization/lock.h"
 #include "base/time.h"
 #include "chrome/common/url_constants.h"
 #include "googleurl/src/gurl.h"
@@ -30,34 +33,16 @@ class BrowsingDataIndexedDBHelper
   // Contains detailed information about an indexed database.
   struct IndexedDBInfo {
     IndexedDBInfo(
-        const std::string& protocol,
-        const std::string& host,
-        unsigned short port,
-        const std::string& database_identifier,
-        const std::string& origin,
-        const FilePath& file_path,
+        const GURL& origin,
         int64 size,
-        base::Time last_modified)
-        : protocol(protocol),
-          host(host),
-          port(port),
-          database_identifier(database_identifier),
-          origin(origin),
-          file_path(file_path),
-          size(size),
-          last_modified(last_modified) {
-    }
+        base::Time last_modified);
+    ~IndexedDBInfo();
 
     bool IsFileSchemeData() {
-      return protocol == chrome::kFileScheme;
+      return origin.SchemeIsFile();
     }
 
-    std::string protocol;
-    std::string host;
-    unsigned short port;
-    std::string database_identifier;
-    std::string origin;
-    FilePath file_path;
+    GURL origin;
     int64 size;
     base::Time last_modified;
   };
@@ -70,13 +55,14 @@ class BrowsingDataIndexedDBHelper
   // callback.
   // This must be called only in the UI thread.
   virtual void StartFetching(
-      Callback1<const std::vector<IndexedDBInfo>& >::Type* callback) = 0;
+      const base::Callback<void(const std::list<IndexedDBInfo>&)>&
+          callback) = 0;
   // Cancels the notification callback (i.e., the window that created it no
   // longer exists).
   // This must be called only in the UI thread.
   virtual void CancelNotification() = 0;
-  // Requests a single indexed database file to be deleted in the WEBKIT thread.
-  virtual void DeleteIndexedDBFile(const FilePath& file_path) = 0;
+  // Requests a single indexed database to be deleted in the WEBKIT thread.
+  virtual void DeleteIndexedDB(const GURL& origin) = 0;
 
  protected:
   friend class base::RefCountedThreadSafe<BrowsingDataIndexedDBHelper>;
@@ -89,7 +75,12 @@ class BrowsingDataIndexedDBHelper
 class CannedBrowsingDataIndexedDBHelper
     : public BrowsingDataIndexedDBHelper {
  public:
-  explicit CannedBrowsingDataIndexedDBHelper(Profile* profile);
+  CannedBrowsingDataIndexedDBHelper();
+
+  // Return a copy of the IndexedDB helper. Only one consumer can use the
+  // StartFetching method at a time, so we need to create a copy of the helper
+  // every time we instantiate a cookies tree model for it.
+  CannedBrowsingDataIndexedDBHelper* Clone();
 
   // Add a indexed database to the set of canned indexed databases that is
   // returned by this helper.
@@ -104,16 +95,45 @@ class CannedBrowsingDataIndexedDBHelper
 
   // BrowsingDataIndexedDBHelper methods.
   virtual void StartFetching(
-      Callback1<const std::vector<IndexedDBInfo>& >::Type* callback);
-  virtual void CancelNotification() {}
-  virtual void DeleteIndexedDBFile(const FilePath& file_path) {}
+      const base::Callback<void(const std::list<IndexedDBInfo>&)>&
+          callback) OVERRIDE;
+  virtual void CancelNotification() OVERRIDE;
+  virtual void DeleteIndexedDB(const GURL& origin) OVERRIDE {}
 
  private:
+  struct PendingIndexedDBInfo {
+    PendingIndexedDBInfo();
+    PendingIndexedDBInfo(const GURL& origin, const string16& description);
+    ~PendingIndexedDBInfo();
+
+    GURL origin;
+    string16 description;
+  };
+
   virtual ~CannedBrowsingDataIndexedDBHelper();
 
-  Profile* profile_;
+  // Convert the pending indexed db info to indexed db info objects.
+  void ConvertPendingInfoInWebKitThread();
 
-  std::vector<IndexedDBInfo> indexed_db_info_;
+  void NotifyInUIThread();
+
+  // Lock to protect access to pending_indexed_db_info_;
+  mutable base::Lock lock_;
+
+  // This may mutate on WEBKIT and UI threads.
+  std::list<PendingIndexedDBInfo> pending_indexed_db_info_;
+
+  // This only mutates on the WEBKIT thread.
+  std::list<IndexedDBInfo> indexed_db_info_;
+
+  // This only mutates on the UI thread.
+  base::Callback<void(const std::list<IndexedDBInfo>&)> completion_callback_;
+
+  // Indicates whether or not we're currently fetching information:
+  // it's true when StartFetching() is called in the UI thread, and it's reset
+  // after we notified the callback in the UI thread.
+  // This only mutates on the UI thread.
+  bool is_fetching_;
 
   DISALLOW_COPY_AND_ASSIGN(CannedBrowsingDataIndexedDBHelper);
 };
