@@ -4,15 +4,16 @@
 
 #include "chrome/browser/cancelable_request.h"
 
-CancelableRequestProvider::CancelableRequestProvider() : next_handle_(1) {
+CancelableRequestProvider::CancelableRequestProvider()
+    : next_handle_(1) {
 }
 
 CancelableRequestProvider::~CancelableRequestProvider() {
   // There may be requests whose result callback has not been run yet. We need
   // to cancel them otherwise they may try and call us back after we've been
-  // deleted, or do other bad things. This can occur on shutdown (or profile
-  // destruction) when a request is scheduled, completed (but not dispatched),
-  // then the Profile is deleted.
+  // deleted, or do other bad things. This can occur on shutdown (or browser
+  // context destruction) when a request is scheduled, completed (but not
+  // dispatched), then the BrowserContext is deleted.
   base::AutoLock lock(pending_request_lock_);
   while (!pending_requests_.empty())
     CancelRequestLocked(pending_requests_.begin());
@@ -28,6 +29,8 @@ CancelableRequestProvider::Handle CancelableRequestProvider::AddRequest(
     handle = next_handle_;
     pending_requests_[next_handle_] = request;
     ++next_handle_;
+    DCHECK(next_handle_)
+        << "next_handle_ may have wrapped around to invalid state.";
   }
 
   consumer->OnRequestAdded(this, handle);
@@ -104,3 +107,34 @@ void CancelableRequestBase::Init(CancelableRequestProvider* provider,
   consumer_ = consumer;
   handle_ = handle;
 }
+
+void CancelableRequestBase::DoForward(const base::Closure& forwarded_call,
+                                      bool force_async) {
+  if (force_async || callback_thread_ != MessageLoop::current()) {
+    callback_thread_->PostTask(
+        FROM_HERE,
+        base::Bind(&CancelableRequestBase::ExecuteCallback, this,
+                   forwarded_call));
+  } else {
+    // We can do synchronous callbacks when we're on the same thread.
+    ExecuteCallback(forwarded_call);
+  }
+}
+
+void CancelableRequestBase::ExecuteCallback(
+    const base::Closure& forwarded_call) {
+  DCHECK_EQ(callback_thread_, MessageLoop::current());
+
+  if (!canceled_.IsSet()) {
+    WillExecute();
+
+    // Execute the callback.
+    forwarded_call.Run();
+  }
+
+  // Notify the provider that the request is complete. The provider will
+  // notify the consumer for us. Note that it is possible for the callback to
+  // cancel this request; we must check canceled again.
+  if (!canceled_.IsSet())
+    NotifyCompleted();
+}
