@@ -1,11 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 // CancelableRequestProviders and Consumers work together to make requests that
 // execute on a background thread in the provider and return data to the
-// consumer. These class collaborate to keep a list of open requests and to
-// make sure that requests to not outlive either of the objects involved in the
+// consumer. These classes collaborate to keep a list of open requests and to
+// make sure that requests do not outlive either of the objects involved in the
 // transaction.
 //
 // If you do not need to return data to the consumer, do not use this system,
@@ -18,9 +18,10 @@
 //
 //   class MyClass {
 //     void MakeRequest() {
-//       frontend_service->StartRequest(some_input1, some_input2,
+//       frontend_service->StartRequest(
+//           some_input1, some_input2,
 //           &callback_consumer_,
-//           NewCallback(this, &MyClass:RequestComplete));
+//           base::Bind(&MyClass:RequestComplete, base::Unretained(this)));
 //       // StartRequest() returns a Handle which may be retained for use with
 //       // CancelRequest() if required, e.g. in MyClass's destructor.
 //     }
@@ -48,8 +49,9 @@
 //       AddRequest(request, consumer);
 //
 //       // Send the parameters and the request to the backend thread.
-//       backend_thread_->PostTask(FROM_HERE,
-//           NewRunnableMethod(backend_, &Backend::DoRequest, request,
+//       backend_thread_->PostTask(
+//           FROM_HERE,
+//           base::Bind(&Backend::DoRequest, backend_, request,
 //                             some_input1, some_input2));
 //
 //       // The handle will have been set by AddRequest.
@@ -83,22 +85,23 @@
 //     }
 //   };
 
-#ifndef CHROME_BROWSER_CANCELABLE_REQUEST_H__
-#define CHROME_BROWSER_CANCELABLE_REQUEST_H__
+#ifndef CHROME_BROWSER_CANCELABLE_REQUEST_H_
+#define CHROME_BROWSER_CANCELABLE_REQUEST_H_
 #pragma once
 
 #include <map>
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/callback.h"
+#include "base/callback_internal.h"
 #include "base/logging.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
 #include "base/synchronization/cancellation_flag.h"
 #include "base/synchronization/lock.h"
-#include "base/task.h"
 #include "build/build_config.h"
 
 class CancelableRequestBase;
@@ -208,7 +211,8 @@ class CancelableRequestConsumerBase {
 // The type T should be small and easily copyable (like a pointer
 // or an integer).
 template<class T>
-class CancelableRequestConsumerTSimple : public CancelableRequestConsumerBase {
+class CancelableRequestConsumerTSimple
+    : public CancelableRequestConsumerBase {
  public:
   CancelableRequestConsumerTSimple();
 
@@ -244,6 +248,9 @@ class CancelableRequestConsumerTSimple : public CancelableRequestConsumerBase {
   // Cancels all requests outstanding.
   void CancelAllRequests();
 
+  // Cancels all requests outstanding matching the client data.
+  void CancelAllRequestsForClientData(T client_data);
+
   // Returns the handle for the first request that has the specified client data
   // (in |handle|). Returns true if there is a request for the specified client
   // data, false otherwise.
@@ -344,16 +351,32 @@ size_t CancelableRequestConsumerTSimple<T>::PendingRequestCount() const {
 
 template<class T>
 void CancelableRequestConsumerTSimple<T>::CancelAllRequests() {
+  // TODO(atwilson): This code is not thread safe as it is called from the
+  // consumer thread (via the destructor) and accesses pending_requests_
+  // without acquiring the provider lock (http://crbug.com/85970).
   PendingRequestList copied_requests(pending_requests_);
   for (typename PendingRequestList::iterator i = copied_requests.begin();
-       i != copied_requests.end(); ++i)
+       i != copied_requests.end(); ++i) {
     i->first.provider->CancelRequest(i->first.handle);
+  }
   copied_requests.clear();
 
   // That should have cleared all the pending items.
   DCHECK(pending_requests_.empty());
 }
 
+template<class T>
+void CancelableRequestConsumerTSimple<T>::CancelAllRequestsForClientData(
+    T client_data) {
+  PendingRequestList copied_requests(pending_requests_);
+  for (typename PendingRequestList::const_iterator i = copied_requests.begin();
+      i != copied_requests.end(); ++i) {
+    if (i->second == client_data)
+      i->first.provider->CancelRequest(i->first.handle);
+  }
+  copied_requests.clear();
+}
+
 template<class T>
 bool CancelableRequestConsumerTSimple<T>::GetFirstHandleForClientData(
     T client_data,
@@ -387,6 +410,13 @@ template<class T>
 void CancelableRequestConsumerTSimple<T>::OnRequestAdded(
     CancelableRequestProvider* provider,
     CancelableRequestProvider::Handle handle) {
+  // Make sure we're not mixing/matching providers (since the provider is
+  // responsible for providing thread safety until http://crbug.com/85970 is
+  // fixed, we can't use the same consumer with multiple providers).
+#ifndef NDEBUG
+  if (!pending_requests_.empty())
+    DCHECK(pending_requests_.begin()->first.provider == provider);
+#endif
   DCHECK(pending_requests_.find(PendingRequest(provider, handle)) ==
          pending_requests_.end());
   pending_requests_[PendingRequest(provider, handle)] = get_initial_t();
@@ -423,7 +453,8 @@ void CancelableRequestConsumerTSimple<T>::DidExecute(
 // See CancelableRequestConsumerTSimple. The default value for T
 // is given in |initial_t|.
 template<class T, T initial_t>
-class CancelableRequestConsumerT : public CancelableRequestConsumerTSimple<T> {
+class CancelableRequestConsumerT
+    : public CancelableRequestConsumerTSimple<T> {
  public:
   CancelableRequestConsumerT();
   virtual ~CancelableRequestConsumerT();
@@ -494,17 +525,17 @@ class CancelableRequestBase
   friend class CancelableRequestProvider;
 
   // Initializes most things to empty, Init() must be called to complete
-  // initialization of the object. This will be done by the provider when
-  // the request is dispatched.
+  // initialization of the object. This will be done by the provider when the
+  // request is dispatched.
   //
-  // This must be called on the same thread the callback will be executed on,
-  // it will save that thread for later.
+  // This must be called on the same thread the callback will be executed on, it
+  // will save that thread for later.
   //
-  // This two-phase init is done so that the constructor can have no
-  // parameters, which makes it much more convenient for derived classes,
-  // which can be common. The derived classes need only declare the variables
-  // they provide in the constructor rather than many lines of internal
-  // tracking data that are passed to the base class (us).
+  // This two-phase init is done so that the constructor can have no parameters,
+  // which makes it much more convenient for derived classes, which can be
+  // common. The derived classes need only declare the variables they provide in
+  // the constructor rather than many lines of internal tracking data that are
+  // passed to the base class (us).
   //
   // In addition, not all of the information (for example, the handle) is known
   // at construction time.
@@ -538,6 +569,10 @@ class CancelableRequestBase
             CancelableRequestProvider::Handle handle,
             CancelableRequestConsumerBase* consumer);
 
+  // Attempts to execute callback on |callback_thread_| if the request has not
+  // been canceled yet.
+  void DoForward(const base::Closure& forwarded_call, bool force_async);
+
   // Tells the provider that the request is complete, which then tells the
   // consumer.
   void NotifyCompleted() const {
@@ -572,6 +607,11 @@ class CancelableRequestBase
   base::CancellationFlag canceled_;
 
  private:
+  // Executes the |forwarded_call| and notifies the provider and the consumer
+  // that this request has been completed. This must be called on the
+  // |callback_thread_|.
+  void ExecuteCallback(const base::Closure& forwarded_call);
+
   DISALLOW_COPY_AND_ASSIGN(CancelableRequestBase);
 };
 
@@ -604,6 +644,8 @@ class CancelableRequestBase
 template<typename CB>
 class CancelableRequest : public CancelableRequestBase {
  public:
+  // TODO(ajwong): After all calls have been migrated, remove this template in
+  // favor of the specialization on base::Callback<Sig>.
   typedef CB CallbackType;          // CallbackRunner<...>
   typedef typename CB::TupleType TupleType;  // Tuple of the callback args.
 
@@ -632,8 +674,9 @@ class CancelableRequest : public CancelableRequestBase {
         // We can do synchronous callbacks when we're on the same thread.
         ExecuteCallback(param);
       } else {
-        callback_thread_->PostTask(FROM_HERE, NewRunnableMethod(this,
-            &CancelableRequest<CB>::ExecuteCallback, param));
+        callback_thread_->PostTask(
+            FROM_HERE,
+            base::Bind(&CancelableRequest<CB>::ExecuteCallback, this, param));
       }
     }
   }
@@ -642,8 +685,9 @@ class CancelableRequest : public CancelableRequestBase {
   void ForwardResultAsync(const TupleType& param) {
     DCHECK(callback_.get());
     if (!canceled()) {
-      callback_thread_->PostTask(FROM_HERE, NewRunnableMethod(this,
-          &CancelableRequest<CB>::ExecuteCallback, param));
+      callback_thread_->PostTask(
+          FROM_HERE,
+          base::Bind(&CancelableRequest<CB>::ExecuteCallback, this, param));
     }
   }
 
@@ -659,11 +703,13 @@ class CancelableRequest : public CancelableRequestBase {
 
       // Execute the callback.
       callback_->RunWithParams(param);
-
-      // Notify the provider that the request is complete. The provider will
-      // notify the consumer for us.
-      NotifyCompleted();
     }
+
+    // Notify the provider that the request is complete. The provider will
+    // notify the consumer for us. Note that it is possible for the callback to
+    // cancel this request; we must check canceled again.
+    if (!canceled_.IsSet())
+      NotifyCompleted();
   }
 
   // This should only be executed if !canceled_.IsSet(),
@@ -671,6 +717,272 @@ class CancelableRequest : public CancelableRequestBase {
   scoped_ptr<CallbackType> callback_;
 };
 
+// CancelableRequest<> wraps a normal base::Callback<> for use in the
+// CancelableRequest framework.
+//
+// When using this class, the provider MUST call Init() (on the base class)
+// before this is valid.
+//
+// The two functions ForwardResult(), and ForwardResultAsync() are used to
+// invoke the wrapped callback on the correct thread.  They are the same
+// except that ForwardResult() will run the callback synchronously if
+// we are already on the right thread.
+//
+// The caller does not need to check for cancel before calling ForwardResult()
+// or ForwardResultAsync(). If the request has not been canceled, one of the
+// these MUST be called.
+//
+// If there are any pointers in the parameters, they must live at least as
+// long as the request so that it can be forwarded to the other thread.
+// For complex objects, this would typically be done by having a derived
+// request own the data itself.
+//
+// The following specializations handle different arities of base::Callbacks<>.
+template<>
+class CancelableRequest<base::Closure> : public CancelableRequestBase {
+ public:
+  typedef base::Closure CallbackType;
+
+  explicit CancelableRequest(const CallbackType& callback)
+      : CancelableRequestBase(),
+        callback_(callback) {
+    DCHECK(!callback.is_null()) << "Callback must be initialized.";
+  }
+
+  void ForwardResult(void) {
+    if (canceled()) return;
+    DoForward(callback_, false);
+  }
+
+  void ForwardResultAsync() {
+    if (canceled()) return;
+    DoForward(callback_, true);
+  }
+
+ protected:
+  virtual ~CancelableRequest() {}
+
+ private:
+  CallbackType callback_;
+};
+
+template<typename A1>
+class CancelableRequest<base::Callback<void(A1)> >
+    : public CancelableRequestBase {
+ public:
+  typedef base::Callback<void(A1)> CallbackType;
+
+  explicit CancelableRequest(const CallbackType& callback)
+      : CancelableRequestBase(),
+        callback_(callback) {
+    DCHECK(!callback.is_null()) << "Callback must be initialized.";
+  }
+
+  void ForwardResult(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1), false);
+  }
+
+  void ForwardResultAsync(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1), true);
+  }
+
+ protected:
+  virtual ~CancelableRequest() {}
+
+ private:
+  CallbackType callback_;
+};
+
+template<typename A1, typename A2>
+class CancelableRequest<base::Callback<void(A1,A2)> >
+    : public CancelableRequestBase {
+ public:
+  typedef base::Callback<void(A1,A2)> CallbackType;
+
+  explicit CancelableRequest(const CallbackType& callback)
+      : CancelableRequestBase(),
+        callback_(callback) {
+    DCHECK(!callback.is_null()) << "Callback must be initialized.";
+  }
+
+  void ForwardResult(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2), false);
+  }
+
+  void ForwardResultAsync(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2), true);
+  }
+
+ protected:
+  virtual ~CancelableRequest() {}
+
+ private:
+  CallbackType callback_;
+};
+
+template<typename A1, typename A2, typename A3>
+class CancelableRequest<base::Callback<void(A1,A2,A3)> >
+    : public CancelableRequestBase {
+ public:
+  typedef base::Callback<void(A1,A2,A3)> CallbackType;
+
+  explicit CancelableRequest(const CallbackType& callback)
+      : CancelableRequestBase(),
+        callback_(callback) {
+    DCHECK(!callback.is_null()) << "Callback must be initialized.";
+  }
+
+  void ForwardResult(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3), false);
+  }
+
+  void ForwardResultAsync(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3), true);
+  }
+
+ protected:
+  virtual ~CancelableRequest() {}
+
+ private:
+  CallbackType callback_;
+};
+
+template<typename A1, typename A2, typename A3, typename A4>
+class CancelableRequest<base::Callback<void(A1, A2, A3, A4)> >
+    : public CancelableRequestBase {
+ public:
+  typedef base::Callback<void(A1, A2, A3, A4)> CallbackType;
+
+  explicit CancelableRequest(const CallbackType& callback)
+      : CancelableRequestBase(),
+        callback_(callback) {
+    DCHECK(!callback.is_null()) << "Callback must be initialized.";
+  }
+
+  void ForwardResult(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3,
+      typename base::internal::CallbackParamTraits<A4>::ForwardType a4) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3, a4), false);
+  }
+
+  void ForwardResultAsync(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3,
+      typename base::internal::CallbackParamTraits<A4>::ForwardType a4) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3, a4), true);
+  }
+
+ protected:
+  virtual ~CancelableRequest() {}
+
+ private:
+  CallbackType callback_;
+};
+
+template<typename A1, typename A2, typename A3, typename A4, typename A5>
+class CancelableRequest<base::Callback<void(A1, A2, A3, A4, A5)> >
+    : public CancelableRequestBase {
+ public:
+  typedef base::Callback<void(A1, A2, A3, A4, A5)> CallbackType;
+
+  explicit CancelableRequest(const CallbackType& callback)
+      : CancelableRequestBase(),
+        callback_(callback) {
+    DCHECK(!callback.is_null()) << "Callback must be initialized.";
+  }
+
+  void ForwardResult(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3,
+      typename base::internal::CallbackParamTraits<A4>::ForwardType a4,
+      typename base::internal::CallbackParamTraits<A5>::ForwardType a5) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3, a4, a5), false);
+  }
+
+  void ForwardResultAsync(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3,
+      typename base::internal::CallbackParamTraits<A4>::ForwardType a4,
+      typename base::internal::CallbackParamTraits<A5>::ForwardType a5) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3, a4, a5), true);
+  }
+
+ protected:
+  virtual ~CancelableRequest() {}
+
+ private:
+  CallbackType callback_;
+};
+
+template<typename A1, typename A2, typename A3, typename A4, typename A5,
+         typename A6>
+class CancelableRequest<base::Callback<void(A1, A2, A3, A4, A5, A6)> >
+    : public CancelableRequestBase {
+ public:
+  typedef base::Callback<void(A1, A2, A3, A4, A5, A6)> CallbackType;
+
+  explicit CancelableRequest(const CallbackType& callback)
+      : CancelableRequestBase(),
+        callback_(callback) {
+    DCHECK(!callback.is_null()) << "Callback must be initialized.";
+  }
+
+  void ForwardResult(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3,
+      typename base::internal::CallbackParamTraits<A4>::ForwardType a4,
+      typename base::internal::CallbackParamTraits<A5>::ForwardType a5,
+      typename base::internal::CallbackParamTraits<A6>::ForwardType a6) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3, a4, a5, a6), false);
+  }
+
+  void ForwardResultAsync(
+      typename base::internal::CallbackParamTraits<A1>::ForwardType a1,
+      typename base::internal::CallbackParamTraits<A2>::ForwardType a2,
+      typename base::internal::CallbackParamTraits<A3>::ForwardType a3,
+      typename base::internal::CallbackParamTraits<A4>::ForwardType a4,
+      typename base::internal::CallbackParamTraits<A5>::ForwardType a5,
+      typename base::internal::CallbackParamTraits<A6>::ForwardType a6) {
+    if (canceled()) return;
+    DoForward(base::Bind(callback_, a1, a2, a3, a4, a5, a6), true);
+  }
+
+ protected:
+  virtual ~CancelableRequest() {}
+
+ private:
+  CallbackType callback_;
+};
+
 // A CancelableRequest with a single value. This is intended for use when
 // the provider provides a single value. The provider fills the result into
 // the value, and notifies the request with a pointer to the value. For example,
@@ -680,7 +992,7 @@ class CancelableRequest : public CancelableRequestBase {
 //      typedef Callback2<Handle, std::vector<Foo>*>::Type FooCallback;
 // 2. Define the CancelableRequest1 type.
 //    typedef CancelableRequest1<FooCallback, std::vector<Foo>> FooRequest;
-// 3. The provider method should then fillin the contents of the vector,
+// 3. The provider method should then fill in the contents of the vector,
 //    forwarding the result like so:
 //    request->ForwardResult(FooRequest::TupleType(request->handle(),
 //                                                 &request->value));
@@ -690,8 +1002,8 @@ template<typename CB, typename Type>
 class CancelableRequest1 : public CancelableRequest<CB> {
  public:
   explicit CancelableRequest1(
-      typename CancelableRequest<CB>::CallbackType* callback)
-      : CancelableRequest<CB>(callback) {
+      const typename CancelableRequest<CB>::CallbackType& callback)
+      : CancelableRequest<CB>(callback), value() {
   }
 
   // The value.
@@ -701,4 +1013,4 @@ class CancelableRequest1 : public CancelableRequest<CB> {
   virtual ~CancelableRequest1() {}
 };
 
-#endif  // CHROME_BROWSER_CANCELABLE_REQUEST_H__
+#endif  // CHROME_BROWSER_CANCELABLE_REQUEST_H_
