@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,7 @@
 #include <map>
 #include <string>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/string16.h"
 #include "net/base/cert_database.h"
 
@@ -43,9 +43,10 @@ class CertificateManagerModel {
   // Accessor for read-only access to the underlying CertDatabase.
   const net::CertDatabase& cert_db() const { return cert_db_; }
 
-  // Refresh the list of certs.  Following this call, the observer
-  // CertificatesRefreshed method will be called so the view can call
-  // FilterAndBuildOrgGroupingMap as necessary to refresh its tree views.
+  // Trigger a refresh of the list of certs, unlock any slots if necessary.
+  // Following this call, the observer CertificatesRefreshed method will be
+  // called so the view can call FilterAndBuildOrgGroupingMap as necessary to
+  // refresh its tree views.
   void Refresh();
 
   // Fill |map| with the certificates matching |filter_type|.
@@ -55,10 +56,12 @@ class CertificateManagerModel {
   // Get the data to be displayed in |column| for the given |cert|.
   string16 GetColumnText(const net::X509Certificate& cert, Column column) const;
 
-  // Import certificates from PKCS #12 encoded |data|, using the given
-  // |password|.  Returns a net error code on failure.
+  // Import private keys and certificates from PKCS #12 encoded
+  // |data|, using the given |password|. If |is_extractable| is false,
+  // mark the private key as unextractable from the module.
+  // Returns a net error code on failure.
   int ImportFromPKCS12(net::CryptoModule* module, const std::string& data,
-                       const string16& password);
+                       const string16& password, bool is_extractable);
 
   // Import CA certificates.
   // Tries to import all the certificates given.  The root will be trusted
@@ -70,7 +73,7 @@ class CertificateManagerModel {
   // |not_imported| should be checked for any certificates that were not
   // imported.
   bool ImportCACerts(const net::CertificateList& certificates,
-                     unsigned int trust_bits,
+                     net::CertDatabase::TrustBits trust_bits,
                      net::CertDatabase::ImportCertFailureList* not_imported);
 
   // Import server certificate.  The first cert should be the server cert.  Any
@@ -91,13 +94,17 @@ class CertificateManagerModel {
   // Returns true on success or false on failure.
   bool SetCertTrust(const net::X509Certificate* cert,
                     net::CertType type,
-                    unsigned int trust_bits);
+                    net::CertDatabase::TrustBits trust_bits);
 
   // Delete the cert.  Returns true on success.  |cert| is still valid when this
   // function returns.
   bool Delete(net::X509Certificate* cert);
 
  private:
+  // Callback used by Refresh() for when the cert slots have been unlocked.
+  // This method does the actual refreshing.
+  void RefreshSlotsUnlocked();
+
   net::CertDatabase cert_db_;
   net::CertificateList cert_list_;
 
