@@ -4,7 +4,7 @@
 
 #include "chrome/browser/certificate_viewer.h"
 
-#include "chrome/browser/cert_store.h"
+#include "content/browser/cert_store.h"
 
 void ShowCertificateViewerByID(gfx::NativeWindow parent, int cert_id) {
   scoped_refptr<net::X509Certificate> cert;
