@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 #define CHROME_BROWSER_CERTIFICATE_VIEWER_H_
 #pragma once
 
-#include "gfx/native_widget_types.h"
+#include "ui/gfx/native_widget_types.h"
 
 namespace net {
 
@@ -22,4 +22,9 @@ void ShowCertificateViewerByID(gfx::NativeWindow parent, int cert_id);
 void ShowCertificateViewer(gfx::NativeWindow parent,
                            net::X509Certificate* cert);
 
+// Opens a certificate viewer under |parent| to display |cert| using a native
+// certificate viewer dialog.
+void ShowNativeCertificateViewer(gfx::NativeWindow parent,
+                                 net::X509Certificate* cert);
+
 #endif  // CHROME_BROWSER_CERTIFICATE_VIEWER_H_
