@@ -1,17 +1,23 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #import "chrome/browser/chrome_browser_application_mac.h"
 
 #import "base/logging.h"
+#include "base/mac/crash_logging.h"
+#import "base/mac/scoped_nsexception_enabler.h"
 #import "base/metrics/histogram.h"
-#import "base/scoped_nsobject.h"
+#import "base/memory/scoped_nsobject.h"
 #import "base/sys_string_conversions.h"
-#import "chrome/app/breakpad_mac.h"
 #import "chrome/browser/app_controller_mac.h"
-#import "chrome/browser/ui/cocoa/objc_method_swizzle.h"
-#import "chrome/browser/ui/cocoa/objc_zombie.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#import "chrome/common/mac/objc_method_swizzle.h"
+#import "chrome/common/mac/objc_zombie.h"
+#include "content/browser/accessibility/browser_accessibility_state.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/web_contents.h"
 
 // The implementation of NSExceptions break various assumptions in the
 // Chrome code.  This category defines a replacement for
@@ -19,18 +25,18 @@
 // the debugger when an exception is raised.  -raise sounds more
 // obvious to intercept, but it doesn't catch the original throw
 // because the objc runtime doesn't use it.
-@interface NSException (NSExceptionSwizzle)
-- (id)chromeInitWithName:(NSString*)aName
-                  reason:(NSString*)aReason
-                userInfo:(NSDictionary *)someUserInfo;
+@interface NSException (CrNSExceptionSwizzle)
+- (id)crInitWithName:(NSString*)aName
+              reason:(NSString*)aReason
+            userInfo:(NSDictionary*)someUserInfo;
 @end
 
 static IMP gOriginalInitIMP = NULL;
 
-@implementation NSException (NSExceptionSwizzle)
-- (id)chromeInitWithName:(NSString*)aName
-                  reason:(NSString*)aReason
-                userInfo:(NSDictionary *)someUserInfo {
+@implementation NSException (CrNSExceptionSwizzle)
+- (id)crInitWithName:(NSString*)aName
+              reason:(NSString*)aReason
+            userInfo:(NSDictionary*)someUserInfo {
   // Method only called when swizzled.
   DCHECK(_cmd == @selector(initWithName:reason:userInfo:));
 
@@ -58,7 +64,7 @@ static IMP gOriginalInitIMP = NULL;
     static NSString* const kNSExceptionKey = @"nsexception";
     NSString* value =
         [NSString stringWithFormat:@"%@ reason %@", aName, aReason];
-    SetCrashKeyValue(kNSExceptionKey, value);
+    base::mac::SetCrashKeyValue(kNSExceptionKey, value);
 
     // Force crash for selected exceptions to generate crash dumps.
     BOOL fatal = NO;
@@ -69,6 +75,27 @@ static IMP gOriginalInitIMP = NULL;
       if ([aReason isEqualToString:kNSMenuItemArrayBoundsCheck]) {
         fatal = YES;
       }
+
+      NSString* const kNoWindowCheck = @"View is not in any window";
+      if ([aReason isEqualToString:kNoWindowCheck]) {
+        fatal = YES;
+      }
+    }
+
+    // Mostly "unrecognized selector sent to (instance|class)".  A
+    // very small number of things like inappropriate nil being passed.
+    if (aName == NSInvalidArgumentException) {
+      fatal = YES;
+
+      // TODO(shess): http://crbug.com/85463 throws this exception
+      // from ImageKit.  Our code is not on the stack, so it needs to
+      // be whitelisted for now.
+      NSString* const kNSURLInitNilCheck =
+          @"*** -[NSURL initFileURLWithPath:isDirectory:]: "
+          @"nil string parameter";
+      if ([aReason isEqualToString:kNSURLInitNilCheck]) {
+        fatal = NO;
+      }
     }
 
     // Dear reader: Something you just did provoked an NSException.
@@ -77,7 +104,8 @@ static IMP gOriginalInitIMP = NULL;
     // (destructors are skipped).  Chrome should be NSException-free,
     // please check your backtrace and see if you can't file a bug
     // with a repro case.
-    if (fatal) {
+    const bool allow = base::mac::GetNSExceptionsAllowed();
+    if (fatal && !allow) {
       LOG(FATAL) << "Someone is trying to raise an exception!  "
                  << base::SysNSStringToUTF8(value);
     } else {
@@ -85,7 +113,7 @@ static IMP gOriginalInitIMP = NULL;
       // exceptions.
       DLOG(ERROR) << "Someone is trying to raise an exception!  "
                   << base::SysNSStringToUTF8(value);
-      NOTREACHED();
+      DCHECK(allow);
     }
   }
 
@@ -110,18 +138,26 @@ size_t BinForException(NSException* exception) {
   // determine where they live in the histogram, so never move them
   // around, only add to the end.
   static NSString* const kKnownNSExceptionNames[] = {
-    // ???
+    // Grab-bag exception, not very common.  CFArray (or other
+    // container) mutated while being enumerated is one case seen in
+    // production.
     NSGenericException,
 
-    // Out-of-range on NSString or NSArray.
+    // Out-of-range on NSString or NSArray.  Quite common.
     NSRangeException,
 
-    // Invalid arg to method, unrecognized selector.
+    // Invalid arg to method, unrecognized selector.  Quite common.
     NSInvalidArgumentException,
 
-    // malloc() returned null in object creation, I think.
+    // malloc() returned null in object creation, I think.  Turns out
+    // to be very uncommon in production, because of the OOM killer.
     NSMallocException,
 
+    // This contains things like windowserver errors, trying to draw
+    // views which aren't in windows, unable to read nib files.  By
+    // far the most common exception seen on the crash server.
+    NSInternalInconsistencyException,
+
     nil
   };
 
@@ -158,32 +194,14 @@ void CancelTerminate() {
 
 namespace {
 
-// Helper to make it easy to get crash keys right.
-// TODO(shess): Find a better home for this.  app/breakpad_mac.h
-// doesn't work.
-class ScopedCrashKey {
- public:
-  ScopedCrashKey(NSString* key, NSString* value)
-      : crash_key_([key retain]) {
-    SetCrashKeyValue(crash_key_.get(), value);
-  }
-  ~ScopedCrashKey() {
-    ClearCrashKeyValue(crash_key_.get());
-  }
-
- private:
-  scoped_nsobject<NSString> crash_key_;
-};
-
-// Do-nothing wrapper so that we can arrange to only swizzle
-// -[NSException raise] when DCHECK() is turned on (as opposed to
-// replicating the preprocess logic which turns DCHECK() on).
-BOOL SwizzleNSExceptionInit() {
+void SwizzleInit() {
+  // Do-nothing wrapper so that we can arrange to only swizzle
+  // -[NSException raise] when DCHECK() is turned on (as opposed to
+  // replicating the preprocess logic which turns DCHECK() on).
   gOriginalInitIMP = ObjcEvilDoers::SwizzleImplementedInstanceMethods(
       [NSException class],
       @selector(initWithName:reason:userInfo:),
-      @selector(chromeInitWithName:reason:userInfo:));
-  return YES;
+      @selector(crInitWithName:reason:userInfo:));
 }
 
 }  // namespace
@@ -193,12 +211,39 @@ BOOL SwizzleNSExceptionInit() {
 + (void)initialize {
   // Turn all deallocated Objective-C objects into zombies, keeping
   // the most recent 10,000 of them on the treadmill.
-  ObjcEvilDoers::ZombieEnable(YES, 10000);
+  ObjcEvilDoers::ZombieEnable(true, 10000);
 }
 
-- init {
-  CHECK(SwizzleNSExceptionInit());
-  return [super init];
+- (id)init {
+  SwizzleInit();
+  if ((self = [super init])) {
+    eventHooks_.reset([[NSMutableArray alloc] init]);
+  }
+  return self;
+}
+
+// Initialize NSApplication using the custom subclass.  Check whether NSApp
+// was already initialized using another class, because that would break
+// some things.
++ (NSApplication*)sharedApplication {
+  NSApplication* app = [super sharedApplication];
+
+  // +sharedApplication initializes the global NSApp, so if a specific
+  // NSApplication subclass is requested, require that to be the one
+  // delivered.  The practical effect is to require a consistent NSApp
+  // across the executable.
+  CHECK([NSApp isKindOfClass:self])
+      << "NSApp must be of type " << [[self className] UTF8String]
+      << ", not " << [[NSApp className] UTF8String];
+
+  // If the message loop was initialized before NSApp is setup, the
+  // message pump will be setup incorrectly.  Failing this implies
+  // that RegisterBrowserCrApp() should be called earlier.
+  CHECK(base::MessagePumpMac::UsingCrApp())
+      << "MessagePumpMac::Create() is using the wrong pump implementation"
+      << " for " << [[self className] UTF8String];
+
+  return app;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -321,10 +366,49 @@ BOOL SwizzleNSExceptionInit() {
         [NSString stringWithFormat:@"%@ tag %d sending %@ to %p",
                   [sender className], tag, actionString, aTarget];
 
-  ScopedCrashKey key(kActionKey, value);
+  base::mac::ScopedCrashKey key(kActionKey, value);
+
+  // Certain third-party code, such as print drivers, can still throw
+  // exceptions and Chromium cannot fix them.  This provides a way to
+  // work around those on a spot basis.
+  bool enableNSExceptions = false;
+
+  // http://crbug.com/80686 , an Epson printer driver.
+  if (anAction == @selector(selectPDE:)) {
+    enableNSExceptions = true;
+  }
+
+  // Minimize the window by keeping this close to the super call.
+  scoped_ptr<base::mac::ScopedNSExceptionEnabler> enabler(NULL);
+  if (enableNSExceptions)
+    enabler.reset(new base::mac::ScopedNSExceptionEnabler());
   return [super sendAction:anAction to:aTarget from:sender];
 }
 
+- (void)addEventHook:(id<CrApplicationEventHookProtocol>)handler {
+  [eventHooks_ addObject:handler];
+}
+
+- (void)removeEventHook:(id<CrApplicationEventHookProtocol>)handler {
+  [eventHooks_ removeObject:handler];
+}
+
+- (BOOL)isHandlingSendEvent {
+  return handlingSendEvent_;
+}
+
+- (void)setHandlingSendEvent:(BOOL)handlingSendEvent {
+  handlingSendEvent_ = handlingSendEvent;
+}
+
+- (void)sendEvent:(NSEvent*)event {
+  base::mac::ScopedSendingEvent sendingEventScoper;
+  for (id<CrApplicationEventHookProtocol> handler in eventHooks_.get()) {
+    [handler hookForEvent:event];
+  }
+  [super sendEvent:event];
+}
+
 // NSExceptions which are caught by the event loop are logged here.
 // NSException uses setjmp/longjmp, which can be very bad for C++, so
 // we attempt to track and report them.
@@ -338,6 +422,20 @@ BOOL SwizzleNSExceptionInit() {
     reportingException = YES;
     chrome_browser_application_mac::RecordExceptionWithUma(anException);
 
+    // http://crbug.com/45928 is a bug about needing to double-close
+    // windows sometimes.  One theory is that |-isHandlingSendEvent|
+    // gets latched to always return |YES|.  Since scopers are used to
+    // manipulate that value, that should not be possible.  One way to
+    // sidestep scopers is setjmp/longjmp (see above).  The following
+    // is to "fix" this while the more fundamental concern is
+    // addressed elsewhere.
+    [self setHandlingSendEvent:NO];
+
+    // If |ScopedNSExceptionEnabler| is used to allow exceptions, and an
+    // uncaught exception is thrown, it will throw past all of the scopers.
+    // Reset the flag so that future exceptions are not masked.
+    base::mac::SetNSExceptionsAllowed(false);
+
     // Store some human-readable information in breakpad keys in case
     // there is a crash.  Since breakpad does not provide infinite
     // storage, we track two exceptions.  The first exception thrown
@@ -360,10 +458,10 @@ BOOL SwizzleNSExceptionInit() {
     NSString* value = [NSString stringWithFormat:@"%@ reason %@",
                                 [anException name], [anException reason]];
     if (!trackedFirstException) {
-      SetCrashKeyValue(kFirstExceptionKey, value);
+      base::mac::SetCrashKeyValue(kFirstExceptionKey, value);
       trackedFirstException = YES;
     } else {
-      SetCrashKeyValue(kLastExceptionKey, value);
+      base::mac::SetCrashKeyValue(kLastExceptionKey, value);
     }
 
     reportingException = NO;
@@ -372,4 +470,22 @@ BOOL SwizzleNSExceptionInit() {
   [super reportException:anException];
 }
 
+- (void)accessibilitySetValue:(id)value forAttribute:(NSString*)attribute {
+  if ([attribute isEqualToString:@"AXEnhancedUserInterface"] &&
+      [value intValue] == 1) {
+    BrowserAccessibilityState::GetInstance()->OnScreenReaderDetected();
+    for (TabContentsIterator it;
+         !it.done();
+         ++it) {
+      if (TabContentsWrapper* contents = *it) {
+        if (RenderViewHost* rvh =
+                contents->web_contents()->GetRenderViewHost()) {
+          rvh->EnableRendererAccessibility();
+        }
+      }
+    }
+  }
+  return [super accessibilitySetValue:value forAttribute:attribute];
+}
+
 @end
