@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,14 +6,17 @@
 #define CHROME_BROWSER_CHROMEOS_BOOT_TIMES_LOADER_H_
 #pragma once
 
+#include <set>
 #include <string>
 
 #include "base/atomic_sequence_num.h"
-#include "base/callback.h"
+#include "base/callback_forward.h"
+#include "base/compiler_specific.h"
 #include "base/time.h"
 #include "chrome/browser/cancelable_request.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/browser/renderer_host/render_widget_host.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 namespace chromeos {
 
@@ -30,9 +33,10 @@ namespace chromeos {
 // . When you want the version invoke: loader.GetBootTimes(&consumer, callback);
 class BootTimesLoader
     : public CancelableRequestProvider,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
   BootTimesLoader();
+  virtual ~BootTimesLoader();
 
   // All fields are 0.0 if they couldn't be found.
   typedef struct BootTimes {
@@ -58,7 +62,7 @@ class BootTimesLoader
   } BootTimes;
 
   // Signature
-  typedef Callback2<Handle, BootTimes>::Type GetBootTimesCallback;
+  typedef base::Callback<void(Handle, BootTimes)> GetBootTimesCallback;
 
   typedef CancelableRequest<GetBootTimesCallback> GetBootTimesRequest;
 
@@ -67,7 +71,7 @@ class BootTimesLoader
   // Asynchronously requests the info.
   Handle GetBootTimes(
       CancelableRequestConsumerBase* consumer,
-      GetBootTimesCallback* callback);
+      const GetBootTimesCallback& callback);
 
   // Add a time marker for login. A timeline will be dumped to
   // /tmp/login-times-sent after login is done. If |send_to_uma| is true
@@ -99,10 +103,10 @@ class BootTimesLoader
   // previous login attempt times.
   void RecordLoginAttempted();
 
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Writes the logout times to a /tmp/logout-times-sent. Unlike login
   // times, we manually call this function for logout times, as we cannot
@@ -116,7 +120,7 @@ class BootTimesLoader
    public:
     Backend() {}
 
-    void GetBootTimes(scoped_refptr<GetBootTimesRequest> request);
+    void GetBootTimes(const scoped_refptr<GetBootTimesRequest>& request);
 
    private:
     friend class base::RefCountedThreadSafe<Backend>;
@@ -156,18 +160,20 @@ class BootTimesLoader
                          const std::string uma_name,
                          const std::string uma_prefix,
                          const std::vector<TimeMarker> login_times);
+  void LoginDone();
 
   // Used to hold the stats at main().
   Stats chrome_main_stats_;
   scoped_refptr<Backend> backend_;
 
   // Used to track notifications for login.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
   base::AtomicSequenceNumber num_tabs_;
   bool have_registered_;
 
   std::vector<TimeMarker> login_time_markers_;
   std::vector<TimeMarker> logout_time_markers_;
+  std::set<RenderWidgetHost*> render_widget_hosts_loading_;
 
   DISALLOW_COPY_AND_ASSIGN(BootTimesLoader);
 };
