@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,9 +8,7 @@
 
 #include <string>
 #include "base/basictypes.h"
-#include "base/command_line.h"
-#include "base/scoped_ptr.h"
-#include "chrome/common/chrome_switches.h"
+#include "base/memory/scoped_ptr.h"
 
 namespace base {
 template <typename T> struct DefaultLazyInstanceTraits;
@@ -18,22 +16,11 @@ template <typename T> struct DefaultLazyInstanceTraits;
 
 namespace chromeos {
 
-class BrightnessLibrary;
 class BurnLibrary;
+class CertLibrary;
 class CryptohomeLibrary;
-class InputMethodLibrary;
-class KeyboardLibrary;
 class LibraryLoader;
-class LoginLibrary;
-class MountLibrary;
 class NetworkLibrary;
-class PowerLibrary;
-class ScreenLockLibrary;
-class SpeechSynthesisLibrary;
-class SyslogsLibrary;
-class SystemLibrary;
-class TouchpadLibrary;
-class UpdateLibrary;
 
 // This class handles access to sub-parts of ChromeOS library. it provides
 // a level of indirection so individual libraries that it exposes can
@@ -45,33 +32,18 @@ class CrosLibrary {
   // mock objects).
   class TestApi {
    public:
-    // Use the stub implementations of the library. This is mainly for
-    // running the chromeos build of chrome on the desktop.
-    void SetUseStubImpl();
-
-    // Reset the stub implementations of the library, called after
-    // SetUseStubImp is called.
+    // Resets the stub implementation of the library and loads libcros.
+    // Used by tests that need to run with libcros (i.e. on a device).
     void ResetUseStubImpl();
 
     // Passing true for own for these setters will cause them to be deleted
     // when the CrosLibrary is deleted (or other mocks are set).
     // Setter for LibraryLoader.
     void SetLibraryLoader(LibraryLoader* loader, bool own);
-    void SetBrightnessLibrary(BrightnessLibrary* library, bool own);
+    void SetCertLibrary(CertLibrary* library, bool own);
     void SetBurnLibrary(BurnLibrary* library, bool own);
     void SetCryptohomeLibrary(CryptohomeLibrary* library, bool own);
-    void SetKeyboardLibrary(KeyboardLibrary* library, bool own);
-    void SetInputMethodLibrary(InputMethodLibrary* library, bool own);
-    void SetLoginLibrary(LoginLibrary* library, bool own);
-    void SetMountLibrary(MountLibrary* library, bool own);
     void SetNetworkLibrary(NetworkLibrary* library, bool own);
-    void SetPowerLibrary(PowerLibrary* library, bool own);
-    void SetScreenLockLibrary(ScreenLockLibrary* library, bool own);
-    void SetSpeechSynthesisLibrary(SpeechSynthesisLibrary* library, bool own);
-    void SetSyslogsLibrary(SyslogsLibrary* library, bool own);
-    void SetSystemLibrary(SystemLibrary* library, bool own);
-    void SetTouchpadLibrary(TouchpadLibrary* library, bool own);
-    void SetUpdateLibrary(UpdateLibrary* library, bool own);
 
    private:
     friend class CrosLibrary;
@@ -79,31 +51,26 @@ class CrosLibrary {
     CrosLibrary* library_;
   };
 
-  // This gets the CrosLibrary.
+  // Sets the global instance. Must be called before any calls to Get().
+  static void Initialize(bool use_stub);
+
+  // Destroys the global instance. Must be called before AtExitManager is
+  // destroyed to ensure a clean shutdown.
+  static void Shutdown();
+
+  // Gets the global instance. Returns NULL if Initialize() has not been
+  // called (or Shutdown() has been called).
   static CrosLibrary* Get();
 
-  BrightnessLibrary* GetBrightnessLibrary();
   BurnLibrary* GetBurnLibrary();
+  CertLibrary* GetCertLibrary();
   CryptohomeLibrary* GetCryptohomeLibrary();
-  InputMethodLibrary* GetInputMethodLibrary();
-  KeyboardLibrary* GetKeyboardLibrary();
-  LoginLibrary* GetLoginLibrary();
-  MountLibrary* GetMountLibrary();
   NetworkLibrary* GetNetworkLibrary();
-  PowerLibrary* GetPowerLibrary();
-  ScreenLockLibrary* GetScreenLockLibrary();
-  SpeechSynthesisLibrary* GetSpeechSynthesisLibrary();
-  SyslogsLibrary* GetSyslogsLibrary();
-  SystemLibrary* GetSystemLibrary();
-  TouchpadLibrary* GetTouchpadLibrary();
-  UpdateLibrary* GetUpdateLibrary();
 
   // Getter for Test API that gives access to internal members of this class.
   TestApi* GetTestApi();
 
-  // Ensures that the library is loaded, loading it if needed. If the library
-  // could not be loaded, returns false.
-  bool EnsureLoaded();
+  bool libcros_loaded() { return libcros_loaded_; }
 
   // Returns an unlocalized string describing the last load error (if any).
   const std::string& load_error_string() {
@@ -114,9 +81,11 @@ class CrosLibrary {
   friend struct base::DefaultLazyInstanceTraits<chromeos::CrosLibrary>;
   friend class CrosLibrary::TestApi;
 
-  CrosLibrary();
+  explicit CrosLibrary(bool use_stub);
   virtual ~CrosLibrary();
 
+  bool LoadLibcros();
+
   LibraryLoader* library_loader_;
 
   bool own_library_loader_;
@@ -136,9 +105,6 @@ class CrosLibrary {
     L* GetDefaultImpl(bool use_stub_impl) {
       if (!library_) {
         own_ = true;
-        if (CommandLine::ForCurrentProcess()->HasSwitch(
-            switches::kForceStubLibcros))
-          use_stub_impl = true;
         library_ = L::GetImpl(use_stub_impl);
       }
       return library_;
@@ -158,26 +124,15 @@ class CrosLibrary {
     bool own_;
   };
 
-  Library<BrightnessLibrary> brightness_lib_;
   Library<BurnLibrary> burn_lib_;
+  Library<CertLibrary> cert_lib_;
   Library<CryptohomeLibrary> crypto_lib_;
-  Library<KeyboardLibrary> keyboard_lib_;
-  Library<InputMethodLibrary> input_method_lib_;
-  Library<LoginLibrary> login_lib_;
-  Library<MountLibrary> mount_lib_;
   Library<NetworkLibrary> network_lib_;
-  Library<PowerLibrary> power_lib_;
-  Library<ScreenLockLibrary> screen_lock_lib_;
-  Library<SpeechSynthesisLibrary> speech_synthesis_lib_;
-  Library<SyslogsLibrary> syslogs_lib_;
-  Library<SystemLibrary> system_lib_;
-  Library<TouchpadLibrary> touchpad_lib_;
-  Library<UpdateLibrary> update_lib_;
 
   // Stub implementations of the libraries should be used.
   bool use_stub_impl_;
   // True if libcros was successfully loaded.
-  bool loaded_;
+  bool libcros_loaded_;
   // True if the last load attempt had an error.
   bool load_error_;
   // Contains the error string from the last load attempt.
@@ -192,11 +147,11 @@ class CrosLibrary {
 class ScopedStubCrosEnabler {
  public:
   ScopedStubCrosEnabler() {
-    chromeos::CrosLibrary::Get()->GetTestApi()->SetUseStubImpl();
+    chromeos::CrosLibrary::Initialize(true);
   }
 
   ~ScopedStubCrosEnabler() {
-    chromeos::CrosLibrary::Get()->GetTestApi()->ResetUseStubImpl();
+    chromeos::CrosLibrary::Shutdown();
   }
 
  private:
