@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,21 +7,14 @@
 
 #include "chrome/browser/chromeos/cros/cros_library.h"
 #include "chrome/browser/chromeos/cros/network_library.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "third_party/cros/chromeos_input_method.h"
+#include "chrome/test/base/in_process_browser_test.h"
 
 namespace chromeos {
 
 class MockCryptohomeLibrary;
 class MockKeyboardLibrary;
-class MockInputMethodLibrary;
 class MockLibraryLoader;
 class MockNetworkLibrary;
-class MockPowerLibrary;
-class MockScreenLockLibrary;
-class MockSpeechSynthesisLibrary;
-class MockSystemLibrary;
-class MockTouchpadLibrary;
 
 // Class for initializing mocks for some parts of CrosLibrary. Once you mock
 // part of CrosLibrary it will be considered as successfully loaded and
@@ -34,7 +27,7 @@ class CrosMock {
   virtual ~CrosMock();
 
   // This method sets up basic mocks that are used by status area items:
-  // LibraryLoader, Language, Network, Power, Touchpad libraries.
+  // LibraryLoader, Language, Network, Power, libraries.
   // Add a call to this method at the beginning of your
   // SetUpInProcessBrowserTestFixture.
   void InitStatusAreaMocks();
@@ -46,24 +39,12 @@ class CrosMock {
   // Initialization of mocks.
   void InitMockCryptohomeLibrary();
   void InitMockKeyboardLibrary();
-  void InitMockInputMethodLibrary();
   void InitMockNetworkLibrary();
-  void InitMockPowerLibrary();
-  void InitMockScreenLockLibrary();
-  void InitMockSpeechSynthesisLibrary();
-  void InitMockTouchpadLibrary();
-  void InitMockSystemLibrary();
 
   // Get mocks.
   MockCryptohomeLibrary* mock_cryptohome_library();
   MockKeyboardLibrary* mock_keyboard_library();
-  MockInputMethodLibrary* mock_input_method_library();
   MockNetworkLibrary* mock_network_library();
-  MockPowerLibrary* mock_power_library();
-  MockScreenLockLibrary* mock_screen_lock_library();
-  MockSpeechSynthesisLibrary* mock_speech_synthesis_library();
-  MockSystemLibrary* mock_system_library();
-  MockTouchpadLibrary* mock_touchpad_library();
 
   // This method sets up corresponding expectations for basic mocks that
   // are used by status area items.
@@ -75,14 +56,7 @@ class CrosMock {
 
   // Methods to setup minimal mocks expectations for status area.
   void SetKeyboardLibraryStatusAreaExpectations();
-  void SetInputMethodLibraryStatusAreaExpectations();
   void SetNetworkLibraryStatusAreaExpectations();
-  void SetPowerLibraryStatusAreaExpectations();
-  void SetPowerLibraryExpectations();
-  void SetSpeechSynthesisLibraryExpectations();
-  void SetSystemLibraryStatusAreaExpectations();
-  void SetSystemLibraryExpectations();
-  void SetTouchpadLibraryExpectations();
 
   void TearDownMocks();
 
@@ -94,19 +68,11 @@ class CrosMock {
   MockLibraryLoader* loader_;
   MockCryptohomeLibrary* mock_cryptohome_library_;
   MockKeyboardLibrary* mock_keyboard_library_;
-  MockInputMethodLibrary* mock_input_method_library_;
   MockNetworkLibrary* mock_network_library_;
-  MockPowerLibrary* mock_power_library_;
-  MockScreenLockLibrary* mock_screen_lock_library_;
-  MockSpeechSynthesisLibrary* mock_speech_synthesis_library_;
-  MockSystemLibrary* mock_system_library_;
-  MockTouchpadLibrary* mock_touchpad_library_;
 
-  ImePropertyList ime_properties_;
-  WifiNetwork wifi_network_;
   WifiNetworkVector wifi_networks_;
-  CellularNetwork cellular_network_;
   CellularNetworkVector cellular_networks_;
+  VirtualNetworkVector virtual_networks_;
   std::string empty_string_;
 
   DISALLOW_COPY_AND_ASSIGN(CrosMock);
