@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,7 @@
 
 #include <string>
 
-#include "base/singleton.h"
+#include "base/memory/singleton.h"
 #include "chrome/browser/chromeos/cros/cros_library.h"
 #include "third_party/cros/chromeos_cryptohome.h"
 
@@ -25,12 +25,10 @@ class CryptohomeLibrary {
     virtual void OnComplete(bool success, int return_code) = 0;
   };
 
-  virtual ~CryptohomeLibrary() {}
+  CryptohomeLibrary();
+  virtual ~CryptohomeLibrary();
 
-  // Asks cryptohomed to try to find the cryptohome for |user_email| and then
-  // use |passhash| to unlock the key.
-  virtual bool CheckKey(const std::string& user_email,
-                        const std::string& passhash) = 0;
+  virtual void Init() = 0;
 
   // Asks cryptohomed to asynchronously try to find the cryptohome for
   // |user_email| and then use |passhash| to unlock the key.
@@ -40,12 +38,6 @@ class CryptohomeLibrary {
                              const std::string& passhash,
                              Delegate* callback) = 0;
 
-  // Asks cryptohomed to try to find the cryptohome for |user_email| and then
-  // change from using |old_hash| to lock the key to using |new_hash|.
-  virtual bool MigrateKey(const std::string& user_email,
-                          const std::string& old_hash,
-                          const std::string& new_hash) = 0;
-
   // Asks cryptohomed to asynchronously try to find the cryptohome for
   // |user_email| and then change from using |old_hash| to lock the
   // key to using |new_hash|.
@@ -56,12 +48,6 @@ class CryptohomeLibrary {
                                const std::string& new_hash,
                                Delegate* callback) = 0;
 
-  // Asks cryptohomed to try to find the cryptohome for |user_email| and then
-  // mount it using |passhash| to unlock the key.
-  virtual bool Mount(const std::string& user_email,
-                     const std::string& passhash,
-                     int* error_code) = 0;
-
   // Asks cryptohomed to asynchronously try to find the cryptohome for
   // |user_email| and then mount it using |passhash| to unlock the key.
   // |create_if_missing| controls whether or not we ask cryptohomed to
@@ -76,22 +62,11 @@ class CryptohomeLibrary {
                           const bool create_if_missing,
                           Delegate* callback) = 0;
 
-  // Asks cryptohomed to mount a tmpfs for BWSI mode.
-  virtual bool MountForBwsi(int* error_code) = 0;
-
   // Asks cryptohomed to asynchronously to mount a tmpfs for BWSI mode.
   // Returns true if the attempt is successfully initiated.
   // d->OnComplete() will be called with status info on completion.
   virtual bool AsyncMountForBwsi(Delegate* callback) = 0;
 
-  // Asks cryptohomed to unmount the currently mounted cryptohome.
-  // Returns false if the cryptohome could not be unmounted, true otherwise.
-  virtual bool Unmount() = 0;
-
-  // Asks cryptohomed to try to find the cryptohome for |user_email| and then
-  // nuke it.
-  virtual bool Remove(const std::string& user_email) = 0;
-
   // Asks cryptohomed to asynchronously try to find the cryptohome for
   // |user_email| and then nuke it.
   virtual bool AsyncRemove(const std::string& user_email,
@@ -100,9 +75,10 @@ class CryptohomeLibrary {
   // Asks cryptohomed if a drive is currently mounted.
   virtual bool IsMounted() = 0;
 
-  // Asks cryptohomed for the system salt.
-  virtual CryptohomeBlob GetSystemSalt() = 0;
-
+  // Passes cryptohomed the owner user. It is used to prevent
+  // deletion of the owner in low disk space cleanup (see above).
+  virtual bool AsyncSetOwnerUser(const std::string& username,
+                                 Delegate* callback) = 0;
 
   // Wrappers of the functions for working with Tpm.
 
@@ -129,6 +105,32 @@ class CryptohomeLibrary {
   // shown to user.
   virtual void TpmClearStoredPassword() = 0;
 
+  virtual bool InstallAttributesGet(const std::string& name,
+                                    std::string* value) = 0;
+  virtual bool InstallAttributesSet(const std::string& name,
+                                    const std::string& value) = 0;
+  virtual bool InstallAttributesFinalize() = 0;
+  virtual bool InstallAttributesIsReady() = 0;
+  virtual bool InstallAttributesIsInvalid() = 0;
+  virtual bool InstallAttributesIsFirstInstall() = 0;
+
+  // Get the PKCS#11 token info from the TPM.  This is different from
+  // the TpmGetPassword because it's getting the PKCS#11 user PIN and
+  // not the TPM password.
+  virtual void Pkcs11GetTpmTokenInfo(std::string* label,
+                                     std::string* user_pin) = 0;
+
+  // Gets the status of the TPM.  This is different from TpmIsReady
+  // because it's getting the staus of the PKCS#11 initialization of
+  // the TPM token, not the TPM itself.
+  virtual bool Pkcs11IsTpmTokenReady() = 0;
+
+  // Returns hash of |password|, salted with the system salt.
+  virtual std::string HashPassword(const std::string& password) = 0;
+
+  // Returns system hash in hex encoded ascii format.
+  virtual std::string GetSystemSalt() = 0;
+
   // Factory function, creates a new instance and returns ownership.
   // For normal usage, access the singleton via CrosLibrary::Get().
   static CryptohomeLibrary* GetImpl(bool stub);
