@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,12 +15,22 @@ extern const char kAccountsPrefAllowNewUser[];
 extern const char kAccountsPrefShowUserNamesOnSignIn[];
 extern const char kAccountsPrefUsers[];
 
+extern const char kSettingProxyEverywhere[];
+
+extern const char kSignedDataRoamingEnabled[];
+
 extern const char kSystemTimezone[];
 
 extern const char kDeviceOwner[];
 
 extern const char kStatsReportingPref[];
 
+extern const char kReleaseChannel[];
+
+extern const char kReportDeviceVersionInfo[];
+extern const char kReportDeviceActivityTimes[];
+extern const char kReportDeviceBootMode[];
+
 }  // namespace chromeos
 
 #endif  // CHROME_BROWSER_CHROMEOS_CROS_SETTINGS_NAMES_H_
