@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,20 +8,26 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <unistd.h>
+
+#include <string>
 
 #include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/eintr_wrapper.h"
 #include "base/metrics/histogram.h"
 #include "base/perftimer.h"
 #include "base/time.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/metrics/metrics_service.h"
-#include "chrome/browser/metrics/user_metrics.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/user_metrics.h"
+
+using content::BrowserThread;
+using content::UserMetricsAction;
 
 namespace chromeos {
 
@@ -32,6 +38,8 @@ ExternalMetrics::ExternalMetrics()
     : test_recorder_(NULL) {
 }
 
+ExternalMetrics::~ExternalMetrics() {}
+
 void ExternalMetrics::Start() {
   // Register user actions external to the browser.
   // chrome/tools/extract_actions.py won't understand these lines, so all of
@@ -44,13 +52,16 @@ void ExternalMetrics::Start() {
   valid_user_actions_.insert("Accel_PrevWindow_F5");
   valid_user_actions_.insert("Accel_BrightnessDown_F6");
   valid_user_actions_.insert("Accel_BrightnessUp_F7");
+  valid_user_actions_.insert("Cryptohome.PKCS11InitFail");
+  valid_user_actions_.insert("Updater.ServerCertificateChanged");
+  valid_user_actions_.insert("Updater.ServerCertificateFailed");
 
   ScheduleCollector();
 }
 
 void ExternalMetrics::RecordActionUI(std::string action_string) {
   if (valid_user_actions_.count(action_string)) {
-    UserMetrics::RecordComputedAction(action_string);
+    content::RecordComputedAction(action_string);
   } else {
     LOG(ERROR) << "undefined UMA action: " << action_string;
   }
@@ -60,7 +71,7 @@ void ExternalMetrics::RecordAction(const char* action) {
   std::string action_string(action);
   BrowserThread::PostTask(
       BrowserThread::UI, FROM_HERE,
-      NewRunnableMethod(this, &ExternalMetrics::RecordActionUI, action_string));
+      base::Bind(&ExternalMetrics::RecordActionUI, this, action_string));
 }
 
 void ExternalMetrics::RecordCrashUI(const std::string& crash_kind) {
@@ -72,7 +83,7 @@ void ExternalMetrics::RecordCrashUI(const std::string& crash_kind) {
 void ExternalMetrics::RecordCrash(const std::string& crash_kind) {
   BrowserThread::PostTask(
       BrowserThread::UI, FROM_HERE,
-      NewRunnableMethod(this, &ExternalMetrics::RecordCrashUI, crash_kind));
+      base::Bind(&ExternalMetrics::RecordCrashUI, this, crash_kind));
 }
 
 void ExternalMetrics::RecordHistogram(const char* histogram_data) {
@@ -86,7 +97,7 @@ void ExternalMetrics::RecordHistogram(const char* histogram_data) {
   }
   // Do not use the UMA_HISTOGRAM_... macros here.  They cache the Histogram
   // instance and thus only work if |name| is constant.
-  scoped_refptr<base::Histogram> counter = base::Histogram::FactoryGet(
+  base::Histogram* counter = base::Histogram::FactoryGet(
       name, min, max, nbuckets, base::Histogram::kUmaTargetedHistogramFlag);
   counter->Add(sample);
 }
@@ -101,7 +112,7 @@ void ExternalMetrics::RecordLinearHistogram(const char* histogram_data) {
   }
   // Do not use the UMA_HISTOGRAM_... macros here.  They cache the Histogram
   // instance and thus only work if |name| is constant.
-  scoped_refptr<base::Histogram> counter = base::LinearHistogram::FactoryGet(
+  base::Histogram* counter = base::LinearHistogram::FactoryGet(
       name, 1, max, max + 1, base::Histogram::kUmaTargetedHistogramFlag);
   counter->Add(sample);
 }
@@ -227,8 +238,8 @@ void ExternalMetrics::CollectEventsAndReschedule() {
 void ExternalMetrics::ScheduleCollector() {
   bool result;
   result = BrowserThread::PostDelayedTask(
-    BrowserThread::FILE, FROM_HERE, NewRunnableMethod(
-        this, &chromeos::ExternalMetrics::CollectEventsAndReschedule),
+    BrowserThread::FILE, FROM_HERE,
+    base::Bind(&chromeos::ExternalMetrics::CollectEventsAndReschedule, this),
     kExternalMetricsCollectionIntervalMs);
   DCHECK(result);
 }
