@@ -1,62 +1,62 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/chromeos/frame/bubble_frame_view.h"
 
-#include "gfx/canvas_skia.h"
-#include "gfx/font.h"
-#include "gfx/insets.h"
-#include "gfx/path.h"
-#include "gfx/rect.h"
+#include <algorithm>
+
+#include "base/utf_string_conversions.h"
 #include "chrome/browser/chromeos/frame/bubble_window.h"
 #include "chrome/browser/chromeos/login/helper.h"
-#include "chrome/browser/ui/views/bubble_border.h"
-#include "grit/theme_resources.h"
-#include "ui/base/resource/resource_bundle.h"
-#include "views/controls/button/image_button.h"
-#include "views/controls/label.h"
-#include "views/controls/throbber.h"
-#include "views/window/hit_test.h"
-#include "views/window/window.h"
-#include "views/window/window_delegate.h"
+#include "grit/ui_resources_standard.h"
+#include "third_party/skia/include/core/SkColor.h"
 #include "third_party/skia/include/core/SkPaint.h"
+#include "ui/base/hit_test.h"
+#include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/font.h"
+#include "ui/gfx/insets.h"
+#include "ui/gfx/path.h"
+#include "ui/gfx/rect.h"
+#include "ui/views/controls/button/image_button.h"
+#include "ui/views/controls/label.h"
+#include "ui/views/controls/throbber.h"
+#include "ui/views/widget/widget.h"
+#include "ui/views/widget/widget_delegate.h"
 
 namespace {
 
-static const int kTitleTopPadding = 10;
-static const int kTitleContentPadding = 10;
-static const int kHorizontalPadding = 10;
+const int kTitleTopPadding = 10;
+const int kTitleContentPadding = 10;
+const int kHorizontalPadding = 10;
 
 // Title font size correction.
 #if defined(CROS_FONTS_USING_BCI)
-static const int kTitleFontSizeDelta = 0;
+const int kTitleFontSizeDelta = 0;
 #else
-static const int kTitleFontSizeDelta = 1;
+const int kTitleFontSizeDelta = 1;
 #endif
 
 }  // namespace
 
 namespace chromeos {
 
-BubbleFrameView::BubbleFrameView(views::Window* frame,
-                                 BubbleWindow::Style style)
-    : frame_(frame),
-      style_(style),
+BubbleFrameView::BubbleFrameView(views::WidgetDelegate* widget_delegate,
+                                 DialogStyle style)
+    : style_(style),
       title_(NULL),
       close_button_(NULL),
       throbber_(NULL) {
-  set_border(new BubbleBorder(BubbleBorder::NONE));
-
-  if (frame_->GetDelegate()->ShouldShowWindowTitle()) {
-    title_ = new views::Label(frame_->GetDelegate()->GetWindowTitle());
+  if (widget_delegate->ShouldShowWindowTitle()) {
+    title_ = new views::Label(widget_delegate->GetWindowTitle());
     title_->SetHorizontalAlignment(views::Label::ALIGN_LEFT);
     title_->SetFont(title_->font().DeriveFont(kFontSizeCorrectionDelta,
                                               gfx::Font::BOLD));
     AddChildView(title_);
   }
 
-  if (style_ & BubbleWindow::STYLE_XBAR) {
+  if (style_ & STYLE_XBAR) {
     ResourceBundle& rb = ResourceBundle::GetSharedInstance();
     close_button_ = new views::ImageButton(this);
     close_button_->SetImage(views::CustomButton::BS_NORMAL,
@@ -68,7 +68,7 @@ BubbleFrameView::BubbleFrameView(views::Window* frame,
     AddChildView(close_button_);
   }
 
-  if (style_ & BubbleWindow::STYLE_THROBBER) {
+  if (style_ & STYLE_THROBBER) {
     throbber_ = CreateDefaultSmoothedThrobber();
     AddChildView(throbber_);
   }
@@ -80,7 +80,7 @@ BubbleFrameView::~BubbleFrameView() {
 void BubbleFrameView::StartThrobber() {
   DCHECK(throbber_ != NULL);
   if (title_)
-    title_->SetText(std::wstring());
+    title_->SetText(string16());
   throbber_->Start();
 }
 
@@ -88,7 +88,7 @@ void BubbleFrameView::StopThrobber() {
   DCHECK(throbber_ != NULL);
   throbber_->Stop();
   if (title_)
-    title_->SetText(frame_->GetDelegate()->GetWindowTitle());
+    title_->SetText(GetWidget()->widget_delegate()->GetWindowTitle());
 }
 
 gfx::Rect BubbleFrameView::GetBoundsForClientView() const {
@@ -131,28 +131,25 @@ void BubbleFrameView::GetWindowMask(const gfx::Size& size,
                                     gfx::Path* window_mask) {
 }
 
-void BubbleFrameView::EnableClose(bool enable) {
-}
-
 void BubbleFrameView::ResetWindowControls() {
 }
 
-gfx::Insets BubbleFrameView::GetInsets() const {
-  gfx::Insets border_insets;
-  border()->GetInsets(&border_insets);
+void BubbleFrameView::UpdateWindowIcon() {
+}
 
-  gfx::Insets insets(kTitleTopPadding,
+gfx::Insets BubbleFrameView::GetInsets() const {
+  return (style_ & STYLE_FLUSH || style_ & STYLE_FLUSH_CONTENT) ?
+         gfx::Insets() :
+         gfx::Insets(kTitleTopPadding,
                      kHorizontalPadding,
                      0,
                      kHorizontalPadding);
-  insets += border_insets;
-  return insets;
 }
 
 gfx::Size BubbleFrameView::GetPreferredSize() {
-  gfx::Size pref = frame_->GetClientView()->GetPreferredSize();
+  gfx::Size pref = GetWidget()->client_view()->GetPreferredSize();
   gfx::Rect bounds(0, 0, pref.width(), pref.height());
-  return frame_->GetNonClientView()->GetWindowBoundsForClientBounds(
+  return GetWidget()->non_client_view()->GetWindowBoundsForClientBounds(
       bounds).size();
 }
 
@@ -169,27 +166,49 @@ void BubbleFrameView::Layout() {
   if (throbber_)
     throbber_size = throbber_->GetPreferredSize();
 
+  // Need to center elements which are shorter.
+  int max_height = std::max(title_size.height(),
+                            std::max(close_button_size.height(),
+                                     throbber_size.height()));
+
+  gfx::Insets title_insets = gfx::Insets();
+  // Need to insert title padding for STYLE_FLUSH_CONTENT.
+  if (style_ & STYLE_FLUSH_CONTENT)
+    title_insets = gfx::Insets(kTitleTopPadding,
+                               kHorizontalPadding,
+                               0,
+                               kHorizontalPadding);
+
   if (title_) {
     title_->SetBounds(
-        insets.left(), insets.top(),
-        std::max(0, width() - insets.width() - close_button_size.width()),
+        insets.left() + title_insets.left(),
+        insets.top() + title_insets.top() +
+            (max_height - title_size.height())/2, // Center.
+        std::max(0, width() - insets.width() - title_insets.width() -
+            close_button_size.width()),
         title_size.height());
   }
 
   if (close_button_) {
     close_button_->SetBounds(
-        width() - insets.right() - close_button_size.width(), insets.top(),
-        close_button_size.width(), close_button_size.height());
+        width() - insets.right() - title_insets.right() -
+            close_button_size.width(),
+        insets.top() + title_insets.top() +
+            (max_height - close_button_size.height())/2,
+        close_button_size.width(),
+        close_button_size.height());
   }
 
   if (throbber_) {
     throbber_->SetBounds(
-        insets.left(), insets.top(),
+        insets.left() + title_insets.left(),
+        insets.top() + title_insets.top() +
+            (max_height - throbber_size.height())/2,
         std::min(throbber_size.width(), width()),
         throbber_size.height());
   }
 
-  int top_height = insets.top();
+  int top_height = insets.top() + title_insets.top();
   if (title_size.height() > 0 ||
       close_button_size.height() > 0 ||
       throbber_size.height() > 0) {
@@ -202,29 +221,23 @@ void BubbleFrameView::Layout() {
       std::max(0, height() - top_height - insets.bottom()));
 }
 
-void BubbleFrameView::Paint(gfx::Canvas* canvas) {
-  // The border of this view creates an anti-aliased round-rect region for the
-  // contents, which we need to fill with the background color.
+void BubbleFrameView::OnPaint(gfx::Canvas* canvas) {
   SkPaint paint;
-  paint.setAntiAlias(true);
   paint.setStyle(SkPaint::kFill_Style);
-  paint.setColor(BubbleWindow::kBackgroundColor);
+  paint.setColor(kBubbleWindowBackgroundColor);
   gfx::Path path;
-  gfx::Rect bounds(GetLocalBounds(false));
+  gfx::Rect bounds(GetContentsBounds());
   SkRect rect;
   rect.set(SkIntToScalar(bounds.x()), SkIntToScalar(bounds.y()),
            SkIntToScalar(bounds.right()), SkIntToScalar(bounds.bottom()));
-  SkScalar radius = SkIntToScalar(BubbleBorder::GetCornerRadius());
-  path.addRoundRect(rect, radius, radius);
-  canvas->AsCanvasSkia()->drawPath(path, paint);
-
-  PaintBorder(canvas);
+  path.addRect(rect);
+  canvas->GetSkCanvas()->drawPath(path, paint);
 }
 
 void BubbleFrameView::ButtonPressed(views::Button* sender,
                                     const views::Event& event) {
   if (close_button_ != NULL && sender == close_button_)
-    frame_->Close();
+    GetWidget()->Close();
 }
 
 }  // namespace chromeos
