@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,8 +7,8 @@
 #pragma once
 
 #include "chrome/browser/chromeos/frame/bubble_window.h"
-#include "views/controls/button/button.h"
-#include "views/window/non_client_view.h"
+#include "ui/views/controls/button/button.h"
+#include "ui/views/window/non_client_view.h"
 
 namespace gfx {
 class Insets;
@@ -21,7 +21,8 @@ class Size;
 namespace views {
 class ImageButton;
 class Label;
-class Window;
+class Throbber;
+class Widget;
 }
 
 namespace chromeos {
@@ -30,37 +31,36 @@ namespace chromeos {
 class BubbleFrameView : public views::NonClientFrameView,
                         public views::ButtonListener {
  public:
-  BubbleFrameView(views::Window* frame, BubbleWindow::Style style);
+  BubbleFrameView(views::WidgetDelegate* widget_delegate,
+                  DialogStyle style);
   virtual ~BubbleFrameView();
 
   // Overridden from views::NonClientFrameView:
-  virtual gfx::Rect GetBoundsForClientView() const;
+  virtual gfx::Rect GetBoundsForClientView() const OVERRIDE;
   virtual gfx::Rect GetWindowBoundsForClientBounds(
-      const gfx::Rect& client_bounds) const;
-  virtual int NonClientHitTest(const gfx::Point& point);
-  virtual void GetWindowMask(const gfx::Size& size, gfx::Path* window_mask);
-  virtual void EnableClose(bool enable);
-  virtual void ResetWindowControls();
+      const gfx::Rect& client_bounds) const OVERRIDE;
+  virtual int NonClientHitTest(const gfx::Point& point) OVERRIDE;
+  virtual void GetWindowMask(const gfx::Size& size, gfx::Path* window_mask)
+      OVERRIDE;
+  virtual void ResetWindowControls() OVERRIDE;
+  virtual void UpdateWindowIcon() OVERRIDE;
 
   // View overrides:
-  virtual gfx::Insets GetInsets() const;
-  virtual gfx::Size GetPreferredSize();
-  virtual void Layout();
-  virtual void Paint(gfx::Canvas* canvas);
+  virtual gfx::Insets GetInsets() const OVERRIDE;
+  virtual gfx::Size GetPreferredSize() OVERRIDE;
+  virtual void Layout() OVERRIDE;
+  virtual void OnPaint(gfx::Canvas* canvas) OVERRIDE;
 
   // Overridden from views::ButtonListener:
-  virtual void ButtonPressed(views::Button* sender,
-                             const views::Event& event);
+  virtual void ButtonPressed(views::Button* sender, const views::Event& event)
+      OVERRIDE;
 
   void StartThrobber();
   void StopThrobber();
 
  private:
-  // The window that owns this view.
-  views::Window* frame_;
-
   // Allows to tweak appearance of the view.
-  BubbleWindow::Style style_;
+  DialogStyle style_;
 
   // Title label
   views::Label* title_;
@@ -80,4 +80,3 @@ class BubbleFrameView : public views::NonClientFrameView,
 }  // namespace chromeos
 
 #endif  // CHROME_BROWSER_CHROMEOS_FRAME_BUBBLE_FRAME_VIEW_H_
-
