@@ -1,13 +1,13 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/chromeos/frame/panel_browser_view.h"
 
+#include "chrome/browser/chromeos/frame/browser_view.h"
 #include "chrome/browser/chromeos/frame/panel_controller.h"
-#include "third_party/cros/chromeos_wm_ipc_enums.h"
-#include "views/widget/widget.h"
-#include "views/window/window.h"
+#include "third_party/cros_system_api/window_manager/chromeos_wm_ipc_enums.h"
+#include "ui/views/widget/widget.h"
 
 namespace {
 
@@ -27,6 +27,8 @@ PanelBrowserView::PanelBrowserView(Browser* browser)
       creator_xid_(0) {
 }
 
+PanelBrowserView::~PanelBrowserView() {}
+
 ////////////////////////////////////////////////////////////////////////////////
 // PanelBrowserView functions
 
@@ -56,13 +58,22 @@ void PanelBrowserView::LimitBounds(gfx::Rect* bounds) const {
 // BrowserView overrides.
 
 void PanelBrowserView::Show() {
+  InitPanelController(true);  // focus when opened
+  ::BrowserView::Show();
+}
+
+void PanelBrowserView::ShowInactive() {
+  InitPanelController(false);
+  ::BrowserView::ShowInactive();
+}
+
+void PanelBrowserView::InitPanelController(bool is_active) {
   if (panel_controller_.get() == NULL) {
     panel_controller_.reset(new PanelController(this, GetNativeHandle()));
     panel_controller_->Init(
-        true /* focus when opened */, bounds(), creator_xid_,
+        is_active, bounds(), creator_xid_,
         WM_IPC_PANEL_USER_RESIZE_HORIZONTALLY_AND_VERTICALLY);
   }
-  ::BrowserView::Show();
 }
 
 void PanelBrowserView::SetBounds(const gfx::Rect& bounds) {
@@ -77,20 +88,19 @@ void PanelBrowserView::Close() {
     panel_controller_->Close();
 }
 
+void PanelBrowserView::FlashFrame(bool flash) {
+  if (panel_controller_.get())
+    panel_controller_->SetUrgent(flash);
+}
+
 void PanelBrowserView::UpdateTitleBar() {
   ::BrowserView::UpdateTitleBar();
   if (panel_controller_.get())
     panel_controller_->UpdateTitleBar();
 }
 
-void PanelBrowserView::ActivationChanged(bool activated) {
-  ::BrowserView::ActivationChanged(activated);
-  if (panel_controller_.get()) {
-    if (activated)
-      panel_controller_->OnFocusIn();
-    else
-      panel_controller_->OnFocusOut();
-  }
+bool PanelBrowserView::IsPanel() const {
+  return true;
 }
 
 void PanelBrowserView::SetCreatorView(PanelBrowserView* creator) {
@@ -99,11 +109,48 @@ void PanelBrowserView::SetCreatorView(PanelBrowserView* creator) {
   creator_xid_ = ui::GetX11WindowFromGtkWidget(GTK_WIDGET(window));
 }
 
-bool PanelBrowserView::GetSavedWindowBounds(gfx::Rect* bounds) const {
-  bool res = ::BrowserView::GetSavedWindowBounds(bounds);
-  if (res)
+WindowOpenDisposition PanelBrowserView::GetDispositionForPopupBounds(
+    const gfx::Rect& bounds) {
+  GdkScreen* screen = gdk_screen_get_default();
+  int width = gdk_screen_get_width(screen);
+  int height = gdk_screen_get_height(screen);
+  return browser::DispositionForPopupBounds(bounds, width, height);
+}
+
+bool PanelBrowserView::GetSavedWindowPlacement(
+    gfx::Rect* bounds,
+    ui::WindowShowState* show_state) const {
+  bool result = ::BrowserView::GetSavedWindowPlacement(bounds, show_state);
+  if (result) {
     LimitBounds(bounds);
-  return res;
+    // Panels have no maximized state.
+    *show_state = ui::SHOW_STATE_NORMAL;
+  }
+  return result;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// views::Widget::Observer overrides.
+
+void PanelBrowserView::OnWidgetActivationChanged(views::Widget* widget,
+                                                 bool active) {
+  ::BrowserView::OnWidgetActivationChanged(widget, active);
+  if (panel_controller_.get()) {
+    if (active)
+      panel_controller_->OnFocusIn();
+    else
+      panel_controller_->OnFocusOut();
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// TabStripModelObserver overrides.
+
+void PanelBrowserView::TabChangedAt(TabContentsWrapper* contents,
+                                    int index,
+                                    TabChangeType change_type) {
+  if (change_type == TabStripModelObserver::TITLE_NOT_LOADING)
+    panel_controller_->SetUrgent(true);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -125,4 +172,8 @@ void PanelBrowserView::ClosePanel() {
   Close();
 }
 
+void PanelBrowserView::ActivatePanel() {
+  Activate();
+}
+
 }  // namespace chromeos
