@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 #define CHROME_BROWSER_CHROMEOS_FRAME_PANEL_BROWSER_VIEW_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/frame/panel_controller.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
 #include "ui/base/x/x11_util.h"
@@ -24,27 +24,47 @@ class PanelBrowserView : public ::BrowserView,
                          public PanelController::Delegate {
  public:
   explicit PanelBrowserView(Browser* browser);
+  virtual ~PanelBrowserView();
 
   // BrowserView overrides.
-  virtual void Show();
-  virtual void SetBounds(const gfx::Rect& bounds);
-  virtual void Close();
-  virtual void UpdateTitleBar();
-  virtual void ActivationChanged(bool activated);
-  virtual void SetCreatorView(PanelBrowserView* creator);
-  virtual bool GetSavedWindowBounds(gfx::Rect* bounds) const;
+  virtual void Show() OVERRIDE;
+  virtual void ShowInactive() OVERRIDE;
+  virtual void SetBounds(const gfx::Rect& bounds) OVERRIDE;
+  virtual void Close() OVERRIDE;
+  virtual void FlashFrame(bool flash) OVERRIDE;
+  virtual void UpdateTitleBar() OVERRIDE;
+  virtual bool IsPanel() const OVERRIDE;
+  virtual WindowOpenDisposition GetDispositionForPopupBounds(
+      const gfx::Rect& bounds) OVERRIDE;
+  virtual bool GetSavedWindowPlacement(
+      gfx::Rect* bounds,
+      ui::WindowShowState* show_state) const OVERRIDE;
 
-  // PanelController::Delegate overrides
-  virtual string16 GetPanelTitle();
-  virtual SkBitmap GetPanelIcon();
-  virtual bool CanClosePanel();
-  virtual void ClosePanel();
-  virtual void OnPanelStateChanged(PanelController::State state) {}
+  // views::Widget::Observer overrides.
+  virtual void OnWidgetActivationChanged(views::Widget* widget,
+                                         bool active) OVERRIDE;
+
+  // BrowserView : TabStripModelObserver overrides.
+  virtual void TabChangedAt(TabContentsWrapper* contents,
+                            int index,
+                            TabChangeType change_type) OVERRIDE;
+
+  // PanelController::Delegate overrides.
+  virtual string16 GetPanelTitle() OVERRIDE;
+  virtual SkBitmap GetPanelIcon() OVERRIDE;
+  virtual bool CanClosePanel() OVERRIDE;
+  virtual void ClosePanel() OVERRIDE;
+  virtual void ActivatePanel() OVERRIDE;
+
+  // Specific to PanelBrowserView.
+  void SetCreatorView(PanelBrowserView* creator);
 
  private:
   // Enforces the min, max, and default bounds.
   void LimitBounds(gfx::Rect* bounds) const;
 
+  void InitPanelController(bool is_active);
+
   // Controls interactions with the window manager for popup panels.
   scoped_ptr<chromeos::PanelController> panel_controller_;
 
