@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,28 +7,33 @@
 #include <vector>
 
 #include "base/logging.h"
-#include "base/scoped_ptr.h"
-#include "base/singleton.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/singleton.h"
 #include "base/string_util.h"
+#include "base/time.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/chromeos/wm_ipc.h"
-#include "chrome/common/notification_service.h"
-#include "gfx/canvas_skia.h"
-#include "grit/app_resources.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "content/public/browser/notification_service.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
-#include "third_party/cros/chromeos_wm_ipc_enums.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
+#include "third_party/cros_system_api/window_manager/chromeos_wm_ipc_enums.h"
 #include "third_party/skia/include/effects/SkBlurMaskFilter.h"
 #include "third_party/skia/include/effects/SkGradientShader.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/controls/button/image_button.h"
-#include "views/controls/image_view.h"
-#include "views/controls/label.h"
-#include "views/event.h"
-#include "views/painter.h"
-#include "views/view.h"
-#include "views/widget/widget_gtk.h"
-#include "views/window/window.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/views/controls/button/image_button.h"
+#include "ui/views/controls/image_view.h"
+#include "ui/views/controls/label.h"
+#include "ui/views/events/event.h"
+#include "ui/views/painter.h"
+#include "ui/views/view.h"
+#include "ui/views/widget/widget.h"
+
+#if defined(TOOLKIT_USES_GTK)
+#include "chrome/browser/chromeos/legacy_window_manager/wm_ipc.h"
+#endif
 
 namespace chromeos {
 
@@ -51,12 +56,22 @@ const int kTitleCornerRadius = 4;
 const int kTitleCloseButtonPad = 6;
 const SkColor kTitleActiveGradientStart = SK_ColorWHITE;
 const SkColor kTitleActiveGradientEnd = 0xffe7edf1;
-const SkColor kTitleActiveColor = SK_ColorBLACK;
-const SkColor kTitleInactiveColor = SK_ColorBLACK;
+const SkColor kTitleUrgentGradientStart = 0xfffea044;
+const SkColor kTitleUrgentGradientEnd = 0xfffa983a;
+const SkColor kTitleActiveTextColor = SK_ColorBLACK;
+const SkColor kTitleInactiveTextColor = SK_ColorBLACK;
+const SkColor kTitleUrgentTextColor = SK_ColorWHITE;
 const SkColor kTitleCloseButtonColor = SK_ColorBLACK;
+// Delay before the urgency can be set after it has been cleared.
+const base::TimeDelta kSetUrgentDelay = base::TimeDelta::FromMilliseconds(500);
 
 // Used to draw the background of the panel title window.
 class TitleBackgroundPainter : public views::Painter {
+ public:
+  explicit TitleBackgroundPainter(PanelController* controller)
+      : panel_controller_(controller) { }
+
+ private:
   virtual void Paint(int w, int h, gfx::Canvas* canvas) {
     SkRect rect = {0, 0, w, h};
     SkPath path;
@@ -72,13 +87,19 @@ class TitleBackgroundPainter : public views::Painter {
     paint.setFlags(SkPaint::kAntiAlias_Flag);
     SkPoint p[2] = { {0, 0}, {0, h} };
     SkColor colors[2] = {kTitleActiveGradientStart, kTitleActiveGradientEnd};
+    if (panel_controller_->urgent()) {
+      colors[0] = kTitleUrgentGradientStart;
+      colors[1] = kTitleUrgentGradientEnd;
+    }
     SkShader* s = SkGradientShader::CreateLinear(
         p, colors, NULL, 2, SkShader::kClamp_TileMode, NULL);
     paint.setShader(s);
     // Need to unref shader, otherwise never deleted.
     s->unref();
-    canvas->AsCanvasSkia()->drawPath(path, paint);
+    canvas->GetSkCanvas()->drawPath(path, paint);
   }
+
+  PanelController* panel_controller_;
 };
 
 static bool resources_initialized;
@@ -114,7 +135,9 @@ PanelController::PanelController(Delegate* delegate,
        expanded_(true),
        mouse_down_(false),
        dragging_(false),
-       client_event_handler_id_(0) {
+       client_event_handler_id_(0),
+       focused_(false),
+       urgent_(false) {
 }
 
 void PanelController::Init(bool initial_focus,
@@ -123,9 +146,14 @@ void PanelController::Init(bool initial_focus,
                            WmIpcPanelUserResizeType resize_type) {
   gfx::Rect title_bounds(0, 0, window_bounds.width(), kTitleHeight);
 
-  title_window_ = new views::WidgetGtk(views::WidgetGtk::TYPE_WINDOW);
-  title_window_->MakeTransparent();
-  title_window_->Init(NULL, title_bounds);
+  title_window_ = new views::Widget;
+  views::Widget::InitParams params(
+      views::Widget::InitParams::TYPE_WINDOW_FRAMELESS);
+  params.transparent = true;
+  params.bounds = title_bounds;
+  title_window_->Init(params);
+
+#if defined(TOOLKIT_USES_GTK)
   gtk_widget_set_size_request(title_window_->GetNativeView(),
                               title_bounds.width(), title_bounds.height());
   title_ = title_window_->GetNativeView();
@@ -148,6 +176,7 @@ void PanelController::Init(bool initial_focus,
 
   client_event_handler_id_ = g_signal_connect(
       panel_, "client-event", G_CALLBACK(OnPanelClientEvent), this);
+#endif
 
   title_content_ = new TitleContentView(this);
   title_window_->SetContentsView(title_content_);
@@ -158,19 +187,32 @@ void PanelController::Init(bool initial_focus,
 void PanelController::UpdateTitleBar() {
   if (!delegate_ || !title_window_)
     return;
-  DCHECK(title_content_);
-  title_content_->title_label()->SetText(
-      UTF16ToWideHack(delegate_->GetPanelTitle()));
+  title_content_->title_label()->SetText(delegate_->GetPanelTitle());
   title_content_->title_icon()->SetImage(delegate_->GetPanelIcon());
 }
 
-bool PanelController::TitleMousePressed(const views::MouseEvent& event) {
-  if (!event.IsOnlyLeftMouseButton()) {
-    return false;
+void PanelController::SetUrgent(bool urgent) {
+  if (!urgent)
+    urgent_cleared_time_ = base::TimeTicks::Now();
+  if (urgent == urgent_)
+    return;
+  if (urgent && focused_)
+    return;  // Don't set urgency for focused panels.
+  if (urgent && base::TimeTicks::Now() < urgent_cleared_time_ + kSetUrgentDelay)
+    return;  // Don't set urgency immediately after clearing it.
+  urgent_ = urgent;
+  if (title_window_) {
+    gtk_window_set_urgency_hint(panel_, urgent ? TRUE : FALSE);
+    title_content_->title_label()->SetDisabledColor(urgent ?
+        kTitleUrgentTextColor : kTitleInactiveTextColor);
+    title_content_->SchedulePaint();
   }
-  GdkEvent* gdk_event = gtk_get_current_event();
-  if (gdk_event->type != GDK_BUTTON_PRESS) {
-    gdk_event_free(gdk_event);
+}
+
+bool PanelController::TitleMousePressed(const views::MouseEvent& event) {
+  if (!event.IsOnlyLeftMouseButton())
+    return false;
+  if (event.type() != ui::ET_MOUSE_PRESSED) {
     NOTREACHED();
     return false;
   }
@@ -182,73 +224,99 @@ bool PanelController::TitleMousePressed(const views::MouseEvent& event) {
   gint title_width = 1;
   gtk_window_get_size(GTK_WINDOW(title_), &title_width, NULL);
 
-  GdkEventButton last_button_event = gdk_event->button;
   mouse_down_ = true;
-  mouse_down_abs_x_ = last_button_event.x_root;
-  mouse_down_abs_y_ = last_button_event.y_root;
   mouse_down_offset_x_ = event.x() - title_width;
   mouse_down_offset_y_ = event.y();
   dragging_ = false;
-  gdk_event_free(gdk_event);
+
+#if defined(TOOLKIT_USES_GTK)
+  const GdkEvent* gdk_event = event.gdk_event();
+  GdkEventButton last_button_event = gdk_event->button;
+  mouse_down_abs_x_ = last_button_event.x_root;
+  mouse_down_abs_y_ = last_button_event.y_root;
+#else
+  const XEvent* xev = event.native_event();
+  gfx::Point abs_location = RootLocationFromXEvent(xev);
+  mouse_down_abs_x_ = abs_location.x();
+  mouse_down_abs_y_ = abs_location.y();
+#endif
   return true;
 }
 
-void PanelController::TitleMouseReleased(
-    const views::MouseEvent& event, bool canceled) {
-  if (!event.IsLeftMouseButton()) {
-    return;
-  }
+void PanelController::TitleMouseReleased(const views::MouseEvent& event) {
+  if (event.IsLeftMouseButton())
+    TitleMouseCaptureLost();
+}
+
+void PanelController::TitleMouseCaptureLost() {
   // Only handle clicks that started in our window.
-  if (!mouse_down_) {
+  if (!mouse_down_)
     return;
-  }
 
   mouse_down_ = false;
   if (!dragging_) {
-    SetState(expanded_ ?
-             PanelController::MINIMIZED : PanelController::EXPANDED);
+    if (expanded_) {
+      // Always activate the panel here, even if we are about to minimize it.
+      // This lets panels like GTalk know that they have been acknowledged, so
+      // they don't change the title again (which would trigger SetUrgent).
+      // Activating the panel also clears the urgent state.
+      delegate_->ActivatePanel();
+      SetState(PanelController::MINIMIZED);
+    } else {
+      // If we're expanding the panel, do so before focusing it.  This lets the
+      // window manager know that the panel is being expanded in response to a
+      // user action; see http://crosbug.com/14735.
+      SetState(PanelController::EXPANDED);
+      delegate_->ActivatePanel();
+    }
   } else {
+#if defined(TOOLKIT_USES_GTK)
     WmIpc::Message msg(WM_IPC_MESSAGE_WM_NOTIFY_PANEL_DRAG_COMPLETE);
     msg.set_param(0, panel_xid_);
     WmIpc::instance()->SendMessage(msg);
+#endif
     dragging_ = false;
   }
 }
 
 void PanelController::SetState(State state) {
+#if defined(TOOLKIT_USES_GTK)
   WmIpc::Message msg(WM_IPC_MESSAGE_WM_SET_PANEL_STATE);
   msg.set_param(0, panel_xid_);
   msg.set_param(1, state == EXPANDED);
   WmIpc::instance()->SendMessage(msg);
+#endif
 }
 
 bool PanelController::TitleMouseDragged(const views::MouseEvent& event) {
-  if (!mouse_down_) {
+  if (!mouse_down_)
     return false;
-  }
-
-  GdkEvent* gdk_event = gtk_get_current_event();
-  if (gdk_event->type != GDK_MOTION_NOTIFY) {
-    gdk_event_free(gdk_event);
+  if (event.type() != ui::ET_MOUSE_MOVED &&
+      event.type() != ui::ET_MOUSE_DRAGGED) {
     NOTREACHED();
     return false;
   }
+
+  const GdkEvent* gdk_event = event.gdk_event();
   GdkEventMotion last_motion_event = gdk_event->motion;
+  int x_root = last_motion_event.x_root;
+  int y_root = last_motion_event.y_root;
+
   if (!dragging_) {
-    if (views::View::ExceededDragThreshold(
-        last_motion_event.x_root - mouse_down_abs_x_,
-        last_motion_event.y_root - mouse_down_abs_y_)) {
+    if (views::View::ExceededDragThreshold(x_root - mouse_down_abs_x_,
+                                           y_root - mouse_down_abs_y_)) {
       dragging_ = true;
     }
   }
+#if defined(TOOLKIT_USES_GTK)
   if (dragging_) {
     WmIpc::Message msg(WM_IPC_MESSAGE_WM_NOTIFY_PANEL_DRAGGED);
     msg.set_param(0, panel_xid_);
-    msg.set_param(1, last_motion_event.x_root - mouse_down_offset_x_);
-    msg.set_param(2, last_motion_event.y_root - mouse_down_offset_y_);
+    msg.set_param(1, x_root - mouse_down_offset_x_);
+    msg.set_param(2, y_root - mouse_down_offset_y_);
     WmIpc::instance()->SendMessage(msg);
   }
-  gdk_event_free(gdk_event);
+#endif
   return true;
 }
 
@@ -263,14 +331,19 @@ bool PanelController::OnPanelClientEvent(
 void PanelController::OnFocusIn() {
   if (title_window_)
     title_content_->OnFocusIn();
+  focused_ = true;
+  // Clear urgent when focused.
+  SetUrgent(false);
 }
 
 void PanelController::OnFocusOut() {
+  focused_ = false;
   if (title_window_)
     title_content_->OnFocusOut();
 }
 
 bool PanelController::PanelClientEvent(GdkEventClient* event) {
+#if defined(TOOLKIT_USES_GTK)
   WmIpc::Message msg;
   WmIpc::instance()->DecodeMessage(*event, &msg);
   if (msg.type() == WM_IPC_MESSAGE_CHROME_NOTIFY_PANEL_STATE) {
@@ -278,12 +351,13 @@ bool PanelController::PanelClientEvent(GdkEventClient* event) {
     if (expanded_ != new_state) {
       expanded_ = new_state;
       State state = new_state ? EXPANDED : MINIMIZED;
-      NotificationService::current()->Notify(
-          NotificationType::PANEL_STATE_CHANGED,
-          Source<PanelController>(this),
-          Details<State>(&state));
+      content::NotificationService::current()->Notify(
+          chrome::NOTIFICATION_PANEL_STATE_CHANGED,
+          content::Source<PanelController>(this),
+          content::Details<State>(&state));
     }
   }
+#endif
   return true;
 }
 
@@ -329,13 +403,17 @@ PanelController::TitleContentView::TitleContentView(
 
   title_icon_ = new views::ImageView();
   AddChildView(title_icon_);
-  title_label_ = new views::Label(std::wstring());
+  title_label_ = new views::Label(string16());
   title_label_->SetHorizontalAlignment(views::Label::ALIGN_LEFT);
+  title_label_->SetAutoColorReadabilityEnabled(false);
+  title_label_->SetEnabledColor(kTitleActiveTextColor);
+  title_label_->SetDisabledColor(kTitleInactiveTextColor);
+  title_label_->SetEnabled(false);
   AddChildView(title_label_);
 
   set_background(
       views::Background::CreateBackgroundPainter(
-          true, new TitleBackgroundPainter()));
+          true, new TitleBackgroundPainter(panel_controller)));
   OnFocusOut();
 }
 
@@ -362,31 +440,32 @@ void PanelController::TitleContentView::Layout() {
 
 bool PanelController::TitleContentView::OnMousePressed(
     const views::MouseEvent& event) {
-  DCHECK(panel_controller_) << "OnMousePressed after Close";
   return panel_controller_->TitleMousePressed(event);
 }
 
 void PanelController::TitleContentView::OnMouseReleased(
-    const views::MouseEvent& event, bool canceled) {
-  DCHECK(panel_controller_) << "MouseReleased after Close";
-  return panel_controller_->TitleMouseReleased(event, canceled);
+    const views::MouseEvent& event) {
+  panel_controller_->TitleMouseReleased(event);
+}
+
+void PanelController::TitleContentView::OnMouseCaptureLost() {
+  panel_controller_->TitleMouseCaptureLost();
 }
 
 bool PanelController::TitleContentView::OnMouseDragged(
     const views::MouseEvent& event) {
-  DCHECK(panel_controller_) << "MouseDragged after Close";
   return panel_controller_->TitleMouseDragged(event);
 }
 
 void PanelController::TitleContentView::OnFocusIn() {
-  title_label_->SetColor(kTitleActiveColor);
+  title_label_->SetEnabled(true);
   title_label_->SetFont(*active_font);
   Layout();
   SchedulePaint();
 }
 
 void PanelController::TitleContentView::OnFocusOut() {
-  title_label_->SetColor(kTitleInactiveColor);
+  title_label_->SetEnabled(false);
   title_label_->SetFont(*inactive_font);
   Layout();
   SchedulePaint();
