@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,11 +6,13 @@
 
 #include <string>
 
-#include "chrome/browser/browser_thread.h"
 #include "chrome/common/net/gaia/gaia_auth_consumer.h"
 #include "chrome/common/net/gaia/gaia_auth_fetcher.h"
+#include "content/public/browser/browser_thread.h"
 #include "third_party/cros/chromeos_cryptohome.h"
 
+using content::BrowserThread;
+
 namespace chromeos {
 
 AuthAttemptState::AuthAttemptState(const std::string& username,
@@ -49,6 +51,24 @@ AuthAttemptState::AuthAttemptState(const std::string& username,
       cryptohome_code_(kCryptohomeMountErrorNone) {
 }
 
+AuthAttemptState::AuthAttemptState(const std::string& username,
+                                   const std::string& password,
+                                   const std::string& ascii_hash,
+                                   const bool user_is_new)
+    : username(username),
+      password(password),
+      ascii_hash(ascii_hash),
+      unlock(true),
+      online_complete_(false),
+      online_outcome_(LoginFailure::NONE),
+      credentials_(GaiaAuthConsumer::ClientLoginResult()),
+      hosted_policy_(GaiaAuthFetcher::HostedAccountsAllowed),
+      is_first_time_user_(user_is_new),
+      cryptohome_complete_(false),
+      cryptohome_outcome_(false),
+      cryptohome_code_(kCryptohomeMountErrorNone) {
+}
+
 AuthAttemptState::~AuthAttemptState() {}
 
 void AuthAttemptState::RecordOnlineLoginStatus(
@@ -123,4 +143,11 @@ int AuthAttemptState::cryptohome_code() {
   return cryptohome_code_;
 }
 
+void AuthAttemptState::SetOAuth1Token(const std::string& token,
+                                      const std::string& secret) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  oauth1_access_token_ = token;
+  oauth1_access_secret_ = secret;
+}
+
 }  // namespace chromeos
