@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,7 +19,7 @@ namespace chromeos {
 
 class AuthAttemptState {
  public:
-  // Used to initalize for a login attempt.
+  // Used to initialize for a login attempt.
   AuthAttemptState(const std::string& username,
                    const std::string& password,
                    const std::string& ascii_hash,
@@ -27,7 +27,13 @@ class AuthAttemptState {
                    const std::string& login_captcha,
                    const bool user_is_new);
 
-  // Used to initalize for a screen unlock attempt.
+  // Used to initialize for a externally authenticated login.
+  AuthAttemptState(const std::string& username,
+                   const std::string& password,
+                   const std::string& ascii_hash,
+                   const bool user_is_new);
+
+  // Used to initialize for a screen unlock attempt.
   AuthAttemptState(const std::string& username, const std::string& ascii_hash);
 
   virtual ~AuthAttemptState();
@@ -61,6 +67,12 @@ class AuthAttemptState {
   virtual bool cryptohome_outcome();
   virtual int cryptohome_code();
 
+  const std::string oauth1_access_token() const { return oauth1_access_token_; }
+  const std::string oauth1_access_secret() const {
+    return oauth1_access_secret_;
+  }
+  void SetOAuth1Token(const std::string& token, const std::string& secret);
+
   // Saved so we can retry client login, and also so we know for whom login
   // has succeeded, in the event of successful completion.
   const std::string username;
@@ -90,6 +102,8 @@ class AuthAttemptState {
   bool cryptohome_complete_;
   bool cryptohome_outcome_;
   int cryptohome_code_;
+  std::string oauth1_access_token_;
+  std::string oauth1_access_secret_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(AuthAttemptState);
