@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -18,7 +18,7 @@ class LoginStatusConsumer;
 const char Authenticator::kSpecialCaseDomain[] = "gmail.com";
 
 Authenticator::Authenticator(LoginStatusConsumer* consumer)
-    : consumer_(consumer) {
+    : consumer_(consumer), authentication_profile_(NULL) {
 }
 
 Authenticator::~Authenticator() {}
@@ -31,11 +31,6 @@ std::string Authenticator::Canonicalize(const std::string& email_address) {
   DCHECK_EQ(parts.size(), 2U) << "email_address should have only one @";
   if (parts[1] == kSpecialCaseDomain)  // only strip '.' for gmail accounts.
     RemoveChars(parts[0], ".", &parts[0]);
-  // Technically the '+' handling here could be removed, as the google
-  // account servers do not tolerate them, so we don't need to either.
-  // TODO(cmasone): remove this, unless this code becomes obsolete altogether.
-  if (parts[0].find('+') != std::string::npos)
-    parts[0].erase(parts[0].find('+'));
   std::string new_email = StringToLowerASCII(JoinString(parts, at));
   VLOG(1) << "Canonicalized " << email_address << " to " << new_email;
   return new_email;
