@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,7 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "chrome/browser/chromeos/login/login_status_consumer.h"
 #include "chrome/common/net/gaia/gaia_auth_consumer.h"
 
@@ -29,12 +29,17 @@ class Authenticator : public base::RefCountedThreadSafe<Authenticator> {
   explicit Authenticator(LoginStatusConsumer* consumer);
   virtual ~Authenticator();
 
+  // Given externally authenticated |username| and |password|, this method
+  // attempts to complete authentication process.
+  virtual void CompleteLogin(Profile* profile,
+                             const std::string& username,
+                             const std::string& password) = 0;
+
   // Given a |username| and |password|, this method attempts to authenticate
   // to login.
   // Optionally |login_token| and |login_captcha| could be provided.
-  // Returns true if we kick off the attempt successfully and false if we can't.
   // Must be called on the UI thread.
-  virtual bool AuthenticateToLogin(Profile* profile,
+  virtual void AuthenticateToLogin(Profile* profile,
                                    const std::string& username,
                                    const std::string& password,
                                    const std::string& login_token,
@@ -42,12 +47,11 @@ class Authenticator : public base::RefCountedThreadSafe<Authenticator> {
 
   // Given a |username| and |password|, this method attempts to
   // authenticate to unlock the computer.
-  // Returns true if we kick off the attempt successfully and false if
-  // we can't. Must be called on the UI thread.
-  virtual bool AuthenticateToUnlock(const std::string& username,
+  // Must be called on the UI thread.
+  virtual void AuthenticateToUnlock(const std::string& username,
                                     const std::string& password) = 0;
 
-  // Initiates off the record ("browse without signing in") login.
+  // Initiates incognito ("browse without signing in") login.
   virtual void LoginOffTheRecord() = 0;
 
   // |credentials| are the tokens that we get back from the ClientLogin API.
@@ -86,6 +90,10 @@ class Authenticator : public base::RefCountedThreadSafe<Authenticator> {
                          const std::string& login_token,
                          const std::string& login_captcha) = 0;
 
+  // Profile (usually off the record ) that was used to perform the last
+  // authentication process.
+  Profile* authentication_profile() { return authentication_profile_; }
+
   // Perform basic canonicalization of |email_address|, taking into account
   // that gmail does not consider '.' or caps inside a username to matter.
   // It also ignores everything after a '+'.
@@ -95,6 +103,7 @@ class Authenticator : public base::RefCountedThreadSafe<Authenticator> {
 
  protected:
   LoginStatusConsumer* consumer_;
+  Profile* authentication_profile_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(Authenticator);
