@@ -1,34 +1,38 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/chromeos/login/camera.h"
 
-#include <stdlib.h>
-#include <fcntl.h>  // low-level i/o
-#include <unistd.h>
-#include <errno.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#include <sys/mman.h>
-#include <sys/ioctl.h>
 #include <asm/types.h>  // for videodev2.h
+#include <errno.h>
+#include <fcntl.h>  // low-level i/o
 #include <linux/videodev2.h>
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <sys/mman.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include <algorithm>
 #include <vector>
 
+#include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/logging.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/threading/thread.h"
 #include "base/time.h"
-#include "chrome/browser/browser_thread.h"
-#include "gfx/size.h"
+#include "content/public/browser/browser_thread.h"
 #include "skia/ext/image_operations.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkColorPriv.h"
+#include "ui/gfx/size.h"
+
+using content::BrowserThread;
 
 namespace chromeos {
 
@@ -113,7 +117,9 @@ gfx::Size get_best_frame_size(int fd,
 }
 
 // Default camera device name.
-const char kDeviceName[] = "/dev/video0";
+const char kDeviceNameFormat[] = "/dev/video%d";
+// Number of times we retry initializing camera with different device index.
+const int kOpenDeviceRetries = 10;
 // Default width of each frame received from the camera.
 const int kFrameWidth = 640;
 // Default height of each frame received from the camera.
@@ -131,7 +137,6 @@ const long kSelectTimeout = 1 * base::Time::kMicrosecondsPerSecond;
 Camera::Camera(Delegate* delegate, base::Thread* thread, bool mirrored)
     : delegate_(delegate),
       thread_(thread),
-      device_name_(kDeviceName),
       device_descriptor_(-1),
       is_capturing_(false),
       desired_width_(kFrameWidth),
@@ -149,33 +154,28 @@ void Camera::ReportFailure() {
   DCHECK(IsOnCameraThread());
   if (device_descriptor_ == -1) {
     BrowserThread::PostTask(
-        BrowserThread::UI,
-        FROM_HERE,
-        NewRunnableMethod(this,
-                          &Camera::OnInitializeFailure));
+        BrowserThread::UI, FROM_HERE,
+        base::Bind(&Camera::OnInitializeFailure, this));
   } else if (!is_capturing_) {
     BrowserThread::PostTask(
-        BrowserThread::UI,
-        FROM_HERE,
-        NewRunnableMethod(this,
-                          &Camera::OnStartCapturingFailure));
+        BrowserThread::UI, FROM_HERE,
+        base::Bind(&Camera::OnStartCapturingFailure, this));
   } else {
     BrowserThread::PostTask(
-        BrowserThread::UI,
-        FROM_HERE,
-        NewRunnableMethod(this,
-                          &Camera::OnCaptureFailure));
+        BrowserThread::UI, FROM_HERE,
+        base::Bind(&Camera::OnCaptureFailure, this));
   }
 }
 
-void Camera::Initialize(int desired_width, int desired_height) {
+void Camera::Initialize(
+    int desired_width,
+    int desired_height,
+    int64 delay_in_ms) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  PostCameraTask(
+  PostCameraTaskWithDelay(
       FROM_HERE,
-      NewRunnableMethod(this,
-                        &Camera::DoInitialize,
-                        desired_width,
-                        desired_height));
+      base::Bind(&Camera::DoInitialize, this, desired_width, desired_height),
+      delay_in_ms);
 }
 
 void Camera::DoInitialize(int desired_width, int desired_height) {
@@ -186,7 +186,15 @@ void Camera::DoInitialize(int desired_width, int desired_height) {
     return;
   }
 
-  int fd = OpenDevice(device_name_.c_str());
+  int fd = -1;
+  for (int i = 0; i < kOpenDeviceRetries; ++i) {
+    device_name_ = StringPrintf(kDeviceNameFormat, i);
+    fd = OpenDevice(device_name_.c_str());
+    if (fd != -1)
+      break;
+    LOG(WARNING) << "Failed to open device: " << device_name_;
+  }
+
   if (fd == -1) {
     ReportFailure();
     return;
@@ -223,7 +231,7 @@ void Camera::DoInitialize(int desired_width, int desired_height) {
   format.fmt.pix.pixelformat = V4L2_PIX_FMT_YUYV;
   format.fmt.pix.field = V4L2_FIELD_INTERLACED;
   if (xioctl(fd, VIDIOC_S_FMT, &format) == -1) {
-    LOG(ERROR) << "VIDIOC_S_FMT failed.";
+    log_errno("VIDIOC_S_FMT failed.");
     ReportFailure();
     return;
   }
@@ -234,19 +242,18 @@ void Camera::DoInitialize(int desired_width, int desired_height) {
   }
 
   device_descriptor_ = fd;
-  frame_width_ = frame_size.width();
-  frame_height_ = frame_size.height();
+  frame_width_ = format.fmt.pix.width;
+  frame_height_ = format.fmt.pix.height;
   desired_width_ = desired_width;
   desired_height_ = desired_height;
   BrowserThread::PostTask(
-      BrowserThread::UI,
-      FROM_HERE,
-      NewRunnableMethod(this, &Camera::OnInitializeSuccess));
+      BrowserThread::UI, FROM_HERE,
+      base::Bind(&Camera::OnInitializeSuccess, this));
 }
 
 void Camera::Uninitialize() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  PostCameraTask(FROM_HERE, NewRunnableMethod(this, &Camera::DoUninitialize));
+  PostCameraTask(FROM_HERE, base::Bind(&Camera::DoUninitialize, this));
 }
 
 void Camera::DoUninitialize() {
@@ -264,8 +271,7 @@ void Camera::DoUninitialize() {
 
 void Camera::StartCapturing() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  PostCameraTask(FROM_HERE,
-                 NewRunnableMethod(this, &Camera::DoStartCapturing));
+  PostCameraTask(FROM_HERE, base::Bind(&Camera::DoStartCapturing, this));
 }
 
 void Camera::DoStartCapturing() {
@@ -295,19 +301,15 @@ void Camera::DoStartCapturing() {
   // No need to post DidProcessCameraThreadMethod() as this method is
   // being posted instead.
   BrowserThread::PostTask(
-      BrowserThread::UI,
-      FROM_HERE,
-      NewRunnableMethod(this,
-                        &Camera::OnStartCapturingSuccess));
+      BrowserThread::UI, FROM_HERE,
+      base::Bind(&Camera::OnStartCapturingSuccess, this));
   is_capturing_ = true;
-  PostCameraTask(FROM_HERE,
-                 NewRunnableMethod(this, &Camera::OnCapture));
+  PostCameraTask(FROM_HERE, base::Bind(&Camera::OnCapture, this));
 }
 
 void Camera::StopCapturing() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  PostCameraTask(FROM_HERE,
-                 NewRunnableMethod(this, &Camera::DoStopCapturing));
+  PostCameraTask(FROM_HERE, base::Bind(&Camera::DoStopCapturing, this));
 }
 
 void Camera::DoStopCapturing() {
@@ -335,11 +337,11 @@ int Camera::OpenDevice(const char* device_name) const {
   DCHECK(IsOnCameraThread());
   struct stat st;
   if (stat(device_name, &st) == -1) {
-    log_errno(base::StringPrintf("Cannot identify %s", device_name));
+    log_errno(base::StringPrintf("Cannot find %s", device_name));
     return -1;
   }
   if (!S_ISCHR(st.st_mode)) {
-    LOG(ERROR) << device_name << "is not adevice";
+    LOG(ERROR) << device_name << "is not a device";
     return -1;
   }
   int fd = open(device_name, O_RDWR | O_NONBLOCK, 0);
@@ -433,8 +435,7 @@ void Camera::OnCapture() {
     // EAGAIN - continue select loop.
   } while (!ReadFrame());
 
-  PostCameraTask(FROM_HERE,
-                 NewRunnableMethod(this, &Camera::OnCapture));
+  PostCameraTask(FROM_HERE, base::Bind(&Camera::OnCapture, this));
 }
 
 bool Camera::ReadFrame() {
@@ -531,9 +532,8 @@ void Camera::ProcessImage(void* data) {
     frame_image_.swap(image);
   }
   BrowserThread::PostTask(
-      BrowserThread::UI,
-      FROM_HERE,
-      NewRunnableMethod(this, &Camera::OnCaptureSuccess));
+      BrowserThread::UI, FROM_HERE,
+      base::Bind(&Camera::OnCaptureSuccess, this));
 }
 
 void Camera::OnInitializeSuccess() {
@@ -578,12 +578,19 @@ bool Camera::IsOnCameraThread() const {
 }
 
 void Camera::PostCameraTask(const tracked_objects::Location& from_here,
-                            Task* task) {
+                            const base::Closure& task) {
+  PostCameraTaskWithDelay(from_here, task, 0);
+}
+
+void Camera::PostCameraTaskWithDelay(
+    const tracked_objects::Location& from_here,
+    const base::Closure& task,
+    int64 delay_in_ms) {
   base::AutoLock lock(thread_lock_);
   if (!thread_)
     return;
   DCHECK(thread_->IsRunning());
-  thread_->message_loop()->PostTask(from_here, task);
+  thread_->message_loop()->PostDelayedTask(from_here, task, delay_in_ms);
 }
 
 }  // namespace chromeos
