@@ -7,11 +7,13 @@
 #pragma once
 
 #include <string>
-#include "base/scoped_ptr.h"
+
+#include "base/compiler_specific.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/login/auth_response_handler.h"
 #include "chrome/browser/chromeos/login/client_login_response_handler.h"
 #include "chrome/browser/chromeos/login/issue_response_handler.h"
-#include "chrome/common/net/url_fetcher.h"
+
 
 class Profile;
 
@@ -22,7 +24,7 @@ namespace chromeos {
 //
 // A CookieFetcher manages its own lifecycle.  It deletes itself once it's
 // done attempting to fetch URLs.
-class CookieFetcher : public URLFetcher::Delegate {
+class CookieFetcher : public content::URLFetcherDelegate {
  public:
   // |profile| is the Profile whose cookie jar you want the cookies in.
   explicit CookieFetcher(Profile* profile);
@@ -31,11 +33,7 @@ class CookieFetcher : public URLFetcher::Delegate {
   // Takes ownership of |cl_handler|, |i_handler|, and |launcher|.
   CookieFetcher(Profile* profile,
                 AuthResponseHandler* cl_handler,
-                AuthResponseHandler* i_handler)
-      : profile_(profile),
-        client_login_handler_(cl_handler),
-        issue_handler_(i_handler) {
-  }
+                AuthResponseHandler* i_handler);
 
   // Given a newline-delineated SID/LSID pair of Google cookies (like
   // those that come back from ClientLogin), try to use them to fetch
@@ -44,18 +42,13 @@ class CookieFetcher : public URLFetcher::Delegate {
   // Either way, we end up by calling launcher_->DoLaunch()
   void AttemptFetch(const std::string& credentials);
 
-  // Overloaded from URLFetcher::Delegate.
-  virtual void OnURLFetchComplete(const URLFetcher* source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  // Overloaded from content::URLFetcherDelegate.
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
  private:
-  virtual ~CookieFetcher() {}
+  virtual ~CookieFetcher();
 
-  scoped_ptr<URLFetcher> fetcher_;
+  scoped_ptr<content::URLFetcher> fetcher_;
   Profile* profile_;
   scoped_ptr<AuthResponseHandler> client_login_handler_;
   scoped_ptr<AuthResponseHandler> issue_handler_;
