@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,8 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
 #include "chrome/browser/chromeos/cros/cryptohome_library.h"
 
 namespace chromeos {
@@ -39,10 +40,10 @@ class CryptohomeOp
       AuthAttemptState* current_attempt,
       AuthAttemptStateResolver* callback);
 
-  virtual bool Initiate() = 0;
+  virtual void Initiate() = 0;
 
   // Implementation of CryptohomeLibrary::Delegate.
-  virtual void OnComplete(bool success, int return_code);
+  virtual void OnComplete(bool success, int return_code) OVERRIDE;
 
  protected:
   CryptohomeOp(AuthAttemptState* current_attempt,
