@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,39 +6,41 @@
 #define CHROME_BROWSER_CHROMEOS_LOGIN_HELP_APP_LAUNCHER_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/login/login_html_dialog.h"
-#include "gfx/native_widget_types.h"
+#include "ui/gfx/native_widget_types.h"
 
 namespace chromeos {
 
 // Provides help content during OOBE / login.
 // Based on connectivity state (offline/online) shows help topic dialog
 // or launches HelpApp in BWSI mode.
-class HelpAppLauncher : public LoginHtmlDialog::Delegate {
+class HelpAppLauncher : public LoginHtmlDialog::Delegate,
+                        public base::RefCountedThreadSafe<HelpAppLauncher> {
  public:
   // IDs of help topics available from HelpApp.
   enum HelpTopic {
     // Showed on basic connectivity issues.
-    HELP_CONNECTIVITY,
+    HELP_CONNECTIVITY = 188752,
     // Showed at EULA screen as "Learn more" about stats/crash reports.
-    HELP_STATS_USAGE,
+    HELP_STATS_USAGE = 183078,
     // Showed whenever there're troubles signing in (offline case).
-    HELP_CANT_ACCESS_ACCOUNT_OFFLINE,
+    HELP_CANT_ACCESS_ACCOUNT_OFFLINE = 188755,
     // Showed whenever there're troubles signing in (online case).
-    HELP_CANT_ACCESS_ACCOUNT,
+    HELP_CANT_ACCESS_ACCOUNT = 188036,
     // Showed in case when account was disabled.
-    HELP_ACCOUNT_DISABLED,
+    HELP_ACCOUNT_DISABLED = 188756,
     // Showed in case when hosted account is used.
-    HELP_HOSTED_ACCOUNT,
-    HELP_TOPIC_COUNT
+    HELP_HOSTED_ACCOUNT = 1054228,
   };
 
   // Parent window is used to show dialog.
   explicit HelpAppLauncher(gfx::NativeWindow parent_window);
+  virtual ~HelpAppLauncher();
 
   // Shows specified help topic.
-  // TODO: Pass topic ID.
   void ShowHelpTopic(HelpTopic help_topic_id);
 
   // Returns true if the dialog is currently open.
@@ -46,7 +48,7 @@ class HelpAppLauncher : public LoginHtmlDialog::Delegate {
 
  protected:
   // LoginHtmlDialog::Delegate implementation:
-  virtual void OnDialogClosed() {}
+  virtual void OnDialogClosed() OVERRIDE {}
 
  private:
   // Shows help topic dialog for specified GURL.
