@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,46 +7,43 @@
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
+#include "chrome/browser/chromeos/input_method/input_method_manager.h"
 #include "chrome/browser/chromeos/input_method/input_method_util.h"
 #include "chrome/browser/chromeos/login/screen_observer.h"
 #include "chrome/browser/profiles/profile_manager.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/renderer_host/site_instance.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/site_instance.h"
 #include "googleurl/src/gurl.h"
-#include "views/widget/widget_gtk.h"
+#include "ui/views/events/event.h"
+
+using content::SiteInstance;
+using content::WebContents;
 
 namespace chromeos {
 
-static const char kHTMLPageDoneUrl[] = "about:blank";
-
-///////////////////////////////////////////////////////////////////////////////
-// HTMLPageDomView
-TabContents* HTMLPageDomView::CreateTabContents(Profile* profile,
-                                                SiteInstance* instance) {
-  return new WizardWebPageViewTabContents(profile,
-                                          instance,
-                                          page_delegate_);
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 // HTMLPageView
 HTMLPageView::HTMLPageView()
-    : dom_view_(new HTMLPageDomView()) {
+    : dom_view_(new WebPageDomView()) {
+}
+
+WebPageDomView* HTMLPageView::dom_view() {
+  return dom_view_;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 // HTMLPageScreen, public:
-HTMLPageScreen::HTMLPageScreen(WizardScreenDelegate* delegate,
+HTMLPageScreen::HTMLPageScreen(ViewScreenDelegate* delegate,
                                const std::string& url)
     : ViewScreen<HTMLPageView>(delegate), url_(url) {
 }
 
+HTMLPageScreen::~HTMLPageScreen() {}
+
 ///////////////////////////////////////////////////////////////////////////////
 // HTMLPageScreen, ViewScreen implementation:
 void HTMLPageScreen::CreateView() {
   ViewScreen<HTMLPageView>::CreateView();
-  view()->SetWebPageDelegate(this);
 }
 
 void HTMLPageScreen::Refresh() {
@@ -54,9 +51,8 @@ void HTMLPageScreen::Refresh() {
   StartTimeoutTimer();
   GURL url(url_);
   Profile* profile = ProfileManager::GetDefaultProfile();
-  view()->InitDOM(profile,
-                  SiteInstance::CreateSiteInstanceForURL(profile, url));
-  view()->SetTabContentsDelegate(this);
+  view()->InitDOM(profile, SiteInstance::CreateForURL(profile, url));
+  view()->SetWebContentsDelegate(this);
   view()->LoadURL(url);
 }
 
@@ -64,46 +60,9 @@ HTMLPageView* HTMLPageScreen::AllocateView() {
   return new HTMLPageView();
 }
 
-///////////////////////////////////////////////////////////////////////////////
-// HTMLPageScreen, TabContentsDelegate implementation:
-void HTMLPageScreen::LoadingStateChanged(TabContents* source) {
-  std::string url = source->GetURL().spec();
-  if (url == kHTMLPageDoneUrl) {
-    source->Stop();
-    // TODO(dpolukhin): use special code for this case but now
-    // ACCOUNT_CREATE_BACK works as we would like, i.e. get to login page.
-    VLOG(1) << "HTMLPageScreen::LoadingStateChanged: " << url;
-    CloseScreen(ScreenObserver::ACCOUNT_CREATE_BACK);
-  }
-}
-
-void HTMLPageScreen::NavigationStateChanged(const TabContents* source,
-                                            unsigned changed_flags) {
-}
-
 void HTMLPageScreen::HandleKeyboardEvent(const NativeWebKeyboardEvent& event) {
-  views::Widget* widget = view()->GetWidget();
-  if (widget && event.os_event && !event.skip_in_browser)
-    static_cast<views::WidgetGtk*>(widget)->HandleKeyboardEvent(event.os_event);
-}
-
-///////////////////////////////////////////////////////////////////////////////
-// HTMLPageScreen, WebPageDelegate implementation:
-void HTMLPageScreen::OnPageLoaded() {
-  StopTimeoutTimer();
-  // Enable input methods (e.g. Chinese, Japanese) so that users could input
-  // their first and last names.
-  if (g_browser_process) {
-    const std::string locale = g_browser_process->GetApplicationLocale();
-    input_method::EnableInputMethods(
-        locale, input_method::kAllInputMethods, "");
-  }
-  view()->ShowPageContent();
-}
-
-void HTMLPageScreen::OnPageLoadFailed(const std::string& url) {
-  VLOG(1) << "HTMLPageScreen::OnPageLoadFailed: " << url;
-  CloseScreen(ScreenObserver::CONNECTION_FAILED);
+  unhandled_keyboard_handler_.HandleKeyboardEvent(event,
+                                                  view()->GetFocusManager());
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -111,7 +70,16 @@ void HTMLPageScreen::OnPageLoadFailed(const std::string& url) {
 void HTMLPageScreen::OnNetworkTimeout() {
   VLOG(1) << "HTMLPageScreen::OnNetworkTimeout";
   // Just show what we have now. We shouldn't exit from the screen on timeout.
-  OnPageLoaded();
+  StopTimeoutTimer();
+  // Enable input methods (e.g. Chinese, Japanese) so that users could input
+  // their first and last names.
+  if (g_browser_process) {
+    const std::string locale = g_browser_process->GetApplicationLocale();
+    input_method::InputMethodManager* manager =
+        input_method::InputMethodManager::GetInstance();
+    manager->EnableInputMethods(locale, input_method::kAllInputMethods, "");
+  }
+  view()->ShowPageContent();
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -122,10 +90,11 @@ void HTMLPageScreen::CloseScreen(ScreenObserver::ExitCodes code) {
   // password.
   if (g_browser_process) {
     const std::string locale = g_browser_process->GetApplicationLocale();
-    input_method::EnableInputMethods(
-        locale, input_method::kKeyboardLayoutsOnly, "");
+    input_method::InputMethodManager* manager =
+        input_method::InputMethodManager::GetInstance();
+    manager->EnableInputMethods(locale, input_method::kKeyboardLayoutsOnly, "");
   }
-  delegate()->GetObserver(this)->OnExit(code);
+  delegate()->GetObserver()->OnExit(code);
 }
 
 }  // namespace chromeos
