@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,11 +8,12 @@
 
 #include <string>
 
-#include "chrome/browser/dom_ui/html_dialog_ui.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
-#include "gfx/native_widget_types.h"
-#include "gfx/size.h"
+#include "base/compiler_specific.h"
+#include "chrome/browser/ui/webui/html_dialog_ui.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "ui/gfx/native_widget_types.h"
+#include "ui/gfx/size.h"
 
 namespace chromeos {
 
@@ -20,7 +21,7 @@ class BubbleFrameView;
 
 // Launches html dialog during OOBE/Login with specified URL and title.
 class LoginHtmlDialog : public HtmlDialogUIDelegate,
-                        public NotificationObserver {
+                        public content::NotificationObserver {
  public:
   // Delegate class to get notifications from the dialog.
   class Delegate {
@@ -28,7 +29,7 @@ class LoginHtmlDialog : public HtmlDialogUIDelegate,
     virtual ~Delegate() {}
 
     // Called when dialog has been closed.
-    virtual void OnDialogClosed() = 0;
+    virtual void OnDialogClosed();
   };
 
   enum Style {
@@ -41,7 +42,7 @@ class LoginHtmlDialog : public HtmlDialogUIDelegate,
                   const std::wstring& title,
                   const GURL& url,
                   Style style);
-  ~LoginHtmlDialog();
+  virtual ~LoginHtmlDialog();
 
   // Shows created dialog.
   void Show();
@@ -49,37 +50,42 @@ class LoginHtmlDialog : public HtmlDialogUIDelegate,
   // Overrides default width/height for dialog.
   void SetDialogSize(int width, int height);
 
+  // Overrides dialog title.
+  void SetDialogTitle(const string16& title);
+
   void set_url(const GURL& url) { url_ = url; }
 
   bool is_open() const { return is_open_; }
 
  protected:
   // HtmlDialogUIDelegate implementation.
-  virtual bool IsDialogModal() const { return true; }
-  virtual std::wstring GetDialogTitle() const { return title_; }
-  virtual GURL GetDialogContentURL() const { return url_; }
-  virtual void GetDOMMessageHandlers(
-      std::vector<DOMMessageHandler*>* handlers) const {}
-  virtual void GetDialogSize(gfx::Size* size) const;
-  virtual std::string GetDialogArgs() const { return std::string(); }
-  virtual void OnDialogClosed(const std::string& json_retval);
-  virtual void OnCloseContents(TabContents* source, bool* out_close_dialog);
-  virtual bool ShouldShowDialogTitle() const { return true; }
+  virtual ui::ModalType GetDialogModalType() const OVERRIDE;
+  virtual string16 GetDialogTitle() const OVERRIDE;
+  virtual GURL GetDialogContentURL() const OVERRIDE;
+  virtual void GetWebUIMessageHandlers(
+      std::vector<content::WebUIMessageHandler*>* handlers) const OVERRIDE;
+  virtual void GetDialogSize(gfx::Size* size) const OVERRIDE;
+  virtual std::string GetDialogArgs() const OVERRIDE;
+  virtual void OnDialogClosed(const std::string& json_retval) OVERRIDE;
+  virtual void OnCloseContents(
+      content::WebContents* source, bool* out_close_dialog) OVERRIDE;
+  virtual bool ShouldShowDialogTitle() const OVERRIDE;
+  virtual bool HandleContextMenu(const ContextMenuParams& params) OVERRIDE;
 
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
  private:
   // Notifications receiver.
   Delegate* delegate_;
 
   gfx::NativeWindow parent_window_;
-  std::wstring title_;
+  string16 title_;
   GURL url_;
   Style style_;
-  NotificationRegistrar notification_registrar_;
+  content::NotificationRegistrar notification_registrar_;
   BubbleFrameView* bubble_frame_view_;
   bool is_open_;
 
