@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,9 +15,12 @@ namespace chromeos {
 
 class MockOwnershipService : public OwnershipService {
  public:
+  MockOwnershipService();
+  virtual ~MockOwnershipService();
+
   MOCK_METHOD0(IsAlreadyOwned, bool(void));
-  MOCK_METHOD0(StartLoadOwnerKeyAttempt, bool(void));
-  MOCK_METHOD0(StartTakeOwnershipAttempt, bool(void));
+  MOCK_METHOD1(GetStatus, OwnershipService::Status(bool));
+  MOCK_METHOD0(StartLoadOwnerKeyAttempt, void(void));
   MOCK_METHOD2(StartSigningAttempt, void(const std::string&,
                                          OwnerManager::Delegate*));
   MOCK_METHOD3(StartVerifyAttempt, void(const std::string&,
