@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,19 +6,17 @@
 
 #include <limits>
 
-#include "base/crypto/rsa_private_key.h"
-#include "base/crypto/signature_creator.h"
-#include "base/crypto/signature_verifier.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
 #include "base/logging.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string_util.h"
+#include "crypto/rsa_private_key.h"
+#include "crypto/signature_creator.h"
+#include "crypto/signature_verifier.h"
 #include "chrome/browser/chromeos/cros/cros_library.h"
-#include "chrome/browser/chromeos/cros/login_library.h"
 #include "chrome/common/extensions/extension_constants.h"
 
-using base::RSAPrivateKey;
 using extension_misc::kSignatureAlgorithm;
 
 namespace chromeos {
@@ -40,13 +38,6 @@ class OwnerKeyUtilsImpl : public OwnerKeyUtils {
  public:
   OwnerKeyUtilsImpl();
 
-  RSAPrivateKey* GenerateKeyPair();
-
-  bool ExportPublicKeyViaDbus(RSAPrivateKey* pair,
-                              LoginLibrary::Delegate* d);
-
-  bool ExportPublicKeyToFile(RSAPrivateKey* pair, const FilePath& key_file);
-
   bool ImportPublicKey(const FilePath& key_file,
                        std::vector<uint8>* output);
 
@@ -56,23 +47,23 @@ class OwnerKeyUtilsImpl : public OwnerKeyUtils {
 
   bool Sign(const std::string& data,
             std::vector<uint8>* OUT_signature,
-            base::RSAPrivateKey* key);
+            crypto::RSAPrivateKey* key);
 
-  RSAPrivateKey* FindPrivateKey(const std::vector<uint8>& key);
+  crypto::RSAPrivateKey* FindPrivateKey(const std::vector<uint8>& key);
 
   FilePath GetOwnerKeyFilePath();
 
  protected:
   virtual ~OwnerKeyUtilsImpl();
 
+  bool ExportPublicKeyToFile(crypto::RSAPrivateKey* pair,
+                             const FilePath& key_file);
+
  private:
   // The file outside the owner's encrypted home directory where her
   // key will live.
   static const char kOwnerKeyFile[];
 
-  // Key generation parameters.
-  static const uint16 kKeySizeInBits;
-
   DISALLOW_COPY_AND_ASSIGN(OwnerKeyUtilsImpl);
 };
 
@@ -87,36 +78,11 @@ OwnerKeyUtils* OwnerKeyUtils::Create() {
 // static
 const char OwnerKeyUtilsImpl::kOwnerKeyFile[] = "/var/lib/whitelist/owner.key";
 
-// We're generating and using 2048-bit RSA keys.
-// static
-const uint16 OwnerKeyUtilsImpl::kKeySizeInBits = 2048;
-
 OwnerKeyUtilsImpl::OwnerKeyUtilsImpl() {}
 
 OwnerKeyUtilsImpl::~OwnerKeyUtilsImpl() {}
 
-RSAPrivateKey* OwnerKeyUtilsImpl::GenerateKeyPair() {
-  return RSAPrivateKey::CreateSensitive(kKeySizeInBits);
-}
-
-bool OwnerKeyUtilsImpl::ExportPublicKeyViaDbus(RSAPrivateKey* pair,
-                                               LoginLibrary::Delegate* d) {
-  DCHECK(pair);
-  bool ok = false;
-
-  std::vector<uint8> to_export;
-  if (!pair->ExportPublicKey(&to_export)) {
-    LOG(ERROR) << "Formatting key for export via dbus failed!";
-    return false;
-  }
-
-  if (CrosLibrary::Get()->EnsureLoaded())
-    ok = CrosLibrary::Get()->GetLoginLibrary()->SetOwnerKeyAsync(to_export, d);
-
-  return ok;
-}
-
-bool OwnerKeyUtilsImpl::ExportPublicKeyToFile(RSAPrivateKey* pair,
+bool OwnerKeyUtilsImpl::ExportPublicKeyToFile(crypto::RSAPrivateKey* pair,
                                               const FilePath& key_file) {
   DCHECK(pair);
   bool ok = false;
@@ -167,7 +133,7 @@ bool OwnerKeyUtilsImpl::ImportPublicKey(const FilePath& key_file,
 bool OwnerKeyUtilsImpl::Verify(const std::string& data,
                                const std::vector<uint8> signature,
                                const std::vector<uint8> public_key) {
-  base::SignatureVerifier verifier;
+  crypto::SignatureVerifier verifier;
   if (!verifier.VerifyInit(kSignatureAlgorithm, sizeof(kSignatureAlgorithm),
                            &signature[0], signature.size(),
                            &public_key[0], public_key.size())) {
@@ -181,9 +147,9 @@ bool OwnerKeyUtilsImpl::Verify(const std::string& data,
 
 bool OwnerKeyUtilsImpl::Sign(const std::string& data,
                              std::vector<uint8>* OUT_signature,
-                             base::RSAPrivateKey* key) {
-  scoped_ptr<base::SignatureCreator> signer(
-      base::SignatureCreator::Create(key));
+                             crypto::RSAPrivateKey* key) {
+  scoped_ptr<crypto::SignatureCreator> signer(
+      crypto::SignatureCreator::Create(key));
   if (!signer->Update(reinterpret_cast<const uint8*>(data.c_str()),
                       data.length())) {
     return false;
@@ -191,9 +157,9 @@ bool OwnerKeyUtilsImpl::Sign(const std::string& data,
   return signer->Final(OUT_signature);
 }
 
-RSAPrivateKey* OwnerKeyUtilsImpl::FindPrivateKey(
+crypto::RSAPrivateKey* OwnerKeyUtilsImpl::FindPrivateKey(
     const std::vector<uint8>& key) {
-  return RSAPrivateKey::FindFromPublicKeyInfo(key);
+  return crypto::RSAPrivateKey::FindFromPublicKeyInfo(key);
 }
 
 FilePath OwnerKeyUtilsImpl::GetOwnerKeyFilePath() {
