@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,9 +10,10 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "base/gtest_prod_util.h"
-#include "base/scoped_ptr.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/cros/cros_library.h"
 #include "chrome/browser/chromeos/cros/cryptohome_library.h"
 #include "chrome/browser/chromeos/login/authenticator.h"
@@ -31,6 +32,10 @@ namespace base {
 class Lock;
 }
 
+namespace crypto {
+class SymmetricKey;
+}
+
 namespace chromeos {
 
 class LoginStatusConsumer;
@@ -78,6 +83,11 @@ class ParallelAuthenticator : public Authenticator,
   explicit ParallelAuthenticator(LoginStatusConsumer* consumer);
   virtual ~ParallelAuthenticator();
 
+  // Authenticator overrides.
+  virtual void CompleteLogin(Profile* profile,
+                             const std::string& username,
+                             const std::string& password) OVERRIDE;
+
   // Given a |username| and |password|, this method attempts to authenticate to
   // the Google accounts servers and your Chrome OS device simultaneously.
   // As soon as we have successfully mounted the encrypted home directory for
@@ -103,55 +113,55 @@ class ParallelAuthenticator : public Authenticator,
   // NOTE: We do not allow HOSTED accounts to log in.  In the event that
   // we are asked to authenticate valid HOSTED account creds, we will
   // call OnLoginFailure() with HOSTED_NOT_ALLOWED.
-  //
-  // Returns true if the attempt gets sent successfully and false if not.
-  bool AuthenticateToLogin(Profile* profile,
-                           const std::string& username,
-                           const std::string& password,
-                           const std::string& login_token,
-                           const std::string& login_captcha);
+  virtual void AuthenticateToLogin(Profile* profile,
+                                   const std::string& username,
+                                   const std::string& password,
+                                   const std::string& login_token,
+                                   const std::string& login_captcha) OVERRIDE;
 
   // Given a |username| and |password|, this method attempts to
   // authenticate to the cached credentials. This will never contact
   // the server even if it's online. The auth result is sent to
   // LoginStatusConsumer in a same way as AuthenticateToLogin does.
-  bool AuthenticateToUnlock(const std::string& username,
-                            const std::string& password);
+  virtual void AuthenticateToUnlock(const std::string& username,
+                                    const std::string& password) OVERRIDE;
 
-  // Initiates off the record ("browse without signing in") login.
+  // Initiates incognito ("browse without signing in") login.
   // Mounts tmpfs and notifies consumer on the success/failure.
-  void LoginOffTheRecord();
+  virtual void LoginOffTheRecord() OVERRIDE;
 
   // These methods must be called on the UI thread, as they make DBus calls
   // and also call back to the login UI.
-  void OnLoginSuccess(const GaiaAuthConsumer::ClientLoginResult& credentials,
-                      bool request_pending);
-  void OnOffTheRecordLoginSuccess();
-  void OnPasswordChangeDetected(
-      const GaiaAuthConsumer::ClientLoginResult& credentials);
-  void OnLoginFailure(const LoginFailure& error);
+  virtual void OnLoginSuccess(
+      const GaiaAuthConsumer::ClientLoginResult& credentials,
+      bool request_pending)  OVERRIDE;
 
-  void RecoverEncryptedData(
+  virtual void OnLoginFailure(const LoginFailure& error) OVERRIDE;
+  virtual void RecoverEncryptedData(
       const std::string& old_password,
-      const GaiaAuthConsumer::ClientLoginResult& credentials);
-  void ResyncEncryptedData(
-      const GaiaAuthConsumer::ClientLoginResult& credentials);
-  void RetryAuth(Profile* profile,
-                 const std::string& username,
-                 const std::string& password,
-                 const std::string& login_token,
-                 const std::string& login_captcha);
-
-  // Call this on the FILE thread.
-  void CheckLocalaccount(const LoginFailure& error);
-
+      const GaiaAuthConsumer::ClientLoginResult& credentials) OVERRIDE;
+  virtual void ResyncEncryptedData(
+      const GaiaAuthConsumer::ClientLoginResult& credentials) OVERRIDE;
+  virtual void RetryAuth(Profile* profile,
+                         const std::string& username,
+                         const std::string& password,
+                         const std::string& login_token,
+                         const std::string& login_captcha) OVERRIDE;
+  // AuthAttemptStateResolver overrides.
   // Attempts to make a decision and call back |consumer_| based on
   // the state we have gathered at the time of call.  If a decision
   // can't be made, defers until the next time this is called.
   // When a decision is made, will call back to |consumer_| on the UI thread.
   //
   // Must be called on the IO thread.
-  void Resolve();
+  virtual void Resolve() OVERRIDE;
+
+  // Call this on the FILE thread.
+  void CheckLocalaccount(const LoginFailure& error);
+
+  void OnOffTheRecordLoginSuccess();
+  void OnPasswordChangeDetected(
+      const GaiaAuthConsumer::ClientLoginResult& credentials);
 
  private:
   // Returns the AuthState we're in, given the status info we have at
@@ -195,6 +205,11 @@ class ParallelAuthenticator : public Authenticator,
     current_state_.reset(new_state);
   }
 
+  // Sets an online attemp for testing.
+  void set_online_attempt(OnlineAttempt* attempt) {
+    current_online_ = attempt;
+  }
+
   // Resets |current_state_| and then posts a task to the UI thread to
   // Initiate() |to_initiate|.
   // Call this method on the IO thread.
@@ -202,6 +217,9 @@ class ParallelAuthenticator : public Authenticator,
 
   // If we don't have the system salt yet, loads it from the CryptohomeLibrary.
   void LoadSystemSalt();
+  // If we don't have supplemental_user_key_ yet, loads it from the NSS DB.
+  // Returns false if the key can not be loaded/created.
+  bool LoadSupplementalUserKey();
 
   // If we haven't already, looks in a file called |filename| next to
   // the browser executable for a "localaccount" name, and retrieves it
@@ -213,21 +231,14 @@ class ParallelAuthenticator : public Authenticator,
 
   void SetLocalaccount(const std::string& new_name);
 
-  // Stores a hash of |password|, salted with the ascii of |system_salt_|.
-  std::string HashPassword(const std::string& password);
+  // Records OAuth1 access token verification failure for |user_account|.
+  void RecordOAuthCheckFailure(const std::string& user_account);
 
-  // Returns the ascii encoding of the system salt.
-  std::string SaltAsAscii();
+  // Signal login completion status for cases when a new user is added via
+  // an external authentication provider (i.e. GAIA extension).
+  void ResolveLoginCompletionStatus();
 
-  // Converts the binary data |binary| into an ascii hex string and stores
-  // it in |hex_string|.  Not guaranteed to be NULL-terminated.
-  // Returns false if |hex_string| is too small, true otherwise.
-  static bool BinaryToHex(const std::vector<unsigned char>& binary,
-                          const unsigned int binary_len,
-                          char* hex_string,
-                          const unsigned int len);
-
-  // Name of a file, next to chrome, that contains a local account username.
+    // Name of a file, next to chrome, that contains a local account username.
   static const char kLocalaccountFile[];
 
   // Milliseconds until we timeout our attempt to hit ClientLogin.
@@ -251,9 +262,6 @@ class ParallelAuthenticator : public Authenticator,
   scoped_refptr<CryptohomeOp> guest_mounter_;
   scoped_refptr<CryptohomeOp> key_checker_;
 
-  std::string ascii_hash_;
-  chromeos::CryptohomeBlob system_salt_;
-
   // When the user has changed her password, but gives us the old one, we will
   // be able to mount her cryptohome, but online authentication will fail.
   // This allows us to present the same behavior to the caller, regardless
@@ -266,9 +274,11 @@ class ParallelAuthenticator : public Authenticator,
   bool checked_for_localaccount_;  // Needed because empty localaccount_ is ok.
   base::Lock localaccount_lock_;  // A lock around checked_for_localaccount_.
 
+  // True if we use OAuth-based authentication flow.
+  bool using_oauth_;
+
   friend class ResolveChecker;
   friend class ParallelAuthenticatorTest;
-  FRIEND_TEST_ALL_PREFIXES(ParallelAuthenticatorTest, SaltToAscii);
   FRIEND_TEST_ALL_PREFIXES(ParallelAuthenticatorTest, ReadLocalaccount);
   FRIEND_TEST_ALL_PREFIXES(ParallelAuthenticatorTest,
                            ReadLocalaccountTrailingWS);
