@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,17 +6,19 @@
 
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/chromeos/login/rounded_rect_painter.h"
+#include "chrome/browser/chromeos/login/textfield_with_margin.h"
 #include "chrome/browser/chromeos/login/wizard_accessibility_helper.h"
 #include "grit/generated_resources.h"
 #include "grit/locale_settings.h"
 #include "ui/base/keycodes/keyboard_codes.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/controls/button/radio_button.h"
-#include "views/controls/label.h"
-#include "views/grid_layout.h"
-#include "views/standard_layout.h"
-#include "views/window/window.h"
+#include "ui/views/controls/button/radio_button.h"
+#include "ui/views/controls/label.h"
+#include "ui/views/controls/textfield/textfield.h"
+#include "ui/views/layout/grid_layout.h"
+#include "ui/views/layout/layout_constants.h"
+#include "ui/views/widget/widget.h"
 
 using views::Button;
 using views::GridLayout;
@@ -27,7 +29,9 @@ using views::Textfield;
 namespace chromeos {
 
 namespace {
+
 const int kPasswordFieldWidthChars = 20;
+
 }  // namespace
 
 PasswordChangedView::PasswordChangedView(Delegate* delegate,
@@ -46,26 +50,30 @@ bool PasswordChangedView::Accept() {
 }
 
 int PasswordChangedView::GetDialogButtons() const {
- return MessageBoxFlags::DIALOGBUTTON_OK;
+  return ui::DIALOG_BUTTON_OK;
 }
 
 views::View* PasswordChangedView::GetInitiallyFocusedView() {
-  if (!full_sync_disabled_) {
-    return views::DialogDelegate::GetInitiallyFocusedView();
-  } else {
-    DCHECK(old_password_field_);
-    return old_password_field_;
-  }
+  DCHECK(old_password_field_);
+  return old_password_field_;
 }
 
-std::wstring PasswordChangedView::GetWindowTitle() const {
-  return UTF16ToWide(
-      l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_DIALOG_BOX_TITLE));
+ui::ModalType PasswordChangedView::GetModalType() const {
+  return ui::MODAL_TYPE_WINDOW;
+}
+
+views::View* PasswordChangedView::GetContentsView() {
+  return this;
+}
+
+string16 PasswordChangedView::GetWindowTitle() const {
+  return l10n_util::GetStringUTF16(
+      IDS_LOGIN_PASSWORD_CHANGED_DIALOG_BOX_TITLE);
 }
 
 gfx::Size PasswordChangedView::GetPreferredSize() {
   // TODO(nkostylev): Once UI is finalized, create locale settings.
-  return gfx::Size(views::Window::GetLocalizedContentsSize(
+  return gfx::Size(views::Widget::GetLocalizedContentsSize(
       IDS_PASSWORD_CHANGED_DIALOG_WIDTH_CHARS,
       IDS_PASSWORD_CHANGED_DIALOG_HEIGHT_LINES));
 }
@@ -75,7 +83,6 @@ void PasswordChangedView::ViewHierarchyChanged(bool is_add,
                                                views::View* child) {
   if (is_add && child == this) {
     Init();
-    WizardAccessibilityHelper::GetInstance()->MaybeEnableAccessibility(this);
   }
 }
 
@@ -88,29 +95,26 @@ void PasswordChangedView::Init() {
   title_label_ = new Label();
   title_label_->SetFont(title_font);
   title_label_->SetText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_TITLE)));
+      l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_TITLE));
   title_label_->SetHorizontalAlignment(Label::ALIGN_LEFT);
 
   description_label_ = new Label();
   description_label_->SetText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_DESC)));
+      l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_DESC));
   description_label_->SetMultiLine(true);
   description_label_->SetHorizontalAlignment(Label::ALIGN_LEFT);
 
   full_sync_radio_ = new RadioButton(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_RESET)),
-      0);
+      l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_RESET), 0);
   full_sync_radio_->set_listener(this);
   full_sync_radio_->SetMultiLine(true);
 
   delta_sync_radio_ = new RadioButton(
-      UTF16ToWide(
-          l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_MIGRATE)),
-      0);
+      l10n_util::GetStringUTF16(IDS_LOGIN_PASSWORD_CHANGED_MIGRATE), 0);
   delta_sync_radio_->set_listener(this);
   delta_sync_radio_->SetMultiLine(true);
 
-  old_password_field_ = new Textfield(Textfield::STYLE_PASSWORD);
+  old_password_field_ = new TextfieldWithMargin(Textfield::STYLE_OBSCURED);
   old_password_field_->set_text_to_display_when_empty(
       l10n_util::GetStringUTF16(IDS_LOGIN_PREVIOUS_PASSWORD));
   old_password_field_->set_default_width_in_chars(kPasswordFieldWidthChars);
@@ -124,44 +128,36 @@ void PasswordChangedView::Init() {
   column_set->AddColumn(GridLayout::LEADING, GridLayout::FILL, 1,
                         GridLayout::USE_PREF, 0, 0);
   column_set = layout->AddColumnSet(1);
-  column_set->AddPaddingColumn(0, kUnrelatedControlLargeHorizontalSpacing);
+  column_set->AddPaddingColumn(
+      0, views::kUnrelatedControlLargeHorizontalSpacing);
   column_set->AddColumn(GridLayout::LEADING, GridLayout::FILL, 1,
                         GridLayout::USE_PREF, 0, 0);
 
   layout->StartRow(0, 0);
   layout->AddView(title_label_);
-  layout->AddPaddingRow(0, kUnrelatedControlVerticalSpacing);
+  layout->AddPaddingRow(0, views::kUnrelatedControlVerticalSpacing);
 
   layout->StartRow(0, 0);
   layout->AddView(description_label_);
-  layout->AddPaddingRow(0, kRelatedControlVerticalSpacing);
-
-  layout->StartRow(0, 0);
-  layout->AddView(full_sync_radio_);
-  layout->AddPaddingRow(0, kRelatedControlVerticalSpacing);
+  layout->AddPaddingRow(0, views::kRelatedControlVerticalSpacing);
 
   layout->StartRow(0, 0);
   layout->AddView(delta_sync_radio_);
-  layout->AddPaddingRow(0, kRelatedControlSmallVerticalSpacing);
+  layout->AddPaddingRow(0, views::kRelatedControlSmallVerticalSpacing);
 
   layout->StartRow(0, 1);
   layout->AddView(
       old_password_field_, 1, 1, GridLayout::LEADING, GridLayout::CENTER);
-  layout->AddPaddingRow(0, kUnrelatedControlVerticalSpacing);
+  layout->AddPaddingRow(0, views::kUnrelatedControlVerticalSpacing);
 
   layout->StartRow(0, 0);
-  layout->AddView(old_password_field_);
+  layout->AddView(full_sync_radio_);
+  layout->AddPaddingRow(0, views::kRelatedControlVerticalSpacing);
 
+  delta_sync_radio_->SetChecked(true);
   // Disable options if needed.
-  if (!full_sync_disabled_) {
-    full_sync_radio_->SetChecked(true);
-    old_password_field_->SetEnabled(false);
-  } else {
+  if (full_sync_disabled_)
     full_sync_radio_->SetEnabled(false);
-    delta_sync_radio_->SetChecked(true);
-    old_password_field_->SetEnabled(true);
-  }
-
 }
 
 bool PasswordChangedView::ExitDialog() {
@@ -188,4 +184,9 @@ void PasswordChangedView::ButtonPressed(Button* sender,
   }
 }
 
+bool PasswordChangedView::HandleKeyEvent(views::Textfield* sender,
+                                         const views::KeyEvent& keystroke) {
+  return false;
+}
+
 }  // namespace chromeos
