@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,7 @@
 #pragma once
 
 #include "chrome/browser/chromeos/login/login_html_dialog.h"
-#include "gfx/native_widget_types.h"
+#include "ui/gfx/native_widget_types.h"
 
 namespace chromeos {
 
@@ -16,6 +16,10 @@ class ProxySettingsDialog : public LoginHtmlDialog {
   ProxySettingsDialog(LoginHtmlDialog::Delegate* delegate,
                       gfx::NativeWindow window);
 
+ protected:
+  // HtmlDialogUIDelegate implementation.
+  virtual void OnDialogClosed(const std::string& json_retval) OVERRIDE;
+
  private:
   DISALLOW_COPY_AND_ASSIGN(ProxySettingsDialog);
 };
