@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,49 +6,48 @@
 #define CHROME_BROWSER_CHROMEOS_LOGIN_SCREEN_LOCKER_TESTER_H_
 #pragma once
 
+#include <string>
+
 #include "base/basictypes.h"
 
 namespace views {
-class Button;
-class Textfield;
 class Widget;
 }  // namespace views
 
 namespace chromeos {
 
-class ScreenLocker;
-
 namespace test {
 
 // ScreenLockerTester provides access to the private state/function
 // of ScreenLocker class. Used to implement unit tests.
 class ScreenLockerTester {
  public:
+  ScreenLockerTester();
+  virtual ~ScreenLockerTester();
+
   // Returns true if the screen is locked.
-  bool IsLocked();
+  virtual bool IsLocked();
 
   // Injects MockAuthenticate that uses given |user| and |password|.
-  void InjectMockAuthenticator(const char* user, const char* password);
+  virtual void InjectMockAuthenticator(const std::string& user,
+                                       const std::string& password);
+
+  // Sets the password text.
+  virtual void SetPassword(const std::string& password) = 0;
+
+  // Gets the password text.
+  virtual std::string GetPassword() = 0;
 
   // Emulates entring a password.
-  void EnterPassword(const char* password);
+  virtual void EnterPassword(const std::string& password) = 0;
 
   // Emulates the ready message from window manager.
-  void EmulateWindowManagerReady();
+  virtual void EmulateWindowManagerReady() = 0;
 
   // Returns the widget for screen locker window.
-  views::Widget* GetWidget();
+  virtual views::Widget* GetWidget() const = 0;
 
-  views::Widget* GetChildWidget();
-
- private:
-  friend class chromeos::ScreenLocker;
-
-  ScreenLockerTester() {}
-
-  views::Textfield* GetPasswordField();
-
-  DISALLOW_COPY_AND_ASSIGN(ScreenLockerTester);
+  virtual views::Widget* GetChildWidget() const = 0;
 };
 
 }  // namespace test
