@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,7 @@
 
 #include <string>
 
+#include "base/compiler_specific.h"
 #include "chrome/browser/chromeos/login/auth_attempt_state.h"
 #include "chrome/browser/chromeos/login/login_status_consumer.h"
 #include "chrome/common/net/gaia/gaia_auth_consumer.h"
@@ -39,19 +40,14 @@ class TestAttemptState : public AuthAttemptState {
   void PresetCryptohomeStatus(bool cryptohome_outcome, int cryptohome_code);
 
   // To allow state to be queried on the main thread during tests.
-  virtual bool online_complete() { return online_complete_; }
-  virtual const LoginFailure& online_outcome() { return online_outcome_; }
-  virtual const GaiaAuthConsumer::ClientLoginResult& credentials() {
-    return credentials_;
-  }
-  virtual bool is_first_time_user() { return is_first_time_user_; }
-  virtual GaiaAuthFetcher::HostedAccountsSetting hosted_policy() {
-    return hosted_policy_;
-  }
-
-  virtual bool cryptohome_complete() { return cryptohome_complete_; }
-  virtual bool cryptohome_outcome() { return cryptohome_outcome_; }
-  virtual int cryptohome_code() { return cryptohome_code_; }
+  virtual bool online_complete() OVERRIDE;
+  virtual const LoginFailure& online_outcome() OVERRIDE;
+  virtual const GaiaAuthConsumer::ClientLoginResult& credentials() OVERRIDE;
+  virtual bool is_first_time_user() OVERRIDE;
+  virtual GaiaAuthFetcher::HostedAccountsSetting hosted_policy() OVERRIDE;
+  virtual bool cryptohome_complete() OVERRIDE;
+  virtual bool cryptohome_outcome() OVERRIDE;
+  virtual int cryptohome_code() OVERRIDE;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(TestAttemptState);
