@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,8 +17,18 @@ const int kNetworkTimeoutSec = 10;
 
 }  // namespace
 
+WebPageScreen::WebPageScreen() {}
+
+WebPageScreen::~WebPageScreen() {}
+
 ///////////////////////////////////////////////////////////////////////////////
-// WebPageScreen, TabContentsDelegate implementation:
+// WebPageScreen, content::WebContentsDelegate implementation:
+
+bool WebPageScreen::ShouldAddNavigationToHistory(
+    const history::HistoryAddPageArgs& add_page_args,
+    content::NavigationType navigation_type) {
+  return false;
+}
 
 bool WebPageScreen::HandleContextMenu(const ContextMenuParams& params) {
   // Just return true because we don't want to show context menue.
@@ -36,7 +46,8 @@ void WebPageScreen::OnNetworkTimeout() {
 
 void WebPageScreen::StartTimeoutTimer() {
   StopTimeoutTimer();
-  timeout_timer_.Start(TimeDelta::FromSeconds(kNetworkTimeoutSec),
+  timeout_timer_.Start(FROM_HERE,
+                       TimeDelta::FromSeconds(kNetworkTimeoutSec),
                        this,
                        &WebPageScreen::OnNetworkTimeout);
 }
