@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,14 +8,17 @@
 
 #include <string>
 
+#include "base/compiler_specific.h"
 #include "chrome/browser/chromeos/cros/cros_in_process_browser_test.h"
 
 class Browser;
 class Profile;
-class WizardController;
 
 namespace chromeos {
 
+class LoginDisplayHost;
+class WizardController;
+
 // Base class for test related to login wizard and its screens.
 // Instead of creating Chrome browser window it creates login wizard window
 // with specified parameters and allows to customize environment at the
@@ -30,8 +33,8 @@ class WizardInProcessBrowserTest : public CrosInProcessBrowserTest {
   virtual void SetUpWizard() {}
 
   // Overriden from InProcessBrowserTest:
-  virtual Browser* CreateBrowser(Profile* profile);
-  virtual void CleanUpOnMainThread();
+  virtual Browser* CreateBrowser(Profile* profile) OVERRIDE;
+  virtual void CleanUpOnMainThread() OVERRIDE;
 
   WizardController* controller() const { return controller_; }
   void set_controller(WizardController* controller) {
@@ -41,6 +44,7 @@ class WizardInProcessBrowserTest : public CrosInProcessBrowserTest {
  private:
   std::string screen_name_;
   WizardController* controller_;
+  LoginDisplayHost* host_;
 
   DISALLOW_COPY_AND_ASSIGN(WizardInProcessBrowserTest);
 };
@@ -48,4 +52,3 @@ class WizardInProcessBrowserTest : public CrosInProcessBrowserTest {
 }  // namespace chromeos
 
 #endif  // CHROME_BROWSER_CHROMEOS_LOGIN_WIZARD_IN_PROCESS_BROWSER_TEST_H_
-
