@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,10 +10,11 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "chrome/browser/chromeos/cros/network_library.h"
 
 namespace views {
-class WindowDelegate;
+class WidgetDelegate;
 }
 
 namespace chromeos {
@@ -25,17 +26,12 @@ class NetworkLoginObserver : public NetworkLibrary::NetworkManagerObserver {
   explicit NetworkLoginObserver(NetworkLibrary* netlib);
   virtual ~NetworkLoginObserver();
 
-  typedef std::map<std::string, bool> WifiFailureMap;
+  typedef std::map<std::string, bool> NetworkFailureMap;
  private:
-  virtual void CreateModalPopup(views::WindowDelegate* view);
-
-  virtual void RefreshStoredNetworks(const WifiNetworkVector& wifi_networks);
+  void CreateModalPopup(views::WidgetDelegate* view);
 
   // NetworkLibrary::NetworkManagerObserver implementation.
-  virtual void OnNetworkManagerChanged(NetworkLibrary* obj);
-
-  // Wifi networks by service path mapped to if it failed previously.
-  WifiFailureMap wifi_network_failures_;
+  virtual void OnNetworkManagerChanged(NetworkLibrary* obj) OVERRIDE;
 
   DISALLOW_COPY_AND_ASSIGN(NetworkLoginObserver);
 };
