@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,13 +9,13 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/notifications/balloon_view_host.h"
 #include "chrome/browser/notifications/balloon_collection.h"
 #include "chrome/browser/notifications/balloon_collection_base.h"
-#include "chrome/common/notification_registrar.h"
-#include "gfx/point.h"
-#include "gfx/rect.h"
+#include "content/public/browser/notification_registrar.h"
+#include "ui/gfx/point.h"
+#include "ui/gfx/rect.h"
 
 namespace gfx {
 class Size;
@@ -30,7 +30,7 @@ class BalloonViewImpl;
 // chromeos shows the all notifications in the notification panel, and
 // this class does not manage the location of balloons.
 class BalloonCollectionImpl : public BalloonCollection,
-                              public NotificationObserver {
+                              public content::NotificationObserver {
  public:
   // An interface to display balloons on the screen.
   // This is used for unit tests to inject a mock ui implementation.
@@ -60,37 +60,38 @@ class BalloonCollectionImpl : public BalloonCollection,
 
   // BalloonCollectionInterface overrides
   virtual void Add(const Notification& notification,
-                   Profile* profile);
-  virtual bool RemoveById(const std::string& id);
-  virtual bool RemoveBySourceOrigin(const GURL& origin);
-  virtual void RemoveAll();
-  virtual bool HasSpace() const;
-  virtual void ResizeBalloon(Balloon* balloon, const gfx::Size& size);
-  virtual void SetPositionPreference(PositionPreference position) {}
-  virtual void DisplayChanged() {}
-  virtual void OnBalloonClosed(Balloon* source);
-  virtual const Balloons& GetActiveBalloons() { return base_.balloons(); }
+                   Profile* profile) OVERRIDE;
+  virtual bool RemoveById(const std::string& id) OVERRIDE;
+  virtual bool RemoveBySourceOrigin(const GURL& origin) OVERRIDE;
+  virtual void RemoveAll() OVERRIDE;
+  virtual bool HasSpace() const OVERRIDE;
+  virtual void ResizeBalloon(Balloon* balloon, const gfx::Size& size) OVERRIDE;
+  virtual void SetPositionPreference(PositionPreference position) OVERRIDE {}
+  virtual void DisplayChanged() OVERRIDE {}
+  virtual void OnBalloonClosed(Balloon* source) OVERRIDE;
+  virtual const Balloons& GetActiveBalloons() OVERRIDE;
 
   // NotificationObserver overrides:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
-  // Adds a callback for DOMUI message. Returns true if the callback
+  // Adds a callback for WebUI message. Returns true if the callback
   // is succssfully registered, or false otherwise. It fails to add if
   // there is no notification that matches NotificationDelegate::id(),
   // or a callback for given message already exists. The callback
   // object is owned and deleted by callee.
-  bool AddDOMUIMessageCallback(const Notification& notification,
-                               const std::string& message,
-                               MessageCallback* callback);
+  bool AddWebUIMessageCallback(
+      const Notification& notification,
+      const std::string& message,
+      const BalloonViewHost::MessageCallback& callback);
 
   // Adds new system notification.
   // |sticky| is used to indicate that the notification
-  // is sticky and cannot be dismissed by a user. |controls| turns on/off
-  // info label and option/dismiss buttons.
+  // is sticky and cannot be dismissed by a user.
   void AddSystemNotification(const Notification& notification,
-                             Profile* profile, bool sticky, bool controls);
+                             Profile* profile,
+                             bool sticky);
 
   // Updates the notification's content. It uses
   // NotificationDelegate::id() to check the equality of notifications.
@@ -104,6 +105,10 @@ class BalloonCollectionImpl : public BalloonCollection,
   // the updated notification is visible.
   bool UpdateAndShowNotification(const Notification& notification);
 
+  // base_ is embedded, so this is a simple accessor for the number of
+  // balloons in the collection.
+  int count() const { return base_.count(); }
+
   // Injects notification ui. Used to inject a mock implementation in tests.
   void set_notification_ui(NotificationUI* ui) {
     notification_ui_.reset(ui);
@@ -134,7 +139,7 @@ class BalloonCollectionImpl : public BalloonCollection,
 
   scoped_ptr<NotificationUI> notification_ui_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(BalloonCollectionImpl);
 };
