@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,33 +12,40 @@
 #include <string>
 
 #include "base/callback.h"
-#include "gfx/native_widget_types.h"
+#include "base/compiler_specific.h"
+#include "ui/gfx/native_widget_types.h"
 
-class ListValue;
 class GURL;
 
+namespace base {
+class ListValue;
+}
+
 namespace chromeos {
 
-typedef Callback1<const ListValue*>::Type MessageCallback;
-
 class BalloonViewHost : public ::BalloonViewHost {
  public:
-  explicit BalloonViewHost(Balloon* balloon) : ::BalloonViewHost(balloon) {}
+  typedef base::Callback<void(const base::ListValue*)> MessageCallback;
+
+  explicit BalloonViewHost(Balloon* balloon);
   virtual ~BalloonViewHost();
 
-  // Adds a callback for DOMUI message. Returns true if the callback
+  // Adds a callback for WebUI message. Returns true if the callback
   // is succssfully registered, or false otherwise. It fails to add if
   // a callback for given message already exists. The callback object
   // is owned and deleted by callee.
-  bool AddDOMUIMessageCallback(const std::string& message,
-                               MessageCallback* callback);
-
-  // Process DOMUI message.
-  virtual void ProcessDOMUIMessage(const ViewHostMsg_DomMessage_Params& params);
+  bool AddWebUIMessageCallback(const std::string& message,
+                               const MessageCallback& callback);
 
  private:
+  // WebContentsDelegate
+  virtual void WebUISend(content::WebContents* tab,
+                         const GURL& source_url,
+                         const std::string& name,
+                         const base::ListValue& args) OVERRIDE;
+
   // A map of message name -> message handling callback.
-  typedef std::map<std::string, MessageCallback*> MessageCallbackMap;
+  typedef std::map<std::string, MessageCallback> MessageCallbackMap;
   MessageCallbackMap message_callbacks_;
 
   DISALLOW_COPY_AND_ASSIGN(BalloonViewHost);
