@@ -1,10 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/bind.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
 #include "base/string16.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
@@ -17,22 +18,16 @@
 #include "chrome/browser/notifications/notification_test_util.h"
 #include "chrome/browser/notifications/notification_ui_manager.h"
 #include "chrome/browser/ui/browser.h"
-#include "chrome/common/notification_service.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/public/browser/notification_service.h"
 #include "ui/base/x/x11_util.h"
 
-namespace {
-
-// The name of ChromeOS's window manager.
-const char* kChromeOsWindowManagerName = "chromeos-wm";
-
-}  // namespace
-
 namespace chromeos {
 
 class NotificationTest : public InProcessBrowserTest,
-                         public NotificationObserver {
+                         public content::NotificationObserver {
  public:
   NotificationTest()
       : under_chromeos_(false),
@@ -40,7 +35,7 @@ class NotificationTest : public InProcessBrowserTest,
         expected_(PanelController::INITIAL) {
   }
 
-  void HandleDOMUIMessage(const ListValue* value) {
+  void HandleWebUIMessage(const ListValue* value) {
     MessageLoop::current()->Quit();
   }
 
@@ -48,13 +43,12 @@ class NotificationTest : public InProcessBrowserTest,
   virtual void SetUp() {
     // Detect if we're running under ChromeOS WindowManager. See
     // the description for "under_chromeos_" below for why we need this.
-    std::string wm_name;
-    bool wm_name_valid = ui::GetWindowManagerName(&wm_name);
+    ui::WindowManagerName wm_type = ui::GuessWindowManager();
     // NOTE: On Chrome OS the wm and Chrome are started in parallel. This
     // means it's possible for us not to be able to get the name of the window
     // manager. We assume that when this happens we're on Chrome OS.
-    under_chromeos_ = (!wm_name_valid ||
-                       wm_name == kChromeOsWindowManagerName);
+    under_chromeos_ = (wm_type == ui::WM_CHROME_OS ||
+                       wm_type == ui::WM_UNKNOWN);
     InProcessBrowserTest::SetUp();
   }
 
@@ -93,6 +87,15 @@ class NotificationTest : public InProcessBrowserTest,
     }
   }
 
+  // Busy loop to wait until the view becomes visible in the panel.
+  void WaitForVisible(BalloonViewImpl* view) {
+    WaitForResize(view);
+    NotificationPanelTester* tester = GetNotificationPanel()->GetTester();
+    while (!tester->IsVisible(view)) {
+      ui_test_utils::RunAllPendingInMessageLoop();
+    }
+  }
+
   // Busy loop to wait until the webkit give some size to the notification.
   void WaitForResize(BalloonViewImpl* view) {
     while (view->bounds().IsEmpty()) {
@@ -101,10 +104,10 @@ class NotificationTest : public InProcessBrowserTest,
   }
 
   // NotificationObserver overrides.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details) {
-    ASSERT_TRUE(NotificationType::PANEL_STATE_CHANGED == type);
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) {
+    ASSERT_TRUE(chrome::NOTIFICATION_PANEL_STATE_CHANGED == type);
     PanelController::State* state =
         reinterpret_cast<PanelController::State*>(details.map_key());
     state_ = *state;
@@ -164,8 +167,9 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestBasic) {
   ui_test_utils::RunAllPendingInMessageLoop();
 }
 
-// [CLOSED] -add->[STICKY_AND_NEW] -mouse-> [KEEP_SIZE] -remove/add->
-// [KEEP_SIZE] -remove-> [CLOSED] -add-> [STICKY_AND_NEW] -remove-> [CLOSED]
+// [CLOSED] -add->[STICKY_AND_NEW] -mouse-> [STICKY_AND_NEW] -remove/add->
+// [STICKY_AND_NEW] -remove-> [CLOSED] -add-> [STICKY_AND_NEW] -remove->
+// [CLOSED]
 IN_PROC_BROWSER_TEST_F(NotificationTest, TestKeepSizeState) {
   BalloonCollectionImpl* collection = GetBalloonCollectionImpl();
   NotificationPanel* panel = GetNotificationPanel();
@@ -180,25 +184,25 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestKeepSizeState) {
   EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
 
   panel->OnMouseMotion(gfx::Point(10, 10));
-  EXPECT_EQ(NotificationPanel::KEEP_SIZE, tester->state());
+  EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
 
   collection->RemoveById("1");
   ui_test_utils::RunAllPendingInMessageLoop();
   EXPECT_EQ(1, tester->GetNewNotificationCount());
   EXPECT_EQ(1, tester->GetNotificationCount());
-  EXPECT_EQ(NotificationPanel::KEEP_SIZE, tester->state());
+  EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
 
   collection->Add(NewMockNotification("1"), browser()->profile());
   ui_test_utils::RunAllPendingInMessageLoop();
   EXPECT_EQ(2, tester->GetNewNotificationCount());
   EXPECT_EQ(2, tester->GetNotificationCount());
-  EXPECT_EQ(NotificationPanel::KEEP_SIZE, tester->state());
+  EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
 
   collection->RemoveById("1");
   ui_test_utils::RunAllPendingInMessageLoop();
   EXPECT_EQ(1, tester->GetNewNotificationCount());
   EXPECT_EQ(1, tester->GetNotificationCount());
-  EXPECT_EQ(NotificationPanel::KEEP_SIZE, tester->state());
+  EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
 
   collection->RemoveById("2");
   ui_test_utils::RunAllPendingInMessageLoop();
@@ -222,7 +226,7 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestSystemNotification) {
   NotificationPanelTester* tester = panel->GetTester();
 
   Notification notify = NewMockNotification(delegate.get());
-  collection->AddSystemNotification(notify, browser()->profile(), true, false);
+  collection->AddSystemNotification(notify, browser()->profile(), true);
 
   EXPECT_EQ(1, tester->GetNewNotificationCount());
   EXPECT_EQ(1, tester->GetStickyNotificationCount());
@@ -288,14 +292,12 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestStateTransition1) {
 // This test depends on the fact that the panel state change occurs
 // quicker than stale timeout, thus the stale timeout cannot be set to
 // 0. This test explicitly controls the stale state instead.
-
-// TODO(oshima): bug chromium-os:7306 Fix this flaky test on ChromeOS.
-IN_PROC_BROWSER_TEST_F(NotificationTest, FLAKY_TestStateTransition2) {
+IN_PROC_BROWSER_TEST_F(NotificationTest, TestStateTransition2) {
   // Register observer here as the registration does not work in SetUp().
-  NotificationRegistrar registrar;
+  content::NotificationRegistrar registrar;
   registrar.Add(this,
-                NotificationType::PANEL_STATE_CHANGED,
-                NotificationService::AllSources());
+                chrome::NOTIFICATION_PANEL_STATE_CHANGED,
+                content::NotificationService::AllSources());
 
   BalloonCollectionImpl* collection = GetBalloonCollectionImpl();
   NotificationPanel* panel = GetNotificationPanel();
@@ -329,7 +331,7 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, FLAKY_TestStateTransition2) {
 
   // The panel must be expanded again when a new system notification is added.
   collection->AddSystemNotification(
-      NewMockNotification("3"), browser()->profile(), true, false);
+      NewMockNotification("3"), browser()->profile(), true);
   EXPECT_EQ(3, tester->GetNotificationCount());
   EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
   WaitForPanelState(tester, PanelController::EXPANDED);
@@ -359,10 +361,10 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, FLAKY_TestStateTransition2) {
 }
 
 IN_PROC_BROWSER_TEST_F(NotificationTest, TestCleanupOnExit) {
-  NotificationRegistrar registrar;
+  content::NotificationRegistrar registrar;
   registrar.Add(this,
-                NotificationType::PANEL_STATE_CHANGED,
-                NotificationService::AllSources());
+                chrome::NOTIFICATION_PANEL_STATE_CHANGED,
+                content::NotificationService::AllSources());
 
   BalloonCollectionImpl* collection = GetBalloonCollectionImpl();
   NotificationPanel* panel = GetNotificationPanel();
@@ -406,8 +408,8 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestCloseOpen) {
   EXPECT_EQ(NotificationPanel::CLOSED, tester->state());
 }
 
-// TODO(oshima): bug chromium-os:7139 Fix this flaky test on ChromeOS.
-IN_PROC_BROWSER_TEST_F(NotificationTest, FLAKY_TestScrollBalloonToVisible) {
+// TODO(lipalani): http://crbug.com/83123
+IN_PROC_BROWSER_TEST_F(NotificationTest, DISABLED_TestScrollBalloonToVisible) {
   BalloonCollectionImpl* collection = GetBalloonCollectionImpl();
   NotificationPanel* panel = GetNotificationPanel();
   NotificationPanelTester* tester = panel->GetTester();
@@ -419,27 +421,25 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, FLAKY_TestScrollBalloonToVisible) {
   // new notification is always visible
   for (int i = 0; i < create_count; i++) {
     {
-      SCOPED_TRACE(base::StringPrintf("new n%d", i));
+      SCOPED_TRACE(base::StringPrintf("new normal %d", i));
       std::string id = base::StringPrintf("n%d", i);
       collection->Add(NewMockNotification(id), profile);
       EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
       BalloonViewImpl* view =
           tester->GetBalloonView(collection, NewMockNotification(id));
-      WaitForResize(view);
-      EXPECT_TRUE(tester->IsVisible(view));
+      WaitForVisible(view);
     }
     {
-      SCOPED_TRACE(base::StringPrintf("new s%d", i));
+      SCOPED_TRACE(base::StringPrintf("new system %d", i));
       std::string id = base::StringPrintf("s%d", i);
       collection->AddSystemNotification(
-          NewMockNotification(id), browser()->profile(), true, false);
-      ui_test_utils::RunAllPendingInMessageLoop();
+          NewMockNotification(id), browser()->profile(), true);
       BalloonViewImpl* view =
           tester->GetBalloonView(collection, NewMockNotification(id));
-      WaitForResize(view);
-      EXPECT_TRUE(tester->IsVisible(view));
+      WaitForVisible(view);
     }
   }
+
   // Update should not change the visibility
   for (int i = 0; i < create_count; i++) {
     {
@@ -484,7 +484,8 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, FLAKY_TestScrollBalloonToVisible) {
   }
 }
 
-IN_PROC_BROWSER_TEST_F(NotificationTest, TestActivateDeactivate) {
+// Seems to be flaky. http://crbug.com/84427
+IN_PROC_BROWSER_TEST_F(NotificationTest, FLAKY_TestActivateDeactivate) {
   BalloonCollectionImpl* collection = GetBalloonCollectionImpl();
   NotificationPanel* panel = GetNotificationPanel();
   NotificationPanelTester* tester = panel->GetTester();
@@ -492,7 +493,7 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestActivateDeactivate) {
 
   collection->Add(NewMockNotification("1"), profile);
   collection->AddSystemNotification(
-      NewMockNotification("2"), profile, true, false);
+      NewMockNotification("2"), profile, true);
   ui_test_utils::RunAllPendingInMessageLoop();
   EXPECT_EQ(NotificationPanel::STICKY_AND_NEW, tester->state());
   BalloonViewImpl* view1 =
@@ -525,7 +526,7 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestCloseDismissAllNonSticky) {
 
   collection->Add(NewMockNotification("1"), profile);
   collection->AddSystemNotification(
-      NewMockNotification("2"), profile, true, false);
+      NewMockNotification("2"), profile, true);
   collection->Add(NewMockNotification("3"), profile);
 
   ui_test_utils::RunAllPendingInMessageLoop();
@@ -540,41 +541,39 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestCloseDismissAllNonSticky) {
   EXPECT_EQ(1, tester->GetStickyNotificationCount());
 }
 
-IN_PROC_BROWSER_TEST_F(NotificationTest, TestAddDOMUIMessageCallback) {
+IN_PROC_BROWSER_TEST_F(NotificationTest, TestAddWebUIMessageCallback) {
   BalloonCollectionImpl* collection = GetBalloonCollectionImpl();
   Profile* profile = browser()->profile();
 
   collection->AddSystemNotification(
-      NewMockNotification("1"), profile, false, false);
+      NewMockNotification("1"), profile, false);
 
-  EXPECT_TRUE(collection->AddDOMUIMessageCallback(
+  EXPECT_TRUE(collection->AddWebUIMessageCallback(
       NewMockNotification("1"),
       "test",
-      NewCallback(
-          static_cast<NotificationTest*>(this),
-          &NotificationTest::HandleDOMUIMessage)));
+      base::Bind(&NotificationTest::HandleWebUIMessage,
+          base::Unretained(static_cast<NotificationTest*>(this)))));
 
   // Adding callback for the same message twice should fail.
-  EXPECT_FALSE(collection->AddDOMUIMessageCallback(
+  EXPECT_FALSE(collection->AddWebUIMessageCallback(
       NewMockNotification("1"),
       "test",
-      NewCallback(
-          static_cast<NotificationTest*>(this),
-          &NotificationTest::HandleDOMUIMessage)));
+      base::Bind(&NotificationTest::HandleWebUIMessage,
+          base::Unretained(static_cast<NotificationTest*>(this)))));
 
   // Adding callback to nonexistent notification should fail.
-  EXPECT_FALSE(collection->AddDOMUIMessageCallback(
+  EXPECT_FALSE(collection->AddWebUIMessageCallback(
       NewMockNotification("2"),
       "test1",
-      NewCallback(
-          static_cast<NotificationTest*>(this),
-          &NotificationTest::HandleDOMUIMessage)));
+      base::Bind(&NotificationTest::HandleWebUIMessage,
+          base::Unretained(static_cast<NotificationTest*>(this)))));
 }
 
-IN_PROC_BROWSER_TEST_F(NotificationTest, TestDOMUIMessageCallback) {
+// Occasional crash: http://crbug.com/96461
+IN_PROC_BROWSER_TEST_F(NotificationTest, TestWebUIMessageCallback) {
   BalloonCollectionImpl* collection = GetBalloonCollectionImpl();
   Profile* profile = browser()->profile();
-  // a notification that sends 'test' domui message back to chrome.
+  // A notification that sends 'test' WebUI message back to chrome.
   const GURL content_url(
       "data:text/html;charset=utf-8,"
       "<html><script>function send() { chrome.send('test', ['']); }</script>"
@@ -583,14 +582,12 @@ IN_PROC_BROWSER_TEST_F(NotificationTest, TestDOMUIMessageCallback) {
       Notification(GURL(), content_url, string16(), string16(),
                    new MockNotificationDelegate("1")),
       profile,
-      false,
       false);
-  EXPECT_TRUE(collection->AddDOMUIMessageCallback(
+  EXPECT_TRUE(collection->AddWebUIMessageCallback(
       NewMockNotification("1"),
       "test",
-      NewCallback(
-          static_cast<NotificationTest*>(this),
-          &NotificationTest::HandleDOMUIMessage)));
+      base::Bind(&NotificationTest::HandleWebUIMessage,
+          base::Unretained(static_cast<NotificationTest*>(this)))));
   MessageLoop::current()->Run();
 }
 
