@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,23 +8,26 @@
 
 #include <algorithm>
 
+#include "base/bind.h"
 #include "chrome/browser/chromeos/notifications/balloon_collection_impl.h"
 #include "chrome/browser/chromeos/notifications/balloon_view.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_source.h"
-#include "gfx/canvas.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_source.h"
 #include "grit/generated_resources.h"
-#include "third_party/cros/chromeos_wm_ipc_enums.h"
+#include "third_party/cros_system_api/window_manager/chromeos_wm_ipc_enums.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/background.h"
-#include "views/controls/native/native_view_host.h"
-#include "views/controls/scroll_view.h"
-#include "views/widget/root_view.h"
-#include "views/widget/widget_gtk.h"
+#include "ui/gfx/canvas.h"
+#include "ui/views/background.h"
+#include "ui/views/controls/native/native_view_host.h"
+#include "ui/views/controls/scroll_view.h"
+#include "ui/views/widget/native_widget_gtk.h"
 
 #define SET_STATE(state) SetState(state, __PRETTY_FUNCTION__)
 
+using views::Widget;
+
 namespace {
 // Minimum and maximum size of balloon content.
 const int kBalloonMinWidth = 300;
@@ -39,6 +42,9 @@ const int kMaxPanelHeight = 400;
 // The duration for a new notification to become stale.
 const int kStaleTimeoutInSeconds = 10;
 
+// Panel Background color
+const SkColor kPanelBackground = 0xFFEEEEEE;
+
 using chromeos::BalloonViewImpl;
 using chromeos::NotificationPanel;
 
@@ -66,91 +72,72 @@ chromeos::BalloonViewImpl* GetBalloonViewOf(const Balloon* balloon) {
   return static_cast<chromeos::BalloonViewImpl*>(balloon->view());
 }
 
-// A WidgetGtk to preevnt recursive calls to PaintNow, which is observed
-// with gtk 2.18.6. See http://crbug.com/42235 for more details.
-class PanelWidget : public views::WidgetGtk {
- public:
-  PanelWidget() : WidgetGtk(TYPE_WINDOW), painting_(false) {
-  }
-
-  virtual ~PanelWidget() {
-    // Enable double buffering because the panel has both pure views control and
-    // native controls (scroll bar).
-    EnableDoubleBuffer(true);
-  }
-
-  // views::WidgetGtk overrides.
-  virtual void PaintNow(const gfx::Rect& update_rect) {
-    if (!painting_) {
-      painting_ = true;
-      WidgetGtk::PaintNow(update_rect);
-      painting_ = false;
-    }
-  }
-
- private:
-  // True if the painting is in progress.
-  bool painting_;
-
-  DISALLOW_COPY_AND_ASSIGN(PanelWidget);
-};
-
-// A WidgetGtk that covers entire ScrollView's viewport. Without this,
-// all renderer's native gtk widgets are moved one by one via
-// View::VisibleBoundsInRootChanged() notification, which makes
-// scrolling not smooth.
-class ViewportWidget : public views::WidgetGtk {
+// A Widget that covers entire ScrollView's viewport. Without this, all
+// renderer's native gtk widgets are moved one by one via
+// View::VisibleBoundsInRootChanged() notification, which makes scrolling not
+// smooth.
+class ViewportWidget : public views::Widget {
  public:
   explicit ViewportWidget(chromeos::NotificationPanel* panel)
-      : WidgetGtk(views::WidgetGtk::TYPE_CHILD),
-        panel_(panel) {
+      : panel_(panel),
+        pointer_inside_panel_(false) {
   }
 
   void UpdateControl() {
-    if (last_point_.get())
-      panel_->OnMouseMotion(*last_point_.get());
+    if (pointer_inside_panel_)
+      panel_->OnMouseMotion(last_point_);
   }
 
-  // views::WidgetGtk overrides.
-  virtual gboolean OnMotionNotify(GtkWidget* widget, GdkEventMotion* event) {
-    gboolean result = WidgetGtk::OnMotionNotify(widget, event);
-
-    int x = 0, y = 0;
-    GetContainedWidgetEventCoordinates(event, &x, &y);
-
-    // The window_contents_' allocation has been moved off the top left
-    // corner, so we need to adjust it.
-    GtkAllocation alloc = widget->allocation;
-    x -= alloc.x;
-    y -= alloc.y;
-
-    if (!last_point_.get()) {
-      last_point_.reset(new gfx::Point(x, y));
-    } else {
-      last_point_->set_x(x);
-      last_point_->set_y(y);
+  // views::Widget overrides:
+  virtual bool OnMouseEvent(const views::MouseEvent& event) OVERRIDE {
+    bool result = Widget::OnMouseEvent(event);
+    switch (event.type()) {
+      case ui::ET_MOUSE_MOVED:
+        OnMouseMoved(event);
+        break;
+      case ui::ET_MOUSE_EXITED:
+        OnMouseExited(event);
+        break;
+      default:
+        break;
     }
-    panel_->OnMouseMotion(*last_point_.get());
     return result;
   }
 
-  virtual gboolean OnLeaveNotify(GtkWidget* widget, GdkEventCrossing* event) {
-    gboolean result = views::WidgetGtk::OnLeaveNotify(widget, event);
+ private:
+  void OnMouseMoved(const views::MouseEvent& event) {
+    // Convert the point to the panel's coordinate system.
+    Widget* top_level_widget = GetTopLevelWidget();
+    // TODO: remove cast when this extends Widget.
+    DCHECK(top_level_widget != static_cast<Widget*>(this));
+    gfx::Rect panel_rect(event.location(), gfx::Size());
+    Widget::ConvertRect(this, top_level_widget, &panel_rect);
+    last_point_ = panel_rect.origin();
+    pointer_inside_panel_ = true;
+    panel_->OnMouseMotion(last_point_);
+  }
+
+  void OnMouseExited(const views::MouseEvent& event) {
+    // Filter gtk's fake mouse leave events that are issued when
+    // window activation changes.
+    if (event.location().x() == 0 && event.location().y() == 0)
+      return;
+
     // Leave notify can happen if the mouse moves into the child gdk window.
     // Make sure the mouse is outside of the panel.
-    gfx::Point p(event->x_root, event->y_root);
-    gfx::Rect bounds;
-    GetBounds(&bounds, true);
-    if (!bounds.Contains(p)) {
+    gfx::Rect bounds = GetWindowScreenBounds();
+    if (!bounds.Contains(event.location().x() + bounds.x(),
+                         event.location().y() + bounds.y())) {
       panel_->OnMouseLeave();
-      last_point_.reset();
+      pointer_inside_panel_ = false;
     }
-    return result;
   }
 
  private:
   chromeos::NotificationPanel* panel_;
-  scoped_ptr<gfx::Point> last_point_;
+  gfx::Point last_point_;
+  bool pointer_inside_panel_;
+
   DISALLOW_COPY_AND_ASSIGN(ViewportWidget);
 };
 
@@ -170,8 +157,8 @@ class BalloonSubContainer : public views::View {
   virtual void Layout() {
     // Layout bottom up
     int height = 0;
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      views::View* child = GetChildViewAt(i);
+    for (int i = child_count() - 1; i >= 0; --i) {
+      views::View* child = child_at(i);
       child->SetBounds(0, height, child->width(), child->height());
       height += child->height() + margin_;
     }
@@ -182,8 +169,8 @@ class BalloonSubContainer : public views::View {
   void UpdateBounds() {
     int height = 0;
     int max_width = 0;
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      views::View* child = GetChildViewAt(i);
+    for (int i = child_count() - 1; i >= 0; --i) {
+      views::View* child = child_at(i);
       height += child->height() + margin_;
       max_width = std::max(max_width, child->width());
     }
@@ -197,9 +184,8 @@ class BalloonSubContainer : public views::View {
   // Returns the bounds that covers new notifications.
   gfx::Rect GetNewBounds() {
     gfx::Rect rect;
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      BalloonViewImpl* view =
-          static_cast<BalloonViewImpl*>(GetChildViewAt(i));
+    for (int i = child_count() - 1; i >= 0; --i) {
+      BalloonViewImpl* view = static_cast<BalloonViewImpl*>(child_at(i));
       if (!view->stale()) {
         if (rect.IsEmpty()) {
           rect = view->bounds();
@@ -214,9 +200,8 @@ class BalloonSubContainer : public views::View {
   // Returns # of new notifications.
   int GetNewCount() {
     int count = 0;
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      BalloonViewImpl* view =
-          static_cast<BalloonViewImpl*>(GetChildViewAt(i));
+    for (int i = child_count() - 1; i >= 0; --i) {
+      BalloonViewImpl* view = static_cast<BalloonViewImpl*>(child_at(i));
       if (!view->stale())
         count++;
     }
@@ -225,25 +210,22 @@ class BalloonSubContainer : public views::View {
 
   // Make all notifications stale.
   void MakeAllStale() {
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      BalloonViewImpl* view =
-          static_cast<BalloonViewImpl*>(GetChildViewAt(i));
+    for (int i = child_count() - 1; i >= 0; --i) {
+      BalloonViewImpl* view = static_cast<BalloonViewImpl*>(child_at(i));
       view->set_stale();
     }
   }
 
   void DismissAll() {
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      BalloonViewImpl* view =
-          static_cast<BalloonViewImpl*>(GetChildViewAt(i));
+    for (int i = child_count() - 1; i >= 0; --i) {
+      BalloonViewImpl* view = static_cast<BalloonViewImpl*>(child_at(i));
       view->Close(true);
     }
   }
 
   BalloonViewImpl* FindBalloonView(const Notification& notification) {
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      BalloonViewImpl* view =
-          static_cast<BalloonViewImpl*>(GetChildViewAt(i));
+    for (int i = child_count() - 1; i >= 0; --i) {
+      BalloonViewImpl* view = static_cast<BalloonViewImpl*>(child_at(i));
       if (view->IsFor(notification)) {
         return view;
       }
@@ -254,14 +236,24 @@ class BalloonSubContainer : public views::View {
   BalloonViewImpl* FindBalloonView(const gfx::Point point) {
     gfx::Point copy(point);
     ConvertPointFromWidget(this, &copy);
-    for (int i = GetChildViewCount() - 1; i >= 0; --i) {
-      views::View* view = GetChildViewAt(i);
+    for (int i = child_count() - 1; i >= 0; --i) {
+      views::View* view = child_at(i);
       if (view->bounds().Contains(copy))
         return static_cast<BalloonViewImpl*>(view);
     }
     return NULL;
   }
 
+  // Returns true if the |view| is in the container.
+  // |view| can be a deleted pointer - we do not dereference it.
+  bool HasChildView(View* view) const {
+    for (int i = 0; i < child_count(); ++i) {
+      if (child_at(i) == view)
+        return true;
+    }
+    return false;
+  }
+
  private:
   gfx::Size preferred_size_;
   int margin_;
@@ -287,8 +279,8 @@ class BalloonContainer : public views::View {
   // views::View overrides.
   virtual void Layout() {
     int margin =
-        (sticky_container_->GetChildViewCount() != 0 &&
-         non_sticky_container_->GetChildViewCount() != 0) ?
+        (sticky_container_->child_count() != 0 &&
+         non_sticky_container_->child_count() != 0) ?
         margin_ : 0;
     sticky_container_->SetBounds(
         0, 0, width(), sticky_container_->height());
@@ -322,9 +314,9 @@ class BalloonContainer : public views::View {
   bool Update(Balloon* balloon) {
     BalloonViewImpl* view = GetBalloonViewOf(balloon);
     View* container = NULL;
-    if (sticky_container_->HasChildView(view)) {
+    if (view->parent() == sticky_container_) {
       container = sticky_container_;
-    } else if (non_sticky_container_->HasChildView(view)) {
+    } else if (view->parent() == non_sticky_container_) {
       container = non_sticky_container_;
     }
     if (container) {
@@ -345,8 +337,8 @@ class BalloonContainer : public views::View {
 
   // Returns the number of notifications added to the panel.
   int GetNotificationCount() {
-    return sticky_container_->GetChildViewCount() +
-        non_sticky_container_->GetChildViewCount();
+    return sticky_container_->child_count() +
+        non_sticky_container_->child_count();
   }
 
   // Returns the # of new notifications.
@@ -357,16 +349,17 @@ class BalloonContainer : public views::View {
 
   // Returns the # of sticky and new notifications.
   int GetStickyNewNotificationCount() {
-    return sticky_container_->GetChildViewCount() +
+    return sticky_container_->child_count() +
         non_sticky_container_->GetNewCount();
   }
 
   // Returns the # of sticky notifications.
   int GetStickyNotificationCount() {
-    return sticky_container_->GetChildViewCount();
+    return sticky_container_->child_count();
   }
 
   // Returns true if the |view| is contained in the panel.
+  // |view| can be a deleted pointer - we do not dereference it.
   bool HasBalloonView(View* view) {
     return sticky_container_->HasChildView(view) ||
         non_sticky_container_->HasChildView(view);
@@ -429,7 +422,6 @@ NotificationPanel::NotificationPanel()
       panel_widget_(NULL),
       container_host_(NULL),
       state_(CLOSED),
-      task_factory_(this),
       min_bounds_(0, 0, kBalloonMinWidth, kBalloonMinHeight),
       stale_timeout_(1000 * kStaleTimeoutInSeconds),
       active_(NULL),
@@ -442,16 +434,22 @@ NotificationPanel::~NotificationPanel() {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// NottificationPanel public.
+// NotificationPanel public.
 
 void NotificationPanel::Show() {
   if (!panel_widget_) {
+    panel_widget_ = new views::Widget;
     // TODO(oshima): Using window because Popup widget behaves weird
     // when resizing. This needs to be investigated.
-    panel_widget_ = new PanelWidget();
+    Widget::InitParams params(Widget::InitParams::TYPE_WINDOW_FRAMELESS);
+    // Enable double buffering because the panel has both pure views
+    // control and native controls (scroll bar).
+    params.double_buffer = true;
+
     gfx::Rect bounds = GetPreferredBounds();
     bounds = bounds.Union(min_bounds_);
-    panel_widget_->Init(NULL, bounds);
+    params.bounds = bounds;
+    panel_widget_->Init(params);
     // Set minimum bounds so that it can grow freely.
     gtk_widget_set_size_request(GTK_WIDGET(panel_widget_->GetNativeView()),
                                 min_bounds_.width(), min_bounds_.height());
@@ -461,24 +459,28 @@ void NotificationPanel::Show() {
 
     panel_widget_->SetContentsView(scroll_view_.get());
 
+#if defined(USE_AURA)
+    native->AttachToView(balloon_container_.get());
+#else
     // Add the view port after scroll_view is attached to the panel widget.
-    ViewportWidget* widget = new ViewportWidget(this);
-    container_host_ = widget;
-    container_host_->Init(NULL, gfx::Rect());
+    ViewportWidget* viewport_widget = new ViewportWidget(this);
+    container_host_ = viewport_widget;
+    container_host_->Init(
+        views::Widget::InitParams(views::Widget::InitParams::TYPE_CONTROL));
     container_host_->SetContentsView(balloon_container_.get());
-    // The window_contents_ is onwed by the WidgetGtk. Increase ref count
+    // The window_contents_ is onwed by the Widget. Increase ref count
     // so that window_contents does not get deleted when detached.
-    g_object_ref(widget->window_contents());
-    native->Attach(widget->window_contents());
-
+    g_object_ref(viewport_widget->GetNativeView());
+    native->Attach(viewport_widget->GetNativeView());
+#endif
     UnregisterNotification();
     panel_controller_.reset(
         new PanelController(this, GTK_WINDOW(panel_widget_->GetNativeView())));
     panel_controller_->Init(false /* don't focus when opened */,
                             gfx::Rect(0, 0, kBalloonMinWidth, 1), 0,
                             WM_IPC_PANEL_USER_RESIZE_VERTICALLY);
-    registrar_.Add(this, NotificationType::PANEL_STATE_CHANGED,
-                   Source<PanelController>(panel_controller_.get()));
+    registrar_.Add(this, chrome::NOTIFICATION_PANEL_STATE_CHANGED,
+                   content::Source<PanelController>(panel_controller_.get()));
   }
   panel_widget_->Show();
 }
@@ -486,22 +488,25 @@ void NotificationPanel::Show() {
 void NotificationPanel::Hide() {
   balloon_container_->DismissAllNonSticky();
   if (panel_widget_) {
-    container_host_->GetRootView()->RemoveChildView(balloon_container_.get());
+    if (container_host_)
+      container_host_->GetRootView()->RemoveChildView(balloon_container_.get());
 
     views::NativeViewHost* native =
         static_cast<views::NativeViewHost*>(scroll_view_->GetContents());
     native->Detach();
     scroll_view_->SetContents(NULL);
-    container_host_->Hide();
-    container_host_->CloseNow();
-    container_host_ = NULL;
+    if (container_host_) {
+      container_host_->Hide();
+      container_host_->CloseNow();
+      container_host_ = NULL;
+    }
 
     UnregisterNotification();
     panel_controller_->Close();
     MessageLoop::current()->DeleteSoon(FROM_HERE, panel_controller_.release());
     // We need to remove & detach the scroll view from hierarchy to
     // avoid GTK deleting child.
-    // TODO(oshima): handle this details in WidgetGtk.
+    // TODO(oshima): handle this details in NativeWidgetGtk.
     panel_widget_->GetRootView()->RemoveChildView(scroll_view_.get());
     panel_widget_->Close();
     panel_widget_ = NULL;
@@ -619,13 +624,18 @@ void NotificationPanel::ClosePanel() {
   UpdatePanel(false);
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// NotificationObserver overrides.
+void NotificationPanel::ActivatePanel() {
+  if (active_)
+    active_->Activated();
+}
 
-void NotificationPanel::Observe(NotificationType type,
-                                const NotificationSource& source,
-                                const NotificationDetails& details) {
-  DCHECK(type == NotificationType::PANEL_STATE_CHANGED);
+////////////////////////////////////////////////////////////////////////////////
+// content::NotificationObserver overrides.
+
+void NotificationPanel::Observe(int type,
+                                const content::NotificationSource& source,
+                                const content::NotificationDetails& details) {
+  DCHECK(type == chrome::NOTIFICATION_PANEL_STATE_CHANGED);
   PanelController::State* state =
       reinterpret_cast<PanelController::State*>(details.map_key());
   switch (*state) {
@@ -661,7 +671,15 @@ void NotificationPanel::OnMouseLeave() {
 
 void NotificationPanel::OnMouseMotion(const gfx::Point& point) {
   SetActiveView(balloon_container_->FindBalloonView(point));
-  SET_STATE(KEEP_SIZE);
+  // We need to set the focus to scroll view to get mouse wheel
+  // working. Setting focus when mouse moves on the panel
+  // because focus may be taken by other view.
+  scroll_view_->RequestFocus();
+  // This method used to set KEEP_SIZE state. However,
+  // some html notifications may want to change their size,
+  // and setting KEEP_SIZE caused them to behave differently
+  // depending on whether user moved mouse over notification
+  // or not.
 }
 
 NotificationPanelTester* NotificationPanel::GetTester() {
@@ -678,18 +696,21 @@ void NotificationPanel::Init() {
   balloon_container_.reset(new BalloonContainer(1));
   balloon_container_->set_parent_owned(false);
   balloon_container_->set_background(
-      views::Background::CreateSolidBackground(ResourceBundle::frame_color));
+      views::Background::CreateSolidBackground(kPanelBackground));
 
   scroll_view_.reset(new views::ScrollView());
+  scroll_view_->set_focusable(true);
   scroll_view_->set_parent_owned(false);
   scroll_view_->set_background(
       views::Background::CreateSolidBackground(SK_ColorWHITE));
 }
 
 void NotificationPanel::UnregisterNotification() {
-  if (panel_controller_.get())
-    registrar_.Remove(this, NotificationType::PANEL_STATE_CHANGED,
-                      Source<PanelController>(panel_controller_.get()));
+  if (panel_controller_.get()) {
+    registrar_.Remove(
+        this, chrome::NOTIFICATION_PANEL_STATE_CHANGED,
+        content::Source<PanelController>(panel_controller_.get()));
+  }
 }
 
 void NotificationPanel::ScrollBalloonToVisible(Balloon* balloon) {
@@ -711,8 +732,7 @@ void NotificationPanel::UpdatePanel(bool update_container_size) {
   switch (state_) {
     case KEEP_SIZE: {
       gfx::Rect min_bounds = GetPreferredBounds();
-      gfx::Rect panel_bounds;
-      panel_widget_->GetBounds(&panel_bounds, true);
+      gfx::Rect panel_bounds = panel_widget_->GetWindowScreenBounds();
       if (min_bounds.height() < panel_bounds.height())
         panel_widget_->SetBounds(min_bounds);
       else if (min_bounds.height() > panel_bounds.height()) {
@@ -756,7 +776,7 @@ void NotificationPanel::UpdateContainerBounds() {
   // Update from WebKit may arrive after the panel is closed/hidden
   // and viewport widget is detached.
   if (native) {
-    native->SetBounds(balloon_container_->bounds());
+    native->SetBoundsRect(balloon_container_->bounds());
     scroll_view_->Layout();
   }
 }
@@ -792,12 +812,12 @@ void NotificationPanel::StartStaleTimer(Balloon* balloon) {
   BalloonViewImpl* view = GetBalloonViewOf(balloon);
   MessageLoop::current()->PostDelayedTask(
       FROM_HERE,
-      task_factory_.NewRunnableMethod(
-          &NotificationPanel::OnStale, view),
+      base::Bind(&NotificationPanel::OnStale, AsWeakPtr(), view),
       stale_timeout_);
 }
 
 void NotificationPanel::OnStale(BalloonViewImpl* view) {
+  // Note: |view| may point to deleted memory.
   if (balloon_container_->HasBalloonView(view) && !view->stale()) {
     view->set_stale();
     // don't update panel on stale
@@ -866,7 +886,7 @@ bool NotificationPanelTester::IsVisible(const BalloonViewImpl* view) const {
   gfx::Point origin(0, 0);
   views::View::ConvertPointToView(view, panel_->balloon_container_.get(),
                                   &origin);
-  return rect.Contains(gfx::Rect(origin, view->bounds().size()));
+  return rect.Contains(gfx::Rect(origin, view->size()));
 }
 
 
