@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,12 +8,12 @@
 #define CHROME_BROWSER_CHROMEOS_NOTIFICATIONS_NOTIFICATION_PANEL_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
-#include "base/task.h"
+#include "base/memory/weak_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/frame/panel_controller.h"
 #include "chrome/browser/chromeos/notifications/balloon_collection_impl.h"
-#include "chrome/common/notification_registrar.h"
-#include "gfx/rect.h"
+#include "content/public/browser/notification_registrar.h"
+#include "ui/gfx/rect.h"
 
 class Balloon;
 class Notification;
@@ -71,7 +71,8 @@ class NotificationPanelTester;
 //
 class NotificationPanel : public PanelController::Delegate,
                           public BalloonCollectionImpl::NotificationUI,
-                          public NotificationObserver {
+                          public content::NotificationObserver,
+                          public base::SupportsWeakPtr<NotificationPanel> {
  public:
   enum State {
     FULL,  // Show all notifications
@@ -89,24 +90,25 @@ class NotificationPanel : public PanelController::Delegate,
   void Hide();
 
   // BalloonCollectionImpl::NotificationUI overrides..
-  virtual void Add(Balloon* balloon);
-  virtual bool Update(Balloon* balloon);
-  virtual void Remove(Balloon* balloon);
-  virtual void Show(Balloon* balloon);
+  virtual void Add(Balloon* balloon) OVERRIDE;
+  virtual bool Update(Balloon* balloon) OVERRIDE;
+  virtual void Remove(Balloon* balloon) OVERRIDE;
+  virtual void Show(Balloon* balloon) OVERRIDE;
   virtual void ResizeNotification(Balloon* balloon,
-                                  const gfx::Size& size);
-  virtual void SetActiveView(BalloonViewImpl* view);
+                                  const gfx::Size& size) OVERRIDE;
+  virtual void SetActiveView(BalloonViewImpl* view) OVERRIDE;
 
   // PanelController::Delegate overrides.
-  virtual string16 GetPanelTitle();
-  virtual SkBitmap GetPanelIcon();
-  virtual bool CanClosePanel();
-  virtual void ClosePanel();
+  virtual string16 GetPanelTitle() OVERRIDE;
+  virtual SkBitmap GetPanelIcon() OVERRIDE;
+  virtual bool CanClosePanel() OVERRIDE;
+  virtual void ClosePanel() OVERRIDE;
+  virtual void ActivatePanel() OVERRIDE;
 
   // NotificationObserver overrides:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Called when a mouse left the panel window.
   void OnMouseLeave();
@@ -175,8 +177,6 @@ class NotificationPanel : public PanelController::Delegate,
   // Panel's state.
   State state_;
 
-  ScopedRunnableMethodFactory<NotificationPanel> task_factory_;
-
   // The minimum size of a notification.
   gfx::Rect min_bounds_;
 
@@ -184,7 +184,7 @@ class NotificationPanel : public PanelController::Delegate,
   int stale_timeout_;
 
   // A registrar to subscribe PANEL_STATE_CHANGED event.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // The notification a mouse pointer is currently on. NULL if the mouse
   // is out of the panel.
