@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,18 +8,25 @@
 
 #include <string>
 
+#include "base/compiler_specific.h"
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
 #include "base/string16.h"
-#include "chrome/browser/chromeos/notifications/balloon_collection_impl.h"
+#include "chrome/browser/chromeos/notifications/balloon_view_host.h"  // MessageCallback
 #include "chrome/browser/notifications/notification_delegate.h"
 #include "googleurl/src/gurl.h"
 
-class MessageCallback;
 class Profile;
 
 namespace chromeos {
 
+#if defined(USE_AURA)
+class BalloonCollectionImplAura;
+typedef class BalloonCollectionImplAura BalloonCollectionImplType;
+#else
+class BalloonCollectionImpl;
+typedef class BalloonCollectionImpl BalloonCollectionImplType;
+#endif
+
 // The system notification object handles the display of a system notification
 
 class SystemNotification {
@@ -50,7 +57,8 @@ class SystemNotification {
   // Same as Show() above with a footer link at the bottom and a callback
   // for when the link is clicked.
   void Show(const string16& message, const string16& link_text,
-            MessageCallback* callback, bool urgent, bool sticky);
+            const BalloonViewHost::MessageCallback& callback,
+            bool urgent, bool sticky);
 
   // Hide will dismiss the notification, if the notification is already
   // hidden it does nothing
@@ -65,12 +73,12 @@ class SystemNotification {
  private:
   class Delegate : public NotificationDelegate {
    public:
-    explicit Delegate(const std::string& id) : id_(id) {}
-    void Display() {}
-    void Error() {}
-    void Close(bool by_user) {}
-    void Click() {}
-    std::string id() const { return id_; }
+    explicit Delegate(const std::string& id);
+    virtual void Display() OVERRIDE {}
+    virtual void Error() OVERRIDE {}
+    virtual void Close(bool by_user) OVERRIDE {}
+    virtual void Click() OVERRIDE {}
+    virtual std::string id() const OVERRIDE;
 
    private:
     std::string id_;
@@ -81,7 +89,7 @@ class SystemNotification {
   void Init(int icon_resource_id);
 
   Profile* profile_;
-  BalloonCollectionImpl* collection_;
+  BalloonCollectionImplType* collection_;
   scoped_refptr<NotificationDelegate> delegate_;
   GURL icon_;
   string16 title_;
