@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,25 +10,29 @@
 #include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_thread.h"
+#include "chrome/browser/chromeos/cros/cros_library.h"
+#include "chrome/browser/chromeos/cros/network_library.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/tab_contents/tab_util.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/jstemplate_builder.h"
-#include "chrome/common/notification_service.h"
-#include "chrome/common/notification_type.h"
 #include "chrome/common/url_constants.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/browser_resources.h"
 #include "grit/generated_resources.h"
+#include "net/base/escape.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
+using content::BrowserThread;
+using content::WebContents;
+
 namespace {
 
 // Maximum time to show a blank page.
@@ -43,52 +47,33 @@ void SetString(DictionaryValue* strings, const char* name, int resource_id) {
 
 namespace chromeos {
 
-// static
-void OfflineLoadPage::Show(int process_host_id, int render_view_id,
-                           const GURL& url, Delegate* delegate) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  if (NetworkStateNotifier::is_connected()) {
-    // Check again in UI thread and proceed if it's connected.
-    delegate->OnBlockingPageComplete(true);
-  } else {
-    TabContents* tab_contents =
-        tab_util::GetTabContentsByID(process_host_id, render_view_id);
-    DCHECK(tab_contents);
-    (new OfflineLoadPage(tab_contents, url, delegate))->Show();
-  }
+OfflineLoadPage::OfflineLoadPage(WebContents* web_contents,
+                                 const GURL& url,
+                                 const CompletionCallback& callback)
+    : ChromeInterstitialPage(web_contents, true, url),
+      callback_(callback),
+      proceeded_(false) {
+  net::NetworkChangeNotifier::AddOnlineStateObserver(this);
 }
 
-OfflineLoadPage::OfflineLoadPage(TabContents* tab_contents,
-                                 const GURL& url,
-                                 Delegate* delegate)
-    : InterstitialPage(tab_contents, true, url),
-      delegate_(delegate),
-      proceeded_(false),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)),
-      in_test_(false) {
-  registrar_.Add(this, NotificationType::NETWORK_STATE_CHANGED,
-                 NotificationService::AllSources());
+OfflineLoadPage::~OfflineLoadPage() {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  net::NetworkChangeNotifier::RemoveOnlineStateObserver(this);
 }
 
 std::string OfflineLoadPage::GetHTMLContents() {
   DictionaryValue strings;
-  // Toggle Cancel button.
-  strings.SetString("display_cancel",
-                    tab()->controller().CanGoBack() ? "inline" : "none");
-
-  int64 time_to_wait = std::max(
-      static_cast<int64>(0),
-      kMaxBlankPeriod -
-      NetworkStateNotifier::GetOfflineDuration().InMilliseconds());
+  int64 time_to_wait = kMaxBlankPeriod;
   // Set the timeout to show the page.
-  strings.SetInteger("timeToWait", static_cast<int>(time_to_wait));
+  strings.SetInteger("time_to_wait", static_cast<int>(time_to_wait));
   // Button labels
-  SetString(&strings, "load_button", IDS_OFFLINE_LOAD_BUTTON);
-  SetString(&strings, "cancel_button", IDS_OFFLINE_CANCEL_BUTTON);
-
   SetString(&strings, "heading", IDS_OFFLINE_LOAD_HEADLINE);
+  SetString(&strings, "try_loading", IDS_OFFLINE_TRY_LOADING);
   SetString(&strings, "network_settings", IDS_OFFLINE_NETWORK_SETTINGS);
 
+  // Activation
+  strings.SetBoolean("show_activation", ShowActivationMessage());
+
   bool rtl = base::i18n::IsRTL();
   strings.SetString("textdirection", rtl ? "rtl" : "ltr");
 
@@ -98,13 +83,14 @@ std::string OfflineLoadPage::GetHTMLContents() {
   strings.SetString("url", failed_url);
 
   // The offline page for app has icons and slightly different message.
-  Profile* profile = tab()->profile();
+  Profile* profile = Profile::FromBrowserContext(tab()->GetBrowserContext());
   DCHECK(profile);
   const Extension* extension = NULL;
   ExtensionService* extensions_service = profile->GetExtensionService();
   // Extension service does not exist in test.
   if (extensions_service)
-    extension = extensions_service->GetExtensionByWebExtent(url());
+    extension = extensions_service->extensions()->GetHostedAppByURL(
+        ExtensionURLInfo(url()));
 
   if (extension)
     GetAppOfflineStrings(extension, failed_url, &strings);
@@ -138,7 +124,7 @@ void OfflineLoadPage::GetAppOfflineStrings(
   strings->SetString(
       "msg",
       l10n_util::GetStringFUTF16(IDS_APP_OFFLINE_LOAD_DESCRIPTION,
-                                 failed_url));
+                                 net::EscapeForHTML(failed_url)));
 }
 
 void OfflineLoadPage::GetNormalOfflineStrings(
@@ -152,7 +138,7 @@ void OfflineLoadPage::GetNormalOfflineStrings(
   strings->SetString(
       "msg",
       l10n_util::GetStringFUTF16(IDS_SITE_OFFLINE_LOAD_DESCRIPTION,
-                                 failed_url));
+                                 net::EscapeForHTML(failed_url)));
 }
 
 void OfflineLoadPage::CommandReceived(const std::string& cmd) {
@@ -170,43 +156,60 @@ void OfflineLoadPage::CommandReceived(const std::string& cmd) {
     Browser* browser = BrowserList::GetLastActive();
     DCHECK(browser);
     browser->ShowOptionsTab(chrome::kInternetOptionsSubPage);
+  } else if (command == "open_activate_broadband") {
+    Browser* browser = BrowserList::GetLastActive();
+    DCHECK(browser);
+    browser->OpenMobilePlanTabAndActivate();
   } else {
     LOG(WARNING) << "Unknown command:" << cmd;
   }
 }
 
+void OfflineLoadPage::NotifyBlockingPageComplete(bool proceed) {
+  BrowserThread::PostTask(
+      BrowserThread::IO, FROM_HERE, base::Bind(callback_, proceed));
+}
+
 void OfflineLoadPage::Proceed() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
   proceeded_ = true;
-  delegate_->OnBlockingPageComplete(true);
+  NotifyBlockingPageComplete(true);
   InterstitialPage::Proceed();
 }
 
 void OfflineLoadPage::DontProceed() {
-  // Inogre if it's already proceeded.
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  // Ignore if it's already proceeded.
   if (proceeded_)
     return;
-  delegate_->OnBlockingPageComplete(false);
+  NotifyBlockingPageComplete(false);
   InterstitialPage::DontProceed();
 }
 
-void OfflineLoadPage::Observe(NotificationType type,
-                              const NotificationSource& source,
-                              const NotificationDetails& details) {
-  if (type.value == NotificationType::NETWORK_STATE_CHANGED) {
-    chromeos::NetworkStateDetails* state_details =
-        Details<chromeos::NetworkStateDetails>(details).ptr();
-    DVLOG(1) << "NetworkStateChanaged notification received: state="
-             << state_details->state();
-    if (state_details->state() ==
-        chromeos::NetworkStateDetails::CONNECTED) {
-      registrar_.Remove(this, NotificationType::NETWORK_STATE_CHANGED,
-                        NotificationService::AllSources());
-      Proceed();
-    }
-  } else {
-    InterstitialPage::Observe(type, source, details);
+void OfflineLoadPage::OnOnlineStateChanged(bool online) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DVLOG(1) << "OnlineStateObserver notification received: state="
+           << (online ? "online" : "offline");
+  if (online) {
+    net::NetworkChangeNotifier::RemoveOnlineStateObserver(this);
+    Proceed();
   }
 }
 
+bool OfflineLoadPage::ShowActivationMessage() {
+  CrosLibrary* cros = CrosLibrary::Get();
+  if (!cros || !cros->GetNetworkLibrary()->cellular_available())
+    return false;
+
+  const CellularNetworkVector& cell_networks =
+      cros->GetNetworkLibrary()->cellular_networks();
+  for (size_t i = 0; i < cell_networks.size(); ++i) {
+    chromeos::ActivationState activation_state =
+        cell_networks[i]->activation_state();
+    if (activation_state == ACTIVATION_STATE_ACTIVATED)
+      return false;
+  }
+  return true;
+}
+
 }  // namespace chromeos
