@@ -1,17 +1,25 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/command_line.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_window.h"
-#include "chrome/browser/chromeos/wm_ipc.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
-#include "third_party/cros/chromeos_wm_ipc_enums.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
+#include "third_party/cros_system_api/window_manager/chromeos_wm_ipc_enums.h"
+
+#if defined(TOOLKIT_USES_GTK)
+#include "chrome/browser/chromeos/legacy_window_manager/wm_ipc.h"
+#endif
+
+using content::OpenURLParams;
+using content::Referrer;
 
 namespace chromeos {
 
@@ -42,10 +50,12 @@ IN_PROC_BROWSER_TEST_F(PanelTest, PanelOpenSmall) {
   CommandLine::ForCurrentProcess()->AppendSwitch(
       switches::kDisablePopupBlocking);
 
-  browser()->OpenURL(url, GURL(), CURRENT_TAB, PageTransition::TYPED);
-
-  // Wait for notification that window.open has been processed.
-  ui_test_utils::WaitForNotification(NotificationType::TAB_ADDED);
+  ui_test_utils::WindowedNotificationObserver tab_added_observer(
+      content::NOTIFICATION_TAB_ADDED,
+      content::NotificationService::AllSources());
+  browser()->OpenURL(OpenURLParams(
+      url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_TYPED, false));
+  tab_added_observer.Wait();
 
   // Find the new browser.
   Browser* new_browser = NULL;
@@ -56,17 +66,28 @@ IN_PROC_BROWSER_TEST_F(PanelTest, PanelOpenSmall) {
   }
 
   ASSERT_TRUE(new_browser);
-  EXPECT_EQ(Browser::TYPE_POPUP, new_browser->type());
+  EXPECT_TRUE(new_browser->is_type_popup());
+  EXPECT_FALSE(new_browser->is_app());
+
+#if defined(TOOLKIT_USES_GTK)
   // This window type tells the cros window manager to treat the window
   // as a panel.
   EXPECT_EQ(
       WM_IPC_WINDOW_CHROME_PANEL_CONTENT,
       WmIpc::instance()->GetWindowType(
           GTK_WIDGET(new_browser->window()->GetNativeHandle()), NULL));
+#endif
 }
 
+#if defined(USE_AURA)
+// crbug.com/105129.
+#define MAYBE_PanelOpenLarge DISABLED_PanelOpenLarge
+#else
+#define MAYBE_PanelOpenLarge PanelOpenLarge
+#endif
+
 // Large popups should open as new tab.
-IN_PROC_BROWSER_TEST_F(PanelTest, PanelOpenLarge) {
+IN_PROC_BROWSER_TEST_F(PanelTest, MAYBE_PanelOpenLarge) {
   const std::string HTML =
       "<html><head><title>PanelOpen</title></head>"
       "<body onload='window.setTimeout(run_tests, 0)'>"
@@ -80,10 +101,12 @@ IN_PROC_BROWSER_TEST_F(PanelTest, PanelOpenLarge) {
   CommandLine::ForCurrentProcess()->AppendSwitch(
       switches::kDisablePopupBlocking);
   int old_tab_count = browser()->tab_count();
-  browser()->OpenURL(url, GURL(), CURRENT_TAB, PageTransition::TYPED);
-
-  // Wait for notification that window.open has been processed.
-  ui_test_utils::WaitForNotification(NotificationType::TAB_ADDED);
+  ui_test_utils::WindowedNotificationObserver tab_added_observer(
+      content::NOTIFICATION_TAB_ADDED,
+      content::NotificationService::AllSources());
+  browser()->OpenURL(OpenURLParams(
+      url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_TYPED, false));
+  tab_added_observer.Wait();
 
   // Shouldn't find a new browser.
   Browser* new_browser = NULL;
