@@ -6,13 +6,13 @@
 
 #include "base/compiler_specific.h"
 #include "base/logging.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/chromeos/panels/panel_scroller_container.h"
 #include "chrome/browser/chromeos/panels/panel_scroller_header.h"
-#include "gfx/canvas.h"
-#include "views/widget/widget_gtk.h"
+#include "ui/gfx/canvas.h"
+#include "ui/views/widget/widget.h"
 
 struct PanelScroller::Panel {
   PanelScrollerHeader* header;
@@ -80,10 +80,11 @@ PanelScroller::~PanelScroller() {
 
 // static
 PanelScroller* PanelScroller::CreateWindow() {
-  views::WidgetGtk* widget =
-      new views::WidgetGtk(views::WidgetGtk::TYPE_WINDOW);
-  widget->set_delete_on_destroy(true);
-  widget->Init(NULL, gfx::Rect(0, 0, 100, 800));
+  views::Widget* widget = new views::Widget;
+  views::Widget::InitParams params(
+      views::Widget::InitParams::TYPE_WINDOW_FRAMELESS);
+  params.bounds = gfx::Rect(0, 0, 100, 800);
+  widget->Init(params);
 
   PanelScroller* scroller = new PanelScroller();
   widget->SetContentsView(scroller);
@@ -124,8 +125,8 @@ void PanelScroller::Layout() {
     if (cur_content_pos < scroll_pos_ + top_header_pixel_count) {
       // This panel is at least partially off the top. Put the header below the
       // others already there.
-      panels_[i]->header->SetBounds(gfx::Rect(0, top_header_pixel_count,
-                                              width(), divider_height_));
+      panels_[i]->header->SetBoundsRect(gfx::Rect(0, top_header_pixel_count,
+                                                  width(), divider_height_));
       top_header_pixel_count += divider_height_;
 
     } else if (cur_content_pos > height() + scroll_pos_ -
@@ -134,12 +135,13 @@ void PanelScroller::Layout() {
       // headers will stack up at the bottom. Counting this header, there are
       // (size() - i) left, which is used in the expression above.
       int top = height() - (panel_count - i) * divider_height_;
-      panels_[i]->header->SetBounds(gfx::Rect(0, top,
-                                              width(), divider_height_));
+      panels_[i]->header->SetBoundsRect(gfx::Rect(0, top,
+                                                  width(), divider_height_));
     } else {
       // Normal header positioning in-flow.
-      panels_[i]->header->SetBounds(gfx::Rect(0, cur_content_pos - scroll_pos_,
-                                              width(), divider_height_));
+      panels_[i]->header->SetBoundsRect(
+          gfx::Rect(0, cur_content_pos - scroll_pos_, width(),
+                    divider_height_));
     }
 
     cur_content_pos += divider_height_;
@@ -147,7 +149,7 @@ void PanelScroller::Layout() {
     // Now position the content. It always goes in-flow ignoring any stacked
     // up headers at the top or bottom.
     int container_height = panels_[i]->container->GetPreferredSize().height();
-    panels_[i]->container->SetBounds(
+    panels_[i]->container->SetBoundsRect(
         gfx::Rect(0, cur_content_pos - scroll_pos_,
                   width(), container_height));
     cur_content_pos += container_height;
@@ -162,10 +164,6 @@ bool PanelScroller::OnMouseDragged(const views::MouseEvent& event) {
   return true;
 }
 
-void PanelScroller::OnMouseReleased(const views::MouseEvent& event,
-                                    bool canceled) {
-}
-
 void PanelScroller::HeaderClicked(PanelScrollerHeader* source) {
   for (size_t i = 0; i < panels_.size(); i++) {
     if (panels_[i]->header == source) {
