@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,28 +9,28 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "ui/base/animation/animation_delegate.h"
 #include "ui/base/animation/slide_animation.h"
-#include "views/view.h"
+#include "ui/views/view.h"
 
 class PanelScrollerHeader;
 
 class PanelScroller : public views::View, public ui::AnimationDelegate {
  public:
   PanelScroller();
-  ~PanelScroller();
+  virtual ~PanelScroller();
 
   static PanelScroller* CreateWindow();
 
-  // View overrides.
+  // Overridden from View:
   virtual void ViewHierarchyChanged(bool is_add,
                                     views::View* parent,
-                                    views::View* child);
-  virtual gfx::Size GetPreferredSize();
-  virtual void Layout();
-  virtual bool OnMousePressed(const views::MouseEvent& event);
-  virtual bool OnMouseDragged(const views::MouseEvent& event);
-  virtual void OnMouseReleased(const views::MouseEvent& event, bool canceled);
+                                    views::View* child) OVERRIDE;
+  virtual gfx::Size GetPreferredSize() OVERRIDE;
+  virtual void Layout() OVERRIDE;
+  virtual bool OnMousePressed(const views::MouseEvent& event) OVERRIDE;
+  virtual bool OnMouseDragged(const views::MouseEvent& event) OVERRIDE;
 
   // Called when a panel header is clicked with the affected container. This
   // function will make sure the panel is fully visible.
@@ -40,7 +40,7 @@ class PanelScroller : public views::View, public ui::AnimationDelegate {
   struct Panel;
 
   // ui::AnimationDelegate overrides.
-  virtual void AnimationProgressed(const ui::Animation* animation);
+  virtual void AnimationProgressed(const ui::Animation* animation) OVERRIDE;
 
   // Scrolls to the panel at the given index. It will be moved to the top.
   void ScrollToPanel(int index);
@@ -67,4 +67,3 @@ class PanelScroller : public views::View, public ui::AnimationDelegate {
 };
 
 #endif  // CHROME_BROWSER_CHROMEOS_PANELS_PANEL_SCROLLER_H_
-
