@@ -1,12 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/chromeos/status/input_method_menu_button.h"
 
-#include "base/string_util.h"
+#include "base/utf_string_conversions.h"
 #include "chrome/browser/chromeos/cros/cros_in_process_browser_test.h"
-#include "chrome/browser/chromeos/cros/mock_input_method_library.h"
 #include "chrome/browser/chromeos/frame/browser_view.h"
 #include "chrome/browser/chromeos/status/status_area_view.h"
 #include "chrome/browser/chromeos/view_ids.h"
@@ -14,6 +13,10 @@
 #include "chrome/browser/ui/browser_window.h"
 #include "grit/theme_resources.h"
 
+#if defined(USE_AURA)
+#include "chrome/browser/ui/views/aura/chrome_shell_delegate.h"
+#endif
+
 namespace chromeos {
 
 class InputMethodMenuButtonTest : public CrosInProcessBrowserTest {
@@ -27,21 +30,25 @@ class InputMethodMenuButtonTest : public CrosInProcessBrowserTest {
     cros_mock_->SetStatusAreaMocksExpectations();
   }
 
-  InputMethodMenuButton* GetInputMethodMenuButton() {
-    BrowserView* view = static_cast<BrowserView*>(browser()->window());
-    return static_cast<StatusAreaView*>(view->
-        GetViewByID(VIEW_ID_STATUS_AREA))->input_method_view();
+  const InputMethodMenuButton* GetInputMethodMenuButton() {
+    const views::View* view =
+#if defined(USE_AURA)
+        ChromeShellDelegate::instance()->GetStatusArea();
+#else
+        static_cast<BrowserView*>(browser()->window());
+#endif
+    return static_cast<const InputMethodMenuButton*>(
+        view->GetViewByID(VIEW_ID_STATUS_BUTTON_INPUT_METHOD));
   }
 };
 
 IN_PROC_BROWSER_TEST_F(InputMethodMenuButtonTest, InitialIndicatorTest) {
-  InputMethodMenuButton* input_method = GetInputMethodMenuButton();
+  const InputMethodMenuButton* input_method = GetInputMethodMenuButton();
   ASSERT_TRUE(input_method != NULL);
 
-  // Since the default input method is "xkb:us::eng", "US" should be set for the
-  // indicator.
-  std::wstring indicator = input_method->text();
-  EXPECT_EQ(L"US", indicator);
+  // By default, show the indicator of the hardware keyboard, which is set
+  // to US for tests.
+  EXPECT_EQ(ASCIIToUTF16("US"), input_method->text());
 }
 
 }  // namespace chromeos
