@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,13 +7,14 @@
 #include "base/command_line.h"
 #include "chrome/browser/browser_shutdown.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/tab_contents/tab_contents_view.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/notification_service.h"
 #include "chrome/common/url_constants.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/browser/web_contents_view.h"
 
 namespace chromeos {
 
@@ -67,10 +68,11 @@ TabCloseableStateWatcher::TabCloseableStateWatcher()
       signing_off_(false),
       guest_session_(
           CommandLine::ForCurrentProcess()->HasSwitch(
-              switches::kGuestSession)) {
+              switches::kGuestSession)),
+      waiting_for_browser_(false) {
   BrowserList::AddObserver(this);
-  notification_registrar_.Add(this, NotificationType::APP_EXITING,
-      NotificationService::AllSources());
+  notification_registrar_.Add(this, content::NOTIFICATION_APP_EXITING,
+      content::NotificationService::AllSources());
 }
 
 TabCloseableStateWatcher::~TabCloseableStateWatcher() {
@@ -80,7 +82,50 @@ TabCloseableStateWatcher::~TabCloseableStateWatcher() {
 }
 
 bool TabCloseableStateWatcher::CanCloseTab(const Browser* browser) const {
-  return browser->type() != Browser::TYPE_NORMAL ? true : can_close_tab_;
+  return browser->is_type_tabbed() ?
+      (can_close_tab_ || waiting_for_browser_) : true;
+}
+
+bool TabCloseableStateWatcher::CanCloseTabs(const Browser* browser,
+    std::vector<int>* indices) const {
+  if (signing_off_ || waiting_for_browser_ || tabstrip_watchers_.size() > 1 ||
+      !browser->is_type_tabbed() ||
+      (browser->profile()->IsOffTheRecord() && !guest_session_))
+    return true;
+
+  if (!can_close_tab_) {
+    indices->clear();
+    return false;
+  }
+
+  TabStripModel* tabstrip_model = browser->tabstrip_model();
+  // If we're not closing all tabs, there's no restriction.
+  if (static_cast<int>(indices->size()) != tabstrip_model->count())
+    return true;
+
+  // If first tab is NTP, it can't be closed.
+  // In TabStripModel::InternalCloseTabs (which calls
+  // Browser::CanCloseContents which in turn calls this method), all
+  // renderer processes of tabs could be terminated before the tabs are actually
+  // closed.
+  // As tabs are being closed, notification TabDetachedAt is called.
+  // When this happens to the last second tab, we would prevent the last NTP
+  // tab from being closed.
+  // If we don't prevent this NTP tab from being closed now, its renderer
+  // process would have been terminated but the tab won't be detached later,
+  // resulting in the "Aw, Snap" page replacing the first NTP.
+  // This is the main purpose of this method CanCloseTabs.
+  for (size_t i = 0; i < indices->size(); ++i) {
+    if ((*indices)[i] == 0) {
+      if (tabstrip_model->GetTabContentsAt(0)->web_contents()->GetURL() ==
+          GURL(chrome::kChromeUINewTabURL)) {  // First tab is NewTabPage.
+        indices->erase(indices->begin() + i);  // Don't close it.
+        return false;
+      }
+      break;
+    }
+  }
+  return true;
 }
 
 bool TabCloseableStateWatcher::CanCloseBrowser(Browser* browser) {
@@ -109,8 +154,10 @@ void TabCloseableStateWatcher::OnWindowCloseCanceled(Browser* browser) {
 // TabCloseableStateWatcher, BrowserList::Observer implementation:
 
 void TabCloseableStateWatcher::OnBrowserAdded(const Browser* browser) {
-  // Only normal browsers may affect closeable state.
-  if (browser->type() != Browser::TYPE_NORMAL)
+  waiting_for_browser_ = false;
+
+  // Only tabbed browsers may affect closeable state.
+  if (!browser->is_type_tabbed())
     return;
 
   // Create TabStripWatcher to observe tabstrip of new browser.
@@ -121,8 +168,8 @@ void TabCloseableStateWatcher::OnBrowserAdded(const Browser* browser) {
 }
 
 void TabCloseableStateWatcher::OnBrowserRemoved(const Browser* browser) {
-  // Only normal browsers may affect closeable state.
-  if (browser->type() != Browser::TYPE_NORMAL)
+  // Only tabbed browsers may affect closeable state.
+  if (!browser->is_type_tabbed())
     return;
 
   // Remove TabStripWatcher for browser that is being removed.
@@ -139,11 +186,12 @@ void TabCloseableStateWatcher::OnBrowserRemoved(const Browser* browser) {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// TabCloseableStateWatcher, NotificationObserver implementation:
+// TabCloseableStateWatcher, content::NotificationObserver implementation:
 
-void TabCloseableStateWatcher::Observe(NotificationType type,
-    const NotificationSource& source, const NotificationDetails& details) {
-  if (type.value != NotificationType::APP_EXITING)
+void TabCloseableStateWatcher::Observe(int type,
+    const content::NotificationSource& source,
+    const content::NotificationDetails& details) {
+  if (type != content::NOTIFICATION_APP_EXITING)
     NOTREACHED();
   if (!signing_off_) {
     signing_off_ = true;
@@ -156,6 +204,9 @@ void TabCloseableStateWatcher::Observe(NotificationType type,
 
 void TabCloseableStateWatcher::OnTabStripChanged(const Browser* browser,
     bool closing_last_tab) {
+  if (waiting_for_browser_)
+    return;
+
   if (!closing_last_tab) {
     CheckAndUpdateState(browser);
     return;
@@ -175,10 +226,8 @@ void TabCloseableStateWatcher::OnTabStripChanged(const Browser* browser,
 
 void TabCloseableStateWatcher::CheckAndUpdateState(
     const Browser* browser_to_check) {
-  // We shouldn't update state if we're signing off, or there's no normal
-  // browser, or browser is always closeable.
-  if (signing_off_ || tabstrip_watchers_.empty() ||
-      (browser_to_check && browser_to_check->type() != Browser::TYPE_NORMAL))
+  if (waiting_for_browser_ || signing_off_ || tabstrip_watchers_.empty() ||
+      (browser_to_check && !browser_to_check->is_type_tabbed()))
     return;
 
   bool new_can_close;
@@ -194,7 +243,7 @@ void TabCloseableStateWatcher::CheckAndUpdateState(
       TabStripModel* tabstrip_model = browser_to_check->tabstrip_model();
       if (tabstrip_model->count() == 1) {
         new_can_close =
-            tabstrip_model->GetTabContentsAt(0)->tab_contents()->GetURL() !=
+            tabstrip_model->GetTabContentsAt(0)->web_contents()->GetURL() !=
                 GURL(chrome::kChromeUINewTabURL);  // Tab is not NewTabPage.
       } else {
         new_can_close = true;
@@ -212,32 +261,40 @@ void TabCloseableStateWatcher::SetCloseableState(bool closeable) {
   can_close_tab_ = closeable;
 
   // Notify of change in tab closeable state.
-  NotificationService::current()->Notify(
-      NotificationType::TAB_CLOSEABLE_STATE_CHANGED,
-      NotificationService::AllSources(),
-      Details<bool>(&can_close_tab_));
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_TAB_CLOSEABLE_STATE_CHANGED,
+      content::NotificationService::AllSources(),
+      content::Details<bool>(&can_close_tab_));
 }
 
-bool TabCloseableStateWatcher::CanCloseBrowserImpl(const Browser* browser,
-    BrowserActionType* action_type) const {
+bool TabCloseableStateWatcher::CanCloseBrowserImpl(
+    const Browser* browser,
+    BrowserActionType* action_type) {
   *action_type = NONE;
 
+  // If we're waiting for a new browser allow the close.
+  if (waiting_for_browser_)
+    return true;
+
   // Browser is always closeable when signing off.
   if (signing_off_)
     return true;
 
-  // Non-normal browsers are always closeable.
-  if (browser->type() != Browser::TYPE_NORMAL)
+  // Non-tabbed browsers are always closeable.
+  if (!browser->is_type_tabbed())
     return true;
 
   // If this is not the last normal browser, it's always closeable.
   if (tabstrip_watchers_.size() > 1)
     return true;
 
-  // If last normal browser is incognito, open a non-incognito window,
-  // and allow closing of incognito one (if not guest).
+  // If last normal browser is incognito, open a non-incognito window, and allow
+  // closing of incognito one (if not guest). When this happens we need to wait
+  // for the new browser before doing any other actions as the new browser may
+  // be created by way of session restore, which is async.
   if (browser->profile()->IsOffTheRecord() && !guest_session_) {
     *action_type = OPEN_WINDOW;
+    waiting_for_browser_ = true;
     return true;
   }
 
