@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,10 +8,11 @@
 
 #include <vector>
 
-#include "chrome/browser/browser_list.h"
+#include "base/compiler_specific.h"
 #include "chrome/browser/tab_closeable_state_watcher.h"
 #include "chrome/browser/tabs/tab_strip_model_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "content/public/browser/notification_registrar.h"
 
 namespace chromeos {
 
@@ -41,15 +42,17 @@ namespace chromeos {
 
 class TabCloseableStateWatcher : public ::TabCloseableStateWatcher,
                                  public BrowserList::Observer,
-                                 public NotificationObserver {
+                                 public content::NotificationObserver {
  public:
   TabCloseableStateWatcher();
   virtual ~TabCloseableStateWatcher();
 
   // TabCloseableStateWatcher implementation:
-  virtual bool CanCloseTab(const Browser* browser) const;
-  virtual bool CanCloseBrowser(Browser* browser);
-  virtual void OnWindowCloseCanceled(Browser* browser);
+  virtual bool CanCloseTab(const Browser* browser) const OVERRIDE;
+  virtual bool CanCloseTabs(const Browser* browser,
+                            std::vector<int>* indices) const OVERRIDE;
+  virtual bool CanCloseBrowser(Browser* browser) OVERRIDE;
+  virtual void OnWindowCloseCanceled(Browser* browser) OVERRIDE;
 
  private:
   enum BrowserActionType {
@@ -59,17 +62,17 @@ class TabCloseableStateWatcher : public ::TabCloseableStateWatcher,
   };
 
   // BrowserList::Observer implementation:
-  virtual void OnBrowserAdded(const Browser* browser);
-  virtual void OnBrowserRemoved(const Browser* browser);
+  virtual void OnBrowserAdded(const Browser* browser) OVERRIDE;
+  virtual void OnBrowserRemoved(const Browser* browser) OVERRIDE;
 
   // NotificationObserver implementation:
-  virtual void Observe(NotificationType type, const NotificationSource& source,
-                       const NotificationDetails& details);
+  virtual void Observe(int type, const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Called by private class TabStripWatcher for TabStripModelObserver
   // notifications.
   // |closing_last_tab| is true if the tab strip is closing the last tab.
-  virtual void OnTabStripChanged(const Browser* browser, bool closing_last_tab);
+  void OnTabStripChanged(const Browser* browser, bool closing_last_tab);
 
   // Utility functions.
 
@@ -86,7 +89,7 @@ class TabCloseableStateWatcher : public ::TabCloseableStateWatcher,
   // Returns true if closing of |browser| is permitted.
   // |action_type| is the action to take regardless if browser is closeable.
   bool CanCloseBrowserImpl(const Browser* browser,
-                           BrowserActionType* action_type) const;
+                           BrowserActionType* action_type);
 
   // Data members.
 
@@ -101,7 +104,12 @@ class TabCloseableStateWatcher : public ::TabCloseableStateWatcher,
   // Is in guest session?
   bool guest_session_;
 
-  NotificationRegistrar notification_registrar_;
+  // Set to true if we're waiting for a new browser to be created. When true we
+  // uncoditionally allow everything as we know a browser is in the process of
+  // being created.
+  bool waiting_for_browser_;
+
+  content::NotificationRegistrar notification_registrar_;
 
   // TabStripWatcher is a TabStripModelObserver that funnels all interesting
   // methods to TabCloseableStateWatcher::OnTabStripChanged. TabStripWatcher is
@@ -115,13 +123,14 @@ class TabCloseableStateWatcher : public ::TabCloseableStateWatcher,
 
     // TabStripModelObserver implementation:
     virtual void TabInsertedAt(TabContentsWrapper* contents, int index,
-                               bool foreground);
+                               bool foreground) OVERRIDE;
     virtual void TabClosingAt(TabStripModel* tab_strip_model,
                               TabContentsWrapper* contents,
-                              int index);
-    virtual void TabDetachedAt(TabContentsWrapper* contents, int index);
+                              int index) OVERRIDE;
+    virtual void TabDetachedAt(TabContentsWrapper* contents,
+                               int index) OVERRIDE;
     virtual void TabChangedAt(TabContentsWrapper* contents, int index,
-                              TabChangeType change_type);
+                              TabChangeType change_type) OVERRIDE;
 
     const Browser* browser() const {
       return browser_;
