@@ -1,25 +1,29 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/chromeos/tab_closeable_state_watcher.h"
 
 #include "base/file_path.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_window.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/app_modal_dialogs/app_modal_dialog.h"
 #include "chrome/browser/ui/app_modal_dialogs/native_app_modal_dialog.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/web_contents.h"
 #include "googleurl/src/gurl.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using content::WebContents;
+
 namespace chromeos {
 
 class TabCloseableStateWatcherTest : public InProcessBrowserTest {
@@ -37,10 +41,11 @@ class TabCloseableStateWatcherTest : public InProcessBrowserTest {
  protected:
   // Wrapper for Browser::AddTabWithURL
   void AddTabWithURL(Browser* browser, const GURL& url) {
-    AddTabAtIndexToBrowser(browser, 0, url, PageTransition::TYPED);
-    // Wait for page to finish loading.
-    ui_test_utils::WaitForNavigation(
-        &browser->GetSelectedTabContents()->controller());
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        content::NotificationService::AllSources());
+    AddTabAtIndexToBrowser(browser, 0, url, content::PAGE_TRANSITION_TYPED);
+    observer.Wait();
   }
 
   // Wrapper for TabCloseableStateWatcher::CanCloseTab
@@ -185,7 +190,7 @@ IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest, CloseCloseableTab) {
   EXPECT_TRUE(CanCloseTab(browser()));
   browser()->CloseTab();
   EXPECT_EQ(1, browser()->tab_count());
-  EXPECT_EQ(ntp_url_, browser()->GetSelectedTabContents()->GetURL());
+  EXPECT_EQ(ntp_url_, browser()->GetSelectedWebContents()->GetURL());
 }
 
 // Tests closing a closeable browser - all tabs in browser should be closed,
@@ -197,7 +202,7 @@ IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest, CloseCloseableBrowser) {
   EXPECT_EQ(1u, BrowserList::size());
   EXPECT_EQ(browser(), *(BrowserList::begin()));
   EXPECT_EQ(1, browser()->tab_count());
-  EXPECT_EQ(ntp_url_, browser()->GetSelectedTabContents()->GetURL());
+  EXPECT_EQ(ntp_url_, browser()->GetSelectedWebContents()->GetURL());
 }
 
 // Tests closing a non-closeable tab and hence non-closeable browser - tab and
@@ -208,17 +213,17 @@ IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest,
   EXPECT_EQ(1, browser()->tab_count());
   NavigateToURL(ntp_url_);
   EXPECT_FALSE(CanCloseTab(browser()));
-  TabContents* tab_contents = browser()->GetSelectedTabContents();
+  WebContents* web_contents = browser()->GetSelectedWebContents();
   browser()->CloseTab();
   EXPECT_EQ(1, browser()->tab_count());
-  EXPECT_EQ(tab_contents, browser()->GetSelectedTabContents());
+  EXPECT_EQ(web_contents, browser()->GetSelectedWebContents());
 
   // Close browser with non-closeable tab.
   browser()->CloseWindow();
   EXPECT_EQ(1u, BrowserList::size());
   EXPECT_EQ(browser(), *(BrowserList::begin()));
   EXPECT_EQ(1, browser()->tab_count());
-  EXPECT_EQ(tab_contents, browser()->GetSelectedTabContents());
+  EXPECT_EQ(web_contents, browser()->GetSelectedWebContents());
 }
 
 // Tests an incognito browsr with a normal browser.
@@ -248,6 +253,8 @@ IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest, SecondIncognitoBrowser) {
 // Tests closing an incognito browser - the incognito browser should close,
 // and a new normal browser opened with a NewTabPage (which is not closeable).
 IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest, CloseIncognitoBrowser) {
+  NavigateToURL(ntp_url_);
+
   // Open an incognito browser.
   Browser* incognito_browser = CreateIncognitoBrowser();
   EXPECT_TRUE(incognito_browser->profile()->IsOffTheRecord());
@@ -262,25 +269,25 @@ IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest, CloseIncognitoBrowser) {
 
   // Close incognito browser.
   incognito_browser->CloseWindow();
-  ui_test_utils::RunAllPendingInMessageLoop();
+  Browser* new_browser = ui_test_utils::WaitForNewBrowser();
   EXPECT_EQ(1u, BrowserList::size());
-  Browser* new_browser = *(BrowserList::begin());
   EXPECT_FALSE(new_browser->profile()->IsOffTheRecord());
   EXPECT_EQ(1, new_browser->tab_count());
-  EXPECT_EQ(ntp_url_, new_browser->GetSelectedTabContents()->GetURL());
+  EXPECT_EQ(ntp_url_, new_browser->GetSelectedWebContents()->GetURL());
 }
 
 // Tests closing of browser with BeforeUnload handler where user clicks cancel
 // (i.e. stay on the page and cancel closing) - browser and its tabs should stay
 // the same.
+// Sporadically crashing test. See http://crbug.com/79333
 IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest,
-                       CloseBrowserWithBeforeUnloadHandlerCancel) {
+                       DISABLED_CloseBrowserWithBeforeUnloadHandlerCancel) {
   // Navigate to URL with BeforeUnload handler.
   NavigateToBeforeUnloadURL();
   EXPECT_TRUE(CanCloseTab(browser()));
 
   // Close browser, click Cancel in BeforeUnload confirm dialog.
-  TabContents* tab_contents = browser()->GetSelectedTabContents();
+  WebContents* web_contents = browser()->GetSelectedWebContents();
   browser()->CloseWindow();
   AppModalDialog* confirm = ui_test_utils::WaitForAppModalDialog();
   confirm->native_dialog()->CancelAppModalDialog();
@@ -288,7 +295,7 @@ IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest,
   EXPECT_EQ(1u, BrowserList::size());
   EXPECT_EQ(browser(), *(BrowserList::begin()));
   EXPECT_EQ(1, browser()->tab_count());
-  EXPECT_EQ(tab_contents, browser()->GetSelectedTabContents());
+  EXPECT_EQ(web_contents, browser()->GetSelectedWebContents());
 
   // Close the browser.
   browser()->CloseWindow();
@@ -314,7 +321,7 @@ IN_PROC_BROWSER_TEST_F(TabCloseableStateWatcherTest,
   EXPECT_EQ(1u, BrowserList::size());
   EXPECT_EQ(browser(), *(BrowserList::begin()));
   EXPECT_EQ(1, browser()->tab_count());
-  EXPECT_EQ(ntp_url_, browser()->GetSelectedTabContents()->GetURL());
+  EXPECT_EQ(ntp_url_, browser()->GetSelectedWebContents()->GetURL());
 }
 
 }  // namespace chromeos
