@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,8 +12,6 @@
 #include "base/gtest_prod_util.h"
 #include "chrome/browser/cancelable_request.h"
 
-class FilePath;
-
 namespace chromeos {
 
 // ChromeOSVersionLoader loads the version of Chrome OS from the file system.
@@ -28,9 +26,13 @@ namespace chromeos {
 //   void OnGetChromeOSVersion(chromeos::VersionLoader::Handle,
 //                             std::string version);
 // . When you want the version invoke:  loader.GetVersion(&consumer, callback);
+//
+// This class also provides the ability to load the bios firmware using
+//   loader.GetFirmware(&consumer, callback);
 class VersionLoader : public CancelableRequestProvider {
  public:
   VersionLoader();
+  virtual ~VersionLoader();
 
   enum VersionFormat {
     VERSION_SHORT,
@@ -39,23 +41,36 @@ class VersionLoader : public CancelableRequestProvider {
   };
 
   // Signature
-  typedef Callback2<Handle, std::string>::Type GetVersionCallback;
-
+  typedef base::Callback<void(Handle, std::string)> GetVersionCallback;
   typedef CancelableRequest<GetVersionCallback> GetVersionRequest;
 
+  typedef base::Callback<void(Handle, std::string)> GetFirmwareCallback;
+  typedef CancelableRequest<GetFirmwareCallback> GetFirmwareRequest;
+
   // Asynchronously requests the version.
   // If |full_version| is true version string with extra info is extracted,
   // otherwise it's in short format x.x.xx.x.
   Handle GetVersion(CancelableRequestConsumerBase* consumer,
-                    GetVersionCallback* callback,
+                    const GetVersionCallback& callback,
                     VersionFormat format);
 
+  Handle GetFirmware(CancelableRequestConsumerBase* consumer,
+                     const GetFirmwareCallback& callback);
+
+  // Parse the version information as a Chrome platfrom, not Chrome OS
+  // TODO(rkc): Change this and everywhere it is used once we switch Chrome OS
+  // over to xx.yyy.zz version numbers instead of 0.xx.yyy.zz
+  // Refer to http://code.google.com/p/chromium-os/issues/detail?id=15789
+  void EnablePlatformVersions(bool enable);
+
   static const char kFullVersionPrefix[];
   static const char kVersionPrefix[];
+  static const char kFirmwarePrefix[];
 
  private:
   FRIEND_TEST_ALL_PREFIXES(VersionLoaderTest, ParseFullVersion);
   FRIEND_TEST_ALL_PREFIXES(VersionLoaderTest, ParseVersion);
+  FRIEND_TEST_ALL_PREFIXES(VersionLoaderTest, ParseFirmware);
 
   // VersionLoader calls into the Backend on the file thread to load
   // and extract the version.
@@ -69,6 +84,10 @@ class VersionLoader : public CancelableRequestProvider {
     void GetVersion(scoped_refptr<GetVersionRequest> request,
                     VersionFormat format);
 
+    // Calls ParseFirmware to get the firmware # and notifies request.
+    // This is invoked on the file thread.
+    void GetFirmware(scoped_refptr<GetFirmwareRequest> request);
+
    private:
     friend class base::RefCountedThreadSafe<Backend>;
 
@@ -82,6 +101,9 @@ class VersionLoader : public CancelableRequestProvider {
   static std::string ParseVersion(const std::string& contents,
                                   const std::string& prefix);
 
+  // Extracts the firmware from the file.
+  static std::string ParseFirmware(const std::string& contents);
+
   scoped_refptr<Backend> backend_;
 
   DISALLOW_COPY_AND_ASSIGN(VersionLoader);
