@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,33 +8,41 @@
 #include <stdlib.h>
 #include <sys/socket.h>
 #include <sys/syscall.h>
-#include <sys/types.h>
 #include <unistd.h>
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/eintr_wrapper.h"
 #include "base/file_path.h"
 #include "base/format_macros.h"
 #include "base/linux_util.h"
 #include "base/logging.h"
+#include "base/memory/singleton.h"
 #include "base/message_loop.h"
 #include "base/path_service.h"
 #include "base/rand_util.h"
-#include "base/singleton.h"
 #include "base/string_util.h"
-#include "base/task.h"
+#include "base/stringprintf.h"
 #include "base/threading/thread.h"
 #include "breakpad/src/client/linux/handler/exception_handler.h"
 #include "breakpad/src/client/linux/minidump_writer/linux_dumper.h"
 #include "breakpad/src/client/linux/minidump_writer/minidump_writer.h"
 #include "chrome/app/breakpad_linux.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/env_vars.h"
+#include "content/public/browser/browser_thread.h"
 
+using content::BrowserThread;
 using google_breakpad::ExceptionHandler;
 
 namespace {
 
+// The length of the control message:
+const unsigned kControlMsgSize =
+    CMSG_SPACE(2*sizeof(int)) + CMSG_SPACE(sizeof(struct ucred));
+// The length of the regular payload:
+const unsigned kCrashContextSize = sizeof(ExceptionHandler::CrashContext);
+
 // Handles the crash dump and frees the allocated BreakpadInfo struct.
 void CrashDumpTask(CrashHandlerHostLinux* handler, BreakpadInfo* info) {
   if (handler->IsShuttingDown())
@@ -53,8 +61,8 @@ void CrashDumpTask(CrashHandlerHostLinux* handler, BreakpadInfo* info) {
 
 // Since classes derived from CrashHandlerHostLinux are singletons, it's only
 // destroyed at the end of the processes lifetime, which is greater in span than
-// the lifetime of the IO message loop.
-DISABLE_RUNNABLE_METHOD_REFCOUNT(CrashHandlerHostLinux);
+// the lifetime of the IO message loop. Thus, all calls to base::Bind() use
+// non-refcounted pointers.
 
 CrashHandlerHostLinux::CrashHandlerHostLinux()
     : shutting_down_(false) {
@@ -76,7 +84,7 @@ CrashHandlerHostLinux::CrashHandlerHostLinux()
 
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(this, &CrashHandlerHostLinux::Init));
+      base::Bind(&CrashHandlerHostLinux::Init, base::Unretained(this)));
 }
 
 CrashHandlerHostLinux::~CrashHandlerHostLinux() {
@@ -112,25 +120,23 @@ void CrashHandlerHostLinux::OnFileCanReadWithoutBlocking(int fd) {
   // for writing the minidump as well as a file descriptor and a credentials
   // block so that they can't lie about their pid.
 
-  // The length of the control message:
-  static const unsigned kControlMsgSize =
-      CMSG_SPACE(2*sizeof(int)) + CMSG_SPACE(sizeof(struct ucred));
-  // The length of the regular payload:
-  static const unsigned kCrashContextSize =
-      sizeof(ExceptionHandler::CrashContext);
-
   const size_t kIovSize = 7;
   struct msghdr msg = {0};
   struct iovec iov[kIovSize];
-  char crash_context[kCrashContextSize];
+
+  // Freed in WriteDumpFile();
+  char* crash_context = new char[kCrashContextSize];
+  // Freed in CrashDumpTask();
   char* guid = new char[kGuidSize + 1];
   char* crash_url = new char[kMaxActiveURLSize + 1];
   char* distro = new char[kDistroSize + 1];
+
   char* tid_buf_addr = NULL;
   int tid_fd = -1;
   uint64_t uptime;
   char control[kControlMsgSize];
-  const ssize_t expected_msg_size = sizeof(crash_context) +
+  const ssize_t expected_msg_size =
+      kCrashContextSize +
       kGuidSize + 1 +
       kMaxActiveURLSize + 1 +
       kDistroSize + 1 +
@@ -138,7 +144,7 @@ void CrashHandlerHostLinux::OnFileCanReadWithoutBlocking(int fd) {
       sizeof(uptime);
 
   iov[0].iov_base = crash_context;
-  iov[0].iov_len = sizeof(crash_context);
+  iov[0].iov_len = kCrashContextSize;
   iov[1].iov_base = guid;
   iov[1].iov_len = kGuidSize + 1;
   iov[2].iov_base = crash_url;
@@ -218,11 +224,12 @@ void CrashHandlerHostLinux::OnFileCanReadWithoutBlocking(int fd) {
     return;
   }
 
-  // Kernel bug workaround (broken in 2.6.30 at least):
+  // Kernel bug workaround (broken in 2.6.30 and 2.6.32, working in 2.6.38).
   // The kernel doesn't translate PIDs in SCM_CREDENTIALS across PID
   // namespaces. Thus |crashing_pid| might be garbage from our point of view.
   // In the future we can remove this workaround, but we have to wait a couple
   // of years to be sure that it's worked its way out into the world.
+  // TODO(thestig) Remove the workaround when Ubuntu Lucid is deprecated.
 
   // The crashing process closes its copy of the signal_fd immediately after
   // calling sendmsg(). We can thus not reliably look for with with
@@ -245,79 +252,52 @@ void CrashHandlerHostLinux::OnFileCanReadWithoutBlocking(int fd) {
     return;
   }
 
-  if (actual_crashing_pid != crashing_pid) {
-    crashing_pid = actual_crashing_pid;
+  crashing_pid = actual_crashing_pid;
 
-    // The crashing TID set inside the compromised context via sys_gettid()
-    // in ExceptionHandler::HandleSignal is also wrong and needs to be
-    // translated.
-    //
-    // We expect the crashing thread to be in sys_read(), waiting for use to
-    // write to |signal_fd|. Most newer kernels where we have the different pid
-    // namespaces also have /proc/[pid]/syscall, so we can look through
-    // |actual_crashing_pid|'s thread group and find the thread that's in the
-    // read syscall with the right arguments.
+  // The crashing TID set inside the compromised context via
+  // sys_gettid() in ExceptionHandler::HandleSignal might be wrong (if
+  // the kernel supports PID namespacing) and may need to be
+  // translated.
+  //
+  // We expect the crashing thread to be in sys_read(), waiting for us to
+  // write to |signal_fd|. Most newer kernels where we have the different pid
+  // namespaces also have /proc/[pid]/syscall, so we can look through
+  // |actual_crashing_pid|'s thread group and find the thread that's in the
+  // read syscall with the right arguments.
 
-    std::string expected_syscall_data;
-    // /proc/[pid]/syscall is formatted as follows:
-    // syscall_number arg1 ... arg6 sp pc
-    // but we just check syscall_number through arg3.
-    base::StringAppendF(&expected_syscall_data, "%d 0x%x %p 0x1 ",
-                        SYS_read, tid_fd, tid_buf_addr);
-    pid_t crashing_tid =
-        base::FindThreadIDWithSyscall(crashing_pid, expected_syscall_data);
-    if (crashing_tid == -1) {
-      // We didn't find the thread we want. Maybe it didn't reach sys_read()
-      // yet, or the kernel doesn't support /proc/[pid]/syscall or the thread
-      // went away.  We'll just take a guess here and assume the crashing
-      // thread is the thread group leader.
-      crashing_tid = crashing_pid;
-    }
-
-    ExceptionHandler::CrashContext* bad_context =
-        reinterpret_cast<ExceptionHandler::CrashContext*>(crash_context);
-    bad_context->tid = crashing_tid;
+  std::string expected_syscall_data;
+  // /proc/[pid]/syscall is formatted as follows:
+  // syscall_number arg1 ... arg6 sp pc
+  // but we just check syscall_number through arg3.
+  base::StringAppendF(&expected_syscall_data, "%d 0x%x %p 0x1 ",
+                      SYS_read, tid_fd, tid_buf_addr);
+  bool syscall_supported = false;
+  pid_t crashing_tid =
+      base::FindThreadIDWithSyscall(crashing_pid,
+                                    expected_syscall_data,
+                                    &syscall_supported);
+  if (crashing_tid == -1) {
+    // We didn't find the thread we want. Maybe it didn't reach
+    // sys_read() yet or the thread went away.  We'll just take a
+    // guess here and assume the crashing thread is the thread group
+    // leader.  If procfs syscall is not supported by the kernel, then
+    // we assume the kernel also does not support TID namespacing and
+    // trust the TID passed by the crashing process.
+    LOG(WARNING) << "Could not translate tid - assuming crashing thread is "
+        "thread group leader; syscall_supported=" << syscall_supported;
+    crashing_tid = crashing_pid;
   }
 
-  bool upload = true;
-  FilePath dumps_path("/tmp");
-  PathService::Get(base::DIR_TEMP, &dumps_path);
-  if (getenv(env_vars::kHeadless)) {
-    upload = false;
-    PathService::Get(chrome::DIR_CRASH_DUMPS, &dumps_path);
-  }
-  const uint64 rand = base::RandUint64();
-  const std::string minidump_filename =
-      StringPrintf("%s/chromium-%s-minidump-%016" PRIx64 ".dmp",
-                   dumps_path.value().c_str(), process_type_.c_str(), rand);
-  if (!google_breakpad::WriteMinidump(minidump_filename.c_str(),
-                                      crashing_pid, crash_context,
-                                      kCrashContextSize)) {
-    LOG(ERROR) << "Failed to write crash dump for pid " << crashing_pid;
-    HANDLE_EINTR(close(signal_fd));
-  }
-
-  // Send the done signal to the process: it can exit now.
-  memset(&msg, 0, sizeof(msg));
-  struct iovec done_iov;
-  done_iov.iov_base = const_cast<char*>("\x42");
-  done_iov.iov_len = 1;
-  msg.msg_iov = &done_iov;
-  msg.msg_iovlen = 1;
-
-  HANDLE_EINTR(sendmsg(signal_fd, &msg, MSG_DONTWAIT | MSG_NOSIGNAL));
-  HANDLE_EINTR(close(signal_fd));
+  ExceptionHandler::CrashContext* bad_context =
+      reinterpret_cast<ExceptionHandler::CrashContext*>(crash_context);
+  bad_context->tid = crashing_tid;
 
   // Sanitize the string data a bit more
   guid[kGuidSize] = crash_url[kMaxActiveURLSize] = distro[kDistroSize] = 0;
 
+  // Freed in CrashDumpTask();
   BreakpadInfo* info = new BreakpadInfo;
 
-  char* minidump_filename_str = new char[minidump_filename.length() + 1];
-  minidump_filename.copy(minidump_filename_str, minidump_filename.length());
-  minidump_filename_str[minidump_filename.length()] = '\0';
-  info->filename = minidump_filename_str;
-
   info->process_type_length = process_type_.length();
   char* process_type_str = new char[info->process_type_length + 1];
   process_type_.copy(process_type_str, info->process_type_length);
@@ -333,12 +313,74 @@ void CrashHandlerHostLinux::OnFileCanReadWithoutBlocking(int fd) {
   info->distro_length = strlen(distro);
   info->distro = distro;
 
-  info->upload = upload;
+  info->upload = (getenv(env_vars::kHeadless) == NULL);
   info->process_start_time = uptime;
 
+  BrowserThread::PostTask(
+      BrowserThread::FILE, FROM_HERE,
+      base::Bind(&CrashHandlerHostLinux::WriteDumpFile,
+                 base::Unretained(this),
+                 info,
+                 crashing_pid,
+                 crash_context,
+                 signal_fd));
+}
+
+void CrashHandlerHostLinux::WriteDumpFile(BreakpadInfo* info,
+                                          pid_t crashing_pid,
+                                          char* crash_context,
+                                          int signal_fd) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::FILE));
+
+  FilePath dumps_path("/tmp");
+  PathService::Get(base::DIR_TEMP, &dumps_path);
+  if (!info->upload)
+    PathService::Get(chrome::DIR_CRASH_DUMPS, &dumps_path);
+  const uint64 rand = base::RandUint64();
+  const std::string minidump_filename =
+      base::StringPrintf("%s/chromium-%s-minidump-%016" PRIx64 ".dmp",
+                         dumps_path.value().c_str(),
+                         process_type_.c_str(),
+                         rand);
+  if (!google_breakpad::WriteMinidump(minidump_filename.c_str(),
+                                      crashing_pid, crash_context,
+                                      kCrashContextSize)) {
+    LOG(ERROR) << "Failed to write crash dump for pid " << crashing_pid;
+  }
+  delete[] crash_context;
+
+  // Freed in CrashDumpTask();
+  char* minidump_filename_str = new char[minidump_filename.length() + 1];
+  minidump_filename.copy(minidump_filename_str, minidump_filename.length());
+  minidump_filename_str[minidump_filename.length()] = '\0';
+  info->filename = minidump_filename_str;
+
+  BrowserThread::PostTask(
+      BrowserThread::IO, FROM_HERE,
+      base::Bind(&CrashHandlerHostLinux::QueueCrashDumpTask,
+                 base::Unretained(this),
+                 info,
+                 signal_fd));
+}
+
+void CrashHandlerHostLinux::QueueCrashDumpTask(BreakpadInfo* info,
+                                               int signal_fd) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
+
+  // Send the done signal to the process: it can exit now.
+  struct msghdr msg = {0};
+  struct iovec done_iov;
+  done_iov.iov_base = const_cast<char*>("\x42");
+  done_iov.iov_len = 1;
+  msg.msg_iov = &done_iov;
+  msg.msg_iovlen = 1;
+
+  HANDLE_EINTR(sendmsg(signal_fd, &msg, MSG_DONTWAIT | MSG_NOSIGNAL));
+  HANDLE_EINTR(close(signal_fd));
+
   uploader_thread_->message_loop()->PostTask(
       FROM_HERE,
-      NewRunnableFunction(&CrashDumpTask, this, info));
+      base::Bind(&CrashDumpTask, base::Unretained(this), info));
 }
 
 void CrashHandlerHostLinux::WillDestroyCurrentMessageLoop() {
@@ -354,6 +396,22 @@ bool CrashHandlerHostLinux::IsShuttingDown() const {
   return shutting_down_;
 }
 
+ExtensionCrashHandlerHostLinux::ExtensionCrashHandlerHostLinux() {
+  InitCrashUploaderThread();
+}
+
+ExtensionCrashHandlerHostLinux::~ExtensionCrashHandlerHostLinux() {
+}
+
+void ExtensionCrashHandlerHostLinux::SetProcessType() {
+  process_type_ = "extension";
+}
+
+// static
+ExtensionCrashHandlerHostLinux* ExtensionCrashHandlerHostLinux::GetInstance() {
+  return Singleton<ExtensionCrashHandlerHostLinux>::get();
+}
+
 GpuCrashHandlerHostLinux::GpuCrashHandlerHostLinux() {
   InitCrashUploaderThread();
 }
@@ -386,6 +444,22 @@ PluginCrashHandlerHostLinux* PluginCrashHandlerHostLinux::GetInstance() {
   return Singleton<PluginCrashHandlerHostLinux>::get();
 }
 
+PpapiCrashHandlerHostLinux::PpapiCrashHandlerHostLinux() {
+  InitCrashUploaderThread();
+}
+
+PpapiCrashHandlerHostLinux::~PpapiCrashHandlerHostLinux() {
+}
+
+void PpapiCrashHandlerHostLinux::SetProcessType() {
+  process_type_ = "ppapi";
+}
+
+// static
+PpapiCrashHandlerHostLinux* PpapiCrashHandlerHostLinux::GetInstance() {
+  return Singleton<PpapiCrashHandlerHostLinux>::get();
+}
+
 RendererCrashHandlerHostLinux::RendererCrashHandlerHostLinux() {
   InitCrashUploaderThread();
 }
