@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,16 +6,24 @@
 #define CHROME_BROWSER_CRASH_HANDLER_HOST_LINUX_H_
 #pragma once
 
+#include "base/compiler_specific.h"
+#include "base/message_loop.h"
+
+#if defined(USE_LINUX_BREAKPAD)
+#include <sys/types.h>
+
 #include <string>
 
-#include "base/message_loop.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 
-template <typename T> struct DefaultSingletonTraits;
+class BreakpadInfo;
 
 namespace base {
 class Thread;
 }
+#endif  // defined(USE_LINUX_BREAKPAD)
+
+template <typename T> struct DefaultSingletonTraits;
 
 // This is the base class for singleton objects which crash dump renderers and
 // plugins on Linux. We perform the crash dump from the browser because it
@@ -37,11 +45,11 @@ class CrashHandlerHostLinux : public MessageLoopForIO::Watcher,
   }
 
   // MessagePumbLibevent::Watcher impl:
-  virtual void OnFileCanWriteWithoutBlocking(int fd);
-  virtual void OnFileCanReadWithoutBlocking(int fd);
+  virtual void OnFileCanWriteWithoutBlocking(int fd) OVERRIDE;
+  virtual void OnFileCanReadWithoutBlocking(int fd) OVERRIDE;
 
   // MessageLoop::DestructionObserver impl:
-  virtual void WillDestroyCurrentMessageLoop();
+  virtual void WillDestroyCurrentMessageLoop() OVERRIDE;
 
 #if defined(USE_LINUX_BREAKPAD)
   // Whether we are shutting down or not.
@@ -65,6 +73,15 @@ class CrashHandlerHostLinux : public MessageLoopForIO::Watcher,
 #if defined(USE_LINUX_BREAKPAD)
   // This is here on purpose to make CrashHandlerHostLinux abstract.
   virtual void SetProcessType() = 0;
+
+  // Do work on the FILE thread for OnFileCanReadWithoutBlocking().
+  void WriteDumpFile(BreakpadInfo* info,
+                     pid_t crashing_pid,
+                     char* crash_context,
+                     int signal_fd);
+
+  // Continue OnFileCanReadWithoutBlocking()'s work on the IO thread.
+  void QueueCrashDumpTask(BreakpadInfo* info, int signal_fd);
 #endif
 
   int process_socket_;
@@ -79,6 +96,23 @@ class CrashHandlerHostLinux : public MessageLoopForIO::Watcher,
   DISALLOW_COPY_AND_ASSIGN(CrashHandlerHostLinux);
 };
 
+class ExtensionCrashHandlerHostLinux : public CrashHandlerHostLinux {
+ public:
+  // Returns the singleton instance.
+  static ExtensionCrashHandlerHostLinux* GetInstance();
+
+ private:
+  friend struct DefaultSingletonTraits<ExtensionCrashHandlerHostLinux>;
+  ExtensionCrashHandlerHostLinux();
+  virtual ~ExtensionCrashHandlerHostLinux();
+
+#if defined(USE_LINUX_BREAKPAD)
+  virtual void SetProcessType() OVERRIDE;
+#endif
+
+  DISALLOW_COPY_AND_ASSIGN(ExtensionCrashHandlerHostLinux);
+};
+
 class GpuCrashHandlerHostLinux : public CrashHandlerHostLinux {
  public:
   // Returns the singleton instance.
@@ -90,7 +124,7 @@ class GpuCrashHandlerHostLinux : public CrashHandlerHostLinux {
   virtual ~GpuCrashHandlerHostLinux();
 
 #if defined(USE_LINUX_BREAKPAD)
-  virtual void SetProcessType();
+  virtual void SetProcessType() OVERRIDE;
 #endif
 
   DISALLOW_COPY_AND_ASSIGN(GpuCrashHandlerHostLinux);
@@ -107,12 +141,29 @@ class PluginCrashHandlerHostLinux : public CrashHandlerHostLinux {
   virtual ~PluginCrashHandlerHostLinux();
 
 #if defined(USE_LINUX_BREAKPAD)
-  virtual void SetProcessType();
+  virtual void SetProcessType() OVERRIDE;
 #endif
 
   DISALLOW_COPY_AND_ASSIGN(PluginCrashHandlerHostLinux);
 };
 
+class PpapiCrashHandlerHostLinux : public CrashHandlerHostLinux {
+ public:
+  // Returns the singleton instance.
+  static PpapiCrashHandlerHostLinux* GetInstance();
+
+ private:
+  friend struct DefaultSingletonTraits<PpapiCrashHandlerHostLinux>;
+  PpapiCrashHandlerHostLinux();
+  virtual ~PpapiCrashHandlerHostLinux();
+
+#if defined(USE_LINUX_BREAKPAD)
+  virtual void SetProcessType() OVERRIDE;
+#endif
+
+  DISALLOW_COPY_AND_ASSIGN(PpapiCrashHandlerHostLinux);
+};
+
 class RendererCrashHandlerHostLinux : public CrashHandlerHostLinux {
  public:
   // Returns the singleton instance.
@@ -124,7 +175,7 @@ class RendererCrashHandlerHostLinux : public CrashHandlerHostLinux {
   virtual ~RendererCrashHandlerHostLinux();
 
 #if defined(USE_LINUX_BREAKPAD)
-  virtual void SetProcessType();
+  virtual void SetProcessType() OVERRIDE;
 #endif
 
   DISALLOW_COPY_AND_ASSIGN(RendererCrashHandlerHostLinux);
