@@ -1,30 +1,32 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/custom_home_pages_table_model.h"
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/i18n/rtl.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
-#include "gfx/codec/png_codec.h"
+#include "content/public/browser/web_contents.h"
 #include "googleurl/src/gurl.h"
-#include "grit/app_resources.h"
 #include "grit/generated_resources.h"
+#include "grit/ui_resources.h"
 #include "net/base/net_util.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/models/table_model_observer.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/codec/png_codec.h"
 
 struct CustomHomePagesTableModel::Entry {
-  Entry() : title_handle(0), fav_icon_handle(0) {}
+  Entry() : title_handle(0), favicon_handle(0) {}
 
   // URL of the page.
   GURL url;
@@ -39,7 +41,7 @@ struct CustomHomePagesTableModel::Entry {
   HistoryService::Handle title_handle;
 
   // If non-zero, indicates we're loading the favicon for the page.
-  FaviconService::Handle fav_icon_handle;
+  FaviconService::Handle favicon_handle;
 };
 
 CustomHomePagesTableModel::CustomHomePagesTableModel(Profile* profile)
@@ -59,18 +61,76 @@ void CustomHomePagesTableModel::SetURLs(const std::vector<GURL>& urls) {
     entries_[i].url = urls[i];
     entries_[i].title.erase();
     entries_[i].icon.reset();
-    LoadTitleAndFavIcon(&(entries_[i]));
+    LoadTitleAndFavicon(&(entries_[i]));
   }
   // Complete change, so tell the view to just rebuild itself.
   if (observer_)
     observer_->OnModelChanged();
 }
 
+/**
+ * Move a number of existing entries to a new position, reordering the table.
+ *
+ * We determine the range of elements affected by the move, save the moved
+ * elements, compact the remaining ones, and re-insert moved elements.
+ * Expects |index_list| to be ordered ascending.
+ */
+void CustomHomePagesTableModel::MoveURLs(int insert_before,
+                                         const std::vector<int>& index_list)
+{
+  DCHECK(insert_before >= 0 && insert_before <= RowCount());
+
+  // The range of elements that needs to be reshuffled is [ |first|, |last| ).
+  int first = std::min(insert_before, index_list.front());
+  int last = std::max(insert_before, index_list.back() + 1);
+
+  // Save the dragged elements. Also, adjust insertion point if it is before a
+  // dragged element.
+  std::vector<Entry> moved_entries;
+  for (size_t i = 0; i < index_list.size(); ++i) {
+    moved_entries.push_back(entries_[index_list[i]]);
+    if (index_list[i] == insert_before)
+      insert_before++;
+  }
+
+  // Compact the range between beginning and insertion point, moving downwards.
+  size_t skip_count = 0;
+  for (int i = first; i < insert_before; ++i) {
+    if (skip_count < index_list.size() && index_list[skip_count] == i)
+      skip_count++;
+    else
+      entries_[i - skip_count]=entries_[i];
+  }
+
+  // Moving items down created a gap. We start compacting up after it.
+  first = insert_before;
+  insert_before -= skip_count;
+
+  // Now compact up for elements after the insertion point.
+  skip_count = 0;
+  for (int i = last - 1; i >= first; --i) {
+    if (skip_count < index_list.size() &&
+        index_list[index_list.size() - skip_count - 1] == i) {
+      skip_count++;
+    } else {
+      entries_[i + skip_count] = entries_[i];
+    }
+  }
+
+  // Insert moved elements.
+  std::copy(moved_entries.begin(), moved_entries.end(),
+      entries_.begin() + insert_before);
+
+  // Possibly large change, so tell the view to just rebuild itself.
+  if (observer_)
+    observer_->OnModelChanged();
+}
+
 void CustomHomePagesTableModel::Add(int index, const GURL& url) {
   DCHECK(index >= 0 && index <= RowCount());
   entries_.insert(entries_.begin() + static_cast<size_t>(index), Entry());
   entries_[index].url = url;
-  LoadTitleAndFavIcon(&(entries_[index]));
+  LoadTitleAndFavicon(&(entries_[index]));
   if (observer_)
     observer_->OnItemsAdded(index, 1);
 }
@@ -86,11 +146,11 @@ void CustomHomePagesTableModel::Remove(int index) {
     if (history_service)
       history_service->CancelRequest(entry->title_handle);
   }
-  if (entry->fav_icon_handle) {
+  if (entry->favicon_handle) {
     FaviconService* favicon_service =
         profile_->GetFaviconService(Profile::EXPLICIT_ACCESS);
     if (favicon_service)
-      favicon_service->CancelRequest(entry->fav_icon_handle);
+      favicon_service->CancelRequest(entry->favicon_handle);
   }
   entries_.erase(entries_.begin() + static_cast<size_t>(index));
   if (observer_)
@@ -111,11 +171,15 @@ void CustomHomePagesTableModel::SetToCurrentlyOpenPages() {
       continue;  // Skip incognito browsers.
 
     for (int tab_index = 0; tab_index < browser->tab_count(); ++tab_index) {
-      const GURL url = browser->GetTabContentsAt(tab_index)->GetURL();
+      const GURL url = browser->GetWebContentsAt(tab_index)->GetURL();
+      // TODO(tbreisacher) remove kChromeUISettingsHost  once options is deleted
+      // and replaced by options2
       if (!url.is_empty() &&
           !(url.SchemeIs(chrome::kChromeUIScheme) &&
-            url.host() == chrome::kChromeUISettingsHost))
+            (url.host() == chrome::kChromeUISettingsHost ||
+             url.host() == chrome::kChromeUIUberHost))) {
         Add(add_index++, url);
+      }
     }
   }
 }
@@ -152,20 +216,22 @@ void CustomHomePagesTableModel::SetObserver(ui::TableModelObserver* observer) {
   observer_ = observer;
 }
 
-void CustomHomePagesTableModel::LoadTitleAndFavIcon(Entry* entry) {
+void CustomHomePagesTableModel::LoadTitleAndFavicon(Entry* entry) {
   HistoryService* history_service =
       profile_->GetHistoryService(Profile::EXPLICIT_ACCESS);
   if (history_service) {
     entry->title_handle = history_service->QueryURL(entry->url, false,
-        &query_consumer_,
-        NewCallback(this, &CustomHomePagesTableModel::OnGotTitle));
+        &history_query_consumer_,
+        base::Bind(&CustomHomePagesTableModel::OnGotTitle,
+                   base::Unretained(this)));
   }
   FaviconService* favicon_service =
       profile_->GetFaviconService(Profile::EXPLICIT_ACCESS);
   if (favicon_service) {
-    entry->fav_icon_handle = favicon_service->GetFaviconForURL(entry->url,
-        &query_consumer_,
-        NewCallback(this, &CustomHomePagesTableModel::OnGotFavIcon));
+    entry->favicon_handle = favicon_service->GetFaviconForURL(entry->url,
+        history::FAVICON, &favicon_query_consumer_,
+        base::Bind(&CustomHomePagesTableModel::OnGotFavicon,
+                   base::Unretained(this)));
   }
 }
 
@@ -188,25 +254,22 @@ void CustomHomePagesTableModel::OnGotTitle(HistoryService::Handle handle,
   }
 }
 
-void CustomHomePagesTableModel::OnGotFavIcon(
+void CustomHomePagesTableModel::OnGotFavicon(
     FaviconService::Handle handle,
-    bool know_fav_icon,
-    scoped_refptr<RefCountedMemory> image_data,
-    bool is_expired,
-    GURL icon_url) {
+    history::FaviconData favicon) {
   int entry_index;
   Entry* entry =
-      GetEntryByLoadHandle(&Entry::fav_icon_handle, handle, &entry_index);
+      GetEntryByLoadHandle(&Entry::favicon_handle, handle, &entry_index);
   if (!entry) {
     // The URLs changed before we were called back.
     return;
   }
-  entry->fav_icon_handle = 0;
-  if (know_fav_icon && image_data.get() && image_data->size()) {
+  entry->favicon_handle = 0;
+  if (favicon.is_valid()) {
     int width, height;
     std::vector<unsigned char> decoded_data;
-    if (gfx::PNGCodec::Decode(image_data->front(),
-                              image_data->size(),
+    if (gfx::PNGCodec::Decode(favicon.image_data->front(),
+                              favicon.image_data->size(),
                               gfx::PNGCodec::FORMAT_BGRA, &decoded_data,
                               &width, &height)) {
       entry->icon.setConfig(SkBitmap::kARGB_8888_Config, width, height);
