@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,44 +6,44 @@
 
 namespace browser_defaults {
 
+#if defined(USE_AURA) || defined(OS_CHROMEOS)
+const bool kOSSupportsOtherBrowsers = false;
+#else
+const bool kOSSupportsOtherBrowsers = true;
+#endif
+
 #if defined(OS_CHROMEOS)
 
 // Make the regular omnibox text two points larger than the nine-point font
 // used in the tab strip (11pt / 72pt/in * 96px/in = 14.667px).
-const double kAutocompleteEditFontPixelSize = 14.7;
-const double kAutocompleteEditFontPixelSizeInPopup = 10.0;
+const int kAutocompleteEditFontPixelSize = 15;
 
-// This is only used by AutocompletePopupViewGtk which is unused
-// unless TOOLKIT_VIEWS is undefined:
-const int kAutocompletePopupFontSize = 7;
+const int kAutocompleteEditFontPixelSizeInPopup = 10;
 
 const SessionStartupPref::Type kDefaultSessionStartupType =
     SessionStartupPref::LAST;
 const int kMiniTabWidth = 64;
 const bool kCanToggleSystemTitleBar = false;
-const bool kRestorePopups = true;
+const bool kRestorePopups = false;
 const bool kShowImportOnBookmarkBar = false;
 const bool kShowExitMenuItem = true;
-const bool kShowAboutMenuItem = true;
-const bool kOSSupportsOtherBrowsers = false;
 const bool kDownloadPageHasShowInFolder = true;
 const bool kSizeTabButtonToTopOfTabStrip = true;
-const bool kBootstrapSyncAuthentication = true;
+const bool kSyncAutoStarts = true;
 const bool kShowOtherBrowsersInAboutMemory = false;
 const bool kAlwaysOpenIncognitoWindow = true;
-const bool kShowCancelButtonInTaskManager = true;
 
 #elif defined(TOOLKIT_USES_GTK)
 
-// 13.4px = 10pt @ 96dpi.
-const double kAutocompleteEditFontPixelSize = 13.4;
+// 14px = 10.5pt @ 96dpi.
+const int kAutocompleteEditFontPixelSize = 14;
 
-// On Windows, popup windows' autocomplete box have a font 5/6 the size of a
-// regular window, which we duplicate here for GTK.
-const double kAutocompleteEditFontPixelSizeInPopup =
+// On Windows, popup windows' location text uses a font 5/6 the size of
+// that in a regular window, which we duplicate here for GTK.
+const int kAutocompleteEditFontPixelSizeInPopup =
     kAutocompleteEditFontPixelSize * 5.0 / 6.0;
 
-const int kAutocompletePopupFontSize = 10;
+const bool kShowCancelButtonInTaskManager = true;
 
 #if defined(TOOLKIT_VIEWS)
 const bool kCanToggleSystemTitleBar = false;
@@ -51,6 +51,8 @@ const bool kCanToggleSystemTitleBar = false;
 const bool kCanToggleSystemTitleBar = true;
 #endif
 
+#else
+const bool kShowCancelButtonInTaskManager = false;
 #endif
 
 #if !defined(OS_CHROMEOS)
@@ -63,17 +65,13 @@ const bool kShowImportOnBookmarkBar = true;
 const bool kDownloadPageHasShowInFolder = true;
 #if defined(OS_MACOSX)
 const bool kShowExitMenuItem = false;
-const bool kShowAboutMenuItem = false;
 #else
 const bool kShowExitMenuItem = true;
-const bool kShowAboutMenuItem = true;
 #endif
-const bool kOSSupportsOtherBrowsers = true;
 const bool kSizeTabButtonToTopOfTabStrip = false;
-const bool kBootstrapSyncAuthentication = false;
+const bool kSyncAutoStarts = false;
 const bool kShowOtherBrowsersInAboutMemory = true;
 const bool kAlwaysOpenIncognitoWindow = false;
-const bool kShowCancelButtonInTaskManager = false;
 #endif
 
 #if defined(OS_MACOSX)
@@ -82,9 +80,15 @@ const bool kBrowserAliveWithNoWindows = true;
 const bool kBrowserAliveWithNoWindows = false;
 #endif
 
-bool bookmarks_enabled = true;
+const int kBookmarkBarHeight = 28;
+const int kNewtabBookmarkBarHeight = 57;
 
-bool skip_restore = false;
+const ui::ResourceBundle::FontStyle kAssociatedNetworkFontStyle =
+    ui::ResourceBundle::BoldFont;
+
+const int kInfoBarBorderPaddingVertical = 5;
+
+bool bookmarks_enabled = true;
 
 bool enable_help_app = true;
 
