@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,19 +10,17 @@
 
 #include "build/build_config.h"
 #include "chrome/browser/prefs/session_startup_pref.h"
+#include "ui/base/resource/resource_bundle.h"
 
 namespace browser_defaults {
 
 #if defined(USE_X11)
 
-// Size of the font in pixels used in the autocomplete box for normal windows.
-extern const double kAutocompleteEditFontPixelSize;
+// Size of the font used in the autocomplete box for normal windows, in pixels.
+extern const int kAutocompleteEditFontPixelSize;
 
-// Size of the font in pixels used in the autocomplete box for popup windows.
-extern const double kAutocompleteEditFontPixelSizeInPopup;
-
-// Size of the font used in the autocomplete popup.
-extern const int kAutocompletePopupFontSize;
+// Size of the font used in the autocomplete box for popup windows, in pixels.
+extern const int kAutocompleteEditFontPixelSizeInPopup;
 
 // Can the user toggle the system title bar?
 extern const bool kCanToggleSystemTitleBar;
@@ -48,9 +46,6 @@ extern const bool kShowImportOnBookmarkBar;
 // Should the exit menu item be shown in the toolbar menu?
 extern const bool kShowExitMenuItem;
 
-// Should the about menu item be shown in the toolbar app menu?
-extern const bool kShowAboutMenuItem;
-
 // Does the OS support other browsers? If not, operations such as default
 // browser check are not done.
 extern const bool kOSSupportsOtherBrowsers;
@@ -61,9 +56,11 @@ extern const bool kDownloadPageHasShowInFolder;
 // Should the tab strip be sized to the top of the tab strip?
 extern const bool kSizeTabButtonToTopOfTabStrip;
 
-// Whether we should bootstrap the sync authentication using cookies instead of
-// asking the user for credentials.
-extern const bool kBootstrapSyncAuthentication;
+// If true, we want to automatically start sync signin whenever we have
+// credentials (user doesn't need to go through the startup flow). This is
+// typically enabled on platforms (like ChromeOS) that have their own
+// distinct signin flow.
+extern const bool kSyncAutoStarts;
 
 // Should other browsers be shown in about:memory page?
 extern const bool kShowOtherBrowsersInAboutMemory;
@@ -74,6 +71,17 @@ extern const bool kAlwaysOpenIncognitoWindow;
 // Should the close button be shown in the Task Manager dialog?
 extern const bool kShowCancelButtonInTaskManager;
 
+// Preferred height of the bookmarks bar when shown on every page and
+// when shown only on the new tab page.
+extern const int kBookmarkBarHeight;
+extern const int kNewtabBookmarkBarHeight;
+
+// ChromiumOS network menu font
+extern const ui::ResourceBundle::FontStyle kAssociatedNetworkFontStyle;
+
+// Preferred infobar border padding in pixels.
+extern const int kInfoBarBorderPaddingVertical;
+
 //=============================================================================
 // Runtime "const" - set only once after parsing command line option and should
 // never be modified after that.
@@ -81,9 +89,6 @@ extern const bool kShowCancelButtonInTaskManager;
 // Are bookmark enabled? True by default.
 extern bool bookmarks_enabled;
 
-// Should we skip the user's preference for restore. False by default.
-extern bool skip_restore;
-
 // Whether HelpApp is enabled. True by default. This is only used by Chrome OS
 // today.
 extern bool enable_help_app;
