@@ -8,17 +8,18 @@
 
 #include "base/file_util.h"
 #include "base/json/json_reader.h"
-#include "base/string_util.h"
+#include "base/json/json_value_serializer.h"
+#include "base/stringprintf.h"
 #include "base/string_number_conversions.h"
+#include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "base/sys_info.h"
 #include "base/path_service.h"
 #include "chrome/browser/diagnostics/diagnostics_test.h"
-#include "chrome/browser/platform_util.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_version_info.h"
-#include "chrome/common/json_value_serializer.h"
+#include "ui/base/text/bytes_formatting.h"
 
 #if defined(OS_WIN)
 #include "base/win/windows_version.h"
@@ -47,27 +48,21 @@ class OperatingSystemTest : public DiagnosticTest {
   virtual int GetId() { return 0; }
 
   virtual bool ExecuteImpl(DiagnosticsModel::Observer* observer) {
-    int version = 0;
-    int major = 0;
-    int minor = 0;
 #if defined(OS_WIN)
-    version = base::win::GetVersion();
-    if (version < base::win::VERSION_XP) {
-      RecordFailure(ASCIIToUTF16("Windows 2000 or earlier"));
-      return false;
-    }
-    base::win::GetServicePackLevel(&major, &minor);
-    if ((version == base::win::VERSION_XP) && (major < 2)) {
-      RecordFailure(ASCIIToUTF16("XP Service Pack 1 or earlier"));
+    base::win::Version version = base::win::GetVersion();
+    if ((version < base::win::VERSION_XP) ||
+        ((version == base::win::VERSION_XP) &&
+         (base::win::OSInfo::GetInstance()->service_pack().major < 2))) {
+      RecordFailure(ASCIIToUTF16("Must have Windows XP SP2 or later"));
       return false;
     }
 #else
     // TODO(port): define the OS criteria for Linux and Mac.
 #endif  // defined(OS_WIN)
-    RecordSuccess(ASCIIToUTF16(StringPrintf("%s %s (%d [%d:%d])",
+    RecordSuccess(ASCIIToUTF16(base::StringPrintf(
+        "%s %s",
         base::SysInfo::OperatingSystemName().c_str(),
-        base::SysInfo::OperatingSystemVersion().c_str(),
-        version, major, minor)));
+        base::SysInfo::OperatingSystemVersion().c_str())));
     return true;
   }
 
@@ -87,7 +82,7 @@ class ConflictingDllsTest : public DiagnosticTest {
     EnumerateModulesModel* model = EnumerateModulesModel::GetInstance();
     model->set_limited_mode(true);
     model->ScanNow();
-    ListValue* list = model->GetModuleList();
+    scoped_ptr<ListValue> list(model->GetModuleList());
     if (!model->confirmed_bad_modules_detected() &&
         !model->suspected_bad_modules_detected()) {
       RecordSuccess(ASCIIToUTF16("No conflicting modules found"));
@@ -145,8 +140,7 @@ class InstallTypeTest : public DiagnosticTest {
       RecordFailure(ASCIIToUTF16("Path provider failure"));
       return false;
     }
-    user_level_ = InstallUtil::IsPerUserInstall(
-        chrome_exe.ToWStringHack().c_str());
+    user_level_ = InstallUtil::IsPerUserInstall(chrome_exe.value().c_str());
     const char* type = user_level_ ? "User Level" : "System Level";
     string16 install_type(ASCIIToUTF16(type));
 #else
@@ -182,7 +176,8 @@ class VersionTest : public DiagnosticTest {
       RecordFailure(ASCIIToUTF16("Empty Version"));
       return true;
     }
-    std::string version_modifier = platform_util::GetVersionStringModifier();
+    std::string version_modifier =
+        chrome::VersionInfo::GetVersionStringModifier();
     if (!version_modifier.empty())
       current_version += " " + version_modifier;
 #if defined(GOOGLE_CHROME_BUILD)
@@ -215,8 +210,8 @@ const TestPathInfo kPathsToTest[] = {
       false, false, true, 500 * kOneKilo},
   {"Dictionaries Directory", chrome::DIR_APP_DICTIONARIES,
       true, true, false, 0},
-  {"Inspector Directory", chrome::DIR_INSPECTOR,
-      true, false, false, 0}
+  {"Resources file", chrome::FILE_RESOURCES_PACK,
+      false, false, false, 0}
 };
 
 // Check that the user's data directory exists and the paths are writable.
@@ -241,7 +236,8 @@ class PathTest : public DiagnosticTest {
       return false;
     }
     if (!file_util::PathExists(dir_or_file)) {
-      RecordFailure(ASCIIToUTF16("Path not found"));
+      RecordFailure(ASCIIToUTF16("Path not found: ") +
+                    dir_or_file.LossyDisplayName());
       return true;
     }
 
@@ -252,15 +248,18 @@ class PathTest : public DiagnosticTest {
       file_util::GetFileSize(dir_or_file, &dir_or_file_size);
     }
     if (!dir_or_file_size && !path_info_.is_optional) {
-      RecordFailure(ASCIIToUTF16("Cannot obtain size"));
+      RecordFailure(ASCIIToUTF16("Cannot obtain size for: ") +
+                    dir_or_file.LossyDisplayName());
       return true;
     }
-    DataUnits units = GetByteDisplayUnits(dir_or_file_size);
-    string16 printable_size = FormatBytes(dir_or_file_size, units, true);
+    string16 printable_size = ui::FormatBytes(dir_or_file_size);
 
     if (path_info_.max_size > 0) {
       if (dir_or_file_size > path_info_.max_size) {
-        RecordFailure(ASCIIToUTF16("Path is too big: ") + printable_size);
+        RecordFailure(ASCIIToUTF16("Path contents too large (") +
+                      printable_size +
+                      ASCIIToUTF16(") for: ") +
+                      dir_or_file.LossyDisplayName());
         return true;
       }
     }
@@ -269,7 +268,8 @@ class PathTest : public DiagnosticTest {
       return true;
     }
     if (!file_util::PathIsWritable(dir_or_file)) {
-      RecordFailure(ASCIIToUTF16("Path is not writable"));
+      RecordFailure(ASCIIToUTF16("Path is not writable: ") +
+                    dir_or_file.LossyDisplayName());
       return true;
     }
     RecordSuccess(ASCIIToUTF16("Path exists and is writable: ")
@@ -299,8 +299,7 @@ class DiskSpaceTest : public DiagnosticTest {
       RecordFailure(ASCIIToUTF16("Unable to query free space"));
       return true;
     }
-    DataUnits units = GetByteDisplayUnits(disk_space);
-    string16 printable_size = FormatBytes(disk_space, units, true);
+    string16 printable_size = ui::FormatBytes(disk_space);
     if (disk_space < 80 * kOneMeg) {
       RecordFailure(ASCIIToUTF16("Low disk space : ") + printable_size);
       return true;
@@ -316,7 +315,7 @@ class DiskSpaceTest : public DiagnosticTest {
 // Checks that a given json file can be correctly parsed.
 class JSONTest : public DiagnosticTest {
  public:
-  JSONTest(const FilePath& path, const string16 name, int64 max_file_size)
+  JSONTest(const FilePath& path, const string16& name, int64 max_file_size)
       : DiagnosticTest(name), path_(path), max_file_size_(max_file_size) {
   }
 
@@ -381,7 +380,7 @@ DiagnosticTest* MakeDictonaryDirTest() {
   return new PathTest(kPathsToTest[2]);
 }
 
-DiagnosticTest* MakeInspectorDirTest() {
+DiagnosticTest* MakeResourcesFileTest() {
   return new PathTest(kPathsToTest[3]);
 }
 
