@@ -1,23 +1,36 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/download/download_prefs.h"
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/file_util.h"
+#include "base/logging.h"
 #include "base/string_split.h"
 #include "base/string_util.h"
 #include "base/sys_string_conversions.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_thread.h"
+#include "chrome/browser/download/chrome_download_manager_delegate.h"
 #include "chrome/browser/download/download_extensions.h"
+#include "chrome/browser/download/download_service.h"
+#include "chrome/browser/download/download_service_factory.h"
 #include "chrome/browser/download/download_util.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/profiles/profile.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/download_manager.h"
+#include "content/public/browser/save_page_type.h"
+
+using content::BrowserThread;
+using content::DownloadManager;
 
 DownloadPrefs::DownloadPrefs(PrefService* prefs) : prefs_(prefs) {
   prompt_for_download_.Init(prefs::kPromptForDownload, prefs, NULL);
   download_path_.Init(prefs::kDownloadDefaultDirectory, prefs, NULL);
+  save_file_type_.Init(prefs::kSaveFileType, prefs, NULL);
 
   // We store any file extension that should be opened automatically at
   // download completion in this pref.
@@ -32,7 +45,8 @@ DownloadPrefs::DownloadPrefs(PrefService* prefs) : prefs_(prefs) {
 #elif defined(OS_WIN)
     FilePath path(UTF8ToWide(extensions[i]));
 #endif
-    if (!extensions[i].empty() && download_util::IsFileSafe(path))
+    if (!extensions[i].empty() &&
+        download_util::GetFileDangerLevel(path) == download_util::NotDangerous)
       auto_open_.insert(path.value());
   }
 }
@@ -43,21 +57,32 @@ DownloadPrefs::~DownloadPrefs() {
 
 // static
 void DownloadPrefs::RegisterUserPrefs(PrefService* prefs) {
-  prefs->RegisterBooleanPref(prefs::kPromptForDownload, false);
-  prefs->RegisterStringPref(prefs::kDownloadExtensionsToOpen, "");
-  prefs->RegisterBooleanPref(prefs::kDownloadDirUpgraded, false);
+  prefs->RegisterBooleanPref(prefs::kPromptForDownload,
+                             false,
+                             PrefService::SYNCABLE_PREF);
+  prefs->RegisterStringPref(prefs::kDownloadExtensionsToOpen,
+                            "",
+                            PrefService::UNSYNCABLE_PREF);
+  prefs->RegisterBooleanPref(prefs::kDownloadDirUpgraded,
+                             false,
+                             PrefService::UNSYNCABLE_PREF);
+  prefs->RegisterIntegerPref(prefs::kSaveFileType,
+                             content::SAVE_PAGE_TYPE_AS_COMPLETE_HTML,
+                             PrefService::UNSYNCABLE_PREF);
 
   // The default download path is userprofile\download.
   const FilePath& default_download_path =
       download_util::GetDefaultDownloadDirectory();
   prefs->RegisterFilePathPref(prefs::kDownloadDefaultDirectory,
-                              default_download_path);
+                              default_download_path,
+                              PrefService::UNSYNCABLE_PREF);
 
 #if defined(OS_CHROMEOS)
   // Ensure that the download directory specified in the preferences exists.
   BrowserThread::PostTask(
       BrowserThread::FILE, FROM_HERE,
-      NewRunnableFunction(&file_util::CreateDirectory, default_download_path));
+      base::Bind(base::IgnoreResult(&file_util::CreateDirectory),
+                 default_download_path));
 #endif  // defined(OS_CHROMEOS)
 
   // If the download path is dangerous we forcefully reset it. But if we do
@@ -74,6 +99,34 @@ void DownloadPrefs::RegisterUserPrefs(PrefService* prefs) {
   }
 }
 
+// static
+DownloadPrefs* DownloadPrefs::FromDownloadManager(
+    DownloadManager* download_manager) {
+  ChromeDownloadManagerDelegate* delegate =
+      static_cast<ChromeDownloadManagerDelegate*>(download_manager->delegate());
+  return delegate->download_prefs();
+}
+
+// static
+DownloadPrefs* DownloadPrefs::FromBrowserContext(
+    content::BrowserContext* browser_context) {
+  Profile* profile = static_cast<Profile*>(browser_context);
+  DownloadService* download_service =
+      DownloadServiceFactory::GetForProfile(profile);
+  return FromDownloadManager(download_service->GetDownloadManager());
+}
+
+bool DownloadPrefs::PromptForDownload() const {
+  // If the DownloadDirectory policy is set, then |prompt_for_download_| should
+  // always be false.
+  DCHECK(!download_path_.IsManaged() || !prompt_for_download_.GetValue());
+  return *prompt_for_download_;
+}
+
+bool DownloadPrefs::IsDownloadPathManaged() const {
+  return download_path_.IsManaged();
+}
+
 bool DownloadPrefs::IsAutoOpenUsed() const {
   return !auto_open_.empty();
 }
@@ -105,11 +158,6 @@ void DownloadPrefs::DisableAutoOpenBasedOnExtension(const FilePath& file_name) {
   SaveAutoOpenState();
 }
 
-void DownloadPrefs::ResetToDefaults() {
-  // TODO(phajdan.jr): Should we reset rest of prefs here?
-  ResetAutoOpen();
-}
-
 void DownloadPrefs::ResetAutoOpen() {
   auto_open_.clear();
   SaveAutoOpenState();
