@@ -4,32 +4,25 @@
 
 #include "chrome/browser/download/download_request_infobar_delegate.h"
 
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "grit/generated_resources.h"
-#include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
 DownloadRequestInfoBarDelegate::DownloadRequestInfoBarDelegate(
-    TabContents* tab,
+    InfoBarTabHelper* infobar_helper,
     DownloadRequestLimiter::TabDownloadState* host)
-    : ConfirmInfoBarDelegate(tab),
+    : ConfirmInfoBarDelegate(infobar_helper),
       host_(host) {
-  if (tab)
-    tab->AddInfoBar(this);
 }
 
 DownloadRequestInfoBarDelegate::~DownloadRequestInfoBarDelegate() {
+  if (host_)
+    host_->Cancel();
 }
 
-void DownloadRequestInfoBarDelegate::InfoBarClosed() {
-  Cancel();
-  // This will delete us.
-  ConfirmInfoBarDelegate::InfoBarClosed();
-}
-
-SkBitmap* DownloadRequestInfoBarDelegate::GetIcon() const {
-  return ResourceBundle::GetSharedInstance().GetBitmapNamed(
+gfx::Image* DownloadRequestInfoBarDelegate::GetIcon() const {
+  return &ResourceBundle::GetSharedInstance().GetNativeImageNamed(
       IDR_INFOBAR_MULTIPLE_DOWNLOADS);
 }
 
@@ -37,10 +30,6 @@ string16 DownloadRequestInfoBarDelegate::GetMessageText() const {
   return l10n_util::GetStringUTF16(IDS_MULTI_DOWNLOAD_WARNING);
 }
 
-int DownloadRequestInfoBarDelegate::GetButtons() const {
-  return BUTTON_OK | BUTTON_CANCEL;
-}
-
 string16 DownloadRequestInfoBarDelegate::GetButtonLabel(
     InfoBarButton button) const {
   return l10n_util::GetStringUTF16((button == BUTTON_OK) ?
@@ -55,11 +44,3 @@ bool DownloadRequestInfoBarDelegate::Accept() {
 
   return !host_;
 }
-
-bool DownloadRequestInfoBarDelegate::Cancel() {
-  if (host_) {
-    host_->Cancel();
-    host_ = NULL;
-  }
-  return true;
-}
