@@ -8,9 +8,7 @@
 
 #include "base/basictypes.h"
 #include "chrome/browser/download/download_request_limiter.h"
-#include "chrome/browser/tab_contents/infobar_delegate.h"
-
-class TabContents;
+#include "chrome/browser/tab_contents/confirm_infobar_delegate.h"
 
 // An infobar delegate that presents the user with a choice to allow or deny
 // multiple downloads from the same site. This confirmation step protects
@@ -19,7 +17,7 @@ class TabContents;
 class DownloadRequestInfoBarDelegate : public ConfirmInfoBarDelegate {
  public:
   DownloadRequestInfoBarDelegate(
-      TabContents* tab,
+      InfoBarTabHelper* infobar_helper,
       DownloadRequestLimiter::TabDownloadState* host);
 
   void set_host(DownloadRequestLimiter::TabDownloadState* host) {
@@ -30,13 +28,10 @@ class DownloadRequestInfoBarDelegate : public ConfirmInfoBarDelegate {
   virtual ~DownloadRequestInfoBarDelegate();
 
   // ConfirmInfoBarDelegate:
-  virtual void InfoBarClosed();
-  virtual SkBitmap* GetIcon() const;
-  virtual string16 GetMessageText() const;
-  virtual int GetButtons() const;
-  virtual string16 GetButtonLabel(InfoBarButton button) const;
-  virtual bool Accept();
-  virtual bool Cancel();
+  virtual gfx::Image* GetIcon() const OVERRIDE;
+  virtual string16 GetMessageText() const OVERRIDE;
+  virtual string16 GetButtonLabel(InfoBarButton button) const OVERRIDE;
+  virtual bool Accept() OVERRIDE;
 
   DownloadRequestLimiter::TabDownloadState* host_;
 
