@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -15,34 +15,31 @@
 #include "base/i18n/rtl.h"
 #include "base/i18n/time_formatting.h"
 #include "base/lazy_instance.h"
+#include "base/metrics/histogram.h"
 #include "base/path_service.h"
 #include "base/string16.h"
 #include "base/string_number_conversions.h"
-#include "base/stringprintf.h"
 #include "base/sys_string_conversions.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/utf_string_conversions.h"
+#include "base/value_conversions.h"
 #include "base/values.h"
 #include "base/win/windows_version.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/download/download_extensions.h"
-#include "chrome/browser/download/download_item.h"
 #include "chrome/browser/download/download_item_model.h"
-#include "chrome/browser/download/download_manager.h"
-#include "chrome/browser/extensions/crx_installer.h"
-#include "chrome/browser/extensions/extension_install_ui.h"
-#include "chrome/browser/extensions/extension_service.h"
-#include "chrome/browser/history/download_create_info.h"
-#include "chrome/browser/net/chrome_url_request_context.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/resource_dispatcher_host.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_paths.h"
-#include "chrome/common/notification_service.h"
 #include "chrome/common/time_format.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/rect.h"
+#include "content/browser/download/download_create_info.h"
+#include "content/browser/download/download_file.h"
+#include "content/browser/download/download_types.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/download_item.h"
+#include "content/public/browser/download_manager.h"
+#include "content/public/common/url_constants.h"
 #include "grit/generated_resources.h"
 #include "grit/locale_settings.h"
 #include "grit/theme_resources.h"
@@ -53,76 +50,71 @@
 #include "third_party/skia/include/core/SkShader.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/text/bytes_formatting.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/image/image.h"
+#include "ui/gfx/rect.h"
 
 #if defined(TOOLKIT_VIEWS)
 #include "ui/base/dragdrop/os_exchange_data.h"
-#include "views/drag_utils.h"
+#include "ui/views/drag_utils.h"
 #endif
 
 #if defined(TOOLKIT_USES_GTK)
 #if defined(TOOLKIT_VIEWS)
 #include "ui/base/dragdrop/drag_drop_types.h"
-#include "views/widget/widget_gtk.h"
+#include "ui/views/widget/native_widget_gtk.h"
 #elif defined(TOOLKIT_GTK)
 #include "chrome/browser/ui/gtk/custom_drag.h"
+#include "chrome/browser/ui/gtk/unity_service.h"
 #endif  // defined(TOOLKIT_GTK)
 #endif  // defined(TOOLKIT_USES_GTK)
 
-#if defined(OS_WIN)
-#include "app/win/win_util.h"
+#if defined(OS_WIN) && !defined(USE_AURA)
 #include "base/win/scoped_comptr.h"
-#include "chrome/browser/browser_list.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
 #include "ui/base/dragdrop/drag_source.h"
 #include "ui/base/dragdrop/os_exchange_data_provider_win.h"
 #endif
 
+// TODO(phajdan.jr): Find some standard location for this, maintaining
+// the same value on all platforms.
+static const double PI = 3.141592653589793;
+
+using content::DownloadFile;
+using content::DownloadItem;
+
+namespace {
+
+// Returns a string constant to be used as the |danger_type| value in
+// CreateDownloadItemValue().  We only return strings for DANGEROUS_FILE,
+// DANGEROUS_URL and DANGEROUS_CONTENT because the |danger_type| value is only
+// defined if the value of |state| is |DANGEROUS|.
+const char* GetDangerTypeString(content::DownloadDangerType danger_type) {
+  switch (danger_type) {
+    case content::DOWNLOAD_DANGER_TYPE_DANGEROUS_FILE:
+      return "DANGEROUS_FILE";
+    case content::DOWNLOAD_DANGER_TYPE_DANGEROUS_URL:
+      return "DANGEROUS_URL";
+    case content::DOWNLOAD_DANGER_TYPE_DANGEROUS_CONTENT:
+      return "DANGEROUS_CONTENT";
+    default:
+      // We shouldn't be returning a danger type string if it is
+      // NOT_DANGEROUS or MAYBE_DANGEROUS_CONTENT.
+      NOTREACHED();
+      return "";
+  }
+}
+
+}  // namespace
+
 namespace download_util {
 
 // How many times to cycle the complete animation. This should be an odd number
 // so that the animation ends faded out.
 static const int kCompleteAnimationCycles = 5;
 
-// The maximum number of 'uniquified' files we will try to create.
-// This is used when the filename we're trying to download is already in use,
-// so we create a new unique filename by appending " (nnn)" before the
-// extension, where 1 <= nnn <= kMaxUniqueFiles.
-// Also used by code that cleans up said files.
-static const int kMaxUniqueFiles = 100;
-
-namespace {
-
-#if defined(OS_WIN)
-// Returns whether the specified extension is automatically integrated into the
-// windows shell.
-bool IsShellIntegratedExtension(const string16& extension) {
-  string16 extension_lower = StringToLowerASCII(extension);
-
-  static const wchar_t* const integrated_extensions[] = {
-    // See <http://msdn.microsoft.com/en-us/library/ms811694.aspx>.
-    L"local",
-    // Right-clicking on shortcuts can be magical.
-    L"lnk",
-  };
-
-  for (int i = 0; i < arraysize(integrated_extensions); ++i) {
-    if (extension_lower == integrated_extensions[i])
-      return true;
-  }
-
-  // See <http://www.juniper.net/security/auto/vulnerabilities/vuln2612.html>.
-  // That vulnerability report is not exactly on point, but files become magical
-  // if their end in a CLSID.  Here we block extensions that look like CLSIDs.
-  if (extension_lower.size() > 0 && extension_lower.at(0) == L'{' &&
-      extension_lower.at(extension_lower.length() - 1) == L'}')
-    return true;
-
-  return false;
-}
-#endif  // OS_WIN
-
-}  // namespace
-
 // Download temporary file creation --------------------------------------------
 
 class DefaultDownloadDirectory {
@@ -144,19 +136,12 @@ class DefaultDownloadDirectory {
 };
 
 static base::LazyInstance<DefaultDownloadDirectory>
-    g_default_download_directory(base::LINKER_INITIALIZED);
+    g_default_download_directory = LAZY_INSTANCE_INITIALIZER;
 
 const FilePath& GetDefaultDownloadDirectory() {
   return g_default_download_directory.Get().path();
 }
 
-bool CreateTemporaryFileForDownload(FilePath* temp_file) {
-  if (file_util::CreateTemporaryFileInDir(GetDefaultDownloadDirectory(),
-                                          temp_file))
-    return true;
-  return file_util::CreateTemporaryFile(temp_file);
-}
-
 bool DownloadPathIsDangerous(const FilePath& download_path) {
   FilePath desktop_dir;
   if (!PathService::Get(chrome::DIR_USER_DESKTOP, &desktop_dir)) {
@@ -166,131 +151,17 @@ bool DownloadPathIsDangerous(const FilePath& download_path) {
   return (download_path == desktop_dir);
 }
 
-void GenerateExtension(const FilePath& file_name,
-                       const std::string& mime_type,
-                       FilePath::StringType* generated_extension) {
-  // We're worried about two things here:
-  //
-  // 1) Usability.  If the site fails to provide a file extension, we want to
-  //    guess a reasonable file extension based on the content type.
-  //
-  // 2) Shell integration.  Some file extensions automatically integrate with
-  //    the shell.  We block these extensions to prevent a malicious web site
-  //    from integrating with the user's shell.
+void GenerateFileNameFromRequest(const DownloadItem& download_item,
+                                 FilePath* generated_name) {
+  std::string default_file_name(
+      l10n_util::GetStringUTF8(IDS_DEFAULT_DOWNLOAD_FILENAME));
 
-  // See if our file name already contains an extension.
-  FilePath::StringType extension = file_name.Extension();
-  if (!extension.empty())
-    extension.erase(extension.begin());  // Erase preceding '.'.
-
-#if defined(OS_WIN)
-  static const FilePath::CharType default_extension[] =
-      FILE_PATH_LITERAL("download");
-
-  // Rename shell-integrated extensions.
-  if (IsShellIntegratedExtension(extension))
-    extension.assign(default_extension);
-#endif
-
-  if (extension.empty()) {
-    // The GetPreferredExtensionForMimeType call will end up going to disk.  Do
-    // this on another thread to avoid slowing the IO thread.
-    // http://crbug.com/61827
-    base::ThreadRestrictions::ScopedAllowIO allow_io;
-    net::GetPreferredExtensionForMimeType(mime_type, &extension);
-  }
-
-  generated_extension->swap(extension);
-}
-
-void GenerateFileNameFromInfo(DownloadCreateInfo* info,
-                              FilePath* generated_name) {
-  GenerateFileName(GURL(info->url),
-                   info->content_disposition,
-                   info->referrer_charset,
-                   info->mime_type,
-                   generated_name);
-}
-
-void GenerateFileName(const GURL& url,
-                      const std::string& content_disposition,
-                      const std::string& referrer_charset,
-                      const std::string& mime_type,
-                      FilePath* generated_name) {
-#if defined(OS_WIN)
-  FilePath default_file_path(
-      l10n_util::GetStringUTF16(IDS_DEFAULT_DOWNLOAD_FILENAME));
-#elif defined(OS_POSIX)
-  std::string default_file =
-      l10n_util::GetStringUTF8(IDS_DEFAULT_DOWNLOAD_FILENAME);
-  FilePath default_file_path(
-      base::SysWideToNativeMB(base::SysUTF8ToWide(default_file)));
-#endif
-
-  *generated_name = net::GetSuggestedFilename(GURL(url),
-                                              content_disposition,
-                                              referrer_charset,
-                                              default_file_path);
-
-  DCHECK(!generated_name->empty());
-
-  GenerateSafeFileName(mime_type, generated_name);
-}
-
-void GenerateSafeFileName(const std::string& mime_type, FilePath* file_name) {
-  // Make sure we get the right file extension
-  FilePath::StringType extension;
-  GenerateExtension(*file_name, mime_type, &extension);
-  *file_name = file_name->ReplaceExtension(extension);
-
-#if defined(OS_WIN)
-  // Prepend "_" to the file name if it's a reserved name
-  FilePath::StringType leaf_name = file_name->BaseName().value();
-  DCHECK(!leaf_name.empty());
-  if (app::win::IsReservedName(leaf_name)) {
-    leaf_name = FilePath::StringType(FILE_PATH_LITERAL("_")) + leaf_name;
-    *file_name = file_name->DirName();
-    if (file_name->value() == FilePath::kCurrentDirectory) {
-      *file_name = FilePath(leaf_name);
-    } else {
-      *file_name = file_name->Append(leaf_name);
-    }
-  }
-#endif
-}
-
-void OpenChromeExtension(Profile* profile,
-                         DownloadManager* download_manager,
-                         const DownloadItem& download_item) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  DCHECK(download_item.is_extension_install());
-
-  ExtensionService* service = profile->GetExtensionService();
-  CHECK(service);
-  NotificationService* nservice = NotificationService::current();
-  GURL nonconst_download_url = download_item.url();
-  nservice->Notify(NotificationType::EXTENSION_READY_FOR_INSTALL,
-                   Source<DownloadManager>(download_manager),
-                   Details<GURL>(&nonconst_download_url));
-
-  scoped_refptr<CrxInstaller> installer(
-      new CrxInstaller(service, new ExtensionInstallUI(profile)));
-  installer->set_delete_source(true);
-
-  if (UserScript::HasUserScriptFileExtension(download_item.url())) {
-    installer->InstallUserScript(download_item.full_path(),
-                                 download_item.url());
-    return;
-  }
-
-  bool is_gallery_download = service->IsDownloadFromGallery(
-      download_item.url(), download_item.referrer_url());
-  installer->set_original_mime_type(download_item.original_mime_type());
-  installer->set_apps_require_extension_mime_type(true);
-  installer->set_original_url(download_item.url());
-  installer->set_is_gallery_install(is_gallery_download);
-  installer->InstallCrx(download_item.full_path());
-  installer->set_allow_silent_install(is_gallery_download);
+  *generated_name = net::GenerateFileName(download_item.GetURL(),
+                                          download_item.GetContentDisposition(),
+                                          download_item.GetReferrerCharset(),
+                                          download_item.GetSuggestedFilename(),
+                                          download_item.GetMimeType(),
+                                          default_file_name);
 }
 
 // Download progress painting --------------------------------------------------
@@ -336,9 +207,9 @@ void PaintDownloadProgress(gfx::Canvas* canvas,
 
 #if defined(TOOLKIT_VIEWS)
   // Mirror the positions if necessary.
-  int mirrored_x = containing_view->MirroredLeftPointForRect(background_bounds);
+  int mirrored_x = containing_view->GetMirroredXForRect(background_bounds);
   background_bounds.set_x(mirrored_x);
-  mirrored_x = containing_view->MirroredLeftPointForRect(foreground_bounds);
+  mirrored_x = containing_view->GetMirroredXForRect(foreground_bounds);
   foreground_bounds.set_x(mirrored_x);
 #endif
 
@@ -392,7 +263,7 @@ void PaintDownloadProgress(gfx::Canvas* canvas,
     foreground_paint.setShader(shader);
     foreground_paint.setAntiAlias(true);
     shader->unref();
-    canvas->AsCanvasSkia()->drawPath(path, foreground_paint);
+    canvas->GetSkCanvas()->drawPath(path, foreground_paint);
     return;
   }
 
@@ -423,18 +294,52 @@ void PaintDownloadComplete(gfx::Canvas* canvas,
                             complete->width(), complete->height());
 #if defined(TOOLKIT_VIEWS)
   // Mirror the positions if necessary.
-  complete_bounds.set_x(
-      containing_view->MirroredLeftPointForRect(complete_bounds));
+  complete_bounds.set_x(containing_view->GetMirroredXForRect(complete_bounds));
 #endif
 
   // Start at full opacity, then loop back and forth five times before ending
   // at zero opacity.
-  static const double PI = 3.141592653589793;
   double opacity = sin(animation_progress * PI * kCompleteAnimationCycles +
                    PI/2) / 2 + 0.5;
 
   canvas->SaveLayerAlpha(static_cast<int>(255.0 * opacity), complete_bounds);
-  canvas->AsCanvasSkia()->drawARGB(0, 255, 255, 255, SkXfermode::kClear_Mode);
+  canvas->GetSkCanvas()->drawARGB(0, 255, 255, 255, SkXfermode::kClear_Mode);
+  canvas->DrawBitmapInt(*complete, complete_bounds.x(), complete_bounds.y());
+  canvas->Restore();
+}
+
+void PaintDownloadInterrupted(gfx::Canvas* canvas,
+#if defined(TOOLKIT_VIEWS)
+                              views::View* containing_view,
+#endif
+                              int origin_x,
+                              int origin_y,
+                              double animation_progress,
+                              PaintDownloadProgressSize size) {
+  // Load up our common bitmaps.
+  if (!g_foreground_16) {
+    ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+    g_foreground_16 = rb.GetBitmapNamed(IDR_DOWNLOAD_PROGRESS_FOREGROUND_16);
+    g_foreground_32 = rb.GetBitmapNamed(IDR_DOWNLOAD_PROGRESS_FOREGROUND_32);
+  }
+
+  SkBitmap* complete = (size == BIG) ? g_foreground_32 : g_foreground_16;
+
+  gfx::Rect complete_bounds(origin_x, origin_y,
+                            complete->width(), complete->height());
+#if defined(TOOLKIT_VIEWS)
+  // Mirror the positions if necessary.
+  complete_bounds.set_x(containing_view->GetMirroredXForRect(complete_bounds));
+#endif
+
+  // Start at zero opacity, then loop back and forth five times before ending
+  // at full opacity.
+  double opacity = sin(
+      (1.0 - animation_progress) * PI * kCompleteAnimationCycles + PI/2) / 2 +
+          0.5;
+
+  canvas->SaveLayerAlpha(static_cast<int>(255.0 * opacity), complete_bounds);
+  canvas->GetSkCanvas()->drawARGB(0, 255, 255, 255, SkXfermode::kClear_Mode);
   canvas->DrawBitmapInt(*complete, complete_bounds.x(), complete_bounds.y());
   canvas->Restore();
 }
@@ -463,7 +368,7 @@ int GetBigProgressIconOffset() {
 #if defined(TOOLKIT_VIEWS)
 // Download dragging
 void DragDownload(const DownloadItem* download,
-                  SkBitmap* icon,
+                  gfx::Image* icon,
                   gfx::NativeView view) {
   DCHECK(download);
 
@@ -472,23 +377,26 @@ void DragDownload(const DownloadItem* download,
 
   if (icon) {
     drag_utils::CreateDragImageForFile(
-        download->GetFileNameToReportUser(), icon, &data);
+        download->GetFileNameToReportUser(), *icon, &data);
   }
 
-  const FilePath full_path = download->full_path();
-  data.SetFilename(full_path.ToWStringHack());
+  const FilePath full_path = download->GetFullPath();
+  data.SetFilename(full_path);
 
-  std::string mime_type = download->mime_type();
+  std::string mime_type = download->GetMimeType();
   if (mime_type.empty())
     net::GetMimeTypeFromFile(full_path, &mime_type);
 
-  // Add URL so that we can load supported files when dragged to TabContents.
+  // Add URL so that we can load supported files when dragged to WebContents.
   if (net::IsSupportedMimeType(mime_type)) {
-    data.SetURL(GURL(WideToUTF8(full_path.ToWStringHack())),
-                     download->GetFileNameToReportUser().ToWStringHack());
+    data.SetURL(net::FilePathToFileURL(full_path),
+                download->GetFileNameToReportUser().LossyDisplayName());
   }
 
-#if defined(OS_WIN)
+#if defined(USE_AURA)
+  // TODO(beng):
+  NOTIMPLEMENTED();
+#elif defined(OS_WIN)
   scoped_refptr<ui::DragSource> drag_source(new ui::DragSource);
 
   // Run the drag and drop loop
@@ -499,7 +407,9 @@ void DragDownload(const DownloadItem* download,
   GtkWidget* root = gtk_widget_get_toplevel(view);
   if (!root)
     return;
-  views::WidgetGtk* widget = views::WidgetGtk::GetViewForNative(root);
+
+  views::NativeWidgetGtk* widget = static_cast<views::NativeWidgetGtk*>(
+      views::Widget::GetWidgetForNativeView(root)->native_widget());
   if (!widget)
     return;
 
@@ -509,7 +419,7 @@ void DragDownload(const DownloadItem* download,
 }
 #elif defined(USE_X11)
 void DragDownload(const DownloadItem* download,
-                  SkBitmap* icon,
+                  gfx::Image* icon,
                   gfx::NativeView view) {
   DownloadItemDrag::BeginDrag(download, icon);
 }
@@ -519,59 +429,85 @@ DictionaryValue* CreateDownloadItemValue(DownloadItem* download, int id) {
   DictionaryValue* file_value = new DictionaryValue();
 
   file_value->SetInteger("started",
-      static_cast<int>(download->start_time().ToTimeT()));
+      static_cast<int>(download->GetStartTime().ToTimeT()));
   file_value->SetString("since_string",
-      TimeFormat::RelativeDate(download->start_time(), NULL));
+      TimeFormat::RelativeDate(download->GetStartTime(), NULL));
   file_value->SetString("date_string",
-      base::TimeFormatShortDate(download->start_time()));
+      base::TimeFormatShortDate(download->GetStartTime()));
   file_value->SetInteger("id", id);
-  file_value->SetString("file_path",
-      WideToUTF16Hack(download->GetTargetFilePath().ToWStringHack()));
+
+  FilePath download_path(download->GetTargetFilePath());
+  file_value->Set("file_path", base::CreateFilePathValue(download_path));
+  file_value->SetString("file_url",
+                        net::FilePathToFileURL(download_path).spec());
+
   // Keep file names as LTR.
-  string16 file_name = WideToUTF16Hack(
-      download->GetFileNameToReportUser().ToWStringHack());
+  string16 file_name = download->GetFileNameToReportUser().LossyDisplayName();
   file_name = base::i18n::GetDisplayStringInLTRDirectionality(file_name);
   file_value->SetString("file_name", file_name);
-  file_value->SetString("url", download->url().spec());
-  file_value->SetBoolean("otr", download->is_otr());
+  file_value->SetString("url", download->GetURL().spec());
+  file_value->SetBoolean("otr", download->IsOtr());
+  file_value->SetInteger("total", static_cast<int>(download->GetTotalBytes()));
+  file_value->SetBoolean("file_externally_removed",
+                         download->GetFileExternallyRemoved());
 
-  if (download->state() == DownloadItem::IN_PROGRESS) {
-    if (download->safety_state() == DownloadItem::DANGEROUS) {
+  if (download->IsInProgress()) {
+    if (download->GetSafetyState() == DownloadItem::DANGEROUS) {
       file_value->SetString("state", "DANGEROUS");
-    } else if (download->is_paused()) {
+      // These are the only danger states we expect to see (and the UI is
+      // equipped to handle):
+      DCHECK(download->GetDangerType() ==
+                 content::DOWNLOAD_DANGER_TYPE_DANGEROUS_FILE ||
+             download->GetDangerType() ==
+                 content::DOWNLOAD_DANGER_TYPE_DANGEROUS_URL ||
+             download->GetDangerType() ==
+                 content::DOWNLOAD_DANGER_TYPE_DANGEROUS_CONTENT);
+      const char* danger_type_value =
+          GetDangerTypeString(download->GetDangerType());
+      file_value->SetString("danger_type", danger_type_value);
+    } else if (download->IsPaused()) {
       file_value->SetString("state", "PAUSED");
     } else {
       file_value->SetString("state", "IN_PROGRESS");
     }
 
     file_value->SetString("progress_status_text",
-       GetProgressStatusText(download));
+        GetProgressStatusText(download));
 
     file_value->SetInteger("percent",
         static_cast<int>(download->PercentComplete()));
     file_value->SetInteger("received",
-        static_cast<int>(download->received_bytes()));
-  } else if (download->state() == DownloadItem::CANCELLED) {
-    file_value->SetString("state", "CANCELLED");
-  } else if (download->state() == DownloadItem::COMPLETE) {
-    if (download->safety_state() == DownloadItem::DANGEROUS) {
-      file_value->SetString("state", "DANGEROUS");
-    } else {
-      file_value->SetString("state", "COMPLETE");
-    }
-  }
+        static_cast<int>(download->GetReceivedBytes()));
+  } else if (download->IsInterrupted()) {
+    file_value->SetString("state", "INTERRUPTED");
 
-  file_value->SetInteger("total",
-      static_cast<int>(download->total_bytes()));
+    file_value->SetString("progress_status_text",
+        GetProgressStatusText(download));
+
+    file_value->SetInteger("percent",
+        static_cast<int>(download->PercentComplete()));
+    file_value->SetInteger("received",
+        static_cast<int>(download->GetReceivedBytes()));
+  } else if (download->IsCancelled()) {
+    file_value->SetString("state", "CANCELLED");
+  } else if (download->IsComplete()) {
+    if (download->GetSafetyState() == DownloadItem::DANGEROUS)
+      file_value->SetString("state", "DANGEROUS");
+    else
+      file_value->SetString("state", "COMPLETE");
+  } else if (download->GetState() == DownloadItem::REMOVING) {
+    file_value->SetString("state", "REMOVING");
+  } else {
+    NOTREACHED() << "state undefined";
+  }
 
   return file_value;
 }
 
 string16 GetProgressStatusText(DownloadItem* download) {
-  int64 total = download->total_bytes();
-  int64 size = download->received_bytes();
-  DataUnits amount_units = GetByteDisplayUnits(size);
-  string16 received_size = FormatBytes(size, amount_units, true);
+  int64 total = download->GetTotalBytes();
+  int64 size = download->GetReceivedBytes();
+  string16 received_size = ui::FormatBytes(size);
   string16 amount = received_size;
 
   // Adjust both strings for the locale direction since we don't yet know which
@@ -579,8 +515,7 @@ string16 GetProgressStatusText(DownloadItem* download) {
   base::i18n::AdjustStringForLocaleDirection(&amount);
 
   if (total) {
-    amount_units = GetByteDisplayUnits(total);
-    string16 total_text = FormatBytes(total, amount_units, true);
+    string16 total_text = ui::FormatBytes(total);
     base::i18n::AdjustStringForLocaleDirection(&total_text);
 
     base::i18n::AdjustStringForLocaleDirection(&received_size);
@@ -591,13 +526,12 @@ string16 GetProgressStatusText(DownloadItem* download) {
     amount.assign(received_size);
   }
   int64 current_speed = download->CurrentSpeed();
-  amount_units = GetByteDisplayUnits(current_speed);
-  string16 speed_text = FormatSpeed(current_speed, amount_units, true);
+  string16 speed_text = ui::FormatSpeed(current_speed);
   base::i18n::AdjustStringForLocaleDirection(&speed_text);
 
   base::TimeDelta remaining;
   string16 time_remaining;
-  if (download->is_paused())
+  if (download->IsPaused())
     time_remaining = l10n_util::GetStringUTF16(IDS_DOWNLOAD_PROGRESS_PAUSED);
   else if (download->TimeRemaining(&remaining))
     time_remaining = TimeFormat::TimeRemaining(remaining);
@@ -615,7 +549,9 @@ string16 GetProgressStatusText(DownloadItem* download) {
 void UpdateAppIconDownloadProgress(int download_count,
                                    bool progress_known,
                                    float progress) {
-#if defined(OS_WIN)
+#if defined(USE_AURA)
+  // TODO(davemoore) Implement once UX for download is decided <104742>
+#elif defined(OS_WIN)
   // Taskbar progress bar is only supported on Win7.
   if (base::win::GetVersion() < base::win::VERSION_WIN7)
     return;
@@ -647,135 +583,56 @@ void UpdateAppIconDownloadProgress(int download_count,
     else if (!progress_known)
       taskbar->SetProgressState(frame, TBPF_INDETERMINATE);
     else
-      taskbar->SetProgressValue(frame, (int)(progress * 100), 100);
+      taskbar->SetProgressValue(frame, static_cast<int>(progress * 100), 100);
   }
+#elif defined(TOOLKIT_GTK)
+  unity::SetDownloadCount(download_count);
+  unity::SetProgressFraction(progress);
 #endif
 }
 #endif
 
-// Appends the passed the number between parenthesis the path before the
-// extension.
-void AppendNumberToPath(FilePath* path, int number) {
-  *path = path->InsertBeforeExtensionASCII(StringPrintf(" (%d)", number));
-}
-
-// Attempts to find a number that can be appended to that path to make it
-// unique. If |path| does not exist, 0 is returned.  If it fails to find such
-// a number, -1 is returned.
-int GetUniquePathNumber(const FilePath& path) {
-  if (!file_util::PathExists(path))
-    return 0;
-
-  FilePath new_path;
-  for (int count = 1; count <= kMaxUniqueFiles; ++count) {
-    new_path = FilePath(path);
-    AppendNumberToPath(&new_path, count);
-
-    if (!file_util::PathExists(new_path))
-      return count;
-  }
-
-  return -1;
-}
-
-void DownloadUrl(
-    const GURL& url,
-    const GURL& referrer,
-    const std::string& referrer_charset,
-    const DownloadSaveInfo& save_info,
-    ResourceDispatcherHost* rdh,
-    int render_process_host_id,
-    int render_view_id,
-    URLRequestContextGetter* request_context_getter) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-
-  net::URLRequestContext* context =
-      request_context_getter->GetURLRequestContext();
-  context->set_referrer_charset(referrer_charset);
-
-  rdh->BeginDownload(url,
-                     referrer,
-                     save_info,
-                     true,  // Show "Save as" UI.
-                     render_process_host_id,
-                     render_view_id,
-                     context);
-}
-
-void CancelDownloadRequest(ResourceDispatcherHost* rdh,
-                           int render_process_id,
-                           int request_id) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  rdh->CancelRequest(render_process_id, request_id, false);
-}
-
 int GetUniquePathNumberWithCrDownload(const FilePath& path) {
-  if (!file_util::PathExists(path) &&
-      !file_util::PathExists(GetCrDownloadPath(path)))
-    return 0;
-
-  FilePath new_path;
-  for (int count = 1; count <= kMaxUniqueFiles; ++count) {
-    new_path = FilePath(path);
-    AppendNumberToPath(&new_path, count);
-
-    if (!file_util::PathExists(new_path) &&
-        !file_util::PathExists(GetCrDownloadPath(new_path)))
-      return count;
-  }
-
-  return -1;
-}
-
-namespace {
-
-// NOTE: If index is 0, deletes files that do not have the " (nnn)" appended.
-void DeleteUniqueDownloadFile(const FilePath& path, int index) {
-  FilePath new_path(path);
-  if (index > 0)
-    AppendNumberToPath(&new_path, index);
-  file_util::Delete(new_path, false);
-}
-
-}  // namespace
-
-void EraseUniqueDownloadFiles(const FilePath& path) {
-  FilePath cr_path = GetCrDownloadPath(path);
-
-  for (int index = 0; index <= kMaxUniqueFiles; ++index) {
-    DeleteUniqueDownloadFile(path, index);
-    DeleteUniqueDownloadFile(cr_path, index);
-  }
+  return DownloadFile::GetUniquePathNumberWithSuffix(
+      path, FILE_PATH_LITERAL(".crdownload"));
 }
 
 FilePath GetCrDownloadPath(const FilePath& suggested_path) {
-  FilePath::StringType file_name;
-  base::SStringPrintf(
-      &file_name,
-      PRFilePathLiteral FILE_PATH_LITERAL(".crdownload"),
-      suggested_path.value().c_str());
-  return FilePath(file_name);
+  return DownloadFile::AppendSuffixToPath(
+      suggested_path, FILE_PATH_LITERAL(".crdownload"));
 }
 
-// TODO(erikkay,phajdan.jr): This is apparently not being exercised in tests.
-bool IsDangerous(DownloadCreateInfo* info, Profile* profile) {
-  DownloadDangerLevel danger_level = GetFileDangerLevel(
-      info->suggested_path.BaseName());
-
-  if (danger_level == Dangerous) {
-    return true;
-  } else if (danger_level == AllowOnUserGesture && !info->has_user_gesture) {
-    return true;
-  } else if (info->is_extension_install) {
-    ExtensionService* service = profile->GetExtensionService();
-    if (!service ||
-        !service->IsDownloadFromGallery(info->url, info->referrer_url)) {
-      // Extensions that are not from the gallery are considered dangerous.
+bool IsSavableURL(const GURL& url) {
+  for (int i = 0; chrome::GetSavableSchemes()[i] != NULL; ++i) {
+    if (url.SchemeIs(chrome::GetSavableSchemes()[i])) {
       return true;
     }
   }
-
   return false;
 }
 
+void RecordShelfClose(int size, int in_progress, bool autoclose) {
+  static const int kMaxShelfSize = 16;
+  if (autoclose) {
+    UMA_HISTOGRAM_ENUMERATION("Download.ShelfSizeOnAutoClose",
+                              size,
+                              kMaxShelfSize);
+    UMA_HISTOGRAM_ENUMERATION("Download.ShelfInProgressSizeOnAutoClose",
+                              in_progress,
+                              kMaxShelfSize);
+  } else {
+    UMA_HISTOGRAM_ENUMERATION("Download.ShelfSizeOnUserClose",
+                              size,
+                              kMaxShelfSize);
+    UMA_HISTOGRAM_ENUMERATION("Download.ShelfInProgressSizeOnUserClose",
+                              in_progress,
+                              kMaxShelfSize);
+  }
+}
+
+void RecordDownloadCount(ChromeDownloadCountTypes type) {
+  UMA_HISTOGRAM_ENUMERATION(
+      "Download.CountsChrome", type, DOWNLOAD_COUNT_TYPES_LAST_ENTRY);
+}
+
 }  // namespace download_util
