@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -12,30 +12,29 @@
 
 #include "base/basictypes.h"
 #include "base/file_path.h"
+#include "base/memory/ref_counted.h"
 #include "base/string16.h"
-#include "gfx/native_widget_types.h"
+#include "ui/gfx/native_widget_types.h"
 
 #if defined(TOOLKIT_VIEWS)
-#include "views/view.h"
+#include "ui/views/view.h"
 #endif
 
+class GURL;
+
+namespace base {
+class DictionaryValue;
+}
+
+namespace content {
+class DownloadItem;
+}
+
 namespace gfx {
 class Canvas;
+class Image;
 }
 
-class BaseDownloadItemModel;
-class DictionaryValue;
-class DownloadItem;
-class DownloadManager;
-class GURL;
-class Profile;
-class ResourceDispatcherHost;
-class SkBitmap;
-class URLRequestContextGetter;
-
-struct DownloadCreateInfo;
-struct DownloadSaveInfo;
-
 namespace download_util {
 
 // Download temporary file creation --------------------------------------------
@@ -43,38 +42,14 @@ namespace download_util {
 // Return the default download directory.
 const FilePath& GetDefaultDownloadDirectory();
 
-// Create a temporary file for a download in the user's default download
-// directory and return true if was successful in creating the file.
-bool CreateTemporaryFileForDownload(FilePath* path);
-
 // Return true if the |download_path| is dangerous path.
 bool DownloadPathIsDangerous(const FilePath& download_path);
 
-// Create an extension based on the file name and mime type.
-void GenerateExtension(const FilePath& file_name,
-                       const std::string& mime_type,
-                       FilePath::StringType* generated_extension);
-
-// Create a file name based on the response from the server.
-void GenerateFileNameFromInfo(DownloadCreateInfo* info,
-                              FilePath* generated_name);
-
-// Create a file name based on the response from the server.
-void GenerateFileName(const GURL& url,
-                      const std::string& content_disposition,
-                      const std::string& referrer_charset,
-                      const std::string& mime_type,
-                      FilePath* generated_name);
-
-// Used to make sure we have a safe file extension and filename for a
-// download.  |file_name| can either be just the file name or it can be a
-// full path to a file.
-void GenerateSafeFileName(const std::string& mime_type, FilePath* file_name);
-
-// Opens downloaded Chrome extension file (*.crx).
-void OpenChromeExtension(Profile* profile,
-                         DownloadManager* download_manager,
-                         const DownloadItem& download_item);
+// Generate a filename based on the response from the server.  Similar
+// in operation to net::GenerateFileName(), but uses a localized
+// default name.
+void GenerateFileNameFromRequest(const content::DownloadItem& download_item,
+                                 FilePath* generated_name);
 
 // Download progress animations ------------------------------------------------
 
@@ -142,23 +117,33 @@ void PaintDownloadComplete(gfx::Canvas* canvas,
                            double animation_progress,
                            PaintDownloadProgressSize size);
 
+void PaintDownloadInterrupted(gfx::Canvas* canvas,
+#if defined(TOOLKIT_VIEWS)
+                              views::View* containing_view,
+#endif
+                              int origin_x,
+                              int origin_y,
+                              double animation_progress,
+                              PaintDownloadProgressSize size);
+
 // Drag support ----------------------------------------------------------------
 
 // Helper function for download views to use when acting as a drag source for a
 // DownloadItem. If |icon| is NULL, no image will be accompany the drag. |view|
 // is only required for Mac OS X, elsewhere it can be NULL.
-void DragDownload(const DownloadItem* download,
-                  SkBitmap* icon,
+void DragDownload(const content::DownloadItem* download,
+                  gfx::Image* icon,
                   gfx::NativeView view);
 
 // Helpers ---------------------------------------------------------------------
 
 // Creates a representation of a download in a format that the downloads
 // HTML page can understand.
-DictionaryValue* CreateDownloadItemValue(DownloadItem* download, int id);
+base::DictionaryValue* CreateDownloadItemValue(content::DownloadItem* download,
+                                               int id);
 
 // Get the localized status text for an in-progress download.
-string16 GetProgressStatusText(DownloadItem* download);
+string16 GetProgressStatusText(content::DownloadItem* download);
 
 // Update the application icon to indicate overall download progress.
 // |download_count| is the number of downloads currently in progress. If
@@ -169,46 +154,42 @@ void UpdateAppIconDownloadProgress(int download_count,
                                    bool progress_known,
                                    float progress);
 
-// Appends the passed the number between parenthesis the path before the
-// extension.
-void AppendNumberToPath(FilePath* path, int number);
-
-// Attempts to find a number that can be appended to that path to make it
-// unique. If |path| does not exist, 0 is returned.  If it fails to find such
-// a number, -1 is returned.
-int GetUniquePathNumber(const FilePath& path);
-
-// Download the URL. Must be called on the IO thread.
-void DownloadUrl(const GURL& url,
-                 const GURL& referrer,
-                 const std::string& referrer_charset,
-                 const DownloadSaveInfo& save_info,
-                 ResourceDispatcherHost* rdh,
-                 int render_process_host_id,
-                 int render_view_id,
-                 URLRequestContextGetter* request_context_getter);
-
-// Tells the resource dispatcher host to cancel a download request.
-// Must be called on the IO thread.
-void CancelDownloadRequest(ResourceDispatcherHost* rdh,
-                           int render_process_id,
-                           int request_id);
-
 // Same as GetUniquePathNumber, except that it also checks the existence
 // of its .crdownload intermediate path.
 // If |path| does not exist, 0 is returned.  If it fails to find such
 // a number, -1 is returned.
 int GetUniquePathNumberWithCrDownload(const FilePath& path);
 
-// Erases all downloaded files with the specified path and name prefix.
-// Used by download UI tests to clean up the download directory.
-void EraseUniqueDownloadFiles(const FilePath& path_prefix);
-
 // Returns a .crdownload intermediate path for the |suggested_path|.
 FilePath GetCrDownloadPath(const FilePath& suggested_path);
 
-// Whether a given download should be considered potentially dangerous.
-bool IsDangerous(DownloadCreateInfo *info, Profile* profile);
+// Check whether we can do the saving page operation for the specified URL.
+bool IsSavableURL(const GURL& url);
+
+// Record the total number of items and the number of in-progress items showing
+// in the shelf when it closes.  Set |autoclose| to true when the shelf is
+// closing itself, false when the user explicitly closed it.
+void RecordShelfClose(int size, int in_progress, bool autoclose);
+
+// Used for counting UMA stats. Similar to content's
+// download_stats::DownloadCountTypes but from the chrome layer.
+enum ChromeDownloadCountTypes {
+  // The download was initiated by navigating to a URL (e.g. by user click).
+  INITIATED_BY_NAVIGATION_COUNT = 0,
+
+  // The download was initiated by invoking a context menu within a page.
+  INITIATED_BY_CONTEXT_MENU_COUNT,
+
+  // The download was initiated by the WebStore installer.
+  INITIATED_BY_WEBSTORE_INSTALLER_COUNT,
+
+  // The download was initiated by the ImageBurner (cros).
+  INITIATED_BY_IMAGE_BURNER_COUNT,
+
+  DOWNLOAD_COUNT_TYPES_LAST_ENTRY,
+};
+
+void RecordDownloadCount(ChromeDownloadCountTypes type);
 
 }  // namespace download_util
 
