@@ -4,16 +4,17 @@
 
 #include "chrome/browser/enumerate_modules_model_win.h"
 
+#include <algorithm>
 #include <Tlhelp32.h>
 #include <wintrust.h>
 
-#include "app/win/win_util.h"
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/environment.h"
 #include "base/file_path.h"
 #include "base/file_version_info_win.h"
+#include "base/i18n/case_conversion.h"
 #include "base/metrics/histogram.h"
-#include "base/sha2.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
 #include "base/time.h"
@@ -22,13 +23,17 @@
 #include "base/version.h"
 #include "base/win/registry.h"
 #include "base/win/scoped_handle.h"
+#include "crypto/sha2.h"
 #include "chrome/browser/net/service_providers_win.h"
 #include "chrome/common/chrome_constants.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/notification_service.h"
+#include "content/public/browser/notification_service.h"
 #include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util.h"
 
+using content::BrowserThread;
+
 // The period of time (in milliseconds) to wait until checking to see if any
 // incompatible modules exist.
 static const int kModuleCheckDelayMs = 60 * 1000;
@@ -42,6 +47,11 @@ static const ModuleEnumerator::RecommendedAction kUninstallLink =
     static_cast<ModuleEnumerator::RecommendedAction>(
         ModuleEnumerator::UNINSTALL | ModuleEnumerator::SEE_LINK);
 
+// Short-hand for things on the blacklist we are investigating and have info.
+static const ModuleEnumerator::RecommendedAction kInvestigatingLink =
+    static_cast<ModuleEnumerator::RecommendedAction>(
+        ModuleEnumerator::INVESTIGATING | ModuleEnumerator::SEE_LINK);
+
 // A sort method that sorts by bad modules first, then by full name (including
 // path).
 static bool ModuleSort(const ModuleEnumerator::Module& a,
@@ -75,8 +85,23 @@ struct FindModule {
   const ModuleEnumerator::Module& module;
 };
 
+// Returns the long path name given a short path name. A short path name is a
+// path that follows the 8.3 convention and has ~x in it. If the path is already
+// a long path name, the function returns the current path without modification.
+bool ConvertToLongPath(const string16& short_path, string16* long_path) {
+  wchar_t long_path_buf[MAX_PATH];
+  DWORD return_value = GetLongPathName(short_path.c_str(), long_path_buf,
+                                       MAX_PATH);
+  if (return_value != 0 && return_value < MAX_PATH) {
+    *long_path = long_path_buf;
+    return true;
+  }
+
+  return false;
 }
 
+}  // namespace
+
 // The browser process module blacklist. This lists modules that are known
 // to cause compatibility issues within the browser process. When adding to this
 // list, make sure that all paths are lower-case, in long pathname form, end
@@ -90,6 +115,11 @@ struct FindModule {
 const ModuleEnumerator::BlacklistEntry ModuleEnumerator::kModuleBlacklist[] = {
   // NOTE: Please keep this list sorted by dll name, then location.
 
+  // Version 3.2.1.6 seems to be implicated in most cases (and 3.2.2.2 in some).
+  // There is a more recent version available for download.
+  // accelerator.dll, "%programfiles%\\speedbit video accelerator\\".
+  { "7ba9402f", "c9132d48", "", "", "", kInvestigatingLink },
+
   // apiqq0.dll, "%temp%\\".
   { "26134911", "59145acf", "", "", "", kUninstallLink },
 
@@ -99,6 +129,11 @@ const ModuleEnumerator::BlacklistEntry ModuleEnumerator::kModuleBlacklist[] = {
   // arking1.dll, "%systemroot%\\system32\\".
   { "c60ca062", "23d01d5b", "", "", "", kUninstallLink },
 
+  // Said to belong to Killer NIC from BigFoot Networks (not verified). Versions
+  // 6.0.0.7 and 6.0.0.10 implicated.
+  // bfllr.dll, "%systemroot%\\system32\\".
+  { "6bb57633", "23d01d5b", "", "", "", kInvestigatingLink },
+
   // clickpotatolitesahook.dll, "". Different version each report.
   { "0396e037.dll", "", "", "", "", kUninstallLink },
 
@@ -108,9 +143,20 @@ const ModuleEnumerator::BlacklistEntry ModuleEnumerator::kModuleBlacklist[] = {
   // cwalsp.dll, "%systemroot%\\system32\\".
   { "e579a039", "23d01d5b", "", "", "", kUninstallLink },
 
+  // datamngr.dll, "%programfiles%\\windows searchqu toolbar\\".
+  { "7add320b", "7a3c8be3", "", "", "", kUninstallLink },
+
   // dsoqq0.dll, "%temp%\\".
   { "1c4df325", "59145acf", "", "", "", kUninstallLink },
 
+  // flt.dll, "%programfiles%\\tueagles\\".
+  { "6d01f4a1", "7935e9c2", "", "", "", kUninstallLink },
+
+  // This looks like a malware edition of a Brazilian Bank plugin, sometimes
+  // referred to as Malware.Banc.A.
+  // gbieh.dll, "%programfiles%\\gbplugin\\".
+  { "4cb4f2e3", "88e4a3b1", "", "", "", kUninstallLink },
+
   // hblitesahook.dll. Each report has different version number in location.
   { "5d10b363", "", "", "", "", kUninstallLink },
 
@@ -156,10 +202,25 @@ const ModuleEnumerator::BlacklistEntry ModuleEnumerator::kModuleBlacklist[] = {
   // nodqq0.dll, "%temp%\\".
   { "b86ce04d", "59145acf", "", "", "", kUninstallLink },
 
+  // nProtect GameGuard Anti-cheat system. Every report has a different
+  // location, since it is installed into and run from a game folder. Various
+  // versions implicated.
+  // npggnt.des, no fixed location.
+  { "f2c8790d", "", "", "", "", kInvestigatingLink },
+
   // nvlsp.dll,
   // "%programfiles%\\nvidia corporation\\networkaccessmanager\\bin32\\".
   { "37f907e2", "3ad0ff23", "", "", "", INVESTIGATING },
 
+  // post0.dll, "%systemroot%\\system32\\".
+  { "7405c0c8", "23d01d5b", "", "", "", kUninstallLink },
+
+  // questbrwsearch.dll, "%programfiles%\\questbrwsearch\\".
+  { "0953ed09", "f0d5eeda", "", "", "", kUninstallLink },
+
+  // questscan.dll, "%programfiles%\\questscan\\".
+  { "f4f3391e", "119d20f7", "", "", "", kUninstallLink },
+
   // radhslib.dll (Naomi web filter), "%programfiles%\\rnamfler\\".
   // See http://crbug.com/12517.
   { "7edcd250", "0733dc3e", "", "", "", INVESTIGATING },
@@ -168,7 +229,13 @@ const ModuleEnumerator::BlacklistEntry ModuleEnumerator::kModuleBlacklist[] = {
   { "a1ed94a7", "ea9d6b36", "", "", "", kUninstallLink },
 
   // rooksdol.dll, "%programfiles%\\trusteer\\rapport\\bin\\".
-  { "802aefef", "06120e13", "", "", "", INVESTIGATING },
+  { "802aefef", "06120e13", "", "", "3.5.1008.40", UPDATE },
+
+  // scanquery.dll, "%programfiles%\\scanquery\\".
+  { "0b52d2ae", "a4cc88b1", "", "", "", kUninstallLink },
+
+  // sdata.dll, "%programdata%\\srtserv\\".
+  { "1936d5cc", "223c44be", "", "", "", kUninstallLink },
 
   // searchtree.dll,
   // "%programfiles%\\contentwatch\\internet protection\\modules\\".
@@ -177,6 +244,11 @@ const ModuleEnumerator::BlacklistEntry ModuleEnumerator::kModuleBlacklist[] = {
   // sgprxy.dll, "%commonprogramfiles%\\is3\\anti-spyware\\".
   { "005965ea", "bc5673f2", "", "", "", INVESTIGATING },
 
+  // swi_filter_0001.dll (Sophos Web Intelligence),
+  // "%programfiles%\\sophos\\sophos anti-virus\\web intelligence\\".
+  // A small random sample all showed version 1.0.5.0.
+  { "61112d7b", "25fb120f", "", "", "", kInvestigatingLink },
+
   // twking0.dll, "%systemroot%\\system32\\".
   { "0355549b", "23d01d5b", "", "", "", kUninstallLink },
 
@@ -211,7 +283,7 @@ static void GenerateHash(const std::string& input, std::string* output) {
   }
 
   uint8 hash[4];
-  base::SHA256HashString(input, hash, sizeof(hash));
+  crypto::SHA256HashString(input, hash, sizeof(hash));
   *output = StringToLowerASCII(base::HexEncode(hash, sizeof(hash)));
 }
 
@@ -220,10 +292,10 @@ static void GenerateHash(const std::string& input, std::string* output) {
 // static
 void ModuleEnumerator::NormalizeModule(Module* module) {
   string16 path = module->location;
-  if (!app::win::ConvertToLongPath(path, &module->location))
+  if (!ConvertToLongPath(path, &module->location))
     module->location = path;
 
-  module->location = l10n_util::ToLower(module->location);
+  module->location = base::i18n::ToLower(module->location);
 
   // Location contains the filename, so the last slash is where the path
   // ends.
@@ -312,7 +384,8 @@ ModuleEnumerator::ModuleStatus ModuleEnumerator::Match(
 }
 
 ModuleEnumerator::ModuleEnumerator(EnumerateModulesModel* observer)
-    : observer_(observer),
+    : enumerated_modules_(NULL),
+      observer_(observer),
       limited_mode_(false),
       callback_thread_id_(BrowserThread::ID_COUNT) {
 }
@@ -328,9 +401,8 @@ void ModuleEnumerator::ScanNow(ModulesVector* list, bool limited_mode) {
   if (!limited_mode_) {
     CHECK(BrowserThread::GetCurrentThreadIdentifier(&callback_thread_id_));
     DCHECK(!BrowserThread::CurrentlyOn(BrowserThread::FILE));
-    BrowserThread::PostTask(
-        BrowserThread::FILE, FROM_HERE,
-            NewRunnableMethod(this, &ModuleEnumerator::ScanImpl));
+    BrowserThread::PostTask(BrowserThread::FILE, FROM_HERE,
+                            base::Bind(&ModuleEnumerator::ScanImpl, this));
   } else {
     // Run it synchronously.
     ScanImpl();
@@ -372,9 +444,8 @@ void ModuleEnumerator::ScanImpl() {
 
   if (!limited_mode_) {
     // Send a reply back on the UI thread.
-    BrowserThread::PostTask(
-        callback_thread_id_, FROM_HERE,
-        NewRunnableMethod(this, &ModuleEnumerator::ReportBack));
+    BrowserThread::PostTask(callback_thread_id_, FROM_HERE,
+                            base::Bind(&ModuleEnumerator::ReportBack, this));
   } else {
     // We are on the main thread already.
     ReportBack();
@@ -523,6 +594,7 @@ void ModuleEnumerator::PreparePathMappings() {
   std::vector<string16> env_vars;
   env_vars.push_back(L"LOCALAPPDATA");
   env_vars.push_back(L"ProgramFiles");
+  env_vars.push_back(L"ProgramData");
   env_vars.push_back(L"USERPROFILE");
   env_vars.push_back(L"SystemRoot");
   env_vars.push_back(L"TEMP");
@@ -533,8 +605,8 @@ void ModuleEnumerator::PreparePathMappings() {
     std::string path;
     if (environment->GetVar(WideToASCII(*variable).c_str(), &path)) {
       path_mapping_.push_back(
-          std::make_pair(l10n_util::ToLower(UTF8ToWide(path)) + L"\\",
-                         L"%" + l10n_util::ToLower(*variable) + L"%"));
+          std::make_pair(base::i18n::ToLower(UTF8ToUTF16(path)) + L"\\",
+                         L"%" + base::i18n::ToLower(*variable) + L"%"));
     }
   }
 }
@@ -701,6 +773,27 @@ EnumerateModulesModel* EnumerateModulesModel::GetInstance() {
   return Singleton<EnumerateModulesModel>::get();
 }
 
+bool EnumerateModulesModel::ShouldShowConflictWarning() const {
+  // If the user has acknowledged the conflict notification, then we don't need
+  // to show it again (because the scanning only happens once per the lifetime
+  // of the process). If we were to run the scanning more than once, then we'd
+  // need to clear the flag somewhere when we are ready to show it again.
+  if (conflict_notification_acknowledged_)
+    return false;
+
+  return confirmed_bad_modules_detected_ > 0;
+}
+
+void EnumerateModulesModel::AcknowledgeConflictNotification() {
+  if (!conflict_notification_acknowledged_) {
+    conflict_notification_acknowledged_ = true;
+    content::NotificationService::current()->Notify(
+        chrome::NOTIFICATION_MODULE_INCOMPATIBILITY_BADGE_CHANGE,
+        content::Source<EnumerateModulesModel>(this),
+        content::NotificationService::NoDetails());
+  }
+}
+
 void EnumerateModulesModel::ScanNow() {
   if (scanning_)
     return;  // A scan is already in progress.
@@ -716,7 +809,7 @@ void EnumerateModulesModel::ScanNow() {
   module_enumerator_->ScanNow(&enumerated_modules_, limited_mode_);
 }
 
-ListValue* EnumerateModulesModel::GetModuleList() {
+ListValue* EnumerateModulesModel::GetModuleList() const {
   if (scanning_)
     return NULL;
 
@@ -806,13 +899,14 @@ ListValue* EnumerateModulesModel::GetModuleList() {
 }
 
 EnumerateModulesModel::EnumerateModulesModel()
-    : scanning_(false),
-      limited_mode_(false),
+    : limited_mode_(false),
+      scanning_(false),
+      conflict_notification_acknowledged_(false),
       confirmed_bad_modules_detected_(0),
       suspected_bad_modules_detected_(0) {
   const CommandLine& cmd_line = *CommandLine::ForCurrentProcess();
   if (cmd_line.HasSwitch(switches::kConflictingModulesCheck)) {
-    check_modules_timer_.Start(
+    check_modules_timer_.Start(FROM_HERE,
         base::TimeDelta::FromMilliseconds(kModuleCheckDelayMs),
         this, &EnumerateModulesModel::ScanNow);
   }
@@ -848,10 +942,10 @@ void EnumerateModulesModel::DoneScanning() {
   if (limited_mode_)
     return;
 
-  NotificationService::current()->Notify(
-      NotificationType::MODULE_LIST_ENUMERATED,
-      Source<EnumerateModulesModel>(this),
-      NotificationService::NoDetails());
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_MODULE_LIST_ENUMERATED,
+      content::Source<EnumerateModulesModel>(this),
+      content::NotificationService::NoDetails());
 
   // Command line flag must be enabled for the notification to get sent out.
   // Otherwise we'd get the badge (while the feature is disabled) when we
@@ -860,17 +954,14 @@ void EnumerateModulesModel::DoneScanning() {
   if (!cmd_line.HasSwitch(switches::kConflictingModulesCheck))
     return;
 
-  if (suspected_bad_modules_detected_ || confirmed_bad_modules_detected_) {
-    bool found_confirmed_bad_modules = confirmed_bad_modules_detected_  > 0;
-    NotificationService::current()->Notify(
-        NotificationType::MODULE_INCOMPATIBILITY_DETECTED,
-        Source<EnumerateModulesModel>(this),
-        Details<bool>(&found_confirmed_bad_modules));
-  }
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_MODULE_INCOMPATIBILITY_BADGE_CHANGE,
+      content::Source<EnumerateModulesModel>(this),
+      content::NotificationService::NoDetails());
 }
 
 GURL EnumerateModulesModel::ConstructHelpCenterUrl(
-    const ModuleEnumerator::Module& module) {
+    const ModuleEnumerator::Module& module) const {
   if (!(module.recommended_action & ModuleEnumerator::SEE_LINK))
     return GURL();
 
