@@ -4,12 +4,15 @@
 
 #include "chrome/browser/extensions/autoupdate_interceptor.h"
 
+#include "base/bind.h"
 #include "base/file_util.h"
 #include "base/threading/thread_restrictions.h"
-#include "chrome/browser/browser_thread.h"
+#include "content/public/browser/browser_thread.h"
 #include "net/url_request/url_request_test_job.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using content::BrowserThread;
+
 // This is a specialized version of net::URLRequestTestJob that lets us specify
 // response data and make sure the response code is 200, which the autoupdate
 // code relies on.
@@ -31,11 +34,11 @@ class AutoUpdateTestRequestJob : public net::URLRequestTestJob {
 
 
 AutoUpdateInterceptor::AutoUpdateInterceptor() {
-  net::URLRequest::RegisterRequestInterceptor(this);
+  net::URLRequest::Deprecated::RegisterRequestInterceptor(this);
 }
 
 AutoUpdateInterceptor::~AutoUpdateInterceptor() {
-  net::URLRequest::UnregisterRequestInterceptor(this);
+  net::URLRequest::Deprecated::UnregisterRequestInterceptor(this);
 }
 
 net::URLRequestJob* AutoUpdateInterceptor::MaybeIntercept(
@@ -86,5 +89,5 @@ void AutoUpdateInterceptor::SetResponseOnIOThread(const std::string url,
                                                   const FilePath& path) {
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(this, &AutoUpdateInterceptor::SetResponse, url, path));
+      base::Bind(&AutoUpdateInterceptor::SetResponse, this, url, path));
 }
