@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,8 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/string16.h"
 
 class Extension;
 class FilePath;
@@ -22,6 +23,6 @@ class GURL;
 // NOTE: The caller takes ownership of the directory at extension->path() on the
 // returned object.
 scoped_refptr<Extension> ConvertUserScriptToExtension(
-    const FilePath& user_script, const GURL& original_url, std::string* error);
+    const FilePath& user_script, const GURL& original_url, string16* error);
 
 #endif  // CHROME_BROWSER_EXTENSIONS_CONVERT_USER_SCRIPT_H_
