@@ -1,24 +1,32 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/execute_code_in_tab_function.h"
 
-#include "base/callback.h"
+#include "base/bind.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/extensions/extension_tabs_module.h"
 #include "chrome/browser/extensions/extension_tabs_module_constants.h"
-#include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/file_reader.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/extensions/extension_error_utils.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/common/extensions/extension_file_util.h"
+#include "chrome/common/extensions/extension_l10n_util.h"
+#include "chrome/common/extensions/extension_message_bundle.h"
+#include "chrome/common/extensions/extension_messages.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/web_contents.h"
+
+using content::BrowserThread;
 
 namespace keys = extension_tabs_module_constants;
 
@@ -74,13 +82,12 @@ bool ExecuteCodeInTabFunction::RunImpl() {
     }
   }
 
-  DCHECK(browser);
-  DCHECK(contents);
-
   // NOTE: This can give the wrong answer due to race conditions, but it is OK,
   // we check again in the renderer.
+  CHECK(browser);
+  CHECK(contents);
   if (!GetExtension()->CanExecuteScriptOnPage(
-          contents->tab_contents()->GetURL(), NULL, &error_)) {
+          contents->web_contents()->GetURL(), NULL, &error_)) {
     return false;
   }
 
@@ -113,15 +120,60 @@ bool ExecuteCodeInTabFunction::RunImpl() {
   }
 
   scoped_refptr<FileReader> file_reader(new FileReader(
-      resource_, NewCallback(this, &ExecuteCodeInTabFunction::DidLoadFile)));
+      resource_, base::Bind(&ExecuteCodeInTabFunction::DidLoadFile, this)));
   file_reader->Start();
-  AddRef();  // Keep us alive until DidLoadFile is called.
 
   return true;
 }
 
 void ExecuteCodeInTabFunction::DidLoadFile(bool success,
                                            const std::string& data) {
+  std::string function_name = name();
+  const Extension* extension = GetExtension();
+
+  // Check if the file is CSS and needs localization.
+  if (success &&
+      function_name == TabsInsertCSSFunction::function_name() &&
+      extension != NULL &&
+      data.find(ExtensionMessageBundle::kMessageBegin) != std::string::npos) {
+    BrowserThread::PostTask(
+        BrowserThread::FILE, FROM_HERE,
+        base::Bind(&ExecuteCodeInTabFunction::LocalizeCSS, this,
+                   data,
+                   extension->id(),
+                   extension->path(),
+                   extension->default_locale()));
+  } else {
+    DidLoadAndLocalizeFile(success, data);
+  }
+}
+
+void ExecuteCodeInTabFunction::LocalizeCSS(
+    const std::string& data,
+    const std::string& extension_id,
+    const FilePath& extension_path,
+    const std::string& extension_default_locale) {
+  scoped_ptr<SubstitutionMap> localization_messages(
+      extension_file_util::LoadExtensionMessageBundleSubstitutionMap(
+          extension_path, extension_id, extension_default_locale));
+
+  // We need to do message replacement on the data, so it has to be mutable.
+  std::string css_data = data;
+  std::string error;
+  ExtensionMessageBundle::ReplaceMessagesWithExternalDictionary(
+      *localization_messages, &css_data, &error);
+
+  // Call back DidLoadAndLocalizeFile on the UI thread. The success parameter
+  // is always true, because if loading had failed, we wouldn't have had
+  // anything to localize.
+  BrowserThread::PostTask(
+      BrowserThread::UI, FROM_HERE,
+      base::Bind(&ExecuteCodeInTabFunction::DidLoadAndLocalizeFile, this,
+                 true, css_data));
+}
+
+void ExecuteCodeInTabFunction::DidLoadAndLocalizeFile(bool success,
+                                                      const std::string& data) {
   if (success) {
     Execute(data);
   } else {
@@ -136,7 +188,6 @@ void ExecuteCodeInTabFunction::DidLoadFile(bool success,
 #endif  // OS_WIN
     SendResponse(false);
   }
-  Release();  // Balance the AddRef taken in RunImpl
 }
 
 bool ExecuteCodeInTabFunction::Execute(const std::string& code_string) {
@@ -165,24 +216,54 @@ bool ExecuteCodeInTabFunction::Execute(const std::string& code_string) {
   } else if (function_name != TabsExecuteScriptFunction::function_name()) {
     DCHECK(false);
   }
-  if (!contents->tab_contents()->ExecuteCode(request_id(), extension->id(),
-      is_js_code, code_string, all_frames_)) {
-    SendResponse(false);
-    return false;
-  }
-  registrar_.Add(this, NotificationType::TAB_CODE_EXECUTED,
-                 NotificationService::AllSources());
-  AddRef();  // balanced in Observe()
+
+  ExtensionMsg_ExecuteCode_Params params;
+  params.request_id = request_id();
+  params.extension_id = extension->id();
+  params.is_javascript = is_js_code;
+  params.code = code_string;
+  params.all_frames = all_frames_;
+  params.in_main_world = false;
+  contents->web_contents()->GetRenderViewHost()->Send(
+      new ExtensionMsg_ExecuteCode(
+          contents->web_contents()->GetRenderViewHost()->routing_id(), params));
+
+  Observe(contents->web_contents());
+  AddRef();  // balanced in OnExecuteCodeFinished()
   return true;
 }
 
-void ExecuteCodeInTabFunction::Observe(NotificationType type,
-                                       const NotificationSource& source,
-                                       const NotificationDetails& details) {
-  std::pair<int, bool>* result_details =
-      Details<std::pair<int, bool> >(details).ptr();
-  if (result_details->first == request_id()) {
-    SendResponse(result_details->second);
-    Release();  // balanced in Execute()
+bool ExecuteCodeInTabFunction::OnMessageReceived(const IPC::Message& message) {
+  if (message.type() != ExtensionHostMsg_ExecuteCodeFinished::ID)
+    return false;
+
+  int message_request_id;
+  void* iter = NULL;
+  if (!message.ReadInt(&iter, &message_request_id)) {
+    NOTREACHED() << "malformed extension message";
+    return true;
   }
+
+  if (message_request_id != request_id())
+    return false;
+
+  IPC_BEGIN_MESSAGE_MAP(ExecuteCodeInTabFunction, message)
+    IPC_MESSAGE_HANDLER(ExtensionHostMsg_ExecuteCodeFinished,
+                        OnExecuteCodeFinished)
+  IPC_END_MESSAGE_MAP()
+  return true;
+}
+
+void ExecuteCodeInTabFunction::OnExecuteCodeFinished(int request_id,
+                                                     bool success,
+                                                     const std::string& error) {
+  if (!error.empty()) {
+    CHECK(!success);
+    error_ = error;
+  }
+
+  SendResponse(success);
+
+  Observe(NULL);
+  Release();  // balanced in Execute()
 }
