@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,26 +6,34 @@
 
 #include "base/string_util.h"
 #include "base/stringprintf.h"
-#include "chrome/browser/extensions/extension_test_api.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_test_api.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/browser/web_applications/web_app.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_service.h"
 
 namespace {
 
 const char kTestServerPort[] = "testServer.port";
+const char kTestDataDirectory[] = "testDataDirectory";
 
 };  // namespace
 
+ExtensionApiTest::ExtensionApiTest() {}
+
+ExtensionApiTest::~ExtensionApiTest() {}
+
 ExtensionApiTest::ResultCatcher::ResultCatcher()
     : profile_restriction_(NULL),
       waiting_(false) {
-  registrar_.Add(this, NotificationType::EXTENSION_TEST_PASSED,
-                 NotificationService::AllSources());
-  registrar_.Add(this, NotificationType::EXTENSION_TEST_FAILED,
-                 NotificationService::AllSources());
+  registrar_.Add(this, chrome::NOTIFICATION_EXTENSION_TEST_PASSED,
+                 content::NotificationService::AllSources());
+  registrar_.Add(this, chrome::NOTIFICATION_EXTENSION_TEST_FAILED,
+                 content::NotificationService::AllSources());
 }
 
 ExtensionApiTest::ResultCatcher::~ResultCatcher() {
@@ -55,15 +63,15 @@ bool ExtensionApiTest::ResultCatcher::GetNextResult() {
 }
 
 void ExtensionApiTest::ResultCatcher::Observe(
-    NotificationType type, const NotificationSource& source,
-    const NotificationDetails& details) {
+    int type, const content::NotificationSource& source,
+    const content::NotificationDetails& details) {
   if (profile_restriction_ &&
-      Source<Profile>(source).ptr() != profile_restriction_) {
+      content::Source<Profile>(source).ptr() != profile_restriction_) {
     return;
   }
 
-  switch (type.value) {
-    case NotificationType::EXTENSION_TEST_PASSED:
+  switch (type) {
+    case chrome::NOTIFICATION_EXTENSION_TEST_PASSED:
       VLOG(1) << "Got EXTENSION_TEST_PASSED notification.";
       results_.push_back(true);
       messages_.push_back("");
@@ -71,10 +79,10 @@ void ExtensionApiTest::ResultCatcher::Observe(
         MessageLoopForUI::current()->Quit();
       break;
 
-    case NotificationType::EXTENSION_TEST_FAILED:
+    case chrome::NOTIFICATION_EXTENSION_TEST_FAILED:
       VLOG(1) << "Got EXTENSION_TEST_FAILED notification.";
       results_.push_back(false);
-      messages_.push_back(*(Details<std::string>(details).ptr()));
+      messages_.push_back(*(content::Details<std::string>(details).ptr()));
       if (waiting_)
         MessageLoopForUI::current()->Quit();
       break;
@@ -87,6 +95,8 @@ void ExtensionApiTest::ResultCatcher::Observe(
 void ExtensionApiTest::SetUpInProcessBrowserTestFixture() {
   DCHECK(!test_config_.get()) << "Previous test did not clear config state.";
   test_config_.reset(new DictionaryValue());
+  test_config_->SetString(kTestDataDirectory,
+                          net::FilePathToFileURL(test_data_dir_).spec());
   ExtensionTestGetConfigFunction::set_test_config_state(test_config_.get());
 }
 
@@ -96,37 +106,89 @@ void ExtensionApiTest::TearDownInProcessBrowserTestFixture() {
 }
 
 bool ExtensionApiTest::RunExtensionTest(const char* extension_name) {
-  return RunExtensionTestImpl(extension_name, "", false);
+  return RunExtensionTestImpl(extension_name, "", kFlagEnableFileAccess);
 }
 
 bool ExtensionApiTest::RunExtensionTestIncognito(const char* extension_name) {
-  return RunExtensionTestImpl(extension_name, "", true);
+  return RunExtensionTestImpl(
+      extension_name, "", kFlagEnableIncognito | kFlagEnableFileAccess);
+}
+
+bool ExtensionApiTest::RunComponentExtensionTest(const char* extension_name) {
+  return RunExtensionTestImpl(
+      extension_name, "", kFlagEnableFileAccess | kFlagLoadAsComponent);
+}
+
+bool ExtensionApiTest::RunExtensionTestNoFileAccess(
+    const char* extension_name) {
+  return RunExtensionTestImpl(extension_name, "", kFlagNone);
+}
+
+bool ExtensionApiTest::RunExtensionTestIncognitoNoFileAccess(
+    const char* extension_name) {
+  return RunExtensionTestImpl(extension_name, "", kFlagEnableIncognito);
 }
 
 bool ExtensionApiTest::RunExtensionSubtest(const char* extension_name,
                                            const std::string& page_url) {
   DCHECK(!page_url.empty()) << "Argument page_url is required.";
-  return RunExtensionTestImpl(extension_name, page_url, false);
+  return RunExtensionTestImpl(extension_name, page_url, kFlagEnableFileAccess);
+}
+
+bool ExtensionApiTest::RunExtensionSubtestNoFileAccess(
+    const char* extension_name,
+    const std::string& page_url) {
+  DCHECK(!page_url.empty()) << "Argument page_url is required.";
+  return RunExtensionTestImpl(extension_name, page_url, kFlagNone);
+}
+
+bool ExtensionApiTest::RunExtensionSubtestIncognito(
+    const char* extension_name,
+    const std::string& page_url) {
+  DCHECK(!page_url.empty()) << "Argument page_url is required.";
+  return RunExtensionTestImpl(extension_name, page_url,
+                              kFlagEnableIncognito | kFlagEnableFileAccess);
+}
+
+bool ExtensionApiTest::RunExtensionSubtestIncognitoNoFileAccess(
+    const char* extension_name,
+    const std::string& page_url) {
+  DCHECK(!page_url.empty()) << "Argument page_url is required.";
+  return RunExtensionTestImpl(extension_name, page_url, kFlagEnableIncognito);
 }
 
 bool ExtensionApiTest::RunPageTest(const std::string& page_url) {
   return RunExtensionSubtest("", page_url);
 }
 
+bool ExtensionApiTest::RunPlatformAppTest(const char* extension_name) {
+  return RunExtensionTestImpl(extension_name, "", kFlagLaunchAppShell);
+}
+
 // Load |extension_name| extension and/or |page_url| and wait for
 // PASSED or FAILED notification.
 bool ExtensionApiTest::RunExtensionTestImpl(const char* extension_name,
                                             const std::string& page_url,
-                                            bool enable_incognito) {
+                                            int flags) {
+  bool enable_incognito = (flags & kFlagEnableIncognito) != 0;
+  bool enable_fileaccess = (flags & kFlagEnableFileAccess) != 0;
+  bool load_as_component = (flags & kFlagLoadAsComponent) != 0;
+  bool launch_shell = (flags & kFlagLaunchAppShell) != 0;
+
   ResultCatcher catcher;
   DCHECK(!std::string(extension_name).empty() || !page_url.empty()) <<
       "extension_name and page_url cannot both be empty";
 
+  const Extension* extension = NULL;
   if (!std::string(extension_name).empty()) {
-    bool loaded = enable_incognito ?
-        LoadExtensionIncognito(test_data_dir_.AppendASCII(extension_name)) :
-        LoadExtension(test_data_dir_.AppendASCII(extension_name));
-    if (!loaded) {
+    FilePath extension_path = test_data_dir_.AppendASCII(extension_name);
+    if (load_as_component) {
+      extension = LoadExtensionAsComponent(extension_path);
+    } else {
+      extension = LoadExtensionWithOptions(extension_path,
+                                           enable_incognito, enable_fileaccess);
+    }
+    if (!extension) {
       message_ = "Failed to load extension.";
       return false;
     }
@@ -143,16 +205,19 @@ bool ExtensionApiTest::RunExtensionTestImpl(const char* extension_name,
       DCHECK(!std::string(extension_name).empty()) <<
           "Relative page_url given with no extension_name";
 
-      ExtensionService* service = browser()->profile()->GetExtensionService();
-      const Extension* extension =
-          service->GetExtensionById(last_loaded_extension_id_, false);
-      if (!extension)
-        return false;
-
       url = extension->GetResourceURL(page_url);
     }
 
     ui_test_utils::NavigateToURL(browser(), url);
+
+  } else if (launch_shell) {
+    web_app::SetDisableShortcutCreationForTests(true);
+    Browser::OpenApplication(
+        browser()->profile(),
+        extension,
+        extension_misc::LAUNCH_SHELL,
+        GURL(),
+        NEW_WINDOW);
   }
 
   if (!catcher.GetNextResult()) {
@@ -163,28 +228,29 @@ bool ExtensionApiTest::RunExtensionTestImpl(const char* extension_name,
   }
 }
 
-// Test that exactly one extension loaded.
+// Test that exactly one extension is loaded, and return it.
 const Extension* ExtensionApiTest::GetSingleLoadedExtension() {
   ExtensionService* service = browser()->profile()->GetExtensionService();
 
-  int found_extension_index = -1;
-  for (size_t i = 0; i < service->extensions()->size(); ++i) {
+  const Extension* extension = NULL;
+  for (ExtensionSet::const_iterator it = service->extensions()->begin();
+       it != service->extensions()->end(); ++it) {
     // Ignore any component extensions. They are automatically loaded into all
     // profiles and aren't the extension we're looking for here.
-    if (service->extensions()->at(i)->location() == Extension::COMPONENT)
+    if ((*it)->location() == Extension::COMPONENT)
       continue;
 
-    if (found_extension_index != -1) {
+    if (extension != NULL) {
+      // TODO(yoz): this is misleading; it counts component extensions.
       message_ = base::StringPrintf(
           "Expected only one extension to be present.  Found %u.",
           static_cast<unsigned>(service->extensions()->size()));
       return NULL;
     }
 
-    found_extension_index = static_cast<int>(i);
+    extension = *it;
   }
 
-  const Extension* extension = service->extensions()->at(found_extension_index);
   if (!extension) {
     message_ = "extension pointer is NULL.";
     return NULL;
@@ -197,8 +263,8 @@ bool ExtensionApiTest::StartTestServer() {
     return false;
 
   // Build a dictionary of values that tests can use to build URLs that
-  // access the test server.  Tests can see these values using the extension
-  // API function chrome.test.getConfig().
+  // access the test server and local file system.  Tests can see these values
+  // using the extension API function chrome.test.getConfig().
   test_config_->SetInteger(kTestServerPort,
                            test_server()->host_port_pair().port());
 
