@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,11 @@
 #include <string>
 
 #include "base/values.h"
-#include "chrome/browser/browser_list.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/extensions/extension.h"
-#include "chrome/common/notification_service.h"
 #include "chrome/common/render_messages.h"
+#include "content/public/browser/notification_service.h"
 
 namespace {
 // Errors.
@@ -37,15 +38,15 @@ bool BrowserActionFunction::RunImpl() {
   if (!RunBrowserAction())
     return false;
 
-  NotificationService::current()->Notify(
-      NotificationType::EXTENSION_BROWSER_ACTION_UPDATED,
-      Source<ExtensionAction>(browser_action_),
-      NotificationService::NoDetails());
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_EXTENSION_BROWSER_ACTION_UPDATED,
+      content::Source<ExtensionAction>(browser_action_),
+      content::NotificationService::NoDetails());
   return true;
 }
 
 bool BrowserActionSetIconFunction::RunBrowserAction() {
-  BinaryValue* binary = NULL;
+  base::BinaryValue* binary = NULL;
   EXTENSION_FUNCTION_VALIDATE(details_->GetBinary("imageData", &binary));
   IPC::Message bitmap_pickle(binary->GetBuffer(), binary->GetSize());
   void* iter = NULL;
