@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,17 +10,21 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "chrome/browser/browser_list.h"
+#include "base/compiler_specific.h"
 #include "chrome/browser/extensions/extension_tabs_module.h"
 #include "chrome/browser/tabs/tab_strip_model_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "content/public/browser/notification_registrar.h"
 #if defined(TOOLKIT_VIEWS)
-#include "views/view.h"
-#include "views/focus/focus_manager.h"
+#include "ui/views/focus/widget_focus_manager.h"
 #elif defined(TOOLKIT_GTK)
-#include "ui/base/x/active_window_watcher_x.h"
+#include "ui/base/x/active_window_watcher_x_observer.h"
 #endif
 
+namespace content {
+class WebContents;
+}
+
 // The ExtensionBrowserEventRouter listens to Browser window & tab events
 // and routes them to listeners inside extension process renderers.
 // ExtensionBrowserEventRouter listens to *all* events, but will only route
@@ -30,27 +34,27 @@ class ExtensionBrowserEventRouter : public TabStripModelObserver,
 #if defined(TOOLKIT_VIEWS)
                                     public views::WidgetFocusChangeListener,
 #elif defined(TOOLKIT_GTK)
-                                    public ui::ActiveWindowWatcherX::Observer,
+                                    public ui::ActiveWindowWatcherXObserver,
 #endif
                                     public BrowserList::Observer,
-                                    public NotificationObserver {
+                                    public content::NotificationObserver {
  public:
   explicit ExtensionBrowserEventRouter(Profile* profile);
-  ~ExtensionBrowserEventRouter();
+  virtual ~ExtensionBrowserEventRouter();
 
   // Must be called once. Subsequent calls have no effect.
   void Init();
 
   // BrowserList::Observer
-  virtual void OnBrowserAdded(const Browser* browser);
-  virtual void OnBrowserRemoved(const Browser* browser);
-  virtual void OnBrowserSetLastActive(const Browser* browser);
+  virtual void OnBrowserAdded(const Browser* browser) OVERRIDE;
+  virtual void OnBrowserRemoved(const Browser* browser) OVERRIDE;
+  virtual void OnBrowserSetLastActive(const Browser* browser) OVERRIDE;
 
 #if defined(TOOLKIT_VIEWS)
-  virtual void NativeFocusWillChange(gfx::NativeView focused_before,
-                                     gfx::NativeView focused_now);
+  virtual void OnNativeFocusChange(gfx::NativeView focused_before,
+                                   gfx::NativeView focused_now) OVERRIDE;
 #elif defined(TOOLKIT_GTK)
-  virtual void ActiveWindowChanged(GdkWindow* active_window);
+  virtual void ActiveWindowChanged(GdkWindow* active_window) OVERRIDE;
 #endif
 
   // Called from Observe() on BROWSER_WINDOW_READY (not a part of
@@ -59,25 +63,29 @@ class ExtensionBrowserEventRouter : public TabStripModelObserver,
 
   // TabStripModelObserver
   virtual void TabInsertedAt(TabContentsWrapper* contents, int index,
-                             bool foreground);
+                             bool active) OVERRIDE;
   virtual void TabClosingAt(TabStripModel* tab_strip_model,
                             TabContentsWrapper* contents,
-                            int index);
-  virtual void TabDetachedAt(TabContentsWrapper* contents, int index);
-  virtual void TabSelectedAt(TabContentsWrapper* old_contents,
-                             TabContentsWrapper* new_contents,
-                             int index,
-                             bool user_gesture);
+                            int index) OVERRIDE;
+  virtual void TabDetachedAt(TabContentsWrapper* contents, int index) OVERRIDE;
+  virtual void ActiveTabChanged(TabContentsWrapper* old_contents,
+                                TabContentsWrapper* new_contents,
+                                int index,
+                                bool user_gesture) OVERRIDE;
+  virtual void TabSelectionChanged(
+      TabStripModel* tab_strip_model,
+      const TabStripSelectionModel& old_model) OVERRIDE;
   virtual void TabMoved(TabContentsWrapper* contents, int from_index,
-                        int to_index);
+                        int to_index) OVERRIDE;
   virtual void TabChangedAt(TabContentsWrapper* contents, int index,
-                            TabChangeType change_type);
+                            TabChangeType change_type) OVERRIDE;
   virtual void TabReplacedAt(TabStripModel* tab_strip_model,
                              TabContentsWrapper* old_contents,
                              TabContentsWrapper* new_contents,
-                             int index);
-  virtual void TabPinnedStateChanged(TabContentsWrapper* contents, int index);
-  virtual void TabStripEmpty();
+                             int index) OVERRIDE;
+  virtual void TabPinnedStateChanged(TabContentsWrapper* contents,
+                                     int index) OVERRIDE;
+  virtual void TabStripEmpty() OVERRIDE;
 
   // Page Action execute event.
   void PageActionExecuted(Profile* profile,
@@ -91,21 +99,48 @@ class ExtensionBrowserEventRouter : public TabStripModelObserver,
                              const std::string& extension_id,
                              Browser* browser);
 
-  // NotificationObserver.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
  private:
   // "Synthetic" event. Called from TabInsertedAt if new tab is detected.
-  void TabCreatedAt(TabContents* contents, int index, bool foreground);
+  void TabCreatedAt(content::WebContents* contents, int index, bool active);
 
   // Internal processing of tab updated events. Is called by both TabChangedAt
   // and Observe/NAV_ENTRY_COMMITTED.
-  void TabUpdated(TabContents* contents, bool did_navigate);
+  void TabUpdated(content::WebContents* contents, bool did_navigate);
+
+  // The DispatchEvent methods forward events to the |profile|'s event router.
+  // The ExtensionBrowserEventRouter listens to events for all profiles,
+  // so we avoid duplication by dropping events destined for other profiles.
+  void DispatchEvent(Profile* profile,
+                     const char* event_name,
+                     const std::string& json_args);
+
+  void DispatchEventToExtension(Profile* profile,
+                                const std::string& extension_id,
+                                const char* event_name,
+                                const std::string& json_args);
+
+  void DispatchEventsAcrossIncognito(Profile* profile,
+                                     const char* event_name,
+                                     const std::string& json_args,
+                                     const std::string& cross_incognito_args);
+
+  void DispatchEventWithTab(Profile* profile,
+                            const std::string& extension_id,
+                            const char* event_name,
+                            const content::WebContents* web_contents,
+                            bool active);
+
+  void DispatchSimpleBrowserEvent(Profile* profile,
+                                  const int window_id,
+                                  const char* event_name);
 
   // Packages |changed_properties| as a tab updated event for the tab |contents|
   // and dispatches the event to the extension.
-  void DispatchTabUpdatedEvent(TabContents* contents,
+  void DispatchTabUpdatedEvent(content::WebContents* contents,
                                DictionaryValue* changed_properties);
 
   // Called to dispatch a deprecated style page action click event that was
@@ -124,12 +159,12 @@ class ExtensionBrowserEventRouter : public TabStripModelObserver,
 
   // Register ourselves to receive the various notifications we are interested
   // in for a tab.
-  void RegisterForTabNotifications(TabContents* contents);
+  void RegisterForTabNotifications(content::WebContents* contents);
 
   // Removes notifications added in RegisterForTabNotifications.
-  void UnregisterForTabNotifications(TabContents* contents);
+  void UnregisterForTabNotifications(content::WebContents* contents);
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   bool initialized_;
 
@@ -145,18 +180,18 @@ class ExtensionBrowserEventRouter : public TabStripModelObserver,
     // std::map<> by value.
     TabEntry();
 
-    // Update the load state of the tab based on its TabContents.  Returns true
+    // Update the load state of the tab based on its WebContents.  Returns true
     // if the state changed, false otherwise.  Whether the state has changed or
     // not is used to determine if events needs to be sent to extensions during
     // processing of TabChangedAt(). This method will "hold" a state-change
     // to "loading", until the DidNavigate() method which should always follow
     // it. Returns NULL if no updates should be sent.
-    DictionaryValue* UpdateLoadState(const TabContents* contents);
+    DictionaryValue* UpdateLoadState(const content::WebContents* contents);
 
     // Indicates that a tab load has resulted in a navigation and the
     // destination url is available for inspection. Returns NULL if no updates
     // should be sent.
-    DictionaryValue* DidNavigate(const TabContents* contents);
+    DictionaryValue* DidNavigate(const content::WebContents* contents);
 
    private:
     // Whether we are waiting to fire the 'complete' status change. This will
@@ -170,18 +205,22 @@ class ExtensionBrowserEventRouter : public TabStripModelObserver,
 
   // Gets the TabEntry for the given |contents|. Returns TabEntry* if
   // found, NULL if not.
-  TabEntry* GetTabEntry(const TabContents* contents);
+  TabEntry* GetTabEntry(const content::WebContents* contents);
 
   std::map<int, TabEntry> tab_entries_;
 
+  // The main profile that owns this event router.
+  Profile* profile_;
+
+  // The profile the currently focused window belongs to; either the main or
+  // incognito profile or NULL (none of the above). We remember this in order
+  // to correctly handle focus changes between non-OTR and OTR windows.
+  Profile* focused_profile_;
+
   // The currently focused window. We keep this so as to avoid sending multiple
   // windows.onFocusChanged events with the same windowId.
   int focused_window_id_;
 
-  // The main profile (non-OTR) profile which will be used to send events not
-  // associated with any browser.
-  Profile* profile_;
-
   DISALLOW_COPY_AND_ASSIGN(ExtensionBrowserEventRouter);
 };
 
