@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,62 +9,87 @@
 #include <string>
 
 #include "base/command_line.h"
+#include "base/compiler_specific.h"
 #include "base/file_path.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_type.h"
-#include "chrome/test/in_process_browser_test.h"
+#include "base/scoped_temp_dir.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_types.h"
+
+class Extension;
 
 // Base class for extension browser tests. Provides utilities for loading,
 // unloading, and installing extensions.
 class ExtensionBrowserTest
-    : public InProcessBrowserTest, public NotificationObserver {
+    : public InProcessBrowserTest, public content::NotificationObserver {
  protected:
   ExtensionBrowserTest();
 
-  virtual void SetUpCommandLine(CommandLine* command_line);
-  bool LoadExtension(const FilePath& path);
+  virtual void SetUpCommandLine(CommandLine* command_line) OVERRIDE;
+
+  const Extension* LoadExtension(const FilePath& path);
 
   // Same as above, but enables the extension in incognito mode first.
-  bool LoadExtensionIncognito(const FilePath& path);
+  const Extension* LoadExtensionIncognito(const FilePath& path);
+
+  const Extension* LoadExtensionWithOptions(const FilePath& path,
+                                            bool incognito_enabled,
+                                            bool fileaccess_enabled);
+
+  // Loads extension and imitates that it is a component extension.
+  const Extension* LoadExtensionAsComponent(const FilePath& path);
 
   // Pack the extension in |dir_path| into a crx file and return its path.
   // Return an empty FilePath if there were errors.
   FilePath PackExtension(const FilePath& dir_path);
 
+  // Pack the extension in |dir_path| into a crx file at |crx_path|, using the
+  // key |pem_path|. If |pem_path| does not exist, create a new key at
+  // |pem_out_path|.
+  // Return the path to the crx file, or an empty FilePath if there were errors.
+  FilePath PackExtensionWithOptions(const FilePath& dir_path,
+                                    const FilePath& crx_path,
+                                    const FilePath& pem_path,
+                                    const FilePath& pem_out_path);
+
   // |expected_change| indicates how many extensions should be installed (or
   // disabled, if negative).
   // 1 means you expect a new install, 0 means you expect an upgrade, -1 means
   // you expect a failed upgrade.
-  bool InstallExtension(const FilePath& path, int expected_change) {
+  const Extension* InstallExtension(const FilePath& path, int expected_change) {
     return InstallOrUpdateExtension("", path, INSTALL_UI_TYPE_NONE,
                                     expected_change);
   }
 
+  // Installs extension as if it came from the Chrome Webstore.
+  const Extension* InstallExtensionFromWebstore(
+      const FilePath& path, int expected_change);
+
   // Same as above but passes an id to CrxInstaller and does not allow a
   // privilege increase.
-  bool UpdateExtension(const std::string& id, const FilePath& path,
-                       int expected_change) {
+  const Extension* UpdateExtension(const std::string& id, const FilePath& path,
+                                   int expected_change) {
     return InstallOrUpdateExtension(id, path, INSTALL_UI_TYPE_NONE,
                                     expected_change);
   }
 
   // Same as |InstallExtension| but with the normal extension UI showing up
   // (for e.g. info bar on success).
-  bool InstallExtensionWithUI(const FilePath& path, int expected_change) {
+  const Extension* InstallExtensionWithUI(const FilePath& path,
+                                          int expected_change) {
     return InstallOrUpdateExtension("", path, INSTALL_UI_TYPE_NORMAL,
                                     expected_change);
   }
-  bool InstallExtensionWithUIAutoConfirm(const FilePath& path,
-                                         int expected_change,
-                                         Profile* profile) {
+  const Extension* InstallExtensionWithUIAutoConfirm(const FilePath& path,
+                                                     int expected_change,
+                                                     Profile* profile) {
     return InstallOrUpdateExtension("", path, INSTALL_UI_TYPE_AUTO_CONFIRM,
-                                    expected_change, profile);
+                                    expected_change, profile, false);
   }
 
   // Begins install process but simulates a user cancel.
-  bool StartInstallButCancel(const FilePath& path) {
+  const Extension* StartInstallButCancel(const FilePath& path) {
     return InstallOrUpdateExtension("", path, INSTALL_UI_TYPE_CANCEL, 0);
   }
 
@@ -99,10 +124,10 @@ class ExtensionBrowserTest
   // crashed.
   bool WaitForExtensionCrash(const std::string& extension_id);
 
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   bool loaded_;
   bool installed_;
@@ -113,6 +138,9 @@ class ExtensionBrowserTest
   int extension_installs_observed_;
 
  private:
+  // Temporary directory for testing.
+  ScopedTempDir temp_dir_;
+
   // Specifies the type of UI (if any) to show during installation and what
   // user action to simulate.
   enum InstallUIType {
@@ -122,14 +150,16 @@ class ExtensionBrowserTest
     INSTALL_UI_TYPE_AUTO_CONFIRM,
   };
 
-  bool InstallOrUpdateExtension(const std::string& id, const FilePath& path,
-                                InstallUIType ui_type,
-                                int expected_change);
-  bool InstallOrUpdateExtension(const std::string& id, const FilePath& path,
-                                InstallUIType ui_type,
-                                int expected_change,
-                                Profile* profile);
-  bool LoadExtensionImpl(const FilePath& path, bool incognito_enabled);
+  const Extension* InstallOrUpdateExtension(const std::string& id,
+                                            const FilePath& path,
+                                            InstallUIType ui_type,
+                                            int expected_change);
+  const Extension* InstallOrUpdateExtension(const std::string& id,
+                                            const FilePath& path,
+                                            InstallUIType ui_type,
+                                            int expected_change,
+                                            Profile* profile,
+                                            bool from_webstore);
 
   bool WaitForExtensionHostsToLoad();
 
