@@ -1,34 +1,35 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/extensions/extension_browsertest.h"
-#include "chrome/browser/extensions/extension_test_message_listener.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_test_message_listener.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/tab_contents/render_view_context_menu.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "net/base/mock_host_resolver.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebContextMenuData.h"
 #include "ui/base/models/menu_model.h"
 #include "webkit/glue/context_menu.h"
 
-using ui::MenuModel;
 using WebKit::WebContextMenuData;
+using content::WebContents;
+using ui::MenuModel;
 
+namespace {
 // This test class helps us sidestep platform-specific issues with popping up a
 // real context menu, while still running through the actual code in
 // RenderViewContextMenu where extension items get added and executed.
 class TestRenderViewContextMenu : public RenderViewContextMenu {
  public:
-  TestRenderViewContextMenu(TabContents* tab_contents,
+  TestRenderViewContextMenu(WebContents* web_contents,
                             const ContextMenuParams& params)
-      : RenderViewContextMenu(tab_contents, params) {}
+      : RenderViewContextMenu(web_contents, params) {}
 
   virtual ~TestRenderViewContextMenu() {}
 
@@ -113,6 +114,8 @@ class TestRenderViewContextMenu : public RenderViewContextMenu {
   }
 };
 
+}  // namespace
+
 class ExtensionContextMenuBrowserTest : public ExtensionBrowserTest {
  public:
   // Helper to load an extension from context_menus/|subdirectory| in the
@@ -131,14 +134,16 @@ class ExtensionContextMenuBrowserTest : public ExtensionBrowserTest {
 
   TestRenderViewContextMenu* CreateMenu(Browser* browser,
                                         const GURL& page_url,
-                                        const GURL& link_url) {
-    TabContents* tab_contents = browser->GetSelectedTabContents();
+                                        const GURL& link_url,
+                                        const GURL& frame_url) {
+    WebContents* web_contents = browser->GetSelectedWebContents();
     WebContextMenuData data;
     ContextMenuParams params(data);
     params.page_url = page_url;
     params.link_url = link_url;
+    params.frame_url = frame_url;
     TestRenderViewContextMenu* menu =
-        new TestRenderViewContextMenu(tab_contents, params);
+        new TestRenderViewContextMenu(web_contents, params);
     menu->Init();
     return menu;
   }
@@ -151,9 +156,9 @@ class ExtensionContextMenuBrowserTest : public ExtensionBrowserTest {
   // Returns a pointer to the currently loaded extension with |name|, or null
   // if not found.
   const Extension* GetExtensionNamed(std::string name) {
-    const ExtensionList* extensions =
+    const ExtensionSet* extensions =
         browser()->profile()->GetExtensionService()->extensions();
-    ExtensionList::const_iterator i;
+    ExtensionSet::const_iterator i;
     for (i = extensions->begin(); i != extensions->end(); ++i) {
       if ((*i)->name() == name) {
         return *i;
@@ -180,9 +185,10 @@ class ExtensionContextMenuBrowserTest : public ExtensionBrowserTest {
   // was found.
   bool MenuHasItemWithLabel(const GURL& page_url,
                             const GURL& link_url,
+                            const GURL& frame_url,
                             const std::string& label) {
     scoped_ptr<TestRenderViewContextMenu> menu(
-        CreateMenu(browser(), page_url, link_url));
+        CreateMenu(browser(), page_url, link_url, frame_url));
     return menu->HasExtensionItemWithLabel(label);
   }
 };
@@ -200,7 +206,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, Simple) {
 
   // Create and build our test context menu.
   scoped_ptr<TestRenderViewContextMenu> menu(
-      CreateMenu(browser(), page_url, GURL()));
+      CreateMenu(browser(), page_url, GURL(), GURL()));
 
   // Look for the extension item in the menu, and execute it.
   int command_id = IDC_EXTENSIONS_CONTEXT_CUSTOM_FIRST;
@@ -224,9 +230,11 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, Patterns) {
   // Check that a document url that should match the items' patterns appears.
   GURL google_url("http://www.google.com");
   ASSERT_TRUE(MenuHasItemWithLabel(google_url,
+                                   GURL(),
                                    GURL(),
                                    std::string("test_item1")));
   ASSERT_TRUE(MenuHasItemWithLabel(google_url,
+                                   GURL(),
                                    GURL(),
                                    std::string("test_item2")));
 
@@ -234,8 +242,10 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, Patterns) {
   GURL test_url("http://www.test.com");
   ASSERT_FALSE(MenuHasItemWithLabel(test_url,
                                     GURL(),
+                                   GURL(),
                                     std::string("test_item1")));
   ASSERT_FALSE(MenuHasItemWithLabel(test_url,
+                                    GURL(),
                                     GURL(),
                                     std::string("test_item2")));
 }
@@ -260,7 +270,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, LongTitle) {
   // truncated.
   GURL url("http://foo.com/");
   scoped_ptr<TestRenderViewContextMenu> menu(
-      CreateMenu(browser(), url, GURL()));
+      CreateMenu(browser(), url, GURL(), GURL()));
 
   string16 label;
   ASSERT_TRUE(menu->GetItemLabel(item->id(), &label));
@@ -325,7 +335,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, Separators) {
 
   GURL url("http://www.google.com/");
   scoped_ptr<TestRenderViewContextMenu> menu(
-      CreateMenu(browser(), url, GURL()));
+      CreateMenu(browser(), url, GURL(), GURL()));
 
   // The top-level item should be an "automagic parent" with the extension's
   // name.
@@ -348,7 +358,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, Separators) {
   ui_test_utils::NavigateToURL(browser(),
                                GURL(extension->GetResourceURL("test2.html")));
   listener2.WaitUntilSatisfied();
-  menu.reset(CreateMenu(browser(), url, GURL()));
+  menu.reset(CreateMenu(browser(), url, GURL(), GURL()));
   ASSERT_TRUE(menu->GetMenuModelAndItemIndex(
       IDC_EXTENSIONS_CONTEXT_CUSTOM_FIRST, &model, &index));
   EXPECT_EQ(UTF8ToUTF16("parent"), model->GetLabelAt(index));
@@ -368,21 +378,30 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, TargetURLs) {
   GURL non_google_url("http://www.foo.com");
 
   // No target url - the item should not appear.
-  ASSERT_FALSE(MenuHasItemWithLabel(google_url, GURL(), std::string("item1")));
+  ASSERT_FALSE(MenuHasItemWithLabel(
+      google_url, GURL(), GURL(), std::string("item1")));
 
   // A matching target url - the item should appear.
   ASSERT_TRUE(MenuHasItemWithLabel(google_url,
                                    google_url,
+                                   GURL(),
                                    std::string("item1")));
 
   // A non-matching target url - the item should not appear.
   ASSERT_FALSE(MenuHasItemWithLabel(google_url,
                                     non_google_url,
+                                    GURL(),
                                     std::string("item1")));
 }
 
-// Tests adding a simple context menu item.
-IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, IncognitoSplit) {
+// Tests adding of context menus in incognito mode.
+#if defined(OS_LINUX)
+// Flakily hangs on Linux/CrOS - http://crbug.com/88317
+#define MAYBE_IncognitoSplit DISABLED_IncognitoSplit
+#else
+#define MAYBE_IncognitoSplit IncognitoSplit
+#endif
+IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, MAYBE_IncognitoSplit) {
   ExtensionTestMessageListener created("created item regular", false);
   ExtensionTestMessageListener created_incognito("created item incognito",
                                                  false);
@@ -403,14 +422,13 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, IncognitoSplit) {
   GURL page_url("http://www.google.com");
 
   // Create and build our test context menu.
-  Browser* browser_incognito = BrowserList::FindBrowserWithType(
-      browser()->profile()->GetOffTheRecordProfile(),
-      Browser::TYPE_NORMAL, false);
+  Browser* browser_incognito = BrowserList::FindTabbedBrowser(
+      browser()->profile()->GetOffTheRecordProfile(), false);
   ASSERT_TRUE(browser_incognito);
   scoped_ptr<TestRenderViewContextMenu> menu(
-      CreateMenu(browser(), page_url, GURL()));
+      CreateMenu(browser(), page_url, GURL(), GURL()));
   scoped_ptr<TestRenderViewContextMenu> menu_incognito(
-      CreateMenu(browser_incognito, page_url, GURL()));
+      CreateMenu(browser_incognito, page_url, GURL(), GURL()));
 
   // Look for the extension item in the menu, and execute it.
   int command_id = IDC_EXTENSIONS_CONTEXT_CUSTOM_FIRST;
@@ -427,3 +445,25 @@ IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, IncognitoSplit) {
   menu_incognito->ExecuteCommand(command_id);
   ASSERT_TRUE(onclick_incognito.WaitUntilSatisfied());
 }
+
+// Tests that items with a context of frames only appear when the menu is
+// invoked in a frame.
+IN_PROC_BROWSER_TEST_F(ExtensionContextMenuBrowserTest, Frames) {
+  ExtensionTestMessageListener listener("created items", false);
+  ASSERT_TRUE(LoadContextMenuExtension("frames"));
+  ASSERT_TRUE(listener.WaitUntilSatisfied());
+
+  GURL page_url("http://www.google.com");
+  GURL no_frame_url;
+  GURL frame_url("http://www.google.com");
+
+  ASSERT_TRUE(MenuHasItemWithLabel(
+      page_url, GURL(), no_frame_url, std::string("Page item")));
+  ASSERT_FALSE(MenuHasItemWithLabel(
+      page_url, GURL(), no_frame_url, std::string("Frame item")));
+
+  ASSERT_TRUE(MenuHasItemWithLabel(
+      page_url, GURL(), frame_url, std::string("Page item")));
+  ASSERT_TRUE(MenuHasItemWithLabel(
+      page_url, GURL(), frame_url, std::string("Frame item")));
+}
