@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,16 +6,23 @@
 
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/extensions/extension_service.h"
-#include "chrome/browser/extensions/extension_tabs_module.h"
+#include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_action.h"
 #include "chrome/common/extensions/extension_constants.h"
-#include "chrome/common/extensions/extension.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
+#include "content/public/browser/web_contents.h"
+#include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
+#include "ui/base/l10n/l10n_util.h"
+
+using content::OpenURLParams;
+using content::Referrer;
+using content::WebContents;
 
 enum MenuEntries {
   NAME = 0,
@@ -61,9 +68,9 @@ void ExtensionContextMenuModel::InitCommonCommands() {
 
   AddItem(NAME, UTF8ToUTF16(extension->name()));
   AddSeparator();
-  AddItemWithStringId(CONFIGURE, IDS_EXTENSIONS_OPTIONS);
+  AddItemWithStringId(CONFIGURE, IDS_EXTENSIONS_OPTIONS_MENU_ITEM);
   AddItemWithStringId(DISABLE, IDS_EXTENSIONS_DISABLE);
-  AddItemWithStringId(UNINSTALL, IDS_EXTENSIONS_UNINSTALL);
+  AddItem(UNINSTALL, l10n_util::GetStringUTF16(IDS_EXTENSIONS_UNINSTALL));
   if (extension->browser_action())
     AddItemWithStringId(HIDE, IDS_EXTENSIONS_HIDE_BUTTON);
   AddSeparator();
@@ -86,11 +93,14 @@ bool ExtensionContextMenuModel::IsCommandIdEnabled(int command_id) const {
     // homepage, we just disable this menu item.
     return extension->GetHomepageURL().is_valid();
   } else if (command_id == INSPECT_POPUP) {
-    TabContents* contents = browser_->GetSelectedTabContents();
+    WebContents* contents = browser_->GetSelectedWebContents();
     if (!contents)
       return false;
 
     return extension_action_->HasPopup(ExtensionTabUtil::GetTabId(contents));
+  } else if (command_id == DISABLE || command_id == UNINSTALL) {
+    // Some extension types can not be disabled or uninstalled.
+    return Extension::UserMayDisable(extension->location());
   }
   return true;
 }
@@ -107,8 +117,10 @@ void ExtensionContextMenuModel::ExecuteCommand(int command_id) {
 
   switch (command_id) {
     case NAME: {
-      browser_->OpenURL(extension->GetHomepageURL(), GURL(),
-                        NEW_FOREGROUND_TAB, PageTransition::LINK);
+      OpenURLParams params(extension->GetHomepageURL(), Referrer(),
+                           NEW_FOREGROUND_TAB, content::PAGE_TRANSITION_LINK,
+                           false);
+      browser_->OpenURL(params);
       break;
     }
     case CONFIGURE:
@@ -127,14 +139,14 @@ void ExtensionContextMenuModel::ExecuteCommand(int command_id) {
       break;
     }
     case UNINSTALL: {
-      AddRef();  // Balanced in InstallUIProceed and InstallUIAbort.
-      install_ui_.reset(new ExtensionInstallUI(profile_));
-      install_ui_->ConfirmUninstall(this, extension);
+      AddRef();  // Balanced in Accepted() and Canceled()
+      extension_uninstall_dialog_.reset(
+          ExtensionUninstallDialog::Create(profile_, this));
+      extension_uninstall_dialog_->ConfirmUninstall(extension);
       break;
     }
     case MANAGE: {
-      browser_->OpenURL(GURL(chrome::kChromeUIExtensionsURL), GURL(),
-                        SINGLETON_TAB, PageTransition::LINK);
+      browser_->ShowOptionsTab(chrome::kExtensionsSubPage);
       break;
     }
     case INSPECT_POPUP: {
@@ -147,14 +159,15 @@ void ExtensionContextMenuModel::ExecuteCommand(int command_id) {
   }
 }
 
-void ExtensionContextMenuModel::InstallUIProceed() {
+void ExtensionContextMenuModel::ExtensionUninstallAccepted() {
   if (GetExtension())
-    profile_->GetExtensionService()->UninstallExtension(extension_id_, false);
+    profile_->GetExtensionService()->UninstallExtension(extension_id_, false,
+                                                        NULL);
 
   Release();
 }
 
-void ExtensionContextMenuModel::InstallUIAbort() {
+void ExtensionContextMenuModel::ExtensionUninstallCanceled() {
   Release();
 }
 
