@@ -1,9 +1,9 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 // Defines the Chrome Extensions Cookies API functions for accessing internet
-// cookies, as specified in chrome/common/extensions/api/extension_api.json.
+// cookies, as specified in the extension API JSON.
 
 #ifndef CHROME_BROWSER_EXTENSIONS_EXTENSION_COOKIES_API_H_
 #define CHROME_BROWSER_EXTENSIONS_EXTENSION_COOKIES_API_H_
@@ -11,38 +11,38 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
-#include "base/singleton.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
 #include "base/time.h"
 #include "chrome/browser/extensions/extension_function.h"
 #include "chrome/browser/net/chrome_cookie_notification_details.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "googleurl/src/gurl.h"
-#include "net/base/cookie_monster.h"
 
+namespace base {
 class DictionaryValue;
+}
+
+namespace net {
+class CookieList;
 class URLRequestContextGetter;
+}
 
 // Observes CookieMonster notifications and routes them as events to the
 // extension system.
-class ExtensionCookiesEventRouter : public NotificationObserver {
+class ExtensionCookiesEventRouter : public content::NotificationObserver {
  public:
-  // Single instance of the event router.
-  static ExtensionCookiesEventRouter* GetInstance();
+  explicit ExtensionCookiesEventRouter(Profile* profile);
+  virtual ~ExtensionCookiesEventRouter();
 
   void Init();
 
  private:
-  friend struct DefaultSingletonTraits<ExtensionCookiesEventRouter>;
-
-  ExtensionCookiesEventRouter() {}
-  virtual ~ExtensionCookiesEventRouter() {}
-
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Handler for the COOKIE_CHANGED event. The method takes the details of such
   // an event and constructs a suitable JSON formatted extension event from it.
@@ -56,7 +56,9 @@ class ExtensionCookiesEventRouter : public NotificationObserver {
                      GURL& cookie_domain);
 
   // Used for tracking registrations to CookieMonster notifications.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
+
+  Profile* profile_;
 
   DISALLOW_COPY_AND_ASSIGN(ExtensionCookiesEventRouter);
 };
@@ -75,7 +77,7 @@ class CookiesFunction : public AsyncExtensionFunction {
   // URL is invalid or isn't found in the dictionary. If check_host_permissions
   // is true, the URL is also checked against the extension's host permissions,
   // and if there is no permission for the URL, this function returns false.
-  bool ParseUrl(const DictionaryValue* details, GURL* url,
+  bool ParseUrl(const base::DictionaryValue* details, GURL* url,
                 bool check_host_permissions);
 
   // Checks the given details dictionary for a 'storeId' value, and retrieves
@@ -85,8 +87,8 @@ class CookiesFunction : public AsyncExtensionFunction {
   // assigns the internal error_ value if that occurs.
   // At least one of the output parameters store and store_id should be
   // non-NULL.
-  bool ParseStoreContext(const DictionaryValue* details,
-                         URLRequestContextGetter** context,
+  bool ParseStoreContext(const base::DictionaryValue* details,
+                         net::URLRequestContextGetter** context,
                          std::string* store_id);
 };
 
@@ -94,51 +96,53 @@ class CookiesFunction : public AsyncExtensionFunction {
 class GetCookieFunction : public CookiesFunction {
  public:
   GetCookieFunction();
-  ~GetCookieFunction();
-  virtual bool RunImpl();
+  virtual ~GetCookieFunction();
+  virtual bool RunImpl() OVERRIDE;
   DECLARE_EXTENSION_FUNCTION_NAME("cookies.get")
 
  private:
   void GetCookieOnIOThread();
   void RespondOnUIThread();
+  void GetCookieCallback(const net::CookieList& cookie_list);
 
   std::string name_;
   GURL url_;
   std::string store_id_;
-  scoped_refptr<URLRequestContextGetter> store_context_;
-  net::CookieList cookie_list_;
+  scoped_refptr<net::URLRequestContextGetter> store_context_;
 };
 
 // Implements the cookies.getAll() extension function.
 class GetAllCookiesFunction : public CookiesFunction {
  public:
   GetAllCookiesFunction();
-  ~GetAllCookiesFunction();
-  virtual bool RunImpl();
+  virtual ~GetAllCookiesFunction();
+  virtual bool RunImpl() OVERRIDE;
   DECLARE_EXTENSION_FUNCTION_NAME("cookies.getAll")
 
  private:
   void GetAllCookiesOnIOThread();
   void RespondOnUIThread();
+  void GetAllCookiesCallback(const net::CookieList& cookie_list);
 
-  DictionaryValue* details_;
+  base::DictionaryValue* details_;
   GURL url_;
   std::string store_id_;
-  scoped_refptr<URLRequestContextGetter> store_context_;
-  net::CookieList cookie_list_;
+  scoped_refptr<net::URLRequestContextGetter> store_context_;
 };
 
 // Implements the cookies.set() extension function.
 class SetCookieFunction : public CookiesFunction {
  public:
   SetCookieFunction();
-  ~SetCookieFunction();
-  virtual bool RunImpl();
+  virtual ~SetCookieFunction();
+  virtual bool RunImpl() OVERRIDE;
   DECLARE_EXTENSION_FUNCTION_NAME("cookies.set")
 
  private:
   void SetCookieOnIOThread();
   void RespondOnUIThread();
+  void PullCookie(bool set_cookie_);
+  void PullCookieCallback(const net::CookieList& cookie_list);
 
   GURL url_;
   std::string name_;
@@ -149,24 +153,36 @@ class SetCookieFunction : public CookiesFunction {
   bool http_only_;
   base::Time expiration_time_;
   bool success_;
-  scoped_refptr<URLRequestContextGetter> store_context_;
+  std::string store_id_;
+  scoped_refptr<net::URLRequestContextGetter> store_context_;
 };
 
 // Implements the cookies.remove() extension function.
 class RemoveCookieFunction : public CookiesFunction {
  public:
-  virtual bool RunImpl();
-  // RemoveCookieFunction is sync.
-  virtual void Run();
+  RemoveCookieFunction();
+  virtual ~RemoveCookieFunction();
+  virtual bool RunImpl() OVERRIDE;
   DECLARE_EXTENSION_FUNCTION_NAME("cookies.remove")
+
+ private:
+  void RemoveCookieOnIOThread();
+  void RespondOnUIThread();
+  void RemoveCookieCallback();
+
+  GURL url_;
+  std::string name_;
+  bool success_;
+  std::string store_id_;
+  scoped_refptr<net::URLRequestContextGetter> store_context_;
 };
 
 // Implements the cookies.getAllCookieStores() extension function.
 class GetAllCookieStoresFunction : public CookiesFunction {
  public:
-  virtual bool RunImpl();
+  virtual bool RunImpl() OVERRIDE;
   // GetAllCookieStoresFunction is sync.
-  virtual void Run();
+  virtual void Run() OVERRIDE;
   DECLARE_EXTENSION_FUNCTION_NAME("cookies.getAllCookieStores")
 };
 
