@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,6 +11,7 @@
 namespace extension_cookies_api_constants {
 
 // Keys.
+extern const char kCauseKey[];
 extern const char kCookieKey[];
 extern const char kDomainKey[];
 extern const char kExpirationDateKey[];
@@ -27,6 +28,13 @@ extern const char kTabIdsKey[];
 extern const char kUrlKey[];
 extern const char kValueKey[];
 
+// Cause Constants
+extern const char kEvictedChangeCause[];
+extern const char kExpiredChangeCause[];
+extern const char kExpiredOverwriteChangeCause[];
+extern const char kExplicitChangeCause[];
+extern const char kOverwriteChangeCause[];
+
 // Events.
 extern const char kOnChanged[];
 
