@@ -3,20 +3,28 @@
 // found in the LICENSE file.
 
 #include "base/process_util.h"
-#include "chrome/browser/extensions/crashed_extension_infobar.h"
+#include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/extension_browsertest.h"
 #include "chrome/browser/extensions/extension_host.h"
 #include "chrome/browser/extensions/extension_process_manager.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/notifications/balloon_host.h"
+#include "chrome/browser/notifications/notification.h"
+#include "chrome/browser/notifications/notification_delegate.h"
+#include "chrome/browser/notifications/notification_ui_manager.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/render_process_host.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/tab_contents/infobar_delegate.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/browser.h"
-#include "chrome/common/result_codes.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/render_process_host.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/common/result_codes.h"
+
+using content::NavigationController;
+using content::WebContents;
 
 class ExtensionCrashRecoveryTest : public ExtensionBrowserTest {
  protected:
@@ -28,77 +36,83 @@ class ExtensionCrashRecoveryTest : public ExtensionBrowserTest {
     return browser()->profile()->GetExtensionProcessManager();
   }
 
-  ConfirmInfoBarDelegate* GetInfoBarDelegate(int index) {
-    TabContents* current_tab = browser()->GetSelectedTabContents();
-    EXPECT_LT(index, current_tab->infobar_delegate_count());
-    return current_tab->GetInfoBarDelegateAt(index)->AsConfirmInfoBarDelegate();
+  Balloon* GetNotificationDelegate(size_t index) {
+    NotificationUIManager* manager =
+        g_browser_process->notification_ui_manager();
+    BalloonCollection::Balloons balloons =
+        manager->balloon_collection()->GetActiveBalloons();
+    return balloons.at(index);
   }
 
-  void AcceptInfoBar(int index) {
-    ConfirmInfoBarDelegate* infobar = GetInfoBarDelegate(index);
-    ASSERT_TRUE(infobar);
-    infobar->Accept();
+  void AcceptNotification(size_t index) {
+    Balloon* balloon = GetNotificationDelegate(index);
+    ASSERT_TRUE(balloon);
+    balloon->OnClick();
     WaitForExtensionLoad();
   }
 
-  void CancelInfoBar(int index) {
-    ConfirmInfoBarDelegate* infobar = GetInfoBarDelegate(index);
-    ASSERT_TRUE(infobar);
-    infobar->Cancel();
+  void CancelNotification(size_t index) {
+    Balloon* balloon = GetNotificationDelegate(index);
+    NotificationUIManager* manager =
+        g_browser_process->notification_ui_manager();
+    ASSERT_TRUE(manager->CancelById(balloon->notification().notification_id()));
   }
 
-  void CrashExtension(size_t index) {
-    ASSERT_LT(index, GetExtensionService()->extensions()->size());
+  size_t CountBalloons() {
+    NotificationUIManager* manager =
+        g_browser_process->notification_ui_manager();
+    BalloonCollection::Balloons balloons =
+        manager->balloon_collection()->GetActiveBalloons();
+    return balloons.size();
+  }
+
+  void CrashExtension(std::string extension_id) {
     const Extension* extension =
-        GetExtensionService()->extensions()->at(index);
+        GetExtensionService()->extensions()->GetByID(extension_id);
     ASSERT_TRUE(extension);
-    std::string extension_id(extension->id());
-    ExtensionHost* extension_host =
-        GetExtensionProcessManager()->GetBackgroundHostForExtension(extension);
+    ExtensionHost* extension_host = GetExtensionProcessManager()->
+        GetBackgroundHostForExtension(extension_id);
     ASSERT_TRUE(extension_host);
 
-    RenderProcessHost* extension_rph =
+    content::RenderProcessHost* extension_rph =
         extension_host->render_view_host()->process();
-    base::KillProcess(extension_rph->GetHandle(), ResultCodes::KILLED, false);
+    base::KillProcess(extension_rph->GetHandle(), content::RESULT_CODE_KILLED,
+                      false);
     ASSERT_TRUE(WaitForExtensionCrash(extension_id));
-    ASSERT_FALSE(
-        GetExtensionProcessManager()->GetBackgroundHostForExtension(extension));
+    ASSERT_FALSE(GetExtensionProcessManager()->
+                 GetBackgroundHostForExtension(extension_id));
   }
 
-  void CheckExtensionConsistency(size_t index) {
-    ASSERT_LT(index, GetExtensionService()->extensions()->size());
+  void CheckExtensionConsistency(std::string extension_id) {
     const Extension* extension =
-        GetExtensionService()->extensions()->at(index);
+        GetExtensionService()->extensions()->GetByID(extension_id);
     ASSERT_TRUE(extension);
-    ExtensionHost* extension_host =
-        GetExtensionProcessManager()->GetBackgroundHostForExtension(extension);
+    ExtensionHost* extension_host = GetExtensionProcessManager()->
+        GetBackgroundHostForExtension(extension_id);
     ASSERT_TRUE(extension_host);
     ASSERT_TRUE(GetExtensionProcessManager()->HasExtensionHost(extension_host));
     ASSERT_TRUE(extension_host->IsRenderViewLive());
-    ASSERT_EQ(extension_host->render_view_host()->process(),
-        GetExtensionProcessManager()->GetExtensionProcess(extension->id()));
+    extensions::ProcessMap* process_map =
+        browser()->profile()->GetExtensionService()->process_map();
+    ASSERT_TRUE(process_map->Contains(
+        extension_id, extension_host->render_view_host()->process()->GetID()));
   }
 
   void LoadTestExtension() {
     ExtensionBrowserTest::SetUpInProcessBrowserTestFixture();
-    const size_t size_before = GetExtensionService()->extensions()->size();
-    ASSERT_TRUE(LoadExtension(
-        test_data_dir_.AppendASCII("common").AppendASCII("background_page")));
-    const Extension* extension = GetExtensionService()->extensions()->back();
+    const Extension* extension = LoadExtension(
+        test_data_dir_.AppendASCII("common").AppendASCII("background_page"));
     ASSERT_TRUE(extension);
     first_extension_id_ = extension->id();
-    CheckExtensionConsistency(size_before);
+    CheckExtensionConsistency(first_extension_id_);
   }
 
   void LoadSecondExtension() {
-    int offset = GetExtensionService()->extensions()->size();
-    ASSERT_TRUE(LoadExtension(
-        test_data_dir_.AppendASCII("install").AppendASCII("install")));
-    const Extension* extension =
-        GetExtensionService()->extensions()->at(offset);
+    const Extension* extension = LoadExtension(
+        test_data_dir_.AppendASCII("install").AppendASCII("install"));
     ASSERT_TRUE(extension);
     second_extension_id_ = extension->id();
-    CheckExtensionConsistency(offset);
+    CheckExtensionConsistency(second_extension_id_);
   }
 
   std::string first_extension_id_;
@@ -110,14 +124,14 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, Basic) {
   const size_t crash_size_before =
       GetExtensionService()->terminated_extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
   ASSERT_EQ(crash_size_before + 1,
             GetExtensionService()->terminated_extensions()->size());
-  AcceptInfoBar(0);
+  AcceptNotification(0);
 
-  SCOPED_TRACE("after clicking the infobar");
-  CheckExtensionConsistency(size_before);
+  SCOPED_TRACE("after clicking the balloon");
+  CheckExtensionConsistency(first_extension_id_);
   ASSERT_EQ(crash_size_before,
             GetExtensionService()->terminated_extensions()->size());
 }
@@ -127,16 +141,17 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, CloseAndReload) {
   const size_t crash_size_before =
       GetExtensionService()->terminated_extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
+
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
   ASSERT_EQ(crash_size_before + 1,
             GetExtensionService()->terminated_extensions()->size());
 
-  CancelInfoBar(0);
+  CancelNotification(0);
   ReloadExtension(first_extension_id_);
 
   SCOPED_TRACE("after reloading");
-  CheckExtensionConsistency(size_before);
+  CheckExtensionConsistency(first_extension_id_);
   ASSERT_EQ(crash_size_before,
             GetExtensionService()->terminated_extensions()->size());
 }
@@ -144,155 +159,92 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, CloseAndReload) {
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, ReloadIndependently) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 
   ReloadExtension(first_extension_id_);
 
   SCOPED_TRACE("after reloading");
-  CheckExtensionConsistency(size_before);
+  CheckExtensionConsistency(first_extension_id_);
 
-  TabContents* current_tab = browser()->GetSelectedTabContents();
+  WebContents* current_tab = browser()->GetSelectedWebContents();
   ASSERT_TRUE(current_tab);
 
-  // The infobar should automatically hide after the extension is successfully
+  // The balloon should automatically hide after the extension is successfully
   // reloaded.
-  ASSERT_EQ(0, current_tab->infobar_delegate_count());
+  ASSERT_EQ(0U, CountBalloons());
 }
 
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
                        ReloadIndependentlyChangeTabs) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 
-  TabContents* original_tab = browser()->GetSelectedTabContents();
+  WebContents* original_tab = browser()->GetSelectedWebContents();
   ASSERT_TRUE(original_tab);
-  ASSERT_EQ(1, original_tab->infobar_delegate_count());
+  ASSERT_EQ(1U, CountBalloons());
 
-  // Open a new tab so the info bar will not be in the current tab.
+  // Open a new tab, but the balloon will still be there.
   browser()->NewTab();
-  TabContents* new_current_tab = browser()->GetSelectedTabContents();
+  WebContents* new_current_tab = browser()->GetSelectedWebContents();
   ASSERT_TRUE(new_current_tab);
   ASSERT_NE(new_current_tab, original_tab);
-  ASSERT_EQ(0, new_current_tab->infobar_delegate_count());
+  ASSERT_EQ(1U, CountBalloons());
 
   ReloadExtension(first_extension_id_);
 
   SCOPED_TRACE("after reloading");
-  CheckExtensionConsistency(size_before);
+  CheckExtensionConsistency(first_extension_id_);
 
-  // The infobar should automatically hide after the extension is successfully
+  // The balloon should automatically hide after the extension is successfully
   // reloaded.
-  ASSERT_EQ(0, original_tab->infobar_delegate_count());
+  ASSERT_EQ(0U, CountBalloons());
 }
 
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
                        ReloadIndependentlyNavigatePage) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 
-  TabContents* current_tab = browser()->GetSelectedTabContents();
+  WebContents* current_tab = browser()->GetSelectedWebContents();
   ASSERT_TRUE(current_tab);
-  ASSERT_EQ(1, current_tab->infobar_delegate_count());
+  ASSERT_EQ(1U, CountBalloons());
 
   // Navigate to another page.
   ui_test_utils::NavigateToURL(browser(),
       ui_test_utils::GetTestUrl(FilePath(FilePath::kCurrentDirectory),
                                 FilePath(FILE_PATH_LITERAL("title1.html"))));
-  ASSERT_EQ(1, current_tab->infobar_delegate_count());
+  ASSERT_EQ(1U, CountBalloons());
 
   ReloadExtension(first_extension_id_);
 
   SCOPED_TRACE("after reloading");
-  CheckExtensionConsistency(size_before);
+  CheckExtensionConsistency(first_extension_id_);
 
-  // The infobar should automatically hide after the extension is successfully
+  // The balloon should automatically hide after the extension is successfully
   // reloaded.
-  ASSERT_EQ(0, current_tab->infobar_delegate_count());
-}
-
-IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
-                       ReloadIndependentlyTwoInfoBars) {
-  const size_t size_before = GetExtensionService()->extensions()->size();
-  LoadTestExtension();
-
-  // Open a new window so that there will be an info bar in each.
-  Browser* browser2 = CreateBrowser(browser()->profile());
-
-  CrashExtension(size_before);
-  ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
-
-  TabContents* current_tab = browser()->GetSelectedTabContents();
-  ASSERT_TRUE(current_tab);
-  ASSERT_EQ(1, current_tab->infobar_delegate_count());
-
-  TabContents* current_tab2 = browser2->GetSelectedTabContents();
-  ASSERT_TRUE(current_tab2);
-  ASSERT_EQ(1, current_tab2->infobar_delegate_count());
-
-  ReloadExtension(first_extension_id_);
-
-  SCOPED_TRACE("after reloading");
-  CheckExtensionConsistency(size_before);
-
-  // Both infobars should automatically hide after the extension is successfully
-  // reloaded.
-  ASSERT_EQ(0, current_tab->infobar_delegate_count());
-  ASSERT_EQ(0, current_tab2->infobar_delegate_count());
-}
-
-IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
-                       ReloadIndependentlyTwoInfoBarsSameBrowser) {
-  const size_t size_before = GetExtensionService()->extensions()->size();
-  LoadTestExtension();
-
-  // Open a new window so that there will be an info bar in each.
-  Browser* browser2 = CreateBrowser(browser()->profile());
-
-  CrashExtension(size_before);
-  ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
-
-  TabContents* current_tab = browser()->GetSelectedTabContents();
-  ASSERT_TRUE(current_tab);
-  ASSERT_EQ(1, current_tab->infobar_delegate_count());
-
-  TabContents* current_tab2 = browser2->GetSelectedTabContents();
-  ASSERT_TRUE(current_tab2);
-  ASSERT_EQ(1, current_tab2->infobar_delegate_count());
-
-  // Move second window into first browser so there will be multiple tabs
-  // with the info bar for the same extension in one browser.
-  TabContentsWrapper* contents =
-      browser2->tabstrip_model()->DetachTabContentsAt(0);
-  browser()->tabstrip_model()->AppendTabContents(contents, true);
-  current_tab2 = browser()->GetSelectedTabContents();
-  ASSERT_EQ(1, current_tab2->infobar_delegate_count());
-  ASSERT_NE(current_tab2, current_tab);
-
-  ReloadExtension(first_extension_id_);
-
-  SCOPED_TRACE("after reloading");
-  CheckExtensionConsistency(size_before);
-
-  // Both infobars should automatically hide after the extension is successfully
-  // reloaded.
-  ASSERT_EQ(0, current_tab2->infobar_delegate_count());
-  browser()->SelectPreviousTab();
-  ASSERT_EQ(current_tab, browser()->GetSelectedTabContents());
-  ASSERT_EQ(0, current_tab->infobar_delegate_count());
+  ASSERT_EQ(0U, CountBalloons());
 }
 
 // Make sure that when we don't do anything about the crashed extension
 // and close the browser, it doesn't crash. The browser is closed implicitly
 // at the end of each browser test.
-IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, ShutdownWhileCrashed) {
+//
+// http://crbug.com/84719
+#if defined(OS_LINUX)
+#define MAYBE_ShutdownWhileCrashed DISABLED_ShutdownWhileCrashed
+#else
+#define MAYBE_ShutdownWhileCrashed ShutdownWhileCrashed
+#endif  // defined(OS_LINUX)
+
+IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, MAYBE_ShutdownWhileCrashed) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 }
 
@@ -300,26 +252,26 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, TwoExtensionsCrashFirst) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
   LoadSecondExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
-  AcceptInfoBar(0);
+  AcceptNotification(0);
 
-  SCOPED_TRACE("after clicking the infobar");
-  CheckExtensionConsistency(size_before);
-  CheckExtensionConsistency(size_before + 1);
+  SCOPED_TRACE("after clicking the balloon");
+  CheckExtensionConsistency(first_extension_id_);
+  CheckExtensionConsistency(second_extension_id_);
 }
 
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, TwoExtensionsCrashSecond) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
   LoadSecondExtension();
-  CrashExtension(size_before + 1);
+  CrashExtension(second_extension_id_);
   ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
-  AcceptInfoBar(0);
+  AcceptNotification(0);
 
-  SCOPED_TRACE("after clicking the infobar");
-  CheckExtensionConsistency(size_before);
-  CheckExtensionConsistency(size_before + 1);
+  SCOPED_TRACE("after clicking the balloon");
+  CheckExtensionConsistency(first_extension_id_);
+  CheckExtensionConsistency(second_extension_id_);
 }
 
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
@@ -329,63 +281,72 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
       GetExtensionService()->terminated_extensions()->size();
   LoadTestExtension();
   LoadSecondExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
   ASSERT_EQ(crash_size_before + 1,
             GetExtensionService()->terminated_extensions()->size());
-  CrashExtension(size_before);
+  CrashExtension(second_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
   ASSERT_EQ(crash_size_before + 2,
             GetExtensionService()->terminated_extensions()->size());
 
   {
-    SCOPED_TRACE("first infobar");
-    AcceptInfoBar(0);
-    CheckExtensionConsistency(size_before);
+    SCOPED_TRACE("first balloon");
+    AcceptNotification(0);
+    CheckExtensionConsistency(first_extension_id_);
   }
 
   {
-    SCOPED_TRACE("second infobar");
-    AcceptInfoBar(0);
-    CheckExtensionConsistency(size_before);
-    CheckExtensionConsistency(size_before + 1);
+    SCOPED_TRACE("second balloon");
+    AcceptNotification(0);
+    CheckExtensionConsistency(first_extension_id_);
+    CheckExtensionConsistency(second_extension_id_);
   }
 }
 
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, TwoExtensionsOneByOne) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
   LoadSecondExtension();
-  CrashExtension(size_before);
+  CrashExtension(second_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 
   {
-    SCOPED_TRACE("first infobar");
-    AcceptInfoBar(0);
-    CheckExtensionConsistency(size_before);
+    SCOPED_TRACE("first balloon");
+    AcceptNotification(0);
+    CheckExtensionConsistency(first_extension_id_);
   }
 
   {
-    SCOPED_TRACE("second infobar");
-    AcceptInfoBar(0);
-    CheckExtensionConsistency(size_before);
-    CheckExtensionConsistency(size_before + 1);
+    SCOPED_TRACE("second balloon");
+    AcceptNotification(0);
+    CheckExtensionConsistency(first_extension_id_);
+    CheckExtensionConsistency(second_extension_id_);
   }
 }
 
+// http://crbug.com/84719
+#if defined(OS_LINUX)
+#define MAYBE_TwoExtensionsShutdownWhileCrashed \
+    DISABLED_TwoExtensionsShutdownWhileCrashed
+#else
+#define MAYBE_TwoExtensionsShutdownWhileCrashed \
+    TwoExtensionsShutdownWhileCrashed
+#endif  // defined(OS_LINUX)
+
 // Make sure that when we don't do anything about the crashed extensions
 // and close the browser, it doesn't crash. The browser is closed implicitly
 // at the end of each browser test.
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
-                       TwoExtensionsShutdownWhileCrashed) {
+                       MAYBE_TwoExtensionsShutdownWhileCrashed) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
   LoadSecondExtension();
-  CrashExtension(size_before);
+  CrashExtension(second_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 }
 
@@ -394,17 +355,20 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
   LoadSecondExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
-  CrashExtension(size_before);
+  CrashExtension(second_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 
-  CancelInfoBar(0);
-  AcceptInfoBar(1);
+  // Accept notification 1 before canceling notification 0.
+  // Otherwise, on Linux and Windows, there is a race here, in which
+  // canceled notifications do not immediately go away.
+  AcceptNotification(1);
+  CancelNotification(0);
 
-  SCOPED_TRACE("infobars done");
+  SCOPED_TRACE("balloons done");
   ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
-  CheckExtensionConsistency(size_before);
+  CheckExtensionConsistency(second_extension_id_);
 }
 
 IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
@@ -412,38 +376,67 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
   const size_t size_before = GetExtensionService()->extensions()->size();
   LoadTestExtension();
   LoadSecondExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
-  CrashExtension(size_before);
+  CrashExtension(second_extension_id_);
   ASSERT_EQ(size_before, GetExtensionService()->extensions()->size());
 
   {
     SCOPED_TRACE("first: reload");
-    TabContents* current_tab = browser()->GetSelectedTabContents();
+    WebContents* current_tab = browser()->GetSelectedWebContents();
     ASSERT_TRUE(current_tab);
-    // At the beginning we should have one infobar displayed for each extension.
-    ASSERT_EQ(2, current_tab->infobar_delegate_count());
+    // At the beginning we should have one balloon displayed for each extension.
+    ASSERT_EQ(2U, CountBalloons());
     ReloadExtension(first_extension_id_);
-    // One of the infobars should hide after the extension is reloaded.
-    ASSERT_EQ(1, current_tab->infobar_delegate_count());
-    CheckExtensionConsistency(size_before);
+    // One of the balloons should hide after the extension is reloaded.
+    ASSERT_EQ(1U, CountBalloons());
+    CheckExtensionConsistency(first_extension_id_);
   }
 
   {
-    SCOPED_TRACE("second: infobar");
-    AcceptInfoBar(0);
-    CheckExtensionConsistency(size_before);
-    CheckExtensionConsistency(size_before + 1);
+    SCOPED_TRACE("second: balloon");
+    AcceptNotification(0);
+    CheckExtensionConsistency(first_extension_id_);
+    CheckExtensionConsistency(second_extension_id_);
   }
 }
 
-IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, CrashAndUnloadAll) {
+IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, CrashAndUninstall) {
   const size_t size_before = GetExtensionService()->extensions()->size();
   const size_t crash_size_before =
       GetExtensionService()->terminated_extensions()->size();
   LoadTestExtension();
   LoadSecondExtension();
-  CrashExtension(size_before);
+  CrashExtension(first_extension_id_);
+  ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
+  ASSERT_EQ(crash_size_before + 1,
+            GetExtensionService()->terminated_extensions()->size());
+
+  ASSERT_EQ(1U, CountBalloons());
+  UninstallExtension(first_extension_id_);
+  MessageLoop::current()->RunAllPending();
+
+  SCOPED_TRACE("after uninstalling");
+  ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
+  ASSERT_EQ(crash_size_before,
+            GetExtensionService()->terminated_extensions()->size());
+  ASSERT_EQ(0U, CountBalloons());
+}
+
+// http://crbug.com/84719
+#if defined(OS_LINUX)
+#define MAYBE_CrashAndUnloadAll DISABLED_CrashAndUnloadAll
+#else
+#define MAYBE_CrashAndUnloadAll CrashAndUnloadAll
+#endif  // defined(OS_LINUX)
+
+IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, MAYBE_CrashAndUnloadAll) {
+  const size_t size_before = GetExtensionService()->extensions()->size();
+  const size_t crash_size_before =
+      GetExtensionService()->terminated_extensions()->size();
+  LoadTestExtension();
+  LoadSecondExtension();
+  CrashExtension(first_extension_id_);
   ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
   ASSERT_EQ(crash_size_before + 1,
             GetExtensionService()->terminated_extensions()->size());
@@ -452,3 +445,45 @@ IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest, CrashAndUnloadAll) {
   ASSERT_EQ(crash_size_before,
             GetExtensionService()->terminated_extensions()->size());
 }
+
+// Test that when an extension with a background page that has a tab open
+// crashes, the tab stays open, and reloading it reloads the extension.
+// Regression test for issue 71629.
+IN_PROC_BROWSER_TEST_F(ExtensionCrashRecoveryTest,
+                       ReloadTabsWithBackgroundPage) {
+  TabStripModel* tab_strip = browser()->tabstrip_model();
+  const size_t size_before = GetExtensionService()->extensions()->size();
+  const size_t crash_size_before =
+      GetExtensionService()->terminated_extensions()->size();
+  LoadTestExtension();
+
+  // Open a tab extension.
+  browser()->NewTab();
+  ui_test_utils::NavigateToURL(
+      browser(),
+      GURL("chrome-extension://" + first_extension_id_ + "/background.html"));
+
+  const int tabs_before = tab_strip->count();
+  CrashExtension(first_extension_id_);
+
+  // Tab should still be open, and extension should be crashed.
+  EXPECT_EQ(tabs_before, tab_strip->count());
+  EXPECT_EQ(size_before, GetExtensionService()->extensions()->size());
+  EXPECT_EQ(crash_size_before + 1,
+            GetExtensionService()->terminated_extensions()->size());
+
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        content::Source<NavigationController>(
+            &browser()->GetSelectedTabContentsWrapper()->web_contents()->
+                GetController()));
+    browser()->Reload(CURRENT_TAB);
+    observer.Wait();
+  }
+  // Extension should now be loaded.
+  SCOPED_TRACE("after reloading the tab");
+  CheckExtensionConsistency(first_extension_id_);
+  ASSERT_EQ(size_before + 1, GetExtensionService()->extensions()->size());
+  ASSERT_EQ(0U, CountBalloons());
+}
