@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,15 +9,20 @@
 #include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/values.h"
-#include "chrome/browser/debugger/devtools_manager.h"
 #include "chrome/browser/extensions/extension_devtools_events.h"
 #include "chrome/browser/extensions/extension_devtools_manager.h"
 #include "chrome/browser/extensions/extension_event_router.h"
-#include "chrome/browser/extensions/extension_tabs_module.h"
+#include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
-#include "chrome/common/devtools_messages.h"
+#include "content/public/browser/devtools_agent_host_registry.h"
+#include "content/public/browser/devtools_manager.h"
+#include "content/public/browser/web_contents.h"
+
+using content::DevToolsAgentHost;
+using content::DevToolsAgentHostRegistry;
+using content::DevToolsManager;
+using content::WebContents;
 
 ExtensionDevToolsBridge::ExtensionDevToolsBridge(int tab_id,
                                                  Profile* profile)
@@ -34,16 +39,11 @@ ExtensionDevToolsBridge::ExtensionDevToolsBridge(int tab_id,
 ExtensionDevToolsBridge::~ExtensionDevToolsBridge() {
 }
 
-static std::string FormatDevToolsMessage(int seq,
-                                         const std::string& domain,
-                                         const std::string& command,
-                                         DictionaryValue* arguments) {
-
+static std::string FormatDevToolsMessage(int id, const std::string& method) {
   DictionaryValue message;
-  message.SetInteger("seq", seq);
-  message.SetString("domain", domain);
-  message.SetString("command", command);
-  message.Set("arguments", arguments);
+  message.SetInteger("id", id);
+  message.SetString("method", method);
+  message.Set("params", new DictionaryValue);
 
   std::string json;
   base::JSONWriter::Write(&message, false, &json);
@@ -61,51 +61,31 @@ bool ExtensionDevToolsBridge::RegisterAsDevToolsClientHost() {
                                    &browser, &tab_strip,
                                    &contents, &tab_index)) {
     DevToolsManager* devtools_manager = DevToolsManager::GetInstance();
-    if (devtools_manager->GetDevToolsClientHostFor(contents->
-            render_view_host()) != NULL)
+    DevToolsAgentHost* agent = DevToolsAgentHostRegistry::GetDevToolsAgentHost(
+        contents->web_contents()->GetRenderViewHost());
+    if (devtools_manager->GetDevToolsClientHostFor(agent))
       return false;
 
-    devtools_manager->RegisterDevToolsClientHostFor(
-        contents->render_view_host(), this);
+    devtools_manager->RegisterDevToolsClientHostFor(agent, this);
 
     // Following messages depend on inspector protocol that is not yet
     // finalized.
 
-    // 1. Set injected script content.
-    DictionaryValue* arguments = new DictionaryValue();
-    arguments->SetString("scriptSource", "'{}'");
-    devtools_manager->ForwardToDevToolsAgent(
+    // 1. Start timeline profiler.
+    devtools_manager->DispatchOnInspectorBackend(
         this,
-        DevToolsAgentMsg_DispatchOnInspectorBackend(
-            FormatDevToolsMessage(0,
-                                  "Inspector",
-                                  "setInjectedScriptSource",
-                                  arguments)));
+        FormatDevToolsMessage(2, "Timeline.start"));
 
-    // 2. Report front-end is loaded.
-    devtools_manager->ForwardToDevToolsAgent(
+    // 2. Enable network resource tracking.
+    devtools_manager->DispatchOnInspectorBackend(
         this,
-        DevToolsAgentMsg_FrontendLoaded());
+        FormatDevToolsMessage(3, "Network.enable"));
 
-    // 3. Do not break on exceptions.
-    arguments = new DictionaryValue();
-    arguments->SetInteger("pauseOnExceptionsState", 0);
-    devtools_manager->ForwardToDevToolsAgent(
+    // 3. Enable page events.
+    devtools_manager->DispatchOnInspectorBackend(
         this,
-        DevToolsAgentMsg_DispatchOnInspectorBackend(
-            FormatDevToolsMessage(1,
-                                  "Debugger",
-                                  "setPauseOnExceptionsState",
-                                  arguments)));
+        FormatDevToolsMessage(4, "Page.enable"));
 
-    // 4. Start timeline profiler.
-    devtools_manager->ForwardToDevToolsAgent(
-        this,
-        DevToolsAgentMsg_DispatchOnInspectorBackend(
-            FormatDevToolsMessage(2,
-                                  "Inspector",
-                                  "startTimelineProfiler",
-                                  new DictionaryValue())));
     return true;
   }
   return false;
@@ -113,8 +93,7 @@ bool ExtensionDevToolsBridge::RegisterAsDevToolsClientHost() {
 
 void ExtensionDevToolsBridge::UnregisterAsDevToolsClientHost() {
   DCHECK_EQ(MessageLoop::current()->type(), MessageLoop::TYPE_UI);
-
-  NotifyCloseListener();
+  DevToolsManager::GetInstance()->ClientHostClosing(this);
 }
 
 // If the tab we are looking at is going away then we fire a closing event at
@@ -132,15 +111,7 @@ void ExtensionDevToolsBridge::InspectedTabClosing() {
   extension_devtools_manager_->BridgeClosingForTab(tab_id_);
 }
 
-void ExtensionDevToolsBridge::SendMessageToClient(const IPC::Message& msg) {
-  IPC_BEGIN_MESSAGE_MAP(ExtensionDevToolsBridge, msg)
-    IPC_MESSAGE_HANDLER(DevToolsClientMsg_DispatchOnInspectorFrontend,
-                        OnDispatchOnInspectorFrontend);
-    IPC_MESSAGE_UNHANDLED_ERROR()
-  IPC_END_MESSAGE_MAP()
-}
-
-void ExtensionDevToolsBridge::OnDispatchOnInspectorFrontend(
+void ExtensionDevToolsBridge::DispatchOnInspectorFrontend(
     const std::string& data) {
   DCHECK_EQ(MessageLoop::current()->type(), MessageLoop::TYPE_UI);
 
@@ -148,3 +119,8 @@ void ExtensionDevToolsBridge::OnDispatchOnInspectorFrontend(
   profile_->GetExtensionEventRouter()->DispatchEventToRenderers(
       on_page_event_name_, json, profile_, GURL());
 }
+
+void ExtensionDevToolsBridge::TabReplaced(WebContents* new_tab) {
+  // We don't update the tab id as it needs to remain the same so that we can
+  // properly unregister.
+}
