@@ -3,17 +3,26 @@
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/extension_apitest.h"
-#include "chrome/browser/geolocation/location_arbitrator.h"
-#include "chrome/browser/geolocation/mock_location_provider.h"
+#include "content/test/mock_geolocation.h"
 
 class GeolocationApiTest : public ExtensionApiTest {
  public:
-  // InProcessBrowserTest
-  void SetUpInProcessBrowserTestFixture() {
-    ExtensionApiTest::SetUpInProcessBrowserTestFixture();
-    GeolocationArbitrator::SetProviderFactoryForTest(
-        &NewAutoSuccessMockLocationProvider);
+  GeolocationApiTest() {
   }
+
+  // InProcessBrowserTest
+  virtual void SetUpInProcessBrowserTestFixture() {
+    ExtensionApiTest::SetUpInProcessBrowserTestFixture();
+    mock_geolocation_.Setup();
+  }
+
+  // InProcessBrowserTest
+  virtual void TearDownInProcessBrowserTestFixture() {
+    mock_geolocation_.TearDown();
+  }
+
+ private:
+  content::MockGeolocation mock_geolocation_;
 };
 
 IN_PROC_BROWSER_TEST_F(GeolocationApiTest,
