@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,8 +9,8 @@
 #include "chrome/browser/extensions/extension_function.h"
 
 class GetAcceptLanguagesFunction : public SyncExtensionFunction {
-  ~GetAcceptLanguagesFunction() {}
-  virtual bool RunImpl();
+  virtual ~GetAcceptLanguagesFunction() {}
+  virtual bool RunImpl() OVERRIDE;
   DECLARE_EXTENSION_FUNCTION_NAME("i18n.getAcceptLanguages")
 };
 
