@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,8 +12,8 @@
 
 #include "base/basictypes.h"
 #include "chrome/browser/extensions/image_loading_tracker.h"
-#include "gfx/insets.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "ui/gfx/insets.h"
 
 class Extension;
 
@@ -25,7 +25,7 @@ class ExtensionIconManager : public ImageLoadingTracker::Observer {
   // Start loading the icon for the given extension.
   void LoadIcon(const Extension* extension);
 
-  // This returns a bitmap of width/height kFavIconSize, loaded either from an
+  // This returns a bitmap of width/height kFaviconSize, loaded either from an
   // entry specified in the extension's 'icon' section of the manifest, or a
   // default extension icon.
   const SkBitmap& GetIcon(const std::string& extension_id);
@@ -34,8 +34,8 @@ class ExtensionIconManager : public ImageLoadingTracker::Observer {
   void RemoveIcon(const std::string& extension_id);
 
   // Implements the ImageLoadingTracker::Observer interface.
-  virtual void OnImageLoaded(SkBitmap* image, ExtensionResource resource,
-                             int index);
+  virtual void OnImageLoaded(SkBitmap* image, const ExtensionResource& resource,
+                             int index) OVERRIDE;
 
   void set_monochrome(bool value) { monochrome_ = value; }
   void set_padding(const gfx::Insets& value) { padding_ = value; }
