@@ -1,21 +1,23 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_window.h"
 #include "chrome/browser/extensions/browser_action_test_util.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/extension_test_message_listener.h"
 #include "chrome/browser/extensions/user_script_master.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/public/browser/web_contents.h"
 #include "net/base/mock_host_resolver.h"
 
+using content::WebContents;
+
 IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoNoScript) {
   ASSERT_TRUE(StartTestServer());
 
@@ -29,15 +31,14 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoNoScript) {
       browser()->profile(),
       test_server()->GetURL("files/extensions/test_file.html"));
 
-  Browser* otr_browser = BrowserList::FindBrowserWithType(
-      browser()->profile()->GetOffTheRecordProfile(), Browser::TYPE_NORMAL,
-      false);
-  TabContents* tab = otr_browser->GetSelectedTabContents();
+  Browser* otr_browser = BrowserList::FindTabbedBrowser(
+      browser()->profile()->GetOffTheRecordProfile(), false);
+  WebContents* tab = otr_browser->GetSelectedWebContents();
 
   // Verify the script didn't run.
   bool result = false;
   ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-      tab->render_view_host(), L"",
+      tab->GetRenderViewHost(), L"",
       L"window.domAutomationController.send(document.title == 'Unmodified')",
       &result));
   EXPECT_TRUE(result);
@@ -67,15 +68,14 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoYesScript) {
       browser()->profile(),
       test_server()->GetURL("files/extensions/test_file.html"));
 
-  Browser* otr_browser = BrowserList::FindBrowserWithType(
-      browser()->profile()->GetOffTheRecordProfile(), Browser::TYPE_NORMAL,
-      false);
-  TabContents* tab = otr_browser->GetSelectedTabContents();
+  Browser* otr_browser = BrowserList::FindTabbedBrowser(
+      browser()->profile()->GetOffTheRecordProfile(), false);
+  WebContents* tab = otr_browser->GetSelectedWebContents();
 
   // Verify the script ran.
   bool result = false;
   ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-      tab->render_view_host(), L"",
+      tab->GetRenderViewHost(), L"",
       L"window.domAutomationController.send(document.title == 'modified')",
       &result));
   EXPECT_TRUE(result);
@@ -83,14 +83,14 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoYesScript) {
 
 // Tests that an extension which is enabled for incognito mode doesn't
 // accidentially create and incognito profile.
-IN_PROC_BROWSER_TEST_F(ExtensionApiTest, DontCreateIncognitoProfile) {
+// Test disabled due to http://crbug.com/89054.
+IN_PROC_BROWSER_TEST_F(ExtensionApiTest, DISABLED_DontCreateIncognitoProfile) {
   ASSERT_FALSE(browser()->profile()->HasOffTheRecordProfile());
   ASSERT_TRUE(RunExtensionTestIncognito(
       "incognito/dont_create_profile")) << message_;
   ASSERT_FALSE(browser()->profile()->HasOffTheRecordProfile());
 }
 
-// Tests that the APIs in an incognito-enabled extension work properly.
 IN_PROC_BROWSER_TEST_F(ExtensionApiTest, Incognito) {
   host_resolver()->AddRule("*", "127.0.0.1");
   ASSERT_TRUE(StartTestServer());
@@ -108,16 +108,9 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, Incognito) {
   EXPECT_TRUE(catcher.GetNextResult()) << catcher.message();
 }
 
-// Hangs flakily on mac and linux: http://crbug.com/53991
-#if defined(OS_MACOSX) || defined(OS_LINUX)
-#define MAYBE_IncognitoSplitMode DISABLED_IncognitoSplitMode
-#else
-#define MAYBE_IncognitoSplitMode IncognitoSplitMode
-#endif
-
 // Tests that the APIs in an incognito-enabled split-mode extension work
 // properly.
-IN_PROC_BROWSER_TEST_F(ExtensionApiTest, MAYBE_IncognitoSplitMode) {
+IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoSplitMode) {
   host_resolver()->AddRule("*", "127.0.0.1");
   ASSERT_TRUE(StartTestServer());
 
@@ -129,6 +122,9 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, MAYBE_IncognitoSplitMode) {
   catcher_incognito.RestrictToProfile(
       browser()->profile()->GetOffTheRecordProfile());
 
+  ExtensionTestMessageListener listener("waiting", true);
+  ExtensionTestMessageListener listener_incognito("waiting_incognito", true);
+
   // Open incognito window and navigate to test page.
   ui_test_utils::OpenURLOffTheRecord(
       browser()->profile(),
@@ -138,9 +134,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, MAYBE_IncognitoSplitMode) {
       .AppendASCII("incognito").AppendASCII("split")));
 
   // Wait for both extensions to be ready before telling them to proceed.
-  ExtensionTestMessageListener listener("waiting", true);
   EXPECT_TRUE(listener.WaitUntilSatisfied());
-  ExtensionTestMessageListener listener_incognito("waiting", true);
   EXPECT_TRUE(listener_incognito.WaitUntilSatisfied());
   listener.Reply("go");
   listener_incognito.Reply("go");
@@ -168,8 +162,14 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoDisabled) {
   EXPECT_TRUE(catcher.GetNextResult()) << catcher.message();
 }
 
+#if defined(OS_WIN)
+// http://crbug.com/104438.
+#define MAYBE_IncognitoPopup DISABLED_IncognitoPopup
+#else
+#define MAYBE_IncognitoPopup IncognitoPopup
+#endif
 // Test that opening a popup from an incognito browser window works properly.
-IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoPopup) {
+IN_PROC_BROWSER_TEST_F(ExtensionApiTest, MAYBE_IncognitoPopup) {
   host_resolver()->AddRule("*", "127.0.0.1");
   ASSERT_TRUE(StartTestServer());
 
@@ -183,9 +183,8 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, IncognitoPopup) {
       browser()->profile(),
       test_server()->GetURL("files/extensions/test_file.html"));
 
-  Browser* incognito_browser = BrowserList::FindBrowserWithType(
-      browser()->profile()->GetOffTheRecordProfile(), Browser::TYPE_NORMAL,
-      false);
+  Browser* incognito_browser = BrowserList::FindTabbedBrowser(
+      browser()->profile()->GetOffTheRecordProfile(), false);
 
   // Simulate the incognito's browser action being clicked.
   BrowserActionTestUtil(incognito_browser).Press(0);
