@@ -6,17 +6,20 @@
 #define CHROME_BROWSER_EXTENSIONS_EXTENSION_INFOBAR_DELEGATE_H_
 #pragma once
 
-#include "chrome/browser/tab_contents/infobar_delegate.h"
+#include "base/memory/scoped_ptr.h"
+#include "chrome/browser/tab_contents/confirm_infobar_delegate.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 class Browser;
 class Extension;
 class ExtensionHost;
-class TabContents;
+class GURL;
 
 // The InfobarDelegate for creating and managing state for the ExtensionInfobar
 // plus monitor when the extension goes away.
 class ExtensionInfoBarDelegate : public InfoBarDelegate,
-                                 public NotificationObserver {
+                                 public content::NotificationObserver {
  public:
   // The observer for when the delegate dies.
   class DelegateObserver {
@@ -28,31 +31,35 @@ class ExtensionInfoBarDelegate : public InfoBarDelegate,
   };
 
   ExtensionInfoBarDelegate(Browser* browser,
-                           TabContents* contents,
+                           InfoBarTabHelper* infobar_helper,
                            const Extension* extension,
-                           const GURL& url);
+                           const GURL& url,
+                           int height);
+
   const Extension* extension() { return extension_; }
   ExtensionHost* extension_host() { return extension_host_.get(); }
+  int height() { return height_; }
 
   void set_observer(DelegateObserver* observer) { observer_ = observer; }
 
-  bool closing() { return closing_; }
+  bool closing() const { return closing_; }
 
  private:
   virtual ~ExtensionInfoBarDelegate();
 
   // InfoBarDelegate:
-  virtual InfoBar* CreateInfoBar();
-  virtual bool EqualsDelegate(InfoBarDelegate* delegate) const;
-  virtual void InfoBarDismissed();
-  virtual void InfoBarClosed();
-  virtual Type GetInfoBarType() const;
-  virtual ExtensionInfoBarDelegate* AsExtensionInfoBarDelegate();
+  virtual InfoBar* CreateInfoBar(InfoBarTabHelper* owner) OVERRIDE;
+  virtual bool EqualsDelegate(InfoBarDelegate* delegate) const OVERRIDE;
+  virtual void InfoBarDismissed() OVERRIDE;
+  virtual Type GetInfoBarType() const OVERRIDE;
+  virtual ExtensionInfoBarDelegate* AsExtensionInfoBarDelegate() OVERRIDE;
 
-  // NotificationObserver:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
+
+  Browser* browser_;
 
   // The extension host we are showing the InfoBar for. The delegate needs to
   // own this since the InfoBar gets deleted and recreated when you switch tabs
@@ -64,8 +71,10 @@ class ExtensionInfoBarDelegate : public InfoBarDelegate,
   DelegateObserver* observer_;
 
   const Extension* extension_;
-  TabContents* tab_contents_;
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
+
+  // The requested height of the infobar (in pixels).
+  int height_;
 
   // Whether we are currently animating to close. This is used to ignore
   // ExtensionView::PreferredSizeChanged notifications.
