@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,20 +12,24 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "base/gtest_prod_util.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string16.h"
 #include "chrome/browser/extensions/extension_icon_manager.h"
-#include "chrome/common/extensions/extension_extent.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "chrome/common/extensions/url_pattern_set.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 struct ContextMenuParams;
 
 class Extension;
 class Profile;
 class SkBitmap;
-class TabContents;
+
+namespace content {
+class WebContents;
+}
 
 // Represents a menu item added by an extension.
 class ExtensionMenuItem {
@@ -36,7 +40,7 @@ class ExtensionMenuItem {
   // An Id uniquely identifies a context menu item registered by an extension.
   struct Id {
     Id();
-    Id(Profile* profile, std::string extension_id, int uid);
+    Id(Profile* profile, const std::string& extension_id, int uid);
     ~Id();
 
     bool operator==(const Id& other) const;
@@ -58,6 +62,7 @@ class ExtensionMenuItem {
     IMAGE = 32,
     VIDEO = 64,
     AUDIO = 128,
+    FRAME = 256,
   };
 
   // An item can be only one of these types.
@@ -99,7 +104,10 @@ class ExtensionMenuItem {
     uint32 value_;  // A bitmask of Context values.
   };
 
-  ExtensionMenuItem(const Id& id, std::string title, bool checked, Type type,
+  ExtensionMenuItem(const Id& id,
+                    const std::string& title,
+                    bool checked,
+                    Type type,
                     const ContextList& contexts);
   virtual ~ExtensionMenuItem();
 
@@ -113,21 +121,21 @@ class ExtensionMenuItem {
   ContextList contexts() const { return contexts_; }
   Type type() const { return type_; }
   bool checked() const { return checked_; }
-  const ExtensionExtent& document_url_patterns() const {
+  const URLPatternSet& document_url_patterns() const {
     return document_url_patterns_;
   }
-  const ExtensionExtent& target_url_patterns() const {
+  const URLPatternSet& target_url_patterns() const {
     return target_url_patterns_;
   }
 
   // Simple mutator methods.
-  void set_title(std::string new_title) { title_ = new_title; }
+  void set_title(const std::string& new_title) { title_ = new_title; }
   void set_contexts(ContextList contexts) { contexts_ = contexts; }
   void set_type(Type type) { type_ = type; }
-  void set_document_url_patterns(const ExtensionExtent& patterns) {
+  void set_document_url_patterns(const URLPatternSet& patterns) {
     document_url_patterns_ = patterns;
   }
-  void set_target_url_patterns(const ExtensionExtent& patterns) {
+  void set_target_url_patterns(const URLPatternSet& patterns) {
     target_url_patterns_ = patterns;
   }
 
@@ -174,11 +182,11 @@ class ExtensionMenuItem {
 
   // Patterns for restricting what documents this item will appear for. This
   // applies to the frame where the click took place.
-  ExtensionExtent document_url_patterns_;
+  URLPatternSet document_url_patterns_;
 
   // Patterns for restricting where items appear based on the src/href
   // attribute of IMAGE/AUDIO/VIDEO/LINK tags.
-  ExtensionExtent target_url_patterns_;
+  URLPatternSet target_url_patterns_;
 
   // Any children this item may have.
   List children_;
@@ -187,13 +195,9 @@ class ExtensionMenuItem {
 };
 
 // This class keeps track of menu items added by extensions.
-class ExtensionMenuManager : public NotificationObserver {
+class ExtensionMenuManager : public content::NotificationObserver {
  public:
-  // A bitmask of values from URLPattern::SchemeMasks indicating the schemes
-  // of pages where we'll show extension menu items.
-  static const int kAllowedSchemes;
-
-  ExtensionMenuManager();
+  explicit ExtensionMenuManager(Profile* profile);
   virtual ~ExtensionMenuManager();
 
   // Returns the ids of extensions which have menu items registered.
@@ -230,28 +234,30 @@ class ExtensionMenuManager : public NotificationObserver {
   bool RemoveContextMenuItem(const ExtensionMenuItem::Id& id);
 
   // Removes all items for the given extension id.
-  void RemoveAllContextItems(std::string extension_id);
+  void RemoveAllContextItems(const std::string& extension_id);
 
   // Returns the item with the given |id| or NULL.
   ExtensionMenuItem* GetItemById(const ExtensionMenuItem::Id& id) const;
 
+  // Notify the ExtensionMenuManager that an item has been updated not through
+  // an explicit call into ExtensionMenuManager. For example, if an item is
+  // acquired by a call to GetItemById and changed, then this should be called.
+  // Returns true if the item was found or false otherwise.
+  bool ItemUpdated(const ExtensionMenuItem::Id& id);
+
   // Called when a menu item is clicked on by the user.
-  void ExecuteCommand(Profile* profile, TabContents* tab_contents,
+  void ExecuteCommand(Profile* profile, content::WebContents* web_contents,
                       const ContextMenuParams& params,
                       const ExtensionMenuItem::Id& menuItemId);
 
-  // This returns a bitmap of width/height kFavIconSize, loaded either from an
+  // This returns a bitmap of width/height kFaviconSize, loaded either from an
   // entry specified in the extension's 'icon' section of the manifest, or a
   // default extension icon.
   const SkBitmap& GetIconForExtension(const std::string& extension_id);
 
-  // Implements the NotificationObserver interface.
-  virtual void Observe(NotificationType type, const NotificationSource& source,
-                       const NotificationDetails& details);
-
-  // Returns true if |url| has an allowed scheme for extension context menu
-  // items. This checks against kAllowedSchemes.
-  static bool HasAllowedScheme(const GURL& url);
+  // Implements the content::NotificationObserver interface.
+  virtual void Observe(int type, const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
  private:
   FRIEND_TEST_ALL_PREFIXES(ExtensionMenuManagerTest, DeleteParent);
@@ -261,6 +267,12 @@ class ExtensionMenuManager : public NotificationObserver {
   // items in the same group (i.e. that are adjacent in the list).
   void RadioItemSelected(ExtensionMenuItem* item);
 
+  // Make sure that there is only one radio item selected at once in any run.
+  // If there are no radio items selected, then the first item in the run
+  // will get selected. If there are multiple radio items selected, then only
+  // the last one will get selcted.
+  void SanitizeRadioList(const ExtensionMenuItem::List& item_list);
+
   // Returns true if item is a descendant of an item with id |ancestor_id|.
   bool DescendantOf(ExtensionMenuItem* item,
                     const ExtensionMenuItem::Id& ancestor_id);
@@ -274,7 +286,7 @@ class ExtensionMenuManager : public NotificationObserver {
   // items.
   std::map<ExtensionMenuItem::Id, ExtensionMenuItem*> items_by_id_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   ExtensionIconManager icon_manager_;
 
