@@ -1,63 +1,28 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/extension_metrics_module.h"
 
+#include <algorithm>
+
 #include "base/metrics/histogram.h"
-#include "base/values.h"
 #include "chrome/common/extensions/extension.h"
-#include "chrome/browser/metrics/user_metrics.h"
-#include "chrome/browser/ui/options/options_util.h"
-#include "chrome/installer/util/google_update_settings.h"
+#include "content/public/browser/user_metrics.h"
 
 using base::Histogram;
 using base::LinearHistogram;
+using content::UserMetricsAction;
 
-namespace {
-
-// Build the full name of a metrics for the given extension.  Each metric
-// is made up of the unique name within the extension followed by the
-// extension's id.  This keeps the metrics from one extension unique from
-// other extensions, as well as those metrics from chrome itself.
-std::string BuildMetricName(const std::string& name,
-                            const Extension* extension) {
-  std::string full_name(name);
-  full_name += extension->id();
-  return full_name;
-}
-
-}  // anonymous namespace
-
-// These extension function classes are enabled only if the
-// enable-metrics-extension-api command line switch is used.  Refer to
-// extension_function_dispatcher.cc to see how they are enabled.
-
-bool MetricsSetEnabledFunction::RunImpl() {
-  bool enabled = false;
-  EXTENSION_FUNCTION_VALIDATE(args_->GetBoolean(0, &enabled));
-
-  // Using OptionsUtil is better because it actually ensures we reset all the
-  // necessary threads. This is the main way for starting / stopping UMA and
-  // crash reporting.
-  // This method will return the resulting enabled, which we send to JS.
-  bool result = OptionsUtil::ResolveMetricsReportingEnabled(enabled);
-  result_.reset(Value::CreateBooleanValue(result));
-  return true;
-}
-
-bool MetricsGetEnabledFunction::RunImpl() {
-  bool enabled = GoogleUpdateSettings::GetCollectStatsConsent();
-  result_.reset(Value::CreateBooleanValue(enabled));
-  return true;
-}
+const size_t kMaxBuckets = 10000; // We don't ever want more than these many
+                                  // buckets; there is no real need for them
+                                  // and would cause crazy memory usage
 
 bool MetricsRecordUserActionFunction::RunImpl() {
   std::string name;
   EXTENSION_FUNCTION_VALIDATE(args_->GetString(0, &name));
 
-  name = BuildMetricName(name, GetExtension());
-  UserMetrics::RecordComputedAction(name, profile());
+  content::RecordComputedAction(name);
   return true;
 }
 
@@ -74,16 +39,28 @@ bool MetricsHistogramHelperFunction::RecordValue(const std::string& name,
                                                  int max,
                                                  size_t buckets,
                                                  int sample) {
-  std::string full_name = BuildMetricName(name, GetExtension());
-  scoped_refptr<Histogram> counter;
+  // Make sure toxic values don't get to internal code.
+  // Fix for maximums
+  min = std::min(min, INT_MAX - 3);
+  max = std::min(max, INT_MAX - 3);
+  buckets = std::min(buckets, kMaxBuckets);
+  // Fix for minimums.
+  min = std::max(min, 1);
+  max = std::max(max, min + 1);
+  buckets = std::max(buckets, static_cast<size_t>(3));
+  // Trim buckets down to a maximum of the given range + over/underflow buckets
+  if (buckets > static_cast<size_t>(max - min + 2))
+    buckets = max - min + 2;
+
+  Histogram* counter;
   if (type == Histogram::LINEAR_HISTOGRAM) {
-    counter = LinearHistogram::FactoryGet(full_name,
+    counter = LinearHistogram::FactoryGet(name,
                                           min,
                                           max,
                                           buckets,
                                           Histogram::kUmaTargetedHistogramFlag);
   } else {
-    counter = Histogram::FactoryGet(full_name,
+    counter = Histogram::FactoryGet(name,
                                     min,
                                     max,
                                     buckets,
