@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,7 @@
 
 #include "base/json/json_writer.h"
 #include "base/lazy_instance.h"
+#include "base/metrics/histogram.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
@@ -13,7 +14,10 @@
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search_engines/template_url.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/browser/ui/browser.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "chrome/common/extensions/extension_constants.h"
+#include "content/public/browser/notification_service.h"
 
 namespace events {
 const char kOnInputStarted[] = "omnibox.onInputStarted";
@@ -36,10 +40,11 @@ const char kDescriptionStylesType[] = "type";
 const char kDescriptionStylesOffset[] = "offset";
 const char kDescriptionStylesLength[] = "length";
 
-static base::LazyInstance<PropertyAccessor<ExtensionOmniboxSuggestion> >
-    g_extension_omnibox_suggestion_property_accessor(base::LINKER_INITIALIZED);
+static base::LazyInstance<base::PropertyAccessor<ExtensionOmniboxSuggestion> >
+    g_extension_omnibox_suggestion_property_accessor =
+        LAZY_INSTANCE_INITIALIZER;
 
-PropertyAccessor<ExtensionOmniboxSuggestion>& GetPropertyAccessor() {
+base::PropertyAccessor<ExtensionOmniboxSuggestion>& GetPropertyAccessor() {
   return g_extension_omnibox_suggestion_property_accessor.Get();
 }
 
@@ -95,9 +100,10 @@ void ExtensionOmniboxEventRouter::OnInputEntered(
   profile->GetExtensionEventRouter()->DispatchEventToExtension(
       extension_id, events::kOnInputEntered, json_args, profile, GURL());
 
-  NotificationService::current()->Notify(
-      NotificationType::EXTENSION_OMNIBOX_INPUT_ENTERED,
-      Source<Profile>(profile), NotificationService::NoDetails());
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_EXTENSION_OMNIBOX_INPUT_ENTERED,
+      content::Source<Profile>(profile),
+      content::NotificationService::NoDetails());
 }
 
 // static
@@ -136,10 +142,10 @@ bool OmniboxSendSuggestionsFunction::RunImpl() {
     }
   }
 
-  NotificationService::current()->Notify(
-      NotificationType::EXTENSION_OMNIBOX_SUGGESTIONS_READY,
-      Source<Profile>(profile_),
-      Details<ExtensionOmniboxSuggestions>(&suggestions));
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_EXTENSION_OMNIBOX_SUGGESTIONS_READY,
+      content::Source<Profile>(profile_->GetOriginalProfile()),
+      content::Details<ExtensionOmniboxSuggestions>(&suggestions));
 
   return true;
 }
@@ -167,10 +173,10 @@ bool OmniboxSetDefaultSuggestionFunction::RunImpl() {
       profile_->GetExtensionService()->GetPropertyBag(GetExtension()),
       suggestion);
 
-  NotificationService::current()->Notify(
-      NotificationType::EXTENSION_OMNIBOX_DEFAULT_SUGGESTION_CHANGED,
-      Source<Profile>(profile_),
-      NotificationService::NoDetails());
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_EXTENSION_OMNIBOX_DEFAULT_SUGGESTION_CHANGED,
+      content::Source<Profile>(profile_->GetOriginalProfile()),
+      content::NotificationService::NoDetails());
 
   return true;
 }
@@ -261,5 +267,31 @@ void ApplyDefaultSuggestionForExtensionKeyword(
     }
   }
 
-  match->contents.assign(UTF16ToWide(description));
+  match->contents.assign(description);
+}
+
+void LaunchAppFromOmnibox(const AutocompleteMatch& match,
+                          Profile* profile,
+                          WindowOpenDisposition disposition) {
+  ExtensionService* service = profile->GetExtensionService();
+  const Extension* extension =
+      service->GetInstalledApp(match.destination_url);
+  // While the Omnibox popup is open, the extension can be updated, changing
+  // its URL and leaving us with no extension being found. In this case, we
+  // ignore the request.
+  if (!extension)
+    return;
+
+  UMA_HISTOGRAM_ENUMERATION(extension_misc::kAppLaunchHistogram,
+                            extension_misc::APP_LAUNCH_OMNIBOX_APP,
+                            extension_misc::APP_LAUNCH_BUCKET_BOUNDARY);
+
+  // Look at the preferences to find the right launch container.  If no
+  // preference is set, launch as a regular tab.
+  extension_misc::LaunchContainer launch_container =
+      service->extension_prefs()->GetLaunchContainer(
+          extension, ExtensionPrefs::LAUNCH_REGULAR);
+
+  Browser::OpenApplication(profile, extension, launch_container, GURL(),
+                           disposition);
 }
