@@ -1,17 +1,21 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/extensions/extension_apitest.h"
-#include "chrome/browser/extensions/extension_dom_ui.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_web_ui.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/prefs/scoped_user_pref_update.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
+
+using content::WebContents;
 
 class ExtensionOverrideTest : public ExtensionApiTest {
  protected:
@@ -19,7 +23,7 @@ class ExtensionOverrideTest : public ExtensionApiTest {
     // There should be no duplicate entries in the preferences.
     const DictionaryValue* overrides =
         browser()->profile()->GetPrefs()->GetDictionary(
-            ExtensionDOMUI::kExtensionURLOverrides);
+            ExtensionWebUI::kExtensionURLOverrides);
 
     ListValue* values = NULL;
     if (!overrides->GetList("history", &values))
@@ -40,14 +44,14 @@ class ExtensionOverrideTest : public ExtensionApiTest {
     return true;
   }
 
-#if defined(TOUCH_UI)
+#if defined(USE_VIRTUAL_KEYBOARD)
   // Navigate to the keyboard page, and ensure we have arrived at an
   // extension URL.
   void NavigateToKeyboard() {
     ui_test_utils::NavigateToURL(browser(), GURL("chrome://keyboard/"));
-    TabContents* tab = browser()->GetSelectedTabContents();
-    ASSERT_TRUE(tab->controller().GetActiveEntry());
-    EXPECT_TRUE(tab->controller().GetActiveEntry()->url().
+    WebContents* tab = browser()->GetSelectedWebContents();
+    ASSERT_TRUE(tab->GetController().GetActiveEntry());
+    EXPECT_TRUE(tab->GetController().GetActiveEntry()->url().
                 SchemeIs(chrome::kExtensionScheme));
   }
 #endif
@@ -60,9 +64,9 @@ IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, OverrideNewtab) {
     // Navigate to the new tab page.  The overridden new tab page
     // will call chrome.test.notifyPass() .
     ui_test_utils::NavigateToURL(browser(), GURL("chrome://newtab/"));
-    TabContents* tab = browser()->GetSelectedTabContents();
-    ASSERT_TRUE(tab->controller().GetActiveEntry());
-    EXPECT_TRUE(tab->controller().GetActiveEntry()->url().
+    WebContents* tab = browser()->GetSelectedWebContents();
+    ASSERT_TRUE(tab->GetController().GetActiveEntry());
+    EXPECT_TRUE(tab->GetController().GetActiveEntry()->GetURL().
                 SchemeIs(chrome::kExtensionScheme));
 
     ASSERT_TRUE(catcher.GetNextResult());
@@ -85,12 +89,11 @@ IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, MAYBE_OverrideNewtabIncognito) {
   // new tab page because we can't load chrome-extension URLs in incognito.
   ui_test_utils::OpenURLOffTheRecord(browser()->profile(),
                                      GURL("chrome://newtab/"));
-  Browser* otr_browser = BrowserList::FindBrowserWithType(
-      browser()->profile()->GetOffTheRecordProfile(), Browser::TYPE_NORMAL,
-      false);
-  TabContents* tab = otr_browser->GetSelectedTabContents();
-  ASSERT_TRUE(tab->controller().GetActiveEntry());
-  EXPECT_FALSE(tab->controller().GetActiveEntry()->url().
+  Browser* otr_browser = BrowserList::FindTabbedBrowser(
+      browser()->profile()->GetOffTheRecordProfile(), false);
+  WebContents* tab = otr_browser->GetSelectedWebContents();
+  ASSERT_TRUE(tab->GetController().GetActiveEntry());
+  EXPECT_FALSE(tab->GetController().GetActiveEntry()->GetURL().
                SchemeIs(chrome::kExtensionScheme));
 }
 
@@ -114,15 +117,16 @@ IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, MAYBE_OverrideHistory) {
 
 // Regression test for http://crbug.com/41442.
 IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, ShouldNotCreateDuplicateEntries) {
-  ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII("override/history")));
+  const Extension* extension =
+      LoadExtension(test_data_dir_.AppendASCII("override/history"));
+  ASSERT_TRUE(extension);
 
   // Simulate several LoadExtension() calls happening over the lifetime of
   // a preferences file without corresponding UnloadExtension() calls.
   for (size_t i = 0; i < 3; ++i) {
-    ExtensionDOMUI::RegisterChromeURLOverrides(
+    ExtensionWebUI::RegisterChromeURLOverrides(
         browser()->profile(),
-        browser()->profile()->GetExtensionService()->extensions()->back()->
-            GetChromeURLOverrides());
+        extension->GetChromeURLOverrides());
   }
 
   ASSERT_TRUE(CheckHistoryOverridesContainsNoDupes());
@@ -137,8 +141,11 @@ IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, ShouldCleanUpDuplicateEntries) {
   for (size_t i = 0; i < 3; ++i)
     list->Append(Value::CreateStringValue("http://www.google.com/"));
 
-  browser()->profile()->GetPrefs()->GetMutableDictionary(
-      ExtensionDOMUI::kExtensionURLOverrides)->Set("history", list);
+  {
+    DictionaryPrefUpdate update(browser()->profile()->GetPrefs(),
+                                ExtensionWebUI::kExtensionURLOverrides);
+    update.Get()->Set("history", list);
+  }
 
   ASSERT_FALSE(CheckHistoryOverridesContainsNoDupes());
 
@@ -147,7 +154,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, ShouldCleanUpDuplicateEntries) {
   ASSERT_TRUE(CheckHistoryOverridesContainsNoDupes());
 }
 
-#if defined(TOUCH_UI)
+#if defined(USE_VIRTUAL_KEYBOARD)
 IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, OverrideKeyboard) {
   ASSERT_TRUE(RunExtensionTest("override/keyboard")) << message_;
   {
@@ -157,8 +164,9 @@ IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, OverrideKeyboard) {
   }
 
   // Load the failing version.  This should take precedence.
-  ASSERT_TRUE(LoadExtension(
-      test_data_dir_.AppendASCII("override").AppendASCII("keyboard_fails")));
+  const Extension* extension = LoadExtension(
+      test_data_dir_.AppendASCII("override").AppendASCII("keyboard_fails"));
+  ASSERT_TRUE(extension);
   {
     ResultCatcher catcher;
     NavigateToKeyboard();
@@ -166,9 +174,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionOverrideTest, OverrideKeyboard) {
   }
 
   // Unload the failing version.  We should be back to passing now.
-  const ExtensionList *extensions =
-      browser()->profile()->GetExtensionService()->extensions();
-  UnloadExtension((*extensions->rbegin())->id());
+  UnloadExtension(extension->id());
   {
     ResultCatcher catcher;
     NavigateToKeyboard();
