@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,18 +7,21 @@
 #include <string>
 
 #include "base/string_number_conversions.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/extensions/extension_page_actions_module_constants.h"
-#include "chrome/browser/extensions/extension_tabs_module.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_tab_helper.h"
+#include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_action.h"
 #include "chrome/common/extensions/extension_error_utils.h"
 #include "chrome/common/render_messages.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
+
+using content::NavigationEntry;
 
 namespace keys = extension_page_actions_module_constants;
 
@@ -32,7 +35,6 @@ const char kIconIndexOutOfBounds[] = "Page action icon index out of bounds.";
 const char kNoIconSpecified[] = "Page action has no icons to show.";
 }
 
-// TODO(EXTENSIONS_DEPRECATED): obsolete API.
 bool PageActionFunction::SetPageActionEnabled(bool enable) {
   std::string page_action_id;
   EXTENSION_FUNCTION_VALIDATE(args_->GetString(0, &page_action_id));
@@ -79,8 +81,9 @@ bool PageActionFunction::SetPageActionEnabled(bool enable) {
   }
 
   // Make sure the URL hasn't changed.
-  NavigationEntry* entry = contents->controller().GetActiveEntry();
-  if (!entry || url != entry->url().spec()) {
+  NavigationEntry* entry =
+      contents->web_contents()->GetController().GetActiveEntry();
+  if (!entry || url != entry->GetURL().spec()) {
     error_ = ExtensionErrorUtils::FormatErrorMessage(kUrlNotActiveError, url);
     return false;
   }
@@ -89,7 +92,7 @@ bool PageActionFunction::SetPageActionEnabled(bool enable) {
   page_action->SetIsVisible(tab_id, enable);
   page_action->SetTitle(tab_id, title);
   page_action->SetIconIndex(tab_id, icon_id);
-  contents->tab_contents()->PageActionStateChanged();
+  contents->extension_tab_helper()->PageActionStateChanged();
 
   return true;
 }
@@ -111,7 +114,7 @@ bool PageActionFunction::InitCommon(int tab_id) {
         kNoTabError, base::IntToString(tab_id));
     return false;
   }
-  contents_ = wrapper->tab_contents();
+  contents_ = wrapper;
 
   return true;
 }
@@ -123,7 +126,7 @@ bool PageActionFunction::SetVisible(bool visible) {
     return false;
 
   page_action_->SetIsVisible(tab_id, visible);
-  contents_->PageActionStateChanged();
+  contents_->extension_tab_helper()->PageActionStateChanged();
   return true;
 }
 
@@ -154,7 +157,7 @@ bool PageActionSetIconFunction::RunImpl() {
 
   // setIcon can take a variant argument: either a canvas ImageData, or an
   // icon index.
-  BinaryValue* binary;
+  base::BinaryValue* binary;
   int icon_index;
   if (args->GetBinary("imageData", &binary)) {
     IPC::Message bitmap_pickle(binary->GetBuffer(), binary->GetSize());
@@ -175,7 +178,7 @@ bool PageActionSetIconFunction::RunImpl() {
     EXTENSION_FUNCTION_VALIDATE(false);
   }
 
-  contents_->PageActionStateChanged();
+  contents_->extension_tab_helper()->PageActionStateChanged();
   return true;
 }
 
@@ -192,7 +195,7 @@ bool PageActionSetTitleFunction::RunImpl() {
   EXTENSION_FUNCTION_VALIDATE(args->GetString("title", &title));
 
   page_action_->SetTitle(tab_id, title);
-  contents_->PageActionStateChanged();
+  contents_->extension_tab_helper()->PageActionStateChanged();
   return true;
 }
 
@@ -215,7 +218,7 @@ bool PageActionSetPopupFunction::RunImpl() {
     popup_url = GetExtension()->GetResourceURL(popup_string);
 
   page_action_->SetPopupUrl(tab_id, popup_url);
-  contents_->PageActionStateChanged();
+  contents_->extension_tab_helper()->PageActionStateChanged();
   return true;
 }
 
@@ -241,7 +244,7 @@ bool PageActionSetBadgeBackgroundColorFunction::RunImpl() {
   SkColor color = SkColorSetARGB(color_array[3], color_array[0], color_array[1],
                                  color_array[2]);
   page_action_->SetBadgeBackgroundColor(tab_id, color);
-  contents_->PageActionStateChanged();
+  contents_->extension_tab_helper()->PageActionStateChanged();
   return true;
 }
 
@@ -267,7 +270,7 @@ bool PageActionSetBadgeTextColorFunction::RunImpl() {
   SkColor color = SkColorSetARGB(color_array[3], color_array[0], color_array[1],
                                  color_array[2]);
   page_action_->SetBadgeTextColor(tab_id, color);
-  contents_->PageActionStateChanged();
+  contents_->extension_tab_helper()->PageActionStateChanged();
   return true;
 }
 
@@ -286,6 +289,6 @@ bool PageActionSetBadgeTextFunction::RunImpl() {
   EXTENSION_FUNCTION_VALIDATE(args->GetString("text", &text));
 
   page_action_->SetBadgeText(tab_id, text);
-  contents_->PageActionStateChanged();
+  contents_->extension_tab_helper()->PageActionStateChanged();
   return true;
 }
