@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,8 +8,9 @@
 
 #include <string>
 
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "base/compiler_specific.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 class ExtensionTestSendMessageFunction;
 
@@ -42,12 +43,12 @@ class ExtensionTestSendMessageFunction;
 // either make it a local variable inside your test body, or if it's a member
 // variable of a ExtensionBrowserTest subclass, override the
 // InProcessBrowserTest::CleanUpOnMainThread() method and clean it up there.
-class ExtensionTestMessageListener : public NotificationObserver {
+class ExtensionTestMessageListener : public content::NotificationObserver {
  public:
   // We immediately start listening for |expected_message|.
   ExtensionTestMessageListener(const std::string& expected_message,
                                bool will_reply);
-  ~ExtensionTestMessageListener();
+  virtual ~ExtensionTestMessageListener();
 
   // This returns true immediately if we've already gotten the expected
   // message, or waits until it arrives. Returns false if the wait is
@@ -58,15 +59,18 @@ class ExtensionTestMessageListener : public NotificationObserver {
   // WaitUntilSatisfied has returned true, and if will_reply is true.
   void Reply(const std::string& message);
 
-  // Implements the NotificationObserver interface.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // Convenience method that formats int as a string and sends it.
+  void Reply(int message);
+
+  // Implements the content::NotificationObserver interface.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   bool was_satisfied() const { return satisfied_; }
 
  private:
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // The message we're expecting.
   std::string expected_message_;
