@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,23 +6,20 @@
 #define CHROME_BROWSER_EXTENSIONS_EXTENSION_TOOLBAR_MODEL_H_
 #pragma once
 
+#include "base/compiler_specific.h"
 #include "base/observer_list.h"
 #include "chrome/common/extensions/extension.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 class ExtensionService;
 class PrefService;
 
 // Model for the browser actions toolbar.
-class ExtensionToolbarModel : public NotificationObserver {
+class ExtensionToolbarModel : public content::NotificationObserver {
  public:
   explicit ExtensionToolbarModel(ExtensionService* service);
-  ~ExtensionToolbarModel();
-
-  // Notifies the toolbar model that the Profile that suplied its
-  // prefs is being destroyed.
-  void DestroyingProfile();
+  virtual ~ExtensionToolbarModel();
 
   // A class which is informed of changes to the model; represents the view of
   // MVC.
@@ -77,10 +74,10 @@ class ExtensionToolbarModel : public NotificationObserver {
   int OriginalIndexToIncognito(int original_index);
 
  private:
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // To be called after the extension service is ready; gets loaded extensions
   // from the extension service and their saved order from the pref service
@@ -117,7 +114,7 @@ class ExtensionToolbarModel : public NotificationObserver {
   // chevron).
   int visible_icon_count_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 };
 
 #endif  // CHROME_BROWSER_EXTENSIONS_EXTENSION_TOOLBAR_MODEL_H_
