@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,12 +12,11 @@ namespace extension_webnavigation_api_constants {
 
 // Keys.
 extern const char kErrorKey[];
+extern const char kErrorOccurredKey[];
 extern const char kFrameIdKey[];
-extern const char kRequestIdKey[];
 extern const char kSourceTabIdKey[];
-extern const char kSourceUrlKey[];
+extern const char kSourceFrameIdKey[];
 extern const char kTabIdKey[];
-extern const char kTargetUrlKey[];
 extern const char kTimeStampKey[];
 extern const char kTransitionTypeKey[];
 extern const char kTransitionQualifiersKey[];
@@ -25,11 +24,12 @@ extern const char kUrlKey[];
 
 // Events.
 extern const char kOnBeforeNavigate[];
-extern const char kOnBeforeRetarget[];
 extern const char kOnCommitted[];
 extern const char kOnCompleted[];
+extern const char kOnCreatedNavigationTarget[];
 extern const char kOnDOMContentLoaded[];
 extern const char kOnErrorOccurred[];
+extern const char kOnReferenceFragmentUpdated[];
 
 }  // namespace extension_webnavigation_api_constants
 
