@@ -6,8 +6,15 @@
 
 #include "base/logging.h"
 #include "base/values.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/extensions/external_extension_provider_impl.h"
+#include "content/public/browser/browser_thread.h"
+
+using content::BrowserThread;
+
+ExternalExtensionLoader::ExternalExtensionLoader()
+    : owner_(NULL),
+      running_(false) {
+}
 
 void ExternalExtensionLoader::Init(
     ExternalExtensionProviderImpl* owner) {
@@ -28,6 +35,8 @@ void ExternalExtensionLoader::OwnerShutdown() {
   owner_ = NULL;
 }
 
+ExternalExtensionLoader::~ExternalExtensionLoader() {}
+
 void ExternalExtensionLoader::LoadFinished() {
   CHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
   running_ = false;
