@@ -1,19 +1,21 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/browser/extensions/extension_browser_event_router.h"
-#include "chrome/browser/extensions/extension_tabs_module.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/sessions/restore_tab_helper.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/omnibox/location_bar.h"
-#include "chrome/common/extensions/extension_action.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/extensions/extension.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/common/extensions/extension_action.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/public/browser/web_contents.h"
 
 IN_PROC_BROWSER_TEST_F(ExtensionApiTest, PageAction) {
   ASSERT_TRUE(test_server()->Start());
@@ -29,8 +31,8 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, PageAction) {
   }
 
   // Test that we received the changes.
-  int tab_id =
-      browser()->GetSelectedTabContents()->controller().session_id().id();
+  int tab_id = browser()->GetSelectedTabContentsWrapper()->
+      restore_tab_helper()->session_id().id();
   ExtensionAction* action = extension->page_action();
   ASSERT_TRUE(action);
   EXPECT_EQ("Modified", action->GetTitle(tab_id));
@@ -39,7 +41,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, PageAction) {
     // Simulate the page action being clicked.
     ResultCatcher catcher;
     int tab_id =
-        ExtensionTabUtil::GetTabId(browser()->GetSelectedTabContents());
+        ExtensionTabUtil::GetTabId(browser()->GetSelectedWebContents());
     ExtensionService* service = browser()->profile()->GetExtensionService();
     service->browser_event_router()->PageActionExecuted(
         browser()->profile(), extension->id(), "", tab_id, "", 0);
@@ -55,7 +57,8 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, PageAction) {
   }
 
   // Test that we received the changes.
-  tab_id = browser()->GetSelectedTabContents()->controller().session_id().id();
+  tab_id = browser()->GetSelectedTabContentsWrapper()->restore_tab_helper()->
+      session_id().id();
   EXPECT_FALSE(action->GetIcon(tab_id).isNull());
 }
 
@@ -66,7 +69,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, PageActionAddPopup) {
   const Extension* extension = GetSingleLoadedExtension();
   ASSERT_TRUE(extension) << message_;
 
-  int tab_id = ExtensionTabUtil::GetTabId(browser()->GetSelectedTabContents());
+  int tab_id = ExtensionTabUtil::GetTabId(browser()->GetSelectedWebContents());
 
   ExtensionAction* page_action = extension->page_action();
   ASSERT_TRUE(page_action)
@@ -112,7 +115,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, PageActionRemovePopup) {
   const Extension* extension = GetSingleLoadedExtension();
   ASSERT_TRUE(extension) << message_;
 
-  int tab_id = ExtensionTabUtil::GetTabId(browser()->GetSelectedTabContents());
+  int tab_id = ExtensionTabUtil::GetTabId(browser()->GetSelectedWebContents());
 
   ExtensionAction* page_action = extension->page_action();
   ASSERT_TRUE(page_action)
@@ -153,7 +156,7 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, OldPageActions) {
   {
     ResultCatcher catcher;
     int tab_id =
-        ExtensionTabUtil::GetTabId(browser()->GetSelectedTabContents());
+        ExtensionTabUtil::GetTabId(browser()->GetSelectedWebContents());
     ExtensionService* service = browser()->profile()->GetExtensionService();
     service->browser_event_router()->PageActionExecuted(
         browser()->profile(), extension->id(), "action", tab_id, "", 1);
@@ -162,7 +165,8 @@ IN_PROC_BROWSER_TEST_F(ExtensionApiTest, OldPageActions) {
 }
 
 // Tests popups in page actions.
-IN_PROC_BROWSER_TEST_F(ExtensionApiTest, ShowPageActionPopup) {
+// Flaky on the trybots. See http://crbug.com/96725.
+IN_PROC_BROWSER_TEST_F(ExtensionApiTest, FLAKY_ShowPageActionPopup) {
   ASSERT_TRUE(RunExtensionTest("page_action/popup")) << message_;
   const Extension* extension = GetSingleLoadedExtension();
   ASSERT_TRUE(extension) << message_;
