@@ -9,14 +9,18 @@
 #include <string>
 
 #include "base/file_path.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/scoped_temp_dir.h"
-#include "chrome/browser/utility_process_host.h"
+#include "chrome/common/extensions/extension.h"
+#include "content/browser/utility_process_host.h"
 
-class DictionaryValue;
 class Extension;
 class ResourceDispatcherHost;
 
+namespace base {
+class DictionaryValue;
+}
+
 class SandboxedExtensionUnpackerClient
     : public base::RefCountedThreadSafe<SandboxedExtensionUnpackerClient> {
  public:
@@ -25,12 +29,16 @@ class SandboxedExtensionUnpackerClient
   //
   // extension_root - The path to the extension root inside of temp_dir.
   //
+  // original_manifest - The parsed but unmodified version of the manifest,
+  // with no modifications such as localization, etc.
+  //
   // extension - The extension that was unpacked. The client is responsible
   // for deleting this memory.
   virtual void OnUnpackSuccess(const FilePath& temp_dir,
                                const FilePath& extension_root,
+                               const base::DictionaryValue* original_manifest,
                                const Extension* extension) = 0;
-  virtual void OnUnpackFailure(const std::string& error) = 0;
+  virtual void OnUnpackFailure(const string16& error) = 0;
 
  protected:
   friend class base::RefCountedThreadSafe<SandboxedExtensionUnpackerClient>;
@@ -94,13 +102,74 @@ class SandboxedExtensionUnpacker : public UtilityProcessHost::Client {
   // sandboxed subprocess. Otherwise, it is done in-process.
   SandboxedExtensionUnpacker(const FilePath& crx_path,
                              ResourceDispatcherHost* rdh,
-                             SandboxedExtensionUnpackerClient* cilent);
+                             Extension::Location location,
+                             int creation_flags,
+                             SandboxedExtensionUnpackerClient* client);
 
   // Start unpacking the extension. The client is called with the results.
   void Start();
 
  private:
   class ProcessHostClient;
+
+  // Enumerate all the ways unpacking can fail.  Calls to ReportFailure()
+  // take a failure reason as an argument, and put it in histogram
+  // Extensions.SandboxUnpackFailureReason.
+  enum FailureReason {
+    // SandboxedExtensionUnpacker::CreateTempDirectory()
+    COULD_NOT_GET_TEMP_DIRECTORY,
+    COULD_NOT_CREATE_TEMP_DIRECTORY,
+
+    // SandboxedExtensionUnpacker::Start()
+    FAILED_TO_COPY_EXTENSION_FILE_TO_TEMP_DIRECTORY,
+    COULD_NOT_GET_SANDBOX_FRIENDLY_PATH,
+
+    // SandboxedExtensionUnpacker::OnUnpackExtensionSucceeded()
+    COULD_NOT_LOCALIZE_EXTENSION,
+    INVALID_MANIFEST,
+
+    // SandboxedExtensionUnpacker::OnUnpackExtensionFailed()
+    UNPACKER_CLIENT_FAILED,
+
+    // SandboxedExtensionUnpacker::OnProcessCrashed()
+    UTILITY_PROCESS_CRASHED_WHILE_TRYING_TO_INSTALL,
+
+    // SandboxedExtensionUnpacker::ValidateSignature()
+    CRX_FILE_NOT_READABLE,
+    CRX_HEADER_INVALID,
+    CRX_MAGIC_NUMBER_INVALID,
+    CRX_VERSION_NUMBER_INVALID,
+    CRX_EXCESSIVELY_LARGE_KEY_OR_SIGNATURE,
+    CRX_ZERO_KEY_LENGTH,
+    CRX_ZERO_SIGNATURE_LENGTH,
+    CRX_PUBLIC_KEY_INVALID,
+    CRX_SIGNATURE_INVALID,
+    CRX_SIGNATURE_VERIFICATION_INITIALIZATION_FAILED,
+    CRX_SIGNATURE_VERIFICATION_FAILED,
+
+    // SandboxedExtensionUnpacker::RewriteManifestFile()
+    ERROR_SERIALIZING_MANIFEST_JSON,
+    ERROR_SAVING_MANIFEST_JSON,
+
+    // SandboxedExtensionUnpacker::RewriteImageFiles()
+    COULD_NOT_READ_IMAGE_DATA_FROM_DISK,
+    DECODED_IMAGES_DO_NOT_MATCH_THE_MANIFEST,
+    INVALID_PATH_FOR_BROWSER_IMAGE,
+    ERROR_REMOVING_OLD_IMAGE_FILE,
+    INVALID_PATH_FOR_BITMAP_IMAGE,
+    ERROR_RE_ENCODING_THEME_IMAGE,
+    ERROR_SAVING_THEME_IMAGE,
+
+    // SandboxedExtensionUnpacker::RewriteCatalogFiles()
+    COULD_NOT_READ_CATALOG_DATA_FROM_DISK,
+    INVALID_CATALOG_DATA,
+    INVALID_PATH_FOR_CATALOG,
+    ERROR_SERIALIZING_CATALOG,
+    ERROR_SAVING_CATALOG,
+
+    NUM_FAILURE_REASONS
+  };
+
   friend class ProcessHostClient;
   friend class SandboxedExtensionUnpackerTest;
 
@@ -124,17 +193,21 @@ class SandboxedExtensionUnpacker : public UtilityProcessHost::Client {
   // Starts the utility process that unpacks our extension.
   void StartProcessOnIOThread(const FilePath& temp_crx_path);
 
-  // SandboxedExtensionUnpacker
-  virtual void OnUnpackExtensionSucceeded(const DictionaryValue& manifest);
-  virtual void OnUnpackExtensionFailed(const std::string& error_message);
-  virtual void OnProcessCrashed(int exit_code);
+  // UtilityProcessHost::Client
+  virtual bool OnMessageReceived(const IPC::Message& message) OVERRIDE;
+  virtual void OnProcessCrashed(int exit_code) OVERRIDE;
 
-  void ReportFailure(const std::string& message);
-  void ReportSuccess();
+  // IPC message handlers.
+  void OnUnpackExtensionSucceeded(const base::DictionaryValue& manifest);
+  void OnUnpackExtensionFailed(const string16& error_message);
+
+  void ReportFailure(FailureReason reason, const string16& message);
+  void ReportSuccess(const base::DictionaryValue& original_manifest);
 
   // Overwrites original manifest with safe result from utility process.
   // Returns NULL on error. Caller owns the returned object.
-  DictionaryValue* RewriteManifestFile(const DictionaryValue& manifest);
+  base::DictionaryValue* RewriteManifestFile(
+      const base::DictionaryValue& manifest);
 
   // Overwrites original files with safe results from utility process.
   // Reports error and returns false if it fails.
@@ -145,7 +218,7 @@ class SandboxedExtensionUnpacker : public UtilityProcessHost::Client {
   FilePath crx_path_;
 
   // Our client's thread. This is the thread we respond on.
-  BrowserThread::ID thread_identifier_;
+  content::BrowserThread::ID thread_identifier_;
 
   // ResourceDispatcherHost to pass to the utility process.
   ResourceDispatcherHost* rdh_;
@@ -167,6 +240,16 @@ class SandboxedExtensionUnpacker : public UtilityProcessHost::Client {
 
   // The public key that was extracted from the CRX header.
   std::string public_key_;
+
+  // Time at which unpacking started. Used to compute the time unpacking takes.
+  base::TimeTicks unpack_start_time_;
+
+  // Location to use for the unpacked extension.
+  Extension::Location location_;
+
+  // Creation flags to use for the extension.  These flags will be used
+  // when calling Extenion::Create() by the crx installer.
+  int creation_flags_;
 };
 
 #endif  // CHROME_BROWSER_EXTENSIONS_SANDBOXED_EXTENSION_UNPACKER_H_
