@@ -3,15 +3,19 @@
 // found in the LICENSE file.
 
 #include "base/file_util.h"
+#include "base/memory/ref_counted.h"
+#include "base/message_loop.h"
 #include "base/path_service.h"
-#include "base/ref_counted.h"
+#include "base/scoped_temp_dir.h"
 #include "base/scoped_temp_dir.h"
 #include "base/string_util.h"
+#include "base/values.h"
 #include "chrome/browser/extensions/sandboxed_extension_unpacker.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/extensions/extension_unpacker.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
@@ -19,6 +23,7 @@
 namespace errors = extension_manifest_errors;
 namespace keys = extension_manifest_keys;
 
+using content::BrowserThread;
 using testing::_;
 using testing::Invoke;
 
@@ -26,6 +31,7 @@ namespace {
 
 void OnUnpackSuccess(const FilePath& temp_dir,
                      const FilePath& extension_root,
+                     const DictionaryValue* original_manifest,
                      const Extension* extension) {
   // Don't delete temp_dir here, we need to do some post op checking.
 }
@@ -37,16 +43,17 @@ class MockSandboxedExtensionUnpackerClient
  public:
   virtual ~MockSandboxedExtensionUnpackerClient() {}
 
-  MOCK_METHOD3(OnUnpackSuccess,
+  MOCK_METHOD4(OnUnpackSuccess,
                void(const FilePath& temp_dir,
                     const FilePath& extension_root,
+                    const DictionaryValue* original_manifest,
                     const Extension* extension));
 
   MOCK_METHOD1(OnUnpackFailure,
-               void(const std::string& error));
+               void(const string16& error));
 
   void DelegateToFake() {
-    ON_CALL(*this, OnUnpackSuccess(_, _, _))
+    ON_CALL(*this, OnUnpackSuccess(_, _, _, _))
         .WillByDefault(Invoke(::OnUnpackSuccess));
   }
 };
@@ -54,7 +61,9 @@ class MockSandboxedExtensionUnpackerClient
 class SandboxedExtensionUnpackerTest : public testing::Test {
  public:
   virtual void SetUp() {
-    file_thread_.reset(new BrowserThread(BrowserThread::FILE, &loop_));
+    ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
+    file_thread_.reset(new content::TestBrowserThread(BrowserThread::FILE,
+                                                      &loop_));
     // It will delete itself.
     client_ = new MockSandboxedExtensionUnpackerClient;
     client_->DelegateToFake();
@@ -65,9 +74,6 @@ class SandboxedExtensionUnpackerTest : public testing::Test {
     // it posts a task to it.
     sandboxed_unpacker_ = NULL;
     loop_.RunAllPending();
-    // Clean up finally.
-    ASSERT_TRUE(file_util::Delete(install_dir_, true)) <<
-        install_dir_.value();
   }
 
   void SetupUnpacker(const std::string& crx_name) {
@@ -78,28 +84,25 @@ class SandboxedExtensionUnpackerTest : public testing::Test {
         .AppendASCII(crx_name);
     ASSERT_TRUE(file_util::PathExists(original_path)) << original_path.value();
 
-    // Try bots won't let us write into DIR_TEST_DATA, so we have to create
-    // a temp folder to play in.
-    ASSERT_TRUE(PathService::Get(base::DIR_TEMP, &install_dir_));
-    install_dir_ =
-        install_dir_.AppendASCII("sandboxed_extension_unpacker_test");
-    file_util::Delete(install_dir_, true);
-    file_util::CreateDirectory(install_dir_);
-
-    FilePath crx_path = install_dir_.AppendASCII(crx_name);
+    // Try bots won't let us write into DIR_TEST_DATA, so we have to write the
+    // CRX to the temp directory, and create a subdirectory into which to
+    // unpack it.
+    FilePath crx_path = temp_dir_.path().AppendASCII(crx_name);
     ASSERT_TRUE(file_util::CopyFile(original_path, crx_path)) <<
         "Original path: " << original_path.value() <<
         ", Crx path: " << crx_path.value();
 
-    unpacker_.reset(new ExtensionUnpacker(crx_path));
+    unpacker_.reset(new ExtensionUnpacker(
+        crx_path, Extension::INTERNAL, Extension::NO_FLAGS));
 
     // Build a temp area where the extension will be unpacked.
-    ASSERT_TRUE(PathService::Get(base::DIR_TEMP, &temp_dir_));
-    temp_dir_ = temp_dir_.AppendASCII("sandboxed_extension_unpacker_test_Temp");
-    ASSERT_TRUE(file_util::CreateDirectory(temp_dir_));
+    temp_path_ =
+        temp_dir_.path().AppendASCII("sandboxed_extension_unpacker_test_Temp");
+    ASSERT_TRUE(file_util::CreateDirectory(temp_path_));
 
     sandboxed_unpacker_ =
-        new SandboxedExtensionUnpacker(crx_path, NULL, client_);
+        new SandboxedExtensionUnpacker(crx_path, NULL, Extension::INTERNAL,
+                                       Extension::NO_FLAGS, client_);
 
     // Hack since SandboxedExtensionUnpacker gets its background thread id from
     // the Start call, but we don't call it here.
@@ -109,9 +112,9 @@ class SandboxedExtensionUnpackerTest : public testing::Test {
 
   bool PrepareUnpackerEnv() {
     sandboxed_unpacker_->extension_root_ =
-      install_dir_.AppendASCII(extension_filenames::kTempExtensionName);
+      temp_dir_.path().AppendASCII(extension_filenames::kTempExtensionName);
 
-    if (!sandboxed_unpacker_->temp_dir_.Set(install_dir_))
+    if (!sandboxed_unpacker_->temp_dir_.Set(temp_dir_.path()))
       return false;
     sandboxed_unpacker_->public_key_ =
       "ocnapchkplbmjmpfehjocmjnipfmogkh";
@@ -124,18 +127,19 @@ class SandboxedExtensionUnpackerTest : public testing::Test {
   }
 
   FilePath GetInstallPath() {
-    return install_dir_.AppendASCII(extension_filenames::kTempExtensionName);
+    return temp_dir_.path().AppendASCII(
+        extension_filenames::kTempExtensionName);
   }
 
   bool TempFilesRemoved() {
     // Check that temporary files were cleaned up.
-    file_util::FileEnumerator::FILE_TYPE files_and_dirs =
-      static_cast<file_util::FileEnumerator::FILE_TYPE>(
+    file_util::FileEnumerator::FileType files_and_dirs =
+      static_cast<file_util::FileEnumerator::FileType>(
         file_util::FileEnumerator::DIRECTORIES |
         file_util::FileEnumerator::FILES);
 
     file_util::FileEnumerator temp_iterator(
-      temp_dir_,
+      temp_path_,
       true,  // recursive
       files_and_dirs
     );
@@ -152,17 +156,17 @@ class SandboxedExtensionUnpackerTest : public testing::Test {
   }
 
  protected:
-  FilePath install_dir_;
-  FilePath temp_dir_;
+  ScopedTempDir temp_dir_;
+  FilePath temp_path_;
   MockSandboxedExtensionUnpackerClient* client_;
   scoped_ptr<ExtensionUnpacker> unpacker_;
   scoped_refptr<SandboxedExtensionUnpacker> sandboxed_unpacker_;
   MessageLoop loop_;
-  scoped_ptr<BrowserThread> file_thread_;
+  scoped_ptr<content::TestBrowserThread> file_thread_;
 };
 
 TEST_F(SandboxedExtensionUnpackerTest, NoCatalogsSuccess) {
-  EXPECT_CALL(*client_, OnUnpackSuccess(_, _, _));
+  EXPECT_CALL(*client_, OnUnpackSuccess(_, _, _, _));
   EXPECT_CALL(*client_, OnUnpackFailure(_)).Times(0);
 
   SetupUnpacker("no_l10n.crx");
@@ -184,7 +188,7 @@ TEST_F(SandboxedExtensionUnpackerTest, NoCatalogsSuccess) {
 }
 
 TEST_F(SandboxedExtensionUnpackerTest, WithCatalogsSuccess) {
-  EXPECT_CALL(*client_, OnUnpackSuccess(_, _, _));
+  EXPECT_CALL(*client_, OnUnpackSuccess(_, _, _, _));
   EXPECT_CALL(*client_, OnUnpackFailure(_)).Times(0);
 
   SetupUnpacker("good_l10n.crx");
