@@ -1,21 +1,22 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/common/chrome_paths.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "googleurl/src/gurl.h"
 #include "net/test/test_server.h"
 
 // Tests that we throw errors when you try using extension APIs that aren't
 // supported in content scripts.
-//
-// If you have added a new API to extension_api.json and this test starts
-// failing, most likely you need to either mark it as "unprivileged" (if it
-// should be available in content scripts) or update the list of privileged APIs
-// in renderer_extension_bindings.js.
-IN_PROC_BROWSER_TEST_F(ExtensionApiTest, Stubs) {
+// Timey-outy on mac. http://crbug.com/89116
+#if defined(OS_MACOSX)
+#define MAYBE_Stubs DISABLED_Stubs
+#else
+#define MAYBE_Stubs Stubs
+#endif
+IN_PROC_BROWSER_TEST_F(ExtensionApiTest, MAYBE_Stubs) {
   ASSERT_TRUE(test_server()->Start());
 
   ASSERT_TRUE(RunExtensionTest("stubs")) << message_;
