@@ -1,16 +1,19 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/test_extension_prefs.h"
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/file_util.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
 #include "base/message_loop_proxy.h"
-#include "base/scoped_ptr.h"
+#include "base/synchronization/waitable_event.h"
 #include "base/values.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/extensions/extension_pref_store.h"
+#include "chrome/browser/extensions/extension_pref_value_map.h"
 #include "chrome/browser/extensions/extension_prefs.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/prefs/pref_service_mock_builder.h"
@@ -18,8 +21,11 @@
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/json_pref_store.h"
+#include "content/public/browser/browser_thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using content::BrowserThread;
+
 namespace {
 
 // Mock ExtensionPrefs class with artificial clock to guarantee that no two
@@ -28,9 +34,9 @@ namespace {
 class MockExtensionPrefs : public ExtensionPrefs {
  public:
   MockExtensionPrefs(PrefService* prefs,
-                     const FilePath& root_dir_,
-                     ExtensionPrefStore* pref_store)
-    : ExtensionPrefs(prefs, root_dir_, pref_store),
+                     const FilePath& root_dir,
+                     ExtensionPrefValueMap* extension_pref_value_map)
+    : ExtensionPrefs(prefs, root_dir, extension_pref_value_map),
       currentTime(base::Time::Now()) {}
   ~MockExtensionPrefs() {}
 
@@ -45,7 +51,9 @@ class MockExtensionPrefs : public ExtensionPrefs {
 
 }  // namespace
 
-TestExtensionPrefs::TestExtensionPrefs() : pref_service_(NULL) {
+TestExtensionPrefs::TestExtensionPrefs()
+    : pref_service_(NULL),
+      extensions_disabled_(false) {
   EXPECT_TRUE(temp_dir_.CreateUniqueTempDir());
   preferences_file_ = temp_dir_.path().AppendASCII("Preferences");
   extensions_dir_ = temp_dir_.path().AppendASCII("Extensions");
@@ -64,20 +72,35 @@ void TestExtensionPrefs::RecreateExtensionPrefs() {
     // The PrefService writes its persistent file on the file thread, so we
     // need to wait for any pending I/O to complete before creating a new
     // PrefService.
-    MessageLoop file_loop;
-    BrowserThread file_thread(BrowserThread::FILE, &file_loop);
-    pref_service_->SavePersistentPrefs();
-    file_loop.RunAllPending();
+    base::WaitableEvent io_finished(false, false);
+    pref_service_-> CommitPendingWrite();
+    EXPECT_TRUE(BrowserThread::PostTask(
+        BrowserThread::FILE,
+        FROM_HERE,
+        base::Bind(&base::WaitableEvent::Signal,
+                   base::Unretained(&io_finished))));
+
+    // If the FILE thread is in fact the current thread (possible in testing
+    // scenarios), we have to ensure the task has a chance to run. If the FILE
+    // thread is a different thread, the test must ensure that thread is running
+    // (otherwise the Wait below will hang).
+    MessageLoop::current()->RunAllPending();
+
+    io_finished.Wait();
   }
 
-  ExtensionPrefStore* pref_store = new ExtensionPrefStore;
+  extension_pref_value_map_.reset(new ExtensionPrefValueMap);
   PrefServiceMockBuilder builder;
   builder.WithUserFilePrefs(preferences_file_);
-  builder.WithExtensionPrefs(pref_store);
+  builder.WithExtensionPrefs(
+      new ExtensionPrefStore(extension_pref_value_map_.get(), false));
   pref_service_.reset(builder.Create());
   ExtensionPrefs::RegisterUserPrefs(pref_service_.get());
-  prefs_.reset(new MockExtensionPrefs(pref_service_.get(), temp_dir_.path(),
-                                      pref_store));
+
+  prefs_.reset(new MockExtensionPrefs(pref_service_.get(),
+                                      temp_dir_.path(),
+                                      extension_pref_value_map_.get()));
+  prefs_->Init(extensions_disabled_);
 }
 
 scoped_refptr<Extension> TestExtensionPrefs::AddExtension(std::string name) {
@@ -87,22 +110,41 @@ scoped_refptr<Extension> TestExtensionPrefs::AddExtension(std::string name) {
   return AddExtensionWithManifest(dictionary, Extension::INTERNAL);
 }
 
+scoped_refptr<Extension> TestExtensionPrefs::AddApp(std::string name) {
+  DictionaryValue dictionary;
+  dictionary.SetString(extension_manifest_keys::kName, name);
+  dictionary.SetString(extension_manifest_keys::kVersion, "0.1");
+  dictionary.SetString(extension_manifest_keys::kApp, "true");
+  dictionary.SetString(extension_manifest_keys::kLaunchWebURL,
+                       "http://example.com");
+  return AddExtensionWithManifest(dictionary, Extension::INTERNAL);
+
+}
+
 scoped_refptr<Extension> TestExtensionPrefs::AddExtensionWithManifest(
     const DictionaryValue& manifest, Extension::Location location) {
+  return AddExtensionWithManifestAndFlags(manifest, location,
+                                          Extension::STRICT_ERROR_CHECKS);
+}
+
+scoped_refptr<Extension> TestExtensionPrefs::AddExtensionWithManifestAndFlags(
+    const DictionaryValue& manifest,
+    Extension::Location location,
+    int extra_flags) {
   std::string name;
   EXPECT_TRUE(manifest.GetString(extension_manifest_keys::kName, &name));
   FilePath path =  extensions_dir_.AppendASCII(name);
   std::string errors;
   scoped_refptr<Extension> extension = Extension::Create(
-      path, location, manifest, false, &errors);
-  EXPECT_TRUE(extension);
+      path, location, manifest, extra_flags, &errors);
+  EXPECT_TRUE(extension) << errors;
   if (!extension)
     return NULL;
 
   EXPECT_TRUE(Extension::IdIsValid(extension->id()));
-  const bool kInitialIncognitoEnabled = false;
   prefs_->OnExtensionInstalled(extension, Extension::ENABLED,
-                               kInitialIncognitoEnabled);
+                               extra_flags & Extension::FROM_WEBSTORE,
+                               StringOrdinal::CreateInitialOrdinal());
   return extension;
 }
 
@@ -110,3 +152,12 @@ std::string TestExtensionPrefs::AddExtensionAndReturnId(std::string name) {
   scoped_refptr<Extension> extension(AddExtension(name));
   return extension->id();
 }
+
+PrefService* TestExtensionPrefs::CreateIncognitoPrefService() const {
+  return pref_service_->CreateIncognitoPrefService(
+      new ExtensionPrefStore(extension_pref_value_map_.get(), true));
+}
+
+void TestExtensionPrefs::set_extensions_disabled(bool extensions_disabled) {
+  extensions_disabled_ = extensions_disabled;
+}
