@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,14 +8,18 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/scoped_temp_dir.h"
 #include "chrome/common/extensions/extension.h"
 
-class DictionaryValue;
 class ExtensionPrefs;
+class ExtensionPrefValueMap;
 class PrefService;
 
+namespace base {
+class DictionaryValue;
+}
+
 // This is a test class intended to make it easier to work with ExtensionPrefs
 // in tests.
 class TestExtensionPrefs {
@@ -24,6 +28,7 @@ class TestExtensionPrefs {
   virtual ~TestExtensionPrefs();
 
   ExtensionPrefs* prefs() { return prefs_.get(); }
+  const ExtensionPrefs& const_prefs() const { return *prefs_.get(); }
   PrefService* pref_service() { return pref_service_.get(); }
   const FilePath& temp_dir() const { return temp_dir_.path(); }
 
@@ -35,23 +40,41 @@ class TestExtensionPrefs {
   // our ExtensionPrefs, and returns it.
   scoped_refptr<Extension> AddExtension(std::string name);
 
+  // As above, but the extension is an app.
+  scoped_refptr<Extension> AddApp(std::string name);
+
   // Similar to AddExtension, but takes a dictionary with manifest values.
   scoped_refptr<Extension> AddExtensionWithManifest(
-      const DictionaryValue& manifest, Extension::Location location);
+      const base::DictionaryValue& manifest, Extension::Location location);
+
+  // Similar to AddExtension, but takes a dictionary with manifest values
+  // and extension flags.
+  scoped_refptr<Extension> AddExtensionWithManifestAndFlags(
+      const base::DictionaryValue& manifest,
+      Extension::Location location,
+      int extra_flags);
 
   // Similar to AddExtension, this adds a new test Extension. This is useful for
   // cases when you don't need the Extension object, but just the id it was
   // assigned.
   std::string AddExtensionAndReturnId(std::string name);
 
+  PrefService* CreateIncognitoPrefService() const;
+
+  // Allows disabling the loading of preferences of extensions. Becomes
+  // active after calling RecreateExtensionPrefs(). Defaults to false.
+  void set_extensions_disabled(bool extensions_disabled);
+
  protected:
   ScopedTempDir temp_dir_;
   FilePath preferences_file_;
   FilePath extensions_dir_;
   scoped_ptr<PrefService> pref_service_;
   scoped_ptr<ExtensionPrefs> prefs_;
+  scoped_ptr<ExtensionPrefValueMap> extension_pref_value_map_;
 
  private:
+  bool extensions_disabled_;
   DISALLOW_COPY_AND_ASSIGN(TestExtensionPrefs);
 };
 
