@@ -1,23 +1,29 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/geolocation/access_token_store.h"
-
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/browser/geolocation/chrome_access_token_store.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/test/test_browser_thread.h"
+#include "content/public/browser/access_token_store.h"
+
+using content::AccessTokenStore;
+using content::BrowserThread;
 
 namespace {
+
 // The token store factory implementation expects to be used from any well-known
 // chrome thread other than UI. We could use any arbitrary thread; IO is
 // a good choice as this is the expected usage.
 const BrowserThread::ID kExpectedClientThreadId = BrowserThread::IO;
 const char* kRefServerUrl1 = "https://test.domain.example/foo?id=bar.bar";
 const char* kRefServerUrl2 = "http://another.domain.example/foo?id=bar.bar#2";
-}  // namespace
+const char* kOldDefaultNetworkProviderUrl = "https://www.google.com/loc/json";
 
 class GeolocationAccessTokenStoreTest
     : public InProcessBrowserTest {
@@ -30,10 +36,10 @@ class GeolocationAccessTokenStoreTest
       const string16* token_to_set);
 
   void OnAccessTokenStoresLoaded(
-        AccessTokenStore::AccessTokenSet access_token_set);
+      AccessTokenStore::AccessTokenSet access_token_set,
+      net::URLRequestContextGetter* context_getter);
 
   scoped_refptr<AccessTokenStore> token_store_;
-  CancelableRequestConsumer request_consumer_;
   GURL ref_url_;
   const string16* token_to_expect_;
   const string16* token_to_set_;
@@ -41,51 +47,20 @@ class GeolocationAccessTokenStoreTest
 
 void StartTestStepFromClientThread(
     scoped_refptr<AccessTokenStore>* store,
-    CancelableRequestConsumerBase* consumer,
-    AccessTokenStore::LoadAccessTokensCallbackType* callback) {
+    const AccessTokenStore::LoadAccessTokensCallbackType& callback) {
   ASSERT_TRUE(BrowserThread::CurrentlyOn(kExpectedClientThreadId));
   if (*store == NULL)
-    (*store) = NewChromePrefsAccessTokenStore();
-  (*store)->LoadAccessTokens(consumer, callback);
+    (*store) = new ChromeAccessTokenStore();
+  (*store)->LoadAccessTokens(callback);
 }
 
 struct TokenLoadClientForTest {
-  void NotReachedCallback(AccessTokenStore::AccessTokenSet /*tokens*/) {
+  void NotReachedCallback(AccessTokenStore::AccessTokenSet /*tokens*/,
+                          net::URLRequestContextGetter* /*context_getter*/) {
     NOTREACHED() << "This request should have been canceled before callback";
   }
 };
 
-void RunCancelTestInClientTread() {
-  ASSERT_TRUE(BrowserThread::CurrentlyOn(kExpectedClientThreadId));
-  scoped_refptr<AccessTokenStore> store(NewChromePrefsAccessTokenStore());
-  CancelableRequestConsumer consumer;
-  TokenLoadClientForTest load_client;
-
-  // Single request, canceled explicitly
-  CancelableRequestProvider::Handle first_handle =
-      store->LoadAccessTokens(&consumer, NewCallback(
-          &load_client, &TokenLoadClientForTest::NotReachedCallback));
-  EXPECT_TRUE(consumer.HasPendingRequests());
-  // Test this handle is valid.
-  consumer.GetClientData(store.get(), first_handle);
-  store->CancelRequest(first_handle);
-  EXPECT_FALSE(consumer.HasPendingRequests());
-
-  // 2 requests, canceled globally.
-  store->LoadAccessTokens(&consumer, NewCallback(
-      &load_client, &TokenLoadClientForTest::NotReachedCallback));
-  store->LoadAccessTokens(&consumer, NewCallback(
-      &load_client, &TokenLoadClientForTest::NotReachedCallback));
-  EXPECT_TRUE(consumer.HasPendingRequests());
-  EXPECT_EQ(2u, consumer.PendingRequestCount());
-  consumer.CancelAllRequests();
-  EXPECT_FALSE(consumer.HasPendingRequests());
-  EXPECT_EQ(0u, consumer.PendingRequestCount());
-
-  BrowserThread::PostTask(
-      BrowserThread::UI, FROM_HERE, new MessageLoop::QuitTask);
-}
-
 void GeolocationAccessTokenStoreTest::DoTestStepAndWaitForResults(
     const char* ref_url, const string16* token_to_expect,
     const string16* token_to_set) {
@@ -94,19 +69,23 @@ void GeolocationAccessTokenStoreTest::DoTestStepAndWaitForResults(
   token_to_set_ = token_to_set;
 
   BrowserThread::PostTask(
-      kExpectedClientThreadId, FROM_HERE, NewRunnableFunction(
-          &StartTestStepFromClientThread, &token_store_, &request_consumer_,
-          NewCallback(this,
-              &GeolocationAccessTokenStoreTest::OnAccessTokenStoresLoaded)));
+      kExpectedClientThreadId, FROM_HERE,
+      base::Bind(
+          &StartTestStepFromClientThread,
+          &token_store_,
+          base::Bind(
+              &GeolocationAccessTokenStoreTest::OnAccessTokenStoresLoaded,
+              base::Unretained(this))));
   ui_test_utils::RunMessageLoop();
 }
 
 void GeolocationAccessTokenStoreTest::OnAccessTokenStoresLoaded(
-    AccessTokenStore::AccessTokenSet access_token_set) {
+    AccessTokenStore::AccessTokenSet access_token_set,
+    net::URLRequestContextGetter* context_getter) {
   ASSERT_TRUE(BrowserThread::CurrentlyOn(kExpectedClientThreadId))
       << "Callback from token factory should be from the same thread as the "
          "LoadAccessTokenStores request was made on";
-  EXPECT_TRUE(token_to_set_ || token_to_expect_) << "No work to do?";
+  DCHECK(context_getter);
   AccessTokenStore::AccessTokenSet::const_iterator item =
       access_token_set.find(ref_url_);
   if (!token_to_expect_) {
@@ -117,12 +96,11 @@ void GeolocationAccessTokenStoreTest::OnAccessTokenStoresLoaded(
   }
 
   if (token_to_set_) {
-    scoped_refptr<AccessTokenStore> store(
-        NewChromePrefsAccessTokenStore());
+    scoped_refptr<AccessTokenStore> store(new ChromeAccessTokenStore());
     store->SaveAccessToken(ref_url_, *token_to_set_);
   }
   BrowserThread::PostTask(
-      BrowserThread::UI, FROM_HERE, new MessageLoop::QuitTask);
+      BrowserThread::UI, FROM_HERE, MessageLoop::QuitClosure());
 }
 
 IN_PROC_BROWSER_TEST_F(GeolocationAccessTokenStoreTest, SetAcrossInstances) {
@@ -143,9 +121,17 @@ IN_PROC_BROWSER_TEST_F(GeolocationAccessTokenStoreTest, SetAcrossInstances) {
   DoTestStepAndWaitForResults(kRefServerUrl1, &ref_token1, NULL);
 }
 
-IN_PROC_BROWSER_TEST_F(GeolocationAccessTokenStoreTest, CancelRequest) {
-  BrowserThread::PostTask(
-      kExpectedClientThreadId, FROM_HERE, NewRunnableFunction(
-          RunCancelTestInClientTread));
-  ui_test_utils::RunMessageLoop();
+IN_PROC_BROWSER_TEST_F(GeolocationAccessTokenStoreTest, OldUrlRemoval) {
+  const string16 ref_token1 = ASCIIToUTF16("jksdfo90,'s#\"#1*(");
+  ASSERT_TRUE(BrowserThread::CurrentlyOn(BrowserThread::UI));
+
+  // Set a token for the old default network provider url.
+  DoTestStepAndWaitForResults(kOldDefaultNetworkProviderUrl,
+                              NULL, &ref_token1);
+  // Check that the token related to the old default network provider url
+  // was deleted.
+  DoTestStepAndWaitForResults(kOldDefaultNetworkProviderUrl,
+                              NULL, NULL);
 }
+
+}  // namespace
