@@ -1,16 +1,19 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/geolocation/geolocation_settings_state.h"
 
+#include <string>
+
 #include "base/string_piece.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/geolocation/geolocation_content_settings_map.h"
+#include "chrome/browser/content_settings/host_content_settings_map.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/browser/navigation_details.h"
+#include "content/public/browser/navigation_entry.h"
 #include "net/base/net_util.h"
 
 GeolocationSettingsState::GeolocationSettingsState(Profile* profile)
@@ -27,13 +30,13 @@ void GeolocationSettingsState::OnGeolocationPermissionSet(
 }
 
 void GeolocationSettingsState::DidNavigate(
-    const NavigationController::LoadCommittedDetails& details) {
+    const content::LoadCommittedDetails& details) {
   if (details.entry)
-    embedder_url_ = details.entry->url();
+    embedder_url_ = details.entry->GetURL();
   if (state_map_.empty())
     return;
   if (!details.entry ||
-      details.previous_url.GetOrigin() != details.entry->url().GetOrigin()) {
+      details.previous_url.GetOrigin() != details.entry->GetURL().GetOrigin()) {
     state_map_.clear();
     return;
   }
@@ -53,8 +56,9 @@ void GeolocationSettingsState::GetDetailedInfo(
     unsigned int* tab_state_flags) const {
   DCHECK(tab_state_flags);
   DCHECK(embedder_url_.is_valid());
-  const ContentSetting default_setting =
-      profile_->GetGeolocationContentSettingsMap()->GetDefaultContentSetting();
+  ContentSetting default_setting =
+      profile_->GetHostContentSettingsMap()->GetDefaultContentSetting(
+          CONTENT_SETTINGS_TYPE_GEOLOCATION, NULL);
   std::set<std::string> formatted_hosts;
   std::set<std::string> repeated_formatted_hosts;
 
@@ -82,8 +86,11 @@ void GeolocationSettingsState::GetDetailedInfo(
     }
 
     const ContentSetting saved_setting =
-        profile_->GetGeolocationContentSettingsMap()->GetContentSetting(
-            i->first, embedder_url_);
+        profile_->GetHostContentSettingsMap()->GetContentSetting(
+            i->first,
+            embedder_url_,
+            CONTENT_SETTINGS_TYPE_GEOLOCATION,
+            std::string());
     if (saved_setting != default_setting)
       *tab_state_flags |= TABSTATE_HAS_EXCEPTION;
     if (saved_setting != i->second)
@@ -95,9 +102,8 @@ void GeolocationSettingsState::GetDetailedInfo(
 
 std::string GeolocationSettingsState::GURLToFormattedHost(
     const GURL& url) const {
-  std::wstring display_host_wide;
-  net::AppendFormattedHost(
-      url, UTF8ToWide(profile_->GetPrefs()->GetString(prefs::kAcceptLanguages)),
-      &display_host_wide, NULL, NULL);
-  return WideToUTF8(display_host_wide);
+  string16 display_host;
+  net::AppendFormattedHost(url,
+      profile_->GetPrefs()->GetString(prefs::kAcceptLanguages), &display_host);
+  return UTF16ToUTF8(display_host);
 }
