@@ -1,26 +1,32 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/geolocation/geolocation_settings_state.h"
+#include <string>
 
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/geolocation/geolocation_content_settings_map.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/test/testing_profile.h"
+#include "base/message_loop.h"
+#include "chrome/browser/content_settings/host_content_settings_map.h"
+#include "chrome/browser/geolocation/geolocation_settings_state.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/public/browser/navigation_details.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using content::BrowserThread;
+using content::NavigationEntry;
+
 namespace {
 
 class GeolocationSettingsStateTests : public testing::Test {
  public:
-   GeolocationSettingsStateTests()
+  GeolocationSettingsStateTests()
     : ui_thread_(BrowserThread::UI, &message_loop_) {
   }
 
  protected:
   MessageLoop message_loop_;
-  BrowserThread ui_thread_;
+  content::TestBrowserThread ui_thread_;
 };
 
 TEST_F(GeolocationSettingsStateTests, ClearOnNewOrigin) {
@@ -28,19 +34,27 @@ TEST_F(GeolocationSettingsStateTests, ClearOnNewOrigin) {
   GeolocationSettingsState state(&profile);
   GURL url_0("http://www.example.com");
 
-  NavigationEntry entry;
-  entry.set_url(url_0);
-  NavigationController::LoadCommittedDetails load_committed_details;
-  load_committed_details.entry = &entry;
+  scoped_ptr<NavigationEntry> entry(NavigationEntry::Create());
+  entry->SetURL(url_0);
+  content::LoadCommittedDetails load_committed_details;
+  load_committed_details.entry = entry.get();
   state.DidNavigate(load_committed_details);
 
-  profile.GetGeolocationContentSettingsMap()->SetContentSetting(
-      url_0, url_0, CONTENT_SETTING_ALLOW);
+  profile.GetHostContentSettingsMap()->SetContentSetting(
+      ContentSettingsPattern::FromURLNoWildcard(url_0),
+      ContentSettingsPattern::FromURLNoWildcard(url_0),
+      CONTENT_SETTINGS_TYPE_GEOLOCATION,
+      std::string(),
+      CONTENT_SETTING_ALLOW);
   state.OnGeolocationPermissionSet(url_0, true);
 
   GURL url_1("http://www.example1.com");
-  profile.GetGeolocationContentSettingsMap()->SetContentSetting(
-    url_1, url_0, CONTENT_SETTING_BLOCK);
+  profile.GetHostContentSettingsMap()->SetContentSetting(
+    ContentSettingsPattern::FromURLNoWildcard(url_1),
+    ContentSettingsPattern::FromURLNoWildcard(url_0),
+    CONTENT_SETTINGS_TYPE_GEOLOCATION,
+    std::string(),
+    CONTENT_SETTING_BLOCK);
   state.OnGeolocationPermissionSet(url_1, false);
 
   GeolocationSettingsState::StateMap state_map =
@@ -108,7 +122,7 @@ TEST_F(GeolocationSettingsStateTests, ClearOnNewOrigin) {
   EXPECT_EQ(state_map.size(), new_state_map.size());
 
   GURL different_url("http://foo.com");
-  entry.set_url(different_url);
+  entry->SetURL(different_url);
   state.DidNavigate(load_committed_details);
 
   EXPECT_TRUE(state.state_map().empty());
@@ -125,24 +139,36 @@ TEST_F(GeolocationSettingsStateTests, ShowPortOnSameHost) {
   GeolocationSettingsState state(&profile);
   GURL url_0("http://www.example.com");
 
-  NavigationEntry entry;
-  entry.set_url(url_0);
-  NavigationController::LoadCommittedDetails load_committed_details;
-  load_committed_details.entry = &entry;
+  scoped_ptr<NavigationEntry> entry(NavigationEntry::Create());
+  entry->SetURL(url_0);
+  content::LoadCommittedDetails load_committed_details;
+  load_committed_details.entry = entry.get();
   state.DidNavigate(load_committed_details);
 
-  profile.GetGeolocationContentSettingsMap()->SetContentSetting(
-      url_0, url_0, CONTENT_SETTING_ALLOW);
+  profile.GetHostContentSettingsMap()->SetContentSetting(
+      ContentSettingsPattern::FromURLNoWildcard(url_0),
+      ContentSettingsPattern::FromURLNoWildcard(url_0),
+      CONTENT_SETTINGS_TYPE_GEOLOCATION,
+      std::string(),
+      CONTENT_SETTING_ALLOW);
   state.OnGeolocationPermissionSet(url_0, true);
 
   GURL url_1("https://www.example.com");
-  profile.GetGeolocationContentSettingsMap()->SetContentSetting(
-      url_1, url_0, CONTENT_SETTING_ALLOW);
+  profile.GetHostContentSettingsMap()->SetContentSetting(
+      ContentSettingsPattern::FromURLNoWildcard(url_1),
+      ContentSettingsPattern::FromURLNoWildcard(url_0),
+      CONTENT_SETTINGS_TYPE_GEOLOCATION,
+      std::string(),
+      CONTENT_SETTING_ALLOW);
   state.OnGeolocationPermissionSet(url_1, true);
 
   GURL url_2("http://www.example1.com");
-  profile.GetGeolocationContentSettingsMap()->SetContentSetting(
-  url_2, url_0, CONTENT_SETTING_ALLOW);
+  profile.GetHostContentSettingsMap()->SetContentSetting(
+      ContentSettingsPattern::FromURLNoWildcard(url_2),
+      ContentSettingsPattern::FromURLNoWildcard(url_0),
+      CONTENT_SETTINGS_TYPE_GEOLOCATION,
+      std::string(),
+      CONTENT_SETTING_ALLOW);
   state.OnGeolocationPermissionSet(url_2, true);
 
   GeolocationSettingsState::StateMap state_map =
