@@ -1,7 +1,8 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "chrome/browser/google/google_util.h"
 #include "chrome/installer/util/google_update_settings.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
@@ -33,3 +34,18 @@ TEST_F(GoogleUpdateTest, LastRunTime) {
 }
 
 #endif  // defined(OS_WIN)
+
+TEST_F(GoogleUpdateTest, IsOrganicFirstRunBrandCodes) {
+  // Test some brand codes to ensure that future changes to this method won't
+  // go unnoticed.
+  EXPECT_FALSE(google_util::IsOrganicFirstRun("CHFO"));
+  EXPECT_FALSE(google_util::IsOrganicFirstRun("CHMA"));
+  EXPECT_TRUE(google_util::IsOrganicFirstRun("EUBA"));
+  EXPECT_TRUE(google_util::IsOrganicFirstRun("GGRA"));
+
+#if defined(OS_MACOSX)
+  // An empty brand string on Mac is used for channels other than stable,
+  // which are always organic.
+  EXPECT_TRUE(google_util::IsOrganicFirstRun(""));
+#endif
+}
