@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,32 +6,45 @@
 
 #include <vector>
 
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
+#include "chrome/browser/google/google_util.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/prefs/pref_service.h"
-#include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search_engines/template_url.h"
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/notification_service.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/common/url_fetcher.h"
 #include "grit/generated_resources.h"
 #include "net/base/load_flags.h"
+#include "net/base/net_util.h"
+#include "net/url_request/url_request_context_getter.h"
 #include "net/url_request/url_request_status.h"
 #include "ui/base/l10n/l10n_util.h"
 
+using content::NavigationController;
+using content::OpenURLParams;
+using content::Referrer;
+using content::WebContents;
+
 namespace {
 
-InfoBarDelegate* CreateInfobar(TabContents* tab_contents,
+InfoBarDelegate* CreateInfobar(InfoBarTabHelper* infobar_helper,
                                GoogleURLTracker* google_url_tracker,
                                const GURL& new_google_url) {
-  InfoBarDelegate* infobar = new GoogleURLTrackerInfoBarDelegate(tab_contents,
+  InfoBarDelegate* infobar = new GoogleURLTrackerInfoBarDelegate(infobar_helper,
       google_url_tracker, new_google_url);
-  tab_contents->AddInfoBar(infobar);
+  infobar_helper->AddInfoBar(infobar);
   return infobar;
 }
 
@@ -40,10 +53,10 @@ InfoBarDelegate* CreateInfobar(TabContents* tab_contents,
 // GoogleURLTrackerInfoBarDelegate --------------------------------------------
 
 GoogleURLTrackerInfoBarDelegate::GoogleURLTrackerInfoBarDelegate(
-    TabContents* tab_contents,
+    InfoBarTabHelper* infobar_helper,
     GoogleURLTracker* google_url_tracker,
     const GURL& new_google_url)
-    : ConfirmInfoBarDelegate(tab_contents),
+    : ConfirmInfoBarDelegate(infobar_helper),
       google_url_tracker_(google_url_tracker),
       new_google_url_(new_google_url) {
 }
@@ -59,29 +72,42 @@ bool GoogleURLTrackerInfoBarDelegate::Cancel() {
   return true;
 }
 
-void GoogleURLTrackerInfoBarDelegate::InfoBarClosed() {
-  google_url_tracker_->InfoBarClosed();
-  delete this;
+string16 GoogleURLTrackerInfoBarDelegate::GetLinkText() const {
+  return l10n_util::GetStringUTF16(IDS_LEARN_MORE);
+}
+
+bool GoogleURLTrackerInfoBarDelegate::LinkClicked(
+    WindowOpenDisposition disposition) {
+  OpenURLParams params(
+      google_util::AppendGoogleLocaleParam(GURL(
+          "https://www.google.com/support/chrome/bin/answer.py?answer=1618699")),
+      Referrer(),
+      (disposition == CURRENT_TAB) ? NEW_FOREGROUND_TAB : disposition,
+      content::PAGE_TRANSITION_LINK, false);
+  owner()->web_contents()->OpenURL(params);
+  return false;
 }
 
 GoogleURLTrackerInfoBarDelegate::~GoogleURLTrackerInfoBarDelegate() {
+  google_url_tracker_->InfoBarClosed();
 }
 
 string16 GoogleURLTrackerInfoBarDelegate::GetMessageText() const {
-  // TODO(ukai): change new_google_url to google_base_domain?
   return l10n_util::GetStringFUTF16(IDS_GOOGLE_URL_TRACKER_INFOBAR_MESSAGE,
-                                    UTF8ToUTF16(new_google_url_.spec()));
-}
-
-int GoogleURLTrackerInfoBarDelegate::GetButtons() const {
-  return BUTTON_OK | BUTTON_CANCEL;
+                                    GetHost(true), GetHost(false));
 }
 
 string16 GoogleURLTrackerInfoBarDelegate::GetButtonLabel(
     InfoBarButton button) const {
-  return l10n_util::GetStringUTF16((button == BUTTON_OK) ?
-                                   IDS_CONFIRM_MESSAGEBOX_YES_BUTTON_LABEL :
-                                   IDS_CONFIRM_MESSAGEBOX_NO_BUTTON_LABEL);
+  bool new_host = (button == BUTTON_OK);
+  return l10n_util::GetStringFUTF16(new_host ?
+      IDS_GOOGLE_URL_TRACKER_INFOBAR_SWITCH :
+      IDS_GOOGLE_URL_TRACKER_INFOBAR_DONT_SWITCH, GetHost(new_host));
+}
+
+string16 GoogleURLTrackerInfoBarDelegate::GetHost(bool new_host) const {
+  return net::StripWWW(UTF8ToUTF16(
+      (new_host ? new_google_url_ : google_url_tracker_->GoogleURL()).host()));
 }
 
 
@@ -96,29 +122,24 @@ GoogleURLTracker::GoogleURLTracker()
     : infobar_creator_(&CreateInfobar),
       google_url_(g_browser_process->local_state()->GetString(
           prefs::kLastKnownGoogleURL)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(runnable_method_factory_(this)),
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_ptr_factory_(this)),
       fetcher_id_(0),
       queue_wakeup_task_(true),
       in_startup_sleep_(true),
       already_fetched_(false),
       need_to_fetch_(false),
-      request_context_available_(!!Profile::GetDefaultRequestContext()),
       need_to_prompt_(false),
       controller_(NULL),
       infobar_(NULL) {
-  registrar_.Add(this, NotificationType::DEFAULT_REQUEST_CONTEXT_AVAILABLE,
-                 NotificationService::AllSources());
-
-  net::NetworkChangeNotifier::AddObserver(this);
+  net::NetworkChangeNotifier::AddIPAddressObserver(this);
 
   MessageLoop::current()->PostTask(FROM_HERE,
-                                   runnable_method_factory_.NewRunnableMethod(
-                                   &GoogleURLTracker::QueueWakeupTask));
+      base::Bind(&GoogleURLTracker::QueueWakeupTask,
+                 weak_ptr_factory_.GetWeakPtr()));
 }
 
 GoogleURLTracker::~GoogleURLTracker() {
-  runnable_method_factory_.RevokeAll();
-  net::NetworkChangeNotifier::RemoveObserver(this);
+  net::NetworkChangeNotifier::RemoveIPAddressObserver(this);
 }
 
 // static
@@ -167,10 +188,11 @@ void GoogleURLTracker::QueueWakeupTask() {
   // browser is starting up, and if so, come back later", but there is currently
   // no function to do this.
   static const int kStartFetchDelayMS = 5000;
+
   MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      runnable_method_factory_.NewRunnableMethod(
-          &GoogleURLTracker::FinishSleep),
-      kStartFetchDelayMS);
+      base::Bind(&GoogleURLTracker::FinishSleep,
+                 weak_ptr_factory_.GetWeakPtr()),
+      base::TimeDelta::FromMilliseconds(kStartFetchDelayMS));
 }
 
 void GoogleURLTracker::FinishSleep() {
@@ -185,8 +207,7 @@ void GoogleURLTracker::StartFetchIfDesirable() {
   //
   // See comments in header on the class, on RequestServerCheck(), and on the
   // various members here for more detail on exactly what the conditions are.
-  if (in_startup_sleep_ || already_fetched_ || !need_to_fetch_ ||
-      !request_context_available_)
+  if (in_startup_sleep_ || already_fetched_ || !need_to_fetch_)
     return;
 
   if (CommandLine::ForCurrentProcess()->HasSwitch(
@@ -194,34 +215,30 @@ void GoogleURLTracker::StartFetchIfDesirable() {
     return;
 
   already_fetched_ = true;
-  fetcher_.reset(URLFetcher::Create(fetcher_id_, GURL(kSearchDomainCheckURL),
-                                    URLFetcher::GET, this));
+  fetcher_.reset(content::URLFetcher::Create(
+      fetcher_id_, GURL(kSearchDomainCheckURL), content::URLFetcher::GET,
+      this));
   ++fetcher_id_;
-  // We don't want this fetch to affect existing state in the profile.  For
+  // We don't want this fetch to affect existing state in local_state.  For
   // example, if a user has no Google cookies, this automatic check should not
   // cause one to be set, lest we alarm the user.
-  fetcher_->set_load_flags(net::LOAD_DISABLE_CACHE |
-                           net::LOAD_DO_NOT_SAVE_COOKIES);
-  fetcher_->set_request_context(Profile::GetDefaultRequestContext());
+  fetcher_->SetLoadFlags(net::LOAD_DISABLE_CACHE |
+                         net::LOAD_DO_NOT_SAVE_COOKIES);
+  fetcher_->SetRequestContext(g_browser_process->system_request_context());
 
   // Configure to max_retries at most kMaxRetries times for 5xx errors.
   static const int kMaxRetries = 5;
-  fetcher_->set_max_retries(kMaxRetries);
+  fetcher_->SetMaxRetries(kMaxRetries);
 
   fetcher_->Start();
 }
 
-void GoogleURLTracker::OnURLFetchComplete(const URLFetcher* source,
-                                          const GURL& url,
-                                          const net::URLRequestStatus& status,
-                                          int response_code,
-                                          const ResponseCookies& cookies,
-                                          const std::string& data) {
+void GoogleURLTracker::OnURLFetchComplete(const content::URLFetcher* source) {
   // Delete the fetcher on this function's exit.
-  scoped_ptr<URLFetcher> clean_up_fetcher(fetcher_.release());
+  scoped_ptr<content::URLFetcher> clean_up_fetcher(fetcher_.release());
 
   // Don't update the URL if the request didn't succeed.
-  if (!status.is_success() || (response_code != 200)) {
+  if (!source->GetStatus().is_success() || (source->GetResponseCode() != 200)) {
     already_fetched_ = false;
     return;
   }
@@ -229,7 +246,8 @@ void GoogleURLTracker::OnURLFetchComplete(const URLFetcher* source,
   // See if the response data was one we want to use, and if so, convert to the
   // appropriate Google base URL.
   std::string url_str;
-  TrimWhitespace(data, TRIM_ALL, &url_str);
+  source->GetResponseAsString(&url_str);
+  TrimWhitespace(url_str, TRIM_ALL, &url_str);
 
   if (!StartsWithASCII(url_str, ".google.", false))
     return;
@@ -268,15 +286,17 @@ void GoogleURLTracker::AcceptGoogleURL(const GURL& new_google_url) {
                                               google_url_.spec());
   g_browser_process->local_state()->SetString(prefs::kLastPromptedGoogleURL,
                                               google_url_.spec());
-  NotificationService::current()->Notify(NotificationType::GOOGLE_URL_UPDATED,
-                                         NotificationService::AllSources(),
-                                         NotificationService::NoDetails());
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_GOOGLE_URL_UPDATED,
+      content::NotificationService::AllSources(),
+      content::NotificationService::NoDetails());
   need_to_prompt_ = false;
 }
 
 void GoogleURLTracker::CancelGoogleURL(const GURL& new_google_url) {
   g_browser_process->local_state()->SetString(prefs::kLastPromptedGoogleURL,
                                               new_google_url.spec());
+  need_to_prompt_ = false;
 }
 
 void GoogleURLTracker::InfoBarClosed() {
@@ -293,39 +313,33 @@ void GoogleURLTracker::RedoSearch() {
   replacements.SetHost(google_url_.host().data(),
                        url_parse::Component(0, google_url_.host().length()));
   GURL new_search_url(search_url_.ReplaceComponents(replacements));
-  if (new_search_url.is_valid())
-    controller_->tab_contents()->OpenURL(new_search_url, GURL(), CURRENT_TAB,
-                                         PageTransition::GENERATED);
+  if (new_search_url.is_valid()) {
+    OpenURLParams params(new_search_url, Referrer(), CURRENT_TAB,
+                         content::PAGE_TRANSITION_GENERATED, false);
+    controller_->GetWebContents()->OpenURL(params);
+  }
 }
 
-void GoogleURLTracker::Observe(NotificationType type,
-                               const NotificationSource& source,
-                               const NotificationDetails& details) {
-  switch (type.value) {
-    case NotificationType::DEFAULT_REQUEST_CONTEXT_AVAILABLE:
-      registrar_.Remove(this,
-                        NotificationType::DEFAULT_REQUEST_CONTEXT_AVAILABLE,
-                        NotificationService::AllSources());
-      request_context_available_ = true;
-      StartFetchIfDesirable();
-      break;
-
-    case NotificationType::NAV_ENTRY_PENDING: {
+void GoogleURLTracker::Observe(int type,
+                               const content::NotificationSource& source,
+                               const content::NotificationDetails& details) {
+  switch (type) {
+    case content::NOTIFICATION_NAV_ENTRY_PENDING: {
       NavigationController* controller =
-          Source<NavigationController>(source).ptr();
-      OnNavigationPending(source, controller->pending_entry()->url());
+          content::Source<NavigationController>(source).ptr();
+      OnNavigationPending(source, controller->GetPendingEntry()->GetURL());
       break;
     }
 
-    case NotificationType::NAV_ENTRY_COMMITTED:
-    case NotificationType::TAB_CLOSED:
+    case content::NOTIFICATION_NAV_ENTRY_COMMITTED:
+    case content::NOTIFICATION_TAB_CLOSED:
       OnNavigationCommittedOrTabClosed(
-          Source<NavigationController>(source).ptr()->tab_contents(),
-          type.value);
+          content::Source<NavigationController>(source).ptr()->
+              GetWebContents(), type);
       break;
 
     default:
-      NOTREACHED() << "Unknown notification received:" << type.value;
+      NOTREACHED() << "Unknown notification received:" << type;
   }
 }
 
@@ -338,32 +352,33 @@ void GoogleURLTracker::SearchCommitted() {
   if (registrar_.IsEmpty() && (need_to_prompt_ || fetcher_.get())) {
     // This notification will fire a bit later in the same call chain we're
     // currently in.
-    registrar_.Add(this, NotificationType::NAV_ENTRY_PENDING,
-                   NotificationService::AllSources());
+    registrar_.Add(this, content::NOTIFICATION_NAV_ENTRY_PENDING,
+                   content::NotificationService::AllSources());
   }
 }
 
-void GoogleURLTracker::OnNavigationPending(const NotificationSource& source,
-                                           const GURL& pending_url) {
-  controller_ = Source<NavigationController>(source).ptr();
+void GoogleURLTracker::OnNavigationPending(
+    const content::NotificationSource& source,
+    const GURL& pending_url) {
+  controller_ = content::Source<NavigationController>(source).ptr();
   search_url_ = pending_url;
-  registrar_.Remove(this, NotificationType::NAV_ENTRY_PENDING,
-                    NotificationService::AllSources());
+  registrar_.Remove(this, content::NOTIFICATION_NAV_ENTRY_PENDING,
+                    content::NotificationService::AllSources());
   // Start listening for the commit notification. We also need to listen for the
   // tab close command since that means the load will never commit.
-  registrar_.Add(this, NotificationType::NAV_ENTRY_COMMITTED,
-                 Source<NavigationController>(controller_));
-  registrar_.Add(this, NotificationType::TAB_CLOSED,
-                 Source<NavigationController>(controller_));
+  registrar_.Add(this, content::NOTIFICATION_NAV_ENTRY_COMMITTED,
+                 content::Source<NavigationController>(controller_));
+  registrar_.Add(this, content::NOTIFICATION_TAB_CLOSED,
+                 content::Source<NavigationController>(controller_));
 }
 
 void GoogleURLTracker::OnNavigationCommittedOrTabClosed(
-    TabContents* tab_contents,
-    NotificationType::Type type) {
+    WebContents* web_contents,
+    int type) {
   registrar_.RemoveAll();
 
-  if (type == NotificationType::NAV_ENTRY_COMMITTED) {
-    ShowGoogleURLInfoBarIfNecessary(tab_contents);
+  if (type == content::NOTIFICATION_NAV_ENTRY_COMMITTED) {
+    ShowGoogleURLInfoBarIfNecessary(web_contents);
   } else {
     controller_ = NULL;
     infobar_ = NULL;
@@ -371,10 +386,17 @@ void GoogleURLTracker::OnNavigationCommittedOrTabClosed(
 }
 
 void GoogleURLTracker::ShowGoogleURLInfoBarIfNecessary(
-    TabContents* tab_contents) {
+    WebContents* web_contents) {
   if (!need_to_prompt_)
     return;
   DCHECK(!fetched_google_url_.is_empty());
 
-  infobar_ = (*infobar_creator_)(tab_contents, this, fetched_google_url_);
+  // |tab_contents| can be NULL during tests.
+  InfoBarTabHelper* infobar_helper = NULL;
+  if (web_contents) {
+    TabContentsWrapper* wrapper =
+        TabContentsWrapper::GetCurrentWrapperForContents(web_contents);
+    infobar_helper = wrapper->infobar_tab_helper();
+  }
+  infobar_ = (*infobar_creator_)(infobar_helper, this, fetched_google_url_);
 }
