@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,18 +9,21 @@
 #include <string>
 
 #include "base/gtest_prod_util.h"
-#include "base/scoped_ptr.h"
-#include "chrome/browser/tab_contents/infobar_delegate.h"
-#include "chrome/common/net/url_fetcher.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
+#include "chrome/browser/tab_contents/confirm_infobar_delegate.h"
+#include "content/public/common/url_fetcher_delegate.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/network_change_notifier.h"
 
-class NavigationController;
 class PrefService;
-class TabContents;
-class TemplateURL;
+
+namespace content {
+class NavigationController;
+class WebContents;
+}
 
 // This object is responsible for checking the Google URL once per network
 // change, and if necessary prompting the user to see if they want to change to
@@ -35,9 +38,9 @@ class TemplateURL;
 // To protect users' privacy and reduce server load, no updates will be
 // performed (ever) unless at least one consumer registers interest by calling
 // RequestServerCheck().
-class GoogleURLTracker : public URLFetcher::Delegate,
-                         public NotificationObserver,
-                         public net::NetworkChangeNotifier::Observer {
+class GoogleURLTracker : public content::URLFetcherDelegate,
+                         public content::NotificationObserver,
+                         public net::NetworkChangeNotifier::IPAddressObserver {
  public:
   // Only the main browser process loop should call this, when setting up
   // g_browser_process->google_url_tracker_.  No code other than the
@@ -84,12 +87,12 @@ class GoogleURLTracker : public URLFetcher::Delegate,
  private:
   friend class GoogleURLTrackerTest;
 
-  typedef InfoBarDelegate* (*InfobarCreator)(TabContents*,
+  typedef InfoBarDelegate* (*InfobarCreator)(InfoBarTabHelper*,
                                              GoogleURLTracker*,
                                              const GURL&);
 
   // Registers consumer interest in getting an updated URL from the server.
-  // It will be notified as NotificationType::GOOGLE_URL_UPDATED, so the
+  // It will be notified as chrome::GOOGLE_URL_UPDATED, so the
   // consumer should observe this notification before calling this.
   void SetNeedToFetch();
 
@@ -105,38 +108,33 @@ class GoogleURLTracker : public URLFetcher::Delegate,
   // it and can currently do so.
   void StartFetchIfDesirable();
 
-  // URLFetcher::Delegate
-  virtual void OnURLFetchComplete(const URLFetcher *source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  // content::URLFetcherDelegate
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
-  // NetworkChangeNotifier::Observer
-  virtual void OnIPAddressChanged();
+  // NetworkChangeNotifier::IPAddressObserver
+  virtual void OnIPAddressChanged() OVERRIDE;
 
   void SearchCommitted();
-  void OnNavigationPending(const NotificationSource& source,
+  void OnNavigationPending(const content::NotificationSource& source,
                            const GURL& pending_url);
-  void OnNavigationCommittedOrTabClosed(TabContents* tab_contents,
-                                        NotificationType::Type type);
-  void ShowGoogleURLInfoBarIfNecessary(TabContents* tab_contents);
+  void OnNavigationCommittedOrTabClosed(content::WebContents* web_contents,
+                                        int type);
+  void ShowGoogleURLInfoBarIfNecessary(content::WebContents* web_contents);
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
   InfobarCreator infobar_creator_;
   // TODO(ukai): GoogleURLTracker should track google domain (e.g. google.co.uk)
   // rather than URL (e.g. http://www.google.co.uk/), so that user could
   // configure to use https in search engine templates.
   GURL google_url_;
   GURL fetched_google_url_;
-  ScopedRunnableMethodFactory<GoogleURLTracker> runnable_method_factory_;
-  scoped_ptr<URLFetcher> fetcher_;
+  base::WeakPtrFactory<GoogleURLTracker> weak_ptr_factory_;
+  scoped_ptr<content::URLFetcher> fetcher_;
   int fetcher_id_;
   bool queue_wakeup_task_;
   bool in_startup_sleep_;  // True if we're in the five-second "no fetching"
@@ -147,15 +145,11 @@ class GoogleURLTracker : public URLFetcher::Delegate,
                            // updated URL.  If this is never set, we won't
                            // bother to fetch anything.
                            // Consumers should observe
-                           // NotificationType::GOOGLE_URL_UPDATED.
-  bool request_context_available_;
-                           // True when the profile has been loaded and the
-                           // default request context created, so we can
-                           // actually do the fetch with the right data.
+                           // chrome::GOOGLE_URL_UPDATED.
   bool need_to_prompt_;    // True if the last fetched Google URL is not
                            // matched with current user's default Google URL
                            // nor the last prompted Google URL.
-  NavigationController* controller_;
+  content::NavigationController* controller_;
   InfoBarDelegate* infobar_;
   GURL search_url_;
 
@@ -167,14 +161,15 @@ class GoogleURLTracker : public URLFetcher::Delegate,
 // code can subclass it.
 class GoogleURLTrackerInfoBarDelegate : public ConfirmInfoBarDelegate {
  public:
-  GoogleURLTrackerInfoBarDelegate(TabContents* tab_contents,
+  GoogleURLTrackerInfoBarDelegate(InfoBarTabHelper* infobar_helper,
                                   GoogleURLTracker* google_url_tracker,
                                   const GURL& new_google_url);
 
-  // ConfirmInfoBarDelegate
-  virtual bool Accept();
-  virtual bool Cancel();
-  virtual void InfoBarClosed();
+  // ConfirmInfoBarDelegate:
+  virtual bool Accept() OVERRIDE;
+  virtual bool Cancel() OVERRIDE;
+  virtual string16 GetLinkText() const OVERRIDE;
+  virtual bool LinkClicked(WindowOpenDisposition disposition) OVERRIDE;
 
  protected:
   virtual ~GoogleURLTrackerInfoBarDelegate();
@@ -183,10 +178,12 @@ class GoogleURLTrackerInfoBarDelegate : public ConfirmInfoBarDelegate {
   const GURL new_google_url_;
 
  private:
-  // ConfirmInfoBarDelegate
-  virtual string16 GetMessageText() const;
-  virtual int GetButtons() const;
-  virtual string16 GetButtonLabel(InfoBarButton button) const;
+  // ConfirmInfoBarDelegate:
+  virtual string16 GetMessageText() const OVERRIDE;
+  virtual string16 GetButtonLabel(InfoBarButton button) const OVERRIDE;
+
+  // Returns the portion of the appropriate hostname to display.
+  string16 GetHost(bool new_host) const;
 
   DISALLOW_COPY_AND_ASSIGN(GoogleURLTrackerInfoBarDelegate);
 };
