@@ -7,33 +7,35 @@
 #include "base/command_line.h"
 #include "base/message_loop.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/infobar_delegate.h"
-#include "chrome/common/net/url_fetcher.h"
-#include "chrome/common/net/url_request_context_getter.h"
-#include "chrome/common/net/test_url_fetcher_factory.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/browser/prefs/browser_prefs.h"
+#include "chrome/browser/tab_contents/confirm_infobar_delegate.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/testing_browser_process.h"
-#include "chrome/test/testing_pref_service.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_browser_process.h"
+#include "chrome/test/base/testing_pref_service.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/common/url_fetcher.h"
+#include "content/test/test_browser_thread.h"
+#include "content/test/test_url_fetcher_factory.h"
 #include "net/url_request/url_request.h"
-#include "net/url_request/url_request_unittest.h"
+#include "net/url_request/url_request_context_getter.h"
+#include "net/url_request/url_request_test_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using content::BrowserThread;
+
 // TestNotificationObserver ---------------------------------------------------
 
 namespace {
 
-class TestNotificationObserver : public NotificationObserver {
+class TestNotificationObserver : public content::NotificationObserver {
  public:
   TestNotificationObserver();
   virtual ~TestNotificationObserver();
 
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details);
   bool notified() const { return notified_; }
   void clear_notified() { notified_ = false; }
 
@@ -47,9 +49,10 @@ TestNotificationObserver::TestNotificationObserver() : notified_(false) {
 TestNotificationObserver::~TestNotificationObserver() {
 }
 
-void TestNotificationObserver::Observe(NotificationType type,
-                                       const NotificationSource& source,
-                                       const NotificationDetails& details) {
+void TestNotificationObserver::Observe(
+    int type,
+    const content::NotificationSource& source,
+    const content::NotificationDetails& details) {
   notified_ = true;
 }
 
@@ -68,7 +71,7 @@ class TestInfoBarDelegate : public InfoBarDelegate {
   virtual ~TestInfoBarDelegate();
 
   // InfoBarDelegate:
-  virtual InfoBar* CreateInfoBar();
+  virtual InfoBar* CreateInfoBar(InfoBarTabHelper* infobar_helper) OVERRIDE;
 
   GoogleURLTracker* google_url_tracker_;
   GURL new_google_url_;
@@ -84,12 +87,12 @@ TestInfoBarDelegate::TestInfoBarDelegate(GoogleURLTracker* google_url_tracker,
 TestInfoBarDelegate::~TestInfoBarDelegate() {
 }
 
-InfoBar* TestInfoBarDelegate::CreateInfoBar() {
+InfoBar* TestInfoBarDelegate::CreateInfoBar(InfoBarTabHelper* infobar_helper) {
   return NULL;
 }
 
 InfoBarDelegate* CreateTestInfobar(
-    TabContents* tab_contents,
+    InfoBarTabHelper* infobar_helper,
     GoogleURLTracker* google_url_tracker,
     const GURL& new_google_url) {
   return new TestInfoBarDelegate(google_url_tracker, new_google_url);
@@ -109,7 +112,6 @@ class GoogleURLTrackerTest : public testing::Test {
   virtual void SetUp();
   virtual void TearDown();
 
-  void CreateRequestContext();
   TestURLFetcher* GetFetcherByID(int expected_id);
   void MockSearchDomainCheckResponse(int expected_id,
                                      const std::string& domain);
@@ -132,68 +134,41 @@ class GoogleURLTrackerTest : public testing::Test {
   scoped_ptr<TestNotificationObserver> observer_;
 
  private:
-  MessageLoop* message_loop_;
-  BrowserThread* io_thread_;
+  MessageLoop message_loop_;
+  content::TestBrowserThread io_thread_;
   scoped_ptr<net::NetworkChangeNotifier> network_change_notifier_;
-  scoped_ptr<TestingProfile> testing_profile_;
+  ScopedTestingLocalState local_state_;
 
   TestURLFetcherFactory fetcher_factory_;
-  NotificationRegistrar registrar_;
-
-  URLRequestContextGetter* original_default_request_context_;
+  content::NotificationRegistrar registrar_;
 };
 
 GoogleURLTrackerTest::GoogleURLTrackerTest()
     : observer_(new TestNotificationObserver),
-      message_loop_(NULL),
-      io_thread_(NULL),
-      original_default_request_context_(NULL) {
+      message_loop_(MessageLoop::TYPE_IO),
+      io_thread_(BrowserThread::IO, &message_loop_),
+      local_state_(static_cast<TestingBrowserProcess*>(g_browser_process)) {
 }
 
 GoogleURLTrackerTest::~GoogleURLTrackerTest() {
 }
 
 void GoogleURLTrackerTest::SetUp() {
-  original_default_request_context_ = Profile::GetDefaultRequestContext();
-  Profile::set_default_request_context(NULL);
-  message_loop_ = new MessageLoop(MessageLoop::TYPE_IO);
-  io_thread_ = new BrowserThread(BrowserThread::IO, message_loop_);
   network_change_notifier_.reset(net::NetworkChangeNotifier::CreateMock());
-  testing_profile_.reset(new TestingProfile);
-  TestingBrowserProcess* testing_browser_process =
-      static_cast<TestingBrowserProcess*>(g_browser_process);
-  PrefService* pref_service = testing_profile_->GetPrefs();
-  testing_browser_process->SetPrefService(pref_service);
   GoogleURLTracker* tracker = new GoogleURLTracker;
   tracker->queue_wakeup_task_ = false;
   MessageLoop::current()->RunAllPending();
-  testing_browser_process->SetGoogleURLTracker(tracker);
+  static_cast<TestingBrowserProcess*>(g_browser_process)->SetGoogleURLTracker(
+      tracker);
 
-  URLFetcher::set_factory(&fetcher_factory_);
   g_browser_process->google_url_tracker()->infobar_creator_ =
       &CreateTestInfobar;
 }
 
 void GoogleURLTrackerTest::TearDown() {
-  URLFetcher::set_factory(NULL);
-  TestingBrowserProcess* testing_browser_process =
-      static_cast<TestingBrowserProcess*>(g_browser_process);
-  testing_browser_process->SetGoogleURLTracker(NULL);
-  testing_browser_process->SetPrefService(NULL);
-  testing_profile_.reset();
+  static_cast<TestingBrowserProcess*>(g_browser_process)->SetGoogleURLTracker(
+      NULL);
   network_change_notifier_.reset();
-  delete io_thread_;
-  delete message_loop_;
-  Profile::set_default_request_context(original_default_request_context_);
-  original_default_request_context_ = NULL;
-}
-
-void GoogleURLTrackerTest::CreateRequestContext() {
-  testing_profile_->CreateRequestContext();
-  Profile::set_default_request_context(testing_profile_->GetRequestContext());
-  NotificationService::current()->Notify(
-      NotificationType::DEFAULT_REQUEST_CONTEXT_AVAILABLE,
-      NotificationService::AllSources(), NotificationService::NoDetails());
 }
 
 TestURLFetcher* GoogleURLTrackerTest::GetFetcherByID(int expected_id) {
@@ -206,19 +181,20 @@ void GoogleURLTrackerTest::MockSearchDomainCheckResponse(
   TestURLFetcher* fetcher = fetcher_factory_.GetFetcherByID(expected_id);
   if (!fetcher)
     return;
-  fetcher->delegate()->OnURLFetchComplete(fetcher,
-      GURL(GoogleURLTracker::kSearchDomainCheckURL), net::URLRequestStatus(),
-      200, ResponseCookies(), domain);
+  fetcher->set_url(GURL(GoogleURLTracker::kSearchDomainCheckURL));
+  fetcher->set_response_code(200);
+  fetcher->SetResponseString(domain);
+  fetcher->delegate()->OnURLFetchComplete(fetcher);
   // At this point, |fetcher| is deleted.
   MessageLoop::current()->RunAllPending();
 }
 
 void GoogleURLTrackerTest::RequestServerCheck() {
   if (!registrar_.IsRegistered(observer_.get(),
-                               NotificationType::GOOGLE_URL_UPDATED,
-                               NotificationService::AllSources())) {
-    registrar_.Add(observer_.get(), NotificationType::GOOGLE_URL_UPDATED,
-                   NotificationService::AllSources());
+                               chrome::NOTIFICATION_GOOGLE_URL_UPDATED,
+                               content::NotificationService::AllSources())) {
+    registrar_.Add(observer_.get(), chrome::NOTIFICATION_GOOGLE_URL_UPDATED,
+                   content::NotificationService::AllSources());
   }
   GoogleURLTracker::RequestServerCheck();
   MessageLoop::current()->RunAllPending();
@@ -257,8 +233,8 @@ void GoogleURLTrackerTest::SearchCommitted(const GURL& search_url) {
       g_browser_process->google_url_tracker();
   google_url_tracker->SearchCommitted();
   if (google_url_tracker->registrar_.IsRegistered(google_url_tracker,
-      NotificationType::NAV_ENTRY_PENDING,
-      NotificationService::AllSources()))
+      content::NOTIFICATION_NAV_ENTRY_PENDING,
+      content::NotificationService::AllSources()))
     google_url_tracker->search_url_ = search_url;
 }
 
@@ -314,7 +290,6 @@ void GoogleURLTrackerTest::ExpectDefaultURLs() {
 // Tests ----------------------------------------------------------------------
 
 TEST_F(GoogleURLTrackerTest, DontFetchWhenNoOneRequestsCheck) {
-  CreateRequestContext();
   ExpectDefaultURLs();
   FinishSleep();
   // No one called RequestServerCheck() so nothing should have happened.
@@ -324,7 +299,6 @@ TEST_F(GoogleURLTrackerTest, DontFetchWhenNoOneRequestsCheck) {
 }
 
 TEST_F(GoogleURLTrackerTest, UpdateOnFirstRun) {
-  CreateRequestContext();
   RequestServerCheck();
   EXPECT_FALSE(GetFetcherByID(0));
   ExpectDefaultURLs();
@@ -339,7 +313,6 @@ TEST_F(GoogleURLTrackerTest, UpdateOnFirstRun) {
 }
 
 TEST_F(GoogleURLTrackerTest, DontUpdateWhenUnchanged) {
-  CreateRequestContext();
   SetLastPromptedGoogleURL(GURL("http://www.google.co.uk/"));
 
   RequestServerCheck();
@@ -358,7 +331,6 @@ TEST_F(GoogleURLTrackerTest, DontUpdateWhenUnchanged) {
 }
 
 TEST_F(GoogleURLTrackerTest, UpdatePromptedURLOnReturnToPreviousLocation) {
-  CreateRequestContext();
   SetLastPromptedGoogleURL(GURL("http://www.google.co.jp/"));
   SetGoogleURL(GURL("http://www.google.co.uk/"));
   RequestServerCheck();
@@ -371,7 +343,6 @@ TEST_F(GoogleURLTrackerTest, UpdatePromptedURLOnReturnToPreviousLocation) {
 }
 
 TEST_F(GoogleURLTrackerTest, RefetchOnIPAddressChange) {
-  CreateRequestContext();
   RequestServerCheck();
   FinishSleep();
   MockSearchDomainCheckResponse(0, ".google.co.uk");
@@ -389,7 +360,6 @@ TEST_F(GoogleURLTrackerTest, RefetchOnIPAddressChange) {
 }
 
 TEST_F(GoogleURLTrackerTest, DontRefetchWhenNoOneRequestsCheck) {
-  CreateRequestContext();
   FinishSleep();
   NotifyIPAddressChanged();
   // No one called RequestServerCheck() so nothing should have happened.
@@ -399,7 +369,6 @@ TEST_F(GoogleURLTrackerTest, DontRefetchWhenNoOneRequestsCheck) {
 }
 
 TEST_F(GoogleURLTrackerTest, FetchOnLateRequest) {
-  CreateRequestContext();
   FinishSleep();
   NotifyIPAddressChanged();
 
@@ -413,7 +382,6 @@ TEST_F(GoogleURLTrackerTest, FetchOnLateRequest) {
 }
 
 TEST_F(GoogleURLTrackerTest, SearchingDoesNothingIfNoNeedToPrompt) {
-  CreateRequestContext();
   RequestServerCheck();
   FinishSleep();
   MockSearchDomainCheckResponse(0, ".google.co.uk");
@@ -433,7 +401,6 @@ TEST_F(GoogleURLTrackerTest, SearchingDoesNothingIfNoNeedToPrompt) {
 }
 
 TEST_F(GoogleURLTrackerTest, InfobarClosed) {
-  CreateRequestContext();
   SetLastPromptedGoogleURL(GURL("http://www.google.co.uk/"));
   RequestServerCheck();
   FinishSleep();
@@ -452,7 +419,6 @@ TEST_F(GoogleURLTrackerTest, InfobarClosed) {
 }
 
 TEST_F(GoogleURLTrackerTest, InfobarRefused) {
-  CreateRequestContext();
   SetLastPromptedGoogleURL(GURL("http://www.google.co.uk/"));
   RequestServerCheck();
   FinishSleep();
@@ -472,7 +438,6 @@ TEST_F(GoogleURLTrackerTest, InfobarRefused) {
 }
 
 TEST_F(GoogleURLTrackerTest, InfobarAccepted) {
-  CreateRequestContext();
   SetLastPromptedGoogleURL(GURL("http://www.google.co.uk/"));
   RequestServerCheck();
   FinishSleep();
