@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,13 +9,13 @@
 #include "base/compiler_specific.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
-#include "base/scoped_ptr.h"
 #include "base/scoped_temp_dir.h"
 #include "base/string16.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
-#include "chrome/browser/browser_thread.h"
+#include "chrome/browser/bookmarks/bookmark_utils.h"
 #include "chrome/browser/history/archived_database.h"
 #include "chrome/browser/history/expire_history_backend.h"
 #include "chrome/browser/history/history_database.h"
@@ -23,16 +23,19 @@
 #include "chrome/browser/history/text_database_manager.h"
 #include "chrome/browser/history/thumbnail_database.h"
 #include "chrome/browser/history/top_sites.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/thumbnail_score.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "chrome/tools/profiles/thumbnail-inl.h"
-#include "gfx/codec/jpeg_codec.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "ui/gfx/codec/jpeg_codec.h"
 
 using base::Time;
 using base::TimeDelta;
 using base::TimeTicks;
+using content::BrowserThread;
 
 // Filename constants.
 static const FilePath::CharType kHistoryFile[] = FILE_PATH_LITERAL("History");
@@ -65,9 +68,11 @@ class ExpireHistoryTest : public testing::Test,
   void AddExampleSourceData(const GURL& url, URLID* id);
 
   // Returns true if the given favicon/thumanil has an entry in the DB.
-  bool HasFavIcon(FavIconID favicon_id);
+  bool HasFavicon(FaviconID favicon_id);
   bool HasThumbnail(URLID url_id);
 
+  FaviconID GetFavicon(const GURL& page_url, IconType icon_type);
+
   // Returns the number of text matches for the given URL in the example data
   // added by AddExampleData.
   int CountTextMatchesForURL(const GURL& url);
@@ -85,7 +90,7 @@ class ExpireHistoryTest : public testing::Test,
 
   void StarURL(const GURL& url) {
     bookmark_model_.AddURL(
-        bookmark_model_.GetBookmarkBarNode(), 0, string16(), url);
+        bookmark_model_.bookmark_bar_node(), 0, string16(), url);
   }
 
   static bool IsStringInFile(const FilePath& filename, const char* str);
@@ -99,8 +104,8 @@ class ExpireHistoryTest : public testing::Test,
   BookmarkModel bookmark_model_;
 
   MessageLoopForUI message_loop_;
-  BrowserThread ui_thread_;
-  BrowserThread db_thread_;
+  content::TestBrowserThread ui_thread_;
+  content::TestBrowserThread db_thread_;
 
   ExpireHistoryBackend expirer_;
 
@@ -117,7 +122,7 @@ class ExpireHistoryTest : public testing::Test,
   // Notifications intended to be broadcast, we can check these values to make
   // sure that the deletor is doing the correct broadcasts. We own the details
   // pointers.
-  typedef std::vector< std::pair<NotificationType, HistoryDetails*> >
+  typedef std::vector< std::pair<int, HistoryDetails*> >
       NotificationList;
   NotificationList notifications_;
 
@@ -137,7 +142,7 @@ class ExpireHistoryTest : public testing::Test,
 
     FilePath thumb_name = path().Append(kThumbnailFile);
     thumb_db_.reset(new ThumbnailDatabase);
-    if (thumb_db_->Init(thumb_name, NULL) != sql::INIT_OK)
+    if (thumb_db_->Init(thumb_name, NULL, main_db_.get()) != sql::INIT_OK)
       thumb_db_.reset();
 
     text_db_.reset(new TextDatabaseManager(path(),
@@ -166,7 +171,7 @@ class ExpireHistoryTest : public testing::Test,
   }
 
   // BroadcastNotificationDelegate implementation.
-  void BroadcastNotifications(NotificationType type,
+  void BroadcastNotifications(int type,
                               HistoryDetails* details_deleted) {
     // This gets called when there are notifications to broadcast. Instead, we
     // store them so we can tell that the correct notifications were sent.
@@ -198,39 +203,41 @@ void ExpireHistoryTest::AddExampleData(URLID url_ids[3], Time visit_times[4]) {
 
   // Two favicons. The first two URLs will share the same one, while the last
   // one will have a unique favicon.
-  FavIconID favicon1 = thumb_db_->AddFavIcon(GURL("http://favicon/url1"));
-  FavIconID favicon2 = thumb_db_->AddFavIcon(GURL("http://favicon/url2"));
+  FaviconID favicon1 = thumb_db_->AddFavicon(GURL("http://favicon/url1"),
+                                             FAVICON);
+  FaviconID favicon2 = thumb_db_->AddFavicon(GURL("http://favicon/url2"),
+                                             FAVICON);
 
   // Three URLs.
   URLRow url_row1(GURL("http://www.google.com/1"));
   url_row1.set_last_visit(visit_times[0]);
-  url_row1.set_favicon_id(favicon1);
   url_row1.set_visit_count(1);
   url_ids[0] = main_db_->AddURL(url_row1);
+  thumb_db_->AddIconMapping(url_row1.url(), favicon1);
 
   URLRow url_row2(GURL("http://www.google.com/2"));
   url_row2.set_last_visit(visit_times[2]);
-  url_row2.set_favicon_id(favicon1);
   url_row2.set_visit_count(2);
   url_row2.set_typed_count(1);
   url_ids[1] = main_db_->AddURL(url_row2);
+  thumb_db_->AddIconMapping(url_row2.url(), favicon1);
 
   URLRow url_row3(GURL("http://www.google.com/3"));
   url_row3.set_last_visit(visit_times[3]);
-  url_row3.set_favicon_id(favicon2);
   url_row3.set_visit_count(1);
   url_ids[2] = main_db_->AddURL(url_row3);
+  thumb_db_->AddIconMapping(url_row3.url(), favicon2);
 
-  // Thumbnails for each URL.
-  scoped_ptr<SkBitmap> thumbnail(
+  // Thumbnails for each URL. |thumbnail| takes ownership of decoded SkBitmap.
+  gfx::Image thumbnail(
       gfx::JPEGCodec::Decode(kGoogleThumbnail, sizeof(kGoogleThumbnail)));
   ThumbnailScore score(0.25, true, true, Time::Now());
 
   Time time;
   GURL gurl;
-  top_sites_->SetPageThumbnail(url_row1.url(), *thumbnail, score);
-  top_sites_->SetPageThumbnail(url_row2.url(), *thumbnail, score);
-  top_sites_->SetPageThumbnail(url_row3.url(), *thumbnail, score);
+  top_sites_->SetPageThumbnail(url_row1.url(), &thumbnail, score);
+  top_sites_->SetPageThumbnail(url_row2.url(), &thumbnail, score);
+  top_sites_->SetPageThumbnail(url_row3.url(), &thumbnail, score);
 
   // Four visits.
   VisitRow visit_row1;
@@ -249,7 +256,7 @@ void ExpireHistoryTest::AddExampleData(URLID url_ids[3], Time visit_times[4]) {
   visit_row3.url_id = url_ids[1];
   visit_row3.visit_time = visit_times[2];
   visit_row3.is_indexed = true;
-  visit_row3.transition = PageTransition::TYPED;
+  visit_row3.transition = content::PAGE_TRANSITION_TYPED;
   main_db_->AddVisit(&visit_row3, SOURCE_BROWSED);
 
   VisitRow visit_row4;
@@ -291,31 +298,39 @@ void ExpireHistoryTest::AddExampleSourceData(const GURL& url, URLID* id) {
 
   // Four times for each visit.
   VisitRow visit_row1(url_id, last_visit_time - TimeDelta::FromDays(4), 0,
-                      PageTransition::TYPED, 0);
+                      content::PAGE_TRANSITION_TYPED, 0);
   main_db_->AddVisit(&visit_row1, SOURCE_SYNCED);
 
   VisitRow visit_row2(url_id, last_visit_time - TimeDelta::FromDays(3), 0,
-                      PageTransition::TYPED, 0);
+                      content::PAGE_TRANSITION_TYPED, 0);
   main_db_->AddVisit(&visit_row2, SOURCE_BROWSED);
 
   VisitRow visit_row3(url_id, last_visit_time - TimeDelta::FromDays(2), 0,
-                      PageTransition::TYPED, 0);
+                      content::PAGE_TRANSITION_TYPED, 0);
   main_db_->AddVisit(&visit_row3, SOURCE_EXTENSION);
 
-  VisitRow visit_row4(url_id, last_visit_time, 0, PageTransition::TYPED, 0);
+  VisitRow visit_row4(
+      url_id, last_visit_time, 0, content::PAGE_TRANSITION_TYPED, 0);
   main_db_->AddVisit(&visit_row4, SOURCE_FIREFOX_IMPORTED);
 }
 
-bool ExpireHistoryTest::HasFavIcon(FavIconID favicon_id) {
-  if (!thumb_db_.get())
+bool ExpireHistoryTest::HasFavicon(FaviconID favicon_id) {
+  if (!thumb_db_.get() || favicon_id == 0)
     return false;
   Time last_updated;
   std::vector<unsigned char> icon_data_unused;
   GURL icon_url;
-  return thumb_db_->GetFavIcon(favicon_id, &last_updated, &icon_data_unused,
+  return thumb_db_->GetFavicon(favicon_id, &last_updated, &icon_data_unused,
                                &icon_url);
 }
 
+FaviconID ExpireHistoryTest::GetFavicon(const GURL& page_url,
+                                        IconType icon_type) {
+  IconMapping icon_mapping;
+  thumb_db_->GetIconMappingForPageURL(page_url, icon_type, &icon_mapping);
+  return icon_mapping.icon_id;
+}
+
 bool ExpireHistoryTest::HasThumbnail(URLID url_id) {
   // TODO(sky): fix this. This test isn't really valid for TopSites. For
   // TopSites we should be checking URL always, not the id.
@@ -323,7 +338,7 @@ bool ExpireHistoryTest::HasThumbnail(URLID url_id) {
   if (!main_db_->GetURLRow(url_id, &info))
     return false;
   GURL url = info.url();
-  scoped_refptr<RefCountedBytes> data;
+  scoped_refptr<RefCountedMemory> data;
   return top_sites_->GetPageThumbnail(url, &data);
 }
 
@@ -363,71 +378,54 @@ void ExpireHistoryTest::EnsureURLInfoGone(const URLRow& row) {
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_FALSE(HasThumbnail(row.id()));
 
-  // Check the notifications. There should be a delete notification with this
-  // URL in it. There should also be a "typed URL changed" notification if the
-  // row is marked typed.
   bool found_delete_notification = false;
-  bool found_typed_changed_notification = false;
   for (size_t i = 0; i < notifications_.size(); i++) {
-    if (notifications_[i].first == NotificationType::HISTORY_URLS_DELETED) {
+    if (notifications_[i].first == chrome::NOTIFICATION_HISTORY_URLS_DELETED) {
       const URLsDeletedDetails* deleted_details =
           reinterpret_cast<URLsDeletedDetails*>(notifications_[i].second);
       if (deleted_details->urls.find(row.url()) !=
           deleted_details->urls.end()) {
         found_delete_notification = true;
       }
-    } else if (notifications_[i].first ==
-               NotificationType::HISTORY_TYPED_URLS_MODIFIED) {
-      // See if we got a typed URL changed notification.
-      const URLsModifiedDetails* modified_details =
-          reinterpret_cast<URLsModifiedDetails*>(notifications_[i].second);
-      for (size_t cur_url = 0; cur_url < modified_details->changed_urls.size();
-           cur_url++) {
-        if (modified_details->changed_urls[cur_url].url() == row.url())
-          found_typed_changed_notification = true;
-      }
-    } else if (notifications_[i].first ==
-               NotificationType::HISTORY_URL_VISITED) {
-      // See if we got a visited URL notification.
-      const URLVisitedDetails* visited_details =
-          reinterpret_cast<URLVisitedDetails*>(notifications_[i].second);
-      if (visited_details->row.url() == row.url())
-          found_typed_changed_notification = true;
+    } else {
+      EXPECT_NE(notifications_[i].first,
+                chrome::NOTIFICATION_HISTORY_URL_VISITED);
+      EXPECT_NE(notifications_[i].first,
+                chrome::NOTIFICATION_HISTORY_TYPED_URLS_MODIFIED);
     }
   }
   EXPECT_TRUE(found_delete_notification);
-  EXPECT_EQ(row.typed_count() > 0, found_typed_changed_notification);
 }
 
 TEST_F(ExpireHistoryTest, DeleteFaviconsIfPossible) {
   // Add a favicon record.
   const GURL favicon_url("http://www.google.com/favicon.ico");
-  FavIconID icon_id = thumb_db_->AddFavIcon(favicon_url);
+  FaviconID icon_id = thumb_db_->AddFavicon(favicon_url, FAVICON);
   EXPECT_TRUE(icon_id);
-  EXPECT_TRUE(HasFavIcon(icon_id));
+  EXPECT_TRUE(HasFavicon(icon_id));
 
   // The favicon should be deletable with no users.
-  std::set<FavIconID> favicon_set;
+  std::set<FaviconID> favicon_set;
   favicon_set.insert(icon_id);
   expirer_.DeleteFaviconsIfPossible(favicon_set);
-  EXPECT_FALSE(HasFavIcon(icon_id));
+  EXPECT_FALSE(HasFavicon(icon_id));
 
   // Add back the favicon.
-  icon_id = thumb_db_->AddFavIcon(favicon_url);
+  icon_id = thumb_db_->AddFavicon(favicon_url, TOUCH_ICON);
   EXPECT_TRUE(icon_id);
-  EXPECT_TRUE(HasFavIcon(icon_id));
+  EXPECT_TRUE(HasFavicon(icon_id));
 
   // Add a page that references the favicon.
   URLRow row(GURL("http://www.google.com/2"));
   row.set_visit_count(1);
-  row.set_favicon_id(icon_id);
   EXPECT_TRUE(main_db_->AddURL(row));
+  thumb_db_->AddIconMapping(row.url(), icon_id);
 
   // Favicon should not be deletable.
   favicon_set.clear();
   favicon_set.insert(icon_id);
   expirer_.DeleteFaviconsIfPossible(favicon_set);
-  EXPECT_TRUE(HasFavIcon(icon_id));
+  EXPECT_TRUE(HasFavicon(icon_id));
 }
 
 // static
@@ -449,7 +447,8 @@ TEST_F(ExpireHistoryTest, FLAKY_DeleteURLAndFavicon) {
   // Verify things are the way we expect with a URL row, favicon, thumbnail.
   URLRow last_row;
   ASSERT_TRUE(main_db_->GetURLRow(url_ids[2], &last_row));
-  EXPECT_TRUE(HasFavIcon(last_row.favicon_id()));
+  FaviconID favicon_id = GetFavicon(last_row.url(), FAVICON);
+  EXPECT_TRUE(HasFavicon(favicon_id));
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_TRUE(HasThumbnail(url_ids[2]));
 
@@ -498,7 +497,8 @@ TEST_F(ExpireHistoryTest, FLAKY_DeleteURLAndFavicon) {
 
   // All the normal data + the favicon should be gone.
   EnsureURLInfoGone(last_row);
-  EXPECT_FALSE(HasFavIcon(last_row.favicon_id()));
+  EXPECT_FALSE(GetFavicon(last_row.url(), FAVICON));
+  EXPECT_FALSE(HasFavicon(favicon_id));
 }
 
 // Deletes a URL with a favicon that other URLs reference, so that the favicon
@@ -511,7 +511,8 @@ TEST_F(ExpireHistoryTest, DeleteURLWithoutFavicon) {
   // Verify things are the way we expect with a URL row, favicon, thumbnail.
   URLRow last_row;
   ASSERT_TRUE(main_db_->GetURLRow(url_ids[1], &last_row));
-  EXPECT_TRUE(HasFavIcon(last_row.favicon_id()));
+  FaviconID favicon_id = GetFavicon(last_row.url(), FAVICON);
+  EXPECT_TRUE(HasFavicon(favicon_id));
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_TRUE(HasThumbnail(url_ids[1]));
 
@@ -523,9 +524,9 @@ TEST_F(ExpireHistoryTest, DeleteURLWithoutFavicon) {
   // Delete the URL and its dependencies.
   expirer_.DeleteURL(last_row.url());
 
-  // All the normal data + the favicon should be gone.
+  // All the normal data except the favicon should be gone.
   EnsureURLInfoGone(last_row);
-  EXPECT_TRUE(HasFavIcon(last_row.favicon_id()));
+  EXPECT_TRUE(HasFavicon(favicon_id));
 }
 
 // DeleteURL should not delete starred urls.
@@ -548,7 +549,8 @@ TEST_F(ExpireHistoryTest, DontDeleteStarredURL) {
   ASSERT_TRUE(main_db_->GetRowForURL(url, &url_row));
 
   // And the favicon should exist.
-  EXPECT_TRUE(HasFavIcon(url_row.favicon_id()));
+  FaviconID favicon_id = GetFavicon(url_row.url(), FAVICON);
+  EXPECT_TRUE(HasFavicon(favicon_id));
 
   // But there should be no fts.
   ASSERT_EQ(0, CountTextMatchesForURL(url_row.url()));
@@ -563,13 +565,50 @@ TEST_F(ExpireHistoryTest, DontDeleteStarredURL) {
   // ASSERT_TRUE(HasThumbnail(url_row.id()));
 
   // Unstar the URL and delete again.
-  bookmark_model_.SetURLStarred(url, string16(), false);
+  bookmark_utils::RemoveAllBookmarks(&bookmark_model_, url);
   expirer_.DeleteURL(url);
 
   // Now it should be completely deleted.
   EnsureURLInfoGone(url_row);
 }
 
+// Deletes multiple URLs at once.  The favicon for the third one but
+// not the first two should be deleted.
+TEST_F(ExpireHistoryTest, DeleteURLs) {
+  URLID url_ids[3];
+  Time visit_times[4];
+  AddExampleData(url_ids, visit_times);
+
+  // Verify things are the way we expect with URL rows, favicons,
+  // thumbnails.
+  URLRow rows[3];
+  FaviconID favicon_ids[3];
+  std::vector<GURL> urls;
+  // Push back a bogus URL (which shouldn't change anything).
+  urls.push_back(GURL());
+  for (size_t i = 0; i < arraysize(rows); ++i) {
+    ASSERT_TRUE(main_db_->GetURLRow(url_ids[i], &rows[i]));
+    favicon_ids[i] = GetFavicon(rows[i].url(), FAVICON);
+    EXPECT_TRUE(HasFavicon(favicon_ids[i]));
+    // TODO(sky): fix this, see comment in HasThumbnail.
+    // EXPECT_TRUE(HasThumbnail(url_ids[i]));
+    urls.push_back(rows[i].url());
+  }
+
+  StarURL(rows[0].url());
+
+  // Delete the URLs and their dependencies.
+  expirer_.DeleteURLs(urls);
+
+  // First one should still be around (since it was starred).
+  ASSERT_TRUE(main_db_->GetRowForURL(rows[0].url(), &rows[0]));
+  EnsureURLInfoGone(rows[1]);
+  EnsureURLInfoGone(rows[2]);
+  EXPECT_TRUE(HasFavicon(favicon_ids[0]));
+  EXPECT_TRUE(HasFavicon(favicon_ids[1]));
+  EXPECT_FALSE(HasFavicon(favicon_ids[2]));
+}
+
 // Expires all URLs more recent than a given time, with no starred items.
 // Our time threshold is such that one URL should be updated (we delete one of
 // the two visits) and one is deleted.
@@ -617,13 +656,15 @@ TEST_F(ExpireHistoryTest, FlushRecentURLsUnstarred) {
   EXPECT_EQ(0, temp_row.typed_count());
 
   // Verify that the middle URL's favicon and thumbnail is still there.
-  EXPECT_TRUE(HasFavIcon(url_row1.favicon_id()));
+  FaviconID favicon_id = GetFavicon(url_row1.url(), FAVICON);
+  EXPECT_TRUE(HasFavicon(favicon_id));
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_TRUE(HasThumbnail(url_row1.id()));
 
   // Verify that the last URL was deleted.
+  FaviconID favicon_id2 = GetFavicon(url_row2.url(), FAVICON);
   EnsureURLInfoGone(url_row2);
-  EXPECT_FALSE(HasFavIcon(url_row2.favicon_id()));
+  EXPECT_FALSE(HasFavicon(favicon_id2));
 }
 
 // Expires only a specific URLs more recent than a given time, with no starred
@@ -674,13 +715,14 @@ TEST_F(ExpireHistoryTest, FlushRecentURLsUnstarredRestricted) {
   EXPECT_EQ(0, temp_row.typed_count());
 
   // Verify that the middle URL's favicon and thumbnail is still there.
-  EXPECT_TRUE(HasFavIcon(url_row1.favicon_id()));
+  FaviconID favicon_id = GetFavicon(url_row1.url(), FAVICON);
+  EXPECT_TRUE(HasFavicon(favicon_id));
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_TRUE(HasThumbnail(url_row1.id()));
 
   // Verify that the last URL was not touched.
   EXPECT_TRUE(main_db_->GetURLRow(url_ids[2], &temp_row));
-  EXPECT_TRUE(HasFavIcon(url_row2.favicon_id()));
+  EXPECT_TRUE(HasFavicon(favicon_id));
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_TRUE(HasThumbnail(url_row2.id()));
 }
@@ -722,10 +764,12 @@ TEST_F(ExpireHistoryTest, FlushRecentURLsStarred) {
   // that may have been updated since the time threshold. Since the URL still
   // exists in history, this should not be a privacy problem, we only update
   // the visit counts in this case for consistency anyway.
-  EXPECT_TRUE(HasFavIcon(new_url_row1.favicon_id()));
+  FaviconID favicon_id = GetFavicon(url_row1.url(), FAVICON);
+  EXPECT_TRUE(HasFavicon(favicon_id));
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_TRUE(HasThumbnail(new_url_row1.id()));
-  EXPECT_TRUE(HasFavIcon(new_url_row2.favicon_id()));
+  favicon_id = GetFavicon(url_row1.url(), FAVICON);
+  EXPECT_TRUE(HasFavicon(favicon_id));
   // TODO(sky): fix this, see comment in HasThumbnail.
   // EXPECT_TRUE(HasThumbnail(new_url_row2.id()));
 }
