@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -25,17 +25,13 @@
 #include "chrome/browser/history/history.h"
 
 #include "base/callback.h"
+#include "base/memory/ref_counted.h"
 #include "base/message_loop.h"
 #include "base/path_service.h"
-#include "base/ref_counted.h"
 #include "base/string_util.h"
-#include "base/task.h"
+#include "base/threading/thread.h"
 #include "chrome/browser/autocomplete/history_url_provider.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/browser_window.h"
-#include "chrome/browser/history/download_create_info.h"
 #include "chrome/browser/history/history_backend.h"
 #include "chrome/browser/history/history_notifications.h"
 #include "chrome/browser/history/history_types.h"
@@ -44,12 +40,16 @@
 #include "chrome/browser/history/top_sites.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/profile_error_dialog.h"
 #include "chrome/browser/visitedlink/visitedlink_master.h"
 #include "chrome/common/chrome_constants.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/thumbnail_score.h"
 #include "chrome/common/url_constants.h"
+#include "content/browser/download/download_persistent_store_info.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/notification_service.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "third_party/skia/include/core/SkBitmap.h"
@@ -69,56 +69,66 @@ static const char* kHistoryThreadName = "Chrome_HistoryThread";
 // Release when the Backend has a reference to us).
 class HistoryService::BackendDelegate : public HistoryBackend::Delegate {
  public:
-  explicit BackendDelegate(HistoryService* history_service)
+  BackendDelegate(HistoryService* history_service, Profile* profile)
       : history_service_(history_service),
-        message_loop_(MessageLoop::current()) {
+        message_loop_(MessageLoop::current()),
+        profile_(profile) {
   }
 
-  virtual void NotifyProfileError(int message_id) {
+  virtual void NotifyProfileError(int backend_id,
+                                  sql::InitStatus init_status) OVERRIDE {
+    // Send to the history service on the main thread.
+    message_loop_->PostTask(
+        FROM_HERE,
+        base::Bind(&HistoryService::NotifyProfileError, history_service_.get(),
+                   backend_id, init_status));
+  }
+
+  virtual void SetInMemoryBackend(int backend_id,
+      history::InMemoryHistoryBackend* backend) OVERRIDE {
     // Send the backend to the history service on the main thread.
-    message_loop_->PostTask(FROM_HERE, NewRunnableMethod(history_service_.get(),
-        &HistoryService::NotifyProfileError, message_id));
+    message_loop_->PostTask(
+        FROM_HERE,
+        base::Bind(&HistoryService::SetInMemoryBackend, history_service_.get(),
+                   backend_id, backend));
   }
 
-  virtual void SetInMemoryBackend(
-      history::InMemoryHistoryBackend* backend) {
-    // Send the backend to the history service on the main thread.
-    message_loop_->PostTask(FROM_HERE, NewRunnableMethod(history_service_.get(),
-        &HistoryService::SetInMemoryBackend, backend));
-  }
-
-  virtual void BroadcastNotifications(NotificationType type,
-                                      history::HistoryDetails* details) {
+  virtual void BroadcastNotifications(
+      int type,
+      history::HistoryDetails* details) OVERRIDE {
     // Send the notification on the history thread.
-    if (NotificationService::current()) {
-      Details<history::HistoryDetails> det(details);
-      NotificationService::current()->Notify(type,
-                                             NotificationService::AllSources(),
-                                             det);
+    if (content::NotificationService::current()) {
+      content::Details<history::HistoryDetails> det(details);
+      content::NotificationService::current()->Notify(
+          type, content::Source<Profile>(profile_), det);
     }
     // Send the notification to the history service on the main thread.
-    message_loop_->PostTask(FROM_HERE, NewRunnableMethod(history_service_.get(),
-        &HistoryService::BroadcastNotifications, type, details));
+    message_loop_->PostTask(
+        FROM_HERE,
+        base::Bind(&HistoryService::BroadcastNotificationsHelper,
+                   history_service_.get(), type, base::Owned(details)));
   }
 
-  virtual void DBLoaded() {
-    message_loop_->PostTask(FROM_HERE, NewRunnableMethod(history_service_.get(),
-        &HistoryService::OnDBLoaded));
+  virtual void DBLoaded(int backend_id) OVERRIDE {
+    message_loop_->PostTask(
+        FROM_HERE,
+        base::Bind(&HistoryService::OnDBLoaded, history_service_.get(),
+                   backend_id));
   }
 
-  virtual void StartTopSitesMigration() {
-    message_loop_->PostTask(FROM_HERE, NewRunnableMethod(history_service_.get(),
-        &HistoryService::StartTopSitesMigration));
+  virtual void StartTopSitesMigration(int backend_id) OVERRIDE {
+    message_loop_->PostTask(
+        FROM_HERE,
+        base::Bind(&HistoryService::StartTopSitesMigration,
+                   history_service_.get(), backend_id));
   }
 
  private:
   scoped_refptr<HistoryService> history_service_;
   MessageLoop* message_loop_;
+  Profile* profile_;
 };
 
-// static
-const history::StarID HistoryService::kBookmarkBarID = 1;
-
 // The history thread is intentionally not a BrowserThread because the
 // sync integration unit tests depend on being able to create more than one
 // history thread.
@@ -126,6 +136,7 @@ HistoryService::HistoryService()
     : thread_(new base::Thread(kHistoryThreadName)),
       profile_(NULL),
       backend_loaded_(false),
+      current_backend_id_(-1),
       bookmark_service_(NULL),
       no_db_(false),
       needs_top_sites_migration_(false) {
@@ -135,14 +146,15 @@ HistoryService::HistoryService(Profile* profile)
     : thread_(new base::Thread(kHistoryThreadName)),
       profile_(profile),
       backend_loaded_(false),
+      current_backend_id_(-1),
       bookmark_service_(NULL),
       no_db_(false),
       needs_top_sites_migration_(false) {
   DCHECK(profile_);
-  registrar_.Add(this, NotificationType::HISTORY_URLS_DELETED,
-                 Source<Profile>(profile_));
-  registrar_.Add(this, NotificationType::TEMPLATE_URL_REMOVED,
-                 Source<Profile>(profile_));
+  registrar_.Add(this, chrome::NOTIFICATION_HISTORY_URLS_DELETED,
+                 content::Source<Profile>(profile_));
+  registrar_.Add(this, chrome::NOTIFICATION_TEMPLATE_URL_REMOVED,
+                 content::Source<Profile>(profile_));
 }
 
 HistoryService::~HistoryService() {
@@ -182,14 +194,21 @@ void HistoryService::UnloadBackend() {
   // run before we release our backend refptr, the last reference will be held
   // by this thread and the destructor will be called from here.
   //
-  // Therefore, we create a task to run the Closing operation first. This holds
-  // a reference to the backend. Then we release our reference, then we schedule
-  // the task to run. After the task runs, it will delete its reference from
-  // the history thread, ensuring everything works properly.
-  Task* closing_task =
-      NewRunnableMethod(history_backend_.get(), &HistoryBackend::Closing);
-  history_backend_ = NULL;
+  // Therefore, we create a closure to run the Closing operation first. This
+  // holds a reference to the backend. Then we release our reference, then we
+  // schedule the task to run. After the task runs, it will delete its reference
+  // from the history thread, ensuring everything works properly.
+  //
+  // TODO(ajwong): Cleanup HistoryBackend lifetime issues.
+  //     See http://crbug.com/99767.
+  history_backend_->AddRef();
+  base::Closure closing_task =
+      base::Bind(&HistoryBackend::Closing, history_backend_.get());
   ScheduleTask(PRIORITY_NORMAL, closing_task);
+  closing_task.Reset();
+  HistoryBackend* raw_ptr = history_backend_.get();
+  history_backend_ = NULL;
+  thread_->message_loop()->ReleaseSoon(FROM_HERE, raw_ptr);
 }
 
 void HistoryService::Cleanup() {
@@ -260,7 +279,7 @@ HistoryService::Handle HistoryService::GetMostRecentKeywordSearchTerms(
     const string16& prefix,
     int max_count,
     CancelableRequestConsumerBase* consumer,
-    GetMostRecentKeywordSearchTermsCallback* callback) {
+    const GetMostRecentKeywordSearchTermsCallback& callback) {
   return Schedule(PRIORITY_UI, &HistoryBackend::GetMostRecentKeywordSearchTerms,
                   consumer,
                   new history::GetMostRecentKeywordSearchTermsRequest(callback),
@@ -272,27 +291,25 @@ void HistoryService::URLsNoLongerBookmarked(const std::set<GURL>& urls) {
                     urls);
 }
 
-HistoryService::Handle HistoryService::ScheduleDBTask(
-    HistoryDBTask* task,
-    CancelableRequestConsumerBase* consumer) {
+void HistoryService::ScheduleDBTask(HistoryDBTask* task,
+                                    CancelableRequestConsumerBase* consumer) {
   history::HistoryDBTaskRequest* request = new history::HistoryDBTaskRequest(
-      NewCallback(task, &HistoryDBTask::DoneRunOnMainThread));
+      base::Bind(&HistoryDBTask::DoneRunOnMainThread, task));
   request->value = task;  // The value is the task to execute.
-  return Schedule(PRIORITY_UI, &HistoryBackend::ProcessDBTask, consumer,
-                  request);
+  Schedule(PRIORITY_UI, &HistoryBackend::ProcessDBTask, consumer, request);
 }
 
 HistoryService::Handle HistoryService::QuerySegmentUsageSince(
     CancelableRequestConsumerBase* consumer,
     const Time from_time,
     int max_result_count,
-    SegmentQueryCallback* callback) {
+    const SegmentQueryCallback& callback) {
   return Schedule(PRIORITY_UI, &HistoryBackend::QuerySegmentUsage,
                   consumer, new history::QuerySegmentUsageRequest(callback),
                   from_time, max_result_count);
 }
 
-void HistoryService::SetOnBackendDestroyTask(Task* task) {
+void HistoryService::SetOnBackendDestroyTask(const base::Closure& task) {
   ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::SetOnBackendDestroyTask,
                     MessageLoop::current(), task);
 }
@@ -301,7 +318,7 @@ void HistoryService::AddPage(const GURL& url,
                              const void* id_scope,
                              int32 page_id,
                              const GURL& referrer,
-                             PageTransition::Type transition,
+                             content::PageTransition transition,
                              const history::RedirectList& redirects,
                              history::VisitSource visit_source,
                              bool did_replace_entry) {
@@ -314,7 +331,7 @@ void HistoryService::AddPage(const GURL& url,
                              const void* id_scope,
                              int32 page_id,
                              const GURL& referrer,
-                             PageTransition::Type transition,
+                             content::PageTransition transition,
                              const history::RedirectList& redirects,
                              history::VisitSource visit_source,
                              bool did_replace_entry) {
@@ -430,63 +447,64 @@ void HistoryService::SetPageContents(const GURL& url,
                     url, contents);
 }
 
-void HistoryService::SetPageThumbnail(const GURL& page_url,
-                                      const SkBitmap& thumbnail,
-                                      const ThumbnailScore& score) {
-  if (!CanAddURL(page_url))
-    return;
-
-  ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::SetPageThumbnail,
-                    page_url, thumbnail, score);
-}
-
 HistoryService::Handle HistoryService::GetPageThumbnail(
     const GURL& page_url,
     CancelableRequestConsumerBase* consumer,
-    ThumbnailDataCallback* callback) {
+    const ThumbnailDataCallback& callback) {
   return Schedule(PRIORITY_NORMAL, &HistoryBackend::GetPageThumbnail, consumer,
                   new history::GetPageThumbnailRequest(callback), page_url);
 }
 
 void HistoryService::GetFavicon(FaviconService::GetFaviconRequest* request,
-                                const GURL& icon_url) {
-  Schedule(PRIORITY_NORMAL, &HistoryBackend::GetFavIcon, NULL, request,
-           icon_url);
+                                const GURL& icon_url,
+                                history::IconType icon_type) {
+  Schedule(PRIORITY_NORMAL, &HistoryBackend::GetFavicon, NULL, request,
+           icon_url, icon_type);
 }
 
 void HistoryService::UpdateFaviconMappingAndFetch(
     FaviconService::GetFaviconRequest* request,
     const GURL& page_url,
-    const GURL& icon_url) {
-  Schedule(PRIORITY_NORMAL, &HistoryBackend::UpdateFavIconMappingAndFetch, NULL,
-           request, page_url, icon_url);
+    const GURL& icon_url,
+    history::IconType icon_type) {
+  Schedule(PRIORITY_NORMAL, &HistoryBackend::UpdateFaviconMappingAndFetch, NULL,
+           request, page_url, icon_url, history::FAVICON);
 }
 
 void HistoryService::GetFaviconForURL(
     FaviconService::GetFaviconRequest* request,
-    const GURL& page_url) {
-  Schedule(PRIORITY_NORMAL, &HistoryBackend::GetFavIconForURL, NULL, request,
-           page_url);
+    const GURL& page_url,
+    int icon_types) {
+  Schedule(PRIORITY_NORMAL, &HistoryBackend::GetFaviconForURL, NULL, request,
+           page_url, icon_types);
 }
 
 void HistoryService::SetFavicon(const GURL& page_url,
                                 const GURL& icon_url,
-                                const std::vector<unsigned char>& image_data) {
+                                const std::vector<unsigned char>& image_data,
+                                history::IconType icon_type) {
   if (!CanAddURL(page_url))
     return;
 
-  ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::SetFavIcon,
+  ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::SetFavicon,
       page_url, icon_url,
-      scoped_refptr<RefCountedMemory>(new RefCountedBytes(image_data)));
+      scoped_refptr<RefCountedMemory>(new RefCountedBytes(image_data)),
+      icon_type);
 }
 
 void HistoryService::SetFaviconOutOfDateForPage(const GURL& page_url) {
   ScheduleAndForget(PRIORITY_NORMAL,
-                    &HistoryBackend::SetFavIconOutOfDateForPage, page_url);
+                    &HistoryBackend::SetFaviconOutOfDateForPage, page_url);
+}
+
+void HistoryService::CloneFavicon(const GURL& old_page_url,
+                                  const GURL& new_page_url) {
+  ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::CloneFavicon,
+                    old_page_url, new_page_url);
 }
 
 void HistoryService::SetImportedFavicons(
-    const std::vector<history::ImportedFavIconUsage>& favicon_usage) {
+    const std::vector<history::ImportedFaviconUsage>& favicon_usage) {
   ScheduleAndForget(PRIORITY_NORMAL,
                     &HistoryBackend::SetImportedFavicons, favicon_usage);
 }
@@ -499,7 +517,7 @@ HistoryService::Handle HistoryService::QueryURL(
     const GURL& url,
     bool want_visits,
     CancelableRequestConsumerBase* consumer,
-    QueryURLCallback* callback) {
+    const QueryURLCallback& callback) {
   return Schedule(PRIORITY_UI, &HistoryBackend::QueryURL, consumer,
                   new history::QueryURLRequest(callback), url, want_visits);
 }
@@ -509,18 +527,27 @@ HistoryService::Handle HistoryService::QueryURL(
 // Handle creation of a download by creating an entry in the history service's
 // 'downloads' table.
 HistoryService::Handle HistoryService::CreateDownload(
-    const DownloadCreateInfo& create_info,
+    int32 id,
+    const DownloadPersistentStoreInfo& create_info,
     CancelableRequestConsumerBase* consumer,
-    HistoryService::DownloadCreateCallback* callback) {
+    const HistoryService::DownloadCreateCallback& callback) {
   return Schedule(PRIORITY_NORMAL, &HistoryBackend::CreateDownload, consumer,
-                  new history::DownloadCreateRequest(callback), create_info);
+                  new history::DownloadCreateRequest(callback), id,
+                  create_info);
+}
+
+HistoryService::Handle HistoryService::GetNextDownloadId(
+    CancelableRequestConsumerBase* consumer,
+    const DownloadNextIdCallback& callback) {
+  return Schedule(PRIORITY_NORMAL, &HistoryBackend::GetNextDownloadId, consumer,
+                  new history::DownloadNextIdRequest(callback));
 }
 
 // Handle queries for a list of all downloads in the history database's
 // 'downloads' table.
 HistoryService::Handle HistoryService::QueryDownloads(
     CancelableRequestConsumerBase* consumer,
-    DownloadQueryCallback* callback) {
+    const DownloadQueryCallback& callback) {
   return Schedule(PRIORITY_NORMAL, &HistoryBackend::QueryDownloads, consumer,
                   new history::DownloadQueryRequest(callback));
 }
@@ -534,11 +561,8 @@ void HistoryService::CleanUpInProgressEntries() {
 
 // Handle updates for a particular download. This is a 'fire and forget'
 // operation, so we don't need to be called back.
-void HistoryService::UpdateDownload(int64 received_bytes,
-                                    int32 state,
-                                    int64 db_handle) {
-  ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::UpdateDownload,
-                    received_bytes, state, db_handle);
+void HistoryService::UpdateDownload(const DownloadPersistentStoreInfo& data) {
+  ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::UpdateDownload, data);
 }
 
 void HistoryService::UpdateDownloadPath(const FilePath& path,
@@ -564,7 +588,7 @@ HistoryService::Handle HistoryService::QueryHistory(
     const string16& text_query,
     const history::QueryOptions& options,
     CancelableRequestConsumerBase* consumer,
-    QueryHistoryCallback* callback) {
+    const QueryHistoryCallback& callback) {
   return Schedule(PRIORITY_UI, &HistoryBackend::QueryHistory, consumer,
                   new history::QueryHistoryRequest(callback),
                   text_query, options);
@@ -573,7 +597,7 @@ HistoryService::Handle HistoryService::QueryHistory(
 HistoryService::Handle HistoryService::QueryRedirectsFrom(
     const GURL& from_url,
     CancelableRequestConsumerBase* consumer,
-    QueryRedirectsCallback* callback) {
+    const QueryRedirectsCallback& callback) {
   return Schedule(PRIORITY_UI, &HistoryBackend::QueryRedirectsFrom, consumer,
       new history::QueryRedirectsRequest(callback), from_url);
 }
@@ -581,23 +605,23 @@ HistoryService::Handle HistoryService::QueryRedirectsFrom(
 HistoryService::Handle HistoryService::QueryRedirectsTo(
     const GURL& to_url,
     CancelableRequestConsumerBase* consumer,
-    QueryRedirectsCallback* callback) {
+    const QueryRedirectsCallback& callback) {
   return Schedule(PRIORITY_NORMAL, &HistoryBackend::QueryRedirectsTo, consumer,
       new history::QueryRedirectsRequest(callback), to_url);
 }
 
-HistoryService::Handle HistoryService::GetVisitCountToHost(
+HistoryService::Handle HistoryService::GetVisibleVisitCountToHost(
     const GURL& url,
     CancelableRequestConsumerBase* consumer,
-    GetVisitCountToHostCallback* callback) {
-  return Schedule(PRIORITY_UI, &HistoryBackend::GetVisitCountToHost, consumer,
-      new history::GetVisitCountToHostRequest(callback), url);
+    const GetVisibleVisitCountToHostCallback& callback) {
+  return Schedule(PRIORITY_UI, &HistoryBackend::GetVisibleVisitCountToHost,
+      consumer, new history::GetVisibleVisitCountToHostRequest(callback), url);
 }
 
 HistoryService::Handle HistoryService::QueryTopURLsAndRedirects(
     int result_count,
     CancelableRequestConsumerBase* consumer,
-    QueryTopURLsAndRedirectsCallback* callback) {
+    const QueryTopURLsAndRedirectsCallback& callback) {
   return Schedule(PRIORITY_NORMAL, &HistoryBackend::QueryTopURLsAndRedirects,
       consumer, new history::QueryTopURLsAndRedirectsRequest(callback),
       result_count);
@@ -607,21 +631,21 @@ HistoryService::Handle HistoryService::QueryMostVisitedURLs(
     int result_count,
     int days_back,
     CancelableRequestConsumerBase* consumer,
-    QueryMostVisitedURLsCallback* callback) {
+    const QueryMostVisitedURLsCallback& callback) {
   return Schedule(PRIORITY_NORMAL, &HistoryBackend::QueryMostVisitedURLs,
                   consumer,
                   new history::QueryMostVisitedURLsRequest(callback),
                   result_count, days_back);
 }
 
-void HistoryService::Observe(NotificationType type,
-                             const NotificationSource& source,
-                             const NotificationDetails& details) {
+void HistoryService::Observe(int type,
+                             const content::NotificationSource& source,
+                             const content::NotificationDetails& details) {
   if (!thread_)
     return;
 
-  switch (type.value) {
-    case NotificationType::HISTORY_URLS_DELETED: {
+  switch (type) {
+    case chrome::NOTIFICATION_HISTORY_URLS_DELETED: {
       // Update the visited link system for deleted URLs. We will update the
       // visited link system for added URLs as soon as we get the add
       // notification (we don't have to wait for the backend, which allows us to
@@ -633,7 +657,7 @@ void HistoryService::Observe(NotificationType type,
       // isn't. Therefore, we update the delete URLs after the fact.
       if (!profile_)
         return;  // No profile, probably unit testing.
-      Details<history::URLsDeletedDetails> deleted_details(details);
+      content::Details<history::URLsDeletedDetails> deleted_details(details);
       VisitedLinkMaster* visited_links = profile_->GetVisitedLinkMaster();
       if (!visited_links)
         return;  // Nobody to update.
@@ -644,8 +668,9 @@ void HistoryService::Observe(NotificationType type,
       break;
     }
 
-    case NotificationType::TEMPLATE_URL_REMOVED:
-      DeleteAllSearchTermsForKeyword(*(Details<TemplateURLID>(details).ptr()));
+    case chrome::NOTIFICATION_TEMPLATE_URL_REMOVED:
+      DeleteAllSearchTermsForKeyword(
+          *(content::Details<TemplateURLID>(details).ptr()));
       break;
 
     default:
@@ -677,8 +702,8 @@ void HistoryService::ScheduleAutocomplete(HistoryURLProvider* provider,
 }
 
 void HistoryService::ScheduleTask(SchedulePriority priority,
-                                  Task* task) {
-  // TODO(brettw): do prioritization.
+                                  const base::Closure& task) {
+  // TODO(brettw): Do prioritization.
   thread_->message_loop()->PostTask(FROM_HERE, task);
 }
 
@@ -697,19 +722,22 @@ bool HistoryService::CanAddURL(const GURL& url) {
       url.SchemeIs(chrome::kChromeInternalScheme))
     return false;
 
-  if (url.SchemeIs(chrome::kAboutScheme)) {
-    if (LowerCaseEqualsASCII(url.path(), "blank"))
-      return false;
-    // We allow all other about URLs since the user may like to see things
-    // like "about:memory" or "about:histograms" in their history and
-    // autocomplete.
-  }
+  // Allow all about: and chrome: URLs except about:blank, since the user may
+  // like to see "chrome://memory/", etc. in their history and autocomplete.
+  if (url == GURL(chrome::kAboutBlankURL))
+    return false;
 
   return true;
 }
 
-void HistoryService::SetInMemoryBackend(
+void HistoryService::SetInMemoryBackend(int backend_id,
     history::InMemoryHistoryBackend* mem_backend) {
+  if (!history_backend_ || current_backend_id_ != backend_id) {
+    VLOG(1) << "Message from obsolete backend";
+    // Cleaning up the memory backend.
+    delete mem_backend;
+    return;
+  }
   DCHECK(!in_memory_backend_.get()) << "Setting mem DB twice";
   in_memory_backend_.reset(mem_backend);
 
@@ -717,10 +745,15 @@ void HistoryService::SetInMemoryBackend(
   in_memory_backend_->AttachToHistoryService(profile_);
 }
 
-void HistoryService::NotifyProfileError(int message_id) {
-  Source<HistoryService> source(this);
-  NotificationService::current()->Notify(NotificationType::PROFILE_ERROR,
-                                         source, Details<int>(&message_id));
+void HistoryService::NotifyProfileError(int backend_id,
+                                        sql::InitStatus init_status) {
+  if (!history_backend_ || current_backend_id_ != backend_id) {
+    VLOG(1) << "Message from obsolete backend";
+    return;
+  }
+  ShowProfileErrorDialog(
+      (init_status == sql::INIT_FAILURE) ?
+      IDS_COULDNT_OPEN_PROFILE_ERROR : IDS_PROFILE_TOO_NEW_ERROR);
 }
 
 void HistoryService::DeleteURL(const GURL& url) {
@@ -728,24 +761,27 @@ void HistoryService::DeleteURL(const GURL& url) {
   ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::DeleteURL, url);
 }
 
+void HistoryService::DeleteURLsForTest(const std::vector<GURL>& urls) {
+  // We will update the visited links when we observe the delete
+  // notifications.
+  ScheduleAndForget(PRIORITY_NORMAL, &HistoryBackend::DeleteURLs, urls);
+}
+
 void HistoryService::ExpireHistoryBetween(
     const std::set<GURL>& restrict_urls,
     Time begin_time, Time end_time,
     CancelableRequestConsumerBase* consumer,
-    ExpireHistoryCallback* callback) {
+    const base::Closure& callback) {
 
   // We will update the visited links when we observe the delete notifications.
   Schedule(PRIORITY_UI, &HistoryBackend::ExpireHistoryBetween, consumer,
-           new history::ExpireHistoryRequest(callback),
+           new CancelableRequest<base::Closure>(callback),
            restrict_urls, begin_time, end_time);
 }
 
-void HistoryService::BroadcastNotifications(
-    NotificationType type,
-    history::HistoryDetails* details_deleted) {
-  // We take ownership of the passed-in pointer and delete it. It was made for
-  // us on another thread, so the caller doesn't know when we will handle it.
-  scoped_ptr<history::HistoryDetails> details(details_deleted);
+void HistoryService::BroadcastNotificationsHelper(
+    int type,
+    history::HistoryDetails* details) {
   // TODO(evanm): this is currently necessitated by generate_profile, which
   // runs without a browser process. generate_profile should really create
   // a browser process, at which point this check can then be nuked.
@@ -757,23 +793,25 @@ void HistoryService::BroadcastNotifications(
 
   // The source of all of our notifications is the profile. Note that this
   // pointer is NULL in unit tests.
-  Source<Profile> source(profile_);
+  content::Source<Profile> source(profile_);
 
   // The details object just contains the pointer to the object that the
   // backend has allocated for us. The receiver of the notification will cast
   // this to the proper type.
-  Details<history::HistoryDetails> det(details_deleted);
+  content::Details<history::HistoryDetails> det(details);
 
-  NotificationService::current()->Notify(type, source, det);
+  content::NotificationService::current()->Notify(type, source, det);
 }
 
 void HistoryService::LoadBackendIfNecessary() {
   if (!thread_ || history_backend_)
     return;  // Failed to init, or already started loading.
 
+  ++current_backend_id_;
   scoped_refptr<HistoryBackend> backend(
       new HistoryBackend(history_dir_,
-                         new BackendDelegate(this),
+                         current_backend_id_,
+                         new BackendDelegate(this, profile_),
                          bookmark_service_));
   history_backend_.swap(backend);
 
@@ -786,11 +824,16 @@ void HistoryService::LoadBackendIfNecessary() {
   ScheduleAndForget(PRIORITY_UI, &HistoryBackend::Init, languages, no_db_);
 }
 
-void HistoryService::OnDBLoaded() {
+void HistoryService::OnDBLoaded(int backend_id) {
+  if (!history_backend_ || current_backend_id_ != backend_id) {
+    VLOG(1) << "Message from obsolete backend";
+    return;
+  }
   backend_loaded_ = true;
-  NotificationService::current()->Notify(NotificationType::HISTORY_LOADED,
-                                         Source<Profile>(profile_),
-                                         Details<HistoryService>(this));
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_HISTORY_LOADED,
+      content::Source<Profile>(profile_),
+      content::Details<HistoryService>(this));
   if (thread_ && profile_) {
     // We don't want to force creation of TopSites.
     history::TopSites* ts = profile_->GetTopSitesWithoutCreating();
@@ -799,7 +842,11 @@ void HistoryService::OnDBLoaded() {
   }
 }
 
-void HistoryService::StartTopSitesMigration() {
+void HistoryService::StartTopSitesMigration(int backend_id) {
+  if (!history_backend_ || current_backend_id_ != backend_id) {
+    VLOG(1) << "Message from obsolete backend";
+    return;
+  }
   needs_top_sites_migration_ = true;
   if (thread_ && profile_) {
     // We don't want to force creation of TopSites.
