@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,47 +6,40 @@
 #define CHROME_BROWSER_HISTORY_HISTORY_H_
 #pragma once
 
+#include <set>
 #include <vector>
 
 #include "base/basictypes.h"
 #include "base/callback.h"
 #include "base/file_path.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string16.h"
-#include "base/task.h"
 #include "chrome/browser/cancelable_request.h"
-#include "chrome/browser/favicon_service.h"
+#include "chrome/browser/favicon/favicon_service.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/search_engines/template_url_id.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/page_transition_types.h"
 #include "chrome/common/ref_counted_util.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/common/page_transition_types.h"
+#include "sql/init_status.h"
 
 class BookmarkService;
-struct DownloadCreateInfo;
+struct DownloadPersistentStoreInfo;
 class FilePath;
 class GURL;
 class HistoryURLProvider;
 struct HistoryURLProviderParams;
-class InMemoryURLDatabase;
-class MainPagesRequest;
 class PageUsageData;
 class PageUsageRequest;
 class Profile;
-class SkBitmap;
-struct ThumbnailScore;
 
 namespace base {
 class Thread;
 class Time;
 }
 
-namespace browser_sync {
-class HistoryModelWorker;
-class TypedUrlDataTypeController;
-}
-
 namespace history {
 class InMemoryHistoryBackend;
 class InMemoryURLIndex;
@@ -91,16 +84,12 @@ class HistoryDBTask : public base::RefCountedThreadSafe<HistoryDBTask> {
 // This service is thread safe. Each request callback is invoked in the
 // thread that made the request.
 class HistoryService : public CancelableRequestProvider,
-                       public NotificationObserver,
+                       public content::NotificationObserver,
                        public base::RefCountedThreadSafe<HistoryService> {
  public:
   // Miscellaneous commonly-used types.
   typedef std::vector<PageUsageData*> PageUsageDataList;
 
-  // ID (both star_id and group_id) of the bookmark bar.
-  // This entry always exists.
-  static const history::StarID kBookmarkBarID;
-
   // Must call Init after construction.
   explicit HistoryService(Profile* profile);
   // The empty constructor is provided only for testing.
@@ -186,7 +175,7 @@ class HistoryService : public CancelableRequestProvider,
                const void* id_scope,
                int32 page_id,
                const GURL& referrer,
-               PageTransition::Type transition,
+               content::PageTransition transition,
                const history::RedirectList& redirects,
                history::VisitSource visit_source,
                bool did_replace_entry);
@@ -198,14 +187,14 @@ class HistoryService : public CancelableRequestProvider,
                const void* id_scope,
                int32 page_id,
                const GURL& referrer,
-               PageTransition::Type transition,
+               content::PageTransition transition,
                const history::RedirectList& redirects,
                history::VisitSource visit_source,
                bool did_replace_entry);
 
   // For adding pages to history where no tracking information can be done.
   void AddPage(const GURL& url, history::VisitSource visit_source) {
-    AddPage(url, NULL, 0, GURL(), PageTransition::LINK,
+    AddPage(url, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
             history::RedirectList(), visit_source, false);
   }
 
@@ -263,11 +252,11 @@ class HistoryService : public CancelableRequestProvider,
   // empty.
   //
   // If success is false, neither the row nor the vector will be valid.
-  typedef Callback4<Handle,
-                    bool,  // Success flag, when false, nothing else is valid.
-                    const history::URLRow*,
-                    history::VisitVector*>::Type
-      QueryURLCallback;
+  typedef base::Callback<void(
+      Handle,
+      bool,  // Success flag, when false, nothing else is valid.
+      const history::URLRow*,
+      history::VisitVector*)> QueryURLCallback;
 
   // Queries the basic information about the URL in the history database. If
   // the caller is interested in the visits (each time the URL is visited),
@@ -276,12 +265,12 @@ class HistoryService : public CancelableRequestProvider,
   Handle QueryURL(const GURL& url,
                   bool want_visits,
                   CancelableRequestConsumerBase* consumer,
-                  QueryURLCallback* callback);
+                  const QueryURLCallback& callback);
 
   // Provides the result of a query. See QueryResults in history_types.h.
   // The common use will be to use QueryResults.Swap to suck the contents of
   // the results out of the passed in parameter and take ownership of them.
-  typedef Callback2<Handle, history::QueryResults*>::Type
+  typedef base::Callback<void(Handle, history::QueryResults*)>
       QueryHistoryCallback;
 
   // Queries all history with the given options (see QueryOptions in
@@ -294,7 +283,7 @@ class HistoryService : public CancelableRequestProvider,
   Handle QueryHistory(const string16& text_query,
                       const history::QueryOptions& options,
                       CancelableRequestConsumerBase* consumer,
-                      QueryHistoryCallback* callback);
+                      const QueryHistoryCallback& callback);
 
   // Called when the results of QueryRedirectsFrom are available.
   // The given vector will contain a list of all redirects, not counting
@@ -305,65 +294,69 @@ class HistoryService : public CancelableRequestProvider,
   // redirect, the vector will be empty. If the history system failed for
   // some reason, success will additionally be false. If the given page
   // has redirected to multiple destinations, this will pick a random one.
-  typedef Callback4<Handle,
-                    GURL,  // from_url
-                    bool,  // success
-                    history::RedirectList*>::Type
-      QueryRedirectsCallback;
+  typedef base::Callback<void(Handle,
+                              GURL,  // from_url
+                              bool,  // success
+                              history::RedirectList*)> QueryRedirectsCallback;
 
   // Schedules a query for the most recent redirect coming out of the given
   // URL. See the RedirectQuerySource above, which is guaranteed to be called
   // if the request is not canceled.
   Handle QueryRedirectsFrom(const GURL& from_url,
                             CancelableRequestConsumerBase* consumer,
-                            QueryRedirectsCallback* callback);
+                            const QueryRedirectsCallback& callback);
 
   // Schedules a query to get the most recent redirects ending at the given
   // URL.
   Handle QueryRedirectsTo(const GURL& to_url,
                           CancelableRequestConsumerBase* consumer,
-                          QueryRedirectsCallback* callback);
+                          const QueryRedirectsCallback& callback);
 
-  typedef Callback4<Handle,
-                    bool,        // Were we able to determine the # of visits?
-                    int,         // Number of visits.
-                    base::Time>::Type // Time of first visit. Only first bool is
-                                 // true and int is > 0.
-      GetVisitCountToHostCallback;
+  typedef base::Callback<
+      void(Handle,
+           bool,        // Were we able to determine the # of visits?
+           int,         // Number of visits.
+           base::Time)> // Time of first visit. Only set if bool
+                        // is true and int is > 0.
+      GetVisibleVisitCountToHostCallback;
 
-  // Requests the number of visits to all urls on the scheme/host/post
-  // identified by url. This is only valid for http and https urls.
-  Handle GetVisitCountToHost(const GURL& url,
-                             CancelableRequestConsumerBase* consumer,
-                             GetVisitCountToHostCallback* callback);
+  // Requests the number of user-visible visits (i.e. no redirects or subframes)
+  // to all urls on the same scheme/host/port as |url|.  This is only valid for
+  // HTTP and HTTPS URLs.
+  Handle GetVisibleVisitCountToHost(
+      const GURL& url,
+      CancelableRequestConsumerBase* consumer,
+      const GetVisibleVisitCountToHostCallback& callback);
 
   // Called when QueryTopURLsAndRedirects completes. The vector contains a list
   // of the top |result_count| URLs.  For each of these URLs, there is an entry
   // in the map containing redirects from the URL.  For example, if we have the
   // redirect chain A -> B -> C and A is a top visited URL, then A will be in
   // the vector and "A => {B -> C}" will be in the map.
-  typedef Callback4<Handle,
-                    bool,  // Did we get the top urls and redirects?
-                    std::vector<GURL>*,  // List of top URLs.
-                    history::RedirectMap*>::Type  // Redirects for top URLs.
+  typedef base::Callback<
+      void(Handle,
+           bool,  // Did we get the top urls and redirects?
+           std::vector<GURL>*,  // List of top URLs.
+           history::RedirectMap*)>  // Redirects for top URLs.
       QueryTopURLsAndRedirectsCallback;
 
   // Request the top |result_count| most visited URLs and the chain of redirects
   // leading to each of these URLs.
   // TODO(Nik): remove this. Use QueryMostVisitedURLs instead.
-  Handle QueryTopURLsAndRedirects(int result_count,
-                                  CancelableRequestConsumerBase* consumer,
-                                  QueryTopURLsAndRedirectsCallback* callback);
+  Handle QueryTopURLsAndRedirects(
+      int result_count,
+      CancelableRequestConsumerBase* consumer,
+      const QueryTopURLsAndRedirectsCallback& callback);
 
-  typedef Callback2<Handle, history::MostVisitedURLList>::Type
-                    QueryMostVisitedURLsCallback;
+  typedef base::Callback<void(Handle, history::MostVisitedURLList)>
+      QueryMostVisitedURLsCallback;
 
   // Request the |result_count| most visited URLs and the chain of
   // redirects leading to each of these URLs. |days_back| is the
   // number of days of history to use. Used by TopSites.
   Handle QueryMostVisitedURLs(int result_count, int days_back,
                               CancelableRequestConsumerBase* consumer,
-                              QueryMostVisitedURLsCallback* callback);
+                              const QueryMostVisitedURLsCallback& callback);
 
   // Thumbnails ----------------------------------------------------------------
 
@@ -373,28 +366,22 @@ class HistoryService : public CancelableRequestProvider,
   //
   // This function will be called even on error conditions or if there is no
   // thumbnail for that page. In these cases, the data pointer will be NULL.
-  typedef Callback2<Handle, scoped_refptr<RefCountedBytes> >::Type
+  typedef base::Callback<void(Handle, scoped_refptr<RefCountedBytes>)>
       ThumbnailDataCallback;
 
-  // Sets the thumbnail for a given URL. The URL must be in the history
-  // database or the request will be ignored.
-  void SetPageThumbnail(const GURL& url,
-                        const SkBitmap& thumbnail,
-                        const ThumbnailScore& score);
-
   // Requests a page thumbnail. See ThumbnailDataCallback definition above.
   Handle GetPageThumbnail(const GURL& page_url,
                           CancelableRequestConsumerBase* consumer,
-                          ThumbnailDataCallback* callback);
+                          const ThumbnailDataCallback& callback);
 
   // Database management operations --------------------------------------------
 
   // Delete all the information related to a single url.
   void DeleteURL(const GURL& url);
 
-  // Implemented by the caller of ExpireHistoryBetween, and
-  // is called when the history service has deleted the history.
-  typedef Callback0::Type ExpireHistoryCallback;
+  // Delete all the information related to a list of urls.  (Deleting
+  // URLs one by one is slow as it has to flush to disk each time.)
+  void DeleteURLsForTest(const std::vector<GURL>& urls);
 
   // Removes all visits in the selected time range (including the start time),
   // updating the URLs accordingly. This deletes the associated data, including
@@ -407,33 +394,40 @@ class HistoryService : public CancelableRequestProvider,
   void ExpireHistoryBetween(const std::set<GURL>& restrict_urls,
                             base::Time begin_time, base::Time end_time,
                             CancelableRequestConsumerBase* consumer,
-                            ExpireHistoryCallback* callback);
+                            const base::Closure& callback);
 
   // Downloads -----------------------------------------------------------------
 
   // Implemented by the caller of 'CreateDownload' below, and is called when the
   // history service has created a new entry for a download in the history db.
-  typedef Callback2<DownloadCreateInfo, int64>::Type
-      DownloadCreateCallback;
+  typedef base::Callback<void(int32, int64)> DownloadCreateCallback;
 
   // Begins a history request to create a new persistent entry for a download.
   // 'info' contains all the download's creation state, and 'callback' runs
   // when the history service request is complete.
-  Handle CreateDownload(const DownloadCreateInfo& info,
+  Handle CreateDownload(int32 id,
+                        const DownloadPersistentStoreInfo& info,
                         CancelableRequestConsumerBase* consumer,
-                        DownloadCreateCallback* callback);
+                        const DownloadCreateCallback& callback);
+
+  // Implemented by the caller of 'GetNextDownloadId' below.
+  typedef base::Callback<void(int)> DownloadNextIdCallback;
+
+  // Runs the callback with the next available download id.
+  Handle GetNextDownloadId(CancelableRequestConsumerBase* consumer,
+                           const DownloadNextIdCallback& callback);
 
   // Implemented by the caller of 'QueryDownloads' below, and is called when the
   // history service has retrieved a list of all download state. The call
-  typedef Callback1<std::vector<DownloadCreateInfo>*>::Type
+  typedef base::Callback<void(std::vector<DownloadPersistentStoreInfo>*)>
       DownloadQueryCallback;
 
   // Begins a history request to retrieve the state of all downloads in the
   // history db. 'callback' runs when the history service request is complete,
-  // at which point 'info' contains an array of DownloadCreateInfo, one per
-  // download.
+  // at which point 'info' contains an array of DownloadPersistentStoreInfo, one
+  // per download.
   Handle QueryDownloads(CancelableRequestConsumerBase* consumer,
-                        DownloadQueryCallback* callback);
+                        const DownloadQueryCallback& callback);
 
   // Begins a request to clean up entries that has been corrupted (because of
   // the crash, for example).
@@ -442,7 +436,7 @@ class HistoryService : public CancelableRequestProvider,
   // Called to update the history service about the current state of a download.
   // This is a 'fire and forget' query, so just pass the relevant state info to
   // the database with no need for a callback.
-  void UpdateDownload(int64 received_bytes, int32 state, int64 db_handle);
+  void UpdateDownload(const DownloadPersistentStoreInfo& data);
 
   // Called to update the history service about the path of a download.
   // This is a 'fire and forget' query.
@@ -461,7 +455,7 @@ class HistoryService : public CancelableRequestProvider,
 
   // Visit Segments ------------------------------------------------------------
 
-  typedef Callback2<Handle, std::vector<PageUsageData*>*>::Type
+  typedef base::Callback<void(Handle, std::vector<PageUsageData*>*)>
       SegmentQueryCallback;
 
   // Query usage data for all visit segments since the provided time.
@@ -480,7 +474,7 @@ class HistoryService : public CancelableRequestProvider,
   Handle QuerySegmentUsageSince(CancelableRequestConsumerBase* consumer,
                                 const base::Time from_time,
                                 int max_result_count,
-                                SegmentQueryCallback* callback);
+                                const SegmentQueryCallback& callback);
 
   // Set the presentation index for the segment identified by |segment_id|.
   void SetSegmentPresentationIndex(int64 segment_id, int index);
@@ -496,8 +490,9 @@ class HistoryService : public CancelableRequestProvider,
   // Deletes all search terms for the specified keyword.
   void DeleteAllSearchTermsForKeyword(TemplateURLID keyword_id);
 
-  typedef Callback2<Handle, std::vector<history::KeywordSearchTermVisit>*>::Type
-      GetMostRecentKeywordSearchTermsCallback;
+  typedef base::Callback<
+      void(Handle, std::vector<history::KeywordSearchTermVisit>*)>
+          GetMostRecentKeywordSearchTermsCallback;
 
   // Returns up to max_count of the most recent search terms starting with the
   // specified text. The matching is case insensitive. The results are ordered
@@ -508,7 +503,7 @@ class HistoryService : public CancelableRequestProvider,
       const string16& prefix,
       int max_count,
       CancelableRequestConsumerBase* consumer,
-      GetMostRecentKeywordSearchTermsCallback* callback);
+      const GetMostRecentKeywordSearchTermsCallback& callback);
 
   // Bookmarks -----------------------------------------------------------------
 
@@ -517,12 +512,10 @@ class HistoryService : public CancelableRequestProvider,
 
   // Generic Stuff -------------------------------------------------------------
 
-  typedef Callback0::Type HistoryDBTaskCallback;
-
   // Schedules a HistoryDBTask for running on the history backend thread. See
   // HistoryDBTask for details on what this does.
-  virtual Handle ScheduleDBTask(HistoryDBTask* task,
-                                CancelableRequestConsumerBase* consumer);
+  virtual void ScheduleDBTask(HistoryDBTask* task,
+                              CancelableRequestConsumerBase* consumer);
 
   // Returns true if top sites needs to be migrated out of history into its own
   // db.
@@ -539,15 +532,19 @@ class HistoryService : public CancelableRequestProvider,
   // There can be only one closing task, so this will override any previously
   // set task. We will take ownership of the pointer and delete it when done.
   // The task will be run on the calling thread (this function is threadsafe).
-  void SetOnBackendDestroyTask(Task* task);
+  void SetOnBackendDestroyTask(const base::Closure& task);
 
   // Used for unit testing and potentially importing to get known information
   // into the database. This assumes the URL doesn't exist in the database
   //
   // Calling this function many times may be slow because each call will
   // dispatch to the history thread and will be a separate database
-  // transaction. If this functionality is needed for importing many URLs, a
-  // version that takes an array should probably be added.
+  // transaction. If this functionality is needed for importing many URLs,
+  // callers should use AddPagesWithDetails() instead.
+  //
+  // Note that this routine (and AddPageWithDetails()) always adds a single
+  // visit using the |last_visit| timestamp, and a PageTransition type of LINK,
+  // if |visit_source| != SYNCED.
   void AddPageWithDetails(const GURL& url,
                           const string16& title,
                           int visit_count,
@@ -562,7 +559,7 @@ class HistoryService : public CancelableRequestProvider,
 
   // Starts the TopSites migration in the HistoryThread. Called by the
   // BackendDelegate.
-  void StartTopSitesMigration();
+  void StartTopSitesMigration(int backend_id);
 
   // Called by TopSites after the thumbnails were read and it is safe
   // to delete the thumbnails DB.
@@ -573,7 +570,7 @@ class HistoryService : public CancelableRequestProvider,
   static bool CanAddURL(const GURL& url);
 
  protected:
-  ~HistoryService();
+  virtual ~HistoryService();
 
   // These are not currently used, hopefully we can do something in the future
   // to ensure that the most important things happen first.
@@ -596,13 +593,12 @@ class HistoryService : public CancelableRequestProvider,
   template<typename Info, typename Callback> friend class DownloadRequest;
   friend class PageUsageRequest;
   friend class RedirectRequest;
-  friend class FavIconRequest;
   friend class TestingProfile;
 
-  // Implementation of NotificationObserver.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // Implementation of content::NotificationObserver.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Low-level Init().  Same as the public version, but adds a |no_db| parameter
   // that is only set by unittests which causes the backend to not init its DB.
@@ -619,21 +615,19 @@ class HistoryService : public CancelableRequestProvider,
   // Broadcasts the given notification. This is called by the backend so that
   // the notification will be broadcast on the main thread.
   //
-  // The |details_deleted| pointer will be sent as the "details" for the
-  // notification. The function takes ownership of the pointer and deletes it
-  // when the notification is sent (it is coming from another thread, so must
-  // be allocated on the heap).
-  void BroadcastNotifications(NotificationType type,
-                              history::HistoryDetails* details_deleted);
+  // Compared to BroadcastNotifications(), this function does not take
+  // ownership of |details|.
+  void BroadcastNotificationsHelper(int type,
+                                    history::HistoryDetails* details);
 
   // Initializes the backend.
   void LoadBackendIfNecessary();
 
   // Notification from the backend that it has finished loading. Sends
   // notification (NOTIFY_HISTORY_LOADED) and sets backend_loaded_ to true.
-  void OnDBLoaded();
+  void OnDBLoaded(int backend_id);
 
-  // FavIcon -------------------------------------------------------------------
+  // Favicon -------------------------------------------------------------------
 
   // These favicon methods are exposed to the FaviconService. Instead of calling
   // these methods directly you should call the respective method on the
@@ -641,46 +635,54 @@ class HistoryService : public CancelableRequestProvider,
 
   // Used by the FaviconService to get a favicon from the history backend.
   void GetFavicon(FaviconService::GetFaviconRequest* request,
-                  const GURL& icon_url);
+                  const GURL& icon_url,
+                  history::IconType icon_type);
 
   // Used by the FaviconService to update the favicon mappings on the history
   // backend.
   void UpdateFaviconMappingAndFetch(FaviconService::GetFaviconRequest* request,
                                     const GURL& page_url,
-                                    const GURL& icon_url);
+                                    const GURL& icon_url,
+                                    history::IconType icon_type);
 
   // Used by the FaviconService to get a favicon from the history backend.
   void GetFaviconForURL(FaviconService::GetFaviconRequest* request,
-                        const GURL& page_url);
+                        const GURL& page_url,
+                        int icon_types);
 
   // Used by the FaviconService to mark the favicon for the page as being out
   // of date.
   void SetFaviconOutOfDateForPage(const GURL& page_url);
 
+  // Used by the FaviconService to clone favicons from one page to another,
+  // provided that other page does not already have favicons.
+  void CloneFavicon(const GURL& old_page_url, const GURL& new_page_url);
+
   // Used by the FaviconService for importing many favicons for many pages at
   // once. The pages must exist, any favicon sets for unknown pages will be
   // discarded. Existing favicons will not be overwritten.
   void SetImportedFavicons(
-      const std::vector<history::ImportedFavIconUsage>& favicon_usage);
+      const std::vector<history::ImportedFaviconUsage>& favicon_usage);
 
   // Used by the FaviconService to set the favicon for a page on the history
   // backend.
   void SetFavicon(const GURL& page_url,
                   const GURL& icon_url,
-                  const std::vector<unsigned char>& image_data);
+                  const std::vector<unsigned char>& image_data,
+                  history::IconType icon_type);
 
 
   // Sets the in-memory URL database. This is called by the backend once the
   // database is loaded to make it available.
-  void SetInMemoryBackend(history::InMemoryHistoryBackend* mem_backend);
+  void SetInMemoryBackend(int backend_id,
+                          history::InMemoryHistoryBackend* mem_backend);
 
   // Called by our BackendDelegate when there is a problem reading the database.
-  // |message_id| is the relevant message in the string table to display.
-  void NotifyProfileError(int message_id);
+  void NotifyProfileError(int backend_id, sql::InitStatus init_status);
 
   // Call to schedule a given task for running on the history thread with the
   // specified priority. The task will have ownership taken.
-  void ScheduleTask(SchedulePriority priority, Task* task);
+  void ScheduleTask(SchedulePriority priority, const base::Closure& task);
 
   // Schedule ------------------------------------------------------------------
   //
@@ -698,8 +700,8 @@ class HistoryService : public CancelableRequestProvider,
     if (consumer)
       AddRequest(request, consumer);
     ScheduleTask(priority,
-                 NewRunnableMethod(history_backend_.get(), func,
-                                   scoped_refptr<RequestType>(request)));
+                 base::Bind(func, history_backend_.get(),
+                            scoped_refptr<RequestType>(request)));
     return request->handle();
   }
 
@@ -714,9 +716,8 @@ class HistoryService : public CancelableRequestProvider,
     if (consumer)
       AddRequest(request, consumer);
     ScheduleTask(priority,
-                 NewRunnableMethod(history_backend_.get(), func,
-                                   scoped_refptr<RequestType>(request),
-                                   a));
+                 base::Bind(func, history_backend_.get(),
+                            scoped_refptr<RequestType>(request), a));
     return request->handle();
   }
 
@@ -735,9 +736,8 @@ class HistoryService : public CancelableRequestProvider,
     if (consumer)
       AddRequest(request, consumer);
     ScheduleTask(priority,
-                 NewRunnableMethod(history_backend_.get(), func,
-                                   scoped_refptr<RequestType>(request),
-                                   a, b));
+                 base::Bind(func, history_backend_.get(),
+                            scoped_refptr<RequestType>(request), a, b));
     return request->handle();
   }
 
@@ -758,9 +758,8 @@ class HistoryService : public CancelableRequestProvider,
     if (consumer)
       AddRequest(request, consumer);
     ScheduleTask(priority,
-                 NewRunnableMethod(history_backend_.get(), func,
-                                   scoped_refptr<RequestType>(request),
-                                   a, b, c));
+                 base::Bind(func, history_backend_.get(),
+                            scoped_refptr<RequestType>(request), a, b, c));
     return request->handle();
   }
 
@@ -774,7 +773,7 @@ class HistoryService : public CancelableRequestProvider,
                          BackendFunc func) {  // Function to call on backend.
     DCHECK(thread_) << "History service being called after cleanup";
     LoadBackendIfNecessary();
-    ScheduleTask(priority, NewRunnableMethod(history_backend_.get(), func));
+    ScheduleTask(priority, base::Bind(func, history_backend_.get()));
   }
 
   template<typename BackendFunc, typename ArgA>
@@ -783,7 +782,7 @@ class HistoryService : public CancelableRequestProvider,
                          const ArgA& a) {
     DCHECK(thread_) << "History service being called after cleanup";
     LoadBackendIfNecessary();
-    ScheduleTask(priority, NewRunnableMethod(history_backend_.get(), func, a));
+    ScheduleTask(priority, base::Bind(func, history_backend_.get(), a));
   }
 
   template<typename BackendFunc, typename ArgA, typename ArgB>
@@ -793,8 +792,7 @@ class HistoryService : public CancelableRequestProvider,
                          const ArgB& b) {
     DCHECK(thread_) << "History service being called after cleanup";
     LoadBackendIfNecessary();
-    ScheduleTask(priority, NewRunnableMethod(history_backend_.get(), func,
-                                             a, b));
+    ScheduleTask(priority, base::Bind(func, history_backend_.get(), a, b));
   }
 
   template<typename BackendFunc, typename ArgA, typename ArgB, typename ArgC>
@@ -805,8 +803,7 @@ class HistoryService : public CancelableRequestProvider,
                          const ArgC& c) {
     DCHECK(thread_) << "History service being called after cleanup";
     LoadBackendIfNecessary();
-    ScheduleTask(priority, NewRunnableMethod(history_backend_.get(), func,
-                                             a, b, c));
+    ScheduleTask(priority, base::Bind(func, history_backend_.get(), a, b, c));
   }
 
   template<typename BackendFunc,
@@ -822,11 +819,11 @@ class HistoryService : public CancelableRequestProvider,
                          const ArgD& d) {
     DCHECK(thread_) << "History service being called after cleanup";
     LoadBackendIfNecessary();
-    ScheduleTask(priority, NewRunnableMethod(history_backend_.get(), func,
-                                             a, b, c, d));
+    ScheduleTask(priority, base::Bind(func, history_backend_.get(),
+                                      a, b, c, d));
   }
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // Some void primitives require some internal processing in the main thread
   // when done. We use this internal consumer for this purpose.
@@ -856,6 +853,10 @@ class HistoryService : public CancelableRequestProvider,
   // completed.
   bool backend_loaded_;
 
+  // The id of the current backend. This is only valid when history_backend_
+  // is not NULL.
+  int current_backend_id_;
+
   // Cached values from Init(), used whenever we need to reload the backend.
   FilePath history_dir_;
   BookmarkService* bookmark_service_;
