@@ -1,20 +1,23 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <vector>
 
+#include "base/bind.h"
 #include "base/message_loop.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/history/history.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/test/test_browser_thread.h"
 #include "googleurl/src/gurl.h"
 
+using content::BrowserThread;
+
 namespace {
 
 // Note: WaitableEvent is not used for synchronization between the main thread
@@ -54,7 +57,7 @@ class HistoryEnumerator : public HistoryService::URLEnumerator {
     BrowserThread::PostTask(
         BrowserThread::UI,
         FROM_HERE,
-        NewRunnableMethod(history, &HistoryService::IterateURLs, this));
+        base::Bind(&HistoryService::IterateURLs, history, this));
     ui_test_utils::RunMessageLoop();
   }
 
@@ -66,7 +69,7 @@ class HistoryEnumerator : public HistoryService::URLEnumerator {
     BrowserThread::PostTask(
         BrowserThread::UI,
         FROM_HERE,
-        new MessageLoop::QuitTask());
+        MessageLoop::QuitClosure());
   }
 
   std::vector<GURL>& urls() { return urls_; }
@@ -108,10 +111,8 @@ class HistoryBrowserTest : public InProcessBrowserTest {
     HistoryService* history = GetHistoryService();
     BrowserThread::PostTask(BrowserThread::UI,
                             FROM_HERE,
-                            NewRunnableMethod(history,
-                                              &HistoryService::ScheduleDBTask,
-                                              task,
-                                              &request_consumer));
+                            base::Bind(&HistoryService::ScheduleDBTask,
+                                       history, task, &request_consumer));
     ui_test_utils::RunMessageLoop();
   }
 
@@ -141,15 +142,8 @@ IN_PROC_BROWSER_TEST_F(HistoryBrowserTest, SavingHistoryEnabled) {
   }
 }
 
-// Times out on Vista only.  http://crbug.com/57994
-#if defined(OS_WIN)
-#define MAYBE_SavingHistoryDisabled DISABLED_SavingHistoryDisabled
-#else
-#define MAYBE_SavingHistoryDisabled SavingHistoryDisabled
-#endif
-
 // Test that disabling saving browser history really works.
-IN_PROC_BROWSER_TEST_F(HistoryBrowserTest, MAYBE_SavingHistoryDisabled) {
+IN_PROC_BROWSER_TEST_F(HistoryBrowserTest, SavingHistoryDisabled) {
   GetPrefs()->SetBoolean(prefs::kSavingBrowserHistoryDisabled, true);
 
   EXPECT_TRUE(GetProfile()->GetHistoryService(Profile::EXPLICIT_ACCESS));
@@ -163,18 +157,9 @@ IN_PROC_BROWSER_TEST_F(HistoryBrowserTest, MAYBE_SavingHistoryDisabled) {
   ExpectEmptyHistory();
 }
 
-// Times out on Vista only.  http://crbug.com/57994
-#if defined(OS_WIN)
-#define MAYBE_SavingHistoryEnabledThenDisabled \
-    DISABLED_SavingHistoryEnabledThenDisabled
-#else
-#define MAYBE_SavingHistoryEnabledThenDisabled SavingHistoryEnabledThenDisabled
-#endif
-
 // Test that changing the pref takes effect immediately
 // when the browser is running.
-IN_PROC_BROWSER_TEST_F(HistoryBrowserTest,
-    MAYBE_SavingHistoryEnabledThenDisabled) {
+IN_PROC_BROWSER_TEST_F(HistoryBrowserTest, SavingHistoryEnabledThenDisabled) {
   EXPECT_FALSE(GetPrefs()->GetBoolean(prefs::kSavingBrowserHistoryDisabled));
 
   ui_test_utils::WaitForHistoryToLoad(browser());
