@@ -6,7 +6,8 @@
 
 namespace history {
 
-URLVisitedDetails::URLVisitedDetails() : transition(PageTransition::LINK) {}
+URLVisitedDetails::URLVisitedDetails()
+    : transition(content::PAGE_TRANSITION_LINK) {}
 
 URLVisitedDetails::~URLVisitedDetails() {}
 
@@ -24,9 +25,9 @@ URLsStarredDetails::URLsStarredDetails(bool being_starred)
 
 URLsStarredDetails::~URLsStarredDetails() {}
 
-FavIconChangeDetails::FavIconChangeDetails() {}
+FaviconChangeDetails::FaviconChangeDetails() {}
 
-FavIconChangeDetails::~FavIconChangeDetails() {}
+FaviconChangeDetails::~FaviconChangeDetails() {}
 
 KeywordSearchTermDetails::KeywordSearchTermDetails() : keyword_id(0) {}
 
