@@ -30,7 +30,7 @@ struct URLVisitedDetails : public HistoryDetails {
   URLVisitedDetails();
   virtual ~URLVisitedDetails();
 
-  PageTransition::Type transition;
+  content::PageTransition transition;
   URLRow row;
 
   // A list of redirects leading up to the URL represented by this struct. If
@@ -77,9 +77,9 @@ struct URLsStarredDetails : public HistoryDetails {
 };
 
 // Details for NOTIFY_FAVICON_CHANGED.
-struct FavIconChangeDetails : public HistoryDetails {
-  FavIconChangeDetails();
-  virtual ~FavIconChangeDetails();
+struct FaviconChangeDetails : public HistoryDetails {
+  FaviconChangeDetails();
+  virtual ~FaviconChangeDetails();
 
   std::set<GURL> urls;
 };
@@ -87,7 +87,7 @@ struct FavIconChangeDetails : public HistoryDetails {
 // Details for HISTORY_KEYWORD_SEARCH_TERM_UPDATED.
 struct KeywordSearchTermDetails : public HistoryDetails {
   KeywordSearchTermDetails();
-  ~KeywordSearchTermDetails();
+  virtual ~KeywordSearchTermDetails();
 
   GURL url;
   TemplateURLID keyword_id;
