@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,7 @@
 #include <limits>
 
 #include "base/logging.h"
-#include "base/stl_util-inl.h"
-
-using base::Time;
-using base::TimeDelta;
+#include "base/stl_util.h"
 
 namespace history {
 
@@ -43,7 +40,6 @@ URLRow& URLRow::operator=(const URLRow& other) {
   typed_count_ = other.typed_count_;
   last_visit_ = other.last_visit_;
   hidden_ = other.hidden_;
-  favicon_id_ = other.favicon_id_;
   return *this;
 }
 
@@ -55,16 +51,14 @@ void URLRow::Swap(URLRow* other) {
   std::swap(typed_count_, other->typed_count_);
   std::swap(last_visit_, other->last_visit_);
   std::swap(hidden_, other->hidden_);
-  std::swap(favicon_id_, other->favicon_id_);
 }
 
 void URLRow::Initialize() {
   id_ = 0;
   visit_count_ = 0;
   typed_count_ = 0;
-  last_visit_ = Time();
+  last_visit_ = base::Time();
   hidden_ = false;
-  favicon_id_ = 0;
 }
 
 // VisitRow --------------------------------------------------------------------
@@ -73,15 +67,15 @@ VisitRow::VisitRow()
     : visit_id(0),
       url_id(0),
       referring_visit(0),
-      transition(PageTransition::LINK),
+      transition(content::PAGE_TRANSITION_LINK),
       segment_id(0),
       is_indexed(false) {
 }
 
 VisitRow::VisitRow(URLID arg_url_id,
-                   Time arg_visit_time,
+                   base::Time arg_visit_time,
                    VisitID arg_referring_visit,
-                   PageTransition::Type arg_transition,
+                   content::PageTransition arg_transition,
                    SegmentID arg_segment_id)
     : visit_id(0),
       url_id(arg_url_id),
@@ -97,18 +91,18 @@ VisitRow::~VisitRow() {
 
 // Favicons -------------------------------------------------------------------
 
-ImportedFavIconUsage::ImportedFavIconUsage() {
+ImportedFaviconUsage::ImportedFaviconUsage() {
 }
 
-ImportedFavIconUsage::~ImportedFavIconUsage() {
+ImportedFaviconUsage::~ImportedFaviconUsage() {
 }
 
 // StarredEntry ----------------------------------------------------------------
 
 StarredEntry::StarredEntry()
     : id(0),
-      parent_group_id(0),
-      group_id(0),
+      parent_folder_id(0),
+      folder_id(0),
       visual_order(0),
       type(URL),
       url_id(0) {
@@ -121,13 +115,13 @@ void StarredEntry::Swap(StarredEntry* other) {
   std::swap(id, other->id);
   title.swap(other->title);
   std::swap(date_added, other->date_added);
-  std::swap(parent_group_id, other->parent_group_id);
-  std::swap(group_id, other->group_id);
+  std::swap(parent_folder_id, other->parent_folder_id);
+  std::swap(folder_id, other->folder_id);
   std::swap(visual_order, other->visual_order);
   std::swap(type, other->type);
   url.Swap(&other->url);
   std::swap(url_id, other->url_id);
-  std::swap(date_group_modified, other->date_group_modified);
+  std::swap(date_folder_modified, other->date_folder_modified);
 }
 
 // URLResult -------------------------------------------------------------------
@@ -149,7 +143,7 @@ URLResult::URLResult(const GURL& url,
 URLResult::~URLResult() {
 }
 
-void URLResult::Swap(URLResult* other) {
+void URLResult::SwapResult(URLResult* other) {
   URLRow::Swap(other);
   std::swap(visit_time_, other->visit_time_);
   snippet_.Swap(&other->snippet_);
@@ -177,7 +171,7 @@ const size_t* QueryResults::MatchesForURL(const GURL& url,
 
   // All entries in the map should have at least one index, otherwise it
   // shouldn't be in the map.
-  DCHECK(found->second->size() > 0);
+  DCHECK(!found->second->empty());
   if (num_matches)
     *num_matches = found->second->size();
   return &found->second->front();
@@ -192,7 +186,7 @@ void QueryResults::Swap(QueryResults* other) {
 
 void QueryResults::AppendURLBySwapping(URLResult* result) {
   URLResult* new_result = new URLResult;
-  new_result->Swap(result);
+  new_result->SwapResult(result);
 
   results_.push_back(new_result);
   AddURLUsageAtIndex(new_result->url(), results_.size() - 1);
@@ -302,7 +296,7 @@ void QueryResults::AdjustResultMap(size_t begin, size_t end, ptrdiff_t delta) {
 
 // QueryOptions ----------------------------------------------------------------
 
-QueryOptions::QueryOptions() : max_count(0) {}
+QueryOptions::QueryOptions() : max_count(0), body_only(false) {}
 
 void QueryOptions::SetRecentDayRange(int days_ago) {
   end_time = base::Time::Now();
@@ -311,7 +305,7 @@ void QueryOptions::SetRecentDayRange(int days_ago) {
 
 // KeywordSearchTermVisit -----------------------------------------------------
 
-KeywordSearchTermVisit::KeywordSearchTermVisit() {}
+KeywordSearchTermVisit::KeywordSearchTermVisit() : visits(0) {}
 
 KeywordSearchTermVisit::~KeywordSearchTermVisit() {}
 
@@ -325,12 +319,10 @@ KeywordSearchTermRow::~KeywordSearchTermRow() {}
 
 MostVisitedURL::MostVisitedURL() {}
 
-MostVisitedURL::MostVisitedURL(const GURL& in_url,
-                               const GURL& in_favicon_url,
-                               const string16& in_title)
-    : url(in_url),
-      favicon_url(in_favicon_url),
-      title(in_title) {
+MostVisitedURL::MostVisitedURL(const GURL& url,
+                               const string16& title)
+    : url(url),
+      title(title) {
 }
 
 MostVisitedURL::~MostVisitedURL() {}
@@ -356,7 +348,7 @@ HistoryAddPageArgs::HistoryAddPageArgs(
     int32 arg_page_id,
     const GURL& arg_referrer,
     const history::RedirectList& arg_redirects,
-    PageTransition::Type arg_transition,
+    content::PageTransition arg_transition,
     VisitSource arg_source,
     bool arg_did_replace_entry)
       : url(arg_url),
@@ -386,4 +378,47 @@ MostVisitedThumbnails::MostVisitedThumbnails() {}
 
 MostVisitedThumbnails::~MostVisitedThumbnails() {}
 
+// Autocomplete thresholds -----------------------------------------------------
+
+const int kLowQualityMatchTypedLimit = 1;
+const int kLowQualityMatchVisitLimit = 4;
+const int kLowQualityMatchAgeLimitInDays = 3;
+
+base::Time AutocompleteAgeThreshold() {
+  return (base::Time::Now() -
+          base::TimeDelta::FromDays(kLowQualityMatchAgeLimitInDays));
+}
+
+bool RowQualifiesAsSignificant(const URLRow& row,
+                               const base::Time& threshold) {
+  const base::Time& real_threshold =
+      threshold.is_null() ? AutocompleteAgeThreshold() : threshold;
+  return (row.typed_count() >= kLowQualityMatchTypedLimit) ||
+         (row.visit_count() >= kLowQualityMatchVisitLimit) ||
+         (row.last_visit() >= real_threshold);
+}
+
+// IconMapping ----------------------------------------------------------------
+
+IconMapping::IconMapping()
+    : mapping_id(0),
+      icon_id(0),
+      icon_type(INVALID_ICON) {
+}
+
+IconMapping::~IconMapping() {}
+
+
+FaviconData::FaviconData()
+  : known_icon(false),
+    expired(false),
+    icon_type(history::INVALID_ICON) {
+}
+
+FaviconData::~FaviconData() {}
+
+bool FaviconData::is_valid() {
+  return known_icon && image_data.get() && image_data->size();
+}
+
 }  // namespace history
