@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,15 +13,15 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/ref_counted_memory.h"
+#include "base/memory/ref_counted_memory.h"
 #include "base/stack_container.h"
 #include "base/string16.h"
 #include "base/time.h"
 #include "chrome/browser/history/snippet.h"
 #include "chrome/browser/search_engines/template_url_id.h"
-#include "chrome/common/page_transition_types.h"
 #include "chrome/common/ref_counted_util.h"
 #include "chrome/common/thumbnail_score.h"
+#include "content/public/common/page_transition_types.h"
 #include "googleurl/src/gurl.h"
 
 namespace history {
@@ -40,8 +40,9 @@ typedef std::vector<GURL> RedirectList;
 typedef int64 StarID;  // Unique identifier for star entries.
 typedef int64 UIStarID;  // Identifier for star entries that come from the UI.
 typedef int64 DownloadID;   // Identifier for a download.
-typedef int64 FavIconID;  // For FavIcons.
+typedef int64 FaviconID;  // For favicons.
 typedef int64 SegmentID;  // URL segments for the most visited view.
+typedef int64 IconMappingID; // For page url and icon mapping.
 
 // URLRow ---------------------------------------------------------------------
 
@@ -73,6 +74,13 @@ class URLRow {
   URLRow& operator=(const URLRow& other);
 
   URLID id() const { return id_; }
+
+  // Sets the id of the row. The id should only be manually set when a row has
+  // been retrieved from the history database or other dataset based on criteria
+  // other than its id (i.e. by URL) and when the id has not yet been set in the
+  // row.
+  void set_id(URLID id) { id_ = id; }
+
   const GURL& url() const { return url_; }
 
   const string16& title() const {
@@ -86,6 +94,10 @@ class URLRow {
     }
   }
 
+  // The number of times this URL has been visited. This will often match the
+  // number of entries in the visit table for this URL, but won't always. It's
+  // really designed for autocomplete ranking, so some "useless" transitions
+  // from the visit table aren't counted in this tally.
   int visit_count() const {
     return visit_count_;
   }
@@ -93,7 +105,11 @@ class URLRow {
     visit_count_ = visit_count;
   }
 
-  // Number of times the URL was typed in the Omnibox.
+  // Number of times the URL was typed in the Omnibox. This "should" match
+  // the number of TYPED transitions in the visit table. It's used primarily
+  // for faster autocomplete ranking. If you need to know the actual number of
+  // TYPED transitions, you should query the visit table since there could be
+  // something out of sync.
   int typed_count() const {
     return typed_count_;
   }
@@ -116,16 +132,10 @@ class URLRow {
     hidden_ = hidden;
   }
 
-  // ID of the favicon. A value of 0 means the favicon isn't known yet.
-  FavIconID favicon_id() const { return favicon_id_; }
-  void set_favicon_id(FavIconID favicon_id) {
-    favicon_id_ = favicon_id;
-  }
-
+ protected:
   // Swaps the contents of this URLRow with another, which allows it to be
   // destructively copied without memory allocations.
-  // (Virtual because it's overridden by URLResult.)
-  virtual void Swap(URLRow* other);
+  void Swap(URLRow* other);
 
  private:
   // This class writes directly into this structure and clears our dirty bits
@@ -137,8 +147,9 @@ class URLRow {
   // This excludes objects which autoinitialize such as strings.
   void Initialize();
 
-  // The row ID of this URL. Immutable except for the database which sets it
-  // when it pulls them out.
+  // The row ID of this URL from the history database. This is immutable except
+  // when retrieving the row from the database or when determining if the URL
+  // referenced by the URLRow already exists in the database.
   URLID id_;
 
   // The URL of this row. Immutable except for the database which sets it
@@ -162,27 +173,24 @@ class URLRow {
   // is usually for subframes.
   bool hidden_;
 
-  // The ID of the favicon for this url.
-  FavIconID favicon_id_;
-
   // We support the implicit copy constuctor and operator=.
 };
 
 // The enumeration of all possible sources of visits is listed below.
-// The source will be propogated along with a URL or a visit item
+// The source will be propagated along with a URL or a visit item
 // and eventually be stored in the history database,
 // visit_source table specifically.
 // Different from page transition types, they describe the origins of visits.
 // (Warning): Please don't change any existing values while it is ok to add
 // new values when needed.
-typedef enum {
+enum VisitSource {
   SOURCE_SYNCED = 0,         // Synchronized from somewhere else.
   SOURCE_BROWSED = 1,        // User browsed.
   SOURCE_EXTENSION = 2,      // Added by an externsion.
   SOURCE_FIREFOX_IMPORTED = 3,
   SOURCE_IE_IMPORTED = 4,
   SOURCE_SAFARI_IMPORTED = 5,
-} VisitSource;
+};
 
 typedef int64 VisitID;
 // Structure to hold the mapping between each visit's id and its source.
@@ -198,7 +206,7 @@ class VisitRow {
   VisitRow(URLID arg_url_id,
            base::Time arg_visit_time,
            VisitID arg_referring_visit,
-           PageTransition::Type arg_transition,
+           content::PageTransition arg_transition,
            SegmentID arg_segment_id);
   ~VisitRow();
 
@@ -215,7 +223,7 @@ class VisitRow {
   VisitID referring_visit;
 
   // A combination of bits from PageTransition.
-  PageTransition::Type transition;
+  content::PageTransition transition;
 
   // The segment id (see visitsegment_database.*).
   // If 0, the segment id is null in the table.
@@ -239,12 +247,16 @@ class VisitRow {
 // We pass around vectors of visits a lot
 typedef std::vector<VisitRow> VisitVector;
 
+// The basic information associated with a visit (timestamp, type of visit),
+// used by HistoryBackend::AddVisits() to create new visits for a URL.
+typedef std::pair<base::Time, content::PageTransition> VisitInfo;
+
 // Favicons -------------------------------------------------------------------
 
 // Used by the importer to set favicons for imported bookmarks.
-struct ImportedFavIconUsage {
-  ImportedFavIconUsage();
-  ~ImportedFavIconUsage();
+struct ImportedFaviconUsage {
+  ImportedFaviconUsage();
+  ~ImportedFaviconUsage();
 
   // The URL of the favicon.
   GURL favicon_url;
@@ -268,30 +280,33 @@ struct PageVisit {
 
 // StarredEntry ---------------------------------------------------------------
 
-// StarredEntry represents either a starred page, or a star grouping (where
-// a star grouping consists of child starred entries). Use the type to
-// determine the type of a particular entry.
+// StarredEntry represents either a starred page, or a folder (where a folder
+// consists of child starred entries). Use the type to determine the type of a
+// particular entry.
 //
 // The database internally uses the id field to uniquely identify a starred
 // entry. On the other hand, the UI, which is anything routed through
 // HistoryService and HistoryBackend (including BookmarkBarView), uses the
-// url field to uniquely identify starred entries of type URL and the group_id
-// field to uniquely identify starred entries of type USER_GROUP. For example,
+// url field to uniquely identify starred entries of type URL and the folder_id
+// field to uniquely identify starred entries of type USER_FOLDER. For example,
 // HistoryService::UpdateStarredEntry identifies the entry by url (if the
-// type is URL) or group_id (if the type is not URL).
+// type is URL) or folder_id (if the type is not URL).
 struct StarredEntry {
   enum Type {
-    // Type represents a starred URL (StarredEntry).
+    // Type represents a starred URL.
     URL,
 
-    // The bookmark bar grouping.
+    // The bookmark bar folder.
     BOOKMARK_BAR,
 
-    // User created group.
-    USER_GROUP,
+    // User created folder.
+    USER_FOLDER,
 
     // The "other bookmarks" folder that holds uncategorized bookmarks.
-    OTHER
+    OTHER,
+
+    // The mobile folder.
+    MOBILE,
   };
 
   StarredEntry();
@@ -308,17 +323,17 @@ struct StarredEntry {
   // When this was added.
   base::Time date_added;
 
-  // Group ID of the star group this entry is in. If 0, this entry is not
-  // in a star group.
-  UIStarID parent_group_id;
+  // Folder ID of the folder this entry is in. If 0, this entry is not in a
+  // folder.
+  UIStarID parent_folder_id;
 
-  // Unique identifier for groups. This is assigned by the UI.
+  // Unique identifier for folders. This is assigned by the UI.
   //
   // WARNING: this is NOT the same as id, id is assigned by the database,
   // this is assigned by the UI. See note about StarredEntry for more info.
-  UIStarID group_id;
+  UIStarID folder_id;
 
-  // Visual order within the parent. Only valid if group_id is not 0.
+  // Visual order within the parent. Only valid if folder_id is not 0.
   int visual_order;
 
   // Type of this entry (see enum).
@@ -331,9 +346,9 @@ struct StarredEntry {
   // starred.
   URLID url_id;
 
-  // Time the entry was last modified. This is only used for groups and
-  // indicates the last time a URL was added as a child to the group.
-  base::Time date_group_modified;
+  // Time the entry was last modified. This is only used for folders and
+  // indicates the last time a URL was added as a child to the folder.
+  base::Time date_folder_modified;
 };
 
 // URLResult -------------------------------------------------------------------
@@ -345,7 +360,7 @@ class URLResult : public URLRow {
   // Constructor that create a URLResult from the specified URL and title match
   // positions from title_matches.
   URLResult(const GURL& url, const Snippet::MatchPositions& title_matches);
-  ~URLResult();
+  virtual ~URLResult();
 
   base::Time visit_time() const { return visit_time_; }
   void set_visit_time(base::Time visit_time) { visit_time_ = visit_time; }
@@ -359,7 +374,7 @@ class URLResult : public URLRow {
     return title_match_positions_;
   }
 
-  virtual void Swap(URLResult* other);
+  void SwapResult(URLResult* other);
 
  private:
   friend class HistoryBackend;
@@ -507,6 +522,10 @@ struct QueryOptions {
   // the most recent first, so older results may not be returned if there is not
   // enough room. When 0, this will return everything (the default).
   int max_count;
+
+  // Only search within the page body if true, otherwise search all columns
+  // including url and time. Defaults to false.
+  bool body_only;
 };
 
 // KeywordSearchTermVisit -----------------------------------------------------
@@ -517,11 +536,9 @@ struct KeywordSearchTermVisit {
   KeywordSearchTermVisit();
   ~KeywordSearchTermVisit();
 
-  // The time of the visit.
-  base::Time time;
-
-  // The search term that was used.
-  string16 term;
+  string16 term;    // The search term that was used.
+  int visits;       // The visit count.
+  base::Time time;  // The time of the most recent visit.
 };
 
 // KeywordSearchTermRow --------------------------------------------------------
@@ -531,14 +548,9 @@ struct KeywordSearchTermRow {
   KeywordSearchTermRow();
   ~KeywordSearchTermRow();
 
-  // ID of the keyword.
-  TemplateURLID keyword_id;
-
-  // ID of the url.
-  URLID url_id;
-
-  // The search term that was used.
-  string16 term;
+  TemplateURLID keyword_id;  // ID of the keyword.
+  URLID url_id;              // ID of the url.
+  string16 term;             // The search term that was used.
 };
 
 // MostVisitedURL --------------------------------------------------------------
@@ -546,13 +558,10 @@ struct KeywordSearchTermRow {
 // Holds the per-URL information of the most visited query.
 struct MostVisitedURL {
   MostVisitedURL();
-  MostVisitedURL(const GURL& in_url,
-                 const GURL& in_favicon_url,
-                 const string16& in_title);
+  MostVisitedURL(const GURL& url, const string16& title);
   ~MostVisitedURL();
 
   GURL url;
-  GURL favicon_url;
   string16 title;
 
   RedirectList redirects;
@@ -574,7 +583,7 @@ class HistoryAddPageArgs
                      int32 arg_page_id,
                      const GURL& arg_referrer,
                      const history::RedirectList& arg_redirects,
-                     PageTransition::Type arg_transition,
+                     content::PageTransition arg_transition,
                      VisitSource arg_source,
                      bool arg_did_replace_entry);
 
@@ -590,7 +599,7 @@ class HistoryAddPageArgs
 
   GURL referrer;
   history::RedirectList redirects;
-  PageTransition::Type transition;
+  content::PageTransition transition;
   VisitSource visit_source;
   bool did_replace_entry;
 
@@ -664,6 +673,77 @@ class MostVisitedThumbnails
   DISALLOW_COPY_AND_ASSIGN(MostVisitedThumbnails);
 };
 
+// Autocomplete thresholds -----------------------------------------------------
+
+// Constants which specify, when considered altogether, 'significant'
+// history items. These are used to filter out insignificant items
+// for consideration as autocomplete candidates.
+extern const int kLowQualityMatchTypedLimit;
+extern const int kLowQualityMatchVisitLimit;
+extern const int kLowQualityMatchAgeLimitInDays;
+
+// Returns the date threshold for considering an history item as significant.
+base::Time AutocompleteAgeThreshold();
+
+// Return true if |row| qualifies as an autocomplete candidate. If |time_cache|
+// is_null() then this function determines a new time threshold each time it is
+// called. Since getting system time can be costly (such as for cases where
+// this function will be called in a loop over many history items), you can
+// provide a non-null |time_cache| by simply initializing |time_cache| with
+// AutocompleteAgeThreshold() (or any other desired time in the past).
+bool RowQualifiesAsSignificant(const URLRow& row, const base::Time& threshold);
+
+// Defines the icon types. They are also stored in icon_type field of favicons
+// table.
+enum IconType {
+  INVALID_ICON = 0x0,
+  FAVICON = 1 << 0,
+  TOUCH_ICON = 1 << 1,
+  TOUCH_PRECOMPOSED_ICON = 1 << 2
+};
+
+// Used for the mapping between the page and icon.
+struct IconMapping {
+  IconMapping();
+  ~IconMapping();
+
+  // The unique id of the mapping.
+  IconMappingID mapping_id;
+
+  // The url of a web page.
+  GURL page_url;
+
+  // The unique id of the icon.
+  FaviconID icon_id;
+
+  // The type of icon.
+  IconType icon_type;
+};
+
+// Defines the favicon stored in history backend.
+struct FaviconData {
+  FaviconData();
+  ~FaviconData();
+
+  // Returns true if the icon is known and image has data.
+  bool is_valid();
+
+  // Indicates whether the icon is known by the history backend.
+  bool known_icon;
+
+  // The bits of image.
+  scoped_refptr<RefCountedMemory> image_data;
+
+  // Indicates whether image is expired.
+  bool expired;
+
+  // The icon's URL.
+  GURL icon_url;
+
+  // The type of favicon.
+  history::IconType icon_type;
+};
+
 }  // namespace history
 
 #endif  // CHROME_BROWSER_HISTORY_HISTORY_TYPES_H_
