@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -22,22 +22,19 @@
 #include <algorithm>
 #include <string>
 
-#include "app/sql/connection.h"
-#include "app/sql/statement.h"
 #include "base/basictypes.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/callback.h"
 #include "base/command_line.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
+#include "base/memory/scoped_vector.h"
 #include "base/message_loop.h"
 #include "base/path_service.h"
 #include "base/scoped_temp_dir.h"
-#include "base/scoped_vector.h"
 #include "base/string_util.h"
-#include "base/task.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/download/download_item.h"
-#include "chrome/browser/history/download_create_info.h"
 #include "chrome/browser/history/history.h"
 #include "chrome/browser/history/history_backend.h"
 #include "chrome/browser/history/history_database.h"
@@ -46,26 +43,26 @@
 #include "chrome/browser/history/in_memory_history_backend.h"
 #include "chrome/browser/history/page_usage_data.h"
 #include "chrome/common/chrome_paths.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_source.h"
 #include "chrome/common/thumbnail_score.h"
 #include "chrome/tools/profiles/thumbnail-inl.h"
-#include "gfx/codec/jpeg_codec.h"
+#include "content/browser/download/download_persistent_store_info.h"
+#include "content/public/browser/download_item.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_source.h"
+#include "sql/connection.h"
+#include "sql/statement.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "ui/gfx/codec/jpeg_codec.h"
 
 using base::Time;
 using base::TimeDelta;
+using content::DownloadItem;
 
 namespace history {
 class HistoryTest;
 }
 
-// Specialize RunnableMethodTraits for HistoryTest so we can create callbacks.
-// None of these callbacks can outlast the test, so there is not need to retain
-// the HistoryTest object.
-DISABLE_RUNNABLE_METHOD_REFCOUNT(history::HistoryTest);
-
 namespace history {
 
 namespace {
@@ -88,12 +85,14 @@ class BackendDelegate : public HistoryBackend::Delegate {
       : history_test_(history_test) {
   }
 
-  virtual void NotifyProfileError(int message_id);
-  virtual void SetInMemoryBackend(InMemoryHistoryBackend* backend);
-  virtual void BroadcastNotifications(NotificationType type,
-                                      HistoryDetails* details);
-  virtual void DBLoaded() {}
-  virtual void StartTopSitesMigration() {}
+  virtual void NotifyProfileError(int backend_id,
+                                  sql::InitStatus init_status) OVERRIDE {}
+  virtual void SetInMemoryBackend(int backend_id,
+                                  InMemoryHistoryBackend* backend) OVERRIDE;
+  virtual void BroadcastNotifications(int type,
+                                      HistoryDetails* details) OVERRIDE;
+  virtual void DBLoaded(int backend_id) OVERRIDE {}
+  virtual void StartTopSitesMigration(int backend_id) OVERRIDE {}
  private:
   HistoryTest* history_test_;
 };
@@ -116,7 +115,7 @@ class HistoryTest : public testing::Test {
   // assigning the values to backend_ and db_.
   void CreateBackendAndDatabase() {
     backend_ =
-        new HistoryBackend(history_dir_, new BackendDelegate(this), NULL);
+        new HistoryBackend(history_dir_, 0, new BackendDelegate(this), NULL);
     backend_->Init(std::string(), false);
     db_ = backend_->db_.get();
     DCHECK(in_mem_backend_.get()) << "Mem backend should have been set by "
@@ -164,7 +163,7 @@ class HistoryTest : public testing::Test {
 
     // Make sure we don't have any event pending that could disrupt the next
     // test.
-    MessageLoop::current()->PostTask(FROM_HERE, new MessageLoop::QuitTask);
+    MessageLoop::current()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
     MessageLoop::current()->Run();
   }
 
@@ -172,7 +171,7 @@ class HistoryTest : public testing::Test {
     DCHECK(history_service_.get());
 
     history_service_->NotifyRenderProcessHostDestruction(0);
-    history_service_->SetOnBackendDestroyTask(new MessageLoop::QuitTask);
+    history_service_->SetOnBackendDestroyTask(MessageLoop::QuitClosure());
     history_service_->Cleanup();
     history_service_ = NULL;
 
@@ -184,8 +183,17 @@ class HistoryTest : public testing::Test {
   }
 
   int64 AddDownload(int32 state, const Time& time) {
-    DownloadCreateInfo download(FilePath(FILE_PATH_LITERAL("foo-path")),
-                                GURL("foo-url"), time, 0, 512, state, 0, false);
+    DownloadPersistentStoreInfo download(
+        FilePath(FILE_PATH_LITERAL("foo-path")),
+        GURL("foo-url"),
+        GURL(""),
+        time,
+        time,
+        0,
+        512,
+        state,
+        0,
+        0);
     return db_->CreateDownload(download);
   }
 
@@ -194,7 +202,8 @@ class HistoryTest : public testing::Test {
   // found, this will return false and those structures will not be changed.
   bool QueryURL(HistoryService* history, const GURL& url) {
     history->QueryURL(url, true, &consumer_,
-                      NewCallback(this, &HistoryTest::SaveURLAndQuit));
+                      base::Bind(&HistoryTest::SaveURLAndQuit,
+                                 base::Unretained(this)));
     MessageLoop::current()->Run();  // Will be exited in SaveURLAndQuit.
     return query_url_success_;
   }
@@ -219,7 +228,8 @@ class HistoryTest : public testing::Test {
   // returning true on success. False means the URL was not found.
   bool QueryRedirectsFrom(HistoryService* history, const GURL& url) {
     history->QueryRedirectsFrom(url, &consumer_,
-        NewCallback(this, &HistoryTest::OnRedirectQueryComplete));
+        base::Bind(&HistoryTest::OnRedirectQueryComplete,
+                   base::Unretained(this)));
     MessageLoop::current()->Run();  // Will be exited in *QueryComplete.
     return redirect_query_success_;
   }
@@ -278,22 +288,20 @@ class HistoryTest : public testing::Test {
   HistoryDatabase* db_;  // Cached reference to the backend's database.
 };
 
-void BackendDelegate::NotifyProfileError(int message_id) {
-}
-
-void BackendDelegate::SetInMemoryBackend(InMemoryHistoryBackend* backend) {
+void BackendDelegate::SetInMemoryBackend(int backend_id,
+                                         InMemoryHistoryBackend* backend) {
   // Save the in-memory backend to the history test object, this happens
   // synchronously, so we don't have to do anything fancy.
   history_test_->in_mem_backend_.reset(backend);
 }
 
-void BackendDelegate::BroadcastNotifications(NotificationType type,
+void BackendDelegate::BroadcastNotifications(int type,
                                              HistoryDetails* details) {
   // Currently, just send the notifications directly to the in-memory database.
   // We may want do do something more fancy in the future.
-  Details<HistoryDetails> det(details);
+  content::Details<HistoryDetails> det(details);
   history_test_->in_mem_backend_->Observe(type,
-      Source<HistoryTest>(NULL), det);
+      content::Source<HistoryTest>(NULL), det);
 
   // The backend passes ownership of the details pointer to us.
   delete details;
@@ -307,7 +315,7 @@ TEST_F(HistoryTest, ClearBrowsingData_Downloads) {
   Time month_ago = now - TimeDelta::FromDays(30);
 
   // Initially there should be nothing in the downloads database.
-  std::vector<DownloadCreateInfo> downloads;
+  std::vector<DownloadPersistentStoreInfo> downloads;
   db_->QueryDownloads(&downloads);
   EXPECT_EQ(0U, downloads.size());
 
@@ -320,21 +328,22 @@ TEST_F(HistoryTest, ClearBrowsingData_Downloads) {
   EXPECT_NE(0, AddDownload(DownloadItem::COMPLETE, now - one_day));
   EXPECT_NE(0, AddDownload(DownloadItem::COMPLETE, now));
   EXPECT_NE(0, AddDownload(DownloadItem::COMPLETE, now + one_day));
-  // Try the other three states.
-  EXPECT_NE(0, AddDownload(DownloadItem::COMPLETE,    month_ago));
+  // Try the other four states.
+  EXPECT_NE(0, AddDownload(DownloadItem::COMPLETE, month_ago));
   EXPECT_NE(0, in_progress = AddDownload(DownloadItem::IN_PROGRESS, month_ago));
-  EXPECT_NE(0, AddDownload(DownloadItem::CANCELLED,   month_ago));
-  EXPECT_NE(0, removing = AddDownload(DownloadItem::REMOVING,    month_ago));
+  EXPECT_NE(0, AddDownload(DownloadItem::CANCELLED, month_ago));
+  EXPECT_NE(0, AddDownload(DownloadItem::INTERRUPTED, month_ago));
+  EXPECT_NE(0, removing = AddDownload(DownloadItem::REMOVING, month_ago));
 
   // Test to see if inserts worked.
   db_->QueryDownloads(&downloads);
-  EXPECT_EQ(8U, downloads.size());
+  EXPECT_EQ(9U, downloads.size());
 
   // Try removing from current timestamp. This should delete the one in the
   // future and one very recent one.
   db_->RemoveDownloadsBetween(now, Time());
   db_->QueryDownloads(&downloads);
-  EXPECT_EQ(6U, downloads.size());
+  EXPECT_EQ(7U, downloads.size());
 
   // Try removing from two months ago. This should not delete items that are
   // 'in progress' or in 'removing' state.
@@ -357,8 +366,16 @@ TEST_F(HistoryTest, ClearBrowsingData_Downloads) {
             downloads[2].start_time.ToInternalValue());
 
   // Change state so we can delete the downloads.
-  EXPECT_TRUE(db_->UpdateDownload(512, DownloadItem::COMPLETE, in_progress));
-  EXPECT_TRUE(db_->UpdateDownload(512, DownloadItem::CANCELLED, removing));
+  DownloadPersistentStoreInfo data;
+  data.received_bytes = 512;
+  data.state = DownloadItem::COMPLETE;
+  data.end_time = base::Time::Now();
+  data.opened = false;
+  data.db_handle = in_progress;
+  EXPECT_TRUE(db_->UpdateDownload(data));
+  data.state = DownloadItem::CANCELLED;
+  data.db_handle = removing;
+  EXPECT_TRUE(db_->UpdateDownload(data));
 
   // Try removing from Time=0. This should delete all.
   db_->RemoveDownloadsBetween(Time(), Time());
@@ -390,7 +407,7 @@ TEST_F(HistoryTest, AddPage) {
   // Add the page once from a child frame.
   const GURL test_url("http://www.google.com/");
   history->AddPage(test_url, NULL, 0, GURL(),
-                   PageTransition::MANUAL_SUBFRAME,
+                   content::PAGE_TRANSITION_MANUAL_SUBFRAME,
                    history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_url));
@@ -399,7 +416,7 @@ TEST_F(HistoryTest, AddPage) {
   EXPECT_TRUE(query_url_row_.hidden());  // Hidden because of child frame.
 
   // Add the page once from the main frame (should unhide it).
-  history->AddPage(test_url, NULL, 0, GURL(), PageTransition::LINK,
+  history->AddPage(test_url, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
                    history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_url));
@@ -423,7 +440,7 @@ TEST_F(HistoryTest, AddPageSameTimes) {
   // Make sure that two pages added at the same time with no intervening
   // additions have different timestamps.
   history->AddPage(test_urls[0], now, NULL, 0, GURL(),
-                   PageTransition::LINK,
+                   content::PAGE_TRANSITION_LINK,
                    history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_urls[0]));
@@ -431,7 +448,7 @@ TEST_F(HistoryTest, AddPageSameTimes) {
   EXPECT_TRUE(now == query_url_row_.last_visit());  // gtest doesn't like Time
 
   history->AddPage(test_urls[1], now, NULL, 0, GURL(),
-                   PageTransition::LINK,
+                   content::PAGE_TRANSITION_LINK,
                    history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_urls[1]));
@@ -442,7 +459,7 @@ TEST_F(HistoryTest, AddPageSameTimes) {
   // Make sure the next page, at a different time, is also correct.
   history->AddPage(test_urls[2], now + TimeDelta::FromMinutes(1),
                    NULL, 0, GURL(),
-                   PageTransition::LINK,
+                   content::PAGE_TRANSITION_LINK,
                    history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_urls[2]));
@@ -457,8 +474,8 @@ TEST_F(HistoryTest, AddRedirect) {
   ASSERT_TRUE(history->Init(history_dir_, NULL));
 
   const char* first_sequence[] = {
-    "http://first.page/",
-    "http://second.page/"};
+    "http://first.page.com/",
+    "http://second.page.com/"};
   int first_count = arraysize(first_sequence);
   history::RedirectList first_redirects;
   for (int i = 0; i < first_count; i++)
@@ -467,7 +484,7 @@ TEST_F(HistoryTest, AddRedirect) {
   // Add the sequence of pages as a server with no referrer. Note that we need
   // to have a non-NULL page ID scope.
   history->AddPage(first_redirects.back(), MakeFakeHost(1), 0, GURL(),
-                   PageTransition::LINK, first_redirects,
+                   content::PAGE_TRANSITION_LINK, first_redirects,
                    history::SOURCE_BROWSED,  true);
 
   // The first page should be added once with a link visit type (because we set
@@ -476,8 +493,9 @@ TEST_F(HistoryTest, AddRedirect) {
   EXPECT_EQ(1, query_url_row_.visit_count());
   ASSERT_EQ(1U, query_url_visits_.size());
   int64 first_visit = query_url_visits_[0].visit_id;
-  EXPECT_EQ(PageTransition::LINK |
-            PageTransition::CHAIN_START, query_url_visits_[0].transition);
+  EXPECT_EQ(content::PAGE_TRANSITION_LINK |
+            content::PAGE_TRANSITION_CHAIN_START,
+            query_url_visits_[0].transition);
   EXPECT_EQ(0, query_url_visits_[0].referring_visit);  // No referrer.
 
   // The second page should be a server redirect type with a referrer of the
@@ -486,8 +504,9 @@ TEST_F(HistoryTest, AddRedirect) {
   EXPECT_EQ(1, query_url_row_.visit_count());
   ASSERT_EQ(1U, query_url_visits_.size());
   int64 second_visit = query_url_visits_[0].visit_id;
-  EXPECT_EQ(PageTransition::SERVER_REDIRECT |
-            PageTransition::CHAIN_END, query_url_visits_[0].transition);
+  EXPECT_EQ(content::PAGE_TRANSITION_SERVER_REDIRECT |
+            content::PAGE_TRANSITION_CHAIN_END,
+            query_url_visits_[0].transition);
   EXPECT_EQ(first_visit, query_url_visits_[0].referring_visit);
 
   // Check that the redirect finding function successfully reports it.
@@ -501,11 +520,12 @@ TEST_F(HistoryTest, AddRedirect) {
   // so we pass in a CLIENT_REDIRECT qualifier to mock that behavior.
   history::RedirectList second_redirects;
   second_redirects.push_back(first_redirects[1]);
-  second_redirects.push_back(GURL("http://last.page/"));
+  second_redirects.push_back(GURL("http://last.page.com/"));
   history->AddPage(second_redirects[1], MakeFakeHost(1), 1,
                    second_redirects[0],
-                   static_cast<PageTransition::Type>(PageTransition::LINK |
-                       PageTransition::CLIENT_REDIRECT),
+                   static_cast<content::PageTransition>(
+                       content::PAGE_TRANSITION_LINK |
+                       content::PAGE_TRANSITION_CLIENT_REDIRECT),
                    second_redirects, history::SOURCE_BROWSED, true);
 
   // The last page (source of the client redirect) should NOT have an
@@ -518,11 +538,88 @@ TEST_F(HistoryTest, AddRedirect) {
   EXPECT_TRUE(QueryURL(history, second_redirects[1]));
   EXPECT_EQ(1, query_url_row_.visit_count());
   ASSERT_EQ(1U, query_url_visits_.size());
-  EXPECT_EQ(PageTransition::CLIENT_REDIRECT |
-            PageTransition::CHAIN_END, query_url_visits_[0].transition);
+  EXPECT_EQ(content::PAGE_TRANSITION_CLIENT_REDIRECT |
+            content::PAGE_TRANSITION_CHAIN_END,
+            query_url_visits_[0].transition);
   EXPECT_EQ(second_visit, query_url_visits_[0].referring_visit);
 }
 
+TEST_F(HistoryTest, MakeIntranetURLsTyped) {
+  scoped_refptr<HistoryService> history(new HistoryService);
+  history_service_ = history;
+  ASSERT_TRUE(history->Init(history_dir_, NULL));
+
+  // Add a non-typed visit to an intranet URL on an unvisited host.  This should
+  // get promoted to a typed visit.
+  const GURL test_url("http://intranet_host/path");
+  history->AddPage(test_url, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(1, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(content::PAGE_TRANSITION_TYPED,
+      content::PageTransitionStripQualifier(query_url_visits_[0].transition));
+
+  // Add more visits on the same host.  None of these should be promoted since
+  // there is already a typed visit.
+
+  // Different path.
+  const GURL test_url2("http://intranet_host/different_path");
+  history->AddPage(test_url2, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url2));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(content::PAGE_TRANSITION_LINK,
+      content::PageTransitionStripQualifier(query_url_visits_[0].transition));
+
+  // No path.
+  const GURL test_url3("http://intranet_host/");
+  history->AddPage(test_url3, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url3));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(content::PAGE_TRANSITION_LINK,
+      content::PageTransitionStripQualifier(query_url_visits_[0].transition));
+
+  // Different scheme.
+  const GURL test_url4("https://intranet_host/");
+  history->AddPage(test_url4, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url4));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(content::PAGE_TRANSITION_LINK,
+      content::PageTransitionStripQualifier(query_url_visits_[0].transition));
+
+  // Different transition.
+  const GURL test_url5("http://intranet_host/another_path");
+  history->AddPage(test_url5, NULL, 0, GURL(),
+                   content::PAGE_TRANSITION_AUTO_BOOKMARK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url5));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(content::PAGE_TRANSITION_AUTO_BOOKMARK,
+      content::PageTransitionStripQualifier(query_url_visits_[0].transition));
+
+  // Original URL.
+  history->AddPage(test_url, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url));
+  EXPECT_EQ(2, query_url_row_.visit_count());
+  EXPECT_EQ(1, query_url_row_.typed_count());
+  ASSERT_EQ(2U, query_url_visits_.size());
+  EXPECT_EQ(content::PAGE_TRANSITION_LINK,
+      content::PageTransitionStripQualifier(query_url_visits_[1].transition));
+}
+
 TEST_F(HistoryTest, Typed) {
   scoped_refptr<HistoryService> history(new HistoryService);
   history_service_ = history;
@@ -530,7 +627,7 @@ TEST_F(HistoryTest, Typed) {
 
   // Add the page once as typed.
   const GURL test_url("http://www.google.com/");
-  history->AddPage(test_url, NULL, 0, GURL(), PageTransition::TYPED,
+  history->AddPage(test_url, NULL, 0, GURL(), content::PAGE_TRANSITION_TYPED,
                    history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_url));
@@ -540,7 +637,7 @@ TEST_F(HistoryTest, Typed) {
   EXPECT_EQ(1, query_url_row_.typed_count());
 
   // Add the page again not typed.
-  history->AddPage(test_url, NULL, 0, GURL(), PageTransition::LINK,
+  history->AddPage(test_url, NULL, 0, GURL(), content::PAGE_TRANSITION_LINK,
                    history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_url));
@@ -551,7 +648,7 @@ TEST_F(HistoryTest, Typed) {
 
   // Add the page again as a generated URL.
   history->AddPage(test_url, NULL, 0, GURL(),
-                   PageTransition::GENERATED, history::RedirectList(),
+                   content::PAGE_TRANSITION_GENERATED, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_url));
 
@@ -561,7 +658,7 @@ TEST_F(HistoryTest, Typed) {
 
   // Add the page again as a reload.
   history->AddPage(test_url, NULL, 0, GURL(),
-                   PageTransition::RELOAD, history::RedirectList(),
+                   content::PAGE_TRANSITION_RELOAD, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   EXPECT_TRUE(QueryURL(history, test_url));
 
@@ -611,14 +708,14 @@ TEST_F(HistoryTest, Segments) {
   // Add a URL.
   const GURL existing_url("http://www.google.com/");
   history->AddPage(existing_url, scope, 0, GURL(),
-                   PageTransition::TYPED, history::RedirectList(),
+                   content::PAGE_TRANSITION_TYPED, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
 
   // Make sure a segment was created.
   history->QuerySegmentUsageSince(
       &consumer_, Time::Now() - TimeDelta::FromDays(1), 10,
-      NewCallback(static_cast<HistoryTest*>(this),
-                  &HistoryTest::OnSegmentUsageAvailable));
+      base::Bind(&HistoryTest::OnSegmentUsageAvailable,
+                 base::Unretained(this)));
 
   // Wait for processing.
   MessageLoop::current()->Run();
@@ -630,14 +727,14 @@ TEST_F(HistoryTest, Segments) {
   // Add a URL which doesn't create a segment.
   const GURL link_url("http://yahoo.com/");
   history->AddPage(link_url, scope, 0, GURL(),
-                   PageTransition::LINK, history::RedirectList(),
+                   content::PAGE_TRANSITION_LINK, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
 
   // Query again
   history->QuerySegmentUsageSince(
       &consumer_, Time::Now() - TimeDelta::FromDays(1), 10,
-      NewCallback(static_cast<HistoryTest*>(this),
-                  &HistoryTest::OnSegmentUsageAvailable));
+      base::Bind(&HistoryTest::OnSegmentUsageAvailable,
+                 base::Unretained(this)));
 
   // Wait for processing.
   MessageLoop::current()->Run();
@@ -648,14 +745,14 @@ TEST_F(HistoryTest, Segments) {
 
   // Add a page linked from existing_url.
   history->AddPage(GURL("http://www.google.com/foo"), scope, 3, existing_url,
-                   PageTransition::LINK, history::RedirectList(),
+                   content::PAGE_TRANSITION_LINK, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
 
   // Query again
   history->QuerySegmentUsageSince(
       &consumer_, Time::Now() - TimeDelta::FromDays(1), 10,
-      NewCallback(static_cast<HistoryTest*>(this),
-                  &HistoryTest::OnSegmentUsageAvailable));
+      base::Bind(&HistoryTest::OnSegmentUsageAvailable,
+                 base::Unretained(this)));
 
   // Wait for processing.
   MessageLoop::current()->Run();
@@ -683,14 +780,15 @@ TEST_F(HistoryTest, MostVisitedURLs) {
 
   // Add two pages.
   history->AddPage(url0, scope, 0, GURL(),
-                   PageTransition::TYPED, history::RedirectList(),
+                   content::PAGE_TRANSITION_TYPED, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   history->AddPage(url1, scope, 0, GURL(),
-                   PageTransition::TYPED, history::RedirectList(),
+                   content::PAGE_TRANSITION_TYPED, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   history->QueryMostVisitedURLs(20, 90, &consumer_,
-                                NewCallback(static_cast<HistoryTest*>(this),
-                                    &HistoryTest::OnMostVisitedURLsAvailable));
+                                base::Bind(
+                                    &HistoryTest::OnMostVisitedURLsAvailable,
+                                    base::Unretained(this)));
   MessageLoop::current()->Run();
 
   EXPECT_EQ(2U, most_visited_urls_.size());
@@ -699,11 +797,12 @@ TEST_F(HistoryTest, MostVisitedURLs) {
 
   // Add another page.
   history->AddPage(url2, scope, 0, GURL(),
-                   PageTransition::TYPED, history::RedirectList(),
+                   content::PAGE_TRANSITION_TYPED, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   history->QueryMostVisitedURLs(20, 90, &consumer_,
-                                NewCallback(static_cast<HistoryTest*>(this),
-                                    &HistoryTest::OnMostVisitedURLsAvailable));
+                                base::Bind(
+                                    &HistoryTest::OnMostVisitedURLsAvailable,
+                                    base::Unretained(this)));
   MessageLoop::current()->Run();
 
   EXPECT_EQ(3U, most_visited_urls_.size());
@@ -713,11 +812,12 @@ TEST_F(HistoryTest, MostVisitedURLs) {
 
   // Revisit url2, making it the top URL.
   history->AddPage(url2, scope, 0, GURL(),
-                   PageTransition::TYPED, history::RedirectList(),
+                   content::PAGE_TRANSITION_TYPED, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   history->QueryMostVisitedURLs(20, 90, &consumer_,
-                                NewCallback(static_cast<HistoryTest*>(this),
-                                    &HistoryTest::OnMostVisitedURLsAvailable));
+                                base::Bind(
+                                    &HistoryTest::OnMostVisitedURLsAvailable,
+                                    base::Unretained(this)));
   MessageLoop::current()->Run();
 
   EXPECT_EQ(3U, most_visited_urls_.size());
@@ -727,11 +827,12 @@ TEST_F(HistoryTest, MostVisitedURLs) {
 
   // Revisit url1, making it the top URL.
   history->AddPage(url1, scope, 0, GURL(),
-                   PageTransition::TYPED, history::RedirectList(),
+                   content::PAGE_TRANSITION_TYPED, history::RedirectList(),
                    history::SOURCE_BROWSED, false);
   history->QueryMostVisitedURLs(20, 90, &consumer_,
-                                NewCallback(static_cast<HistoryTest*>(this),
-                                    &HistoryTest::OnMostVisitedURLsAvailable));
+                                base::Bind(
+                                    &HistoryTest::OnMostVisitedURLsAvailable,
+                                    base::Unretained(this)));
   MessageLoop::current()->Run();
 
   EXPECT_EQ(3U, most_visited_urls_.size());
@@ -746,11 +847,12 @@ TEST_F(HistoryTest, MostVisitedURLs) {
 
   // Visit url4 using redirects.
   history->AddPage(url4, scope, 0, GURL(),
-                   PageTransition::TYPED, redirects,
+                   content::PAGE_TRANSITION_TYPED, redirects,
                    history::SOURCE_BROWSED, false);
   history->QueryMostVisitedURLs(20, 90, &consumer_,
-                                NewCallback(static_cast<HistoryTest*>(this),
-                                    &HistoryTest::OnMostVisitedURLsAvailable));
+                                base::Bind(
+                                    &HistoryTest::OnMostVisitedURLsAvailable,
+                                    base::Unretained(this)));
   MessageLoop::current()->Run();
 
   EXPECT_EQ(4U, most_visited_urls_.size());
