@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,9 +6,9 @@
 #define CHROME_BROWSER_HISTORY_IN_MEMORY_DATABASE_H_
 #pragma once
 
-#include "app/sql/connection.h"
 #include "base/basictypes.h"
 #include "chrome/browser/history/url_database.h"
+#include "sql/connection.h"
 
 class FilePath;
 
@@ -33,7 +33,7 @@ class InMemoryDatabase : public URLDatabase {
 
  protected:
   // Implemented for URLDatabase.
-  virtual sql::Connection& GetDB();
+  virtual sql::Connection& GetDB() OVERRIDE;
 
  private:
   // Initializes the database connection, this is the shared code between
