@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,6 @@
 #include <vector>
 
 #include "base/command_line.h"
-#include "base/metrics/histogram.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
@@ -17,40 +16,33 @@
 #include "chrome/browser/history/in_memory_url_index.h"
 #include "chrome/browser/history/url_database.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_source.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_source.h"
 
 namespace history {
 
-// If a page becomes starred we use this id in place of the real starred id.
-// See note in OnURLsStarred.
-static const StarID kBogusStarredID = 0x0FFFFFFF;
-
 InMemoryHistoryBackend::InMemoryHistoryBackend()
     : profile_(NULL) {
 }
 
 InMemoryHistoryBackend::~InMemoryHistoryBackend() {
+  if (index_.get())
+    index_->ShutDown();
 }
 
 bool InMemoryHistoryBackend::Init(const FilePath& history_filename,
+                                  const FilePath& history_dir,
                                   URLDatabase* db,
                                   const std::string& languages) {
   db_.reset(new InMemoryDatabase);
   bool success = db_->InitFromDisk(history_filename);
-#if 0
-  // TODO(mrossetti): Temporary removal to help determine if the
-  // InMemoryURLIndex is contributing to a startup slowdown.
   if (!CommandLine::ForCurrentProcess()->HasSwitch(
-      switches::kDisableHistoryQuickProvider)) {
-    index_.reset(new InMemoryURLIndex());
-    base::TimeTicks beginning_time = base::TimeTicks::Now();
+          switches::kDisableHistoryQuickProvider)) {
+    index_.reset(new InMemoryURLIndex(history_dir));
     index_->Init(db, languages);
-    UMA_HISTOGRAM_TIMES("Autocomplete.HistoryDatabaseIndexingTime",
-                        base::TimeTicks::Now() - beginning_time);
   }
-#endif
   return success;
 }
 
@@ -70,25 +62,28 @@ void InMemoryHistoryBackend::AttachToHistoryService(Profile* profile) {
 
   // Register for the notifications we care about.
   // We only want notifications for the associated profile.
-  Source<Profile> source(profile_);
-  registrar_.Add(this, NotificationType::HISTORY_URL_VISITED, source);
-  registrar_.Add(this, NotificationType::HISTORY_TYPED_URLS_MODIFIED, source);
-  registrar_.Add(this, NotificationType::HISTORY_URLS_DELETED, source);
-  registrar_.Add(this, NotificationType::HISTORY_KEYWORD_SEARCH_TERM_UPDATED,
+  content::Source<Profile> source(profile_);
+  registrar_.Add(this, chrome::NOTIFICATION_HISTORY_URL_VISITED, source);
+  registrar_.Add(this, chrome::NOTIFICATION_HISTORY_TYPED_URLS_MODIFIED,
                  source);
-  registrar_.Add(this, NotificationType::TEMPLATE_URL_REMOVED, source);
+  registrar_.Add(this, chrome::NOTIFICATION_HISTORY_URLS_DELETED, source);
+  registrar_.Add(this,
+                 chrome::NOTIFICATION_HISTORY_KEYWORD_SEARCH_TERM_UPDATED,
+                 source);
+  registrar_.Add(this, chrome::NOTIFICATION_TEMPLATE_URL_REMOVED, source);
 }
 
-void InMemoryHistoryBackend::Observe(NotificationType type,
-                                     const NotificationSource& source,
-                                     const NotificationDetails& details) {
-  switch (type.value) {
-    case NotificationType::HISTORY_URL_VISITED: {
-      Details<history::URLVisitedDetails> visited_details(details);
-      PageTransition::Type primary_type =
-          PageTransition::StripQualifier(visited_details->transition);
+void InMemoryHistoryBackend::Observe(
+    int type,
+    const content::NotificationSource& source,
+    const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_HISTORY_URL_VISITED: {
+      content::Details<history::URLVisitedDetails> visited_details(details);
+      content::PageTransition primary_type =
+          content::PageTransitionStripQualifier(visited_details->transition);
       if (visited_details->row.typed_count() > 0 ||
-          primary_type == PageTransition::KEYWORD ||
+          primary_type == content::PAGE_TRANSITION_KEYWORD ||
           HasKeyword(visited_details->row.url())) {
         URLsModifiedDetails modified_details;
         modified_details.changed_urls.push_back(visited_details->row);
@@ -96,20 +91,21 @@ void InMemoryHistoryBackend::Observe(NotificationType type,
       }
       break;
     }
-    case NotificationType::HISTORY_KEYWORD_SEARCH_TERM_UPDATED:
+    case chrome::NOTIFICATION_HISTORY_KEYWORD_SEARCH_TERM_UPDATED:
       OnKeywordSearchTermUpdated(
-          *Details<history::KeywordSearchTermDetails>(details).ptr());
+          *content::Details<history::KeywordSearchTermDetails>(details).ptr());
       break;
-    case NotificationType::HISTORY_TYPED_URLS_MODIFIED:
+    case chrome::NOTIFICATION_HISTORY_TYPED_URLS_MODIFIED:
       OnTypedURLsModified(
-          *Details<history::URLsModifiedDetails>(details).ptr());
+          *content::Details<history::URLsModifiedDetails>(details).ptr());
       break;
-    case NotificationType::HISTORY_URLS_DELETED:
-      OnURLsDeleted(*Details<history::URLsDeletedDetails>(details).ptr());
+    case chrome::NOTIFICATION_HISTORY_URLS_DELETED:
+      OnURLsDeleted(
+          *content::Details<history::URLsDeletedDetails>(details).ptr());
       break;
-    case NotificationType::TEMPLATE_URL_REMOVED:
+    case chrome::NOTIFICATION_TEMPLATE_URL_REMOVED:
       db_->DeleteAllSearchTermsForKeyword(
-          *(Details<TemplateURLID>(details).ptr()));
+          *(content::Details<TemplateURLID>(details).ptr()));
       break;
     default:
       // For simplicity, the unit tests send us all notifications, even when
@@ -134,7 +130,9 @@ void InMemoryHistoryBackend::OnTypedURLsModified(
     if (id)
       db_->UpdateURLRow(id, *i);
     else
-      db_->AddURL(*i);
+      id = db_->AddURL(*i);
+    if (index_.get())
+      index_->UpdateURL(id, *i);
   }
 }
 
@@ -147,6 +145,8 @@ void InMemoryHistoryBackend::OnURLsDeleted(const URLsDeletedDetails& details) {
     db_.reset(new InMemoryDatabase);
     if (!db_->InitFromScratch())
       db_.reset();
+    if (index_.get())
+      index_->ReloadFromHistory(db_.get());
     return;
   }
 
@@ -158,6 +158,8 @@ void InMemoryHistoryBackend::OnURLsDeleted(const URLsDeletedDetails& details) {
       // We typically won't have most of them since we only have a subset of
       // history, so ignore errors.
       db_->DeleteURLRow(id);
+      if (index_.get())
+        index_->DeleteURL(id);
     }
   }
 }
