@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,7 @@
 // in-line autocomplete.
 //
 // It is created on the history thread and passed to the main thread where
-// operations can be completed synchronously. It listenes for notifications
+// operations can be completed synchronously. It listens for notifications
 // from the "regular" history backend and keeps itself in sync.
 
 #ifndef CHROME_BROWSER_HISTORY_IN_MEMORY_HISTORY_BACKEND_H_
@@ -18,13 +18,12 @@
 
 #include "base/basictypes.h"
 #include "base/gtest_prod_util.h"
-#include "base/scoped_ptr.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "base/memory/scoped_ptr.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 class FilePath;
 class GURL;
-class HistoryDatabase;
 class Profile;
 
 namespace history {
@@ -36,13 +35,20 @@ class URLDatabase;
 struct URLsDeletedDetails;
 struct URLsModifiedDetails;
 
-class InMemoryHistoryBackend : public NotificationObserver {
+class InMemoryHistoryBackend : public content::NotificationObserver {
  public:
   InMemoryHistoryBackend();
-  ~InMemoryHistoryBackend();
+  virtual ~InMemoryHistoryBackend();
 
-  // Initializes with data from the given history database.
+  // Initializes the backend from the history database pointed to by the
+  // full path in |history_filename|. |history_dir| is the path to the
+  // directory containing the history database and is also used
+  // as the directory where the InMemoryURLIndex's cache is kept. |db| is
+  // used for building the InMemoryURLIndex. |languages| gives the
+  // preferred user languages with which URLs and page titles are
+  // interpreted while decomposing into words and characters during indexing.
   bool Init(const FilePath& history_filename,
+            const FilePath& history_dir,
             URLDatabase* db,
             const std::string& languages);
 
@@ -59,9 +65,9 @@ class InMemoryHistoryBackend : public NotificationObserver {
   }
 
   // Notification callback.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Return the quick history index.
   history::InMemoryURLIndex* InMemoryIndex() const { return index_.get(); }
@@ -81,7 +87,7 @@ class InMemoryHistoryBackend : public NotificationObserver {
   // Returns true if there is a keyword associated with the specified url.
   bool HasKeyword(const GURL& url);
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   scoped_ptr<InMemoryDatabase> db_;
 
