@@ -7,33 +7,42 @@
 #include <algorithm>
 #include <set>
 
-#include "base/command_line.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/logging.h"
 #include "base/md5.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/dom_ui/most_visited_handler.h"
 #include "chrome/browser/history/history_backend.h"
 #include "chrome/browser/history/history_notifications.h"
 #include "chrome/browser/history/page_usage_data.h"
 #include "chrome/browser/history/top_sites_backend.h"
 #include "chrome/browser/history/top_sites_cache.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/prefs/scoped_user_pref_update.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/common/chrome_switches.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/browser/ui/webui/ntp/most_visited_handler.h"
+#include "chrome/browser/ui/webui/ntp/new_tab_ui.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/thumbnail_score.h"
-#include "gfx/codec/jpeg_codec.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/navigation_details.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/locale_settings.h"
-#include "third_party/skia/include/core/SkBitmap.h"
+#include "grit/theme_resources.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/image/image_util.h"
+
+using content::BrowserThread;
+using content::NavigationController;
 
 namespace history {
 
@@ -44,7 +53,6 @@ static const size_t kTopSitesNumber = 20;
 // temp_images_ for details.
 static const size_t kMaxTempTopImages = 8;
 
-static const size_t kTopSitesShown = 8;
 static const int kDaysOfHistory = 90;
 // Time from startup to first HistoryService query.
 static const int64 kUpdateIntervalSecs = 15;
@@ -52,18 +60,18 @@ static const int64 kUpdateIntervalSecs = 15;
 static const int64 kMinUpdateIntervalMinutes = 1;
 static const int64 kMaxUpdateIntervalMinutes = 60;
 
-// IDs of the sites we force into top sites.
-static const int kPrepopulatePageIDs[] =
-    { IDS_CHROME_WELCOME_URL, IDS_THEMES_GALLERY_URL };
+// Use 100 quality (highest quality) because we're very sensitive to
+// artifacts for these small sized, highly detailed images.
+static const int kTopSitesImageQuality = 100;
 
-// Favicons of the sites we force into top sites.
-static const char kPrepopulateFaviconURLs[][54] =
-    { "chrome://theme/IDR_NEWTAB_CHROME_WELCOME_PAGE_FAVICON",
-      "chrome://theme/IDR_NEWTAB_THEMES_GALLERY_FAVICON" };
-
-static const int kPrepopulateTitleIDs[] =
-    { IDS_NEW_TAB_CHROME_WELCOME_PAGE_TITLE,
-      IDS_NEW_TAB_THEMES_GALLERY_PAGE_TITLE };
+const TopSites::PrepopulatedPage kPrepopulatedPages[] = {
+  { IDS_CHROME_WELCOME_URL, IDS_NEW_TAB_CHROME_WELCOME_PAGE_TITLE,
+    IDR_PRODUCT_LOGO_16, IDR_NEWTAB_CHROME_WELCOME_PAGE_THUMBNAIL,
+    SkColorSetRGB(0, 147, 60) },
+  { IDS_WEBSTORE_URL, IDS_EXTENSION_WEB_STORE_TITLE,
+    IDR_WEBSTORE_ICON_16, IDR_NEWTAB_WEBSTORE_THUMBNAIL,
+    SkColorSetRGB(63, 132, 197) }
+};
 
 namespace {
 
@@ -79,7 +87,7 @@ class LoadThumbnailsFromHistoryTask : public HistoryDBTask {
         result_count_(result_count) {
     // l10n_util isn't thread safe, so cache for use on the db thread.
     ignore_urls_.insert(l10n_util::GetStringUTF8(IDS_CHROME_WELCOME_URL));
-    ignore_urls_.insert(l10n_util::GetStringUTF8(IDS_THEMES_GALLERY_URL));
+    ignore_urls_.insert(l10n_util::GetStringUTF8(IDS_WEBSTORE_URL));
   }
 
   virtual bool RunOnDBThread(history::HistoryBackend* backend,
@@ -127,7 +135,7 @@ class LoadThumbnailsFromHistoryTask : public HistoryDBTask {
 }  // namespace
 
 TopSites::TopSites(Profile* profile)
-    : backend_(new TopSitesBackend()),
+    : backend_(NULL),
       cache_(new TopSitesCache()),
       thread_safe_cache_(new TopSitesCache()),
       profile_(profile),
@@ -140,24 +148,41 @@ TopSites::TopSites(Profile* profile)
   if (!profile_)
     return;
 
-  if (NotificationService::current()) {
-    registrar_.Add(this, NotificationType::HISTORY_URLS_DELETED,
-                   Source<Profile>(profile_));
-    registrar_.Add(this, NotificationType::NAV_ENTRY_COMMITTED,
-                   NotificationService::AllSources());
+  if (content::NotificationService::current()) {
+    registrar_.Add(this, chrome::NOTIFICATION_HISTORY_URLS_DELETED,
+                   content::Source<Profile>(profile_));
+    // Listen for any nav commits. We'll ignore those not related to this
+    // profile when we get the notification.
+    registrar_.Add(this, content::NOTIFICATION_NAV_ENTRY_COMMITTED,
+                   content::NotificationService::AllSources());
   }
 
-  blacklist_ = profile_->GetPrefs()->
-      GetMutableDictionary(prefs::kNTPMostVisitedURLsBlacklist);
-  pinned_urls_ = profile_->GetPrefs()->
-      GetMutableDictionary(prefs::kNTPMostVisitedPinnedURLs);
+  // We create update objects here to be sure that dictionaries are created
+  // in the user preferences.
+  DictionaryPrefUpdate(profile_->GetPrefs(),
+                       prefs::kNTPMostVisitedURLsBlacklist).Get();
+  DictionaryPrefUpdate(profile_->GetPrefs(),
+                       prefs::kNTPMostVisitedPinnedURLs).Get();
+
+  // Now the dictionaries are guaranteed to exist and we can cache pointers
+  // to them.
+  blacklist_ =
+      profile_->GetPrefs()->GetDictionary(prefs::kNTPMostVisitedURLsBlacklist);
+  pinned_urls_ =
+      profile_->GetPrefs()->GetDictionary(prefs::kNTPMostVisitedPinnedURLs);
+  DCHECK(blacklist_ != NULL);
+  DCHECK(pinned_urls_ != NULL);
 }
 
 void TopSites::Init(const FilePath& db_name) {
+  // Create the backend here, rather than in the constructor, so that
+  // unit tests that do not need the backend can run without a problem.
+  backend_ = new TopSitesBackend;
   backend_->Init(db_name);
   backend_->GetMostVisitedThumbnails(
-      &cancelable_consumer_,
-      NewCallback(this, &TopSites::OnGotMostVisitedThumbnails));
+      &top_sites_consumer_,
+      base::Bind(&TopSites::OnGotMostVisitedThumbnails,
+                 base::Unretained(this)));
 
   // History may have already finished loading by the time we're created.
   HistoryService* history = profile_->GetHistoryServiceWithoutCreating();
@@ -170,7 +195,7 @@ void TopSites::Init(const FilePath& db_name) {
 }
 
 bool TopSites::SetPageThumbnail(const GURL& url,
-                                const SkBitmap& thumbnail,
+                                gfx::Image* thumbnail,
                                 const ThumbnailScore& score) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
 
@@ -181,8 +206,8 @@ bool TopSites::SetPageThumbnail(const GURL& url,
   }
 
   bool add_temp_thumbnail = false;
-  if (!cache_->IsKnownURL(url)) {
-    if (cache_->top_sites().size() < kTopSitesNumber) {
+  if (!IsKnownURL(url)) {
+    if (!IsFull()) {
       add_temp_thumbnail = true;
     } else {
       return false;  // This URL is not known to us.
@@ -208,7 +233,7 @@ bool TopSites::SetPageThumbnail(const GURL& url,
 }
 
 void TopSites::GetMostVisitedURLs(CancelableRequestConsumer* consumer,
-                                  GetTopSitesCallback* callback) {
+                                  const GetTopSitesCallback& callback) {
   // WARNING: this may be invoked on any thread.
   scoped_refptr<CancelableRequest<GetTopSitesCallback> > request(
       new CancelableRequest<GetTopSitesCallback>(callback));
@@ -227,16 +252,50 @@ void TopSites::GetMostVisitedURLs(CancelableRequestConsumer* consumer,
 
     filtered_urls = thread_safe_cache_->top_sites();
   }
-  request->ForwardResult(GetTopSitesCallback::TupleType(filtered_urls));
+  request->ForwardResult(filtered_urls);
 }
 
 bool TopSites::GetPageThumbnail(const GURL& url,
-                                scoped_refptr<RefCountedBytes>* bytes) {
+                                scoped_refptr<RefCountedMemory>* bytes) {
+  // WARNING: this may be invoked on any thread.
+  {
+    base::AutoLock lock(lock_);
+    if (thread_safe_cache_->GetPageThumbnail(url, bytes))
+      return true;
+  }
+
+  // Resource bundle is thread safe.
+  for (size_t i = 0; i < arraysize(kPrepopulatedPages); i++) {
+    if (url.spec() == l10n_util::GetStringUTF8(kPrepopulatedPages[i].url_id)) {
+      *bytes = ResourceBundle::GetSharedInstance().LoadDataResourceBytes(
+          kPrepopulatedPages[i].thumbnail_id);
+      return true;
+    }
+  }
+
+  return false;
+}
+
+bool TopSites::GetPageThumbnailScore(const GURL& url,
+                                     ThumbnailScore* score) {
   // WARNING: this may be invoked on any thread.
   base::AutoLock lock(lock_);
-  return thread_safe_cache_->GetPageThumbnail(url, bytes);
+  return thread_safe_cache_->GetPageThumbnailScore(url, score);
 }
 
+bool TopSites::GetTemporaryPageThumbnailScore(const GURL& url,
+                                              ThumbnailScore* score) {
+  for (TempImages::iterator i = temp_images_.begin(); i != temp_images_.end();
+       ++i) {
+    if (i->first == url) {
+      *score = i->second.thumbnail_score;
+      return true;
+    }
+  }
+  return false;
+}
+
+
 // Returns the index of |url| in |urls|, or -1 if not found.
 static int IndexOf(const MostVisitedURLList& urls, const GURL& url) {
   for (size_t i = 0; i < urls.size(); i++) {
@@ -248,14 +307,18 @@ static int IndexOf(const MostVisitedURLList& urls, const GURL& url) {
 
 void TopSites::MigrateFromHistory() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  DCHECK_EQ(history_state_, HISTORY_LOADING);
+
+  if (history_state_ != HISTORY_LOADING) {
+    // This can happen if history was unloaded then loaded again.
+    return;
+  }
 
   history_state_ = HISTORY_MIGRATING;
   profile_->GetHistoryService(Profile::EXPLICIT_ACCESS)->ScheduleDBTask(
       new LoadThumbnailsFromHistoryTask(
           this,
           num_results_to_request_from_history()),
-      &cancelable_consumer_);
+      &history_consumer_);
   MigratePinnedURLs();
 }
 
@@ -286,13 +349,13 @@ void TopSites::FinishHistoryMigration(const ThumbnailMigration& data) {
   // that notifies us when done. When done we'll know everything was written and
   // we can tell history to finish its part of migration.
   backend_->DoEmptyRequest(
-      &cancelable_consumer_,
-      NewCallback(this, &TopSites::OnHistoryMigrationWrittenToDisk));
+      &top_sites_consumer_,
+      base::Bind(&TopSites::OnHistoryMigrationWrittenToDisk,
+                 base::Unretained(this)));
 }
 
 void TopSites::HistoryLoaded() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  DCHECK_NE(history_state_, HISTORY_LOADED);
 
   if (history_state_ != HISTORY_MIGRATING) {
     // No migration from history is needed.
@@ -305,6 +368,20 @@ void TopSites::HistoryLoaded() {
       MoveStateToLoaded();
     }
   }
+  // else case can happen if history is unloaded, then loaded again.
+}
+
+void TopSites::SyncWithHistory() {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  if (loaded_ && temp_images_.size()) {
+    // If we have temporary thumbnails it means there isn't much data, and most
+    // likely the user is first running Chrome. During this time we throttle
+    // updating from history by 30 seconds. If the user creates a new tab page
+    // during this window of time we force updating from history so that the new
+    // tab page isn't so far out of date.
+    timer_.Stop();
+    StartQueryForMostVisited();
+  }
 }
 
 bool TopSites::HasBlacklistedItems() const {
@@ -316,15 +393,27 @@ void TopSites::AddBlacklistedURL(const GURL& url) {
 
   RemovePinnedURL(url);
   Value* dummy = Value::CreateNullValue();
-  blacklist_->SetWithoutPathExpansion(GetURLHash(url), dummy);
+  {
+    DictionaryPrefUpdate update(profile_->GetPrefs(),
+                                prefs::kNTPMostVisitedURLsBlacklist);
+    DictionaryValue* blacklist = update.Get();
+    blacklist->SetWithoutPathExpansion(GetURLHash(url), dummy);
+  }
 
   ResetThreadSafeCache();
+  NotifyTopSitesChanged();
 }
 
 void TopSites::RemoveBlacklistedURL(const GURL& url) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  blacklist_->RemoveWithoutPathExpansion(GetURLHash(url), NULL);
+  {
+    DictionaryPrefUpdate update(profile_->GetPrefs(),
+                                prefs::kNTPMostVisitedURLsBlacklist);
+    DictionaryValue* blacklist = update.Get();
+    blacklist->RemoveWithoutPathExpansion(GetURLHash(url), NULL);
+  }
   ResetThreadSafeCache();
+  NotifyTopSitesChanged();
 }
 
 bool TopSites::IsBlacklisted(const GURL& url) {
@@ -334,8 +423,14 @@ bool TopSites::IsBlacklisted(const GURL& url) {
 
 void TopSites::ClearBlacklistedURLs() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  blacklist_->Clear();
+  {
+    DictionaryPrefUpdate update(profile_->GetPrefs(),
+                                prefs::kNTPMostVisitedURLsBlacklist);
+    DictionaryValue* blacklist = update.Get();
+    blacklist->Clear();
+  }
   ResetThreadSafeCache();
+  NotifyTopSitesChanged();
 }
 
 void TopSites::AddPinnedURL(const GURL& url, size_t pinned_index) {
@@ -349,9 +444,16 @@ void TopSites::AddPinnedURL(const GURL& url, size_t pinned_index) {
     RemovePinnedURL(url);
 
   Value* index = Value::CreateIntegerValue(pinned_index);
-  pinned_urls_->SetWithoutPathExpansion(GetURLString(url), index);
+
+  {
+    DictionaryPrefUpdate update(profile_->GetPrefs(),
+                                prefs::kNTPMostVisitedPinnedURLs);
+    DictionaryValue* pinned_urls = update.Get();
+    pinned_urls->SetWithoutPathExpansion(GetURLString(url), index);
+  }
 
   ResetThreadSafeCache();
+  NotifyTopSitesChanged();
 }
 
 bool TopSites::IsURLPinned(const GURL& url) {
@@ -362,9 +464,15 @@ bool TopSites::IsURLPinned(const GURL& url) {
 void TopSites::RemovePinnedURL(const GURL& url) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
 
-  pinned_urls_->RemoveWithoutPathExpansion(GetURLString(url), NULL);
+  {
+    DictionaryPrefUpdate update(profile_->GetPrefs(),
+                                prefs::kNTPMostVisitedPinnedURLs);
+    DictionaryValue* pinned_urls = update.Get();
+    pinned_urls->RemoveWithoutPathExpansion(GetURLString(url), NULL);
+  }
 
   ResetThreadSafeCache();
+  NotifyTopSitesChanged();
 }
 
 bool TopSites::GetPinnedURLAtIndex(size_t index, GURL* url) {
@@ -386,7 +494,8 @@ void TopSites::Shutdown() {
   // Cancel all requests so that the service doesn't callback to us after we've
   // invoked Shutdown (this could happen if we have a pending request and
   // Shutdown is invoked).
-  cancelable_consumer_.CancelAllRequests();
+  history_consumer_.CancelAllRequests();
+  top_sites_consumer_.CancelAllRequests();
   backend_->Shutdown();
 }
 
@@ -443,12 +552,21 @@ CancelableRequestProvider::Handle TopSites::StartQueryForMostVisited() {
     return hs->QueryMostVisitedURLs(
         num_results_to_request_from_history(),
         kDaysOfHistory,
-        &cancelable_consumer_,
-        NewCallback(this, &TopSites::OnTopSitesAvailableFromHistory));
+        &history_consumer_,
+        base::Bind(&TopSites::OnTopSitesAvailableFromHistory,
+                   base::Unretained(this)));
   }
   return 0;
 }
 
+bool TopSites::IsKnownURL(const GURL& url) {
+  return loaded_ && cache_->IsKnownURL(url);
+}
+
+bool TopSites::IsFull() {
+  return loaded_ && cache_->top_sites().size() >= kTopSitesNumber;
+}
+
 TopSites::~TopSites() {
 }
 
@@ -500,22 +618,19 @@ bool TopSites::SetPageThumbnailEncoded(const GURL& url,
 }
 
 // static
-bool TopSites::EncodeBitmap(const SkBitmap& bitmap,
+bool TopSites::EncodeBitmap(gfx::Image* bitmap,
                             scoped_refptr<RefCountedBytes>* bytes) {
-  *bytes = new RefCountedBytes();
-  SkAutoLockPixels bitmap_lock(bitmap);
-  std::vector<unsigned char> data;
-  if (!gfx::JPEGCodec::Encode(
-          reinterpret_cast<unsigned char*>(bitmap.getAddr32(0, 0)),
-          gfx::JPEGCodec::FORMAT_BGRA, bitmap.width(),
-          bitmap.height(),
-          static_cast<int>(bitmap.rowBytes()), 90,
-          &data)) {
+  if (!bitmap)
     return false;
-  }
+  *bytes = new RefCountedBytes();
+  std::vector<unsigned char> data;
+  if (!gfx::JPEGEncodedDataFromImage(*bitmap, kTopSitesImageQuality, &data))
+    return false;
+
   // As we're going to cache this data, make sure the vector is only as big as
-  // it needs to be.
-  (*bytes)->data = data;
+  // it needs to be, as JPEGCodec::Encode() over-allocates data.capacity().
+  // (In a C++0x future, we can just call shrink_to_fit() in Encode())
+  (*bytes)->data() = data;
   return true;
 }
 
@@ -560,13 +675,12 @@ int TopSites::GetRedirectDistanceForURL(const MostVisitedURL& most_visited,
 // static
 MostVisitedURLList TopSites::GetPrepopulatePages() {
   MostVisitedURLList urls;
-  urls.resize(arraysize(kPrepopulatePageIDs));
-  for (size_t i = 0; i < arraysize(kPrepopulatePageIDs); ++i) {
+  urls.resize(arraysize(kPrepopulatedPages));
+  for (size_t i = 0; i < urls.size(); ++i) {
     MostVisitedURL& url = urls[i];
-    url.url = GURL(l10n_util::GetStringUTF8(kPrepopulatePageIDs[i]));
+    url.url = GURL(l10n_util::GetStringUTF8(kPrepopulatedPages[i].url_id));
     url.redirects.push_back(url.url);
-    url.favicon_url = GURL(kPrepopulateFaviconURLs[i]);
-    url.title = l10n_util::GetStringUTF16(kPrepopulateTitleIDs[i]);
+    url.title = l10n_util::GetStringUTF16(kPrepopulatedPages[i].title_id);
   }
   return urls;
 }
@@ -604,7 +718,14 @@ void TopSites::MigratePinnedURLs() {
         tmp_map[GURL(url_string)] = index;
     }
   }
-  pinned_urls_->Clear();
+
+  {
+    DictionaryPrefUpdate update(profile_->GetPrefs(),
+                                prefs::kNTPMostVisitedPinnedURLs);
+    DictionaryValue* pinned_urls = update.Get();
+    pinned_urls->Clear();
+  }
+
   for (std::map<GURL, size_t>::iterator it = tmp_map.begin();
        it != tmp_map.end(); ++it)
     AddPinnedURL(it->first, it->second);
@@ -618,7 +739,8 @@ void TopSites::ApplyBlacklistAndPinnedURLs(const MostVisitedURLList& urls,
       urls_copy.push_back(urls[i]);
   }
 
-  for (size_t pinned_index = 0; pinned_index < kTopSitesShown; pinned_index++) {
+  for (size_t pinned_index = 0; pinned_index < kTopSitesNumber;
+       pinned_index++) {
     GURL url;
     bool found = GetPinnedURLAtIndex(pinned_index, &url);
     if (!found)
@@ -641,7 +763,7 @@ void TopSites::ApplyBlacklistAndPinnedURLs(const MostVisitedURLList& urls,
 
   // Add non-pinned URLs in the empty spots.
   size_t current_url = 0;  // Index into the remaining URLs in urls_copy.
-  for (size_t i = 0; i < kTopSitesShown && current_url < urls_copy.size();
+  for (size_t i = 0; i < kTopSitesNumber && current_url < urls_copy.size();
        i++) {
     if (i == out->size()) {
       out->push_back(urls_copy[current_url]);
@@ -665,11 +787,11 @@ std::string TopSites::GetURLString(const GURL& url) {
 std::string TopSites::GetURLHash(const GURL& url) {
   // We don't use canonical URLs here to be able to blacklist only one of
   // the two 'duplicate' sites, e.g. 'gmail.com' and 'mail.google.com'.
-  return MD5String(url.spec());
+  return base::MD5String(url.spec());
 }
 
 base::TimeDelta TopSites::GetUpdateDelay() {
-  if (cache_->top_sites().size() <= arraysize(kPrepopulateTitleIDs))
+  if (cache_->top_sites().size() <= arraysize(kPrepopulatedPages))
     return base::TimeDelta::FromSeconds(30);
 
   int64 range = kMaxUpdateIntervalMinutes - kMinUpdateIntervalMinutes;
@@ -687,18 +809,18 @@ void TopSites::ProcessPendingCallbacks(
        i != pending_callbacks.end(); ++i) {
     scoped_refptr<CancelableRequest<GetTopSitesCallback> > request = *i;
     if (!request->canceled())
-      request->ForwardResult(GetTopSitesCallback::TupleType(urls));
+      request->ForwardResult(urls);
   }
 }
 
-void TopSites::Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details) {
+void TopSites::Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) {
   if (!loaded_)
     return;
 
-  if (type == NotificationType::HISTORY_URLS_DELETED) {
-    Details<history::URLsDeletedDetails> deleted_details(details);
+  if (type == chrome::NOTIFICATION_HISTORY_URLS_DELETED) {
+    content::Details<history::URLsDeletedDetails> deleted_details(details);
     if (deleted_details->all_history) {
       SetTopSites(MostVisitedURLList());
       backend_->ResetDatabase();
@@ -723,13 +845,17 @@ void TopSites::Observe(NotificationType type,
       SetTopSites(new_top_sites);
     }
     StartQueryForMostVisited();
-  } else if (type == NotificationType::NAV_ENTRY_COMMITTED) {
-    if (cache_->top_sites().size() < kTopSitesNumber) {
-      NavigationController::LoadCommittedDetails* load_details =
-          Details<NavigationController::LoadCommittedDetails>(details).ptr();
+  } else if (type == content::NOTIFICATION_NAV_ENTRY_COMMITTED) {
+    NavigationController* controller =
+        content::Source<NavigationController>(source).ptr();
+    Profile* profile = Profile::FromBrowserContext(
+        controller->GetWebContents()->GetBrowserContext());
+    if (profile == profile_ && !IsFull()) {
+      content::LoadCommittedDetails* load_details =
+          content::Details<content::LoadCommittedDetails>(details).ptr();
       if (!load_details)
         return;
-      const GURL& url = load_details->entry->url();
+      const GURL& url = load_details->entry->GetURL();
       if (!cache_->IsKnownURL(url) && HistoryService::CanAddURL(url)) {
         // To avoid slamming history we throttle requests when the url updates.
         // To do otherwise negatively impacts perf tests.
@@ -747,8 +873,9 @@ void TopSites::SetTopSites(const MostVisitedURLList& new_top_sites) {
 
   TopSitesDelta delta;
   DiffMostVisited(cache_->top_sites(), top_sites, &delta);
-  if (!delta.deleted.empty() || !delta.added.empty() || !delta.moved.empty())
+  if (!delta.deleted.empty() || !delta.added.empty() || !delta.moved.empty()) {
     backend_->UpdateTopSites(delta);
+  }
 
   last_num_urls_changed_ = delta.added.size() + delta.moved.size();
 
@@ -782,6 +909,7 @@ void TopSites::SetTopSites(const MostVisitedURLList& new_top_sites) {
 
   ResetThreadSafeCache();
   ResetThreadSafeImageCache();
+  NotifyTopSitesChanged();
 
   // Restart the timer that queries history for top sites. This is done to
   // ensure we stay in sync with history.
@@ -816,9 +944,10 @@ void TopSites::MoveStateToLoaded() {
 
   ProcessPendingCallbacks(pending_callbacks, filtered_urls);
 
-  NotificationService::current()->Notify(NotificationType::TOP_SITES_LOADED,
-                                         Source<Profile>(profile_),
-                                         Details<TopSites>(this));
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_TOP_SITES_LOADED,
+      content::Source<Profile>(profile_),
+      content::Details<TopSites>(this));
 }
 
 void TopSites::ResetThreadSafeCache() {
@@ -831,7 +960,13 @@ void TopSites::ResetThreadSafeCache() {
 void TopSites::ResetThreadSafeImageCache() {
   base::AutoLock lock(lock_);
   thread_safe_cache_->SetThumbnails(cache_->images());
-  thread_safe_cache_->RemoveUnreferencedThumbnails();
+}
+
+void TopSites::NotifyTopSitesChanged() {
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_TOP_SITES_CHANGED,
+      content::Source<TopSites>(this),
+      content::NotificationService::NoDetails());
 }
 
 void TopSites::RestartQueryForTopSitesTimer(base::TimeDelta delta) {
@@ -842,7 +977,7 @@ void TopSites::RestartQueryForTopSitesTimer(base::TimeDelta delta) {
 
   timer_start_time_ = base::TimeTicks::Now();
   timer_.Stop();
-  timer_.Start(delta, this, &TopSites::TimerFired);
+  timer_.Start(FROM_HERE, delta, this, &TopSites::TimerFired);
 }
 
 void TopSites::OnHistoryMigrationWrittenToDisk(TopSitesBackend::Handle handle) {
@@ -902,10 +1037,10 @@ void TopSites::OnTopSitesAvailableFromHistory(
   SetTopSites(pages);
 
   // Used only in testing.
-  NotificationService::current()->Notify(
-      NotificationType::TOP_SITES_UPDATED,
-      Source<TopSites>(this),
-      Details<CancelableRequestProvider::Handle>(&handle));
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_TOP_SITES_UPDATED,
+      content::Source<TopSites>(this),
+      content::Details<CancelableRequestProvider::Handle>(&handle));
 }
 
 }  // namespace history
