@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,24 +12,29 @@
 #include <utility>
 
 #include "base/basictypes.h"
+#include "base/callback.h"
 #include "base/gtest_prod_util.h"
-#include "base/ref_counted.h"
-#include "base/ref_counted_memory.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/ref_counted_memory.h"
 #include "base/synchronization/lock.h"
 #include "base/time.h"
 #include "base/timer.h"
 #include "chrome/browser/cancelable_request.h"
-#include "chrome/browser/history/history_types.h"
 #include "chrome/browser/history/history.h"
+#include "chrome/browser/history/history_types.h"
 #include "chrome/browser/history/page_usage_data.h"
 #include "chrome/common/thumbnail_score.h"
 #include "googleurl/src/gurl.h"
+#include "third_party/skia/include/core/SkColor.h"
+#include "ui/gfx/image/image.h"
 
-class DictionaryValue;
 class FilePath;
-class SkBitmap;
 class Profile;
 
+namespace base {
+class DictionaryValue;
+}
+
 namespace history {
 
 class TopSitesCache;
@@ -46,7 +51,7 @@ class TopSitesTest;
 // db using TopSitesBackend.
 class TopSites
     : public base::RefCountedThreadSafe<TopSites>,
-      public NotificationObserver,
+      public content::NotificationObserver,
       public CancelableRequestProvider {
  public:
   explicit TopSites(Profile* profile);
@@ -58,26 +63,36 @@ class TopSites
   // was updated. False means either the URL wasn't known to us, or we felt
   // that our current thumbnail was superior to the given one.
   bool SetPageThumbnail(const GURL& url,
-                        const SkBitmap& thumbnail,
+                        gfx::Image* thumbnail,
                         const ThumbnailScore& score);
 
   // Callback for GetMostVisitedURLs.
-  typedef Callback1<const MostVisitedURLList&>::Type GetTopSitesCallback;
+  typedef base::Callback<void(const MostVisitedURLList&)> GetTopSitesCallback;
   typedef std::set<scoped_refptr<CancelableRequest<GetTopSitesCallback> > >
       PendingCallbackSet;
 
   // Returns a list of most visited URLs via a callback.
   // This may be invoked on any thread.
-  // NOTE: the callback may be called immediately if we have the data cached.
+  // NOTE: the callback is called immediately if we have the data cached.
   void GetMostVisitedURLs(CancelableRequestConsumer* consumer,
-                          GetTopSitesCallback* callback);
+                          const GetTopSitesCallback& callback);
 
   // Get a thumbnail for a given page. Returns true iff we have the thumbnail.
   // This may be invoked on any thread.
   // As this method may be invoked on any thread the ref count needs to be
-  // upped before this method returns, so this takes a scoped_refptr*.
+  // incremented before this method returns, so this takes a scoped_refptr*.
   bool GetPageThumbnail(const GURL& url,
-                        scoped_refptr<RefCountedBytes>* bytes);
+                        scoped_refptr<RefCountedMemory>* bytes);
+
+  // Get a thumbnail score for a given page. Returns true iff we have the
+  // thumbnail score.  This may be invoked on any thread. The score will
+  // be copied to |score|.
+  virtual bool GetPageThumbnailScore(const GURL& url, ThumbnailScore* score);
+
+  // Get a temporary thumbnail score for a given page. Returns true iff we
+  // have the thumbnail score. Useful when checking if we should update a
+  // thumbnail for a given page. The score will be copied to |score|.
+  bool GetTemporaryPageThumbnailScore(const GURL& url, ThumbnailScore* score);
 
   // Invoked from History if migration is needed. If this is invoked it will
   // be before HistoryLoaded is invoked.
@@ -90,6 +105,10 @@ class TopSites
   // not invoked at this point then we load from the top sites service.
   void HistoryLoaded();
 
+  // Asks TopSites to refresh what it thinks the top sites are. This may do
+  // nothing.
+  void SyncWithHistory();
+
   // Blacklisted URLs
 
   // Returns true if there is at least one item in the blacklist.
@@ -144,6 +163,35 @@ class TopSites
 
   bool loaded() const { return loaded_; }
 
+  // Returns true if the given URL is known to the top sites service.
+  // This function also returns false if TopSites isn't loaded yet.
+  virtual bool IsKnownURL(const GURL& url);
+
+  // Returns true if the top sites list is full (i.e. we already have the
+  // maximum number of top sites).  This function also returns false if
+  // TopSites isn't loaded yet.
+  virtual bool IsFull();
+
+  // Returns the set of prepopulate pages.
+  static MostVisitedURLList GetPrepopulatePages();
+
+  struct PrepopulatedPage {
+    // The string resource for the url.
+    int url_id;
+    // The string resource for the page title.
+    int title_id;
+    // The raw data resource for the favicon.
+    int favicon_id;
+    // The raw data resource for the thumbnail.
+    int thumbnail_id;
+    // The best color to highlight the page (should roughly match favicon).
+    SkColor color;
+  };
+
+ protected:
+  // For allowing inheritance.
+  virtual ~TopSites();
+
  private:
   friend class base::RefCountedThreadSafe<TopSites>;
   friend class TopSitesTest;
@@ -151,7 +199,9 @@ class TopSites
   typedef std::pair<GURL, Images> TempImage;
   typedef std::list<TempImage> TempImages;
 
-  // Enumeration of the possible states history can be in.
+  // Enumeration of the possible states history can be in. These values do not
+  // necessarily reflect the loaded state of history. In particular if the
+  // history backend is unloaded |history_state_| may be HISTORY_LOADED.
   enum HistoryLoadState {
     // We're waiting for history to finish loading.
     HISTORY_LOADING,
@@ -177,8 +227,6 @@ class TopSites
     TOP_SITES_LOADED
   };
 
-  ~TopSites();
-
   // Sets the thumbnail without writing to the database. Useful when
   // reading last known top sites from the DB.
   // Returns true if the thumbnail was set, false if the existing one is better.
@@ -194,7 +242,7 @@ class TopSites
 
   // Encodes the bitmap to bytes for storage to the db. Returns true if the
   // bitmap was successfully encoded.
-  static bool EncodeBitmap(const SkBitmap& bitmap,
+  static bool EncodeBitmap(gfx::Image* bitmap,
                            scoped_refptr<RefCountedBytes>* bytes);
 
   // Removes the cached thumbnail for url. Does nothing if |url| if not cached
@@ -215,9 +263,6 @@ class TopSites
   static int GetRedirectDistanceForURL(const MostVisitedURL& most_visited,
                                        const GURL& url);
 
-  // Returns the set of prepopulate pages.
-  static MostVisitedURLList GetPrepopulatePages();
-
   // Add prepopulated pages: 'welcome to Chrome' and themes gallery to |urls|.
   // Returns true if any pages were added.
   static bool AddPrepopulatedPages(MostVisitedURLList* urls);
@@ -247,10 +292,10 @@ class TopSites
       const PendingCallbackSet& pending_callbacks,
       const MostVisitedURLList& urls);
 
-  // Implementation of NotificationObserver.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // Implementation of content::NotificationObserver.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Resets top_sites_ and updates the db (in the background). All mutations to
   // top_sites_ *must* go through this.
@@ -267,6 +312,8 @@ class TopSites
 
   void ResetThreadSafeImageCache();
 
+  void NotifyTopSitesChanged();
+
   // Stops and starts timer with a delay of |delta|.
   void RestartQueryForTopSitesTimer(base::TimeDelta delta);
 
@@ -299,7 +346,10 @@ class TopSites
   // Lock used to access |thread_safe_cache_|.
   mutable base::Lock lock_;
 
-  CancelableRequestConsumer cancelable_consumer_;
+  // Need a separate consumer for each CancelableRequestProvider we interact
+  // with (HistoryService and TopSitesBackend).
+  CancelableRequestConsumer history_consumer_;
+  CancelableRequestConsumer top_sites_consumer_;
 
   // Timer that asks history for the top sites. This is used to make sure our
   // data stays in sync with history.
@@ -308,7 +358,7 @@ class TopSites
   // The time we started |timer_| at. Only valid if |timer_| is running.
   base::TimeTicks timer_start_time_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // The number of URLs changed on the last update.
   size_t last_num_urls_changed_;
@@ -331,12 +381,12 @@ class TopSites
   // storing all URLs, but filtering on access. It is a dictionary,
   // key is the URL, value is a dummy value. This is owned by the
   // PrefService.
-  DictionaryValue* blacklist_;
+  const base::DictionaryValue* blacklist_;
 
   // This is a dictionary for the pinned URLs for the the most visited part of
   // the new tab page. Key is the URL, value is index where it is pinned at (may
   // be the same as key). This is owned by the PrefService.
-  DictionaryValue* pinned_urls_;
+  const base::DictionaryValue* pinned_urls_;
 
   // See description above HistoryLoadState.
   HistoryLoadState history_state_;
@@ -350,6 +400,8 @@ class TopSites
   DISALLOW_COPY_AND_ASSIGN(TopSites);
 };
 
+extern const TopSites::PrepopulatedPage kPrepopulatedPages[2];
+
 }  // namespace history
 
 #endif  // CHROME_BROWSER_HISTORY_TOP_SITES_H_
