@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,16 +9,15 @@
 #include <map>
 #include <string>
 
-#include "app/sql/meta_table.h"
-#include "base/ref_counted.h"
+#include "base/gtest_prod_util.h"
+#include "base/memory/ref_counted.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/history/url_database.h"  // For DBCloseScoper.
+#include "sql/meta_table.h"
 
 class FilePath;
-class RefCountedMemory;
-class SkBitmap;
 
-namespace app {
+namespace sql {
 class Connection;
 }
 
@@ -66,10 +65,16 @@ class TopSitesDatabase {
   bool RemoveURL(const MostVisitedURL& url);
 
  private:
+  FRIEND_TEST_ALL_PREFIXES(TopSitesDatabaseTest, UpgradeToVersion2);
+
   // Creates the thumbnail table, returning true if the table already exists
   // or was successfully created.
   bool InitThumbnailTable();
 
+  // Upgrades the thumbnail table to version 2, returning true if the
+  // upgrade was successful.
+  bool UpgradeToVersion2();
+
   // Adds a new URL to the database.
   void AddPageThumbnail(const MostVisitedURL& url,
                         int new_rank,
@@ -79,7 +84,8 @@ class TopSitesDatabase {
   void UpdatePageRankNoTransaction(const MostVisitedURL& url, int new_rank);
 
   // Updates thumbnail of a URL that's already in the database.
-  void UpdatePageThumbnail(const MostVisitedURL& url,
+  // Returns true if the database query succeeds.
+  bool UpdatePageThumbnail(const MostVisitedURL& url,
                            const Images& thumbnail);
 
   // Returns the URL's current rank or -1 if it is not present.
