@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,10 @@
 #define CHROME_BROWSER_HISTORY_URL_DATABASE_H_
 #pragma once
 
-#include "app/sql/statement.h"
 #include "base/basictypes.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/search_engines/template_url_id.h"
+#include "sql/statement.h"
 
 class GURL;
 
@@ -111,41 +111,73 @@ class URLDatabase {
 
   // Enumeration ---------------------------------------------------------------
 
-  // A basic enumerator to enumerate urls
-  class URLEnumerator {
+  // A basic enumerator to enumerate urls database.
+  class URLEnumeratorBase {
    public:
-    URLEnumerator() : initialized_(false) {
-    }
-
-    // Retreives the next url. Returns false if no more urls are available
-    bool GetNextURL(history::URLRow* r);
+    URLEnumeratorBase();
+    virtual ~URLEnumeratorBase();
 
    private:
     friend class URLDatabase;
 
     bool initialized_;
     sql::Statement statement_;
+
+    DISALLOW_COPY_AND_ASSIGN(URLEnumeratorBase);
   };
 
-  // Initializes the given enumerator to enumerator all URLs in the database
+  // A basic enumerator to enumerate urls
+  class URLEnumerator : public URLEnumeratorBase {
+   public:
+    URLEnumerator();
+
+    // Retreives the next url. Returns false if no more urls are available
+    bool GetNextURL(history::URLRow* r);
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(URLEnumerator);
+  };
+
+  // A basic enumerator to enumerate icon mapping, it is only used for icon
+  // mapping migration.
+  class IconMappingEnumerator : public URLEnumeratorBase {
+   public:
+    IconMappingEnumerator();
+
+    // Retreives the next url. Returns false if no more urls are available
+    bool GetNextIconMapping(IconMapping* r);
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(IconMappingEnumerator);
+  };
+
+  // Initializes the given enumerator to enumerator all URLs in the database.
   bool InitURLEnumeratorForEverything(URLEnumerator* enumerator);
 
+  // Initializes the given enumerator to enumerator all URLs in the database
+  // that are historically significant: ones having been visited within 3 days,
+  // having their URL manually typed more than once, or having been visited
+  // more than 3 times.
+  bool InitURLEnumeratorForSignificant(URLEnumerator* enumerator);
+
   // Favicons ------------------------------------------------------------------
 
-  // Check whether a favicon is used by any URLs in the database.
-  bool IsFavIconUsed(FavIconID favicon_id);
-
   // Autocomplete --------------------------------------------------------------
 
-  // Fills the given array with URLs matching the given prefix. They will be
+  // Fills the given array with URLs matching the given prefix.  They will be
   // sorted by typed count, then by visit count, then by visit date (most recent
   // first) up to the given maximum number.  If |typed_only| is true, only urls
-  // that have been typed once are returned. Called by HistoryURLProvider.
-  void AutocompleteForPrefix(const string16& prefix,
+  // that have been typed once are returned.  For caller convenience, returns
+  // whether any results were found.
+  bool AutocompleteForPrefix(const std::string& prefix,
                              size_t max_results,
                              bool typed_only,
                              std::vector<URLRow>* results);
 
+  // Returns true if the database holds some past typed navigation to a URL on
+  // the provided hostname.
+  bool IsTypedHost(const std::string& host);
+
   // Tries to find the shortest URL beginning with |base| that strictly
   // prefixes |url|, and has minimum visit_ and typed_counts as specified.
   // If found, fills in |info| and returns true; otherwise returns false,
@@ -188,6 +220,11 @@ class URLDatabase {
   // the favicon couldn't be updated.
   bool MigrateFromVersion11ToVersion12();
 
+  // Initializes the given enumerator to enumerator all URL and icon mappings
+  // in the database. Only used for icon mapping migration.
+  bool InitIconMappingEnumeratorForEverything(
+      IconMappingEnumerator* enumerator);
+
  protected:
   friend class VisitDatabase;
 
@@ -214,18 +251,14 @@ class URLDatabase {
   // different name but the same schema.
   bool CreateURLTable(bool is_temporary);
   // We have two tiers of indices for the URL table. The main tier is used by
-  // all URL databases, and is an index over the URL itself. The main history
-  // DB also creates indices over the favicons and bookmark IDs. The archived
-  // and in-memory databases don't need these supplimentary indices so we can
-  // save space by not creating them.
-  void CreateMainURLIndex();
-  void CreateSupplimentaryURLIndices();
+  // all URL databases, and is an index over the URL itself.
+  bool CreateMainURLIndex();
 
   // Ensures the keyword search terms table exists.
   bool InitKeywordSearchTermsTable();
 
   // Creates the indices used for keyword search terms.
-  void CreateKeywordSearchTermsIndices();
+  bool CreateKeywordSearchTermsIndices();
 
   // Deletes the keyword search terms table.
   bool DropKeywordSearchTermsTable();
@@ -261,7 +294,7 @@ class URLDatabase {
 // string dynamically anyway, use the constant, it will save space.
 #define HISTORY_URL_ROW_FIELDS \
     " urls.id, urls.url, urls.title, urls.visit_count, urls.typed_count, " \
-    "urls.last_visit_time, urls.hidden, urls.favicon_id "
+    "urls.last_visit_time, urls.hidden "
 
 }  // history
 
