@@ -1,14 +1,15 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "app/sql/connection.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
 #include "base/path_service.h"
+#include "base/scoped_temp_dir.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/history/url_database.h"
+#include "sql/connection.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using base::Time;
@@ -38,33 +39,31 @@ class URLDatabaseTest : public testing::Test,
   URLDatabaseTest() {
   }
 
- private:
-  // Test setup.
-  void SetUp() {
-    FilePath temp_dir;
-    PathService::Get(base::DIR_TEMP, &temp_dir);
-    db_file_ = temp_dir.AppendASCII("URLTest.db");
-
-    EXPECT_TRUE(db_.Open(db_file_));
-
-    // Initialize the tables for this test.
-    CreateURLTable(false);
-    CreateMainURLIndex();
-    CreateSupplimentaryURLIndices();
-    InitKeywordSearchTermsTable();
-    CreateKeywordSearchTermsIndices();
-  }
-  void TearDown() {
-    db_.Close();
-    file_util::Delete(db_file_, false);
-  }
-
+ protected:
   // Provided for URL/VisitDatabase.
   virtual sql::Connection& GetDB() {
     return db_;
   }
 
-  FilePath db_file_;
+ private:
+  // Test setup.
+  void SetUp() {
+    ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
+    FilePath db_file = temp_dir_.path().AppendASCII("URLTest.db");
+
+    EXPECT_TRUE(db_.Open(db_file));
+
+    // Initialize the tables for this test.
+    CreateURLTable(false);
+    CreateMainURLIndex();
+    InitKeywordSearchTermsTable();
+    CreateKeywordSearchTermsIndices();
+  }
+  void TearDown() {
+    db_.Close();
+  }
+
+  ScopedTempDir temp_dir_;
   sql::Connection db_;
 };
 
@@ -125,8 +124,7 @@ TEST_F(URLDatabaseTest, AddURL) {
 
 // Tests adding, querying and deleting keyword visits.
 TEST_F(URLDatabaseTest, KeywordSearchTermVisit) {
-  const GURL url1("http://www.google.com/");
-  URLRow url_info1(url1);
+  URLRow url_info1(GURL("http://www.google.com/"));
   url_info1.set_title(UTF8ToUTF16("Google"));
   url_info1.set_visit_count(4);
   url_info1.set_typed_count(2);
@@ -165,8 +163,7 @@ TEST_F(URLDatabaseTest, KeywordSearchTermVisit) {
 
 // Make sure deleting a URL also deletes a keyword visit.
 TEST_F(URLDatabaseTest, DeleteURLDeletesKeywordSearchTermVisit) {
-  const GURL url1("http://www.google.com/");
-  URLRow url_info1(url1);
+  URLRow url_info1(GURL("http://www.google.com/"));
   url_info1.set_title(UTF8ToUTF16("Google"));
   url_info1.set_visit_count(4);
   url_info1.set_typed_count(2);
@@ -187,4 +184,87 @@ TEST_F(URLDatabaseTest, DeleteURLDeletesKeywordSearchTermVisit) {
   ASSERT_EQ(0U, matches.size());
 }
 
+TEST_F(URLDatabaseTest, EnumeratorForSignificant) {
+  std::set<std::string> good_urls;
+  // Add URLs which do and don't meet the criteria.
+  URLRow url_no_match(GURL("http://www.url_no_match.com/"));
+  EXPECT_TRUE(AddURL(url_no_match));
+
+  std::string url_string2("http://www.url_match_visit_count.com/");
+  good_urls.insert("http://www.url_match_visit_count.com/");
+  URLRow url_match_visit_count(GURL("http://www.url_match_visit_count.com/"));
+  url_match_visit_count.set_visit_count(kLowQualityMatchVisitLimit);
+  EXPECT_TRUE(AddURL(url_match_visit_count));
+
+  good_urls.insert("http://www.url_match_typed_count.com/");
+  URLRow url_match_typed_count(GURL("http://www.url_match_typed_count.com/"));
+  url_match_typed_count.set_typed_count(kLowQualityMatchTypedLimit);
+  EXPECT_TRUE(AddURL(url_match_typed_count));
+
+  good_urls.insert("http://www.url_match_last_visit.com/");
+  URLRow url_match_last_visit(GURL("http://www.url_match_last_visit.com/"));
+  url_match_last_visit.set_last_visit(Time::Now() - TimeDelta::FromDays(1));
+  EXPECT_TRUE(AddURL(url_match_last_visit));
+
+  URLRow url_no_match_last_visit(GURL(
+      "http://www.url_no_match_last_visit.com/"));
+  url_no_match_last_visit.set_last_visit(Time::Now() -
+      TimeDelta::FromDays(kLowQualityMatchAgeLimitInDays + 1));
+  EXPECT_TRUE(AddURL(url_no_match_last_visit));
+
+  URLDatabase::URLEnumerator history_enum;
+  EXPECT_TRUE(InitURLEnumeratorForSignificant(&history_enum));
+  URLRow row;
+  int row_count = 0;
+  for (; history_enum.GetNextURL(&row); ++row_count)
+    EXPECT_EQ(1U, good_urls.count(row.url().spec()));
+  EXPECT_EQ(3, row_count);
+}
+
+TEST_F(URLDatabaseTest, IconMappingEnumerator) {
+  const GURL url1("http://www.google.com/");
+  URLRow url_info1(url1);
+  url_info1.set_title(UTF8ToUTF16("Google"));
+  url_info1.set_visit_count(4);
+  url_info1.set_typed_count(2);
+  url_info1.set_last_visit(Time::Now() - TimeDelta::FromDays(1));
+  url_info1.set_hidden(false);
+
+  // Insert a row with favicon
+  URLID url_id1 = AddURL(url_info1);
+  ASSERT_TRUE(url_id1 != 0);
+
+  FaviconID icon_id = 1;
+  sql::Statement statement(GetDB().GetCachedStatement(
+      SQL_FROM_HERE,
+      "UPDATE urls SET favicon_id =? WHERE id=?"));
+
+  ASSERT_TRUE(statement.is_valid());
+
+  statement.BindInt64(0, icon_id);
+  statement.BindInt64(1, url_id1);
+  ASSERT_TRUE(statement.Run());
+
+  // Insert another row without favicon
+  const GURL url2("http://www.google.com/no_icon");
+  URLRow url_info2(url2);
+  url_info2.set_title(UTF8ToUTF16("Google"));
+  url_info2.set_visit_count(4);
+  url_info2.set_typed_count(2);
+  url_info2.set_last_visit(Time::Now() - TimeDelta::FromDays(1));
+  url_info2.set_hidden(false);
+
+  // Insert a row with favicon
+  URLID url_id2 = AddURL(url_info2);
+  ASSERT_TRUE(url_id2 != 0);
+
+  IconMappingEnumerator e;
+  InitIconMappingEnumeratorForEverything(&e);
+  IconMapping icon_mapping;
+  ASSERT_TRUE(e.GetNextIconMapping(&icon_mapping));
+  ASSERT_EQ(url1, icon_mapping.page_url);
+  ASSERT_EQ(icon_id, icon_mapping.icon_id);
+  ASSERT_FALSE(e.GetNextIconMapping(&icon_mapping));
+}
+
 }  // namespace history
