@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,12 +9,12 @@
 #include <map>
 #include <set>
 
-#include "app/sql/statement.h"
 #include "base/logging.h"
 #include "base/string_number_conversions.h"
 #include "chrome/browser/history/url_database.h"
-#include "chrome/common/page_transition_types.h"
 #include "chrome/common/url_constants.h"
+#include "content/public/common/page_transition_types.h"
+#include "sql/statement.h"
 
 // Rows, in order, of the visit table.
 #define HISTORY_VISIT_ROW_FIELDS \
@@ -62,26 +62,33 @@ bool VisitDatabase::InitVisitTable() {
         return false;
   }
 
-  // Index over url so we can quickly find visits for a page. This will just
-  // fail if it already exists and we'll ignore it.
-  GetDB().Execute("CREATE INDEX visits_url_index ON visits (url)");
+  // Index over url so we can quickly find visits for a page.
+  if (!GetDB().Execute(
+          "CREATE INDEX IF NOT EXISTS visits_url_index ON visits (url)"))
+    return false;
 
   // Create an index over from visits so that we can efficiently find
-  // referrers and redirects. Ignore failures because it likely already exists.
-  GetDB().Execute("CREATE INDEX visits_from_index ON visits (from_visit)");
+  // referrers and redirects.
+  if (!GetDB().Execute(
+          "CREATE INDEX IF NOT EXISTS visits_from_index ON "
+          "visits (from_visit)"))
+    return false;
 
   // Create an index over time so that we can efficiently find the visits in a
-  // given time range (most history views are time-based). Ignore failures
-  // because it likely already exists.
-  GetDB().Execute("CREATE INDEX visits_time_index ON visits (visit_time)");
+  // given time range (most history views are time-based).
+  if (!GetDB().Execute(
+          "CREATE INDEX IF NOT EXISTS visits_time_index ON "
+          "visits (visit_time)"))
+    return false;
 
   return true;
 }
 
 bool VisitDatabase::DropVisitTable() {
-  GetDB().Execute("DROP TABLE visit_source");
   // This will also drop the indices over the table.
-  return GetDB().Execute("DROP TABLE visits");
+  return
+      GetDB().Execute("DROP TABLE IF EXISTS visit_source") &&
+      GetDB().Execute("DROP TABLE visits");
 }
 
 // Must be in sync with HISTORY_VISIT_ROW_FIELDS.
@@ -91,19 +98,21 @@ void VisitDatabase::FillVisitRow(sql::Statement& statement, VisitRow* visit) {
   visit->url_id = statement.ColumnInt64(1);
   visit->visit_time = base::Time::FromInternalValue(statement.ColumnInt64(2));
   visit->referring_visit = statement.ColumnInt64(3);
-  visit->transition = PageTransition::FromInt(statement.ColumnInt(4));
+  visit->transition = content::PageTransitionFromInt(statement.ColumnInt(4));
   visit->segment_id = statement.ColumnInt64(5);
   visit->is_indexed = !!statement.ColumnInt(6);
 }
 
 // static
-void VisitDatabase::FillVisitVector(sql::Statement& statement,
+bool VisitDatabase::FillVisitVector(sql::Statement& statement,
                                     VisitVector* visits) {
   while (statement.Step()) {
     history::VisitRow visit;
     FillVisitRow(statement, &visit);
     visits->push_back(visit);
   }
+
+  return statement.Succeeded();
 }
 
 VisitID VisitDatabase::AddVisit(VisitRow* visit, VisitSource source) {
@@ -111,17 +120,18 @@ VisitID VisitDatabase::AddVisit(VisitRow* visit, VisitSource source) {
       "INSERT INTO visits "
       "(url, visit_time, from_visit, transition, segment_id, is_indexed) "
       "VALUES (?,?,?,?,?,?)"));
-  if (!statement)
-    return 0;
-
   statement.BindInt64(0, visit->url_id);
   statement.BindInt64(1, visit->visit_time.ToInternalValue());
   statement.BindInt64(2, visit->referring_visit);
   statement.BindInt64(3, visit->transition);
   statement.BindInt64(4, visit->segment_id);
   statement.BindInt64(5, visit->is_indexed);
-  if (!statement.Run())
+
+  if (!statement.Run()) {
+    VLOG(0) << "Failed to execute visit insert statement:  "
+            << "url_id = " << visit->url_id;
     return 0;
+  }
 
   visit->visit_id = GetDB().GetLastInsertRowId();
 
@@ -129,13 +139,14 @@ VisitID VisitDatabase::AddVisit(VisitRow* visit, VisitSource source) {
     // Record the source of this visit when it is not browsed.
     sql::Statement statement1(GetDB().GetCachedStatement(SQL_FROM_HERE,
         "INSERT INTO visit_source (id, source) VALUES (?,?)"));
-    if (!statement1.is_valid())
-      return 0;
-
     statement1.BindInt64(0, visit->visit_id);
     statement1.BindInt64(1, source);
-    if (!statement1.Run())
+
+    if (!statement1.Run()) {
+      VLOG(0) << "Failed to execute visit_source insert statement:  "
+              << "url_id = " << visit->visit_id;
       return 0;
+    }
   }
 
   return visit->visit_id;
@@ -146,27 +157,23 @@ void VisitDatabase::DeleteVisit(const VisitRow& visit) {
   // "source" be the deleted visit's source.
   sql::Statement update_chain(GetDB().GetCachedStatement(SQL_FROM_HERE,
       "UPDATE visits SET from_visit=? WHERE from_visit=?"));
-  if (!update_chain)
-    return;
   update_chain.BindInt64(0, visit.referring_visit);
   update_chain.BindInt64(1, visit.visit_id);
-  update_chain.Run();
+  if (!update_chain.Run())
+    return;
 
   // Now delete the actual visit.
   sql::Statement del(GetDB().GetCachedStatement(SQL_FROM_HERE,
       "DELETE FROM visits WHERE id=?"));
-  if (!del)
-    return;
   del.BindInt64(0, visit.visit_id);
-  del.Run();
+  if (!del.Run())
+    return;
 
   // Try to delete the entry in visit_source table as well.
   // If the visit was browsed, there is no corresponding entry in visit_source
   // table, and nothing will be deleted.
   del.Assign(GetDB().GetCachedStatement(SQL_FROM_HERE,
              "DELETE FROM visit_source WHERE id=?"));
-  if (!del.is_valid())
-    return;
   del.BindInt64(0, visit.visit_id);
   del.Run();
 }
@@ -174,10 +181,8 @@ void VisitDatabase::DeleteVisit(const VisitRow& visit) {
 bool VisitDatabase::GetRowForVisit(VisitID visit_id, VisitRow* out_visit) {
   sql::Statement statement(GetDB().GetCachedStatement(SQL_FROM_HERE,
       "SELECT" HISTORY_VISIT_ROW_FIELDS "FROM visits WHERE id=?"));
-  if (!statement)
-    return false;
-
   statement.BindInt64(0, visit_id);
+
   if (!statement.Step())
     return false;
 
@@ -201,9 +206,6 @@ bool VisitDatabase::UpdateVisitRow(const VisitRow& visit) {
       "UPDATE visits SET "
       "url=?,visit_time=?,from_visit=?,transition=?,segment_id=?,is_indexed=? "
       "WHERE id=?"));
-  if (!statement)
-    return false;
-
   statement.BindInt64(0, visit.url_id);
   statement.BindInt64(1, visit.visit_time.ToInternalValue());
   statement.BindInt64(2, visit.referring_visit);
@@ -211,6 +213,7 @@ bool VisitDatabase::UpdateVisitRow(const VisitRow& visit) {
   statement.BindInt64(4, visit.segment_id);
   statement.BindInt64(5, visit.is_indexed);
   statement.BindInt64(6, visit.visit_id);
+
   return statement.Run();
 }
 
@@ -222,15 +225,22 @@ bool VisitDatabase::GetVisitsForURL(URLID url_id, VisitVector* visits) {
       "FROM visits "
       "WHERE url=? "
       "ORDER BY visit_time ASC"));
-  if (!statement)
-    return false;
-
   statement.BindInt64(0, url_id);
-  FillVisitVector(statement, visits);
-  return true;
+  return FillVisitVector(statement, visits);
 }
 
-void VisitDatabase::GetAllVisitsInRange(base::Time begin_time,
+bool VisitDatabase::GetIndexedVisitsForURL(URLID url_id, VisitVector* visits) {
+  visits->clear();
+
+  sql::Statement statement(GetDB().GetCachedStatement(SQL_FROM_HERE,
+      "SELECT" HISTORY_VISIT_ROW_FIELDS
+      "FROM visits "
+      "WHERE url=? AND is_indexed=1"));
+  statement.BindInt64(0, url_id);
+  return FillVisitVector(statement, visits);
+}
+
+bool VisitDatabase::GetAllVisitsInRange(base::Time begin_time,
                                         base::Time end_time,
                                         int max_results,
                                         VisitVector* visits) {
@@ -240,8 +250,6 @@ void VisitDatabase::GetAllVisitsInRange(base::Time begin_time,
       "SELECT" HISTORY_VISIT_ROW_FIELDS "FROM visits "
       "WHERE visit_time >= ? AND visit_time < ?"
       "ORDER BY visit_time LIMIT ?"));
-  if (!statement)
-    return;
 
   // See GetVisibleVisitsInRange for more info on how these times are bound.
   int64 end = end_time.ToInternalValue();
@@ -250,14 +258,14 @@ void VisitDatabase::GetAllVisitsInRange(base::Time begin_time,
   statement.BindInt64(2,
       max_results ? max_results : std::numeric_limits<int64>::max());
 
-  FillVisitVector(statement, visits);
+  return FillVisitVector(statement, visits);
 }
 
-void VisitDatabase::GetVisitsInRangeForTransition(
+bool VisitDatabase::GetVisitsInRangeForTransition(
     base::Time begin_time,
     base::Time end_time,
     int max_results,
-    PageTransition::Type transition,
+    content::PageTransition transition,
     VisitVector* visits) {
   DCHECK(visits);
   visits->clear();
@@ -267,19 +275,17 @@ void VisitDatabase::GetVisitsInRangeForTransition(
       "WHERE visit_time >= ? AND visit_time < ? "
       "AND (transition & ?) == ?"
       "ORDER BY visit_time LIMIT ?"));
-  if (!statement)
-    return;
 
   // See GetVisibleVisitsInRange for more info on how these times are bound.
   int64 end = end_time.ToInternalValue();
   statement.BindInt64(0, begin_time.ToInternalValue());
   statement.BindInt64(1, end ? end : std::numeric_limits<int64>::max());
-  statement.BindInt(2, PageTransition::CORE_MASK);
+  statement.BindInt(2, content::PAGE_TRANSITION_CORE_MASK);
   statement.BindInt(3, transition);
   statement.BindInt64(4,
       max_results ? max_results : std::numeric_limits<int64>::max());
 
-  FillVisitVector(statement, visits);
+  return FillVisitVector(statement, visits);
 }
 
 void VisitDatabase::GetVisibleVisitsInRange(base::Time begin_time,
@@ -296,8 +302,6 @@ void VisitDatabase::GetVisibleVisitsInRange(base::Time begin_time,
       "AND (transition & ?) NOT IN (?, ?, ?) "  // NO SUBFRAME or
                                                 // KEYWORD_GENERATED
       "ORDER BY visit_time DESC, id DESC"));
-  if (!statement)
-    return;
 
   // Note that we use min/max values for querying unlimited ranges of time using
   // the same statement. Since the time has an index, this will be about the
@@ -305,11 +309,11 @@ void VisitDatabase::GetVisibleVisitsInRange(base::Time begin_time,
   int64 end = end_time.ToInternalValue();
   statement.BindInt64(0, begin_time.ToInternalValue());
   statement.BindInt64(1, end ? end : std::numeric_limits<int64>::max());
-  statement.BindInt(2, PageTransition::CHAIN_END);
-  statement.BindInt(3, PageTransition::CORE_MASK);
-  statement.BindInt(4, PageTransition::AUTO_SUBFRAME);
-  statement.BindInt(5, PageTransition::MANUAL_SUBFRAME);
-  statement.BindInt(6, PageTransition::KEYWORD_GENERATED);
+  statement.BindInt(2, content::PAGE_TRANSITION_CHAIN_END);
+  statement.BindInt(3, content::PAGE_TRANSITION_CORE_MASK);
+  statement.BindInt(4, content::PAGE_TRANSITION_AUTO_SUBFRAME);
+  statement.BindInt(5, content::PAGE_TRANSITION_MANUAL_SUBFRAME);
+  statement.BindInt(6, content::PAGE_TRANSITION_KEYWORD_GENERATED);
 
   std::set<URLID> found_urls;
   while (statement.Step()) {
@@ -335,9 +339,6 @@ VisitID VisitDatabase::GetMostRecentVisitForURL(URLID url_id,
       "WHERE url=? "
       "ORDER BY visit_time DESC, id DESC "
       "LIMIT 1"));
-  if (!statement)
-    return 0;
-
   statement.BindInt64(0, url_id);
   if (!statement.Step())
     return 0;  // No visits for this URL.
@@ -362,13 +363,10 @@ bool VisitDatabase::GetMostRecentVisitsForURL(URLID url_id,
       "WHERE url=? "
       "ORDER BY visit_time DESC, id DESC "
       "LIMIT ?"));
-  if (!statement)
-    return false;
-
   statement.BindInt64(0, url_id);
   statement.BindInt(1, max_results);
-  FillVisitVector(statement, visits);
-  return true;
+
+  return FillVisitVector(statement, visits);
 }
 
 bool VisitDatabase::GetRedirectFromVisit(VisitID from_visit,
@@ -379,14 +377,11 @@ bool VisitDatabase::GetRedirectFromVisit(VisitID from_visit,
       "FROM visits v JOIN urls u ON v.url = u.id "
       "WHERE v.from_visit = ? "
       "AND (v.transition & ?) != 0"));  // IS_REDIRECT_MASK
-  if (!statement)
-    return false;
-
   statement.BindInt64(0, from_visit);
-  statement.BindInt(1, PageTransition::IS_REDIRECT_MASK);
+  statement.BindInt(1, content::PAGE_TRANSITION_IS_REDIRECT_MASK);
 
   if (!statement.Step())
-    return false;  // No redirect from this visit.
+    return false;  // No redirect from this visit. (Or SQL error)
   if (to_visit)
     *to_visit = statement.ColumnInt64(0);
   if (to_url)
@@ -419,9 +414,9 @@ bool VisitDatabase::GetRedirectToVisit(VisitID to_visit,
   return true;
 }
 
-bool VisitDatabase::GetVisitCountToHost(const GURL& url,
-                                        int* count,
-                                        base::Time* first_visit) {
+bool VisitDatabase::GetVisibleVisitCountToHost(const GURL& url,
+                                               int* count,
+                                               base::Time* first_visit) {
   if (!url.SchemeIs(chrome::kHttpScheme) && !url.SchemeIs(chrome::kHttpsScheme))
     return false;
 
@@ -432,24 +427,26 @@ bool VisitDatabase::GetVisitCountToHost(const GURL& url,
   // The query becomes:
   // 'url >= http://google.com/' and url < http://google.com0'.
   // 0 is used as it is one character greater than '/'.
-  GURL search_url(url);
-  const std::string host_query_min = search_url.GetOrigin().spec();
-
+  const std::string host_query_min = url.GetOrigin().spec();
   if (host_query_min.empty())
     return false;
 
-  std::string host_query_max = host_query_min;
-  host_query_max[host_query_max.size() - 1] = '0';
-
+  // We also want to restrict ourselves to main frame navigations that are not
+  // in the middle of redirect chains, hence the transition checks.
   sql::Statement statement(GetDB().GetCachedStatement(SQL_FROM_HERE,
       "SELECT MIN(v.visit_time), COUNT(*) "
       "FROM visits v INNER JOIN urls u ON v.url = u.id "
-      "WHERE (u.url >= ? AND u.url < ?)"));
-  if (!statement)
-    return false;
-
+      "WHERE u.url >= ? AND u.url < ? "
+      "AND (transition & ?) != 0 "
+      "AND (transition & ?) NOT IN (?, ?, ?)"));
   statement.BindString(0, host_query_min);
-  statement.BindString(1, host_query_max);
+  statement.BindString(1,
+      host_query_min.substr(0, host_query_min.size() - 1) + '0');
+  statement.BindInt(2, content::PAGE_TRANSITION_CHAIN_END);
+  statement.BindInt(3, content::PAGE_TRANSITION_CORE_MASK);
+  statement.BindInt(4, content::PAGE_TRANSITION_AUTO_SUBFRAME);
+  statement.BindInt(5, content::PAGE_TRANSITION_MANUAL_SUBFRAME);
+  statement.BindInt(6, content::PAGE_TRANSITION_KEYWORD_GENERATED);
 
   if (!statement.Step()) {
     // We've never been to this page before.
@@ -457,6 +454,9 @@ bool VisitDatabase::GetVisitCountToHost(const GURL& url,
     return true;
   }
 
+  if (!statement.Succeeded())
+    return false;
+
   *first_visit = base::Time::FromInternalValue(statement.ColumnInt64(0));
   *count = statement.ColumnInt(1);
   return true;
@@ -465,7 +465,7 @@ bool VisitDatabase::GetVisitCountToHost(const GURL& url,
 bool VisitDatabase::GetStartDate(base::Time* first_visit) {
   sql::Statement statement(GetDB().GetCachedStatement(SQL_FROM_HERE,
       "SELECT MIN(visit_time) FROM visits WHERE visit_time != 0"));
-  if (!statement || !statement.Step() || statement.ColumnInt64(0) == 0) {
+  if (!statement.Step() || statement.ColumnInt64(0) == 0) {
     *first_visit = base::Time::Now();
     return false;
   }
@@ -499,8 +499,6 @@ void VisitDatabase::GetVisitsSource(const VisitVector& visits,
     }
     sql.append(") ORDER BY id");
     sql::Statement statement(GetDB().GetUniqueStatement(sql.c_str()));
-    if (!statement)
-      return;
 
     // Get the source entries out of the query result.
     while (statement.Step()) {
