@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,9 +11,10 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop_proxy.h"
-#include "base/ref_counted.h"
-#include "gfx/scoped_image.h"
+#include "ui/gfx/image/image.h"
 
 #if defined(TOOLKIT_USES_GTK)
 #include "base/file_path.h"
@@ -28,9 +29,6 @@ typedef std::wstring IconGroupID;
 typedef std::string IconGroupID;
 #endif
 
-class MessageLoop;
-class SkBitmap;
-
 ////////////////////////////////////////////////////////////////////////////////
 //
 // A facility to read a file containing an icon asynchronously in the IO
@@ -42,7 +40,8 @@ class IconLoader : public base::RefCountedThreadSafe<IconLoader> {
   enum IconSize {
     SMALL = 0,  // 16x16
     NORMAL,     // 32x32
-    LARGE
+    LARGE,      // Windows: 32x32, Linux: 48x48, Mac: Unsupported
+    ALL,        // All sizes available
   };
 
   class Delegate {
@@ -50,7 +49,7 @@ class IconLoader : public base::RefCountedThreadSafe<IconLoader> {
     // Invoked when an icon has been read. |source| is the IconLoader. If the
     // icon has been successfully loaded, result is non-null. This method must
     // return true if it is taking ownership of the returned bitmap.
-    virtual bool OnBitmapLoaded(IconLoader* source, SkBitmap* result) = 0;
+    virtual bool OnImageLoaded(IconLoader* source, gfx::Image* result) = 0;
 
    protected:
     virtual ~Delegate() {}
@@ -77,19 +76,10 @@ class IconLoader : public base::RefCountedThreadSafe<IconLoader> {
 
   IconSize icon_size_;
 
-  gfx::ScopedImage<SkBitmap> bitmap_;
+  scoped_ptr<gfx::Image> image_;
 
   Delegate* delegate_;
 
-#if defined(TOOLKIT_USES_GTK)
-  // On X11 we use gdk's pixbuf loader, which has to execute on the UI
-  // thread, so we only read the file on the background thread and parse it
-  // on the main thread.
-  void ParseIcon();
-  FilePath filename_;
-  std::string icon_data_;
-#endif
-
   DISALLOW_COPY_AND_ASSIGN(IconLoader);
 };
 
