@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,11 @@
 #include <windows.h>
 #include <shellapi.h>
 
+#include "base/bind.h"
 #include "base/message_loop.h"
 #include "base/threading/thread.h"
-#include "gfx/icon_util.h"
-#include "gfx/size.h"
+#include "ui/gfx/icon_util.h"
+#include "ui/gfx/size.h"
 
 void IconLoader::ReadIcon() {
   int size = 0;
@@ -33,17 +34,9 @@ void IconLoader::ReadIcon() {
                      SHGFI_ICON | size | SHGFI_USEFILEATTRIBUTES))
     return;
 
-  ICONINFO icon_info = { 0 };
-  BITMAP bitmap_info = { 0 };
-
-  BOOL r = ::GetIconInfo(file_info.hIcon, &icon_info);
-  DCHECK(r);
-  r = ::GetObject(icon_info.hbmMask, sizeof(bitmap_info), &bitmap_info);
-  DCHECK(r);
-
-  gfx::Size icon_size(bitmap_info.bmWidth, bitmap_info.bmHeight);
-  bitmap_.Set(IconUtil::CreateSkBitmapFromHICON(file_info.hIcon, icon_size));
-
+  image_.reset(new gfx::Image(
+      IconUtil::CreateSkBitmapFromHICON(file_info.hIcon)));
+  DestroyIcon(file_info.hIcon);
   target_message_loop_->PostTask(FROM_HERE,
-      NewRunnableMethod(this, &IconLoader::NotifyDelegate));
+      base::Bind(&IconLoader::NotifyDelegate, this));
 }
