@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -48,30 +48,29 @@
 
 #include <map>
 
-#include "base/callback.h"
 #include "base/hash_tables.h"
 #include "chrome/browser/cancelable_request.h"
 #include "chrome/browser/icon_loader.h"
+#include "ui/gfx/image/image.h"
 
 class FilePath;
-class SkBitmap;
 
 class IconManager : public IconLoader::Delegate,
                     public CancelableRequestProvider {
  public:
   IconManager();
-  ~IconManager();
+  virtual ~IconManager();
 
   // Synchronous call to examine the internal caches for the icon. Returns the
   // icon if we have already loaded it, NULL if we don't have it and must load
   // it via 'LoadIcon'. The returned bitmap is owned by the IconManager and must
   // not be free'd by the caller. If the caller needs to modify the icon, it
   // must make a copy and modify the copy.
-  SkBitmap* LookupIcon(const FilePath& file_name,
-                       IconLoader::IconSize size);
+  gfx::Image* LookupIcon(const FilePath& file_name,
+                         IconLoader::IconSize size);
 
   typedef CancelableRequestProvider::Handle Handle;
-  typedef Callback2<Handle, SkBitmap*>::Type IconRequestCallback;
+  typedef base::Callback<void(Handle, gfx::Image*)> IconRequestCallback;
 
   // Asynchronous call to lookup and return the icon associated with file. The
   // work is done on the file thread, with the callbacks running on the UI
@@ -82,10 +81,10 @@ class IconManager : public IconLoader::Delegate,
   Handle LoadIcon(const FilePath& file_name,
                   IconLoader::IconSize size,
                   CancelableRequestConsumerBase* consumer,
-                  IconRequestCallback* callback);
+                  const IconRequestCallback& callback);
 
   // IconLoader::Delegate interface.
-  virtual bool OnBitmapLoaded(IconLoader* source, SkBitmap* result);
+  virtual bool OnImageLoaded(IconLoader* source, gfx::Image* result) OVERRIDE;
 
   // Get the identifying string for the given file. The implementation
   // is in icon_manager_[platform].cc.
@@ -102,7 +101,7 @@ class IconManager : public IconLoader::Delegate,
     IconLoader::IconSize size;
   };
 
-  typedef std::map<CacheKey, SkBitmap*> IconMap;
+  typedef std::map<CacheKey, gfx::Image*> IconMap;
   IconMap icon_cache_;
 
   typedef CancelableRequest<IconRequestCallback> IconRequest;
