@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,53 +12,51 @@
 #include "base/i18n/icu_string_conversions.h"
 #include "base/message_loop.h"
 #include "base/path_service.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
+#include "base/string_number_conversions.h"
 #include "base/string_split.h"
 #include "base/string_util.h"
-#include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
-#include "base/values.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/importer/firefox_importer_utils.h"
 #include "chrome/browser/importer/importer_bridge.h"
-#include "chrome/browser/importer/importer_data_types.h"
+#include "chrome/browser/importer/importer_util.h"
 #include "chrome/browser/importer/mork_reader.h"
 #include "chrome/browser/importer/nss_decryptor.h"
 #include "chrome/browser/search_engines/template_url.h"
 #include "chrome/browser/search_engines/template_url_parser.h"
 #include "chrome/common/time_format.h"
 #include "chrome/common/url_constants.h"
+#include "googleurl/src/gurl.h"
 #include "grit/generated_resources.h"
 #include "net/base/data_url.h"
-#include "webkit/glue/password_form.h"
+#include "webkit/forms/password_form.h"
 
-using base::Time;
-using importer::BOOKMARKS_HTML;
-using importer::FAVORITES;
-using importer::HISTORY;
-using importer::HOME_PAGE;
-using importer::PASSWORDS;
-using importer::ProfileInfo;
-using importer::SEARCH_ENGINES;
-using webkit_glue::PasswordForm;
-
-// Firefox2Importer.
-
-Firefox2Importer::Firefox2Importer() : parsing_bookmarks_html_file_(false) {
+namespace {
+const char kItemOpen[] = "<DT><A";
+const char kItemClose[] = "</A>";
+const char kFeedURLAttribute[] = "FEEDURL";
+const char kHrefAttribute[] = "HREF";
+const char kIconAttribute[] = "ICON";
+const char kShortcutURLAttribute[] = "SHORTCUTURL";
+const char kAddDateAttribute[] = "ADD_DATE";
+const char kPostDataAttribute[] = "POST_DATA";
 }
 
-Firefox2Importer::~Firefox2Importer() {
-}
+Firefox2Importer::Firefox2Importer() : parsing_bookmarks_html_file_(false) {}
 
-void Firefox2Importer::StartImport(const importer::ProfileInfo& profile_info,
-                                   uint16 items,
-                                   ImporterBridge* bridge) {
+Firefox2Importer::~Firefox2Importer() {}
+
+void Firefox2Importer::StartImport(
+    const importer::SourceProfile& source_profile,
+    uint16 items,
+    ImporterBridge* bridge) {
   bridge_ = bridge;
-  source_path_ = profile_info.source_path;
-  app_path_ = profile_info.app_path;
+  source_path_ = source_profile.source_path;
+  app_path_ = source_profile.app_path;
 
   parsing_bookmarks_html_file_ =
-      (profile_info.browser_type == importer::BOOKMARKS_HTML);
+      (source_profile.importer_type == importer::TYPE_BOOKMARKS_FILE);
 
   // The order here is important!
   bridge_->NotifyStarted();
@@ -117,11 +115,11 @@ void Firefox2Importer::LoadDefaultBookmarks(const FilePath& app_path,
       continue;
 
     // Get the bookmark.
-    std::wstring title;
+    string16 title;
     GURL url, favicon;
-    std::wstring shortcut;
-    Time add_date;
-    std::wstring post_data;
+    string16 shortcut;
+    base::Time add_date;
+    string16 post_data;
     if (ParseBookmarkFromLine(line, charset, &title, &url,
                               &favicon, &shortcut, &add_date,
                               &post_data))
@@ -130,8 +128,8 @@ void Firefox2Importer::LoadDefaultBookmarks(const FilePath& app_path,
 }
 
 // static
-TemplateURL* Firefox2Importer::CreateTemplateURL(const std::wstring& title,
-                                                 const std::wstring& keyword,
+TemplateURL* Firefox2Importer::CreateTemplateURL(const string16& title,
+                                                 const string16& keyword,
                                                  const GURL& url) {
   // Skip if the keyword or url is invalid.
   if (keyword.empty() && url.is_valid())
@@ -140,8 +138,8 @@ TemplateURL* Firefox2Importer::CreateTemplateURL(const std::wstring& title,
   TemplateURL* t_url = new TemplateURL();
   // We set short name by using the title if it exists.
   // Otherwise, we use the shortcut.
-  t_url->set_short_name(WideToUTF16Hack(!title.empty() ? title : keyword));
-  t_url->set_keyword(WideToUTF16Hack(keyword));
+  t_url->set_short_name(!title.empty() ? title : keyword);
+  t_url->set_keyword(keyword);
   t_url->SetURL(TemplateURLRef::DisplayURLToURLRef(UTF8ToUTF16(url.spec())),
                 0, 0);
   return t_url;
@@ -151,21 +149,21 @@ TemplateURL* Firefox2Importer::CreateTemplateURL(const std::wstring& title,
 void Firefox2Importer::ImportBookmarksFile(
     const FilePath& file_path,
     const std::set<GURL>& default_urls,
-    bool import_to_bookmark_bar,
-    const std::wstring& first_folder_name,
     Importer* importer,
     std::vector<ProfileWriter::BookmarkEntry>* bookmarks,
     std::vector<TemplateURL*>* template_urls,
-    std::vector<history::ImportedFavIconUsage>* favicons) {
+    std::vector<history::ImportedFaviconUsage>* favicons) {
   std::string content;
   file_util::ReadFileToString(file_path, &content);
   std::vector<std::string> lines;
   base::SplitString(content, '\n', &lines);
 
-  std::vector<ProfileWriter::BookmarkEntry> toolbar_bookmarks;
-  std::wstring last_folder = first_folder_name;
+  string16 last_folder;
   bool last_folder_on_toolbar = false;
-  std::vector<std::wstring> path;
+  bool last_folder_is_empty = true;
+  bool has_subfolder = false;
+  base::Time last_folder_add_date;
+  std::vector<string16> path;
   size_t toolbar_folder = 0;
   std::string charset;
   for (size_t i = 0; i < lines.size() && (!importer || !importer->cancelled());
@@ -179,23 +177,32 @@ void Firefox2Importer::ImportBookmarksFile(
 
     // Get the folder name.
     if (ParseFolderNameFromLine(line, charset, &last_folder,
-                                &last_folder_on_toolbar))
+                                &last_folder_on_toolbar,
+                                &last_folder_add_date))
       continue;
 
     // Get the bookmark entry.
-    std::wstring title, shortcut;
+    string16 title;
+    string16 shortcut;
     GURL url, favicon;
-    Time add_date;
-    std::wstring post_data;
+    base::Time add_date;
+    string16 post_data;
+    bool is_bookmark;
     // TODO(jcampan): http://b/issue?id=1196285 we do not support POST based
     //                keywords yet.
-    if (ParseBookmarkFromLine(line, charset, &title,
-                              &url, &favicon, &shortcut, &add_date,
-                              &post_data) &&
+    is_bookmark = ParseBookmarkFromLine(line, charset, &title,
+                                        &url, &favicon, &shortcut, &add_date,
+                                        &post_data) ||
+        ParseMinimumBookmarkFromLine(line, charset, &title, &url);
+
+    if (is_bookmark)
+      last_folder_is_empty = false;
+
+    if (is_bookmark &&
         post_data.empty() &&
         CanImportURL(GURL(url)) &&
         default_urls.find(url) == default_urls.end()) {
-      if (toolbar_folder > path.size() && path.size() > 0) {
+      if (toolbar_folder > path.size() && !path.empty()) {
         NOTREACHED();  // error in parsing.
         break;
       }
@@ -205,18 +212,19 @@ void Firefox2Importer::ImportBookmarksFile(
       entry.url = url;
       entry.title = title;
 
-      if (import_to_bookmark_bar && toolbar_folder) {
-        // Flatten the items in toolbar.
+      if (toolbar_folder) {
+        // The toolbar folder should be at the top level.
         entry.in_toolbar = true;
-        entry.path.assign(path.begin() + toolbar_folder, path.end());
-        toolbar_bookmarks.push_back(entry);
+        entry.path.assign(path.begin() + toolbar_folder - 1, path.end());
       } else {
-        // Insert the item into the "Imported from Firefox" folder.
+        // Add this bookmark to the list of |bookmarks|.
+        if (!has_subfolder && !last_folder.empty()) {
+          path.push_back(last_folder);
+          last_folder.clear();
+        }
         entry.path.assign(path.begin(), path.end());
-        if (import_to_bookmark_bar)
-          entry.path.erase(entry.path.begin());
-        bookmarks->push_back(entry);
       }
+      bookmarks->push_back(entry);
 
       // Save the favicon. DataURLToFaviconUsage will handle the case where
       // there is no favicon.
@@ -235,22 +243,52 @@ void Firefox2Importer::ImportBookmarksFile(
     }
 
     // Bookmarks in sub-folder are encapsulated with <DL> tag.
-    if (StartsWithASCII(line, "<DL>", true)) {
-      path.push_back(last_folder);
-      last_folder.clear();
+    if (StartsWithASCII(line, "<DL>", false)) {
+      has_subfolder = true;
+      if (!last_folder.empty()) {
+        path.push_back(last_folder);
+        last_folder.clear();
+      }
       if (last_folder_on_toolbar && !toolbar_folder)
         toolbar_folder = path.size();
-    } else if (StartsWithASCII(line, "</DL>", true)) {
+
+      // Mark next folder empty as initial state.
+      last_folder_is_empty = true;
+    } else if (StartsWithASCII(line, "</DL>", false)) {
       if (path.empty())
         break;  // Mismatch <DL>.
+
+      string16 folder_title = path.back();
       path.pop_back();
+
+      if (last_folder_is_empty) {
+        // Empty folder should be added explicitly.
+        ProfileWriter::BookmarkEntry entry;
+        entry.is_folder = true;
+        entry.creation_time = last_folder_add_date;
+        entry.title = folder_title;
+        if (toolbar_folder) {
+          // The toolbar folder should be at the top level.
+          // Make sure we don't add the toolbar folder itself if it is empty.
+          if (toolbar_folder <= path.size()) {
+            entry.in_toolbar = true;
+            entry.path.assign(path.begin() + toolbar_folder - 1, path.end());
+            bookmarks->push_back(entry);
+          }
+        } else {
+          // Add this folder to the list of |bookmarks|.
+          entry.path.assign(path.begin(), path.end());
+          bookmarks->push_back(entry);
+        }
+
+        // Parent folder include current one, so it's not empty.
+        last_folder_is_empty = false;
+      }
+
       if (toolbar_folder > path.size())
         toolbar_folder = 0;
     }
   }
-
-  bookmarks->insert(bookmarks->begin(), toolbar_bookmarks.begin(),
-                    toolbar_bookmarks.end());
 }
 
 void Firefox2Importer::ImportBookmarks() {
@@ -262,27 +300,20 @@ void Firefox2Importer::ImportBookmarks() {
   // Parse the bookmarks.html file.
   std::vector<ProfileWriter::BookmarkEntry> bookmarks, toolbar_bookmarks;
   std::vector<TemplateURL*> template_urls;
-  std::vector<history::ImportedFavIconUsage> favicons;
+  std::vector<history::ImportedFaviconUsage> favicons;
   FilePath file = source_path_;
   if (!parsing_bookmarks_html_file_)
     file = file.AppendASCII("bookmarks.html");
-  std::wstring first_folder_name;
-  first_folder_name = bridge_->GetLocalizedString(
-      parsing_bookmarks_html_file_ ? IDS_BOOKMARK_GROUP :
-                                     IDS_BOOKMARK_GROUP_FROM_FIREFOX);
 
-  ImportBookmarksFile(file, default_urls, import_to_bookmark_bar(),
-                      first_folder_name, this, &bookmarks, &template_urls,
+  ImportBookmarksFile(file, default_urls, this, &bookmarks, &template_urls,
                       &favicons);
 
   // Write data into profile.
   if (!bookmarks.empty() && !cancelled()) {
-    int options = 0;
-    if (import_to_bookmark_bar())
-      options |= ProfileWriter::IMPORT_TO_BOOKMARK_BAR;
-    if (bookmark_bar_disabled())
-      options |= ProfileWriter::BOOKMARK_BAR_DISABLED;
-    bridge_->AddBookmarkEntries(bookmarks, first_folder_name, options);
+    string16 first_folder_name = bridge_->GetLocalizedString(
+        parsing_bookmarks_html_file_ ? IDS_BOOKMARK_GROUP :
+                                       IDS_BOOKMARK_GROUP_FROM_FIREFOX);
+    bridge_->AddBookmarks(bookmarks, first_folder_name);
   }
   if (!parsing_bookmarks_html_file_ && !template_urls.empty() &&
       !cancelled()) {
@@ -290,18 +321,15 @@ void Firefox2Importer::ImportBookmarks() {
   } else {
     STLDeleteContainerPointers(template_urls.begin(), template_urls.end());
   }
-  if (!favicons.empty()) {
-    bridge_->SetFavIcons(favicons);
-  }
+  if (!favicons.empty())
+    bridge_->SetFavicons(favicons);
 }
 
 void Firefox2Importer::ImportPasswords() {
   // Initializes NSS3.
   NSSDecryptor decryptor;
-  if (!decryptor.Init(source_path_.ToWStringHack(),
-                      source_path_.ToWStringHack()) &&
-      !decryptor.Init(app_path_.ToWStringHack(),
-                      source_path_.ToWStringHack())) {
+  if (!decryptor.Init(source_path_, source_path_) &&
+      !decryptor.Init(app_path_, source_path_)) {
     return;
   }
 
@@ -314,7 +342,7 @@ void Firefox2Importer::ImportPasswords() {
 
   std::string content;
   file_util::ReadFileToString(file, &content);
-  std::vector<PasswordForm> forms;
+  std::vector<webkit::forms::PasswordForm> forms;
   decryptor.ParseSignons(content, &forms);
 
   if (!cancelled()) {
@@ -336,9 +364,8 @@ void Firefox2Importer::ImportSearchEngines() {
   std::vector<TemplateURL*> search_engines;
   ParseSearchEnginesFromXMLFiles(files, &search_engines);
 
-  int default_index =
-      GetFirefoxDefaultSearchEngineIndex(search_engines, source_path_);
-  bridge_->SetKeywords(search_engines, default_index, true);
+  // Import the list of search engines, but do not override the default.
+  bridge_->SetKeywords(search_engines, -1 /*default_keyword_index*/, true);
 }
 
 void Firefox2Importer::ImportHomepage() {
@@ -365,8 +392,9 @@ void Firefox2Importer::GetSearchEnginesXMLFiles(
 bool Firefox2Importer::ParseCharsetFromLine(const std::string& line,
                                             std::string* charset) {
   const char kCharset[] = "charset=";
-  if (StartsWithASCII(line, "<META", true) &&
-      line.find("CONTENT=\"") != std::string::npos) {
+  if (StartsWithASCII(line, "<META", false) &&
+      (line.find("CONTENT=\"") != std::string::npos ||
+          line.find("content=\"") != std::string::npos)) {
     size_t begin = line.find(kCharset);
     if (begin == std::string::npos)
       return false;
@@ -381,11 +409,13 @@ bool Firefox2Importer::ParseCharsetFromLine(const std::string& line,
 // static
 bool Firefox2Importer::ParseFolderNameFromLine(const std::string& line,
                                                const std::string& charset,
-                                               std::wstring* folder_name,
-                                               bool* is_toolbar_folder) {
+                                               string16* folder_name,
+                                               bool* is_toolbar_folder,
+                                               base::Time* add_date) {
   const char kFolderOpen[] = "<DT><H3";
   const char kFolderClose[] = "</H3>";
   const char kToolbarFolderAttribute[] = "PERSONAL_TOOLBAR_FOLDER";
+  const char kAddDateAttribute[] = "ADD_DATE";
 
   if (!StartsWithASCII(line, kFolderOpen, true))
     return false;
@@ -396,13 +426,23 @@ bool Firefox2Importer::ParseFolderNameFromLine(const std::string& line,
   if (end == std::string::npos || tag_end < arraysize(kFolderOpen))
     return false;
 
-  base::CodepageToWide(line.substr(tag_end, end - tag_end), charset.c_str(),
-                       base::OnStringConversionError::SKIP, folder_name);
+  base::CodepageToUTF16(line.substr(tag_end, end - tag_end), charset.c_str(),
+                        base::OnStringConversionError::SKIP, folder_name);
   HTMLUnescape(folder_name);
 
   std::string attribute_list = line.substr(arraysize(kFolderOpen),
       tag_end - arraysize(kFolderOpen) - 1);
   std::string value;
+
+  // Add date
+  if (GetAttribute(attribute_list, kAddDateAttribute, &value)) {
+    int64 time;
+    base::StringToInt64(value, &time);
+    // Upper bound it at 32 bits.
+    if (0 < time && time < (1LL << 32))
+      *add_date = base::Time::FromTimeT(time);
+  }
+
   if (GetAttribute(attribute_list, kToolbarFolderAttribute, &value) &&
       LowerCaseEqualsASCII(value, "true"))
     *is_toolbar_folder = true;
@@ -415,27 +455,18 @@ bool Firefox2Importer::ParseFolderNameFromLine(const std::string& line,
 // static
 bool Firefox2Importer::ParseBookmarkFromLine(const std::string& line,
                                              const std::string& charset,
-                                             std::wstring* title,
+                                             string16* title,
                                              GURL* url,
                                              GURL* favicon,
-                                             std::wstring* shortcut,
-                                             Time* add_date,
-                                             std::wstring* post_data) {
-  const char kItemOpen[] = "<DT><A";
-  const char kItemClose[] = "</A>";
-  const char kFeedURLAttribute[] = "FEEDURL";
-  const char kHrefAttribute[] = "HREF";
-  const char kIconAttribute[] = "ICON";
-  const char kShortcutURLAttribute[] = "SHORTCUTURL";
-  const char kAddDateAttribute[] = "ADD_DATE";
-  const char kPostDataAttribute[] = "POST_DATA";
-
+                                             string16* shortcut,
+                                             base::Time* add_date,
+                                             string16* post_data) {
   title->clear();
   *url = GURL();
   *favicon = GURL();
   shortcut->clear();
   post_data->clear();
-  *add_date = Time();
+  *add_date = base::Time();
 
   if (!StartsWithASCII(line, kItemOpen, true))
     return false;
@@ -456,18 +487,16 @@ bool Firefox2Importer::ParseBookmarkFromLine(const std::string& line,
     return false;
 
   // Title
-  base::CodepageToWide(line.substr(tag_end, end - tag_end), charset.c_str(),
-                       base::OnStringConversionError::SKIP, title);
+  base::CodepageToUTF16(line.substr(tag_end, end - tag_end), charset.c_str(),
+                        base::OnStringConversionError::SKIP, title);
   HTMLUnescape(title);
 
   // URL
   if (GetAttribute(attribute_list, kHrefAttribute, &value)) {
-    std::wstring w_url;
-    base::CodepageToWide(value, charset.c_str(),
-                         base::OnStringConversionError::SKIP, &w_url);
-    HTMLUnescape(&w_url);
-
-    string16 url16 = WideToUTF16Hack(w_url);
+    string16 url16;
+    base::CodepageToUTF16(value, charset.c_str(),
+                          base::OnStringConversionError::SKIP, &url16);
+    HTMLUnescape(&url16);
 
     *url = GURL(url16);
   }
@@ -478,8 +507,8 @@ bool Firefox2Importer::ParseBookmarkFromLine(const std::string& line,
 
   // Keyword
   if (GetAttribute(attribute_list, kShortcutURLAttribute, &value)) {
-    base::CodepageToWide(value, charset.c_str(),
-                         base::OnStringConversionError::SKIP, shortcut);
+    base::CodepageToUTF16(value, charset.c_str(),
+                          base::OnStringConversionError::SKIP, shortcut);
     HTMLUnescape(shortcut);
   }
 
@@ -489,19 +518,69 @@ bool Firefox2Importer::ParseBookmarkFromLine(const std::string& line,
     base::StringToInt64(value, &time);
     // Upper bound it at 32 bits.
     if (0 < time && time < (1LL << 32))
-      *add_date = Time::FromTimeT(time);
+      *add_date = base::Time::FromTimeT(time);
   }
 
   // Post data.
   if (GetAttribute(attribute_list, kPostDataAttribute, &value)) {
-    base::CodepageToWide(value, charset.c_str(),
-                         base::OnStringConversionError::SKIP, post_data);
+    base::CodepageToUTF16(value, charset.c_str(),
+                          base::OnStringConversionError::SKIP, post_data);
     HTMLUnescape(post_data);
   }
 
   return true;
 }
 
+// static
+bool Firefox2Importer::ParseMinimumBookmarkFromLine(const std::string& line,
+                                                    const std::string& charset,
+                                                    string16* title,
+                                                    GURL* url) {
+  const char kItemOpen[] = "<DT><A";
+  const char kItemClose[] = "</";
+  const char kHrefAttributeUpper[] = "HREF";
+  const char kHrefAttributeLower[] = "href";
+
+  title->clear();
+  *url = GURL();
+
+  // Case-insensitive check of open tag.
+  if (!StartsWithASCII(line, kItemOpen, false))
+    return false;
+
+  // Find any close tag.
+  size_t end = line.find(kItemClose);
+  size_t tag_end = line.rfind('>', end) + 1;
+  if (end == std::string::npos || tag_end < arraysize(kItemOpen))
+    return false;  // No end tag or start tag is broken.
+
+  std::string attribute_list = line.substr(arraysize(kItemOpen),
+      tag_end - arraysize(kItemOpen) - 1);
+
+  // Title
+  base::CodepageToUTF16(line.substr(tag_end, end - tag_end), charset.c_str(),
+                        base::OnStringConversionError::SKIP, title);
+  HTMLUnescape(title);
+
+  // URL
+  std::string value;
+  if (GetAttribute(attribute_list, kHrefAttributeUpper, &value) ||
+      GetAttribute(attribute_list, kHrefAttributeLower, &value)) {
+    if (charset.length() != 0) {
+      string16 url16;
+      base::CodepageToUTF16(value, charset.c_str(),
+                            base::OnStringConversionError::SKIP, &url16);
+      HTMLUnescape(&url16);
+
+      *url = GURL(url16);
+    } else {
+      *url = GURL(value);
+    }
+  }
+
+  return true;
+}
+
 // static
 bool Firefox2Importer::GetAttribute(const std::string& attribute_list,
                                     const std::string& attribute,
@@ -531,8 +610,8 @@ bool Firefox2Importer::GetAttribute(const std::string& attribute_list,
 }
 
 // static
-void Firefox2Importer::HTMLUnescape(std::wstring *text) {
-  string16 text16 = WideToUTF16Hack(*text);
+void Firefox2Importer::HTMLUnescape(string16* text) {
+  string16 text16 = *text;
   ReplaceSubstringsAfterOffset(
       &text16, 0, ASCIIToUTF16("&lt;"), ASCIIToUTF16("<"));
   ReplaceSubstringsAfterOffset(
@@ -543,7 +622,7 @@ void Firefox2Importer::HTMLUnescape(std::wstring *text) {
       &text16, 0, ASCIIToUTF16("&quot;"), ASCIIToUTF16("\""));
   ReplaceSubstringsAfterOffset(
       &text16, 0, ASCIIToUTF16("&#39;"), ASCIIToUTF16("\'"));
-  text->assign(UTF16ToWideHack(text16));
+  text->assign(text16);
 }
 
 // static
@@ -564,7 +643,7 @@ void Firefox2Importer::FindXMLFilesInDir(
 void Firefox2Importer::DataURLToFaviconUsage(
     const GURL& link_url,
     const GURL& favicon_data,
-    std::vector<history::ImportedFavIconUsage>* favicons) {
+    std::vector<history::ImportedFaviconUsage>* favicons) {
   if (!link_url.is_valid() || !favicon_data.is_valid() ||
       !favicon_data.SchemeIs(chrome::kDataScheme))
     return;
@@ -575,9 +654,10 @@ void Firefox2Importer::DataURLToFaviconUsage(
       data.empty())
     return;
 
-  history::ImportedFavIconUsage usage;
-  if (!ReencodeFavicon(reinterpret_cast<const unsigned char*>(&data[0]),
-                       data.size(), &usage.png_data))
+  history::ImportedFaviconUsage usage;
+  if (!importer::ReencodeFavicon(
+          reinterpret_cast<const unsigned char*>(&data[0]),
+          data.size(), &usage.png_data))
     return;  // Unable to decode.
 
   // We need to make up a URL for the favicon. We use a version of the page's
