@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,47 +7,56 @@
 #include <set>
 
 #include "base/file_util.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/scoped_ptr.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
-#include "base/values.h"
+#include "base/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/importer/firefox2_importer.h"
 #include "chrome/browser/importer/firefox_importer_utils.h"
 #include "chrome/browser/importer/importer_bridge.h"
-#include "chrome/browser/importer/importer_data_types.h"
+#include "chrome/browser/importer/importer_util.h"
 #include "chrome/browser/importer/nss_decryptor.h"
 #include "chrome/browser/search_engines/template_url.h"
 #include "chrome/common/time_format.h"
-#include "chrome/common/sqlite_utils.h"
+#include "content/public/browser/browser_thread.h"
+#include "googleurl/src/gurl.h"
 #include "grit/generated_resources.h"
-#include "webkit/glue/password_form.h"
+#include "sql/connection.h"
+#include "sql/statement.h"
+#include "webkit/forms/password_form.h"
 
-using base::Time;
-using importer::BOOKMARKS_HTML;
-using importer::FAVORITES;
-using importer::HISTORY;
-using importer::HOME_PAGE;
-using importer::PASSWORDS;
-using importer::ProfileInfo;
-using importer::SEARCH_ENGINES;
-using webkit_glue::PasswordForm;
+using content::BrowserThread;
+
+namespace {
+
+// Original definition is in http://mxr.mozilla.org/firefox/source/toolkit/
+//  components/places/public/nsINavBookmarksService.idl
+enum BookmarkItemType {
+  TYPE_BOOKMARK = 1,
+  TYPE_FOLDER = 2,
+  TYPE_SEPARATOR = 3,
+  TYPE_DYNAMIC_CONTAINER = 4
+};
+
+}  // namespace
 
 struct Firefox3Importer::BookmarkItem {
   int parent;
   int id;
   GURL url;
-  std::wstring title;
-  int type;
+  string16 title;
+  BookmarkItemType type;
   std::string keyword;
   base::Time date_added;
   int64 favicon;
+  bool empty_folder;
 };
 
 Firefox3Importer::Firefox3Importer() {
-#if defined(OS_LINUX)
+#if defined(OS_POSIX) && !defined(OS_MACOSX)
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
   locale_ = g_browser_process->GetApplicationLocale();
 #endif
@@ -56,15 +65,16 @@ Firefox3Importer::Firefox3Importer() {
 Firefox3Importer::~Firefox3Importer() {
 }
 
-void Firefox3Importer::StartImport(const importer::ProfileInfo& profile_info,
-                                   uint16 items,
-                                   ImporterBridge* bridge) {
-#if defined(OS_LINUX)
+void Firefox3Importer::StartImport(
+    const importer::SourceProfile& source_profile,
+    uint16 items,
+    ImporterBridge* bridge) {
+#if defined(OS_POSIX) && !defined(OS_MACOSX)
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::FILE));
 #endif
   bridge_ = bridge;
-  source_path_ = profile_info.source_path;
-  app_path_ = profile_info.app_path;
+  source_path_ = source_profile.source_path;
+  app_path_ = source_profile.app_path;
 
   // The order here is important!
   bridge_->NotifyStarted();
@@ -103,48 +113,45 @@ void Firefox3Importer::ImportHistory() {
   if (!file_util::PathExists(file))
     return;
 
-  sqlite3* sqlite;
-  if (sqlite3_open(WideToUTF8(file.ToWStringHack()).c_str(),
-                   &sqlite) != SQLITE_OK) {
+  sql::Connection db;
+  if (!db.Open(file))
     return;
-  }
-  sqlite_utils::scoped_sqlite_db_ptr db(sqlite);
 
-  SQLStatement s;
   // |visit_type| represent the transition type of URLs (typed, click,
   // redirect, bookmark, etc.) We eliminate some URLs like sub-frames and
   // redirects, since we don't want them to appear in history.
   // Firefox transition types are defined in:
   //   toolkit/components/places/public/nsINavHistoryService.idl
-  const char* stmt = "SELECT h.url, h.title, h.visit_count, "
-                     "h.hidden, h.typed, v.visit_date "
-                     "FROM moz_places h JOIN moz_historyvisits v "
-                     "ON h.id = v.place_id "
-                     "WHERE v.visit_type <= 3";
+  const char* query = "SELECT h.url, h.title, h.visit_count, "
+                      "h.hidden, h.typed, v.visit_date "
+                      "FROM moz_places h JOIN moz_historyvisits v "
+                      "ON h.id = v.place_id "
+                      "WHERE v.visit_type <= 3";
 
-  if (s.prepare(db.get(), stmt) != SQLITE_OK)
+  sql::Statement s(db.GetUniqueStatement(query));
+  if (!s)
     return;
 
   std::vector<history::URLRow> rows;
-  while (s.step() == SQLITE_ROW && !cancelled()) {
-    GURL url(s.column_string(0));
+  while (s.Step() && !cancelled()) {
+    GURL url(s.ColumnString(0));
 
     // Filter out unwanted URLs.
     if (!CanImportURL(url))
       continue;
 
     history::URLRow row(url);
-    row.set_title(UTF8ToUTF16(s.column_string(1)));
-    row.set_visit_count(s.column_int(2));
-    row.set_hidden(s.column_int(3) == 1);
-    row.set_typed_count(s.column_int(4));
-    row.set_last_visit(Time::FromTimeT(s.column_int64(5)/1000000));
+    row.set_title(s.ColumnString16(1));
+    row.set_visit_count(s.ColumnInt(2));
+    row.set_hidden(s.ColumnInt(3) == 1);
+    row.set_typed_count(s.ColumnInt(4));
+    row.set_last_visit(base::Time::FromTimeT(s.ColumnInt64(5)/1000000));
 
     rows.push_back(row);
   }
-  if (!rows.empty() && !cancelled()) {
+
+  if (!rows.empty() && !cancelled())
     bridge_->SetHistoryItems(rows, history::SOURCE_FIREFOX_IMPORTED);
-  }
 }
 
 void Firefox3Importer::ImportBookmarks() {
@@ -152,35 +159,31 @@ void Firefox3Importer::ImportBookmarks() {
   if (!file_util::PathExists(file))
     return;
 
-  sqlite3* sqlite;
-  if (sqlite3_open(WideToUTF8(file.ToWStringHack()).c_str(),
-                   &sqlite) != SQLITE_OK) {
+  sql::Connection db;
+  if (!db.Open(file))
     return;
-  }
-  sqlite_utils::scoped_sqlite_db_ptr db(sqlite);
 
   // Get the bookmark folders that we are interested in.
   int toolbar_folder_id = -1;
   int menu_folder_id = -1;
   int unsorted_folder_id = -1;
-  LoadRootNodeID(db.get(), &toolbar_folder_id, &menu_folder_id,
-                 &unsorted_folder_id);
+  LoadRootNodeID(&db, &toolbar_folder_id, &menu_folder_id, &unsorted_folder_id);
 
   // Load livemark IDs.
   std::set<int> livemark_id;
-  LoadLivemarkIDs(db.get(), &livemark_id);
+  LoadLivemarkIDs(&db, &livemark_id);
 
   // Load the default bookmarks. Its storage is the same as Firefox 2.
   std::set<GURL> default_urls;
   Firefox2Importer::LoadDefaultBookmarks(app_path_, &default_urls);
 
   BookmarkList list;
-  GetTopBookmarkFolder(db.get(), toolbar_folder_id, &list);
-  GetTopBookmarkFolder(db.get(), menu_folder_id, &list);
-  GetTopBookmarkFolder(db.get(), unsorted_folder_id, &list);
+  GetTopBookmarkFolder(&db, toolbar_folder_id, &list);
+  GetTopBookmarkFolder(&db, menu_folder_id, &list);
+  GetTopBookmarkFolder(&db, unsorted_folder_id, &list);
   size_t count = list.size();
   for (size_t i = 0; i < count; ++i)
-    GetWholeBookmarkFolder(db.get(), &list, i);
+    GetWholeBookmarkFolder(&db, &list, i, NULL);
 
   std::vector<ProfileWriter::BookmarkEntry> bookmarks;
   std::vector<TemplateURL*> template_urls;
@@ -189,66 +192,69 @@ void Firefox3Importer::ImportBookmarks() {
   // TODO(jcampan): http://b/issue?id=1196285 we do not support POST based
   //                keywords yet.  We won't include them in the list.
   std::set<int> post_keyword_ids;
-  SQLStatement s;
-  const char* stmt = "SELECT b.id FROM moz_bookmarks b "
+  const char* query = "SELECT b.id FROM moz_bookmarks b "
       "INNER JOIN moz_items_annos ia ON ia.item_id = b.id "
       "INNER JOIN moz_anno_attributes aa ON ia.anno_attribute_id = aa.id "
       "WHERE aa.name = 'bookmarkProperties/POSTData'";
-  if (s.prepare(db.get(), stmt) == SQLITE_OK) {
-    while (s.step() == SQLITE_ROW && !cancelled())
-      post_keyword_ids.insert(s.column_int(0));
+  sql::Statement s(db.GetUniqueStatement(query));
+  if (s) {
+    while (s.Step() && !cancelled())
+      post_keyword_ids.insert(s.ColumnInt(0));
   } else {
     NOTREACHED();
     return;
   }
 
-  std::wstring firefox_folder =
-      bridge_->GetLocalizedString(IDS_BOOKMARK_GROUP_FROM_FIREFOX);
   for (size_t i = 0; i < list.size(); ++i) {
     BookmarkItem* item = list[i];
 
-    // The type of bookmark items is 1.
-    if (item->type != 1)
+    if (item->type == TYPE_FOLDER) {
+      // Folders are added implicitly on adding children, so we only explicitly
+      // add empty folders.
+      if (!item->empty_folder)
+        continue;
+    } else if (item->type == TYPE_BOOKMARK) {
+      // Import only valid bookmarks
+      if (!CanImportURL(item->url))
+        continue;
+    } else {
       continue;
+    }
 
     // Skip the default bookmarks and unwanted URLs.
-    if (!CanImportURL(item->url) ||
-        default_urls.find(item->url) != default_urls.end() ||
+    if (default_urls.find(item->url) != default_urls.end() ||
         post_keyword_ids.find(item->id) != post_keyword_ids.end())
       continue;
 
     // Find the bookmark path by tracing their links to parent folders.
-    std::vector<std::wstring> path;
+    std::vector<string16> path;
     BookmarkItem* child = item;
     bool found_path = false;
     bool is_in_toolbar = false;
     while (child->parent >= 0) {
       BookmarkItem* parent = list[child->parent];
-      if (parent->id == toolbar_folder_id) {
-        // This bookmark entry should be put in the bookmark bar.
-        // But we put it in the Firefox group after first run, so
-        // that do not mess up the bookmark bar.
-        if (import_to_bookmark_bar()) {
-          is_in_toolbar = true;
-        } else {
-          path.insert(path.begin(), parent->title);
-          path.insert(path.begin(), firefox_folder);
-        }
-        found_path = true;
-        break;
-      } else if (parent->id == menu_folder_id ||
-                 parent->id == unsorted_folder_id) {
-        // After the first run, the item will be placed in a folder in
-        // the "Other bookmarks".
-        if (!import_to_bookmark_bar())
-          path.insert(path.begin(), firefox_folder);
-        found_path = true;
-        break;
-      } else if (livemark_id.find(parent->id) != livemark_id.end()) {
-        // If the entry is under a livemark folder, we don't import it.
+      if (livemark_id.find(parent->id) != livemark_id.end()) {
+        // Don't import live bookmarks.
         break;
       }
-      path.insert(path.begin(), parent->title);
+
+      if (parent->id != menu_folder_id) {
+        // To avoid excessive nesting, omit the name for the bookmarks menu
+        // folder.
+        path.insert(path.begin(), parent->title);
+      }
+
+      if (parent->id == toolbar_folder_id)
+        is_in_toolbar = true;
+
+      if (parent->id == toolbar_folder_id ||
+          parent->id == menu_folder_id ||
+          parent->id == unsorted_folder_id) {
+        // We've reached a root node, hooray!
+        found_path = true;
+        break;
+      }
+
       child = parent;
     }
 
@@ -261,29 +267,29 @@ void Firefox3Importer::ImportBookmarks() {
     entry.url = item->url;
     entry.path = path;
     entry.in_toolbar = is_in_toolbar;
+    entry.is_folder = item->type == TYPE_FOLDER;
 
     bookmarks.push_back(entry);
 
-    if (item->favicon)
-      favicon_map[item->favicon].insert(item->url);
+    if (item->type == TYPE_BOOKMARK) {
+      if (item->favicon)
+        favicon_map[item->favicon].insert(item->url);
 
-    // This bookmark has a keyword, we import it to our TemplateURL model.
-    TemplateURL* t_url = Firefox2Importer::CreateTemplateURL(
-        item->title, UTF8ToWide(item->keyword), item->url);
-    if (t_url)
-      template_urls.push_back(t_url);
+      // This bookmark has a keyword, we import it to our TemplateURL model.
+      TemplateURL* t_url = Firefox2Importer::CreateTemplateURL(
+          item->title, UTF8ToUTF16(item->keyword), item->url);
+      if (t_url)
+        template_urls.push_back(t_url);
+    }
   }
 
   STLDeleteContainerPointers(list.begin(), list.end());
 
   // Write into profile.
   if (!bookmarks.empty() && !cancelled()) {
-    const std::wstring& first_folder_name =
+    const string16& first_folder_name =
         bridge_->GetLocalizedString(IDS_BOOKMARK_GROUP_FROM_FIREFOX);
-    int options = 0;
-    if (import_to_bookmark_bar())
-      options = ProfileWriter::IMPORT_TO_BOOKMARK_BAR;
-    bridge_->AddBookmarkEntries(bookmarks, first_folder_name, options);
+    bridge_->AddBookmarks(bookmarks, first_folder_name);
   }
   if (!template_urls.empty() && !cancelled()) {
     bridge_->SetKeywords(template_urls, -1, false);
@@ -291,23 +297,21 @@ void Firefox3Importer::ImportBookmarks() {
     STLDeleteContainerPointers(template_urls.begin(), template_urls.end());
   }
   if (!favicon_map.empty() && !cancelled()) {
-    std::vector<history::ImportedFavIconUsage> favicons;
-    LoadFavicons(db.get(), favicon_map, &favicons);
-    bridge_->SetFavIcons(favicons);
+    std::vector<history::ImportedFaviconUsage> favicons;
+    LoadFavicons(&db, favicon_map, &favicons);
+    bridge_->SetFavicons(favicons);
   }
 }
 
 void Firefox3Importer::ImportPasswords() {
   // Initializes NSS3.
   NSSDecryptor decryptor;
-  if (!decryptor.Init(source_path_.ToWStringHack(),
-                      source_path_.ToWStringHack()) &&
-      !decryptor.Init(app_path_.ToWStringHack(),
-                      source_path_.ToWStringHack())) {
+  if (!decryptor.Init(source_path_, source_path_) &&
+      !decryptor.Init(app_path_, source_path_)) {
     return;
   }
 
-  std::vector<PasswordForm> forms;
+  std::vector<webkit::forms::PasswordForm> forms;
   FilePath source_path = source_path_;
   FilePath file = source_path.AppendASCII("signons.sqlite");
   if (file_util::PathExists(file)) {
@@ -337,9 +341,9 @@ void Firefox3Importer::ImportSearchEngines() {
 
   std::vector<TemplateURL*> search_engines;
   ParseSearchEnginesFromXMLFiles(files, &search_engines);
-  int default_index =
-      GetFirefoxDefaultSearchEngineIndex(search_engines, source_path_);
-  bridge_->SetKeywords(search_engines, default_index, true);
+
+  // Import the list of search engines, but do not override the default.
+  bridge_->SetKeywords(search_engines, -1 /*default_keyword_index*/, true);
 }
 
 void Firefox3Importer::ImportHomepage() {
@@ -355,21 +359,18 @@ void Firefox3Importer::GetSearchEnginesXMLFiles(
   if (!file_util::PathExists(file))
     return;
 
-  sqlite3* sqlite;
-  if (sqlite3_open(WideToUTF8(file.ToWStringHack()).c_str(),
-                   &sqlite) != SQLITE_OK) {
+  sql::Connection db;
+  if (!db.Open(file))
     return;
-  }
-  sqlite_utils::scoped_sqlite_db_ptr db(sqlite);
 
-  SQLStatement s;
-  const char* stmt = "SELECT engineid FROM engine_data "
-                     "WHERE engineid NOT IN "
-                     "(SELECT engineid FROM engine_data "
-                     "WHERE name='hidden') "
-                     "ORDER BY value ASC";
+  const char* query = "SELECT engineid FROM engine_data "
+                      "WHERE engineid NOT IN "
+                      "(SELECT engineid FROM engine_data "
+                      "WHERE name='hidden') "
+                      "ORDER BY value ASC";
 
-  if (s.prepare(db.get(), stmt) != SQLITE_OK)
+  sql::Statement s(db.GetUniqueStatement(query));
+  if (!s)
     return;
 
   FilePath app_path = app_path_.AppendASCII("searchplugins");
@@ -378,35 +379,37 @@ void Firefox3Importer::GetSearchEnginesXMLFiles(
   // Firefox doesn't store a search engine in its sqlite database unless the
   // user has added a engine. So we get search engines from sqlite db as well
   // as from the file system.
-  if (s.step() == SQLITE_ROW) {
-    const std::wstring kAppPrefix = L"[app]/";
-    const std::wstring kProfilePrefix = L"[profile]/";
+  if (s.Step()) {
+    const std::string kAppPrefix("[app]/");
+    const std::string kProfilePrefix("[profile]/");
     do {
       FilePath file;
-      std::wstring engine = UTF8ToWide(s.column_string(0));
+      std::string engine(s.ColumnString(0));
 
       // The string contains [app]/<name>.xml or [profile]/<name>.xml where
       // the [app] and [profile] need to be replaced with the actual app or
       // profile path.
       size_t index = engine.find(kAppPrefix);
-      if (index != std::wstring::npos) {
+      if (index != std::string::npos) {
         // Remove '[app]/'.
-        file = app_path.Append(FilePath::FromWStringHack(
-                                   engine.substr(index + kAppPrefix.length())));
-      } else if ((index = engine.find(kProfilePrefix)) != std::wstring::npos) {
+        file = app_path.AppendASCII(engine.substr(index + kAppPrefix.length()));
+      } else if ((index = engine.find(kProfilePrefix)) != std::string::npos) {
         // Remove '[profile]/'.
-          file = profile_path.Append(
-              FilePath::FromWStringHack(
-                  engine.substr(index + kProfilePrefix.length())));
+          file = profile_path.AppendASCII(
+              engine.substr(index + kProfilePrefix.length()));
       } else {
         // Looks like absolute path to the file.
-        file = FilePath::FromWStringHack(engine);
+#if defined(OS_WIN)
+        file = FilePath(UTF8ToWide(engine));
+#else
+        file = FilePath(engine);
+#endif
       }
       files->push_back(file);
-    } while (s.step() == SQLITE_ROW && !cancelled());
+    } while (s.Step() && !cancelled());
   }
 
-#if defined(OS_LINUX)
+#if defined(OS_POSIX) && !defined(OS_MACOSX)
   // Ubuntu-flavored Firefox3 supports locale-specific search engines via
   // locale-named subdirectories. They fall back to en-US.
   // See http://crbug.com/53899
@@ -429,22 +432,22 @@ void Firefox3Importer::GetSearchEnginesXMLFiles(
   }
 }
 
-void Firefox3Importer::LoadRootNodeID(sqlite3* db,
+void Firefox3Importer::LoadRootNodeID(sql::Connection* db,
                                       int* toolbar_folder_id,
                                       int* menu_folder_id,
                                       int* unsorted_folder_id) {
-  const char kToolbarFolderName[] = "toolbar";
-  const char kMenuFolderName[] = "menu";
-  const char kUnsortedFolderName[] = "unfiled";
+  static const char* kToolbarFolderName = "toolbar";
+  static const char* kMenuFolderName = "menu";
+  static const char* kUnsortedFolderName = "unfiled";
 
-  SQLStatement s;
-  const char* stmt = "SELECT root_name, folder_id FROM moz_bookmarks_roots";
-  if (s.prepare(db, stmt) != SQLITE_OK)
+  const char* query = "SELECT root_name, folder_id FROM moz_bookmarks_roots";
+  sql::Statement s(db->GetUniqueStatement(query));
+  if (!s)
     return;
 
-  while (s.step() == SQLITE_ROW) {
-    std::string folder = s.column_string(0);
-    int id = s.column_int(1);
+  while (s.Step()) {
+    std::string folder = s.ColumnString(0);
+    int id = s.ColumnInt(1);
     if (folder == kToolbarFolderName)
       *toolbar_folder_id = id;
     else if (folder == kMenuFolderName)
@@ -454,78 +457,85 @@ void Firefox3Importer::LoadRootNodeID(sqlite3* db,
   }
 }
 
-void Firefox3Importer::LoadLivemarkIDs(sqlite3* db,
+void Firefox3Importer::LoadLivemarkIDs(sql::Connection* db,
                                        std::set<int>* livemark) {
-  const char kFeedAnnotation[] = "livemark/feedURI";
+  static const char* kFeedAnnotation = "livemark/feedURI";
   livemark->clear();
 
-  SQLStatement s;
-  const char* stmt = "SELECT b.item_id "
-                     "FROM moz_anno_attributes a "
-                     "JOIN moz_items_annos b ON a.id = b.anno_attribute_id "
-                     "WHERE a.name = ? ";
-  if (s.prepare(db, stmt) != SQLITE_OK)
+  const char* query = "SELECT b.item_id "
+                      "FROM moz_anno_attributes a "
+                      "JOIN moz_items_annos b ON a.id = b.anno_attribute_id "
+                      "WHERE a.name = ? ";
+  sql::Statement s(db->GetUniqueStatement(query));
+  if (!s)
     return;
 
-  s.bind_string(0, kFeedAnnotation);
-  while (s.step() == SQLITE_ROW && !cancelled())
-    livemark->insert(s.column_int(0));
+  s.BindString(0, kFeedAnnotation);
+  while (s.Step() && !cancelled())
+    livemark->insert(s.ColumnInt(0));
 }
 
-void Firefox3Importer::GetTopBookmarkFolder(sqlite3* db, int folder_id,
+void Firefox3Importer::GetTopBookmarkFolder(sql::Connection* db,
+                                            int folder_id,
                                             BookmarkList* list) {
-  SQLStatement s;
-  const char* stmt = "SELECT b.title "
-         "FROM moz_bookmarks b "
-         "WHERE b.type = 2 AND b.id = ? "
-         "ORDER BY b.position";
-  if (s.prepare(db, stmt) != SQLITE_OK)
+  const char* query = "SELECT b.title "
+                     "FROM moz_bookmarks b "
+                     "WHERE b.type = 2 AND b.id = ? "
+                     "ORDER BY b.position";
+  sql::Statement s(db->GetUniqueStatement(query));
+  if (!s)
     return;
 
-  s.bind_int(0, folder_id);
-  if (s.step() == SQLITE_ROW) {
+  s.BindInt(0, folder_id);
+  if (s.Step()) {
     BookmarkItem* item = new BookmarkItem;
     item->parent = -1;  // The top level folder has no parent.
     item->id = folder_id;
-    item->title = s.column_wstring(0);
-    item->type = 2;
+    item->title = s.ColumnString16(0);
+    item->type = TYPE_FOLDER;
     item->favicon = 0;
+    item->empty_folder = true;
     list->push_back(item);
   }
 }
 
-void Firefox3Importer::GetWholeBookmarkFolder(sqlite3* db, BookmarkList* list,
-                                              size_t position) {
+void Firefox3Importer::GetWholeBookmarkFolder(sql::Connection* db,
+                                              BookmarkList* list,
+                                              size_t position,
+                                              bool* empty_folder) {
   if (position >= list->size()) {
     NOTREACHED();
     return;
   }
 
-  SQLStatement s;
-  const char* stmt = "SELECT b.id, h.url, COALESCE(b.title, h.title), "
+  const char* query = "SELECT b.id, h.url, COALESCE(b.title, h.title), "
          "b.type, k.keyword, b.dateAdded, h.favicon_id "
          "FROM moz_bookmarks b "
          "LEFT JOIN moz_places h ON b.fk = h.id "
          "LEFT JOIN moz_keywords k ON k.id = b.keyword_id "
          "WHERE b.type IN (1,2) AND b.parent = ? "
          "ORDER BY b.position";
-  if (s.prepare(db, stmt) != SQLITE_OK)
+  sql::Statement s(db->GetUniqueStatement(query));
+  if (!s)
     return;
 
-  s.bind_int(0, (*list)[position]->id);
+  s.BindInt(0, (*list)[position]->id);
   BookmarkList temp_list;
-  while (s.step() == SQLITE_ROW) {
+  while (s.Step()) {
     BookmarkItem* item = new BookmarkItem;
     item->parent = static_cast<int>(position);
-    item->id = s.column_int(0);
-    item->url = GURL(s.column_string(1));
-    item->title = s.column_wstring(2);
-    item->type = s.column_int(3);
-    item->keyword = s.column_string(4);
-    item->date_added = Time::FromTimeT(s.column_int64(5)/1000000);
-    item->favicon = s.column_int64(6);
+    item->id = s.ColumnInt(0);
+    item->url = GURL(s.ColumnString(1));
+    item->title = s.ColumnString16(2);
+    item->type = static_cast<BookmarkItemType>(s.ColumnInt(3));
+    item->keyword = s.ColumnString(4);
+    item->date_added = base::Time::FromTimeT(s.ColumnInt64(5)/1000000);
+    item->favicon = s.ColumnInt64(6);
+    item->empty_folder = true;
 
     temp_list.push_back(item);
+    if (empty_folder != NULL)
+      *empty_folder = false;
   }
 
   // Appends all items to the list.
@@ -533,40 +543,41 @@ void Firefox3Importer::GetWholeBookmarkFolder(sqlite3* db, BookmarkList* list,
        i != temp_list.end(); ++i) {
     list->push_back(*i);
     // Recursive add bookmarks in sub-folders.
-    if ((*i)->type == 2)
-      GetWholeBookmarkFolder(db, list, list->size() - 1);
+    if ((*i)->type == TYPE_FOLDER)
+      GetWholeBookmarkFolder(db, list, list->size() - 1, &(*i)->empty_folder);
   }
 }
 
 void Firefox3Importer::LoadFavicons(
-    sqlite3* db,
+    sql::Connection* db,
     const FaviconMap& favicon_map,
-    std::vector<history::ImportedFavIconUsage>* favicons) {
-  SQLStatement s;
-  const char* stmt = "SELECT url, data FROM moz_favicons WHERE id=?";
-  if (s.prepare(db, stmt) != SQLITE_OK)
+    std::vector<history::ImportedFaviconUsage>* favicons) {
+  const char* query = "SELECT url, data FROM moz_favicons WHERE id=?";
+  sql::Statement s(db->GetUniqueStatement(query));
+  if (!s)
     return;
 
   for (FaviconMap::const_iterator i = favicon_map.begin();
        i != favicon_map.end(); ++i) {
-    s.bind_int64(0, i->first);
-    if (s.step() == SQLITE_ROW) {
-      history::ImportedFavIconUsage usage;
+    s.BindInt64(0, i->first);
+    if (s.Step()) {
+      history::ImportedFaviconUsage usage;
 
-      usage.favicon_url = GURL(s.column_string(0));
+      usage.favicon_url = GURL(s.ColumnString(0));
       if (!usage.favicon_url.is_valid())
         continue;  // Don't bother importing favicons with invalid URLs.
 
       std::vector<unsigned char> data;
-      if (!s.column_blob_as_vector(1, &data) || data.empty())
+      s.ColumnBlobAsVector(1, &data);
+      if (data.empty())
         continue;  // Data definitely invalid.
 
-      if (!ReencodeFavicon(&data[0], data.size(), &usage.png_data))
+      if (!importer::ReencodeFavicon(&data[0], data.size(), &usage.png_data))
         continue;  // Unable to decode.
 
       usage.urls = i->second;
       favicons->push_back(usage);
     }
-    s.reset();
+    s.Reset();
   }
 }
