@@ -1,12 +1,14 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/importer/firefox_importer_unittest_utils.h"
 
+#include "base/bind.h"
 #include "base/base_switches.h"
 #include "base/command_line.h"
 #include "base/file_path.h"
+#include "base/file_util.h"
 #include "base/message_loop.h"
 #include "base/test/test_timeouts.h"
 #include "chrome/browser/importer/firefox_importer_utils.h"
@@ -29,32 +31,35 @@ const char kTestChannelID[] = "T1";
 // |channel| - IPC Channel to use for communication.
 // |handle| - On return, the process handle to use to communicate with the
 // child.
-bool LaunchNSSDecrypterChildProcess(const std::wstring& nss_path,
-    const IPC::Channel& channel, base::ProcessHandle* handle) {
+bool LaunchNSSDecrypterChildProcess(const FilePath& nss_path,
+    IPC::Channel* channel, base::ProcessHandle* handle) {
   CommandLine cl(*CommandLine::ForCurrentProcess());
   cl.AppendSwitchASCII(switches::kTestChildProcess, "NSSDecrypterChildProcess");
 
-  FilePath ff_dylib_dir = FilePath::FromWStringHack(nss_path);
   // Set env variable needed for FF encryption libs to load.
   // See "chrome/browser/importer/nss_decryptor_mac.mm" for an explanation of
   // why we need this.
   base::environment_vector env;
   std::pair<std::string, std::string> dyld_override;
   dyld_override.first = "DYLD_FALLBACK_LIBRARY_PATH";
-  dyld_override.second = ff_dylib_dir.value();
+  dyld_override.second = nss_path.value();
   env.push_back(dyld_override);
 
-  base::file_handle_mapping_vector fds_to_map;
-  const int ipcfd = channel.GetClientFileDescriptor();
-  if (ipcfd > -1) {
-    fds_to_map.push_back(std::pair<int,int>(ipcfd, kPrimaryIPCChannel + 3));
-  } else {
+  int ipcfd = channel->TakeClientFileDescriptor();
+  if (ipcfd == -1)
     return false;
-  }
+
+  file_util::ScopedFD client_file_descriptor_closer(&ipcfd);
+  base::file_handle_mapping_vector fds_to_map;
+  fds_to_map.push_back(std::pair<int,int>(ipcfd, kPrimaryIPCChannel + 3));
 
   bool debug_on_start = CommandLine::ForCurrentProcess()->HasSwitch(
                             switches::kDebugChildren);
-  return base::LaunchApp(cl.argv(), env, fds_to_map, debug_on_start, handle);
+  base::LaunchOptions options;
+  options.environ = &env;
+  options.fds_to_remap = &fds_to_map;
+  options.wait = debug_on_start;
+  return base::LaunchProcess(cl.argv(), options, handle);
 }
 
 }  // namespace
@@ -123,7 +128,7 @@ FFUnitTestDecryptorProxy::FFUnitTestDecryptorProxy()
     : child_process_(0) {
 }
 
-bool FFUnitTestDecryptorProxy::Setup(const std::wstring& nss_path) {
+bool FFUnitTestDecryptorProxy::Setup(const FilePath& nss_path) {
   // Create a new message loop and spawn the child process.
   message_loop_.reset(new MessageLoopForIO());
 
@@ -136,7 +141,7 @@ bool FFUnitTestDecryptorProxy::Setup(const std::wstring& nss_path) {
 
   // Spawn child and set up sync IPC connection.
   bool ret = LaunchNSSDecrypterChildProcess(nss_path,
-                                            *(channel_.get()),
+                                            channel_.get(),
                                             &child_process_);
   return ret && (child_process_ != 0);
 }
@@ -166,7 +171,7 @@ class CancellableQuitMsgLoop : public base::RefCounted<CancellableQuitMsgLoop> {
 // Spin until either a client response arrives or a timeout occurs.
 bool FFUnitTestDecryptorProxy::WaitForClientResponse() {
   // What we're trying to do here is to wait for an RPC message to go over the
-  // wire and the client to reply.  If the client does not replyy by a given
+  // wire and the client to reply.  If the client does not reply by a given
   // timeout we kill the message loop.
   // The way we do this is to post a CancellableQuitMsgLoop for 3 seconds in
   // the future and cancel it if an RPC message comes back earlier.
@@ -176,7 +181,7 @@ bool FFUnitTestDecryptorProxy::WaitForClientResponse() {
       new CancellableQuitMsgLoop());
   MessageLoop::current()->PostDelayedTask(
       FROM_HERE,
-      NewRunnableMethod(quit_task.get(), &CancellableQuitMsgLoop::QuitNow),
+      base::Bind(&CancellableQuitMsgLoop::QuitNow, quit_task.get()),
       TestTimeouts::action_max_timeout_ms());
 
   message_loop_->Run();
@@ -185,8 +190,8 @@ bool FFUnitTestDecryptorProxy::WaitForClientResponse() {
   return ret;
 }
 
-bool FFUnitTestDecryptorProxy::DecryptorInit(const std::wstring& dll_path,
-    const std::wstring& db_path) {
+bool FFUnitTestDecryptorProxy::DecryptorInit(const FilePath& dll_path,
+                                             const FilePath& db_path) {
   channel_->Send(new Msg_Decryptor_Init(dll_path, db_path));
   bool ok = WaitForClientResponse();
   if (ok && listener_->got_result) {
@@ -219,7 +224,7 @@ class FFDecryptorClientChannelListener : public IPC::Channel::Listener {
     sender_ = sender;
   }
 
-  void OnDecryptor_Init(std::wstring dll_path, std::wstring db_path) {
+  void OnDecryptor_Init(FilePath dll_path, FilePath db_path) {
     bool ret = decryptor_.Init(dll_path, db_path);
     sender_->Send(new Msg_Decryptor_InitReturnCode(ret));
   }
