@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #include <shlobj.h>
 
 #include "base/file_util.h"
+#include "base/string16.h"
 #include "base/win/registry.h"
 
 // NOTE: Keep these in order since we need test all those paths according
@@ -40,9 +41,9 @@ int GetCurrentFirefoxMajorVersionFromRegistry() {
   return highest_version;
 }
 
-std::wstring GetFirefoxInstallPathFromRegistry() {
+FilePath GetFirefoxInstallPathFromRegistry() {
   // Detects the path that Firefox is installed in.
-  std::wstring registry_path = L"Software\\Mozilla\\Mozilla Firefox";
+  string16 registry_path = L"Software\\Mozilla\\Mozilla Firefox";
   wchar_t buffer[MAX_PATH];
   DWORD buffer_length = sizeof(buffer);
   base::win::RegKey reg_key(HKEY_LOCAL_MACHINE, registry_path.c_str(),
@@ -50,16 +51,16 @@ std::wstring GetFirefoxInstallPathFromRegistry() {
   LONG result = reg_key.ReadValue(L"CurrentVersion", buffer,
                                   &buffer_length, NULL);
   if (result != ERROR_SUCCESS)
-    return std::wstring();
-  registry_path += L"\\" + std::wstring(buffer) + L"\\Main";
+    return FilePath();
+
+  registry_path += L"\\" + string16(buffer) + L"\\Main";
   buffer_length = sizeof(buffer);
   base::win::RegKey reg_key_directory(HKEY_LOCAL_MACHINE,
                                       registry_path.c_str(), KEY_READ);
   result = reg_key_directory.ReadValue(L"Install Directory", buffer,
                                        &buffer_length, NULL);
-  if (result != ERROR_SUCCESS)
-    return std::wstring();
-  return buffer;
+
+  return (result != ERROR_SUCCESS) ? FilePath() : FilePath(buffer);
 }
 
 FilePath GetProfilesINI() {
