@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,41 +15,59 @@
 #include <string>
 #include <vector>
 
-#include "app/win/scoped_co_mem.h"
-#include "app/win/scoped_com_initializer.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
-#include "base/scoped_comptr_win.h"
+#include "base/string16.h"
 #include "base/string_split.h"
 #include "base/string_util.h"
 #include "base/time.h"
-#include "base/values.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/registry.h"
+#include "base/win/scoped_co_mem.h"
+#include "base/win/scoped_com_initializer.h"
+#include "base/win/scoped_comptr.h"
 #include "base/win/scoped_handle.h"
 #include "base/win/windows_version.h"
-#include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/importer/importer_bridge.h"
 #include "chrome/browser/importer/importer_data_types.h"
 #include "chrome/browser/password_manager/ie7_password.h"
 #include "chrome/browser/search_engines/template_url.h"
-#include "chrome/browser/search_engines/template_url_model.h"
 #include "chrome/browser/search_engines/template_url_prepopulate_data.h"
+#include "chrome/browser/search_engines/template_url_service.h"
 #include "chrome/common/time_format.h"
 #include "chrome/common/url_constants.h"
 #include "googleurl/src/gurl.h"
 #include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util.h"
-#include "webkit/glue/password_form.h"
-
-using base::Time;
-using webkit_glue::PasswordForm;
+#include "webkit/forms/password_form.h"
 
 namespace {
 
+// Registry key paths from which we import IE settings.
+const char16 kStorage2Path[] =
+  L"Software\\Microsoft\\Internet Explorer\\IntelliForms\\Storage2";
+const char16 kSearchScopePath[] =
+  L"Software\\Microsoft\\Internet Explorer\\SearchScopes";
+const char16 kIESettingsMain[] =
+  L"Software\\Microsoft\\Internet Explorer\\Main";
+const char16 kIEFavoritesOrderKey[] =
+  L"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\"
+  L"MenuOrder\\Favorites";
+const char16 kIEVersionKey[] =
+  L"Software\\Microsoft\\Internet Explorer";
+const char16 kIEToolbarKey[] =
+  L"Software\\Microsoft\\Internet Explorer\\Toolbar";
+
+// A struct that hosts the information of AutoComplete data in PStore.
+struct AutoCompleteInfo {
+  string16 key;
+  std::vector<string16> data;
+  bool is_url;
+};
+
 // Gets the creation time of the given file or directory.
-static Time GetFileCreationTime(const std::wstring& file) {
-  Time creation_time;
+base::Time GetFileCreationTime(const string16& file) {
+  base::Time creation_time;
   base::win::ScopedHandle file_handle(
       CreateFile(file.c_str(), GENERIC_READ,
                  FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
@@ -57,30 +75,220 @@ static Time GetFileCreationTime(const std::wstring& file) {
                  FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS, NULL));
   FILETIME creation_filetime;
   if (GetFileTime(file_handle, &creation_filetime, NULL, NULL))
-    creation_time = Time::FromFileTime(creation_filetime);
+    creation_time = base::Time::FromFileTime(creation_filetime);
   return creation_time;
 }
 
+// Safely read an object of type T from a raw sequence of bytes.
+template<typename T>
+bool BinaryRead(T* data, size_t offset, const std::vector<uint8>& blob) {
+  if (offset + sizeof(T) > blob.size())
+    return false;
+  memcpy(data, &blob[offset], sizeof(T));
+  return true;
+}
+
+// Safely read an ITEMIDLIST from a raw sequence of bytes.
+//
+// An ITEMIDLIST is a list of SHITEMIDs, terminated by a SHITEMID with
+// .cb = 0. Here, before simply casting &blob[offset] to LPITEMIDLIST,
+// we verify that the list structure is not overrunning the boundary of
+// the binary blob.
+LPCITEMIDLIST BinaryReadItemIDList(size_t offset, size_t idlist_size,
+                                   const std::vector<uint8>& blob) {
+  size_t head = 0;
+  while (true) {
+    SHITEMID id;
+    if (head >= idlist_size || !BinaryRead(&id, offset + head, blob))
+      return NULL;
+    if (id.cb == 0)
+      break;
+    head += id.cb;
+  }
+  return reinterpret_cast<LPCITEMIDLIST>(&blob[offset]);
+}
+
+// Compares the two bookmarks in the order of IE's Favorites menu.
+// Returns true if rhs should come later than lhs (lhs < rhs).
+struct IEOrderBookmarkComparator {
+  bool operator()(const ProfileWriter::BookmarkEntry& lhs,
+                  const ProfileWriter::BookmarkEntry& rhs) const {
+    static const uint32 kNotSorted = 0xfffffffb; // IE uses this magic value.
+    FilePath lhs_prefix;
+    FilePath rhs_prefix;
+    for (size_t i = 0; i <= lhs.path.size() && i <= rhs.path.size(); ++i) {
+      const FilePath::StringType lhs_i =
+        (i < lhs.path.size() ? lhs.path[i] : lhs.title + L".url");
+      const FilePath::StringType rhs_i =
+        (i < rhs.path.size() ? rhs.path[i] : rhs.title + L".url");
+      lhs_prefix = lhs_prefix.Append(lhs_i);
+      rhs_prefix = rhs_prefix.Append(rhs_i);
+      if (lhs_i == rhs_i)
+        continue;
+      // The first path element that differs between the two.
+      std::map<FilePath, uint32>::const_iterator lhs_iter =
+        sort_index_->find(lhs_prefix);
+      std::map<FilePath, uint32>::const_iterator rhs_iter =
+        sort_index_->find(rhs_prefix);
+      uint32 lhs_sort_index = (lhs_iter == sort_index_->end() ? kNotSorted
+        : lhs_iter->second);
+      uint32 rhs_sort_index = (rhs_iter == sort_index_->end() ? kNotSorted
+        : rhs_iter->second);
+      if (lhs_sort_index != rhs_sort_index)
+        return lhs_sort_index < rhs_sort_index;
+      // If they have the same sort order, sort alphabetically.
+      return lhs_i < rhs_i;
+    }
+    return lhs.path.size() < rhs.path.size();
+  }
+  const std::map<FilePath, uint32>* sort_index_;
+};
+
+// IE stores the order of the Favorites menu in registry under:
+// HKCU\Software\Microsoft\Windows\CurrentVersion\Explorer\MenuOrder\Favorites.
+// The folder hierarchy of Favorites menu is directly mapped to the key
+// hierarchy in the registry.
+//
+// If the order of the items in a folder is customized by user, the order is
+// recorded in the REG_BINARY value named "Order" of the corresponding key.
+// The content of the "Order" value is a raw binary dump of an array of the
+// following data structure
+//   struct {
+//     uint32 size;  // Note that ITEMIDLIST is variably-sized.
+//     uint32 sort_index;  // 0 means this is the first item, 1 the second, ...
+//     ITEMIDLIST item_id;
+//   };
+// where each item_id should correspond to a favorites link file (*.url) in
+// the current folder.
+bool ParseFavoritesOrderBlob(
+    const Importer* importer,
+    const std::vector<uint8>& blob,
+    const FilePath& path,
+    std::map<FilePath, uint32>* sort_index) WARN_UNUSED_RESULT {
+  static const int kItemCountOffset = 16;
+  static const int kItemListStartOffset = 20;
+
+  // Read the number of items.
+  uint32 item_count = 0;
+  if (!BinaryRead(&item_count, kItemCountOffset, blob))
+    return false;
+
+  // Traverse over the items.
+  size_t base_offset = kItemListStartOffset;
+  for (uint32 i = 0; i < item_count && !importer->cancelled(); ++i) {
+    static const int kSizeOffset = 0;
+    static const int kSortIndexOffset = 4;
+    static const int kItemIDListOffset = 8;
+
+    // Read the size (number of bytes) of the current item.
+    uint32 item_size = 0;
+    if (!BinaryRead(&item_size, base_offset + kSizeOffset, blob) ||
+        base_offset + item_size <= base_offset || // checking overflow
+        base_offset + item_size > blob.size())
+      return false;
+
+    // Read the sort index of the current item.
+    uint32 item_sort_index = 0;
+    if (!BinaryRead(&item_sort_index, base_offset + kSortIndexOffset, blob))
+      return false;
+
+    // Read the file name from the ITEMIDLIST structure.
+    LPCITEMIDLIST idlist = BinaryReadItemIDList(
+      base_offset + kItemIDListOffset, item_size - kItemIDListOffset, blob);
+    TCHAR item_filename[MAX_PATH];
+    if (!idlist || FAILED(SHGetPathFromIDList(idlist, item_filename)))
+      return false;
+    FilePath item_relative_path =
+      path.Append(FilePath(item_filename).BaseName());
+
+    // Record the retrieved information and go to the next item.
+    sort_index->insert(std::make_pair(item_relative_path, item_sort_index));
+    base_offset += item_size;
+  }
+  return true;
+}
+
+bool ParseFavoritesOrderRegistryTree(
+    const Importer* importer,
+    const base::win::RegKey& key,
+    const FilePath& path,
+    std::map<FilePath, uint32>* sort_index) WARN_UNUSED_RESULT {
+  // Parse the order information of the current folder.
+  DWORD blob_length = 0;
+  if (key.ReadValue(L"Order", NULL, &blob_length, NULL) == ERROR_SUCCESS) {
+    std::vector<uint8> blob(blob_length);
+    if (blob_length > 0 &&
+        key.ReadValue(L"Order", reinterpret_cast<DWORD*>(&blob[0]),
+                      &blob_length, NULL) == ERROR_SUCCESS) {
+      if (!ParseFavoritesOrderBlob(importer, blob, path, sort_index))
+        return false;
+    }
+  }
+
+  // Recursively parse subfolders.
+  for (base::win::RegistryKeyIterator child(key.Handle(), L"");
+       child.Valid() && !importer->cancelled();
+       ++child) {
+    base::win::RegKey subkey(key.Handle(), child.Name(), KEY_READ);
+    if (subkey.Valid()) {
+      FilePath subpath(path.Append(child.Name()));
+      if (!ParseFavoritesOrderRegistryTree(importer, subkey, subpath,
+                                           sort_index)) {
+        return false;
+      }
+    }
+  }
+  return true;
+}
+
+bool ParseFavoritesOrderInfo(
+    const Importer* importer,
+    std::map<FilePath, uint32>* sort_index) WARN_UNUSED_RESULT {
+  base::win::RegKey key(HKEY_CURRENT_USER, kIEFavoritesOrderKey, KEY_READ);
+  if (!key.Valid())
+    return false;
+  return ParseFavoritesOrderRegistryTree(importer, key, FilePath(), sort_index);
+}
+
+// Read the sort order from registry. If failed, we don't touch the list
+// and use the default (alphabetical) order.
+void SortBookmarksInIEOrder(
+    const Importer* importer,
+    std::vector<ProfileWriter::BookmarkEntry>* bookmarks) {
+  std::map<FilePath, uint32> sort_index;
+  if (!ParseFavoritesOrderInfo(importer, &sort_index))
+    return;
+  IEOrderBookmarkComparator compare = {&sort_index};
+  std::sort(bookmarks->begin(), bookmarks->end(), compare);
+}
+
 }  // namespace
 
 // static
 // {E161255A-37C3-11D2-BCAA-00C04fD929DB}
-const GUID IEImporter::kPStoreAutocompleteGUID = {0xe161255a, 0x37c3, 0x11d2,
-    {0xbc, 0xaa, 0x00, 0xc0, 0x4f, 0xd9, 0x29, 0xdb}};
+const GUID IEImporter::kPStoreAutocompleteGUID = {
+    0xe161255a, 0x37c3, 0x11d2,
+    { 0xbc, 0xaa, 0x00, 0xc0, 0x4f, 0xd9, 0x29, 0xdb }
+};
 // {A79029D6-753E-4e27-B807-3D46AB1545DF}
-const GUID IEImporter::kUnittestGUID = { 0xa79029d6, 0x753e, 0x4e27,
-    {0xb8, 0x7, 0x3d, 0x46, 0xab, 0x15, 0x45, 0xdf}};
+const GUID IEImporter::kUnittestGUID = {
+    0xa79029d6, 0x753e, 0x4e27,
+    { 0xb8, 0x7, 0x3d, 0x46, 0xab, 0x15, 0x45, 0xdf }
+};
 
-void IEImporter::StartImport(const ProfileInfo& profile_info,
+IEImporter::IEImporter() {
+}
+
+void IEImporter::StartImport(const importer::SourceProfile& source_profile,
                              uint16 items,
                              ImporterBridge* bridge) {
   bridge_ = bridge;
-  source_path_ = profile_info.source_path.ToWStringHack();
+  source_path_ = source_profile.source_path;
 
   bridge_->NotifyStarted();
 
   // Some IE settings (such as Protected Storage) are obtained via COM APIs.
-  app::win::ScopedCOMInitializer com_initializer;
+  base::win::ScopedCOMInitializer com_initializer;
 
   if ((items & importer::HOME_PAGE) && !cancelled())
     ImportHomepage();  // Doesn't have a UI item.
@@ -112,9 +320,10 @@ void IEImporter::StartImport(const ProfileInfo& profile_info,
   bridge_->NotifyEnded();
 }
 
-void IEImporter::ImportFavorites() {
-  std::wstring path;
+IEImporter::~IEImporter() {
+}
 
+void IEImporter::ImportFavorites() {
   FavoritesInfo info;
   if (!GetFavoritesInfo(&info))
     return;
@@ -123,12 +332,66 @@ void IEImporter::ImportFavorites() {
   ParseFavoritesFolder(info, &bookmarks);
 
   if (!bookmarks.empty() && !cancelled()) {
-    const std::wstring& first_folder_name =
-        UTF16ToWide(l10n_util::GetStringUTF16(IDS_BOOKMARK_GROUP_FROM_IE));
-    int options = 0;
-    if (import_to_bookmark_bar())
-      options = ProfileWriter::IMPORT_TO_BOOKMARK_BAR;
-    bridge_->AddBookmarkEntries(bookmarks, first_folder_name, options);
+    const string16& first_folder_name =
+        l10n_util::GetStringUTF16(IDS_BOOKMARK_GROUP_FROM_IE);
+    bridge_->AddBookmarks(bookmarks, first_folder_name);
+  }
+}
+
+void IEImporter::ImportHistory() {
+  const std::string kSchemes[] = {chrome::kHttpScheme,
+                                  chrome::kHttpsScheme,
+                                  chrome::kFtpScheme,
+                                  chrome::kFileScheme};
+  int total_schemes = arraysize(kSchemes);
+
+  base::win::ScopedComPtr<IUrlHistoryStg2> url_history_stg2;
+  HRESULT result;
+  result = url_history_stg2.CreateInstance(CLSID_CUrlHistory, NULL,
+                                           CLSCTX_INPROC_SERVER);
+  if (FAILED(result))
+    return;
+  base::win::ScopedComPtr<IEnumSTATURL> enum_url;
+  if (SUCCEEDED(result = url_history_stg2->EnumUrls(enum_url.Receive()))) {
+    std::vector<history::URLRow> rows;
+    STATURL stat_url;
+    ULONG fetched;
+    while (!cancelled() &&
+           (result = enum_url->Next(1, &stat_url, &fetched)) == S_OK) {
+      string16 url_string;
+      if (stat_url.pwcsUrl) {
+        url_string = stat_url.pwcsUrl;
+        CoTaskMemFree(stat_url.pwcsUrl);
+      }
+      string16 title_string;
+      if (stat_url.pwcsTitle) {
+        title_string = stat_url.pwcsTitle;
+        CoTaskMemFree(stat_url.pwcsTitle);
+      }
+
+      GURL url(url_string);
+      // Skips the URLs that are invalid or have other schemes.
+      if (!url.is_valid() ||
+          (std::find(kSchemes, kSchemes + total_schemes, url.scheme()) ==
+           kSchemes + total_schemes))
+        continue;
+
+      history::URLRow row(url);
+      row.set_title(title_string);
+      row.set_last_visit(base::Time::FromFileTime(stat_url.ftLastVisited));
+      if (stat_url.dwFlags == STATURL_QUERYFLAG_TOPLEVEL) {
+        row.set_visit_count(1);
+        row.set_hidden(false);
+      } else {
+        row.set_hidden(true);
+      }
+
+      rows.push_back(row);
+    }
+
+    if (!rows.empty() && !cancelled()) {
+      bridge_->SetHistoryItems(rows, history::SOURCE_IE_IMPORTED);
+    }
   }
 }
 
@@ -154,7 +417,7 @@ void IEImporter::ImportPasswordsIE6() {
     return;
   }
 
-  ScopedComPtr<IPStore, &IID_IPStore> pstore;
+  base::win::ScopedComPtr<IPStore, &IID_IPStore> pstore;
   HRESULT result = PStoreCreateInstance(pstore.Receive(), 0, 0, 0);
   if (result != S_OK) {
     FreeLibrary(pstorec_dll);
@@ -164,7 +427,7 @@ void IEImporter::ImportPasswordsIE6() {
   std::vector<AutoCompleteInfo> ac_list;
 
   // Enumerates AutoComplete items in the protected database.
-  ScopedComPtr<IEnumPStoreItems, &IID_IEnumPStoreItems> item;
+  base::win::ScopedComPtr<IEnumPStoreItems, &IID_IEnumPStoreItems> item;
   result = pstore->EnumItems(0, &AutocompleteGUID,
                              &AutocompleteGUID, 0, item.Receive());
   if (result != PST_E_OK) {
@@ -182,16 +445,16 @@ void IEImporter::ImportPasswordsIE6() {
     if (SUCCEEDED(result)) {
       AutoCompleteInfo ac;
       ac.key = item_name;
-      std::wstring data;
+      string16 data;
       data.insert(0, reinterpret_cast<wchar_t*>(buffer),
                   length / sizeof(wchar_t));
 
       // The key name is always ended with ":StringData".
       const wchar_t kDataSuffix[] = L":StringData";
       size_t i = ac.key.rfind(kDataSuffix);
-      if (i != std::wstring::npos && ac.key.substr(i) == kDataSuffix) {
+      if (i != string16::npos && ac.key.substr(i) == kDataSuffix) {
         ac.key.erase(i);
-        ac.is_url = (ac.key.find(L"://") != std::wstring::npos);
+        ac.is_url = (ac.key.find(L"://") != string16::npos);
         ac_list.push_back(ac);
         base::SplitString(data, L'\0', &ac_list[ac_list.size() - 1].data);
       }
@@ -215,7 +478,7 @@ void IEImporter::ImportPasswordsIE6() {
       continue;
     }
 
-    PasswordForm form;
+    webkit::forms::PasswordForm form;
     GURL::Replacements rp;
     rp.ClearUsername();
     rp.ClearPassword();
@@ -257,9 +520,6 @@ void IEImporter::ImportPasswordsIE7() {
     return;
   }
 
-  const wchar_t kStorage2Path[] =
-      L"Software\\Microsoft\\Internet Explorer\\IntelliForms\\Storage2";
-
   base::win::RegKey key(HKEY_CURRENT_USER, kStorage2Path, KEY_READ);
   base::win::RegistryValueIterator reg_iterator(HKEY_CURRENT_USER,
                                                 kStorage2Path);
@@ -275,7 +535,7 @@ void IEImporter::ImportPasswordsIE7() {
                         &password_info.encrypted_data.front(),
                         &value_len, NULL) == ERROR_SUCCESS) {
         password_info.url_hash = reg_iterator.Name();
-        password_info.date_created = Time::Now();
+        password_info.date_created = base::Time::Now();
 
         bridge_->AddIE7PasswordInfo(password_info);
       }
@@ -285,86 +545,20 @@ void IEImporter::ImportPasswordsIE7() {
   }
 }
 
-// Reads history information from COM interface.
-void IEImporter::ImportHistory() {
-  const std::string kSchemes[] = {chrome::kHttpScheme,
-                                  chrome::kHttpsScheme,
-                                  chrome::kFtpScheme,
-                                  chrome::kFileScheme};
-  int total_schemes = arraysize(kSchemes);
-
-  ScopedComPtr<IUrlHistoryStg2> url_history_stg2;
-  HRESULT result;
-  result = url_history_stg2.CreateInstance(CLSID_CUrlHistory, NULL,
-                                           CLSCTX_INPROC_SERVER);
-  if (FAILED(result))
-    return;
-  ScopedComPtr<IEnumSTATURL> enum_url;
-  if (SUCCEEDED(result = url_history_stg2->EnumUrls(enum_url.Receive()))) {
-    std::vector<history::URLRow> rows;
-    STATURL stat_url;
-    ULONG fetched;
-    while (!cancelled() &&
-           (result = enum_url->Next(1, &stat_url, &fetched)) == S_OK) {
-      std::wstring url_string;
-      std::wstring title_string;
-      if (stat_url.pwcsUrl) {
-        url_string = stat_url.pwcsUrl;
-        CoTaskMemFree(stat_url.pwcsUrl);
-      }
-      if (stat_url.pwcsTitle) {
-        title_string = stat_url.pwcsTitle;
-        CoTaskMemFree(stat_url.pwcsTitle);
-      }
-
-      GURL url(url_string);
-      // Skips the URLs that are invalid or have other schemes.
-      if (!url.is_valid() ||
-          (std::find(kSchemes, kSchemes + total_schemes, url.scheme()) ==
-           kSchemes + total_schemes))
-        continue;
-
-      history::URLRow row(url);
-      row.set_title(title_string);
-      row.set_last_visit(Time::FromFileTime(stat_url.ftLastVisited));
-      if (stat_url.dwFlags == STATURL_QUERYFLAG_TOPLEVEL) {
-        row.set_visit_count(1);
-        row.set_hidden(false);
-      } else {
-        row.set_hidden(true);
-      }
-
-      rows.push_back(row);
-    }
-
-    if (!rows.empty() && !cancelled()) {
-      bridge_->SetHistoryItems(rows, history::SOURCE_IE_IMPORTED);
-    }
-  }
-}
-
 void IEImporter::ImportSearchEngines() {
   // On IE, search engines are stored in the registry, under:
   // Software\Microsoft\Internet Explorer\SearchScopes
   // Each key represents a search engine. The URL value contains the URL and
   // the DisplayName the name.
-  // The default key's name is contained under DefaultScope.
-  const wchar_t kSearchScopePath[] =
-      L"Software\\Microsoft\\Internet Explorer\\SearchScopes";
-
-  base::win::RegKey key(HKEY_CURRENT_USER, kSearchScopePath, KEY_READ);
-  std::wstring default_search_engine_name;
-  const TemplateURL* default_search_engine = NULL;
   std::map<std::string, TemplateURL*> search_engines_map;
-  key.ReadValue(L"DefaultScope", &default_search_engine_name);
   base::win::RegistryKeyIterator key_iterator(HKEY_CURRENT_USER,
                                               kSearchScopePath);
   while (key_iterator.Valid()) {
-    std::wstring sub_key_name = kSearchScopePath;
+    string16 sub_key_name = kSearchScopePath;
     sub_key_name.append(L"\\").append(key_iterator.Name());
     base::win::RegKey sub_key(HKEY_CURRENT_USER, sub_key_name.c_str(),
                               KEY_READ);
-    std::wstring wide_url;
+    string16 wide_url;
     if ((sub_key.ReadValue(L"URL", &wide_url) != ERROR_SUCCESS) ||
         wide_url.empty()) {
       VLOG(1) << "No URL for IE search engine at " << key_iterator.Name();
@@ -374,7 +568,7 @@ void IEImporter::ImportSearchEngines() {
     // For the name, we try the default value first (as Live Search uses a
     // non displayable name in DisplayName, and the readable name under the
     // default value).
-    std::wstring name;
+    string16 name;
     if ((sub_key.ReadValue(NULL, &name) != ERROR_SUCCESS) || name.empty()) {
       // Try the displayable name.
       if ((sub_key.ReadValue(L"DisplayName", &name) != ERROR_SUCCESS) ||
@@ -397,43 +591,32 @@ void IEImporter::ImportSearchEngines() {
       template_url->SetURL(url, 0, 0);
       // Give this a keyword to facilitate tab-to-search, if possible.
       GURL gurl = GURL(url);
-      template_url->set_keyword(TemplateURLModel::GenerateKeyword(gurl,
-                                                                  false));
+      template_url->set_keyword(TemplateURLService::GenerateKeyword(gurl,
+                                                                    false));
       template_url->set_logo_id(
           TemplateURLPrepopulateData::GetSearchEngineLogo(gurl));
       template_url->set_show_in_default_list(true);
       search_engines_map[url] = template_url;
     }
-    if (template_url && key_iterator.Name() == default_search_engine_name) {
-      DCHECK(!default_search_engine);
-      default_search_engine = template_url;
-    }
     ++key_iterator;
   }
 
   // ProfileWriter::AddKeywords() requires a vector and we have a map.
-  std::map<std::string, TemplateURL*>::iterator t_iter;
+  std::map<std::string, TemplateURL*>::iterator i;
   std::vector<TemplateURL*> search_engines;
-  int default_search_engine_index = -1;
-  for (t_iter = search_engines_map.begin(); t_iter != search_engines_map.end();
-       ++t_iter) {
-    search_engines.push_back(t_iter->second);
-    if (default_search_engine == t_iter->second) {
-      default_search_engine_index =
-          static_cast<int>(search_engines.size()) - 1;
-    }
-  }
-  bridge_->SetKeywords(search_engines, default_search_engine_index, true);
+  for (i = search_engines_map.begin(); i != search_engines_map.end(); ++i)
+    search_engines.push_back(i->second);
+
+  // Import the list of search engines, but do not override the default.
+  bridge_->SetKeywords(search_engines, -1 /*default_keyword_index*/, true);
 }
 
 void IEImporter::ImportHomepage() {
-  const wchar_t kIESettingsMain[] =
-      L"Software\\Microsoft\\Internet Explorer\\Main";
-  const wchar_t kIEHomepage[] = L"Start Page";
-  const wchar_t kIEDefaultHomepage[] = L"Default_Page_URL";
+  const wchar_t* kIEHomepage = L"Start Page";
+  const wchar_t* kIEDefaultHomepage = L"Default_Page_URL";
 
   base::win::RegKey key(HKEY_CURRENT_USER, kIESettingsMain, KEY_READ);
-  std::wstring homepage_url;
+  string16 homepage_url;
   if (key.ReadValue(kIEHomepage, &homepage_url) != ERROR_SUCCESS ||
       homepage_url.empty())
     return;
@@ -444,7 +627,7 @@ void IEImporter::ImportHomepage() {
 
   // Check to see if this is the default website and skip import.
   base::win::RegKey keyDefault(HKEY_LOCAL_MACHINE, kIESettingsMain, KEY_READ);
-  std::wstring default_homepage_url;
+  string16 default_homepage_url;
   LONG result = keyDefault.ReadValue(kIEDefaultHomepage, &default_homepage_url);
   if (result == ERROR_SUCCESS && !default_homepage_url.empty()) {
     if (homepage.spec() == GURL(default_homepage_url).spec())
@@ -454,11 +637,37 @@ void IEImporter::ImportHomepage() {
   bridge_->AddHomePage(homepage);
 }
 
-bool IEImporter::GetFavoritesInfo(IEImporter::FavoritesInfo *info) {
+string16 IEImporter::ResolveInternetShortcut(const string16& file) {
+  base::win::ScopedCoMem<wchar_t> url;
+  base::win::ScopedComPtr<IUniformResourceLocator> url_locator;
+  HRESULT result = url_locator.CreateInstance(CLSID_InternetShortcut, NULL,
+                                              CLSCTX_INPROC_SERVER);
+  if (FAILED(result))
+    return string16();
+
+  base::win::ScopedComPtr<IPersistFile> persist_file;
+  result = persist_file.QueryFrom(url_locator);
+  if (FAILED(result))
+    return string16();
+
+  // Loads the Internet Shortcut from persistent storage.
+  result = persist_file->Load(file.c_str(), STGM_READ);
+  if (FAILED(result))
+    return string16();
+
+  result = url_locator->GetURL(&url);
+  // GetURL can return S_FALSE (FAILED(S_FALSE) is false) when url == NULL.
+  if (FAILED(result) || (url == NULL))
+    return string16();
+
+  return string16(url);
+}
+
+bool IEImporter::GetFavoritesInfo(IEImporter::FavoritesInfo* info) {
   if (!source_path_.empty()) {
     // Source path exists during testing.
     info->path = source_path_;
-    file_util::AppendToPath(&info->path, L"Favorites");
+    info->path = info->path.AppendASCII("Favorites");
     info->links_folder = L"Links";
     return true;
   }
@@ -468,7 +677,7 @@ bool IEImporter::GetFavoritesInfo(IEImporter::FavoritesInfo *info) {
   if (FAILED(SHGetFolderPath(NULL, CSIDL_FAVORITES, NULL,
                              SHGFP_TYPE_CURRENT, buffer)))
     return false;
-  info->path = buffer;
+  info->path = FilePath(buffer);
 
   // There is a Links folder under Favorites folder in Windows Vista, but it
   // is not recording in Vista's registry. So in Vista, we assume the Links
@@ -477,8 +686,7 @@ bool IEImporter::GetFavoritesInfo(IEImporter::FavoritesInfo *info) {
   if (base::win::GetVersion() < base::win::VERSION_VISTA) {
     // The Link folder name is stored in the registry.
     DWORD buffer_length = sizeof(buffer);
-    base::win::RegKey reg_key(HKEY_CURRENT_USER,
-        L"Software\\Microsoft\\Internet Explorer\\Toolbar", KEY_READ);
+    base::win::RegKey reg_key(HKEY_CURRENT_USER, kIEToolbarKey, KEY_READ);
     if (reg_key.ReadValue(L"LinksFolderName", buffer,
                           &buffer_length, NULL) != ERROR_SUCCESS)
       return false;
@@ -492,9 +700,6 @@ bool IEImporter::GetFavoritesInfo(IEImporter::FavoritesInfo *info) {
 
 void IEImporter::ParseFavoritesFolder(const FavoritesInfo& info,
                                       BookmarkVector* bookmarks) {
-  std::wstring ie_folder =
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_BOOKMARK_GROUP_FROM_IE));
-  BookmarkVector toolbar_bookmarks;
   FilePath file;
   std::vector<FilePath::StringType> file_list;
   FilePath favorites_path(info.path);
@@ -526,7 +731,7 @@ void IEImporter::ParseFavoritesFolder(const FavoritesInfo& info,
     //   C:\Users\Foo\Favorites\Links\Bar\Baz.url -> "Links\Bar"
     FilePath::StringType relative_string =
         shortcut.DirName().value().substr(favorites_path_len);
-    if (relative_string.size() > 0 && FilePath::IsSeparator(relative_string[0]))
+    if (!relative_string.empty() && FilePath::IsSeparator(relative_string[0]))
       relative_string = relative_string.substr(1);
     FilePath relative_path(relative_string);
 
@@ -538,49 +743,16 @@ void IEImporter::ParseFavoritesFolder(const FavoritesInfo& info,
     if (!relative_path.empty())
       relative_path.GetComponents(&entry.path);
 
-    // Flatten the bookmarks in Link folder onto bookmark toolbar. Otherwise,
-    // put it into "Other bookmarks".
-    if (import_to_bookmark_bar() &&
-        (entry.path.size() > 0 && entry.path[0] == info.links_folder)) {
+    // Add the bookmark.
+    if (!entry.path.empty() && entry.path[0] == info.links_folder) {
+      // Bookmarks in the Link folder should be imported to the toolbar.
       entry.in_toolbar = true;
-      entry.path.erase(entry.path.begin());
-      toolbar_bookmarks.push_back(entry);
-    } else {
-      // We put the bookmarks in a "Imported From IE"
-      // folder, so that we don't mess up the "Other bookmarks".
-      if (!import_to_bookmark_bar())
-        entry.path.insert(entry.path.begin(), ie_folder);
-      bookmarks->push_back(entry);
     }
+    bookmarks->push_back(entry);
   }
-  bookmarks->insert(bookmarks->begin(), toolbar_bookmarks.begin(),
-                    toolbar_bookmarks.end());
-}
 
-std::wstring IEImporter::ResolveInternetShortcut(const std::wstring& file) {
-  app::win::ScopedCoMem<wchar_t> url;
-  ScopedComPtr<IUniformResourceLocator> url_locator;
-  HRESULT result = url_locator.CreateInstance(CLSID_InternetShortcut, NULL,
-                                              CLSCTX_INPROC_SERVER);
-  if (FAILED(result))
-    return std::wstring();
-
-  ScopedComPtr<IPersistFile> persist_file;
-  result = persist_file.QueryFrom(url_locator);
-  if (FAILED(result))
-    return std::wstring();
-
-  // Loads the Internet Shortcut from persistent storage.
-  result = persist_file->Load(file.c_str(), STGM_READ);
-  if (FAILED(result))
-    return std::wstring();
-
-  result = url_locator->GetURL(&url);
-  // GetURL can return S_FALSE (FAILED(S_FALSE) is false) when url == NULL.
-  if (FAILED(result) || (url == NULL))
-    return std::wstring();
-
-  return std::wstring(url);
+  // Reflect the menu order in IE.
+  SortBookmarksInIEOrder(this, bookmarks);
 }
 
 int IEImporter::CurrentIEVersion() const {
@@ -588,8 +760,7 @@ int IEImporter::CurrentIEVersion() const {
   if (version < 0) {
     wchar_t buffer[128];
     DWORD buffer_length = sizeof(buffer);
-    base::win::RegKey reg_key(HKEY_LOCAL_MACHINE,
-        L"Software\\Microsoft\\Internet Explorer", KEY_READ);
+    base::win::RegKey reg_key(HKEY_LOCAL_MACHINE, kIEVersionKey, KEY_READ);
     LONG result = reg_key.ReadValue(L"Version", buffer, &buffer_length, NULL);
     version = ((result == ERROR_SUCCESS)? _wtoi(buffer) : 0);
   }
