@@ -50,14 +50,10 @@
 #include "base/message_loop.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
-#include "base/values.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/importer/firefox_importer_utils.h"
-#include "chrome/browser/importer/importer.h"
 #include "chrome/browser/importer/importer_bridge.h"
 
-using base::Time;
-
 namespace {
 
 // Convert a hex character (0-9, A-F) to its corresponding byte value.
@@ -176,8 +172,10 @@ bool MorkReader::ParseMap(const std::string& first_line,
                           StringMap* map) {
   // If the first line is the a=c line (column map), just skip over it.
   std::string line(first_line);
-  if (StartsWithASCII(line, "< <(a=c)>", true))
-    ReadLine(&line);
+  if (StartsWithASCII(line, "< <(a=c)>", true)) {
+    if (!ReadLine(&line))
+      return false;
+  }
 
   std::string key;
   do {
@@ -535,7 +533,7 @@ void AddToHistory(MorkReader::ColumnDataList* column_values,
     int64 date;
     base::StringToInt64(values[kLastVisitColumn], &date);
     if (date != 0)
-      row.set_last_visit(Time::FromTimeT(date / 1000000));
+      row.set_last_visit(base::Time::FromTimeT(date / 1000000));
 
     bool is_typed = (values[kTypedColumn] == "1");
     if (is_typed)
