@@ -4,36 +4,89 @@
 
 #include "chrome/browser/importer/profile_writer.h"
 
-#include "base/string_util.h"
+#include <map>
+#include <set>
+#include <string>
+
+#include "base/string_number_conversions.h"
+#include "base/stringprintf.h"
 #include "base/threading/thread.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
-#include "chrome/browser/importer/importer.h"
 #include "chrome/browser/password_manager/password_store.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search_engines/template_url.h"
-#include "chrome/browser/search_engines/template_url_model.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/browser/search_engines/template_url_service.h"
+#include "chrome/browser/search_engines/template_url_service_factory.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
 
-using webkit_glue::PasswordForm;
+namespace {
 
-ProfileWriter::BookmarkEntry::BookmarkEntry() : in_toolbar(false) {}
+// Generates a unique folder name. If |folder_name| is not unique, then this
+// repeatedly tests for '|folder_name| + (i)' until a unique name is found.
+string16 GenerateUniqueFolderName(BookmarkModel* model,
+                                  const string16& folder_name) {
+  // Build a set containing the bookmark bar folder names.
+  std::set<string16> existing_folder_names;
+  const BookmarkNode* bookmark_bar = model->bookmark_bar_node();
+  for (int i = 0; i < bookmark_bar->child_count(); ++i) {
+    const BookmarkNode* node = bookmark_bar->GetChild(i);
+    if (node->is_folder())
+      existing_folder_names.insert(node->GetTitle());
+  }
+
+  // If the given name is unique, use it.
+  if (existing_folder_names.find(folder_name) == existing_folder_names.end())
+    return folder_name;
+
+  // Otherwise iterate until we find a unique name.
+  for (size_t i = 1; i <= existing_folder_names.size(); ++i) {
+    string16 name = folder_name + ASCIIToUTF16(" (") + base::IntToString16(i) +
+        ASCIIToUTF16(")");
+    if (existing_folder_names.find(name) == existing_folder_names.end())
+      return name;
+  }
+
+  NOTREACHED();
+  return folder_name;
+}
+
+// Shows the bookmarks toolbar.
+void ShowBookmarkBar(Profile* profile) {
+  profile->GetPrefs()->SetBoolean(prefs::kShowBookmarkBar, true);
+}
+
+}  // namespace
+
+ProfileWriter::BookmarkEntry::BookmarkEntry()
+    : in_toolbar(false),
+      is_folder(false) {}
 
 ProfileWriter::BookmarkEntry::~BookmarkEntry() {}
 
+bool ProfileWriter::BookmarkEntry::operator==(
+    const ProfileWriter::BookmarkEntry& other) const {
+  return (in_toolbar == other.in_toolbar &&
+          is_folder == other.is_folder &&
+          url == other.url &&
+          path == other.path &&
+          title == other.title &&
+          creation_time == other.creation_time);
+}
+
 ProfileWriter::ProfileWriter(Profile* profile) : profile_(profile) {}
 
 bool ProfileWriter::BookmarkModelIsLoaded() const {
   return profile_->GetBookmarkModel()->IsLoaded();
 }
 
-bool ProfileWriter::TemplateURLModelIsLoaded() const {
-  return profile_->GetTemplateURLModel()->loaded();
+bool ProfileWriter::TemplateURLServiceIsLoaded() const {
+  return TemplateURLServiceFactory::GetForProfile(profile_)->loaded();
 }
 
-void ProfileWriter::AddPasswordForm(const PasswordForm& form) {
+void ProfileWriter::AddPasswordForm(const webkit::forms::PasswordForm& form) {
   profile_->GetPasswordStore(Profile::EXPLICIT_ACCESS)->AddLogin(form);
 }
 
@@ -57,96 +110,122 @@ void ProfileWriter::AddHomepage(const GURL& home_page) {
   const PrefService::Preference* pref = prefs->FindPreference(prefs::kHomePage);
   if (pref && !pref->IsManaged()) {
     prefs->SetString(prefs::kHomePage, home_page.spec());
-    prefs->ScheduleSavePersistentPrefs();
   }
 }
 
-void ProfileWriter::AddBookmarkEntry(
-    const std::vector<BookmarkEntry>& bookmark,
-    const std::wstring& first_folder_name,
-    int options) {
+void ProfileWriter::AddBookmarks(const std::vector<BookmarkEntry>& bookmarks,
+                                 const string16& top_level_folder_name) {
+  if (bookmarks.empty())
+    return;
+
   BookmarkModel* model = profile_->GetBookmarkModel();
   DCHECK(model->IsLoaded());
 
-  bool import_to_bookmark_bar = ((options & IMPORT_TO_BOOKMARK_BAR) != 0);
-  std::wstring real_first_folder = import_to_bookmark_bar ? first_folder_name :
-      GenerateUniqueFolderName(model, first_folder_name);
+  // If the bookmark bar is currently empty, we should import directly to it.
+  // Otherwise, we should import everything to a subfolder.
+  const BookmarkNode* bookmark_bar = model->bookmark_bar_node();
+  bool import_to_top_level = bookmark_bar->empty();
 
-  bool show_bookmark_toolbar = false;
-  std::set<const BookmarkNode*> groups_added_to;
-  bool import_mode = false;
-  if (bookmark.size() > 1) {
-    model->BeginImportMode();
-    import_mode = true;
+  // Reorder bookmarks so that the toolbar entries come first.
+  std::vector<BookmarkEntry> toolbar_bookmarks;
+  std::vector<BookmarkEntry> reordered_bookmarks;
+  for (std::vector<BookmarkEntry>::const_iterator it = bookmarks.begin();
+       it != bookmarks.end(); ++it) {
+    if (it->in_toolbar)
+      toolbar_bookmarks.push_back(*it);
+    else
+      reordered_bookmarks.push_back(*it);
   }
-  for (std::vector<BookmarkEntry>::const_iterator it = bookmark.begin();
-       it != bookmark.end(); ++it) {
-    // Don't insert this url if it isn't valid.
-    if (!it->url.is_valid())
+  reordered_bookmarks.insert(reordered_bookmarks.begin(),
+                             toolbar_bookmarks.begin(),
+                             toolbar_bookmarks.end());
+
+  // If the user currently has no bookmarks in the bookmark bar, make sure that
+  // at least some of the imported bookmarks end up there.  Otherwise, we'll end
+  // up with just a single folder containing the imported bookmarks, which makes
+  // for unnecessary nesting.
+  bool add_all_to_top_level = import_to_top_level && toolbar_bookmarks.empty();
+
+  model->BeginImportMode();
+
+  std::set<const BookmarkNode*> folders_added_to;
+  const BookmarkNode* top_level_folder = NULL;
+  for (std::vector<BookmarkEntry>::const_iterator bookmark =
+         reordered_bookmarks.begin();
+       bookmark != reordered_bookmarks.end(); ++bookmark) {
+    // Disregard any bookmarks with invalid urls.
+    if (!bookmark->is_folder && !bookmark->url.is_valid())
       continue;
 
-    // We suppose that bookmarks are unique by Title, URL, and Folder.  Since
-    // checking for uniqueness may not be always the user's intention we have
-    // this as an option.
-    if (options & ADD_IF_UNIQUE && DoesBookmarkExist(model, *it,
-        real_first_folder, import_to_bookmark_bar))
-      continue;
+    const BookmarkNode* parent = NULL;
+    if (import_to_top_level && (add_all_to_top_level || bookmark->in_toolbar)) {
+      // Add directly to the bookmarks bar.
+      parent = bookmark_bar;
+    } else {
+      // Add to a folder that will contain all the imported bookmarks not added
+      // to the bar.  The first time we do so, create the folder.
+      if (!top_level_folder) {
+        string16 name = GenerateUniqueFolderName(model, top_level_folder_name);
+        top_level_folder = model->AddFolder(bookmark_bar,
+                                            bookmark_bar->child_count(),
+                                            name);
+      }
+      parent = top_level_folder;
+    }
+
+    // Ensure any enclosing folders are present in the model.  The bookmark's
+    // enclosing folder structure should be
+    //   path[0] > path[1] > ... > path[size() - 1]
+    for (std::vector<string16>::const_iterator folder_name =
+             bookmark->path.begin();
+         folder_name != bookmark->path.end(); ++folder_name) {
+      if (bookmark->in_toolbar && parent == bookmark_bar &&
+          folder_name == bookmark->path.begin()) {
+        // If we're importing directly to the bookmarks bar, skip over the
+        // folder named "Bookmarks Toolbar" (or any non-Firefox equivalent).
+        continue;
+      }
 
-    // Set up groups in BookmarkModel in such a way that path[i] is
-    // the subgroup of path[i-1]. Finally they construct a path in the
-    // model:
-    //   path[0] \ path[1] \ ... \ path[size() - 1]
-    const BookmarkNode* parent =
-        (it->in_toolbar ? model->GetBookmarkBarNode() : model->other_node());
-    for (std::vector<std::wstring>::const_iterator i = it->path.begin();
-         i != it->path.end(); ++i) {
       const BookmarkNode* child = NULL;
-      const std::wstring& folder_name = (!import_to_bookmark_bar &&
-          !it->in_toolbar && (i == it->path.begin())) ? real_first_folder : *i;
-
-      for (int index = 0; index < parent->GetChildCount(); ++index) {
+      for (int index = 0; index < parent->child_count(); ++index) {
         const BookmarkNode* node = parent->GetChild(index);
-        if ((node->type() == BookmarkNode::BOOKMARK_BAR ||
-             node->type() == BookmarkNode::FOLDER) &&
-            node->GetTitle() == WideToUTF16Hack(folder_name)) {
+        if (node->is_folder() && node->GetTitle() == *folder_name) {
           child = node;
           break;
         }
       }
-      if (child == NULL)
-        child = model->AddGroup(parent, parent->GetChildCount(),
-                                WideToUTF16Hack(folder_name));
+      if (!child)
+        child = model->AddFolder(parent, parent->child_count(), *folder_name);
       parent = child;
     }
-    groups_added_to.insert(parent);
-    model->AddURLWithCreationTime(parent, parent->GetChildCount(),
-        WideToUTF16Hack(it->title), it->url, it->creation_time);
 
-    // If some items are put into toolbar, it looks like the user was using
-    // it in their last browser. We turn on the bookmarks toolbar.
-    if (it->in_toolbar)
-      show_bookmark_toolbar = true;
+    folders_added_to.insert(parent);
+    if (bookmark->is_folder) {
+      model->AddFolder(parent, parent->child_count(), bookmark->title);
+    } else {
+      model->AddURLWithCreationTime(parent, parent->child_count(),
+                                    bookmark->title, bookmark->url,
+                                    bookmark->creation_time);
+    }
   }
 
-  // Reset the date modified time of the groups we added to. We do this to
-  // make sure the 'recently added to' combobox in the bubble doesn't get random
-  // groups.
+  // In order to keep the imported-to folders from appearing in the 'recently
+  // added to' combobox, reset their modified times.
   for (std::set<const BookmarkNode*>::const_iterator i =
-          groups_added_to.begin();
-       i != groups_added_to.end(); ++i) {
-    model->ResetDateGroupModified(*i);
+           folders_added_to.begin();
+       i != folders_added_to.end(); ++i) {
+    model->ResetDateFolderModified(*i);
   }
 
-  if (import_mode) {
-    model->EndImportMode();
-  }
+  model->EndImportMode();
 
-  if (show_bookmark_toolbar && !(options & BOOKMARK_BAR_DISABLED))
-    ShowBookmarkBar();
+  // If the user was previously using a toolbar, we should show the bar.
+  if (import_to_top_level && !add_all_to_top_level)
+    ShowBookmarkBar(profile_);
 }
 
 void ProfileWriter::AddFavicons(
-    const std::vector<history::ImportedFavIconUsage>& favicons) {
+    const std::vector<history::ImportedFaviconUsage>& favicons) {
   profile_->GetFaviconService(Profile::EXPLICIT_ACCESS)->
       SetImportedFavicons(favicons);
 }
@@ -189,8 +268,8 @@ static std::string BuildHostPathKey(const TemplateURL* t_url,
 }
 
 // Builds a set that contains an entry of the host+path for each TemplateURL in
-// the TemplateURLModel that has a valid search url.
-static void BuildHostPathMap(const TemplateURLModel& model,
+// the TemplateURLService that has a valid search url.
+static void BuildHostPathMap(const TemplateURLService& model,
                              HostPathMap* host_path_map) {
   std::vector<const TemplateURL*> template_urls = model.GetTemplateURLs();
   for (size_t i = 0; i < template_urls.size(); ++i) {
@@ -214,7 +293,8 @@ static void BuildHostPathMap(const TemplateURLModel& model,
 void ProfileWriter::AddKeywords(const std::vector<TemplateURL*>& template_urls,
                                 int default_keyword_index,
                                 bool unique_on_host_and_path) {
-  TemplateURLModel* model = profile_->GetTemplateURLModel();
+  TemplateURLService* model =
+      TemplateURLServiceFactory::GetForProfile(profile_);
   HostPathMap host_path_map;
   if (unique_on_host_and_path)
     BuildHostPathMap(*model, &host_path_map);
@@ -226,7 +306,7 @@ void ProfileWriter::AddKeywords(const std::vector<TemplateURL*>& template_urls,
         default_keyword_index >= 0 &&
         (i - template_urls.begin() == default_keyword_index);
 
-    // TemplateURLModel requires keywords to be unique. If there is already a
+    // TemplateURLService requires keywords to be unique. If there is already a
     // TemplateURL with this keyword, don't import it again.
     const TemplateURL* turl_with_keyword =
         model->GetTemplateURLForKeyword(t_url->keyword());
@@ -267,94 +347,4 @@ void ProfileWriter::AddKeywords(const std::vector<TemplateURL*>& template_urls,
   }
 }
 
-void ProfileWriter::ShowBookmarkBar() {
-  DCHECK(profile_);
-
-  PrefService* prefs = profile_->GetPrefs();
-  // Check whether the bookmark bar is shown in current pref.
-  if (!prefs->GetBoolean(prefs::kShowBookmarkBar)) {
-    // Set the pref and notify the notification service.
-    prefs->SetBoolean(prefs::kShowBookmarkBar, true);
-    prefs->ScheduleSavePersistentPrefs();
-    Source<Profile> source(profile_);
-    NotificationService::current()->Notify(
-        NotificationType::BOOKMARK_BAR_VISIBILITY_PREF_CHANGED, source,
-        NotificationService::NoDetails());
-  }
-}
-
 ProfileWriter::~ProfileWriter() {}
-
-std::wstring ProfileWriter::GenerateUniqueFolderName(
-    BookmarkModel* model,
-    const std::wstring& folder_name) {
-  // Build a set containing the folder names of the other folder.
-  std::set<std::wstring> other_folder_names;
-  const BookmarkNode* other = model->other_node();
-
-  for (int i = 0, child_count = other->GetChildCount(); i < child_count; ++i) {
-    const BookmarkNode* node = other->GetChild(i);
-    if (node->is_folder())
-      other_folder_names.insert(UTF16ToWideHack(node->GetTitle()));
-  }
-
-  if (other_folder_names.find(folder_name) == other_folder_names.end())
-    return folder_name;  // Name is unique, use it.
-
-  // Otherwise iterate until we find a unique name.
-  for (int i = 1; i < 100; ++i) {
-    std::wstring name = folder_name + StringPrintf(L" (%d)", i);
-    if (other_folder_names.find(name) == other_folder_names.end())
-      return name;
-  }
-
-  return folder_name;
-}
-
-bool ProfileWriter::DoesBookmarkExist(
-    BookmarkModel* model,
-    const BookmarkEntry& entry,
-    const std::wstring& first_folder_name,
-    bool import_to_bookmark_bar) {
-  std::vector<const BookmarkNode*> nodes_with_same_url;
-  model->GetNodesByURL(entry.url, &nodes_with_same_url);
-  if (nodes_with_same_url.empty())
-    return false;
-
-  for (size_t i = 0; i < nodes_with_same_url.size(); ++i) {
-    const BookmarkNode* node = nodes_with_same_url[i];
-    if (WideToUTF16Hack(entry.title) != node->GetTitle())
-      continue;
-
-    // Does the path match?
-    bool found_match = true;
-    const BookmarkNode* parent = node->GetParent();
-    for (std::vector<std::wstring>::const_reverse_iterator path_it =
-             entry.path.rbegin();
-         (path_it != entry.path.rend()) && found_match; ++path_it) {
-      const std::wstring& folder_name =
-          (!import_to_bookmark_bar && path_it + 1 == entry.path.rend()) ?
-          first_folder_name : *path_it;
-      if (NULL == parent || *path_it != folder_name)
-        found_match = false;
-      else
-        parent = parent->GetParent();
-    }
-
-    // We need a post test to differentiate checks such as
-    // /home/hello and /hello. The parent should either by the other folder
-    // node, or the bookmarks bar, depending upon import_to_bookmark_bar and
-    // entry.in_toolbar.
-    if (found_match &&
-        ((import_to_bookmark_bar && entry.in_toolbar && parent !=
-          model->GetBookmarkBarNode()) ||
-         ((!import_to_bookmark_bar || !entry.in_toolbar) &&
-           parent != model->other_node()))) {
-      found_match = false;
-    }
-
-    if (found_match)
-      return true;  // Found a match with the same url path and title.
-  }
-  return false;
-}
