@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,26 +10,20 @@
 #include <vector>
 
 #include "base/file_util.h"
-#include "base/message_loop.h"
-#include "base/scoped_nsobject.h"
+#include "base/mac/mac_util.h"
+#include "base/memory/scoped_nsobject.h"
 #include "base/string16.h"
 #include "base/sys_string_conversions.h"
 #include "base/time.h"
-#include "base/values.h"
+#include "base/utf_string_conversions.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/importer/importer_bridge.h"
-#include "chrome/browser/importer/importer_data_types.h"
-#include "chrome/common/sqlite_utils.h"
+#include "chrome/browser/importer/importer_util.h"
 #include "chrome/common/url_constants.h"
 #include "googleurl/src/gurl.h"
 #include "grit/generated_resources.h"
 #include "net/base/data_url.h"
-
-using importer::FAVORITES;
-using importer::HISTORY;
-using importer::NONE;
-using importer::PASSWORDS;
-using importer::ProfileInfo;
+#include "sql/statement.h"
 
 namespace {
 
@@ -56,7 +50,7 @@ SafariImporter::~SafariImporter() {
 
 // static
 bool SafariImporter::CanImport(const FilePath& library_dir,
-                               uint16 *services_supported) {
+                               uint16* services_supported) {
   DCHECK(services_supported);
   *services_supported = importer::NONE;
 
@@ -67,35 +61,35 @@ bool SafariImporter::CanImport(const FilePath& library_dir,
   FilePath bookmarks_path = safari_dir.Append("Bookmarks.plist");
   FilePath history_path = safari_dir.Append("History.plist");
 
-  using file_util::PathExists;
-  if (PathExists(bookmarks_path))
+  if (file_util::PathExists(bookmarks_path))
     *services_supported |= importer::FAVORITES;
-  if (PathExists(history_path))
+  if (file_util::PathExists(history_path))
     *services_supported |= importer::HISTORY;
 
   return *services_supported != importer::NONE;
 }
 
-void SafariImporter::StartImport(const importer::ProfileInfo& profile_info,
-                                 uint16 services_supported,
+void SafariImporter::StartImport(const importer::SourceProfile& source_profile,
+                                 uint16 items,
                                  ImporterBridge* bridge) {
   bridge_ = bridge;
   // The order here is important!
   bridge_->NotifyStarted();
+
   // In keeping with import on other platforms (and for other browsers), we
   // don't import the home page (since it may lead to a useless homepage); see
   // crbug.com/25603.
-  if ((services_supported & importer::HISTORY) && !cancelled()) {
+  if ((items & importer::HISTORY) && !cancelled()) {
     bridge_->NotifyItemStarted(importer::HISTORY);
     ImportHistory();
     bridge_->NotifyItemEnded(importer::HISTORY);
   }
-  if ((services_supported & importer::FAVORITES) && !cancelled()) {
+  if ((items & importer::FAVORITES) && !cancelled()) {
     bridge_->NotifyItemStarted(importer::FAVORITES);
     ImportBookmarks();
     bridge_->NotifyItemEnded(importer::FAVORITES);
   }
-  if ((services_supported & importer::PASSWORDS) && !cancelled()) {
+  if ((items & importer::PASSWORDS) && !cancelled()) {
     bridge_->NotifyItemStarted(importer::PASSWORDS);
     ImportPasswords();
     bridge_->NotifyItemEnded(importer::PASSWORDS);
@@ -104,100 +98,102 @@ void SafariImporter::StartImport(const importer::ProfileInfo& profile_info,
 }
 
 void SafariImporter::ImportBookmarks() {
+  string16 toolbar_name =
+      bridge_->GetLocalizedString(IDS_BOOKMARK_BAR_FOLDER_NAME);
   std::vector<ProfileWriter::BookmarkEntry> bookmarks;
-  ParseBookmarks(&bookmarks);
+  ParseBookmarks(toolbar_name, &bookmarks);
 
   // Write bookmarks into profile.
   if (!bookmarks.empty() && !cancelled()) {
-    const std::wstring& first_folder_name =
+    const string16& first_folder_name =
         bridge_->GetLocalizedString(IDS_BOOKMARK_GROUP_FROM_SAFARI);
-    int options = 0;
-    if (import_to_bookmark_bar())
-      options = ProfileWriter::IMPORT_TO_BOOKMARK_BAR;
-    bridge_->AddBookmarkEntries(bookmarks, first_folder_name, options);
+    bridge_->AddBookmarks(bookmarks, first_folder_name);
   }
 
   // Import favicons.
-  sqlite_utils::scoped_sqlite_db_ptr db(OpenFavIconDB());
+  sql::Connection db;
+  if (!OpenDatabase(&db))
+    return;
+
   FaviconMap favicon_map;
-  ImportFavIconURLs(db.get(), &favicon_map);
+  ImportFaviconURLs(&db, &favicon_map);
   // Write favicons into profile.
   if (!favicon_map.empty() && !cancelled()) {
-    std::vector<history::ImportedFavIconUsage> favicons;
-    LoadFaviconData(db.get(), favicon_map, &favicons);
-    bridge_->SetFavIcons(favicons);
+    std::vector<history::ImportedFaviconUsage> favicons;
+    LoadFaviconData(&db, favicon_map, &favicons);
+    bridge_->SetFavicons(favicons);
   }
 }
 
-sqlite3* SafariImporter::OpenFavIconDB() {
-  // Construct ~/Library/Safari/WebIcons.db path
+bool SafariImporter::OpenDatabase(sql::Connection* db) {
+  // Construct ~/Library/Safari/WebIcons.db path.
   NSString* library_dir = [NSString
       stringWithUTF8String:library_dir_.value().c_str()];
   NSString* safari_dir = [library_dir
       stringByAppendingPathComponent:@"Safari"];
   NSString* favicons_db_path = [safari_dir
-    stringByAppendingPathComponent:@"WebpageIcons.db"];
+      stringByAppendingPathComponent:@"WebpageIcons.db"];
 
-  sqlite3* favicons_db;
-  const char* safariicons_dbname = [favicons_db_path fileSystemRepresentation];
-  if (sqlite3_open(safariicons_dbname, &favicons_db) != SQLITE_OK)
-    return NULL;
-
-  return favicons_db;
+  const char* db_path = [favicons_db_path fileSystemRepresentation];
+  return db->Open(FilePath(db_path));
 }
 
-void SafariImporter::ImportFavIconURLs(sqlite3* db, FaviconMap* favicon_map) {
-  SQLStatement s;
-  const char* stmt = "SELECT iconID, url FROM PageURL;";
-  if (s.prepare(db, stmt) != SQLITE_OK)
+void SafariImporter::ImportFaviconURLs(sql::Connection* db,
+                                       FaviconMap* favicon_map) {
+  const char* query = "SELECT iconID, url FROM PageURL;";
+  sql::Statement s(db->GetUniqueStatement(query));
+  if (!s)
     return;
 
-  while (s.step() == SQLITE_ROW && !cancelled()) {
-    int64 icon_id = s.column_int(0);
-    GURL url = GURL(s.column_string(1));
+  while (s.Step() && !cancelled()) {
+    int64 icon_id = s.ColumnInt64(0);
+    GURL url = GURL(s.ColumnString(1));
     (*favicon_map)[icon_id].insert(url);
   }
 }
 
-void SafariImporter::LoadFaviconData(sqlite3* db,
-                                     const FaviconMap& favicon_map,
-                        std::vector<history::ImportedFavIconUsage>* favicons) {
-  SQLStatement s;
-  const char* stmt = "SELECT i.url, d.data "
-                     "FROM IconInfo i JOIN IconData d "
-                     "ON i.iconID = d.iconID "
-                     "WHERE i.iconID = ?;";
-  if (s.prepare(db, stmt) != SQLITE_OK)
+void SafariImporter::LoadFaviconData(
+    sql::Connection* db,
+    const FaviconMap& favicon_map,
+    std::vector<history::ImportedFaviconUsage>* favicons) {
+  const char* query = "SELECT i.url, d.data "
+                      "FROM IconInfo i JOIN IconData d "
+                      "ON i.iconID = d.iconID "
+                      "WHERE i.iconID = ?;";
+  sql::Statement s(db->GetUniqueStatement(query));
+  if (!s)
     return;
 
   for (FaviconMap::const_iterator i = favicon_map.begin();
        i != favicon_map.end(); ++i) {
-    s.bind_int64(0, i->first);
-    if (s.step() == SQLITE_ROW) {
-      history::ImportedFavIconUsage usage;
+    s.Reset();
+    s.BindInt64(0, i->first);
+    if (s.Step()) {
+      history::ImportedFaviconUsage usage;
 
-      usage.favicon_url = GURL(s.column_string(0));
+      usage.favicon_url = GURL(s.ColumnString(0));
       if (!usage.favicon_url.is_valid())
         continue;  // Don't bother importing favicons with invalid URLs.
 
       std::vector<unsigned char> data;
-      if (!s.column_blob_as_vector(1, &data) || data.empty())
+      s.ColumnBlobAsVector(1, &data);
+      if (data.empty())
         continue;  // Data definitely invalid.
 
-      if (!ReencodeFavicon(&data[0], data.size(), &usage.png_data))
+      if (!importer::ReencodeFavicon(&data[0], data.size(), &usage.png_data))
         continue;  // Unable to decode.
 
       usage.urls = i->second;
       favicons->push_back(usage);
     }
-    s.reset();
   }
 }
 
 void SafariImporter::RecursiveReadBookmarksFolder(
     NSDictionary* bookmark_folder,
-    const std::vector<std::wstring>& parent_path_elements,
+    const std::vector<string16>& parent_path_elements,
     bool is_in_toolbar,
+    const string16& toolbar_name,
     std::vector<ProfileWriter::BookmarkEntry>* out_bookmarks) {
   DCHECK(bookmark_folder);
 
@@ -213,32 +209,43 @@ void SafariImporter::RecursiveReadBookmarksFolder(
   if (!is_top_level_bookmarks_container) {
     // Top level containers sometimes don't have title attributes.
     if (![type isEqualToString:@"WebBookmarkTypeList"] || !title) {
-      DCHECK(false) << "Type =("
-      << (type ? base::SysNSStringToUTF8(type) : "Null Type")
-      << ") Title=(" << (title ? base::SysNSStringToUTF8(title) : "Null title")
-      << ")";
+      NOTREACHED() << "Type=("
+                   << (type ? base::SysNSStringToUTF8(type) : "Null type")
+                   << ") Title=("
+                   << (title ? base::SysNSStringToUTF8(title) : "Null title")
+                   << ")";
       return;
     }
   }
 
-  std::vector<std::wstring> path_elements(parent_path_elements);
-  // Is this the toolbar folder?
-  if ([title isEqualToString:@"BookmarksBar"]) {
-    // Be defensive, the toolbar items shouldn't have a prepended path.
-    path_elements.clear();
-    is_in_toolbar = true;
-  } else if ([title isEqualToString:@"BookmarksMenu"]) {
-    // top level container for normal bookmarks.
-    path_elements.clear();
-  } else if (!is_top_level_bookmarks_container) {
-    if (title)
-      path_elements.push_back(base::SysNSStringToWide(title));
+  NSArray* elements = [bookmark_folder objectForKey:@"Children"];
+  if (!elements && (!parent_path_elements.empty() || !is_in_toolbar)) {
+    // This is an empty folder, so add it explicitly; but don't add the toolbar
+    // folder if it is empty.  Note that all non-empty folders are added
+    // implicitly when their children are added.
+    ProfileWriter::BookmarkEntry entry;
+    // Safari doesn't specify a creation time for the folder.
+    entry.creation_time = base::Time::Now();
+    entry.title = base::SysNSStringToUTF16(title);
+    entry.path = parent_path_elements;
+    entry.in_toolbar = is_in_toolbar;
+    entry.is_folder = true;
+
+    out_bookmarks->push_back(entry);
+    return;
   }
 
-  NSArray* elements = [bookmark_folder objectForKey:@"Children"];
-  // TODO(jeremy) Does Chrome support importing empty folders?
-  if (!elements)
-    return;
+  std::vector<string16> path_elements(parent_path_elements);
+  // Create a folder for the toolbar, but not for the bookmarks menu.
+  if (path_elements.empty() && [title isEqualToString:@"BookmarksBar"]) {
+    is_in_toolbar = true;
+    path_elements.push_back(toolbar_name);
+  } else if (!is_top_level_bookmarks_container &&
+             !(path_elements.empty() &&
+               [title isEqualToString:@"BookmarksMenu"])) {
+    if (title)
+      path_elements.push_back(base::SysNSStringToUTF16(title));
+  }
 
   // Iterate over individual bookmarks.
   for (NSDictionary* bookmark in elements) {
@@ -251,11 +258,12 @@ void SafariImporter::RecursiveReadBookmarksFolder(
       RecursiveReadBookmarksFolder(bookmark,
                                    path_elements,
                                    is_in_toolbar,
+                                   toolbar_name,
                                    out_bookmarks);
     }
 
     // If we didn't see a bookmark folder, then we're expecting a bookmark
-    // item, if that's not what we got then ignore it.
+    // item.  If that's not what we got then ignore it.
     if (![type isEqualToString:@"WebBookmarkTypeLeaf"])
       continue;
 
@@ -270,7 +278,7 @@ void SafariImporter::RecursiveReadBookmarksFolder(
     ProfileWriter::BookmarkEntry entry;
     // Safari doesn't specify a creation time for the bookmark.
     entry.creation_time = base::Time::Now();
-    entry.title = base::SysNSStringToWide(title);
+    entry.title = base::SysNSStringToUTF16(title);
     entry.url = GURL(base::SysNSStringToUTF8(url));
     entry.path = path_elements;
     entry.in_toolbar = is_in_toolbar;
@@ -280,6 +288,7 @@ void SafariImporter::RecursiveReadBookmarksFolder(
 }
 
 void SafariImporter::ParseBookmarks(
+    const string16& toolbar_name,
     std::vector<ProfileWriter::BookmarkEntry>* bookmarks) {
   DCHECK(bookmarks);
 
@@ -298,9 +307,9 @@ void SafariImporter::ParseBookmarks(
     return;
 
   // Recursively read in bookmarks.
-  std::vector<std::wstring> parent_path_elements;
+  std::vector<string16> parent_path_elements;
   RecursiveReadBookmarksFolder(bookmarks_dict, parent_path_elements, false,
-                               bookmarks);
+                               toolbar_name, bookmarks);
 }
 
 void SafariImporter::ImportPasswords() {
@@ -326,7 +335,7 @@ double SafariImporter::HistoryTimeToEpochTime(NSString* history_time) {
   // Add Difference between Unix epoch and CFAbsoluteTime epoch in seconds.
   // Unix epoch is 1970-01-01 00:00:00.0 UTC,
   // CF epoch is 2001-01-01 00:00:00.0 UTC.
-  return CFStringGetDoubleValue(reinterpret_cast<CFStringRef>(history_time)) +
+  return CFStringGetDoubleValue(base::mac::NSToCFCast(history_time)) +
       kCFAbsoluteTimeIntervalSince1970;
 }
 
@@ -352,13 +361,11 @@ void SafariImporter::ParseHistoryItems(
       objectForKey:@"WebHistoryDates"];
 
   for (NSDictionary* history_item in safari_history_items) {
-    using base::SysNSStringToUTF8;
-    using base::SysNSStringToUTF16;
     NSString* url_ns = [history_item objectForKey:@""];
     if (!url_ns)
       continue;
 
-    GURL url(SysNSStringToUTF8(url_ns));
+    GURL url(base::SysNSStringToUTF8(url_ns));
 
     if (!CanImportSafariURL(url))
       continue;
@@ -371,7 +378,7 @@ void SafariImporter::ParseHistoryItems(
     if (!title_ns)
       title_ns = url_ns;
 
-    row.set_title(SysNSStringToUTF16(title_ns));
+    row.set_title(base::SysNSStringToUTF16(title_ns));
     int visit_count = [[history_item objectForKey:@"visitCount"]
                           intValue];
     row.set_visit_count(visit_count);
