@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,57 +6,24 @@
 
 #include <limits>
 
+#include "base/bind.h"
 #include "base/rand_util.h"
-#include "base/string_split.h"
-#include "base/string_util.h"
 #include "base/string_number_conversions.h"
+#include "base/string_split.h"
 #include "base/utf_string_conversions.h"
-#include "base/values.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/first_run/first_run.h"
 #include "chrome/browser/importer/importer_bridge.h"
 #include "chrome/browser/importer/importer_data_types.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/libxml_utils.h"
-#include "chrome/common/net/url_request_context_getter.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/common/url_fetcher.h"
 #include "grit/generated_resources.h"
-#include "net/base/cookie_monster.h"
-#include "net/base/data_url.h"
-#include "net/url_request/url_request_context.h"
+#include "net/base/load_flags.h"
 
-using importer::FAVORITES;
-using importer::NONE;
-using importer::ProfileInfo;
+using content::BrowserThread;
 
-//
-// ToolbarImporterUtils
-//
-static const char* kGoogleDomainUrl = "http://.google.com/";
-static const wchar_t kSplitStringToken = L';';
-static const char* kGoogleDomainSecureCookieId = "SID=";
-
-bool toolbar_importer_utils::IsGoogleGAIACookieInstalled() {
-  net::CookieStore* store =
-      Profile::GetDefaultRequestContext()->GetCookieStore();
-  GURL url(kGoogleDomainUrl);
-  net::CookieOptions options;
-  options.set_include_httponly();  // The SID cookie might be httponly.
-  std::string cookies = store->GetCookiesWithOptions(url, options);
-  std::vector<std::string> cookie_list;
-  base::SplitString(cookies, kSplitStringToken, &cookie_list);
-  for (std::vector<std::string>::iterator current = cookie_list.begin();
-       current != cookie_list.end();
-       ++current) {
-    size_t position = (*current).find(kGoogleDomainSecureCookieId);
-    if (0 == position)
-      return true;
-  }
-  return false;
-}
-
-//
 // Toolbar5Importer
-//
 const char Toolbar5Importer::kXmlApiReplyXmlTag[] = "xml_api_reply";
 const char Toolbar5Importer::kBookmarksXmlTag[] = "bookmarks";
 const char Toolbar5Importer::kBookmarkXmlTag[] = "bookmark";
@@ -99,13 +66,16 @@ Toolbar5Importer::~Toolbar5Importer() {
   DCHECK(!data_fetcher_);
 }
 
-void Toolbar5Importer::StartImport(const importer::ProfileInfo& profile_info,
-                                   uint16 items,
-                                   ImporterBridge* bridge) {
+void Toolbar5Importer::StartImport(
+    const importer::SourceProfile& source_profile,
+    uint16 items,
+    ImporterBridge* bridge) {
   DCHECK(bridge);
 
   bridge_ = bridge;
   items_to_import_ = items;
+  DCHECK(source_profile.request_context_getter);
+  request_context_getter_ = source_profile.request_context_getter;
   state_ = INITIALIZED;
 
   bridge_->NotifyStarted();
@@ -128,29 +98,25 @@ void Toolbar5Importer::Cancel() {
   } else {
     BrowserThread::PostTask(
         BrowserThread::UI, FROM_HERE,
-        NewRunnableMethod(this, &Toolbar5Importer::Cancel));
+        base::Bind(&Toolbar5Importer::Cancel, this));
   }
 }
 
-void Toolbar5Importer::OnURLFetchComplete(
-    const URLFetcher* source,
-    const GURL& url,
-    const net::URLRequestStatus& status,
-    int response_code,
-    const ResponseCookies& cookies,
-    const std::string& data) {
+void Toolbar5Importer::OnURLFetchComplete(const content::URLFetcher* source) {
   if (cancelled()) {
     EndImport();
     return;
   }
 
-  if (200 != response_code) {  // HTTP/Ok
+  if (200 != source->GetResponseCode()) {  // HTTP/Ok
     // Cancelling here will update the UI and bypass the rest of bookmark
     // import.
     EndImportBookmarks();
     return;
   }
 
+  std::string data;
+  source->GetResponseAsString(&data);
   switch (state_) {
     case GET_AUTHORIZATION_TOKEN:
       GetBookmarkDataFromServer(data);
@@ -245,8 +211,10 @@ void Toolbar5Importer::GetAuthenticationFromServer() {
                      random_string);
   GURL url(url_string);
 
-  token_fetcher_ = new  URLFetcher(url, URLFetcher::GET, this);
-  token_fetcher_->set_request_context(Profile::GetDefaultRequestContext());
+  token_fetcher_ = content::URLFetcher::Create(
+      url, content::URLFetcher::GET, this);
+  token_fetcher_->SetRequestContext(request_context_getter_.get());
+  token_fetcher_->SetLoadFlags(net::LOAD_DO_NOT_SAVE_COOKIES);
   token_fetcher_->Start();
 }
 
@@ -278,8 +246,10 @@ void Toolbar5Importer::GetBookmarkDataFromServer(const std::string& response) {
                       token);
   GURL url(conn_string);
 
-  data_fetcher_ = new URLFetcher(url, URLFetcher::GET, this);
-  data_fetcher_->set_request_context(Profile::GetDefaultRequestContext());
+  data_fetcher_ = content::URLFetcher::Create(
+      url, content::URLFetcher::GET, this);
+  data_fetcher_->SetRequestContext(request_context_getter_.get());
+  data_fetcher_->SetLoadFlags(net::LOAD_DO_NOT_SAVE_COOKIES);
   data_fetcher_->Start();
 }
 
@@ -297,8 +267,7 @@ void Toolbar5Importer::GetBookmarksFromServerDataResponse(
     // Construct Bookmarks
     std::vector<ProfileWriter::BookmarkEntry> bookmarks;
     if (ParseBookmarksFromReader(&reader, &bookmarks,
-        WideToUTF16(bridge_->GetLocalizedString(
-            IDS_BOOKMARK_GROUP_FROM_GOOGLE_TOOLBAR))))
+        bridge_->GetLocalizedString(IDS_BOOKMARK_GROUP_FROM_GOOGLE_TOOLBAR)))
       AddBookmarksToChrome(bookmarks);
   }
   EndImportBookmarks();
@@ -493,7 +462,7 @@ bool Toolbar5Importer::ExtractTitleFromXmlReader(
   if (!ExtractNamedValueFromXmlReader(reader, kTitleXmlTag, &buffer)) {
     return false;
   }
-  entry->title = UTF8ToWide(buffer);
+  entry->title = UTF8ToUTF16(buffer);
   return true;
 }
 
@@ -549,20 +518,20 @@ bool Toolbar5Importer::ExtractFoldersFromXmlReader(
   if (!reader->Read() || !LocateNextOpenTag(reader))
     return false;
 
-  std::vector<std::wstring> label_vector;
+  std::vector<string16> label_vector;
   while (kLabelXmlTag == reader->NodeName()) {
     std::string label_buffer;
     if (!reader->ReadElementContent(&label_buffer)) {
       label_buffer = "";
     }
-    label_vector.push_back(UTF8ToWide(label_buffer));
+    label_vector.push_back(UTF8ToUTF16(label_buffer));
     LocateNextOpenTag(reader);
   }
 
   if (0 == label_vector.size()) {
-    if (!FirstRun::IsChromeFirstRun()) {
+    if (!first_run::IsChromeFirstRun()) {
       bookmark_folders->resize(1);
-      (*bookmark_folders)[0].push_back(UTF16ToWide(bookmark_group_string));
+      (*bookmark_folders)[0].push_back(bookmark_group_string);
     }
     return true;
   }
@@ -573,15 +542,15 @@ bool Toolbar5Importer::ExtractFoldersFromXmlReader(
   for (size_t index = 0; index < label_vector.size(); ++index) {
     // If this is the first run then we place favorites with no labels
     // in the title bar.  Else they are placed in the "Google Toolbar" folder.
-    if (!FirstRun::IsChromeFirstRun() || !label_vector[index].empty()) {
-      (*bookmark_folders)[index].push_back(UTF16ToWide(bookmark_group_string));
+    if (!first_run::IsChromeFirstRun() || !label_vector[index].empty()) {
+      (*bookmark_folders)[index].push_back(bookmark_group_string);
     }
 
     // If the label and is in the form "xxx:yyy:zzz" this was created from an
     // IE or Firefox folder.  We undo the label creation and recreate the
     // correct folder.
-    std::vector<std::wstring> folder_names;
-    base::SplitString(label_vector[index], L':', &folder_names);
+    std::vector<string16> folder_names;
+    base::SplitString(label_vector[index], ':', &folder_names);
     (*bookmark_folders)[index].insert((*bookmark_folders)[index].end(),
         folder_names.begin(), folder_names.end());
   }
@@ -589,14 +558,11 @@ bool Toolbar5Importer::ExtractFoldersFromXmlReader(
   return true;
 }
 
-// Bookmark creation
 void  Toolbar5Importer::AddBookmarksToChrome(
     const std::vector<ProfileWriter::BookmarkEntry>& bookmarks) {
   if (!bookmarks.empty() && !cancelled()) {
-    const std::wstring& first_folder_name =
+    const string16& first_folder_name =
         bridge_->GetLocalizedString(IDS_BOOKMARK_GROUP_FROM_GOOGLE_TOOLBAR);
-    int options = ProfileWriter::ADD_IF_UNIQUE |
-        (import_to_bookmark_bar() ? ProfileWriter::IMPORT_TO_BOOKMARK_BAR : 0);
-    bridge_->AddBookmarkEntries(bookmarks, first_folder_name, options);
+    bridge_->AddBookmarks(bookmarks, first_folder_name);
   }
 }
