@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,23 +10,22 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
-#include "base/scoped_vector.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/scoped_vector.h"
+#include "base/memory/weak_ptr.h"
 #include "base/string16.h"
-#include "base/task.h"
-#include "base/timer.h"
 #include "chrome/browser/instant/instant_commit_type.h"
 #include "chrome/browser/instant/instant_loader_delegate.h"
 #include "chrome/browser/search_engines/template_url_id.h"
-#include "chrome/common/page_transition_types.h"
-#include "gfx/native_widget_types.h"
-#include "gfx/rect.h"
+#include "chrome/common/instant_types.h"
+#include "content/public/common/page_transition_types.h"
 #include "googleurl/src/gurl.h"
+#include "ui/gfx/native_widget_types.h"
+#include "ui/gfx/rect.h"
 
 struct AutocompleteMatch;
 class InstantDelegate;
 class InstantLoader;
-class InstantLoaderManager;
 class InstantTest;
 class PrefService;
 class Profile;
@@ -41,37 +40,16 @@ class TemplateURL;
 // by way of |DestroyPreviewContents|, which results in |HideInstant| being
 // invoked on the delegate. Similarly the preview may be committed at any time
 // by invoking |CommitCurrentPreview|, which results in |CommitInstant|
-// being invoked on the delegate.
+// being invoked on the delegate. Also see |PrepareForCommit| below.
 class InstantController : public InstantLoaderDelegate {
  public:
-  // Variations of instant support.
-  // TODO(sky): nuke these when we decide the default behavior.
-  enum Type {
-    // NOTE: these values are persisted to prefs. Don't change them!
-
-    FIRST_TYPE = 0,
-
-    // Search results are shown for the best guess of what we think the user was
-    // planning on typing.
-    PREDICTIVE_TYPE = 0,
-
-    // Search results are shown for exactly what was typed.
-    VERBATIM_TYPE,
-
-    // Variant of predictive that does not auto-complete after a delay.
-    PREDICTIVE_NO_AUTO_COMPLETE_TYPE,
-
-    LAST_TYPE = PREDICTIVE_NO_AUTO_COMPLETE_TYPE
-  };
-
-
   // Amount of time to wait before starting the instant animation.
   static const int kAutoCommitPauseTimeMS = 1000;
   // Duration of the instant animation in which the colors change.
   static const int kAutoCommitFadeInTimeMS = 300;
 
   InstantController(Profile* profile, InstantDelegate* delegate);
-  ~InstantController();
+  virtual ~InstantController();
 
   // Registers instant related preferences.
   static void RegisterUserPrefs(PrefService* prefs);
@@ -79,12 +57,9 @@ class InstantController : public InstantLoaderDelegate {
   // Records instant metrics.
   static void RecordMetrics(Profile* profile);
 
-  // Returns true if either type of instant is enabled.
+  // Returns true if instant is enabled.
   static bool IsEnabled(Profile* profile);
 
-  // Returns true if the specified type of instant is enabled.
-  static bool IsEnabled(Profile* profile, Type type);
-
   // Enables instant.
   static void Enable(Profile* profile);
 
@@ -95,14 +70,14 @@ class InstantController : public InstantLoaderDelegate {
   // Returns false if there is no instant preview showing.
   static bool CommitIfCurrent(InstantController* controller);
 
-  // Invoked as the user types in the omnibox with the url to navigate to.  If
-  // the url is empty and there is a preview TabContents it is destroyed. If url
-  // is non-empty and the preview TabContents has not been created it is
+  // Invoked as the user types in the omnibox with the url to navigate to. If
+  // the url is valid and a preview TabContents has not been created, it is
   // created. If |verbatim| is true search results are shown for |user_text|
   // rather than the best guess as to what the search thought the user meant.
   // |verbatim| only matters if the AutocompleteMatch is for a search engine
-  // that supports instant.
-  void Update(TabContentsWrapper* tab_contents,
+  // that supports instant. Returns true if the attempt to update does not
+  // result in the preview TabContents being destroyed.
+  bool Update(TabContentsWrapper* tab_contents,
               const AutocompleteMatch& match,
               const string16& user_text,
               bool verbatim,
@@ -113,19 +88,33 @@ class InstantController : public InstantLoaderDelegate {
   // when showing results for a search provider that supports instant.
   void SetOmniboxBounds(const gfx::Rect& bounds);
 
-  // Destroys the preview TabContents. Does nothing if the preview TabContents
-  // has not been created.
+  // Notifies the delegate to hide the preview and destroys the preview
+  // TabContents. Does nothing if the preview TabContents has not been created.
   void DestroyPreviewContents();
 
+  // Notifies the delegate to hide the preview but leaves it around in hopes it
+  // can be subsequently used. The preview will not be used until Update() (with
+  // valid parameters) is invoked.
+  void Hide();
+
   // Returns true if we're showing the last URL passed to |Update|. If this is
   // false a commit does not result in committing the last url passed to update.
   // A return value of false happens if we're in the process of determining if
   // the page supports instant.
-  bool IsCurrent();
+  bool IsCurrent() const;
+
+  // Returns true if the caller should proceed with committing the preview. A
+  // return value of false means that there is no valid preview to commit. This
+  // is used by Browser, when the user presses <Enter>, to decide whether to
+  // load the omnibox contents through Instant or otherwise. This is needed
+  // because calls to |Update| don't necessarily result in a preview being
+  // shown, such as in the HIDDEN and SILENT field trials.
+  bool PrepareForCommit();
 
   // Invoked when the user does some gesture that should trigger making the
-  // current previewed page the permanent page.
-  void CommitCurrentPreview(InstantCommitType type);
+  // current previewed page the permanent page.  Returns the TCW that contains
+  // the committed preview.
+  TabContentsWrapper* CommitCurrentPreview(InstantCommitType type);
 
   // Sets InstantController so that when the mouse is released the preview is
   // committed.
@@ -141,100 +130,79 @@ class InstantController : public InstantLoaderDelegate {
   // lost focus. Commit or discard the preview accordingly.
   void OnAutocompleteLostFocus(gfx::NativeView view_gaining_focus);
 
+  // The autocomplete edit has gained focus. Preload the instant URL of the
+  // default search engine, in anticipation of the user typing a query.
+  void OnAutocompleteGotFocus(TabContentsWrapper* tab_contents);
+
   // Releases the preview TabContents passing ownership to the caller. This is
   // intended to be called when the preview TabContents is committed. This does
-  // not notify the delegate.
+  // not notify the delegate. |tab_contents| is the underlying tab onto which
+  // the preview will be committed. It can be NULL when the underlying tab is
+  // irrelevant, for example when |type| is INSTANT_COMMIT_DESTROY.
   // WARNING: be sure and invoke CompleteRelease after adding the returned
   // TabContents to a tabstrip.
-  TabContentsWrapper* ReleasePreviewContents(InstantCommitType type);
+  TabContentsWrapper* ReleasePreviewContents(InstantCommitType type,
+                                             TabContentsWrapper* tab_contents);
 
   // Does cleanup after the preview contents has been added to the tabstrip.
   // Invoke this if you explicitly invoke ReleasePreviewContents.
-  void CompleteRelease(TabContents* tab);
+  void CompleteRelease(TabContentsWrapper* tab);
 
   // TabContents the match is being shown for.
   TabContentsWrapper* tab_contents() const { return tab_contents_; }
 
   // The preview TabContents; may be null.
-  TabContentsWrapper* GetPreviewContents();
-
-  // Returns true if |Update| has been invoked without a corresponding call to
-  // |DestroyPreviewContents| or |CommitCurrentPreview|.
-  bool is_active() const { return is_active_; }
+  TabContentsWrapper* GetPreviewContents() const;
 
   // Returns true if the preview TabContents is ready to be displayed. In some
   // situations this may return false yet GetPreviewContents() returns non-NULL.
   bool is_displayable() const { return is_displayable_; }
 
   // Returns the transition type of the last AutocompleteMatch passed to Update.
-  PageTransition::Type last_transition_type() const {
+  content::PageTransition last_transition_type() const {
     return last_transition_type_;
   }
 
-  // Returns true if we're showing results from a provider that supports the
-  // instant API. See description of |MightSupportInstant| for how this
-  // differs from actual loading state.
-  bool IsShowingInstant();
-
-  // Returns true if we're attempting to use the instant API with the last URL
-  // passed to |Update|. The value of this may change if it turns the provider
-  // doesn't really support the instant API.
-  // The value of |IsShowingInstant| indicates whether what is currently
-  // displayed supports instant, whereas this returns the loading state.  The
-  // state of |IsShowingInstant| differs when transitioning from a non-search
-  // provider to a search provider that supports instant (or the other way
-  // around). For example, if |Update| is passed www.foo.com, followed by a
-  // search string then this returns true, but |IsShowingInstant| returns false
-  // (until the search provider loads, then both return true).
-  bool MightSupportInstant();
-
   // InstantLoaderDelegate
-  virtual void ShowInstantLoader(InstantLoader* loader);
+  virtual void InstantStatusChanged(InstantLoader* loader) OVERRIDE;
   virtual void SetSuggestedTextFor(InstantLoader* loader,
-                                   const string16& text);
-  virtual gfx::Rect GetInstantBounds();
-  virtual bool ShouldCommitInstantOnMouseUp();
-  virtual void CommitInstantLoader(InstantLoader* loader);
-  virtual void InstantLoaderDoesntSupportInstant(InstantLoader* loader);
-  virtual void AddToBlacklist(InstantLoader* loader, const GURL& url);
+                                   const string16& text,
+                                   InstantCompleteBehavior behavior) OVERRIDE;
+  virtual gfx::Rect GetInstantBounds() OVERRIDE;
+  virtual bool ShouldCommitInstantOnMouseUp() OVERRIDE;
+  virtual void CommitInstantLoader(InstantLoader* loader) OVERRIDE;
+  virtual void InstantLoaderDoesntSupportInstant(
+      InstantLoader* loader) OVERRIDE;
+  virtual void AddToBlacklist(InstantLoader* loader,
+                              const GURL& url) OVERRIDE;
+  virtual void SwappedTabContents(InstantLoader* loader) OVERRIDE;
 
  private:
   friend class InstantTest;
 
   typedef std::set<std::string> HostBlacklist;
 
-  // Destroys the current loaders but remains actives.
-  void DestroyAndLeaveActive();
-
-  // Returns the TabContents of the pending loader (or NULL). This is only used
-  // for testing.
-  TabContentsWrapper* GetPendingPreviewContents();
-
-  // Returns true if we should update immediately.
-  bool ShouldUpdateNow(TemplateURLID instant_id, const GURL& url);
-
-  // Schedules a delayed update to load the specified url.
-  void ScheduleUpdate(const GURL& url);
-
-  // Invoked from the timer to process the last scheduled url.
-  void ProcessScheduledUpdate();
+  // Updates |is_displayable_| and if necessary notifies the delegate.
+  void UpdateIsDisplayable();
 
   // Updates InstantLoaderManager and its current InstantLoader. This is invoked
   // internally from Update.
   void UpdateLoader(const TemplateURL* template_url,
                     const GURL& url,
-                    PageTransition::Type transition_type,
+                    content::PageTransition transition_type,
                     const string16& user_text,
                     bool verbatim,
                     string16* suggested_text);
 
-  // Returns true if a preview should be shown for |match|. If |match| has
-  // a TemplateURL that supports the instant API it is set in |template_url|.
-  bool ShouldShowPreviewFor(const AutocompleteMatch& match,
-                            const TemplateURL** template_url);
+  // Returns true if instant should be used for the specified match. Instant is
+  // only used if |match| corresponds to the default search provider.
+  bool ShouldUseInstant(const AutocompleteMatch& match);
 
-  // Marks the specified search engine id as not supporting instant.
-  void BlacklistFromInstant(TemplateURLID id);
+  // Returns true if |template_url| is a valid TemplateURL for use by instant.
+  bool IsValidInstantTemplateURL(const TemplateURL* template_url);
+
+  // Marks the loader as not supporting instant.
+  void BlacklistFromInstant();
 
   // Returns true if the specified id has been blacklisted from supporting
   // instant.
@@ -253,30 +221,20 @@ class InstantController : public InstantLoaderDelegate {
   // Destroys all loaders scheduled for destruction in |ScheduleForDestroy|.
   void DestroyLoaders();
 
-  // Returns the TemplateURL to use for the specified AutocompleteMatch, or
-  // NULL if there is no TemplateURL for |match|.
-  const TemplateURL* GetTemplateURL(const AutocompleteMatch& match);
-
-  // If instant is enabled for the specified profile the type of instant is set
-  // in |type| and true is returned. Otherwise returns false.
-  static bool GetType(Profile* profile, Type* type);
-
-  // Returns a string description for the currently enabled type. This is used
-  // for histograms.
-  static std::string GetTypeString(Profile* profile);
-
   InstantDelegate* delegate_;
 
   // The TabContents last passed to |Update|.
   TabContentsWrapper* tab_contents_;
 
-  // See description above getter for details.
-  bool is_active_;
-
-  // Has notification been sent out that the preview TabContents is ready to be
-  // shown?
+  // True if |loader_| is ready to be displayed.
   bool is_displayable_;
 
+  // Set to true in Hide() and false in UpdateLoader(). Used when we persist
+  // the |loader_|, but it isn't up to date.
+  bool is_out_of_date_;
+
+  scoped_ptr<InstantLoader> loader_;
+
   // See description above setter.
   gfx::Rect omnibox_bounds_;
 
@@ -284,9 +242,7 @@ class InstantController : public InstantLoaderDelegate {
   bool commit_on_mouse_up_;
 
   // See description above getter.
-  PageTransition::Type last_transition_type_;
-
-  scoped_ptr<InstantLoaderManager> loader_manager_;
+  content::PageTransition last_transition_type_;
 
   // The IDs of any search engines that don't support instant. We assume all
   // search engines support instant, but if we determine an engine doesn't
@@ -294,22 +250,17 @@ class InstantController : public InstantLoaderDelegate {
   // reset/commit.
   std::set<TemplateURLID> blacklisted_ids_;
 
-  base::OneShotTimer<InstantController> update_timer_;
-
   // Used by ScheduleForDestroy; see it for details.
-  ScopedRunnableMethodFactory<InstantController> destroy_factory_;
-
-  // URL last pased to ScheduleUpdate.
-  GURL scheduled_url_;
-
-  Type type_;
+  base::WeakPtrFactory<InstantController> weak_factory_;
 
   // List of InstantLoaders to destroy. See ScheduleForDestroy for details.
   ScopedVector<InstantLoader> loaders_to_destroy_;
 
-  // The set of hosts that we don't use instant with. This is shared across all
-  // instances and only maintained for the current session.
-  static HostBlacklist* host_blacklist_;
+  // The URL of the most recent match passed to |Update|.
+  GURL last_url_;
+
+  // The most recent user_text passed to |Update|.
+  string16 last_user_text_;
 
   DISALLOW_COPY_AND_ASSIGN(InstantController);
 };
