@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #pragma once
 
 #include "base/string16.h"
+#include "chrome/common/instant_types.h"
 
 class TabContentsWrapper;
 
@@ -18,17 +19,10 @@ class Rect;
 // InstantController for details.
 class InstantDelegate {
  public:
-  // Invoked when instant starts loading, but before the preview tab contents is
-  // ready to be shown. This may be used to animate between the states.
-  // This is followed by ShowInstant and/or HideInstant.
-  virtual void PrepareForInstant() = 0;
-
   // Invoked when the instant TabContents should be shown.
   virtual void ShowInstant(TabContentsWrapper* preview_contents) = 0;
 
-  // Invoked when the instant TabContents should be hidden. Instant still may be
-  // active at the time this is invoked. Use |is_active()| to determine if
-  // instant is still active.
+  // Invoked when the instant TabContents should be hidden.
   virtual void HideInstant() = 0;
 
   // Invoked when the user does something that should result in the preview
@@ -37,7 +31,8 @@ class InstantDelegate {
   virtual void CommitInstant(TabContentsWrapper* preview_contents) = 0;
 
   // Invoked when the suggested text is to change to |text|.
-  virtual void SetSuggestedText(const string16& text) = 0;
+  virtual void SetSuggestedText(const string16& text,
+                                InstantCompleteBehavior behavior) = 0;
 
   // Returns the bounds instant will be placed at in screen coordinates.
   virtual gfx::Rect GetInstantBounds() = 0;
