@@ -1,22 +1,23 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/intranet_redirect_detector.h"
 
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/rand_util.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/prefs/pref_service.h"
-#include "chrome/browser/profiles/profile.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/notification_service.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/common/url_fetcher.h"
 #include "net/base/load_flags.h"
 #include "net/base/net_errors.h"
 #include "net/base/registry_controlled_domain.h"
+#include "net/url_request/url_request_context_getter.h"
 #include "net/url_request/url_request_status.h"
 
 const size_t IntranetRedirectDetector::kNumCharsInHostnames = 10;
@@ -24,29 +25,25 @@ const size_t IntranetRedirectDetector::kNumCharsInHostnames = 10;
 IntranetRedirectDetector::IntranetRedirectDetector()
     : redirect_origin_(g_browser_process->local_state()->GetString(
           prefs::kLastKnownIntranetRedirectOrigin)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(fetcher_factory_(this)),
-      in_sleep_(true),
-      request_context_available_(Profile::GetDefaultRequestContext() != NULL) {
-  registrar_.Add(this, NotificationType::DEFAULT_REQUEST_CONTEXT_AVAILABLE,
-                 NotificationService::AllSources());
-
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)),
+      in_sleep_(true) {
   // Because this function can be called during startup, when kicking off a URL
   // fetch can eat up 20 ms of time, we delay seven seconds, which is hopefully
   // long enough to be after startup, but still get results back quickly.
   // Ideally, instead of this timer, we'd do something like "check if the
   // browser is starting up, and if so, come back later", but there is currently
   // no function to do this.
-  static const int kStartFetchDelayMS = 7000;
+  static const int kStartFetchDelaySeconds = 7;
   MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      fetcher_factory_.NewRunnableMethod(
-          &IntranetRedirectDetector::FinishSleep),
-      kStartFetchDelayMS);
+      base::Bind(&IntranetRedirectDetector::FinishSleep,
+                 weak_factory_.GetWeakPtr()),
+      base::TimeDelta::FromSeconds(kStartFetchDelaySeconds));
 
-  net::NetworkChangeNotifier::AddObserver(this);
+  net::NetworkChangeNotifier::AddIPAddressObserver(this);
 }
 
 IntranetRedirectDetector::~IntranetRedirectDetector() {
-  net::NetworkChangeNotifier::RemoveObserver(this);
+  net::NetworkChangeNotifier::RemoveIPAddressObserver(this);
   STLDeleteElements(&fetchers_);
 }
 
@@ -70,16 +67,6 @@ void IntranetRedirectDetector::FinishSleep() {
   STLDeleteElements(&fetchers_);
   resulting_origins_.clear();
 
-  StartFetchesIfPossible();
-}
-
-void IntranetRedirectDetector::StartFetchesIfPossible() {
-  // Bail if a fetch isn't appropriate right now.  This function will be called
-  // again each time one of the preconditions changes, so we'll fetch
-  // immediately once all of them are met.
-  if (in_sleep_ || !request_context_available_)
-    return;
-
   // The detector is not needed in Chrome Frame since we have no omnibox there.
   const CommandLine* cmd_line = CommandLine::ForCurrentProcess();
   if (cmd_line->HasSwitch(switches::kDisableBackgroundNetworking) ||
@@ -94,32 +81,29 @@ void IntranetRedirectDetector::StartFetchesIfPossible() {
     for (size_t j = 0; j < kNumCharsInHostnames; ++j)
       url_string += ('a' + base::RandInt(0, 'z' - 'a'));
     GURL random_url(url_string + '/');
-    URLFetcher* fetcher = new URLFetcher(random_url, URLFetcher::HEAD, this);
+    content::URLFetcher* fetcher = content::URLFetcher::Create(
+        random_url, content::URLFetcher::HEAD, this);
     // We don't want these fetches to affect existing state in the profile.
-    fetcher->set_load_flags(net::LOAD_DISABLE_CACHE |
-                            net::LOAD_DO_NOT_SAVE_COOKIES);
-    fetcher->set_request_context(Profile::GetDefaultRequestContext());
+    fetcher->SetLoadFlags(net::LOAD_DISABLE_CACHE |
+                          net::LOAD_DO_NOT_SAVE_COOKIES);
+    fetcher->SetRequestContext(g_browser_process->system_request_context());
     fetcher->Start();
     fetchers_.insert(fetcher);
   }
 }
 
 void IntranetRedirectDetector::OnURLFetchComplete(
-    const URLFetcher* source,
-    const GURL& url,
-    const net::URLRequestStatus& status,
-    int response_code,
-    const ResponseCookies& cookies,
-    const std::string& data) {
+    const content::URLFetcher* source) {
   // Delete the fetcher on this function's exit.
-  Fetchers::iterator fetcher = fetchers_.find(const_cast<URLFetcher*>(source));
+  Fetchers::iterator fetcher = fetchers_.find(
+      const_cast<content::URLFetcher*>(source));
   DCHECK(fetcher != fetchers_.end());
-  scoped_ptr<URLFetcher> clean_up_fetcher(*fetcher);
+  scoped_ptr<content::URLFetcher> clean_up_fetcher(*fetcher);
   fetchers_.erase(fetcher);
 
   // If any two fetches result in the same domain/host, we set the redirect
   // origin to that; otherwise we set it to nothing.
-  if (!status.is_success() || (response_code != 200)) {
+  if (!source->GetStatus().is_success() || (source->GetResponseCode() != 200)) {
     if ((resulting_origins_.empty()) ||
         ((resulting_origins_.size() == 1) &&
          resulting_origins_.front().is_valid())) {
@@ -128,8 +112,8 @@ void IntranetRedirectDetector::OnURLFetchComplete(
     }
     redirect_origin_ = GURL();
   } else {
-    DCHECK(url.is_valid());
-    GURL origin(url.GetOrigin());
+    DCHECK(source->GetURL().is_valid());
+    GURL origin(source->GetURL().GetOrigin());
     if (resulting_origins_.empty()) {
       resulting_origins_.push_back(origin);
       return;
@@ -158,14 +142,6 @@ void IntranetRedirectDetector::OnURLFetchComplete(
           redirect_origin_.spec() : std::string());
 }
 
-void IntranetRedirectDetector::Observe(NotificationType type,
-                                       const NotificationSource& source,
-                                       const NotificationDetails& details) {
-  DCHECK_EQ(NotificationType::DEFAULT_REQUEST_CONTEXT_AVAILABLE, type.value);
-  request_context_available_ = true;
-  StartFetchesIfPossible();
-}
-
 void IntranetRedirectDetector::OnIPAddressChanged() {
   // If a request is already scheduled, do not scheduled yet another one.
   if (in_sleep_)
@@ -176,9 +152,9 @@ void IntranetRedirectDetector::OnIPAddressChanged() {
   in_sleep_ = true;
   static const int kNetworkSwitchDelayMS = 1000;
   MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      fetcher_factory_.NewRunnableMethod(
-          &IntranetRedirectDetector::FinishSleep),
-      kNetworkSwitchDelayMS);
+      base::Bind(&IntranetRedirectDetector::FinishSleep,
+                 weak_factory_.GetWeakPtr()),
+      base::TimeDelta::FromMilliseconds(kNetworkSwitchDelayMS));
 }
 
 IntranetRedirectHostResolverProc::IntranetRedirectHostResolverProc(
