@@ -10,9 +10,10 @@
 #include <string>
 #include <vector>
 
-#include "chrome/common/net/url_fetcher.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "base/memory/weak_ptr.h"
+#include "content/public/common/url_fetcher_delegate.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/host_resolver_proc.h"
 #include "net/base/network_change_notifier.h"
@@ -35,9 +36,9 @@ class PrefService;
 // Consumers should call RedirectOrigin(), which is guaranteed to synchronously
 // return a value at all times (even during startup or in unittest mode).  If no
 // redirection is in place, the returned GURL will be empty.
-class IntranetRedirectDetector : public URLFetcher::Delegate,
-                                 public NotificationObserver,
-                                 public net::NetworkChangeNotifier::Observer {
+class IntranetRedirectDetector
+    : public content::URLFetcherDelegate,
+      public net::NetworkChangeNotifier::IPAddressObserver {
  public:
   // Only the main browser process loop should call this, when setting up
   // g_browser_process->intranet_redirect_detector_.  No code other than the
@@ -46,7 +47,7 @@ class IntranetRedirectDetector : public URLFetcher::Delegate,
   // since there aren't useful public functions on this object for consumers to
   // access anyway).
   IntranetRedirectDetector();
-  ~IntranetRedirectDetector();
+  virtual ~IntranetRedirectDetector();
 
   // Returns the current redirect origin.  This will be empty if no redirection
   // is in place.
@@ -59,43 +60,25 @@ class IntranetRedirectDetector : public URLFetcher::Delegate,
   static const size_t kNumCharsInHostnames;
 
  private:
-  typedef std::set<URLFetcher*> Fetchers;
+  typedef std::set<content::URLFetcher*> Fetchers;
 
   // Called when the seven second startup sleep or the one second network
   // switch sleep has finished.  Runs any pending fetch.
   void FinishSleep();
 
-  // Starts the fetches to determine the redirect URL if we can currently do so.
-  void StartFetchesIfPossible();
+  // content::URLFetcherDelegate
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
-  // URLFetcher::Delegate
-  virtual void OnURLFetchComplete(const URLFetcher* source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  // NetworkChangeNotifier::IPAddressObserver
+  virtual void OnIPAddressChanged() OVERRIDE;
 
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
-
-  // NetworkChangeNotifier::Observer
-  virtual void OnIPAddressChanged();
-
-  NotificationRegistrar registrar_;
   GURL redirect_origin_;
-  ScopedRunnableMethodFactory<IntranetRedirectDetector> fetcher_factory_;
+  base::WeakPtrFactory<IntranetRedirectDetector> weak_factory_;
   Fetchers fetchers_;
   std::vector<GURL> resulting_origins_;
   bool in_sleep_;  // True if we're in the seven-second "no fetching" period
                    // that begins at browser start, or the one-second "no
                    // fetching" period that begins after network switches.
-  bool request_context_available_;
-                           // True when the profile has been loaded and the
-                           // default request context created, so we can
-                           // actually do the fetch with the right data.
 
   DISALLOW_COPY_AND_ASSIGN(IntranetRedirectDetector);
 };
@@ -110,7 +93,7 @@ class IntranetRedirectHostResolverProc : public net::HostResolverProc {
                       net::AddressFamily address_family,
                       net::HostResolverFlags host_resolver_flags,
                       net::AddressList* addrlist,
-                      int* os_error);
+                      int* os_error) OVERRIDE;
 };
 
 #endif  // CHROME_BROWSER_INTRANET_REDIRECT_DETECTOR_H_
