@@ -7,19 +7,20 @@
 #include "chrome/browser/jankometer.h"
 
 #include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/command_line.h"
+#include "base/memory/ref_counted.h"
 #include "base/message_loop.h"
 #include "base/metrics/histogram.h"
 #include "base/metrics/stats_counters.h"
-#include "base/ref_counted.h"
 #include "base/string_util.h"
 #include "base/threading/thread.h"
 #include "base/threading/watchdog.h"
 #include "base/time.h"
 #include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/common/chrome_switches.h"
+#include "content/public/browser/browser_thread.h"
 
 #if defined(TOOLKIT_USES_GTK)
 #include "chrome/browser/ui/gtk/gtk_util.h"
@@ -27,6 +28,7 @@
 
 using base::TimeDelta;
 using base::TimeTicks;
+using content::BrowserThread;
 
 namespace {
 
@@ -119,8 +121,8 @@ class JankObserverHelper {
   // Counters for the two types of jank we measure.
   base::StatsCounter slow_processing_counter_;  // Msgs w/ long proc time.
   base::StatsCounter queueing_delay_counter_;   // Msgs w/ long queueing delay.
-  scoped_refptr<base::Histogram> process_times_;  // Time spent proc. task.
-  scoped_refptr<base::Histogram> total_times_;  // Total queueing plus proc.
+  base::Histogram* const process_times_;  // Time spent proc. task.
+  base::Histogram* const total_times_;  // Total queueing plus proc.
   JankWatchdog total_time_watchdog_;  // Watching for excessive total_time.
 
   DISALLOW_COPY_AND_ASSIGN(JankObserverHelper);
@@ -135,13 +137,13 @@ JankObserverHelper::JankObserverHelper(
       events_till_measurement_(0),
       slow_processing_counter_(std::string("Chrome.SlowMsg") + thread_name),
       queueing_delay_counter_(std::string("Chrome.DelayMsg") + thread_name),
+      process_times_(base::Histogram::FactoryGet(
+          std::string("Chrome.ProcMsgL ") + thread_name,
+          1, 3600000, 50, base::Histogram::kUmaTargetedHistogramFlag)),
+      total_times_(base::Histogram::FactoryGet(
+          std::string("Chrome.TotalMsgL ") + thread_name,
+          1, 3600000, 50, base::Histogram::kUmaTargetedHistogramFlag)),
       total_time_watchdog_(excessive_duration, thread_name, watchdog_enable) {
-  process_times_ = base::Histogram::FactoryGet(
-      std::string("Chrome.ProcMsgL ") + thread_name,
-      1, 3600000, 50, base::Histogram::kUmaTargetedHistogramFlag);
-  total_times_ = base::Histogram::FactoryGet(
-      std::string("Chrome.TotalMsgL ") + thread_name,
-      1, 3600000, 50, base::Histogram::kUmaTargetedHistogramFlag);
   if (discard_count_ > 0) {
     // Select a vaguely random sample-start-point.
     events_till_measurement_ = static_cast<int>(
@@ -235,25 +237,25 @@ class IOJankObserver : public base::RefCountedThreadSafe<IOJankObserver>,
     MessageLoop::current()->RemoveTaskObserver(this);
   }
 
-  virtual void WillProcessIOEvent() {
+  virtual void WillProcessIOEvent() OVERRIDE {
     if (!helper_.MessageWillBeMeasured())
       return;
     helper_.StartProcessingTimers(base::TimeDelta());
   }
 
-  virtual void DidProcessIOEvent() {
+  virtual void DidProcessIOEvent() OVERRIDE {
     helper_.EndProcessingTimers();
   }
 
-  virtual void WillProcessTask(const Task* task) {
+  virtual void WillProcessTask(base::TimeTicks time_posted) OVERRIDE {
     if (!helper_.MessageWillBeMeasured())
       return;
     base::TimeTicks now = base::TimeTicks::Now();
-    const base::TimeDelta queueing_time = now - task->tracked_birth_time();
+    const base::TimeDelta queueing_time = now - time_posted;
     helper_.StartProcessingTimers(queueing_time);
   }
 
-  virtual void DidProcessTask(const Task* task) {
+  virtual void DidProcessTask(base::TimeTicks time_posted) OVERRIDE {
     helper_.EndProcessingTimers();
   }
 
@@ -291,22 +293,23 @@ class UIJankObserver : public base::RefCountedThreadSafe<UIJankObserver>,
     MessageLoopForUI::current()->RemoveObserver(this);
   }
 
-  virtual void WillProcessTask(const Task* task) {
+  virtual void WillProcessTask(base::TimeTicks time_posted) OVERRIDE {
     if (!helper_.MessageWillBeMeasured())
       return;
     base::TimeTicks now = base::TimeTicks::Now();
-    const base::TimeDelta queueing_time = now - task->tracked_birth_time();
+    const base::TimeDelta queueing_time = now - time_posted;
     helper_.StartProcessingTimers(queueing_time);
   }
 
-  virtual void DidProcessTask(const Task* task) {
+  virtual void DidProcessTask(base::TimeTicks time_posted) OVERRIDE {
     helper_.EndProcessingTimers();
   }
 
 #if defined(OS_WIN)
-  virtual void WillProcessMessage(const MSG& msg) {
+  virtual base::EventStatus WillProcessEvent(
+      const base::NativeEvent& event) OVERRIDE {
     if (!helper_.MessageWillBeMeasured())
-      return;
+      return base::EVENT_CONTINUE;
     // GetMessageTime returns a LONG (signed 32-bit) and GetTickCount returns
     // a DWORD (unsigned 32-bit). They both wrap around when the time is longer
     // than they can hold. I'm not sure if GetMessageTime wraps around to 0,
@@ -316,17 +319,26 @@ class UIJankObserver : public base::RefCountedThreadSafe<UIJankObserver>,
     // Therefore, I cast to DWORD so if it wraps to -1 we will correct it. If
     // it doesn't, then our time delta will be negative if a message happens
     // to straddle the wraparound point, it will still be OK.
-    DWORD cur_message_issue_time = static_cast<DWORD>(msg.time);
+    DWORD cur_message_issue_time = static_cast<DWORD>(event.time);
     DWORD cur_time = GetTickCount();
     base::TimeDelta queueing_time =
         base::TimeDelta::FromMilliseconds(cur_time - cur_message_issue_time);
 
     helper_.StartProcessingTimers(queueing_time);
+    return base::EVENT_CONTINUE;
   }
 
-  virtual void DidProcessMessage(const MSG& msg) {
+  virtual void DidProcessEvent(const base::NativeEvent& event) OVERRIDE {
     helper_.EndProcessingTimers();
   }
+#elif defined(USE_AURA)
+  virtual base::EventStatus WillProcessEvent(
+      const base::NativeEvent& event) OVERRIDE {
+    return base::EVENT_CONTINUE;
+  }
+
+  virtual void DidProcessEvent(const base::NativeEvent& event) OVERRIDE {
+  }
 #elif defined(TOOLKIT_USES_GTK)
   virtual void WillProcessEvent(GdkEvent* event) {
     if (!helper_.MessageWillBeMeasured())
@@ -398,8 +410,7 @@ void InstallJankometer(const CommandLine& parsed_command_line) {
           io_watchdog_enabled));
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(io_observer->get(),
-                        &IOJankObserver::AttachToCurrentThread));
+      base::Bind(&IOJankObserver::AttachToCurrentThread, io_observer->get()));
 }
 
 void UninstallJankometer() {
