@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,40 +12,46 @@
 #include <string>
 #include <vector>
 
-#include "base/callback.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/command_line.h"
 #include "base/file_util.h"
 #include "base/path_service.h"
-#include "base/scoped_comptr_win.h"
 #include "base/string_util.h"
 #include "base/threading/thread.h"
 #include "base/utf_string_conversions.h"
+#include "base/win/scoped_comptr.h"
 #include "base/win/windows_version.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/favicon_service.h"
+#include "chrome/browser/favicon/favicon_service.h"
 #include "chrome/browser/history/history.h"
 #include "chrome/browser/history/page_usage_data.h"
+#include "chrome/browser/history/top_sites.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sessions/session_types.h"
 #include "chrome/browser/sessions/tab_restore_service.h"
+#include "chrome/browser/sessions/tab_restore_service_factory.h"
 #include "chrome/browser/shell_integration.h"
 #include "chrome/common/chrome_constants.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/url_constants.h"
-#include "gfx/codec/png_codec.h"
-#include "gfx/icon_util.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/notification_source.h"
 #include "googleurl/src/gurl.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/gfx/codec/png_codec.h"
+#include "ui/gfx/icon_util.h"
 
+using content::BrowserThread;
 
 namespace {
 
 // COM interfaces used in this file.
 // These interface declarations are copied from Windows SDK 7.0.
-// TODO(hbono): Bug 16903: to be deleted them when we use Windows SDK 7.0.
+// TODO(hbono): Bug 16903: delete them when we use Windows SDK 7.0.
 #ifndef __IObjectArray_INTERFACE_DEFINED__
 #define __IObjectArray_INTERFACE_DEFINED__
 
@@ -117,7 +123,7 @@ ICustomDestinationList : public IUnknown {
 // Class IDs used in this file.
 // These class IDs must be defined in an anonymous namespace to avoid
 // conflicts with ones defined in "shell32.lib" of Visual Studio 2008.
-// TODO(hbono): Bug 16903: to be deleted them when we use Windows SDK 7.0.
+// TODO(hbono): Bug 16903: delete them when we use Windows SDK 7.0.
 const CLSID CLSID_DestinationList = {
   0x77f10cf0, 0x3db5, 0x4966, {0xb5, 0x20, 0xb7, 0xc5, 0x4f, 0xd3, 0x5e, 0xd6}
 };
@@ -170,12 +176,12 @@ class PropVariantString {
 // An IShellLink object is almost the same as an application shortcut, and it
 // requires three items: the absolute path to an application, an argument
 // string, and a title string.
-HRESULT AddShellLink(ScopedComPtr<IObjectCollection> collection,
+HRESULT AddShellLink(base::win::ScopedComPtr<IObjectCollection> collection,
                      const std::wstring& application,
                      const std::wstring& switches,
                      scoped_refptr<ShellLinkItem> item) {
   // Create an IShellLink object.
-  ScopedComPtr<IShellLink> link;
+  base::win::ScopedComPtr<IShellLink> link;
   HRESULT result = link.CreateInstance(CLSID_ShellLink, NULL,
                                        CLSCTX_INPROC_SERVER);
   if (FAILED(result))
@@ -220,7 +226,7 @@ HRESULT AddShellLink(ScopedComPtr<IObjectCollection> collection,
   // 4. Call the IPropertyStore::SetValue() function to Set the title property
   //    of the IShellLink object.
   // 5. Commit the transaction.
-  ScopedComPtr<IPropertyStore> property_store;
+  base::win::ScopedComPtr<IPropertyStore> property_store;
   result = link.QueryInterface(property_store.Receive());
   if (FAILED(result))
     return result;
@@ -281,7 +287,7 @@ bool CreateIconFile(const SkBitmap& bitmap,
 //   switches, use an empty string.
 // * data (ShellLinkItemList)
 //   A list of ShellLinkItem objects to be added under the specified category.
-HRESULT UpdateCategory(ScopedComPtr<ICustomDestinationList> list,
+HRESULT UpdateCategory(base::win::ScopedComPtr<ICustomDestinationList> list,
                        int category_id,
                        const std::wstring& application,
                        const std::wstring& switches,
@@ -298,7 +304,7 @@ HRESULT UpdateCategory(ScopedComPtr<ICustomDestinationList> list,
   // Create an EnumerableObjectCollection object.
   // We once add the given items to this collection object and add this
   // collection to the JumpList.
-  ScopedComPtr<IObjectCollection> collection;
+  base::win::ScopedComPtr<IObjectCollection> collection;
   HRESULT result = collection.CreateInstance(CLSID_EnumerableObjectCollection,
                                              NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(result))
@@ -317,7 +323,7 @@ HRESULT UpdateCategory(ScopedComPtr<ICustomDestinationList> list,
   // and use it.
   // It seems the ICustomDestinationList::AppendCategory() function just
   // replaces all items in the given category with the ones in the new list.
-  ScopedComPtr<IObjectArray> object_array;
+  base::win::ScopedComPtr<IObjectArray> object_array;
   result = collection.QueryInterface(object_array.Receive());
   if (FAILED(result))
     return false;
@@ -332,12 +338,12 @@ HRESULT UpdateCategory(ScopedComPtr<ICustomDestinationList> list,
 //   We should use AddUserTasks() instead of AppendCategory().
 // * The items in the "Task" category are static.
 //   We don't have to use a list.
-HRESULT UpdateTaskCategory(ScopedComPtr<ICustomDestinationList> list,
+HRESULT UpdateTaskCategory(base::win::ScopedComPtr<ICustomDestinationList> list,
                            const std::wstring& chrome_path,
                            const std::wstring& chrome_switches) {
   // Create an EnumerableObjectCollection object to be added items of the
   // "Task" category. (We can also use this object for the "Task" category.)
-  ScopedComPtr<IObjectCollection> collection;
+  base::win::ScopedComPtr<IObjectCollection> collection;
   HRESULT result = collection.CreateInstance(CLSID_EnumerableObjectCollection,
                                              NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(result))
@@ -372,7 +378,7 @@ HRESULT UpdateTaskCategory(ScopedComPtr<ICustomDestinationList> list,
   // ICustomDestinationList::AddUserTasks() also uses the IObjectArray
   // interface to retrieve each item in the list. So, we retrieve the
   // IObjectArray interface from the EnumerableObjectCollection object.
-  ScopedComPtr<IObjectArray> object_array;
+  base::win::ScopedComPtr<IObjectArray> object_array;
   result = collection.QueryInterface(object_array.Receive());
   if (FAILED(result))
     return result;
@@ -396,7 +402,7 @@ bool UpdateJumpList(const wchar_t* app_id,
     return true;
 
   // Create an ICustomDestinationList object and attach it to our application.
-  ScopedComPtr<ICustomDestinationList> destination_list;
+  base::win::ScopedComPtr<ICustomDestinationList> destination_list;
   HRESULT result = destination_list.CreateInstance(CLSID_DestinationList, NULL,
                                                    CLSCTX_INPROC_SERVER);
   if (FAILED(result))
@@ -411,7 +417,7 @@ bool UpdateJumpList(const wchar_t* app_id,
   // It seems Windows 7 RC (Build 7100) automatically checks the items in this
   // removed list and prevent us from adding the same item.
   UINT max_slots;
-  ScopedComPtr<IObjectArray> removed;
+  base::win::ScopedComPtr<IObjectArray> removed;
   result = destination_list->BeginList(&max_slots, __uuidof(*removed),
                                        reinterpret_cast<void**>(&removed));
   if (FAILED(result))
@@ -429,7 +435,7 @@ bool UpdateJumpList(const wchar_t* app_id,
   if (!user_data_dir.empty())
     command_line.AppendSwitchPath(switches::kUserDataDir, user_data_dir);
 
-  std::wstring chrome_switches = command_line.command_line_string();
+  std::wstring chrome_switches = command_line.GetCommandLineString();
 
   // We allocate 60% of the given JumpList slots to "most-visited" items
   // and 40% to "recently-closed" items, respectively.
@@ -475,109 +481,15 @@ bool UpdateJumpList(const wchar_t* app_id,
   return true;
 }
 
-// Represents a task which updates an application JumpList.
-// This task encapsulates all I/O tasks and OS-specific tasks required for
-// updating a JumpList from Chromium, such as:
-// * Deleting the directory containing temporary icon files;
-// * Creating temporary icon files used by the JumpList;
-// * Creating an ICustomDestinationList instance;
-// * Adding items in the ICustomDestinationList instance.
-// To spawn this task,
-// 1. Prepare objects required by this task:
-//    * a std::wstring that contains a temporary icons;
-//    * a ShellLinkItemList that contains the items of the "Most Visited"
-//      category, and;
-//    * a ShellLinkItemList that contains the items of the "Recently Closed"
-//      category.
-// 2. Create a JumpListUpdateTask instance, and;
-// 3. Post this task to the file thread.
-class JumpListUpdateTask : public Task {
- public:
-  JumpListUpdateTask(const wchar_t* app_id,
-                     const FilePath& icon_dir,
-                     const ShellLinkItemList& most_visited_pages,
-                     const ShellLinkItemList& recently_closed_pages)
-    : app_id_(app_id),
-      icon_dir_(icon_dir),
-      most_visited_pages_(most_visited_pages),
-      recently_closed_pages_(recently_closed_pages) {
-  }
-
- private:
-  // Represents an entry point of this task.
-  // When we post this task to a file thread, the thread calls this function.
-  void Run();
-
-  // App id to associate with the jump list.
-  std::wstring app_id_;
-
-  // The directory which contains JumpList icons.
-  FilePath icon_dir_;
-
-  // Items in the "Most Visited" category of the application JumpList.
-  ShellLinkItemList most_visited_pages_;
-
-  // Items in the "Recently Closed" category of the application JumpList.
-  ShellLinkItemList recently_closed_pages_;
-};
-
-void JumpListUpdateTask::Run() {
-  // Delete the directory which contains old icon files, rename the current
-  // icon directory, and create a new directory which contains new JumpList
-  // icon files.
-  FilePath icon_dir_old(icon_dir_.value() + L"Old");
-  if (file_util::PathExists(icon_dir_old))
-    file_util::Delete(icon_dir_old, true);
-  file_util::Move(icon_dir_, icon_dir_old);
-  file_util::CreateDirectory(icon_dir_);
-
-  // Create temporary icon files for shortcuts in the "Most Visited" category.
-  for (ShellLinkItemList::const_iterator item = most_visited_pages_.begin();
-       item != most_visited_pages_.end(); ++item) {
-    SkBitmap icon_bitmap;
-    if ((*item)->data().get() &&
-        gfx::PNGCodec::Decode((*item)->data()->front(),
-                              (*item)->data()->size(),
-                              &icon_bitmap)) {
-      FilePath icon_path;
-      if (CreateIconFile(icon_bitmap, icon_dir_, &icon_path))
-        (*item)->SetIcon(icon_path.value(), 0, true);
-    }
-  }
-
-  // Create temporary icon files for shortcuts in the "Recently Closed"
-  // category.
-  for (ShellLinkItemList::const_iterator item = recently_closed_pages_.begin();
-       item != recently_closed_pages_.end(); ++item) {
-    SkBitmap icon_bitmap;
-    if ((*item)->data().get() &&
-        gfx::PNGCodec::Decode((*item)->data()->front(),
-                              (*item)->data()->size(),
-                              &icon_bitmap)) {
-      FilePath icon_path;
-      if (CreateIconFile(icon_bitmap, icon_dir_, &icon_path))
-        (*item)->SetIcon(icon_path.value(), 0, true);
-    }
-  }
-
-  // We finished collecting all resources needed for updating an appliation
-  // JumpList. So, create a new JumpList and replace the current JumpList
-  // with it.
-  UpdateJumpList(app_id_.c_str(), most_visited_pages_, recently_closed_pages_);
-
-  // Delete all items in these lists now since we don't need the ShellLinkItem
-  // objects in these lists.
-  most_visited_pages_.clear();
-  recently_closed_pages_.clear();
-}
-
 }  // namespace
 
-JumpList::JumpList() : profile_(NULL) {
+JumpList::JumpList()
+    : profile_(NULL),
+      handle_(NULL) {
 }
 
 JumpList::~JumpList() {
-  RemoveObserver();
+  Terminate();
 }
 
 // static
@@ -596,42 +508,153 @@ bool JumpList::AddObserver(Profile* profile) {
   if (base::win::GetVersion() < base::win::VERSION_WIN7 || !profile)
     return false;
 
-  TabRestoreService* tab_restore_service = profile->GetTabRestoreService();
+  TabRestoreService* tab_restore_service =
+      TabRestoreServiceFactory::GetForProfile(profile);
   if (!tab_restore_service)
     return false;
 
   app_id_ = ShellIntegration::GetChromiumAppId(profile->GetPath());
   icon_dir_ = profile->GetPath().Append(chrome::kJumpListIconDirname);
   profile_ = profile;
+  history::TopSites* top_sites = profile_->GetTopSites();
+  if (top_sites) {
+    // TopSites updates itself after a delay. This is especially noticable when
+    // your profile is empty. Ask TopSites to update itself when jumplist is
+    // initialized.
+    top_sites->SyncWithHistory();
+    // Register for notification when TopSites changes so that we can update
+    // ourself.
+    registrar_.Add(this, chrome::NOTIFICATION_TOP_SITES_CHANGED,
+                   content::Source<history::TopSites>(top_sites));
+    // Register for notification when profile is destroyed to ensure that all
+    // observers are detatched at that time.
+    registrar_.Add(this, chrome::NOTIFICATION_PROFILE_DESTROYED,
+                   content::Source<Profile>(profile_));
+  }
   tab_restore_service->AddObserver(this);
   return true;
 }
 
+void JumpList::Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_TOP_SITES_CHANGED: {
+      // Most visited urls changed, query again.
+      history::TopSites* top_sites = profile_->GetTopSites();
+      if (top_sites) {
+        top_sites->GetMostVisitedURLs(
+            &topsites_consumer_,
+            base::Bind(&JumpList::OnMostVisitedURLsAvailable,
+                       base::Unretained(this)));
+      }
+      break;
+    }
+    case chrome::NOTIFICATION_PROFILE_DESTROYED: {
+      // Profile was destroyed, do clean-up.
+      Terminate();
+      break;
+    }
+    default:
+      NOTREACHED() << "Unexpected notification type.";
+  }
+}
+
 void JumpList::RemoveObserver() {
-  if (profile_ && profile_->GetTabRestoreService())
-    profile_->GetTabRestoreService()->RemoveObserver(this);
+  if (profile_) {
+    TabRestoreService* tab_restore_service =
+        TabRestoreServiceFactory::GetForProfile(profile_);
+    if (tab_restore_service)
+      tab_restore_service->RemoveObserver(this);
+    registrar_.Remove(
+        this, chrome::NOTIFICATION_TOP_SITES_CHANGED,
+        content::Source<history::TopSites>(profile_->GetTopSites()));
+    registrar_.Remove(
+        this, chrome::NOTIFICATION_PROFILE_DESTROYED,
+        content::Source<Profile>(profile_));
+  }
   profile_ = NULL;
 }
 
-void JumpList::TabRestoreServiceChanged(TabRestoreService* service) {
-  // Added or removed a tab.
-  // Exit if we are updating the application JumpList.
-  if (!icon_urls_.empty())
-    return;
+void JumpList::CancelPendingUpdate() {
+  if (handle_) {
+    FaviconService* favicon_service =
+        profile_->GetFaviconService(Profile::EXPLICIT_ACCESS);
+    favicon_service->CancelRequest(handle_);
+    handle_ = NULL;
+  }
+}
 
-  // Send a query to HistoryService and retrieve the "Most Visited" pages.
-  // This code is copied from MostVisitedHandler::HandleGetMostVisited() to
-  // emulate its behaviors.
-  const int kMostVisitedScope = 90;
-  const int kMostVisitedCount = 9;
-  int result_count = kMostVisitedCount;
-  HistoryService* history_service =
-      profile_->GetHistoryService(Profile::EXPLICIT_ACCESS);
-  history_service->QuerySegmentUsageSince(
-      &most_visited_consumer_,
-      base::Time::Now() - base::TimeDelta::FromDays(kMostVisitedScope),
-      result_count,
-      NewCallback(this, &JumpList::OnSegmentUsageAvailable));
+void JumpList::Terminate() {
+  CancelPendingUpdate();
+  RemoveObserver();
+}
+
+void JumpList::OnMostVisitedURLsAvailable(
+    const history::MostVisitedURLList& data) {
+
+  // If we have a pending favicon request, cancel it here (it is out of date).
+  CancelPendingUpdate();
+
+  {
+    base::AutoLock auto_lock(list_lock_);
+    most_visited_pages_.clear();
+    for (size_t i = 0; i < data.size(); i++) {
+      const history::MostVisitedURL& url = data[i];
+      scoped_refptr<ShellLinkItem> link(new ShellLinkItem);
+      std::string url_string = url.url.spec();
+      link->SetArguments(UTF8ToWide(url_string));
+      link->SetTitle(!url.title.empty()? url.title : link->arguments());
+      most_visited_pages_.push_back(link);
+      icon_urls_.push_back(make_pair(url_string, link));
+    }
+  }
+
+  // Send a query that retrieves the first favicon.
+  StartLoadingFavicon();
+}
+
+void JumpList::TabRestoreServiceChanged(TabRestoreService* service) {
+  // if we have a pending handle request, cancel it here (it is out of date).
+  CancelPendingUpdate();
+
+  // local list to pass to methods
+  ShellLinkItemList temp_list;
+
+  // Create a list of ShellLinkItems from the "Recently Closed" pages.
+  // As noted above, we create a ShellLinkItem objects with the following
+  // parameters.
+  // * arguments
+  //   The last URL of the tab object.
+  // * title
+  //   The title of the last URL.
+  // * icon
+  //   An empty string. This value is to be updated in OnFaviconDataAvailable().
+  // This code is copied from
+  // RecentlyClosedTabsHandler::TabRestoreServiceChanged() to emulate it.
+  const int kRecentlyClosedCount = 4;
+  TabRestoreService* tab_restore_service =
+      TabRestoreServiceFactory::GetForProfile(profile_);
+  const TabRestoreService::Entries& entries = tab_restore_service->entries();
+  for (TabRestoreService::Entries::const_iterator it = entries.begin();
+       it != entries.end(); ++it) {
+    const TabRestoreService::Entry* entry = *it;
+    if (entry->type == TabRestoreService::TAB) {
+      AddTab(static_cast<const TabRestoreService::Tab*>(entry),
+             &temp_list, kRecentlyClosedCount);
+    } else if (entry->type == TabRestoreService::WINDOW) {
+      AddWindow(static_cast<const TabRestoreService::Window*>(entry),
+                &temp_list, kRecentlyClosedCount);
+    }
+  }
+  // Lock recently_closed_pages and copy temp_list into it.
+  {
+    base::AutoLock auto_lock(list_lock_);
+    recently_closed_pages_ = temp_list;
+  }
+
+  // Send a query that retrieves the first favicon.
+  StartLoadingFavicon();
 }
 
 void JumpList::TabRestoreServiceDestroyed(TabRestoreService* service) {
@@ -642,16 +665,12 @@ bool JumpList::AddTab(const TabRestoreService::Tab* tab,
                       size_t max_items) {
   // This code adds the URL and the title strings of the given tab to the
   // specified list.
-  // This code is copied from RecentlyClosedTabsHandler::TabToValue().
-  if (tab->navigations.empty() || list->size() >= max_items)
-    return false;
-
-  const TabNavigation& current_navigation =
-      tab->navigations.at(tab->current_navigation_index);
-  if (current_navigation.virtual_url() == GURL(chrome::kChromeUINewTabURL))
+  if (list->size() >= max_items)
     return false;
 
   scoped_refptr<ShellLinkItem> link(new ShellLinkItem);
+  const TabNavigation& current_navigation =
+      tab->navigations.at(tab->current_navigation_index);
   std::string url = current_navigation.virtual_url().spec();
   link->SetArguments(UTF8ToWide(url));
   link->SetTitle(current_navigation.title());
@@ -660,125 +679,118 @@ bool JumpList::AddTab(const TabRestoreService::Tab* tab,
   return true;
 }
 
-bool JumpList::AddWindow(const TabRestoreService::Window* window,
+void JumpList::AddWindow(const TabRestoreService::Window* window,
                          ShellLinkItemList* list,
                          size_t max_items) {
   // This code enumerates al the tabs in the given window object and add their
   // URLs and titles to the list.
-  // This code is copied from RecentlyClosedTabsHandler::WindowToValue().
-  if (window->tabs.empty()) {
-    NOTREACHED();
-    return false;
-  }
+  DCHECK(!window->tabs.empty());
+
   for (size_t i = 0; i < window->tabs.size(); ++i) {
     if (!AddTab(&window->tabs[i], list, max_items))
-      return false;
+      return;
   }
-
-  return true;
 }
 
-bool JumpList::StartLoadingFavIcon() {
-  if (icon_urls_.empty())
-    return false;
-
-  // Ask FaviconService if it has a fav icon of a URL.
-  // When FaviocnService has one, it will call OnFavIconDataAvailable().
-  GURL url(icon_urls_.front().first);
+bool JumpList::StartLoadingFavicon() {
+  GURL url;
+  {
+    base::AutoLock auto_lock(list_lock_);
+    if (icon_urls_.empty())
+      return false;
+    // Ask FaviconService if it has a favicon of a URL.
+    // When FaviconService has one, it will call OnFaviconDataAvailable().
+    url = GURL(icon_urls_.front().first);
+  }
   FaviconService* favicon_service =
       profile_->GetFaviconService(Profile::EXPLICIT_ACCESS);
-  FaviconService::Handle handle = favicon_service->GetFaviconForURL(
-      url, &fav_icon_consumer_,
-      NewCallback(this, &JumpList::OnFavIconDataAvailable));
+  handle_ = favicon_service->GetFaviconForURL(
+      url, history::FAVICON, &favicon_consumer_,
+      base::Bind(&JumpList::OnFaviconDataAvailable, base::Unretained(this)));
   return true;
 }
 
-void JumpList::OnSegmentUsageAvailable(
-    CancelableRequestProvider::Handle handle,
-    std::vector<PageUsageData*>* data) {
-  // Create a list of ShellLinkItem objects from the given list of
-  // PageUsageData objects.
-  // The command that opens a web page with chrome is:
-  //   "chrome.exe <url-to-the-web-page>".
-  // So, we create a ShellLinkItem object with the following parameters.
-  // * arguments
-  //   The URL of a PageUsagedata object (converted to std::wstring).
-  // * title
-  //   The title of a PageUsageData object. If this string is empty, we use
-  //   the URL as our "Most Visited" page does.
-  // * icon
-  //   An empty string. This value is to be updated in OnFavIconDataAvailable().
-  most_visited_pages_.clear();
-  for (std::vector<PageUsageData*>::const_iterator page = data->begin();
-       page != data->end(); ++page) {
-    scoped_refptr<ShellLinkItem> link(new ShellLinkItem);
-    std::string url = (*page)->GetURL().spec();
-    link->SetArguments(UTF8ToWide(url));
-    link->SetTitle(
-        !(*page)->GetTitle().empty() ? (*page)->GetTitle() : link->arguments());
-    most_visited_pages_.push_back(link);
-    icon_urls_.push_back(make_pair(url, link));
-  }
-
-  // Create a list of ShellLinkItems from the "Recently Closed" pages.
-  // As noted above, we create a ShellLinkItem objects with the following
-  // parameters.
-  // * arguments
-  //   The last URL of the tab object.
-  // * title
-  //   The title of the last URL.
-  // * icon
-  //   An empty string. This value is to be updated in OnFavIconDataAvailable().
-  // This code is copied from
-  // RecentlyClosedTabsHandler::TabRestoreServiceChanged() to emulate it.
-  const int kRecentlyClosedCount = 4;
-  recently_closed_pages_.clear();
-  TabRestoreService* tab_restore_service = profile_->GetTabRestoreService();
-  const TabRestoreService::Entries& entries = tab_restore_service->entries();
-  for (TabRestoreService::Entries::const_iterator it = entries.begin();
-       it != entries.end(); ++it) {
-    const TabRestoreService::Entry* entry = *it;
-    if (entry->type == TabRestoreService::TAB) {
-      AddTab(static_cast<const TabRestoreService::Tab*>(entry),
-             &recently_closed_pages_, kRecentlyClosedCount);
-    } else if (entry->type == TabRestoreService::WINDOW) {
-      AddWindow(static_cast<const TabRestoreService::Window*>(entry),
-                &recently_closed_pages_, kRecentlyClosedCount);
-    }
-  }
-
-  // Send a query that retrieves the first fav icon.
-  StartLoadingFavIcon();
-}
-
-void JumpList::OnFavIconDataAvailable(
+void JumpList::OnFaviconDataAvailable(
     FaviconService::Handle handle,
-    bool know_favicon,
-    scoped_refptr<RefCountedMemory> data,
-    bool expired,
-    GURL icon_url) {
-  // Attach the received data to the ShellLinkItem object.
-  // This data will be decoded by JumpListUpdateTask.
-  if (know_favicon && data.get() && data->size()) {
-    if (!icon_urls_.empty() && icon_urls_.front().second)
-      icon_urls_.front().second->SetIconData(data);
-  }
+    history::FaviconData favicon) {
+  // If there is currently a favicon request in progress, it is now outdated,
+  // as we have received another, so nullify the handle from the old request.
+  handle_ = NULL;
+  // lock the list to set icon data and pop the url
+  {
+    base::AutoLock auto_lock(list_lock_);
+    // Attach the received data to the ShellLinkItem object.
+    // This data will be decoded by the RunUpdate method.
+    if (favicon.is_valid()) {
+      if (!icon_urls_.empty() && icon_urls_.front().second)
+        icon_urls_.front().second->SetIconData(favicon.image_data);
+    }
 
-  // if we need to load more fav icons, we send another query and exit.
-  if (!icon_urls_.empty())
-    icon_urls_.pop_front();
-  if (StartLoadingFavIcon())
+    if (!icon_urls_.empty())
+      icon_urls_.pop_front();
+  }
+  // if we need to load more favicons, we send another query and exit.
+  if (StartLoadingFavicon())
     return;
 
-  // Finished Loading all fav icons needed by the application JumpList.
-  // We create a JumpListUpdateTask that creates icon files, and we post it to
+  // Finished loading all favicons needed by the application JumpList.
+  // We use a RunnableMethod that creates icon files, and we post it to
   // the file thread.
   BrowserThread::PostTask(
       BrowserThread::FILE, FROM_HERE,
-      new JumpListUpdateTask(app_id_.c_str(), icon_dir_, most_visited_pages_,
-                             recently_closed_pages_));
+      base::Bind(&JumpList::RunUpdate, this));
+}
 
-  // Delete all items in these lists since we don't need these lists any longer.
-  most_visited_pages_.clear();
-  recently_closed_pages_.clear();
+void JumpList::RunUpdate() {
+  ShellLinkItemList local_most_visited_pages;
+  ShellLinkItemList local_recently_closed_pages;
+
+  {
+    base::AutoLock auto_lock(list_lock_);
+    // Make sure we are not out of date: if icon_urls_ is not empty, then
+    // another notification has been received since we processed this one
+    if (!icon_urls_.empty())
+      return;
+
+    // Make local copies of lists so we can release the lock.
+    local_most_visited_pages = most_visited_pages_;
+    local_recently_closed_pages = recently_closed_pages_;
+  }
+
+  // Delete the directory which contains old icon files, rename the current
+  // icon directory, and create a new directory which contains new JumpList
+  // icon files.
+  FilePath icon_dir_old(icon_dir_.value() + L"Old");
+  if (file_util::PathExists(icon_dir_old))
+    file_util::Delete(icon_dir_old, true);
+  file_util::Move(icon_dir_, icon_dir_old);
+  file_util::CreateDirectory(icon_dir_);
+
+  // Create temporary icon files for shortcuts in the "Most Visited" category.
+  DecodeIconData(local_most_visited_pages);
+
+  // Create temporary icon files for shortcuts in the "Recently Closed"
+  // category.
+  DecodeIconData(local_recently_closed_pages);
+
+  // We finished collecting all resources needed for updating an appliation
+  // JumpList. So, create a new JumpList and replace the current JumpList
+  // with it.
+  UpdateJumpList(app_id_.c_str(), local_most_visited_pages,
+                 local_recently_closed_pages);
+}
+
+void JumpList::DecodeIconData(const ShellLinkItemList& item_list) {
+  for (ShellLinkItemList::const_iterator item = item_list.begin();
+      item != item_list.end(); ++item) {
+    SkBitmap icon_bitmap;
+    if ((*item)->data().get() &&
+        gfx::PNGCodec::Decode((*item)->data()->front(),
+                              (*item)->data()->size(),
+                              &icon_bitmap)) {
+      FilePath icon_path;
+      if (CreateIconFile(icon_bitmap, icon_dir_, &icon_path))
+        (*item)->SetIcon(icon_path.value(), 0, true);
+    }
+  }
 }
