@@ -1,44 +1,29 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "app/gfx/gl/gl_implementation.h"
 #include "base/basictypes.h"
 #include "base/file_path.h"
+#include "base/stringprintf.h"
 #include "base/string_util.h"
 #include "base/test/test_timeouts.h"
 #include "base/threading/platform_thread.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/test/test_launcher_utils.h"
+#include "chrome/test/base/test_launcher_utils.h"
 #include "chrome/test/ui/ui_layout_test.h"
 #include "chrome/test/ui/ui_test.h"
 #include "net/base/net_util.h"
+#include "ui/gfx/gl/gl_implementation.h"
 
 class MediaTest : public UITest {
  protected:
-  virtual void SetUp() {
-    EXPECT_TRUE(test_launcher_utils::OverrideGLImplementation(
-        &launch_arguments_,
-        gfx::kGLImplementationOSMesaName));
-
-#if defined(OS_MACOSX)
-    // Accelerated compositing does not work with OSMesa. AcceleratedSurface
-    // assumes GL contexts are native.
-    launch_arguments_.AppendSwitch(switches::kDisableAcceleratedCompositing);
-#endif
-
-    UITest::SetUp();
-  }
-
   void PlayMedia(const char* tag, const char* media_file) {
     FilePath test_file(test_data_directory_);
     test_file = test_file.AppendASCII("media/player.html");
 
     GURL player_gurl = net::FilePathToFileURL(test_file);
-    std::string url = StringPrintf("%s?%s=%s",
-                                   player_gurl.spec().c_str(),
-                                   tag,
-                                   media_file);
+    std::string url = base::StringPrintf(
+        "%s?%s=%s", player_gurl.spec().c_str(), tag, media_file);
 
     NavigateToURL(GURL(url));
 
@@ -46,12 +31,16 @@ class MediaTest : public UITest {
     const std::wstring kPlaying = L"PLAYING";
     const std::wstring kFailed = L"FAILED";
     const std::wstring kError = L"ERROR";
-    for (int i = 0; i < 10; ++i) {
-      base::PlatformThread::Sleep(TestTimeouts::action_timeout_ms());
+    const base::TimeDelta kSleepInterval =
+        base::TimeDelta::FromMilliseconds(250);
+    const int kNumIntervals =
+        TestTimeouts::action_timeout() / kSleepInterval;
+    for (int i = 0; i < kNumIntervals; ++i) {
       const std::wstring& title = GetActiveTabTitle();
       if (title == kPlaying || title == kFailed ||
           StartsWith(title, kError, true))
         break;
+      base::PlatformThread::Sleep(kSleepInterval);
     }
 
     EXPECT_EQ(kPlaying, GetActiveTabTitle());
@@ -66,14 +55,19 @@ class MediaTest : public UITest {
   }
 };
 
-#if defined(OS_LINUX) || defined(OS_WIN)
-// Test appears to be fine on linux, but let's first change to flaky and
-// see how that goes.
-// http://crbug.com/56364
-#define MediaUILayoutTest FLAKY_MediaUILayoutTest
+#if defined(OS_MACOSX)
+// http://crbug.com/88834 - VideoBearTheora, VideoBearWav and VideoBearWebm
+// are flaky on Mac.
+#define MAYBE_VideoBearTheora FLAKY_VideoBearTheora
+#define MAYBE_VideoBearWavPcm FLAKY_VideoBearWavPcm
+#define MAYBE_VideoBearWebm FLAKY_VideoBearWebm
+#else
+#define MAYBE_VideoBearTheora VideoBearTheora
+#define MAYBE_VideoBearWavPcm  VideoBearWavPcm
+#define MAYBE_VideoBearWebm VideoBearWebm
 #endif
 
-TEST_F(MediaTest, VideoBearTheora) {
+TEST_F(MediaTest, MAYBE_VideoBearTheora) {
   PlayVideo("bear.ogv");
 }
 
@@ -81,7 +75,7 @@ TEST_F(MediaTest, VideoBearSilentTheora) {
   PlayVideo("bear_silent.ogv");
 }
 
-TEST_F(MediaTest, VideoBearWebm) {
+TEST_F(MediaTest, MAYBE_VideoBearWebm) {
   PlayVideo("bear.webm");
 }
 
@@ -99,8 +93,44 @@ TEST_F(MediaTest, VideoBearSilentMp4) {
 }
 #endif
 
-TEST_F(MediaTest, VideoBearWav) {
-  PlayVideo("bear.wav");
+#if defined(OS_CHROMEOS)
+#if defined(GOOGLE_CHROME_BUILD) || defined(USE_PROPRIETARY_CODECS)
+TEST_F(MediaTest, VideoBearAviMp3Mpeg4) {
+  PlayVideo("bear_mpeg4_mp3.avi");
+}
+
+TEST_F(MediaTest, VideoBearAviMp3Divx) {
+  PlayVideo("bear_divx_mp3.avi");
+}
+
+TEST_F(MediaTest, VideoBear3gpAacH264) {
+  PlayVideo("bear_h264_aac.3gp");
+}
+
+TEST_F(MediaTest, VideoBear3gpAmrnbMpeg4) {
+  PlayVideo("bear_mpeg4_amrnb.3gp");
+}
+
+// TODO(ihf): Enable these audio codecs for CrOS.
+// TEST_F(MediaTest, VideoBearWavAlaw) {
+//   PlayVideo("bear_alaw.wav");
+// }
+// TEST_F(MediaTest, VideoBearWavGsmms) {
+//   PlayVideo("bear_gsmms.wav");
+// }
+
+TEST_F(MediaTest, VideoBearWavMulaw) {
+  PlayVideo("bear_mulaw.wav");
+}
+
+TEST_F(MediaTest, VideoBearFlac) {
+  PlayVideo("bear.flac");
+}
+#endif
+#endif
+
+TEST_F(MediaTest, MAYBE_VideoBearWavPcm) {
+  PlayVideo("bear_pcm.wav");
 }
 
 TEST_F(UILayoutTest, MediaUILayoutTest) {
