@@ -1,27 +1,30 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/memory_details.h"
 
-#include <unistd.h>
-#include <fcntl.h>
 #include <dirent.h>
+#include <fcntl.h>
+#include <unistd.h>
 
 #include <set>
 
+#include "base/bind.h"
 #include "base/eintr_wrapper.h"
 #include "base/file_version_info.h"
-#include "base/string_util.h"
 #include "base/process_util.h"
+#include "base/string_util.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_child_process_host.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/zygote_host_linux.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/url_constants.h"
+#include "content/browser/zygote_host_linux.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/common/process_type.h"
 #include "grit/chromium_strings.h"
 
+using content::BrowserThread;
+
 // Known browsers which we collect details for.
 enum BrowserType {
   CHROME = 0,
@@ -155,9 +158,9 @@ static void GetProcessDataMemoryInformation(
     pmi.num_processes = 1;
 
     if (pmi.pid == base::GetCurrentProcId())
-      pmi.type = ChildProcessInfo::BROWSER_PROCESS;
+      pmi.type = content::PROCESS_TYPE_BROWSER;
     else
-      pmi.type = ChildProcessInfo::UNKNOWN_PROCESS;
+      pmi.type = content::PROCESS_TYPE_UNKNOWN;
 
     base::ProcessMetrics* metrics =
         base::ProcessMetrics::CreateProcessMetrics(*i);
@@ -200,7 +203,7 @@ static void GetAllChildren(const std::vector<Process>& processes,
 }
 
 void MemoryDetails::CollectProcessData(
-    std::vector<ProcessMemoryInformation> child_info) {
+    const std::vector<ProcessMemoryInformation>& child_info) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::FILE));
 
   std::vector<Process> processes;
@@ -242,6 +245,21 @@ void MemoryDetails::CollectProcessData(
   GetProcessDataMemoryInformation(current_browser_processes, &current_browser);
   current_browser.name = WideToUTF16(chrome::kBrowserAppName);
   current_browser.process_name = ASCIIToUTF16("chrome");
+
+  for (std::vector<ProcessMemoryInformation>::iterator
+       i = current_browser.processes.begin();
+       i != current_browser.processes.end(); ++i) {
+    // Check if this is one of the child processes whose data we collected
+    // on the IO thread, and if so copy over that data.
+    for (size_t child = 0; child < child_info.size(); child++) {
+      if (child_info[child].pid != i->pid)
+        continue;
+      i->titles = child_info[child].titles;
+      i->type = child_info[child].type;
+      break;
+    }
+  }
+
   process_data_.push_back(current_browser);
 
   // For each browser process, collect a list of its children and get the
@@ -269,5 +287,5 @@ void MemoryDetails::CollectProcessData(
   // Finally return to the browser thread.
   BrowserThread::PostTask(
       BrowserThread::UI, FROM_HERE,
-      NewRunnableMethod(this, &MemoryDetails::CollectChildInfoOnUIThread));
+      base::Bind(&MemoryDetails::CollectChildInfoOnUIThread, this));
 }
