@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,31 +8,30 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/file_path.h"
 #include "base/file_version_info.h"
 #include "base/mac/mac_util.h"
-#include "base/string_util.h"
 #include "base/process_util.h"
+#include "base/string_util.h"
 #include "base/threading/thread.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_child_process_host.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/process_info_snapshot.h"
-#include "chrome/browser/renderer_host/backing_store_manager.h"
-#include "chrome/browser/renderer_host/render_process_host.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_version_info.h"
 #include "chrome/common/url_constants.h"
+#include "content/browser/renderer_host/backing_store_manager.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/common/process_type.h"
 #include "grit/chromium_strings.h"
 #include "ui/base/l10n/l10n_util.h"
 
+using content::BrowserThread;
+
 // TODO(viettrungluu): Many of the TODOs below are subsumed by a general need to
 // refactor the about:memory code (not just on Mac, but probably on other
 // platforms as well). I've filed crbug.com/25456.
 
-class RenderViewHostDelegate;
-
 // Known browsers which we collect details for. |CHROME_BROWSER| *must* be the
 // first browser listed. The order here must match those in |process_template|
 // (in |MemoryDetails::MemoryDetails()| below).
@@ -51,7 +50,7 @@ enum BrowserType {
 
 
 MemoryDetails::MemoryDetails() {
-  static const std::string google_browser_name =
+  const std::string google_browser_name =
       l10n_util::GetStringUTF8(IDS_PRODUCT_NAME);
   // (Human and process) names of browsers; should match the ordering for
   // |BrowserProcess| (i.e., |BrowserType|).
@@ -84,7 +83,7 @@ ProcessData* MemoryDetails::ChromeBrowser() {
 }
 
 void MemoryDetails::CollectProcessData(
-    std::vector<ProcessMemoryInformation> child_info) {
+    const std::vector<ProcessMemoryInformation>& child_info) {
   // This must be run on the file thread to avoid jank (|ProcessInfoSnapshot|
   // runs /bin/ps, which isn't instantaneous).
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::FILE));
@@ -113,11 +112,24 @@ void MemoryDetails::CollectProcessData(
     }
   }
 
+  // The helper might show up as these different flavors depending on the
+  // executable flags required.
+  std::vector<std::string> helper_names;
+  helper_names.push_back(chrome::kHelperProcessExecutableName);
+  for (const char* const* suffix = chrome::kHelperFlavorSuffixes;
+       *suffix;
+       ++suffix) {
+    std::string helper_name = chrome::kHelperProcessExecutableName;
+    helper_name.append(1, ' ');
+    helper_name.append(*suffix);
+    helper_names.push_back(helper_name);
+  }
+
   // Get PIDs of helpers.
   std::vector<base::ProcessId> helper_pids;
-  {
-    base::NamedProcessIterator helper_it(chrome::kHelperProcessExecutableName,
-                                         NULL);
+  for (size_t i = 0; i < helper_names.size(); ++i) {
+    std::string helper_name = helper_names[i];
+    base::NamedProcessIterator helper_it(helper_name, NULL);
     while (const base::ProcessEntry* entry = helper_it.NextProcessEntry()) {
       helper_pids.push_back(entry->pid());
       all_pids.push_back(entry->pid());
@@ -135,7 +147,7 @@ void MemoryDetails::CollectProcessData(
          it != pids_by_browser[index].end(); ++it) {
       ProcessMemoryInformation info;
       info.pid = *it;
-      info.type = ChildProcessInfo::UNKNOWN_PROCESS;
+      info.type = content::PROCESS_TYPE_UNKNOWN;
 
       // Try to get version information. To do this, we need first to get the
       // executable's name (we can only believe |proc_info.command| if it looks
@@ -186,7 +198,7 @@ void MemoryDetails::CollectProcessData(
   // Finally return to the browser thread.
   BrowserThread::PostTask(
       BrowserThread::UI, FROM_HERE,
-      NewRunnableMethod(this, &MemoryDetails::CollectChildInfoOnUIThread));
+      base::Bind(&MemoryDetails::CollectChildInfoOnUIThread, this));
 }
 
 void MemoryDetails::CollectProcessDataChrome(
@@ -196,9 +208,9 @@ void MemoryDetails::CollectProcessDataChrome(
   ProcessMemoryInformation info;
   info.pid = pid;
   if (info.pid == base::GetCurrentProcId())
-    info.type = ChildProcessInfo::BROWSER_PROCESS;
+    info.type = content::PROCESS_TYPE_BROWSER;
   else
-    info.type = ChildProcessInfo::UNKNOWN_PROCESS;
+    info.type = content::PROCESS_TYPE_UNKNOWN;
 
   chrome::VersionInfo version_info;
   if (version_info.is_valid()) {
