@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,24 +6,26 @@
 
 #include <set>
 
+#include "base/bind.h"
 #include "base/threading/thread.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/history/history.h"
-#include "chrome/browser/in_process_webkit/webkit_context.h"
 #include "chrome/browser/profiles/profile_manager.h"
-#include "chrome/browser/renderer_host/backing_store_manager.h"
-#include "chrome/browser/renderer_host/render_process_host.h"
-#include "chrome/browser/renderer_host/resource_dispatcher_host.h"
-#include "chrome/browser/safe_browsing/safe_browsing_service.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/webdata/web_data_service.h"
-#include "chrome/common/net/url_request_context_getter.h"
-#include "chrome/common/notification_service.h"
 #include "chrome/common/render_messages.h"
+#include "content/browser/in_process_webkit/webkit_context.h"
+#include "content/browser/renderer_host/backing_store_manager.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/render_process_host.h"
 #include "net/proxy/proxy_resolver.h"
+#include "net/proxy/proxy_service.h"
 #include "net/url_request/url_request_context.h"
+#include "net/url_request/url_request_context_getter.h"
 #include "third_party/tcmalloc/chromium/src/google/malloc_extension.h"
-#include "v8/include/v8.h"
+
+using content::BrowserThread;
 
 // PurgeMemoryHelper -----------------------------------------------------------
 
@@ -33,27 +35,25 @@
 class PurgeMemoryIOHelper
     : public base::RefCountedThreadSafe<PurgeMemoryIOHelper> {
  public:
-  explicit PurgeMemoryIOHelper(SafeBrowsingService* safe_browsing_service)
-      : safe_browsing_service_(safe_browsing_service) {
+  PurgeMemoryIOHelper() {
   }
 
   void AddRequestContextGetter(
-      scoped_refptr<URLRequestContextGetter> request_context_getter);
+      scoped_refptr<net::URLRequestContextGetter> request_context_getter);
 
   void PurgeMemoryOnIOThread();
 
  private:
-  typedef scoped_refptr<URLRequestContextGetter> RequestContextGetter;
+  typedef scoped_refptr<net::URLRequestContextGetter> RequestContextGetter;
   typedef std::set<RequestContextGetter> RequestContextGetters;
 
   RequestContextGetters request_context_getters_;
-  scoped_refptr<SafeBrowsingService> safe_browsing_service_;
 
   DISALLOW_COPY_AND_ASSIGN(PurgeMemoryIOHelper);
 };
 
 void PurgeMemoryIOHelper::AddRequestContextGetter(
-    scoped_refptr<URLRequestContextGetter> request_context_getter) {
+    scoped_refptr<net::URLRequestContextGetter> request_context_getter) {
   request_context_getters_.insert(request_context_getter);
 }
 
@@ -65,15 +65,11 @@ void PurgeMemoryIOHelper::PurgeMemoryOnIOThread() {
        i != request_context_getters_.end(); ++i)
     (*i)->GetURLRequestContext()->proxy_service()->PurgeMemory();
 
-  // Close the Safe Browsing database, freeing memory used to cache sqlite as
-  // well as a number of in-memory structures.
-  safe_browsing_service_->CloseDatabase();
-
-  // The appcache service listens for this notification.
-  NotificationService::current()->Notify(
-      NotificationType::PURGE_MEMORY,
-      Source<void>(NULL),
-      NotificationService::NoDetails());
+  // The appcache and safe browsing services listen for this notification.
+  content::NotificationService::current()->Notify(
+      content::NOTIFICATION_PURGE_MEMORY,
+      content::Source<void>(NULL),
+      content::NotificationService::NoDetails());
 }
 
 // -----------------------------------------------------------------------------
@@ -95,14 +91,12 @@ void MemoryPurger::PurgeBrowser() {
 
   // Per-profile cleanup.
   scoped_refptr<PurgeMemoryIOHelper> purge_memory_io_helper(
-      new PurgeMemoryIOHelper(g_browser_process->resource_dispatcher_host()->
-          safe_browsing_service()));
+      new PurgeMemoryIOHelper());
   ProfileManager* profile_manager = g_browser_process->profile_manager();
-  for (ProfileManager::iterator i(profile_manager->begin());
-       i != profile_manager->end(); ++i) {
-    Profile* profile = *i;
+  std::vector<Profile*> profiles(profile_manager->GetLoadedProfiles());
+  for (size_t i = 0; i < profiles.size(); ++i) {
     purge_memory_io_helper->AddRequestContextGetter(
-        make_scoped_refptr(profile->GetRequestContext()));
+        make_scoped_refptr(profiles[i]->GetRequestContext()));
 
     // NOTE: Some objects below may be duplicates across profiles.  We could
     // conceivably put all these in sets and then iterate over the sets.
@@ -111,31 +105,32 @@ void MemoryPurger::PurgeBrowser() {
     // Spinning up the history service is expensive, so we avoid doing it if it
     // hasn't been done already.
     HistoryService* history_service =
-        profile->GetHistoryServiceWithoutCreating();
+        profiles[i]->GetHistoryServiceWithoutCreating();
     if (history_service)
       history_service->UnloadBackend();
 
     // Unload all web databases (freeing memory used to cache sqlite).
     WebDataService* web_data_service =
-        profile->GetWebDataServiceWithoutCreating();
+        profiles[i]->GetWebDataServiceWithoutCreating();
     if (web_data_service)
       web_data_service->UnloadDatabase();
 
     // Ask all WebKitContexts to purge memory (freeing memory used to cache
     // the LocalStorage sqlite DB).  WebKitContext creation is basically free so
     // we don't bother with a "...WithoutCreating()" function.
-    profile->GetWebKitContext()->PurgeMemory();
+    profiles[i]->GetWebKitContext()->PurgeMemory();
   }
 
-  BrowserThread::PostTask(BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(purge_memory_io_helper.get(),
-                        &PurgeMemoryIOHelper::PurgeMemoryOnIOThread));
+  BrowserThread::PostTask(
+      BrowserThread::IO, FROM_HERE,
+      base::Bind(&PurgeMemoryIOHelper::PurgeMemoryOnIOThread,
+                 purge_memory_io_helper.get()));
 
   // TODO(pkasting):
   // * Purge AppCache memory.  Not yet implemented sufficiently.
   // * Browser-side DatabaseTracker.  Not implemented sufficiently.
 
-#if (defined(OS_WIN) || defined(OS_LINUX)) && defined(USE_TCMALLOC)
+#if !defined(OS_MACOSX) && defined(USE_TCMALLOC)
   // Tell tcmalloc to release any free pages it's still holding.
   //
   // TODO(pkasting): A lot of the above calls kick off actions on other threads.
@@ -152,13 +147,14 @@ void MemoryPurger::PurgeRenderers() {
   // Concern: Telling a bunch of renderer processes to destroy their data may
   // cause them to page everything in to do it, which could take a lot of time/
   // cause jank.
-  for (RenderProcessHost::iterator i(RenderProcessHost::AllHostsIterator());
+  for (content::RenderProcessHost::iterator i(
+          content::RenderProcessHost::AllHostsIterator());
        !i.IsAtEnd(); i.Advance())
     PurgeRendererForHost(i.GetCurrentValue());
 }
 
 // static
-void MemoryPurger::PurgeRendererForHost(RenderProcessHost* host) {
+void MemoryPurger::PurgeRendererForHost(content::RenderProcessHost* host) {
   // Direct the renderer to free everything it can.
-  host->Send(new ViewMsg_PurgeMemory());
+  host->Send(new ChromeViewMsg_PurgeMemory());
 }
