@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,8 +9,9 @@
 
 #include "base/basictypes.h"
 #include "base/file_util.h"
+#include "base/lazy_instance.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/perftimer.h"
-#include "base/scoped_ptr.h"
 #include "base/string_util.h"
 #include "base/sys_info.h"
 #include "base/third_party/nspr/prtime.h"
@@ -19,13 +20,16 @@
 #include "chrome/browser/autocomplete/autocomplete.h"
 #include "chrome/browser/autocomplete/autocomplete_match.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/gpu_process_host_ui_shim.h"
+#include "chrome/browser/plugin_prefs.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/common/chrome_version_info.h"
 #include "chrome/common/logging_chrome.h"
 #include "chrome/common/pref_names.h"
+#include "content/browser/gpu/gpu_data_manager.h"
 #include "googleurl/src/gurl.h"
-#include "webkit/plugins/npapi/webplugininfo.h"
+#include "ui/gfx/screen.h"
+#include "webkit/plugins/webplugininfo.h"
 
 #define OPEN_ELEMENT_FOR_SCOPE(name) ScopedElement scoped_element(this, name)
 
@@ -34,6 +38,37 @@
 extern "C" IMAGE_DOS_HEADER __ImageBase;
 #endif
 
+namespace {
+
+// Returns the date at which the current metrics client ID was created as
+// a string containing milliseconds since the epoch, or "0" if none was found.
+std::string GetInstallDate() {
+  PrefService* pref = g_browser_process->local_state();
+  if (pref) {
+    return pref->GetString(prefs::kMetricsClientIDTimestamp);
+  } else {
+    NOTREACHED();
+    return "0";
+  }
+}
+
+// Returns the plugin preferences corresponding for this user, if available.
+// If multiple user profiles are loaded, returns the preferences corresponding
+// to an arbitrary one of the profiles.
+PluginPrefs* GetPluginPrefs() {
+  ProfileManager* profile_manager = g_browser_process->profile_manager();
+  std::vector<Profile*> profiles = profile_manager->GetLoadedProfiles();
+  if (profiles.empty())
+    return NULL;
+
+  return PluginPrefs::GetForProfile(profiles.front());
+}
+
+}  // namespace
+
+static base::LazyInstance<std::string>::Leaky
+  g_version_extension = LAZY_INSTANCE_INITIALIZER;
+
 MetricsLog::MetricsLog(const std::string& client_id, int session_id)
     : MetricsLogBase(client_id, session_id, MetricsLog::GetVersionString()) {}
 
@@ -44,6 +79,7 @@ void MetricsLog::RegisterPrefs(PrefService* local_state) {
   local_state->RegisterListPref(prefs::kStabilityPluginStats);
 }
 
+// static
 int64 MetricsLog::GetIncrementalUptime(PrefService* pref) {
   base::TimeTicks now = base::TimeTicks::Now();
   static base::TimeTicks last_updated_time(now);
@@ -59,16 +95,6 @@ int64 MetricsLog::GetIncrementalUptime(PrefService* pref) {
   return incremental_time;
 }
 
-std::string MetricsLog::GetInstallDate() const {
-  PrefService* pref = g_browser_process->local_state();
-  if (pref) {
-    return pref->GetString(prefs::kMetricsClientIDTimestamp);
-  } else {
-    NOTREACHED();
-    return "0";
-  }
-}
-
 // static
 std::string MetricsLog::GetVersionString() {
   chrome::VersionInfo version_info;
@@ -78,15 +104,21 @@ std::string MetricsLog::GetVersionString() {
   }
 
   std::string version = version_info.Version();
-  if (!version_extension_.empty())
-    version += version_extension_;
+  if (!version_extension().empty())
+    version += version_extension();
   if (!version_info.IsOfficialBuild())
     version.append("-devel");
   return version;
 }
 
-MetricsLog* MetricsLog::AsMetricsLog() {
-  return this;
+// static
+void MetricsLog::set_version_extension(const std::string& extension) {
+  g_version_extension.Get() = extension;
+}
+
+// static
+const std::string& MetricsLog::version_extension() {
+  return g_version_extension.Get();
 }
 
 void MetricsLog::RecordIncrementalStabilityElements() {
@@ -98,7 +130,7 @@ void MetricsLog::RecordIncrementalStabilityElements() {
   OPEN_ELEMENT_FOR_SCOPE("profile");
   WriteCommonEventAttributes();
 
-  WriteInstallElement();  // Supply appversion.
+  WriteInstallElement();
 
   {
     OPEN_ELEMENT_FOR_SCOPE("stability");  // Minimal set of stability elements.
@@ -150,39 +182,40 @@ void MetricsLog::WritePluginStabilityElements(PrefService* pref) {
   // Now log plugin stability info.
   const ListValue* plugin_stats_list = pref->GetList(
       prefs::kStabilityPluginStats);
-  if (plugin_stats_list) {
-    OPEN_ELEMENT_FOR_SCOPE("plugins");
-    for (ListValue::const_iterator iter = plugin_stats_list->begin();
-         iter != plugin_stats_list->end(); ++iter) {
-      if (!(*iter)->IsType(Value::TYPE_DICTIONARY)) {
-        NOTREACHED();
-        continue;
-      }
-      DictionaryValue* plugin_dict = static_cast<DictionaryValue*>(*iter);
+  if (!plugin_stats_list)
+    return;
 
-      std::string plugin_name;
-      plugin_dict->GetString(prefs::kStabilityPluginName, &plugin_name);
-
-      OPEN_ELEMENT_FOR_SCOPE("pluginstability");
-      // Use "filename" instead of "name", otherwise we need to update the
-      // UMA servers.
-      WriteAttribute("filename", CreateBase64Hash(plugin_name));
-
-      int launches = 0;
-      plugin_dict->GetInteger(prefs::kStabilityPluginLaunches, &launches);
-      WriteIntAttribute("launchcount", launches);
-
-      int instances = 0;
-      plugin_dict->GetInteger(prefs::kStabilityPluginInstances, &instances);
-      WriteIntAttribute("instancecount", instances);
-
-      int crashes = 0;
-      plugin_dict->GetInteger(prefs::kStabilityPluginCrashes, &crashes);
-      WriteIntAttribute("crashcount", crashes);
+  OPEN_ELEMENT_FOR_SCOPE("plugins");
+  for (ListValue::const_iterator iter = plugin_stats_list->begin();
+       iter != plugin_stats_list->end(); ++iter) {
+    if (!(*iter)->IsType(Value::TYPE_DICTIONARY)) {
+      NOTREACHED();
+      continue;
     }
+    DictionaryValue* plugin_dict = static_cast<DictionaryValue*>(*iter);
 
-    pref->ClearPref(prefs::kStabilityPluginStats);
+    std::string plugin_name;
+    plugin_dict->GetString(prefs::kStabilityPluginName, &plugin_name);
+
+    OPEN_ELEMENT_FOR_SCOPE("pluginstability");
+    // Use "filename" instead of "name", otherwise we need to update the
+    // UMA servers.
+    WriteAttribute("filename", CreateBase64Hash(plugin_name));
+
+    int launches = 0;
+    plugin_dict->GetInteger(prefs::kStabilityPluginLaunches, &launches);
+    WriteIntAttribute("launchcount", launches);
+
+    int instances = 0;
+    plugin_dict->GetInteger(prefs::kStabilityPluginInstances, &instances);
+    WriteIntAttribute("instancecount", instances);
+
+    int crashes = 0;
+    plugin_dict->GetInteger(prefs::kStabilityPluginCrashes, &crashes);
+    WriteIntAttribute("crashcount", crashes);
   }
+
+  pref->ClearPref(prefs::kStabilityPluginStats);
 }
 
 void MetricsLog::WriteRequiredStabilityAttributes(PrefService* pref) {
@@ -269,12 +302,14 @@ void MetricsLog::WriteRealtimeStabilityAttributes(PrefService* pref) {
 }
 
 void MetricsLog::WritePluginList(
-    const std::vector<webkit::npapi::WebPluginInfo>& plugin_list) {
+    const std::vector<webkit::WebPluginInfo>& plugin_list) {
   DCHECK(!locked_);
 
+  PluginPrefs* plugin_prefs = GetPluginPrefs();
+
   OPEN_ELEMENT_FOR_SCOPE("plugins");
 
-  for (std::vector<webkit::npapi::WebPluginInfo>::const_iterator iter =
+  for (std::vector<webkit::WebPluginInfo>::const_iterator iter =
            plugin_list.begin();
        iter != plugin_list.end(); ++iter) {
     OPEN_ELEMENT_FOR_SCOPE("plugin");
@@ -282,9 +317,16 @@ void MetricsLog::WritePluginList(
     // Plugin name and filename are hashed for the privacy of those
     // testing unreleased new extensions.
     WriteAttribute("name", CreateBase64Hash(UTF16ToUTF8(iter->name)));
-    WriteAttribute("filename",
-        CreateBase64Hash(WideToUTF8(iter->path.BaseName().ToWStringHack())));
+    std::string filename_bytes =
+#if defined(OS_WIN)
+        UTF16ToUTF8(iter->path.BaseName().value());
+#else
+        iter->path.BaseName().value();
+#endif
+    WriteAttribute("filename", CreateBase64Hash(filename_bytes));
     WriteAttribute("version", UTF16ToUTF8(iter->version));
+    if (plugin_prefs)
+      WriteIntAttribute("disabled", !plugin_prefs->IsPluginEnabled(*iter));
   }
 }
 
@@ -295,7 +337,7 @@ void MetricsLog::WriteInstallElement() {
 }
 
 void MetricsLog::RecordEnvironment(
-         const std::vector<webkit::npapi::WebPluginInfo>& plugin_list,
+         const std::vector<webkit::WebPluginInfo>& plugin_list,
          const DictionaryValue* profile_metrics) {
   DCHECK(!locked_);
 
@@ -333,22 +375,19 @@ void MetricsLog::RecordEnvironment(
 
   {
     OPEN_ELEMENT_FOR_SCOPE("gpu");
-    WriteIntAttribute(
-        "vendorid",
-        GpuProcessHostUIShim::GetInstance()->gpu_info().vendor_id());
-    WriteIntAttribute(
-        "deviceid",
-        GpuProcessHostUIShim::GetInstance()->gpu_info().device_id());
+    GpuDataManager* gpu_data_manager = GpuDataManager::GetInstance();
+    if (gpu_data_manager) {
+      WriteIntAttribute("vendorid", gpu_data_manager->gpu_info().vendor_id);
+      WriteIntAttribute("deviceid", gpu_data_manager->gpu_info().device_id);
+    }
   }
 
   {
     OPEN_ELEMENT_FOR_SCOPE("display");
-    int width = 0;
-    int height = 0;
-    base::SysInfo::GetPrimaryDisplayDimensions(&width, &height);
-    WriteIntAttribute("xsize", width);
-    WriteIntAttribute("ysize", height);
-    WriteIntAttribute("screens", base::SysInfo::DisplayCount());
+    const gfx::Size display_size = gfx::Screen::GetPrimaryMonitorSize();
+    WriteIntAttribute("xsize", display_size.width());
+    WriteIntAttribute("ysize", display_size.height());
+    WriteIntAttribute("screens", gfx::Screen::GetNumMonitors());
   }
 
   {
@@ -458,15 +497,28 @@ void MetricsLog::RecordOmniboxOpenedURL(const AutocompleteLog& log) {
   WriteAttribute("targetidhash", "");
   // TODO(kochi): Properly track windows.
   WriteIntAttribute("window", 0);
+  if (log.tab_id != -1) {
+    // If we know what tab the autocomplete URL was opened in, log it.
+    WriteIntAttribute("tab", static_cast<int>(log.tab_id));
+  }
   WriteCommonEventAttributes();
 
   {
     OPEN_ELEMENT_FOR_SCOPE("autocomplete");
 
     WriteIntAttribute("typedlength", static_cast<int>(log.text.length()));
+    std::vector<string16> terms;
+    WriteIntAttribute("numterms",
+        static_cast<int>(Tokenize(log.text, kWhitespaceUTF16, &terms)));
     WriteIntAttribute("selectedindex", static_cast<int>(log.selected_index));
     WriteIntAttribute("completedlength",
                       static_cast<int>(log.inline_autocompleted_length));
+    if (log.elapsed_time_since_user_first_modified_omnibox !=
+        base::TimeDelta::FromMilliseconds(-1)) {
+      // Only upload the typing duration if it is set/valid.
+      WriteInt64Attribute("typingduration",
+          log.elapsed_time_since_user_first_modified_omnibox.InMilliseconds());
+    }
     const std::string input_type(
         AutocompleteInput::TypeToString(log.input_type));
     if (!input_type.empty())
