@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,19 +9,21 @@
 #define CHROME_BROWSER_METRICS_METRICS_LOG_H_
 #pragma once
 
+#include <string>
+#include <vector>
+
 #include "base/basictypes.h"
-#include "chrome/common/metrics_helpers.h"
-#include "chrome/common/page_transition_types.h"
+#include "chrome/common/metrics/metrics_log_base.h"
 
 struct AutocompleteLog;
-class DictionaryValue;
-class GURL;
 class PrefService;
 
-namespace webkit {
-namespace npapi {
-struct WebPluginInfo;
+namespace base {
+class DictionaryValue;
 }
+
+namespace webkit {
+struct WebPluginInfo;
 }
 
 class MetricsLog : public MetricsLogBase {
@@ -34,6 +36,18 @@ class MetricsLog : public MetricsLogBase {
 
   static void RegisterPrefs(PrefService* prefs);
 
+  // Get the amount of uptime in seconds since this function was last called.
+  // This updates the cumulative uptime metric for uninstall as a side effect.
+  static int64 GetIncrementalUptime(PrefService* pref);
+
+  // Get the current version of the application as a string.
+  static std::string GetVersionString();
+
+  // Use |extension| in all uploaded appversions in addition to the standard
+  // version string.
+  static void set_version_extension(const std::string& extension);
+  static const std::string& version_extension();
+
   // Records the current operating environment.  Takes the list of installed
   // plugins as a parameter because that can't be obtained synchronously
   // from the UI thread.
@@ -41,8 +55,8 @@ class MetricsLog : public MetricsLogBase {
   // that are to be recorded. Each value in profile_metrics should be a
   // dictionary giving the metrics for the profile.
   void RecordEnvironment(
-      const std::vector<webkit::npapi::WebPluginInfo>& plugin_list,
-      const DictionaryValue* profile_metrics);
+      const std::vector<webkit::WebPluginInfo>& plugin_list,
+      const base::DictionaryValue* profile_metrics);
 
   // Records the input text, available choices, and selected entry when the
   // user uses the Omnibox to open a URL.
@@ -54,23 +68,9 @@ class MetricsLog : public MetricsLogBase {
   // uma log upload, just as we send histogram data.
   void RecordIncrementalStabilityElements();
 
-  // Get the amount of uptime in seconds since this function was last called.
-  // This updates the cumulative uptime metric for uninstall as a side effect.
-  static int64 GetIncrementalUptime(PrefService* pref);
-
-  // Get the current version of the application as a string.
-  static std::string GetVersionString();
-
-  virtual MetricsLog* AsMetricsLog();
-
  private:
   FRIEND_TEST_ALL_PREFIXES(MetricsLogTest, ChromeOSStabilityData);
 
-  // Returns the date at which the current metrics client ID was created as
-  // a string containing milliseconds since the epoch, or "0" if none was found.
-  std::string GetInstallDate() const;
-
-
   // Writes application stability metrics (as part of the profile log).
   // NOTE: Has the side-effect of clearing those counts.
   void WriteStabilityElement(PrefService* pref);
@@ -89,19 +89,20 @@ class MetricsLog : public MetricsLogBase {
 
   // Writes the list of installed plugins.
   void WritePluginList(
-      const std::vector<webkit::npapi::WebPluginInfo>& plugin_list);
+      const std::vector<webkit::WebPluginInfo>& plugin_list);
 
   // Within the profile group, write basic install info including appversion.
   void WriteInstallElement();
 
   // Writes all profile metrics. This invokes WriteProfileMetrics for each key
   // in all_profiles_metrics that starts with kProfilePrefix.
-  void WriteAllProfilesMetrics(const DictionaryValue& all_profiles_metrics);
+  void WriteAllProfilesMetrics(
+      const base::DictionaryValue& all_profiles_metrics);
 
   // Writes metrics for the profile identified by key. This writes all
   // key/value pairs in profile_metrics.
   void WriteProfileMetrics(const std::string& key,
-                           const DictionaryValue& profile_metrics);
+                           const base::DictionaryValue& profile_metrics);
 
   DISALLOW_COPY_AND_ASSIGN(MetricsLog);
 };
