@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -20,6 +20,7 @@
 #include "chrome/common/json_pref_store.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
+#include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/ui/ui_test.h"
@@ -68,15 +69,16 @@ TEST_F(MetricsServiceTest, CloseRenderersNormally) {
   local_state->RegisterIntegerPref(prefs::kStabilityRendererCrashCount, 0);
   EXPECT_TRUE(local_state->GetBoolean(prefs::kStabilityExitedCleanly));
   EXPECT_EQ(1, local_state->GetInteger(prefs::kStabilityLaunchCount));
+#if defined(USE_VIRTUAL_KEYBOARD)
+  // The keyboard page loads.
+  EXPECT_EQ(4, local_state->GetInteger(prefs::kStabilityPageLoadCount));
+#else
   EXPECT_EQ(3, local_state->GetInteger(prefs::kStabilityPageLoadCount));
+#endif
   EXPECT_EQ(0, local_state->GetInteger(prefs::kStabilityRendererCrashCount));
 }
 
 TEST_F(MetricsServiceTest, DISABLED_CrashRenderers) {
-  // This doesn't make sense to test in single process mode.
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   OpenTabs();
 
   {
@@ -96,12 +98,11 @@ TEST_F(MetricsServiceTest, DISABLED_CrashRenderers) {
     defined(GOOGLE_CHROME_BUILD)
     expected_crashes_ = 1;
 #endif
-
     ASSERT_TRUE(tab->NavigateToURLAsync(GURL(chrome::kAboutCrashURL)));
   }
 
   // Give the browser a chance to notice the crashed tab.
-  base::PlatformThread::Sleep(TestTimeouts::action_timeout_ms());
+  base::PlatformThread::Sleep(TestTimeouts::action_timeout());
 
   QuitBrowser();
 
@@ -112,6 +113,11 @@ TEST_F(MetricsServiceTest, DISABLED_CrashRenderers) {
   local_state->RegisterIntegerPref(prefs::kStabilityRendererCrashCount, 0);
   EXPECT_TRUE(local_state->GetBoolean(prefs::kStabilityExitedCleanly));
   EXPECT_EQ(1, local_state->GetInteger(prefs::kStabilityLaunchCount));
+#if defined(USE_VIRTUAL_KEYBOARD)
+  // The keyboard page loads.
+  EXPECT_EQ(5, local_state->GetInteger(prefs::kStabilityPageLoadCount));
+#else
   EXPECT_EQ(4, local_state->GetInteger(prefs::kStabilityPageLoadCount));
+#endif
   EXPECT_EQ(1, local_state->GetInteger(prefs::kStabilityRendererCrashCount));
 }
