@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 #define CHROME_BROWSER_MOCK_BROWSING_DATA_APPCACHE_HELPER_H_
 #pragma once
 
-#include "base/callback.h"
+#include "base/callback_forward.h"
 #include "chrome/browser/browsing_data_appcache_helper.h"
 
 class MockBrowsingDataAppCacheHelper
@@ -14,9 +14,9 @@ class MockBrowsingDataAppCacheHelper
  public:
   explicit MockBrowsingDataAppCacheHelper(Profile* profile);
 
-  virtual void StartFetching(Callback0::Type* completion_callback);
-  virtual void CancelNotification();
-  virtual void DeleteAppCacheGroup(const GURL& manifest_url);
+  virtual void StartFetching(const base::Closure& completion_callback) OVERRIDE;
+  virtual void CancelNotification() OVERRIDE;
+  virtual void DeleteAppCacheGroup(const GURL& manifest_url) OVERRIDE;
 
  private:
   virtual ~MockBrowsingDataAppCacheHelper();
