@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,13 +11,16 @@
 #include "chrome/browser/nacl_host/nacl_process_host.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/common/logging_chrome.h"
 #include "chrome/common/nacl_cmd_line.h"
 #include "chrome/common/nacl_messages.h"
+#include "content/public/browser/browser_child_process_host.h"
+#include "content/public/common/child_process_host.h"
 
-NaClBrokerHost::NaClBrokerHost(
-    ResourceDispatcherHost* resource_dispatcher_host)
-    : BrowserChildProcessHost(NACL_BROKER_PROCESS, resource_dispatcher_host),
-      stopping_(false) {
+NaClBrokerHost::NaClBrokerHost()
+    : stopping_(false) {
+  process_.reset(content::BrowserChildProcessHost::Create(
+      content::PROCESS_TYPE_NACL_BROKER, this));
 }
 
 NaClBrokerHost::~NaClBrokerHost() {
@@ -25,7 +28,8 @@ NaClBrokerHost::~NaClBrokerHost() {
 
 bool NaClBrokerHost::Init() {
   // Create the channel that will be used for communicating with the broker.
-  if (!CreateChannel())
+  std::string channel_id = process_->GetHost()->CreateChannel();
+  if (channel_id.empty())
     return false;
 
   // Create the path to the nacl broker/loader executable.
@@ -39,10 +43,11 @@ bool NaClBrokerHost::Init() {
 
   cmd_line->AppendSwitchASCII(switches::kProcessType,
                               switches::kNaClBrokerProcess);
+  cmd_line->AppendSwitchASCII(switches::kProcessChannelID, channel_id);
+  if (logging::DialogsAreSuppressed())
+    cmd_line->AppendSwitch(switches::kNoErrorDialogs);
 
-  cmd_line->AppendSwitchASCII(switches::kProcessChannelID, channel_id());
-
-  BrowserChildProcessHost::Launch(FilePath(), cmd_line);
+  process_->Launch(FilePath(), cmd_line);
   return true;
 }
 
@@ -57,7 +62,8 @@ bool NaClBrokerHost::OnMessageReceived(const IPC::Message& msg) {
 
 bool NaClBrokerHost::LaunchLoader(
     const std::wstring& loader_channel_id) {
-  return Send(new NaClProcessMsg_LaunchLoaderThroughBroker(loader_channel_id));
+  return process_->Send(
+      new NaClProcessMsg_LaunchLoaderThroughBroker(loader_channel_id));
 }
 
 void NaClBrokerHost::OnLoaderLaunched(const std::wstring& loader_channel_id,
@@ -67,5 +73,5 @@ void NaClBrokerHost::OnLoaderLaunched(const std::wstring& loader_channel_id,
 
 void NaClBrokerHost::StopBroker() {
   stopping_ = true;
-  Send(new NaClProcessMsg_StopBroker());
+  process_->Send(new NaClProcessMsg_StopBroker());
 }
