@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,11 +7,17 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "chrome/browser/browser_child_process_host.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/process.h"
+#include "content/public/browser/browser_child_process_host_delegate.h"
 
-class NaClBrokerHost : public BrowserChildProcessHost {
+namespace content {
+class BrowserChildProcessHost;
+}
+
+class NaClBrokerHost : public content::BrowserChildProcessHostDelegate {
  public:
-  explicit NaClBrokerHost(ResourceDispatcherHost* resource_dispatcher_host);
+  NaClBrokerHost();
   ~NaClBrokerHost();
 
   // This function starts the broker process. It needs to be called
@@ -26,17 +32,17 @@ class NaClBrokerHost : public BrowserChildProcessHost {
   void StopBroker();
 
  private:
-  virtual bool CanShutdown() { return true; }
-
   // Handler for NaClProcessMsg_LoaderLaunched message
   void OnLoaderLaunched(const std::wstring& loader_channel_id,
                         base::ProcessHandle handle);
 
-  // IPC::Channel::Listener
+  // BrowserChildProcessHostDelegate implementation:
   virtual bool OnMessageReceived(const IPC::Message& msg);
 
   bool stopping_;
 
+  scoped_ptr<content::BrowserChildProcessHost> process_;
+
   DISALLOW_COPY_AND_ASSIGN(NaClBrokerHost);
 };
 
