@@ -4,9 +4,11 @@
 
 #include "chrome/browser/net/connection_tester.h"
 
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
+#include "base/memory/weak_ptr.h"
 #include "base/message_loop.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/utf_string_conversions.h"
@@ -14,22 +16,24 @@
 #include "chrome/common/chrome_switches.h"
 #include "net/base/cert_verifier.h"
 #include "net/base/cookie_monster.h"
-#include "net/base/dnsrr_resolver.h"
 #include "net/base/host_resolver.h"
 #include "net/base/host_resolver_impl.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_util.h"
 #include "net/base/ssl_config_service_defaults.h"
-#include "net/disk_cache/disk_cache.h"
 #include "net/ftp/ftp_network_layer.h"
 #include "net/http/http_auth_handler_factory.h"
 #include "net/http/http_cache.h"
-#include "net/http/http_network_layer.h"
+#include "net/http/http_network_session.h"
+#include "net/http/http_server_properties_impl.h"
+#include "net/proxy/dhcp_proxy_script_fetcher_factory.h"
 #include "net/proxy/proxy_config_service_fixed.h"
 #include "net/proxy/proxy_script_fetcher_impl.h"
+#include "net/proxy/proxy_service.h"
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_context.h"
+#include "net/url_request/url_request_context_storage.h"
 
 namespace {
 
@@ -42,52 +46,58 @@ class ExperimentURLRequestContext : public net::URLRequestContext {
  public:
   explicit ExperimentURLRequestContext(
       net::URLRequestContext* proxy_request_context)
-      : proxy_request_context_(proxy_request_context) {}
+      : proxy_request_context_(proxy_request_context),
+        ALLOW_THIS_IN_INITIALIZER_LIST(storage_(this)) {}
 
   int Init(const ConnectionTester::Experiment& experiment) {
     int rv;
 
     // Create a custom HostResolver for this experiment.
+    scoped_ptr<net::HostResolver> host_resolver_tmp;
     rv = CreateHostResolver(experiment.host_resolver_experiment,
-                            &host_resolver_);
+                            &host_resolver_tmp);
     if (rv != net::OK)
       return rv;  // Failure.
+    storage_.set_host_resolver(host_resolver_tmp.release());
 
     // Create a custom ProxyService for this this experiment.
+    scoped_ptr<net::ProxyService> proxy_service_tmp;
     rv = CreateProxyService(experiment.proxy_settings_experiment,
-                            &proxy_service_);
+                            &proxy_service_tmp);
     if (rv != net::OK)
       return rv;  // Failure.
+    storage_.set_proxy_service(proxy_service_tmp.release());
 
     // The rest of the dependencies are standard, and don't depend on the
     // experiment being run.
-    cert_verifier_ = new net::CertVerifier;
-    dnsrr_resolver_ = new net::DnsRRResolver;
-    ftp_transaction_factory_ = new net::FtpNetworkLayer(host_resolver_);
-    ssl_config_service_ = new net::SSLConfigServiceDefaults;
-    http_auth_handler_factory_ = net::HttpAuthHandlerFactory::CreateDefault(
-        host_resolver_);
-    http_transaction_factory_ = new net::HttpCache(
-        net::HttpNetworkLayer::CreateFactory(host_resolver_, cert_verifier_,
-            dnsrr_resolver_, NULL /* dns_cert_checker */,
-            NULL /* ssl_host_info_factory */, proxy_service_,
-            ssl_config_service_, http_auth_handler_factory_, NULL, NULL),
-        NULL /* net_log */,
-        net::HttpCache::DefaultBackend::InMemory(0));
+    storage_.set_cert_verifier(new net::CertVerifier);
+    storage_.set_ftp_transaction_factory(
+        new net::FtpNetworkLayer(host_resolver()));
+    storage_.set_ssl_config_service(new net::SSLConfigServiceDefaults);
+    storage_.set_http_auth_handler_factory(
+        net::HttpAuthHandlerFactory::CreateDefault(host_resolver()));
+    storage_.set_http_server_properties(new net::HttpServerPropertiesImpl);
+
+    net::HttpNetworkSession::Params session_params;
+    session_params.host_resolver = host_resolver();
+    session_params.cert_verifier = cert_verifier();
+    session_params.proxy_service = proxy_service();
+    session_params.http_auth_handler_factory = http_auth_handler_factory();
+    session_params.http_server_properties = http_server_properties();
+    session_params.ssl_config_service = ssl_config_service();
+    scoped_refptr<net::HttpNetworkSession> network_session(
+        new net::HttpNetworkSession(session_params));
+    storage_.set_http_transaction_factory(new net::HttpCache(
+        network_session,
+        net::HttpCache::DefaultBackend::InMemory(0)));
     // In-memory cookie store.
-    cookie_store_ = new net::CookieMonster(NULL, NULL);
+    storage_.set_cookie_store(new net::CookieMonster(NULL, NULL));
 
     return net::OK;
   }
 
  protected:
   virtual ~ExperimentURLRequestContext() {
-    delete ftp_transaction_factory_;
-    delete http_transaction_factory_;
-    delete http_auth_handler_factory_;
-    delete dnsrr_resolver_;
-    delete cert_verifier_;
-    delete host_resolver_;
   }
 
  private:
@@ -96,13 +106,15 @@ class ExperimentURLRequestContext : public net::URLRequestContext {
   // error code.
   int CreateHostResolver(
       ConnectionTester::HostResolverExperiment experiment,
-      net::HostResolver** host_resolver) {
+      scoped_ptr<net::HostResolver>* host_resolver) {
     // Create a vanilla HostResolver that disables caching.
     const size_t kMaxJobs = 50u;
+    const size_t kMaxRetryAttempts = 4u;
     net::HostResolverImpl* impl =
-        new net::HostResolverImpl(NULL, NULL, kMaxJobs, NULL);
+        new net::HostResolverImpl(NULL, NULL, kMaxJobs, kMaxRetryAttempts,
+                                  NULL);
 
-    *host_resolver = impl;
+    host_resolver->reset(impl);
 
     // Modify it slightly based on the experiment being run.
     switch (experiment) {
@@ -160,7 +172,7 @@ class ExperimentURLRequestContext : public net::URLRequestContext {
   // error code.
   int CreateProxyService(
       ConnectionTester::ProxySettingsExperiment experiment,
-      scoped_refptr<net::ProxyService>* proxy_service) {
+      scoped_ptr<net::ProxyService>* proxy_service) {
     // Create an appropriate proxy config service.
     scoped_ptr<net::ProxyConfigService> config_service;
     int rv = CreateProxyConfigService(experiment, &config_service);
@@ -174,12 +186,20 @@ class ExperimentURLRequestContext : public net::URLRequestContext {
       return net::ERR_NOT_IMPLEMENTED;
     }
 
-    *proxy_service = net::ProxyService::CreateUsingV8ProxyResolver(
+    net::DhcpProxyScriptFetcherFactory dhcp_factory;
+    if (CommandLine::ForCurrentProcess()->HasSwitch(
+        switches::kDisableDhcpWpad)) {
+      dhcp_factory.set_enabled(false);
+    }
+
+    proxy_service->reset(net::ProxyService::CreateUsingV8ProxyResolver(
         config_service.release(),
         0u,
         new net::ProxyScriptFetcherImpl(proxy_request_context_),
+        dhcp_factory.Create(proxy_request_context_),
         host_resolver(),
-        NULL);
+        NULL,
+        NULL));
 
     return net::OK;
   }
@@ -189,7 +209,7 @@ class ExperimentURLRequestContext : public net::URLRequestContext {
   // Otherwise returns a network error code.
   int CreateSystemProxyConfigService(
       scoped_ptr<net::ProxyConfigService>* config_service) {
-#if defined(OS_LINUX)
+#if defined(OS_LINUX) || defined(OS_OPENBSD)
     // TODO(eroman): This is not supported on Linux yet, because of how
     // construction needs ot happen on the UI thread.
     return net::ERR_NOT_IMPLEMENTED;
@@ -225,6 +245,7 @@ class ExperimentURLRequestContext : public net::URLRequestContext {
   }
 
   const scoped_refptr<net::URLRequestContext> proxy_request_context_;
+  net::URLRequestContextStorage storage_;
 };
 
 }  // namespace
@@ -237,7 +258,9 @@ class ConnectionTester::TestRunner : public net::URLRequest::Delegate {
  public:
   // |tester| must remain alive throughout the TestRunner's lifetime.
   // |tester| will be notified of completion.
-  explicit TestRunner(ConnectionTester* tester) : tester_(tester) {}
+  explicit TestRunner(ConnectionTester* tester)
+      : tester_(tester),
+        ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {}
 
   // Starts running |experiment|. Notifies tester->OnExperimentCompleted() when
   // it is done.
@@ -258,10 +281,13 @@ class ConnectionTester::TestRunner : public net::URLRequest::Delegate {
 
   // Called when the request has completed (for both success and failure).
   void OnResponseCompleted(net::URLRequest* request);
+  void OnExperimentCompletedWithResult(int result);
 
   ConnectionTester* tester_;
   scoped_ptr<net::URLRequest> request_;
 
+  base::WeakPtrFactory<TestRunner> weak_factory_;
+
   DISALLOW_COPY_AND_ASSIGN(TestRunner);
 };
 
@@ -303,9 +329,20 @@ void ConnectionTester::TestRunner::OnResponseCompleted(
     net::URLRequest* request) {
   int result = net::OK;
   if (!request->status().is_success()) {
-    DCHECK_NE(net::ERR_IO_PENDING, request->status().os_error());
-    result = request->status().os_error();
+    DCHECK_NE(net::ERR_IO_PENDING, request->status().error());
+    result = request->status().error();
   }
+
+  // Post a task to notify the parent rather than handling it right away,
+  // to avoid re-entrancy problems with URLRequest. (Don't want the caller
+  // to end up deleting the URLRequest while in the middle of processing).
+  MessageLoop::current()->PostTask(
+      FROM_HERE,
+      base::Bind(&TestRunner::OnExperimentCompletedWithResult,
+                 weak_factory_.GetWeakPtr(), result));
+}
+
+void ConnectionTester::TestRunner::OnExperimentCompletedWithResult(int result) {
   tester_->OnExperimentCompleted(result);
 }
 
