@@ -1,27 +1,29 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/net/connection_tester.h"
 
-#include "chrome/browser/browser_thread.h"
-#include "chrome/test/testing_pref_service.h"
+#include "chrome/test/base/testing_pref_service.h"
+#include "content/test/test_browser_thread.h"
 #include "net/base/cert_verifier.h"
 #include "net/base/cookie_monster.h"
-#include "net/base/dnsrr_resolver.h"
 #include "net/base/mock_host_resolver.h"
 #include "net/base/ssl_config_service_defaults.h"
 #include "net/ftp/ftp_network_layer.h"
 #include "net/http/http_auth_handler_factory.h"
 #include "net/http/http_network_layer.h"
+#include "net/http/http_network_session.h"
+#include "net/http/http_server_properties_impl.h"
 #include "net/proxy/proxy_config_service_fixed.h"
-#include "net/socket/client_socket_factory.h"
-#include "net/spdy/spdy_session_pool.h"
+#include "net/proxy/proxy_service.h"
 #include "net/test/test_server.h"
 #include "net/url_request/url_request_context.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
 
+using content::BrowserThread;
+
 namespace {
 
 // This is a testing delegate which simply counts how many times each of
@@ -85,61 +87,59 @@ class ConnectionTesterDelegate : public ConnectionTester::Delegate {
 class ConnectionTesterTest : public PlatformTest {
  public:
   ConnectionTesterTest()
-      : test_server_(net::TestServer::TYPE_HTTP,
+      : message_loop_(MessageLoop::TYPE_IO),
+        io_thread_(BrowserThread::IO, &message_loop_),
+        test_server_(net::TestServer::TYPE_HTTP,
             FilePath(FILE_PATH_LITERAL("net/data/url_request_unittest"))),
-        client_socket_factory_(net::ClientSocketFactory::GetDefaultFactory()),
-        proxy_script_fetcher_context_(new net::URLRequestContext),
-        message_loop_(MessageLoop::TYPE_IO),
-        io_thread_(BrowserThread::IO, &message_loop_) {
+        proxy_script_fetcher_context_(new net::URLRequestContext) {
     InitializeRequestContext();
   }
 
  protected:
+  // Destroy last the MessageLoop last to give a chance for objects like
+  // ObserverListThreadSave to shut down properly.  For example,
+  // SSLClientAuthCache calls RemoveObserver when destroyed, but if the
+  // MessageLoop is already destroyed, then the RemoveObserver will be a
+  // no-op, and the ObserverList will contain invalid entries.
+  MessageLoop message_loop_;
+  content::TestBrowserThread io_thread_;
   net::TestServer test_server_;
   ConnectionTesterDelegate test_delegate_;
-  net::ClientSocketFactory* const client_socket_factory_;
   net::MockHostResolver host_resolver_;
   net::CertVerifier cert_verifier_;
-  net::DnsRRResolver dnsrr_resolver_;
-  scoped_refptr<net::ProxyService> proxy_service_;
+  scoped_ptr<net::ProxyService> proxy_service_;
   scoped_refptr<net::SSLConfigService> ssl_config_service_;
   scoped_ptr<net::HttpTransactionFactory> http_transaction_factory_;
   net::HttpAuthHandlerRegistryFactory http_auth_handler_factory_;
   scoped_refptr<net::URLRequestContext> proxy_script_fetcher_context_;
+  net::HttpServerPropertiesImpl http_server_properties_impl_;
 
  private:
   void InitializeRequestContext() {
     proxy_script_fetcher_context_->set_host_resolver(&host_resolver_);
     proxy_script_fetcher_context_->set_cert_verifier(&cert_verifier_);
-    proxy_script_fetcher_context_->set_dnsrr_resolver(&dnsrr_resolver_);
     proxy_script_fetcher_context_->set_http_auth_handler_factory(
         &http_auth_handler_factory_);
-    proxy_service_ = net::ProxyService::CreateDirect();
-    proxy_script_fetcher_context_->set_proxy_service(proxy_service_);
-    ssl_config_service_ = net::SSLConfigService::CreateSystemSSLConfigService();
+    proxy_service_.reset(net::ProxyService::CreateDirect());
+    proxy_script_fetcher_context_->set_proxy_service(proxy_service_.get());
+    ssl_config_service_ = new net::SSLConfigServiceDefaults;
+    net::HttpNetworkSession::Params session_params;
+    session_params.host_resolver = &host_resolver_;
+    session_params.cert_verifier = &cert_verifier_;
+    session_params.http_auth_handler_factory = &http_auth_handler_factory_;
+    session_params.ssl_config_service = ssl_config_service_;
+    session_params.proxy_service = proxy_service_.get();
+    session_params.http_server_properties = &http_server_properties_impl_;
+    scoped_refptr<net::HttpNetworkSession> network_session(
+        new net::HttpNetworkSession(session_params));
     http_transaction_factory_.reset(
-        new net::HttpNetworkLayer(
-            client_socket_factory_,
-            &host_resolver_,
-            &cert_verifier_,
-            &dnsrr_resolver_,
-            NULL /* DNS cert provenance checker */,
-            NULL /* ssl_host_info_factory */,
-            proxy_service_.get(),
-            ssl_config_service_,
-            new net::SpdySessionPool(ssl_config_service_),
-            &http_auth_handler_factory_,
-            NULL /* NetworkDelegate */,
-            NULL /* NetLog */));
+        new net::HttpNetworkLayer(network_session));
     proxy_script_fetcher_context_->set_http_transaction_factory(
         http_transaction_factory_.get());
     // In-memory cookie store.
     proxy_script_fetcher_context_->set_cookie_store(
         new net::CookieMonster(NULL, NULL));
   }
-
-  MessageLoop message_loop_;
-  BrowserThread io_thread_;
 };
 
 TEST_F(ConnectionTesterTest, RunAllTests) {
@@ -176,7 +176,9 @@ TEST_F(ConnectionTesterTest, DeleteWhileInProgress) {
   // TODO(eroman): Is this URL right?
   tester->RunAllTests(test_server_.GetURL("echoall"));
 
-  MessageLoop::current()->RunAllPending();
+  // Don't run the message loop at all.  Otherwise the experiment's request may
+  // complete and post a task to run the next experiment before we quit the
+  // message loop.
 
   EXPECT_EQ(1, test_delegate_.start_connection_test_suite_count());
   EXPECT_EQ(1, test_delegate_.start_connection_test_experiment_count());
@@ -194,7 +196,7 @@ TEST_F(ConnectionTesterTest, DeleteWhileInProgress) {
   // |backup_task| that it will try to deref during the destructor, but
   // depending on the order that pending tasks were deleted in, it might
   // already be invalid! See http://crbug.com/43291.
-  MessageLoop::current()->PostTask(FROM_HERE, new MessageLoop::QuitTask());
+  MessageLoop::current()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
   MessageLoop::current()->Run();
 }
 
