@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,18 +13,20 @@
 #include "net/base/net_log.h"
 #include "webkit/glue/resource_loader_bridge.h"
 
+namespace content {
+struct ResourceResponse;
+}
+
 namespace net {
 class URLRequest;
 }  // namespace net
 
-struct ResourceResponse;
-
 // LoadTimingObserver watches the NetLog event stream and collects the network
 // timing information.
 //
 // LoadTimingObserver lives completely on the IOThread and ignores events from
 // other threads.  It is not safe to use from other threads.
-class LoadTimingObserver : public ChromeNetLog::ThreadSafeObserver {
+class LoadTimingObserver : public ChromeNetLog::ThreadSafeObserverImpl {
  public:
   struct URLRequestRecord {
     URLRequestRecord();
@@ -36,6 +38,19 @@ class LoadTimingObserver : public ChromeNetLog::ThreadSafeObserver {
     base::TimeTicks base_ticks;
   };
 
+  struct HTTPStreamJobRecord {
+    HTTPStreamJobRecord();
+
+    uint32 socket_log_id;
+    bool socket_reused;
+    base::TimeTicks connect_start;
+    base::TimeTicks connect_end;
+    base::TimeTicks dns_start;
+    base::TimeTicks dns_end;
+    base::TimeTicks ssl_start;
+    base::TimeTicks ssl_end;
+  };
+
   struct ConnectJobRecord {
     base::TimeTicks dns_start;
     base::TimeTicks dns_end;
@@ -47,7 +62,7 @@ class LoadTimingObserver : public ChromeNetLog::ThreadSafeObserver {
   };
 
   LoadTimingObserver();
-  ~LoadTimingObserver();
+  virtual ~LoadTimingObserver();
 
   URLRequestRecord* GetURLRequestRecord(uint32 source_id);
 
@@ -56,12 +71,14 @@ class LoadTimingObserver : public ChromeNetLog::ThreadSafeObserver {
                           const base::TimeTicks& time,
                           const net::NetLog::Source& source,
                           net::NetLog::EventPhase phase,
-                          net::NetLog::EventParameters* params);
+                          net::NetLog::EventParameters* params) OVERRIDE;
 
   static void PopulateTimingInfo(net::URLRequest* request,
-                                 ResourceResponse* response);
+                                 content::ResourceResponse* response);
 
  private:
+  FRIEND_TEST_ALL_PREFIXES(LoadTimingObserverTest,
+                           HTTPStreamJobRecord);
   FRIEND_TEST_ALL_PREFIXES(LoadTimingObserverTest,
                            ConnectJobRecord);
   FRIEND_TEST_ALL_PREFIXES(LoadTimingObserverTest,
@@ -73,6 +90,12 @@ class LoadTimingObserver : public ChromeNetLog::ThreadSafeObserver {
                             net::NetLog::EventPhase phase,
                             net::NetLog::EventParameters* params);
 
+  void OnAddHTTPStreamJobEntry(net::NetLog::EventType type,
+                               const base::TimeTicks& time,
+                               const net::NetLog::Source& source,
+                               net::NetLog::EventPhase phase,
+                               net::NetLog::EventParameters* params);
+
   void OnAddConnectJobEntry(net::NetLog::EventType type,
                             const base::TimeTicks& time,
                             const net::NetLog::Source& source,
@@ -89,9 +112,11 @@ class LoadTimingObserver : public ChromeNetLog::ThreadSafeObserver {
   void DeleteURLRequestRecord(uint32 source_id);
 
   typedef base::hash_map<uint32, URLRequestRecord> URLRequestToRecordMap;
+  typedef base::hash_map<uint32, HTTPStreamJobRecord> HTTPStreamJobToRecordMap;
   typedef base::hash_map<uint32, ConnectJobRecord> ConnectJobToRecordMap;
   typedef base::hash_map<uint32, SocketRecord> SocketToRecordMap;
   URLRequestToRecordMap url_request_to_record_;
+  HTTPStreamJobToRecordMap http_stream_job_to_record_;
   ConnectJobToRecordMap connect_job_to_record_;
   SocketToRecordMap socket_to_record_;
   uint32 last_connect_job_id_;
