@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -50,10 +50,13 @@ PassiveLogCollector::SourceInfo::~SourceInfo() {}
 //----------------------------------------------------------------------------
 
 PassiveLogCollector::PassiveLogCollector()
-    : ThreadSafeObserver(net::NetLog::LOG_BASIC),
+    : ThreadSafeObserverImpl(net::NetLog::LOG_BASIC),
       ALLOW_THIS_IN_INITIALIZER_LIST(connect_job_tracker_(this)),
       ALLOW_THIS_IN_INITIALIZER_LIST(url_request_tracker_(this)),
       ALLOW_THIS_IN_INITIALIZER_LIST(socket_stream_tracker_(this)),
+      ALLOW_THIS_IN_INITIALIZER_LIST(http_stream_job_tracker_(this)),
+      ALLOW_THIS_IN_INITIALIZER_LIST(
+          exponential_backoff_throttling_tracker_(this)),
       num_events_seen_(0) {
 
   // Define the mapping between source types and the tracker objects.
@@ -63,13 +66,25 @@ PassiveLogCollector::PassiveLogCollector()
   trackers_[net::NetLog::SOURCE_SOCKET_STREAM] = &socket_stream_tracker_;
   trackers_[net::NetLog::SOURCE_CONNECT_JOB] = &connect_job_tracker_;
   trackers_[net::NetLog::SOURCE_SOCKET] = &socket_tracker_;
-  trackers_[net::NetLog::SOURCE_INIT_PROXY_RESOLVER] =
-      &init_proxy_resolver_tracker_;
+  trackers_[net::NetLog::SOURCE_PROXY_SCRIPT_DECIDER] =
+      &proxy_script_decider_tracker_;
   trackers_[net::NetLog::SOURCE_SPDY_SESSION] = &spdy_session_tracker_;
   trackers_[net::NetLog::SOURCE_HOST_RESOLVER_IMPL_REQUEST] =
       &dns_request_tracker_;
   trackers_[net::NetLog::SOURCE_HOST_RESOLVER_IMPL_JOB] = &dns_job_tracker_;
   trackers_[net::NetLog::SOURCE_DISK_CACHE_ENTRY] = &disk_cache_entry_tracker_;
+  trackers_[net::NetLog::SOURCE_MEMORY_CACHE_ENTRY] = &mem_cache_entry_tracker_;
+  trackers_[net::NetLog::SOURCE_HTTP_STREAM_JOB] = &http_stream_job_tracker_;
+  trackers_[net::NetLog::SOURCE_EXPONENTIAL_BACKOFF_THROTTLING] =
+      &exponential_backoff_throttling_tracker_;
+  trackers_[net::NetLog::SOURCE_DNS_TRANSACTION] = &dns_transaction_tracker_;
+  trackers_[net::NetLog::SOURCE_ASYNC_HOST_RESOLVER_REQUEST] =
+      &async_host_resolver_request_tracker_;
+  trackers_[net::NetLog::SOURCE_UDP_SOCKET] = &udp_socket_tracker_;
+  trackers_[net::NetLog::SOURCE_CERT_VERIFIER_JOB] =
+      &cert_verifier_job_tracker_;
+  trackers_[net::NetLog::SOURCE_HTTP_PIPELINED_CONNECTION] =
+      &http_pipelined_connection_tracker_;
   // Make sure our mapping is up-to-date.
   for (size_t i = 0; i < arraysize(trackers_); ++i)
     DCHECK(trackers_[i]) << "Unhandled SourceType: " << i;
@@ -89,7 +104,7 @@ void PassiveLogCollector::OnAddEntry(
   ChromeNetLog::Entry entry(num_events_seen_++, type, time, source, phase,
                             params);
 
-  SourceTrackerInterface* tracker = GetTrackerForSourceType_(entry.source.type);
+  SourceTrackerInterface* tracker = GetTrackerForSourceType(entry.source.type);
   if (tracker)
     tracker->OnAddEntry(entry);
 }
@@ -101,10 +116,10 @@ void PassiveLogCollector::Clear() {
 }
 
 PassiveLogCollector::SourceTrackerInterface*
-PassiveLogCollector::GetTrackerForSourceType_(
+PassiveLogCollector::GetTrackerForSourceType(
     net::NetLog::SourceType source_type) {
-  DCHECK_LE(source_type, static_cast<int>(arraysize(trackers_)));
-  DCHECK_GE(source_type, 0);
+  CHECK_LT(source_type, static_cast<int>(arraysize(trackers_)));
+  CHECK_GE(source_type, 0);
   return trackers_[source_type];
 }
 
@@ -231,8 +246,8 @@ void PassiveLogCollector::SourceTracker::DeleteSourceInfo(
     return;
   }
   // The source should not be in the deletion queue.
-  DCHECK(std::find(deletion_queue_.begin(), deletion_queue_.end(),
-                   source_id) == deletion_queue_.end());
+  CHECK(std::find(deletion_queue_.begin(), deletion_queue_.end(),
+                  source_id) == deletion_queue_.end());
   ReleaseAllReferencesToDependencies(&(it->second));
   sources_.erase(it);
 }
@@ -285,7 +300,7 @@ void PassiveLogCollector::SourceTracker::EraseFromDeletionQueue(
   DeletionQueue::iterator it =
       std::remove(deletion_queue_.begin(), deletion_queue_.end(),
                   source_id);
-  DCHECK(it != deletion_queue_.end());
+  CHECK(it != deletion_queue_.end());
   deletion_queue_.erase(it);
 }
 
@@ -335,7 +350,7 @@ void PassiveLogCollector::SourceTracker::AddReferenceToSourceDependency(
   DCHECK(parent_);
   DCHECK_NE(source.type, net::NetLog::SOURCE_NONE);
   SourceTracker* tracker = static_cast<SourceTracker*>(
-      parent_->GetTrackerForSourceType_(source.type));
+      parent_->GetTrackerForSourceType(source.type));
   DCHECK(tracker);
 
   // Tell the owning tracker to increment the reference count of |source|.
@@ -345,8 +360,7 @@ void PassiveLogCollector::SourceTracker::AddReferenceToSourceDependency(
   info->dependencies.push_back(source);
 }
 
-void
-PassiveLogCollector::SourceTracker::ReleaseAllReferencesToDependencies(
+void PassiveLogCollector::SourceTracker::ReleaseAllReferencesToDependencies(
     SourceInfo* info) {
   // Release all references |info| was holding to other sources.
   for (SourceDependencyList::const_iterator it = info->dependencies.begin();
@@ -357,7 +371,7 @@ PassiveLogCollector::SourceTracker::ReleaseAllReferencesToDependencies(
     DCHECK(parent_);
     DCHECK_NE(source.type, net::NetLog::SOURCE_NONE);
     SourceTracker* tracker = static_cast<SourceTracker*>(
-        parent_->GetTrackerForSourceType_(source.type));
+        parent_->GetTrackerForSourceType(source.type));
     DCHECK(tracker);
 
     // Tell the owning tracker to decrement the reference count of |source|.
@@ -417,7 +431,9 @@ PassiveLogCollector::SocketTracker::DoAddEntry(const ChromeNetLog::Entry& entry,
   //               to summarize transaction read/writes for each SOCKET_IN_USE
   //               section.
   if (entry.type == net::NetLog::TYPE_SOCKET_BYTES_SENT ||
-      entry.type == net::NetLog::TYPE_SOCKET_BYTES_RECEIVED) {
+      entry.type == net::NetLog::TYPE_SOCKET_BYTES_RECEIVED ||
+      entry.type == net::NetLog::TYPE_SSL_SOCKET_BYTES_SENT ||
+      entry.type == net::NetLog::TYPE_SSL_SOCKET_BYTES_RECEIVED) {
     return ACTION_NONE;
   }
 
@@ -445,13 +461,18 @@ PassiveLogCollector::RequestTracker::RequestTracker(PassiveLogCollector* parent)
 PassiveLogCollector::SourceTracker::Action
 PassiveLogCollector::RequestTracker::DoAddEntry(
     const ChromeNetLog::Entry& entry, SourceInfo* out_info) {
-  if (entry.type == net::NetLog::TYPE_SOCKET_POOL_BOUND_TO_CONNECT_JOB ||
-      entry.type == net::NetLog::TYPE_SOCKET_POOL_BOUND_TO_SOCKET) {
+  if (entry.type == net::NetLog::TYPE_HTTP_STREAM_REQUEST_BOUND_TO_JOB) {
     const net::NetLog::Source& source_dependency =
         static_cast<net::NetLogSourceParameter*>(entry.params.get())->value();
     AddReferenceToSourceDependency(source_dependency, out_info);
   }
 
+  // Don't keep read bytes around in the log, to save memory.
+  if (entry.type == net::NetLog::TYPE_URL_REQUEST_JOB_BYTES_READ ||
+      entry.type == net::NetLog::TYPE_URL_REQUEST_JOB_FILTERED_BYTES_READ) {
+    return ACTION_NONE;
+  }
+
   AddEntryToSourceInfo(entry, out_info);
 
   // If the request has ended, move it to the graveyard.
@@ -469,27 +490,27 @@ PassiveLogCollector::RequestTracker::DoAddEntry(
 }
 
 //----------------------------------------------------------------------------
-// InitProxyResolverTracker
+// ProxyScriptDeciderTracker
 //----------------------------------------------------------------------------
 
-const size_t PassiveLogCollector::InitProxyResolverTracker::kMaxNumSources = 20;
-const size_t PassiveLogCollector::InitProxyResolverTracker::kMaxGraveyardSize =
-    3;
+const size_t PassiveLogCollector::ProxyScriptDeciderTracker::kMaxNumSources
+    = 20;
+const size_t PassiveLogCollector::ProxyScriptDeciderTracker::kMaxGraveyardSize
+    = 3;
 
-PassiveLogCollector::InitProxyResolverTracker::InitProxyResolverTracker()
+PassiveLogCollector::ProxyScriptDeciderTracker::ProxyScriptDeciderTracker()
     : SourceTracker(kMaxNumSources, kMaxGraveyardSize, NULL) {
 }
 
 PassiveLogCollector::SourceTracker::Action
-PassiveLogCollector::InitProxyResolverTracker::DoAddEntry(
+PassiveLogCollector::ProxyScriptDeciderTracker::DoAddEntry(
     const ChromeNetLog::Entry& entry, SourceInfo* out_info) {
   AddEntryToSourceInfo(entry, out_info);
-  if (entry.type == net::NetLog::TYPE_INIT_PROXY_RESOLVER &&
+  if (entry.type == net::NetLog::TYPE_PROXY_SCRIPT_DECIDER &&
       entry.phase == net::NetLog::PHASE_END) {
     return ACTION_MOVE_TO_GRAVEYARD;
-  } else {
-    return ACTION_NONE;
   }
+  return ACTION_NONE;
 }
 
 //----------------------------------------------------------------------------
@@ -510,9 +531,8 @@ PassiveLogCollector::SpdySessionTracker::DoAddEntry(
   if (entry.type == net::NetLog::TYPE_SPDY_SESSION &&
       entry.phase == net::NetLog::PHASE_END) {
     return ACTION_MOVE_TO_GRAVEYARD;
-  } else {
-    return ACTION_NONE;
   }
+  return ACTION_NONE;
 }
 
 //----------------------------------------------------------------------------
@@ -533,9 +553,8 @@ PassiveLogCollector::DNSRequestTracker::DoAddEntry(
   if (entry.type == net::NetLog::TYPE_HOST_RESOLVER_IMPL_REQUEST &&
       entry.phase == net::NetLog::PHASE_END) {
     return ACTION_MOVE_TO_GRAVEYARD;
-  } else {
-    return ACTION_NONE;
   }
+  return ACTION_NONE;
 }
 
 //----------------------------------------------------------------------------
@@ -556,9 +575,8 @@ PassiveLogCollector::DNSJobTracker::DoAddEntry(const ChromeNetLog::Entry& entry,
   if (entry.type == net::NetLog::TYPE_HOST_RESOLVER_IMPL_JOB &&
       entry.phase == net::NetLog::PHASE_END) {
     return ACTION_MOVE_TO_GRAVEYARD;
-  } else {
-    return ACTION_NONE;
   }
+  return ACTION_NONE;
 }
 
 //----------------------------------------------------------------------------
@@ -578,10 +596,221 @@ PassiveLogCollector::DiskCacheEntryTracker::DoAddEntry(
   AddEntryToSourceInfo(entry, out_info);
 
   // If the request has ended, move it to the graveyard.
-  if (entry.type == net::NetLog::TYPE_DISK_CACHE_ENTRY &&
+  if (entry.type == net::NetLog::TYPE_DISK_CACHE_ENTRY_IMPL &&
+      entry.phase == net::NetLog::PHASE_END) {
+    return ACTION_MOVE_TO_GRAVEYARD;
+  }
+  return ACTION_NONE;
+}
+
+//----------------------------------------------------------------------------
+// MemCacheEntryTracker
+//----------------------------------------------------------------------------
+
+const size_t PassiveLogCollector::MemCacheEntryTracker::kMaxNumSources = 100;
+const size_t PassiveLogCollector::MemCacheEntryTracker::kMaxGraveyardSize = 25;
+
+PassiveLogCollector::MemCacheEntryTracker::MemCacheEntryTracker()
+    : SourceTracker(kMaxNumSources, kMaxGraveyardSize, NULL) {
+}
+
+PassiveLogCollector::SourceTracker::Action
+PassiveLogCollector::MemCacheEntryTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry, SourceInfo* out_info) {
+  AddEntryToSourceInfo(entry, out_info);
+
+  // If the request has ended, move it to the graveyard.
+  if (entry.type == net::NetLog::TYPE_DISK_CACHE_MEM_ENTRY_IMPL &&
+      entry.phase == net::NetLog::PHASE_END) {
+    return ACTION_MOVE_TO_GRAVEYARD;
+  }
+  return ACTION_NONE;
+}
+
+//----------------------------------------------------------------------------
+// HttpStreamJobTracker
+//----------------------------------------------------------------------------
+
+const size_t PassiveLogCollector::HttpStreamJobTracker::kMaxNumSources = 100;
+const size_t PassiveLogCollector::HttpStreamJobTracker::kMaxGraveyardSize = 25;
+
+PassiveLogCollector::HttpStreamJobTracker::HttpStreamJobTracker(
+    PassiveLogCollector* parent)
+    : SourceTracker(kMaxNumSources, kMaxGraveyardSize, parent) {
+}
+
+PassiveLogCollector::SourceTracker::Action
+PassiveLogCollector::HttpStreamJobTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry, SourceInfo* out_info) {
+  if (entry.type == net::NetLog::TYPE_SOCKET_POOL_BOUND_TO_CONNECT_JOB ||
+      entry.type == net::NetLog::TYPE_SOCKET_POOL_BOUND_TO_SOCKET) {
+    const net::NetLog::Source& source_dependency =
+        static_cast<net::NetLogSourceParameter*>(entry.params.get())->value();
+    AddReferenceToSourceDependency(source_dependency, out_info);
+  }
+
+  AddEntryToSourceInfo(entry, out_info);
+
+  // If the request has ended, move it to the graveyard.
+  if (entry.type == net::NetLog::TYPE_HTTP_STREAM_JOB &&
+      entry.phase == net::NetLog::PHASE_END) {
+    return ACTION_MOVE_TO_GRAVEYARD;
+  }
+  return ACTION_NONE;
+}
+
+//----------------------------------------------------------------------------
+// ExponentialBackoffThrottlingTracker
+//----------------------------------------------------------------------------
+
+const size_t PassiveLogCollector::
+    ExponentialBackoffThrottlingTracker::kMaxNumSources = 100;
+const size_t PassiveLogCollector::
+    ExponentialBackoffThrottlingTracker::kMaxGraveyardSize = 25;
+
+PassiveLogCollector::
+    ExponentialBackoffThrottlingTracker::ExponentialBackoffThrottlingTracker(
+        PassiveLogCollector* parent)
+        : SourceTracker(kMaxNumSources, kMaxGraveyardSize, parent) {
+}
+
+PassiveLogCollector::SourceTracker::Action
+PassiveLogCollector::ExponentialBackoffThrottlingTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry, SourceInfo* out_info) {
+  AddEntryToSourceInfo(entry, out_info);
+  return ACTION_NONE;
+}
+
+//----------------------------------------------------------------------------
+// DnsTransactionTracker
+//----------------------------------------------------------------------------
+
+const size_t PassiveLogCollector::DnsTransactionTracker::kMaxNumSources = 100;
+const size_t PassiveLogCollector::DnsTransactionTracker::kMaxGraveyardSize = 15;
+
+PassiveLogCollector::DnsTransactionTracker::DnsTransactionTracker()
+    : SourceTracker(kMaxNumSources, kMaxGraveyardSize, NULL) {
+}
+
+PassiveLogCollector::SourceTracker::Action
+PassiveLogCollector::DnsTransactionTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry,
+    SourceInfo* out_info) {
+  AddEntryToSourceInfo(entry, out_info);
+  if (entry.type == net::NetLog::TYPE_DNS_TRANSACTION &&
+      entry.phase == net::NetLog::PHASE_END) {
+    return ACTION_MOVE_TO_GRAVEYARD;
+  }
+  return ACTION_NONE;
+}
+
+//----------------------------------------------------------------------------
+// AsyncHostResolverRequestTracker
+//----------------------------------------------------------------------------
+
+const size_t
+PassiveLogCollector::AsyncHostResolverRequestTracker::kMaxNumSources = 100;
+
+const size_t
+PassiveLogCollector::AsyncHostResolverRequestTracker::kMaxGraveyardSize = 15;
+
+PassiveLogCollector::
+    AsyncHostResolverRequestTracker::AsyncHostResolverRequestTracker()
+        : SourceTracker(kMaxNumSources, kMaxGraveyardSize, NULL) {
+}
+
+PassiveLogCollector::SourceTracker::Action
+PassiveLogCollector::AsyncHostResolverRequestTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry,
+    SourceInfo* out_info) {
+  AddEntryToSourceInfo(entry, out_info);
+  if (entry.type == net::NetLog::TYPE_ASYNC_HOST_RESOLVER_REQUEST &&
+      entry.phase == net::NetLog::PHASE_END) {
+    return ACTION_MOVE_TO_GRAVEYARD;
+  }
+  return ACTION_NONE;
+}
+
+//----------------------------------------------------------------------------
+// UDPSocketTracker
+//----------------------------------------------------------------------------
+
+const size_t PassiveLogCollector::UDPSocketTracker::kMaxNumSources = 200;
+const size_t PassiveLogCollector::UDPSocketTracker::kMaxGraveyardSize = 15;
+
+PassiveLogCollector::UDPSocketTracker::UDPSocketTracker()
+    : SourceTracker(kMaxNumSources, kMaxGraveyardSize, NULL) {
+}
+
+PassiveLogCollector::UDPSocketTracker::Action
+PassiveLogCollector::UDPSocketTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry,
+    SourceInfo* out_info) {
+  if (entry.type == net::NetLog::TYPE_UDP_BYTES_SENT ||
+      entry.type == net::NetLog::TYPE_UDP_BYTES_RECEIVED) {
+    return ACTION_NONE;
+  }
+
+  AddEntryToSourceInfo(entry, out_info);
+
+  if (entry.type == net::NetLog::TYPE_SOCKET_ALIVE &&
       entry.phase == net::NetLog::PHASE_END) {
     return ACTION_MOVE_TO_GRAVEYARD;
   }
 
   return ACTION_NONE;
 }
+
+//----------------------------------------------------------------------------
+// CertVerifierJobTracker
+//----------------------------------------------------------------------------
+
+const size_t
+PassiveLogCollector::CertVerifierJobTracker::kMaxNumSources = 100;
+
+const size_t
+PassiveLogCollector::CertVerifierJobTracker::kMaxGraveyardSize = 15;
+
+PassiveLogCollector::
+    CertVerifierJobTracker::CertVerifierJobTracker()
+        : SourceTracker(kMaxNumSources, kMaxGraveyardSize, NULL) {
+}
+
+PassiveLogCollector::SourceTracker::Action
+PassiveLogCollector::CertVerifierJobTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry,
+    SourceInfo* out_info) {
+  AddEntryToSourceInfo(entry, out_info);
+  if (entry.type == net::NetLog::TYPE_CERT_VERIFIER_JOB &&
+      entry.phase == net::NetLog::PHASE_END) {
+    return ACTION_MOVE_TO_GRAVEYARD;
+  }
+  return ACTION_NONE;
+}
+
+//----------------------------------------------------------------------------
+// HttpPipelinedConnectionTracker
+//----------------------------------------------------------------------------
+
+const size_t
+PassiveLogCollector::HttpPipelinedConnectionTracker::kMaxNumSources = 100;
+
+const size_t
+PassiveLogCollector::HttpPipelinedConnectionTracker::kMaxGraveyardSize = 25;
+
+PassiveLogCollector::
+    HttpPipelinedConnectionTracker::HttpPipelinedConnectionTracker()
+        : SourceTracker(kMaxNumSources, kMaxGraveyardSize, NULL) {
+}
+
+PassiveLogCollector::SourceTracker::Action
+PassiveLogCollector::HttpPipelinedConnectionTracker::DoAddEntry(
+    const ChromeNetLog::Entry& entry,
+    SourceInfo* out_info) {
+  AddEntryToSourceInfo(entry, out_info);
+  if (entry.type == net::NetLog::TYPE_HTTP_PIPELINED_CONNECTION &&
+      entry.phase == net::NetLog::PHASE_END) {
+    return ACTION_MOVE_TO_GRAVEYARD;
+  }
+  return ACTION_NONE;
+}
