@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,7 +12,7 @@
 
 #include "base/gtest_prod_util.h"
 #include "base/hash_tables.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/time.h"
 #include "chrome/browser/net/chrome_net_log.h"
 #include "net/base/net_log.h"
@@ -36,7 +36,7 @@
 // The PassiveLogCollector is owned by the ChromeNetLog itself, and is not
 // thread safe.  The ChromeNetLog is responsible for calling it in a thread safe
 // manner.
-class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
+class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserverImpl {
  public:
   typedef std::vector<net::NetLog::Source> SourceDependencyList;
 
@@ -90,12 +90,12 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
   class GlobalSourceTracker : public SourceTrackerInterface {
    public:
     GlobalSourceTracker();
-    ~GlobalSourceTracker();
+    virtual ~GlobalSourceTracker();
 
     // SourceTrackerInterface implementation:
-    virtual void OnAddEntry(const ChromeNetLog::Entry& entry);
-    virtual void Clear();
-    virtual void AppendAllEntries(ChromeNetLog::EntryList* out) const;
+    virtual void OnAddEntry(const ChromeNetLog::Entry& entry) OVERRIDE;
+    virtual void Clear() OVERRIDE;
+    virtual void AppendAllEntries(ChromeNetLog::EntryList* out) const OVERRIDE;
 
    private:
     typedef std::deque<ChromeNetLog::Entry> CircularEntryList;
@@ -119,9 +119,9 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
     virtual ~SourceTracker();
 
     // SourceTrackerInterface implementation:
-    virtual void OnAddEntry(const ChromeNetLog::Entry& entry);
-    virtual void Clear();
-    virtual void AppendAllEntries(ChromeNetLog::EntryList* out) const;
+    virtual void OnAddEntry(const ChromeNetLog::Entry& entry) OVERRIDE;
+    virtual void Clear() OVERRIDE;
+    virtual void AppendAllEntries(ChromeNetLog::EntryList* out) const OVERRIDE;
 
 #ifdef UNIT_TEST
     // Helper used to inspect the current state by unit-tests.
@@ -200,10 +200,9 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
     explicit ConnectJobTracker(PassiveLogCollector* parent);
 
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
    private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
     DISALLOW_COPY_AND_ASSIGN(ConnectJobTracker);
   };
 
@@ -215,11 +214,10 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
     SocketTracker();
 
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
-
    private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
     DISALLOW_COPY_AND_ASSIGN(SocketTracker);
   };
 
@@ -231,29 +229,27 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
     explicit RequestTracker(PassiveLogCollector* parent);
 
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
-
    private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
     DISALLOW_COPY_AND_ASSIGN(RequestTracker);
   };
 
   // Specialization of SourceTracker for handling
-  // SOURCE_INIT_PROXY_RESOLVER.
-  class InitProxyResolverTracker : public SourceTracker {
+  // SOURCE_PROXY_SCRIPT_DECIDER.
+  class ProxyScriptDeciderTracker : public SourceTracker {
    public:
     static const size_t kMaxNumSources;
     static const size_t kMaxGraveyardSize;
 
-    InitProxyResolverTracker();
-
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
+    ProxyScriptDeciderTracker();
 
    private:
-    DISALLOW_COPY_AND_ASSIGN(InitProxyResolverTracker);
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
+    DISALLOW_COPY_AND_ASSIGN(ProxyScriptDeciderTracker);
   };
 
   // Tracks the log entries for the last seen SOURCE_SPDY_SESSION.
@@ -264,11 +260,10 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
     SpdySessionTracker();
 
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
-
    private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
     DISALLOW_COPY_AND_ASSIGN(SpdySessionTracker);
   };
 
@@ -280,11 +275,10 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
     DNSRequestTracker();
 
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
-
    private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
     DISALLOW_COPY_AND_ASSIGN(DNSRequestTracker);
   };
 
@@ -296,11 +290,10 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
     DNSJobTracker();
 
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
-
    private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
     DISALLOW_COPY_AND_ASSIGN(DNSJobTracker);
   };
 
@@ -312,23 +305,140 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
     DiskCacheEntryTracker();
 
-   protected:
-    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
-                              SourceInfo* out_info);
-
    private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
     DISALLOW_COPY_AND_ASSIGN(DiskCacheEntryTracker);
   };
 
+  // Tracks the log entries for the last seen SOURCE_DISK_CACHE_ENTRY.
+  class MemCacheEntryTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    MemCacheEntryTracker();
+
+   protected:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(MemCacheEntryTracker);
+  };
+
+  class HttpStreamJobTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    explicit HttpStreamJobTracker(PassiveLogCollector* parent);
+
+   private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+    DISALLOW_COPY_AND_ASSIGN(HttpStreamJobTracker);
+  };
+
+  class ExponentialBackoffThrottlingTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    explicit ExponentialBackoffThrottlingTracker(PassiveLogCollector* parent);
+
+   private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+    DISALLOW_COPY_AND_ASSIGN(ExponentialBackoffThrottlingTracker);
+  };
+
+  // Tracks the log entries for the last seen SOURCE_DNS_TRANSACTION.
+  class DnsTransactionTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    DnsTransactionTracker();
+
+   private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
+    DISALLOW_COPY_AND_ASSIGN(DnsTransactionTracker);
+  };
+
+  // Tracks the log entries for the last seen SOURCE_ASYNC_HOST_RESOLVER_REQUEST
+  class AsyncHostResolverRequestTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    AsyncHostResolverRequestTracker();
+
+   private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
+    DISALLOW_COPY_AND_ASSIGN(AsyncHostResolverRequestTracker);
+  };
+
+
+  // Tracks the log entries for the last seen SOURCE_UDP_SOCKET.
+  class UDPSocketTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    UDPSocketTracker();
+
+   private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
+    DISALLOW_COPY_AND_ASSIGN(UDPSocketTracker);
+  };
+
+  // Tracks the log entries for the last seen SOURCE_CERT_VERIFIER_JOB.
+  class CertVerifierJobTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    CertVerifierJobTracker();
+
+   private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
+    DISALLOW_COPY_AND_ASSIGN(CertVerifierJobTracker);
+  };
+
+  // Tracks the log entries for the last seen SOURCE_HTTP_PIPELINED_CONNECTION.
+  class HttpPipelinedConnectionTracker : public SourceTracker {
+   public:
+    static const size_t kMaxNumSources;
+    static const size_t kMaxGraveyardSize;
+
+    HttpPipelinedConnectionTracker();
+
+   private:
+    virtual Action DoAddEntry(const ChromeNetLog::Entry& entry,
+                              SourceInfo* out_info) OVERRIDE;
+
+    DISALLOW_COPY_AND_ASSIGN(HttpPipelinedConnectionTracker);
+  };
+
   PassiveLogCollector();
-  ~PassiveLogCollector();
+  virtual ~PassiveLogCollector();
 
   // ThreadSafeObserver implementation:
   virtual void OnAddEntry(net::NetLog::EventType type,
                           const base::TimeTicks& time,
                           const net::NetLog::Source& source,
                           net::NetLog::EventPhase phase,
-                          net::NetLog::EventParameters* params);
+                          net::NetLog::EventParameters* params) OVERRIDE;
 
   // Clears all of the passively logged data.
   void Clear();
@@ -339,7 +449,7 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
 
  private:
   // Returns the tracker to use for sources of type |source_type|, or NULL.
-  SourceTrackerInterface* GetTrackerForSourceType_(
+  SourceTrackerInterface* GetTrackerForSourceType(
       net::NetLog::SourceType source_type);
 
   FRIEND_TEST_ALL_PREFIXES(PassiveLogCollectorTest,
@@ -352,11 +462,19 @@ class PassiveLogCollector : public ChromeNetLog::ThreadSafeObserver {
   SocketTracker socket_tracker_;
   RequestTracker url_request_tracker_;
   RequestTracker socket_stream_tracker_;
-  InitProxyResolverTracker init_proxy_resolver_tracker_;
+  ProxyScriptDeciderTracker proxy_script_decider_tracker_;
   SpdySessionTracker spdy_session_tracker_;
   DNSRequestTracker dns_request_tracker_;
   DNSJobTracker dns_job_tracker_;
   DiskCacheEntryTracker disk_cache_entry_tracker_;
+  MemCacheEntryTracker mem_cache_entry_tracker_;
+  HttpStreamJobTracker http_stream_job_tracker_;
+  ExponentialBackoffThrottlingTracker exponential_backoff_throttling_tracker_;
+  DnsTransactionTracker dns_transaction_tracker_;
+  AsyncHostResolverRequestTracker async_host_resolver_request_tracker_;
+  UDPSocketTracker udp_socket_tracker_;
+  CertVerifierJobTracker cert_verifier_job_tracker_;
+  HttpPipelinedConnectionTracker http_pipelined_connection_tracker_;
 
   // This array maps each NetLog::SourceType to one of the tracker instances
   // defined above. Use of this array avoid duplicating the list of trackers
