@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,15 +8,15 @@
 #include <sstream>
 #include <string>
 
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/scoped_ptr.h"
 #include "base/string_number_conversions.h"
 #include "base/timer.h"
 #include "base/values.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/net/predictor_api.h"
+#include "chrome/browser/net/predictor.h"
 #include "chrome/browser/net/url_info.h"
 #include "chrome/common/net/predictor_common.h"
+#include "content/test/test_browser_thread.h"
 #include "net/base/address_list.h"
 #include "net/base/mock_host_resolver.h"
 #include "net/base/winsock_init.h"
@@ -24,6 +24,7 @@
 
 using base::Time;
 using base::TimeDelta;
+using content::BrowserThread;
 
 namespace chrome_browser_net {
 
@@ -62,10 +63,9 @@ class WaitForResolutionHelper {
 class PredictorTest : public testing::Test {
  public:
   PredictorTest()
-      : io_thread_(BrowserThread::IO, &loop_),
-        host_resolver_(new net::MockCachingHostResolver()),
-        default_max_queueing_delay_(TimeDelta::FromMilliseconds(
-            PredictorInit::kMaxSpeculativeResolveQueueDelayMs)) {
+      : ui_thread_(BrowserThread::UI, &loop_),
+        io_thread_(BrowserThread::IO, &loop_),
+        host_resolver_(new net::MockCachingHostResolver()) {
   }
 
  protected:
@@ -73,6 +73,10 @@ class PredictorTest : public testing::Test {
 #if defined(OS_WIN)
     net::EnsureWinsockInit();
 #endif
+    Predictor::set_max_parallel_resolves(
+        Predictor::kMaxSpeculativeParallelResolves);
+    Predictor::set_max_queueing_delay(
+        Predictor::kMaxSpeculativeResolveQueueDelayMs);
     // Since we are using a caching HostResolver, the following latencies will
     // only be incurred by the first request, after which the result will be
     // cached internally by |host_resolver_|.
@@ -85,7 +89,7 @@ class PredictorTest : public testing::Test {
 
   void WaitForResolution(Predictor* predictor, const UrlList& hosts) {
     HelperTimer* timer = new HelperTimer();
-    timer->Start(TimeDelta::FromMilliseconds(100),
+    timer->Start(FROM_HERE, TimeDelta::FromMilliseconds(100),
                  new WaitForResolutionHelper(predictor, hosts, timer),
                  &WaitForResolutionHelper::Run);
     MessageLoop::current()->Run();
@@ -95,65 +99,51 @@ class PredictorTest : public testing::Test {
   // IMPORTANT: do not move this below |host_resolver_|; the host resolver
   // must not outlive the message loop, otherwise bad things can happen
   // (like posting to a deleted message loop).
-  MessageLoop loop_;
-  BrowserThread io_thread_;
+  MessageLoopForUI loop_;
+  content::TestBrowserThread ui_thread_;
+  content::TestBrowserThread io_thread_;
 
  protected:
   scoped_ptr<net::MockCachingHostResolver> host_resolver_;
-
-  // Shorthand to access TimeDelta of PredictorInit::kMaxQueueingDelayMs.
-  // (It would be a static constant... except style rules preclude that :-/ ).
-  const TimeDelta default_max_queueing_delay_;
 };
 
 //------------------------------------------------------------------------------
 
 TEST_F(PredictorTest, StartupShutdownTest) {
-  scoped_refptr<Predictor> testing_master(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
-  testing_master->Shutdown();
+  Predictor testing_master(true);
+  testing_master.Shutdown();
 }
 
 
 TEST_F(PredictorTest, ShutdownWhenResolutionIsPendingTest) {
-  scoped_refptr<net::WaitingHostResolverProc> resolver_proc(
-      new net::WaitingHostResolverProc(NULL));
-  host_resolver_->Reset(resolver_proc);
+  scoped_ptr<net::HostResolver> host_resolver(new net::HangingHostResolver());
 
-  scoped_refptr<Predictor> testing_master(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
+  Predictor testing_master(true);
+  testing_master.SetHostResolver(host_resolver.get());
 
   GURL localhost("http://localhost:80");
   UrlList names;
   names.push_back(localhost);
 
-  testing_master->ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
+  testing_master.ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
 
-  MessageLoop::current()->PostDelayedTask(FROM_HERE,
-                                          new MessageLoop::QuitTask(), 500);
+  MessageLoop::current()->PostDelayedTask(
+      FROM_HERE,
+      MessageLoop::QuitClosure(),
+      base::TimeDelta::FromMilliseconds(500));
   MessageLoop::current()->Run();
 
-  EXPECT_FALSE(testing_master->WasFound(localhost));
+  EXPECT_FALSE(testing_master.WasFound(localhost));
 
-  testing_master->Shutdown();
+  testing_master.Shutdown();
 
   // Clean up after ourselves.
-  resolver_proc->Signal();
   MessageLoop::current()->RunAllPending();
 }
 
 TEST_F(PredictorTest, SingleLookupTest) {
-  scoped_refptr<Predictor> testing_master(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
+  Predictor testing_master(true);
+  testing_master.SetHostResolver(host_resolver_.get());
 
   GURL goog("http://www.google.com:80");
 
@@ -162,30 +152,27 @@ TEST_F(PredictorTest, SingleLookupTest) {
 
   // Try to flood the predictor with many concurrent requests.
   for (int i = 0; i < 10; i++)
-    testing_master->ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
+    testing_master.ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
 
-  WaitForResolution(testing_master, names);
+  WaitForResolution(&testing_master, names);
 
-  EXPECT_TRUE(testing_master->WasFound(goog));
+  EXPECT_TRUE(testing_master.WasFound(goog));
 
   MessageLoop::current()->RunAllPending();
 
-  EXPECT_GT(testing_master->peak_pending_lookups(), names.size() / 2);
-  EXPECT_LE(testing_master->peak_pending_lookups(), names.size());
-  EXPECT_LE(testing_master->peak_pending_lookups(),
-            testing_master->max_concurrent_dns_lookups());
+  EXPECT_GT(testing_master.peak_pending_lookups(), names.size() / 2);
+  EXPECT_LE(testing_master.peak_pending_lookups(), names.size());
+  EXPECT_LE(testing_master.peak_pending_lookups(),
+            testing_master.max_concurrent_dns_lookups());
 
-  testing_master->Shutdown();
+  testing_master.Shutdown();
 }
 
 TEST_F(PredictorTest, ConcurrentLookupTest) {
   host_resolver_->rules()->AddSimulatedFailure("*.notfound");
 
-  scoped_refptr<Predictor> testing_master(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
+  Predictor testing_master(true);
+  testing_master.SetHostResolver(host_resolver_.get());
 
   GURL goog("http://www.google.com:80"),
       goog2("http://gmail.google.com.com:80"),
@@ -205,37 +192,34 @@ TEST_F(PredictorTest, ConcurrentLookupTest) {
 
   // Try to flood the predictor with many concurrent requests.
   for (int i = 0; i < 10; i++)
-    testing_master->ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
+    testing_master.ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
 
-  WaitForResolution(testing_master, names);
+  WaitForResolution(&testing_master, names);
 
-  EXPECT_TRUE(testing_master->WasFound(goog));
-  EXPECT_TRUE(testing_master->WasFound(goog3));
-  EXPECT_TRUE(testing_master->WasFound(goog2));
-  EXPECT_TRUE(testing_master->WasFound(goog4));
-  EXPECT_FALSE(testing_master->WasFound(bad1));
-  EXPECT_FALSE(testing_master->WasFound(bad2));
+  EXPECT_TRUE(testing_master.WasFound(goog));
+  EXPECT_TRUE(testing_master.WasFound(goog3));
+  EXPECT_TRUE(testing_master.WasFound(goog2));
+  EXPECT_TRUE(testing_master.WasFound(goog4));
+  EXPECT_FALSE(testing_master.WasFound(bad1));
+  EXPECT_FALSE(testing_master.WasFound(bad2));
 
   MessageLoop::current()->RunAllPending();
 
-  EXPECT_FALSE(testing_master->WasFound(bad1));
-  EXPECT_FALSE(testing_master->WasFound(bad2));
+  EXPECT_FALSE(testing_master.WasFound(bad1));
+  EXPECT_FALSE(testing_master.WasFound(bad2));
 
-  EXPECT_LE(testing_master->peak_pending_lookups(), names.size());
-  EXPECT_LE(testing_master->peak_pending_lookups(),
-            testing_master->max_concurrent_dns_lookups());
+  EXPECT_LE(testing_master.peak_pending_lookups(), names.size());
+  EXPECT_LE(testing_master.peak_pending_lookups(),
+            testing_master.max_concurrent_dns_lookups());
 
-  testing_master->Shutdown();
+  testing_master.Shutdown();
 }
 
 TEST_F(PredictorTest, MassiveConcurrentLookupTest) {
   host_resolver_->rules()->AddSimulatedFailure("*.notfound");
 
-  scoped_refptr<Predictor> testing_master(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
+  Predictor testing_master(true);
+  testing_master.SetHostResolver(host_resolver_.get());
 
   UrlList names;
   for (int i = 0; i < 100; i++)
@@ -244,17 +228,17 @@ TEST_F(PredictorTest, MassiveConcurrentLookupTest) {
 
   // Try to flood the predictor with many concurrent requests.
   for (int i = 0; i < 10; i++)
-    testing_master->ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
+    testing_master.ResolveList(names, UrlInfo::PAGE_SCAN_MOTIVATED);
 
-  WaitForResolution(testing_master, names);
+  WaitForResolution(&testing_master, names);
 
   MessageLoop::current()->RunAllPending();
 
-  EXPECT_LE(testing_master->peak_pending_lookups(), names.size());
-  EXPECT_LE(testing_master->peak_pending_lookups(),
-            testing_master->max_concurrent_dns_lookups());
+  EXPECT_LE(testing_master.peak_pending_lookups(), names.size());
+  EXPECT_LE(testing_master.peak_pending_lookups(),
+            testing_master.max_concurrent_dns_lookups());
 
-  testing_master->Shutdown();
+  testing_master.Shutdown();
 }
 
 //------------------------------------------------------------------------------
@@ -263,12 +247,12 @@ TEST_F(PredictorTest, MassiveConcurrentLookupTest) {
 
 // Return a motivation_list if we can find one for the given motivating_host (or
 // NULL if a match is not found).
-static ListValue* FindSerializationMotivation(
-    const GURL& motivation, const ListValue& referral_list) {
+static ListValue* FindSerializationMotivation(const GURL& motivation,
+                                              const ListValue& referral_list) {
   CHECK_LT(0u, referral_list.GetSize());  // Room for version.
   int format_version = -1;
   CHECK(referral_list.GetInteger(0, &format_version));
-  CHECK_EQ(Predictor::PREDICTOR_REFERRER_VERSION, format_version);
+  CHECK_EQ(Predictor::kPredictorReferrerVersion, format_version);
   ListValue* motivation_list(NULL);
   for (size_t i = 1; i < referral_list.GetSize(); ++i) {
     referral_list.GetList(i, &motivation_list);
@@ -282,8 +266,9 @@ static ListValue* FindSerializationMotivation(
 
 // Create a new empty serialization list.
 static ListValue* NewEmptySerializationList() {
-  ListValue* list = new ListValue;
-  list->Append(new FundamentalValue(Predictor::PREDICTOR_REFERRER_VERSION));
+  base::ListValue* list = new base::ListValue;
+  list->Append(
+      new base::FundamentalValue(Predictor::kPredictorReferrerVersion));
   return list;
 }
 
@@ -316,8 +301,8 @@ static void AddToSerializedList(const GURL& motivation,
   // case, during deserialization, the latency value we supply plus the
   // existing value(s) will be added to the referrer.
 
-  subresource_list->Append(new StringValue(subresource.spec()));
-  subresource_list->Append(new FundamentalValue(use_rate));
+  subresource_list->Append(new base::StringValue(subresource.spec()));
+  subresource_list->Append(new base::FundamentalValue(use_rate));
 }
 
 static const int kLatencyNotFound = -1;
@@ -339,7 +324,7 @@ static bool GetDataFromSerialization(const GURL& motivation,
   for (size_t i = 0; i < subresource_list->GetSize();) {
     std::string url_spec;
     EXPECT_TRUE(subresource_list->GetString(i++, &url_spec));
-    EXPECT_TRUE(subresource_list->GetReal(i++, use_rate));
+    EXPECT_TRUE(subresource_list->GetDouble(i++, use_rate));
     if (subresource == GURL(url_spec)) {
       return true;
     }
@@ -351,30 +336,25 @@ static bool GetDataFromSerialization(const GURL& motivation,
 
 // Make sure nil referral lists really have no entries, and no latency listed.
 TEST_F(PredictorTest, ReferrerSerializationNilTest) {
-  scoped_refptr<Predictor> predictor(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
+  Predictor predictor(true);
+  predictor.SetHostResolver(host_resolver_.get());
+
   scoped_ptr<ListValue> referral_list(NewEmptySerializationList());
-  predictor->SerializeReferrers(referral_list.get());
+  predictor.SerializeReferrers(referral_list.get());
   EXPECT_EQ(1U, referral_list->GetSize());
   EXPECT_FALSE(GetDataFromSerialization(
     GURL("http://a.com:79"), GURL("http://b.com:78"),
       *referral_list.get(), NULL));
 
-  predictor->Shutdown();
+  predictor.Shutdown();
 }
 
 // Make sure that when a serialization list includes a value, that it can be
 // deserialized into the database, and can be extracted back out via
 // serialization without being changed.
 TEST_F(PredictorTest, ReferrerSerializationSingleReferrerTest) {
-  scoped_refptr<Predictor> predictor(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
+  Predictor predictor(true);
+  predictor.SetHostResolver(host_resolver_.get());
   const GURL motivation_url("http://www.google.com:91");
   const GURL subresource_url("http://icons.google.com:90");
   const double kUseRate = 23.4;
@@ -383,32 +363,39 @@ TEST_F(PredictorTest, ReferrerSerializationSingleReferrerTest) {
   AddToSerializedList(motivation_url, subresource_url,
       kUseRate, referral_list.get());
 
-  predictor->DeserializeReferrers(*referral_list.get());
+  predictor.DeserializeReferrers(*referral_list.get());
 
   ListValue recovered_referral_list;
-  predictor->SerializeReferrers(&recovered_referral_list);
+  predictor.SerializeReferrers(&recovered_referral_list);
   EXPECT_EQ(2U, recovered_referral_list.GetSize());
   double rate;
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, subresource_url, recovered_referral_list, &rate));
   EXPECT_EQ(rate, kUseRate);
 
-  predictor->Shutdown();
+  predictor.Shutdown();
 }
 
+// Verify that two floats are within 1% of each other in value.
+#define EXPECT_SIMILAR(a, b) do { \
+    double espilon_ratio = 1.01;  \
+    if ((a) < 0.)  \
+      espilon_ratio = 1 / espilon_ratio;  \
+    EXPECT_LT(a, espilon_ratio * (b));   \
+    EXPECT_GT((a) * espilon_ratio, b);   \
+    } while (0)
+
+
 // Make sure the Trim() functionality works as expected.
 TEST_F(PredictorTest, ReferrerSerializationTrimTest) {
-  scoped_refptr<Predictor> predictor(
-      new Predictor(host_resolver_.get(),
-                    default_max_queueing_delay_,
-                    PredictorInit::kMaxSpeculativeParallelResolves,
-                    false));
+  Predictor predictor(true);
+  predictor.SetHostResolver(host_resolver_.get());
   GURL motivation_url("http://www.google.com:110");
 
   GURL icon_subresource_url("http://icons.google.com:111");
-  const double kRateIcon = 16.0 * Predictor::kPersistWorthyExpectedValue;
+  const double kRateIcon = 16.0 * Predictor::kDiscardableExpectedValue;
   GURL img_subresource_url("http://img.google.com:118");
-  const double kRateImg = 8.0 * Predictor::kPersistWorthyExpectedValue;
+  const double kRateImg = 8.0 * Predictor::kDiscardableExpectedValue;
 
   scoped_ptr<ListValue> referral_list(NewEmptySerializationList());
   AddToSerializedList(
@@ -416,57 +403,61 @@ TEST_F(PredictorTest, ReferrerSerializationTrimTest) {
   AddToSerializedList(
       motivation_url, img_subresource_url, kRateImg, referral_list.get());
 
-  predictor->DeserializeReferrers(*referral_list.get());
+  predictor.DeserializeReferrers(*referral_list.get());
 
   ListValue recovered_referral_list;
-  predictor->SerializeReferrers(&recovered_referral_list);
+  predictor.SerializeReferrers(&recovered_referral_list);
   EXPECT_EQ(2U, recovered_referral_list.GetSize());
   double rate;
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, icon_subresource_url, recovered_referral_list,
       &rate));
-  EXPECT_EQ(rate, kRateIcon);
+  EXPECT_SIMILAR(rate, kRateIcon);
 
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, img_subresource_url, recovered_referral_list, &rate));
-  EXPECT_EQ(rate, kRateImg);
+  EXPECT_SIMILAR(rate, kRateImg);
 
-  // Each time we Trim, the user_rate figures should reduce by a factor of two,
-  // until they both are small, an then a trim will delete the whole entry.
-  predictor->TrimReferrers();
-  predictor->SerializeReferrers(&recovered_referral_list);
+  // Each time we Trim 24 times, the user_rate figures should reduce by a factor
+  // of two,  until they are small, and then a trim will delete the whole entry.
+  for (int i = 0; i < 24; ++i)
+    predictor.TrimReferrersNow();
+  predictor.SerializeReferrers(&recovered_referral_list);
   EXPECT_EQ(2U, recovered_referral_list.GetSize());
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, icon_subresource_url, recovered_referral_list, &rate));
-  EXPECT_EQ(rate, kRateIcon / 2);
+  EXPECT_SIMILAR(rate, kRateIcon / 2);
 
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, img_subresource_url, recovered_referral_list, &rate));
-  EXPECT_EQ(rate, kRateImg / 2);
+  EXPECT_SIMILAR(rate, kRateImg / 2);
 
-  predictor->TrimReferrers();
-  predictor->SerializeReferrers(&recovered_referral_list);
+  for (int i = 0; i < 24; ++i)
+    predictor.TrimReferrersNow();
+  predictor.SerializeReferrers(&recovered_referral_list);
   EXPECT_EQ(2U, recovered_referral_list.GetSize());
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, icon_subresource_url, recovered_referral_list, &rate));
-  EXPECT_EQ(rate, kRateIcon / 4);
+  EXPECT_SIMILAR(rate, kRateIcon / 4);
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, img_subresource_url, recovered_referral_list, &rate));
-  EXPECT_EQ(rate, kRateImg / 4);
+  EXPECT_SIMILAR(rate, kRateImg / 4);
 
-  predictor->TrimReferrers();
-  predictor->SerializeReferrers(&recovered_referral_list);
+  for (int i = 0; i < 24; ++i)
+    predictor.TrimReferrersNow();
+  predictor.SerializeReferrers(&recovered_referral_list);
   EXPECT_EQ(2U, recovered_referral_list.GetSize());
   EXPECT_TRUE(GetDataFromSerialization(
       motivation_url, icon_subresource_url, recovered_referral_list, &rate));
-  EXPECT_EQ(rate, kRateIcon / 8);
+  EXPECT_SIMILAR(rate, kRateIcon / 8);
 
   // Img is below threshold, and so it gets deleted.
   EXPECT_FALSE(GetDataFromSerialization(
       motivation_url, img_subresource_url, recovered_referral_list, &rate));
 
-  predictor->TrimReferrers();
-  predictor->SerializeReferrers(&recovered_referral_list);
+  for (int i = 0; i < 24; ++i)
+    predictor.TrimReferrersNow();
+  predictor.SerializeReferrers(&recovered_referral_list);
   // Icon is also trimmed away, so entire set gets discarded.
   EXPECT_EQ(1U, recovered_referral_list.GetSize());
   EXPECT_FALSE(GetDataFromSerialization(
@@ -474,7 +465,7 @@ TEST_F(PredictorTest, ReferrerSerializationTrimTest) {
   EXPECT_FALSE(GetDataFromSerialization(
       motivation_url, img_subresource_url, recovered_referral_list, &rate));
 
-  predictor->Shutdown();
+  predictor.Shutdown();
 }
 
 
@@ -585,4 +576,25 @@ TEST_F(PredictorTest, CanonicalizeUrl) {
             long_https.GetWithEmptyPath());
 }
 
+TEST_F(PredictorTest, DiscardPredictorResults) {
+  Predictor predictor(true);
+  predictor.SetHostResolver(host_resolver_.get());
+  ListValue referral_list;
+  predictor.SerializeReferrers(&referral_list);
+  EXPECT_EQ(1U, referral_list.GetSize());
+
+  GURL host_1("http://test_1");
+  GURL host_2("http://test_2");
+  predictor.LearnFromNavigation(host_1, host_2);
+
+  predictor.SerializeReferrers(&referral_list);
+  EXPECT_EQ(2U, referral_list.GetSize());
+
+  predictor.DiscardAllResults();
+  predictor.SerializeReferrers(&referral_list);
+  EXPECT_EQ(1U, referral_list.GetSize());
+
+  predictor.Shutdown();
+}
+
 }  // namespace chrome_browser_net
