@@ -14,25 +14,28 @@
 #include <set>
 #include <string>
 
-#include "base/scoped_ptr.h"
-#include "base/task.h"
-#include "chrome/common/net/url_fetcher.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
+#include "base/threading/non_thread_safe.h"
+#include "content/public/common/url_fetcher_delegate.h"
 #include "net/base/sdch_manager.h"
 
-class SdchDictionaryFetcher : public URLFetcher::Delegate,
-                              public SdchFetcher {
+class SdchDictionaryFetcher
+    : public content::URLFetcherDelegate,
+      public net::SdchFetcher,
+      public base::NonThreadSafe {
  public:
   SdchDictionaryFetcher();
   virtual ~SdchDictionaryFetcher();
 
+  // Stop fetching dictionaries, and abandon any current URLFetcheer operations
+  // so that the IO thread can be stopped.
+  static void Shutdown();
+
   // Implementation of SdchFetcher class.
   // This method gets the requested dictionary, and then calls back into the
   // SdchManager class with the dictionary's text.
-  virtual void Schedule(const GURL& dictionary_url);
-
-  // Stop fetching dictionaries, and abandon any current URLFetcheer operations
-  // so that the IO thread can be stopped.
-  static void Shutdown() { SdchManager::Shutdown(); }
+  virtual void Schedule(const GURL& dictionary_url) OVERRIDE;
 
  private:
   // Delay in ms between Schedule and actual download.
@@ -48,25 +51,20 @@ class SdchDictionaryFetcher : public URLFetcher::Delegate,
   // in the  |fetch_queue_|.
   void StartFetching();
 
-  // Implementation of URLFetcher::Delegate. Called after transmission
+  // Implementation of content::URLFetcherDelegate. Called after transmission
   // completes (either successfully or with failure).
-  virtual void OnURLFetchComplete(const URLFetcher* source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
   // A queue of URLs that are being used to download dictionaries.
   std::queue<GURL> fetch_queue_;
   // The currently outstanding URL fetch of a dicitonary.
   // If this is null, then there is no outstanding request.
-  scoped_ptr<URLFetcher> current_fetch_;
+  scoped_ptr<content::URLFetcher> current_fetch_;
 
   // Always spread out the dictionary fetches, so that they don't steal
   // bandwidth from the actual page load.  Create delayed tasks to spread out
   // the download.
-  ScopedRunnableMethodFactory<SdchDictionaryFetcher> method_factory_;
+  base::WeakPtrFactory<SdchDictionaryFetcher> weak_factory_;
   bool task_is_pending_;
 
   // Althought the SDCH spec does not preclude a server from using a single URL
