@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,15 +10,18 @@ namespace net {
 class SSLConfigService;
 }  // namespace net
 
-class Profile;
+class PrefService;
 
-// An interface for creating SSLConfigService objects for the current platform.
+// An interface for creating SSLConfigService objects.
 class SSLConfigServiceManager {
  public:
-  // Create an instance of the default SSLConfigServiceManager for the current
-  // platform. The lifetime of the profile must be longer than that of the
-  // manager.
-  static SSLConfigServiceManager* CreateDefaultManager(Profile* profile);
+  // Create an instance of the SSLConfigServiceManager. The lifetime of the
+  // PrefService objects must be longer than that of the manager. Get SSL
+  // preferences from local_state object.
+  static SSLConfigServiceManager* CreateDefaultManager(
+      PrefService* local_state);
+
+  static void RegisterPrefs(PrefService* prefs);
 
   virtual ~SSLConfigServiceManager() {}
 
