@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -30,7 +30,7 @@ namespace {
 // a url string, and these need to be updated when the URL is converted from
 // UTF8 to UTF16. Instead of this after-the-fact adjustment, we should parse it
 // in the correct string format to begin with.
-url_parse::Component UTF8ComponentToWideComponent(
+url_parse::Component UTF8ComponentToUTF16Component(
     const std::string& text_utf8,
     const url_parse::Component& component_utf8) {
   if (component_utf8.len == -1)
@@ -40,37 +40,37 @@ url_parse::Component UTF8ComponentToWideComponent(
       text_utf8.substr(0, component_utf8.begin);
   std::string component_string = text_utf8.substr(component_utf8.begin,
                                                   component_utf8.len);
-  std::wstring before_component_string_w = UTF8ToWide(before_component_string);
-  std::wstring component_string_w = UTF8ToWide(component_string);
-  url_parse::Component component_w(before_component_string_w.length(),
-                                   component_string_w.length());
-  return component_w;
+  string16 before_component_string_16 = UTF8ToUTF16(before_component_string);
+  string16 component_string_16 = UTF8ToUTF16(component_string);
+  url_parse::Component component_16(before_component_string_16.length(),
+                                    component_string_16.length());
+  return component_16;
 }
 
-void UTF8PartsToWideParts(const std::string& text_utf8,
-                          const url_parse::Parsed& parts_utf8,
-                          url_parse::Parsed* parts) {
+void UTF8PartsToUTF16Parts(const std::string& text_utf8,
+                           const url_parse::Parsed& parts_utf8,
+                           url_parse::Parsed* parts) {
   if (IsStringASCII(text_utf8)) {
     *parts = parts_utf8;
     return;
   }
 
   parts->scheme =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.scheme);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.scheme);
   parts ->username =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.username);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.username);
   parts->password =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.password);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.password);
   parts->host =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.host);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.host);
   parts->port =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.port);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.port);
   parts->path =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.path);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.path);
   parts->query =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.query);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.query);
   parts->ref =
-      UTF8ComponentToWideComponent(text_utf8, parts_utf8.ref);
+      UTF8ComponentToUTF16Component(text_utf8, parts_utf8.ref);
 }
 
 TrimPositions TrimWhitespaceUTF8(const std::string& input,
@@ -80,10 +80,10 @@ TrimPositions TrimWhitespaceUTF8(const std::string& input,
   // twice. Please feel free to file a bug if this function hurts the
   // performance of Chrome.
   DCHECK(IsStringUTF8(input));
-  std::wstring input_wide = UTF8ToWide(input);
-  std::wstring output_wide;
-  TrimPositions result = TrimWhitespace(input_wide, positions, &output_wide);
-  *output = WideToUTF8(output_wide);
+  string16 input16 = UTF8ToUTF16(input);
+  string16 output16;
+  TrimPositions result = TrimWhitespace(input16, positions, &output16);
+  *output = UTF16ToUTF8(output16);
   return result;
 }
 
@@ -173,7 +173,7 @@ static std::string FixupPath(const std::string& text) {
   GURL file_url = net::FilePathToFileURL(FilePath(filename));
   if (file_url.is_valid()) {
     return UTF16ToUTF8(net::FormatUrl(file_url, std::string(),
-        net::kFormatUrlOmitUsernamePassword, UnescapeRule::NORMAL, NULL,
+        net::kFormatUrlOmitUsernamePassword, net::UnescapeRule::NORMAL, NULL,
         NULL, NULL));
   }
 
@@ -406,10 +406,10 @@ std::string URLFixerUpper::SegmentURL(const std::string& text,
         chrome::kFtpScheme : chrome::kHttpScheme);
   }
 
-  // Not segmenting file schemes or nonstandard schemes.
-  if ((scheme == chrome::kFileScheme) ||
-      !url_util::IsStandard(scheme.c_str(),
-      url_parse::Component(0, static_cast<int>(scheme.length()))))
+  // Proceed with about and chrome schemes, but not file or nonstandard schemes.
+  if ((scheme != chrome::kAboutScheme) && (scheme != chrome::kChromeUIScheme) &&
+      ((scheme == chrome::kFileScheme) || !url_util::IsStandard(scheme.c_str(),
+           url_parse::Component(0, static_cast<int>(scheme.length())))))
     return scheme;
 
   if (parts->scheme.is_valid()) {
@@ -427,7 +427,7 @@ std::string URLFixerUpper::SegmentURL(const std::string& text,
 
   // Construct the text to parse by inserting the scheme.
   std::string inserted_text(scheme);
-  inserted_text.append("://");
+  inserted_text.append(chrome::kStandardSchemeSeparator);
   std::string text_to_parse(text.begin(), first_nonwhite);
   text_to_parse.append(inserted_text);
   text_to_parse.append(first_nonwhite, text.end());
@@ -478,11 +478,16 @@ GURL URLFixerUpper::FixupURL(const std::string& text,
   if (scheme == chrome::kFileScheme)
     return GURL(parts.scheme.is_valid() ? text : FixupPath(text));
 
+  // Parse and rebuild about: and chrome: URLs, except about:blank.
+  bool chrome_url = !LowerCaseEqualsASCII(trimmed, chrome::kAboutBlankURL) &&
+      ((scheme == chrome::kAboutScheme) || (scheme == chrome::kChromeUIScheme));
+
   // For some schemes whose layouts we understand, we rebuild it.
-  if (url_util::IsStandard(scheme.c_str(),
+  if (chrome_url || url_util::IsStandard(scheme.c_str(),
           url_parse::Component(0, static_cast<int>(scheme.length())))) {
-    std::string url(scheme);
-    url.append("://");
+    // Replace the about: scheme with the chrome: scheme.
+    std::string url(chrome_url ? chrome::kChromeUIScheme : scheme);
+    url.append(chrome::kStandardSchemeSeparator);
 
     // We need to check whether the |username| is valid because it is our
     // responsibility to append the '@' to delineate the user information from
@@ -494,6 +499,8 @@ GURL URLFixerUpper::FixupURL(const std::string& text,
     }
 
     FixupHost(trimmed, parts.host, parts.scheme.is_valid(), desired_tld, &url);
+    if (chrome_url && !parts.host.is_valid())
+      url.append(chrome::kChromeUIDefaultHost);
     FixupPort(trimmed, parts.port, &url);
     FixupPath(trimmed, parts.path, &url);
     FixupQuery(trimmed, parts.query, &url);
@@ -505,7 +512,7 @@ GURL URLFixerUpper::FixupURL(const std::string& text,
   // In the worst-case, we insert a scheme if the URL lacks one.
   if (!parts.scheme.is_valid()) {
     std::string fixed_scheme(scheme);
-    fixed_scheme.append("://");
+    fixed_scheme.append(chrome::kStandardSchemeSeparator);
     trimmed.insert(0, fixed_scheme);
   }
 
@@ -531,19 +538,23 @@ GURL URLFixerUpper::FixupRelativeFile(const FilePath& base_dir,
   PrepareStringForFileOps(text, &trimmed);
 
   bool is_file = true;
+  // Avoid recognizing definite non-file URLs as file paths.
+  GURL gurl(trimmed);
+  if (gurl.is_valid() && gurl.IsStandard())
+    is_file = false;
   FilePath full_path;
-  if (!ValidPathForFile(trimmed, &full_path)) {
+  if (is_file && !ValidPathForFile(trimmed, &full_path)) {
     // Not a path as entered, try unescaping it in case the user has
     // escaped things. We need to go through 8-bit since the escaped values
     // only represent 8-bit values.
 #if defined(OS_WIN)
-    std::wstring unescaped = UTF8ToWide(UnescapeURLComponent(
+    std::wstring unescaped = UTF8ToWide(net::UnescapeURLComponent(
         WideToUTF8(trimmed),
-        UnescapeRule::SPACES | UnescapeRule::URL_SPECIAL_CHARS));
+        net::UnescapeRule::SPACES | net::UnescapeRule::URL_SPECIAL_CHARS));
 #elif defined(OS_POSIX)
-    std::string unescaped = UnescapeURLComponent(
+    std::string unescaped = net::UnescapeURLComponent(
         trimmed,
-        UnescapeRule::SPACES | UnescapeRule::URL_SPECIAL_CHARS);
+        net::UnescapeRule::SPACES | net::UnescapeRule::URL_SPECIAL_CHARS);
 #endif
 
     if (!ValidPathForFile(unescaped, &full_path))
@@ -558,7 +569,7 @@ GURL URLFixerUpper::FixupRelativeFile(const FilePath& base_dir,
     GURL file_url = net::FilePathToFileURL(full_path);
     if (file_url.is_valid())
       return GURL(UTF16ToUTF8(net::FormatUrl(file_url, std::string(),
-          net::kFormatUrlOmitUsernamePassword, UnescapeRule::NORMAL, NULL,
+          net::kFormatUrlOmitUsernamePassword, net::UnescapeRule::NORMAL, NULL,
           NULL, NULL)));
     // Invalid files fall through to regular processing.
   }
@@ -572,19 +583,13 @@ GURL URLFixerUpper::FixupRelativeFile(const FilePath& base_dir,
   return FixupURL(text_utf8, std::string());
 }
 
-// Deprecated functions. To be removed when all callers are updated.
-std::wstring URLFixerUpper::SegmentURL(const std::wstring& text,
-                                       url_parse::Parsed* parts) {
-  std::string text_utf8 = WideToUTF8(text);
+string16 URLFixerUpper::SegmentURL(const string16& text,
+                                   url_parse::Parsed* parts) {
+  std::string text_utf8 = UTF16ToUTF8(text);
   url_parse::Parsed parts_utf8;
   std::string scheme_utf8 = SegmentURL(text_utf8, &parts_utf8);
-  UTF8PartsToWideParts(text_utf8, parts_utf8, parts);
-  return UTF8ToWide(scheme_utf8);
-}
-GURL URLFixerUpper::FixupRelativeFile(const std::wstring& base_dir,
-                                      const std::wstring& text) {
-  return FixupRelativeFile(FilePath::FromWStringHack(base_dir),
-                           FilePath::FromWStringHack(text));
+  UTF8PartsToUTF16Parts(text_utf8, parts_utf8, parts);
+  return UTF8ToUTF16(scheme_utf8);
 }
 
 void URLFixerUpper::OffsetComponent(int offset, url_parse::Component* part) {
