@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,7 @@
 
 #include <string>
 
+#include "base/string16.h"
 #include "googleurl/src/gurl.h"
 
 namespace url_parse {
@@ -29,8 +30,7 @@ namespace URLFixerUpper {
   // Returns the canonicalized scheme, or the empty string when |text| is only
   // whitespace.
   std::string SegmentURL(const std::string& text, url_parse::Parsed* parts);
-  // Deprecated temporary compatibility function.
-  std::wstring SegmentURL(const std::wstring& text, url_parse::Parsed* parts);
+  string16 SegmentURL(const string16& text, url_parse::Parsed* parts);
 
   // Converts |text| to a fixed-up URL and returns it. Attempts to make
   // some "smart" adjustments to obviously-invalid input where possible.
@@ -41,6 +41,11 @@ namespace URLFixerUpper {
   // be valid, so check the return value's validity or use
   // possibly_invalid_spec().
   //
+  // Schemes "about" and "chrome" are normalized to "chrome://", with slashes.
+  // "about:blank" is unaltered, as Webkit allows frames to access about:blank.
+  // Additionally, if a chrome URL does not have a valid host, as in "about:",
+  // the returned URL will have the host "version", as in "chrome://version".
+  //
   // If |desired_tld| is non-empty, it represents the TLD the user wishes to
   // append in the case of an incomplete domain.  We check that this is not a
   // file path and there does not appear to be a valid TLD already, then append
@@ -59,9 +64,6 @@ namespace URLFixerUpper {
   // should use FixupURL() directly.  This function should only be used when
   // relative path handling is desired, as for command line processing.
   GURL FixupRelativeFile(const FilePath& base_dir, const FilePath& text);
-  // Deprecated temporary compatibility function.
-  GURL FixupRelativeFile(const std::wstring& base_dir,
-                         const std::wstring& text);
 
   // Offsets the beginning index of |part| by |offset|, which is allowed to be
   // negative.  In some cases, the desired component does not exist at the given
