@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,7 +12,7 @@
 #include "base/format_macros.h"
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
-#include "base/string_util.h"
+#include "base/stringprintf.h"
 
 using base::Time;
 using base::TimeDelta;
@@ -120,10 +120,11 @@ void UrlInfo::RemoveFromQueue() {
   }
   // Make a custom linear histogram for the region from 0 to boundary.
   const size_t kBucketCount = 52;
-  static scoped_refptr<base::Histogram> histogram =
-      base::LinearHistogram::FactoryTimeGet(
-          "DNS.QueueRecycledUnder2", TimeDelta(), kBoundary, kBucketCount,
-          base::Histogram::kUmaTargetedHistogramFlag);
+  static base::Histogram* histogram(NULL);
+  if (!histogram)
+    histogram = base::LinearHistogram::FactoryTimeGet(
+        "DNS.QueueRecycledUnder2", TimeDelta(), kBoundary, kBucketCount,
+        base::Histogram::kUmaTargetedHistogramFlag);
   histogram->AddTime(queue_duration_);
 }
 
@@ -259,10 +260,10 @@ static std::string HoursMinutesSeconds(int seconds) {
 }
 
 // static
-void UrlInfo::GetHtmlTable(const UrlInfoTable host_infos,
-                               const char* description,
-                               const bool brief,
-                               std::string* output) {
+void UrlInfo::GetHtmlTable(const UrlInfoTable& host_infos,
+                           const char* description,
+                           bool brief,
+                           std::string* output) {
   if (0 == host_infos.size())
     return;
   output->append(description);
