@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,17 +9,16 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
-#include "gfx/point.h"
-#include "gfx/rect.h"
-#include "gfx/size.h"
+#include "base/memory/scoped_ptr.h"
+#include "ui/gfx/point.h"
+#include "ui/gfx/rect.h"
+#include "ui/gfx/size.h"
 
 class Balloon;
 class BalloonCollection;
 class BalloonHost;
 class Notification;
 class Profile;
-class SiteInstance;
 
 // Interface for a view that displays a balloon.
 class BalloonView {
