@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,7 +12,6 @@
 #include <string>
 
 #include "base/callback.h"
-#include "base/scoped_ptr.h"
 
 class Balloon;
 class GURL;
@@ -21,7 +20,7 @@ class Profile;
 
 namespace gfx {
 class Size;
-}  // namespace gfx
+}
 
 class BalloonCollection {
  public:
@@ -48,11 +47,9 @@ class BalloonCollection {
 
   static BalloonCollection* Create();
 
-  BalloonCollection()
-      : space_change_listener_(NULL) {
-  }
+  BalloonCollection();
 
-  virtual ~BalloonCollection() {}
+  virtual ~BalloonCollection();
 
   // Adds a new balloon for the specified notification.
   virtual void Add(const Notification& notification,
@@ -95,8 +92,8 @@ class BalloonCollection {
     space_change_listener_ = listener;
   }
 
-  void set_on_collection_changed_callback(Callback0::Type* callback) {
-    on_collection_changed_callback_.reset(callback);
+  void set_on_collection_changed_callback(const base::Closure& callback) {
+    on_collection_changed_callback_ = callback;
   }
 
  protected:
@@ -105,7 +102,7 @@ class BalloonCollection {
 
   // For use only with testing. This callback is invoked when a balloon
   // is added or removed from the collection.
-  scoped_ptr<Callback0::Type> on_collection_changed_callback_;
+  base::Closure on_collection_changed_callback_;
 };
 
 #endif  // CHROME_BROWSER_NOTIFICATIONS_BALLOON_COLLECTION_H_
