@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,11 +11,15 @@
 #include <deque>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
+#include "base/memory/weak_ptr.h"
 #include "base/message_loop.h"
 #include "chrome/browser/notifications/balloon_collection.h"
 #include "chrome/browser/notifications/balloon_collection_base.h"
-#include "gfx/point.h"
-#include "gfx/rect.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "ui/gfx/point.h"
+#include "ui/gfx/rect.h"
 
 // Mac balloons grow from the top down and have close buttons on top, so
 // offsetting is not necessary for easy multiple-closing.  Other platforms grow
@@ -30,7 +34,8 @@
 // shown on the screen.  It positions new notifications according to
 // a layout, and monitors for balloons being closed, which it reports
 // up to its parent, the notification UI manager.
-class BalloonCollectionImpl : public BalloonCollection
+class BalloonCollectionImpl : public BalloonCollection,
+                              public content::NotificationObserver
 #if USE_OFFSETS
                             , public MessageLoopForUI::Observer
 #endif
@@ -41,27 +46,36 @@ class BalloonCollectionImpl : public BalloonCollection
 
   // BalloonCollection interface.
   virtual void Add(const Notification& notification,
-                   Profile* profile);
-  virtual bool RemoveById(const std::string& id);
-  virtual bool RemoveBySourceOrigin(const GURL& source_origin);
-  virtual void RemoveAll();
-  virtual bool HasSpace() const;
-  virtual void ResizeBalloon(Balloon* balloon, const gfx::Size& size);
-  virtual void SetPositionPreference(PositionPreference position);
-  virtual void DisplayChanged();
-  virtual void OnBalloonClosed(Balloon* source);
-  virtual const Balloons& GetActiveBalloons() { return base_.balloons(); }
+                   Profile* profile) OVERRIDE;
+  virtual bool RemoveById(const std::string& id) OVERRIDE;
+  virtual bool RemoveBySourceOrigin(const GURL& source_origin) OVERRIDE;
+  virtual void RemoveAll() OVERRIDE;
+  virtual bool HasSpace() const OVERRIDE;
+  virtual void ResizeBalloon(Balloon* balloon, const gfx::Size& size) OVERRIDE;
+  virtual void SetPositionPreference(PositionPreference position) OVERRIDE;
+  virtual void DisplayChanged() OVERRIDE;
+  virtual void OnBalloonClosed(Balloon* source) OVERRIDE;
+  virtual const Balloons& GetActiveBalloons() OVERRIDE;
+
+  // content::NotificationObserver interface.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // MessageLoopForUI::Observer interface.
-#if defined(OS_WIN)
-  virtual void WillProcessMessage(const MSG& event) {}
-  virtual void DidProcessMessage(const MSG& event);
-#endif
-#if defined(OS_LINUX)
-  virtual void WillProcessEvent(GdkEvent* event) {}
-  virtual void DidProcessEvent(GdkEvent* event);
+#if defined(OS_WIN) || defined(USE_AURA)
+  virtual base::EventStatus WillProcessEvent(
+      const base::NativeEvent& event) OVERRIDE;
+  virtual void DidProcessEvent(const base::NativeEvent& event) OVERRIDE;
+#elif defined(TOOLKIT_USES_GTK)
+  virtual void WillProcessEvent(GdkEvent* event) OVERRIDE;
+  virtual void DidProcessEvent(GdkEvent* event) OVERRIDE;
 #endif
 
+  // base_ is embedded, so this is a simple accessor for the number of
+  // balloons in the collection.
+  int count() const { return base_.count(); }
+
  protected:
   // Calculates layout values for the balloons including
   // the scaling, the max/min sizes, and the upper left corner of each.
@@ -72,6 +86,7 @@ class BalloonCollectionImpl : public BalloonCollection
     // These enumerations all are based on a screen orientation where
     // the origin is the top-left.
     enum Placement {
+      INVALID,
       VERTICALLY_FROM_TOP_LEFT,
       VERTICALLY_FROM_TOP_RIGHT,
       VERTICALLY_FROM_BOTTOM_LEFT,
@@ -120,12 +135,23 @@ class BalloonCollectionImpl : public BalloonCollection
     // to be used as the initial position for an animation into view.
     gfx::Point OffScreenLocation() const;
 
+    // Returns true if the layout requires offsetting for keeping the close
+    // buttons under the cursor during rapid-close interaction.
+    bool RequiresOffsets() const;
+
+    // Returns true if there is change to the offset that requires the balloons
+    // to be repositioned.
+    bool ComputeOffsetToMoveAbovePanels(const gfx::Rect& panel_bounds);
+
    private:
     // Layout parameters
     int VerticalEdgeMargin() const;
     int HorizontalEdgeMargin() const;
     int InterBalloonMargin() const;
 
+    bool NeedToMoveAboveLeftSidePanels() const;
+    bool NeedToMoveAboveRightSidePanels() const;
+
     // Minimum and maximum size of balloon content.
     static const int kBalloonMinWidth = 300;
     static const int kBalloonMaxWidth = 300;
@@ -134,17 +160,32 @@ class BalloonCollectionImpl : public BalloonCollection
 
     Placement placement_;
     gfx::Rect work_area_;
+
+    // The offset that guarantees that the notificaitions shown in the
+    // lower-right or lower-left corner of the screen will go above currently
+    // shown panels and will not be obscured by them.
+    int offset_to_move_above_panels_;
+
     DISALLOW_COPY_AND_ASSIGN(Layout);
   };
 
-  // Creates a new balloon. Overridable by unit tests.  The caller is
-  // responsible for freeing the pointer returned.
+  // Creates a new balloon. Overridable by derived classes and unit tests.
+  // The caller is responsible for freeing the pointer returned.
   virtual Balloon* MakeBalloon(const Notification& notification,
                                Profile* profile);
 
+  // Protected implementation of Add with additional add_to_front parameter
+  // for use by derived classes.
+  void AddImpl(const Notification& notification,
+               Profile* profile,
+               bool add_to_front);
+
   // Gets a bounding box for all the current balloons in screen coordinates.
   gfx::Rect GetBalloonsBoundingBox() const;
 
+  BalloonCollectionBase& base() { return base_; }
+  Layout& layout() { return layout_; }
+
  private:
   // Adjusts the positions of the balloons (e.g., when one is closed).
   // Implemented by each platform for specific UI requirements.
@@ -164,6 +205,8 @@ class BalloonCollectionImpl : public BalloonCollection
   // The layout parameters for balloons in this collection.
   Layout layout_;
 
+  content::NotificationRegistrar registrar_;
+
 #if USE_OFFSETS
   // Start and stop observing all UI events.
   void AddMessageLoopObserver();
@@ -179,7 +222,7 @@ class BalloonCollectionImpl : public BalloonCollection
   bool IsCursorInBalloonCollection() const;
 
   // Factory for generating delayed reposition tasks on mouse motion.
-  ScopedRunnableMethodFactory<BalloonCollectionImpl> reposition_factory_;
+  base::WeakPtrFactory<BalloonCollectionImpl> reposition_factory_;
 
   // Is the balloon collection currently listening for UI events?
   bool added_as_message_loop_observer_;
