@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,26 +10,33 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string16.h"
-#include "chrome/browser/prefs/pref_change_registrar.h"
+#include "chrome/browser/content_settings/content_settings_provider.h"
+#include "chrome/browser/profiles/profile_keyed_service.h"
 #include "chrome/common/content_settings.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "googleurl/src/gurl.h"
+#include "third_party/WebKit/Source/WebKit/chromium/public/WebNotificationPresenter.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebTextDirection.h"
 
+class ContentSettingsPattern;
 class Notification;
 class NotificationUIManager;
-class NotificationsPrefsCache;
-class PrefService;
 class Profile;
 class TabContents;
-struct ViewHostMsg_ShowNotification_Params;
+
+namespace content {
+class WebContents;
+struct ShowDesktopNotificationHostMsgParams;
+}
 
 // The DesktopNotificationService is an object, owned by the Profile,
 // which provides the creation of desktop "toasts" to web pages and workers.
-class DesktopNotificationService : public NotificationObserver {
+class DesktopNotificationService : public content::NotificationObserver,
+                                   public ProfileKeyedService {
  public:
   enum DesktopNotificationSource {
     PageNotification,
@@ -42,19 +49,21 @@ class DesktopNotificationService : public NotificationObserver {
 
   // Requests permission (using an info-bar) for a given origin.
   // |callback_context| contains an opaque value to pass back to the
-  // requesting process when the info-bar finishes.
+  // requesting process when the info-bar finishes.  A NULL tab can be given if
+  // none exist (i.e. background tab), in which case the currently selected tab
+  // will be used.
   void RequestPermission(const GURL& origin,
                          int process_id,
                          int route_id,
                          int callback_context,
-                         TabContents* tab);
+                         content::WebContents* tab);
 
   // ShowNotification is called on the UI thread handling IPCs from a child
   // process, identified by |process_id| and |route_id|.  |source| indicates
   // whether the script is in a worker or page. |params| contains all the
   // other parameters supplied by the worker or page.
   bool ShowDesktopNotification(
-      const ViewHostMsg_ShowNotification_Params& params,
+      const content::ShowDesktopNotificationHostMsgParams& params,
       int process_id, int route_id, DesktopNotificationSource source);
 
   // Cancels a notification.  If it has already been shown, it will be
@@ -68,12 +77,10 @@ class DesktopNotificationService : public NotificationObserver {
   void GrantPermission(const GURL& origin);
   void DenyPermission(const GURL& origin);
 
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
-
-  NotificationsPrefsCache* prefs_cache() { return prefs_cache_; }
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Creates a data:xxxx URL which contains the full HTML for a notification
   // using supplied icon, title, and text, run through a template which contains
@@ -90,46 +97,38 @@ class DesktopNotificationService : public NotificationObserver {
                                 const std::vector<std::string>& subst);
 
   // The default content setting determines how to handle origins that haven't
-  // been allowed or denied yet.
-  ContentSetting GetDefaultContentSetting();
+  // been allowed or denied yet. If |provider_id| is not NULL, the id of the
+  // provider which provided the default setting is assigned to it.
+  ContentSetting GetDefaultContentSetting(std::string* provider_id);
   void SetDefaultContentSetting(ContentSetting setting);
-  bool IsDefaultContentSettingManaged() const;
 
   // NOTE: This should only be called on the UI thread.
   void ResetToDefaultContentSetting();
 
-  // Returns all origins that explicitly have been allowed.
-  std::vector<GURL> GetAllowedOrigins();
+  // Returns all notifications settings. |settings| is cleared before
+  // notifications setting are passed to it.
+  void GetNotificationsSettings(ContentSettingsForOneType* settings);
 
-  // Returns all origins that explicitly have been denied.
-  std::vector<GURL> GetBlockedOrigins();
-
-  // Removes an origin from the "explicitly allowed" set.
-  void ResetAllowedOrigin(const GURL& origin);
-
-  // Removes an origin from the "explicitly denied" set.
-  void ResetBlockedOrigin(const GURL& origin);
+  // Clears the notifications setting for the given pattern.
+  void ClearSetting(const ContentSettingsPattern& pattern);
 
   // Clears the sets of explicitly allowed and denied origins.
   void ResetAllOrigins();
 
-  static void RegisterUserPrefs(PrefService* user_prefs);
-
   ContentSetting GetContentSetting(const GURL& origin);
 
+  // Checks to see if a given origin has permission to create desktop
+  // notifications.
+  WebKit::WebNotificationPresenter::Permission
+      HasPermission(const GURL& origin);
+
  private:
-  void InitPrefs();
   void StartObserving();
   void StopObserving();
 
-  void OnPrefsChanged(const std::string& pref_name);
-
   // Takes a notification object and shows it in the UI.
   void ShowNotification(const Notification& notification);
 
-  // Save a permission change to the profile.
-  void PersistPermissionChange(const GURL& origin, bool is_allowed);
-
   // Returns a display name for an origin, to be used in permission infobar
   // or on the frame of the notification toast.  Different from the origin
   // itself when dealing with extensions.
@@ -138,19 +137,16 @@ class DesktopNotificationService : public NotificationObserver {
   // Notifies the observers when permissions settings change.
   void NotifySettingsChange();
 
+  NotificationUIManager* GetUIManager();
+
   // The profile which owns this object.
   Profile* profile_;
 
-  // A cache of preferences which is accessible only on the IO thread
-  // to service synchronous IPCs.
-  scoped_refptr<NotificationsPrefsCache> prefs_cache_;
-
   // Non-owned pointer to the notification manager which manages the
   // UI for desktop toasts.
   NotificationUIManager* ui_manager_;
 
-  PrefChangeRegistrar prefs_registrar_;
-  NotificationRegistrar notification_registrar_;
+  content::NotificationRegistrar notification_registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(DesktopNotificationService);
 };
