@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,9 +6,17 @@
 
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/prefs/browser_prefs.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/common/render_messages_params.h"
-#include "chrome/test/testing_pref_service.h"
+#include "chrome/test/base/testing_pref_service.h"
+#include "content/public/common/show_desktop_notification_params.h"
+
+#if defined(USE_AURA)
+#include "third_party/WebKit/Source/WebKit/chromium/public/WebKit.h"
+#include "ui/aura/root_window.h"
+#endif
+
+using content::BrowserThread;
 
 // static
 const int MockBalloonCollection::kMockBalloonSpace = 5;
@@ -16,6 +24,10 @@ const int MockBalloonCollection::kMockBalloonSpace = 5;
 // static
 std::string DesktopNotificationsTest::log_output_;
 
+MockBalloonCollection::MockBalloonCollection() {}
+
+MockBalloonCollection::~MockBalloonCollection() {}
+
 void MockBalloonCollection::Add(const Notification& notification,
                                 Profile* profile) {
   // Swap in a logging proxy for the purpose of logging calls that
@@ -30,6 +42,10 @@ void MockBalloonCollection::Add(const Notification& notification,
   BalloonCollectionImpl::Add(test_notification, profile);
 }
 
+bool MockBalloonCollection::HasSpace() const {
+  return count() < kMockBalloonSpace;
+}
+
 Balloon* MockBalloonCollection::MakeBalloon(const Notification& notification,
                                             Profile* profile) {
   // Start with a normal balloon but mock out the view.
@@ -50,6 +66,10 @@ void MockBalloonCollection::OnBalloonClosed(Balloon* source) {
   }
 }
 
+const BalloonCollection::Balloons& MockBalloonCollection::GetActiveBalloons() {
+  return balloons_;
+}
+
 int MockBalloonCollection::UppermostVerticalPosition() {
   int min = 0;
   std::deque<Balloon*>::iterator iter;
@@ -69,12 +89,18 @@ DesktopNotificationsTest::~DesktopNotificationsTest() {
 }
 
 void DesktopNotificationsTest::SetUp() {
+#if defined(USE_AURA)
+  WebKit::initialize(&webkit_platform_support_);
+  // MockBalloonCollection retrieves information about the screen on creation.
+  // So it is necessary to make sure the desktop gets created first.
+  aura::RootWindow::GetInstance();
+#endif
+
+  browser::RegisterLocalState(&local_state_);
   profile_.reset(new TestingProfile());
   balloon_collection_ = new MockBalloonCollection();
-  ui_manager_.reset(
-      new NotificationUIManager(profile_->GetTestingPrefService()));
-  ui_manager_->Initialize(balloon_collection_);
-  balloon_collection_->set_space_change_listener(ui_manager_.get());
+  ui_manager_.reset(NotificationUIManager::Create(&local_state_,
+                                                  balloon_collection_));
   service_.reset(new DesktopNotificationService(profile(), ui_manager_.get()));
   log_output_.clear();
 }
@@ -83,11 +109,15 @@ void DesktopNotificationsTest::TearDown() {
   service_.reset(NULL);
   ui_manager_.reset(NULL);
   profile_.reset(NULL);
+#if defined(USE_AURA)
+  aura::RootWindow::DeleteInstance();
+  WebKit::shutdown();
+#endif
 }
 
-ViewHostMsg_ShowNotification_Params
+content::ShowDesktopNotificationHostMsgParams
 DesktopNotificationsTest::StandardTestNotification() {
-  ViewHostMsg_ShowNotification_Params params;
+  content::ShowDesktopNotificationHostMsgParams params;
   params.notification_id = 0;
   params.origin = GURL("http://www.google.com");
   params.is_html = false;
@@ -99,7 +129,8 @@ DesktopNotificationsTest::StandardTestNotification() {
 }
 
 TEST_F(DesktopNotificationsTest, TestShow) {
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   params.notification_id = 1;
 
   EXPECT_TRUE(service_->ShowDesktopNotification(
@@ -107,7 +138,7 @@ TEST_F(DesktopNotificationsTest, TestShow) {
   MessageLoopForUI::current()->RunAllPending();
   EXPECT_EQ(1, balloon_collection_->count());
 
-  ViewHostMsg_ShowNotification_Params params2;
+  content::ShowDesktopNotificationHostMsgParams params2;
   params2.origin = GURL("http://www.google.com");
   params2.is_html = true;
   params2.contents_url = GURL("http://www.google.com/notification.html");
@@ -124,7 +155,8 @@ TEST_F(DesktopNotificationsTest, TestShow) {
 }
 
 TEST_F(DesktopNotificationsTest, TestClose) {
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   params.notification_id = 1;
 
   // Request a notification; should open a balloon.
@@ -148,7 +180,8 @@ TEST_F(DesktopNotificationsTest, TestCancel) {
   int route_id = 0;
   int notification_id = 1;
 
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   params.notification_id = notification_id;
 
   // Request a notification; should open a balloon.
@@ -173,7 +206,8 @@ TEST_F(DesktopNotificationsTest, TestCancel) {
 
 #if defined(OS_WIN) || defined(TOOLKIT_VIEWS)
 TEST_F(DesktopNotificationsTest, TestPositioning) {
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   std::string expected_log;
   // Create some toasts.  After each but the first, make sure there
   // is a minimum separation between the toasts.
@@ -193,7 +227,7 @@ TEST_F(DesktopNotificationsTest, TestPositioning) {
 }
 
 TEST_F(DesktopNotificationsTest, TestVariableSize) {
-  ViewHostMsg_ShowNotification_Params params;
+  content::ShowDesktopNotificationHostMsgParams params;
   params.origin = GURL("http://long.google.com");
   params.is_html = false;
   params.icon_url = GURL("/icon.png");
@@ -240,7 +274,8 @@ TEST_F(DesktopNotificationsTest, TestQueueing) {
   int route_id = 0;
 
   // Request lots of identical notifications.
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   const int kLotsOfToasts = 20;
   for (int id = 1; id <= kLotsOfToasts; ++id) {
     params.notification_id = id;
@@ -294,7 +329,8 @@ TEST_F(DesktopNotificationsTest, TestQueueing) {
 TEST_F(DesktopNotificationsTest, TestEarlyDestruction) {
   // Create some toasts and then prematurely delete the notification service,
   // just to make sure nothing crashes/leaks.
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   for (int id = 0; id <= 3; ++id) {
     params.notification_id = id;
     EXPECT_TRUE(service_->ShowDesktopNotification(
@@ -306,7 +342,8 @@ TEST_F(DesktopNotificationsTest, TestEarlyDestruction) {
 TEST_F(DesktopNotificationsTest, TestUserInputEscaping) {
   // Create a test script with some HTML; assert that it doesn't get into the
   // data:// URL that's produced for the balloon.
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   params.title = ASCIIToUTF16("<script>window.alert('uh oh');</script>");
   params.body = ASCIIToUTF16("<i>this text is in italics</i>");
   params.notification_id = 1;
@@ -326,7 +363,8 @@ TEST_F(DesktopNotificationsTest, TestUserInputEscaping) {
 
 TEST_F(DesktopNotificationsTest, TestBoundingBox) {
   // Create some notifications.
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   for (int id = 0; id <= 3; ++id) {
     params.notification_id = id;
     EXPECT_TRUE(service_->ShowDesktopNotification(
@@ -359,11 +397,12 @@ TEST_F(DesktopNotificationsTest, TestBoundingBox) {
 
 TEST_F(DesktopNotificationsTest, TestPositionPreference) {
   // Set position preference to lower right.
-  profile_->GetPrefs()->SetInteger(prefs::kDesktopNotificationPosition,
-                                   BalloonCollection::LOWER_RIGHT);
+  local_state_.SetInteger(prefs::kDesktopNotificationPosition,
+                          BalloonCollection::LOWER_RIGHT);
 
   // Create some notifications.
-  ViewHostMsg_ShowNotification_Params params = StandardTestNotification();
+  content::ShowDesktopNotificationHostMsgParams params =
+      StandardTestNotification();
   for (int id = 0; id <= 3; ++id) {
     params.notification_id = id;
     EXPECT_TRUE(service_->ShowDesktopNotification(
@@ -398,8 +437,8 @@ TEST_F(DesktopNotificationsTest, TestPositionPreference) {
 
   // Now change the position to upper right.  This should cause an immediate
   // repositioning, and we check for the reverse ordering.
-  profile_->GetPrefs()->SetInteger(prefs::kDesktopNotificationPosition,
-                                   BalloonCollection::UPPER_RIGHT);
+  local_state_.SetInteger(prefs::kDesktopNotificationPosition,
+                          BalloonCollection::UPPER_RIGHT);
   last_x = -1;
   last_y = -1;
 
@@ -424,8 +463,8 @@ TEST_F(DesktopNotificationsTest, TestPositionPreference) {
 
   // Now change the position to upper left.  Confirm that the X value for the
   // balloons gets smaller.
-  profile_->GetPrefs()->SetInteger(prefs::kDesktopNotificationPosition,
-                                   BalloonCollection::UPPER_LEFT);
+  local_state_.SetInteger(prefs::kDesktopNotificationPosition,
+                          BalloonCollection::UPPER_LEFT);
 
   int current_x = (*balloons.begin())->GetPosition().x();
   EXPECT_LT(current_x, last_x);
