@@ -1,32 +1,31 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/notifications/notification_options_menu_model.h"
 
+#include <string>
+
 #include "base/compiler_specific.h"
 #include "base/logging.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/notifications/balloon_collection.h"
 #include "chrome/browser/notifications/desktop_notification_service.h"
+#include "chrome/browser/notifications/desktop_notification_service_factory.h"
 #include "chrome/browser/notifications/notification.h"
 #include "chrome/browser/notifications/notification_ui_manager.h"
-#include "chrome/browser/notifications/notifications_prefs_cache.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/content_settings_types.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/url_constants.h"
+#include "content/public/browser/web_contents_delegate.h"
 #include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util.h"
 
-#if defined(OS_WIN)
-#include "chrome/browser/ui/views/browser_dialogs.h"
-#endif  // OS_WIN
-
 // Menu commands
 const int kTogglePermissionCommand = 0;
 const int kToggleExtensionCommand = 1;
@@ -68,8 +67,6 @@ bool CornerSelectionMenuModel::IsCommandIdChecked(int command_id) const {
   NotificationUIManager* ui = g_browser_process->notification_ui_manager();
   BalloonCollection::PositionPreference current = ui->GetPositionPreference();
 
-  LOG(INFO) << "Current position preference: " << current;
-
   if (command_id == kCornerUpperLeft)
     return (current == BalloonCollection::UPPER_LEFT);
   else if (command_id == kCornerUpperRight)
@@ -99,8 +96,6 @@ bool CornerSelectionMenuModel::GetAcceleratorForCommandId(
 void CornerSelectionMenuModel::ExecuteCommand(int command_id) {
   NotificationUIManager* ui = g_browser_process->notification_ui_manager();
 
-  LOG(INFO) << "Executing command: " << command_id;
-
   if (command_id == kCornerUpperLeft)
     ui->SetPositionPreference(BalloonCollection::UPPER_LEFT);
   else if (command_id == kCornerUpperRight)
@@ -118,24 +113,34 @@ void CornerSelectionMenuModel::ExecuteCommand(int command_id) {
 NotificationOptionsMenuModel::NotificationOptionsMenuModel(Balloon* balloon)
     : ALLOW_THIS_IN_INITIALIZER_LIST(ui::SimpleMenuModel(this)),
       balloon_(balloon) {
-
   const Notification& notification = balloon->notification();
   const GURL& origin = notification.origin_url();
 
   if (origin.SchemeIs(chrome::kExtensionScheme)) {
-    const string16 disable_label = l10n_util::GetStringUTF16(
-        IDS_EXTENSIONS_DISABLE);
-    AddItem(kToggleExtensionCommand, disable_label);
-  } else {
+    ExtensionService* extension_service =
+        balloon_->profile()->GetExtensionService();
+    const Extension* extension =
+        extension_service->extensions()->GetExtensionOrAppByURL(
+            ExtensionURLInfo(origin));
+    // We get back no extension here when we show the notification after
+    // the extension has crashed.
+    if (extension) {
+      const string16 disable_label = l10n_util::GetStringUTF16(
+          IDS_EXTENSIONS_DISABLE);
+      AddItem(kToggleExtensionCommand, disable_label);
+    }
+  } else if (!notification.display_source().empty()) {
     const string16 disable_label = l10n_util::GetStringFUTF16(
         IDS_NOTIFICATION_BALLOON_REVOKE_MESSAGE,
         notification.display_source());
     AddItem(kTogglePermissionCommand, disable_label);
   }
 
-  const string16 settings_label = l10n_util::GetStringUTF16(
-      IDS_NOTIFICATIONS_SETTINGS_BUTTON);
-  AddItem(kOpenContentSettingsCommand, settings_label);
+  if (!notification.display_source().empty()) {
+    const string16 settings_label = l10n_util::GetStringUTF16(
+        IDS_NOTIFICATIONS_SETTINGS_BUTTON);
+    AddItem(kOpenContentSettingsCommand, settings_label);
+  }
 
   corner_menu_model_.reset(new CornerSelectionMenuModel(balloon));
   AddSubMenu(kCornerSelectionSubMenu,
@@ -154,26 +159,26 @@ bool NotificationOptionsMenuModel::IsItemForCommandIdDynamic(int command_id)
 
 string16 NotificationOptionsMenuModel::GetLabelForCommandId(int command_id)
     const {
-  // TODO(tfarina,johnnyg): Removed this code if we decide to close
-  // notifications after permissions are revoked.
+  // TODO(tfarina,johnnyg): Remove this code if we decide to close notifications
+  // after permissions are revoked.
   if (command_id == kTogglePermissionCommand ||
       command_id == kToggleExtensionCommand) {
     const Notification& notification = balloon_->notification();
     const GURL& origin = notification.origin_url();
 
     DesktopNotificationService* service =
-        balloon_->profile()->GetDesktopNotificationService();
+        DesktopNotificationServiceFactory::GetForProfile(balloon_->profile());
     if (origin.SchemeIs(chrome::kExtensionScheme)) {
-      ExtensionService* ext_service =
+      ExtensionService* extension_service =
           balloon_->profile()->GetExtensionService();
-      const Extension* extension = ext_service->GetExtensionByURL(origin);
+      const Extension* extension =
+          extension_service->extensions()->GetExtensionOrAppByURL(
+              ExtensionURLInfo(origin));
       if (extension) {
-        ExtensionPrefs* extension_prefs = ext_service->extension_prefs();
-        const std::string& id = extension->id();
-        if (extension_prefs->GetExtensionState(id) == Extension::ENABLED)
-          return l10n_util::GetStringUTF16(IDS_EXTENSIONS_DISABLE);
-        else
-          return l10n_util::GetStringUTF16(IDS_EXTENSIONS_ENABLE);
+        return l10n_util::GetStringUTF16(
+            extension_service->IsExtensionEnabled(extension->id()) ?
+                IDS_EXTENSIONS_DISABLE :
+                IDS_EXTENSIONS_ENABLE);
       }
     } else {
       if (service->GetContentSetting(origin) == CONTENT_SETTING_ALLOW) {
@@ -212,8 +217,8 @@ bool NotificationOptionsMenuModel::GetAcceleratorForCommandId(
 
 void NotificationOptionsMenuModel::ExecuteCommand(int command_id) {
   DesktopNotificationService* service =
-      balloon_->profile()->GetDesktopNotificationService();
-  ExtensionService* ext_service =
+      DesktopNotificationServiceFactory::GetForProfile(balloon_->profile());
+  ExtensionService* extension_service =
       balloon_->profile()->GetExtensionService();
   const GURL& origin = balloon_->notification().origin_url();
   switch (command_id) {
@@ -224,33 +229,27 @@ void NotificationOptionsMenuModel::ExecuteCommand(int command_id) {
         service->GrantPermission(origin);
       break;
     case kToggleExtensionCommand: {
-      const Extension* extension = ext_service->GetExtensionByURL(origin);
+      const Extension* extension =
+          extension_service->extensions()->GetExtensionOrAppByURL(
+              ExtensionURLInfo(origin));
       if (extension) {
-        ExtensionPrefs* extension_prefs = ext_service->extension_prefs();
         const std::string& id = extension->id();
-        if (extension_prefs->GetExtensionState(id) == Extension::ENABLED)
-          ext_service->DisableExtension(id);
+        if (extension_service->IsExtensionEnabled(id))
+          extension_service->DisableExtension(id);
         else
-          ext_service->EnableExtension(id);
+          extension_service->EnableExtension(id);
       }
       break;
     }
     case kOpenContentSettingsCommand: {
-      Browser* browser = BrowserList::GetLastActive();
-      if (browser) {
-        static_cast<TabContentsDelegate*>(browser)->ShowContentSettingsWindow(
-            CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
-      } else {
-#if defined(OS_WIN)
-        if (CommandLine::ForCurrentProcess()->HasSwitch(
-                switches::kChromeFrame)) {
-          // We may not have a browser if this is a chrome frame process.
-          browser::ShowContentSettingsWindow(NULL,
-                                             CONTENT_SETTINGS_TYPE_DEFAULT,
-                                             balloon_->profile());
-        }
-#endif  // OS_WIN
+      Browser* browser =
+          BrowserList::GetLastActiveWithProfile(balloon_->profile());
+      if (!browser) {
+        // It is possible that there is no browser window (e.g. when there are
+        // background pages, or for a chrome frame process on windows).
+        browser = Browser::Create(balloon_->profile());
       }
+      browser->ShowContentSettingsPage(CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
       break;
     }
     default:
