@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,21 +10,21 @@
 
 #include "chrome/browser/notifications/notification_object_proxy.h"
 #include "chrome/browser/notifications/balloon.h"
-#include "gfx/size.h"
+#include "ui/gfx/size.h"
 
 // NotificationDelegate which does nothing, useful for testing when
 // the notification events are not important.
 class MockNotificationDelegate : public NotificationDelegate {
  public:
-  explicit MockNotificationDelegate(const std::string& id) : id_(id) {}
-  virtual ~MockNotificationDelegate() {}
+  explicit MockNotificationDelegate(const std::string& id);
+  virtual ~MockNotificationDelegate();
 
   // NotificationDelegate interface.
-  virtual void Display() {}
-  virtual void Error() {}
-  virtual void Close(bool by_user) {}
-  virtual void Click() {}
-  virtual std::string id() const { return id_; }
+  virtual void Display() OVERRIDE {}
+  virtual void Error() OVERRIDE {}
+  virtual void Close(bool by_user) OVERRIDE {}
+  virtual void Click() OVERRIDE {}
+  virtual std::string id() const OVERRIDE;
 
  private:
   std::string id_;
@@ -77,12 +77,14 @@ class MockBalloonView : public BalloonView {
  public:
   explicit MockBalloonView(Balloon * balloon) :
       balloon_(balloon) {}
-  void Show(Balloon* balloon) {}
-  void Update() {}
-  void RepositionToBalloon() {}
-  void Close(bool by_user) { balloon_->OnClose(by_user); }
-  gfx::Size GetSize() const { return balloon_->content_size(); }
-  BalloonHost* GetHost() const { return NULL; }
+
+  // BalloonView:
+  virtual void Show(Balloon* balloon) OVERRIDE {}
+  virtual void Update() OVERRIDE {}
+  virtual void RepositionToBalloon() OVERRIDE {}
+  virtual void Close(bool by_user) OVERRIDE;
+  virtual gfx::Size GetSize() const OVERRIDE;
+  virtual BalloonHost* GetHost() const OVERRIDE;
 
  private:
   // Non-owned pointer.
