@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,28 +8,28 @@
 
 #include <deque>
 #include <string>
+#include <vector>
 
 #include "base/id_map.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/timer.h"
 #include "chrome/browser/notifications/balloon.h"
 #include "chrome/browser/notifications/balloon_collection.h"
 #include "chrome/browser/prefs/pref_member.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
 class Notification;
 class PrefService;
 class Profile;
 class QueuedNotification;
-class SiteInstance;
 
 // The notification manager manages use of the desktop for notifications.
 // It maintains a queue of pending notifications when space becomes constrained.
 class NotificationUIManager
     : public BalloonCollection::BalloonSpaceChangeListener,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
-  explicit NotificationUIManager(PrefService* local_state);
   virtual ~NotificationUIManager();
 
   // Creates an initialized UI manager with a new balloon collection
@@ -37,6 +37,12 @@ class NotificationUIManager
   // Except for unit tests, this is the way to construct the object.
   static NotificationUIManager* Create(PrefService* local_state);
 
+  // Creates an initialized UI manager with the given balloon collection
+  // and the listener relationship setup.
+  // Used primarily by unit tests.
+  static NotificationUIManager* Create(PrefService* local_state,
+                                       BalloonCollection* balloons);
+
   // Registers preferences.
   static void RegisterPrefs(PrefService* prefs);
 
@@ -73,11 +79,18 @@ class NotificationUIManager
   // be placed on the screen.
   void SetPositionPreference(BalloonCollection::PositionPreference preference);
 
+  // Retrieves an ordered list of all queued notifications.
+  // Used only for automation/testing.
+  void GetQueuedNotificationsForTesting(
+      std::vector<const Notification*>* notifications);
+
  private:
-  // NotificationObserver override.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  explicit NotificationUIManager(PrefService* local_state);
+
+  // content::NotificationObserver override.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Attempts to display notifications from the show_queue if the user
   // is active.
@@ -87,12 +100,15 @@ class NotificationUIManager
   void ShowNotifications();
 
   // BalloonCollectionObserver implementation.
-  virtual void OnBalloonSpaceChanged();
+  virtual void OnBalloonSpaceChanged() OVERRIDE;
 
   // Replace an existing notification with this one if applicable;
   // returns true if the replacement happened.
   bool TryReplacement(const Notification& notification);
 
+  // Checks the user state to decide if we want to show the notification.
+  void CheckUserState();
+
   // An owned pointer to the collection of active balloons.
   scoped_ptr<BalloonCollection> balloon_collection_;
 
@@ -101,11 +117,15 @@ class NotificationUIManager
   NotificationDeque show_queue_;
 
   // Registrar for the other kind of notifications (event signaling).
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // Prefs listener for the position preference.
   IntegerPrefMember position_pref_;
 
+  // Used by screen-saver and full-screen handling support.
+  bool is_user_active_;
+  base::RepeatingTimer<NotificationUIManager> user_state_check_timer_;
+
   DISALLOW_COPY_AND_ASSIGN(NotificationUIManager);
 };
 
