@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,34 +6,49 @@
 
 #include <string>
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/command_line.h"
 #include "base/i18n/time_formatting.h"
+#include "base/string16.h"
 #include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/cert_store.h"
+#include "chrome/browser/page_info_model_observer.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/ssl/ssl_manager.h"
+#include "chrome/browser/ssl/ssl_error_info.h"
+#include "content/browser/cert_store.h"
+#include "content/browser/ssl/ssl_manager.h"
+#include "content/public/browser/ssl_status.h"
+#include "content/public/common/url_constants.h"
+#include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
 #include "net/base/cert_status_flags.h"
-#include "net/base/ssl_connection_status_flags.h"
 #include "net/base/ssl_cipher_suite_names.h"
+#include "net/base/ssl_connection_status_flags.h"
 #include "net/base/x509_certificate.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
-#if defined(OS_MACOSX)
-#include "base/mac/mac_util.h"
-#endif
+using content::SSLStatus;
 
 PageInfoModel::PageInfoModel(Profile* profile,
                              const GURL& url,
-                             const NavigationEntry::SSLStatus& ssl,
+                             const SSLStatus& ssl,
                              bool show_history,
                              PageInfoModelObserver* observer)
     : observer_(observer) {
   Init();
 
+  if (url.SchemeIs(chrome::kChromeUIScheme)) {
+    sections_.push_back(
+        SectionInfo(ICON_STATE_INTERNAL_PAGE,
+                    string16(),
+                    l10n_util::GetStringUTF16(IDS_PAGE_INFO_INTERNAL_PAGE),
+                    SECTION_INFO_INTERNAL_PAGE));
+    return;
+  }
+
   SectionStateIcon icon_id = ICON_STATE_OK;
   string16 headline;
   string16 description;
@@ -48,19 +63,12 @@ PageInfoModel::PageInfoModel(Profile* profile,
     empty_subject_name = true;
   }
 
-  // Some of what IsCertStatusError classifies as errors we want to show as
-  // warnings instead.
-  static const int cert_warnings =
-      net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION |
-      net::CERT_STATUS_NO_REVOCATION_MECHANISM;
-  int status_with_warnings_removed = ssl.cert_status() & ~cert_warnings;
-
-  if (ssl.cert_id() &&
-      CertStore::GetInstance()->RetrieveCert(ssl.cert_id(), &cert) &&
-      !net::IsCertStatusError(status_with_warnings_removed)) {
-    // No error found so far, check cert_status warnings.
-    int cert_status = ssl.cert_status();
-    if (cert_status & cert_warnings) {
+  if (ssl.cert_id &&
+      CertStore::GetInstance()->RetrieveCert(ssl.cert_id, &cert) &&
+      (!net::IsCertStatusError(ssl.cert_status) ||
+       net::IsCertStatusMinorError(ssl.cert_status))) {
+    // There are no major errors. Check for minor errors.
+    if (net::IsCertStatusMinorError(ssl.cert_status)) {
       string16 issuer_name(UTF8ToUTF16(cert->issuer().GetDisplayName()));
       if (issuer_name.empty()) {
         issuer_name.assign(l10n_util::GetStringUTF16(
@@ -70,17 +78,17 @@ PageInfoModel::PageInfoModel(Profile* profile,
           IDS_PAGE_INFO_SECURITY_TAB_SECURE_IDENTITY, issuer_name));
 
       description += ASCIIToUTF16("\n\n");
-      if (cert_status & net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION) {
+      if (ssl.cert_status & net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION) {
         description += l10n_util::GetStringUTF16(
             IDS_PAGE_INFO_SECURITY_TAB_UNABLE_TO_CHECK_REVOCATION);
-      } else if (cert_status & net::CERT_STATUS_NO_REVOCATION_MECHANISM) {
+      } else if (ssl.cert_status & net::CERT_STATUS_NO_REVOCATION_MECHANISM) {
         description += l10n_util::GetStringUTF16(
             IDS_PAGE_INFO_SECURITY_TAB_NO_REVOCATION_MECHANISM);
       } else {
         NOTREACHED() << "Need to specify string for this warning";
       }
       icon_id = ICON_STATE_WARNING_MINOR;
-    } else if ((ssl.cert_status() & net::CERT_STATUS_IS_EV) != 0) {
+    } else if (ssl.cert_status & net::CERT_STATUS_IS_EV) {
       // EV HTTPS page.
       DCHECK(!cert->subject().organization_names.empty());
       headline =
@@ -110,7 +118,7 @@ PageInfoModel::PageInfoModel(Profile* profile,
           UTF8ToUTF16(cert->subject().organization_names[0]),
           locality,
           UTF8ToUTF16(cert->issuer().GetDisplayName())));
-    } else if ((ssl.cert_status() & net::CERT_STATUS_IS_DNSSEC) != 0) {
+    } else if (ssl.cert_status & net::CERT_STATUS_IS_DNSSEC) {
       // DNSSEC authenticated page.
       if (empty_subject_name)
         headline.clear();  // Don't display any title.
@@ -136,10 +144,19 @@ PageInfoModel::PageInfoModel(Profile* profile,
     // HTTP or HTTPS with errors (not warnings).
     description.assign(l10n_util::GetStringUTF16(
         IDS_PAGE_INFO_SECURITY_TAB_INSECURE_IDENTITY));
-    icon_id = ssl.security_style() == SECURITY_STYLE_UNAUTHENTICATED ?
+    icon_id = ssl.security_style == content::SECURITY_STYLE_UNAUTHENTICATED ?
         ICON_STATE_WARNING_MAJOR : ICON_STATE_ERROR;
 
-    if (ssl.cert_status() & net::CERT_STATUS_NON_UNIQUE_NAME) {
+    const string16 bullet = UTF8ToUTF16("\n • ");
+    std::vector<SSLErrorInfo> errors;
+    SSLErrorInfo::GetErrorsForCertStatus(ssl.cert_id, ssl.cert_status,
+                                         url, &errors);
+    for (size_t i = 0; i < errors.size(); ++i) {
+      description += bullet;
+      description += errors[i].short_description();
+    }
+
+    if (ssl.cert_status & net::CERT_STATUS_NON_UNIQUE_NAME) {
       description += ASCIIToUTF16("\n\n");
       description += l10n_util::GetStringUTF16(
           IDS_PAGE_INFO_SECURITY_TAB_NON_UNIQUE_NAME);
@@ -158,24 +175,24 @@ PageInfoModel::PageInfoModel(Profile* profile,
   icon_id = ICON_STATE_OK;
   headline.clear();
   description.clear();
-  if (!ssl.cert_id()) {
+  if (!ssl.cert_id) {
     // Not HTTPS.
-    DCHECK_EQ(ssl.security_style(), SECURITY_STYLE_UNAUTHENTICATED);
-    icon_id = ssl.security_style() == SECURITY_STYLE_UNAUTHENTICATED ?
+    DCHECK_EQ(ssl.security_style, content::SECURITY_STYLE_UNAUTHENTICATED);
+    icon_id = ssl.security_style == content::SECURITY_STYLE_UNAUTHENTICATED ?
         ICON_STATE_WARNING_MAJOR : ICON_STATE_ERROR;
     description.assign(l10n_util::GetStringFUTF16(
         IDS_PAGE_INFO_SECURITY_TAB_NOT_ENCRYPTED_CONNECTION_TEXT,
         subject_name));
-  } else if (ssl.security_bits() < 0) {
+  } else if (ssl.security_bits < 0) {
     // Security strength is unknown.  Say nothing.
     icon_id = ICON_STATE_ERROR;
-  } else if (ssl.security_bits() == 0) {
-    DCHECK_NE(ssl.security_style(), SECURITY_STYLE_UNAUTHENTICATED);
+  } else if (ssl.security_bits == 0) {
+    DCHECK_NE(ssl.security_style, content::SECURITY_STYLE_UNAUTHENTICATED);
     icon_id = ICON_STATE_ERROR;
     description.assign(l10n_util::GetStringFUTF16(
         IDS_PAGE_INFO_SECURITY_TAB_NOT_ENCRYPTED_CONNECTION_TEXT,
         subject_name));
-  } else if (ssl.security_bits() < 80) {
+  } else if (ssl.security_bits < 80) {
     icon_id = ICON_STATE_ERROR;
     description.assign(l10n_util::GetStringFUTF16(
         IDS_PAGE_INFO_SECURITY_TAB_WEAK_ENCRYPTION_CONNECTION_TEXT,
@@ -184,24 +201,26 @@ PageInfoModel::PageInfoModel(Profile* profile,
     description.assign(l10n_util::GetStringFUTF16(
         IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_CONNECTION_TEXT,
         subject_name,
-        base::IntToString16(ssl.security_bits())));
-    if (ssl.displayed_insecure_content() || ssl.ran_insecure_content()) {
-      icon_id = ssl.ran_insecure_content() ?
+        base::IntToString16(ssl.security_bits)));
+    if (ssl.content_status) {
+      bool ran_insecure_content =
+          !!(ssl.content_status & SSLStatus::RAN_INSECURE_CONTENT);
+      icon_id = ran_insecure_content ?
           ICON_STATE_ERROR : ICON_STATE_WARNING_MINOR;
       description.assign(l10n_util::GetStringFUTF16(
           IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_SENTENCE_LINK,
           description,
-          l10n_util::GetStringUTF16(ssl.ran_insecure_content() ?
+          l10n_util::GetStringUTF16(ran_insecure_content ?
               IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_INSECURE_CONTENT_ERROR :
               IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_INSECURE_CONTENT_WARNING)));
     }
   }
 
   uint16 cipher_suite =
-      net::SSLConnectionStatusToCipherSuite(ssl.connection_status());
-  if (ssl.security_bits() > 0 && cipher_suite) {
+      net::SSLConnectionStatusToCipherSuite(ssl.connection_status);
+  if (ssl.security_bits > 0 && cipher_suite) {
     int ssl_version =
-        net::SSLConnectionStatusToVersion(ssl.connection_status());
+        net::SSLConnectionStatusToVersion(ssl.connection_status);
     const char* ssl_version_str;
     net::SSLVersionToString(&ssl_version_str, ssl_version);
     description += ASCIIToUTF16("\n\n");
@@ -209,10 +228,10 @@ PageInfoModel::PageInfoModel(Profile* profile,
         IDS_PAGE_INFO_SECURITY_TAB_SSL_VERSION,
         ASCIIToUTF16(ssl_version_str));
 
-    bool did_fallback = (ssl.connection_status() &
+    bool did_fallback = (ssl.connection_status &
                          net::SSL_CONNECTION_SSL3_FALLBACK) != 0;
     bool no_renegotiation =
-        (ssl.connection_status() &
+        (ssl.connection_status &
         net::SSL_CONNECTION_NO_RENEGOTIATION_EXTENSION) != 0;
     const char *key_exchange, *cipher, *mac;
     net::SSLCipherSuiteToStrings(&key_exchange, &cipher, &mac, cipher_suite);
@@ -224,7 +243,7 @@ PageInfoModel::PageInfoModel(Profile* profile,
 
     description += ASCIIToUTF16("\n\n");
     uint8 compression_id =
-        net::SSLConnectionStatusToCompression(ssl.connection_status());
+        net::SSLConnectionStatusToCompression(ssl.connection_status);
     if (compression_id) {
       const char* compression;
       net::SSLCompressionToString(&compression, compression_id);
@@ -238,7 +257,8 @@ PageInfoModel::PageInfoModel(Profile* profile,
 
     if (did_fallback) {
       // For now, only SSLv3 fallback will trigger a warning icon.
-      icon_id = ICON_STATE_ERROR;
+      if (icon_id < ICON_STATE_WARNING_MINOR)
+        icon_id = ICON_STATE_WARNING_MINOR;
       description += ASCIIToUTF16("\n\n");
       description += l10n_util::GetStringUTF16(
           IDS_PAGE_INFO_SECURITY_TAB_FALLBACK_MESSAGE);
@@ -262,22 +282,15 @@ PageInfoModel::PageInfoModel(Profile* profile,
   HistoryService* history = profile->GetHistoryService(
       Profile::EXPLICIT_ACCESS);
   if (show_history && history) {
-    history->GetVisitCountToHost(
+    history->GetVisibleVisitCountToHost(
         url,
         &request_consumer_,
-        NewCallback(this, &PageInfoModel::OnGotVisitCountToHost));
+        base::Bind(&PageInfoModel::OnGotVisitCountToHost,
+                   base::Unretained(this)));
   }
 }
 
-PageInfoModel::~PageInfoModel() {
-#if defined(OS_MACOSX)
-  // Release the NSImages.
-  for (std::vector<gfx::NativeImage>::iterator it = icons_.begin();
-       it != icons_.end(); ++it) {
-    base::mac::NSObjectRelease(*it);
-  }
-#endif
-}
+PageInfoModel::~PageInfoModel() {}
 
 int PageInfoModel::GetSectionCount() {
   return sections_.size();
@@ -288,7 +301,7 @@ PageInfoModel::SectionInfo PageInfoModel::GetSectionInfo(int index) {
   return sections_[index];
 }
 
-gfx::NativeImage PageInfoModel::GetIconImage(SectionStateIcon icon_id) {
+gfx::Image* PageInfoModel::GetIconImage(SectionStateIcon icon_id) {
   if (icon_id == ICON_NONE)
     return NULL;
   // The bubble uses new, various icons.
@@ -329,7 +342,7 @@ void PageInfoModel::OnGotVisitCountToHost(HistoryService::Handle handle,
             base::TimeFormatShortDate(first_visit)),
         SECTION_INFO_FIRST_VISIT));
   }
-  observer_->ModelChanged();
+  observer_->OnPageInfoModelChanged();
 }
 
 PageInfoModel::PageInfoModel() : observer_(NULL) {
@@ -339,20 +352,11 @@ PageInfoModel::PageInfoModel() : observer_(NULL) {
 void PageInfoModel::Init() {
   // Loads the icons into the vector. The order must match the SectionStateIcon
   // enum.
-  icons_.push_back(GetBitmapNamed(IDR_PAGEINFO_GOOD));
-  icons_.push_back(GetBitmapNamed(IDR_PAGEINFO_WARNING_MINOR));
-  icons_.push_back(GetBitmapNamed(IDR_PAGEINFO_WARNING_MAJOR));
-  icons_.push_back(GetBitmapNamed(IDR_PAGEINFO_BAD));
-  icons_.push_back(GetBitmapNamed(IDR_PAGEINFO_INFO));
-}
-
-gfx::NativeImage PageInfoModel::GetBitmapNamed(int resource_id) {
   ResourceBundle& rb = ResourceBundle::GetSharedInstance();
-  gfx::NativeImage image = rb.GetNativeImageNamed(resource_id);
-#if defined(OS_MACOSX)
-  // Unlike other platforms, the Mac ResourceBundle does not keep a shared image
-  // cache. These are released in the dtor.
-  base::mac::NSObjectRetain(image);
-#endif
-  return image;
+  icons_.push_back(&rb.GetNativeImageNamed(IDR_PAGEINFO_GOOD));
+  icons_.push_back(&rb.GetNativeImageNamed(IDR_PAGEINFO_WARNING_MINOR));
+  icons_.push_back(&rb.GetNativeImageNamed(IDR_PAGEINFO_WARNING_MAJOR));
+  icons_.push_back(&rb.GetNativeImageNamed(IDR_PAGEINFO_BAD));
+  icons_.push_back(&rb.GetNativeImageNamed(IDR_PAGEINFO_INFO));
+  icons_.push_back(&rb.GetNativeImageNamed(IDR_PRODUCT_LOGO_26));
 }
