@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,33 +8,32 @@
 
 #include <vector>
 
-#include "gfx/native_widget_types.h"
 #include "base/string16.h"
 #include "chrome/browser/cancelable_request.h"
 #include "chrome/browser/history/history.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
 #include "googleurl/src/gurl.h"
+#include "ui/gfx/image/image.h"
 
-class PrefService;
+class PageInfoModelObserver;
 class Profile;
 
+namespace content {
+struct SSLStatus;
+}
+
 // The model that provides the information that should be displayed in the page
 // info dialog/bubble.
 class PageInfoModel {
  public:
-  class PageInfoModelObserver {
-   public:
-    virtual ~PageInfoModelObserver() {}
-
-    virtual void ModelChanged() = 0;
-  };
-
   enum SectionInfoType {
     SECTION_INFO_IDENTITY = 0,
     SECTION_INFO_CONNECTION,
     SECTION_INFO_FIRST_VISIT,
+    SECTION_INFO_INTERNAL_PAGE,  // Used for chrome:// pages, etc.
   };
 
+  // NOTE: ICON_STATE_OK ... ICON_STATE_ERROR must be listed in increasing
+  // order of severity.  Code may depend on this order.
   enum SectionStateIcon {
     // No icon.
     ICON_NONE = -1,
@@ -47,7 +46,9 @@ class PageInfoModel {
     // For example, unverified identity over HTTPS.
     ICON_STATE_ERROR,
     // An information icon.
-    ICON_STATE_INFO
+    ICON_STATE_INFO,
+    // Icon for internal pages.
+    ICON_STATE_INTERNAL_PAGE,
   };
 
   struct SectionInfo {
@@ -77,7 +78,7 @@ class PageInfoModel {
 
   PageInfoModel(Profile* profile,
                 const GURL& url,
-                const NavigationEntry::SSLStatus& ssl,
+                const content::SSLStatus& ssl,
                 bool show_history,
                 PageInfoModelObserver* observer);
   ~PageInfoModel();
@@ -86,7 +87,7 @@ class PageInfoModel {
   SectionInfo GetSectionInfo(int index);
 
   // Returns the native image type for an icon with the given id.
-  gfx::NativeImage GetIconImage(SectionStateIcon icon_id);
+  gfx::Image* GetIconImage(SectionStateIcon icon_id);
 
   // Callback from history service with number of visits to url.
   void OnGotVisitCountToHost(HistoryService::Handle handle,
@@ -101,16 +102,12 @@ class PageInfoModel {
   // Shared initialization for default and testing constructor.
   void Init();
 
-  // Wrapper for ResourceBundle::GetNativeImage() so that Mac can retain its
-  // icons.
-  gfx::NativeImage GetBitmapNamed(int resource_id);
-
   PageInfoModelObserver* observer_;
 
   std::vector<SectionInfo> sections_;
 
   // All possible icons that go next to the text descriptions to indicate state.
-  std::vector<gfx::NativeImage> icons_;
+  std::vector<gfx::Image*> icons_;
 
   // Used to request number of visits.
   CancelableRequestConsumer request_consumer_;
