@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,7 +12,7 @@ class JpegMetadataParser : public FileMetadataParser {
  public:
   explicit JpegMetadataParser(const FilePath& path);
   // Implementation of MetadataParser
-  virtual bool Parse();
+  virtual bool Parse() OVERRIDE;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(JpegMetadataParser);
