@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #pragma once
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "chrome/browser/parsers/metadata_parser_factory.h"
 
 class FilePath;
@@ -16,8 +17,10 @@ class MetadataParserJpegFactory : public MetadataParserFactory {
   MetadataParserJpegFactory();
 
   // Implementation of MetadataParserFactory
-  virtual bool CanParse(const FilePath& path, char* bytes, int bytes_size);
-  virtual MetadataParser* CreateParser(const FilePath& path);
+  virtual bool CanParse(const FilePath& path,
+                        char* bytes,
+                        int bytes_size) OVERRIDE;
+  virtual MetadataParser* CreateParser(const FilePath& path) OVERRIDE;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MetadataParserJpegFactory);
