@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,12 +10,13 @@
 #include "base/string_split.h"
 #include "base/string_util.h"
 #include "chrome/browser/password_manager/password_manager.h"
+#include "chrome/browser/password_manager/password_store.h"
 #include "chrome/browser/profiles/profile.h"
-#include "webkit/glue/password_form_dom_manager.h"
+#include "webkit/forms/password_form_dom_manager.h"
 
 using base::Time;
-using webkit_glue::PasswordForm;
-using webkit_glue::PasswordFormMap;
+using webkit::forms::PasswordForm;
+using webkit::forms::PasswordFormMap;
 
 PasswordFormManager::PasswordFormManager(Profile* profile,
                                          PasswordManager* password_manager,
@@ -39,7 +40,6 @@ PasswordFormManager::PasswordFormManager(Profile* profile,
 }
 
 PasswordFormManager::~PasswordFormManager() {
-  CancelLoginsQuery();
   UMA_HISTOGRAM_ENUMERATION("PasswordManager.ActionsTaken",
                             GetActionsTaken(),
                             kMaxNumActionsTaken);
@@ -293,20 +293,24 @@ void PasswordFormManager::OnRequestDone(int handle,
     return;
   }
 
-  // Proceed to autofill (note that we provide the choices but don't
-  // actually prefill a value if the ACTION paths don't match).
-  bool wait_for_username = observed_form_.action.GetWithEmptyPath() !=
-                           preferred_match_->action.GetWithEmptyPath();
+  // Proceed to autofill.
+  // Note that we provide the choices but don't actually prefill a value if
+  // either: (1) we are in Incognito mode, or (2) the ACTION paths don't match.
+  bool wait_for_username =
+      profile_->IsOffTheRecord() ||
+      observed_form_.action.GetWithEmptyPath() !=
+          preferred_match_->action.GetWithEmptyPath();
   if (wait_for_username)
     manager_action_ = kManagerActionNone;
   else
-    manager_action_ = kManagerActionAutoFilled;
+    manager_action_ = kManagerActionAutofilled;
   password_manager_->Autofill(observed_form_, best_matches_,
                               preferred_match_, wait_for_username);
 }
 
 void PasswordFormManager::OnPasswordStoreRequestDone(
-    int handle, const std::vector<PasswordForm*>& result) {
+    CancelableRequestProvider::Handle handle,
+    const std::vector<PasswordForm*>& result) {
   DCHECK_EQ(state_, MATCHING_PHASE);
   DCHECK_EQ(pending_login_query_, handle);
 
@@ -316,6 +320,7 @@ void PasswordFormManager::OnPasswordStoreRequestDone(
   }
 
   OnRequestDone(handle, result);
+  pending_login_query_ = 0;
 }
 
 bool PasswordFormManager::IgnoreResult(const PasswordForm& form) const {
@@ -420,16 +425,6 @@ void PasswordFormManager::UpdateLogin() {
   }
 }
 
-void PasswordFormManager::CancelLoginsQuery() {
-  PasswordStore* password_store =
-      profile_->GetPasswordStore(Profile::EXPLICIT_ACCESS);
-  if (!password_store) {
-    // Can be NULL in unit tests.
-    return;
-  }
-  password_store->CancelLoginsQuery(pending_login_query_);
-}
-
 int PasswordFormManager::ScoreResult(const PasswordForm& candidate) const {
   DCHECK_EQ(state_, MATCHING_PHASE);
   // For scoring of candidate login data:
