@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,27 +6,32 @@
 
 #include <vector>
 
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/threading/platform_thread.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/metrics/user_metrics.h"
 #include "chrome/browser/password_manager/password_form_manager.h"
 #include "chrome/browser/password_manager/password_manager_delegate.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/common/notification_registrar.h"
+#include "chrome/common/autofill_messages.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/common/render_messages.h"
-#include "chrome/common/render_messages_params.h"
+#include "content/public/browser/user_metrics.h"
+#include "content/public/common/frame_navigate_params.h"
 #include "grit/generated_resources.h"
 
-using webkit_glue::PasswordForm;
-using webkit_glue::PasswordFormMap;
+using content::UserMetricsAction;
+using content::WebContents;
+using webkit::forms::PasswordForm;
+using webkit::forms::PasswordFormMap;
 
 // static
 void PasswordManager::RegisterUserPrefs(PrefService* prefs) {
-  prefs->RegisterBooleanPref(prefs::kPasswordManagerEnabled, true);
-  prefs->RegisterBooleanPref(prefs::kPasswordManagerAllowShowPasswords, true);
+  prefs->RegisterBooleanPref(prefs::kPasswordManagerEnabled,
+                             true,
+                             PrefService::SYNCABLE_PREF);
+  prefs->RegisterBooleanPref(prefs::kPasswordManagerAllowShowPasswords,
+                             true,
+                             PrefService::UNSYNCABLE_PREF);
 }
 
 // This routine is called when PasswordManagers are constructed.
@@ -46,13 +51,15 @@ static void ReportMetrics(bool password_manager_enabled) {
   ran_once = true;
 
   if (password_manager_enabled)
-    UserMetrics::RecordAction(UserMetricsAction("PasswordManager_Enabled"));
+    content::RecordAction(UserMetricsAction("PasswordManager_Enabled"));
   else
-    UserMetrics::RecordAction(UserMetricsAction("PasswordManager_Disabled"));
+    content::RecordAction(UserMetricsAction("PasswordManager_Disabled"));
 }
 
-PasswordManager::PasswordManager(PasswordManagerDelegate* delegate)
-    : login_managers_deleter_(&pending_login_managers_),
+PasswordManager::PasswordManager(WebContents* web_contents,
+                                 PasswordManagerDelegate* delegate)
+    : content::WebContentsObserver(web_contents),
+      login_managers_deleter_(&pending_login_managers_),
       delegate_(delegate),
       observer_(NULL) {
   DCHECK(delegate_);
@@ -152,9 +159,9 @@ void PasswordManager::DidStopLoading() {
   }
 }
 
-void PasswordManager::DidNavigateAnyFramePostCommit(
-      const NavigationController::LoadCommittedDetails& details,
-      const ViewHostMsg_FrameNavigate_Params& params) {
+void PasswordManager::DidNavigateAnyFrame(
+      const content::LoadCommittedDetails& details,
+      const content::FrameNavigateParams& params) {
   if (params.password_form.origin.is_valid())
     ProvisionallySavePassword(params.password_form);
 }
@@ -162,8 +169,9 @@ void PasswordManager::DidNavigateAnyFramePostCommit(
 bool PasswordManager::OnMessageReceived(const IPC::Message& message) {
   bool handled = true;
   IPC_BEGIN_MESSAGE_MAP(PasswordManager, message)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_PasswordFormsFound, OnPasswordFormsFound)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_PasswordFormsVisible,
+    IPC_MESSAGE_HANDLER(AutofillHostMsg_PasswordFormsFound,
+                        OnPasswordFormsFound)
+    IPC_MESSAGE_HANDLER(AutofillHostMsg_PasswordFormsVisible,
                         OnPasswordFormsVisible)
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
@@ -219,8 +227,8 @@ void PasswordManager::Autofill(
     case PasswordForm::SCHEME_HTML: {
       // Note the check above is required because the observer_ for a non-HTML
       // schemed password form may have been freed, so we need to distinguish.
-      webkit_glue::PasswordFormFillData fill_data;
-      webkit_glue::PasswordFormDomManager::InitFillData(form_for_autofill,
+      webkit::forms::PasswordFormFillData fill_data;
+      webkit::forms::PasswordFormDomManager::InitFillData(form_for_autofill,
                                                         best_matches,
                                                         preferred_match,
                                                         wait_for_username,
@@ -230,9 +238,8 @@ void PasswordManager::Autofill(
     }
     default:
       if (observer_) {
-        observer_->OnAutofillDataAvailable(
-            UTF16ToWideHack(preferred_match->username_value),
-            UTF16ToWideHack(preferred_match->password_value));
+        observer_->OnAutofillDataAvailable(preferred_match->username_value,
+                                           preferred_match->password_value);
       }
   }
 }
