@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,16 +7,19 @@
 #include "base/message_loop.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/password_manager/password_manager.h"
 #include "chrome/browser/password_manager/password_manager_delegate.h"
 #include "chrome/browser/password_manager/password_store.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/testing_profile.h"
-#include "testing/gtest/include/gtest/gtest.h"
+#include "chrome/test/base/chrome_render_view_host_test_harness.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/browser/tab_contents/test_tab_contents.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gmock/include/gmock/gmock.h"
+#include "testing/gtest/include/gtest/gtest.h"
 
-using webkit_glue::PasswordForm;
+using content::BrowserThread;
+using webkit::forms::PasswordForm;
 using testing::_;
 using testing::DoAll;
 using ::testing::Exactly;
@@ -26,7 +29,7 @@ using ::testing::Return;
 class MockPasswordManagerDelegate : public PasswordManagerDelegate {
  public:
   MOCK_METHOD1(FillPasswordForm, void(
-     const webkit_glue::PasswordFormFillData&));
+     const webkit::forms::PasswordFormFillData&));
   MOCK_METHOD1(AddSavePasswordInfoBar, void(PasswordFormManager*));
   MOCK_METHOD0(GetProfileForPasswordManager, Profile*());
   MOCK_METHOD0(DidLastPageLoadEncounterSSLErrors, bool());
@@ -36,11 +39,14 @@ class TestingProfileWithPasswordStore : public TestingProfile {
  public:
   explicit TestingProfileWithPasswordStore(PasswordStore* store)
       : store_(store) {}
+  virtual ~TestingProfileWithPasswordStore() {
+    store_->Shutdown();
+  }
   virtual PasswordStore* GetPasswordStore(ServiceAccessType access) {
     return store_;
   }
  private:
-  PasswordStore* store_;
+  scoped_refptr<PasswordStore> store_;
 };
 
 class MockPasswordStore : public PasswordStore {
@@ -60,9 +66,9 @@ class MockPasswordStore : public PasswordStore {
   MOCK_METHOD1(GetAutofillableLoginsImpl, void(GetLoginsRequest*));
   MOCK_METHOD1(GetBlacklistLoginsImpl, void(GetLoginsRequest*));
   MOCK_METHOD1(FillAutofillableLogins,
-      bool(std::vector<webkit_glue::PasswordForm*>*));
+      bool(std::vector<webkit::forms::PasswordForm*>*));
   MOCK_METHOD1(FillBlacklistLogins,
-      bool(std::vector<webkit_glue::PasswordForm*>*));
+      bool(std::vector<webkit::forms::PasswordForm*>*));
 };
 
 ACTION_P2(InvokeConsumer, handle, forms) {
@@ -73,18 +79,20 @@ ACTION_P(SaveToScopedPtr, scoped) {
   scoped->reset(arg0);
 }
 
-class PasswordManagerTest : public testing::Test {
+class PasswordManagerTest : public ChromeRenderViewHostTestHarness {
  public:
   PasswordManagerTest()
-      : ui_thread_(BrowserThread::UI, &message_loop_) {}
+      : ui_thread_(BrowserThread::UI, MessageLoopForUI::current()) {}
  protected:
 
   virtual void SetUp() {
     store_ = new MockPasswordStore();
-    profile_.reset(new TestingProfileWithPasswordStore(store_));
+    browser_context_.reset(new TestingProfileWithPasswordStore(store_));
+    ChromeRenderViewHostTestHarness::SetUp();
+
     EXPECT_CALL(delegate_, GetProfileForPasswordManager())
-        .WillRepeatedly(Return(profile_.get()));
-    manager_.reset(new PasswordManager(&delegate_));
+        .WillRepeatedly(Return(profile()));
+    manager_.reset(new PasswordManager(contents(), &delegate_));
     EXPECT_CALL(delegate_, DidLastPageLoadEncounterSSLErrors())
         .WillRepeatedly(Return(false));
   }
@@ -92,6 +100,7 @@ class PasswordManagerTest : public testing::Test {
   virtual void TearDown() {
     manager_.reset();
     store_ = NULL;
+    ChromeRenderViewHostTestHarness::TearDown();
   }
 
   PasswordForm MakeSimpleForm() {
@@ -110,10 +119,8 @@ class PasswordManagerTest : public testing::Test {
   PasswordManager* manager() { return manager_.get(); }
 
   // We create a UI thread to satisfy PasswordStore.
-  MessageLoopForUI message_loop_;
-  BrowserThread ui_thread_;
+  content::TestBrowserThread ui_thread_;
 
-  scoped_ptr<Profile> profile_;
   scoped_refptr<MockPasswordStore> store_;
   MockPasswordManagerDelegate delegate_;  // Owned by manager_.
   scoped_ptr<PasswordManager> manager_;
