@@ -1,44 +1,50 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/basictypes.h"
-#include "base/stl_util-inl.h"
-#include "base/string_util.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/scoped_temp_dir.h"
+#include "base/stl_util.h"
+#include "base/string_util.h"
+#include "base/synchronization/waitable_event.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
-#include "base/synchronization/waitable_event.h"
-#include "chrome/browser/password_manager/password_store_change.h"
-#include "chrome/browser/password_manager/password_store_default.h"
 #include "chrome/browser/password_manager/password_form_data.h"
+#include "chrome/browser/password_manager/password_store_change.h"
+#include "chrome/browser/password_manager/password_store_consumer.h"
+#include "chrome/browser/password_manager/password_store_default.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/webdata/web_data_service.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_observer_mock.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_source.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/signaling_task.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_source.h"
+#include "content/test/notification_observer_mock.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using base::WaitableEvent;
+using content::BrowserThread;
 using testing::_;
 using testing::DoAll;
 using testing::ElementsAreArray;
 using testing::Pointee;
 using testing::Property;
 using testing::WithArg;
-using webkit_glue::PasswordForm;
+using webkit::forms::PasswordForm;
 
 namespace {
 
 class MockPasswordStoreConsumer : public PasswordStoreConsumer {
  public:
   MOCK_METHOD2(OnPasswordStoreRequestDone,
-               void(int, const std::vector<webkit_glue::PasswordForm*>&));
+               void(CancelableRequestProvider::Handle,
+                    const std::vector<webkit::forms::PasswordForm*>&));
 };
 
 class MockWebDataServiceConsumer : public WebDataServiceConsumer {
@@ -60,9 +66,9 @@ class DBThreadObserverHelper :
     BrowserThread::PostTask(
         BrowserThread::DB,
         FROM_HERE,
-        NewRunnableMethod(this,
-                          &DBThreadObserverHelper::AddObserverTask,
-                          make_scoped_refptr(password_store)));
+        base::Bind(&DBThreadObserverHelper::AddObserverTask,
+                   this,
+                   make_scoped_refptr(password_store)));
     done_event_.Wait();
   }
 
@@ -71,7 +77,7 @@ class DBThreadObserverHelper :
     registrar_.RemoveAll();
   }
 
-  NotificationObserverMock& observer() {
+  content::NotificationObserverMock& observer() {
     return observer_;
   }
 
@@ -81,14 +87,14 @@ class DBThreadObserverHelper :
   void AddObserverTask(PasswordStore* password_store) {
     DCHECK(BrowserThread::CurrentlyOn(BrowserThread::DB));
     registrar_.Add(&observer_,
-                   NotificationType::LOGINS_CHANGED,
-                   Source<PasswordStore>(password_store));
+                   chrome::NOTIFICATION_LOGINS_CHANGED,
+                   content::Source<PasswordStore>(password_store));
     done_event_.Signal();
   }
 
   WaitableEvent done_event_;
-  NotificationRegistrar registrar_;
-  NotificationObserverMock observer_;
+  content::NotificationRegistrar registrar_;
+  content::NotificationObserverMock observer_;
 };
 
 }  // anonymous namespace
@@ -118,14 +124,15 @@ class PasswordStoreDefaultTest : public testing::Test {
 
   virtual void TearDown() {
     wds_->Shutdown();
-    MessageLoop::current()->PostTask(FROM_HERE, new MessageLoop::QuitTask);
+    MessageLoop::current()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
     MessageLoop::current()->Run();
     db_thread_.Stop();
   }
 
   MessageLoopForUI message_loop_;
-  BrowserThread ui_thread_;
-  BrowserThread db_thread_;  // PasswordStore, WDS schedule work on this thread.
+  content::TestBrowserThread ui_thread_;
+  // PasswordStore, WDS schedule work on this thread.
+  content::TestBrowserThread db_thread_;
 
   scoped_ptr<LoginDatabase> login_db_;
   scoped_ptr<TestingProfile> profile_;
@@ -148,9 +155,10 @@ MATCHER(EmptyWDResult, "") {
 }
 
 TEST_F(PasswordStoreDefaultTest, NonASCIIData) {
-  // Prentend that the migration has already taken place.
+  // Pretend that the migration has already taken place.
   profile_->GetPrefs()->RegisterBooleanPref(prefs::kLoginDatabaseMigrated,
-                                            true);
+                                            true,
+                                            PrefService::UNSYNCABLE_PREF);
 
   // Initializing the PasswordStore shouldn't trigger a migration.
   scoped_refptr<PasswordStoreDefault> store(
@@ -184,7 +192,7 @@ TEST_F(PasswordStoreDefaultTest, NonASCIIData) {
   // yet another task to notify us that it's safe to carry on with the test.
   WaitableEvent done(false, false);
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   MockPasswordStoreConsumer consumer;
@@ -279,11 +287,11 @@ TEST_F(PasswordStoreDefaultTest, Migration) {
   // task to notify us that it's safe to carry on with the test.
   WaitableEvent done(false, false);
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Initializing the PasswordStore should trigger a migration.
-  scoped_refptr<PasswordStoreDefault> store(
+  scoped_refptr<PasswordStore> store(
       new PasswordStoreDefault(login_db_.release(),
           profile_.get(), wds_.get()));
   store->Init();
@@ -295,7 +303,7 @@ TEST_F(PasswordStoreDefaultTest, Migration) {
   // Again, the WDS schedules tasks to run on the DB thread, so schedule a task
   // to signal us when it is safe to continue.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Let the WDS callbacks proceed so the logins can be migrated.
@@ -340,7 +348,7 @@ TEST_F(PasswordStoreDefaultTest, Migration) {
 
   // Wait for the WDS methods to execute on the DB thread.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Handle the callback from the WDS.
@@ -354,7 +362,7 @@ TEST_F(PasswordStoreDefaultTest, Migration) {
 
   // Wait for the WDS methods to execute on the DB thread.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Handle the callback from the WDS.
@@ -362,6 +370,8 @@ TEST_F(PasswordStoreDefaultTest, Migration) {
 
   STLDeleteElements(&expected_autofillable);
   STLDeleteElements(&expected_blacklisted);
+
+  store->Shutdown();
 }
 
 TEST_F(PasswordStoreDefaultTest, MigrationAlreadyDone) {
@@ -390,15 +400,16 @@ TEST_F(PasswordStoreDefaultTest, MigrationAlreadyDone) {
   // task to notify us that it's safe to carry on with the test.
   WaitableEvent done(false, false);
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
-  // Prentend that the migration has already taken place.
+  // Pretend that the migration has already taken place.
   profile_->GetPrefs()->RegisterBooleanPref(prefs::kLoginDatabaseMigrated,
-                                            true);
+                                            true,
+                                            PrefService::UNSYNCABLE_PREF);
 
   // Initializing the PasswordStore shouldn't trigger a migration.
-  scoped_refptr<PasswordStoreDefault> store(
+  scoped_refptr<PasswordStore> store(
       new PasswordStoreDefault(login_db_.release(), profile_.get(),
                                wds_.get()));
   store->Init();
@@ -419,15 +430,18 @@ TEST_F(PasswordStoreDefaultTest, MigrationAlreadyDone) {
   MessageLoop::current()->Run();
 
   STLDeleteElements(&unexpected_autofillable);
+
+  store->Shutdown();
 }
 
 TEST_F(PasswordStoreDefaultTest, Notifications) {
-  // Prentend that the migration has already taken place.
+  // Pretend that the migration has already taken place.
   profile_->GetPrefs()->RegisterBooleanPref(prefs::kLoginDatabaseMigrated,
-                                            true);
+                                            true,
+                                            PrefService::UNSYNCABLE_PREF);
 
   // Initializing the PasswordStore shouldn't trigger a migration.
-  scoped_refptr<PasswordStoreDefault> store(
+  scoped_refptr<PasswordStore> store(
       new PasswordStoreDefault(login_db_.release(), profile_.get(),
                                wds_.get()));
   store->Init();
@@ -453,11 +467,11 @@ TEST_F(PasswordStoreDefaultTest, Notifications) {
   };
 
   EXPECT_CALL(helper->observer(),
-              Observe(NotificationType(NotificationType::LOGINS_CHANGED),
-                      Source<PasswordStore>(store),
-                      Property(&Details<const PasswordStoreChangeList>::ptr,
-                               Pointee(ElementsAreArray(
-                                   expected_add_changes)))));
+      Observe(int(chrome::NOTIFICATION_LOGINS_CHANGED),
+          content::Source<PasswordStore>(store),
+          Property(&content::Details<const PasswordStoreChangeList>::ptr,
+                   Pointee(ElementsAreArray(
+                       expected_add_changes)))));
 
   // Adding a login should trigger a notification.
   store->AddLogin(*form);
@@ -466,7 +480,7 @@ TEST_F(PasswordStoreDefaultTest, Notifications) {
   // yet another task to notify us that it's safe to carry on with the test.
   WaitableEvent done(false, false);
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Change the password.
@@ -477,18 +491,18 @@ TEST_F(PasswordStoreDefaultTest, Notifications) {
   };
 
   EXPECT_CALL(helper->observer(),
-              Observe(NotificationType(NotificationType::LOGINS_CHANGED),
-                      Source<PasswordStore>(store),
-                      Property(&Details<const PasswordStoreChangeList>::ptr,
-                               Pointee(ElementsAreArray(
-                                   expected_update_changes)))));
+      Observe(int(chrome::NOTIFICATION_LOGINS_CHANGED),
+              content::Source<PasswordStore>(store),
+              Property(&content::Details<const PasswordStoreChangeList>::ptr,
+                       Pointee(ElementsAreArray(
+                           expected_update_changes)))));
 
   // Updating the login with the new password should trigger a notification.
   store->UpdateLogin(*form);
 
   // Wait for PasswordStore to send the notification.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   const PasswordStoreChange expected_delete_changes[] = {
@@ -496,17 +510,19 @@ TEST_F(PasswordStoreDefaultTest, Notifications) {
   };
 
   EXPECT_CALL(helper->observer(),
-              Observe(NotificationType(NotificationType::LOGINS_CHANGED),
-                      Source<PasswordStore>(store),
-                      Property(&Details<const PasswordStoreChangeList>::ptr,
-                               Pointee(ElementsAreArray(
-                                   expected_delete_changes)))));
+      Observe(int(chrome::NOTIFICATION_LOGINS_CHANGED),
+              content::Source<PasswordStore>(store),
+              Property(&content::Details<const PasswordStoreChangeList>::ptr,
+                       Pointee(ElementsAreArray(
+                           expected_delete_changes)))));
 
   // Deleting the login should trigger a notification.
   store->RemoveLogin(*form);
 
   // Wait for PasswordStore to send the notification.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
+
+  store->Shutdown();
 }
