@@ -1,41 +1,46 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <windows.h>
-#include <wincrypt.h>
 #include <string>
 #include <vector>
+#include <wincrypt.h>
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/scoped_ptr.h"
 #include "base/scoped_temp_dir.h"
-#include "base/stl_util-inl.h"
-#include "base/time.h"
+#include "base/stl_util.h"
 #include "base/synchronization/waitable_event.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/password_manager/password_form_data.h"
-#include "chrome/browser/password_manager/password_store_win.h"
+#include "base/time.h"
 #include "chrome/browser/password_manager/ie7_password.h"
+#include "chrome/browser/password_manager/password_form_data.h"
+#include "chrome/browser/password_manager/password_store_consumer.h"
+#include "chrome/browser/password_manager/password_store_win.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/webdata/web_data_service.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/signaling_task.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using base::WaitableEvent;
+using content::BrowserThread;
 using testing::_;
 using testing::DoAll;
 using testing::WithArg;
-using webkit_glue::PasswordForm;
+using webkit::forms::PasswordForm;
 
 namespace {
 
 class MockPasswordStoreConsumer : public PasswordStoreConsumer {
  public:
   MOCK_METHOD2(OnPasswordStoreRequestDone,
-               void(int, const std::vector<webkit_glue::PasswordForm*>&));
+               void(CancelableRequestProvider::Handle,
+                    const std::vector<webkit::forms::PasswordForm*>&));
 };
 
 class MockWebDataServiceConsumer : public WebDataServiceConsumer {
@@ -115,14 +120,15 @@ class PasswordStoreWinTest : public testing::Test {
   virtual void TearDown() {
     if (wds_.get())
       wds_->Shutdown();
-    MessageLoop::current()->PostTask(FROM_HERE, new MessageLoop::QuitTask);
+    MessageLoop::current()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
     MessageLoop::current()->Run();
     db_thread_.Stop();
   }
 
   MessageLoopForUI message_loop_;
-  BrowserThread ui_thread_;
-  BrowserThread db_thread_;  // PasswordStore, WDS schedule work on this thread.
+  content::TestBrowserThread ui_thread_;
+  // PasswordStore, WDS schedule work on this thread.
+  content::TestBrowserThread db_thread_;
 
   scoped_ptr<LoginDatabase> login_db_;
   scoped_ptr<TestingProfile> profile_;
@@ -144,7 +150,8 @@ MATCHER(EmptyWDResult, "") {
       arg)->GetValue().empty();
 }
 
-TEST_F(PasswordStoreWinTest, ConvertIE7Login) {
+// Hangs flakily, http://crbug.com/71385.
+TEST_F(PasswordStoreWinTest, DISABLED_ConvertIE7Login) {
   IE7PasswordInfo password_info;
   ASSERT_TRUE(CreateIE7PasswordInfo(L"http://example.com/origin",
                                     base::Time::FromDoubleT(1),
@@ -160,15 +167,16 @@ TEST_F(PasswordStoreWinTest, ConvertIE7Login) {
   // task to notify us that it's safe to carry on with the test.
   WaitableEvent done(false, false);
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Prentend that the migration has already taken place.
   profile_->GetPrefs()->RegisterBooleanPref(prefs::kLoginDatabaseMigrated,
-                                            true);
+                                            true,
+                                            PrefService::UNSYNCABLE_PREF);
 
   // Initializing the PasswordStore shouldn't trigger a migration.
-  scoped_refptr<PasswordStoreWin> store(
+  scoped_refptr<PasswordStore> store(
       new PasswordStoreWin(login_db_.release(), profile_.get(), wds_.get()));
   EXPECT_TRUE(store->Init());
 
@@ -217,15 +225,19 @@ TEST_F(PasswordStoreWinTest, ConvertIE7Login) {
   MessageLoop::current()->Run();
 
   STLDeleteElements(&forms);
+
+  store->Shutdown();
 }
 
-TEST_F(PasswordStoreWinTest, OutstandingWDSQueries) {
+// Crashy.  http://crbug.com/86558
+TEST_F(PasswordStoreWinTest, DISABLED_OutstandingWDSQueries) {
   // Prentend that the migration has already taken place.
   profile_->GetPrefs()->RegisterBooleanPref(prefs::kLoginDatabaseMigrated,
-                                            true);
+                                            true,
+                                            PrefService::UNSYNCABLE_PREF);
 
   // Initializing the PasswordStore shouldn't trigger a migration.
-  scoped_refptr<PasswordStoreWin> store(
+  scoped_refptr<PasswordStore> store(
       new PasswordStoreWin(login_db_.release(), profile_.get(), wds_.get()));
   EXPECT_TRUE(store->Init());
 
@@ -247,6 +259,7 @@ TEST_F(PasswordStoreWinTest, OutstandingWDSQueries) {
   store->GetLogins(*form, &consumer);
 
   // Release the PSW and the WDS before the query can return.
+  store->Shutdown();
   store = NULL;
   wds_->Shutdown();
   wds_ = NULL;
@@ -266,15 +279,16 @@ TEST_F(PasswordStoreWinTest, DISABLED_MultipleWDSQueriesOnDifferentThreads) {
   // task to notify us that it's safe to carry on with the test.
   WaitableEvent done(false, false);
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Prentend that the migration has already taken place.
   profile_->GetPrefs()->RegisterBooleanPref(prefs::kLoginDatabaseMigrated,
-                                            true);
+                                            true,
+                                            PrefService::UNSYNCABLE_PREF);
 
   // Initializing the PasswordStore shouldn't trigger a migration.
-  scoped_refptr<PasswordStoreWin> store(
+  scoped_refptr<PasswordStore> store(
       new PasswordStoreWin(login_db_.release(), profile_.get(), wds_.get()));
   EXPECT_TRUE(store->Init());
 
@@ -335,6 +349,8 @@ TEST_F(PasswordStoreWinTest, DISABLED_MultipleWDSQueriesOnDifferentThreads) {
   MessageLoop::current()->Run();
 
   STLDeleteElements(&forms);
+
+  store->Shutdown();
 }
 
 TEST_F(PasswordStoreWinTest, Migration) {
@@ -419,11 +435,11 @@ TEST_F(PasswordStoreWinTest, Migration) {
   // task to notify us that it's safe to carry on with the test.
   WaitableEvent done(false, false);
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Initializing the PasswordStore should trigger a migration.
-  scoped_refptr<PasswordStoreWin> store(
+  scoped_refptr<PasswordStore> store(
       new PasswordStoreWin(login_db_.release(), profile_.get(), wds_.get()));
   store->Init();
 
@@ -434,7 +450,7 @@ TEST_F(PasswordStoreWinTest, Migration) {
   // Again, the WDS schedules tasks to run on the DB thread, so schedule a task
   // to signal us when it is safe to continue.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Let the WDS callbacks proceed so the logins can be migrated.
@@ -479,7 +495,7 @@ TEST_F(PasswordStoreWinTest, Migration) {
 
   // Wait for the WDS methods to execute on the DB thread.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Handle the callback from the WDS.
@@ -493,7 +509,7 @@ TEST_F(PasswordStoreWinTest, Migration) {
 
   // Wait for the WDS methods to execute on the DB thread.
   BrowserThread::PostTask(BrowserThread::DB, FROM_HERE,
-      new SignalingTask(&done));
+      base::Bind(&WaitableEvent::Signal, base::Unretained(&done)));
   done.Wait();
 
   // Handle the callback from the WDS.
@@ -502,3 +518,86 @@ TEST_F(PasswordStoreWinTest, Migration) {
   STLDeleteElements(&expected_autofillable);
   STLDeleteElements(&expected_blacklisted);
 }
+
+TEST_F(PasswordStoreWinTest, EmptyLogins) {
+  scoped_refptr<PasswordStore> store(
+      new PasswordStoreWin(login_db_.release(), profile_.get(), wds_.get()));
+  store->Init();
+
+  PasswordFormData form_data = {
+    PasswordForm::SCHEME_HTML,
+    "http://example.com/",
+    "http://example.com/origin",
+    "http://example.com/action",
+    L"submit_element",
+    L"username_element",
+    L"password_element",
+    L"",
+    L"",
+    true, false, 1,
+  };
+  scoped_ptr<PasswordForm> form(CreatePasswordFormFromData(form_data));
+
+  MockPasswordStoreConsumer consumer;
+
+  // Make sure we quit the MessageLoop even if the test fails.
+  ON_CALL(consumer, OnPasswordStoreRequestDone(_, _))
+      .WillByDefault(QuitUIMessageLoop());
+
+  VectorOfForms expect_none;
+  // expect that we get no results;
+  EXPECT_CALL(consumer, OnPasswordStoreRequestDone(
+      _, ContainsAllPasswordForms(expect_none)))
+      .WillOnce(DoAll(WithArg<1>(STLDeleteElements0()), QuitUIMessageLoop()));
+
+  store->GetLogins(*form, &consumer);
+  MessageLoop::current()->Run();
+
+  store->Shutdown();
+}
+
+TEST_F(PasswordStoreWinTest, EmptyBlacklistLogins) {
+  scoped_refptr<PasswordStore> store(
+      new PasswordStoreWin(login_db_.release(), profile_.get(), wds_.get()));
+  store->Init();
+
+  MockPasswordStoreConsumer consumer;
+
+  // Make sure we quit the MessageLoop even if the test fails.
+  ON_CALL(consumer, OnPasswordStoreRequestDone(_, _))
+      .WillByDefault(QuitUIMessageLoop());
+
+  VectorOfForms expect_none;
+  // expect that we get no results;
+  EXPECT_CALL(consumer, OnPasswordStoreRequestDone(
+      _, ContainsAllPasswordForms(expect_none)))
+      .WillOnce(DoAll(WithArg<1>(STLDeleteElements0()), QuitUIMessageLoop()));
+
+  store->GetBlacklistLogins(&consumer);
+  MessageLoop::current()->Run();
+
+  store->Shutdown();
+}
+
+TEST_F(PasswordStoreWinTest, EmptyAutofillableLogins) {
+  scoped_refptr<PasswordStore> store(
+      new PasswordStoreWin(login_db_.release(), profile_.get(), wds_.get()));
+  store->Init();
+
+  MockPasswordStoreConsumer consumer;
+
+  // Make sure we quit the MessageLoop even if the test fails.
+  ON_CALL(consumer, OnPasswordStoreRequestDone(_, _))
+      .WillByDefault(QuitUIMessageLoop());
+
+  VectorOfForms expect_none;
+  // expect that we get no results;
+  EXPECT_CALL(consumer, OnPasswordStoreRequestDone(
+      _, ContainsAllPasswordForms(expect_none)))
+      .WillOnce(DoAll(WithArg<1>(STLDeleteElements0()), QuitUIMessageLoop()));
+
+  store->GetAutofillableLogins(&consumer);
+  MessageLoop::current()->Run();
+
+  store->Shutdown();
+}
