@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,10 @@
 #define CHROME_BROWSER_PLATFORM_UTIL_H_
 #pragma once
 
+#include <string>
+
 #include "base/string16.h"
-#include "gfx/native_widget_types.h"
+#include "ui/gfx/native_widget_types.h"
 
 class FilePath;
 class GURL;
@@ -15,9 +17,11 @@ class GURL;
 namespace platform_util {
 
 // Show the given file in a file manager. If possible, select the file.
+// Must be called from the UI thread.
 void ShowItemInFolder(const FilePath& full_path);
 
 // Open the given file in the desktop's default manner.
+// Must be called from the UI thread.
 void OpenItem(const FilePath& full_path);
 
 // Open the given external protocol URL in the desktop's default manner.
@@ -41,25 +45,6 @@ void ActivateWindow(gfx::NativeWindow window);
 // whether the view has the visible attribute set.
 bool IsVisible(gfx::NativeView view);
 
-// Pops up an error box with an OK button. If |parent| is non-null, the box
-// will be modal on it. (On Mac, it is always app-modal.) Generally speaking,
-// this function should not be used for much. Infobars are preferred.
-void SimpleErrorBox(gfx::NativeWindow parent,
-                    const string16& title,
-                    const string16& message);
-
-// Pops up a dialog box with two buttons (Yes/No), with the default button of
-// Yes. If |parent| is non-null, the box will be modal on it. (On Mac, it is
-// always app-modal.) Returns true if the Yes button was chosen.
-bool SimpleYesNoBox(gfx::NativeWindow parent,
-                    const string16& title,
-                    const string16& message);
-
-// Return a human readable modifier for the version string.  For a
-// branded Chrome (not Chromium), this modifier is the channel (dev,
-// beta, but "" for stable).
-std::string GetVersionStringModifier();
-
-}
+}  // platform_util
 
 #endif  // CHROME_BROWSER_PLATFORM_UTIL_H_
