@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,11 @@
 
 #include <Carbon/Carbon.h>
 #import <Cocoa/Cocoa.h>
-#import <CoreServices/CoreServices.h>
+#include <CoreServices/CoreServices.h>
 
 #include "base/file_path.h"
 #include "base/logging.h"
+#include "base/mac/mac_logging.h"
 #include "base/mac/scoped_aedesc.h"
 #include "base/sys_string_conversions.h"
 #include "googleurl/src/gurl.h"
@@ -20,7 +21,7 @@
 namespace platform_util {
 
 void ShowItemInFolder(const FilePath& full_path) {
-  DCHECK_EQ([NSThread currentThread], [NSThread mainThread]);
+  DCHECK([NSThread isMainThread]);
   NSString* path_string = base::SysUTF8ToNSString(full_path.value());
   if (!path_string || ![[NSWorkspace sharedWorkspace] selectFile:path_string
                                         inFileViewerRootedAtPath:nil])
@@ -35,22 +36,20 @@ void ShowItemInFolder(const FilePath& full_path) {
 // Instead, an AppleEvent is constructed to tell the Finder to open the
 // document.
 void OpenItem(const FilePath& full_path) {
-  DCHECK_EQ([NSThread currentThread], [NSThread mainThread]);
+  DCHECK([NSThread isMainThread]);
   NSString* path_string = base::SysUTF8ToNSString(full_path.value());
   if (!path_string)
     return;
 
-  OSErr status;
-
   // Create the target of this AppleEvent, the Finder.
   base::mac::ScopedAEDesc<AEAddressDesc> address;
   const OSType finderCreatorCode = 'MACS';
-  status = AECreateDesc(typeApplSignature,  // type
-                        &finderCreatorCode,  // data
-                        sizeof(finderCreatorCode),  // dataSize
-                        address.OutPointer());  // result
+  OSErr status = AECreateDesc(typeApplSignature,  // type
+                              &finderCreatorCode,  // data
+                              sizeof(finderCreatorCode),  // dataSize
+                              address.OutPointer());  // result
   if (status != noErr) {
-    LOG(WARNING) << "Could not create OpenItem() AE target";
+    OSSTATUS_LOG(WARNING, status) << "Could not create OpenItem() AE target";
     return;
   }
 
@@ -63,7 +62,7 @@ void OpenItem(const FilePath& full_path) {
                               kAnyTransactionID,  // transactionID
                               theEvent.OutPointer());  // result
   if (status != noErr) {
-    LOG(WARNING) << "Could not create OpenItem() AE event";
+    OSSTATUS_LOG(WARNING, status) << "Could not create OpenItem() AE event";
     return;
   }
 
@@ -74,7 +73,7 @@ void OpenItem(const FilePath& full_path) {
                         false,  // isRecord
                         fileList.OutPointer());  // resultList
   if (status != noErr) {
-    LOG(WARNING) << "Could not create OpenItem() AE file list";
+    OSSTATUS_LOG(WARNING, status) << "Could not create OpenItem() AE file list";
     return;
   }
 
@@ -89,7 +88,8 @@ void OpenItem(const FilePath& full_path) {
                       &pathRef,  // dataPtr
                       sizeof(pathRef));  // dataSize
     if (status != noErr) {
-      LOG(WARNING) << "Could not add file path to AE list in OpenItem()";
+      OSSTATUS_LOG(WARNING, status)
+          << "Could not add file path to AE list in OpenItem()";
       return;
     }
   } else {
@@ -102,7 +102,8 @@ void OpenItem(const FilePath& full_path) {
                           keyDirectObject,  // theAEKeyword
                           fileList);  // theAEDesc
   if (status != noErr) {
-    LOG(WARNING) << "Could not put the AE file list the path in OpenItem()";
+    OSSTATUS_LOG(WARNING, status)
+        << "Could not put the AE file list the path in OpenItem()";
     return;
   }
 
@@ -116,12 +117,13 @@ void OpenItem(const FilePath& full_path) {
                   NULL, // idleProc
                   NULL);  // filterProc
   if (status != noErr) {
-    LOG(WARNING) << "Could not send AE to Finder in OpenItem()";
+    OSSTATUS_LOG(WARNING, status)
+        << "Could not send AE to Finder in OpenItem()";
   }
 }
 
 void OpenExternal(const GURL& url) {
-  DCHECK_EQ([NSThread currentThread], [NSThread mainThread]);
+  DCHECK([NSThread isMainThread]);
   NSString* url_string = base::SysUTF8ToNSString(url.spec());
   NSURL* ns_url = [NSURL URLWithString:url_string];
   if (!ns_url || ![[NSWorkspace sharedWorkspace] openURL:ns_url])
@@ -152,58 +154,4 @@ bool IsVisible(gfx::NativeView view) {
           [[view window] isVisible]);
 }
 
-void SimpleErrorBox(gfx::NativeWindow parent,
-                    const string16& title,
-                    const string16& message) {
-  // Ignore the title; it's the window title on other platforms and ignorable.
-  NSAlert* alert = [[[NSAlert alloc] init] autorelease];
-  [alert addButtonWithTitle:l10n_util::GetNSString(IDS_OK)];
-  [alert setMessageText:base::SysUTF16ToNSString(message)];
-  [alert setAlertStyle:NSWarningAlertStyle];
-  [alert runModal];
-}
-
-bool SimpleYesNoBox(gfx::NativeWindow parent,
-                    const string16& title,
-                    const string16& message) {
-  // Ignore the title; it's the window title on other platforms and ignorable.
-  NSAlert* alert = [[[NSAlert alloc] init] autorelease];
-  [alert setMessageText:base::SysUTF16ToNSString(message)];
-  [alert setAlertStyle:NSWarningAlertStyle];
-
-  [alert addButtonWithTitle:
-      l10n_util::GetNSString(IDS_CONFIRM_MESSAGEBOX_YES_BUTTON_LABEL)];
-  [alert addButtonWithTitle:
-      l10n_util::GetNSString(IDS_CONFIRM_MESSAGEBOX_NO_BUTTON_LABEL)];
-
-  NSInteger result = [alert runModal];
-  return result == NSAlertFirstButtonReturn;
-}
-
-std::string GetVersionStringModifier() {
-#if defined(GOOGLE_CHROME_BUILD)
-  // Use the main application bundle and not the framework bundle. Keystone
-  // keys don't live in the framework.
-  NSBundle* bundle = [NSBundle mainBundle];
-  NSString* channel = [bundle objectForInfoDictionaryKey:@"KSChannelID"];
-
-  // Only ever return "", "unknown", "beta" or "dev" in a branded build.
-  if (![bundle objectForInfoDictionaryKey:@"KSProductID"]) {
-    // This build is not Keystone-enabled, it can't have a channel.
-    channel = @"unknown";
-  } else if (!channel) {
-    // For the stable channel, KSChannelID is not set.
-    channel = @"";
-  } else if ([channel isEqual:@"beta"] || [channel isEqual:@"dev"]) {
-    // do nothing.
-  } else {
-    channel = @"unknown";
-  }
-
-  return base::SysNSStringToUTF8(channel);
-#else
-  return std::string();
-#endif
-}
-
 }  // namespace platform_util
