@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,26 +9,27 @@
 #include <shellapi.h>
 #include <shlobj.h>
 
-#include "app/win/scoped_co_mem.h"
-#include "app/win/shell.h"
-#include "app/win/win_util.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
-#include "base/path_service.h"
 #include "base/logging.h"
-#include "base/scoped_comptr_win.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/registry.h"
-#include "chrome/installer/util/google_update_settings.h"
-#include "chrome/installer/util/google_update_constants.h"
-#include "chrome/installer/util/install_util.h"
-#include "gfx/native_widget_types.h"
+#include "base/win/scoped_co_mem.h"
+#include "base/win/scoped_comptr.h"
+#include "content/public/browser/browser_thread.h"
 #include "googleurl/src/gurl.h"
+#include "ui/base/win/shell.h"
+#include "ui/gfx/native_widget_types.h"
 
-namespace platform_util {
+using content::BrowserThread;
 
-void ShowItemInFolder(const FilePath& full_path) {
+namespace {
+
+void ShowItemInFolderOnFileThread(const FilePath& full_path) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::FILE));
   FilePath dir = full_path.DirName();
   // ParseDisplayName will fail if the directory is "C:", it must be "C:\\".
   if (dir.value() == L"" || !file_util::EnsureEndsWithSeparator(&dir))
@@ -52,7 +53,7 @@ void ShowItemInFolder(const FilePath& full_path) {
     // the process.
     HMODULE shell32_base = GetModuleHandle(L"shell32.dll");
     if (!shell32_base) {
-      NOTREACHED();
+      NOTREACHED() << " " << __FUNCTION__ << "(): Can't open shell32.dll";
       return;
     }
     open_folder_and_select_itemsPtr =
@@ -60,23 +61,23 @@ void ShowItemInFolder(const FilePath& full_path) {
             (GetProcAddress(shell32_base, "SHOpenFolderAndSelectItems"));
   }
   if (!open_folder_and_select_itemsPtr) {
-    ShellExecute(NULL, _T("open"), dir.value().c_str(), NULL, NULL, SW_SHOW);
+    ShellExecute(NULL, L"open", dir.value().c_str(), NULL, NULL, SW_SHOW);
     return;
   }
 
-  ScopedComPtr<IShellFolder> desktop;
+  base::win::ScopedComPtr<IShellFolder> desktop;
   HRESULT hr = SHGetDesktopFolder(desktop.Receive());
   if (FAILED(hr))
     return;
 
-  app::win::ScopedCoMem<ITEMIDLIST> dir_item;
+  base::win::ScopedCoMem<ITEMIDLIST> dir_item;
   hr = desktop->ParseDisplayName(NULL, NULL,
                                  const_cast<wchar_t *>(dir.value().c_str()),
                                  NULL, &dir_item, NULL);
   if (FAILED(hr))
     return;
 
-  app::win::ScopedCoMem<ITEMIDLIST> file_item;
+  base::win::ScopedCoMem<ITEMIDLIST> file_item;
   hr = desktop->ParseDisplayName(NULL, NULL,
       const_cast<wchar_t *>(full_path.value().c_str()),
       NULL, &file_item, NULL);
@@ -86,12 +87,47 @@ void ShowItemInFolder(const FilePath& full_path) {
   const ITEMIDLIST* highlight[] = {
     {file_item},
   };
-  (*open_folder_and_select_itemsPtr)(dir_item, arraysize(highlight),
-                                     highlight, NULL);
+
+  hr = (*open_folder_and_select_itemsPtr)(dir_item, arraysize(highlight),
+                                          highlight, NULL);
+
+  if (FAILED(hr)) {
+    // On some systems, the above call mysteriously fails with "file not
+    // found" even though the file is there.  In these cases, ShellExecute()
+    // seems to work as a fallback (although it won't select the file).
+    if (hr == ERROR_FILE_NOT_FOUND) {
+      ShellExecute(NULL, L"open", dir.value().c_str(), NULL, NULL, SW_SHOW);
+    } else {
+      LPTSTR message = NULL;
+      DWORD message_length = FormatMessage(
+          FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
+          0, hr, 0, reinterpret_cast<LPTSTR>(&message), 0, NULL);
+      LOG(WARNING) << " " << __FUNCTION__
+                   << "(): Can't open full_path = \""
+                   << full_path.value() << "\""
+                   << " hr = " << hr
+                   << " " << reinterpret_cast<LPTSTR>(&message);
+      if (message)
+        LocalFree(message);
+    }
+  }
+}
+
+}  // namespace
+
+namespace platform_util {
+
+void ShowItemInFolder(const FilePath& full_path) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  BrowserThread::PostTask(BrowserThread::FILE, FROM_HERE,
+      base::Bind(&ShowItemInFolderOnFileThread, full_path));
 }
 
 void OpenItem(const FilePath& full_path) {
-  app::win::OpenItemViaShell(full_path);
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  BrowserThread::PostTask(
+      BrowserThread::FILE, FROM_HERE,
+      base::Bind(base::IgnoreResult(&ui::win::OpenItemViaShell), full_path));
 }
 
 void OpenExternal(const GURL& url) {
@@ -138,6 +174,7 @@ void OpenExternal(const GURL& url) {
   }
 }
 
+#if !defined(USE_AURA)
 gfx::NativeWindow GetTopLevel(gfx::NativeView view) {
   return ::GetAncestor(view, GA_ROOT);
 }
@@ -158,34 +195,6 @@ bool IsVisible(gfx::NativeView view) {
   // MSVC complains if we don't include != 0.
   return ::IsWindowVisible(view) != 0;
 }
-
-void SimpleErrorBox(gfx::NativeWindow parent,
-                    const string16& title,
-                    const string16& message) {
-  app::win::MessageBox(parent, message, title, MB_OK | MB_SETFOREGROUND);
-}
-
-bool SimpleYesNoBox(gfx::NativeWindow parent,
-                    const string16& title,
-                    const string16& message) {
-  return app::win::MessageBox(parent, message.c_str(), title.c_str(),
-      MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) == IDYES;
-}
-
-std::string GetVersionStringModifier() {
-#if defined(GOOGLE_CHROME_BUILD)
-  FilePath module;
-  string16 channel;
-  if (PathService::Get(base::FILE_MODULE, &module)) {
-    bool is_system_install =
-        !InstallUtil::IsPerUserInstall(module.value().c_str());
-
-    GoogleUpdateSettings::GetChromeChannel(is_system_install, &channel);
-  }
-  return UTF16ToASCII(channel);
-#else
-  return std::string();
 #endif
-}
 
 }  // namespace platform_util
