@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 #define CHROME_BROWSER_PREFERENCES_MOCK_MAC_H_
 #pragma once
 
-#include "base/scoped_cftyperef.h"
+#include "base/mac/scoped_cftyperef.h"
 #include "chrome/browser/preferences_mac.h"
 
 // Mock preferences wrapper for testing code that interacts with CFPreferences.
@@ -15,17 +15,20 @@ class MockPreferences : public MacPreferences {
   MockPreferences();
   virtual ~MockPreferences();
 
-  virtual CFPropertyListRef CopyAppValue(CFStringRef key,
-                                         CFStringRef applicationID);
+  virtual Boolean AppSynchronize(CFStringRef applicationID) OVERRIDE;
 
-  virtual Boolean AppValueIsForced(CFStringRef key, CFStringRef applicationID);
+  virtual CFPropertyListRef CopyAppValue(CFStringRef key,
+                                         CFStringRef applicationID) OVERRIDE;
+
+  virtual Boolean AppValueIsForced(CFStringRef key,
+                                   CFStringRef applicationID) OVERRIDE;
 
   // Adds a preference item with the given info to the test set.
   void AddTestItem(CFStringRef key, CFPropertyListRef value, bool is_forced);
 
  private:
-  scoped_cftyperef<CFMutableDictionaryRef> values_;
-  scoped_cftyperef<CFMutableSetRef> forced_;
+  base::mac::ScopedCFTypeRef<CFMutableDictionaryRef> values_;
+  base::mac::ScopedCFTypeRef<CFMutableSetRef> forced_;
 };
 
 #endif  // CHROME_BROWSER_PREFERENCES_MOCK_MAC_H_
