@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,12 +8,10 @@
 
 #include "base/basictypes.h"
 #include "base/command_line.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/values.h"
 #include "chrome/browser/prefs/value_map_pref_store.h"
 
-class DictionaryValue;
-
 // This PrefStore keeps track of preferences set by command-line switches,
 // such as proxy settings.
 class CommandLinePrefStore : public ValueMapPrefStore {
@@ -32,6 +30,11 @@ class CommandLinePrefStore : public ValueMapPrefStore {
     const char* preference_path;
   };
 
+  struct IntegerSwitchToPreferenceMapEntry {
+    const char* switch_name;
+    const char* preference_path;
+  };
+
   // |set_value| indicates what the preference should be set to if the switch
   // is present.
   struct BooleanSwitchToPreferenceMapEntry {
@@ -39,7 +42,6 @@ class CommandLinePrefStore : public ValueMapPrefStore {
     const char* preference_path;
     bool set_value;
   };
-  static const BooleanSwitchToPreferenceMapEntry boolean_switch_map_[];
 
   // Using the string and boolean maps, apply command-line switches to their
   // corresponding preferences in this pref store.
@@ -48,10 +50,16 @@ class CommandLinePrefStore : public ValueMapPrefStore {
   // Determines the proxy mode preference from the given proxy switches.
   void ApplyProxyMode();
 
+  // Apply the SSL/TLS preferences from the given switches.
+  void ApplySSLSwitches();
+
   // Weak reference.
   const CommandLine* command_line_;
 
+  // Mappings of command line switches to prefs.
+  static const BooleanSwitchToPreferenceMapEntry boolean_switch_map_[];
   static const StringSwitchToPreferenceMapEntry string_switch_map_[];
+  static const IntegerSwitchToPreferenceMapEntry integer_switch_map_[];
 
   DISALLOW_COPY_AND_ASSIGN(CommandLinePrefStore);
 };
