@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,22 +6,27 @@
 #define CHROME_BROWSER_PREFS_DEFAULT_PREF_STORE_H_
 #pragma once
 
-#include <map>
+#include <string>
 
-#include "base/basictypes.h"
+#include "base/values.h"
 #include "chrome/browser/prefs/value_map_pref_store.h"
 
 // This PrefStore keeps track of default preference values set when a
 // preference is registered with the PrefService.
 class DefaultPrefStore : public ValueMapPrefStore {
  public:
-  DefaultPrefStore() {}
-  virtual ~DefaultPrefStore() {}
+  DefaultPrefStore();
+  virtual ~DefaultPrefStore();
 
   // Stores a new |value| for |key|. Assumes ownership of |value|.
-  void SetDefaultValue(const std::string& key, Value* value) {
-    SetValue(key, value);
-  }
+  void SetDefaultValue(const std::string& key, Value* value);
+
+  // Removes the value for |key|.
+  void RemoveDefaultValue(const std::string& key);
+
+  // Returns the registered type for |key| or Value::TYPE_NULL if the |key|
+  // has not been registered.
+  base::Value::Type GetType(const std::string& key) const;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(DefaultPrefStore);
