@@ -10,6 +10,10 @@
 PrefChangeRegistrar::PrefChangeRegistrar() : service_(NULL) {}
 
 PrefChangeRegistrar::~PrefChangeRegistrar() {
+  // If you see an invalid memory access in this destructor, this
+  // PrefChangeRegistrar might be subscribed to an OffTheRecordProfileImpl that
+  // has been destroyed. This should not happen any more but be warned.
+  // Feel free to contact battre@chromium.org in case this happens.
   RemoveAll();
 }
 
@@ -18,7 +22,8 @@ void PrefChangeRegistrar::Init(PrefService* service) {
   service_ = service;
 }
 
-void PrefChangeRegistrar::Add(const char* path, NotificationObserver* obs) {
+void PrefChangeRegistrar::Add(const char* path,
+                              content::NotificationObserver* obs) {
   if (!service_) {
     NOTREACHED();
     return;
@@ -32,7 +37,8 @@ void PrefChangeRegistrar::Add(const char* path, NotificationObserver* obs) {
   service_->AddPrefObserver(path, obs);
 }
 
-void PrefChangeRegistrar::Remove(const char* path, NotificationObserver* obs) {
+void PrefChangeRegistrar::Remove(const char* path,
+                                 content::NotificationObserver* obs) {
   if (!service_) {
     NOTREACHED();
     return;
