@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,17 +9,17 @@
 #include <set>
 
 #include "base/basictypes.h"
-#include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/prefs/pref_change_registrar.h"
-#include "chrome/common/notification_observer.h"
+#include "chrome/browser/prefs/pref_service.h"
+#include "content/public/browser/notification_observer.h"
 
 // Observes the state of a set of preferences and allows to query their combined
 // managed bits.
-class PrefSetObserver : public NotificationObserver {
+class PrefSetObserver : public content::NotificationObserver {
  public:
   // Initialize with an empty set of preferences.
   PrefSetObserver(PrefService* pref_service,
-                  NotificationObserver* observer);
+                  content::NotificationObserver* observer);
   virtual ~PrefSetObserver();
 
   // Add a |pref| to the set of preferences to observe.
@@ -35,25 +35,25 @@ class PrefSetObserver : public NotificationObserver {
   // Create a pref set observer for all preferences relevant to proxies.
   static PrefSetObserver* CreateProxyPrefSetObserver(
       PrefService* pref_service,
-      NotificationObserver* observer);
+      content::NotificationObserver* observer);
 
   // Create a pref set observer for all preferences relevant to default search.
   static PrefSetObserver* CreateDefaultSearchPrefSetObserver(
       PrefService* pref_service,
-      NotificationObserver* observer);
+      content::NotificationObserver* observer);
 
  private:
-  // Overridden from NotificationObserver.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // Overridden from content::NotificationObserver.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   typedef std::set<std::string> PrefSet;
   PrefSet prefs_;
 
   PrefService* pref_service_;
   PrefChangeRegistrar registrar_;
-  NotificationObserver* observer_;
+  content::NotificationObserver* observer_;
 
   DISALLOW_COPY_AND_ASSIGN(PrefSetObserver);
 };
