@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,10 +11,15 @@
 #include "chrome/browser/defaults.h"
 #include "chrome/browser/net/url_fixer_upper.h"
 #include "chrome/browser/prefs/pref_service.h"
-#include "chrome/browser/prefs/scoped_pref_update.h"
+#include "chrome/browser/prefs/scoped_user_pref_update.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/pref_names.h"
 
+#ifdef OS_MACOSX
+#include "chrome/browser/first_run/first_run.h"
+#include "chrome/browser/ui/cocoa/window_restore_utils.h"
+#endif
+
 namespace {
 
 // For historical reasons the enum and value registered in the prefs don't line
@@ -45,9 +50,26 @@ SessionStartupPref::Type PrefValueToType(int pref_value) {
 
 // static
 void SessionStartupPref::RegisterUserPrefs(PrefService* prefs) {
+  SessionStartupPref::Type type = browser_defaults::kDefaultSessionStartupType;
+
+#ifdef OS_MACOSX
+  // During first run the calling code relies on |DEFAULT| session preference
+  // value to avoid session restore.  That is respected here.
+  if (!first_run::IsChromeFirstRun()) {
+    // |DEFAULT| really means "Don't restore".  The actual default value could
+    // change, so explicitly set both.
+    if (restore_utils::IsWindowRestoreEnabled())
+      type = SessionStartupPref::LAST;
+    else
+      type = SessionStartupPref::DEFAULT;
+  }
+#endif
+
   prefs->RegisterIntegerPref(prefs::kRestoreOnStartup,
-      TypeToPrefValue(browser_defaults::kDefaultSessionStartupType));
-  prefs->RegisterListPref(prefs::kURLsToRestoreOnStartup);
+                             TypeToPrefValue(type),
+                             PrefService::SYNCABLE_PREF);
+  prefs->RegisterListPref(prefs::kURLsToRestoreOnStartup,
+                          PrefService::SYNCABLE_PREF);
 }
 
 // static
@@ -70,9 +92,8 @@ void SessionStartupPref::SetStartupPref(PrefService* prefs,
     // Always save the URLs, that way the UI can remain consistent even if the
     // user changes the startup type pref.
     // Ownership of the ListValue retains with the pref service.
-    ScopedPrefUpdate update(prefs, prefs::kURLsToRestoreOnStartup);
-    ListValue* url_pref_list =
-        prefs->GetMutableList(prefs::kURLsToRestoreOnStartup);
+    ListPrefUpdate update(prefs, prefs::kURLsToRestoreOnStartup);
+    ListValue* url_pref_list = update.Get();
     DCHECK(url_pref_list);
     url_pref_list->Clear();
     for (size_t i = 0; i < pref.urls.size(); ++i) {
