@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 
 #include <algorithm>
 
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/values.h"
 
 ValueMapPrefStore::ValueMapPrefStore() {}
@@ -14,7 +14,7 @@ ValueMapPrefStore::ValueMapPrefStore() {}
 ValueMapPrefStore::~ValueMapPrefStore() {}
 
 PrefStore::ReadResult ValueMapPrefStore::GetValue(const std::string& key,
-                                                  Value** value) const {
+                                                  const Value** value) const {
   return prefs_.GetValue(key, value) ? READ_OK : READ_NO_VALUE;
 }
 
@@ -26,6 +26,10 @@ void ValueMapPrefStore::RemoveObserver(PrefStore::Observer* observer) {
   observers_.RemoveObserver(observer);
 }
 
+size_t ValueMapPrefStore::NumberOfObservers() const {
+  return observers_.size();
+}
+
 void ValueMapPrefStore::SetValue(const std::string& key, Value* value) {
   if (prefs_.SetValue(key, value))
     FOR_EACH_OBSERVER(Observer, observers_, OnPrefValueChanged(key));
@@ -37,5 +41,21 @@ void ValueMapPrefStore::RemoveValue(const std::string& key) {
 }
 
 void ValueMapPrefStore::NotifyInitializationCompleted() {
-  FOR_EACH_OBSERVER(Observer, observers_, OnInitializationCompleted());
+  FOR_EACH_OBSERVER(Observer, observers_, OnInitializationCompleted(true));
+}
+
+ValueMapPrefStore::iterator ValueMapPrefStore::begin() {
+  return prefs_.begin();
+}
+
+ValueMapPrefStore::iterator ValueMapPrefStore::end() {
+  return prefs_.end();
+}
+
+ValueMapPrefStore::const_iterator ValueMapPrefStore::begin() const {
+  return prefs_.begin();
+}
+
+ValueMapPrefStore::const_iterator ValueMapPrefStore::end() const {
+  return prefs_.end();
 }
