@@ -1,58 +1,68 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/printing/cloud_print/cloud_print_setup_flow.h"
 
 #include "base/json/json_writer.h"
-#include "base/singleton.h"
+#include "base/memory/singleton.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/dom_ui/chrome_url_data_manager.h"
-#include "chrome/browser/dom_ui/dom_ui_util.h"
-#include "chrome/browser/platform_util.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/printing/cloud_print/cloud_print_proxy_service.h"
+#include "chrome/browser/printing/cloud_print/cloud_print_proxy_service_factory.h"
 #include "chrome/browser/printing/cloud_print/cloud_print_setup_message_handler.h"
 #include "chrome/browser/printing/cloud_print/cloud_print_setup_source.h"
 #include "chrome/browser/printing/cloud_print/cloud_print_url.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
 #include "chrome/browser/service/service_process_control.h"
-#include "chrome/browser/service/service_process_control_manager.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_dialogs.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/dialog_style.h"
+#include "chrome/browser/ui/webui/chrome_url_data_manager.h"
 #include "chrome/common/net/gaia/gaia_auth_fetcher.h"
 #include "chrome/common/net/gaia/gaia_constants.h"
 #include "chrome/common/net/gaia/google_service_auth_error.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/service_messages.h"
-#include "gfx/font.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/chromium_strings.h"
 #include "grit/locale_settings.h"
 #include "ui/base/l10n/l10n_font_util.h"
+#include "ui/gfx/font.h"
 
-#if defined(TOOLKIT_GTK)
-#include "chrome/browser/ui/gtk/html_dialog_gtk.h"
-#endif  // defined(TOOLKIT_GTK)
+using content::OpenURLParams;
+using content::Referrer;
+using content::WebContents;
+using content::WebUIMessageHandler;
 
-#if defined(TOOLKIT_VIEWS)
-#include "chrome/browser/ui/views/browser_dialogs.h"
-#endif  // defined(TOOLKIT_GTK)
+namespace {
 
-static const wchar_t kGaiaLoginIFrameXPath[] = L"//iframe[@id='gaialogin']";
-static const wchar_t kDoneIframeXPath[] = L"//iframe[@id='setupdone']";
+string16 SetupIframeXPath() {
+  return ASCIIToUTF16("//iframe[@id='cloudprintsetup']");
+}
+
+string16 DoneIframeXPath() {
+  return ASCIIToUTF16("//iframe[@id='setupdone']");
+}
+
+}  // end namespace
 
 ////////////////////////////////////////////////////////////////////////////////
 // CloudPrintSetupFlow implementation.
 
 // static
 CloudPrintSetupFlow* CloudPrintSetupFlow::OpenDialog(
-    Profile* profile, Delegate* delegate, gfx::NativeWindow parent_window) {
+    Profile* profile,
+    const base::WeakPtr<Delegate>& delegate,
+    gfx::NativeWindow parent_window) {
+  DCHECK(profile);
+  Browser* browser = BrowserList::GetLastActiveWithProfile(profile);
   // Set the arguments for showing the gaia login page.
   DictionaryValue args;
   args.SetString("user", "");
@@ -74,38 +84,27 @@ CloudPrintSetupFlow* CloudPrintSetupFlow::OpenDialog(
   // invoked in the context of a "token expired" notfication. If we don't have
   // a brower, use the underlying dialog system to show the dialog without
   // using a browser.
-  Browser* browser = BrowserList::GetLastActive();
-  if (browser) {
-    browser->BrowserShowHtmlDialog(flow, parent_window);
-  } else {
-#if defined(TOOLKIT_VIEWS)
-    browser::ShowHtmlDialogView(parent_window, profile, flow);
-#elif defined(TOOLKIT_GTK)
-    HtmlDialogGtk* html_dialog =
-        new HtmlDialogGtk(profile, flow, parent_window);
-    html_dialog->InitDialog();
-#endif  // defined(TOOLKIT_VIEWS)
-  // TODO(sanjeevr): Implement the "no browser" scenario for the Mac.
-  }
+  if (!parent_window && browser && browser->window())
+    parent_window = browser->window()->GetNativeHandle();
+  browser::ShowHtmlDialog(parent_window, profile, browser, flow, STYLE_GENERIC);
   return flow;
 }
 
-CloudPrintSetupFlow::CloudPrintSetupFlow(const std::string& args,
-                                         Profile* profile,
-                                         Delegate* delegate,
-                                         bool setup_done)
-    : dom_ui_(NULL),
+CloudPrintSetupFlow::CloudPrintSetupFlow(
+    const std::string& args,
+    Profile* profile,
+    const base::WeakPtr<Delegate>& delegate,
+    bool setup_done)
+    : web_ui_(NULL),
       dialog_start_args_(args),
+      last_auth_error_(GoogleServiceAuthError::None()),
       setup_done_(setup_done),
       process_control_(NULL),
       delegate_(delegate) {
   // TODO(hclam): The data source should be added once.
   profile_ = profile;
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(ChromeURLDataManager::GetInstance(),
-                        &ChromeURLDataManager::AddDataSource,
-                        make_scoped_refptr(new CloudPrintSetupSource())));
+  profile->GetChromeURLDataManager()->AddDataSource(
+      new CloudPrintSetupSource());
 }
 
 CloudPrintSetupFlow::~CloudPrintSetupFlow() {
@@ -122,8 +121,8 @@ GURL CloudPrintSetupFlow::GetDialogContentURL() const {
   return GURL("chrome://cloudprintsetup/setupflow");
 }
 
-void CloudPrintSetupFlow::GetDOMMessageHandlers(
-    std::vector<DOMMessageHandler*>* handlers) const {
+void CloudPrintSetupFlow::GetWebUIMessageHandlers(
+    std::vector<WebUIMessageHandler*>* handlers) const {
   // Create the message handler only after we are asked, the caller is
   // responsible for deleting the objects.
   handlers->push_back(
@@ -134,8 +133,8 @@ void CloudPrintSetupFlow::GetDOMMessageHandlers(
 void CloudPrintSetupFlow::GetDialogSize(gfx::Size* size) const {
   PrefService* prefs = profile_->GetPrefs();
   gfx::Font approximate_web_font(
-      UTF8ToUTF16(prefs->GetString(prefs::kWebKitSansSerifFontFamily)),
-      prefs->GetInteger(prefs::kWebKitDefaultFontSize));
+      prefs->GetString(prefs::kWebKitGlobalSansSerifFontFamily),
+      prefs->GetInteger(prefs::kWebKitGlobalDefaultFontSize));
 
   if (setup_done_) {
     *size = ui::GetLocalizedContentsSizeForFont(
@@ -148,6 +147,23 @@ void CloudPrintSetupFlow::GetDialogSize(gfx::Size* size) const {
         IDS_CLOUD_PRINT_SETUP_WIZARD_HEIGHT_LINES,
         approximate_web_font);
   }
+
+#if !defined(OS_WIN)
+  // NOTE(scottbyer):The following comment comes from
+  // SyncSetupFlow::GetDialogSize, where this hack was copied from. By starting
+  // off development of the UI on Windows, the hack is seemingly backwards.
+
+  // NOTE(akalin): This is a hack to work around a problem with font height on
+  // windows.  Basically font metrics are incorrectly returned in logical units
+  // instead of pixels on Windows.  Logical units are very commonly 96 DPI
+  // so our localized char/line counts are too small by a factor of 96/72.
+  // So we compensate for this on non-windows platform.
+
+  // TODO(scottbyer): Fix the root cause, kill the hacks.
+  float scale_hack = 96.f/72.f;
+  size->set_width(size->width() * scale_hack);
+  size->set_height(size->height() * scale_hack);
+#endif
 }
 
 // A callback to notify the delegate that the dialog closed.
@@ -156,42 +172,40 @@ void CloudPrintSetupFlow::OnDialogClosed(const std::string& json_retval) {
   if (authenticator_.get())
     authenticator_->CancelRequest();
 
-  if (delegate_) {
+  if (delegate_)
     delegate_->OnDialogClosed();
-  }
   delete this;
 }
 
 std::string CloudPrintSetupFlow::GetDialogArgs() const {
-    return dialog_start_args_;
+  return dialog_start_args_;
 }
 
-void CloudPrintSetupFlow::OnCloseContents(TabContents* source,
+void CloudPrintSetupFlow::OnCloseContents(WebContents* source,
                                           bool* out_close_dialog) {
 }
 
-std::wstring CloudPrintSetupFlow::GetDialogTitle() const {
-  return UTF16ToWideHack(
-      l10n_util::GetStringUTF16(IDS_CLOUD_PRINT_SETUP_DIALOG_TITLE));
+string16 CloudPrintSetupFlow::GetDialogTitle() const {
+  return l10n_util::GetStringFUTF16(IDS_CLOUD_PRINT_SETUP_DIALOG_TITLE,
+      l10n_util::GetStringUTF16(IDS_GOOGLE_CLOUD_PRINT));
 }
 
-bool CloudPrintSetupFlow::IsDialogModal() const {
+ui::ModalType CloudPrintSetupFlow::GetDialogModalType() const {
   // We are always modeless.
-  return false;
+  return ui::MODAL_TYPE_NONE;
 }
 
 bool CloudPrintSetupFlow::ShouldShowDialogTitle() const {
   return true;
 }
 
-///////////////////////////////////////////////////////////////////////////////
-// GaiaAuthConsumer implementation.
-void CloudPrintSetupFlow::OnClientLoginFailure(
-    const GoogleServiceAuthError& error) {
-  ShowGaiaFailed(error);
-  authenticator_.reset();
+bool CloudPrintSetupFlow::HandleContextMenu(
+    const ContextMenuParams& params) {
+  return true;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// GaiaAuthConsumer implementation.
 void CloudPrintSetupFlow::OnClientLoginSuccess(
     const GaiaAuthConsumer::ClientLoginResult& credentials) {
   // Save the token for the cloud print proxy.
@@ -201,21 +215,29 @@ void CloudPrintSetupFlow::OnClientLoginSuccess(
   ShowGaiaSuccessAndSettingUp();
   authenticator_.reset();
 
-  profile_->GetCloudPrintProxyService()->EnableForUser(credentials.lsid,
-                                                       login_);
+  CloudPrintProxyServiceFactory::GetForProfile(profile_)->EnableForUser(
+      credentials.lsid, login_);
   // TODO(sanjeevr): Should we wait and verify that the enable succeeded?
   ShowSetupDone();
 }
 
+void CloudPrintSetupFlow::OnClientLoginFailure(
+    const GoogleServiceAuthError& error) {
+  last_auth_error_ = error;
+  ShowGaiaFailed(error);
+  authenticator_.reset();
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // Methods called by CloudPrintSetupMessageHandler
-void CloudPrintSetupFlow::Attach(DOMUI* dom_ui) {
-  dom_ui_ = dom_ui;
+void CloudPrintSetupFlow::Attach(content::WebUI* web_ui) {
+  web_ui_ = web_ui;
 }
 
 void CloudPrintSetupFlow::OnUserSubmittedAuth(const std::string& user,
                                               const std::string& password,
-                                              const std::string& captcha) {
+                                              const std::string& captcha,
+                                              const std::string& access_code) {
   // Save the login name only.
   login_ = user;
 
@@ -224,46 +246,56 @@ void CloudPrintSetupFlow::OnUserSubmittedAuth(const std::string& user,
       new GaiaAuthFetcher(this, GaiaConstants::kChromeSource,
                           profile_->GetRequestContext()));
 
-  authenticator_->StartClientLogin(user, password,
-                                   GaiaConstants::kCloudPrintService,
-                                   "", captcha,
-                                   GaiaAuthFetcher::HostedAccountsAllowed);
+  if (!access_code.empty()) {
+    authenticator_->StartClientLogin(user, access_code,
+                                     GaiaConstants::kCloudPrintService,
+                                     std::string(), std::string(),
+                                     GaiaAuthFetcher::HostedAccountsAllowed);
+  } else {
+    authenticator_->StartClientLogin(user, password,
+                                     GaiaConstants::kCloudPrintService,
+                                     last_auth_error_.captcha().token, captcha,
+                                     GaiaAuthFetcher::HostedAccountsAllowed);
+  }
 }
 
 void CloudPrintSetupFlow::OnUserClickedLearnMore() {
-  dom_ui_->tab_contents()->OpenURL(CloudPrintURL::GetCloudPrintLearnMoreURL(),
-                                   GURL(), NEW_FOREGROUND_TAB,
-                                   PageTransition::LINK);
+  OpenURLParams params(CloudPrintURL::GetCloudPrintLearnMoreURL(),
+                       Referrer(), NEW_FOREGROUND_TAB,
+                       content::PAGE_TRANSITION_LINK, false);
+  web_ui_->GetWebContents()->OpenURL(params);
 }
 
 void CloudPrintSetupFlow::OnUserClickedPrintTestPage() {
-  dom_ui_->tab_contents()->OpenURL(CloudPrintURL::GetCloudPrintTestPageURL(),
-                                   GURL(), NEW_FOREGROUND_TAB,
-                                   PageTransition::LINK);
+  OpenURLParams params(CloudPrintURL::GetCloudPrintTestPageURL(),
+                       Referrer(), NEW_FOREGROUND_TAB,
+                       content::PAGE_TRANSITION_LINK, false);
+  web_ui_->GetWebContents()->OpenURL(params);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// Helper methods for showing contents of the DOM UI
+// Helper methods for showing contents of the Web UI
 void CloudPrintSetupFlow::ShowGaiaLogin(const DictionaryValue& args) {
-  if (dom_ui_)
-    dom_ui_->CallJavascriptFunction(L"cloudprint.showSetupLogin");
+  if (web_ui_)
+    web_ui_->CallJavascriptFunction("cloudprint.showSetupLogin");
 
   std::string json;
   base::JSONWriter::Write(&args, false, &json);
-  std::wstring javascript = std::wstring(L"showGaiaLogin") +
-      L"(" + UTF8ToWide(json) + L");";
-  ExecuteJavascriptInIFrame(kGaiaLoginIFrameXPath, javascript);
+  string16 javascript = UTF8ToUTF16("cloudprint.showGaiaLogin(" + json + ");");
+
+  ExecuteJavascriptInIFrame(SetupIframeXPath(), javascript);
 }
 
 void CloudPrintSetupFlow::ShowGaiaSuccessAndSettingUp() {
-  ExecuteJavascriptInIFrame(kGaiaLoginIFrameXPath,
-                            L"showGaiaSuccessAndSettingUp();");
+  ExecuteJavascriptInIFrame(
+      SetupIframeXPath(),
+      ASCIIToUTF16("cloudprint.showGaiaSuccessAndSettingUp();"));
 }
 
 void CloudPrintSetupFlow::ShowGaiaFailed(const GoogleServiceAuthError& error) {
   DictionaryValue args;
   args.SetString("pageToShow", "cloudprintsetup");
-  args.SetString("user", "");
+  args.SetString("user", login_);
   args.SetInteger("error", error.state());
   args.SetBoolean("editable_user", true);
   args.SetString("captchaUrl", error.captcha().image_url.spec());
@@ -273,37 +305,38 @@ void CloudPrintSetupFlow::ShowGaiaFailed(const GoogleServiceAuthError& error) {
 void CloudPrintSetupFlow::ShowSetupDone() {
   setup_done_ = true;
   string16 product_name = l10n_util::GetStringUTF16(IDS_PRODUCT_NAME);
-  std::wstring message =
-      UTF16ToWideHack(l10n_util::GetStringFUTF16(IDS_CLOUD_PRINT_SETUP_DONE,
-                                                 product_name,
-                                                 UTF8ToUTF16(login_)));
-  std::wstring javascript = L"cloudprint.setMessage('" + message + L"');";
-  ExecuteJavascriptInIFrame(kDoneIframeXPath, javascript);
+  string16 message = l10n_util::GetStringFUTF16(IDS_CLOUD_PRINT_SETUP_DONE,
+                                                product_name,
+                                                UTF8ToUTF16(login_));
+  string16 javascript(ASCIIToUTF16("cloudprint.setMessage('") + message +
+                      ASCIIToUTF16("');"));
+  ExecuteJavascriptInIFrame(DoneIframeXPath(), javascript);
 
-  if (dom_ui_) {
+  if (web_ui_) {
     PrefService* prefs = profile_->GetPrefs();
     gfx::Font approximate_web_font(
-        UTF8ToUTF16(prefs->GetString(prefs::kWebKitSansSerifFontFamily)),
-        prefs->GetInteger(prefs::kWebKitDefaultFontSize));
+        prefs->GetString(prefs::kWebKitGlobalSansSerifFontFamily),
+        prefs->GetInteger(prefs::kWebKitGlobalDefaultFontSize));
     gfx::Size done_size = ui::GetLocalizedContentsSizeForFont(
         IDS_CLOUD_PRINT_SETUP_WIZARD_DONE_WIDTH_CHARS,
         IDS_CLOUD_PRINT_SETUP_WIZARD_DONE_HEIGHT_LINES,
         approximate_web_font);
 
-    FundamentalValue new_width(done_size.width());
-    FundamentalValue new_height(done_size.height());
-    dom_ui_->CallJavascriptFunction(L"cloudprint.showSetupDone",
+    base::FundamentalValue new_width(done_size.width());
+    base::FundamentalValue new_height(done_size.height());
+    web_ui_->CallJavascriptFunction("cloudprint.showSetupDone",
                                     new_width, new_height);
   }
 
-  ExecuteJavascriptInIFrame(kDoneIframeXPath, L"cloudprint.onPageShown();");
+  ExecuteJavascriptInIFrame(DoneIframeXPath(),
+                            ASCIIToUTF16("cloudprint.onPageShown();"));
 }
 
 void CloudPrintSetupFlow::ExecuteJavascriptInIFrame(
-    const std::wstring& iframe_xpath,
-    const std::wstring& js) {
-  if (dom_ui_) {
-    RenderViewHost* rvh = dom_ui_->tab_contents()->render_view_host();
+    const string16& iframe_xpath,
+    const string16& js) {
+  if (web_ui_) {
+    RenderViewHost* rvh = web_ui_->GetWebContents()->GetRenderViewHost();
     rvh->ExecuteJavascriptInWebFrame(iframe_xpath, js);
   }
 }
