@@ -1,35 +1,40 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/printing/cloud_print/cloud_print_setup_message_handler.h"
 
-#include "base/scoped_ptr.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
-#include "chrome/browser/dom_ui/dom_ui_util.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/printing/cloud_print/cloud_print_setup_flow.h"
-
-DOMMessageHandler* CloudPrintSetupMessageHandler::Attach(DOMUI* dom_ui) {
-  // Pass the DOMUI object to the setup flow.
-  flow_->Attach(dom_ui);
-  return DOMMessageHandler::Attach(dom_ui);
-}
+#include "content/public/browser/web_ui.h"
 
 void CloudPrintSetupMessageHandler::RegisterMessages() {
-  dom_ui_->RegisterMessageCallback("SubmitAuth",
-      NewCallback(this, &CloudPrintSetupMessageHandler::HandleSubmitAuth));
-  dom_ui_->RegisterMessageCallback("PrintTestPage",
-      NewCallback(this, &CloudPrintSetupMessageHandler::HandlePrintTestPage));
-  dom_ui_->RegisterMessageCallback("LearnMore",
-      NewCallback(this, &CloudPrintSetupMessageHandler::HandleLearnMore));
+  // Pass the WebUI object to the setup flow.
+  flow_->Attach(web_ui());
+
+  web_ui()->RegisterMessageCallback("SubmitAuth",
+      base::Bind(&CloudPrintSetupMessageHandler::HandleSubmitAuth,
+                 base::Unretained(this)));
+  web_ui()->RegisterMessageCallback("PrintTestPage",
+      base::Bind(&CloudPrintSetupMessageHandler::HandlePrintTestPage,
+                 base::Unretained(this)));
+  web_ui()->RegisterMessageCallback("LearnMore",
+      base::Bind(&CloudPrintSetupMessageHandler::HandleLearnMore,
+                 base::Unretained(this)));
 }
 
 void CloudPrintSetupMessageHandler::HandleSubmitAuth(const ListValue* args) {
-  std::string json(dom_ui_util::GetJsonResponseFromFirstArgumentInList(args));
-  std::string username, password, captcha;
-  if (json.empty())
+  std::string json;
+  bool ret = args->GetString(0, &json);
+  std::string username, password, captcha, access_code;
+  if (!ret || json.empty()) {
+    NOTREACHED() << "Empty json string";
     return;
+  }
 
   scoped_ptr<Value> parsed_value(base::JSONReader::Read(json, false));
   if (!parsed_value.get() || !parsed_value->IsType(Value::TYPE_DICTIONARY)) {
@@ -40,14 +45,15 @@ void CloudPrintSetupMessageHandler::HandleSubmitAuth(const ListValue* args) {
   DictionaryValue* result = static_cast<DictionaryValue*>(parsed_value.get());
   if (!result->GetString("user", &username) ||
       !result->GetString("pass", &password) ||
-      !result->GetString("captcha", &captcha)) {
+      !result->GetString("captcha", &captcha) ||
+      !result->GetString("access_code", &access_code)) {
     NOTREACHED() << "Unable to parse auth data";
     return;
   }
 
   // Pass the information to the flow.
   if (flow_)
-    flow_->OnUserSubmittedAuth(username, password, captcha);
+    flow_->OnUserSubmittedAuth(username, password, captcha, access_code);
 }
 
 void CloudPrintSetupMessageHandler::HandlePrintTestPage(const ListValue* args) {
