@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,25 +7,31 @@
 
 #include <functional>
 
+#include "base/bind.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
+#include "base/memory/singleton.h"
 #include "base/path_service.h"
-#include "base/singleton.h"
 #include "base/threading/thread_restrictions.h"
+#include "base/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/dom_ui/chrome_url_data_manager.h"
 #include "chrome/browser/printing/cloud_print/cloud_print_url.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/webui/chrome_url_data_manager.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/web_contents.h"
+#include "content/test/test_browser_thread.h"
 #include "net/url_request/url_request_filter.h"
 #include "net/url_request/url_request_test_job.h"
-#include "net/url_request/url_request_unittest.h"
+#include "net/url_request/url_request_test_util.h"
+
+using content::BrowserThread;
 
 namespace {
 
@@ -148,7 +154,7 @@ class PrintDialogCloudTest : public InProcessBrowserTest {
 
     virtual void OnResponseCompleted(net::URLRequest* request) {
       BrowserThread::PostTask(BrowserThread::UI, FROM_HERE,
-                              new MessageLoop::QuitTask());
+                              MessageLoop::QuitClosure());
     }
   };
 
@@ -198,7 +204,9 @@ class PrintDialogCloudTest : public InProcessBrowserTest {
         test_data_directory_.AppendASCII("printing/cloud_print_uitest.pdf");
     BrowserThread::PostTask(
         BrowserThread::UI, FROM_HERE,
-        NewRunnableFunction(&PrintDialogCloud::CreateDialogImpl, path_to_pdf));
+        base::Bind(&internal_cloud_print_helpers::CreateDialogFullImpl,
+                   path_to_pdf, string16(), string16(),
+                   std::string("application/pdf"), true, false));
   }
 
   bool handler_added_;
@@ -219,7 +227,13 @@ net::URLRequestJob* PrintDialogCloudTest::Factory(net::URLRequest* request,
   return new SimpleTestJob(request);
 }
 
-IN_PROC_BROWSER_TEST_F(PrintDialogCloudTest, HandlersRegistered) {
+#if defined(OS_WIN)
+// http://crbug.com/94864 for OS_WIN issue
+#define MAYBE_HandlersRegistered DISABLED_HandlersRegistered
+#else
+#define MAYBE_HandlersRegistered HandlersRegistered
+#endif
+IN_PROC_BROWSER_TEST_F(PrintDialogCloudTest, MAYBE_HandlersRegistered) {
   BrowserList::SetLastActive(browser());
   ASSERT_TRUE(BrowserList::GetLastActive());
 
@@ -230,28 +244,13 @@ IN_PROC_BROWSER_TEST_F(PrintDialogCloudTest, HandlersRegistered) {
   ui_test_utils::RunMessageLoop();
 
   ASSERT_TRUE(TestController::GetInstance()->result());
+
+  // Close the dialog before finishing the test.
+  ui_test_utils::WindowedNotificationObserver signal(
+      content::NOTIFICATION_TAB_CLOSED,
+      content::NotificationService::AllSources());
+  EXPECT_TRUE(ui_test_utils::SendKeyPressSync(browser(), ui::VKEY_ESCAPE,
+                                              false, false, false, false));
+  signal.Wait();
 }
 
-#if defined(OS_CHROMEOS)
-// Disabled until the extern URL is live so that the Print menu item
-// can be enabled for Chromium OS.
-IN_PROC_BROWSER_TEST_F(PrintDialogCloudTest, DISABLED_DialogGrabbed) {
-  BrowserList::SetLastActive(browser());
-  ASSERT_TRUE(BrowserList::GetLastActive());
-
-  AddTestHandlers();
-
-  // This goes back one step further for the Chrome OS case, to making
-  // sure 'window.print()' gets to the right place.
-  ASSERT_TRUE(browser()->GetSelectedTabContents());
-  ASSERT_TRUE(browser()->GetSelectedTabContents()->render_view_host());
-
-  std::wstring window_print(L"window.print()");
-  browser()->GetSelectedTabContents()->render_view_host()->
-      ExecuteJavascriptInWebFrame(std::wstring(), window_print);
-
-  ui_test_utils::RunMessageLoop();
-
-  ASSERT_TRUE(TestController::GetInstance()->result());
-}
-#endif
