@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,13 +7,12 @@
 #pragma once
 
 #include "base/basictypes.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/scoped_ptr.h"
 #include "chrome/browser/printing/print_job_worker_owner.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 
-class GURL;
 class Thread;
 
 namespace printing {
@@ -34,7 +33,7 @@ class PrinterQuery;
 // reference to the job to be sure it is kept alive. All the code in this class
 // runs in the UI thread.
 class PrintJob : public PrintJobWorkerOwner,
-                 public NotificationObserver,
+                 public content::NotificationObserver,
                  public MessageLoop::DestructionObserver {
  public:
   // Create a empty PrintJob. When initializing with this constructor,
@@ -42,24 +41,25 @@ class PrintJob : public PrintJobWorkerOwner,
   PrintJob();
 
   // Grabs the ownership of the PrintJobWorker from another job, which is
-  // usually a PrinterQuery.
-  void Initialize(PrintJobWorkerOwner* job, PrintedPagesSource* source);
+  // usually a PrinterQuery. Set the expected page count of the print job.
+  void Initialize(PrintJobWorkerOwner* job, PrintedPagesSource* source,
+                  int page_count);
 
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
-  // PrintJobWorkerOwner
+  // PrintJobWorkerOwner implementation.
   virtual void GetSettingsDone(const PrintSettings& new_settings,
-                               PrintingContext::Result result);
-  virtual PrintJobWorker* DetachWorker(PrintJobWorkerOwner* new_owner);
-  virtual MessageLoop* message_loop();
-  virtual const PrintSettings& settings() const;
-  virtual int cookie() const;
+                               PrintingContext::Result result) OVERRIDE;
+  virtual PrintJobWorker* DetachWorker(PrintJobWorkerOwner* new_owner) OVERRIDE;
+  virtual MessageLoop* message_loop() OVERRIDE;
+  virtual const PrintSettings& settings() const OVERRIDE;
+  virtual int cookie() const OVERRIDE;
 
-  // DestructionObserver
-  virtual void WillDestroyCurrentMessageLoop();
+  // DestructionObserver implementation.
+  virtual void WillDestroyCurrentMessageLoop() OVERRIDE;
 
   // Starts the actual printing. Signals the worker that it should begin to
   // spool as soon as data is available.
@@ -87,9 +87,6 @@ class PrintJob : public PrintJobWorkerOwner,
   // and the end of the spooling.
   bool is_job_pending() const;
 
-  // Returns true if the Print... dialog box is currently displayed.
-  bool is_print_dialog_box_shown() const;
-
   // Access the current printed document. Warning: may be NULL.
   PrintedDocument* document() const;
 
@@ -111,7 +108,7 @@ class PrintJob : public PrintJobWorkerOwner,
   // eventual deadlock.
   void ControlledWorkerShutdown();
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // Main message loop reference. Used to send notifications in the right
   // thread.
@@ -135,9 +132,6 @@ class PrintJob : public PrintJobWorkerOwner,
   // Is the worker thread printing.
   bool is_job_pending_;
 
-  // Is the Print... dialog box currently shown.
-  bool is_print_dialog_box_shown_;
-
   // Is Canceling? If so, try to not cause recursion if on FAILED notification,
   // the notified calls Cancel() again.
   bool is_canceling_;
