@@ -1,23 +1,26 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/printing/print_job_manager.h"
 
-#include "chrome/browser/browser_process.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/printing/print_job.h"
 #include "chrome/browser/printing/printer_query.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/notification_service.h"
 #include "printing/printed_document.h"
 #include "printing/printed_page.h"
 
+using content::BrowserThread;
+
 namespace printing {
 
 PrintJobManager::PrintJobManager() {
-  registrar_.Add(this, NotificationType::PRINT_JOB_EVENT,
-                 NotificationService::AllSources());
+  registrar_.Add(this, chrome::NOTIFICATION_PRINT_JOB_EVENT,
+                 content::NotificationService::AllSources());
 }
 
 PrintJobManager::~PrintJobManager() {
@@ -25,6 +28,12 @@ PrintJobManager::~PrintJobManager() {
   queued_queries_.clear();
 }
 
+void PrintJobManager::InitOnUIThread(PrefService* prefs) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  printing_enabled_.Init(prefs::kPrintingEnabled, prefs, NULL);
+  printing_enabled_.MoveToThread(BrowserThread::IO);
+}
+
 void PrintJobManager::OnQuit() {
 #if defined(OS_MACOSX)
   // OnQuit is too late to try to wait for jobs on the Mac, since the runloop
@@ -82,22 +91,18 @@ void PrintJobManager::PopPrinterQuery(int document_cookie,
   }
 }
 
+// static
+void PrintJobManager::RegisterPrefs(PrefService* prefs) {
+  prefs->RegisterBooleanPref(prefs::kPrintingEnabled, true);
+}
 
-void PrintJobManager::Observe(NotificationType type,
-                              const NotificationSource& source,
-                              const NotificationDetails& details) {
-  switch (type.value) {
-    case NotificationType::PRINT_JOB_EVENT: {
-      OnPrintJobEvent(Source<PrintJob>(source).ptr(),
-                      *Details<JobEventDetails>(details).ptr());
-      break;
-    }
-    case NotificationType::PREF_CHANGED: {
-      const std::string* pref_name = Details<std::string>(details).ptr();
-      if (*pref_name == prefs::kPrintingEnabled) {
-        PrefService *local_state = g_browser_process->local_state();
-        set_printing_enabled(local_state->GetBoolean(prefs::kPrintingEnabled));
-      }
+void PrintJobManager::Observe(int type,
+                              const content::NotificationSource& source,
+                              const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_PRINT_JOB_EVENT: {
+      OnPrintJobEvent(content::Source<PrintJob>(source).ptr(),
+                      *content::Details<JobEventDetails>(details).ptr());
       break;
     }
     default: {
@@ -161,14 +166,8 @@ void PrintJobManager::OnPrintJobEvent(
   }
 }
 
-bool PrintJobManager::printing_enabled() {
-  base::AutoLock lock(enabled_lock_);
-  return printing_enabled_;
-}
-
-void PrintJobManager::set_printing_enabled(bool printing_enabled) {
-  base::AutoLock lock(enabled_lock_);
-  printing_enabled_ = printing_enabled;
+bool PrintJobManager::printing_enabled() const {
+  return *printing_enabled_;
 }
 
 }  // namespace printing
