@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,23 +8,28 @@
 
 #include <vector>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/synchronization/lock.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "chrome/browser/prefs/pref_member.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+
+class PrefService;
 
 namespace printing {
 
 class JobEventDetails;
-class PrintedDocument;
 class PrintJob;
-class PrintedPage;
 class PrinterQuery;
 
-class PrintJobManager : public NotificationObserver {
+class PrintJobManager : public content::NotificationObserver {
  public:
   PrintJobManager();
-  ~PrintJobManager();
+  virtual ~PrintJobManager();
+
+  // Registers for changes to the printing enabled preference in |prefs|.
+  // This method should be called on the UI thread.
+  void InitOnUIThread(PrefService* prefs);
 
   // On browser quit, we should wait to have the print job finished.
   void OnQuit();
@@ -42,14 +47,16 @@ class PrintJobManager : public NotificationObserver {
   // called from any thread. Current use case is poping from the browser thread.
   void PopPrinterQuery(int document_cookie, scoped_refptr<PrinterQuery>* job);
 
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  static void RegisterPrefs(PrefService* prefs);
 
-  bool printing_enabled();
+  // content::NotificationObserver
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
-  void set_printing_enabled(bool printing_enabled);
+  // Only accessed on the IO thread. UI thread can query
+  // prefs::kPrintingEnabled via g_browser_process->local_state() directly.
+  bool printing_enabled() const;
 
  private:
   typedef std::vector<scoped_refptr<PrintJob> > PrintJobs;
@@ -59,25 +66,26 @@ class PrintJobManager : public NotificationObserver {
   void OnPrintJobEvent(PrintJob* print_job,
                        const JobEventDetails& event_details);
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
+
+  // Printing is enabled/disabled. For printing with the native print dialog,
+  // this variable is checked at only one place, by
+  // PrintingMessageFilter::OnGetDefaultPrintSettings. If its value is true
+  // at that point, then the initiated print flow will complete itself,
+  // even if the value of this variable changes afterwards.
+  // In the print preview workflow, this variable is checked in
+  // PrintingMessageFilter::OnUpdatePrintSettings, which gets called multiple
+  // times in the print preview workflow.
+  BooleanPrefMember printing_enabled_;
 
   // Used to serialize access to queued_workers_.
   base::Lock lock_;
 
-  // Used to serialize access to printing_enabled_
-  base::Lock enabled_lock_;
-
   PrinterQueries queued_queries_;
 
   // Current print jobs that are active.
   PrintJobs current_jobs_;
 
-  // Printing is enabled/disabled. This variable is checked at only one place,
-  // by RenderMessageFilter::OnGetDefaultPrintSettings. If its value is true
-  // at that point, then the initiated print flow will complete itself,
-  // even if the value of this variable changes afterwards.
-  bool printing_enabled_;
-
   DISALLOW_COPY_AND_ASSIGN(PrintJobManager);
 };
 
