@@ -1,4 +1,4 @@
-// Copyright (c) 2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,22 +6,19 @@
 #include "base/string16.h"
 #include "chrome/browser/printing/print_job.h"
 #include "chrome/browser/printing/print_job_worker.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_service.h"
 #include "printing/printed_pages_source.h"
-#include "googleurl/src/gurl.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace {
 
 class TestSource : public printing::PrintedPagesSource {
  public:
-  virtual string16 RenderSourceName() {
+  virtual string16 RenderSourceName() OVERRIDE {
     return string16();
   }
-  virtual GURL RenderSourceUrl() {
-    return GURL();
-  }
 };
 
 class TestPrintJobWorker : public printing::PrintJobWorker {
@@ -58,6 +55,7 @@ class TestOwner : public printing::PrintJobWorkerOwner {
   virtual int cookie() const {
     return 42;
   }
+
  private:
   printing::PrintSettings settings_;
 };
@@ -73,47 +71,43 @@ class TestPrintJob : public printing::PrintJob {
   volatile bool* check_;
 };
 
-class TestPrintNotifObserv : public NotificationObserver {
+class TestPrintNotifObserv : public content::NotificationObserver {
  public:
-  // NotificationObserver
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details) {
-    EXPECT_FALSE(true);
+  // content::NotificationObserver
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) {
+    ADD_FAILURE();
   }
 };
 
 }  // namespace
 
-// Crashes, Bug 55181.
-#if defined(OS_WIN)
-#define MAYBE_SimplePrint DISABLED_SimplePrint
-#else
-#define MAYBE_SimplePrint SimplePrint
-#endif
-TEST(PrintJobTest, MAYBE_SimplePrint) {
+typedef testing::Test PrintJobTest;
+
+TEST_F(PrintJobTest, SimplePrint) {
   // Test the multi-threaded nature of PrintJob to make sure we can use it with
   // known lifetime.
 
   // This message loop is actually never run.
   MessageLoop current;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
   TestPrintNotifObserv observ;
-  registrar_.Add(&observ, NotificationType::ALL,
-                 NotificationService::AllSources());
+  registrar_.Add(&observ, content::NOTIFICATION_ALL,
+                 content::NotificationService::AllSources());
   volatile bool check = false;
   scoped_refptr<printing::PrintJob> job(new TestPrintJob(&check));
   EXPECT_EQ(MessageLoop::current(), job->message_loop());
   scoped_refptr<TestOwner> owner(new TestOwner);
   TestSource source;
-  job->Initialize(owner, &source);
+  job->Initialize(owner, &source, 1);
   job->Stop();
   job = NULL;
   EXPECT_TRUE(check);
 }
 
-TEST(PrintJobTest, SimplePrintLateInit) {
+TEST_F(PrintJobTest, SimplePrintLateInit) {
   volatile bool check = false;
   MessageLoop current;
   scoped_refptr<printing::PrintJob> job(new TestPrintJob(&check));
@@ -135,7 +129,6 @@ TEST(PrintJobTest, SimplePrintLateInit) {
   job->FlushJob(timeout_ms);
   job->DisconnectSource();
   job->is_job_pending();
-  job->is_print_dialog_box_shown();
   job->document();
   // Private
   job->UpdatePrintedDocument(NULL);
