@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,13 +6,18 @@
 #define CHROME_BROWSER_PRINTING_PRINT_JOB_WORKER_H__
 #pragma once
 
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/task.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/threading/thread.h"
-#include "gfx/native_widget_types.h"
 #include "printing/page_number.h"
 #include "printing/printing_context.h"
+#include "printing/print_job_constants.h"
+#include "ui/gfx/native_widget_types.h"
+
+namespace base {
+class DictionaryValue;
+}
 
 namespace printing {
 
@@ -30,7 +35,7 @@ class PrintJobWorkerOwner;
 class PrintJobWorker : public base::Thread {
  public:
   explicit PrintJobWorker(PrintJobWorkerOwner* owner);
-  ~PrintJobWorker();
+  virtual ~PrintJobWorker();
 
   void SetNewOwner(PrintJobWorkerOwner* new_owner);
 
@@ -40,7 +45,11 @@ class PrintJobWorker : public base::Thread {
                    gfx::NativeView parent_view,
                    int document_page_count,
                    bool has_selection,
-                   bool use_overlays);
+                   MarginType margin_type);
+
+  // Set the new print settings. This function takes ownership of
+  // |new_settings|.
+  void SetSettings(const base::DictionaryValue* const new_settings);
 
   // Starts the printing loop. Every pages are printed as soon as the data is
   // available. Makes sure the new_document is the right one.
@@ -49,7 +58,7 @@ class PrintJobWorker : public base::Thread {
   // Updates the printed document.
   void OnDocumentChanged(PrintedDocument* new_document);
 
-  // Unqueues waiting pages. Called when PrintJob receives a
+  // Dequeues waiting pages. Called when PrintJob receives a
   // NOTIFY_PRINTED_DOCUMENT_UPDATED notification. It's time to look again if
   // the next page can be printed.
   void OnNewPage();
@@ -57,9 +66,6 @@ class PrintJobWorker : public base::Thread {
   // This is the only function that can be called in a thread.
   void Cancel();
 
-  // Cancels the Print... dialog box if shown, noop otherwise.
-  void DismissDialog();
-
  protected:
   // Retrieves the context for testing only.
   PrintingContext* printing_context() { return printing_context_.get(); }
@@ -71,10 +77,9 @@ class PrintJobWorker : public base::Thread {
   // notifications are sent this way, except USER_INIT_DONE, USER_INIT_CANCELED
   // and DEFAULT_INIT_DONE. These three are sent through PrintJob::InitDone().
   class NotificationTask;
-  friend struct RunnableMethodTraits<PrintJobWorker>;
 
   // Renders a page in the printer.
-  void SpoolPage(PrintedPage& page);
+  void SpoolPage(PrintedPage* page);
 
   // Closes the job since spooling is done.
   void OnDocumentDone();
@@ -95,6 +100,10 @@ class PrintJobWorker : public base::Thread {
   // back into the IO thread for GetSettingsDone().
   void GetSettingsWithUIDone(PrintingContext::Result result);
 
+  // Called on the UI thread to update the print settings. This function takes
+  // the ownership of |new_settings|.
+  void UpdatePrintSettings(const base::DictionaryValue* const new_settings);
+
   // Reports settings back to owner_.
   void GetSettingsDone(PrintingContext::Result result);
 
@@ -116,17 +125,12 @@ class PrintJobWorker : public base::Thread {
   // Current page number to print.
   PageNumber page_number_;
 
+  // Used to generate a WeakPtr for callbacks.
+  base::WeakPtrFactory<PrintJobWorker> weak_factory_;
+
   DISALLOW_COPY_AND_ASSIGN(PrintJobWorker);
 };
 
 }  // namespace printing
 
-template <>
-struct RunnableMethodTraits<printing::PrintJobWorker> {
-  void RetainCallee(printing::PrintJobWorker* obj);
-  void ReleaseCallee(printing::PrintJobWorker* obj);
- private:
-  scoped_refptr<printing::PrintJobWorkerOwner> owner_;
-};
-
 #endif  // CHROME_BROWSER_PRINTING_PRINT_JOB_WORKER_H__
