@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,52 +8,20 @@
 #define CHROME_BROWSER_PROFILES_PROFILE_H_
 #pragma once
 
+#include <string>
+
 #include "base/basictypes.h"
+#include "base/hash_tables.h"
 #include "base/logging.h"
-
-namespace base {
-class Time;
-}
-
-#if defined(OS_CHROMEOS)
-namespace chromeos {
-class EnterpriseExtensionObserver;
-class ProxyConfigServiceImpl;
-}
-#endif
-
-namespace history {
-class TopSites;
-}
-
-namespace net {
-class TransportSecurityState;
-class SSLConfigService;
-}
-
-namespace policy {
-class ProfilePolicyContext;
-}
-
-namespace webkit_database {
-class DatabaseTracker;
-}
-
-namespace fileapi {
-class SandboxedFileSystemContext;
-}
+#include "chrome/browser/net/preconnect.h" // TODO: remove this.
+#include "chrome/browser/net/pref_proxy_config_tracker.h"
+#include "chrome/common/extensions/extension_constants.h"
+#include "content/public/browser/browser_context.h"
 
 class AutocompleteClassifier;
-class BackgroundContentsService;
 class BookmarkModel;
-class BrowserSignin;
-class BrowserThemeProvider;
 class ChromeAppCacheService;
-class ChromeBlobStorageContext;
-class ChromeURLRequestContextGetter;
-class CloudPrintProxyService;
-class DesktopNotificationService;
-class DownloadManager;
+class ChromeURLDataManager;
 class Extension;
 class ExtensionDevToolsManager;
 class ExtensionEventRouter;
@@ -61,50 +29,68 @@ class ExtensionInfoMap;
 class ExtensionMessageService;
 class ExtensionProcessManager;
 class ExtensionService;
+class ExtensionSpecialStoragePolicy;
 class FaviconService;
-class FilePath;
-class FindBarState;
-class GeolocationContentSettingsMap;
-class GeolocationPermissionContext;
+class GAIAInfoUpdateService;
 class HistoryService;
 class HostContentSettingsMap;
-class HostZoomMap;
-class NTPResourceCache;
-class NavigationController;
 class PasswordStore;
-class PersonalDataManager;
-class PinnedTabService;
-class PrefProxyConfigTracker;
 class PrefService;
-class PrerenderManager;
-class ProfileSyncFactory;
-class ProfileSyncService;
 class ProfileSyncService;
 class PromoCounter;
-class PromoCounter;
-class SQLitePersistentCookieStore;
-class SSLConfigServiceManager;
-class SSLHostState;
-class SessionService;
-class SpellCheckHost;
-class StatusTray;
-class TabRestoreService;
+class ProtocolHandlerRegistry;
+class SpeechInputPreferences;
 class TemplateURLFetcher;
-class TemplateURLModel;
+class TestingProfile;
 class TokenService;
-class TransportSecurityPersister;
-class URLRequestContextGetter;
 class UserScriptMaster;
 class UserStyleSheetWatcher;
-class VisitedLinkEventListener;
 class VisitedLinkMaster;
 class WebDataService;
-class WebKitContext;
-class WebResourceService;
 
-typedef intptr_t ProfileId;
+namespace base {
+class Time;
+}
 
-class Profile {
+namespace chromeos {
+class LibCrosServiceLibraryImpl;
+class ResetDefaultProxyConfigServiceTask;
+}
+
+namespace chrome_browser_net {
+class Predictor;
+}
+
+namespace content {
+class WebUI;
+}
+
+namespace android {
+class TabContentsProvider;
+}
+
+namespace fileapi {
+class FileSystemContext;
+}
+
+namespace history {
+class TopSites;
+class ShortcutsBackend;
+}
+
+namespace net {
+class SSLConfigService;
+}
+
+#if !defined(OS_MACOSX) && !defined(OS_CHROMEOS) && defined(OS_POSIX)
+// Local profile ids are used to associate resources stored outside the profile
+// directory, like saved passwords in GNOME Keyring / KWallet, with a profile.
+// With high probability, they are unique on the local machine. They are almost
+// certainly not unique globally, by design. Do not send them over the network.
+typedef int LocalProfileId;
+#endif
+
+class Profile : public content::BrowserContext {
  public:
   // Profile services are accessed with the following parameter. This parameter
   // defines what the caller plans to do with the service.
@@ -118,7 +104,7 @@ class Profile {
   enum ServiceAccessType {
     // The caller plans to perform a read or write that takes place as a result
     // of the user input. Use this flag when the operation you are doing can be
-    // performed while off the record. (ex: creating a bookmark)
+    // performed while incognito. (ex: creating a bookmark)
     //
     // Since EXPLICIT_ACCESS means "as a result of a user action", this request
     // always succeeds.
@@ -127,12 +113,54 @@ class Profile {
     // The caller plans to call a method that will permanently change some data
     // in the profile, as part of Chrome's implicit data logging. Use this flag
     // when you are about to perform an operation which is incompatible with the
-    // off the record mode.
+    // incognito mode.
     IMPLICIT_ACCESS
   };
 
-  // Value that represents no profile Id.
-  static const ProfileId InvalidProfileId;
+  enum CreateStatus {
+    // Profile services were not created.
+    CREATE_STATUS_FAIL,
+    // Profile created but before initializing extensions and promo resources.
+    CREATE_STATUS_CREATED,
+    // Profile is created, extensions and promo resources are initialized.
+    CREATE_STATUS_INITIALIZED,
+  };
+
+  class Delegate {
+   public:
+    // Called when creation of the profile is finished.
+    virtual void OnProfileCreated(Profile* profile, bool success) = 0;
+  };
+
+  // Whitelist access to deprecated API in order to prevent new regressions.
+  class Deprecated {
+   private:
+    friend bool IsGoogleGAIACookieInstalled();
+
+    friend class AutofillDownloadManager;
+    friend class BrowserListTabContentsProvider;
+    friend class MetricsService;
+    friend class SafeBrowsingServiceTestHelper;
+    friend class SdchDictionaryFetcher;
+    friend class SyncTest;
+    friend class Toolbar5Importer;
+    friend class TranslateManager;
+    friend class android::TabContentsProvider;
+    friend class chromeos::LibCrosServiceLibraryImpl;
+    friend class chromeos::ResetDefaultProxyConfigServiceTask;
+
+    static net::URLRequestContextGetter* GetDefaultRequestContext() {
+      return Profile::GetDefaultRequestContext();
+    }
+  };
+
+  // Key used to bind profile to the widget with which it is associated.
+  static const char* const kProfileKey;
+
+#if !defined(OS_MACOSX) && !defined(OS_CHROMEOS) && defined(OS_POSIX)
+  // Value that represents no local profile id.
+  static const LocalProfileId kInvalidLocalProfileId;
+#endif
 
   Profile();
   virtual ~Profile() {}
@@ -144,43 +172,68 @@ class Profile {
   // Create a new profile given a path.
   static Profile* CreateProfile(const FilePath& path);
 
-  // Returns the request context for the "default" profile.  This may be called
-  // from any thread.  This CAN return NULL if a first request context has not
-  // yet been created.  If necessary, listen on the UI thread for
-  // NOTIFY_DEFAULT_REQUEST_CONTEXT_AVAILABLE.
-  static URLRequestContextGetter* GetDefaultRequestContext();
+  // Same as above, but uses async initialization.
+  static Profile* CreateProfileAsync(const FilePath& path,
+                                     Delegate* delegate);
 
-  // Returns a unique Id that can be used to identify this profile at runtime.
-  // This Id is not persistent and will not survive a restart of the browser.
-  virtual ProfileId GetRuntimeId() = 0;
+  // Returns the profile corresponding to the given browser context.
+  static Profile* FromBrowserContext(content::BrowserContext* browser_context);
+
+  // Returns the profile corresponding to the given WebUI.
+  static Profile* FromWebUI(content::WebUI* web_ui);
+
+  // content::BrowserContext implementation ------------------------------------
 
-  // Returns the path of the directory where this profile's data is stored.
   virtual FilePath GetPath() = 0;
+  virtual SSLHostState* GetSSLHostState() = 0;
+  virtual content::DownloadManager* GetDownloadManager() = 0;
+  virtual net::URLRequestContextGetter* GetRequestContext() = 0;
+  virtual net::URLRequestContextGetter* GetRequestContextForRenderProcess(
+      int renderer_child_id) = 0;
+  virtual net::URLRequestContextGetter* GetRequestContextForMedia() = 0;
+  virtual const content::ResourceContext& GetResourceContext() = 0;
+  virtual content::HostZoomMap* GetHostZoomMap() = 0;
+  virtual content::GeolocationPermissionContext*
+      GetGeolocationPermissionContext() = 0;
+  virtual SpeechInputPreferences* GetSpeechInputPreferences() = 0;
+  virtual quota::QuotaManager* GetQuotaManager() = 0;
+  virtual webkit_database::DatabaseTracker* GetDatabaseTracker() = 0;
+  virtual WebKitContext* GetWebKitContext() = 0;
+  virtual ChromeAppCacheService* GetAppCacheService() = 0;
+  virtual ChromeBlobStorageContext* GetBlobStorageContext() = 0;
+  virtual fileapi::FileSystemContext* GetFileSystemContext() = 0;
 
-  // Return whether this profile is off the record. Default is false.
+  // content::BrowserContext implementation ------------------------------------
+
+  // Typesafe upcast.
+  virtual TestingProfile* AsTestingProfile();
+
+  // Returns the name associated with this profile. This name is displayed in
+  // the browser frame.
+  virtual std::string GetProfileName() = 0;
+
+  // Return whether this profile is incognito. Default is false.
   virtual bool IsOffTheRecord() = 0;
 
-  // Return the off the record version of this profile. The returned pointer
+  // Return the incognito version of this profile. The returned pointer
   // is owned by the receiving profile. If the receiving profile is off the
   // record, the same profile is returned.
+  //
+  // WARNING: This will create the OffTheRecord profile if it doesn't already
+  // exist. If this isn't what you want, you need to check
+  // HasOffTheRecordProfile() first.
   virtual Profile* GetOffTheRecordProfile() = 0;
 
-  // Destroys the off the record profile.
+  // Destroys the incognito profile.
   virtual void DestroyOffTheRecordProfile() = 0;
 
-  // True if an off the record profile exists.
+  // True if an incognito profile exists.
   virtual bool HasOffTheRecordProfile() = 0;
 
   // Return the original "recording" profile. This method returns this if the
-  // profile is not off the record.
+  // profile is not incognito.
   virtual Profile* GetOriginalProfile() = 0;
 
-  // Returns a pointer to the ChromeAppCacheService instance for this profile.
-  virtual ChromeAppCacheService* GetAppCacheService() = 0;
-
-  // Returns a pointer to the DatabaseTracker instance for this profile.
-  virtual webkit_database::DatabaseTracker* GetDatabaseTracker() = 0;
-
   // Returns a pointer to the TopSites (thumbnail manager) instance
   // for this profile.
   virtual history::TopSites* GetTopSites() = 0;
@@ -217,15 +270,9 @@ class Profile {
   // Accessor. The instance is created at startup.
   virtual ExtensionEventRouter* GetExtensionEventRouter() = 0;
 
-  // Retrieves a pointer to the SSLHostState associated with this profile.
-  // The SSLHostState is lazily created the first time that this method is
-  // called.
-  virtual SSLHostState* GetSSLHostState() = 0;
-
-  // Retrieves a pointer to the TransportSecurityState associated with
-  // this profile.  The TransportSecurityState is lazily created the
-  // first time that this method is called.
-  virtual net::TransportSecurityState* GetTransportSecurityState() = 0;
+  // Accessor. The instance is created upon first access.
+  virtual ExtensionSpecialStoragePolicy*
+      GetExtensionSpecialStoragePolicy() = 0;
 
   // Retrieves a pointer to the FaviconService associated with this
   // profile.  The FaviconService is lazily created the first time
@@ -240,6 +287,9 @@ class Profile {
   // the ServiceAccessType definition above.
   virtual FaviconService* GetFaviconService(ServiceAccessType access) = 0;
 
+  // Accessor. The instance is created upon first access.
+  virtual GAIAInfoUpdateService* GetGAIAInfoUpdateService() = 0;
+
   // Retrieves a pointer to the HistoryService associated with this
   // profile.  The HistoryService is lazily created the first time
   // that this method is called.
@@ -262,6 +312,11 @@ class Profile {
   // this method is called.
   virtual AutocompleteClassifier* GetAutocompleteClassifier() = 0;
 
+  // Returns the ShortcutsBackend for this profile. This is owned by
+  // the Profile and created on the first call. Callers that outlive the life of
+  // this profile need to be sure they refcount the returned value.
+  virtual history::ShortcutsBackend* GetShortcutsBackend() = 0;
+
   // Returns the WebDataService for this profile. This is owned by
   // the Profile. Callers that outlive the life of this profile need to be
   // sure they refcount the returned value.
@@ -275,6 +330,8 @@ class Profile {
   virtual WebDataService* GetWebDataServiceWithoutCreating() = 0;
 
   // Returns the PasswordStore for this profile. This is owned by the Profile.
+  // This may return NULL if the implementation is unable to create a
+  // password store (e.g. a corrupt database).
   virtual PasswordStore* GetPasswordStore(ServiceAccessType access) = 0;
 
   // Retrieves a pointer to the PrefService that manages the preferences
@@ -282,60 +339,23 @@ class Profile {
   // time that this method is called.
   virtual PrefService* GetPrefs() = 0;
 
-  // Returns the TemplateURLModel for this profile. This is owned by the
-  // the Profile.
-  virtual TemplateURLModel* GetTemplateURLModel() = 0;
+  // Retrieves a pointer to the PrefService that manages the preferences
+  // for OffTheRecord Profiles.  This PrefService is lazily created the first
+  // time that this method is called.
+  virtual PrefService* GetOffTheRecordPrefs() = 0;
 
   // Returns the TemplateURLFetcher for this profile. This is owned by the
   // profile.
   virtual TemplateURLFetcher* GetTemplateURLFetcher() = 0;
 
-  // Returns the DownloadManager associated with this profile.
-  virtual DownloadManager* GetDownloadManager() = 0;
-  virtual bool HasCreatedDownloadManager() const = 0;
-
-  // Returns the PersonalDataManager associated with this profile.
-  virtual PersonalDataManager* GetPersonalDataManager() = 0;
-
-  // Returns the FileSystemContext associated to this profile.  The context
-  // is lazily created the first time this method is called.  This is owned
-  // by the profile.
-  virtual fileapi::SandboxedFileSystemContext* GetFileSystemContext() = 0;
-
-  // Returns the BrowserSignin object assigned to this profile.
-  virtual BrowserSignin* GetBrowserSignin() = 0;
-
-  // Init our themes system.
-  virtual void InitThemes() = 0;
-
-  // Set the theme to the specified extension.
-  virtual void SetTheme(const Extension* extension) = 0;
-
-  // Set the theme to the machine's native theme.
-  virtual void SetNativeTheme() = 0;
-
-  // Clear the theme and reset it to default.
-  virtual void ClearTheme() = 0;
-
-  // Gets the theme that was last set. Returns NULL if the theme is no longer
-  // installed, if there is no installed theme, or the theme was cleared.
-  virtual const Extension* GetTheme() = 0;
-
-  // Returns or creates the ThemeProvider associated with this profile
-  virtual BrowserThemeProvider* GetThemeProvider() = 0;
-
-  // Returns the request context information associated with this profile.  Call
-  // this only on the UI thread, since it can send notifications that should
-  // happen on the UI thread.
-  virtual URLRequestContextGetter* GetRequestContext() = 0;
-
-  // Returns the request context for media resources asociated with this
-  // profile.
-  virtual URLRequestContextGetter* GetRequestContextForMedia() = 0;
-
   // Returns the request context used for extension-related requests.  This
   // is only used for a separate cookie store currently.
-  virtual URLRequestContextGetter* GetRequestContextForExtensions() = 0;
+  virtual net::URLRequestContextGetter* GetRequestContextForExtensions() = 0;
+
+  // Returns the request context used within an installed app that has
+  // requested isolated storage.
+  virtual net::URLRequestContextGetter* GetRequestContextForIsolatedApp(
+      const std::string& app_id) = 0;
 
   // Called by the ExtensionService that lives in this profile. Gives the
   // profile a chance to react to the load event before the EXTENSION_LOADED
@@ -349,7 +369,8 @@ class Profile {
   // profile clean up its RequestContexts once all the listeners to the
   // EXTENSION_UNLOADED notification have finished running.
   virtual void UnregisterExtensionWithRequestContexts(
-      const Extension* extension) {}
+      const std::string& extension_id,
+      const extension_misc::UnloadedExtensionReason) {}
 
   // Returns the SSLConfigService for this profile.
   virtual net::SSLConfigService* GetSSLConfigService() = 0;
@@ -357,42 +378,12 @@ class Profile {
   // Returns the Hostname <-> Content settings map for this profile.
   virtual HostContentSettingsMap* GetHostContentSettingsMap() = 0;
 
-  // Returns the Hostname <-> Zoom Level map for this profile.
-  virtual HostZoomMap* GetHostZoomMap() = 0;
-
-  // Returns the geolocation settings map for this profile.
-  virtual GeolocationContentSettingsMap* GetGeolocationContentSettingsMap() = 0;
-
-  // Returns the geolocation permission context for this profile.
-  virtual GeolocationPermissionContext* GetGeolocationPermissionContext() = 0;
-
   // Returns the user style sheet watcher.
   virtual UserStyleSheetWatcher* GetUserStyleSheetWatcher() = 0;
 
-  // Returns the find bar state for this profile.  The find bar state is lazily
-  // created the first time that this method is called.
-  virtual FindBarState* GetFindBarState() = 0;
-
-  // Returns the session service for this profile. This may return NULL. If
-  // this profile supports a session service (it isn't off the record), and
-  // the session service hasn't yet been created, this forces creation of
-  // the session service.
-  //
-  // This returns NULL in two situations: the profile is off the record, or the
-  // session service has been explicitly shutdown (browser is exiting). Callers
-  // should always check the return value for NULL.
-  virtual SessionService* GetSessionService() = 0;
-
-  // If this profile has a session service, it is shut down. To properly record
-  // the current state this forces creation of the session service, then shuts
-  // it down.
-  virtual void ShutdownSessionService() = 0;
-
-  // Returns true if this profile has a session service.
-  virtual bool HasSessionService() const = 0;
-
   // Returns true if this profile has a profile sync service.
-  virtual bool HasProfileSyncService() const = 0;
+  // TODO(tim): Bug 93922 - remove this.
+  virtual bool HasProfileSyncService() = 0;
 
   // Returns true if the last time this profile was open it was exited cleanly.
   virtual bool DidLastSessionExitCleanly() = 0;
@@ -400,23 +391,19 @@ class Profile {
   // Returns the BookmarkModel, creating if not yet created.
   virtual BookmarkModel* GetBookmarkModel() = 0;
 
+  // Returns the ProtocolHandlerRegistry, creating if not yet created.
+  virtual ProtocolHandlerRegistry* GetProtocolHandlerRegistry() = 0;
+
   // Returns the Gaia Token Service, creating if not yet created.
   virtual TokenService* GetTokenService() = 0;
 
   // Returns the ProfileSyncService, creating if not yet created.
+  // TODO(tim): Bug 93922 - remove this.
   virtual ProfileSyncService* GetProfileSyncService() = 0;
 
-  // Returns the ProfileSyncService, creating if not yet created, with
-  // the specified CrOS username.
-  virtual ProfileSyncService* GetProfileSyncService(
-      const std::string& cros_user) = 0;
-
-  // Returns the CloudPrintProxyService, creating if not yet created.
-  virtual CloudPrintProxyService* GetCloudPrintProxyService() = 0;
-
   // Return whether 2 profiles are the same. 2 profiles are the same if they
   // represent the same profile. This can happen if there is pointer equality
-  // or if one profile is the off the record version of another profile (or vice
+  // or if one profile is the incognito version of another profile (or vice
   // versa).
   virtual bool IsSameProfile(Profile* profile) = 0;
 
@@ -426,90 +413,90 @@ class Profile {
   // the user started chrome.
   virtual base::Time GetStartTime() const = 0;
 
-  // Returns the TabRestoreService. This returns NULL when off the record.
-  virtual TabRestoreService* GetTabRestoreService() = 0;
-
-  virtual void ResetTabRestoreService() = 0;
-
-  // May return NULL.
-  virtual SpellCheckHost* GetSpellCheckHost() = 0;
-
-  // If |force| is false, and the spellchecker is already initialized (or is in
-  // the process of initializing), then do nothing. Otherwise clobber the
-  // current spellchecker and replace it with a new one.
-  virtual void ReinitializeSpellCheckHost(bool force) = 0;
-
-  // Returns the WebKitContext assigned to this profile.
-  virtual WebKitContext* GetWebKitContext() = 0;
-
-  // Returns the provider of desktop notifications for this profile.
-  virtual DesktopNotificationService* GetDesktopNotificationService() = 0;
-
-  // Returns the service that manages BackgroundContents for this profile.
-  virtual BackgroundContentsService* GetBackgroundContentsService() const = 0;
-
-  // Returns the StatusTray, which provides an API for displaying status icons
-  // in the system status tray. Returns NULL if status icons are not supported
-  // on this platform (or this is a unit test).
-  virtual StatusTray* GetStatusTray() = 0;
-
   // Marks the profile as cleanly shutdown.
   //
   // NOTE: this is invoked internally on a normal shutdown, but is public so
   // that it can be invoked when the user logs out/powers down (WM_ENDSESSION).
   virtual void MarkAsCleanShutdown() = 0;
 
-  virtual void InitExtensions() = 0;
+  // Initializes extensions machinery.
+  // Component extensions are always enabled, external and user extensions
+  // are controlled by |extensions_enabled|.
+  virtual void InitExtensions(bool extensions_enabled) = 0;
 
-  // Start up service that gathers data from a web resource feed.
-  virtual void InitWebResources() = 0;
+  // Start up service that gathers data from a promo resource feed.
+  virtual void InitPromoResources() = 0;
 
-  // Returns the new tab page resource cache.
-  virtual NTPResourceCache* GetNTPResourceCache() = 0;
+  // Register URLRequestFactories for protocols registered with
+  // registerProtocolHandler.
+  virtual void InitRegisteredProtocolHandlers() = 0;
 
   // Returns the last directory that was chosen for uploading or opening a file.
   virtual FilePath last_selected_directory() = 0;
   virtual void set_last_selected_directory(const FilePath& path) = 0;
 
-  // Returns a pointer to the ChromeBlobStorageContext instance for this
-  // profile.
-  virtual ChromeBlobStorageContext* GetBlobStorageContext() = 0;
-
   // Returns the IO-thread-accessible profile data for this profile.
   virtual ExtensionInfoMap* GetExtensionInfoMap() = 0;
 
   // Returns the PromoCounter for Instant, or NULL if not applicable.
   virtual PromoCounter* GetInstantPromoCounter() = 0;
 
-  // Gets the policy context associated with this profile.
-  virtual policy::ProfilePolicyContext* GetPolicyContext() = 0;
+  // Returns the ChromeURLDataManager for this profile.
+  virtual ChromeURLDataManager* GetChromeURLDataManager() = 0;
 
 #if defined(OS_CHROMEOS)
-  // Changes application locale.
-  // "Keep local" means that changes should not be propagated to other devices.
-  virtual void ChangeApplicationLocale(
-      const std::string& locale, bool keep_local) = 0;
+  enum AppLocaleChangedVia {
+    // Caused by chrome://settings change.
+    APP_LOCALE_CHANGED_VIA_SETTINGS,
+    // Locale has been reverted via LocaleChangeGuard.
+    APP_LOCALE_CHANGED_VIA_REVERT,
+    // From login screen.
+    APP_LOCALE_CHANGED_VIA_LOGIN,
+    // Source unknown.
+    APP_LOCALE_CHANGED_VIA_UNKNOWN
+  };
 
-  // Returns ChromeOS's ProxyConfigServiceImpl, creating if not yet created.
-  virtual chromeos::ProxyConfigServiceImpl*
-      GetChromeOSProxyConfigServiceImpl() = 0;
+  // Changes application locale for a profile.
+  virtual void ChangeAppLocale(
+      const std::string& locale, AppLocaleChangedVia via) = 0;
+
+  // Called after login.
+  virtual void OnLogin() = 0;
 
   // Creates ChromeOS's EnterpriseExtensionListener.
   virtual void SetupChromeOSEnterpriseExtensionObserver() = 0;
 
+  // Initializes Chrome OS's preferences.
+  virtual void InitChromeOSPreferences() = 0;
 #endif  // defined(OS_CHROMEOS)
 
   // Returns the helper object that provides the proxy configuration service
   // access to the the proxy configuration possibly defined by preferences.
   virtual PrefProxyConfigTracker* GetProxyConfigTracker() = 0;
 
-  // Returns the PrerenderManager used to prerender entire webpages for this
-  // profile.
-  virtual PrerenderManager* GetPrerenderManager() = 0;
+  // Returns the Predictor object used for dns prefetch.
+  virtual chrome_browser_net::Predictor* GetNetworkPredictor() = 0;
+
+  // Deletes all network related data since |time|. It deletes transport
+  // security state since |time| and it also delete HttpServerProperties data.
+  // The implementation is free to run this on a background thread, so when this
+  // method returns data is not guaranteed to be deleted.
+  virtual void ClearNetworkingHistorySince(base::Time time) = 0;
+
+  // Returns the home page for this profile.
+  virtual GURL GetHomePage() = 0;
+
+  // Makes the session state, e.g., cookies, persistent across the next restart.
+  virtual void SaveSessionState() {}
+
+  std::string GetDebugName();
+
+  // Returns whether it is a guest session.
+  static bool IsGuestSession();
 
 #ifdef UNIT_TEST
   // Use with caution.  GetDefaultRequestContext may be called on any thread!
-  static void set_default_request_context(URLRequestContextGetter* c) {
+  static void set_default_request_context(net::URLRequestContextGetter* c) {
     default_request_context_ = c;
   }
 #endif
@@ -546,9 +533,20 @@ class Profile {
   Profile* CreateOffTheRecordProfile();
 
  protected:
-  static URLRequestContextGetter* default_request_context_;
+  friend class OffTheRecordProfileImpl;
+
+  static net::URLRequestContextGetter* default_request_context_;
 
  private:
+  // ***DEPRECATED**: You should be passing in the specific profile's
+  // URLRequestContextGetter or using the system URLRequestContextGetter.
+  //
+  // Returns the request context for the "default" profile.  This may be called
+  // from any thread.  This CAN return NULL if a first request context has not
+  // yet been created.  If necessary, listen on the UI thread for
+  // NOTIFY_DEFAULT_REQUEST_CONTEXT_AVAILABLE.
+  static net::URLRequestContextGetter* GetDefaultRequestContext();
+
   bool restored_last_session_;
 
   // Accessibility events will only be propagated when the pause
@@ -558,4 +556,17 @@ class Profile {
   int accessibility_pause_level_;
 };
 
+#if defined(COMPILER_GCC)
+namespace BASE_HASH_NAMESPACE {
+
+template<>
+struct hash<Profile*> {
+  std::size_t operator()(Profile* const& p) const {
+    return reinterpret_cast<std::size_t>(p);
+  }
+};
+
+}  // namespace BASE_HASH_NAMESPACE
+#endif
+
 #endif  // CHROME_BROWSER_PROFILES_PROFILE_H_
