@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,11 +8,12 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "chrome/browser/chromeos/offline/offline_load_page.h"
-#include "chrome/browser/renderer_host/resource_handler.h"
+#include "content/browser/renderer_host/resource_handler.h"
+#include "net/base/completion_callback.h"
 
-class MessageLoop;
+class ChromeAppCacheService;
 class ResourceDispatcherHost;
 
 namespace net {
@@ -20,46 +21,49 @@ class URLRequest;
 }  // namespace net
 
 // Used to show an offline interstitial page when the network is not available.
-class OfflineResourceHandler : public ResourceHandler,
-                               public chromeos::OfflineLoadPage::Delegate {
+class OfflineResourceHandler : public ResourceHandler {
  public:
   OfflineResourceHandler(ResourceHandler* handler,
                          int host_id,
                          int render_view_id,
                          ResourceDispatcherHost* rdh,
-                         net::URLRequest* request);
-  ~OfflineResourceHandler() {}
+                         net::URLRequest* request,
+                         ChromeAppCacheService* appcache_service);
+  virtual ~OfflineResourceHandler();
 
   // ResourceHandler implementation:
-  virtual bool OnUploadProgress(int request_id, uint64 position, uint64 size);
+  virtual bool OnUploadProgress(int request_id, uint64 position,
+      uint64 size) OVERRIDE;
   virtual bool OnRequestRedirected(int request_id, const GURL& new_url,
-                           ResourceResponse* response, bool* defer);
-  virtual bool OnResponseStarted(int request_id, ResourceResponse* response);
-  virtual bool OnWillStart(int request_id, const GURL& url, bool* defer);
+      content::ResourceResponse* response, bool* defer) OVERRIDE;
+  virtual bool OnResponseStarted(int request_id,
+      content::ResourceResponse* response) OVERRIDE;
+  virtual bool OnWillStart(int request_id, const GURL& url,
+      bool* defer) OVERRIDE;
   virtual bool OnWillRead(int request_id, net::IOBuffer** buf, int* buf_size,
-                          int min_size);
-  virtual bool OnReadCompleted(int request_id, int* bytes_read);
+      int min_size) OVERRIDE;
+  virtual bool OnReadCompleted(int request_id, int* bytes_read) OVERRIDE;
   virtual bool OnResponseCompleted(int request_id,
                                    const net::URLRequestStatus& status,
-                                   const std::string& security_info);
-  virtual void OnRequestClosed();
-
-  // chromeos::OfflineLoadPage::Delegate
-  virtual void OnBlockingPageComplete(bool proceed);
+                                   const std::string& security_info) OVERRIDE;
+  virtual void OnRequestClosed() OVERRIDE;
 
  private:
-  // Erease the state assocaited with a deferred load request.
+  // OfflineLoadPage callback.
+  void OnBlockingPageComplete(bool proceed);
+
+  // Erase the state associated with a deferred load request.
   void ClearRequestInfo();
   bool IsRemote(const GURL& url) const;
 
   // Resume the deferred load request.
   void Resume();
 
-  // Tells if chrome should show the offline page.
+  // True if chrome should show the offline page.
   bool ShouldShowOfflinePage(const GURL& url) const;
 
-  // Shows the offline interstitinal page in UI thread.
-  void ShowOfflinePage();
+  // A callback to tell if an appcache exists.
+  void OnCanHandleOfflineComplete(int rv);
 
   scoped_refptr<ResourceHandler> next_handler_;
 
@@ -67,11 +71,14 @@ class OfflineResourceHandler : public ResourceHandler,
   int render_view_id_;
   ResourceDispatcherHost* rdh_;
   net::URLRequest* request_;
+  ChromeAppCacheService* const appcache_service_;
 
   // The state for deferred load quest.
   int deferred_request_id_;
   GURL deferred_url_;
 
+  net::CancelableCompletionCallback appcache_completion_callback_;
+
   DISALLOW_COPY_AND_ASSIGN(OfflineResourceHandler);
 };
 
