@@ -1,16 +1,23 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/renderer_host/safe_browsing_resource_handler.h"
 
 #include "base/logging.h"
-#include "chrome/browser/renderer_host/global_request_id.h"
-#include "chrome/browser/renderer_host/resource_dispatcher_host.h"
-#include "chrome/browser/renderer_host/resource_message_filter.h"
-#include "chrome/common/resource_response.h"
-#include "net/base/net_errors.h"
+#include "chrome/browser/browser_process.h"
+#include "chrome/browser/prerender/prerender_final_status.h"
+#include "chrome/browser/prerender/prerender_tracker.h"
+#include "chrome/browser/renderer_host/chrome_url_request_user_data.h"
+#include "content/browser/renderer_host/resource_dispatcher_host.h"
+#include "content/public/browser/global_request_id.h"
+#include "content/public/common/resource_response.h"
 #include "net/base/io_buffer.h"
+#include "net/base/load_flags.h"
+#include "net/base/net_errors.h"
+#include "net/url_request/url_request.h"
+
+using content::GlobalRequestID;
 
 // Maximum time in milliseconds to wait for the safe browsing service to
 // verify a URL. After this amount of time the outstanding check will be
@@ -20,22 +27,36 @@ static const int kCheckUrlTimeoutMs = 5000;
 // TODO(eroman): Downgrade these CHECK()s to DCHECKs once there is more
 //               unit test coverage.
 
+// static
+SafeBrowsingResourceHandler* SafeBrowsingResourceHandler::Create(
+    ResourceHandler* handler,
+    int render_process_host_id,
+    int render_view_id,
+    bool is_subresource,
+    SafeBrowsingService* safe_browsing,
+    ResourceDispatcherHost* resource_dispatcher_host) {
+  return new SafeBrowsingResourceHandler(
+      handler, render_process_host_id, render_view_id,
+      is_subresource, safe_browsing, resource_dispatcher_host);
+}
+
 SafeBrowsingResourceHandler::SafeBrowsingResourceHandler(
     ResourceHandler* handler,
     int render_process_host_id,
     int render_view_id,
-    ResourceType::Type resource_type,
+    bool is_subresource,
     SafeBrowsingService* safe_browsing,
     ResourceDispatcherHost* resource_dispatcher_host)
     : state_(STATE_NONE),
       defer_state_(DEFERRED_NONE),
+      safe_browsing_result_(SafeBrowsingService::SAFE),
       deferred_request_id_(-1),
       next_handler_(handler),
       render_process_host_id_(render_process_host_id),
       render_view_id_(render_view_id),
       safe_browsing_(safe_browsing),
       rdh_(resource_dispatcher_host),
-      resource_type_(resource_type) {
+      is_subresource_(is_subresource) {
 }
 
 SafeBrowsingResourceHandler::~SafeBrowsingResourceHandler() {
@@ -50,11 +71,14 @@ bool SafeBrowsingResourceHandler::OnUploadProgress(int request_id,
 bool SafeBrowsingResourceHandler::OnRequestRedirected(
     int request_id,
     const GURL& new_url,
-    ResourceResponse* response,
+    content::ResourceResponse* response,
     bool* defer) {
   CHECK(state_ == STATE_NONE);
   CHECK(defer_state_ == DEFERRED_NONE);
 
+  // Save the redirect urls for possible malware detail reporting later.
+  redirect_urls_.push_back(new_url);
+
   // We need to check the new URL before following the redirect.
   if (CheckUrl(new_url)) {
     return next_handler_->OnRequestRedirected(
@@ -75,7 +99,7 @@ bool SafeBrowsingResourceHandler::OnRequestRedirected(
 }
 
 bool SafeBrowsingResourceHandler::OnResponseStarted(
-    int request_id, ResourceResponse* response) {
+    int request_id, content::ResourceResponse* response) {
   CHECK(state_ == STATE_NONE);
   CHECK(defer_state_ == DEFERRED_NONE);
   return next_handler_->OnResponseStarted(request_id, response);
@@ -85,7 +109,7 @@ void SafeBrowsingResourceHandler::OnCheckUrlTimeout() {
   CHECK(state_ == STATE_CHECKING_URL);
   CHECK(defer_state_ != DEFERRED_NONE);
   safe_browsing_->CancelCheck(this);
-  OnBrowseUrlCheckResult(deferred_url_, SafeBrowsingService::URL_SAFE);
+  OnBrowseUrlCheckResult(deferred_url_, SafeBrowsingService::SAFE);
 }
 
 bool SafeBrowsingResourceHandler::OnWillStart(int request_id,
@@ -145,7 +169,7 @@ void SafeBrowsingResourceHandler::OnBrowseUrlCheckResult(
   safe_browsing_result_ = result;
   state_ = STATE_NONE;
 
-  if (result == SafeBrowsingService::URL_SAFE) {
+  if (result == SafeBrowsingService::SAFE) {
     // Log how much time the safe browsing check cost us.
     base::TimeDelta pause_delta;
     pause_delta = base::TimeTicks::Now() - url_check_start_time_;
@@ -154,7 +178,33 @@ void SafeBrowsingResourceHandler::OnBrowseUrlCheckResult(
     // Continue the request.
     ResumeRequest();
   } else {
-    StartDisplayingBlockingPage(url, result);
+    bool should_show_blocking_page = true;
+    const net::URLRequest* request = rdh_->GetURLRequest(
+        GlobalRequestID(render_process_host_id_, deferred_request_id_));
+    if (request->load_flags() & net::LOAD_PREFETCH) {
+      // Don't prefetch resources that fail safe browsing, disallow
+      // them.
+      rdh_->CancelRequest(render_process_host_id_, deferred_request_id_, false);
+      should_show_blocking_page = false;
+    } else {
+      ChromeURLRequestUserData* user_data =
+          ChromeURLRequestUserData::Get(request);
+      if (user_data && user_data->is_prerender()) {
+        prerender::PrerenderTracker* prerender_tracker = g_browser_process->
+            prerender_tracker();
+        if (prerender_tracker->TryCancelOnIOThread(
+                render_process_host_id_,
+                render_view_id_,
+                prerender::FINAL_STATUS_SAFE_BROWSING)) {
+          rdh_->CancelRequest(render_process_host_id_, deferred_request_id_,
+                              false);
+          should_show_blocking_page = false;
+        }
+      }
+    }
+    if (should_show_blocking_page) {
+      StartDisplayingBlockingPage(url, result);
+    }
   }
 
   Release();  // Balances the AddRef() in CheckingUrl().
@@ -168,7 +218,6 @@ void SafeBrowsingResourceHandler::StartDisplayingBlockingPage(
   CHECK(deferred_request_id_ != -1);
 
   state_ = STATE_DISPLAYING_BLOCKING_PAGE;
-  AddRef();  // Balanced in OnBlockingPageComplete().
 
   // Grab the original url of this request as well.
   GURL original_url;
@@ -180,7 +229,13 @@ void SafeBrowsingResourceHandler::StartDisplayingBlockingPage(
     original_url = url;
 
   safe_browsing_->DisplayBlockingPage(
-      url, original_url, resource_type_, result, this, render_process_host_id_,
+      url,
+      original_url,
+      redirect_urls_,
+      is_subresource_,
+      result,
+      base::Bind(&SafeBrowsingResourceHandler::OnBlockingPageComplete, this),
+      render_process_host_id_,
       render_view_id_);
 }
 
@@ -191,13 +246,17 @@ void SafeBrowsingResourceHandler::OnBlockingPageComplete(bool proceed) {
   state_ = STATE_NONE;
 
   if (proceed) {
-    safe_browsing_result_ = SafeBrowsingService::URL_SAFE;
-    ResumeRequest();
+    safe_browsing_result_ = SafeBrowsingService::SAFE;
+    net::URLRequest* request = rdh_->GetURLRequest(
+        GlobalRequestID(render_process_host_id_, deferred_request_id_));
+
+    // The request could be canceled by renderer at this stage.
+    // As a result, click proceed will do nothing (crbug.com/76460).
+    if (request)
+      ResumeRequest();
   } else {
     rdh_->CancelRequest(render_process_host_id_, deferred_request_id_, false);
   }
-
-  Release();  // Balances the AddRef() in StartDisplayingBlockingPage().
 }
 
 void SafeBrowsingResourceHandler::Shutdown() {
@@ -215,7 +274,7 @@ bool SafeBrowsingResourceHandler::CheckUrl(const GURL& url) {
   CHECK(state_ == STATE_NONE);
   bool succeeded_synchronously = safe_browsing_->CheckBrowseUrl(url, this);
   if (succeeded_synchronously) {
-    safe_browsing_result_ = SafeBrowsingService::URL_SAFE;
+    safe_browsing_result_ = SafeBrowsingService::SAFE;
     safe_browsing_->LogPauseDelay(base::TimeDelta());  // No delay.
     return true;
   }
@@ -227,7 +286,8 @@ bool SafeBrowsingResourceHandler::CheckUrl(const GURL& url) {
   url_check_start_time_ = base::TimeTicks::Now();
 
   // Start a timer to abort the check if it takes too long.
-  timer_.Start(base::TimeDelta::FromMilliseconds(kCheckUrlTimeoutMs),
+  timer_.Start(FROM_HERE,
+               base::TimeDelta::FromMilliseconds(kCheckUrlTimeoutMs),
                this, &SafeBrowsingResourceHandler::OnCheckUrlTimeout);
 
   return false;
@@ -278,7 +338,7 @@ void SafeBrowsingResourceHandler::ResumeRedirect() {
   // Retrieve the details for the paused OnReceivedRedirect().
   int request_id = deferred_request_id_;
   GURL redirect_url = deferred_url_;
-  scoped_refptr<ResourceResponse> redirect_response =
+  scoped_refptr<content::ResourceResponse> redirect_response =
       deferred_redirect_response_;
 
   ClearDeferredRequestInfo();
