@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,12 +7,13 @@
 #pragma once
 
 #include <string>
+#include <vector>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/time.h"
 #include "base/timer.h"
-#include "chrome/browser/renderer_host/resource_handler.h"
 #include "chrome/browser/safe_browsing/safe_browsing_service.h"
+#include "content/browser/renderer_host/resource_handler.h"
 
 class ResourceDispatcherHost;
 
@@ -41,35 +42,37 @@ class ResourceDispatcherHost;
 class SafeBrowsingResourceHandler : public ResourceHandler,
                                     public SafeBrowsingService::Client {
  public:
-  SafeBrowsingResourceHandler(ResourceHandler* handler,
-                              int render_process_host_id,
-                              int render_view_id,
-                              ResourceType::Type resource_type,
-                              SafeBrowsingService* safe_browsing,
-                              ResourceDispatcherHost* resource_dispatcher_host);
+  static SafeBrowsingResourceHandler* Create(
+      ResourceHandler* handler,
+      int render_process_host_id,
+      int render_view_id,
+      bool is_subresource,
+      SafeBrowsingService* safe_browsing,
+      ResourceDispatcherHost* resource_dispatcher_host);
 
   // ResourceHandler implementation:
-  virtual bool OnUploadProgress(int request_id, uint64 position, uint64 size);
-  virtual bool OnRequestRedirected(int request_id, const GURL& new_url,
-                                   ResourceResponse* response, bool* defer);
-  virtual bool OnResponseStarted(int request_id, ResourceResponse* response);
-  virtual bool OnWillStart(int request_id, const GURL& url, bool* defer);
-  virtual bool OnWillRead(int request_id, net::IOBuffer** buf, int* buf_size,
-                          int min_size);
-  virtual bool OnReadCompleted(int request_id, int* bytes_read);
+  virtual bool OnUploadProgress(
+      int request_id, uint64 position, uint64 size) OVERRIDE;
+  virtual bool OnRequestRedirected(
+      int request_id, const GURL& new_url, content::ResourceResponse* response,
+      bool* defer) OVERRIDE;
+  virtual bool OnResponseStarted(
+      int request_id, content::ResourceResponse* response) OVERRIDE;
+  virtual bool OnWillStart(
+      int request_id, const GURL& url, bool* defer) OVERRIDE;
+  virtual bool OnWillRead(
+      int request_id, net::IOBuffer** buf, int* buf_size,
+      int min_size) OVERRIDE;
+  virtual bool OnReadCompleted(int request_id, int* bytes_read) OVERRIDE;
   virtual bool OnResponseCompleted(int request_id,
                                    const net::URLRequestStatus& status,
-                                   const std::string& security_info);
-  virtual void OnRequestClosed();
+                                   const std::string& security_info) OVERRIDE;
+  virtual void OnRequestClosed() OVERRIDE;
 
   // SafeBrowsingService::Client implementation, called on the IO thread once
   // the URL has been classified.
   virtual void OnBrowseUrlCheckResult(
-      const GURL& url, SafeBrowsingService::UrlCheckResult result);
-
-  // SafeBrowsingService::Client implementation, called on the IO thread when
-  // the user has decided to proceed with the current request, or go back.
-  virtual void OnBlockingPageComplete(bool proceed);
+      const GURL& url, SafeBrowsingService::UrlCheckResult result) OVERRIDE;
 
  private:
   // Describes what phase of the check a handler is in.
@@ -86,7 +89,14 @@ class SafeBrowsingResourceHandler : public ResourceHandler,
     DEFERRED_REDIRECT,
   };
 
-  ~SafeBrowsingResourceHandler();
+  SafeBrowsingResourceHandler(ResourceHandler* handler,
+                              int render_process_host_id,
+                              int render_view_id,
+                              bool is_subresource,
+                              SafeBrowsingService* safe_browsing,
+                              ResourceDispatcherHost* resource_dispatcher_host);
+
+  virtual ~SafeBrowsingResourceHandler();
 
   // Cancels any in progress safe browsing actions.
   void Shutdown();
@@ -104,6 +114,10 @@ class SafeBrowsingResourceHandler : public ResourceHandler,
   void StartDisplayingBlockingPage(const GURL& url,
                                    SafeBrowsingService::UrlCheckResult result);
 
+  // Called on the IO thread when the user has decided to proceed with the
+  // current request, or go back.
+  void OnBlockingPageComplete(bool proceed);
+
   // Resumes the request, by continuing the deferred action (either starting the
   // request, or following a redirect).
   void ResumeRequest();
@@ -131,18 +145,21 @@ class SafeBrowsingResourceHandler : public ResourceHandler,
   // Timer to abort the safe browsing check if it takes too long.
   base::OneShotTimer<SafeBrowsingResourceHandler> timer_;
 
+  // The redirect chain for this resource
+  std::vector<GURL> redirect_urls_;
+
   // Details on the deferred request (either a start or redirect). It is only
   // valid to access these members when defer_state_ != DEFERRED_NONE.
   GURL deferred_url_;
   int deferred_request_id_;
-  scoped_refptr<ResourceResponse> deferred_redirect_response_;
+  scoped_refptr<content::ResourceResponse> deferred_redirect_response_;
 
   scoped_refptr<ResourceHandler> next_handler_;
   int render_process_host_id_;
   int render_view_id_;
   scoped_refptr<SafeBrowsingService> safe_browsing_;
   ResourceDispatcherHost* rdh_;
-  ResourceType::Type resource_type_;
+  bool is_subresource_;
 
   DISALLOW_COPY_AND_ASSIGN(SafeBrowsingResourceHandler);
 };
