@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,7 +15,8 @@ cr.define('options', function() {
    * Encapsulated handling of about page.
    */
   function AboutPage() {
-    OptionsPage.call(this, 'about', templateData.aboutPage, 'aboutPage');
+    OptionsPage.call(this, 'about', templateData.aboutPageTabTitle,
+                     'aboutPage');
   }
 
   cr.addSingletonGetter(AboutPage);
@@ -37,6 +38,12 @@ cr.define('options', function() {
      */
     statusMessageQueueFlushStarted_: false,
 
+    /**
+     * The selected release channel.
+     * @type {string}
+     */
+    selectedChannel_: '',
+
     // Initialize AboutPage.
     initializePage: function() {
       // Call base class implementation to start preference initialization.
@@ -47,17 +54,13 @@ cr.define('options', function() {
       };
 
       $('moreInfoButton').onclick = function(event) {
-        $('aboutPageLessInfo').classList.add('hidden');
-        $('aboutPageMoreInfo').classList.remove('hidden');
+        $('aboutPageLessInfo').hidden = true;
+        $('aboutPageMoreInfo').hidden = false;
       };
 
-      if (cr.commandLine.options['--bwsi']) {
-        $('channelSelect').disabled = true;
-      } else {
-        $('channelSelect').onchange = function(event) {
-          var channel = event.target.value;
-          chrome.send('SetReleaseTrack', [channel]);
-        };
+      var self = this;
+      $('channelSelect').onchange = function(event) {
+        self.channelSelectOnChanged_(event.target.value);
       }
 
       // Notify the handler that the page is ready.
@@ -70,6 +73,11 @@ cr.define('options', function() {
       $('osVersion1').textContent = versionString;
     },
 
+    updateOSFirmware_: function(firmwareString) {
+      $('osFirmware0').textContent = firmwareString;
+      $('osFirmware1').textContent = firmwareString;
+    },
+
     /**
      * Updates the status message like "Checking for update...".
      * @param {string} message The message to be shown.
@@ -116,6 +124,45 @@ cr.define('options', function() {
       $('checkNow').disabled = !enable;
     },
 
+    enableReleaseChannel_: function(enable) {
+      $('channelSelect').disabled = !enable;
+    },
+
+    setReleaseChannel_: function(channel) {
+      // Write the value into the pref which will end up in the policy.
+      // Eventually, the update engine will use the policy value as the
+      // source truth for the update channel (see http://crosbug/17015).
+      Preferences.setStringPref("cros.system.releaseChannel", channel);
+      this.selectedChannel_ = channel;
+      chrome.send('SetReleaseTrack', [channel]);
+    },
+
+    // This function is called when the user changes the release channel from
+    // the 'channelSelect' <select> element. It either calls back into Chrome to
+    // switch the channel or displays a confirmation box if switching to dev.
+    channelSelectOnChanged_: function(value) {
+      if (value == 'dev-channel') {
+        // Open confirm dialog.
+        var self = this;
+        AlertOverlay.show(
+          localStrings.getString('channel_warning_header'),
+          localStrings.getString('channel_warning_text'),
+          localStrings.getString('ok'),
+          localStrings.getString('cancel'),
+          function() {
+            // Ok, so set release track and update selected channel.
+            $('channelWarningBlock').hidden = false;
+            self.setReleaseChannel_(value); },
+          function() {
+            // Cancel, so switch back to previous selected channel.
+            self.updateSelectedOption_(self.selectedChannel_); }
+          );
+      } else {
+        $('channelWarningBlock').hidden = true;
+        this.setReleaseChannel_(value);
+      }
+    },
+
     // Updates the selected option in 'channelSelect' <select> element.
     updateSelectedOption_: function(value) {
       var options = $('channelSelect').querySelectorAll('option');
@@ -123,8 +170,11 @@ cr.define('options', function() {
         var option = options[i];
         if (option.value == value) {
           option.selected = true;
+          this.selectedChannel_ = value;
         }
       }
+      if (value == 'dev-channel')
+        $('channelWarningBlock').hidden = false;
     },
 
     // Changes the "check now" button to "restart now" button.
@@ -141,6 +191,10 @@ cr.define('options', function() {
     AboutPage.getInstance().updateOSVersion_(versionString);
   };
 
+  AboutPage.updateOSFirmwareCallback = function(firmwareString) {
+    AboutPage.getInstance().updateOSFirmware_(firmwareString);
+  };
+
   AboutPage.updateStatusCallback = function(message, insertDelay) {
     AboutPage.getInstance().updateStatus_(message, insertDelay);
   };
@@ -149,6 +203,10 @@ cr.define('options', function() {
     AboutPage.getInstance().updateEnable_(enable);
   };
 
+  AboutPage.updateEnableReleaseChannelCallback = function(enable) {
+    AboutPage.getInstance().enableReleaseChannel_(enable);
+  };
+
   AboutPage.updateSelectedOptionCallback = function(value) {
     AboutPage.getInstance().updateSelectedOption_(value);
   };
