@@ -11,7 +11,7 @@ var OptionsPage = options.OptionsPage;
   // Encapsulated handling of advanced options page.
   //
   function AdvancedOptions() {
-    OptionsPage.call(this, 'advanced', templateData.advancedPage,
+    OptionsPage.call(this, 'advanced', templateData.advancedPageTabTitle,
                      'advancedPage');
   }
 
@@ -30,13 +30,13 @@ var OptionsPage = options.OptionsPage;
 
       // Set up click handlers for buttons.
       $('privacyContentSettingsButton').onclick = function(event) {
-        OptionsPage.showPageByName('content');
+        OptionsPage.navigateToPage('content');
         OptionsPage.showTab($('cookies-nav-tab'));
         chrome.send('coreOptionsUserMetricsAction',
             ['Options_ContentSettings']);
       };
       $('privacyClearDataButton').onclick = function(event) {
-        OptionsPage.showOverlay('clearBrowserDataOverlay');
+        OptionsPage.navigateToPage('clearBrowserData');
         chrome.send('coreOptionsUserMetricsAction', ['Options_ClearData']);
       };
 
@@ -56,15 +56,20 @@ var OptionsPage = options.OptionsPage;
       }
 
       $('fontSettingsCustomizeFontsButton').onclick = function(event) {
-        OptionsPage.showPageByName('fontSettings');
+        OptionsPage.navigateToPage('fonts');
         chrome.send('coreOptionsUserMetricsAction', ['Options_FontSettings']);
       };
       $('defaultFontSize').onchange = function(event) {
         chrome.send('defaultFontSizeAction',
             [String(event.target.options[event.target.selectedIndex].value)]);
       };
+      $('defaultZoomFactor').onchange = function(event) {
+        chrome.send('defaultZoomFactorAction',
+            [String(event.target.options[event.target.selectedIndex].value)]);
+      };
+
       $('language-button').onclick = function(event) {
-        OptionsPage.showPageByName('language');
+        OptionsPage.navigateToPage('languages');
         chrome.send('coreOptionsUserMetricsAction',
             ['Options_LanuageAndSpellCheckSettings']);
       };
@@ -75,8 +80,7 @@ var OptionsPage = options.OptionsPage;
         };
       } else {
         $('certificatesManageButton').onclick = function(event) {
-          OptionsPage.showPageByName('certificateManager');
-          OptionsPage.showTab($('personal-certs-nav-tab'));
+          OptionsPage.navigateToPage('certificates');
           chrome.send('coreOptionsUserMetricsAction',
                       ['Options_ManageSSLCertificates']);
         };
@@ -89,65 +93,44 @@ var OptionsPage = options.OptionsPage;
         $('downloadLocationChangeButton').onclick = function(event) {
           chrome.send('selectDownloadLocation');
         };
-
-        // Remove Windows-style accelerators from the Browse button label.
-        // TODO(csilv): Remove this after the accelerator has been removed from
-        // the localized strings file, pending removal of old options window.
-        $('downloadLocationChangeButton').textContent =
-            localStrings.getStringWithoutAccelerator(
-                'downloadLocationChangeButton');
-      } else {
-        $('proxiesConfigureButton').onclick = function(event) {
-          OptionsPage.showPageByName('proxy');
-          chrome.send('coreOptionsUserMetricsAction',
-              ['Options_ShowProxySettings']);
-        };
+        // This text field is always disabled. Setting ".disabled = true" isn't
+        // enough, since a policy can disable it but shouldn't re-enable when
+        // it is removed.
+        $('downloadLocationPath').setDisabled('readonly', true);
       }
 
-      if (cr.isWindows) {
-        $('sslCheckRevocation').onclick = function(event) {
-          chrome.send('checkRevocationCheckboxAction',
-              [String($('sslCheckRevocation').checked)]);
-        };
-        $('sslUseSSL3').onclick = function(event) {
-          chrome.send('useSSL3CheckboxAction',
-              [String($('sslUseSSL3').checked)]);
-        };
-        $('sslUseTLS1').onclick = function(event) {
-          chrome.send('useTLS1CheckboxAction',
-              [String($('sslUseTLS1').checked)]);
-        };
-        $('gearSettingsConfigureGearsButton').onclick = function(event) {
-          chrome.send('showGearsSettings');
+      $('sslCheckRevocation').onclick = function(event) {
+        chrome.send('checkRevocationCheckboxAction',
+            [String($('sslCheckRevocation').checked)]);
+      };
+
+      if ($('backgroundModeCheckbox')) {
+        $('backgroundModeCheckbox').onclick = function(event) {
+          chrome.send('backgroundModeAction',
+              [String($('backgroundModeCheckbox').checked)]);
         };
       }
 
       // 'cloudPrintProxyEnabled' is true for Chrome branded builds on
       // certain platforms, or could be enabled by a lab.
-      if (!cr.isChromeOS &&
-          localStrings.getString('enable-cloud-print-proxy') == 'true') {
-        $('cloudPrintProxySetupButton').onclick = function(event) {
-          if ($('cloudPrintProxyManageButton').style.display == 'none') {
+      if (!cr.isChromeOS) {
+        $('cloudPrintConnectorSetupButton').onclick = function(event) {
+          if ($('cloudPrintManageButton').style.display == 'none') {
             // Disable the button, set it's text to the intermediate state.
-            $('cloudPrintProxySetupButton').textContent =
-              localStrings.getString('cloudPrintProxyEnablingButton');
-            $('cloudPrintProxySetupButton').disabled = true;
+            $('cloudPrintConnectorSetupButton').textContent =
+              localStrings.getString('cloudPrintConnectorEnablingButton');
+            $('cloudPrintConnectorSetupButton').disabled = true;
             chrome.send('showCloudPrintSetupDialog');
           } else {
-            chrome.send('disableCloudPrintProxy');
+            chrome.send('disableCloudPrintConnector');
           }
         };
-        $('cloudPrintProxyManageButton').onclick = function(event) {
-          chrome.send('showCloudPrintManagePage');
-        };
       }
+      $('cloudPrintManageButton').onclick = function(event) {
+        chrome.send('showCloudPrintManagePage');
+      };
 
-      if ($('remotingSetupButton')) {
-        $('remotingSetupButton').onclick = function(event) {
-          chrome.send('showRemotingSetupDialog');
-        }
-      }
-    }
+  }
   };
 
   //
@@ -172,17 +155,14 @@ var OptionsPage = options.OptionsPage;
   }
 
   // Set the font size selected item.
-  AdvancedOptions.SetFontSize = function(fixed_font_size_value,
-      font_size_value) {
+  AdvancedOptions.SetFontSize = function(font_size_value) {
     var selectCtl = $('defaultFontSize');
-    if (fixed_font_size_value == font_size_value) {
-      for (var i = 0; i < selectCtl.options.length; i++) {
-        if (selectCtl.options[i].value == font_size_value) {
-          selectCtl.selectedIndex = i;
-          if ($('Custom'))
-            selectCtl.remove($('Custom').index);
-          return;
-        }
+    for (var i = 0; i < selectCtl.options.length; i++) {
+      if (selectCtl.options[i].value == font_size_value) {
+        selectCtl.selectedIndex = i;
+        if ($('Custom'))
+          selectCtl.remove($('Custom').index);
+        return;
       }
     }
 
@@ -196,10 +176,28 @@ var OptionsPage = options.OptionsPage;
     $('Custom').selected = true;
   };
 
-  // Set the download path.
-  AdvancedOptions.SetDownloadLocationPath = function(path) {
-    if (!cr.isChromeOS)
-      $('downloadLocationPath').value = path;
+  /**
+    * Populate the page zoom selector with values received from the caller.
+    * @param {Array} items An array of items to populate the selector.
+    *     each object is an array with three elements as follows:
+    *       0: The title of the item (string).
+    *       1: The value of the item (number).
+    *       2: Whether the item should be selected (boolean).
+    */
+  AdvancedOptions.SetupPageZoomSelector = function(items) {
+    var element = $('defaultZoomFactor');
+
+    // Remove any existing content.
+    element.textContent = '';
+
+    // Insert new child nodes into select element.
+    var value, title, selected;
+    for (var i = 0; i < items.length; i++) {
+      title = items[i][0];
+      value = items[i][1];
+      selected = items[i][2];
+      element.appendChild(new Option(title, value, false, selected));
+    }
   };
 
   // Set the enabled state for the autoOpenFileTypesResetToDefault button.
@@ -216,8 +214,10 @@ var OptionsPage = options.OptionsPage;
 
   // Set the enabled state for the proxy settings button.
   AdvancedOptions.SetupProxySettingsSection = function(disabled, label) {
-    $('proxiesConfigureButton').disabled = disabled;
-    $('proxiesLabel').textContent = label;
+    if (!cr.isChromeOS) {
+      $('proxiesConfigureButton').disabled = disabled;
+      $('proxiesLabel').textContent = label;
+    }
   };
 
   // Set the checked state for the sslCheckRevocation checkbox.
@@ -227,59 +227,37 @@ var OptionsPage = options.OptionsPage;
     $('sslCheckRevocation').disabled = disabled;
   };
 
-  // Set the checked state for the sslUseSSL3 checkbox.
-  AdvancedOptions.SetUseSSL3CheckboxState = function(checked, disabled) {
-    $('sslUseSSL3').checked = checked;
-    $('sslUseSSL3').disabled = disabled;
-  };
-
-  // Set the checked state for the sslUseTLS1 checkbox.
-  AdvancedOptions.SetUseTLS1CheckboxState = function(checked, disabled) {
-    $('sslUseTLS1').checked = checked;
-    $('sslUseTLS1').disabled = disabled;
+  // Set the checked state for the backgroundModeCheckbox element.
+  AdvancedOptions.SetBackgroundModeCheckboxState = function(checked) {
+    $('backgroundModeCheckbox').checked = checked;
   };
 
   // Set the Cloud Print proxy UI to enabled, disabled, or processing.
-  AdvancedOptions.SetupCloudPrintProxySection = function(disabled, label) {
+  AdvancedOptions.SetupCloudPrintConnectorSection = function(
+        disabled, label, allowed) {
     if (!cr.isChromeOS) {
-      $('cloudPrintProxyLabel').textContent = label;
-      if (disabled) {
-        $('cloudPrintProxySetupButton').textContent =
-          localStrings.getString('cloudPrintProxyDisabledButton');
-        $('cloudPrintProxyManageButton').style.display = 'none';
+      $('cloudPrintConnectorLabel').textContent = label;
+      if (disabled || !allowed) {
+        $('cloudPrintConnectorSetupButton').textContent =
+          localStrings.getString('cloudPrintConnectorDisabledButton');
+        $('cloudPrintManageButton').style.display = 'none';
       } else {
-        $('cloudPrintProxySetupButton').textContent =
-          localStrings.getString('cloudPrintProxyEnabledButton');
-        $('cloudPrintProxyManageButton').style.display = 'inline';
+        $('cloudPrintConnectorSetupButton').textContent =
+          localStrings.getString('cloudPrintConnectorEnabledButton');
+        $('cloudPrintManageButton').style.display = 'inline';
       }
-      $('cloudPrintProxySetupButton').disabled = false;
+      $('cloudPrintConnectorSetupButton').disabled = !allowed;
     }
   };
 
-  AdvancedOptions.RemoveCloudPrintProxySection = function() {
+  AdvancedOptions.RemoveCloudPrintConnectorSection = function() {
     if (!cr.isChromeOS) {
-      var proxySectionElm = $('cloud-print-proxy-section');
-      proxySectionElm.parentNode.removeChild(proxySectionElm);
+      var connectorSectionElm = $('cloud-print-connector-section');
+      if (connectorSectionElm)
+        connectorSectionElm.parentNode.removeChild(connectorSectionElm);
     }
   };
 
-  AdvancedOptions.SetRemotingStatus = function(enabled, status) {
-    if (enabled) {
-      $('remotingSetupButton').style.display = 'none';
-      $('remotingStopButton').style.display = 'inline';
-    } else {
-      $('remotingSetupButton').style.display = 'inline';
-      $('remotingStopButton').style.display = 'none';
-    }
-    $('remotingStatus').textContent = status;
-  };
-
-  AdvancedOptions.RemoveRemotingSection = function() {
-    var proxySectionElm = $('remoting-section');
-    if (proxySectionElm)
-      proxySectionElm.parentNode.removeChild(proxySectionElm);
-  };
-
   // Export
   return {
     AdvancedOptions: AdvancedOptions
