@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,7 +15,7 @@ cr.define('options', function() {
    */
   function ContentSettings() {
     this.activeNavTab = null;
-    OptionsPage.call(this, 'content', templateData.contentSettingsPage,
+    OptionsPage.call(this, 'content', templateData.contentSettingsPageTabTitle,
                      'content-settings-page');
   }
 
@@ -33,36 +33,47 @@ cr.define('options', function() {
           this.pageDiv.querySelectorAll('.exceptions-list-button');
       for (var i = 0; i < exceptionsButtons.length; i++) {
         exceptionsButtons[i].onclick = function(event) {
-          ContentSettingsExceptionsArea.getInstance().showList(
+          var page = ContentSettingsExceptionsArea.getInstance();
+          page.showList(
               event.target.getAttribute('contentType'));
-          OptionsPage.showPageByName('contentExceptions');
+          OptionsPage.navigateToPage('contentExceptions');
+          // Add on the proper hash for the content type, and store that in the
+          // history so back/forward and tab restore works.
+          var hash = event.target.getAttribute('contentType');
+          window.history.replaceState({pageName: page.name}, page.title,
+                                      '/' + page.name + "#" + hash);
+        };
+      }
+
+      var manageHandlersButton = $('manage-handlers-button');
+      if (manageHandlersButton) {
+        manageHandlersButton.onclick = function(event) {
+          OptionsPage.navigateToPage('handlers');
+        };
+      }
+
+      var manageIntentsButton = $('manage-intents-button');
+      if (manageIntentsButton) {
+        manageIntentsButton.onclick = function(event) {
+          OptionsPage.navigateToPage('intents');
         };
       }
 
       // Cookies filter page ---------------------------------------------------
-      $('block-third-party-cookies').onclick = function(event) {
-        chrome.send('setAllowThirdPartyCookies',
-                    [String($('block-third-party-cookies').checked)]);
-      };
-
       $('show-cookies-button').onclick = function(event) {
         chrome.send('coreOptionsUserMetricsAction', ['Options_ShowCookies']);
-        OptionsPage.showPageByName('cookiesView');
+        OptionsPage.navigateToPage('cookies');
       };
 
-      if (!templateData.enable_click_to_play)
-        $('click_to_play').style.display = 'none';
+      if (!templateData.enable_web_intents && $('intent-section'))
+        $('intent-section').hidden = true;
     },
+  };
 
-    /**
-     * Handles a hash value in the URL (such as bar in
-     * chrome://options/foo#bar).
-     * @param {string} hash The hash value.
-     */
-    handleHash: function(hash) {
-      ContentSettingsExceptionsArea.getInstance().showList(hash);
-      OptionsPage.showPageByName('contentExceptions');
-    },
+  ContentSettings.updateHandlersEnabledRadios = function(enabled) {
+    var selector = '#content-settings-page input[type=radio][value=' +
+        (enabled ? 'allow' : 'block') + '].handler-radio';
+    document.querySelector(selector).checked = true;
   };
 
   /**
@@ -76,10 +87,13 @@ cr.define('options', function() {
                              dict[group]['value'] + ']').checked = true;
       var radios = document.querySelectorAll('input[type=radio][name=' +
                                              group + ']');
+      var managedBy = dict[group]['managedBy'];
       for (var i = 0, len = radios.length; i < len; i++) {
-        radios[i].disabled = dict[group]['managed'];
+        radios[i].disabled = (managedBy != 'default');
+        radios[i].controlledBy = managedBy;
       }
     }
+    OptionsPage.updateManagedBannerVisibility();
   };
 
   /**
@@ -92,27 +106,26 @@ cr.define('options', function() {
     var exceptionsList =
         document.querySelector('div[contentType=' + type + ']' +
                                ' list[mode=normal]');
-
     exceptionsList.setExceptions(list);
   };
 
+  ContentSettings.setHandlers = function(list) {
+    $('handlers-list').setHandlers(list);
+  };
+
+  ContentSettings.setIgnoredHandlers = function(list) {
+    $('ignored-handlers-list').setHandlers(list);
+  };
+
   ContentSettings.setOTRExceptions = function(type, list) {
     var exceptionsList =
         document.querySelector('div[contentType=' + type + ']' +
                                ' list[mode=otr]');
 
-    exceptionsList.parentNode.classList.remove('hidden');
+    exceptionsList.parentNode.hidden = false;
     exceptionsList.setExceptions(list);
   };
 
-  /**
-   * Sets the initial value for the Third Party Cookies checkbox.
-   * @param {boolean=} block True if we are blocking third party cookies.
-   */
-  ContentSettings.setBlockThirdPartyCookies = function(block) {
-    $('block-third-party-cookies').checked = block;
-  };
-
   /**
    * The browser's response to a request to check the validity of a given URL
    * pattern.
@@ -130,10 +143,6 @@ cr.define('options', function() {
     exceptionsList.patternValidityCheckComplete(pattern, valid);
   };
 
-  ContentSettings.setClearLocalDataOnShutdownLabel = function(label) {
-    $('clear-cookies-on-exit-label').innerText = label;
-  };
-
   // Export
   return {
     ContentSettings: ContentSettings
