@@ -6,20 +6,19 @@
 // in js/cr/ui/notification.js .
 
 cr.define('options', function() {
-
   const OptionsPage = options.OptionsPage;
-  const AddLanguageOverlay = options.language.AddLanguageOverlay;
-  const LanguageList = options.language.LanguageList;
+  const LanguageList = options.LanguageList;
 
   // Some input methods like Chinese Pinyin have config pages.
   // This is the map of the input method names to their config page names.
   const INPUT_METHOD_ID_TO_CONFIG_PAGE_NAME = {
-    'chewing': 'languageChewing',
-    'hangul': 'languageHangul',
     'mozc': 'languageMozc',
+    'mozc-chewing': 'languageChewing',
     'mozc-dv': 'languageMozc',
+    'mozc-hangul': 'languageHangul',
     'mozc-jp': 'languageMozc',
     'pinyin': 'languagePinyin',
+    'pinyin-dv': 'languagePinyin',
   };
 
   /////////////////////////////////////////////////////////////////////////////
@@ -30,7 +29,7 @@ cr.define('options', function() {
    * @constructor
    */
   function LanguageOptions(model) {
-    OptionsPage.call(this, 'language', localStrings.getString('languagePage'),
+    OptionsPage.call(this, 'languages', templateData.languagePageTabTitle,
                      'languagePage');
   }
 
@@ -60,7 +59,7 @@ cr.define('options', function() {
 
       if (cr.isChromeOS) {
         this.initializeInputMethodList_();
-        this.initializeLanguageCodeToInputMehotdIdsMap_();
+        this.initializeLanguageCodeToInputMethodIdsMap_();
       }
       Preferences.getInstance().addEventListener(this.spellCheckDictionaryPref,
           this.handleSpellCheckDictionaryPrefChange_.bind(this));
@@ -74,15 +73,9 @@ cr.define('options', function() {
           var addLanguageCode = match[1];
           $('language-options-list').addLanguage(addLanguageCode);
         } else {
-          OptionsPage.showOverlay('addLanguageOverlay');
+          OptionsPage.navigateToPage('addLanguage');
         }
       };
-      // Set up remove button.
-      $('language-options-remove-button').addEventListener('click',
-          this.handleRemoveButtonClick_.bind(this));
-
-      // Setup add language overlay page.
-      OptionsPage.registerOverlay(AddLanguageOverlay.getInstance());
 
       if (cr.isChromeOS) {
         // Listen to user clicks on the add language list.
@@ -96,12 +89,31 @@ cr.define('options', function() {
             this.handleAddLanguageOkButtonClick_.bind(this));
 
         // Show experimental features if enabled.
-        if (templateData.experimentalSpellCheckFeatures == 'true') {
-          $('auto-spell-correction-option').classList.remove('hidden');
+        if (templateData.experimentalSpellCheckFeatures == 'true')
+          $('auto-spell-correction-option').hidden = false;
+
+        // Handle spell check enable/disable.
+        if (!cr.isMac) {
+          Preferences.getInstance().addEventListener(
+              this.enableSpellCheckPref,
+              this.updateEnableSpellCheck_.bind(this));
         }
       }
+
+      // Listen to user clicks on the "Change touch keyboard settings..."
+      // button (if it exists).
+      var virtualKeyboardButton = $('language-options-virtual-keyboard');
+      if (virtualKeyboardButton) {
+        // TODO(yusukes): would be better to hide the button if no virtual
+        // keyboard is registered.
+        virtualKeyboardButton.onclick = function(e) {
+          OptionsPage.navigateToPage('virtualKeyboards');
+        };
+      }
     },
 
+    // The preference is a boolean that enables/disables spell checking.
+    enableSpellCheckPref: 'browser.enable_spellchecking',
     // The preference is a CSV string that describes preload engines
     // (i.e. active input methods).
     preloadEnginesPref: 'settings.language.preload_engines',
@@ -136,7 +148,10 @@ cr.define('options', function() {
                                this.handleCheckboxClick_.bind(this));
         var label = document.createElement('label');
         label.appendChild(input);
-        label.appendChild(document.createTextNode(inputMethod.displayName));
+        // Adding a space between the checkbox and the text. This is a bit
+        // dirty, but we rely on a space character for all other checkboxes.
+        label.appendChild(document.createTextNode(
+            ' ' + inputMethod.displayName));
         label.style.display = 'none';
         label.languageCodeSet = inputMethod.languageCodeSet;
         // Add the configure button if the config page is present for this
@@ -170,7 +185,7 @@ cr.define('options', function() {
         // as checkbox click.
         e.preventDefault();
         chrome.send('inputMethodOptionsOpen', [inputMethodId]);
-        OptionsPage.showPageByName(pageName);
+        OptionsPage.navigateToPage(pageName);
       }
       return button;
     },
@@ -207,7 +222,8 @@ cr.define('options', function() {
       this.updateSelectedLanguageName_(languageCode);
       if (cr.isWindows || cr.isChromeOS)
         this.updateUiLanguageButton_(languageCode);
-      this.updateSpellCheckLanguageButton_(languageCode);
+      if (!cr.isMac)
+        this.updateSpellCheckLanguageButton_(languageCode);
       if (cr.isChromeOS)
         this.updateInputMethodList_(languageCode);
       this.updateLanguageListInAddLanguageOverlay_();
@@ -278,7 +294,7 @@ cr.define('options', function() {
      * Initializes the map of language code to input method IDs.
      * @private
      */
-    initializeLanguageCodeToInputMehotdIdsMap_: function() {
+    initializeLanguageCodeToInputMethodIdsMap_: function() {
       var inputMethodList = templateData.inputMethodList;
       for (var i = 0; i < inputMethodList.length; i++) {
         var inputMethod = inputMethodList[i];
@@ -309,7 +325,13 @@ cr.define('options', function() {
         languageDisplayName += ' - ' + languageNativeDisplayName;
       }
       // Update the currently selected language name.
-      $('language-options-language-name').textContent = languageDisplayName;
+      var languageName = $('language-options-language-name');
+      if (languageDisplayName) {
+        languageName.hidden = false;
+        languageName.textContent = languageDisplayName;
+      } else {
+        languageName.hidden = true;
+      }
     },
 
     /**
@@ -332,15 +354,23 @@ cr.define('options', function() {
       } else if (languageCode in templateData.uiLanguageCodeSet) {
         // If the language is supported as UI language, users can click on
         // the button to change the UI language.
-        uiLanguageButton.textContent =
-            localStrings.getString('display_in_this_language');
-        uiLanguageButton.className = '';
-        // Send the change request to Chrome.
-        uiLanguageButton.onclick = function(e) {
-          chrome.send('uiLanguageChange', [languageCode]);
+        if (cr.commandLine && cr.commandLine.options['--bwsi']) {
+          // In the guest mode for ChromeOS, changing UI language does not make
+          // sense because it does not take effect after browser restart.
+          uiLanguageButton.hidden = true;
+        } else {
+          uiLanguageButton.textContent =
+              localStrings.getString('display_in_this_language');
+          uiLanguageButton.className = '';
+          // Send the change request to Chrome.
+          uiLanguageButton.onclick = function(e) {
+            chrome.send('uiLanguageChange', [languageCode]);
+          }
         }
-        $('language-options-ui-restart-button').onclick = function(e) {
-          chrome.send('uiLanguageRestart');
+        if (cr.isChromeOS) {
+          $('language-options-ui-restart-button').onclick = function(e) {
+            chrome.send('uiLanguageRestart');
+          }
         }
       } else {
         // If the language is not supported as UI language, the button
@@ -360,6 +390,7 @@ cr.define('options', function() {
      * @private
      */
     updateSpellCheckLanguageButton_: function(languageCode) {
+      var display = 'block';
       var spellCheckLanguageButton = $(
           'language-options-spell-check-language-button');
       // Check if the language code matches the current spell check language.
@@ -382,6 +413,8 @@ cr.define('options', function() {
         // Add an event listner to the click event.
         spellCheckLanguageButton.addEventListener('click',
             this.handleSpellCheckLanguageButtonClick_.bind(this));
+      } else if (!languageCode) {
+        display = 'none';
       } else {
         // If the language is not supported as spell check language, the
         // button just says that this language cannot be used for spell
@@ -391,7 +424,7 @@ cr.define('options', function() {
         spellCheckLanguageButton.className = 'text-button';
         spellCheckLanguageButton.onclick = undefined;
       }
-      spellCheckLanguageButton.style.display = 'block';
+      spellCheckLanguageButton.style.display = display;
       $('language-options-ui-notification-bar').style.display = 'none';
     },
 
@@ -425,9 +458,8 @@ cr.define('options', function() {
           label.style.display = 'none';
         }
       }
-      if (focusInputMethodId == 'remove') {
-        $('language-options-remove-button').focus();
-      } else if (focusInputMethodId == 'add') {
+
+      if (focusInputMethodId == 'add') {
         $('language-options-add-button').focus();
       }
     },
@@ -469,6 +501,7 @@ cr.define('options', function() {
       var value = e.value.value;
       this.preloadEngines_ = this.filterBadPreloadEngines_(value.split(','));
       this.updateCheckboxesFromPreloadEngines_();
+      $('language-options-list').updateDeletable();
     },
 
     /**
@@ -521,7 +554,7 @@ cr.define('options', function() {
         this.updateCheckboxesFromPreloadEngines_();
         this.savePreloadEnginesPref_();
       }
-      OptionsPage.clearOverlays();
+      OptionsPage.closeOverlay();
     },
 
     /**
@@ -533,38 +566,33 @@ cr.define('options', function() {
       if (selectedIndex >= 0) {
         var selection = languagesSelect.options[selectedIndex];
         $('language-options-list').addLanguage(String(selection.value));
-        OptionsPage.clearOverlays();
+        OptionsPage.closeOverlay();
       }
     },
 
     /**
-     * Handles remove button's click event.
-     * @param {Event} e Click event.
+     * Checks if languageCode is deletable or not.
+     * @param {String} languageCode the languageCode to check for deletability.
      */
-    handleRemoveButtonClick_: function(e) {
-      var languageOptionsList = $('language-options-list');
-      var languageCode = languageOptionsList.getSelectedLanguageCode();
+    languageIsDeletable: function(languageCode) {
       // Don't allow removing the language if it's as UI language.
-      if (languageCode == templateData.currentUiLanguageCode) {
-        this.showNotification_(
-            localStrings.getString('this_language_is_currently_in_use'),
-            localStrings.getString('ok_button'));
-        return;
-      }
-      if (cr.isChromeOS) {
-        // Disable input methods associated with |languageCode|.
-        // Don't allow removing the language if cerntain conditions are met.
-        // See removePreloadEnginesByLanguageCode_() for details.
-        if (!this.removePreloadEnginesByLanguageCode_(languageCode)) {
-          this.showNotification_(
-              localStrings.getString('please_add_another_language'),
-              localStrings.getString('ok_button'));
-          return;
-        }
-      }
-      languageOptionsList.removeSelectedLanguage();
+      if (languageCode == templateData.currentUiLanguageCode)
+        return false;
+      return (!cr.isChromeOS ||
+              this.canDeleteLanguage_(languageCode));
     },
 
+    /**
+     * Handles browse.enable_spellchecking change.
+     * @param {Event} e Change event.
+     * @private
+     */
+     updateEnableSpellCheck_: function() {
+       var value = !$('enable-spell-check').checked;
+
+       $('language-options-spell-check-language-button').disabled = value;
+     },
+
     /**
      * Handles spellCheckDictionaryPref change.
      * @param {Event} e Change event.
@@ -592,19 +620,15 @@ cr.define('options', function() {
     },
 
     /**
-     * Removes preload engines associated with the given language code.
-     * However, this function does not remove engines (input methods) that
-     * are used for other active languages. For instance, if "xkb:us::eng"
-     * is used for English and Filipino, and the two languages are active,
-     * this function does not remove "xkb:us::eng" when either of these
-     * languages is removed. Instead, it'll remove "xkb:us::eng" when the
-     * both languages are gone.
+     * Checks whether it's possible to remove the language specified by
+     * languageCode and returns true if possible. This function returns false
+     * if the removal causes the number of preload engines to be zero.
      *
      * @param {string} languageCode Language code (ex. "fr").
      * @return {boolean} Returns true on success.
      * @private
      */
-    removePreloadEnginesByLanguageCode_: function(languageCode) {
+    canDeleteLanguage_: function(languageCode) {
       // First create the set of engines to be removed from input methods
       // associated with the language code.
       var enginesToBeRemovedSet = {};
@@ -614,6 +638,7 @@ cr.define('options', function() {
       }
 
       // Then eliminate engines that are also used for other active languages.
+      // For instance, if "xkb:us::eng" is used for both English and Filipino.
       var languageCodes = $('language-options-list').getLanguageCodes();
       for (var i = 0; i < languageCodes.length; i++) {
         // Skip the target language code.
@@ -642,12 +667,7 @@ cr.define('options', function() {
       }
       // Don't allow this operation if it causes the number of preload
       // engines to be zero.
-      if (newPreloadEngines.length == 0) {
-        return false;
-      }
-      this.preloadEngines_ = newPreloadEngines;
-      this.savePreloadEnginesPref_();
-      return true;
+      return (newPreloadEngines.length > 0);
     },
 
     /**
@@ -692,6 +712,8 @@ cr.define('options', function() {
           this.preloadEngines_.push(checkboxes[i].inputMethodId);
         }
       }
+      var languageOptionsList = $('language-options-list');
+      languageOptionsList.updateDeletable();
     },
 
     /**
@@ -720,7 +742,7 @@ cr.define('options', function() {
       return filteredPreloadEngines;
     },
 
-    // TODO(kochi): This is an adapted copy from new_new_tab.js.
+    // TODO(kochi): This is an adapted copy from new_tab.js.
     // If this will go as final UI, refactor this to share the component with
     // new new tab page.
     /**
@@ -790,5 +812,4 @@ cr.define('options', function() {
   return {
     LanguageOptions: LanguageOptions
   };
-
 });
