@@ -35,15 +35,37 @@ cr.define('options', function() {
       this.intervalId = setInterval(this.updatePosition.bind(this), 250);
     },
 
+  /**
+   * Attach the bubble to the element.
+   */
+    attachTo: function(element) {
+      var parent = element.parentElement;
+      if (!parent)
+        return;
+      if (parent.tagName == 'TD') {
+        // To make absolute positioning work inside a table cell we need
+        // to wrap the bubble div into another div with position:relative.
+        // This only works properly if the element is the first child of the
+        // table cell which is true for all options pages.
+        this.wrapper = cr.doc.createElement('div');
+        this.wrapper.className = 'search-bubble-wrapper';
+        this.wrapper.appendChild(this);
+        parent.insertBefore(this.wrapper, element);
+      } else {
+        parent.insertBefore(this, element);
+      }
+    },
+
     /**
      * Clear the interval timer and remove the element from the page.
      */
     dispose: function() {
       clearInterval(this.intervalId);
 
-      var parent = this.parentNode;
+      var child = this.wrapper || this;
+      var parent = child.parentNode;
       if (parent)
-        parent.removeChild(this);
+        parent.removeChild(child);
     },
 
     /**
@@ -52,7 +74,7 @@ cr.define('options', function() {
      */
     updatePosition: function() {
       // This bubble is 'owned' by the next sibling.
-      var owner = this.nextSibling;
+      var owner = (this.wrapper || this).nextSibling;
 
       // If there isn't an offset parent, we have nothing to do.
       if (!owner.offsetParent)
@@ -81,8 +103,8 @@ cr.define('options', function() {
    * @constructor
    */
   function SearchPage() {
-    OptionsPage.call(this, 'search', templateData.searchPage, 'searchPage');
-    this.searchActive = false;
+    OptionsPage.call(this, 'search', templateData.searchPageTabTitle,
+        'searchPage');
   }
 
   cr.addSingletonGetter(SearchPage);
@@ -91,6 +113,13 @@ cr.define('options', function() {
     // Inherit SearchPage from OptionsPage.
     __proto__: OptionsPage.prototype,
 
+    /**
+     * A boolean to prevent recursion. Used by setSearchText_().
+     * @type {Boolean}
+     * @private
+     */
+    insideSetSearchText_: false,
+
     /**
      * Initialize the page.
      */
@@ -104,14 +133,20 @@ cr.define('options', function() {
       var searchField = document.createElement('input');
       searchField.id = 'search-field';
       searchField.type = 'search';
-      searchField.setAttribute('autosave', 'org.chromium.options.search');
-      searchField.setAttribute('results', '10');
-      searchField.setAttribute('incremental', 'true');
+      searchField.incremental = true;
+      searchField.placeholder = localStrings.getString('searchPlaceholder');
+      searchField.setAttribute('aria-label', searchField.placeholder);
+      this.searchField = searchField;
 
       // Replace the contents of the navigation tab with the search field.
       self.tab.textContent = '';
       self.tab.appendChild(searchField);
-      self.tab.onclick = self.tab.onkeypress = undefined;
+      self.tab.onclick = self.tab.onkeydown = self.tab.onkeypress = undefined;
+      self.tab.tabIndex = -1;
+      self.tab.setAttribute('role', '');
+
+      // Don't allow the focus on the search navbar. http://crbug.com/77989
+      self.tab.onfocus = self.tab.blur;
 
       // Handle search events. (No need to throttle, WebKit's search field
       // will do that automatically.)
@@ -119,6 +154,33 @@ cr.define('options', function() {
         self.setSearchText_(this.value);
       };
 
+      // We update the history stack every time the search field blurs. This way
+      // we get a history entry for each search, roughly, but not each letter
+      // typed.
+      searchField.onblur = function(e) {
+        var query = SearchPage.canonicalizeQuery(searchField.value);
+        if (!query)
+          return;
+
+        // Don't push the same page onto the history stack more than once (if
+        // the user clicks in the search field and away several times).
+        var currentHash = location.hash;
+        var newHash = '#' + escape(query);
+        if (currentHash == newHash)
+          return;
+
+        // If there is no hash on the current URL, the history entry has no
+        // search query. Replace the history entry with no search with an entry
+        // that does have a search. Otherwise, add it onto the history stack.
+        var historyFunction = currentHash ? window.history.pushState :
+                                            window.history.replaceState;
+        historyFunction.call(
+            window.history,
+            {pageName: self.name},
+            self.title,
+            '/' + self.name + newHash);
+      };
+
       // Install handler for key presses.
       document.addEventListener('keydown',
                                 this.keyDownEventHandler_.bind(this));
@@ -166,12 +228,15 @@ cr.define('options', function() {
       if (!this.searchActive_ && !active)
         return;
 
-      if (this.searchActive_ != active) {
-        this.searchActive_ = active;
-        if (!active) {
-          // Just wipe out any active search text since it's no longer relevant.
-          $('search-field').value = '';
-        }
+      this.searchActive_ = active;
+
+      if (active) {
+        var hash = location.hash;
+        if (hash)
+          this.searchField.value = unescape(hash.slice(1));
+      } else {
+        // Just wipe out any active search text since it's no longer relevant.
+        this.searchField.value = '';
       }
 
       var pagesToSearch = this.getSearchablePages_();
@@ -185,23 +250,35 @@ cr.define('options', function() {
         // sections (ie titles, button strips).  We do this before changing
         // the page visibility to avoid excessive re-draw.
         for (var i = 0, childDiv; childDiv = page.pageDiv.children[i]; i++) {
-          if (active) {
-            if (childDiv.tagName != 'SECTION')
-              childDiv.classList.add('search-hidden');
+          if (childDiv.classList.contains('displaytable')) {
+            childDiv.setAttribute('searching', active ? 'true' : 'false');
+            for (var j = 0, subDiv; subDiv = childDiv.children[j]; j++) {
+              if (active) {
+                if (subDiv.tagName != 'SECTION')
+                  subDiv.classList.add('search-hidden');
+              } else {
+                subDiv.classList.remove('search-hidden');
+              }
+            }
           } else {
-            childDiv.classList.remove('search-hidden');
+            if (active)
+              childDiv.classList.add('search-hidden');
+            else
+              childDiv.classList.remove('search-hidden');
           }
         }
 
         if (active) {
           // When search is active, remove the 'hidden' tag.  This tag may have
           // been added by the OptionsPage.
-          page.pageDiv.classList.remove('hidden');
+          page.pageDiv.hidden = false;
         }
       }
 
-      // After hiding all page content, remove any search results.
-      if (!active) {
+      if (active) {
+        this.setSearchText_(this.searchField.value);
+      } else {
+        // After hiding all page content, remove any search results.
         this.unhighlightMatches_();
         this.removeSearchBubbles_();
       }
@@ -213,17 +290,28 @@ cr.define('options', function() {
      * @private
      */
     setSearchText_: function(text) {
-      // Consider whitespace-only strings as empty.
-      if (!text.replace(/^\s+/, '').length)
-        text = '';
+      // Prevent recursive execution of this method.
+      if (this.insideSetSearchText_) return;
+      this.insideSetSearchText_ = true;
+
+      // Cleanup the search query string.
+      text = SearchPage.canonicalizeQuery(text);
+
+      // Notify listeners about the new search query, some pages may wish to
+      // show/hide elements based on the query.
+      var event = new cr.Event('searchChanged');
+      event.searchText = text;
+      this.dispatchEvent(event);
 
       // Toggle the search page if necessary.
       if (text.length) {
         if (!this.searchActive_)
-          OptionsPage.showPageByName(this.name);
+          OptionsPage.navigateToPage(this.name);
       } else {
         if (this.searchActive_)
           OptionsPage.showDefaultPage();
+
+        this.insideSetSearchText_ = false;
         return;
       }
 
@@ -256,14 +344,12 @@ cr.define('options', function() {
         }
         if (pageMatch)
           foundMatches = true;
-        for (var i = 0, childDiv; childDiv = page.pageDiv.children[i]; i++) {
-          if (childDiv.tagName == 'SECTION') {
-            if (pageMatch) {
-              childDiv.classList.remove('search-hidden');
-            } else {
-              childDiv.classList.add('search-hidden');
-            }
-          }
+        var elements = page.pageDiv.querySelectorAll('.displaytable > section');
+        for (var i = 0, node; node = elements[i]; i++) {
+          if (pageMatch)
+            node.classList.remove('search-hidden');
+          else
+            node.classList.add('search-hidden');
         }
       }
 
@@ -271,10 +357,11 @@ cr.define('options', function() {
         // Search all top-level sections for anchored string matches.
         for (var key in pagesToSearch) {
           page = pagesToSearch[key];
-          for (var i = 0, childDiv; childDiv = page.pageDiv.children[i]; i++) {
-            if (childDiv.tagName == 'SECTION' &&
-                this.performReplace_(regEx, replaceString, childDiv)) {
-              childDiv.classList.remove('search-hidden');
+          var elements =
+              page.pageDiv.querySelectorAll('.displaytable > section');
+          for (var i = 0, node; node = elements[i]; i++) {
+            if (this.performReplace_(regEx, replaceString, node)) {
+              node.classList.remove('search-hidden');
               foundMatches = true;
             }
           }
@@ -315,6 +402,9 @@ cr.define('options', function() {
       length = bubbleControls.length;
       for (var i = 0; i < length; i++)
         this.createSearchBubble_(bubbleControls[i], text);
+
+      // Cleanup the recursion-prevention variable.
+      this.insideSetSearchText_ = false;
     },
 
     /**
@@ -396,15 +486,16 @@ cr.define('options', function() {
      * @private
      */
     createSearchBubble_: function(element, text) {
-      // avoid appending multiple ballons to a button.
+      // avoid appending multiple bubbles to a button.
       var sibling = element.previousElementSibling;
-      if (sibling && sibling.classList.contains('search-bubble'))
+      if (sibling && (sibling.classList.contains('search-bubble') ||
+                      sibling.classList.contains('search-bubble-wrapper')))
         return;
 
       var parent = element.parentElement;
       if (parent) {
         var bubble = new SearchBubble(text);
-        parent.insertBefore(bubble, element);
+        bubble.attachTo(element);
         bubble.updatePosition();
       }
     },
@@ -465,14 +556,38 @@ cr.define('options', function() {
      * @private
      */
     keyDownEventHandler_: function(event) {
-      // Focus the search field on an unused forward-slash.
-      if (event.keyCode == 191 &&
-          !/INPUT|SELECT|BUTTON|TEXTAREA/.test(event.target.tagName)) {
-        $('search-field').focus();
-        event.stopPropagation();
-        event.preventDefault();
+      const ESCAPE_KEY_CODE = 27;
+      const FORWARD_SLASH_KEY_CODE = 191;
+
+      switch(event.keyCode) {
+        case ESCAPE_KEY_CODE:
+          if (event.target == this.searchField) {
+            this.setSearchText_('');
+            this.searchField.blur();
+            event.stopPropagation();
+            event.preventDefault();
+          }
+          break;
+        case FORWARD_SLASH_KEY_CODE:
+          if (!/INPUT|SELECT|BUTTON|TEXTAREA/.test(event.target.tagName) &&
+              !event.ctrlKey && !event.altKey) {
+            this.searchField.focus();
+            event.stopPropagation();
+            event.preventDefault();
+          }
+          break;
       }
-    }
+    },
+  };
+
+  /**
+   * Standardizes a user-entered text query by removing extra whitespace.
+   * @param {string} The user-entered text.
+   * @return {string} The trimmed query.
+   */
+  SearchPage.canonicalizeQuery = function(text) {
+    // Trim beginning and ending whitespace.
+    return text.replace(/^\s+|\s+$/g, '');
   };
 
   // Export
