@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -28,11 +28,44 @@ const cr = (function() {
    */
   const isLinux = /Linux/.test(navigator.userAgent);
 
+  /**
+   * Whether this uses GTK or not.
+   * @type {boolean}
+   */
+  const isGTK = /GTK/.test(chrome.toolkit);
+
   /**
    * Whether this uses the views toolkit or not.
    * @type {boolean}
    */
-  const isViews = isWindows || isChromeOS;
+  const isViews = /views/.test(chrome.toolkit);
+
+  /**
+   * Whether this window is optimized for touch-based input.
+   * @type {boolean}
+   */
+  const isTouchOptimized = !!chrome.touchOptimized;
+
+  /**
+   * Sets the os and toolkit attributes in the <html> element so that platform
+   * specific css rules can be applied.
+   */
+  function enablePlatformSpecificCSSRules() {
+    if (isMac)
+      doc.documentElement.setAttribute('os', 'mac');
+    if (isWindows)
+      doc.documentElement.setAttribute('os', 'windows');
+    if (isChromeOS)
+      doc.documentElement.setAttribute('os', 'chromeos');
+    if (isLinux)
+      doc.documentElement.setAttribute('os', 'linux');
+    if (isGTK)
+      doc.documentElement.setAttribute('toolkit', 'gtk');
+    if (isViews)
+      doc.documentElement.setAttribute('toolkit', 'views');
+    if (isTouchOptimized)
+      doc.documentElement.setAttribute('touch-optimized', '');
+  }
 
   /**
    * Builds an object structure for the provided namespace path,
@@ -102,6 +135,16 @@ const cr = (function() {
     target.dispatchEvent(e);
   }
 
+  /**
+   * Converts a camelCase javascript property name to a hyphenated-lower-case
+   * attribute name.
+   * @param {string} jsName The javascript camelCase property name.
+   * @return {string} The equivalent hyphenated-lower-case attribute name.
+   */
+  function getAttributeName(jsName) {
+    return jsName.replace(/([A-Z])/g, '-$1').toLowerCase();
+  }
+
   /**
    * The kind of property to define in {@code defineProperty}.
    * @enum {number}
@@ -130,11 +173,9 @@ const cr = (function() {
    * property.
    * @param {string} name
    * @param {cr.PropertyKind} kind
-   * @param {*} defaultValue The default value. This is only used for the ATTR
-   *    kind.
    * @return {function():*} The getter for the property.
    */
-  function getGetter(name, kind, defaultValue) {
+  function getGetter(name, kind) {
     switch (kind) {
       case PropertyKind.JS:
         var privateName = name + '_';
@@ -142,23 +183,14 @@ const cr = (function() {
           return this[privateName];
         };
       case PropertyKind.ATTR:
-        // For attr with default value we return the default value if the
-        // element is missing the attribute.
-        if (defaultValue == undefined) {
-          return function() {
-            return this.getAttribute(name);
-          };
-        } else {
-          return function() {
-            // WebKit uses null for non existant attributes.
-            var value = this.getAttribute(name);
-            return value !== null ? value : defaultValue;
-          };
-        }
-      case PropertyKind.BOOL_ATTR:
-        // Boolean attributes don't support default values.
+        var attributeName = getAttributeName(name);
         return function() {
-          return this.hasAttribute(name);
+          return this.getAttribute(attributeName);
+        };
+      case PropertyKind.BOOL_ATTR:
+        var attributeName = getAttributeName(name);
+        return function() {
+          return this.hasAttribute(attributeName);
         };
     }
   }
@@ -170,9 +202,11 @@ const cr = (function() {
    *     for.
    * @param {cr.PropertyKind} kind The kind of property we are getting the
    *     setter for.
+   * @param {function(*):void} opt_setHook A function to run after the property
+   *     is set, but before the propertyChange event is fired.
    * @return {function(*):void} The function to use as a setter.
    */
-  function getSetter(name, kind) {
+  function getSetter(name, kind, opt_setHook) {
     switch (kind) {
       case PropertyKind.JS:
         var privateName = name + '_';
@@ -180,27 +214,38 @@ const cr = (function() {
           var oldValue = this[privateName];
           if (value !== oldValue) {
             this[privateName] = value;
+            if (opt_setHook)
+              opt_setHook.call(this, value, oldValue);
             dispatchPropertyChange(this, name, value, oldValue);
           }
         };
 
       case PropertyKind.ATTR:
+        var attributeName = getAttributeName(name);
         return function(value) {
-          var oldValue = this[name];
+          var oldValue = this[attributeName];
           if (value !== oldValue) {
-            this.setAttribute(name, value);
+            if (value == undefined)
+              this.removeAttribute(attributeName);
+            else
+              this.setAttribute(attributeName, value);
+            if (opt_setHook)
+              opt_setHook.call(this, value, oldValue);
             dispatchPropertyChange(this, name, value, oldValue);
           }
         };
 
       case PropertyKind.BOOL_ATTR:
+        var attributeName = getAttributeName(name);
         return function(value) {
-          var oldValue = this[name];
+          var oldValue = this[attributeName];
           if (value !== oldValue) {
             if (value)
-              this.setAttribute(name, name);
+              this.setAttribute(attributeName, name);
             else
-              this.removeAttribute(name);
+              this.removeAttribute(attributeName);
+            if (opt_setHook)
+              opt_setHook.call(this, value, oldValue);
             dispatchPropertyChange(this, name, value, oldValue);
           }
         };
@@ -210,29 +255,23 @@ const cr = (function() {
   /**
    * Defines a property on an object. When the setter changes the value a
    * property change event with the type {@code name + 'Change'} is fired.
-   * @param {!Object} The object to define the property for.
-   * @param {string} The name of the property.
+   * @param {!Object} obj The object to define the property for.
+   * @param {string} name The name of the property.
    * @param {cr.PropertyKind=} opt_kind What kind of underlying storage to use.
-   * @param {*} opt_defaultValue The default value.
+   * @param {function(*):void} opt_setHook A function to run after the
+   *     property is set, but before the propertyChange event is fired.
    */
-  function defineProperty(obj, name, opt_kind, opt_default) {
+  function defineProperty(obj, name, opt_kind, opt_setHook) {
     if (typeof obj == 'function')
       obj = obj.prototype;
 
     var kind = opt_kind || PropertyKind.JS;
 
-    if (!obj.__lookupGetter__(name)) {
-      // For js properties we set the default value on the prototype.
-      if (kind == PropertyKind.JS && arguments.length > 3)  {
-        var privateName = name + '_';
-        obj[privateName] = opt_default;
-      }
-      obj.__defineGetter__(name, getGetter(name, kind, opt_default));
-    }
+    if (!obj.__lookupGetter__(name))
+      obj.__defineGetter__(name, getGetter(name, kind));
 
-    if (!obj.__lookupSetter__(name)) {
-      obj.__defineSetter__(name, getSetter(name, kind));
-    }
+    if (!obj.__lookupSetter__(name))
+      obj.__defineSetter__(name, getSetter(name, kind, opt_setHook));
   }
 
   /**
@@ -333,6 +372,8 @@ const cr = (function() {
     isWindows: isWindows,
     isLinux: isLinux,
     isViews: isViews,
+    isTouchOptimized: isTouchOptimized,
+    enablePlatformSpecificCSSRules: enablePlatformSpecificCSSRules,
     define: define,
     defineProperty: defineProperty,
     PropertyKind: PropertyKind,
