@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,6 +9,7 @@
 #include "base/hash_tables.h"
 #include "base/lazy_instance.h"
 #include "grit/theme_resources_map.h"
+#include "grit/theme_resources_standard_map.h"
 
 namespace {
 
@@ -23,6 +24,10 @@ class ThemeMap {
     for (size_t i = 0; i < kThemeResourcesSize; ++i) {
       id_map_[kThemeResources[i].name] = kThemeResources[i].value;
     }
+    for (size_t i = 0; i < kThemeResourcesStandardSize; ++i) {
+      id_map_[kThemeResourcesStandard[i].name] =
+          kThemeResourcesStandard[i].value;
+    }
   }
 
   int GetId(const std::string& resource_name) {
@@ -36,7 +41,7 @@ class ThemeMap {
   StringIntMap id_map_;
 };
 
-static base::LazyInstance<ThemeMap> g_theme_ids(base::LINKER_INITIALIZED);
+static base::LazyInstance<ThemeMap> g_theme_ids = LAZY_INSTANCE_INITIALIZER;
 
 }  // namespace
 
