@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -52,6 +52,7 @@
 //           <url>,<weight> where weight is an integer indicating the number of
 //           unique views for the URL.
 
+#include <algorithm>
 #include <fstream>
 #include <vector>
 
@@ -59,19 +60,19 @@
 #include "base/file_path.h"
 #include "base/file_util.h"
 #include "base/logging.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
 #include "base/rand_util.h"
-#include "base/scoped_ptr.h"
-#include "base/sha2.h"
 #include "base/string_number_conversions.h"
+#include "base/string_piece.h"
 #include "base/string_util.h"
 #include "base/time.h"
 #include "chrome/browser/safe_browsing/bloom_filter.h"
 #include "chrome/browser/safe_browsing/safe_browsing_util.h"
 #include "chrome/common/chrome_paths.h"
-#include "chrome/common/sqlite_compiled_statement.h"
-#include "chrome/common/sqlite_utils.h"
+#include "crypto/sha2.h"
 #include "googleurl/src/gurl.h"
+#include "sql/statement.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using base::Time;
@@ -79,20 +80,6 @@ using base::TimeDelta;
 
 namespace {
 
-// Ensures the SafeBrowsing database is closed properly.
-class ScopedPerfDatabase {
- public:
-  explicit ScopedPerfDatabase(sqlite3* db) : db_(db) {}
-  ~ScopedPerfDatabase() {
-    sqlite3_close(db_);
-  }
-
- private:
-  sqlite3* db_;
-
-  DISALLOW_COPY_AND_ASSIGN(ScopedPerfDatabase);
-};
-
 // Command line flags.
 const char kFilterVerbose[] = "filter-verbose";
 const char kFilterStart[] = "filter-start";
@@ -101,7 +88,7 @@ const char kFilterCsv[] = "filter-csv";
 const char kFilterNumChecks[] = "filter-num-checks";
 
 // Number of hash checks to make during performance testing.
-static const int kNumHashChecks = 10000000;
+const int kNumHashChecks = 10000000;
 
 // Returns the path to the data used in this test, relative to the top of the
 // source directory.
@@ -139,35 +126,30 @@ void BuildBloomFilter(int size_multiplier,
 // to look up a given prefix than performing SQL queries.
 bool ReadDatabase(const FilePath& path, std::vector<SBPrefix>* prefixes) {
   FilePath database_file = path.Append(FILE_PATH_LITERAL("database"));
-  sqlite3* db = NULL;
-  if (sqlite_utils::OpenSqliteDb(database_file, &db) != SQLITE_OK) {
-    sqlite3_close(db);
+  sql::Connection db;
+  if (!db.Open(database_file))
     return false;
-  }
-
-  ScopedPerfDatabase database(db);
-  scoped_ptr<SqliteStatementCache> sql_cache(new SqliteStatementCache(db));
 
   // Get the number of items in the add_prefix table.
-  std::string sql = "SELECT COUNT(*) FROM add_prefix";
-  SQLITE_UNIQUE_STATEMENT(count_statement, *sql_cache, sql.c_str());
-  if (!count_statement.is_valid())
+  const char* query = "SELECT COUNT(*) FROM add_prefix";
+  sql::Statement count_statement(db.GetCachedStatement(SQL_FROM_HERE, query));
+  if (!count_statement)
     return false;
 
-  if (count_statement->step() != SQLITE_ROW)
+  if (!count_statement.Step())
     return false;
 
-  const int count = count_statement->column_int(0);
+  const int count = count_statement.ColumnInt(0);
 
-  // Load them into a prefix vector and sort
+  // Load them into a prefix vector and sort.
   prefixes->reserve(count);
-  sql = "SELECT prefix FROM add_prefix";
-  SQLITE_UNIQUE_STATEMENT(prefix_statement, *sql_cache, sql.c_str());
-  if (!prefix_statement.is_valid())
+  query = "SELECT prefix FROM add_prefix";
+  sql::Statement prefix_statement(db.GetCachedStatement(SQL_FROM_HERE, query));
+  if (!prefix_statement)
     return false;
 
-  while (prefix_statement->step() == SQLITE_ROW)
-    prefixes->push_back(prefix_statement->column_int(0));
+  while (prefix_statement.Step())
+    prefixes->push_back(prefix_statement.ColumnInt(0));
 
   DCHECK(static_cast<int>(prefixes->size()) == count);
   sort(prefixes->begin(), prefixes->end());
@@ -195,7 +177,7 @@ int GeneratePrefixHits(const std::string url,
   for (size_t i = 0; i < hosts.size(); ++i) {
     for (size_t j = 0; j < paths.size(); ++j) {
       SBPrefix prefix;
-      base::SHA256HashString(hosts[i] + paths[j], &prefix, sizeof(prefix));
+      crypto::SHA256HashString(hosts[i] + paths[j], &prefix, sizeof(prefix));
       if (bloom_filter->Exists(prefix))
         prefixes->push_back(prefix);
     }
@@ -238,7 +220,7 @@ void CalculateBloomFilterFalsePositives(
 
   // Read in data file line at a time.
   FilePath url_file = data_dir.Append(FILE_PATH_LITERAL("urls"));
-  std::ifstream url_stream(WideToASCII(url_file.ToWStringHack()).c_str());
+  std::ifstream url_stream(url_file.value().c_str());
 
   // Keep track of stats
   int hits = 0;
@@ -261,7 +243,8 @@ void CalculateBloomFilterFalsePositives(
     if (use_weights) {
       std::string::size_type pos = url.find_last_of(",");
       if (pos != std::string::npos) {
-        base::StringToInt(url.begin() + pos + 1, url.end(), &weight);
+        base::StringToInt(base::StringPiece(url.begin() + pos + 1, url.end()),
+                          &weight);
         url = url.substr(0, pos);
       }
     }
@@ -380,4 +363,3 @@ TEST(SafeBrowsingBloomFilter, HashTime) {
             << ", per-check (us): "        << time_per_check
             << std::endl;
 }
-
