@@ -7,26 +7,29 @@
 #ifndef NDEBUG
 #include "base/base64.h"
 #endif
+#include "base/bind.h"
 #include "base/environment.h"
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
 #include "base/rand_util.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
-#include "base/task.h"
+#include "base/stringprintf.h"
 #include "base/timer.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/safe_browsing/protocol_parser.h"
 #include "chrome/browser/safe_browsing/safe_browsing_service.h"
 #include "chrome/common/chrome_version_info.h"
 #include "chrome/common/env_vars.h"
-#include "chrome/common/net/url_request_context_getter.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/common/url_fetcher.h"
 #include "net/base/escape.h"
 #include "net/base/load_flags.h"
+#include "net/url_request/url_request_context_getter.h"
 #include "net/url_request/url_request_status.h"
 
 using base::Time;
 using base::TimeDelta;
+using content::BrowserThread;
 
 // Maximum time, in seconds, from start up before we must issue an update query.
 static const int kSbTimerStartIntervalSec = 5 * 60;
@@ -47,7 +50,7 @@ class SBProtocolManagerFactoryImpl : public SBProtocolManagerFactory {
       const std::string& client_name,
       const std::string& client_key,
       const std::string& wrapped_key,
-      URLRequestContextGetter* request_context_getter,
+      net::URLRequestContextGetter* request_context_getter,
       const std::string& info_url_prefix,
       const std::string& mackey_url_prefix,
       bool disable_auto_update) {
@@ -71,7 +74,7 @@ SafeBrowsingProtocolManager* SafeBrowsingProtocolManager::Create(
     const std::string& client_name,
     const std::string& client_key,
     const std::string& wrapped_key,
-    URLRequestContextGetter* request_context_getter,
+    net::URLRequestContextGetter* request_context_getter,
     const std::string& info_url_prefix,
     const std::string& mackey_url_prefix,
     bool disable_auto_update) {
@@ -89,7 +92,7 @@ SafeBrowsingProtocolManager::SafeBrowsingProtocolManager(
     const std::string& client_name,
     const std::string& client_key,
     const std::string& wrapped_key,
-    URLRequestContextGetter* request_context_getter,
+    net::URLRequestContextGetter* request_context_getter,
     const std::string& http_url_prefix,
     const std::string& https_url_prefix,
     bool disable_auto_update)
@@ -126,9 +129,15 @@ SafeBrowsingProtocolManager::SafeBrowsingProtocolManager(
 }
 
 // static
-void SafeBrowsingProtocolManager::RecordGetHashResult(ResultType result_type) {
-  UMA_HISTOGRAM_ENUMERATION("SB2.GetHashResult", result_type,
-                            GET_HASH_RESULT_MAX);
+void SafeBrowsingProtocolManager::RecordGetHashResult(
+    bool is_download, ResultType result_type) {
+  if (is_download) {
+    UMA_HISTOGRAM_ENUMERATION("SB2.GetHashResultDownload", result_type,
+                              GET_HASH_RESULT_MAX);
+  } else {
+    UMA_HISTOGRAM_ENUMERATION("SB2.GetHashResult", result_type,
+                              GET_HASH_RESULT_MAX);
+  }
 }
 
 SafeBrowsingProtocolManager::~SafeBrowsingProtocolManager() {
@@ -161,16 +170,17 @@ void SafeBrowsingProtocolManager::GetFullHash(
   }
   bool use_mac = !client_key_.empty();
   GURL gethash_url = GetHashUrl(use_mac);
-  URLFetcher* fetcher = new URLFetcher(gethash_url, URLFetcher::POST, this);
+  content::URLFetcher* fetcher = content::URLFetcher::Create(
+      gethash_url, content::URLFetcher::POST, this);
   hash_requests_[fetcher] = check;
 
   std::string get_hash;
   SafeBrowsingProtocolParser parser;
   parser.FormatGetHash(prefixes, &get_hash);
 
-  fetcher->set_load_flags(net::LOAD_DISABLE_CACHE);
-  fetcher->set_request_context(request_context_getter_);
-  fetcher->set_upload_data("text/plain", get_hash);
+  fetcher->SetLoadFlags(net::LOAD_DISABLE_CACHE);
+  fetcher->SetRequestContext(request_context_getter_);
+  fetcher->SetUploadData("text/plain", get_hash);
   fetcher->Start();
 }
 
@@ -188,7 +198,7 @@ void SafeBrowsingProtocolManager::GetNextUpdate() {
     IssueUpdateRequest();
 }
 
-// URLFetcher::Delegate implementation -----------------------------------------
+// content::URLFetcherDelegate implementation ----------------------------------
 
 // All SafeBrowsing request responses are handled here.
 // TODO(paulg): Clarify with the SafeBrowsing team whether a failed parse of a
@@ -199,22 +209,17 @@ void SafeBrowsingProtocolManager::GetNextUpdate() {
 //              do will report all the chunks we have. If that chunk is still
 //              required, the SafeBrowsing servers will tell us to get it again.
 void SafeBrowsingProtocolManager::OnURLFetchComplete(
-    const URLFetcher* source,
-    const GURL& url,
-    const net::URLRequestStatus& status,
-    int response_code,
-    const ResponseCookies& cookies,
-    const std::string& data) {
-  scoped_ptr<const URLFetcher> fetcher;
+    const content::URLFetcher* source) {
+  scoped_ptr<const content::URLFetcher> fetcher;
   bool parsed_ok = true;
   bool must_back_off = false;  // Reduce SafeBrowsing service query frequency.
 
   // See if this is a safebrowsing report fetcher. We don't take any action for
   // the response to those.
-  std::set<const URLFetcher*>::iterator sit = safebrowsing_reports_.find(
-      source);
+  std::set<const content::URLFetcher*>::iterator sit =
+      safebrowsing_reports_.find(source);
   if (sit != safebrowsing_reports_.end()) {
-    const URLFetcher* report = *sit;
+    const content::URLFetcher* report = *sit;
     safebrowsing_reports_.erase(sit);
     delete report;
     return;
@@ -227,23 +232,26 @@ void SafeBrowsingProtocolManager::OnURLFetchComplete(
     SafeBrowsingService::SafeBrowsingCheck* check = it->second;
     std::vector<SBFullHashResult> full_hashes;
     bool can_cache = false;
-    if (response_code == 200 || response_code == 204) {
+    if (source->GetResponseCode() == 200 || source->GetResponseCode() == 204) {
       // For tracking our GetHash false positive (204) rate, compared to real
       // (200) responses.
-      if (response_code == 200)
-        RecordGetHashResult(GET_HASH_STATUS_200);
+      if (source->GetResponseCode() == 200)
+        RecordGetHashResult(check->is_download, GET_HASH_STATUS_200);
       else
-        RecordGetHashResult(GET_HASH_STATUS_204);
+        RecordGetHashResult(check->is_download, GET_HASH_STATUS_204);
       can_cache = true;
       gethash_error_count_ = 0;
       gethash_back_off_mult_ = 1;
       bool re_key = false;
       SafeBrowsingProtocolParser parser;
-      parsed_ok = parser.ParseGetHash(data.data(),
-                                      static_cast<int>(data.length()),
-                                      client_key_,
-                                      &re_key,
-                                      &full_hashes);
+      std::string data;
+      source->GetResponseAsString(&data);
+      parsed_ok = parser.ParseGetHash(
+          data.data(),
+          static_cast<int>(data.length()),
+          client_key_,
+          &re_key,
+          &full_hashes);
       if (!parsed_ok) {
         // If we fail to parse it, we must still inform the SafeBrowsingService
         // so that it doesn't hold up the user's request indefinitely. Not sure
@@ -255,12 +263,12 @@ void SafeBrowsingProtocolManager::OnURLFetchComplete(
       }
     } else {
       HandleGetHashError(Time::Now());
-      if (status.status() == net::URLRequestStatus::FAILED) {
-        VLOG(1) << "SafeBrowsing GetHash request for: " << source->url()
-                << " failed with os error: " << status.os_error();
+      if (source->GetStatus().status() == net::URLRequestStatus::FAILED) {
+        VLOG(1) << "SafeBrowsing GetHash request for: " << source->GetURL()
+                << " failed with error: " << source->GetStatus().error();
       } else {
-        VLOG(1) << "SafeBrowsing GetHash request for: " << source->url()
-                << " failed with error: " << response_code;
+        VLOG(1) << "SafeBrowsing GetHash request for: " << source->GetURL()
+                << " failed with error: " << source->GetResponseCode();
       }
     }
 
@@ -285,13 +293,14 @@ void SafeBrowsingProtocolManager::OnURLFetchComplete(
       update_timer_.Stop();
     }
 
-    if (response_code == 200) {
+    if (source->GetResponseCode() == 200) {
       // We have data from the SafeBrowsing service.
-      parsed_ok = HandleServiceResponse(source->url(),
-                                        data.data(),
-                                        static_cast<int>(data.length()));
+      std::string data;
+      source->GetResponseAsString(&data);
+      parsed_ok = HandleServiceResponse(
+          source->GetURL(), data.data(), static_cast<int>(data.length()));
       if (!parsed_ok) {
-        VLOG(1) << "SafeBrowsing request for: " << source->url()
+        VLOG(1) << "SafeBrowsing request for: " << source->GetURL()
                 << " failed parse.";
         must_back_off = true;
         chunk_request_urls_.clear();
@@ -329,12 +338,12 @@ void SafeBrowsingProtocolManager::OnURLFetchComplete(
       if (request_type_ == CHUNK_REQUEST)
         chunk_request_urls_.clear();
       UpdateFinished(false);
-      if (status.status() == net::URLRequestStatus::FAILED) {
-        VLOG(1) << "SafeBrowsing request for: " << source->url()
-                << " failed with os error: " << status.os_error();
+      if (source->GetStatus().status() == net::URLRequestStatus::FAILED) {
+        VLOG(1) << "SafeBrowsing request for: " << source->GetURL()
+                << " failed with error: " << source->GetStatus().error();
       } else {
-        VLOG(1) << "SafeBrowsing request for: " << source->url()
-                << " failed with error: " << response_code;
+        VLOG(1) << "SafeBrowsing request for: " << source->GetURL()
+                << " failed with error: " << source->GetResponseCode();
       }
     }
   }
@@ -424,7 +433,7 @@ bool SafeBrowsingProtocolManager::HandleServiceResponse(const GURL& url,
         std::string data_str;
         data_str.assign(data, length);
         std::string encoded_chunk;
-        base::Base64Encode(data, &encoded_chunk);
+        base::Base64Encode(data_str, &encoded_chunk);
         VLOG(1) << "ParseChunk error for chunk: " << chunk_url.url
                 << ", client_key: " << client_key_
                 << ", wrapped_key: " << wrapped_key_
@@ -455,9 +464,8 @@ bool SafeBrowsingProtocolManager::HandleServiceResponse(const GURL& url,
       wrapped_key_ = wrapped_key;
       BrowserThread::PostTask(
           BrowserThread::UI, FROM_HERE,
-          NewRunnableMethod(
-              sb_service_, &SafeBrowsingService::OnNewMacKeys, client_key_,
-              wrapped_key_));
+          base::Bind(&SafeBrowsingService::OnNewMacKeys,
+                     sb_service_, client_key_, wrapped_key_));
       break;
     }
 
@@ -495,8 +503,8 @@ void SafeBrowsingProtocolManager::ForceScheduleNextUpdate(
   DCHECK_GE(next_update_msec, 0);
   // Unschedule any current timer.
   update_timer_.Stop();
-  update_timer_.Start(TimeDelta::FromMilliseconds(next_update_msec), this,
-                      &SafeBrowsingProtocolManager::GetNextUpdate);
+  update_timer_.Start(FROM_HERE, TimeDelta::FromMilliseconds(next_update_msec),
+                      this, &SafeBrowsingProtocolManager::GetNextUpdate);
 }
 
 // According to section 5 of the SafeBrowsing protocol specification, we must
@@ -555,9 +563,10 @@ void SafeBrowsingProtocolManager::IssueChunkRequest() {
   DCHECK(!next_chunk.url.empty());
   GURL chunk_url = NextChunkUrl(next_chunk.url);
   request_type_ = CHUNK_REQUEST;
-  request_.reset(new URLFetcher(chunk_url, URLFetcher::GET, this));
-  request_->set_load_flags(net::LOAD_DISABLE_CACHE);
-  request_->set_request_context(request_context_getter_);
+  request_.reset(content::URLFetcher::Create(
+      chunk_url, content::URLFetcher::GET, this));
+  request_->SetLoadFlags(net::LOAD_DISABLE_CACHE);
+  request_->SetRequestContext(request_context_getter_);
   chunk_request_start_ = base::Time::Now();
   request_->Start();
 }
@@ -565,9 +574,10 @@ void SafeBrowsingProtocolManager::IssueChunkRequest() {
 void SafeBrowsingProtocolManager::IssueKeyRequest() {
   GURL key_url = MacKeyUrl();
   request_type_ = GETKEY_REQUEST;
-  request_.reset(new URLFetcher(key_url, URLFetcher::GET, this));
-  request_->set_load_flags(net::LOAD_DISABLE_CACHE);
-  request_->set_request_context(request_context_getter_);
+  request_.reset(content::URLFetcher::Create(
+      key_url, content::URLFetcher::GET, this));
+  request_->SetLoadFlags(net::LOAD_DISABLE_CACHE);
+  request_->SetRequestContext(request_context_getter_);
   request_->Start();
 }
 
@@ -606,14 +616,16 @@ void SafeBrowsingProtocolManager::OnGetChunksComplete(
         SBListChunkRanges(safe_browsing_util::kMalwareList), use_mac));
 
   GURL update_url = UpdateUrl(use_mac);
-  request_.reset(new URLFetcher(update_url, URLFetcher::POST, this));
-  request_->set_load_flags(net::LOAD_DISABLE_CACHE);
-  request_->set_request_context(request_context_getter_);
-  request_->set_upload_data("text/plain", list_data);
+  request_.reset(content::URLFetcher::Create(
+      update_url, content::URLFetcher::POST, this));
+  request_->SetLoadFlags(net::LOAD_DISABLE_CACHE);
+  request_->SetRequestContext(request_context_getter_);
+  request_->SetUploadData("text/plain", list_data);
   request_->Start();
 
   // Begin the update request timeout.
-  update_timer_.Start(TimeDelta::FromSeconds(kSbMaxUpdateWaitSec), this,
+  update_timer_.Start(FROM_HERE, TimeDelta::FromSeconds(kSbMaxUpdateWaitSec),
+                      this,
                       &SafeBrowsingProtocolManager::UpdateResponseTimeout);
 }
 
@@ -643,13 +655,19 @@ void SafeBrowsingProtocolManager::ReportSafeBrowsingHit(
     const GURL& page_url,
     const GURL& referrer_url,
     bool is_subresource,
-    SafeBrowsingService::UrlCheckResult threat_type) {
+    SafeBrowsingService::UrlCheckResult threat_type,
+    const std::string& post_data) {
   GURL report_url = SafeBrowsingHitUrl(malicious_url, page_url,
                                        referrer_url, is_subresource,
                                        threat_type);
-  URLFetcher* report = new URLFetcher(report_url, URLFetcher::GET, this);
-  report->set_load_flags(net::LOAD_DISABLE_CACHE);
-  report->set_request_context(request_context_getter_);
+  content::URLFetcher* report = content::URLFetcher::Create(
+      report_url,
+      post_data.empty() ? content::URLFetcher::GET : content::URLFetcher::POST,
+      this);
+  report->SetLoadFlags(net::LOAD_DISABLE_CACHE);
+  report->SetRequestContext(request_context_getter_);
+  if (!post_data.empty())
+    report->SetUploadData("text/plain", post_data);
   report->Start();
   safebrowsing_reports_.insert(report);
 }
@@ -658,12 +676,13 @@ void SafeBrowsingProtocolManager::ReportSafeBrowsingHit(
 void SafeBrowsingProtocolManager::ReportMalwareDetails(
     const std::string& report) {
   GURL report_url = MalwareDetailsUrl();
-  URLFetcher* fetcher = new URLFetcher(report_url, URLFetcher::POST, this);
-  fetcher->set_load_flags(net::LOAD_DISABLE_CACHE);
-  fetcher->set_request_context(request_context_getter_);
-  fetcher->set_upload_data("application/octet-stream", report);
+  content::URLFetcher* fetcher = content::URLFetcher::Create(
+      report_url, content::URLFetcher::POST, this);
+  fetcher->SetLoadFlags(net::LOAD_DISABLE_CACHE);
+  fetcher->SetRequestContext(request_context_getter_);
+  fetcher->SetUploadData("application/octet-stream", report);
   // Don't try too hard to send reports on failures.
-  fetcher->set_automatically_retry_on_5xx(false);
+  fetcher->SetAutomaticallyRetryOn5xx(false);
   fetcher->Start();
   safebrowsing_reports_.insert(fetcher);
 }
@@ -715,9 +734,9 @@ std::string SafeBrowsingProtocolManager::ComposeUrl(
     const std::string& additional_query) {
   DCHECK(!prefix.empty() && !method.empty() &&
          !client_name.empty() && !version.empty());
-  std::string url = StringPrintf("%s/%s?client=%s&appver=%s&pver=2.2",
-                                 prefix.c_str(), method.c_str(),
-                                 client_name.c_str(), version.c_str());
+  std::string url = base::StringPrintf("%s/%s?client=%s&appver=%s&pver=2.2",
+                                       prefix.c_str(), method.c_str(),
+                                       client_name.c_str(), version.c_str());
   if (!additional_query.empty()) {
     DCHECK(url.find("?") != std::string::npos);
     url.append("&");
@@ -756,23 +775,44 @@ GURL SafeBrowsingProtocolManager::SafeBrowsingHitUrl(
     const GURL& referrer_url, bool is_subresource,
     SafeBrowsingService::UrlCheckResult threat_type) const {
   DCHECK(threat_type == SafeBrowsingService::URL_MALWARE ||
-         threat_type == SafeBrowsingService::URL_PHISHING);
+         threat_type == SafeBrowsingService::URL_PHISHING ||
+         threat_type == SafeBrowsingService::BINARY_MALWARE_URL ||
+         threat_type == SafeBrowsingService::BINARY_MALWARE_HASH ||
+         threat_type == SafeBrowsingService::CLIENT_SIDE_PHISHING_URL);
   // The malware and phishing hits go over HTTP.
   std::string url = ComposeUrl(http_url_prefix_, "report", client_name_,
                                version_, additional_query_);
-  return GURL(StringPrintf("%s&evts=%s&evtd=%s&evtr=%s&evhr=%s&evtb=%d",
-      url.c_str(),
-      (threat_type == SafeBrowsingService::URL_MALWARE) ?
-                           "malblhit" : "phishblhit",
-      EscapeQueryParamValue(malicious_url.spec(), true).c_str(),
-      EscapeQueryParamValue(page_url.spec(), true).c_str(),
-      EscapeQueryParamValue(referrer_url.spec(), true).c_str(),
+  std::string threat_list = "none";
+  switch (threat_type) {
+    case SafeBrowsingService::URL_MALWARE:
+      threat_list = "malblhit";
+      break;
+    case SafeBrowsingService::URL_PHISHING:
+      threat_list = "phishblhit";
+      break;
+    case SafeBrowsingService::BINARY_MALWARE_URL:
+      threat_list = "binurlhit";
+      break;
+    case SafeBrowsingService::BINARY_MALWARE_HASH:
+      threat_list = "binhashhit";
+      break;
+    case SafeBrowsingService::CLIENT_SIDE_PHISHING_URL:
+      threat_list = "phishcsdhit";
+      break;
+    default:
+      NOTREACHED();
+  }
+  return GURL(base::StringPrintf("%s&evts=%s&evtd=%s&evtr=%s&evhr=%s&evtb=%d",
+      url.c_str(), threat_list.c_str(),
+      net::EscapeQueryParamValue(malicious_url.spec(), true).c_str(),
+      net::EscapeQueryParamValue(page_url.spec(), true).c_str(),
+      net::EscapeQueryParamValue(referrer_url.spec(), true).c_str(),
       is_subresource));
 }
 
 GURL SafeBrowsingProtocolManager::MalwareDetailsUrl() const {
   // The malware details go over HTTPS.
-  std::string url = StringPrintf(
+  std::string url = base::StringPrintf(
           "%s/clientreport/malware?client=%s&appver=%s&pver=1.0",
           https_url_prefix_.c_str(),
           client_name_.c_str(),
