@@ -18,26 +18,22 @@
 
 #include "base/gtest_prod_util.h"
 #include "base/hash_tables.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/time.h"
 #include "base/timer.h"
 #include "chrome/browser/safe_browsing/chunk_range.h"
 #include "chrome/browser/safe_browsing/protocol_parser.h"
 #include "chrome/browser/safe_browsing/safe_browsing_service.h"
 #include "chrome/browser/safe_browsing/safe_browsing_util.h"
-#include "chrome/common/net/url_fetcher.h"
-
-namespace net {
-class URLRequestStatus;
-}  // namespace net
+#include "content/public/common/url_fetcher_delegate.h"
 
 #if defined(COMPILER_GCC)
 // Allows us to use URLFetchers in a hash_map with gcc (MSVC is okay without
 // specifying this).
 namespace __gnu_cxx {
 template<>
-struct hash<const URLFetcher*> {
-  size_t operator()(const URLFetcher* fetcher) const {
+struct hash<const content::URLFetcher*> {
+  size_t operator()(const content::URLFetcher* fetcher) const {
     return reinterpret_cast<size_t>(fetcher);
   }
 };
@@ -55,7 +51,7 @@ class SBProtocolManagerFactory {
       const std::string& client_name,
       const std::string& client_key,
       const std::string& wrapped_key,
-      URLRequestContextGetter* request_context_getter,
+      net::URLRequestContextGetter* request_context_getter,
       const std::string& info_url_prefix,
       const std::string& mackey_url_prefix,
       bool disable_auto_update) = 0;
@@ -63,7 +59,7 @@ class SBProtocolManagerFactory {
   DISALLOW_COPY_AND_ASSIGN(SBProtocolManagerFactory);
 };
 
-class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
+class SafeBrowsingProtocolManager : public content::URLFetcherDelegate {
   FRIEND_TEST_ALL_PREFIXES(SafeBrowsingProtocolManagerTest, TestBackOffTimes);
   FRIEND_TEST_ALL_PREFIXES(SafeBrowsingProtocolManagerTest, TestChunkStrings);
   FRIEND_TEST_ALL_PREFIXES(SafeBrowsingProtocolManagerTest, TestGetHashUrl);
@@ -93,7 +89,7 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
       const std::string& client_name,
       const std::string& client_key,
       const std::string& wrapped_key,
-      URLRequestContextGetter* request_context_getter,
+      net::URLRequestContextGetter* request_context_getter,
       const std::string& info_url_prefix,
       const std::string& mackey_url_prefix,
       bool disable_auto_update);
@@ -102,13 +98,8 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
   // of the SafeBrowsing service.
   virtual void Initialize();
 
-  // URLFetcher::Delegate interface.
-  virtual void OnURLFetchComplete(const URLFetcher* source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  // content::URLFetcherDelegate interface.
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
   // API used by the SafeBrowsingService for issuing queries. When the results
   // are available, SafeBrowsingService::HandleGetHashResults is called.
@@ -132,13 +123,14 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
   void OnChunkInserted();
 
   // For UMA users we report to Google when a SafeBrowsing interstitial is shown
-  // to the user.  We assume that the threat type is either URL_MALWARE or
-  // URL_PHISHING.
+  // to the user.  |threat_type| should be one of the types known by
+  // SafeBrowsingHitUrl.
   void ReportSafeBrowsingHit(const GURL& malicious_url,
                              const GURL& page_url,
                              const GURL& referrer_url,
                              bool is_subresource,
-                             SafeBrowsingService::UrlCheckResult threat_type);
+                             SafeBrowsingService::UrlCheckResult threat_type,
+                             const std::string& post_data);
 
   // Users can opt-in on the SafeBrowsing interstitial to send detailed
   // malware reports. |report| is the serialized report.
@@ -188,22 +180,26 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
     GET_HASH_RESULT_MAX
   };
 
-  // Record a GetHash result.
-  static void RecordGetHashResult(ResultType result_type);
+  // Record a GetHash result. |is_download| indicates if the get
+  // hash is triggered by download related lookup.
+  static void RecordGetHashResult(bool is_download,
+                                  ResultType result_type);
 
  protected:
   // Constructs a SafeBrowsingProtocolManager for |sb_service| that issues
   // network requests using |request_context_getter|. When |disable_auto_update|
   // is true, protocol manager won't schedule next update until
   // ForceScheduleNextUpdate is called.
-  SafeBrowsingProtocolManager(SafeBrowsingService* sb_service,
-                              const std::string& client_name,
-                              const std::string& client_key,
-                              const std::string& wrapped_key,
-                              URLRequestContextGetter* request_context_getter,
-                              const std::string& http_url_prefix,
-                              const std::string& https_url_prefix,
-                              bool disable_auto_update);
+  SafeBrowsingProtocolManager(
+      SafeBrowsingService* sb_service,
+      const std::string& client_name,
+      const std::string& client_key,
+      const std::string& wrapped_key,
+      net::URLRequestContextGetter* request_context_getter,
+      const std::string& http_url_prefix,
+      const std::string& https_url_prefix,
+      bool disable_auto_update);
+
  private:
   friend class SBProtocolManagerFactoryImpl;
 
@@ -312,7 +308,7 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
   // Current active request (in case we need to cancel) for updates or chunks
   // from the SafeBrowsing service. We can only have one of these outstanding
   // at any given time unlike GetHash requests, which are tracked separately.
-  scoped_ptr<URLFetcher> request_;
+  scoped_ptr<content::URLFetcher> request_;
 
   // The kind of request that is currently in progress.
   SafeBrowsingRequestType request_type_;
@@ -340,7 +336,7 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
   std::deque<ChunkUrl> chunk_request_urls_;
 
   // Map of GetHash requests.
-  typedef base::hash_map<const URLFetcher*,
+  typedef base::hash_map<const content::URLFetcher*,
                          SafeBrowsingService::SafeBrowsingCheck*> HashRequests;
   HashRequests hash_requests_;
 
@@ -382,7 +378,7 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
 
   // Track outstanding SafeBrowsing report fetchers for clean up.
   // We add both "hit" and "detail" fetchers in this set.
-  std::set<const URLFetcher*> safebrowsing_reports_;
+  std::set<const content::URLFetcher*> safebrowsing_reports_;
 
   // The safe browsing client name sent in each request.
   std::string client_name_;
@@ -392,7 +388,7 @@ class SafeBrowsingProtocolManager : public URLFetcher::Delegate {
   std::string additional_query_;
 
   // The context we use to issue network requests.
-  scoped_refptr<URLRequestContextGetter> request_context_getter_;
+  scoped_refptr<net::URLRequestContextGetter> request_context_getter_;
 
   // URL prefix where browser fetches safebrowsing chunk updates, hashes, and
   // reports hits to the safebrowsing list for UMA users.
