@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -8,28 +8,30 @@
 
 #include <string>
 
+#include "base/bind.h"
 #include "base/i18n/rtl.h"
 #include "base/lazy_instance.h"
 #include "base/string_number_conversions.h"
+#include "base/string_piece.h"
+#include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/dom_operation_notification_details.h"
-#include "chrome/browser/dom_ui/new_tab_ui.h"
 #include "chrome/browser/google/google_util.h"
-#include "chrome/browser/metrics/user_metrics.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/safe_browsing/malware_details.h"
 #include "chrome/browser/safe_browsing/safe_browsing_service.h"
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
 #include "chrome/browser/tab_contents/tab_util.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/ui/webui/ntp/new_tab_ui.h"
 #include "chrome/common/jstemplate_builder.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/user_metrics.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/browser_resources.h"
 #include "grit/generated_resources.h"
 #include "grit/locale_settings.h"
@@ -37,6 +39,12 @@
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
+using content::BrowserThread;
+using content::OpenURLParams;
+using content::Referrer;
+using content::UserMetricsAction;
+using content::WebContents;
+
 // For malware interstitial pages, we link the problematic URL to Google's
 // diagnostic page.
 #if defined(GOOGLE_CHROME_BUILD)
@@ -47,17 +55,17 @@ static const char* const kSbDiagnosticUrl =
     "http://safebrowsing.clients.google.com/safebrowsing/diagnostic?site=%s&client=chromium";
 #endif
 
-static const char* const kSbReportPhishingUrl =
+static const char* const kSbReportPhishingErrorUrl =
     "http://www.google.com/safebrowsing/report_error/";
 
 // URL for the "Learn more" link on the multi threat malware blocking page.
 static const char* const kLearnMoreMalwareUrl =
-    "http://www.google.com/support/bin/answer.py?answer=45449&topic=360"
+    "https://www.google.com/support/bin/answer.py?answer=45449&topic=360"
     "&sa=X&oi=malwarewarninglink&resnum=1&ct=help";
 
 // URL for the "Learn more" link on the phishing blocking page.
 static const char* const kLearnMorePhishingUrl =
-    "http://www.google.com/support/bin/answer.py?answer=106318";
+    "https://www.google.com/support/bin/answer.py?answer=106318";
 
 // URL for the "Safe Browsing Privacy Policies" link on the blocking page.
 // Note: this page is not yet localized.
@@ -76,6 +84,12 @@ static const char* const kPrivacyLinkHtml =
     "<a href=\"\" onclick=\"sendCommand('showPrivacy'); return false;\" "
     "onmousedown=\"return false;\">%s</a>";
 
+// After a malware interstitial where the user opted-in to the report
+// but clicked "proceed anyway", we delay the call to
+// MalwareDetails::FinishCollection() by this much time (in
+// milliseconds).
+static const int64 kMalwareDetailsProceedDelayMilliSeconds = 3000;
+
 // The commands returned by the page when the user performs an action.
 static const char* const kShowDiagnosticCommand = "showDiagnostic";
 static const char* const kReportErrorCommand = "reportError";
@@ -92,7 +106,7 @@ static const char* const kBoxChecked = "boxchecked";
 SafeBrowsingBlockingPageFactory* SafeBrowsingBlockingPage::factory_ = NULL;
 
 static base::LazyInstance<SafeBrowsingBlockingPage::UnsafeResourceMap>
-    g_unsafe_resource_map(base::LINKER_INITIALIZED);
+    g_unsafe_resource_map = LAZY_INSTANCE_INITIALIZER;
 
 // The default SafeBrowsingBlockingPageFactory.  Global, made a singleton so we
 // don't leak it.
@@ -101,9 +115,9 @@ class SafeBrowsingBlockingPageFactoryImpl
  public:
   SafeBrowsingBlockingPage* CreateSafeBrowsingPage(
       SafeBrowsingService* service,
-      TabContents* tab_contents,
+      WebContents* web_contents,
       const SafeBrowsingBlockingPage::UnsafeResourceList& unsafe_resources) {
-    return new SafeBrowsingBlockingPage(service, tab_contents,
+    return new SafeBrowsingBlockingPage(service, web_contents,
                                         unsafe_resources);
   }
 
@@ -117,23 +131,24 @@ class SafeBrowsingBlockingPageFactoryImpl
 };
 
 static base::LazyInstance<SafeBrowsingBlockingPageFactoryImpl>
-    g_safe_browsing_blocking_page_factory_impl(base::LINKER_INITIALIZED);
+    g_safe_browsing_blocking_page_factory_impl = LAZY_INSTANCE_INITIALIZER;
 
 SafeBrowsingBlockingPage::SafeBrowsingBlockingPage(
     SafeBrowsingService* sb_service,
-    TabContents* tab_contents,
+    WebContents* web_contents,
     const UnsafeResourceList& unsafe_resources)
-    : InterstitialPage(tab_contents,
-                       IsMainPage(unsafe_resources),
-                       unsafe_resources[0].url),
+    : ChromeInterstitialPage(web_contents,
+                             IsMainPageLoadBlocked(unsafe_resources),
+                             unsafe_resources[0].url),
+      malware_details_proceed_delay_ms_(
+          kMalwareDetailsProceedDelayMilliSeconds),
       sb_service_(sb_service),
-      is_main_frame_(IsMainPage(unsafe_resources)),
-      unsafe_resources_(unsafe_resources),
-      malware_details_(NULL) {
+      is_main_frame_load_blocked_(IsMainPageLoadBlocked(unsafe_resources)),
+      unsafe_resources_(unsafe_resources) {
   RecordUserAction(SHOW);
-  if (!is_main_frame_) {
+  if (!is_main_frame_load_blocked_) {
     navigation_entry_index_to_remove_ =
-        tab()->controller().last_committed_entry_index();
+        tab()->GetController().GetLastCommittedEntryIndex();
   } else {
     navigation_entry_index_to_remove_ = -1;
   }
@@ -147,12 +162,13 @@ SafeBrowsingBlockingPage::SafeBrowsingBlockingPage(
       unsafe_resources[0].threat_type == SafeBrowsingService::URL_MALWARE &&
       malware_details_ == NULL &&
       CanShowMalwareDetailsOption()) {
-    malware_details_ = new MalwareDetails(tab(), unsafe_resources[0]);
+    malware_details_ = MalwareDetails::NewMalwareDetails(
+        sb_service_, tab(), unsafe_resources[0]);
   }
 }
 
 bool SafeBrowsingBlockingPage::CanShowMalwareDetailsOption() {
-  return (!tab()->profile()->IsOffTheRecord() &&
+  return (!tab()->GetBrowserContext()->IsOffTheRecord() &&
           tab()->GetURL().SchemeIs(chrome::kHttpScheme));
 }
 
@@ -174,15 +190,20 @@ std::string SafeBrowsingBlockingPage::GetHTMLContents() {
     PopulateMultipleThreatStringDictionary(&strings);
     html = rb.GetRawDataResource(
         IDR_SAFE_BROWSING_MULTIPLE_THREAT_BLOCK).as_string();
-  } else if (unsafe_resources_[0].threat_type ==
-             SafeBrowsingService::URL_MALWARE) {
-    PopulateMalwareStringDictionary(&strings);
-    html = rb.GetRawDataResource(IDR_SAFE_BROWSING_MALWARE_BLOCK).as_string();
-  } else {  // Phishing.
-    DCHECK(unsafe_resources_[0].threat_type ==
-           SafeBrowsingService::URL_PHISHING);
-    PopulatePhishingStringDictionary(&strings);
-    html = rb.GetRawDataResource(IDR_SAFE_BROWSING_PHISHING_BLOCK).as_string();
+  } else {
+    SafeBrowsingService::UrlCheckResult threat_type =
+        unsafe_resources_[0].threat_type;
+    if (threat_type == SafeBrowsingService::URL_MALWARE) {
+      PopulateMalwareStringDictionary(&strings);
+      html = rb.GetRawDataResource(
+          IDR_SAFE_BROWSING_MALWARE_BLOCK).as_string();
+    } else {  // Phishing.
+      DCHECK(threat_type == SafeBrowsingService::URL_PHISHING ||
+             threat_type == SafeBrowsingService::CLIENT_SIDE_PHISHING_URL);
+      PopulatePhishingStringDictionary(&strings);
+      html = rb.GetRawDataResource(
+          IDR_SAFE_BROWSING_PHISHING_BLOCK).as_string();
+    }
   }
 
   return jstemplate_builder::GetTemplatesHtml(html, &strings, "template_root");
@@ -220,14 +241,16 @@ void SafeBrowsingBlockingPage::PopulateMultipleThreatStringDictionary(
   for (UnsafeResourceList::const_iterator iter = unsafe_resources_.begin();
        iter != unsafe_resources_.end(); ++iter) {
     const SafeBrowsingService::UnsafeResource& resource = *iter;
+    SafeBrowsingService::UrlCheckResult threat_type = resource.threat_type;
     DictionaryValue* current_error_strings = new DictionaryValue;
-    if (resource.threat_type == SafeBrowsingService::URL_MALWARE) {
+    if (threat_type == SafeBrowsingService::URL_MALWARE) {
       malware = true;
       current_error_strings->SetString("type", "malware");
       current_error_strings->SetString("typeLabel", malware_label);
       current_error_strings->SetString("errorLink", malware_link);
     } else {
-      DCHECK(resource.threat_type == SafeBrowsingService::URL_PHISHING);
+      DCHECK(threat_type == SafeBrowsingService::URL_PHISHING ||
+             threat_type == SafeBrowsingService::CLIENT_SIDE_PHISHING_URL);
       phishing = true;
       current_error_strings->SetString("type", "phishing");
       current_error_strings->SetString("typeLabel", phishing_label);
@@ -286,7 +309,7 @@ void SafeBrowsingBlockingPage::PopulateMultipleThreatStringDictionary(
 
 void SafeBrowsingBlockingPage::PopulateMalwareStringDictionary(
     DictionaryValue* strings) {
-  std::string diagnostic_link = StringPrintf(kSbDiagnosticHtml,
+  std::string diagnostic_link = base::StringPrintf(kSbDiagnosticHtml,
       l10n_util::GetStringUTF8(
         IDS_SAFE_BROWSING_MALWARE_DIAGNOSTIC_PAGE).c_str());
 
@@ -294,7 +317,7 @@ void SafeBrowsingBlockingPage::PopulateMalwareStringDictionary(
   // Check to see if we're blocking the main page, or a sub-resource on the
   // main page.
   string16 description1, description3, description5;
-  if (is_main_frame_) {
+  if (is_main_frame_load_blocked_) {
     description1 = l10n_util::GetStringFUTF16(
         IDS_SAFE_BROWSING_MALWARE_DESCRIPTION1, UTF8ToUTF16(url().host()));
   } else {
@@ -304,7 +327,7 @@ void SafeBrowsingBlockingPage::PopulateMalwareStringDictionary(
         UTF8ToUTF16(url().host()));
   }
 
-  std::string proceed_link = StringPrintf(kPLinkHtml,
+  std::string proceed_link = base::StringPrintf(kPLinkHtml,
       l10n_util::GetStringUTF8(IDS_SAFE_BROWSING_MALWARE_PROCEED_LINK).c_str());
   description3 =
       l10n_util::GetStringFUTF16(IDS_SAFE_BROWSING_MALWARE_DESCRIPTION3,
@@ -338,7 +361,7 @@ void SafeBrowsingBlockingPage::PopulateMalwareStringDictionary(
     // Show the checkbox for sending malware details.
     strings->SetBoolean(kDisplayCheckBox, true);
 
-    std::string privacy_link = StringPrintf(
+    std::string privacy_link = base::StringPrintf(
         kPrivacyLinkHtml,
         l10n_util::GetStringUTF8(
             IDS_SAFE_BROWSING_PRIVACY_POLICY_PAGE).c_str());
@@ -348,8 +371,9 @@ void SafeBrowsingBlockingPage::PopulateMalwareStringDictionary(
                            IDS_SAFE_BROWSING_MALWARE_REPORTING_AGREE,
                            UTF8ToUTF16(privacy_link)));
 
+    Profile* profile = Profile::FromBrowserContext(tab()->GetBrowserContext());
     const PrefService::Preference* pref =
-        tab()->profile()->GetPrefs()->FindPreference(
+        profile->GetPrefs()->FindPreference(
             prefs::kSafeBrowsingReportingEnabled);
 
     bool value;
@@ -363,8 +387,10 @@ void SafeBrowsingBlockingPage::PopulateMalwareStringDictionary(
 
 void SafeBrowsingBlockingPage::PopulatePhishingStringDictionary(
     DictionaryValue* strings) {
-  std::string proceed_link = StringPrintf(kPLinkHtml, l10n_util::GetStringUTF8(
-      IDS_SAFE_BROWSING_PHISHING_PROCEED_LINK).c_str());
+  std::string proceed_link = base::StringPrintf(
+      kPLinkHtml,
+      l10n_util::GetStringUTF8(
+          IDS_SAFE_BROWSING_PHISHING_PROCEED_LINK).c_str());
   string16 description3 = l10n_util::GetStringFUTF16(
       IDS_SAFE_BROWSING_PHISHING_DESCRIPTION3,
       UTF8ToUTF16(proceed_link));
@@ -405,22 +431,29 @@ void SafeBrowsingBlockingPage::CommandReceived(const std::string& cmd) {
   if (command == kLearnMoreCommand) {
     // User pressed "Learn more".
     GURL url;
-    if (unsafe_resources_[0].threat_type == SafeBrowsingService::URL_MALWARE) {
+    SafeBrowsingService::UrlCheckResult threat_type =
+        unsafe_resources_[0].threat_type;
+    if (threat_type == SafeBrowsingService::URL_MALWARE) {
       url = google_util::AppendGoogleLocaleParam(GURL(kLearnMoreMalwareUrl));
-    } else if (unsafe_resources_[0].threat_type ==
-               SafeBrowsingService::URL_PHISHING) {
+    } else if (threat_type == SafeBrowsingService::URL_PHISHING ||
+               threat_type == SafeBrowsingService::CLIENT_SIDE_PHISHING_URL) {
       url = google_util::AppendGoogleLocaleParam(GURL(kLearnMorePhishingUrl));
     } else {
       NOTREACHED();
     }
-    tab()->OpenURL(url, GURL(), CURRENT_TAB, PageTransition::LINK);
+
+    OpenURLParams params(
+        url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_LINK, false);
+    tab()->OpenURL(params);
     return;
   }
 
   if (command == kShowPrivacyCommand) {
     // User pressed "Safe Browsing privacy policy".
     GURL url(kSbPrivacyPolicyUrl);
-    tab()->OpenURL(url, GURL(), CURRENT_TAB, PageTransition::LINK);
+    OpenURLParams params(
+        url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_LINK, false);
+    tab()->OpenURL(params);
     return;
   }
 
@@ -431,8 +464,24 @@ void SafeBrowsingBlockingPage::CommandReceived(const std::string& cmd) {
   }
 
   if (command == kTakeMeBackCommand) {
-    DontProceed();
-    // We are deleted after this.
+    if (is_main_frame_load_blocked_) {
+      // If the load is blocked, we want to close the interstitial and discard
+      // the pending entry.
+      DontProceed();
+      // We are deleted after this.
+      return;
+    }
+
+    // Otherwise the offending entry has committed, and we need to go back or
+    // to a safe page.  We will close the interstitial when that page commits.
+    if (tab()->GetController().CanGoBack()) {
+      tab()->GetController().GoBack();
+    } else {
+      tab()->GetController().LoadURL(GURL(chrome::kChromeUINewTabURL),
+                                     content::Referrer(),
+                                     content::PAGE_TRANSITION_START_PAGE,
+                                     std::string());
+    }
     return;
   }
 
@@ -442,8 +491,9 @@ void SafeBrowsingBlockingPage::CommandReceived(const std::string& cmd) {
   size_t colon_index = command.find(':');
   if (colon_index != std::string::npos) {
     DCHECK(colon_index < command.size() - 1);
-    bool result = base::StringToInt(command.begin() + colon_index + 1,
-                                    command.end(),
+    bool result = base::StringToInt(base::StringPiece(command.begin() +
+                                                      colon_index + 1,
+                                                      command.end()),
                                     &element_index);
     command = command.substr(0, colon_index);
     DCHECK(result);
@@ -459,25 +509,35 @@ void SafeBrowsingBlockingPage::CommandReceived(const std::string& cmd) {
     // User pressed "Report error" for a phishing site.
     // Note that we cannot just put a link in the interstitial at this point.
     // It is not OK to navigate in the context of an interstitial page.
-    DCHECK(unsafe_resources_[element_index].threat_type ==
-           SafeBrowsingService::URL_PHISHING);
+    SafeBrowsingService::UrlCheckResult threat_type =
+        unsafe_resources_[element_index].threat_type;
+    DCHECK(threat_type == SafeBrowsingService::URL_PHISHING ||
+           threat_type == SafeBrowsingService::CLIENT_SIDE_PHISHING_URL);
     GURL report_url =
-        safe_browsing_util::GeneratePhishingReportUrl(kSbReportPhishingUrl,
-                                                      bad_url_spec);
-    tab()->OpenURL(report_url, GURL(), CURRENT_TAB, PageTransition::LINK);
+        safe_browsing_util::GeneratePhishingReportUrl(
+            kSbReportPhishingErrorUrl,
+            bad_url_spec,
+            threat_type == SafeBrowsingService::CLIENT_SIDE_PHISHING_URL);
+    OpenURLParams params(
+        report_url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_LINK,
+        false);
+    tab()->OpenURL(params);
     return;
   }
 
   if (command == kShowDiagnosticCommand) {
     // We're going to take the user to Google's SafeBrowsing diagnostic page.
     std::string diagnostic =
-        StringPrintf(kSbDiagnosticUrl,
-                     EscapeQueryParamValue(bad_url_spec, true).c_str());
+        base::StringPrintf(kSbDiagnosticUrl,
+            net::EscapeQueryParamValue(bad_url_spec, true).c_str());
     GURL diagnostic_url(diagnostic);
     diagnostic_url = google_util::AppendGoogleLocaleParam(diagnostic_url);
     DCHECK(unsafe_resources_[element_index].threat_type ==
            SafeBrowsingService::URL_MALWARE);
-    tab()->OpenURL(diagnostic_url, GURL(), CURRENT_TAB, PageTransition::LINK);
+    OpenURLParams params(
+        diagnostic_url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_LINK,
+        false);
+    tab()->OpenURL(params);
     return;
   }
 
@@ -485,13 +545,15 @@ void SafeBrowsingBlockingPage::CommandReceived(const std::string& cmd) {
 }
 
 void SafeBrowsingBlockingPage::SetReportingPreference(bool report) {
-  PrefService* pref = tab()->profile()->GetPrefs();
+  Profile* profile = Profile::FromBrowserContext(tab()->GetBrowserContext());
+  PrefService* pref = profile->GetPrefs();
   pref->SetBoolean(prefs::kSafeBrowsingReportingEnabled, report);
 }
 
 void SafeBrowsingBlockingPage::Proceed() {
   RecordUserAction(PROCEED);
-  FinishMalwareDetails();  // Send the malware details, if we opted to.
+  // Send the malware details, if we opted to.
+  FinishMalwareDetails(malware_details_proceed_delay_ms_);
 
   NotifySafeBrowsingService(sb_service_, unsafe_resources_, true);
 
@@ -529,7 +591,8 @@ void SafeBrowsingBlockingPage::DontProceed() {
   }
 
   RecordUserAction(DONT_PROCEED);
-  FinishMalwareDetails();  // Send the malware details, if we opted to.
+  // Send the malware details, if we opted to.
+  FinishMalwareDetails(0);  // No delay
 
   NotifySafeBrowsingService(sb_service_, unsafe_resources_, false);
 
@@ -544,10 +607,16 @@ void SafeBrowsingBlockingPage::DontProceed() {
 
   // We don't remove the navigation entry if the tab is being destroyed as this
   // would trigger a navigation that would cause trouble as the render view host
-  // for the tab has by then already been destroyed.
-  if (navigation_entry_index_to_remove_ != -1 && !tab()->is_being_destroyed()) {
-    tab()->controller().RemoveEntryAtIndex(navigation_entry_index_to_remove_,
-                                           GURL(chrome::kChromeUINewTabURL));
+  // for the tab has by then already been destroyed.  We also don't delete the
+  // current entry if it has been committed again, which is possible on a page
+  // that had a subresource warning.
+  int last_committed_index =
+      tab()->GetController().GetLastCommittedEntryIndex();
+  if (navigation_entry_index_to_remove_ != -1 &&
+      navigation_entry_index_to_remove_ != last_committed_index &&
+      !tab()->IsBeingDestroyed()) {
+    tab()->GetController().RemoveEntryAtIndex(
+        navigation_entry_index_to_remove_);
     navigation_entry_index_to_remove_ = -1;
   }
   InterstitialPage::DontProceed();
@@ -563,7 +632,8 @@ void SafeBrowsingBlockingPage::RecordUserAction(BlockingPageEvent event) {
   PopulateMultipleThreatStringDictionary(&strings);
 
   string16 title;
-  DCHECK(strings.GetString("title", &title));
+  bool success = strings.GetString("title", &title);
+  DCHECK(success);
 
   std::string action = "SBInterstitial";
   if (title ==
@@ -592,25 +662,24 @@ void SafeBrowsingBlockingPage::RecordUserAction(BlockingPageEvent event) {
       NOTREACHED() << "Unexpected event: " << event;
   }
 
-  UserMetrics::RecordComputedAction(action);
+  content::RecordComputedAction(action);
 }
 
-void SafeBrowsingBlockingPage::FinishMalwareDetails() {
+void SafeBrowsingBlockingPage::FinishMalwareDetails(int64 delay_ms) {
   if (malware_details_ == NULL)
     return;  // Not all interstitials have malware details (eg phishing).
 
+  Profile* profile = Profile::FromBrowserContext(tab()->GetBrowserContext());
   const PrefService::Preference* pref =
-      tab()->profile()->GetPrefs()->FindPreference(
-          prefs::kSafeBrowsingReportingEnabled);
+      profile->GetPrefs()->FindPreference(prefs::kSafeBrowsingReportingEnabled);
 
   bool value;
   if (pref && pref->GetValue()->GetAsBoolean(&value) && value) {
-    // Give the details object to the service class, so it can send it.
-    BrowserThread::PostTask(
+    // Finish the malware details collection, send it over.
+    BrowserThread::PostDelayedTask(
         BrowserThread::IO, FROM_HERE,
-        NewRunnableMethod(
-            sb_service_, &SafeBrowsingService::ReportMalwareDetails,
-            malware_details_));
+        base::Bind(&MalwareDetails::FinishCollection, malware_details_.get()),
+        delay_ms);
   }
 }
 
@@ -621,9 +690,8 @@ void SafeBrowsingBlockingPage::NotifySafeBrowsingService(
     bool proceed) {
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(
-          sb_service, &SafeBrowsingService::OnBlockingPageDone,
-          unsafe_resources, proceed));
+      base::Bind(&SafeBrowsingService::OnBlockingPageDone,
+                 sb_service, unsafe_resources, proceed));
 }
 
 // static
@@ -636,13 +704,12 @@ SafeBrowsingBlockingPage::UnsafeResourceMap*
 void SafeBrowsingBlockingPage::ShowBlockingPage(
     SafeBrowsingService* sb_service,
     const SafeBrowsingService::UnsafeResource& unsafe_resource) {
-  TabContents* tab_contents = tab_util::GetTabContentsByID(
+  WebContents* web_contents = tab_util::GetWebContentsByID(
       unsafe_resource.render_process_host_id, unsafe_resource.render_view_id);
 
   InterstitialPage* interstitial =
-      InterstitialPage::GetInterstitialPage(tab_contents);
-  if (interstitial &&
-      unsafe_resource.resource_type == ResourceType::MAIN_FRAME) {
+      InterstitialPage::GetInterstitialPage(web_contents);
+  if (interstitial && !unsafe_resource.is_subresource) {
     // There is already an interstitial showing and we are about to display a
     // new one for the main frame. Just hide the current one, it is now
     // irrelevent
@@ -660,19 +727,26 @@ void SafeBrowsingBlockingPage::ShowBlockingPage(
     if (!factory_)
       factory_ = g_safe_browsing_blocking_page_factory_impl.Pointer();
     SafeBrowsingBlockingPage* blocking_page =
-        factory_->CreateSafeBrowsingPage(sb_service, tab_contents, resources);
+        factory_->CreateSafeBrowsingPage(sb_service, web_contents, resources);
     blocking_page->Show();
     return;
   }
 
   // This is an interstitial for a page's resource, let's queue it.
   UnsafeResourceMap* unsafe_resource_map = GetUnsafeResourcesMap();
-  (*unsafe_resource_map)[tab_contents].push_back(unsafe_resource);
+  (*unsafe_resource_map)[web_contents].push_back(unsafe_resource);
 }
 
 // static
-bool SafeBrowsingBlockingPage::IsMainPage(
+bool SafeBrowsingBlockingPage::IsMainPageLoadBlocked(
     const UnsafeResourceList& unsafe_resources) {
-  return unsafe_resources.size() == 1 &&
-         unsafe_resources[0].resource_type == ResourceType::MAIN_FRAME;
+  // Client-side phishing detection interstitials never block the main frame
+  // load, since they happen after the page is finished loading.
+  if (unsafe_resources[0].threat_type ==
+      SafeBrowsingService::CLIENT_SIDE_PHISHING_URL) {
+    return false;
+  }
+
+  // Otherwise, check the threat type.
+  return unsafe_resources.size() == 1 && !unsafe_resources[0].is_subresource;
 }
