@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -32,20 +32,23 @@
 #include <map>
 #include <vector>
 
-#include "chrome/browser/tab_contents/interstitial_page.h"
 #include "chrome/browser/safe_browsing/safe_browsing_service.h"
+#include "chrome/browser/tab_contents/chrome_interstitial_page.h"
 #include "googleurl/src/gurl.h"
 
-class DictionaryValue;
 class MessageLoop;
 class SafeBrowsingBlockingPageFactory;
 class MalwareDetails;
 class TabContents;
 
-class SafeBrowsingBlockingPage : public InterstitialPage {
+namespace base {
+class DictionaryValue;
+}
+
+class SafeBrowsingBlockingPage : public ChromeInterstitialPage {
  public:
   typedef std::vector<SafeBrowsingService::UnsafeResource> UnsafeResourceList;
-  typedef std::map<TabContents*, UnsafeResourceList> UnsafeResourceMap;
+  typedef std::map<content::WebContents*, UnsafeResourceList> UnsafeResourceMap;
 
   virtual ~SafeBrowsingBlockingPage();
 
@@ -64,24 +67,33 @@ class SafeBrowsingBlockingPage : public InterstitialPage {
     factory_ = factory;
   }
 
-  // InterstitialPage method:
-  virtual std::string GetHTMLContents();
-  virtual void SetReportingPreference(bool report);
-  virtual void Proceed();
-  virtual void DontProceed();
+  // ChromeInterstitialPage method:
+  virtual std::string GetHTMLContents() OVERRIDE;
+  virtual void Proceed() OVERRIDE;
+  virtual void DontProceed() OVERRIDE;
 
  protected:
   friend class SafeBrowsingBlockingPageTest;
 
-  // InterstitialPage method:
-  virtual void CommandReceived(const std::string& command);
+  // ChromeInterstitialPage method:
+  virtual void CommandReceived(const std::string& command) OVERRIDE;
+
+  void SetReportingPreference(bool report);
 
   // Don't instanciate this class directly, use ShowBlockingPage instead.
   SafeBrowsingBlockingPage(SafeBrowsingService* service,
-                           TabContents* tab_contents,
+                           content::WebContents* web_contents,
                            const UnsafeResourceList& unsafe_resources);
 
+  // After a malware interstitial where the user opted-in to the
+  // report but clicked "proceed anyway", we delay the call to
+  // MalwareDetails::FinishCollection() by this much time (in
+  // milliseconds), in order to get data from the blocked resource itself.
+  int64 malware_details_proceed_delay_ms_;
+
  private:
+  FRIEND_TEST_ALL_PREFIXES(SafeBrowsingBlockingPageTest, MalwareReports);
+
   enum BlockingPageEvent {
     SHOW,
     PROCEED,
@@ -90,13 +102,13 @@ class SafeBrowsingBlockingPage : public InterstitialPage {
 
   // Fills the passed dictionary with the strings passed to JS Template when
   // creating the HTML.
-  void PopulateMultipleThreatStringDictionary(DictionaryValue* strings);
-  void PopulateMalwareStringDictionary(DictionaryValue* strings);
-  void PopulatePhishingStringDictionary(DictionaryValue* strings);
+  void PopulateMultipleThreatStringDictionary(base::DictionaryValue* strings);
+  void PopulateMalwareStringDictionary(base::DictionaryValue* strings);
+  void PopulatePhishingStringDictionary(base::DictionaryValue* strings);
 
   // A helper method used by the Populate methods above used to populate common
   // fields.
-  void PopulateStringDictionary(DictionaryValue* strings,
+  void PopulateStringDictionary(base::DictionaryValue* strings,
                                 const string16& title,
                                 const string16& headline,
                                 const string16& description1,
@@ -115,7 +127,7 @@ class SafeBrowsingBlockingPage : public InterstitialPage {
   // pending malware details object, we look at the user's
   // preferences, and if the option to send malware details is
   // enabled, the report is scheduled to be sent on the |sb_service_|.
-  void FinishMalwareDetails();
+  void FinishMalwareDetails(int64 delay_ms);
 
   // A list of SafeBrowsingService::UnsafeResource for a tab that the user
   // should be warned about.  They are queued when displaying more than one
@@ -128,18 +140,22 @@ class SafeBrowsingBlockingPage : public InterstitialPage {
                                         const UnsafeResourceList& resources,
                                         bool proceed);
 
-  // Returns true if the passed |unsafe_resources| is for the main page.
-  static bool IsMainPage(const UnsafeResourceList& unsafe_resources);
+  // Returns true if the passed |unsafe_resources| is blocking the load of
+  // the main page.
+  static bool IsMainPageLoadBlocked(
+      const UnsafeResourceList& unsafe_resources);
 
- private:
   friend class SafeBrowsingBlockingPageFactoryImpl;
 
   // For reporting back user actions.
   SafeBrowsingService* sb_service_;
   MessageLoop* report_loop_;
 
-  // Whether the flagged resource is the main page (or a sub-resource is false).
-  bool is_main_frame_;
+  // True if the interstitial is blocking the main page because it is on one
+  // of our lists.  False if a subresource is being blocked, or in the case of
+  // client-side detection where the interstitial is shown after page load
+  // finishes.
+  bool is_main_frame_load_blocked_;
 
   // The index of a navigation entry that should be removed when DontProceed()
   // is invoked, -1 if not entry should be removed.
@@ -168,7 +184,7 @@ class SafeBrowsingBlockingPageFactory {
 
   virtual SafeBrowsingBlockingPage* CreateSafeBrowsingPage(
       SafeBrowsingService* service,
-      TabContents* tab_contents,
+      content::WebContents* web_contents,
       const SafeBrowsingBlockingPage::UnsafeResourceList& unsafe_resources) = 0;
 };
 
