@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,16 +10,20 @@
 #include <vector>
 
 #include "base/file_path.h"
-#include "base/scoped_ptr.h"
+#include "base/gtest_prod_util.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/synchronization/lock.h"
-#include "base/task.h"
 #include "chrome/browser/safe_browsing/safe_browsing_store.h"
-#include "testing/gtest/include/gtest/gtest_prod.h"
 
 namespace base {
   class Time;
 }
 
+namespace safe_browsing {
+class PrefixSet;
+}
+
 class BloomFilter;
 class GURL;
 class MessageLoop;
@@ -32,28 +36,39 @@ class SafeBrowsingDatabaseFactory {
   SafeBrowsingDatabaseFactory() { }
   virtual ~SafeBrowsingDatabaseFactory() { }
   virtual SafeBrowsingDatabase* CreateSafeBrowsingDatabase(
-      bool enable_download_protection) = 0;
+      bool enable_download_protection,
+      bool enable_client_side_whitelist,
+      bool enable_download_whitelist) = 0;
  private:
   DISALLOW_COPY_AND_ASSIGN(SafeBrowsingDatabaseFactory);
 };
 
 
-
-// Encapsulates on-disk databases that for safebrowsing. There are two
-// databases: browse database and download database. The browse database
-// contains information about phishing and malware urls. The download
-// database contains URLs for bad binaries (e.g: those containing virus)
-// and hash of these downloaded contents. These on-disk databases are shared
-// among all profiles, as it doesn't contain user-specific data. This object
-// is not thread-safe, i.e. all its methods should be used on the same thread
-// that it was created on.
+// Encapsulates on-disk databases that for safebrowsing. There are
+// four databases: browse, download, download whitelist and
+// client-side detection (csd) whitelist databases. The browse database contains
+// information about phishing and malware urls. The download database contains
+// URLs for bad binaries (e.g: those containing virus) and hash of
+// these downloaded contents. The download whitelist contains whitelisted
+// download hosting sites as well as whitelisted binary signing certificates
+// etc.  The csd whitelist database contains URLs that will never be considered
+// as phishing by the client-side phishing detection. These on-disk databases
+// are shared among all profiles, as it doesn't contain user-specific data. This
+// object is not thread-safe, i.e. all its methods should be used on the same
+// thread that it was created on.
 class SafeBrowsingDatabase {
  public:
   // Factory method for obtaining a SafeBrowsingDatabase implementation.
   // It is not thread safe.
   // |enable_download_protection| is used to control the download database
   // feature.
-  static SafeBrowsingDatabase* Create(bool enable_download_protection);
+  // |enable_client_side_whitelist| is used to control the csd whitelist
+  // database feature.
+  // |enable_download_whitelist| is used to control the download whitelist
+  // database feature.
+  static SafeBrowsingDatabase* Create(bool enable_download_protection,
+                                      bool enable_client_side_whitelist,
+                                      bool enable_download_whitelist);
 
   // Makes the passed |factory| the factory used to instantiate
   // a SafeBrowsingDatabase. This is used for tests.
@@ -80,12 +95,32 @@ class SafeBrowsingDatabase {
                                  std::vector<SBFullHashResult>* full_hits,
                                  base::Time last_update) = 0;
 
-  // Returns false if |url| is not in Download database. If it returns true,
-  // |prefix_hits| should contain the prefix for |url|.
-  // This function could ONLY be accessed from creation thread.
-  virtual bool ContainsDownloadUrl(const GURL& url,
+  // Returns false if none of |urls| are in Download database. If it returns
+  // true, |prefix_hits| should contain the prefixes for the URLs that were in
+  // the database.  This function could ONLY be accessed from creation thread.
+  virtual bool ContainsDownloadUrl(const std::vector<GURL>& urls,
                                    std::vector<SBPrefix>* prefix_hits) = 0;
 
+  // Returns false if |prefix| is not in Download database.
+  // This function could ONLY be accessed from creation thread.
+  virtual bool ContainsDownloadHashPrefix(const SBPrefix& prefix) = 0;
+
+  // Returns false if |url| is not on the client-side phishing detection
+  // whitelist.  Otherwise, this function returns true.  Note: the whitelist
+  // only contains full-length hashes so we don't return any prefix hit.
+  // This function should only be called from the IO thread.
+  virtual bool ContainsCsdWhitelistedUrl(const GURL& url) = 0;
+
+  // The download whitelist is used for two purposes: a white-domain list of
+  // sites that are considered to host only harmless binaries as well as a
+  // whitelist of arbitrary strings such as hashed certificate authorities that
+  // are considered to be trusted.  The two methods below let you lookup
+  // the whitelist either for a URL or an arbitrary string.  These methods will
+  // return false if no match is found and true otherwise.
+  // This function could ONLY be accessed from the IO thread.
+  virtual bool ContainsDownloadWhitelistedUrl(const GURL& url) = 0;
+  virtual bool ContainsDownloadWhitelistedString(const std::string& str) = 0;
+
   // A database transaction should look like:
   //
   // std::vector<SBListChunkRanges> lists;
@@ -130,6 +165,14 @@ class SafeBrowsingDatabase {
   // Filename for download URL and download binary hash database.
   static FilePath DownloadDBFilename(const FilePath& db_base_filename);
 
+  // Filename for client-side phishing detection whitelist databsae.
+  static FilePath CsdWhitelistDBFilename(
+      const FilePath& csd_whitelist_base_filename);
+
+  // Filename for download whitelist databsae.
+  static FilePath DownloadWhitelistDBFilename(
+      const FilePath& download_whitelist_base_filename);
+
   // Enumerate failures for histogramming purposes.  DO NOT CHANGE THE
   // ORDERING OF THESE VALUES.
   enum FailureType {
@@ -145,7 +188,8 @@ class SafeBrowsingDatabase {
     FAILURE_DATABASE_STORE_DELETE,
     FAILURE_DOWNLOAD_DATABASE_UPDATE_BEGIN,
     FAILURE_DOWNLOAD_DATABASE_UPDATE_FINISH,
-
+    FAILURE_WHITELIST_DATABASE_UPDATE_BEGIN,
+    FAILURE_WHITELIST_DATABASE_UPDATE_FINISH,
     // Memory space for histograms is determined by the max.  ALWAYS
     // ADD NEW VALUES BEFORE THIS ONE.
     FAILURE_DATABASE_MAX
@@ -162,12 +206,15 @@ class SafeBrowsingDatabase {
 
 class SafeBrowsingDatabaseNew : public SafeBrowsingDatabase {
  public:
-  // Create a database with a browse store and download store. Takes ownership
-  // of browse_store and download_store. When |download_store| is NULL,
-  // the database will ignore any operations related download (url hashes and
-  // binary hashes).
+  // Create a database with a browse, download, download whitelist and
+  // csd whitelist store objects. Takes ownership of all the store objects.
+  // When |download_store| is NULL, the database will ignore any operations
+  // related download (url hashes and binary hashes).  The same is true for
+  // the |csd_whitelist_store| and |download_whitelist_store|.
   SafeBrowsingDatabaseNew(SafeBrowsingStore* browse_store,
-                          SafeBrowsingStore* download_store);
+                          SafeBrowsingStore* download_store,
+                          SafeBrowsingStore* csd_whitelist_store,
+                          SafeBrowsingStore* download_whitelist_store);
 
   // Create a database with a browse store. This is a legacy interface that
   // useds Sqlite.
@@ -176,29 +223,46 @@ class SafeBrowsingDatabaseNew : public SafeBrowsingDatabase {
   virtual ~SafeBrowsingDatabaseNew();
 
   // Implement SafeBrowsingDatabase interface.
-  virtual void Init(const FilePath& filename);
-  virtual bool ResetDatabase();
+  virtual void Init(const FilePath& filename) OVERRIDE;
+  virtual bool ResetDatabase() OVERRIDE;
   virtual bool ContainsBrowseUrl(const GURL& url,
                                  std::string* matching_list,
                                  std::vector<SBPrefix>* prefix_hits,
                                  std::vector<SBFullHashResult>* full_hits,
-                                 base::Time last_update);
-  virtual bool ContainsDownloadUrl(const GURL& url,
-                                   std::vector<SBPrefix>* prefix_hits);
-
-  virtual bool UpdateStarted(std::vector<SBListChunkRanges>* lists);
+                                 base::Time last_update) OVERRIDE;
+  virtual bool ContainsDownloadUrl(const std::vector<GURL>& urls,
+                                   std::vector<SBPrefix>* prefix_hits) OVERRIDE;
+  virtual bool ContainsDownloadHashPrefix(const SBPrefix& prefix) OVERRIDE;
+  virtual bool ContainsCsdWhitelistedUrl(const GURL& url) OVERRIDE;
+  virtual bool ContainsDownloadWhitelistedUrl(const GURL& url) OVERRIDE;
+  virtual bool ContainsDownloadWhitelistedString(
+      const std::string& str) OVERRIDE;
+  virtual bool UpdateStarted(std::vector<SBListChunkRanges>* lists) OVERRIDE;
   virtual void InsertChunks(const std::string& list_name,
-                            const SBChunkList& chunks);
-  virtual void DeleteChunks(const std::vector<SBChunkDelete>& chunk_deletes);
-  virtual void UpdateFinished(bool update_succeeded);
-  virtual void CacheHashResults(const std::vector<SBPrefix>& prefixes,
-                                const std::vector<SBFullHashResult>& full_hits);
+                            const SBChunkList& chunks) OVERRIDE;
+  virtual void DeleteChunks(
+      const std::vector<SBChunkDelete>& chunk_deletes) OVERRIDE;
+  virtual void UpdateFinished(bool update_succeeded) OVERRIDE;
+  virtual void CacheHashResults(
+      const std::vector<SBPrefix>& prefixes,
+      const std::vector<SBFullHashResult>& full_hits) OVERRIDE;
 
  private:
   friend class SafeBrowsingDatabaseTest;
-  FRIEND_TEST(SafeBrowsingDatabaseTest, HashCaching);
+  FRIEND_TEST_ALL_PREFIXES(SafeBrowsingDatabaseTest, HashCaching);
 
-  // Return the browse_store_ or download_store_ based on list_id.
+  // A SafeBrowsing whitelist contains a list of whitelisted full-hashes (stored
+  // in a sorted vector) as well as a boolean flag indicating whether all
+  // lookups in the whitelist should be considered matches for safety.
+  typedef std::pair<std::vector<SBFullHash>, bool> SBWhitelist;
+
+  // Returns true if the whitelist is disabled or if any of the given hashes
+  // matches the whitelist.
+  bool ContainsWhitelistedHashes(const SBWhitelist& whitelist,
+                                 const std::vector<SBFullHash>& hashes);
+
+  // Return the browse_store_, download_store_, download_whitelist_store or
+  // csd_whitelist_store_ based on list_id.
   SafeBrowsingStore* GetStore(int list_id);
 
     // Deletes the files on disk.
@@ -210,6 +274,16 @@ class SafeBrowsingDatabaseNew : public SafeBrowsingDatabase {
   // Writes the current bloom filter to disk.
   void WriteBloomFilter();
 
+  // Loads the given full-length hashes to the given whitelist.  If the number
+  // of hashes is too large or if the kill switch URL is on the whitelist
+  // we will whitelist everything.
+  void LoadWhitelist(const std::vector<SBAddFullHash>& full_hashes,
+                     SBWhitelist* whitelist);
+
+  // Call this method if an error occured with the given whitelist.  This will
+  // result in all lookups to the whitelist to return true.
+  void WhitelistEverything(SBWhitelist* whitelist);
+
   // Helpers for handling database corruption.
   // |OnHandleCorruptDatabase()| runs |ResetDatabase()| and sets
   // |corruption_detected_|, |HandleCorruptDatabase()| posts
@@ -228,6 +302,18 @@ class SafeBrowsingDatabaseNew : public SafeBrowsingDatabase {
 
   void UpdateDownloadStore();
   void UpdateBrowseStore();
+  void UpdateWhitelistStore(const FilePath& store_filename,
+                            SafeBrowsingStore* store,
+                            SBWhitelist* whitelist);
+
+  // Helper function to compare addprefixes in download_store_ with |prefixes|.
+  // The |list_bit| indicates which list (download url or download hash)
+  // to compare.
+  // Returns true if there is a match, |*prefix_hits| will contain the actual
+  // matching prefixes.
+  bool MatchDownloadAddPrefixes(int list_bit,
+                                const std::vector<SBPrefix>& prefixes,
+                                std::vector<SBPrefix>* prefix_hits);
 
   // Used to verify that various calls are made from the thread the
   // object was created on.
@@ -235,7 +321,8 @@ class SafeBrowsingDatabaseNew : public SafeBrowsingDatabase {
 
   // Lock for protecting access to variables that may be used on the
   // IO thread.  This includes |browse_bloom_filter_|, |full_browse_hashes_|,
-  // |pending_browse_hashes_|, and |prefix_miss_cache_|.
+  // |pending_browse_hashes_|, |prefix_miss_cache_|, |csd_whitelist_|, and
+  // |csd_whitelist_all_urls_|.
   base::Lock lookup_lock_;
 
   // Underlying persistent store for chunk data.
@@ -247,6 +334,19 @@ class SafeBrowsingDatabaseNew : public SafeBrowsingDatabase {
   FilePath download_filename_;
   scoped_ptr<SafeBrowsingStore> download_store_;
 
+  // For the client-side phishing detection whitelist chunks and full-length
+  // hashes.  This list only contains 256 bit hashes.
+  FilePath csd_whitelist_filename_;
+  scoped_ptr<SafeBrowsingStore> csd_whitelist_store_;
+
+  // For the download whitelist chunks and full-length hashes.  This list only
+  // contains 256 bit hashes.
+  FilePath download_whitelist_filename_;
+  scoped_ptr<SafeBrowsingStore> download_whitelist_store_;
+
+  SBWhitelist csd_whitelist_;
+  SBWhitelist download_whitelist_;
+
   // Bloom filter generated from the add-prefixes in |browse_store_|.
   // Only browse_store_ requires the BloomFilter for fast query.
   FilePath bloom_filter_filename_;
@@ -266,12 +366,19 @@ class SafeBrowsingDatabaseNew : public SafeBrowsingDatabase {
   std::set<SBPrefix> prefix_miss_cache_;
 
   // Used to schedule resetting the database because of corruption.
-  ScopedRunnableMethodFactory<SafeBrowsingDatabaseNew> reset_factory_;
+  base::WeakPtrFactory<SafeBrowsingDatabaseNew> reset_factory_;
 
   // Set if corruption is detected during the course of an update.
   // Causes the update functions to fail with no side effects, until
   // the next call to |UpdateStarted()|.
   bool corruption_detected_;
+
+  // Set to true if any chunks are added or deleted during an update.
+  // Used to optimize away database update.
+  bool change_detected_;
+
+  // Used to check if a prefix was in the database.
+  scoped_ptr<safe_browsing::PrefixSet> prefix_set_;
 };
 
 #endif  // CHROME_BROWSER_SAFE_BROWSING_SAFE_BROWSING_DATABASE_H_
