@@ -1,38 +1,39 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
 // Unit tests for the SafeBrowsing storage system.
 
-#include "app/sql/connection.h"
-#include "app/sql/statement.h"
 #include "base/file_util.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
 #include "base/scoped_temp_dir.h"
-#include "base/sha2.h"
 #include "base/time.h"
 #include "chrome/browser/safe_browsing/safe_browsing_database.h"
 #include "chrome/browser/safe_browsing/safe_browsing_store_file.h"
-#include "chrome/browser/safe_browsing/safe_browsing_store_sqlite.h"
 #include "chrome/browser/safe_browsing/safe_browsing_store_unittest_helper.h"
+#include "content/test/test_browser_thread.h"
+#include "crypto/sha2.h"
 #include "googleurl/src/gurl.h"
+#include "sql/connection.h"
+#include "sql/statement.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
 
 using base::Time;
+using content::BrowserThread;
 
 namespace {
 
 SBPrefix Sha256Prefix(const std::string& str) {
   SBPrefix prefix;
-  base::SHA256HashString(str, &prefix, sizeof(prefix));
+  crypto::SHA256HashString(str, &prefix, sizeof(prefix));
   return prefix;
 }
 
 SBFullHash Sha256Hash(const std::string& str) {
   SBFullHash hash;
-  base::SHA256HashString(str, &hash, sizeof(hash));
+  crypto::SHA256HashString(str, &hash, sizeof(hash));
   return hash;
 }
 
@@ -63,6 +64,21 @@ void InsertAddChunkHostPrefixUrl(SBChunk* chunk,
                                 Sha256Prefix(url));
 }
 
+// Same as InsertAddChunkHostPrefixUrl, but with full hashes.
+void InsertAddChunkHostFullHashes(SBChunk* chunk,
+                                  int chunk_number,
+                                  const std::string& host_name,
+                                  const std::string& url) {
+  chunk->chunk_number = chunk_number;
+  chunk->is_add = true;
+  SBChunkHost host;
+  host.host = Sha256Prefix(host_name);
+  host.entry = SBEntry::Create(SBEntry::ADD_FULL_HASH, 1);
+  host.entry->set_chunk_id(chunk->chunk_number);
+  host.entry->SetFullHashAt(0, Sha256Hash(url));
+  chunk->hosts.push_back(host);
+}
+
 // Same as InsertAddChunkHostPrefixUrl, but with two urls for prefixes.
 void InsertAddChunkHost2PrefixUrls(SBChunk* chunk,
                                    int chunk_number,
@@ -148,26 +164,6 @@ void InsertSubChunkHost2PrefixUrls(SBChunk* chunk,
   chunk->hosts.push_back(host);
 }
 
-// Same as InsertSubChunkHost2PrefixUrls, but with full hashes.
-void InsertSubChunkHost2FullHashes(SBChunk* chunk,
-                                   int chunk_number,
-                                   int chunk_id_to_sub,
-                                   const std::string& host_name,
-                                   const std::string& url1,
-                                   const std::string& url2) {
-  chunk->chunk_number = chunk_number;
-  chunk->is_add = false;
-  SBChunkHost host;
-  host.host = Sha256Prefix(host_name);
-  host.entry = SBEntry::Create(SBEntry::SUB_FULL_HASH, 2);
-  host.entry->set_chunk_id(chunk->chunk_number);
-  host.entry->SetFullHashAt(0, Sha256Hash(url1));
-  host.entry->SetChunkIdAtPrefix(0, chunk_id_to_sub);
-  host.entry->SetFullHashAt(1, Sha256Hash(url2));
-  host.entry->SetChunkIdAtPrefix(1, chunk_id_to_sub);
-  chunk->hosts.push_back(host);
-}
-
 // Same as InsertSubChunkHost2PrefixUrls, but with full hashes.
 void InsertSubChunkHostFullHash(SBChunk* chunk,
                                 int chunk_number,
@@ -218,57 +214,6 @@ class ScopedLogMessageIgnorer {
   }
 };
 
-// Helper function which corrupts the root page of the indicated
-// table.  After this the table can be opened successfully, and
-// queries to other tables work, and possibly queries to this table
-// which only hit an index may work, but queries which hit the table
-// itself should not.  Returns |true| on success.
-bool CorruptSqliteTable(const FilePath& filename,
-                        const std::string& table_name) {
-  size_t root_page;  // Root page of the table.
-  size_t page_size;  // Page size of the database.
-
-  sql::Connection db;
-  if (!db.Open(filename))
-    return false;
-
-  sql::Statement stmt(db.GetUniqueStatement("PRAGMA page_size"));
-  if (!stmt.Step())
-    return false;
-  page_size = stmt.ColumnInt(0);
-
-  stmt.Assign(db.GetUniqueStatement(
-      "SELECT rootpage FROM sqlite_master WHERE name = ?"));
-  stmt.BindString(0, "sub_prefix");
-  if (!stmt.Step())
-    return false;
-  root_page = stmt.ColumnInt(0);
-
-  // The page numbers are 1-based.
-  const size_t root_page_offset = (root_page - 1) * page_size;
-
-  // Corrupt the file by overwriting the table's root page.
-  FILE* fp = file_util::OpenFile(filename, "r+");
-  if (!fp)
-    return false;
-
-  file_util::ScopedFILE file_closer(fp);
-  if (fseek(fp, root_page_offset, SEEK_SET) == -1)
-    return false;
-
-  for (size_t i = 0; i < page_size; ++i) {
-    fputc('!', fp);  // Character experimentally verified.
-  }
-
-  // Close the file manually because if there is an error in the
-  // close, it's likely because the data could not be flushed to the
-  // file.
-  if (!file_util::CloseFile(file_closer.release()))
-    return false;
-
-  return true;
-}
-
 }  // namespace
 
 class SafeBrowsingDatabaseTest : public PlatformTest {
@@ -421,7 +366,12 @@ TEST_F(SafeBrowsingDatabaseTest, ListNameForBrowseAndDownload) {
   MessageLoop loop(MessageLoop::TYPE_DEFAULT);
   SafeBrowsingStoreFile* browse_store = new SafeBrowsingStoreFile();
   SafeBrowsingStoreFile* download_store = new SafeBrowsingStoreFile();
-  database_.reset(new SafeBrowsingDatabaseNew(browse_store, download_store));
+  SafeBrowsingStoreFile* csd_whitelist_store = new SafeBrowsingStoreFile();
+  SafeBrowsingStoreFile* download_whitelist_store = new SafeBrowsingStoreFile();
+  database_.reset(new SafeBrowsingDatabaseNew(browse_store,
+                                              download_store,
+                                              csd_whitelist_store,
+                                              download_whitelist_store));
   database_->Init(database_filename_);
 
   SBChunkList chunks;
@@ -456,10 +406,25 @@ TEST_F(SafeBrowsingDatabaseTest, ListNameForBrowseAndDownload) {
   chunks.push_back(chunk);
   database_->InsertChunks(safe_browsing_util::kBinHashList, chunks);
 
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 5, "www.forwhitelist.com/",
+                               "www.forwhitelist.com/a.html");
+  chunks.clear();
+  chunks.push_back(chunk);
+  database_->InsertChunks(safe_browsing_util::kCsdWhiteList, chunks);
+
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 6, "www.download.com/",
+                               "www.download.com/");
+
+  chunks.clear();
+  chunks.push_back(chunk);
+  database_->InsertChunks(safe_browsing_util::kDownloadWhiteList, chunks);
+
   database_->UpdateFinished(true);
 
   GetListsInfo(&lists);
-  EXPECT_EQ(4U, lists.size());
+  ASSERT_EQ(5U, lists.size());
   EXPECT_TRUE(lists[0].name == safe_browsing_util::kMalwareList);
   EXPECT_EQ(lists[0].adds, "1");
   EXPECT_TRUE(lists[0].subs.empty());
@@ -469,9 +434,13 @@ TEST_F(SafeBrowsingDatabaseTest, ListNameForBrowseAndDownload) {
   EXPECT_TRUE(lists[2].name == safe_browsing_util::kBinUrlList);
   EXPECT_EQ(lists[2].adds, "3");
   EXPECT_TRUE(lists[2].subs.empty());
-  EXPECT_TRUE(lists[3].name == safe_browsing_util::kBinHashList);
-  EXPECT_EQ(lists[3].adds, "4");
+  // kBinHashList is ignored.  (http://crbug.com/108130)
+  EXPECT_TRUE(lists[3].name == safe_browsing_util::kCsdWhiteList);
+  EXPECT_EQ(lists[3].adds, "5");
   EXPECT_TRUE(lists[3].subs.empty());
+  EXPECT_TRUE(lists[4].name == safe_browsing_util::kDownloadWhiteList);
+  EXPECT_EQ(lists[4].adds, "6");
+  EXPECT_TRUE(lists[4].subs.empty());
   database_.reset();
 }
 
@@ -1084,84 +1053,6 @@ TEST_F(SafeBrowsingDatabaseTest, HashCaching) {
       Time::Now()));
 }
 
-// Test that corrupt databases are appropriately handled, even if the
-// corruption is detected in the midst of the update.
-// TODO(shess): Disabled until ScopedLogMessageIgnorer resolved.
-// http://crbug.com/56448
-TEST_F(SafeBrowsingDatabaseTest, DISABLED_SqliteCorruptionHandling) {
-  // Re-create the database in a captive message loop so that we can
-  // influence task-posting.  Database specifically needs to the
-  // SQLite-backed.
-  database_.reset();
-  MessageLoop loop(MessageLoop::TYPE_DEFAULT);
-  SafeBrowsingStoreSqlite* store = new SafeBrowsingStoreSqlite();
-  database_.reset(new SafeBrowsingDatabaseNew(store, NULL));
-  database_->Init(database_filename_);
-
-  // This will cause an empty database to be created.
-  std::vector<SBListChunkRanges> lists;
-  EXPECT_TRUE(database_->UpdateStarted(&lists));
-  database_->UpdateFinished(true);
-
-  // Create a sub chunk to insert.
-  SBChunkList chunks;
-  SBChunk chunk;
-  SBChunkHost host;
-  host.host = Sha256Prefix("www.subbed.com/");
-  host.entry = SBEntry::Create(SBEntry::SUB_PREFIX, 1);
-  host.entry->set_chunk_id(7);
-  host.entry->SetChunkIdAtPrefix(0, 19);
-  host.entry->SetPrefixAt(0, Sha256Prefix("www.subbed.com/notevil1.html"));
-  chunk.chunk_number = 7;
-  chunk.is_add = false;
-  chunk.hosts.clear();
-  chunk.hosts.push_back(host);
-  chunks.clear();
-  chunks.push_back(chunk);
-
-  // Corrupt the |sub_prefix| table.
-  ASSERT_TRUE(CorruptSqliteTable(database_filename_, "sub_prefix"));
-
-  {
-    // The following code will cause DCHECKs, so suppress the crashes.
-    ScopedLogMessageIgnorer ignorer;
-
-    // Start an update.  The insert will fail due to corruption.
-    EXPECT_TRUE(database_->UpdateStarted(&lists));
-    VLOG(1) << "Expect failed check on: sqlite error 11";
-    database_->InsertChunks(safe_browsing_util::kMalwareList, chunks);
-
-    // Database file still exists until the corruption handler has run.
-    EXPECT_TRUE(file_util::PathExists(database_filename_));
-
-    // Flush through the corruption-handler task.
-    VLOG(1) << "Expect failed check on: SafeBrowsing database reset";
-    MessageLoop::current()->RunAllPending();
-  }
-
-  // Database file should not exist.
-  EXPECT_FALSE(file_util::PathExists(database_filename_));
-
-  // Finish the transaction.  This should short-circuit, so no
-  // DCHECKs.
-  database_->InsertChunks(safe_browsing_util::kMalwareList, chunks);
-  database_->UpdateFinished(true);
-
-  // Flush through any posted tasks.
-  MessageLoop::current()->RunAllPending();
-
-  // Database file should still not exist.
-  EXPECT_FALSE(file_util::PathExists(database_filename_));
-
-  // Run the update again successfully.
-  EXPECT_TRUE(database_->UpdateStarted(&lists));
-  database_->InsertChunks(safe_browsing_util::kMalwareList, chunks);
-  database_->UpdateFinished(true);
-  EXPECT_TRUE(file_util::PathExists(database_filename_));
-
-  database_.reset();
-}
-
 // Test that corrupt databases are appropriately handled, even if the
 // corruption is detected in the midst of the update.
 // TODO(shess): Disabled until ScopedLogMessageIgnorer resolved.
@@ -1173,7 +1064,7 @@ TEST_F(SafeBrowsingDatabaseTest, DISABLED_FileCorruptionHandling) {
   database_.reset();
   MessageLoop loop(MessageLoop::TYPE_DEFAULT);
   SafeBrowsingStoreFile* store = new SafeBrowsingStoreFile();
-  database_.reset(new SafeBrowsingDatabaseNew(store, NULL));
+  database_.reset(new SafeBrowsingDatabaseNew(store, NULL, NULL, NULL));
   database_->Init(database_filename_);
 
   // This will cause an empty database to be created.
@@ -1242,11 +1133,15 @@ TEST_F(SafeBrowsingDatabaseTest, ContainsDownloadUrl) {
   MessageLoop loop(MessageLoop::TYPE_DEFAULT);
   SafeBrowsingStoreFile* browse_store = new SafeBrowsingStoreFile();
   SafeBrowsingStoreFile* download_store = new SafeBrowsingStoreFile();
-  database_.reset(new SafeBrowsingDatabaseNew(browse_store, download_store));
+  SafeBrowsingStoreFile* csd_whitelist_store = new SafeBrowsingStoreFile();
+  database_.reset(new SafeBrowsingDatabaseNew(browse_store,
+                                              download_store,
+                                              csd_whitelist_store,
+                                              NULL));
   database_->Init(database_filename_);
 
   const char kEvil1Host[] = "www.evil1.com/";
-  const char kEvil1Url1[] = "www.evil1.com/download1.html";
+  const char kEvil1Url1[] = "www.evil1.com/download1/";
   const char kEvil1Url2[] = "www.evil1.com/download2.html";
 
   SBChunkList chunks;
@@ -1260,33 +1155,273 @@ TEST_F(SafeBrowsingDatabaseTest, ContainsDownloadUrl) {
   database_->InsertChunks(safe_browsing_util::kBinUrlList, chunks);
   database_->UpdateFinished(true);
 
-  const Time now = Time::Now();
   std::vector<SBPrefix> prefix_hits;
-  std::string matching_list;
+  std::vector<GURL> urls(1);
 
-  EXPECT_TRUE(database_->ContainsDownloadUrl(
-      GURL(std::string("http://") + kEvil1Url1), &prefix_hits));
+  urls[0] = GURL(std::string("http://") + kEvil1Url1);
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
   EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url1));
-  EXPECT_EQ(prefix_hits.size(), 1U);
 
-  EXPECT_TRUE(database_->ContainsDownloadUrl(
-      GURL(std::string("http://") + kEvil1Url2), &prefix_hits));
+  urls[0] = GURL(std::string("http://") + kEvil1Url2);
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
   EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url2));
-  EXPECT_EQ(prefix_hits.size(), 1U);
 
-  EXPECT_TRUE(database_->ContainsDownloadUrl(
-      GURL(std::string("https://") + kEvil1Url2), &prefix_hits));
+  urls[0] = GURL(std::string("https://") + kEvil1Url2);
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
   EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url2));
-  EXPECT_EQ(prefix_hits.size(), 1U);
 
-  EXPECT_TRUE(database_->ContainsDownloadUrl(
-      GURL(std::string("ftp://") + kEvil1Url2), &prefix_hits));
+  urls[0] = GURL(std::string("ftp://") + kEvil1Url2);
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
   EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url2));
-  EXPECT_EQ(prefix_hits.size(), 1U);
 
-  EXPECT_FALSE(database_->ContainsDownloadUrl(GURL("http://www.randomevil.com"),
-                                              &prefix_hits));
-  EXPECT_EQ(prefix_hits.size(), 0U);
+  urls[0] = GURL("http://www.randomevil.com");
+  EXPECT_FALSE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+
+  // Should match with query args stripped.
+  urls[0] = GURL(std::string("http://") + kEvil1Url2 + "?blah");
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
+  EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url2));
+
+  // Should match with extra path stuff and query args stripped.
+  urls[0] = GURL(std::string("http://") + kEvil1Url1 + "foo/bar?blah");
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
+  EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url1));
+
+  // First hit in redirect chain is malware.
+  urls.clear();
+  urls.push_back(GURL(std::string("http://") + kEvil1Url1));
+  urls.push_back(GURL("http://www.randomevil.com"));
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
+  EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url1));
+
+  // Middle hit in redirect chain is malware.
+  urls.clear();
+  urls.push_back(GURL("http://www.randomevil.com"));
+  urls.push_back(GURL(std::string("http://") + kEvil1Url1));
+  urls.push_back(GURL("http://www.randomevil2.com"));
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
+  EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url1));
+
+  // Final hit in redirect chain is malware.
+  urls.clear();
+  urls.push_back(GURL("http://www.randomevil.com"));
+  urls.push_back(GURL(std::string("http://") + kEvil1Url1));
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 1U);
+  EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url1));
+
+  // Multiple hits in redirect chain are in malware list.
+  urls.clear();
+  urls.push_back(GURL(std::string("http://") + kEvil1Url1));
+  urls.push_back(GURL(std::string("https://") + kEvil1Url2));
+  EXPECT_TRUE(database_->ContainsDownloadUrl(urls, &prefix_hits));
+  ASSERT_EQ(prefix_hits.size(), 2U);
+  EXPECT_EQ(prefix_hits[0], Sha256Prefix(kEvil1Url1));
+  EXPECT_EQ(prefix_hits[1], Sha256Prefix(kEvil1Url2));
+  database_.reset();
+}
+
+// Checks that the whitelists are handled properly.
+TEST_F(SafeBrowsingDatabaseTest, Whitelists) {
+  database_.reset();
+  MessageLoop loop(MessageLoop::TYPE_DEFAULT);
+  // We expect all calls to ContainsCsdWhitelistedUrl in particular to be made
+  // from the IO thread.  In general the whitelist lookups are thread-safe.
+  content::TestBrowserThread io_thread(BrowserThread::IO, &loop);
+
+  // If the whitelist is disabled everything should match the whitelist.
+  database_.reset(new SafeBrowsingDatabaseNew(new SafeBrowsingStoreFile(),
+                                              NULL, NULL, NULL));
+  database_->Init(database_filename_);
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://www.phishing.com/"))));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://www.phishing.com/"))));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedString("asdf"));
+
+  SafeBrowsingStoreFile* browse_store = new SafeBrowsingStoreFile();
+  SafeBrowsingStoreFile* csd_whitelist_store = new SafeBrowsingStoreFile();
+  SafeBrowsingStoreFile* download_whitelist_store = new SafeBrowsingStoreFile();
+  database_.reset(new SafeBrowsingDatabaseNew(browse_store, NULL,
+                                              csd_whitelist_store,
+                                              download_whitelist_store));
+  database_->Init(database_filename_);
+
+  const char kGood1Host[] = "www.good1.com/";
+  const char kGood1Url1[] = "www.good1.com/a/b.html";
+  const char kGood1Url2[] = "www.good1.com/b/";
+
+  const char kGood2Host[] = "www.good2.com/";
+  const char kGood2Url1[] = "www.good2.com/c";  // Should match '/c/bla'.
+
+  // good3.com/a/b/c/d/e/f/g/ should match because it's a whitelist.
+  const char kGood3Host[] = "good3.com/";
+  const char kGood3Url1[] = "good3.com/";
+
+  const char kGoodString[] = "good_string";
+
+  SBChunkList download_chunks, csd_chunks;
+  SBChunk chunk;
+  // Add two simple chunks to the csd whitelist.
+  InsertAddChunkHost2FullHashes(&chunk, 1, kGood1Host,
+                                kGood1Url1, kGood1Url2);
+  csd_chunks.push_back(chunk);
+
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 2, kGood2Host, kGood2Url1);
+  csd_chunks.push_back(chunk);
+
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 2, kGood2Host, kGood2Url1);
+  download_chunks.push_back(chunk);
+
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 3, kGoodString, kGoodString);
+  download_chunks.push_back(chunk);
+
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 4, kGood3Host, kGood3Url1);
+  download_chunks.push_back(chunk);
+
+  std::vector<SBListChunkRanges> lists;
+  EXPECT_TRUE(database_->UpdateStarted(&lists));
+  database_->InsertChunks(safe_browsing_util::kCsdWhiteList,
+                          csd_chunks);
+  database_->InsertChunks(safe_browsing_util::kDownloadWhiteList,
+                          download_chunks);
+  database_->UpdateFinished(true);
+
+  EXPECT_FALSE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood1Host)));
+
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood1Url1)));
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood1Url1 + "?a=b")));
+
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood1Url2)));
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood1Url2 + "/c.html")));
+
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("https://") + kGood1Url2 + "/c.html")));
+
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood2Url1 + "/c")));
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood2Url1 + "/c?bla")));
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://") + kGood2Url1 + "/c/bla")));
+
+  EXPECT_FALSE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://www.google.com/"))));
+
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://") + kGood2Url1 + "/c")));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://") + kGood2Url1 + "/c?bla")));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://") + kGood2Url1 + "/c/bla")));
+
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://good3.com/a/b/c/d/e/f/g/"))));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://a.b.good3.com/"))));
+
+  EXPECT_FALSE(database_->ContainsDownloadWhitelistedString("asdf"));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedString(kGoodString));
+
+  EXPECT_FALSE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://www.google.com/"))));
+
+  // Test that the kill-switch works as intended.
+  csd_chunks.clear();
+  download_chunks.clear();
+  lists.clear();
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 5, "sb-ssl.google.com/",
+                               "sb-ssl.google.com/safebrowsing/csd/killswitch");
+  csd_chunks.push_back(chunk);
+
+  chunk.hosts.clear();
+  InsertAddChunkHostFullHashes(&chunk, 5, "sb-ssl.google.com/",
+                               "sb-ssl.google.com/safebrowsing/csd/killswitch");
+  download_chunks.push_back(chunk);
+
+  EXPECT_TRUE(database_->UpdateStarted(&lists));
+  database_->InsertChunks(safe_browsing_util::kCsdWhiteList, csd_chunks);
+  database_->InsertChunks(safe_browsing_util::kDownloadWhiteList,
+                          download_chunks);
+  database_->UpdateFinished(true);
+
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("https://") + kGood1Url2 + "/c.html")));
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://www.google.com/"))));
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://www.phishing_url.com/"))));
+
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("https://") + kGood1Url2 + "/c.html")));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://www.google.com/"))));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://www.phishing_url.com/"))));
+
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedString("asdf"));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedString(kGoodString));
+
+  // Remove the kill-switch and verify that we can recover.
+  csd_chunks.clear();
+  download_chunks.clear();
+  lists.clear();
+  SBChunk sub_chunk;
+  InsertSubChunkHostFullHash(&sub_chunk, 1, 5,
+                             "sb-ssl.google.com/",
+                             "sb-ssl.google.com/safebrowsing/csd/killswitch");
+  csd_chunks.push_back(sub_chunk);
+
+  sub_chunk.hosts.clear();
+  InsertSubChunkHostFullHash(&sub_chunk, 1, 5,
+                             "sb-ssl.google.com/",
+                             "sb-ssl.google.com/safebrowsing/csd/killswitch");
+  download_chunks.push_back(sub_chunk);
+
+  EXPECT_TRUE(database_->UpdateStarted(&lists));
+  database_->InsertChunks(safe_browsing_util::kCsdWhiteList, csd_chunks);
+  database_->InsertChunks(safe_browsing_util::kDownloadWhiteList,
+                          download_chunks);
+  database_->UpdateFinished(true);
+
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("https://") + kGood1Url2 + "/c.html")));
+  EXPECT_TRUE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("https://") + kGood2Url1 + "/c/bla")));
+  EXPECT_FALSE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://www.google.com/"))));
+  EXPECT_FALSE(database_->ContainsCsdWhitelistedUrl(
+      GURL(std::string("http://www.phishing_url.com/"))));
+
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("https://") + kGood2Url1 + "/c/bla")));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("https://good3.com/"))));
+  EXPECT_TRUE(database_->ContainsDownloadWhitelistedString(kGoodString));
+  EXPECT_FALSE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://www.google.com/"))));
+  EXPECT_FALSE(database_->ContainsDownloadWhitelistedUrl(
+      GURL(std::string("http://www.phishing_url.com/"))));
+  EXPECT_FALSE(database_->ContainsDownloadWhitelistedString("asdf"));
+
   database_.reset();
 }
 
@@ -1389,35 +1524,64 @@ TEST_F(SafeBrowsingDatabaseTest, SameHostEntriesOkay) {
       &listname, &prefixes, &full_hashes, now));
 }
 
-TEST_F(SafeBrowsingDatabaseTest, BinHashEntries) {
-  database_.reset();
-  MessageLoop loop(MessageLoop::TYPE_DEFAULT);
-  SafeBrowsingStoreFile* browse_store = new SafeBrowsingStoreFile();
-  SafeBrowsingStoreFile* download_store = new SafeBrowsingStoreFile();
-  database_.reset(new SafeBrowsingDatabaseNew(browse_store, download_store));
-  database_->Init(database_filename_);
-
+// Test that an empty update doesn't actually update the database.
+// This isn't a functionality requirement, but it is a useful
+// optimization.
+TEST_F(SafeBrowsingDatabaseTest, EmptyUpdate) {
   SBChunkList chunks;
   SBChunk chunk;
-  // Insert one host.
-  InsertAddChunkHostPrefixValue(&chunk, 1, 0, 0x31313131);
-  // Insert a second host, which has the same host prefix as the first one.
-  InsertAddChunkHostPrefixValue(&chunk, 1, 0, 0x32323232);
-  chunks.push_back(chunk);
 
-  // Insert the testing chunks into database.
+  FilePath filename = database_->BrowseDBFilename(database_filename_);
+
+  // Prime the database.
   std::vector<SBListChunkRanges> lists;
   EXPECT_TRUE(database_->UpdateStarted(&lists));
-  database_->InsertChunks(safe_browsing_util::kBinHashList, chunks);
+
+  InsertAddChunkHostPrefixUrl(&chunk, 1, "www.evil.com/",
+                              "www.evil.com/malware.html");
+  chunks.clear();
+  chunks.push_back(chunk);
+  database_->InsertChunks(safe_browsing_util::kMalwareList, chunks);
   database_->UpdateFinished(true);
 
-  GetListsInfo(&lists);
-  ASSERT_EQ(4U, lists.size());
-  EXPECT_EQ(std::string(safe_browsing_util::kBinHashList), lists[3].name);
-  EXPECT_EQ("1", lists[3].adds);
-  EXPECT_TRUE(lists[3].subs.empty());
+  // Get an older time to reset the lastmod time for detecting whether
+  // the file has been updated.
+  base::PlatformFileInfo before_info, after_info;
+  ASSERT_TRUE(file_util::GetFileInfo(filename, &before_info));
+  const base::Time old_last_modified =
+      before_info.last_modified - base::TimeDelta::FromSeconds(10);
 
-  // TODO(lzheng): Query database and verifies the prefixes once that API
-  // database is available in database.
-  database_.reset();
+  // Inserting another chunk updates the database file.  The sleep is
+  // needed because otherwise the entire test can finish w/in the
+  // resolution of the lastmod time.
+  ASSERT_TRUE(file_util::SetLastModifiedTime(filename, old_last_modified));
+  ASSERT_TRUE(file_util::GetFileInfo(filename, &before_info));
+  EXPECT_TRUE(database_->UpdateStarted(&lists));
+  chunk.hosts.clear();
+  InsertAddChunkHostPrefixUrl(&chunk, 2, "www.foo.com/",
+                              "www.foo.com/malware.html");
+  chunks.clear();
+  chunks.push_back(chunk);
+  database_->InsertChunks(safe_browsing_util::kMalwareList, chunks);
+  database_->UpdateFinished(true);
+  ASSERT_TRUE(file_util::GetFileInfo(filename, &after_info));
+  EXPECT_LT(before_info.last_modified, after_info.last_modified);
+
+  // Deleting a chunk updates the database file.
+  ASSERT_TRUE(file_util::SetLastModifiedTime(filename, old_last_modified));
+  ASSERT_TRUE(file_util::GetFileInfo(filename, &before_info));
+  EXPECT_TRUE(database_->UpdateStarted(&lists));
+  AddDelChunk(safe_browsing_util::kMalwareList, chunk.chunk_number);
+  database_->UpdateFinished(true);
+  ASSERT_TRUE(file_util::GetFileInfo(filename, &after_info));
+  EXPECT_LT(before_info.last_modified, after_info.last_modified);
+
+  // Simply calling |UpdateStarted()| then |UpdateFinished()| does not
+  // update the database file.
+  ASSERT_TRUE(file_util::SetLastModifiedTime(filename, old_last_modified));
+  ASSERT_TRUE(file_util::GetFileInfo(filename, &before_info));
+  EXPECT_TRUE(database_->UpdateStarted(&lists));
+  database_->UpdateFinished(true);
+  ASSERT_TRUE(file_util::GetFileInfo(filename, &after_info));
+  EXPECT_EQ(before_info.last_modified, after_info.last_modified);
 }
