@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -10,71 +10,69 @@
 #pragma once
 
 #include <deque>
+#include <map>
 #include <set>
 #include <string>
 #include <vector>
 
+#include "base/callback.h"
 #include "base/hash_tables.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/message_loop_helpers.h"
+#include "base/observer_list.h"
 #include "base/synchronization/lock.h"
 #include "base/time.h"
 #include "chrome/browser/safe_browsing/safe_browsing_util.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "googleurl/src/gurl.h"
-#include "webkit/glue/resource_type.h"
 
 class MalwareDetails;
+class PrefChangeRegistrar;
 class PrefService;
 class SafeBrowsingDatabase;
 class SafeBrowsingProtocolManager;
 class SafeBrowsingServiceFactory;
-class URLRequestContextGetter;
 
 namespace base {
 class Thread;
 }
 
+namespace net {
+class URLRequestContextGetter;
+}
+
+namespace safe_browsing {
+class ClientSideDetectionService;
+class DownloadProtectionService;
+}
+
 // Construction needs to happen on the main thread.
 class SafeBrowsingService
-    : public base::RefCountedThreadSafe<SafeBrowsingService> {
+    : public base::RefCountedThreadSafe<
+          SafeBrowsingService, content::BrowserThread::DeleteOnUIThread>,
+      public content::NotificationObserver {
  public:
+  class Client;
   // Users of this service implement this interface to be notified
   // asynchronously of the result.
   enum UrlCheckResult {
-    URL_SAFE,
+    SAFE,
     URL_PHISHING,
     URL_MALWARE,
-    BINARY_MALWARE,  // This binary is a malware.
+    BINARY_MALWARE_URL,  // Binary url leads to a malware.
+    BINARY_MALWARE_HASH,  // Binary hash indicates this is a malware.
+
+    // Url detected by the client-side phishing model.  Note that unlike the
+    // above values, this does not correspond to a downloaded list.
+    CLIENT_SIDE_PHISHING_URL,
   };
 
-  class Client {
-   public:
-    virtual ~Client() {}
-
-    void OnSafeBrowsingResult(const GURL& url, UrlCheckResult result) {
-      OnBrowseUrlCheckResult(url, result);
-      OnDownloadUrlCheckResult(url, result);
-      // TODO(lzheng): This is not implemented yet.
-      // OnDownloadHashCheckResult(url, result);
-    }
-
-    // Called when the user has made a decision about how to handle the
-    // SafeBrowsing interstitial page.
-    virtual void OnBlockingPageComplete(bool proceed) {}
-
-   protected:
-    // Called when the result of checking a browse URL is known.
-    virtual void OnBrowseUrlCheckResult(const GURL& url,
-                                        UrlCheckResult result) {}
-
-    // Called when the result of checking a download URL is known.
-    virtual void OnDownloadUrlCheckResult(const GURL& url,
-                                          UrlCheckResult result) {}
-
-    // Called when the result of checking a download binary hash is known.
-    virtual void OnDownloadHashCheckResult(const GURL& url,
-                                           UrlCheckResult result) {}
-  };
+  // Passed a boolean indicating whether or not it is OK to proceed with
+  // loading an URL.
+  typedef base::Callback<void(bool /*proceed*/)> UrlCheckCallback;
 
   // Structure used to pass parameters between the IO and UI thread when
   // interacting with the blocking page.
@@ -84,30 +82,79 @@ class SafeBrowsingService
 
     GURL url;
     GURL original_url;
-    ResourceType::Type resource_type;
+    std::vector<GURL> redirect_urls;
+    bool is_subresource;
     UrlCheckResult threat_type;
-    Client* client;
+    UrlCheckCallback callback;
     int render_process_host_id;
     int render_view_id;
   };
 
-  // Bundle of SafeBrowsing state for one URL check.
+  // Bundle of SafeBrowsing state for one URL or hash prefix check.
   struct SafeBrowsingCheck {
     SafeBrowsingCheck();
     ~SafeBrowsingCheck();
 
-    GURL url;
+    // Either |urls| or |prefix| is used to lookup database.
+    std::vector<GURL> urls;
+    scoped_ptr<SBFullHash> full_hash;
+
     Client* client;
     bool need_get_hash;
-    base::Time start;  // Time that check was sent to SB service.
+    base::TimeTicks start;  // When check was sent to SB service.
     UrlCheckResult result;
+    bool is_download;  // If this check for download url or hash.
     std::vector<SBPrefix> prefix_hits;
     std::vector<SBFullHashResult> full_hits;
 
+    // Vends weak pointers for TimeoutCallback().  If the response is
+    // received before the timeout fires, factory is destructed and
+    // the timeout won't be fired.
+    // TODO(lzheng): We should consider to use this time out check
+    // for browsing too (instead of implementin in
+    // safe_browsing_resource_handler.cc).
+    scoped_ptr<base::WeakPtrFactory<SafeBrowsingService> > timeout_factory_;
+
    private:
     DISALLOW_COPY_AND_ASSIGN(SafeBrowsingCheck);
   };
 
+  // Observer class can be used to get notified when a SafeBrowsing hit
+  // was found.
+  class Observer {
+   public:
+    // The |resource| must not be accessed after OnSafeBrowsingHit returns.
+    // This method will be called on the UI thread.
+    virtual void OnSafeBrowsingHit(const UnsafeResource& resource) = 0;
+
+   protected:
+    Observer() {}
+    virtual ~Observer() {}
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(Observer);
+  };
+
+  class Client {
+   public:
+    virtual ~Client() {}
+
+    void OnSafeBrowsingResult(const SafeBrowsingCheck& check);
+
+   protected:
+    // Called when the result of checking a browse URL is known.
+    virtual void OnBrowseUrlCheckResult(const GURL& url,
+                                        UrlCheckResult result) {}
+
+    // Called when the result of checking a download URL is known.
+    virtual void OnDownloadUrlCheckResult(const std::vector<GURL>& url_chain,
+                                          UrlCheckResult result) {}
+
+    // Called when the result of checking a download binary hash is known.
+    virtual void OnDownloadHashCheckResult(const std::string& hash,
+                                           UrlCheckResult result) {}
+  };
+
   // Makes the passed |factory| the factory used to instanciate
   // a SafeBrowsingService. Useful for tests.
   static void RegisterFactory(SafeBrowsingServiceFactory* factory) {
@@ -117,10 +164,6 @@ class SafeBrowsingService
   // Create an instance of the safe browsing service.
   static SafeBrowsingService* CreateSafeBrowsingService();
 
-  // Called on UI thread to decide if safe browsing related stats
-  // could be reported.
-  bool CanReportStats() const;
-
   // Called on the UI thread to initialize the service.
   void Initialize();
 
@@ -130,6 +173,10 @@ class SafeBrowsingService
   // Returns true if the url's scheme can be checked.
   bool CanCheckUrl(const GURL& url) const;
 
+  // Called on UI thread to decide if safe browsing related stats
+  // could be reported.
+  virtual bool CanReportStats() const;
+
   // Called on UI thread to decide if the download file's sha256 hash
   // should be calculated for safebrowsing.
   bool DownloadBinHashNeeded() const;
@@ -142,7 +189,31 @@ class SafeBrowsingService
 
   // Check if the prefix for |url| is in safebrowsing download add lists.
   // Result will be passed to callback in |client|.
-  bool CheckDownloadUrl(const GURL& url, Client* client);
+  virtual bool CheckDownloadUrl(const std::vector<GURL>& url_chain,
+                                Client* client);
+
+  // Check if the prefix for |full_hash| is in safebrowsing binhash add lists.
+  // Result will be passed to callback in |client|.
+  virtual bool CheckDownloadHash(const std::string& full_hash, Client* client);
+
+  // Check if the |url| matches any of the full-length hashes from the
+  // client-side phishing detection whitelist.  Returns true if there was a
+  // match and false otherwise.  To make sure we are conservative we will return
+  // true if an error occurs. This method is expected to be called on the IO
+  // thread.
+  virtual bool MatchCsdWhitelistUrl(const GURL& url);
+
+  // Check if the |url| matches any of the full-length hashes from the
+  // download whitelist.  Returns true if there was a match and false otherwise.
+  // To make sure we are conservative we will return true if an error occurs.
+  // This method is expected to be called on the IO thread.
+  virtual bool MatchDownloadWhitelistUrl(const GURL& url);
+
+  // Check if |str| matches any of the full-length hashes from the download
+  // whitelist.  Returns true if there was a match and false otherwise.
+  // To make sure we are conservative we will return true if an error occurs.
+  // This method is expected to be called on the IO thread.
+  virtual bool MatchDownloadWhitelistString(const std::string& str);
 
   // Called on the IO thread to cancel a pending check if the result is no
   // longer needed.
@@ -155,12 +226,20 @@ class SafeBrowsingService
   // chain). Otherwise, |original_url| = |url|.
   void DisplayBlockingPage(const GURL& url,
                            const GURL& original_url,
-                           ResourceType::Type resource_type,
+                           const std::vector<GURL>& redirect_urls,
+                           bool is_subresource,
                            UrlCheckResult result,
-                           Client* client,
+                           const UrlCheckCallback& callback,
                            int render_process_host_id,
                            int render_view_id);
 
+  // Same as above but gets invoked on the UI thread.
+  virtual void DoDisplayBlockingPage(const UnsafeResource& resource);
+
+  // Returns true if we already displayed an interstitial for that resource.
+  // Called on the UI thread.
+  bool IsWhitelisted(const UnsafeResource& resource);
+
   // Called on the IO thread when the SafeBrowsingProtocolManager has received
   // the full hash results for prefix hits detected in the database.
   void HandleGetHashResults(
@@ -187,22 +266,27 @@ class SafeBrowsingService
   void OnNewMacKeys(const std::string& client_key,
                     const std::string& wrapped_key);
 
-  // Notification on the UI thread from the advanced options UI.
-  void OnEnable(bool enabled);
-
   bool enabled() const { return enabled_; }
 
+  bool download_protection_enabled() const {
+    return enabled_ && enable_download_protection_;
+  }
+
+  safe_browsing::ClientSideDetectionService*
+      safe_browsing_detection_service() const {
+    return csd_service_.get();
+  }
+
+  // The DownloadProtectionService is not valid after the SafeBrowsingService
+  // is destroyed.
+  safe_browsing::DownloadProtectionService*
+      download_protection_service() const {
+    return download_service_.get();
+  }
+
   // Preference handling.
   static void RegisterPrefs(PrefService* prefs);
 
-  // Called on the IO thread to try to close the database, freeing the memory
-  // associated with it.  The database will be automatically reopened as needed.
-  //
-  // NOTE: Actual database closure is asynchronous, and until it happens, the IO
-  // thread is not allowed to access it; may not actually trigger a close if one
-  // is already pending or doing so would cause problems.
-  void CloseDatabase();
-
   // Called on the IO thread to reset the database.
   void ResetDatabase();
 
@@ -212,9 +296,23 @@ class SafeBrowsingService
   // the current page is 'safe'.
   void LogPauseDelay(base::TimeDelta time);
 
-  // When a safebrowsing blocking page goes away, it calls this method
-  // so the service can serialize and send MalwareDetails.
-  virtual void ReportMalwareDetails(scoped_refptr<MalwareDetails> details);
+  // Called on the IO thread by the MalwareDetails with the serialized
+  // protocol buffer, so the service can send it over.
+  virtual void SendSerializedMalwareDetails(const std::string& serialized);
+
+  // Report hits to the unsafe contents (malware, phishing, unsafe download URL)
+  // to the server. Can only be called on UI thread.  If |post_data| is
+  // non-empty, the request will be sent as a POST instead of a GET.
+  virtual void ReportSafeBrowsingHit(const GURL& malicious_url,
+                                     const GURL& page_url,
+                                     const GURL& referrer_url,
+                                     bool is_subresource,
+                                     UrlCheckResult threat_type,
+                                     const std::string& post_data);
+
+  // Add and remove observers.  These methods must be invoked on the UI thread.
+  void AddObserver(Observer* observer);
+  void RemoveObserver(Observer* remove);
 
  protected:
   // Creates the safe browsing service.  Need to initialize before using.
@@ -236,16 +334,18 @@ class SafeBrowsingService
   struct QueuedCheck {
     Client* client;
     GURL url;
-    base::Time start;
+    base::TimeTicks start;  // When check was queued.
   };
 
-  friend class base::RefCountedThreadSafe<SafeBrowsingService>;
+  friend struct content::BrowserThread::DeleteOnThread<
+      content::BrowserThread::UI>;
+  friend class base::DeleteHelper<SafeBrowsingService>;
   friend class SafeBrowsingServiceTest;
 
   // Called to initialize objects that are used on the io_thread.
   void OnIOInitialize(const std::string& client_key,
                       const std::string& wrapped_key,
-                      URLRequestContextGetter* request_context_getter);
+                      net::URLRequestContextGetter* request_context_getter);
 
   // Called to shutdown operations on the io_thread.
   void OnIOShutdown();
@@ -260,6 +360,14 @@ class SafeBrowsingService
   // db thread can call GetDatabase() directly.
   bool MakeDatabaseAvailable();
 
+  // Called on the IO thread to try to close the database, freeing the memory
+  // associated with it.  The database will be automatically reopened as needed.
+  //
+  // NOTE: Actual database closure is asynchronous, and until it happens, the IO
+  // thread is not allowed to access it; may not actually trigger a close if one
+  // is already pending or doing so would cause problems.
+  void CloseDatabase();
+
   // Should only be called on db thread as SafeBrowsingDatabase is not
   // threadsafe.
   SafeBrowsingDatabase* GetDatabase();
@@ -300,6 +408,10 @@ class SafeBrowsingService
   // UI.
   void Start();
 
+  // Stops the SafeBrowsingService. This can be called when the safe browsing
+  // preference is disabled.
+  void Stop();
+
   // Called on the db thread to close the database.  See CloseDatabase().
   void OnCloseDatabase();
 
@@ -320,23 +432,59 @@ class SafeBrowsingService
   bool HandleOneCheck(SafeBrowsingCheck* check,
                       const std::vector<SBFullHashResult>& full_hashes);
 
-  // Invoked on the UI thread to show the blocking page.
-  void DoDisplayBlockingPage(const UnsafeResource& resource);
+  // Call protocol manager on IO thread to report hits of unsafe contents.
+  void ReportSafeBrowsingHitOnIOThread(const GURL& malicious_url,
+                                       const GURL& page_url,
+                                       const GURL& referrer_url,
+                                       bool is_subresource,
+                                       UrlCheckResult threat_type,
+                                       const std::string& post_data);
 
-  // As soon as we create a blocking page, we schedule this method to
-  // report hits to the malware or phishing list to the server.
-  void ReportSafeBrowsingHit(const GURL& malicious_url,
-                             const GURL& page_url,
-                             const GURL& referrer_url,
-                             bool is_subresource,
-                             UrlCheckResult threat_type);
+  // Checks the download hash on safe_browsing_thread_.
+  void CheckDownloadHashOnSBThread(SafeBrowsingCheck* check);
 
   // Invoked by CheckDownloadUrl. It checks the download URL on
   // safe_browsing_thread_.
   void CheckDownloadUrlOnSBThread(SafeBrowsingCheck* check);
 
-  // Call the Client's callback in IO thread after CheckDownloadUrl finishes.
-  void CheckDownloadUrlDone(SafeBrowsingCheck* check, UrlCheckResult result);
+  // The callback function when a safebrowsing check is timed out. Client will
+  // be notified that the safebrowsing check is SAFE when this happens.
+  void TimeoutCallback(SafeBrowsingCheck* check);
+
+  // Calls the Client's callback on IO thread after CheckDownloadUrl finishes.
+  void CheckDownloadUrlDone(SafeBrowsingCheck* check);
+
+  // Calls the Client's callback on IO thread after CheckDownloadHash finishes.
+  void CheckDownloadHashDone(SafeBrowsingCheck* check);
+
+  // Helper function that calls safe browsing client and cleans up |checks_|.
+  void SafeBrowsingCheckDone(SafeBrowsingCheck* check);
+
+  // Helper function to set |check| with default values and start a safe
+  // browsing check with timeout of |timeout_ms|. |task| will be called upon
+  // success, otherwise TimeoutCallback will be called.
+  void StartDownloadCheck(SafeBrowsingCheck* check,
+                          Client* client,
+                          const base::Closure& task,
+                          int64 timeout_ms);
+
+  // Adds the given entry to the whitelist.  Called on the UI thread.
+  void UpdateWhitelist(const UnsafeResource& resource);
+
+  // content::NotificationObserver override
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
+
+  // Starts following the safe browsing preference on |pref_service|.
+  void AddPrefService(PrefService* pref_service);
+
+  // Stop following the safe browsing preference on |pref_service|.
+  void RemovePrefService(PrefService* pref_service);
+
+  // Checks if any profile is currently using the safe browsing service, and
+  // starts or stops the service accordingly.
+  void RefreshState();
 
   // The factory used to instanciate a SafeBrowsingService object.
   // Useful for tests, so they can provide their own implementation of
@@ -348,8 +496,8 @@ class SafeBrowsingService
   // Used for issuing only one GetHash request for a given prefix.
   GetHashRequests gethash_requests_;
 
-  // The sqlite database.  We don't use a scoped_ptr because it needs to be
-  // destructed on a different thread than this object.
+  // The persistent database.  We don't use a scoped_ptr because it
+  // needs to be destructed on a different thread than this object.
   SafeBrowsingDatabase* database_;
 
   // Lock used to prevent possible data races due to compiler optimizations.
@@ -358,6 +506,7 @@ class SafeBrowsingService
   // Handles interaction with SafeBrowsing servers.
   SafeBrowsingProtocolManager* protocol_manager_;
 
+  // Only access this whitelist from the UI thread.
   std::vector<WhiteListedEntry> white_listed_entries_;
 
   // Whether the service is running. 'enabled_' is used by SafeBrowsingService
@@ -368,6 +517,13 @@ class SafeBrowsingService
   // so we allow this feature to be exersized.
   bool enable_download_protection_;
 
+  // Indicate if client-side phishing detection whitelist should be enabled
+  // or not.
+  bool enable_csd_whitelist_;
+
+  // Indicate if the download whitelist should be enabled or not.
+  bool enable_download_whitelist_;
+
   // The SafeBrowsing thread that runs database operations.
   //
   // Note: Functions that run on this thread should run synchronously and return
@@ -388,6 +544,34 @@ class SafeBrowsingService
 
   std::deque<QueuedCheck> queued_checks_;
 
+  // When download url check takes this long, client's callback will be called
+  // without waiting for the result.
+  int64 download_urlcheck_timeout_ms_;
+
+  // Similar to |download_urlcheck_timeout_ms_|, but for download hash checks.
+  int64 download_hashcheck_timeout_ms_;
+
+  ObserverList<Observer> observer_list_;
+
+  // Used to track purge memory notifications. Lives on the IO thread.
+  scoped_ptr<content::NotificationRegistrar> registrar_;
+
+  // Tracks existing PrefServices, and the safe browsing preference on each.
+  // This is used to determine if any profile is currently using the safe
+  // browsing service, and to start it up or shut it down accordingly.
+  std::map<PrefService*, PrefChangeRegistrar*> prefs_map_;
+
+  // Used to track creation and destruction of profiles on the UI thread.
+  content::NotificationRegistrar prefs_registrar_;
+
+  // The ClientSideDetectionService is managed by the SafeBrowsingService,
+  // since its running state and lifecycle depends on SafeBrowsingService's.
+  scoped_ptr<safe_browsing::ClientSideDetectionService> csd_service_;
+
+  // The DownloadProtectionService is managed by the SafeBrowsingService,
+  // since its running state and lifecycle depends on SafeBrowsingService's.
+  scoped_ptr<safe_browsing::DownloadProtectionService> download_service_;
+
   DISALLOW_COPY_AND_ASSIGN(SafeBrowsingService);
 };
 
