@@ -1,14 +1,16 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/safe_browsing/safe_browsing_util.h"
 
 #include "base/base64.h"
-#include "base/hmac.h"
-#include "base/sha2.h"
+#include "base/logging.h"
 #include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "chrome/browser/google/google_util.h"
+#include "crypto/hmac.h"
+#include "crypto/sha2.h"
 #include "googleurl/src/gurl.h"
 #include "googleurl/src/url_util.h"
 #include "net/base/escape.h"
@@ -162,11 +164,15 @@ void SBEntry::SetFullHashAt(int index, const SBFullHash& full_hash) {
 
 namespace safe_browsing_util {
 
+// Listnames that browser can process.
 const char kMalwareList[] = "goog-malware-shavar";
 const char kPhishingList[] = "goog-phish-shavar";
-
 const char kBinUrlList[] = "goog-badbinurl-shavar";
-const char kBinHashList[] = "goog-badbinhash-shavar";
+// We don't use the bad binary digest list anymore.  Use a fake listname to be
+// sure we don't request it accidentally.
+const char kBinHashList[] = "goog-badbin-digestvar-disabled";
+const char kCsdWhiteList[] = "goog-csdwhite-sha256";
+const char kDownloadWhiteList[] = "goog-downloadwhite-digest256";
 
 int GetListId(const std::string& name) {
   int id;
@@ -176,8 +182,12 @@ int GetListId(const std::string& name) {
     id = PHISH;
   } else if (name == safe_browsing_util::kBinUrlList) {
     id = BINURL;
-  }  else if (name == safe_browsing_util::kBinHashList) {
+  } else if (name == safe_browsing_util::kBinHashList) {
     id = BINHASH;
+  } else if (name == safe_browsing_util::kCsdWhiteList) {
+    id = CSDWHITELIST;
+  } else if (name == safe_browsing_util::kDownloadWhiteList) {
+    id = DOWNLOADWHITELIST;
   } else {
     id = INVALID;
   }
@@ -198,6 +208,12 @@ bool GetListName(int list_id, std::string* list) {
     case BINHASH:
       *list = safe_browsing_util::kBinHashList;
       break;
+    case CSDWHITELIST:
+      *list = safe_browsing_util::kCsdWhiteList;
+      break;
+    case DOWNLOADWHITELIST:
+      *list = safe_browsing_util::kDownloadWhiteList;
+      break;
     default:
       return false;
   }
@@ -211,9 +227,9 @@ std::string Unescape(const std::string& url) {
   int loop_var = 0;
   do {
     old_unescaped_str = unescaped_str;
-    unescaped_str = UnescapeURLComponent(old_unescaped_str,
-        UnescapeRule::CONTROL_CHARS | UnescapeRule::SPACES |
-        UnescapeRule::URL_SPECIAL_CHARS);
+    unescaped_str = net::UnescapeURLComponent(old_unescaped_str,
+        net::UnescapeRule::CONTROL_CHARS | net::UnescapeRule::SPACES |
+        net::UnescapeRule::URL_SPECIAL_CHARS);
   } while (unescaped_str != old_unescaped_str && ++loop_var <=
            kMaxLoopIterations);
 
@@ -409,42 +425,58 @@ void GeneratePathsToCheck(const GURL& url, std::vector<std::string>* paths) {
     paths->push_back(path + "?" + query);
 }
 
-int CompareFullHashes(const GURL& url,
-                      const std::vector<SBFullHashResult>& full_hashes) {
-  if (full_hashes.empty())
-    return -1;
-
+void GeneratePatternsToCheck(const GURL& url, std::vector<std::string>* urls) {
   std::vector<std::string> hosts, paths;
   GenerateHostsToCheck(url, &hosts);
   GeneratePathsToCheck(url, &paths);
-
   for (size_t h = 0; h < hosts.size(); ++h) {
     for (size_t p = 0; p < paths.size(); ++p) {
-      SBFullHash key;
-      base::SHA256HashString(hosts[h] + paths[p],
-                             key.full_hash,
-                             sizeof(SBFullHash));
-
-      for (size_t i = 0; i < full_hashes.size(); ++i) {
-        if (key == full_hashes[i].hash)
-          return static_cast<int>(i);
-      }
+      urls->push_back(hosts[h] + paths[p]);
     }
   }
+}
 
+int GetHashIndex(const SBFullHash& hash,
+                 const std::vector<SBFullHashResult>& full_hashes) {
+  for (size_t i = 0; i < full_hashes.size(); ++i) {
+    if (hash == full_hashes[i].hash)
+      return static_cast<int>(i);
+  }
+  return -1;
+}
+
+int GetUrlHashIndex(const GURL& url,
+                    const std::vector<SBFullHashResult>& full_hashes) {
+  if (full_hashes.empty())
+    return -1;
+
+  std::vector<std::string> patterns;
+  GeneratePatternsToCheck(url, &patterns);
+
+  for (size_t i = 0; i < patterns.size(); ++i) {
+    SBFullHash key;
+    crypto::SHA256HashString(patterns[i], key.full_hash, sizeof(SBFullHash));
+    int index = GetHashIndex(key, full_hashes);
+    if (index != -1)
+      return index;
+  }
   return -1;
 }
 
 bool IsPhishingList(const std::string& list_name) {
-  return list_name.find("-phish-") != std::string::npos;
+  return list_name.compare(kPhishingList) == 0;
 }
 
 bool IsMalwareList(const std::string& list_name) {
-  return list_name.find("-malware-") != std::string::npos;
+  return list_name.compare(kMalwareList) == 0;
 }
 
 bool IsBadbinurlList(const std::string& list_name) {
-  return list_name.find("-badbinurl-") != std::string::npos;
+  return list_name.compare(kBinUrlList) == 0;
+}
+
+bool IsBadbinhashList(const std::string& list_name) {
+  return list_name.compare(kBinHashList) == 0;
 }
 
 static void DecodeWebSafe(std::string* decoded) {
@@ -469,7 +501,7 @@ bool VerifyMAC(const std::string& key, const std::string& mac,
   std::string decoded_mac;
   base::Base64Decode(mac_copy, &decoded_mac);
 
-  base::HMAC hmac(base::HMAC::SHA1);
+  crypto::HMAC hmac(crypto::HMAC::SHA1);
   if (!hmac.Init(decoded_key))
     return false;
   const std::string data_str(data, data_length);
@@ -481,14 +513,16 @@ bool VerifyMAC(const std::string& key, const std::string& mac,
 }
 
 GURL GeneratePhishingReportUrl(const std::string& report_page,
-                               const std::string& url_to_report) {
+                               const std::string& url_to_report,
+                               bool is_client_side_detection) {
   icu::Locale locale = icu::Locale::getDefault();
   const char* lang = locale.getLanguage();
   if (!lang)
     lang = "en";  // fallback
-  const std::string continue_esc =
-      EscapeQueryParamValue(StringPrintf(kContinueUrlFormat, lang), true);
-  const std::string current_esc = EscapeQueryParamValue(url_to_report, true);
+  const std::string continue_esc = net::EscapeQueryParamValue(
+      base::StringPrintf(kContinueUrlFormat, lang), true);
+  const std::string current_esc = net::EscapeQueryParamValue(url_to_report,
+                                                             true);
 
 #if defined(OS_WIN)
   BrowserDistribution* dist = BrowserDistribution::GetDistribution();
@@ -496,11 +530,23 @@ GURL GeneratePhishingReportUrl(const std::string& report_page,
 #else
   std::string client_name("googlechrome");
 #endif
+  if (is_client_side_detection)
+    client_name.append("_csd");
 
-  GURL report_url(report_page +
-      StringPrintf(kReportParams, client_name.c_str(), continue_esc.c_str(),
-                   current_esc.c_str()));
+  GURL report_url(report_page + base::StringPrintf(kReportParams,
+                                                   client_name.c_str(),
+                                                   continue_esc.c_str(),
+                                                   current_esc.c_str()));
   return google_util::AppendGoogleLocaleParam(report_url);
 }
 
+void StringToSBFullHash(const std::string& hash_in, SBFullHash* hash_out) {
+  DCHECK_EQ(crypto::kSHA256Length, hash_in.size());
+  memcpy(hash_out->full_hash, hash_in.data(), crypto::kSHA256Length);
+}
+
+std::string SBFullHashToString(const SBFullHash& hash) {
+  DCHECK_EQ(crypto::kSHA256Length, sizeof(hash.full_hash));
+  return std::string(hash.full_hash, sizeof(hash.full_hash));
+}
 }  // namespace safe_browsing_util
