@@ -9,16 +9,17 @@
 #endif
 
 #include "base/command_line.h"
-#include "base/scoped_vector.h"
+#include "base/memory/scoped_vector.h"
 #include "base/string16.h"
 #include "base/string_util.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/google/google_util.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/search_engines/search_engine_type.h"
 #include "chrome/browser/search_engines/search_terms_data.h"
 #include "chrome/browser/search_engines/template_url.h"
-#include "chrome/browser/search_engines/template_url_model.h"
+#include "chrome/browser/search_engines/template_url_service.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
 #include "googleurl/src/gurl.h"
@@ -78,7 +79,10 @@ struct PrepopulatedEngine {
   //
   // IDs > 1000 are reserved for distribution custom engines.
   //
-  // NOTE: CHANGE THE ABOVE NUMBERS IF YOU ADD A NEW ENGINE; ID conflicts = bad!
+  // NOTES:
+  //   CHANGE THE ABOVE NUMBERS IF YOU ADD A NEW ENGINE; ID conflicts = bad!
+  //   CHANGE kMaxPrepopulatedEngineID below if you add new engine outside
+  //       of the current range or it will not be counted in stats.
   const int id;
 };
 
@@ -1181,12 +1185,16 @@ const PrepopulatedEngine google = {
   NULL,
   "http://www.google.com/favicon.ico",
   L"{google:baseURL}search?{google:RLZ}{google:acceptedSuggestion}"
-      L"{google:originalQueryForSuggestion}sourceid=chrome&ie={inputEncoding}&"
-      L"q={searchTerms}",
+      L"{google:originalQueryForSuggestion}{google:searchFieldtrialParameter}"
+      L"{google:instantFieldTrialGroupParameter}"
+      L"sourceid=chrome&ie={inputEncoding}&q={searchTerms}",
   "UTF-8",
-  L"{google:baseSuggestURL}search?client=chrome&hl={language}&q={searchTerms}",
-  L"{google:baseURL}webhp?{google:RLZ}sourceid=chrome-instant"
-      L"&ie={inputEncoding}&ion=1{searchTerms}&nord=1",
+  L"{google:baseSuggestURL}search?{google:searchFieldtrialParameter}"
+      L"{google:instantFieldTrialGroupParameter}"
+      L"client=chrome&hl={language}&q={searchTerms}",
+  L"{google:baseURL}webhp?{google:RLZ}sourceid=chrome-instant&"
+      L"{google:instantFieldTrialGroupParameter}"
+      L"ie={inputEncoding}{google:instantEnabledParameter}{searchTerms}",
   SEARCH_ENGINE_GOOGLE,
   IDR_SEARCH_ENGINE_LOGO_GOOGLE,
   1,
@@ -1315,7 +1323,7 @@ const PrepopulatedEngine mail_ru = {
   "http://img.go.mail.ru/favicon.ico",
   L"http://go.mail.ru/search?q={searchTerms}",
   "windows-1251",
-  NULL,
+  L"http://suggests.go.mail.ru/chrome?q={searchTerms}",
   NULL,
   SEARCH_ENGINE_MAILRU,
   IDR_SEARCH_ENGINE_LOGO_MAILRU,
@@ -1467,32 +1475,6 @@ const PrepopulatedEngine onet = {
   75,
 };
 
-const PrepopulatedEngine pogodak_ba = {
-  L"Pogodak!",
-  L"pogodak.ba",
-  "http://www.pogodak.ba/favicon.ico",
-  L"http://www.pogodak.ba/search.jsp?q={searchTerms}",
-  "UTF-8",
-  NULL,
-  NULL,
-  SEARCH_ENGINE_POGODAK,
-  IDR_SEARCH_ENGINE_LOGO_POGODAK,
-  24,
-};
-
-const PrepopulatedEngine pogodak_hr = {
-  L"Pogodak!",
-  L"pogodak.hr",
-  "http://www.pogodak.hr/favicon.ico",
-  L"http://www.pogodak.hr/search.jsp?q={searchTerms}",
-  "UTF-8",
-  NULL,
-  NULL,
-  SEARCH_ENGINE_POGODAK,
-  IDR_SEARCH_ENGINE_LOGO_POGODAK,
-  24,
-};
-
 const PrepopulatedEngine pogodak_rs = {
   L"Pogodak!",
   L"pogodak.rs",
@@ -1506,19 +1488,6 @@ const PrepopulatedEngine pogodak_rs = {
   24,
 };
 
-const PrepopulatedEngine pogodok = {
-  L"\x041f\x043e\x0433\x043e\x0434\x043e\x043a!",
-  L"pogodok.com.mk",
-  "http://www.pogodok.com.mk/favicon.ico",
-  L"http://www.pogodok.com.mk/search.jsp?q={searchTerms}",
-  "UTF-8",
-  NULL,
-  NULL,
-  SEARCH_ENGINE_POGODOK_MK,
-  IDR_SEARCH_ENGINE_LOGO_POGODOK_MK,
-  24,  // Really the same engine as Pogodak, just has a small name change.
-};
-
 const PrepopulatedEngine rambler = {
   L"Rambler",
   L"rambler.ru",
@@ -2361,7 +2330,7 @@ const PrepopulatedEngine* engines_AU[] =
 
 // Bosnia and Herzegovina
 const PrepopulatedEngine* engines_BA[] =
-    { &google, &pogodak_ba, &yahoo, &bing, };
+    { &google, &yahoo, &bing, };
 
 // Belgium
 const PrepopulatedEngine* engines_BE[] =
@@ -2494,7 +2463,7 @@ const PrepopulatedEngine* engines_HN[] =
 
 // Croatia
 const PrepopulatedEngine* engines_HR[] =
-    { &google, &yahoo, &pogodak_hr, &bing_hr_HR, };
+    { &google, &yahoo, &bing_hr_HR, };
 
 // Hungary
 const PrepopulatedEngine* engines_HU[] =
@@ -2602,7 +2571,7 @@ const PrepopulatedEngine* engines_ME[] =
 
 // Macedonia
 const PrepopulatedEngine* engines_MK[] =
-    { &google, &pogodok, &yahoo, &bing, };
+    { &google, &yahoo, &bing, };
 
 // Mexico
 const PrepopulatedEngine* engines_MX[] =
@@ -2788,17 +2757,16 @@ const PrepopulatedEngine* kAllEngines[] =
       &delfi_lv, &diri, &eniro_fi, &eniro_se, &fonecta_02_fi, &go, &goo,
       &google, &guruji, &hispavista, &in, &jabse, &jubii, &kvasir, &latne,
       &leit, &libero, &mail_ru, &maktoob, &masrawy, &mynet, &najdi, &nate,
-      &naver, &neti, &netsprint, &nur_kz, &ok, &onet, &pogodak_ba, &pogodak_hr,
-      &pogodak_rs, &pogodok, &rambler, &rediff, &rednano, &sanook, &sapo,
-      &search_de_CH, &search_fr_CH, &seznam, &spray, &terra_ar, &terra_es, &tut,
-      &uol, &virgilio, &walla, &wp, &yahoo, &yahoo_ar, &yahoo_at, &yahoo_au,
-      &yahoo_br, &yahoo_ca, &yahoo_ch, &yahoo_cl, &yahoo_cn, &yahoo_co,
-      &yahoo_de, &yahoo_dk, &yahoo_es, &yahoo_fi, &yahoo_fr, &yahoo_hk,
-      &yahoo_id, &yahoo_in, &yahoo_it, &yahoo_jp, &yahoo_kr, &yahoo_malaysia,
-      &yahoo_mx, &yahoo_nl, &yahoo_no, &yahoo_nz, &yahoo_pe, &yahoo_ph,
-      &yahoo_qc, &yahoo_ru, &yahoo_se, &yahoo_sg, &yahoo_th, &yahoo_tw,
-      &yahoo_uk, &yahoo_ve, &yahoo_vn, &yamli, &yandex_ru, &yandex_ua,
-      &zoznam };
+      &naver, &neti, &netsprint, &nur_kz, &ok, &onet, &pogodak_rs, &rambler,
+      &rediff, &rednano, &sanook, &sapo, &search_de_CH, &search_fr_CH, &seznam,
+      &spray, &terra_ar, &terra_es, &tut, &uol, &virgilio, &walla, &wp, &yahoo,
+      &yahoo_ar, &yahoo_at, &yahoo_au, &yahoo_br, &yahoo_ca, &yahoo_ch,
+      &yahoo_cl, &yahoo_cn, &yahoo_co, &yahoo_de, &yahoo_dk, &yahoo_es,
+      &yahoo_fi, &yahoo_fr, &yahoo_hk, &yahoo_id, &yahoo_in, &yahoo_it,
+      &yahoo_jp, &yahoo_kr, &yahoo_malaysia, &yahoo_mx, &yahoo_nl, &yahoo_no,
+      &yahoo_nz, &yahoo_pe, &yahoo_ph, &yahoo_qc, &yahoo_ru, &yahoo_se,
+      &yahoo_sg, &yahoo_th, &yahoo_tw, &yahoo_uk, &yahoo_ve, &yahoo_vn, &yamli,
+      &yandex_ru, &yandex_ua, &zoznam };
 
 
 // Geographic mappings /////////////////////////////////////////////////////////
@@ -3340,18 +3308,29 @@ void GetPrepopulationSetFromCountryID(PrefService* prefs,
 
 namespace TemplateURLPrepopulateData {
 
+// The following id is for UMA stats only. Please update
+// kMaxPrepopulatedEngineID if it changes upwards.
+const int kMaxPrepopulatedEngineID = 101;
+
 void RegisterUserPrefs(PrefService* prefs) {
-  prefs->RegisterIntegerPref(prefs::kCountryIDAtInstall, kCountryIDUnknown);
-  prefs->RegisterListPref(prefs::kSearchProviderOverrides);
-  prefs->RegisterIntegerPref(prefs::kSearchProviderOverridesVersion, -1);
+  prefs->RegisterIntegerPref(prefs::kCountryIDAtInstall,
+                             kCountryIDUnknown,
+                             PrefService::UNSYNCABLE_PREF);
+  prefs->RegisterListPref(prefs::kSearchProviderOverrides,
+                          PrefService::UNSYNCABLE_PREF);
+  prefs->RegisterIntegerPref(prefs::kSearchProviderOverridesVersion,
+                             -1,
+                             PrefService::UNSYNCABLE_PREF);
   // Obsolete pref, for migration.
-  prefs->RegisterIntegerPref(prefs::kGeoIDAtInstall, -1);
+  prefs->RegisterIntegerPref(prefs::kGeoIDAtInstall,
+                             -1,
+                             PrefService::UNSYNCABLE_PREF);
 }
 
 int GetDataVersion(PrefService* prefs) {
   // Increment this if you change the above data in ways that mean users with
   // existing data should get a new version.
-  const int kCurrentDataVersion = 33;
+  const int kCurrentDataVersion = 38;
   if (!prefs)
     return kCurrentDataVersion;
   // If a version number exist in the preferences file, it overrides the
@@ -3374,7 +3353,7 @@ TemplateURL* MakePrepopulatedTemplateURL(const wchar_t* name,
   TemplateURL* new_turl = new TemplateURL();
   new_turl->SetURL(WideToUTF8(search_url), 0, 0);
   if (favicon_url)
-    new_turl->SetFavIconURL(GURL(favicon_url));
+    new_turl->SetFaviconURL(GURL(favicon_url));
   if (suggest_url)
     new_turl->SetSuggestionsURL(WideToUTF8(suggest_url), 0, 0);
   if (instant_url)
@@ -3387,12 +3366,13 @@ TemplateURL* MakePrepopulatedTemplateURL(const wchar_t* name,
   new_turl->set_show_in_default_list(true);
   new_turl->set_safe_for_autoreplace(true);
   new_turl->set_date_created(Time());
+  new_turl->set_last_modified(Time());
   std::vector<std::string> turl_encodings;
   turl_encodings.push_back(encoding);
   new_turl->set_input_encodings(turl_encodings);
   new_turl->set_search_engine_type(search_engine_type);
   new_turl->set_logo_id(logo_id);
-  new_turl->set_prepopulate_id(id);
+  new_turl->SetPrepopulateId(id);
   return new_turl;
 }
 
@@ -3421,8 +3401,8 @@ void GetPrepopulatedTemplateFromPrefs(PrefService* prefs,
   for (size_t i = 0; i != num_engines; ++i) {
     Value* val;
     DictionaryValue* engine;
-    list->GetDictionary(i, &engine);
-    if (engine->Get("name", &val) && val->GetAsString(&name) &&
+    if (list->GetDictionary(i, &engine) &&
+        engine->Get("name", &val) && val->GetAsString(&name) &&
         engine->Get("keyword", &val) && val->GetAsString(&keyword) &&
         engine->Get("search_url", &val) && val->GetAsString(&search_url) &&
         engine->Get("suggest_url", &val) && val->GetAsString(&suggest_url) &&
@@ -3525,7 +3505,7 @@ static GURL GetOriginForSearchURL(const STR& url_string) {
     turl.SetURL(url_utf8_string, 0, 0);
 
     UIThreadSearchTermsData search_terms_data;
-    url = TemplateURLModel::GenerateSearchURLUsingTermsData(
+    url = TemplateURLService::GenerateSearchURLUsingTermsData(
         &turl, search_terms_data);
   }
   return url.GetOrigin();
@@ -3571,4 +3551,13 @@ int GetSearchEngineLogo(const GURL& url_to_find) {
   return kNoSearchEngineLogo;
 }
 
+TemplateURL* FindPrepopulatedEngine(const std::string& search_url) {
+  GURL search_origin(GetOriginForSearchURL(search_url));
+  // First check if it is a Google URL. User may have a custom search provider
+  // with a hard-coded Google domain instead of {google:baseURL}.
+  if (google_util::IsGoogleHomePageUrl((search_origin.spec())))
+    return MakePrepopulateTemplateURLFromPrepopulateEngine(google);
+  return GetEngineForOrigin(NULL, search_origin);
+}
+
 }  // namespace TemplateURLPrepopulateData
