@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,8 @@
 #define CHROME_BROWSER_SEARCH_ENGINES_TEMPLATE_URL_PREPOPULATE_DATA_H_
 #pragma once
 
+#include <stddef.h>
+#include <string>
 #include <vector>
 
 class GURL;
@@ -14,6 +16,8 @@ class TemplateURL;
 
 namespace TemplateURLPrepopulateData {
 
+extern const int kMaxPrepopulatedEngineID;
+
 void RegisterUserPrefs(PrefService* prefs);
 
 // Returns the current version of the prepopulate data, so callers can know when
@@ -31,6 +35,7 @@ void GetPrepopulatedEngines(PrefService* prefs,
 
 // Returns the default search provider specified by the prepopulate data.
 // The caller owns the returned value, which may be NULL.
+// If |prefs| is NULL, search provider overrides from preferences are not used.
 TemplateURL* GetPrepopulatedDefaultSearch(PrefService* prefs);
 
 // Returns a TemplateURL from the prepopulated data which has the same origin
@@ -41,6 +46,11 @@ TemplateURL* GetEngineForOrigin(PrefService* prefs, const GURL& url_to_find);
 // Returns search engine logo for URLs known to have a search engine logo.
 int GetSearchEngineLogo(const GURL& url_to_find);
 
+// Returns the prepopulated search provider whose search URL origin matches the
+// origin of |search_url| or NULL if none is found. The caller is responsible
+// for deleting the returned TemplateURL.
+TemplateURL* FindPrepopulatedEngine(const std::string& search_url);
+
 }  // namespace TemplateURLPrepopulateData
 
 #endif  // CHROME_BROWSER_SEARCH_ENGINES_TEMPLATE_URL_PREPOPULATE_DATA_H_
