@@ -151,6 +151,31 @@ TEST_F(TemplateURLTest, URLRefTestEncoding) {
   ASSERT_EQ("http://fooxxutf-8ya/", result.spec());
 }
 
+// Test that setting the prepopulate ID from TemplateURL causes the stored
+// TemplateURLRef to handle parsing the URL parameters differently.
+TEST_F(TemplateURLTest, SetPrepopulatedAndParse) {
+  TemplateURL t_url;
+  t_url.SetURL("http://foo{fhqwhgads}", 0, 0);
+  TemplateURLRef::Replacements replacements;
+  bool valid = false;
+
+  t_url.SetPrepopulateId(0);
+  EXPECT_EQ("http://foo{fhqwhgads}",
+            t_url.url()->ParseURL("http://foo{fhqwhgads}",
+            &replacements,
+            &valid));
+  EXPECT_TRUE(replacements.empty());
+  EXPECT_TRUE(valid);
+
+  t_url.SetPrepopulateId(123);
+  EXPECT_EQ("http://foo",
+            t_url.url()->ParseURL("http://foo{fhqwhgads}",
+            &replacements,
+            &valid));
+  EXPECT_TRUE(replacements.empty());
+  EXPECT_TRUE(valid);
+}
+
 TEST_F(TemplateURLTest, InputEncodingBeforeSearchTerm) {
   TemplateURL t_url;
   TemplateURLRef ref(
@@ -184,7 +209,7 @@ TEST_F(TemplateURLTest, URLRefTestSearchTermsUsingTermsData) {
     { "{google:baseURL}{language}{searchTerms}", string16(),
       "http://example.com/e/yy" },
     { "{google:baseSuggestURL}{searchTerms}", string16(),
-      "http://clients1.example.com/complete/" }
+      "http://example.com/complete/" }
   };
 
   TestSearchTermsData search_terms_data("http://example.com/e/");
@@ -243,17 +268,17 @@ TEST_F(TemplateURLTest, URLRefTermToWide) {
   }
 }
 
-TEST_F(TemplateURLTest, SetFavIcon) {
+TEST_F(TemplateURLTest, SetFavicon) {
   TemplateURL url;
   GURL favicon_url("http://favicon.url");
-  url.SetFavIconURL(favicon_url);
+  url.SetFaviconURL(favicon_url);
   ASSERT_EQ(1U, url.image_refs().size());
-  ASSERT_TRUE(favicon_url == url.GetFavIconURL());
+  ASSERT_TRUE(favicon_url == url.GetFaviconURL());
 
   GURL favicon_url2("http://favicon2.url");
-  url.SetFavIconURL(favicon_url2);
+  url.SetFaviconURL(favicon_url2);
   ASSERT_EQ(1U, url.image_refs().size());
-  ASSERT_TRUE(favicon_url2 == url.GetFavIconURL());
+  ASSERT_TRUE(favicon_url2 == url.GetFaviconURL());
 }
 
 TEST_F(TemplateURLTest, DisplayURLToURLRef) {
@@ -448,12 +473,11 @@ TEST_F(TemplateURLTest, GoogleBaseSuggestURL) {
     const char* const base_url;
     const char* const base_suggest_url;
   } data[] = {
-    { "http://google.com/", "http://clients1.google.com/complete/", },
-    { "http://www.google.com/", "http://clients1.google.com/complete/", },
-    { "http://www.google.co.uk/", "http://clients1.google.co.uk/complete/", },
-    { "http://www.google.com.by/",
-      "http://clients1.google.com.by/complete/", },
-    { "http://google.com/intl/xx/", "http://clients1.google.com/complete/", },
+    { "http://google.com/", "http://google.com/complete/", },
+    { "http://www.google.com/", "http://www.google.com/complete/", },
+    { "http://www.google.co.uk/", "http://www.google.co.uk/complete/", },
+    { "http://www.google.com.by/", "http://www.google.com.by/complete/", },
+    { "http://google.com/intl/xx/", "http://google.com/complete/", },
   };
 
   for (size_t i = 0; i < ARRAYSIZE_UNSAFE(data); ++i)
@@ -486,11 +510,21 @@ TEST_F(TemplateURLTest, ParseParameterKnown) {
 }
 
 TEST_F(TemplateURLTest, ParseParameterUnknown) {
-  std::string parsed_url("{}");
+  std::string parsed_url("{fhqwhgads}");
   TemplateURLRef url_ref(parsed_url, 0, 0);
   TemplateURLRef::Replacements replacements;
-  EXPECT_FALSE(url_ref.ParseParameter(0, 1, &parsed_url, &replacements));
-  EXPECT_EQ("{}", parsed_url);
+
+  // By default, TemplateURLRef should not consider itself prepopulated.
+  // Therefore we should not replace the unknown parameter.
+  EXPECT_FALSE(url_ref.ParseParameter(0, 10, &parsed_url, &replacements));
+  EXPECT_EQ("{fhqwhgads}", parsed_url);
+  EXPECT_TRUE(replacements.empty());
+
+  // If the TemplateURLRef is prepopulated, we should remove unknown parameters.
+  parsed_url = "{fhqwhgads}";
+  url_ref.set_prepopulated(true);
+  EXPECT_FALSE(url_ref.ParseParameter(0, 10, &parsed_url, &replacements));
+  EXPECT_EQ("", parsed_url);
   EXPECT_TRUE(replacements.empty());
 }
 
