@@ -1,16 +1,17 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/process_util.h"
 #include "base/test/test_timeouts.h"
 #include "chrome/browser/service/service_process_control.h"
-#include "chrome/browser/service/service_process_control_manager.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/common/chrome_version_info.h"
 #include "chrome/common/service_process_util.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
 
 class ServiceProcessControlBrowserTest
     : public InProcessBrowserTest {
@@ -21,82 +22,79 @@ class ServiceProcessControlBrowserTest
   ~ServiceProcessControlBrowserTest() {
     base::CloseProcessHandle(service_process_handle_);
     service_process_handle_ = base::kNullProcessHandle;
-    // Delete all instances of ServiceProcessControl.
-    ServiceProcessControlManager::GetInstance()->Shutdown();
   }
 
+#if defined(OS_MACOSX)
+  virtual void TearDown() {
+    // ForceServiceProcessShutdown removes the process from launched on Mac.
+    ForceServiceProcessShutdown("", 0);
+  }
+#endif  // OS_MACOSX
+
  protected:
   void LaunchServiceProcessControl() {
-    ServiceProcessControl* process =
-        ServiceProcessControlManager::GetInstance()->GetProcessControl(
-            browser()->profile());
-    process_ = process;
-
     // Launch the process asynchronously.
-    process->Launch(
-        NewRunnableMethod(
-            this,
-            &ServiceProcessControlBrowserTest::ProcessControlLaunched),
-        NewRunnableMethod(
-            this,
-            &ServiceProcessControlBrowserTest::ProcessControlLaunchFailed));
+    ServiceProcessControl::GetInstance()->Launch(
+        base::Bind(&ServiceProcessControlBrowserTest::ProcessControlLaunched,
+                   this),
+        base::Bind(
+            &ServiceProcessControlBrowserTest::ProcessControlLaunchFailed,
+            this));
 
     // Then run the message loop to keep things running.
     ui_test_utils::RunMessageLoop();
   }
 
-  // Send a remoting host status request and wait reply from the service.
+  // Send a Cloud Print status request and wait for a reply from the service.
   void SendRequestAndWait() {
-    process()->GetCloudPrintProxyStatus(NewCallback(
-        this, &ServiceProcessControlBrowserTest::CloudPrintStatusCallback));
+    ServiceProcessControl::GetInstance()->GetCloudPrintProxyInfo(
+        base::Bind(&ServiceProcessControlBrowserTest::CloudPrintInfoCallback,
+                   base::Unretained(this)));
     ui_test_utils::RunMessageLoop();
   }
 
-  void CloudPrintStatusCallback(
-      bool enabled, std::string email) {
+  void CloudPrintInfoCallback(
+      const cloud_print::CloudPrintProxyInfo& proxy_info) {
     MessageLoop::current()->Quit();
   }
 
   void Disconnect() {
-    // This will delete all instances of ServiceProcessControl and close the IPC
-    // connections.
-    ServiceProcessControlManager::GetInstance()->Shutdown();
-    process_ = NULL;
+    // This will close the IPC connection.
+    ServiceProcessControl::GetInstance()->Disconnect();
   }
 
   void WaitForShutdown() {
     EXPECT_TRUE(base::WaitForSingleProcess(
         service_process_handle_,
-        TestTimeouts::wait_for_terminate_timeout_ms()));
+        TestTimeouts::action_max_timeout_ms()));
   }
 
   void ProcessControlLaunched() {
-    base::ProcessId service_pid = GetServiceProcessPid();
+    base::ProcessId service_pid;
+    EXPECT_TRUE(GetServiceProcessData(NULL, &service_pid));
     EXPECT_NE(static_cast<base::ProcessId>(0), service_pid);
     EXPECT_TRUE(base::OpenProcessHandleWithAccess(
         service_pid,
-        base::kProcessAccessWaitForTermination,
+        base::kProcessAccessWaitForTermination |
+        // we need query permission to get exit code
+        base::kProcessAccessQueryInformation,
         &service_process_handle_));
     // Quit the current message. Post a QuitTask instead of just calling Quit()
     // because this can get invoked in the context of a Launch() call and we
     // may not be in Run() yet.
-    MessageLoop::current()->PostTask(FROM_HERE, new MessageLoop::QuitTask());
+    MessageLoop::current()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
   }
 
   void ProcessControlLaunchFailed() {
     ADD_FAILURE();
     // Quit the current message.
-    MessageLoop::current()->PostTask(FROM_HERE, new MessageLoop::QuitTask());
+    MessageLoop::current()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
   }
 
-  ServiceProcessControl* process() { return process_; }
-
  private:
-  ServiceProcessControl* process_;
   base::ProcessHandle service_process_handle_;
 };
 
-#if defined(OS_WIN)
 // They way that the IPC is implemented only works on windows. This has to
 // change when we implement a different scheme for IPC.
 // Times out flakily, http://crbug.com/70076.
@@ -105,50 +103,48 @@ IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest,
   LaunchServiceProcessControl();
 
   // Make sure we are connected to the service process.
-  EXPECT_TRUE(process()->is_connected());
+  EXPECT_TRUE(ServiceProcessControl::GetInstance()->IsConnected());
   SendRequestAndWait();
 
   // And then shutdown the service process.
-  EXPECT_TRUE(process()->Shutdown());
+  EXPECT_TRUE(ServiceProcessControl::GetInstance()->Shutdown());
 }
 
-// This tests the case when a service process is launched when browser
-// starts but we try to launch it again in the remoting setup dialog.
+// This tests the case when a service process is launched when the browser
+// starts but we try to launch it again while setting up Cloud Print.
 IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, LaunchTwice) {
   // Launch the service process the first time.
   LaunchServiceProcessControl();
 
   // Make sure we are connected to the service process.
-  EXPECT_TRUE(process()->is_connected());
+  EXPECT_TRUE(ServiceProcessControl::GetInstance()->IsConnected());
   SendRequestAndWait();
 
   // Launch the service process again.
   LaunchServiceProcessControl();
-  EXPECT_TRUE(process()->is_connected());
+  EXPECT_TRUE(ServiceProcessControl::GetInstance()->IsConnected());
   SendRequestAndWait();
 
   // And then shutdown the service process.
-  EXPECT_TRUE(process()->Shutdown());
+  EXPECT_TRUE(ServiceProcessControl::GetInstance()->Shutdown());
 }
 
 static void DecrementUntilZero(int* count) {
   (*count)--;
   if (!(*count))
-    MessageLoop::current()->PostTask(FROM_HERE, new MessageLoop::QuitTask());
+    MessageLoop::current()->PostTask(FROM_HERE, MessageLoop::QuitClosure());
 }
 
 // Invoke multiple Launch calls in succession and ensure that all the tasks
 // get invoked.
-IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, MultipleLaunchTasks) {
-  ServiceProcessControl* process =
-      ServiceProcessControlManager::GetInstance()->GetProcessControl(
-          browser()->profile());
+IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest,
+                       MultipleLaunchTasks) {
+  ServiceProcessControl* process = ServiceProcessControl::GetInstance();
   int launch_count = 5;
   for (int i = 0; i < launch_count; i++) {
     // Launch the process asynchronously.
-    process->Launch(
-        NewRunnableFunction(&DecrementUntilZero, &launch_count),
-        new MessageLoop::QuitTask());
+    process->Launch(base::Bind(&DecrementUntilZero, &launch_count),
+                    MessageLoop::QuitClosure());
   }
   // Then run the message loop to keep things running.
   ui_test_utils::RunMessageLoop();
@@ -159,13 +155,11 @@ IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, MultipleLaunchTasks) {
 
 // Make sure using the same task for success and failure tasks works.
 IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, SameLaunchTask) {
-  ServiceProcessControl* process =
-      ServiceProcessControlManager::GetInstance()->GetProcessControl(
-          browser()->profile());
+  ServiceProcessControl* process = ServiceProcessControl::GetInstance();
   int launch_count = 5;
   for (int i = 0; i < launch_count; i++) {
     // Launch the process asynchronously.
-    Task * task = NewRunnableFunction(&DecrementUntilZero, &launch_count);
+    base::Closure task = base::Bind(&DecrementUntilZero, &launch_count);
     process->Launch(task, task);
   }
   // Then run the message loop to keep things running.
@@ -177,32 +171,37 @@ IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, SameLaunchTask) {
 
 // Tests whether disconnecting from the service IPC causes the service process
 // to die.
-IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, DieOnDisconnect) {
+IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest,
+                       DieOnDisconnect) {
   // Launch the service process.
   LaunchServiceProcessControl();
   // Make sure we are connected to the service process.
-  EXPECT_TRUE(process()->is_connected());
+  EXPECT_TRUE(ServiceProcessControl::GetInstance()->IsConnected());
   Disconnect();
   WaitForShutdown();
 }
 
-IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, ForceShutdown) {
+IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest,
+                       ForceShutdown) {
   // Launch the service process.
   LaunchServiceProcessControl();
   // Make sure we are connected to the service process.
-  EXPECT_TRUE(process()->is_connected());
+  EXPECT_TRUE(ServiceProcessControl::GetInstance()->IsConnected());
+  base::ProcessId service_pid;
+  EXPECT_TRUE(GetServiceProcessData(NULL, &service_pid));
+  EXPECT_NE(static_cast<base::ProcessId>(0), service_pid);
   chrome::VersionInfo version_info;
-  ForceServiceProcessShutdown(version_info.Version());
+  ForceServiceProcessShutdown(version_info.Version(), service_pid);
   WaitForShutdown();
 }
 
 IN_PROC_BROWSER_TEST_F(ServiceProcessControlBrowserTest, CheckPid) {
-  EXPECT_EQ(0, GetServiceProcessPid());
+  base::ProcessId service_pid;
+  EXPECT_FALSE(GetServiceProcessData(NULL, &service_pid));
   // Launch the service process.
   LaunchServiceProcessControl();
-  EXPECT_NE(static_cast<base::ProcessId>(0), GetServiceProcessPid());
+  EXPECT_TRUE(GetServiceProcessData(NULL, &service_pid));
+  EXPECT_NE(static_cast<base::ProcessId>(0), service_pid);
+  // Disconnect from service process.
+  ServiceProcessControl::GetInstance()->Disconnect();
 }
-
-#endif
-
-DISABLE_RUNNABLE_METHOD_REFCOUNT(ServiceProcessControlBrowserTest);
