@@ -1,11 +1,12 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/string_util.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/automation/tab_proxy.h"
+#include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
+#include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/ui/ui_test.h"
 #include "grit/generated_resources.h"
 #include "net/base/net_util.h"
@@ -79,8 +80,9 @@ class SessionHistoryTest : public UITest {
   net::TestServer test_server_;
 };
 
-#if defined(OS_WIN)
-// See http://crbug.com/61619
+#if defined(OS_WIN) || defined(OS_MACOSX)
+// See http://crbug.com/61619 on windows.
+// See http://crbug.com/102094 on mac.
 #define MAYBE_BasicBackForward FLAKY_BasicBackForward
 #else
 #define MAYBE_BasicBackForward BasicBackForward
@@ -91,7 +93,7 @@ TEST_F(SessionHistoryTest, MAYBE_BasicBackForward) {
 
   // about:blank should be loaded first.
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   ASSERT_TRUE(tab_->NavigateToURL(
       test_server_.GetURL("files/session_history/bot1.html")));
@@ -132,10 +134,10 @@ TEST_F(SessionHistoryTest, MAYBE_BasicBackForward) {
   EXPECT_EQ(L"bot1", GetTabTitle());
 
   ASSERT_TRUE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   ASSERT_TRUE(tab_->GoForward());
   EXPECT_EQ(L"bot1", GetTabTitle());
@@ -144,21 +146,18 @@ TEST_F(SessionHistoryTest, MAYBE_BasicBackForward) {
   EXPECT_EQ(L"bot3", GetTabTitle());
 }
 
-#if defined(OS_WIN)
-// See http://crbug.com/61619
-#define MAYBE_FrameBackForward FLAKY_FrameBackForward
-#else
-#define MAYBE_FrameBackForward FrameBackForward
-#endif
-
 // Test that back/forward works when navigating in subframes.
-TEST_F(SessionHistoryTest, MAYBE_FrameBackForward) {
+#if defined(OS_WIN)
+// http://crbug.com/48833
+#define FrameBackForward FLAKY_FrameBackForward
+#endif
+TEST_F(SessionHistoryTest, FrameBackForward) {
   ASSERT_TRUE(test_server_.Start());
 
   // about:blank should be loaded first.
   GURL home(homepage());
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
   EXPECT_EQ(home, GetTabURL());
 
   GURL frames(test_server_.GetURL("files/session_history/frames.html"));
@@ -185,7 +184,7 @@ TEST_F(SessionHistoryTest, MAYBE_FrameBackForward) {
   EXPECT_EQ(frames, GetTabURL());
 
   ASSERT_TRUE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
   EXPECT_EQ(home, GetTabURL());
 
   ASSERT_TRUE(tab_->GoForward());
@@ -215,15 +214,9 @@ TEST_F(SessionHistoryTest, MAYBE_FrameBackForward) {
   EXPECT_EQ(frames, GetTabURL());
 }
 
-#if defined(OS_WIN)
 // See http://crbug.com/61619
-#define MAYBE_FrameFormBackForward FLAKY_FrameFormBackForward
-#else
-#define MAYBE_FrameFormBackForward FrameFormBackForward
-#endif
-
 // Test that back/forward preserves POST data and document state in subframes.
-TEST_F(SessionHistoryTest, MAYBE_FrameFormBackForward) {
+TEST_F(SessionHistoryTest, FLAKY_FrameFormBackForward) {
   ASSERT_TRUE(test_server_.Start());
 
   // about:blank should be loaded first.
@@ -340,7 +333,7 @@ TEST_F(SessionHistoryTest, MAYBE_FragmentBackForward) {
 
   // about:blank should be loaded first.
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   GURL fragment(test_server_.GetURL("files/session_history/fragment.html"));
   ASSERT_TRUE(tab_->NavigateToURL(fragment));
@@ -411,7 +404,7 @@ TEST_F(SessionHistoryTest, FLAKY_JavascriptHistory) {
 
   // about:blank should be loaded first.
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   ASSERT_TRUE(tab_->NavigateToURL(
       test_server_.GetURL("files/session_history/bot1.html")));
@@ -445,10 +438,10 @@ TEST_F(SessionHistoryTest, FLAKY_JavascriptHistory) {
   // history is [blank, bot1, bot2, *bot3]
 
   JavascriptGo("-3");
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   JavascriptGo("1");
   EXPECT_EQ(L"bot1", GetTabTitle());
@@ -466,10 +459,10 @@ TEST_F(SessionHistoryTest, FLAKY_JavascriptHistory) {
   EXPECT_EQ(L"bot1", GetTabTitle());
 
   JavascriptGo("-1");
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   JavascriptGo("1");
   EXPECT_EQ(L"bot1", GetTabTitle());
@@ -500,7 +493,7 @@ TEST_F(SessionHistoryTest, FAILS_LocationReplace) {
 TEST_F(SessionHistoryTest, FLAKY_HistorySearchXSS) {
   // about:blank should be loaded first.
   ASSERT_FALSE(tab_->GoBack());
-  EXPECT_EQ(L"", GetTabTitle());
+  EXPECT_EQ(L"about:blank", GetTabTitle());
 
   GURL url(std::string(chrome::kChromeUIHistoryURL) +
       "#q=%3Cimg%20src%3Dx%3Ax%20onerror%3D%22document.title%3D'XSS'%22%3E");
