@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,9 +7,10 @@
 #include <limits>
 
 #include "base/file_util.h"
+#include "base/memory/scoped_vector.h"
 #include "base/metrics/histogram.h"
-#include "base/scoped_vector.h"
 #include "net/base/file_stream.h"
+#include "net/base/net_errors.h"
 
 using base::TimeTicks;
 
@@ -21,6 +22,13 @@ static const int32 kFileSignature = 0x53534E53;
 
 namespace {
 
+// The file header is the first bytes written to the file,
+// and is used to identify the file as one written by us.
+struct FileHeader {
+  int32 signature;
+  int32 version;
+};
+
 // SessionFileReader ----------------------------------------------------------
 
 // SessionFileReader is responsible for reading the set of SessionCommands that
@@ -38,7 +46,8 @@ class SessionFileReader {
         buffer_position_(0),
         available_count_(0) {
     file_.reset(new net::FileStream());
-    file_->Open(path, base::PLATFORM_FILE_OPEN | base::PLATFORM_FILE_READ);
+    if (file_util::PathExists(path))
+      file_->Open(path, base::PLATFORM_FILE_OPEN | base::PLATFORM_FILE_READ);
   }
   // Reads the contents of the file specified in the constructor, returning
   // true on success. It is up to the caller to free all SessionCommands
@@ -81,13 +90,13 @@ bool SessionFileReader::Read(BaseSessionService::SessionType type,
                              std::vector<SessionCommand*>* commands) {
   if (!file_->IsOpen())
     return false;
-  int32 header[2];
+  FileHeader header;
   int read_count;
   TimeTicks start_time = TimeTicks::Now();
   read_count = file_->ReadUntilComplete(reinterpret_cast<char*>(&header),
                                         sizeof(header));
-  if (read_count != sizeof(header) || header[0] != kFileSignature ||
-      header[1] != kFileCurrentVersion)
+  if (read_count != sizeof(header) || header.signature != kFileSignature ||
+      header.version != kFileCurrentVersion)
     return false;
 
   ScopedVector<SessionCommand> read_commands;
@@ -235,9 +244,7 @@ void SessionBackend::ReadLastSessionCommands(
     return;
   Init();
   ReadLastSessionCommandsImpl(&(request->commands));
-  request->ForwardResult(
-      BaseSessionService::InternalGetCommandsRequest::TupleType(
-          request->handle(), request));
+  request->ForwardResult(request->handle(), request);
 }
 
 bool SessionBackend::ReadLastSessionCommandsImpl(
@@ -288,9 +295,7 @@ void SessionBackend::ReadCurrentSessionCommands(
     return;
   Init();
   ReadCurrentSessionCommandsImpl(&(request->commands));
-  request->ForwardResult(
-      BaseSessionService::InternalGetCommandsRequest::TupleType(
-          request->handle(), request));
+  request->ForwardResult(request->handle(), request);
 }
 
 bool SessionBackend::ReadCurrentSessionCommandsImpl(
@@ -312,27 +317,28 @@ bool SessionBackend::AppendCommandsToFile(net::FileStream* file,
     else
       UMA_HISTOGRAM_COUNTS("SessionRestore.command_size", total_size);
     wrote = file->Write(reinterpret_cast<const char*>(&total_size),
-                        sizeof(total_size), NULL);
+                        sizeof(total_size), net::CompletionCallback());
     if (wrote != sizeof(total_size)) {
       NOTREACHED() << "error writing";
       return false;
     }
     id_type command_id = (*i)->id();
     wrote = file->Write(reinterpret_cast<char*>(&command_id),
-                        sizeof(command_id), NULL);
+                        sizeof(command_id), net::CompletionCallback());
     if (wrote != sizeof(command_id)) {
       NOTREACHED() << "error writing";
       return false;
     }
     if (content_size > 0) {
       wrote = file->Write(reinterpret_cast<char*>((*i)->contents()),
-                          content_size, NULL);
+                          content_size, net::CompletionCallback());
       if (wrote != content_size) {
         NOTREACHED() << "error writing";
         return false;
       }
     }
   }
+  file->Flush();
   return true;
 }
 
@@ -346,7 +352,8 @@ void SessionBackend::ResetFile() {
     // reopening to avoid the possibility of scanners locking the file out
     // from under us once we close it. If truncation fails, we'll try to
     // recreate.
-    if (current_session_file_->Truncate(sizeof_header()) != sizeof_header())
+    const int header_size = static_cast<int>(sizeof(FileHeader));
+    if (current_session_file_->Truncate(header_size) != header_size)
       current_session_file_.reset(NULL);
   }
   if (!current_session_file_.get())
@@ -357,17 +364,16 @@ void SessionBackend::ResetFile() {
 net::FileStream* SessionBackend::OpenAndWriteHeader(const FilePath& path) {
   DCHECK(!path.empty());
   scoped_ptr<net::FileStream> file(new net::FileStream());
-  file->Open(path, base::PLATFORM_FILE_CREATE_ALWAYS |
-             base::PLATFORM_FILE_WRITE | base::PLATFORM_FILE_EXCLUSIVE_WRITE |
-             base::PLATFORM_FILE_EXCLUSIVE_READ);
-  if (!file->IsOpen())
+  if (file->Open(path, base::PLATFORM_FILE_CREATE_ALWAYS |
+      base::PLATFORM_FILE_WRITE | base::PLATFORM_FILE_EXCLUSIVE_WRITE |
+      base::PLATFORM_FILE_EXCLUSIVE_READ) != net::OK)
     return NULL;
-  int32 header[2];
-  header[0] = kFileSignature;
-  header[1] = kFileCurrentVersion;
+  FileHeader header;
+  header.signature = kFileSignature;
+  header.version = kFileCurrentVersion;
   int wrote = file->Write(reinterpret_cast<char*>(&header),
-                          sizeof(header), NULL);
-  if (wrote != sizeof_header())
+                          sizeof(header), net::CompletionCallback());
+  if (wrote != sizeof(header))
     return NULL;
   return file.release();
 }
