@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,13 +6,59 @@
 #include "chrome/browser/defaults.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sessions/tab_restore_service.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/sessions/tab_restore_service_factory.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/browser_window.h"
-#include "chrome/common/page_transition_types.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/common/page_transition_types.h"
+
+namespace {
+
+// BrowserList::Observer implementation that waits for a browser to be
+// removed.
+class BrowserListObserverImpl : public BrowserList::Observer {
+ public:
+  BrowserListObserverImpl() : did_remove_(false), running_(false) {
+    BrowserList::AddObserver(this);
+  }
+
+  ~BrowserListObserverImpl() {
+    BrowserList::RemoveObserver(this);
+  }
+
+  // Returns when a browser has been removed.
+  void Run() {
+    running_ = true;
+    if (!did_remove_)
+      ui_test_utils::RunMessageLoop();
+  }
+
+  // BrowserList::Observer
+  virtual void OnBrowserAdded(const Browser* browser) OVERRIDE {
+  }
+  virtual void OnBrowserRemoved(const Browser* browser) OVERRIDE {
+    did_remove_ = true;
+    if (running_)
+      MessageLoop::current()->Quit();
+  }
+
+ private:
+  // Was OnBrowserRemoved invoked?
+  bool did_remove_;
+
+  // Was Run invoked?
+  bool running_;
+
+  DISALLOW_COPY_AND_ASSIGN(BrowserListObserverImpl);
+};
+
+}  // namespace
 
 typedef InProcessBrowserTest SessionRestoreTest;
 
@@ -61,10 +107,9 @@ IN_PROC_BROWSER_TEST_F(SessionRestoreTest,
   ASSERT_EQ(1, new_browser->tab_count());
 
   // And the first url should be url.
-  EXPECT_EQ(url, new_browser->GetTabContentsAt(0)->GetURL());
+  EXPECT_EQ(url, new_browser->GetWebContentsAt(0)->GetURL());
 }
 
-
 IN_PROC_BROWSER_TEST_F(SessionRestoreTest, RestoreIndividualTabFromWindow) {
   GURL url1(ui_test_utils::GetTestUrl(
       FilePath(FilePath::kCurrentDirectory),
@@ -78,13 +123,23 @@ IN_PROC_BROWSER_TEST_F(SessionRestoreTest, RestoreIndividualTabFromWindow) {
 
   // Add and navigate three tabs.
   ui_test_utils::NavigateToURL(browser(), url1);
-  browser()->AddSelectedTabWithURL(url2, PageTransition::LINK);
-  ui_test_utils::WaitForNavigationInCurrentTab(browser());
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        content::NotificationService::AllSources());
+    browser()->AddSelectedTabWithURL(url2, content::PAGE_TRANSITION_LINK);
+    observer.Wait();
+  }
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        content::NotificationService::AllSources());
+    browser()->AddSelectedTabWithURL(url3, content::PAGE_TRANSITION_LINK);
+    observer.Wait();
+  }
 
-  browser()->AddSelectedTabWithURL(url3, PageTransition::LINK);
-  ui_test_utils::WaitForNavigationInCurrentTab(browser());
-
-  TabRestoreService* service = browser()->profile()->GetTabRestoreService();
+  TabRestoreService* service =
+      TabRestoreServiceFactory::GetForProfile(browser()->profile());
   service->ClearEntries();
 
   browser()->window()->Close();
@@ -103,7 +158,7 @@ IN_PROC_BROWSER_TEST_F(SessionRestoreTest, RestoreIndividualTabFromWindow) {
     const TabRestoreService::Tab& tab = *it;
     // If this tab held url2, then restore this single tab.
     if (tab.navigations[0].virtual_url() == url2) {
-      service->RestoreEntryById(NULL, tab.id, false);
+      service->RestoreEntryById(NULL, tab.id, UNKNOWN);
       break;
     }
   }
@@ -123,7 +178,8 @@ IN_PROC_BROWSER_TEST_F(SessionRestoreTest, WindowWithOneTab) {
   // Add a single tab.
   ui_test_utils::NavigateToURL(browser(), url);
 
-  TabRestoreService* service = browser()->profile()->GetTabRestoreService();
+  TabRestoreService* service =
+      TabRestoreServiceFactory::GetForProfile(browser()->profile());
   service->ClearEntries();
   EXPECT_EQ(0U, service->entries().size());
 
@@ -137,8 +193,44 @@ IN_PROC_BROWSER_TEST_F(SessionRestoreTest, WindowWithOneTab) {
       static_cast<TabRestoreService::Tab*>(service->entries().front());
 
   // Restore the tab.
-  service->RestoreEntryById(NULL, tab->id, false);
+  service->RestoreEntryById(NULL, tab->id, UNKNOWN);
 
   // Make sure the restore was successful.
   EXPECT_EQ(0U, service->entries().size());
 }
+
+// Verifies we remember the last browser window when closing the last
+// non-incognito window while an incognito window is open.
+IN_PROC_BROWSER_TEST_F(SessionRestoreTest, IncognitotoNonIncognito) {
+  // Turn on session restore.
+  SessionStartupPref pref(SessionStartupPref::LAST);
+  SessionStartupPref::SetStartupPref(browser()->profile(), pref);
+
+  GURL url(ui_test_utils::GetTestUrl(
+      FilePath(FilePath::kCurrentDirectory),
+      FilePath(FILE_PATH_LITERAL("title1.html"))));
+
+  // Add a single tab.
+  ui_test_utils::NavigateToURL(browser(), url);
+
+  // Create a new incognito window.
+  Browser* incognito_browser = CreateIncognitoBrowser();
+  incognito_browser->AddBlankTab(true);
+  incognito_browser->window()->Show();
+
+  // Close the normal browser. After this we only have the incognito window
+  // open. We wait until the window closes as window closing is async.
+  {
+    BrowserListObserverImpl observer;
+    browser()->window()->Close();
+    observer.Run();
+  }
+
+  // Create a new window, which should trigger session restore.
+  incognito_browser->NewWindow();
+
+  // The first tab should have 'url' as its url.
+  Browser* new_browser = ui_test_utils::WaitForNewBrowser();
+  ASSERT_TRUE(new_browser);
+  EXPECT_EQ(url, new_browser->GetWebContentsAt(0)->GetURL());
+}
