@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,8 @@
 #include <string>
 #include <vector>
 
-#include "base/ref_counted.h"
+#include "base/basictypes.h"
+#include "base/memory/scoped_ptr.h"
 
 class SessionBackend;
 class SessionCommand;
@@ -23,10 +24,9 @@ class TabNavigation;
 
 class SessionServiceTestHelper {
  public:
-  explicit SessionServiceTestHelper() {}
-
-  explicit SessionServiceTestHelper(SessionService* service)
-      : service_(service) {}
+  explicit SessionServiceTestHelper();
+  explicit SessionServiceTestHelper(SessionService* service);
+  ~SessionServiceTestHelper();
 
   void RestoreSessionFromCommands(const std::vector<SessionCommand*>& commands,
                                   std::vector<SessionWindow*>* valid_windows);
@@ -62,14 +62,14 @@ class SessionServiceTestHelper {
       const std::vector<SessionWindow*>& windows,
       size_t nav_count);
 
-  void set_service(SessionService* service) { service_ = service; }
+  void set_service(SessionService* service) { service_.reset(service); }
   SessionService* ReleaseService() { return service_.release(); }
   SessionService* service() { return service_.get(); }
 
   SessionBackend* backend();
 
  private:
-  scoped_refptr<SessionService> service_;
+  scoped_ptr<SessionService> service_;
 
   DISALLOW_COPY_AND_ASSIGN(SessionServiceTestHelper);
 };
