@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,23 +8,33 @@
 #include <iterator>
 #include <map>
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/callback.h"
-#include "base/scoped_vector.h"
-#include "base/stl_util-inl.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_window.h"
+#include "base/memory/scoped_vector.h"
+#include "base/metrics/histogram.h"
+#include "base/stl_util.h"
+#include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_tab_helper.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sessions/session_service.h"
+#include "chrome/browser/sessions/session_service_factory.h"
 #include "chrome/browser/sessions/session_command.h"
 #include "chrome/browser/sessions/session_types.h"
+#include "chrome/browser/sessions/tab_restore_service_delegate.h"
 #include "chrome/browser/sessions/tab_restore_service_observer.h"
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/tabs/tab_strip_model.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/extensions/extension.h"
+#include "chrome/common/extensions/extension_constants.h"
+#include "chrome/common/url_constants.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
 
 using base::Time;
+using content::NavigationController;
+using content::NavigationEntry;
+using content::WebContents;
 
 // TimeFactory-----------------------------------------------------------------
 
@@ -50,7 +60,7 @@ TabRestoreService::Entry::~Entry() {}
 // TabRestoreService ----------------------------------------------------------
 
 // static
-const size_t TabRestoreService::kMaxEntries = 10;
+const size_t TabRestoreService::kMaxEntries = 25;
 
 // Identifier for commands written to file.
 // The ordering in the file is as follows:
@@ -149,6 +159,17 @@ void RemoveEntryByID(SessionID::id_type id,
   }
 }
 
+void RecordAppLaunch(Profile* profile, const TabRestoreService::Tab& tab) {
+  GURL url = tab.navigations.at(tab.current_navigation_index).virtual_url();
+  DCHECK(profile->GetExtensionService());
+  if (!profile->GetExtensionService()->IsInstalledApp(url))
+    return;
+
+  UMA_HISTOGRAM_ENUMERATION(extension_misc::kAppLaunchHistogram,
+                            extension_misc::APP_LAUNCH_NTP_RECENTLY_CLOSED,
+                            extension_misc::APP_LAUNCH_BUCKET_BOUNDARY);
+}
+
 }  // namespace
 
 TabRestoreService::Tab::Tab()
@@ -174,16 +195,12 @@ TabRestoreService::TabRestoreService(Profile* profile,
                          FilePath()),
       load_state_(NOT_LOADED),
       restoring_(false),
-      reached_max_(false),
       entries_to_write_(0),
       entries_written_(0),
       time_factory_(time_factory) {
 }
 
 TabRestoreService::~TabRestoreService() {
-  if (backend())
-    Save();
-
   FOR_EACH_OBSERVER(TabRestoreServiceObserver, observer_list_,
                     TabRestoreServiceDestroyed(this));
   STLDeleteElements(&entries_);
@@ -199,46 +216,45 @@ void TabRestoreService::RemoveObserver(TabRestoreServiceObserver* observer) {
   observer_list_.RemoveObserver(observer);
 }
 
-void TabRestoreService::CreateHistoricalTab(NavigationController* tab) {
+void TabRestoreService::CreateHistoricalTab(NavigationController* tab,
+                                            int index) {
   if (restoring_)
     return;
 
-  Browser* browser = Browser::GetBrowserForController(tab, NULL);
-  if (closing_browsers_.find(browser) != closing_browsers_.end())
+  TabRestoreServiceDelegate* delegate =
+      TabRestoreServiceDelegate::FindDelegateForController(tab, NULL);
+  if (closing_delegates_.find(delegate) != closing_delegates_.end())
     return;
 
   scoped_ptr<Tab> local_tab(new Tab());
-  PopulateTab(local_tab.get(), browser, tab);
+  PopulateTab(local_tab.get(), index, delegate, tab);
   if (local_tab->navigations.empty())
     return;
 
   AddEntry(local_tab.release(), true, true);
 }
 
-void TabRestoreService::BrowserClosing(Browser* browser) {
-  if (browser->type() != Browser::TYPE_NORMAL ||
-      browser->tab_count() == 0)
-    return;
-
-  closing_browsers_.insert(browser);
+void TabRestoreService::BrowserClosing(TabRestoreServiceDelegate* delegate) {
+  closing_delegates_.insert(delegate);
 
   scoped_ptr<Window> window(new Window());
-  window->selected_tab_index = browser->selected_index();
+  window->selected_tab_index = delegate->GetSelectedIndex();
   window->timestamp = TimeNow();
   // Don't use std::vector::resize() because it will push copies of an empty tab
   // into the vector, which will give all tabs in a window the same ID.
-  for (int i = 0; i < browser->tab_count(); ++i) {
+  for (int i = 0; i < delegate->GetTabCount(); ++i) {
     window->tabs.push_back(Tab());
   }
   size_t entry_index = 0;
-  for (int tab_index = 0; tab_index < browser->tab_count(); ++tab_index) {
+  for (int tab_index = 0; tab_index < delegate->GetTabCount(); ++tab_index) {
     PopulateTab(&(window->tabs[entry_index]),
-                browser,
-                &browser->GetTabContentsAt(tab_index)->controller());
+                tab_index,
+                delegate,
+                &delegate->GetWebContentsAt(tab_index)->GetController());
     if (window->tabs[entry_index].navigations.empty()) {
       window->tabs.erase(window->tabs.begin() + entry_index);
     } else {
-      window->tabs[entry_index].browser_id = browser->session_id().id();
+      window->tabs[entry_index].browser_id = delegate->GetSessionID().id();
       entry_index++;
     }
   }
@@ -255,8 +271,8 @@ void TabRestoreService::BrowserClosing(Browser* browser) {
   }
 }
 
-void TabRestoreService::BrowserClosed(Browser* browser) {
-  closing_browsers_.erase(browser);
+void TabRestoreService::BrowserClosed(TabRestoreServiceDelegate* delegate) {
+  closing_delegates_.erase(delegate);
 }
 
 void TabRestoreService::ClearEntries() {
@@ -281,16 +297,17 @@ const TabRestoreService::Entries& TabRestoreService::entries() const {
   return entries_;
 }
 
-void TabRestoreService::RestoreMostRecentEntry(Browser* browser) {
+void TabRestoreService::RestoreMostRecentEntry(
+    TabRestoreServiceDelegate* delegate) {
   if (entries_.empty())
     return;
 
-  RestoreEntryById(browser, entries_.front()->id, false);
+  RestoreEntryById(delegate, entries_.front()->id, UNKNOWN);
 }
 
-void TabRestoreService::RestoreEntryById(Browser* browser,
+void TabRestoreService::RestoreEntryById(TabRestoreServiceDelegate* delegate,
                                          SessionID::id_type id,
-                                         bool replace_existing_tab) {
+                                         WindowOpenDisposition disposition) {
   Entries::iterator i = GetEntryIteratorById(id);
   if (i == entries_.end()) {
     // Don't hoark here, we allow an invalid id.
@@ -317,39 +334,41 @@ void TabRestoreService::RestoreEntryById(Browser* browser,
     i = entries_.end();
   }
 
-  // |browser| will be NULL in cases where one isn't already available (eg,
+  // |delegate| will be NULL in cases where one isn't already available (eg,
   // when invoked on Mac OS X with no windows open). In this case, create a
   // new browser into which we restore the tabs.
   if (entry->type == TAB) {
     Tab* tab = static_cast<Tab*>(entry);
-    browser = RestoreTab(*tab, browser, replace_existing_tab);
-    browser->window()->Show();
+    delegate = RestoreTab(*tab, delegate, disposition);
+    delegate->ShowBrowserWindow();
   } else if (entry->type == WINDOW) {
-    Browser* current_browser = browser;
+    TabRestoreServiceDelegate* current_delegate = delegate;
     Window* window = static_cast<Window*>(entry);
 
     // When restoring a window, either the entire window can be restored, or a
     // single tab within it. If the entry's ID matches the one to restore, then
     // the entire window will be restored.
     if (!restoring_tab_in_window) {
-      browser = Browser::Create(profile());
+      delegate = TabRestoreServiceDelegate::Create(profile());
       for (size_t tab_i = 0; tab_i < window->tabs.size(); ++tab_i) {
         const Tab& tab = window->tabs[tab_i];
-        TabContents* restored_tab =
-            browser->AddRestoredTab(tab.navigations, browser->tab_count(),
-                                    tab.current_navigation_index,
-                                    tab.extension_app_id,
-                                    (static_cast<int>(tab_i) ==
-                                        window->selected_tab_index),
-                                    tab.pinned, tab.from_last_session,
-                                    tab.session_storage_namespace);
-        if (restored_tab)
-          restored_tab->controller().LoadIfNecessary();
+        WebContents* restored_tab =
+            delegate->AddRestoredTab(tab.navigations, delegate->GetTabCount(),
+                                     tab.current_navigation_index,
+                                     tab.extension_app_id,
+                                     static_cast<int>(tab_i) ==
+                                         window->selected_tab_index,
+                                     tab.pinned, tab.from_last_session,
+                                     tab.session_storage_namespace);
+        if (restored_tab) {
+          restored_tab->GetController().LoadIfNecessary();
+          RecordAppLaunch(profile(), tab);
+        }
       }
       // All the window's tabs had the same former browser_id.
       if (window->tabs[0].has_browser()) {
         UpdateTabBrowserIDs(window->tabs[0].browser_id,
-                            browser->session_id().id());
+                            delegate->GetSessionID().id());
       }
     } else {
       // Restore a single tab from the window. Find the tab that matches the ID
@@ -358,7 +377,7 @@ void TabRestoreService::RestoreEntryById(Browser* browser,
            tab_i != window->tabs.end(); ++tab_i) {
         const Tab& tab = *tab_i;
         if (tab.id == id) {
-          browser = RestoreTab(tab, browser, replace_existing_tab);
+          delegate = RestoreTab(tab, delegate, disposition);
           window->tabs.erase(tab_i);
           // If restoring the tab leaves the window with nothing else, delete it
           // as well.
@@ -370,21 +389,21 @@ void TabRestoreService::RestoreEntryById(Browser* browser,
             // any one is restored, it goes into the same window as the tab
             // being restored now.
             UpdateTabBrowserIDs(tab.browser_id,
-                                browser->session_id().id());
+                                delegate->GetSessionID().id());
             for (std::vector<Tab>::iterator tab_j = window->tabs.begin();
                  tab_j != window->tabs.end(); ++tab_j) {
-              (*tab_j).browser_id = browser->session_id().id();
+              (*tab_j).browser_id = delegate->GetSessionID().id();
             }
           }
           break;
         }
       }
     }
-    browser->window()->Show();
+    delegate->ShowBrowserWindow();
 
-    if (replace_existing_tab && current_browser &&
-        current_browser->GetSelectedTabContents()) {
-      current_browser->CloseTab();
+    if (disposition == CURRENT_TAB && current_delegate &&
+        current_delegate->GetSelectedWebContents()) {
+      current_delegate->CloseTab();
     }
   } else {
     NOTREACHED();
@@ -399,19 +418,22 @@ void TabRestoreService::RestoreEntryById(Browser* browser,
 }
 
 void TabRestoreService::LoadTabsFromLastSession() {
-  if (load_state_ != NOT_LOADED || reached_max_)
+  if (load_state_ != NOT_LOADED || entries_.size() == kMaxEntries)
     return;
 
   load_state_ = LOADING;
 
+  SessionService* session_service =
+      SessionServiceFactory::GetForProfile(profile());
   if (!profile()->restored_last_session() &&
       !profile()->DidLastSessionExitCleanly() &&
-      profile()->GetSessionService()) {
+      session_service) {
     // The previous session crashed and wasn't restored. Load the tabs/windows
     // that were open at the point of crash from the session service.
-    profile()->GetSessionService()->GetLastSession(
-        &load_consumer_,
-        NewCallback(this, &TabRestoreService::OnGotPreviousSession));
+    session_service->GetLastSession(
+        &crash_consumer_,
+        base::Bind(&TabRestoreService::OnGotPreviousSession,
+                   base::Unretained(this)));
   } else {
     load_state_ |= LOADED_LAST_SESSION;
   }
@@ -421,10 +443,16 @@ void TabRestoreService::LoadTabsFromLastSession() {
   // crash (the call to GetLastSession above requests those).
   ScheduleGetLastSessionCommands(
       new InternalGetCommandsRequest(
-          NewCallback(this, &TabRestoreService::OnGotLastSessionCommands)),
+          base::Bind(&TabRestoreService::OnGotLastSessionCommands,
+                     base::Unretained(this))),
       &load_consumer_);
 }
 
+void TabRestoreService::Shutdown() {
+  if (backend())
+    Save();
+}
+
 void TabRestoreService::Save() {
   int to_write_count = std::min(entries_to_write_,
                                 static_cast<int>(entries_.size()));
@@ -459,35 +487,42 @@ void TabRestoreService::Save() {
 }
 
 void TabRestoreService::PopulateTab(Tab* tab,
-                                    Browser* browser,
+                                    int index,
+                                    TabRestoreServiceDelegate* delegate,
                                     NavigationController* controller) {
-  const int pending_index = controller->pending_entry_index();
-  int entry_count = controller->entry_count();
+  const int pending_index = controller->GetPendingEntryIndex();
+  int entry_count = controller->GetEntryCount();
   if (entry_count == 0 && pending_index == 0)
     entry_count++;
   tab->navigations.resize(static_cast<int>(entry_count));
   for (int i = 0; i < entry_count; ++i) {
     NavigationEntry* entry = (i == pending_index) ?
-        controller->pending_entry() : controller->GetEntryAtIndex(i);
+        controller->GetPendingEntry() : controller->GetEntryAtIndex(i);
     tab->navigations[i].SetFromNavigationEntry(*entry);
   }
   tab->timestamp = TimeNow();
   tab->current_navigation_index = controller->GetCurrentEntryIndex();
   if (tab->current_navigation_index == -1 && entry_count > 0)
     tab->current_navigation_index = 0;
+  tab->tabstrip_index = index;
 
-  const Extension* extension = controller->tab_contents()->extension_app();
-  if (extension)
-    tab->extension_app_id = extension->id();
+  TabContentsWrapper* wrapper =
+      TabContentsWrapper::GetCurrentWrapperForContents(
+          controller->GetWebContents());
+  // wrapper is NULL in some browser tests.
+  if (wrapper) {
+    const Extension* extension =
+        wrapper->extension_tab_helper()->extension_app();
+    if (extension)
+      tab->extension_app_id = extension->id();
+  }
 
-  tab->session_storage_namespace = controller->session_storage_namespace();
+  tab->session_storage_namespace = controller->GetSessionStorageNamespace();
 
-  // Browser may be NULL during unit tests.
-  if (browser) {
-    tab->browser_id = browser->session_id().id();
-    tab->tabstrip_index =
-        browser->tabstrip_model()->GetIndexOfController(controller);
-    tab->pinned = browser->tabstrip_model()->IsTabPinned(tab->tabstrip_index);
+  // Delegate may be NULL during unit tests.
+  if (delegate) {
+    tab->browser_id = delegate->GetSessionID().id();
+    tab->pinned = delegate->IsTabPinned(tab->tabstrip_index);
   }
 }
 
@@ -497,25 +532,42 @@ void TabRestoreService::NotifyTabsChanged() {
 }
 
 void TabRestoreService::AddEntry(Entry* entry, bool notify, bool to_front) {
+  if (!FilterEntry(entry) || (entries_.size() >= kMaxEntries && !to_front)) {
+    delete entry;
+    return;
+  }
+
   if (to_front)
     entries_.push_front(entry);
   else
     entries_.push_back(entry);
+
+  PruneEntries();
+
   if (notify)
-    PruneAndNotify();
+    NotifyTabsChanged();
+
   // Start the save timer, when it fires we'll generate the commands.
   StartSaveTimer();
   entries_to_write_++;
 }
 
-void TabRestoreService::PruneAndNotify() {
-  while (entries_.size() > kMaxEntries) {
-    delete entries_.back();
-    entries_.pop_back();
-    reached_max_ = true;
+void TabRestoreService::PruneEntries() {
+  Entries new_entries;
+
+  for (TabRestoreService::Entries::const_iterator iter = entries_.begin();
+       iter != entries_.end(); ++iter) {
+    TabRestoreService::Entry* entry = *iter;
+
+    if (FilterEntry(entry) &&
+        new_entries.size() < kMaxEntries) {
+      new_entries.push_back(entry);
+    } else {
+      delete entry;
+    }
   }
 
-  NotifyTabsChanged();
+  entries_ = new_entries;
 }
 
 TabRestoreService::Entries::iterator TabRestoreService::GetEntryIteratorById(
@@ -577,7 +629,7 @@ void TabRestoreService::ScheduleCommandsForTab(const Tab& tab,
   int first_index_to_persist = selected_index;
   for (int i = selected_index - 1; i >= 0 &&
        valid_count_before_selected < max_persist_navigation_count; --i) {
-    if (ShouldTrackEntry(navigations[i])) {
+    if (ShouldTrackEntry(navigations[i].virtual_url())) {
       first_index_to_persist = i;
       valid_count_before_selected++;
     }
@@ -606,7 +658,7 @@ void TabRestoreService::ScheduleCommandsForTab(const Tab& tab,
   // Then write the navigations.
   for (int i = first_index_to_persist, wrote_count = 0;
        i < max_index && wrote_count < 2 * max_persist_navigation_count; ++i) {
-    if (ShouldTrackEntry(navigations[i])) {
+    if (ShouldTrackEntry(navigations[i].virtual_url())) {
       // Creating a NavigationEntry isn't the most efficient way to go about
       // this, but it simplifies the code and makes it less error prone as we
       // add new data to NavigationEntry.
@@ -669,7 +721,7 @@ int TabRestoreService::GetSelectedNavigationIndexToPersist(const Tab& tab) {
   // Find the first navigation to persist. We won't persist the selected
   // navigation if ShouldTrackEntry returns false.
   while (selected_index >= 0 &&
-         !ShouldTrackEntry(navigations[selected_index])) {
+         !ShouldTrackEntry(navigations[selected_index].virtual_url())) {
     selected_index--;
   }
 
@@ -679,7 +731,7 @@ int TabRestoreService::GetSelectedNavigationIndexToPersist(const Tab& tab) {
   // Couldn't find a navigation to persist going back, go forward.
   selected_index = tab.current_navigation_index + 1;
   while (selected_index < max_index &&
-         !ShouldTrackEntry(navigations[selected_index])) {
+         !ShouldTrackEntry(navigations[selected_index].virtual_url())) {
     selected_index++;
   }
 
@@ -855,51 +907,58 @@ void TabRestoreService::CreateEntriesFromCommands(
     }
   }
 
-  // If there was corruption some of the entries won't be valid. Prune any
-  // entries with no navigations.
+  // If there was corruption some of the entries won't be valid.
   ValidateAndDeleteEmptyEntries(&(entries.get()));
 
   loaded_entries->swap(entries.get());
 }
 
-Browser* TabRestoreService::RestoreTab(const Tab& tab,
-                                       Browser* browser,
-                                       bool replace_existing_tab) {
-  // |browser| will be NULL in cases where one isn't already available (eg,
-  // when invoked on Mac OS X with no windows open). In this case, create a
-  // new browser into which we restore the tabs.
-  if (replace_existing_tab && browser) {
-    browser->ReplaceRestoredTab(tab.navigations,
-                                tab.current_navigation_index,
-                                tab.from_last_session,
-                                tab.extension_app_id,
-                                tab.session_storage_namespace);
+TabRestoreServiceDelegate* TabRestoreService::RestoreTab(
+    const Tab& tab,
+    TabRestoreServiceDelegate* delegate,
+    WindowOpenDisposition disposition) {
+  if (disposition == CURRENT_TAB && delegate) {
+    delegate->ReplaceRestoredTab(tab.navigations,
+                                 tab.current_navigation_index,
+                                 tab.from_last_session,
+                                 tab.extension_app_id,
+                                 tab.session_storage_namespace);
   } else {
-    if (tab.has_browser())
-      browser = BrowserList::FindBrowserWithID(tab.browser_id);
+    // We only respsect the tab's original browser if there's no disposition.
+    if (disposition == UNKNOWN && tab.has_browser())
+      delegate = TabRestoreServiceDelegate::FindDelegateWithID(tab.browser_id);
 
     int tab_index = -1;
-    if (browser) {
+
+    // |delegate| will be NULL in cases where one isn't already available (eg,
+    // when invoked on Mac OS X with no windows open). In this case, create a
+    // new browser into which we restore the tabs.
+    if (delegate && disposition != NEW_WINDOW) {
       tab_index = tab.tabstrip_index;
     } else {
-      browser = Browser::Create(profile());
-      if (tab.has_browser()) {
-        UpdateTabBrowserIDs(tab.browser_id, browser->session_id().id());
-      }
+      delegate = TabRestoreServiceDelegate::Create(profile());
+      if (tab.has_browser())
+        UpdateTabBrowserIDs(tab.browser_id, delegate->GetSessionID().id());
     }
 
-    if (tab_index < 0 || tab_index > browser->tab_count()) {
-      tab_index = browser->tab_count();
+    // Place the tab at the end if the tab index is no longer valid or
+    // we were passed a specific disposition.
+    if (tab_index < 0 || tab_index > delegate->GetTabCount() ||
+        disposition != UNKNOWN) {
+      tab_index = delegate->GetTabCount();
     }
 
-    browser->AddRestoredTab(tab.navigations,
-                            tab_index,
-                            tab.current_navigation_index,
-                            tab.extension_app_id,
-                            true, tab.pinned, tab.from_last_session,
-                            tab.session_storage_namespace);
+    delegate->AddRestoredTab(tab.navigations,
+                             tab_index,
+                             tab.current_navigation_index,
+                             tab.extension_app_id,
+                             disposition != NEW_BACKGROUND_TAB,
+                             tab.pinned,
+                             tab.from_last_session,
+                             tab.session_storage_namespace);
   }
-  return browser;
+  RecordAppLaunch(profile(), tab);
+  return delegate;
 }
 
 
@@ -910,42 +969,91 @@ bool TabRestoreService::ValidateTab(Tab* tab) {
   tab->current_navigation_index =
       std::max(0, std::min(tab->current_navigation_index,
                            static_cast<int>(tab->navigations.size()) - 1));
+
   return true;
 }
 
+bool TabRestoreService::ValidateWindow(Window* window) {
+  window->selected_tab_index =
+      std::max(0, std::min(window->selected_tab_index,
+                           static_cast<int>(window->tabs.size() - 1)));
+
+  int i = 0;
+  for (std::vector<Tab>::iterator tab_i = window->tabs.begin();
+       tab_i != window->tabs.end();) {
+    if (!ValidateTab(&(*tab_i))) {
+      tab_i = window->tabs.erase(tab_i);
+      if (i < window->selected_tab_index)
+        window->selected_tab_index--;
+      else if (i == window->selected_tab_index)
+        window->selected_tab_index = 0;
+    } else {
+      ++tab_i;
+      ++i;
+    }
+  }
+
+  if (window->tabs.empty())
+    return false;
+
+  return true;
+}
+
+bool TabRestoreService::ValidateEntry(Entry* entry) {
+  if (entry->type == TAB)
+    return ValidateTab(static_cast<Tab*>(entry));
+
+  if (entry->type == WINDOW)
+    return ValidateWindow(static_cast<Window*>(entry));
+
+  NOTREACHED();
+  return false;
+}
+
+bool TabRestoreService::IsTabInteresting(const Tab* tab) {
+  if (tab->navigations.empty())
+    return false;
+
+  if (tab->navigations.size() > 1)
+    return true;
+
+  return tab->pinned ||
+      tab->navigations.at(0).virtual_url() !=
+          GURL(chrome::kChromeUINewTabURL);
+}
+
+bool TabRestoreService::IsWindowInteresting(const Window* window) {
+  if (window->tabs.empty())
+    return false;
+
+  if (window->tabs.size() > 1)
+    return true;
+
+  return IsTabInteresting(&window->tabs[0]);
+}
+
+bool TabRestoreService::FilterEntry(Entry* entry) {
+  if (!ValidateEntry(entry))
+    return false;
+
+  if (entry->type == TAB)
+    return IsTabInteresting(static_cast<Tab*>(entry));
+  else if (entry->type == WINDOW)
+    return IsWindowInteresting(static_cast<Window*>(entry));
+
+  NOTREACHED();
+  return false;
+}
+
 void TabRestoreService::ValidateAndDeleteEmptyEntries(
     std::vector<Entry*>* entries) {
   std::vector<Entry*> valid_entries;
   std::vector<Entry*> invalid_entries;
 
-  size_t max_valid = kMaxEntries - entries_.size();
   // Iterate from the back so that we keep the most recently closed entries.
   for (std::vector<Entry*>::reverse_iterator i = entries->rbegin();
        i != entries->rend(); ++i) {
-    bool valid_entry = false;
-    if (valid_entries.size() != max_valid) {
-      if ((*i)->type == TAB) {
-        Tab* tab = static_cast<Tab*>(*i);
-        if (ValidateTab(tab))
-          valid_entry = true;
-      } else {
-        Window* window = static_cast<Window*>(*i);
-        for (std::vector<Tab>::iterator tab_i = window->tabs.begin();
-             tab_i != window->tabs.end();) {
-          if (!ValidateTab(&(*tab_i)))
-            tab_i = window->tabs.erase(tab_i);
-          else
-            ++tab_i;
-        }
-        if (!window->tabs.empty()) {
-          window->selected_tab_index =
-              std::max(0, std::min(window->selected_tab_index,
-                                   static_cast<int>(window->tabs.size() - 1)));
-          valid_entry = true;
-        }
-      }
-    }
-    if (valid_entry)
+    if (ValidateEntry(*i))
       valid_entries.push_back(*i);
     else
       invalid_entries.push_back(*i);
@@ -1026,7 +1134,7 @@ void TabRestoreService::LoadStateChanged() {
   // We're done loading.
   load_state_ ^= LOADING;
 
-  if (staging_entries_.empty() || reached_max_) {
+  if (staging_entries_.empty() || entries_.size() >= kMaxEntries) {
     STLDeleteElements(&staging_entries_);
     return;
   }
@@ -1035,7 +1143,9 @@ void TabRestoreService::LoadStateChanged() {
     // If we add all the staged entries we'll end up with more than
     // kMaxEntries. Delete entries such that we only end up with
     // at most kMaxEntries.
-    DCHECK(entries_.size() < kMaxEntries);
+    int surplus = kMaxEntries - entries_.size();
+    CHECK_LE(0, surplus);
+    CHECK_GE(static_cast<int>(staging_entries_.size()), surplus);
     STLDeleteContainerPointers(
         staging_entries_.begin() + (kMaxEntries - entries_.size()),
         staging_entries_.end());
@@ -1059,7 +1169,8 @@ void TabRestoreService::LoadStateChanged() {
   // the front, not the end and we just added the entries to the end).
   entries_to_write_ = staging_entries_.size();
 
-  PruneAndNotify();
+  PruneEntries();
+  NotifyTabsChanged();
 }
 
 Time TabRestoreService::TimeNow() const {
