@@ -1,19 +1,28 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/renderer_host/test/test_render_view_host.h"
-#include "chrome/browser/sessions/session_types.h"
+#include "base/stringprintf.h"
+#include "base/utf_string_conversions.h"
+#include "chrome/browser/sessions/session_service_factory.h"
 #include "chrome/browser/sessions/session_service.h"
+#include "chrome/browser/sessions/session_types.h"
 #include "chrome/browser/sessions/tab_restore_service.h"
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/browser/tab_contents/test_tab_contents.h"
-#include "chrome/test/render_view_test.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/common/url_constants.h"
+#include "chrome/test/base/chrome_render_view_host_test_harness.h"
+#include "chrome/test/base/chrome_render_view_test.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/browser/tab_contents/test_tab_contents.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/test/render_view_test.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebKit.h"
 
+typedef TabRestoreService::Tab Tab;
+
+using content::NavigationEntry;
+
 // Create subclass that overrides TimeNow so that we can control the time used
 // for closed tabs and windows.
 class TabRestoreTimeFactory : public TabRestoreService::TimeFactory {
@@ -30,7 +39,7 @@ class TabRestoreTimeFactory : public TabRestoreService::TimeFactory {
   base::Time time_;
 };
 
-class TabRestoreServiceTest : public RenderViewHostTestHarness {
+class TabRestoreServiceTest : public ChromeRenderViewHostTestHarness {
  public:
   TabRestoreServiceTest() {
     url1_ = GURL("http://1");
@@ -44,16 +53,17 @@ class TabRestoreServiceTest : public RenderViewHostTestHarness {
  protected:
   // testing::Test overrides
   virtual void SetUp() {
-    RenderViewHostTestHarness::SetUp();
+    WebKit::initialize(&webkit_platform_support_);
+    ChromeRenderViewHostTestHarness::SetUp();
     time_factory_ = new TabRestoreTimeFactory();
-    service_ = new TabRestoreService(profile(), time_factory_);
-    WebKit::initialize(&webkitclient_);
+    service_.reset(new TabRestoreService(profile(), time_factory_));
   }
 
   virtual void TearDown() {
-    service_ = NULL;
+    service_->Shutdown();
+    service_.reset();
     delete time_factory_;
-    RenderViewHostTestHarness::TearDown();
+    ChromeRenderViewHostTestHarness::TearDown();
     WebKit::shutdown();
   }
 
@@ -74,26 +84,28 @@ class TabRestoreServiceTest : public RenderViewHostTestHarness {
   void RecreateService() {
     // Must set service to null first so that it is destroyed before the new
     // one is created.
-    service_ = NULL;
-    service_ = new TabRestoreService(profile(), time_factory_);
+    service_->Shutdown();
+    service_.reset();
+    service_.reset(new TabRestoreService(profile(), time_factory_));
     service_->LoadTabsFromLastSession();
   }
 
   // Adds a window with one tab and url to the profile's session service.
   // If |pinned| is true, the tab is marked as pinned in the session service.
   void AddWindowWithOneTabToSessionService(bool pinned) {
-    SessionService* session_service = profile()->GetSessionService();
+    SessionService* session_service =
+        SessionServiceFactory::GetForProfile(profile());
     SessionID tab_id;
     SessionID window_id;
-    session_service->SetWindowType(window_id, Browser::TYPE_NORMAL);
+    session_service->SetWindowType(window_id, Browser::TYPE_TABBED);
     session_service->SetTabWindow(window_id, tab_id);
     session_service->SetTabIndexInWindow(window_id, tab_id, 0);
     session_service->SetSelectedTabInWindow(window_id, 0);
     if (pinned)
       session_service->SetPinnedState(window_id, tab_id, true);
-    NavigationEntry entry;
-    entry.set_url(url1_);
-    session_service->UpdateTabNavigation(window_id, tab_id, 0, entry);
+    scoped_ptr<NavigationEntry> entry(NavigationEntry::Create());;
+    entry->SetURL(url1_);
+    session_service->UpdateTabNavigation(window_id, tab_id, 0, *entry.get());
   }
 
   // Creates a SessionService and assigns it to the Profile. The SessionService
@@ -103,7 +115,7 @@ class TabRestoreServiceTest : public RenderViewHostTestHarness {
   void CreateSessionServiceWithOneWindow(bool pinned) {
     // The profile takes ownership of this.
     SessionService* session_service = new SessionService(profile());
-    profile()->set_session_service(session_service);
+    SessionServiceFactory::SetForTestProfile(profile(), session_service);
 
     AddWindowWithOneTabToSessionService(pinned);
 
@@ -114,16 +126,17 @@ class TabRestoreServiceTest : public RenderViewHostTestHarness {
   GURL url1_;
   GURL url2_;
   GURL url3_;
-  scoped_refptr<TabRestoreService> service_;
+  scoped_ptr<TabRestoreService> service_;
   TabRestoreTimeFactory* time_factory_;
-  RenderViewTest::RendererWebKitClientImplNoSandbox webkitclient_;
+  content::RenderViewTest::RendererWebKitPlatformSupportImplNoSandbox
+      webkit_platform_support_;
 };
 
 TEST_F(TabRestoreServiceTest, Basic) {
   AddThreeNavigations();
 
   // Have the service record the tab.
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   // Make sure an entry was created.
   ASSERT_EQ(1U, service_->entries().size());
@@ -131,7 +144,7 @@ TEST_F(TabRestoreServiceTest, Basic) {
   // Make sure the entry matches.
   TabRestoreService::Entry* entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  TabRestoreService::Tab* tab = static_cast<TabRestoreService::Tab*>(entry);
+  Tab* tab = static_cast<Tab*>(entry);
   EXPECT_FALSE(tab->pinned);
   EXPECT_TRUE(tab->extension_app_id.empty());
   ASSERT_EQ(3U, tab->navigations.size());
@@ -145,7 +158,7 @@ TEST_F(TabRestoreServiceTest, Basic) {
   NavigateToIndex(1);
 
   // And check again.
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   // There should be two entries now.
   ASSERT_EQ(2U, service_->entries().size());
@@ -153,7 +166,7 @@ TEST_F(TabRestoreServiceTest, Basic) {
   // Make sure the entry matches
   entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  tab = static_cast<TabRestoreService::Tab*>(entry);
+  tab = static_cast<Tab*>(entry);
   EXPECT_FALSE(tab->pinned);
   ASSERT_EQ(3U, tab->navigations.size());
   EXPECT_EQ(url1_, tab->navigations[0].virtual_url());
@@ -167,7 +180,7 @@ TEST_F(TabRestoreServiceTest, Basic) {
 // Make sure TabRestoreService doesn't create an entry for a tab with no
 // navigations.
 TEST_F(TabRestoreServiceTest, DontCreateEmptyTab) {
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
   EXPECT_TRUE(service_->entries().empty());
 }
 
@@ -176,7 +189,7 @@ TEST_F(TabRestoreServiceTest, Restore) {
   AddThreeNavigations();
 
   // Have the service record the tab.
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   // Recreate the service and have it load the tabs.
   RecreateService();
@@ -187,7 +200,7 @@ TEST_F(TabRestoreServiceTest, Restore) {
   // And verify the entry.
   TabRestoreService::Entry* entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  TabRestoreService::Tab* tab = static_cast<TabRestoreService::Tab*>(entry);
+  Tab* tab = static_cast<Tab*>(entry);
   EXPECT_FALSE(tab->pinned);
   ASSERT_EQ(3U, tab->navigations.size());
   EXPECT_TRUE(url1_ == tab->navigations[0].virtual_url());
@@ -203,7 +216,7 @@ TEST_F(TabRestoreServiceTest, RestorePinnedAndApp) {
   AddThreeNavigations();
 
   // Have the service record the tab.
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   // One entry should be created.
   ASSERT_EQ(1U, service_->entries().size());
@@ -212,7 +225,7 @@ TEST_F(TabRestoreServiceTest, RestorePinnedAndApp) {
   // these tests.
   TabRestoreService::Entry* entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  TabRestoreService::Tab* tab = static_cast<TabRestoreService::Tab*>(entry);
+  Tab* tab = static_cast<Tab*>(entry);
   tab->pinned = true;
   const std::string extension_app_id("test");
   tab->extension_app_id = extension_app_id;
@@ -226,7 +239,7 @@ TEST_F(TabRestoreServiceTest, RestorePinnedAndApp) {
   // And verify the entry.
   entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  tab = static_cast<TabRestoreService::Tab*>(entry);
+  tab = static_cast<Tab*>(entry);
   EXPECT_TRUE(tab->pinned);
   ASSERT_EQ(3U, tab->navigations.size());
   EXPECT_TRUE(url1_ == tab->navigations[0].virtual_url());
@@ -239,12 +252,12 @@ TEST_F(TabRestoreServiceTest, RestorePinnedAndApp) {
 // Make sure we persist entries to disk that have post data.
 TEST_F(TabRestoreServiceTest, DontPersistPostData) {
   AddThreeNavigations();
-  controller().GetEntryAtIndex(0)->set_has_post_data(true);
-  controller().GetEntryAtIndex(1)->set_has_post_data(true);
-  controller().GetEntryAtIndex(2)->set_has_post_data(true);
+  controller().GetEntryAtIndex(0)->SetHasPostData(true);
+  controller().GetEntryAtIndex(1)->SetHasPostData(true);
+  controller().GetEntryAtIndex(2)->SetHasPostData(true);
 
   // Have the service record the tab.
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
   ASSERT_EQ(1U, service_->entries().size());
 
   // Recreate the service and have it load the tabs.
@@ -256,8 +269,8 @@ TEST_F(TabRestoreServiceTest, DontPersistPostData) {
   const TabRestoreService::Entry* restored_entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, restored_entry->type);
 
-  const TabRestoreService::Tab* restored_tab =
-      static_cast<const TabRestoreService::Tab*>(restored_entry);
+  const Tab* restored_tab =
+      static_cast<const Tab*>(restored_entry);
   // There should be 3 navs.
   ASSERT_EQ(3U, restored_tab->navigations.size());
   EXPECT_EQ(time_factory_->TimeNow().ToInternalValue(),
@@ -271,7 +284,7 @@ TEST_F(TabRestoreServiceTest, DontLoadTwice) {
   AddThreeNavigations();
 
   // Have the service record the tab.
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
   ASSERT_EQ(1U, service_->entries().size());
 
   // Recreate the service and have it load the tabs.
@@ -287,7 +300,8 @@ TEST_F(TabRestoreServiceTest, DontLoadTwice) {
 TEST_F(TabRestoreServiceTest, LoadPreviousSession) {
   CreateSessionServiceWithOneWindow(false);
 
-  profile()->GetSessionService()->MoveCurrentSessionToLastSession();
+  SessionServiceFactory::GetForProfile(profile())->
+      MoveCurrentSessionToLastSession();
 
   service_->LoadTabsFromLastSession();
 
@@ -310,7 +324,8 @@ TEST_F(TabRestoreServiceTest, LoadPreviousSession) {
 TEST_F(TabRestoreServiceTest, DontLoadAfterRestore) {
   CreateSessionServiceWithOneWindow(false);
 
-  profile()->GetSessionService()->MoveCurrentSessionToLastSession();
+  SessionServiceFactory::GetForProfile(profile())->
+      MoveCurrentSessionToLastSession();
 
   profile()->set_restored_last_session(true);
 
@@ -324,7 +339,8 @@ TEST_F(TabRestoreServiceTest, DontLoadAfterRestore) {
 TEST_F(TabRestoreServiceTest, DontLoadAfterCleanExit) {
   CreateSessionServiceWithOneWindow(false);
 
-  profile()->GetSessionService()->MoveCurrentSessionToLastSession();
+  SessionServiceFactory::GetForProfile(profile())->
+      MoveCurrentSessionToLastSession();
 
   profile()->set_last_session_exited_cleanly(true);
 
@@ -336,11 +352,12 @@ TEST_F(TabRestoreServiceTest, DontLoadAfterCleanExit) {
 TEST_F(TabRestoreServiceTest, LoadPreviousSessionAndTabs) {
   CreateSessionServiceWithOneWindow(false);
 
-  profile()->GetSessionService()->MoveCurrentSessionToLastSession();
+  SessionServiceFactory::GetForProfile(profile())->
+      MoveCurrentSessionToLastSession();
 
   AddThreeNavigations();
 
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   RecreateService();
 
@@ -363,7 +380,7 @@ TEST_F(TabRestoreServiceTest, LoadPreviousSessionAndTabs) {
   // Then the closed tab.
   entry = *(++service_->entries().begin());
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  TabRestoreService::Tab* tab = static_cast<TabRestoreService::Tab*>(entry);
+  Tab* tab = static_cast<Tab*>(entry);
   ASSERT_FALSE(tab->pinned);
   ASSERT_EQ(3U, tab->navigations.size());
   EXPECT_EQ(2, tab->current_navigation_index);
@@ -378,11 +395,12 @@ TEST_F(TabRestoreServiceTest, LoadPreviousSessionAndTabs) {
 TEST_F(TabRestoreServiceTest, LoadPreviousSessionAndTabsPinned) {
   CreateSessionServiceWithOneWindow(true);
 
-  profile()->GetSessionService()->MoveCurrentSessionToLastSession();
+  SessionServiceFactory::GetForProfile(profile())->
+      MoveCurrentSessionToLastSession();
 
   AddThreeNavigations();
 
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   RecreateService();
 
@@ -404,7 +422,7 @@ TEST_F(TabRestoreServiceTest, LoadPreviousSessionAndTabsPinned) {
   // Then the closed tab.
   entry = *(++service_->entries().begin());
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  TabRestoreService::Tab* tab = static_cast<TabRestoreService::Tab*>(entry);
+  Tab* tab = static_cast<Tab*>(entry);
   ASSERT_FALSE(tab->pinned);
   ASSERT_EQ(3U, tab->navigations.size());
   EXPECT_EQ(2, tab->current_navigation_index);
@@ -421,11 +439,12 @@ TEST_F(TabRestoreServiceTest, ManyWindowsInSessionService) {
   for (size_t i = 0; i < TabRestoreService::kMaxEntries; ++i)
     AddWindowWithOneTabToSessionService(false);
 
-  profile()->GetSessionService()->MoveCurrentSessionToLastSession();
+  SessionServiceFactory::GetForProfile(profile())->
+      MoveCurrentSessionToLastSession();
 
   AddThreeNavigations();
 
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   RecreateService();
 
@@ -454,7 +473,7 @@ TEST_F(TabRestoreServiceTest, TimestampSurvivesRestore) {
   AddThreeNavigations();
 
   // Have the service record the tab.
-  service_->CreateHistoricalTab(&controller());
+  service_->CreateHistoricalTab(&controller(), -1);
 
   // Make sure an entry was created.
   ASSERT_EQ(1U, service_->entries().size());
@@ -462,7 +481,7 @@ TEST_F(TabRestoreServiceTest, TimestampSurvivesRestore) {
   // Make sure the entry matches.
   TabRestoreService::Entry* entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, entry->type);
-  TabRestoreService::Tab* tab = static_cast<TabRestoreService::Tab*>(entry);
+  Tab* tab = static_cast<Tab*>(entry);
   tab->timestamp = tab_timestamp;
 
   // Set this, otherwise previous session won't be loaded.
@@ -476,8 +495,115 @@ TEST_F(TabRestoreServiceTest, TimestampSurvivesRestore) {
   // And verify the entry.
   TabRestoreService::Entry* restored_entry = service_->entries().front();
   ASSERT_EQ(TabRestoreService::TAB, restored_entry->type);
-  TabRestoreService::Tab* restored_tab =
-      static_cast<TabRestoreService::Tab*>(restored_entry);
+  Tab* restored_tab =
+      static_cast<Tab*>(restored_entry);
   EXPECT_EQ(tab_timestamp.ToInternalValue(),
             restored_tab->timestamp.ToInternalValue());
 }
+
+TEST_F(TabRestoreServiceTest, PruneEntries) {
+  service_->ClearEntries();
+  ASSERT_TRUE(service_->entries().empty());
+
+  const size_t max_entries = TabRestoreService::kMaxEntries;
+  for (size_t i = 0; i < max_entries + 5; i++) {
+    TabNavigation navigation;
+    navigation.set_virtual_url(GURL(StringPrintf("http://%d",
+                                                 static_cast<int>(i))));
+    navigation.set_title(ASCIIToUTF16(StringPrintf("%d", static_cast<int>(i))));
+
+    Tab* tab = new Tab();
+    tab->navigations.push_back(navigation);
+    tab->current_navigation_index = 0;
+
+    service_->entries_.push_back(tab);
+  }
+
+  // Only keep kMaxEntries around.
+  EXPECT_EQ(max_entries + 5, service_->entries_.size());
+  service_->PruneEntries();
+  EXPECT_EQ(max_entries, service_->entries_.size());
+  // Pruning again does nothing.
+  service_->PruneEntries();
+  EXPECT_EQ(max_entries, service_->entries_.size());
+
+  // Prune older first.
+  TabNavigation navigation;
+  navigation.set_virtual_url(GURL("http://recent"));
+  navigation.set_title(ASCIIToUTF16("Most recent"));
+  Tab* tab = new Tab();
+  tab->navigations.push_back(navigation);
+  tab->current_navigation_index = 0;
+  service_->entries_.push_front(tab);
+  EXPECT_EQ(max_entries + 1, service_->entries_.size());
+  service_->PruneEntries();
+  EXPECT_EQ(max_entries, service_->entries_.size());
+  EXPECT_EQ(GURL("http://recent"),
+      static_cast<Tab*>(service_->entries_.front())->
+          navigations[0].virtual_url());
+
+  // Ignore NTPs.
+  navigation.set_virtual_url(GURL(chrome::kChromeUINewTabURL));
+  navigation.set_title(ASCIIToUTF16("New tab"));
+
+  tab = new Tab();
+  tab->navigations.push_back(navigation);
+  tab->current_navigation_index = 0;
+  service_->entries_.push_front(tab);
+
+  EXPECT_EQ(max_entries + 1, service_->entries_.size());
+  service_->PruneEntries();
+  EXPECT_EQ(max_entries, service_->entries_.size());
+  EXPECT_EQ(GURL("http://recent"),
+      static_cast<Tab*>(service_->entries_.front())->
+          navigations[0].virtual_url());
+
+  // Don't prune pinned NTPs.
+  tab = new Tab();
+  tab->pinned = true;
+  tab->current_navigation_index = 0;
+  tab->navigations.push_back(navigation);
+  service_->entries_.push_front(tab);
+  EXPECT_EQ(max_entries + 1, service_->entries_.size());
+  service_->PruneEntries();
+  EXPECT_EQ(max_entries, service_->entries_.size());
+  EXPECT_EQ(GURL(chrome::kChromeUINewTabURL),
+      static_cast<Tab*>(service_->entries_.front())->
+          navigations[0].virtual_url());
+
+  // Don't prune NTPs that have multiple navigations.
+  // (Erase the last NTP first.)
+  service_->entries_.erase(service_->entries_.begin());
+  tab = new Tab();
+  tab->current_navigation_index = 1;
+  tab->navigations.push_back(navigation);
+  tab->navigations.push_back(navigation);
+  service_->entries_.push_front(tab);
+  EXPECT_EQ(max_entries, service_->entries_.size());
+  service_->PruneEntries();
+  EXPECT_EQ(max_entries, service_->entries_.size());
+  EXPECT_EQ(GURL(chrome::kChromeUINewTabURL),
+      static_cast<Tab*>(service_->entries_.front())->
+          navigations[1].virtual_url());
+}
+
+// Regression test for crbug.com/106082
+TEST_F(TabRestoreServiceTest, PruneIsCalled) {
+  CreateSessionServiceWithOneWindow(false);
+
+  SessionServiceFactory::GetForProfile(profile())->
+      MoveCurrentSessionToLastSession();
+
+  profile()->set_restored_last_session(true);
+
+  const size_t max_entries = TabRestoreService::kMaxEntries;
+  for (size_t i = 0; i < max_entries + 5; i++) {
+    NavigateAndCommit(GURL(StringPrintf("http://%d", static_cast<int>(i))));
+    service_->CreateHistoricalTab(&controller(), -1);
+  }
+
+  EXPECT_EQ(max_entries, service_->entries_.size());
+  // This should not crash.
+  service_->LoadTabsFromLastSession();
+  EXPECT_EQ(max_entries, service_->entries_.size());
+}
