@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,24 +10,27 @@
 #include "base/file_util.h"
 #include "base/message_loop.h"
 #include "base/scoped_temp_dir.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_thread.h"
+#include "chrome/browser/web_applications/web_app.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths_internal.h"
+#include "content/test/test_browser_thread.h"
 #include "googleurl/src/gurl.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 #if defined(OS_WIN)
 #include "chrome/installer/util/browser_distribution.h"
-#elif defined(OS_LINUX)
+#elif defined(OS_POSIX) && !defined(OS_MACOSX)
 #include "base/environment.h"
-#endif  // defined(OS_LINUX)
+#endif
 
 #define FPL FILE_PATH_LITERAL
 
-#if defined(OS_LINUX)
+using content::BrowserThread;
+
+#if defined(OS_POSIX) && !defined(OS_MACOSX)
 namespace {
 
 // Provides mock environment variables values based on a stored map.
@@ -77,7 +80,7 @@ TEST(ShellIntegrationTest, GetDesktopShortcutTemplate) {
   const char kTestData2[] = "a different testing string";
 
   MessageLoop message_loop;
-  BrowserThread file_thread(BrowserThread::FILE, &message_loop);
+  content::TestBrowserThread file_thread(BrowserThread::FILE, &message_loop);
 
   {
     ScopedTempDir temp_dir;
@@ -184,18 +187,29 @@ TEST(ShellIntegrationTest, GetDesktopFileContents) {
       "Icon=/opt/google/chrome/product_logo_48.png\n"
       "Type=Application\n"
       "Categories=Application;Network;WebBrowser;\n"
-      "MimeType=text/html;text/xml;application/xhtml_xml;\n",
+      "MimeType=text/html;text/xml;application/xhtml_xml;\n"
+      "X-Ayatana-Desktop-Shortcuts=NewWindow;\n"
+      "\n"
+      "[NewWindow Shortcut Group]\n"
+      "Name=Open New Window\n"
+      "Exec=/opt/google/chrome/google-chrome\n"
+      "TargetEnvironment=Unity\n",
 
       "#!/usr/bin/env xdg-open\n"
       "[Desktop Entry]\n"
       "Version=1.0\n"
       "Encoding=UTF-8\n"
       "Name=GMail\n"
-      "Exec=/opt/google/chrome/google-chrome --app=\"http://gmail.com/\"\n"
+      "Exec=/opt/google/chrome/google-chrome --app=http://gmail.com/\n"
       "Terminal=false\n"
       "Icon=chrome-http__gmail.com\n"
       "Type=Application\n"
       "Categories=Application;Network;WebBrowser;\n"
+#if !defined(USE_AURA)
+      // Aura Chrome creates browser window in a single X11 window, so
+      // WMClass does not matter.
+      "StartupWMClass=gmail.com\n"
+#endif
     },
 
     // Make sure we don't insert duplicate shebangs.
@@ -204,12 +218,20 @@ TEST(ShellIntegrationTest, GetDesktopFileContents) {
       "chrome-http__gmail.com",
 
       "#!/some/shebang\n"
+      "[Desktop Entry]\n"
       "Name=Google Chrome\n"
       "Exec=/opt/google/chrome/google-chrome %U\n",
 
       "#!/usr/bin/env xdg-open\n"
+      "[Desktop Entry]\n"
       "Name=GMail\n"
-      "Exec=/opt/google/chrome/google-chrome --app=\"http://gmail.com/\"\n"
+      "Exec=/opt/google/chrome/google-chrome --app=http://gmail.com/\n"
+      "Icon=chrome-http__gmail.com\n"
+#if !defined(USE_AURA)
+      // Aura Chrome creates browser window in a single X11 window, so
+      // WMClass does not matter.
+      "StartupWMClass=gmail.com\n"
+#endif
     },
 
     // Make sure i18n-ed comments are removed.
@@ -217,13 +239,21 @@ TEST(ShellIntegrationTest, GetDesktopFileContents) {
       "GMail",
       "chrome-http__gmail.com",
 
+      "[Desktop Entry]\n"
       "Name=Google Chrome\n"
       "Exec=/opt/google/chrome/google-chrome %U\n"
       "Comment[pl]=Jakis komentarz.\n",
 
       "#!/usr/bin/env xdg-open\n"
+      "[Desktop Entry]\n"
       "Name=GMail\n"
-      "Exec=/opt/google/chrome/google-chrome --app=\"http://gmail.com/\"\n"
+      "Exec=/opt/google/chrome/google-chrome --app=http://gmail.com/\n"
+      "Icon=chrome-http__gmail.com\n"
+#if !defined(USE_AURA)
+      // Aura Chrome creates browser window in a single X11 window, so
+      // WMClass does not matter.
+      "StartupWMClass=gmail.com\n"
+#endif
     },
 
     // Make sure that empty icons are replaced by the chrome icon.
@@ -231,15 +261,22 @@ TEST(ShellIntegrationTest, GetDesktopFileContents) {
       "GMail",
       "",
 
+      "[Desktop Entry]\n"
       "Name=Google Chrome\n"
       "Exec=/opt/google/chrome/google-chrome %U\n"
       "Comment[pl]=Jakis komentarz.\n"
       "Icon=/opt/google/chrome/product_logo_48.png\n",
 
       "#!/usr/bin/env xdg-open\n"
+      "[Desktop Entry]\n"
       "Name=GMail\n"
-      "Exec=/opt/google/chrome/google-chrome --app=\"http://gmail.com/\"\n"
+      "Exec=/opt/google/chrome/google-chrome --app=http://gmail.com/\n"
       "Icon=/opt/google/chrome/product_logo_48.png\n"
+#if !defined(USE_AURA)
+      // Aura Chrome creates browser window in a single X11 window, so
+      // WMClass does not matter.
+      "StartupWMClass=gmail.com\n"
+#endif
     },
 
     // Now we're starting to be more evil...
@@ -247,49 +284,81 @@ TEST(ShellIntegrationTest, GetDesktopFileContents) {
       "Ownz0red\nExec=rm -rf /",
       "chrome-http__evil.com_evil",
 
+      "[Desktop Entry]\n"
       "Name=Google Chrome\n"
       "Exec=/opt/google/chrome/google-chrome %U\n",
 
       "#!/usr/bin/env xdg-open\n"
+      "[Desktop Entry]\n"
       "Name=http://evil.com/evil%20--join-the-b0tnet\n"
       "Exec=/opt/google/chrome/google-chrome "
-      "--app=\"http://evil.com/evil%20--join-the-b0tnet\"\n"
+      "--app=http://evil.com/evil%20--join-the-b0tnet\n"
+      "Icon=chrome-http__evil.com_evil\n"
+#if !defined(USE_AURA)
+      // Aura Chrome creates browser window in a single X11 window, so
+      // WMClass does not matter.
+      "StartupWMClass=evil.com__evil%20--join-the-b0tnet\n"
+#endif
     },
     { "http://evil.com/evil; rm -rf /; \"; rm -rf $HOME >ownz0red",
       "Innocent Title",
       "chrome-http__evil.com_evil",
 
+      "[Desktop Entry]\n"
       "Name=Google Chrome\n"
       "Exec=/opt/google/chrome/google-chrome %U\n",
 
       "#!/usr/bin/env xdg-open\n"
+      "[Desktop Entry]\n"
       "Name=Innocent Title\n"
       "Exec=/opt/google/chrome/google-chrome "
-      "--app=\"http://evil.com/evil%3B%20rm%20-rf%20/%3B%20%22%3B%20rm%20"
-      "-rf%20%24HOME%20%3Eownz0red\"\n"
+      "\"--app=http://evil.com/evil;%20rm%20-rf%20/;%20%22;%20rm%20"
+      // Note: $ is escaped as \$ within an arg to Exec, and then
+      // the \ is escaped as \\ as all strings in a Desktop file should
+      // be; finally, \\ becomes \\\\ when represented in a C++ string!
+      "-rf%20\\\\$HOME%20%3Eownz0red\"\n"
+      "Icon=chrome-http__evil.com_evil\n"
+#if !defined(USE_AURA)
+      // Aura Chrome creates browser window in a single X11 window, so
+      // WMClass does not matter.
+      "StartupWMClass=evil.com__evil;%20rm%20-rf%20_;%20%22;%20"
+      "rm%20-rf%20$HOME%20%3Eownz0red\n"
+#endif
     },
     { "http://evil.com/evil | cat `echo ownz0red` >/dev/null",
       "Innocent Title",
       "chrome-http__evil.com_evil",
 
+      "[Desktop Entry]\n"
       "Name=Google Chrome\n"
       "Exec=/opt/google/chrome/google-chrome %U\n",
 
       "#!/usr/bin/env xdg-open\n"
+      "[Desktop Entry]\n"
       "Name=Innocent Title\n"
       "Exec=/opt/google/chrome/google-chrome "
-      "--app=\"http://evil.com/evil%20%7C%20cat%20%60echo%20ownz0red"
-      "%60%20%3E/dev/null\"\n"
+      "--app=http://evil.com/evil%20%7C%20cat%20%60echo%20ownz0red"
+      "%60%20%3E/dev/null\n"
+      "Icon=chrome-http__evil.com_evil\n"
+#if !defined(USE_AURA)
+      // Aura Chrome creates browser window in a single X11 window, so
+      // WMClass does not matter.
+      "StartupWMClass=evil.com__evil%20%7C%20cat%20%60echo%20ownz0red"
+      "%60%20%3E_dev_null\n"
+#endif
     },
   };
   for (size_t i = 0; i < ARRAYSIZE_UNSAFE(test_cases); i++) {
-    EXPECT_EQ(test_cases[i].expected_output,
-              ShellIntegration::GetDesktopFileContents(
-                  test_cases[i].template_contents,
-                  GURL(test_cases[i].url),
-                  EmptyString16(),
-                  ASCIIToUTF16(test_cases[i].title),
-                  test_cases[i].icon_name));
+    SCOPED_TRACE(i);
+    EXPECT_EQ(
+        test_cases[i].expected_output,
+        ShellIntegration::GetDesktopFileContents(
+            test_cases[i].template_contents,
+            web_app::GenerateApplicationNameFromURL(GURL(test_cases[i].url)),
+            GURL(test_cases[i].url),
+            "",
+            ASCIIToUTF16(test_cases[i].title),
+            test_cases[i].icon_name));
   }
 }
 #elif defined(OS_WIN)
@@ -303,7 +372,7 @@ TEST(ShellIntegrationTest, GetChromiumAppIdTest) {
   FilePath default_user_data_dir;
   chrome::GetDefaultUserDataDirectory(&default_user_data_dir);
   FilePath default_profile_path =
-      default_user_data_dir.Append(chrome::kNotSignedInProfile);
+      default_user_data_dir.AppendASCII(chrome::kInitialProfile);
   EXPECT_EQ(BrowserDistribution::GetDistribution()->GetBrowserAppId(),
             ShellIntegration::GetChromiumAppId(default_profile_path));
 
