@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,15 +8,21 @@
 
 #include <vector>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/string16.h"
-#include "base/task.h"
+
+class SkBitmap;
+class SkCanvas;
+
+namespace content {
+class WebContents;
+}
 
 namespace gfx {
+class Canvas;
 class Rect;
 }
-class SkBitmap;
-class TabContents;
 
 // SpeechInputBubble displays a popup info bubble during speech recognition,
 // points to the html element which requested speech input and shows recognition
@@ -50,22 +56,23 @@ class SpeechInputBubble {
 
   // Factory method to create new instances.
   // Creates the bubble, call |Show| to display it on screen.
-  // |tab_contents| is the TabContents hosting the page.
+  // |web_contents| is the WebContents hosting the page.
   // |element_rect| is the display bounds of the html element requesting speech
   // input (in page coordinates).
-  static SpeechInputBubble* Create(TabContents* tab_contents,
+  static SpeechInputBubble* Create(content::WebContents* web_contents,
                                    Delegate* delegate,
                                    const gfx::Rect& element_rect);
 
   // This is implemented by platform specific code to create the underlying
   // bubble window. Not to be called directly by users of this class.
-  static SpeechInputBubble* CreateNativeBubble(TabContents* tab_contents,
-                                               Delegate* delegate,
-                                               const gfx::Rect& element_rect);
+  static SpeechInputBubble* CreateNativeBubble(
+      content::WebContents* web_contents,
+      Delegate* delegate,
+      const gfx::Rect& element_rect);
 
   // |Create| uses the currently registered FactoryMethod to create the
   // SpeechInputBubble instances. FactoryMethod is intended for testing.
-  typedef SpeechInputBubble* (*FactoryMethod)(TabContents*,
+  typedef SpeechInputBubble* (*FactoryMethod)(content::WebContents*,
                                               Delegate*,
                                               const gfx::Rect&);
   // Sets the factory used by the static method Create. SpeechInputBubble does
@@ -77,6 +84,10 @@ class SpeechInputBubble {
 
   virtual ~SpeechInputBubble() {}
 
+  // Indicates to the user that audio hardware is initializing. If the bubble is
+  // hidden, |Show| must be called to make it appear on screen.
+  virtual void SetWarmUpMode() = 0;
+
   // Indicates to the user that audio recording is in progress. If the bubble is
   // hidden, |Show| must be called to make it appear on screen.
   virtual void SetRecordingMode() = 0;
@@ -96,11 +107,11 @@ class SpeechInputBubble {
   // |Delegate::InfoBubbleFocusChanged| as well.
   virtual void Hide() = 0;
 
-  // Updates the current captured audio volume displayed on screen.
-  virtual void SetInputVolume(float volume) = 0;
+  // Updates and draws the current captured audio volume displayed on screen.
+  virtual void SetInputVolume(float volume, float noise_volume) = 0;
 
-  // Returns the TabContents for which this bubble gets displayed.
-  virtual TabContents* tab_contents() = 0;
+  // Returns the WebContents for which this bubble gets displayed.
+  virtual content::WebContents* web_contents() = 0;
 
   // The horizontal distance between the start of the html widget and the speech
   // bubble's arrow.
@@ -117,29 +128,29 @@ class SpeechInputBubbleBase : public SpeechInputBubble {
   // The current display mode of the bubble, useful only for the platform
   // specific implementation.
   enum DisplayMode {
+    DISPLAY_MODE_WARM_UP,
     DISPLAY_MODE_RECORDING,
     DISPLAY_MODE_RECOGNIZING,
     DISPLAY_MODE_MESSAGE
   };
 
-  explicit SpeechInputBubbleBase(TabContents* tab_contents);
+  explicit SpeechInputBubbleBase(content::WebContents* web_contents);
   virtual ~SpeechInputBubbleBase();
 
   // SpeechInputBubble methods
-  virtual void SetRecordingMode();
-  virtual void SetRecognizingMode();
-  virtual void SetMessage(const string16& text);
-  virtual void SetInputVolume(float volume);
-  virtual TabContents* tab_contents() { return tab_contents_; }
+  virtual void SetWarmUpMode() OVERRIDE;
+  virtual void SetRecordingMode() OVERRIDE;
+  virtual void SetRecognizingMode() OVERRIDE;
+  virtual void SetMessage(const string16& text) OVERRIDE;
+  virtual void SetInputVolume(float volume, float noise_volume) OVERRIDE;
+  virtual content::WebContents* web_contents() OVERRIDE;
 
  protected:
   // Updates the platform specific UI layout for the current display mode.
   virtual void UpdateLayout() = 0;
 
-  // Sets the given image as the image to display in the speech bubble.
-  // TODO(satish): Make the SetRecognizingMode call use this to show an
-  // animation while waiting for results.
-  virtual void SetImage(const SkBitmap& image) = 0;
+  // Overridden by subclasses to copy |icon_image()| to the screen.
+  virtual void UpdateImage() = 0;
 
   DisplayMode display_mode() {
     return display_mode_;
@@ -149,13 +160,22 @@ class SpeechInputBubbleBase : public SpeechInputBubble {
     return message_text_;
   }
 
+  SkBitmap icon_image();
+
  private:
   void DoRecognizingAnimationStep();
+  void DoWarmingUpAnimationStep();
+  void SetImage(const SkBitmap& image);
+
+  void DrawVolumeOverlay(SkCanvas* canvas,
+                         const SkBitmap& bitmap,
+                         float volume);
 
   // Task factory used for animation timer.
-  ScopedRunnableMethodFactory<SpeechInputBubbleBase> task_factory_;
+  base::WeakPtrFactory<SpeechInputBubbleBase> weak_factory_;
   int animation_step_;  // Current index/step of the animation.
   std::vector<SkBitmap> animation_frames_;
+  std::vector<SkBitmap> warming_up_frames_;
 
   DisplayMode display_mode_;
   string16 message_text_;  // Text displayed in DISPLAY_MODE_MESSAGE
@@ -163,14 +183,10 @@ class SpeechInputBubbleBase : public SpeechInputBubble {
   scoped_ptr<SkBitmap> mic_image_;
   // A temporary buffer image used in creating the above mic image.
   scoped_ptr<SkBitmap> buffer_image_;
-  // TabContents in which this this bubble gets displayed.
-  TabContents* tab_contents_;
-
-  static SkBitmap* mic_full_;  // Mic image with full volume.
-  static SkBitmap* mic_empty_;  // Mic image with zero volume.
-  static SkBitmap* mic_mask_;  // Gradient mask used by the volume indicator.
-  static SkBitmap* spinner_;  // Spinner image for the progress animation.
-  static const int kRecognizingAnimationStepMs;
+  // WebContents in which this this bubble gets displayed.
+  content::WebContents* web_contents_;
+  // The current image displayed in the bubble's icon widget.
+  scoped_ptr<SkBitmap> icon_image_;
 };
 
 // This typedef is to workaround the issue with certain versions of
