@@ -1,27 +1,31 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/speech/speech_input_bubble_controller.h"
 
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "base/bind.h"
 #include "chrome/browser/tab_contents/tab_util.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_source.h"
-#include "chrome/common/notification_type.h"
-#include "gfx/rect.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_source.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/web_contents.h"
+#include "ui/gfx/rect.h"
+
+using content::BrowserThread;
+using content::WebContents;
 
 namespace speech_input {
 
 SpeechInputBubbleController::SpeechInputBubbleController(Delegate* delegate)
     : delegate_(delegate),
       current_bubble_caller_id_(0),
-      registrar_(new NotificationRegistrar) {
+      registrar_(new content::NotificationRegistrar) {
 }
 
 SpeechInputBubbleController::~SpeechInputBubbleController() {
-  DCHECK(bubbles_.size() == 0);
+  DCHECK(bubbles_.empty());
 }
 
 void SpeechInputBubbleController::CreateBubble(int caller_id,
@@ -31,20 +35,28 @@ void SpeechInputBubbleController::CreateBubble(int caller_id,
   if (!BrowserThread::CurrentlyOn(BrowserThread::UI)) {
     BrowserThread::PostTask(
         BrowserThread::UI, FROM_HERE,
-        NewRunnableMethod(this, &SpeechInputBubbleController::CreateBubble,
+        base::Bind(&SpeechInputBubbleController::CreateBubble, this,
                           caller_id, render_process_id, render_view_id,
                           element_rect));
     return;
   }
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  TabContents* tab_contents = tab_util::GetTabContentsByID(render_process_id,
+  WebContents* web_contents = tab_util::GetWebContentsByID(render_process_id,
                                                            render_view_id);
 
   DCHECK_EQ(0u, bubbles_.count(caller_id));
-  SpeechInputBubble* bubble = SpeechInputBubble::Create(tab_contents, this,
+  SpeechInputBubble* bubble = SpeechInputBubble::Create(web_contents, this,
                                                         element_rect);
-  if (!bubble)  // could be null if tab or display rect were invalid.
+  if (!bubble) {
+    // Could be null if tab or display rect were invalid.
+    // Simulate the cancel button being clicked to inform the delegate.
+    BrowserThread::PostTask(
+          BrowserThread::IO, FROM_HERE,
+          base::Bind(
+              &SpeechInputBubbleController::InvokeDelegateButtonClicked,
+              this, caller_id, SpeechInputBubble::BUTTON_CANCEL));
     return;
+  }
 
   bubbles_[caller_id] = bubble;
 
@@ -52,28 +64,34 @@ void SpeechInputBubbleController::CreateBubble(int caller_id,
 }
 
 void SpeechInputBubbleController::CloseBubble(int caller_id) {
-  ProcessRequestInUiThread(caller_id, REQUEST_CLOSE, string16(), 0);
+  ProcessRequestInUiThread(caller_id, REQUEST_CLOSE, string16(), 0, 0);
+}
+
+void SpeechInputBubbleController::SetBubbleWarmUpMode(int caller_id) {
+  ProcessRequestInUiThread(caller_id, REQUEST_SET_WARM_UP_MODE,
+                           string16(), 0, 0);
 }
 
 void SpeechInputBubbleController::SetBubbleRecordingMode(int caller_id) {
   ProcessRequestInUiThread(caller_id, REQUEST_SET_RECORDING_MODE,
-                           string16(), 0);
+                           string16(), 0, 0);
 }
 
 void SpeechInputBubbleController::SetBubbleRecognizingMode(int caller_id) {
   ProcessRequestInUiThread(caller_id, REQUEST_SET_RECOGNIZING_MODE,
-                           string16(), 0);
+                           string16(), 0, 0);
 }
 
 void SpeechInputBubbleController::SetBubbleInputVolume(int caller_id,
-                                                       float volume) {
+                                                       float volume,
+                                                       float noise_volume) {
   ProcessRequestInUiThread(caller_id, REQUEST_SET_INPUT_VOLUME, string16(),
-                           volume);
+                           volume, noise_volume);
 }
 
 void SpeechInputBubbleController::SetBubbleMessage(int caller_id,
                                                    const string16& text) {
-  ProcessRequestInUiThread(caller_id, REQUEST_SET_MESSAGE, text, 0);
+  ProcessRequestInUiThread(caller_id, REQUEST_SET_MESSAGE, text, 0, 0);
 }
 
 void SpeechInputBubbleController::UpdateTabContentsSubscription(
@@ -83,10 +101,10 @@ void SpeechInputBubbleController::UpdateTabContentsSubscription(
   // If there are any other bubbles existing for the same TabContents, we would
   // have subscribed to tab close notifications on their behalf and we need to
   // stay registered. So we don't change the subscription in such cases.
-  TabContents* tab_contents = bubbles_[caller_id]->tab_contents();
+  WebContents* web_contents = bubbles_[caller_id]->web_contents();
   for (BubbleCallerIdMap::iterator iter = bubbles_.begin();
        iter != bubbles_.end(); ++iter) {
-    if (iter->second->tab_contents() == tab_contents &&
+    if (iter->second->web_contents() == web_contents &&
         iter->first != caller_id) {
       // At least one other bubble exists for the same TabContents. So don't
       // make any change to the subscription.
@@ -95,28 +113,28 @@ void SpeechInputBubbleController::UpdateTabContentsSubscription(
   }
 
   if (action == BUBBLE_ADDED) {
-    registrar_->Add(this, NotificationType::TAB_CONTENTS_DESTROYED,
-                    Source<TabContents>(tab_contents));
+    registrar_->Add(this, content::NOTIFICATION_WEB_CONTENTS_DESTROYED,
+                    content::Source<WebContents>(web_contents));
   } else {
-    registrar_->Remove(this, NotificationType::TAB_CONTENTS_DESTROYED,
-                    Source<TabContents>(tab_contents));
+    registrar_->Remove(this, content::NOTIFICATION_WEB_CONTENTS_DESTROYED,
+                    content::Source<WebContents>(web_contents));
   }
 }
 
-void SpeechInputBubbleController::Observe(NotificationType type,
-                                          const NotificationSource& source,
-                                          const NotificationDetails& details) {
-  if (type == NotificationType::TAB_CONTENTS_DESTROYED) {
+void SpeechInputBubbleController::Observe(
+    int type,
+    const content::NotificationSource& source,
+    const content::NotificationDetails& details) {
+  if (type == content::NOTIFICATION_WEB_CONTENTS_DESTROYED) {
     // Cancel all bubbles and active recognition sessions for this tab.
-    TabContents* tab_contents = Source<TabContents>(source).ptr();
+    WebContents* web_contents = content::Source<WebContents>(source).ptr();
     BubbleCallerIdMap::iterator iter = bubbles_.begin();
     while (iter != bubbles_.end()) {
-      if (iter->second->tab_contents() == tab_contents) {
+      if (iter->second->web_contents() == web_contents) {
         BrowserThread::PostTask(BrowserThread::IO, FROM_HERE,
-            NewRunnableMethod(
-                this,
+            base::Bind(
                 &SpeechInputBubbleController::InvokeDelegateButtonClicked,
-                iter->first, SpeechInputBubble::BUTTON_CANCEL));
+                this, iter->first, SpeechInputBubble::BUTTON_CANCEL));
         CloseBubble(iter->first);
         // We expect to have a very small number of items in this map so
         // redo-ing from start is ok.
@@ -131,11 +149,12 @@ void SpeechInputBubbleController::Observe(NotificationType type,
 }
 
 void SpeechInputBubbleController::ProcessRequestInUiThread(
-    int caller_id, RequestType type, const string16& text, float volume) {
+    int caller_id, RequestType type, const string16& text, float volume,
+    float noise_volume) {
   if (!BrowserThread::CurrentlyOn(BrowserThread::UI)) {
-    BrowserThread::PostTask(BrowserThread::UI, FROM_HERE, NewRunnableMethod(
-        this, &SpeechInputBubbleController::ProcessRequestInUiThread,
-        caller_id, type, text, volume));
+    BrowserThread::PostTask(BrowserThread::UI, FROM_HERE, base::Bind(
+        &SpeechInputBubbleController::ProcessRequestInUiThread, this,
+        caller_id, type, text, volume, noise_volume));
     return;
   }
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
@@ -144,7 +163,7 @@ void SpeechInputBubbleController::ProcessRequestInUiThread(
   if (!bubbles_.count(caller_id))
     return;
 
-  bool change_active_bubble = (type == REQUEST_SET_RECORDING_MODE ||
+  bool change_active_bubble = (type == REQUEST_SET_WARM_UP_MODE ||
                                type == REQUEST_SET_MESSAGE);
   if (change_active_bubble) {
     if (current_bubble_caller_id_ && current_bubble_caller_id_ != caller_id)
@@ -154,6 +173,9 @@ void SpeechInputBubbleController::ProcessRequestInUiThread(
 
   SpeechInputBubble* bubble = bubbles_[caller_id];
   switch (type) {
+    case REQUEST_SET_WARM_UP_MODE:
+      bubble->SetWarmUpMode();
+      break;
     case REQUEST_SET_RECORDING_MODE:
       bubble->SetRecordingMode();
       break;
@@ -164,7 +186,7 @@ void SpeechInputBubbleController::ProcessRequestInUiThread(
       bubble->SetMessage(text);
       break;
     case REQUEST_SET_INPUT_VOLUME:
-      bubble->SetInputVolume(volume);
+      bubble->SetInputVolume(volume, noise_volume);
       break;
     case REQUEST_CLOSE:
       if (current_bubble_caller_id_ == caller_id)
@@ -189,10 +211,9 @@ void SpeechInputBubbleController::InfoBubbleButtonClicked(
 
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(
-          this,
+      base::Bind(
           &SpeechInputBubbleController::InvokeDelegateButtonClicked,
-          current_bubble_caller_id_, button));
+          this, current_bubble_caller_id_, button));
 }
 
 void SpeechInputBubbleController::InfoBubbleFocusChanged() {
@@ -201,14 +222,12 @@ void SpeechInputBubbleController::InfoBubbleFocusChanged() {
 
   int old_bubble_caller_id = current_bubble_caller_id_;
   current_bubble_caller_id_ = 0;
-  bubbles_[old_bubble_caller_id]->Hide();
 
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(
-          this,
+      base::Bind(
           &SpeechInputBubbleController::InvokeDelegateFocusChanged,
-          old_bubble_caller_id));
+          this, old_bubble_caller_id));
 }
 
 void SpeechInputBubbleController::InvokeDelegateButtonClicked(
