@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,15 +8,18 @@
 #include <map>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/speech/speech_input_bubble.h"
-#include "chrome/common/notification_observer.h"
+#include "content/public/browser/notification_observer.h"
 
 namespace gfx {
 class Rect;
 }
+
+namespace content {
 class NotificationRegistrar;
+}
 
 namespace speech_input {
 
@@ -28,7 +31,7 @@ namespace speech_input {
 class SpeechInputBubbleController
     : public base::RefCountedThreadSafe<SpeechInputBubbleController>,
       public SpeechInputBubbleDelegate,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
   // All methods of this delegate are called in the IO thread.
   class Delegate {
@@ -55,6 +58,10 @@ class SpeechInputBubbleController
                     int render_view_id,
                     const gfx::Rect& element_rect);
 
+  // Indicates to the user that audio hardware is warming up. This also makes
+  // the bubble visible if not already visible.
+  void SetBubbleWarmUpMode(int caller_id);
+
   // Indicates to the user that audio recording is in progress. This also makes
   // the bubble visible if not already visible.
   void SetBubbleRecordingMode(int caller_id);
@@ -68,22 +75,24 @@ class SpeechInputBubbleController
   void SetBubbleMessage(int caller_id, const string16& text);
 
   // Updates the current captured audio volume displayed on screen.
-  void SetBubbleInputVolume(int caller_id, float volume);
+  void SetBubbleInputVolume(int caller_id, float volume, float noise_volume);
 
   void CloseBubble(int caller_id);
 
   // SpeechInputBubble::Delegate methods.
-  virtual void InfoBubbleButtonClicked(SpeechInputBubble::Button button);
-  virtual void InfoBubbleFocusChanged();
+  virtual void InfoBubbleButtonClicked(
+      SpeechInputBubble::Button button) OVERRIDE;
+  virtual void InfoBubbleFocusChanged() OVERRIDE;
 
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
  private:
   // The various calls received by this object and handled in the UI thread.
   enum RequestType {
+    REQUEST_SET_WARM_UP_MODE,
     REQUEST_SET_RECORDING_MODE,
     REQUEST_SET_RECOGNIZING_MODE,
     REQUEST_SET_MESSAGE,
@@ -102,7 +111,8 @@ class SpeechInputBubbleController
   void ProcessRequestInUiThread(int caller_id,
                                 RequestType type,
                                 const string16& text,
-                                float volume);
+                                float volume,
+                                float noise_volume);
 
   // Called whenever a bubble was added to or removed from the list. If the
   // bubble was being added, this method registers for close notifications with
@@ -115,7 +125,7 @@ class SpeechInputBubbleController
   // Only accessed in the IO thread.
   Delegate* delegate_;
 
-  //*** The following are accessed only in the UI thread.
+  // *** The following are accessed only in the UI thread.
 
   // The caller id for currently visible bubble (since only one bubble is
   // visible at any time).
@@ -126,7 +136,7 @@ class SpeechInputBubbleController
   typedef std::map<int, SpeechInputBubble*> BubbleCallerIdMap;
   BubbleCallerIdMap bubbles_;
 
-  scoped_ptr<NotificationRegistrar> registrar_;
+  scoped_ptr<content::NotificationRegistrar> registrar_;
 };
 
 // This typedef is to workaround the issue with certain versions of
