@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,12 +6,18 @@
 
 #include "chrome/browser/speech/speech_input_bubble.h"
 
-#import "base/scoped_nsobject.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/tab_contents/tab_contents_view.h"
+#import "base/memory/scoped_nsobject.h"
+#include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/cocoa/browser_window_cocoa.h"
+#include "chrome/browser/ui/cocoa/browser_window_controller.h"
+#include "chrome/browser/ui/cocoa/location_bar/location_bar_view_mac.h"
 #import "chrome/browser/ui/cocoa/speech_input_window_controller.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/browser/web_contents_view.h"
 #include "skia/ext/skia_utils_mac.h"
 
+using content::WebContents;
+
 namespace {
 
 // A class to bridge between the speech recognition C++ code and the Objective-C
@@ -19,14 +25,14 @@ namespace {
 // more information on how this gets used.
 class SpeechInputBubbleImpl : public SpeechInputBubbleBase {
  public:
-  SpeechInputBubbleImpl(TabContents* tab_contents,
+  SpeechInputBubbleImpl(WebContents* web_contents,
                         Delegate* delegate,
                         const gfx::Rect& element_rect);
   virtual ~SpeechInputBubbleImpl();
   virtual void Show();
   virtual void Hide();
   virtual void UpdateLayout();
-  virtual void SetImage(const SkBitmap& image);
+  virtual void UpdateImage();
 
  private:
   scoped_nsobject<SpeechInputWindowController> window_;
@@ -34,10 +40,10 @@ class SpeechInputBubbleImpl : public SpeechInputBubbleBase {
   gfx::Rect element_rect_;
 };
 
-SpeechInputBubbleImpl::SpeechInputBubbleImpl(TabContents* tab_contents,
+SpeechInputBubbleImpl::SpeechInputBubbleImpl(WebContents* web_contents,
                                              Delegate* delegate,
                                              const gfx::Rect& element_rect)
-    : SpeechInputBubbleBase(tab_contents),
+    : SpeechInputBubbleBase(web_contents),
       delegate_(delegate),
       element_rect_(element_rect) {
 }
@@ -47,9 +53,9 @@ SpeechInputBubbleImpl::~SpeechInputBubbleImpl() {
     [window_.get() close];
 }
 
-void SpeechInputBubbleImpl::SetImage(const SkBitmap& image) {
+void SpeechInputBubbleImpl::UpdateImage() {
   if (window_.get())
-    [window_.get() setImage:gfx::SkBitmapToNSImage(image)];
+    [window_.get() setImage:gfx::SkBitmapToNSImage(icon_image())];
 }
 
 void SpeechInputBubbleImpl::Show() {
@@ -60,22 +66,35 @@ void SpeechInputBubbleImpl::Show() {
 
   // Find the screen coordinates for the given tab and position the bubble's
   // arrow anchor point inside that to point at the bottom-left of the html
-  // input element rect.
-  gfx::NativeView view = tab_contents()->view()->GetNativeView();
+  // input element rect if the position is valid, otherwise point it towards
+  // the page icon in the omnibox.
+  gfx::NativeView view = web_contents()->GetView()->GetNativeView();
+  NSWindow* parentWindow = web_contents()->GetView()->GetTopLevelNativeWindow();
   NSRect tab_bounds = [view bounds];
-  NSPoint anchor = NSMakePoint(
-      tab_bounds.origin.x + element_rect_.x() + kBubbleTargetOffsetX,
-      tab_bounds.origin.y + tab_bounds.size.height - element_rect_.y() -
-      element_rect_.height());
+  int anchor_x = tab_bounds.origin.x + element_rect_.x() +
+                 element_rect_.width() - kBubbleTargetOffsetX;
+  int anchor_y = tab_bounds.origin.y + tab_bounds.size.height -
+                 element_rect_.y() - element_rect_.height();
+  NSPoint anchor = NSZeroPoint;
+  if (anchor_x < 0 || anchor_y < 0 ||
+      anchor_x > NSWidth([parentWindow frame]) ||
+      anchor_y > NSHeight([parentWindow frame])) {
+    LocationBarViewMac* locationBar =
+        [[parentWindow windowController] locationBarBridge];
+    anchor = locationBar->GetPageInfoBubblePoint();
+  } else {
+    anchor = NSMakePoint(anchor_x, anchor_y);
+  }
   anchor = [view convertPoint:anchor toView:nil];
   anchor = [[view window] convertBaseToScreen:anchor];
 
   window_.reset([[SpeechInputWindowController alloc]
-      initWithParentWindow:tab_contents()->view()->GetTopLevelNativeWindow()
+      initWithParentWindow:parentWindow
                   delegate:delegate_
                 anchoredAt:anchor]);
 
   UpdateLayout();
+  [window_.get() show];
 }
 
 void SpeechInputBubbleImpl::Hide() {
@@ -91,15 +110,16 @@ void SpeechInputBubbleImpl::UpdateLayout() {
     return;
 
   [window_.get() updateLayout:display_mode()
-                  messageText:message_text()];
+                  messageText:message_text()
+                    iconImage:gfx::SkBitmapToNSImage(icon_image())];
 }
 
 }  // namespace
 
 SpeechInputBubble* SpeechInputBubble::CreateNativeBubble(
-    TabContents* tab_contents,
+    WebContents* web_contents,
     Delegate* delegate,
     const gfx::Rect& element_rect) {
-  return new SpeechInputBubbleImpl(tab_contents, delegate, element_rect);
+  return new SpeechInputBubbleImpl(web_contents, delegate, element_rect);
 }
 
