@@ -1,12 +1,12 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ssl/ssl_error_info.h"
 
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/cert_store.h"
 #include "chrome/common/time_format.h"
+#include "content/browser/cert_store.h"
 #include "googleurl/src/gurl.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
@@ -153,9 +153,15 @@ SSLErrorInfo SSLErrorInfo::CreateError(ErrorType error_type,
       break;
     case CERT_INVALID:
       title = l10n_util::GetStringUTF16(IDS_CERT_ERROR_INVALID_CERT_TITLE);
-      details = l10n_util::GetStringUTF16(IDS_CERT_ERROR_INVALID_CERT_DETAILS);
+      details = l10n_util::GetStringFUTF16(
+          IDS_CERT_ERROR_INVALID_CERT_DETAILS,
+          UTF8ToUTF16(request_url.host()));
       short_description =
           l10n_util::GetStringUTF16(IDS_CERT_ERROR_INVALID_CERT_DESCRIPTION);
+      extra_info.push_back(
+          l10n_util::GetStringUTF16(IDS_CERT_ERROR_EXTRA_INFO_1));
+      extra_info.push_back(l10n_util::GetStringUTF16(
+          IDS_CERT_ERROR_INVALID_CERT_EXTRA_INFO_2));
       break;
     case CERT_WEAK_SIGNATURE_ALGORITHM:
       title = l10n_util::GetStringUTF16(
@@ -171,6 +177,18 @@ SSLErrorInfo SSLErrorInfo::CreateError(ErrorType error_type,
           l10n_util::GetStringUTF16(
               IDS_CERT_ERROR_WEAK_SIGNATURE_ALGORITHM_EXTRA_INFO_2));
       break;
+    case CERT_WEAK_KEY:
+      title = l10n_util::GetStringUTF16(IDS_CERT_ERROR_WEAK_KEY_TITLE);
+      details = l10n_util::GetStringFUTF16(
+          IDS_CERT_ERROR_WEAK_KEY_DETAILS, UTF8ToUTF16(request_url.host()));
+      short_description = l10n_util::GetStringUTF16(
+          IDS_CERT_ERROR_WEAK_KEY_DESCRIPTION);
+      extra_info.push_back(
+          l10n_util::GetStringUTF16(IDS_CERT_ERROR_EXTRA_INFO_1));
+      extra_info.push_back(
+          l10n_util::GetStringUTF16(
+              IDS_CERT_ERROR_WEAK_KEY_EXTRA_INFO_2));
+      break;
     case CERT_NOT_IN_DNS:
       title = l10n_util::GetStringUTF16(IDS_CERT_ERROR_NOT_IN_DNS_TITLE);
       details = l10n_util::GetStringUTF16(IDS_CERT_ERROR_NOT_IN_DNS_DETAILS);
@@ -215,6 +233,8 @@ SSLErrorInfo::ErrorType SSLErrorInfo::NetErrorToErrorType(int net_error) {
       return CERT_INVALID;
     case net::ERR_CERT_WEAK_SIGNATURE_ALGORITHM:
       return CERT_WEAK_SIGNATURE_ALGORITHM;
+    case net::ERR_CERT_WEAK_KEY:
+      return CERT_WEAK_KEY;
     case net::ERR_CERT_NOT_IN_DNS:
       return CERT_NOT_IN_DNS;
     default:
@@ -225,10 +245,10 @@ SSLErrorInfo::ErrorType SSLErrorInfo::NetErrorToErrorType(int net_error) {
 
 // static
 int SSLErrorInfo::GetErrorsForCertStatus(int cert_id,
-                                         int cert_status,
+                                         net::CertStatus cert_status,
                                          const GURL& url,
                                          std::vector<SSLErrorInfo>* errors) {
-  const int kErrorFlags[] = {
+  const net::CertStatus kErrorFlags[] = {
     net::CERT_STATUS_COMMON_NAME_INVALID,
     net::CERT_STATUS_DATE_INVALID,
     net::CERT_STATUS_AUTHORITY_INVALID,
@@ -236,7 +256,8 @@ int SSLErrorInfo::GetErrorsForCertStatus(int cert_id,
     net::CERT_STATUS_UNABLE_TO_CHECK_REVOCATION,
     net::CERT_STATUS_REVOKED,
     net::CERT_STATUS_INVALID,
-    net::CERT_STATUS_WEAK_SIGNATURE_ALGORITHM
+    net::CERT_STATUS_WEAK_SIGNATURE_ALGORITHM,
+    net::CERT_STATUS_WEAK_KEY
   };
 
   const ErrorType kErrorTypes[] = {
@@ -247,7 +268,8 @@ int SSLErrorInfo::GetErrorsForCertStatus(int cert_id,
     CERT_UNABLE_TO_CHECK_REVOCATION,
     CERT_REVOKED,
     CERT_INVALID,
-    CERT_WEAK_SIGNATURE_ALGORITHM
+    CERT_WEAK_SIGNATURE_ALGORITHM,
+    CERT_WEAK_KEY
   };
   DCHECK(arraysize(kErrorFlags) == arraysize(kErrorTypes));
 
