@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,6 +10,7 @@
 #include <vector>
 
 #include "base/string16.h"
+#include "net/base/cert_status_flags.h"
 #include "net/base/x509_certificate.h"
 
 class GURL;
@@ -31,6 +32,7 @@ class SSLErrorInfo {
     CERT_INVALID,
     CERT_WEAK_SIGNATURE_ALGORITHM,
     CERT_NOT_IN_DNS,
+    CERT_WEAK_KEY,
     UNKNOWN
   };
 
@@ -46,9 +48,10 @@ class SSLErrorInfo {
   // Populates the specified |errors| vector with the errors contained in
   // |cert_status|.  Returns the number of errors found.
   // Callers only interested in the error count can pass NULL for |errors|.
-  static int GetErrorsForCertStatus(int cert_status,
-                                    int cert_id,
-                                    const GURL& request_url,
+  // TODO(wtc): Document |cert_id| and |url| arguments.
+  static int GetErrorsForCertStatus(int cert_id,
+                                    net::CertStatus cert_status,
+                                    const GURL& url,
                                     std::vector<SSLErrorInfo>* errors);
 
   // A title describing the error, usually to be used with the details below.
