@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,7 @@
 
 #include "ui/base/models/menu_model.h"
 
-StatusIcon::StatusIcon()
-{
+StatusIcon::StatusIcon() {
 }
 
 StatusIcon::~StatusIcon() {
