@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,8 @@
 #define CHROME_BROWSER_STATUS_ICONS_STATUS_ICON_H_
 #pragma once
 
+#include "base/memory/scoped_ptr.h"
 #include "base/observer_list.h"
-#include "base/scoped_ptr.h"
 #include "base/string16.h"
 
 class SkBitmap;
@@ -30,6 +30,12 @@ class StatusIcon {
   // Sets the hover text for this status icon.
   virtual void SetToolTip(const string16& tool_tip) = 0;
 
+  // Displays a notification balloon with the specified contents.
+  // Depending on the platform it might not appear by the icon tray.
+  virtual void DisplayBalloon(const SkBitmap& icon,
+                              const string16& title,
+                              const string16& contents) = 0;
+
   // Set the context menu for this icon. The icon takes ownership of the passed
   // context menu. Passing NULL results in no menu at all.
   void SetContextMenu(ui::MenuModel* menu);
@@ -43,6 +49,8 @@ class StatusIcon {
     // (i.e. if there's a context menu set on this status icon, and the user
     // right clicks on the icon to display the context menu, OnClicked will not
     // be called).
+    // Note: Chrome OS displays the context menu on left button clicks.
+    // This will only be fired for this platform if no context menu is present.
     virtual void OnClicked() = 0;
   };
 
