@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,15 +12,16 @@
 #include "chrome/browser/sync/protocol/service_constants.h"
 #include "chrome/browser/sync/sessions/session_state.h"
 #include "chrome/browser/sync/syncable/directory_manager.h"
+#include "chrome/browser/sync/syncable/model_type.h"
 
 namespace browser_sync {
 
-static const sync_api::SyncManager::Status init_status =
-  { sync_api::SyncManager::Status::OFFLINE };
-
-AllStatus::AllStatus() : status_(init_status) {
+AllStatus::AllStatus() {
+  status_.summary = sync_api::SyncManager::Status::OFFLINE;
   status_.initial_sync_ended = true;
   status_.notifications_enabled = false;
+  status_.cryptographer_ready = false;
+  status_.crypto_has_pending_keys = false;
 }
 
 AllStatus::~AllStatus() {
@@ -31,13 +32,10 @@ sync_api::SyncManager::Status AllStatus::CreateBlankStatus() const {
   // whose values accumulate (e.g. lifetime counters like updates_received)
   // are not to be cleared here.
   sync_api::SyncManager::Status status = status_;
-  status.syncing = true;
   status.unsynced_count = 0;
   status.conflicting_count = 0;
   status.initial_sync_ended = false;
-  status.syncer_stuck = false;
   status.max_consecutive_errors = 0;
-  status.server_broken = false;
   status.updates_available = 0;
   return status;
 }
@@ -52,20 +50,20 @@ sync_api::SyncManager::Status AllStatus::CalcSyncing(
   // But this is only used for status, so it is better to have visibility.
   status.conflicting_count += snapshot->num_conflicting_updates;
 
-  status.syncing |= snapshot->syncer_status.syncing;
-  status.syncing = snapshot->has_more_to_sync && snapshot->is_silenced;
+  if (event.what_happened == SyncEngineEvent::SYNC_CYCLE_BEGIN) {
+    status.syncing = true;
+  } else if (event.what_happened == SyncEngineEvent::SYNC_CYCLE_ENDED) {
+    status.syncing = false;
+  }
+
   status.initial_sync_ended |= snapshot->is_share_usable;
-  status.syncer_stuck |= snapshot->syncer_status.syncer_stuck;
 
   const sessions::ErrorCounters& errors(snapshot->errors);
   if (errors.consecutive_errors > status.max_consecutive_errors)
     status.max_consecutive_errors = errors.consecutive_errors;
 
-  // 100 is an arbitrary limit.
-  if (errors.consecutive_transient_error_commits > 100)
-    status.server_broken = true;
-
   status.updates_available += snapshot->num_server_changes_remaining;
+  status.sync_protocol_error = snapshot->errors.sync_protocol_error;
 
   // Accumulate update count only once per session to avoid double-counting.
   // TODO(ncarter): Make this realtime by having the syncer_status
@@ -75,6 +73,21 @@ sync_api::SyncManager::Status AllStatus::CalcSyncing(
         snapshot->syncer_status.num_updates_downloaded_total;
     status.tombstone_updates_received +=
         snapshot->syncer_status.num_tombstone_updates_downloaded_total;
+    status.num_local_overwrites_total +=
+        snapshot->syncer_status.num_local_overwrites;
+    status.num_server_overwrites_total +=
+        snapshot->syncer_status.num_server_overwrites;
+    if (snapshot->syncer_status.num_updates_downloaded_total == 0) {
+      ++status.empty_get_updates;
+    } else {
+      ++status.nonempty_get_updates;
+    }
+    if (snapshot->syncer_status.num_successful_commits == 0 &&
+        snapshot->syncer_status.num_updates_downloaded_total == 0) {
+      ++status.useless_sync_cycles;
+    } else {
+      ++status.useful_sync_cycles;
+    }
   }
   return status;
 }
@@ -82,11 +95,9 @@ sync_api::SyncManager::Status AllStatus::CalcSyncing(
 void AllStatus::CalcStatusChanges() {
   const bool unsynced_changes = status_.unsynced_count > 0;
   const bool online = status_.authenticated &&
-    status_.server_reachable && status_.server_up && !status_.server_broken;
+    status_.server_reachable && status_.server_up;
   if (online) {
-    if (status_.syncer_stuck)
-      status_.summary = sync_api::SyncManager::Status::CONFLICT;
-    else if (unsynced_changes || status_.syncing)
+    if (status_.syncing)
       status_.summary = sync_api::SyncManager::Status::SYNCING;
     else
       status_.summary = sync_api::SyncManager::Status::READY;
@@ -102,17 +113,20 @@ void AllStatus::CalcStatusChanges() {
 void AllStatus::OnSyncEngineEvent(const SyncEngineEvent& event) {
   ScopedStatusLock lock(this);
   switch (event.what_happened) {
-    case SyncEngineEvent::SYNC_CYCLE_ENDED:
+    case SyncEngineEvent::SYNC_CYCLE_BEGIN:
     case SyncEngineEvent::STATUS_CHANGED:
+    case SyncEngineEvent::SYNC_CYCLE_ENDED:
       status_ = CalcSyncing(event);
       break;
-    case SyncEngineEvent::SYNCER_THREAD_PAUSED:
-    case SyncEngineEvent::SYNCER_THREAD_RESUMED:
-    case SyncEngineEvent::SYNCER_THREAD_WAITING_FOR_CONNECTION:
-    case SyncEngineEvent::SYNCER_THREAD_CONNECTED:
     case SyncEngineEvent::STOP_SYNCING_PERMANENTLY:
-    case SyncEngineEvent::SYNCER_THREAD_EXITING:
+    case SyncEngineEvent::UPDATED_TOKEN:
+    case SyncEngineEvent::CLEAR_SERVER_DATA_FAILED:
+    case SyncEngineEvent::CLEAR_SERVER_DATA_SUCCEEDED:
        break;
+    case SyncEngineEvent::ACTIONABLE_ERROR:
+      status_ = CreateBlankStatus();
+      status_.sync_protocol_error = event.snapshot->errors.sync_protocol_error;
+      break;
     default:
       LOG(ERROR) << "Unrecognized Syncer Event: " << event.what_happened;
       break;
@@ -121,19 +135,14 @@ void AllStatus::OnSyncEngineEvent(const SyncEngineEvent& event) {
 
 void AllStatus::HandleServerConnectionEvent(
     const ServerConnectionEvent& event) {
-  if (ServerConnectionEvent::STATUS_CHANGED == event.what_happened) {
-    ScopedStatusLock lock(this);
-    status_.server_up = IsGoodReplyFromServer(event.connection_code);
-    status_.server_reachable = event.server_reachable;
+  ScopedStatusLock lock(this);
+  status_.server_up = IsGoodReplyFromServer(event.connection_code);
+  status_.server_reachable = event.server_reachable;
 
-    if (event.connection_code == HttpResponse::SERVER_CONNECTION_OK) {
-      if (!status_.authenticated) {
-        status_ = CreateBlankStatus();
-      }
-      status_.authenticated = true;
-    } else {
-      status_.authenticated = false;
-    }
+  if (event.connection_code == HttpResponse::SERVER_CONNECTION_OK) {
+    status_.authenticated = true;
+  } else {
+    status_.authenticated = false;
   }
 }
 
@@ -147,9 +156,9 @@ void AllStatus::SetNotificationsEnabled(bool notifications_enabled) {
   status_.notifications_enabled = notifications_enabled;
 }
 
-void AllStatus::IncrementNotificationsSent() {
+void AllStatus::IncrementNotifiableCommits() {
   ScopedStatusLock lock(this);
-  ++status_.notifications_sent;
+  ++status_.notifiable_commits;
 }
 
 void AllStatus::IncrementNotificationsReceived() {
@@ -157,6 +166,26 @@ void AllStatus::IncrementNotificationsReceived() {
   ++status_.notifications_received;
 }
 
+void AllStatus::SetEncryptedTypes(syncable::ModelTypeSet types) {
+  ScopedStatusLock lock(this);
+  status_.encrypted_types = types;
+}
+
+void AllStatus::SetCryptographerReady(bool ready) {
+  ScopedStatusLock lock(this);
+  status_.cryptographer_ready = ready;
+}
+
+void AllStatus::SetCryptoHasPendingKeys(bool has_pending_keys) {
+  ScopedStatusLock lock(this);
+  status_.crypto_has_pending_keys = has_pending_keys;
+}
+
+void AllStatus::SetUniqueId(const std::string& guid) {
+  ScopedStatusLock lock(this);
+  status_.unique_id = guid;
+}
+
 ScopedStatusLock::ScopedStatusLock(AllStatus* allstatus)
     : allstatus_(allstatus) {
   allstatus->mutex_.Acquire();
