@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,14 +6,9 @@
 #define CHROME_BROWSER_SYNC_ENGINE_APPLY_UPDATES_COMMAND_H_
 #pragma once
 
+#include "base/compiler_specific.h"
 #include "chrome/browser/sync/engine/model_changing_syncer_command.h"
 
-namespace syncable {
-class WriteTransaction;
-class MutableEntry;
-class Id;
-}
-
 namespace browser_sync {
 
 class ApplyUpdatesCommand : public ModelChangingSyncerCommand {
@@ -21,8 +16,12 @@ class ApplyUpdatesCommand : public ModelChangingSyncerCommand {
   ApplyUpdatesCommand();
   virtual ~ApplyUpdatesCommand();
 
+ protected:
   // ModelChangingSyncerCommand implementation.
-  virtual void ModelChangingExecuteImpl(sessions::SyncSession* session);
+  virtual std::set<ModelSafeGroup> GetGroupsToChange(
+      const sessions::SyncSession& session) const OVERRIDE;
+  virtual SyncerError ModelChangingExecuteImpl(
+      sessions::SyncSession* session) OVERRIDE;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(ApplyUpdatesCommand);
