@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,7 @@
 #define CHROME_BROWSER_SYNC_ENGINE_POST_COMMIT_MESSAGE_COMMAND_H_
 #pragma once
 
+#include "base/compiler_specific.h"
 #include "chrome/browser/sync/engine/syncer_command.h"
 
 namespace browser_sync {
@@ -16,7 +17,7 @@ class PostCommitMessageCommand : public SyncerCommand {
   virtual ~PostCommitMessageCommand();
 
   // SyncerCommand implementation.
-  virtual void ExecuteImpl(sessions::SyncSession* session);
+  virtual SyncerError ExecuteImpl(sessions::SyncSession* session) OVERRIDE;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(PostCommitMessageCommand);
