@@ -1,11 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <vector>
 
-#include "base/string_util.h"
-#include "chrome/browser/sync/engine/mock_model_safe_workers.h"
+#include "base/location.h"
+#include "base/stringprintf.h"
 #include "chrome/browser/sync/engine/process_commit_response_command.h"
 #include "chrome/browser/sync/protocol/bookmark_specifics.pb.h"
 #include "chrome/browser/sync/protocol/sync.pb.h"
@@ -13,8 +13,9 @@
 #include "chrome/browser/sync/syncable/directory_manager.h"
 #include "chrome/browser/sync/syncable/syncable.h"
 #include "chrome/browser/sync/syncable/syncable_id.h"
-#include "chrome/test/sync/engine/syncer_command_test.h"
-#include "chrome/test/sync/engine/test_id_factory.h"
+#include "chrome/browser/sync/test/engine/fake_model_worker.h"
+#include "chrome/browser/sync/test/engine/syncer_command_test.h"
+#include "chrome/browser/sync/test/engine/test_id_factory.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace browser_sync {
@@ -42,16 +43,19 @@ class ProcessCommitResponseCommandTestWithParam
     workers()->clear();
     mutable_routing_info()->clear();
 
-    // GROUP_PASSIVE worker.
-    workers()->push_back(make_scoped_refptr(new ModelSafeWorker()));
-    // GROUP_UI worker.
-    workers()->push_back(make_scoped_refptr(new MockUIModelWorker()));
+    workers()->push_back(
+        make_scoped_refptr(new FakeModelWorker(GROUP_DB)));
+    workers()->push_back(
+        make_scoped_refptr(new FakeModelWorker(GROUP_UI)));
     (*mutable_routing_info())[syncable::BOOKMARKS] = GROUP_UI;
     (*mutable_routing_info())[syncable::PREFERENCES] = GROUP_UI;
-    (*mutable_routing_info())[syncable::AUTOFILL] = GROUP_PASSIVE;
+    (*mutable_routing_info())[syncable::AUTOFILL] = GROUP_DB;
 
     commit_set_.reset(new sessions::OrderedCommitSet(routing_info()));
     SyncerCommandTestWithParam<T>::SetUp();
+    // Need to explicitly use this-> to avoid obscure template
+    // warning.
+    this->ExpectNoGroupsToChange(command_);
   }
 
  protected:
@@ -90,8 +94,10 @@ class ProcessCommitResponseCommandTestWithParam
                           int64* metahandle_out) {
     ScopedDirLookup dir(syncdb()->manager(), syncdb()->name());
     ASSERT_TRUE(dir.good());
-    WriteTransaction trans(dir, UNITTEST, __FILE__, __LINE__);
-    Id predecessor_id = dir->GetLastChildId(&trans, parent_id);
+    WriteTransaction trans(FROM_HERE, UNITTEST, dir);
+    Id predecessor_id;
+    ASSERT_TRUE(
+        dir->GetLastChildIdForTest(&trans, parent_id, &predecessor_id));
     MutableEntry entry(&trans, syncable::CREATE, parent_id, name);
     ASSERT_TRUE(entry.good());
     entry.Put(syncable::ID, item_id);
@@ -123,7 +129,8 @@ class ProcessCommitResponseCommandTestWithParam
                                      const Id& parent_id,
                                      const string& name,
                                      syncable::ModelType model_type) {
-    sessions::StatusController* sync_state = session()->status_controller();
+    sessions::StatusController* sync_state =
+        session()->mutable_status_controller();
     bool is_folder = true;
     int64 metahandle = 0;
     CreateUnsyncedItem(item_id, parent_id, name, is_folder, model_type,
@@ -136,7 +143,7 @@ class ProcessCommitResponseCommandTestWithParam
 
     ScopedDirLookup dir(syncdb()->manager(), syncdb()->name());
     ASSERT_TRUE(dir.good());
-    WriteTransaction trans(dir, UNITTEST, __FILE__, __LINE__);
+    WriteTransaction trans(FROM_HERE, UNITTEST, dir);
     MutableEntry entry(&trans, syncable::GET_BY_ID, item_id);
     ASSERT_TRUE(entry.good());
     entry.Put(syncable::SYNCING, true);
@@ -157,7 +164,7 @@ class ProcessCommitResponseCommandTestWithParam
 
     sync_pb::ClientToServerResponse* response =
         sync_state->mutable_commit_response();
-    response->set_error_code(sync_pb::ClientToServerResponse::SUCCESS);
+    response->set_error_code(sync_pb::SyncEnums::SUCCESS);
     sync_pb::CommitResponse_EntryResponse* entry_response =
         response->mutable_commit()->add_entryresponse();
     entry_response->set_response_type(CommitResponse::SUCCESS);
@@ -184,7 +191,8 @@ class ProcessCommitResponseCommandTestWithParam
   }
 
   void SetLastErrorCode(CommitResponse::ResponseType error_code) {
-    sessions::StatusController* sync_state = session()->status_controller();
+    sessions::StatusController* sync_state =
+        session()->mutable_status_controller();
     sync_pb::ClientToServerResponse* response =
         sync_state->mutable_commit_response();
     sync_pb::CommitResponse_EntryResponse* entry_response =
@@ -228,12 +236,14 @@ TEST_F(ProcessCommitResponseCommandTest, MultipleCommitIdProjections) {
   CreateUnprocessedCommitResult(autofill_id2, id_factory_.root(),
                                 "Autofill 2", syncable::AUTOFILL);
 
+  ExpectGroupsToChange(command_, GROUP_UI, GROUP_DB);
   command_.ExecuteImpl(session());
 
   ScopedDirLookup dir(syncdb()->manager(), syncdb()->name());
   ASSERT_TRUE(dir.good());
-  ReadTransaction trans(dir, __FILE__, __LINE__);
-  Id new_fid = dir->GetFirstChildId(&trans, id_factory_.root());
+  ReadTransaction trans(FROM_HERE, dir);
+  Id new_fid;
+  ASSERT_TRUE(dir->GetFirstChildId(&trans, id_factory_.root(), &new_fid));
   ASSERT_FALSE(new_fid.IsRoot());
   EXPECT_TRUE(new_fid.ServerKnows());
   EXPECT_FALSE(bookmark_folder_id.ServerKnows());
@@ -246,7 +256,8 @@ TEST_F(ProcessCommitResponseCommandTest, MultipleCommitIdProjections) {
       << "Bookmark folder should have a valid (positive) server base revision";
 
   // Look at the two bookmarks in bookmark_folder.
-  Id cid = dir->GetFirstChildId(&trans, new_fid);
+  Id cid;
+  ASSERT_TRUE(dir->GetFirstChildId(&trans, new_fid, &cid));
   Entry b1(&trans, syncable::GET_BY_ID, cid);
   Entry b2(&trans, syncable::GET_BY_ID, b1.Get(syncable::NEXT_ID));
   CheckEntry(&b1, "bookmark 1", syncable::BOOKMARKS, new_fid);
@@ -286,8 +297,10 @@ TEST_F(ProcessCommitResponseCommandTest, NewFolderCommitKeepsChildOrder) {
   {
     ScopedDirLookup dir(syncdb()->manager(), syncdb()->name());
     ASSERT_TRUE(dir.good());
-    ReadTransaction trans(dir, __FILE__, __LINE__);
-    ASSERT_EQ(folder_id, dir->GetFirstChildId(&trans, id_factory_.root()));
+    ReadTransaction trans(FROM_HERE, dir);
+    Id child_id;
+    ASSERT_TRUE(dir->GetFirstChildId(&trans, id_factory_.root(), &child_id));
+    ASSERT_EQ(folder_id, child_id);
   }
 
   // The first 25 children of the parent folder will be part of the commit
@@ -297,8 +310,8 @@ TEST_F(ProcessCommitResponseCommandTest, NewFolderCommitKeepsChildOrder) {
   for (; i < batch_size; ++i) {
     // Alternate between new and old child items, just for kicks.
     Id id = (i % 4 < 2) ? id_factory_.NewLocalId() : id_factory_.NewServerId();
-    CreateUnprocessedCommitResult(id, folder_id, StringPrintf("Item %d", i),
-                                  syncable::BOOKMARKS);
+    CreateUnprocessedCommitResult(
+        id, folder_id, base::StringPrintf("Item %d", i), syncable::BOOKMARKS);
   }
   // The second 25 children will be unsynced items but NOT part of the commit
   // batch.  When the ID of the parent folder changes during the commit,
@@ -307,22 +320,24 @@ TEST_F(ProcessCommitResponseCommandTest, NewFolderCommitKeepsChildOrder) {
   for (; i < 2*batch_size; ++i) {
     // Alternate between new and old child items, just for kicks.
     Id id = (i % 4 < 2) ? id_factory_.NewLocalId() : id_factory_.NewServerId();
-    CreateUnsyncedItem(id, folder_id, StringPrintf("Item %d", i), false,
-                       syncable::BOOKMARKS, NULL);
+    CreateUnsyncedItem(id, folder_id, base::StringPrintf("Item %d", i),
+                       false, syncable::BOOKMARKS, NULL);
   }
 
   // Process the commit response for the parent folder and the first
   // 25 items.  This should apply the values indicated by
   // each CommitResponse_EntryResponse to the syncable Entries.  All new
   // items in the commit batch should have their IDs changed to server IDs.
+  ExpectGroupToChange(command_, GROUP_UI);
   command_.ExecuteImpl(session());
 
   ScopedDirLookup dir(syncdb()->manager(), syncdb()->name());
   ASSERT_TRUE(dir.good());
-  ReadTransaction trans(dir, __FILE__, __LINE__);
+  ReadTransaction trans(FROM_HERE, dir);
   // Lookup the parent folder by finding a child of the root.  We can't use
   // folder_id here, because it changed during the commit.
-  Id new_fid = dir->GetFirstChildId(&trans, id_factory_.root());
+  Id new_fid;
+  ASSERT_TRUE(dir->GetFirstChildId(&trans, id_factory_.root(), &new_fid));
   ASSERT_FALSE(new_fid.IsRoot());
   EXPECT_TRUE(new_fid.ServerKnows());
   EXPECT_FALSE(folder_id.ServerKnows());
@@ -334,7 +349,8 @@ TEST_F(ProcessCommitResponseCommandTest, NewFolderCommitKeepsChildOrder) {
   ASSERT_LT(0, parent.Get(BASE_VERSION))
       << "Parent should have a valid (positive) server base revision";
 
-  Id cid = dir->GetFirstChildId(&trans, new_fid);
+  Id cid;
+  ASSERT_TRUE(dir->GetFirstChildId(&trans, new_fid, &cid));
   int child_count = 0;
   // Now loop over all the children of the parent folder, verifying
   // that they are in their original order by checking to see that their
@@ -343,7 +359,8 @@ TEST_F(ProcessCommitResponseCommandTest, NewFolderCommitKeepsChildOrder) {
     SCOPED_TRACE(::testing::Message("Examining item #") << child_count);
     Entry c(&trans, syncable::GET_BY_ID, cid);
     DCHECK(c.good());
-    ASSERT_EQ(StringPrintf("Item %d", child_count), c.Get(NON_UNIQUE_NAME));
+    ASSERT_EQ(base::StringPrintf("Item %d", child_count),
+              c.Get(NON_UNIQUE_NAME));
     ASSERT_EQ(new_fid, c.Get(syncable::PARENT_ID));
     if (child_count < batch_size) {
       ASSERT_FALSE(c.Get(IS_UNSYNCED)) << "Item should be committed";
@@ -416,6 +433,7 @@ TEST_P(MixedResult, ExtensionActivity) {
     (*records)["xyz"].extension_id = "xyz";
     (*records)["xyz"].bookmark_write_count = 4U;
   }
+  ExpectGroupsToChange(command_, GROUP_UI, GROUP_DB);
   command_.ExecuteImpl(session());
 
   ExtensionsActivityMonitor::Records final_monitor_records;
