@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,9 +10,8 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/callback.h"
 #include "base/gtest_prod_util.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/synchronization/lock.h"
 #include "chrome/browser/sync/engine/conflict_resolver.h"
 #include "chrome/browser/sync/engine/syncer_types.h"
@@ -21,28 +20,14 @@
 #include "chrome/browser/sync/syncable/directory_event.h"
 #include "chrome/browser/sync/syncable/model_type.h"
 #include "chrome/browser/sync/util/extensions_activity_monitor.h"
-#include "chrome/common/deprecated/event_sys.h"
-#include "chrome/common/deprecated/event_sys-inl.h"
 
 namespace syncable {
-class Directory;
-class DirectoryManager;
 class Entry;
-class Id;
 class MutableEntry;
-class WriteTransaction;
 }  // namespace syncable
 
 namespace browser_sync {
 
-class ModelSafeWorker;
-class ServerConnectionManager;
-class SyncProcessState;
-class URLFactory;
-struct HttpResponse;
-
-static const int kDefaultMaxCommitBatchSize = 25;
-
 enum SyncerStep {
   SYNCER_BEGIN,
   CLEANUP_DISABLED_TYPES,
@@ -58,7 +43,7 @@ enum SyncerStep {
   BUILD_AND_PROCESS_CONFLICT_SETS,
   RESOLVE_CONFLICTS,
   APPLY_UPDATES_TO_RESOLVE_CONFLICTS,
-  CLEAR_PRIVATE_DATA,
+  CLEAR_PRIVATE_DATA,  // TODO(tim): Rename 'private' to 'user'.
   SYNCER_END
 };
 
@@ -76,54 +61,31 @@ class Syncer {
  public:
   typedef std::vector<int64> UnsyncedMetaHandles;
 
-  // The constructor may be called from a thread that is not the Syncer's
-  // dedicated thread, to allow some flexibility in the setup.
   Syncer();
-  ~Syncer();
+  virtual ~Syncer();
 
   // Called by other threads to tell the syncer to stop what it's doing
   // and return early from SyncShare, if possible.
   bool ExitRequested();
   void RequestEarlyExit();
 
-  // Cause one sync cycle to occur.  Like a good parent, it is the caller's
-  // responsibility to clean up after the syncer when it finishes a sync share
-  // operation and honor server mandated throttles.
-  void SyncShare(sessions::SyncSession* session);
-
-  // Limit the batch size of commit operations to a specified number of items.
-  void set_max_commit_batch_size(int x) { max_commit_batch_size_ = x; }
+  // Runs a sync cycle from |first_step| to |last_step|.
+  virtual void SyncShare(sessions::SyncSession* session,
+                         SyncerStep first_step,
+                         SyncerStep last_step);
 
  private:
-  void RequestNudge(int milliseconds);
-
   // Implements the PROCESS_CLIENT_COMMAND syncer step.
-  void ProcessClientCommand(sessions::SyncSession *session);
-
-  // This is the bottom-most SyncShare variant, and does not cause transient
-  // state to be reset in session.
-  // Like SyncShare(), but |first_step| and |last_step| are provided to perform
-  // a partial sync cycle, stopping after |last_step| is performed.
-  void SyncShare(sessions::SyncSession* session,
-                 SyncerStep first_step,
-                 SyncerStep last_step);
+  void ProcessClientCommand(sessions::SyncSession* session);
 
   bool early_exit_requested_;
   base::Lock early_exit_requested_lock_;
 
-  int32 max_commit_batch_size_;
-
   ConflictResolver resolver_;
 
-  // A callback hook used in unittests to simulate changes between conflict set
-  // building and conflict resolution.
-  Callback0::Type* pre_conflict_resolution_closure_;
-
   friend class SyncerTest;
   FRIEND_TEST_ALL_PREFIXES(SyncerTest, NameClashWithResolver);
   FRIEND_TEST_ALL_PREFIXES(SyncerTest, IllegalAndLegalUpdates);
-  FRIEND_TEST_ALL_PREFIXES(SusanDeletingTest,
-                           NewServerItemInAFolderHierarchyWeHaveDeleted3);
   FRIEND_TEST_ALL_PREFIXES(SyncerTest, TestCommitListOrderingAndNewParent);
   FRIEND_TEST_ALL_PREFIXES(SyncerTest,
                            TestCommitListOrderingAndNewParentAndChild);
@@ -146,30 +108,10 @@ class Syncer {
   DISALLOW_COPY_AND_ASSIGN(Syncer);
 };
 
-// Inline utility functions.
-
-// Given iterator ranges from two collections sorted according to a common
-// strict weak ordering, return true if the two ranges contain any common
-// items, and false if they do not. This function is in this header so that it
-// can be tested.
-template <class Iterator1, class Iterator2>
-bool SortedCollectionsIntersect(Iterator1 begin1, Iterator1 end1,
-                                Iterator2 begin2, Iterator2 end2) {
-  Iterator1 i1 = begin1;
-  Iterator2 i2 = begin2;
-  while (i1 != end1 && i2 != end2) {
-    if (*i1 == *i2)
-      return true;
-    if (*i1 > *i2)
-      ++i2;
-    else
-      ++i1;
-  }
-  return false;
-}
 // Utility function declarations.
 void CopyServerFields(syncable::Entry* src, syncable::MutableEntry* dest);
 void ClearServerData(syncable::MutableEntry* entry);
+const char* SyncerStepToString(const SyncerStep);
 
 }  // namespace browser_sync
 
