@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -29,11 +29,11 @@ UpdateApplicator::UpdateApplicator(ConflictResolver* resolver,
       pointer_(begin),
       group_filter_(group_filter),
       progress_(false),
-      routing_info_(routes) {
-    size_t item_count = end - begin;
-    VLOG(1) << "UpdateApplicator created for " << item_count << " items.";
-    successful_ids_.reserve(item_count);
-  }
+      routing_info_(routes),
+      application_results_(end - begin) {
+  size_t item_count = end - begin;
+  DVLOG(1) << "UpdateApplicator created for " << item_count << " items.";
+}
 
 UpdateApplicator::~UpdateApplicator() {
 }
@@ -48,12 +48,12 @@ bool UpdateApplicator::AttemptOneApplication(
     if (!progress_)
       return false;
 
-    VLOG(1) << "UpdateApplicator doing additional pass.";
+    DVLOG(1) << "UpdateApplicator doing additional pass.";
     pointer_ = begin_;
     progress_ = false;
 
     // Clear the tracked failures to avoid double-counting.
-    conflicting_ids_.clear();
+    application_results_.ClearConflicts();
   }
 
   syncable::Entry read_only(trans, syncable::GET_BY_HANDLE, *pointer_);
@@ -69,18 +69,22 @@ bool UpdateApplicator::AttemptOneApplication(
     case SUCCESS:
       Advance();
       progress_ = true;
-      successful_ids_.push_back(entry.Get(syncable::ID));
+      application_results_.AddSuccess(entry.Get(syncable::ID));
       break;
     case CONFLICT:
       pointer_++;
-      conflicting_ids_.push_back(entry.Get(syncable::ID));
+      application_results_.AddConflict(entry.Get(syncable::ID));
+      break;
+    case CONFLICT_ENCRYPTION:
+      pointer_++;
+      application_results_.AddEncryptionConflict(entry.Get(syncable::ID));
       break;
     default:
       NOTREACHED();
       break;
   }
-  VLOG(1) << "Apply Status for " << entry.Get(syncable::META_HANDLE)
-          << " is " << updateResponse;
+  DVLOG(1) << "Apply Status for " << entry.Get(syncable::META_HANDLE)
+           << " is " << updateResponse;
 
   return true;
 }
@@ -91,15 +95,26 @@ void UpdateApplicator::Advance() {
 }
 
 bool UpdateApplicator::SkipUpdate(const syncable::Entry& entry) {
-  ModelSafeGroup g =
-      GetGroupForModelType(entry.GetServerModelType(), routing_info_);
-  if (g != group_filter_)
+  syncable::ModelType type = entry.GetServerModelType();
+  ModelSafeGroup g = GetGroupForModelType(type, routing_info_);
+  // The set of updates passed to the UpdateApplicator should already
+  // be group-filtered.
+  if (g != group_filter_) {
+    NOTREACHED();
     return true;
+  }
+  if (g == GROUP_PASSIVE &&
+      !routing_info_.count(type) &&
+      type != syncable::UNSPECIFIED &&
+      type != syncable::TOP_LEVEL_FOLDER) {
+    DVLOG(1) << "Skipping update application, type not permitted.";
+    return true;
+  }
   return false;
 }
 
 bool UpdateApplicator::AllUpdatesApplied() const {
-  return conflicting_ids_.empty() && begin_ == end_;
+  return application_results_.no_conflicts() && begin_ == end_;
 }
 
 void UpdateApplicator::SaveProgressIntoSessionState(
@@ -108,15 +123,57 @@ void UpdateApplicator::SaveProgressIntoSessionState(
   DCHECK(begin_ == end_ || ((pointer_ == end_) && !progress_))
       << "SaveProgress called before updates exhausted.";
 
+  application_results_.SaveProgress(conflict_progress, update_progress);
+}
+
+UpdateApplicator::ResultTracker::ResultTracker(size_t num_results) {
+  successful_ids_.reserve(num_results);
+}
+
+UpdateApplicator::ResultTracker::~ResultTracker() {
+}
+
+void UpdateApplicator::ResultTracker::AddConflict(syncable::Id id) {
+  conflicting_ids_.push_back(id);
+}
+
+void UpdateApplicator::ResultTracker::AddEncryptionConflict(syncable::Id id) {
+  encryption_conflict_ids_.push_back(id);
+}
+
+void UpdateApplicator::ResultTracker::AddSuccess(syncable::Id id) {
+  successful_ids_.push_back(id);
+}
+
+void UpdateApplicator::ResultTracker::SaveProgress(
+    sessions::ConflictProgress* conflict_progress,
+    sessions::UpdateProgress* update_progress) {
   vector<syncable::Id>::const_iterator i;
   for (i = conflicting_ids_.begin(); i != conflicting_ids_.end(); ++i) {
     conflict_progress->AddConflictingItemById(*i);
     update_progress->AddAppliedUpdate(CONFLICT, *i);
   }
+  for (i = encryption_conflict_ids_.begin();
+       i != encryption_conflict_ids_.end(); ++i) {
+    // Encryption conflicts should not put the syncer into a stuck state. We
+    // mark as conflict, so that we reattempt to apply updates, but add it to
+    // the list of nonblocking conflicts instead of normal conflicts.
+    conflict_progress->AddNonblockingConflictingItemById(*i);
+    update_progress->AddAppliedUpdate(CONFLICT, *i);
+  }
   for (i = successful_ids_.begin(); i != successful_ids_.end(); ++i) {
     conflict_progress->EraseConflictingItemById(*i);
     update_progress->AddAppliedUpdate(SUCCESS, *i);
   }
 }
 
+void UpdateApplicator::ResultTracker::ClearConflicts() {
+  conflicting_ids_.clear();
+  encryption_conflict_ids_.clear();
+}
+
+bool UpdateApplicator::ResultTracker::no_conflicts() const {
+  return conflicting_ids_.empty();
+}
+
 }  // namespace browser_sync
