@@ -1,22 +1,30 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
+
 #include "chrome/browser/sync/glue/bookmark_change_processor.h"
 
 #include <stack>
 #include <vector>
 
+#include "base/location.h"
 #include "base/string16.h"
 #include "base/string_util.h"
-
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/bookmarks/bookmark_utils.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/favicon_service.h"
+#include "chrome/browser/favicon/favicon_service.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/sync/internal_api/change_record.h"
+#include "chrome/browser/sync/internal_api/read_node.h"
+#include "chrome/browser/sync/internal_api/write_node.h"
+#include "chrome/browser/sync/internal_api/write_transaction.h"
 #include "chrome/browser/sync/profile_sync_service.h"
-#include "gfx/codec/png_codec.h"
+#include "content/public/browser/browser_thread.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "ui/gfx/codec/png_codec.h"
+
+using content::BrowserThread;
 
 namespace browser_sync {
 
@@ -53,7 +61,7 @@ void BookmarkChangeProcessor::UpdateSyncNodeProperties(
   dst->SetIsFolder(src->is_folder());
   dst->SetTitle(UTF16ToWideHack(src->GetTitle()));
   if (!src->is_folder())
-    dst->SetURL(src->GetURL());
+    dst->SetURL(src->url());
   SetSyncNodeFavicon(src, model, dst);
 }
 
@@ -61,7 +69,7 @@ void BookmarkChangeProcessor::UpdateSyncNodeProperties(
 void BookmarkChangeProcessor::EncodeFavicon(const BookmarkNode* src,
                                             BookmarkModel* model,
                                             std::vector<unsigned char>* dst) {
-  const SkBitmap& favicon = model->GetFavIcon(src);
+  const SkBitmap& favicon = model->GetFavicon(src);
 
   dst->clear();
 
@@ -84,7 +92,7 @@ void BookmarkChangeProcessor::RemoveOneSyncNode(
     return;
   }
   // This node should have no children.
-  DCHECK(sync_node.GetFirstChildId() == sync_api::kInvalidId);
+  DCHECK(!sync_node.HasChildren());
   // Remove association and delete the sync node.
   model_associator_->Disassociate(sync_node.GetId());
   sync_node.Remove();
@@ -92,10 +100,10 @@ void BookmarkChangeProcessor::RemoveOneSyncNode(
 
 void BookmarkChangeProcessor::RemoveSyncNodeHierarchy(
     const BookmarkNode* topmost) {
-  sync_api::WriteTransaction trans(share_handle());
+  sync_api::WriteTransaction trans(FROM_HERE, share_handle());
 
   // Later logic assumes that |topmost| has been unlinked.
-  DCHECK(!topmost->GetParent());
+  DCHECK(topmost->is_root());
 
   // A BookmarkModel deletion event means that |node| and all its children were
   // deleted. Sync backend expects children to be deleted individually, so we do
@@ -107,19 +115,19 @@ void BookmarkChangeProcessor::RemoveSyncNodeHierarchy(
   int index = 0;
   while (node) {
     // The top of |index_stack| should always be |node|'s index.
-    DCHECK(!node->GetParent() || (node->GetParent()->IndexOfChild(node) ==
+    DCHECK(node->is_root() || (node->parent()->GetIndexOf(node) ==
       index_stack.top()));
-    if (index == node->GetChildCount()) {
+    if (index == node->child_count()) {
       // If we've processed all of |node|'s children, delete |node| and move
       // on to its successor.
       RemoveOneSyncNode(&trans, node);
-      node = node->GetParent();
+      node = node->parent();
       index = index_stack.top() + 1;      // (top() + 0) was what we removed.
       index_stack.pop();
     } else {
       // If |node| has an unprocessed child, process it next after pushing the
       // current state onto the stack.
-      DCHECK_LT(index, node->GetChildCount());
+      DCHECK_LT(index, node->child_count());
       index_stack.push(index);
       node = node->GetChild(index);
       index = 0;
@@ -128,7 +136,8 @@ void BookmarkChangeProcessor::RemoveSyncNodeHierarchy(
   DCHECK(index_stack.empty());  // Nothing should be left on the stack.
 }
 
-void BookmarkChangeProcessor::Loaded(BookmarkModel* model) {
+void BookmarkChangeProcessor::Loaded(BookmarkModel* model,
+                                     bool ids_reassigned) {
   NOTREACHED();
 }
 
@@ -145,7 +154,7 @@ void BookmarkChangeProcessor::BookmarkNodeAdded(BookmarkModel* model,
   DCHECK(share_handle());
 
   // Acquire a scoped write lock via a transaction.
-  sync_api::WriteTransaction trans(share_handle());
+  sync_api::WriteTransaction trans(FROM_HERE, share_handle());
 
   CreateSyncNode(parent, model, index, &trans, model_associator_,
                  error_handler());
@@ -163,8 +172,7 @@ int64 BookmarkChangeProcessor::CreateSyncNode(const BookmarkNode* parent,
   sync_api::WriteNode sync_child(trans);
 
   // Actually create the node with the appropriate initial position.
-  if (!PlaceSyncNode(CREATE, parent, index, trans, &sync_child, associator,
-                     error_handler)) {
+  if (!PlaceSyncNode(CREATE, parent, index, trans, &sync_child, associator)) {
     error_handler->OnUnrecoverableError(FROM_HERE,
         "Sync node creation failed; recovery unlikely");
     return sync_api::kInvalidId;
@@ -189,16 +197,16 @@ void BookmarkChangeProcessor::BookmarkNodeRemoved(BookmarkModel* model,
 }
 
 void BookmarkChangeProcessor::BookmarkNodeChanged(BookmarkModel* model,
-                                                    const BookmarkNode* node) {
+                                                  const BookmarkNode* node) {
   DCHECK(running());
   // We shouldn't see changes to the top-level nodes.
-  if (node == model->GetBookmarkBarNode() || node == model->other_node()) {
+  if (model->is_permanent_node(node)) {
     NOTREACHED() << "Saw update to permanent node!";
     return;
   }
 
   // Acquire a scoped write lock via a transaction.
-  sync_api::WriteTransaction trans(share_handle());
+  sync_api::WriteTransaction trans(FROM_HERE, share_handle());
 
   // Lookup the sync node that's associated with |node|.
   sync_api::WriteNode sync_node(&trans);
@@ -212,10 +220,10 @@ void BookmarkChangeProcessor::BookmarkNodeChanged(BookmarkModel* model,
   DCHECK_EQ(sync_node.GetIsFolder(), node->is_folder());
   DCHECK_EQ(model_associator_->GetChromeNodeFromSyncId(
             sync_node.GetParentId()),
-            node->GetParent());
+            node->parent());
   // This node's index should be one more than the predecessor's index.
-  DCHECK_EQ(node->GetParent()->IndexOfChild(node),
-            CalculateBookmarkModelInsertionIndex(node->GetParent(),
+  DCHECK_EQ(node->parent()->GetIndexOf(node),
+            CalculateBookmarkModelInsertionIndex(node->parent(),
                                                  &sync_node));
 }
 
@@ -226,13 +234,13 @@ void BookmarkChangeProcessor::BookmarkNodeMoved(BookmarkModel* model,
   DCHECK(running());
   const BookmarkNode* child = new_parent->GetChild(new_index);
   // We shouldn't see changes to the top-level nodes.
-  if (child == model->GetBookmarkBarNode() || child == model->other_node()) {
+  if (model->is_permanent_node(child)) {
     NOTREACHED() << "Saw update to permanent node!";
     return;
   }
 
   // Acquire a scoped write lock via a transaction.
-  sync_api::WriteTransaction trans(share_handle());
+  sync_api::WriteTransaction trans(FROM_HERE, share_handle());
 
   // Lookup the sync node that's associated with |child|.
   sync_api::WriteNode sync_node(&trans);
@@ -242,14 +250,15 @@ void BookmarkChangeProcessor::BookmarkNodeMoved(BookmarkModel* model,
   }
 
   if (!PlaceSyncNode(MOVE, new_parent, new_index, &trans, &sync_node,
-                     model_associator_, error_handler())) {
+                     model_associator_)) {
     error_handler()->OnUnrecoverableError(FROM_HERE, std::string());
     return;
   }
 }
 
-void BookmarkChangeProcessor::BookmarkNodeFavIconLoaded(BookmarkModel* model,
-      const BookmarkNode* node) {
+void BookmarkChangeProcessor::BookmarkNodeFaviconChanged(
+    BookmarkModel* model,
+    const BookmarkNode* node) {
   DCHECK(running());
   BookmarkNodeChanged(model, node);
 }
@@ -258,11 +267,11 @@ void BookmarkChangeProcessor::BookmarkNodeChildrenReordered(
     BookmarkModel* model, const BookmarkNode* node) {
 
   // Acquire a scoped write lock via a transaction.
-  sync_api::WriteTransaction trans(share_handle());
+  sync_api::WriteTransaction trans(FROM_HERE, share_handle());
 
   // The given node's children got reordered. We need to reorder all the
   // children of the corresponding sync node.
-  for (int i = 0; i < node->GetChildCount(); ++i) {
+  for (int i = 0; i < node->child_count(); ++i) {
     sync_api::WriteNode sync_child(&trans);
     if (!model_associator_->InitSyncNodeFromChromeId(node->GetChild(i)->id(),
                                                      &sync_child)) {
@@ -273,7 +282,7 @@ void BookmarkChangeProcessor::BookmarkNodeChildrenReordered(
               model_associator_->GetSyncIdFromChromeId(node->id()));
 
     if (!PlaceSyncNode(MOVE, node, i, &trans, &sync_child,
-                       model_associator_, error_handler())) {
+                       model_associator_)) {
       error_handler()->OnUnrecoverableError(FROM_HERE, std::string());
       return;
     }
@@ -283,12 +292,10 @@ void BookmarkChangeProcessor::BookmarkNodeChildrenReordered(
 // static
 bool BookmarkChangeProcessor::PlaceSyncNode(MoveOrCreate operation,
       const BookmarkNode* parent, int index, sync_api::WriteTransaction* trans,
-      sync_api::WriteNode* dst, BookmarkModelAssociator* associator,
-      UnrecoverableErrorHandler* error_handler) {
+      sync_api::WriteNode* dst, BookmarkModelAssociator* associator) {
   sync_api::ReadNode sync_parent(trans);
   if (!associator->InitSyncNodeFromChromeId(parent->id(), &sync_parent)) {
     LOG(WARNING) << "Parent lookup failed";
-    error_handler->OnUnrecoverableError(FROM_HERE, std::string());
     return false;
   }
 
@@ -342,8 +349,8 @@ int BookmarkChangeProcessor::CalculateBookmarkModelInsertionIndex(
   const BookmarkNode* predecessor =
       model_associator_->GetChromeNodeFromSyncId(predecessor_id);
   DCHECK(predecessor);
-  DCHECK_EQ(predecessor->GetParent(), parent);
-  return parent->IndexOfChild(predecessor) + 1;
+  DCHECK_EQ(predecessor->parent(), parent);
+  return parent->GetIndexOf(predecessor) + 1;
 }
 
 // ApplyModelChanges is called by the sync backend after changes have been made
@@ -351,8 +358,7 @@ int BookmarkChangeProcessor::CalculateBookmarkModelInsertionIndex(
 // model.
 void BookmarkChangeProcessor::ApplyChangesFromSyncModel(
     const sync_api::BaseTransaction* trans,
-    const sync_api::SyncManager::ChangeRecord* changes,
-    int change_count) {
+    const sync_api::ImmutableChangeRecordList& changes) {
   if (!running())
     return;
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
@@ -382,43 +388,52 @@ void BookmarkChangeProcessor::ApplyChangesFromSyncModel(
   // A parent to hold nodes temporarily orphaned by parent deletion.  It is
   // lazily created inside the loop.
   const BookmarkNode* foster_parent = NULL;
-  for (int i = 0; i < change_count; ++i) {
+
+  // Whether we have passed all the deletes (which should be at the
+  // front of the list).
+  bool passed_deletes = false;
+  for (sync_api::ChangeRecordList::const_iterator it =
+           changes.Get().begin(); it != changes.Get().end(); ++it) {
     const BookmarkNode* dst =
-        model_associator_->GetChromeNodeFromSyncId(changes[i].id);
+        model_associator_->GetChromeNodeFromSyncId(it->id);
     // Ignore changes to the permanent top-level nodes.  We only care about
     // their children.
-    if ((dst == model->GetBookmarkBarNode()) || (dst == model->other_node()))
+    if (model->is_permanent_node(dst))
       continue;
-    if (changes[i].action ==
-        sync_api::SyncManager::ChangeRecord::ACTION_DELETE) {
+    if (it->action ==
+        sync_api::ChangeRecord::ACTION_DELETE) {
       // Deletions should always be at the front of the list.
-      DCHECK(i == 0 || changes[i-1].action == changes[i].action);
+      DCHECK(!passed_deletes);
       // Children of a deleted node should not be deleted; they may be
       // reparented by a later change record.  Move them to a temporary place.
-      DCHECK(dst) << "Could not find node to be deleted";
-      const BookmarkNode* parent = dst->GetParent();
-      if (dst->GetChildCount()) {
+      if (!dst) // Can't do anything if we can't find the chrome node.
+        continue;
+      const BookmarkNode* parent = dst->parent();
+      if (!dst->empty()) {
         if (!foster_parent) {
-          foster_parent = model->AddGroup(model->other_node(),
-                                          model->other_node()->GetChildCount(),
-                                          string16());
+          foster_parent = model->AddFolder(model->other_node(),
+                                           model->other_node()->child_count(),
+                                           string16());
         }
-        for (int i = dst->GetChildCount() - 1; i >= 0; --i) {
+        for (int i = dst->child_count() - 1; i >= 0; --i) {
           model->Move(dst->GetChild(i), foster_parent,
-                      foster_parent->GetChildCount());
+                      foster_parent->child_count());
         }
       }
-      DCHECK_EQ(dst->GetChildCount(), 0) << "Node being deleted has children";
-      model_associator_->Disassociate(changes[i].id);
-      model->Remove(parent, parent->IndexOfChild(dst));
+      DCHECK_EQ(dst->child_count(), 0) << "Node being deleted has children";
+      model_associator_->Disassociate(it->id);
+      int index = parent->GetIndexOf(dst);
+      if (index > -1)
+        model->Remove(parent, index);
       dst = NULL;
     } else {
-      DCHECK_EQ((changes[i].action ==
-          sync_api::SyncManager::ChangeRecord::ACTION_ADD), (dst == NULL))
+      DCHECK_EQ((it->action ==
+          sync_api::ChangeRecord::ACTION_ADD), (dst == NULL))
           << "ACTION_ADD should be seen if and only if the node is unknown.";
+      passed_deletes = true;
 
       sync_api::ReadNode src(trans);
-      if (!src.InitByIdLookup(changes[i].id)) {
+      if (!src.InitByIdLookup(it->id)) {
         error_handler()->OnUnrecoverableError(FROM_HERE,
             "ApplyModelChanges was passed a bad ID");
         return;
@@ -430,12 +445,15 @@ void BookmarkChangeProcessor::ApplyChangesFromSyncModel(
   // Clean up the temporary node.
   if (foster_parent) {
     // There should be no nodes left under the foster parent.
-    DCHECK_EQ(foster_parent->GetChildCount(), 0);
-    model->Remove(foster_parent->GetParent(),
-                  foster_parent->GetParent()->IndexOfChild(foster_parent));
+    DCHECK_EQ(foster_parent->child_count(), 0);
+    model->Remove(foster_parent->parent(),
+                  foster_parent->parent()->GetIndexOf(foster_parent));
     foster_parent = NULL;
   }
 
+  // The visibility of the mobile node may need to change.
+  model_associator_->UpdatePermanentNodeVisibility();
+
   // We are now ready to hear about bookmarks changes again.
   model->AddObserver(this);
 }
@@ -470,9 +488,9 @@ const BookmarkNode* BookmarkChangeProcessor::CreateOrUpdateBookmarkNode(
 
     if (!src->GetIsFolder())
       model->SetURL(dst, src->GetURL());
-    model->SetTitle(dst, WideToUTF16Hack(src->GetTitle()));
+    model->SetTitle(dst, UTF8ToUTF16(src->GetTitle()));
 
-    SetBookmarkFavicon(src, dst, model->profile());
+    SetBookmarkFavicon(src, dst, model);
   }
 
   return dst;
@@ -487,17 +505,17 @@ const BookmarkNode* BookmarkChangeProcessor::CreateBookmarkNode(
     BookmarkModel* model,
     int index) {
   DCHECK(parent);
-  DCHECK(index >= 0 && index <= parent->GetChildCount());
+  DCHECK(index >= 0 && index <= parent->child_count());
 
   const BookmarkNode* node;
   if (sync_node->GetIsFolder()) {
-    node = model->AddGroup(parent, index,
-                           WideToUTF16Hack(sync_node->GetTitle()));
+    node = model->AddFolder(parent, index,
+                            UTF8ToUTF16(sync_node->GetTitle()));
   } else {
     node = model->AddURL(parent, index,
-                         WideToUTF16Hack(sync_node->GetTitle()),
+                         UTF8ToUTF16(sync_node->GetTitle()),
                          sync_node->GetURL());
-    SetBookmarkFavicon(sync_node, node, model->profile());
+    SetBookmarkFavicon(sync_node, node, model);
   }
   return node;
 }
@@ -507,13 +525,14 @@ const BookmarkNode* BookmarkChangeProcessor::CreateBookmarkNode(
 bool BookmarkChangeProcessor::SetBookmarkFavicon(
     sync_api::BaseNode* sync_node,
     const BookmarkNode* bookmark_node,
-    Profile* profile) {
+    BookmarkModel* bookmark_model) {
   std::vector<unsigned char> icon_bytes_vector;
   sync_node->GetFaviconBytes(&icon_bytes_vector);
   if (icon_bytes_vector.empty())
     return false;
 
-  ApplyBookmarkFavicon(bookmark_node, profile, icon_bytes_vector);
+  ApplyBookmarkFavicon(bookmark_node, bookmark_model->profile(),
+                       icon_bytes_vector);
 
   return true;
 }
@@ -529,17 +548,18 @@ void BookmarkChangeProcessor::ApplyBookmarkFavicon(
   // destination URL, which is not correct, but since the favicon URL
   // is used as a key in the history's thumbnail DB, this gives us a value
   // which does not collide with others.
-  GURL fake_icon_url = bookmark_node->GetURL();
+  GURL fake_icon_url = bookmark_node->url();
 
   HistoryService* history =
       profile->GetHistoryService(Profile::EXPLICIT_ACCESS);
   FaviconService* favicon_service =
       profile->GetFaviconService(Profile::EXPLICIT_ACCESS);
 
-  history->AddPage(bookmark_node->GetURL(), history::SOURCE_SYNCED);
-  favicon_service->SetFavicon(bookmark_node->GetURL(),
+  history->AddPageNoVisitForBookmark(bookmark_node->url());
+  favicon_service->SetFavicon(bookmark_node->url(),
                               fake_icon_url,
-                              icon_bytes_vector);
+                              icon_bytes_vector,
+                              history::FAVICON);
 }
 
 // static
