@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,13 +8,16 @@
 
 #include <vector>
 
-#include "chrome/browser/bookmarks/bookmark_model.h"
-#include "chrome/browser/sync/engine/syncapi.h"
-#include "chrome/browser/sync/glue/change_processor.h"
+#include "base/compiler_specific.h"
+#include "chrome/browser/bookmarks/bookmark_model_observer.h"
 #include "chrome/browser/sync/glue/bookmark_model_associator.h"
+#include "chrome/browser/sync/glue/change_processor.h"
 #include "chrome/browser/sync/glue/sync_backend_host.h"
 
-class ProfileSyncService;
+namespace sync_api {
+class WriteNode;
+class WriteTransaction;
+}  // namespace sync_api
 
 namespace browser_sync {
 
@@ -31,33 +34,32 @@ class BookmarkChangeProcessor : public BookmarkModelObserver,
 
   // BookmarkModelObserver implementation.
   // BookmarkModel -> sync_api model change application.
-  virtual void Loaded(BookmarkModel* model);
-  virtual void BookmarkModelBeingDeleted(BookmarkModel* model);
+  virtual void Loaded(BookmarkModel* model, bool ids_reassigned) OVERRIDE;
+  virtual void BookmarkModelBeingDeleted(BookmarkModel* model) OVERRIDE;
   virtual void BookmarkNodeMoved(BookmarkModel* model,
                                  const BookmarkNode* old_parent,
                                  int old_index,
                                  const BookmarkNode* new_parent,
-                                 int new_index);
+                                 int new_index) OVERRIDE;
   virtual void BookmarkNodeAdded(BookmarkModel* model,
                                  const BookmarkNode* parent,
-                                 int index);
+                                 int index) OVERRIDE;
   virtual void BookmarkNodeRemoved(BookmarkModel* model,
                                    const BookmarkNode* parent,
                                    int index,
-                                   const BookmarkNode* node);
+                                   const BookmarkNode* node) OVERRIDE;
   virtual void BookmarkNodeChanged(BookmarkModel* model,
-                                   const BookmarkNode* node);
-  virtual void BookmarkNodeFavIconLoaded(BookmarkModel* model,
-                                         const BookmarkNode* node);
+                                   const BookmarkNode* node) OVERRIDE;
+  virtual void BookmarkNodeFaviconChanged(BookmarkModel* model,
+                                          const BookmarkNode* node) OVERRIDE;
   virtual void BookmarkNodeChildrenReordered(BookmarkModel* model,
-                                             const BookmarkNode* node);
+                                             const BookmarkNode* node) OVERRIDE;
 
   // The change processor implementation, responsible for applying changes from
   // the sync model to the bookmarks model.
   virtual void ApplyChangesFromSyncModel(
       const sync_api::BaseTransaction* trans,
-      const sync_api::SyncManager::ChangeRecord* changes,
-      int change_count);
+      const sync_api::ImmutableChangeRecordList& changes) OVERRIDE;
 
   // The following methods are static and hence may be invoked at any time,
   // and do not depend on having a running ChangeProcessor.
@@ -75,7 +77,7 @@ class BookmarkChangeProcessor : public BookmarkModelObserver,
   // for the bookmark in question.
   static bool SetBookmarkFavicon(sync_api::BaseNode* sync_node,
                                  const BookmarkNode* bookmark_node,
-                                 Profile* profile);
+                                 BookmarkModel* model);
 
   // Applies the favicon data in |icon_bytes_vector| to |bookmark_node|.
   // |profile| is the profile that contains the HistoryService and BookmarkModel
@@ -103,8 +105,8 @@ class BookmarkChangeProcessor : public BookmarkModelObserver,
                               UnrecoverableErrorHandler* error_handler);
 
  protected:
-  virtual void StartImpl(Profile* profile);
-  virtual void StopImpl();
+  virtual void StartImpl(Profile* profile) OVERRIDE;
+  virtual void StopImpl() OVERRIDE;
 
  private:
   enum MoveOrCreate {
@@ -140,8 +142,7 @@ class BookmarkChangeProcessor : public BookmarkModelObserver,
                             int index,
                             sync_api::WriteTransaction* trans,
                             sync_api::WriteNode* dst,
-                            BookmarkModelAssociator* associator,
-                            UnrecoverableErrorHandler* error_handler);
+                            BookmarkModelAssociator* associator);
 
   // Copy properties (but not position) from |src| to |dst|.
   static void UpdateSyncNodeProperties(const BookmarkNode* src,
