@@ -1,12 +1,17 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/sync/glue/change_processor.h"
-#include "chrome/browser/profile.h"
+#include "chrome/browser/profiles/profile.h"
 
 namespace browser_sync {
 
+ChangeProcessor::ChangeProcessor(UnrecoverableErrorHandler* error_handler)
+    : running_(false),
+      error_handler_(error_handler),
+      share_handle_(NULL) {}
+
 ChangeProcessor::~ChangeProcessor() {
   DCHECK(!running_) << "ChangeProcessor dtor while running";
 }
@@ -27,4 +32,19 @@ void ChangeProcessor::Stop() {
   running_ = false;
 }
 
+bool ChangeProcessor::IsRunning() const {
+  return running_;
+}
+
+// Not implemented by default.
+void ChangeProcessor::CommitChangesFromSyncModel() {}
+
+UnrecoverableErrorHandler* ChangeProcessor::error_handler() const {
+  return error_handler_;
+}
+
+sync_api::UserShare* ChangeProcessor::share_handle() const {
+  return share_handle_;
+}
+
 }  // namespace browser_sync
