@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,20 +7,25 @@
 #pragma once
 
 #include <map>
+#include <string>
 
 #include "base/callback.h"
-#include "chrome/browser/browser_thread.h"
+#include "base/location.h"
+#include "base/message_loop_helpers.h"
 #include "chrome/browser/sync/engine/model_safe_worker.h"
+#include "chrome/browser/sync/internal_api/includes/unrecoverable_error_handler.h"
 #include "chrome/browser/sync/syncable/model_type.h"
-#include "chrome/browser/sync/unrecoverable_error_handler.h"
+#include "content/public/browser/browser_thread.h"
+
+class SyncError;
 
 namespace browser_sync {
 
 // Data type controllers need to be refcounted threadsafe, as they may
 // need to run model associator or change processor on other threads.
 class DataTypeController
-    : public base::RefCountedThreadSafe<DataTypeController,
-                                        BrowserThread::DeleteOnUIThread>,
+    : public base::RefCountedThreadSafe<
+          DataTypeController, content::BrowserThread::DeleteOnUIThread>,
       public UnrecoverableErrorHandler {
  public:
   enum State {
@@ -32,8 +37,10 @@ class DataTypeController
     ASSOCIATING,    // Model association is in progress.
     RUNNING,        // The controller is running and the data type is
                     // in sync with the cloud.
-    STOPPING        // The controller is in the process of stopping
+    STOPPING,       // The controller is in the process of stopping
                     // and is waiting for dependent services to stop.
+    DISABLED        // The controller was started but encountered an error
+                    // so it is disabled waiting for it to be stopped.
   };
 
   enum StartResult {
@@ -51,7 +58,7 @@ class DataTypeController
     MAX_START_RESULT
   };
 
-  typedef Callback1<StartResult>::Type StartCallback;
+  typedef base::Callback<void(StartResult, const SyncError&)> StartCallback;
 
   typedef std::map<syncable::ModelType,
                    scoped_refptr<DataTypeController> > TypeMap;
@@ -63,7 +70,7 @@ class DataTypeController
   // activation.  Upon completion, the start_callback will be invoked
   // on the UI thread.  See the StartResult enum above for details on the
   // possible start results.
-  virtual void Start(StartCallback* start_callback) = 0;
+  virtual void Start(const StartCallback& start_callback) = 0;
 
   // Synchronously stops the data type.  If called after Start() is
   // called but before the start callback is called, the start is
@@ -71,33 +78,26 @@ class DataTypeController
   // result.
   virtual void Stop() = 0;
 
-  // Returns true if the user has indicated that they want this data
-  // type to be enabled.
-  virtual bool enabled() = 0;
-
   // Unique model type for this data type controller.
-  virtual syncable::ModelType type() = 0;
+  virtual syncable::ModelType type() const = 0;
 
   // Name of this data type.  For logging purposes only.
-  virtual const char* name() const = 0;
+  virtual std::string name() const = 0;
 
   // The model safe group of this data type.  This should reflect the
   // thread that should be used to modify the data type's native
   // model.
-  virtual browser_sync::ModelSafeGroup model_safe_group() = 0;
+  virtual browser_sync::ModelSafeGroup model_safe_group() const = 0;
 
   // Current state of the data type controller.
-  virtual State state() = 0;
-
-  // TODO(sync): Make this protected.  It currently causes a build error on
-  // Linux to do so.
-  virtual ~DataTypeController() {}
+  virtual State state() const = 0;
 
  protected:
-  friend class base::RefCountedThreadSafe<DataTypeController>;
-  friend class BrowserThread;
-  friend class DeleteTask<DataTypeController>;
-  friend class ShutdownTask;
+  friend struct content::BrowserThread::DeleteOnThread<
+      content::BrowserThread::UI>;
+  friend class base::DeleteHelper<DataTypeController>;
+
+  virtual ~DataTypeController() {}
 };
 
 }  // namespace browser_sync
