@@ -7,22 +7,26 @@
 #include "base/message_loop.h"
 #include "base/message_loop_proxy.h"
 #include "base/string_number_conversions.h"
-#include "chrome/browser/browser_thread.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/common/content_client.h"
+#include "content/public/common/url_fetcher.h"
 #include "net/base/cookie_monster.h"
 #include "net/base/host_resolver.h"
 #include "net/base/load_flags.h"
+#include "net/base/net_errors.h"
 #include "net/http/http_cache.h"
 #include "net/http/http_network_layer.h"
 #include "net/http/http_response_headers.h"
 #include "net/proxy/proxy_service.h"
 #include "net/url_request/url_request_context.h"
 #include "net/url_request/url_request_status.h"
-#include "webkit/glue/webkit_glue.h"
+
+using content::BrowserThread;
 
 namespace browser_sync {
 
 HttpBridge::RequestContextGetter::RequestContextGetter(
-    URLRequestContextGetter* baseline_context_getter)
+    net::URLRequestContextGetter* baseline_context_getter)
     : baseline_context_getter_(baseline_context_getter) {
 }
 
@@ -49,7 +53,7 @@ HttpBridge::RequestContextGetter::GetIOMessageLoopProxy() const {
 }
 
 HttpBridgeFactory::HttpBridgeFactory(
-    URLRequestContextGetter* baseline_context_getter) {
+    net::URLRequestContextGetter* baseline_context_getter) {
   DCHECK(baseline_context_getter != NULL);
   request_context_getter_ =
       new HttpBridge::RequestContextGetter(baseline_context_getter);
@@ -73,12 +77,12 @@ HttpBridge::RequestContext::RequestContext(
     : baseline_context_(baseline_context) {
 
   // Create empty, in-memory cookie store.
-  cookie_store_ = new net::CookieMonster(NULL, NULL);
+  set_cookie_store(new net::CookieMonster(NULL, NULL));
 
   // We don't use a cache for bridged loads, but we do want to share proxy info.
-  host_resolver_ = baseline_context->host_resolver();
-  proxy_service_ = baseline_context->proxy_service();
-  ssl_config_service_ = baseline_context->ssl_config_service();
+  set_host_resolver(baseline_context->host_resolver());
+  set_proxy_service(baseline_context->proxy_service());
+  set_ssl_config_service(baseline_context->ssl_config_service());
 
   // We want to share the HTTP session data with the network layer factory,
   // which includes auth_cache for proxies.
@@ -87,7 +91,7 @@ HttpBridge::RequestContext::RequestContext(
   net::HttpNetworkSession* session =
       baseline_context->http_transaction_factory()->GetSession();
   DCHECK(session);
-  http_transaction_factory_ = net::HttpNetworkLayer::CreateFactory(session);
+  set_http_transaction_factory(new net::HttpNetworkLayer(session));
 
   // TODO(timsteele): We don't currently listen for pref changes of these
   // fields or CookiePolicy; I'm not sure we want to strictly follow the
@@ -96,29 +100,32 @@ HttpBridge::RequestContext::RequestContext(
   // should be tied to whatever the sync servers expect (if anything). These
   // fields should probably just be settable by sync backend; though we should
   // figure out if we need to give the user explicit control over policies etc.
-  accept_language_ = baseline_context->accept_language();
-  accept_charset_ = baseline_context->accept_charset();
+  set_accept_language(baseline_context->accept_language());
+  set_accept_charset(baseline_context->accept_charset());
 
   // We default to the browser's user agent. This can (and should) be overridden
   // with set_user_agent.
-  user_agent_ = webkit_glue::GetUserAgent(GURL());
+  set_user_agent(content::GetUserAgent(GURL()));
 
-  net_log_ = baseline_context->net_log();
+  set_net_log(baseline_context->net_log());
 }
 
 HttpBridge::RequestContext::~RequestContext() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  delete http_transaction_factory_;
+  delete http_transaction_factory();
 }
 
+HttpBridge::URLFetchState::URLFetchState() : url_poster(NULL),
+                                             aborted(false),
+                                             request_completed(false),
+                                             request_succeeded(false),
+                                             http_response_code(-1),
+                                             error_code(-1) {}
+HttpBridge::URLFetchState::~URLFetchState() {}
+
 HttpBridge::HttpBridge(HttpBridge::RequestContextGetter* context_getter)
     : context_getter_for_request_(context_getter),
-      url_poster_(NULL),
       created_on_loop_(MessageLoop::current()),
-      request_completed_(false),
-      request_succeeded_(false),
-      http_response_code_(-1),
-      os_error_code_(-1),
       http_post_completed_(false, false) {
 }
 
@@ -127,7 +134,10 @@ HttpBridge::~HttpBridge() {
 
 void HttpBridge::SetUserAgent(const char* user_agent) {
   DCHECK_EQ(MessageLoop::current(), created_on_loop_);
-  DCHECK(!request_completed_);
+  if (DCHECK_IS_ON()) {
+    base::AutoLock lock(fetch_state_lock_);
+    DCHECK(!fetch_state_.request_completed);
+  }
   context_getter_for_request_->set_user_agent(user_agent);
 }
 
@@ -139,7 +149,10 @@ void HttpBridge::SetExtraRequestHeaders(const char * headers) {
 
 void HttpBridge::SetURL(const char* url, int port) {
   DCHECK_EQ(MessageLoop::current(), created_on_loop_);
-  DCHECK(!request_completed_);
+  if (DCHECK_IS_ON()) {
+    base::AutoLock lock(fetch_state_lock_);
+    DCHECK(!fetch_state_.request_completed);
+  }
   DCHECK(url_for_request_.is_empty())
       << "HttpBridge::SetURL called more than once?!";
   GURL temp(url);
@@ -154,7 +167,10 @@ void HttpBridge::SetPostPayload(const char* content_type,
                                 int content_length,
                                 const char* content) {
   DCHECK_EQ(MessageLoop::current(), created_on_loop_);
-  DCHECK(!request_completed_);
+  if (DCHECK_IS_ON()) {
+    base::AutoLock lock(fetch_state_lock_);
+    DCHECK(!fetch_state_.request_completed);
+  }
   DCHECK(content_type_.empty()) << "Bridge payload already set.";
   DCHECK_GE(content_length, 0) << "Content length < 0";
   content_type_ = content_type;
@@ -168,80 +184,110 @@ void HttpBridge::SetPostPayload(const char* content_type,
   }
 }
 
-bool HttpBridge::MakeSynchronousPost(int* os_error_code, int* response_code) {
+bool HttpBridge::MakeSynchronousPost(int* error_code, int* response_code) {
   DCHECK_EQ(MessageLoop::current(), created_on_loop_);
-  DCHECK(!request_completed_);
+  if (DCHECK_IS_ON()) {
+    base::AutoLock lock(fetch_state_lock_);
+    DCHECK(!fetch_state_.request_completed);
+  }
   DCHECK(url_for_request_.is_valid()) << "Invalid URL for request";
   DCHECK(!content_type_.empty()) << "Payload not set";
 
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(this, &HttpBridge::CallMakeAsynchronousPost));
+  if (!BrowserThread::PostTask(
+          BrowserThread::IO, FROM_HERE,
+          base::Bind(&HttpBridge::CallMakeAsynchronousPost, this))) {
+    // This usually happens when we're in a unit test.
+    LOG(WARNING) << "Could not post CallMakeAsynchronousPost task";
+    return false;
+  }
 
-  if (!http_post_completed_.Wait())  // Block until network request completes.
-    NOTREACHED();                    // See OnURLFetchComplete.
+  // Block until network request completes or is aborted. See
+  // OnURLFetchComplete and Abort.
+  http_post_completed_.Wait();
 
-  DCHECK(request_completed_);
-  *os_error_code = os_error_code_;
-  *response_code = http_response_code_;
-  return request_succeeded_;
+  base::AutoLock lock(fetch_state_lock_);
+  DCHECK(fetch_state_.request_completed || fetch_state_.aborted);
+  *error_code = fetch_state_.error_code;
+  *response_code = fetch_state_.http_response_code;
+  return fetch_state_.request_succeeded;
 }
 
 void HttpBridge::MakeAsynchronousPost() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  DCHECK(!request_completed_);
+  base::AutoLock lock(fetch_state_lock_);
+  DCHECK(!fetch_state_.request_completed);
+  if (fetch_state_.aborted)
+    return;
 
-  url_poster_ = new URLFetcher(url_for_request_, URLFetcher::POST, this);
-  url_poster_->set_request_context(context_getter_for_request_);
-  url_poster_->set_upload_data(content_type_, request_content_);
-  url_poster_->set_extra_request_headers(extra_headers_);
-  url_poster_->set_load_flags(net::LOAD_DO_NOT_SEND_COOKIES);
-  url_poster_->Start();
+  fetch_state_.url_poster = content::URLFetcher::Create(
+      url_for_request_, content::URLFetcher::POST, this);
+  fetch_state_.url_poster->SetRequestContext(context_getter_for_request_);
+  fetch_state_.url_poster->SetUploadData(content_type_, request_content_);
+  fetch_state_.url_poster->SetExtraRequestHeaders(extra_headers_);
+  fetch_state_.url_poster->SetLoadFlags(net::LOAD_DO_NOT_SEND_COOKIES);
+  fetch_state_.url_poster->Start();
 }
 
 int HttpBridge::GetResponseContentLength() const {
   DCHECK_EQ(MessageLoop::current(), created_on_loop_);
-  DCHECK(request_completed_);
-  return response_content_.size();
+  base::AutoLock lock(fetch_state_lock_);
+  DCHECK(fetch_state_.request_completed);
+  return fetch_state_.response_content.size();
 }
 
 const char* HttpBridge::GetResponseContent() const {
   DCHECK_EQ(MessageLoop::current(), created_on_loop_);
-  DCHECK(request_completed_);
-  return response_content_.data();
+  base::AutoLock lock(fetch_state_lock_);
+  DCHECK(fetch_state_.request_completed);
+  return fetch_state_.response_content.data();
 }
 
 const std::string HttpBridge::GetResponseHeaderValue(
     const std::string& name) const {
 
   DCHECK_EQ(MessageLoop::current(), created_on_loop_);
-  DCHECK(request_completed_);
+  base::AutoLock lock(fetch_state_lock_);
+  DCHECK(fetch_state_.request_completed);
+
   std::string value;
-  response_headers_->EnumerateHeader(NULL, name, &value);
+  fetch_state_.response_headers->EnumerateHeader(NULL, name, &value);
   return value;
 }
 
-void HttpBridge::OnURLFetchComplete(const URLFetcher *source,
-                                    const GURL &url,
-                                    const net::URLRequestStatus &status,
-                                    int response_code,
-                                    const ResponseCookies &cookies,
-                                    const std::string &data) {
+void HttpBridge::Abort() {
+  base::AutoLock lock(fetch_state_lock_);
+  DCHECK(!fetch_state_.aborted);
+  if (fetch_state_.aborted || fetch_state_.request_completed)
+    return;
+
+  fetch_state_.aborted = true;
+  BrowserThread::DeleteSoon(BrowserThread::IO, FROM_HERE,
+                            fetch_state_.url_poster);
+  fetch_state_.url_poster = NULL;
+  fetch_state_.error_code = net::ERR_ABORTED;
+  http_post_completed_.Signal();
+}
+
+void HttpBridge::OnURLFetchComplete(const content::URLFetcher *source) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
+  base::AutoLock lock(fetch_state_lock_);
+  if (fetch_state_.aborted)
+    return;
 
-  request_completed_ = true;
-  request_succeeded_ = (net::URLRequestStatus::SUCCESS == status.status());
-  http_response_code_ = response_code;
-  os_error_code_ = status.os_error();
+  fetch_state_.request_completed = true;
+  fetch_state_.request_succeeded =
+      (net::URLRequestStatus::SUCCESS == source->GetStatus().status());
+  fetch_state_.http_response_code = source->GetResponseCode();
+  fetch_state_.error_code = source->GetStatus().error();
 
-  response_content_ = data;
-  response_headers_ = source->response_headers();
+  source->GetResponseAsString(&fetch_state_.response_content);
+  fetch_state_.response_headers = source->GetResponseHeaders();
 
   // End of the line for url_poster_. It lives only on the IO loop.
   // We defer deletion because we're inside a callback from a component of the
   // URLFetcher, so it seems most natural / "polite" to let the stack unwind.
-  MessageLoop::current()->DeleteSoon(FROM_HERE, url_poster_);
-  url_poster_ = NULL;
+  MessageLoop::current()->DeleteSoon(FROM_HERE, fetch_state_.url_poster);
+  fetch_state_.url_poster = NULL;
 
   // Wake the blocked syncer thread in MakeSynchronousPost.
   // WARNING: DONT DO ANYTHING AFTER THIS CALL! |this| may be deleted!
