@@ -8,18 +8,26 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
+#include "base/basictypes.h"
+#include "base/compiler_specific.h"
+#include "base/gtest_prod_util.h"
+#include "base/memory/ref_counted.h"
+#include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event.h"
-#include "chrome/browser/sync/engine/syncapi.h"
-#include "chrome/common/net/url_fetcher.h"
-#include "chrome/common/net/url_request_context_getter.h"
+#include "chrome/browser/sync/internal_api/http_post_provider_factory.h"
+#include "chrome/browser/sync/internal_api/http_post_provider_interface.h"
+#include "content/public/common/url_fetcher_delegate.h"
 #include "googleurl/src/gurl.h"
+#include "net/url_request/url_request_context_getter.h"
 #include "net/url_request/url_request_context.h"
-#include "testing/gtest/include/gtest/gtest_prod.h"
 
 class MessageLoop;
 class HttpBridgeTest;
 
+namespace net {
+class HttpResponseHeaders;
+}
+
 namespace browser_sync {
 
 // A bridge between the syncer and Chromium HTTP layers.
@@ -30,17 +38,11 @@ namespace browser_sync {
 // needs to stick around across context switches, etc.
 class HttpBridge : public base::RefCountedThreadSafe<HttpBridge>,
                    public sync_api::HttpPostProviderInterface,
-                   public URLFetcher::Delegate {
+                   public content::URLFetcherDelegate {
  public:
   // A request context used for HTTP requests bridged from the sync backend.
   // A bridged RequestContext has a dedicated in-memory cookie store and does
   // not use a cache. Thus the same type can be used for incognito mode.
-  // TODO(timsteele): We subclass here instead of add a factory method on
-  // ChromeURLRequestContext because:
-  // 1) we want the ability to set_user_agent
-  // 2) avoids ifdefs for now
-  // 3) not sure we want to strictly follow settings for cookie policy,
-  //    accept lang/charset, since changing these could break syncing.
   class RequestContext : public net::URLRequestContext {
    public:
     // |baseline_context| is used to obtain the accept-language,
@@ -53,7 +55,7 @@ class HttpBridge : public base::RefCountedThreadSafe<HttpBridge>,
     // the browser's UA string.
     void set_user_agent(const std::string& ua) { user_agent_ = ua; }
 
-    virtual const std::string& GetUserAgent(const GURL& url) const {
+    virtual const std::string& GetUserAgent(const GURL& url) const OVERRIDE {
       // If the user agent is set explicitly return that, otherwise call the
       // base class method to return default value.
       return user_agent_.empty() ?
@@ -62,7 +64,7 @@ class HttpBridge : public base::RefCountedThreadSafe<HttpBridge>,
 
    private:
     // The destructor MUST be called on the IO thread.
-    ~RequestContext();
+    virtual ~RequestContext();
 
     std::string user_agent_;
     net::URLRequestContext* baseline_context_;
@@ -71,25 +73,26 @@ class HttpBridge : public base::RefCountedThreadSafe<HttpBridge>,
   };
 
   // Lazy-getter for RequestContext objects.
-  class RequestContextGetter : public URLRequestContextGetter {
+  class RequestContextGetter : public net::URLRequestContextGetter {
    public:
     explicit RequestContextGetter(
-        URLRequestContextGetter* baseline_context_getter);
+        net::URLRequestContextGetter* baseline_context_getter);
 
     void set_user_agent(const std::string& ua) { user_agent_ = ua; }
     bool is_user_agent_set() const { return !user_agent_.empty(); }
 
-    // URLRequestContextGetter implementation.
-    virtual net::URLRequestContext* GetURLRequestContext();
-    virtual scoped_refptr<base::MessageLoopProxy> GetIOMessageLoopProxy() const;
+    // net::URLRequestContextGetter implementation.
+    virtual net::URLRequestContext* GetURLRequestContext() OVERRIDE;
+    virtual scoped_refptr<base::MessageLoopProxy>
+        GetIOMessageLoopProxy() const OVERRIDE;
 
    private:
-    ~RequestContextGetter() {}
+    virtual ~RequestContextGetter() {}
 
     // User agent to apply to the net::URLRequestContext.
     std::string user_agent_;
 
-    scoped_refptr<URLRequestContextGetter> baseline_context_getter_;
+    scoped_refptr<net::URLRequestContextGetter> baseline_context_getter_;
 
     // Lazily initialized by GetURLRequestContext().
     scoped_refptr<RequestContext> context_;
@@ -100,32 +103,29 @@ class HttpBridge : public base::RefCountedThreadSafe<HttpBridge>,
   explicit HttpBridge(RequestContextGetter* context);
 
   // sync_api::HttpPostProvider implementation.
-  virtual void SetUserAgent(const char* user_agent);
-  virtual void SetExtraRequestHeaders(const char* headers);
-  virtual void SetURL(const char* url, int port);
+  virtual void SetUserAgent(const char* user_agent) OVERRIDE;
+  virtual void SetExtraRequestHeaders(const char* headers) OVERRIDE;
+  virtual void SetURL(const char* url, int port) OVERRIDE;
   virtual void SetPostPayload(const char* content_type, int content_length,
-                              const char* content);
-  virtual bool MakeSynchronousPost(int* os_error_code, int* response_code);
+                              const char* content) OVERRIDE;
+  virtual bool MakeSynchronousPost(int* error_code,
+                                   int* response_code) OVERRIDE;
+  virtual void Abort() OVERRIDE;
 
   // WARNING: these response content methods are used to extract plain old data
   // and not null terminated strings, so you should make sure you have read
   // GetResponseContentLength() characters when using GetResponseContent. e.g
   // string r(b->GetResponseContent(), b->GetResponseContentLength()).
-  virtual int GetResponseContentLength() const;
-  virtual const char* GetResponseContent() const;
+  virtual int GetResponseContentLength() const OVERRIDE;
+  virtual const char* GetResponseContent() const OVERRIDE;
   virtual const std::string GetResponseHeaderValue(
-      const std::string& name) const;
+      const std::string& name) const OVERRIDE;
 
-  // URLFetcher::Delegate implementation.
-  virtual void OnURLFetchComplete(const URLFetcher* source,
-                                  const GURL& url,
-                                  const net::URLRequestStatus& status,
-                                  int response_code,
-                                  const ResponseCookies& cookies,
-                                  const std::string& data);
+  // content::URLFetcherDelegate implementation.
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
 #if defined(UNIT_TEST)
-  URLRequestContextGetter* GetRequestContextGetter() const {
+  net::URLRequestContextGetter* GetRequestContextGetter() const {
     return context_getter_for_request_;
   }
 #endif
@@ -150,13 +150,6 @@ class HttpBridge : public base::RefCountedThreadSafe<HttpBridge>,
   // RequestContext definition for details.
   scoped_refptr<RequestContextGetter> context_getter_for_request_;
 
-  // Our hook into the network layer is a URLFetcher. USED ONLY ON THE IO LOOP,
-  // so we can block created_on_loop_ while the fetch is in progress.
-  // NOTE: This is not a scoped_ptr for a reason. It must be deleted on the same
-  // thread that created it, which isn't the same thread |this| gets deleted on.
-  // We must manually delete url_poster_ on the IO loop.
-  URLFetcher* url_poster_;
-
   // The message loop of the thread we were created on. This is the thread that
   // will block on MakeSynchronousPost while the IO thread fetches data from
   // the network.
@@ -171,39 +164,64 @@ class HttpBridge : public base::RefCountedThreadSafe<HttpBridge>,
   std::string content_type_;
   std::string request_content_;
   std::string extra_headers_;
-  scoped_refptr<net::HttpResponseHeaders> response_headers_;
-
-  // Cached response data.
-  bool request_completed_;
-  bool request_succeeded_;
-  int http_response_code_;
-  int os_error_code_;
-  std::string response_content_;
 
   // A waitable event we use to provide blocking semantics to
   // MakeSynchronousPost. We block created_on_loop_ while the IO loop fetches
   // network request.
   base::WaitableEvent http_post_completed_;
 
+  struct URLFetchState {
+    URLFetchState();
+    ~URLFetchState();
+    // Our hook into the network layer is a URLFetcher. USED ONLY ON THE IO
+    // LOOP, so we can block created_on_loop_ while the fetch is in progress.
+    // NOTE: This is not a scoped_ptr for a reason. It must be deleted on the
+    // same thread that created it, which isn't the same thread |this| gets
+    // deleted on. We must manually delete url_poster_ on the IO loop.
+    content::URLFetcher* url_poster;
+
+    // Used to support 'Abort' functionality.
+    bool aborted;
+
+    // Cached response data.
+    bool request_completed;
+    bool request_succeeded;
+    int http_response_code;
+    int error_code;
+    std::string response_content;
+    scoped_refptr<net::HttpResponseHeaders> response_headers;
+  };
+
+  // This lock synchronizes use of state involved in the flow to fetch a URL
+  // using URLFetcher.  Because we can Abort() from any thread, for example,
+  // this flow needs to be synchronized to gracefully clean up URLFetcher and
+  // return appropriate values in |error_code|.
+  mutable base::Lock fetch_state_lock_;
+  URLFetchState fetch_state_;
+
   DISALLOW_COPY_AND_ASSIGN(HttpBridge);
 };
 
-class HttpBridgeFactory
-  : public sync_api::HttpPostProviderFactory {
+class HttpBridgeFactory : public sync_api::HttpPostProviderFactory {
  public:
-  explicit HttpBridgeFactory(URLRequestContextGetter* baseline_context_getter);
+  explicit HttpBridgeFactory(
+      net::URLRequestContextGetter* baseline_context_getter);
   virtual ~HttpBridgeFactory();
-  virtual sync_api::HttpPostProviderInterface* Create();
-  virtual void Destroy(sync_api::HttpPostProviderInterface* http);
+
+  // sync_api::HttpPostProviderFactory:
+  virtual sync_api::HttpPostProviderInterface* Create() OVERRIDE;
+  virtual void Destroy(sync_api::HttpPostProviderInterface* http) OVERRIDE;
+
  private:
   // This request context is built on top of the baseline context and shares
   // common components.
   HttpBridge::RequestContextGetter* GetRequestContextGetter();
+
   scoped_refptr<HttpBridge::RequestContextGetter> request_context_getter_;
+
   DISALLOW_COPY_AND_ASSIGN(HttpBridgeFactory);
 };
 
 }  //  namespace browser_sync
 
 #endif  // CHROME_BROWSER_SYNC_GLUE_HTTP_BRIDGE_H_
-
