@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,8 +7,11 @@
 #pragma once
 
 #include "base/basictypes.h"
+#include "base/synchronization/lock.h"
 #include "chrome/browser/sync/syncable/model_type.h"
 
+class SyncError;
+
 namespace sync_api {
 class BaseNode;
 }
@@ -27,10 +30,12 @@ class AssociatorInterface {
   // should be identical and corresponding. Returns true on
   // success. On failure of this step, we should abort the sync
   // operation and report an error to the user.
-  virtual bool AssociateModels() = 0;
+  // TODO(zea): return a SyncError instead of passing one in.
+  virtual bool AssociateModels(SyncError* error) = 0;
 
   // Clears all the associations between the chrome and sync models.
-  virtual bool DisassociateModels() = 0;
+  // TODO(zea): return a SyncError instead of passing one in.
+  virtual bool DisassociateModels(SyncError* error) = 0;
 
   // The has_nodes out parameter is set to true if the sync model has
   // nodes other than the permanent tagged nodes.  The method may
@@ -43,6 +48,12 @@ class AssociatorInterface {
   // method is only implemented for model associators that are invoked
   // off the main thread.
   virtual void AbortAssociation() = 0;
+
+  // Returns whether the datatype is ready for encryption/decryption if the
+  // sync service requires it.
+  // TODO(zea): This should be implemented automatically for each datatype, see
+  // http://crbug.com/76232.
+  virtual bool CryptoReadyIfNecessary() = 0;
 };
 
 // In addition to the generic methods, association can refer to operations
@@ -79,6 +90,31 @@ class PerDataTypeAssociatorInterface : public AssociatorInterface {
   virtual void Disassociate(int64 sync_id) = 0;
 };
 
+template <class Node, class IDType>
+class AbortablePerDataTypeAssociatorInterface
+    : public PerDataTypeAssociatorInterface<Node, IDType> {
+ public:
+  AbortablePerDataTypeAssociatorInterface() : pending_abort_(false) {}
+
+  // Implementation of AssociatorInterface methods.
+  virtual void AbortAssociation() {
+    base::AutoLock lock(pending_abort_lock_);
+    pending_abort_ = true;
+  }
+
+ protected:
+  // Overridable by tests.
+  virtual bool IsAbortPending() {
+    base::AutoLock lock(pending_abort_lock_);
+    return pending_abort_;
+  }
+
+  // Lock to ensure exclusive access to the pending_abort_ flag.
+  base::Lock pending_abort_lock_;
+  // Set to true if there's a pending abort.
+  bool pending_abort_;
+};
+
 }  // namespace browser_sync
 
 #endif  // CHROME_BROWSER_SYNC_GLUE_MODEL_ASSOCIATOR_H_
