@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,14 +6,21 @@
 
 #include <set>
 
-#include "base/stl_util-inl.h"
+#include "base/location.h"
+#include "base/stl_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/password_manager/password_store.h"
-#include "chrome/browser/sync/engine/syncapi.h"
+#include "chrome/browser/sync/api/sync_error.h"
+#include "chrome/browser/sync/internal_api/read_node.h"
+#include "chrome/browser/sync/internal_api/read_transaction.h"
+#include "chrome/browser/sync/internal_api/write_node.h"
+#include "chrome/browser/sync/internal_api/write_transaction.h"
 #include "chrome/browser/sync/profile_sync_service.h"
 #include "chrome/browser/sync/protocol/password_specifics.pb.h"
 #include "net/base/escape.h"
-#include "webkit/glue/password_form.h"
+#include "webkit/forms/password_form.h"
+
+using content::BrowserThread;
 
 namespace browser_sync {
 
@@ -38,105 +45,122 @@ PasswordModelAssociator::PasswordModelAssociator(
 
 PasswordModelAssociator::~PasswordModelAssociator() {}
 
-bool PasswordModelAssociator::AssociateModels() {
+bool PasswordModelAssociator::AssociateModels(SyncError* error) {
   DCHECK(expected_loop_ == MessageLoop::current());
   {
     base::AutoLock lock(abort_association_pending_lock_);
     abort_association_pending_ = false;
   }
 
-  sync_api::WriteTransaction trans(
-      sync_service_->backend()->GetUserShareHandle());
-  sync_api::ReadNode password_root(&trans);
-  if (!password_root.InitByTagLookup(kPasswordTag)) {
-    LOG(ERROR) << "Server did not create the top-level password node. We "
-               << "might be running against an out-of-date server.";
-    return false;
-  }
-
-  std::vector<webkit_glue::PasswordForm*> passwords;
+  // We must not be holding a transaction when we interact with the password
+  // store, as it can post tasks to the UI thread which can itself be blocked
+  // on our transaction, resulting in deadlock. (http://crbug.com/70658)
+  std::vector<webkit::forms::PasswordForm*> passwords;
   if (!password_store_->FillAutofillableLogins(&passwords) ||
       !password_store_->FillBlacklistLogins(&passwords)) {
     STLDeleteElements(&passwords);
-    LOG(ERROR) << "Could not get the password entries.";
+    error->Reset(FROM_HERE,
+                 "Could not get the password entries.",
+                 model_type());
     return false;
   }
 
   std::set<std::string> current_passwords;
   PasswordVector new_passwords;
   PasswordVector updated_passwords;
-
-  for (std::vector<webkit_glue::PasswordForm*>::iterator ix = passwords.begin();
-       ix != passwords.end(); ++ix) {
-    if (IsAbortPending())
+  {
+    sync_api::WriteTransaction trans(FROM_HERE, sync_service_->GetUserShare());
+    sync_api::ReadNode password_root(&trans);
+    if (!password_root.InitByTagLookup(kPasswordTag)) {
+      error->Reset(FROM_HERE,
+                   "Server did not create the top-level password node. We "
+                   "might be running against an out-of-date server.",
+                   model_type());
       return false;
-    std::string tag = MakeTag(**ix);
+    }
 
-    sync_api::ReadNode node(&trans);
-    if (node.InitByClientTagLookup(syncable::PASSWORDS, tag)) {
-      const sync_pb::PasswordSpecificsData& password =
-          node.GetPasswordSpecifics();
-      DCHECK_EQ(tag, MakeTag(password));
-
-      webkit_glue::PasswordForm new_password;
-
-      if (MergePasswords(password, **ix, &new_password)) {
-        sync_api::WriteNode write_node(&trans);
-        if (!write_node.InitByClientTagLookup(syncable::PASSWORDS, tag)) {
-          STLDeleteElements(&passwords);
-          LOG(ERROR) << "Failed to edit password sync node.";
-          return false;
-        }
-        WriteToSyncNode(new_password, &write_node);
-        updated_passwords.push_back(new_password);
-      }
-
-      Associate(&tag, node.GetId());
-    } else {
-      sync_api::WriteNode node(&trans);
-      if (!node.InitUniqueByCreation(syncable::PASSWORDS,
-                                     password_root, tag)) {
-        STLDeleteElements(&passwords);
-        LOG(ERROR) << "Failed to create password sync node.";
+    for (std::vector<webkit::forms::PasswordForm*>::iterator ix =
+             passwords.begin();
+         ix != passwords.end(); ++ix) {
+      if (IsAbortPending()) {
+        error->Reset(FROM_HERE, "Abort pending", model_type());
         return false;
       }
+      std::string tag = MakeTag(**ix);
 
-      WriteToSyncNode(**ix, &node);
+      sync_api::ReadNode node(&trans);
+      if (node.InitByClientTagLookup(syncable::PASSWORDS, tag)) {
+        const sync_pb::PasswordSpecificsData& password =
+            node.GetPasswordSpecifics();
+        DCHECK_EQ(tag, MakeTag(password));
 
-      Associate(&tag, node.GetId());
+        webkit::forms::PasswordForm new_password;
+
+        if (MergePasswords(password, **ix, &new_password)) {
+          sync_api::WriteNode write_node(&trans);
+          if (!write_node.InitByClientTagLookup(syncable::PASSWORDS, tag)) {
+            STLDeleteElements(&passwords);
+            error->Reset(FROM_HERE,
+                         "Failed to edit password sync node.",
+                         model_type());
+            return false;
+          }
+          WriteToSyncNode(new_password, &write_node);
+          updated_passwords.push_back(new_password);
+        }
+
+        Associate(&tag, node.GetId());
+      } else {
+        sync_api::WriteNode node(&trans);
+        if (!node.InitUniqueByCreation(syncable::PASSWORDS,
+                                       password_root, tag)) {
+          STLDeleteElements(&passwords);
+          error->Reset(FROM_HERE,
+                       "Failed to create password sync node.",
+                       model_type());
+          return false;
+        }
+
+        WriteToSyncNode(**ix, &node);
+
+        Associate(&tag, node.GetId());
+      }
+
+      current_passwords.insert(tag);
     }
 
-    current_passwords.insert(tag);
-  }
+    STLDeleteElements(&passwords);
 
-  STLDeleteElements(&passwords);
+    int64 sync_child_id = password_root.GetFirstChildId();
+    while (sync_child_id != sync_api::kInvalidId) {
+      sync_api::ReadNode sync_child_node(&trans);
+      if (!sync_child_node.InitByIdLookup(sync_child_id)) {
+        error->Reset(FROM_HERE, "Failed to fetch child node.", model_type());
+        return false;
+      }
+      const sync_pb::PasswordSpecificsData& password =
+          sync_child_node.GetPasswordSpecifics();
+      std::string tag = MakeTag(password);
 
-  int64 sync_child_id = password_root.GetFirstChildId();
-  while (sync_child_id != sync_api::kInvalidId) {
-    sync_api::ReadNode sync_child_node(&trans);
-    if (!sync_child_node.InitByIdLookup(sync_child_id)) {
-      LOG(ERROR) << "Failed to fetch child node.";
-      return false;
-    }
-    const sync_pb::PasswordSpecificsData& password =
-        sync_child_node.GetPasswordSpecifics();
-    std::string tag = MakeTag(password);
+      // The password only exists on the server.  Add it to the local
+      // model.
+      if (current_passwords.find(tag) == current_passwords.end()) {
+        webkit::forms::PasswordForm new_password;
 
-    // The password only exists on the server.  Add it to the local
-    // model.
-    if (current_passwords.find(tag) == current_passwords.end()) {
-      webkit_glue::PasswordForm new_password;
+        CopyPassword(password, &new_password);
+        Associate(&tag, sync_child_node.GetId());
+        new_passwords.push_back(new_password);
+      }
 
-      CopyPassword(password, &new_password);
-      Associate(&tag, sync_child_node.GetId());
-      new_passwords.push_back(new_password);
+      sync_child_id = sync_child_node.GetSuccessorId();
     }
-
-    sync_child_id = sync_child_node.GetSuccessorId();
   }
 
+  // We must not be holding a transaction when we interact with the password
+  // store, as it can post tasks to the UI thread which can itself be blocked
+  // on our transaction, resulting in deadlock. (http://crbug.com/70658)
   if (!WriteToPasswordStore(&new_passwords, &updated_passwords, NULL)) {
-    LOG(ERROR) << "Failed to write passwords.";
+    error->Reset(FROM_HERE, "Failed to write passwords.", model_type());
     return false;
   }
 
@@ -161,7 +185,7 @@ bool PasswordModelAssociator::DeleteAllNodes(
   return true;
 }
 
-bool PasswordModelAssociator::DisassociateModels() {
+bool PasswordModelAssociator::DisassociateModels(SyncError* error) {
   id_map_.clear();
   id_map_inverse_.clear();
   return true;
@@ -176,8 +200,7 @@ bool PasswordModelAssociator::SyncModelHasUserCreatedNodes(bool* has_nodes) {
                << "might be running against an out-of-date server.";
     return false;
   }
-  sync_api::ReadTransaction trans(
-      sync_service_->backend()->GetUserShareHandle());
+  sync_api::ReadTransaction trans(FROM_HERE, sync_service_->GetUserShare());
 
   sync_api::ReadNode password_node(&trans);
   if (!password_node.InitByIdLookup(password_sync_id)) {
@@ -188,7 +211,7 @@ bool PasswordModelAssociator::SyncModelHasUserCreatedNodes(bool* has_nodes) {
 
   // The sync model has user created nodes if the password folder has any
   // children.
-  *has_nodes = sync_api::kInvalidId != password_node.GetFirstChildId();
+  *has_nodes = password_node.HasChildren();
   return true;
 }
 
@@ -198,6 +221,13 @@ void PasswordModelAssociator::AbortAssociation() {
   abort_association_pending_ = true;
 }
 
+bool PasswordModelAssociator::CryptoReadyIfNecessary() {
+  // We only access the cryptographer while holding a transaction.
+  sync_api::ReadTransaction trans(FROM_HERE, sync_service_->GetUserShare());
+  // We always encrypt passwords, so no need to check if encryption is enabled.
+  return sync_service_->IsCryptographerReady(&trans);
+}
+
 const std::string* PasswordModelAssociator::GetChromeNodeFromSyncId(
     int64 sync_id) {
   return NULL;
@@ -241,8 +271,7 @@ void PasswordModelAssociator::Disassociate(int64 sync_id) {
 
 bool PasswordModelAssociator::GetSyncIdForTaggedNode(const std::string& tag,
                                                      int64* sync_id) {
-  sync_api::ReadTransaction trans(
-      sync_service_->backend()->GetUserShareHandle());
+  sync_api::ReadTransaction trans(FROM_HERE, sync_service_->GetUserShare());
   sync_api::ReadNode sync_node(&trans);
   if (!sync_node.InitByTagLookup(tag.c_str()))
     return false;
@@ -274,15 +303,21 @@ bool PasswordModelAssociator::WriteToPasswordStore(
       password_store_->RemoveLoginImpl(*password);
     }
   }
+
+  if (new_passwords || updated_passwords || deleted_passwords) {
+    // We have to notify password store observers of the change by hand since
+    // we use internal password store interfaces to make changes synchronously.
+    password_store_->PostNotifyLoginsChanged();
+  }
   return true;
 }
 
 // static
 void PasswordModelAssociator::CopyPassword(
         const sync_pb::PasswordSpecificsData& password,
-        webkit_glue::PasswordForm* new_password) {
+        webkit::forms::PasswordForm* new_password) {
   new_password->scheme =
-      static_cast<webkit_glue::PasswordForm::Scheme>(password.scheme());
+      static_cast<webkit::forms::PasswordForm::Scheme>(password.scheme());
   new_password->signon_realm = password.signon_realm();
   new_password->origin = GURL(password.origin());
   new_password->action = GURL(password.action());
@@ -305,8 +340,8 @@ void PasswordModelAssociator::CopyPassword(
 // static
 bool PasswordModelAssociator::MergePasswords(
         const sync_pb::PasswordSpecificsData& password,
-        const webkit_glue::PasswordForm& password_form,
-        webkit_glue::PasswordForm* new_password) {
+        const webkit::forms::PasswordForm& password_form,
+        webkit::forms::PasswordForm* new_password) {
   DCHECK(new_password);
 
   if (password.scheme() == password_form.scheme &&
@@ -341,7 +376,7 @@ bool PasswordModelAssociator::MergePasswords(
 
 // static
 void PasswordModelAssociator::WriteToSyncNode(
-         const webkit_glue::PasswordForm& password_form,
+         const webkit::forms::PasswordForm& password_form,
          sync_api::WriteNode* node) {
   sync_pb::PasswordSpecificsData password;
   password.set_scheme(password_form.scheme);
@@ -362,7 +397,7 @@ void PasswordModelAssociator::WriteToSyncNode(
 
 // static
 std::string PasswordModelAssociator::MakeTag(
-                const webkit_glue::PasswordForm& password) {
+                const webkit::forms::PasswordForm& password) {
   return MakeTag(password.origin.spec(),
                  UTF16ToUTF8(password.username_element),
                  UTF16ToUTF8(password.username_value),
@@ -387,11 +422,11 @@ std::string PasswordModelAssociator::MakeTag(
     const std::string& username_value,
     const std::string& password_element,
     const std::string& signon_realm) {
-  return EscapePath(origin_url) + "|" +
-         EscapePath(username_element) + "|" +
-         EscapePath(username_value) + "|" +
-         EscapePath(password_element) + "|" +
-         EscapePath(signon_realm);
+  return net::EscapePath(origin_url) + "|" +
+         net::EscapePath(username_element) + "|" +
+         net::EscapePath(username_value) + "|" +
+         net::EscapePath(password_element) + "|" +
+         net::EscapePath(signon_realm);
 }
 
 }  // namespace browser_sync
