@@ -1,12 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/sync/glue/password_model_worker.h"
 
-#include "base/callback.h"
-#include "base/ref_counted.h"
-#include "base/task.h"
+#include "base/bind.h"
+#include "base/memory/ref_counted.h"
 #include "base/synchronization/waitable_event.h"
 #include "chrome/browser/password_manager/password_store.h"
 
@@ -21,17 +20,22 @@ PasswordModelWorker::PasswordModelWorker(PasswordStore* password_store)
 
 PasswordModelWorker::~PasswordModelWorker() {}
 
-void PasswordModelWorker::DoWorkAndWaitUntilDone(Callback0::Type* work) {
+SyncerError PasswordModelWorker::DoWorkAndWaitUntilDone(
+    const WorkCallback& work) {
   WaitableEvent done(false, false);
+  SyncerError error = UNSET;
   password_store_->ScheduleTask(
-      NewRunnableMethod(this, &PasswordModelWorker::CallDoWorkAndSignalTask,
-                        work, &done));
+      base::Bind(&PasswordModelWorker::CallDoWorkAndSignalTask,
+                 this, work, &done, &error));
   done.Wait();
+  return error;
 }
 
-void PasswordModelWorker::CallDoWorkAndSignalTask(Callback0::Type* work,
-                                                  WaitableEvent* done) {
-  work->Run();
+void PasswordModelWorker::CallDoWorkAndSignalTask(
+    const WorkCallback& work,
+    WaitableEvent* done,
+    SyncerError *error) {
+  *error = work.Run();
   done->Signal();
 }
 
@@ -39,9 +43,4 @@ ModelSafeGroup PasswordModelWorker::GetModelSafeGroup() {
   return GROUP_PASSWORD;
 }
 
-bool PasswordModelWorker::CurrentThreadIsWorkThread() {
-  // TODO(ncarter): How to determine this?
-  return true;
-}
-
 }  // namespace browser_sync
