@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,16 +7,12 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "chrome/browser/sessions/session_backend.h"
-#include "chrome/browser/sessions/session_service.h"
-#include "chrome/browser/sync/engine/syncapi.h"
+#include "base/compiler_specific.h"
 #include "chrome/browser/sync/glue/change_processor.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_type.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_types.h"
 
-class NotificationDetails;
-class NotificationSource;
 class Profile;
 
 namespace browser_sync {
@@ -29,40 +25,51 @@ class UnrecoverableErrorHandler;
 // model, and vice versa. All operations and use of this class are
 // from the UI thread.
 class SessionChangeProcessor : public ChangeProcessor,
-                               public NotificationObserver {
+                               public content::NotificationObserver {
  public:
   // Does not take ownership of either argument.
   SessionChangeProcessor(
       UnrecoverableErrorHandler* error_handler,
       SessionModelAssociator* session_model_associator);
+  // For testing only.
+  SessionChangeProcessor(
+      UnrecoverableErrorHandler* error_handler,
+      SessionModelAssociator* session_model_associator,
+      bool setup_for_test);
   virtual ~SessionChangeProcessor();
 
-  // NotificationObserver implementation.
+  // content::NotificationObserver implementation.
   // BrowserSessionProvider -> sync_api model change application.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // ChangeProcessor implementation.
   // sync_api model -> BrowserSessionProvider change application.
   virtual void ApplyChangesFromSyncModel(
       const sync_api::BaseTransaction* trans,
-      const sync_api::SyncManager::ChangeRecord* changes,
-      int change_count);
+      const sync_api::ImmutableChangeRecordList& changes) OVERRIDE;
 
  protected:
   // ChangeProcessor implementation.
-  virtual void StartImpl(Profile* profile);
-  virtual void StopImpl();
+  virtual void StartImpl(Profile* profile) OVERRIDE;
+  virtual void StopImpl() OVERRIDE;
+
  private:
+  friend class ScopedStopObserving<SessionChangeProcessor>;
+
   void StartObserving();
   void StopObserving();
-  SessionModelAssociator* session_model_associator_;
-  NotificationRegistrar notification_registrar_;
 
-  // Owner of the SessionService.  Non-NULL iff |running()| is true.
+  SessionModelAssociator* session_model_associator_;
+  content::NotificationRegistrar notification_registrar_;
+
+  // Profile being synced. Non-null if |running()| is true.
   Profile* profile_;
 
+  // To bypass some checks/codepaths not applicable in tests.
+  bool setup_for_test_;
+
   DISALLOW_COPY_AND_ASSIGN(SessionChangeProcessor);
 };
 
