@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,35 +11,38 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
+#include "base/format_macros.h"
 #include "base/gtest_prod_util.h"
-#include "base/observer_list.h"
-#include "base/scoped_vector.h"
-#include "base/string_util.h"
+#include "base/memory/weak_ptr.h"
+#include "base/stringprintf.h"
 #include "base/threading/non_thread_safe.h"
+#include "base/time.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_window.h"
 #include "chrome/browser/sessions/session_id.h"
 #include "chrome/browser/sessions/session_service.h"
 #include "chrome/browser/sessions/session_types.h"
-#include "chrome/browser/sync/engine/syncapi.h"
-#include "chrome/browser/sync/glue/foreign_session_tracker.h"
 #include "chrome/browser/sync/glue/model_associator.h"
-#include "chrome/browser/sync/protocol/session_specifics.pb.h"
+#include "chrome/browser/sync/glue/synced_session_tracker.h"
+#include "chrome/browser/sync/glue/synced_tab_delegate.h"
+#include "chrome/browser/sync/glue/synced_window_delegate.h"
 #include "chrome/browser/sync/syncable/model_type.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/common/notification_registrar.h"
 
 class Profile;
 class ProfileSyncService;
 
 namespace sync_api {
+class BaseTransaction;
 class ReadNode;
-class WriteNode;
 class WriteTransaction;
 }  // namespace sync_api
 
 namespace sync_pb {
+class SessionHeader;
 class SessionSpecifics;
+class SessionTab;
+class SessionWindow;
+class TabNavigation;
 }  // namespace sync_pb
 
 namespace browser_sync {
@@ -49,27 +52,33 @@ static const char kSessionsTag[] = "google_chrome_sessions";
 // Contains all logic for associating the Chrome sessions model and
 // the sync sessions model.
 class SessionModelAssociator
-    : public PerDataTypeAssociatorInterface<TabContents, size_t>,
+    : public PerDataTypeAssociatorInterface<SyncedTabDelegate, size_t>,
+      public base::SupportsWeakPtr<SessionModelAssociator>,
       public base::NonThreadSafe {
  public:
   // Does not take ownership of sync_service.
   explicit SessionModelAssociator(ProfileSyncService* sync_service);
+  SessionModelAssociator(ProfileSyncService* sync_service,
+                         bool setup_for_test);
   virtual ~SessionModelAssociator();
 
   // The has_nodes out parameter is set to true if the sync model has
   // nodes other than the permanent tagged nodes.  The method may
   // return false if an error occurred.
-  virtual bool SyncModelHasUserCreatedNodes(bool* has_nodes);
+  virtual bool SyncModelHasUserCreatedNodes(bool* has_nodes) OVERRIDE;
 
   // AssociatorInterface and PerDataTypeAssociator Interface implementation.
-  virtual void AbortAssociation() {
+  virtual void AbortAssociation() OVERRIDE {
     // No implementation needed, this associator runs on the main thread.
   }
 
+  // See ModelAssociator interface.
+  virtual bool CryptoReadyIfNecessary() OVERRIDE;
+
   // Returns sync id for the given chrome model id.
   // Returns sync_api::kInvalidId if the sync node is not found for the given
   // chrome id.
-  virtual int64 GetSyncIdFromChromeId(const size_t& id);
+  virtual int64 GetSyncIdFromChromeId(const size_t& id) OVERRIDE;
 
   // Returns sync id for the given session tag.
   // Returns sync_api::kInvalidId if the sync node is not found for the given
@@ -77,17 +86,18 @@ class SessionModelAssociator
   virtual int64 GetSyncIdFromSessionTag(const std::string& tag);
 
   // Not used.
-  virtual const TabContents* GetChromeNodeFromSyncId(int64 sync_id) {
-    NOTREACHED();
-    return NULL;
-  }
+  virtual const SyncedTabDelegate* GetChromeNodeFromSyncId(int64 sync_id)
+      OVERRIDE;
 
   // Not used.
-  bool InitSyncNodeFromChromeId(const size_t& id,
-                                sync_api::BaseNode* sync_node) {
-    NOTREACHED();
-    return false;
-  }
+  virtual bool InitSyncNodeFromChromeId(const size_t& id,
+                                        sync_api::BaseNode* sync_node) OVERRIDE;
+
+  // Not used.
+  virtual void Associate(const SyncedTabDelegate* tab, int64 sync_id) OVERRIDE;
+
+  // Not used.
+  virtual void Disassociate(int64 sync_id) OVERRIDE;
 
   // Resync local window information. Updates the local sessions header node
   // with the status of open windows and the order of tabs they contain. Should
@@ -95,29 +105,27 @@ class SessionModelAssociator
   // single tab.
   //
   // If |reload_tabs| is true, will also resync all tabs (same as calling
-  // ReassociateTabs with a vector of all tabs).
-  void ReassociateWindows(bool reload_tabs);
+  // AssociateTabs with a vector of all tabs).
+  // Returns: false if the local session's sync nodes were deleted and
+  // reassociation is necessary, true otherwise.
+  bool AssociateWindows(bool reload_tabs);
 
   // Loads and reassociates the local tabs referenced in |tabs|.
-  void ReassociateTabs(const std::vector<TabContents*>& tabs);
+  // Returns: false if the local session's sync nodes were deleted and
+  // reassociation is necessary, true otherwise.
+  bool AssociateTabs(const std::vector<SyncedTabDelegate*>& tabs);
 
   // Reassociates a single tab with the sync model. Will check if the tab
   // already is associated with a sync node and allocate one if necessary.
-  void ReassociateTab(const TabContents& tab);
-
-  // Associate a local tab and it's sync node. Will overwrite the contents of
-  // the sync node with new specifics built from the tab.
-  virtual void Associate(const TabContents* tab, int64 sync_id);
-
-  // Looks up the specified sync node, and marks that tab as closed, then marks
-  // the node as free and deletes association.
-  virtual void Disassociate(int64 sync_id);
+  // Returns: false if the local session's sync nodes were deleted and
+  // reassociation is necessary, true otherwise.
+  bool AssociateTab(const SyncedTabDelegate& tab);
 
   // Load any foreign session info stored in sync db and update the sync db
   // with local client data. Processes/reuses any sync nodes owned by this
   // client and creates any further sync nodes needed to store local header and
   // tab info.
-  virtual bool AssociateModels();
+  virtual bool AssociateModels(SyncError* error) OVERRIDE;
 
   // Initializes the given sync node from the given chrome node id.
   // Returns false if no sync node was found for the given chrome node id or
@@ -127,55 +135,80 @@ class SessionModelAssociator
 
   // Clear local sync data buffers. Does not delete sync nodes to avoid
   // tombstones. TODO(zea): way to eventually delete orphaned nodes.
-  virtual bool DisassociateModels();
+  virtual bool DisassociateModels(SyncError* error) OVERRIDE;
 
   // Returns the tag used to uniquely identify this machine's session in the
   // sync model.
-  inline const std::string& GetCurrentMachineTag() {
+  const std::string& GetCurrentMachineTag() const {
     DCHECK(!current_machine_tag_.empty());
     return current_machine_tag_;
   }
 
   // Load and associate window and tab data for a foreign session
   bool AssociateForeignSpecifics(const sync_pb::SessionSpecifics& specifics,
-                                 int64 modification_time);
+                                 const base::Time& modification_time);
 
   // Removes a foreign session from our internal bookkeeping.
-  void DisassociateForeignSession(const std::string& foreign_session_tag);
+  // Returns true if the session was found and deleted, false if no data was
+  // found for that session.
+  bool DisassociateForeignSession(const std::string& foreign_session_tag);
 
-  // Builds a list of all foreign sessions.
-  // Caller does NOT own ForeignSession objects.
-  bool GetAllForeignSessions(std::vector<const ForeignSession*>* sessions);
+  // Attempts to asynchronously refresh the sessions sync data. If new data is
+  // received, the FOREIGN_SESSIONS_UPDATED notification is sent. No
+  // notification will be sent otherwise. This method is not guaranteed to
+  // trigger a sync cycle.
+  void AttemptSessionsDataRefresh() const;
 
-  // Loads all windows for foreign session with session tag |tag|.
-  // Caller does NOT own ForeignSession objects.
+  // Sets |*local_session| to point to the associator's representation of the
+  // local machine. Used primarily for testing.
+  bool GetLocalSession(const SyncedSession* * local_session);
+
+  // Builds a list of all foreign sessions. Caller does NOT own SyncedSession
+  // objects.
+  // Returns true if foreign sessions were found, false otherwise.
+  bool GetAllForeignSessions(std::vector<const SyncedSession*>* sessions);
+
+  // Loads all windows for foreign session with session tag |tag|. Caller does
+  // NOT own SyncedSession objects.
+  // Returns true if the foreign session was found, false otherwise.
   bool GetForeignSession(const std::string& tag,
-                         std::vector<SessionWindow*>* windows);
+                         std::vector<const SessionWindow*>* windows);
 
   // Looks up the foreign tab identified by |tab_id| and belonging to foreign
-  // session |tag|.
-  // Caller does NOT own the SessionTab object.
+  // session |tag|. Caller does NOT own the SessionTab object.
+  // Returns true if the foreign session and tab were found, false otherwise.
   bool GetForeignTab(const std::string& tag,
                      const SessionID::id_type tab_id,
                      const SessionTab** tab);
 
-  // Specifies whether the window has tabs to sync. The new tab page does not
-  // count. If no tabs to sync, it returns true, otherwise false;
-  static bool SessionWindowHasNoTabsToSync(const SessionWindow& window);
+  // Triggers garbage collection of stale sessions (as defined by
+  // |stale_session_threshold_days_|). This is called automatically every
+  // time we start up (via AssociateModels).
+  void DeleteStaleSessions();
+
+  // Set the threshold of inactivity (in days) at which we consider sessions
+  // stale.
+  void SetStaleSessionThreshold(size_t stale_session_threshold_days);
+
+  // Delete a foreign session and all it's sync data.
+  void DeleteForeignSession(const std::string& tag);
 
   // Control which local tabs we're interested in syncing.
-  // Ensures the profile matches sync's profile and that the tab has at least
-  // one navigation entry and is not an empty tab.
-  bool IsValidTab(const TabContents& tab);
-
-  // Control which foreign tabs we're interested in displaying.
-  // Checks that the tab has navigations and is not a new tab.
-  // Note: a new tab page with back/forward history is valid.
-  static bool IsValidSessionTab(const SessionTab& tab);
+  // Ensures the profile matches sync's profile and that the tab has valid
+  // entries.
+  bool ShouldSyncTab(const SyncedTabDelegate& tab) const;
 
   // Returns the syncable model type.
   static syncable::ModelType model_type() { return syncable::SESSIONS; }
 
+  // Testing only. Will cause the associator to call MessageLoop::Quit()
+  // when a local change is made, or when timeout_milli occurs, whichever is
+  // first.
+  void BlockUntilLocalChangeForTest(int64 timeout_milli);
+
+  // Callback for when the session name has been computed.
+  void OnSessionNameInitialized(const std::string& name);
+
  private:
   FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceSessionTest, WriteSessionToNode);
   FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceSessionTest,
@@ -184,8 +217,15 @@ class SessionModelAssociator
                            WriteForeignSessionToNode);
   FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceSessionTest, TabNodePoolEmpty);
   FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceSessionTest, TabNodePoolNonEmpty);
+  FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceSessionTest, ValidTabs);
+  FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceSessionTest, ExistingTabs);
+  FRIEND_TEST_ALL_PREFIXES(SessionModelAssociatorTest, PopulateSessionHeader);
   FRIEND_TEST_ALL_PREFIXES(SessionModelAssociatorTest, PopulateSessionWindow);
   FRIEND_TEST_ALL_PREFIXES(SessionModelAssociatorTest, PopulateSessionTab);
+  FRIEND_TEST_ALL_PREFIXES(SessionModelAssociatorTest,
+                           InitializeCurrentSessionName);
+  FRIEND_TEST_ALL_PREFIXES(SessionModelAssociatorTest,
+                           TabNodePool);
 
   // Keep all the links to local tab data in one place.
   class TabLinks {
@@ -198,11 +238,11 @@ class SessionModelAssociator
           tab_(NULL) {}
 
     // We only ever have either a SessionTab (for foreign tabs), or a
-    // TabContents (for local tabs).
-    TabLinks(int64 sync_id, const TabContents* tab)
+    // SyncedTabDelegate (for local tabs).
+    TabLinks(int64 sync_id, const SyncedTabDelegate* tab)
       : sync_id_(sync_id),
         session_tab_(NULL) {
-      tab_ = const_cast<TabContents*>(tab);
+      tab_ = const_cast<SyncedTabDelegate*>(tab);
     }
     TabLinks(int64 sync_id, const SessionTab* session_tab)
       : sync_id_(sync_id),
@@ -210,13 +250,14 @@ class SessionModelAssociator
       session_tab_ = const_cast<SessionTab*>(session_tab);
     }
 
-    inline int64 sync_id() const { return sync_id_; }
-    inline const SessionTab* session_tab() const { return session_tab_; }
-    inline const TabContents* tab() const { return tab_; }
+    int64 sync_id() const { return sync_id_; }
+    const SessionTab* session_tab() const { return session_tab_; }
+    const SyncedTabDelegate* tab() const { return tab_; }
+
    private:
     int64 sync_id_;
     SessionTab* session_tab_;
-    TabContents* tab_;
+    SyncedTabDelegate* tab_;
   };
 
   // A pool for managing free/used tab sync nodes. Performs lazy creation
@@ -224,6 +265,7 @@ class SessionModelAssociator
   class TabNodePool {
    public:
     explicit TabNodePool(ProfileSyncService* sync_service);
+    ~TabNodePool();
 
     // Add a previously allocated tab sync node to our pool. Increases the size
     // of tab_syncid_pool_ by one and marks the new tab node as free.
@@ -255,23 +297,27 @@ class SessionModelAssociator
     void FreeTabNode(int64 sync_id);
 
     // Clear tab pool.
-    inline void clear() { tab_syncid_pool_.clear(); }
+    void clear() {
+      tab_syncid_pool_.clear();
+      tab_pool_fp_ = -1;
+    }
 
     // Return the number of tab nodes this client currently has allocated
     // (including both free and used nodes)
-    inline size_t capacity() const { return tab_syncid_pool_.size(); }
+    size_t capacity() const { return tab_syncid_pool_.size(); }
 
     // Return empty status (all tab nodes are in use).
-    inline bool empty() const { return tab_pool_fp_ == -1; }
+    bool empty() const { return tab_pool_fp_ == -1; }
 
     // Return full status (no tab nodes are in use).
-    inline bool full() {
+    bool full() {
       return tab_pool_fp_ == static_cast<int64>(tab_syncid_pool_.size())-1;
     }
 
-    inline void set_machine_tag(const std::string& machine_tag) {
+    void set_machine_tag(const std::string& machine_tag) {
       machine_tag_ = machine_tag;
     }
+
    private:
     // Pool of all available syncid's for tab's we have created.
     std::vector<int64> tab_syncid_pool_;
@@ -295,28 +341,26 @@ class SessionModelAssociator
   // Datatypes for accessing local tab data.
   typedef std::map<SessionID::id_type, TabLinks> TabLinksMap;
 
-  // Delete all foreign session/window/tab objects allocated dynamically.
-  // This is comprised of ForeignSession*, IDToSessionTabMap*, and any orphaned
-  // SessionTab*'s.
-  void DeleteForeignSessions();
-
   // Determine if a window is of a type we're interested in syncing.
-  static bool ShouldSyncWindowType(const Browser::Type& type);
+  static bool ShouldSyncWindow(const SyncedWindowDelegate* window);
 
   // Build a sync tag from tab_node_id.
-  static inline std::string TabIdToTag(
+  static std::string TabIdToTag(
       const std::string machine_tag,
       size_t tab_node_id) {
-    return StringPrintf("%s %lu",
-        machine_tag.c_str(), static_cast<unsigned long>(tab_node_id));
+    return base::StringPrintf("%s %"PRIuS"", machine_tag.c_str(), tab_node_id);
   }
 
   // Initializes the tag corresponding to this machine.
   void InitializeCurrentMachineTag(sync_api::WriteTransaction* trans);
 
+  // Initializes the user visible name for this session
+  void InitializeCurrentSessionName();
+
   // Updates the server data based upon the current client session.  If no node
   // corresponding to this machine exists in the sync model, one is created.
-  void UpdateSyncModelDataFromClient();
+  // Returns true on success, false if association failed.
+  bool UpdateSyncModelDataFromClient();
 
   // Pulls the current sync model from the sync database and returns true upon
   // update of the client model. Will associate any foreign sessions as well as
@@ -326,22 +370,30 @@ class SessionModelAssociator
 
   // Fills a tab sync node with data from a TabContents object.
   // (from a local navigation event)
-  bool WriteTabContentsToSyncModel(const TabContents& tab,
-                                   const int64 sync_id,
-                                   sync_api::WriteTransaction* trans);
+  // Returns true on success, false if we need to reassociate due to corruption.
+  bool WriteTabContentsToSyncModel(const SyncedWindowDelegate& window,
+                                   const SyncedTabDelegate& tab,
+                                   const int64 sync_id);
+
+  // Used to populate a session header from the session specifics header
+  // provided.
+  static void PopulateSessionHeaderFromSpecifics(
+    const sync_pb::SessionHeader& header_specifics,
+    const base::Time& mtime,
+    SyncedSession* session_header);
 
   // Used to populate a session window from the session specifics window
   // provided. Tracks any foreign session data created through |tracker|.
   static void PopulateSessionWindowFromSpecifics(
-      std::string foreign_session_tag,
+      const std::string& foreign_session_tag,
       const sync_pb::SessionWindow& window,
-      const int64 mtime,
+      const base::Time& mtime,
       SessionWindow* session_window,
-      ForeignSessionTracker* tracker);
+      SyncedSessionTracker* tracker);
 
   // Used to populate a session tab from the session specifics tab provided.
   static void PopulateSessionTabFromSpecifics(const sync_pb::SessionTab& tab,
-                                              const int64 mtime,
+                                              const base::Time& mtime,
                                               SessionTab* session_tab);
 
   // Used to populate a session tab from the session specifics tab provided.
@@ -354,39 +406,25 @@ class SessionModelAssociator
      const TabNavigation* navigation,
      sync_pb::TabNavigation* tab_navigation);
 
-  // Returns the session service from |sync_service_|.
-  SessionService* GetSessionService();
+  // Returns true if this tab belongs to this profile and belongs to a window,
+  // false otherwise.
+  bool IsValidTab(const SyncedTabDelegate& tab) const;
 
-  // Internal method used in the callback to obtain the current session.
-  // We don't own |windows|.
-  void OnGotSession(int handle, std::vector<SessionWindow*>* windows);
+  // Having a valid entry is defined as the url being valid and and having a
+  // syncable scheme (non chrome:// and file:// url's). In other words, we don't
+  // want to sync a tab that is nothing but chrome:// and file:// navigations or
+  // invalid url's.
+  bool TabHasValidEntry(const SyncedTabDelegate& tab) const;
 
-  // Populate a session specifics header from a list of SessionWindows
-  void PopulateSessionSpecificsHeader(
-      const std::vector<SessionWindow*>& windows,
-      sync_pb::SessionHeader* header_s);
+  // For testing only.
+  void QuitLoopForSubtleTesting();
 
-  // Populates the window portion of the session specifics.
-  void PopulateSessionSpecificsWindow(const SessionWindow& window,
-                                      sync_pb::SessionWindow* session_window);
-
-  // Syncs all the tabs in |window| with the local sync db. Will allocate tab
-  // nodes if needed.
-  bool SyncLocalWindowToSyncModel(const SessionWindow& window);
-
-  // Fills a tab sync node with data from a SessionTab object.
-  // (from ReadCurrentSessions)
-  bool WriteSessionTabToSyncModel(const SessionTab& tab,
-                                  const int64 sync_id,
-                                  sync_api::WriteTransaction* trans);
-
-  // Populates the tab portion of the session specifics.
-  void PopulateSessionSpecificsTab(const SessionTab& tab,
-                                   sync_pb::SessionTab* session_tab);
-
-  // Local client name.
+  // Unique client tag.
   std::string current_machine_tag_;
 
+  // User-visible machine name.
+  std::string current_session_name_;
+
   // Pool of all used/available sync nodes associated with tabs.
   TabNodePool tab_pool_;
 
@@ -397,14 +435,29 @@ class SessionModelAssociator
   // Mapping of current open (local) tabs to their sync identifiers.
   TabLinksMap tab_map_;
 
-  ForeignSessionTracker foreign_session_tracker_;
+  SyncedSessionTracker synced_session_tracker_;
 
   // Weak pointer.
   ProfileSyncService* sync_service_;
 
+  // Number of days without activity after which we consider a session to be
+  // stale and a candidate for garbage collection.
+  size_t stale_session_threshold_days_;
+
   // Consumer used to obtain the current session.
   CancelableRequestConsumer consumer_;
 
+  // To avoid certain checks not applicable to tests.
+  bool setup_for_test_;
+
+  // During integration tests, we sometimes need to block until a local change
+  // is made.
+  bool waiting_for_change_;
+  base::WeakPtrFactory<SessionModelAssociator> test_weak_factory_;
+
+  // Profile being synced.
+  const Profile* const profile_;
+
   DISALLOW_COPY_AND_ASSIGN(SessionModelAssociator);
 };
 
