@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,10 +8,10 @@
 
 #include <set>
 
+#include "base/callback_forward.h"
 #include "chrome/browser/sync/glue/sync_backend_host.h"
-#include "base/task.h"
 #include "chrome/browser/sync/profile_sync_test_util.h"
-#include "chrome/common/notification_type.h"
+#include "content/public/browser/notification_types.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
 namespace browser_sync {
@@ -21,11 +21,13 @@ class SyncBackendHostMock : public SyncBackendHost {
   SyncBackendHostMock();
   virtual ~SyncBackendHostMock();
 
-  MOCK_METHOD3(ConfigureDataTypes,
-               void(const DataTypeController::TypeMap&,
-                    const std::set<syncable::ModelType>&, CancelableTask*));
-  MOCK_METHOD0(RequestPause, bool());
-  MOCK_METHOD0(RequestResume, bool());
+  MOCK_METHOD6(ConfigureDataTypes,
+               void(syncable::ModelTypeSet,
+                    syncable::ModelTypeSet,
+                    sync_api::ConfigureReason,
+                    base::Callback<void(syncable::ModelTypeSet)>,
+                    base::Callback<void()>,
+                    bool));
   MOCK_METHOD0(StartSyncingWithServer, void());
 };
 
