@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #pragma once
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "chrome/browser/sync/glue/model_associator.h"
 #include "chrome/browser/sync/syncable/model_type.h"
 
@@ -14,8 +15,6 @@ class ProfileSyncService;
 
 namespace browser_sync {
 
-class UnrecoverableErrorHandler;
-
 // Contains all logic for associating the Chrome themes model and the
 // sync themes model.
 class ThemeModelAssociator : public AssociatorInterface {
@@ -27,13 +26,14 @@ class ThemeModelAssociator : public AssociatorInterface {
   static syncable::ModelType model_type() { return syncable::THEMES; }
 
   // AssociatorInterface implementation.
-  virtual bool AssociateModels();
-  virtual bool DisassociateModels();
-  virtual bool SyncModelHasUserCreatedNodes(bool* has_nodes);
-  virtual void AbortAssociation() {
+  virtual bool AssociateModels(SyncError* error) OVERRIDE;
+  virtual bool DisassociateModels(SyncError* error) OVERRIDE;
+  virtual bool SyncModelHasUserCreatedNodes(bool* has_nodes) OVERRIDE;
+  virtual void AbortAssociation() OVERRIDE {
     // No implementation needed, this associator runs on the main
     // thread.
   }
+  virtual bool CryptoReadyIfNecessary() OVERRIDE;
 
  private:
   ProfileSyncService* sync_service_;
