@@ -7,15 +7,13 @@
 #include <string>
 
 #include "base/logging.h"
-#include "base/scoped_ptr.h"
-#include "chrome/browser/extensions/extension_install_ui.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/extension_updater.h"
-#if defined(TOOLKIT_USES_GTK)
-#include "chrome/browser/ui/gtk/gtk_theme_provider.h"
-#endif
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sync/protocol/theme_specifics.pb.h"
+#include "chrome/browser/themes/theme_service.h"
+#include "chrome/browser/themes/theme_service_factory.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "googleurl/src/gurl.h"
@@ -26,6 +24,7 @@ const char kCurrentThemeClientTag[] = "current_theme";
 
 namespace {
 
+// TODO(akalin): Remove this.
 bool IsSystemThemeDistinctFromDefaultTheme() {
 #if defined(TOOLKIT_USES_GTK)
   return true;
@@ -34,14 +33,6 @@ bool IsSystemThemeDistinctFromDefaultTheme() {
 #endif
 }
 
-bool UseSystemTheme(Profile* profile) {
-#if defined(TOOLKIT_USES_GTK)
-  return GtkThemeProvider::GetFrom(profile)->UseGtkTheme();
-#else
-  return false;
-#endif
-}
-
 }  // namespace
 
 bool AreThemeSpecificsEqual(const sync_pb::ThemeSpecifics& a,
@@ -90,78 +81,49 @@ void SetCurrentThemeFromThemeSpecifics(
     // (i.e., those not on either Google gallery).
     std::string id(theme_specifics.custom_theme_id());
     GURL update_url(theme_specifics.custom_theme_update_url());
-    VLOG(1) << "Applying theme " << id << " with update_url " << update_url;
-    ExtensionService* extensions_service = profile->GetExtensionService();
+    DVLOG(1) << "Applying theme " << id << " with update_url " << update_url;
+    ExtensionServiceInterface* extensions_service =
+        profile->GetExtensionService();
     CHECK(extensions_service);
     const Extension* extension = extensions_service->GetExtensionById(id, true);
     if (extension) {
       if (!extension->is_theme()) {
-        VLOG(1) << "Extension " << id << " is not a theme; aborting";
+        DVLOG(1) << "Extension " << id << " is not a theme; aborting";
         return;
       }
-      ExtensionPrefs* extension_prefs = extensions_service->extension_prefs();
-      CHECK(extension_prefs);
-      // TODO(akalin): GetExtensionState() isn't very safe as it
-      // returns Extension::ENABLED by default; either change it to
-      // return something else by default or create a separate
-      // function that does so.
-      if (extension_prefs->GetExtensionState(extension->id()) !=
-          Extension::ENABLED) {
-        VLOG(1) << "Theme " << id << " is not enabled; aborting";
+      if (!extensions_service->IsExtensionEnabled(id)) {
+        DVLOG(1) << "Theme " << id << " is not enabled; aborting";
         return;
       }
-      // Get previous theme info before we set the new theme.
-      std::string previous_theme_id;
-      {
-        const Extension* current_theme = profile->GetTheme();
-        if (current_theme) {
-          DCHECK(current_theme->is_theme());
-          previous_theme_id = current_theme->id();
-        }
-      }
-      bool previous_use_system_theme = UseSystemTheme(profile);
       // An enabled theme extension with the given id was found, so
       // just set the current theme to it.
-      profile->SetTheme(extension);
-      // Pretend the theme was just installed.
-      ExtensionInstallUI::ShowThemeInfoBar(
-          previous_theme_id, previous_use_system_theme,
-          extension, profile);
+      ThemeServiceFactory::GetForProfile(profile)->SetTheme(extension);
     } else {
       // No extension with this id exists -- we must install it; we do
       // so by adding it as a pending extension and then triggering an
       // auto-update cycle.
-      // Themes don't need to install silently as they just pop up an
-      // informational dialog after installation instead of a
-      // confirmation dialog.
-      const bool kInstallSilently = false;
-      const bool kEnableOnInstall = true;
-      const bool kEnableIncognitoOnInstall = false;
-      extensions_service->AddPendingExtensionFromSync(
-          id, update_url, &IsTheme,
-          kInstallSilently, kEnableOnInstall, kEnableIncognitoOnInstall);
-      ExtensionUpdater* extension_updater = extensions_service->updater();
-      // Auto-updates should now be on always (see the construction of
-      // the ExtensionService in ProfileImpl::InitExtensions()).
-      if (!extension_updater) {
-        LOG(DFATAL) << "Extension updater unexpectedly NULL; "
-                    << "auto-updates may be turned off";
+      const bool kInstallSilently = true;
+      if (!extensions_service->pending_extension_manager()->AddFromSync(
+              id, update_url, &IsTheme,
+              kInstallSilently)) {
+        LOG(WARNING) << "Could not add pending extension for " << id;
         return;
       }
-      extension_updater->CheckNow();
+      extensions_service->CheckForUpdatesSoon();
     }
   } else if (theme_specifics.use_system_theme_by_default()) {
-    profile->SetNativeTheme();
+    ThemeServiceFactory::GetForProfile(profile)->SetNativeTheme();
   } else {
-    profile->ClearTheme();
+    ThemeServiceFactory::GetForProfile(profile)->UseDefaultTheme();
   }
 }
 
 bool UpdateThemeSpecificsOrSetCurrentThemeIfNecessary(
     Profile* profile, sync_pb::ThemeSpecifics* theme_specifics) {
   if (!theme_specifics->use_custom_theme() &&
-      (profile->GetTheme() || (UseSystemTheme(profile) &&
-                               IsSystemThemeDistinctFromDefaultTheme()))) {
+      (ThemeServiceFactory::GetThemeForProfile(profile) ||
+       (ThemeServiceFactory::GetForProfile(profile)->UsingNativeTheme() &&
+        IsSystemThemeDistinctFromDefaultTheme()))) {
     GetThemeSpecificsFromCurrentTheme(profile, theme_specifics);
     return true;
   } else {
@@ -174,14 +136,15 @@ void GetThemeSpecificsFromCurrentTheme(
     Profile* profile,
     sync_pb::ThemeSpecifics* theme_specifics) {
   DCHECK(profile);
-  const Extension* current_theme = profile->GetTheme();
+  const Extension* current_theme =
+      ThemeServiceFactory::GetThemeForProfile(profile);
   if (current_theme) {
     DCHECK(current_theme->is_theme());
   }
   GetThemeSpecificsFromCurrentThemeHelper(
       current_theme,
       IsSystemThemeDistinctFromDefaultTheme(),
-      UseSystemTheme(profile),
+      ThemeServiceFactory::GetForProfile(profile)->UsingNativeTheme(),
       theme_specifics);
 }
 
@@ -195,8 +158,6 @@ void GetThemeSpecificsFromCurrentThemeHelper(
   if (is_system_theme_distinct_from_default_theme) {
     theme_specifics->set_use_system_theme_by_default(
         use_system_theme_by_default);
-  } else {
-    DCHECK(!use_system_theme_by_default);
   }
   if (use_custom_theme) {
     DCHECK(current_theme);
