@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,39 +6,62 @@
 #define CHROME_BROWSER_SYNC_PROFILE_SYNC_SERVICE_H_
 #pragma once
 
+#include <list>
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "base/gtest_prod_util.h"
+#include "base/location.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
-#include "base/scoped_ptr.h"
 #include "base/string16.h"
 #include "base/time.h"
 #include "base/timer.h"
-#include "chrome/browser/prefs/pref_member.h"
-#include "chrome/browser/sync/engine/syncapi.h"
+#include "chrome/browser/profiles/profile_keyed_service.h"
+#include "chrome/browser/sync/backend_unrecoverable_error_handler.h"
+#include "chrome/browser/sync/engine/model_safe_worker.h"
+#include "chrome/browser/sync/failed_datatypes_handler.h"
 #include "chrome/browser/sync/glue/data_type_controller.h"
 #include "chrome/browser/sync/glue/data_type_manager.h"
-#include "chrome/browser/sync/glue/session_model_associator.h"
 #include "chrome/browser/sync/glue/sync_backend_host.h"
+#include "chrome/browser/sync/internal_api/includes/unrecoverable_error_handler.h"
+#include "chrome/browser/sync/internal_api/sync_manager.h"
 #include "chrome/browser/sync/profile_sync_service_observer.h"
-#include "chrome/browser/sync/signin_manager.h"
-#include "chrome/browser/sync/sync_setup_wizard.h"
 #include "chrome/browser/sync/syncable/model_type.h"
-#include "chrome/browser/sync/unrecoverable_error_handler.h"
+#include "chrome/browser/sync/sync_js_controller.h"
+#include "chrome/browser/sync/sync_prefs.h"
+#include "chrome/browser/sync/sync_setup_wizard.h"
 #include "chrome/common/net/gaia/google_service_auth_error.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_types.h"
 #include "googleurl/src/gurl.h"
-#include "jingle/notifier/base/notifier_options.h"
 
-class NotificationDetails;
-class NotificationSource;
-class NotificationType;
 class Profile;
-class ProfileSyncFactory;
-class TabContents;
-class TokenMigrator;
+class ProfileSyncComponentsFactory;
+class SigninManager;
+class SyncGlobalError;
+
+namespace browser_sync {
+class BackendMigrator;
+class ChangeProcessor;
+class DataTypeManager;
+class JsController;
+class SessionModelAssociator;
+namespace sessions { struct SyncSessionSnapshot; }
+}
+
+namespace sync_api {
+class BaseTransaction;
+struct SyncCredentials;
+struct UserShare;
+}
+
+namespace sync_pb {
+class EncryptedData;
+}  // namespace sync_pb
 
 // ProfileSyncService is the layer between browser subsystems like bookmarks,
 // and the sync backend.  Each subsystem is logically thought of as being
@@ -83,8 +106,13 @@ class TokenMigrator;
 //      are differentiated by the DataTypeController state.
 //
 class ProfileSyncService : public browser_sync::SyncFrontend,
+                           public browser_sync::SyncPrefObserver,
                            public browser_sync::UnrecoverableErrorHandler,
-                           public NotificationObserver {
+                           public content::NotificationObserver,
+                           public ProfileKeyedService,
+                           // TODO(lipalani): crbug.com/100829. Instead of
+                           // doing this vend weak pointers from a factory.
+                           public base::SupportsWeakPtr<ProfileSyncService> {
  public:
   typedef ProfileSyncServiceObserver Observer;
   typedef browser_sync::SyncBackendHost::Status Status;
@@ -97,6 +125,8 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
     START_FROM_WRENCH = 2,   // Sync was started from the Wrench menu.
     START_FROM_OPTIONS = 3,  // Sync was started from Wrench->Options.
     START_FROM_BOOKMARK_MANAGER = 4,  // Sync was started from Bookmark manager.
+    START_FROM_PROFILE_MENU = 5,  // Sync was started from multiprofile menu.
+    START_FROM_URL = 6,  // Sync was started from a typed URL.
 
     // Events regarding cancellation of the signon process of sync.
     CANCEL_FROM_SIGNON_WITHOUT_AUTH = 10,   // Cancelled before submitting
@@ -120,14 +150,31 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
     CLEAR_SUCCEEDED = 4,
   };
 
+  enum StartBehavior {
+    AUTO_START,
+    MANUAL_START,
+  };
+
+  enum PassphraseType {
+    IMPLICIT,
+    EXPLICIT,
+  };
+
+  enum PassphraseSource {
+    INTERNAL,
+    USER_PROVIDED,
+  };
+
   // Default sync server URL.
   static const char* kSyncServerUrl;
   // Sync server URL for dev channel users
   static const char* kDevServerUrl;
 
-  ProfileSyncService(ProfileSyncFactory* factory_,
+  // Takes ownership of |factory|.
+  ProfileSyncService(ProfileSyncComponentsFactory* factory,
                      Profile* profile,
-                     const std::string& cros_user);
+                     SigninManager* signin,
+                     StartBehavior start_behavior);
   virtual ~ProfileSyncService();
 
   // Initializes the object. This should be called every time an object of this
@@ -136,13 +183,13 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
 
   void RegisterAuthNotifications();
 
-  // Return whether all sync tokens are loaded and
-  // available for the backend to start up.
+  // Same as AreCredentialsAvailable(false).
   bool AreCredentialsAvailable();
 
-  // Loads credentials migrated from the old user settings db.
-  void LoadMigratedCredentials(const std::string& username,
-                               const std::string& token);
+  // Return whether all sync tokens are loaded and available for the backend to
+  // start up. Also checks for OAuth login token if |check_oauth_login_token| is
+  // true.
+  bool AreCredentialsAvailable(bool check_oauth_login_token);
 
   // Registers a data type controller with the sync service.  This
   // makes the data type controller available for use, it does not
@@ -176,21 +223,31 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   virtual void SetSyncSetupCompleted();
 
   // SyncFrontend implementation.
-  virtual void OnBackendInitialized();
-  virtual void OnSyncCycleCompleted();
-  virtual void OnAuthError();
-  virtual void OnStopSyncingPermanently();
-  virtual void OnClearServerDataFailed();
-  virtual void OnClearServerDataTimeout();
-  virtual void OnClearServerDataSucceeded();
-  virtual void OnPassphraseRequired(bool for_decryption);
-  virtual void OnPassphraseAccepted();
+  virtual void OnBackendInitialized(
+      const browser_sync::WeakHandle<browser_sync::JsBackend>& js_backend,
+      bool success) OVERRIDE;
+  virtual void OnSyncCycleCompleted() OVERRIDE;
+  virtual void OnSyncConfigureRetry() OVERRIDE;
+  virtual void OnAuthError() OVERRIDE;
+  virtual void OnStopSyncingPermanently() OVERRIDE;
+  virtual void OnClearServerDataFailed() OVERRIDE;
+  virtual void OnClearServerDataSucceeded() OVERRIDE;
+  virtual void OnPassphraseRequired(
+      sync_api::PassphraseRequiredReason reason,
+      const sync_pb::EncryptedData& pending_keys) OVERRIDE;
+  virtual void OnPassphraseAccepted() OVERRIDE;
+  virtual void OnEncryptedTypesChanged(
+      syncable::ModelTypeSet encrypted_types,
+      bool encrypt_everything) OVERRIDE;
+  virtual void OnEncryptionComplete() OVERRIDE;
+  virtual void OnMigrationNeededForTypes(
+      syncable::ModelTypeSet types) OVERRIDE;
+  virtual void OnDataTypesChanged(
+      syncable::ModelTypeSet to_add) OVERRIDE;
+  virtual void OnActionableError(
+      const browser_sync::SyncProtocolError& error) OVERRIDE;
 
-  // Called when a user enters credentials through UI.
-  virtual void OnUserSubmittedAuth(const std::string& username,
-                                   const std::string& password,
-                                   const std::string& captcha,
-                                   const std::string& access_code);
+  void OnClearServerDataTimeout();
 
   // Update the last auth error and notify observers of error state.
   void UpdateAuthErrorState(const GoogleServiceAuthError& error);
@@ -201,7 +258,7 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   // and all data types will be synced.  |sync_everything| means "sync all
   // current and future data types."
   virtual void OnUserChoseDatatypes(bool sync_everything,
-      const syncable::ModelTypeSet& chosen_types);
+      syncable::ModelTypeSet chosen_types);
 
   // Called when a user cancels any setup dialog (login, etc).
   virtual void OnUserCancelledDialog();
@@ -210,9 +267,7 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   browser_sync::SyncBackendHost::StatusSummary QuerySyncStatusSummary();
   virtual browser_sync::SyncBackendHost::Status QueryDetailedSyncStatus();
 
-  const GoogleServiceAuthError& GetAuthError() const {
-    return last_auth_error_;
-  }
+  virtual const GoogleServiceAuthError& GetAuthError() const;
 
   // Displays a dialog for the user to enter GAIA credentials and attempt
   // re-authentication, and returns true if it actually opened the dialog.
@@ -224,15 +279,24 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   bool WizardIsVisible() const {
     return wizard_.IsVisible();
   }
-  virtual void ShowLoginDialog(gfx::NativeWindow parent_window);
+
+  SyncSetupWizard& get_wizard() { return wizard_; }
+
+  // Shows the login screen of the Sync setup wizard.
+  virtual void ShowLoginDialog();
 
   // This method handles clicks on "sync error" UI, showing the appropriate
   // dialog for the error condition (relogin / enter passphrase).
-  virtual void ShowErrorUI(gfx::NativeWindow parent_window);
+  virtual void ShowErrorUI();
 
-  void ShowConfigure(gfx::NativeWindow parent_window);
-  void PromptForExistingPassphrase(gfx::NativeWindow parent_window);
-  void SigninForPassphraseMigration(gfx::NativeWindow parent_window);
+  // Shows the configure screen of the Sync setup wizard. If |sync_everything|
+  // is true, shows the corresponding page in the customize screen; otherwise,
+  // displays the page that gives the user the ability to select which data
+  // types to sync.
+  void ShowConfigure(bool sync_everything);
+
+  virtual void ShowSyncSetup(const std::string& sub_page);
+  void ShowSyncSetupWithWizard(SyncSetupWizard::State state);
 
   // Pretty-printed strings for a given StatusSummary.
   static std::string BuildSyncStatusSummaryText(
@@ -246,54 +310,39 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   // currently no need to do so) we need to protect this with a lock.
   // TODO(timsteele): What happens if the bookmark model is loaded, a change
   // takes place, and the backend isn't initialized yet?
-  bool sync_initialized() const { return backend_initialized_; }
+  virtual bool sync_initialized() const;
   virtual bool unrecoverable_error_detected() const;
   const std::string& unrecoverable_error_message() {
     return unrecoverable_error_message_;
   }
   tracked_objects::Location unrecoverable_error_location() {
-    return unrecoverable_error_location_.get() ?
-        *unrecoverable_error_location_.get() : tracked_objects::Location();
+    return unrecoverable_error_location_;
   }
 
-  bool UIShouldDepictAuthInProgress() const {
-    return is_auth_in_progress_;
-  }
+  // Tracks whether the user is currently authenticating or not. This is used
+  // by the sync_ui_util helper routines to allow the UI to properly display
+  // an "authenticating..." status message instead of an auth error when we are
+  // in the process of trying to update credentials.
+  // TODO(atwilson): This state should reside up in the UI or in a profile-
+  // specific SyncUIUtil object rather than in ProfileSyncService.
+  virtual bool UIShouldDepictAuthInProgress() const;
+  virtual void SetUIShouldDepictAuthInProgress(bool auth_in_progress);
 
-  bool tried_creating_explicit_passphrase() const {
-    return tried_creating_explicit_passphrase_;
-  }
+  // Returns true if OnPassphraseRequired has been called for any reason.
+  virtual bool IsPassphraseRequired() const;
 
-  bool tried_setting_explicit_passphrase() const {
-    return tried_setting_explicit_passphrase_;
-  }
+  // Returns true if OnPassphraseRequired has been called for decryption and
+  // we have an encrypted data type enabled.
+  virtual bool IsPassphraseRequiredForDecryption() const;
 
-  bool observed_passphrase_required() const {
-    return observed_passphrase_required_;
+  sync_api::PassphraseRequiredReason passphrase_required_reason() const {
+    return passphrase_required_reason_;
   }
 
-  bool passphrase_required_for_decryption() const {
-    return passphrase_required_for_decryption_;
-  }
-
-  // A timestamp marking the last time the service observed a transition from
-  // the SYNCING state to the READY state. Note that this does not reflect the
-  // last time we polled the server to see if there were any changes; the
-  // timestamp is only snapped when syncing takes place and we download or
-  // upload some bookmark entity.
-  const base::Time& last_synced_time() const { return last_synced_time_; }
-
   // Returns a user-friendly string form of last synced time (in minutes).
   virtual string16 GetLastSyncedTimeString() const;
 
-  // Returns the authenticated username of the sync user, or empty if none
-  // exists. It will only exist if the authentication service provider (e.g
-  // GAIA) has confirmed the username is authentic.
-  virtual string16 GetAuthenticatedUsername() const;
-
-  const std::string& last_attempted_user_email() const {
-    return last_attempted_user_email_;
-  }
+  ProfileSyncComponentsFactory* factory() { return factory_.get(); }
 
   // The profile we are syncing for.
   Profile* profile() const { return profile_; }
@@ -306,6 +355,10 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   // Returns true if |observer| has already been added as an observer.
   bool HasObserver(Observer* observer) const;
 
+  // Returns a weak pointer to the service's JsController.
+  // Overrideable for testing purposes.
+  virtual base::WeakPtr<browser_sync::JsController> GetJsController();
+
   // Record stats on various events.
   static void SyncEvent(SyncEventCodes code);
 
@@ -316,84 +369,158 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
 
   // Returns whether sync is managed, i.e. controlled by configuration
   // management. If so, the user is not allowed to configure sync.
-  bool IsManaged();
+  bool IsManaged() const;
 
   // UnrecoverableErrorHandler implementation.
   virtual void OnUnrecoverableError(
       const tracked_objects::Location& from_here,
-      const std::string& message);
+      const std::string& message) OVERRIDE;
 
-  browser_sync::SyncBackendHost* backend() { return backend_.get(); }
+  // The functions below (until ActivateDataType()) should only be
+  // called if sync_initialized() is true.
 
+  // TODO(akalin): This is called mostly by ModelAssociators and
+  // tests.  Figure out how to pass the handle to the ModelAssociators
+  // directly, figure out how to expose this to tests, and remove this
+  // function.
+  virtual sync_api::UserShare* GetUserShare() const;
+
+  // TODO(akalin): These two functions are used only by
+  // ProfileSyncServiceHarness.  Figure out a different way to expose
+  // this info to that class, and remove these functions.
+
+  virtual const browser_sync::sessions::SyncSessionSnapshot*
+      GetLastSessionSnapshot() const;
+
+  // Returns whether or not the underlying sync engine has made any
+  // local changes to items that have not yet been synced with the
+  // server.
+  bool HasUnsyncedItems() const;
+
+  // Used by ProfileSyncServiceHarness.  May return NULL.
+  browser_sync::BackendMigrator* GetBackendMigratorForTest();
+
+  // Get the current routing information for all enabled model types.
+  // If a model type is not enabled (that is, if the syncer should not
+  // be trying to sync it), it is not in this map.
+  //
+  // TODO(akalin): This function is used by
+  // sync_ui_util::ConstructAboutInformation() and by some test
+  // classes.  Figure out a different way to expose this info and
+  // remove this function.
+  void GetModelSafeRoutingInfo(
+      browser_sync::ModelSafeRoutingInfo* out) const;
+
+  // Overridden by tests.
+  // TODO(zea): Remove these and have the dtc's call directly into the SBH.
   virtual void ActivateDataType(
-      browser_sync::DataTypeController* data_type_controller,
-      browser_sync::ChangeProcessor* change_processor);
-  virtual void DeactivateDataType(
-      browser_sync::DataTypeController* data_type_controller,
+      syncable::ModelType type, browser_sync::ModelSafeGroup group,
       browser_sync::ChangeProcessor* change_processor);
+  virtual void DeactivateDataType(syncable::ModelType type);
 
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // SyncPrefObserver implementation.
+  virtual void OnSyncManagedPrefChange(bool is_sync_managed) OVERRIDE;
+
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Changes which data types we're going to be syncing to |preferred_types|.
   // If it is running, the DataTypeManager will be instructed to reconfigure
   // the sync backend so that exactly these datatypes are actively synced.  See
   // class comment for more on what it means for a datatype to be Preferred.
   virtual void ChangePreferredDataTypes(
-      const syncable::ModelTypeSet& preferred_types);
+      syncable::ModelTypeSet preferred_types);
 
   // Get the set of currently enabled data types (as chosen or configured by
   // the user).  See class comment for more on what it means for a datatype
   // to be Preferred.
-  virtual void GetPreferredDataTypes(
-      syncable::ModelTypeSet* preferred_types) const;
+  virtual syncable::ModelTypeSet GetPreferredDataTypes() const;
 
   // Gets the set of all data types that could be allowed (the set that
   // should be advertised to the user).  These will typically only change
   // via a command-line option.  See class comment for more on what it means
   // for a datatype to be Registered.
-  virtual void GetRegisteredDataTypes(
-      syncable::ModelTypeSet* registered_types) const;
+  virtual syncable::ModelTypeSet GetRegisteredDataTypes() const;
 
   // Checks whether the Cryptographer is ready to encrypt and decrypt updates
-  // for sensitive data types.
-  virtual bool IsCryptographerReady() const;
+  // for sensitive data types. Caller must be holding a
+  // syncapi::BaseTransaction to ensure thread safety.
+  virtual bool IsCryptographerReady(
+      const sync_api::BaseTransaction* trans) const;
 
-  // Returns true if a secondary passphrase is being used.
+  // Returns true if a secondary passphrase is being used. It is not legal
+  // to call this method before the backend is initialized.
   virtual bool IsUsingSecondaryPassphrase() const;
 
   // Sets the Cryptographer's passphrase, or caches it until that is possible.
   // This will check asynchronously whether the passphrase is valid and notify
   // ProfileSyncServiceObservers via the NotificationService when the outcome
   // is known.
-  // |is_explicit| is true if the call is in response to the user explicitly
-  // setting a passphrase as opposed to implicitly (from the users' perspective)
-  // using their Google Account password.  An implicit SetPassphrase will *not*
-  // *not* override an explicit passphrase set previously.
-  // |is_creation| is true if the call is in response to the user setting
-  // up a new passphrase, and false if it's being set in response to a prompt
-  // for an existing passphrase.
+  // |type| == EXPLICIT if the call is in response to the user setting a
+  // custom explicit passphrase as opposed to implicitly (from the users'
+  // perspective) using their Google Account password. Once an explicit
+  // passphrase is set, it can never be overwritten (not even by another
+  // explicit passphrase).
+  // |source| == USER_PROVIDED corresponds to the user having manually provided
+  // this passphrase. It should only be INTERNAL for passphrases intercepted
+  // from the Google Sign-in Success notification. Note that if the data is
+  // encrypted with an old Google Account password, the user may still have to
+  // provide an "implicit" passphrase.
   virtual void SetPassphrase(const std::string& passphrase,
-                             bool is_explicit,
-                             bool is_creation);
+                             PassphraseType type,
+                             PassphraseSource source);
+
+  // Turns on encryption for all data. Callers must call OnUserChoseDatatypes()
+  // after calling this to force the encryption to occur.
+  virtual void EnableEncryptEverything();
+
+  // Returns true if we are currently set to encrypt all the sync data. Note:
+  // this is based on the cryptographer's settings, so if the user has recently
+  // requested encryption to be turned on, this may not be true yet. For that,
+  // encryption_pending() must be checked.
+  virtual bool EncryptEverythingEnabled() const;
+
+  // Fills |encrypted_types| with the set of currently encrypted types. Does
+  // not account for types pending encryption.
+  virtual syncable::ModelTypeSet GetEncryptedDataTypes() const;
+
+  // Returns true if the syncer is waiting for new datatypes to be encrypted.
+  virtual bool encryption_pending() const;
 
   // Returns whether processing changes is allowed.  Check this before doing
   // any model-modifying operations.
   bool ShouldPushChanges();
 
   const GURL& sync_service_url() const { return sync_service_url_; }
-  SigninManager* signin() { return signin_.get(); }
-  const std::string& cros_user() const { return cros_user_; }
+  bool auto_start_enabled() const { return auto_start_enabled_; }
+  SigninManager* signin() const { return signin_; }
+
+  // Stops the sync backend and sets the flag for suppressing sync startup.
+  void StopAndSuppress();
+
+  // Resets the flag for suppressing sync startup and starts the sync backend.
+  void UnsuppressAndStart();
+
+  // Marks all currently registered types as "acknowledged" so we won't prompt
+  // the user about them any more.
+  void AcknowledgeSyncedTypes();
+
+  SyncGlobalError* sync_global_error() { return sync_global_error_.get(); }
+
+  virtual const FailedDatatypesHandler& failed_datatypes_handler();
+
+  browser_sync::DataTypeManager::ConfigureStatus configure_status() {
+    return configure_status_;
+  }
+
+  // ProfileKeyedService implementation.
+  virtual void Shutdown() OVERRIDE;
 
  protected:
-  // Used by ProfileSyncServiceMock only.
-  //
-  // TODO(akalin): Separate this class out into an abstract
-  // ProfileSyncService interface and a ProfileSyncServiceImpl class
-  // so we don't need this hack anymore.
-  ProfileSyncService();
+  // Used by test classes that derive from ProfileSyncService.
+  virtual browser_sync::SyncBackendHost* GetBackendForTest();
 
   // Helper to install and configure a data type manager.
   void ConfigureDataTypeManager();
@@ -402,23 +529,26 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   void StartUp();
   // Shuts down the backend sync components.
   // |sync_disabled| indicates if syncing is being disabled or not.
-  void Shutdown(bool sync_disabled);
-
-  // Methods to register and remove preferences.
-  void RegisterPreferences();
-  void ClearPreferences();
+  void ShutdownImpl(bool sync_disabled);
 
   // Return SyncCredentials from the TokenService.
   sync_api::SyncCredentials GetCredentials();
 
   // Test need to override this to create backends that allow setting up
   // initial conditions, such as populating sync nodes.
+  //
+  // TODO(akalin): Figure out a better way to do this.  Ideally, we'd
+  // construct the backend outside this class and pass it in to the
+  // contructor or Initialize().
   virtual void CreateBackend();
 
   const browser_sync::DataTypeController::TypeMap& data_type_controllers() {
     return data_type_controllers_;
   }
 
+  // Helper method for managing encryption UI.
+  bool IsEncryptedDatatypeEnabled() const;
+
   // The wizard will try to read the auth state out of the profile sync
   // service using this member. Captcha and error state are reflected.
   GoogleServiceAuthError last_auth_error_;
@@ -427,36 +557,25 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   // other threads.
   scoped_ptr<browser_sync::SyncBackendHost> backend_;
 
-  // Cache of the last name the client attempted to authenticate.
-  std::string last_attempted_user_email_;
-
-  // Whether the user has tried creating an explicit passphrase on this
-  // machine.
-  bool tried_creating_explicit_passphrase_;
-
-  // Whether the user has tried setting an explicit passphrase on this
-  // machine.
-  bool tried_setting_explicit_passphrase_;
-
-  // Whether we have seen a SYNC_PASSPHRASE_REQUIRED since initializing the
-  // backend, telling us that it is safe to send a passphrase down ASAP.
-  bool observed_passphrase_required_;
-
   // Was the last SYNC_PASSPHRASE_REQUIRED notification sent because it
-  // was required for decryption?
-  bool passphrase_required_for_decryption_;
-
-  // Is the user in a passphrase migration?
-  bool passphrase_migration_in_progress_;
+  // was required for encryption, decryption with a cached passphrase, or
+  // because a new passphrase is required?
+  sync_api::PassphraseRequiredReason passphrase_required_reason_;
 
  private:
-  friend class ProfileSyncServiceTest;
   friend class ProfileSyncServicePasswordTest;
-  friend class ProfileSyncServicePreferenceTest;
-  friend class ProfileSyncServiceSessionTest;
+  friend class TestProfileSyncService;
+  friend class ProfileSyncServiceForWizardTest;
   FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceTest, InitialState);
-  FRIEND_TEST_ALL_PREFIXES(ProfileSyncServiceTest,
-                           UnrecoverableErrorSuspendsService);
+
+  // Starts up sync if it is not suppressed and preconditions are met.
+  // Called from Initialize() and UnsuppressAndStart().
+  void TryStart();
+
+  // Called when we've determined that we don't need a passphrase (either
+  // because OnPassphraseAccepted() was called, or because we've gotten a
+  // OnPassphraseRequired() but no data types are enabled).
+  void ResolvePassphraseRequired();
 
   // If |delete_sync_data_folder| is true, then this method will delete all
   // previous "Sync Data" folders. (useful if the folder is partial/corrupt).
@@ -470,7 +589,31 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
 
   void NotifyObservers();
 
-  static const char* GetPrefNameForDataType(syncable::ModelType data_type);
+  void ClearStaleErrors();
+
+  void ClearUnrecoverableError();
+
+  // About-flags experiment names for datatypes that aren't enabled by default
+  // yet.
+  static std::string GetExperimentNameForDataType(
+      syncable::ModelType data_type);
+
+  // Create and register a new datatype controller.
+  void RegisterNewDataType(syncable::ModelType data_type);
+
+  // Helper method to process SyncConfigureDone after unwinding the stack that
+  // originally posted this SyncConfigureDone.
+  void OnSyncConfigureDone(
+      browser_sync::DataTypeManager::ConfigureResult result);
+
+  // Reconfigures the data type manager with the latest enabled types.
+  // Note: Does not initialize the backend if it is not already initialized.
+  // This function needs to be called only after sync has been initialized
+  // (i.e.,only for reconfigurations). The reason we don't initialize the
+  // backend is because if we had encountered an unrecoverable error we dont
+  // want to startup once more.
+  virtual void ReconfigureDatatypeManager();
+
 
   // Time at which we begin an attempt a GAIA authorization.
   base::TimeTicks auth_start_time_;
@@ -479,13 +622,14 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   base::TimeTicks auth_error_time_;
 
   // Factory used to create various dependent objects.
-  ProfileSyncFactory* factory_;
+  scoped_ptr<ProfileSyncComponentsFactory> factory_;
 
   // The profile whose data we are synchronizing.
   Profile* profile_;
 
-  // Email for the ChromiumOS user, if we're running under ChromiumOS.
-  std::string cros_user_;
+  // The class that handles getting, setting, and persisting sync
+  // preferences.
+  browser_sync::SyncPrefs sync_prefs_;
 
   // TODO(ncarter): Put this in a profile, once there is UI for it.
   // This specifies where to find the sync server.
@@ -509,8 +653,9 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
 
   SyncSetupWizard wizard_;
 
-  // Encapsulates user signin with TokenService.
-  scoped_ptr<SigninManager> signin_;
+  // Encapsulates user signin - used to set/get the user's authenticated
+  // email address.
+  SigninManager* signin_;
 
   // True if an unrecoverable error (e.g. violation of an assumed invariant)
   // occurred during syncer operation.  This value should be checked before
@@ -519,53 +664,36 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
 
   // A message sent when an unrecoverable error occurred.
   std::string unrecoverable_error_message_;
-  scoped_ptr<tracked_objects::Location> unrecoverable_error_location_;
-
-  // Contains options specific to how sync clients send and listen to
-  // notifications.
-  notifier::NotifierOptions notifier_options_;
+  tracked_objects::Location unrecoverable_error_location_;
 
   // Manages the start and stop of the various data types.
   scoped_ptr<browser_sync::DataTypeManager> data_type_manager_;
 
   ObserverList<Observer> observers_;
 
-  NotificationRegistrar registrar_;
+  browser_sync::SyncJsController sync_js_controller_;
 
-  ScopedRunnableMethodFactory<ProfileSyncService>
-      scoped_runnable_method_factory_;
+  content::NotificationRegistrar registrar_;
 
-  // The preference that controls whether sync is under control by configuration
-  // management.
-  BooleanPrefMember pref_sync_managed_;
+  base::WeakPtrFactory<ProfileSyncService> weak_factory_;
 
   // This allows us to gracefully handle an ABORTED return code from the
   // DataTypeManager in the event that the server informed us to cease and
   // desist syncing immediately.
   bool expect_sync_configuration_aborted_;
 
-  scoped_ptr<TokenMigrator> token_migrator_;
-
   // Sometimes we need to temporarily hold on to a passphrase because we don't
   // yet have a backend to send it to.  This happens during initialization as
   // we don't StartUp until we have a valid token, which happens after valid
   // credentials were provided.
-  struct CachedPassphrase {
-    std::string value;
-    bool is_explicit;
-    bool is_creation;
-    CachedPassphrase() : is_explicit(false), is_creation(false) {}
+  struct CachedPassphrases {
+    std::string explicit_passphrase;
+    std::string gaia_passphrase;
+    // This distinguishes from GAIA passphrases intercepted by the signin code
+    // (which will always be the most recent GAIA passphrase).
+    bool user_provided_gaia;
   };
-  CachedPassphrase cached_passphrase_;
-
-  // TODO(tim): Remove this once new 'explicit passphrase' code flushes through
-  // dev channel. See bug 62103.
-  // To "migrate" early adopters of password sync on dev channel to the new
-  // model that stores their secondary passphrase preference in the cloud, we
-  // need some extra state since this cloud pref will be empty for all of them
-  // regardless of how they set up sync, and we can't trust
-  // kSyncUsingSecondaryPassphrase due to bugs in that implementation.
-  bool tried_implicit_gaia_remove_when_bug_62103_fixed_;
+  CachedPassphrases cached_passphrases_;
 
   // Keep track of where we are in a server clear operation
   ClearServerDataState clear_server_data_state_;
@@ -577,7 +705,46 @@ class ProfileSyncService : public browser_sync::SyncFrontend,
   // is reworked to allow one-shot commands like clearing server data.
   base::OneShotTimer<ProfileSyncService> clear_server_data_timer_;
 
+  // The current set of encrypted types.  Always a superset of
+  // Cryptographer::SensitiveTypes().
+  syncable::ModelTypeSet encrypted_types_;
+
+  // Whether we want to encrypt everything.
+  bool encrypt_everything_;
+
+  // Whether we're waiting for an attempt to encryption all sync data to
+  // complete. We track this at this layer in order to allow the user to cancel
+  // if they e.g. don't remember their explicit passphrase.
+  bool encryption_pending_;
+
+  // If true, we want to automatically start sync signin whenever we have
+  // credentials (user doesn't need to go through the startup flow). This is
+  // typically enabled on platforms (like ChromeOS) that have their own
+  // distinct signin flow.
+  const bool auto_start_enabled_;
+
+  scoped_ptr<browser_sync::BackendMigrator> migrator_;
+
+  // This is the last |SyncProtocolError| we received from the server that had
+  // an action set on it.
+  browser_sync::SyncProtocolError last_actionable_error_;
+
+  // This is used to show sync errors in the wrench menu.
+  scoped_ptr<SyncGlobalError> sync_global_error_;
+
+  // keeps track of data types that failed to load.
+  FailedDatatypesHandler failed_datatypes_handler_;
+
+  scoped_ptr<browser_sync::BackendUnrecoverableErrorHandler>
+      backend_unrecoverable_error_handler_;
+
+  browser_sync::DataTypeManager::ConfigureStatus configure_status_;
+
   DISALLOW_COPY_AND_ASSIGN(ProfileSyncService);
 };
 
+bool ShouldShowActionOnUI(
+    const browser_sync::SyncProtocolError& error);
+
+
 #endif  // CHROME_BROWSER_SYNC_PROFILE_SYNC_SERVICE_H_
