@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,32 +6,42 @@
 
 #include "testing/gtest/include/gtest/gtest.h"
 
-#include "base/task.h"
+#include "base/bind.h"
+#include "base/location.h"
+#include "base/synchronization/waitable_event.h"
+#include "base/test/test_timeouts.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/password_manager/password_store.h"
+#include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/signin/signin_manager.h"
+#include "chrome/browser/signin/signin_manager_factory.h"
 #include "chrome/browser/sync/abstract_profile_sync_service_test.h"
-#include "chrome/browser/sync/engine/syncapi.h"
 #include "chrome/browser/sync/glue/password_change_processor.h"
 #include "chrome/browser/sync/glue/password_data_type_controller.h"
 #include "chrome/browser/sync/glue/password_model_associator.h"
-#include "chrome/browser/sync/profile_sync_factory.h"
-#include "chrome/browser/sync/profile_sync_factory_mock.h"
+#include "chrome/browser/sync/internal_api/read_node.h"
+#include "chrome/browser/sync/internal_api/read_transaction.h"
+#include "chrome/browser/sync/internal_api/write_node.h"
+#include "chrome/browser/sync/internal_api/write_transaction.h"
+#include "chrome/browser/sync/profile_sync_components_factory.h"
+#include "chrome/browser/sync/profile_sync_components_factory_mock.h"
 #include "chrome/browser/sync/profile_sync_service.h"
 #include "chrome/browser/sync/profile_sync_test_util.h"
 #include "chrome/browser/sync/protocol/password_specifics.pb.h"
 #include "chrome/browser/sync/syncable/directory_manager.h"
 #include "chrome/browser/sync/syncable/syncable.h"
+#include "chrome/browser/sync/test/engine/test_id_factory.h"
 #include "chrome/browser/sync/test_profile_sync_service.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/net/gaia/gaia_constants.h"
-#include "chrome/common/notification_observer_mock.h"
-#include "chrome/common/notification_source.h"
-#include "chrome/common/notification_type.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/sync/engine/test_id_factory.h"
-#include "chrome/test/profile_mock.h"
+#include "chrome/test/base/profile_mock.h"
+#include "content/public/browser/notification_source.h"
+#include "content/test/notification_observer_mock.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gmock/include/gmock/gmock.h"
-#include "webkit/glue/password_form.h"
+#include "webkit/forms/password_form.h"
 
 using base::Time;
 using browser_sync::PasswordChangeProcessor;
@@ -39,6 +49,7 @@ using browser_sync::PasswordDataTypeController;
 using browser_sync::PasswordModelAssociator;
 using browser_sync::TestIdFactory;
 using browser_sync::UnrecoverableErrorHandler;
+using content::BrowserThread;
 using sync_api::SyncManager;
 using sync_api::UserShare;
 using syncable::BASE_VERSION;
@@ -57,15 +68,17 @@ using syncable::UNIQUE_SERVER_TAG;
 using syncable::UNITTEST;
 using syncable::WriteTransaction;
 using testing::_;
+using testing::AtLeast;
 using testing::DoAll;
 using testing::DoDefault;
 using testing::ElementsAre;
 using testing::Eq;
 using testing::Invoke;
+using testing::InvokeWithoutArgs;
 using testing::Return;
 using testing::SaveArg;
 using testing::SetArgumentPointee;
-using webkit_glue::PasswordForm;
+using webkit::forms::PasswordForm;
 
 ACTION_P3(MakePasswordSyncComponents, service, ps, dtc) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::DB));
@@ -73,8 +86,20 @@ ACTION_P3(MakePasswordSyncComponents, service, ps, dtc) {
       new PasswordModelAssociator(service, ps);
   PasswordChangeProcessor* change_processor =
       new PasswordChangeProcessor(model_associator, ps, dtc);
-  return ProfileSyncFactory::SyncComponents(model_associator,
-                                            change_processor);
+  return ProfileSyncComponentsFactory::SyncComponents(model_associator,
+                                                      change_processor);
+}
+
+ACTION_P(AcquireSyncTransaction, password_test_service) {
+  // Check to make sure we can aquire a transaction (will crash if a transaction
+  // is already held by this thread, deadlock if held by another thread).
+  sync_api::WriteTransaction trans(
+      FROM_HERE, password_test_service->GetUserShare());
+  DVLOG(1) << "Sync transaction acquired.";
+}
+
+static void QuitMessageLoop() {
+  MessageLoop::current()->Quit();
 }
 
 class MockPasswordStore : public PasswordStore {
@@ -101,119 +126,42 @@ class MockPasswordStore : public PasswordStore {
 
 class PasswordTestProfileSyncService : public TestProfileSyncService {
  public:
-  PasswordTestProfileSyncService(ProfileSyncFactory* factory,
-                                 Profile* profile,
-                                 const std::string& test_user,
-                                 bool synchronous_backend_initialization,
-                                 Task* initial_condition_setup_task,
-                                 Task* passphrase_accept_task)
-      : TestProfileSyncService(factory, profile, test_user,
+  PasswordTestProfileSyncService(
+      ProfileSyncComponentsFactory* factory,
+      Profile* profile,
+      SigninManager* signin,
+      bool synchronous_backend_initialization,
+      const base::Closure& initial_condition_setup_cb,
+      const base::Closure& passphrase_accept_cb)
+      : TestProfileSyncService(factory,
+                               profile,
+                               signin,
+                               ProfileSyncService::AUTO_START,
                                synchronous_backend_initialization,
-                               initial_condition_setup_task),
-        passphrase_accept_task_(passphrase_accept_task) {}
+                               initial_condition_setup_cb),
+        callback_(passphrase_accept_cb) {}
 
   virtual ~PasswordTestProfileSyncService() {}
 
-  virtual void OnPassphraseRequired(bool for_decryption) {
-    TestProfileSyncService::OnPassphraseRequired(for_decryption);
-    ADD_FAILURE();
-  }
-
   virtual void OnPassphraseAccepted() {
-    TestProfileSyncService::OnPassphraseAccepted();
+    if (!callback_.is_null())
+      callback_.Run();
 
-    if (passphrase_accept_task_) {
-      passphrase_accept_task_->Run();
-    }
+    TestProfileSyncService::OnPassphraseAccepted();
   }
 
  private:
-  Task* passphrase_accept_task_;
+  base::Closure callback_;
 };
 
 class ProfileSyncServicePasswordTest : public AbstractProfileSyncServiceTest {
- protected:
-  ProfileSyncServicePasswordTest()
-      : db_thread_(BrowserThread::DB) {
-  }
-
-  virtual void SetUp() {
-    password_store_ = new MockPasswordStore();
-    db_thread_.Start();
-
-    notification_service_ = new ThreadNotificationService(&db_thread_);
-    notification_service_->Init();
-    registrar_.Add(&observer_,
-        NotificationType::SYNC_CONFIGURE_DONE,
-        NotificationService::AllSources());
-  }
-
-  virtual void TearDown() {
-    service_.reset();
-    notification_service_->TearDown();
-    db_thread_.Stop();
-    MessageLoop::current()->RunAllPending();
-  }
-
-  void StartSyncService(Task* root_task, Task* node_task) {
-    StartSyncService(root_task, node_task, 2, 2);
-  }
-
-  void StartSyncService(Task* root_task, Task* node_task,
-                        int num_resume_expectations,
-                        int num_pause_expectations) {
-    if (!service_.get()) {
-      service_.reset(new PasswordTestProfileSyncService(
-          &factory_, &profile_, "test_user", false, root_task, node_task));
-      service_->RegisterPreferences();
-      profile_.GetPrefs()->SetBoolean(prefs::kSyncPasswords, true);
-      service_->set_num_expected_resumes(num_resume_expectations);
-      service_->set_num_expected_pauses(num_pause_expectations);
-      PasswordDataTypeController* data_type_controller =
-          new PasswordDataTypeController(&factory_,
-                                         &profile_,
-                                         service_.get());
-
-      EXPECT_CALL(factory_, CreatePasswordSyncComponents(_, _, _)).
-          WillOnce(MakePasswordSyncComponents(service_.get(),
-                                              password_store_.get(),
-                                              data_type_controller));
-      EXPECT_CALL(factory_, CreateDataTypeManager(_, _)).
-          WillOnce(ReturnNewDataTypeManager());
-
-      // We need tokens to get the tests going
-      token_service_.IssueAuthTokenForTest(
-          GaiaConstants::kSyncService, "token");
-
-      EXPECT_CALL(profile_, GetTokenService()).
-          WillRepeatedly(Return(&token_service_));
-
-      EXPECT_CALL(profile_, GetPasswordStore(_)).
-          Times(3).
-          WillRepeatedly(Return(password_store_.get()));
-
-      EXPECT_CALL(observer_,
-          Observe(
-              NotificationType(NotificationType::SYNC_CONFIGURE_DONE),_,_));
-
-      service_->RegisterDataTypeController(data_type_controller);
-      service_->Initialize();
-      MessageLoop::current()->Run();
-
-      EXPECT_CALL(
-          observer_,
-          Observe(
-              NotificationType(NotificationType::SYNC_CONFIGURE_DONE),
-              _,_)).
-          WillOnce(QuitUIMessageLoop());
-      service_->SetPassphrase("foo", false, true);
-      MessageLoop::current()->Run();
-    }
+ public:
+  sync_api::UserShare* GetUserShare() {
+    return service_->GetUserShare();
   }
 
   void AddPasswordSyncNode(const PasswordForm& entry) {
-    sync_api::WriteTransaction trans(
-        service_->backend()->GetUserShareHandle());
+    sync_api::WriteTransaction trans(FROM_HERE, service_->GetUserShare());
     sync_api::ReadNode password_root(&trans);
     ASSERT_TRUE(password_root.InitByTagLookup(browser_sync::kPasswordTag));
 
@@ -225,8 +173,109 @@ class ProfileSyncServicePasswordTest : public AbstractProfileSyncServiceTest {
     PasswordModelAssociator::WriteToSyncNode(entry, &node);
   }
 
+ protected:
+  ProfileSyncServicePasswordTest() {}
+
+  virtual void SetUp() {
+    AbstractProfileSyncServiceTest::SetUp();
+    profile_.CreateRequestContext();
+    password_store_ = new MockPasswordStore();
+
+    notification_service_ = new ThreadNotificationService(
+        db_thread_.DeprecatedGetThreadObject());
+    notification_service_->Init();
+    registrar_.Add(&observer_,
+        chrome::NOTIFICATION_SYNC_CONFIGURE_DONE,
+        content::NotificationService::AllSources());
+    registrar_.Add(&observer_,
+        chrome::NOTIFICATION_SYNC_CONFIGURE_BLOCKED,
+        content::NotificationService::AllSources());
+  }
+
+  virtual void TearDown() {
+    password_store_->Shutdown();
+    service_.reset();
+    notification_service_->TearDown();
+    profile_.ResetRequestContext();
+    AbstractProfileSyncServiceTest::TearDown();
+  }
+
+  static void SignalEvent(base::WaitableEvent* done) {
+    done->Signal();
+  }
+
+  void FlushLastDBTask() {
+    base::WaitableEvent done(false, false);
+    BrowserThread::PostTask(
+        BrowserThread::DB, FROM_HERE,
+        base::Bind(&ProfileSyncServicePasswordTest::SignalEvent, &done));
+    done.TimedWait(base::TimeDelta::FromMilliseconds(
+        TestTimeouts::action_timeout_ms()));
+  }
+
+  void StartSyncService(const base::Closure& root_callback,
+                        const base::Closure& node_callback) {
+    if (!service_.get()) {
+      SigninManager* signin = SigninManagerFactory::GetForProfile(&profile_);
+      signin->SetAuthenticatedUsername("test_user");
+      ProfileSyncComponentsFactoryMock* factory =
+          new ProfileSyncComponentsFactoryMock();
+      service_.reset(new PasswordTestProfileSyncService(
+          factory, &profile_, signin, false,
+          root_callback, node_callback));
+      syncable::ModelTypeSet preferred_types =
+          service_->GetPreferredDataTypes();
+      preferred_types.Put(syncable::PASSWORDS);
+      service_->ChangePreferredDataTypes(preferred_types);
+      EXPECT_CALL(profile_, GetProfileSyncService()).WillRepeatedly(
+          Return(service_.get()));
+      PasswordDataTypeController* data_type_controller =
+          new PasswordDataTypeController(factory,
+                                         &profile_,
+                                         service_.get());
+
+      EXPECT_CALL(*factory, CreatePasswordSyncComponents(_, _, _)).
+          Times(AtLeast(1)).  // Can be more if we hit NEEDS_CRYPTO.
+          WillRepeatedly(MakePasswordSyncComponents(service_.get(),
+                                                    password_store_.get(),
+                                                    data_type_controller));
+      EXPECT_CALL(*factory, CreateDataTypeManager(_, _)).
+          WillOnce(ReturnNewDataTypeManager());
+
+      // We need tokens to get the tests going
+      token_service_->IssueAuthTokenForTest(
+          GaiaConstants::kSyncService, "token");
+
+      EXPECT_CALL(profile_, GetTokenService()).
+          WillRepeatedly(Return(token_service_.get()));
+
+      EXPECT_CALL(profile_, GetPasswordStore(_)).
+          Times(AtLeast(2)).  // Can be more if we hit NEEDS_CRYPTO.
+          WillRepeatedly(Return(password_store_.get()));
+
+      EXPECT_CALL(observer_,
+          Observe(
+              int(chrome::NOTIFICATION_SYNC_CONFIGURE_DONE),_,_));
+      EXPECT_CALL(observer_,
+          Observe(
+              int(
+              chrome::NOTIFICATION_SYNC_CONFIGURE_BLOCKED),_,_))
+          .WillOnce(InvokeWithoutArgs(QuitMessageLoop));
+
+      service_->RegisterDataTypeController(data_type_controller);
+      service_->Initialize();
+      MessageLoop::current()->Run();
+      FlushLastDBTask();
+
+      service_->SetPassphrase("foo",
+                              ProfileSyncService::IMPLICIT,
+                              ProfileSyncService::INTERNAL);
+      MessageLoop::current()->Run();
+    }
+  }
+
   void GetPasswordEntriesFromSyncDB(std::vector<PasswordForm>* entries) {
-    sync_api::ReadTransaction trans(service_->backend()->GetUserShareHandle());
+    sync_api::ReadTransaction trans(FROM_HERE, service_->GetUserShare());
     sync_api::ReadNode password_root(&trans);
     ASSERT_TRUE(password_root.InitByTagLookup(browser_sync::kPasswordTag));
 
@@ -268,37 +317,21 @@ class ProfileSyncServicePasswordTest : public AbstractProfileSyncServiceTest {
     EXPECT_CALL(*password_store_, RemoveLoginImpl(_)).Times(0);
   }
 
-  friend class AddPasswordEntriesTask;
-
-  BrowserThread db_thread_;
   scoped_refptr<ThreadNotificationService> notification_service_;
-  NotificationObserverMock observer_;
+  content::NotificationObserverMock observer_;
   ProfileMock profile_;
   scoped_refptr<MockPasswordStore> password_store_;
-  NotificationRegistrar registrar_;
-
+  content::NotificationRegistrar registrar_;
 };
 
-class AddPasswordEntriesTask : public Task {
- public:
-  AddPasswordEntriesTask(ProfileSyncServicePasswordTest* test,
-                         const std::vector<PasswordForm>& entries)
-      : test_(test), entries_(entries) {
-  }
-
-  virtual void Run() {
-    for (size_t i = 0; i < entries_.size(); ++i) {
-      test_->AddPasswordSyncNode(entries_[i]);
-    }
-  }
-
- private:
-  ProfileSyncServicePasswordTest* test_;
-  const std::vector<PasswordForm>& entries_;
-};
+void AddPasswordEntriesCallback(ProfileSyncServicePasswordTest* test,
+                                const std::vector<PasswordForm>& entries) {
+  for (size_t i = 0; i < entries.size(); ++i)
+    test->AddPasswordSyncNode(entries[i]);
+}
 
 TEST_F(ProfileSyncServicePasswordTest, FailModelAssociation) {
-  StartSyncService(NULL, NULL, 1, 2);
+  StartSyncService(base::Closure(), base::Closure());
   EXPECT_TRUE(service_->unrecoverable_error_detected());
 }
 
@@ -308,8 +341,8 @@ TEST_F(ProfileSyncServicePasswordTest, EmptyNativeEmptySync) {
   EXPECT_CALL(*password_store_, FillBlacklistLogins(_))
       .WillOnce(Return(true));
   SetIdleChangeProcessorExpectations();
-  CreateRootTask task(this, syncable::PASSWORDS);
-  StartSyncService(&task, NULL);
+  CreateRootHelper create_root(this, syncable::PASSWORDS);
+  StartSyncService(create_root.callback(), base::Closure());
   std::vector<PasswordForm> sync_entries;
   GetPasswordEntriesFromSyncDB(&sync_entries);
   EXPECT_EQ(0U, sync_entries.size());
@@ -338,8 +371,8 @@ TEST_F(ProfileSyncServicePasswordTest, HasNativeEntriesEmptySync) {
   EXPECT_CALL(*password_store_, FillBlacklistLogins(_))
       .WillOnce(Return(true));
   SetIdleChangeProcessorExpectations();
-  CreateRootTask task(this, syncable::PASSWORDS);
-  StartSyncService(&task, NULL);
+  CreateRootHelper create_root(this, syncable::PASSWORDS);
+  StartSyncService(create_root.callback(), base::Closure());
   std::vector<PasswordForm> sync_forms;
   GetPasswordEntriesFromSyncDB(&sync_forms);
   ASSERT_EQ(1U, sync_forms.size());
@@ -390,8 +423,8 @@ TEST_F(ProfileSyncServicePasswordTest, HasNativeEntriesEmptySyncSameUsername) {
   EXPECT_CALL(*password_store_, FillBlacklistLogins(_))
       .WillOnce(Return(true));
   SetIdleChangeProcessorExpectations();
-  CreateRootTask task(this, syncable::PASSWORDS);
-  StartSyncService(&task, NULL);
+  CreateRootHelper create_root(this, syncable::PASSWORDS);
+  StartSyncService(create_root.callback(), base::Closure());
   std::vector<PasswordForm> sync_forms;
   GetPasswordEntriesFromSyncDB(&sync_forms);
   ASSERT_EQ(2U, sync_forms.size());
@@ -445,9 +478,74 @@ TEST_F(ProfileSyncServicePasswordTest, HasNativeHasSyncNoMerge) {
   EXPECT_CALL(*password_store_, FillBlacklistLogins(_)).WillOnce(Return(true));
   EXPECT_CALL(*password_store_, AddLoginImpl(_)).Times(1);
 
-  CreateRootTask root_task(this, syncable::PASSWORDS);
-  AddPasswordEntriesTask node_task(this, sync_forms);
-  StartSyncService(&root_task, &node_task);
+  CreateRootHelper create_root(this, syncable::PASSWORDS);
+  StartSyncService(create_root.callback(),
+                   base::Bind(&AddPasswordEntriesCallback, this, sync_forms));
+
+  std::vector<PasswordForm> new_sync_forms;
+  GetPasswordEntriesFromSyncDB(&new_sync_forms);
+
+  EXPECT_EQ(2U, new_sync_forms.size());
+  EXPECT_TRUE(ComparePasswords(expected_forms[0], new_sync_forms[0]));
+  EXPECT_TRUE(ComparePasswords(expected_forms[1], new_sync_forms[1]));
+}
+
+// Same as HasNativeHasEmptyNoMerge, but we attempt to aquire a sync transaction
+// every time the password store is accessed.
+TEST_F(ProfileSyncServicePasswordTest, EnsureNoTransactions) {
+  std::vector<PasswordForm*> native_forms;
+  std::vector<PasswordForm> sync_forms;
+  std::vector<PasswordForm> expected_forms;
+  {
+    PasswordForm* new_form = new PasswordForm;
+    new_form->scheme = PasswordForm::SCHEME_HTML;
+    new_form->signon_realm = "pie";
+    new_form->origin = GURL("http://pie.com");
+    new_form->action = GURL("http://pie.com/submit");
+    new_form->username_element = UTF8ToUTF16("name");
+    new_form->username_value = UTF8ToUTF16("tom");
+    new_form->password_element = UTF8ToUTF16("cork");
+    new_form->password_value = UTF8ToUTF16("password1");
+    new_form->ssl_valid = true;
+    new_form->preferred = false;
+    new_form->date_created = base::Time::FromInternalValue(1234);
+    new_form->blacklisted_by_user = false;
+
+    native_forms.push_back(new_form);
+    expected_forms.push_back(*new_form);
+  }
+
+  {
+    PasswordForm new_form;
+    new_form.scheme = PasswordForm::SCHEME_HTML;
+    new_form.signon_realm = "pie2";
+    new_form.origin = GURL("http://pie2.com");
+    new_form.action = GURL("http://pie2.com/submit");
+    new_form.username_element = UTF8ToUTF16("name2");
+    new_form.username_value = UTF8ToUTF16("tom2");
+    new_form.password_element = UTF8ToUTF16("cork2");
+    new_form.password_value = UTF8ToUTF16("password12");
+    new_form.ssl_valid = false;
+    new_form.preferred = true;
+    new_form.date_created = base::Time::FromInternalValue(12345);
+    new_form.blacklisted_by_user = false;
+    sync_forms.push_back(new_form);
+    expected_forms.push_back(new_form);
+  }
+
+  EXPECT_CALL(*password_store_, FillAutofillableLogins(_))
+      .WillOnce(DoAll(SetArgumentPointee<0>(native_forms),
+                      AcquireSyncTransaction(this),
+                      Return(true)));
+  EXPECT_CALL(*password_store_, FillBlacklistLogins(_))
+      .WillOnce(DoAll(AcquireSyncTransaction(this),
+                      Return(true)));
+  EXPECT_CALL(*password_store_, AddLoginImpl(_))
+      .WillOnce(AcquireSyncTransaction(this));
+
+  CreateRootHelper create_root(this, syncable::PASSWORDS);
+  StartSyncService(create_root.callback(),
+                   base::Bind(&AddPasswordEntriesCallback, this, sync_forms));
 
   std::vector<PasswordForm> new_sync_forms;
   GetPasswordEntriesFromSyncDB(&new_sync_forms);
@@ -518,10 +616,9 @@ TEST_F(ProfileSyncServicePasswordTest, HasNativeHasSyncMergeEntry) {
   EXPECT_CALL(*password_store_, FillBlacklistLogins(_)).WillOnce(Return(true));
   EXPECT_CALL(*password_store_, UpdateLoginImpl(_)).Times(1);
 
-  CreateRootTask root_task(this, syncable::PASSWORDS);
-  AddPasswordEntriesTask node_task(this, sync_forms);
-
-  StartSyncService(&root_task, &node_task);
+  CreateRootHelper create_root(this, syncable::PASSWORDS);
+  StartSyncService(create_root.callback(),
+                   base::Bind(&AddPasswordEntriesCallback, this, sync_forms));
 
   std::vector<PasswordForm> new_sync_forms;
   GetPasswordEntriesFromSyncDB(&new_sync_forms);
