@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,32 +6,43 @@
 
 #include "testing/gtest/include/gtest/gtest.h"
 
-#include "base/ref_counted.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
+#include "base/callback.h"
+#include "base/location.h"
+#include "base/memory/ref_counted.h"
 #include "base/string16.h"
 #include "base/threading/thread.h"
 #include "base/time.h"
+#include "base/utf_string_conversions.h"
 #include "chrome/browser/history/history_backend.h"
 #include "chrome/browser/history/history_notifications.h"
 #include "chrome/browser/history/history_types.h"
+#include "chrome/browser/signin/signin_manager.h"
+#include "chrome/browser/signin/signin_manager_factory.h"
 #include "chrome/browser/sync/abstract_profile_sync_service_test.h"
-#include "chrome/browser/sync/engine/syncapi.h"
 #include "chrome/browser/sync/glue/sync_backend_host.h"
 #include "chrome/browser/sync/glue/sync_backend_host_mock.h"
 #include "chrome/browser/sync/glue/typed_url_change_processor.h"
 #include "chrome/browser/sync/glue/typed_url_data_type_controller.h"
 #include "chrome/browser/sync/glue/typed_url_model_associator.h"
-#include "chrome/browser/sync/profile_sync_factory.h"
-#include "chrome/browser/sync/profile_sync_factory_mock.h"
+#include "chrome/browser/sync/internal_api/read_node.h"
+#include "chrome/browser/sync/internal_api/read_transaction.h"
+#include "chrome/browser/sync/internal_api/write_node.h"
+#include "chrome/browser/sync/internal_api/write_transaction.h"
+#include "chrome/browser/sync/profile_sync_components_factory.h"
+#include "chrome/browser/sync/profile_sync_components_factory_mock.h"
 #include "chrome/browser/sync/profile_sync_service.h"
 #include "chrome/browser/sync/profile_sync_test_util.h"
 #include "chrome/browser/sync/protocol/typed_url_specifics.pb.h"
 #include "chrome/browser/sync/syncable/directory_manager.h"
 #include "chrome/browser/sync/test_profile_sync_service.h"
+#include "chrome/browser/sync/test/engine/test_id_factory.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/net/gaia/gaia_constants.h"
-#include "chrome/common/notification_service.h"
-#include "chrome/test/profile_mock.h"
-#include "chrome/test/sync/engine/test_id_factory.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/profile_mock.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/public/browser/notification_service.h"
 #include "googleurl/src/gurl.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
@@ -72,64 +83,80 @@ using testing::Return;
 using testing::SetArgumentPointee;
 using testing::WithArgs;
 
+// Visits with this timestamp are treated as expired.
+static const int EXPIRED_VISIT = -1;
+
 class HistoryBackendMock : public HistoryBackend {
  public:
-  HistoryBackendMock() : HistoryBackend(FilePath(), NULL, NULL) {}
+  HistoryBackendMock() : HistoryBackend(FilePath(), 0, NULL, NULL) {}
+  virtual bool IsExpiredVisitTime(const base::Time& time) OVERRIDE {
+    return time.ToInternalValue() == EXPIRED_VISIT;
+  }
   MOCK_METHOD1(GetAllTypedURLs, bool(std::vector<history::URLRow>* entries));
-  MOCK_METHOD2(GetVisitsForURL, bool(history::URLID id,
-                                     history::VisitVector* visits));
+  MOCK_METHOD3(GetMostRecentVisitsForURL, bool(history::URLID id,
+                                               int max_visits,
+                                               history::VisitVector* visits));
   MOCK_METHOD2(UpdateURL, bool(history::URLID id, const history::URLRow& url));
   MOCK_METHOD3(AddVisits, bool(const GURL& url,
-                               const std::vector<base::Time>& visits,
+                               const std::vector<history::VisitInfo>& visits,
                                history::VisitSource visit_source));
   MOCK_METHOD1(RemoveVisits, bool(const history::VisitVector& visits));
   MOCK_METHOD2(GetURL, bool(const GURL& url_id, history::URLRow* url_row));
-  MOCK_METHOD2(SetPageTitle, void(const GURL& url, const std::wstring& title));
+  MOCK_METHOD2(SetPageTitle, void(const GURL& url, const string16& title));
   MOCK_METHOD1(DeleteURL, void(const GURL& url));
 };
 
 class HistoryServiceMock : public HistoryService {
  public:
   HistoryServiceMock() {}
-  MOCK_METHOD2(ScheduleDBTask, Handle(HistoryDBTask*,
-                                      CancelableRequestConsumerBase*));
+  MOCK_METHOD2(ScheduleDBTask, void(HistoryDBTask*,
+                                    CancelableRequestConsumerBase*));
 };
 
-class RunOnDBThreadTask : public Task {
- public:
-  RunOnDBThreadTask(HistoryBackend* backend, HistoryDBTask* task)
-      : backend_(backend), task_(task) {}
-  virtual void Run() {
-    task_->RunOnDBThread(backend_, NULL);
-    task_ = NULL;
-  }
- private:
-  HistoryBackend* backend_;
-  scoped_refptr<HistoryDBTask> task_;
-};
+void RunOnDBThreadCallback(HistoryBackend* backend,
+                           HistoryDBTask* task) {
+  task->RunOnDBThread(backend, NULL);
+}
 
 ACTION_P2(RunTaskOnDBThread, thread, backend) {
  thread->message_loop()->PostTask(
-    FROM_HERE,
-    new RunOnDBThreadTask(backend, arg0));
- return 0;
+    FROM_HERE, base::Bind(&RunOnDBThreadCallback, base::Unretained(backend),
+                          base::Unretained(arg0)));
 }
 
-ACTION_P3(MakeTypedUrlSyncComponents, service, hb, dtc) {
+ACTION_P4(MakeTypedUrlSyncComponents, profile, service, hb, dtc) {
   TypedUrlModelAssociator* model_associator =
       new TypedUrlModelAssociator(service, hb);
   TypedUrlChangeProcessor* change_processor =
-      new TypedUrlChangeProcessor(model_associator, hb, service);
-  return ProfileSyncFactory::SyncComponents(model_associator, change_processor);
+      new TypedUrlChangeProcessor(profile, model_associator, hb, service);
+  return ProfileSyncComponentsFactory::SyncComponents(model_associator,
+                                                      change_processor);
 }
 
 class ProfileSyncServiceTypedUrlTest : public AbstractProfileSyncServiceTest {
+ public:
+  void AddTypedUrlSyncNode(const history::URLRow& url,
+                           const history::VisitVector& visits) {
+    sync_api::WriteTransaction trans(FROM_HERE, service_->GetUserShare());
+    sync_api::ReadNode typed_url_root(&trans);
+    ASSERT_TRUE(typed_url_root.InitByTagLookup(browser_sync::kTypedUrlTag));
+
+    sync_api::WriteNode node(&trans);
+    std::string tag = url.url().spec();
+    ASSERT_TRUE(node.InitUniqueByCreation(syncable::TYPED_URLS,
+                                          typed_url_root,
+                                          tag));
+    TypedUrlModelAssociator::WriteToSyncNode(url, visits, &node);
+  }
+
  protected:
   ProfileSyncServiceTypedUrlTest()
       : history_thread_("history") {
   }
 
   virtual void SetUp() {
+    AbstractProfileSyncServiceTest::SetUp();
+    profile_.CreateRequestContext();
     history_backend_ = new HistoryBackendMock();
     history_service_ = new HistoryServiceMock();
     EXPECT_CALL((*history_service_.get()), ScheduleDBTask(_, _))
@@ -148,24 +175,36 @@ class ProfileSyncServiceTypedUrlTest : public AbstractProfileSyncServiceTest {
     service_.reset();
     notification_service_->TearDown();
     history_thread_.Stop();
-    MessageLoop::current()->RunAllPending();
+    profile_.ResetRequestContext();
+    AbstractProfileSyncServiceTest::TearDown();
   }
 
-  void StartSyncService(Task* task) {
+  void StartSyncService(const base::Closure& callback) {
     if (!service_.get()) {
+      SigninManager* signin = SigninManagerFactory::GetForProfile(&profile_);
+      signin->SetAuthenticatedUsername("test");
+      ProfileSyncComponentsFactoryMock* factory =
+          new ProfileSyncComponentsFactoryMock();
       service_.reset(
-          new TestProfileSyncService(&factory_, &profile_, "test", false,
-                                     task));
+          new TestProfileSyncService(factory,
+                                     &profile_,
+                                     signin,
+                                     ProfileSyncService::AUTO_START,
+                                     false,
+                                     callback));
+      EXPECT_CALL(profile_, GetProfileSyncService()).WillRepeatedly(
+          Return(service_.get()));
       TypedUrlDataTypeController* data_type_controller =
-          new TypedUrlDataTypeController(&factory_,
+          new TypedUrlDataTypeController(factory,
                                          &profile_,
                                          service_.get());
 
-      EXPECT_CALL(factory_, CreateTypedUrlSyncComponents(_, _, _)).
-          WillOnce(MakeTypedUrlSyncComponents(service_.get(),
+      EXPECT_CALL(*factory, CreateTypedUrlSyncComponents(_, _, _)).
+          WillOnce(MakeTypedUrlSyncComponents(&profile_,
+                                              service_.get(),
                                               history_backend_.get(),
                                               data_type_controller));
-      EXPECT_CALL(factory_, CreateDataTypeManager(_, _)).
+      EXPECT_CALL(*factory, CreateDataTypeManager(_, _)).
           WillOnce(ReturnNewDataTypeManager());
 
       EXPECT_CALL(profile_, GetHistoryServiceWithoutCreating()).
@@ -177,11 +216,11 @@ class ProfileSyncServiceTypedUrlTest : public AbstractProfileSyncServiceTest {
       EXPECT_CALL(profile_, GetHistoryService(_)).
           WillRepeatedly(Return(history_service_.get()));
 
-      token_service_.IssueAuthTokenForTest(
+      token_service_->IssueAuthTokenForTest(
           GaiaConstants::kSyncService, "token");
 
       EXPECT_CALL(profile_, GetTokenService()).
-          WillRepeatedly(Return(&token_service_));
+          WillRepeatedly(Return(token_service_.get()));
 
       service_->RegisterDataTypeController(data_type_controller);
 
@@ -190,23 +229,9 @@ class ProfileSyncServiceTypedUrlTest : public AbstractProfileSyncServiceTest {
     }
   }
 
-  void AddTypedUrlSyncNode(const history::URLRow& url,
-                           const history::VisitVector& visits) {
-    sync_api::WriteTransaction trans(
-        service_->backend()->GetUserShareHandle());
-    sync_api::ReadNode typed_url_root(&trans);
-    ASSERT_TRUE(typed_url_root.InitByTagLookup(browser_sync::kTypedUrlTag));
-
-    sync_api::WriteNode node(&trans);
-    std::string tag = url.url().spec();
-    ASSERT_TRUE(node.InitUniqueByCreation(syncable::TYPED_URLS,
-                                          typed_url_root,
-                                          tag));
-    TypedUrlModelAssociator::WriteToSyncNode(url, visits, &node);
-  }
-
   void GetTypedUrlsFromSyncDB(std::vector<history::URLRow>* urls) {
-    sync_api::ReadTransaction trans(service_->backend()->GetUserShareHandle());
+    urls->clear();
+    sync_api::ReadTransaction trans(FROM_HERE, service_->GetUserShare());
     sync_api::ReadNode typed_url_root(&trans);
     if (!typed_url_root.InitByTagLookup(browser_sync::kTypedUrlTag))
       return;
@@ -222,11 +247,10 @@ class ProfileSyncServiceTypedUrlTest : public AbstractProfileSyncServiceTest {
       history::URLRow new_url(GURL(typed_url.url()));
 
       new_url.set_title(UTF8ToUTF16(typed_url.title()));
-      new_url.set_typed_count(typed_url.typed_count());
-      DCHECK(typed_url.visit_size());
-      new_url.set_visit_count(typed_url.visit_size());
+      DCHECK(typed_url.visits_size());
+      DCHECK_EQ(typed_url.visits_size(), typed_url.visit_transitions_size());
       new_url.set_last_visit(base::Time::FromInternalValue(
-          typed_url.visit(typed_url.visit_size() - 1)));
+          typed_url.visits(typed_url.visits_size() - 1)));
       new_url.set_hidden(typed_url.hidden());
 
       urls->push_back(new_url);
@@ -242,10 +266,11 @@ class ProfileSyncServiceTypedUrlTest : public AbstractProfileSyncServiceTest {
   }
 
   static bool URLsEqual(history::URLRow& lhs, history::URLRow& rhs) {
+    // Only verify the fields we explicitly sync (i.e. don't verify typed_count
+    // or visit_count because we rely on the history DB to manage those values
+    // and they are left unchanged by HistoryBackendMock).
     return (lhs.url().spec().compare(rhs.url().spec()) == 0) &&
            (lhs.title().compare(rhs.title()) == 0) &&
-           (lhs.visit_count() == rhs.visit_count()) &&
-           (lhs.typed_count() == rhs.typed_count()) &&
            (lhs.last_visit() == rhs.last_visit()) &&
            (lhs.hidden() == rhs.hidden());
   }
@@ -256,59 +281,48 @@ class ProfileSyncServiceTypedUrlTest : public AbstractProfileSyncServiceTest {
                                            int64 last_visit,
                                            bool hidden,
                                            history::VisitVector* visits) {
+    // Give each URL a unique ID, to mimic the behavior of the real database.
+    static int unique_url_id = 0;
     GURL gurl(url);
-    URLRow history_url(gurl);
+    URLRow history_url(gurl, ++unique_url_id);
     history_url.set_title(UTF8ToUTF16(title));
     history_url.set_typed_count(typed_count);
     history_url.set_last_visit(
         base::Time::FromInternalValue(last_visit));
     history_url.set_hidden(hidden);
     visits->push_back(history::VisitRow(
-        history_url.id(), history_url.last_visit(), 0, 0, 0));
+        history_url.id(), history_url.last_visit(), 0,
+        content::PAGE_TRANSITION_TYPED, 0));
     history_url.set_visit_count(visits->size());
     return history_url;
   }
 
-  friend class AddTypedUrlEntriesTask;
-  friend class CreateTypedUrlRootTask;
-
   Thread history_thread_;
   scoped_refptr<ThreadNotificationService> notification_service_;
 
   ProfileMock profile_;
-  ProfileSyncFactoryMock factory_;
   scoped_refptr<HistoryBackendMock> history_backend_;
   scoped_refptr<HistoryServiceMock> history_service_;
 };
 
-class AddTypedUrlEntriesTask : public Task {
- public:
-  AddTypedUrlEntriesTask(ProfileSyncServiceTypedUrlTest* test,
-                         const std::vector<history::URLRow>& entries)
-      : test_(test), entries_(entries) {
+void AddTypedUrlEntries(ProfileSyncServiceTypedUrlTest* test,
+                        const std::vector<history::URLRow>& entries) {
+  test->CreateRoot(syncable::TYPED_URLS);
+  for (size_t i = 0; i < entries.size(); ++i) {
+    history::VisitVector visits;
+    visits.push_back(history::VisitRow(
+        entries[i].id(), entries[i].last_visit(), 0,
+        content::PageTransitionFromInt(0), 0));
+    test->AddTypedUrlSyncNode(entries[i], visits);
   }
-
-  virtual void Run() {
-    test_->CreateRoot(syncable::TYPED_URLS);
-    for (size_t i = 0; i < entries_.size(); ++i) {
-      history::VisitVector visits;
-      visits.push_back(history::VisitRow(
-          entries_[i].id(), entries_[i].last_visit(), 0, 0, 0));
-      test_->AddTypedUrlSyncNode(entries_[i], visits);
-    }
-  }
-
- private:
-  ProfileSyncServiceTypedUrlTest* test_;
-  const std::vector<history::URLRow>& entries_;
-};
+}
 
 TEST_F(ProfileSyncServiceTypedUrlTest, EmptyNativeEmptySync) {
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(Return(true));
   SetIdleChangeProcessorExpectations();
-  CreateRootTask task(this, syncable::TYPED_URLS);
-  StartSyncService(&task);
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
   std::vector<history::URLRow> sync_entries;
   GetTypedUrlsFromSyncDB(&sync_entries);
   EXPECT_EQ(0U, sync_entries.size());
@@ -322,17 +336,18 @@ TEST_F(ProfileSyncServiceTypedUrlTest, HasNativeEmptySync) {
 
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(DoAll(SetArgumentPointee<0>(entries), Return(true)));
-  EXPECT_CALL((*history_backend_.get()), GetVisitsForURL(_, _)).
-      WillRepeatedly(DoAll(SetArgumentPointee<1>(visits), Return(true)));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillRepeatedly(DoAll(SetArgumentPointee<2>(visits), Return(true)));
   SetIdleChangeProcessorExpectations();
-  CreateRootTask task(this, syncable::TYPED_URLS);
-  StartSyncService(&task);
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
   std::vector<history::URLRow> sync_entries;
   GetTypedUrlsFromSyncDB(&sync_entries);
   ASSERT_EQ(1U, sync_entries.size());
   EXPECT_TRUE(URLsEqual(entries[0], sync_entries[0]));
 }
 
+
 TEST_F(ProfileSyncServiceTypedUrlTest, HasNativeHasSyncNoMerge) {
   history::VisitVector native_visits;
   history::VisitVector sync_visits;
@@ -345,18 +360,17 @@ TEST_F(ProfileSyncServiceTypedUrlTest, HasNativeHasSyncNoMerge) {
   native_entries.push_back(native_entry);
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(DoAll(SetArgumentPointee<0>(native_entries), Return(true)));
-  EXPECT_CALL((*history_backend_.get()), GetVisitsForURL(_, _)).
-      WillRepeatedly(DoAll(SetArgumentPointee<1>(native_visits), Return(true)));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillRepeatedly(DoAll(SetArgumentPointee<2>(native_visits), Return(true)));
   EXPECT_CALL((*history_backend_.get()),
       AddVisits(_, _, history::SOURCE_SYNCED)).WillRepeatedly(Return(true));
 
   std::vector<history::URLRow> sync_entries;
   sync_entries.push_back(sync_entry);
-  AddTypedUrlEntriesTask task(this, sync_entries);
 
   EXPECT_CALL((*history_backend_.get()), UpdateURL(_, _)).
       WillRepeatedly(Return(true));
-  StartSyncService(&task);
+  StartSyncService(base::Bind(&AddTypedUrlEntries, this, sync_entries));
 
   std::map<std::string, history::URLRow> expected;
   expected[native_entry.url().spec()] = native_entry;
@@ -372,6 +386,23 @@ TEST_F(ProfileSyncServiceTypedUrlTest, HasNativeHasSyncNoMerge) {
   }
 }
 
+TEST_F(ProfileSyncServiceTypedUrlTest, EmptyNativeExpiredSync) {
+  history::VisitVector sync_visits;
+  history::URLRow sync_entry(MakeTypedUrlEntry("http://sync.com", "entry",
+                                               3, EXPIRED_VISIT, false,
+                                               &sync_visits));
+  std::vector<history::URLRow> sync_entries;
+  sync_entries.push_back(sync_entry);
+
+  // Since all our URLs are expired, no backend calls to add new URLs will be
+  // made.
+  EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
+      WillOnce(Return(true));
+  SetIdleChangeProcessorExpectations();
+
+  StartSyncService(base::Bind(&AddTypedUrlEntries, this, sync_entries));
+}
+
 TEST_F(ProfileSyncServiceTypedUrlTest, HasNativeHasSyncMerge) {
   history::VisitVector native_visits;
   history::URLRow native_entry(MakeTypedUrlEntry("http://native.com", "entry",
@@ -381,7 +412,8 @@ TEST_F(ProfileSyncServiceTypedUrlTest, HasNativeHasSyncMerge) {
                                                1, 17, false, &sync_visits));
   history::VisitVector merged_visits;
   merged_visits.push_back(history::VisitRow(
-      sync_entry.id(), base::Time::FromInternalValue(15), 0, 0, 0));
+      sync_entry.id(), base::Time::FromInternalValue(15), 0,
+      content::PageTransitionFromInt(0), 0));
 
   history::URLRow merged_entry(MakeTypedUrlEntry("http://native.com", "name",
                                                  2, 17, false, &merged_visits));
@@ -390,20 +422,19 @@ TEST_F(ProfileSyncServiceTypedUrlTest, HasNativeHasSyncMerge) {
   native_entries.push_back(native_entry);
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(DoAll(SetArgumentPointee<0>(native_entries), Return(true)));
-  EXPECT_CALL((*history_backend_.get()), GetVisitsForURL(_, _)).
-      WillRepeatedly(DoAll(SetArgumentPointee<1>(native_visits), Return(true)));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillRepeatedly(DoAll(SetArgumentPointee<2>(native_visits), Return(true)));
   EXPECT_CALL((*history_backend_.get()),
       AddVisits(_, _, history::SOURCE_SYNCED)). WillRepeatedly(Return(true));
 
   std::vector<history::URLRow> sync_entries;
   sync_entries.push_back(sync_entry);
-  AddTypedUrlEntriesTask task(this, sync_entries);
 
   EXPECT_CALL((*history_backend_.get()), UpdateURL(_, _)).
       WillRepeatedly(Return(true));
   EXPECT_CALL((*history_backend_.get()), SetPageTitle(_, _)).
       WillRepeatedly(Return());
-  StartSyncService(&task);
+  StartSyncService(base::Bind(&AddTypedUrlEntries, this, sync_entries));
 
   std::vector<history::URLRow> new_sync_entries;
   GetTypedUrlsFromSyncDB(&new_sync_entries);
@@ -418,18 +449,19 @@ TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeAdd) {
 
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(Return(true));
-  EXPECT_CALL((*history_backend_.get()), GetVisitsForURL(_, _)).
-      WillOnce(DoAll(SetArgumentPointee<1>(added_visits), Return(true)));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillOnce(DoAll(SetArgumentPointee<2>(added_visits), Return(true)));
 
   SetIdleChangeProcessorExpectations();
-  CreateRootTask task(this, syncable::TYPED_URLS);
-  StartSyncService(&task);
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
 
   history::URLsModifiedDetails details;
   details.changed_urls.push_back(added_entry);
   scoped_refptr<ThreadNotifier> notifier(new ThreadNotifier(&history_thread_));
-  notifier->Notify(NotificationType::HISTORY_TYPED_URLS_MODIFIED,
-                   Details<history::URLsModifiedDetails>(&details));
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_TYPED_URLS_MODIFIED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLsModifiedDetails>(&details));
 
   std::vector<history::URLRow> new_sync_entries;
   GetTypedUrlsFromSyncDB(&new_sync_entries);
@@ -447,22 +479,26 @@ TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeUpdate) {
 
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(DoAll(SetArgumentPointee<0>(original_entries), Return(true)));
-  EXPECT_CALL((*history_backend_.get()), GetVisitsForURL(_, _)).
-      WillRepeatedly(DoAll(SetArgumentPointee<1>(original_visits),
-                           Return(true)));
-  CreateRootTask task(this, syncable::TYPED_URLS);
-  StartSyncService(&task);
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillOnce(DoAll(SetArgumentPointee<2>(original_visits),
+                     Return(true)));
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
 
   history::VisitVector updated_visits;
   history::URLRow updated_entry(MakeTypedUrlEntry("http://mine.com", "entry",
-                                                  7, 15, false,
+                                                  7, 17, false,
                                                   &updated_visits));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillOnce(DoAll(SetArgumentPointee<2>(updated_visits),
+                     Return(true)));
 
   history::URLsModifiedDetails details;
   details.changed_urls.push_back(updated_entry);
   scoped_refptr<ThreadNotifier> notifier(new ThreadNotifier(&history_thread_));
-  notifier->Notify(NotificationType::HISTORY_TYPED_URLS_MODIFIED,
-                   Details<history::URLsModifiedDetails>(&details));
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_TYPED_URLS_MODIFIED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLsModifiedDetails>(&details));
 
   std::vector<history::URLRow> new_sync_entries;
   GetTypedUrlsFromSyncDB(&new_sync_entries);
@@ -470,6 +506,142 @@ TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeUpdate) {
   EXPECT_TRUE(URLsEqual(updated_entry, new_sync_entries[0]));
 }
 
+TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeAddFromVisit) {
+  history::VisitVector added_visits;
+  history::URLRow added_entry(MakeTypedUrlEntry("http://added.com", "entry",
+                                                2, 15, false, &added_visits));
+
+  EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
+      WillOnce(Return(true));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillOnce(DoAll(SetArgumentPointee<2>(added_visits), Return(true)));
+
+  SetIdleChangeProcessorExpectations();
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
+
+  history::URLVisitedDetails details;
+  details.row = added_entry;
+  details.transition = content::PAGE_TRANSITION_TYPED;
+  scoped_refptr<ThreadNotifier> notifier(new ThreadNotifier(&history_thread_));
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_URL_VISITED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLVisitedDetails>(&details));
+
+  std::vector<history::URLRow> new_sync_entries;
+  GetTypedUrlsFromSyncDB(&new_sync_entries);
+  ASSERT_EQ(1U, new_sync_entries.size());
+  EXPECT_TRUE(URLsEqual(added_entry, new_sync_entries[0]));
+}
+
+TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeUpdateFromVisit) {
+  history::VisitVector original_visits;
+  history::URLRow original_entry(MakeTypedUrlEntry("http://mine.com", "entry",
+                                                   2, 15, false,
+                                                   &original_visits));
+  std::vector<history::URLRow> original_entries;
+  original_entries.push_back(original_entry);
+
+  EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
+      WillOnce(DoAll(SetArgumentPointee<0>(original_entries), Return(true)));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillOnce(DoAll(SetArgumentPointee<2>(original_visits),
+                           Return(true)));
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
+
+  history::VisitVector updated_visits;
+  history::URLRow updated_entry(MakeTypedUrlEntry("http://mine.com", "entry",
+                                                  7, 17, false,
+                                                  &updated_visits));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillOnce(DoAll(SetArgumentPointee<2>(updated_visits),
+                           Return(true)));
+
+  history::URLVisitedDetails details;
+  details.row = updated_entry;
+  details.transition = content::PAGE_TRANSITION_TYPED;
+  scoped_refptr<ThreadNotifier> notifier(new ThreadNotifier(&history_thread_));
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_URL_VISITED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLVisitedDetails>(&details));
+
+  std::vector<history::URLRow> new_sync_entries;
+  GetTypedUrlsFromSyncDB(&new_sync_entries);
+  ASSERT_EQ(1U, new_sync_entries.size());
+  EXPECT_TRUE(URLsEqual(updated_entry, new_sync_entries[0]));
+}
+
+TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserIgnoreChangeUpdateFromVisit) {
+  history::VisitVector original_visits;
+  history::URLRow original_entry(MakeTypedUrlEntry("http://mine.com", "entry",
+                                                   2, 15, false,
+                                                   &original_visits));
+  std::vector<history::URLRow> original_entries;
+  original_entries.push_back(original_entry);
+
+  EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
+      WillOnce(DoAll(SetArgumentPointee<0>(original_entries), Return(true)));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillRepeatedly(DoAll(SetArgumentPointee<2>(original_visits),
+                           Return(true)));
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
+  std::vector<history::URLRow> new_sync_entries;
+  GetTypedUrlsFromSyncDB(&new_sync_entries);
+  ASSERT_EQ(1U, new_sync_entries.size());
+  EXPECT_TRUE(URLsEqual(original_entry, new_sync_entries[0]));
+
+  history::VisitVector updated_visits;
+  history::URLRow updated_entry(MakeTypedUrlEntry("http://mine.com", "entry",
+                                                  7, 15, false,
+                                                  &updated_visits));
+  history::URLVisitedDetails details;
+  details.row = updated_entry;
+
+  // Should ignore this change because it's not TYPED.
+  details.transition = content::PAGE_TRANSITION_RELOAD;
+  scoped_refptr<ThreadNotifier> notifier(new ThreadNotifier(&history_thread_));
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_URL_VISITED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLVisitedDetails>(&details));
+
+  GetTypedUrlsFromSyncDB(&new_sync_entries);
+
+  // Should be no changes to the sync DB from this notification.
+  ASSERT_EQ(1U, new_sync_entries.size());
+  EXPECT_TRUE(URLsEqual(original_entry, new_sync_entries[0]));
+
+  // Now, try updating it with a large number of visits not divisible by 10
+  // (should ignore this visit).
+  history::URLRow twelve_visits(MakeTypedUrlEntry("http://mine.com", "entry",
+                                                  12, 15, false,
+                                                  &updated_visits));
+  details.row = twelve_visits;
+  details.transition = content::PAGE_TRANSITION_TYPED;
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_URL_VISITED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLVisitedDetails>(&details));
+  GetTypedUrlsFromSyncDB(&new_sync_entries);
+  // Should be no changes to the sync DB from this notification.
+  ASSERT_EQ(1U, new_sync_entries.size());
+  EXPECT_TRUE(URLsEqual(original_entry, new_sync_entries[0]));
+
+  // Now, try updating it with a large number of visits that is divisible by 10
+  // (should *not* be ignored).
+  history::URLRow twenty_visits(MakeTypedUrlEntry("http://mine.com", "entry",
+                                                  20, 15, false,
+                                                  &updated_visits));
+  details.row = twenty_visits;
+  details.transition = content::PAGE_TRANSITION_TYPED;
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_URL_VISITED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLVisitedDetails>(&details));
+  GetTypedUrlsFromSyncDB(&new_sync_entries);
+  ASSERT_EQ(1U, new_sync_entries.size());
+  EXPECT_TRUE(URLsEqual(twenty_visits, new_sync_entries[0]));
+}
+
 TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeRemove) {
   history::VisitVector original_visits1;
   history::URLRow original_entry1(MakeTypedUrlEntry("http://mine.com", "entry",
@@ -486,18 +658,19 @@ TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeRemove) {
 
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(DoAll(SetArgumentPointee<0>(original_entries), Return(true)));
-  EXPECT_CALL((*history_backend_.get()), GetVisitsForURL(_, _)).
-      WillRepeatedly(DoAll(SetArgumentPointee<1>(original_visits1),
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillRepeatedly(DoAll(SetArgumentPointee<2>(original_visits1),
                            Return(true)));
-  CreateRootTask task(this, syncable::TYPED_URLS);
-  StartSyncService(&task);
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
 
   history::URLsDeletedDetails changes;
   changes.all_history = false;
   changes.urls.insert(GURL("http://mine.com"));
   scoped_refptr<ThreadNotifier> notifier(new ThreadNotifier(&history_thread_));
-  notifier->Notify(NotificationType::HISTORY_URLS_DELETED,
-                   Details<history::URLsDeletedDetails>(&changes));
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_URLS_DELETED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLsDeletedDetails>(&changes));
 
   std::vector<history::URLRow> new_sync_entries;
   GetTypedUrlsFromSyncDB(&new_sync_entries);
@@ -521,19 +694,50 @@ TEST_F(ProfileSyncServiceTypedUrlTest, ProcessUserChangeRemoveAll) {
 
   EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
       WillOnce(DoAll(SetArgumentPointee<0>(original_entries), Return(true)));
-  EXPECT_CALL((*history_backend_.get()), GetVisitsForURL(_, _)).
-      WillRepeatedly(DoAll(SetArgumentPointee<1>(original_visits1),
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillRepeatedly(DoAll(SetArgumentPointee<2>(original_visits1),
                            Return(true)));
-  CreateRootTask task(this, syncable::TYPED_URLS);
-  StartSyncService(&task);
+  CreateRootHelper create_root(this, syncable::TYPED_URLS);
+  StartSyncService(create_root.callback());
 
   history::URLsDeletedDetails changes;
   changes.all_history = true;
   scoped_refptr<ThreadNotifier> notifier(new ThreadNotifier(&history_thread_));
-  notifier->Notify(NotificationType::HISTORY_URLS_DELETED,
-                   Details<history::URLsDeletedDetails>(&changes));
+  notifier->Notify(chrome::NOTIFICATION_HISTORY_URLS_DELETED,
+                   content::Source<Profile>(&profile_),
+                   content::Details<history::URLsDeletedDetails>(&changes));
 
   std::vector<history::URLRow> new_sync_entries;
   GetTypedUrlsFromSyncDB(&new_sync_entries);
   ASSERT_EQ(0U, new_sync_entries.size());
 }
+
+TEST_F(ProfileSyncServiceTypedUrlTest, FailWriteToHistoryBackend) {
+  history::VisitVector native_visits;
+  history::VisitVector sync_visits;
+  history::URLRow native_entry(MakeTypedUrlEntry("http://native.com", "entry",
+                                                 2, 15, false, &native_visits));
+  history::URLRow sync_entry(MakeTypedUrlEntry("http://sync.com", "entry",
+                                               3, 16, false, &sync_visits));
+
+  std::vector<history::URLRow> native_entries;
+  native_entries.push_back(native_entry);
+  EXPECT_CALL((*history_backend_.get()), GetAllTypedURLs(_)).
+      WillOnce(DoAll(SetArgumentPointee<0>(native_entries), Return(true)));
+  EXPECT_CALL((*history_backend_.get()), GetMostRecentVisitsForURL(_, _, _)).
+      WillRepeatedly(DoAll(SetArgumentPointee<2>(native_visits), Return(true)));
+  EXPECT_CALL((*history_backend_.get()),
+      AddVisits(_, _, history::SOURCE_SYNCED)).WillRepeatedly(Return(false));
+
+  std::vector<history::URLRow> sync_entries;
+  sync_entries.push_back(sync_entry);
+
+  EXPECT_CALL((*history_backend_.get()), UpdateURL(_, _)).
+      WillRepeatedly(Return(false));
+  StartSyncService(base::Bind(&AddTypedUrlEntries, this, sync_entries));
+  ASSERT_TRUE(
+      service_->failed_datatypes_handler().GetFailedTypes().Has(
+          syncable::TYPED_URLS));
+  ASSERT_EQ(
+      1u, service_->failed_datatypes_handler().GetFailedTypes().Size());
+}
