@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,18 +13,23 @@
 #define CHROME_BROWSER_SYNC_SESSIONS_SESSION_STATE_H_
 #pragma once
 
+#include <map>
 #include <set>
+#include <string>
+#include <utility>
 #include <vector>
 
 #include "base/basictypes.h"
 #include "chrome/browser/sync/engine/syncer_types.h"
 #include "chrome/browser/sync/engine/syncproto.h"
 #include "chrome/browser/sync/sessions/ordered_commit_set.h"
+#include "chrome/browser/sync/protocol/sync_protocol_error.h"
 #include "chrome/browser/sync/syncable/model_type.h"
+#include "chrome/browser/sync/syncable/model_type_payload_map.h"
 #include "chrome/browser/sync/syncable/syncable.h"
 
-namespace syncable {
-class DirectoryManager;
+namespace base {
+class DictionaryValue;
 }
 
 namespace browser_sync {
@@ -32,15 +37,34 @@ namespace sessions {
 
 class UpdateProgress;
 
+// A container for the source of a sync session. This includes the update
+// source, the datatypes triggering the sync session, and possible session
+// specific payloads which should be sent to the server.
+struct SyncSourceInfo {
+  SyncSourceInfo();
+  explicit SyncSourceInfo(const syncable::ModelTypePayloadMap& t);
+  SyncSourceInfo(
+      const sync_pb::GetUpdatesCallerInfo::GetUpdatesSource& u,
+      const syncable::ModelTypePayloadMap& t);
+  ~SyncSourceInfo();
+
+  // Caller takes ownership of the returned dictionary.
+  base::DictionaryValue* ToValue() const;
+
+  sync_pb::GetUpdatesCallerInfo::GetUpdatesSource updates_source;
+  syncable::ModelTypePayloadMap types;
+};
+
 // Data pertaining to the status of an active Syncer object.
 struct SyncerStatus {
   SyncerStatus();
+  ~SyncerStatus();
+
+  // Caller takes ownership of the returned dictionary.
+  base::DictionaryValue* ToValue() const;
 
   // True when we get such an INVALID_STORE error from the server.
   bool invalid_store;
-  // True iff we're stuck.
-  bool syncer_stuck;
-  bool syncing;
   int num_successful_commits;
   // This is needed for monitoring extensions activity.
   int num_successful_bookmark_commits;
@@ -48,11 +72,24 @@ struct SyncerStatus {
   // Download event counters.
   int num_updates_downloaded_total;
   int num_tombstone_updates_downloaded_total;
+
+  // If the syncer encountered a MIGRATION_DONE code, these are the types that
+  // the client must now "migrate", by purging and re-downloading all updates.
+  syncable::ModelTypeSet types_needing_local_migration;
+
+  // Overwrites due to conflict resolution counters.
+  int num_local_overwrites;
+  int num_server_overwrites;
 };
 
 // Counters for various errors that can occur repeatedly during a sync session.
+// TODO(lipalani) : Rename this structure to Error.
 struct ErrorCounters {
   ErrorCounters();
+
+  // Caller takes ownership of the returned dictionary.
+  base::DictionaryValue* ToValue() const;
+
   int num_conflicting_commits;
 
   // Number of commits hitting transient errors since the last successful
@@ -63,8 +100,21 @@ struct ErrorCounters {
   // transient errors. When any of these succeed, this counter is reset.
   // TODO(chron): Reduce number of weird counters we use.
   int consecutive_errors;
+
+  // Any protocol errors that we received during this sync session.
+  SyncProtocolError sync_protocol_error;
+
+  // Records the most recent results of PostCommit and GetUpdates commands.
+  SyncerError last_download_updates_result;
+  SyncerError last_post_commit_result;
+  SyncerError last_process_commit_response_result;
 };
 
+// Caller takes ownership of the returned dictionary.
+base::DictionaryValue* DownloadProgressMarkersToValue(
+    const std::string
+        (&download_progress_markers)[syncable::MODEL_TYPE_COUNT]);
+
 // An immutable snapshot of state from a SyncSession.  Convenient to use as
 // part of notifications as it is inherently thread-safe.
 struct SyncSessionSnapshot {
@@ -73,26 +123,42 @@ struct SyncSessionSnapshot {
       const ErrorCounters& errors,
       int64 num_server_changes_remaining,
       bool is_share_usable,
-      const syncable::ModelTypeBitSet& initial_sync_ended,
-      std::string download_progress_markers[syncable::MODEL_TYPE_COUNT],
+      syncable::ModelTypeSet initial_sync_ended,
+      const std::string
+          (&download_progress_markers)[syncable::MODEL_TYPE_COUNT],
       bool more_to_sync,
       bool is_silenced,
       int64 unsynced_count,
+      int num_blocking_conflicting_updates,
       int num_conflicting_updates,
-      bool did_commit_items);
+      bool did_commit_items,
+      const SyncSourceInfo& source,
+      size_t num_entries,
+      base::Time sync_start_time,
+      bool retry_scheduled);
   ~SyncSessionSnapshot();
 
+  // Caller takes ownership of the returned dictionary.
+  base::DictionaryValue* ToValue() const;
+
+  std::string ToString() const;
+
   const SyncerStatus syncer_status;
   const ErrorCounters errors;
   const int64 num_server_changes_remaining;
   const bool is_share_usable;
-  const syncable::ModelTypeBitSet initial_sync_ended;
+  const syncable::ModelTypeSet initial_sync_ended;
   const std::string download_progress_markers[syncable::MODEL_TYPE_COUNT];
   const bool has_more_to_sync;
   const bool is_silenced;
   const int64 unsynced_count;
+  const int num_blocking_conflicting_updates;
   const int num_conflicting_updates;
   const bool did_commit_items;
+  const SyncSourceInfo source;
+  const size_t num_entries;
+  base::Time sync_start_time;
+  const bool retry_scheduled;
 };
 
 // Tracks progress of conflicts and their resolution using conflict sets.
@@ -110,15 +176,22 @@ class ConflictProgress {
   std::set<ConflictSet*>::const_iterator ConflictSetsBegin() const;
   std::set<ConflictSet*>::const_iterator ConflictSetsEnd() const;
   std::set<ConflictSet*>::size_type ConflictSetsSize() const;
+  bool HasSimpleConflictItem(const syncable::Id& id) const;
 
   // Various mutators for tracking commit conflicts.
   void AddConflictingItemById(const syncable::Id& the_id);
   void EraseConflictingItemById(const syncable::Id& the_id);
   int ConflictingItemsSize() const { return conflicting_item_ids_.size(); }
-  std::set<syncable::Id>::iterator ConflictingItemsBegin();
-  std::set<syncable::Id>::const_iterator ConflictingItemsBeginConst() const;
+  std::set<syncable::Id>::const_iterator ConflictingItemsBegin() const;
   std::set<syncable::Id>::const_iterator ConflictingItemsEnd() const;
 
+  // Mutators for nonblocking conflicting items (see description below).
+  void AddNonblockingConflictingItemById(const syncable::Id& the_id);
+  void EraseNonblockingConflictingItemById(const syncable::Id& the_id);
+  int NonblockingConflictingItemsSize() const {
+    return nonblocking_conflicting_item_ids_.size();
+  }
+
   void MergeSets(const syncable::Id& set1, const syncable::Id& set2);
   void CleanupSets();
 
@@ -128,6 +201,10 @@ class ConflictProgress {
   std::map<syncable::Id, ConflictSet*> id_to_conflict_set_;
   std::set<ConflictSet*> conflict_sets_;
 
+  // Nonblocking conflicts are not processed by the conflict resolver, but
+  // they will be processed in the APPLY_UDPATES_TO_RESOLVE_CONFLICTS step.
+  std::set<syncable::Id> nonblocking_conflicting_item_ids_;
+
   // Whether a conflicting item was added or removed since
   // the last call to reset_progress_changed(), if any. In practice this
   // points to StatusController::is_dirty_.
@@ -162,6 +239,7 @@ class UpdateProgress {
   int VerifiedUpdatesSize() const { return verified_updates_.size(); }
   bool HasVerifiedUpdates() const { return !verified_updates_.empty(); }
   bool HasAppliedUpdates() const { return !applied_updates_.empty(); }
+  void ClearVerifiedUpdates() { verified_updates_.clear(); }
 
   // Count the number of successful update applications that have happend this
   // cycle. Note that if an item is successfully applied twice, it will be
@@ -182,18 +260,17 @@ class UpdateProgress {
 };
 
 struct SyncCycleControlParameters {
-  SyncCycleControlParameters() : conflict_sets_built(false),
-                                 conflicts_resolved(false),
-                                 items_committed(false) {}
-  // Set to true by BuildAndProcessConflictSetsCommand if the RESOLVE_CONFLICTS
-  // step is needed.
-  bool conflict_sets_built;
-
+  SyncCycleControlParameters() : conflicts_resolved(false),
+                                 items_committed(false),
+                                 debug_info_sent(false) {}
   // Set to true by ResolveConflictsCommand if any forward progress was made.
   bool conflicts_resolved;
 
   // Set to true by PostCommitMessageCommand if any commits were successful.
   bool items_committed;
+
+  // True indicates debug info has been sent once this session.
+  bool debug_info_sent;
 };
 
 // DirtyOnWrite wraps a value such that any write operation will update a
@@ -233,12 +310,12 @@ struct AllModelTypeState {
   ClientToServerResponse commit_response;
   // We GetUpdates for some combination of types at once.
   // requested_update_types stores the set of types which were requested.
-  syncable::ModelTypeBitSet updates_request_types;
+  syncable::ModelTypeSet updates_request_types;
   ClientToServerResponse updates_response;
   // Used to build the shared commit message.
   DirtyOnWrite<std::vector<int64> > unsynced_handles;
   DirtyOnWrite<SyncerStatus> syncer_status;
-  DirtyOnWrite<ErrorCounters> error_counters;
+  DirtyOnWrite<ErrorCounters> error;
   SyncCycleControlParameters control_params;
   DirtyOnWrite<int64> num_server_changes_remaining;
   OrderedCommitSet commit_set;
