@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,31 +11,46 @@
 
 #include "base/gtest_prod_util.h"
 #include "base/time.h"
-#include "chrome/browser/dom_ui/html_dialog_ui.h"
 #include "chrome/browser/sync/profile_sync_service.h"
 #include "chrome/browser/sync/sync_setup_wizard.h"
 #include "chrome/browser/sync/syncable/model_type.h"
-#include "gfx/native_widget_types.h"
-#include "grit/generated_resources.h"
+#include "chrome/browser/ui/webui/html_dialog_ui.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/gfx/native_widget_types.h"
 
-class FlowHandler;
+class SyncSetupFlowHandler;
 class SyncSetupFlowContainer;
 
 // A structure which contains all the configuration information for sync.
 // This can be stored or passed around when the configuration is managed
 // by multiple stages of the wizard.
 struct SyncConfiguration {
+  SyncConfiguration();
+  ~SyncConfiguration();
+
+  bool encrypt_all;
   bool sync_everything;
   syncable::ModelTypeSet data_types;
-  bool use_secondary_passphrase;
+  // We pass a separate |set_xxxxx_passphrase| flag because sometimes the UI
+  // wants to set an empty gaia/secondary passphrase (for example, when the user
+  // doesn't enter a passphrase, but we still want the ProfileSyncService to
+  // generate a new passphrase error if there are still encrypted types
+  // enabled).
+  // TODO(atwilson): Need to change SyncSetupFlow::OnUserConfigured() to
+  // check for the presence of encrypted types itself, rather than relying on
+  // the hack of passing an empty passphrase/waiting for ProfileSyncService to
+  // receive a new PassphraseRequired (http://crbug.com/95939).
+  bool set_secondary_passphrase;
   std::string secondary_passphrase;
+  bool set_gaia_passphrase;
+  std::string gaia_passphrase;
 };
 
 // The state machine used by SyncSetupWizard, exposed in its own header
 // to facilitate testing of SyncSetupWizard.  This class is used to open and
-// run the html dialog and deletes itself when the dialog closes.
-class SyncSetupFlow : public HtmlDialogUIDelegate {
+// run the sync setup overlay in tabbed options and deletes itself when the
+// overlay closes.
+class SyncSetupFlow {
  public:
   virtual ~SyncSetupFlow();
 
@@ -45,23 +60,20 @@ class SyncSetupFlow : public HtmlDialogUIDelegate {
   static SyncSetupFlow* Run(ProfileSyncService* service,
                             SyncSetupFlowContainer* container,
                             SyncSetupWizard::State start,
-                            SyncSetupWizard::State end,
-                            gfx::NativeWindow parent_window);
+                            SyncSetupWizard::State end);
 
-  // Fills |args| with "user" and "error" arguments by querying |service|.
-  static void GetArgsForGaiaLogin(
-      const ProfileSyncService* service,
-      DictionaryValue* args);
+  // Fills |args| with "user" and "error" arguments per our current state.
+  void GetArgsForGaiaLogin(DictionaryValue* args);
 
   // Fills |args| for the configure screen (Choose Data Types/Encryption)
-  static void GetArgsForConfigure(
-      ProfileSyncService* service,
-      DictionaryValue* args);
+  void GetArgsForConfigure(DictionaryValue* args);
 
-  // Fills |args| for the enter passphrase screen.
-  static void GetArgsForEnterPassphrase(
-      const ProfileSyncService* service,
-      DictionaryValue* args);
+  // Attaches the |handler| to this flow. Returns true if successful and false
+  // if a handler has already been attached.
+  bool AttachSyncSetupHandler(SyncSetupFlowHandler* handler);
+
+  // Returns true if a handler is attached to this flow.
+  bool IsAttached() const;
 
   // Triggers a state machine transition to advance_state.
   void Advance(SyncSetupWizard::State advance_state);
@@ -70,55 +82,26 @@ class SyncSetupFlow : public HtmlDialogUIDelegate {
   // obscured by a browser window.
   void Focus();
 
-  // HtmlDialogUIDelegate implementation.
-  // Get the HTML file path for the content to load in the dialog.
-  virtual GURL GetDialogContentURL() const;
-
-  // HtmlDialogUIDelegate implementation.
-  virtual void GetDOMMessageHandlers(
-      std::vector<DOMMessageHandler*>* handlers) const;
-
-  // HtmlDialogUIDelegate implementation.
-  // Get the size of the dialog.
-  virtual void GetDialogSize(gfx::Size* size) const;
-
-  // HtmlDialogUIDelegate implementation.
-  // Gets the JSON string input to use when opening the dialog.
-  virtual std::string GetDialogArgs() const;
-
-  // HtmlDialogUIDelegate implementation.
-  // A callback to notify the delegate that the dialog closed.
-  virtual void OnDialogClosed(const std::string& json_retval);
-
-  // HtmlDialogUIDelegate implementation.
-  virtual void OnCloseContents(TabContents* source, bool* out_close_dialog) {}
-
-  // HtmlDialogUIDelegate implementation.
-  virtual std::wstring GetDialogTitle() const;
-
-  // HtmlDialogUIDelegate implementation.
-  virtual bool IsDialogModal() const;
-  virtual bool ShouldShowDialogTitle() const;
-
   void OnUserSubmittedAuth(const std::string& username,
                            const std::string& password,
                            const std::string& captcha,
                            const std::string& access_code);
 
+  void OnUserSubmittedOAuth(const std::string& oauth1_request_token);
+
   void OnUserConfigured(const SyncConfiguration& configuration);
 
   // The 'passphrase' screen is used when the user is prompted to enter
   // an existing passphrase.
   void OnPassphraseEntry(const std::string& passphrase);
 
-  // The 'first passphrase' screen is for users migrating from a build
-  // without passwords, who are prompted to make a passphrase choice.
-  void OnFirstPassphraseEntry(const std::string& option,
-                              const std::string& passphrase);
+  // The user canceled the passphrase entry without supplying a passphrase.
+  void OnPassphraseCancel();
 
-  void OnGoToDashboard();
+  void OnDialogClosed(const std::string& json_retval);
 
  private:
+  friend class ProfileSyncServiceForWizardTest;
   FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, InitialStepLogin);
   FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, ChooseDataTypesSetsPrefs);
   FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, DialogCancelled);
@@ -130,12 +113,16 @@ class SyncSetupFlow : public HtmlDialogUIDelegate {
   FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest,
                            DiscreteRunChooseDataTypesAbortedByPendingClear);
   FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, EnterPassphraseRequired);
-  FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, PassphraseMigration);
+  FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, NonFatalError);
+  FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, CrosAuthSetup);
+  FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, ShowErrorUIForPasswordTest);
+  FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardTest, ShowErrorUIForPassphraseTest);
+  FRIEND_TEST_ALL_PREFIXES(SyncSetupWizardCrosTest, CrosAuthSetup);
 
   // Use static Run method to get an instance.
   SyncSetupFlow(SyncSetupWizard::State start_state,
                 SyncSetupWizard::State end_state,
-                const std::string& args, SyncSetupFlowContainer* container,
+                SyncSetupFlowContainer* container,
                 ProfileSyncService* service);
 
   // Returns true if |this| should transition its state machine to |state|
@@ -143,8 +130,9 @@ class SyncSetupFlow : public HtmlDialogUIDelegate {
   // a no-op.
   bool ShouldAdvance(SyncSetupWizard::State state);
 
+  void ActivateState(SyncSetupWizard::State state);
+
   SyncSetupFlowContainer* container_;  // Our container.  Don't own this.
-  std::string dialog_start_args_;  // The args to pass to the initial page.
 
   SyncSetupWizard::State current_state_;
   SyncSetupWizard::State end_state_;  // The goal.
@@ -152,17 +140,23 @@ class SyncSetupFlow : public HtmlDialogUIDelegate {
   // Time that the GAIA_LOGIN step was received.
   base::TimeTicks login_start_time_;
 
-  // The handler needed for the entire flow.
-  FlowHandler* flow_handler_;
-  mutable bool owns_flow_handler_;
+  // The handler needed for the entire flow. Weak reference.
+  SyncSetupFlowHandler* flow_handler_;
 
-  // We need this to write the sentinel "setup completed" pref.
+  // We need this to propagate back all user settings changes. Weak reference.
   ProfileSyncService* service_;
 
-  // Currently used only on OS X
-  // TODO(akalin): Add the necessary support to the other OSes and use
-  // this for them.
-  gfx::NativeWindow html_dialog_window_;
+  // Set to true if the user has tried creating an explicit passphrase, so we
+  // can appropriately reflect this in the UI.
+  bool user_tried_creating_explicit_passphrase_;
+
+  // Set to true if the user entered a passphrase, so we can appropriately
+  // reflect this in the UI.
+  bool user_tried_setting_passphrase_;
+
+  // Cache of the last name the client attempted to authenticate.
+  // TODO(atwilson): Move this out of here entirely and up into the UI layer.
+  std::string last_attempted_user_email_;
 
   DISALLOW_COPY_AND_ASSIGN(SyncSetupFlow);
 };
@@ -185,44 +179,4 @@ class SyncSetupFlowContainer {
   DISALLOW_COPY_AND_ASSIGN(SyncSetupFlowContainer);
 };
 
-// The FlowHandler connects the state machine to the dialog backing HTML and
-// JS namespace by implementing DOMMessageHandler and being invoked by the
-// SyncSetupFlow.  Exposed here to facilitate testing.
-class FlowHandler : public DOMMessageHandler {
- public:
-  FlowHandler()  {}
-  virtual ~FlowHandler() {}
-
-  // DOMMessageHandler implementation.
-  virtual void RegisterMessages();
-
-  // Callbacks from the page.
-  void HandleSubmitAuth(const ListValue* args);
-  void HandleConfigure(const ListValue* args);
-  void HandlePassphraseEntry(const ListValue* args);
-  void HandleFirstPassphrase(const ListValue* args);
-  void HandleGoToDashboard(const ListValue* args);
-
-  // These functions control which part of the HTML is visible.
-  void ShowGaiaLogin(const DictionaryValue& args);
-  void ShowGaiaSuccessAndClose();
-  void ShowGaiaSuccessAndSettingUp();
-  void ShowConfigure(const DictionaryValue& args);
-  void ShowPassphraseEntry(const DictionaryValue& args);
-  void ShowFirstPassphrase(const DictionaryValue& args);
-  void ShowSettingUp();
-  void ShowSetupDone(const std::wstring& user);
-  void ShowFirstTimeDone(const std::wstring& user);
-
-  void set_flow(SyncSetupFlow* flow) {
-    flow_ = flow;
-  }
-
- private:
-  void ExecuteJavascriptInIFrame(const std::wstring& iframe_xpath,
-                                 const std::wstring& js);
-  SyncSetupFlow* flow_;
-  DISALLOW_COPY_AND_ASSIGN(FlowHandler);
-};
-
 #endif  // CHROME_BROWSER_SYNC_SYNC_SETUP_FLOW_H_
