@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,24 +9,19 @@
 #include <iterator>
 
 #include "base/logging.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/port.h"
-#include "base/scoped_ptr.h"
+#include "base/string_util.h"
 #include "chrome/browser/sync/syncable/syncable.h"
-#include "chrome/common/deprecated/event_sys-inl.h"
 
 using browser_sync::Cryptographer;
+using browser_sync::UnrecoverableErrorHandler;
 
 namespace syncable {
 
 static const FilePath::CharType kSyncDataDatabaseFilename[] =
     FILE_PATH_LITERAL("SyncData.sqlite3");
 
-DirectoryManagerEvent DirectoryManagerShutdownEvent() {
-  DirectoryManagerEvent event;
-  event.what_happened = DirectoryManagerEvent::SHUTDOWN;
-  return event;
-}
-
 // static
 const FilePath DirectoryManager::GetSyncDataDatabaseFilename() {
   return FilePath(kSyncDataDatabaseFilename);
@@ -39,7 +34,6 @@ const FilePath DirectoryManager::GetSyncDataDatabasePath() const {
 DirectoryManager::DirectoryManager(const FilePath& path)
     : root_path_(path),
       managed_directory_(NULL),
-      channel_(new Channel(DirectoryManagerShutdownEvent())),
       cryptographer_(new Cryptographer) {
 }
 
@@ -47,26 +41,38 @@ DirectoryManager::~DirectoryManager() {
   base::AutoLock lock(lock_);
   DCHECK_EQ(managed_directory_, static_cast<Directory*>(NULL))
       << "Dir " << managed_directory_->name() << " not closed!";
-  delete channel_;
 }
 
-bool DirectoryManager::Open(const std::string& name) {
+bool DirectoryManager::Open(
+    const std::string& name,
+    DirectoryChangeDelegate* delegate,
+    UnrecoverableErrorHandler* unrecoverable_error_handler,
+    const browser_sync::WeakHandle<TransactionObserver>&
+        transaction_observer) {
   bool was_open = false;
-  const DirOpenResult result = OpenImpl(name,
-      GetSyncDataDatabasePath(), &was_open);
+  const DirOpenResult result =
+      OpenImpl(name, GetSyncDataDatabasePath(), delegate,
+               unrecoverable_error_handler,
+               transaction_observer, &was_open);
   return syncable::OPENED == result;
 }
 
 // Opens a directory.  Returns false on error.
-DirOpenResult DirectoryManager::OpenImpl(const std::string& name,
-                                         const FilePath& path,
-                                         bool* was_open) {
+DirOpenResult DirectoryManager::OpenImpl(
+    const std::string& name,
+    const FilePath& path,
+    DirectoryChangeDelegate* delegate,
+    UnrecoverableErrorHandler* unrecoverable_error_handler,
+    const browser_sync::WeakHandle<TransactionObserver>&
+        transaction_observer,
+    bool* was_open) {
   bool opened = false;
   {
     base::AutoLock lock(lock_);
     // Check to see if it's already open.
     if (managed_directory_) {
-      DCHECK_EQ(ComparePathNames(name, managed_directory_->name()), 0)
+      DCHECK_EQ(base::strcasecmp(name.c_str(),
+                                 managed_directory_->name().c_str()), 0)
           << "Can't open more than one directory.";
       opened = *was_open = true;
     }
@@ -76,8 +82,9 @@ DirOpenResult DirectoryManager::OpenImpl(const std::string& name,
     return syncable::OPENED;
   // Otherwise, open it.
 
-  scoped_ptr<Directory> dir(new Directory);
-  const DirOpenResult result = dir->Open(path, name);
+  scoped_ptr<Directory> dir(new Directory(unrecoverable_error_handler));
+  const DirOpenResult result =
+      dir->Open(path, name, delegate, transaction_observer);
   if (syncable::OPENED == result) {
     base::AutoLock lock(lock_);
     managed_directory_ = dir.release();
@@ -92,18 +99,13 @@ void DirectoryManager::Close(const std::string& name) {
   {
     base::AutoLock lock(lock_);
     if (!managed_directory_ ||
-        ComparePathNames(name, managed_directory_->name()) != 0) {
+        base::strcasecmp(name.c_str(),
+                         managed_directory_->name().c_str()) != 0) {
       // It wasn't open.
       return;
     }
   }
 
-  // TODO(timsteele): No lock?!
-  // Notify listeners.
-  managed_directory_->channel()->NotifyListeners(DIRECTORY_CLOSED);
-  DirectoryManagerEvent event = { DirectoryManagerEvent::CLOSED, name };
-  channel_->NotifyListeners(event);
-
   delete managed_directory_;
   managed_directory_ = NULL;
 }
@@ -124,7 +126,8 @@ void DirectoryManager::GetOpenDirectories(DirNames* result) {
 ScopedDirLookup::ScopedDirLookup(DirectoryManager* dirman,
                                  const std::string& name) : dirman_(dirman) {
   dir_ = dirman->managed_directory_ &&
-         (ComparePathNames(name, dirman->managed_directory_->name()) == 0) ?
+         (base::strcasecmp(name.c_str(),
+                           dirman->managed_directory_->name().c_str()) == 0) ?
          dirman->managed_directory_ : NULL;
   good_ = dir_ != NULL;
   good_checked_ = false;
