@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -19,34 +19,21 @@
 #include "base/basictypes.h"
 #include "base/file_path.h"
 #include "base/synchronization/lock.h"
+#include "chrome/browser/sync/internal_api/includes/unrecoverable_error_handler.h"
 #include "chrome/browser/sync/syncable/dir_open_result.h"
-#include "chrome/browser/sync/syncable/path_name_cmp.h"
 #include "chrome/browser/sync/syncable/syncable.h"
 #include "chrome/browser/sync/util/cryptographer.h"
-#include "chrome/common/deprecated/event_sys.h"
+#include "chrome/browser/sync/util/weak_handle.h"
 
 namespace sync_api { class BaseTransaction; }
+namespace syncable { class BaseTransaction; }
 
 namespace syncable {
 
-struct DirectoryManagerEvent {
-  enum {
-    CLOSED,
-    CLOSED_ALL,
-    SHUTDOWN,
-  } what_happened;
-  std::string dirname;
-  typedef DirectoryManagerEvent EventType;
-  static inline bool IsChannelShutdownEvent(const EventType& event) {
-    return SHUTDOWN == event.what_happened;
-  }
-};
-
-DirectoryManagerEvent DirectoryManagerShutdownEvent();
+class DirectoryChangeDelegate;
 
 class DirectoryManager {
  public:
-  typedef EventChannel<DirectoryManagerEvent> Channel;
 
   // root_path specifies where db is stored.
   explicit DirectoryManager(const FilePath& root_path);
@@ -55,13 +42,21 @@ class DirectoryManager {
   static const FilePath GetSyncDataDatabaseFilename();
   const FilePath GetSyncDataDatabasePath() const;
 
-  // Opens a directory.  Returns false on error.
-  // Name parameter is the the user's login,
-  // MUST already have been converted to a common case.
-  bool Open(const std::string& name);
+  // Opens a directory.  Returns false on error.  Name parameter is
+  // the the user's login, MUST already have been converted to a
+  // common case.  Does not take ownership of |delegate|, which must
+  // be non-NULL.  Starts sending events to |delegate| if the returned
+  // result is true.  Note that events to |delegate| may be sent from
+  // *any* thread.  |transaction_observer| must be initialized.
+  bool Open(const std::string& name, DirectoryChangeDelegate* delegate,
+            browser_sync::UnrecoverableErrorHandler*
+                unrecoverable_error_handler,
+            const browser_sync::WeakHandle<TransactionObserver>&
+                transaction_observer);
 
-  // Marks a directory as closed.  It might take a while until all the
-  // file handles and resources are freed by other threads.
+  // Marks a directory as closed and stops sending events to the
+  // delegate.  It might take a while until all the file handles and
+  // resources are freed by other threads.
   void Close(const std::string& name);
 
   // Should be called at App exit.
@@ -71,15 +66,28 @@ class DirectoryManager {
   typedef std::vector<std::string> DirNames;
   void GetOpenDirectories(DirNames* result);
 
-  Channel* channel() const { return channel_; }
+  // Wrappers for cryptographer() that enforce holding a transaction.
+  // Note: the Cryptographer is NOT thread safe. It must only be accessed while
+  // the transaction is still active. The Cryptographer's pointer should not be
+  // stored separately.
+  browser_sync::Cryptographer* GetCryptographer(
+      const sync_api::BaseTransaction* trans) const { return cryptographer(); }
+  browser_sync::Cryptographer* GetCryptographer(
+      const syncable::BaseTransaction* trans) const { return cryptographer(); }
 
+ protected:
   browser_sync::Cryptographer* cryptographer() const {
     return cryptographer_.get();
   }
 
- protected:
-  DirOpenResult OpenImpl(const std::string& name, const FilePath& path,
-                         bool* was_open);
+  DirOpenResult OpenImpl(
+      const std::string& name, const FilePath& path,
+      DirectoryChangeDelegate* delegate,
+      browser_sync::UnrecoverableErrorHandler*
+          unrecoverable_error_handler,
+      const browser_sync::WeakHandle<TransactionObserver>&
+          transaction_observer,
+      bool* was_open);
 
   // Helpers for friend class ScopedDirLookup:
   friend class ScopedDirLookup;
@@ -90,8 +98,6 @@ class DirectoryManager {
   base::Lock lock_;
   Directory* managed_directory_;
 
-  Channel* const channel_;
-
   scoped_ptr<browser_sync::Cryptographer> cryptographer_;
 
  private:
