@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,9 @@
 
 #include <string>
 
+#include "chrome/browser/sync/internal_api/includes/unrecoverable_error_handler_mock.h"
 #include "chrome/browser/sync/syncable/syncable.h"
+#include "chrome/browser/sync/test/null_directory_change_delegate.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -17,22 +19,25 @@ using syncable::EntryKernel;
 
 class MockDirectory : public Directory {
  public:
-  MockDirectory() {
-    init_kernel("myk");
-  }
+  MockDirectory();
+  virtual ~MockDirectory();
+
   MOCK_METHOD1(GetEntryByHandle, syncable::EntryKernel*(int64));
 
   MOCK_METHOD2(set_last_downloadstamp, void(syncable::ModelType, int64));
 
   MOCK_METHOD1(GetEntryByClientTag,
                syncable::EntryKernel*(const std::string&));
+
+ private:
+  syncable::NullDirectoryChangeDelegate delegate_;
+  browser_sync::MockUnrecoverableErrorHandler mock_handler_;
 };
 
 class MockSyncableWriteTransaction : public syncable::WriteTransaction {
  public:
-  explicit MockSyncableWriteTransaction(Directory *directory) :
-       WriteTransaction(directory, syncable::UNITTEST, "dontcare.cpp", 25) {
-  }
+  MockSyncableWriteTransaction(
+      const tracked_objects::Location& from_here, Directory *directory);
 };
 
 
