@@ -1,31 +1,26 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/sync/util/nigori.h"
 
-#if defined(OS_WIN)
-#include <winsock2.h>  // for htonl
-#else
-#include <arpa/inet.h>
-#endif
-
 #include <sstream>
 #include <vector>
 
 #include "base/base64.h"
-#include "base/crypto/encryptor.h"
-#include "base/hmac.h"
 #include "base/logging.h"
 #include "base/rand_util.h"
 #include "base/string_util.h"
+#include "base/sys_byteorder.h"
+#include "crypto/encryptor.h"
+#include "crypto/hmac.h"
 
 using base::Base64Encode;
 using base::Base64Decode;
-using base::Encryptor;
-using base::HMAC;
 using base::RandInt;
-using base::SymmetricKey;
+using crypto::Encryptor;
+using crypto::HMAC;
+using crypto::SymmetricKey;
 
 namespace browser_sync {
 
@@ -103,7 +98,7 @@ bool Nigori::InitByDerivation(const std::string& hostname,
       kDerivedKeySizeInBits));
   DCHECK(mac_key_.get());
 
-  return true;
+  return user_key_.get() && encryption_key_.get() && mac_key_.get();
 }
 
 bool Nigori::InitByImport(const std::string& user_key,
@@ -168,7 +163,8 @@ std::string GenerateRandomString(size_t size) {
 
 // Enc[Kenc,Kmac](value)
 bool Nigori::Encrypt(const std::string& value, std::string* encrypted) const {
-  DCHECK_LT(0U, value.size());
+  if (0U >= value.size())
+    return false;
 
   std::string iv = GenerateRandomString(kIvSize);
 
