@@ -6,36 +6,42 @@
 #define CHROME_BROWSER_TAB_CONTENTS_RENDER_VIEW_CONTEXT_MENU_GTK_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
+#include "base/compiler_specific.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/tab_contents/render_view_context_menu.h"
 #include "chrome/browser/ui/gtk/menu_gtk.h"
-#include "gfx/point.h"
+#include "ui/gfx/point.h"
 
 struct ContextMenuParams;
 
 class RenderViewContextMenuGtk : public RenderViewContextMenu,
                                  public MenuGtk::Delegate {
  public:
-  RenderViewContextMenuGtk(TabContents* web_contents,
+  RenderViewContextMenuGtk(content::WebContents* web_contents,
                            const ContextMenuParams& params,
                            uint32_t triggering_event_time);
 
-  ~RenderViewContextMenuGtk();
+  virtual ~RenderViewContextMenuGtk();
 
   // Show the menu at the given location.
   void Popup(const gfx::Point& point);
 
   // Menu::Delegate implementation ---------------------------------------------
-  virtual void StoppedShowing();
-  virtual bool AlwaysShowIconForCmd(int command_id) const;
+  virtual bool AlwaysShowIconForCmd(int command_id) const OVERRIDE;
+
+  // RenderViewContextMenuDelegate implementation ------------------------------
+  virtual void UpdateMenuItem(int command_id,
+                              bool enabled,
+                              bool hidden,
+                              const string16& title) OVERRIDE;
 
  protected:
   // RenderViewContextMenu implementation --------------------------------------
-  virtual void PlatformInit();
+  virtual void PlatformInit() OVERRIDE;
   // TODO(port): implement.
   virtual bool GetAcceleratorForCommandId(
       int command_id,
-      ui::Accelerator* accelerator);
+      ui::Accelerator* accelerator) OVERRIDE;
 
  private:
   scoped_ptr<MenuGtk> menu_gtk_;
