@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,7 @@
 
 #import <Cocoa/Cocoa.h>
 
-#include "base/scoped_nsobject.h"
+#include "base/memory/scoped_nsobject.h"
 #include "chrome/browser/tab_contents/render_view_context_menu.h"
 
 @class MenuController;
@@ -19,21 +19,26 @@
 
 class RenderViewContextMenuMac : public RenderViewContextMenu {
  public:
-  RenderViewContextMenuMac(TabContents* web_contents,
+  RenderViewContextMenuMac(content::WebContents* web_contents,
                            const ContextMenuParams& params,
                            NSView* parent_view);
   virtual ~RenderViewContextMenuMac();
+  virtual void ExecuteCommand(int id) OVERRIDE;
+
+  // RenderViewContextMenuDelegate implementation.
+  virtual void UpdateMenuItem(int command_id,
+                              bool enabled,
+                              bool hidden,
+                              const string16& title) OVERRIDE;
 
  protected:
-  // RenderViewContextMenu implementation-
-  virtual void PlatformInit();
+  // RenderViewContextMenu implementation.
+  virtual void PlatformInit() OVERRIDE;
   virtual bool GetAcceleratorForCommandId(
       int command_id,
-      ui::Accelerator* accelerator) {
-    return false;
-  }
+      ui::Accelerator* accelerator) OVERRIDE;
 
-  virtual void LookUpInDictionary();
+  virtual void LookUpInDictionary() OVERRIDE;
 
   void InitPlatformMenu();
  private:
