@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,21 +11,26 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/callback.h"
-#include "base/linked_ptr.h"
+#include "base/memory/linked_ptr.h"
 #include "base/timer.h"
-#include "chrome/browser/renderer_host/backing_store.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/browser/renderer_host/backing_store.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/web_contents_observer.h"
 
 class GURL;
+class Profile;
 class RenderWidgetHost;
 class SkBitmap;
-class TabContents;
 
-class ThumbnailGenerator : NotificationObserver {
+namespace history {
+class TopSites;
+}
+
+class ThumbnailGenerator : public content::NotificationObserver,
+                           public content::WebContentsObserver {
  public:
-  typedef Callback1<const SkBitmap&>::Type ThumbnailReadyCallback;
+  typedef base::Callback<void(const SkBitmap&)> ThumbnailReadyCallback;
   // The result of clipping. This can be used to determine if the
   // generated thumbnail is good or not.
   enum ClipResult {
@@ -49,11 +54,10 @@ class ThumbnailGenerator : NotificationObserver {
 
   // This class will do nothing until you call StartThumbnailing.
   ThumbnailGenerator();
-  ~ThumbnailGenerator();
+  virtual ~ThumbnailGenerator();
 
-  // Ensures that we're properly hooked in to generated thumbnails. This can
-  // be called repeatedly and with wild abandon to no ill effect.
-  void StartThumbnailing();
+  // Starts taking thumbnails of the given tab contents.
+  void StartThumbnailing(content::WebContents* web_contents);
 
   // This registers a callback that can receive the resulting SkBitmap
   // from the renderer when it is done rendering it.  This differs
@@ -74,7 +78,7 @@ class ThumbnailGenerator : NotificationObserver {
   // dimensions, but might not be the exact size requested.
   void AskForSnapshot(RenderWidgetHost* renderer,
                       bool prefer_backing_store,
-                      ThumbnailReadyCallback* callback,
+                      const ThumbnailReadyCallback& callback,
                       gfx::Size page_size,
                       gfx::Size desired_size);
 
@@ -93,14 +97,6 @@ class ThumbnailGenerator : NotificationObserver {
   // of |monitor|.
   void MonitorRenderer(RenderWidgetHost* renderer, bool monitor);
 
-#ifdef UNIT_TEST
-  // When true, the class will not use a timeout to do the expiration. This
-  // will cause expiration to happen on the next run of the message loop.
-  // Unit tests case use this to test expiration by choosing when the message
-  // loop runs.
-  void set_no_timeout(bool no_timeout) { no_timeout_ = no_timeout; }
-#endif
-
   // Calculates how "boring" a thumbnail is. The boring score is the
   // 0,1 ranged percentage of pixels that are the most common
   // luma. Higher boring scores indicate that a higher percentage of a
@@ -117,55 +113,37 @@ class ThumbnailGenerator : NotificationObserver {
                                    int desired_height,
                                    ClipResult* clip_result);
 
-  // Update the thumbnail of the given URL if necessary.
-  static void UpdateThumbnailIfNecessary(TabContents* tab_contents,
-                                         const GURL& url);
+  // Update the thumbnail of the given tab contents if necessary.
+  void UpdateThumbnailIfNecessary(content::WebContents* webb_contents);
+
+  // Returns true if we should update the thumbnail of the given URL.
+  static bool ShouldUpdateThumbnail(Profile* profile,
+                                    history::TopSites* top_sites,
+                                    const GURL& url);
+
+  // content::WebContentsObserver overrides.
+  virtual void DidStartLoading() OVERRIDE;
+  virtual void StopNavigation() OVERRIDE;
 
  private:
-  // RenderWidgetHostPaintingObserver implementation.
-  virtual void WidgetWillDestroyBackingStore(RenderWidgetHost* widget,
-                                             BackingStore* backing_store);
-  virtual void WidgetDidUpdateBackingStore(RenderWidgetHost* widget);
-
   virtual void WidgetDidReceivePaintAtSizeAck(
       RenderWidgetHost* widget,
       int tag,
       const gfx::Size& size);
 
-  // NotificationObserver interface.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver interface.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Indicates that the given widget has changed is visibility.
-  void WidgetShown(RenderWidgetHost* widget);
   void WidgetHidden(RenderWidgetHost* widget);
 
-  // Called when the given widget is destroyed.
-  void WidgetDestroyed(RenderWidgetHost* widget);
-
-  // Called when the given tab contents are disconnected (either
+  // Called when the given web contents are disconnected (either
   // through being closed, or because the renderer is no longer there).
-  void TabContentsDisconnected(TabContents* contents);
+  void WebContentsDisconnected(content::WebContents* contents);
 
-  // Timer function called on a delay after a tab has been shown. It will
-  // invalidate the thumbnail for hosts with expired thumbnails in shown_hosts_.
-  void ShownDelayHandler();
-
-  // Removes the given host from the shown_hosts_ list, if it is there.
-  void EraseHostFromShownList(RenderWidgetHost* host);
-
-  NotificationRegistrar registrar_;
-
-  base::OneShotTimer<ThumbnailGenerator> timer_;
-
-  // A list of all RWHs that have been shown and need to have their thumbnail
-  // expired at some time in the future with the "slop" time has elapsed. This
-  // list will normally have 0 or 1 items in it.
-  std::vector<RenderWidgetHost*> shown_hosts_;
-
-  // See the setter above.
-  bool no_timeout_;
+  content::NotificationRegistrar registrar_;
 
   // Map of callback objects by sequence number.
   struct AsyncRequestInfo;
@@ -173,6 +151,8 @@ class ThumbnailGenerator : NotificationObserver {
                    linked_ptr<AsyncRequestInfo> > ThumbnailCallbackMap;
   ThumbnailCallbackMap callback_map_;
 
+  bool load_interrupted_;
+
   DISALLOW_COPY_AND_ASSIGN(ThumbnailGenerator);
 };
 
