@@ -1,9 +1,10 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/common/url_constants.h"
+#include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/ui/ui_test.h"
@@ -37,8 +38,9 @@ class ViewSourceTest : public UITest {
 
 // This test renders a page in view-source and then checks to see if a cookie
 // set in the html was set successfully (it shouldn't because we rendered the
-// page in view source)
-TEST_F(ViewSourceTest, DoesBrowserRenderInViewSource) {
+// page in view source).
+// Flaky; see http://crbug.com/72201.
+TEST_F(ViewSourceTest, FLAKY_DoesBrowserRenderInViewSource) {
   ASSERT_TRUE(test_server_.Start());
 
   std::string cookie = "viewsource_cookie";
