@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,30 +7,34 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
-#include "gfx/native_widget_types.h"
-#include "gfx/point.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "ui/base/dragdrop/drag_source.h"
+#include "ui/gfx/native_widget_types.h"
+#include "ui/gfx/point.h"
 
 class RenderViewHost;
-class TabContents;
+
+namespace content {
+class WebContents;
+}
 
 // An IDropSource implementation for a TabContents. Handles notifications sent
 // by an active drag-drop operation as the user mouses over other drop targets
 // on their system. This object tells Windows whether or not the drag should
 // continue, and supplies the appropriate cursors.
 class WebDragSource : public ui::DragSource,
-                      public NotificationObserver {
+                      public content::NotificationObserver {
  public:
-  // Create a new DragSource for a given HWND and TabContents.
-  WebDragSource(gfx::NativeWindow source_wnd, TabContents* tab_contents);
+  // Create a new DragSource for a given HWND and WebContents.
+  WebDragSource(gfx::NativeWindow source_wnd,
+                content::WebContents* web_contents);
   virtual ~WebDragSource();
 
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details);
 
   void set_effect(DWORD effect) { effect_ = effect; }
 
@@ -56,7 +60,7 @@ class WebDragSource : public ui::DragSource,
   // initiated terminates).
   RenderViewHost* render_view_host_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DWORD effect_;
 
