@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,22 +6,26 @@
 #define CHROME_BROWSER_TAB_CONTENTS_WEB_DROP_TARGET_WIN_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebDragOperation.h"
 #include "ui/base/dragdrop/drop_target.h"
 
 class InterstitialDropTarget;
 class RenderViewHost;
-class TabContents;
+class TabContentsWrapper;
+
+namespace content {
+class WebContents;
+}
 
 // A helper object that provides drop capabilities to a TabContents. The
-// DropTarget handles drags that enter the region of the TabContents by
+// DropTarget handles drags that enter the region of the WebContents by
 // passing on the events to the renderer.
 class WebDropTarget : public ui::DropTarget {
  public:
   // Create a new WebDropTarget associating it with the given HWND and
-  // TabContents.
-  WebDropTarget(HWND source_hwnd, TabContents* contents);
+  // WebContents.
+  WebDropTarget(HWND source_hwnd, content::WebContents* contents);
   virtual ~WebDropTarget();
 
   void set_drag_cursor(WebKit::WebDragOperation op) {
@@ -47,8 +51,13 @@ class WebDropTarget : public ui::DropTarget {
                        DWORD effect);
 
  private:
-  // Our associated TabContents.
-  TabContents* tab_contents_;
+  // Our associated WebContents.
+  content::WebContents* web_contents_;
+
+  // The TabContentsWrapper for |tab_contents_|.
+  // Weak reference; may be NULL if the contents aren't contained in a wrapper
+  // (e.g. WebUI dialogs).
+  TabContentsWrapper* tab_;
 
   // We keep track of the render view host we're dragging over.  If it changes
   // during a drag, we need to re-send the DragEnter message.  WARNING:
