@@ -9,8 +9,9 @@
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/automation/tab_proxy.h"
+#include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
+#include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/automation/window_proxy.h"
 #include "chrome/test/ui/ui_test.h"
 #include "googleurl/src/gurl.h"
@@ -24,15 +25,6 @@
 #define MAYBE_BasicRestoreFromClosedWindow BasicRestoreFromClosedWindow
 #endif
 
-// http://crbug.com/
-#if defined(OS_CHROMEOS) && !defined(NDEBUG)
-#define MAYBE_RestoreWindowAndTab DISABLED_RestoreWindowAndTab
-#define MAYBE_RestoreWindow DISABLED_RestoreWindow
-#else
-#define MAYBE_RestoreWindowAndTab RestoreWindowAndTab
-#define MAYBE_RestoreWindow RestoreWindow
-#endif
-
 class TabRestoreUITest : public UITest {
  public:
   TabRestoreUITest() : UITest() {
@@ -207,8 +199,7 @@ TEST_F(TabRestoreUITest, MiddleTab) {
 
 // Close a tab, switch windows, then restore the tab. The tab should be in its
 // original window and position, and active.
-// This test is flaky. See http://crbug.com/54894
-TEST_F(TabRestoreUITest, FLAKY_RestoreToDifferentWindow) {
+TEST_F(TabRestoreUITest, RestoreToDifferentWindow) {
   scoped_refptr<BrowserProxy> browser_proxy(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(browser_proxy.get());
 
@@ -234,7 +225,7 @@ TEST_F(TabRestoreUITest, FLAKY_RestoreToDifferentWindow) {
   EXPECT_EQ(starting_tab_count + 2, tab_count);
 
   // Create a new browser.
-  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_NORMAL, false));
+  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_TABBED, false));
   int window_count;
   ASSERT_TRUE(automation()->GetBrowserWindowCount(&window_count));
   EXPECT_EQ(2, window_count);
@@ -277,7 +268,7 @@ TEST_F(TabRestoreUITest, MAYBE_BasicRestoreFromClosedWindow) {
   ASSERT_EQ(AUTOMATION_MSG_NAVIGATION_SUCCESS, tab_proxy->NavigateToURL(url2_));
 
   // Create a new browser.
-  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_NORMAL, false));
+  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_TABBED, false));
   int window_count;
   ASSERT_TRUE(automation()->GetBrowserWindowCount(&window_count));
   EXPECT_EQ(2, window_count);
@@ -339,7 +330,7 @@ TEST_F(TabRestoreUITest, DontLoadRestoredTab) {
 
 // Open a window with multiple tabs, close a tab, then close the window.
 // Restore both and make sure the tab goes back into the window.
-TEST_F(TabRestoreUITest, MAYBE_RestoreWindowAndTab) {
+TEST_F(TabRestoreUITest, RestoreWindowAndTab) {
   scoped_refptr<BrowserProxy> browser_proxy(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(browser_proxy.get());
   CheckActiveWindow(browser_proxy.get());
@@ -361,7 +352,7 @@ TEST_F(TabRestoreUITest, MAYBE_RestoreWindowAndTab) {
   EXPECT_EQ(starting_tab_count + 2, tab_count);
 
   // Create a new browser.
-  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_NORMAL, false));
+  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_TABBED, false));
   int window_count;
   ASSERT_TRUE(automation()->GetBrowserWindowCount(&window_count));
   EXPECT_EQ(2, window_count);
@@ -396,8 +387,7 @@ TEST_F(TabRestoreUITest, MAYBE_RestoreWindowAndTab) {
 
 // Open a window with two tabs, close both (closing the window), then restore
 // both. Make sure both restored tabs are in the same window.
-// http://crbug.com/39925
-TEST_F(TabRestoreUITest, FLAKY_RestoreIntoSameWindow) {
+TEST_F(TabRestoreUITest, RestoreIntoSameWindow) {
   scoped_refptr<BrowserProxy> browser_proxy(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(browser_proxy.get());
   CheckActiveWindow(browser_proxy.get());
@@ -412,7 +402,7 @@ TEST_F(TabRestoreUITest, FLAKY_RestoreIntoSameWindow) {
   ASSERT_EQ(AUTOMATION_MSG_NAVIGATION_SUCCESS, tab_proxy->NavigateToURL(url2_));
 
   // Create a new browser.
-  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_NORMAL, false));
+  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_TABBED, false));
   int window_count;
   ASSERT_TRUE(automation()->GetBrowserWindowCount(&window_count));
   EXPECT_EQ(2, window_count);
@@ -552,11 +542,11 @@ TEST_F(TabRestoreUITest, RestoreCrossSiteWithExistingSiteInstance) {
   EXPECT_EQ(http_url2, GetActiveTabURL());
 }
 
-TEST_F(TabRestoreUITest, MAYBE_RestoreWindow) {
+TEST_F(TabRestoreUITest, RestoreWindow) {
   // Create a new window.
   int window_count;
   ASSERT_TRUE(automation()->GetBrowserWindowCount(&window_count));
-  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_NORMAL, false));
+  ASSERT_TRUE(automation()->OpenNewBrowserWindow(Browser::TYPE_TABBED, false));
   int new_window_count = 0;
   ASSERT_TRUE(automation()->GetBrowserWindowCount(&new_window_count));
   EXPECT_EQ(++window_count, new_window_count);
@@ -613,7 +603,7 @@ TEST_F(TabRestoreUITest, MAYBE_RestoreWindow) {
   EXPECT_TRUE(url == url2_);
 }
 
-// Restore tab with special URL about:credits and make sure the page loads
+// Restore tab with special URL chrome://credits/ and make sure the page loads
 // properly after restore. See http://crbug.com/31905.
 TEST_F(TabRestoreUITest, RestoreTabWithSpecialURL) {
   scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
@@ -621,7 +611,7 @@ TEST_F(TabRestoreUITest, RestoreTabWithSpecialURL) {
   CheckActiveWindow(browser.get());
 
   // Navigate new tab to a special URL.
-  const GURL special_url(chrome::kAboutCreditsURL);
+  const GURL special_url(chrome::kChromeUICreditsURL);
   ASSERT_TRUE(browser->AppendTab(special_url));
   scoped_refptr<TabProxy> tab(browser->GetActiveTab());
   ASSERT_TRUE(tab.get());
@@ -654,7 +644,7 @@ TEST_F(TabRestoreUITest, RestoreTabWithSpecialURLOnBack) {
   CheckActiveWindow(browser.get());
 
   // Navigate new tab to a special URL.
-  const GURL special_url(chrome::kAboutCreditsURL);
+  const GURL special_url(chrome::kChromeUICreditsURL);
   ASSERT_TRUE(browser->AppendTab(special_url));
   scoped_refptr<TabProxy> tab(browser->GetActiveTab());
   ASSERT_TRUE(tab.get());
