@@ -1,12 +1,14 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "base/metrics/nacl_histogram.h"
+#include "chrome/browser/browser_shutdown.h"
 #include "chrome/browser/tabs/default_tab_handler.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/browser.h"
 
+using content::SiteInstance;
+
 ////////////////////////////////////////////////////////////////////////////////
 // DefaultTabHandler, public:
 
@@ -14,13 +16,13 @@ DefaultTabHandler::DefaultTabHandler(TabHandlerDelegate* delegate)
     : delegate_(delegate),
       ALLOW_THIS_IN_INITIALIZER_LIST(
           model_(new TabStripModel(this, delegate->GetProfile()))) {
-  UmaNaclHistogramEnumeration(FIRST_TAB_NACL_BASELINE);
   model_->AddObserver(this);
 }
 
 DefaultTabHandler::~DefaultTabHandler() {
   // The tab strip should not have any tabs at this point.
-  DCHECK(model_->empty());
+  if (!browser_shutdown::ShuttingDownWithoutClosingBrowsers())
+    DCHECK(model_->empty());
   model_->RemoveObserver(this);
 }
 
@@ -35,7 +37,6 @@ TabStripModel* DefaultTabHandler::GetTabStripModel() const {
 // DefaultTabHandler, TabStripModelDelegate implementation:
 
 TabContentsWrapper* DefaultTabHandler::AddBlankTab(bool foreground) {
-  UmaNaclHistogramEnumeration(NEW_TAB_NACL_BASELINE);
   return delegate_->AsBrowser()->AddBlankTab(foreground);
 }
 
@@ -61,9 +62,9 @@ int DefaultTabHandler::GetDragActions() const {
 
 TabContentsWrapper* DefaultTabHandler::CreateTabContentsForURL(
     const GURL& url,
-    const GURL& referrer,
+    const content::Referrer& referrer,
     Profile* profile,
-    PageTransition::Type transition,
+    content::PageTransition transition,
     bool defer_load,
     SiteInstance* instance) const {
   return delegate_->AsBrowser()->CreateTabContentsForURL(url,
@@ -95,8 +96,8 @@ bool DefaultTabHandler::RunUnloadListenerBeforeClosing(
   return delegate_->AsBrowser()->RunUnloadListenerBeforeClosing(contents);
 }
 
-bool DefaultTabHandler::CanCloseContentsAt(int index) {
-  return delegate_->AsBrowser()->CanCloseContentsAt(index);
+bool DefaultTabHandler::CanCloseContents(std::vector<int>* indices) {
+  return delegate_->AsBrowser()->CanCloseContents(indices);
 }
 
 bool DefaultTabHandler::CanBookmarkAllTabs() const {
@@ -111,10 +112,6 @@ bool DefaultTabHandler::CanCloseTab() const {
   return delegate_->AsBrowser()->CanCloseTab();
 }
 
-void DefaultTabHandler::ToggleUseVerticalTabs() {
-  delegate_->AsBrowser()->ToggleUseVerticalTabs();
-}
-
 bool DefaultTabHandler::CanRestoreTab() {
   return delegate_->AsBrowser()->CanRestoreTab();
 }
@@ -127,10 +124,6 @@ bool DefaultTabHandler::LargeIconsPermitted() const {
   return delegate_->AsBrowser()->LargeIconsPermitted();
 }
 
-bool DefaultTabHandler::UseVerticalTabs() const {
-  return delegate_->AsBrowser()->UseVerticalTabs();
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // DefaultTabHandler, TabStripModelObserver implementation:
 
@@ -150,19 +143,18 @@ void DefaultTabHandler::TabDetachedAt(TabContentsWrapper* contents, int index) {
   delegate_->AsBrowser()->TabDetachedAt(contents, index);
 }
 
-void DefaultTabHandler::TabDeselectedAt(TabContentsWrapper* contents,
-                                        int index) {
-  delegate_->AsBrowser()->TabDeselectedAt(contents, index);
+void DefaultTabHandler::TabDeactivated(TabContentsWrapper* contents) {
+  delegate_->AsBrowser()->TabDeactivated(contents);
 }
 
-void DefaultTabHandler::TabSelectedAt(TabContentsWrapper* old_contents,
-                                      TabContentsWrapper* new_contents,
-                                      int index,
-                                      bool user_gesture) {
-  delegate_->AsBrowser()->TabSelectedAt(old_contents,
-                                        new_contents,
-                                        index,
-                                        user_gesture);
+void DefaultTabHandler::ActiveTabChanged(TabContentsWrapper* old_contents,
+                                         TabContentsWrapper* new_contents,
+                                         int index,
+                                         bool user_gesture) {
+  delegate_->AsBrowser()->ActiveTabChanged(old_contents,
+                                           new_contents,
+                                           index,
+                                           user_gesture);
 }
 
 void DefaultTabHandler::TabMoved(TabContentsWrapper* contents,
@@ -195,4 +187,3 @@ void DefaultTabHandler::TabStripEmpty() {
 TabHandler* TabHandler::CreateTabHandler(TabHandlerDelegate* delegate) {
   return new DefaultTabHandler(delegate);
 }
-
