@@ -1,21 +1,23 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/tabs/pinned_tab_codec.h"
 
 #include "base/values.h"
-#include "chrome/browser/browser_list.h"
+#include "chrome/browser/extensions/extension_tab_helper.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/extensions/extension.h"
-#include "chrome/common/page_transition_types.h"
 #include "chrome/common/pref_names.h"
-#include "ipc/ipc_message.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
+
+using content::NavigationEntry;
 
 typedef BrowserInit::LaunchWithProfile::Tab Tab;
 
@@ -44,7 +46,8 @@ static void EncodePinnedTab(TabStripModel* model,
 
   TabContentsWrapper* tab_contents = model->GetTabContentsAt(index);
   if (model->IsAppTab(index)) {
-    const Extension* extension = tab_contents->extension_app();
+    const Extension* extension =
+        tab_contents->extension_tab_helper()->extension_app();
     DCHECK(extension);
     value->SetString(kAppID, extension->id());
     // For apps we use the launch url. We do this for the following reason:
@@ -53,11 +56,12 @@ static void EncodePinnedTab(TabStripModel* model,
     value->SetString(kURL, extension->GetFullLaunchURL().spec());
     values->Append(value.release());
   } else {
-    NavigationEntry* entry = tab_contents->controller().GetActiveEntry();
-    if (!entry && tab_contents->controller().entry_count())
-      entry = tab_contents->controller().GetEntryAtIndex(0);
+    NavigationEntry* entry =
+        tab_contents->web_contents()->GetController().GetActiveEntry();
+    if (!entry && tab_contents->web_contents()->GetController().GetEntryCount())
+      entry = tab_contents->web_contents()->GetController().GetEntryAtIndex(0);
     if (entry) {
-      value->SetString(kURL, entry->url().spec());
+      value->SetString(kURL, entry->GetURL().spec());
       values->Append(value.release());
     }
   }
@@ -88,7 +92,7 @@ static bool DecodeTab(const DictionaryValue& value, Tab* tab) {
 
 // static
 void PinnedTabCodec::RegisterUserPrefs(PrefService* prefs) {
-  prefs->RegisterListPref(prefs::kPinnedTabs);
+  prefs->RegisterListPref(prefs::kPinnedTabs, PrefService::UNSYNCABLE_PREF);
 }
 
 // static
@@ -101,13 +105,12 @@ void PinnedTabCodec::WritePinnedTabs(Profile* profile) {
   for (BrowserList::const_iterator i = BrowserList::begin();
        i != BrowserList::end(); ++i) {
     Browser* browser = *i;
-    if (browser->type() == Browser::TYPE_NORMAL &&
+    if (browser->is_type_tabbed() &&
         browser->profile() == profile && HasPinnedTabs(browser)) {
       EncodePinnedTabs(browser, &values);
     }
   }
   prefs->Set(prefs::kPinnedTabs, values);
-  prefs->ScheduleSavePersistentPrefs();
 }
 
 // static
