@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,17 +10,22 @@
 
 #include "base/observer_list.h"
 #include "chrome/browser/tabs/tab_strip_model_observer.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/page_transition_types.h"
+#include "chrome/browser/tabs/tab_strip_selection_model.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/common/page_transition_types.h"
 
-class NavigationController;
 class Profile;
 class TabContents;
 class TabContentsWrapper;
 class TabStripModelDelegate;
 class TabStripModelOrderController;
 
+namespace content {
+class NavigationController;
+class WebContents;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 //
 // TabStripModel
@@ -37,11 +42,11 @@ class TabStripModelOrderController;
 //   if a non-mini tab is added it is forced to be with non-mini tabs. Requests
 //   to move tabs outside the range of the tab type are ignored. For example,
 //   a request to move a mini-tab after non-mini-tabs is ignored.
-//   You'll notice there is no explcit api for making a tab a mini-tab, rather
+//   You'll notice there is no explicit api for making a tab a mini-tab, rather
 //   there are two tab types that are implicitly mini-tabs:
 //   . App. Corresponds to an extension that wants an app tab. App tabs are
-//     identified by TabContents::is_app(). App tabs are always pinneded (you
-//     can't unpin them).
+//     identified by TabContentsWrapper::extension_tab_helper()::is_app().
+//     App tabs are always pinned (you can't unpin them).
 //   . Pinned. Any tab can be pinned. Non-app tabs whose pinned state is changed
 //     are moved to be with other mini-tabs or non-mini tabs.
 //
@@ -57,7 +62,7 @@ class TabStripModelOrderController;
 //  its bookkeeping when such events happen.
 //
 ////////////////////////////////////////////////////////////////////////////////
-class TabStripModel : public NotificationObserver {
+class TabStripModel : public content::NotificationObserver {
  public:
   // Policy for how new tabs are inserted.
   enum InsertionPolicy {
@@ -73,7 +78,7 @@ class TabStripModel : public NotificationObserver {
     CLOSE_NONE                     = 0,
 
     // Indicates the tab was closed by the user. If true,
-    // TabContents::set_closed_by_user_gesture(true) is invoked.
+    // TabContents::SetClosedByUserGesture(true) is invoked.
     CLOSE_USER_GESTURE             = 1 << 0,
 
     // If true the history is recorded so that the tab can be reopened later.
@@ -87,8 +92,8 @@ class TabStripModel : public NotificationObserver {
     // tab.
     ADD_NONE          = 0,
 
-    // The tab should be selected.
-    ADD_SELECTED      = 1 << 0,
+    // The tab should be active.
+    ADD_ACTIVE        = 1 << 0,
 
     // The tab should be pinned.
     ADD_PINNED        = 1 << 1,
@@ -103,9 +108,8 @@ class TabStripModel : public NotificationObserver {
     // certain situations.
     ADD_INHERIT_GROUP = 1 << 3,
 
-    // If set the newly inserted tab's opener is set to the currently selected
-    // tab. If not set the tab may still inherit the group/opener under certain
-    // situations.
+    // If set the newly inserted tab's opener is set to the active tab. If not
+    // set the tab may still inherit the group/opener under certain situations.
     // NOTE: this is ignored if ADD_INHERIT_GROUP is set.
     ADD_INHERIT_OPENER = 1 << 4,
   };
@@ -131,8 +135,8 @@ class TabStripModel : public NotificationObserver {
   // Retrieve the Profile associated with this TabStripModel.
   Profile* profile() const { return profile_; }
 
-  // Retrieve the index of the currently selected TabContents.
-  int selected_index() const { return selected_index_; }
+  // Retrieve the index of the currently active TabContents.
+  int active_index() const { return selection_model_.active(); }
 
   // Returns true if the tabstrip is currently closing all open tabs (via a
   // call to CloseAllTabs). As tabs close, the selection in the tabstrip
@@ -159,7 +163,7 @@ class TabStripModel : public NotificationObserver {
   bool ContainsIndex(int index) const;
 
   // Adds the specified TabContents in the default location. Tabs opened in the
-  // foreground inherit the group of the previously selected tab.
+  // foreground inherit the group of the previously active tab.
   void AppendTabContents(TabContentsWrapper* contents, bool foreground);
 
   // Adds the specified TabContents at the specified location. |add_types| is a
@@ -201,17 +205,27 @@ class TabStripModel : public NotificationObserver {
   TabContentsWrapper* ReplaceTabContentsAt(int index,
                                            TabContentsWrapper* new_contents);
 
+  // Destroys the TabContents at the specified index, but keeps the tab visible
+  // in the tab strip. Used to free memory in low-memory conditions, especially
+  // on Chrome OS. The tab reloads if the user clicks on it.
+  // Returns an empty TabContentsWrapper, used only for testing.
+  TabContentsWrapper* DiscardTabContentsAt(int index);
+
   // Detaches the TabContents at the specified index from this strip. The
   // TabContents is not destroyed, just removed from display. The caller is
   // responsible for doing something with it (e.g. stuffing it into another
   // strip).
   TabContentsWrapper* DetachTabContentsAt(int index);
 
-  // Select the TabContents at the specified index. |user_gesture| is true if
-  // the user actually clicked on the tab or navigated to it using a keyboard
-  // command, false if the tab was selected as a by-product of some other
+  // Makes the tab at the specified index the active tab. |user_gesture| is true
+  // if the user actually clicked on the tab or navigated to it using a keyboard
+  // command, false if the tab was activated as a by-product of some other
   // action.
-  void SelectTabContentsAt(int index, bool user_gesture);
+  void ActivateTabAt(int index, bool user_gesture);
+
+  // Adds tab at |index| to the currently selected tabs, without changing the
+  // active tab index.
+  void AddTabAtToSelection(int index);
 
   // Move the TabContents at the specified index to another index. This method
   // does NOT send Detached/Attached notifications, rather it moves the
@@ -223,8 +237,26 @@ class TabStripModel : public NotificationObserver {
   // tabs mixing.
   void MoveTabContentsAt(int index, int to_position, bool select_after_move);
 
-  // Returns the currently selected TabContents, or NULL if there is none.
-  TabContentsWrapper* GetSelectedTabContents() const;
+  // Moves the selected tabs to |index|. |index| is treated as if the tab strip
+  // did not contain any of the selected tabs. For example, if the tabstrip
+  // contains [A b c D E f] (upper case selected) and this is invoked with 1 the
+  // result is [b A D E c f].
+  // This method maintains that all mini-tabs occur before non-mini-tabs.  When
+  // mini-tabs are selected the move is processed in two chunks: first mini-tabs
+  // are moved, then non-mini-tabs are moved. If the index is after
+  // (mini-tab-count - selected-mini-tab-count), then the index the non-mini
+  // selected tabs are moved to is (index + selected-mini-tab-count). For
+  // example, if the model consists of [A b c D E f] (A b c are mini) and this
+  // is inokved with 2, the result is [b c A D E f]. In this example nothing
+  // special happened because the target index was <= (mini-tab-count -
+  // selected-mini-tab-count). If the target index were 3, then the result would
+  // be [b c A f D F]. A, being mini, can move no further than index 2. The
+  // non-mini-tabs are moved to the target index + selected-mini-tab-count (3 +
+  // 1)
+  void MoveSelectedTabsTo(int index);
+
+  // Returns the currently active TabContents, or NULL if there is none.
+  TabContentsWrapper* GetActiveTabContents() const;
 
   // Returns the TabContentsWrapper at the specified index, or NULL if there is
   // none.
@@ -234,17 +266,18 @@ class TabStripModel : public NotificationObserver {
   // TabStripModel::kNoTab if the TabContents is not in this TabStripModel.
   int GetIndexOfTabContents(const TabContentsWrapper* contents) const;
 
-  // Returns the index of the specified TabContents wrapper given its raw
-  // TabContents, or TabStripModel::kNoTab if the TabContents is not in this
+  // Returns the index of the specified WebContents wrapper given its raw
+  // WebContents, or TabStripModel::kNoTab if the TabContents is not in this
   // TabStripModel.  Note: This is only needed in rare cases where the wrapper
-  // is not already present (such as implementing TabContentsDelegate methods,
+  // is not already present (such as implementing WebContentsDelegate methods,
   // which don't know about the wrapper.  Returns NULL if |contents| is not
   // associated with any wrapper in the model.
-  int GetWrapperIndex(const TabContents* contents) const;
+  int GetWrapperIndex(const content::WebContents* contents) const;
 
   // Returns the index of the specified NavigationController, or kNoTab if it is
   // not in this TabStripModel.
-  int GetIndexOfController(const NavigationController* controller) const;
+  int GetIndexOfController(
+      const content::NavigationController* controller) const;
 
   // Notify any observers that the TabContents at the specified index has
   // changed in some way. See TabChangeType for details of |change_type|.
@@ -266,33 +299,42 @@ class TabStripModel : public NotificationObserver {
   bool TabsAreLoading() const;
 
   // Returns the controller controller that opened the TabContents at |index|.
-  NavigationController* GetOpenerOfTabContentsAt(int index);
+  content::NavigationController* GetOpenerOfTabContentsAt(int index);
+
+  // Changes the |opener| of the TabContents at |index|.
+  // Note: |opener| must be in this tab strip.
+  void SetOpenerOfTabContentsAt(
+      int index,
+      content::NavigationController* opener);
 
   // Returns the index of the next TabContents in the sequence of TabContentses
   // spawned by the specified NavigationController after |start_index|.
   // If |use_group| is true, the group property of the tab is used instead of
   // the opener to find the next tab. Under some circumstances the group
   // relationship may exist but the opener may not.
-  int GetIndexOfNextTabContentsOpenedBy(const NavigationController* opener,
-                                        int start_index,
-                                        bool use_group) const;
+  int GetIndexOfNextTabContentsOpenedBy(
+      const content::NavigationController* opener,
+      int start_index,
+      bool use_group) const;
 
   // Returns the index of the first TabContents in the model opened by the
   // specified opener.
-  int GetIndexOfFirstTabContentsOpenedBy(const NavigationController* opener,
-                                         int start_index) const;
+  int GetIndexOfFirstTabContentsOpenedBy(
+      const content::NavigationController* opener,
+      int start_index) const;
 
   // Returns the index of the last TabContents in the model opened by the
   // specified opener, starting at |start_index|.
-  int GetIndexOfLastTabContentsOpenedBy(const NavigationController* opener,
-                                        int start_index) const;
+  int GetIndexOfLastTabContentsOpenedBy(
+      const content::NavigationController* opener,
+      int start_index) const;
 
   // Called by the Browser when a navigation is about to occur in the specified
   // TabContents. Depending on the tab, and the transition type of the
   // navigation, the TabStripModel may adjust its selection and grouping
   // behavior.
   void TabNavigating(TabContentsWrapper* contents,
-                     PageTransition::Type transition);
+                     content::PageTransition transition);
 
   // Forget all Opener relationships that are stored (but _not_ group
   // relationships!) This is to reduce unpredictable tab switching behavior
@@ -334,6 +376,10 @@ class TabStripModel : public NotificationObserver {
   // Returns true if the tab at |index| is blocked by a tab modal dialog.
   bool IsTabBlocked(int index) const;
 
+  // Returns true if the TabContents at |index| has been discarded to save
+  // memory.  See DiscardTabContentsAt() for details.
+  bool IsTabDiscarded(int index) const;
+
   // Returns the index of the first tab that is not a mini-tab. This returns
   // |count()| if all of the tabs are mini-tabs, and 0 if none of the tabs are
   // mini-tabs.
@@ -346,6 +392,27 @@ class TabStripModel : public NotificationObserver {
   // is between IndexOfFirstNonMiniTab and count().
   int ConstrainInsertionIndex(int index, bool mini_tab);
 
+  // Extends the selection from the anchor to |index|.
+  void ExtendSelectionTo(int index);
+
+  // Toggles the selection at |index|. This does nothing if |index| is selected
+  // and there are no other selected tabs.
+  void ToggleSelectionAt(int index);
+
+  // Makes sure the tabs from the anchor to |index| are selected. This only
+  // adds to the selection.
+  void AddSelectionFromAnchorTo(int index);
+
+  // Returns true if the tab at |index| is selected.
+  bool IsTabSelected(int index) const;
+
+  // Sets the selection to match that of |source|.
+  void SetSelectionFromModel(const TabStripSelectionModel& source);
+
+  const TabStripSelectionModel& selection_model() const {
+    return selection_model_;
+  }
+
   // Command level API /////////////////////////////////////////////////////////
 
   // Adds a TabContents at the best position in the TabStripModel given the
@@ -354,11 +421,11 @@ class TabStripModel : public NotificationObserver {
   // InsertTabContentsAt to do the actual inertion.
   void AddTabContents(TabContentsWrapper* contents,
                       int index,
-                      PageTransition::Type transition,
+                      content::PageTransition transition,
                       int add_types);
 
-  // Closes the selected TabContents.
-  void CloseSelectedTab();
+  // Closes the selected tabs.
+  void CloseSelectedTabs();
 
   // Select adjacent tabs
   void SelectNextTab();
@@ -371,6 +438,10 @@ class TabStripModel : public NotificationObserver {
   void MoveTabNext();
   void MoveTabPrevious();
 
+  // Notifies the observers that the active/foreground tab at |index| was
+  // reselected (ie - it was already active and was clicked again).
+  void ActiveTabClicked(int index);
+
   // View API //////////////////////////////////////////////////////////////////
 
   // Context menu functions.
@@ -385,20 +456,20 @@ class TabStripModel : public NotificationObserver {
     CommandRestoreTab,
     CommandTogglePinned,
     CommandBookmarkAllTabs,
-    CommandUseVerticalTabs,
+    CommandUseCompactNavigationBar,
+    CommandSelectByDomain,
+    CommandSelectByOpener,
     CommandLast
   };
 
-  // Returns true if the specified command is enabled.
+  // Returns true if the specified command is enabled. If |context_index| is
+  // selected the response applies to all selected tabs.
   bool IsContextMenuCommandEnabled(int context_index,
                                    ContextMenuCommand command_id) const;
 
-  // Returns true if the specified command is checked.
-  bool IsContextMenuCommandChecked(int context_index,
-                                   ContextMenuCommand command_id) const;
-
   // Performs the action associated with the specified command for the given
-  // TabStripModel index |context_index|.
+  // TabStripModel index |context_index|.  If |context_index| is selected the
+  // command applies to all selected tabs.
   void ExecuteContextMenuCommand(int context_index,
                                  ContextMenuCommand command_id);
 
@@ -408,18 +479,31 @@ class TabStripModel : public NotificationObserver {
   std::vector<int> GetIndicesClosedByCommand(int index,
                                              ContextMenuCommand id) const;
 
-  // Overridden from notificationObserver:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // Returns true if 'CommandTogglePinned' will pin. |index| is the index
+  // supplied to |ExecuteContextMenuCommand|.
+  bool WillContextMenuPin(int index);
+
+  // Overridden from content::NotificationObserver:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Convert a ContextMenuCommand into a browser command. Returns true if a
   // corresponding browser command exists, false otherwise.
   static bool ContextMenuCommandToBrowserCommand(int cmd_id, int* browser_cmd);
 
  private:
-  // We cannot be constructed without a delegate.
-  TabStripModel();
+  // Gets the set of tab indices whose domain matches the tab at |index|.
+  void GetIndicesWithSameDomain(int index, std::vector<int>* indices);
+
+  // Gets the set of tab indices that have the same opener as the tab at
+  // |index|.
+  void GetIndicesWithSameOpener(int index, std::vector<int>* indices);
+
+  // If |index| is selected all the selected indices are returned, otherwise a
+  // vector with |index| is returned. This is used when executing commands to
+  // determine which indices the command applies to.
+  std::vector<int> GetIndicesForCommand(int index) const;
 
   // Returns true if the specified TabContents is a New Tab at the end of the
   // TabStrip. We check for this because opener relationships are _not_
@@ -436,7 +520,8 @@ class TabStripModel : public NotificationObserver {
   //
   // Returns true if the TabContents were closed immediately, false if we are
   // waiting for the result of an onunload handler.
-  bool InternalCloseTabs(const std::vector<int>& indices, uint32 close_types);
+  bool InternalCloseTabs(const std::vector<int>& in_indices,
+                         uint32 close_types);
 
   // Invoked from InternalCloseTabs and when an extension is removed for an app
   // tab. Notifies observers of TabClosingAt and deletes |contents|. If
@@ -452,12 +537,22 @@ class TabStripModel : public NotificationObserver {
 
   TabContentsWrapper* GetContentsAt(int index) const;
 
-  // The actual implementation of SelectTabContentsAt. Takes the previously
-  // selected contents in |old_contents|, which may actually not be in
-  // |contents_| anymore because it may have been removed by a call to say
-  // DetachTabContentsAt...
-  void ChangeSelectedContentsFrom(
-      TabContentsWrapper* old_contents, int to_index, bool user_gesture);
+  // Notifies the observers if the active tab has changed. If |old_contents| is
+  // non-null a TabDeactivated notification is sent right before sending
+  // ActiveTabChanged notification.
+  void NotifyIfActiveTabChanged(TabContentsWrapper* old_contents,
+                                bool user_gesture);
+
+  // Notifies the observers if the active tab or the tab selection has changed.
+  // If |old_contents| is non-null a TabDeactivated notification is sent right
+  // before sending ActiveTabChanged notification. |old_model| is a snapshot of
+  // |selection_model_| before the change.
+  // Note: This function might end up sending 0 to 3 notifications in the
+  // following order: TabDeactivated, ActiveTabChanged, TabSelectionChanged.
+  void NotifyIfActiveOrSelectionChanged(
+      TabContentsWrapper* old_contents,
+      bool user_gesture,
+      const TabStripSelectionModel& old_model);
 
   // Returns the number of New Tab tabs in the TabStripModel.
   int GetNewTabCount() const;
@@ -472,16 +567,21 @@ class TabStripModel : public NotificationObserver {
                              int to_position,
                              bool select_after_move);
 
+  // Implementation of MoveSelectedTabsTo. Moves |length| of the selected tabs
+  // starting at |start| to |index|. See MoveSelectedTabsTo for more details.
+  void MoveSelectedTabsToImpl(int index, size_t start, size_t length);
+
   // Returns true if the tab represented by the specified data has an opener
   // that matches the specified one. If |use_group| is true, then this will
   // fall back to check the group relationship as well.
   struct TabContentsData;
   static bool OpenerMatches(const TabContentsData* data,
-                            const NavigationController* opener,
+                            const content::NavigationController* opener,
                             bool use_group);
 
   // Sets the group/opener of any tabs that reference |tab| to NULL.
-  void ForgetOpenersAndGroupsReferencing(const NavigationController* tab);
+  void ForgetOpenersAndGroupsReferencing(
+      const content::NavigationController* tab);
 
   // Our delegate.
   TabStripModelDelegate* delegate_;
@@ -495,13 +595,14 @@ class TabStripModel : public NotificationObserver {
         : contents(a_contents),
           reset_group_on_select(false),
           pinned(false),
-          blocked(false) {
+          blocked(false),
+          discarded(false) {
       SetGroup(NULL);
     }
 
     // Create a relationship between this TabContents and other TabContentses.
     // Used to identify which TabContents to select next after one is closed.
-    void SetGroup(NavigationController* a_group) {
+    void SetGroup(content::NavigationController* a_group) {
       group = a_group;
       opener = a_group;
     }
@@ -523,12 +624,12 @@ class TabStripModel : public NotificationObserver {
     // navigation within that tab, the group relationship is lost). This
     // property can safely be used to implement features that depend on a
     // logical group of related tabs.
-    NavigationController* group;
+    content::NavigationController* group;
     // The owner models the same relationship as group, except it is more
     // easily discarded, e.g. when the user switches to a tab not part of the
     // same group. This property is used to determine what tab to select next
     // when one is closed.
-    NavigationController* opener;
+    content::NavigationController* opener;
     // True if our group should be reset the moment selection moves away from
     // this Tab. This is the case for tabs opened in the foreground at the end
     // of the TabStrip while viewing another Tab. If these tabs are closed
@@ -542,15 +643,15 @@ class TabStripModel : public NotificationObserver {
 
     // Is the tab interaction blocked by a modal dialog?
     bool blocked;
+
+    // Has the tab data been discarded to save memory?
+    bool discarded;
   };
 
   // The TabContents data currently hosted within this TabStripModel.
   typedef std::vector<TabContentsData*> TabContentsDataVector;
   TabContentsDataVector contents_data_;
 
-  // The index of the TabContents in |contents_| that is currently selected.
-  int selected_index_;
-
   // A profile associated with this TabStripModel, used when creating new Tabs.
   Profile* profile_;
 
@@ -566,9 +667,11 @@ class TabStripModel : public NotificationObserver {
   TabStripModelObservers observers_;
 
   // A scoped container for notification registries.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
-  DISALLOW_COPY_AND_ASSIGN(TabStripModel);
+  TabStripSelectionModel selection_model_;
+
+  DISALLOW_IMPLICIT_CONSTRUCTORS(TabStripModel);
 };
 
 #endif  // CHROME_BROWSER_TABS_TAB_STRIP_MODEL_H_
