@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,15 +6,20 @@
 #define CHROME_BROWSER_TABS_TAB_STRIP_MODEL_DELEGATE_H_
 #pragma once
 
-#include "chrome/common/page_transition_types.h"
+#include <vector>
+
+#include "content/public/common/page_transition_types.h"
 
 class Browser;
 class DockInfo;
 class GURL;
 class Profile;
-class SiteInstance;
 class TabContents;
 class TabContentsWrapper;
+namespace content {
+class SiteInstance;
+struct Referrer;
+}
 namespace gfx {
 class Rect;
 }
@@ -63,11 +68,11 @@ class TabStripModelDelegate {
   // If |instance| is not null, its process is used to render the tab.
   virtual TabContentsWrapper* CreateTabContentsForURL(
       const GURL& url,
-      const GURL& referrer,
+      const content::Referrer& referrer,
       Profile* profile,
-      PageTransition::Type transition,
+      content::PageTransition transition,
       bool defer_load,
-      SiteInstance* instance) const = 0;
+      content::SiteInstance* instance) const = 0;
 
   // Returns whether some contents can be duplicated.
   virtual bool CanDuplicateContentsAt(int index) = 0;
@@ -97,8 +102,10 @@ class TabStripModelDelegate {
   // Restores the last closed tab if CanRestoreTab would return true.
   virtual void RestoreTab() = 0;
 
-  // Returns whether some contents can be closed.
-  virtual bool CanCloseContentsAt(int index) = 0;
+  // Returns true if all contents in array of |indices| can be closed.
+  // Returns false if one or more of the contents can't be closed.
+  // Indices of contents that cannot be closed will be removed from |indices|.
+  virtual bool CanCloseContents(std::vector<int>* indices) = 0;
 
   // Returns true if we should allow "bookmark all tabs" in this window; this is
   // true when there is more than one bookmarkable tab open.
@@ -110,12 +117,6 @@ class TabStripModelDelegate {
   // Returns true if any of the tabs can be closed.
   virtual bool CanCloseTab() const = 0;
 
-  // Returns true if the vertical tabstrip presentation should be used.
-  virtual bool UseVerticalTabs() const = 0;
-
-  // Toggles the use of the vertical tabstrip.
-  virtual void ToggleUseVerticalTabs() = 0;
-
   // Returns true if the tab strip can use large icons.
   virtual bool LargeIconsPermitted() const = 0;
 
