@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,7 @@
 
 class TabContentsWrapper;
 class TabStripModel;
+class TabStripSelectionModel;
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -56,19 +57,32 @@ class TabStripModelObserver {
   // action is necessary to deal with the TabContents no longer being present.
   virtual void TabDetachedAt(TabContentsWrapper* contents, int index);
 
-  // The selected TabContents is about to change from |old_contents| at |index|.
+  // The active TabContents is about to change from |old_contents|.
   // This gives observers a chance to prepare for an impending switch before it
   // happens.
-  virtual void TabDeselectedAt(TabContentsWrapper* contents, int index);
+  virtual void TabDeactivated(TabContentsWrapper* contents);
 
-  // The selected TabContents changed from |old_contents| to |new_contents| at
-  // |index|. |user_gesture| specifies whether or not this was done by a user
-  // input event (e.g. clicking on a tab, keystroke) or as a side-effect of
-  // some other function.
-  virtual void TabSelectedAt(TabContentsWrapper* old_contents,
-                             TabContentsWrapper* new_contents,
-                             int index,
-                             bool user_gesture);
+  // Sent when the active tab changes. The previously active tab is identified
+  // by |old_contents| and the newly active tab by |new_contents|. |index| is
+  // the index of |new_contents|. |user_gesture| specifies whether or not this
+  // was done by a user input event (e.g. clicking on a tab, keystroke) or as a
+  // side-effect of some other function.
+  // Note: It is possible for the selection to change while the active tab
+  // remains unchanged. For example, control-click may not change the active tab
+  // but does change the selection. In this case |ActiveTabChanged| is not sent.
+  // If you care about any changes to the selection, override
+  // TabSelectionChanged.
+  virtual void ActiveTabChanged(TabContentsWrapper* old_contents,
+                                TabContentsWrapper* new_contents,
+                                int index,
+                                bool user_gesture);
+
+  // Sent when the selection changes in |tab_strip_model|. More precisely when
+  // selected tabs, anchor tab or active tab change. |old_model| is a snapshot
+  // of the selection model before the change. See also ActiveTabChanged for
+  // details.
+  virtual void TabSelectionChanged(TabStripModel* tab_strip_model,
+                                   const TabStripSelectionModel& old_model);
 
   // The specified TabContents at |from_index| was moved to |to_index|.
   virtual void TabMoved(TabContentsWrapper* contents,
@@ -84,8 +98,9 @@ class TabStripModelObserver {
                             int index,
                             TabChangeType change_type);
 
-  // The tab contents was replaced at the specified index. This is invoked when
-  // instant is enabled and the user navigates by way of instant.
+  // The tab contents was replaced at the specified index. This is invoked
+  // when instant is enabled and the user navigates by way of instant or when
+  // prerendering swaps in a prerendered TabContents.
   virtual void TabReplacedAt(TabStripModel* tab_strip_model,
                              TabContentsWrapper* old_contents,
                              TabContentsWrapper* new_contents,
@@ -115,6 +130,10 @@ class TabStripModelObserver {
   // must be dropped.
   virtual void TabStripModelDeleted();
 
+  // Invoked when an active/selected tab at |index| is selected again (ie - the
+  // active/foreground tab is clicked).
+  virtual void ActiveTabClicked(int index);
+
  protected:
   virtual ~TabStripModelObserver() {}
 };
