@@ -1,10 +1,13 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/tabs/tab_strip_model_order_controller.h"
 
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "content/public/browser/web_contents.h"
+
+using content::NavigationController;
 
 ///////////////////////////////////////////////////////////////////////////////
 // TabStripModelOrderController, public:
@@ -22,7 +25,7 @@ TabStripModelOrderController::~TabStripModelOrderController() {
 
 int TabStripModelOrderController::DetermineInsertionIndex(
     TabContentsWrapper* new_contents,
-    PageTransition::Type transition,
+    content::PageTransition transition,
     bool foreground) {
   int tab_count = tabstrip_->count();
   if (!tab_count)
@@ -30,29 +33,30 @@ int TabStripModelOrderController::DetermineInsertionIndex(
 
   // NOTE: TabStripModel enforces that all non-mini-tabs occur after mini-tabs,
   // so we don't have to check here too.
-  if (transition == PageTransition::LINK && tabstrip_->selected_index() != -1) {
+  if (transition == content::PAGE_TRANSITION_LINK &&
+      tabstrip_->active_index() != -1) {
     int delta = (insertion_policy_ == TabStripModel::INSERT_AFTER) ? 1 : 0;
     if (foreground) {
       // If the page was opened in the foreground by a link click in another
       // tab, insert it adjacent to the tab that opened that link.
-      return tabstrip_->selected_index() + delta;
+      return tabstrip_->active_index() + delta;
     }
     NavigationController* opener =
-        &tabstrip_->GetSelectedTabContents()->controller();
+        &tabstrip_->GetActiveTabContents()->web_contents()->GetController();
     // Get the index of the next item opened by this tab, and insert after
     // it...
     int index;
     if (insertion_policy_ == TabStripModel::INSERT_AFTER) {
       index = tabstrip_->GetIndexOfLastTabContentsOpenedBy(
-          opener, tabstrip_->selected_index());
+          opener, tabstrip_->active_index());
     } else {
       index = tabstrip_->GetIndexOfFirstTabContentsOpenedBy(
-          opener, tabstrip_->selected_index());
+          opener, tabstrip_->active_index());
     }
     if (index != TabStripModel::kNoTab)
       return index + delta;
     // Otherwise insert adjacent to opener...
-    return tabstrip_->selected_index() + delta;
+    return tabstrip_->active_index() + delta;
   }
   // In other cases, such as Ctrl+T, open at the end of the strip.
   return DetermineInsertionIndexForAppending();
@@ -64,7 +68,7 @@ int TabStripModelOrderController::DetermineInsertionIndexForAppending() {
 }
 
 int TabStripModelOrderController::DetermineNewSelectedIndex(
-    int removing_index, volatile int* reason) const {
+    int removing_index) const {
   int tab_count = tabstrip_->count();
   DCHECK(removing_index >= 0 && removing_index < tab_count);
   NavigationController* parent_opener =
@@ -73,49 +77,41 @@ int TabStripModelOrderController::DetermineNewSelectedIndex(
   // want to select the first in that child group, not the next tab in the same
   // group of the removed tab.
   NavigationController* removed_controller =
-      &tabstrip_->GetTabContentsAt(removing_index)->controller();
+      &tabstrip_->GetTabContentsAt(removing_index)->
+          web_contents()->GetController();
   // The parent opener should never be the same as the controller being removed.
-  CHECK(parent_opener != removed_controller);
+  DCHECK(parent_opener != removed_controller);
   int index = tabstrip_->GetIndexOfNextTabContentsOpenedBy(removed_controller,
                                                            removing_index,
                                                            false);
-  if (index != TabStripModel::kNoTab) {
-    *reason = 1;
+  if (index != TabStripModel::kNoTab)
     return GetValidIndex(index, removing_index);
-  }
 
   if (parent_opener) {
     // If the tab was in a group, shift selection to the next tab in the group.
     int index = tabstrip_->GetIndexOfNextTabContentsOpenedBy(parent_opener,
                                                              removing_index,
                                                              false);
-    if (index != TabStripModel::kNoTab) {
-      *reason = 2;
+    if (index != TabStripModel::kNoTab)
       return GetValidIndex(index, removing_index);
-    }
 
     // If we can't find a subsequent group member, just fall back to the
     // parent_opener itself. Note that we use "group" here since opener is
     // reset by select operations..
     index = tabstrip_->GetIndexOfController(parent_opener);
-    if (index != TabStripModel::kNoTab) {
-      *reason = 3;
+    if (index != TabStripModel::kNoTab)
       return GetValidIndex(index, removing_index);
-    }
   }
 
   // No opener set, fall through to the default handler...
-  int selected_index = tabstrip_->selected_index();
-  if (selected_index >= (tab_count - 1)) {
-    *reason = 4;
+  int selected_index = tabstrip_->active_index();
+  if (selected_index >= (tab_count - 1))
     return selected_index - 1;
-  }
 
-  *reason = 5;
   return selected_index;
 }
 
-void TabStripModelOrderController::TabSelectedAt(
+void TabStripModelOrderController::ActiveTabChanged(
     TabContentsWrapper* old_contents,
     TabContentsWrapper* new_contents,
     int index,
@@ -132,11 +128,13 @@ void TabStripModelOrderController::TabSelectedAt(
         tabstrip_->ForgetGroup(old_contents);
     }
   }
-  NavigationController* new_opener =
-      tabstrip_->GetOpenerOfTabContentsAt(index);
+  NavigationController* new_opener = tabstrip_->GetOpenerOfTabContentsAt(index);
+
   if (user_gesture && new_opener != old_opener &&
-      new_opener != &old_contents->controller() &&
-      old_opener != &new_contents->controller()) {
+      ((old_contents == NULL && new_opener == NULL) ||
+          new_opener != &old_contents->web_contents()->GetController()) &&
+      ((new_contents == NULL && old_opener == NULL) ||
+          old_opener != &new_contents->web_contents()->GetController())) {
     tabstrip_->ForgetAllOpeners();
   }
 }
