@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,7 @@
 #pragma once
 
 #include "chrome/browser/tabs/tab_strip_model.h"
-#include "chrome/common/page_transition_types.h"
+#include "content/public/common/page_transition_types.h"
 
 class TabContentsWrapper;
 
@@ -33,21 +33,20 @@ class TabStripModelOrderController : public TabStripModelObserver {
   // Determine where to place a newly opened tab by using the supplied
   // transition and foreground flag to figure out how it was opened.
   int DetermineInsertionIndex(TabContentsWrapper* new_contents,
-                              PageTransition::Type transition,
+                              content::PageTransition transition,
                               bool foreground);
 
   // Returns the index to append tabs at.
   int DetermineInsertionIndexForAppending();
 
   // Determine where to shift selection after a tab is closed.
-  // TODO(sky): nuke reason when we figure out what is causing 34135.
-  int DetermineNewSelectedIndex(int removed_index, volatile int* reason) const;
+  int DetermineNewSelectedIndex(int removed_index) const;
 
   // Overridden from TabStripModelObserver:
-  virtual void TabSelectedAt(TabContentsWrapper* old_contents,
-                             TabContentsWrapper* new_contents,
-                             int index,
-                             bool user_gesture);
+  virtual void ActiveTabChanged(TabContentsWrapper* old_contents,
+                                TabContentsWrapper* new_contents,
+                                int index,
+                                bool user_gesture) OVERRIDE;
 
  private:
   // Returns a valid index to be selected after the tab at |removing_index| is
