@@ -1,44 +1,56 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/task_manager/task_manager.h"
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/i18n/number_formatting.h"
 #include "base/i18n/rtl.h"
 #include "base/process_util.h"
 #include "base/string_number_conversions.h"
-#include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "base/threading/thread.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_list.h"
+#include "chrome/browser/background/background_contents_service.h"
+#include "chrome/browser/background/background_contents_service_factory.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/browser_window.h"
-#include "chrome/browser/net/url_request_tracking.h"
+#include "chrome/browser/extensions/extension_host.h"
+#include "chrome/browser/extensions/extension_process_manager.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile_manager.h"
-#include "chrome/browser/renderer_host/render_process_host.h"
-#include "chrome/browser/renderer_host/resource_dispatcher_host.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/task_manager/task_manager_resource_providers.h"
+#include "chrome/browser/task_manager/task_manager_worker_resource_provider.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_view_type.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/common/result_codes.h"
 #include "chrome/common/url_constants.h"
-#include "grit/app_resources.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/renderer_host/resource_dispatcher_host.h"
+#include "content/browser/renderer_host/resource_dispatcher_host_request_info.h"
+#include "content/public/browser/browser_thread.h"
+#include "content/public/browser/render_view_host_delegate.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/common/result_codes.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
-#include "net/url_request/url_request.h"
-#include "net/url_request/url_request_job.h"
+#include "grit/ui_resources.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/text/bytes_formatting.h"
 #include "unicode/coll.h"
 
 #if defined(OS_MACOSX)
-#include "chrome/browser/mach_broker_mac.h"
+#include "content/browser/mach_broker_mac.h"
 #endif
 
+using content::BrowserThread;
+using content::OpenURLParams;
+using content::Referrer;
+
 namespace {
 
 // The delay between updates of the information (in ms).
@@ -60,8 +72,8 @@ int ValueCompare(T value1, T value2) {
 
 string16 FormatStatsSize(const WebKit::WebCache::ResourceTypeStat& stat) {
   return l10n_util::GetStringFUTF16(IDS_TASK_MANAGER_CACHE_SIZE_CELL_TEXT,
-      FormatBytes(stat.size, DATA_UNITS_KIBIBYTE, false),
-      FormatBytes(stat.liveSize, DATA_UNITS_KIBIBYTE, false));
+      ui::FormatBytesWithUnits(stat.size, ui::DATA_UNITS_KIBIBYTE, false),
+      ui::FormatBytesWithUnits(stat.liveSize, ui::DATA_UNITS_KIBIBYTE, false));
 }
 
 }  // namespace
@@ -73,45 +85,37 @@ string16 FormatStatsSize(const WebKit::WebCache::ResourceTypeStat& stat) {
 TaskManagerModel::TaskManagerModel(TaskManager* task_manager)
     : update_requests_(0),
       update_state_(IDLE),
-      goat_salt_(rand()) {
+      goat_salt_(rand()),
+      last_unique_id_(0) {
+  AddResourceProvider(
+      new TaskManagerBrowserProcessResourceProvider(task_manager));
+  AddResourceProvider(
+      new TaskManagerBackgroundContentsResourceProvider(task_manager));
+  AddResourceProvider(new TaskManagerTabContentsResourceProvider(task_manager));
+  AddResourceProvider(
+      new TaskManagerChildProcessResourceProvider(task_manager));
+  AddResourceProvider(
+      new TaskManagerExtensionProcessResourceProvider(task_manager));
 
-  TaskManagerBrowserProcessResourceProvider* browser_provider =
-      new TaskManagerBrowserProcessResourceProvider(task_manager);
-  browser_provider->AddRef();
-  providers_.push_back(browser_provider);
-  TaskManagerBackgroundContentsResourceProvider* bc_provider =
-      new TaskManagerBackgroundContentsResourceProvider(task_manager);
-  bc_provider->AddRef();
-  providers_.push_back(bc_provider);
-  TaskManagerTabContentsResourceProvider* wc_provider =
-      new TaskManagerTabContentsResourceProvider(task_manager);
-  wc_provider->AddRef();
-  providers_.push_back(wc_provider);
-  TaskManagerChildProcessResourceProvider* child_process_provider =
-      new TaskManagerChildProcessResourceProvider(task_manager);
-  child_process_provider->AddRef();
-  providers_.push_back(child_process_provider);
-  TaskManagerExtensionProcessResourceProvider* extension_process_provider =
-      new TaskManagerExtensionProcessResourceProvider(task_manager);
-  extension_process_provider->AddRef();
-  providers_.push_back(extension_process_provider);
-  TaskManagerNotificationResourceProvider* notification_provider =
-      new TaskManagerNotificationResourceProvider(task_manager);
-  notification_provider->AddRef();
-  providers_.push_back(notification_provider);
+  TaskManager::ResourceProvider* provider =
+      TaskManagerNotificationResourceProvider::Create(task_manager);
+  if (provider)
+    AddResourceProvider(provider);
+
+  AddResourceProvider(new TaskManagerWorkerResourceProvider(task_manager));
 }
 
 TaskManagerModel::~TaskManagerModel() {
-  for (ResourceProviderList::iterator iter = providers_.begin();
-       iter != providers_.end(); ++iter) {
-    (*iter)->Release();
-  }
 }
 
 int TaskManagerModel::ResourceCount() const {
   return resources_.size();
 }
 
+int TaskManagerModel::GroupCount() const {
+  return group_map_.size();
+}
+
 void TaskManagerModel::AddObserver(TaskManagerModelObserver* observer) {
   observer_list_.AddObserver(observer);
 }
@@ -120,9 +124,28 @@ void TaskManagerModel::RemoveObserver(TaskManagerModelObserver* observer) {
   observer_list_.RemoveObserver(observer);
 }
 
+int TaskManagerModel::GetResourceUniqueId(int index) const {
+  CHECK_LT(index, ResourceCount());
+  return resources_[index]->get_unique_id();
+}
+
+int TaskManagerModel::GetResourceIndexByUniqueId(const int unique_id) const {
+  for (int resource_index = 0; resource_index < ResourceCount();
+       ++resource_index) {
+    if (GetResourceUniqueId(resource_index) == unique_id)
+      return resource_index;
+  }
+  return -1;
+}
+
 string16 TaskManagerModel::GetResourceTitle(int index) const {
   CHECK_LT(index, ResourceCount());
-  return WideToUTF16Hack(resources_[index]->GetTitle());
+  return resources_[index]->GetTitle();
+}
+
+string16 TaskManagerModel::GetResourceProfileName(int index) const {
+  CHECK_LT(index, ResourceCount());
+  return resources_[index]->GetProfileName();
 }
 
 int64 TaskManagerModel::GetNetworkUsage(int index) const {
@@ -136,8 +159,7 @@ string16 TaskManagerModel::GetResourceNetworkUsage(int index) const {
     return l10n_util::GetStringUTF16(IDS_TASK_MANAGER_NA_CELL_TEXT);
   if (net_usage == 0)
     return ASCIIToUTF16("0");
-  string16 net_byte = FormatSpeed(net_usage, GetByteDisplayUnits(net_usage),
-                                  true);
+  string16 net_byte = ui::FormatSpeed(net_usage);
   // Force number string to have LTR directionality.
   return base::i18n::GetDisplayStringInLTRDirectionality(net_byte);
 }
@@ -149,13 +171,13 @@ double TaskManagerModel::GetCPUUsage(int index) const {
 
 string16 TaskManagerModel::GetResourceCPUUsage(int index) const {
   CHECK_LT(index, ResourceCount());
-  return WideToUTF16Hack(StringPrintf(
+  return UTF8ToUTF16(base::StringPrintf(
 #if defined(OS_MACOSX)
       // Activity Monitor shows %cpu with one decimal digit -- be
       // consistent with that.
-      L"%.1f",
+      "%.1f",
 #else
-      L"%.0f",
+      "%.0f",
 #endif
       GetCPUUsage(resources_[index])));
 }
@@ -224,6 +246,15 @@ string16 TaskManagerModel::GetResourceWebCoreCSSCacheSize(
   return FormatStatsSize(stats.cssStyleSheets);
 }
 
+string16 TaskManagerModel::GetResourceFPS(
+    int index) const {
+  CHECK_LT(index, ResourceCount());
+  if (!resources_[index]->ReportsFPS())
+    return l10n_util::GetStringUTF16(IDS_TASK_MANAGER_NA_CELL_TEXT);
+  double fps = resources_[index]->GetFPS();
+  return UTF8ToUTF16(base::StringPrintf("%.0f", fps));
+}
+
 string16 TaskManagerModel::GetResourceSqliteMemoryUsed(int index) const {
   CHECK_LT(index, ResourceCount());
   if (!resources_[index]->ReportsSqliteMemoryUsed())
@@ -236,12 +267,12 @@ string16 TaskManagerModel::GetResourceV8MemoryAllocatedSize(
   if (!resources_[index]->ReportsV8MemoryStats())
     return l10n_util::GetStringUTF16(IDS_TASK_MANAGER_NA_CELL_TEXT);
   return l10n_util::GetStringFUTF16(IDS_TASK_MANAGER_CACHE_SIZE_CELL_TEXT,
-      FormatBytes(resources_[index]->GetV8MemoryAllocated(),
-                  DATA_UNITS_KIBIBYTE,
-                  false),
-      FormatBytes(resources_[index]->GetV8MemoryUsed(),
-                  DATA_UNITS_KIBIBYTE,
-                  false));
+      ui::FormatBytesWithUnits(resources_[index]->GetV8MemoryAllocated(),
+                               ui::DATA_UNITS_KIBIBYTE,
+                               false),
+      ui::FormatBytesWithUnits(resources_[index]->GetV8MemoryUsed(),
+                               ui::DATA_UNITS_KIBIBYTE,
+                               false));
 }
 
 bool TaskManagerModel::IsResourceFirstInGroup(int index) const {
@@ -253,6 +284,15 @@ bool TaskManagerModel::IsResourceFirstInGroup(int index) const {
   return ((*group)[0] == resource);
 }
 
+bool TaskManagerModel::IsResourceLastInGroup(int index) const {
+  CHECK_LT(index, ResourceCount());
+  TaskManager::Resource* resource = resources_[index];
+  GroupMap::const_iterator iter = group_map_.find(resource->GetProcess());
+  DCHECK(iter != group_map_.end());
+  const ResourceList* group = iter->second;
+  return (group->back() == resource);
+}
+
 bool TaskManagerModel::IsBackgroundResource(int index) const {
   CHECK_LT(index, ResourceCount());
   return resources_[index]->IsBackground();
@@ -290,109 +330,130 @@ std::pair<int, int> TaskManagerModel::GetGroupRangeForResource(int index)
   }
 }
 
+int TaskManagerModel::GetGroupIndexForResource(int index) const {
+  int group_index = -1;
+  for (int i = 0; i <= index; ++i) {
+    if (IsResourceFirstInGroup(i))
+        group_index++;
+  }
+
+  DCHECK(group_index != -1);
+  return group_index;
+}
+
+int TaskManagerModel::GetResourceIndexForGroup(int group_index,
+                                               int index_in_group) const {
+  int group_count = -1;
+  int count_in_group = -1;
+  for (int i = 0; i < ResourceCount(); ++i) {
+    if (IsResourceFirstInGroup(i))
+      group_count++;
+
+    if (group_count == group_index) {
+      count_in_group++;
+      if (count_in_group == index_in_group)
+        return i;
+    } else if (group_count > group_index) {
+      break;
+    }
+  }
+
+  NOTREACHED();
+  return -1;
+}
+
 int TaskManagerModel::CompareValues(int row1, int row2, int col_id) const {
   CHECK(row1 < ResourceCount() && row2 < ResourceCount());
-  switch (col_id) {
-    case IDS_TASK_MANAGER_PAGE_COLUMN: {
-      // Let's do the default, string compare on the resource title.
-      static icu::Collator* collator = NULL;
-      if (!collator) {
-        UErrorCode create_status = U_ZERO_ERROR;
-        collator = icu::Collator::createInstance(create_status);
-        if (!U_SUCCESS(create_status)) {
-          collator = NULL;
-          NOTREACHED();
-        }
+  if (col_id == IDS_TASK_MANAGER_PAGE_COLUMN) {
+    // Let's do the default, string compare on the resource title.
+    static icu::Collator* collator = NULL;
+    if (!collator) {
+      UErrorCode create_status = U_ZERO_ERROR;
+      collator = icu::Collator::createInstance(create_status);
+      if (!U_SUCCESS(create_status)) {
+        collator = NULL;
+        NOTREACHED();
       }
-      string16 title1 = GetResourceTitle(row1);
-      string16 title2 = GetResourceTitle(row2);
-      UErrorCode compare_status = U_ZERO_ERROR;
-      UCollationResult compare_result = collator->compare(
-          static_cast<const UChar*>(title1.c_str()),
-          static_cast<int>(title1.length()),
-          static_cast<const UChar*>(title2.c_str()),
-          static_cast<int>(title2.length()),
-          compare_status);
-      DCHECK(U_SUCCESS(compare_status));
-      return compare_result;
     }
-
-    case IDS_TASK_MANAGER_NET_COLUMN:
-      return ValueCompare<int64>(GetNetworkUsage(resources_[row1]),
-                                 GetNetworkUsage(resources_[row2]));
-
-    case IDS_TASK_MANAGER_CPU_COLUMN:
-      return ValueCompare<double>(GetCPUUsage(resources_[row1]),
-                                  GetCPUUsage(resources_[row2]));
-
-    case IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN: {
-      size_t value1;
-      size_t value2;
-      if (!GetPrivateMemory(row1, &value1) || !GetPrivateMemory(row2, &value2))
-        return 0;
-      return ValueCompare<size_t>(value1, value2);
-    }
-
-    case IDS_TASK_MANAGER_SHARED_MEM_COLUMN: {
-      size_t value1;
-      size_t value2;
-      if (!GetSharedMemory(row1, &value1) || !GetSharedMemory(row2, &value2))
-        return 0;
-      return ValueCompare<size_t>(value1, value2);
-    }
-
-    case IDS_TASK_MANAGER_PHYSICAL_MEM_COLUMN: {
-      size_t value1;
-      size_t value2;
-      if (!GetPhysicalMemory(row1, &value1) ||
-          !GetPhysicalMemory(row2, &value2))
-        return 0;
-      return ValueCompare<size_t>(value1, value2);
-    }
-
-    case IDS_TASK_MANAGER_PROCESS_ID_COLUMN: {
-      int proc1_id = base::GetProcId(resources_[row1]->GetProcess());
-      int proc2_id = base::GetProcId(resources_[row2]->GetProcess());
-      return ValueCompare<int>(proc1_id, proc2_id);
-    }
-
-    case IDS_TASK_MANAGER_WEBCORE_IMAGE_CACHE_COLUMN:
-    case IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN:
-    case IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN: {
-      WebKit::WebCache::ResourceTypeStats stats1 = { { 0 } };
-      WebKit::WebCache::ResourceTypeStats stats2 = { { 0 } };
-      if (resources_[row1]->ReportsCacheStats())
-        stats1 = resources_[row1]->GetWebCoreCacheStats();
-      if (resources_[row2]->ReportsCacheStats())
-        stats2 = resources_[row2]->GetWebCoreCacheStats();
-      if (IDS_TASK_MANAGER_WEBCORE_IMAGE_CACHE_COLUMN == col_id)
-        return ValueCompare<size_t>(stats1.images.size, stats2.images.size);
-      if (IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN == col_id)
-        return ValueCompare<size_t>(stats1.scripts.size, stats2.scripts.size);
-      DCHECK_EQ(IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN, col_id);
-      return ValueCompare<size_t>(stats1.cssStyleSheets.size,
-                                  stats2.cssStyleSheets.size);
-    }
-
-    case IDS_TASK_MANAGER_GOATS_TELEPORTED_COLUMN: {
-      return ValueCompare<int>(GetGoatsTeleported(row1),
-                               GetGoatsTeleported(row2));
-    }
-
-    case IDS_TASK_MANAGER_JAVASCRIPT_MEMORY_ALLOCATED_COLUMN: {
-      size_t value1;
-      size_t value2;
-      bool reports_v8_memory1 = GetV8Memory(row1, &value1);
-      bool reports_v8_memory2 = GetV8Memory(row2, &value2);
-      if (reports_v8_memory1 == reports_v8_memory2)
-        return ValueCompare<size_t>(value1, value2);
-      else
-        return reports_v8_memory1 ? 1 : -1;
-    }
-
-    default:
-      NOTREACHED();
+    string16 title1 = GetResourceTitle(row1);
+    string16 title2 = GetResourceTitle(row2);
+    UErrorCode compare_status = U_ZERO_ERROR;
+    UCollationResult compare_result = collator->compare(
+        static_cast<const UChar*>(title1.c_str()),
+        static_cast<int>(title1.length()),
+        static_cast<const UChar*>(title2.c_str()),
+        static_cast<int>(title2.length()),
+        compare_status);
+    DCHECK(U_SUCCESS(compare_status));
+    return compare_result;
+  } else if (col_id == IDS_TASK_MANAGER_PROFILE_NAME_COLUMN) {
+    string16 profile1 = GetResourceProfileName(row1);
+    string16 profile2 = GetResourceProfileName(row2);
+    return profile1.compare(0, profile1.length(), profile2, 0,
+                            profile2.length());
+  } else if (col_id == IDS_TASK_MANAGER_NET_COLUMN) {
+    return ValueCompare<int64>(GetNetworkUsage(resources_[row1]),
+                               GetNetworkUsage(resources_[row2]));
+  } else if (col_id == IDS_TASK_MANAGER_CPU_COLUMN) {
+    return ValueCompare<double>(GetCPUUsage(resources_[row1]),
+                                GetCPUUsage(resources_[row2]));
+  } else if (col_id == IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN) {
+    size_t value1;
+    size_t value2;
+    if (!GetPrivateMemory(row1, &value1) || !GetPrivateMemory(row2, &value2))
       return 0;
+    return ValueCompare<size_t>(value1, value2);
+  } else if (col_id == IDS_TASK_MANAGER_SHARED_MEM_COLUMN) {
+    size_t value1;
+    size_t value2;
+    if (!GetSharedMemory(row1, &value1) || !GetSharedMemory(row2, &value2))
+      return 0;
+    return ValueCompare<size_t>(value1, value2);
+  } else if (col_id == IDS_TASK_MANAGER_PHYSICAL_MEM_COLUMN) {
+    size_t value1;
+    size_t value2;
+    if (!GetPhysicalMemory(row1, &value1) ||
+        !GetPhysicalMemory(row2, &value2))
+      return 0;
+    return ValueCompare<size_t>(value1, value2);
+  } else if (col_id == IDS_TASK_MANAGER_PROCESS_ID_COLUMN) {
+    int proc1_id = base::GetProcId(resources_[row1]->GetProcess());
+    int proc2_id = base::GetProcId(resources_[row2]->GetProcess());
+    return ValueCompare<int>(proc1_id, proc2_id);
+  } else if (col_id == IDS_TASK_MANAGER_WEBCORE_IMAGE_CACHE_COLUMN ||
+             col_id == IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN ||
+             col_id == IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN) {
+    WebKit::WebCache::ResourceTypeStats stats1 = { { 0 } };
+    WebKit::WebCache::ResourceTypeStats stats2 = { { 0 } };
+    if (resources_[row1]->ReportsCacheStats())
+      stats1 = resources_[row1]->GetWebCoreCacheStats();
+    if (resources_[row2]->ReportsCacheStats())
+      stats2 = resources_[row2]->GetWebCoreCacheStats();
+    if (IDS_TASK_MANAGER_WEBCORE_IMAGE_CACHE_COLUMN == col_id)
+      return ValueCompare<size_t>(stats1.images.size, stats2.images.size);
+    if (IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN == col_id)
+      return ValueCompare<size_t>(stats1.scripts.size, stats2.scripts.size);
+    DCHECK_EQ(IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN, col_id);
+    return ValueCompare<size_t>(stats1.cssStyleSheets.size,
+                                stats2.cssStyleSheets.size);
+  } else if (col_id == IDS_TASK_MANAGER_FPS_COLUMN) {
+    return ValueCompare<float>(resources_[row1]->GetFPS(),
+                               resources_[row2]->GetFPS());
+  } else if (col_id == IDS_TASK_MANAGER_GOATS_TELEPORTED_COLUMN) {
+    return ValueCompare<int>(GetGoatsTeleported(row1),
+                             GetGoatsTeleported(row2));
+  } else if (col_id == IDS_TASK_MANAGER_JAVASCRIPT_MEMORY_ALLOCATED_COLUMN) {
+    size_t value1;
+    size_t value2;
+    bool reports_v8_memory1 = GetV8Memory(row1, &value1);
+    bool reports_v8_memory2 = GetV8Memory(row2, &value2);
+    if (reports_v8_memory1 == reports_v8_memory2)
+      return ValueCompare<size_t>(value1, value2);
+    else
+      return reports_v8_memory1 ? 1 : -1;
+  } else {
+    NOTREACHED();
+    return 0;
   }
 }
 
@@ -407,7 +468,7 @@ TaskManager::Resource::Type TaskManagerModel::GetResourceType(int index) const {
   return resources_[index]->GetType();
 }
 
-TabContents* TaskManagerModel::GetResourceTabContents(int index) const {
+TabContentsWrapper* TaskManagerModel::GetResourceTabContents(int index) const {
   CHECK_LT(index, ResourceCount());
   return resources_[index]->GetTabContents();
 }
@@ -482,6 +543,34 @@ bool TaskManagerModel::GetPhysicalMemory(int index, size_t* result) const {
   return true;
 }
 
+bool TaskManagerModel::GetWebCoreCacheStats(
+    int index, WebKit::WebCache::ResourceTypeStats* result) const {
+  if (!resources_[index]->ReportsCacheStats())
+    return false;
+
+  *result = resources_[index]->GetWebCoreCacheStats();
+  return true;
+}
+
+bool TaskManagerModel::GetFPS(int index, float* result) const {
+  *result = 0;
+  if (!resources_[index]->ReportsFPS())
+    return false;
+
+  *result = resources_[index]->GetFPS();
+  return true;
+}
+
+bool TaskManagerModel::GetSqliteMemoryUsedBytes(
+    int index, size_t* result) const {
+  *result = 0;
+  if (!resources_[index]->ReportsSqliteMemoryUsed())
+    return false;
+
+  *result = resources_[index]->SqliteMemoryUsedBytes();
+  return true;
+}
+
 bool TaskManagerModel::GetV8Memory(int index, size_t* result) const {
   *result = 0;
   if (!resources_[index]->ReportsV8MemoryStats())
@@ -491,6 +580,21 @@ bool TaskManagerModel::GetV8Memory(int index, size_t* result) const {
   return true;
 }
 
+bool TaskManagerModel::CanActivate(int index) const {
+  CHECK_LT(index, ResourceCount());
+  return GetResourceTabContents(index) != NULL;
+}
+
+bool TaskManagerModel::CanInspect(int index) const {
+  CHECK_LT(index, ResourceCount());
+  return resources_[index]->CanInspect();
+}
+
+void TaskManagerModel::Inspect(int index) const {
+  CHECK_LT(index, ResourceCount());
+  resources_[index]->Inspect();
+}
+
 int TaskManagerModel::GetGoatsTeleported(int index) const {
   int seed = goat_salt_ * (index + 1);
   return (seed >> 16) & 255;
@@ -504,9 +608,9 @@ string16 TaskManagerModel::GetMemCellText(int64 number) const {
   base::i18n::AdjustStringForLocaleDirection(&str);
   return l10n_util::GetStringFUTF16(IDS_TASK_MANAGER_MEM_CELL_TEXT, str);
 #else
-  // System expectation is to show "100 KB", "200 MB", etc.
+  // System expectation is to show "100 kB", "200 MB", etc.
   // TODO(thakis): Switch to metric units (as opposed to powers of two).
-  return FormatBytes(number, GetByteDisplayUnits(number), /*show_units=*/true);
+  return ui::FormatBytes(number);
 #endif
 }
 
@@ -522,19 +626,13 @@ void TaskManagerModel::StartUpdating() {
   // If update_state_ is STOPPING, it means a task is still pending.  Setting
   // it to TASK_PENDING ensures the tasks keep being posted (by Refresh()).
   if (update_state_ == IDLE) {
-      MessageLoop::current()->PostDelayedTask(FROM_HERE,
-          NewRunnableMethod(this, &TaskManagerModel::Refresh),
-          kUpdateTimeMs);
+      MessageLoop::current()->PostDelayedTask(
+          FROM_HERE,
+          base::Bind(&TaskManagerModel::Refresh, this),
+          base::TimeDelta::FromMilliseconds(kUpdateTimeMs));
   }
   update_state_ = TASK_PENDING;
 
-  // Register jobs notifications so we can compute network usage (it must be
-  // done from the IO thread).
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(
-         this, &TaskManagerModel::RegisterForJobDoneNotifications));
-
   // Notify resource providers that we are updating.
   for (ResourceProviderList::iterator iter = providers_.begin();
        iter != providers_.end(); ++iter) {
@@ -559,12 +657,6 @@ void TaskManagerModel::StopUpdating() {
     (*iter)->StopUpdating();
   }
 
-  // Unregister jobs notification (must be done from the IO thread).
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(
-          this, &TaskManagerModel::UnregisterForJobDoneNotifications));
-
   // Must clear the resources before the next attempt to start updating.
   Clear();
 }
@@ -575,25 +667,9 @@ void TaskManagerModel::AddResourceProvider(
   providers_.push_back(provider);
 }
 
-void TaskManagerModel::RemoveResourceProvider(
-    TaskManager::ResourceProvider* provider) {
-  DCHECK(provider);
-  ResourceProviderList::iterator iter = std::find(providers_.begin(),
-                                                  providers_.end(),
-                                                  provider);
-  DCHECK(iter != providers_.end());
-  providers_.erase(iter);
-}
-
-void TaskManagerModel::RegisterForJobDoneNotifications() {
-  net::g_url_request_job_tracker.AddObserver(this);
-}
-
-void TaskManagerModel::UnregisterForJobDoneNotifications() {
-  net::g_url_request_job_tracker.RemoveObserver(this);
-}
-
 void TaskManagerModel::AddResource(TaskManager::Resource* resource) {
+  resource->unique_id_ = ++last_unique_id_;
+
   base::ProcessHandle process = resource->GetProcess();
 
   ResourceList* group_entries = NULL;
@@ -719,6 +795,7 @@ void TaskManagerModel::Clear() {
     FOR_EACH_OBSERVER(TaskManagerModelObserver, observer_list_,
                       OnItemsRemoved(0, size));
   }
+  last_unique_id_ = 0;
 }
 
 void TaskManagerModel::ModelChanged() {
@@ -737,6 +814,18 @@ void TaskManagerModel::NotifyResourceTypeStats(
   }
 }
 
+void TaskManagerModel::NotifyFPS(base::ProcessId renderer_id,
+                                 int routing_id,
+                                 float fps) {
+  for (ResourceList::iterator it = resources_.begin();
+       it != resources_.end(); ++it) {
+    if (base::GetProcId((*it)->GetProcess()) == renderer_id &&
+        (*it)->GetRoutingId() == routing_id) {
+      (*it)->NotifyFPS(fps);
+    }
+  }
+}
+
 void TaskManagerModel::NotifyV8HeapStats(base::ProcessId renderer_id,
                                          size_t v8_memory_allocated,
                                          size_t v8_memory_used) {
@@ -782,14 +871,16 @@ void TaskManagerModel::Refresh() {
 
   // Compute the new network usage values.
   displayed_network_usage_map_.clear();
+  base::TimeDelta update_time =
+      base::TimeDelta::FromMilliseconds(kUpdateTimeMs);
   for (ResourceValueMap::iterator iter = current_byte_count_map_.begin();
        iter != current_byte_count_map_.end(); ++iter) {
-    if (kUpdateTimeMs > 1000) {
-      int divider = (kUpdateTimeMs / 1000);
+    if (update_time > base::TimeDelta::FromSeconds(1)) {
+      int divider = update_time.InSeconds();
       displayed_network_usage_map_[iter->first] = iter->second / divider;
     } else {
       displayed_network_usage_map_[iter->first] = iter->second *
-          (1000 / kUpdateTimeMs);
+          (1 / update_time.InSeconds());
     }
 
     // Then we reset the current byte count.
@@ -808,9 +899,10 @@ void TaskManagerModel::Refresh() {
   }
 
   // Schedule the next update.
-  MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      NewRunnableMethod(this, &TaskManagerModel::Refresh),
-      kUpdateTimeMs);
+  MessageLoop::current()->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(&TaskManagerModel::Refresh, this),
+      base::TimeDelta::FromMilliseconds(kUpdateTimeMs));
 }
 
 int64 TaskManagerModel::GetNetworkUsageForResource(
@@ -839,7 +931,7 @@ void TaskManagerModel::BytesRead(BytesReadParam param) {
   TaskManager::Resource* resource = NULL;
   for (ResourceProviderList::iterator iter = providers_.begin();
        iter != providers_.end(); ++iter) {
-    resource = (*iter)->GetResource(param.origin_child_id,
+    resource = (*iter)->GetResource(param.origin_pid,
                                     param.render_process_host_child_id,
                                     param.routing_id);
     if (resource)
@@ -851,9 +943,8 @@ void TaskManagerModel::BytesRead(BytesReadParam param) {
     // tab that started a download was closed, or the request may have had
     // no originating resource associated with it in the first place.
     // We attribute orphaned/unaccounted activity to the Browser process.
-    int browser_pid = base::GetCurrentProcId();
-    CHECK(param.origin_child_id != browser_pid);
-    param.origin_child_id = browser_pid;
+    CHECK(param.origin_pid || (param.render_process_host_child_id != -1));
+    param.origin_pid = 0;
     param.render_process_host_child_id = param.routing_id = -1;
     BytesRead(param);
     return;
@@ -873,48 +964,32 @@ void TaskManagerModel::BytesRead(BytesReadParam param) {
 }
 
 
-// In order to retrieve the network usage, we register for net::URLRequestJob
-// notifications. Every time we get notified some bytes were read we bump a
-// counter of read bytes for the associated resource. When the timer ticks,
-// we'll compute the actual network usage (see the Refresh method).
-void TaskManagerModel::OnJobAdded(net::URLRequestJob* job) {
-}
-
-void TaskManagerModel::OnJobRemoved(net::URLRequestJob* job) {
-}
-
-void TaskManagerModel::OnJobDone(net::URLRequestJob* job,
-                                 const net::URLRequestStatus& status) {
-}
-
-void TaskManagerModel::OnJobRedirect(net::URLRequestJob* job,
-                                     const GURL& location,
-                                     int status_code) {
-}
-
-void TaskManagerModel::OnBytesRead(net::URLRequestJob* job, const char* buf,
-                                   int byte_count) {
+void TaskManagerModel::NotifyBytesRead(const net::URLRequest& request,
+                                       int byte_count) {
   // Only net::URLRequestJob instances created by the ResourceDispatcherHost
   // have a render view associated.  All other jobs will have -1 returned for
   // the render process child and routing ids - the jobs may still match a
   // resource based on their origin id, otherwise BytesRead() will attribute
   // the activity to the Browser resource.
   int render_process_host_child_id = -1, routing_id = -1;
-  ResourceDispatcherHost::RenderViewForRequest(job->request(),
+  ResourceDispatcherHost::RenderViewForRequest(&request,
                                                &render_process_host_child_id,
                                                &routing_id);
 
+  // Get the origin PID of the request's originator.  This will only be set for
+  // plugins - for renderer or browser initiated requests it will be zero.
+  int origin_pid = 0;
+  const ResourceDispatcherHostRequestInfo* info =
+      ResourceDispatcherHost::InfoForRequest(&request);
+  if (info)
+    origin_pid = info->origin_pid();
+
   // This happens in the IO thread, post it to the UI thread.
-  int origin_child_id =
-      chrome_browser_net::GetOriginProcessUniqueIDForRequest(job->request());
   BrowserThread::PostTask(
       BrowserThread::UI, FROM_HERE,
-      NewRunnableMethod(
-          this,
-          &TaskManagerModel::BytesRead,
-          BytesReadParam(origin_child_id,
-          render_process_host_child_id,
-          routing_id, byte_count)));
+      base::Bind(&TaskManagerModel::BytesRead, this,
+                 BytesReadParam(origin_pid, render_process_host_child_id,
+                                routing_id, byte_count)));
 }
 
 bool TaskManagerModel::GetProcessMetricsForRow(
@@ -958,24 +1033,19 @@ void TaskManager::KillProcess(int index) {
   base::ProcessHandle process = model_->GetResourceProcessHandle(index);
   DCHECK(process);
   if (process != base::GetCurrentProcessHandle())
-    base::KillProcess(process, ResultCodes::KILLED, false);
+    base::KillProcess(process, content::RESULT_CODE_KILLED, false);
 }
 
 void TaskManager::ActivateProcess(int index) {
   // GetResourceTabContents returns a pointer to the relevant tab contents for
   // the resource.  If the index doesn't correspond to a Tab (i.e. refers to
   // the Browser process or a plugin), GetTabContents will return NULL.
-  TabContents* chosen_tab_contents = model_->GetResourceTabContents(index);
-  if (chosen_tab_contents)
-    chosen_tab_contents->Activate();
-}
-
-void TaskManager::AddResourceProvider(ResourceProvider* provider) {
-  model_->AddResourceProvider(provider);
-}
-
-void TaskManager::RemoveResourceProvider(ResourceProvider* provider) {
-  model_->RemoveResourceProvider(provider);
+  TabContentsWrapper* chosen_tab_contents =
+      model_->GetResourceTabContents(index);
+  if (chosen_tab_contents) {
+    chosen_tab_contents->web_contents()->GetRenderViewHost()->delegate()->
+        Activate();
+  }
 }
 
 void TaskManager::AddResource(Resource* resource) {
@@ -1001,35 +1071,33 @@ TaskManager* TaskManager::GetInstance() {
 
 void TaskManager::OpenAboutMemory() {
   Browser* browser = BrowserList::GetLastActive();
+  OpenURLParams params(
+      GURL(chrome::kChromeUIMemoryURL), Referrer(), NEW_FOREGROUND_TAB,
+      content::PAGE_TRANSITION_LINK, false);
 
   if (!browser) {
     // On OS X, the task manager can be open without any open browser windows.
-    if (!g_browser_process ||
-        !g_browser_process->profile_manager() ||
-        g_browser_process->profile_manager()->begin() ==
-            g_browser_process->profile_manager()->end())
+    if (!g_browser_process || !g_browser_process->profile_manager())
       return;
-    browser = Browser::Create(*g_browser_process->profile_manager()->begin());
-    browser->OpenURL(GURL(chrome::kAboutMemoryURL), GURL(), NEW_FOREGROUND_TAB,
-                     PageTransition::LINK);
-    browser->window()->Show();
+    Profile* profile =
+        g_browser_process->profile_manager()->GetLastUsedProfile();
+    if (!profile)
+      return;
+    browser = Browser::Create(profile);
+    browser->OpenURL(params);
   } else {
-    browser->OpenURL(GURL(chrome::kAboutMemoryURL), GURL(), NEW_FOREGROUND_TAB,
-                     PageTransition::LINK);
+    browser->OpenURL(params);
 
-    // In case the browser window is minimzed, show it. If |browser| is a
+    // In case the browser window is minimized, show it. If |browser| is a
     // non-tabbed window, the call to OpenURL above will have opened a
-    // TabContents in a tabbed browser, so we need to grab it with GetLastActive
-    // before the call to show().
-    if (browser->type() & (Browser::TYPE_APP |
-                           Browser::TYPE_DEVTOOLS |
-                           Browser::TYPE_POPUP)) {
+    // TabContents in a tabbed browser, so we need to grab it with
+    // GetLastActive before the call to show().
+    if (!browser->is_type_tabbed()) {
       browser = BrowserList::GetLastActive();
       DCHECK(browser);
     }
-
-    browser->window()->Show();
   }
+  browser->window()->Show();
 }
 
 bool TaskManagerModel::GetAndCacheMemoryMetrics(
@@ -1045,3 +1113,54 @@ bool TaskManagerModel::GetAndCacheMemoryMetrics(
   memory_usage_map_.insert(std::make_pair(handle, *usage));
   return true;
 }
+
+namespace {
+
+// Counts the number of extension background pages associated with this profile.
+int CountExtensionBackgroundPagesForProfile(Profile* profile) {
+  int count = 0;
+  ExtensionProcessManager* manager = profile->GetExtensionProcessManager();
+  if (!manager)
+    return count;
+  for (ExtensionProcessManager::const_iterator iter = manager->begin();
+       iter != manager->end();
+       ++iter) {
+    if ((*iter)->extension_host_type() ==
+        chrome::VIEW_TYPE_EXTENSION_BACKGROUND_PAGE) {
+      count++;
+    }
+  }
+  return count;
+}
+
+}  // namespace
+
+// static
+int TaskManager::GetBackgroundPageCount() {
+  int count = 0;
+  ProfileManager* profile_manager = g_browser_process->profile_manager();
+  if (!profile_manager)  // Null when running unit tests.
+    return count;
+  std::vector<Profile*> profiles(profile_manager->GetLoadedProfiles());
+  for (std::vector<Profile*>::const_iterator iter = profiles.begin();
+       iter != profiles.end();
+       ++iter) {
+    Profile* profile = *iter;
+    // Count the number of Background Contents (background pages for hosted
+    // apps). Incognito windows do not support hosted apps, so just check the
+    // main profile.
+    BackgroundContentsService* background_contents_service =
+        BackgroundContentsServiceFactory::GetForProfile(profile);
+    if (background_contents_service)
+      count += background_contents_service->GetBackgroundContents().size();
+
+    // Count the number of extensions with background pages (including
+    // incognito).
+    count += CountExtensionBackgroundPagesForProfile(profile);
+    if (profile->HasOffTheRecordProfile()) {
+      count += CountExtensionBackgroundPagesForProfile(
+          profile->GetOffTheRecordProfile());
+    }
+  }
+  return count;
+}
