@@ -13,24 +13,43 @@
 
 #include "base/basictypes.h"
 #include "base/gtest_prod_util.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/singleton.h"
 #include "base/observer_list.h"
 #include "base/process_util.h"
-#include "base/ref_counted.h"
-#include "base/singleton.h"
 #include "base/string16.h"
 #include "base/timer.h"
 #include "chrome/browser/renderer_host/web_cache_manager.h"
-#include "net/url_request/url_request_job_tracker.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebCache.h"
 
 class Extension;
 class SkBitmap;
-class TabContents;
+class TabContentsWrapper;
 class TaskManagerModel;
 
 namespace base {
 class ProcessMetrics;
 }
+namespace net {
+class URLRequest;
+}
+
+#define TASKMANAGER_RESOURCE_TYPE_LIST(def) \
+    def(BROWSER)         /* The main browser process. */ \
+    def(RENDERER)        /* A normal TabContents renderer process. */ \
+    def(EXTENSION)       /* An extension or app process. */ \
+    def(NOTIFICATION)    /* A notification process. */ \
+    def(PLUGIN)          /* A plugin process. */ \
+    def(WORKER)          /* A web worker process. */ \
+    def(NACL)            /* A NativeClient loader or broker process. */ \
+    def(UTILITY)         /* A browser utility process. */ \
+    def(PROFILE_IMPORT)  /* A profile import process. */ \
+    def(ZYGOTE)          /* A Linux zygote process. */ \
+    def(SANDBOX_HELPER)  /* A sandbox helper process. */ \
+    def(GPU)             /* A graphics process. */
+
+#define TASKMANAGER_RESOURCE_TYPE_LIST_ENUM(a)   a,
+#define TASKMANAGER_RESOURCE_TYPE_LIST_AS_STRING(a)   case a: return #a;
 
 // This class is a singleton.
 class TaskManager {
@@ -42,31 +61,25 @@ class TaskManager {
     virtual ~Resource() {}
 
     enum Type {
-      UNKNOWN = 0,     // An unknown process type.
-      BROWSER,         // The main browser process.
-      RENDERER,        // A normal TabContents renderer process.
-      EXTENSION,       // An extension or app process.
-      NOTIFICATION,    // A notification process.
-      PLUGIN,          // A plugin process.
-      WORKER,          // A web worker process.
-      NACL,            // A NativeClient loader or broker process.
-      UTILITY,         // A browser utility process.
-      PROFILE_IMPORT,  // A profile import process.
-      ZYGOTE,          // A Linux zygote process.
-      SANDBOX_HELPER,  // A sandbox helper process.
-      GPU              // A graphics process.
+      UNKNOWN = 0,
+      TASKMANAGER_RESOURCE_TYPE_LIST(TASKMANAGER_RESOURCE_TYPE_LIST_ENUM)
     };
 
-    virtual std::wstring GetTitle() const = 0;
+    virtual string16 GetTitle() const = 0;
+    virtual string16 GetProfileName() const = 0;
     virtual SkBitmap GetIcon() const = 0;
     virtual base::ProcessHandle GetProcess() const = 0;
     virtual Type GetType() const = 0;
+    virtual int GetRoutingId() const { return 0; }
 
     virtual bool ReportsCacheStats() const { return false; }
     virtual WebKit::WebCache::ResourceTypeStats GetWebCoreCacheStats() const {
       return WebKit::WebCache::ResourceTypeStats();
     }
 
+    virtual bool ReportsFPS() const { return false; }
+    virtual float GetFPS() const { return 0.0f; }
+
     virtual bool ReportsSqliteMemoryUsed() const { return false; }
     virtual size_t SqliteMemoryUsedBytes() const { return 0; }
 
@@ -78,9 +91,15 @@ class TaskManager {
     virtual size_t GetV8MemoryAllocated() const { return 0; }
     virtual size_t GetV8MemoryUsed() const { return 0; }
 
+    // Returns true if this resource can be inspected using developer tools.
+    virtual bool CanInspect() const { return false; }
+
+    // Invokes or reveals developer tools window for this resource.
+    virtual void Inspect() const {}
+
     // A helper function for ActivateFocusedTab.  Returns NULL by default
     // because not all resources have an associated tab.
-    virtual TabContents* GetTabContents() const { return NULL; }
+    virtual TabContentsWrapper* GetTabContents() const { return NULL; }
 
     // Whether this resource does report the network usage accurately.
     // This controls whether 0 or N/A is displayed when no bytes have been
@@ -99,12 +118,30 @@ class TaskManager {
 
     virtual void NotifyResourceTypeStats(
         const WebKit::WebCache::ResourceTypeStats& stats) {}
+    virtual void NotifyFPS(float fps) {}
     virtual void NotifyV8HeapStats(size_t v8_memory_allocated,
                                    size_t v8_memory_used) {}
 
     // Returns true if this resource is not visible to the user because it lives
     // in the background (e.g. extension background page, background contents).
     virtual bool IsBackground() const { return false; }
+
+    static const char* GetResourceTypeAsString(const Type type) {
+      switch (type) {
+        TASKMANAGER_RESOURCE_TYPE_LIST(TASKMANAGER_RESOURCE_TYPE_LIST_AS_STRING)
+        default: return "UNKNOWN";
+      }
+    }
+
+    // Returns resource identifier that is unique within single task manager
+    // session (between StartUpdating and StopUpdating).
+    int get_unique_id() { return unique_id_; }
+   protected:
+    Resource() : unique_id_(0) {}
+
+   private:
+    friend class TaskManagerModel;
+    int unique_id_;
   };
 
   // ResourceProviders are responsible for adding/removing resources to the task
@@ -147,9 +184,6 @@ class TaskManager {
   // index.
   void ActivateProcess(int index);
 
-  void AddResourceProvider(ResourceProvider* provider);
-  void RemoveResourceProvider(ResourceProvider* provider);
-
   // These methods are invoked by the resource providers to add/remove resources
   // to the Task Manager. Note that the resources are owned by the
   // ResourceProviders and are not valid after StopUpdating() has been called
@@ -172,6 +206,11 @@ class TaskManager {
 
   void OpenAboutMemory();
 
+  // Returns the number of background pages that will be displayed in the
+  // TaskManager. Used by the wrench menu code to display a count of background
+  // pages in the "View Background Pages" menu item.
+  static int GetBackgroundPageCount();
+
  private:
   FRIEND_TEST_ALL_PREFIXES(TaskManagerTest, Basic);
   FRIEND_TEST_ALL_PREFIXES(TaskManagerTest, Resources);
@@ -194,6 +233,11 @@ class TaskManager {
   DISALLOW_COPY_AND_ASSIGN(TaskManager);
 };
 
+#undef TASKMANAGER_RESOURCE_TYPE_LIST
+#undef DEFINE_ENUM
+#undef DEFINE_CONVERT_TO_STRING
+
+
 class TaskManagerModelObserver {
  public:
   virtual ~TaskManagerModelObserver() {}
@@ -212,8 +256,7 @@ class TaskManagerModelObserver {
 };
 
 // The model that the TaskManager is using.
-class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
-                         public base::RefCountedThreadSafe<TaskManagerModel> {
+class TaskManagerModel : public base::RefCountedThreadSafe<TaskManagerModel> {
  public:
   explicit TaskManagerModel(TaskManager* task_manager);
 
@@ -222,14 +265,21 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
 
   // Returns number of registered resources.
   int ResourceCount() const;
+  // Returns number of registered groups.
+  int GroupCount() const;
 
   // Methods to return raw resource information.
   int64 GetNetworkUsage(int index) const;
   double GetCPUUsage(int index) const;
   int GetProcessId(int index) const;
+  int GetResourceUniqueId(int index) const;
+  // Returns the index of resource that has the given |unique_id|. Returns -1 if
+  // no resouce has the |unique_id|.
+  int GetResourceIndexByUniqueId(const int unique_id) const;
 
   // Methods to return formatted resource information.
   string16 GetResourceTitle(int index) const;
+  string16 GetResourceProfileName(int index) const;
   string16 GetResourceNetworkUsage(int index) const;
   string16 GetResourceCPUUsage(int index) const;
   string16 GetResourcePrivateMemory(int index) const;
@@ -239,6 +289,7 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   string16 GetResourceWebCoreImageCacheSize(int index) const;
   string16 GetResourceWebCoreScriptsCacheSize(int index) const;
   string16 GetResourceWebCoreCSSCacheSize(int index) const;
+  string16 GetResourceFPS(int index) const;
   string16 GetResourceSqliteMemoryUsed(int index) const;
   string16 GetResourceGoatsTeleported(int index) const;
   string16 GetResourceV8MemoryAllocatedSize(int index) const;
@@ -257,16 +308,40 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   // resource index.
   bool GetPhysicalMemory(int index, size_t* result) const;
 
+  // Gets the statuses of webkit. Return false if the resource for the given row
+  // isn't a renderer.
+  bool GetWebCoreCacheStats(int index,
+                            WebKit::WebCache::ResourceTypeStats* result) const;
+
+  // Gets the fps of the given page. Return false if the resource for the given
+  // row isn't a renderer.
+  bool GetFPS(int index, float* result) const;
+
+  // Gets the sqlite memory (in byte). Return false if the resource for the
+  // given row doesn't report information.
+  bool GetSqliteMemoryUsedBytes(int index, size_t* result) const;
+
   // Gets the amount of memory allocated for javascript. Returns false if the
   // resource for the given row isn't a renderer.
   bool GetV8Memory(int index, size_t* result) const;
 
+  // Returns true if resource for the given row can be activated.
+  bool CanActivate(int index) const;
+
+  // Returns true if resource for the given row can be inspected using developer
+  // tools.
+  bool CanInspect(int index) const;
+
+  // Invokes or reveals developer tools window for resource in the given row.
+  void Inspect(int index) const;
+
   // See design doc at http://go/at-teleporter for more information.
   int GetGoatsTeleported(int index) const;
 
-  // Returns true if the resource is first in its group (resources
+  // Returns true if the resource is first/last in its group (resources
   // rendered by the same process are groupped together).
   bool IsResourceFirstInGroup(int index) const;
+  bool IsResourceLastInGroup(int index) const;
 
   // Returns true if the resource runs in the background (not visible to the
   // user, e.g. extension background pages and BackgroundContents).
@@ -278,6 +353,13 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   // Returns a pair (start, length) of the group range of resource.
   std::pair<int, int> GetGroupRangeForResource(int index) const;
 
+  // Returns an index of groups to which the resource belongs.
+  int GetGroupIndexForResource(int index) const;
+
+  // Returns an index of resource which belongs to the |group_index|th group
+  // and which is the |index_in_group|th resource in group.
+  int GetResourceIndexForGroup(int group_index, int index_in_group) const;
+
   // Compares values in column |col_id| and rows |row1|, |row2|.
   // Returns -1 if value in |row1| is less than value in |row2|,
   // 0 if they are equal, and 1 otherwise.
@@ -290,26 +372,11 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   TaskManager::Resource::Type GetResourceType(int index) const;
 
   // Returns TabContents of given resource or NULL if not applicable.
-  TabContents* GetResourceTabContents(int index) const;
+  TabContentsWrapper* GetResourceTabContents(int index) const;
 
   // Returns Extension of given resource or NULL if not applicable.
   const Extension* GetResourceExtension(int index) const;
 
-  // JobObserver methods:
-  virtual void OnJobAdded(net::URLRequestJob* job);
-  virtual void OnJobRemoved(net::URLRequestJob* job);
-  virtual void OnJobDone(net::URLRequestJob* job,
-                         const net::URLRequestStatus& status);
-  virtual void OnJobRedirect(net::URLRequestJob* job,
-                             const GURL& location,
-                             int status_code);
-  virtual void OnBytesRead(net::URLRequestJob* job,
-                           const char* buf,
-                           int byte_count);
-
-  void AddResourceProvider(TaskManager::ResourceProvider* provider);
-  void RemoveResourceProvider(TaskManager::ResourceProvider* provider);
-
   void AddResource(TaskManager::Resource* resource);
   void RemoveResource(TaskManager::Resource* resource);
 
@@ -326,10 +393,16 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
         base::ProcessId renderer_id,
         const WebKit::WebCache::ResourceTypeStats& stats);
 
+  void NotifyFPS(base::ProcessId renderer_id,
+                 int routing_id,
+                 float fps);
+
   void NotifyV8HeapStats(base::ProcessId renderer_id,
                          size_t v8_memory_allocated,
                          size_t v8_memory_used);
 
+  void NotifyBytesRead(const net::URLRequest& request, int bytes_read);
+
  private:
   friend class base::RefCountedThreadSafe<TaskManagerModel>;
   FRIEND_TEST_ALL_PREFIXES(TaskManagerTest, RefreshCalled);
@@ -345,21 +418,20 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
 
   // This struct is used to exchange information between the io and ui threads.
   struct BytesReadParam {
-    BytesReadParam(int origin_child_id,
+    BytesReadParam(int origin_pid,
                    int render_process_host_child_id,
                    int routing_id,
                    int byte_count)
-        : origin_child_id(origin_child_id),
+        : origin_pid(origin_pid),
           render_process_host_child_id(render_process_host_child_id),
           routing_id(routing_id),
           byte_count(byte_count) {}
 
-    // This is the child ID of the originator of the request. It will often be
-    // the same as the render_process_host_child_id, but will be different when
-    // another sub-process like a plugin is routing requests through a renderer.
-    int origin_child_id;
+    // The process ID that triggered the request.  For plugin requests this
+    // will differ from the renderer process ID.
+    int origin_pid;
 
-    // The child ID of the RenderProcessHist this request was routed through.
+    // The child ID of the RenderProcessHost this request was routed through.
     int render_process_host_child_id;
 
     int routing_id;
@@ -367,7 +439,8 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   };
 
   typedef std::vector<TaskManager::Resource*> ResourceList;
-  typedef std::vector<TaskManager::ResourceProvider*> ResourceProviderList;
+  typedef std::vector<scoped_refptr<TaskManager::ResourceProvider> >
+      ResourceProviderList;
   typedef std::map<base::ProcessHandle, ResourceList*> GroupMap;
   typedef std::map<base::ProcessHandle, base::ProcessMetrics*> MetricsMap;
   typedef std::map<base::ProcessHandle, double> CPUUsageMap;
@@ -381,10 +454,6 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   void AddItem(TaskManager::Resource* resource, bool notify_table);
   void RemoveItem(TaskManager::Resource* resource);
 
-  // Register for network usage updates
-  void RegisterForJobDoneNotifications();
-  void UnregisterForJobDoneNotifications();
-
   // Returns the network usage (in bytes per seconds) for the specified
   // resource. That's the value retrieved at the last timer's tick.
   int64 GetNetworkUsageForResource(TaskManager::Resource* resource) const;
@@ -415,6 +484,9 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   bool GetAndCacheMemoryMetrics(base::ProcessHandle handle,
                                 std::pair<size_t, size_t>* usage) const;
 
+  // Adds a resource provider to be managed.
+  void AddResourceProvider(TaskManager::ResourceProvider* provider);
+
   // The list of providers to the task manager. They are ref counted.
   ResourceProviderList providers_;
 
@@ -461,6 +533,9 @@ class TaskManagerModel : public net::URLRequestJobTracker::JobObserver,
   // A salt lick for the goats.
   int goat_salt_;
 
+  // Resource identifier that is unique within single session.
+  int last_unique_id_;
+
   DISALLOW_COPY_AND_ASSIGN(TaskManagerModel);
 };
 
