@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,11 +10,13 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "base/process_util.h"
 #include "chrome/browser/task_manager/task_manager.h"
-#include "chrome/common/child_process_info.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/child_process_data.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/common/process_type.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebCache.h"
 
 class BackgroundContents;
@@ -22,7 +24,7 @@ class BalloonHost;
 class Extension;
 class ExtensionHost;
 class RenderViewHost;
-class TabContents;
+class TabContentsWrapper;
 
 // These file contains the resource providers used in the task manager.
 
@@ -35,25 +37,36 @@ class TaskManagerRendererResource : public TaskManager::Resource {
   virtual ~TaskManagerRendererResource();
 
   // TaskManager::Resource methods:
-  virtual base::ProcessHandle GetProcess() const;
-  virtual Type GetType() const;
-  virtual bool ReportsCacheStats() const;
-  virtual WebKit::WebCache::ResourceTypeStats GetWebCoreCacheStats() const;
-  virtual bool ReportsV8MemoryStats() const;
-  virtual size_t GetV8MemoryAllocated() const;
-  virtual size_t GetV8MemoryUsed() const;
+  virtual base::ProcessHandle GetProcess() const OVERRIDE;
+  virtual Type GetType() const OVERRIDE;
+  virtual int GetRoutingId() const OVERRIDE;
+
+  virtual bool ReportsCacheStats() const OVERRIDE;
+  virtual WebKit::WebCache::ResourceTypeStats GetWebCoreCacheStats() const
+      OVERRIDE;
+  virtual bool ReportsFPS() const OVERRIDE;
+  virtual float GetFPS() const OVERRIDE;
+  virtual bool ReportsV8MemoryStats() const OVERRIDE;
+  virtual size_t GetV8MemoryAllocated() const OVERRIDE;
+  virtual size_t GetV8MemoryUsed() const OVERRIDE;
+
+  // RenderResources are always inspectable.
+  virtual bool CanInspect() const OVERRIDE;
+  virtual void Inspect() const OVERRIDE;
 
   // RenderResources always provide the network usage.
-  virtual bool SupportNetworkUsage() const;
-  virtual void SetSupportNetworkUsage() { }
+  virtual bool SupportNetworkUsage() const OVERRIDE;
+  virtual void SetSupportNetworkUsage() OVERRIDE { }
 
-  virtual void Refresh();
+  virtual void Refresh() OVERRIDE;
 
   virtual void NotifyResourceTypeStats(
-      const WebKit::WebCache::ResourceTypeStats& stats);
+      const WebKit::WebCache::ResourceTypeStats& stats) OVERRIDE;
+
+  virtual void NotifyFPS(float fps) OVERRIDE;
 
   virtual void NotifyV8HeapStats(size_t v8_memory_allocated,
-                                 size_t v8_memory_used);
+                                 size_t v8_memory_used) OVERRIDE;
 
  private:
   base::ProcessHandle process_;
@@ -67,6 +80,11 @@ class TaskManagerRendererResource : public TaskManager::Resource {
   // This flag is true if we are waiting for the renderer to report its stats.
   bool pending_stats_update_;
 
+  // The fps_ field holds the renderer frames per second.
+  float fps_;
+  // This flag is true if we are waiting for the renderer to report its FPS.
+  bool pending_fps_update_;
+
   // We do a similar dance to gather the V8 memory usage in a process.
   size_t v8_memory_allocated_;
   size_t v8_memory_used_;
@@ -77,46 +95,59 @@ class TaskManagerRendererResource : public TaskManager::Resource {
 
 class TaskManagerTabContentsResource : public TaskManagerRendererResource {
  public:
-  explicit TaskManagerTabContentsResource(TabContents* tab_contents);
+  explicit TaskManagerTabContentsResource(TabContentsWrapper* tab_contents);
   virtual ~TaskManagerTabContentsResource();
 
+  // Called when the underlying tab_contents has been committed, and is thus no
+  // longer an Instant preview.
+  void InstantCommitted();
+
   // TaskManager::Resource methods:
-  virtual Type GetType() const;
-  virtual std::wstring GetTitle() const;
-  virtual SkBitmap GetIcon() const;
-  virtual TabContents* GetTabContents() const;
-  virtual const Extension* GetExtension() const;
+  virtual Type GetType() const OVERRIDE;
+  virtual string16 GetTitle() const OVERRIDE;
+  virtual string16 GetProfileName() const OVERRIDE;
+  virtual SkBitmap GetIcon() const OVERRIDE;
+  virtual TabContentsWrapper* GetTabContents() const OVERRIDE;
+  virtual const Extension* GetExtension() const OVERRIDE;
 
  private:
-  TabContents* tab_contents_;
+  bool IsPrerendering() const;
+
+  // Returns true if contains content rendered by an extension.
+  bool HostsExtension() const;
+
+  static SkBitmap* prerender_icon_;
+  TabContentsWrapper* tab_contents_;
+  bool is_instant_preview_;
 
   DISALLOW_COPY_AND_ASSIGN(TaskManagerTabContentsResource);
 };
 
 class TaskManagerTabContentsResourceProvider
     : public TaskManager::ResourceProvider,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
   explicit TaskManagerTabContentsResourceProvider(TaskManager* task_manager);
 
   virtual TaskManager::Resource* GetResource(int origin_pid,
                                              int render_process_host_id,
-                                             int routing_id);
-  virtual void StartUpdating();
-  virtual void StopUpdating();
+                                             int routing_id) OVERRIDE;
+  virtual void StartUpdating() OVERRIDE;
+  virtual void StopUpdating() OVERRIDE;
 
-  // NotificationObserver method:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver method:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
  private:
   virtual ~TaskManagerTabContentsResourceProvider();
 
-  void Add(TabContents* tab_contents);
-  void Remove(TabContents* tab_contents);
+  void Add(TabContentsWrapper* tab_contents);
+  void Remove(TabContentsWrapper* tab_contents);
+  void Update(TabContentsWrapper* tab_contents);
 
-  void AddToTaskManager(TabContents* tab_contents);
+  void AddToTaskManager(TabContentsWrapper* tab_contents);
 
   // Whether we are currently reporting to the task manager. Used to ignore
   // notifications sent after StopUpdating().
@@ -124,12 +155,12 @@ class TaskManagerTabContentsResourceProvider
 
   TaskManager* task_manager_;
 
-  // Maps the actual resources (the TabContents) to the Task Manager
+  // Maps the actual resources (the TabContentsWrappers) to the Task Manager
   // resources.
-  std::map<TabContents*, TaskManagerTabContentsResource*> resources_;
+  std::map<TabContentsWrapper*, TaskManagerTabContentsResource*> resources_;
 
   // A scoped container for notification registries.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(TaskManagerTabContentsResourceProvider);
 };
@@ -139,19 +170,20 @@ class TaskManagerBackgroundContentsResource
  public:
   TaskManagerBackgroundContentsResource(
       BackgroundContents* background_contents,
-      const std::wstring& application_name);
+      const string16& application_name);
   virtual ~TaskManagerBackgroundContentsResource();
 
   // TaskManager::Resource methods:
-  virtual std::wstring GetTitle() const;
-  virtual SkBitmap GetIcon() const;
-  virtual bool IsBackground() const;
+  virtual string16 GetTitle() const OVERRIDE;
+  virtual string16 GetProfileName() const OVERRIDE;
+  virtual SkBitmap GetIcon() const OVERRIDE;
+  virtual bool IsBackground() const OVERRIDE;
 
-  const std::wstring& application_name() const { return application_name_; }
+  const string16& application_name() const { return application_name_; }
  private:
   BackgroundContents* background_contents_;
 
-  std::wstring application_name_;
+  string16 application_name_;
 
   // The icon painted for BackgroundContents.
   // TODO(atwilson): Use the favicon when there's a way to get the favicon for
@@ -163,30 +195,30 @@ class TaskManagerBackgroundContentsResource
 
 class TaskManagerBackgroundContentsResourceProvider
     : public TaskManager::ResourceProvider,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
   explicit TaskManagerBackgroundContentsResourceProvider(
       TaskManager* task_manager);
 
   virtual TaskManager::Resource* GetResource(int origin_pid,
                                              int render_process_host_id,
-                                             int routing_id);
-  virtual void StartUpdating();
-  virtual void StopUpdating();
+                                             int routing_id) OVERRIDE;
+  virtual void StartUpdating() OVERRIDE;
+  virtual void StopUpdating() OVERRIDE;
 
-  // NotificationObserver method:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver method:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
  private:
   virtual ~TaskManagerBackgroundContentsResourceProvider();
 
-  void Add(BackgroundContents* background_contents, const std::wstring& title);
+  void Add(BackgroundContents* background_contents, const string16& title);
   void Remove(BackgroundContents* background_contents);
 
   void AddToTaskManager(BackgroundContents* background_contents,
-                        const std::wstring& title);
+                        const string16& title);
 
   // Whether we are currently reporting to the task manager. Used to ignore
   // notifications sent after StopUpdating().
@@ -196,35 +228,45 @@ class TaskManagerBackgroundContentsResourceProvider
 
   // Maps the actual resources (the BackgroundContents) to the Task Manager
   // resources.
-  std::map<BackgroundContents*, TaskManagerBackgroundContentsResource*>
-      resources_;
+  typedef std::map<BackgroundContents*, TaskManagerBackgroundContentsResource*>
+      Resources;
+  Resources resources_;
 
   // A scoped container for notification registries.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(TaskManagerBackgroundContentsResourceProvider);
 };
 
 class TaskManagerChildProcessResource : public TaskManager::Resource {
  public:
-  explicit TaskManagerChildProcessResource(const ChildProcessInfo& child_proc);
+  TaskManagerChildProcessResource(content::ProcessType type,
+                                  const string16& name,
+                                  base::ProcessHandle handle);
   virtual ~TaskManagerChildProcessResource();
 
-  // TaskManagerResource methods:
-  virtual std::wstring GetTitle() const;
-  virtual SkBitmap GetIcon() const;
-  virtual base::ProcessHandle GetProcess() const;
-  virtual Type GetType() const;
-  virtual bool SupportNetworkUsage() const;
-  virtual void SetSupportNetworkUsage();
+  // TaskManager::Resource methods:
+  virtual string16 GetTitle() const OVERRIDE;
+  virtual string16 GetProfileName() const OVERRIDE;
+  virtual SkBitmap GetIcon() const OVERRIDE;
+  virtual base::ProcessHandle GetProcess() const OVERRIDE;
+  virtual Type GetType() const OVERRIDE;
+  virtual bool SupportNetworkUsage() const OVERRIDE;
+  virtual void SetSupportNetworkUsage() OVERRIDE;
 
   // Returns the pid of the child process.
   int process_id() const { return pid_; }
 
  private:
-  ChildProcessInfo child_process_;
+  // Returns a localized title for the child process.  For example, a plugin
+  // process would be "Plug-in: Flash" when name is "Flash".
+  string16 GetLocalizedTitle() const;
+
+  content::ProcessType type_;
+  string16 name_;
+  base::ProcessHandle handle_;
   int pid_;
-  mutable std::wstring title_;
+  mutable string16 title_;
   bool network_usage_support_;
 
   // The icon painted for the child processs.
@@ -237,54 +279,57 @@ class TaskManagerChildProcessResource : public TaskManager::Resource {
 
 class TaskManagerChildProcessResourceProvider
     : public TaskManager::ResourceProvider,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
   explicit TaskManagerChildProcessResourceProvider(TaskManager* task_manager);
 
   virtual TaskManager::Resource* GetResource(int origin_pid,
                                              int render_process_host_id,
-                                             int routing_id);
-  virtual void StartUpdating();
-  virtual void StopUpdating();
+                                             int routing_id) OVERRIDE;
+  virtual void StartUpdating() OVERRIDE;
+  virtual void StopUpdating() OVERRIDE;
 
-  // NotificationObserver method:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver method:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
-  // Retrieves the current ChildProcessInfo (performed in the IO thread).
-  virtual void RetrieveChildProcessInfo();
+ private:
+  virtual ~TaskManagerChildProcessResourceProvider();
 
-  // Notifies the UI thread that the ChildProcessInfo have been retrieved.
-  virtual void ChildProcessInfoRetreived();
+  // Retrieves information about the running ChildProcessHosts (performed in the
+  // IO thread).
+  virtual void RetrieveChildProcessData();
+
+  // Notifies the UI thread that the ChildProcessHosts information have been
+  // retrieved.
+  virtual void ChildProcessDataRetreived(
+      const std::vector<content::ChildProcessData>& child_processes);
+
+  void Add(const content::ChildProcessData& child_process_data);
+  void Remove(const content::ChildProcessData& child_process_data);
+
+  void AddToTaskManager(const content::ChildProcessData& child_process_data);
+
+  TaskManager* task_manager_;
 
   // Whether we are currently reporting to the task manager. Used to ignore
   // notifications sent after StopUpdating().
   bool updating_;
 
-  // The list of ChildProcessInfo retrieved when starting the update.
-  std::vector<ChildProcessInfo> existing_child_process_info_;
-
- private:
-  virtual ~TaskManagerChildProcessResourceProvider();
-
-  void Add(const ChildProcessInfo& child_process_info);
-  void Remove(const ChildProcessInfo& child_process_info);
-
-  void AddToTaskManager(const ChildProcessInfo& child_process_info);
-
-  TaskManager* task_manager_;
-
-  // Maps the actual resources (the ChildProcessInfo) to the Task Manager
+  // Maps the actual resources (the ChildProcessData) to the Task Manager
   // resources.
-  std::map<ChildProcessInfo, TaskManagerChildProcessResource*> resources_;
+  typedef std::map<base::ProcessHandle, TaskManagerChildProcessResource*>
+      ChildProcessMap;
+  ChildProcessMap resources_;
 
   // Maps the pids to the resources (used for quick access to the resource on
   // byte read notifications).
-  std::map<int, TaskManagerChildProcessResource*> pid_to_resources_;
+  typedef std::map<int, TaskManagerChildProcessResource*> PidResourceMap;
+  PidResourceMap pid_to_resources_;
 
   // A scoped container for notification registries.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(TaskManagerChildProcessResourceProvider);
 };
@@ -294,20 +339,23 @@ class TaskManagerExtensionProcessResource : public TaskManager::Resource {
   explicit TaskManagerExtensionProcessResource(ExtensionHost* extension_host);
   virtual ~TaskManagerExtensionProcessResource();
 
-  // TaskManagerResource methods:
-  virtual std::wstring GetTitle() const;
-  virtual SkBitmap GetIcon() const;
-  virtual base::ProcessHandle GetProcess() const;
-  virtual Type GetType() const;
-  virtual bool SupportNetworkUsage() const;
-  virtual void SetSupportNetworkUsage();
-  virtual const Extension* GetExtension() const;
+  // TaskManager::Resource methods:
+  virtual string16 GetTitle() const OVERRIDE;
+  virtual string16 GetProfileName() const OVERRIDE;
+  virtual SkBitmap GetIcon() const OVERRIDE;
+  virtual base::ProcessHandle GetProcess() const OVERRIDE;
+  virtual Type GetType() const OVERRIDE;
+  virtual bool CanInspect() const OVERRIDE;
+  virtual void Inspect() const OVERRIDE;
+  virtual bool SupportNetworkUsage() const OVERRIDE;
+  virtual void SetSupportNetworkUsage() OVERRIDE;
+  virtual const Extension* GetExtension() const OVERRIDE;
 
   // Returns the pid of the extension process.
   int process_id() const { return pid_; }
 
   // Returns true if the associated extension has a background page.
-  virtual bool IsBackground() const;
+  virtual bool IsBackground() const OVERRIDE;
 
  private:
   // The icon painted for the extension process.
@@ -318,28 +366,28 @@ class TaskManagerExtensionProcessResource : public TaskManager::Resource {
   // Cached data about the extension.
   base::ProcessHandle process_handle_;
   int pid_;
-  std::wstring title_;
+  string16 title_;
 
   DISALLOW_COPY_AND_ASSIGN(TaskManagerExtensionProcessResource);
 };
 
 class TaskManagerExtensionProcessResourceProvider
     : public TaskManager::ResourceProvider,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
   explicit TaskManagerExtensionProcessResourceProvider(
       TaskManager* task_manager);
 
   virtual TaskManager::Resource* GetResource(int origin_pid,
                                              int render_process_host_id,
-                                             int routing_id);
-  virtual void StartUpdating();
-  virtual void StopUpdating();
+                                             int routing_id) OVERRIDE;
+  virtual void StartUpdating() OVERRIDE;
+  virtual void StopUpdating() OVERRIDE;
 
-  // NotificationObserver method:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver method:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
  private:
   virtual ~TaskManagerExtensionProcessResourceProvider();
@@ -357,7 +405,7 @@ class TaskManagerExtensionProcessResourceProvider
   std::map<int, TaskManagerExtensionProcessResource*> pid_to_resources_;
 
   // A scoped container for notification registries.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   bool updating_;
 
@@ -370,12 +418,15 @@ class TaskManagerNotificationResource : public TaskManager::Resource {
   virtual ~TaskManagerNotificationResource();
 
   // TaskManager::Resource interface
-  virtual std::wstring GetTitle() const;
-  virtual SkBitmap GetIcon() const;
-  virtual base::ProcessHandle GetProcess() const;
-  virtual Type GetType() const;
-  virtual bool SupportNetworkUsage() const;
-  virtual void SetSupportNetworkUsage() { }
+  virtual string16 GetTitle() const OVERRIDE;
+  virtual string16 GetProfileName() const OVERRIDE;
+  virtual SkBitmap GetIcon() const OVERRIDE;
+  virtual base::ProcessHandle GetProcess() const OVERRIDE;
+  virtual Type GetType() const OVERRIDE;
+  virtual bool CanInspect() const OVERRIDE;
+  virtual void Inspect() const OVERRIDE;
+  virtual bool SupportNetworkUsage() const OVERRIDE;
+  virtual void SetSupportNetworkUsage() OVERRIDE { }
 
  private:
   // The icon painted for notifications.       .
@@ -387,30 +438,32 @@ class TaskManagerNotificationResource : public TaskManager::Resource {
   // Cached data about the balloon host.
   base::ProcessHandle process_handle_;
   int pid_;
-  std::wstring title_;
+  string16 title_;
 
   DISALLOW_COPY_AND_ASSIGN(TaskManagerNotificationResource);
 };
 
 class TaskManagerNotificationResourceProvider
     : public TaskManager::ResourceProvider,
-      public NotificationObserver {
+      public content::NotificationObserver {
  public:
-  explicit TaskManagerNotificationResourceProvider(TaskManager* task_manager);
+  static TaskManagerNotificationResourceProvider* Create(
+      TaskManager* task_manager);
 
   // TaskManager::ResourceProvider interface
   virtual TaskManager::Resource* GetResource(int origin_pid,
                                              int render_process_host_id,
-                                             int routing_id);
-  virtual void StartUpdating();
-  virtual void StopUpdating();
+                                             int routing_id) OVERRIDE;
+  virtual void StartUpdating() OVERRIDE;
+  virtual void StopUpdating() OVERRIDE;
 
-  // NotificationObserver interface
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver interface
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
  private:
+  explicit TaskManagerNotificationResourceProvider(TaskManager* task_manager);
   virtual ~TaskManagerNotificationResourceProvider();
 
   void AddToTaskManager(BalloonHost* balloon_host);
@@ -422,7 +475,7 @@ class TaskManagerNotificationResourceProvider
   std::map<BalloonHost*, TaskManagerNotificationResource*> resources_;
 
   // A scoped container for notification registries.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   bool updating_;
 
@@ -434,25 +487,26 @@ class TaskManagerBrowserProcessResource : public TaskManager::Resource {
   TaskManagerBrowserProcessResource();
   virtual ~TaskManagerBrowserProcessResource();
 
-  // TaskManagerResource methods:
-  virtual std::wstring GetTitle() const;
-  virtual SkBitmap GetIcon() const;
-  virtual base::ProcessHandle GetProcess() const;
-  virtual Type GetType() const;
+  // TaskManager::Resource methods:
+  virtual string16 GetTitle() const OVERRIDE;
+  virtual string16 GetProfileName() const OVERRIDE;
+  virtual SkBitmap GetIcon() const OVERRIDE;
+  virtual base::ProcessHandle GetProcess() const OVERRIDE;
+  virtual Type GetType() const OVERRIDE;
 
-  virtual bool SupportNetworkUsage() const;
-  virtual void SetSupportNetworkUsage();
+  virtual bool SupportNetworkUsage() const OVERRIDE;
+  virtual void SetSupportNetworkUsage() OVERRIDE;
 
-  virtual bool ReportsSqliteMemoryUsed() const;
-  virtual size_t SqliteMemoryUsedBytes() const;
+  virtual bool ReportsSqliteMemoryUsed() const OVERRIDE;
+  virtual size_t SqliteMemoryUsedBytes() const OVERRIDE;
 
-  // Returns the pid of the browser process.
-  int process_id() const { return pid_; }
+  virtual bool ReportsV8MemoryStats() const OVERRIDE;
+  virtual size_t GetV8MemoryAllocated() const OVERRIDE;
+  virtual size_t GetV8MemoryUsed() const OVERRIDE;
 
  private:
   base::ProcessHandle process_;
-  int pid_;
-  mutable std::wstring title_;
+  mutable string16 title_;
 
   static SkBitmap* default_icon_;
 
@@ -467,9 +521,9 @@ class TaskManagerBrowserProcessResourceProvider
 
   virtual TaskManager::Resource* GetResource(int origin_pid,
                                              int render_process_host_id,
-                                             int routing_id);
-  virtual void StartUpdating();
-  virtual void StopUpdating();
+                                             int routing_id) OVERRIDE;
+  virtual void StartUpdating() OVERRIDE;
+  virtual void StopUpdating() OVERRIDE;
 
   // Whether we are currently reporting to the task manager. Used to ignore
   // notifications sent after StopUpdating().
@@ -478,8 +532,6 @@ class TaskManagerBrowserProcessResourceProvider
  private:
   virtual ~TaskManagerBrowserProcessResourceProvider();
 
-  void AddToTaskManager(ChildProcessInfo child_process_info);
-
   TaskManager* task_manager_;
   TaskManagerBrowserProcessResource resource_;
 
