@@ -1,42 +1,50 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/themes/browser_theme_pack.h"
 
-#include "base/stl_util-inl.h"
+#include <limits>
+
+#include "base/memory/scoped_ptr.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
-#include "gfx/codec/png_codec.h"
-#include "gfx/skbitmap_operations.h"
-#include "grit/app_resources.h"
+#include "chrome/browser/themes/theme_service.h"
+#include "content/public/browser/browser_thread.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
 #include "net/base/file_stream.h"
 #include "net/base/net_errors.h"
 #include "third_party/skia/include/core/SkCanvas.h"
 #include "ui/base/resource/data_pack.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/codec/png_codec.h"
+#include "ui/gfx/image/image.h"
+#include "ui/gfx/skbitmap_operations.h"
+
+using content::BrowserThread;
 
 namespace {
 
 // Version number of the current theme pack. We just throw out and rebuild
 // theme packs that aren't int-equal to this.
-const int kThemePackVersion = 15;
+const int kThemePackVersion = 18;
 
 // IDs that are in the DataPack won't clash with the positive integer
-// int32_t. kHeaderID should always have the maximum value because we want the
+// uint16. kHeaderID should always have the maximum value because we want the
 // "header" to be written last. That way we can detect whether the pack was
 // successfully written and ignore and regenerate if it was only partially
 // written (i.e. chrome crashed on a different thread while writing the pack).
-const int kHeaderID = UINT_MAX - 1;
-const int kTintsID = UINT_MAX - 2;
-const int kColorsID = UINT_MAX - 3;
-const int kDisplayPropertiesID = UINT_MAX - 4;
-const int kSourceImagesID = UINT_MAX - 5;
+const int kMaxID = 0x0000FFFF;  // Max unsigned 16-bit int.
+const int kHeaderID = kMaxID - 1;
+const int kTintsID = kMaxID - 2;
+const int kColorsID = kMaxID - 3;
+const int kDisplayPropertiesID = kMaxID - 4;
+const int kSourceImagesID = kMaxID - 5;
 
 // Static size of the tint/color/display property arrays that are mmapped.
 const int kTintArraySize = 6;
@@ -174,57 +182,57 @@ struct StringToIntTable {
 
 // Strings used by themes to identify tints in the JSON.
 StringToIntTable kTintTable[] = {
-  { "buttons", BrowserThemeProvider::TINT_BUTTONS },
-  { "frame", BrowserThemeProvider::TINT_FRAME },
-  { "frame_inactive", BrowserThemeProvider::TINT_FRAME_INACTIVE },
-  { "frame_incognito", BrowserThemeProvider::TINT_FRAME_INCOGNITO },
+  { "buttons", ThemeService::TINT_BUTTONS },
+  { "frame", ThemeService::TINT_FRAME },
+  { "frame_inactive", ThemeService::TINT_FRAME_INACTIVE },
+  { "frame_incognito", ThemeService::TINT_FRAME_INCOGNITO },
   { "frame_incognito_inactive",
-    BrowserThemeProvider::TINT_FRAME_INCOGNITO_INACTIVE },
-  { "background_tab", BrowserThemeProvider::TINT_BACKGROUND_TAB },
+    ThemeService::TINT_FRAME_INCOGNITO_INACTIVE },
+  { "background_tab", ThemeService::TINT_BACKGROUND_TAB },
   { NULL, 0 }
 };
 
 // Strings used by themes to identify colors in the JSON.
 StringToIntTable kColorTable[] = {
-  { "frame", BrowserThemeProvider::COLOR_FRAME },
-  { "frame_inactive", BrowserThemeProvider::COLOR_FRAME_INACTIVE },
-  { "frame_incognito", BrowserThemeProvider::COLOR_FRAME_INCOGNITO },
+  { "frame", ThemeService::COLOR_FRAME },
+  { "frame_inactive", ThemeService::COLOR_FRAME_INACTIVE },
+  { "frame_incognito", ThemeService::COLOR_FRAME_INCOGNITO },
   { "frame_incognito_inactive",
-    BrowserThemeProvider::COLOR_FRAME_INCOGNITO_INACTIVE },
-  { "toolbar", BrowserThemeProvider::COLOR_TOOLBAR },
-  { "tab_text", BrowserThemeProvider::COLOR_TAB_TEXT },
-  { "tab_background_text", BrowserThemeProvider::COLOR_BACKGROUND_TAB_TEXT },
-  { "bookmark_text", BrowserThemeProvider::COLOR_BOOKMARK_TEXT },
-  { "ntp_background", BrowserThemeProvider::COLOR_NTP_BACKGROUND },
-  { "ntp_text", BrowserThemeProvider::COLOR_NTP_TEXT },
-  { "ntp_link", BrowserThemeProvider::COLOR_NTP_LINK },
-  { "ntp_link_underline", BrowserThemeProvider::COLOR_NTP_LINK_UNDERLINE },
-  { "ntp_header", BrowserThemeProvider::COLOR_NTP_HEADER },
-  { "ntp_section", BrowserThemeProvider::COLOR_NTP_SECTION },
-  { "ntp_section_text", BrowserThemeProvider::COLOR_NTP_SECTION_TEXT },
-  { "ntp_section_link", BrowserThemeProvider::COLOR_NTP_SECTION_LINK },
+    ThemeService::COLOR_FRAME_INCOGNITO_INACTIVE },
+  { "toolbar", ThemeService::COLOR_TOOLBAR },
+  { "tab_text", ThemeService::COLOR_TAB_TEXT },
+  { "tab_background_text", ThemeService::COLOR_BACKGROUND_TAB_TEXT },
+  { "bookmark_text", ThemeService::COLOR_BOOKMARK_TEXT },
+  { "ntp_background", ThemeService::COLOR_NTP_BACKGROUND },
+  { "ntp_text", ThemeService::COLOR_NTP_TEXT },
+  { "ntp_link", ThemeService::COLOR_NTP_LINK },
+  { "ntp_link_underline", ThemeService::COLOR_NTP_LINK_UNDERLINE },
+  { "ntp_header", ThemeService::COLOR_NTP_HEADER },
+  { "ntp_section", ThemeService::COLOR_NTP_SECTION },
+  { "ntp_section_text", ThemeService::COLOR_NTP_SECTION_TEXT },
+  { "ntp_section_link", ThemeService::COLOR_NTP_SECTION_LINK },
   { "ntp_section_link_underline",
-    BrowserThemeProvider::COLOR_NTP_SECTION_LINK_UNDERLINE },
-  { "control_background", BrowserThemeProvider::COLOR_CONTROL_BACKGROUND },
-  { "button_background", BrowserThemeProvider::COLOR_BUTTON_BACKGROUND },
+    ThemeService::COLOR_NTP_SECTION_LINK_UNDERLINE },
+  { "control_background", ThemeService::COLOR_CONTROL_BACKGROUND },
+  { "button_background", ThemeService::COLOR_BUTTON_BACKGROUND },
   { NULL, 0 }
 };
 
 // Strings used by themes to identify display properties keys in JSON.
 StringToIntTable kDisplayProperties[] = {
   { "ntp_background_alignment",
-    BrowserThemeProvider::NTP_BACKGROUND_ALIGNMENT },
-  { "ntp_background_repeat", BrowserThemeProvider::NTP_BACKGROUND_TILING },
-  { "ntp_logo_alternate", BrowserThemeProvider::NTP_LOGO_ALTERNATE },
+    ThemeService::NTP_BACKGROUND_ALIGNMENT },
+  { "ntp_background_repeat", ThemeService::NTP_BACKGROUND_TILING },
+  { "ntp_logo_alternate", ThemeService::NTP_LOGO_ALTERNATE },
   { NULL, 0 }
 };
 
 // Strings used by the tiling values in JSON.
 StringToIntTable kTilingStrings[] = {
-  { "no-repeat", BrowserThemeProvider::NO_REPEAT },
-  { "repeat-x", BrowserThemeProvider::REPEAT_X },
-  { "repeat-y", BrowserThemeProvider::REPEAT_Y },
-  { "repeat", BrowserThemeProvider::REPEAT },
+  { "no-repeat", ThemeService::NO_REPEAT },
+  { "repeat-x", ThemeService::REPEAT_X },
+  { "repeat-y", ThemeService::REPEAT_Y },
+  { "repeat", ThemeService::REPEAT },
   { NULL, 0 }
 };
 
@@ -246,14 +254,14 @@ struct IntToIntTable {
 // Mapping used in GenerateFrameImages() to associate frame images with the
 // tint ID that should maybe be applied to it.
 IntToIntTable kFrameTintMap[] = {
-  { PRS_THEME_FRAME, BrowserThemeProvider::TINT_FRAME },
-  { PRS_THEME_FRAME_INACTIVE, BrowserThemeProvider::TINT_FRAME_INACTIVE },
-  { PRS_THEME_FRAME_OVERLAY, BrowserThemeProvider::TINT_FRAME },
+  { PRS_THEME_FRAME, ThemeService::TINT_FRAME },
+  { PRS_THEME_FRAME_INACTIVE, ThemeService::TINT_FRAME_INACTIVE },
+  { PRS_THEME_FRAME_OVERLAY, ThemeService::TINT_FRAME },
   { PRS_THEME_FRAME_OVERLAY_INACTIVE,
-    BrowserThemeProvider::TINT_FRAME_INACTIVE },
-  { PRS_THEME_FRAME_INCOGNITO, BrowserThemeProvider::TINT_FRAME_INCOGNITO },
+    ThemeService::TINT_FRAME_INACTIVE },
+  { PRS_THEME_FRAME_INCOGNITO, ThemeService::TINT_FRAME_INCOGNITO },
   { PRS_THEME_FRAME_INCOGNITO_INACTIVE,
-    BrowserThemeProvider::TINT_FRAME_INCOGNITO_INACTIVE }
+    ThemeService::TINT_FRAME_INCOGNITO_INACTIVE }
 };
 
 // Mapping used in GenerateTabBackgroundImages() to associate what frame image
@@ -296,19 +304,17 @@ RefCountedMemory* ReadFileData(const FilePath& path) {
   return NULL;
 }
 
-// Does error checking for invalid incoming data while trying to read an
-// floating point value.
-bool ValidRealValue(ListValue* tint_list, int index, double* out) {
-  if (tint_list->GetReal(index, out))
-    return true;
-
-  int value = 0;
-  if (tint_list->GetInteger(index, &value)) {
-    *out = value;
-    return true;
+// Shifts a bitmap's HSL values. The caller is responsible for deleting
+// the returned image.
+gfx::Image* CreateHSLShiftedImage(const gfx::Image& image,
+                                  const color_utils::HSL& hsl_shift) {
+  std::vector<const SkBitmap*> bitmaps;
+  for (size_t i = 0; i < image.GetNumberOfSkBitmaps(); ++i) {
+    const SkBitmap* bitmap = image.GetSkBitmapAtIndex(i);
+    bitmaps.push_back(new SkBitmap(SkBitmapOperations::CreateHSLShiftedBitmap(
+        *bitmap, hsl_shift)));
   }
-
-  return false;
+  return new gfx::Image(bitmaps);
 }
 
 }  // namespace
@@ -333,7 +339,7 @@ BrowserThemePack* BrowserThemePack::BuildFromExtension(
   DCHECK(extension);
   DCHECK(extension->is_theme());
 
-  BrowserThemePack* pack = new BrowserThemePack;
+  scoped_refptr<BrowserThemePack> pack = new BrowserThemePack;
   pack->BuildHeader(extension);
   pack->BuildTintsFromJSON(extension->GetThemeTints());
   pack->BuildColorsFromJSON(extension->GetThemeColors());
@@ -346,29 +352,30 @@ BrowserThemePack* BrowserThemePack::BuildFromExtension(
                                 &file_paths);
   pack->BuildSourceImagesArray(file_paths);
 
-  if (!pack->LoadRawBitmapsTo(file_paths, &pack->prepared_images_))
+  if (!pack->LoadRawBitmapsTo(file_paths, &pack->prepared_images_)) {
+    delete pack;
     return NULL;
+  }
 
   pack->GenerateFrameImages(&pack->prepared_images_);
 
-#if !defined(OS_MACOSX)
-  // OSX uses its own special buttons that are PDFs that do odd sorts of vector
-  // graphics tricks. Other platforms use bitmaps and we must pre-tint them.
   pack->GenerateTintedButtons(
-      pack->GetTintInternal(BrowserThemeProvider::TINT_BUTTONS),
+      pack->GetTintInternal(ThemeService::TINT_BUTTONS),
       &pack->prepared_images_);
-#endif
 
   pack->GenerateTabBackgroundImages(&pack->prepared_images_);
 
   // The BrowserThemePack is now in a consistent state.
-  return pack;
+  return pack.release();
 }
 
 // static
 scoped_refptr<BrowserThemePack> BrowserThemePack::BuildFromDataPack(
     FilePath path, const std::string& expected_id) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  // Allow IO on UI thread due to deep-seated theme design issues.
+  // (see http://crbug.com/80206)
+  base::ThreadRestrictions::ScopedAllowIO allow_io;
   scoped_refptr<BrowserThemePack> pack(new BrowserThemePack);
   pack->data_pack_.reset(new ui::DataPack);
 
@@ -448,7 +455,7 @@ bool BrowserThemePack::WriteToDisk(FilePath path) const {
   RepackImages(prepared_images_, &reencoded_images);
   AddRawImagesTo(reencoded_images, &resources);
 
-  return ui::DataPack::WritePack(path, resources);
+  return ui::DataPack::WritePack(path, resources, ui::DataPack::BINARY);
 }
 
 bool BrowserThemePack::GetTint(int id, color_utils::HSL* hsl) const {
@@ -493,6 +500,19 @@ bool BrowserThemePack::GetDisplayProperty(int id, int* result) const {
 }
 
 SkBitmap* BrowserThemePack::GetBitmapNamed(int idr_id) const {
+  const gfx::Image* image = GetImageNamed(idr_id);
+  if (!image)
+    return NULL;
+
+  const SkBitmap* bitmap = *image;
+
+  // TODO(sail): This cast should be removed. Currently we use this const_cast
+  // to avoid changing the BrowserThemePack::GetBitmapNamed API. Once we
+  // switch to using gfx::Image everywhere this can be removed.
+  return const_cast<SkBitmap*>(bitmap);
+}
+
+const gfx::Image* BrowserThemePack::GetImageNamed(int idr_id) const {
   int prs_id = GetPersistentIDByIDR(idr_id);
   if (prs_id == -1)
     return NULL;
@@ -527,7 +547,7 @@ SkBitmap* BrowserThemePack::GetBitmapNamed(int idr_id) const {
       return NULL;
     }
 
-    SkBitmap* ret = new SkBitmap(bitmap);
+    gfx::Image* ret = new gfx::Image(new SkBitmap(bitmap));
     loaded_images_[prs_id] = ret;
 
     return ret;
@@ -619,9 +639,9 @@ void BrowserThemePack::BuildTintsFromJSON(DictionaryValue* tints_value) {
         (tint_list->GetSize() == 3)) {
       color_utils::HSL hsl = { -1, -1, -1 };
 
-      if (ValidRealValue(tint_list, 0, &hsl.h) &&
-          ValidRealValue(tint_list, 1, &hsl.s) &&
-          ValidRealValue(tint_list, 2, &hsl.l)) {
+      if (tint_list->GetDouble(0, &hsl.h) &&
+          tint_list->GetDouble(1, &hsl.s) &&
+          tint_list->GetDouble(2, &hsl.l)) {
         int id = GetIntForString(*iter, kTintTable);
         if (id != -1) {
           temp_tints[id] = hsl;
@@ -680,7 +700,7 @@ void BrowserThemePack::ReadColorsFromJSON(
         if (color_list->GetSize() == 4) {
           double alpha;
           int alpha_int;
-          if (color_list->GetReal(3, &alpha)) {
+          if (color_list->GetDouble(3, &alpha)) {
             color = SkColorSetARGB(static_cast<int>(alpha * 255), r, g, b);
           } else if (color_list->GetInteger(3, &alpha_int) &&
                      (alpha_int == 0 || alpha_int == 1)) {
@@ -705,56 +725,56 @@ void BrowserThemePack::ReadColorsFromJSON(
 void BrowserThemePack::GenerateMissingColors(
     std::map<int, SkColor>* colors) {
   // Generate link colors, if missing. (See GetColor()).
-  if (!colors->count(BrowserThemeProvider::COLOR_NTP_HEADER) &&
-      colors->count(BrowserThemeProvider::COLOR_NTP_SECTION)) {
-    (*colors)[BrowserThemeProvider::COLOR_NTP_HEADER] =
-        (*colors)[BrowserThemeProvider::COLOR_NTP_SECTION];
+  if (!colors->count(ThemeService::COLOR_NTP_HEADER) &&
+      colors->count(ThemeService::COLOR_NTP_SECTION)) {
+    (*colors)[ThemeService::COLOR_NTP_HEADER] =
+        (*colors)[ThemeService::COLOR_NTP_SECTION];
   }
 
-  if (!colors->count(BrowserThemeProvider::COLOR_NTP_SECTION_LINK_UNDERLINE) &&
-      colors->count(BrowserThemeProvider::COLOR_NTP_SECTION_LINK)) {
+  if (!colors->count(ThemeService::COLOR_NTP_SECTION_LINK_UNDERLINE) &&
+      colors->count(ThemeService::COLOR_NTP_SECTION_LINK)) {
     SkColor color_section_link =
-        (*colors)[BrowserThemeProvider::COLOR_NTP_SECTION_LINK];
-    (*colors)[BrowserThemeProvider::COLOR_NTP_SECTION_LINK_UNDERLINE] =
+        (*colors)[ThemeService::COLOR_NTP_SECTION_LINK];
+    (*colors)[ThemeService::COLOR_NTP_SECTION_LINK_UNDERLINE] =
         SkColorSetA(color_section_link, SkColorGetA(color_section_link) / 3);
   }
 
-  if (!colors->count(BrowserThemeProvider::COLOR_NTP_LINK_UNDERLINE) &&
-      colors->count(BrowserThemeProvider::COLOR_NTP_LINK)) {
-    SkColor color_link = (*colors)[BrowserThemeProvider::COLOR_NTP_LINK];
-    (*colors)[BrowserThemeProvider::COLOR_NTP_LINK_UNDERLINE] =
+  if (!colors->count(ThemeService::COLOR_NTP_LINK_UNDERLINE) &&
+      colors->count(ThemeService::COLOR_NTP_LINK)) {
+    SkColor color_link = (*colors)[ThemeService::COLOR_NTP_LINK];
+    (*colors)[ThemeService::COLOR_NTP_LINK_UNDERLINE] =
         SkColorSetA(color_link, SkColorGetA(color_link) / 3);
   }
 
   // Generate frame colors, if missing. (See GenerateFrameColors()).
   SkColor frame;
   std::map<int, SkColor>::const_iterator it =
-      colors->find(BrowserThemeProvider::COLOR_FRAME);
+      colors->find(ThemeService::COLOR_FRAME);
   if (it != colors->end()) {
     frame = it->second;
   } else {
-    frame = BrowserThemeProvider::GetDefaultColor(
-        BrowserThemeProvider::COLOR_FRAME);
+    frame = ThemeService::GetDefaultColor(
+        ThemeService::COLOR_FRAME);
   }
 
-  if (!colors->count(BrowserThemeProvider::COLOR_FRAME)) {
-    (*colors)[BrowserThemeProvider::COLOR_FRAME] =
-        HSLShift(frame, GetTintInternal(BrowserThemeProvider::TINT_FRAME));
+  if (!colors->count(ThemeService::COLOR_FRAME)) {
+    (*colors)[ThemeService::COLOR_FRAME] =
+        HSLShift(frame, GetTintInternal(ThemeService::TINT_FRAME));
   }
-  if (!colors->count(BrowserThemeProvider::COLOR_FRAME_INACTIVE)) {
-    (*colors)[BrowserThemeProvider::COLOR_FRAME_INACTIVE] =
+  if (!colors->count(ThemeService::COLOR_FRAME_INACTIVE)) {
+    (*colors)[ThemeService::COLOR_FRAME_INACTIVE] =
         HSLShift(frame, GetTintInternal(
-            BrowserThemeProvider::TINT_FRAME_INACTIVE));
+            ThemeService::TINT_FRAME_INACTIVE));
   }
-  if (!colors->count(BrowserThemeProvider::COLOR_FRAME_INCOGNITO)) {
-    (*colors)[BrowserThemeProvider::COLOR_FRAME_INCOGNITO] =
+  if (!colors->count(ThemeService::COLOR_FRAME_INCOGNITO)) {
+    (*colors)[ThemeService::COLOR_FRAME_INCOGNITO] =
         HSLShift(frame, GetTintInternal(
-            BrowserThemeProvider::TINT_FRAME_INCOGNITO));
+            ThemeService::TINT_FRAME_INCOGNITO));
   }
-  if (!colors->count(BrowserThemeProvider::COLOR_FRAME_INCOGNITO_INACTIVE)) {
-    (*colors)[BrowserThemeProvider::COLOR_FRAME_INCOGNITO_INACTIVE] =
+  if (!colors->count(ThemeService::COLOR_FRAME_INCOGNITO_INACTIVE)) {
+    (*colors)[ThemeService::COLOR_FRAME_INCOGNITO_INACTIVE] =
         HSLShift(frame, GetTintInternal(
-            BrowserThemeProvider::TINT_FRAME_INCOGNITO_INACTIVE));
+            ThemeService::TINT_FRAME_INCOGNITO_INACTIVE));
   }
 }
 
@@ -775,26 +795,26 @@ void BrowserThemePack::BuildDisplayPropertiesFromJSON(
        iter != display_properties_value->end_keys(); ++iter) {
     int property_id = GetIntForString(*iter, kDisplayProperties);
     switch (property_id) {
-      case BrowserThemeProvider::NTP_BACKGROUND_ALIGNMENT: {
+      case ThemeService::NTP_BACKGROUND_ALIGNMENT: {
         std::string val;
         if (display_properties_value->GetString(*iter, &val)) {
-          temp_properties[BrowserThemeProvider::NTP_BACKGROUND_ALIGNMENT] =
-              BrowserThemeProvider::StringToAlignment(val);
+          temp_properties[ThemeService::NTP_BACKGROUND_ALIGNMENT] =
+              ThemeService::StringToAlignment(val);
         }
         break;
       }
-      case BrowserThemeProvider::NTP_BACKGROUND_TILING: {
+      case ThemeService::NTP_BACKGROUND_TILING: {
         std::string val;
         if (display_properties_value->GetString(*iter, &val)) {
-          temp_properties[BrowserThemeProvider::NTP_BACKGROUND_TILING] =
+          temp_properties[ThemeService::NTP_BACKGROUND_TILING] =
               GetIntForString(val, kTilingStrings);
         }
         break;
       }
-      case BrowserThemeProvider::NTP_LOGO_ALTERNATE: {
+      case ThemeService::NTP_LOGO_ALTERNATE: {
         int val = 0;
         if (display_properties_value->GetInteger(*iter, &val))
-          temp_properties[BrowserThemeProvider::NTP_LOGO_ALTERNATE] = val;
+          temp_properties[ThemeService::NTP_LOGO_ALTERNATE] = val;
         break;
       }
     }
@@ -812,7 +832,7 @@ void BrowserThemePack::BuildDisplayPropertiesFromJSON(
 
 void BrowserThemePack::ParseImageNamesFromJSON(
     DictionaryValue* images_value,
-    FilePath images_path,
+    const FilePath& images_path,
     FilePathMap* file_paths) const {
   if (!images_value)
     return;
@@ -874,7 +894,7 @@ bool BrowserThemePack::LoadRawBitmapsTo(
       SkBitmap bitmap;
       if (gfx::PNGCodec::Decode(raw_data->front(), raw_data->size(),
                                 &bitmap)) {
-        (*raw_bitmaps)[it->first] = new SkBitmap(bitmap);
+        (*raw_bitmaps)[it->first] = new gfx::Image(new SkBitmap(bitmap));
       } else {
         NOTREACHED() << "Unable to decode theme image resource " << it->first;
       }
@@ -893,7 +913,7 @@ void BrowserThemePack::GenerateFrameImages(ImageCache* bitmaps) const {
 
   for (size_t i = 0; i < arraysize(kFrameTintMap); ++i) {
     int prs_id = kFrameTintMap[i].key;
-    scoped_ptr<SkBitmap> frame;
+    const gfx::Image* frame = NULL;
     // If there's no frame image provided for the specified id, then load
     // the default provided frame. If that's not provided, skip this whole
     // thing and just use the default images.
@@ -914,25 +934,24 @@ void BrowserThemePack::GenerateFrameImages(ImageCache* bitmaps) const {
     }
 
     if (bitmaps->count(prs_id)) {
-      frame.reset(new SkBitmap(*(*bitmaps)[prs_id]));
+      frame = (*bitmaps)[prs_id];
     } else if (prs_base_id != prs_id && bitmaps->count(prs_base_id)) {
-      frame.reset(new SkBitmap(*(*bitmaps)[prs_base_id]));
+      frame = (*bitmaps)[prs_base_id];
     } else if (prs_base_id == PRS_THEME_FRAME_OVERLAY &&
                bitmaps->count(PRS_THEME_FRAME)) {
       // If there is no theme overlay, don't tint the default frame,
       // because it will overwrite the custom frame image when we cache and
       // reload from disk.
-      frame.reset(NULL);
+      frame = NULL;
     } else {
       // If the theme doesn't specify an image, then apply the tint to
       // the default frame.
-      frame.reset(new SkBitmap(*rb.GetBitmapNamed(IDR_THEME_FRAME)));
+      frame = &rb.GetImageNamed(IDR_THEME_FRAME);
     }
 
-    if (frame.get()) {
-      temp_output[prs_id] = new SkBitmap(
-          SkBitmapOperations::CreateHSLShiftedBitmap(
-              *frame, GetTintInternal(kFrameTintMap[i].value)));
+    if (frame) {
+      temp_output[prs_id] = CreateHSLShiftedImage(
+          *frame, GetTintInternal(kFrameTintMap[i].value));
     }
   }
 
@@ -945,18 +964,18 @@ void BrowserThemePack::GenerateTintedButtons(
   if (button_tint.h != -1 || button_tint.s != -1 || button_tint.l != -1) {
     ResourceBundle& rb = ResourceBundle::GetSharedInstance();
     const std::set<int>& idr_ids =
-        BrowserThemeProvider::GetTintableToolbarButtons();
+        ThemeService::GetTintableToolbarButtons();
     for (std::set<int>::const_iterator it = idr_ids.begin();
          it != idr_ids.end(); ++it) {
       int prs_id = GetPersistentIDByIDR(*it);
       DCHECK(prs_id > 0);
 
       // Fetch the image by IDR...
-      scoped_ptr<SkBitmap> button(new SkBitmap(*rb.GetBitmapNamed(*it)));
+      gfx::Image& button = rb.GetImageNamed(*it);
 
       // but save a version with the persistent ID.
-      (*processed_bitmaps)[prs_id] = new SkBitmap(
-          SkBitmapOperations::CreateHSLShiftedBitmap(*button, button_tint));
+      (*processed_bitmaps)[prs_id] =
+          CreateHSLShiftedImage(button, button_tint);
     }
   }
 }
@@ -971,24 +990,30 @@ void BrowserThemePack::GenerateTabBackgroundImages(ImageCache* bitmaps) const {
     // with a PRS_THEME_FRAME.
     ImageCache::const_iterator it = bitmaps->find(prs_base_id);
     if (it != bitmaps->end()) {
-      SkBitmap bg_tint = SkBitmapOperations::CreateHSLShiftedBitmap(
-          *(it->second), GetTintInternal(
-              BrowserThemeProvider::TINT_BACKGROUND_TAB));
-      int vertical_offset = bitmaps->count(prs_id)
-                            ? kRestoredTabVerticalOffset : 0;
-      SkBitmap* bg_tab = new SkBitmap(SkBitmapOperations::CreateTiledBitmap(
-          bg_tint, 0, vertical_offset, bg_tint.width(), bg_tint.height()));
+      const gfx::Image& image_to_tint = *(it->second);
+      std::vector<const SkBitmap*> tinted_bitmaps;
+      for (size_t j = 0; j < image_to_tint.GetNumberOfSkBitmaps(); ++j) {
+        SkBitmap bg_tint = SkBitmapOperations::CreateHSLShiftedBitmap(
+            *image_to_tint.GetSkBitmapAtIndex(j), GetTintInternal(
+                ThemeService::TINT_BACKGROUND_TAB));
+        int vertical_offset = bitmaps->count(prs_id)
+                              ? kRestoredTabVerticalOffset : 0;
+        SkBitmap* bg_tab = new SkBitmap(SkBitmapOperations::CreateTiledBitmap(
+            bg_tint, 0, vertical_offset, bg_tint.width(), bg_tint.height()));
 
-      // If they've provided a custom image, overlay it.
-      ImageCache::const_iterator overlay_it = bitmaps->find(prs_id);
-      if (overlay_it != bitmaps->end()) {
-        SkBitmap* overlay = overlay_it->second;
-        SkCanvas canvas(*bg_tab);
-        for (int x = 0; x < bg_tab->width(); x += overlay->width())
-          canvas.drawBitmap(*overlay, static_cast<SkScalar>(x), 0, NULL);
+        // If they've provided a custom image, overlay it.
+        ImageCache::const_iterator overlay_it = bitmaps->find(prs_id);
+        if (overlay_it != bitmaps->end()) {
+          const SkBitmap* overlay = *(overlay_it->second);
+          SkCanvas canvas(*bg_tab);
+          for (int x = 0; x < bg_tab->width(); x += overlay->width())
+            canvas.drawBitmap(*overlay, static_cast<SkScalar>(x), 0, NULL);
+        }
+        tinted_bitmaps.push_back(bg_tab);
       }
 
-      temp_output[prs_id] = bg_tab;
+      gfx::Image* tinted_image = new gfx::Image(tinted_bitmaps);
+      temp_output[prs_id] = tinted_image;
     }
   }
 
@@ -1012,7 +1037,6 @@ void BrowserThemePack::RepackImages(const ImageCache& images,
 
 void BrowserThemePack::MergeImageCaches(
     const ImageCache& source, ImageCache* destination) const {
-
   for (ImageCache::const_iterator it = source.begin(); it != source.end();
        ++it) {
     ImageCache::const_iterator bitmap_it = destination->find(it->first);
@@ -1046,5 +1070,5 @@ color_utils::HSL BrowserThemePack::GetTintInternal(int id) const {
     }
   }
 
-  return BrowserThemeProvider::GetDefaultTint(id);
+  return ThemeService::GetDefaultTint(id);
 }
