@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,18 +10,24 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
-#include "base/ref_counted.h"
-#include "gfx/color_utils.h"
-#include "chrome/browser/browser_thread.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/message_loop_helpers.h"
 #include "chrome/common/extensions/extension.h"
+#include "content/public/browser/browser_thread.h"
+#include "ui/gfx/color_utils.h"
 
-class DictionaryValue;
 class FilePath;
 class RefCountedMemory;
 namespace ui {
 class DataPack;
 }
+namespace gfx {
+class Image;
+}
+namespace base {
+class DictionaryValue;
+}
 
 // An optimized representation of a theme, backed by a mmapped DataPack.
 //
@@ -41,7 +47,7 @@ class DataPack;
 // common case, they are backed by mmapped data and the unmmapping operation
 // will trip our IO on the UI thread detector.
 class BrowserThemePack : public base::RefCountedThreadSafe<
-    BrowserThemePack, BrowserThread::DeleteOnFileThread> {
+    BrowserThemePack, content::BrowserThread::DeleteOnFileThread> {
  public:
   // Builds the theme pack from all data from |extension|. This is often done
   // on a separate thread as it takes so long. This can fail and return NULL in
@@ -75,6 +81,9 @@ class BrowserThemePack : public base::RefCountedThreadSafe<
   // for making layout decisions in the interface.
   SkBitmap* GetBitmapNamed(int id) const;
 
+  // Returns an image if we have a custom image for |id|, otherwise NULL.
+  const gfx::Image* GetImageNamed(int id) const;
+
   // Returns the raw PNG encoded data for IDR_THEME_NTP_*. This method is only
   // supposed to work for the NTP attribution and background resources.
   RefCountedMemory* GetRawData(int id) const;
@@ -83,20 +92,21 @@ class BrowserThemePack : public base::RefCountedThreadSafe<
   bool HasCustomImage(int id) const;
 
  private:
-  friend struct BrowserThread::DeleteOnThread<BrowserThread::FILE>;
-  friend class DeleteTask<BrowserThemePack>;
+  friend struct content::BrowserThread::DeleteOnThread<
+      content::BrowserThread::FILE>;
+  friend class base::DeleteHelper<BrowserThemePack>;
   friend class BrowserThemePackTest;
 
   // Cached images. We cache all retrieved and generated bitmaps and keep
   // track of the pointers. We own these and will delete them when we're done
   // using them.
-  typedef std::map<int, SkBitmap*> ImageCache;
+  typedef std::map<int, const gfx::Image*> ImageCache;
 
   // The raw PNG memory associated with a certain id.
   typedef std::map<int, scoped_refptr<RefCountedMemory> > RawImages;
 
-  // The type passed to base::DataPack::WritePack.
-  typedef std::map<uint32, base::StringPiece> RawDataForWriting;
+  // The type passed to ui::DataPack::WritePack.
+  typedef std::map<uint16, base::StringPiece> RawDataForWriting;
 
   // An association between an id and the FilePath that has the image data.
   typedef std::map<int, FilePath> FilePathMap;
@@ -111,23 +121,23 @@ class BrowserThemePack : public base::RefCountedThreadSafe<
 
   // Transforms the JSON tint values into their final versions in the |tints_|
   // array.
-  void BuildTintsFromJSON(DictionaryValue* tints_value);
+  void BuildTintsFromJSON(base::DictionaryValue* tints_value);
 
   // Transforms the JSON color values into their final versions in the
   // |colors_| array and also fills in unspecified colors based on tint values.
-  void BuildColorsFromJSON(DictionaryValue* color_value);
+  void BuildColorsFromJSON(base::DictionaryValue* color_value);
 
   // Implementation details of BuildColorsFromJSON().
-  void ReadColorsFromJSON(DictionaryValue* colors_value,
+  void ReadColorsFromJSON(base::DictionaryValue* colors_value,
                           std::map<int, SkColor>* temp_colors);
   void GenerateMissingColors(std::map<int, SkColor>* temp_colors);
 
   // Transforms the JSON display properties into |display_properties_|.
-  void BuildDisplayPropertiesFromJSON(DictionaryValue* display_value);
+  void BuildDisplayPropertiesFromJSON(base::DictionaryValue* display_value);
 
   // Parses the image names out of an extension.
-  void ParseImageNamesFromJSON(DictionaryValue* images_value,
-                               FilePath images_path,
+  void ParseImageNamesFromJSON(base::DictionaryValue* images_value,
+                               const FilePath& images_path,
                                FilePathMap* file_paths) const;
 
   // Creates the data for |source_images_| from |file_paths|.
