@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,24 +6,21 @@
 
 #include "base/metrics/histogram.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/google/google_util.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/translate/translate_infobar_delegate.h"
+#include "chrome/common/url_constants.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/generated_resources.h"
 #include "grit/locale_settings.h"
 #include "ui/base/l10n/l10n_util.h"
 
-namespace {
-
-const char kAboutGoogleTranslateUrl[] =
-#if defined(OS_CHROMEOS)
-    "http://www.google.com/support/chromeos/bin/answer.py?answer=173424";
-#else
-    "http://www.google.com/support/chrome/bin/answer.py?answer=173424";
-#endif
-
-}  // namespace
+using content::NavigationEntry;
+using content::OpenURLParams;
+using content::Referrer;
+using content::WebContents;
 
 OptionsMenuModel::OptionsMenuModel(
     TranslateInfoBarDelegate* translate_delegate)
@@ -36,7 +33,8 @@ OptionsMenuModel::OptionsMenuModel(
 
   // Populate the menu.
   // Incognito mode does not get any preferences related items.
-  if (!translate_delegate->tab_contents()->profile()->IsOffTheRecord()) {
+  if (!translate_delegate->owner()->web_contents()->
+      GetBrowserContext()->IsOffTheRecord()) {
     AddCheckItem(IDC_TRANSLATE_OPTIONS_ALWAYS,
         l10n_util::GetStringFUTF16(IDS_TRANSLATE_INFOBAR_OPTIONS_ALWAYS,
             original_language, target_language));
@@ -87,6 +85,18 @@ bool OptionsMenuModel::IsCommandIdEnabled(int command_id) const {
       return (!translate_infobar_delegate_->IsLanguageBlacklisted() &&
           !translate_infobar_delegate_->IsSiteBlacklisted());
 
+    case IDC_TRANSLATE_REPORT_BAD_LANGUAGE_DETECTION : {
+      // Until we have a secure URL for reporting language detection errors,
+      // we don't report errors that happened on secure URLs.
+      DCHECK(translate_infobar_delegate_ != NULL);
+      DCHECK(translate_infobar_delegate_->owner() != NULL);
+      DCHECK(translate_infobar_delegate_->owner()->web_contents() != NULL);
+      NavigationEntry* entry = translate_infobar_delegate_->owner()->
+          web_contents()->GetController().GetActiveEntry();
+      // Delegate and tab contents should never be NULL, but active entry
+      // can be NULL when running tests. We want to return false if NULL.
+      return (entry != NULL) && !entry->GetURL().SchemeIsSecure();
+    }
     default:
       break;
   }
@@ -120,12 +130,13 @@ void OptionsMenuModel::ExecuteCommand(int command_id) {
       break;
 
     case IDC_TRANSLATE_OPTIONS_ABOUT: {
-      TabContents* tab_contents = translate_infobar_delegate_->tab_contents();
-      if (tab_contents) {
-        GURL about_url = google_util::AppendGoogleLocaleParam(
-            GURL(kAboutGoogleTranslateUrl));
-        tab_contents->OpenURL(
-            about_url, GURL(), NEW_FOREGROUND_TAB, PageTransition::LINK);
+      WebContents* web_contents =
+          translate_infobar_delegate_->owner()->web_contents();
+      if (web_contents) {
+        OpenURLParams params(
+            GURL(chrome::kAboutGoogleTranslateURL), Referrer(),
+            NEW_FOREGROUND_TAB, content::PAGE_TRANSITION_LINK, false);
+        web_contents->OpenURL(params);
       }
       break;
     }
