@@ -8,64 +8,83 @@
 
 #include "base/metrics/histogram.h"
 #include "chrome/browser/browser_process.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/translate/translate_infobar_view.h"
 #include "chrome/browser/translate/translate_manager.h"
+#include "chrome/browser/translate/translate_tab_helper.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/chrome_constants.h"
+#include "content/public/browser/navigation_details.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/generated_resources.h"
-#include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
+using content::NavigationEntry;
+
+// static
+const size_t TranslateInfoBarDelegate::kNoIndex = static_cast<size_t>(-1);
+
 // static
 TranslateInfoBarDelegate* TranslateInfoBarDelegate::CreateDelegate(
     Type type,
-    TabContents* tab_contents,
+    InfoBarTabHelper* infobar_helper,
+    PrefService* prefs,
     const std::string& original_language,
     const std::string& target_language) {
   DCHECK_NE(TRANSLATION_ERROR, type);
+  // These must be validated by our callers.
+  DCHECK(TranslateManager::IsSupportedLanguage(target_language));
   // The original language can only be "unknown" for the "translating"
   // infobar, which is the case when the user started a translation from the
   // context menu.
-  DCHECK(type == TRANSLATING ||
-      original_language != chrome::kUnknownLanguageCode);
-  if ((original_language != chrome::kUnknownLanguageCode &&
-          !TranslateManager::IsSupportedLanguage(original_language)) ||
-      !TranslateManager::IsSupportedLanguage(target_language))
-    return NULL;
+  DCHECK(TranslateManager::IsSupportedLanguage(original_language) ||
+         ((type == TRANSLATING) &&
+          (original_language == chrome::kUnknownLanguageCode)));
   TranslateInfoBarDelegate* delegate =
-      new TranslateInfoBarDelegate(type, TranslateErrors::NONE, tab_contents,
-                                   original_language, target_language);
-  DCHECK_NE(-1, delegate->target_language_index());
+      new TranslateInfoBarDelegate(type,
+                                   TranslateErrors::NONE,
+                                   infobar_helper,
+                                   prefs,
+                                   original_language,
+                                   target_language);
+  DCHECK_NE(kNoIndex, delegate->target_language_index());
   return delegate;
 }
 
 TranslateInfoBarDelegate* TranslateInfoBarDelegate::CreateErrorDelegate(
     TranslateErrors::Type error,
-    TabContents* tab_contents,
+    InfoBarTabHelper* infobar_helper,
+    PrefService* prefs,
     const std::string& original_language,
     const std::string& target_language) {
-  return new TranslateInfoBarDelegate(TRANSLATION_ERROR, error, tab_contents,
-                                      original_language, target_language);
+  return new TranslateInfoBarDelegate(TRANSLATION_ERROR,
+                                      error,
+                                      infobar_helper,
+                                      prefs,
+                                      original_language,
+                                      target_language);
 }
 
 TranslateInfoBarDelegate::~TranslateInfoBarDelegate() {
 }
 
-std::string TranslateInfoBarDelegate::GetLanguageCodeAt(int index) const {
-  DCHECK(index >= 0 && index < GetLanguageCount());
+std::string TranslateInfoBarDelegate::GetLanguageCodeAt(size_t index) const {
+  DCHECK_LT(index, GetLanguageCount());
   return languages_[index].first;
 }
 
 string16 TranslateInfoBarDelegate::GetLanguageDisplayableNameAt(
-    int index) const {
-  DCHECK(index >= 0 && index < GetLanguageCount());
+    size_t index) const {
+  DCHECK_LT(index, GetLanguageCount());
   return languages_[index].second;
 }
 
 std::string TranslateInfoBarDelegate::GetOriginalLanguageCode() const {
-  return (original_language_index() == -1) ?
+  return (original_language_index() == kNoIndex) ?
       chrome::kUnknownLanguageCode :
       GetLanguageCodeAt(original_language_index());
 }
@@ -74,7 +93,7 @@ std::string TranslateInfoBarDelegate::GetTargetLanguageCode() const {
   return GetLanguageCodeAt(target_language_index());
 }
 
-void TranslateInfoBarDelegate::SetOriginalLanguage(int language_index) {
+void TranslateInfoBarDelegate::SetOriginalLanguage(size_t language_index) {
   DCHECK_LT(language_index, GetLanguageCount());
   original_language_index_ = language_index;
   if (infobar_view_)
@@ -83,8 +102,9 @@ void TranslateInfoBarDelegate::SetOriginalLanguage(int language_index) {
     Translate();
 }
 
-void TranslateInfoBarDelegate::SetTargetLanguage(int language_index) {
+void TranslateInfoBarDelegate::SetTargetLanguage(size_t language_index) {
   DCHECK_LT(language_index, GetLanguageCount());
+  DCHECK_GE(language_index, 0U);
   target_language_index_ = language_index;
   if (infobar_view_)
     infobar_view_->TargetLanguageChanged();
@@ -94,28 +114,29 @@ void TranslateInfoBarDelegate::SetTargetLanguage(int language_index) {
 
 void TranslateInfoBarDelegate::Translate() {
   const std::string& original_language_code = GetOriginalLanguageCode();
-  if (!tab_contents()->profile()->IsOffTheRecord()) {
+  if (!owner()->web_contents()->GetBrowserContext()->IsOffTheRecord()) {
     prefs_.ResetTranslationDeniedCount(original_language_code);
     prefs_.IncrementTranslationAcceptedCount(original_language_code);
   }
 
-  TranslateManager::GetInstance()->TranslatePage(tab_contents_,
+  TranslateManager::GetInstance()->TranslatePage(owner()->web_contents(),
       GetLanguageCodeAt(original_language_index()),
       GetLanguageCodeAt(target_language_index()));
 }
 
 void TranslateInfoBarDelegate::RevertTranslation() {
-  TranslateManager::GetInstance()->RevertTranslation(tab_contents_);
-  tab_contents_->RemoveInfoBar(this);
+  TranslateManager::GetInstance()->RevertTranslation(owner()->web_contents());
+  RemoveSelf();
 }
 
 void TranslateInfoBarDelegate::ReportLanguageDetectionError() {
-  TranslateManager::GetInstance()->ReportLanguageDetectionError(tab_contents_);
+  TranslateManager::GetInstance()->
+      ReportLanguageDetectionError(owner()->web_contents());
 }
 
 void TranslateInfoBarDelegate::TranslationDeclined() {
   const std::string& original_language_code = GetOriginalLanguageCode();
-  if (!tab_contents()->profile()->IsOffTheRecord()) {
+  if (!owner()->web_contents()->GetBrowserContext()->IsOffTheRecord()) {
     prefs_.ResetTranslationAcceptedCount(original_language_code);
     prefs_.IncrementTranslationDeniedCount(original_language_code);
   }
@@ -125,7 +146,9 @@ void TranslateInfoBarDelegate::TranslationDeclined() {
   // translations when getting a LANGUAGE_DETERMINED from the page, which
   // happens when a load stops. That could happen multiple times, including
   // after the user already declined the translation.)
-  tab_contents_->language_state().set_translation_declined(true);
+  TranslateTabHelper* helper = TabContentsWrapper::GetCurrentWrapperForContents(
+      owner()->web_contents())->translate_tab_helper();
+  helper->language_state().set_translation_declined(true);
 }
 
 bool TranslateInfoBarDelegate::IsLanguageBlacklisted() {
@@ -138,7 +161,7 @@ void TranslateInfoBarDelegate::ToggleLanguageBlacklist() {
     prefs_.RemoveLanguageFromBlacklist(original_lang);
   } else {
     prefs_.BlacklistLanguage(original_lang);
-    tab_contents_->RemoveInfoBar(this);
+    RemoveSelf();
   }
 }
 
@@ -156,7 +179,7 @@ void TranslateInfoBarDelegate::ToggleSiteBlacklist() {
     prefs_.RemoveSiteFromBlacklist(host);
   } else {
     prefs_.BlacklistSite(host);
-    tab_contents_->RemoveInfoBar(this);
+    RemoveSelf();
   }
 }
 
@@ -186,7 +209,7 @@ void TranslateInfoBarDelegate::NeverTranslatePageLanguage() {
   std::string original_lang = GetOriginalLanguageCode();
   DCHECK(!prefs_.IsLanguageBlacklisted(original_lang));
   prefs_.BlacklistLanguage(original_lang);
-  tab_contents_->RemoveInfoBar(this);
+  RemoveSelf();
 }
 
 string16 TranslateInfoBarDelegate::GetMessageInfoBarText() {
@@ -240,7 +263,7 @@ void TranslateInfoBarDelegate::MessageInfoBarButtonPressed() {
     return;
   }
   // This is the "Try again..." case.
-  TranslateManager::GetInstance()->TranslatePage(tab_contents_,
+  TranslateManager::GetInstance()->TranslatePage(owner()->web_contents(),
       GetOriginalLanguageCode(), GetTargetLanguageCode());
 }
 
@@ -250,13 +273,13 @@ bool TranslateInfoBarDelegate::ShouldShowMessageInfoBarButton() {
 
 bool TranslateInfoBarDelegate::ShouldShowNeverTranslateButton() {
   DCHECK_EQ(BEFORE_TRANSLATE, type_);
-  return !tab_contents()->profile()->IsOffTheRecord() &&
+  return !owner()->web_contents()->GetBrowserContext()->IsOffTheRecord() &&
       (prefs_.GetTranslationDeniedCount(GetOriginalLanguageCode()) >= 3);
 }
 
 bool TranslateInfoBarDelegate::ShouldShowAlwaysTranslateButton() {
   DCHECK_EQ(BEFORE_TRANSLATE, type_);
-  return !tab_contents()->profile()->IsOffTheRecord() &&
+  return !owner()->web_contents()->GetBrowserContext()->IsOffTheRecord() &&
       (prefs_.GetTranslationAcceptedCount(GetOriginalLanguageCode()) >= 3);
 }
 
@@ -299,19 +322,19 @@ void TranslateInfoBarDelegate::GetAfterTranslateStrings(
 TranslateInfoBarDelegate::TranslateInfoBarDelegate(
     Type type,
     TranslateErrors::Type error,
-    TabContents* tab_contents,
+    InfoBarTabHelper* infobar_helper,
+    PrefService* prefs,
     const std::string& original_language,
     const std::string& target_language)
-    : InfoBarDelegate(tab_contents),
+    : InfoBarDelegate(infobar_helper),
       type_(type),
       background_animation_(NONE),
-      tab_contents_(tab_contents),
-      original_language_index_(-1),
-      initial_original_language_index_(-1),
-      target_language_index_(-1),
+      original_language_index_(kNoIndex),
+      initial_original_language_index_(kNoIndex),
+      target_language_index_(kNoIndex),
       error_(error),
       infobar_view_(NULL),
-      prefs_(tab_contents_->profile()->GetPrefs()) {
+      prefs_(prefs) {
   DCHECK_NE((type_ == TRANSLATION_ERROR), (error == TranslateErrors::NONE));
 
   std::vector<std::string> language_codes;
@@ -341,6 +364,17 @@ TranslateInfoBarDelegate::TranslateInfoBarDelegate(
     if (language_code == target_language)
       target_language_index_ = iter - languages_.begin();
   }
+  DCHECK_NE(kNoIndex, target_language_index_);
+}
+
+bool TranslateInfoBarDelegate::ShouldExpire(
+    const content::LoadCommittedDetails& details) const {
+  // Note: we allow closing this infobar even if the main frame navigation
+  // was programmatic and not initiated by the user - crbug.com/70261 .
+  if (!details.is_navigation_to_different_page() && !details.is_main_frame)
+    return false;
+
+  return InfoBarDelegate::ShouldExpireInternal(details);
 }
 
 void TranslateInfoBarDelegate::InfoBarDismissed() {
@@ -352,17 +386,13 @@ void TranslateInfoBarDelegate::InfoBarDismissed() {
   UMA_HISTOGRAM_COUNTS("Translate.DeclineTranslateCloseInfobar", 1);
 }
 
-void TranslateInfoBarDelegate::InfoBarClosed() {
-  delete this;
-}
-
-SkBitmap* TranslateInfoBarDelegate::GetIcon() const {
-  return ResourceBundle::GetSharedInstance().GetBitmapNamed(
+gfx::Image* TranslateInfoBarDelegate::GetIcon() const {
+  return &ResourceBundle::GetSharedInstance().GetNativeImageNamed(
       IDR_INFOBAR_TRANSLATE);
 }
 
 InfoBarDelegate::Type TranslateInfoBarDelegate::GetInfoBarType() const {
-  return InfoBarDelegate::PAGE_ACTION_TYPE;
+  return PAGE_ACTION_TYPE;
 }
 
 TranslateInfoBarDelegate*
@@ -371,6 +401,7 @@ TranslateInfoBarDelegate*
 }
 
 std::string TranslateInfoBarDelegate::GetPageHost() {
-  NavigationEntry* entry = tab_contents_->controller().GetActiveEntry();
-  return entry ? entry->url().HostNoBrackets() : std::string();
+  NavigationEntry* entry =
+      owner()->web_contents()->GetController().GetActiveEntry();
+  return entry ? entry->GetURL().HostNoBrackets() : std::string();
 }
