@@ -7,13 +7,15 @@
 #pragma once
 
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "chrome/browser/tab_contents/infobar_delegate.h"
+#include "base/compiler_specific.h"
+#include "chrome/browser/infobars/infobar_delegate.h"
 #include "chrome/browser/translate/translate_prefs.h"
 #include "chrome/common/translate_errors.h"
 
-class SkBitmap;
+class PrefService;
 class TranslateInfoBarView;
 
 class TranslateInfoBarDelegate : public InfoBarDelegate {
@@ -33,43 +35,46 @@ class TranslateInfoBarDelegate : public InfoBarDelegate {
     ERROR_TO_NORMAL
   };
 
-  // Factory method to create a non-error translate infobar.
-  // The original and target language specified are the ASCII language codes
-  // (ex: en, fr...).
-  // Returns NULL if it failed, typically if |original_language| or
-  // |target_language| is not a supported language.
+  static const size_t kNoIndex;
+
+  // Factory method to create a non-error translate infobar. |original_language|
+  // and |target_language| must be ASCII language codes (e.g. "en", "fr", etc.)
+  // for languages the TranslateManager supports translating. The lone exception
+  // is when the user initiates translation from the context menu, in which case
+  // it's legal to call this with |type| == TRANSLATING and
+  // |originalLanguage| == kUnknownLanguageCode.
   static TranslateInfoBarDelegate* CreateDelegate(
       Type infobar_type,
-      TabContents* tab_contents,
+      InfoBarTabHelper* infobar_helper,
+      PrefService* prefs,
       const std::string& original_language,
       const std::string& target_language);
 
   // Factory method to create an error translate infobar.
   static TranslateInfoBarDelegate* CreateErrorDelegate(
       TranslateErrors::Type error_type,
-      TabContents* tab_contents,
+      InfoBarTabHelper* infobar_helper,
+      PrefService* prefs,
       const std::string& original_language,
       const std::string& target_language);
 
   virtual ~TranslateInfoBarDelegate();
 
   // Returns the number of languages supported.
-  int GetLanguageCount() const { return static_cast<int>(languages_.size()); }
+  size_t GetLanguageCount() const { return languages_.size(); }
 
   // Returns the ISO code for the language at |index|.
-  std::string GetLanguageCodeAt(int index) const;
+  std::string GetLanguageCodeAt(size_t index) const;
 
   // Returns the displayable name for the language at |index|.
-  string16 GetLanguageDisplayableNameAt(int index) const;
-
-  TabContents* tab_contents() const { return tab_contents_; }
+  string16 GetLanguageDisplayableNameAt(size_t index) const;
 
   Type type() const { return type_; }
 
   TranslateErrors::Type error() const { return error_; }
 
-  int original_language_index() const { return original_language_index_; }
-  int target_language_index() const { return target_language_index_; }
+  size_t original_language_index() const { return original_language_index_; }
+  size_t target_language_index() const { return target_language_index_; }
 
   // Convenience methods.
   std::string GetOriginalLanguageCode() const;
@@ -77,8 +82,8 @@ class TranslateInfoBarDelegate : public InfoBarDelegate {
 
   // Called by the InfoBar to notify that the original/target language has
   // changed and is now the language at |language_index|.
-  virtual void SetOriginalLanguage(int language_index);
-  virtual void SetTargetLanguage(int language_index);
+  virtual void SetOriginalLanguage(size_t language_index);
+  virtual void SetTargetLanguage(size_t language_index);
 
   // Returns true if the current infobar indicates an error (in which case it
   // should get a yellow background instead of a blue one).
@@ -148,7 +153,8 @@ class TranslateInfoBarDelegate : public InfoBarDelegate {
   // For testing.
   TranslateInfoBarDelegate(Type infobar_type,
                            TranslateErrors::Type error,
-                           TabContents* tab_contents,
+                           InfoBarTabHelper* infobar_helper,
+                           PrefService* prefs,
                            const std::string& original_language,
                            const std::string& target_language);
   Type type_;
@@ -156,13 +162,14 @@ class TranslateInfoBarDelegate : public InfoBarDelegate {
  private:
   typedef std::pair<std::string, string16> LanguageNamePair;
 
-  // InfoBarDelegate implementation:
-  virtual InfoBar* CreateInfoBar();
-  virtual void InfoBarDismissed();
-  virtual void InfoBarClosed();
-  virtual SkBitmap* GetIcon() const;
-  virtual InfoBarDelegate::Type GetInfoBarType() const;
-  virtual TranslateInfoBarDelegate* AsTranslateInfoBarDelegate();
+  // InfoBarDelegate:
+  virtual InfoBar* CreateInfoBar(InfoBarTabHelper* infobar_helper) OVERRIDE;
+  virtual void InfoBarDismissed() OVERRIDE;
+  virtual gfx::Image* GetIcon() const OVERRIDE;
+  virtual InfoBarDelegate::Type GetInfoBarType() const OVERRIDE;
+  virtual bool ShouldExpire(
+       const content::LoadCommittedDetails& details) const OVERRIDE;
+  virtual TranslateInfoBarDelegate* AsTranslateInfoBarDelegate() OVERRIDE;
 
   // Gets the host of the page being translated, or an empty string if no URL is
   // associated with the current page.
@@ -172,8 +179,6 @@ class TranslateInfoBarDelegate : public InfoBarDelegate {
   // infobar.
   BackgroundAnimationType background_animation_;
 
-  TabContents* tab_contents_;
-
   // The list supported languages for translation.
   // The pair first string is the language ISO code (ex: en, fr...), the second
   // string is the displayable name on the current locale.
@@ -181,17 +186,17 @@ class TranslateInfoBarDelegate : public InfoBarDelegate {
   std::vector<LanguageNamePair> languages_;
 
   // The index for language the page is originally in.
-  int original_language_index_;
+  size_t original_language_index_;
 
   // The index for language the page is originally in that was originally
   // reported (original_language_index_ changes if the user selects a new
   // original language, but this one does not).  This is necessary to report
   // language detection errors with the right original language even if the user
   // changed the original language.
-  int initial_original_language_index_;
+  size_t initial_original_language_index_;
 
   // The index for language the page should be translated to.
-  int target_language_index_;
+  size_t target_language_index_;
 
   // The error that occurred when trying to translate (NONE if no error).
   TranslateErrors::Type error_;
