@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -34,41 +34,46 @@
 #define CHROME_BROWSER_TRANSPORT_SECURITY_PERSISTER_H_
 #pragma once
 
+#include <string>
+
 #include "base/file_path.h"
-#include "base/ref_counted.h"
-#include "base/task.h"
+#include "base/memory/weak_ptr.h"
+#include "chrome/common/important_file_writer.h"
 #include "net/base/transport_security_state.h"
 
+// Reads and updates on-disk TransportSecurity state.
+// Must be created, used and destroyed only on the IO thread.
 class TransportSecurityPersister
-    : public base::RefCountedThreadSafe<TransportSecurityPersister>,
-      public net::TransportSecurityState::Delegate {
+    : public net::TransportSecurityState::Delegate,
+      public ImportantFileWriter::DataSerializer {
  public:
-  TransportSecurityPersister();
-  void Initialize(net::TransportSecurityState* state,
-                  const FilePath& profile_path);
+  TransportSecurityPersister(net::TransportSecurityState* state,
+                             const FilePath& profile_path,
+                             bool readonly);
+  virtual ~TransportSecurityPersister();
 
   // Called by the TransportSecurityState when it changes its state.
-  virtual void StateIsDirty(net::TransportSecurityState*);
+  virtual void StateIsDirty(net::TransportSecurityState*) OVERRIDE;
+
+  // ImportantFileWriter::DataSerializer:
+  virtual bool SerializeData(std::string* data) OVERRIDE;
 
  private:
-  friend class base::RefCountedThreadSafe<TransportSecurityPersister>;
+  class Loader;
 
-  ~TransportSecurityPersister();
-
-  void Load();
   void CompleteLoad(const std::string& state);
 
-  void Save();
-  void CompleteSave(const std::string& state);
+  net::TransportSecurityState* transport_security_state_;
 
-  // Used on the IO thread to coalesce writes to disk.
-  ScopedRunnableMethodFactory<TransportSecurityPersister> save_coalescer_;
+  // Helper for safely writing the data.
+  ImportantFileWriter writer_;
 
-  scoped_refptr<net::TransportSecurityState>
-      transport_security_state_;  // IO thread only.
+  // Whether or not we're in read-only mode.
+  const bool readonly_;
 
-  // The path to the file in which we store the serialised state.
-  FilePath state_file_;
+  base::WeakPtrFactory<TransportSecurityPersister> weak_ptr_factory_;
+
+  DISALLOW_COPY_AND_ASSIGN(TransportSecurityPersister);
 };
 
 #endif  // CHROME_BROWSER_TRANSPORT_SECURITY_PERSISTER_H_
