@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,14 +6,14 @@
 #define CHROME_BROWSER_UI_APP_MODAL_DIALOGS_NATIVE_APP_MODAL_DIALOG_H_
 #pragma once
 
-#include "gfx/native_widget_types.h"
+#include "ui/gfx/native_widget_types.h"
 
 class JavaScriptAppModalDialog;
 
 class NativeAppModalDialog {
  public:
-  // Returns the buttons to be shown. See MessageBoxFlags for a description of
-  // the values.
+  // Returns the buttons to be shown. See ui::DialogButton for which buttons can
+  // be returned.
   virtual int GetAppModalDialogButtons() const = 0;
 
   // Shows the dialog.
