@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,13 +14,10 @@ class BookmarkNode;
 // bookmark bar.  This cell forwards mouseEntered/mouseExited events
 // to its control view so that pseudo-menu operations
 // (e.g. hover-over to open) can be implemented.
-@interface BookmarkBarFolderButtonCell : BookmarkButtonCell {
- @private
-  scoped_nsobject<NSColor> frameColor_;
-}
+@interface BookmarkBarFolderButtonCell : BookmarkButtonCell
 
 // Create a button cell which draws without a theme and with a frame
-// color provided by the BrowserThemeProvider defaults.
+// color provided by the ThemeService defaults.
 + (id)buttonCellForNode:(const BookmarkNode*)node
             contextMenu:(NSMenu*)contextMenu
                cellText:(NSString*)cellText
