@@ -5,18 +5,19 @@
 #import <Cocoa/Cocoa.h>
 
 #include "base/basictypes.h"
-#include "base/scoped_nsobject.h"
+#include "base/memory/scoped_nsobject.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
+#import "chrome/browser/ui/cocoa/animation_utils.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_bar_constants.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_bar_controller.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_bar_folder_button_cell.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_bar_folder_controller.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_bar_unittest_helper.h"
-#include "chrome/browser/ui/cocoa/browser_test_helper.h"
+#include "chrome/browser/ui/cocoa/cocoa_profile_test.h"
 #import "chrome/browser/ui/cocoa/cocoa_test_helper.h"
 #import "chrome/browser/ui/cocoa/view_resizer_pong.h"
-#include "chrome/test/model_test_utils.h"
+#include "chrome/test/base/model_test_utils.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #import "testing/gtest_mac.h"
 #include "testing/platform_test.h"
@@ -96,7 +97,6 @@ namespace {
 const int kLotsOfNodesCount = 150;
 };
 
-
 // Redirect certain calls so they can be seen by tests.
 
 @interface BookmarkBarControllerChildFolderRedirect : BookmarkBarController {
@@ -120,40 +120,46 @@ const int kLotsOfNodesCount = 150;
 @end
 
 
-class BookmarkBarFolderControllerTest : public CocoaTest {
+class BookmarkBarFolderControllerTest : public CocoaProfileTest {
  public:
-  BrowserTestHelper helper_;
   scoped_nsobject<BookmarkBarControllerChildFolderRedirect> bar_;
-  const BookmarkNode* folderA_;  // owned by model
-  const BookmarkNode* longTitleNode_;  // owned by model
+  const BookmarkNode* folderA_;  // Owned by model.
+  const BookmarkNode* longTitleNode_;  // Owned by model.
 
-  BookmarkBarFolderControllerTest() {
-    BookmarkModel* model = helper_.profile()->GetBookmarkModel();
-    const BookmarkNode* parent = model->GetBookmarkBarNode();
-    const BookmarkNode* folderA = model->AddGroup(parent,
-                                                  parent->GetChildCount(),
-                                                  ASCIIToUTF16("group"));
+  virtual void SetUp() {
+    CocoaProfileTest::SetUp();
+    ASSERT_TRUE(profile());
+
+    CreateModel();
+  }
+
+  void CreateModel() {
+    BookmarkModel* model = profile()->GetBookmarkModel();
+    const BookmarkNode* parent = model->bookmark_bar_node();
+    const BookmarkNode* folderA = model->AddFolder(parent,
+                                                   parent->child_count(),
+                                                   ASCIIToUTF16("folder"));
     folderA_ = folderA;
-    model->AddGroup(parent, parent->GetChildCount(),
-                    ASCIIToUTF16("sibbling group"));
-    const BookmarkNode* folderB = model->AddGroup(folderA,
-                                                  folderA->GetChildCount(),
-                                                  ASCIIToUTF16("subgroup 1"));
-    model->AddGroup(folderA,
-                    folderA->GetChildCount(),
-                    ASCIIToUTF16("subgroup 2"));
-    model->AddURL(folderA, folderA->GetChildCount(), ASCIIToUTF16("title a"),
+    model->AddFolder(parent, parent->child_count(),
+                     ASCIIToUTF16("sibbling folder"));
+    const BookmarkNode* folderB = model->AddFolder(folderA,
+                                                   folderA->child_count(),
+                                                   ASCIIToUTF16("subfolder 1"));
+    model->AddFolder(folderA,
+                     folderA->child_count(),
+                     ASCIIToUTF16("subfolder 2"));
+    model->AddURL(folderA, folderA->child_count(), ASCIIToUTF16("title a"),
                   GURL("http://www.google.com/a"));
     longTitleNode_ = model->AddURL(
-      folderA, folderA->GetChildCount(),
+      folderA, folderA->child_count(),
       ASCIIToUTF16("title super duper long long whoa momma title you betcha"),
       GURL("http://www.google.com/b"));
-    model->AddURL(folderB, folderB->GetChildCount(), ASCIIToUTF16("t"),
+    model->AddURL(folderB, folderB->child_count(), ASCIIToUTF16("t"),
                   GURL("http://www.google.com/c"));
 
     bar_.reset(
       [[BookmarkBarControllerChildFolderRedirect alloc]
-          initWithBrowser:helper_.browser()
+          initWithBrowser:browser()
              initialWidth:300
                  delegate:nil
            resizeDelegate:nil]);
@@ -172,17 +178,17 @@ class BookmarkBarFolderControllerTest : public CocoaTest {
 
   // Remove the bookmark with the long title.
   void RemoveLongTitleNode() {
-    BookmarkModel* model = helper_.profile()->GetBookmarkModel();
-    model->Remove(longTitleNode_->GetParent(),
-                  longTitleNode_->GetParent()->IndexOfChild(longTitleNode_));
+    BookmarkModel* model = profile()->GetBookmarkModel();
+    model->Remove(longTitleNode_->parent(),
+                  longTitleNode_->parent()->GetIndexOf(longTitleNode_));
   }
 
   // Add LOTS of nodes to our model if needed (e.g. scrolling).
   // Returns the number of nodes added.
   int AddLotsOfNodes() {
-    BookmarkModel* model = helper_.profile()->GetBookmarkModel();
+    BookmarkModel* model = profile()->GetBookmarkModel();
     for (int i = 0; i < kLotsOfNodesCount; i++) {
-      model->AddURL(folderA_, folderA_->GetChildCount(),
+      model->AddURL(folderA_, folderA_->child_count(),
                     ASCIIToUTF16("repeated title"),
                     GURL("http://www.google.com/repeated/url"));
     }
@@ -246,7 +252,7 @@ TEST_F(BookmarkBarFolderControllerTest, ReleaseOnClose) {
 }
 
 TEST_F(BookmarkBarFolderControllerTest, BasicPosition) {
- BookmarkButton* parentButton = [[bar_ buttons] objectAtIndex:0];
+  BookmarkButton* parentButton = [[bar_ buttons] objectAtIndex:0];
   EXPECT_TRUE(parentButton);
 
   // If parent is a BookmarkBarController, grow down.
@@ -256,7 +262,7 @@ TEST_F(BookmarkBarFolderControllerTest, BasicPosition) {
                    parentController:nil
                       barController:bar_]);
   [bbfc window];
-  NSPoint pt = [bbfc windowTopLeftForWidth:0];  // screen coords
+  NSPoint pt = [bbfc windowTopLeftForWidth:0 height:100];  // screen coords
   NSPoint buttonOriginInScreen =
       [[parentButton window]
         convertBaseToScreen:[parentButton
@@ -268,8 +274,8 @@ TEST_F(BookmarkBarFolderControllerTest, BasicPosition) {
             bookmarks::kBookmarkMenuOverlap+1);
 
   // Make sure we see the window shift left if it spills off the screen
-  pt = [bbfc windowTopLeftForWidth:0];
-  NSPoint shifted = [bbfc windowTopLeftForWidth:9999999];
+  pt = [bbfc windowTopLeftForWidth:0 height:100];
+  NSPoint shifted = [bbfc windowTopLeftForWidth:9999999 height:100];
   EXPECT_LT(shifted.x, pt.x);
 
   // If parent is a BookmarkBarFolderController, grow right.
@@ -279,7 +285,7 @@ TEST_F(BookmarkBarFolderControllerTest, BasicPosition) {
                     parentController:bbfc.get()
                        barController:bar_]);
   [bbfc2 window];
-  pt = [bbfc2 windowTopLeftForWidth:0];
+  pt = [bbfc2 windowTopLeftForWidth:0 height:100];
   // We're now overlapping the window a bit.
   EXPECT_EQ(pt.x, NSMaxX([[bbfc.get() window] frame]) -
             bookmarks::kBookmarkMenuOverlap);
@@ -288,15 +294,15 @@ TEST_F(BookmarkBarFolderControllerTest, BasicPosition) {
 // Confirm we grow right until end of screen, then start growing left
 // until end of screen again, then right.
 TEST_F(BookmarkBarFolderControllerTest, PositionRightLeftRight) {
-  BookmarkModel* model = helper_.profile()->GetBookmarkModel();
-  const BookmarkNode* parent = model->GetBookmarkBarNode();
+  BookmarkModel* model = profile()->GetBookmarkModel();
+  const BookmarkNode* parent = model->bookmark_bar_node();
   const BookmarkNode* folder = parent;
 
   const int count = 100;
   int i;
   // Make some super duper deeply nested folders.
-  for (i=0; i<count; i++) {
-    folder = model->AddGroup(folder, 0, ASCIIToUTF16("nested folder"));
+  for (i = 0; i < count; i++) {
+    folder = model->AddFolder(folder, 0, ASCIIToUTF16("nested folder"));
   }
 
   // Setup initial state for opening all folders.
@@ -447,9 +453,10 @@ TEST_F(BookmarkBarFolderControllerTest, ChildFolderWidth) {
 }
 
 // Simple scrolling tests.
-TEST_F(BookmarkBarFolderControllerTest, SimpleScroll) {
+// Currently flaky due to a changed definition of the correct menu boundaries.
+TEST_F(BookmarkBarFolderControllerTest, DISABLED_SimpleScroll) {
   scoped_nsobject<BookmarkBarFolderController> bbfc;
-  NSRect screenFrame = [[NSScreen mainScreen] frame];
+  NSRect screenFrame = [[NSScreen mainScreen] visibleFrame];
   CGFloat screenHeight = NSHeight(screenFrame);
   int nodecount = AddLotsOfNodes();
   bbfc.reset(SimpleBookmarkBarFolderController());
@@ -473,7 +480,7 @@ TEST_F(BookmarkBarFolderControllerTest, SimpleScroll) {
   NSScrollView* scrollView = [bbfc scrollView];
 
   // Find the next-to-last button showing at the bottom of the window and
-  // us its center for hit testing.
+  // use its center for hit testing.
   BookmarkButton* targetButton = nil;
   NSPoint scrollPoint = [scrollView documentVisibleRect].origin;
   for (BookmarkButton* button in [bbfc buttons]) {
@@ -505,7 +512,7 @@ TEST_F(BookmarkBarFolderControllerTest, SimpleScroll) {
   bool bothAtOnce = false;
   while ([bbfc canScrollUp]) {
     [bbfc performOneScroll:60];
-    EXPECT_TRUE(NSContainsRect(screenFrame, [window frame]));
+    EXPECT_TRUE(NSContainsRect([[NSScreen mainScreen] frame], [window frame]));
     // Make sure, sometime during our scroll, we have the ability to
     // scroll in either direction.
     if ([bbfc canScrollUp] &&
@@ -518,6 +525,8 @@ TEST_F(BookmarkBarFolderControllerTest, SimpleScroll) {
   EXPECT_FALSE([bbfc canScrollUp]);
   EXPECT_TRUE([bbfc canScrollDown]);
 
+  NSRect wholeScreenRect = [[NSScreen mainScreen] frame];
+
   // Now scroll down and make sure the window size does not change.
   // Also confirm we never scroll the window off the screen the other
   // way.
@@ -526,14 +535,11 @@ TEST_F(BookmarkBarFolderControllerTest, SimpleScroll) {
     // Once we can no longer scroll down the window height changes.
     if (![bbfc canScrollDown])
       break;
-    EXPECT_EQ(screenHeight, NSHeight([window frame]));
-    EXPECT_TRUE(NSContainsRect(screenFrame, [window frame]));
+    EXPECT_TRUE(NSContainsRect(wholeScreenRect, [window frame]));
   }
-  // The final height should be offset from the top of the screen and still
-  // within the screen.
-  CGFloat height = screenHeight - bookmarks::kScrollWindowVerticalMargin;
-  EXPECT_CGFLOAT_EQ(height, NSHeight([window frame]));
-  EXPECT_TRUE(NSContainsRect(screenFrame, [window frame]));
+
+  EXPECT_GT(NSHeight(wholeScreenRect), NSHeight([window frame]));
+  EXPECT_TRUE(NSContainsRect(wholeScreenRect, [window frame]));
 }
 
 // Folder menu sizing and placement while deleting bookmarks
@@ -590,6 +596,13 @@ TEST_F(BookmarkBarFolderControllerTest, MenuPlacementWhileScrollingDeleting) {
   EXPECT_CGFLOAT_EQ(oldTop - bookmarks::kScrollWindowVerticalMargin, newTop);
 }
 
+// Make sure that we return the correct browser window.
+TEST_F(BookmarkBarFolderControllerTest, BrowserWindow) {
+  scoped_nsobject<BookmarkBarFolderController> controller(
+      SimpleBookmarkBarFolderController());
+  EXPECT_EQ([bar_ browserWindow], [controller browserWindow]);
+}
+
 @interface FakedDragInfo : NSObject {
 @public
   NSPoint dropLocation_;
@@ -638,20 +651,22 @@ TEST_F(BookmarkBarFolderControllerTest, MenuPlacementWhileScrollingDeleting) {
 @end
 
 
-class BookmarkBarFolderControllerMenuTest : public CocoaTest {
+class BookmarkBarFolderControllerMenuTest : public CocoaProfileTest {
  public:
-  BrowserTestHelper helper_;
   scoped_nsobject<NSView> parent_view_;
   scoped_nsobject<ViewResizerPong> resizeDelegate_;
   scoped_nsobject<BookmarkBarController> bar_;
 
-  BookmarkBarFolderControllerMenuTest() {
+  virtual void SetUp() {
+    CocoaProfileTest::SetUp();
+    ASSERT_TRUE(browser());
+
     resizeDelegate_.reset([[ViewResizerPong alloc] init]);
     NSRect parent_frame = NSMakeRect(0, 0, 800, 50);
     parent_view_.reset([[NSView alloc] initWithFrame:parent_frame]);
     [parent_view_ setHidden:YES];
     bar_.reset([[BookmarkBarController alloc]
-                initWithBrowser:helper_.browser()
+                initWithBrowser:browser()
                    initialWidth:NSWidth(parent_frame)
                        delegate:nil
                  resizeDelegate:resizeDelegate_.get()]);
@@ -678,8 +693,9 @@ class BookmarkBarFolderControllerMenuTest : public CocoaTest {
 };
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveBarBookmarkToFolder) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  WithNoAnimation at_all;
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b "
       "2f2f3b ] 2f3b ] 3b 4f:[ 4f1f:[ 4f1f1b 4f1f2b 4f1f3b ] 4f2f:[ 4f2f1b "
       "4f2f2b 4f2f3b ] 4f3f:[ 4f3f1b 4f3f2b 4f3f3b ] ] 5b ");
@@ -751,8 +767,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveBarBookmarkToFolder) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragCopyBarBookmarkToFolder) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b "
       "2f2f3b ] 2f3b ] 3b 4f:[ 4f1f:[ 4f1f1b 4f1f2b 4f1f3b ] 4f2f:[ 4f2f1b "
       "4f2f2b 4f2f3b ] 4f3f:[ 4f3f1b 4f3f2b 4f3f3b ] ] 5b ");
@@ -815,8 +831,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragCopyBarBookmarkToFolder) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveBarBookmarkToSubfolder) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b "
       "2f2f3b ] 2f3b ] 3b 4f:[ 4f1f:[ 4f1f1b 4f1f2b 4f1f3b ] 4f2f:[ 4f2f1b "
       "4f2f2b 4f2f3b ] 4f3f:[ 4f3f1b 4f3f2b 4f3f3b ] ] 5b ");
@@ -880,8 +896,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveBarBookmarkToSubfolder) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveWithinFolder) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b "
       "2f2f3b ] 2f3b ] 3b 4f:[ 4f1f:[ 4f1f1b 4f1f2b 4f1f3b ] 4f2f:[ 4f2f1b "
       "4f2f2b 4f2f3b ] 4f3f:[ 4f3f1b 4f3f2b 4f3f3b ] ] 5b ");
@@ -929,8 +945,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveWithinFolder) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragParentOntoChild) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b "
       "2f2f3b ] 2f3b ] 3b 4f:[ 4f1f:[ 4f1f1b 4f1f2b 4f1f3b ] 4f2f:[ 4f2f1b "
       "4f2f2b 4f2f3b ] 4f3f:[ 4f3f1b 4f3f2b 4f3f3b ] ] 5b ");
@@ -966,8 +982,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragParentOntoChild) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveChildToParent) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b "
       "2f2f3b ] 2f3b ] 3b 4f:[ 4f1f:[ 4f1f1b 4f1f2b 4f1f3b ] 4f2f:[ 4f2f1b "
       "4f2f2b 4f2f3b ] 4f3f:[ 4f3f1b 4f3f2b 4f3f3b ] ] 5b ");
@@ -1018,8 +1034,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragMoveChildToParent) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragWindowResizing) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string
       model_string("a b:[ b1 b2 b3 ] reallyReallyLongBookmarkName c ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
@@ -1058,8 +1074,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragWindowResizing) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, MoveRemoveAddButtons) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2b 2f3b ] 3b 4b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
 
@@ -1121,8 +1137,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, MoveRemoveAddButtons) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, ControllerForNode) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2b ] 3b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
 
@@ -1156,8 +1172,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, ControllerForNode) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, MenuSizingAndScrollArrows) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2b 3b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
 
@@ -1165,10 +1181,10 @@ TEST_F(BookmarkBarFolderControllerMenuTest, MenuSizingAndScrollArrows) {
   std::string actualModelString = model_test_utils::ModelStringFromNode(root);
   EXPECT_EQ(model_string, actualModelString);
 
-  const BookmarkNode* parent = model.GetBookmarkBarNode();
-  const BookmarkNode* folder = model.AddGroup(parent,
-                                              parent->GetChildCount(),
-                                              ASCIIToUTF16("BIG"));
+  const BookmarkNode* parent = model.bookmark_bar_node();
+  const BookmarkNode* folder = model.AddFolder(parent,
+                                               parent->child_count(),
+                                               ASCIIToUTF16("BIG"));
 
   // Pop open the new folder window and verify it has one (empty) item.
   BookmarkButton* button = [bar_ buttonWithTitleEqualTo:@"BIG"];
@@ -1187,7 +1203,7 @@ TEST_F(BookmarkBarFolderControllerMenuTest, MenuSizingAndScrollArrows) {
   EXPECT_FALSE([folderController canScrollDown]);
 
   // Now add a real bookmark and reopen.
-  model.AddURL(folder, folder->GetChildCount(), ASCIIToUTF16("a"),
+  model.AddURL(folder, folder->child_count(), ASCIIToUTF16("a"),
                GURL("http://a.com/"));
   folderController = [bar_ folderController];
   EXPECT_TRUE(folderController);
@@ -1213,7 +1229,7 @@ TEST_F(BookmarkBarFolderControllerMenuTest, MenuSizingAndScrollArrows) {
   EXPECT_LT(scrollerWidth, NSWidth([folderView frame]));
 
   // Add a wider bookmark and make sure the button widths match.
-  int reallyWideButtonNumber = folder->GetChildCount();
+  int reallyWideButtonNumber = folder->child_count();
   model.AddURL(folder, reallyWideButtonNumber,
                ASCIIToUTF16("A really, really, really, really, really, "
                             "really long name"),
@@ -1228,7 +1244,7 @@ TEST_F(BookmarkBarFolderControllerMenuTest, MenuSizingAndScrollArrows) {
   // for a scroll up arrow.
   NSUInteger tripWire = 0;  // Prevent a runaway.
   while (![folderController canScrollUp] && ++tripWire < 1000) {
-    model.AddURL(folder, folder->GetChildCount(), ASCIIToUTF16("B"),
+    model.AddURL(folder, folder->child_count(), ASCIIToUTF16("B"),
                  GURL("http://b.com/"));
   }
   EXPECT_TRUE([folderController canScrollUp]);
@@ -1251,13 +1267,13 @@ TEST_F(BookmarkBarFolderControllerMenuTest, MenuSizingAndScrollArrows) {
 
 // See http://crbug.com/46101
 TEST_F(BookmarkBarFolderControllerMenuTest, HoverThenDeleteBookmark) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
-  const BookmarkNode* folder = model.AddGroup(root,
-                                              root->GetChildCount(),
-                                              ASCIIToUTF16("BIG"));
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
+  const BookmarkNode* folder = model.AddFolder(root,
+                                               root->child_count(),
+                                               ASCIIToUTF16("BIG"));
   for (int i = 0; i < kLotsOfNodesCount; i++)
-    model.AddURL(folder, folder->GetChildCount(), ASCIIToUTF16("kid"),
+    model.AddURL(folder, folder->child_count(), ASCIIToUTF16("kid"),
                   GURL("http://kid.com/smile"));
 
   // Pop open the new folder window and hover one of its kids.
@@ -1317,8 +1333,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, HoverThenDeleteBookmark) {
 @end
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragBookmarkData) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b 2f2f3b ] "
                                  "2f3b ] 3b 4b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
@@ -1378,8 +1394,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragBookmarkData) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DragBookmarkDataToTrash) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b 2f2f3b ] "
                                  "2f3b ] 3b 4b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
@@ -1389,7 +1405,7 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragBookmarkDataToTrash) {
   EXPECT_EQ(model_string, actual);
 
   const BookmarkNode* folderNode = root->GetChild(1);
-  int oldFolderChildCount = folderNode->GetChildCount();
+  int oldFolderChildCount = folderNode->child_count();
 
   // Pop open a folder.
   BookmarkButton* button = [bar_ buttonWithTitleEqualTo:@"2f"];
@@ -1407,7 +1423,7 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragBookmarkDataToTrash) {
   [folderController didDragBookmarkToTrash:buttonToDelete];
 
   // There should be one less button in the folder.
-  int newFolderChildCount = folderNode->GetChildCount();
+  int newFolderChildCount = folderNode->child_count();
   EXPECT_EQ(oldFolderChildCount - 1, newFolderChildCount);
   // Verify the model.
   const std::string expected("1b 2f:[ 2f2f:[ 2f2f1b 2f2f2b 2f2f3b ] "
@@ -1420,8 +1436,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DragBookmarkDataToTrash) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, AddURLs) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b 2f2f3b ] "
                                  "2f3b ] 3b 4b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
@@ -1465,8 +1481,8 @@ TEST_F(BookmarkBarFolderControllerMenuTest, AddURLs) {
 }
 
 TEST_F(BookmarkBarFolderControllerMenuTest, DropPositionIndicator) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b 2f2f3b ] "
                                  "2f3b ] 3b 4b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
@@ -1510,9 +1526,12 @@ TEST_F(BookmarkBarFolderControllerMenuTest, DropPositionIndicator) {
 class BookmarkBarFolderControllerClosingTest : public
     BookmarkBarFolderControllerMenuTest {
  public:
-  BookmarkBarFolderControllerClosingTest() {
+  virtual void SetUp() {
+    BookmarkBarFolderControllerMenuTest::SetUp();
+    ASSERT_TRUE(browser());
+
     bar_.reset([[BookmarkBarControllerNoDelete alloc]
-                initWithBrowser:helper_.browser()
+                initWithBrowser:browser()
                    initialWidth:NSWidth([parent_view_ frame])
                        delegate:nil
                  resizeDelegate:resizeDelegate_.get()]);
@@ -1521,8 +1540,8 @@ class BookmarkBarFolderControllerClosingTest : public
 };
 
 TEST_F(BookmarkBarFolderControllerClosingTest, DeleteClosesFolder) {
-  BookmarkModel& model(*helper_.profile()->GetBookmarkModel());
-  const BookmarkNode* root = model.GetBookmarkBarNode();
+  BookmarkModel& model(*profile()->GetBookmarkModel());
+  const BookmarkNode* root = model.bookmark_bar_node();
   const std::string model_string("1b 2f:[ 2f1b 2f2f:[ 2f2f1b 2f2f2b ] "
                                  "2f3b ] 3b ");
   model_test_utils::AddNodesFromModelString(model, root, model_string);
