@@ -1,22 +1,24 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #import <Cocoa/Cocoa.h>
 
 #include "base/basictypes.h"
-#include "base/scoped_nsobject.h"
+#include "base/memory/scoped_nsobject.h"
+#include "base/string16.h"
 #include "base/utf_string_conversions.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_bubble_controller.h"
-#include "chrome/browser/ui/cocoa/browser_test_helper.h"
 #include "chrome/browser/ui/cocoa/browser_window_controller.h"
-#import "chrome/browser/ui/cocoa/cocoa_test_helper.h"
+#include "chrome/browser/ui/cocoa/cocoa_profile_test.h"
 #import "chrome/browser/ui/cocoa/info_bubble_window.h"
-#include "chrome/common/notification_service.h"
+#include "content/public/browser/notification_service.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #import "testing/gtest_mac.h"
 #include "testing/platform_test.h"
 
+using content::WebContents;
+
 // Watch for bookmark pulse notifications so we can confirm they were sent.
 @interface BookmarkPulseObserver : NSObject {
   int notifications_;
@@ -54,10 +56,9 @@
 
 namespace {
 
-class BookmarkBubbleControllerTest : public CocoaTest {
+class BookmarkBubbleControllerTest : public CocoaProfileTest {
  public:
   static int edits_;
-  BrowserTestHelper helper_;
   BookmarkBubbleController* controller_;
 
   BookmarkBubbleControllerTest() : controller_(nil) {
@@ -66,7 +67,7 @@ class BookmarkBubbleControllerTest : public CocoaTest {
 
   virtual void TearDown() {
     [controller_ close];
-    CocoaTest::TearDown();
+    CocoaProfileTest::TearDown();
   }
 
   // Returns a controller but ownership not transferred.
@@ -78,7 +79,7 @@ class BookmarkBubbleControllerTest : public CocoaTest {
     }
     controller_ = [[BookmarkBubbleController alloc]
                       initWithParentWindow:test_window()
-                                     model:helper_.profile()->GetBookmarkModel()
+                                     model:profile()->GetBookmarkModel()
                                       node:node
                          alreadyBookmarked:YES];
     EXPECT_TRUE([controller_ window]);
@@ -89,7 +90,7 @@ class BookmarkBubbleControllerTest : public CocoaTest {
   }
 
   BookmarkModel* GetBookmarkModel() {
-    return helper_.profile()->GetBookmarkModel();
+    return profile()->GetBookmarkModel();
   }
 
   bool IsWindowClosing() {
@@ -104,7 +105,7 @@ int BookmarkBubbleControllerTest::edits_;
 // parent window)
 TEST_F(BookmarkBubbleControllerTest, TestBubbleWindow) {
   BookmarkModel* model = GetBookmarkModel();
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0,
                                            ASCIIToUTF16("Bookie markie title"),
                                            GURL("http://www.google.com"));
@@ -119,7 +120,7 @@ TEST_F(BookmarkBubbleControllerTest, TestBubbleWindow) {
 // Test that we can handle closing the parent window
 TEST_F(BookmarkBubbleControllerTest, TestClosingParentWindow) {
   BookmarkModel* model = GetBookmarkModel();
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0,
                                            ASCIIToUTF16("Bookie markie title"),
                                            GURL("http://www.google.com"));
@@ -137,18 +138,18 @@ TEST_F(BookmarkBubbleControllerTest, TestFillInFolder) {
   // Create some folders, including a nested folder
   BookmarkModel* model = GetBookmarkModel();
   EXPECT_TRUE(model);
-  const BookmarkNode* bookmarkBarNode = model->GetBookmarkBarNode();
+  const BookmarkNode* bookmarkBarNode = model->bookmark_bar_node();
   EXPECT_TRUE(bookmarkBarNode);
-  const BookmarkNode* node1 = model->AddGroup(bookmarkBarNode, 0,
-                                              ASCIIToUTF16("one"));
+  const BookmarkNode* node1 = model->AddFolder(bookmarkBarNode, 0,
+                                               ASCIIToUTF16("one"));
   EXPECT_TRUE(node1);
-  const BookmarkNode* node2 = model->AddGroup(bookmarkBarNode, 1,
-                                              ASCIIToUTF16("two"));
+  const BookmarkNode* node2 = model->AddFolder(bookmarkBarNode, 1,
+                                               ASCIIToUTF16("two"));
   EXPECT_TRUE(node2);
-  const BookmarkNode* node3 = model->AddGroup(bookmarkBarNode, 2,
-                                              ASCIIToUTF16("three"));
+  const BookmarkNode* node3 = model->AddFolder(bookmarkBarNode, 2,
+                                               ASCIIToUTF16("three"));
   EXPECT_TRUE(node3);
-  const BookmarkNode* node4 = model->AddGroup(node2, 0, ASCIIToUTF16("sub"));
+  const BookmarkNode* node4 = model->AddFolder(node2, 0, ASCIIToUTF16("sub"));
   EXPECT_TRUE(node4);
   const BookmarkNode* node5 = model->AddURL(node1, 0, ASCIIToUTF16("title1"),
                                             GURL("http://www.google.com"));
@@ -156,8 +157,8 @@ TEST_F(BookmarkBubbleControllerTest, TestFillInFolder) {
   const BookmarkNode* node6 = model->AddURL(node3, 0, ASCIIToUTF16("title2"),
                                             GURL("http://www.google.com"));
   EXPECT_TRUE(node6);
-  const BookmarkNode* node7 = model->AddURL(node4, 0, ASCIIToUTF16("title3"),
-                                          GURL("http://www.google.com/reader"));
+  const BookmarkNode* node7 = model->AddURL(
+      node4, 0, ASCIIToUTF16("title3"), GURL("http://www.google.com/reader"));
   EXPECT_TRUE(node7);
 
   BookmarkBubbleController* controller = ControllerForNode(node4);
@@ -171,6 +172,16 @@ TEST_F(BookmarkBubbleControllerTest, TestFillInFolder) {
   EXPECT_TRUE([titles containsObject:@"sub"]);
   EXPECT_FALSE([titles containsObject:@"title1"]);
   EXPECT_FALSE([titles containsObject:@"title2"]);
+
+
+  // Verify that the top level folders are displayed correctly.
+  EXPECT_TRUE([titles containsObject:@"Other Bookmarks"]);
+  EXPECT_TRUE([titles containsObject:@"Bookmarks Bar"]);
+  if (model->mobile_node()->IsVisible()) {
+    EXPECT_TRUE([titles containsObject:@"Mobile Bookmarks"]);
+  } else {
+    EXPECT_FALSE([titles containsObject:@"Mobile Bookmarks"]);
+  }
 }
 
 // Confirm ability to handle folders with blank name.
@@ -178,16 +189,16 @@ TEST_F(BookmarkBubbleControllerTest, TestFolderWithBlankName) {
   // Create some folders, including a nested folder
   BookmarkModel* model = GetBookmarkModel();
   EXPECT_TRUE(model);
-  const BookmarkNode* bookmarkBarNode = model->GetBookmarkBarNode();
+  const BookmarkNode* bookmarkBarNode = model->bookmark_bar_node();
   EXPECT_TRUE(bookmarkBarNode);
-  const BookmarkNode* node1 = model->AddGroup(bookmarkBarNode, 0,
-                                              ASCIIToUTF16("one"));
+  const BookmarkNode* node1 = model->AddFolder(bookmarkBarNode, 0,
+                                               ASCIIToUTF16("one"));
   EXPECT_TRUE(node1);
-  const BookmarkNode* node2 = model->AddGroup(bookmarkBarNode, 1,
-                                              ASCIIToUTF16(""));
+  const BookmarkNode* node2 = model->AddFolder(bookmarkBarNode, 1,
+                                               string16());
   EXPECT_TRUE(node2);
-  const BookmarkNode* node3 = model->AddGroup(bookmarkBarNode, 2,
-                                              ASCIIToUTF16("three"));
+  const BookmarkNode* node3 = model->AddFolder(bookmarkBarNode, 2,
+                                               ASCIIToUTF16("three"));
   EXPECT_TRUE(node3);
   const BookmarkNode* node2_1 = model->AddURL(node2, 0, ASCIIToUTF16("title1"),
                                               GURL("http://www.google.com"));
@@ -215,7 +226,7 @@ TEST_F(BookmarkBubbleControllerTest, TestFolderWithBlankName) {
 // Click on edit; bubble gets closed.
 TEST_F(BookmarkBubbleControllerTest, TestEdit) {
   BookmarkModel* model = GetBookmarkModel();
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0,
                                            ASCIIToUTF16("Bookie markie title"),
                                            GURL("http://www.google.com"));
@@ -234,7 +245,7 @@ TEST_F(BookmarkBubbleControllerTest, TestEdit) {
 TEST_F(BookmarkBubbleControllerTest, TestClose) {
     BookmarkModel* model = GetBookmarkModel();
     const BookmarkNode* node = model->AddURL(
-        model->GetBookmarkBarNode(), 0, ASCIIToUTF16("Bookie markie title"),
+        model->bookmark_bar_node(), 0, ASCIIToUTF16("Bookie markie title"),
         GURL("http://www.google.com"));
   EXPECT_EQ(edits_, 0);
 
@@ -255,17 +266,17 @@ TEST_F(BookmarkBubbleControllerTest, TestClose) {
 TEST_F(BookmarkBubbleControllerTest, TestUserEdit) {
   BookmarkModel* model = GetBookmarkModel();
   EXPECT_TRUE(model);
-  const BookmarkNode* bookmarkBarNode = model->GetBookmarkBarNode();
+  const BookmarkNode* bookmarkBarNode = model->bookmark_bar_node();
   EXPECT_TRUE(bookmarkBarNode);
   const BookmarkNode* node = model->AddURL(bookmarkBarNode,
                                            0,
                                            ASCIIToUTF16("short-title"),
                                            GURL("http://www.google.com"));
-  const BookmarkNode* grandma = model->AddGroup(bookmarkBarNode, 0,
-                                                ASCIIToUTF16("grandma"));
+  const BookmarkNode* grandma = model->AddFolder(bookmarkBarNode, 0,
+                                                 ASCIIToUTF16("grandma"));
   EXPECT_TRUE(grandma);
-  const BookmarkNode* grandpa = model->AddGroup(bookmarkBarNode, 0,
-                                                ASCIIToUTF16("grandpa"));
+  const BookmarkNode* grandpa = model->AddFolder(bookmarkBarNode, 0,
+                                                 ASCIIToUTF16("grandpa"));
   EXPECT_TRUE(grandpa);
 
   BookmarkBubbleController* controller = ControllerForNode(node);
@@ -277,14 +288,14 @@ TEST_F(BookmarkBubbleControllerTest, TestUserEdit) {
 
   // Make sure bookmark has changed
   EXPECT_EQ(node->GetTitle(), ASCIIToUTF16("oops"));
-  EXPECT_EQ(node->GetParent()->GetTitle(), ASCIIToUTF16("grandma"));
+  EXPECT_EQ(node->parent()->GetTitle(), ASCIIToUTF16("grandma"));
 }
 
 // Confirm happiness with parent nodes that have the same name.
 TEST_F(BookmarkBubbleControllerTest, TestNewParentSameName) {
   BookmarkModel* model = GetBookmarkModel();
   EXPECT_TRUE(model);
-  const BookmarkNode* bookmarkBarNode = model->GetBookmarkBarNode();
+  const BookmarkNode* bookmarkBarNode = model->bookmark_bar_node();
   EXPECT_TRUE(bookmarkBarNode);
   for (int i=0; i<2; i++) {
     const BookmarkNode* node = model->AddURL(bookmarkBarNode,
@@ -292,13 +303,13 @@ TEST_F(BookmarkBubbleControllerTest, TestNewParentSameName) {
                                              ASCIIToUTF16("short-title"),
                                              GURL("http://www.google.com"));
     EXPECT_TRUE(node);
-    const BookmarkNode* group = model->AddGroup(bookmarkBarNode, 0,
-                                                ASCIIToUTF16("NAME"));
-    EXPECT_TRUE(group);
-    group = model->AddGroup(bookmarkBarNode, 0, ASCIIToUTF16("NAME"));
-    EXPECT_TRUE(group);
-    group = model->AddGroup(bookmarkBarNode, 0, ASCIIToUTF16("NAME"));
-    EXPECT_TRUE(group);
+    const BookmarkNode* folder = model->AddFolder(bookmarkBarNode, 0,
+                                                 ASCIIToUTF16("NAME"));
+    EXPECT_TRUE(folder);
+    folder = model->AddFolder(bookmarkBarNode, 0, ASCIIToUTF16("NAME"));
+    EXPECT_TRUE(folder);
+    folder = model->AddFolder(bookmarkBarNode, 0, ASCIIToUTF16("NAME"));
+    EXPECT_TRUE(folder);
     BookmarkBubbleController* controller = ControllerForNode(node);
     EXPECT_TRUE(controller);
 
@@ -308,20 +319,20 @@ TEST_F(BookmarkBubbleControllerTest, TestNewParentSameName) {
 
     // Make sure bookmark has changed, and that the parent is what we
     // expect.  This proves nobody did searching based on name.
-    EXPECT_EQ(node->GetParent(), bookmarkBarNode->GetChild(i));
+    EXPECT_EQ(node->parent(), bookmarkBarNode->GetChild(i));
   }
 }
 
 // Confirm happiness with nodes with the same Name
 TEST_F(BookmarkBubbleControllerTest, TestDuplicateNodeNames) {
   BookmarkModel* model = GetBookmarkModel();
-  const BookmarkNode* bookmarkBarNode = model->GetBookmarkBarNode();
+  const BookmarkNode* bookmarkBarNode = model->bookmark_bar_node();
   EXPECT_TRUE(bookmarkBarNode);
-  const BookmarkNode* node1 = model->AddGroup(bookmarkBarNode, 0,
-                                              ASCIIToUTF16("NAME"));
+  const BookmarkNode* node1 = model->AddFolder(bookmarkBarNode, 0,
+                                               ASCIIToUTF16("NAME"));
   EXPECT_TRUE(node1);
-  const BookmarkNode* node2 = model->AddGroup(bookmarkBarNode, 0,
-                                              ASCIIToUTF16("NAME"));
+  const BookmarkNode* node2 = model->AddFolder(bookmarkBarNode, 0,
+                                               ASCIIToUTF16("NAME"));
   EXPECT_TRUE(node2);
   BookmarkBubbleController* controller = ControllerForNode(bookmarkBarNode);
   EXPECT_TRUE(controller);
@@ -341,7 +352,7 @@ TEST_F(BookmarkBubbleControllerTest, TestDuplicateNodeNames) {
 TEST_F(BookmarkBubbleControllerTest, TestRemove) {
   BookmarkModel* model = GetBookmarkModel();
   GURL gurl("http://www.google.com");
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0,
                                            ASCIIToUTF16("Bookie markie title"),
                                            gurl);
@@ -358,7 +369,7 @@ TEST_F(BookmarkBubbleControllerTest, TestRemove) {
 TEST_F(BookmarkBubbleControllerTest, PopUpSelectionChanged) {
   BookmarkModel* model = GetBookmarkModel();
   GURL gurl("http://www.google.com");
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0, ASCIIToUTF16("super-title"),
                                            gurl);
   BookmarkBubbleController* controller = ControllerForNode(node);
@@ -377,14 +388,14 @@ TEST_F(BookmarkBubbleControllerTest, PopUpSelectionChanged) {
 TEST_F(BookmarkBubbleControllerTest, EscapeRemovesNewBookmark) {
   BookmarkModel* model = GetBookmarkModel();
   GURL gurl("http://www.google.com");
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0,
                                            ASCIIToUTF16("Bookie markie title"),
                                            gurl);
   BookmarkBubbleController* controller =
       [[BookmarkBubbleController alloc]
           initWithParentWindow:test_window()
-                         model:helper_.profile()->GetBookmarkModel()
+                         model:profile()->GetBookmarkModel()
                           node:node
              alreadyBookmarked:NO];  // The last param is the key difference.
   EXPECT_TRUE([controller window]);
@@ -399,7 +410,7 @@ TEST_F(BookmarkBubbleControllerTest, EscapeRemovesNewBookmark) {
 TEST_F(BookmarkBubbleControllerTest, EscapeDoesntTouchExistingBookmark) {
   BookmarkModel* model = GetBookmarkModel();
   GURL gurl("http://www.google.com");
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0,
                                            ASCIIToUTF16("Bookie markie title"),
                                            gurl);
@@ -415,19 +426,19 @@ TEST_F(BookmarkBubbleControllerTest, TestMenuIndentation) {
   // Create some folders, including a nested folder
   BookmarkModel* model = GetBookmarkModel();
   EXPECT_TRUE(model);
-  const BookmarkNode* bookmarkBarNode = model->GetBookmarkBarNode();
+  const BookmarkNode* bookmarkBarNode = model->bookmark_bar_node();
   EXPECT_TRUE(bookmarkBarNode);
-  const BookmarkNode* node1 = model->AddGroup(bookmarkBarNode, 0,
-                                              ASCIIToUTF16("one"));
+  const BookmarkNode* node1 = model->AddFolder(bookmarkBarNode, 0,
+                                               ASCIIToUTF16("one"));
   EXPECT_TRUE(node1);
-  const BookmarkNode* node2 = model->AddGroup(bookmarkBarNode, 1,
-                                              ASCIIToUTF16("two"));
+  const BookmarkNode* node2 = model->AddFolder(bookmarkBarNode, 1,
+                                               ASCIIToUTF16("two"));
   EXPECT_TRUE(node2);
-  const BookmarkNode* node2_1 = model->AddGroup(node2, 0,
-                                                ASCIIToUTF16("two dot one"));
+  const BookmarkNode* node2_1 = model->AddFolder(node2, 0,
+                                                 ASCIIToUTF16("two dot one"));
   EXPECT_TRUE(node2_1);
-  const BookmarkNode* node3 = model->AddGroup(bookmarkBarNode, 2,
-                                              ASCIIToUTF16("three"));
+  const BookmarkNode* node3 = model->AddFolder(bookmarkBarNode, 2,
+                                               ASCIIToUTF16("three"));
   EXPECT_TRUE(node3);
 
   BookmarkBubbleController* controller = ControllerForNode(node1);
@@ -448,7 +459,7 @@ TEST_F(BookmarkBubbleControllerTest, TestMenuIndentation) {
 TEST_F(BookmarkBubbleControllerTest, BubbleGoesAwayOnNewTab) {
 
   BookmarkModel* model = GetBookmarkModel();
-  const BookmarkNode* node = model->AddURL(model->GetBookmarkBarNode(),
+  const BookmarkNode* node = model->AddURL(model->bookmark_bar_node(),
                                            0,
                                            ASCIIToUTF16("Bookie markie title"),
                                            GURL("http://www.google.com"));
@@ -459,15 +470,15 @@ TEST_F(BookmarkBubbleControllerTest, BubbleGoesAwayOnNewTab) {
   EXPECT_FALSE(IsWindowClosing());
 
   // We can't actually create a new tab here, e.g.
-  //   helper_.browser()->AddTabWithURL(...);
+  //   browser()->AddTabWithURL(...);
   // Many of our browser objects (Browser, Profile, RequestContext)
   // are "just enough" to run tests without being complete.  Instead
   // we fake the notification that would be triggered by a tab
-  // creation.
-  NotificationService::current()->Notify(
-      NotificationType::TAB_CONTENTS_CONNECTED,
-      Source<TabContentsDelegate>(NULL),
-      Details<TabContents>(NULL));
+  // creation. See TabContents::NotifyConnected().
+  content::NotificationService::current()->Notify(
+      content::NOTIFICATION_WEB_CONTENTS_CONNECTED,
+      content::Source<WebContents>(NULL),
+      content::NotificationService::NoDetails());
 
   // Confirm bubble going bye-bye.
   EXPECT_TRUE(IsWindowClosing());
