@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,12 +6,15 @@
 
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_editor_base_controller.h"
 
+#include "base/auto_reset.h"
 #include "base/logging.h"
+#include "base/mac/bundle_locations.h"
 #include "base/mac/mac_util.h"
 #include "base/sys_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/profiles/profile.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_all_tabs_controller.h"
+#import "chrome/browser/ui/cocoa/bookmarks/bookmark_cell_single_line.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_editor_controller.h"
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_tree_browser_cell.h"
 #import "chrome/browser/ui/cocoa/browser_window_controller.h"
@@ -50,15 +53,20 @@
 // the selection path thereto.
 - (NSIndexPath*)selectionPathForNode:(const BookmarkNode*)node;
 
+// Implementation of getExpandedNodes. See description in header for details.
+- (void)getExpandedNodes:(BookmarkExpandedStateTracker::Nodes*)nodes
+                  folder:(BookmarkFolderInfo*)info
+                    path:(std::vector<NSUInteger>*)path
+                    root:(id)root;
 @end
 
 // static; implemented for each platform.  Update this function for new
 // classes derived from BookmarkEditorBaseController.
-void BookmarkEditor::Show(gfx::NativeWindow parent_hwnd,
-                          Profile* profile,
-                          const BookmarkNode* parent,
-                          const EditDetails& details,
-                          Configuration configuration) {
+void BookmarkEditor::ShowNative(gfx::NativeWindow parent_hwnd,
+                                Profile* profile,
+                                const BookmarkNode* parent,
+                                const EditDetails& details,
+                                Configuration configuration) {
   BookmarkEditorBaseController* controller = nil;
   if (details.type == EditDetails::NEW_FOLDER) {
     controller = [[BookmarkAllTabsController alloc]
@@ -85,7 +93,7 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
         importing_(false)
   { }
 
-  virtual void Loaded(BookmarkModel* model) {
+  virtual void Loaded(BookmarkModel* model, bool ids_reassigned) OVERRIDE {
     [controller_ modelChangedPreserveSelection:YES];
   }
 
@@ -93,14 +101,14 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
                                  const BookmarkNode* old_parent,
                                  int old_index,
                                  const BookmarkNode* new_parent,
-                                 int new_index) {
+                                 int new_index) OVERRIDE {
     if (!importing_ && new_parent->GetChild(new_index)->is_folder())
       [controller_ modelChangedPreserveSelection:YES];
   }
 
   virtual void BookmarkNodeAdded(BookmarkModel* model,
                                  const BookmarkNode* parent,
-                                 int index) {
+                                 int index) OVERRIDE {
     if (!importing_ && parent->GetChild(index)->is_folder())
       [controller_ modelChangedPreserveSelection:YES];
   }
@@ -108,36 +116,37 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
   virtual void BookmarkNodeRemoved(BookmarkModel* model,
                                    const BookmarkNode* parent,
                                    int old_index,
-                                   const BookmarkNode* node) {
+                                   const BookmarkNode* node) OVERRIDE {
     [controller_ nodeRemoved:node fromParent:parent];
     if (node->is_folder())
       [controller_ modelChangedPreserveSelection:NO];
   }
 
   virtual void BookmarkNodeChanged(BookmarkModel* model,
-                                   const BookmarkNode* node) {
+                                   const BookmarkNode* node) OVERRIDE {
     if (!importing_ && node->is_folder())
       [controller_ modelChangedPreserveSelection:YES];
   }
 
-  virtual void BookmarkNodeChildrenReordered(BookmarkModel* model,
-                                             const BookmarkNode* node) {
+  virtual void BookmarkNodeChildrenReordered(
+      BookmarkModel* model,
+      const BookmarkNode* node) OVERRIDE {
     if (!importing_)
       [controller_ modelChangedPreserveSelection:YES];
   }
 
-  virtual void BookmarkNodeFavIconLoaded(BookmarkModel* model,
-                                         const BookmarkNode* node) {
+  virtual void BookmarkNodeFaviconChanged(BookmarkModel* model,
+                                          const BookmarkNode* node) OVERRIDE {
     // I care nothing for these 'favicons': I only show folders.
   }
 
-  virtual void BookmarkImportBeginning(BookmarkModel* model) {
+  virtual void BookmarkImportBeginning(BookmarkModel* model) OVERRIDE {
     importing_ = true;
   }
 
   // Invoked after a batch import finishes.  This tells observers to update
   // themselves if they were waiting for the update to finish.
-  virtual void BookmarkImportEnding(BookmarkModel* model) {
+  virtual void BookmarkImportEnding(BookmarkModel* model) OVERRIDE {
     importing_ = false;
     [controller_ modelChangedPreserveSelection:YES];
   }
@@ -161,7 +170,7 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
                    profile:(Profile*)profile
                     parent:(const BookmarkNode*)parent
              configuration:(BookmarkEditor::Configuration)configuration {
-  NSString* nibpath = [base::mac::MainAppBundle()
+  NSString* nibpath = [base::mac::FrameworkBundle()
                         pathForResource:nibName
                                  ofType:@"nib"];
   if ((self = [super initWithWindowNibPath:nibpath owner:self])) {
@@ -238,6 +247,8 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
 }
 
 - (IBAction)ok:(id)sender {
+  NSWindow* window = [self window];
+  [window makeFirstResponder:window];
   // At least one of these two functions should be provided by derived classes.
   BOOL hasWillCommit = [self respondsToSelector:@selector(willCommit)];
   BOOL hasDidCommit = [self respondsToSelector:@selector(didCommit)];
@@ -256,7 +267,7 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
       shouldContinue = [hasDidContinue boolValue];
   }
   if (shouldContinue)
-    [NSApp endSheet:[self window]];
+    [NSApp endSheet:window];
 }
 
 - (IBAction)cancel:(id)sender {
@@ -281,6 +292,10 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
   return profile_->GetBookmarkModel();
 }
 
+- (Profile*)profile {
+  return profile_;
+}
+
 - (const BookmarkNode*)parentNode {
   return parentNode_;
 }
@@ -329,6 +344,53 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
   return selectedNode;
 }
 
+- (void)expandNodes:(const BookmarkExpandedStateTracker::Nodes&)nodes {
+  id treeControllerRoot = [folderTreeController_ arrangedObjects];
+  for (BookmarkExpandedStateTracker::Nodes::const_iterator i = nodes.begin();
+       i != nodes.end(); ++i) {
+    NSIndexPath* path = [self selectionPathForNode:*i];
+    id folderNode = [treeControllerRoot descendantNodeAtIndexPath:path];
+    [folderTreeView_ expandItem:folderNode];
+  }
+}
+
+- (BookmarkExpandedStateTracker::Nodes)getExpandedNodes {
+  BookmarkExpandedStateTracker::Nodes nodes;
+  std::vector<NSUInteger> path;
+  NSArray* folderNodes = [self folderTreeArray];
+  NSUInteger i = 0;
+  for (BookmarkFolderInfo* folder in folderNodes) {
+    path.push_back(i);
+    [self getExpandedNodes:&nodes
+                    folder:folder
+                      path:&path
+                      root:[folderTreeController_ arrangedObjects]];
+    path.clear();
+    ++i;
+  }
+  return nodes;
+}
+
+- (void)getExpandedNodes:(BookmarkExpandedStateTracker::Nodes*)nodes
+                  folder:(BookmarkFolderInfo*)folder
+                    path:(std::vector<NSUInteger>*)path
+                    root:(id)root {
+  NSIndexPath* indexPath = [NSIndexPath indexPathWithIndexes:&(path->front())
+                                                      length:path->size()];
+  id node = [root descendantNodeAtIndexPath:indexPath];
+  if (![folderTreeView_ isItemExpanded:node])
+    return;
+  nodes->insert([folder folderNode]);
+  NSArray* children = [folder children];
+  NSUInteger i = 0;
+  for (BookmarkFolderInfo* childFolder in children) {
+    path->push_back(i);
+    [self getExpandedNodes:nodes folder:childFolder path:path root:root];
+    path->pop_back();
+    ++i;
+  }
+}
+
 - (NSArray*)folderTreeArray {
   return folderTreeArray_.get();
 }
@@ -364,7 +426,7 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
   while (node != rootNode) {
     DCHECK(node);
     nodeStack.push(node);
-    node = node->GetParent();
+    node = node->parent();
   }
   NSUInteger stackSize = nodeStack.size();
 
@@ -392,10 +454,10 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
 
 - (NSMutableArray*)addChildFoldersFromNode:(const BookmarkNode*)node {
   NSMutableArray* childFolders = nil;
-  int childCount = node->GetChildCount();
+  int childCount = node->child_count();
   for (int i = 0; i < childCount; ++i) {
     const BookmarkNode* childNode = node->GetChild(i);
-    if (childNode->type() != BookmarkNode::URL) {
+    if (childNode->is_folder() && childNode->IsVisible()) {
       NSString* childName = base::SysUTF16ToNSString(childNode->GetTitle());
       NSMutableArray* children = [self addChildFoldersFromNode:childNode];
       BookmarkFolderInfo* folderInfo =
@@ -423,6 +485,8 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
 }
 
 - (void)modelChangedPreserveSelection:(BOOL)preserve {
+  if (creatingNewFolders_)
+    return;
   const BookmarkNode* selectedNode = [self selectedNode];
   [self buildFolderTree];
   if (preserve &&
@@ -435,7 +499,7 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
          fromParent:(const BookmarkNode*)parent {
   if (node->is_folder()) {
     if (parentNode_ == node || parentNode_->HasAncestor(node)) {
-      parentNode_ = [self bookmarkModel]->GetBookmarkBarNode();
+      parentNode_ = [self bookmarkModel]->bookmark_bar_node();
       if (configuration_ != BookmarkEditor::SHOW_TREE) {
         // The user can't select a different folder, so just close up shop.
         [self cancel:self];
@@ -463,16 +527,11 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
     if ([subFolderInfo newFolder]) {
       BookmarkModel* model = [self bookmarkModel];
       const BookmarkNode* newFolder =
-      model->AddGroup(parentNode, i,
-                      base::SysNSStringToUTF16([subFolderInfo folderName]));
+          model->AddFolder(parentNode, i,
+              base::SysNSStringToUTF16([subFolderInfo folderName]));
       // Update our dictionary with the actual folder node just created.
       [subFolderInfo setFolderNode:newFolder];
       [subFolderInfo setNewFolder:NO];
-      // If the newly created folder was selected, update the selection path.
-      if (subFolderInfo == selectedFolderInfo) {
-        NSIndexPath* selectionPath = [self selectionPathForNode:newFolder];
-        [self setTableSelectionPath:selectionPath];
-      }
     }
     [self createNewFoldersForFolder:subFolderInfo
                  selectedFolderInfo:selectedFolderInfo];
@@ -486,7 +545,7 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
   if (parentInfo) {
     NSIndexPath* selection = [self selectedIndexPath];
     NSString* newFolderName =
-        l10n_util::GetNSStringWithFixup(IDS_BOOMARK_EDITOR_NEW_FOLDER_NAME);
+        l10n_util::GetNSStringWithFixup(IDS_BOOKMARK_EDITOR_NEW_FOLDER_NAME);
     BookmarkFolderInfo* folderInfo =
         [BookmarkFolderInfo bookmarkFolderInfoWithFolderName:newFolderName];
     [self willChangeValueForKey:@"folderTreeArray"];
@@ -507,21 +566,27 @@ class BookmarkEditorBaseControllerBridge : public BookmarkModelObserver {
     [self setTableSelectionPath:selection];
     NSInteger row = [folderTreeView_ selectedRow];
     DCHECK(row >= 0);
+
+    // Put the cell into single-line mode before putting it into edit mode.
+    // TODO(kushi.p): Remove this when the project hits a 10.6+ only state.
+    NSCell* folderCell = [folderTreeView_ preparedCellAtColumn:0 row:row];
+    if ([folderCell
+          respondsToSelector:@selector(setUsesSingleLineMode:)]) {
+      [folderCell setUsesSingleLineMode:YES];
+    }
+
     [folderTreeView_ editColumn:0 row:row withEvent:nil select:YES];
   }
 }
 
 - (void)createNewFolders {
-  // Turn off notifications while "importing" folders (as created in the sheet).
-  observer_->BookmarkImportBeginning([self bookmarkModel]);
+  AutoReset<BOOL> creatingNewFoldersSetter(&creatingNewFolders_, YES);
   // Scan the tree looking for nodes marked 'newFolder' and create those nodes.
   NSArray* folderTreeArray = [self folderTreeArray];
   for (BookmarkFolderInfo *folderInfo in folderTreeArray) {
     [self createNewFoldersForFolder:folderInfo
                  selectedFolderInfo:[self selectedFolder]];
   }
-  // Notifications back on.
-  observer_->BookmarkImportEnding([self bookmarkModel]);
 }
 
 #pragma mark For Unit Test Use Only
