@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,22 +6,33 @@
 
 #include <vector>
 
+#include "base/mac/bundle_locations.h"
 #import "base/mac/mac_util.h"
 #include "base/sys_string_conversions.h"
+#include "chrome/browser/content_settings/cookie_settings.h"
+#include "chrome/browser/content_settings/tab_specific_content_settings.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
+#include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/ui/cocoa/constrained_window_mac.h"
+#import "chrome/browser/ui/cocoa/content_settings/cookie_details_view_controller.h"
 #import "chrome/browser/ui/cocoa/vertical_gradient_view.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_source.h"
+#include "chrome/browser/ui/collected_cookies_infobar_delegate.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "chrome/common/pref_names.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_source.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
 #include "skia/ext/skia_utils_mac.h"
-#include "third_party/apple/ImageAndTextCell.h"
-#include "third_party/skia/include/core/SkBitmap.h"
 #import "third_party/GTM/AppKit/GTMNSAnimation+Duration.h"
 #import "third_party/GTM/AppKit/GTMUILocalizerAndLayoutTweaker.h"
+#include "third_party/apple_sample_code/ImageAndTextCell.h"
+#include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/l10n/l10n_util_mac.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/image/image.h"
 
 namespace {
 // Colors for the infobar.
@@ -31,8 +42,11 @@ const double kBannerGradientColorBottom[3] =
     {250.0 / 255.0, 230.0 / 255.0, 145.0 / 255.0};
 const double kBannerStrokeColor = 135.0 / 255.0;
 
-// Minimal height for the collected cookies dialog.
-const CGFloat kMinCollectedCookiesViewHeight = 116;
+enum TabViewItemIndices {
+  kAllowedCookiesTabIndex = 0,
+  kBlockedCookiesTabIndex
+};
+
 } // namespace
 
 #pragma mark Bridge between the constrained window delegate and the sheet
@@ -66,23 +80,21 @@ const CGFloat kMinCollectedCookiesViewHeight = 116;
 #pragma mark Constrained window delegate
 
 CollectedCookiesMac::CollectedCookiesMac(NSWindow* parent,
-                                         TabContents* tab_contents)
+                                         TabContentsWrapper* wrapper)
     : ConstrainedWindowMacDelegateCustomSheet(
         [[[CollectedCookiesSheetBridge alloc]
             initWithCollectedCookiesMac:this] autorelease],
-        @selector(sheetDidEnd:returnCode:contextInfo:)),
-      tab_contents_(tab_contents) {
-  TabSpecificContentSettings* content_settings =
-      tab_contents->GetTabSpecificContentSettings();
-  registrar_.Add(this, NotificationType::COLLECTED_COOKIES_SHOWN,
-                 Source<TabSpecificContentSettings>(content_settings));
+        @selector(sheetDidEnd:returnCode:contextInfo:)) {
+  TabSpecificContentSettings* content_settings = wrapper->content_settings();
+  registrar_.Add(this, chrome::NOTIFICATION_COLLECTED_COOKIES_SHOWN,
+                 content::Source<TabSpecificContentSettings>(content_settings));
 
   sheet_controller_ = [[CollectedCookiesWindowController alloc]
-      initWithTabContents:tab_contents];
+      initWithTabContentsWrapper:wrapper];
 
   set_sheet([sheet_controller_ window]);
 
-  window_ = tab_contents->CreateConstrainedDialog(this);
+  window_ = new ConstrainedWindowMac(wrapper, this);
 }
 
 CollectedCookiesMac::~CollectedCookiesMac() {
@@ -97,12 +109,10 @@ void CollectedCookiesMac::DeleteDelegate() {
   delete this;
 }
 
-void CollectedCookiesMac::Observe(NotificationType type,
-                                  const NotificationSource& source,
-                                  const NotificationDetails& details) {
-  DCHECK(type == NotificationType::COLLECTED_COOKIES_SHOWN);
-  DCHECK_EQ(Source<TabSpecificContentSettings>(source).ptr(),
-            tab_contents_->GetTabSpecificContentSettings());
+void CollectedCookiesMac::Observe(int type,
+                                  const content::NotificationSource& source,
+                                  const content::NotificationDetails& details) {
+  DCHECK(type == chrome::NOTIFICATION_COLLECTED_COOKIES_SHOWN);
   window_->CloseConstrainedWindow();
 }
 
@@ -131,16 +141,15 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
 @synthesize allowedTreeController = allowedTreeController_;
 @synthesize blockedTreeController = blockedTreeController_;
 
-- (id)initWithTabContents:(TabContents*)tabContents {
-  DCHECK(tabContents);
-  infoBarVisible_ = NO;
-  tabContents_ = tabContents;
+- (id)initWithTabContentsWrapper:(TabContentsWrapper*)wrapper {
+  DCHECK(wrapper);
 
   NSString* nibpath =
-      [base::mac::MainAppBundle() pathForResource:@"CollectedCookies"
-                                          ofType:@"nib"];
+      [base::mac::FrameworkBundle() pathForResource:@"CollectedCookies"
+                                             ofType:@"nib"];
   if ((self = [super initWithWindowNibPath:nibpath owner:self])) {
-    [self loadTreeModelFromTabContents];
+    wrapper_ = wrapper;
+    [self loadTreeModelFromTabContentsWrapper];
 
     animation_.reset([[NSViewAnimation alloc] init]);
     [animation_ setAnimationBlockingMode:NSAnimationNonblocking];
@@ -176,32 +185,42 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
   [infoBar_ setStrokeColor:bannerStrokeColor];
 
   // Change the label of the blocked cookies part if necessary.
-  if (tabContents_->profile()->GetHostContentSettingsMap()->
-      BlockThirdPartyCookies()) {
+  Profile* profile = wrapper_->profile();
+  if (profile->GetPrefs()->GetBoolean(prefs::kBlockThirdPartyCookies)) {
     [blockedCookiesText_ setStringValue:l10n_util::GetNSString(
         IDS_COLLECTED_COOKIES_BLOCKED_THIRD_PARTY_BLOCKING_ENABLED)];
     CGFloat textDeltaY = [GTMUILocalizerAndLayoutTweaker
         sizeToFitFixedWidthTextField:blockedCookiesText_];
 
-    // Shrink the upper custom view.
-    NSView* upperContentView = [[splitView_ subviews] objectAtIndex:0];
-    NSRect frame = [upperContentView frame];
-    [splitView_ setPosition:(frame.size.height - textDeltaY/2.0)
-           ofDividerAtIndex:0];
-
-    // Shrink the lower outline view.
-    frame = [lowerScrollView_ frame];
+    // Shrink the blocked cookies outline view.
+    NSRect frame = [blockedScrollView_ frame];
     frame.size.height -= textDeltaY;
-    [lowerScrollView_ setFrame:frame];
+    [blockedScrollView_ setFrame:frame];
 
     // Move the label down so it actually fits.
     frame = [blockedCookiesText_ frame];
     frame.origin.y -= textDeltaY;
     [blockedCookiesText_ setFrame:frame];
   }
+
+  detailsViewController_.reset([[CookieDetailsViewController alloc] init]);
+
+  NSView* detailView = [detailsViewController_.get() view];
+  NSRect viewFrameRect = [cookieDetailsViewPlaceholder_ frame];
+  [[detailsViewController_.get() view] setFrame:viewFrameRect];
+  [[cookieDetailsViewPlaceholder_ superview]
+      replaceSubview:cookieDetailsViewPlaceholder_
+                with:detailView];
+
+  [self tabView:tabView_ didSelectTabViewItem:[tabView_ selectedTabViewItem]];
 }
 
 - (void)windowWillClose:(NSNotification*)notif {
+  if (contentSettingsChanged_) {
+    InfoBarTabHelper* infobar_helper = wrapper_->infobar_tab_helper();
+    infobar_helper->AddInfoBar(
+        new CollectedCookiesInfoBarDelegate(infobar_helper));
+  }
   [allowedOutlineView_ setDelegate:nil];
   [blockedOutlineView_ setDelegate:nil];
   [animation_ stopAnimation];
@@ -224,11 +243,11 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
         CookieTreeNode::DetailedInfo::TYPE_ORIGIN) {
       continue;
     }
+    Profile* profile = wrapper_->profile();
     CookieTreeOriginNode* origin_node =
         static_cast<CookieTreeOriginNode*>(cookie);
-    origin_node->CreateContentException(
-        tabContents_->profile()->GetHostContentSettingsMap(),
-        setting);
+    origin_node->CreateContentException(CookieSettings::GetForProfile(profile),
+                                        setting);
     if (!lastDomain.empty())
       multipleDomainsChanged = YES;
     lastDomain = origin_node->GetTitle();
@@ -237,6 +256,7 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
     [self showInfoBarForMultipleDomainsAndSetting:setting];
   else
     [self showInfoBarForDomain:lastDomain setting:setting];
+  contentSettingsChanged_ = YES;
 }
 
 - (IBAction)allowOrigin:(id)sender {
@@ -254,20 +274,6 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
       forTreeController:allowedTreeController_];
 }
 
-- (CGFloat)    splitView:(NSSplitView *)sender
-  constrainMinCoordinate:(CGFloat)proposedMin
-             ofSubviewAt:(NSInteger)offset {
-  return proposedMin + kMinCollectedCookiesViewHeight;
-}
-- (CGFloat)    splitView:(NSSplitView *)sender
-  constrainMaxCoordinate:(CGFloat)proposedMax
-             ofSubviewAt:(NSInteger)offset {
-  return proposedMax - kMinCollectedCookiesViewHeight;
-}
-- (BOOL)splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview {
-  return YES;
-}
-
 - (CocoaCookieTreeNode*)cocoaAllowedTreeModel {
   return cocoaAllowedTreeModel_.get();
 }
@@ -350,9 +356,8 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
 
 // Initializes the |allowedTreeModel_| and |blockedTreeModel_|, and builds
 // the |cocoaAllowedTreeModel_| and |cocoaBlockedTreeModel_|.
-- (void)loadTreeModelFromTabContents {
-  TabSpecificContentSettings* content_settings =
-      tabContents_->GetTabSpecificContentSettings();
+- (void)loadTreeModelFromTabContentsWrapper {
+  TabSpecificContentSettings* content_settings = wrapper_->content_settings();
   allowedTreeModel_.reset(content_settings->GetAllowedCookiesTreeModel());
   blockedTreeModel_.reset(content_settings->GetBlockedCookiesTreeModel());
 
@@ -449,16 +454,13 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
   NSWindow* sheet = [self window];
   NSRect sheetFrame = [sheet frame];
   NSRect infoBarFrame = [infoBar_ frame];
-  NSRect splitViewFrame = [splitView_ frame];
+  NSRect tabViewFrame = [tabView_ frame];
 
   // Calculate the end position of the info bar and set it to its start
   // position.
   infoBarFrame.origin.y = NSHeight(sheetFrame);
   infoBarFrame.size.width = NSWidth(sheetFrame);
-  NSRect infoBarStartFrame = infoBarFrame;
-  infoBarStartFrame.origin.y += NSHeight(infoBarFrame);
-  infoBarStartFrame.size.height = 0.0;
-  [infoBar_ setFrame:infoBarStartFrame];
+  [infoBar_ setFrame:infoBarFrame];
   [[[self window] contentView] addSubview:infoBar_];
 
   // Calculate the new position of the sheet.
@@ -471,14 +473,12 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
           infoBar_, NSViewAnimationTargetKey,
           [NSValue valueWithRect:infoBarFrame],
               NSViewAnimationEndFrameKey,
-          [NSValue valueWithRect:infoBarStartFrame],
-              NSViewAnimationStartFrameKey,
           nil]];
-  // Make sure the split view ends up in the right position.
+  // Make sure the tab view ends up in the right position.
   [animations addObject:
       [NSDictionary dictionaryWithObjectsAndKeys:
-          splitView_, NSViewAnimationTargetKey,
-          [NSValue valueWithRect:splitViewFrame],
+          tabView_, NSViewAnimationTargetKey,
+          [NSValue valueWithRect:tabViewFrame],
               NSViewAnimationEndFrameKey,
           nil]];
 
@@ -497,4 +497,21 @@ void CollectedCookiesMac::OnSheetDidEnd(NSWindow* sheet) {
   [animation_ startAnimation];
 }
 
+- (void)         tabView:(NSTabView*)tabView
+    didSelectTabViewItem:(NSTabViewItem*)tabViewItem {
+  NSTreeController* treeController = nil;
+  switch ([tabView indexOfTabViewItem:tabViewItem]) {
+    case kAllowedCookiesTabIndex:
+      treeController = allowedTreeController_;
+      break;
+    case kBlockedCookiesTabIndex:
+      treeController = blockedTreeController_;
+      break;
+    default:
+      NOTREACHED();
+      return;
+  }
+  [detailsViewController_ configureBindingsForTreeController:treeController];
+}
+
 @end
