@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,19 +8,18 @@
 #include "base/logging.h"
 #include "base/sys_string_conversions.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/blocked_content_container.h"
-#include "chrome/browser/content_setting_bubble_model.h"
 #include "chrome/browser/content_settings/host_content_settings_map.h"
-#include "chrome/browser/plugin_updater.h"
+#include "chrome/browser/ui/content_settings/content_setting_bubble_model.h"
 #import "chrome/browser/ui/cocoa/hyperlink_button_cell.h"
 #import "chrome/browser/ui/cocoa/info_bubble_view.h"
 #import "chrome/browser/ui/cocoa/l10n_util.h"
-#import "chrome/browser/ui/cocoa/options/content_settings_dialog_controller.h"
+#include "content/public/browser/plugin_service.h"
 #include "grit/generated_resources.h"
 #include "skia/ext/skia_utils_mac.h"
 #import "third_party/GTM/AppKit/GTMUILocalizerAndLayoutTweaker.h"
 #include "ui/base/l10n/l10n_util.h"
-#include "webkit/glue/plugins/plugin_list.h"
+
+using content::PluginService;
 
 namespace {
 
@@ -121,22 +120,25 @@ NSTextField* LabelWithFrame(NSString* text, const NSRect& frame) {
   scoped_ptr<ContentSettingBubbleModel> model(contentSettingBubbleModel);
   DCHECK(model.get());
 
-  NSString* const nibPaths[] = {
-    @"ContentBlockedCookies",
-    @"ContentBlockedImages",
-    @"ContentBlockedJavaScript",
-    @"ContentBlockedPlugins",
-    @"ContentBlockedPopups",
-    @"ContentBubbleGeolocation",
-    @"",  // Notifications do not have a bubble.
-  };
-  COMPILE_ASSERT(arraysize(nibPaths) == CONTENT_SETTINGS_NUM_TYPES,
-                 nibPaths_requires_an_entry_for_every_setting_type);
   const int settingsType = model->content_type();
-  // Nofifications do not have a bubble.
-  CHECK_NE(settingsType, CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
-  DCHECK_LT(settingsType, CONTENT_SETTINGS_NUM_TYPES);
-  if ((self = [super initWithWindowNibPath:nibPaths[settingsType]
+  NSString* nibPath = @"";
+  switch (settingsType) {
+    case CONTENT_SETTINGS_TYPE_COOKIES:
+      nibPath = @"ContentBlockedCookies"; break;
+    case CONTENT_SETTINGS_TYPE_IMAGES:
+      nibPath = @"ContentBlockedImages"; break;
+    case CONTENT_SETTINGS_TYPE_JAVASCRIPT:
+      nibPath = @"ContentBlockedJavaScript"; break;
+    case CONTENT_SETTINGS_TYPE_PLUGINS:
+      nibPath = @"ContentBlockedPlugins"; break;
+    case CONTENT_SETTINGS_TYPE_POPUPS:
+      nibPath = @"ContentBlockedPopups"; break;
+    case CONTENT_SETTINGS_TYPE_GEOLOCATION:
+      nibPath = @"ContentBlockedGeolocation"; break;
+    default:
+      NOTREACHED();
+  }
+  if ((self = [super initWithWindowNibPath:nibPath
                               parentWindow:parentWindow
                                 anchoredAt:anchoredAt])) {
     contentSettingBubbleModel_.reset(model.release());
@@ -213,15 +215,18 @@ NSTextField* LabelWithFrame(NSString* text, const NSRect& frame) {
   // Cell must be set immediately after construction.
   [button setCell:cell.get()];
 
-  // If the link text is too long, clamp it.
-  [button sizeToFit];
-  int maxWidth = NSWidth([[self bubble] frame]) - 2 * NSMinX(referenceFrame);
+  // Size to fit the button and add a little extra padding for the small-text
+  // hyperlink button, which sizeToFit gets wrong.
+  [GTMUILocalizerAndLayoutTweaker sizeToFitView:button];
   NSRect buttonFrame = [button frame];
-  if (NSWidth(buttonFrame) > maxWidth) {
-    buttonFrame.size.width = maxWidth;
-    [button setFrame:buttonFrame];
-  }
+  buttonFrame.size.width += 2;
 
+  // If the link text is too long, clamp it.
+  int maxWidth = NSWidth([[self bubble] frame]) - 2 * NSMinX(referenceFrame);
+  if (NSWidth(buttonFrame) > maxWidth)
+    buttonFrame.size.width = maxWidth;
+
+  [button setFrame:buttonFrame];
   [button setTarget:self];
   [button setAction:@selector(popupLinkClicked:)];
   return button;
@@ -242,7 +247,7 @@ NSTextField* LabelWithFrame(NSString* text, const NSRect& frame) {
     for (std::set<std::string>::iterator it = plugins.begin();
          it != plugins.end(); ++it) {
       NSString* name = SysUTF16ToNSString(
-          NPAPI::PluginList::Singleton()->GetPluginGroupName(*it));
+          PluginService::GetInstance()->GetPluginGroupName(*it));
       if ([name length] == 0)
         name = base::SysUTF8ToNSString(*it);
       [pluginArray addObject:name];
