@@ -1,80 +1,83 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #import "chrome/browser/ui/cocoa/download/download_item_cell.h"
 
 #include "base/sys_string_conversions.h"
-#include "chrome/browser/download/download_item.h"
 #include "chrome/browser/download/download_item_model.h"
-#include "chrome/browser/download/download_manager.h"
 #include "chrome/browser/download/download_util.h"
-#import "chrome/browser/themes/browser_theme_provider.h"
-#import "chrome/browser/ui/cocoa/download/download_item_cell.h"
+#import "chrome/browser/themes/theme_service.h"
+#import "chrome/browser/ui/cocoa/download/background_theme.h"
 #import "chrome/browser/ui/cocoa/image_utils.h"
 #import "chrome/browser/ui/cocoa/themed_window.h"
-#include "gfx/canvas_skia_paint.h"
+#include "content/public/browser/download_item.h"
+#include "content/public/browser/download_manager.h"
 #include "grit/theme_resources.h"
 #import "third_party/GTM/AppKit/GTMNSAnimation+Duration.h"
 #import "third_party/GTM/AppKit/GTMNSColor+Luminance.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/text/text_elider.h"
+#include "ui/gfx/canvas_skia_paint.h"
+#include "ui/gfx/scoped_ns_graphics_context_save_gstate_mac.h"
 
-namespace {
-
-// Distance from top border to icon
+// Distance from top border to icon.
 const CGFloat kImagePaddingTop = 7;
 
-// Distance from left border to icon
+// Distance from left border to icon.
 const CGFloat kImagePaddingLeft = 9;
 
-// Width of icon
+// Width of icon.
 const CGFloat kImageWidth = 16;
 
-// Height of icon
+// Height of icon.
 const CGFloat kImageHeight = 16;
 
-// x coordinate of download name string, in view coords
+// x coordinate of download name string, in view coords.
 const CGFloat kTextPosLeft = kImagePaddingLeft +
     kImageWidth + download_util::kSmallProgressIconOffset;
 
-// Distance from end of download name string to dropdown area
+// Distance from end of download name string to dropdown area.
 const CGFloat kTextPaddingRight = 3;
 
 // y coordinate of download name string, in view coords, when status message
-// is visible
+// is visible.
 const CGFloat kPrimaryTextPosTop = 3;
 
 // y coordinate of download name string, in view coords, when status message
-// is not visible
+// is not visible.
 const CGFloat kPrimaryTextOnlyPosTop = 10;
 
-// y coordinate of status message, in view coords
+// y coordinate of status message, in view coords.
 const CGFloat kSecondaryTextPosTop = 18;
 
-// Grey value of status text
+// Grey value of status text.
 const CGFloat kSecondaryTextColor = 0.5;
 
 // Width of dropdown area on the right (includes 1px for the border on each
 // side).
 const CGFloat kDropdownAreaWidth = 14;
 
-// Width of dropdown arrow
+// Width of dropdown arrow.
 const CGFloat kDropdownArrowWidth = 5;
 
-// Height of dropdown arrow
+// Height of dropdown arrow.
 const CGFloat kDropdownArrowHeight = 3;
 
 // Vertical displacement of dropdown area, relative to the "centered" position.
 const CGFloat kDropdownAreaY = -2;
 
-// Duration of the two-lines-to-one-line animation, in seconds
+// Duration of the two-lines-to-one-line animation, in seconds.
+NSTimeInterval kShowStatusDuration = 0.3;
 NSTimeInterval kHideStatusDuration = 0.3;
 
-// Duration of the 'download complete' animation, in seconds
+// Duration of the 'download complete' animation, in seconds.
 const int kCompleteAnimationDuration = 2.5;
 
-}
+// Duration of the 'download interrupted' animation, in seconds.
+const int kInterruptedAnimationDuration = 2.5;
+
+using content::DownloadItem;
 
 // This is a helper class to animate the fading out of the status text.
 @interface DownloadItemCellAnimation : NSAnimation {
@@ -85,90 +88,17 @@ const int kCompleteAnimationDuration = 2.5;
                 animationCurve:(NSAnimationCurve)animationCurve;
 @end
 
-class BackgroundTheme : public ui::ThemeProvider {
-public:
-  BackgroundTheme(ui::ThemeProvider* provider);
-
-  virtual void Init(Profile* profile) { }
-  virtual SkBitmap* GetBitmapNamed(int id) const { return nil; }
-  virtual SkColor GetColor(int id) const { return SkColor(); }
-  virtual bool GetDisplayProperty(int id, int* result) const { return false; }
-  virtual bool ShouldUseNativeFrame() const { return false; }
-  virtual bool HasCustomImage(int id) const { return false; }
-  virtual RefCountedMemory* GetRawData(int id) const { return NULL; }
-  virtual NSImage* GetNSImageNamed(int id, bool allow_default) const;
-  virtual NSColor* GetNSImageColorNamed(int id, bool allow_default) const;
-  virtual NSColor* GetNSColor(int id, bool allow_default) const;
-  virtual NSColor* GetNSColorTint(int id, bool allow_default) const;
-  virtual NSGradient* GetNSGradient(int id) const;
-
-private:
-  ui::ThemeProvider* provider_;
-  scoped_nsobject<NSGradient> buttonGradient_;
-  scoped_nsobject<NSGradient> buttonPressedGradient_;
-  scoped_nsobject<NSColor> borderColor_;
-};
-
-BackgroundTheme::BackgroundTheme(ui::ThemeProvider* provider) :
-    provider_(provider) {
-  NSColor* bgColor = [NSColor colorWithCalibratedRed:241/255.0
-                                               green:245/255.0
-                                                blue:250/255.0
-                                               alpha:77/255.0];
-  NSColor* clickedColor = [NSColor colorWithCalibratedRed:239/255.0
-                                                    green:245/255.0
-                                                     blue:252/255.0
-                                                    alpha:51/255.0];
-
-  borderColor_.reset(
-      [[NSColor colorWithCalibratedWhite:0 alpha:36/255.0] retain]);
-  buttonGradient_.reset([[NSGradient alloc]
-      initWithColors:[NSArray arrayWithObject:bgColor]]);
-  buttonPressedGradient_.reset([[NSGradient alloc]
-      initWithColors:[NSArray arrayWithObject:clickedColor]]);
-}
-
-NSImage* BackgroundTheme::GetNSImageNamed(int id, bool allow_default) const {
-  return nil;
-}
-
-NSColor* BackgroundTheme::GetNSImageColorNamed(int id,
-                                               bool allow_default) const {
-  return nil;
-}
-
-NSColor* BackgroundTheme::GetNSColor(int id, bool allow_default) const {
-  return provider_->GetNSColor(id, allow_default);
-}
-
-NSColor* BackgroundTheme::GetNSColorTint(int id, bool allow_default) const {
-  if (id == BrowserThemeProvider::TINT_BUTTONS)
-    return borderColor_.get();
-
-  return provider_->GetNSColorTint(id, allow_default);
-}
-
-NSGradient* BackgroundTheme::GetNSGradient(int id) const {
-  switch (id) {
-    case BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON:
-    case BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON_INACTIVE:
-      return buttonGradient_.get();
-    case BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON_PRESSED:
-    case BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON_PRESSED_INACTIVE:
-      return buttonPressedGradient_.get();
-    default:
-      return provider_->GetNSGradient(id);
-  }
-}
-
 @interface DownloadItemCell(Private)
 - (void)updateTrackingAreas:(id)sender;
+- (void)setupToggleStatusVisibilityAnimation;
+- (void)showSecondaryTitle;
 - (void)hideSecondaryTitle;
 - (void)animation:(NSAnimation*)animation
        progressed:(NSAnimationProgress)progress;
 - (NSString*)elideTitle:(int)availableWidth;
 - (NSString*)elideStatus:(int)availableWidth;
-- (ui::ThemeProvider*)backgroundThemeWrappingProvider:(ui::ThemeProvider*)provider;
+- (ui::ThemeProvider*)backgroundThemeWrappingProvider:
+    (ui::ThemeProvider*)provider;
 - (BOOL)pressedWithDefaultThemeOnPart:(DownloadItemMousePosition)part;
 - (NSColor*)titleColorForPart:(DownloadItemMousePosition)part;
 - (void)drawSecondaryTitleInRect:(NSRect)innerFrame;
@@ -182,7 +112,7 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
 - (void)setInitialState {
   isStatusTextVisible_ = NO;
   titleY_ = kPrimaryTextPosTop;
-  statusAlpha_ = 1.0;
+  statusAlpha_ = 0.0;
 
   [self setFont:[NSFont systemFontOfSize:
       [NSFont systemFontSizeForControlSize:NSSmallControlSize]]];
@@ -217,8 +147,8 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
   if ([completionAnimation_ isAnimating])
     [completionAnimation_ stopAnimation];
-  if ([hideStatusAnimation_ isAnimating])
-    [hideStatusAnimation_ stopAnimation];
+  if ([toggleStatusVisibilityAnimation_ isAnimating])
+    [toggleStatusVisibilityAnimation_ stopAnimation];
   if (trackingAreaButton_) {
     [[self controlView] removeTrackingArea:trackingAreaButton_];
     trackingAreaButton_.reset();
@@ -240,15 +170,14 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
   if (statusText.empty()) {
     // Remove the status text label.
     [self hideSecondaryTitle];
-    isStatusTextVisible_ = NO;
   } else {
     // Set status text.
     NSString* statusString = base::SysUTF16ToNSString(statusText);
     [self setSecondaryTitle:statusString];
-    isStatusTextVisible_ = YES;
+    [self showSecondaryTitle];
   }
 
-  switch (downloadModel->download()->state()) {
+  switch (downloadModel->download()->GetState()) {
     case DownloadItem::COMPLETE:
       // Small downloads may start in a complete state due to asynchronous
       // notifications. In this case, we'll get a second complete notification
@@ -267,8 +196,23 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
     case DownloadItem::CANCELLED:
       percentDone_ = -1;
       break;
+    case DownloadItem::INTERRUPTED:
+      // Small downloads may start in an interrupted state due to asynchronous
+      // notifications. In this case, we'll get a second complete notification
+      // via the observers, so we ignore it and avoid creating a second complete
+      // animation.
+      if (completionAnimation_.get())
+        break;
+      completionAnimation_.reset([[DownloadItemCellAnimation alloc]
+          initWithDownloadItemCell:self
+                          duration:kInterruptedAnimationDuration
+                    animationCurve:NSAnimationLinear]);
+      [completionAnimation_.get() setDelegate:self];
+      [completionAnimation_.get() startAnimation];
+      percentDone_ = -2;
+      break;
     case DownloadItem::IN_PROGRESS:
-      percentDone_ = downloadModel->download()->is_paused() ?
+      percentDone_ = downloadModel->download()->IsPaused() ?
           -1 : downloadModel->download()->PercentComplete();
       break;
     default:
@@ -394,7 +338,7 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
 
 - (NSString*)elideTitle:(int)availableWidth {
   NSFont* font = [self font];
-  gfx::Font font_chr(base::SysNSStringToUTF16([font fontName]),
+  gfx::Font font_chr(base::SysNSStringToUTF8([font fontName]),
                      [font pointSize]);
 
   return base::SysUTF16ToNSString(
@@ -403,17 +347,18 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
 
 - (NSString*)elideStatus:(int)availableWidth {
   NSFont* font = [self secondaryFont];
-  gfx::Font font_chr(base::SysNSStringToUTF16([font fontName]),
+  gfx::Font font_chr(base::SysNSStringToUTF8([font fontName]),
                      [font pointSize]);
 
   return base::SysUTF16ToNSString(ui::ElideText(
       base::SysNSStringToUTF16([self secondaryTitle]),
       font_chr,
       availableWidth,
-      false));
+      ui::ELIDE_AT_END));
 }
 
-- (ui::ThemeProvider*)backgroundThemeWrappingProvider:(ui::ThemeProvider*)provider {
+- (ui::ThemeProvider*)backgroundThemeWrappingProvider:
+    (ui::ThemeProvider*)provider {
   if (!themeProvider_.get()) {
     themeProvider_.reset(new BackgroundTheme(provider));
   }
@@ -435,7 +380,7 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
   ui::ThemeProvider* themeProvider =
       [[[self controlView] window] themeProvider];
   NSColor* themeTextColor =
-      themeProvider->GetNSColor(BrowserThemeProvider::COLOR_BOOKMARK_TEXT,
+      themeProvider->GetNSColor(ThemeService::COLOR_BOOKMARK_TEXT,
                                 true);
   return [self pressedWithDefaultThemeOnPart:part]
       ? [NSColor alternateSelectedControlTextColor] : themeTextColor;
@@ -463,13 +408,17 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
           nil];
   NSPoint secondaryPos =
       NSMakePoint(innerFrame.origin.x + kTextPosLeft, kSecondaryTextPosTop);
+
+  gfx::ScopedNSGraphicsContextSaveGState contextSave;
+  NSGraphicsContext* nsContext = [NSGraphicsContext currentContext];
+  CGContextRef cgContext = (CGContextRef)[nsContext graphicsPort];
+  [nsContext setCompositingOperation:NSCompositeSourceOver];
+  CGContextSetAlpha(cgContext, statusAlpha_);
   [secondaryText drawAtPoint:secondaryPos
               withAttributes:secondaryTextAttributes];
 }
 
 - (void)drawWithFrame:(NSRect)cellFrame inView:(NSView*)controlView {
-  // Constants from Cole.  Will kConstant them once the feedback loop
-  // is complete.
   NSRect drawFrame = NSInsetRect(cellFrame, 1.5, 1.5);
   NSRect innerFrame = NSInsetRect(cellFrame, 2, 2);
 
@@ -491,8 +440,8 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
   if (!isDefaultTheme) {
     themeProvider = [self backgroundThemeWrappingProvider:themeProvider];
     bgGradient = themeProvider->GetNSGradient(
-        active ? BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON :
-                 BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON_INACTIVE);
+        active ? ThemeService::GRADIENT_TOOLBAR_BUTTON :
+                 ThemeService::GRADIENT_TOOLBAR_BUTTON_INACTIVE);
   }
 
   NSRect buttonDrawRect, dropdownDrawRect;
@@ -580,10 +529,17 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
     canvas.set_composite_alpha(true);
     if (completionAnimation_.get()) {
       if ([completionAnimation_ isAnimating]) {
-        download_util::PaintDownloadComplete(&canvas,
-            x, y,
-            [completionAnimation_ currentValue],
-            download_util::SMALL);
+        if (percentDone_ == -1) {
+          download_util::PaintDownloadComplete(&canvas,
+              x, y,
+              [completionAnimation_ currentValue],
+              download_util::SMALL);
+        } else {
+          download_util::PaintDownloadInterrupted(&canvas,
+              x, y,
+              [completionAnimation_ currentValue],
+              download_util::SMALL);
+        }
       }
     } else if (percentDone_ >= 0) {
       download_util::PaintDownloadProgress(&canvas,
@@ -595,10 +551,8 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
   }
 
   // Draw icon
-  NSRect imageRect = NSZeroRect;
-  imageRect.size = [[self image] size];
   [[self image] drawInRect:[self imageRectForBounds:cellFrame]
-                  fromRect:imageRect
+                  fromRect:NSZeroRect
                  operation:NSCompositeSourceOver
                   fraction:[self isEnabled] ? 1.0 : 0.5
               neverFlipped:YES];
@@ -627,8 +581,7 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
   [triangle lineToPoint:p3];
   [triangle closePath];
 
-  NSGraphicsContext* context = [NSGraphicsContext currentContext];
-  [context saveGraphicsState];
+  gfx::ScopedNSGraphicsContextSaveGState scopedGState;
 
   scoped_nsobject<NSShadow> shadow([[NSShadow alloc] init]);
   [shadow.get() setShadowColor:[NSColor whiteColor]];
@@ -640,8 +593,6 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
   [fill setFill];
 
   [triangle fill];
-
-  [context restoreGraphicsState];
 }
 
 - (NSRect)imageRectForBounds:(NSRect)cellFrame {
@@ -651,28 +602,50 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
                     kImageHeight);
 }
 
-- (void)hideSecondaryTitle {
-  if (isStatusTextVisible_) {
-    // No core animation -- text in CA layers is not subpixel antialiased :-/
-    hideStatusAnimation_.reset([[DownloadItemCellAnimation alloc]
-        initWithDownloadItemCell:self
-                        duration:kHideStatusDuration
-                  animationCurve:NSAnimationEaseIn]);
-    [hideStatusAnimation_.get() setDelegate:self];
-    [hideStatusAnimation_.get() startAnimation];
+- (void)setupToggleStatusVisibilityAnimation {
+  if (toggleStatusVisibilityAnimation_ &&
+      [toggleStatusVisibilityAnimation_ isAnimating]) {
+    // If the animation is running, cancel the animation and show/hide the
+    // status text immediately.
+    [toggleStatusVisibilityAnimation_ stopAnimation];
+    [self animation:toggleStatusVisibilityAnimation_ progressed:1.0];
+    toggleStatusVisibilityAnimation_.reset();
   } else {
-    // If the download is done so quickly that the status line is never visible,
-    // don't show an animation
-    [self animation:nil progressed:1.0];
+    // Don't use core animation -- text in CA layers is not subpixel antialiased
+    toggleStatusVisibilityAnimation_.reset([[DownloadItemCellAnimation alloc]
+        initWithDownloadItemCell:self
+                        duration:kShowStatusDuration
+                  animationCurve:NSAnimationEaseIn]);
+    [toggleStatusVisibilityAnimation_.get() setDelegate:self];
+    [toggleStatusVisibilityAnimation_.get() startAnimation];
   }
 }
 
+- (void)showSecondaryTitle {
+  if (isStatusTextVisible_)
+    return;
+  isStatusTextVisible_ = YES;
+  [self setupToggleStatusVisibilityAnimation];
+}
+
+- (void)hideSecondaryTitle {
+  if (!isStatusTextVisible_)
+    return;
+  isStatusTextVisible_ = NO;
+  [self setupToggleStatusVisibilityAnimation];
+}
+
 - (void)animation:(NSAnimation*)animation
-      progressed:(NSAnimationProgress)progress {
-  if (animation == hideStatusAnimation_ || animation == nil) {
-    titleY_ = progress*kPrimaryTextOnlyPosTop +
-        (1 - progress)*kPrimaryTextPosTop;
-    statusAlpha_ = 1 - progress;
+   progressed:(NSAnimationProgress)progress {
+  if (animation == toggleStatusVisibilityAnimation_) {
+    if (isStatusTextVisible_) {
+      titleY_ = (1 - progress)*kPrimaryTextOnlyPosTop + kPrimaryTextPosTop;
+      statusAlpha_ = progress;
+    } else {
+      titleY_ = progress*kPrimaryTextOnlyPosTop +
+          (1 - progress)*kPrimaryTextPosTop;
+      statusAlpha_ = 1 - progress;
+    }
     [[self controlView] setNeedsDisplay:YES];
   } else if (animation == completionAnimation_) {
     [[self controlView] setNeedsDisplay:YES];
@@ -680,12 +653,24 @@ NSGradient* BackgroundTheme::GetNSGradient(int id) const {
 }
 
 - (void)animationDidEnd:(NSAnimation *)animation {
-  if (animation == hideStatusAnimation_)
-    hideStatusAnimation_.reset();
+  if (animation == toggleStatusVisibilityAnimation_)
+    toggleStatusVisibilityAnimation_.reset();
   else if (animation == completionAnimation_)
     completionAnimation_.reset();
 }
 
+- (BOOL)isStatusTextVisible {
+  return isStatusTextVisible_;
+}
+
+- (CGFloat)statusTextAlpha {
+  return statusAlpha_;
+}
+
+- (void)skipVisibilityAnimation {
+  [toggleStatusVisibilityAnimation_ setCurrentProgress:1.0];
+}
+
 @end
 
 @implementation DownloadItemCellAnimation
