@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,13 +8,11 @@
 
 #import <Cocoa/Cocoa.h>
 
+#include "base/compiler_specific.h"
 #include "chrome/browser/download/download_shelf.h"
 
 class BaseDownloadItemModel;
-class CustomDrawButton;
-class DownloadItemMac;
 
-@class ShelfView;
 @class DownloadShelfController;
 
 // A class to bridge the chromium download shelf to mac gui. This is just a
@@ -22,16 +20,17 @@ class DownloadItemMac;
 
 class DownloadShelfMac : public DownloadShelf {
  public:
-  explicit DownloadShelfMac(Browser* browser,
-                            DownloadShelfController* controller);
+  DownloadShelfMac(Browser* browser, DownloadShelfController* controller);
 
   // DownloadShelf implementation.
-  virtual void AddDownload(BaseDownloadItemModel* download_model);
-  virtual bool IsShowing() const;
-  virtual bool IsClosing() const;
-  virtual void Show();
-  virtual void Close();
-  virtual Browser* browser() const { return browser_; }
+  virtual bool IsShowing() const OVERRIDE;
+  virtual bool IsClosing() const OVERRIDE;
+  virtual Browser* browser() const OVERRIDE;
+
+ protected:
+  virtual void DoAddDownload(BaseDownloadItemModel* download_model) OVERRIDE;
+  virtual void DoShow() OVERRIDE;
+  virtual void DoClose() OVERRIDE;
 
  private:
   // The browser that owns this shelf.
