@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,28 +7,32 @@
 #include <cmath>
 #include <string>
 
-#include "app/mac/nsimage_cache.h"
 #include "base/sys_string_conversions.h"
 #include "chrome/browser/extensions/extension_browser_event_router.h"
-#include "chrome/browser/extensions/extension_host.h"
-#include "chrome/browser/extensions/extension_toolbar_model.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_toolbar_model.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/sessions/restore_tab_helper.h"
 #include "chrome/browser/ui/browser.h"
 #import "chrome/browser/ui/cocoa/extensions/browser_action_button.h"
 #import "chrome/browser/ui/cocoa/extensions/browser_actions_container_view.h"
 #import "chrome/browser/ui/cocoa/extensions/chevron_menu_button.h"
 #import "chrome/browser/ui/cocoa/extensions/extension_popup_controller.h"
+#import "chrome/browser/ui/cocoa/image_button_cell.h"
 #import "chrome/browser/ui/cocoa/menu_button.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/extensions/extension_action.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_source.h"
 #include "chrome/common/pref_names.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_source.h"
+#include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
 #import "third_party/GTM/AppKit/GTMNSAnimation+Duration.h"
+#include "ui/gfx/mac/nsimage_cache.h"
 
 NSString* const kBrowserActionVisibilityChangedNotification =
     @"BrowserActionVisibilityChangedNotification";
@@ -38,10 +42,6 @@ const CGFloat kAnimationDuration = 0.2;
 
 const CGFloat kChevronWidth = 14.0;
 
-// Image used for the overflow button.
-NSString* const kOverflowChevronsName =
-    @"browser_actions_overflow_Template.pdf";
-
 // Since the container is the maximum height of the toolbar, we have
 // to move the buttons up by this amount in order to have them look
 // vertically centered within the toolbar.
@@ -178,21 +178,21 @@ const CGFloat kBrowserActionBubbleYOffset = 3.0;
 
 // A helper class to proxy extension notifications to the view controller's
 // appropriate methods.
-class ExtensionServiceObserverBridge : public NotificationObserver,
-                                        public ExtensionToolbarModel::Observer {
+class ExtensionServiceObserverBridge : public content::NotificationObserver,
+                                       public ExtensionToolbarModel::Observer {
  public:
   ExtensionServiceObserverBridge(BrowserActionsController* owner,
                                   Profile* profile) : owner_(owner) {
-    registrar_.Add(this, NotificationType::EXTENSION_HOST_VIEW_SHOULD_CLOSE,
-                   Source<Profile>(profile));
+    registrar_.Add(this, chrome::NOTIFICATION_EXTENSION_HOST_VIEW_SHOULD_CLOSE,
+                   content::Source<Profile>(profile));
   }
 
-  // Overridden from NotificationObserver.
-  void Observe(NotificationType type,
-               const NotificationSource& source,
-               const NotificationDetails& details) {
-    switch (type.value) {
-      case NotificationType::EXTENSION_HOST_VIEW_SHOULD_CLOSE: {
+  // Overridden from content::NotificationObserver.
+  void Observe(int type,
+               const content::NotificationSource& source,
+               const content::NotificationDetails& details) {
+    switch (type) {
+      case chrome::NOTIFICATION_EXTENSION_HOST_VIEW_SHOULD_CLOSE: {
         ExtensionPopupController* popup = [ExtensionPopupController popup];
         if (popup && ![popup isClosing])
           [popup close];
@@ -220,7 +220,7 @@ class ExtensionServiceObserverBridge : public NotificationObserver,
   BrowserActionsController* owner_;
 
   // Used for registering to receive notifications and automatic clean up.
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   DISALLOW_COPY_AND_ASSIGN(ExtensionServiceObserverBridge);
 };
@@ -360,7 +360,7 @@ class ExtensionServiceObserverBridge : public NotificationObserver,
     // Migration code to the new VisibleIconCount pref.
     // TODO(mpcomplete): remove this at some point.
     double predefinedWidth =
-        profile_->GetPrefs()->GetReal(prefs::kBrowserActionContainerWidth);
+        profile_->GetPrefs()->GetDouble(prefs::kBrowserActionContainerWidth);
     if (predefinedWidth != 0) {
       int iconWidth = kBrowserActionWidth + kBrowserActionButtonPadding;
       int extraWidth = kChevronWidth;
@@ -419,7 +419,9 @@ class ExtensionServiceObserverBridge : public NotificationObserver,
 }
 
 + (void)registerUserPrefs:(PrefService*)prefs {
-  prefs->RegisterRealPref(prefs::kBrowserActionContainerWidth, 0);
+  prefs->RegisterDoublePref(prefs::kBrowserActionContainerWidth,
+                            0,
+                            PrefService::UNSYNCABLE_PREF);
 }
 
 #pragma mark -
@@ -721,19 +723,12 @@ class ExtensionServiceObserverBridge : public NotificationObserver,
     NOTREACHED() << "No current tab.";
     return;
   }
+  // If an extension popup is already open, it will get closed when it
+  // loses focus.
 
   ExtensionAction* action = [button extension]->browser_action();
   if (action->HasPopup(tabId)) {
     GURL popupUrl = action->GetPopupUrl(tabId);
-    // If a popup is already showing, check if the popup URL is the same. If so,
-    // then close the popup.
-    ExtensionPopupController* popup = [ExtensionPopupController popup];
-    if (popup &&
-        [[popup window] isVisible] &&
-        [popup extensionHost]->GetURL() == popupUrl) {
-      [popup close];
-      return;
-    }
     NSPoint arrowPoint = [self popupPointForBrowserAction:[button extension]];
     [ExtensionPopupController showURL:popupUrl
                             inBrowser:browser_
@@ -749,8 +744,9 @@ class ExtensionServiceObserverBridge : public NotificationObserver,
 
 - (BOOL)shouldDisplayBrowserAction:(const Extension*)extension {
   // Only display incognito-enabled extensions while in incognito mode.
-  return (!profile_->IsOffTheRecord() ||
-          profile_->GetExtensionService()->IsIncognitoEnabled(extension));
+  return
+      (!profile_->IsOffTheRecord() ||
+       profile_->GetExtensionService()->IsIncognitoEnabled(extension->id()));
 }
 
 - (void)showChevronIfNecessaryInFrame:(NSRect)frame animate:(BOOL)animate {
@@ -778,9 +774,14 @@ class ExtensionServiceObserverBridge : public NotificationObserver,
     chevronMenuButton_.reset([[ChevronMenuButton alloc] init]);
     [chevronMenuButton_ setBordered:NO];
     [chevronMenuButton_ setShowsBorderOnlyWhileMouseInside:YES];
-    NSImage* chevronImage =
-        app::mac::GetCachedImageWithName(kOverflowChevronsName);
-    [chevronMenuButton_ setImage:chevronImage];
+
+    [[chevronMenuButton_ cell] setImageID:IDR_BROWSER_ACTIONS_OVERFLOW
+                           forButtonState:image_button_cell::kDefaultState];
+    [[chevronMenuButton_ cell] setImageID:IDR_BROWSER_ACTIONS_OVERFLOW_H
+                           forButtonState:image_button_cell::kHoverState];
+    [[chevronMenuButton_ cell] setImageID:IDR_BROWSER_ACTIONS_OVERFLOW_P
+                           forButtonState:image_button_cell::kPressedState];
+
     [containerView_ addSubview:chevronMenuButton_];
   }
 
@@ -844,11 +845,11 @@ class ExtensionServiceObserverBridge : public NotificationObserver,
 }
 
 - (int)currentTabId {
-  TabContents* selected_tab = browser_->GetSelectedTabContents();
+  TabContentsWrapper* selected_tab = browser_->GetSelectedTabContentsWrapper();
   if (!selected_tab)
     return -1;
 
-  return selected_tab->controller().session_id().id();
+  return selected_tab->restore_tab_helper()->session_id().id();
 }
 
 #pragma mark -
