@@ -1,16 +1,18 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/cocoa/gradient_button_cell.h"
 
 #include "base/logging.h"
-#import "base/scoped_nsobject.h"
-#import "chrome/browser/themes/browser_theme_provider.h"
+#import "base/memory/scoped_nsobject.h"
+#import "chrome/browser/themes/theme_service.h"
 #import "chrome/browser/ui/cocoa/image_utils.h"
+#import "chrome/browser/ui/cocoa/nsview_additions.h"
 #import "chrome/browser/ui/cocoa/themed_window.h"
 #include "grit/theme_resources.h"
 #import "third_party/GTM/AppKit/GTMNSColor+Luminance.h"
+#include "ui/gfx/scoped_ns_graphics_context_save_gstate_mac.h"
 
 @interface GradientButtonCell (Private)
 - (void)sharedInit;
@@ -319,9 +321,6 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
 - (void)setShowsBorderOnlyWhileMouseInside:(BOOL)showOnly {
   [super setShowsBorderOnlyWhileMouseInside:showOnly];
   if (showOnly) {
-    if (trackingArea_.get()) {
-      [self setShowsBorderOnlyWhileMouseInside:NO];
-    }
     [self updateTrackingAreas];
   } else {
     if (trackingArea_) {
@@ -395,8 +394,8 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
       } else {
         clickedGradient = themeProvider ? themeProvider->GetNSGradient(
             active ?
-              BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON_PRESSED :
-              BrowserThemeProvider::GRADIENT_TOOLBAR_BUTTON_PRESSED_INACTIVE) :
+                ThemeService::GRADIENT_TOOLBAR_BUTTON_PRESSED :
+                ThemeService::GRADIENT_TOOLBAR_BUTTON_PRESSED_INACTIVE) :
             nil;
       }
       [clickedGradient drawInBezierPath:innerPath angle:90.0];
@@ -405,7 +404,7 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
 
   // Visually indicate unclicked, enabled buttons.
   if (!showClickedGradient && [self isEnabled]) {
-    [NSGraphicsContext saveGraphicsState];
+    gfx::ScopedNSGraphicsContextSaveGState scopedGState;
     [innerPath addClip];
 
     // Draw the inner glow.
@@ -425,8 +424,6 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
 
     // Draw the gradient inside.
     [gradient drawInBezierPath:innerPath angle:90.0];
-
-    [NSGraphicsContext restoreGraphicsState];
   }
 
   // Don't draw anything else for disabled flat buttons.
@@ -441,8 +438,8 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
                                        alpha:0.3 * outerStrokeAlphaMult_];
   } else {
     strokeColor = themeProvider ? themeProvider->GetNSColor(
-        active ? BrowserThemeProvider::COLOR_TOOLBAR_BUTTON_STROKE :
-                 BrowserThemeProvider::COLOR_TOOLBAR_BUTTON_STROKE_INACTIVE,
+        active ? ThemeService::COLOR_TOOLBAR_BUTTON_STROKE :
+                 ThemeService::COLOR_TOOLBAR_BUTTON_STROKE_INACTIVE,
         true) : [NSColor colorWithCalibratedWhite:0.0
                                             alpha:0.3 * outerStrokeAlphaMult_];
   }
@@ -459,10 +456,13 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
                    innerFrame:(NSRect*)returnInnerFrame
                     innerPath:(NSBezierPath**)returnInnerPath
                      clipPath:(NSBezierPath**)returnClipPath {
+  const CGFloat lineWidth = [controlView cr_lineWidth];
+  const CGFloat halfLineWidth = lineWidth / 2.0;
+
   // Constants from Cole.  Will kConstant them once the feedback loop
   // is complete.
-  NSRect drawFrame = NSInsetRect(cellFrame, 1.5, 1.5);
-  NSRect innerFrame = NSInsetRect(cellFrame, 2, 1);
+  NSRect drawFrame = NSInsetRect(cellFrame, 1.5 * lineWidth, 1.5 * lineWidth);
+  NSRect innerFrame = NSInsetRect(cellFrame, lineWidth, lineWidth);
   const CGFloat radius = 3.5;
 
   ButtonType type = [[(NSControl*)controlView cell] tag];
@@ -493,13 +493,16 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
     *returnInnerPath = [NSBezierPath bezierPathWithRoundedRect:drawFrame
                                                        xRadius:radius
                                                        yRadius:radius];
+    [*returnInnerPath setLineWidth:lineWidth];
   }
   if (returnClipPath) {
     DCHECK(*returnClipPath == nil);
-    NSRect clipPathRect = NSInsetRect(drawFrame, -0.5, -0.5);
-    *returnClipPath = [NSBezierPath bezierPathWithRoundedRect:clipPathRect
-                                                      xRadius:radius + 0.5
-                                                      yRadius:radius + 0.5];
+    NSRect clipPathRect =
+        NSInsetRect(drawFrame, -halfLineWidth, -halfLineWidth);
+    *returnClipPath = [NSBezierPath
+        bezierPathWithRoundedRect:clipPathRect
+                          xRadius:radius + halfLineWidth
+                          yRadius:radius + halfLineWidth];
   }
 }
 
@@ -551,11 +554,12 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
   // If this is the left side of a segmented button, draw a slight shadow.
   ButtonType type = [[(NSControl*)controlView cell] tag];
   if (type == kLeftButtonWithShadowType) {
+    const CGFloat lineWidth = [controlView cr_lineWidth];
     NSRect borderRect, contentRect;
-    NSDivideRect(cellFrame, &borderRect, &contentRect, 1.0, NSMaxXEdge);
+    NSDivideRect(cellFrame, &borderRect, &contentRect, lineWidth, NSMaxXEdge);
     NSColor* stroke = themeProvider ? themeProvider->GetNSColor(
-        active ? BrowserThemeProvider::COLOR_TOOLBAR_BUTTON_STROKE :
-                 BrowserThemeProvider::COLOR_TOOLBAR_BUTTON_STROKE_INACTIVE,
+        active ? ThemeService::COLOR_TOOLBAR_BUTTON_STROKE :
+                 ThemeService::COLOR_TOOLBAR_BUTTON_STROKE_INACTIVE,
         true) : [NSColor blackColor];
 
     [[stroke colorWithAlphaComponent:0.2] set];
@@ -566,27 +570,28 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
 }
 
 - (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView*)controlView {
+  const CGFloat lineWidth = [controlView cr_lineWidth];
+
   if (shouldTheme_) {
     BOOL isTemplate = [[self image] isTemplate];
 
-    [NSGraphicsContext saveGraphicsState];
+    gfx::ScopedNSGraphicsContextSaveGState scopedGState;
 
     CGContextRef context =
         (CGContextRef)([[NSGraphicsContext currentContext] graphicsPort]);
 
-    BrowserThemeProvider* themeProvider = static_cast<BrowserThemeProvider*>(
+    ThemeService* themeProvider = static_cast<ThemeService*>(
         [[controlView window] themeProvider]);
     NSColor* color = themeProvider ?
-        themeProvider->GetNSColorTint(BrowserThemeProvider::TINT_BUTTONS,
-                                      true) :
+        themeProvider->GetNSColorTint(ThemeService::TINT_BUTTONS, true) :
         [NSColor blackColor];
 
     if (isTemplate && themeProvider && themeProvider->UsingDefaultTheme()) {
       scoped_nsobject<NSShadow> shadow([[NSShadow alloc] init]);
       [shadow.get() setShadowColor:themeProvider->GetNSColor(
-          BrowserThemeProvider::COLOR_TOOLBAR_BEZEL, true)];
-      [shadow.get() setShadowOffset:NSMakeSize(0.0, -1.0)];
-      [shadow setShadowBlurRadius:1.0];
+          ThemeService::COLOR_TOOLBAR_BEZEL, true)];
+      [shadow.get() setShadowOffset:NSMakeSize(0.0, -lineWidth)];
+      [shadow setShadowBlurRadius:lineWidth];
       [shadow set];
     }
 
@@ -604,12 +609,10 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
       NSRectFillUsingOperation(cellFrame, NSCompositeSourceAtop);
     }
     CGContextEndTransparencyLayer(context);
-
-    [NSGraphicsContext restoreGraphicsState];
   } else {
     // NSCell draws these off-center for some reason, probably because of the
     // positioning of the control in the xib.
-    [super drawInteriorWithFrame:NSOffsetRect(cellFrame, 0, 1)
+    [super drawInteriorWithFrame:NSOffsetRect(cellFrame, 0, lineWidth)
                           inView:controlView];
   }
 
@@ -624,6 +627,10 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
   }
 }
 
+- (int)verticalTextOffset {
+  return 1;
+}
+
 // Overriden from NSButtonCell so we can display a nice fadeout effect for
 // button titles that overflow.
 // This method is copied in the most part from GTMFadeTruncatingTextFieldCell,
@@ -632,9 +639,9 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
 // We can't use GTMFadeTruncatingTextFieldCell because there's no easy way to
 // get it to work with NSButtonCell.
 // TODO(jeremy): Move this to GTM.
-- (NSRect)drawTitle:(NSAttributedString *)title
+- (NSRect)drawTitle:(NSAttributedString*)title
           withFrame:(NSRect)cellFrame
-             inView:(NSView *)controlView {
+             inView:(NSView*)controlView {
   NSSize size = [title size];
 
   // Empirically, Cocoa will draw an extra 2 pixels past NSWidth(cellFrame)
@@ -642,6 +649,9 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
   const CGFloat kOverflowBeforeClip = 2;
   // Don't complicate drawing unless we need to clip.
   if (floor(size.width) <= (NSWidth(cellFrame) + kOverflowBeforeClip)) {
+    cellFrame.origin.y += ([self verticalTextOffset] - 1);
+    // The super is called to provide the background shadow "highlight" for
+    // non-clipping text.
     return [super drawTitle:title withFrame:cellFrame inView:controlView];
   }
 
@@ -653,27 +663,30 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
 
   // Draw non-gradient part without transparency layer, as light text on a dark
   // background looks bad with a gradient layer.
-  [[NSGraphicsContext currentContext] saveGraphicsState];
-  [NSBezierPath clipRect:solidPart];
+  NSPoint textOffset = NSZeroPoint;
+  {
+    gfx::ScopedNSGraphicsContextSaveGState scopedGState;
+    [NSBezierPath clipRect:solidPart];
 
-  // 11 is the magic number needed to make this match the native NSButtonCell's
-  // label display.
-  CGFloat textLeft = [[self image] size].width + 11;
+    // 11 is the magic number needed to make this match the native
+    // NSButtonCell's label display.
+    CGFloat textLeft = [[self image] size].width + 11;
 
-  // For some reason, the height of cellFrame as passed in is totally bogus.
-  // For vertical centering purposes, we need the bounds of the containing
-  // view.
-  NSRect buttonFrame = [[self controlView] frame];
+    // For some reason, the height of cellFrame as passed in is totally bogus.
+    // For vertical centering purposes, we need the bounds of the containing
+    // view.
+    NSRect buttonFrame = [[self controlView] frame];
 
-  // Off-by-one to match native NSButtonCell's version.
-  NSPoint textOffset = NSMakePoint(textLeft,
-                        (NSHeight(buttonFrame) - size.height)/2 + 1);
-  [title drawAtPoint:textOffset];
-  [[NSGraphicsContext currentContext] restoreGraphicsState];
+    // Call the vertical offset to match native NSButtonCell's version.
+    textOffset = NSMakePoint(textLeft,
+                             (NSHeight(buttonFrame) - size.height) / 2 +
+                             [self verticalTextOffset]);
+    [title drawAtPoint:textOffset];
+  }
 
   // Draw the gradient part with a transparency layer. This makes the text look
   // suboptimal, but since it fades out, that's ok.
-  [[NSGraphicsContext currentContext] saveGraphicsState];
+  gfx::ScopedNSGraphicsContextSaveGState scopedGState;
   [NSBezierPath clipRect:gradientPart];
   CGContextRef context = static_cast<CGContextRef>(
       [[NSGraphicsContext currentContext] graphicsPort]);
@@ -681,8 +694,7 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
                                           NSRectToCGRect(gradientPart), 0);
   [title drawAtPoint:textOffset];
 
-  // TODO(alcor): switch this to GTMLinearRGBShading if we ever need on 10.4
-  NSColor *color = [NSColor textColor]; //[self textColor];
+  NSColor *color = [NSColor textColor];
   NSColor *alphaColor = [color colorWithAlphaComponent:0.0];
   NSGradient *mask = [[NSGradient alloc] initWithStartingColor:color
                                                    endingColor:alphaColor];
@@ -696,7 +708,6 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
               options:NSGradientDrawsBeforeStartingLocation];
   [mask release];
   CGContextEndTransparencyLayer(context);
-  [[NSGraphicsContext currentContext] restoreGraphicsState];
 
   return cellFrame;
 }
@@ -718,7 +729,7 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
     [self updateTrackingAreas];
 }
 
-- (void)updateTrackingAreas {
+- (BOOL)isMouseReallyInside {
   BOOL mouseInView = NO;
   NSView* controlView = [self controlView];
   NSWindow* window = [controlView window];
@@ -728,6 +739,12 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
     mousePoint = [controlView convertPointFromBase:mousePoint];
     mouseInView = [controlView mouse:mousePoint inRect:bounds];
   }
+  return mouseInView;
+}
+
+- (void)updateTrackingAreas {
+  NSView* controlView = [self controlView];
+  BOOL mouseInView = [self isMouseReallyInside];
 
   if (trackingArea_.get())
     [controlView removeTrackingArea:trackingArea_];
@@ -738,12 +755,12 @@ static const NSTimeInterval kAnimationContinuousCycleDuration = 0.4;
     options |= NSTrackingAssumeInside;
 
   trackingArea_.reset([[NSTrackingArea alloc]
-                        initWithRect:bounds
+                        initWithRect:[controlView bounds]
                              options:options
                                owner:self
                             userInfo:nil]);
   if (isMouseInside_ != mouseInView) {
-    isMouseInside_ = mouseInView;
+    [self setMouseInside:mouseInView animate:NO];
     [controlView setNeedsDisplay:YES];
   }
 }
