@@ -1,11 +1,12 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/cocoa/history_menu_bridge.h"
 
-#include "base/callback.h"
-#include "base/stl_util-inl.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
+#include "base/stl_util.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
 #include "base/sys_string_conversions.h"
@@ -14,18 +15,23 @@
 #include "chrome/browser/history/page_usage_data.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sessions/session_types.h"
+#include "chrome/browser/sessions/tab_restore_service_factory.h"
 #import "chrome/browser/ui/cocoa/history_menu_cocoa_controller.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/url_constants.h"
-#include "gfx/codec/png_codec.h"
-#include "grit/app_resources.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_source.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
 #include "skia/ext/skia_utils_mac.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/codec/png_codec.h"
+#include "ui/gfx/image/image.h"
+#include "ui/gfx/mac/nsimage_cache.h"
 
 namespace {
 
@@ -35,11 +41,11 @@ const NSUInteger kMaximumMenuWidthInChars = 50;
 // When trimming, use this many chars from each side.
 const NSUInteger kMenuTrimSizeInChars = 25;
 
-// Number of days to consider when getting the number of most visited items.
-const int kMostVisitedScope = 90;
+// Number of days to consider when getting the number of visited items.
+const int kVisitedScope = 90;
 
-// The number of most visisted results to get.
-const int kMostVisitedCount = 9;
+// The number of visisted results to get.
+const int kVisitedCount = 15;
 
 // The number of recently closed items to get.
 const unsigned int kRecentlyClosedCount = 10;
@@ -82,7 +88,7 @@ HistoryMenuBridge::HistoryMenuBridge(Profile* profile)
       Init();
     }
 
-    tab_restore_service_ = profile_->GetTabRestoreService();
+    tab_restore_service_ = TabRestoreServiceFactory::GetForProfile(profile_);
     if (tab_restore_service_) {
       tab_restore_service_->AddObserver(this);
       tab_restore_service_->LoadTabsFromLastSession();
@@ -90,7 +96,7 @@ HistoryMenuBridge::HistoryMenuBridge(Profile* profile)
   }
 
   ResourceBundle& rb = ResourceBundle::GetSharedInstance();
-  default_favicon_.reset([rb.GetNativeImageNamed(IDR_DEFAULT_FAVICON) retain]);
+  default_favicon_.reset([gfx::GetCachedImageWithName(@"nav.pdf") retain]);
 
   // Set the static icons in the menu.
   NSMenuItem* item = [HistoryMenu() itemWithTag:IDC_SHOW_HISTORY];
@@ -98,9 +104,9 @@ HistoryMenuBridge::HistoryMenuBridge(Profile* profile)
 
   // The service is not ready for use yet, so become notified when it does.
   if (!history_service_) {
-    registrar_.Add(this,
-                   NotificationType::HISTORY_LOADED,
-                   NotificationService::AllSources());
+    registrar_.Add(
+        this, chrome::NOTIFICATION_HISTORY_LOADED,
+        content::Source<Profile>(profile_));
   }
 }
 
@@ -109,13 +115,17 @@ HistoryMenuBridge::HistoryMenuBridge(Profile* profile)
 // task cancellation is not done manually here in the dtor.
 HistoryMenuBridge::~HistoryMenuBridge() {
   // Unregister ourselves as observers and notifications.
-  const NotificationSource& src = NotificationService::AllSources();
+  DCHECK(profile_);
   if (history_service_) {
-    registrar_.Remove(this, NotificationType::HISTORY_TYPED_URLS_MODIFIED, src);
-    registrar_.Remove(this, NotificationType::HISTORY_URL_VISITED, src);
-    registrar_.Remove(this, NotificationType::HISTORY_URLS_DELETED, src);
+    registrar_.Remove(this, chrome::NOTIFICATION_HISTORY_TYPED_URLS_MODIFIED,
+                      content::Source<Profile>(profile_));
+    registrar_.Remove(this, chrome::NOTIFICATION_HISTORY_URL_VISITED,
+                      content::Source<Profile>(profile_));
+    registrar_.Remove(this, chrome::NOTIFICATION_HISTORY_URLS_DELETED,
+                      content::Source<Profile>(profile_));
   } else {
-    registrar_.Remove(this, NotificationType::HISTORY_LOADED, src);
+    registrar_.Remove(this, chrome::NOTIFICATION_HISTORY_LOADED,
+                      content::Source<Profile>(profile_));
   }
 
   if (tab_restore_service_)
@@ -130,12 +140,12 @@ HistoryMenuBridge::~HistoryMenuBridge() {
   }
 }
 
-void HistoryMenuBridge::Observe(NotificationType type,
-                                const NotificationSource& source,
-                                const NotificationDetails& details) {
+void HistoryMenuBridge::Observe(int type,
+                                const content::NotificationSource& source,
+                                const content::NotificationDetails& details) {
   // A history service is now ready. Check to see if it's the one for the main
   // profile. If so, perform final initialization.
-  if (type == NotificationType::HISTORY_LOADED) {
+  if (type == chrome::NOTIFICATION_HISTORY_LOADED) {
     HistoryService* hs =
         profile_->GetHistoryService(Profile::EXPLICIT_ACCESS);
     if (hs != NULL && hs->BackendLoaded()) {
@@ -144,8 +154,8 @@ void HistoryMenuBridge::Observe(NotificationType type,
 
       // Found our HistoryService, so stop listening for this notification.
       registrar_.Remove(this,
-                        NotificationType::HISTORY_LOADED,
-                        NotificationService::AllSources());
+                        chrome::NOTIFICATION_HISTORY_LOADED,
+                        content::Source<Profile>(profile_));
     }
   }
 
@@ -253,6 +263,19 @@ void HistoryMenuBridge::TabRestoreServiceDestroyed(
   // Intentionally left blank. We hold a weak reference to the service.
 }
 
+void HistoryMenuBridge::ResetMenu() {
+  NSMenu* menu = HistoryMenu();
+  ClearMenuSection(menu, kVisited);
+  ClearMenuSection(menu, kRecentlyClosed);
+}
+
+void HistoryMenuBridge::BuildMenu() {
+  // If the history service is ready, use it. Otherwise, a Notification will
+  // force an update when it's loaded.
+  if (history_service_)
+    CreateMenu();
+}
+
 HistoryMenuBridge::HistoryItem* HistoryMenuBridge::HistoryItemForMenuItem(
     NSMenuItem* item) {
   std::map<NSMenuItem*, HistoryItem*>::iterator it = menu_item_map_.find(item);
@@ -345,10 +368,12 @@ NSMenuItem* HistoryMenuBridge::AddItemToMenu(HistoryItem* item,
 }
 
 void HistoryMenuBridge::Init() {
-  const NotificationSource& source = NotificationService::AllSources();
-  registrar_.Add(this, NotificationType::HISTORY_TYPED_URLS_MODIFIED, source);
-  registrar_.Add(this, NotificationType::HISTORY_URL_VISITED, source);
-  registrar_.Add(this, NotificationType::HISTORY_URLS_DELETED, source);
+  registrar_.Add(this, chrome::NOTIFICATION_HISTORY_TYPED_URLS_MODIFIED,
+                 content::Source<Profile>(profile_));
+  registrar_.Add(this, chrome::NOTIFICATION_HISTORY_URL_VISITED,
+                 content::Source<Profile>(profile_));
+  registrar_.Add(this, chrome::NOTIFICATION_HISTORY_URLS_DELETED,
+                 content::Source<Profile>(profile_));
 }
 
 void HistoryMenuBridge::CreateMenu() {
@@ -358,35 +383,40 @@ void HistoryMenuBridge::CreateMenu() {
   create_in_progress_ = true;
   need_recreate_ = false;
 
-  history_service_->QuerySegmentUsageSince(
+  DCHECK(history_service_);
+
+  history::QueryOptions options;
+  options.max_count = kVisitedCount;
+  options.SetRecentDayRange(kVisitedScope);
+
+  history_service_->QueryHistory(
+      string16(),
+      options,
       &cancelable_request_consumer_,
-      base::Time::Now() - base::TimeDelta::FromDays(kMostVisitedScope),
-      kMostVisitedCount,
-      NewCallback(this, &HistoryMenuBridge::OnVisitedHistoryResults));
+      base::Bind(&HistoryMenuBridge::OnVisitedHistoryResults,
+                 base::Unretained(this)));
 }
 
 void HistoryMenuBridge::OnVisitedHistoryResults(
     CancelableRequestProvider::Handle handle,
-    std::vector<PageUsageData*>* results) {
+    history::QueryResults* results) {
   NSMenu* menu = HistoryMenu();
-  ClearMenuSection(menu, kMostVisited);
-  NSInteger top_item = [menu indexOfItemWithTag:kMostVisitedTitle] + 1;
+  ClearMenuSection(menu, kVisited);
+  NSInteger top_item = [menu indexOfItemWithTag:kVisitedTitle] + 1;
 
   size_t count = results->size();
   for (size_t i = 0; i < count; ++i) {
-    PageUsageData* history_item = (*results)[i];
+    const history::URLResult& result = (*results)[i];
+
+    HistoryItem* item = new HistoryItem;
+    item->title = result.title();
+    item->url = result.url();
+
+    // Need to explicitly get the favicon for each row.
+    GetFaviconForHistoryItem(item);
 
-    HistoryItem* item = new HistoryItem();
-    item->title = history_item->GetTitle();
-    item->url = history_item->GetURL();
-    if (history_item->HasFavIcon()) {
-      const SkBitmap* icon = history_item->GetFavIcon();
-      item->icon.reset([gfx::SkBitmapToNSImage(*icon) retain]);
-    } else {
-      GetFaviconForHistoryItem(item);
-    }
     // This will add |item| to the |menu_item_map_|, which takes ownership.
-    AddItemToMenu(item, HistoryMenu(), kMostVisited, top_item + i);
+    AddItemToMenu(item, HistoryMenu(), kVisited, top_item + i);
   }
 
   // We are already invalid by the time we finished, darn.
@@ -398,14 +428,10 @@ void HistoryMenuBridge::OnVisitedHistoryResults(
 
 HistoryMenuBridge::HistoryItem* HistoryMenuBridge::HistoryItemForTab(
     const TabRestoreService::Tab& entry) {
-  if (entry.navigations.empty())
-    return NULL;
+  DCHECK(!entry.navigations.empty());
 
   const TabNavigation& current_navigation =
       entry.navigations.at(entry.current_navigation_index);
-  if (current_navigation.virtual_url() == GURL(chrome::kChromeUINewTabURL))
-    return NULL;
-
   HistoryItem* item = new HistoryItem();
   item->title = current_navigation.title();
   item->url = current_navigation.virtual_url();
@@ -420,19 +446,16 @@ HistoryMenuBridge::HistoryItem* HistoryMenuBridge::HistoryItemForTab(
 void HistoryMenuBridge::GetFaviconForHistoryItem(HistoryItem* item) {
   FaviconService* service =
       profile_->GetFaviconService(Profile::EXPLICIT_ACCESS);
-  FaviconService::Handle handle = service->GetFaviconForURL(item->url,
-      &favicon_consumer_,
-      NewCallback(this, &HistoryMenuBridge::GotFaviconData));
+  FaviconService::Handle handle = service->GetFaviconForURL(
+      item->url, history::FAVICON, &favicon_consumer_,
+      base::Bind(&HistoryMenuBridge::GotFaviconData, base::Unretained(this)));
   favicon_consumer_.SetClientData(service, handle, item);
   item->icon_handle = handle;
   item->icon_requested = true;
 }
 
 void HistoryMenuBridge::GotFaviconData(FaviconService::Handle handle,
-                                       bool know_favicon,
-                                       scoped_refptr<RefCountedMemory> data,
-                                       bool expired,
-                                       GURL url) {
+                                       history::FaviconData favicon) {
   // Since we're going to do Cocoa-y things, make sure this is the main thread.
   DCHECK([NSThread isMainThread]);
 
@@ -446,8 +469,9 @@ void HistoryMenuBridge::GotFaviconData(FaviconService::Handle handle,
   // Convert the raw data to Skia and then to a NSImage.
   // TODO(rsesek): Is there an easier way to do this?
   SkBitmap icon;
-  if (know_favicon && data.get() && data->size() &&
-      gfx::PNGCodec::Decode(data->front(), data->size(), &icon)) {
+  if (favicon.is_valid() &&
+      gfx::PNGCodec::Decode(favicon.image_data->front(),
+          favicon.image_data->size(), &icon)) {
     NSImage* image = gfx::SkBitmapToNSImage(icon);
     if (image) {
       // The conversion was successful.
