@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -18,6 +18,14 @@ const CGFloat kBubbleArrowXOffset = kBubbleArrowWidth + kBubbleCornerRadius;
 enum BubbleArrowLocation {
   kTopLeft,
   kTopRight,
+  kNoArrow,
+};
+
+enum BubbleAlignment {
+  // The tip of the arrow points to the anchor point.
+  kAlignArrowToAnchor,
+  // The edge nearest to the arrow is lined up with the anchor point.
+  kAlignEdgeToAnchorEdge,
 };
 
 }  // namespace info_bubble
@@ -27,9 +35,11 @@ enum BubbleArrowLocation {
 @interface InfoBubbleView : NSView {
  @private
   info_bubble::BubbleArrowLocation arrowLocation_;
+  info_bubble::BubbleAlignment alignment_;
 }
 
-@property (assign, nonatomic) info_bubble::BubbleArrowLocation arrowLocation;
+@property(assign, nonatomic) info_bubble::BubbleArrowLocation arrowLocation;
+@property(assign, nonatomic) info_bubble::BubbleAlignment alignment;
 
 // Returns the point location in view coordinates of the tip of the arrow.
 - (NSPoint)arrowTip;
