@@ -1,20 +1,28 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #import <Cocoa/Cocoa.h>
 
 #include "base/logging.h"  // for NOTREACHED()
+#include "base/mac/bundle_locations.h"
 #include "base/mac/mac_util.h"
 #include "base/sys_string_conversions.h"
-#include "chrome/browser/tab_contents/infobar_delegate.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
+#include "chrome/browser/tab_contents/confirm_infobar_delegate.h"
+#include "chrome/browser/tab_contents/link_infobar_delegate.h"
 #import "chrome/browser/ui/cocoa/animatable_view.h"
+#import "chrome/browser/ui/cocoa/browser_window_controller.h"
 #include "chrome/browser/ui/cocoa/event_utils.h"
+#import "chrome/browser/ui/cocoa/hyperlink_text_view.h"
 #include "chrome/browser/ui/cocoa/infobars/infobar.h"
 #import "chrome/browser/ui/cocoa/infobars/infobar_container_controller.h"
 #import "chrome/browser/ui/cocoa/infobars/infobar_controller.h"
-#include "skia/ext/skia_utils_mac.h"
+#import "chrome/browser/ui/cocoa/infobars/infobar_gradient_view.h"
+#import "chrome/browser/ui/cocoa/location_bar/location_bar_view_mac.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "third_party/GTM/AppKit/GTMUILocalizerAndLayoutTweaker.h"
+#include "ui/gfx/image/image.h"
 #include "webkit/glue/window_open_disposition.h"
 
 namespace {
@@ -23,74 +31,18 @@ const float kAnimateOpenDuration = 0.12;
 const float kAnimateCloseDuration = 0.12;
 }
 
-// This simple subclass of |NSTextView| just doesn't show the (text) cursor
-// (|NSTextView| displays the cursor with full keyboard accessibility enabled).
-@interface InfoBarTextView : NSTextView
-- (void)fixupCursor;
-@end
-
-@implementation InfoBarTextView
-
-// Never draw the insertion point (otherwise, it shows up without any user
-// action if full keyboard accessibility is enabled).
-- (BOOL)shouldDrawInsertionPoint {
-  return NO;
-}
-
-- (NSRange)selectionRangeForProposedRange:(NSRange)proposedSelRange
-                              granularity:(NSSelectionGranularity)granularity {
-  // Do not allow selections.
-  return NSMakeRange(0, 0);
-}
-
-// Convince NSTextView to not show an I-Beam cursor when the cursor is over the
-// text view but not over actual text.
-//
-// http://www.mail-archive.com/cocoa-dev@lists.apple.com/msg10791.html
-// "NSTextView sets the cursor over itself dynamically, based on considerations
-// including the text under the cursor. It does so in -mouseEntered:,
-// -mouseMoved:, and -cursorUpdate:, so those would be points to consider
-// overriding."
-- (void)mouseMoved:(NSEvent*)e {
-  [super mouseMoved:e];
-  [self fixupCursor];
-}
-
-- (void)mouseEntered:(NSEvent*)e {
-  [super mouseEntered:e];
-  [self fixupCursor];
-}
-
-- (void)cursorUpdate:(NSEvent*)e {
-  [super cursorUpdate:e];
-  [self fixupCursor];
-}
-
-- (void)fixupCursor {
-  if ([[NSCursor currentCursor] isEqual:[NSCursor IBeamCursor]])
-    [[NSCursor arrowCursor] set];
-}
-
-@end
-
 @interface InfoBarController (PrivateMethods)
 // Sets |label_| based on |labelPlaceholder_|, sets |labelPlaceholder_| to nil.
 - (void)initializeLabel;
 
-// Asks the container controller to remove the infobar for this delegate.  This
-// call will trigger a notification that starts the infobar animating closed.
-- (void)removeInfoBar;
-
 // Performs final cleanup after an animation is finished or stopped, including
 // notifying the InfoBarDelegate that the infobar was closed and removing the
 // infobar from its container, if necessary.
 - (void)cleanUpAfterAnimation:(BOOL)finished;
 
-// Sets the info bar message to the specified |message|, with a hypertext
-// style link. |link| will be inserted into message at |linkOffset|.
-- (void)setLabelToMessage:(NSString*)message
-                 withLink:(NSString*)link
-                 atOffset:(NSUInteger)linkOffset;
+// Returns the point, in gradient view coordinates, at which the apex of the
+// infobar tip should be drawn.
+- (NSPoint)pointForTipApex;
 @end
 
 @implementation InfoBarController
@@ -98,11 +50,13 @@ const float kAnimateCloseDuration = 0.12;
 @synthesize containerController = containerController_;
 @synthesize delegate = delegate_;
 
-- (id)initWithDelegate:(InfoBarDelegate*)delegate {
+- (id)initWithDelegate:(InfoBarDelegate*)delegate
+                 owner:(InfoBarTabHelper*)owner {
   DCHECK(delegate);
   if ((self = [super initWithNibName:@"InfoBar"
-                              bundle:base::mac::MainAppBundle()])) {
+                              bundle:base::mac::FrameworkBundle()])) {
     delegate_ = delegate;
+    owner_ = owner;
   }
   return self;
 }
@@ -112,7 +66,7 @@ const float kAnimateCloseDuration = 0.12;
 - (void)awakeFromNib {
   DCHECK(delegate_);
   if (delegate_->GetIcon()) {
-    [image_ setImage:gfx::SkBitmapToNSImage(*(delegate_->GetIcon()))];
+    [image_ setImage:*(delegate_->GetIcon())];
   } else {
     // No icon, remove it from the view and grow the textfield to include the
     // space.
@@ -121,11 +75,22 @@ const float kAnimateCloseDuration = 0.12;
     labelFrame.size.width += NSMinX(imageFrame) - NSMinX(labelFrame);
     labelFrame.origin.x = imageFrame.origin.x;
     [image_ removeFromSuperview];
+    image_ = nil;
     [labelPlaceholder_ setFrame:labelFrame];
   }
   [self initializeLabel];
 
   [self addAdditionalControls];
+
+  infoBarView_.tipApex = [self pointForTipApex];
+  [infoBarView_ setInfobarType:delegate_->GetInfoBarType()];
+}
+
+- (void)dealloc {
+  [okButton_ setTarget:nil];
+  [cancelButton_ setTarget:nil];
+  [closeButton_ setTarget:nil];
+  [super dealloc];
 }
 
 // Called when someone clicks on the embedded link.
@@ -137,6 +102,10 @@ const float kAnimateCloseDuration = 0.12;
   return YES;
 }
 
+- (BOOL)isOwned {
+  return !!owner_;
+}
+
 // Called when someone clicks on the ok button.
 - (void)ok:(id)sender {
   // Subclasses must override this method if they do not hide the ok button.
@@ -151,7 +120,23 @@ const float kAnimateCloseDuration = 0.12;
 
 // Called when someone clicks on the close button.
 - (void)dismiss:(id)sender {
-  [self removeInfoBar];
+  if (![self isOwned])
+    return;
+  delegate_->InfoBarDismissed();
+  [self removeSelf];
+}
+
+- (void)removeSelf {
+  // |owner_| should never be NULL here.  If it is, then someone violated what
+  // they were supposed to do -- e.g. a ConfirmInfoBarDelegate subclass returned
+  // true from Accept() or Cancel() even though the infobar was already closing.
+  // In the worst case, if we also switched tabs during that process, then
+  // |this| has already been destroyed.  But if that's the case, then we're
+  // going to deref a garbage |this| pointer here whether we check |owner_| or
+  // not, and in other cases (where we're still closing and |this| is valid),
+  // checking |owner_| here will avoid a NULL deref.
+  if (owner_)
+    owner_->RemoveInfoBar(delegate_);
 }
 
 - (AnimatableView*)animatableView {
@@ -182,6 +167,9 @@ const float kAnimateCloseDuration = 0.12;
 }
 
 - (void)animateClosed {
+  // Notify the container of our intentions.
+  [containerController_ willRemoveController:self];
+
   // Start animating closed.  We will receive a notification when the animation
   // is done, at which point we can remove our view from the hierarchy and
   // notify the delegate that the infobar was closed.
@@ -198,21 +186,7 @@ const float kAnimateCloseDuration = 0.12;
 }
 
 - (void)infobarWillClose {
-  // Default implementation does nothing.
-}
-
-- (void)setLabelToMessage:(NSString*)message {
-  NSMutableDictionary* attributes = [NSMutableDictionary dictionary];
-  NSFont* font = [NSFont labelFontOfSize:
-      [NSFont systemFontSizeForControlSize:NSRegularControlSize]];
-  [attributes setObject:font
-                 forKey:NSFontAttributeName];
-  [attributes setObject:[NSCursor arrowCursor]
-                 forKey:NSCursorAttributeName];
-  scoped_nsobject<NSAttributedString> attributedString(
-      [[NSAttributedString alloc] initWithString:message
-                                      attributes:attributes]);
-  [[label_.get() textStorage] setAttributedString:attributedString];
+  owner_ = NULL;
 }
 
 - (void)removeButtons {
@@ -220,10 +194,26 @@ const float kAnimateCloseDuration = 0.12;
   NSRect labelFrame = [label_.get() frame];
   labelFrame.size.width = NSMaxX([cancelButton_ frame]) - NSMinX(labelFrame);
   [okButton_ removeFromSuperview];
+  okButton_ = nil;
   [cancelButton_ removeFromSuperview];
+  cancelButton_ = nil;
   [label_.get() setFrame:labelFrame];
 }
 
+- (void)disablePopUpMenu:(NSMenu*)menu {
+  // Remove the menu if visible.
+  [menu cancelTracking];
+
+  // If the menu is re-opened, prevent queries to update items.
+  [menu setDelegate:nil];
+
+  // Prevent target/action messages to the controller.
+  for (NSMenuItem* item in [menu itemArray]) {
+    [item setEnabled:NO];
+    [item setTarget:nil];
+  }
+}
+
 @end
 
 @implementation InfoBarController (PrivateMethods)
@@ -233,25 +223,13 @@ const float kAnimateCloseDuration = 0.12;
   // The former doesn't show links in a nice way, but the latter can't be added
   // in IB without a containing scroll view, so create the NSTextView
   // programmatically.
-  label_.reset([[InfoBarTextView alloc]
+  label_.reset([[HyperlinkTextView alloc]
       initWithFrame:[labelPlaceholder_ frame]]);
   [label_.get() setAutoresizingMask:[labelPlaceholder_ autoresizingMask]];
   [[labelPlaceholder_ superview]
       replaceSubview:labelPlaceholder_ with:label_.get()];
   labelPlaceholder_ = nil;  // Now released.
   [label_.get() setDelegate:self];
-  [label_.get() setEditable:NO];
-  [label_.get() setDrawsBackground:NO];
-  [label_.get() setHorizontallyResizable:NO];
-  [label_.get() setVerticallyResizable:NO];
-}
-
-- (void)removeInfoBar {
-  // TODO(rohitrao): This method can be called even if the infobar has already
-  // been removed and |delegate_| is NULL.  Is there a way to rewrite the code
-  // so that inner event loops don't cause us to try and remove the infobar
-  // twice?  http://crbug.com/54253
-  [containerController_ removeDelegate:delegate_];
 }
 
 - (void)cleanUpAfterAnimation:(BOOL)finished {
@@ -259,7 +237,7 @@ const float kAnimateCloseDuration = 0.12;
   if (!infoBarClosing_)
     return;
 
-  // Notify the delegate that the infobar was closed.  The delegate may delete
+  // Notify the delegate that the infobar was closed.  The delegate will delete
   // itself as a result of InfoBarClosed(), so we null out its pointer.
   if (delegate_) {
     delegate_->InfoBarClosed();
@@ -282,75 +260,18 @@ const float kAnimateCloseDuration = 0.12;
   [self cleanUpAfterAnimation:YES];
 }
 
-// TODO(joth): This method factors out some common functionality between the
-// various derived infobar classes, however the class hierarchy itself could
-// use refactoring to reduce this duplication. http://crbug.com/38924
-- (void)setLabelToMessage:(NSString*)message
-                 withLink:(NSString*)link
-                 atOffset:(NSUInteger)linkOffset {
-  if (linkOffset == std::wstring::npos) {
-    // linkOffset == std::wstring::npos means the link should be right-aligned,
-    // which is not supported on Mac (http://crbug.com/47728).
-    NOTIMPLEMENTED();
-    linkOffset = [message length];
+- (NSPoint)pointForTipApex {
+  BrowserWindowController* windowController =
+      [containerController_ browserWindowController];
+  if (!windowController) {
+    // This should only happen in unit tests.
+    return NSZeroPoint;
   }
-  // Create an attributes dictionary for the entire message.  We have
-  // to expicitly set the font the control's font.  We also override
-  // the cursor to give us the normal cursor rather than the text
-  // insertion cursor.
-  NSMutableDictionary* linkAttributes = [NSMutableDictionary dictionary];
-  [linkAttributes setObject:[NSCursor arrowCursor]
-                     forKey:NSCursorAttributeName];
-  NSFont* font = [NSFont labelFontOfSize:
-      [NSFont systemFontSizeForControlSize:NSRegularControlSize]];
-  [linkAttributes setObject:font
-                     forKey:NSFontAttributeName];
 
-  // Create the attributed string for the main message text.
-  scoped_nsobject<NSMutableAttributedString> infoText(
-      [[NSMutableAttributedString alloc] initWithString:message]);
-  [infoText.get() addAttributes:linkAttributes
-                    range:NSMakeRange(0, [infoText.get() length])];
-  // Add additional attributes to style the link text appropriately as
-  // well as linkify it.
-  [linkAttributes setObject:[NSColor blueColor]
-                     forKey:NSForegroundColorAttributeName];
-  [linkAttributes setObject:[NSNumber numberWithBool:YES]
-                     forKey:NSUnderlineStyleAttributeName];
-  [linkAttributes setObject:[NSCursor pointingHandCursor]
-                     forKey:NSCursorAttributeName];
-  [linkAttributes setObject:[NSNumber numberWithInt:NSSingleUnderlineStyle]
-                     forKey:NSUnderlineStyleAttributeName];
-  [linkAttributes setObject:[NSString string]  // dummy value
-                     forKey:NSLinkAttributeName];
-
-  // Insert the link text into the string at the appropriate offset.
-  scoped_nsobject<NSAttributedString> attributedString(
-      [[NSAttributedString alloc] initWithString:link
-                                      attributes:linkAttributes]);
-  [infoText.get() insertAttributedString:attributedString.get()
-                                 atIndex:linkOffset];
-  // Update the label view with the new text.
-  [[label_.get() textStorage] setAttributedString:infoText];
-}
-
-@end
-
-
-/////////////////////////////////////////////////////////////////////////
-// AlertInfoBarController implementation
-
-@implementation AlertInfoBarController
-
-// Alert infobars have a text message.
-- (void)addAdditionalControls {
-  // No buttons.
-  [self removeButtons];
-
-  // Insert the text.
-  AlertInfoBarDelegate* delegate = delegate_->AsAlertInfoBarDelegate();
-  DCHECK(delegate);
-  [self setLabelToMessage:base::SysUTF16ToNSString(delegate->GetMessageText())];
+  LocationBarViewMac* locationBar = [windowController locationBarBridge];
+  NSPoint point = locationBar->GetPageInfoBubblePoint();
+  point = [infoBarView_ convertPoint:point fromView:nil];
+  return point;
 }
 
 @end
@@ -375,19 +296,28 @@ const float kAnimateCloseDuration = 0.12;
   DCHECK(delegate);
   size_t offset = std::wstring::npos;
   string16 message = delegate->GetMessageTextWithOffset(&offset);
-  [self setLabelToMessage:base::SysUTF16ToNSString(message)
-                 withLink:base::SysUTF16ToNSString(delegate->GetLinkText())
-                 atOffset:offset];
+  string16 link = delegate->GetLinkText();
+  NSFont* font = [NSFont labelFontOfSize:
+                  [NSFont systemFontSizeForControlSize:NSRegularControlSize]];
+  HyperlinkTextView* view = (HyperlinkTextView*)label_.get();
+  [view setMessageAndLink:base::SysUTF16ToNSString(message)
+                 withLink:base::SysUTF16ToNSString(link)
+                 atOffset:offset
+                     font:font
+             messageColor:[NSColor blackColor]
+                linkColor:[NSColor blueColor]];
 }
 
 // Called when someone clicks on the link in the infobar.  This method
 // is called by the InfobarTextField on its delegate (the
 // LinkInfoBarController).
 - (void)linkClicked {
+  if (![self isOwned])
+    return;
   WindowOpenDisposition disposition =
       event_utils::WindowOpenDispositionFromNSEvent([NSApp currentEvent]);
-  if (delegate_ && delegate_->AsLinkInfoBarDelegate()->LinkClicked(disposition))
-    [self removeInfoBar];
+  if (delegate_->AsLinkInfoBarDelegate()->LinkClicked(disposition))
+    [self removeSelf];
 }
 
 @end
@@ -400,14 +330,18 @@ const float kAnimateCloseDuration = 0.12;
 
 // Called when someone clicks on the "OK" button.
 - (IBAction)ok:(id)sender {
-  if (delegate_ && delegate_->AsConfirmInfoBarDelegate()->Accept())
-    [self removeInfoBar];
+  if (![self isOwned])
+    return;
+  if (delegate_->AsConfirmInfoBarDelegate()->Accept())
+    [self removeSelf];
 }
 
 // Called when someone clicks on the "Cancel" button.
 - (IBAction)cancel:(id)sender {
-  if (delegate_ && delegate_->AsConfirmInfoBarDelegate()->Cancel())
-    [self removeInfoBar];
+  if (![self isOwned])
+    return;
+  if (delegate_->AsConfirmInfoBarDelegate()->Cancel())
+    [self removeSelf];
 }
 
 // Confirm infobars can have OK and/or cancel buttons, depending on
@@ -421,31 +355,14 @@ const float kAnimateCloseDuration = 0.12;
   NSRect okButtonFrame = [okButton_ frame];
   NSRect cancelButtonFrame = [cancelButton_ frame];
 
-  DCHECK(NSMaxX(okButtonFrame) < NSMinX(cancelButtonFrame))
-      << "Cancel button expected to be on the right of the Ok button in nib";
+  DCHECK(NSMaxX(cancelButtonFrame) < NSMinX(okButtonFrame))
+      << "Ok button expected to be on the right of the Cancel button in nib";
 
-  CGFloat rightEdge = NSMaxX(cancelButtonFrame);
+  CGFloat rightEdge = NSMaxX(okButtonFrame);
   CGFloat spaceBetweenButtons =
-      NSMinX(cancelButtonFrame) - NSMaxX(okButtonFrame);
+      NSMinX(okButtonFrame) - NSMaxX(cancelButtonFrame);
   CGFloat spaceBeforeButtons =
-      NSMinX(okButtonFrame) - NSMaxX([label_.get() frame]);
-
-  // Update and position the Cancel button if needed.  Otherwise, hide it.
-  if (visibleButtons & ConfirmInfoBarDelegate::BUTTON_CANCEL) {
-    [cancelButton_ setTitle:base::SysUTF16ToNSString(
-          delegate->GetButtonLabel(ConfirmInfoBarDelegate::BUTTON_CANCEL))];
-    [GTMUILocalizerAndLayoutTweaker sizeToFitView:cancelButton_];
-    cancelButtonFrame = [cancelButton_ frame];
-
-    // Position the cancel button to the left of the Close button.
-    cancelButtonFrame.origin.x = rightEdge - cancelButtonFrame.size.width;
-    [cancelButton_ setFrame:cancelButtonFrame];
-
-    // Update the rightEdge
-    rightEdge = NSMinX(cancelButtonFrame);
-  } else {
-    [cancelButton_ removeFromSuperview];
-  }
+      NSMinX(cancelButtonFrame) - NSMaxX([label_.get() frame]);
 
   // Update and position the OK button if needed.  Otherwise, hide it.
   if (visibleButtons & ConfirmInfoBarDelegate::BUTTON_OK) {
@@ -454,20 +371,38 @@ const float kAnimateCloseDuration = 0.12;
     [GTMUILocalizerAndLayoutTweaker sizeToFitView:okButton_];
     okButtonFrame = [okButton_ frame];
 
-    // If we had a Cancel button, leave space between the buttons.
-    if (visibleButtons & ConfirmInfoBarDelegate::BUTTON_CANCEL) {
-      rightEdge -= spaceBetweenButtons;
-    }
-
-    // Position the OK button on our current right edge.
+    // Position the ok button to the left of the Close button.
     okButtonFrame.origin.x = rightEdge - okButtonFrame.size.width;
     [okButton_ setFrame:okButtonFrame];
 
-
     // Update the rightEdge
     rightEdge = NSMinX(okButtonFrame);
   } else {
     [okButton_ removeFromSuperview];
+    okButton_ = nil;
+  }
+
+  // Update and position the Cancel button if needed.  Otherwise, hide it.
+  if (visibleButtons & ConfirmInfoBarDelegate::BUTTON_CANCEL) {
+    [cancelButton_ setTitle:base::SysUTF16ToNSString(
+          delegate->GetButtonLabel(ConfirmInfoBarDelegate::BUTTON_CANCEL))];
+    [GTMUILocalizerAndLayoutTweaker sizeToFitView:cancelButton_];
+    cancelButtonFrame = [cancelButton_ frame];
+
+    // If we had a Ok button, leave space between the buttons.
+    if (visibleButtons & ConfirmInfoBarDelegate::BUTTON_OK) {
+      rightEdge -= spaceBetweenButtons;
+    }
+
+    // Position the Cancel button on our current right edge.
+    cancelButtonFrame.origin.x = rightEdge - cancelButtonFrame.size.width;
+    [cancelButton_ setFrame:cancelButtonFrame];
+
+    // Update the rightEdge.
+    rightEdge = NSMinX(cancelButtonFrame);
+  } else {
+    [cancelButton_ removeFromSuperview];
+    cancelButton_ = nil;
   }
 
   // If we had either button, leave space before the edge of the textfield.
@@ -485,31 +420,31 @@ const float kAnimateCloseDuration = 0.12;
   // Set the text and link.
   NSString* message = base::SysUTF16ToNSString(delegate->GetMessageText());
   string16 link = delegate->GetLinkText();
-  if (link.empty()) {
-    // Simple case: no link, so just set the message directly.
-    [self setLabelToMessage:message];
-  } else {
-    // Inserting the link unintentionally causes the text to have a slightly
-    // different result to the simple case above: text is truncated on word
-    // boundaries (if needed) rather than elided with ellipses.
-
+  if (!link.empty()) {
     // Add spacing between the label and the link.
     message = [message stringByAppendingString:@"   "];
-    [self setLabelToMessage:message
-                   withLink:base::SysUTF16ToNSString(link)
-                   atOffset:[message length]];
   }
+  NSFont* font = [NSFont labelFontOfSize:
+      [NSFont systemFontSizeForControlSize:NSRegularControlSize]];
+  HyperlinkTextView* view = (HyperlinkTextView*)label_.get();
+  [view setMessageAndLink:message
+                 withLink:base::SysUTF16ToNSString(link)
+                 atOffset:[message length]
+                     font:font
+             messageColor:[NSColor blackColor]
+                linkColor:[NSColor blueColor]];
 }
 
 // Called when someone clicks on the link in the infobar.  This method
 // is called by the InfobarTextField on its delegate (the
 // LinkInfoBarController).
 - (void)linkClicked {
+  if (![self isOwned])
+    return;
   WindowOpenDisposition disposition =
       event_utils::WindowOpenDispositionFromNSEvent([NSApp currentEvent]);
-  if (delegate_ &&
-      delegate_->AsConfirmInfoBarDelegate()->LinkClicked(disposition))
-    [self removeInfoBar];
+  if (delegate_->AsConfirmInfoBarDelegate()->LinkClicked(disposition))
+    [self removeSelf];
 }
 
 @end
@@ -518,20 +453,14 @@ const float kAnimateCloseDuration = 0.12;
 //////////////////////////////////////////////////////////////////////////
 // CreateInfoBar() implementations
 
-InfoBar* AlertInfoBarDelegate::CreateInfoBar() {
-  AlertInfoBarController* controller =
-      [[AlertInfoBarController alloc] initWithDelegate:this];
-  return new InfoBar(controller);
-}
-
-InfoBar* LinkInfoBarDelegate::CreateInfoBar() {
+InfoBar* LinkInfoBarDelegate::CreateInfoBar(InfoBarTabHelper* owner) {
   LinkInfoBarController* controller =
-      [[LinkInfoBarController alloc] initWithDelegate:this];
-  return new InfoBar(controller);
+      [[LinkInfoBarController alloc] initWithDelegate:this owner:owner];
+  return new InfoBar(controller, this);
 }
 
-InfoBar* ConfirmInfoBarDelegate::CreateInfoBar() {
+InfoBar* ConfirmInfoBarDelegate::CreateInfoBar(InfoBarTabHelper* owner) {
   ConfirmInfoBarController* controller =
-      [[ConfirmInfoBarController alloc] initWithDelegate:this];
-  return new InfoBar(controller);
+      [[ConfirmInfoBarController alloc] initWithDelegate:this owner:owner];
+  return new InfoBar(controller, this);
 }
