@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,8 +9,8 @@
 #include "chrome/browser/ui/app_modal_dialogs/native_app_modal_dialog.h"
 
 #include "base/logging.h"
-#include "base/scoped_nsobject.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_nsobject.h"
+#include "base/memory/scoped_ptr.h"
 
 #if __OBJC__
 @class NSAlert;
@@ -26,12 +26,12 @@ class JSModalDialogCocoa : public NativeAppModalDialog {
   virtual ~JSModalDialogCocoa();
 
   // Overridden from NativeAppModalDialog:
-  virtual int GetAppModalDialogButtons() const;
-  virtual void ShowAppModalDialog();
-  virtual void ActivateAppModalDialog();
-  virtual void CloseAppModalDialog();
-  virtual void AcceptAppModalDialog();
-  virtual void CancelAppModalDialog();
+  virtual int GetAppModalDialogButtons() const OVERRIDE;
+  virtual void ShowAppModalDialog() OVERRIDE;
+  virtual void ActivateAppModalDialog() OVERRIDE;
+  virtual void CloseAppModalDialog() OVERRIDE;
+  virtual void AcceptAppModalDialog() OVERRIDE;
+  virtual void CancelAppModalDialog() OVERRIDE;
 
   JavaScriptAppModalDialog* dialog() const { return dialog_.get(); }
 
