@@ -1,11 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #import <Cocoa/Cocoa.h>
 
 #import "base/mac/cocoa_protocols.h"
-#include "base/scoped_nsobject.h"
+#include "base/memory/scoped_nsobject.h"
 #import "chrome/browser/ui/cocoa/location_bar/autocomplete_text_field.h"
 #import "chrome/browser/ui/cocoa/location_bar/autocomplete_text_field_cell.h"
 #import "chrome/browser/ui/cocoa/location_bar/autocomplete_text_field_editor.h"
@@ -209,7 +209,6 @@ TEST_F(AutocompleteTextFieldTest, Display) {
   // Test focussed drawing.
   [test_window() makePretendKeyWindowAndSetFirstResponder:field_];
   [field_ display];
-  [test_window() clearPretendKeyWindowAndFirstResponder];
 }
 
 TEST_F(AutocompleteTextFieldObserverTest, FlagsChanged) {
@@ -331,6 +330,7 @@ TEST_F(AutocompleteTextFieldObserverTest, ResetFieldEditorContinuesEditing) {
 
   // This should begin editing and indicate a change.
   EXPECT_CALL(field_observer_, OnDidBeginEditing());
+  EXPECT_CALL(field_observer_, OnBeforeChange());
   EXPECT_CALL(field_observer_, OnDidChange());
   [editor shouldChangeTextInRange:NSMakeRange(0, 0) replacementString:@""];
   [editor didChangeText];
@@ -600,7 +600,7 @@ TEST_F(AutocompleteTextFieldTest, RightDecorationMouseDown) {
 // TODO(shess): Really, this should test that things are forwarded to
 // the cell, and the cell tests should test that the right things are
 // selected.  It's easier to mock the event here, though.  This code's
-// event-mockers might be worth promoting to |test_event_utils.h| or
+// event-mockers might be worth promoting to |cocoa_test_event_utils.h| or
 // |cocoa_test_helper.h|.
 TEST_F(AutocompleteTextFieldTest, DecorationMenu) {
   AutocompleteTextFieldCell* cell = [field_ cell];
@@ -749,11 +749,13 @@ TEST_F(AutocompleteTextFieldObserverTest, SendsEditingMessages) {
 
   // This should begin editing and indicate a change.
   EXPECT_CALL(field_observer_, OnDidBeginEditing());
+  EXPECT_CALL(field_observer_, OnBeforeChange());
   EXPECT_CALL(field_observer_, OnDidChange());
   [editor shouldChangeTextInRange:NSMakeRange(0, 0) replacementString:@""];
   [editor didChangeText];
 
   // Further changes don't send the begin message.
+  EXPECT_CALL(field_observer_, OnBeforeChange());
   EXPECT_CALL(field_observer_, OnDidChange());
   [editor shouldChangeTextInRange:NSMakeRange(0, 0) replacementString:@""];
   [editor didChangeText];
