@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 #define CHROME_BROWSER_UI_COCOA_LOCATION_BAR_IMAGE_DECORATION_H_
 #pragma once
 
-#import "base/scoped_nsobject.h"
+#import "base/memory/scoped_nsobject.h"
 #include "chrome/browser/ui/cocoa/location_bar/location_bar_decoration.h"
 
 // |LocationBarDecoration| which sizes and draws itself according to
@@ -24,8 +24,8 @@ class ImageDecoration : public LocationBarDecoration {
   NSRect GetDrawRectInFrame(NSRect frame);
 
   // Implement |LocationBarDecoration|.
-  virtual CGFloat GetWidthForSpace(CGFloat width);
-  virtual void DrawInFrame(NSRect frame, NSView* control_view);
+  virtual CGFloat GetWidthForSpace(CGFloat width) OVERRIDE;
+  virtual void DrawInFrame(NSRect frame, NSView* control_view) OVERRIDE;
 
  private:
   scoped_nsobject<NSImage> image_;
