@@ -1,22 +1,20 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #import "chrome/browser/ui/cocoa/location_bar/location_bar_view_mac.h"
 
-#include "base/stl_util-inl.h"
+#include "base/bind.h"
+#include "base/message_loop.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
 #include "base/sys_string_conversions.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/alternate_nav_url_fetcher.h"
 #import "chrome/browser/app_controller_mac.h"
-#import "chrome/browser/autocomplete/autocomplete_edit_view_mac.h"
 #import "chrome/browser/autocomplete/autocomplete_popup_model.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/command_updater.h"
-#include "chrome/browser/content_setting_image_model.h"
-#include "chrome/browser/content_setting_bubble_model.h"
 #include "chrome/browser/defaults.h"
 #include "chrome/browser/extensions/extension_browser_event_router.h"
 #include "chrome/browser/extensions/extension_service.h"
@@ -24,9 +22,9 @@
 #include "chrome/browser/instant/instant_controller.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search_engines/template_url.h"
-#include "chrome/browser/search_engines/template_url_model.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/search_engines/template_url_service.h"
+#include "chrome/browser/search_engines/template_url_service_factory.h"
+#include "chrome/browser/ui/browser_list.h"
 #import "chrome/browser/ui/cocoa/content_settings/content_setting_bubble_cocoa.h"
 #include "chrome/browser/ui/cocoa/event_utils.h"
 #import "chrome/browser/ui/cocoa/extensions/extension_action_context_menu.h"
@@ -41,19 +39,27 @@
 #import "chrome/browser/ui/cocoa/location_bar/page_action_decoration.h"
 #import "chrome/browser/ui/cocoa/location_bar/selected_keyword_decoration.h"
 #import "chrome/browser/ui/cocoa/location_bar/star_decoration.h"
+#import "chrome/browser/ui/cocoa/omnibox/omnibox_view_mac.h"
+#include "chrome/browser/ui/content_settings/content_setting_bubble_model.h"
+#include "chrome/browser/ui/content_settings/content_setting_image_model.h"
 #include "chrome/browser/ui/omnibox/location_bar_util.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_action.h"
 #include "chrome/common/extensions/extension_resource.h"
-#include "chrome/common/notification_service.h"
-#include "net/base/net_util.h"
+#include "chrome/common/pref_names.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "net/base/net_util.h"
 #include "skia/ext/skia_utils_mac.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/l10n/l10n_util_mac.h"
 #include "ui/base/resource/resource_bundle.h"
 
+using content::WebContents;
+
 namespace {
 
 // Vertical space between the bottom edge of the location_bar and the first run
@@ -71,27 +77,27 @@ LocationBarViewMac::LocationBarViewMac(
     ToolbarModel* toolbar_model,
     Profile* profile,
     Browser* browser)
-    : edit_view_(new AutocompleteEditViewMac(this, toolbar_model, profile,
-                                             command_updater, field)),
+    : omnibox_view_(new OmniboxViewMac(this, toolbar_model, profile,
+                                       command_updater, field)),
       command_updater_(command_updater),
       field_(field),
       disposition_(CURRENT_TAB),
       location_icon_decoration_(new LocationIconDecoration(this)),
       selected_keyword_decoration_(
-          new SelectedKeywordDecoration(
-              AutocompleteEditViewMac::GetFieldFont())),
+          new SelectedKeywordDecoration(OmniboxViewMac::GetFieldFont())),
       ev_bubble_decoration_(
           new EVBubbleDecoration(location_icon_decoration_.get(),
-                                 AutocompleteEditViewMac::GetFieldFont())),
+                                 OmniboxViewMac::GetFieldFont())),
       star_decoration_(new StarDecoration(command_updater)),
       keyword_hint_decoration_(
-          new KeywordHintDecoration(AutocompleteEditViewMac::GetFieldFont())),
+          new KeywordHintDecoration(OmniboxViewMac::GetFieldFont())),
       profile_(profile),
       browser_(browser),
       toolbar_model_(toolbar_model),
-      update_instant_(true),
-      transition_(PageTransition::TYPED),
-      first_run_bubble_(this) {
+      transition_(content::PageTransitionFromInt(
+          content::PAGE_TRANSITION_TYPED |
+          content::PAGE_TRANSITION_FROM_ADDRESS_BAR)),
+      weak_ptr_factory_(this) {
   for (size_t i = 0; i < CONTENT_SETTINGS_NUM_TYPES; ++i) {
     DCHECK_EQ(i, content_setting_decorations_.size());
     ContentSettingsType type = static_cast<ContentSettingsType>(i);
@@ -100,8 +106,11 @@ LocationBarViewMac::LocationBarViewMac(
   }
 
   registrar_.Add(this,
-      NotificationType::EXTENSION_PAGE_ACTION_VISIBILITY_CHANGED,
-      NotificationService::AllSources());
+      chrome::NOTIFICATION_EXTENSION_PAGE_ACTION_VISIBILITY_CHANGED,
+      content::NotificationService::AllSources());
+
+  edit_bookmarks_enabled_.Init(prefs::kEditBookmarksEnabled,
+                               profile_->GetPrefs(), this);
 }
 
 LocationBarViewMac::~LocationBarViewMac() {
@@ -109,16 +118,15 @@ LocationBarViewMac::~LocationBarViewMac() {
   [[field_ cell] clearDecorations];
 }
 
-void LocationBarViewMac::ShowFirstRunBubble(FirstRun::BubbleType bubble_type) {
+void LocationBarViewMac::ShowFirstRunBubble() {
   // We need the browser window to be shown before we can show the bubble, but
   // we get called before that's happened.
-  Task* task = first_run_bubble_.NewRunnableMethod(
-      &LocationBarViewMac::ShowFirstRunBubbleInternal, bubble_type);
-  MessageLoop::current()->PostTask(FROM_HERE, task);
+  MessageLoop::current()->PostTask(FROM_HERE,
+      base::Bind(&LocationBarViewMac::ShowFirstRunBubbleInternal,
+          weak_ptr_factory_.GetWeakPtr()));
 }
 
-void LocationBarViewMac::ShowFirstRunBubbleInternal(
-    FirstRun::BubbleType bubble_type) {
+void LocationBarViewMac::ShowFirstRunBubbleInternal() {
   if (!field_ || ![field_ window])
     return;
 
@@ -134,35 +142,35 @@ void LocationBarViewMac::ShowFirstRunBubbleInternal(
   [FirstRunBubbleController showForView:field_ offset:kOffset profile:profile_];
 }
 
-std::wstring LocationBarViewMac::GetInputString() const {
+string16 LocationBarViewMac::GetInputString() const {
   return location_input_;
 }
 
-void LocationBarViewMac::SetSuggestedText(const string16& text) {
-  edit_view_->SetInstantSuggestion(
-      edit_view_->model()->UseVerbatimInstant() ? string16() : text);
+void LocationBarViewMac::SetSuggestedText(const string16& text,
+                                          InstantCompleteBehavior behavior) {
+  omnibox_view_->model()->SetSuggestedText(text, behavior);
 }
 
 WindowOpenDisposition LocationBarViewMac::GetWindowOpenDisposition() const {
   return disposition_;
 }
 
-PageTransition::Type LocationBarViewMac::GetPageTransition() const {
+content::PageTransition LocationBarViewMac::GetPageTransition() const {
   return transition_;
 }
 
 void LocationBarViewMac::AcceptInput() {
   WindowOpenDisposition disposition =
       event_utils::WindowOpenDispositionFromNSEvent([NSApp currentEvent]);
-  edit_view_->model()->AcceptInput(disposition, false);
+  omnibox_view_->model()->AcceptInput(disposition, false);
 }
 
 void LocationBarViewMac::FocusLocation(bool select_all) {
-  edit_view_->FocusLocation(select_all);
+  omnibox_view_->FocusLocation(select_all);
 }
 
 void LocationBarViewMac::FocusSearch() {
-  edit_view_->SetForcedQuery();
+  omnibox_view_->SetForcedQuery();
 }
 
 void LocationBarViewMac::UpdateContentSettingsIcons() {
@@ -177,10 +185,10 @@ void LocationBarViewMac::UpdatePageActions() {
   RefreshPageActionDecorations();
   Layout();
   if (page_action_decorations_.size() != count_before) {
-    NotificationService::current()->Notify(
-        NotificationType::EXTENSION_PAGE_ACTION_COUNT_CHANGED,
-        Source<LocationBar>(this),
-        NotificationService::NoDetails());
+    content::NotificationService::current()->Notify(
+        chrome::NOTIFICATION_EXTENSION_PAGE_ACTION_COUNT_CHANGED,
+        content::Source<LocationBar>(this),
+        content::NotificationService::NoDetails());
   }
 }
 
@@ -189,89 +197,42 @@ void LocationBarViewMac::InvalidatePageActions() {
   DeletePageActionDecorations();
   Layout();
   if (page_action_decorations_.size() != count_before) {
-    NotificationService::current()->Notify(
-        NotificationType::EXTENSION_PAGE_ACTION_COUNT_CHANGED,
-        Source<LocationBar>(this),
-        NotificationService::NoDetails());
+    content::NotificationService::current()->Notify(
+        chrome::NOTIFICATION_EXTENSION_PAGE_ACTION_COUNT_CHANGED,
+        content::Source<LocationBar>(this),
+        content::NotificationService::NoDetails());
   }
 }
 
-void LocationBarViewMac::SaveStateToContents(TabContents* contents) {
+void LocationBarViewMac::SaveStateToContents(WebContents* contents) {
   // TODO(shess): Why SaveStateToContents vs SaveStateToTab?
-  edit_view_->SaveStateToTab(contents);
+  omnibox_view_->SaveStateToTab(contents);
 }
 
-void LocationBarViewMac::Update(const TabContents* contents,
+void LocationBarViewMac::Update(const WebContents* contents,
                                 bool should_restore_state) {
-  bool star_enabled = browser_defaults::bookmarks_enabled &&
-      [field_ isEditable] && !toolbar_model_->input_in_progress();
+  bool star_enabled = IsStarEnabled();
   command_updater_->UpdateCommandEnabled(IDC_BOOKMARK_PAGE, star_enabled);
   star_decoration_->SetVisible(star_enabled);
   RefreshPageActionDecorations();
   RefreshContentSettingsDecorations();
-  // AutocompleteEditView restores state if the tab is non-NULL.
-  edit_view_->Update(should_restore_state ? contents : NULL);
+  // OmniboxView restores state if the tab is non-NULL.
+  omnibox_view_->Update(should_restore_state ? contents : NULL);
   OnChanged();
 }
 
-void LocationBarViewMac::OnAutocompleteWillClosePopup() {
-  if (!update_instant_)
-    return;
-
-  InstantController* controller = browser_->instant();
-  if (controller && !controller->commit_on_mouse_up())
-    controller->DestroyPreviewContents();
-  SetSuggestedText(string16());
-}
-
-void LocationBarViewMac::OnAutocompleteLosingFocus(gfx::NativeView unused) {
-  SetSuggestedText(string16());
-
-  InstantController* instant = browser_->instant();
-  if (!instant)
-    return;
-
-  // If |IsMouseDownFromActivate()| returns false, the RenderWidgetHostView did
-  // not receive a mouseDown event.  Therefore, we should destroy the preview.
-  // Otherwise, the RWHV was clicked, so we commit the preview.
-  if (!instant->is_displayable() || !instant->GetPreviewContents() ||
-      !instant->IsMouseDownFromActivate()) {
-    instant->DestroyPreviewContents();
-  } else if (instant->IsShowingInstant()) {
-    instant->SetCommitOnMouseUp();
-  } else {
-    instant->CommitCurrentPreview(INSTANT_COMMIT_FOCUS_LOST);
-  }
-}
-
-void LocationBarViewMac::OnAutocompleteWillAccept() {
-  update_instant_ = false;
-}
-
-bool LocationBarViewMac::OnCommitSuggestedText(const std::wstring& typed_text) {
-  return edit_view_->CommitSuggestText();
-}
-
-bool LocationBarViewMac::AcceptCurrentInstantPreview() {
-  return InstantController::CommitIfCurrent(browser_->instant());
-}
-
-void LocationBarViewMac::OnPopupBoundsChanged(const gfx::Rect& bounds) {
-  InstantController* instant = browser_->instant();
-  if (instant)
-    instant->SetOmniboxBounds(bounds);
-}
-
-void LocationBarViewMac::OnAutocompleteAccept(const GURL& url,
-                                              WindowOpenDisposition disposition,
-                                              PageTransition::Type transition,
-                                              const GURL& alternate_nav_url) {
+void LocationBarViewMac::OnAutocompleteAccept(
+    const GURL& url,
+    WindowOpenDisposition disposition,
+    content::PageTransition transition,
+    const GURL& alternate_nav_url) {
   // WARNING: don't add an early return here. The calls after the if must
   // happen.
   if (url.is_valid()) {
-    location_input_ = UTF8ToWide(url.spec());
+    location_input_ = UTF8ToUTF16(url.spec());
     disposition_ = disposition;
-    transition_ = transition;
+    transition_ = content::PageTransitionFromInt(
+        transition | content::PAGE_TRANSITION_FROM_ADDRESS_BAR);
 
     if (command_updater_) {
       if (!alternate_nav_url.is_valid()) {
@@ -294,44 +255,15 @@ void LocationBarViewMac::OnAutocompleteAccept(const GURL& url,
       }
     }
   }
-
-  if (browser_->instant() && !edit_view_->model()->popup_model()->IsOpen())
-    browser_->instant()->DestroyPreviewContents();
-
-  update_instant_ = true;
 }
 
 void LocationBarViewMac::OnChanged() {
   // Update the location-bar icon.
-  const int resource_id = edit_view_->GetIcon();
-  NSImage* image = AutocompleteEditViewMac::ImageForResource(resource_id);
+  const int resource_id = omnibox_view_->GetIcon();
+  NSImage* image = OmniboxViewMac::ImageForResource(resource_id);
   location_icon_decoration_->SetImage(image);
   ev_bubble_decoration_->SetImage(image);
   Layout();
-
-  InstantController* instant = browser_->instant();
-  string16 suggested_text;
-  if (update_instant_ && instant && GetTabContents()) {
-    if (edit_view_->model()->user_input_in_progress() &&
-        edit_view_->model()->popup_model()->IsOpen()) {
-      instant->Update
-          (browser_->GetSelectedTabContentsWrapper(),
-           edit_view_->model()->CurrentMatch(),
-           WideToUTF16(edit_view_->GetText()),
-           edit_view_->model()->UseVerbatimInstant(),
-           &suggested_text);
-      if (!instant->MightSupportInstant()) {
-        edit_view_->model()->FinalizeInstantQuery(std::wstring(),
-                                                  std::wstring());
-      }
-    } else {
-      instant->DestroyPreviewContents();
-      edit_view_->model()->FinalizeInstantQuery(std::wstring(),
-                                                std::wstring());
-    }
-  }
-
-  SetSuggestedText(suggested_text);
 }
 
 void LocationBarViewMac::OnSelectionBoundsChanged() {
@@ -352,18 +284,36 @@ void LocationBarViewMac::OnKillFocus() {
   // Do nothing.
 }
 
-SkBitmap LocationBarViewMac::GetFavIcon() const {
-  NOTIMPLEMENTED();
-  return SkBitmap();
+SkBitmap LocationBarViewMac::GetFavicon() const {
+  return browser_->GetCurrentPageIcon();
 }
 
-std::wstring LocationBarViewMac::GetTitle() const {
-  NOTIMPLEMENTED();
-  return std::wstring();
+string16 LocationBarViewMac::GetTitle() const {
+  return browser_->GetWindowTitleForCurrentTab();
+}
+
+InstantController* LocationBarViewMac::GetInstant() {
+  return browser_->instant();
+}
+
+TabContentsWrapper* LocationBarViewMac::GetTabContentsWrapper() const {
+  return browser_->GetSelectedTabContentsWrapper();
 }
 
 void LocationBarViewMac::Revert() {
-  edit_view_->RevertAll();
+  omnibox_view_->RevertAll();
+}
+
+const OmniboxView* LocationBarViewMac::location_entry() const {
+    return omnibox_view_.get();
+  }
+
+OmniboxView* LocationBarViewMac::location_entry() {
+    return omnibox_view_.get();
+  }
+
+LocationBarTesting* LocationBarViewMac::GetLocationBarForTesting() {
+  return this;
 }
 
 // TODO(pamg): Change all these, here and for other platforms, to size_t.
@@ -380,8 +330,8 @@ int LocationBarViewMac::PageActionVisibleCount() {
   return result;
 }
 
-TabContents* LocationBarViewMac::GetTabContents() const {
-  return browser_->GetSelectedTabContents();
+WebContents* LocationBarViewMac::GetWebContents() const {
+  return browser_->GetSelectedWebContents();
 }
 
 PageActionDecoration* LocationBarViewMac::GetPageActionDecoration(
@@ -400,7 +350,7 @@ PageActionDecoration* LocationBarViewMac::GetPageActionDecoration(
 void LocationBarViewMac::SetPreviewEnabledPageAction(
     ExtensionAction* page_action, bool preview_enabled) {
   DCHECK(page_action);
-  TabContents* contents = GetTabContents();
+  WebContents* contents = GetWebContents();
   if (!contents)
     return;
   RefreshPageActionDecorations();
@@ -412,8 +362,7 @@ void LocationBarViewMac::SetPreviewEnabledPageAction(
     return;
 
   decoration->set_preview_enabled(preview_enabled);
-  decoration->UpdateVisibility(contents,
-      GURL(WideToUTF8(toolbar_model_->GetText())));
+  decoration->UpdateVisibility(contents, GURL(toolbar_model_->GetText()));
 }
 
 NSPoint LocationBarViewMac::GetPageActionBubblePoint(
@@ -475,8 +424,7 @@ void LocationBarViewMac::TestPageActionPressed(size_t index) {
 
 void LocationBarViewMac::SetEditable(bool editable) {
   [field_ setEditable:editable ? YES : NO];
-  star_decoration_->SetVisible(browser_defaults::bookmarks_enabled &&
-      editable && !toolbar_model_->input_in_progress());
+  star_decoration_->SetVisible(IsStarEnabled());
   UpdatePageActions();
   Layout();
 }
@@ -520,32 +468,37 @@ NSPoint LocationBarViewMac::GetPageInfoBubblePoint() const {
   }
 }
 
-NSImage* LocationBarViewMac::GetKeywordImage(const std::wstring& keyword) {
-  const TemplateURL* template_url =
-      profile_->GetTemplateURLModel()->GetTemplateURLForKeyword(
-          WideToUTF16Hack(keyword));
+NSImage* LocationBarViewMac::GetKeywordImage(const string16& keyword) {
+  const TemplateURL* template_url = TemplateURLServiceFactory::GetForProfile(
+      profile_)->GetTemplateURLForKeyword(keyword);
   if (template_url && template_url->IsExtensionKeyword()) {
     const SkBitmap& bitmap = profile_->GetExtensionService()->
         GetOmniboxIcon(template_url->GetExtensionId());
     return gfx::SkBitmapToNSImage(bitmap);
   }
 
-  return AutocompleteEditViewMac::ImageForResource(IDR_OMNIBOX_SEARCH);
+  return OmniboxViewMac::ImageForResource(IDR_OMNIBOX_SEARCH);
 }
 
-void LocationBarViewMac::Observe(NotificationType type,
-                                 const NotificationSource& source,
-                                 const NotificationDetails& details) {
-  switch (type.value) {
-    case NotificationType::EXTENSION_PAGE_ACTION_VISIBILITY_CHANGED: {
-      TabContents* contents = GetTabContents();
-      if (Details<TabContents>(contents) != details)
+void LocationBarViewMac::Observe(int type,
+                                 const content::NotificationSource& source,
+                                 const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_EXTENSION_PAGE_ACTION_VISIBILITY_CHANGED: {
+      WebContents* contents = GetWebContents();
+      if (content::Details<WebContents>(contents) != details)
         return;
 
       [field_ updateCursorAndToolTipRects];
       [field_ setNeedsDisplay:YES];
       break;
     }
+
+    case chrome::NOTIFICATION_PREF_CHANGED:
+      star_decoration_->SetVisible(IsStarEnabled());
+      OnChanged();
+      break;
+
     default:
       NOTREACHED() << "Unexpected notification";
       break;
@@ -559,12 +512,12 @@ void LocationBarViewMac::PostNotification(NSString* notification) {
 
 bool LocationBarViewMac::RefreshContentSettingsDecorations() {
   const bool input_in_progress = toolbar_model_->input_in_progress();
-  TabContents* tab_contents =
-      input_in_progress ? NULL : browser_->GetSelectedTabContents();
+  WebContents* web_contents =
+      input_in_progress ? NULL : browser_->GetSelectedWebContents();
   bool icons_updated = false;
   for (size_t i = 0; i < content_setting_decorations_.size(); ++i) {
     icons_updated |=
-        content_setting_decorations_[i]->UpdateFromTabContents(tab_contents);
+        content_setting_decorations_[i]->UpdateFromWebContents(web_contents);
   }
   return icons_updated;
 }
@@ -588,10 +541,12 @@ void LocationBarViewMac::RefreshPageActionDecorations() {
   if (!service)
     return;
 
+  // Find all the page actions.
   std::vector<ExtensionAction*> page_actions;
-  for (size_t i = 0; i < service->extensions()->size(); ++i) {
-    if (service->extensions()->at(i)->page_action())
-      page_actions.push_back(service->extensions()->at(i)->page_action());
+  for (ExtensionSet::const_iterator it = service->extensions()->begin();
+       it != service->extensions()->end(); ++it) {
+    if ((*it)->page_action())
+      page_actions.push_back((*it)->page_action());
   }
 
   // On startup we sometimes haven't loaded any extensions. This makes sure
@@ -608,11 +563,11 @@ void LocationBarViewMac::RefreshPageActionDecorations() {
   if (page_action_decorations_.empty())
     return;
 
-  TabContents* contents = GetTabContents();
+  WebContents* contents = GetWebContents();
   if (!contents)
     return;
 
-  GURL url = GURL(WideToUTF8(toolbar_model_->GetText()));
+  GURL url = GURL(toolbar_model_->GetText());
   for (size_t i = 0; i < page_action_decorations_.size(); ++i) {
     page_action_decorations_[i]->UpdateVisibility(
         toolbar_model_->input_in_progress() ? NULL : contents, url);
@@ -651,30 +606,29 @@ void LocationBarViewMac::Layout() {
   keyword_hint_decoration_->SetVisible(false);
 
   // Get the keyword to use for keyword-search and hinting.
-  const std::wstring keyword(edit_view_->model()->keyword());
+  const string16 keyword = omnibox_view_->model()->keyword();
   string16 short_name;
   bool is_extension_keyword = false;
   if (!keyword.empty()) {
-    short_name = profile_->GetTemplateURLModel()->
-        GetKeywordShortName(WideToUTF16Hack(keyword), &is_extension_keyword);
+    short_name = TemplateURLServiceFactory::GetForProfile(profile_)->
+        GetKeywordShortName(keyword, &is_extension_keyword);
   }
 
-  const bool is_keyword_hint = edit_view_->model()->is_keyword_hint();
+  const bool is_keyword_hint = omnibox_view_->model()->is_keyword_hint();
 
   if (!keyword.empty() && !is_keyword_hint) {
     // Switch from location icon to keyword mode.
     location_icon_decoration_->SetVisible(false);
     selected_keyword_decoration_->SetVisible(true);
-    selected_keyword_decoration_->SetKeyword(UTF16ToWideHack(short_name),
-                                             is_extension_keyword);
+    selected_keyword_decoration_->SetKeyword(short_name, is_extension_keyword);
     selected_keyword_decoration_->SetImage(GetKeywordImage(keyword));
   } else if (toolbar_model_->GetSecurityLevel() == ToolbarModel::EV_SECURE) {
     // Switch from location icon to show the EV bubble instead.
     location_icon_decoration_->SetVisible(false);
     ev_bubble_decoration_->SetVisible(true);
 
-    std::wstring label(toolbar_model_->GetEVCertName());
-    ev_bubble_decoration_->SetFullLabel(base::SysWideToNSString(label));
+    string16 label(toolbar_model_->GetEVCertName());
+    ev_bubble_decoration_->SetFullLabel(base::SysUTF16ToNSString(label));
   } else if (!keyword.empty() && is_keyword_hint) {
     keyword_hint_decoration_->SetKeyword(short_name,
                                          is_extension_keyword);
@@ -690,3 +644,10 @@ void LocationBarViewMac::Layout() {
 
   [field_ setNeedsDisplay:YES];
 }
+
+bool LocationBarViewMac::IsStarEnabled() {
+  return [field_ isEditable] &&
+         browser_defaults::bookmarks_enabled &&
+         !toolbar_model_->input_in_progress() &&
+         edit_bookmarks_enabled_.GetValue();
+}
