@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -21,22 +21,22 @@ class LocationIconDecoration : public ImageDecoration {
   virtual ~LocationIconDecoration();
 
   // Allow dragging the current URL.
-  virtual bool IsDraggable();
-  virtual NSPasteboard* GetDragPasteboard();
-  virtual NSImage* GetDragImage() { return GetImage(); }
-  virtual NSRect GetDragImageFrame(NSRect frame) {
-    return GetDrawRectInFrame(frame);
-  }
+  virtual bool IsDraggable() OVERRIDE;
+  virtual NSPasteboard* GetDragPasteboard() OVERRIDE;
+  virtual NSImage* GetDragImage() OVERRIDE;
+  virtual NSRect GetDragImageFrame(NSRect frame) OVERRIDE;
 
   // Get the point where the page info bubble should point within the
   // decoration's frame, in the |owner_|'s coordinates.
   NSPoint GetBubblePointInFrame(NSRect frame);
 
   // Show the page info panel on click.
-  virtual bool OnMousePressed(NSRect frame);
-  virtual bool AcceptsMousePress() { return true; }
+  virtual bool OnMousePressed(NSRect frame) OVERRIDE;
+  virtual bool AcceptsMousePress() OVERRIDE;
+  virtual NSString* GetToolTip() OVERRIDE;
 
  private:
+  NSRect drag_frame_;
   // The location bar view that owns us.
   LocationBarViewMac* owner_;
 
