@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,20 +10,22 @@
 
 #import <Cocoa/Cocoa.h>
 
+#include "base/string16.h"
 #include "chrome/browser/ui/cocoa/location_bar/bubble_decoration.h"
 
 class SelectedKeywordDecoration : public BubbleDecoration {
  public:
   SelectedKeywordDecoration(NSFont* font);
+  virtual ~SelectedKeywordDecoration();
 
   // Calculates appropriate full and partial label strings based on
   // inputs.
-  void SetKeyword(const std::wstring& keyword, bool is_extension_keyword);
+  void SetKeyword(const string16& keyword, bool is_extension_keyword);
 
   // Determines what combination of labels and image will best fit
   // within |width|, makes those current for |BubbleDecoration|, and
   // return the resulting width.
-  virtual CGFloat GetWidthForSpace(CGFloat width);
+  virtual CGFloat GetWidthForSpace(CGFloat width) OVERRIDE;
 
   void SetImage(NSImage* image);
 
