@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,7 @@
 
 #import <Cocoa/Cocoa.h>
 
-#include "base/scoped_nsobject.h"
+#include "base/memory/scoped_nsobject.h"
 #import "chrome/browser/ui/cocoa/url_drop_target.h"
 
 @class NewTabButton;
@@ -32,10 +32,13 @@
   NSPoint dropArrowPosition_;
 }
 
-@property(assign, nonatomic) IBOutlet NewTabButton* newTabButton;
 @property(assign, nonatomic) BOOL dropArrowShown;
 @property(assign, nonatomic) NSPoint dropArrowPosition;
 
+// Name starts with "get" because methods staring with "new" return retained
+// objects according to Cocoa's create rule.
+- (NewTabButton*)getNewTabButton;
+
 @end
 
 // Protected methods subclasses can override to alter behavior. Clients should
@@ -45,4 +48,8 @@
 - (BOOL)doubleClickMinimizesWindow;
 @end
 
+@interface TabStripView(TestingAPI)
+- (void)setNewTabButton:(NewTabButton*)button;
+@end
+
 #endif  // CHROME_BROWSER_UI_COCOA_TABS_TAB_STRIP_VIEW_H_
