@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #include <algorithm>
 #include <vector>
 
+#include "base/mac/bundle_locations.h"
 #include "base/mac/mac_util.h"
 #include "base/sys_string_conversions.h"
 #include "chrome/browser/browser_process.h"
@@ -31,6 +32,7 @@ const struct ColumnWidth {
 } columnWidths[] = {
   // Note that arraysize includes the trailing \0. That's intended.
   { IDS_TASK_MANAGER_PAGE_COLUMN, 120, 600 },
+  { IDS_TASK_MANAGER_PROFILE_NAME_COLUMN, 60, 200 },
   { IDS_TASK_MANAGER_PHYSICAL_MEM_COLUMN,
       arraysize("800 MiB") * kCharWidth, -1 },
   { IDS_TASK_MANAGER_SHARED_MEM_COLUMN,
@@ -49,6 +51,8 @@ const struct ColumnWidth {
       arraysize("2000.0K (2000.0 live)") * kCharWidth, -1 },
   { IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN,
       arraysize("2000.0K (2000.0 live)") * kCharWidth, -1 },
+  { IDS_TASK_MANAGER_FPS_COLUMN,
+      arraysize("100") * kCharWidth, -1 },
   { IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN,
       arraysize("800 kB") * kCharWidth, -1 },
   { IDS_TASK_MANAGER_JAVASCRIPT_MEMORY_ALLOCATED_COLUMN,
@@ -104,7 +108,7 @@ class SortHelper {
 
 - (id)initWithTaskManagerObserver:(TaskManagerMac*)taskManagerObserver
      highlightBackgroundResources:(bool)highlightBackgroundResources {
-  NSString* nibpath = [base::mac::MainAppBundle()
+  NSString* nibpath = [base::mac::FrameworkBundle()
                         pathForResource:@"TaskManager"
                                  ofType:@"nib"];
   if ((self = [super initWithWindowNibPath:nibpath owner:self])) {
@@ -161,7 +165,7 @@ class SortHelper {
   // Use the model indices to get the new view indices of the selection, and
   // set selection to that. This assumes that no rows were added or removed
   // (in that case, the selection is cleared before -reloadData is called).
-  if (modelSelection.size() > 0)
+  if (!modelSelection.empty())
     DCHECK_EQ([tableView_ numberOfRows], model_->ResourceCount());
   NSMutableIndexSet* indexSet = [NSMutableIndexSet indexSet];
   for (size_t i = 0; i < modelSelection.size(); ++i)
@@ -218,8 +222,10 @@ class SortHelper {
   scoped_nsobject<NSTableColumn> column([[NSTableColumn alloc]
       initWithIdentifier:[NSNumber numberWithInt:columnId]]);
 
-  NSTextAlignment textAlignment = columnId == IDS_TASK_MANAGER_PAGE_COLUMN ?
-      NSLeftTextAlignment : NSRightTextAlignment;
+  NSTextAlignment textAlignment =
+      (columnId == IDS_TASK_MANAGER_PAGE_COLUMN ||
+       columnId == IDS_TASK_MANAGER_PROFILE_NAME_COLUMN) ?
+          NSLeftTextAlignment : NSRightTextAlignment;
 
   [[column.get() headerCell]
       setStringValue:l10n_util::GetNSStringWithFixup(columnId)];
@@ -283,7 +289,7 @@ class SortHelper {
   // Initially, sort on the tab name.
   [tableView_ setSortDescriptors:
       [NSArray arrayWithObject:[nameColumn sortDescriptorPrototype]]];
-
+  [self addColumnWithId:IDS_TASK_MANAGER_PROFILE_NAME_COLUMN visible:NO];
   [self addColumnWithId:IDS_TASK_MANAGER_PHYSICAL_MEM_COLUMN visible:YES];
   [self addColumnWithId:IDS_TASK_MANAGER_SHARED_MEM_COLUMN visible:NO];
   [self addColumnWithId:IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN visible:NO];
@@ -295,6 +301,7 @@ class SortHelper {
   [self addColumnWithId:IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN
                 visible:NO];
   [self addColumnWithId:IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN visible:NO];
+  [self addColumnWithId:IDS_TASK_MANAGER_FPS_COLUMN visible:YES];
   [self addColumnWithId:IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN visible:NO];
   [self addColumnWithId:IDS_TASK_MANAGER_JAVASCRIPT_MEMORY_ALLOCATED_COLUMN
                 visible:NO];
@@ -394,8 +401,13 @@ class SortHelper {
   if ([cell respondsToSelector:@selector(setBackgroundColor:)]) {
     NSColor* color = nil;
     if (taskManagerObserver_->IsBackgroundRow(viewToModelMap_[row]) &&
-        ![tableView isRowSelected:row])
+        ![tableView isRowSelected:row]) {
       color = backgroundResourceColor_.get();
+      if ((row % 2) == 1 && [tableView usesAlternatingRowBackgroundColors]) {
+        color = [color blendedColorWithFraction:0.05
+                                        ofColor:[NSColor blackColor]];
+      }
+    }
     [cell setBackgroundColor:color];
 
     // The icon at the left is an |NSButtonCell|, which does not
@@ -421,6 +433,9 @@ class SortHelper {
     case IDS_TASK_MANAGER_PAGE_COLUMN:  // Process
       return base::SysUTF16ToNSString(model_->GetResourceTitle(row));
 
+    case IDS_TASK_MANAGER_PROFILE_NAME_COLUMN:  // Profile Name
+      return base::SysUTF16ToNSString(model_->GetResourceProfileName(row));
+
     case IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN:  // Memory
       if (!model_->IsResourceFirstInGroup(row))
         return @"";
@@ -467,6 +482,9 @@ class SortHelper {
       return base::SysUTF16ToNSString(
           model_->GetResourceWebCoreCSSCacheSize(row));
 
+    case IDS_TASK_MANAGER_FPS_COLUMN:
+      return base::SysUTF16ToNSString(model_->GetResourceFPS(row));
+
     case IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN:
       if (!model_->IsResourceFirstInGroup(row))
         return @"";
