@@ -6,68 +6,68 @@
 
 #include <algorithm>
 
-#include "ui/base/l10n/l10n_util.h"
-#include "app/mac/nsimage_cache.h"
+#include "base/mac/bundle_locations.h"
 #include "base/mac/mac_util.h"
-#include "base/singleton.h"
+#include "base/memory/singleton.h"
+#include "base/string_util.h"
 #include "base/sys_string_conversions.h"
+#include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/autocomplete/autocomplete.h"
 #include "chrome/browser/autocomplete/autocomplete_classifier.h"
-#include "chrome/browser/autocomplete/autocomplete_edit_view.h"
 #include "chrome/browser/autocomplete/autocomplete_match.h"
-#include "chrome/browser/background_page_tracker.h"
 #include "chrome/browser/net/url_fixer_upper.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/search_engines/template_url_model.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
-#include "chrome/browser/upgrade_detector.h"
+#include "chrome/browser/search_engines/template_url_service.h"
+#include "chrome/browser/themes/theme_service.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_window.h"
-#import "chrome/browser/ui/cocoa/accelerators_cocoa.h"
 #import "chrome/browser/ui/cocoa/background_gradient_view.h"
-#import "chrome/browser/ui/cocoa/encoding_menu_controller_delegate_mac.h"
+#include "chrome/browser/ui/cocoa/drag_util.h"
 #import "chrome/browser/ui/cocoa/extensions/browser_action_button.h"
 #import "chrome/browser/ui/cocoa/extensions/browser_actions_container_view.h"
 #import "chrome/browser/ui/cocoa/extensions/browser_actions_controller.h"
 #import "chrome/browser/ui/cocoa/gradient_button_cell.h"
+#import "chrome/browser/ui/cocoa/image_button_cell.h"
 #import "chrome/browser/ui/cocoa/location_bar/autocomplete_text_field_editor.h"
 #import "chrome/browser/ui/cocoa/location_bar/location_bar_view_mac.h"
 #import "chrome/browser/ui/cocoa/menu_button.h"
 #import "chrome/browser/ui/cocoa/menu_controller.h"
 #import "chrome/browser/ui/cocoa/toolbar/back_forward_menu_controller.h"
 #import "chrome/browser/ui/cocoa/toolbar/reload_button.h"
+#import "chrome/browser/ui/cocoa/toolbar/toolbar_button.h"
 #import "chrome/browser/ui/cocoa/toolbar/toolbar_view.h"
 #import "chrome/browser/ui/cocoa/view_id_util.h"
 #import "chrome/browser/ui/cocoa/wrench_menu/wrench_menu_controller.h"
+#include "chrome/browser/ui/global_error_service.h"
+#include "chrome/browser/ui/global_error_service_factory.h"
+#include "chrome/browser/ui/omnibox/omnibox_view.h"
 #include "chrome/browser/ui/toolbar/toolbar_model.h"
 #include "chrome/browser/ui/toolbar/wrench_menu_model.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_service.h"
-#include "chrome/common/notification_type.h"
+#include "chrome/browser/upgrade_detector.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
-#include "gfx/rect.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "ui/base/l10n/l10n_util.h"
 #include "ui/base/l10n/l10n_util_mac.h"
-#include "ui/base/models/accelerator_cocoa.h"
-#include "ui/base/models/menu_model.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/image/image.h"
+#include "ui/gfx/rect.h"
+
+using content::OpenURLParams;
+using content::Referrer;
+using content::WebContents;
 
 namespace {
 
-// Names of images in the bundle for buttons.
-NSString* const kBackButtonImageName = @"back_Template.pdf";
-NSString* const kForwardButtonImageName = @"forward_Template.pdf";
-NSString* const kReloadButtonReloadImageName = @"reload_Template.pdf";
-NSString* const kReloadButtonStopImageName = @"stop_Template.pdf";
-NSString* const kHomeButtonImageName = @"home_Template.pdf";
-NSString* const kWrenchButtonImageName = @"tools_Template.pdf";
-
 // Height of the toolbar in pixels when the bookmark bar is closed.
 const CGFloat kBaseToolbarHeight = 35.0;
 
@@ -82,7 +82,8 @@ const CGFloat kWrenchMenuLeftPadding = 3.0;
 
 }  // namespace
 
-@interface ToolbarController(Private)
+@interface ToolbarController()
+@property(assign, nonatomic) Browser* browser;
 - (void)addAccessibilityDescriptions;
 - (void)initCommandStatus:(CommandUpdater*)commands;
 - (void)prefChanged:(std::string*)prefName;
@@ -100,51 +101,29 @@ const CGFloat kWrenchMenuLeftPadding = 3.0;
 
 namespace ToolbarControllerInternal {
 
-// A C++ delegate that handles the accelerators in the wrench menu.
-class WrenchAcceleratorDelegate : public ui::AcceleratorProvider {
- public:
-  virtual bool GetAcceleratorForCommandId(int command_id,
-      ui::Accelerator* accelerator_generic) {
-    // Downcast so that when the copy constructor is invoked below, the key
-    // string gets copied, too.
-    ui::AcceleratorCocoa* out_accelerator =
-        static_cast<ui::AcceleratorCocoa*>(accelerator_generic);
-    AcceleratorsCocoa* keymap = AcceleratorsCocoa::GetInstance();
-    const ui::AcceleratorCocoa* accelerator =
-        keymap->GetAcceleratorForCommand(command_id);
-    if (accelerator) {
-      *out_accelerator = *accelerator;
-      return true;
-    }
-    return false;
-  }
-};
-
 // A class registered for C++ notifications. This is used to detect changes in
 // preferences and upgrade available notifications. Bridges the notification
 // back to the ToolbarController.
-class NotificationBridge : public NotificationObserver {
+class NotificationBridge : public content::NotificationObserver {
  public:
   explicit NotificationBridge(ToolbarController* controller)
       : controller_(controller) {
-    registrar_.Add(this, NotificationType::UPGRADE_RECOMMENDED,
-                   NotificationService::AllSources());
-    registrar_.Add(this,
-                   NotificationType::BACKGROUND_PAGE_TRACKER_CHANGED,
-                   NotificationService::AllSources());
+    registrar_.Add(this, chrome::NOTIFICATION_UPGRADE_RECOMMENDED,
+                   content::NotificationService::AllSources());
+    registrar_.Add(this, chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED,
+                   content::Source<Profile>([controller browser]->profile()));
   }
 
-  // Overridden from NotificationObserver:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details) {
-    switch (type.value) {
-      case NotificationType::PREF_CHANGED:
-        [controller_ prefChanged:Details<std::string>(details).ptr()];
+  // Overridden from content::NotificationObserver:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) {
+    switch (type) {
+      case chrome::NOTIFICATION_PREF_CHANGED:
+        [controller_ prefChanged:content::Details<std::string>(details).ptr()];
         break;
-      case NotificationType::BACKGROUND_PAGE_TRACKER_CHANGED:
-        // fall-through
-      case NotificationType::UPGRADE_RECOMMENDED:
+      case chrome::NOTIFICATION_UPGRADE_RECOMMENDED:
+      case chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED:
         [controller_ badgeWrenchMenuIfNeeded];
         break;
       default:
@@ -155,13 +134,15 @@ class NotificationBridge : public NotificationObserver {
  private:
   ToolbarController* controller_;  // weak, owns us
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 };
 
 }  // namespace ToolbarControllerInternal
 
 @implementation ToolbarController
 
+@synthesize browser = browser_;
+
 - (id)initWithModel:(ToolbarModel*)model
            commands:(CommandUpdater*)commands
             profile:(Profile*)profile
@@ -170,7 +151,7 @@ class NotificationBridge : public NotificationObserver {
        nibFileNamed:(NSString*)nibName {
   DCHECK(model && commands && profile && [nibName length]);
   if ((self = [super initWithNibName:nibName
-                              bundle:base::mac::MainAppBundle()])) {
+                              bundle:base::mac::FrameworkBundle()])) {
     toolbarModel_ = model;
     commands_ = commands;
     profile_ = profile;
@@ -231,28 +212,56 @@ class NotificationBridge : public NotificationObserver {
 // Now we can hook up bridges that rely on UI objects such as the location
 // bar and button state.
 - (void)awakeFromNib {
-  // A bug in AppKit (<rdar://7298597>, <http://openradar.me/7298597>) causes
-  // images loaded directly from nibs in a framework to not get their "template"
-  // flags set properly. Thus, despite the images being set on the buttons in
-  // the xib, we must set them in code.
-  [backButton_ setImage:app::mac::GetCachedImageWithName(kBackButtonImageName)];
-  [forwardButton_ setImage:
-      app::mac::GetCachedImageWithName(kForwardButtonImageName)];
-  [reloadButton_ setImage:
-      app::mac::GetCachedImageWithName(kReloadButtonReloadImageName)];
-  [homeButton_ setImage:
-      app::mac::GetCachedImageWithName(kHomeButtonImageName)];
-  [wrenchButton_ setImage:
-      app::mac::GetCachedImageWithName(kWrenchButtonImageName)];
+  [[backButton_ cell] setImageID:IDR_BACK
+                  forButtonState:image_button_cell::kDefaultState];
+  [[backButton_ cell] setImageID:IDR_BACK_H
+                  forButtonState:image_button_cell::kHoverState];
+  [[backButton_ cell] setImageID:IDR_BACK_P
+                  forButtonState:image_button_cell::kPressedState];
+  [[backButton_ cell] setImageID:IDR_BACK_D
+                  forButtonState:image_button_cell::kDisabledState];
+
+  [[forwardButton_ cell] setImageID:IDR_FORWARD
+                     forButtonState:image_button_cell::kDefaultState];
+  [[forwardButton_ cell] setImageID:IDR_FORWARD_H
+                     forButtonState:image_button_cell::kHoverState];
+  [[forwardButton_ cell] setImageID:IDR_FORWARD_P
+                     forButtonState:image_button_cell::kPressedState];
+  [[forwardButton_ cell] setImageID:IDR_FORWARD_D
+                     forButtonState:image_button_cell::kDisabledState];
+
+  [[reloadButton_ cell] setImageID:IDR_RELOAD
+                    forButtonState:image_button_cell::kDefaultState];
+  [[reloadButton_ cell] setImageID:IDR_RELOAD_H
+                    forButtonState:image_button_cell::kHoverState];
+  [[reloadButton_ cell] setImageID:IDR_RELOAD_P
+                    forButtonState:image_button_cell::kPressedState];
+
+  [[homeButton_ cell] setImageID:IDR_HOME
+                  forButtonState:image_button_cell::kDefaultState];
+  [[homeButton_ cell] setImageID:IDR_HOME_H
+                  forButtonState:image_button_cell::kHoverState];
+  [[homeButton_ cell] setImageID:IDR_HOME_P
+                  forButtonState:image_button_cell::kPressedState];
+
+  [[wrenchButton_ cell] setImageID:IDR_TOOLS
+                    forButtonState:image_button_cell::kDefaultState];
+  [[wrenchButton_ cell] setImageID:IDR_TOOLS_H
+                    forButtonState:image_button_cell::kHoverState];
+  [[wrenchButton_ cell] setImageID:IDR_TOOLS_P
+                    forButtonState:image_button_cell::kPressedState];
+
   [self badgeWrenchMenuIfNeeded];
 
   [wrenchButton_ setOpenMenuOnClick:YES];
 
-  [backButton_ setShowsBorderOnlyWhileMouseInside:YES];
-  [forwardButton_ setShowsBorderOnlyWhileMouseInside:YES];
-  [reloadButton_ setShowsBorderOnlyWhileMouseInside:YES];
-  [homeButton_ setShowsBorderOnlyWhileMouseInside:YES];
-  [wrenchButton_ setShowsBorderOnlyWhileMouseInside:YES];
+  [backButton_ setOpenMenuOnRightClick:YES];
+  [forwardButton_ setOpenMenuOnRightClick:YES];
+
+  [backButton_ setHandleMiddleClick:YES];
+  [forwardButton_ setHandleMiddleClick:YES];
+  [reloadButton_ setHandleMiddleClick:YES];
+  [homeButton_ setHandleMiddleClick:YES];
 
   [self initCommandStatus:commands_];
   locationBarView_.reset(new LocationBarViewMac(locationBar_,
@@ -266,8 +275,6 @@ class NotificationBridge : public NotificationObserver {
   PrefService* prefs = profile_->GetPrefs();
   showHomeButton_.Init(prefs::kShowHomeButton, prefs,
                        notificationBridge_.get());
-  showPageOptionButtons_.Init(prefs::kShowPageOptionsButtons, prefs,
-                              notificationBridge_.get());
   [self showOptionalHomeButton];
   [self installWrenchMenu];
 
@@ -291,12 +298,12 @@ class NotificationBridge : public NotificationObserver {
   // helps us remember to release it.
   locationBarRetainer_.reset([locationBar_ retain]);
   trackingArea_.reset(
-      [[NSTrackingArea alloc] initWithRect:NSZeroRect // Ignored
+      [[CrTrackingArea alloc] initWithRect:NSZeroRect // Ignored
                                    options:NSTrackingMouseMoved |
                                            NSTrackingInVisibleRect |
                                            NSTrackingMouseEnteredAndExited |
                                            NSTrackingActiveAlways
-                                     owner:self
+                              proxiedOwner:self
                                   userInfo:nil]);
   NSView* toolbarView = [self view];
   [toolbarView addTrackingArea:trackingArea_.get()];
@@ -419,7 +426,7 @@ class NotificationBridge : public NotificationObserver {
   [homeButton_ setEnabled:commands->IsCommandEnabled(IDC_HOME) ? YES : NO];
 }
 
-- (void)updateToolbarWithContents:(TabContents*)tab
+- (void)updateToolbarWithContents:(WebContents*)tab
                shouldRestoreState:(BOOL)shouldRestore {
   locationBarView_->Update(tab, shouldRestore ? true : false);
 
@@ -525,14 +532,11 @@ class NotificationBridge : public NotificationObserver {
 // Install the menu wrench buttons. Calling this repeatedly is inexpensive so it
 // can be done every time the buttons are shown.
 - (void)installWrenchMenu {
-  if (wrenchMenuModel_.get())
+  if (wrenchMenuController_.get())
     return;
-  acceleratorDelegate_.reset(
-      new ToolbarControllerInternal::WrenchAcceleratorDelegate());
 
-  wrenchMenuModel_.reset(new WrenchMenuModel(
-      acceleratorDelegate_.get(), browser_));
-  [wrenchMenuController_ setModel:wrenchMenuModel_.get()];
+  wrenchMenuController_.reset(
+      [[WrenchMenuController alloc] initWithBrowser:browser_]);
   [wrenchMenuController_ setUseWithPopUpButtonCell:YES];
   [wrenchButton_ setAttachedMenu:[wrenchMenuController_ menu]];
 }
@@ -542,43 +546,16 @@ class NotificationBridge : public NotificationObserver {
 }
 
 - (void)badgeWrenchMenuIfNeeded {
-
-  int badgeResource = 0;
   if (UpgradeDetector::GetInstance()->notify_upgrade()) {
-    badgeResource = IDR_UPDATE_BADGE;
-  } else if (BackgroundPageTracker::GetInstance()->
-             GetUnacknowledgedBackgroundPageCount() > 0) {
-    badgeResource = IDR_BACKGROUND_BADGE;
-  } else {
-    // No badge - clear the badge if one is already set.
-    if ([[wrenchButton_ cell] overlayImage])
-      [[wrenchButton_ cell] setOverlayImage:nil];
+    [[wrenchButton_ cell]
+        setOverlayImageID:UpgradeDetector::GetInstance()->GetIconResourceID(
+            UpgradeDetector::UPGRADE_ICON_TYPE_BADGE)];
     return;
   }
 
-  NSImage* badge =
-      ResourceBundle::GetSharedInstance().GetNativeImageNamed(badgeResource);
-  NSImage* wrenchImage =
-      app::mac::GetCachedImageWithName(kWrenchButtonImageName);
-  NSSize wrenchImageSize = [wrenchImage size];
-  NSSize badgeSize = [badge size];
-
-  scoped_nsobject<NSImage> overlayImage(
-      [[NSImage alloc] initWithSize:wrenchImageSize]);
-
-  // Draw badge in the upper right corner of the button.
-  NSPoint overlayPosition =
-      NSMakePoint(wrenchImageSize.width - badgeSize.width,
-                  wrenchImageSize.height - badgeSize.height);
-
-  [overlayImage lockFocus];
-  [badge drawAtPoint:overlayPosition
-            fromRect:NSZeroRect
-           operation:NSCompositeSourceOver
-            fraction:1.0];
-  [overlayImage unlockFocus];
-
-  [[wrenchButton_ cell] setOverlayImage:overlayImage];
+  int error_badge_id = GlobalErrorServiceFactory::GetForProfile(
+      browser_->profile())->GetFirstBadgeResourceID();
+  [[wrenchButton_ cell] setOverlayImageID:error_badge_id];
 }
 
 - (void)prefChanged:(std::string*)prefName {
@@ -756,6 +733,10 @@ class NotificationBridge : public NotificationObserver {
   return browserActionsController_.get();
 }
 
+- (NSView*)wrenchButton {
+  return wrenchButton_;
+}
+
 // (URLDropTargetController protocol)
 - (void)dropURLs:(NSArray*)urls inView:(NSView*)view at:(NSPoint)point {
   // TODO(viettrungluu): This code is more or less copied from the code in
@@ -774,8 +755,13 @@ class NotificationBridge : public NotificationObserver {
   GURL url(URLFixerUpper::FixupURL(
       base::SysNSStringToUTF8([urls objectAtIndex:0]), std::string()));
 
-  browser_->GetSelectedTabContents()->OpenURL(url, GURL(), CURRENT_TAB,
-                                              PageTransition::TYPED);
+  if (url.SchemeIs(chrome::kJavaScriptScheme)) {
+    browser_->window()->GetLocationBar()->location_entry()->SetUserText(
+        OmniboxView::StripJavascriptSchemas(UTF8ToUTF16(url.spec())));
+  }
+  OpenURLParams params(
+      url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_TYPED, false);
+  browser_->GetSelectedWebContents()->OpenURL(params);
 }
 
 // (URLDropTargetController protocol)
@@ -787,12 +773,12 @@ class NotificationBridge : public NotificationObserver {
   // If the input is plain text, classify the input and make the URL.
   AutocompleteMatch match;
   browser_->profile()->GetAutocompleteClassifier()->Classify(
-      base::SysNSStringToWide(text),
-      std::wstring(), false, &match, NULL);
+      base::SysNSStringToUTF16(text), string16(), false, false, &match, NULL);
   GURL url(match.destination_url);
 
-  browser_->GetSelectedTabContents()->OpenURL(url, GURL(), CURRENT_TAB,
-                                              PageTransition::TYPED);
+  OpenURLParams params(
+      url, Referrer(), CURRENT_TAB, content::PAGE_TRANSITION_TYPED, false);
+  browser_->GetSelectedWebContents()->OpenURL(params);
 }
 
 // (URLDropTargetController protocol)
@@ -805,4 +791,9 @@ class NotificationBridge : public NotificationObserver {
   // Do nothing.
 }
 
+// (URLDropTargetController protocol)
+- (BOOL)isUnsupportedDropData:(id<NSDraggingInfo>)info {
+  return drag_util::IsUnsupportedDropData(profile_, info);
+}
+
 @end
