@@ -8,10 +8,10 @@
 
 namespace browser {
 
-void UnlockSlotIfNecessary(net::CryptoModule* module,
-                           browser::CryptoModulePasswordReason reason,
-                           const std::string& host,
-                           Callback0::Type* callback) {
+void UnlockSlotsIfNecessary(const net::CryptoModuleList& modules,
+                            browser::CryptoModulePasswordReason reason,
+                            const std::string& host,
+                            const base::Closure& callback) {
   // TODO(bulach): implement me.
   NOTREACHED();
 }
@@ -19,7 +19,7 @@ void UnlockSlotIfNecessary(net::CryptoModule* module,
 void UnlockCertSlotIfNecessary(net::X509Certificate* cert,
                                browser::CryptoModulePasswordReason reason,
                                const std::string& host,
-                               Callback0::Type* callback) {
+                               const base::Closure& callback) {
   // TODO(bulach): implement me.
   NOTREACHED();
 }
