@@ -1,14 +1,12 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/message_loop.h"
+#include "base/string16.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/tab_contents/tab_contents_view.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_navigator.h"
@@ -16,26 +14,40 @@
 #include "chrome/browser/ui/find_bar/find_bar.h"
 #include "chrome/browser/ui/find_bar/find_bar_controller.h"
 #include "chrome/browser/ui/find_bar/find_notification_details.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/browser/ui/find_bar/find_tab_helper.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/url_constants.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/browser/web_contents_view.h"
 #include "net/test/test_server.h"
+#include "ui/base/accelerators/accelerator.h"
 #include "ui/base/keycodes/keyboard_codes.h"
 
 #if defined(TOOLKIT_VIEWS)
 #include "chrome/browser/ui/views/find_bar_host.h"
-#include "views/focus/focus_manager.h"
+#include "ui/views/focus/focus_manager.h"
+#include "ui/views/widget/widget.h"
 #elif defined(TOOLKIT_GTK)
 #include "chrome/browser/ui/gtk/slide_animator_gtk.h"
 #elif defined(OS_MACOSX)
 #include "chrome/browser/ui/cocoa/find_bar/find_bar_bridge.h"
 #endif
 
+using content::NavigationController;
+using content::WebContents;
+
 const std::string kSimplePage = "404_is_enough_for_us.html";
 const std::string kFramePage = "files/find_in_page/frames.html";
 const std::string kFrameData = "files/find_in_page/framedata_general.html";
 const std::string kUserSelectPage = "files/find_in_page/user-select.html";
 const std::string kCrashPage = "files/find_in_page/crash_1341577.html";
 const std::string kTooFewMatchesPage = "files/find_in_page/bug_1155639.html";
+const std::string kLongTextareaPage = "files/find_in_page/large_textarea.html";
 const std::string kEndState = "files/find_in_page/end_state.html";
 const std::string kPrematureEnd = "files/find_in_page/premature_end.html";
 const std::string kMoveIfOver = "files/find_in_page/move_if_obscuring.html";
@@ -91,7 +103,7 @@ class FindInPageControllerTest : public InProcessBrowserTest {
 
   string16 GetFindBarMatchCountTextForBrowser(Browser* browser) {
     FindBarTesting* find_bar =
-      browser->GetFindBarController()->find_bar()->GetFindBarTesting();
+        browser->GetFindBarController()->find_bar()->GetFindBarTesting();
     return find_bar->GetMatchCountText();
   }
 
@@ -99,6 +111,12 @@ class FindInPageControllerTest : public InProcessBrowserTest {
     return GetFindBarMatchCountTextForBrowser(browser());
   }
 
+  int GetFindBarWidthForBrowser(Browser* browser) {
+    FindBarTesting* find_bar =
+        browser->GetFindBarController()->find_bar()->GetFindBarTesting();
+    return find_bar->GetWidth();
+  }
+
   void EnsureFindBoxOpenForBrowser(Browser* browser) {
     browser->ShowFindBar();
     gfx::Point position;
@@ -115,7 +133,7 @@ class FindInPageControllerTest : public InProcessBrowserTest {
 
 // Platform independent FindInPage that takes |const wchar_t*|
 // as an input.
-int FindInPageWchar(TabContents* tab,
+int FindInPageWchar(TabContentsWrapper* tab,
                     const wchar_t* search_str,
                     bool forward,
                     bool case_sensitive,
@@ -135,7 +153,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPageFrames) {
 
   // Try incremental search (mimicking user typing in).
   int ordinal = 0;
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(18, FindInPageWchar(tab, L"g",
                                 kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(1, ordinal);
@@ -206,12 +224,12 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPageFrames) {
 }
 
 // Specifying a prototype so that we can add the WARN_UNUSED_RESULT attribute.
-bool FocusedOnPage(TabContents* tab_contents, std::string* result)
+bool FocusedOnPage(WebContents* web_contents, std::string* result)
     WARN_UNUSED_RESULT;
 
-bool FocusedOnPage(TabContents* tab_contents, std::string* result) {
+bool FocusedOnPage(WebContents* web_contents, std::string* result) {
   return ui_test_utils::ExecuteJavaScriptAndExtractString(
-      tab_contents->render_view_host(),
+      web_contents->GetRenderViewHost(),
       L"",
       L"window.domAutomationController.send(getFocusedElement());",
       result);
@@ -227,12 +245,12 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPageEndState) {
   GURL url = test_server()->GetURL(kEndState);
   ui_test_utils::NavigateToURL(browser(), url);
 
-  TabContents* tab_contents = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab_contents = browser()->GetSelectedTabContentsWrapper();
   ASSERT_TRUE(NULL != tab_contents);
 
   // Verify that nothing has focus.
   std::string result;
-  ASSERT_TRUE(FocusedOnPage(tab_contents, &result));
+  ASSERT_TRUE(FocusedOnPage(tab_contents->web_contents(), &result));
   ASSERT_STREQ("{nothing focused}", result.c_str());
 
   // Search for a text that exists within a link on the page.
@@ -242,10 +260,11 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPageEndState) {
   EXPECT_EQ(1, ordinal);
 
   // End the find session, which should set focus to the link.
-  tab_contents->StopFinding(FindBarController::kKeepSelection);
+  tab_contents->
+      find_tab_helper()->StopFinding(FindBarController::kKeepSelection);
 
   // Verify that the link is focused.
-  ASSERT_TRUE(FocusedOnPage(tab_contents, &result));
+  ASSERT_TRUE(FocusedOnPage(tab_contents->web_contents(), &result));
   EXPECT_STREQ("link1", result.c_str());
 
   // Search for a text that exists within a link on the page.
@@ -255,16 +274,17 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPageEndState) {
 
   // Move the selection to link 1, after searching.
   ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractString(
-      tab_contents->render_view_host(),
+      tab_contents->web_contents()->GetRenderViewHost(),
       L"",
       L"window.domAutomationController.send(selectLink1());",
       &result));
 
   // End the find session.
-  tab_contents->StopFinding(FindBarController::kKeepSelection);
+  tab_contents->
+      find_tab_helper()->StopFinding(FindBarController::kKeepSelection);
 
   // Verify that link2 is not focused.
-  ASSERT_TRUE(FocusedOnPage(tab_contents, &result));
+  ASSERT_TRUE(FocusedOnPage(tab_contents->web_contents(), &result));
   EXPECT_STREQ("", result.c_str());
 }
 
@@ -279,7 +299,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPageOrdinal) {
 
   // Search for 'o', which should make the first item active and return
   // '1 in 3' (1st ordinal of a total of 3 matches).
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   int ordinal = 0;
   EXPECT_EQ(3, FindInPageWchar(tab, L"o",
                                kFwd, kIgnoreCase, &ordinal));
@@ -316,13 +336,11 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
   GURL url = test_server()->GetURL(kSelectChangesOrdinal);
   ui_test_utils::NavigateToURL(browser(), url);
 
-  TabContents* tab_contents = browser()->GetSelectedTabContents();
-  ASSERT_TRUE(NULL != tab_contents);
-
   // Search for a text that exists within a link on the page.
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
+  ASSERT_TRUE(NULL != tab);
   int ordinal = 0;
-  EXPECT_EQ(4, FindInPageWchar(tab_contents,
+  EXPECT_EQ(4, FindInPageWchar(tab,
                                L"google",
                                kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(1, ordinal);
@@ -330,7 +348,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
   // Move the selection to link 1, after searching.
   std::string result;
   ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractString(
-      tab_contents->render_view_host(),
+      tab->web_contents()->GetRenderViewHost(),
       L"",
       L"window.domAutomationController.send(selectLink1());",
       &result));
@@ -343,7 +361,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
   EXPECT_EQ(3, ordinal);
 
   // End the find session.
-  tab_contents->StopFinding(FindBarController::kKeepSelection);
+  tab->find_tab_helper()->StopFinding(FindBarController::kKeepSelection);
 }
 
 // This test loads a page with frames and makes sure the ordinal returned makes
@@ -357,7 +375,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPageMultiFramesOrdinal) {
 
   // Search for 'a', which should make the first item active and return
   // '1 in 7' (1st ordinal of a total of 7 matches).
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   int ordinal = 0;
   EXPECT_EQ(7,
             FindInPageWchar(tab, L"a", kFwd, kIgnoreCase, &ordinal));
@@ -408,7 +426,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindInPage_Issue5132) {
 
   // Search for 'goa' three times (6 matches on page).
   int ordinal = 0;
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(6, FindInPageWchar(tab, L"goa",
                                kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(1, ordinal);
@@ -437,7 +455,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindUnselectableText) {
   ui_test_utils::NavigateToURL(browser(), url);
 
   int ordinal = 0;
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(tab, L"text", kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(1, ordinal);
 }
@@ -460,7 +478,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindCrash_Issue1341577) {
   // TODO(jungshik): According to a native Malayalam speaker, it's ok not
   // to find U+0D4C. Still need to investigate further this issue.
   int ordinal = 0;
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   FindInPageWchar(tab, L"\u0D4C", kFwd, kIgnoreCase, &ordinal);
   FindInPageWchar(tab, L"\u0D4C", kFwd, kIgnoreCase, &ordinal);
 
@@ -484,7 +502,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindCrash_Issue14491) {
 
   // This used to crash the tab.
   int ordinal = 0;
-  EXPECT_EQ(0, FindInPageWchar(browser()->GetSelectedTabContents(),
+  EXPECT_EQ(0, FindInPageWchar(browser()->GetSelectedTabContentsWrapper(),
                                L"s", kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(0, ordinal);
 }
@@ -507,12 +525,31 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindRestarts_Issue1155639) {
   // This string appears 5 times at the bottom of a long page. If Find restarts
   // properly after a timeout, it will find 5 matches, not just 1.
   int ordinal = 0;
-  EXPECT_EQ(5, FindInPageWchar(browser()->GetSelectedTabContents(),
+  EXPECT_EQ(5, FindInPageWchar(browser()->GetSelectedTabContentsWrapper(),
                                L"008.xml",
                                kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(1, ordinal);
 }
 
+// Make sure we don't get into an infinite loop when text box contains very
+// large amount of text.
+IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindRestarts_Issue70505) {
+  ASSERT_TRUE(test_server()->Start());
+
+  // First we navigate to our page.
+  GURL url = test_server()->GetURL(kLongTextareaPage);
+  ui_test_utils::NavigateToURL(browser(), url);
+
+  // If this test hangs on the FindInPage call, then it might be a regression
+  // such as the one found in issue http://crbug.com/70505.
+  int ordinal = 0;
+  FindInPageWchar(browser()->GetSelectedTabContentsWrapper(),
+                  L"a", kFwd, kIgnoreCase, &ordinal);
+  EXPECT_EQ(1, ordinal);
+  // TODO(finnur): We cannot reliably get the matchcount for this Find call
+  // until we fix issue http://crbug.com/71176.
+}
+
 // This tests bug 11761: FindInPage terminates search prematurely.
 // This test is not expected to pass until bug 11761 is fixed.
 IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
@@ -523,7 +560,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
   GURL url = test_server()->GetURL(kPrematureEnd);
   ui_test_utils::NavigateToURL(browser(), url);
 
-  TabContents* tab_contents = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab_contents = browser()->GetSelectedTabContentsWrapper();
   ASSERT_TRUE(NULL != tab_contents);
 
   // Search for a text that exists within a link on the page.
@@ -551,8 +588,13 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindDisappearOnNavigate) {
   EXPECT_TRUE(fully_visible);
 
   // Reload the tab and make sure Find window doesn't go away.
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP,
+      content::Source<NavigationController>(
+          &browser()->GetSelectedTabContentsWrapper()->web_contents()->
+              GetController()));
   browser()->Reload(CURRENT_TAB);
-  ui_test_utils::WaitForNavigationInCurrentTab(browser());
+  observer.Wait();
 
   EXPECT_TRUE(GetFindBarWindowInfo(&position, &fully_visible));
   EXPECT_TRUE(fully_visible);
@@ -611,16 +653,8 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
   EXPECT_FALSE(fully_visible);
 }
 
-// TODO(rohitrao): The FindMovesWhenObscuring test does not pass on mac.
-// http://crbug.com/22036
-#if defined(OS_MACOSX)
-#define MAYBE_FindMovesWhenObscuring FAILS_FindMovesWhenObscuring
-#else
-#define MAYBE_FindMovesWhenObscuring FindMovesWhenObscuring
-#endif
-
 // Make sure Find box moves out of the way if it is obscuring the active match.
-IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_FindMovesWhenObscuring) {
+IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindMovesWhenObscuring) {
   ASSERT_TRUE(test_server()->Start());
 
   GURL url = test_server()->GetURL(kMoveIfOver);
@@ -641,7 +675,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_FindMovesWhenObscuring) {
 
   // Search for 'Chromium' which the Find box is obscuring.
   int ordinal = 0;
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   int index = 0;
   for (; index < kMoveIterations; ++index) {
     EXPECT_EQ(kMoveIterations, FindInPageWchar(tab, L"Chromium",
@@ -673,12 +707,20 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_FindMovesWhenObscuring) {
 #if defined(OS_MACOSX)
 // FindNextInNewTabUsesPrepopulate times-out, at least on Mac.
 // See http://crbug.com/43070
-#define FindNextInNewTabUsesPrepopulate DISABLED_FindNextInNewTabUsesPrepopulate
+#define MAYBE_FindNextInNewTabUsesPrepopulate \
+    DISABLED_FindNextInNewTabUsesPrepopulate
+#elif defined(OS_WIN) || defined(USE_AURA)
+// Occasionally times-out on Windows or aura, too.
+// See http://crbug.com/43070
+#define MAYBE_FindNextInNewTabUsesPrepopulate \
+    DISABLED_FindNextInNewTabUsesPrepopulate
+#else
+#define MAYBE_FindNextInNewTabUsesPrepopulate FindNextInNewTabUsesPrepopulate
 #endif
 
 // Make sure F3 in a new tab works if Find has previous string to search for.
 IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
-                       FindNextInNewTabUsesPrepopulate) {
+                       MAYBE_FindNextInNewTabUsesPrepopulate) {
   ASSERT_TRUE(test_server()->Start());
 
   // First we navigate to any page.
@@ -687,7 +729,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
 
   // Search for 'no_match'. No matches should be found.
   int ordinal = 0;
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(0, FindInPageWchar(tab, L"no_match",
                                kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(0, ordinal);
@@ -715,36 +757,27 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
 
 #if defined(TOOLKIT_VIEWS)
 // Make sure Find box grabs the Esc accelerator and restores it again.
-#if defined(OS_LINUX)
-// TODO(oshima): On Gtk/Linux, a focus out event is asynchronous and
-// hiding a find bar does not immediately update the target
-// accelerator. The last condition fails in most cases due to this
-// behavior. See http://crbug.com/26870.
-IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
-                       DISABLED_AcceleratorRestoring) {
-#else
-  IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, AcceleratorRestoring) {
-#endif
+IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, AcceleratorRestoring) {
   ASSERT_TRUE(test_server()->Start());
 
   // First we navigate to any page.
   GURL url = test_server()->GetURL(kSimplePage);
   ui_test_utils::NavigateToURL(browser(), url);
 
-  views::FocusManager* focus_manager =
-      views::FocusManager::GetFocusManagerForNativeWindow(
-          browser()->window()->GetNativeHandle());
+  gfx::NativeWindow window = browser()->window()->GetNativeHandle();
+  views::Widget* widget = views::Widget::GetWidgetForNativeWindow(window);
+  views::FocusManager* focus_manager = widget->GetFocusManager();
 
   // See where Escape is registered.
-  views::Accelerator escape(ui::VKEY_ESCAPE, false, false, false);
-  views::AcceleratorTarget* old_target =
+  ui::Accelerator escape(ui::VKEY_ESCAPE, false, false, false);
+  ui::AcceleratorTarget* old_target =
       focus_manager->GetCurrentTargetForAccelerator(escape);
   EXPECT_TRUE(old_target != NULL);
 
   browser()->ShowFindBar();
 
   // Our Find bar should be the new target.
-  views::AcceleratorTarget* new_target =
+  ui::AcceleratorTarget* new_target =
       focus_manager->GetCurrentTargetForAccelerator(escape);
 
   EXPECT_TRUE(new_target != NULL);
@@ -757,6 +790,13 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest,
   // The accelerator for Escape should be back to what it was before.
   EXPECT_EQ(old_target,
             focus_manager->GetCurrentTargetForAccelerator(escape));
+
+  // Show find bar again with animation on, and the target should be
+  // on find bar.
+  DropdownBarHost::disable_animations_during_testing_ = false;
+  browser()->ShowFindBar();
+  EXPECT_EQ(new_target,
+            focus_manager->GetCurrentTargetForAccelerator(escape));
 }
 #endif  // TOOLKIT_VIEWS
 
@@ -775,13 +815,14 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, StayActive) {
   // simulating keypresses here for searching for something and pressing
   // backspace, but that's been proven flaky in the past, so we go straight to
   // tab_contents.
-  TabContents* tab_contents = browser()->GetSelectedTabContents();
+  FindTabHelper* find_tab_helper =
+      browser()->GetSelectedTabContentsWrapper()->find_tab_helper();
   // Stop the (non-existing) find operation, and clear the selection (which
   // signals the UI is still active).
-  tab_contents->StopFinding(FindBarController::kClearSelection);
+  find_tab_helper->StopFinding(FindBarController::kClearSelection);
   // Make sure the Find UI flag hasn't been cleared, it must be so that the UI
   // still responds to browser window resizing.
-  ASSERT_TRUE(tab_contents->find_ui_active());
+  ASSERT_TRUE(find_tab_helper->find_ui_active());
 }
 
 // Make sure F3 works after you FindNext a couple of times and end the Find
@@ -795,7 +836,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, RestartSearchFromF3) {
 
   // Search for 'page'. Should have 1 match.
   int ordinal = 0;
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(tab, L"page", kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(1, ordinal);
 
@@ -826,31 +867,28 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, PreferPreviousSearch) {
 
   // Find "Default".
   int ordinal = 0;
-  TabContents* tab1 = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab1 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(tab1, L"Default", kFwd, kIgnoreCase, &ordinal));
 
   // Create a second tab.
   // For some reason we can't use AddSelectedTabWithURL here on ChromeOS. It
   // could be some delicate assumption about the tab starting off unselected or
   // something relating to user gesture.
-  browser::NavigateParams params(browser(), url, PageTransition::TYPED);
-  params.disposition = NEW_BACKGROUND_TAB;
-  params.tabstrip_add_types = TabStripModel::ADD_NONE;
-  browser::Navigate(&params);
-  browser()->SelectTabContentsAt(1, false);
-  TabContents* tab2 = browser()->GetSelectedTabContents();
+  browser()->AddBlankTab(true);
+  ui_test_utils::NavigateToURL(browser(), url);
+  TabContentsWrapper* tab2 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_NE(tab1, tab2);
 
   // Find "given".
   FindInPageWchar(tab2, L"given", kFwd, kIgnoreCase, &ordinal);
 
   // Switch back to first tab.
-  browser()->SelectTabContentsAt(0, false);
+  browser()->ActivateTabAt(0, false);
   browser()->GetFindBarController()->EndFindSession(
       FindBarController::kKeepSelection);
   // Simulate F3.
   ui_test_utils::FindInPage(tab1, string16(), kFwd, kIgnoreCase, &ordinal);
-  EXPECT_EQ(tab1->find_text(), WideToUTF16(L"Default"));
+  EXPECT_EQ(tab1->find_tab_helper()->find_text(), WideToUTF16(L"Default"));
 }
 
 // This tests that whenever you close and reopen the Find bar, it should show
@@ -869,7 +907,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, PrepopulateSameTab) {
 
   // Search for the word "page".
   int ordinal = 0;
-  TabContents* tab1 = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab1 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(tab1, L"page", kFwd, kIgnoreCase, &ordinal));
 
   // Open the Find box.
@@ -908,13 +946,13 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, PrepopulateInNewTab) {
 
   // Search for the word "page".
   int ordinal = 0;
-  TabContents* tab1 = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab1 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(tab1, L"page", kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(ASCIIToUTF16("1 of 1"), GetMatchCountText());
 
   // Now create a second tab and load the same page.
-  browser()->AddSelectedTabWithURL(url, PageTransition::TYPED);
-  TabContents* tab2 = browser()->GetSelectedTabContents();
+  browser()->AddSelectedTabWithURL(url, content::PAGE_TRANSITION_TYPED);
+  TabContentsWrapper* tab2 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_NE(tab1, tab2);
 
   // Open the Find box.
@@ -924,7 +962,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, PrepopulateInNewTab) {
   // in the first tab.
   EXPECT_EQ(ASCIIToUTF16("page"), GetFindBarText());
   // But it should not seem like a search has been issued.
-  EXPECT_EQ(ASCIIToUTF16(""), GetMatchCountText());
+  EXPECT_EQ(string16(), GetMatchCountText());
 }
 
 // This makes sure that we can search for A in tabA, then for B in tabB and
@@ -944,7 +982,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, PrepopulatePreserveLast) {
 
   // Search for the word "page".
   int ordinal = 0;
-  TabContents* tab1 = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab1 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(tab1, L"page", kFwd, kIgnoreCase, &ordinal));
 
   // Open the Find box.
@@ -957,12 +995,9 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, PrepopulatePreserveLast) {
       FindBarController::kKeepSelection);
 
   // Now create a second tab and load the same page.
-  browser::NavigateParams params(browser(), url, PageTransition::TYPED);
-  params.disposition = NEW_BACKGROUND_TAB;
-  params.tabstrip_add_types = TabStripModel::ADD_NONE;
-  browser::Navigate(&params);
-  browser()->SelectTabContentsAt(1, false);
-  TabContents* tab2 = browser()->GetSelectedTabContents();
+  browser()->AddBlankTab(true);
+  ui_test_utils::NavigateToURL(browser(), url);
+  TabContentsWrapper* tab2 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_NE(tab1, tab2);
 
   // Search for the word "text".
@@ -970,7 +1005,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, PrepopulatePreserveLast) {
 
   // Go back to the first tab and make sure we have NOT switched the prepopulate
   // text to "text".
-  browser()->SelectTabContentsAt(0, false);
+  browser()->ActivateTabAt(0, false);
 
   // Open the Find box.
   EnsureFindBoxOpen();
@@ -1018,7 +1053,7 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_NoIncognitoPrepopulate) {
 
   // Search for the word "page" in the normal browser tab.
   int ordinal = 0;
-  TabContents* tab1 = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab1 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(tab1, L"page", kFwd, kIgnoreCase, &ordinal));
 
   // Open the Find box.
@@ -1032,9 +1067,12 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_NoIncognitoPrepopulate) {
   // Open a new incognito window and navigate to the same page.
   Profile* incognito_profile = browser()->profile()->GetOffTheRecordProfile();
   Browser* incognito_browser = Browser::Create(incognito_profile);
-  incognito_browser->AddSelectedTabWithURL(url, PageTransition::START_PAGE);
-  ui_test_utils::WaitForNavigation(
-      &incognito_browser->GetSelectedTabContents()->controller());
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP,
+      content::NotificationService::AllSources());
+  incognito_browser->AddSelectedTabWithURL(
+      url, content::PAGE_TRANSITION_START_PAGE);
+  observer.Wait();
   incognito_browser->window()->Show();
 
   // Open the find box and make sure that it is prepopulated with "page".
@@ -1042,7 +1080,8 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_NoIncognitoPrepopulate) {
   EXPECT_EQ(ASCIIToUTF16("page"), GetFindBarTextForBrowser(incognito_browser));
 
   // Search for the word "text" in the incognito tab.
-  TabContents* incognito_tab = incognito_browser->GetSelectedTabContents();
+  TabContentsWrapper* incognito_tab =
+      incognito_browser->GetSelectedTabContentsWrapper();
   EXPECT_EQ(1, FindInPageWchar(incognito_tab, L"text",
                                kFwd, kIgnoreCase, &ordinal));
   EXPECT_EQ(ASCIIToUTF16("text"), GetFindBarTextForBrowser(incognito_browser));
@@ -1052,8 +1091,8 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_NoIncognitoPrepopulate) {
       FindBarController::kKeepSelection);
 
   // Now open a new tab in the original (non-incognito) browser.
-  browser()->AddSelectedTabWithURL(url, PageTransition::TYPED);
-  TabContents* tab2 = browser()->GetSelectedTabContents();
+  browser()->AddSelectedTabWithURL(url, content::PAGE_TRANSITION_TYPED);
+  TabContentsWrapper* tab2 = browser()->GetSelectedTabContentsWrapper();
   EXPECT_NE(tab1, tab2);
 
   // Open the Find box and make sure it is prepopulated with the search term
@@ -1070,12 +1109,42 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, ActivateLinkNavigatesPage) {
   GURL url = test_server()->GetURL(kLinkPage);
   ui_test_utils::NavigateToURL(browser(), url);
 
-  TabContents* tab = browser()->GetSelectedTabContents();
+  TabContentsWrapper* tab = browser()->GetSelectedTabContentsWrapper();
   int ordinal = 0;
   FindInPageWchar(tab, L"link", kFwd, kIgnoreCase, &ordinal);
   EXPECT_EQ(ordinal, 1);
 
   // End the find session, click on the link.
-  tab->StopFinding(FindBarController::kActivateSelection);
-  EXPECT_TRUE(ui_test_utils::WaitForNavigationInCurrentTab(browser()));
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP,
+      content::Source<NavigationController>(
+          &tab->web_contents()->GetController()));
+  tab->find_tab_helper()->StopFinding(FindBarController::kActivateSelection);
+  observer.Wait();
+}
+
+// Tests that FindBar fits within a narrow browser window.
+IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FitWindow) {
+  Browser::CreateParams params(Browser::TYPE_POPUP, browser()->profile());
+  params.initial_bounds = gfx::Rect(0, 0, 250, 500);
+  Browser* popup = Browser::CreateWithParams(params);
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP,
+      content::NotificationService::AllSources());
+  popup->AddSelectedTabWithURL(GURL(chrome::kAboutBlankURL),
+                               content::PAGE_TRANSITION_LINK);
+  // Wait for the page to finish loading.
+  observer.Wait();
+  popup->window()->Show();
+
+  // On GTK, bounds change is asynchronous.
+  MessageLoop::current()->RunAllPending();
+
+  EnsureFindBoxOpenForBrowser(popup);
+
+  // GTK adjusts FindBar size asynchronously.
+  MessageLoop::current()->RunAllPending();
+
+  ASSERT_LE(GetFindBarWidthForBrowser(popup),
+            popup->window()->GetBounds().width());
 }
