@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,13 +11,14 @@
 
 #include "base/basictypes.h"
 #include "base/string16.h"
+#include "chrome/browser/profiles/profile_keyed_service.h"
 
 class Profile;
 
-class FindBarState {
+class FindBarState : public ProfileKeyedService {
  public:
   FindBarState() {}
-  ~FindBarState() {}
+  virtual ~FindBarState() {}
 
   string16 last_prepopulate_text() const {
     return last_prepopulate_text_;
@@ -27,11 +28,6 @@ class FindBarState {
     last_prepopulate_text_ = text;
   }
 
-  // Retrieves the last prepopulate text for a given Profile.  If the profile is
-  // off the record and has an empty prepopulate text, falls back to the
-  // prepopulate text from the normal profile.
-  static string16 GetLastPrepopulateText(Profile* profile);
-
  private:
   string16 last_prepopulate_text_;
 
