@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,35 +7,38 @@
 #include <algorithm>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/i18n/rtl.h"
+#include "base/message_loop.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/extensions/extension_browser_event_router.h"
 #include "chrome/browser/extensions/extension_context_menu_model.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/image_loading_tracker.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/sessions/restore_tab_helper.h"
 #include "chrome/browser/ui/browser.h"
-#include "chrome/browser/ui/gtk/cairo_cached_surface.h"
-#include "chrome/browser/ui/gtk/extension_popup_gtk.h"
+#include "chrome/browser/ui/gtk/extensions/extension_popup_gtk.h"
 #include "chrome/browser/ui/gtk/gtk_chrome_button.h"
 #include "chrome/browser/ui/gtk/gtk_chrome_shrinkable_hbox.h"
-#include "chrome/browser/ui/gtk/gtk_theme_provider.h"
+#include "chrome/browser/ui/gtk/gtk_theme_service.h"
 #include "chrome/browser/ui/gtk/gtk_util.h"
 #include "chrome/browser/ui/gtk/hover_controller_gtk.h"
 #include "chrome/browser/ui/gtk/menu_gtk.h"
 #include "chrome/browser/ui/gtk/view_id_util.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_action.h"
 #include "chrome/common/extensions/extension_resource.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_service.h"
-#include "chrome/common/notification_source.h"
-#include "chrome/common/notification_type.h"
-#include "gfx/canvas_skia_paint.h"
-#include "gfx/gtk_util.h"
-#include "grit/app_resources.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_source.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
+#include "ui/base/gtk/gtk_compat.h"
+#include "ui/gfx/canvas_skia_paint.h"
+#include "ui/gfx/gtk_util.h"
 
 namespace {
 
@@ -56,12 +59,11 @@ const int kSeparatorPadding = 2;
 // Width of the invisible gripper for resizing the toolbar.
 const int kResizeGripperWidth = 4;
 
-const char* kDragTarget = "application/x-chrome-browseraction";
+const char kDragTarget[] = "application/x-chrome-browseraction";
 
 GtkTargetEntry GetDragTargetEntry() {
-  static std::string drag_target_string(kDragTarget);
   GtkTargetEntry drag_target;
-  drag_target.target = const_cast<char*>(drag_target_string.c_str());
+  drag_target.target = const_cast<char*>(kDragTarget);
   drag_target.flags = GTK_TARGET_SAME_APP;
   drag_target.info = 0;
   return drag_target;
@@ -78,14 +80,14 @@ gint WidthForIconCount(gint icon_count) {
 
 using ui::SimpleMenuModel;
 
-class BrowserActionButton : public NotificationObserver,
+class BrowserActionButton : public content::NotificationObserver,
                             public ImageLoadingTracker::Observer,
                             public ExtensionContextMenuModel::PopupDelegate,
                             public MenuGtk::Delegate {
  public:
   BrowserActionButton(BrowserActionsToolbarGtk* toolbar,
                       const Extension* extension,
-                      GtkThemeProvider* theme_provider)
+                      GtkThemeService* theme_provider)
       : toolbar_(toolbar),
         extension_(extension),
         image_(NULL),
@@ -126,8 +128,9 @@ class BrowserActionButton : public NotificationObserver,
     signals_.ConnectAfter(widget(), "expose-event",
                           G_CALLBACK(OnExposeEvent), this);
 
-    registrar_.Add(this, NotificationType::EXTENSION_BROWSER_ACTION_UPDATED,
-                   Source<ExtensionAction>(extension->browser_action()));
+    registrar_.Add(
+        this, chrome::NOTIFICATION_EXTENSION_BROWSER_ACTION_UPDATED,
+        content::Source<ExtensionAction>(extension->browser_action()));
   }
 
   ~BrowserActionButton() {
@@ -147,17 +150,18 @@ class BrowserActionButton : public NotificationObserver,
   const Extension* extension() { return extension_; }
 
   // NotificationObserver implementation.
-  void Observe(NotificationType type,
-               const NotificationSource& source,
-               const NotificationDetails& details) {
-    if (type == NotificationType::EXTENSION_BROWSER_ACTION_UPDATED)
+  void Observe(int type,
+               const content::NotificationSource& source,
+               const content::NotificationDetails& details) {
+    if (type == chrome::NOTIFICATION_EXTENSION_BROWSER_ACTION_UPDATED)
       UpdateState();
     else
       NOTREACHED();
   }
 
   // ImageLoadingTracker::Observer implementation.
-  void OnImageLoaded(SkBitmap* image, ExtensionResource resource, int index) {
+  void OnImageLoaded(SkBitmap* image, const ExtensionResource& resource,
+                     int index) {
     if (image) {
       default_skbitmap_ = *image;
       default_icon_ = gfx::GdkPixbufFromSkBitmap(image);
@@ -266,9 +270,9 @@ class BrowserActionButton : public NotificationObserver,
   }
 
   static gboolean OnButtonPress(GtkWidget* widget,
-                                GdkEvent* event,
+                                GdkEventButton* event,
                                 BrowserActionButton* action) {
-    if (event->button.button != 3)
+    if (event->button != 3)
       return FALSE;
 
     MenuGtk* menu = action->GetContextMenu();
@@ -276,7 +280,7 @@ class BrowserActionButton : public NotificationObserver,
       return FALSE;
 
     action->button_->SetPaintOverride(GTK_STATE_ACTIVE);
-    menu->Popup(widget, event);
+    menu->PopupForWidget(widget, event->button, event->time);
 
     return TRUE;
   }
@@ -304,8 +308,9 @@ class BrowserActionButton : public NotificationObserver,
       return FALSE;
 
     gfx::CanvasSkiaPaint canvas(event, false);
-    gfx::Rect bounding_rect(widget->allocation);
-    action->PaintBadge(&canvas, bounding_rect, tab_id);
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(widget, &allocation);
+    action->PaintBadge(&canvas, gfx::Rect(allocation), tab_id);
     return FALSE;
   }
 
@@ -328,7 +333,7 @@ class BrowserActionButton : public NotificationObserver,
   scoped_ptr<CustomDrawButton> button_;
 
   // The top level widget (parent of |button_|).
-  OwnedWidgetGtk alignment_;
+  ui::OwnedWidgetGtk alignment_;
 
   // The one image subwidget in |button_|. We keep this out so we don't alter
   // the widget hierarchy while changing the button image because changing the
@@ -349,7 +354,7 @@ class BrowserActionButton : public NotificationObserver,
   SkBitmap default_skbitmap_;
 
   ui::GtkSignalRegistrar signals_;
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // The context menu view and model for this extension action.
   scoped_ptr<MenuGtk> context_menu_;
@@ -363,7 +368,7 @@ class BrowserActionButton : public NotificationObserver,
 BrowserActionsToolbarGtk::BrowserActionsToolbarGtk(Browser* browser)
     : browser_(browser),
       profile_(browser->profile()),
-      theme_provider_(GtkThemeProvider::GetFrom(browser->profile())),
+      theme_service_(GtkThemeService::GetFrom(browser->profile())),
       model_(NULL),
       hbox_(gtk_hbox_new(FALSE, 0)),
       button_hbox_(gtk_chrome_shrinkable_hbox_new(TRUE, FALSE, kButtonPadding)),
@@ -372,14 +377,14 @@ BrowserActionsToolbarGtk::BrowserActionsToolbarGtk(Browser* browser)
       resize_animation_(this),
       desired_width_(0),
       start_width_(0),
-      method_factory_(this) {
+      weak_factory_(this) {
   ExtensionService* extension_service = profile_->GetExtensionService();
   // The |extension_service| can be NULL in Incognito.
   if (!extension_service)
     return;
 
   overflow_button_.reset(new CustomDrawButton(
-      theme_provider_,
+      theme_service_,
       IDR_BROWSER_ACTIONS_OVERFLOW,
       IDR_BROWSER_ACTIONS_OVERFLOW_P,
       IDR_BROWSER_ACTIONS_OVERFLOW_H,
@@ -388,7 +393,8 @@ BrowserActionsToolbarGtk::BrowserActionsToolbarGtk(Browser* browser)
 
   GtkWidget* gripper = gtk_button_new();
   gtk_widget_set_size_request(gripper, kResizeGripperWidth, -1);
-  GTK_WIDGET_UNSET_FLAGS(gripper, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus(gripper, FALSE);
+
   gtk_widget_add_events(gripper, GDK_POINTER_MOTION_MASK);
   signals_.Connect(gripper, "motion-notify-event",
                    G_CALLBACK(OnGripperMotionNotifyThunk), this);
@@ -445,9 +451,9 @@ BrowserActionsToolbarGtk::BrowserActionsToolbarGtk(Browser* browser)
   ViewIDUtil::SetID(button_hbox_.get(), VIEW_ID_BROWSER_ACTION_TOOLBAR);
 
   registrar_.Add(this,
-                 NotificationType::BROWSER_THEME_CHANGED,
-                 NotificationService::AllSources());
-  theme_provider_->InitThemesFor(this);
+                 chrome::NOTIFICATION_BROWSER_THEME_CHANGED,
+                 content::Source<ThemeService>(theme_service_));
+  theme_service_->InitThemesFor(this);
 }
 
 BrowserActionsToolbarGtk::~BrowserActionsToolbarGtk() {
@@ -458,11 +464,11 @@ BrowserActionsToolbarGtk::~BrowserActionsToolbarGtk() {
 }
 
 int BrowserActionsToolbarGtk::GetCurrentTabId() {
-  TabContents* selected_tab = browser_->GetSelectedTabContents();
+  TabContentsWrapper* selected_tab = browser_->GetSelectedTabContentsWrapper();
   if (!selected_tab)
     return -1;
 
-  return selected_tab->controller().session_id().id();
+  return selected_tab->restore_tab_helper()->session_id().id();
 }
 
 void BrowserActionsToolbarGtk::Update() {
@@ -472,14 +478,12 @@ void BrowserActionsToolbarGtk::Update() {
   }
 }
 
-void BrowserActionsToolbarGtk::Observe(NotificationType type,
-                                       const NotificationSource& source,
-                                       const NotificationDetails& details) {
-  DCHECK(NotificationType::BROWSER_THEME_CHANGED == type);
-  if (theme_provider_->UseGtkTheme())
-    gtk_widget_show(separator_);
-  else
-    gtk_widget_hide(separator_);
+void BrowserActionsToolbarGtk::Observe(
+    int type,
+    const content::NotificationSource& source,
+    const content::NotificationDetails& details) {
+  DCHECK(chrome::NOTIFICATION_BROWSER_THEME_CHANGED == type);
+  gtk_widget_set_visible(separator_, theme_service_->UsingNativeTheme());
 }
 
 void BrowserActionsToolbarGtk::SetupDrags() {
@@ -517,7 +521,7 @@ void BrowserActionsToolbarGtk::CreateButtonForExtension(
 
   RemoveButtonForExtension(extension);
   linked_ptr<BrowserActionButton> button(
-      new BrowserActionButton(this, extension, theme_provider_));
+      new BrowserActionButton(this, extension, theme_service_));
   gtk_chrome_shrinkable_hbox_pack_start(
       GTK_CHROME_SHRINKABLE_HBOX(button_hbox_.get()), button->widget(), 0);
   gtk_box_reorder_child(GTK_BOX(button_hbox_.get()), button->widget(), index);
@@ -562,17 +566,14 @@ void BrowserActionsToolbarGtk::RemoveButtonForExtension(
 }
 
 void BrowserActionsToolbarGtk::UpdateVisibility() {
-  if (button_count() == 0)
-    gtk_widget_hide(widget());
-  else
-    gtk_widget_show(widget());
+  gtk_widget_set_visible(widget(), button_count() != 0);
 }
 
 bool BrowserActionsToolbarGtk::ShouldDisplayBrowserAction(
     const Extension* extension) {
   // Only display incognito-enabled extensions while in incognito mode.
   return (!profile_->IsOffTheRecord() ||
-          profile_->GetExtensionService()->IsIncognitoEnabled(extension));
+          profile_->GetExtensionService()->IsIncognitoEnabled(extension->id()));
 }
 
 void BrowserActionsToolbarGtk::HidePopup() {
@@ -583,7 +584,11 @@ void BrowserActionsToolbarGtk::HidePopup() {
 
 void BrowserActionsToolbarGtk::AnimateToShowNIcons(int count) {
   desired_width_ = WidthForIconCount(count);
-  start_width_ = button_hbox_->allocation.width;
+
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(button_hbox_.get(), &allocation);
+  start_width_ = allocation.width;
+
   resize_animation_.Reset();
   resize_animation_.Show();
 }
@@ -599,7 +604,7 @@ void BrowserActionsToolbarGtk::BrowserActionAdded(const Extension* extension,
     return;
 
   // Animate the addition if we are showing all browser action buttons.
-  if (!GTK_WIDGET_VISIBLE(overflow_area_)) {
+  if (!gtk_widget_get_visible(overflow_area_)) {
     AnimateToShowNIcons(button_count());
     model_->SetVisibleIconCount(button_count());
   }
@@ -616,7 +621,7 @@ void BrowserActionsToolbarGtk::BrowserActionRemoved(
 
   RemoveButtonForExtension(extension);
 
-  if (!GTK_WIDGET_VISIBLE(overflow_area_)) {
+  if (!gtk_widget_get_visible(overflow_area_)) {
     AnimateToShowNIcons(button_count());
     model_->SetVisibleIconCount(button_count());
   }
@@ -734,7 +739,7 @@ void BrowserActionsToolbarGtk::UpdateChevronVisibility() {
   }
 
   if (button_count() > showing_icon_count) {
-    if (!GTK_WIDGET_VISIBLE(overflow_area_)) {
+    if (!gtk_widget_get_visible(overflow_area_)) {
       if (drag_button_) {
         // During drags, when the overflow chevron shows for the first time,
         // take that much space away from |button_hbox_| to make the drag look
@@ -762,8 +767,12 @@ gboolean BrowserActionsToolbarGtk::OnDragMotion(GtkWidget* widget,
   if (!drag_button_)
     return FALSE;
 
-  if (base::i18n::IsRTL())
-    x = widget->allocation.width - x;
+  if (base::i18n::IsRTL()) {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(widget, &allocation);
+    x = allocation.width - x;
+  }
+
   drop_index_ = x < kButtonWidth ? 0 : x / (kButtonWidth + kButtonPadding);
 
   // We will go ahead and reorder the child in order to provide visual feedback
@@ -802,7 +811,7 @@ gboolean BrowserActionsToolbarGtk::OnDragFailed(GtkWidget* widget,
 void BrowserActionsToolbarGtk::OnHierarchyChanged(
     GtkWidget* widget, GtkWidget* previous_toplevel) {
   GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
-  if (!GTK_WIDGET_TOPLEVEL(toplevel))
+  if (!gtk_widget_is_toplevel(toplevel))
     return;
 
   signals_.Connect(toplevel, "set-focus", G_CALLBACK(OnSetFocusThunk), this);
@@ -816,8 +825,10 @@ void BrowserActionsToolbarGtk::OnSetFocus(GtkWidget* widget,
   // call stack.
   if (!popup || popup->being_inspected())
     return;
-  MessageLoop::current()->PostTask(FROM_HERE,
-      method_factory_.NewRunnableMethod(&BrowserActionsToolbarGtk::HidePopup));
+  MessageLoop::current()->PostTask(
+      FROM_HERE,
+      base::Bind(&BrowserActionsToolbarGtk::HidePopup,
+                 weak_factory_.GetWeakPtr()));
 }
 
 gboolean BrowserActionsToolbarGtk::OnGripperMotionNotify(
@@ -827,10 +838,18 @@ gboolean BrowserActionsToolbarGtk::OnGripperMotionNotify(
 
   // Calculate how much the user dragged the gripper and subtract that off the
   // button container's width.
-  int distance_dragged = base::i18n::IsRTL() ?
-      -event->x :
-      event->x - widget->allocation.width;
-  gint new_width = button_hbox_->allocation.width - distance_dragged;
+  int distance_dragged;
+  if (base::i18n::IsRTL()) {
+    distance_dragged = -event->x;
+  } else {
+    GtkAllocation widget_allocation;
+    gtk_widget_get_allocation(widget, &widget_allocation);
+    distance_dragged = event->x - widget_allocation.width;
+  }
+
+  GtkAllocation button_hbox_allocation;
+  gtk_widget_get_allocation(button_hbox_.get(), &button_hbox_allocation);
+  gint new_width = button_hbox_allocation.width - distance_dragged;
   SetButtonHBoxWidth(new_width);
 
   return FALSE;
@@ -848,7 +867,7 @@ gboolean BrowserActionsToolbarGtk::OnGripperExpose(GtkWidget* gripper,
 // dragging.
 gboolean BrowserActionsToolbarGtk::OnGripperEnterNotify(
     GtkWidget* gripper, GdkEventCrossing* event) {
-  gdk_window_set_cursor(gripper->window,
+  gdk_window_set_cursor(gtk_widget_get_window(gripper),
                         gfx::GetCursor(GDK_SB_H_DOUBLE_ARROW));
   return FALSE;
 }
@@ -856,17 +875,19 @@ gboolean BrowserActionsToolbarGtk::OnGripperEnterNotify(
 gboolean BrowserActionsToolbarGtk::OnGripperLeaveNotify(
     GtkWidget* gripper, GdkEventCrossing* event) {
   if (!(event->state & GDK_BUTTON1_MASK))
-    gdk_window_set_cursor(gripper->window, NULL);
+    gdk_window_set_cursor(gtk_widget_get_window(gripper), NULL);
   return FALSE;
 }
 
 gboolean BrowserActionsToolbarGtk::OnGripperButtonRelease(
     GtkWidget* gripper, GdkEventButton* event) {
-  gfx::Rect gripper_rect(0, 0,
-                         gripper->allocation.width, gripper->allocation.height);
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(gripper, &allocation);
+  gfx::Rect gripper_rect(0, 0, allocation.width, allocation.height);
+
   gfx::Point release_point(event->x, event->y);
   if (!gripper_rect.Contains(release_point))
-    gdk_window_set_cursor(gripper->window, NULL);
+    gdk_window_set_cursor(gtk_widget_get_window(gripper), NULL);
 
   // After the user resizes the toolbar, we want to smartly resize it to be
   // the perfect size to fit the buttons.
@@ -950,7 +971,8 @@ gboolean BrowserActionsToolbarGtk::OnOverflowMenuButtonPress(
   if (!menu)
     return FALSE;
 
-  menu->PopupAsContext(event->time);
+  menu->PopupAsContext(gfx::Point(event->x_root, event->y_root),
+                       event->time);
   return TRUE;
 }
 
