@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,37 +11,51 @@
 #include <vector>
 
 #include "base/command_line.h"
-#include "base/singleton.h"
+#include "base/i18n/rtl.h"
+#include "base/memory/singleton.h"
 #include "base/string_piece.h"
 #include "base/string_tokenizer.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
+#include "chrome/browser/browser_process.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/profiles/profile_info_cache.h"
+#include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/gtk/accelerators_gtk.h"
+#include "chrome/browser/ui/gtk/avatar_menu_button_gtk.h"
 #include "chrome/browser/ui/gtk/browser_window_gtk.h"
 #include "chrome/browser/ui/gtk/custom_button.h"
 #if defined(USE_GCONF)
 #include "chrome/browser/ui/gtk/gconf_titlebar_listener.h"
 #endif
-#include "chrome/browser/ui/gtk/gtk_theme_provider.h"
+#include "chrome/browser/ui/gtk/gtk_theme_service.h"
 #include "chrome/browser/ui/gtk/gtk_util.h"
 #include "chrome/browser/ui/gtk/menu_gtk.h"
 #include "chrome/browser/ui/gtk/nine_box.h"
 #include "chrome/browser/ui/gtk/tabs/tab_strip_gtk.h"
+#include "chrome/browser/ui/gtk/unity_service.h"
 #include "chrome/browser/ui/toolbar/encoding_menu_controller.h"
 #include "chrome/browser/ui/toolbar/wrench_menu_model.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
-#include "gfx/gtk_util.h"
-#include "gfx/skbitmap_operations.h"
-#include "grit/app_resources.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
+#include "ui/base/gtk/gtk_hig_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/x/active_window_watcher_x.h"
+#include "ui/gfx/gtk_util.h"
+#include "ui/gfx/image/image.h"
+#include "ui/gfx/skbitmap_operations.h"
+
+using content::WebContents;
 
 namespace {
 
@@ -57,10 +71,10 @@ const int kNormalVerticalOffset = kTitlebarHeight + kTabOuterThickness;
 // A linux specific menu item for toggling window decorations.
 const int kShowWindowDecorationsCommand = 200;
 
-const int kOTRBottomSpacing = 1;
-// There are 2 px on each side of the OTR avatar (between the frame border and
+const int kAvatarBottomSpacing = 1;
+// There are 2 px on each side of the avatar (between the frame border and
 // it on the left, and between it and the tabstrip on the right).
-const int kOTRSideSpacing = 2;
+const int kAvatarSideSpacing = 2;
 
 // The thickness of the custom frame border; we need it here to enlarge the
 // close button whent the custom frame border isn't showing but the custom
@@ -100,7 +114,7 @@ gboolean OnMouseMoveEvent(GtkWidget* widget, GdkEventMotion* event,
 GdkPixbuf* GetOTRAvatar() {
   static GdkPixbuf* otr_avatar = NULL;
   if (!otr_avatar) {
-    ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+    ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
     otr_avatar = rb.GetRTLEnabledPixbufNamed(IDR_OTR_ICON);
   }
   return otr_avatar;
@@ -134,6 +148,15 @@ GdkColor PickLuminosityContrastingColor(const GdkColor* base,
     return *one;
 }
 
+// Returns true if there are multiple profiles created. This is used to
+// determine whether to display the avatar image.
+bool HasMultipleProfiles() {
+  ProfileInfoCache& cache =
+      g_browser_process->profile_manager()->GetProfileInfoCache();
+  return ProfileManager::IsMultipleProfilesEnabled() &&
+      cache.GetNumberOfProfiles() > 1;
+}
+
 }  // namespace
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -143,8 +166,7 @@ GdkColor PickLuminosityContrastingColor(const GdkColor* base,
 // user clicks on the favicon) and all of its submenus.
 class PopupPageMenuModel : public ui::SimpleMenuModel {
  public:
-  explicit PopupPageMenuModel(ui::SimpleMenuModel::Delegate* delegate,
-                              Browser* browser);
+  PopupPageMenuModel(ui::SimpleMenuModel::Delegate* delegate, Browser* browser);
   virtual ~PopupPageMenuModel() { }
 
  private:
@@ -206,15 +228,18 @@ BrowserTitlebar::BrowserTitlebar(BrowserWindowGtk* browser_window,
       titlebar_right_buttons_vbox_(NULL),
       titlebar_left_buttons_hbox_(NULL),
       titlebar_right_buttons_hbox_(NULL),
-      titlebar_left_spy_frame_(NULL),
-      titlebar_right_spy_frame_(NULL),
+      titlebar_left_avatar_frame_(NULL),
+      titlebar_right_avatar_frame_(NULL),
+      avatar_(NULL),
       top_padding_left_(NULL),
       top_padding_right_(NULL),
       app_mode_favicon_(NULL),
       app_mode_title_(NULL),
       using_custom_frame_(false),
       window_has_focus_(false),
-      theme_provider_(NULL) {
+      window_has_mouse_(false),
+      display_avatar_on_left_(false),
+      theme_service_(NULL) {
   Init();
 }
 
@@ -243,7 +268,7 @@ void BrowserTitlebar::Init() {
   // generated so they don't interfere with alignment when everything is
   // stuffed in the other box.
   //
-  // Each vbox has the contains the following hierarchy if it contains buttons:
+  // Each vbox has the following hierarchy if it contains buttons:
   //
   // +- VBox (titlebar_{l,r}_buttons_vbox_) ---------+
   // |+- Fixed (top_padding_{l,r}_) ----------------+|
@@ -279,31 +304,44 @@ void BrowserTitlebar::Init() {
   g_signal_connect(window_, "window-state-event",
                    G_CALLBACK(OnWindowStateChangedThunk), this);
 
-  // Allocate the two button boxes on the left and right parts of the bar, and
-  // spyguy frames in case of incognito mode.
+  if (IsTypePanel()) {
+    g_signal_connect(window_, "enter-notify-event",
+                     G_CALLBACK(OnEnterNotifyThunk), this);
+    g_signal_connect(window_, "leave-notify-event",
+                     G_CALLBACK(OnLeaveNotifyThunk), this);
+  }
+
+  // Allocate the two button boxes on the left and right parts of the bar. These
+  // are always allocated, but only displayed in incognito mode or when using
+  // multiple profiles.
   titlebar_left_buttons_vbox_ = gtk_vbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(container_hbox_), titlebar_left_buttons_vbox_,
                      FALSE, FALSE, 0);
-  if (browser_window_->browser()->profile()->IsOffTheRecord() &&
-      browser_window_->browser()->type() == Browser::TYPE_NORMAL) {
-    titlebar_left_spy_frame_ = gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
-    gtk_widget_set_no_show_all(titlebar_left_spy_frame_, TRUE);
-    gtk_alignment_set_padding(GTK_ALIGNMENT(titlebar_left_spy_frame_), 0,
-        kOTRBottomSpacing, kOTRSideSpacing, kOTRSideSpacing);
-    gtk_box_pack_start(GTK_BOX(container_hbox_), titlebar_left_spy_frame_,
-                       FALSE, FALSE, 0);
-
-    titlebar_right_spy_frame_ = gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
-    gtk_widget_set_no_show_all(titlebar_right_spy_frame_, TRUE);
-    gtk_alignment_set_padding(GTK_ALIGNMENT(titlebar_right_spy_frame_), 0,
-        kOTRBottomSpacing, kOTRSideSpacing, kOTRSideSpacing);
-    gtk_box_pack_end(GTK_BOX(container_hbox_), titlebar_right_spy_frame_,
+  titlebar_left_avatar_frame_ = gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
+  gtk_widget_set_no_show_all(titlebar_left_avatar_frame_, TRUE);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(titlebar_left_avatar_frame_), 0,
+      kAvatarBottomSpacing, kAvatarSideSpacing, kAvatarSideSpacing);
+  gtk_box_pack_start(GTK_BOX(container_hbox_), titlebar_left_avatar_frame_,
                      FALSE, FALSE, 0);
-  }
+
+  titlebar_right_avatar_frame_ = gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
+  gtk_widget_set_no_show_all(titlebar_right_avatar_frame_, TRUE);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(titlebar_right_avatar_frame_), 0,
+      kAvatarBottomSpacing, kAvatarSideSpacing, kAvatarSideSpacing);
+  gtk_box_pack_end(GTK_BOX(container_hbox_), titlebar_right_avatar_frame_,
+                   FALSE, FALSE, 0);
+
   titlebar_right_buttons_vbox_ = gtk_vbox_new(FALSE, 0);
   gtk_box_pack_end(GTK_BOX(container_hbox_), titlebar_right_buttons_vbox_,
                    FALSE, FALSE, 0);
 
+  // Create the Avatar button and listen for notifications. It must always be
+  // created because a new profile can be added at any time.
+  avatar_button_.reset(new AvatarMenuButtonGtk(browser_window_->browser()));
+
+  registrar_.Add(this, chrome::NOTIFICATION_PROFILE_CACHED_INFO_CHANGED,
+                 content::NotificationService::AllSources());
+
 #if defined(USE_GCONF)
   // Either read the gconf database and register for updates (on GNOME), or use
   // the default value (anywhere else).
@@ -312,9 +350,11 @@ void BrowserTitlebar::Init() {
   BuildButtons(kDefaultButtonString);
 #endif
 
+  UpdateAvatar();
+
   // We use an alignment to control the titlebar height.
   titlebar_alignment_ = gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
-  if (browser_window_->browser()->type() == Browser::TYPE_NORMAL) {
+  if (browser_window_->browser()->is_type_tabbed()) {
     gtk_box_pack_start(GTK_BOX(container_hbox_), titlebar_alignment_, TRUE,
                        TRUE, 0);
 
@@ -337,34 +377,43 @@ void BrowserTitlebar::Init() {
     GtkWidget* favicon_event_box = gtk_event_box_new();
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(favicon_event_box), FALSE);
     g_signal_connect(favicon_event_box, "button-press-event",
-                     G_CALLBACK(OnButtonPressedThunk), this);
+                     G_CALLBACK(OnFaviconMenuButtonPressedThunk), this);
     gtk_box_pack_start(GTK_BOX(app_mode_hbox), favicon_event_box, FALSE,
                        FALSE, 0);
     // We use the app logo as a placeholder image so the title doesn't jump
     // around.
-    ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+    ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
     app_mode_favicon_ = gtk_image_new_from_pixbuf(
         rb.GetRTLEnabledPixbufNamed(IDR_PRODUCT_LOGO_16));
     g_object_set_data(G_OBJECT(app_mode_favicon_), "left-align-popup",
                       reinterpret_cast<void*>(true));
     gtk_container_add(GTK_CONTAINER(favicon_event_box), app_mode_favicon_);
 
+    if (IsTypePanel()) {
+      panel_wrench_button_.reset(
+          BuildTitlebarButton(IDR_BALLOON_WRENCH, IDR_BALLOON_WRENCH_P,
+                              IDR_BALLOON_WRENCH_H, app_mode_hbox, FALSE,
+                              IDS_PANEL_WINDOW_SETTINGS_BUTTON_TOOLTIP));
+      g_signal_connect(panel_wrench_button_->widget(), "button-press-event",
+                       G_CALLBACK(OnPanelSettingsMenuButtonPressedThunk), this);
+      gtk_widget_set_no_show_all(panel_wrench_button_->widget(), TRUE);
+    }
+
     app_mode_title_ = gtk_label_new(NULL);
     gtk_label_set_ellipsize(GTK_LABEL(app_mode_title_), PANGO_ELLIPSIZE_END);
     gtk_misc_set_alignment(GTK_MISC(app_mode_title_), 0.0, 0.5);
     gtk_box_pack_start(GTK_BOX(app_mode_hbox), app_mode_title_, TRUE, TRUE,
                        0);
 
-    // Register with the theme provider to set the |app_mode_title_| label
-    // color.
-    theme_provider_ = GtkThemeProvider::GetFrom(
-        browser_window_->browser()->profile());
-    registrar_.Add(this, NotificationType::BROWSER_THEME_CHANGED,
-                   NotificationService::AllSources());
-    theme_provider_->InitThemesFor(this);
     UpdateTitleAndIcon();
   }
 
+  theme_service_ = GtkThemeService::GetFrom(
+      browser_window_->browser()->profile());
+  registrar_.Add(this, chrome::NOTIFICATION_BROWSER_THEME_CHANGED,
+                 content::Source<ThemeService>(theme_service_));
+  theme_service_->InitThemesFor(this);
+
   gtk_widget_show_all(container_);
 
   ui::ActiveWindowWatcherX::AddObserver(this);
@@ -401,26 +450,26 @@ void BrowserTitlebar::BuildButtons(const std::string& button_string) {
         left_side = false;
     } else {
       base::StringPiece token = tokenizer.token_piece();
-      if (token == "minimize") {
+      if (token == "minimize" && !IsTypePanel()) {
         (left_side ? left_count : right_count)++;
         GtkWidget* parent_box = GetButtonHBox(left_side);
         minimize_button_.reset(
             BuildTitlebarButton(IDR_MINIMIZE, IDR_MINIMIZE_P,
-                                IDR_MINIMIZE_H, parent_box,
+                                IDR_MINIMIZE_H, parent_box, true,
                                 IDS_XPFRAME_MINIMIZE_TOOLTIP));
 
         gtk_widget_size_request(minimize_button_->widget(),
                                 &minimize_button_req_);
-      } else if (token == "maximize") {
+      } else if (token == "maximize" && !IsTypePanel()) {
         (left_side ? left_count : right_count)++;
         GtkWidget* parent_box = GetButtonHBox(left_side);
         restore_button_.reset(
             BuildTitlebarButton(IDR_RESTORE, IDR_RESTORE_P,
-                                IDR_RESTORE_H, parent_box,
+                                IDR_RESTORE_H, parent_box, true,
                                 IDS_XPFRAME_RESTORE_TOOLTIP));
         maximize_button_.reset(
             BuildTitlebarButton(IDR_MAXIMIZE, IDR_MAXIMIZE_P,
-                                IDR_MAXIMIZE_H, parent_box,
+                                IDR_MAXIMIZE_H, parent_box, true,
                                 IDS_XPFRAME_MAXIMIZE_TOOLTIP));
 
         gtk_util::SetButtonClickableByMouseButtons(maximize_button_->widget(),
@@ -432,7 +481,7 @@ void BrowserTitlebar::BuildButtons(const std::string& button_string) {
         GtkWidget* parent_box = GetButtonHBox(left_side);
         close_button_.reset(
             BuildTitlebarButton(IDR_CLOSE, IDR_CLOSE_P,
-                                IDR_CLOSE_H, parent_box,
+                                IDR_CLOSE_H, parent_box, true,
                                 IDS_XPFRAME_CLOSE_TOOLTIP));
         close_button_->set_flipped(left_side);
 
@@ -445,27 +494,7 @@ void BrowserTitlebar::BuildButtons(const std::string& button_string) {
 
   // If we are in incognito mode, add the spy guy to either the end of the left
   // or the beginning of the right depending on which side has fewer buttons.
-  if (browser_window_->browser()->profile()->IsOffTheRecord() &&
-      browser_window_->browser()->type() == Browser::TYPE_NORMAL) {
-    GtkWidget* spy_guy = gtk_image_new_from_pixbuf(GetOTRAvatar());
-    gtk_misc_set_alignment(GTK_MISC(spy_guy), 0.0, 1.0);
-    gtk_widget_set_size_request(spy_guy, -1, 0);
-    gtk_widget_show(spy_guy);
-
-    // Remove previous state.
-    gtk_util::RemoveAllChildren(titlebar_left_spy_frame_);
-    gtk_util::RemoveAllChildren(titlebar_right_spy_frame_);
-
-    if (right_count > left_count) {
-      gtk_container_add(GTK_CONTAINER(titlebar_left_spy_frame_), spy_guy);
-      gtk_widget_show(titlebar_left_spy_frame_);
-      gtk_widget_hide(titlebar_right_spy_frame_);
-    } else {
-      gtk_container_add(GTK_CONTAINER(titlebar_right_spy_frame_), spy_guy);
-      gtk_widget_show(titlebar_right_spy_frame_);
-      gtk_widget_hide(titlebar_left_spy_frame_);
-    }
-  }
+  display_avatar_on_left_ = right_count > left_count;
 
   // Now show the correct widgets in the two hierarchies.
   if (using_custom_frame_) {
@@ -505,7 +534,8 @@ GtkWidget* BrowserTitlebar::GetButtonHBox(bool left_side) {
 }
 
 CustomDrawButton* BrowserTitlebar::BuildTitlebarButton(int image,
-    int image_pressed, int image_hot, GtkWidget* box, int tooltip) {
+    int image_pressed, int image_hot, GtkWidget* box, bool start,
+    int tooltip) {
   CustomDrawButton* button = new CustomDrawButton(image, image_pressed,
                                                   image_hot, 0);
   gtk_widget_add_events(GTK_WIDGET(button->widget()), GDK_POINTER_MOTION_MASK);
@@ -516,24 +546,49 @@ CustomDrawButton* BrowserTitlebar::BuildTitlebarButton(int image,
   std::string localized_tooltip = l10n_util::GetStringUTF8(tooltip);
   gtk_widget_set_tooltip_text(button->widget(),
                               localized_tooltip.c_str());
-  gtk_box_pack_start(GTK_BOX(box), button->widget(), FALSE, FALSE, 0);
+  if (start)
+    gtk_box_pack_start(GTK_BOX(box), button->widget(), FALSE, FALSE, 0);
+  else
+    gtk_box_pack_end(GTK_BOX(box), button->widget(), FALSE, FALSE, 0);
   return button;
 }
 
+void BrowserTitlebar::UpdateButtonBackground(CustomDrawButton* button) {
+  SkColor color = theme_service_->GetColor(
+      ThemeService::COLOR_BUTTON_BACKGROUND);
+  SkBitmap* background =
+      theme_service_->GetBitmapNamed(IDR_THEME_WINDOW_CONTROL_BACKGROUND);
+
+  // TODO(erg): For now, we just use a completely black mask and we can get
+  // away with this in the short term because our buttons are rectangles. We
+  // should get Glen to make properly hinted masks that match our min/max/close
+  // buttons (which have some odd alpha blending around the
+  // edges). http://crbug.com/103661
+  SkBitmap mask;
+  mask.setConfig(SkBitmap::kARGB_8888_Config,
+                 button->SurfaceWidth(), button->SurfaceHeight(), 0);
+  mask.allocPixels();
+  mask.eraseColor(SK_ColorBLACK);
+
+  button->SetBackground(color, background, &mask);
+}
+
 void BrowserTitlebar::UpdateCustomFrame(bool use_custom_frame) {
   using_custom_frame_ = use_custom_frame;
-  if (use_custom_frame) {
-    if (titlebar_left_buttons_vbox_)
-      gtk_widget_show_all(titlebar_left_buttons_vbox_);
-    if (titlebar_right_buttons_vbox_)
-      gtk_widget_show_all(titlebar_right_buttons_vbox_);
-  } else {
+  if (!use_custom_frame ||
+      (browser_window_->IsMaximized() && unity::IsRunning())) {
     if (titlebar_left_buttons_vbox_)
       gtk_widget_hide(titlebar_left_buttons_vbox_);
     if (titlebar_right_buttons_vbox_)
       gtk_widget_hide(titlebar_right_buttons_vbox_);
+  } else {
+    if (titlebar_left_buttons_vbox_)
+      gtk_widget_show_all(titlebar_left_buttons_vbox_);
+    if (titlebar_right_buttons_vbox_)
+      gtk_widget_show_all(titlebar_right_buttons_vbox_);
   }
   UpdateTitlebarAlignment();
+  UpdateMaximizeRestoreVisibility();
 }
 
 void BrowserTitlebar::UpdateTitleAndIcon() {
@@ -541,46 +596,62 @@ void BrowserTitlebar::UpdateTitleAndIcon() {
     return;
 
   // Get the page title and elide it to the available space.
-  string16 title = browser_window_->browser()->GetWindowTitleForCurrentTab();
-  gtk_label_set_text(GTK_LABEL(app_mode_title_), UTF16ToUTF8(title).c_str());
+  std::string title;
+  BrowserWindowGtk::TitleDecoration title_decoration =
+      browser_window_->GetWindowTitle(&title);
 
-  // Note: this isn't browser_window_->browser()->type() & Browser::TYPE_APP
-  // because we want to exclude Browser::TYPE_APP_POPUP.
-  if (browser_window_->browser()->type() == Browser::TYPE_APP ||
-      browser_window_->browser()->type() == Browser::TYPE_APP_PANEL) {
-    // Update the system app icon.  We don't need to update the icon in the top
-    // left of the custom frame, that will get updated when the throbber is
-    // updated.
-    SkBitmap icon = browser_window_->browser()->GetCurrentPageIcon();
-    if (icon.empty()) {
-      gtk_util::SetWindowIcon(window_);
-    } else {
-      GdkPixbuf* icon_pixbuf = gfx::GdkPixbufFromSkBitmap(&icon);
-      gtk_window_set_icon(window_, icon_pixbuf);
-      g_object_unref(icon_pixbuf);
+  if (title_decoration == BrowserWindowGtk::PANGO_MARKUP) {
+    gtk_label_set_markup(GTK_LABEL(app_mode_title_), title.c_str());
+  } else {
+    DCHECK_EQ(BrowserWindowGtk::PLAIN_TEXT, title_decoration);
+    gtk_label_set_text(GTK_LABEL(app_mode_title_), title.c_str());
+  }
+
+  if (browser_window_->browser()->is_app()) {
+    switch (browser_window_->browser()->type()) {
+      case Browser::TYPE_POPUP: {
+        // Update the system app icon.  We don't need to update the icon in the
+        // top left of the custom frame, that will get updated when the
+        // throbber is updated.
+        SkBitmap icon = browser_window_->browser()->GetCurrentPageIcon();
+        if (icon.empty()) {
+          gtk_util::SetWindowIcon(window_);
+        } else {
+          GdkPixbuf* icon_pixbuf = gfx::GdkPixbufFromSkBitmap(&icon);
+          gtk_window_set_icon(window_, icon_pixbuf);
+          g_object_unref(icon_pixbuf);
+        }
+        break;
+      }
+      case Browser::TYPE_TABBED: {
+        NOTREACHED() << "We should never have a tabbed app window.";
+        break;
+      }
+      case Browser::TYPE_PANEL: {
+        break;
+      }
     }
   }
 }
 
-void BrowserTitlebar::UpdateThrobber(TabContents* tab_contents) {
+void BrowserTitlebar::UpdateThrobber(WebContents* web_contents) {
   DCHECK(app_mode_favicon_);
 
-  if (tab_contents && tab_contents->is_loading()) {
+  if (web_contents && web_contents->IsLoading()) {
     GdkPixbuf* icon_pixbuf =
-        throbber_.GetNextFrame(tab_contents->waiting_for_response());
+        throbber_.GetNextFrame(web_contents->IsWaitingForResponse());
     gtk_image_set_from_pixbuf(GTK_IMAGE(app_mode_favicon_), icon_pixbuf);
   } else {
-    ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+    ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
 
-    // Note: this isn't browser_window_->browser()->type() & Browser::TYPE_APP
-    // because we want to exclude Browser::TYPE_APP_POPUP.
-    if (browser_window_->browser()->type() == Browser::TYPE_APP ||
-        browser_window_->browser()->type() == Browser::TYPE_APP_PANEL) {
+    // Note: we want to exclude the application popup window.
+    if (browser_window_->browser()->is_app() &&
+        browser_window_->browser()->is_type_popup()) {
       SkBitmap icon = browser_window_->browser()->GetCurrentPageIcon();
       if (icon.empty()) {
         // Fallback to the Chromium icon if the page has no icon.
         gtk_image_set_from_pixbuf(GTK_IMAGE(app_mode_favicon_),
-            rb.GetPixbufNamed(IDR_PRODUCT_LOGO_16));
+            rb.GetNativeImageNamed(IDR_PRODUCT_LOGO_16));
       } else {
         GdkPixbuf* icon_pixbuf = gfx::GdkPixbufFromSkBitmap(&icon);
         gtk_image_set_from_pixbuf(GTK_IMAGE(app_mode_favicon_), icon_pixbuf);
@@ -588,14 +659,14 @@ void BrowserTitlebar::UpdateThrobber(TabContents* tab_contents) {
       }
     } else {
       gtk_image_set_from_pixbuf(GTK_IMAGE(app_mode_favicon_),
-          rb.GetPixbufNamed(IDR_PRODUCT_LOGO_16));
+          rb.GetNativeImageNamed(IDR_PRODUCT_LOGO_16));
     }
     throbber_.Reset();
   }
 }
 
 void BrowserTitlebar::UpdateTitlebarAlignment() {
-  if (browser_window_->browser()->type() == Browser::TYPE_NORMAL) {
+  if (browser_window_->browser()->is_type_tabbed()) {
     int top_padding = 0;
     int side_padding = 0;
     int vertical_offset = kNormalVerticalOffset;
@@ -605,7 +676,8 @@ void BrowserTitlebar::UpdateTitlebarAlignment() {
         top_padding = kTitlebarHeight;
       } else if (using_custom_frame_ && browser_window_->IsMaximized()) {
         vertical_offset = 0;
-        side_padding = kMaximizedTabstripPadding;
+        if (!unity::IsRunning())
+          side_padding = kMaximizedTabstripPadding;
       }
     }
 
@@ -671,7 +743,7 @@ void BrowserTitlebar::UpdateTextColor() {
   if (!app_mode_title_)
     return;
 
-  if (theme_provider_ && theme_provider_->UseGtkTheme()) {
+  if (theme_service_ && theme_service_->UsingNativeTheme()) {
     // We don't really have any good options here.
     //
     // Colors from window manager themes aren't exposed in GTK; the window
@@ -687,17 +759,70 @@ void BrowserTitlebar::UpdateTextColor() {
     // color.
     GdkColor frame_color;
     if (window_has_focus_) {
-      frame_color = theme_provider_->GetGdkColor(
-          BrowserThemeProvider::COLOR_FRAME);
+      frame_color = theme_service_->GetGdkColor(
+          ThemeService::COLOR_FRAME);
     } else {
-      frame_color = theme_provider_->GetGdkColor(
-          BrowserThemeProvider::COLOR_FRAME_INACTIVE);
+      frame_color = theme_service_->GetGdkColor(
+          ThemeService::COLOR_FRAME_INACTIVE);
     }
     GdkColor text_color = PickLuminosityContrastingColor(
-        &frame_color, &gtk_util::kGdkWhite, &gtk_util::kGdkBlack);
+        &frame_color, &ui::kGdkWhite, &ui::kGdkBlack);
     gtk_util::SetLabelColor(app_mode_title_, &text_color);
   } else {
-    gtk_util::SetLabelColor(app_mode_title_, &gtk_util::kGdkWhite);
+    gtk_util::SetLabelColor(app_mode_title_, &ui::kGdkWhite);
+  }
+}
+
+void BrowserTitlebar::UpdateAvatar() {
+  // Remove previous state.
+  gtk_util::RemoveAllChildren(titlebar_left_avatar_frame_);
+  gtk_util::RemoveAllChildren(titlebar_right_avatar_frame_);
+
+  if (!ShouldDisplayAvatar())
+    return;
+
+  if (!avatar_) {
+    if (IsOffTheRecord()) {
+      avatar_ = gtk_image_new_from_pixbuf(GetOTRAvatar());
+      gtk_misc_set_alignment(GTK_MISC(avatar_), 0.0, 1.0);
+      gtk_widget_set_size_request(avatar_, -1, 0);
+    } else {
+      // Is using multi-profile avatar.
+      avatar_ = avatar_button_->widget();
+    }
+  }
+
+  gtk_widget_show_all(avatar_);
+
+  if (display_avatar_on_left_) {
+    gtk_container_add(GTK_CONTAINER(titlebar_left_avatar_frame_), avatar_);
+    gtk_widget_show(titlebar_left_avatar_frame_);
+    gtk_widget_hide(titlebar_right_avatar_frame_);
+  } else {
+    gtk_container_add(GTK_CONTAINER(titlebar_right_avatar_frame_), avatar_);
+    gtk_widget_show(titlebar_right_avatar_frame_);
+    gtk_widget_hide(titlebar_left_avatar_frame_);
+  }
+
+  if (IsOffTheRecord())
+    return;
+
+  ProfileInfoCache& cache =
+      g_browser_process->profile_manager()->GetProfileInfoCache();
+  Profile* profile = browser_window_->browser()->profile();
+  size_t index = cache.GetIndexOfProfileWithPath(profile->GetPath());
+  if (index != std::string::npos) {
+    bool is_gaia_picture =
+        cache.IsUsingGAIAPictureOfProfileAtIndex(index) &&
+        cache.GetGAIAPictureOfProfileAtIndex(index);
+    avatar_button_->SetIcon(
+        cache.GetAvatarIconOfProfileAtIndex(index), is_gaia_picture);
+
+    BubbleGtk::ArrowLocationGtk arrow_location =
+        display_avatar_on_left_ ^ base::i18n::IsRTL() ?
+            BubbleGtk::ARROW_LOCATION_TOP_LEFT :
+            BubbleGtk::ARROW_LOCATION_TOP_RIGHT;
+    avatar_button_->set_menu_arrow_location(arrow_location);
   }
 }
 
@@ -709,7 +834,7 @@ void BrowserTitlebar::ShowFaviconMenu(GdkEventButton* event) {
     favicon_menu_.reset(new MenuGtk(NULL, favicon_menu_model_.get()));
   }
 
-  favicon_menu_->Popup(app_mode_favicon_, reinterpret_cast<GdkEvent*>(event));
+  favicon_menu_->PopupForWidget(app_mode_favicon_, event->button, event->time);
 }
 
 void BrowserTitlebar::MaximizeButtonClicked() {
@@ -719,14 +844,18 @@ void BrowserTitlebar::MaximizeButtonClicked() {
   } else {
     GtkWidget* widget = GTK_WIDGET(window_);
     GdkScreen* screen = gtk_widget_get_screen(widget);
-    gint monitor = gdk_screen_get_monitor_at_window(screen, widget->window);
+    gint monitor = gdk_screen_get_monitor_at_window(
+        screen, gtk_widget_get_window(widget));
     GdkRectangle screen_rect;
     gdk_screen_get_monitor_geometry(screen, monitor, &screen_rect);
 
     gint x, y;
     gtk_window_get_position(window_, &x, &y);
-    gint width = widget->allocation.width;
-    gint height = widget->allocation.height;
+
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(widget, &allocation);
+    gint width = allocation.width;
+    gint height = allocation.height;
 
     if (event->button.button == 3) {
       x = 0;
@@ -763,7 +892,7 @@ gboolean BrowserTitlebar::OnWindowStateChanged(GtkWindow* window,
 
 gboolean BrowserTitlebar::OnScroll(GtkWidget* widget, GdkEventScroll* event) {
   TabStripModel* tabstrip_model = browser_window_->browser()->tabstrip_model();
-  int index = tabstrip_model->selected_index();
+  int index = tabstrip_model->active_index();
   if (event->direction == GDK_SCROLL_LEFT ||
       event->direction == GDK_SCROLL_UP) {
     if (index != 0)
@@ -774,6 +903,32 @@ gboolean BrowserTitlebar::OnScroll(GtkWidget* widget, GdkEventScroll* event) {
   return TRUE;
 }
 
+gboolean BrowserTitlebar::OnEnterNotify(GtkWidget* widget,
+                                        GdkEventCrossing* event) {
+  // Ignore if entered from a child widget.
+  if (event->detail == GDK_NOTIFY_INFERIOR)
+    return FALSE;
+
+  if (window_ && panel_wrench_button_.get())
+    gtk_widget_show(panel_wrench_button_->widget());
+
+  window_has_mouse_ = TRUE;
+  return FALSE;
+}
+
+gboolean BrowserTitlebar::OnLeaveNotify(GtkWidget* widget,
+                                        GdkEventCrossing* event) {
+  // Ignore if left towards a child widget.
+  if (event->detail == GDK_NOTIFY_INFERIOR)
+    return FALSE;
+
+  if (window_ && panel_wrench_button_.get() && !window_has_focus_)
+    gtk_widget_hide(panel_wrench_button_->widget());
+
+  window_has_mouse_ = FALSE;
+  return FALSE;
+}
+
 // static
 void BrowserTitlebar::OnButtonClicked(GtkWidget* button) {
   if (close_button_.get() && close_button_->widget() == button) {
@@ -787,8 +942,8 @@ void BrowserTitlebar::OnButtonClicked(GtkWidget* button) {
   }
 }
 
-gboolean BrowserTitlebar::OnButtonPressed(GtkWidget* widget,
-                                          GdkEventButton* event) {
+gboolean BrowserTitlebar::OnFaviconMenuButtonPressed(GtkWidget* widget,
+                                                     GdkEventButton* event) {
   if (event->button != 1)
     return FALSE;
 
@@ -796,13 +951,60 @@ gboolean BrowserTitlebar::OnButtonPressed(GtkWidget* widget,
   return TRUE;
 }
 
-void BrowserTitlebar::ShowContextMenu() {
+gboolean BrowserTitlebar::OnPanelSettingsMenuButtonPressed(
+    GtkWidget* widget, GdkEventButton* event) {
+  if (event->button != 1)
+    return FALSE;
+
+  browser_window_->ShowSettingsMenu(widget, event);
+  return TRUE;
+}
+
+void BrowserTitlebar::ShowContextMenu(GdkEventButton* event) {
   if (!context_menu_.get()) {
     context_menu_model_.reset(new ContextMenuModel(this));
     context_menu_.reset(new MenuGtk(NULL, context_menu_model_.get()));
   }
 
-  context_menu_->PopupAsContext(gtk_get_current_event_time());
+  context_menu_->PopupAsContext(gfx::Point(event->x_root, event->y_root),
+                                event->time);
+}
+
+void BrowserTitlebar::SendEnterNotifyToCloseButtonIfUnderMouse() {
+  gint x;
+  gint y;
+  GtkAllocation widget_allocation = close_button_->WidgetAllocation();
+  gtk_widget_get_pointer(GTK_WIDGET(close_button_->widget()), &x, &y);
+
+  gfx::Rect button_rect(0, 0, widget_allocation.width,
+                        widget_allocation.height);
+  if (!button_rect.Contains(x, y)) {
+    // Mouse is not over the close button.
+    return;
+  }
+
+  // Create and emit an enter-notify-event on close button.
+  GValue return_value;
+  return_value.g_type = G_TYPE_BOOLEAN;
+  g_value_set_boolean(&return_value, false);
+
+  GdkEvent* event = gdk_event_new(GDK_ENTER_NOTIFY);
+  event->crossing.window = GTK_BUTTON(close_button_->widget())->event_window;
+  event->crossing.send_event = FALSE;
+  event->crossing.subwindow = close_button_->widget()->window;
+  event->crossing.time = gtk_util::XTimeNow();
+  event->crossing.x = x;
+  event->crossing.y = y;
+  event->crossing.x_root = widget_allocation.x;
+  event->crossing.y_root = widget_allocation.y;
+  event->crossing.mode = GDK_CROSSING_NORMAL;
+  event->crossing.detail = GDK_NOTIFY_ANCESTOR;
+  event->crossing.focus = true;
+  event->crossing.state = 0;
+
+  g_signal_emit_by_name(GTK_OBJECT(close_button_->widget()),
+                        "enter-notify-event", event,
+                        &return_value);
 }
 
 bool BrowserTitlebar::IsCommandIdEnabled(int command_id) const {
@@ -821,11 +1023,11 @@ bool BrowserTitlebar::IsCommandIdChecked(int command_id) const {
 
   EncodingMenuController controller;
   if (controller.DoesCommandBelongToEncodingMenu(command_id)) {
-    TabContents* tab_contents =
-        browser_window_->browser()->GetSelectedTabContents();
-    if (tab_contents) {
+    WebContents* web_contents =
+        browser_window_->browser()->GetSelectedWebContents();
+    if (web_contents) {
       return controller.IsItemChecked(browser_window_->browser()->profile(),
-                                      tab_contents->encoding(),
+                                      web_contents->GetEncoding(),
                                       command_id);
     }
     return false;
@@ -856,12 +1058,27 @@ bool BrowserTitlebar::GetAcceleratorForCommandId(
   return accelerator_gtk;
 }
 
-void BrowserTitlebar::Observe(NotificationType type,
-                              const NotificationSource& source,
-                              const NotificationDetails& details) {
-  switch (type.value) {
-    case NotificationType::BROWSER_THEME_CHANGED:
+void BrowserTitlebar::Observe(int type,
+                              const content::NotificationSource& source,
+                              const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_BROWSER_THEME_CHANGED: {
       UpdateTextColor();
+
+      if (minimize_button_.get())
+        UpdateButtonBackground(minimize_button_.get());
+      if (maximize_button_.get())
+        UpdateButtonBackground(maximize_button_.get());
+      if (restore_button_.get())
+        UpdateButtonBackground(restore_button_.get());
+      if (close_button_.get())
+        UpdateButtonBackground(close_button_.get());
+      break;
+    }
+
+    case chrome::NOTIFICATION_PROFILE_CACHED_INFO_CHANGED:
+      if (!IsOffTheRecord())
+        UpdateAvatar();
       break;
 
     default:
@@ -875,10 +1092,30 @@ void BrowserTitlebar::ActiveWindowChanged(GdkWindow* active_window) {
   if (!window_)
     return;
 
-  window_has_focus_ = GTK_WIDGET(window_)->window == active_window;
+  window_has_focus_ =
+      gtk_widget_get_window(GTK_WIDGET(window_)) == active_window;
+  if (IsTypePanel()) {
+    if (window_has_focus_ || window_has_mouse_)
+      gtk_widget_show(panel_wrench_button_->widget());
+    else
+      gtk_widget_hide(panel_wrench_button_->widget());
+  }
   UpdateTextColor();
 }
 
+bool BrowserTitlebar::IsTypePanel() {
+  return browser_window_->browser()->is_type_panel();
+}
+
+bool BrowserTitlebar::ShouldDisplayAvatar() {
+  return (IsOffTheRecord() || HasMultipleProfiles()) &&
+      browser_window_->browser()->is_type_tabbed();
+}
+
+bool BrowserTitlebar::IsOffTheRecord() {
+  return browser_window_->browser()->profile()->IsOffTheRecord();
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // BrowserTitlebar::Throbber implementation
 // TODO(tc): Handle anti-clockwise spinning when waiting for a connection.
@@ -891,7 +1128,7 @@ static std::vector<GdkPixbuf*>* g_throbber_waiting_frames = NULL;
 // square GdkPixbufs that get stored in |frames|.
 static void MakeThrobberFrames(int resource_id,
                                std::vector<GdkPixbuf*>* frames) {
-  ResourceBundle &rb = ResourceBundle::GetSharedInstance();
+  ui::ResourceBundle &rb = ui::ResourceBundle::GetSharedInstance();
   SkBitmap* frame_strip = rb.GetBitmapNamed(resource_id);
 
   // Each frame of the animation is a square, so we use the height as the
