@@ -10,27 +10,27 @@
 
 #include "base/base_paths.h"
 #include "base/command_line.h"
+#include "base/debug/trace_event.h"
 #include "base/i18n/rtl.h"
 #include "base/logging.h"
+#include "base/memory/singleton.h"
 #include "base/path_service.h"
-#include "base/singleton.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/background_page_tracker.h"
-#include "chrome/browser/metrics/user_metrics.h"
 #include "chrome/browser/net/url_fixer_upper.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
+#include "chrome/browser/themes/theme_service.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/global_error_service.h"
+#include "chrome/browser/ui/global_error_service_factory.h"
 #include "chrome/browser/ui/gtk/accelerators_gtk.h"
 #include "chrome/browser/ui/gtk/back_forward_button_gtk.h"
+#include "chrome/browser/ui/gtk/bookmarks/bookmark_sub_menu_model_gtk.h"
 #include "chrome/browser/ui/gtk/browser_actions_toolbar_gtk.h"
 #include "chrome/browser/ui/gtk/browser_window_gtk.h"
-#include "chrome/browser/ui/gtk/cairo_cached_surface.h"
 #include "chrome/browser/ui/gtk/custom_button.h"
 #include "chrome/browser/ui/gtk/gtk_chrome_button.h"
-#include "chrome/browser/ui/gtk/gtk_theme_provider.h"
+#include "chrome/browser/ui/gtk/gtk_theme_service.h"
 #include "chrome/browser/ui/gtk/gtk_util.h"
 #include "chrome/browser/ui/gtk/location_bar_view_gtk.h"
 #include "chrome/browser/ui/gtk/reload_button_gtk.h"
@@ -39,20 +39,29 @@
 #include "chrome/browser/ui/gtk/view_id_util.h"
 #include "chrome/browser/ui/toolbar/encoding_menu_controller.h"
 #include "chrome/browser/upgrade_detector.h"
-#include "chrome/common/notification_details.h"
-#include "chrome/common/notification_service.h"
-#include "chrome/common/notification_type.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
-#include "gfx/canvas_skia_paint.h"
-#include "gfx/gtk_util.h"
-#include "gfx/skbitmap_operations.h"
+#include "content/public/browser/notification_details.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/user_metrics.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "ui/base/accelerators/accelerator_gtk.h"
 #include "ui/base/dragdrop/gtk_dnd_util.h"
 #include "ui/base/l10n/l10n_util.h"
-#include "ui/base/models/accelerator_gtk.h"
+#include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/canvas_skia_paint.h"
+#include "ui/gfx/gtk_util.h"
+#include "ui/gfx/image/cairo_cached_surface.h"
+#include "ui/gfx/skbitmap_operations.h"
+
+using content::HostZoomMap;
+using content::UserMetricsAction;
+using content::WebContents;
 
 namespace {
 
@@ -87,24 +96,22 @@ BrowserToolbarGtk::BrowserToolbarGtk(Browser* browser, BrowserWindowGtk* window)
     : toolbar_(NULL),
       location_bar_(new LocationBarViewGtk(browser)),
       model_(browser->toolbar_model()),
-      wrench_menu_model_(this, browser),
+      is_wrench_menu_model_valid_(true),
       browser_(browser),
-      window_(window),
-      profile_(NULL) {
+      window_(window) {
+  wrench_menu_model_.reset(new WrenchMenuModel(this, browser_));
+
   browser_->command_updater()->AddCommandObserver(IDC_BACK, this);
   browser_->command_updater()->AddCommandObserver(IDC_FORWARD, this);
   browser_->command_updater()->AddCommandObserver(IDC_HOME, this);
   browser_->command_updater()->AddCommandObserver(IDC_BOOKMARK_PAGE, this);
 
   registrar_.Add(this,
-                 NotificationType::BROWSER_THEME_CHANGED,
-                 NotificationService::AllSources());
+                 chrome::NOTIFICATION_UPGRADE_RECOMMENDED,
+                 content::NotificationService::AllSources());
   registrar_.Add(this,
-                 NotificationType::UPGRADE_RECOMMENDED,
-                 NotificationService::AllSources());
-  registrar_.Add(this,
-                 NotificationType::BACKGROUND_PAGE_TRACKER_CHANGED,
-                 NotificationService::AllSources());
+                 chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED,
+                 content::Source<Profile>(browser_->profile()));
 }
 
 BrowserToolbarGtk::~BrowserToolbarGtk() {
@@ -118,12 +125,13 @@ BrowserToolbarGtk::~BrowserToolbarGtk() {
   wrench_menu_.reset();
 }
 
-void BrowserToolbarGtk::Init(Profile* profile,
-                             GtkWindow* top_level_window) {
-  // Make sure to tell the location bar the profile before calling its Init.
-  SetProfile(profile);
+void BrowserToolbarGtk::Init(GtkWindow* top_level_window) {
+  Profile* profile = browser_->profile();
+  theme_service_ = GtkThemeService::GetFrom(profile);
+  registrar_.Add(this,
+                 chrome::NOTIFICATION_BROWSER_THEME_CHANGED,
+                 content::Source<ThemeService>(theme_service_));
 
-  theme_provider_ = GtkThemeProvider::GetFrom(profile);
   offscreen_entry_.Own(gtk_entry_new());
 
   show_home_button_.Init(prefs::kShowHomeButton, profile->GetPrefs(), this);
@@ -134,7 +142,7 @@ void BrowserToolbarGtk::Init(Profile* profile,
   event_box_ = gtk_event_box_new();
   // Make the event box transparent so themes can use transparent toolbar
   // backgrounds.
-  if (!theme_provider_->UseGtkTheme())
+  if (!theme_service_->UsingNativeTheme())
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(event_box_), FALSE);
 
   toolbar_ = gtk_hbox_new(FALSE, 0);
@@ -147,29 +155,33 @@ void BrowserToolbarGtk::Init(Profile* profile,
 
   toolbar_left_ = gtk_hbox_new(FALSE, kToolbarWidgetSpacing);
 
+  GtkSizeGroup* size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
   back_.reset(new BackForwardButtonGtk(browser_, false));
   g_signal_connect(back_->widget(), "clicked",
                    G_CALLBACK(OnButtonClickThunk), this);
+  gtk_size_group_add_widget(size_group, back_->widget());
   gtk_box_pack_start(GTK_BOX(toolbar_left_), back_->widget(), FALSE,
                      FALSE, 0);
 
   forward_.reset(new BackForwardButtonGtk(browser_, true));
   g_signal_connect(forward_->widget(), "clicked",
                    G_CALLBACK(OnButtonClickThunk), this);
+  gtk_size_group_add_widget(size_group, forward_->widget());
   gtk_box_pack_start(GTK_BOX(toolbar_left_), forward_->widget(), FALSE,
                      FALSE, 0);
 
   reload_.reset(new ReloadButtonGtk(location_bar_.get(), browser_));
+  gtk_size_group_add_widget(size_group, reload_->widget());
   gtk_box_pack_start(GTK_BOX(toolbar_left_), reload_->widget(), FALSE, FALSE,
                      0);
 
-  home_.reset(new CustomDrawButton(GtkThemeProvider::GetFrom(profile_),
-      IDR_HOME, IDR_HOME_P, IDR_HOME_H, 0, GTK_STOCK_HOME,
-      GTK_ICON_SIZE_SMALL_TOOLBAR));
+  home_.reset(new CustomDrawButton(theme_service_, IDR_HOME, IDR_HOME_P,
+      IDR_HOME_H, 0, GTK_STOCK_HOME, GTK_ICON_SIZE_SMALL_TOOLBAR));
   gtk_widget_set_tooltip_text(home_->widget(),
       l10n_util::GetStringUTF8(IDS_TOOLTIP_HOME).c_str());
   g_signal_connect(home_->widget(), "clicked",
                    G_CALLBACK(OnButtonClickThunk), this);
+  gtk_size_group_add_widget(size_group, home_->widget());
   gtk_box_pack_start(GTK_BOX(toolbar_left_), home_->widget(), FALSE, FALSE,
                      kToolbarWidgetSpacing);
   gtk_util::SetButtonTriggersNavigation(home_->widget());
@@ -177,6 +189,8 @@ void BrowserToolbarGtk::Init(Profile* profile,
   gtk_box_pack_start(GTK_BOX(toolbar_), toolbar_left_, FALSE, FALSE,
                      kToolbarLeftAreaPadding);
 
+  g_object_unref(size_group);
+
   location_hbox_ = gtk_hbox_new(FALSE, 0);
   location_bar_->Init(ShouldOnlyShowLocation());
   gtk_box_pack_start(GTK_BOX(location_hbox_), location_bar_->widget(), TRUE,
@@ -194,11 +208,9 @@ void BrowserToolbarGtk::Init(Profile* profile,
   }
 
   wrench_menu_image_ = gtk_image_new_from_pixbuf(
-      theme_provider_->GetRTLEnabledPixbufNamed(IDR_TOOLS));
-  wrench_menu_button_.reset(new CustomDrawButton(
-      GtkThemeProvider::GetFrom(profile_),
-      IDR_TOOLS, IDR_TOOLS_P, IDR_TOOLS_H, 0,
-      wrench_menu_image_));
+      theme_service_->GetRTLEnabledPixbufNamed(IDR_TOOLS));
+  wrench_menu_button_.reset(new CustomDrawButton(theme_service_, IDR_TOOLS,
+      IDR_TOOLS_P, IDR_TOOLS_H, 0, wrench_menu_image_));
   GtkWidget* wrench_button = wrench_menu_button_->widget();
 
   gtk_widget_set_tooltip_text(
@@ -207,7 +219,7 @@ void BrowserToolbarGtk::Init(Profile* profile,
           l10n_util::GetStringUTF16(IDS_PRODUCT_NAME)).c_str());
   g_signal_connect(wrench_button, "button-press-event",
                    G_CALLBACK(OnMenuButtonPressEventThunk), this);
-  GTK_WIDGET_UNSET_FLAGS(wrench_button, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus(wrench_button, FALSE);
 
   // Put the wrench button in a box so that we can paint the update notification
   // over it.
@@ -217,9 +229,12 @@ void BrowserToolbarGtk::Init(Profile* profile,
   gtk_container_add(GTK_CONTAINER(wrench_box), wrench_button);
   gtk_box_pack_start(GTK_BOX(toolbar_), wrench_box, FALSE, FALSE, 4);
 
-  wrench_menu_.reset(new MenuGtk(this, &wrench_menu_model_));
-  registrar_.Add(this, NotificationType::ZOOM_LEVEL_CHANGED,
-                 Source<Profile>(browser_->profile()));
+  wrench_menu_.reset(new MenuGtk(this, wrench_menu_model_.get()));
+  // The bookmark menu model needs to be able to force the wrench menu to close.
+  wrench_menu_model_->bookmark_sub_menu_model()->SetMenuGtk(wrench_menu_.get());
+
+  registrar_.Add(this, content::NOTIFICATION_ZOOM_LEVEL_CHANGED,
+      content::Source<HostZoomMap>(profile->GetHostZoomMap()));
 
   if (ShouldOnlyShowLocation()) {
     gtk_widget_show(event_box_);
@@ -240,7 +255,7 @@ void BrowserToolbarGtk::Init(Profile* profile,
   location_bar_->UpdateContentSettingsIcons();
 
   SetViewIDs();
-  theme_provider_->InitThemesFor(this);
+  theme_service_->InitThemesFor(this);
 }
 
 void BrowserToolbarGtk::SetViewIDs() {
@@ -275,8 +290,12 @@ void BrowserToolbarGtk::UpdateForBookmarkBarVisibility(
 
 void BrowserToolbarGtk::ShowAppMenu() {
   wrench_menu_->Cancel();
+
+  if (!is_wrench_menu_model_valid_)
+    RebuildWrenchMenu();
+
   wrench_menu_button_->SetPaintOverride(GTK_STATE_ACTIVE);
-  UserMetrics::RecordAction(UserMetricsAction("ShowAppMenu"));
+  content::RecordAction(UserMetricsAction("ShowAppMenu"));
   wrench_menu_->PopupAsFromKeyEvent(wrench_menu_button_->widget());
 }
 
@@ -297,7 +316,7 @@ void BrowserToolbarGtk::EnabledStateChangedForCommand(int id, bool enabled) {
       break;
   }
   if (widget) {
-    if (!enabled && GTK_WIDGET_STATE(widget) == GTK_STATE_PRELIGHT) {
+    if (!enabled && gtk_widget_get_state(widget) == GTK_STATE_PRELIGHT) {
       // If we're disabling a widget, GTK will helpfully restore it to its
       // previous state when we re-enable it, even if that previous state
       // is the prelight.  This looks bad.  See the bug for a simple repro.
@@ -316,20 +335,17 @@ void BrowserToolbarGtk::StoppedShowing() {
   gtk_chrome_button_set_hover_state(
       GTK_CHROME_BUTTON(wrench_menu_button_->widget()), 0.0);
   wrench_menu_button_->UnsetPaintOverride();
-
-  // Stop showing the BG page badge when we close the wrench menu.
-  BackgroundPageTracker::GetInstance()->AcknowledgeBackgroundPages();
 }
 
 GtkIconSet* BrowserToolbarGtk::GetIconSetForId(int idr) {
-  return theme_provider_->GetIconSetForId(idr);
+  return theme_service_->GetIconSetForId(idr);
 }
 
 // Always show images because we desire that some icons always show
 // regardless of the system setting.
 bool BrowserToolbarGtk::AlwaysShowIconForCmd(int command_id) const {
   return command_id == IDC_UPGRADE_DIALOG ||
-         command_id == IDC_VIEW_BACKGROUND_PAGES;
+      BookmarkSubMenuModel::IsBookmarkItemCommandId(command_id);
 }
 
 // ui::AcceleratorProvider
@@ -344,16 +360,16 @@ bool BrowserToolbarGtk::GetAcceleratorForCommandId(
   return !!accelerator_gtk;
 }
 
-// NotificationObserver --------------------------------------------------------
+// content::NotificationObserver -----------------------------------------------
 
-void BrowserToolbarGtk::Observe(NotificationType type,
-                                const NotificationSource& source,
-                                const NotificationDetails& details) {
-  if (type == NotificationType::PREF_CHANGED) {
-    NotifyPrefChanged(Details<std::string>(details).ptr());
-  } else if (type == NotificationType::BROWSER_THEME_CHANGED) {
+void BrowserToolbarGtk::Observe(int type,
+                                const content::NotificationSource& source,
+                                const content::NotificationDetails& details) {
+  if (type == chrome::NOTIFICATION_PREF_CHANGED) {
+    NotifyPrefChanged(content::Details<std::string>(details).ptr());
+  } else if (type == chrome::NOTIFICATION_BROWSER_THEME_CHANGED) {
     // Update the spacing around the menu buttons
-    bool use_gtk = theme_provider_->UseGtkTheme();
+    bool use_gtk = theme_service_->UsingNativeTheme();
     int border = use_gtk ? 0 : 2;
     gtk_container_set_border_width(
         GTK_CONTAINER(wrench_menu_button_->widget()), border);
@@ -380,15 +396,17 @@ void BrowserToolbarGtk::Observe(NotificationType type,
       // we set the initial value in Init()).
       gtk_image_set_from_pixbuf(
           GTK_IMAGE(wrench_menu_image_),
-          theme_provider_->GetRTLEnabledPixbufNamed(IDR_TOOLS));
+          theme_service_->GetRTLEnabledPixbufNamed(IDR_TOOLS));
     }
 
     UpdateRoundedness();
-  } else if (type == NotificationType::UPGRADE_RECOMMENDED ||
-             type == NotificationType::BACKGROUND_PAGE_TRACKER_CHANGED) {
+  } else if (type == chrome::NOTIFICATION_UPGRADE_RECOMMENDED) {
     // Redraw the wrench menu to update the badge.
     gtk_widget_queue_draw(wrench_menu_button_->widget());
-  } else if (type == NotificationType::ZOOM_LEVEL_CHANGED) {
+  } else if (type == chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED) {
+    is_wrench_menu_model_valid_ = false;
+    gtk_widget_queue_draw(wrench_menu_button_->widget());
+  } else if (type == content::NOTIFICATION_ZOOM_LEVEL_CHANGED) {
     // If our zoom level changed, we need to tell the menu to update its state,
     // since the menu could still be open.
     wrench_menu_->UpdateMenu();
@@ -399,15 +417,7 @@ void BrowserToolbarGtk::Observe(NotificationType type,
 
 // BrowserToolbarGtk, public ---------------------------------------------------
 
-void BrowserToolbarGtk::SetProfile(Profile* profile) {
-  if (profile == profile_)
-    return;
-
-  profile_ = profile;
-  location_bar_->SetProfile(profile);
-}
-
-void BrowserToolbarGtk::UpdateTabContents(TabContents* contents,
+void BrowserToolbarGtk::UpdateWebContents(WebContents* contents,
                                           bool should_restore_state) {
   location_bar_->Update(should_restore_state ? contents : NULL);
 
@@ -437,7 +447,7 @@ bool BrowserToolbarGtk::UpdateRoundedness() {
   // We still round the corners if we are in chrome theme mode, but we do it by
   // drawing theme resources rather than changing the physical shape of the
   // widget.
-  bool should_be_rounded = theme_provider_->UseGtkTheme() &&
+  bool should_be_rounded = theme_service_->UsingNativeTheme() &&
       window_->ShouldDrawContentDropShadow();
 
   if (should_be_rounded == gtk_util::IsActingAsRoundedWindow(alignment_))
@@ -456,13 +466,15 @@ bool BrowserToolbarGtk::UpdateRoundedness() {
 
 gboolean BrowserToolbarGtk::OnAlignmentExpose(GtkWidget* widget,
                                               GdkEventExpose* e) {
+  TRACE_EVENT0("ui::gtk", "BrowserToolbarGtk::OnAlignmentExpose");
+
   // We may need to update the roundedness of the toolbar's top corners. In
   // this case, don't draw; we'll be called again soon enough.
   if (UpdateRoundedness())
     return TRUE;
 
   // We don't need to render the toolbar image in GTK mode.
-  if (theme_provider_->UseGtkTheme())
+  if (theme_service_->UsingNativeTheme())
     return FALSE;
 
   cairo_t* cr = gdk_cairo_create(GDK_DRAWABLE(widget->window));
@@ -472,8 +484,8 @@ gboolean BrowserToolbarGtk::OnAlignmentExpose(GtkWidget* widget,
   gfx::Point tabstrip_origin =
       window_->tabstrip()->GetTabStripOriginForWidget(widget);
   // Fill the entire region with the toolbar color.
-  GdkColor color = theme_provider_->GetGdkColor(
-      BrowserThemeProvider::COLOR_TOOLBAR);
+  GdkColor color = theme_service_->GetGdkColor(
+      ThemeService::COLOR_TOOLBAR);
   gdk_cairo_set_source_color(cr, &color);
   cairo_fill(cr);
 
@@ -483,25 +495,27 @@ gboolean BrowserToolbarGtk::OnAlignmentExpose(GtkWidget* widget,
   // between the edge of the toolbar and where we anchor the corner images.
   const int kShadowThickness = 2;
 
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
   gfx::Rect area(e->area);
-  gfx::Rect right(widget->allocation.x + widget->allocation.width -
-                      kCornerWidth,
-                  widget->allocation.y - kShadowThickness,
+  gfx::Rect right(allocation.x + allocation.width - kCornerWidth,
+                  allocation.y - kShadowThickness,
                   kCornerWidth,
-                  widget->allocation.height + kShadowThickness);
-  gfx::Rect left(widget->allocation.x - kShadowThickness,
-                 widget->allocation.y - kShadowThickness,
+                  allocation.height + kShadowThickness);
+  gfx::Rect left(allocation.x - kShadowThickness,
+                 allocation.y - kShadowThickness,
                  kCornerWidth,
-                 widget->allocation.height + kShadowThickness);
+                 allocation.height + kShadowThickness);
 
   if (window_->ShouldDrawContentDropShadow()) {
     // Leave room to draw rounded corners.
     area = area.Subtract(right).Subtract(left);
   }
 
-  CairoCachedSurface* background = theme_provider_->GetSurfaceNamed(
-      IDR_THEME_TOOLBAR, widget);
-  background->SetSource(cr, tabstrip_origin.x(), tabstrip_origin.y());
+  const gfx::Image* background =
+      theme_service_->GetImageNamed(IDR_THEME_TOOLBAR);
+  background->ToCairo()->SetSource(
+      cr, widget, tabstrip_origin.x(), tabstrip_origin.y());
   cairo_pattern_set_extend(cairo_get_source(cr), CAIRO_EXTEND_REPEAT);
   cairo_rectangle(cr, area.x(), area.y(), area.width(), area.height());
   cairo_fill(cr);
@@ -520,21 +534,21 @@ gboolean BrowserToolbarGtk::OnAlignmentExpose(GtkWidget* widget,
     cairo_surface_t* target = cairo_surface_create_similar(
         cairo_get_target(cr),
         CAIRO_CONTENT_COLOR_ALPHA,
-        widget->allocation.x + widget->allocation.width,
-        widget->allocation.y + widget->allocation.height);
+        allocation.x + allocation.width,
+        allocation.y + allocation.height);
     cairo_t* copy_cr = cairo_create(target);
 
+    ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
+
     cairo_set_operator(copy_cr, CAIRO_OPERATOR_SOURCE);
     if (draw_left_corner) {
-      CairoCachedSurface* left_corner = theme_provider_->GetSurfaceNamed(
-          IDR_CONTENT_TOP_LEFT_CORNER_MASK, widget);
-      left_corner->SetSource(copy_cr, left.x(), left.y());
+      rb.GetNativeImageNamed(IDR_CONTENT_TOP_LEFT_CORNER_MASK).ToCairo()->
+          SetSource(copy_cr, widget, left.x(), left.y());
       cairo_paint(copy_cr);
     }
     if (draw_right_corner) {
-      CairoCachedSurface* right_corner = theme_provider_->GetSurfaceNamed(
-          IDR_CONTENT_TOP_RIGHT_CORNER_MASK, widget);
-      right_corner->SetSource(copy_cr, right.x(), right.y());
+      rb.GetNativeImageNamed(IDR_CONTENT_TOP_RIGHT_CORNER_MASK).ToCairo()->
+          SetSource(copy_cr, widget, right.x(), right.y());
       // We fill a path rather than just painting because we don't want to
       // overwrite the left corner.
       cairo_rectangle(copy_cr, right.x(), right.y(),
@@ -544,7 +558,8 @@ gboolean BrowserToolbarGtk::OnAlignmentExpose(GtkWidget* widget,
 
     // Draw the background. CAIRO_OPERATOR_IN uses the existing pixel data as
     // an alpha mask.
-    background->SetSource(copy_cr, tabstrip_origin.x(), tabstrip_origin.y());
+    background->ToCairo()->SetSource(copy_cr, widget,
+                                     tabstrip_origin.x(), tabstrip_origin.y());
     cairo_set_operator(copy_cr, CAIRO_OPERATOR_IN);
     cairo_pattern_set_extend(cairo_get_source(copy_cr), CAIRO_EXTEND_REPEAT);
     cairo_paint(copy_cr);
@@ -563,10 +578,13 @@ gboolean BrowserToolbarGtk::OnAlignmentExpose(GtkWidget* widget,
 
 gboolean BrowserToolbarGtk::OnLocationHboxExpose(GtkWidget* location_hbox,
                                                  GdkEventExpose* e) {
-  if (theme_provider_->UseGtkTheme()) {
+  TRACE_EVENT0("ui::gtk", "BrowserToolbarGtk::OnLocationHboxExpose");
+  if (theme_service_->UsingNativeTheme()) {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(location_hbox, &allocation);
     gtk_util::DrawTextEntryBackground(offscreen_entry_.get(),
                                       location_hbox, &e->area,
-                                      &location_hbox->allocation);
+                                      &allocation);
   }
 
   return FALSE;
@@ -589,8 +607,11 @@ gboolean BrowserToolbarGtk::OnMenuButtonPressEvent(GtkWidget* button,
   if (event->button != 1)
     return FALSE;
 
+  if (!is_wrench_menu_model_valid_)
+    RebuildWrenchMenu();
+
   wrench_menu_button_->SetPaintOverride(GTK_STATE_ACTIVE);
-  wrench_menu_->Popup(button, reinterpret_cast<GdkEvent*>(event));
+  wrench_menu_->PopupForWidget(button, event->button, event->time);
 
   return TRUE;
 }
@@ -607,7 +628,8 @@ void BrowserToolbarGtk::OnDragDataReceived(GtkWidget* widget,
   if (!url.is_valid())
     return;
 
-  bool url_is_newtab = url.spec() == chrome::kChromeUINewTabURL;
+  bool url_is_newtab = url.SchemeIs(chrome::kChromeUIScheme) &&
+                       url.host() == chrome::kChromeUINewTabHost;
   home_page_is_new_tab_page_.SetValue(url_is_newtab);
   if (!url_is_newtab)
     home_page_.SetValue(url.spec());
@@ -615,11 +637,8 @@ void BrowserToolbarGtk::OnDragDataReceived(GtkWidget* widget,
 
 void BrowserToolbarGtk::NotifyPrefChanged(const std::string* pref) {
   if (!pref || *pref == prefs::kShowHomeButton) {
-    if (show_home_button_.GetValue() && !ShouldOnlyShowLocation()) {
-      gtk_widget_show(home_->widget());
-    } else {
-      gtk_widget_hide(home_->widget());
-    }
+    bool visible = show_home_button_.GetValue() && !ShouldOnlyShowLocation();
+    gtk_widget_set_visible(home_->widget(), visible);
   }
 
   if (!pref ||
@@ -631,30 +650,41 @@ void BrowserToolbarGtk::NotifyPrefChanged(const std::string* pref) {
 
 bool BrowserToolbarGtk::ShouldOnlyShowLocation() const {
   // If we're a popup window, only show the location bar (omnibox).
-  return browser_->type() != Browser::TYPE_NORMAL;
+  return !browser_->is_type_tabbed();
+}
+
+void BrowserToolbarGtk::RebuildWrenchMenu() {
+  wrench_menu_model_.reset(new WrenchMenuModel(this, browser_));
+  wrench_menu_.reset(new MenuGtk(this, wrench_menu_model_.get()));
+  is_wrench_menu_model_valid_ = true;
 }
 
 gboolean BrowserToolbarGtk::OnWrenchMenuButtonExpose(GtkWidget* sender,
                                                      GdkEventExpose* expose) {
-  const SkBitmap* badge = NULL;
+  TRACE_EVENT0("ui::gtk", "BrowserToolbarGtk::OnWrenchMenuButtonExpose");
+  int resource_id = 0;
   if (UpgradeDetector::GetInstance()->notify_upgrade()) {
-    badge = theme_provider_->GetBitmapNamed(IDR_UPDATE_BADGE);
-  } else if (BackgroundPageTracker::GetInstance()->
-             GetUnacknowledgedBackgroundPageCount()) {
-    badge = theme_provider_->GetBitmapNamed(IDR_BACKGROUND_BADGE);
+    resource_id = UpgradeDetector::GetInstance()->GetIconResourceID(
+            UpgradeDetector::UPGRADE_ICON_TYPE_BADGE);
   } else {
-    return FALSE;
+    resource_id = GlobalErrorServiceFactory::GetForProfile(
+        browser_->profile())->GetFirstBadgeResourceID();
   }
 
+  if (!resource_id)
+    return FALSE;
+
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(sender, &allocation);
+
   // Draw the chrome app menu icon onto the canvas.
+  const SkBitmap* badge = theme_service_->GetBitmapNamed(resource_id);
   gfx::CanvasSkiaPaint canvas(expose, false);
-  int x_offset = base::i18n::IsRTL() ? 0 :
-      sender->allocation.width - badge->width();
+  int x_offset = base::i18n::IsRTL() ? 0 : allocation.width - badge->width();
   int y_offset = 0;
-  canvas.DrawBitmapInt(
-      *badge,
-      sender->allocation.x + x_offset,
-      sender->allocation.y + y_offset);
+  canvas.DrawBitmapInt(*badge,
+                       allocation.x + x_offset,
+                       allocation.y + y_offset);
 
   return FALSE;
 }
