@@ -1,27 +1,27 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/ui/gtk/certificate_viewer.h"
-
 #include <gtk/gtk.h>
 
 #include <algorithm>
 #include <vector>
 
 #include "base/i18n/time_formatting.h"
-#include "base/nss_util.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string_number_conversions.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/ui/gtk/certificate_dialogs.h"
+#include "chrome/browser/certificate_viewer.h"
+#include "chrome/browser/ui/certificate_dialogs.h"
 #include "chrome/browser/ui/gtk/gtk_util.h"
 #include "chrome/common/net/x509_certificate_model.h"
-#include "gfx/gtk_util.h"
 #include "grit/generated_resources.h"
 #include "net/base/x509_certificate.h"
+#include "ui/base/gtk/gtk_hig_constants.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/gfx/gtk_util.h"
+#include "ui/gfx/native_widget_types.h"
 
 namespace {
 
@@ -44,9 +44,11 @@ void AddKeyValue(GtkTable* table, int row, const std::string& text,
       gtk_util::IndentWidget(
           gtk_util::LeftAlignMisc(gtk_label_new(text.c_str()))),
       0, 1, row, row + 1);
+  GtkWidget* label = gtk_label_new(value.c_str());
+  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
   gtk_table_attach_defaults(
       table,
-      gtk_util::LeftAlignMisc(gtk_label_new(value.c_str())),
+      gtk_util::LeftAlignMisc(label),
       1, 2, row, row + 1);
 }
 
@@ -121,10 +123,9 @@ class CertificateViewer {
 // CertificateViewer implementation.
 
 // Close button callback.
-void OnDialogResponse(GtkDialog* dialog, gint response_id,
-                      gpointer user_data) {
+void OnResponse(GtkWidget* dialog, int response_id) {
   // "Close" was clicked.
-  gtk_widget_destroy(GTK_WIDGET(dialog));
+  gtk_widget_destroy(dialog);
 }
 
 void OnDestroy(GtkDialog* dialog, CertificateViewer* cert_viewer) {
@@ -147,15 +148,16 @@ CertificateViewer::CertificateViewer(
       GTK_STOCK_CLOSE,
       GTK_RESPONSE_CLOSE,
       NULL);
-  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog_)->vbox),
-                      gtk_util::kContentAreaSpacing);
+
+  GtkWidget* content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog_));
+  gtk_box_set_spacing(GTK_BOX(content_area), ui::kContentAreaSpacing);
 
   x509_certificate_model::RegisterDynamicOids();
   InitGeneralPage();
   InitDetailsPage();
 
   notebook_ = gtk_notebook_new();
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog_)->vbox), notebook_);
+  gtk_container_add(GTK_CONTAINER(content_area), notebook_);
 
   gtk_notebook_append_page(
       GTK_NOTEBOOK(notebook_),
@@ -173,7 +175,7 @@ CertificateViewer::CertificateViewer(
               l10n_util::GetStringUTF8(
                   IDS_CERT_INFO_DETAILS_TAB_LABEL)).c_str()));
 
-  g_signal_connect(dialog_, "response", G_CALLBACK(OnDialogResponse), NULL);
+  g_signal_connect(dialog_, "response", G_CALLBACK(OnResponse), NULL);
   g_signal_connect(dialog_, "destroy", G_CALLBACK(OnDestroy), this);
 }
 
@@ -183,11 +185,11 @@ CertificateViewer::~CertificateViewer() {
 
 void CertificateViewer::InitGeneralPage() {
   net::X509Certificate::OSCertHandle cert = cert_chain_list_.front();
-  general_page_vbox_ = gtk_vbox_new(FALSE, gtk_util::kContentAreaSpacing);
+  general_page_vbox_ = gtk_vbox_new(FALSE, ui::kContentAreaSpacing);
   gtk_container_set_border_width(GTK_CONTAINER(general_page_vbox_),
-                                 gtk_util::kContentAreaBorder);
+                                 ui::kContentAreaBorder);
 
-  GtkWidget* uses_vbox = gtk_vbox_new(FALSE, gtk_util::kControlSpacing);
+  GtkWidget* uses_vbox = gtk_vbox_new(FALSE, ui::kControlSpacing);
   gtk_box_pack_start(GTK_BOX(general_page_vbox_), uses_vbox, FALSE, FALSE, 0);
   gtk_box_pack_start(
       GTK_BOX(uses_vbox),
@@ -209,8 +211,8 @@ void CertificateViewer::InitGeneralPage() {
 
   const int num_rows = 21;
   GtkTable* table = GTK_TABLE(gtk_table_new(num_rows, 2, FALSE));
-  gtk_table_set_col_spacing(table, 0, gtk_util::kLabelSpacing);
-  gtk_table_set_row_spacings(table, gtk_util::kControlSpacing);
+  gtk_table_set_col_spacing(table, 0, ui::kLabelSpacing);
+  gtk_table_set_row_spacings(table, ui::kControlSpacing);
 
   gtk_box_pack_start(GTK_BOX(general_page_vbox_), GTK_WIDGET(table),
                      FALSE, FALSE, 0);
@@ -486,6 +488,32 @@ void CertificateViewer::FillTreeStoreWithCertFields(
       FIELDS_VALUE,
       x509_certificate_model::ProcessRawBitsSignatureWrap(cert).c_str(),
       -1);
+
+  GtkTreeIter top_fingerprints_iter;
+  gtk_tree_store_append(store, &top_fingerprints_iter, &top);
+  gtk_tree_store_set(
+      store, &top_fingerprints_iter,
+      FIELDS_NAME,
+      l10n_util::GetStringUTF8(IDS_CERT_INFO_FINGERPRINTS_GROUP).c_str(),
+      FIELDS_VALUE, "",
+      -1);
+
+  GtkTreeIter fingerprints_iter;
+  gtk_tree_store_append(store, &fingerprints_iter, &top_fingerprints_iter);
+  gtk_tree_store_set(
+      store, &fingerprints_iter,
+      FIELDS_NAME,
+      l10n_util::GetStringUTF8(IDS_CERT_INFO_SHA256_FINGERPRINT_LABEL).c_str(),
+      FIELDS_VALUE, x509_certificate_model::HashCertSHA256(cert).c_str(),
+      -1);
+
+  gtk_tree_store_append(store, &fingerprints_iter, &top_fingerprints_iter);
+  gtk_tree_store_set(
+      store, &fingerprints_iter,
+      FIELDS_NAME,
+      l10n_util::GetStringUTF8(IDS_CERT_INFO_SHA1_FINGERPRINT_LABEL).c_str(),
+      FIELDS_VALUE, x509_certificate_model::HashCertSHA1(cert).c_str(),
+      -1);
 }
 
 // static
@@ -498,11 +526,11 @@ GtkTreeStore* CertificateViewer::CreateFieldsTreeStore(
 }
 
 void CertificateViewer::InitDetailsPage() {
-  details_page_vbox_ = gtk_vbox_new(FALSE, gtk_util::kContentAreaSpacing);
+  details_page_vbox_ = gtk_vbox_new(FALSE, ui::kContentAreaSpacing);
   gtk_container_set_border_width(GTK_CONTAINER(details_page_vbox_),
-                                 gtk_util::kContentAreaBorder);
+                                 ui::kContentAreaBorder);
 
-  GtkWidget* hierarchy_vbox = gtk_vbox_new(FALSE, gtk_util::kControlSpacing);
+  GtkWidget* hierarchy_vbox = gtk_vbox_new(FALSE, ui::kControlSpacing);
   gtk_box_pack_start(GTK_BOX(details_page_vbox_), hierarchy_vbox,
                      FALSE, FALSE, 0);
 
@@ -541,7 +569,7 @@ void CertificateViewer::InitDetailsPage() {
   gtk_box_pack_start(GTK_BOX(hierarchy_vbox),
                      hierarchy_scroll_window, FALSE, FALSE, 0);
 
-  GtkWidget* fields_vbox = gtk_vbox_new(FALSE, gtk_util::kControlSpacing);
+  GtkWidget* fields_vbox = gtk_vbox_new(FALSE, ui::kControlSpacing);
   gtk_box_pack_start(GTK_BOX(details_page_vbox_), fields_vbox,
                      TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(fields_vbox),
@@ -571,7 +599,7 @@ void CertificateViewer::InitDetailsPage() {
   gtk_box_pack_start(GTK_BOX(fields_vbox),
                      fields_scroll_window, TRUE, TRUE, 0);
 
-  GtkWidget* value_vbox = gtk_vbox_new(FALSE, gtk_util::kControlSpacing);
+  GtkWidget* value_vbox = gtk_vbox_new(FALSE, ui::kControlSpacing);
   gtk_box_pack_start(GTK_BOX(details_page_vbox_), value_vbox,
                      TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(value_vbox),
@@ -671,7 +699,7 @@ void CertificateViewer::OnExportClicked(GtkButton *button,
     return;
   }
 
-  ShowCertExportDialog(GTK_WINDOW(viewer->dialog_),
+  ShowCertExportDialog(NULL, GTK_WINDOW(viewer->dialog_),
                        viewer->cert_chain_list_[cert_index]);
 }
 
@@ -681,14 +709,14 @@ void CertificateViewer::Show() {
 
 } // namespace
 
-void ShowCertificateViewer(gfx::NativeWindow parent,
-                           net::X509Certificate::OSCertHandle cert) {
+void ShowNativeCertificateViewer(gfx::NativeWindow parent,
+                                 net::X509Certificate::OSCertHandle cert) {
   net::X509Certificate::OSCertHandles cert_chain;
   x509_certificate_model::GetCertChainFromCert(cert, &cert_chain);
   (new CertificateViewer(parent, cert_chain))->Show();
 }
 
-void ShowCertificateViewer(gfx::NativeWindow parent,
-                           net::X509Certificate* cert) {
-  ShowCertificateViewer(parent, cert->os_cert_handle());
+void ShowNativeCertificateViewer(gfx::NativeWindow parent,
+                                 net::X509Certificate* cert) {
+  ShowNativeCertificateViewer(parent, cert->os_cert_handle());
 }
