@@ -1,19 +1,17 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/gtk/gtk_util.h"
 
 #include <cairo/cairo.h>
-#include <gdk/gdkx.h>
-#include <gtk/gtk.h>
 
+#include <algorithm>
 #include <cstdarg>
 #include <map>
 
 #include "base/environment.h"
 #include "base/i18n/rtl.h"
-#include "base/linux_util.h"
 #include "base/logging.h"
 #include "base/nix/xdg_util.h"
 #include "base/string_number_conversions.h"
@@ -21,43 +19,56 @@
 #include "chrome/browser/autocomplete/autocomplete.h"
 #include "chrome/browser/autocomplete/autocomplete_classifier.h"
 #include "chrome/browser/autocomplete/autocomplete_match.h"
-#include "chrome/browser/browser_list.h"
-#include "chrome/browser/browser_window.h"
+#include "chrome/browser/event_disposition.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/ui/gtk/cairo_cached_surface.h"
-#include "chrome/browser/ui/gtk/gtk_theme_provider.h"
-#include "chrome/common/renderer_preferences.h"
-#include "gfx/gtk_util.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/gtk/gtk_theme_service.h"
+#include "content/browser/disposition_utils.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/public/browser/web_contents.h"
+#include "content/public/common/renderer_preferences.h"
 #include "googleurl/src/gurl.h"
 #include "grit/theme_resources.h"
-#include "third_party/skia/include/core/SkBitmap.h"
-#include "third_party/skia/include/core/SkColor.h"
+#include "grit/theme_resources_standard.h"
+#include "ui/base/events.h"
+#include "ui/base/gtk/gtk_compat.h"
+#include "ui/base/gtk/gtk_hig_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/text/text_elider.h"
 #include "ui/base/x/x11_util.h"
+#include "ui/gfx/gtk_util.h"
+#include "ui/gfx/image/cairo_cached_surface.h"
+#include "ui/gfx/image/image.h"
 
 #if defined(OS_CHROMEOS)
 #include "chrome/browser/chromeos/frame/browser_view.h"
 #include "chrome/browser/chromeos/native_dialog_window.h"
-#include "chrome/browser/chromeos/options/options_window_view.h"
-#include "views/window/window.h"
 #else
 #include "chrome/browser/ui/gtk/browser_window_gtk.h"
 #endif
 
-using WebKit::WebDragOperationsMask;
-using WebKit::WebDragOperation;
-using WebKit::WebDragOperationNone;
-using WebKit::WebDragOperationCopy;
-using WebKit::WebDragOperationLink;
-using WebKit::WebDragOperationMove;
+// These conflict with base/tracked_objects.h, so need to come last.
+#include <gdk/gdkx.h>  // NOLINT
+#include <gtk/gtk.h>   // NOLINT
+
+using content::WebContents;
 
 namespace {
 
+#if defined(GOOGLE_CHROME_BUILD)
+static const char* kIconName = "google-chrome";
+#else
+static const char* kIconName = "chromium-browser";
+#endif
+
 const char kBoldLabelMarkup[] = "<span weight='bold'>%s</span>";
 
+// Max size of each component of the button tooltips.
+const size_t kMaxTooltipTitleLength = 100;
+const size_t kMaxTooltipURLLength = 400;
+
 // Callback used in RemoveAllChildren.
 void RemoveWidget(GtkWidget* widget, gpointer container) {
   gtk_container_remove(GTK_CONTAINER(container), widget);
@@ -69,7 +80,7 @@ gboolean OnMouseButtonPressed(GtkWidget* widget, GdkEventButton* event,
                               gpointer userdata) {
   if (event->type == GDK_BUTTON_PRESS) {
     if (gtk_button_get_focus_on_click(GTK_BUTTON(widget)) &&
-        !GTK_WIDGET_HAS_FOCUS(widget)) {
+        !gtk_widget_has_focus(widget)) {
       gtk_widget_grab_focus(widget);
     }
 
@@ -93,12 +104,13 @@ gboolean OnMouseButtonReleased(GtkWidget* widget, GdkEventButton* event,
 // Returns the approximate number of characters that can horizontally fit in
 // |pixel_width| pixels.
 int GetCharacterWidthForPixels(GtkWidget* widget, int pixel_width) {
-  DCHECK(GTK_WIDGET_REALIZED(widget))
+  DCHECK(gtk_widget_get_realized(widget))
       << " widget must be realized to compute font metrics correctly";
 
   PangoContext* context = gtk_widget_create_pango_context(widget);
+  GtkStyle* style = gtk_widget_get_style(widget);
   PangoFontMetrics* metrics = pango_context_get_metrics(context,
-      widget->style->font_desc, pango_context_get_language(context));
+      style->font_desc, pango_context_get_language(context));
 
   // This technique (max of char and digit widths) matches the code in
   // gtklabel.c.
@@ -115,15 +127,17 @@ int GetCharacterWidthForPixels(GtkWidget* widget, int pixel_width) {
 void OnLabelRealize(GtkWidget* label, gpointer pixel_width) {
   gtk_label_set_width_chars(
       GTK_LABEL(label),
-      GetCharacterWidthForPixels(label,GPOINTER_TO_INT(pixel_width)));
+      GetCharacterWidthForPixels(label, GPOINTER_TO_INT(pixel_width)));
 }
 
 // Ownership of |icon_list| is passed to the caller.
 GList* GetIconList() {
-  ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+  ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
   GList* icon_list = NULL;
-  icon_list = g_list_append(icon_list, rb.GetPixbufNamed(IDR_PRODUCT_ICON_32));
-  icon_list = g_list_append(icon_list, rb.GetPixbufNamed(IDR_PRODUCT_LOGO_16));
+  icon_list = g_list_append(icon_list,
+                            rb.GetNativeImageNamed(IDR_PRODUCT_LOGO_32));
+  icon_list = g_list_append(icon_list,
+                            rb.GetNativeImageNamed(IDR_PRODUCT_LOGO_16));
   return icon_list;
 }
 
@@ -145,10 +159,10 @@ gboolean PaintNoBackground(GtkWidget* widget,
 
 #if defined(OS_CHROMEOS)
 
-TabContents* GetBrowserWindowSelectedTabContents(BrowserWindow* window) {
+WebContents* GetBrowserWindowSelectedWebContents(BrowserWindow* window) {
   chromeos::BrowserView* browser_view = static_cast<chromeos::BrowserView*>(
       window);
-  return browser_view->GetSelectedTabContents();
+  return browser_view->GetSelectedWebContents();
 }
 
 GtkWidget* GetBrowserWindowFocusedWidget(BrowserWindow* window) {
@@ -165,10 +179,10 @@ GtkWidget* GetBrowserWindowFocusedWidget(BrowserWindow* window) {
 
 #else
 
-TabContents* GetBrowserWindowSelectedTabContents(BrowserWindow* window) {
+WebContents* GetBrowserWindowSelectedWebContents(BrowserWindow* window) {
   BrowserWindowGtk* browser_window = static_cast<BrowserWindowGtk*>(
       window);
-  return browser_window->browser()->GetSelectedTabContents();
+  return browser_window->browser()->GetSelectedWebContents();
 }
 
 GtkWidget* GetBrowserWindowFocusedWidget(BrowserWindow* window) {
@@ -181,33 +195,35 @@ GtkWidget* GetBrowserWindowFocusedWidget(BrowserWindow* window) {
 
 namespace event_utils {
 
-WindowOpenDisposition DispositionFromEventFlags(guint event_flags) {
-  if ((event_flags & GDK_BUTTON2_MASK) || (event_flags & GDK_CONTROL_MASK)) {
-    return (event_flags & GDK_SHIFT_MASK) ?
-        NEW_FOREGROUND_TAB : NEW_BACKGROUND_TAB;
-  }
+// TODO(shinyak) This function will be removed after refactoring.
+WindowOpenDisposition DispositionFromGdkState(guint state) {
+  int event_flags = EventFlagsFromGdkState(state);
+  return browser::DispositionFromEventFlags(event_flags);
+}
 
-  if (event_flags & GDK_SHIFT_MASK)
-    return NEW_WINDOW;
-  return false /*event.IsAltDown()*/ ? SAVE_TO_DISK : CURRENT_TAB;
+int EventFlagsFromGdkState(guint state) {
+  int flags = 0;
+  flags |= (state & GDK_LOCK_MASK) ? ui::EF_CAPS_LOCK_DOWN : 0;
+  flags |= (state & GDK_CONTROL_MASK) ? ui::EF_CONTROL_DOWN : 0;
+  flags |= (state & GDK_SHIFT_MASK) ? ui::EF_SHIFT_DOWN : 0;
+  flags |= (state & GDK_MOD1_MASK) ? ui::EF_ALT_DOWN : 0;
+  flags |= (state & GDK_BUTTON1_MASK) ? ui::EF_LEFT_MOUSE_BUTTON : 0;
+  flags |= (state & GDK_BUTTON2_MASK) ? ui::EF_MIDDLE_MOUSE_BUTTON : 0;
+  flags |= (state & GDK_BUTTON3_MASK) ? ui::EF_RIGHT_MOUSE_BUTTON : 0;
+  return flags;
 }
 
 }  // namespace event_utils
 
 namespace gtk_util {
 
-const GdkColor kGdkWhite = GDK_COLOR_RGB(0xff, 0xff, 0xff);
-const GdkColor kGdkGray  = GDK_COLOR_RGB(0x7f, 0x7f, 0x7f);
-const GdkColor kGdkBlack = GDK_COLOR_RGB(0x00, 0x00, 0x00);
-const GdkColor kGdkGreen = GDK_COLOR_RGB(0x00, 0xff, 0x00);
-
 GtkWidget* CreateLabeledControlsGroup(std::vector<GtkWidget*>* labels,
                                       const char* text, ...) {
   va_list ap;
   va_start(ap, text);
   GtkWidget* table = gtk_table_new(0, 2, FALSE);
-  gtk_table_set_col_spacing(GTK_TABLE(table), 0, kLabelSpacing);
-  gtk_table_set_row_spacings(GTK_TABLE(table), kControlSpacing);
+  gtk_table_set_col_spacing(GTK_TABLE(table), 0, ui::kLabelSpacing);
+  gtk_table_set_row_spacings(GTK_TABLE(table), ui::kControlSpacing);
 
   for (guint row = 0; text; ++row) {
     gtk_table_resize(GTK_TABLE(table), row + 1, 2);
@@ -262,11 +278,12 @@ GtkWidget* CreateBoldLabel(const std::string& text) {
 void GetWidgetSizeFromCharacters(
     GtkWidget* widget, double width_chars, double height_lines,
     int* width, int* height) {
-  DCHECK(GTK_WIDGET_REALIZED(widget))
+  DCHECK(gtk_widget_get_realized(widget))
       << " widget must be realized to compute font metrics correctly";
   PangoContext* context = gtk_widget_create_pango_context(widget);
+  GtkStyle* style = gtk_widget_get_style(widget);
   PangoFontMetrics* metrics = pango_context_get_metrics(context,
-      widget->style->font_desc, pango_context_get_language(context));
+      style->font_desc, pango_context_get_language(context));
   if (width) {
     *width = static_cast<int>(
         pango_font_metrics_get_approximate_char_width(metrics) *
@@ -285,7 +302,7 @@ void GetWidgetSizeFromCharacters(
 void GetWidgetSizeFromResources(
     GtkWidget* widget, int width_chars, int height_lines,
     int* width, int* height) {
-  DCHECK(GTK_WIDGET_REALIZED(widget))
+  DCHECK(gtk_widget_get_realized(widget))
       << " widget must be realized to compute font metrics correctly";
 
   double chars = 0;
@@ -330,49 +347,7 @@ void SetWindowSizeFromResources(GtkWindow* window,
   gtk_window_set_resizable(window, resizable ? TRUE : FALSE);
 }
 
-void CenterOverWindow(GtkWindow* window, GtkWindow* parent) {
-  gfx::Rect frame_bounds = gtk_util::GetWidgetScreenBounds(GTK_WIDGET(parent));
-  gfx::Point origin = frame_bounds.origin();
-  gfx::Size size = gtk_util::GetWidgetSize(GTK_WIDGET(window));
-  origin.Offset(
-      (frame_bounds.width() - size.width()) / 2,
-      (frame_bounds.height() - size.height()) / 2);
-
-  // Prevent moving window out of monitor bounds.
-  GdkScreen* screen = gtk_window_get_screen(parent);
-  if (screen) {
-    // It would be better to check against workarea for given monitor
-    // but getting workarea for particular monitor is tricky.
-    gint monitor = gdk_screen_get_monitor_at_window(screen,
-        GTK_WIDGET(parent)->window);
-    GdkRectangle rect;
-    gdk_screen_get_monitor_geometry(screen, monitor, &rect);
-
-    // Check the right bottom corner.
-    if (origin.x() > rect.x + rect.width - size.width())
-      origin.set_x(rect.x + rect.width - size.width());
-    if (origin.y() > rect.y + rect.height - size.height())
-      origin.set_y(rect.y + rect.height - size.height());
-
-    // Check the left top corner.
-    if (origin.x() < rect.x)
-      origin.set_x(rect.x);
-    if (origin.y() < rect.y)
-      origin.set_y(rect.y);
-  }
-
-  gtk_window_move(window, origin.x(), origin.y());
-
-  // Move to user expected desktop if window is already visible.
-  if (GTK_WIDGET(window)->window) {
-    ui::ChangeWindowDesktop(
-        ui::GetX11WindowFromGtkWidget(GTK_WIDGET(window)),
-        ui::GetX11WindowFromGtkWidget(GTK_WIDGET(parent)));
-  }
-}
-
 void MakeAppModalWindowGroup() {
-#if GTK_CHECK_VERSION(2, 14, 0)
   // Older versions of GTK+ don't give us gtk_window_group_list() which is what
   // we need to add current non-browser modal dialogs to the list. If
   // we have 2.14+ we can do things the correct way.
@@ -390,20 +365,9 @@ void MakeAppModalWindowGroup() {
     g_list_free(all_windows);
   }
   g_object_unref(window_group);
-#else
-  // Otherwise just grab all browser windows and be slightly broken.
-  GtkWindowGroup* window_group = gtk_window_group_new();
-  for (BrowserList::const_iterator it = BrowserList::begin();
-       it != BrowserList::end(); ++it) {
-    gtk_window_group_add_window(window_group,
-                                (*it)->window()->GetNativeHandle());
-  }
-  g_object_unref(window_group);
-#endif
 }
 
 void AppModalDismissedUngroupWindows() {
-#if GTK_CHECK_VERSION(2, 14, 0)
   if (BrowserList::begin() != BrowserList::end()) {
     std::vector<GtkWindow*> transient_windows;
 
@@ -432,17 +396,6 @@ void AppModalDismissedUngroupWindows() {
       gtk_window_group_add_window(group, *it);
     }
   }
-#else
-  // This is slightly broken in the case where a different window had a dialog,
-  // but its the best we can do since we don't have newer gtk stuff.
-  for (BrowserList::const_iterator it = BrowserList::begin();
-       it != BrowserList::end(); ++it) {
-    GtkWindowGroup* window_group = gtk_window_group_new();
-    gtk_window_group_add_window(window_group,
-                                (*it)->window()->GetNativeHandle());
-    g_object_unref(window_group);
-  }
-#endif
 }
 
 void RemoveAllChildren(GtkWidget* container) {
@@ -450,13 +403,13 @@ void RemoveAllChildren(GtkWidget* container) {
 }
 
 void ForceFontSizePixels(GtkWidget* widget, double size_pixels) {
-  GtkStyle* style = widget->style;
-  PangoFontDescription* font_desc = style->font_desc;
+  PangoFontDescription* font_desc = pango_font_description_new();
   // pango_font_description_set_absolute_size sets the font size in device
   // units, which for us is pixels.
   pango_font_description_set_absolute_size(font_desc,
                                            PANGO_SCALE * size_pixels);
   gtk_widget_modify_font(widget, font_desc);
+  pango_font_description_free(font_desc);
 }
 
 void UndoForceFontSize(GtkWidget* widget) {
@@ -464,17 +417,21 @@ void UndoForceFontSize(GtkWidget* widget) {
 }
 
 gfx::Point GetWidgetScreenPosition(GtkWidget* widget) {
-  if (!widget->window) {
+  GdkWindow* window = gtk_widget_get_window(widget);
+
+  if (!window) {
     NOTREACHED() << "Must only be called on realized widgets.";
     return gfx::Point(0, 0);
   }
 
   gint x, y;
-  gdk_window_get_origin(widget->window, &x, &y);
+  gdk_window_get_origin(window, &x, &y);
 
-  if (!GTK_IS_WINDOW(widget)) {
-    x += widget->allocation.x;
-    y += widget->allocation.y;
+  if (!gtk_widget_get_has_window(widget)) {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(widget, &allocation);
+    x += allocation.x;
+    y += allocation.y;
   }
 
   return gfx::Point(x, y);
@@ -482,8 +439,12 @@ gfx::Point GetWidgetScreenPosition(GtkWidget* widget) {
 
 gfx::Rect GetWidgetScreenBounds(GtkWidget* widget) {
   gfx::Point position = GetWidgetScreenPosition(widget);
+
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+
   return gfx::Rect(position.x(), position.y(),
-                   widget->allocation.width, widget->allocation.height);
+                   allocation.width, allocation.height);
 }
 
 gfx::Size GetWidgetSize(GtkWidget* widget) {
@@ -500,35 +461,6 @@ void ConvertWidgetPointToScreen(GtkWidget* widget, gfx::Point* p) {
   p->SetPoint(p->x() + position.x(), p->y() + position.y());
 }
 
-void InitRCStyles() {
-  static const char kRCText[] =
-      // Make our dialogs styled like the GNOME HIG.
-      //
-      // TODO(evanm): content-area-spacing was introduced in a later
-      // version of GTK, so we need to set that manually on all dialogs.
-      // Perhaps it would make sense to have a shared FixupDialog() function.
-      "style \"gnome-dialog\" {\n"
-      "  xthickness = 12\n"
-      "  GtkDialog::action-area-border = 0\n"
-      "  GtkDialog::button-spacing = 6\n"
-      "  GtkDialog::content-area-spacing = 18\n"
-      "  GtkDialog::content-area-border = 12\n"
-      "}\n"
-      // Note we set it at the "application" priority, so users can override.
-      "widget \"GtkDialog\" style : application \"gnome-dialog\"\n"
-
-      // Make our about dialog special, so the image is flush with the edge.
-      "style \"about-dialog\" {\n"
-      "  GtkDialog::action-area-border = 12\n"
-      "  GtkDialog::button-spacing = 6\n"
-      "  GtkDialog::content-area-spacing = 18\n"
-      "  GtkDialog::content-area-border = 0\n"
-      "}\n"
-      "widget \"about-dialog\" style : application \"about-dialog\"\n";
-
-  gtk_rc_parse_string(kRCText);
-}
-
 GtkWidget* CenterWidgetInHBox(GtkWidget* hbox, GtkWidget* widget,
                               bool pack_at_end, int padding) {
   GtkWidget* centering_vbox = gtk_vbox_new(FALSE, 0);
@@ -541,14 +473,9 @@ GtkWidget* CenterWidgetInHBox(GtkWidget* hbox, GtkWidget* widget,
   return centering_vbox;
 }
 
-bool IsScreenComposited() {
-  GdkScreen* screen = gdk_screen_get_default();
-  return gdk_screen_is_composited(screen) == TRUE;
-}
-
 void EnumerateTopLevelWindows(ui::EnumerateWindowsDelegate* delegate) {
   std::vector<XID> stack;
-  if (!ui::GetXWindowStack(&stack)) {
+  if (!ui::GetXWindowStack(ui::GetX11RootWindow(), &stack)) {
     // Window Manager doesn't support _NET_CLIENT_LIST_STACKING, so fall back
     // to old school enumeration of all X windows.  Some WMs parent 'top-level'
     // windows in unnamed actual top-level windows (ion WM), so extend the
@@ -589,12 +516,27 @@ void SetButtonTriggersNavigation(GtkWidget* button) {
 int MirroredLeftPointForRect(GtkWidget* widget, const gfx::Rect& bounds) {
   if (!base::i18n::IsRTL())
     return bounds.x();
-  return widget->allocation.width - bounds.x() - bounds.width();
+
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+  return allocation.width - bounds.x() - bounds.width();
+}
+
+int MirroredRightPointForRect(GtkWidget* widget, const gfx::Rect& bounds) {
+  if (!base::i18n::IsRTL())
+    return bounds.right();
+
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+  return allocation.width - bounds.x();
 }
 
 int MirroredXCoordinate(GtkWidget* widget, int x) {
-  if (base::i18n::IsRTL())
-    return widget->allocation.width - x;
+  if (base::i18n::IsRTL()) {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(widget, &allocation);
+    return allocation.width - x;
+  }
   return x;
 }
 
@@ -611,10 +553,24 @@ void SetWindowIcon(GtkWindow* window) {
   g_list_free(icon_list);
 }
 
-void SetDefaultWindowIcon() {
-  GList* icon_list = GetIconList();
-  gtk_window_set_default_icon_list(icon_list);
-  g_list_free(icon_list);
+void SetDefaultWindowIcon(GtkWindow* window) {
+  GtkIconTheme* theme =
+      gtk_icon_theme_get_for_screen(gtk_widget_get_screen(GTK_WIDGET(window)));
+
+  if (gtk_icon_theme_has_icon(theme, kIconName)) {
+    gtk_window_set_default_icon_name(kIconName);
+    // Sometimes the WM fails to update the icon when we tell it to. The above
+    // line should be enough to update all existing windows, but it can fail,
+    // e.g. with Lucid/metacity. The following line seems to fix the common
+    // case where the first window created doesn't have an icon.
+    gtk_window_set_icon_name(window, kIconName);
+  } else {
+    GList* icon_list = GetIconList();
+    gtk_window_set_default_icon_list(icon_list);
+    // Same logic applies here.
+    gtk_window_set_icon_list(window, icon_list);
+    g_list_free(icon_list);
+  }
 }
 
 GtkWidget* AddButtonToDialog(GtkWidget* dialog, const gchar* text,
@@ -640,7 +596,7 @@ GtkWidget* BuildDialogButton(GtkWidget* dialog, int ids_id,
 }
 
 GtkWidget* CreateEntryImageHBox(GtkWidget* entry, GtkWidget* image) {
-  GtkWidget* hbox = gtk_hbox_new(FALSE, gtk_util::kControlSpacing);
+  GtkWidget* hbox = gtk_hbox_new(FALSE, ui::kControlSpacing);
   gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
   return hbox;
@@ -656,12 +612,12 @@ void SetLabelColor(GtkWidget* label, const GdkColor* color) {
 GtkWidget* IndentWidget(GtkWidget* content) {
   GtkWidget* content_alignment = gtk_alignment_new(0.0, 0.5, 1.0, 1.0);
   gtk_alignment_set_padding(GTK_ALIGNMENT(content_alignment), 0, 0,
-                            gtk_util::kGroupIndent, 0);
+                            ui::kGroupIndent, 0);
   gtk_container_add(GTK_CONTAINER(content_alignment), content);
   return content_alignment;
 }
 
-void UpdateGtkFontSettings(RendererPreferences* prefs) {
+void UpdateGtkFontSettings(content::RendererPreferences* prefs) {
   DCHECK(prefs);
 
   // From http://library.gnome.org/devel/gtk/unstable/GtkSettings.html, this is
@@ -685,9 +641,9 @@ void UpdateGtkFontSettings(RendererPreferences* prefs) {
 
   // Set some reasonable defaults.
   prefs->should_antialias_text = true;
-  prefs->hinting = RENDERER_PREFERENCES_HINTING_SYSTEM_DEFAULT;
+  prefs->hinting = content::RENDERER_PREFERENCES_HINTING_SYSTEM_DEFAULT;
   prefs->subpixel_rendering =
-      RENDERER_PREFERENCES_SUBPIXEL_RENDERING_SYSTEM_DEFAULT;
+      content::RENDERER_PREFERENCES_SUBPIXEL_RENDERING_SYSTEM_DEFAULT;
 
   if (cursor_blink) {
     // Dividing by 2*1000ms follows the WebKit GTK port and makes the blink
@@ -705,25 +661,30 @@ void UpdateGtkFontSettings(RendererPreferences* prefs) {
     prefs->should_antialias_text = antialias;
 
     if (hinting == 0 || strcmp(hint_style, "hintnone") == 0) {
-      prefs->hinting = RENDERER_PREFERENCES_HINTING_NONE;
+      prefs->hinting = content::RENDERER_PREFERENCES_HINTING_NONE;
     } else if (strcmp(hint_style, "hintslight") == 0) {
-      prefs->hinting = RENDERER_PREFERENCES_HINTING_SLIGHT;
+      prefs->hinting = content::RENDERER_PREFERENCES_HINTING_SLIGHT;
     } else if (strcmp(hint_style, "hintmedium") == 0) {
-      prefs->hinting = RENDERER_PREFERENCES_HINTING_MEDIUM;
+      prefs->hinting = content::RENDERER_PREFERENCES_HINTING_MEDIUM;
     } else if (strcmp(hint_style, "hintfull") == 0) {
-      prefs->hinting = RENDERER_PREFERENCES_HINTING_FULL;
+      prefs->hinting = content::RENDERER_PREFERENCES_HINTING_FULL;
     }
 
     if (strcmp(rgba_style, "none") == 0) {
-      prefs->subpixel_rendering = RENDERER_PREFERENCES_SUBPIXEL_RENDERING_NONE;
+      prefs->subpixel_rendering =
+          content::RENDERER_PREFERENCES_SUBPIXEL_RENDERING_NONE;
     } else if (strcmp(rgba_style, "rgb") == 0) {
-      prefs->subpixel_rendering = RENDERER_PREFERENCES_SUBPIXEL_RENDERING_RGB;
+      prefs->subpixel_rendering =
+          content::RENDERER_PREFERENCES_SUBPIXEL_RENDERING_RGB;
     } else if (strcmp(rgba_style, "bgr") == 0) {
-      prefs->subpixel_rendering = RENDERER_PREFERENCES_SUBPIXEL_RENDERING_BGR;
+      prefs->subpixel_rendering =
+          content::RENDERER_PREFERENCES_SUBPIXEL_RENDERING_BGR;
     } else if (strcmp(rgba_style, "vrgb") == 0) {
-      prefs->subpixel_rendering = RENDERER_PREFERENCES_SUBPIXEL_RENDERING_VRGB;
+      prefs->subpixel_rendering =
+          content::RENDERER_PREFERENCES_SUBPIXEL_RENDERING_VRGB;
     } else if (strcmp(rgba_style, "vbgr") == 0) {
-      prefs->subpixel_rendering = RENDERER_PREFERENCES_SUBPIXEL_RENDERING_VBGR;
+      prefs->subpixel_rendering =
+          content::RENDERER_PREFERENCES_SUBPIXEL_RENDERING_VBGR;
     }
   }
 
@@ -733,24 +694,40 @@ void UpdateGtkFontSettings(RendererPreferences* prefs) {
     g_free(rgba_style);
 }
 
-gfx::Point ScreenPoint(GtkWidget* widget) {
-  int x, y;
-  gdk_display_get_pointer(gtk_widget_get_display(widget), NULL, &x, &y,
-                          NULL);
-  return gfx::Point(x, y);
-}
-
-gfx::Point ClientPoint(GtkWidget* widget) {
-  int x, y;
-  gtk_widget_get_pointer(widget, &x, &y);
-  return gfx::Point(x, y);
-}
-
 GdkPoint MakeBidiGdkPoint(gint x, gint y, gint width, bool ltr) {
   GdkPoint point = {ltr ? x : width - x, y};
   return point;
 }
 
+std::string BuildTooltipTitleFor(string16 title, const GURL& url) {
+  const std::string& url_str = url.possibly_invalid_spec();
+  const std::string& title_str = UTF16ToUTF8(title);
+
+  std::string truncated_url = UTF16ToUTF8(ui::TruncateString(
+      UTF8ToUTF16(url_str), kMaxTooltipURLLength));
+  gchar* escaped_url_cstr = g_markup_escape_text(truncated_url.c_str(),
+                                                 truncated_url.size());
+  std::string escaped_url(escaped_url_cstr);
+  g_free(escaped_url_cstr);
+
+  std::string tooltip;
+  if (url_str == title_str || title.empty()) {
+    return escaped_url;
+  } else {
+    std::string truncated_title = UTF16ToUTF8(ui::TruncateString(
+        title, kMaxTooltipTitleLength));
+    gchar* escaped_title_cstr = g_markup_escape_text(truncated_title.c_str(),
+                                                     truncated_title.size());
+    std::string escaped_title(escaped_title_cstr);
+    g_free(escaped_title_cstr);
+
+    if (!escaped_url.empty())
+      return std::string("<b>") + escaped_title + "</b>\n" + escaped_url;
+    else
+      return std::string("<b>") + escaped_title + "</b>";
+  }
+}
+
 void DrawTextEntryBackground(GtkWidget* offscreen_entry,
                              GtkWidget* widget_to_draw_on,
                              GdkRectangle* dirty_rec,
@@ -787,23 +764,36 @@ void DrawTextEntryBackground(GtkWidget* offscreen_entry,
   g_object_unref(our_style);
 }
 
+void SetLayoutText(PangoLayout* layout, const string16& text) {
+  // Pango is really easy to overflow and send into a computational death
+  // spiral that can corrupt the screen. Assume that we'll never have more than
+  // 2000 characters, which should be a safe assumption until we all get robot
+  // eyes. http://crbug.com/66576
+  std::string text_utf8 = UTF16ToUTF8(text);
+  if (text_utf8.length() > 2000)
+    text_utf8 = text_utf8.substr(0, 2000);
+
+  pango_layout_set_text(layout, text_utf8.data(), text_utf8.length());
+}
+
 void DrawThemedToolbarBackground(GtkWidget* widget,
                                  cairo_t* cr,
                                  GdkEventExpose* event,
                                  const gfx::Point& tabstrip_origin,
-                                 GtkThemeProvider* theme_provider) {
+                                 GtkThemeService* theme_service) {
   // Fill the entire region with the toolbar color.
-  GdkColor color = theme_provider->GetGdkColor(
-      BrowserThemeProvider::COLOR_TOOLBAR);
+  GdkColor color = theme_service->GetGdkColor(
+      ThemeService::COLOR_TOOLBAR);
   gdk_cairo_set_source_color(cr, &color);
   cairo_fill(cr);
 
   // The toolbar is supposed to blend in with the active tab, so we have to pass
   // coordinates for the IDR_THEME_TOOLBAR bitmap relative to the top of the
   // tab strip.
-  CairoCachedSurface* background = theme_provider->GetSurfaceNamed(
-      IDR_THEME_TOOLBAR, widget);
-  background->SetSource(cr, tabstrip_origin.x(), tabstrip_origin.y());
+  const gfx::Image* background =
+      theme_service->GetImageNamed(IDR_THEME_TOOLBAR);
+  background->ToCairo()->SetSource(cr, widget,
+                                   tabstrip_origin.x(), tabstrip_origin.y());
   // We tile the toolbar background in both directions.
   cairo_pattern_set_extend(cairo_get_source(cr), CAIRO_EXTEND_REPEAT);
   cairo_rectangle(cr,
@@ -824,71 +814,26 @@ GdkColor AverageColors(GdkColor color_one, GdkColor color_two) {
 }
 
 void SetAlwaysShowImage(GtkWidget* image_menu_item) {
-  // Compile time check: if it's available, just use the API.
-  // GTK_CHECK_VERSION is TRUE if the passed version is compatible.
-#if GTK_CHECK_VERSION(2, 16, 1)
   gtk_image_menu_item_set_always_show_image(
       GTK_IMAGE_MENU_ITEM(image_menu_item), TRUE);
-#else
-  // Run time check: if the API is not available, set the property manually.
-  // This will still only work with GTK 2.16+ as the property doesn't exist
-  // in earlier versions.
-  // gtk_check_version() returns NULL if the passed version is compatible.
-  if (!gtk_check_version(2, 16, 1)) {
-    GValue true_value = { 0 };
-    g_value_init(&true_value, G_TYPE_BOOLEAN);
-    g_value_set_boolean(&true_value, TRUE);
-    g_object_set_property(G_OBJECT(image_menu_item), "always-show-image",
-                          &true_value);
-  }
-#endif
-}
-
-void StackPopupWindow(GtkWidget* popup, GtkWidget* toplevel) {
-  DCHECK(GTK_IS_WINDOW(popup) && GTK_WIDGET_TOPLEVEL(popup) &&
-         GTK_WIDGET_REALIZED(popup));
-  DCHECK(GTK_IS_WINDOW(toplevel) && GTK_WIDGET_TOPLEVEL(toplevel) &&
-         GTK_WIDGET_REALIZED(toplevel));
-
-  // Stack the |popup| window directly above the |toplevel| window.
-  // The popup window is a direct child of the root window, so we need to
-  // find a similar ancestor for the toplevel window (which might have been
-  // reparented by a window manager).  We grab the server while we're doing
-  // this -- otherwise, we'll get an error if the window manager reparents the
-  // toplevel window right after we call GetHighestAncestorWindow().
-  gdk_x11_display_grab(gtk_widget_get_display(toplevel));
-  XID toplevel_window_base = ui::GetHighestAncestorWindow(
-      ui::GetX11WindowFromGtkWidget(toplevel),
-      ui::GetX11RootWindow());
-  if (toplevel_window_base) {
-    XID window_xid = ui::GetX11WindowFromGtkWidget(popup);
-    XID window_parent = ui::GetParentWindow(window_xid);
-    if (window_parent == ui::GetX11RootWindow()) {
-      ui::RestackWindow(window_xid, toplevel_window_base, true);
-    } else {
-      // The window manager shouldn't reparent override-redirect windows.
-      DLOG(ERROR) << "override-redirect window " << window_xid
-                  << "'s parent is " << window_parent
-                  << ", rather than root window "
-                  << ui::GetX11RootWindow();
-    }
-  }
-  gdk_x11_display_ungrab(gtk_widget_get_display(toplevel));
 }
 
 gfx::Rect GetWidgetRectRelativeToToplevel(GtkWidget* widget) {
-  DCHECK(GTK_WIDGET_REALIZED(widget));
+  DCHECK(gtk_widget_get_realized(widget));
 
   GtkWidget* toplevel = gtk_widget_get_toplevel(widget);
   DCHECK(toplevel);
-  DCHECK(GTK_WIDGET_REALIZED(toplevel));
+  DCHECK(gtk_widget_get_realized(toplevel));
 
   gint x = 0, y = 0;
   gtk_widget_translate_coordinates(widget,
                                    toplevel,
                                    0, 0,
                                    &x, &y);
-  return gfx::Rect(x, y, widget->allocation.width, widget->allocation.height);
+
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+  return gfx::Rect(x, y, allocation.width, allocation.height);
 }
 
 void SuppressDefaultPainting(GtkWidget* container) {
@@ -905,16 +850,18 @@ WindowOpenDisposition DispositionForCurrentButtonPressEvent() {
 
   guint state = event->button.state;
   gdk_event_free(event);
-  return event_utils::DispositionFromEventFlags(state);
+  return event_utils::DispositionFromGdkState(state);
 }
 
 bool GrabAllInput(GtkWidget* widget) {
   guint time = gtk_get_current_event_time();
 
-  if (!GTK_WIDGET_VISIBLE(widget))
+  if (!gtk_widget_get_visible(widget))
     return false;
 
-  if (!gdk_pointer_grab(widget->window, TRUE,
+  GdkWindow* gdk_window = gtk_widget_get_window(widget);
+  if (!gdk_pointer_grab(gdk_window,
+                        TRUE,
                         GdkEventMask(GDK_BUTTON_PRESS_MASK |
                                      GDK_BUTTON_RELEASE_MASK |
                                      GDK_ENTER_NOTIFY_MASK |
@@ -924,8 +871,8 @@ bool GrabAllInput(GtkWidget* widget) {
     return false;
   }
 
-  if (!gdk_keyboard_grab(widget->window, TRUE, time) == 0) {
-    gdk_display_pointer_ungrab(gdk_drawable_get_display(widget->window), time);
+  if (!gdk_keyboard_grab(gdk_window, TRUE, time) == 0) {
+    gdk_display_pointer_ungrab(gdk_drawable_get_display(gdk_window), time);
     return false;
   }
 
@@ -938,15 +885,18 @@ gfx::Rect WidgetBounds(GtkWidget* widget) {
   //
   //   Widget coordinates are a bit odd; for historical reasons, they are
   //   defined as widget->window coordinates for widgets that are not
-  //   GTK_NO_WINDOW widgets, and are relative to widget->allocation.x,
-  //   widget->allocation.y for widgets that are GTK_NO_WINDOW widgets.
+  //   GTK_NO_WINDOW widgets, and are relative to allocation.x, allocation.y
+  //   for widgets that are GTK_NO_WINDOW widgets.
   //
   // So the base is always (0,0).
-  return gfx::Rect(0, 0, widget->allocation.width, widget->allocation.height);
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+  return gfx::Rect(0, 0, allocation.width, allocation.height);
 }
 
 void SetWMLastUserActionTime(GtkWindow* window) {
-  gdk_x11_window_set_user_time(GTK_WIDGET(window)->window, XTimeNow());
+  gdk_x11_window_set_user_time(gtk_widget_get_window(GTK_WIDGET(window)),
+                               XTimeNow());
 }
 
 guint32 XTimeNow() {
@@ -965,8 +915,8 @@ bool URLFromPrimarySelection(Profile* profile, GURL* url) {
   // Use autocomplete to clean up the text, going so far as to turn it into
   // a search query if necessary.
   AutocompleteMatch match;
-  profile->GetAutocompleteClassifier()->Classify(UTF8ToWide(selection_text),
-      std::wstring(), false, &match, NULL);
+  profile->GetAutocompleteClassifier()->Classify(UTF8ToUTF16(selection_text),
+      string16(), false, false, &match, NULL);
   g_free(selection_text);
   if (!match.destination_url.is_valid())
     return false;
@@ -1006,10 +956,34 @@ void GetTextColors(GdkColor* normal_base,
 
 #if defined(OS_CHROMEOS)
 
+GtkWindow* GetLastActiveBrowserWindow() {
+  if (Browser* b = BrowserList::GetLastActive()) {
+    if (!b->is_type_tabbed()) {
+      b = BrowserList::FindTabbedBrowser(b->profile(), true);
+    }
+
+    if (b)
+      return GTK_WINDOW(b->window()->GetNativeHandle());
+  }
+
+  return NULL;
+}
+
+int GetNativeDialogFlags(GtkWindow* dialog) {
+  int flags = chromeos::DIALOG_FLAG_DEFAULT;
+
+  if (gtk_window_get_resizable(dialog))
+    flags |= chromeos::DIALOG_FLAG_RESIZEABLE;
+  if (gtk_window_get_modal(dialog))
+    flags |= chromeos::DIALOG_FLAG_MODAL;
+
+  return flags;
+}
+
 GtkWindow* GetDialogTransientParent(GtkWindow* dialog) {
   GtkWindow* parent = gtk_window_get_transient_for(dialog);
   if (!parent)
-    parent = chromeos::GetOptionsViewParent();
+    parent = GetLastActiveBrowserWindow();
 
   return parent;
 }
@@ -1025,9 +999,7 @@ void ShowDialog(GtkWidget* dialog) {
 
   chromeos::ShowNativeDialog(GetDialogTransientParent(GTK_WINDOW(dialog)),
       dialog,
-      gtk_window_get_resizable(GTK_WINDOW(dialog)) ?
-          chromeos::DIALOG_FLAG_RESIZEABLE :
-          chromeos::DIALOG_FLAG_DEFAULT,
+      GetNativeDialogFlags(GTK_WINDOW(dialog)),
       gfx::Size(width, height),
       gfx::Size());
 }
@@ -1037,9 +1009,9 @@ void ShowDialogWithLocalizedSize(GtkWidget* dialog,
                                  int height_id,
                                  bool resizeable) {
   int width = (width_id == -1) ? 0 :
-      views::Window::GetLocalizedContentsWidth(width_id);
+      views::Widget::GetLocalizedContentsWidth(width_id);
   int height = (height_id == -1) ? 0 :
-      views::Window::GetLocalizedContentsHeight(height_id);
+      views::Widget::GetLocalizedContentsHeight(height_id);
 
   chromeos::ShowNativeDialog(GetDialogTransientParent(GTK_WINDOW(dialog)),
       dialog,
@@ -1049,14 +1021,14 @@ void ShowDialogWithLocalizedSize(GtkWidget* dialog,
       gfx::Size());
 }
 
-void ShowModalDialogWithMinLocalizedWidth(GtkWidget* dialog,
-                                          int width_id) {
+void ShowDialogWithMinLocalizedWidth(GtkWidget* dialog,
+                                     int width_id) {
   int width = (width_id == -1) ? 0 :
-      views::Window::GetLocalizedContentsWidth(width_id);
+      views::Widget::GetLocalizedContentsWidth(width_id);
 
   chromeos::ShowNativeDialog(GetDialogTransientParent(GTK_WINDOW(dialog)),
       dialog,
-      chromeos::DIALOG_FLAG_MODAL,
+      GetNativeDialogFlags(GTK_WINDOW(dialog)),
       gfx::Size(),
       gfx::Size(width, 0));
 }
@@ -1097,8 +1069,8 @@ void ShowDialogWithLocalizedSize(GtkWidget* dialog,
   gtk_widget_show_all(dialog);
 }
 
-void ShowModalDialogWithMinLocalizedWidth(GtkWidget* dialog,
-                                          int width_id) {
+void ShowDialogWithMinLocalizedWidth(GtkWidget* dialog,
+                                     int width_id) {
   gtk_widget_show_all(dialog);
 
   // Suggest a minimum size.
@@ -1143,18 +1115,14 @@ string16 GetStockPreferencesMenuLabel() {
 
 bool IsWidgetAncestryVisible(GtkWidget* widget) {
   GtkWidget* parent = widget;
-  while (parent && GTK_WIDGET_VISIBLE(parent))
+  while (parent && gtk_widget_get_visible(parent))
     parent = parent->parent;
   return !parent;
 }
 
-void SetGtkFont(const std::string& font_name) {
-  g_object_set(gtk_settings_get_default(),
-               "gtk-font-name", font_name.c_str(), NULL);
-}
-
 void SetLabelWidth(GtkWidget* label, int pixel_width) {
   gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
   // Do the simple thing in LTR because the bug only affects right-aligned
   // text. Also, when using the workaround, the label tries to maintain
@@ -1163,7 +1131,7 @@ void SetLabelWidth(GtkWidget* label, int pixel_width) {
     gtk_widget_set_size_request(label, pixel_width, -1);
   } else {
     // The label has to be realized before we can adjust its width.
-    if (GTK_WIDGET_REALIZED(label)) {
+    if (gtk_widget_get_realized(label)) {
       OnLabelRealize(label, GINT_TO_POINTER(pixel_width));
     } else {
       g_signal_connect(label, "realize", G_CALLBACK(OnLabelRealize),
@@ -1181,28 +1149,6 @@ void InitLabelSizeRequestAndEllipsizeMode(GtkWidget* label) {
   gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
 }
 
-GdkDragAction WebDragOpToGdkDragAction(WebDragOperationsMask op) {
-  GdkDragAction action = static_cast<GdkDragAction>(0);
-  if (op & WebDragOperationCopy)
-    action = static_cast<GdkDragAction>(action | GDK_ACTION_COPY);
-  if (op & WebDragOperationLink)
-    action = static_cast<GdkDragAction>(action | GDK_ACTION_LINK);
-  if (op & WebDragOperationMove)
-    action = static_cast<GdkDragAction>(action | GDK_ACTION_MOVE);
-  return action;
-}
-
-WebDragOperationsMask GdkDragActionToWebDragOp(GdkDragAction action) {
-  WebDragOperationsMask op = WebDragOperationNone;
-  if (action & GDK_ACTION_COPY)
-    op = static_cast<WebDragOperationsMask>(op | WebDragOperationCopy);
-  if (action & GDK_ACTION_LINK)
-    op = static_cast<WebDragOperationsMask>(op | WebDragOperationLink);
-  if (action & GDK_ACTION_MOVE)
-    op = static_cast<WebDragOperationsMask>(op | WebDragOperationMove);
-  return op;
-}
-
 void ApplyMessageDialogQuirks(GtkWidget* dialog) {
   if (gtk_window_get_modal(GTK_WINDOW(dialog))) {
     // Work around a KDE 3 window manager bug.
@@ -1225,9 +1171,9 @@ void DoCutCopyPaste(BrowserWindow* window,
   if (widget == NULL)
     return;  // Do nothing if no focused widget.
 
-  TabContents* current_tab = GetBrowserWindowSelectedTabContents(window);
+  WebContents* current_tab = GetBrowserWindowSelectedWebContents(window);
   if (current_tab && widget == current_tab->GetContentNativeView()) {
-    (current_tab->render_view_host()->*method)();
+    (current_tab->GetRenderViewHost()->*method)();
   } else {
     guint id;
     if ((id = g_signal_lookup(signal, G_OBJECT_TYPE(widget))) != 0)
