@@ -7,10 +7,11 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/compiler_specific.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/ui/app_modal_dialogs/native_app_modal_dialog.h"
-#include "gfx/native_widget_types.h"
 #include "ui/base/gtk/gtk_signal.h"
+#include "ui/gfx/native_widget_types.h"
 
 typedef struct _GtkWidget GtkWidget;
 
@@ -22,16 +23,16 @@ class JSModalDialogGtk : public NativeAppModalDialog {
                    gfx::NativeWindow parent_window);
   virtual ~JSModalDialogGtk();
 
-  // Overridden from NativeAppModalDialog:
-  virtual int GetAppModalDialogButtons() const;
-  virtual void ShowAppModalDialog();
-  virtual void ActivateAppModalDialog();
-  virtual void CloseAppModalDialog();
-  virtual void AcceptAppModalDialog();
-  virtual void CancelAppModalDialog();
+  // NativeAppModalDialog:
+  virtual int GetAppModalDialogButtons() const OVERRIDE;
+  virtual void ShowAppModalDialog() OVERRIDE;
+  virtual void ActivateAppModalDialog() OVERRIDE;
+  virtual void CloseAppModalDialog() OVERRIDE;
+  virtual void AcceptAppModalDialog() OVERRIDE;
+  virtual void CancelAppModalDialog() OVERRIDE;
 
  private:
-  CHROMEGTK_CALLBACK_1(JSModalDialogGtk, void, OnDialogResponse, int);
+  CHROMEGTK_CALLBACK_1(JSModalDialogGtk, void, OnResponse, int);
 
   scoped_ptr<JavaScriptAppModalDialog> dialog_;
   GtkWidget* gtk_dialog_;
