@@ -7,9 +7,7 @@
 #pragma once
 
 #include "chrome/browser/notifications/balloon_host.h"
-#include "gfx/native_widget_types.h"
-
-class RenderWidgetHostViewGtk;
+#include "ui/gfx/native_widget_types.h"
 
 // BalloonViewHost class is a delegate to the renderer host for the HTML
 // notification.  When initialized it creates a new RenderViewHost and loads
@@ -19,9 +17,7 @@ class BalloonViewHost : public BalloonHost {
  public:
   explicit BalloonViewHost(Balloon* balloon);
 
-  ~BalloonViewHost() {
-    Shutdown();
-  }
+  virtual ~BalloonViewHost();
 
   // Changes the size of the balloon.
   void UpdateActualSize(const gfx::Size& new_size);
@@ -29,14 +25,7 @@ class BalloonViewHost : public BalloonHost {
   // Accessors.
   gfx::NativeView native_view() const;
 
- protected:
-  virtual void InitRenderWidgetHostView();
-  virtual RenderWidgetHostView* render_widget_host_view() const;
-
  private:
-  // The GTK-specific widget host view.  Owned by its native view.
-  RenderWidgetHostViewGtk* render_widget_host_view_;
-
   DISALLOW_COPY_AND_ASSIGN(BalloonViewHost);
 };
 
