@@ -6,16 +6,17 @@
 #define CHROME_BROWSER_UI_GTK_OVERFLOW_BUTTON_H_
 #pragma once
 
-#include "chrome/browser/ui/gtk/owned_widget_gtk.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "base/compiler_specific.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "ui/base/gtk/owned_widget_gtk.h"
 
 typedef struct _GtkWidget GtkWidget;
 class Profile;
 
 // An overflow chevron button. The button itself is a plain gtk_chrome_button,
 // and this class handles theming it.
-class OverflowButton : public NotificationObserver {
+class OverflowButton : public content::NotificationObserver {
  public:
   explicit OverflowButton(Profile* profile);
   virtual ~OverflowButton();
@@ -23,16 +24,16 @@ class OverflowButton : public NotificationObserver {
   GtkWidget* widget() { return widget_.get(); }
 
  private:
-  // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // content::NotificationObserver implementation.
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
-  OwnedWidgetGtk widget_;
+  ui::OwnedWidgetGtk widget_;
 
   Profile* profile_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 };
 
 #endif  // CHROME_BROWSER_UI_GTK_OVERFLOW_BUTTON_H_
