@@ -8,6 +8,8 @@
 
 #include <gtk/gtk.h>
 
+#include "base/compiler_specific.h"
+#include "chrome/browser/status_icons/desktop_notification_balloon.h"
 #include "chrome/browser/status_icons/status_icon.h"
 #include "ui/base/gtk/gtk_signal.h"
 
@@ -20,16 +22,19 @@ class StatusIconGtk : public StatusIcon {
   virtual ~StatusIconGtk();
 
   // Overridden from StatusIcon:
-  virtual void SetImage(const SkBitmap& image);
-  virtual void SetPressedImage(const SkBitmap& image);
-  virtual void SetToolTip(const string16& tool_tip);
+  virtual void SetImage(const SkBitmap& image) OVERRIDE;
+  virtual void SetPressedImage(const SkBitmap& image) OVERRIDE;
+  virtual void SetToolTip(const string16& tool_tip) OVERRIDE;
+  virtual void DisplayBalloon(const SkBitmap& icon,
+                              const string16& title,
+                              const string16& contents) OVERRIDE;
 
   // Exposed for testing.
   CHROMEGTK_CALLBACK_0(StatusIconGtk, void, OnClick);
 
  protected:
   // Overridden from StatusIcon.
-  virtual void UpdatePlatformContextMenu(ui::MenuModel* menu);
+  virtual void UpdatePlatformContextMenu(ui::MenuModel* menu) OVERRIDE;
 
  private:
   // Callback invoked when user right-clicks on the status icon.
@@ -41,6 +46,9 @@ class StatusIconGtk : public StatusIcon {
   // The context menu for this icon (if any).
   scoped_ptr<MenuGtk> menu_;
 
+  // Notification balloon.
+  DesktopNotificationBalloon notification_;
+
   DISALLOW_COPY_AND_ASSIGN(StatusIconGtk);
 };
 
