@@ -10,26 +10,33 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/task.h"
+#include "base/compiler_specific.h"
+#include "base/memory/weak_ptr.h"
 #include "base/message_loop.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
-#include "chrome/browser/ui/gtk/owned_widget_gtk.h"
 #include "chrome/browser/ui/gtk/tabs/tab_gtk.h"
 #include "chrome/browser/ui/gtk/tabstrip_origin_provider.h"
 #include "chrome/browser/ui/gtk/view_id_util.h"
-#include "chrome/common/notification_observer.h"
-#include "gfx/rect.h"
+#include "chrome/browser/ui/tabs/hover_tab_selector.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "ui/base/gtk/gtk_signal.h"
+#include "ui/base/gtk/owned_widget_gtk.h"
+#include "ui/gfx/rect.h"
 
 class BrowserWindowGtk;
 class CustomDrawButton;
 class DraggedTabControllerGtk;
-class GtkThemeProvider;
+class GtkThemeService;
+
+namespace gfx {
+class Image;
+}
 
 class TabStripGtk : public TabStripModelObserver,
                     public TabGtk::TabDelegate,
                     public MessageLoopForUI::Observer,
-                    public NotificationObserver,
+                    public content::NotificationObserver,
                     public TabstripOriginProvider,
                     public ViewIDUtil::Delegate {
  public:
@@ -83,8 +90,8 @@ class TabStripGtk : public TabStripModelObserver,
   // Destroys the active drag controller.
   void DestroyDragController();
 
-  // Removes the drag source tab from this tabstrip, and deletes it.
-  void DestroyDraggedSourceTab(TabGtk* tab);
+  // Removes |dragged_tab| from this tabstrip, and deletes it.
+  void DestroyDraggedTab(TabGtk* dragged_tab);
 
   // Retrieve the ideal bounds for the Tab at the specified index.
   gfx::Rect GetIdealBounds(int index);
@@ -95,63 +102,73 @@ class TabStripGtk : public TabStripModelObserver,
   void SetVerticalOffset(int offset);
 
   // TabstripOriginProvider implementation -------------------------------------
-  virtual gfx::Point GetTabStripOriginForWidget(GtkWidget* widget);
+  virtual gfx::Point GetTabStripOriginForWidget(GtkWidget* widget) OVERRIDE;
 
   // ViewIDUtil::Delegate implementation ---------------------------------------
-  virtual GtkWidget* GetWidgetForViewID(ViewID id);
+  virtual GtkWidget* GetWidgetForViewID(ViewID id) OVERRIDE;
 
  protected:
   // TabStripModelObserver implementation:
   virtual void TabInsertedAt(TabContentsWrapper* contents,
                              int index,
-                             bool foreground);
-  virtual void TabDetachedAt(TabContentsWrapper* contents, int index);
-  virtual void TabSelectedAt(TabContentsWrapper* old_contents,
-                             TabContentsWrapper* contents,
-                             int index,
-                             bool user_gesture);
+                             bool foreground) OVERRIDE;
+  virtual void TabDetachedAt(TabContentsWrapper* contents, int index) OVERRIDE;
   virtual void TabMoved(TabContentsWrapper* contents,
                         int from_index,
-                        int to_index);
+                        int to_index) OVERRIDE;
+  virtual void ActiveTabChanged(TabContentsWrapper* old_contents,
+                                TabContentsWrapper* new_contents,
+                                int index,
+                                bool user_gesture) OVERRIDE;
+  virtual void TabSelectionChanged(
+      TabStripModel* tab_strip_model,
+      const TabStripSelectionModel& old_model) OVERRIDE;
   virtual void TabChangedAt(TabContentsWrapper* contents, int index,
-                            TabChangeType change_type);
+                            TabChangeType change_type) OVERRIDE;
   virtual void TabReplacedAt(TabStripModel* tab_strip_model,
                              TabContentsWrapper* old_contents,
                              TabContentsWrapper* new_contents,
-                             int index);
-  virtual void TabMiniStateChanged(TabContentsWrapper* contents, int index);
+                             int index) OVERRIDE;
+  virtual void TabMiniStateChanged(TabContentsWrapper* contents,
+                                   int index) OVERRIDE;
   virtual void TabBlockedStateChanged(TabContentsWrapper* contents,
-                                      int index);
+                                      int index) OVERRIDE;
 
   // TabGtk::TabDelegate implementation:
-  virtual bool IsTabSelected(const TabGtk* tab) const;
-  virtual bool IsTabPinned(const TabGtk* tab) const;
-  virtual bool IsTabDetached(const TabGtk* tab) const;
-  virtual void SelectTab(TabGtk* tab);
-  virtual void CloseTab(TabGtk* tab);
+  virtual bool IsTabActive(const TabGtk* tab) const OVERRIDE;
+  virtual bool IsTabSelected(const TabGtk* tab) const OVERRIDE;
+  virtual bool IsTabPinned(const TabGtk* tab) const OVERRIDE;
+  virtual bool IsTabDetached(const TabGtk* tab) const OVERRIDE;
+  virtual void ActivateTab(TabGtk* tab) OVERRIDE;
+  virtual void ToggleTabSelection(TabGtk* tab) OVERRIDE;
+  virtual void ExtendTabSelection(TabGtk* tab) OVERRIDE;
+  virtual void CloseTab(TabGtk* tab) OVERRIDE;
   virtual bool IsCommandEnabledForTab(
-      TabStripModel::ContextMenuCommand command_id, const TabGtk* tab) const;
+      TabStripModel::ContextMenuCommand command_id,
+      const TabGtk* tab) const OVERRIDE;
   virtual void ExecuteCommandForTab(
-      TabStripModel::ContextMenuCommand command_id, TabGtk* tab);
+      TabStripModel::ContextMenuCommand command_id, TabGtk* tab) OVERRIDE;
   virtual void StartHighlightTabsForCommand(
-      TabStripModel::ContextMenuCommand command_id, TabGtk* tab);
+      TabStripModel::ContextMenuCommand command_id, TabGtk* tab) OVERRIDE;
   virtual void StopHighlightTabsForCommand(
-      TabStripModel::ContextMenuCommand command_id, TabGtk* tab);
-  virtual void StopAllHighlighting();
-  virtual void MaybeStartDrag(TabGtk* tab, const gfx::Point& point);
-  virtual void ContinueDrag(GdkDragContext* context);
-  virtual bool EndDrag(bool canceled);
-  virtual bool HasAvailableDragActions() const;
-  virtual ui::ThemeProvider* GetThemeProvider();
+      TabStripModel::ContextMenuCommand command_id, TabGtk* tab) OVERRIDE;
+  virtual void StopAllHighlighting() OVERRIDE;
+  virtual void MaybeStartDrag(TabGtk* tab, const gfx::Point& point) OVERRIDE;
+  virtual void ContinueDrag(GdkDragContext* context) OVERRIDE;
+  virtual bool EndDrag(bool canceled) OVERRIDE;
+  virtual bool HasAvailableDragActions() const OVERRIDE;
+  virtual GtkThemeService* GetThemeProvider() OVERRIDE;
+  virtual TabStripMenuController* GetTabStripMenuControllerForTab(
+      TabGtk* tab) OVERRIDE;
 
   // MessageLoop::Observer implementation:
-  virtual void WillProcessEvent(GdkEvent* event);
-  virtual void DidProcessEvent(GdkEvent* event);
+  virtual void WillProcessEvent(GdkEvent* event) OVERRIDE;
+  virtual void DidProcessEvent(GdkEvent* event) OVERRIDE;
 
-  // Overridden from NotificationObserver:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // Overridden from content::NotificationObserver:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // Horizontal gap between mini-tabs and normal tabs.
   static const int mini_to_non_mini_gap_;
@@ -220,12 +237,17 @@ class TabStripGtk : public TabStripModelObserver,
     GtkWidget* container;
 
     // The drop indicator image.
-    GdkPixbuf* drop_arrow;
+    gfx::Image* drop_arrow;
 
    private:
     DISALLOW_COPY_AND_ASSIGN(DropInfo);
   };
 
+  // Map signal handler that sets initial z-ordering. The widgets need to be
+  // realized before we can set the stacking. We use the "map" signal since the
+  // "realize" signal is called before the child widgets get realized.
+  CHROMEGTK_CALLBACK_0(TabStripGtk, void, OnMap);
+
   // expose-event handler that redraws the tabstrip
   CHROMEGTK_CALLBACK_1(TabStripGtk, gboolean, OnExpose, GdkEventExpose*);
 
@@ -260,17 +282,20 @@ class TabStripGtk : public TabStripModelObserver,
   // Returns true if |rects| are all areas that match up with tab favicons.
   // |rects| must be sorted from left to right.  |tabs_to_paint| are the tab
   // positions that match the rects.
-  bool CanPaintOnlyFavIcons(const GdkRectangle* rects,
+  bool CanPaintOnlyFavicons(const GdkRectangle* rects,
                             int num_rects,
                             std::vector<int>* tabs_to_paint);
 
   // Paints the tab favicon areas for tabs in |tabs_to_paint|.
-  void PaintOnlyFavIcons(GdkEventExpose* event,
+  void PaintOnlyFavicons(GdkEventExpose* event,
                          const std::vector<int>& tabs_to_paint);
 
   // Initializes the new tab button.
   CustomDrawButton* MakeNewTabButton();
 
+  // Sets the theme specific background on the new tab button.
+  void SetNewTabButtonBackground();
+
   // Gets the number of Tabs in the collection.
   int GetTabCount() const;
 
@@ -311,14 +336,16 @@ class TabStripGtk : public TabStripModelObserver,
   // Returns the x-coordinate tabs start from.
   int tab_start_x() const;
 
-  // Perform an animated resize-relayout of the TabStrip immediately. The
-  // value returned indicates whether a resize actually took place.
-  bool ResizeLayoutTabs();
+  // Perform an animated resize-relayout of the TabStrip immediately.
+  void ResizeLayoutTabs();
 
   // Returns whether or not the cursor is currently in the "tab strip zone"
   // which is defined as the region above the TabStrip and a bit below it.
   bool IsCursorInTabStripZone() const;
 
+  // Reset the Z-ordering of tabs.
+  void ReStack();
+
   // Ensure that the message loop observer used for event spying is added and
   // removed appropriately so we can tell when to resize layout the tab strip.
   void AddMessageLoopObserver();
@@ -371,7 +398,7 @@ class TabStripGtk : public TabStripModelObserver,
 
   // Returns the image to use for indicating a drop on a tab. If is_down is
   // true, this returns an arrow pointing down.
-  static GdkPixbuf* GetDropArrowImage(bool is_down);
+  static gfx::Image* GetDropArrowImage(bool is_down);
 
   // -- Animations -------------------------------------------------------------
 
@@ -384,7 +411,7 @@ class TabStripGtk : public TabStripModelObserver,
 
   // Starts various types of TabStrip animations.
   void StartInsertTabAnimation(int index);
-  void StartRemoveTabAnimation(int index, TabContents* contents);
+  void StartRemoveTabAnimation(int index, content::WebContents* contents);
   void StartMoveTabAnimation(int from_index, int to_index);
   void StartMiniTabAnimation(int index);
   void StartMiniMoveTabAnimation(int from_index,
@@ -396,8 +423,6 @@ class TabStripGtk : public TabStripModelObserver,
   // Optionally a full Layout will be performed, specified by |layout|.
   void FinishAnimation(TabAnimation* animation, bool layout);
 
-  NotificationRegistrar registrar_;
-
   // The Tabs we contain, and their last generated "good" bounds.
   std::vector<TabData> tab_data_;
 
@@ -422,7 +447,7 @@ class TabStripGtk : public TabStripModelObserver,
   bool needs_resize_layout_;
 
   // The GtkFixed widget.
-  OwnedWidgetGtk tabstrip_;
+  ui::OwnedWidgetGtk tabstrip_;
 
   // The bounds of the tabstrip.
   gfx::Rect bounds_;
@@ -438,7 +463,7 @@ class TabStripGtk : public TabStripModelObserver,
   BrowserWindowGtk* window_;
 
   // Theme resources.
-  GtkThemeProvider* theme_provider_;
+  GtkThemeService* theme_service_;
 
   // The currently running animation.
   scoped_ptr<TabAnimation> active_animation_;
@@ -446,6 +471,9 @@ class TabStripGtk : public TabStripModelObserver,
   // The New Tab button.
   scoped_ptr<CustomDrawButton> newtab_button_;
 
+  // The bounds of the bitmap surface used to paint the New Tab button.
+  gfx::Rect newtab_surface_bounds_;
+
   // Valid for the lifetime of a drag over us.
   scoped_ptr<DropInfo> drop_info_;
 
@@ -455,11 +483,19 @@ class TabStripGtk : public TabStripModelObserver,
 
   // A factory that is used to construct a delayed callback to the
   // ResizeLayoutTabsNow method.
-  ScopedRunnableMethodFactory<TabStripGtk> resize_layout_factory_;
+  base::WeakPtrFactory<TabStripGtk> weak_factory_;
+
+  // A different factory for calls to Layout().
+  base::WeakPtrFactory<TabStripGtk> layout_factory_;
 
   // True if the tabstrip has already been added as a MessageLoop observer.
   bool added_as_message_loop_observer_;
 
+  // Helper for performing tab selection as a result of dragging over a tab.
+  HoverTabSelector hover_tab_selector_;
+
+  content::NotificationRegistrar registrar_;
+
   DISALLOW_COPY_AND_ASSIGN(TabStripGtk);
 };
 
