@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,22 +19,25 @@
 #include "chrome/browser/defaults.h"
 #include "chrome/browser/memory_purger.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/prefs/scoped_user_pref_update.h"
 #include "chrome/browser/ui/gtk/gtk_chrome_link_button.h"
-#include "chrome/browser/ui/gtk/gtk_theme_provider.h"
+#include "chrome/browser/ui/gtk/gtk_theme_service.h"
 #include "chrome/browser/ui/gtk/gtk_tree.h"
 #include "chrome/browser/ui/gtk/gtk_util.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
-#include "gfx/gtk_util.h"
-#include "grit/app_resources.h"
 #include "grit/chromium_strings.h"
+#include "grit/ui_resources.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "ui/base/gtk/gtk_hig_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/models/simple_menu_model.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/gtk_util.h"
+#include "ui/gfx/image/image.h"
 
 #if defined(TOOLKIT_VIEWS)
-#include "views/controls/menu/menu_2.h"
+#include "ui/views/controls/menu/menu_2.h"
 #else
 #include "chrome/browser/ui/gtk/menu_gtk.h"
 #endif
@@ -57,6 +60,7 @@ const gint kTaskManagerPurgeMemory = 3;
 enum TaskManagerColumn {
   kTaskManagerIcon,
   kTaskManagerPage,
+  kTaskManagerProfileName,
   kTaskManagerSharedMem,
   kTaskManagerPrivateMem,
   kTaskManagerCPU,
@@ -66,15 +70,25 @@ enum TaskManagerColumn {
   kTaskManagerWebCoreImageCache,
   kTaskManagerWebCoreScriptsCache,
   kTaskManagerWebCoreCssCache,
+  kTaskManagerFPS,
   kTaskManagerSqliteMemoryUsed,
   kTaskManagerGoatsTeleported,
+  // Columns below this point are not visible in the task manager.
+  kTaskManagerBackgroundColor,
   kTaskManagerColumnCount,
 };
 
+const TaskManagerColumn kTaskManagerLastVisibleColumn =
+    kTaskManagerGoatsTeleported;
+
+static const GdkColor kHighlightColor = GDK_COLOR_RGB(0xff, 0xfa, 0xcd);
+
 TaskManagerColumn TaskManagerResourceIDToColumnID(int id) {
   switch (id) {
     case IDS_TASK_MANAGER_PAGE_COLUMN:
       return kTaskManagerPage;
+    case IDS_TASK_MANAGER_PROFILE_NAME_COLUMN:
+      return kTaskManagerProfileName;
     case IDS_TASK_MANAGER_SHARED_MEM_COLUMN:
       return kTaskManagerSharedMem;
     case IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN:
@@ -93,6 +107,8 @@ TaskManagerColumn TaskManagerResourceIDToColumnID(int id) {
       return kTaskManagerWebCoreScriptsCache;
     case IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN:
       return kTaskManagerWebCoreCssCache;
+    case IDS_TASK_MANAGER_FPS_COLUMN:
+      return kTaskManagerFPS;
     case IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN:
       return kTaskManagerSqliteMemoryUsed;
     case IDS_TASK_MANAGER_GOATS_TELEPORTED_COLUMN:
@@ -107,6 +123,8 @@ int TaskManagerColumnIDToResourceID(int id) {
   switch (id) {
     case kTaskManagerPage:
       return IDS_TASK_MANAGER_PAGE_COLUMN;
+    case kTaskManagerProfileName:
+      return IDS_TASK_MANAGER_PROFILE_NAME_COLUMN;
     case kTaskManagerSharedMem:
       return IDS_TASK_MANAGER_SHARED_MEM_COLUMN;
     case kTaskManagerPrivateMem:
@@ -125,6 +143,8 @@ int TaskManagerColumnIDToResourceID(int id) {
       return IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN;
     case kTaskManagerWebCoreCssCache:
       return IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN;
+    case kTaskManagerFPS:
+      return IDS_TASK_MANAGER_FPS_COLUMN;
     case kTaskManagerSqliteMemoryUsed:
       return IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN;
     case kTaskManagerGoatsTeleported:
@@ -165,13 +185,20 @@ void TreeViewInsertColumnWithPixbuf(GtkWidget* treeview, int resid) {
   GtkTreeViewColumn* column = gtk_tree_view_column_new();
   gtk_tree_view_column_set_title(column,
                                  l10n_util::GetStringUTF8(resid).c_str());
+  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(treeview), colid);
   GtkCellRenderer* image_renderer = gtk_cell_renderer_pixbuf_new();
   gtk_tree_view_column_pack_start(column, image_renderer, FALSE);
   gtk_tree_view_column_add_attribute(column, image_renderer,
                                      "pixbuf", kTaskManagerIcon);
+  gtk_tree_view_column_add_attribute(column, image_renderer,
+                                     "cell-background-gdk",
+                                     kTaskManagerBackgroundColor);
   GtkCellRenderer* text_renderer = gtk_cell_renderer_text_new();
   gtk_tree_view_column_pack_start(column, text_renderer, TRUE);
-  gtk_tree_view_column_add_attribute(column, text_renderer, "text", colid);
+  gtk_tree_view_column_add_attribute(column, text_renderer, "markup", colid);
+  gtk_tree_view_column_add_attribute(column, text_renderer,
+                                     "cell-background-gdk",
+                                     kTaskManagerBackgroundColor);
   gtk_tree_view_column_set_resizable(column, TRUE);
   // This is temporary: we'll turn expanding off after getting the size.
   gtk_tree_view_column_set_expand(column, TRUE);
@@ -183,10 +210,12 @@ void TreeViewInsertColumnWithPixbuf(GtkWidget* treeview, int resid) {
 void TreeViewInsertColumnWithName(GtkWidget* treeview,
                                   TaskManagerColumn colid, const char* name) {
   GtkCellRenderer* renderer = gtk_cell_renderer_text_new();
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeview), -1,
-                                              name, renderer,
-                                              "text", colid,
-                                              NULL);
+  gtk_tree_view_insert_column_with_attributes(
+      GTK_TREE_VIEW(treeview), -1,
+      name, renderer,
+      "text", colid,
+      "cell-background-gdk", kTaskManagerBackgroundColor,
+      NULL);
   GtkTreeViewColumn* column = gtk_tree_view_get_column(
       GTK_TREE_VIEW(treeview), TreeViewColumnIndexFromID(colid));
   gtk_tree_view_column_set_resizable(column, TRUE);
@@ -219,7 +248,7 @@ class TaskManagerGtk::ContextMenuController
   explicit ContextMenuController(TaskManagerGtk* task_manager)
       : task_manager_(task_manager) {
     menu_model_.reset(new ui::SimpleMenuModel(this));
-    for (int i = kTaskManagerPage; i < kTaskManagerColumnCount; i++) {
+    for (int i = kTaskManagerPage; i <= kTaskManagerLastVisibleColumn; i++) {
       menu_model_->AddCheckItemWithStringId(
           i, TaskManagerColumnIDToResourceID(i));
     }
@@ -232,15 +261,13 @@ class TaskManagerGtk::ContextMenuController
 
   virtual ~ContextMenuController() {}
 
+  void RunMenu(const gfx::Point& point, guint32 event_time) {
 #if defined(TOOLKIT_VIEWS)
-  void RunMenu(const gfx::Point& point) {
     menu_->RunContextMenuAt(point);
-  }
 #else
-  void RunMenu() {
-    menu_->PopupAsContext(gtk_get_current_event_time());
-  }
+    menu_->PopupAsContext(point, event_time);
 #endif
+  }
 
   void Cancel() {
     task_manager_ = NULL;
@@ -298,14 +325,15 @@ class TaskManagerGtk::ContextMenuController
   DISALLOW_COPY_AND_ASSIGN(ContextMenuController);
 };
 
-TaskManagerGtk::TaskManagerGtk()
+TaskManagerGtk::TaskManagerGtk(bool highlight_background_resources)
   : task_manager_(TaskManager::GetInstance()),
     model_(TaskManager::GetInstance()->model()),
     dialog_(NULL),
     treeview_(NULL),
     process_list_(NULL),
     process_count_(0),
-    ignore_selection_changed_(false) {
+    ignore_selection_changed_(false),
+    highlight_background_resources_(highlight_background_resources) {
   Init();
 }
 
@@ -335,12 +363,20 @@ void TaskManagerGtk::OnModelChanged() {
 
 void TaskManagerGtk::OnItemsChanged(int start, int length) {
   GtkTreeIter iter;
-  gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(process_list_), &iter,
-                                NULL, start);
+  if (!gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(process_list_), &iter,
+                                     NULL, start)) {
+    NOTREACHED() << "Can't get child " << start <<
+        " from GTK_TREE_MODEL(process_list_)";
+  }
 
   for (int i = start; i < start + length; i++) {
     SetRowDataFromModel(i, &iter);
-    gtk_tree_model_iter_next(GTK_TREE_MODEL(process_list_), &iter);
+    if (i != start + length - 1) {
+      if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(process_list_), &iter)) {
+        NOTREACHED() << "Can't get next GtkTreeIter object from process_list_ "
+                        "iterator at position " << i;
+      }
+    }
   }
 }
 
@@ -394,13 +430,26 @@ void TaskManagerGtk::OnItemsRemoved(int start, int length) {
 ////////////////////////////////////////////////////////////////////////////////
 // TaskManagerGtk, public:
 
+void TaskManagerGtk::Close() {
+  // Blow away our dialog - this will cause TaskManagerGtk to free itself.
+  gtk_widget_destroy(dialog_);
+  DCHECK(!instance_);
+}
+
 // static
-void TaskManagerGtk::Show() {
+void TaskManagerGtk::Show(bool highlight_background_resources) {
+  if (instance_ &&
+      instance_->highlight_background_resources_ !=
+          highlight_background_resources) {
+    instance_->Close();
+    DCHECK(!instance_);
+  }
+
   if (instance_) {
     // If there's a Task manager window open already, just activate it.
     gtk_util::PresentWindow(instance_->dialog_, 0);
   } else {
-    instance_ = new TaskManagerGtk;
+    instance_ = new TaskManagerGtk(highlight_background_resources);
     instance_->model_->StartUpdating();
   }
 }
@@ -434,7 +483,8 @@ void TaskManagerGtk::Init() {
   }
 
   gtk_dialog_add_button(GTK_DIALOG(dialog_),
-      l10n_util::GetStringUTF8(IDS_TASK_MANAGER_KILL).c_str(),
+      gfx::ConvertAcceleratorsFromWindowsStyle(
+          l10n_util::GetStringUTF8(IDS_TASK_MANAGER_KILL)).c_str(),
       kTaskManagerResponseKill);
 
   // The response button should not be sensitive when the dialog is first opened
@@ -449,19 +499,26 @@ void TaskManagerGtk::Init() {
 
   // Setting the link widget to secondary positions the button on the left side
   // of the action area (vice versa for RTL layout).
-  gtk_button_box_set_child_secondary(
-      GTK_BUTTON_BOX(GTK_DIALOG(dialog_)->action_area), link, TRUE);
+  GtkWidget* action_area = gtk_dialog_get_action_area(GTK_DIALOG(dialog_));
+  gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(action_area), link, TRUE);
 
   ConnectAccelerators();
 
-  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog_)->vbox),
-                      gtk_util::kContentAreaSpacing);
+  GtkWidget* content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog_));
+  gtk_box_set_spacing(GTK_BOX(content_area), ui::kContentAreaSpacing);
 
   destroy_handler_id_ = g_signal_connect(dialog_, "destroy",
                                          G_CALLBACK(OnDestroyThunk), this);
   g_signal_connect(dialog_, "response", G_CALLBACK(OnResponseThunk), this);
+  // GTK does menu on mouse-up while views does menu on mouse-down,
+  // so connect to different handlers.
+#if defined(TOOLKIT_VIEWS)
   g_signal_connect(dialog_, "button-release-event",
-                   G_CALLBACK(OnButtonReleaseEventThunk), this);
+                   G_CALLBACK(OnButtonEventThunk), this);
+#else
+  g_signal_connect(dialog_, "button-press-event",
+                   G_CALLBACK(OnButtonEventThunk), this);
+#endif
   gtk_widget_add_events(dialog_,
                         GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 
@@ -478,6 +535,10 @@ void TaskManagerGtk::Init() {
   gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(treeview_), TRUE);
   g_signal_connect(treeview_, "row-activated",
                    G_CALLBACK(OnRowActivatedThunk), this);
+#if defined(TOOLKIT_GTK)
+  g_signal_connect(treeview_, "button-press-event",
+                   G_CALLBACK(OnButtonEventThunk), this);
+#endif
 
   // |selection| is owned by |treeview_|.
   GtkTreeSelection* selection = gtk_tree_view_get_selection(
@@ -541,9 +602,9 @@ void TaskManagerGtk::ConnectAccelerators() {
 void TaskManagerGtk::CreateTaskManagerTreeview() {
   process_list_ = gtk_list_store_new(kTaskManagerColumnCount,
       GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-      G_TYPE_STRING);
+      G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_COLOR);
 
   // Support sorting on all columns.
   process_list_sort_ = gtk_tree_model_sort_new_with_model(
@@ -551,6 +612,9 @@ void TaskManagerGtk::CreateTaskManagerTreeview() {
   gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(process_list_sort_),
                                   kTaskManagerPage,
                                   ComparePage, this, NULL);
+  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(process_list_sort_),
+                                  kTaskManagerPage,
+                                  CompareProfileName, this, NULL);
   gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(process_list_sort_),
                                   kTaskManagerSharedMem,
                                   CompareSharedMemory, this, NULL);
@@ -578,6 +642,9 @@ void TaskManagerGtk::CreateTaskManagerTreeview() {
   gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(process_list_sort_),
                                   kTaskManagerWebCoreCssCache,
                                   CompareWebCoreCssCache, this, NULL);
+  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(process_list_sort_),
+                                  kTaskManagerFPS,
+                                  CompareFPS, this, NULL);
   gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(process_list_sort_),
                                   kTaskManagerSqliteMemoryUsed,
                                   CompareSqliteMemoryUsed, this, NULL);
@@ -588,6 +655,8 @@ void TaskManagerGtk::CreateTaskManagerTreeview() {
 
   // Insert all the columns.
   TreeViewInsertColumnWithPixbuf(treeview_, IDS_TASK_MANAGER_PAGE_COLUMN);
+  TreeViewInsertColumnWithPixbuf(treeview_,
+                                 IDS_TASK_MANAGER_PROFILE_NAME_COLUMN);
   TreeViewInsertColumn(treeview_, IDS_TASK_MANAGER_SHARED_MEM_COLUMN);
   TreeViewInsertColumn(treeview_, IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN);
   TreeViewInsertColumn(treeview_, IDS_TASK_MANAGER_CPU_COLUMN);
@@ -599,10 +668,12 @@ void TaskManagerGtk::CreateTaskManagerTreeview() {
   TreeViewInsertColumn(treeview_,
                        IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN);
   TreeViewInsertColumn(treeview_, IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN);
+  TreeViewInsertColumn(treeview_, IDS_TASK_MANAGER_FPS_COLUMN);
   TreeViewInsertColumn(treeview_, IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN);
   TreeViewInsertColumn(treeview_, IDS_TASK_MANAGER_GOATS_TELEPORTED_COLUMN);
 
   // Hide some columns by default.
+  TreeViewColumnSetVisible(treeview_, kTaskManagerProfileName, false);
   TreeViewColumnSetVisible(treeview_, kTaskManagerSharedMem, false);
   TreeViewColumnSetVisible(treeview_, kTaskManagerProcessID, false);
   TreeViewColumnSetVisible(treeview_, kTaskManagerJavaScriptMemory, false);
@@ -640,6 +711,9 @@ std::string TaskManagerGtk::GetModelText(int row, int col_id) {
     case IDS_TASK_MANAGER_PAGE_COLUMN:  // Process
       return UTF16ToUTF8(model_->GetResourceTitle(row));
 
+    case IDS_TASK_MANAGER_PROFILE_NAME_COLUMN:  // Profile name
+      return UTF16ToUTF8(model_->GetResourceProfileName(row));
+
     case IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN:  // Memory
       return UTF16ToUTF8(model_->GetResourcePrivateMemory(row));
 
@@ -667,6 +741,9 @@ std::string TaskManagerGtk::GetModelText(int row, int col_id) {
     case IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN:
       return UTF16ToUTF8(model_->GetResourceWebCoreCSSCacheSize(row));
 
+    case IDS_TASK_MANAGER_FPS_COLUMN:
+      return UTF16ToUTF8(model_->GetResourceFPS(row));
+
     case IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN:
       return UTF16ToUTF8(model_->GetResourceSqliteMemoryUsed(row));
 
@@ -682,10 +759,10 @@ std::string TaskManagerGtk::GetModelText(int row, int col_id) {
 GdkPixbuf* TaskManagerGtk::GetModelIcon(int row) {
   SkBitmap icon = model_->GetResourceIcon(row);
   if (icon.pixelRef() ==
-      ResourceBundle::GetSharedInstance().GetBitmapNamed(
+      ui::ResourceBundle::GetSharedInstance().GetBitmapNamed(
           IDR_DEFAULT_FAVICON)->pixelRef()) {
     return static_cast<GdkPixbuf*>(g_object_ref(
-        GtkThemeProvider::GetDefaultFavicon(true)));
+        GtkThemeService::GetDefaultFavicon(true)->ToGdkPixbuf()));
   }
 
   return gfx::GdkPixbufFromSkBitmap(&icon);
@@ -694,8 +771,11 @@ GdkPixbuf* TaskManagerGtk::GetModelIcon(int row) {
 void TaskManagerGtk::SetRowDataFromModel(int row, GtkTreeIter* iter) {
   GdkPixbuf* icon = GetModelIcon(row);
   std::string page = GetModelText(row, IDS_TASK_MANAGER_PAGE_COLUMN);
-  std::string shared_mem = GetModelText(
-      row, IDS_TASK_MANAGER_SHARED_MEM_COLUMN);
+  std::string profile_name =
+      GetModelText(row, IDS_TASK_MANAGER_PROFILE_NAME_COLUMN);
+  gchar* page_markup = g_markup_escape_text(page.c_str(), page.length());
+  std::string shared_mem =
+      GetModelText(row, IDS_TASK_MANAGER_SHARED_MEM_COLUMN);
   std::string priv_mem = GetModelText(row, IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN);
   std::string cpu = GetModelText(row, IDS_TASK_MANAGER_CPU_COLUMN);
   std::string net = GetModelText(row, IDS_TASK_MANAGER_NET_COLUMN);
@@ -704,31 +784,43 @@ void TaskManagerGtk::SetRowDataFromModel(int row, GtkTreeIter* iter) {
   // Querying the renderer metrics is slow as it has to do IPC, so only do it
   // when the columns are visible.
   std::string javascript_memory;
-  if (TreeViewColumnIsVisible(treeview_, kTaskManagerJavaScriptMemory))
-    javascript_memory = GetModelText(
-        row, IDS_TASK_MANAGER_JAVASCRIPT_MEMORY_ALLOCATED_COLUMN);
+  if (TreeViewColumnIsVisible(treeview_, kTaskManagerJavaScriptMemory)) {
+    javascript_memory =
+        GetModelText(row, IDS_TASK_MANAGER_JAVASCRIPT_MEMORY_ALLOCATED_COLUMN);
+  }
   std::string wk_img_cache;
-  if (TreeViewColumnIsVisible(treeview_, kTaskManagerWebCoreImageCache))
-    wk_img_cache = GetModelText(
-        row, IDS_TASK_MANAGER_WEBCORE_IMAGE_CACHE_COLUMN);
+  if (TreeViewColumnIsVisible(treeview_, kTaskManagerWebCoreImageCache)) {
+    wk_img_cache =
+        GetModelText(row, IDS_TASK_MANAGER_WEBCORE_IMAGE_CACHE_COLUMN);
+  }
   std::string wk_scripts_cache;
-  if (TreeViewColumnIsVisible(treeview_, kTaskManagerWebCoreScriptsCache))
-    wk_scripts_cache = GetModelText(
-        row, IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN);
+  if (TreeViewColumnIsVisible(treeview_, kTaskManagerWebCoreScriptsCache)) {
+    wk_scripts_cache =
+        GetModelText(row, IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN);
+  }
   std::string wk_css_cache;
-  if (TreeViewColumnIsVisible(treeview_, kTaskManagerWebCoreCssCache))
-    wk_css_cache = GetModelText(
-        row, IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN);
+  if (TreeViewColumnIsVisible(treeview_, kTaskManagerWebCoreCssCache)) {
+    wk_css_cache =
+        GetModelText(row, IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN);
+  }
+  std::string fps;
+  if (TreeViewColumnIsVisible(treeview_, kTaskManagerFPS))
+    fps = GetModelText(row, IDS_TASK_MANAGER_FPS_COLUMN);
   std::string sqlite_memory;
-  if (TreeViewColumnIsVisible(treeview_, kTaskManagerSqliteMemoryUsed))
-    sqlite_memory = GetModelText(
-        row, IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN);
+  if (TreeViewColumnIsVisible(treeview_, kTaskManagerSqliteMemoryUsed)) {
+    sqlite_memory =
+        GetModelText(row, IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN);
+  }
 
-  std::string goats = GetModelText(
-      row, IDS_TASK_MANAGER_GOATS_TELEPORTED_COLUMN);
+  std::string goats =
+      GetModelText(row, IDS_TASK_MANAGER_GOATS_TELEPORTED_COLUMN);
+
+  bool is_background = model_->IsBackgroundResource(row) &&
+      highlight_background_resources_;
   gtk_list_store_set(process_list_, iter,
                      kTaskManagerIcon, icon,
-                     kTaskManagerPage, page.c_str(),
+                     kTaskManagerPage, page_markup,
+                     kTaskManagerProfileName, profile_name.c_str(),
                      kTaskManagerSharedMem, shared_mem.c_str(),
                      kTaskManagerPrivateMem, priv_mem.c_str(),
                      kTaskManagerCPU, cpu.c_str(),
@@ -738,10 +830,14 @@ void TaskManagerGtk::SetRowDataFromModel(int row, GtkTreeIter* iter) {
                      kTaskManagerWebCoreImageCache, wk_img_cache.c_str(),
                      kTaskManagerWebCoreScriptsCache, wk_scripts_cache.c_str(),
                      kTaskManagerWebCoreCssCache, wk_css_cache.c_str(),
+                     kTaskManagerFPS, fps.c_str(),
                      kTaskManagerSqliteMemoryUsed, sqlite_memory.c_str(),
                      kTaskManagerGoatsTeleported, goats.c_str(),
+                     kTaskManagerBackgroundColor,
+                     is_background ? &kHighlightColor : NULL,
                      -1);
   g_object_unref(icon);
+  g_free(page_markup);
 }
 
 void TaskManagerGtk::KillSelectedProcesses() {
@@ -762,21 +858,13 @@ void TaskManagerGtk::KillSelectedProcesses() {
   g_list_free(paths);
 }
 
-#if defined(TOOLKIT_VIEWS)
-void TaskManagerGtk::ShowContextMenu(const gfx::Point& point) {
+void TaskManagerGtk::ShowContextMenu(const gfx::Point& point,
+                                     guint32 event_time) {
   if (!menu_controller_.get())
     menu_controller_.reset(new ContextMenuController(this));
 
-  menu_controller_->RunMenu(point);
+  menu_controller_->RunMenu(point, event_time);
 }
-#else
-void TaskManagerGtk::ShowContextMenu() {
-  if (!menu_controller_.get())
-    menu_controller_.reset(new ContextMenuController(this));
-
-  menu_controller_->RunMenu();
-}
-#endif
 
 void TaskManagerGtk::OnLinkActivated() {
   task_manager_->OpenAboutMemory();
@@ -821,15 +909,15 @@ void TaskManagerGtk::OnDestroy(GtkWidget* dialog) {
   delete this;
 }
 
-void TaskManagerGtk::OnResponse(GtkWidget* dialog, gint response_id) {
+void TaskManagerGtk::OnResponse(GtkWidget* dialog, int response_id) {
   if (response_id == GTK_RESPONSE_DELETE_EVENT) {
     // Store the dialog's size so we can restore it the next time it's opened.
     if (g_browser_process->local_state()) {
       gfx::Rect dialog_bounds = gtk_util::GetDialogBounds(GTK_WIDGET(dialog));
 
-      DictionaryValue* placement_pref =
-          g_browser_process->local_state()->GetMutableDictionary(
-              prefs::kTaskManagerWindowPlacement);
+      DictionaryPrefUpdate update(g_browser_process->local_state(),
+                                  prefs::kTaskManagerWindowPlacement);
+      DictionaryValue* placement_pref = update.Get();
       // Note that we store left/top for consistency with Windows, but that we
       // *don't* restore them.
       placement_pref->SetInteger("left", dialog_bounds.x());
@@ -931,15 +1019,14 @@ void TaskManagerGtk::OnRowActivated(GtkWidget* widget,
   task_manager_->ActivateProcess(row);
 }
 
-gboolean TaskManagerGtk::OnButtonReleaseEvent(GtkWidget* widget,
-                                              GdkEventButton* event) {
+gboolean TaskManagerGtk::OnButtonEvent(GtkWidget* widget,
+                                       GdkEventButton* event) {
+  // GTK does menu on mouse-up while views does menu on mouse-down,
+  // so this function can be called from either signal.
   if (event->button == 3) {
-#if defined(TOOLKIT_VIEWS)
-    gfx::Point pt(event->x_root, event->y_root);
-    ShowContextMenu(pt);
-#else
-    ShowContextMenu();
-#endif
+    ShowContextMenu(gfx::Point(event->x_root, event->y_root),
+                    event->time);
+    return TRUE;
   }
 
   return FALSE;
