@@ -7,7 +7,7 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "ui/base/gtk/gtk_integers.h"
+#include "ui/base/glib/glib_integers.h"
 #include "ui/base/gtk/gtk_signal.h"
 
 typedef struct _GtkWidget GtkWidget;
@@ -18,11 +18,11 @@ class UpdateRecommendedDialog {
   static void Show(GtkWindow* parent);
 
  private:
-  CHROMEGTK_CALLBACK_1(UpdateRecommendedDialog, void, OnResponse, gint);
-
   explicit UpdateRecommendedDialog(GtkWindow* parent);
   ~UpdateRecommendedDialog();
 
+  CHROMEGTK_CALLBACK_1(UpdateRecommendedDialog, void, OnResponse, int);
+
   GtkWidget* dialog_;
 
   DISALLOW_COPY_AND_ASSIGN(UpdateRecommendedDialog);
