@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 #define CHROME_BROWSER_UI_LOGIN_LOGIN_MODEL_H_
 #pragma once
 
-#include <string>
+#include "base/string16.h"
 
 // Simple Model & Observer interfaces for a LoginView to facilitate exchanging
 // information.
@@ -14,8 +14,8 @@ class LoginModelObserver {
  public:
   // Called by the model when a username,password pair has been identified
   // as a match for the pending login prompt.
-  virtual void OnAutofillDataAvailable(const std::wstring& username,
-                                       const std::wstring& password) = 0;
+  virtual void OnAutofillDataAvailable(const string16& username,
+                                       const string16& password) = 0;
 
  protected:
   virtual ~LoginModelObserver() {}
