@@ -1,23 +1,66 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/toolbar/back_forward_menu_model.h"
 
 #include "base/path_service.h"
+#include "base/string16.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/history/history.h"
 #include "chrome/browser/profiles/profile_manager.h"
-#include "chrome/browser/renderer_host/test/test_render_view_host.h"
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/navigation_entry.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/tab_contents/test_tab_contents.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/common/url_constants.h"
+#include "chrome/test/base/chrome_render_view_host_test_harness.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/browser/tab_contents/test_tab_contents.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
+#include "content/test/test_browser_thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/skia/include/core/SkBitmap.h"
+#include "ui/gfx/codec/png_codec.h"
 
-class BackFwdMenuModelTest : public RenderViewHostTestHarness {
+using content::BrowserThread;
+
+namespace {
+
+// Creates a bitmap of the specified color.
+SkBitmap CreateBitmap(SkColor color) {
+  SkBitmap bitmap;
+  bitmap.setConfig(SkBitmap::kARGB_8888_Config, 16, 16);
+  bitmap.allocPixels();
+  bitmap.eraseColor(color);
+  return bitmap;
+}
+
+class FaviconDelegate : public ui::MenuModelDelegate {
  public:
+  FaviconDelegate() : was_called_(false) {}
+
+  void OnIconChanged(int model_index) {
+    was_called_ = true;
+    MessageLoop::current()->Quit();
+  }
+
+  bool was_called() const { return was_called_; }
+
+ private:
+  bool was_called_;
+
+  DISALLOW_COPY_AND_ASSIGN(FaviconDelegate);
+};
+
+}  // namespace
+
+class BackFwdMenuModelTest : public ChromeRenderViewHostTestHarness {
+ public:
+  BackFwdMenuModelTest()
+      : ui_thread_(BrowserThread::UI, &message_loop_) {
+  }
+
   void ValidateModel(BackForwardMenuModel* model, int history_items,
                      int chapter_stops) {
     int h = std::min(BackForwardMenuModel::kMaxHistoryItems, history_items);
@@ -33,7 +76,7 @@ class BackFwdMenuModelTest : public RenderViewHostTestHarness {
 
   void LoadURLAndUpdateState(const char* url, const char* title) {
     NavigateAndCommit(GURL(url));
-    controller().GetLastCommittedEntry()->set_title(UTF8ToUTF16(title));
+    controller().GetLastCommittedEntry()->SetTitle(UTF8ToUTF16(title));
   }
 
   // Navigate back or forward the given amount and commits the entry (which
@@ -58,16 +101,18 @@ class BackFwdMenuModelTest : public RenderViewHostTestHarness {
     controller().GoForward();
     contents()->CommitPendingNavigation();
   }
+
+  content::TestBrowserThread ui_thread_;
 };
 
 TEST_F(BackFwdMenuModelTest, BasicCase) {
   scoped_ptr<BackForwardMenuModel> back_model(new BackForwardMenuModel(
       NULL, BackForwardMenuModel::BACKWARD_MENU));
-  back_model->set_test_tab_contents(contents());
+  back_model->set_test_web_contents(contents());
 
   scoped_ptr<BackForwardMenuModel> forward_model(new BackForwardMenuModel(
       NULL, BackForwardMenuModel::FORWARD_MENU));
-  forward_model->set_test_tab_contents(contents());
+  forward_model->set_test_web_contents(contents());
 
   EXPECT_EQ(0, back_model->GetItemCount());
   EXPECT_EQ(0, forward_model->GetItemCount());
@@ -131,11 +176,11 @@ TEST_F(BackFwdMenuModelTest, BasicCase) {
 TEST_F(BackFwdMenuModelTest, MaxItemsTest) {
   scoped_ptr<BackForwardMenuModel> back_model(new BackForwardMenuModel(
       NULL, BackForwardMenuModel::BACKWARD_MENU));
-  back_model->set_test_tab_contents(contents());
+  back_model->set_test_web_contents(contents());
 
   scoped_ptr<BackForwardMenuModel> forward_model(new BackForwardMenuModel(
       NULL, BackForwardMenuModel::FORWARD_MENU));
-  forward_model->set_test_tab_contents(contents());
+  forward_model->set_test_web_contents(contents());
 
   // Seed the controller with 32 URLs
   LoadURLAndUpdateState("http://www.a.com/1", "A1");
@@ -213,11 +258,11 @@ TEST_F(BackFwdMenuModelTest, MaxItemsTest) {
 TEST_F(BackFwdMenuModelTest, ChapterStops) {
   scoped_ptr<BackForwardMenuModel> back_model(new BackForwardMenuModel(
     NULL, BackForwardMenuModel::BACKWARD_MENU));
-  back_model->set_test_tab_contents(contents());
+  back_model->set_test_web_contents(contents());
 
   scoped_ptr<BackForwardMenuModel> forward_model(new BackForwardMenuModel(
       NULL, BackForwardMenuModel::FORWARD_MENU));
-  forward_model->set_test_tab_contents(contents());
+  forward_model->set_test_web_contents(contents());
 
   // Seed the controller with 32 URLs.
   int i = 0;
@@ -301,7 +346,7 @@ TEST_F(BackFwdMenuModelTest, ChapterStops) {
   // Check to see if the chapter stops have the right labels.
   int index = BackForwardMenuModel::kMaxHistoryItems;
   // Empty string indicates item is a separator.
-  EXPECT_EQ(ASCIIToUTF16(""), back_model->GetLabelAt(index++));
+  EXPECT_EQ(string16(), back_model->GetLabelAt(index++));
   EXPECT_EQ(ASCIIToUTF16("F3"), back_model->GetLabelAt(index++));
   EXPECT_EQ(ASCIIToUTF16("E3"), back_model->GetLabelAt(index++));
   EXPECT_EQ(ASCIIToUTF16("D3"), back_model->GetLabelAt(index++));
@@ -309,7 +354,7 @@ TEST_F(BackFwdMenuModelTest, ChapterStops) {
   // The menu should only show a maximum of 5 chapter stops.
   EXPECT_EQ(ASCIIToUTF16("B3"), back_model->GetLabelAt(index));
   // Empty string indicates item is a separator.
-  EXPECT_EQ(ASCIIToUTF16(""), back_model->GetLabelAt(index + 1));
+  EXPECT_EQ(string16(), back_model->GetLabelAt(index + 1));
   EXPECT_EQ(back_model->GetShowFullHistoryLabel(),
             back_model->GetLabelAt(index + 2));
 
@@ -327,7 +372,7 @@ TEST_F(BackFwdMenuModelTest, ChapterStops) {
   EXPECT_EQ(ASCIIToUTF16("A3"), back_model->GetLabelAt(index));
   GoBack();
   // It is now a separator.
-  EXPECT_EQ(ASCIIToUTF16(""), back_model->GetLabelAt(index));
+  EXPECT_EQ(string16(), back_model->GetLabelAt(index));
   // Undo our position change.
   NavigateToOffset(6);
 
@@ -350,7 +395,7 @@ TEST_F(BackFwdMenuModelTest, ChapterStops) {
   // Check to see if the chapter stops have the right labels.
   index = BackForwardMenuModel::kMaxHistoryItems;
   // Empty string indicates item is a separator.
-  EXPECT_EQ(ASCIIToUTF16(""), forward_model->GetLabelAt(index++));
+  EXPECT_EQ(string16(), forward_model->GetLabelAt(index++));
   EXPECT_EQ(ASCIIToUTF16("E3"), forward_model->GetLabelAt(index++));
   EXPECT_EQ(ASCIIToUTF16("F3"), forward_model->GetLabelAt(index++));
   EXPECT_EQ(ASCIIToUTF16("G3"), forward_model->GetLabelAt(index++));
@@ -358,7 +403,7 @@ TEST_F(BackFwdMenuModelTest, ChapterStops) {
   // The menu should only show a maximum of 5 chapter stops.
   EXPECT_EQ(ASCIIToUTF16("I3"), forward_model->GetLabelAt(index));
   // Empty string indicates item is a separator.
-  EXPECT_EQ(ASCIIToUTF16(""), forward_model->GetLabelAt(index + 1));
+  EXPECT_EQ(string16(), forward_model->GetLabelAt(index + 1));
   EXPECT_EQ(forward_model->GetShowFullHistoryLabel(),
       forward_model->GetLabelAt(index + 2));
 
@@ -420,3 +465,96 @@ TEST_F(BackFwdMenuModelTest, ChapterStops) {
   EXPECT_EQ(1, back_model->GetIndexOfNextChapterStop(2, false));
   EXPECT_EQ(-1, back_model->GetIndexOfNextChapterStop(1, false));
 }
+
+TEST_F(BackFwdMenuModelTest, EscapeLabel) {
+  scoped_ptr<BackForwardMenuModel> back_model(new BackForwardMenuModel(
+      NULL, BackForwardMenuModel::BACKWARD_MENU));
+  back_model->set_test_web_contents(contents());
+
+  EXPECT_EQ(0, back_model->GetItemCount());
+  EXPECT_FALSE(back_model->ItemHasCommand(1));
+
+  LoadURLAndUpdateState("http://www.a.com/1", "A B");
+  LoadURLAndUpdateState("http://www.a.com/1", "A & B");
+  LoadURLAndUpdateState("http://www.a.com/2", "A && B");
+  LoadURLAndUpdateState("http://www.a.com/2", "A &&& B");
+  LoadURLAndUpdateState("http://www.a.com/3", "");
+
+  EXPECT_EQ(6, back_model->GetItemCount());
+
+  // On Mac ui::MenuModel::GetLabelAt should return unescaped strings.
+#if defined(OS_MACOSX)
+  EXPECT_EQ(ASCIIToUTF16("A B"), back_model->GetLabelAt(3));
+  EXPECT_EQ(ASCIIToUTF16("A & B"), back_model->GetLabelAt(2));
+  EXPECT_EQ(ASCIIToUTF16("A && B"), back_model->GetLabelAt(1));
+  EXPECT_EQ(ASCIIToUTF16("A &&& B"), back_model->GetLabelAt(0));
+#else
+  EXPECT_EQ(ASCIIToUTF16("A B"), back_model->GetLabelAt(3));
+  EXPECT_EQ(ASCIIToUTF16("A && B"), back_model->GetLabelAt(2));
+  EXPECT_EQ(ASCIIToUTF16("A &&&& B"), back_model->GetLabelAt(1));
+  EXPECT_EQ(ASCIIToUTF16("A &&&&&& B"), back_model->GetLabelAt(0));
+#endif // defined(OS_MACOSX)
+}
+
+// Test asynchronous loading of favicon from history service.
+TEST_F(BackFwdMenuModelTest, FaviconLoadTest) {
+  profile()->CreateHistoryService(true, false);
+  profile()->CreateFaviconService();
+  Browser browser(Browser::TYPE_TABBED, profile());
+  FaviconDelegate favicon_delegate;
+
+  BackForwardMenuModel back_model(
+      &browser, BackForwardMenuModel::BACKWARD_MENU);
+  back_model.set_test_web_contents(controller().GetWebContents());
+  back_model.SetMenuModelDelegate(&favicon_delegate);
+
+  SkBitmap new_icon(CreateBitmap(SK_ColorRED));
+  std::vector<unsigned char> icon_data;
+  gfx::PNGCodec::EncodeBGRASkBitmap(new_icon, false, &icon_data);
+
+  GURL url1 = GURL("http://www.a.com/1");
+  GURL url2 = GURL("http://www.a.com/2");
+  GURL url1_favicon("http://www.a.com/1/favicon.ico");
+
+  NavigateAndCommit(url1);
+  // Navigate to a new URL so that url1 will be in the BackForwardMenuModel.
+  NavigateAndCommit(url2);
+
+  // Set the desired favicon for url1.
+  profile()->GetHistoryService(Profile::EXPLICIT_ACCESS)->AddPage(url1,
+      history::SOURCE_BROWSED);
+  profile()->GetFaviconService(Profile::EXPLICIT_ACCESS)->SetFavicon(url1,
+      url1_favicon, icon_data, history::FAVICON);
+
+  // Will return the current icon (default) but start an anync call
+  // to retrieve the favicon from the favicon service.
+  SkBitmap default_icon;
+  back_model.GetIconAt(0, &default_icon);
+
+  // Make the favicon service run GetFavIconForURL,
+  // FaviconDelegate.OnIconChanged will be called.
+  MessageLoop::current()->Run();
+
+  // Verify that the callback executed.
+  EXPECT_TRUE(favicon_delegate.was_called());
+
+  // Verify the bitmaps match.
+  SkBitmap valid_icon;
+  // This time we will get the new favicon returned.
+  back_model.GetIconAt(0, &valid_icon);
+  SkAutoLockPixels a(new_icon);
+  SkAutoLockPixels b(valid_icon);
+  SkAutoLockPixels c(default_icon);
+  // Verify we did not get the default favicon.
+  EXPECT_NE(0, memcmp(default_icon.getPixels(), valid_icon.getPixels(),
+               default_icon.getSize()));
+  // Verify we did get the expected favicon.
+  EXPECT_EQ(0, memcmp(new_icon.getPixels(), valid_icon.getPixels(),
+              new_icon.getSize()));
+
+  // Make sure the browser deconstructor doesn't have problems.
+  browser.CloseAllTabs();
+  // This is required to prevent the message loop from hanging.
+  profile()->DestroyHistoryService();
+}
+
