@@ -4,47 +4,52 @@
 
 #include "chrome/browser/ui/views/browser_actions_container.h"
 
-#include "base/stl_util-inl.h"
+#include "base/bind.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/browser_window.h"
 #include "chrome/browser/extensions/extension_browser_event_router.h"
 #include "chrome/browser/extensions/extension_host.h"
-#include "chrome/browser/extensions/extension_tabs_module.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/extensions/extension_tabs_module.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/renderer_host/render_view_host.h"
-#include "chrome/browser/renderer_host/render_widget_host_view.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
+#include "chrome/browser/sessions/restore_tab_helper.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/browser/ui/view_ids.h"
 #include "chrome/browser/ui/views/detachable_toolbar_view.h"
 #include "chrome/browser/ui/views/extensions/browser_action_drag_data.h"
 #include "chrome/browser/ui/views/extensions/extension_popup.h"
 #include "chrome/browser/ui/views/toolbar_view.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/extensions/extension_action.h"
 #include "chrome/common/extensions/extension_resource.h"
-#include "chrome/common/notification_source.h"
-#include "chrome/common/notification_type.h"
 #include "chrome/common/pref_names.h"
-#include "gfx/canvas.h"
-#include "gfx/canvas_skia.h"
-#include "grit/app_resources.h"
+#include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/renderer_host/render_widget_host_view.h"
+#include "content/public/browser/notification_source.h"
 #include "grit/generated_resources.h"
+#include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
 #include "third_party/skia/include/core/SkBitmap.h"
-#include "third_party/skia/include/core/SkTypeface.h"
 #include "third_party/skia/include/effects/SkGradientShader.h"
+#include "ui/base/accessibility/accessible_view_state.h"
 #include "ui/base/animation/slide_animation.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/controls/button/menu_button.h"
-#include "views/controls/button/text_button.h"
-#include "views/controls/menu/menu_2.h"
-#include "views/drag_utils.h"
-#include "views/window/window.h"
-
-#include "grit/theme_resources.h"
+#include "ui/base/theme_provider.h"
+#include "ui/gfx/canvas.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/views/controls/button/menu_button.h"
+#include "ui/views/controls/button/text_button.h"
+#include "ui/views/controls/menu/menu_item_view.h"
+#include "ui/views/controls/menu/menu_model_adapter.h"
+#include "ui/views/controls/menu/menu_runner.h"
+#include "ui/views/controls/resize_area.h"
+#include "ui/views/drag_utils.h"
+#include "ui/views/metrics.h"
 
 // Horizontal spacing between most items in the container, as well as after the
 // last item or chevron (if visible).
@@ -61,25 +66,25 @@ bool BrowserActionsContainer::disable_animations_during_testing_ = false;
 BrowserActionButton::BrowserActionButton(const Extension* extension,
                                          BrowserActionsContainer* panel)
     : ALLOW_THIS_IN_INITIALIZER_LIST(
-          MenuButton(this, std::wstring(), NULL, false)),
+          MenuButton(this, string16(), NULL, false)),
       browser_action_(extension->browser_action()),
       extension_(extension),
       ALLOW_THIS_IN_INITIALIZER_LIST(tracker_(this)),
-      showing_context_menu_(false),
-      panel_(panel) {
+      panel_(panel),
+      context_menu_(NULL) {
   set_border(NULL);
   set_alignment(TextButton::ALIGN_CENTER);
 
   // No UpdateState() here because View hierarchy not setup yet. Our parent
   // should call UpdateState() after creation.
 
-  registrar_.Add(this, NotificationType::EXTENSION_BROWSER_ACTION_UPDATED,
-                 Source<ExtensionAction>(browser_action_));
+  registrar_.Add(this, chrome::NOTIFICATION_EXTENSION_BROWSER_ACTION_UPDATED,
+                 content::Source<ExtensionAction>(browser_action_));
 }
 
 void BrowserActionButton::Destroy() {
-  if (showing_context_menu_) {
-    context_menu_menu_->CancelMenu();
+  if (context_menu_) {
+    context_menu_->Cancel();
     MessageLoop::current()->DeleteSoon(FROM_HERE, this);
   } else {
     delete this;
@@ -97,7 +102,7 @@ void BrowserActionButton::ViewHierarchyChanged(
 
     // LoadImage is not guaranteed to be synchronous, so we might see the
     // callback OnImageLoaded execute immediately. It (through UpdateState)
-    // expects GetParent() to return the owner for this button, so this
+    // expects parent() to return the owner for this button, so this
     // function is as early as we can start this request.
     tracker_.LoadImage(extension_, extension_->GetResource(relative_path),
                        gfx::Size(Extension::kBrowserActionIconMaxSize,
@@ -114,7 +119,7 @@ void BrowserActionButton::ButtonPressed(views::Button* sender,
 }
 
 void BrowserActionButton::OnImageLoaded(SkBitmap* image,
-                                        ExtensionResource resource,
+                                        const ExtensionResource& resource,
                                         int index) {
   if (image)
     default_icon_ = *image;
@@ -168,19 +173,9 @@ void BrowserActionButton::UpdateState() {
   string16 name = UTF8ToUTF16(browser_action()->GetTitle(tab_id));
   if (name.empty())
     name = UTF8ToUTF16(extension()->name());
-  SetTooltipText(UTF16ToWideHack(name));
+  SetTooltipText(name);
   SetAccessibleName(name);
-  GetParent()->SchedulePaint();
-}
-
-void BrowserActionButton::Observe(NotificationType type,
-                                  const NotificationSource& source,
-                                  const NotificationDetails& details) {
-  DCHECK(type == NotificationType::EXTENSION_BROWSER_ACTION_UPDATED);
-  UpdateState();
-  // The browser action may have become visible/hidden so we need to make
-  // sure the state gets updated.
-  panel_->OnBrowserActionVisibilityChanged();
+  parent()->SchedulePaint();
 }
 
 bool BrowserActionButton::IsPopup() {
@@ -193,6 +188,16 @@ GURL BrowserActionButton::GetPopupUrl() {
   return (tab_id < 0) ? GURL() : browser_action_->GetPopupUrl(tab_id);
 }
 
+void BrowserActionButton::Observe(int type,
+                                  const content::NotificationSource& source,
+                                  const content::NotificationDetails& details) {
+  DCHECK(type == chrome::NOTIFICATION_EXTENSION_BROWSER_ACTION_UPDATED);
+  UpdateState();
+  // The browser action may have become visible/hidden so we need to make
+  // sure the state gets updated.
+  panel_->OnBrowserActionVisibilityChanged();
+}
+
 bool BrowserActionButton::Activate() {
   if (!IsPopup())
     return true;
@@ -209,44 +214,36 @@ bool BrowserActionButton::Activate() {
   return false;
 }
 
-bool BrowserActionButton::OnMousePressed(const views::MouseEvent& e) {
-  if (!e.IsRightMouseButton()) {
-    return IsPopup() ?
-        MenuButton::OnMousePressed(e) : TextButton::OnMousePressed(e);
+bool BrowserActionButton::OnMousePressed(const views::MouseEvent& event) {
+  if (!event.IsRightMouseButton()) {
+    return IsPopup() ? MenuButton::OnMousePressed(event)
+                     : TextButton::OnMousePressed(event);
   }
 
-  // Get the top left point of this button in screen coordinates.
-  gfx::Point point = gfx::Point(0, 0);
-  ConvertPointToScreen(this, &point);
-
-  // Make the menu appear below the button.
-  point.Offset(0, height());
-
-  ShowContextMenu(point, true);
+  ShowContextMenu(gfx::Point(), true);
   return false;
 }
 
-void BrowserActionButton::OnMouseReleased(const views::MouseEvent& e,
-                                          bool canceled) {
-  if (IsPopup() || showing_context_menu_) {
+void BrowserActionButton::OnMouseReleased(const views::MouseEvent& event) {
+  if (IsPopup() || context_menu_) {
     // TODO(erikkay) this never actually gets called (probably because of the
     // loss of focus).
-    MenuButton::OnMouseReleased(e, canceled);
+    MenuButton::OnMouseReleased(event);
   } else {
-    TextButton::OnMouseReleased(e, canceled);
+    TextButton::OnMouseReleased(event);
   }
 }
 
-bool BrowserActionButton::OnKeyReleased(const views::KeyEvent& e) {
-  return IsPopup() ?
-      MenuButton::OnKeyReleased(e) : TextButton::OnKeyReleased(e);
+void BrowserActionButton::OnMouseExited(const views::MouseEvent& event) {
+  if (IsPopup() || context_menu_)
+    MenuButton::OnMouseExited(event);
+  else
+    TextButton::OnMouseExited(event);
 }
 
-void BrowserActionButton::OnMouseExited(const views::MouseEvent& e) {
-  if (IsPopup() || showing_context_menu_)
-    MenuButton::OnMouseExited(e);
-  else
-    TextButton::OnMouseExited(e);
+bool BrowserActionButton::OnKeyReleased(const views::KeyEvent& event) {
+  return IsPopup() ? MenuButton::OnKeyReleased(event)
+                   : TextButton::OnKeyReleased(event);
 }
 
 void BrowserActionButton::ShowContextMenu(const gfx::Point& p,
@@ -254,17 +251,24 @@ void BrowserActionButton::ShowContextMenu(const gfx::Point& p,
   if (!extension()->ShowConfigureContextMenus())
     return;
 
-  showing_context_menu_ = true;
   SetButtonPushed();
 
   // Reconstructs the menu every time because the menu's contents are dynamic.
-  context_menu_contents_ =
-      new ExtensionContextMenuModel(extension(), panel_->browser(), panel_);
-  context_menu_menu_.reset(new views::Menu2(context_menu_contents_.get()));
-  context_menu_menu_->RunContextMenuAt(p);
+  scoped_refptr<ExtensionContextMenuModel> context_menu_contents_(
+      new ExtensionContextMenuModel(extension(), panel_->browser(), panel_));
+  views::MenuModelAdapter menu_model_adapter(context_menu_contents_.get());
+  views::MenuRunner menu_runner(menu_model_adapter.CreateMenu());
+
+  context_menu_ = menu_runner.GetMenu();
+  gfx::Point screen_loc;
+  views::View::ConvertPointToScreen(this, &screen_loc);
+  if (menu_runner.RunMenuAt(GetWidget(), NULL, gfx::Rect(screen_loc, size()),
+          views::MenuItemView::TOPLEFT, views::MenuRunner::HAS_MNEMONICS) ==
+      views::MenuRunner::MENU_DELETED)
+    return;
 
   SetButtonNotPushed();
-  showing_context_menu_ = false;
+  context_menu_ = NULL;
 }
 
 void BrowserActionButton::SetButtonPushed() {
@@ -288,11 +292,9 @@ BrowserActionView::BrowserActionView(const Extension* extension,
                                      BrowserActionsContainer* panel)
     : panel_(panel) {
   button_ = new BrowserActionButton(extension, panel);
-  button_->SetDragController(panel_);
+  button_->set_drag_controller(panel_);
   AddChildView(button_);
   button_->UpdateState();
-  SetAccessibleName(
-      l10n_util::GetStringUTF16(IDS_ACCNAME_EXTENSIONS_BROWSER_ACTION));
 }
 
 BrowserActionView::~BrowserActionView() {
@@ -307,8 +309,7 @@ gfx::Canvas* BrowserActionView::GetIconWithBadge() {
   if (icon.isNull())
     icon = button_->default_icon();
 
-  gfx::Canvas* canvas = new gfx::CanvasSkia(icon.width(), icon.height(), false);
-  canvas->DrawBitmapInt(icon, 0, 0);
+  gfx::Canvas* canvas = new gfx::CanvasSkia(icon, false);
 
   if (tab_id >= 0) {
     gfx::Rect bounds(icon.width(), icon.height() + ToolbarView::kVertSpacing);
@@ -318,10 +319,6 @@ gfx::Canvas* BrowserActionView::GetIconWithBadge() {
   return canvas;
 }
 
-AccessibilityTypes::Role BrowserActionView::GetAccessibleRole() {
-  return AccessibilityTypes::ROLE_GROUPING;
-}
-
 void BrowserActionView::Layout() {
   // We can't rely on button_->GetPreferredSize() here because that's not set
   // correctly until the first call to
@@ -334,6 +331,12 @@ void BrowserActionView::Layout() {
                      BrowserActionsContainer::IconHeight());
 }
 
+void BrowserActionView::GetAccessibleState(ui::AccessibleViewState* state) {
+  state->name = l10n_util::GetStringUTF16(
+      IDS_ACCNAME_EXTENSIONS_BROWSER_ACTION);
+  state->role = ui::AccessibilityTypes::ROLE_GROUPING;
+}
+
 void BrowserActionView::PaintChildren(gfx::Canvas* canvas) {
   View::PaintChildren(canvas);
   ExtensionAction* action = button()->browser_action();
@@ -362,7 +365,7 @@ BrowserActionsContainer::BrowserActionsContainer(Browser* browser,
       drop_indicator_position_(-1),
       ALLOW_THIS_IN_INITIALIZER_LIST(task_factory_(this)),
       ALLOW_THIS_IN_INITIALIZER_LIST(show_menu_task_factory_(this)) {
-  SetID(VIEW_ID_BROWSER_ACTION_TOOLBAR);
+  set_id(VIEW_ID_BROWSER_ACTION_TOOLBAR);
 
   if (profile_->GetExtensionService()) {
     model_ = profile_->GetExtensionService()->toolbar_model();
@@ -371,32 +374,32 @@ BrowserActionsContainer::BrowserActionsContainer(Browser* browser,
 
   resize_animation_.reset(new ui::SlideAnimation(this));
   resize_area_ = new views::ResizeArea(this);
-  resize_area_->SetAccessibleName(
-      l10n_util::GetStringUTF16(IDS_ACCNAME_SEPARATOR));
   AddChildView(resize_area_);
 
-  chevron_ = new views::MenuButton(NULL, std::wstring(), this, false);
+  chevron_ = new views::MenuButton(NULL, string16(), this, false);
   chevron_->set_border(NULL);
   chevron_->EnableCanvasFlippingForRTLUI(true);
   chevron_->SetAccessibleName(
       l10n_util::GetStringUTF16(IDS_ACCNAME_EXTENSIONS_CHEVRON));
   chevron_->SetVisible(false);
   AddChildView(chevron_);
-
-  SetAccessibleName(l10n_util::GetStringUTF16(IDS_ACCNAME_EXTENSIONS));
 }
 
 BrowserActionsContainer::~BrowserActionsContainer() {
   if (model_)
     model_->RemoveObserver(this);
   StopShowFolderDropMenuTimer();
+  if (popup_)
+    popup_->GetWidget()->RemoveObserver(this);
   HidePopup();
   DeleteBrowserActionViews();
 }
 
 // Static.
 void BrowserActionsContainer::RegisterUserPrefs(PrefService* prefs) {
-  prefs->RegisterIntegerPref(prefs::kBrowserActionContainerWidth, 0);
+  prefs->RegisterIntegerPref(prefs::kBrowserActionContainerWidth,
+                             0,
+                             PrefService::UNSYNCABLE_PREF);
 }
 
 void BrowserActionsContainer::Init() {
@@ -418,8 +421,8 @@ void BrowserActionsContainer::Init() {
 }
 
 int BrowserActionsContainer::GetCurrentTabId() const {
-  TabContents* tab_contents = browser_->GetSelectedTabContents();
-  return tab_contents ? tab_contents->controller().session_id().id() : -1;
+  TabContentsWrapper* tab = browser_->GetSelectedTabContentsWrapper();
+  return tab ? tab->restore_tab_helper()->session_id().id() : -1;
 }
 
 BrowserActionView* BrowserActionsContainer::GetBrowserActionView(
@@ -454,13 +457,8 @@ void BrowserActionsContainer::CreateBrowserActionViews() {
 }
 
 void BrowserActionsContainer::DeleteBrowserActionViews() {
-  if (!browser_action_views_.empty()) {
-    for (size_t i = 0; i < browser_action_views_.size(); ++i)
-      RemoveChildView(browser_action_views_[i]);
-    STLDeleteContainerPointers(browser_action_views_.begin(),
-                               browser_action_views_.end());
-    browser_action_views_.clear();
-  }
+  HidePopup();
+  STLDeleteElements(&browser_action_views_);
 }
 
 void BrowserActionsContainer::OnBrowserActionVisibilityChanged() {
@@ -472,7 +470,7 @@ void BrowserActionsContainer::OnBrowserActionVisibilityChanged() {
 size_t BrowserActionsContainer::VisibleBrowserActions() const {
   size_t visible_actions = 0;
   for (size_t i = 0; i < browser_action_views_.size(); ++i) {
-    if (browser_action_views_[i]->IsVisible())
+    if (browser_action_views_[i]->visible())
       ++visible_actions;
   }
   return visible_actions;
@@ -505,19 +503,12 @@ void BrowserActionsContainer::OnBrowserActionExecuted(
   // We can get the execute event for browser actions that are not visible,
   // since buttons can be activated from the overflow menu (chevron). In that
   // case we show the popup as originating from the chevron.
-  View* reference_view = button->GetParent()->IsVisible() ? button : chevron_;
-  gfx::Point origin;
-  View::ConvertPointToScreen(reference_view, &origin);
-  gfx::Rect rect = reference_view->bounds();
-  rect.set_origin(origin);
-
-  gfx::NativeWindow frame_window = browser_->window()->GetNativeHandle();
-  BubbleBorder::ArrowLocation arrow_location = base::i18n::IsRTL() ?
-      BubbleBorder::TOP_LEFT : BubbleBorder::TOP_RIGHT;
-
-  popup_ = ExtensionPopup::Show(button->GetPopupUrl(), browser_,
-      browser_->profile(), frame_window, rect, arrow_location, true,
-      inspect_with_devtools, ExtensionPopup::BUBBLE_CHROME, this);
+  View* reference_view = button->parent()->visible() ? button : chevron_;
+  views::BubbleBorder::ArrowLocation arrow_location = base::i18n::IsRTL() ?
+      views::BubbleBorder::TOP_LEFT : views::BubbleBorder::TOP_RIGHT;
+  popup_ = ExtensionPopup::ShowPopup(button->GetPopupUrl(), browser_,
+               reference_view, arrow_location, inspect_with_devtools);
+  popup_->GetWidget()->AddObserver(this);
   popup_button_ = button;
   popup_button_->SetButtonPushed();
 }
@@ -576,40 +567,6 @@ void BrowserActionsContainer::Layout() {
   }
 }
 
-void BrowserActionsContainer::Paint(gfx::Canvas* canvas) {
-  // TODO(sky/glen): Instead of using a drop indicator, animate the icons while
-  // dragging (like we do for tab dragging).
-  if (drop_indicator_position_ > -1) {
-    // The two-pixel width drop indicator.
-    static const int kDropIndicatorWidth = 2;
-    gfx::Rect indicator_bounds(
-        drop_indicator_position_ - (kDropIndicatorWidth / 2),
-        ToolbarView::kVertSpacing, kDropIndicatorWidth, IconHeight());
-
-    // Color of the drop indicator.
-    static const SkColor kDropIndicatorColor = SK_ColorBLACK;
-    canvas->FillRectInt(kDropIndicatorColor, indicator_bounds.x(),
-                        indicator_bounds.y(), indicator_bounds.width(),
-                        indicator_bounds.height());
-  }
-}
-
-void BrowserActionsContainer::ViewHierarchyChanged(bool is_add,
-                                                   views::View* parent,
-                                                   views::View* child) {
-  // No extensions (e.g., incognito).
-  if (!model_)
-    return;
-
-  if (is_add && child == this) {
-    // Initial toolbar button creation and placement in the widget hierarchy.
-    // We do this here instead of in the constructor because AddBrowserAction
-    // calls Layout on the Toolbar, which needs this object to be constructed
-    // before its Layout function is called.
-    CreateBrowserActionViews();
-  }
-}
-
 bool BrowserActionsContainer::GetDropFormats(
     int* formats,
     std::set<OSExchangeData::CustomFormat>* custom_formats) {
@@ -634,8 +591,8 @@ void BrowserActionsContainer::OnDragEntered(
 int BrowserActionsContainer::OnDragUpdated(
     const views::DropTargetEvent& event) {
   // First check if we are above the chevron (overflow) menu.
-  if (GetViewForPoint(event.location()) == chevron_) {
-    if (show_menu_task_factory_.empty() && !overflow_menu_)
+  if (GetEventHandlerForPoint(event.location()) == chevron_) {
+    if (!show_menu_task_factory_.HasWeakPtrs() && !overflow_menu_)
       StartShowFolderDropMenuTimer();
     return ui::DragDropTypes::DRAG_MOVE;
   }
@@ -699,7 +656,7 @@ void BrowserActionsContainer::OnDragExited() {
 int BrowserActionsContainer::OnPerformDrop(
     const views::DropTargetEvent& event) {
   BrowserActionDragData data;
-  if (!data.Read(event.GetData()))
+  if (!data.Read(event.data()))
     return ui::DragDropTypes::DRAG_NONE;
 
   // Make sure we have the same view as we started with.
@@ -709,9 +666,8 @@ int BrowserActionsContainer::OnPerformDrop(
 
   size_t i = 0;
   for (; i < browser_action_views_.size(); ++i) {
-    int view_x =
-        browser_action_views_[i]->GetBounds(APPLY_MIRRORING_TRANSFORMATION).x();
-    if (!browser_action_views_[i]->IsVisible() ||
+    int view_x = browser_action_views_[i]->GetMirroredBounds().x();
+    if (!browser_action_views_[i]->visible() ||
         (base::i18n::IsRTL() ? (view_x < drop_indicator_position_) :
             (view_x >= drop_indicator_position_))) {
       // We have reached the end of the visible icons or found one that has a
@@ -738,12 +694,10 @@ int BrowserActionsContainer::OnPerformDrop(
   return ui::DragDropTypes::DRAG_MOVE;
 }
 
-void BrowserActionsContainer::OnThemeChanged() {
-  LoadImages();
-}
-
-AccessibilityTypes::Role BrowserActionsContainer::GetAccessibleRole() {
-  return AccessibilityTypes::ROLE_GROUPING;
+void BrowserActionsContainer::GetAccessibleState(
+    ui::AccessibleViewState* state) {
+  state->role = ui::AccessibilityTypes::ROLE_GROUPING;
+  state->name = l10n_util::GetStringUTF16(IDS_ACCNAME_EXTENSIONS);
 }
 
 void BrowserActionsContainer::RunMenu(View* source, const gfx::Point& pt) {
@@ -751,13 +705,13 @@ void BrowserActionsContainer::RunMenu(View* source, const gfx::Point& pt) {
     overflow_menu_ = new BrowserActionOverflowMenuController(
         this, chevron_, browser_action_views_, VisibleBrowserActions());
     overflow_menu_->set_observer(this);
-    overflow_menu_->RunMenu(GetWindow()->GetNativeWindow(), false);
+    overflow_menu_->RunMenu(GetWidget(), false);
   }
 }
 
-void BrowserActionsContainer::WriteDragData(View* sender,
-                                            const gfx::Point& press_pt,
-                                            OSExchangeData* data) {
+void BrowserActionsContainer::WriteDragDataForView(View* sender,
+                                                   const gfx::Point& press_pt,
+                                                   OSExchangeData* data) {
   DCHECK(data);
 
   for (size_t i = 0; i < browser_action_views_.size(); ++i) {
@@ -778,14 +732,14 @@ void BrowserActionsContainer::WriteDragData(View* sender,
   }
 }
 
-int BrowserActionsContainer::GetDragOperations(View* sender,
-                                               const gfx::Point& p) {
+int BrowserActionsContainer::GetDragOperationsForView(View* sender,
+                                                      const gfx::Point& p) {
   return ui::DragDropTypes::DRAG_MOVE;
 }
 
-bool BrowserActionsContainer::CanStartDrag(View* sender,
-                                           const gfx::Point& press_pt,
-                                           const gfx::Point& p) {
+bool BrowserActionsContainer::CanStartDragForView(View* sender,
+                                                  const gfx::Point& press_pt,
+                                                  const gfx::Point& p) {
   return true;
 }
 
@@ -832,12 +786,15 @@ void BrowserActionsContainer::InspectPopup(ExtensionAction* action) {
   OnBrowserActionExecuted(GetBrowserActionView(action)->button(), true);
 }
 
-void BrowserActionsContainer::ExtensionPopupIsClosing(ExtensionPopup* popup) {
-  // ExtensionPopup is ref-counted, so we don't need to delete it.
-  DCHECK_EQ(popup_, popup);
+void BrowserActionsContainer::OnWidgetClosing(views::Widget* widget) {
+  DCHECK_EQ(popup_->GetWidget(), widget);
+  popup_->GetWidget()->RemoveObserver(this);
   popup_ = NULL;
-  popup_button_->SetButtonNotPushed();
-  popup_button_ = NULL;
+  // |popup_button_| is NULL if the extension has been removed.
+  if (popup_button_) {
+    popup_button_->SetButtonNotPushed();
+    popup_button_ = NULL;
+  }
 }
 
 void BrowserActionsContainer::MoveBrowserAction(const std::string& extension_id,
@@ -851,8 +808,11 @@ void BrowserActionsContainer::MoveBrowserAction(const std::string& extension_id,
 }
 
 void BrowserActionsContainer::HidePopup() {
-  if (popup_)
-    popup_->Close();
+  if (popup_) {
+    popup_->GetWidget()->Close();
+    // NULL out popup_button_ in case it's being deleted.
+    popup_button_ = NULL;
+  }
 }
 
 void BrowserActionsContainer::TestExecuteBrowserAction(int index) {
@@ -863,11 +823,47 @@ void BrowserActionsContainer::TestExecuteBrowserAction(int index) {
 void BrowserActionsContainer::TestSetIconVisibilityCount(size_t icons) {
   model_->SetVisibleIconCount(icons);
   chevron_->SetVisible(icons < browser_action_views_.size());
-  container_width_ = IconCountToWidth(icons, chevron_->IsVisible());
+  container_width_ = IconCountToWidth(icons, chevron_->visible());
   Layout();
   SchedulePaint();
 }
 
+void BrowserActionsContainer::OnPaint(gfx::Canvas* canvas) {
+  // TODO(sky/glen): Instead of using a drop indicator, animate the icons while
+  // dragging (like we do for tab dragging).
+  if (drop_indicator_position_ > -1) {
+    // The two-pixel width drop indicator.
+    static const int kDropIndicatorWidth = 2;
+    gfx::Rect indicator_bounds(
+        drop_indicator_position_ - (kDropIndicatorWidth / 2),
+        ToolbarView::kVertSpacing, kDropIndicatorWidth, IconHeight());
+
+    // Color of the drop indicator.
+    static const SkColor kDropIndicatorColor = SK_ColorBLACK;
+    canvas->FillRect(kDropIndicatorColor, indicator_bounds);
+  }
+}
+
+void BrowserActionsContainer::OnThemeChanged() {
+  LoadImages();
+}
+
+void BrowserActionsContainer::ViewHierarchyChanged(bool is_add,
+                                                   views::View* parent,
+                                                   views::View* child) {
+  // No extensions (e.g., incognito).
+  if (!model_)
+    return;
+
+  if (is_add && child == this) {
+    // Initial toolbar button creation and placement in the widget hierarchy.
+    // We do this here instead of in the constructor because AddBrowserAction
+    // calls Layout on the Toolbar, which needs this object to be constructed
+    // before its Layout function is called.
+    CreateBrowserActionViews();
+  }
+}
+
 // static
 int BrowserActionsContainer::IconWidth(bool include_padding) {
   static bool initialized = false;
@@ -913,7 +909,7 @@ void BrowserActionsContainer::BrowserActionAdded(const Extension* extension,
     index = model_->OriginalIndexToIncognito(index);
   BrowserActionView* view = new BrowserActionView(extension, this);
   browser_action_views_.insert(browser_action_views_.begin() + index, view);
-  AddChildView(index, view);
+  AddChildViewAt(view, index);
 
   // If we are still initializing the container, don't bother animating.
   if (!model_->extensions_initialized())
@@ -941,7 +937,6 @@ void BrowserActionsContainer::BrowserActionRemoved(const Extension* extension) {
   for (BrowserActionViews::iterator iter = browser_action_views_.begin();
        iter != browser_action_views_.end(); ++iter) {
     if ((*iter)->button()->extension() == extension) {
-      RemoveChildView(*iter);
       delete *iter;
       browser_action_views_.erase(iter);
 
@@ -1000,7 +995,7 @@ void BrowserActionsContainer::SetContainerWidth() {
   if (visible_actions < 0)  // All icons should be visible.
     visible_actions = model_->size();
   chevron_->SetVisible(static_cast<size_t>(visible_actions) < model_->size());
-  container_width_ = IconCountToWidth(visible_actions, chevron_->IsVisible());
+  container_width_ = IconCountToWidth(visible_actions, chevron_->visible());
 }
 
 void BrowserActionsContainer::CloseOverflowMenu() {
@@ -1009,14 +1004,15 @@ void BrowserActionsContainer::CloseOverflowMenu() {
 }
 
 void BrowserActionsContainer::StopShowFolderDropMenuTimer() {
-  show_menu_task_factory_.RevokeAll();
+  show_menu_task_factory_.InvalidateWeakPtrs();
 }
 
 void BrowserActionsContainer::StartShowFolderDropMenuTimer() {
-  int delay = View::GetMenuShowDelay();
-  MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      show_menu_task_factory_.NewRunnableMethod(
-          &BrowserActionsContainer::ShowDropFolder),
+  int delay = views::GetMenuShowDelay();
+  MessageLoop::current()->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(&BrowserActionsContainer::ShowDropFolder,
+                 show_menu_task_factory_.GetWeakPtr()),
       delay);
 }
 
@@ -1026,7 +1022,7 @@ void BrowserActionsContainer::ShowDropFolder() {
   overflow_menu_ = new BrowserActionOverflowMenuController(
       this, chevron_, browser_action_views_, VisibleBrowserActions());
   overflow_menu_->set_observer(this);
-  overflow_menu_->RunMenu(GetWindow()->GetNativeWindow(), true);
+  overflow_menu_->RunMenu(GetWidget(), true);
 }
 
 void BrowserActionsContainer::SetDropIndicator(int x_pos) {
@@ -1101,6 +1097,7 @@ void BrowserActionsContainer::SaveDesiredSizeAndAnimate(
 bool BrowserActionsContainer::ShouldDisplayBrowserAction(
     const Extension* extension) {
   // Only display incognito-enabled extensions while in incognito mode.
-  return (!profile_->IsOffTheRecord() ||
-          profile_->GetExtensionService()->IsIncognitoEnabled(extension));
+  return
+      (!profile_->IsOffTheRecord() ||
+       profile_->GetExtensionService()->IsIncognitoEnabled(extension->id()));
 }
