@@ -10,24 +10,26 @@
 #include <string>
 #include <vector>
 
-#include "base/task.h"
+#include "base/basictypes.h"
+#include "base/compiler_specific.h"
+#include "base/memory/weak_ptr.h"
+#include "base/message_loop_helpers.h"
 #include "chrome/browser/extensions/extension_context_menu_model.h"
 #include "chrome/browser/extensions/extension_toolbar_model.h"
 #include "chrome/browser/extensions/image_loading_tracker.h"
-#include "chrome/browser/ui/views/browser_bubble.h"
 #include "chrome/browser/ui/views/extensions/browser_action_overflow_menu_controller.h"
 #include "chrome/browser/ui/views/extensions/extension_popup.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
 #include "ui/base/animation/animation_delegate.h"
 #include "ui/base/animation/tween.h"
-#include "views/controls/button/menu_button.h"
-#include "views/controls/menu/view_menu_delegate.h"
-#include "views/controls/resize_area.h"
-#include "views/view.h"
+#include "ui/views/controls/button/menu_button.h"
+#include "ui/views/controls/menu/view_menu_delegate.h"
+#include "ui/views/controls/resize_area_delegate.h"
+#include "ui/views/drag_controller.h"
+#include "ui/views/view.h"
 
 class Browser;
-class BrowserActionsContainer;
 class BrowserActionOverflowMenuController;
 class BrowserActionsContainer;
 class Extension;
@@ -36,16 +38,13 @@ class ExtensionPopup;
 class PrefService;
 class Profile;
 
-namespace gfx {
-class CanvasSkia;
-}
-
 namespace ui {
 class SlideAnimation;
 }
 
 namespace views {
-class Menu2;
+class MenuItemView;
+class ResizeArea;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -57,7 +56,7 @@ class Menu2;
 class BrowserActionButton : public views::MenuButton,
                             public views::ButtonListener,
                             public ImageLoadingTracker::Observer,
-                            public NotificationObserver {
+                            public content::NotificationObserver {
  public:
   BrowserActionButton(const Extension* extension,
                       BrowserActionsContainer* panel);
@@ -74,42 +73,48 @@ class BrowserActionButton : public views::MenuButton,
   // Returns the default icon, if any.
   const SkBitmap& default_icon() const { return default_icon_; }
 
-  // Overridden from views::View:
-  virtual void ViewHierarchyChanged(bool is_add, View* parent, View* child);
+  // Does this button's action have a popup?
+  virtual bool IsPopup();
+  virtual GURL GetPopupUrl();
 
   // Overridden from views::ButtonListener:
-  virtual void ButtonPressed(views::Button* sender, const views::Event& event);
+  virtual void ButtonPressed(views::Button* sender,
+                             const views::Event& event) OVERRIDE;
 
   // Overridden from ImageLoadingTracker.
-  virtual void OnImageLoaded(
-      SkBitmap* image, ExtensionResource resource, int index);
+  virtual void OnImageLoaded(SkBitmap* image,
+                             const ExtensionResource& resource,
+                             int index) OVERRIDE;
 
-  // Overridden from NotificationObserver:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
+  // Overridden from content::NotificationObserver:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) OVERRIDE;
 
   // MenuButton behavior overrides.  These methods all default to TextButton
   // behavior unless this button is a popup.  In that case, it uses MenuButton
   // behavior.  MenuButton has the notion of a child popup being shown where the
   // button will stay in the pushed state until the "menu" (a popup in this
   // case) is dismissed.
-  virtual bool Activate();
-  virtual bool OnMousePressed(const views::MouseEvent& e);
-  virtual void OnMouseReleased(const views::MouseEvent& e, bool canceled);
-  virtual bool OnKeyReleased(const views::KeyEvent& e);
-  virtual void OnMouseExited(const views::MouseEvent& event);
-  virtual void ShowContextMenu(const gfx::Point& p, bool is_mouse_gesture);
-
-  // Does this button's action have a popup?
-  virtual bool IsPopup();
-  virtual GURL GetPopupUrl();
+  virtual bool Activate() OVERRIDE;
+  virtual bool OnMousePressed(const views::MouseEvent& event) OVERRIDE;
+  virtual void OnMouseReleased(const views::MouseEvent& event) OVERRIDE;
+  virtual void OnMouseExited(const views::MouseEvent& event) OVERRIDE;
+  virtual bool OnKeyReleased(const views::KeyEvent& event) OVERRIDE;
+  virtual void ShowContextMenu(const gfx::Point& p,
+                               bool is_mouse_gesture) OVERRIDE;
 
   // Notifications when to set button state to pushed/not pushed (for when the
   // popup/context menu is hidden or shown by the container).
   void SetButtonPushed();
   void SetButtonNotPushed();
 
+ protected:
+  // Overridden from views::View:
+  virtual void ViewHierarchyChanged(bool is_add,
+                                    View* parent,
+                                    View* child) OVERRIDE;
+
  private:
   virtual ~BrowserActionButton();
 
@@ -124,9 +129,6 @@ class BrowserActionButton : public views::MenuButton,
   // asynchronously.
   ImageLoadingTracker tracker_;
 
-  // Whether we are currently showing/just finished showing a context menu.
-  bool showing_context_menu_;
-
   // The default icon for our browser action. This might be non-empty if the
   // browser action had a value for default_icon in the manifest.
   SkBitmap default_icon_;
@@ -134,12 +136,12 @@ class BrowserActionButton : public views::MenuButton,
   // The browser action shelf.
   BrowserActionsContainer* panel_;
 
-  scoped_refptr<ExtensionContextMenuModel> context_menu_contents_;
-  scoped_ptr<views::Menu2> context_menu_menu_;
+  // The context menu.  This member is non-NULL only when the menu is shown.
+  views::MenuItemView* context_menu_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
-  friend class DeleteTask<BrowserActionButton>;
+  friend class base::DeleteHelper<BrowserActionButton>;
 
   DISALLOW_COPY_AND_ASSIGN(BrowserActionButton);
 };
@@ -162,15 +164,15 @@ class BrowserActionView : public views::View {
   // returned object.
   gfx::Canvas* GetIconWithBadge();
 
-  // Accessibility accessors, overridden from View.
-  virtual AccessibilityTypes::Role GetAccessibleRole();
+  // Overridden from views::View:
+  virtual void Layout() OVERRIDE;
+  virtual void GetAccessibleState(ui::AccessibleViewState* state) OVERRIDE;
+
+ protected:
+  // Overridden from views::View to paint the badge on top of children.
+  virtual void PaintChildren(gfx::Canvas* canvas) OVERRIDE;
 
  private:
-  virtual void Layout();
-
-  // Override PaintChildren so that we can paint the badge on top of children.
-  virtual void PaintChildren(gfx::Canvas* canvas);
-
   // The container for this view.
   BrowserActionsContainer* panel_;
 
@@ -256,12 +258,12 @@ class BrowserActionsContainer
     : public views::View,
       public views::ViewMenuDelegate,
       public views::DragController,
-      public views::ResizeArea::ResizeAreaDelegate,
+      public views::ResizeAreaDelegate,
       public ui::AnimationDelegate,
       public ExtensionToolbarModel::Observer,
       public BrowserActionOverflowMenuController::Observer,
       public ExtensionContextMenuModel::PopupDelegate,
-      public ExtensionPopup::Observer {
+      public views::Widget::Observer {
  public:
   BrowserActionsContainer(Browser* browser, views::View* owner_view);
   virtual ~BrowserActionsContainer();
@@ -277,6 +279,7 @@ class BrowserActionsContainer
   bool animating() const { return animation_target_size_ > 0; }
 
   // Returns the chevron, if any.
+  views::View* chevron() { return chevron_; }
   const views::View* chevron() const { return chevron_; }
 
   // Returns the profile this container is associated with.
@@ -316,51 +319,47 @@ class BrowserActionsContainer
                                bool inspect_with_devtools);
 
   // Overridden from views::View:
-  virtual gfx::Size GetPreferredSize();
-  virtual void Layout();
-  virtual void Paint(gfx::Canvas* canvas);
-  virtual void ViewHierarchyChanged(bool is_add,
-                                    views::View* parent,
-                                    views::View* child);
-  virtual bool GetDropFormats(
-      int* formats, std::set<ui::OSExchangeData::CustomFormat>* custom_formats);
-  virtual bool AreDropTypesRequired();
-  virtual bool CanDrop(const ui::OSExchangeData& data);
-  virtual void OnDragEntered(const views::DropTargetEvent& event);
-  virtual int OnDragUpdated(const views::DropTargetEvent& event);
-  virtual void OnDragExited();
-  virtual int OnPerformDrop(const views::DropTargetEvent& event);
-  virtual void OnThemeChanged();
-  virtual AccessibilityTypes::Role GetAccessibleRole();
+  virtual gfx::Size GetPreferredSize() OVERRIDE;
+  virtual void Layout() OVERRIDE;
+  virtual bool GetDropFormats(int* formats,
+      std::set<ui::OSExchangeData::CustomFormat>* custom_formats) OVERRIDE;
+  virtual bool AreDropTypesRequired() OVERRIDE;
+  virtual bool CanDrop(const ui::OSExchangeData& data) OVERRIDE;
+  virtual void OnDragEntered(const views::DropTargetEvent& event) OVERRIDE;
+  virtual int OnDragUpdated(const views::DropTargetEvent& event) OVERRIDE;
+  virtual void OnDragExited() OVERRIDE;
+  virtual int OnPerformDrop(const views::DropTargetEvent& event) OVERRIDE;
+  virtual void GetAccessibleState(ui::AccessibleViewState* state) OVERRIDE;
 
   // Overridden from views::ViewMenuDelegate:
-  virtual void RunMenu(View* source, const gfx::Point& pt);
+  virtual void RunMenu(View* source, const gfx::Point& pt) OVERRIDE;
 
   // Overridden from views::DragController:
-  virtual void WriteDragData(View* sender,
-                             const gfx::Point& press_pt,
-                             ui::OSExchangeData* data);
-  virtual int GetDragOperations(View* sender, const gfx::Point& p);
-  virtual bool CanStartDrag(View* sender,
-                            const gfx::Point& press_pt,
-                            const gfx::Point& p);
+  virtual void WriteDragDataForView(View* sender,
+                                    const gfx::Point& press_pt,
+                                    ui::OSExchangeData* data) OVERRIDE;
+  virtual int GetDragOperationsForView(View* sender,
+                                       const gfx::Point& p) OVERRIDE;
+  virtual bool CanStartDragForView(View* sender,
+                                   const gfx::Point& press_pt,
+                                   const gfx::Point& p) OVERRIDE;
 
-  // Overridden from ResizeArea::ResizeAreaDelegate:
-  virtual void OnResize(int resize_amount, bool done_resizing);
+  // Overridden from views::ResizeAreaDelegate:
+  virtual void OnResize(int resize_amount, bool done_resizing) OVERRIDE;
 
   // Overridden from ui::AnimationDelegate:
-  virtual void AnimationProgressed(const ui::Animation* animation);
-  virtual void AnimationEnded(const ui::Animation* animation);
+  virtual void AnimationProgressed(const ui::Animation* animation) OVERRIDE;
+  virtual void AnimationEnded(const ui::Animation* animation) OVERRIDE;
 
   // Overridden from BrowserActionOverflowMenuController::Observer:
   virtual void NotifyMenuDeleted(
-      BrowserActionOverflowMenuController* controller);
+      BrowserActionOverflowMenuController* controller) OVERRIDE;
 
   // Overridden from ExtensionContextMenuModel::PopupDelegate
-  virtual void InspectPopup(ExtensionAction* action);
+  virtual void InspectPopup(ExtensionAction* action) OVERRIDE;
 
-  // Overriden from ExtensionPopup::Delegate
-  virtual void ExtensionPopupIsClosing(ExtensionPopup* popup);
+  // Overridden from views::Widget::Observer
+  virtual void OnWidgetClosing(views::Widget* widget) OVERRIDE;
 
   // Moves a browser action with |id| to |new_index|.
   void MoveBrowserAction(const std::string& extension_id, size_t new_index);
@@ -384,6 +383,14 @@ class BrowserActionsContainer
   // animates to open/closed status.
   static bool disable_animations_during_testing_;
 
+ protected:
+  // Overridden from views::View:
+  virtual void ViewHierarchyChanged(bool is_add,
+                                    views::View* parent,
+                                    views::View* child) OVERRIDE;
+  virtual void OnPaint(gfx::Canvas* canvas) OVERRIDE;
+  virtual void OnThemeChanged() OVERRIDE;
+
  private:
   friend class BrowserActionView;  // So it can access IconHeight().
   friend class ShowFolderMenuTask;
@@ -397,10 +404,12 @@ class BrowserActionsContainer
   static int IconHeight();
 
   // ExtensionToolbarModel::Observer implementation.
-  virtual void BrowserActionAdded(const Extension* extension, int index);
-  virtual void BrowserActionRemoved(const Extension* extension);
-  virtual void BrowserActionMoved(const Extension* extension, int index);
-  virtual void ModelLoaded();
+  virtual void BrowserActionAdded(const Extension* extension,
+                                  int index) OVERRIDE;
+  virtual void BrowserActionRemoved(const Extension* extension) OVERRIDE;
+  virtual void BrowserActionMoved(const Extension* extension,
+                                  int index) OVERRIDE;
+  virtual void ModelLoaded() OVERRIDE;
 
   void LoadImages();
 
@@ -441,7 +450,7 @@ class BrowserActionsContainer
 
   // Animate to the target size (unless testing, in which case we go straight to
   // the target size).  This also saves the target number of visible icons in
-  // the pref if we're not off the record.
+  // the pref if we're not incognito.
   void SaveDesiredSizeAndAnimate(ui::Tween::Type type,
                                  size_t num_visible_icons);
 
@@ -504,10 +513,10 @@ class BrowserActionsContainer
   // The x position for where to draw the drop indicator. -1 if no indicator.
   int drop_indicator_position_;
 
-  ScopedRunnableMethodFactory<BrowserActionsContainer> task_factory_;
+  base::WeakPtrFactory<BrowserActionsContainer> task_factory_;
 
   // Handles delayed showing of the overflow menu when hovering.
-  ScopedRunnableMethodFactory<BrowserActionsContainer> show_menu_task_factory_;
+  base::WeakPtrFactory<BrowserActionsContainer> show_menu_task_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(BrowserActionsContainer);
 };
