@@ -1,36 +1,46 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/views/create_application_shortcut_view.h"
 
-#include "base/callback.h"
+#include <algorithm>
+
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/windows_version.h"
+#include "chrome/browser/extensions/extension_tab_helper.h"
+#include "chrome/browser/favicon/favicon_tab_helper.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/tab_contents/tab_contents_delegate.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/browser/ui/web_applications/web_app_ui.h"
+#include "chrome/browser/ui/webui/extensions/extension_icon_source.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_resource.h"
 #include "chrome/common/pref_names.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/codec/png_codec.h"
+#include "content/public/browser/web_contents_delegate.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/generated_resources.h"
 #include "grit/locale_settings.h"
+#include "grit/theme_resources.h"
 #include "net/base/load_flags.h"
 #include "net/url_request/url_request.h"
-#include "third_party/skia/include/core/SkRect.h"
+#include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkPaint.h"
+#include "third_party/skia/include/core/SkRect.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/controls/button/checkbox.h"
-#include "views/controls/image_view.h"
-#include "views/controls/label.h"
-#include "views/grid_layout.h"
-#include "views/standard_layout.h"
-#include "views/window/window.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/codec/png_codec.h"
+#include "ui/views/controls/button/checkbox.h"
+#include "ui/views/controls/image_view.h"
+#include "ui/views/controls/label.h"
+#include "ui/views/layout/grid_layout.h"
+#include "ui/views/layout/layout_constants.h"
+#include "ui/views/widget/widget.h"
 
 namespace {
 
@@ -50,7 +60,7 @@ class AppInfoView : public views::View {
   void UpdateIcon(const SkBitmap& new_icon);
 
   // Overridden from views::View:
-  virtual void Paint(gfx::Canvas* canvas);
+  virtual void OnPaint(gfx::Canvas* canvas);
 
  private:
   // Initializes the controls
@@ -84,7 +94,7 @@ void AppInfoView::Init(const string16& title_text,
   icon_->SetImage(icon);
   icon_->SetImageSize(gfx::Size(kAppIconSize, kAppIconSize));
 
-  title_ = new views::Label(UTF16ToWide(title_text));
+  title_ = new views::Label(title_text);
   title_->SetMultiLine(true);
   title_->SetHorizontalAlignment(views::Label::ALIGN_LEFT);
   title_->SetFont(ResourceBundle::GetSharedInstance().GetFont(
@@ -100,10 +110,10 @@ void AppInfoView::Init(const string16& title_text,
 void AppInfoView::PrepareDescriptionLabel(const string16& description) {
   DCHECK(!description.empty());
 
-  static const size_t kMaxLength = 200;
-  static const wchar_t* const kEllipsis = L" ... ";
+  const size_t kMaxLength = 200;
+  const string16 kEllipsis(ASCIIToUTF16(" ... "));
 
-  std::wstring text = UTF16ToWide(description);
+  string16 text = description;
   if (text.length() > kMaxLength) {
     text = text.substr(0, kMaxLength);
     text += kEllipsis;
@@ -143,20 +153,18 @@ void AppInfoView::SetupLayout() {
 
 void AppInfoView::UpdateText(const string16& title,
                              const string16& description) {
-  title_->SetText(UTF16ToWide(title));
+  title_->SetText(title);
   PrepareDescriptionLabel(description);
 
   SetupLayout();
 }
 
 void AppInfoView::UpdateIcon(const SkBitmap& new_icon) {
-  DCHECK(icon_ != NULL);
-
   icon_->SetImage(new_icon);
 }
 
-void AppInfoView::Paint(gfx::Canvas* canvas) {
-  gfx::Rect bounds = GetLocalBounds(true);
+void AppInfoView::OnPaint(gfx::Canvas* canvas) {
+  gfx::Rect bounds = GetLocalBounds();
 
   SkRect border_rect = {
     SkIntToScalar(bounds.x()),
@@ -169,7 +177,7 @@ void AppInfoView::Paint(gfx::Canvas* canvas) {
   border_paint.setAntiAlias(true);
   border_paint.setARGB(0xFF, 0xC8, 0xC8, 0xC8);
 
-  canvas->AsCanvasSkia()->drawRoundRect(
+  canvas->GetSkCanvas()->drawRoundRect(
       border_rect, SkIntToScalar(2), SkIntToScalar(2), border_paint);
 
   SkRect inner_rect = {
@@ -182,8 +190,8 @@ void AppInfoView::Paint(gfx::Canvas* canvas) {
   SkPaint inner_paint;
   inner_paint.setAntiAlias(true);
   inner_paint.setARGB(0xFF, 0xF8, 0xF8, 0xF8);
-  canvas->AsCanvasSkia()->drawRoundRect(
-      inner_rect, SkIntToScalar(1.5), SkIntToScalar(1.5), inner_paint);
+  canvas->GetSkCanvas()->drawRoundRect(
+      inner_rect, SkDoubleToScalar(1.5), SkDoubleToScalar(1.5), inner_paint);
 }
 
 }  // namespace
@@ -191,16 +199,18 @@ void AppInfoView::Paint(gfx::Canvas* canvas) {
 namespace browser {
 
 void ShowCreateWebAppShortcutsDialog(gfx::NativeWindow parent_window,
-                                     TabContents* tab_contents) {
-  views::Window::CreateChromeWindow(parent_window, gfx::Rect(),
-      new CreateUrlApplicationShortcutView(tab_contents))->Show();
+                                     TabContentsWrapper* tab_contents) {
+  views::Widget::CreateWindowWithParent(
+      new CreateUrlApplicationShortcutView(tab_contents),
+      parent_window)->Show();
 }
 
 void ShowCreateChromeAppShortcutsDialog(gfx::NativeWindow parent_window,
                                         Profile* profile,
                                         const Extension* app) {
-  views::Window::CreateChromeWindow(parent_window, gfx::Rect(),
-      new CreateChromeApplicationShortcutView(profile, app))->Show();
+  views::Widget::CreateWindowWithParent(
+      new CreateChromeApplicationShortcutView(profile, app),
+      parent_window)->Show();
 }
 
 }  // namespace browser
@@ -226,7 +236,12 @@ class CreateUrlApplicationShortcutView::IconDownloadCallbackFunctor {
 };
 
 CreateApplicationShortcutView::CreateApplicationShortcutView(Profile* profile)
-    : profile_(profile) {}
+    : profile_(profile),
+      app_info_(NULL),
+      create_shortcuts_label_(NULL),
+      desktop_check_box_(NULL),
+      menu_check_box_(NULL),
+      quick_launch_check_box_(NULL) {}
 
 CreateApplicationShortcutView::~CreateApplicationShortcutView() {}
 
@@ -235,30 +250,30 @@ void CreateApplicationShortcutView::InitControls() {
   app_info_ = new AppInfoView(shortcut_info_.title, shortcut_info_.description,
                               shortcut_info_.favicon);
   create_shortcuts_label_ = new views::Label(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_LABEL)));
+      l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_LABEL));
   create_shortcuts_label_->SetHorizontalAlignment(views::Label::ALIGN_LEFT);
 
-  desktop_check_box_ = AddCheckbox(UTF16ToWide(
-      l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_DESKTOP_CHKBOX)),
+  desktop_check_box_ = AddCheckbox(
+      l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_DESKTOP_CHKBOX),
       profile_->GetPrefs()->GetBoolean(prefs::kWebAppCreateOnDesktop));
 
   menu_check_box_ = NULL;
   quick_launch_check_box_ = NULL;
 
 #if defined(OS_WIN)
-  menu_check_box_ = AddCheckbox(UTF16ToWide(
-      l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_START_MENU_CHKBOX)),
+  menu_check_box_ = AddCheckbox(
+      l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_START_MENU_CHKBOX),
       profile_->GetPrefs()->GetBoolean(prefs::kWebAppCreateInAppsMenu));
 
   quick_launch_check_box_ = AddCheckbox(
       (base::win::GetVersion() >= base::win::VERSION_WIN7) ?
-        UTF16ToWide(l10n_util::GetStringUTF16(IDS_PIN_TO_TASKBAR_CHKBOX)) :
-        UTF16ToWide(l10n_util::GetStringUTF16(
-            IDS_CREATE_SHORTCUTS_QUICK_LAUNCH_BAR_CHKBOX)),
+        l10n_util::GetStringUTF16(IDS_PIN_TO_TASKBAR_CHKBOX) :
+        l10n_util::GetStringUTF16(
+            IDS_CREATE_SHORTCUTS_QUICK_LAUNCH_BAR_CHKBOX),
       profile_->GetPrefs()->GetBoolean(prefs::kWebAppCreateInQuickLaunchBar));
-#elif defined(OS_LINUX)
+#elif defined(OS_POSIX)
   menu_check_box_ = AddCheckbox(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_MENU_CHKBOX)),
+      l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_MENU_CHKBOX),
       profile_->GetPrefs()->GetBoolean(prefs::kWebAppCreateInAppsMenu));
 #endif
 
@@ -280,22 +295,22 @@ void CreateApplicationShortcutView::InitControls() {
   layout->StartRow(0, kHeaderColumnSetId);
   layout->AddView(app_info_);
 
-  layout->AddPaddingRow(0, kPanelSubVerticalSpacing);
+  layout->AddPaddingRow(0, views::kPanelSubVerticalSpacing);
   layout->StartRow(0, kHeaderColumnSetId);
   layout->AddView(create_shortcuts_label_);
 
-  layout->AddPaddingRow(0, kLabelToControlVerticalSpacing);
+  layout->AddPaddingRow(0, views::kLabelToControlVerticalSpacing);
   layout->StartRow(0, kTableColumnSetId);
   layout->AddView(desktop_check_box_);
 
   if (menu_check_box_ != NULL) {
-    layout->AddPaddingRow(0, kRelatedControlSmallVerticalSpacing);
+    layout->AddPaddingRow(0, views::kRelatedControlSmallVerticalSpacing);
     layout->StartRow(0, kTableColumnSetId);
     layout->AddView(menu_check_box_);
   }
 
   if (quick_launch_check_box_ != NULL) {
-    layout->AddPaddingRow(0, kRelatedControlSmallVerticalSpacing);
+    layout->AddPaddingRow(0, views::kRelatedControlSmallVerticalSpacing);
     layout->StartRow(0, kTableColumnSetId);
     layout->AddView(quick_launch_check_box_);
   }
@@ -309,18 +324,16 @@ gfx::Size CreateApplicationShortcutView::GetPreferredSize() {
   return gfx::Size(kDialogWidth, height);
 }
 
-std::wstring CreateApplicationShortcutView::GetDialogButtonLabel(
-    MessageBoxFlags::DialogButton button) const {
-  if (button == MessageBoxFlags::DIALOGBUTTON_OK) {
-    return UTF16ToWide(l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_COMMIT));
-  }
-
-  return std::wstring();
+string16 CreateApplicationShortcutView::GetDialogButtonLabel(
+    ui::DialogButton button) const {
+  if (button == ui::DIALOG_BUTTON_OK)
+    return l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_COMMIT);
+  return string16();
 }
 
 bool CreateApplicationShortcutView::IsDialogButtonEnabled(
-    MessageBoxFlags::DialogButton button) const {
-  if (button == MessageBoxFlags::DIALOGBUTTON_OK)
+    ui::DialogButton button) const {
+  if (button == ui::DIALOG_BUTTON_OK)
     return desktop_check_box_->checked() ||
            ((menu_check_box_ != NULL) &&
             menu_check_box_->checked()) ||
@@ -338,24 +351,16 @@ bool CreateApplicationShortcutView::CanMaximize() const {
   return false;
 }
 
-bool CreateApplicationShortcutView::IsAlwaysOnTop() const {
-  return false;
+ui::ModalType CreateApplicationShortcutView::GetModalType() const {
+  return ui::MODAL_TYPE_WINDOW;
 }
 
-bool CreateApplicationShortcutView::HasAlwaysOnTopMenu() const {
-  return false;
-}
-
-bool CreateApplicationShortcutView::IsModal() const {
-  return true;
-}
-
-std::wstring CreateApplicationShortcutView::GetWindowTitle() const {
-  return UTF16ToWide(l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_TITLE));
+string16 CreateApplicationShortcutView::GetWindowTitle() const {
+  return l10n_util::GetStringUTF16(IDS_CREATE_SHORTCUTS_TITLE);
 }
 
 bool CreateApplicationShortcutView::Accept() {
-  if (!IsDialogButtonEnabled(MessageBoxFlags::DIALOGBUTTON_OK))
+  if (!IsDialogButtonEnabled(ui::DIALOG_BUTTON_OK))
     return false;
 
   shortcut_info_.create_on_desktop = desktop_check_box_->checked();
@@ -371,9 +376,7 @@ bool CreateApplicationShortcutView::Accept() {
   shortcut_info_.create_in_quick_launch_bar = false;
 #endif
 
-  web_app::CreateShortcut(profile_->GetPath(),
-                          shortcut_info_,
-                          NULL);
+  web_app::CreateShortcut(profile_->GetPath(), shortcut_info_);
   return true;
 }
 
@@ -383,7 +386,7 @@ views::View* CreateApplicationShortcutView::GetContentsView() {
 }
 
 views::Checkbox* CreateApplicationShortcutView::AddCheckbox(
-    const std::wstring& text, bool checked) {
+    const string16& text, bool checked) {
   views::Checkbox* checkbox = new views::Checkbox(text);
   checkbox->SetChecked(checked);
   checkbox->set_listener(this);
@@ -407,13 +410,14 @@ void CreateApplicationShortcutView::ButtonPressed(views::Button* sender,
 }
 
 CreateUrlApplicationShortcutView::CreateUrlApplicationShortcutView(
-    TabContents* tab_contents)
+    TabContentsWrapper* tab_contents)
     : CreateApplicationShortcutView(tab_contents->profile()),
       tab_contents_(tab_contents),
       pending_download_(NULL)  {
 
   web_app::GetShortcutInfoForTab(tab_contents_, &shortcut_info_);
-  const WebApplicationInfo& app_info = tab_contents_->web_app_info();
+  const WebApplicationInfo& app_info =
+      tab_contents_->extension_tab_helper()->web_app_info();
   if (!app_info.icons.empty()) {
     web_app::GetIconsInfo(app_info, &unprocessed_icons_);
     FetchIcon();
@@ -431,9 +435,11 @@ bool CreateUrlApplicationShortcutView::Accept() {
   if (!CreateApplicationShortcutView::Accept())
     return false;
 
-  tab_contents_->SetAppIcon(shortcut_info_.favicon);
-  if (tab_contents_->delegate())
-    tab_contents_->delegate()->ConvertContentsToApplication(tab_contents_);
+  tab_contents_->extension_tab_helper()->SetAppIcon(shortcut_info_.favicon);
+  if (tab_contents_->web_contents()->GetDelegate()) {
+    tab_contents_->web_contents()->GetDelegate()->ConvertContentsToApplication(
+        tab_contents_->web_contents());
+  }
   return true;
 }
 
@@ -447,11 +453,13 @@ void CreateUrlApplicationShortcutView::FetchIcon() {
   pending_download_ = new IconDownloadCallbackFunctor(this);
   DCHECK(pending_download_);
 
-  tab_contents_->fav_icon_helper().DownloadImage(
+  tab_contents_->favicon_tab_helper()->DownloadImage(
       unprocessed_icons_.back().url,
       std::max(unprocessed_icons_.back().width,
                unprocessed_icons_.back().height),
-      NewCallback(pending_download_, &IconDownloadCallbackFunctor::Run));
+      history::FAVICON,
+      base::Bind(&IconDownloadCallbackFunctor::Run,
+                 base::Unretained(pending_download_)));
 
   unprocessed_icons_.pop_back();
 }
@@ -474,8 +482,7 @@ CreateChromeApplicationShortcutView::CreateChromeApplicationShortcutView(
       CreateApplicationShortcutView(profile),
       app_(app),
       ALLOW_THIS_IN_INITIALIZER_LIST(tracker_(this)) {
-
-  shortcut_info_.extension_id = UTF8ToUTF16(app_->id());
+  shortcut_info_.extension_id = app_->id();
   shortcut_info_.url = GURL(app_->launch_web_url());
   shortcut_info_.title = UTF8ToUTF16(app_->name());
   shortcut_info_.description = UTF8ToUTF16(app_->description());
@@ -499,24 +506,27 @@ CreateChromeApplicationShortcutView::CreateChromeApplicationShortcutView(
         ExtensionIconSet::MATCH_SMALLER);
   }
 
+  InitControls();
+
+  // tracker_.LoadImage() can call OnImageLoaded() before it returns if the
+  // image is cached.  This is very rare.  app_info_ must be initialized
+  // when OnImageLoaded() is called, so we check it here.
+  CHECK(app_info_);
   tracker_.LoadImage(app_,
                      icon_resource,
                      max_size,
                      ImageLoadingTracker::DONT_CACHE);
-
-  InitControls();
 }
 
 CreateChromeApplicationShortcutView::~CreateChromeApplicationShortcutView() {}
 
 // Called by tracker_ when the app's icon is loaded.
 void CreateChromeApplicationShortcutView::OnImageLoaded(
-    SkBitmap* image, ExtensionResource resource, int index) {
-  if (image->isNull()) {
-    NOTREACHED() << "Corrupt image in profile?";
-    return;
-  }
+    SkBitmap* image, const ExtensionResource& resource, int index) {
+  if (!image || image->isNull())
+    image = ExtensionIconSource::LoadImageByResourceId(IDR_APP_DEFAULT_ICON);
+
   shortcut_info_.favicon = *image;
+  CHECK(app_info_);
   static_cast<AppInfoView*>(app_info_)->UpdateIcon(shortcut_info_.favicon);
 }
-
