@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,24 +10,26 @@
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
+#include "chrome/browser/themes/theme_service.h"
 #include "chrome/browser/ui/find_bar/find_bar_controller.h"
 #include "chrome/browser/ui/find_bar/find_bar_state.h"
+#include "chrome/browser/ui/find_bar/find_bar_state_factory.h"
+#include "chrome/browser/ui/find_bar/find_tab_helper.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/browser/ui/view_ids.h"
 #include "chrome/browser/ui/views/find_bar_host.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
-#include "gfx/canvas.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/ui_resources_standard.h"
 #include "third_party/skia/include/effects/SkGradientShader.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/background.h"
-#include "views/controls/button/image_button.h"
-#include "views/controls/label.h"
-#include "views/focus/focus_manager.h"
-#include "views/widget/widget.h"
+#include "ui/gfx/canvas.h"
+#include "ui/views/controls/button/image_button.h"
+#include "ui/views/controls/label.h"
+#include "ui/views/controls/textfield/textfield.h"
+#include "ui/views/widget/widget.h"
 
 // The amount of whitespace to have before the find button.
 static const int kWhiteSpaceAfterMatchCountLabel = 1;
@@ -56,26 +58,6 @@ static const SkColor kBackgroundColorMatch = SK_ColorWHITE;
 // The background color of the match count label when no results are found.
 static const SkColor kBackgroundColorNoMatch = SkColorSetRGB(255, 102, 102);
 
-// The background images for the dialog. They are split into a left, a middle
-// and a right part. The middle part determines the height of the dialog. The
-// middle part is stretched to fill any remaining part between the left and the
-// right image, after sizing the dialog to kWindowWidth.
-static const SkBitmap* kDialog_left = NULL;
-static const SkBitmap* kDialog_middle = NULL;
-static const SkBitmap* kDialog_right = NULL;
-
-// When we are animating, we draw only the top part of the left and right
-// edges to give the illusion that the find dialog is attached to the
-// window during this animation; this is the height of the items we draw.
-static const int kAnimatingEdgeHeight = 5;
-
-// The background image for the Find text box, which we draw behind the Find box
-// to provide the Chrome look to the edge of the text box.
-static const SkBitmap* kBackground = NULL;
-
-// The rounded edge on the left side of the Find text box.
-static const SkBitmap* kBackground_left = NULL;
-
 // The default number of average characters that the text box will be. This
 // number brings the width on a "regular fonts" system to about 300px.
 static const int kDefaultCharWidth = 43;
@@ -85,20 +67,19 @@ static const int kDefaultCharWidth = 43;
 
 FindBarView::FindBarView(FindBarHost* host)
     : DropdownBarView(host),
-#if defined(OS_LINUX)
-      ignore_contents_changed_(false),
-#endif
       find_text_(NULL),
       match_count_text_(NULL),
       focus_forwarder_view_(NULL),
       find_previous_button_(NULL),
       find_next_button_(NULL),
-      close_button_(NULL) {
-  SetID(VIEW_ID_FIND_IN_PAGE);
+      close_button_(NULL),
+      text_box_background_(NULL),
+      text_box_background_left_(NULL) {
+  set_id(VIEW_ID_FIND_IN_PAGE);
   ResourceBundle& rb = ResourceBundle::GetSharedInstance();
 
   find_text_ = new SearchTextfieldView();
-  find_text_->SetID(VIEW_ID_FIND_IN_PAGE_TEXT_FIELD);
+  find_text_->set_id(VIEW_ID_FIND_IN_PAGE_TEXT_FIELD);
   find_text_->SetFont(rb.GetFont(ResourceBundle::BaseFont));
   find_text_->set_default_width_in_chars(kDefaultCharWidth);
   find_text_->SetController(this);
@@ -107,7 +88,6 @@ FindBarView::FindBarView(FindBarHost* host)
 
   match_count_text_ = new views::Label();
   match_count_text_->SetFont(rb.GetFont(ResourceBundle::BaseFont));
-  match_count_text_->SetColor(kTextColorMatchCount);
   match_count_text_->SetHorizontalAlignment(views::Label::ALIGN_CENTER);
   AddChildView(match_count_text_);
 
@@ -117,22 +97,22 @@ FindBarView::FindBarView(FindBarHost* host)
 
   find_previous_button_ = new views::ImageButton(this);
   find_previous_button_->set_tag(FIND_PREVIOUS_TAG);
-  find_previous_button_->SetFocusable(true);
+  find_previous_button_->set_focusable(true);
   find_previous_button_->SetImage(views::CustomButton::BS_NORMAL,
       rb.GetBitmapNamed(IDR_FINDINPAGE_PREV));
   find_previous_button_->SetImage(views::CustomButton::BS_HOT,
       rb.GetBitmapNamed(IDR_FINDINPAGE_PREV_H));
   find_previous_button_->SetImage(views::CustomButton::BS_DISABLED,
       rb.GetBitmapNamed(IDR_FINDINPAGE_PREV_P));
-  find_previous_button_->SetTooltipText(UTF16ToWide(
-      l10n_util::GetStringUTF16(IDS_FIND_IN_PAGE_PREVIOUS_TOOLTIP)));
+  find_previous_button_->SetTooltipText(
+      l10n_util::GetStringUTF16(IDS_FIND_IN_PAGE_PREVIOUS_TOOLTIP));
   find_previous_button_->SetAccessibleName(
       l10n_util::GetStringUTF16(IDS_ACCNAME_PREVIOUS));
   AddChildView(find_previous_button_);
 
   find_next_button_ = new views::ImageButton(this);
   find_next_button_->set_tag(FIND_NEXT_TAG);
-  find_next_button_->SetFocusable(true);
+  find_next_button_->set_focusable(true);
   find_next_button_->SetImage(views::CustomButton::BS_NORMAL,
       rb.GetBitmapNamed(IDR_FINDINPAGE_NEXT));
   find_next_button_->SetImage(views::CustomButton::BS_HOT,
@@ -140,14 +120,14 @@ FindBarView::FindBarView(FindBarHost* host)
   find_next_button_->SetImage(views::CustomButton::BS_DISABLED,
       rb.GetBitmapNamed(IDR_FINDINPAGE_NEXT_P));
   find_next_button_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_FIND_IN_PAGE_NEXT_TOOLTIP)));
+      l10n_util::GetStringUTF16(IDS_FIND_IN_PAGE_NEXT_TOOLTIP));
   find_next_button_->SetAccessibleName(
       l10n_util::GetStringUTF16(IDS_ACCNAME_NEXT));
   AddChildView(find_next_button_);
 
   close_button_ = new views::ImageButton(this);
   close_button_->set_tag(CLOSE_TAG);
-  close_button_->SetFocusable(true);
+  close_button_->set_focusable(true);
   close_button_->SetImage(views::CustomButton::BS_NORMAL,
                           rb.GetBitmapNamed(IDR_CLOSE_BAR));
   close_button_->SetImage(views::CustomButton::BS_HOT,
@@ -155,34 +135,31 @@ FindBarView::FindBarView(FindBarHost* host)
   close_button_->SetImage(views::CustomButton::BS_PUSHED,
                           rb.GetBitmapNamed(IDR_CLOSE_BAR_P));
   close_button_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_FIND_IN_PAGE_CLOSE_TOOLTIP)));
+      l10n_util::GetStringUTF16(IDS_FIND_IN_PAGE_CLOSE_TOOLTIP));
   close_button_->SetAccessibleName(
       l10n_util::GetStringUTF16(IDS_ACCNAME_CLOSE));
   AddChildView(close_button_);
 
-  if (kDialog_left == NULL) {
-    // Background images for the dialog.
-    kDialog_left = rb.GetBitmapNamed(IDR_FIND_DIALOG_LEFT);
-    kDialog_middle = rb.GetBitmapNamed(IDR_FIND_DIALOG_MIDDLE);
-    kDialog_right = rb.GetBitmapNamed(IDR_FIND_DIALOG_RIGHT);
+  SetBackground(rb.GetBitmapNamed(IDR_FIND_DLG_LEFT_BACKGROUND),
+                rb.GetBitmapNamed(IDR_FIND_DLG_RIGHT_BACKGROUND));
 
-    // Background images for the Find edit box.
-    kBackground = rb.GetBitmapNamed(IDR_FIND_BOX_BACKGROUND);
-    kBackground_left = rb.GetBitmapNamed(IDR_FIND_BOX_BACKGROUND_LEFT);
-  }
+  SetBorder(IDR_FIND_DIALOG_LEFT, IDR_FIND_DIALOG_MIDDLE,
+            IDR_FIND_DIALOG_RIGHT);
+
+  preferred_height_ = rb.GetBitmapNamed(IDR_FIND_DIALOG_MIDDLE)->height();
+
+  // Background images for the Find edit box.
+  text_box_background_ = rb.GetBitmapNamed(IDR_FIND_BOX_BACKGROUND);
+  text_box_background_left_ = rb.GetBitmapNamed(IDR_FIND_BOX_BACKGROUND_LEFT);
+
+  EnableCanvasFlippingForRTLUI(true);
 }
 
 FindBarView::~FindBarView() {
 }
 
 void FindBarView::SetFindText(const string16& find_text) {
-#if defined(OS_LINUX)
-  ignore_contents_changed_ = true;
-#endif
   find_text_->SetText(find_text);
-#if defined(OS_LINUX)
-  ignore_contents_changed_ = false;
-#endif
 }
 
 string16 FindBarView::GetFindText() const {
@@ -194,7 +171,7 @@ string16 FindBarView::GetFindSelectedText() const {
 }
 
 string16 FindBarView::GetMatchCountText() const {
-  return WideToUTF16Hack(match_count_text_->GetText());
+  return match_count_text_->GetText();
 }
 
 void FindBarView::UpdateForResult(const FindNotificationDetails& result,
@@ -210,22 +187,20 @@ void FindBarView::UpdateForResult(const FindNotificationDetails& result,
     find_text_->SelectAll();
   }
 
-  if (!find_text.empty() && have_valid_range) {
-    match_count_text_->SetText(UTF16ToWide(
-        l10n_util::GetStringFUTF16(IDS_FIND_IN_PAGE_COUNT,
-            base::IntToString16(result.active_match_ordinal()),
-            base::IntToString16(result.number_of_matches()))));
-
-    UpdateMatchCountAppearance(result.number_of_matches() == 0 &&
-                               result.final_update());
-  } else {
+  if (find_text.empty() || !have_valid_range) {
     // If there was no text entered, we don't show anything in the result count
     // area.
-    match_count_text_->SetText(std::wstring());
-
-    UpdateMatchCountAppearance(false);
+    ClearMatchCount();
+    return;
   }
 
+  match_count_text_->SetText(l10n_util::GetStringFUTF16(IDS_FIND_IN_PAGE_COUNT,
+      base::IntToString16(result.active_match_ordinal()),
+      base::IntToString16(result.number_of_matches())));
+
+  UpdateMatchCountAppearance(result.number_of_matches() == 0 &&
+                             result.final_update());
+
   // The match_count label may have increased/decreased in size so we need to
   // do a layout and repaint the dialog so that the find text field doesn't
   // partially overlap the match-count label when it increases on no matches.
@@ -234,30 +209,13 @@ void FindBarView::UpdateForResult(const FindNotificationDetails& result,
 }
 
 void FindBarView::ClearMatchCount() {
-  match_count_text_->SetText(L"");
+  match_count_text_->SetText(string16());
   UpdateMatchCountAppearance(false);
   Layout();
   SchedulePaint();
 }
 
 void FindBarView::SetFocusAndSelection(bool select_all) {
-#if defined(OS_CHROMEOS)
-  // TODO(altimofeev): this workaround is needed only when the FindBar was
-  // opened from the wrench menu (it also works in the accelerator case, but it
-  // is not really needed).
-
-  // Restore focus to allow the find bar's external focus tracker to save the
-  // view that should be activated later (the tracker is created after the
-  // wrench menu has received the focus).
-  find_text_->GetFocusManager()->RestoreFocusedView();
-  find_text_->RequestFocus();
-  // Storing is needed here because the view that has focus before the wrench
-  // menu activation will get focus just after the wrench menu is closed.
-  // The FindBar has it's own focus tracker, so it will focus the correct view
-  // on close.
-  find_text_->GetFocusManager()->StoreFocusedView();
-  // Request focus again since the call to StoreFocusedView unfocuses the view.
-#endif
   find_text_->RequestFocus();
   if (select_all && !find_text_->text().empty())
     find_text_->SelectAll();
@@ -266,47 +224,9 @@ void FindBarView::SetFocusAndSelection(bool select_all) {
 ///////////////////////////////////////////////////////////////////////////////
 // FindBarView, views::View overrides:
 
-void FindBarView::Paint(gfx::Canvas* canvas) {
-  SkPaint paint;
-
-  // Determine the find bar size as well as the offset from which to tile the
-  // toolbar background image.  First, get the widget bounds.
-  gfx::Rect bounds;
-  GetWidget()->GetBounds(&bounds, true);
-  // Now convert from screen to parent coordinates.
-  gfx::Point origin(bounds.origin());
-  BrowserView* browser_view = host()->browser_view();
-  ConvertPointToView(NULL, browser_view, &origin);
-  bounds.set_origin(origin);
-  // Finally, calculate the background image tiling offset.
-  origin = browser_view->OffsetPointForToolbarBackgroundImage(origin);
-
-  // First, we draw the background image for the whole dialog (3 images: left,
-  // middle and right). Note, that the window region has been set by the
-  // controller, so the whitespace in the left and right background images is
-  // actually outside the window region and is therefore not drawn. See
-  // FindInPageWidgetWin::CreateRoundedWindowEdges() for details.
-  ui::ThemeProvider* tp = GetThemeProvider();
-  canvas->TileImageInt(*tp->GetBitmapNamed(IDR_THEME_TOOLBAR), origin.x(),
-                       origin.y(), 0, 0, bounds.width(), bounds.height());
-
-  // Now flip the canvas for the rest of the graphics if in RTL mode.
-  canvas->Save();
-  if (base::i18n::IsRTL()) {
-    canvas->TranslateInt(width(), 0);
-    canvas->ScaleInt(-1, 1);
-  }
-
-  canvas->DrawBitmapInt(*kDialog_left, 0, 0);
-
-  // Stretch the middle background to cover all of the area between the two
-  // other images.
-  canvas->TileImageInt(*kDialog_middle, kDialog_left->width(), 0,
-      bounds.width() - kDialog_left->width() - kDialog_right->width(),
-      kDialog_middle->height());
-
-  canvas->DrawBitmapInt(*kDialog_right, bounds.width() - kDialog_right->width(),
-                        0);
+void FindBarView::OnPaint(gfx::Canvas* canvas) {
+  // Paint drop down bar border and background.
+  DropdownBarView::OnPaint(canvas);
 
   // Then we draw the background image for the Find Textfield. We start by
   // calculating the position of background images for the Find text box.
@@ -314,35 +234,30 @@ void FindBarView::Paint(gfx::Canvas* canvas) {
   gfx::Point back_button_origin = find_previous_button_->bounds().origin();
 
   // Draw the image to the left that creates a curved left edge for the box.
-  canvas->TileImageInt(*kBackground_left,
-      find_text_x - kBackground_left->width(), back_button_origin.y(),
-      kBackground_left->width(), kBackground_left->height());
+  canvas->TileImageInt(*text_box_background_left_,
+      find_text_x - text_box_background_left_->width(),
+      back_button_origin.y(), text_box_background_left_->width(),
+      text_box_background_left_->height());
 
   // Draw the top and bottom border for whole text box (encompasses both the
   // find_text_ edit box and the match_count_text_ label).
-  canvas->TileImageInt(*kBackground, find_text_x, back_button_origin.y(),
+  canvas->TileImageInt(*text_box_background_, find_text_x,
+                       back_button_origin.y(),
                        back_button_origin.x() - find_text_x,
-                       kBackground->height());
-
-  if (animation_offset() > 0) {
-    // While animating we draw the curved edges at the point where the
-    // controller told us the top of the window is: |animation_offset()|.
-    canvas->TileImageInt(*kDialog_left, bounds.x(), animation_offset(),
-                         kDialog_left->width(), kAnimatingEdgeHeight);
-    canvas->TileImageInt(*kDialog_right,
-        bounds.width() - kDialog_right->width(), animation_offset(),
-        kDialog_right->width(), kAnimatingEdgeHeight);
-  }
-
-  canvas->Restore();
+                       text_box_background_->height());
 }
 
 void FindBarView::Layout() {
-  gfx::Size panel_size = GetPreferredSize();
+  int panel_width = GetPreferredSize().width();
+
+  // Stay within view bounds.
+  int view_width = width();
+  if (view_width && view_width < panel_width)
+    panel_width = view_width;
 
   // First we draw the close button on the far right.
   gfx::Size sz = close_button_->GetPreferredSize();
-  close_button_->SetBounds(panel_size.width() - sz.width() -
+  close_button_->SetBounds(panel_width - sz.width() -
                                kMarginRightOfCloseButton,
                            (height() - sz.height()) / 2,
                            sz.width(),
@@ -390,8 +305,9 @@ void FindBarView::Layout() {
 
   // And whatever space is left in between, gets filled up by the find edit box.
   sz = find_text_->GetPreferredSize();
-  sz.set_width(match_count_x - kMarginLeftOfFindTextfield);
-  find_text_->SetBounds(match_count_x - sz.width(),
+  sz.set_width(std::max(0, match_count_x - kMarginLeftOfFindTextfield));
+  int find_text_x = std::max(0, match_count_x - sz.width());
+  find_text_->SetBounds(find_text_x,
                         (height() - sz.height()) / 2 + 1,
                         sz.width(),
                         sz.height());
@@ -417,7 +333,7 @@ void FindBarView::ViewHierarchyChanged(bool is_add, View* parent, View* child) {
 
 gfx::Size FindBarView::GetPreferredSize() {
   gfx::Size prefsize = find_text_->GetPreferredSize();
-  prefsize.set_height(kDialog_middle->height());
+  prefsize.set_height(preferred_height_);
 
   // Add up all the preferred sizes and margins of the rest of the controls.
   prefsize.Enlarge(kMarginLeftOfCloseButton + kMarginRightOfCloseButton +
@@ -439,9 +355,9 @@ void FindBarView::ButtonPressed(
     case FIND_NEXT_TAG:
       if (!find_text_->text().empty()) {
         find_bar_host()->GetFindBarController()->tab_contents()->
-            StartFinding(find_text_->text(),
-                         sender->tag() == FIND_NEXT_TAG,
-                         false);  // Not case sensitive.
+            find_tab_helper()->StartFinding(find_text_->text(),
+                                            sender->tag() == FIND_NEXT_TAG,
+                                            false);  // Not case sensitive.
       }
       if (event.IsMouseEvent()) {
         // If mouse event, we move the focus back to the text-field, so that the
@@ -463,16 +379,10 @@ void FindBarView::ButtonPressed(
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// FindBarView, views::Textfield::Controller implementation:
+// FindBarView, views::TextfieldController implementation:
 
 void FindBarView::ContentsChanged(views::Textfield* sender,
                                   const string16& new_contents) {
-#if defined(OS_LINUX)
-  // On gtk setting the text in the find view causes a notification.
-  if (ignore_contents_changed_)
-    return;
-#endif
-
   FindBarController* controller = find_bar_host()->GetFindBarController();
   DCHECK(controller);
   // We must guard against a NULL tab_contents, which can happen if the text
@@ -480,16 +390,18 @@ void FindBarView::ContentsChanged(views::Textfield* sender,
   // can lead to crashes, as exposed by automation testing in issue 8048.
   if (!controller->tab_contents())
     return;
+  FindTabHelper* find_tab_helper =
+      controller->tab_contents()->find_tab_helper();
 
   // When the user changes something in the text box we check the contents and
   // if the textbox contains something we set it as the new search string and
   // initiate search (even though old searches might be in progress).
   if (!new_contents.empty()) {
     // The last two params here are forward (true) and case sensitive (false).
-    controller->tab_contents()->StartFinding(new_contents, true, false);
+    find_tab_helper->StartFinding(new_contents, true, false);
   } else {
-    controller->tab_contents()->StopFinding(FindBarController::kClearSelection);
-    UpdateForResult(controller->tab_contents()->find_result(), string16());
+    find_tab_helper->StopFinding(FindBarController::kClearSelection);
+    UpdateForResult(find_tab_helper->find_result(), string16());
 
     // Clearing the text box should clear the prepopulate state so that when
     // we close and reopen the Find box it doesn't show the search we just
@@ -497,7 +409,8 @@ void FindBarView::ContentsChanged(views::Textfield* sender,
     // sent for a lot more things than just the user nulling out the search
     // terms. See http://crbug.com/45372.
     FindBarState* find_bar_state =
-        controller->tab_contents()->profile()->GetFindBarState();
+        FindBarStateFactory::GetForProfile(
+            controller->tab_contents()->profile());
     find_bar_state->set_last_prepopulate_text(string16());
   }
 }
@@ -511,15 +424,17 @@ bool FindBarView::HandleKeyEvent(views::Textfield* sender,
   if (find_bar_host()->MaybeForwardKeyEventToWebpage(key_event))
     return true;  // Handled, we are done!
 
-  if (key_event.GetKeyCode() == ui::VKEY_RETURN) {
+  if (key_event.key_code() == ui::VKEY_RETURN) {
     // Pressing Return/Enter starts the search (unless text box is empty).
     string16 find_string = find_text_->text();
     if (!find_string.empty()) {
+      FindBarController* controller = find_bar_host()->GetFindBarController();
+      FindTabHelper* find_tab_helper =
+          controller->tab_contents()->find_tab_helper();
       // Search forwards for enter, backwards for shift-enter.
-      find_bar_host()->GetFindBarController()->tab_contents()->StartFinding(
-          find_string,
-          !key_event.IsShiftDown(),
-          false);  // Not case sensitive.
+      find_tab_helper->StartFinding(find_string,
+                                    !key_event.IsShiftDown(),
+                                    false);  // Not case sensitive.
     }
   }
 
@@ -530,12 +445,14 @@ void FindBarView::UpdateMatchCountAppearance(bool no_match) {
   if (no_match) {
     match_count_text_->set_background(
         views::Background::CreateSolidBackground(kBackgroundColorNoMatch));
-    match_count_text_->SetColor(kTextColorNoMatch);
+    match_count_text_->SetEnabledColor(kTextColorNoMatch);
   } else {
     match_count_text_->set_background(
       views::Background::CreateSolidBackground(kBackgroundColorMatch));
-    match_count_text_->SetColor(kTextColorMatchCount);
+    match_count_text_->SetEnabledColor(kTextColorMatchCount);
   }
+  match_count_text_->SetBackgroundColor(
+      match_count_text_->background()->get_color());
 }
 
 bool FindBarView::FocusForwarderView::OnMousePressed(
@@ -554,6 +471,8 @@ FindBarView::SearchTextfieldView::~SearchTextfieldView() {
 }
 
 void FindBarView::SearchTextfieldView::RequestFocus() {
+  if (HasFocus())
+    return;
   views::View::RequestFocus();
   SelectAll();
 }
@@ -566,7 +485,7 @@ void FindBarView::OnThemeChanged() {
   ResourceBundle& rb = ResourceBundle::GetSharedInstance();
   if (GetThemeProvider()) {
     close_button_->SetBackground(
-        GetThemeProvider()->GetColor(BrowserThemeProvider::COLOR_TAB_TEXT),
+        GetThemeProvider()->GetColor(ThemeService::COLOR_TAB_TEXT),
         rb.GetBitmapNamed(IDR_CLOSE_BAR),
         rb.GetBitmapNamed(IDR_CLOSE_BAR_MASK));
   }
