@@ -1,28 +1,28 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/views/frame/browser_view_layout.h"
 
-#include "chrome/browser/sidebar/sidebar_manager.h"
 #include "chrome/browser/ui/find_bar/find_bar.h"
 #include "chrome/browser/ui/find_bar/find_bar_controller.h"
 #include "chrome/browser/ui/view_ids.h"
-#include "chrome/browser/ui/views/bookmark_bar_view.h"
-#include "chrome/browser/ui/views/download_shelf_view.h"
+#include "chrome/browser/ui/views/bookmarks/bookmark_bar_view.h"
 #include "chrome/browser/ui/views/frame/browser_frame.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
 #include "chrome/browser/ui/views/frame/contents_container.h"
+#include "chrome/browser/ui/views/infobars/infobar_container_view.h"
 #include "chrome/browser/ui/views/tab_contents/tab_contents_container.h"
-#include "chrome/browser/ui/views/tabs/side_tab_strip.h"
-#include "chrome/browser/ui/views/tabs/tab_strip.h"
+#include "chrome/browser/ui/views/tabs/abstract_tab_strip_view.h"
 #include "chrome/browser/ui/views/toolbar_view.h"
-#include "gfx/scrollbar_size.h"
-#include "views/controls/single_split_view.h"
-#include "views/window/window.h"
+#include "ui/base/hit_test.h"
+#include "ui/gfx/point.h"
+#include "ui/gfx/scrollbar_size.h"
+#include "ui/gfx/size.h"
+#include "ui/views/controls/single_split_view.h"
 
-#if defined(OS_LINUX)
-#include "views/window/hit_test.h"
+#if !defined(OS_CHROMEOS) || defined(USE_AURA)
+#include "chrome/browser/ui/views/download/download_shelf_view.h"
 #endif
 
 namespace {
@@ -32,9 +32,20 @@ namespace {
 const int kTabShadowSize = 2;
 // The vertical overlap between the TabStrip and the Toolbar.
 const int kToolbarTabStripVerticalOverlap = 3;
-// An offset distance between certain toolbars and the toolbar that preceded
-// them in layout.
-const int kSeparationLineHeight = 1;
+// The number of pixels the bookmark bar should overlap the spacer by if the
+// spacer is visible.
+const int kSpacerBookmarkBarOverlap = 1;
+
+// Combines View::ConvertPointToView and View::HitTest for a given |point|.
+// Converts |point| from |src| to |dst| and hit tests it against |dst|. The
+// converted |point| can then be retrieved and used for additional tests.
+bool ConvertedHitTest(views::View* src, views::View* dst, gfx::Point* point) {
+  DCHECK(src);
+  DCHECK(dst);
+  DCHECK(point);
+  views::View::ConvertPointToView(src, dst, point);
+  return dst->HitTest(*point);
+}
 
 }  // namespace
 
@@ -53,6 +64,9 @@ BrowserViewLayout::BrowserViewLayout()
       find_bar_y_(0) {
 }
 
+BrowserViewLayout::~BrowserViewLayout() {
+}
+
 gfx::Size BrowserViewLayout::GetMinimumSize() {
   // TODO(noname): In theory the tabstrip width should probably be
   // (OTR + tabstrip + caption buttons) width.
@@ -69,8 +83,9 @@ gfx::Size BrowserViewLayout::GetMinimumSize() {
   if (active_bookmark_bar_ &&
       browser()->SupportsWindowFeature(Browser::FEATURE_BOOKMARKBAR)) {
     bookmark_bar_size = active_bookmark_bar_->GetMinimumSize();
-    bookmark_bar_size.Enlarge(0, -(kSeparationLineHeight +
-        active_bookmark_bar_->GetToolbarOverlap(true)));
+    bookmark_bar_size.Enlarge(0,
+        -(views::NonClientFrameView::kClientEdgeThickness +
+            active_bookmark_bar_->GetToolbarOverlap(true)));
   }
   gfx::Size contents_size(contents_split_->GetMinimumSize());
 
@@ -94,11 +109,8 @@ gfx::Rect BrowserViewLayout::GetFindBarBoundingBox() const {
 
   // First determine the bounding box of the content area in Widget
   // coordinates.
-  gfx::Rect bounding_box(contents_container_->bounds());
-
-  gfx::Point topleft;
-  views::View::ConvertPointToWidget(contents_container_, &topleft);
-  bounding_box.set_origin(topleft);
+  gfx::Rect bounding_box = contents_container_->ConvertRectToWidget(
+      contents_container_->GetLocalBounds());
 
   // Adjust the position and size of the bounding box by the find bar offset
   // calculated during the last Layout.
@@ -130,21 +142,19 @@ int BrowserViewLayout::NonClientHitTest(
   // area of the window. So we need to treat hit-tests in these regions as
   // hit-tests of the titlebar.
 
-  views::View* parent = browser_view_->GetParent();
+  views::View* parent = browser_view_->parent();
 
   gfx::Point point_in_browser_view_coords(point);
   views::View::ConvertPointToView(
       parent, browser_view_, &point_in_browser_view_coords);
+  gfx::Point test_point(point);
 
   // Determine if the TabStrip exists and is capable of being clicked on. We
   // might be a popup window without a TabStrip.
   if (browser_view_->IsTabStripVisible()) {
     // See if the mouse pointer is within the bounds of the TabStrip.
-    gfx::Point point_in_tabstrip_coords(point);
-    views::View::ConvertPointToView(parent, tabstrip_,
-                                    &point_in_tabstrip_coords);
-    if (tabstrip_->HitTest(point_in_tabstrip_coords)) {
-      if (tabstrip_->IsPositionInWindowCaption(point_in_tabstrip_coords))
+    if (ConvertedHitTest(parent, tabstrip_, &test_point)) {
+      if (tabstrip_->IsPositionInWindowCaption(test_point))
         return HTCAPTION;
       return HTCLIENT;
     }
@@ -207,7 +217,7 @@ void BrowserViewLayout::Installed(views::View* host) {
 void BrowserViewLayout::Uninstalled(views::View* host) {}
 
 void BrowserViewLayout::ViewAdded(views::View* host, views::View* view) {
-  switch (view->GetID()) {
+  switch (view->id()) {
     case VIEW_ID_CONTENTS_SPLIT: {
       contents_split_ = static_cast<views::SingleSplitView*>(view);
       // We're installed as the LayoutManager before BrowserView creates the
@@ -220,7 +230,11 @@ void BrowserViewLayout::ViewAdded(views::View* host, views::View* view) {
       infobar_container_ = view;
       break;
     case VIEW_ID_DOWNLOAD_SHELF:
+#if !defined(OS_CHROMEOS) || defined(USE_AURA)
       download_shelf_ = static_cast<DownloadShelfView*>(view);
+#else
+      NOTREACHED();
+#endif
       break;
     case VIEW_ID_BOOKMARK_BAR:
       active_bookmark_bar_ = static_cast<BookmarkBarView*>(view);
@@ -229,13 +243,13 @@ void BrowserViewLayout::ViewAdded(views::View* host, views::View* view) {
       toolbar_ = static_cast<ToolbarView*>(view);
       break;
     case VIEW_ID_TAB_STRIP:
-      tabstrip_ = static_cast<BaseTabStrip*>(view);
+      tabstrip_ = static_cast<AbstractTabStripView*>(view);
       break;
   }
 }
 
 void BrowserViewLayout::ViewRemoved(views::View* host, views::View* view) {
-  switch (view->GetID()) {
+  switch (view->id()) {
     case VIEW_ID_BOOKMARK_BAR:
       active_bookmark_bar_ = NULL;
       break;
@@ -243,11 +257,11 @@ void BrowserViewLayout::ViewRemoved(views::View* host, views::View* view) {
 }
 
 void BrowserViewLayout::Layout(views::View* host) {
-  vertical_layout_rect_ = browser_view_->GetLocalBounds(true);
-  int top = LayoutTabStrip();
-  if (browser_view_->IsTabStripVisible() && !browser_view_->UseVerticalTabs()) {
+  vertical_layout_rect_ = browser_view_->GetLocalBounds();
+  int top = LayoutTabStripRegion();
+  if (browser_view_->IsTabStripVisible()) {
     tabstrip_->SetBackgroundOffset(gfx::Point(
-        tabstrip_->MirroredX() + browser_view_->MirroredX(),
+        tabstrip_->GetMirroredX() + browser_view_->GetMirroredX(),
         browser_view_->frame()->GetHorizontalTabStripVerticalOffset(false)));
   }
   top = LayoutToolbar(top);
@@ -284,41 +298,38 @@ const Browser* BrowserViewLayout::browser() const {
   return browser_view_->browser();
 }
 
-int BrowserViewLayout::LayoutTabStrip() {
+int BrowserViewLayout::LayoutTabStripRegion() {
   if (!browser_view_->IsTabStripVisible()) {
     tabstrip_->SetVisible(false);
     tabstrip_->SetBounds(0, 0, 0, 0);
     return 0;
   }
 
+  // This retrieves the bounds for the tab strip based on whether or not we show
+  // anything to the left of it, like the incognito avatar.
   gfx::Rect tabstrip_bounds(
       browser_view_->frame()->GetBoundsForTabStrip(tabstrip_));
   gfx::Point tabstrip_origin(tabstrip_bounds.origin());
-  views::View::ConvertPointToView(browser_view_->GetParent(), browser_view_,
+  views::View::ConvertPointToView(browser_view_->parent(), browser_view_,
                                   &tabstrip_origin);
   tabstrip_bounds.set_origin(tabstrip_origin);
 
-  if (browser_view_->UseVerticalTabs())
-    vertical_layout_rect_.Inset(tabstrip_bounds.width(), 0, 0, 0);
-
   tabstrip_->SetVisible(true);
-  tabstrip_->SetBounds(tabstrip_bounds);
-  return browser_view_->UseVerticalTabs() ?
-      tabstrip_bounds.y() : tabstrip_bounds.bottom();
+  tabstrip_->SetBoundsRect(tabstrip_bounds);
+  return tabstrip_bounds.bottom();
 }
 
 int BrowserViewLayout::LayoutToolbar(int top) {
   int browser_view_width = vertical_layout_rect_.width();
-  bool visible = browser_view_->IsToolbarVisible();
-  toolbar_->location_bar()->SetFocusable(visible);
+  bool toolbar_visible = browser_view_->IsToolbarVisible();
+  toolbar_->location_bar()->SetLocationEntryFocusable(toolbar_visible);
   int y = top;
-  if (!browser_view_->UseVerticalTabs()) {
-    y -= ((visible && browser_view_->IsTabStripVisible()) ?
-          kToolbarTabStripVerticalOverlap : 0);
-  }
-  int height = visible ? toolbar_->GetPreferredSize().height() : 0;
-  toolbar_->SetVisible(visible);
+  y -= (toolbar_visible && browser_view_->IsTabStripVisible()) ?
+        kToolbarTabStripVerticalOverlap : 0;
+  int height = toolbar_visible ? toolbar_->GetPreferredSize().height() : 0;
+  toolbar_->SetVisible(toolbar_visible);
   toolbar_->SetBounds(vertical_layout_rect_.x(), y, browser_view_width, height);
+
   return y + height;
 }
 
@@ -348,7 +359,8 @@ int BrowserViewLayout::LayoutBookmarkBar(int top) {
 
   active_bookmark_bar_->set_infobar_visible(InfobarVisible());
   int bookmark_bar_height = active_bookmark_bar_->GetPreferredSize().height();
-  y -= kSeparationLineHeight + active_bookmark_bar_->GetToolbarOverlap(false);
+  y -= views::NonClientFrameView::kClientEdgeThickness +
+      active_bookmark_bar_->GetToolbarOverlap(false);
   active_bookmark_bar_->SetVisible(true);
   active_bookmark_bar_->SetBounds(vertical_layout_rect_.x(), y,
                                   vertical_layout_rect_.width(),
@@ -357,93 +369,46 @@ int BrowserViewLayout::LayoutBookmarkBar(int top) {
 }
 
 int BrowserViewLayout::LayoutInfoBar(int top) {
-  bool visible = InfobarVisible();
-  int height = visible ? infobar_container_->GetPreferredSize().height() : 0;
-  infobar_container_->SetVisible(visible);
-  infobar_container_->SetBounds(vertical_layout_rect_.x(), top,
-                                vertical_layout_rect_.width(), height);
-  return top + height;
-}
-
-// |browser_reserved_rect| is in browser_view_ coordinates.
-// |future_source_bounds| is in |source|'s parent coordinates.
-// |future_parent_offset| is required, since parent view is not moved yet.
-// Note that |future_parent_offset| is relative to browser_view_, not to
-// the parent view.
-void BrowserViewLayout::UpdateReservedContentsRect(
-    const gfx::Rect& browser_reserved_rect,
-    TabContentsContainer* source,
-    const gfx::Rect& future_source_bounds,
-    const gfx::Point& future_parent_offset) {
-  gfx::Point resize_corner_origin(browser_reserved_rect.origin());
-  // Convert |resize_corner_origin| from browser_view_ to source's parent
-  // coordinates.
-  views::View::ConvertPointToView(browser_view_, source->GetParent(),
-                                  &resize_corner_origin);
-  // Create |reserved_rect| in source's parent coordinates.
-  gfx::Rect reserved_rect(resize_corner_origin, browser_reserved_rect.size());
-  // Apply source's parent future offset to it.
-  reserved_rect.Offset(-future_parent_offset.x(), -future_parent_offset.y());
-  if (future_source_bounds.Intersects(reserved_rect)) {
-    // |source| is not properly positioned yet to use ConvertPointToView,
-    // so convert it into |source|'s coordinates manually.
-    reserved_rect.Offset(-future_source_bounds.x(), -future_source_bounds.y());
-  } else {
-    reserved_rect = gfx::Rect();
-  }
-
-  source->SetReservedContentsRect(reserved_rect);
+  // Raise the |infobar_container_| by its vertical overlap.
+  infobar_container_->SetVisible(InfobarVisible());
+  int height;
+  int overlapped_top = top -
+      static_cast<InfoBarContainerView*>(infobar_container_)->
+          GetVerticalOverlap(&height);
+  infobar_container_->SetBounds(vertical_layout_rect_.x(),
+                                overlapped_top,
+                                vertical_layout_rect_.width(),
+                                height);
+  return overlapped_top + height;
 }
 
 void BrowserViewLayout::LayoutTabContents(int top, int bottom) {
-  // The ultimate idea is to calcualte bounds and reserved areas for all
+  // The ultimate idea is to calculate bounds and reserved areas for all
   // contents views first and then resize them all, so every view
   // (and its contents) is resized and laid out only once.
 
   // The views hierarcy (see browser_view.h for more details):
-  // 1) Sidebar is not allowed:
-  //     contents_split_ -> [contents_container_ | devtools]
-  // 2) Sidebar is allowed:
-  //     contents_split_ ->
-  //         [sidebar_split -> [contents_container_ | sidebar]] | devtools
+  // contents_split_ -> [contents_container_ | devtools]
 
-  gfx::Rect sidebar_split_bounds;
   gfx::Rect contents_bounds;
-  gfx::Rect sidebar_bounds;
   gfx::Rect devtools_bounds;
 
   gfx::Rect contents_split_bounds(vertical_layout_rect_.x(), top,
                                   vertical_layout_rect_.width(),
                                   std::max(0, bottom - top));
-  contents_split_->CalculateChildrenBounds(
-      contents_split_bounds, &sidebar_split_bounds, &devtools_bounds);
   gfx::Point contents_split_offset(
       contents_split_bounds.x() - contents_split_->bounds().x(),
       contents_split_bounds.y() - contents_split_->bounds().y());
-  gfx::Point sidebar_split_offset(contents_split_offset);
-  sidebar_split_offset.Offset(sidebar_split_bounds.x(),
-                              sidebar_split_bounds.y());
 
-  views::SingleSplitView* sidebar_split = browser_view_->sidebar_split_;
-  if (sidebar_split) {
-    DCHECK(sidebar_split == contents_split_->GetChildViewAt(0));
-    sidebar_split->CalculateChildrenBounds(
-        sidebar_split_bounds, &contents_bounds, &sidebar_bounds);
-  } else {
-    contents_bounds = sidebar_split_bounds;
-  }
-
-  // Layout resize corner, sidebar mini tabs and calculate reserved contents
-  // rects here as all contents view bounds are already determined, but not yet
-  // set at this point, so contents will be laid out once at most.
-  // TODO(alekseys): layout sidebar minitabs and adjust reserved rect
-  // accordingly.
+  // Layout resize corner and calculate reserved contents rects here as all
+  // contents view bounds are already determined, but not yet set at this point,
+  // so contents will be laid out once at most.
   gfx::Rect browser_reserved_rect;
-  if (!browser_view_->frame_->GetWindow()->IsMaximized() &&
-      !browser_view_->frame_->GetWindow()->IsFullscreen()) {
+  if (!browser_view_->frame_->IsMaximized() &&
+      !browser_view_->frame_->IsFullscreen()) {
     gfx::Size resize_corner_size = browser_view_->GetResizeCornerSize();
     if (!resize_corner_size.IsEmpty()) {
-      gfx::Rect bounds = browser_view_->GetLocalBounds(false);
+      gfx::Rect bounds = browser_view_->GetContentsBounds();
       gfx::Point resize_corner_origin(
           bounds.right() - resize_corner_size.width(),
           bounds.bottom() - resize_corner_size.height());
@@ -452,25 +417,8 @@ void BrowserViewLayout::LayoutTabContents(int top, int bottom) {
     }
   }
 
-  UpdateReservedContentsRect(browser_reserved_rect,
-                             browser_view_->contents_container_,
-                             contents_bounds,
-                             sidebar_split_offset);
-  if (sidebar_split) {
-    UpdateReservedContentsRect(browser_reserved_rect,
-                               browser_view_->sidebar_container_,
-                               sidebar_bounds,
-                               sidebar_split_offset);
-  }
-  UpdateReservedContentsRect(browser_reserved_rect,
-                             browser_view_->devtools_container_,
-                             devtools_bounds,
-                             contents_split_offset);
-
   // Now it's safe to actually resize all contents views in the hierarchy.
-  contents_split_->SetBounds(contents_split_bounds);
-  if (sidebar_split)
-    sidebar_split->SetBounds(sidebar_split_bounds);
+  contents_split_->SetBoundsRect(contents_split_bounds);
 }
 
 int BrowserViewLayout::GetTopMarginForActiveContent() {
@@ -479,24 +427,19 @@ int BrowserViewLayout::GetTopMarginForActiveContent() {
     return 0;
   }
 
-  if (contents_split_->GetChildViewAt(1) &&
-      contents_split_->GetChildViewAt(1)->IsVisible())
+  if (contents_split_->child_at(1) && contents_split_->child_at(1)->visible())
     return 0;
 
-  if (SidebarManager::IsSidebarAllowed()) {
-    views::View* sidebar_split = contents_split_->GetChildViewAt(0);
-    if (sidebar_split->GetChildViewAt(1) &&
-        sidebar_split->GetChildViewAt(1)->IsVisible())
-      return 0;
-  }
-
   // Adjust for separator.
-  return active_bookmark_bar_->height() - kSeparationLineHeight;
+  return active_bookmark_bar_->height() -
+      views::NonClientFrameView::kClientEdgeThickness;
 }
 
 int BrowserViewLayout::LayoutDownloadShelf(int bottom) {
+#if !defined(OS_CHROMEOS) || defined(USE_AURA)
   // Re-layout the shelf either if it is visible or if it's close animation
-  // is currently running.
+  // is currently running.  ChromiumOS uses ActiveDownloadsUI instead of
+  // DownloadShelf.
   if (browser_view_->IsDownloadShelfVisible() ||
       (download_shelf_ && download_shelf_->IsClosing())) {
     bool visible = browser()->SupportsWindowFeature(
@@ -509,6 +452,7 @@ int BrowserViewLayout::LayoutDownloadShelf(int bottom) {
     download_shelf_->Layout();
     bottom -= height;
   }
+#endif
   return bottom;
 }
 
