@@ -4,21 +4,31 @@
 
 #include "chrome/browser/ui/views/frame/glass_browser_frame_view.h"
 
+#include "base/command_line.h"
+#include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/app/chrome_dll_resource.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
+#include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/themes/theme_service.h"
+#include "chrome/browser/ui/views/avatar_menu_button.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
-#include "chrome/browser/ui/views/tabs/side_tab_strip.h"
 #include "chrome/browser/ui/views/tabs/tab.h"
 #include "chrome/browser/ui/views/tabs/tab_strip.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/icon_util.h"
-#include "grit/app_resources.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "chrome/common/chrome_switches.h"
+#include "chrome/common/pref_names.h"
+#include "content/public/browser/notification_service.h"
+#include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
+#include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/base/theme_provider.h"
-#include "views/window/client_view.h"
-#include "views/window/window_resources.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/icon_util.h"
+#include "ui/gfx/image/image.h"
+#include "ui/views/window/client_view.h"
 
 HICON GlassBrowserFrameView::throbber_icons_[
     GlassBrowserFrameView::kThrobberIconCount];
@@ -26,21 +36,19 @@ HICON GlassBrowserFrameView::throbber_icons_[
 namespace {
 // There are 3 px of client edge drawn inside the outer frame borders.
 const int kNonClientBorderThickness = 3;
-// Vertical tabs have 4 px border.
-const int kNonClientVerticalTabStripBorderThickness = 4;
 // Besides the frame border, there's another 11 px of empty space atop the
 // window in restored mode, to use to drag the window around.
 const int kNonClientRestoredExtraThickness = 11;
 // In the window corners, the resize areas don't actually expand bigger, but the
 // 16 px at the end of the top and bottom edges triggers diagonal resizing.
 const int kResizeAreaCornerSize = 16;
-// The OTR avatar ends 2 px above the bottom of the tabstrip (which, given the
+// The avatar ends 2 px above the bottom of the tabstrip (which, given the
 // way the tabstrip draws its bottom edge, will appear like a 1 px gap to the
 // user).
-const int kOTRBottomSpacing = 2;
-// There are 2 px on each side of the OTR avatar (between the frame border and
+const int kAvatarBottomSpacing = 2;
+// There are 2 px on each side of the avatar (between the frame border and
 // it on the left, and between it and the tabstrip on the right).
-const int kOTRSideSpacing = 2;
+const int kAvatarSideSpacing = 2;
 // The content left/right images have a shadow built into them.
 const int kContentEdgeShadowThickness = 2;
 // The top 1 px of the tabstrip is shadow; in maximized mode we push this off
@@ -61,13 +69,17 @@ const int kNewTabCaptionMaximizedSpacing = 16;
 
 GlassBrowserFrameView::GlassBrowserFrameView(BrowserFrame* frame,
                                              BrowserView* browser_view)
-    : BrowserNonClientFrameView(),
-      frame_(frame),
-      browser_view_(browser_view),
+    : BrowserNonClientFrameView(frame, browser_view),
       throbber_running_(false),
       throbber_frame_(0) {
-  if (frame_->GetWindow()->GetDelegate()->ShouldShowWindowIcon())
+  if (browser_view->ShouldShowWindowIcon())
     InitThrobberIcons();
+
+  UpdateAvatarInfo();
+  if (!browser_view->IsOffTheRecord()) {
+    registrar_.Add(this, chrome::NOTIFICATION_PROFILE_CACHED_INFO_CHANGED,
+                   content::NotificationService::AllSources());
+  }
 }
 
 GlassBrowserFrameView::~GlassBrowserFrameView() {
@@ -77,17 +89,11 @@ GlassBrowserFrameView::~GlassBrowserFrameView() {
 // GlassBrowserFrameView, BrowserNonClientFrameView implementation:
 
 gfx::Rect GlassBrowserFrameView::GetBoundsForTabStrip(
-    BaseTabStrip* tabstrip) const {
-  if (browser_view_->UseVerticalTabs()) {
-    gfx::Size ps = tabstrip->GetPreferredSize();
-    return gfx::Rect(NonClientBorderThickness(),
-        NonClientTopBorderHeight(false, false), ps.width(),
-        browser_view_->height());
-  }
+    views::View* tabstrip) const {
   int minimize_button_offset =
-      std::min(frame_->GetMinimizeButtonOffset(), width());
-  int tabstrip_x = browser_view_->ShouldShowOffTheRecordAvatar() ?
-      (otr_avatar_bounds_.right() + kOTRSideSpacing) :
+      std::min(frame()->GetMinimizeButtonOffset(), width());
+  int tabstrip_x = browser_view()->ShouldShowAvatar() ?
+      (avatar_bounds_.right() + kAvatarSideSpacing) :
       NonClientBorderThickness();
   // In RTL languages, we have moved an avatar icon left by the size of window
   // controls to prevent it from being rendered over them. So, we use its x
@@ -95,22 +101,21 @@ gfx::Rect GlassBrowserFrameView::GetBoundsForTabStrip(
   // a tab strip until the left end of this window without considering the size
   // of window controls in RTL languages.
   if (base::i18n::IsRTL()) {
-    if (!browser_view_->ShouldShowOffTheRecordAvatar() &&
-        frame_->GetWindow()->IsMaximized())
-      tabstrip_x += otr_avatar_bounds_.x();
+    if (!browser_view()->ShouldShowAvatar() && frame()->IsMaximized())
+      tabstrip_x += avatar_bounds_.x();
     minimize_button_offset = width();
   }
   int tabstrip_width = minimize_button_offset - tabstrip_x -
-      (frame_->GetWindow()->IsMaximized() ?
+      (frame()->IsMaximized() ?
           kNewTabCaptionMaximizedSpacing : kNewTabCaptionRestoredSpacing);
   return gfx::Rect(tabstrip_x, GetHorizontalTabStripVerticalOffset(false),
                    std::max(0, tabstrip_width),
-                   tabstrip->GetPreferredHeight());
+                   tabstrip->GetPreferredSize().height());
 }
 
 int GlassBrowserFrameView::GetHorizontalTabStripVerticalOffset(
     bool restored) const {
-  return NonClientTopBorderHeight(restored, true);
+  return NonClientTopBorderHeight(restored);
 }
 
 void GlassBrowserFrameView::UpdateThrobber(bool running) {
@@ -125,6 +130,28 @@ void GlassBrowserFrameView::UpdateThrobber(bool running) {
   }
 }
 
+gfx::Size GlassBrowserFrameView::GetMinimumSize() {
+  gfx::Size min_size(browser_view()->GetMinimumSize());
+
+  // Account for the client area insets.
+  gfx::Insets insets = GetClientAreaInsets();
+  min_size.Enlarge(insets.width(), insets.height());
+  // Client area insets do not include the shadow thickness.
+  min_size.Enlarge(2 * kContentEdgeShadowThickness, 0);
+
+  // Ensure that the minimum width is enough to hold a tab strip with minimum
+  // width at its usual insets.
+  if (browser_view()->IsTabStripVisible()) {
+    AbstractTabStripView* tabstrip = browser_view()->tabstrip();
+    int min_tabstrip_width = tabstrip->GetMinimumSize().width();
+    int min_tabstrip_area_width =
+        width() - GetBoundsForTabStrip(tabstrip).width() + min_tabstrip_width;
+    min_size.set_width(std::max(min_tabstrip_area_width, min_size.width()));
+  }
+
+  return min_size;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // GlassBrowserFrameView, views::NonClientFrameView implementation:
 
@@ -132,14 +159,10 @@ gfx::Rect GlassBrowserFrameView::GetBoundsForClientView() const {
   return client_view_bounds_;
 }
 
-bool GlassBrowserFrameView::AlwaysUseNativeFrame() const {
-  return frame_->AlwaysUseNativeFrame();
-}
-
 gfx::Rect GlassBrowserFrameView::GetWindowBoundsForClientBounds(
     const gfx::Rect& client_bounds) const {
-  HWND hwnd = frame_->GetWindow()->GetNativeWindow();
-  if (!browser_view_->IsTabStripVisible() && hwnd) {
+  HWND hwnd = frame()->GetNativeWindow();
+  if (!browser_view()->IsTabStripVisible() && hwnd) {
     // If we don't have a tabstrip, we're either a popup or an app window, in
     // which case we have a standard size non-client area and can just use
     // AdjustWindowRectEx to obtain it. We check for a non-NULL window handle in
@@ -150,12 +173,11 @@ gfx::Rect GlassBrowserFrameView::GetWindowBoundsForClientBounds(
     return gfx::Rect(rect);
   }
 
-  int top_height = NonClientTopBorderHeight(false, false);
-  int border_thickness = NonClientBorderThickness();
-  return gfx::Rect(std::max(0, client_bounds.x() - border_thickness),
-                   std::max(0, client_bounds.y() - top_height),
-                   client_bounds.width() + (2 * border_thickness),
-                   client_bounds.height() + top_height + border_thickness);
+  gfx::Insets insets = GetClientAreaInsets();
+  return gfx::Rect(std::max(0, client_bounds.x() - insets.left()),
+                   std::max(0, client_bounds.y() - insets.top()),
+                   client_bounds.width() + insets.width(),
+                   client_bounds.height() + insets.height());
 }
 
 int GlassBrowserFrameView::NonClientHitTest(const gfx::Point& point) {
@@ -163,11 +185,15 @@ int GlassBrowserFrameView::NonClientHitTest(const gfx::Point& point) {
   // Windows can figure this out.  If the point isn't within our bounds, then
   // it's in the native portion of the frame, so again Windows can figure it
   // out.
-  if (!browser_view_->IsBrowserTypeNormal() || !bounds().Contains(point))
+  if (!browser_view()->IsBrowserTypeNormal() || !bounds().Contains(point))
     return HTNOWHERE;
 
-  int frame_component =
-      frame_->GetWindow()->GetClientView()->NonClientHitTest(point);
+  // See if the point is within the avatar menu button.
+  if (avatar_button() &&
+      avatar_button()->GetMirroredBounds().Contains(point))
+    return HTCLIENT;
+
+  int frame_component = frame()->client_view()->NonClientHitTest(point);
 
   // See if we're in the sysmenu region.  We still have to check the tabstrip
   // first so that clicks in a tab don't get treated as sysmenu clicks.
@@ -184,7 +210,7 @@ int GlassBrowserFrameView::NonClientHitTest(const gfx::Point& point) {
   int window_component = GetHTComponentForFrame(point, frame_border_thickness,
       nonclient_border_thickness, frame_border_thickness,
       kResizeAreaCornerSize - frame_border_thickness,
-      frame_->GetWindow()->GetDelegate()->CanResize());
+      frame()->widget_delegate()->CanResize());
   // Fall back to the caption if no other component matches.
   return (window_component == HTNOWHERE) ? HTCAPTION : window_component;
 }
@@ -192,63 +218,59 @@ int GlassBrowserFrameView::NonClientHitTest(const gfx::Point& point) {
 ///////////////////////////////////////////////////////////////////////////////
 // GlassBrowserFrameView, views::View overrides:
 
-void GlassBrowserFrameView::Paint(gfx::Canvas* canvas) {
-  if (!browser_view_->IsTabStripVisible())
+void GlassBrowserFrameView::OnPaint(gfx::Canvas* canvas) {
+  if (!browser_view()->IsTabStripVisible())
     return;  // Nothing is visible, so don't bother to paint.
 
   PaintToolbarBackground(canvas);
-  if (browser_view_->ShouldShowOffTheRecordAvatar())
-    PaintOTRAvatar(canvas);
-  if (!frame_->GetWindow()->IsMaximized())
+  if (!frame()->IsMaximized())
     PaintRestoredClientEdge(canvas);
 }
 
 void GlassBrowserFrameView::Layout() {
-  LayoutOTRAvatar();
+  LayoutAvatar();
   LayoutClientView();
 }
 
+bool GlassBrowserFrameView::HitTest(const gfx::Point& l) const {
+  return (avatar_button() &&
+          avatar_button()->GetMirroredBounds().Contains(l)) ||
+      !frame()->client_view()->bounds().Contains(l);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // GlassBrowserFrameView, private:
 
 int GlassBrowserFrameView::FrameBorderThickness() const {
-  views::Window* window = frame_->GetWindow();
-  return (window->IsMaximized() || window->IsFullscreen()) ?
+  return (frame()->IsMaximized() || frame()->IsFullscreen()) ?
       0 : GetSystemMetrics(SM_CXSIZEFRAME);
 }
 
 int GlassBrowserFrameView::NonClientBorderThickness() const {
-  views::Window* window = frame_->GetWindow();
-  if (window->IsMaximized() || window->IsFullscreen())
+  if (frame()->IsMaximized() || frame()->IsFullscreen())
     return 0;
 
-  return browser_view_->UseVerticalTabs() ?
-      kNonClientVerticalTabStripBorderThickness :
-      kNonClientBorderThickness;
+  return kNonClientBorderThickness;
 }
 
 int GlassBrowserFrameView::NonClientTopBorderHeight(
-    bool restored,
-    bool ignore_vertical_tabs) const {
-  if (!restored && frame_->GetWindow()->IsFullscreen())
+    bool restored) const {
+  if (!restored && frame()->IsFullscreen())
     return 0;
   // We'd like to use FrameBorderThickness() here, but the maximized Aero glass
   // frame has a 0 frame border around most edges and a CYSIZEFRAME-thick border
   // at the top (see AeroGlassFrame::OnGetMinMaxInfo()).
-  if (browser_view_->IsTabStripVisible() && !ignore_vertical_tabs &&
-      browser_view_->UseVerticalTabs())
-    return GetSystemMetrics(SM_CYSIZEFRAME) + GetSystemMetrics(SM_CYCAPTION);
   return GetSystemMetrics(SM_CYSIZEFRAME) +
-      ((!restored && browser_view_->IsMaximized()) ?
+      ((!restored && browser_view()->IsMaximized()) ?
       -kTabstripTopShadowThickness : kNonClientRestoredExtraThickness);
 }
 
 void GlassBrowserFrameView::PaintToolbarBackground(gfx::Canvas* canvas) {
   ui::ThemeProvider* tp = GetThemeProvider();
 
-  gfx::Rect toolbar_bounds(browser_view_->GetToolbarBounds());
+  gfx::Rect toolbar_bounds(browser_view()->GetToolbarBounds());
   gfx::Point toolbar_origin(toolbar_bounds.origin());
-  View::ConvertPointToView(browser_view_, this, &toolbar_origin);
+  View::ConvertPointToView(browser_view(), this, &toolbar_origin);
   toolbar_bounds.set_origin(toolbar_origin);
   int x = toolbar_bounds.x();
   int w = toolbar_bounds.width();
@@ -258,106 +280,51 @@ void GlassBrowserFrameView::PaintToolbarBackground(gfx::Canvas* canvas) {
   SkBitmap* toolbar_left = tp->GetBitmapNamed(IDR_CONTENT_TOP_LEFT_CORNER);
   SkBitmap* toolbar_center = tp->GetBitmapNamed(IDR_CONTENT_TOP_CENTER);
 
-  if (browser_view_->UseVerticalTabs()) {
-    gfx::Point tabstrip_origin(browser_view_->tabstrip()->bounds().origin());
-    View::ConvertPointToView(browser_view_, this, &tabstrip_origin);
-    int y = tabstrip_origin.y();
+  // Tile the toolbar image starting at the frame edge on the left and where
+  // the tabstrip is on the top.
+  int y = toolbar_bounds.y();
+  int dest_y = y + (kFrameShadowThickness * 2);
+  canvas->TileImageInt(*theme_toolbar, x,
+                       dest_y - GetHorizontalTabStripVerticalOffset(false), x,
+                       dest_y, w, theme_toolbar->height());
 
-    // Tile the toolbar image starting at the frame edge on the left and where
-    // the horizontal tabstrip would be on the top.
-    canvas->TileImageInt(*theme_toolbar, x,
-                         y - GetHorizontalTabStripVerticalOffset(false), x, y,
-                         w, theme_toolbar->height());
+  // Draw rounded corners for the tab.
+  SkBitmap* toolbar_left_mask =
+      tp->GetBitmapNamed(IDR_CONTENT_TOP_LEFT_CORNER_MASK);
+  SkBitmap* toolbar_right_mask =
+      tp->GetBitmapNamed(IDR_CONTENT_TOP_RIGHT_CORNER_MASK);
 
-    // Draw left edge.
-    int dest_y = y - kNonClientBorderThickness;
-    canvas->DrawBitmapInt(*toolbar_left, 0, 0, kNonClientBorderThickness,
-                          kNonClientBorderThickness, left_x, dest_y,
-                          kNonClientBorderThickness, kNonClientBorderThickness,
-                          false);
+  // We mask out the corners by using the DestinationIn transfer mode,
+  // which keeps the RGB pixels from the destination and the alpha from
+  // the source.
+  SkPaint paint;
+  paint.setXfermodeMode(SkXfermode::kDstIn_Mode);
 
-    // Draw center edge. We need to draw a while line above the toolbar for the
-    // image to overlay nicely.
-    int center_offset =
-        -kContentEdgeShadowThickness + kNonClientBorderThickness;
-    canvas->FillRectInt(SK_ColorWHITE, x + center_offset, y - 1,
-                        w - (2 * center_offset), 1);
-    canvas->TileImageInt(*toolbar_center, x + center_offset, dest_y,
-                         w - (2 * center_offset), toolbar_center->height());
+  // Mask out the top left corner.
+  canvas->DrawBitmapInt(*toolbar_left_mask, left_x, y, paint);
 
-    // Right edge.
-    SkBitmap* toolbar_right = tp->GetBitmapNamed(IDR_CONTENT_TOP_RIGHT_CORNER);
-    canvas->DrawBitmapInt(*toolbar_right,
-        toolbar_right->width() - kNonClientBorderThickness, 0,
-        kNonClientBorderThickness, kNonClientBorderThickness,
-        x + w - center_offset, dest_y, kNonClientBorderThickness,
-        kNonClientBorderThickness, false);
-  } else {
-    // Tile the toolbar image starting at the frame edge on the left and where
-    // the tabstrip is on the top.
-    int y = toolbar_bounds.y();
-    int dest_y = y + (kFrameShadowThickness * 2);
-    canvas->TileImageInt(*theme_toolbar, x,
-                         dest_y - GetHorizontalTabStripVerticalOffset(false), x,
-                         dest_y, w, theme_toolbar->height());
+  // Mask out the top right corner.
+  int right_x =
+      x + w + kContentEdgeShadowThickness - toolbar_right_mask->width();
+  canvas->DrawBitmapInt(*toolbar_right_mask, right_x, y, paint);
 
-    // Draw rounded corners for the tab.
-    SkBitmap* toolbar_left_mask =
-        tp->GetBitmapNamed(IDR_CONTENT_TOP_LEFT_CORNER_MASK);
-    SkBitmap* toolbar_right_mask =
-        tp->GetBitmapNamed(IDR_CONTENT_TOP_RIGHT_CORNER_MASK);
+  // Draw left edge.
+  canvas->DrawBitmapInt(*toolbar_left, left_x, y);
 
-    // We mask out the corners by using the DestinationIn transfer mode,
-    // which keeps the RGB pixels from the destination and the alpha from
-    // the source.
-    SkPaint paint;
-    paint.setXfermodeMode(SkXfermode::kDstIn_Mode);
+  // Draw center edge.
+  canvas->TileImageInt(*toolbar_center, left_x + toolbar_left->width(), y,
+      right_x - (left_x + toolbar_left->width()), toolbar_center->height());
 
-    // Mask out the top left corner.
-    canvas->DrawBitmapInt(*toolbar_left_mask, left_x, y, paint);
-
-    // Mask out the top right corner.
-    int right_x =
-        x + w + kContentEdgeShadowThickness - toolbar_right_mask->width();
-    canvas->DrawBitmapInt(*toolbar_right_mask, right_x, y, paint);
-
-    // Draw left edge.
-    canvas->DrawBitmapInt(*toolbar_left, left_x, y);
-
-    // Draw center edge.
-    canvas->TileImageInt(*toolbar_center, left_x + toolbar_left->width(), y,
-        right_x - (left_x + toolbar_left->width()), toolbar_center->height());
-
-    // Right edge.
-    canvas->DrawBitmapInt(*tp->GetBitmapNamed(IDR_CONTENT_TOP_RIGHT_CORNER),
-                          right_x, y);
-  }
+  // Right edge.
+  canvas->DrawBitmapInt(*tp->GetBitmapNamed(IDR_CONTENT_TOP_RIGHT_CORNER),
+                        right_x, y);
 
   // Draw the content/toolbar separator.
-  canvas->FillRectInt(ResourceBundle::toolbar_separator_color,
-      x + kClientEdgeThickness, toolbar_bounds.bottom() - kClientEdgeThickness,
-      w - (2 * kClientEdgeThickness), kClientEdgeThickness);
-}
-
-void GlassBrowserFrameView::PaintOTRAvatar(gfx::Canvas* canvas) {
-  // In RTL mode, the avatar icon should be looking the opposite direction.
-  canvas->Save();
-  if (base::i18n::IsRTL()) {
-    canvas->TranslateInt(width(), 0);
-    canvas->ScaleInt(-1, 1);
-  }
-
-  SkBitmap otr_avatar_icon = browser_view_->GetOTRAvatarIcon();
-  int w = otr_avatar_bounds_.width();
-  int h = otr_avatar_bounds_.height();
-  canvas->DrawBitmapInt(otr_avatar_icon, 0,
-      // Bias the rounding to select a region that's lower rather than higher,
-      // as the shadows at the image top mean the apparent center is below the
-      // real center.
-      ((otr_avatar_icon.height() - otr_avatar_bounds_.height()) + 1) / 2, w, h,
-      otr_avatar_bounds_.x(), otr_avatar_bounds_.y(), w, h, false);
-
-  canvas->Restore();
+  canvas->FillRect(ResourceBundle::toolbar_separator_color,
+      gfx::Rect(x + kClientEdgeThickness,
+                toolbar_bounds.bottom() - kClientEdgeThickness,
+                w - (2 * kClientEdgeThickness),
+                kClientEdgeThickness));
 }
 
 void GlassBrowserFrameView::PaintRestoredClientEdge(gfx::Canvas* canvas) {
@@ -366,11 +333,9 @@ void GlassBrowserFrameView::PaintRestoredClientEdge(gfx::Canvas* canvas) {
 
   // The client edges start below the toolbar upper corner images regardless
   // of how tall the toolbar itself is.
-  int client_area_top = browser_view_->UseVerticalTabs() ?
-      client_area_bounds.y() :
-      (frame_->GetWindow()->GetClientView()->y() +
-      browser_view_->GetToolbarBounds().y() +
-      tp->GetBitmapNamed(IDR_CONTENT_TOP_LEFT_CORNER)->height());
+  int client_area_top = frame()->client_view()->y() +
+      browser_view()->GetToolbarBounds().y() +
+      tp->GetBitmapNamed(IDR_CONTENT_TOP_LEFT_CORNER)->height();
   int client_area_bottom =
       std::max(client_area_top, height() - NonClientBorderThickness());
   int client_area_height = client_area_bottom - client_area_top;
@@ -398,58 +363,67 @@ void GlassBrowserFrameView::PaintRestoredClientEdge(gfx::Canvas* canvas) {
   // where not covered by the toolbar image.  NOTE: We do this after drawing the
   // images because the images are meant to alpha-blend atop the frame whereas
   // these rects are meant to be fully opaque, without anything overlaid.
-  SkColor toolbar_color = tp->GetColor(BrowserThemeProvider::COLOR_TOOLBAR);
-  canvas->FillRectInt(toolbar_color,
-      client_area_bounds.x() - kClientEdgeThickness, client_area_top,
-      kClientEdgeThickness,
-      client_area_bottom + kClientEdgeThickness - client_area_top);
-  canvas->FillRectInt(toolbar_color, client_area_bounds.x(), client_area_bottom,
-                      client_area_bounds.width(), kClientEdgeThickness);
-  canvas->FillRectInt(toolbar_color, client_area_bounds.right(),
-      client_area_top, kClientEdgeThickness,
-      client_area_bottom + kClientEdgeThickness - client_area_top);
+  SkColor toolbar_color = tp->GetColor(ThemeService::COLOR_TOOLBAR);
+  canvas->FillRect(toolbar_color,
+      gfx::Rect(client_area_bounds.x() - kClientEdgeThickness, client_area_top,
+                kClientEdgeThickness,
+                client_area_bottom + kClientEdgeThickness - client_area_top));
+  canvas->FillRect(toolbar_color,
+                   gfx::Rect(client_area_bounds.x(), client_area_bottom,
+                             client_area_bounds.width(), kClientEdgeThickness));
+  canvas->FillRect(toolbar_color, gfx::Rect(client_area_bounds.right(),
+       client_area_top, kClientEdgeThickness,
+       client_area_bottom + kClientEdgeThickness - client_area_top));
 }
 
-void GlassBrowserFrameView::LayoutOTRAvatar() {
-  int otr_x = NonClientBorderThickness() + kOTRSideSpacing;
+void GlassBrowserFrameView::LayoutAvatar() {
+  // Even though the avatar is used for both incognito and profiles we always
+  // use the incognito icon to layout the avatar button. The profile icon
+  // can be customized so we can't depend on its size to perform layout.
+  SkBitmap incognito_icon = browser_view()->GetOTRAvatarIcon();
+
+  int avatar_x = NonClientBorderThickness() + kAvatarSideSpacing;
   // Move this avatar icon by the size of window controls to prevent it from
   // being rendered over them in RTL languages. This code also needs to adjust
   // the width of a tab strip to avoid decreasing this size twice. (See the
   // comment in GetBoundsForTabStrip().)
   if (base::i18n::IsRTL())
-    otr_x += width() - frame_->GetMinimizeButtonOffset();
+    avatar_x += width() - frame()->GetMinimizeButtonOffset();
 
-  SkBitmap otr_avatar_icon = browser_view_->GetOTRAvatarIcon();
-  int otr_bottom, otr_restored_y;
-  if (browser_view_->UseVerticalTabs()) {
-    otr_bottom = NonClientTopBorderHeight(false, false) - kOTRBottomSpacing;
-    otr_restored_y = kFrameShadowThickness;
-  } else {
-    otr_bottom = GetHorizontalTabStripVerticalOffset(false) +
-        browser_view_->GetTabStripHeight() - kOTRBottomSpacing;
-    otr_restored_y = otr_bottom - otr_avatar_icon.height();
-  }
-  int otr_y = frame_->GetWindow()->IsMaximized() ?
-      (NonClientTopBorderHeight(false, true) + kTabstripTopShadowThickness) :
-      otr_restored_y;
-  otr_avatar_bounds_.SetRect(otr_x, otr_y, otr_avatar_icon.width(),
-      browser_view_->ShouldShowOffTheRecordAvatar() ? (otr_bottom - otr_y) : 0);
+  int avatar_bottom = GetHorizontalTabStripVerticalOffset(false) +
+      browser_view()->GetTabStripHeight() - kAvatarBottomSpacing;
+  int avatar_restored_y = avatar_bottom - incognito_icon.height();
+  int avatar_y = frame()->IsMaximized() ?
+      (NonClientTopBorderHeight(false) + kTabstripTopShadowThickness) :
+      avatar_restored_y;
+  avatar_bounds_.SetRect(avatar_x, avatar_y, incognito_icon.width(),
+      browser_view()->ShouldShowAvatar() ? (avatar_bottom - avatar_y) : 0);
+
+  if (avatar_button())
+    avatar_button()->SetBoundsRect(avatar_bounds_);
 }
 
 void GlassBrowserFrameView::LayoutClientView() {
   client_view_bounds_ = CalculateClientAreaBounds(width(), height());
 }
 
+gfx::Insets GlassBrowserFrameView::GetClientAreaInsets() const {
+  if (!browser_view()->IsTabStripVisible())
+    return gfx::Insets();
+
+  const int top_height = NonClientTopBorderHeight(false);
+  const int border_thickness = NonClientBorderThickness();
+  return gfx::Insets(top_height,
+                     border_thickness,
+                     border_thickness,
+                     border_thickness);
+}
+
 gfx::Rect GlassBrowserFrameView::CalculateClientAreaBounds(int width,
                                                            int height) const {
-  if (!browser_view_->IsTabStripVisible())
-    return gfx::Rect(0, 0, this->width(), this->height());
-
-  int top_height = NonClientTopBorderHeight(false, false);
-  int border_thickness = NonClientBorderThickness();
-  return gfx::Rect(border_thickness, top_height,
-                   std::max(0, width - (2 * border_thickness)),
-                   std::max(0, height - top_height - border_thickness));
+  gfx::Rect bounds(0, 0, width, height);
+  bounds.Inset(GetClientAreaInsets());
+  return bounds;
 }
 
 void GlassBrowserFrameView::StartThrobber() {
@@ -457,7 +431,7 @@ void GlassBrowserFrameView::StartThrobber() {
     throbber_running_ = true;
     throbber_frame_ = 0;
     InitThrobberIcons();
-    SendMessage(frame_->GetWindow()->GetNativeWindow(), WM_SETICON,
+    SendMessage(frame()->GetNativeWindow(), WM_SETICON,
                 static_cast<WPARAM>(ICON_SMALL),
                 reinterpret_cast<LPARAM>(throbber_icons_[throbber_frame_]));
   }
@@ -470,8 +444,8 @@ void GlassBrowserFrameView::StopThrobber() {
     HICON frame_icon = NULL;
 
     // Check if hosted BrowserView has a window icon to use.
-    if (browser_view_->ShouldShowWindowIcon()) {
-      SkBitmap icon = browser_view_->GetWindowIcon();
+    if (browser_view()->ShouldShowWindowIcon()) {
+      SkBitmap icon = browser_view()->GetWindowIcon();
       if (!icon.isNull())
         frame_icon = IconUtil::CreateHICONFromSkBitmap(icon);
     }
@@ -479,13 +453,13 @@ void GlassBrowserFrameView::StopThrobber() {
     // Fallback to class icon.
     if (!frame_icon) {
       frame_icon = reinterpret_cast<HICON>(GetClassLongPtr(
-          frame_->GetWindow()->GetNativeWindow(), GCLP_HICONSM));
+          frame()->GetNativeWindow(), GCLP_HICONSM));
     }
 
     // This will reset the small icon which we set in the throbber code.
     // WM_SETICON with NULL icon restores the icon for title bar but not
     // for taskbar. See http://crbug.com/29996
-    SendMessage(frame_->GetWindow()->GetNativeWindow(), WM_SETICON,
+    SendMessage(frame()->GetNativeWindow(), WM_SETICON,
                 static_cast<WPARAM>(ICON_SMALL),
                 reinterpret_cast<LPARAM>(frame_icon));
   }
@@ -493,11 +467,25 @@ void GlassBrowserFrameView::StopThrobber() {
 
 void GlassBrowserFrameView::DisplayNextThrobberFrame() {
   throbber_frame_ = (throbber_frame_ + 1) % kThrobberIconCount;
-  SendMessage(frame_->GetWindow()->GetNativeWindow(), WM_SETICON,
+  SendMessage(frame()->GetNativeWindow(), WM_SETICON,
               static_cast<WPARAM>(ICON_SMALL),
               reinterpret_cast<LPARAM>(throbber_icons_[throbber_frame_]));
 }
 
+void GlassBrowserFrameView::Observe(
+    int type,
+    const content::NotificationSource& source,
+    const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_PROFILE_CACHED_INFO_CHANGED:
+      UpdateAvatarInfo();
+      break;
+    default:
+      NOTREACHED() << "Got a notification we didn't register for!";
+      break;
+  }
+}
+
 // static
 void GlassBrowserFrameView::InitThrobberIcons() {
   static bool initialized = false;
