@@ -8,20 +8,25 @@
 
 #include "chrome/browser/ui/views/infobars/translate_infobar_base.h"
 
-class InfoBarTextButton;
-
 class TranslateMessageInfoBar : public TranslateInfoBarBase {
  public:
-  explicit TranslateMessageInfoBar(TranslateInfoBarDelegate* delegate);
-
-  virtual void Layout();
-
-  // views::ButtonListener implementation:
-  virtual void ButtonPressed(views::Button* sender, const views::Event& event);
+  TranslateMessageInfoBar(InfoBarTabHelper* owner,
+                          TranslateInfoBarDelegate* delegate);
 
  private:
+  virtual ~TranslateMessageInfoBar();
+
+  // TranslateInfoBarBase:
+  virtual void Layout() OVERRIDE;
+  virtual void ViewHierarchyChanged(bool is_add,
+                                    View* parent,
+                                    View* child) OVERRIDE;
+  virtual void ButtonPressed(views::Button* sender,
+                             const views::Event& event) OVERRIDE;
+  virtual int ContentMinimumWidth() const OVERRIDE;
+
   views::Label* label_;
-  InfoBarTextButton* button_;
+  views::TextButton* button_;
 
   DISALLOW_COPY_AND_ASSIGN(TranslateMessageInfoBar);
 };
