@@ -4,21 +4,24 @@
 
 #include "chrome/browser/ui/views/location_bar/click_handler.h"
 
-#include "chrome/browser/tab_contents/navigation_controller.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/browser/ui/views/location_bar/location_bar_view.h"
-#include "views/view.h"
+#include "content/public/browser/navigation_controller.h"
+#include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/web_contents.h"
+#include "ui/views/view.h"
+
+using content::NavigationEntry;
+using content::WebContents;
 
 ClickHandler::ClickHandler(const views::View* owner,
-                           const LocationBarView* location_bar)
+                           LocationBarView* location_bar)
     : owner_(owner),
       location_bar_(location_bar) {
 }
 
-void ClickHandler::OnMouseReleased(const views::MouseEvent& event,
-                                   bool canceled) {
-  if (canceled || !owner_->HitTest(event.location()))
+void ClickHandler::OnMouseReleased(const views::MouseEvent& event) {
+  if (!owner_->HitTest(event.location()))
     return;
 
   // Do not show page info if the user has been editing the location
@@ -26,12 +29,11 @@ void ClickHandler::OnMouseReleased(const views::MouseEvent& event,
   if (location_bar_->location_entry()->IsEditingOrEmpty())
     return;
 
-  TabContents* tab = location_bar_->GetTabContentsWrapper()->tab_contents();
-  NavigationEntry* nav_entry = tab->controller().GetActiveEntry();
+  WebContents* tab = location_bar_->GetTabContentsWrapper()->web_contents();
+  NavigationEntry* nav_entry = tab->GetController().GetActiveEntry();
   if (!nav_entry) {
     NOTREACHED();
     return;
   }
-  tab->ShowPageInfo(nav_entry->url(), nav_entry->ssl(), true);
+  tab->ShowPageInfo(nav_entry->GetURL(), nav_entry->GetSSL(), true);
 }
-
