@@ -4,8 +4,13 @@
 
 #include "chrome/browser/ui/views/location_bar/location_icon_view.h"
 
-LocationIconView::LocationIconView(const LocationBarView* location_bar)
+#include "base/utf_string_conversions.h"
+#include "grit/generated_resources.h"
+#include "ui/base/l10n/l10n_util.h"
+
+LocationIconView::LocationIconView(LocationBarView* location_bar)
     : ALLOW_THIS_IN_INITIALIZER_LIST(click_handler_(this, location_bar)) {
+  SetTooltipText(l10n_util::GetStringUTF16(IDS_TOOLTIP_LOCATION_ICON));
 }
 
 LocationIconView::~LocationIconView() {
@@ -17,7 +22,14 @@ bool LocationIconView::OnMousePressed(const views::MouseEvent& event) {
   return true;
 }
 
-void LocationIconView::OnMouseReleased(const views::MouseEvent& event,
-                                       bool canceled) {
-  click_handler_.OnMouseReleased(event, canceled);
+void LocationIconView::OnMouseReleased(const views::MouseEvent& event) {
+  click_handler_.OnMouseReleased(event);
+}
+
+void LocationIconView::ShowTooltip(bool show) {
+  if (show) {
+    SetTooltipText(l10n_util::GetStringUTF16(IDS_TOOLTIP_LOCATION_ICON));
+  } else {
+    SetTooltipText(string16());
+  }
 }
