@@ -7,7 +7,7 @@
 #pragma once
 
 #include "chrome/browser/ui/views/location_bar/click_handler.h"
-#include "views/controls/image_view.h"
+#include "ui/views/controls/image_view.h"
 
 class LocationBarView;
 namespace views {
@@ -19,12 +19,15 @@ class MouseEvent;
 // status on https pages, or a globe for other URLs.
 class LocationIconView : public views::ImageView {
  public:
-  explicit LocationIconView(const LocationBarView* location_bar);
+  explicit LocationIconView(LocationBarView* location_bar);
   virtual ~LocationIconView();
 
-  // Overridden from view.
-  virtual bool OnMousePressed(const views::MouseEvent& event);
-  virtual void OnMouseReleased(const views::MouseEvent& event, bool canceled);
+  // Overridden from views::ImageView:
+  virtual bool OnMousePressed(const views::MouseEvent& event) OVERRIDE;
+  virtual void OnMouseReleased(const views::MouseEvent& event) OVERRIDE;
+
+  // Whether we should show the tooltip for this icon or not.
+  void ShowTooltip(bool show);
 
  private:
   ClickHandler click_handler_;
