@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,12 +6,16 @@
 #define CHROME_BROWSER_UI_VIEWS_LOCATION_BAR_PAGE_ACTION_WITH_BADGE_VIEW_H_
 #pragma once
 
-#include "gfx/size.h"
-#include "views/view.h"
+#include "base/compiler_specific.h"
+#include "ui/gfx/size.h"
+#include "ui/views/view.h"
 
 class GURL;
 class PageActionImageView;
-class TabContents;
+
+namespace content {
+class WebContents;
+}
 
 // A container for the PageActionImageView plus its badge.
 class PageActionWithBadgeView : public views::View {
@@ -20,13 +24,14 @@ class PageActionWithBadgeView : public views::View {
 
   PageActionImageView* image_view() { return image_view_; }
 
-  virtual AccessibilityTypes::Role GetAccessibleRole();
-  virtual gfx::Size GetPreferredSize();
+  // View overrides:
+  virtual void GetAccessibleState(ui::AccessibleViewState* state) OVERRIDE;
+  virtual gfx::Size GetPreferredSize() OVERRIDE;
 
-  void UpdateVisibility(TabContents* contents, const GURL& url);
+  void UpdateVisibility(content::WebContents* contents, const GURL& url);
 
  private:
-  virtual void Layout();
+  virtual void Layout() OVERRIDE;
 
   // The button this view contains.
   PageActionImageView* image_view_;
