@@ -6,18 +6,16 @@
 #define CHROME_BROWSER_UI_VIEWS_LOCATION_BAR_STAR_VIEW_H_
 #pragma once
 
-#include "chrome/browser/ui/views/info_bubble.h"
-#include "views/controls/image_view.h"
+#include "ui/views/controls/image_view.h"
 
 class CommandUpdater;
-class InfoBubble;
 
 namespace views {
 class KeyEvent;
 class MouseEvent;
 }
 
-class StarView : public views::ImageView, public InfoBubbleDelegate {
+class StarView : public views::ImageView {
  public:
   explicit StarView(CommandUpdater* command_updater);
   virtual ~StarView();
@@ -27,17 +25,12 @@ class StarView : public views::ImageView, public InfoBubbleDelegate {
 
  private:
   // views::ImageView overrides:
-  virtual AccessibilityTypes::Role GetAccessibleRole();
-  virtual bool GetTooltipText(const gfx::Point& p, std::wstring* tooltip);
-  virtual bool OnMousePressed(const views::MouseEvent& event);
-  virtual void OnMouseReleased(const views::MouseEvent& event, bool canceled);
-  virtual bool OnKeyPressed(const views::KeyEvent& e);
-
-  // InfoBubbleDelegate overrides:
-  virtual void InfoBubbleClosing(InfoBubble* info_bubble,
-                                 bool closed_by_escape);
-  virtual bool CloseOnEscape();
-  virtual bool FadeInOnShow() { return false; }
+  virtual void GetAccessibleState(ui::AccessibleViewState* state) OVERRIDE;
+  virtual bool GetTooltipText(const gfx::Point& p,
+                              string16* tooltip) const OVERRIDE;
+  virtual bool OnMousePressed(const views::MouseEvent& event) OVERRIDE;
+  virtual void OnMouseReleased(const views::MouseEvent& event) OVERRIDE;
+  virtual bool OnKeyPressed(const views::KeyEvent& event) OVERRIDE;
 
   // The CommandUpdater for the Browser object that owns the location bar.
   CommandUpdater* command_updater_;
