@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,12 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "gfx/native_widget_types.h"
-#include "views/window/dialog_delegate.h"
+#include "ui/gfx/native_widget_types.h"
+#include "ui/views/window/dialog_delegate.h"
 
+namespace views {
 class MessageBoxView;
+}
 
 // A dialog box that tells the user that s/he needs to restart Chrome
 // for a change to take effect.
@@ -21,21 +23,22 @@ class RestartMessageBox : public views::DialogDelegate {
 
  protected:
   // views::DialogDelegate:
-  virtual int GetDialogButtons() const;
-  virtual std::wstring GetDialogButtonLabel(
-      MessageBoxFlags::DialogButton button) const;
-  virtual std::wstring GetWindowTitle() const;
+  virtual int GetDialogButtons() const OVERRIDE;
+  virtual string16 GetDialogButtonLabel(ui::DialogButton button) const OVERRIDE;
+  virtual string16 GetWindowTitle() const OVERRIDE;
 
-  // views::WindowDelegate:
-  virtual void DeleteDelegate();
-  virtual bool IsModal() const;
-  virtual views::View* GetContentsView();
+  // views::WidgetDelegate:
+  virtual void DeleteDelegate() OVERRIDE;
+  virtual ui::ModalType GetModalType() const OVERRIDE;
+  virtual views::View* GetContentsView() OVERRIDE;
+  virtual views::Widget* GetWidget() OVERRIDE;
+  virtual const views::Widget* GetWidget() const OVERRIDE;
 
  private:
   explicit RestartMessageBox(gfx::NativeWindow parent_window);
   virtual ~RestartMessageBox();
 
-  MessageBoxView* message_box_view_;
+  views::MessageBoxView* message_box_view_;
 
   DISALLOW_COPY_AND_ASSIGN(RestartMessageBox);
 };
