@@ -6,13 +6,12 @@
 
 #include <algorithm>
 
+#include "base/bind.h"
 #include "base/i18n/rtl.h"
 #include "base/message_loop.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/point.h"
+#include "chrome/browser/themes/theme_service.h"
 #include "googleurl/src/gurl.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
@@ -24,12 +23,13 @@
 #include "ui/base/animation/linear_animation.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/base/text/text_elider.h"
-#include "views/controls/label.h"
-#include "views/controls/scrollbar/native_scroll_bar.h"
-#include "views/screen.h"
-#include "views/widget/root_view.h"
-#include "views/widget/widget.h"
-#include "views/window/window.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/point.h"
+#include "ui/gfx/screen.h"
+#include "ui/views/controls/label.h"
+#include "ui/views/controls/scrollbar/native_scroll_bar.h"
+#include "ui/views/widget/root_view.h"
+#include "ui/views/widget/widget.h"
 
 using views::Widget;
 
@@ -82,7 +82,7 @@ class StatusBubbleViews::StatusView : public views::Label,
         popup_(popup),
         opacity_start_(0),
         opacity_end_(0),
-        theme_provider_(theme_provider) {
+        theme_service_(theme_provider) {
     ResourceBundle& rb = ResourceBundle::GetSharedInstance();
     gfx::Font font(rb.GetFont(ResourceBundle::BaseFont));
     SetFont(font);
@@ -151,12 +151,12 @@ class StatusBubbleViews::StatusView : public views::Label,
   void AnimateToState(double state);
   void AnimationEnded(const Animation* animation);
 
-  virtual void Paint(gfx::Canvas* canvas);
+  virtual void OnPaint(gfx::Canvas* canvas);
 
   BubbleStage stage_;
   BubbleStyle style_;
 
-  ScopedRunnableMethodFactory<StatusBubbleViews::StatusView> timer_factory_;
+  base::WeakPtrFactory<StatusBubbleViews::StatusView> timer_factory_;
 
   // Manager, owns us.
   StatusBubble* status_bubble_;
@@ -174,7 +174,7 @@ class StatusBubbleViews::StatusView : public views::Label,
   double opacity_end_;
 
   // Holds the theme provider of the frame that created us.
-  ui::ThemeProvider* theme_provider_;
+  ui::ThemeProvider* theme_service_;
 };
 
 void StatusBubbleViews::StatusView::SetText(const string16& text,
@@ -198,7 +198,6 @@ void StatusBubbleViews::StatusView::Show() {
   SetOpacity(1.0);
   popup_->Show();
   stage_ = BUBBLE_SHOWN;
-  PaintNow();
 }
 
 void StatusBubbleViews::StatusView::Hide() {
@@ -211,11 +210,13 @@ void StatusBubbleViews::StatusView::Hide() {
 }
 
 void StatusBubbleViews::StatusView::StartTimer(int time) {
-  if (!timer_factory_.empty())
-    timer_factory_.RevokeAll();
+  if (timer_factory_.HasWeakPtrs())
+    timer_factory_.InvalidateWeakPtrs();
 
-  MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      timer_factory_.NewRunnableMethod(&StatusBubbleViews::StatusView::OnTimer),
+  MessageLoop::current()->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(&StatusBubbleViews::StatusView::OnTimer,
+                timer_factory_.GetWeakPtr()),
       time);
 }
 
@@ -230,8 +231,8 @@ void StatusBubbleViews::StatusView::OnTimer() {
 }
 
 void StatusBubbleViews::StatusView::CancelTimer() {
-  if (!timer_factory_.empty())
-    timer_factory_.RevokeAll();
+  if (timer_factory_.HasWeakPtrs())
+    timer_factory_.InvalidateWeakPtrs();
 }
 
 void StatusBubbleViews::StatusView::RestartTimer(int delay) {
@@ -264,6 +265,7 @@ void StatusBubbleViews::StatusView::StartHiding() {
     StartTimer(kHideDelay);
   } else if (stage_ == BUBBLE_SHOWING_TIMER) {
     stage_ = BUBBLE_HIDDEN;
+    popup_->Hide();
     CancelTimer();
   } else if (stage_ == BUBBLE_SHOWING_FADE) {
     stage_ = BUBBLE_HIDING_FADE;
@@ -335,16 +337,15 @@ void StatusBubbleViews::StatusView::SetStyle(BubbleStyle style) {
   }
 }
 
-void StatusBubbleViews::StatusView::Paint(gfx::Canvas* canvas) {
+void StatusBubbleViews::StatusView::OnPaint(gfx::Canvas* canvas) {
   SkPaint paint;
   paint.setStyle(SkPaint::kFill_Style);
   paint.setFlags(SkPaint::kAntiAlias_Flag);
   SkColor toolbar_color =
-      theme_provider_->GetColor(BrowserThemeProvider::COLOR_TOOLBAR);
+      theme_service_->GetColor(ThemeService::COLOR_TOOLBAR);
   paint.setColor(toolbar_color);
 
-  gfx::Rect popup_bounds;
-  popup_->GetBounds(&popup_bounds, true);
+  gfx::Rect popup_bounds = popup_->GetWindowScreenBounds();
 
   // Figure out how to round the bubble's four corners.
   SkScalar rad[8];
@@ -415,7 +416,7 @@ void StatusBubbleViews::StatusView::Paint(gfx::Canvas* canvas) {
   SkPaint shadow_paint;
   shadow_paint.setFlags(SkPaint::kAntiAlias_Flag);
   shadow_paint.setColor(kShadowColor);
-  canvas->AsCanvasSkia()->drawPath(shadow_path, shadow_paint);
+  canvas->GetSkCanvas()->drawPath(shadow_path, shadow_paint);
 
   // Draw the bubble.
   rect.set(SkIntToScalar(kShadowThickness),
@@ -424,7 +425,7 @@ void StatusBubbleViews::StatusView::Paint(gfx::Canvas* canvas) {
            SkIntToScalar(height - kShadowThickness));
   SkPath path;
   path.addRoundRect(rect, rad, SkPath::kCW_Direction);
-  canvas->AsCanvasSkia()->drawPath(path, paint);
+  canvas->GetSkCanvas()->drawPath(path, paint);
 
   // Draw highlight text and then the text body. In order to make sure the text
   // is aligned to the right on RTL UIs, we mirror the text bounds if the
@@ -437,9 +438,9 @@ void StatusBubbleViews::StatusView::Paint(gfx::Canvas* canvas) {
                         kShadowThickness,
                         std::max(0, text_width),
                         std::max(0, text_height));
-  body_bounds.set_x(MirroredLeftPointForRect(body_bounds));
+  body_bounds.set_x(GetMirroredXForRect(body_bounds));
   SkColor text_color =
-      theme_provider_->GetColor(BrowserThemeProvider::COLOR_TAB_TEXT);
+      theme_service_->GetColor(ThemeService::COLOR_TAB_TEXT);
 
   // DrawStringInt doesn't handle alpha, so we'll do the blending ourselves.
   text_color = SkColorSetARGB(
@@ -473,7 +474,8 @@ class StatusBubbleViews::StatusViewExpander : public ui::LinearAnimation,
   }
 
   // Manage the expansion of the bubble.
-  void StartExpansion(string16 expanded_text, int current_width,
+  void StartExpansion(const string16& expanded_text,
+                      int current_width,
                       int expansion_end);
 
   // Set width of fully expanded bubble.
@@ -511,7 +513,8 @@ void StatusBubbleViews::StatusViewExpander::AnimationEnded(
 }
 
 void StatusBubbleViews::StatusViewExpander::StartExpansion(
-    string16 expanded_text, int expansion_start,
+    const string16& expanded_text,
+    int expansion_start,
     int expansion_end) {
   expanded_text_ = expanded_text;
   expansion_start_ = expansion_start;
@@ -558,20 +561,21 @@ StatusBubbleViews::~StatusBubbleViews() {
 
 void StatusBubbleViews::Init() {
   if (!popup_.get()) {
-    popup_.reset(Widget::CreatePopupWidget(Widget::Transparent,
-                                           Widget::NotAcceptEvents,
-                                           Widget::NotDeleteOnDestroy,
-                                           Widget::MirrorOriginInRTL));
+    popup_.reset(new Widget);
     views::Widget* frame = base_view_->GetWidget();
     if (!view_)
       view_ = new StatusView(this, popup_.get(), frame->GetThemeProvider());
     if (!expand_view_.get())
       expand_view_.reset(new StatusViewExpander(this, view_));
+    Widget::InitParams params(Widget::InitParams::TYPE_POPUP);
+    params.transparent = true;
+    params.accept_events = false;
+    params.ownership = views::Widget::InitParams::WIDGET_OWNS_NATIVE_WIDGET;
+    params.parent_widget = frame;
+    popup_->Init(params);
     popup_->SetOpacity(0x00);
-    popup_->Init(frame->GetNativeView(), gfx::Rect());
     popup_->SetContentsView(view_);
     Reposition();
-    popup_->Show();
   }
 }
 
@@ -593,7 +597,7 @@ gfx::Size StatusBubbleViews::GetPreferredSize() {
 
 void StatusBubbleViews::SetBounds(int x, int y, int w, int h) {
   original_position_.SetPoint(x, y);
-  position_.SetPoint(base_view_->MirroredXWithWidthInsideView(x, w), y);
+  position_.SetPoint(base_view_->GetMirroredXWithWidthInView(x, w), y);
   size_.SetSize(w, h);
   Reposition();
 }
@@ -620,9 +624,9 @@ void StatusBubbleViews::SetStatus(const string16& status_text) {
   }
 }
 
-void StatusBubbleViews::SetURL(const GURL& url, const string16& languages) {
-  languages_ = languages;
+void StatusBubbleViews::SetURL(const GURL& url, const std::string& languages) {
   url_ = url;
+  languages_ = languages;
   if (size_.IsEmpty())
     return;  // We have no bounds, don't attempt to show the popup.
 
@@ -644,15 +648,13 @@ void StatusBubbleViews::SetURL(const GURL& url, const string16& languages) {
   }
 
   // Set Elided Text corresponding to the GURL object.
-  gfx::Rect popup_bounds;
-  popup_->GetBounds(&popup_bounds, true);
+  gfx::Rect popup_bounds = popup_->GetWindowScreenBounds();
   int text_width = static_cast<int>(popup_bounds.width() -
       (kShadowThickness * 2) - kTextPositionX - kTextHorizPadding - 1);
-  url_text_ = ui::ElideUrl(url, view_->Label::font(),
-      text_width, UTF16ToWideHack(languages));
+  url_text_ = ui::ElideUrl(url, view_->Label::font(), text_width, languages);
 
   std::wstring original_url_text =
-      UTF16ToWideHack(net::FormatUrl(url, UTF16ToUTF8(languages)));
+      UTF16ToWideHack(net::FormatUrl(url, languages));
 
   // An URL is always treated as a left-to-right string. On right-to-left UIs
   // we need to explicitly mark the URL as LTR to make sure it is displayed
@@ -669,9 +671,11 @@ void StatusBubbleViews::SetURL(const GURL& url, const string16& languages) {
     if (is_expanded_ && !url.is_empty())
       ExpandBubble();
     else if (original_url_text.length() > url_text_.length())
-      MessageLoop::current()->PostDelayedTask(FROM_HERE,
-          expand_timer_factory_.NewRunnableMethod(
-          &StatusBubbleViews::ExpandBubble), kExpandHoverDelay);
+      MessageLoop::current()->PostDelayedTask(
+          FROM_HERE,
+          base::Bind(&StatusBubbleViews::ExpandBubble,
+                     expand_timer_factory_.GetWeakPtr()),
+          kExpandHoverDelay);
   }
 }
 
@@ -707,7 +711,7 @@ void StatusBubbleViews::AvoidMouse(const gfx::Point& location) {
   gfx::Point top_left;
   views::View::ConvertPointToScreen(base_view_, &top_left);
   // Border included.
-  int window_width = base_view_->GetLocalBounds(true).width();
+  int window_width = base_view_->GetLocalBounds().width();
 
   // Get the cursor position relative to the popup.
   gfx::Point relative_location = location;
@@ -755,7 +759,7 @@ void StatusBubbleViews::AvoidMouse(const gfx::Point& location) {
     // download shelf.
     gfx::NativeView widget = base_view_->GetWidget()->GetNativeView();
     gfx::Rect monitor_rect =
-        views::Screen::GetMonitorWorkAreaNearestWindow(widget);
+        gfx::Screen::GetMonitorWorkAreaNearestWindow(widget);
     const int bubble_bottom_y = top_left.y() + position_.y() + size_.height();
 
     if (bubble_bottom_y + offset > monitor_rect.height() ||
@@ -793,18 +797,17 @@ bool StatusBubbleViews::IsFrameVisible() {
   if (!frame->IsVisible())
     return false;
 
-  views::Window* window = frame->GetWindow();
+  views::Widget* window = frame->GetTopLevelWidget();
   return !window || !window->IsMinimized();
 }
 
 void StatusBubbleViews::ExpandBubble() {
   // Elide URL to maximum possible size, then check actual length (it may
   // still be too long to fit) before expanding bubble.
-  gfx::Rect popup_bounds;
-  popup_->GetBounds(&popup_bounds, true);
+  gfx::Rect popup_bounds = popup_->GetWindowScreenBounds();
   int max_status_bubble_width = GetMaxStatusBubbleWidth();
   url_text_ = ui::ElideUrl(url_, view_->Label::font(),
-      max_status_bubble_width, UTF16ToWideHack(languages_));
+      max_status_bubble_width, languages_);
   int expanded_bubble_width =std::max(GetStandardStatusBubbleWidth(),
       std::min(view_->Label::font().GetStringWidth(url_text_) +
                    (kShadowThickness * 2) + kTextPositionX +
@@ -832,6 +835,6 @@ void StatusBubbleViews::SetBubbleWidth(int width) {
 }
 
 void StatusBubbleViews::CancelExpandTimer() {
-  if (!expand_timer_factory_.empty())
-    expand_timer_factory_.RevokeAll();
+  if (expand_timer_factory_.HasWeakPtrs())
+    expand_timer_factory_.InvalidateWeakPtrs();
 }
