@@ -8,23 +8,28 @@
 
 #include "base/command_line.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/browser/tabs/tab_strip_selection_model.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/tab_contents/core_tab_helper.h"
 #include "chrome/browser/ui/view_ids.h"
 #include "chrome/browser/ui/views/tabs/tab_controller.h"
 #include "chrome/common/chrome_switches.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/favicon_size.h"
-#include "gfx/font.h"
-#include "grit/app_resources.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
+#include "ui/base/accessibility/accessible_view_state.h"
+#include "ui/base/animation/animation_container.h"
 #include "ui/base/animation/slide_animation.h"
 #include "ui/base/animation/throb_animation.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/text/text_elider.h"
 #include "ui/base/theme_provider.h"
-#include "views/controls/button/image_button.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/favicon_size.h"
+#include "ui/gfx/font.h"
+#include "ui/views/controls/button/image_button.h"
 
 // How long the pulse throb takes.
 static const int kPulseDurationMs = 200;
@@ -46,7 +51,7 @@ class TabCloseButton : public views::ImageButton {
   }
   virtual ~TabCloseButton() {}
 
-  virtual bool OnMousePressed(const views::MouseEvent& event) {
+  virtual bool OnMousePressed(const views::MouseEvent& event) OVERRIDE {
     bool handled = ImageButton::OnMousePressed(event);
     // Explicitly mark midle-mouse clicks as non-handled to ensure the tab
     // sees them.
@@ -56,20 +61,41 @@ class TabCloseButton : public views::ImageButton {
   // We need to let the parent know about mouse state so that it
   // can highlight itself appropriately. Note that Exit events
   // fire before Enter events, so this works.
-  virtual void OnMouseEntered(const views::MouseEvent& event) {
+  virtual void OnMouseEntered(const views::MouseEvent& event) OVERRIDE {
     CustomButton::OnMouseEntered(event);
-    GetParent()->OnMouseEntered(event);
+    parent()->OnMouseEntered(event);
   }
 
-  virtual void OnMouseExited(const views::MouseEvent& event) {
+  virtual void OnMouseExited(const views::MouseEvent& event) OVERRIDE {
     CustomButton::OnMouseExited(event);
-    GetParent()->OnMouseExited(event);
+    parent()->OnMouseExited(event);
   }
 
  private:
   DISALLOW_COPY_AND_ASSIGN(TabCloseButton);
 };
 
+// Draws the icon image at the center of |bounds|.
+void DrawIconCenter(gfx::Canvas* canvas,
+                    const SkBitmap& image,
+                    int image_offset,
+                    int icon_width,
+                    int icon_height,
+                    const gfx::Rect& bounds,
+                    bool filter) {
+  // Center the image within bounds.
+  int dst_x = bounds.x() - (icon_width - bounds.width()) / 2;
+  int dst_y = bounds.y() - (icon_height - bounds.height()) / 2;
+  // NOTE: the clipping is a work around for 69528, it shouldn't be necessary.
+  canvas->Save();
+  canvas->ClipRect(gfx::Rect(dst_x, dst_y, icon_width, icon_height));
+  canvas->DrawBitmapInt(image,
+                        image_offset, 0, icon_width, icon_height,
+                        dst_x, dst_y, icon_width, icon_height,
+                        filter);
+  canvas->Restore();
+}
+
 }  // namespace
 
 // static
@@ -81,25 +107,25 @@ int BaseTab::font_height_ = 0;
 // FaviconCrashAnimation
 //
 //  A custom animation subclass to manage the favicon crash animation.
-class BaseTab::FavIconCrashAnimation : public ui::LinearAnimation,
+class BaseTab::FaviconCrashAnimation : public ui::LinearAnimation,
                                        public ui::AnimationDelegate {
  public:
-  explicit FavIconCrashAnimation(BaseTab* target)
+  explicit FaviconCrashAnimation(BaseTab* target)
       : ALLOW_THIS_IN_INITIALIZER_LIST(ui::LinearAnimation(1000, 25, this)),
         target_(target) {
   }
-  virtual ~FavIconCrashAnimation() {}
+  virtual ~FaviconCrashAnimation() {}
 
   // ui::Animation overrides:
   virtual void AnimateToState(double state) {
     const double kHidingOffset = 27;
 
     if (state < .5) {
-      target_->SetFavIconHidingOffset(
+      target_->SetFaviconHidingOffset(
           static_cast<int>(floor(kHidingOffset * 2.0 * state)));
     } else {
-      target_->DisplayCrashedFavIcon();
-      target_->SetFavIconHidingOffset(
+      target_->DisplayCrashedFavicon();
+      target_->SetFaviconHidingOffset(
           static_cast<int>(
               floor(kHidingOffset - ((state - .5) * 2.0 * kHidingOffset))));
     }
@@ -107,27 +133,28 @@ class BaseTab::FavIconCrashAnimation : public ui::LinearAnimation,
 
   // ui::AnimationDelegate overrides:
   virtual void AnimationCanceled(const ui::Animation* animation) {
-    target_->SetFavIconHidingOffset(0);
+    target_->SetFaviconHidingOffset(0);
   }
 
  private:
   BaseTab* target_;
 
-  DISALLOW_COPY_AND_ASSIGN(FavIconCrashAnimation);
+  DISALLOW_COPY_AND_ASSIGN(FaviconCrashAnimation);
 };
 
 BaseTab::BaseTab(TabController* controller)
     : controller_(controller),
       closing_(false),
       dragging_(false),
+      favicon_hiding_offset_(0),
       loading_animation_frame_(0),
+      should_display_crashed_favicon_(false),
       throbber_disabled_(false),
       theme_provider_(NULL),
-      fav_icon_hiding_offset_(0),
-      should_display_crashed_favicon_(false) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(hover_controller_(this)) {
   BaseTab::InitResources();
 
-  SetID(VIEW_ID_TAB);
+  set_id(VIEW_ID_TAB);
 
   // Add the Close Button.
   close_button_ = new TabCloseButton(this);
@@ -139,7 +166,7 @@ BaseTab::BaseTab(TabController* controller)
   close_button_->SetImage(views::CustomButton::BS_PUSHED,
                           rb.GetBitmapNamed(IDR_TAB_CLOSE_P));
   close_button_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_TOOLTIP_CLOSE_TAB)));
+      l10n_util::GetStringUTF16(IDS_TOOLTIP_CLOSE_TAB));
   close_button_->SetAccessibleName(
       l10n_util::GetStringUTF16(IDS_ACCNAME_CLOSE));
   // Disable animation so that the red danger sign shows up immediately
@@ -147,31 +174,53 @@ BaseTab::BaseTab(TabController* controller)
   close_button_->SetAnimationDuration(0);
   AddChildView(close_button_);
 
-  SetContextMenuController(this);
+  set_context_menu_controller(this);
 }
 
 BaseTab::~BaseTab() {
 }
 
 void BaseTab::SetData(const TabRendererData& data) {
+  if (data_.Equals(data))
+    return;
+
   TabRendererData old(data_);
   data_ = data;
 
-  if (data_.crashed) {
-    if (!should_display_crashed_favicon_ && !IsPerformingCrashAnimation())
-      StartCrashAnimation();
+  if (data_.IsCrashed()) {
+    if (!should_display_crashed_favicon_ && !IsPerformingCrashAnimation()) {
+      // When --reload-killed-tabs is specified, then the idea is that
+      // when tab is killed, the tab has no visual indication that it
+      // died and should reload when the tab is next focused without
+      // the user seeing the killed tab page.
+      //
+      // The only exception to this is when the tab is in the
+      // foreground (i.e. when it's the selected tab), because we
+      // don't want to go into an infinite loop reloading a page that
+      // will constantly get killed, or if it's the only tab.  So this
+      // code makes it so that the favicon will only be shown for
+      // killed tabs when the tab is currently selected.
+      if (CommandLine::ForCurrentProcess()->
+          HasSwitch(switches::kReloadKilledTabs) && !IsSelected()) {
+        // If we're reloading killed tabs, we don't want to display
+        // the crashed animation at all if the process was killed and
+        // the tab wasn't the current tab.
+        if (data_.crashed_status != base::TERMINATION_STATUS_PROCESS_WAS_KILLED)
+          StartCrashAnimation();
+      } else {
+        StartCrashAnimation();
+      }
+    }
   } else {
     if (IsPerformingCrashAnimation())
       StopCrashAnimation();
-    ResetCrashedFavIcon();
+    ResetCrashedFavicon();
   }
 
-  // Sets the accessible name for the tab.
-  SetAccessibleName(data_.title);
-
   DataChanged(old);
 
   Layout();
+  SchedulePaint();
 }
 
 void BaseTab::UpdateLoadingAnimation(TabRendererData::NetworkState state) {
@@ -214,27 +263,26 @@ void BaseTab::StopPulse() {
   pulse_animation_.reset(NULL);
 }
 
-bool BaseTab::IsSelected() const {
-  return controller() ? controller()->IsTabSelected(this) : true;
+void BaseTab::set_animation_container(ui::AnimationContainer* container) {
+  animation_container_ = container;
+  hover_controller_.SetAnimationContainer(container);
 }
 
 bool BaseTab::IsCloseable() const {
   return controller() ? controller()->IsTabCloseable(this) : true;
 }
 
-void BaseTab::OnMouseEntered(const views::MouseEvent& e) {
-  if (!hover_animation_.get()) {
-    hover_animation_.reset(new ui::SlideAnimation(this));
-    hover_animation_->SetContainer(animation_container_.get());
-    hover_animation_->SetSlideDuration(kHoverDurationMs);
-  }
-  hover_animation_->SetTweenType(ui::Tween::EASE_OUT);
-  hover_animation_->Show();
+bool BaseTab::IsActive() const {
+  return controller() ? controller()->IsActiveTab(this) : true;
 }
 
-void BaseTab::OnMouseExited(const views::MouseEvent& e) {
-  hover_animation_->SetTweenType(ui::Tween::EASE_IN);
-  hover_animation_->Hide();
+bool BaseTab::IsSelected() const {
+  return controller() ? controller()->IsTabSelected(this) : true;
+}
+
+ui::ThemeProvider* BaseTab::GetThemeProvider() const {
+  ui::ThemeProvider* tp = View::GetThemeProvider();
+  return tp ? tp : theme_provider_;
 }
 
 bool BaseTab::OnMousePressed(const views::MouseEvent& event) {
@@ -242,13 +290,24 @@ bool BaseTab::OnMousePressed(const views::MouseEvent& event) {
     return false;
 
   if (event.IsOnlyLeftMouseButton()) {
-    // Store whether or not we were selected just now... we only want to be
-    // able to drag foreground tabs, so we don't start dragging the tab if
-    // it was in the background.
-    bool just_selected = !IsSelected();
-    if (just_selected)
+    TabStripSelectionModel original_selection;
+    original_selection.Copy(controller()->GetSelectionModel());
+    if (event.IsShiftDown() && event.IsControlDown()) {
+      controller()->AddSelectionFromAnchorTo(this);
+    } else if (event.IsShiftDown()) {
+      controller()->ExtendSelectionTo(this);
+    } else if (event.IsControlDown()) {
+      controller()->ToggleSelected(this);
+      if (!IsSelected()) {
+        // Don't allow dragging non-selected tabs.
+        return false;
+      }
+    } else if (!IsSelected()) {
       controller()->SelectTab(this);
-    controller()->MaybeStartDrag(this, event);
+    } else if (IsActive()) {
+      controller()->ClickActiveTab(this);
+    }
+    controller()->MaybeStartDrag(this, event, original_selection);
   }
   return true;
 }
@@ -259,7 +318,7 @@ bool BaseTab::OnMouseDragged(const views::MouseEvent& event) {
   return true;
 }
 
-void BaseTab::OnMouseReleased(const views::MouseEvent& event, bool canceled) {
+void BaseTab::OnMouseReleased(const views::MouseEvent& event) {
   if (!controller())
     return;
 
@@ -268,7 +327,7 @@ void BaseTab::OnMouseReleased(const views::MouseEvent& event, bool canceled) {
   // In some cases, ending the drag will schedule the tab for destruction; if
   // so, bail immediately, since our members are already dead and we shouldn't
   // do anything else except drop the tab where it is.
-  if (controller()->EndDrag(canceled))
+  if (controller()->EndDrag(false))
     return;
 
   // Close tab on middle click, but only if the button is released over the tab
@@ -286,28 +345,43 @@ void BaseTab::OnMouseReleased(const views::MouseEvent& event, bool canceled) {
       if (closest_tab)
         controller()->CloseTab(closest_tab);
     }
+  } else if (event.IsOnlyLeftMouseButton() && !event.IsShiftDown() &&
+             !event.IsControlDown()) {
+    // If the tab was already selected mouse pressed doesn't change the
+    // selection. Reset it now to handle the case where multiple tabs were
+    // selected.
+    controller()->SelectTab(this);
   }
 }
 
-bool BaseTab::GetTooltipText(const gfx::Point& p, std::wstring* tooltip) {
+void BaseTab::OnMouseCaptureLost() {
+  if (controller())
+    controller()->EndDrag(true);
+}
+
+void BaseTab::OnMouseEntered(const views::MouseEvent& event) {
+  hover_controller_.Show();
+}
+
+void BaseTab::OnMouseExited(const views::MouseEvent& event) {
+  hover_controller_.Hide();
+}
+
+bool BaseTab::GetTooltipText(const gfx::Point& p, string16* tooltip) const {
   if (data_.title.empty())
     return false;
 
   // Only show the tooltip if the title is truncated.
-  if (font_->GetStringWidth(data_.title) > title_bounds().width()) {
-    *tooltip = UTF16ToWide(data_.title);
+  if (font_->GetStringWidth(data_.title) > GetTitleBounds().width()) {
+    *tooltip = data_.title;
     return true;
   }
   return false;
 }
 
-AccessibilityTypes::Role BaseTab::GetAccessibleRole() {
-  return AccessibilityTypes::ROLE_PAGETAB;
-}
-
-ui::ThemeProvider* BaseTab::GetThemeProvider() {
-  ui::ThemeProvider* tp = View::GetThemeProvider();
-  return tp ? tp : theme_provider_;
+void BaseTab::GetAccessibleState(ui::AccessibleViewState* state) {
+  state->role = ui::AccessibilityTypes::ROLE_PAGETAB;
+  state->name = data_.title;
 }
 
 void BaseTab::AdvanceLoadingAnimation(TabRendererData::NetworkState old_state,
@@ -345,53 +419,44 @@ void BaseTab::AdvanceLoadingAnimation(TabRendererData::NetworkState old_state,
   } else {
     loading_animation_frame_ = 0;
   }
-  SchedulePaint();
+  ScheduleIconPaint();
 }
 
-void BaseTab::PaintIcon(gfx::Canvas* canvas, int x, int y) {
-  if (base::i18n::IsRTL()) {
-    x = width() - x -
-        (data().favicon.isNull() ? kFavIconSize : data().favicon.width());
-  }
+void BaseTab::PaintIcon(gfx::Canvas* canvas) {
+  gfx::Rect bounds = GetIconBounds();
+  if (bounds.IsEmpty())
+    return;
 
-  int favicon_x = x;
-  if (!data().favicon.isNull() && data().favicon.width() != kFavIconSize)
-    favicon_x += (data().favicon.width() - kFavIconSize) / 2;
+  bounds.set_x(GetMirroredXForRect(bounds));
 
   if (data().network_state != TabRendererData::NETWORK_STATE_NONE) {
     ui::ThemeProvider* tp = GetThemeProvider();
     SkBitmap frames(*tp->GetBitmapNamed(
         (data().network_state == TabRendererData::NETWORK_STATE_WAITING) ?
         IDR_THROBBER_WAITING : IDR_THROBBER));
-    int image_size = frames.height();
-    int image_offset = loading_animation_frame_ * image_size;
-    int dst_y = (height() - image_size) / 2;
-    canvas->Save();
-    canvas->ClipRectInt(favicon_x, dst_y, image_size, image_size);
-    canvas->DrawBitmapInt(frames, image_offset, 0, image_size,
-                          image_size, favicon_x, dst_y, image_size, image_size,
-                          false);
-    canvas->Restore();
+
+    int icon_size = frames.height();
+    int image_offset = loading_animation_frame_ * icon_size;
+    DrawIconCenter(canvas, frames, image_offset,
+                   icon_size, icon_size, bounds, false);
   } else {
     canvas->Save();
-    canvas->ClipRectInt(0, 0, width(), height());
+    canvas->ClipRect(GetLocalBounds());
     if (should_display_crashed_favicon_) {
       ResourceBundle& rb = ResourceBundle::GetSharedInstance();
-      SkBitmap crashed_fav_icon(*rb.GetBitmapNamed(IDR_SAD_FAVICON));
-      canvas->DrawBitmapInt(crashed_fav_icon, 0, 0, crashed_fav_icon.width(),
-          crashed_fav_icon.height(), favicon_x,
-          (height() - crashed_fav_icon.height()) / 2 + fav_icon_hiding_offset_,
-          kFavIconSize, kFavIconSize, true);
+      SkBitmap crashed_favicon(*rb.GetBitmapNamed(IDR_SAD_FAVICON));
+      bounds.set_y(bounds.y() + favicon_hiding_offset_);
+      DrawIconCenter(canvas, crashed_favicon, 0,
+                     crashed_favicon.width(),
+                     crashed_favicon.height(), bounds, true);
     } else {
       if (!data().favicon.isNull()) {
         // TODO(pkasting): Use code in tab_icon_view.cc:PaintIcon() (or switch
         // to using that class to render the favicon).
-        int size = data().favicon.width();
-        canvas->DrawBitmapInt(data().favicon, 0, 0,
-                              data().favicon.width(),
-                              data().favicon.height(),
-                              x, y + fav_icon_hiding_offset_, size, size,
-                              true);
+        DrawIconCenter(canvas, data().favicon, 0,
+                       data().favicon.width(),
+                       data().favicon.height(),
+                       bounds, true);
       }
     }
     canvas->Restore();
@@ -400,18 +465,25 @@ void BaseTab::PaintIcon(gfx::Canvas* canvas, int x, int y) {
 
 void BaseTab::PaintTitle(gfx::Canvas* canvas, SkColor title_color) {
   // Paint the Title.
+  const gfx::Rect& title_bounds = GetTitleBounds();
   string16 title = data().title;
+
   if (title.empty()) {
     title = data().loading ?
         l10n_util::GetStringUTF16(IDS_TAB_LOADING_TITLE) :
-        TabContents::GetDefaultTitle();
+        CoreTabHelper::GetDefaultTitle();
   } else {
     Browser::FormatTitleForDisplay(&title);
   }
 
+#if defined(OS_WIN)
+  canvas->AsCanvasSkia()->DrawFadeTruncatingString(title,
+      gfx::CanvasSkia::TruncateFadeTail, 0, *font_, title_color, title_bounds);
+#else
   canvas->DrawStringInt(title, *font_, title_color,
-                        title_bounds().x(), title_bounds().y(),
-                        title_bounds().width(), title_bounds().height());
+                        title_bounds.x(), title_bounds.y(),
+                        title_bounds.width(), title_bounds.height());
+#endif
 }
 
 void BaseTab::AnimationProgressed(const ui::Animation* animation) {
@@ -431,29 +503,41 @@ void BaseTab::ButtonPressed(views::Button* sender, const views::Event& event) {
   controller()->CloseTab(this);
 }
 
-void BaseTab::ShowContextMenu(views::View* source,
-                              const gfx::Point& p,
-                              bool is_mouse_gesture) {
-  if (controller())
-    controller()->ShowContextMenu(this, p);
+void BaseTab::ShowContextMenuForView(views::View* source,
+                                     const gfx::Point& p,
+                                     bool is_mouse_gesture) {
+  if (controller() && !closing())
+    controller()->ShowContextMenuForTab(this, p);
 }
 
-void BaseTab::SetFavIconHidingOffset(int offset) {
-  fav_icon_hiding_offset_ = offset;
-  SchedulePaint();
+int BaseTab::loading_animation_frame() const {
+  return loading_animation_frame_;
 }
 
-void BaseTab::DisplayCrashedFavIcon() {
+bool BaseTab::should_display_crashed_favicon() const {
+  return should_display_crashed_favicon_;
+}
+
+int BaseTab::favicon_hiding_offset() const {
+  return favicon_hiding_offset_;
+}
+
+void BaseTab::SetFaviconHidingOffset(int offset) {
+  favicon_hiding_offset_ = offset;
+  ScheduleIconPaint();
+}
+
+void BaseTab::DisplayCrashedFavicon() {
   should_display_crashed_favicon_ = true;
 }
 
-void BaseTab::ResetCrashedFavIcon() {
+void BaseTab::ResetCrashedFavicon() {
   should_display_crashed_favicon_ = false;
 }
 
 void BaseTab::StartCrashAnimation() {
   if (!crash_animation_.get())
-    crash_animation_.reset(new FavIconCrashAnimation(this));
+    crash_animation_.reset(new FaviconCrashAnimation(this));
   crash_animation_->Stop();
   crash_animation_->Start();
 }
@@ -468,6 +552,19 @@ bool BaseTab::IsPerformingCrashAnimation() const {
   return crash_animation_.get() && crash_animation_->is_animating();
 }
 
+void BaseTab::ScheduleIconPaint() {
+  gfx::Rect bounds = GetIconBounds();
+  if (bounds.IsEmpty())
+    return;
+
+  // Extends the area to the bottom when sad_favicon is
+  // animating.
+  if (IsPerformingCrashAnimation())
+    bounds.set_height(height() - bounds.y());
+  bounds.set_x(GetMirroredXForRect(bounds));
+  SchedulePaintInRect(bounds);
+}
+
 // static
 void BaseTab::InitResources() {
   static bool initialized = false;
