@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,25 +8,25 @@
 
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/defaults.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/favicon_size.h"
-#include "gfx/font.h"
-#include "gfx/path.h"
-#include "gfx/skbitmap_operations.h"
-#include "grit/app_resources.h"
+#include "chrome/browser/themes/theme_service.h"
+#include "chrome/browser/ui/tabs/tab_resources.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "grit/ui_resources.h"
 #include "third_party/skia/include/effects/SkGradientShader.h"
 #include "ui/base/animation/multi_animation.h"
-#include "ui/base/animation/slide_animation.h"
 #include "ui/base/animation/throb_animation.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/controls/button/image_button.h"
-#include "views/widget/tooltip_manager.h"
-#include "views/widget/widget.h"
-#include "views/window/non_client_view.h"
-#include "views/window/window.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/favicon_size.h"
+#include "ui/gfx/font.h"
+#include "ui/gfx/path.h"
+#include "ui/gfx/skbitmap_operations.h"
+#include "ui/views/controls/button/image_button.h"
+#include "ui/views/widget/tooltip_manager.h"
+#include "ui/views/widget/widget.h"
+#include "ui/views/window/non_client_view.h"
 
 static const int kLeftPadding = 16;
 static const int kTopPadding = 6;
@@ -34,10 +34,11 @@ static const int kRightPadding = 15;
 static const int kBottomPadding = 5;
 static const int kDropShadowHeight = 2;
 static const int kToolbarOverlap = 1;
-static const int kFavIconTitleSpacing = 4;
-static const int kTitleCloseButtonSpacing = 5;
+static const int kFaviconTitleSpacing = 4;
+static const int kTitleCloseButtonSpacing = 3;
 static const int kStandardTitleWidth = 175;
 static const int kCloseButtonVertFuzz = 0;
+static const int kTabIconSize = gfx::kFaviconSize;
 static const int kCloseButtonHorzFuzz = 5;
 
 // Vertical adjustment to the favicon when the tab has a large icon.
@@ -52,6 +53,13 @@ static const int kMiniTabRendererAsNormalTabWidth =
 
 // How opaque to make the hover state (out of 1).
 static const double kHoverOpacity = 0.33;
+static const double kHoverSlideOpacity = 0.5;
+
+// Opacity for non-active selected tabs.
+static const double kSelectedTabOpacity = .45;
+
+// Selected (but not active) tabs have their throb value scaled down by this.
+static const double kSelectedTabThrobScale = .5;
 
 Tab::TabImage Tab::tab_alpha_ = {0};
 Tab::TabImage Tab::tab_active_ = {0};
@@ -77,11 +85,6 @@ static const SkColor kMiniTitleChangeGradientColor1 = SK_ColorWHITE;
 static const SkColor kMiniTitleChangeGradientColor2 =
     SkColorSetARGB(0, 255, 255, 255);
 
-// Hit mask constants.
-static const SkScalar kTabCapWidth = 15;
-static const SkScalar kTabTopCurveWidth = 4;
-static const SkScalar kTabBottomCurveWidth = 3;
-
 // static
 const char Tab::kViewClassName[] = "browser/tabs/Tab";
 
@@ -92,7 +95,7 @@ Tab::Tab(TabController* controller)
     : BaseTab(controller),
       showing_icon_(false),
       showing_close_button_(false),
-      close_button_color_(NULL) {
+      close_button_color_(0) {
   InitTabResources();
 }
 
@@ -142,7 +145,7 @@ gfx::Size Tab::GetMinimumUnselectedSize() {
 // static
 gfx::Size Tab::GetMinimumSelectedSize() {
   gfx::Size minimum_size = GetMinimumUnselectedSize();
-  minimum_size.set_width(kLeftPadding + kFavIconSize + kRightPadding);
+  minimum_size.set_width(kLeftPadding + gfx::kFaviconSize + kRightPadding);
   return minimum_size;
 }
 
@@ -150,7 +153,7 @@ gfx::Size Tab::GetMinimumSelectedSize() {
 gfx::Size Tab::GetStandardSize() {
   gfx::Size standard_size = GetMinimumUnselectedSize();
   standard_size.set_width(
-      standard_size.width() + kFavIconTitleSpacing + kStandardTitleWidth);
+      standard_size.width() + kFaviconTitleSpacing + kStandardTitleWidth);
   return standard_size;
 }
 
@@ -162,6 +165,14 @@ int Tab::GetMiniWidth() {
 ////////////////////////////////////////////////////////////////////////////////
 // Tab, protected:
 
+const gfx::Rect& Tab::GetTitleBounds() const {
+  return title_bounds_;
+}
+
+const gfx::Rect& Tab::GetIconBounds() const {
+  return favicon_bounds_;
+}
+
 void Tab::DataChanged(const TabRendererData& old) {
   if (data().blocked == old.blocked)
     return;
@@ -175,7 +186,7 @@ void Tab::DataChanged(const TabRendererData& old) {
 ////////////////////////////////////////////////////////////////////////////////
 // Tab, views::View overrides:
 
-void Tab::Paint(gfx::Canvas* canvas) {
+void Tab::OnPaint(gfx::Canvas* canvas) {
   // Don't paint if we're narrower than we can render correctly. (This should
   // only happen during animations).
   if (width() < GetMinimumUnselectedSize().width() && !data().mini)
@@ -191,8 +202,8 @@ void Tab::Paint(gfx::Canvas* canvas) {
 
   SkColor title_color = GetThemeProvider()->
       GetColor(IsSelected() ?
-          BrowserThemeProvider::COLOR_TAB_TEXT :
-          BrowserThemeProvider::COLOR_BACKGROUND_TAB_TEXT);
+          ThemeService::COLOR_TAB_TEXT :
+          ThemeService::COLOR_BACKGROUND_TAB_TEXT);
 
   if (!data().mini || width() > kMiniTabRendererAsNormalTabWidth)
     PaintTitle(canvas, title_color);
@@ -211,14 +222,14 @@ void Tab::Paint(gfx::Canvas* canvas) {
 }
 
 void Tab::Layout() {
-  gfx::Rect lb = GetLocalBounds(false);
+  gfx::Rect lb = GetContentsBounds();
   if (lb.IsEmpty())
     return;
   lb.Inset(kLeftPadding, kTopPadding, kRightPadding, kBottomPadding);
 
   // The height of the content of the Tab is the largest of the favicon,
   // the title text and the close button graphic.
-  int content_height = std::max(kFavIconSize, font_height());
+  int content_height = std::max(kTabIconSize, font_height());
   gfx::Size close_button_size(close_button()->GetPreferredSize());
   content_height = std::max(content_height, close_button_size.height());
 
@@ -226,23 +237,17 @@ void Tab::Layout() {
   showing_icon_ = ShouldShowIcon();
   if (showing_icon_) {
     // Use the size of the favicon as apps use a bigger favicon size.
-    int favicon_size =
-        !data().favicon.empty() ? data().favicon.width() : kFavIconSize;
-    int favicon_top = kTopPadding + content_height / 2 - favicon_size / 2;
+    int favicon_top = kTopPadding + content_height / 2 - kTabIconSize / 2;
     int favicon_left = lb.x();
-    if (favicon_size != kFavIconSize) {
-      favicon_left -= (favicon_size - kFavIconSize) / 2;
-      favicon_top -= kAppTapFaviconVerticalAdjustment;
-    }
     favicon_bounds_.SetRect(favicon_left, favicon_top,
-                            favicon_size, favicon_size);
+                            kTabIconSize, kTabIconSize);
     if (data().mini && width() < kMiniTabRendererAsNormalTabWidth) {
       // Adjust the location of the favicon when transitioning from a normal
       // tab to a mini-tab.
       int mini_delta = kMiniTabRendererAsNormalTabWidth - GetMiniWidth();
       int ideal_delta = width() - GetMiniWidth();
       if (ideal_delta < mini_delta) {
-        int ideal_x = (GetMiniWidth() - favicon_size) / 2;
+        int ideal_x = (GetMiniWidth() - kTabIconSize) / 2;
         int x = favicon_bounds_.x() + static_cast<int>(
             (1 - static_cast<float>(ideal_delta) /
              static_cast<float>(mini_delta)) *
@@ -269,7 +274,7 @@ void Tab::Layout() {
     close_button()->SetVisible(false);
   }
 
-  int title_left = favicon_bounds_.right() + kFavIconTitleSpacing;
+  int title_left = favicon_bounds_.right() + kFaviconTitleSpacing;
   int title_top = kTopPadding + (content_height - font_height()) / 2;
   // Size the Title text to fill the remaining space.
   if (!data().mini || width() >= kMiniTabRendererAsNormalTabWidth) {
@@ -282,7 +287,7 @@ void Tab::Layout() {
       title_top -= (text_height - minimum_size.height()) / 2;
 
     int title_width;
-    if (close_button()->IsVisible()) {
+    if (close_button()->visible()) {
       title_width = std::max(close_button()->x() -
                              kTitleCloseButtonSpacing - title_left, 0);
     } else {
@@ -295,70 +300,45 @@ void Tab::Layout() {
 
   // Certain UI elements within the Tab (the favicon, etc.) are not represented
   // as child Views (which is the preferred method).  Instead, these UI elements
-  // are drawn directly on the canvas from within Tab::Paint(). The Tab's child
-  // Views (for example, the Tab's close button which is a views::Button
+  // are drawn directly on the canvas from within Tab::OnPaint(). The Tab's
+  // child Views (for example, the Tab's close button which is a views::Button
   // instance) are automatically mirrored by the mirroring infrastructure in
   // views. The elements Tab draws directly on the canvas need to be manually
   // mirrored if the View's layout is right-to-left.
-  title_bounds_.set_x(MirroredLeftPointForRect(title_bounds_));
+  title_bounds_.set_x(GetMirroredXForRect(title_bounds_));
 }
 
 void Tab::OnThemeChanged() {
   LoadTabImages();
 }
 
+std::string Tab::GetClassName() const {
+  return kViewClassName;
+}
+
 bool Tab::HasHitTestMask() const {
   return true;
 }
 
 void Tab::GetHitTestMask(gfx::Path* path) const {
-  DCHECK(path);
-
-  SkScalar h = SkIntToScalar(height());
-  SkScalar w = SkIntToScalar(width());
-
-  path->moveTo(0, h);
-
-  // Left end cap.
-  path->lineTo(kTabBottomCurveWidth, h - kTabBottomCurveWidth);
-  path->lineTo(kTabCapWidth - kTabTopCurveWidth, kTabTopCurveWidth);
-  path->lineTo(kTabCapWidth, 0);
-
-  // Connect to the right cap.
-  path->lineTo(w - kTabCapWidth, 0);
-
-  // Right end cap.
-  path->lineTo(w - kTabCapWidth + kTabTopCurveWidth, kTabTopCurveWidth);
-  path->lineTo(w - kTabBottomCurveWidth, h - kTabBottomCurveWidth);
-  path->lineTo(w, h);
-
-  // Close out the path.
-  path->lineTo(0, h);
-  path->close();
+  TabResources::GetHitTestMask(width(), height(), path);
 }
 
-bool Tab::GetTooltipTextOrigin(const gfx::Point& p, gfx::Point* origin) {
-  origin->set_x(title_bounds().x() + 10);
+bool Tab::GetTooltipTextOrigin(const gfx::Point& p, gfx::Point* origin) const {
+  origin->set_x(title_bounds_.x() + 10);
   origin->set_y(-views::TooltipManager::GetTooltipHeight() - 4);
   return true;
 }
 
-void Tab::OnMouseMoved(const views::MouseEvent& e) {
-  hover_point_ = e.location();
-  // We need to redraw here because otherwise the hover glow does not update
-  // and follow the new mouse position.
-  SchedulePaint();
+void Tab::OnMouseMoved(const views::MouseEvent& event) {
+  hover_controller().SetLocation(event.location());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Tab, private
 
-void Tab::PaintIcon(gfx::Canvas* canvas) {
-  BaseTab::PaintIcon(canvas, favicon_bounds_.x(), favicon_bounds_.y());
-}
-
 void Tab::PaintTabBackground(gfx::Canvas* canvas) {
-  if (IsSelected()) {
+  if (IsActive()) {
     PaintActiveTabBackground(canvas);
   } else {
     if (mini_title_animation_.get() && mini_title_animation_->is_animating())
@@ -370,8 +350,8 @@ void Tab::PaintTabBackground(gfx::Canvas* canvas) {
     if (throb_value > 0) {
       canvas->SaveLayerAlpha(static_cast<int>(throb_value * 0xff),
                              gfx::Rect(width(), height()));
-      canvas->AsCanvasSkia()->drawARGB(0, 255, 255, 255,
-                                       SkXfermode::kClear_Mode);
+      canvas->GetSkCanvas()->drawARGB(0, 255, 255, 255,
+                                      SkXfermode::kClear_Mode);
       PaintActiveTabBackground(canvas);
       canvas->Restore();
     }
@@ -380,13 +360,13 @@ void Tab::PaintTabBackground(gfx::Canvas* canvas) {
 
 void Tab::PaintInactiveTabBackgroundWithTitleChange(gfx::Canvas* canvas) {
   // Render the inactive tab background. We'll use this for clipping.
-  gfx::CanvasSkia background_canvas(width(), height(), false);
+  gfx::CanvasSkia background_canvas(size(), false);
   PaintInactiveTabBackground(&background_canvas);
 
   SkBitmap background_image = background_canvas.ExtractBitmap();
 
   // Draw a radial gradient to hover_canvas.
-  gfx::CanvasSkia hover_canvas(width(), height(), false);
+  gfx::CanvasSkia hover_canvas(size(), false);
   int radius = kMiniTitleChangeGradientRadius;
   int x0 = width() + radius - kMiniTitleChangeInitialXOffset;
   int x1 = radius;
@@ -413,7 +393,8 @@ void Tab::PaintInactiveTabBackgroundWithTitleChange(gfx::Canvas* canvas) {
       SkShader::kClamp_TileMode);
   paint.setShader(shader);
   shader->unref();
-  hover_canvas.DrawRectInt(x - radius, -radius, radius * 2, radius * 2, paint);
+  hover_canvas.DrawRect(gfx::Rect(x - radius, -radius, radius * 2, radius * 2),
+                        paint);
 
   // Draw the radial gradient clipped to the background into hover_image.
   SkBitmap hover_image = SkBitmapOperations::CreateMaskedBitmap(
@@ -433,21 +414,17 @@ void Tab::PaintInactiveTabBackgroundWithTitleChange(gfx::Canvas* canvas) {
 }
 
 void Tab::PaintInactiveTabBackground(gfx::Canvas* canvas) {
-  bool is_otr = data().off_the_record;
-
   // The tab image needs to be lined up with the background image
   // so that it feels partially transparent.  These offsets represent the tab
   // position within the frame background image.
-  int offset = GetX(views::View::APPLY_MIRRORING_TRANSFORMATION) +
-      background_offset_.x();
+  int offset = GetMirroredX() + background_offset_.x();
 
   int tab_id;
-  if (GetWidget() &&
-      GetWidget()->GetWindow()->GetNonClientView()->UseNativeFrame()) {
+  if (GetWidget() && GetWidget()->GetTopLevelWidget()->ShouldUseNativeFrame()) {
     tab_id = IDR_THEME_TAB_BACKGROUND_V;
   } else {
-    tab_id = is_otr ? IDR_THEME_TAB_BACKGROUND_INCOGNITO :
-                      IDR_THEME_TAB_BACKGROUND;
+    tab_id = data().incognito ? IDR_THEME_TAB_BACKGROUND_INCOGNITO :
+                                IDR_THEME_TAB_BACKGROUND;
   }
 
   SkBitmap* tab_bg = GetThemeProvider()->GetBitmapNamed(tab_id);
@@ -465,7 +442,7 @@ void Tab::PaintInactiveTabBackground(gfx::Canvas* canvas) {
   // We need a CanvasSkia object to be able to extract the bitmap from.
   // We draw everything to this canvas and then output it to the canvas
   // parameter in addition to using it to mask the hover glow if needed.
-  gfx::CanvasSkia background_canvas(width(), height(), false);
+  gfx::CanvasSkia background_canvas(size(), false);
 
   // Draw left edge.  Don't draw over the toolbar, as we're not the foreground
   // tab.
@@ -503,12 +480,8 @@ void Tab::PaintInactiveTabBackground(gfx::Canvas* canvas) {
   canvas->DrawBitmapInt(background_canvas.ExtractBitmap(), 0, 0);
 
   if (!GetThemeProvider()->HasCustomImage(tab_id) &&
-      hover_animation() && hover_animation()->IsShowing()) {
-    SkBitmap hover_glow = DrawHoverGlowBitmap(width(), height());
-    // Draw the hover glow clipped to the background into hover_image.
-    SkBitmap hover_image = SkBitmapOperations::CreateMaskedBitmap(
-        hover_glow, background_canvas.ExtractBitmap());
-    canvas->DrawBitmapInt(hover_image, 0, 0);
+      hover_controller().ShouldDraw()) {
+    hover_controller().Draw(canvas, background_canvas.ExtractBitmap());
   }
 
   // Now draw the highlights/shadows around the tab edge.
@@ -523,8 +496,7 @@ void Tab::PaintInactiveTabBackground(gfx::Canvas* canvas) {
 }
 
 void Tab::PaintActiveTabBackground(gfx::Canvas* canvas) {
-  int offset = GetX(views::View::APPLY_MIRRORING_TRANSFORMATION) +
-      background_offset_.x();
+  int offset = GetMirroredX() + background_offset_.x();
   ui::ThemeProvider* tp = GetThemeProvider();
   DCHECK(tp) << "Unable to get theme provider";
 
@@ -564,50 +536,10 @@ void Tab::PaintActiveTabBackground(gfx::Canvas* canvas) {
   canvas->DrawBitmapInt(*tab_image->image_r, width() - tab_image->r_width, 0);
 }
 
-SkBitmap Tab::DrawHoverGlowBitmap(int width_input, int height_input) {
-  // Draw a radial gradient to hover_canvas so we can export the bitmap.
-  gfx::CanvasSkia hover_canvas(width_input, height_input, false);
-
-  // Draw a radial gradient to hover_canvas.
-  int radius = width() / 3;
-
-  SkPaint paint;
-  paint.setStyle(SkPaint::kFill_Style);
-  paint.setFlags(SkPaint::kAntiAlias_Flag);
-  SkPoint loc = { SkIntToScalar(hover_point_.x()),
-                  SkIntToScalar(hover_point_.y()) };
-  SkColor colors[2];
-  const ui::SlideAnimation* hover_slide = hover_animation();
-  int hover_alpha = 0;
-  if (hover_slide) {
-    hover_alpha =
-        static_cast<int>(255 * kHoverOpacity * hover_slide->GetCurrentValue());
-  }
-  colors[0] = SkColorSetARGB(hover_alpha, 255, 255, 255);
-  colors[1] = SkColorSetARGB(0, 255, 255, 255);
-  SkShader* shader = SkGradientShader::CreateRadial(
-      loc,
-      SkIntToScalar(radius),
-      colors,
-      NULL,
-      2,
-      SkShader::kClamp_TileMode);
-  // Shader can end up null when radius = 0.
-  // If so, this results in default full tab glow behavior.
-  if (shader) {
-    paint.setShader(shader);
-    shader->unref();
-    hover_canvas.DrawRectInt(hover_point_.x() - radius,
-                             hover_point_.y() - radius,
-                             radius * 2, radius * 2, paint);
-  }
-  return hover_canvas.ExtractBitmap();
-}
-
 int Tab::IconCapacity() const {
   if (height() < GetMinimumUnselectedSize().height())
     return 0;
-  return (width() - kLeftPadding - kRightPadding) / kFavIconSize;
+  return (width() - kLeftPadding - kRightPadding) / kTabIconSize;
 }
 
 bool Tab::ShouldShowIcon() const {
@@ -615,8 +547,8 @@ bool Tab::ShouldShowIcon() const {
     return true;
   if (!data().show_icon) {
     return false;
-  } else if (IsSelected()) {
-    // The selected tab clips favicon before close button.
+  } else if (IsActive()) {
+    // The active tab clips favicon before close button.
     return IconCapacity() >= 2;
   }
   // Non-selected tabs clip close button before favicon.
@@ -624,17 +556,24 @@ bool Tab::ShouldShowIcon() const {
 }
 
 bool Tab::ShouldShowCloseBox() const {
-  // The selected tab never clips close button.
-  return !data().mini && IsCloseable() &&
-      (IsSelected() || IconCapacity() >= 3);
+  // The active tab never clips close button.
+  return !data().mini && IsCloseable() && (IsActive() || IconCapacity() >= 3);
 }
 
 double Tab::GetThrobValue() {
-  if (pulse_animation() && pulse_animation()->is_animating())
-    return pulse_animation()->GetCurrentValue() * kHoverOpacity;
+  bool is_selected = IsSelected();
+  double min = is_selected ? kSelectedTabOpacity : 0;
+  double scale = is_selected ? kSelectedTabThrobScale : 1;
 
-  return hover_animation() ?
-      kHoverOpacity * hover_animation()->GetCurrentValue() : 0;
+  if (pulse_animation() && pulse_animation()->is_animating())
+    return pulse_animation()->GetCurrentValue() * kHoverOpacity * scale + min;
+
+  if (hover_controller().ShouldDraw()) {
+    return kHoverOpacity * hover_controller().GetAnimationValue() * scale +
+        min;
+  }
+
+  return is_selected ? kSelectedTabOpacity : 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
