@@ -9,30 +9,33 @@
 #include "base/metrics/stats_table.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/browser_list.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/browser_window.h"
 #include "chrome/browser/memory_purger.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/prefs/scoped_user_pref_update.h"
+#include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/views/browser_dialogs.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "ui/base/accelerators/accelerator.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/models/table_model_observer.h"
-#include "views/accelerator.h"
-#include "views/background.h"
-#include "views/controls/button/native_button.h"
-#include "views/controls/link.h"
-#include "views/controls/menu/menu.h"
-#include "views/controls/table/group_table_view.h"
-#include "views/controls/table/table_view_observer.h"
-#include "views/standard_layout.h"
-#include "views/widget/widget.h"
-#include "views/window/dialog_delegate.h"
-#include "views/window/window.h"
+#include "ui/views/background.h"
+#include "ui/views/context_menu_controller.h"
+#include "ui/views/controls/button/text_button.h"
+#include "ui/views/controls/link.h"
+#include "ui/views/controls/link_listener.h"
+#include "ui/views/controls/menu/menu.h"
+#include "ui/views/controls/table/group_table_model.h"
+#include "ui/views/controls/table/group_table_view.h"
+#include "ui/views/controls/table/table_view_observer.h"
+#include "ui/views/layout/layout_constants.h"
+#include "ui/views/widget/widget.h"
+#include "ui/views/window/dialog_delegate.h"
 
 // The task manager window default size.
 static const int kDefaultWidth = 460;
@@ -40,7 +43,7 @@ static const int kDefaultHeight = 270;
 
 // Yellow highlight used when highlighting background resources.
 static const SkColor kBackgroundResourceHighlight =
-    SkColorSetRGB(0xff,0xf1,0xcd);
+    SkColorSetRGB(0xff, 0xf1, 0xcd);
 
 namespace {
 
@@ -92,6 +95,9 @@ string16 TaskManagerTableModel::GetText(int row, int col_id) {
     case IDS_TASK_MANAGER_PAGE_COLUMN:  // Process
       return model_->GetResourceTitle(row);
 
+    case IDS_TASK_MANAGER_PROFILE_NAME_COLUMN:  // Profile Name
+      return model_->GetResourceProfileName(row);
+
     case IDS_TASK_MANAGER_NET_COLUMN:  // Net
       return model_->GetResourceNetworkUsage(row);
 
@@ -138,6 +144,9 @@ string16 TaskManagerTableModel::GetText(int row, int col_id) {
         return string16();
       return model_->GetResourceWebCoreCSSCacheSize(row);
 
+    case IDS_TASK_MANAGER_FPS_COLUMN:
+      return model_->GetResourceFPS(row);
+
     case IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN:
       if (!model_->IsResourceFirstInGroup(row))
         return string16();
@@ -222,7 +231,7 @@ bool TaskManagerTableModel::IsBackgroundResource(int row) {
 class BackgroundColorGroupTableView : public views::GroupTableView {
  public:
   BackgroundColorGroupTableView(TaskManagerTableModel* model,
-                                std::vector<ui::TableColumn> columns,
+                                const std::vector<ui::TableColumn>& columns,
                                 bool highlight_background_resources)
       : views::GroupTableView(model, columns, views::ICON_AND_TEXT,
                               false, true, true, true),
@@ -252,11 +261,10 @@ class BackgroundColorGroupTableView : public views::GroupTableView {
 };
 
 // The Task manager UI container.
-class TaskManagerView : public views::View,
-                        public views::ButtonListener,
-                        public views::DialogDelegate,
+class TaskManagerView : public views::ButtonListener,
+                        public views::DialogDelegateView,
                         public views::TableViewObserver,
-                        public views::LinkController,
+                        public views::LinkListener,
                         public views::ContextMenuController,
                         public views::Menu::Delegate {
  public:
@@ -269,42 +277,45 @@ class TaskManagerView : public views::View,
   static void Show(bool highlight_background_resources);
 
   // views::View
-  virtual void Layout();
-  virtual gfx::Size GetPreferredSize();
+  virtual void Layout() OVERRIDE;
+  virtual gfx::Size GetPreferredSize() OVERRIDE;
   virtual void ViewHierarchyChanged(bool is_add, views::View* parent,
-                                    views::View* child);
+                                    views::View* child) OVERRIDE;
 
   // ButtonListener implementation.
-  virtual void ButtonPressed(views::Button* sender, const views::Event& event);
+  virtual void ButtonPressed(views::Button* sender,
+                             const views::Event& event) OVERRIDE;
 
   // views::DialogDelegate
-  virtual bool CanResize() const;
-  virtual bool CanMaximize() const;
-  virtual bool ExecuteWindowsCommand(int command_id);
-  virtual std::wstring GetWindowTitle() const;
-  virtual std::wstring GetWindowName() const;
-  virtual int GetDialogButtons() const;
-  virtual void WindowClosing();
-  virtual views::View* GetContentsView();
+  virtual bool CanResize() const OVERRIDE;
+  virtual bool CanMaximize() const OVERRIDE;
+  virtual bool ExecuteWindowsCommand(int command_id) OVERRIDE;
+  virtual string16 GetWindowTitle() const OVERRIDE;
+  virtual std::string GetWindowName() const OVERRIDE;
+  virtual int GetDialogButtons() const OVERRIDE;
+  virtual void WindowClosing() OVERRIDE;
+  virtual views::View* GetContentsView() OVERRIDE;
 
   // views::TableViewObserver implementation.
-  virtual void OnSelectionChanged();
-  virtual void OnDoubleClick();
-  virtual void OnKeyDown(ui::KeyboardCode keycode);
+  virtual void OnSelectionChanged() OVERRIDE;
+  virtual void OnDoubleClick() OVERRIDE;
+  virtual void OnKeyDown(ui::KeyboardCode keycode) OVERRIDE;
 
-  // views::LinkController implementation.
-  virtual void LinkActivated(views::Link* source, int event_flags);
+  // views::LinkListener implementation.
+  virtual void LinkClicked(views::Link* source, int event_flags) OVERRIDE;
 
   // Called by the column picker to pick up any new stat counters that
   // may have appeared since last time.
   void UpdateStatsCounters();
 
-  // Menu::Delegate
-  virtual void ShowContextMenu(views::View* source,
-                               const gfx::Point& p,
-                               bool is_mouse_gesture);
-  virtual bool IsItemChecked(int id) const;
-  virtual void ExecuteCommand(int id);
+  // views::ContextMenuController
+  virtual void ShowContextMenuForView(views::View* source,
+                                      const gfx::Point& p,
+                                      bool is_mouse_gesture) OVERRIDE;
+
+  // views::Menu::Delegate
+  virtual bool IsItemChecked(int id) const OVERRIDE;
+  virtual void ExecuteCommand(int id) OVERRIDE;
 
  private:
   // Creates the child controls.
@@ -322,8 +333,8 @@ class TaskManagerView : public views::View,
   // Restores saved always on top state from a previous session.
   bool GetSavedAlwaysOnTopState(bool* always_on_top) const;
 
-  views::NativeButton* purge_memory_button_;
-  views::NativeButton* kill_button_;
+  views::TextButton* purge_memory_button_;
+  views::TextButton* kill_button_;
   views::Link* about_memory_link_;
   views::GroupTableView* tab_table_;
 
@@ -377,6 +388,9 @@ void TaskManagerView::Init() {
   columns_.push_back(ui::TableColumn(IDS_TASK_MANAGER_PAGE_COLUMN,
                                      ui::TableColumn::LEFT, -1, 1));
   columns_.back().sortable = true;
+  columns_.push_back(ui::TableColumn(IDS_TASK_MANAGER_PROFILE_NAME_COLUMN,
+                                     ui::TableColumn::LEFT, -1, 0));
+  columns_.back().sortable = true;
   columns_.push_back(ui::TableColumn(IDS_TASK_MANAGER_PHYSICAL_MEM_COLUMN,
                                      ui::TableColumn::RIGHT, -1, 0));
   columns_.back().sortable = true;
@@ -406,6 +420,9 @@ void TaskManagerView::Init() {
   columns_.push_back(ui::TableColumn(IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN,
                                      ui::TableColumn::RIGHT, -1, 0));
   columns_.back().sortable = true;
+  columns_.push_back(ui::TableColumn(IDS_TASK_MANAGER_FPS_COLUMN,
+                                     ui::TableColumn::RIGHT, -1, 0));
+  columns_.back().sortable = true;
   columns_.push_back(ui::TableColumn(IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN,
                                      ui::TableColumn::RIGHT, -1, 0));
   columns_.back().sortable = true;
@@ -418,6 +435,7 @@ void TaskManagerView::Init() {
       table_model_.get(), columns_, highlight_background_resources_);
 
   // Hide some columns by default
+  tab_table_->SetColumnVisibility(IDS_TASK_MANAGER_PROFILE_NAME_COLUMN, false);
   tab_table_->SetColumnVisibility(IDS_TASK_MANAGER_PROCESS_ID_COLUMN, false);
   tab_table_->SetColumnVisibility(IDS_TASK_MANAGER_SHARED_MEM_COLUMN, false);
   tab_table_->SetColumnVisibility(IDS_TASK_MANAGER_PRIVATE_MEM_COLUMN, false);
@@ -436,22 +454,23 @@ void TaskManagerView::Init() {
 
   UpdateStatsCounters();
   tab_table_->SetObserver(this);
-  tab_table_->SetContextMenuController(this);
-  SetContextMenuController(this);
+  tab_table_->set_context_menu_controller(this);
+  set_context_menu_controller(this);
   // If we're running with --purge-memory-button, add a "Purge memory" button.
   if (CommandLine::ForCurrentProcess()->HasSwitch(
       switches::kPurgeMemoryButton)) {
-    purge_memory_button_ = new views::NativeButton(this,
+    purge_memory_button_ = new views::NativeTextButton(this,
         UTF16ToWide(l10n_util::GetStringUTF16(IDS_TASK_MANAGER_PURGE_MEMORY)));
   }
-  kill_button_ = new views::NativeButton(
+  kill_button_ = new views::NativeTextButton(
       this, UTF16ToWide(l10n_util::GetStringUTF16(IDS_TASK_MANAGER_KILL)));
-  kill_button_->AddAccelerator(views::Accelerator(ui::VKEY_E,
-                                                  false, false, false));
+  kill_button_->AddAccelerator(ui::Accelerator(ui::VKEY_E, false, false,
+                                               false));
   kill_button_->SetAccessibleKeyboardShortcut(L"E");
-  about_memory_link_ = new views::Link(UTF16ToWide(
-      l10n_util::GetStringUTF16(IDS_TASK_MANAGER_ABOUT_MEMORY_LINK)));
-  about_memory_link_->SetController(this);
+  kill_button_->set_prefix_type(views::TextButtonBase::PREFIX_SHOW);
+  about_memory_link_ = new views::Link(
+      l10n_util::GetStringUTF16(IDS_TASK_MANAGER_ABOUT_MEMORY_LINK));
+  about_memory_link_->set_listener(this);
 
   // Makes sure our state is consistent.
   OnSelectionChanged();
@@ -505,31 +524,35 @@ void TaskManagerView::ViewHierarchyChanged(bool is_add,
 }
 
 void TaskManagerView::Layout() {
-  // kPanelHorizMargin is too big.
+  // views::kPanelHorizMargin is too big.
   const int kTableButtonSpacing = 12;
 
   gfx::Size size = kill_button_->GetPreferredSize();
   int prefered_width = size.width();
   int prefered_height = size.height();
 
-  tab_table_->SetBounds(x() + kPanelHorizMargin,
-                        y() + kPanelVertMargin,
-                        width() - 2 * kPanelHorizMargin,
-                        height() - 2 * kPanelVertMargin - prefered_height);
+  tab_table_->SetBounds(
+      x() + views::kPanelHorizMargin,
+      y() + views::kPanelVertMargin,
+      width() - 2 * views::kPanelHorizMargin,
+      height() - 2 * views::kPanelVertMargin - prefered_height);
 
   // y-coordinate of button top left.
-  gfx::Rect parent_bounds = GetParent()->GetLocalBounds(false);
-  int y_buttons = parent_bounds.bottom() - prefered_height - kButtonVEdgeMargin;
+  gfx::Rect parent_bounds = parent()->GetContentsBounds();
+  int y_buttons =
+      parent_bounds.bottom() - prefered_height - views::kButtonVEdgeMargin;
 
-  kill_button_->SetBounds(x() + width() - prefered_width - kPanelHorizMargin,
-                          y_buttons,
-                          prefered_width,
-                          prefered_height);
+  kill_button_->SetBounds(
+      x() + width() - prefered_width - views::kPanelHorizMargin,
+      y_buttons,
+      prefered_width,
+      prefered_height);
 
   if (purge_memory_button_) {
     size = purge_memory_button_->GetPreferredSize();
     purge_memory_button_->SetBounds(
-        kill_button_->x() - size.width() - kUnrelatedControlHorizontalSpacing,
+        kill_button_->x() - size.width() -
+            views::kUnrelatedControlHorizontalSpacing,
         y_buttons, size.width(), size.height());
   }
 
@@ -540,7 +563,7 @@ void TaskManagerView::Layout() {
   // bottom of buttons vertically.
   int link_y_offset = std::max(0, prefered_height - link_prefered_height) / 2;
   about_memory_link_->SetBounds(
-      x() + kPanelHorizMargin,
+      x() + views::kPanelHorizMargin,
       y_buttons + prefered_height - link_prefered_height - link_y_offset,
       link_prefered_width,
       link_prefered_height);
@@ -555,18 +578,18 @@ void TaskManagerView::Show(bool highlight_background_resources) {
   if (instance_) {
     if (instance_->highlight_background_resources_ !=
         highlight_background_resources) {
-      instance_->window()->Close();
+      instance_->GetWidget()->Close();
     } else {
       // If there's a Task manager window open already, just activate it.
-      instance_->window()->Activate();
+      instance_->GetWidget()->Activate();
       return;
     }
   }
   instance_ = new TaskManagerView(highlight_background_resources);
-  views::Window::CreateChromeWindow(NULL, gfx::Rect(), instance_);
+  views::Widget::CreateWindow(instance_);
   instance_->InitAlwaysOnTopState();
   instance_->model_->StartUpdating();
-  instance_->window()->Show();
+  instance_->GetWidget()->Show();
 
   // Set the initial focus to the list of tasks.
   views::FocusManager* focus_manager = instance_->GetFocusManager();
@@ -597,11 +620,12 @@ bool TaskManagerView::CanMaximize() const {
 }
 
 bool TaskManagerView::ExecuteWindowsCommand(int command_id) {
+#if defined(OS_WIN) && !defined(USE_AURA)
   if (command_id == IDC_ALWAYS_ON_TOP) {
     is_always_on_top_ = !is_always_on_top_;
 
     // Change the menu check state.
-    HMENU system_menu = GetSystemMenu(GetWindow()->GetNativeWindow(), FALSE);
+    HMENU system_menu = GetSystemMenu(GetWidget()->GetNativeWindow(), FALSE);
     MENUITEMINFO menu_info;
     memset(&menu_info, 0, sizeof(MENUITEMINFO));
     menu_info.cbSize = sizeof(MENUITEMINFO);
@@ -614,35 +638,39 @@ bool TaskManagerView::ExecuteWindowsCommand(int command_id) {
     r = SetMenuItemInfo(system_menu, IDC_ALWAYS_ON_TOP, FALSE, &menu_info);
 
     // Now change the actual window's behavior.
-    window()->SetIsAlwaysOnTop(is_always_on_top_);
+    GetWidget()->SetAlwaysOnTop(is_always_on_top_);
 
     // Save the state.
     if (g_browser_process->local_state()) {
-      DictionaryValue* window_preferences =
-          g_browser_process->local_state()->GetMutableDictionary(
-              WideToUTF8(GetWindowName()).c_str());
+      DictionaryPrefUpdate update(g_browser_process->local_state(),
+                                  GetWindowName().c_str());
+      DictionaryValue* window_preferences = update.Get();
       window_preferences->SetBoolean("always_on_top", is_always_on_top_);
     }
     return true;
   }
+#endif
   return false;
 }
 
-std::wstring TaskManagerView::GetWindowTitle() const {
-  return UTF16ToWide(l10n_util::GetStringUTF16(IDS_TASK_MANAGER_TITLE));
+string16 TaskManagerView::GetWindowTitle() const {
+  return l10n_util::GetStringUTF16(IDS_TASK_MANAGER_TITLE);
 }
 
-std::wstring TaskManagerView::GetWindowName() const {
-  return UTF8ToWide(prefs::kTaskManagerWindowPlacement);
+std::string TaskManagerView::GetWindowName() const {
+  return prefs::kTaskManagerWindowPlacement;
 }
 
 int TaskManagerView::GetDialogButtons() const {
-  return MessageBoxFlags::DIALOGBUTTON_NONE;
+  return ui::DIALOG_BUTTON_NONE;
 }
 
 void TaskManagerView::WindowClosing() {
   // Now that the window is closed, we can allow a new one to be opened.
-  instance_ = NULL;
+  // (WindowClosing comes in asynchronously from the call to Close() and we
+  // may have already opened a new instance).
+  if (instance_ == this)
+    instance_ = NULL;
   task_manager_->OnWindowClosed();
 }
 
@@ -673,15 +701,14 @@ void TaskManagerView::OnKeyDown(ui::KeyboardCode keycode) {
     ActivateFocusedTab();
 }
 
-// views::LinkController implementation
-void TaskManagerView::LinkActivated(views::Link* source, int event_flags) {
+void TaskManagerView::LinkClicked(views::Link* source, int event_flags) {
   DCHECK(source == about_memory_link_);
   task_manager_->OpenAboutMemory();
 }
 
-void TaskManagerView::ShowContextMenu(views::View* source,
-                                      const gfx::Point& p,
-                                      bool is_mouse_gesture) {
+void TaskManagerView::ShowContextMenuForView(views::View* source,
+                                             const gfx::Point& p,
+                                             bool is_mouse_gesture) {
   UpdateStatsCounters();
   scoped_ptr<views::Menu> menu(views::Menu::Create(
       this, views::Menu::TOPLEFT, source->GetWidget()->GetNativeView()));
@@ -704,7 +731,7 @@ void TaskManagerView::ExecuteCommand(int id) {
 void TaskManagerView::InitAlwaysOnTopState() {
   is_always_on_top_ = false;
   if (GetSavedAlwaysOnTopState(&is_always_on_top_))
-    window()->SetIsAlwaysOnTop(is_always_on_top_);
+    GetWidget()->SetAlwaysOnTop(is_always_on_top_);
   AddAlwaysOnTopSystemMenuItem();
 }
 
@@ -719,12 +746,13 @@ void TaskManagerView::ActivateFocusedTab() {
 }
 
 void TaskManagerView::AddAlwaysOnTopSystemMenuItem() {
+#if defined(OS_WIN) && !defined(USE_AURA)
   // The Win32 API requires that we own the text.
   always_on_top_menu_text_ =
       UTF16ToWide(l10n_util::GetStringUTF16(IDS_ALWAYS_ON_TOP));
 
   // Let's insert a menu to the window.
-  HMENU system_menu = ::GetSystemMenu(GetWindow()->GetNativeWindow(), FALSE);
+  HMENU system_menu = ::GetSystemMenu(GetWidget()->GetNativeWindow(), FALSE);
   int index = ::GetMenuItemCount(system_menu) - 1;
   if (index < 0) {
     // Paranoia check.
@@ -748,6 +776,7 @@ void TaskManagerView::AddAlwaysOnTopSystemMenuItem() {
   menu_info.wID = IDC_ALWAYS_ON_TOP;
   menu_info.dwTypeData = const_cast<wchar_t*>(always_on_top_menu_text_.c_str());
   ::InsertMenuItem(system_menu, index, TRUE, &menu_info);
+#endif
 }
 
 bool TaskManagerView::GetSavedAlwaysOnTopState(bool* always_on_top) const {
@@ -755,8 +784,7 @@ bool TaskManagerView::GetSavedAlwaysOnTopState(bool* always_on_top) const {
     return false;
 
   const DictionaryValue* dictionary =
-      g_browser_process->local_state()->GetDictionary(
-          WideToUTF8(GetWindowName()).c_str());
+      g_browser_process->local_state()->GetDictionary(GetWindowName().c_str());
   return dictionary &&
       dictionary->GetBoolean("always_on_top", always_on_top) && always_on_top;
 }
