@@ -7,48 +7,54 @@
 #include "base/i18n/number_formatting.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/accessibility/browser_accessibility_state.h"
-#include "chrome/browser/background_page_tracker.h"
-#include "chrome/browser/metrics/user_metrics.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/themes/browser_theme_provider.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/global_error_service.h"
+#include "chrome/browser/ui/global_error_service_factory.h"
 #include "chrome/browser/ui/toolbar/wrench_menu_model.h"
 #include "chrome/browser/ui/view_ids.h"
 #include "chrome/browser/ui/views/browser_actions_container.h"
 #include "chrome/browser/ui/views/event_utils.h"
-#include "chrome/browser/ui/views/frame/browser_view.h"
+#include "chrome/browser/ui/views/window.h"
+#include "chrome/browser/ui/views/wrench_menu.h"
 #include "chrome/browser/upgrade_detector.h"
-#include "chrome/common/notification_service.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
-#include "gfx/canvas.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/skbitmap_operations.h"
+#include "content/browser/accessibility/browser_accessibility_state.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/user_metrics.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "grit/theme_resources_standard.h"
+#include "ui/base/accessibility/accessible_view_state.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/controls/button/button_dropdown.h"
-#include "views/focus/view_storage.h"
-#include "views/widget/tooltip_manager.h"
-#include "views/window/non_client_view.h"
-#include "views/window/window.h"
-
-#if defined(OS_CHROMEOS)
-#include "chrome/browser/chromeos/cros/cros_library.h"
-#include "chrome/browser/chromeos/cros/update_library.h"
-#include "chrome/browser/chromeos/dom_ui/wrench_menu_ui.h"
-#include "views/controls/menu/menu_2.h"
-#endif
-#include "chrome/browser/ui/views/wrench_menu.h"
+#include "ui/base/theme_provider.h"
+#include "ui/gfx/canvas.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/skbitmap_operations.h"
+#include "ui/views/controls/button/button_dropdown.h"
+#include "ui/views/controls/menu/menu_listener.h"
+#include "ui/views/focus/view_storage.h"
+#include "ui/views/widget/tooltip_manager.h"
+#include "ui/views/window/non_client_view.h"
 
 #if defined(OS_WIN)
 #include "chrome/browser/enumerate_modules_model_win.h"
+#include "chrome/browser/ui/views/critical_notification_bubble_view.h"
+#if !defined(USE_AURA)
+#include "chrome/browser/ui/views/app_menu_button_win.h"
+#endif
 #endif
 
+using content::UserMetricsAction;
+using content::WebContents;
+
+// static
+const char ToolbarView::kViewClassName[] = "browser/ui/views/ToolbarView";
 // The space between items is 4 px in general.
 const int ToolbarView::kStandardSpacing = 4;
 // The top of the toolbar has an edge we have to skip over in addition to the 4
@@ -87,19 +93,13 @@ ToolbarView::ToolbarView(Browser* browser)
       back_(NULL),
       forward_(NULL),
       reload_(NULL),
-#if defined(OS_CHROMEOS)
-      feedback_(NULL),
-#endif
       home_(NULL),
       location_bar_(NULL),
       browser_actions_(NULL),
       app_menu_(NULL),
-      profile_(NULL),
       browser_(browser),
-      profiles_menu_contents_(NULL),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)),
-      destroyed_flag_(NULL) {
-  SetID(VIEW_ID_TOOLBAR);
+      profiles_menu_contents_(NULL) {
+  set_id(VIEW_ID_TOOLBAR);
 
   browser_->command_updater()->AddCommandObserver(IDC_BACK, this);
   browser_->command_updater()->AddCommandObserver(IDC_FORWARD, this);
@@ -113,107 +113,87 @@ ToolbarView::ToolbarView(Browser* browser)
         IDR_LOCATIONBG_POPUPMODE_EDGE);
   }
 
-  if (!IsUpgradeRecommended()) {
-    registrar_.Add(this, NotificationType::UPGRADE_RECOMMENDED,
-                   NotificationService::AllSources());
-  }
-  registrar_.Add(this, NotificationType::MODULE_INCOMPATIBILITY_DETECTED,
-                 NotificationService::AllSources());
+  registrar_.Add(this, chrome::NOTIFICATION_UPGRADE_RECOMMENDED,
+                 content::NotificationService::AllSources());
+#if defined(OS_WIN)
+  registrar_.Add(this, chrome::NOTIFICATION_CRITICAL_UPGRADE_INSTALLED,
+                 content::NotificationService::AllSources());
+#endif
   registrar_.Add(this,
-                 NotificationType::BACKGROUND_PAGE_TRACKER_CHANGED,
-                 NotificationService::AllSources());
+                 chrome::NOTIFICATION_MODULE_INCOMPATIBILITY_BADGE_CHANGE,
+                 content::NotificationService::AllSources());
+  registrar_.Add(this, chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED,
+                 content::Source<Profile>(browser_->profile()));
 }
 
 ToolbarView::~ToolbarView() {
-  if (destroyed_flag_)
-    *destroyed_flag_ = true;
-
   // NOTE: Don't remove the command observers here.  This object gets destroyed
   // after the Browser (which owns the CommandUpdater), so the CommandUpdater is
   // already gone.
 }
 
-void ToolbarView::Init(Profile* profile) {
+void ToolbarView::Init() {
   back_menu_model_.reset(new BackForwardMenuModel(
       browser_, BackForwardMenuModel::BACKWARD_MENU));
   forward_menu_model_.reset(new BackForwardMenuModel(
       browser_, BackForwardMenuModel::FORWARD_MENU));
-  wrench_menu_model_.reset(new WrenchMenuModel(this, browser_));
-#if defined(OS_CHROMEOS)
-  if (chromeos::MenuUI::IsEnabled()) {
-      wrench_menu_2_.reset(
-          chromeos::WrenchMenuUI::CreateMenu2(wrench_menu_model_.get()));
-  }
-#endif
   back_ = new views::ButtonDropDown(this, back_menu_model_.get());
-  back_->set_triggerable_event_flags(views::Event::EF_LEFT_BUTTON_DOWN |
-                                     views::Event::EF_MIDDLE_BUTTON_DOWN);
+  back_->set_triggerable_event_flags(ui::EF_LEFT_MOUSE_BUTTON |
+                                     ui::EF_MIDDLE_MOUSE_BUTTON);
   back_->set_tag(IDC_BACK);
   back_->SetImageAlignment(views::ImageButton::ALIGN_RIGHT,
                            views::ImageButton::ALIGN_TOP);
-  back_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_TOOLTIP_BACK)));
+  back_->SetTooltipText(l10n_util::GetStringUTF16(IDS_TOOLTIP_BACK));
   back_->SetAccessibleName(l10n_util::GetStringUTF16(IDS_ACCNAME_BACK));
-  back_->SetID(VIEW_ID_BACK_BUTTON);
+  back_->set_id(VIEW_ID_BACK_BUTTON);
 
   forward_ = new views::ButtonDropDown(this, forward_menu_model_.get());
-  forward_->set_triggerable_event_flags(views::Event::EF_LEFT_BUTTON_DOWN |
-                                        views::Event::EF_MIDDLE_BUTTON_DOWN);
+  forward_->set_triggerable_event_flags(ui::EF_LEFT_MOUSE_BUTTON |
+                                        ui::EF_MIDDLE_MOUSE_BUTTON);
   forward_->set_tag(IDC_FORWARD);
-  forward_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_TOOLTIP_FORWARD)));
+  forward_->SetTooltipText(l10n_util::GetStringUTF16(IDS_TOOLTIP_FORWARD));
   forward_->SetAccessibleName(l10n_util::GetStringUTF16(IDS_ACCNAME_FORWARD));
-  forward_->SetID(VIEW_ID_FORWARD_BUTTON);
+  forward_->set_id(VIEW_ID_FORWARD_BUTTON);
 
   // Have to create this before |reload_| as |reload_|'s constructor needs it.
-  location_bar_ = new LocationBarView(profile, browser_->command_updater(),
-      model_, this, (display_mode_ == DISPLAYMODE_LOCATION) ?
+  location_bar_ = new LocationBarView(browser_, model_, this,
+      (display_mode_ == DISPLAYMODE_LOCATION) ?
           LocationBarView::POPUP : LocationBarView::NORMAL);
 
   reload_ = new ReloadButton(location_bar_, browser_);
-  reload_->set_triggerable_event_flags(views::Event::EF_LEFT_BUTTON_DOWN |
-                                       views::Event::EF_MIDDLE_BUTTON_DOWN);
+  reload_->set_triggerable_event_flags(ui::EF_LEFT_MOUSE_BUTTON |
+                                       ui::EF_MIDDLE_MOUSE_BUTTON);
   reload_->set_tag(IDC_RELOAD);
-  reload_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_TOOLTIP_RELOAD)));
+  reload_->SetTooltipText(l10n_util::GetStringUTF16(IDS_TOOLTIP_RELOAD));
   reload_->SetAccessibleName(l10n_util::GetStringUTF16(IDS_ACCNAME_RELOAD));
-  reload_->SetID(VIEW_ID_RELOAD_BUTTON);
-
-#if defined(OS_CHROMEOS)
-  feedback_ = new views::ImageButton(this);
-  feedback_->set_tag(IDC_FEEDBACK);
-  feedback_->set_triggerable_event_flags(views::Event::EF_LEFT_BUTTON_DOWN |
-                                         views::Event::EF_MIDDLE_BUTTON_DOWN);
-  feedback_->set_tag(IDC_FEEDBACK);
-  feedback_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_TOOLTIP_FEEDBACK)));
-  feedback_->SetID(VIEW_ID_FEEDBACK_BUTTON);
-#endif
+  reload_->set_id(VIEW_ID_RELOAD_BUTTON);
 
   home_ = new views::ImageButton(this);
-  home_->set_triggerable_event_flags(views::Event::EF_LEFT_BUTTON_DOWN |
-                                     views::Event::EF_MIDDLE_BUTTON_DOWN);
+  home_->set_triggerable_event_flags(ui::EF_LEFT_MOUSE_BUTTON |
+                                     ui::EF_MIDDLE_MOUSE_BUTTON);
   home_->set_tag(IDC_HOME);
-  home_->SetTooltipText(
-      UTF16ToWide(l10n_util::GetStringUTF16(IDS_TOOLTIP_HOME)));
+  home_->SetTooltipText(l10n_util::GetStringUTF16(IDS_TOOLTIP_HOME));
   home_->SetAccessibleName(l10n_util::GetStringUTF16(IDS_ACCNAME_HOME));
-  home_->SetID(VIEW_ID_HOME_BUTTON);
+  home_->set_id(VIEW_ID_HOME_BUTTON);
 
   browser_actions_ = new BrowserActionsContainer(browser_, this);
 
-  app_menu_ = new views::MenuButton(NULL, std::wstring(), this, false);
+#if defined(OS_WIN) && !defined(USE_AURA)
+  app_menu_ = new AppMenuButtonWin(this);
+#else
+  app_menu_ = new views::MenuButton(NULL, string16(), this, false);
+#endif
   app_menu_->set_border(NULL);
   app_menu_->EnableCanvasFlippingForRTLUI(true);
   app_menu_->SetAccessibleName(l10n_util::GetStringUTF16(IDS_ACCNAME_APP));
-  app_menu_->SetTooltipText(UTF16ToWide(l10n_util::GetStringFUTF16(
+  app_menu_->SetTooltipText(l10n_util::GetStringFUTF16(
       IDS_APPMENU_TOOLTIP,
-      l10n_util::GetStringUTF16(IDS_PRODUCT_NAME))));
-  app_menu_->SetID(VIEW_ID_APP_MENU);
+      l10n_util::GetStringUTF16(IDS_PRODUCT_NAME)));
+  app_menu_->set_id(VIEW_ID_APP_MENU);
 
   // Add any necessary badges to the menu item based on the system state.
-  if (IsUpgradeRecommended() || ShouldShowIncompatibilityWarning() ||
-      ShouldShowBackgroundPageBadge()) {
-    UpdateAppMenuBadge();
+  if (IsUpgradeRecommended() || ShouldShowIncompatibilityWarning()) {
+    UpdateAppMenuState();
   }
   LoadImages();
 
@@ -224,34 +204,23 @@ void ToolbarView::Init(Profile* profile) {
   AddChildView(home_);
   AddChildView(location_bar_);
   AddChildView(browser_actions_);
-#if defined(OS_CHROMEOS)
-  AddChildView(feedback_);
-#endif
   AddChildView(app_menu_);
 
   location_bar_->Init();
-  show_home_button_.Init(prefs::kShowHomeButton, profile->GetPrefs(), this);
+  show_home_button_.Init(prefs::kShowHomeButton,
+                         browser_->profile()->GetPrefs(), this);
   browser_actions_->Init();
 
-  SetProfile(profile);
-
   // Accessibility specific tooltip text.
   if (BrowserAccessibilityState::GetInstance()->IsAccessibleBrowser()) {
     back_->SetTooltipText(
-        UTF16ToWide(l10n_util::GetStringUTF16(IDS_ACCNAME_TOOLTIP_BACK)));
+        l10n_util::GetStringUTF16(IDS_ACCNAME_TOOLTIP_BACK));
     forward_->SetTooltipText(
-        UTF16ToWide(l10n_util::GetStringUTF16(IDS_ACCNAME_TOOLTIP_FORWARD)));
+        l10n_util::GetStringUTF16(IDS_ACCNAME_TOOLTIP_FORWARD));
   }
 }
 
-void ToolbarView::SetProfile(Profile* profile) {
-  if (profile != profile_) {
-    profile_ = profile;
-    location_bar_->SetProfile(profile);
-  }
-}
-
-void ToolbarView::Update(TabContents* tab, bool should_restore_state) {
+void ToolbarView::Update(WebContents* tab, bool should_restore_state) {
   if (location_bar_)
     location_bar_->Update(should_restore_state ? tab : NULL);
 
@@ -259,12 +228,8 @@ void ToolbarView::Update(TabContents* tab, bool should_restore_state) {
     browser_actions_->RefreshBrowserActionViews();
 }
 
-void ToolbarView::SetPaneFocusAndFocusLocationBar(int view_storage_id) {
-  SetPaneFocus(view_storage_id, location_bar_);
-}
-
-void ToolbarView::SetPaneFocusAndFocusAppMenu(int view_storage_id) {
-  SetPaneFocus(view_storage_id, app_menu_);
+void ToolbarView::SetPaneFocusAndFocusAppMenu() {
+  SetPaneFocus(app_menu_);
 }
 
 bool ToolbarView::IsAppMenuFocused() {
@@ -272,101 +237,114 @@ bool ToolbarView::IsAppMenuFocused() {
 }
 
 void ToolbarView::AddMenuListener(views::MenuListener* listener) {
-#if defined(OS_CHROMEOS)
-  if (chromeos::MenuUI::IsEnabled()) {
-    DCHECK(wrench_menu_2_.get());
-    wrench_menu_2_->AddMenuListener(listener);
-    return;
-  }
-#endif
-  menu_listeners_.push_back(listener);
+  menu_listeners_.AddObserver(listener);
 }
 
 void ToolbarView::RemoveMenuListener(views::MenuListener* listener) {
-#if defined(OS_CHROMEOS)
-  if (chromeos::MenuUI::IsEnabled()) {
-    DCHECK(wrench_menu_2_.get());
-    wrench_menu_2_->RemoveMenuListener(listener);
-    return;
+  menu_listeners_.RemoveObserver(listener);
+}
+
+SkBitmap ToolbarView::GetAppMenuIcon(views::CustomButton::ButtonState state) {
+  ui::ThemeProvider* tp = GetThemeProvider();
+
+  int id = 0;
+  switch (state) {
+    case views::CustomButton::BS_NORMAL: id = IDR_TOOLS;   break;
+    case views::CustomButton::BS_HOT:    id = IDR_TOOLS_H; break;
+    case views::CustomButton::BS_PUSHED: id = IDR_TOOLS_P; break;
+    default:                             NOTREACHED();     break;
   }
+  SkBitmap icon = *tp->GetBitmapNamed(id);
+
+#if defined(OS_WIN)
+  // Keep track of whether we were showing the badge before, so we don't send
+  // multiple UMA events for example when multiple Chrome windows are open.
+  static bool incompatibility_badge_showing = false;
+  // Save the old value before resetting it.
+  bool was_showing = incompatibility_badge_showing;
+  incompatibility_badge_showing = false;
 #endif
-  for (std::vector<views::MenuListener*>::iterator i(menu_listeners_.begin());
-       i != menu_listeners_.end(); ++i) {
-    if (*i == listener) {
-      menu_listeners_.erase(i);
-      return;
-    }
+
+  int error_badge_id = GlobalErrorServiceFactory::GetForProfile(
+      browser_->profile())->GetFirstBadgeResourceID();
+
+  bool add_badge = IsUpgradeRecommended() ||
+                   ShouldShowIncompatibilityWarning() || error_badge_id;
+  if (!add_badge)
+    return icon;
+
+  // Draw the chrome app menu icon onto the canvas.
+  scoped_ptr<gfx::CanvasSkia> canvas(new gfx::CanvasSkia(icon, false));
+
+  SkBitmap badge;
+  // Only one badge can be active at any given time. The Upgrade notification
+  // is deemed most important, then the DLL conflict badge.
+  if (IsUpgradeRecommended()) {
+    badge = *tp->GetBitmapNamed(
+        UpgradeDetector::GetInstance()->GetIconResourceID(
+            UpgradeDetector::UPGRADE_ICON_TYPE_BADGE));
+  } else if (ShouldShowIncompatibilityWarning()) {
+#if defined(OS_WIN)
+    if (!was_showing)
+      content::RecordAction(UserMetricsAction("ConflictBadge"));
+    badge = *tp->GetBitmapNamed(IDR_CONFLICT_BADGE);
+    incompatibility_badge_showing = true;
+#else
+    NOTREACHED();
+#endif
+  } else if (error_badge_id) {
+    badge = *tp->GetBitmapNamed(error_badge_id);
+  } else {
+    NOTREACHED();
   }
+
+  canvas->DrawBitmapInt(badge, icon.width() - badge.width(), kBadgeTopMargin);
+
+  return canvas->ExtractBitmap();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // ToolbarView, AccessiblePaneView overrides:
 
-bool ToolbarView::SetPaneFocus(
-    int view_storage_id, views::View* initial_focus) {
-  if (!AccessiblePaneView::SetPaneFocus(view_storage_id, initial_focus))
+bool ToolbarView::SetPaneFocus(views::View* initial_focus) {
+  if (!AccessiblePaneView::SetPaneFocus(initial_focus))
     return false;
 
   location_bar_->SetShowFocusRect(true);
   return true;
 }
 
-AccessibilityTypes::Role ToolbarView::GetAccessibleRole() {
-  return AccessibilityTypes::ROLE_TOOLBAR;
+void ToolbarView::GetAccessibleState(ui::AccessibleViewState* state) {
+  state->role = ui::AccessibilityTypes::ROLE_TOOLBAR;
+  state->name = l10n_util::GetStringUTF16(IDS_ACCNAME_TOOLBAR);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// ToolbarView, Menu::BaseControllerDelegate overrides:
+// ToolbarView, Menu::Delegate overrides:
 
 bool ToolbarView::GetAcceleratorInfo(int id, ui::Accelerator* accel) {
   return GetWidget()->GetAccelerator(id, accel);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// ToolbarView, views::MenuDelegate implementation:
+// ToolbarView, views::ViewMenuDelegate implementation:
 
 void ToolbarView::RunMenu(views::View* source, const gfx::Point& /* pt */) {
-  DCHECK_EQ(VIEW_ID_APP_MENU, source->GetID());
+  DCHECK_EQ(VIEW_ID_APP_MENU, source->id());
 
-  bool destroyed_flag = false;
-  destroyed_flag_ = &destroyed_flag;
-#if defined(OS_CHROMEOS)
-  if (chromeos::MenuUI::IsEnabled()) {
-    gfx::Point screen_loc;
-    views::View::ConvertPointToScreen(app_menu_, &screen_loc);
-    gfx::Rect bounds(screen_loc, app_menu_->size());
-    if (base::i18n::IsRTL())
-      bounds.set_x(bounds.x() - app_menu_->size().width());
-    wrench_menu_2_->RunMenuAt(gfx::Point(bounds.right(), bounds.bottom()),
-                              views::Menu2::ALIGN_TOPRIGHT);
-    // TODO(oshima): nuke this once we made decision about go or no go
-    // for domui menu.
-    goto cleanup;
-  }
-#endif
-  wrench_menu_ = new WrenchMenu(browser_);
-  wrench_menu_->Init(wrench_menu_model_.get());
+  wrench_menu_.reset(new WrenchMenu(browser_));
+  WrenchMenuModel model(this, browser_);
+  wrench_menu_->Init(&model);
 
-  for (size_t i = 0; i < menu_listeners_.size(); ++i)
-    menu_listeners_[i]->OnMenuOpened();
+  FOR_EACH_OBSERVER(views::MenuListener, menu_listeners_, OnMenuOpened());
 
   wrench_menu_->RunMenu(app_menu_);
-
-#if defined(OS_CHROMEOS)
- cleanup:
-#endif
-  if (destroyed_flag)
-    return;
-  destroyed_flag_ = NULL;
-
-  // Stop showing the background app badge also.
-  BackgroundPageTracker::GetInstance()->AcknowledgeBackgroundPages();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // ToolbarView, LocationBarView::Delegate implementation:
 
-TabContentsWrapper* ToolbarView::GetTabContentsWrapper() {
+TabContentsWrapper* ToolbarView::GetTabContentsWrapper() const {
   return browser_->GetSelectedTabContentsWrapper();
 }
 
@@ -421,27 +399,32 @@ void ToolbarView::ButtonPressed(views::Button* sender,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// ToolbarView, NotificationObserver implementation:
+// ToolbarView, content::NotificationObserver implementation:
 
-void ToolbarView::Observe(NotificationType type,
-                          const NotificationSource& source,
-                          const NotificationDetails& details) {
-  if (type == NotificationType::PREF_CHANGED) {
-    std::string* pref_name = Details<std::string>(details).ptr();
-    if (*pref_name == prefs::kShowHomeButton) {
-      Layout();
-      SchedulePaint();
+void ToolbarView::Observe(int type,
+                          const content::NotificationSource& source,
+                          const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_PREF_CHANGED: {
+      std::string* pref_name = content::Details<std::string>(details).ptr();
+      if (*pref_name == prefs::kShowHomeButton) {
+        Layout();
+        SchedulePaint();
+      }
+      break;
     }
-  } else if (type == NotificationType::UPGRADE_RECOMMENDED) {
-    UpdateAppMenuBadge();
-  } else if (type == NotificationType::MODULE_INCOMPATIBILITY_DETECTED) {
-    bool confirmed_bad = *Details<bool>(details).ptr();
-    if (confirmed_bad)
-      UpdateAppMenuBadge();
-  } else if (type ==
-      NotificationType::BACKGROUND_PAGE_TRACKER_CHANGED) {
-    // Force a repaint to add/remove the badge.
-    UpdateAppMenuBadge();
+    case chrome::NOTIFICATION_UPGRADE_RECOMMENDED:
+    case chrome::NOTIFICATION_MODULE_INCOMPATIBILITY_BADGE_CHANGE:
+    case chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED:
+      UpdateAppMenuState();
+      break;
+#if defined(OS_WIN)
+    case chrome::NOTIFICATION_CRITICAL_UPGRADE_INSTALLED:
+      ShowCriticalNotification();
+      break;
+#endif
+    default:
+      NOTREACHED();
   }
 }
 
@@ -455,13 +438,13 @@ bool ToolbarView::GetAcceleratorForCommandId(int command_id,
   // TODO(cpu) Bug 1109102. Query WebKit land for the actual bindings.
   switch (command_id) {
     case IDC_CUT:
-      *accelerator = views::Accelerator(ui::VKEY_X, false, true, false);
+      *accelerator = ui::Accelerator(ui::VKEY_X, false, true, false);
       return true;
     case IDC_COPY:
-      *accelerator = views::Accelerator(ui::VKEY_C, false, true, false);
+      *accelerator = ui::Accelerator(ui::VKEY_C, false, true, false);
       return true;
     case IDC_PASTE:
-      *accelerator = views::Accelerator(ui::VKEY_V, false, true, false);
+      *accelerator = ui::Accelerator(ui::VKEY_V, false, true, false);
       return true;
   }
   // Else, we retrieve the accelerator information from the frame.
@@ -479,13 +462,11 @@ gfx::Size ToolbarView::GetPreferredSize() {
         reload_->GetPreferredSize().width() + kStandardSpacing +
         (show_home_button_.GetValue() ?
             (home_->GetPreferredSize().width() + kButtonSpacing) : 0) +
+        location_bar_->GetPreferredSize().width() +
         browser_actions_->GetPreferredSize().width() +
-#if defined(OS_CHROMEOS)
-        feedback_->GetPreferredSize().width() + kButtonSpacing +
-#endif
         app_menu_->GetPreferredSize().width() + kEdgeSpacing;
 
-    static SkBitmap normal_background;
+    CR_DEFINE_STATIC_LOCAL(SkBitmap, normal_background, ());
     if (normal_background.isNull()) {
       ResourceBundle& rb = ResourceBundle::GetSharedInstance();
       normal_background = *rb.GetBitmapNamed(IDR_CONTENT_TOP_CENTER);
@@ -495,7 +476,7 @@ gfx::Size ToolbarView::GetPreferredSize() {
   }
 
   int vertical_spacing = PopupTopSpacing() +
-      (GetWindow()->GetNonClientView()->UseNativeFrame() ?
+      (GetWidget()->ShouldUseNativeFrame() ?
           kPopupBottomSpacingGlass : kPopupBottomSpacingNonGlass);
   return gfx::Size(0, location_bar_->GetPreferredSize().height() +
       vertical_spacing);
@@ -506,9 +487,10 @@ void ToolbarView::Layout() {
   if (back_ == NULL)
     return;
 
+  bool maximized = browser_->window() && browser_->window()->IsMaximized();
   if (!IsDisplayModeNormal()) {
-    int edge_width = (browser_->window() && browser_->window()->IsMaximized()) ?
-        0 : kPopupBackgroundEdge->width();  // See Paint().
+    int edge_width = maximized ?
+        0 : kPopupBackgroundEdge->width();  // See OnPaint().
     location_bar_->SetBounds(edge_width, PopupTopSpacing(),
         width() - (edge_width * 2), location_bar_->GetPreferredSize().height());
     return;
@@ -527,7 +509,7 @@ void ToolbarView::Layout() {
   //                Layout() in this case.
   //                http://crbug.com/5540
   int back_width = back_->GetPreferredSize().width();
-  if (browser_->window() && browser_->window()->IsMaximized())
+  if (maximized)
     back_->SetBounds(0, child_y, back_width + kEdgeSpacing, child_height);
   else
     back_->SetBounds(kEdgeSpacing, child_y, back_width, child_height);
@@ -548,20 +530,15 @@ void ToolbarView::Layout() {
   }
 
   int browser_actions_width = browser_actions_->GetPreferredSize().width();
-#if defined(OS_CHROMEOS)
-  int feedback_menu_width = feedback_->GetPreferredSize().width() +
-                            kButtonSpacing;
-#endif
   int app_menu_width = app_menu_->GetPreferredSize().width();
   int location_x = home_->x() + home_->width() + kStandardSpacing;
   int available_width = width() - kEdgeSpacing - app_menu_width -
-#if defined(OS_CHROMEOS)
-      feedback_menu_width -
-#endif
       browser_actions_width - location_x;
+  int location_y = child_y;
+  int location_bar_height = child_height;
 
-  location_bar_->SetBounds(location_x, child_y, std::max(available_width, 0),
-                           child_height);
+  location_bar_->SetBounds(location_x, location_y, std::max(available_width, 0),
+                           location_bar_height);
 
   browser_actions_->SetBounds(location_bar_->x() + location_bar_->width(), 0,
                               browser_actions_width, height());
@@ -574,19 +551,16 @@ void ToolbarView::Layout() {
   //                required.
   browser_actions_->Layout();
 
-#if defined(OS_CHROMEOS)
-  feedback_->SetBounds(browser_actions_->x() + browser_actions_width, child_y,
-                       feedback_->GetPreferredSize().width(), child_height);
-  app_menu_->SetBounds(feedback_->x() + feedback_->width() + kButtonSpacing,
-                       child_y, app_menu_width, child_height);
-#else
+  // Extend the app menu to the screen's right edge in maximized mode just like
+  // we extend the back button to the left edge.
+  if (maximized)
+    app_menu_width += kEdgeSpacing;
   app_menu_->SetBounds(browser_actions_->x() + browser_actions_width, child_y,
                        app_menu_width, child_height);
-#endif
 }
 
-void ToolbarView::Paint(gfx::Canvas* canvas) {
-  View::Paint(canvas);
+void ToolbarView::OnPaint(gfx::Canvas* canvas) {
+  View::OnPaint(canvas);
 
   if (IsDisplayModeNormal())
     return;
@@ -603,21 +577,66 @@ void ToolbarView::Paint(gfx::Canvas* canvas) {
   // For glass, we need to draw a black line below the location bar to separate
   // it from the content area.  For non-glass, the NonClientView draws the
   // toolbar background below the location bar for us.
-  if (GetWindow()->GetNonClientView()->UseNativeFrame())
-    canvas->FillRectInt(SK_ColorBLACK, 0, height() - 1, width(), 1);
+  // NOTE: Keep this in sync with BrowserView::GetInfoBarSeparatorColor()!
+  if (GetWidget()->ShouldUseNativeFrame())
+    canvas->FillRect(SK_ColorBLACK, gfx::Rect(0, height() - 1, width(), 1));
+}
+
+// Note this method is ignored on Windows, but needs to be implemented for
+// linux, where it is called before CanDrop().
+bool ToolbarView::GetDropFormats(
+    int* formats,
+    std::set<OSExchangeData::CustomFormat>* custom_formats) {
+  *formats = ui::OSExchangeData::URL | ui::OSExchangeData::STRING;
+  return true;
+}
+
+bool ToolbarView::CanDrop(const ui::OSExchangeData& data) {
+  // To support loading URLs by dropping into the toolbar, we need to support
+  // dropping URLs and/or text.
+  return data.HasURL() || data.HasString();
+}
+
+int ToolbarView::OnDragUpdated(const views::DropTargetEvent& event) {
+  if (event.source_operations() & ui::DragDropTypes::DRAG_COPY) {
+    return ui::DragDropTypes::DRAG_COPY;
+  } else if (event.source_operations() & ui::DragDropTypes::DRAG_LINK) {
+    return ui::DragDropTypes::DRAG_LINK;
+  }
+  return ui::DragDropTypes::DRAG_NONE;
+}
+
+int ToolbarView::OnPerformDrop(const views::DropTargetEvent& event) {
+  return location_bar_->location_entry()->OnPerformDrop(event);
 }
 
 void ToolbarView::OnThemeChanged() {
   LoadImages();
 }
 
+std::string ToolbarView::GetClassName() const {
+  return kViewClassName;
+}
+
+bool ToolbarView::AcceleratorPressed(const ui::Accelerator& accelerator) {
+  const views::View* focused_view = focus_manager_->GetFocusedView();
+  if (focused_view == location_bar_)
+    return false;  // Let location bar handle all accelerator events.
+  return AccessiblePaneView::AcceleratorPressed(accelerator);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // ToolbarView, protected:
 
 // Override this so that when the user presses F6 to rotate toolbar panes,
-// the location bar gets focus, not the first control in the toolbar.
-views::View* ToolbarView::GetDefaultFocusableChild() {
-  return location_bar_;
+// the location bar gets focus, not the first control in the toolbar - and
+// also so that it selects all content in the location bar.
+bool ToolbarView::SetPaneFocusAndFocusDefault() {
+  if (!SetPaneFocus(location_bar_))
+    return false;
+
+  location_bar_->SelectAll();
+  return true;
 }
 
 void ToolbarView::RemovePaneFocus() {
@@ -629,30 +648,29 @@ void ToolbarView::RemovePaneFocus() {
 // ToolbarView, private:
 
 bool ToolbarView::IsUpgradeRecommended() {
-#if defined(OS_CHROMEOS)
-  return (chromeos::CrosLibrary::Get()->GetUpdateLibrary()->status().status ==
-          chromeos::UPDATE_STATUS_UPDATED_NEED_REBOOT);
-#else
   return (UpgradeDetector::GetInstance()->notify_upgrade());
-#endif
-}
-
-bool ToolbarView::ShouldShowBackgroundPageBadge() {
-  return BackgroundPageTracker::GetInstance()->
-      GetUnacknowledgedBackgroundPageCount() > 0;
 }
 
 bool ToolbarView::ShouldShowIncompatibilityWarning() {
 #if defined(OS_WIN)
   EnumerateModulesModel* loaded_modules = EnumerateModulesModel::GetInstance();
-  return loaded_modules->confirmed_bad_modules_detected() > 0;
+  return loaded_modules->ShouldShowConflictWarning();
 #else
   return false;
 #endif
 }
 
 int ToolbarView::PopupTopSpacing() const {
-  return GetWindow()->GetNonClientView()->UseNativeFrame() ?
+  // TODO(beng): For some reason GetWidget() returns NULL here in some
+  //             unidentified circumstances on ChromeOS. This means GetWidget()
+  //             succeeded but we were (probably) unable to locate a
+  //             NativeWidgetGtk* on it using
+  //             NativeWidget::GetNativeWidgetForNativeView.
+  //             I am throwing in a NULL check for now to stop the hurt, but
+  //             it's possible the crash may just show up somewhere else.
+  const views::Widget* widget = GetWidget();
+  DCHECK(widget) << "If you hit this please talk to beng";
+  return widget && widget->ShouldUseNativeFrame() ?
       0 : kPopupTopSpacingNonGlass;
 }
 
@@ -690,15 +708,6 @@ void ToolbarView::LoadImages() {
   reload_->SetToggledImage(views::CustomButton::BS_DISABLED,
       tp->GetBitmapNamed(IDR_STOP_D));
 
-#if defined(OS_CHROMEOS)
-  feedback_->SetImage(views::CustomButton::BS_NORMAL,
-      tp->GetBitmapNamed(IDR_FEEDBACK));
-  feedback_->SetImage(views::CustomButton::BS_HOT,
-      tp->GetBitmapNamed(IDR_FEEDBACK_H));
-  feedback_->SetImage(views::CustomButton::BS_PUSHED,
-      tp->GetBitmapNamed(IDR_FEEDBACK_P));
-#endif
-
   home_->SetImage(views::CustomButton::BS_NORMAL, tp->GetBitmapNamed(IDR_HOME));
   home_->SetImage(views::CustomButton::BS_HOT, tp->GetBitmapNamed(IDR_HOME_H));
   home_->SetImage(views::CustomButton::BS_PUSHED,
@@ -709,67 +718,25 @@ void ToolbarView::LoadImages() {
   app_menu_->SetPushedIcon(GetAppMenuIcon(views::CustomButton::BS_PUSHED));
 }
 
-void ToolbarView::UpdateAppMenuBadge() {
+void ToolbarView::ShowCriticalNotification() {
+#if defined(OS_WIN)
+  CriticalNotificationBubbleView* bubble_delegate =
+      new CriticalNotificationBubbleView(app_menu_);
+  browser::CreateViewsBubble(bubble_delegate);
+  bubble_delegate->StartFade(true);
+#endif
+}
+
+void ToolbarView::UpdateAppMenuState() {
+  string16 accname_app = l10n_util::GetStringUTF16(IDS_ACCNAME_APP);
+  if (IsUpgradeRecommended()) {
+    accname_app = l10n_util::GetStringFUTF16(
+        IDS_ACCNAME_APP_UPGRADE_RECOMMENDED, accname_app);
+  }
+  app_menu_->SetAccessibleName(accname_app);
+
   app_menu_->SetIcon(GetAppMenuIcon(views::CustomButton::BS_NORMAL));
   app_menu_->SetHoverIcon(GetAppMenuIcon(views::CustomButton::BS_HOT));
   app_menu_->SetPushedIcon(GetAppMenuIcon(views::CustomButton::BS_PUSHED));
   SchedulePaint();
 }
-
-SkBitmap ToolbarView::GetAppMenuIcon(views::CustomButton::ButtonState state) {
-  ui::ThemeProvider* tp = GetThemeProvider();
-
-  int id = 0;
-  switch (state) {
-    case views::CustomButton::BS_NORMAL: id = IDR_TOOLS;   break;
-    case views::CustomButton::BS_HOT:    id = IDR_TOOLS_H; break;
-    case views::CustomButton::BS_PUSHED: id = IDR_TOOLS_P; break;
-    default:                             NOTREACHED();     break;
-  }
-  SkBitmap icon = *tp->GetBitmapNamed(id);
-
-#if defined(OS_WIN)
-  // Keep track of whether we were showing the badge before, so we don't send
-  // multiple UMA events for example when multiple Chrome windows are open.
-  static bool incompatibility_badge_showing = false;
-  // Save the old value before resetting it.
-  bool was_showing = incompatibility_badge_showing;
-  incompatibility_badge_showing = false;
-#endif
-
-  bool add_badge = IsUpgradeRecommended() ||
-                   ShouldShowIncompatibilityWarning() ||
-                   ShouldShowBackgroundPageBadge();
-  if (!add_badge)
-    return icon;
-
-  // Draw the chrome app menu icon onto the canvas.
-  scoped_ptr<gfx::CanvasSkia> canvas(
-      new gfx::CanvasSkia(icon.width(), icon.height(), false));
-  canvas->DrawBitmapInt(icon, 0, 0);
-
-  SkBitmap badge;
-  // Only one badge can be active at any given time. The Upgrade notification
-  // is deemed most important, then the temporary background page badge,
-  // then the DLL conflict badge.
-  if (IsUpgradeRecommended()) {
-    badge = *tp->GetBitmapNamed(IDR_UPDATE_BADGE);
-  } else if (ShouldShowBackgroundPageBadge()) {
-    badge = *tp->GetBitmapNamed(IDR_BACKGROUND_BADGE);
-  } else if (ShouldShowIncompatibilityWarning()) {
-#if defined(OS_WIN)
-    if (!was_showing)
-      UserMetrics::RecordAction(UserMetricsAction("ConflictBadge"), profile_);
-    badge = *tp->GetBitmapNamed(IDR_CONFLICT_BADGE);
-    incompatibility_badge_showing = true;
-#else
-    NOTREACHED();
-#endif
-  } else {
-    NOTREACHED();
-  }
-
-  canvas->DrawBitmapInt(badge, icon.width() - badge.width(), kBadgeTopMargin);
-
-  return canvas->ExtractBitmap();
-}
