@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,8 @@
 #define CHROME_BROWSER_UI_VIEWS_UNHANDLED_KEYBOARD_EVENT_HANDLER_H_
 #pragma once
 
-#include "views/view.h"
-#include "chrome/common/native_web_keyboard_event.h"
+#include "content/public/browser/native_web_keyboard_event.h"
+#include "ui/views/view.h"
 
 namespace views {
 class FocusManager;
@@ -24,6 +24,7 @@ class UnhandledKeyboardEventHandler {
                            views::FocusManager* focus_manager);
 
  private:
+#if defined(OS_WIN)
   // Whether to ignore the next Char keyboard event.
   // If a RawKeyDown event was handled as a shortcut key, then we're done
   // handling it and should eat any Char event that the translate phase may
@@ -31,6 +32,7 @@ class UnhandledKeyboardEventHandler {
   // such as a beep if DefWindowProc() has no default handling for the given
   // Char.)
   bool ignore_next_char_event_;
+#endif
 
   DISALLOW_COPY_AND_ASSIGN(UnhandledKeyboardEventHandler);
 };
