@@ -4,39 +4,49 @@
 
 #include "chrome/browser/ui/views/wrench_menu.h"
 
+#include <algorithm>
 #include <cmath>
+#include <set>
 
 #include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/metrics/user_metrics.h"
+#include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
 #include "chrome/browser/ui/browser.h"
-#include "chrome/common/notification_observer.h"
-#include "chrome/common/notification_registrar.h"
-#include "chrome/common/notification_source.h"
-#include "chrome/common/notification_type.h"
-#include "gfx/canvas.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/skia_util.h"
+#include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/views/bookmarks/bookmark_menu_delegate.h"
+#include "chrome/common/chrome_notification_types.h"
+#include "content/public/browser/notification_observer.h"
+#include "content/public/browser/notification_registrar.h"
+#include "content/public/browser/notification_source.h"
+#include "content/public/browser/notification_types.h"
+#include "content/public/browser/user_metrics.h"
+#include "content/public/browser/web_contents.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
 #include "third_party/skia/include/core/SkPaint.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "views/background.h"
-#include "views/controls/button/image_button.h"
-#include "views/controls/button/menu_button.h"
-#include "views/controls/button/text_button.h"
-#include "views/controls/label.h"
-#include "views/controls/menu/menu_config.h"
-#include "views/controls/menu/menu_item_view.h"
-#include "views/controls/menu/menu_scroll_view_container.h"
-#include "views/controls/menu/submenu_view.h"
-#include "views/window/window.h"
+#include "ui/gfx/canvas.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/skia_util.h"
+#include "ui/views/background.h"
+#include "ui/views/controls/button/image_button.h"
+#include "ui/views/controls/button/menu_button.h"
+#include "ui/views/controls/button/text_button.h"
+#include "ui/views/controls/label.h"
+#include "ui/views/controls/menu/menu_config.h"
+#include "ui/views/controls/menu/menu_item_view.h"
+#include "ui/views/controls/menu/menu_runner.h"
+#include "ui/views/controls/menu/menu_scroll_view_container.h"
+#include "ui/views/controls/menu/submenu_view.h"
+#include "ui/views/widget/widget.h"
 
+using content::HostZoomMap;
+using content::UserMetricsAction;
+using content::WebContents;
 using ui::MenuModel;
 using views::CustomButton;
 using views::ImageButton;
@@ -136,46 +146,49 @@ class MenuButtonBackground : public views::Background {
     int h = view->height();
     switch (TypeAdjustedForRTL()) {
       case LEFT_BUTTON:
-        canvas->FillRectInt(background_color(state), 1, 1, w, h - 2);
-        canvas->FillRectInt(border_color(state), 2, 0, w, 1);
-        canvas->FillRectInt(border_color(state), 1, 1, 1, 1);
-        canvas->FillRectInt(border_color(state), 0, 2, 1, h - 4);
-        canvas->FillRectInt(border_color(state), 1, h - 2, 1, 1);
-        canvas->FillRectInt(border_color(state), 2, h - 1, w, 1);
+        canvas->FillRect(background_color(state), gfx::Rect(1, 1, w, h - 2));
+        canvas->FillRect(border_color(state), gfx::Rect(2, 0, w, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(1, 1, 1, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(0, 2, 1, h - 4));
+        canvas->FillRect(border_color(state), gfx::Rect(1, h - 2, 1, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(2, h - 1, w, 1));
         break;
 
       case CENTER_BUTTON: {
-        canvas->FillRectInt(background_color(state), 1, 1, w - 2, h - 2);
+        canvas->FillRect(background_color(state),
+                         gfx::Rect(1, 1, w - 2, h - 2));
         SkColor left_color = state != CustomButton::BS_NORMAL ?
             border_color(state) : border_color(left_button_->state());
-        canvas->FillRectInt(left_color, 0, 0, 1, h);
-        canvas->FillRectInt(border_color(state), 1, 0, w - 2, 1);
-        canvas->FillRectInt(border_color(state), 1, h - 1, w - 2, 1);
+        canvas->FillRect(left_color, gfx::Rect(0, 0, 1, h));
+        canvas->FillRect(border_color(state), gfx::Rect(1, 0, w - 2, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(1, h - 1, w - 2, 1));
         SkColor right_color = state != CustomButton::BS_NORMAL ?
             border_color(state) : border_color(right_button_->state());
-        canvas->FillRectInt(right_color, w - 1, 0, 1, h);
+        canvas->FillRect(right_color, gfx::Rect(w - 1, 0, 1, h));
         break;
       }
 
       case RIGHT_BUTTON:
-        canvas->FillRectInt(background_color(state), 0, 1, w - 1, h - 2);
-        canvas->FillRectInt(border_color(state), 0, 0, w - 2, 1);
-        canvas->FillRectInt(border_color(state), w - 2, 1, 1, 1);
-        canvas->FillRectInt(border_color(state), w - 1, 2, 1, h - 4);
-        canvas->FillRectInt(border_color(state), w - 2, h - 2, 1, 1);
-        canvas->FillRectInt(border_color(state), 0, h - 1, w - 2, 1);
+        canvas->FillRect(background_color(state),
+                         gfx::Rect(0, 1, w - 1, h - 2));
+        canvas->FillRect(border_color(state), gfx::Rect(0, 0, w - 2, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(w - 2, 1, 1, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(w - 1, 2, 1, h - 4));
+        canvas->FillRect(border_color(state), gfx::Rect(w - 2, h - 2, 1, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(0, h - 1, w - 2, 1));
         break;
 
       case SINGLE_BUTTON:
-        canvas->FillRectInt(background_color(state), 1, 1, w - 2, h - 2);
-        canvas->FillRectInt(border_color(state), 2, 0, w - 4, 1);
-        canvas->FillRectInt(border_color(state), 1, 1, 1, 1);
-        canvas->FillRectInt(border_color(state), 0, 2, 1, h - 4);
-        canvas->FillRectInt(border_color(state), 1, h - 2, 1, 1);
-        canvas->FillRectInt(border_color(state), 2, h - 1, w - 4, 1);
-        canvas->FillRectInt(border_color(state), w - 2, 1, 1, 1);
-        canvas->FillRectInt(border_color(state), w - 1, 2, 1, h - 4);
-        canvas->FillRectInt(border_color(state), w - 2, h - 2, 1, 1);
+        canvas->FillRect(background_color(state),
+                         gfx::Rect(1, 1, w - 2, h - 2));
+        canvas->FillRect(border_color(state), gfx::Rect(2, 0, w - 4, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(1, 1, 1, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(0, 2, 1, h - 4));
+        canvas->FillRect(border_color(state), gfx::Rect(1, h - 2, 1, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(2, h - 1, w - 4, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(w - 2, 1, 1, 1));
+        canvas->FillRect(border_color(state), gfx::Rect(w - 1, 2, 1, h - 4));
+        canvas->FillRect(border_color(state), gfx::Rect(w - 2, h - 2, 1, 1));
         break;
 
       default:
@@ -231,14 +244,8 @@ class ScheduleAllView : public views::View {
  public:
   ScheduleAllView() {}
 
-  virtual void SchedulePaint(const gfx::Rect& r, bool urgent) {
-    if (!IsVisible())
-      return;
-
-    if (GetParent()) {
-      GetParent()->SchedulePaint(GetBounds(APPLY_MIRRORING_TRANSFORMATION),
-                                 urgent);
-    }
+  virtual void SchedulePaintInRect(const gfx::Rect& r) {
+    View::SchedulePaintInRect(gfx::Rect(0, 0, width(), height()));
   }
 
  private:
@@ -253,8 +260,8 @@ string16 GetAccessibleNameForWrenchMenuItem(
   ui::Accelerator menu_accelerator;
   if (model->GetAcceleratorAt(item_index, &menu_accelerator)) {
     accelerator_text =
-        views::Accelerator(menu_accelerator.GetKeyCode(),
-                           menu_accelerator.modifiers()).GetShortcutText();
+        ui::Accelerator(menu_accelerator.key_code(),
+                        menu_accelerator.modifiers()).GetShortcutText();
   }
 
   return MenuItemView::GetAccessibleNameForMenuItem(
@@ -280,11 +287,11 @@ class WrenchMenuView : public ScheduleAllView, public views::ButtonListener {
                                       int index,
                                       MenuButtonBackground** background,
                                       int acc_string_id) {
-    TextButton* button =
-        new TextButton(this, UTF16ToWide(l10n_util::GetStringUTF16(string_id)));
+    TextButton* button = new TextButton(this,
+                                        l10n_util::GetStringUTF16(string_id));
     button->SetAccessibleName(
         GetAccessibleNameForWrenchMenuItem(menu_model_, index, acc_string_id));
-    button->SetFocusable(true);
+    button->set_focusable(true);
     button->set_request_focus_on_press(false);
     button->set_tag(index);
     button->SetEnabled(menu_model_->IsEnabledAt(index));
@@ -296,7 +303,6 @@ class WrenchMenuView : public ScheduleAllView, public views::ButtonListener {
       *background = bg;
     button->set_border(new MenuButtonBorder());
     button->set_alignment(TextButton::ALIGN_CENTER);
-    button->SetNormalHasBorder(true);
     button->SetFont(views::MenuConfig::instance().font);
     button->ClearMaxTextSize();
     AddChildView(button);
@@ -344,14 +350,14 @@ class WrenchMenu::CutCopyPasteView : public WrenchMenuView {
   gfx::Size GetPreferredSize() {
     // Returned height doesn't matter as MenuItemView forces everything to the
     // height of the menuitemview.
-    return gfx::Size(GetMaxChildViewPreferredWidth() * GetChildViewCount(), 0);
+    return gfx::Size(GetMaxChildViewPreferredWidth() * child_count(), 0);
   }
 
   void Layout() {
     // All buttons are given the same width.
     int width = GetMaxChildViewPreferredWidth();
-    for (int i = 0; i < GetChildViewCount(); ++i)
-      GetChildViewAt(i)->SetBounds(i * width, 0, width, height());
+    for (int i = 0; i < child_count(); ++i)
+      child_at(i)->SetBounds(i * width, 0, width, height());
   }
 
   // ButtonListener
@@ -363,8 +369,8 @@ class WrenchMenu::CutCopyPasteView : public WrenchMenuView {
   // Returns the max preferred width of all the children.
   int GetMaxChildViewPreferredWidth() {
     int width = 0;
-    for (int i = 0; i < GetChildViewCount(); ++i)
-      width = std::max(width, GetChildViewAt(i)->GetPreferredSize().width());
+    for (int i = 0; i < child_count(); ++i)
+      width = std::max(width, child_at(i)->GetPreferredSize().width());
     return width;
   }
 
@@ -380,7 +386,7 @@ static const int kZoomPadding = 6;
 // the zoom, a label showing the current zoom percent, and a button to go
 // full-screen.
 class WrenchMenu::ZoomView : public WrenchMenuView,
-                             public NotificationObserver {
+                             public content::NotificationObserver {
  public:
   ZoomView(WrenchMenu* menu,
            MenuModel* menu_model,
@@ -399,8 +405,9 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
         NULL, IDS_ACCNAME_ZOOM_MINUS2);
 
     zoom_label_ = new Label(
-        UTF16ToWide(l10n_util::GetStringFUTF16Int(IDS_ZOOM_PERCENT, 100)));
-    zoom_label_->SetColor(MenuConfig::instance().text_color);
+        l10n_util::GetStringFUTF16Int(IDS_ZOOM_PERCENT, 100));
+    zoom_label_->SetAutoColorReadabilityEnabled(false);
+    zoom_label_->SetEnabledColor(MenuConfig::instance().text_color);
     zoom_label_->SetHorizontalAlignment(Label::ALIGN_RIGHT);
     MenuButtonBackground* center_bg =
         new MenuButtonBackground(MenuButtonBackground::CENTER_BUTTON);
@@ -421,7 +428,7 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
         ImageButton::BS_NORMAL,
         ResourceBundle::GetSharedInstance().GetBitmapNamed(
             IDR_FULLSCREEN_MENU_BUTTON));
-    fullscreen_button_->SetFocusable(true);
+    fullscreen_button_->set_focusable(true);
     fullscreen_button_->set_request_focus_on_press(false);
     fullscreen_button_->set_tag(fullscreen_index);
     fullscreen_button_->SetImageAlignment(
@@ -437,8 +444,10 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
 
     UpdateZoomControls();
 
-    registrar_.Add(this, NotificationType::ZOOM_LEVEL_CHANGED,
-                   Source<Profile>(menu->browser_->profile()));
+    registrar_.Add(
+        this, content::NOTIFICATION_ZOOM_LEVEL_CHANGED,
+        content::Source<HostZoomMap>(
+            menu->browser_->profile()->GetHostZoomMap()));
   }
 
   gfx::Size GetPreferredSize() {
@@ -458,22 +467,22 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
                                 decrement_button_->GetPreferredSize().width());
     gfx::Rect bounds(0, 0, button_width, height());
 
-    decrement_button_->SetBounds(bounds);
+    decrement_button_->SetBoundsRect(bounds);
 
     x += bounds.width();
     bounds.set_x(x);
     bounds.set_width(zoom_label_width_);
-    zoom_label_->SetBounds(bounds);
+    zoom_label_->SetBoundsRect(bounds);
 
     x += bounds.width();
     bounds.set_x(x);
     bounds.set_width(button_width);
-    increment_button_->SetBounds(bounds);
+    increment_button_->SetBoundsRect(bounds);
 
     x += bounds.width() + kZoomPadding;
     bounds.set_x(x);
     bounds.set_width(fullscreen_button_->GetPreferredSize().width());
-    fullscreen_button_->SetBounds(bounds);
+    fullscreen_button_->SetBoundsRect(bounds);
   }
 
   // ButtonListener:
@@ -486,11 +495,11 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
     }
   }
 
-  // NotificationObserver:
-  virtual void Observe(NotificationType type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details) {
-    DCHECK_EQ(NotificationType::ZOOM_LEVEL_CHANGED, type.value);
+  // content::NotificationObserver:
+  virtual void Observe(int type,
+                       const content::NotificationSource& source,
+                       const content::NotificationDetails& details) {
+    DCHECK_EQ(content::NOTIFICATION_ZOOM_LEVEL_CHANGED, type);
     UpdateZoomControls();
   }
 
@@ -498,15 +507,14 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
   void UpdateZoomControls() {
     bool enable_increment = false;
     bool enable_decrement = false;
-    TabContents* selected_tab = menu_->browser_->GetSelectedTabContents();
+    WebContents* selected_tab = menu_->browser_->GetSelectedWebContents();
     int zoom = 100;
     if (selected_tab)
       zoom = selected_tab->GetZoomPercent(&enable_increment, &enable_decrement);
     increment_button_->SetEnabled(enable_increment);
     decrement_button_->SetEnabled(enable_decrement);
-    zoom_label_->SetText(UTF16ToWide(l10n_util::GetStringFUTF16Int(
-                                     IDS_ZOOM_PERCENT,
-                                     zoom)));
+    zoom_label_->SetText(
+        l10n_util::GetStringFUTF16Int(IDS_ZOOM_PERCENT, zoom));
 
     zoom_label_width_ = MaxWidthForZoomLabel();
   }
@@ -520,10 +528,10 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
 
     int max_w = 0;
 
-    TabContents* selected_tab = menu_->browser_->GetSelectedTabContents();
+    WebContents* selected_tab = menu_->browser_->GetSelectedWebContents();
     if (selected_tab) {
-      int min_percent = selected_tab->minimum_zoom_percent();
-      int max_percent = selected_tab->maximum_zoom_percent();
+      int min_percent = selected_tab->GetMinimumZoomPercent();
+      int max_percent = selected_tab->GetMaximumZoomPercent();
 
       int step = (max_percent - min_percent) / 10;
       for (int i = min_percent; i <= max_percent; i += step) {
@@ -542,7 +550,7 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
   // Index of the fullscreen menu item in the model.
   const int fullscreen_index_;
 
-  NotificationRegistrar registrar_;
+  content::NotificationRegistrar registrar_;
 
   // Button for incrementing the zoom.
   TextButton* increment_button_;
@@ -564,44 +572,152 @@ class WrenchMenu::ZoomView : public WrenchMenuView,
 // WrenchMenu ------------------------------------------------------------------
 
 WrenchMenu::WrenchMenu(Browser* browser)
-    : browser_(browser),
+    : root_(NULL),
+      browser_(browser),
       selected_menu_model_(NULL),
-      selected_index_(0) {
+      selected_index_(0),
+      bookmark_menu_(NULL),
+      first_bookmark_command_id_(0) {
+  registrar_.Add(this, chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED,
+                 content::Source<Profile>(browser_->profile()));
+}
+
+WrenchMenu::~WrenchMenu() {
+  if (bookmark_menu_delegate_.get()) {
+    BookmarkModel* model = browser_->profile()->GetBookmarkModel();
+    if (model)
+      model->RemoveObserver(this);
+  }
 }
 
 void WrenchMenu::Init(ui::MenuModel* model) {
-  DCHECK(!root_.get());
-  root_.reset(new MenuItemView(this));
-  root_->SetAccessibleName(l10n_util::GetStringUTF16(IDS_ACCNAME_APP));
+  DCHECK(!root_);
+  root_ = new MenuItemView(this);
   root_->set_has_icons(true);  // We have checks, radios and icons, set this
                                // so we get the taller menu style.
   int next_id = 1;
-  PopulateMenu(root_.get(), model, &next_id);
+  PopulateMenu(root_, model, &next_id);
+  first_bookmark_command_id_ = next_id + 1;
+  menu_runner_.reset(new views::MenuRunner(root_));
 }
 
 void WrenchMenu::RunMenu(views::MenuButton* host) {
-  // Up the ref count while the menu is displaying. This way if the window is
-  // deleted while we're running we won't prematurely delete the menu.
-  // TODO(sky): fix this, the menu should really take ownership of the menu
-  // (57890).
-  scoped_refptr<WrenchMenu> dont_delete_while_running(this);
   gfx::Point screen_loc;
   views::View::ConvertPointToScreen(host, &screen_loc);
   gfx::Rect bounds(screen_loc, host->size());
-  UserMetrics::RecordAction(UserMetricsAction("ShowAppMenu"));
-  root_->RunMenuAt(host->GetWindow()->GetNativeWindow(), host, bounds,
-      base::i18n::IsRTL() ? MenuItemView::TOPLEFT : MenuItemView::TOPRIGHT,
-      true);
+  content::RecordAction(UserMetricsAction("ShowAppMenu"));
+  if (menu_runner_->RunMenuAt(host->GetWidget(), host, bounds,
+          MenuItemView::TOPRIGHT, views::MenuRunner::HAS_MNEMONICS) ==
+      views::MenuRunner::MENU_DELETED)
+    return;
+  if (bookmark_menu_delegate_.get()) {
+    BookmarkModel* model = browser_->profile()->GetBookmarkModel();
+    if (model)
+      model->RemoveObserver(this);
+  }
   if (selected_menu_model_)
     selected_menu_model_->ActivatedAt(selected_index_);
 }
 
+string16 WrenchMenu::GetTooltipText(int id,
+                                    const gfx::Point& p) const {
+  return is_bookmark_command(id) ?
+      bookmark_menu_delegate_->GetTooltipText(id, p) : string16();
+}
+
+bool WrenchMenu::IsTriggerableEvent(views::MenuItemView* menu,
+                                    const views::MouseEvent& e) {
+  return is_bookmark_command(menu->GetCommand()) ?
+      bookmark_menu_delegate_->IsTriggerableEvent(menu, e) :
+      MenuDelegate::IsTriggerableEvent(menu, e);
+}
+
+bool WrenchMenu::GetDropFormats(
+      MenuItemView* menu,
+      int* formats,
+      std::set<ui::OSExchangeData::CustomFormat>* custom_formats) {
+  CreateBookmarkMenu();
+  return bookmark_menu_delegate_.get() &&
+      bookmark_menu_delegate_->GetDropFormats(menu, formats, custom_formats);
+}
+
+bool WrenchMenu::AreDropTypesRequired(MenuItemView* menu) {
+  CreateBookmarkMenu();
+  return bookmark_menu_delegate_.get() &&
+      bookmark_menu_delegate_->AreDropTypesRequired(menu);
+}
+
+bool WrenchMenu::CanDrop(MenuItemView* menu,
+                         const ui::OSExchangeData& data) {
+  CreateBookmarkMenu();
+  return bookmark_menu_delegate_.get() &&
+      bookmark_menu_delegate_->CanDrop(menu, data);
+}
+
+int WrenchMenu::GetDropOperation(
+    MenuItemView* item,
+    const views::DropTargetEvent& event,
+    DropPosition* position) {
+  return is_bookmark_command(item->GetCommand()) ?
+      bookmark_menu_delegate_->GetDropOperation(item, event, position) :
+      ui::DragDropTypes::DRAG_NONE;
+}
+
+int WrenchMenu::OnPerformDrop(MenuItemView* menu,
+                              DropPosition position,
+                              const views::DropTargetEvent& event) {
+  if (!is_bookmark_command(menu->GetCommand()))
+    return ui::DragDropTypes::DRAG_NONE;
+
+  int result = bookmark_menu_delegate_->OnPerformDrop(menu, position, event);
+  return result;
+}
+
+bool WrenchMenu::ShowContextMenu(MenuItemView* source,
+                                 int id,
+                                 const gfx::Point& p,
+                                 bool is_mouse_gesture) {
+  return is_bookmark_command(id) ?
+      bookmark_menu_delegate_->ShowContextMenu(source, id, p,
+                                               is_mouse_gesture) :
+      false;
+}
+
+bool WrenchMenu::CanDrag(MenuItemView* menu) {
+  return is_bookmark_command(menu->GetCommand()) ?
+      bookmark_menu_delegate_->CanDrag(menu) : false;
+}
+
+void WrenchMenu::WriteDragData(MenuItemView* sender,
+                               ui::OSExchangeData* data) {
+  DCHECK(is_bookmark_command(sender->GetCommand()));
+  return bookmark_menu_delegate_->WriteDragData(sender, data);
+}
+
+int WrenchMenu::GetDragOperations(MenuItemView* sender) {
+  return is_bookmark_command(sender->GetCommand()) ?
+      bookmark_menu_delegate_->GetDragOperations(sender) :
+      MenuDelegate::GetDragOperations(sender);
+}
+
+int WrenchMenu::GetMaxWidthForMenu(MenuItemView* menu) {
+  return is_bookmark_command(menu->GetCommand()) ?
+      bookmark_menu_delegate_->GetMaxWidthForMenu(menu) :
+      MenuDelegate::GetMaxWidthForMenu(menu);
+}
+
 bool WrenchMenu::IsItemChecked(int id) const {
+  if (is_bookmark_command(id))
+    return false;
+
   const Entry& entry = id_to_entry_.find(id)->second;
   return entry.first->IsItemCheckedAt(entry.second);
 }
 
 bool WrenchMenu::IsCommandEnabled(int id) const {
+  if (is_bookmark_command(id))
+    return true;
+
   if (id == 0)
     return false;  // The root item.
 
@@ -614,7 +730,13 @@ bool WrenchMenu::IsCommandEnabled(int id) const {
       entry.first->IsEnabledAt(entry.second);
 }
 
-void WrenchMenu::ExecuteCommand(int id) {
+void WrenchMenu::ExecuteCommand(int id, int mouse_event_flags) {
+  if (is_bookmark_command(id)) {
+    bookmark_menu_delegate_->ExecuteCommand(id, mouse_event_flags);
+    return;
+  }
+
+  // Not a bookmark
   const Entry& entry = id_to_entry_.find(id)->second;
   int command_id = entry.first->GetCommandIdAt(entry.second);
 
@@ -628,8 +750,16 @@ void WrenchMenu::ExecuteCommand(int id) {
   return entry.first->ActivatedAt(entry.second);
 }
 
-bool WrenchMenu::GetAccelerator(int id, views::Accelerator* accelerator) {
-  const Entry& entry = id_to_entry_.find(id)->second;
+bool WrenchMenu::GetAccelerator(int id, ui::Accelerator* accelerator) {
+  if (is_bookmark_command(id))
+    return false;
+  IDToEntry::iterator ix = id_to_entry_.find(id);
+  if (ix == id_to_entry_.end()) {
+    // There is no entry for this id.
+    return false;
+  }
+
+  const Entry& entry = ix->second;
   int command_id = entry.first->GetCommandIdAt(entry.second);
   if (command_id == IDC_CUT || command_id == IDC_ZOOM_MINUS) {
     // These have special child views; don't show the accelerator for them.
@@ -640,12 +770,35 @@ bool WrenchMenu::GetAccelerator(int id, views::Accelerator* accelerator) {
   if (!entry.first->GetAcceleratorAt(entry.second, &menu_accelerator))
     return false;
 
-  *accelerator = views::Accelerator(menu_accelerator.GetKeyCode(),
-                                    menu_accelerator.modifiers());
+  *accelerator = ui::Accelerator(menu_accelerator.key_code(),
+                                 menu_accelerator.modifiers());
   return true;
 }
 
-WrenchMenu::~WrenchMenu() {
+void WrenchMenu::WillShowMenu(MenuItemView* menu) {
+  if (menu == bookmark_menu_)
+    CreateBookmarkMenu();
+}
+
+void WrenchMenu::BookmarkModelChanged() {
+  DCHECK(bookmark_menu_delegate_.get());
+  if (!bookmark_menu_delegate_->is_mutating_model())
+    root_->Cancel();
+}
+
+
+void WrenchMenu::Observe(int type,
+                         const content::NotificationSource& source,
+                         const content::NotificationDetails& details) {
+  switch (type) {
+    case chrome::NOTIFICATION_GLOBAL_ERRORS_CHANGED:
+      // A change in the global errors list can add or remove items from the
+      // menu. Close the menu to avoid have a stale menu on-screen.
+      root_->Cancel();
+      break;
+    default:
+      NOTREACHED();
+  }
 }
 
 void WrenchMenu::PopulateMenu(MenuItemView* parent,
@@ -661,23 +814,35 @@ void WrenchMenu::PopulateMenu(MenuItemView* parent,
     if (model->GetTypeAt(index) == MenuModel::TYPE_SUBMENU)
       PopulateMenu(item, model->GetSubmenuModelAt(index), next_id);
 
-    if (model->GetCommandIdAt(index) == IDC_CUT) {
-      DCHECK_EQ(MenuModel::TYPE_COMMAND, model->GetTypeAt(index));
-      DCHECK_LT(i + 2, max);
-      DCHECK_EQ(IDC_COPY, model->GetCommandIdAt(index + 1));
-      DCHECK_EQ(IDC_PASTE, model->GetCommandIdAt(index + 2));
-      item->SetTitle(UTF16ToWide(l10n_util::GetStringUTF16(IDS_EDIT2)));
-      item->AddChildView(
-          new CutCopyPasteView(this, model, index, index + 1, index + 2));
-      i += 2;
-    } else if (model->GetCommandIdAt(index) == IDC_ZOOM_MINUS) {
-      DCHECK_EQ(MenuModel::TYPE_COMMAND, model->GetTypeAt(index));
-      DCHECK_EQ(IDC_ZOOM_PLUS, model->GetCommandIdAt(index + 1));
-      DCHECK_EQ(IDC_FULLSCREEN, model->GetCommandIdAt(index + 2));
-      item->SetTitle(UTF16ToWide(l10n_util::GetStringUTF16(IDS_ZOOM_MENU2)));
-      item->AddChildView(
-          new ZoomView(this, model, index, index + 1, index + 2));
-      i += 2;
+    switch (model->GetCommandIdAt(index)) {
+      case IDC_CUT:
+        DCHECK_EQ(MenuModel::TYPE_COMMAND, model->GetTypeAt(index));
+        DCHECK_LT(i + 2, max);
+        DCHECK_EQ(IDC_COPY, model->GetCommandIdAt(index + 1));
+        DCHECK_EQ(IDC_PASTE, model->GetCommandIdAt(index + 2));
+        item->SetTitle(l10n_util::GetStringUTF16(IDS_EDIT2));
+        item->AddChildView(
+            new CutCopyPasteView(this, model, index, index + 1, index + 2));
+        i += 2;
+        break;
+
+      case IDC_ZOOM_MINUS:
+        DCHECK_EQ(MenuModel::TYPE_COMMAND, model->GetTypeAt(index));
+        DCHECK_EQ(IDC_ZOOM_PLUS, model->GetCommandIdAt(index + 1));
+        DCHECK_EQ(IDC_FULLSCREEN, model->GetCommandIdAt(index + 2));
+        item->SetTitle(l10n_util::GetStringUTF16(IDS_ZOOM_MENU2));
+        item->AddChildView(
+            new ZoomView(this, model, index, index + 1, index + 2));
+        i += 2;
+        break;
+
+      case IDC_BOOKMARKS_MENU:
+        DCHECK(!bookmark_menu_);
+        bookmark_menu_ = item;
+        break;
+
+      default:
+        break;
     }
   }
 }
@@ -694,13 +859,14 @@ MenuItemView* WrenchMenu::AppendMenuItem(MenuItemView* parent,
 
   MenuItemView* menu_item = parent->AppendMenuItemFromModel(model, index, id);
 
-  if (menu_item)
+  if (menu_item) {
     menu_item->SetVisible(model->IsVisibleAt(index));
 
-  if (menu_type == MenuModel::TYPE_COMMAND && model->HasIcons()) {
-    SkBitmap icon;
-    if (model->GetIconAt(index, &icon))
-      menu_item->SetIcon(icon);
+    if (menu_type == MenuModel::TYPE_COMMAND && model->HasIcons()) {
+      SkBitmap icon;
+      if (model->GetIconAt(index, &icon))
+        menu_item->SetIcon(icon);
+    }
   }
 
   return menu_item;
@@ -711,3 +877,27 @@ void WrenchMenu::CancelAndEvaluate(MenuModel* model, int index) {
   selected_index_ = index;
   root_->Cancel();
 }
+
+void WrenchMenu::CreateBookmarkMenu() {
+  if (bookmark_menu_delegate_.get())
+    return;  // Already created the menu.
+
+  BookmarkModel* model = browser_->profile()->GetBookmarkModel();
+  if (!model->IsLoaded())
+    return;
+
+  model->AddObserver(this);
+
+  // TODO(oshima): Replace with views only API.
+  views::Widget* parent = views::Widget::GetWidgetForNativeWindow(
+      browser_->window()->GetNativeHandle());
+  bookmark_menu_delegate_.reset(
+      new BookmarkMenuDelegate(browser_->profile(),
+                               NULL,
+                               parent,
+                               first_bookmark_command_id_));
+  bookmark_menu_delegate_->Init(
+      this, bookmark_menu_, model->bookmark_bar_node(), 0,
+      BookmarkMenuDelegate::SHOW_PERMANENT_FOLDERS,
+      bookmark_utils::LAUNCH_WRENCH_MENU);
+}
