@@ -1,19 +1,22 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/file_util.h"
 #include "base/test/test_timeouts.h"
-#include "chrome/browser/net/url_request_mock_http_job.h"
 #include "chrome/browser/ui/view_ids.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/automation/window_proxy.h"
 #include "chrome/test/ui/ui_test.h"
-#include "net/url_request/url_request_unittest.h"
-#include "ui/base/message_box_flags.h"
-#include "views/event.h"
+#include "content/browser/net/url_request_mock_http_job.h"
+#include "net/url_request/url_request_test_util.h"
+#include "ui/base/events.h"
+#include "ui/base/ui_base_types.h"
+
+using base::TimeDelta;
 
 const std::string NOLISTENERS_HTML =
     "<html><head><title>nolisteners</title></head><body></body></html>";
@@ -103,25 +106,13 @@ class UnloadTest : public UITest {
     UITest::SetUp();
   }
 
-  void WaitForBrowserClosed() {
-    const int kCheckDelayMs = 100;
-    for (int max_wait_time = TestTimeouts::action_max_timeout_ms();
-         max_wait_time > 0; max_wait_time -= kCheckDelayMs) {
-      CrashAwareSleep(kCheckDelayMs);
-      if (!IsBrowserRunning())
-        break;
-    }
-
-    EXPECT_FALSE(IsBrowserRunning());
-  }
-
   void CheckTitle(const std::wstring& expected_title) {
-    const int kCheckDelayMs = 100;
-    for (int max_wait_time = TestTimeouts::action_max_timeout_ms();
-         max_wait_time > 0; max_wait_time -= kCheckDelayMs) {
-      CrashAwareSleep(kCheckDelayMs);
+    const TimeDelta kCheckDelay = TimeDelta::FromMilliseconds(100);
+    for (TimeDelta max_wait_time = TestTimeouts::action_max_timeout();
+         max_wait_time > TimeDelta(); max_wait_time -= kCheckDelay) {
       if (expected_title == GetActiveTabTitle())
         break;
+      base::PlatformThread::Sleep(kCheckDelay);
     }
 
     EXPECT_EQ(expected_title, GetActiveTabTitle());
@@ -165,9 +156,9 @@ class UnloadTest : public UITest {
     EXPECT_TRUE(CloseBrowser(browser.get(), &application_closed));
   }
 
-  void ClickModalDialogButton(ui::MessageBoxFlags::DialogButton button) {
+  void ClickModalDialogButton(ui::DialogButton button) {
     bool modal_dialog_showing = false;
-    ui::MessageBoxFlags::DialogButton available_buttons;
+    ui::DialogButton available_buttons;
     EXPECT_TRUE(automation()->WaitForAppModalDialog());
     EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
         &available_buttons));
@@ -190,7 +181,6 @@ TEST_F(UnloadTest, DISABLED_CrossSiteInfiniteUnloadAsync) {
   NavigateToDataURL(INFINITE_UNLOAD_HTML, L"infiniteunload");
   // Must navigate to a non-data URL to trigger cross-site codepath.
   NavigateToNolistenersFileTwiceAsync();
-  ASSERT_TRUE(IsBrowserRunning());
 }
 
 // Navigate to a page with an infinite unload handler.
@@ -204,7 +194,6 @@ TEST_F(UnloadTest, CrossSiteInfiniteUnloadSync) {
   NavigateToDataURL(INFINITE_UNLOAD_HTML, L"infiniteunload");
   // Must navigate to a non-data URL to trigger cross-site codepath.
   NavigateToNolistenersFileTwice();
-  ASSERT_TRUE(IsBrowserRunning());
 }
 
 // TODO(creis): This test is currently failing intermittently on Linux and
@@ -245,11 +234,10 @@ TEST_F(UnloadTest, MAYBE_CrossSiteInfiniteUnloadAsyncInputEvent) {
   gfx::Rect bounds;
   ASSERT_TRUE(window->GetViewBounds(VIEW_ID_TAB_0, &bounds, false));
   ASSERT_TRUE(browser->SimulateDrag(bounds.CenterPoint(), bounds.CenterPoint(),
-                                    views::Event::EF_LEFT_BUTTON_DOWN, false));
+                                    ui::EF_LEFT_MOUSE_BUTTON, false));
 
   // The title should update before the timeout in CheckTitle.
   CheckTitle(L"Title Of Awesomeness");
-  ASSERT_TRUE(IsBrowserRunning());
 }
 
 // Navigate to a page with an infinite beforeunload handler.
@@ -264,7 +252,6 @@ TEST_F(UnloadTest, FLAKY_CrossSiteInfiniteBeforeUnloadAsync) {
   NavigateToDataURL(INFINITE_BEFORE_UNLOAD_HTML, L"infinitebeforeunload");
   // Must navigate to a non-data URL to trigger cross-site codepath.
   NavigateToNolistenersFileTwiceAsync();
-  ASSERT_TRUE(IsBrowserRunning());
 }
 
 // Navigate to a page with an infinite beforeunload handler.
@@ -278,7 +265,6 @@ TEST_F(UnloadTest, CrossSiteInfiniteBeforeUnloadSync) {
   NavigateToDataURL(INFINITE_BEFORE_UNLOAD_HTML, L"infinitebeforeunload");
   // Must navigate to a non-data URL to trigger cross-site codepath.
   NavigateToNolistenersFileTwice();
-  ASSERT_TRUE(IsBrowserRunning());
 }
 
 // Tests closing the browser on a page with no unload listeners registered.
@@ -300,8 +286,12 @@ TEST_F(UnloadTest, BrowserCloseBeforeUnloadOK) {
   NavigateToDataURL(BEFORE_UNLOAD_HTML, L"beforeunload");
 
   CloseBrowserAsync(browser.get());
-  ClickModalDialogButton(ui::MessageBoxFlags::DIALOGBUTTON_OK);
-  WaitForBrowserClosed();
+  ClickModalDialogButton(ui::DIALOG_BUTTON_OK);
+
+  int exit_code = -1;
+  ASSERT_TRUE(launcher_->WaitForBrowserProcessToQuit(
+                  TestTimeouts::action_max_timeout_ms(), &exit_code));
+  EXPECT_EQ(0, exit_code);  // Expect a clean shutown.
 }
 
 // Tests closing the browser with a beforeunload handler and clicking
@@ -312,17 +302,32 @@ TEST_F(UnloadTest, BrowserCloseBeforeUnloadCancel) {
   NavigateToDataURL(BEFORE_UNLOAD_HTML, L"beforeunload");
 
   CloseBrowserAsync(browser.get());
-  ClickModalDialogButton(ui::MessageBoxFlags::DIALOGBUTTON_CANCEL);
+  ClickModalDialogButton(ui::DIALOG_BUTTON_CANCEL);
+
   // There's no real graceful way to wait for something _not_ to happen, so
   // we just wait a short period.
-  CrashAwareSleep(500);
-  ASSERT_TRUE(IsBrowserRunning());
+  base::PlatformThread::Sleep(TestTimeouts::action_timeout());
 
   CloseBrowserAsync(browser.get());
-  ClickModalDialogButton(ui::MessageBoxFlags::DIALOGBUTTON_OK);
-  WaitForBrowserClosed();
+  ClickModalDialogButton(ui::DIALOG_BUTTON_OK);
+
+  int exit_code = -1;
+  ASSERT_TRUE(launcher_->WaitForBrowserProcessToQuit(
+                  TestTimeouts::action_max_timeout_ms(), &exit_code));
+  EXPECT_EQ(0, exit_code);  // Expect a clean shutdown.
 }
 
+// Tests terminating the browser with a beforeunload handler.
+// Currently only ChromeOS shuts down gracefully.
+#if defined(OS_CHROMEOS)
+TEST_F(UnloadTest, BrowserTerminateBeforeUnload) {
+  scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
+  ASSERT_TRUE(browser.get());
+  NavigateToDataURL(BEFORE_UNLOAD_HTML, L"beforeunload");
+  TerminateBrowser();
+}
+#endif
+
 #if defined(OS_LINUX)
 // Fails sometimes on Linux valgrind. http://crbug.com/45675
 #define MAYBE_BrowserCloseWithInnerFocusedFrame \
@@ -341,8 +346,12 @@ TEST_F(UnloadTest, MAYBE_BrowserCloseWithInnerFocusedFrame) {
   NavigateToDataURL(INNER_FRAME_WITH_FOCUS_HTML, L"innerframewithfocus");
 
   CloseBrowserAsync(browser.get());
-  ClickModalDialogButton(ui::MessageBoxFlags::DIALOGBUTTON_OK);
-  WaitForBrowserClosed();
+  ClickModalDialogButton(ui::DIALOG_BUTTON_OK);
+
+  int exit_code = -1;
+  ASSERT_TRUE(launcher_->WaitForBrowserProcessToQuit(
+                  TestTimeouts::action_max_timeout_ms(), &exit_code));
+  EXPECT_EQ(0, exit_code);  // Expect a clean shutdown.
 }
 
 // Tests closing the browser with a beforeunload handler that takes
@@ -362,8 +371,19 @@ TEST_F(UnloadTest, BrowserCloseInfiniteUnload) {
   LoadUrlAndQuitBrowser(INFINITE_UNLOAD_HTML, L"infiniteunload");
 }
 
+#if defined(OS_WIN)
+// Flakily fails, times out: http://crbug.com/78803
+#define MAYBE_BrowserCloseInfiniteBeforeUnload \
+    DISABLED_BrowserCloseInfiniteBeforeUnload
+#elif defined(OS_CHROMEOS)
+// Flakily fails: http://crbug.com/86469
+#define MAYBE_BrowserCloseInfiniteBeforeUnload \
+    FLAKY_BrowserCloseInfiniteBeforeUnload
+#else
+#define MAYBE_BrowserCloseInfiniteBeforeUnload BrowserCloseInfiniteBeforeUnload
+#endif
 // Tests closing the browser with a beforeunload handler that hangs.
-TEST_F(UnloadTest, BrowserCloseInfiniteBeforeUnload) {
+TEST_F(UnloadTest, MAYBE_BrowserCloseInfiniteBeforeUnload) {
   // Tests makes no sense in single-process mode since the renderer is hung.
   if (CommandLine::ForCurrentProcess()->HasSwitch(switches::kSingleProcess))
     return;
@@ -381,9 +401,17 @@ TEST_F(UnloadTest, BrowserCloseInfiniteUnloadAlert) {
   LoadUrlAndQuitBrowser(INFINITE_UNLOAD_ALERT_HTML, L"infiniteunloadalert");
 }
 
+#if defined(OS_WIN)
+// Flakily fails, times out: http://crbug.com/78803
+#define MAYBE_BrowserCloseInfiniteBeforeUnloadAlert \
+    DISABLED_BrowserCloseInfiniteBeforeUnloadAlert
+#else
+#define MAYBE_BrowserCloseInfiniteBeforeUnloadAlert \
+    BrowserCloseInfiniteBeforeUnloadAlert
+#endif
 // Tests closing the browser with a beforeunload handler that hangs then
 // pops up an alert.
-TEST_F(UnloadTest, BrowserCloseInfiniteBeforeUnloadAlert) {
+TEST_F(UnloadTest, MAYBE_BrowserCloseInfiniteBeforeUnloadAlert) {
   // Tests makes no sense in single-process mode since the renderer is hung.
   if (CommandLine::ForCurrentProcess()->HasSwitch(switches::kSingleProcess))
     return;
@@ -413,11 +441,13 @@ TEST_F(UnloadTest, BrowserCloseTwoSecondBeforeUnloadAlert) {
 // http://crbug.com/45281
 #define MAYBE_BrowserCloseTabWhenOtherTabHasListener \
     DISABLED_BrowserCloseTabWhenOtherTabHasListener
-#else
-// Flaky on Linux under valgrind. http://crbug.com/46781
-// TODO(stuartmorgan): Switch to just disabling for valgrind.
+#elif defined(OS_CHROMEOS)
+// http://crbug.com/86769
 #define MAYBE_BrowserCloseTabWhenOtherTabHasListener \
     FLAKY_BrowserCloseTabWhenOtherTabHasListener
+#else
+#define MAYBE_BrowserCloseTabWhenOtherTabHasListener \
+    BrowserCloseTabWhenOtherTabHasListener
 #endif
 
 // Tests that if there's a renderer process with two tabs, one of which has an
@@ -437,14 +467,12 @@ TEST_F(UnloadTest, MAYBE_BrowserCloseTabWhenOtherTabHasListener) {
   // Simulate a click to force user_gesture to true; if we don't, the resulting
   // popup will be constrained, which isn't what we want to test.
   ASSERT_TRUE(window->SimulateOSClick(tab_view_bounds.CenterPoint(),
-                                      views::Event::EF_LEFT_BUTTON_DOWN));
+                                      ui::EF_LEFT_MOUSE_BUTTON));
   ASSERT_TRUE(browser->WaitForTabCountToBecome(2));
 
+  CheckTitle(L"popup");
   scoped_refptr<TabProxy> popup_tab(browser->GetActiveTab());
   ASSERT_TRUE(popup_tab.get());
-  std::wstring popup_title;
-  EXPECT_TRUE(popup_tab->GetTabTitle(&popup_title));
-  EXPECT_EQ(std::wstring(L"popup"), popup_title);
   EXPECT_TRUE(popup_tab->Close(true));
 
   ASSERT_TRUE(browser->WaitForTabCountToBecome(1));
