@@ -1,28 +1,32 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include <vector>
-#include <string>
 #include <cstdio>
+#include <string>
+#include <vector>
 
-#include "base/message_loop.h"
 #include "base/file_util.h"
+#include "base/message_loop.h"
 #include "base/path_service.h"
 #include "base/process_util.h"
 #include "base/shared_memory.h"
 #include "base/string_util.h"
-#include "chrome/browser/browser_thread.h"
-#include "chrome/browser/visitedlink/visitedlink_master.h"
 #include "chrome/browser/visitedlink/visitedlink_event_listener.h"
-#include "chrome/browser/renderer_host/browser_render_process_host.h"
-#include "chrome/browser/renderer_host/test/test_render_view_host.h"
+#include "chrome/browser/visitedlink/visitedlink_master.h"
 #include "chrome/common/render_messages.h"
 #include "chrome/renderer/visitedlink_slave.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/chrome_render_view_host_test_harness.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/browser/renderer_host/render_process_host_impl.h"
+#include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
+#include "content/test/test_browser_thread.h"
 #include "googleurl/src/gurl.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using content::BrowserThread;
+
 namespace {
 
 // a nice long URL that we can append numbers to to get new URLs
@@ -51,7 +55,7 @@ class TrackingVisitedLinkEventListener : public VisitedLinkMaster::Listener {
            i < g_slaves.size(); i++) {
         base::SharedMemoryHandle new_handle = base::SharedMemory::NULLHandle();
         table->ShareToProcess(base::GetCurrentProcessHandle(), &new_handle);
-        g_slaves[i]->Init(new_handle);
+        g_slaves[i]->OnUpdateVisitedLinks(new_handle);
       }
     }
   }
@@ -102,7 +106,7 @@ class VisitedLinkTest : public testing::Test {
       master_.reset(NULL);
 
     if (history_service_.get()) {
-      history_service_->SetOnBackendDestroyTask(new MessageLoop::QuitTask);
+      history_service_->SetOnBackendDestroyTask(MessageLoop::QuitClosure());
       history_service_->Cleanup();
       history_service_ = NULL;
 
@@ -112,6 +116,9 @@ class VisitedLinkTest : public testing::Test {
       // our destroy task.
       MessageLoop::current()->Run();
     }
+
+    // Wait for all pending file I/O to be completed.
+    BrowserThread::GetBlockingPool()->FlushForTesting();
   }
 
   // Loads the database from disk and makes sure that the same URLs are present
@@ -134,8 +141,7 @@ class VisitedLinkTest : public testing::Test {
     base::SharedMemoryHandle new_handle = base::SharedMemory::NULLHandle();
     master_->shared_memory()->ShareToProcess(
         base::GetCurrentProcessHandle(), &new_handle);
-    bool success = slave.Init(new_handle);
-    ASSERT_TRUE(success);
+    slave.OnUpdateVisitedLinks(new_handle);
     g_slaves.push_back(&slave);
 
     bool found;
@@ -177,8 +183,8 @@ class VisitedLinkTest : public testing::Test {
   ScopedTempDir temp_dir_;
 
   MessageLoop message_loop_;
-  BrowserThread ui_thread_;
-  BrowserThread file_thread_;
+  content::TestBrowserThread ui_thread_;
+  content::TestBrowserThread file_thread_;
 
   // Filenames for the services;
   FilePath history_dir_;
@@ -276,7 +282,7 @@ TEST_F(VisitedLinkTest, DeleteAll) {
     base::SharedMemoryHandle new_handle = base::SharedMemory::NULLHandle();
     master_->shared_memory()->ShareToProcess(
         base::GetCurrentProcessHandle(), &new_handle);
-    ASSERT_TRUE(slave.Init(new_handle));
+    slave.OnUpdateVisitedLinks(new_handle);
     g_slaves.push_back(&slave);
 
     // Add the test URLs.
@@ -325,8 +331,7 @@ TEST_F(VisitedLinkTest, Resizing) {
   base::SharedMemoryHandle new_handle = base::SharedMemory::NULLHandle();
   master_->shared_memory()->ShareToProcess(
       base::GetCurrentProcessHandle(), &new_handle);
-  bool success = slave.Init(new_handle);
-  ASSERT_TRUE(success);
+  slave.OnUpdateVisitedLinks(new_handle);
   g_slaves.push_back(&slave);
 
   int32 used_count = master_->GetUsedCount();
@@ -400,7 +405,7 @@ TEST_F(VisitedLinkTest, Rebuild) {
   // complete before we set the task because the rebuild completion message
   // is posted to the message loop; until we Run() it, rebuild can not
   // complete.
-  master_->set_rebuild_complete_task(new MessageLoop::QuitTask);
+  master_->set_rebuild_complete_task(MessageLoop::QuitClosure());
   MessageLoop::current()->Run();
 
   // Test that all URLs were written to the database properly.
@@ -421,7 +426,7 @@ TEST_F(VisitedLinkTest, BigImport) {
     master_->AddURL(TestURL(i));
 
   // Wait for the rebuild to complete.
-  master_->set_rebuild_complete_task(new MessageLoop::QuitTask);
+  master_->set_rebuild_complete_task(MessageLoop::QuitClosure());
   MessageLoop::current()->Run();
 
   // Ensure that the right number of URLs are present
@@ -455,15 +460,17 @@ TEST_F(VisitedLinkTest, Listener) {
 
 class VisitCountingProfile : public TestingProfile {
  public:
-  explicit VisitCountingProfile(VisitedLinkEventListener* event_listener)
+  VisitCountingProfile()
       : add_count_(0),
         add_event_count_(0),
         reset_event_count_(0),
-        event_listener_(event_listener) {}
+        event_listener_(ALLOW_THIS_IN_INITIALIZER_LIST(
+            new VisitedLinkEventListener(this))) {}
 
   virtual VisitedLinkMaster* GetVisitedLinkMaster() {
     if (!visited_link_master_.get()) {
-      visited_link_master_.reset(new VisitedLinkMaster(event_listener_, this));
+      visited_link_master_.reset(
+          new VisitedLinkMaster(event_listener_.get(), this));
       visited_link_master_->Init();
     }
     return visited_link_master_.get();
@@ -487,47 +494,36 @@ class VisitCountingProfile : public TestingProfile {
   int add_count_;
   int add_event_count_;
   int reset_event_count_;
-  VisitedLinkEventListener* event_listener_;
+  scoped_ptr<VisitedLinkEventListener> event_listener_;
   scoped_ptr<VisitedLinkMaster> visited_link_master_;
 };
 
-class VisitCountingRenderProcessHost : public MockRenderProcessHost {
+// Stub out as little as possible, borrowing from RenderProcessHost.
+class VisitRelayingRenderProcessHost : public RenderProcessHostImpl {
  public:
-  explicit VisitCountingRenderProcessHost(Profile* profile)
-      : MockRenderProcessHost(profile) {}
-
-  virtual void AddVisitedLinks(
-      const VisitedLinkCommon::Fingerprints& visited_links) {
-    VisitCountingProfile* counting_profile =
-        static_cast<VisitCountingProfile*>(profile());
-    counting_profile->CountAddEvent(visited_links.size());
-  }
-  virtual void ResetVisitedLinks() {
-    VisitCountingProfile* counting_profile =
-        static_cast<VisitCountingProfile*>(profile());
-    counting_profile->CountResetEvent();
-  }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(VisitCountingRenderProcessHost);
-};
-
-// Stub out as little as possible, borrowing from MockRenderProcessHost.
-class VisitRelayingRenderProcessHost : public BrowserRenderProcessHost {
- public:
-  explicit VisitRelayingRenderProcessHost(Profile* profile)
-      : BrowserRenderProcessHost(profile) {
+  explicit VisitRelayingRenderProcessHost(
+      content::BrowserContext* browser_context)
+          : RenderProcessHostImpl(browser_context) {
+    content::NotificationService::current()->Notify(
+        content::NOTIFICATION_RENDERER_PROCESS_CREATED,
+        content::Source<RenderProcessHost>(this),
+        content::NotificationService::NoDetails());
   }
   virtual ~VisitRelayingRenderProcessHost() {
+    content::NotificationService::current()->Notify(
+        content::NOTIFICATION_RENDERER_PROCESS_TERMINATED,
+        content::Source<content::RenderProcessHost>(this),
+        content::NotificationService::NoDetails());
   }
 
-  virtual bool Init() { return true; }
+  virtual bool Init(bool is_accessibility_enabled) {
+    return true;
+  }
 
   virtual void CancelResourceRequests(int render_widget_id) {
   }
 
-  virtual void CrossSiteClosePageACK(int new_render_process_host_id,
-                                     int new_request_id) {
+  virtual void CrossSiteSwapOutACK(const ViewMsg_SwapOut_Params& params) {
   }
 
   virtual bool WaitForPaintMsg(int render_widget_id,
@@ -538,12 +534,17 @@ class VisitRelayingRenderProcessHost : public BrowserRenderProcessHost {
 
   virtual bool Send(IPC::Message* msg) {
     VisitCountingProfile* counting_profile =
-        static_cast<VisitCountingProfile*>(profile());
+        static_cast<VisitCountingProfile*>(
+            Profile::FromBrowserContext(GetBrowserContext()));
 
-    if (msg->type() == ViewMsg_VisitedLink_Add::ID)
-      counting_profile->CountAddEvent(1);
-    else if (msg->type() == ViewMsg_VisitedLink_Reset::ID)
+    if (msg->type() == ChromeViewMsg_VisitedLink_Add::ID) {
+      void* iter = NULL;
+      std::vector<uint64> fingerprints;
+      CHECK(IPC::ReadParam(msg, &iter, &fingerprints));
+      counting_profile->CountAddEvent(fingerprints.size());
+    } else if (msg->type() == ChromeViewMsg_VisitedLink_Reset::ID) {
       counting_profile->CountResetEvent();
+    }
 
     delete msg;
     return true;
@@ -558,53 +559,44 @@ class VisitRelayingRenderProcessHost : public BrowserRenderProcessHost {
 };
 
 class VisitedLinkRenderProcessHostFactory
-    : public MockRenderProcessHostFactory {
+    : public content::RenderProcessHostFactory {
  public:
   VisitedLinkRenderProcessHostFactory()
-      : MockRenderProcessHostFactory(),
-        relay_mode_(false) {}
-  virtual RenderProcessHost* CreateRenderProcessHost(Profile* profile) const {
-    if (relay_mode_)
-      return new VisitRelayingRenderProcessHost(profile);
-    else
-      return new VisitCountingRenderProcessHost(profile);
+      : content::RenderProcessHostFactory() {}
+  virtual content::RenderProcessHost* CreateRenderProcessHost(
+      content::BrowserContext* browser_context) const OVERRIDE {
+    return new VisitRelayingRenderProcessHost(browser_context);
   }
 
-  void set_relay_mode(bool mode) { relay_mode_ = mode; }
-
  private:
-  bool relay_mode_;
 
   DISALLOW_COPY_AND_ASSIGN(VisitedLinkRenderProcessHostFactory);
 };
 
-class VisitedLinkEventsTest : public RenderViewHostTestHarness {
+class VisitedLinkEventsTest : public ChromeRenderViewHostTestHarness {
  public:
   VisitedLinkEventsTest()
-      : RenderViewHostTestHarness(),
+      : ui_thread_(BrowserThread::UI, &message_loop_),
         file_thread_(BrowserThread::FILE, &message_loop_) {}
-  ~VisitedLinkEventsTest() {
-    // This ends up using the file thread to schedule the delete.
-    profile_.reset();
-    message_loop_.RunAllPending();
-  }
+  virtual ~VisitedLinkEventsTest() {}
   virtual void SetFactoryMode() {}
   virtual void SetUp() {
     SetFactoryMode();
-    event_listener_.reset(new VisitedLinkEventListener());
     rvh_factory_.set_render_process_host_factory(&vc_rph_factory_);
-    profile_.reset(new VisitCountingProfile(event_listener_.get()));
-    RenderViewHostTestHarness::SetUp();
+    browser_context_.reset(new VisitCountingProfile());
+    ChromeRenderViewHostTestHarness::SetUp();
   }
 
   VisitCountingProfile* profile() const {
-    return static_cast<VisitCountingProfile*>(profile_.get());
+    return static_cast<VisitCountingProfile*>(browser_context_.get());
   }
 
   void WaitForCoalescense() {
     // Let the timer fire.
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-                                            new MessageLoop::QuitTask(), 110);
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        MessageLoop::QuitClosure(),
+        base::TimeDelta::FromMilliseconds(110));
     MessageLoop::current()->Run();
   }
 
@@ -612,20 +604,15 @@ class VisitedLinkEventsTest : public RenderViewHostTestHarness {
   VisitedLinkRenderProcessHostFactory vc_rph_factory_;
 
  private:
-  scoped_ptr<VisitedLinkEventListener> event_listener_;
-  BrowserThread file_thread_;
+  content::TestBrowserThread ui_thread_;
+  content::TestBrowserThread file_thread_;
 
   DISALLOW_COPY_AND_ASSIGN(VisitedLinkEventsTest);
 };
 
-class VisitedLinkRelayTest : public VisitedLinkEventsTest {
- public:
-  virtual void SetFactoryMode() { vc_rph_factory_.set_relay_mode(true); }
-};
-
 TEST_F(VisitedLinkEventsTest, Coalescense) {
   // add some URLs to master.
-  VisitedLinkMaster* master = profile_->GetVisitedLinkMaster();
+  VisitedLinkMaster* master = profile()->GetVisitedLinkMaster();
   // Add a few URLs.
   master->AddURL(GURL("http://acidtests.org/"));
   master->AddURL(GURL("http://google.com/"));
@@ -676,9 +663,9 @@ TEST_F(VisitedLinkEventsTest, Coalescense) {
   EXPECT_EQ(1, profile()->reset_event_count());
 }
 
-TEST_F(VisitedLinkRelayTest, Basics) {
-  VisitedLinkMaster* master = profile_->GetVisitedLinkMaster();
-  rvh()->CreateRenderView(string16());
+TEST_F(VisitedLinkEventsTest, Basics) {
+  VisitedLinkMaster* master = profile()->GetVisitedLinkMaster();
+  rvh()->CreateRenderView(string16(), -1);
 
   // Add a few URLs.
   master->AddURL(GURL("http://acidtests.org/"));
@@ -700,9 +687,9 @@ TEST_F(VisitedLinkRelayTest, Basics) {
   EXPECT_EQ(1, profile()->reset_event_count());
 }
 
-TEST_F(VisitedLinkRelayTest, TabVisibility) {
-  VisitedLinkMaster* master = profile_->GetVisitedLinkMaster();
-  rvh()->CreateRenderView(string16());
+TEST_F(VisitedLinkEventsTest, TabVisibility) {
+  VisitedLinkMaster* master = profile()->GetVisitedLinkMaster();
+  rvh()->CreateRenderView(string16(), -1);
 
   // Simulate tab becoming inactive.
   rvh()->WasHidden();
@@ -745,30 +732,3 @@ TEST_F(VisitedLinkRelayTest, TabVisibility) {
   EXPECT_EQ(1, profile()->add_event_count());
   EXPECT_EQ(1, profile()->reset_event_count());
 }
-
-TEST_F(VisitedLinkRelayTest, WebViewReadiness) {
-  VisitedLinkMaster* master = profile_->GetVisitedLinkMaster();
-
-  // Add a few URLs.
-  master->AddURL(GURL("http://acidtests.org/"));
-  master->AddURL(GURL("http://google.com/"));
-  master->AddURL(GURL("http://chromium.org/"));
-
-  WaitForCoalescense();
-
-  std::set<GURL> deleted_urls;
-  deleted_urls.insert(GURL("http://acidtests.org/"));
-  master->DeleteURLs(deleted_urls);
-
-  // We shouldn't have any events, because RenderView hasn't been created,
-  // and we ensure that updates are sent until it is.
-  EXPECT_EQ(0, profile()->add_event_count());
-  EXPECT_EQ(0, profile()->reset_event_count());
-
-  rvh()->CreateRenderView(string16());
-
-  // We should now have just a reset event: adds are eaten up by a reset
-  // that followed.
-  EXPECT_EQ(0, profile()->add_event_count());
-  EXPECT_EQ(1, profile()->reset_event_count());
-}
