@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,26 +7,31 @@
 #pragma once
 
 #include <map>
+#include <string>
 #include <vector>
 
-#include "app/sql/init_status.h"
+#include "base/callback_forward.h"
 #include "base/file_path.h"
-#include "base/ref_counted.h"
+#include "base/location.h"
+#include "base/message_loop_helpers.h"
+#include "base/memory/ref_counted.h"
 #include "base/synchronization/lock.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/search_engines/template_url_id.h"
-#include "webkit/glue/form_field.h"
+#include "content/public/browser/browser_thread.h"
+#include "sql/init_status.h"
 
+class AutocompleteSyncableService;
 class AutofillChange;
-class AutoFillProfile;
+class AutofillProfile;
+class AutofillProfileSyncableService;
 class CreditCard;
 class GURL;
 #if defined(OS_WIN)
 struct IE7PasswordInfo;
 #endif
 class MessageLoop;
+class Profile;
 class SkBitmap;
-class Task;
 class TemplateURL;
 class WebDatabase;
 
@@ -34,9 +39,16 @@ namespace base {
 class Thread;
 }
 
-namespace webkit_glue {
+namespace webkit {
+namespace forms {
+struct FormField;
 struct PasswordForm;
 }
+}
+
+namespace webkit_glue {
+struct WebIntentServiceData;
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -69,10 +81,11 @@ typedef enum {
   TOKEN_RESULT,                // WDResult<std::vector<std::string>>
   AUTOFILL_VALUE_RESULT,       // WDResult<std::vector<string16>>
   AUTOFILL_CHANGES,            // WDResult<std::vector<AutofillChange>>
-  AUTOFILL_PROFILE_RESULT,     // WDResult<AutoFillProfile>
-  AUTOFILL_PROFILES_RESULT,    // WDResult<std::vector<AutoFillProfile*>>
+  AUTOFILL_PROFILE_RESULT,     // WDResult<AutofillProfile>
+  AUTOFILL_PROFILES_RESULT,    // WDResult<std::vector<AutofillProfile*>>
   AUTOFILL_CREDITCARD_RESULT,  // WDResult<CreditCard>
-  AUTOFILL_CREDITCARDS_RESULT  // WDResult<std::vector<CreditCard*>>
+  AUTOFILL_CREDITCARDS_RESULT, // WDResult<std::vector<CreditCard*>>
+  WEB_INTENTS_RESULT           // WDResult<std::vector<string16>>
 } WDResultType;
 
 typedef std::vector<AutofillChange> AutofillChangeList;
@@ -99,6 +112,11 @@ struct WDKeywordsResult {
   int64 default_search_provider_id;
   // Version of the built-in keywords. A value of 0 indicates a first run.
   int builtin_keyword_version;
+  // Backup of the default search provider. NULL if the backup is invalid.
+  TemplateURL* default_search_provider_backup;
+  // Indicates if default search provider has been changed by something
+  // other than user's action in the browser.
+  bool did_default_search_provider_change;
 };
 
 //
@@ -161,8 +179,8 @@ template <class T> class WDObjectResult : public WDTypedResult {
 class WebDataServiceConsumer;
 
 class WebDataService
-    : public base::RefCountedThreadSafe<WebDataService,
-                                        BrowserThread::DeleteOnUIThread> {
+    : public base::RefCountedThreadSafe<
+          WebDataService, content::BrowserThread::DeleteOnUIThread> {
  public:
   // All requests return an opaque handle of the following type.
   typedef int Handle;
@@ -183,8 +201,12 @@ class WebDataService
     virtual ~WebDataRequest();
 
     Handle GetHandle() const;
-    WebDataServiceConsumer* GetConsumer() const;
-    bool IsCancelled() const;
+
+    // Retrieves the |consumer_| set in the constructor.  If the request was
+    // cancelled via the |Cancel()| method then |true| is returned and
+    // |*consumer| is set to NULL.  The |consumer| parameter may be set to NULL
+    // if only the return value is desired.
+    bool IsCancelled(WebDataServiceConsumer** consumer) const;
 
     // This can be invoked from any thread. From this point we assume that
     // our consumer_ reference is invalid.
@@ -203,8 +225,13 @@ class WebDataService
     scoped_refptr<WebDataService> service_;
     MessageLoop* message_loop_;
     Handle handle_;
-    bool canceled_;
+
+    // A lock to protect against simultaneous cancellations of the request.
+    // Cancellation affects both the |cancelled_| flag and |consumer_|.
+    mutable base::Lock cancel_lock_;
+    bool cancelled_;
     WebDataServiceConsumer* consumer_;
+
     WDTypedResult* result_;
 
     DISALLOW_COPY_AND_ASSIGN(WebDataRequest);
@@ -228,9 +255,7 @@ class WebDataService
     virtual ~GenericRequest() {
     }
 
-    T GetArgument() {
-      return arg_;
-    }
+    const T& arg() const { return arg_; }
 
    private:
     T arg_;
@@ -251,13 +276,9 @@ class WebDataService
 
     virtual ~GenericRequest2() { }
 
-    T GetArgument1() {
-      return arg1_;
-    }
+    const T& arg1() const { return arg1_; }
 
-    U GetArgument2() {
-      return arg2_;
-    }
+    const U& arg2() const { return arg2_; }
 
    private:
     T arg1_;
@@ -266,6 +287,13 @@ class WebDataService
 
   WebDataService();
 
+  // Notifies listeners on the UI thread that multiple changes have been made to
+  // to Autofill records of the database.
+  // NOTE: This method is intended to be called from the DB thread.  It
+  // it asynchronously notifies listeners on the UI thread.
+  // |web_data_service| may be NULL for testing purposes.
+  static void NotifyOfMultipleAutofillChanges(WebDataService* web_data_service);
+
   // Initializes the web data service. Returns false on failure
   // Takes the path of the profile directory as its argument.
   bool Init(const FilePath& profile_path);
@@ -298,7 +326,7 @@ class WebDataService
   // done on the background thread.
   //
   // Many of the keyword related methods do not return a handle. This is because
-  // the caller (TemplateURLModel) does not need to know when the request is
+  // the caller (TemplateURLService) does not need to know when the request is
   // done.
   void AddKeyword(const TemplateURL& url);
 
@@ -337,6 +365,31 @@ class WebDataService
   // specified web app.
   Handle GetWebAppImages(const GURL& app_url, WebDataServiceConsumer* consumer);
 
+  //////////////////////////////////////////////////////////////////////////////
+  //
+  // Web Intents
+  //
+  //////////////////////////////////////////////////////////////////////////////
+
+  // Adds a web intent service registration.
+  void AddWebIntentService(const webkit_glue::WebIntentServiceData& service);
+
+  // Removes a web intent service registration.
+  void RemoveWebIntentService(const webkit_glue::WebIntentServiceData& service);
+
+  // Get all web intent services registered for the specified |action|.
+  // |consumer| must not be NULL.
+  Handle GetWebIntentServices(const string16& action,
+                              WebDataServiceConsumer* consumer);
+
+  // Get all web intent services registered using the specified |service_url|.
+  // |consumer| must not be NULL.
+  Handle GetWebIntentServicesForURL(const string16& service_url,
+                                    WebDataServiceConsumer* consumer);
+
+  // Get all web intent services registered. |consumer| must not be NULL.
+  Handle GetAllWebIntentServices(WebDataServiceConsumer* consumer);
+
   //////////////////////////////////////////////////////////////////////////////
   //
   // Token Service
@@ -351,7 +404,7 @@ class WebDataService
   void RemoveAllTokens();
 
   // Null on failure. Success is WDResult<std::vector<std::string> >
-  Handle GetAllTokens(WebDataServiceConsumer* consumer);
+  virtual Handle GetAllTokens(WebDataServiceConsumer* consumer);
 
   //////////////////////////////////////////////////////////////////////////////
   //
@@ -363,13 +416,13 @@ class WebDataService
   //////////////////////////////////////////////////////////////////////////////
 
   // Adds |form| to the list of remembered password forms.
-  void AddLogin(const webkit_glue::PasswordForm& form);
+  void AddLogin(const webkit::forms::PasswordForm& form);
 
   // Updates the remembered password form.
-  void UpdateLogin(const webkit_glue::PasswordForm& form);
+  void UpdateLogin(const webkit::forms::PasswordForm& form);
 
   // Removes |form| from the list of remembered password forms.
-  void RemoveLogin(const webkit_glue::PasswordForm& form);
+  void RemoveLogin(const webkit::forms::PasswordForm& form);
 
   // Removes all logins created in the specified daterange
   void RemoveLoginsCreatedBetween(const base::Time& delete_begin,
@@ -382,7 +435,7 @@ class WebDataService
   // |consumer| will be notified when the request is done. The result is of
   // type WDResult<std::vector<PasswordForm*>>.
   // The result will be null on failure. The |consumer| owns all PasswordForm's.
-  Handle GetLogins(const webkit_glue::PasswordForm& form,
+  Handle GetLogins(const webkit::forms::PasswordForm& form,
                    WebDataServiceConsumer* consumer);
 
   // Gets the complete list of password forms that have not been blacklisted and
@@ -415,12 +468,13 @@ class WebDataService
 
   //////////////////////////////////////////////////////////////////////////////
   //
-  // AutoFill.
+  // Autofill.
   //
   //////////////////////////////////////////////////////////////////////////////
 
   // Schedules a task to add form fields to the web database.
-  virtual void AddFormFields(const std::vector<webkit_glue::FormField>& fields);
+  virtual void AddFormFields(
+      const std::vector<webkit::forms::FormField>& fields);
 
   // Initiates the request for a vector of values which have been entered in
   // form input fields named |name|.  The method OnWebDataServiceRequestDone of
@@ -437,31 +491,35 @@ class WebDataService
   void RemoveFormValueForElementName(const string16& name,
                                      const string16& value);
 
-  // Schedules a task to add an AutoFill profile to the web database.
-  void AddAutoFillProfileGUID(const AutoFillProfile& profile);
+  // Schedules a task to add an Autofill profile to the web database.
+  void AddAutofillProfile(const AutofillProfile& profile);
 
-  // Schedules a task to update an AutoFill profile in the web database.
-  void UpdateAutoFillProfileGUID(const AutoFillProfile& profile);
+  // Schedules a task to update an Autofill profile in the web database.
+  void UpdateAutofillProfile(const AutofillProfile& profile);
 
-  // Schedules a task to remove an AutoFill profile from the web database.
+  // Schedules a task to remove an Autofill profile from the web database.
   // |guid| is the identifer of the profile to remove.
-  void RemoveAutoFillProfileGUID(const std::string& guid);
+  void RemoveAutofillProfile(const std::string& guid);
 
-  // Initiates the request for all AutoFill profiles.  The method
+  // Initiates the request for all Autofill profiles.  The method
   // OnWebDataServiceRequestDone of |consumer| gets called when the request is
   // finished, with the profiles included in the argument |result|.  The
   // consumer owns the profiles.
-  Handle GetAutoFillProfiles(WebDataServiceConsumer* consumer);
+  Handle GetAutofillProfiles(WebDataServiceConsumer* consumer);
+
+  // Remove "trashed" profile guids from the web database and optionally send
+  // notifications to tell Sync that the items have been removed.
+  void EmptyMigrationTrash(bool notify_sync);
 
   // Schedules a task to add credit card to the web database.
-  void AddCreditCardGUID(const CreditCard& credit_card);
+  void AddCreditCard(const CreditCard& credit_card);
 
   // Schedules a task to update credit card in the web database.
-  void UpdateCreditCardGUID(const CreditCard& credit_card);
+  void UpdateCreditCard(const CreditCard& credit_card);
 
   // Schedules a task to remove a credit card from the web database.
   // |guid| is identifer of the credit card to remove.
-  void RemoveCreditCardGUID(const std::string& guid);
+  void RemoveCreditCard(const std::string& guid);
 
   // Initiates the request for all credit cards.  The method
   // OnWebDataServiceRequestDone of |consumer| gets called when the request is
@@ -469,19 +527,29 @@ class WebDataService
   // consumer owns the credit cards.
   Handle GetCreditCards(WebDataServiceConsumer* consumer);
 
-  // Removes AutoFill records from the database.
-  void RemoveAutoFillProfilesAndCreditCardsModifiedBetween(
+  // Removes Autofill records from the database.
+  void RemoveAutofillProfilesAndCreditCardsModifiedBetween(
       const base::Time& delete_begin,
       const base::Time& delete_end);
 
+  // Returns the syncable service for Autofill addresses and credit cards stored
+  // in this table. The returned service is owned by |this| object.
+  virtual AutofillProfileSyncableService*
+      GetAutofillProfileSyncableService() const;
+
+  // Returns the syncable service for field autocomplete stored in this table.
+  // The returned service is owned by |this| object.
+  virtual AutocompleteSyncableService*
+      GetAutocompleteSyncableService() const;
+
   // Testing
 #ifdef UNIT_TEST
   void set_failed_init(bool value) { failed_init_ = value; }
 #endif
 
  protected:
-  friend class TemplateURLModelTest;
-  friend class TemplateURLModelTestingProfile;
+  friend class TemplateURLServiceTest;
+  friend class TemplateURLServiceTestingProfile;
   friend class WebDataServiceTest;
   friend class WebDataRequest;
 
@@ -502,10 +570,9 @@ class WebDataService
   //
   //////////////////////////////////////////////////////////////////////////////
  private:
-  friend class base::RefCountedThreadSafe<WebDataService>;
-  friend class BrowserThread;
-  friend class DeleteTask<WebDataService>;
-  friend class ShutdownTask;
+  friend struct content::BrowserThread::DeleteOnThread<
+      content::BrowserThread::UI>;
+  friend class base::DeleteHelper<WebDataService>;
 
   typedef GenericRequest2<std::vector<const TemplateURL*>,
                           std::vector<TemplateURL*> > SetKeywordsRequest;
@@ -516,17 +583,24 @@ class WebDataService
   // Initialize the database, if it hasn't already been initialized.
   void InitializeDatabaseIfNecessary();
 
+  // Initialize any syncable services.
+  void InitializeSyncableServices();
+
   // The notification method.
   void NotifyDatabaseLoadedOnUIThread();
 
   // Commit any pending transaction and deletes the database.
   void ShutdownDatabase();
 
+  // Deletes the syncable services.
+  void ShutdownSyncableServices();
+
   // Commit the current transaction and creates a new one.
   void Commit();
 
   // Schedule a task on our worker thread.
-  void ScheduleTask(Task* t);
+  void ScheduleTask(const tracked_objects::Location& from_here,
+                    const base::Closure& task);
 
   // Schedule a commit if one is not already pending.
   void ScheduleCommit();
@@ -556,6 +630,19 @@ class WebDataService
   void RemoveWebAppImpl(GenericRequest<GURL>* request);
   void GetWebAppImagesImpl(GenericRequest<GURL>* request);
 
+  //////////////////////////////////////////////////////////////////////////////
+  //
+  // Web Intents.
+  //
+  //////////////////////////////////////////////////////////////////////////////
+  void AddWebIntentServiceImpl(
+      GenericRequest<webkit_glue::WebIntentServiceData>* request);
+  void RemoveWebIntentServiceImpl(
+      GenericRequest<webkit_glue::WebIntentServiceData>* request);
+  void GetWebIntentServicesImpl(GenericRequest<string16>* request);
+  void GetWebIntentServicesForURLImpl(GenericRequest<string16>* request);
+  void GetAllWebIntentServicesImpl(GenericRequest<std::string>* request);
+
   //////////////////////////////////////////////////////////////////////////////
   //
   // Token Service.
@@ -572,12 +659,12 @@ class WebDataService
   // Password manager.
   //
   //////////////////////////////////////////////////////////////////////////////
-  void AddLoginImpl(GenericRequest<webkit_glue::PasswordForm>* request);
-  void UpdateLoginImpl(GenericRequest<webkit_glue::PasswordForm>* request);
-  void RemoveLoginImpl(GenericRequest<webkit_glue::PasswordForm>* request);
+  void AddLoginImpl(GenericRequest<webkit::forms::PasswordForm>* request);
+  void UpdateLoginImpl(GenericRequest<webkit::forms::PasswordForm>* request);
+  void RemoveLoginImpl(GenericRequest<webkit::forms::PasswordForm>* request);
   void RemoveLoginsCreatedBetweenImpl(
       GenericRequest2<base::Time, base::Time>* request);
-  void GetLoginsImpl(GenericRequest<webkit_glue::PasswordForm>* request);
+  void GetLoginsImpl(GenericRequest<webkit::forms::PasswordForm>* request);
   void GetAutofillableLoginsImpl(WebDataRequest* request);
   void GetBlacklistLoginsImpl(WebDataRequest* request);
 #if defined(OS_WIN)
@@ -588,26 +675,27 @@ class WebDataService
 
   //////////////////////////////////////////////////////////////////////////////
   //
-  // AutoFill.
+  // Autofill.
   //
   //////////////////////////////////////////////////////////////////////////////
   void AddFormElementsImpl(
-      GenericRequest<std::vector<webkit_glue::FormField> >* request);
+      GenericRequest<std::vector<webkit::forms::FormField> >* request);
   void GetFormValuesForElementNameImpl(WebDataRequest* request,
       const string16& name, const string16& prefix, int limit);
   void RemoveFormElementsAddedBetweenImpl(
       GenericRequest2<base::Time, base::Time>* request);
   void RemoveFormValueForElementNameImpl(
       GenericRequest2<string16, string16>* request);
-  void AddAutoFillProfileGUIDImpl(GenericRequest<AutoFillProfile>* request);
-  void UpdateAutoFillProfileGUIDImpl(GenericRequest<AutoFillProfile>* request);
-  void RemoveAutoFillProfileGUIDImpl(GenericRequest<std::string>* request);
-  void GetAutoFillProfilesImpl(WebDataRequest* request);
-  void AddCreditCardGUIDImpl(GenericRequest<CreditCard>* request);
-  void UpdateCreditCardGUIDImpl(GenericRequest<CreditCard>* request);
-  void RemoveCreditCardGUIDImpl(GenericRequest<std::string>* request);
+  void AddAutofillProfileImpl(GenericRequest<AutofillProfile>* request);
+  void UpdateAutofillProfileImpl(GenericRequest<AutofillProfile>* request);
+  void RemoveAutofillProfileImpl(GenericRequest<std::string>* request);
+  void GetAutofillProfilesImpl(WebDataRequest* request);
+  void EmptyMigrationTrashImpl(GenericRequest<bool>* request);
+  void AddCreditCardImpl(GenericRequest<CreditCard>* request);
+  void UpdateCreditCardImpl(GenericRequest<CreditCard>* request);
+  void RemoveCreditCardImpl(GenericRequest<std::string>* request);
   void GetCreditCardsImpl(WebDataRequest* request);
-  void RemoveAutoFillProfilesAndCreditCardsModifiedBetweenImpl(
+  void RemoveAutofillProfilesAndCreditCardsModifiedBetweenImpl(
       GenericRequest2<base::Time, base::Time>* request);
 
   // True once initialization has started.
@@ -616,9 +704,18 @@ class WebDataService
   // The path with which to initialize the database.
   FilePath path_;
 
-  // Our database.
+  // Our database.  We own the |db_|, but don't use a |scoped_ptr| because the
+  // |db_| lifetime must be managed on the database thread.
   WebDatabase* db_;
 
+  // Syncable services for the database data.  We own the services, but don't
+  // use |scoped_ptr|s because the lifetimes must be managed on the database
+  // thread.
+  // Currently only Autocomplete and Autofill profiles use the new Sync API, but
+  // all the database data should migrate to this API over time.
+  AutocompleteSyncableService* autocomplete_syncable_service_;
+  AutofillProfileSyncableService* autofill_profile_syncable_service_;
+
   // Whether the database failed to initialize.  We use this to avoid
   // continually trying to reinit.
   bool failed_init_;
