@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,8 +8,8 @@
 
 #include "base/basictypes.h"
 #include "base/message_loop.h"
-#include "chrome/browser/browser_thread.h"
 #include "chrome/browser/webdata/web_data_service.h"
+#include "content/public/browser/browser_thread.h"
 
 template <class T>
 class AutofillWebDataServiceConsumer: public WebDataServiceConsumer {
@@ -19,6 +19,7 @@ class AutofillWebDataServiceConsumer: public WebDataServiceConsumer {
 
   virtual void OnWebDataServiceRequestDone(WebDataService::Handle handle,
                                            const WDTypedResult* result) {
+    using content::BrowserThread;
     DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
     handle_ = handle;
     const WDResult<T>* wrapped_result =
