@@ -1,35 +1,35 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 // Defines the IPC messages used by the automation interface.
 
-#include <string>
-#include <vector>
-
-#include "base/string16.h"
-#include "chrome/common/content_settings.h"
-#include "chrome/common/navigation_types.h"
-#include "chrome/test/automation/autocomplete_edit_proxy.h"
-#include "gfx/rect.h"
-#include "googleurl/src/gurl.h"
-#include "ipc/ipc_message_macros.h"
-#include "net/url_request/url_request_status.h"
-
-
-
 // NOTE: All IPC messages have either a routing_id of 0 (for asynchronous
 //       messages), or one that's been assigned by the proxy (for calls
 //       which expect a response).  The routing_id shouldn't be used for
 //       any other purpose in these message types.
 
 // NOTE: All the new IPC messages should go at the end.
-//       The IPC message IDs need to match the reference builds.  Since we now
-//       define the IDs based on __LINE__, to allow these IPC messages to be
-//       used to control an old version of Chrome we need the message IDs to
-//       remain the same.  This means that you should not change the line number
-//       of any of the messages below.  This will be fixed once Xcode supports
-//       __COUNTER__, in which case we can get rid of the __LINE__.
+//       The test <--> browser IPC message IDs need to match the reference
+//       builds.  Since we now define the IDs based on __LINE__, to allow these
+//       IPC messages to be used to control an old version of Chrome we need
+//       the message IDs to remain the same.  This means that you should not
+//       change the line number of these types of messages. You can, however,
+//       change the browser <--> renderer messages.
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 #define IPC_MESSAGE_START AutomationMsgStart
 
@@ -341,10 +341,10 @@ IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_WindowForBrowser,
                             bool /* success flag */,
                             int /* window handle */)
 
-// This message requests the AutocompleteEdit associated with the specified
-// browser handle.
-// The return value contains a success flag and the handle of the omnibox.
-IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_AutocompleteEditForBrowser,
+// TODO(phajdan.jr): Remove when the reference build is updated (this and
+// all others marked "DEPRECATED MESSAGE").
+// (intentionally blank line)
+IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_DEPRECATED_AutocompleteEditForBrowser,
                             int /* browser handle */,
                             bool /* success flag */,
                             int /* AutocompleteEdit handle */)
@@ -533,8 +533,8 @@ IPC_MESSAGE_ROUTED3(AutomationMsg_OpenURL,
 //   - int: handle of the tab
 // Response:
 //  - bool: whether the operation was successful.
-IPC_SYNC_MESSAGE_CONTROL1_0(AutomationMsg_WaitForTabToBeRestored,
-                            int)
+IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_WaitForTabToBeRestored,
+                            int, bool)
 
 // This message is an outgoing message from Chrome to an external host.
 // It is a notification that a navigation happened
@@ -552,15 +552,15 @@ IPC_MESSAGE_ROUTED1(AutomationMsg_DidNavigate,
 // Response:
 //  - bool: whether the operation was successful.
 //  - SecurityStyle: the security style of the tab.
-//  - int: the status of the server's ssl cert (0 means no errors or no ssl
-//         was used).
+//  - net::CertStatus: the status of the server's ssl cert (0 means no errors or
+//                     no ssl was used).
 //  - int: the insecure content state, 0 means no insecure contents.
 
 IPC_SYNC_MESSAGE_CONTROL1_4(AutomationMsg_GetSecurityState,
                             int,
                             bool,
-                            SecurityStyle,
-                            int,
+                            content::SecurityStyle,
+                            net::CertStatus,
                             int)
 
 // This message requests the page type of the page displayed in the specified
@@ -573,7 +573,7 @@ IPC_SYNC_MESSAGE_CONTROL1_4(AutomationMsg_GetSecurityState,
 IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_GetPageType,
                             int,
                             bool,
-                            PageType)
+                            content::PageType)
 
 // This message simulates the user action on the SSL blocking page showing in
 // the specified tab.  This message is only effective if an interstitial page
@@ -647,44 +647,44 @@ IPC_SYNC_MESSAGE_CONTROL4_1(AutomationMsg_SavePage,
                             int,
                             bool)
 
-// This message requests the text currently being displayed in the
-// AutocompleteEdit.  The parameter is the handle to the AutocompleteEdit.
-// The return value is a string indicating the text in the AutocompleteEdit.
-IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_AutocompleteEditGetText,
+// TODO(phajdan.jr): Remove when the reference build is updated (this and
+// all others marked "DEPRECATED MESSAGE").
+// (intentionally blank line)
+IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_DEPRECATED_AutocompleteEditGetText,
                             int /* autocomplete edit handle */,
                             bool /* the requested autocomplete edit exists */,
-                            std::wstring /* omnibox text */)
+                            string16 /* omnibox text */)
 
-// This message sets the text being displayed in the AutocompleteEdit.  The
-// first parameter is the handle to the omnibox and the second parameter is
-// the text to be displayed in the AutocompleteEdit.
-// The return value has no parameters and is returned when the operation has
-// completed.
-IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_AutocompleteEditSetText,
+// TODO(phajdan.jr): Remove when the reference build is updated (this and
+// all others marked "DEPRECATED MESSAGE").
+// (intentionally blank line)
+// (intentionally blank line)
+// (intentionally blank line)
+IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_DEPRECATED_AutocompleteEditSetText,
                             int /* autocomplete edit handle */,
-                            std::wstring /* text to set */,
+                            string16 /* text to set */,
                             bool /* the requested autocomplete edit exists */)
 
-// This message requests if a query to a autocomplete provider is still in
-// progress.  The first parameter in the request is the handle to the
-// autocomplete edit.
-// The first return value indicates if the request succeeded.
-// The second return value indicates if a query is still in progress.
+// TODO(phajdan.jr): Remove when the reference build is updated (this and
+// all others marked "DEPRECATED MESSAGE").
+// (intentionally blank line)
+// (intentionally blank line)
+// (intentionally blank line)
 IPC_SYNC_MESSAGE_CONTROL1_2( \
-    AutomationMsg_AutocompleteEditIsQueryInProgress,
+    AutomationMsg_DEPRECATED_AutocompleteEditIsQueryInProgress,
     int /* autocomplete edit handle*/,
     bool /* the requested autocomplete edit exists */,
     bool /* indicates if a query is in progress */)
 
-// This message requests a list of the autocomplete messages currently being
-// displayed by the popup.  The parameter in the request is a handle to the
-// autocomplete edit.
-// The first return value indicates if the request was successful, while
-// while the second is the actual list of matches.
-IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_AutocompleteEditGetMatches,
+// TODO(phajdan.jr): Remove when the reference build is updated (this and
+// all others marked "DEPRECATED MESSAGE").
+// (intentionally blank line)
+// (intentionally blank line)
+// (intentionally blank line)
+IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_DEPRECATED_AutocompleteEditGetMatches,
                             int /* autocomplete edit handle*/,
                             bool /* the requested autocomplete edit exists */,
-                            std::vector<AutocompleteMatchData> /* matches */)
+                            std::vector<int> /* matches */)
 
 // This message requests the execution of a browser command in the browser
 // for which the handle is specified.
@@ -744,27 +744,27 @@ IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_FindWindowLocation,
                             int /* x */,
                             int /* y */)
 
-// Is the Bookmark bar visible? The return value will indicate whether it is
-// visible or not and whether it is being animated into (or out of its place).
-IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_BookmarkBarVisibility,
+// Gets the bookmark bar visibility, animating and detached states.
+// TODO(phajdan.jr): Adjust the last param when the reference build is updated.
+IPC_SYNC_MESSAGE_CONTROL1_3(AutomationMsg_BookmarkBarVisibility,
                             int /* browser_handle */,
                             bool, /* is_visible */
-                            bool  /* still_animating */)
+                            bool, /* still_animating */ bool /* is_detached */)
 
 // This message requests the number of related info bars opened.  It
 // returns -1 if an error occurred.
 IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_GetInfoBarCount,
                             int /* tab_handle */,
-                            int /* info bar count */)
+                            size_t /* info bar count */)
 
 // This message triggers the action associated with the "accept" button in
 // the info-bar at the specified index.  If |wait for navigation| is true, it
 // won't return until a navigation has occurred.
 IPC_SYNC_MESSAGE_CONTROL3_1(AutomationMsg_ClickInfoBarAccept,
                             int /* tab_handle */,
-                            int /* info bar index */,
+                            size_t /* info bar index */,
                             bool /* wait for navigation */,
-// This line blank on purpose, see comment atop file about __LINE__.
+// (intentionally blank line), see comment atop file about __LINE__.
                             /* navigation result */
                             AutomationMsg_NavigationResponseValues)
 
@@ -779,7 +779,7 @@ IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_GetLastNavigationTime,
 IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_WaitForNavigation,
                             int /* tab_handle */,
                             int64 /* last navigation time */,
-// This line blank on purpose, see comment atop file about __LINE__.
+// (intentionally blank line), see comment atop file about __LINE__.
                             /* navigation result */
                             AutomationMsg_NavigationResponseValues)
 
@@ -852,7 +852,7 @@ IPC_MESSAGE_ROUTED2(AutomationMsg_NavigationFailed,
                     int,
                     GURL)
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
 // This message is an outgoing message from an automation client to Chrome.
 // It is used to reposition a chrome tab window.
 IPC_MESSAGE_CONTROL2(AutomationMsg_TabReposition,
@@ -876,9 +876,9 @@ IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_TabIndex,
                             int)
 
 // This message requests the handle (int64 app-unique identifier) of
-// a valid normal browser window, i.e. normal type and non-incognito mode.
+// a valid tabbed browser window, i.e. normal type and non-incognito mode.
 // On error, the returned handle value is 0.
-IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_FindNormalBrowserWindow,
+IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_FindTabbedBrowserWindow,
                             int)
 
 // This message requests the number of normal browser windows, i.e. normal
@@ -887,19 +887,19 @@ IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_FindNormalBrowserWindow,
 IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_NormalBrowserWindowCount,
                             int)
 
-// Used to put the browser into "extension automation mode" for a given
-// set of Chrome Extensions API functions for the current profile, or turn
-// off automation mode.  The specified tab is used as the conduit for all
-// automated API functions.  It must be an external tab (as in
-// AutomationMsg_CreateExternalTab).
-IPC_MESSAGE_CONTROL2(AutomationMsg_SetEnableExtensionAutomation,
-                     // Tab handle.
+// DEPRECATED MESSAGE - But we must leave this comment and message so as
+// not to perturb line numbers (see comment at top of file re __LINE__).
+// TODO(phajdan.jr): Remove when the reference build is updated (this and
+// all others marked "DEPRECATED MESSAGE").
+// (intentionally blank line)
+IPC_MESSAGE_CONTROL2(AutomationMsg_DeprecatedMessageOne,
+                     // (intentionally blank line)
                      int,
-                     // Empty to disable automation, non-empty to enable
-                     // automation of the specified API functions, single
-                     // entry of "*" to enable automation of all API
-                     // functions.
-                     std::vector<std::string>)
+                     // (intentionally blank line)
+                     // (intentionally blank line)
+                     // (intentionally blank line)
+                     // (intentionally blank line)
+                     int)
 
 // This message tells the browser to start using the new proxy configuration
 // represented by the given JSON string. The parameters used in the JSON
@@ -927,7 +927,7 @@ IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_GetBrowserLocale,
 
 #if defined(OS_WIN)
 IPC_MESSAGE_ROUTED3(AutomationMsg_ForwardContextMenuToExternalHost,
-                    HANDLE /* source menu handle */,
+                    ContextMenuModel /* description of menu */,
                     int    /* align flags */,
                     MiniContextMenuParams /* params */)
 
@@ -1128,25 +1128,25 @@ IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_GetMetricEventDuration,
 IPC_MESSAGE_ROUTED1(AutomationMsg_RequestGoToHistoryEntryOffset,
                     int)   // numbers of entries (negative or positive)
 
-// Silently install the extension in the given crx file.
-IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_InstallExtension,
+// DEPRECATED MESSAGE.
+IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_DEPRECATED_InstallExtension,
                             FilePath /* full path to crx file */,
-                            AutomationMsg_ExtensionResponseValues)
+                            int)
 
-// Silently load the extension in the given directory.  This expects an
-// extension expanded into the directory, not a crx file.
-IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_LoadExpandedExtension,
-                            FilePath /* root directory of extension */,
-                            AutomationMsg_ExtensionResponseValues)
+// DEPRECATED MESSAGE - But we must leave this comment and message so as
+// not to perturb line numbers (see comment at top of file re __LINE__).
+IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_DeprecatedMessageTwo,
+                            int,
+                            int)
 
-// Retrieves a list of the root directories of all enabled extensions
-// that have been installed into Chrome by dropping a .crx file onto
-// Chrome or an equivalent action (including loaded extensions).
-// Other types of extensions are not included on the list (e.g. "component",
-// "app" or "external" extensions) since since CEEE does not yet support them
-// (and it actually only support a single extension in its profile for now).
-IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_GetEnabledExtensions,
-                            std::vector<FilePath>)
+// DEPRECATED MESSAGE - But we must leave this comment and message so as
+// not to perturb line numbers (see comment at top of file re __LINE__).
+// (intentionally blank line)
+// (intentionally blank line)
+// (intentionally blank line)
+// (intentionally blank line)
+IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_DeprecatedMessageThree,
+                            int)
 
 // This message requests the type of the window with the given handle. The
 // return value contains the type (Browser::Type), or -1 if the request
@@ -1300,12 +1300,12 @@ IPC_SYNC_MESSAGE_CONTROL2_2(AutomationMsg_SendJSONRequest,
                             std::string /* JSON response */,
                             bool /* success */)
 
-// Installs an extension from the crx file and returns its id.
-// On error, |extension handle| will be 0.
-IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_InstallExtensionAndGetHandle,
-                            FilePath     /* full path to crx file */,
-                            bool         /* with UI */,
-                            int          /* extension handle */)
+// Installs an extension from a crx file or unpacked extension folder
+// and returns its id. On error, |extension handle| will be 0.
+IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_InstallExtension,
+                            FilePath /* full path to crx or unpacked dir */,
+                            bool     /* with UI */,
+                            int      /* extension handle */)
 
 // Waits for the next extension test result. Sets |test result| as the
 // received result and |message| as any accompanying message with the
@@ -1414,11 +1414,11 @@ IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_WaitForTabCountToBecome,
 // Waits for the infobar count to reach given number.
 IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_WaitForInfoBarCount,
                             int /* tab handle */,
-                            int /* target count */,
+                            size_t /* target count */,
                             bool /* success */)
 
-// Waits for the autocomplete edit to receive focus.
-IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_WaitForAutocompleteEditFocus,
+// TODO(phajdan.jr): Remove this message.
+IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_DEPRECATED_WaitForAutocompleteEditFocus,
                             int /* autocomplete edit handle */,
                             bool /* success */)
 
@@ -1427,21 +1427,115 @@ IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_LoadBlockedPlugins,
                             int /* tab handle */,
                             bool /* success */)
 
+// TODO(phajdan.jr): Remove this message.
 // Captures the entire page for the tab, including those portions not in
 // view, and saves the image as a PNG in the given file location.
+// This message is deprecated, use the JSON testing interface for
+// similar functionality.
 // Request:
 //   -int: Tab handle
 //   -FilePath: Path to save the captured image to
 // Response:
 //   -bool: Whether the method succeeded
-IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_CaptureEntirePageAsPNG,
+IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_CaptureEntirePageAsPNG_Deprecated,
                             int,
                             FilePath,
                             bool)
 
 // Notify the JavaScript engine in the render to change its parameters
 // while performing stress testing.
-IPC_MESSAGE_ROUTED3(AutomationMsg_JavaScriptStressTestControl,
-                    int /* tab handle */,
-                    int /* command */,
-                    int /* type or run */)
+IPC_MESSAGE_CONTROL3(AutomationMsg_JavaScriptStressTestControl,
+                     int /* tab handle */,
+                     int /* command */,
+                     int /* type or run */)
+
+// This message posts a task to the PROCESS_LAUNCHER thread. Once processed
+// the response is sent back. This is useful when you want to make sure all
+// changes to the number of processes have completed.
+IPC_SYNC_MESSAGE_CONTROL0_0(AutomationMsg_WaitForProcessLauncherThreadToGoIdle)
+
+// Gets a handle of the browser that owns the given tab.
+IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_GetParentBrowserOfTab,
+                            int /* tab handle */,
+                            int /* browser handle */,
+                            bool /* success */)
+
+// This message is an outgoing message from Chrome to an external host.
+// It is a notification that a popup window position or dimentions have
+// changed
+// Request:
+//   gfx::Rect - the bounds of the window
+// Response:
+//   None expected
+IPC_MESSAGE_ROUTED1(AutomationMsg_MoveWindow,
+                    gfx::Rect /* window position and dimentions */)
+
+// Is the browser in application mode?
+IPC_SYNC_MESSAGE_CONTROL1_2(AutomationMsg_IsBrowserInApplicationMode,
+                            int /* browser_handle */,
+                            bool /* is_application */,
+                            bool /* success */)
+
+// Call BeginTracing on the browser TraceController. This will tell all
+// processes to start collecting trace events via base/debug/trace_event.h.
+IPC_SYNC_MESSAGE_CONTROL1_1(AutomationMsg_BeginTracing,
+                            std::string /* categories */,
+                            bool /* success */)
+
+// End tracing (called after BeginTracing). This blocks until tracing has
+// stopped on all processes and all the events are ready to be retrieved.
+IPC_SYNC_MESSAGE_CONTROL0_2(AutomationMsg_EndTracing,
+                            size_t /* num_trace_chunks */,
+                            bool /* success */)
+
+// Retrieve trace event data (called after EndTracing). Must call exactly
+// |num_trace_chunks| times.
+// TODO(jbates): See bug 100255, IPC send fails if message is too big. This
+// code can be removed if that limitation is fixed.
+IPC_SYNC_MESSAGE_CONTROL0_2(AutomationMsg_GetTracingOutput,
+                            std::string /* trace_chunk */,
+                            bool /* success */)
+
+// This message notifies the AutomationProvider to append a new tab to the
+// window with the given handle. The tab will be opened in the background
+// like it was middle-clicked.  The return value contains the index of
+// the new tab, or -1 if the request failed.
+// The second parameter is the url to be loaded in the new tab.
+IPC_SYNC_MESSAGE_CONTROL2_1(AutomationMsg_AppendBackgroundTab,
+                            int,
+                            GURL,
+                            int)
+
+// Used on Mac OS X to read the number of active Mach ports used in the browser
+// process.
+IPC_SYNC_MESSAGE_CONTROL0_1(AutomationMsg_GetMachPortCount,
+                            int /* number of Mach ports */)
+
+// Browser -> renderer messages.
+
+// Requests a snapshot.
+IPC_MESSAGE_ROUTED0(AutomationMsg_SnapshotEntirePage)
+
+// Renderer -> browser messages.
+
+// Sent as a response to |AutomationMsg_Snapshot|.
+IPC_MESSAGE_ROUTED3(AutomationMsg_SnapshotEntirePageACK,
+                    bool /* success */,
+                    std::vector<unsigned char> /* png bytes */,
+                    std::string /* error message */)
+
+// Sent when the renderer has scheduled a client redirect to occur.
+IPC_MESSAGE_ROUTED2(AutomationMsg_WillPerformClientRedirect,
+                    int64 /* frame_id */,
+                    double /* # of seconds till redirect will be performed */)
+
+// Sent when the renderer has completed or canceled a client redirect for a
+// particular frame. This message may be sent multiple times for the same
+// redirect.
+IPC_MESSAGE_ROUTED1(AutomationMsg_DidCompleteOrCancelClientRedirect,
+                    int64 /* frame_id */)
+
+// YOUR NEW MESSAGE MIGHT NOT BELONG HERE.
+// This is the section for renderer -> browser automation messages. If it is
+// an automation <-> browser message, put it above this section. The "no line
+// number change" applies only to the automation <-> browser messages.
