@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,24 +10,51 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/mac/crash_logging.h"
 #include "googleurl/src/gurl.h"
 
-class GPUInfo;
+class CommandLine;
+
+namespace content {
+struct GPUInfo;
+}
 
-#if defined(OS_WIN) || defined(OS_MACOSX)
 // The maximum number of active extensions we will report.
 // Also used in chrome/app, but we define it here to avoid a common->app
 // dependency.
 static const int kMaxReportedActiveExtensions = 10;
-#endif
+
+// The maximum number of command line switches to include in the crash
+// report's metadata. Note that the mini-dump itself will also contain the
+// (original) command line arguments within the PEB.
+// Also used in chrome/app, but we define it here to avoid a common->app
+// dependency.
+static const size_t kMaxSwitches = 15;
 
 namespace child_process_logging {
 
-#if defined(OS_LINUX)
+#if defined(OS_LINUX) || defined(OS_OPENBSD)
 // These are declared here so the crash reporter can access them directly in
 // compromised context without going through the standard library.
 extern char g_active_url[];
+extern char g_channel[];
 extern char g_client_id[];
+extern char g_extension_ids[];
+extern char g_gpu_vendor_id[];
+extern char g_gpu_device_id[];
+extern char g_gpu_driver_ver[];
+extern char g_gpu_ps_ver[];
+extern char g_gpu_vs_ver[];
+extern char g_num_extensions[];
+extern char g_num_switches[];
+extern char g_num_views[];
+extern char g_switches[];
+
+// Assume IDs are 32 bytes long.
+static const size_t kExtensionLen = 32;
+
+// Assume command line switches are less than 64 chars.
+static const size_t kSwitchLen = 64;
 #endif
 
 // Sets the URL that is logged if the child process crashes. Use GURL() to clear
@@ -53,7 +80,16 @@ void SetActiveExtensions(const std::set<std::string>& extension_ids);
 void SetNumberOfViews(int number_of_views);
 
 // Sets the data on the gpu to send along with crash reports.
-void SetGpuInfo(const GPUInfo& gpu_info);
+void SetGpuInfo(const content::GPUInfo& gpu_info);
+
+// Sets the command line arguments to send along with crash reports to the
+// values in |command_line|.
+void SetCommandLine(const CommandLine* command_line);
+
+#if defined(OS_LINUX) || defined(OS_OPENBSD)
+// Sets the product channel data to send along with crash reports to |channel|.
+void SetChannel(const std::string& channel);
+#endif
 
 // Simple wrapper class that sets the active URL in it's constructor and clears
 // the active URL in the destructor.
@@ -73,25 +109,20 @@ class ScopedActiveURLSetter {
 
 }  // namespace child_process_logging
 
-#if defined(OS_MACOSX) && __OBJC__
-
-@class NSString;
-
-typedef void (*SetCrashKeyValueFuncPtr)(NSString*, NSString*);
-typedef void (*ClearCrashKeyValueFuncPtr)(NSString*);
+#if defined(OS_MACOSX)
 
 namespace child_process_logging {
-void SetCrashKeyFunctions(SetCrashKeyValueFuncPtr set_key_func,
-                          ClearCrashKeyValueFuncPtr clear_key_func);
+
 void SetActiveURLImpl(const GURL& url,
-                      SetCrashKeyValueFuncPtr set_key_func,
-                      ClearCrashKeyValueFuncPtr clear_key_func);
+                      base::mac::SetCrashKeyValueFuncPtr set_key_func,
+                      base::mac::ClearCrashKeyValueFuncPtr clear_key_func);
 
 extern const int kMaxNumCrashURLChunks;
 extern const int kMaxNumURLChunkValueLength;
 extern const char *kUrlChunkFormatStr;
+
 }  // namespace child_process_logging
 
-#endif  // defined(OS_MACOSX) && __OBJC__
+#endif  // defined(OS_MACOSX)
 
 #endif  // CHROME_COMMON_CHILD_PROCESS_LOGGING_H_
