@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,20 +6,28 @@
 
 #import <Foundation/Foundation.h>
 
+#include "base/command_line.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
+#include "base/stringprintf.h"
+#include "base/sys_string_conversions.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/common/gpu_info.h"
 #include "chrome/installer/util/google_update_settings.h"
+#include "content/public/common/gpu_info.h"
 #include "googleurl/src/gurl.h"
 
 namespace child_process_logging {
 
+using base::mac::SetCrashKeyValueFuncPtr;
+using base::mac::ClearCrashKeyValueFuncPtr;
+using base::mac::SetCrashKeyValue;
+using base::mac::ClearCrashKey;
+
 const int kMaxNumCrashURLChunks = 8;
 const int kMaxNumURLChunkValueLength = 255;
 const char *kUrlChunkFormatStr = "url-chunk-%d";
 const char *kGuidParamName = "guid";
-const char *kGPUVendorIdParamName = "gpu-vendid";
+const char *kGPUVendorIdParamName = "gpu-venid";
 const char *kGPUDeviceIdParamName = "gpu-devid";
 const char *kGPUDriverVersionParamName = "gpu-driver";
 const char *kGPUPixelShaderVersionParamName = "gpu-psver";
@@ -29,19 +37,10 @@ const char *kNumberOfViews = "num-views";
 NSString* const kNumExtensionsName = @"num-extensions";
 NSString* const kExtensionNameFormat = @"extension-%d";
 
-static SetCrashKeyValueFuncPtr g_set_key_func;
-static ClearCrashKeyValueFuncPtr g_clear_key_func;
-
 // Account for the terminating null character.
 static const size_t kClientIdSize = 32 + 1;
 static char g_client_id[kClientIdSize];
 
-void SetCrashKeyFunctions(SetCrashKeyValueFuncPtr set_key_func,
-                          ClearCrashKeyValueFuncPtr clear_key_func) {
-  g_set_key_func = set_key_func;
-  g_clear_key_func = clear_key_func;
-}
-
 void SetActiveURLImpl(const GURL& url,
                       SetCrashKeyValueFuncPtr set_key_func,
                       ClearCrashKeyValueFuncPtr clear_key_func) {
@@ -93,8 +92,7 @@ void SetClientIdImpl(const std::string& client_id,
 }
 
 void SetActiveURL(const GURL& url) {
-  if (g_set_key_func && g_clear_key_func)
-    SetActiveURLImpl(url, g_set_key_func, g_clear_key_func);
+  SetActiveURLImpl(url, SetCrashKeyValue, ClearCrashKey);
 }
 
 void SetClientId(const std::string& client_id) {
@@ -102,8 +100,7 @@ void SetClientId(const std::string& client_id) {
   ReplaceSubstringsAfterOffset(&str, 0, "-", "");
 
   base::strlcpy(g_client_id, str.c_str(), kClientIdSize);
-  if (g_set_key_func)
-    SetClientIdImpl(str, g_set_key_func);
+    SetClientIdImpl(str, SetCrashKeyValue);
 
   std::wstring wstr = ASCIIToWide(str);
   GoogleUpdateSettings::SetMetricsId(wstr);
@@ -114,12 +111,10 @@ std::string GetClientId() {
 }
 
 void SetActiveExtensions(const std::set<std::string>& extension_ids) {
-  if (!g_set_key_func)
-    return;
-
   // Log the count separately to track heavy users.
   const int count = static_cast<int>(extension_ids.size());
-  g_set_key_func(kNumExtensionsName, [NSString stringWithFormat:@"%i", count]);
+  SetCrashKeyValue(kNumExtensionsName,
+                   [NSString stringWithFormat:@"%i", count]);
 
   // Record up to |kMaxReportedActiveExtensions| extensions, clearing
   // keys if there aren't that many.
@@ -127,10 +122,10 @@ void SetActiveExtensions(const std::set<std::string>& extension_ids) {
   for (int i = 0; i < kMaxReportedActiveExtensions; ++i) {
     NSString* key = [NSString stringWithFormat:kExtensionNameFormat, i];
     if (iter != extension_ids.end()) {
-      g_set_key_func(key, [NSString stringWithUTF8String:iter->c_str()]);
+      SetCrashKeyValue(key, [NSString stringWithUTF8String:iter->c_str()]);
       ++iter;
     } else {
-      g_clear_key_func(key);
+      ClearCrashKey(key);
     }
   }
 }
@@ -142,31 +137,30 @@ void SetGpuKeyValue(const char* param_name, const std::string& value_str,
   set_key_func(key, value);
 }
 
-void SetGpuInfoImpl(const GPUInfo& gpu_info,
+void SetGpuInfoImpl(const content::GPUInfo& gpu_info,
                     SetCrashKeyValueFuncPtr set_key_func) {
   SetGpuKeyValue(kGPUVendorIdParamName,
-                 base::UintToString(gpu_info.vendor_id()),
+                 base::StringPrintf("0x%04x", gpu_info.vendor_id),
                  set_key_func);
   SetGpuKeyValue(kGPUDeviceIdParamName,
-                 base::UintToString(gpu_info.device_id()),
+                 base::StringPrintf("0x%04x", gpu_info.device_id),
                  set_key_func);
   SetGpuKeyValue(kGPUDriverVersionParamName,
-                 gpu_info.driver_version(),
+                 gpu_info.driver_version,
                  set_key_func);
   SetGpuKeyValue(kGPUPixelShaderVersionParamName,
-                 base::UintToString(gpu_info.pixel_shader_version()),
+                 gpu_info.pixel_shader_version,
                  set_key_func);
   SetGpuKeyValue(kGPUVertexShaderVersionParamName,
-                 base::UintToString(gpu_info.vertex_shader_version()),
+                 gpu_info.vertex_shader_version,
                  set_key_func);
   SetGpuKeyValue(kGPUGLVersionParamName,
-                 base::UintToString(gpu_info.gl_version()),
+                 gpu_info.gl_version,
                  set_key_func);
 }
 
-void SetGpuInfo(const GPUInfo& gpu_info) {
-  if (g_set_key_func)
-    SetGpuInfoImpl(gpu_info, g_set_key_func);
+void SetGpuInfo(const content::GPUInfo& gpu_info) {
+    SetGpuInfoImpl(gpu_info, SetCrashKeyValue);
 }
 
 
@@ -178,8 +172,37 @@ void SetNumberOfViewsImpl(int number_of_views,
 }
 
 void SetNumberOfViews(int number_of_views) {
-  if (g_set_key_func)
-    SetNumberOfViewsImpl(number_of_views, g_set_key_func);
+    SetNumberOfViewsImpl(number_of_views, SetCrashKeyValue);
+}
+
+void SetCommandLine(const CommandLine* command_line) {
+  DCHECK(command_line);
+  if (!command_line)
+    return;
+
+  // These should match the corresponding strings in breakpad_win.cc.
+  NSString* const kNumSwitchesKey = @"num-switches";
+  NSString* const kSwitchKeyFormat = @"switch-%d";  // 1-based.
+
+  // Note the total number of switches, not including the exec path.
+  const CommandLine::StringVector& argv = command_line->argv();
+  SetCrashKeyValue(kNumSwitchesKey,
+                   [NSString stringWithFormat:@"%zu", argv.size() - 1]);
+
+  size_t key_i = 0;
+  for (size_t i = 1; i < argv.size() && key_i < kMaxSwitches; ++i) {
+    // TODO(shess): Skip boring switches.
+    NSString* key = [NSString stringWithFormat:kSwitchKeyFormat, (key_i + 1)];
+    NSString* value = base::SysUTF8ToNSString(argv[i]);
+    SetCrashKeyValue(key, value);
+    key_i++;
+  }
+
+  // Clear out any stale keys.
+  for (; key_i < kMaxSwitches; ++key_i) {
+    NSString* key = [NSString stringWithFormat:kSwitchKeyFormat, (key_i + 1)];
+    ClearCrashKey(key);
+  }
 }
 
 }  // namespace child_process_logging
