@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -20,9 +20,23 @@ extern const FilePath::CharType kBrowserProcessExecutableName[];
 extern const FilePath::CharType kHelperProcessExecutableName[];
 extern const FilePath::CharType kBrowserProcessExecutablePath[];
 extern const FilePath::CharType kHelperProcessExecutablePath[];
+extern const FilePath::CharType kBrowserProcessExecutableNameChromium[];
+extern const FilePath::CharType kHelperProcessExecutableNameChromium[];
+extern const FilePath::CharType kBrowserProcessExecutablePathChromium[];
+extern const FilePath::CharType kHelperProcessExecutablePathChromium[];
 #if defined(OS_MACOSX)
 extern const FilePath::CharType kFrameworkName[];
-#endif
+
+// The helper .app bundle name and executable name may have one of these
+// suffixes to identify specific features, or it may have no suffix at all.
+// This is a NULL-terminated array of strings. If kHelperFlavorSuffixes
+// contains "EN", "MF", and NULL, it indicates that if the normal helper is
+// named Chromium Helper.app, helper executables could show up at any of
+// Chromium Helper.app/Contents/MacOS/Chromium Helper,
+// Chromium Helper EN.app/Contents/MacOS/Chromium Helper EN, and
+// Chromium Helper MF.app/Contents/MacOS/Chromium Helper MF.
+extern const FilePath::CharType* const kHelperFlavorSuffixes[];
+#endif  // OS_MACOSX
 extern const wchar_t kBrowserAppName[];
 #if defined(OS_WIN)
 extern const wchar_t kStatusTrayWindowClass[];
@@ -30,9 +44,8 @@ extern const wchar_t kStatusTrayWindowClass[];
 extern const wchar_t kMessageWindowClass[];
 extern const wchar_t kCrashReportLog[];
 extern const wchar_t kTestingInterfaceDLL[];
-extern const wchar_t kNotSignedInProfile[];
-extern const wchar_t kNotSignedInID[];
-extern const char    kStatsFilename[];
+extern const char    kInitialProfile[];
+extern const char    kMultiProfileDirPrefix[];
 extern const wchar_t kBrowserResourcesDll[];
 extern const wchar_t kNaClAppName[];
 extern const FilePath::CharType kExtensionFileExtension[];
@@ -41,19 +54,20 @@ extern const FilePath::CharType kExtensionKeyFileExtension[];
 // filenames
 extern const FilePath::CharType kArchivedHistoryFilename[];
 extern const FilePath::CharType kCacheDirname[];
+extern const FilePath::CharType kCRLSetFilename[];
 extern const FilePath::CharType kMediaCacheDirname[];
 extern const FilePath::CharType kOffTheRecordMediaCacheDirname[];
 extern const FilePath::CharType kAppCacheDirname[];
-extern const FilePath::CharType kChromePluginDataDirname[];
 extern const FilePath::CharType kThemePackFilename[];
 extern const FilePath::CharType kCookieFilename[];
+extern const FilePath::CharType kOBCertFilename[];
 extern const FilePath::CharType kExtensionsCookieFilename[];
+extern const FilePath::CharType kIsolatedAppStateDirname[];
 extern const FilePath::CharType kFaviconsFilename[];
 extern const FilePath::CharType kHistoryFilename[];
 extern const FilePath::CharType kLocalStateFilename[];
 extern const FilePath::CharType kPreferencesFilename[];
 extern const FilePath::CharType kSafeBrowsingBaseFilename[];
-extern const FilePath::CharType kSafeBrowsingPhishingModelFilename[];
 extern const FilePath::CharType kSingletonCookieFilename[];
 extern const FilePath::CharType kSingletonSocketFilename[];
 extern const FilePath::CharType kSingletonLockFilename[];
@@ -61,7 +75,6 @@ extern const FilePath::CharType kThumbnailsFilename[];
 extern const FilePath::CharType kNewTabThumbnailsFilename[];
 extern const FilePath::CharType kTopSitesFilename[];
 extern const wchar_t kUserDataDirname[];
-extern const FilePath::CharType kUserScriptsDirname[];
 extern const FilePath::CharType kWebDataFilename[];
 extern const FilePath::CharType kBookmarksFileName[];
 extern const FilePath::CharType kHistoryBookmarksFileName[];
@@ -71,33 +84,40 @@ extern const FilePath::CharType kJumpListIconDirname[];
 extern const FilePath::CharType kWebAppDirname[];
 extern const FilePath::CharType kServiceStateFileName[];
 
-extern const unsigned int kMaxRendererProcessCount;
-extern const int kStatsMaxThreads;
-extern const int kStatsMaxCounters;
-
-// The maximum number of characters of the document's title that we're willing
-// to accept in the browser process.
-extern const size_t kMaxTitleChars;
-// The maximum number of characters in the URL that we're willing to accept
-// in the browser process. It is set low enough to avoid damage to the browser
-// but high enough that a web site can abuse location.hash for a little storage.
-// We have different values for "max accepted" and "max displayed" because
-// a data: URI may be legitimately massive, but the full URI would kill all
-// known operating systems if you dropped it into a UI control.
-extern const size_t kMaxURLChars;
-extern const size_t kMaxURLDisplayChars;
-
 extern const bool kRecordModeEnabled;
 
+// Most sequence numbers are used by a renderer when responding to a browser
+// request for histogram data.  This reserved number is used when a renderer
+// sends an unprovoked update, such as after a page has been loaded.  Using
+// this reserved constant avoids any chance of confusion with a response having
+// a browser-supplied sequence number.
 extern const int kHistogramSynchronizerReservedSequenceNumber;
 
-// The maximum number of session history entries per tab.
-extern const int kMaxSessionHistoryEntries;
-
 // The language code used when the language of a page could not be detected.
 // (Matches what the CLD -Compact Language Detection- library reports.)
 extern const char* const kUnknownLanguageCode;
 
+// If another javascript message box is displayed within
+// kJavascriptMessageExpectedDelay of a previous javascript message box being
+// dismissed, display an option to suppress future message boxes from this
+// contents.
+extern const int kJavascriptMessageExpectedDelay;
+
+// Are touch icons enabled? False by default.
+extern const bool kEnableTouchIcon;
+
+#if defined(OS_LINUX)
+// The highest and lowest assigned OOM score adjustment
+// (oom_score_adj) used by the OomPriority Manager.
+extern const int kLowestRendererOomScore;
+extern const int kHighestRendererOomScore;
+#endif
+
+#if defined(OS_WIN)
+// This is used by the PreRead experiment.
+extern const char kPreReadEnvironmentVariable[];
+#endif
+
 }  // namespace chrome
 
 #endif  // CHROME_COMMON_CHROME_CONSTANTS_H_
