@@ -1,19 +1,93 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/common/chrome_paths_internal.h"
 
-#import <Cocoa/Cocoa.h>
+#import <Foundation/Foundation.h>
+#include <string.h>
+
+#include <string>
 
 #include "base/base_paths.h"
 #include "base/logging.h"
-#include "base/mac/mac_util.h"
+#import "base/mac/foundation_util.h"
+#import "base/mac/mac_util.h"
+#import "base/mac/scoped_nsautorelease_pool.h"
 #include "base/path_service.h"
 #include "chrome/common/chrome_constants.h"
 
 namespace {
+
 const FilePath* g_override_versioned_directory = NULL;
+
+// Return a retained (NOT autoreleased) NSBundle* as the internal
+// implementation of chrome::OuterAppBundle(), which should be the only
+// caller.
+NSBundle* OuterAppBundleInternal() {
+  base::mac::ScopedNSAutoreleasePool pool;
+
+  if (!base::mac::AmIBundled()) {
+    // If unbundled (as in a test), there's no app bundle.
+    return nil;
+  }
+
+  if (!base::mac::IsBackgroundOnlyProcess()) {
+    // Shortcut: in the browser process, just return the main app bundle.
+    return [[NSBundle mainBundle] retain];
+  }
+
+  // From C.app/Contents/Versions/1.2.3.4, go up three steps to get to C.app.
+  FilePath versioned_dir = chrome::GetVersionedDirectory();
+  FilePath outer_app_dir = versioned_dir.DirName().DirName().DirName();
+  const char* outer_app_dir_c = outer_app_dir.value().c_str();
+  NSString* outer_app_dir_ns = [NSString stringWithUTF8String:outer_app_dir_c];
+
+  return [[NSBundle bundleWithPath:outer_app_dir_ns] retain];
+}
+
+const char* ProductDirNameInternal() {
+  base::mac::ScopedNSAutoreleasePool pool;
+
+  // Use OuterAppBundle() to get the main app's bundle. This key needs to live
+  // in the main app's bundle because it will be set differently on the canary
+  // channel, and the autoupdate system dictates that there can be no
+  // differences between channels within the versioned directory. This would
+  // normally use base::mac::FrameworkBundle(), but that references the
+  // framework bundle within the versioned directory. Ordinarily, the profile
+  // should not be accessed from non-browser processes, but those processes do
+  // attempt to get the profile directory, so direct them to look in the outer
+  // browser .app's Info.plist for the CrProductDirName key.
+  NSBundle* bundle = chrome::OuterAppBundle();
+  NSString* product_dir_name_ns =
+      [bundle objectForInfoDictionaryKey:@"CrProductDirName"];
+  const char* product_dir_name = [product_dir_name_ns fileSystemRepresentation];
+
+  if (!product_dir_name) {
+#if defined(GOOGLE_CHROME_BUILD)
+    product_dir_name = "Google/Chrome";
+#else
+    product_dir_name = "Chromium";
+#endif
+  }
+
+  // Leaked, but the only caller initializes a static with this result, so it
+  // only happens once, and that's OK.
+  return strdup(product_dir_name);
+}
+
+// ProductDirName returns the name of the directory inside
+// ~/Library/Application Support that should hold the product application
+// data. This can be overridden by setting the CrProductDirName key in the
+// outer browser .app's Info.plist. The default is "Google/Chrome" for
+// officially-branded builds, and "Chromium" for unbranded builds. For the
+// official canary channel, the Info.plist will have CrProductDirName set
+// to "Google/Chrome Canary".
+std::string ProductDirName() {
+  static const char* product_dir_name = ProductDirNameInternal();
+  return std::string(product_dir_name);
+}
+
 }  // namespace
 
 namespace chrome {
@@ -21,24 +95,7 @@ namespace chrome {
 bool GetDefaultUserDataDirectory(FilePath* result) {
   bool success = false;
   if (result && PathService::Get(base::DIR_APP_DATA, result)) {
-#if defined(GOOGLE_CHROME_BUILD)
-    *result = result->Append("Google").Append("Chrome");
-#else
-    *result = result->Append("Chromium");
-#endif
-    success = true;
-  }
-  return success;
-}
-
-bool GetChromeFrameUserDataDirectory(FilePath* result) {
-  bool success = false;
-  if (result && PathService::Get(base::DIR_APP_DATA, result)) {
-#if defined(GOOGLE_CHROME_BUILD)
-    *result = result->Append("Google").Append("Chrome Frame");
-#else
-    *result = result->Append("Chrome Frame");
-#endif
+    *result = result->Append(ProductDirName());
     success = true;
   }
   return success;
@@ -48,6 +105,10 @@ bool GetUserDocumentsDirectory(FilePath* result) {
   return base::mac::GetUserDirectory(NSDocumentDirectory, result);
 }
 
+bool GetGlobalApplicationSupportDirectory(FilePath* result) {
+  return base::mac::GetLocalDirectory(NSApplicationSupportDirectory, result);
+}
+
 void GetUserCacheDirectory(const FilePath& profile_dir, FilePath* result) {
   // If the profile directory is under ~/Library/Application Support,
   // use a suitable cache directory under ~/Library/Caches.  For
@@ -131,4 +192,11 @@ bool GetLocalLibraryDirectory(FilePath* result) {
   return base::mac::GetLocalDirectory(NSLibraryDirectory, result);
 }
 
+NSBundle* OuterAppBundle() {
+  // Cache this. Foundation leaks it anyway, and this should be the only call
+  // to OuterAppBundleInternal().
+  static NSBundle* bundle = OuterAppBundleInternal();
+  return bundle;
+}
+
 }  // namespace chrome
