@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,13 +8,13 @@
 
 #include "base/logging.h"
 #include "chrome/common/badge_util.h"
-#include "gfx/canvas_skia.h"
-#include "gfx/rect.h"
 #include "googleurl/src/gurl.h"
-#include "grit/app_resources.h"
+#include "grit/ui_resources.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/effects/SkGradientShader.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/gfx/canvas_skia.h"
+#include "ui/gfx/rect.h"
 
 namespace {
 
@@ -147,8 +147,8 @@ void ExtensionAction::PaintBadge(gfx::Canvas* canvas,
   if (badge_width >= kCenterAlignThreshold) {
     rect.fLeft = SkIntToScalar(
                      SkScalarFloor(SkIntToScalar(bounds.x()) +
-                                   SkIntToScalar(bounds.width() / 2.0) -
-                                   SkIntToScalar(badge_width / 2.0)));
+                                   SkIntToScalar(bounds.width()) / 2 -
+                                   SkIntToScalar(badge_width) / 2));
     rect.fRight = rect.fLeft + SkIntToScalar(badge_width);
   } else {
     rect.fRight = SkIntToScalar(bounds.right());
@@ -159,8 +159,8 @@ void ExtensionAction::PaintBadge(gfx::Canvas* canvas,
   rect_paint.setStyle(SkPaint::kFill_Style);
   rect_paint.setAntiAlias(true);
   rect_paint.setColor(background_color);
-  canvas->AsCanvasSkia()->drawRoundRect(rect, SkIntToScalar(2),
-                                        SkIntToScalar(2), rect_paint);
+  canvas->GetSkCanvas()->drawRoundRect(rect, SkIntToScalar(2),
+                                       SkIntToScalar(2), rect_paint);
 
   // Overlay the gradient. It is stretchy, so we do this in three parts.
   ResourceBundle& resource_bundle = ResourceBundle::GetSharedInstance();
@@ -171,24 +171,24 @@ void ExtensionAction::PaintBadge(gfx::Canvas* canvas,
   SkBitmap* gradient_center = resource_bundle.GetBitmapNamed(
       IDR_BROWSER_ACTION_BADGE_CENTER);
 
-  canvas->AsCanvasSkia()->drawBitmap(*gradient_left, rect.fLeft, rect.fTop);
+  canvas->GetSkCanvas()->drawBitmap(*gradient_left, rect.fLeft, rect.fTop);
   canvas->TileImageInt(*gradient_center,
       SkScalarFloor(rect.fLeft) + gradient_left->width(),
       SkScalarFloor(rect.fTop),
       SkScalarFloor(rect.width()) - gradient_left->width() -
                     gradient_right->width(),
       SkScalarFloor(rect.height()));
-  canvas->AsCanvasSkia()->drawBitmap(*gradient_right,
+  canvas->GetSkCanvas()->drawBitmap(*gradient_right,
       rect.fRight - SkIntToScalar(gradient_right->width()), rect.fTop);
 
   // Finally, draw the text centered within the badge. We set a clip in case the
   // text was too large.
   rect.fLeft += kPadding;
   rect.fRight -= kPadding;
-  canvas->AsCanvasSkia()->clipRect(rect);
-  canvas->AsCanvasSkia()->drawText(text.c_str(), text.size(),
-                                   rect.fLeft + (rect.width() - text_width) / 2,
-                                   rect.fTop + kTextSize + kTopTextPadding,
-                                   *text_paint);
+  canvas->GetSkCanvas()->clipRect(rect);
+  canvas->GetSkCanvas()->drawText(text.c_str(), text.size(),
+                                  rect.fLeft + (rect.width() - text_width) / 2,
+                                  rect.fTop + kTextSize + kTopTextPadding,
+                                  *text_paint);
   canvas->Restore();
 }
