@@ -10,11 +10,15 @@
 #include <map>
 
 #include "chrome/common/extensions/extension.h"
+#include "chrome/common/extensions/extension_message_bundle.h"
 
 class Extension;
 class ExtensionMessageBundle;
 class FilePath;
 class GURL;
+namespace base {
+class DictionaryValue;
+}
 
 // Utilties for manipulating the on-disk storage of extensions.
 namespace extension_file_util {
@@ -39,12 +43,17 @@ void UninstallExtension(const FilePath& extensions_dir,
 // on failure, with a description of the error in |error|.
 scoped_refptr<Extension> LoadExtension(const FilePath& extension_root,
                                        Extension::Location location,
-                                       bool require_key,
+                                       int flags,
                                        std::string* error);
 
+// Loads an extension manifest from the specified directory. Returns NULL
+// on failure, with a description of the error in |error|.
+base::DictionaryValue* LoadManifest(const FilePath& extension_root,
+                                    std::string* error);
+
 // Returns true if the given extension object is valid and consistent.
 // Otherwise, a description of the error is returned in |error|.
-bool ValidateExtension(Extension* extension, std::string* error);
+bool ValidateExtension(const Extension* extension, std::string* error);
 
 // Cleans up the extension install directory. It can end up with garbage in it
 // if extensions can't initially be removed when they are uninstalled (eg if a
@@ -66,6 +75,14 @@ ExtensionMessageBundle* LoadExtensionMessageBundle(
     const std::string& default_locale,
     std::string* error);
 
+// Loads the extension message bundle substitution map. Contains at least
+// extension_id item.
+ExtensionMessageBundle::SubstitutionMap*
+    LoadExtensionMessageBundleSubstitutionMap(
+    const FilePath& extension_path,
+    const std::string& extension_id,
+    const std::string& default_locale);
+
 // We need to reserve the namespace of entries that start with "_" for future
 // use by Chrome.
 // If any files or directories are found using "_" prefix and are not on
@@ -82,6 +99,11 @@ FilePath ExtensionURLToRelativeFilePath(const GURL& url);
 // Return an empty file path on failure.
 FilePath GetUserDataTempDir();
 
+// Helper function to delete files. This is used to avoid ugly casts which
+// would be necessary with PostMessage since file_util::Delete is overloaded.
+// TODO(skerner): Make a version of Delete that is not overloaded in file_util.
+void DeleteFile(const FilePath& path, bool recursive);
+
 }  // namespace extension_file_util
 
 #endif  // CHROME_COMMON_EXTENSIONS_EXTENSION_FILE_UTIL_H_
