@@ -1,23 +1,24 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/common/extensions/extension_l10n_util.h"
 
+#include <algorithm>
 #include <set>
 #include <string>
 #include <vector>
 
 #include "base/file_util.h"
-#include "base/linked_ptr.h"
+#include "base/json/json_value_serializer.h"
 #include "base/logging.h"
-#include "base/string_util.h"
+#include "base/memory/linked_ptr.h"
+#include "base/stringprintf.h"
 #include "base/values.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/extensions/extension_file_util.h"
 #include "chrome/common/extensions/extension_message_bundle.h"
-#include "chrome/common/json_value_serializer.h"
 #include "chrome/common/url_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "unicode/uloc.h"
@@ -25,15 +26,15 @@
 namespace errors = extension_manifest_errors;
 namespace keys = extension_manifest_keys;
 
-static std::string* GetProcessLocale() {
-  static std::string locale;
-  return &locale;
+static std::string& GetProcessLocale() {
+  CR_DEFINE_STATIC_LOCAL(std::string, locale, ());
+  return locale;
 }
 
 namespace extension_l10n_util {
 
 void SetProcessLocale(const std::string& locale) {
-  *(GetProcessLocale()) = locale;
+  GetProcessLocale() = locale;
 }
 
 std::string GetDefaultLocaleFromManifest(const DictionaryValue& manifest,
@@ -111,6 +112,20 @@ bool LocalizeManifest(const ExtensionMessageBundle& messages,
   if (!LocalizeManifestValue(keys::kOmniboxKeyword, messages, manifest, error))
     return false;
 
+  ListValue* file_handlers = NULL;
+  if (manifest->GetList(keys::kFileBrowserHandlers, &file_handlers)) {
+    key.assign(keys::kFileBrowserHandlers);
+    for (size_t i = 0; i < file_handlers->GetSize(); i++) {
+      DictionaryValue* handler = NULL;
+      if (!file_handlers->GetDictionary(i, &handler)) {
+        *error = errors::kInvalidFileBrowserHandler;
+        return false;
+      }
+      if (!LocalizeManifestValue(keys::kPageActionDefaultTitle, messages,
+                                 handler, error))
+        return false;
+    }
+  }
   // Add current locale key to the manifest, so we can overwrite prefs
   // with new manifest when chrome locale changes.
   manifest->SetString(keys::kCurrentLocale, CurrentLocaleOrDefault());
@@ -150,8 +165,8 @@ bool AddLocale(const std::set<std::string>& chrome_locales,
   if (chrome_locales.find(locale_name) == chrome_locales.end()) {
     // Warn if there is an extension locale that's not in the Chrome list,
     // but don't fail.
-    LOG(WARNING) << base::StringPrintf("Supplied locale %s is not supported.",
-                                       locale_name.c_str());
+    DLOG(WARNING) << base::StringPrintf("Supplied locale %s is not supported.",
+                                        locale_name.c_str());
     return true;
   }
   // Check if messages file is actually present (but don't check content).
@@ -168,7 +183,7 @@ bool AddLocale(const std::set<std::string>& chrome_locales,
 }
 
 std::string CurrentLocaleOrDefault() {
-  std::string current_locale = l10n_util::NormalizeLocale(*GetProcessLocale());
+  std::string current_locale = l10n_util::NormalizeLocale(GetProcessLocale());
   if (current_locale.empty())
     current_locale = "en";
 
@@ -190,7 +205,7 @@ void GetAllLocales(std::set<std::string>* all_locales) {
 bool GetValidLocales(const FilePath& locale_path,
                      std::set<std::string>* valid_locales,
                      std::string* error) {
-  static std::set<std::string> chrome_locales;
+  std::set<std::string> chrome_locales;
   GetAllLocales(&chrome_locales);
 
   // Enumerate all supplied locales in the extension.
@@ -199,8 +214,11 @@ bool GetValidLocales(const FilePath& locale_path,
                                     file_util::FileEnumerator::DIRECTORIES);
   FilePath locale_folder;
   while (!(locale_folder = locales.Next()).empty()) {
-    std::string locale_name =
-        WideToASCII(locale_folder.BaseName().ToWStringHack());
+    std::string locale_name = locale_folder.BaseName().MaybeAsASCII();
+    if (locale_name.empty()) {
+      NOTREACHED();
+      continue;  // Not ASCII.
+    }
     if (!AddLocale(chrome_locales,
                    locale_folder,
                    locale_name,
@@ -277,13 +295,18 @@ bool ShouldSkipValidation(const FilePath& locales_path,
   // skipping any strings with '.'. This happens sometimes, for example with
   // '.svn' directories.
   FilePath relative_path;
-  if (!locales_path.AppendRelativePath(locale_path, &relative_path))
+  if (!locales_path.AppendRelativePath(locale_path, &relative_path)) {
     NOTREACHED();
-  std::wstring subdir(relative_path.ToWStringHack());
-  if (std::find(subdir.begin(), subdir.end(), L'.') != subdir.end())
+    return true;
+  }
+  std::string subdir = relative_path.MaybeAsASCII();
+  if (subdir.empty())
+    return true;  // Non-ASCII.
+
+  if (std::find(subdir.begin(), subdir.end(), '.') != subdir.end())
     return true;
 
-  if (all_locales.find(WideToASCII(subdir)) == all_locales.end())
+  if (all_locales.find(subdir) == all_locales.end())
     return true;
 
   return false;
