@@ -1,12 +1,12 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/file_path.h"
 #include "base/file_util.h"
-#include "base/linked_ptr.h"
+#include "base/memory/linked_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
-#include "base/scoped_ptr.h"
 #include "base/scoped_temp_dir.h"
 #include "base/values.h"
 #include "chrome/common/chrome_paths.h"
@@ -22,6 +22,49 @@ namespace keys = extension_manifest_keys;
 
 namespace {
 
+// crbug.com/108429: Crashing on Windows Vista bots.
+#if defined(OS_WIN)
+#define GetValidLocalesEmptyLocaleFolder \
+  DISABLED_GetValidLocalesEmptyLocaleFolder
+#define GetValidLocalesWithValidLocaleNoMessagesFile \
+  DISABLED_GetValidLocalesWithValidLocaleNoMessagesFile
+#define GetValidLocalesWithUnsupportedLocale \
+  DISABLED_GetValidLocalesWithUnsupportedLocale
+#define GetValidLocalesWithValidLocalesAndMessagesFile \
+  DISABLED_GetValidLocalesWithValidLocalesAndMessagesFile
+#define LoadMessageCatalogsValidFallback \
+  DISABLED_LoadMessageCatalogsValidFallback
+#define LoadMessageCatalogsMissingFiles DISABLED_LoadMessageCatalogsMissingFiles
+#define LoadMessageCatalogsBadJSONFormat \
+  DISABLED_LoadMessageCatalogsBadJSONFormat
+#define LoadMessageCatalogsDuplicateKeys \
+  DISABLED_LoadMessageCatalogsDuplicateKeys
+#define LocalizeEmptyManifest DISABLED_LocalizeEmptyManifest
+#define LocalizeManifestWithoutNameMsgAndEmptyDescription \
+  DISABLED_LocalizeManifestWithoutNameMsgAndEmptyDescription
+#define LocalizeManifestWithNameMsgAndEmptyDescription \
+  DISABLED_LocalizeManifestWithNameMsgAndEmptyDescription
+#define LocalizeManifestWithBadNameMsg DISABLED_LocalizeManifestWithBadNameMsg
+#define LocalizeManifestWithNameDescriptionDefaultTitleMsgs \
+  DISABLED_LocalizeManifestWithNameDescriptionDefaultTitleMsgs
+#define LocalizeManifestWithNameDescriptionOmniboxMsgs \
+  DISABLED_LocalizeManifestWithNameDescriptionOmniboxMsgs
+#define LocalizeManifestWithNameDescriptionFileHandlerTitle \
+  DISABLED_LocalizeManifestWithNameDescriptionFileHandlerTitle
+#define ShouldRelocalizeManifestWithNullManifest \
+  DISABLED_ShouldRelocalizeManifestWithNullManifest
+#define ShouldRelocalizeManifestEmptyManifest \
+  DISABLED_ShouldRelocalizeManifestEmptyManifest
+#define ShouldRelocalizeManifestWithDefaultLocale \
+  DISABLED_ShouldRelocalizeManifestWithDefaultLocale
+#define ShouldRelocalizeManifestWithCurrentLocale \
+  DISABLED_ShouldRelocalizeManifestWithCurrentLocale
+#define ShouldRelocalizeManifestSameCurrentLocale \
+  DISABLED_ShouldRelocalizeManifestSameCurrentLocale
+#define ShouldRelocalizeManifestDifferentCurrentLocale \
+  DISABLED_ShouldRelocalizeManifestDifferentCurrentLocale
+#endif
+
 TEST(ExtensionL10nUtil, GetValidLocalesEmptyLocaleFolder) {
   ScopedTempDir temp;
   ASSERT_TRUE(temp.CreateUniqueTempDir());
@@ -234,6 +277,10 @@ ExtensionMessageBundle* CreateManifestBundle() {
   omnibox_keyword_tree->SetString("message", "omnibox keyword");
   catalog->Set("omnibox_keyword", omnibox_keyword_tree);
 
+  DictionaryValue* file_handler_title_tree = new DictionaryValue();
+  file_handler_title_tree->SetString("message", "file handler title");
+  catalog->Set("file_handler_title", file_handler_title_tree);
+
   std::vector<linked_ptr<DictionaryValue> > catalogs;
   catalogs.push_back(catalog);
 
@@ -253,7 +300,7 @@ TEST(ExtensionL10nUtil, LocalizeEmptyManifest) {
 
   EXPECT_FALSE(
       extension_l10n_util::LocalizeManifest(*messages, &manifest, &error));
-  EXPECT_EQ(errors::kInvalidName, error);
+  EXPECT_EQ(std::string(errors::kInvalidName), error);
 }
 
 TEST(ExtensionL10nUtil, LocalizeManifestWithoutNameMsgAndEmptyDescription) {
@@ -365,6 +412,36 @@ TEST(ExtensionL10nUtil, LocalizeManifestWithNameDescriptionOmniboxMsgs) {
   EXPECT_TRUE(error.empty());
 }
 
+TEST(ExtensionL10nUtil, LocalizeManifestWithNameDescriptionFileHandlerTitle) {
+  DictionaryValue manifest;
+  manifest.SetString(keys::kName, "__MSG_name__");
+  manifest.SetString(keys::kDescription, "__MSG_description__");
+  ListValue* handlers = new ListValue();
+  manifest.Set(keys::kFileBrowserHandlers, handlers);
+  DictionaryValue* handler = new DictionaryValue();
+  handlers->Append(handler);
+  handler->SetString(keys::kPageActionDefaultTitle,
+                     "__MSG_file_handler_title__");
+
+  std::string error;
+  scoped_ptr<ExtensionMessageBundle> messages(CreateManifestBundle());
+
+  EXPECT_TRUE(
+      extension_l10n_util::LocalizeManifest(*messages, &manifest, &error));
+
+  std::string result;
+  ASSERT_TRUE(manifest.GetString(keys::kName, &result));
+  EXPECT_EQ("name", result);
+
+  ASSERT_TRUE(manifest.GetString(keys::kDescription, &result));
+  EXPECT_EQ("description", result);
+
+  ASSERT_TRUE(handler->GetString(keys::kPageActionDefaultTitle, &result));
+  EXPECT_EQ("file handler title", result);
+
+  EXPECT_TRUE(error.empty());
+}
+
 // Try with NULL manifest.
 TEST(ExtensionL10nUtil, ShouldRelocalizeManifestWithNullManifest) {
   ExtensionInfo info(NULL, "", FilePath(), Extension::LOAD);
