@@ -30,20 +30,22 @@ class ExtensionLocalizationPeer
       const GURL& request_url);
 
   // ResourceLoaderBridge::Peer methods.
-  virtual void OnUploadProgress(uint64 position, uint64 size);
+  virtual void OnUploadProgress(uint64 position, uint64 size) OVERRIDE;
   virtual bool OnReceivedRedirect(
       const GURL& new_url,
       const webkit_glue::ResourceResponseInfo& info,
       bool* has_new_first_party_for_cookies,
-      GURL* new_first_party_for_cookies);
+      GURL* new_first_party_for_cookies) OVERRIDE;
   virtual void OnReceivedResponse(
-      const webkit_glue::ResourceResponseInfo& info,
-      bool content_filtered);
-  virtual void OnDownloadedData(int len) {}
-  virtual void OnReceivedData(const char* data, int len);
-  virtual void OnCompletedRequest(const net::URLRequestStatus& status,
-                                  const std::string& security_info,
-                                  const base::Time& completion_time);
+      const webkit_glue::ResourceResponseInfo& info) OVERRIDE;
+  virtual void OnDownloadedData(int len) OVERRIDE {}
+  virtual void OnReceivedData(const char* data,
+                              int data_length,
+                              int encoded_data_length) OVERRIDE;
+  virtual void OnCompletedRequest(
+      const net::URLRequestStatus& status,
+      const std::string& security_info,
+      const base::TimeTicks& completion_time) OVERRIDE;
 
  private:
   friend class ExtensionLocalizationPeerTest;
@@ -64,7 +66,7 @@ class ExtensionLocalizationPeer
   // We just pass though the response info. This holds the copy of the original.
   webkit_glue::ResourceResponseInfo response_info_;
 
-  // Sends ViewHostMsg_GetExtensionMessageBundle message to the browser to fetch
+  // Sends ExtensionHostMsg_GetMessageBundle message to the browser to fetch
   // message catalog.
   IPC::Message::Sender* message_sender_;
 
