@@ -5,7 +5,7 @@
 #include <map>
 #include <string>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "chrome/common/extensions/extension_message_bundle.h"
 #include "chrome/common/extensions/extension_localization_peer.h"
 #include "ipc/ipc_message.h"
@@ -63,15 +63,16 @@ class MockResourceLoaderBridgePeer
       const webkit_glue::ResourceResponseInfo& info,
       bool* has_new_first_party_for_cookies,
       GURL* new_first_party_for_cookies));
-  MOCK_METHOD2(OnReceivedResponse, void(
-      const webkit_glue::ResourceResponseInfo& info,
-      bool content_filtered));
+  MOCK_METHOD1(OnReceivedResponse, void(
+      const webkit_glue::ResourceResponseInfo& info));
   MOCK_METHOD1(OnDownloadedData, void(int len));
-  MOCK_METHOD2(OnReceivedData, void(const char* data, int len));
+  MOCK_METHOD3(OnReceivedData, void(const char* data,
+                                    int data_length,
+                                    int encoded_data_length));
   MOCK_METHOD3(OnCompletedRequest, void(
       const net::URLRequestStatus& status,
       const std::string& security_info,
-      const base::Time& completion_time));
+      const base::TimeTicks& completion_time));
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MockResourceLoaderBridgePeer);
@@ -125,11 +126,11 @@ TEST_F(ExtensionLocalizationPeerTest, OnReceivedData) {
   EXPECT_TRUE(GetData(filter_peer_.get()).empty());
 
   const std::string data_chunk("12345");
-  filter_peer_->OnReceivedData(data_chunk.c_str(), data_chunk.length());
+  filter_peer_->OnReceivedData(data_chunk.c_str(), data_chunk.length(), -1);
 
   EXPECT_EQ(data_chunk, GetData(filter_peer_.get()));
 
-  filter_peer_->OnReceivedData(data_chunk.c_str(), data_chunk.length());
+  filter_peer_->OnReceivedData(data_chunk.c_str(), data_chunk.length(), -1);
   EXPECT_EQ(data_chunk + data_chunk, GetData(filter_peer_.get()));
 }
 
@@ -139,29 +140,30 @@ TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestBadURLRequestStatus) {
   // It will self-delete once it exits OnCompletedRequest.
   ExtensionLocalizationPeer* filter_peer = filter_peer_.release();
 
-  EXPECT_CALL(*original_peer_, OnReceivedResponse(_, true));
+  EXPECT_CALL(*original_peer_, OnReceivedResponse(_));
   EXPECT_CALL(*original_peer_, OnCompletedRequest(
-    IsURLRequestEqual(net::URLRequestStatus::CANCELED), "", base::Time()));
+    IsURLRequestEqual(net::URLRequestStatus::CANCELED), "", base::TimeTicks()));
 
   net::URLRequestStatus status;
   status.set_status(net::URLRequestStatus::FAILED);
-  filter_peer->OnCompletedRequest(status, "", base::Time());
+  filter_peer->OnCompletedRequest(status, "", base::TimeTicks());
 }
 
 TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestEmptyData) {
   // It will self-delete once it exits OnCompletedRequest.
   ExtensionLocalizationPeer* filter_peer = filter_peer_.release();
 
-  EXPECT_CALL(*original_peer_, OnReceivedData(_, _)).Times(0);
+  EXPECT_CALL(*original_peer_, OnReceivedData(_, _, _)).Times(0);
   EXPECT_CALL(*sender_, Send(_)).Times(0);
 
-  EXPECT_CALL(*original_peer_, OnReceivedResponse(_, true));
+  EXPECT_CALL(*original_peer_, OnReceivedResponse(_));
   EXPECT_CALL(*original_peer_, OnCompletedRequest(
-      IsURLRequestEqual(net::URLRequestStatus::SUCCESS), "", base::Time()));
+      IsURLRequestEqual(net::URLRequestStatus::SUCCESS), "",
+      base::TimeTicks()));
 
   net::URLRequestStatus status;
   status.set_status(net::URLRequestStatus::SUCCESS);
-  filter_peer->OnCompletedRequest(status, "", base::Time());
+  filter_peer->OnCompletedRequest(status, "", base::TimeTicks());
 }
 
 TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestNoCatalogs) {
@@ -174,23 +176,23 @@ TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestNoCatalogs) {
 
   std::string data = GetData(filter_peer);
   EXPECT_CALL(*original_peer_,
-              OnReceivedData(StrEq(data.data()), data.length())).Times(2);
+              OnReceivedData(StrEq(data.data()), data.length(), -1)).Times(2);
 
-  EXPECT_CALL(*original_peer_, OnReceivedResponse(_, true)).Times(2);
+  EXPECT_CALL(*original_peer_, OnReceivedResponse(_)).Times(2);
   EXPECT_CALL(*original_peer_, OnCompletedRequest(
       IsURLRequestEqual(
-          net::URLRequestStatus::SUCCESS), "", base::Time())).Times(2);
+          net::URLRequestStatus::SUCCESS), "", base::TimeTicks())).Times(2);
 
   net::URLRequestStatus status;
   status.set_status(net::URLRequestStatus::SUCCESS);
-  filter_peer->OnCompletedRequest(status, "", base::Time());
+  filter_peer->OnCompletedRequest(status, "", base::TimeTicks());
 
   // Test if Send gets called again (it shouldn't be) when first call returned
   // an empty dictionary.
   filter_peer =
       CreateExtensionLocalizationPeer("text/css", GURL(kExtensionUrl_1));
   SetData(filter_peer, "some text");
-  filter_peer->OnCompletedRequest(status, "", base::Time());
+  filter_peer->OnCompletedRequest(status, "", base::TimeTicks());
 }
 
 TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestWithCatalogs) {
@@ -212,15 +214,16 @@ TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestWithCatalogs) {
   // __MSG_text__ gets replaced with "new text".
   std::string data("some new text");
   EXPECT_CALL(*original_peer_,
-              OnReceivedData(StrEq(data.data()), data.length()));
+              OnReceivedData(StrEq(data.data()), data.length(), -1));
 
-  EXPECT_CALL(*original_peer_, OnReceivedResponse(_, true));
+  EXPECT_CALL(*original_peer_, OnReceivedResponse(_));
   EXPECT_CALL(*original_peer_, OnCompletedRequest(
-      IsURLRequestEqual(net::URLRequestStatus::SUCCESS), "", base::Time()));
+      IsURLRequestEqual(net::URLRequestStatus::SUCCESS), "",
+      base::TimeTicks()));
 
   net::URLRequestStatus status;
   status.set_status(net::URLRequestStatus::SUCCESS);
-  filter_peer->OnCompletedRequest(status, "", base::Time());
+  filter_peer->OnCompletedRequest(status, "", base::TimeTicks());
 }
 
 TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestReplaceMessagesFails) {
@@ -242,13 +245,14 @@ TEST_F(ExtensionLocalizationPeerTest, OnCompletedRequestReplaceMessagesFails) {
 
   // __MSG_missing_message__ is missing, so message stays the same.
   EXPECT_CALL(*original_peer_,
-              OnReceivedData(StrEq(message.data()), message.length()));
+              OnReceivedData(StrEq(message.data()), message.length(), -1));
 
-  EXPECT_CALL(*original_peer_, OnReceivedResponse(_, true));
+  EXPECT_CALL(*original_peer_, OnReceivedResponse(_));
   EXPECT_CALL(*original_peer_, OnCompletedRequest(
-      IsURLRequestEqual(net::URLRequestStatus::SUCCESS), "", base::Time()));
+      IsURLRequestEqual(net::URLRequestStatus::SUCCESS), "",
+      base::TimeTicks()));
 
   net::URLRequestStatus status;
   status.set_status(net::URLRequestStatus::SUCCESS);
-  filter_peer->OnCompletedRequest(status, "", base::Time());
+  filter_peer->OnCompletedRequest(status, "", base::TimeTicks());
 }
