@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,10 +10,10 @@
 #include "base/hash_tables.h"
 #include "base/i18n/rtl.h"
 #include "base/lazy_instance.h"
-#include "base/linked_ptr.h"
-#include "base/scoped_ptr.h"
-#include "base/stl_util-inl.h"
-#include "base/string_util.h"
+#include "base/memory/linked_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/stl_util.h"
+#include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/common/extensions/extension_constants.h"
@@ -325,8 +325,8 @@ struct ExtensionToMessagesMap {
   ExtensionToL10nMessagesMap messages_map;
 };
 
-static base::LazyInstance<ExtensionToMessagesMap> g_extension_to_messages_map(
-    base::LINKER_INITIALIZED);
+static base::LazyInstance<ExtensionToMessagesMap> g_extension_to_messages_map =
+    LAZY_INSTANCE_INITIALIZER;
 
 ExtensionToMessagesMap::ExtensionToMessagesMap() {}
 
@@ -336,7 +336,7 @@ ExtensionToL10nMessagesMap* GetExtensionToL10nMessagesMap() {
   return &g_extension_to_messages_map.Get().messages_map;
 }
 
-L10nMessagesMap* GetL10nMessagesMap(const std::string extension_id) {
+L10nMessagesMap* GetL10nMessagesMap(const std::string& extension_id) {
   ExtensionToL10nMessagesMap::iterator it =
       g_extension_to_messages_map.Get().messages_map.find(extension_id);
   if (it != g_extension_to_messages_map.Get().messages_map.end())
