@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,16 +10,18 @@
 #include <string>
 #include <vector>
 
-#include "base/linked_ptr.h"
+#include "base/memory/linked_ptr.h"
 
+namespace base {
 class DictionaryValue;
+}
 
 // Contains localized extension messages for one locale. Any messages that the
 // locale does not provide are pulled from the default locale.
 class ExtensionMessageBundle {
  public:
   typedef std::map<std::string, std::string> SubstitutionMap;
-  typedef std::vector<linked_ptr<DictionaryValue> > CatalogVector;
+  typedef std::vector<linked_ptr<base::DictionaryValue> > CatalogVector;
 
   // JSON keys of interest for messages file.
   static const char* kContentKey;
@@ -123,12 +125,12 @@ class ExtensionMessageBundle {
   // They replace all $PLACEHOLDERS$ with their value, and return just key/value
   // of the message.
   bool GetMessageValue(const std::string& key,
-                       const DictionaryValue& catalog,
+                       const base::DictionaryValue& catalog,
                        std::string* value,
                        std::string* error) const;
 
   // Get all placeholders for a given message from JSON subtree.
-  bool GetPlaceholders(const DictionaryValue& name_tree,
+  bool GetPlaceholders(const base::DictionaryValue& name_tree,
                        const std::string& name_key,
                        SubstitutionMap* placeholders,
                        std::string* error) const;
@@ -159,6 +161,6 @@ typedef std::map<std::string, L10nMessagesMap > ExtensionToL10nMessagesMap;
 ExtensionToL10nMessagesMap* GetExtensionToL10nMessagesMap();
 
 // Returns message map that matches given extension_id, or NULL.
-L10nMessagesMap* GetL10nMessagesMap(const std::string extension_id);
+L10nMessagesMap* GetL10nMessagesMap(const std::string& extension_id);
 
 #endif  // CHROME_COMMON_EXTENSIONS_EXTENSION_MESSAGE_BUNDLE_H_
