@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,6 +11,7 @@
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_l10n_util.h"
 #include "chrome/common/extensions/extension_resource.h"
+#include "chrome/common/extensions/extension_test_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/base/l10n/l10n_util.h"
 
@@ -33,8 +34,7 @@ TEST(ExtensionResourceTest, CreateWithMissingResourceOnDisk) {
   ASSERT_TRUE(PathService::Get(chrome::DIR_TEST_DATA, &root_path));
   FilePath relative_path;
   relative_path = relative_path.AppendASCII("cira.js");
-  std::string extension_id;
-  Extension::GenerateId("test", &extension_id);
+  std::string extension_id = extension_test_util::MakeId("test");
   ExtensionResource resource(extension_id, root_path, relative_path);
 
   // The path doesn't exist on disk, we will be returned an empty path.
@@ -43,6 +43,10 @@ TEST(ExtensionResourceTest, CreateWithMissingResourceOnDisk) {
   EXPECT_TRUE(resource.GetFilePath().empty());
 }
 
+// crbug.com/108721. Disabled on Windows due to crashing on Vista.
+#if defined(OS_WIN)
+#define CreateWithAllResourcesOnDisk DISABLED_CreateWithAllResourcesOnDisk
+#endif
 TEST(ExtensionResourceTest, CreateWithAllResourcesOnDisk) {
   ScopedTempDir temp;
   ASSERT_TRUE(temp.CreateUniqueTempDir());
@@ -69,8 +73,7 @@ TEST(ExtensionResourceTest, CreateWithAllResourcesOnDisk) {
   }
 
   FilePath path;
-  std::string extension_id;
-  Extension::GenerateId("test", &extension_id);
+  std::string extension_id = extension_test_util::MakeId("test");
   ExtensionResource resource(extension_id, temp.path(),
                              FilePath().AppendASCII(filename));
   FilePath resolved_path = resource.GetFilePath();
