@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,22 +7,22 @@
 #include <set>
 
 #include "base/file_util.h"
-#include "base/scoped_handle.h"
+#include "base/json/json_value_serializer.h"
+#include "base/memory/scoped_handle.h"
 #include "base/scoped_temp_dir.h"
 #include "base/string_util.h"
 #include "base/threading/thread.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
-#include "net/base/file_stream.h"
-#include "chrome/common/common_param_traits.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/extensions/extension_file_util.h"
 #include "chrome/common/extensions/extension_l10n_util.h"
-#include "chrome/common/json_value_serializer.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/common/zip.h"
+#include "content/public/common/common_param_traits.h"
 #include "ipc/ipc_message_utils.h"
+#include "net/base/file_stream.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "webkit/glue/image_decoder.h"
 
@@ -84,8 +84,11 @@ bool PathContainsParentDirectory(const FilePath& path) {
 
 }  // namespace
 
-ExtensionUnpacker::ExtensionUnpacker(const FilePath& extension_path)
-    : extension_path_(extension_path) {
+ExtensionUnpacker::ExtensionUnpacker(const FilePath& extension_path,
+                                     Extension::Location location,
+                                     int creation_flags)
+    : extension_path_(extension_path), location_(location),
+      creation_flags_(creation_flags) {
 }
 
 ExtensionUnpacker::~ExtensionUnpacker() {
@@ -144,7 +147,7 @@ bool ExtensionUnpacker::ReadAllMessageCatalogs(
 }
 
 bool ExtensionUnpacker::Run() {
-  VLOG(1) << "Installing extension " << extension_path_.value();
+  DVLOG(1) << "Installing extension " << extension_path_.value();
 
   // <profile>/Extensions/INSTALL_TEMP/<version>
   temp_install_dir_ =
@@ -161,7 +164,7 @@ bool ExtensionUnpacker::Run() {
     return false;
   }
 
-  if (!Unzip(extension_path_, temp_install_dir_)) {
+  if (!zip::Unzip(extension_path_, temp_install_dir_)) {
     SetError(kCouldNotUnzipExtension);
     return false;
   }
@@ -177,7 +180,11 @@ bool ExtensionUnpacker::Run() {
   // EXTENSION.
   std::string error;
   scoped_refptr<Extension> extension(Extension::Create(
-      temp_install_dir_, Extension::INVALID, *parsed_manifest_, false, &error));
+      temp_install_dir_,
+      location_,
+      *parsed_manifest_,
+      creation_flags_,
+      &error));
   if (!extension.get()) {
     SetError(error);
     return false;
@@ -286,13 +293,14 @@ bool ExtensionUnpacker::ReadMessageCatalog(const FilePath& message_path) {
   scoped_ptr<DictionaryValue> root(
       static_cast<DictionaryValue*>(serializer.Deserialize(NULL, &error)));
   if (!root.get()) {
-    std::string messages_file = WideToASCII(message_path.ToWStringHack());
+    string16 messages_file = message_path.LossyDisplayName();
     if (error.empty()) {
       // If file is missing, Deserialize will fail with empty error.
       SetError(base::StringPrintf("%s %s", errors::kLocalesMessagesFileMissing,
-                                  messages_file.c_str()));
+                                  UTF16ToUTF8(messages_file).c_str()));
     } else {
-      SetError(base::StringPrintf("%s: %s", messages_file.c_str(),
+      SetError(base::StringPrintf("%s: %s",
+                                  UTF16ToUTF8(messages_file).c_str(),
                                   error.c_str()));
     }
     return false;
@@ -300,15 +308,21 @@ bool ExtensionUnpacker::ReadMessageCatalog(const FilePath& message_path) {
 
   FilePath relative_path;
   // message_path was created from temp_install_dir. This should never fail.
-  if (!temp_install_dir_.AppendRelativePath(message_path, &relative_path))
+  if (!temp_install_dir_.AppendRelativePath(message_path, &relative_path)) {
     NOTREACHED();
+    return false;
+  }
 
-  parsed_catalogs_->Set(WideToUTF8(relative_path.DirName().ToWStringHack()),
-                        root.release());
+  std::string dir_name = relative_path.DirName().MaybeAsASCII();
+  if (dir_name.empty()) {
+    NOTREACHED();
+    return false;
+  }
+  parsed_catalogs_->Set(dir_name, root.release());
 
   return true;
 }
 
 void ExtensionUnpacker::SetError(const std::string &error) {
-  error_message_ = error;
+  error_message_ = UTF8ToUTF16(error);
 }
