@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 #ifndef CHROME_COMMON_EXTENSIONS_URL_PATTERN_H_
@@ -14,9 +14,11 @@ class GURL;
 // A pattern that can be used to match URLs. A URLPattern is a very restricted
 // subset of URL syntax:
 //
-// <url-pattern> := <scheme>://<host><path> | '<all_urls>'
-// <scheme> := '*' | 'http' | 'https' | 'file' | 'ftp' | 'chrome'
+// <url-pattern> := <scheme>://<host><port><path> | '<all_urls>'
+// <scheme> := '*' | 'http' | 'https' | 'file' | 'ftp' | 'chrome' |
+//             'chrome-extension' | 'filesystem'
 // <host> := '*' | '*.' <anychar except '/' and '*'>+
+// <port> := [':' ('*' | <port number between 0 and 65535>)]
 // <path> := '/' <any chars>
 //
 // * Host is not used when the scheme is 'file'.
@@ -39,107 +41,55 @@ class GURL;
 // - http:/bar -- scheme separator not found
 // - foo://* -- invalid scheme
 // - chrome:// -- we don't support chrome internal URLs
-//
-// Design rationale:
-// * We need to be able to tell users what 'sites' a given URLPattern will
-//   affect. For example "This extension will interact with the site
-//   'www.google.com'.
-// * We'd like to be able to convert as many existing Greasemonkey @include
-//   patterns to URLPatterns as possible. Greasemonkey @include patterns are
-//   simple globs, so this won't be perfect.
-// * Although we would like to support any scheme, it isn't clear what to tell
-//   users about URLPatterns that affect data or javascript URLs, so those are
-//   left out for now.
-//
-// From a 2008-ish crawl of userscripts.org, the following patterns were found
-// in @include lines:
-// - total lines                    : 24471
-// - @include *                     :   919
-// - @include http://[^\*]+?/       : 11128 (no star in host)
-// - @include http://\*\.[^\*]+?/   :  2325 (host prefixed by *.)
-// - @include http://\*[^\.][^\*]+?/:  1524 (host prefixed by *, no dot -- many
-//                                           appear to only need subdomain
-//                                           matching, not real prefix matching)
-// - @include http://[^\*/]+\*/     :   320 (host suffixed by *)
-// - @include contains .tld         :   297 (host suffixed by .tld -- a special
-//                                           Greasemonkey domain component that
-//                                           tries to match all valid registry-
-//                                           controlled suffixes)
-// - @include http://\*/            :   228 (host is * exactly, but there is
-//                                           more to the pattern)
-//
-// So, we can support at least half of current @include lines without supporting
-// subdomain matching. We can pick up at least another 10% by supporting
-// subdomain matching. It is probably possible to coerce more of the existing
-// patterns to URLPattern, but the resulting pattern will be more restrictive
-// than the original glob, which is probably better than nothing.
 class URLPattern {
  public:
   // A collection of scheme bitmasks for use with valid_schemes.
   enum SchemeMasks {
-    SCHEME_NONE     = 0,
-    SCHEME_HTTP     = 1 << 0,
-    SCHEME_HTTPS    = 1 << 1,
-    SCHEME_FILE     = 1 << 2,
-    SCHEME_FTP      = 1 << 3,
-    SCHEME_CHROMEUI = 1 << 4,
+    SCHEME_NONE       = 0,
+    SCHEME_HTTP       = 1 << 0,
+    SCHEME_HTTPS      = 1 << 1,
+    SCHEME_FILE       = 1 << 2,
+    SCHEME_FTP        = 1 << 3,
+    SCHEME_CHROMEUI   = 1 << 4,
+    SCHEME_EXTENSION  = 1 << 5,
+    SCHEME_FILESYSTEM = 1 << 6,
+
+    // IMPORTANT!
     // SCHEME_ALL will match every scheme, including chrome://, chrome-
     // extension://, about:, etc. Because this has lots of security
-    // implications, third-party extensions should never be able to get access
-    // to URL patterns initialized this way. It should only be used for internal
-    // Chrome code.
+    // implications, third-party extensions should usually not be able to get
+    // access to URL patterns initialized this way. If there is a reason
+    // for violating this general rule, document why this it safe.
     SCHEME_ALL      = -1,
   };
 
   // Error codes returned from Parse().
   enum ParseResult {
-    PARSE_SUCCESS,
+    PARSE_SUCCESS = 0,
     PARSE_ERROR_MISSING_SCHEME_SEPARATOR,
     PARSE_ERROR_INVALID_SCHEME,
     PARSE_ERROR_WRONG_SCHEME_SEPARATOR,
     PARSE_ERROR_EMPTY_HOST,
     PARSE_ERROR_INVALID_HOST_WILDCARD,
     PARSE_ERROR_EMPTY_PATH,
+    PARSE_ERROR_INVALID_PORT,
+    NUM_PARSE_RESULTS
   };
 
   // The <all_urls> string pattern.
   static const char kAllUrlsPattern[];
 
-  // Construct an URLPattern with the given set of allowable schemes. See
-  // valid_schemes_ for more info.
   explicit URLPattern(int valid_schemes);
 
-  // Convenience to construct a URLPattern from a string. The string is expected
-  // to be a valid pattern. If the string is not known ahead of time, use
-  // Parse() instead, which returns success or failure.
+  // Convenience to construct a URLPattern from a string. If the string is not
+  // known ahead of time, use Parse() instead, which returns success or failure.
   URLPattern(int valid_schemes, const std::string& pattern);
 
+  URLPattern();
   ~URLPattern();
 
-  // Gets the bitmask of valid schemes.
-  int valid_schemes() const { return valid_schemes_; }
-  void set_valid_schemes(int valid_schemes) { valid_schemes_ = valid_schemes; }
-
-  // Gets the host the pattern matches. This can be an empty string if the
-  // pattern matches all hosts (the input was <scheme>://*/<whatever>).
-  const std::string& host() const { return host_; }
-  void set_host(const std::string& host) { host_ = host; }
-
-  // Gets whether to match subdomains of host().
-  bool match_subdomains() const { return match_subdomains_; }
-  void set_match_subdomains(bool val) { match_subdomains_ = val; }
-
-  // Gets the path the pattern matches with the leading slash. This can have
-  // embedded asterisks which are interpreted using glob rules.
-  const std::string& path() const { return path_; }
-  void set_path(const std::string& path) {
-    path_ = path;
-    path_escaped_ = "";
-  }
-
-  // Returns true if this pattern matches all urls.
-  bool match_all_urls() const { return match_all_urls_; }
-  void set_match_all_urls(bool val) { match_all_urls_ = val; }
+  bool operator<(const URLPattern& other) const;
+  bool operator==(const URLPattern& other) const;
 
   // Initializes this instance by parsing the provided string. Returns
   // URLPattern::PARSE_SUCCESS on success, or an error code otherwise. On
@@ -147,6 +97,28 @@ class URLPattern {
   // invalid state.
   ParseResult Parse(const std::string& pattern_str);
 
+  // Gets the bitmask of valid schemes.
+  int valid_schemes() const { return valid_schemes_; }
+  void SetValidSchemes(int valid_schemes);
+
+  // Gets the host the pattern matches. This can be an empty string if the
+  // pattern matches all hosts (the input was <scheme>://*/<whatever>).
+  const std::string& host() const { return host_; }
+  void SetHost(const std::string& host);
+
+  // Gets whether to match subdomains of host().
+  bool match_subdomains() const { return match_subdomains_; }
+  void SetMatchSubdomains(bool val);
+
+  // Gets the path the pattern matches with the leading slash. This can have
+  // embedded asterisks which are interpreted using glob rules.
+  const std::string& path() const { return path_; }
+  void SetPath(const std::string& path);
+
+  // Returns true if this pattern matches all urls.
+  bool match_all_urls() const { return match_all_urls_; }
+  void SetMatchAllURLs(bool val);
+
   // Sets the scheme for pattern matches. This can be a single '*' if the
   // pattern matches all valid schemes (as defined by the valid_schemes_
   // property). Returns false on failure (if the scheme is not valid).
@@ -160,7 +132,10 @@ class URLPattern {
   bool IsValidScheme(const std::string& scheme) const;
 
   // Returns true if this instance matches the specified URL.
-  bool MatchesUrl(const GURL& url) const;
+  bool MatchesURL(const GURL& test) const;
+
+  // Returns true if this instance matches the specified security origin.
+  bool MatchesSecurityOrigin(const GURL& test) const;
 
   // Returns true if |test| matches our scheme.
   bool MatchesScheme(const std::string& test) const;
@@ -172,8 +147,15 @@ class URLPattern {
   // Returns true if |test| matches our path.
   bool MatchesPath(const std::string& test) const;
 
+  // Returns true if |port| matches our port.
+  bool MatchesPort(int port) const;
+
+  // Sets the port. Returns false if the port is invalid.
+  bool SetPort(const std::string& port);
+  const std::string& port() const { return port_; }
+
   // Returns a string representing this instance.
-  std::string GetAsString() const;
+  const std::string& GetAsString() const;
 
   // Determine whether there is a URL that would match this instance and another
   // instance. This method is symmetrical: Calling other.OverlapsWith(this)
@@ -200,12 +182,18 @@ class URLPattern {
     };
   };
 
- private:
-  friend class std::vector<URLPattern>;
+  // Get an error string for a ParseResult.
+  static const char* GetParseResultString(URLPattern::ParseResult parse_result);
 
-  // Note: don't use this directly. This exists so URLPattern can be used
-  // with STL containers.
-  URLPattern();
+ private:
+  // Returns true if any of the |schemes| items matches our scheme.
+  bool MatchesAnyScheme(const std::vector<std::string>& schemes) const;
+
+  bool MatchesSecurityOriginHelper(const GURL& test) const;
+
+  // If the URLPattern contains a wildcard scheme, returns a list of
+  // equivalent literal schemes, otherwise returns the current scheme.
+  std::vector<std::string> GetExplicitSchemes() const;
 
   // A bitmask containing the schemes which are considered valid for this
   // pattern. Parse() uses this to decide whether a pattern contains a valid
@@ -226,14 +214,19 @@ class URLPattern {
   // component of the pattern's host was "*".
   bool match_subdomains_;
 
+  // The port.
+  std::string port_;
+
   // The path to match. This is everything after the host of the URL, or
   // everything after the scheme in the case of file:// URLs.
   std::string path_;
 
   // The path with "?" and "\" characters escaped for use with the
-  // MatchPattern() function. This is populated lazily, the first time it is
-  // needed.
-  mutable std::string path_escaped_;
+  // MatchPattern() function.
+  std::string path_escaped_;
+
+  // A string representing this URLPattern.
+  mutable std::string spec_;
 };
 
 typedef std::vector<URLPattern> URLPatternList;
