@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,17 +9,6 @@
 
 namespace {
 
-bool UrlMatchesPatterns(const UserScript::PatternList* patterns,
-                        const GURL& url) {
-  for (UserScript::PatternList::const_iterator pattern = patterns->begin();
-       pattern != patterns->end(); ++pattern) {
-    if (pattern->MatchesUrl(url))
-      return true;
-  }
-
-  return false;
-}
-
 bool UrlMatchesGlobs(const std::vector<std::string>* globs,
                      const GURL& url) {
   for (std::vector<std::string>::const_iterator glob = globs->begin();
@@ -36,21 +25,12 @@ bool UrlMatchesGlobs(const std::vector<std::string>* globs,
 // static
 const char UserScript::kFileExtension[] = ".user.js";
 
-// static
-const int UserScript::kValidUserScriptSchemes =
-    URLPattern::SCHEME_HTTP | URLPattern::SCHEME_HTTPS |
-    URLPattern::SCHEME_FILE | URLPattern::SCHEME_FTP;
-
-bool UserScript::HasUserScriptFileExtension(const GURL& url) {
-  return EndsWith(url.ExtractFileName(), kFileExtension, false);
+bool UserScript::IsURLUserScript(const GURL& url,
+                                 const std::string& mime_type) {
+  return EndsWith(url.ExtractFileName(), kFileExtension, false) &&
+      mime_type != "text/html";
 }
 
-bool UserScript::HasUserScriptFileExtension(const FilePath& path) {
-  static FilePath extension(FilePath().AppendASCII(kFileExtension));
-  return EndsWith(path.BaseName().value(), extension.value(), false);
-}
-
-
 UserScript::File::File(const FilePath& extension_root,
                        const FilePath& relative_path,
                        const GURL& url)
@@ -65,31 +45,37 @@ UserScript::File::~File() {}
 
 UserScript::UserScript()
     : run_location_(DOCUMENT_IDLE), emulate_greasemonkey_(false),
-      match_all_frames_(false), incognito_enabled_(false),
-      allow_file_access_(false) {
+      match_all_frames_(false), incognito_enabled_(false) {
 }
 
 UserScript::~UserScript() {
 }
 
 void UserScript::add_url_pattern(const URLPattern& pattern) {
-  url_patterns_.push_back(pattern);
+  url_set_.AddPattern(pattern);
 }
 
-void UserScript::clear_url_patterns() { url_patterns_.clear(); }
+void UserScript::add_exclude_url_pattern(const URLPattern& pattern) {
+  exclude_url_set_.AddPattern(pattern);
+}
 
-bool UserScript::MatchesUrl(const GURL& url) const {
-  if (url_patterns_.size() > 0) {
-    if (!UrlMatchesPatterns(&url_patterns_, url))
+bool UserScript::MatchesURL(const GURL& url) const {
+  if (!url_set_.is_empty()) {
+    if (!url_set_.MatchesURL(url))
       return false;
   }
 
-  if (globs_.size() > 0) {
+  if (!exclude_url_set_.is_empty()) {
+    if (exclude_url_set_.MatchesURL(url))
+      return false;
+  }
+
+  if (!globs_.empty()) {
     if (!UrlMatchesGlobs(&globs_, url))
       return false;
   }
 
-  if (exclude_globs_.size() > 0) {
+  if (!exclude_globs_.empty()) {
     if (UrlMatchesGlobs(&exclude_globs_, url))
       return false;
   }
@@ -104,51 +90,52 @@ void UserScript::File::Pickle(::Pickle* pickle) const {
 }
 
 void UserScript::File::Unpickle(const ::Pickle& pickle, void** iter) {
-  // Read url.
+  // Read the url from the pickle.
   std::string url;
   CHECK(pickle.ReadString(iter, &url));
   set_url(GURL(url));
 }
 
 void UserScript::Pickle(::Pickle* pickle) const {
-  // Write simple types.
+  // Write the simple types to the pickle.
   pickle->WriteInt(run_location());
   pickle->WriteString(extension_id());
   pickle->WriteBool(emulate_greasemonkey());
   pickle->WriteBool(match_all_frames());
   pickle->WriteBool(is_incognito_enabled());
-  pickle->WriteBool(allow_file_access());
 
-  // Write globs.
-  std::vector<std::string>::const_iterator glob;
-  pickle->WriteSize(globs_.size());
-  for (glob = globs_.begin(); glob != globs_.end(); ++glob) {
-    pickle->WriteString(*glob);
-  }
-  pickle->WriteSize(exclude_globs_.size());
-  for (glob = exclude_globs_.begin(); glob != exclude_globs_.end(); ++glob) {
+  PickleGlobs(pickle, globs_);
+  PickleGlobs(pickle, exclude_globs_);
+  PickleURLPatternSet(pickle, url_set_);
+  PickleURLPatternSet(pickle, exclude_url_set_);
+  PickleScripts(pickle, js_scripts_);
+  PickleScripts(pickle, css_scripts_);
+}
+
+void UserScript::PickleGlobs(::Pickle* pickle,
+                             const std::vector<std::string>& globs) const {
+  pickle->WriteSize(globs.size());
+  for (std::vector<std::string>::const_iterator glob = globs.begin();
+       glob != globs.end(); ++glob) {
     pickle->WriteString(*glob);
   }
+}
 
-  // Write url patterns.
-  pickle->WriteSize(url_patterns_.size());
-  for (PatternList::const_iterator pattern = url_patterns_.begin();
-       pattern != url_patterns_.end(); ++pattern) {
+void UserScript::PickleURLPatternSet(::Pickle* pickle,
+                                     const URLPatternSet& pattern_list) const {
+  pickle->WriteSize(pattern_list.patterns().size());
+  for (URLPatternSet::const_iterator pattern = pattern_list.begin();
+       pattern != pattern_list.end(); ++pattern) {
     pickle->WriteInt(pattern->valid_schemes());
     pickle->WriteString(pattern->GetAsString());
   }
+}
 
-  // Write js scripts.
-  pickle->WriteSize(js_scripts_.size());
-  for (FileList::const_iterator file = js_scripts_.begin();
-    file != js_scripts_.end(); ++file) {
-    file->Pickle(pickle);
-  }
-
-  // Write css scripts.
-  pickle->WriteSize(css_scripts_.size());
-  for (FileList::const_iterator file = css_scripts_.begin();
-    file != css_scripts_.end(); ++file) {
+void UserScript::PickleScripts(::Pickle* pickle,
+                               const FileList& scripts) const {
+  pickle->WriteSize(scripts.size());
+  for (FileList::const_iterator file = scripts.begin();
+       file != scripts.end(); ++file) {
     file->Pickle(pickle);
   }
 }
@@ -164,58 +151,62 @@ void UserScript::Unpickle(const ::Pickle& pickle, void** iter) {
   CHECK(pickle.ReadBool(iter, &emulate_greasemonkey_));
   CHECK(pickle.ReadBool(iter, &match_all_frames_));
   CHECK(pickle.ReadBool(iter, &incognito_enabled_));
-  CHECK(pickle.ReadBool(iter, &allow_file_access_));
 
-  // Read globs.
+  UnpickleGlobs(pickle, iter, &globs_);
+  UnpickleGlobs(pickle, iter, &exclude_globs_);
+  UnpickleURLPatternSet(pickle, iter, &url_set_);
+  UnpickleURLPatternSet(pickle, iter, &exclude_url_set_);
+  UnpickleScripts(pickle, iter, &js_scripts_);
+  UnpickleScripts(pickle, iter, &css_scripts_);
+}
+
+void UserScript::UnpickleGlobs(const ::Pickle& pickle, void** iter,
+                               std::vector<std::string>* globs) {
   size_t num_globs = 0;
   CHECK(pickle.ReadSize(iter, &num_globs));
-  globs_.clear();
+  globs->clear();
   for (size_t i = 0; i < num_globs; ++i) {
     std::string glob;
     CHECK(pickle.ReadString(iter, &glob));
-    globs_.push_back(glob);
+    globs->push_back(glob);
   }
+}
 
-  CHECK(pickle.ReadSize(iter, &num_globs));
-  exclude_globs_.clear();
-  for (size_t i = 0; i < num_globs; ++i) {
-    std::string glob;
-    CHECK(pickle.ReadString(iter, &glob));
-    exclude_globs_.push_back(glob);
-  }
-
-  // Read url patterns.
+void UserScript::UnpickleURLPatternSet(const ::Pickle& pickle, void** iter,
+                                       URLPatternSet* pattern_list) {
   size_t num_patterns = 0;
   CHECK(pickle.ReadSize(iter, &num_patterns));
 
-  url_patterns_.clear();
+  pattern_list->ClearPatterns();
   for (size_t i = 0; i < num_patterns; ++i) {
     int valid_schemes;
     CHECK(pickle.ReadInt(iter, &valid_schemes));
     std::string pattern_str;
     URLPattern pattern(valid_schemes);
     CHECK(pickle.ReadString(iter, &pattern_str));
+
+    // We remove the file scheme if it's not actually allowed (see Extension::
+    // LoadUserScriptHelper), but we need it temporarily while loading the
+    // pattern so that it's valid.
+    bool had_file_scheme = (valid_schemes & URLPattern::SCHEME_FILE) != 0;
+    if (!had_file_scheme)
+      pattern.SetValidSchemes(valid_schemes | URLPattern::SCHEME_FILE);
     CHECK(URLPattern::PARSE_SUCCESS == pattern.Parse(pattern_str));
-    url_patterns_.push_back(pattern);
-  }
+    if (!had_file_scheme)
+      pattern.SetValidSchemes(valid_schemes);
 
-  // Read js scripts.
-  size_t num_js_files = 0;
-  CHECK(pickle.ReadSize(iter, &num_js_files));
-  js_scripts_.clear();
-  for (size_t i = 0; i < num_js_files; ++i) {
-    File file;
-    file.Unpickle(pickle, iter);
-    js_scripts_.push_back(file);
+    pattern_list->AddPattern(pattern);
   }
+}
 
-  // Read css scripts.
-  size_t num_css_files = 0;
-  CHECK(pickle.ReadSize(iter, &num_css_files));
-  css_scripts_.clear();
-  for (size_t i = 0; i < num_css_files; ++i) {
+void UserScript::UnpickleScripts(const ::Pickle& pickle, void** iter,
+                                 FileList* scripts) {
+  size_t num_files = 0;
+  CHECK(pickle.ReadSize(iter, &num_files));
+  scripts->clear();
+  for (size_t i = 0; i < num_files; ++i) {
     File file;
     file.Unpickle(pickle, iter);
-    css_scripts_.push_back(file);
+    scripts->push_back(file);
   }
 }
