@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -16,10 +16,7 @@
 // IPC_MESSAGE_MACROS_LOG_ENABLED doesn't get undefined.
 #if defined(OS_POSIX) && defined(IPC_MESSAGE_LOG_ENABLED)
 #define IPC_MESSAGE_MACROS_LOG_ENABLED
-#include "chrome/common/devtools_messages.h"
-#include "chrome/common/plugin_messages.h"
-#include "chrome/common/render_messages.h"
-#include "chrome/common/worker_messages.h"
+#include "chrome/common/all_messages.h"
 #endif
 
 #if defined(OS_WIN)
@@ -40,31 +37,40 @@
 #include "base/path_service.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
+#include "base/stringprintf.h"
+#include "base/threading/thread_restrictions.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/env_vars.h"
 #include "ipc/ipc_logging.h"
+
 #if defined(OS_WIN)
 #include "base/logging_win.h"
 #include <initguid.h>
 #endif
 
+namespace {
+
 // When true, this means that error dialogs should not be shown.
-static bool dialogs_are_suppressed_ = false;
+bool dialogs_are_suppressed_ = false;
 
 // This should be true for exactly the period between the end of
 // InitChromeLogging() and the beginning of CleanupChromeLogging().
-static bool chrome_logging_initialized_ = false;
+bool chrome_logging_initialized_ = false;
+
+// Set if we called InitChromeLogging() but failed to initialize.
+bool chrome_logging_failed_ = false;
 
 // This should be true for exactly the period between the end of
 // InitChromeLogging() and the beginning of CleanupChromeLogging().
-static bool chrome_logging_redirected_ = false;
+bool chrome_logging_redirected_ = false;
 
 #if defined(OS_WIN)
 // {7FE69228-633E-4f06-80C1-527FEA23E3A7}
-static const GUID kChromeTraceProviderName = {
+const GUID kChromeTraceProviderName = {
     0x7fe69228, 0x633e, 0x4f06,
         { 0x80, 0xc1, 0x52, 0x7f, 0xea, 0x23, 0xe3, 0xa7 } };
 #endif
@@ -73,16 +79,29 @@ static const GUID kChromeTraceProviderName = {
 // silenced.  To record a new error, pass the log string associated
 // with that error in the str parameter.
 MSVC_DISABLE_OPTIMIZE();
-static void SilentRuntimeAssertHandler(const std::string& str) {
+void SilentRuntimeAssertHandler(const std::string& str) {
   base::debug::BreakDebugger();
 }
-static void SilentRuntimeReportHandler(const std::string& str) {
+void SilentRuntimeReportHandler(const std::string& str) {
 }
+#if defined(OS_WIN)
+// Handler to silently dump the current process when there is an assert in
+// chrome.
+void DumpProcessAssertHandler(const std::string& str) {
+  // Get the breakpad pointer from chrome.exe
+  typedef void (__cdecl *DumpProcessFunction)();
+  DumpProcessFunction DumpProcess = reinterpret_cast<DumpProcessFunction>(
+      ::GetProcAddress(::GetModuleHandle(chrome::kBrowserProcessExecutableName),
+                       "DumpProcess"));
+  if (DumpProcess)
+    DumpProcess();
+}
+#endif  // OS_WIN
 MSVC_ENABLE_OPTIMIZE();
 
 // Suppresses error/assertion dialogs and enables the logging of
 // those errors into silenced_errors_.
-static void SuppressDialogs() {
+void SuppressDialogs() {
   if (dialogs_are_suppressed_)
     return;
 
@@ -102,6 +121,8 @@ static void SuppressDialogs() {
   dialogs_are_suppressed_ = true;
 }
 
+}  // anonymous namespace
+
 namespace logging {
 
 LoggingDestination DetermineLogMode(const CommandLine& command_line) {
@@ -166,15 +187,15 @@ FilePath SetUpSymlinkIfNeeded(const FilePath& symlink_path, bool new_log) {
     // We don't care if the unlink fails; we're going to continue anyway.
     if (::unlink(symlink_path.value().c_str()) == -1) {
       if (symlink_exists) // only warn if we might expect it to succeed.
-        PLOG(WARNING) << "Unable to unlink " << symlink_path.value();
+        DPLOG(WARNING) << "Unable to unlink " << symlink_path.value();
     }
     if (!file_util::CreateSymbolicLink(target_path, symlink_path)) {
-      PLOG(ERROR) << "Unable to create symlink " << symlink_path.value()
-                  << " pointing at " << target_path.value();
+      DPLOG(ERROR) << "Unable to create symlink " << symlink_path.value()
+                   << " pointing at " << target_path.value();
     }
   } else {
     if (!file_util::ReadSymbolicLink(symlink_path, &target_path))
-      PLOG(ERROR) << "Unable to read symlink " << symlink_path.value();
+      DPLOG(ERROR) << "Unable to read symlink " << symlink_path.value();
   }
   return target_path;
 }
@@ -182,9 +203,9 @@ FilePath SetUpSymlinkIfNeeded(const FilePath& symlink_path, bool new_log) {
 void RemoveSymlinkAndLog(const FilePath& link_path,
                          const FilePath& target_path) {
   if (::unlink(link_path.value().c_str()) == -1)
-    PLOG(WARNING) << "Unable to unlink symlink " << link_path.value();
+    DPLOG(WARNING) << "Unable to unlink symlink " << link_path.value();
   if (::unlink(target_path.value().c_str()) == -1)
-    PLOG(WARNING) << "Unable to unlink log file " << target_path.value();
+    DPLOG(WARNING) << "Unable to unlink log file " << target_path.value();
 }
 
 }  // anonymous namespace
@@ -213,6 +234,9 @@ void RedirectChromeLogging(const CommandLine& command_line) {
   // defaults to the profile dir.
   FilePath log_path = GetSessionLogFile(command_line);
 
+  // Creating symlink causes us to do blocking IO on UI thread.
+  // Temporarily allow it until we fix http://crbug.com/61143
+  base::ThreadRestrictions::ScopedAllowIO allow_io;
   // Always force a new symlink when redirecting.
   FilePath target_path = SetUpSymlinkIfNeeded(log_path, true);
 
@@ -228,15 +252,14 @@ void RedirectChromeLogging(const CommandLine& command_line) {
                    logging::LOCK_LOG_FILE,
                    logging::APPEND_TO_OLD_LOG_FILE,
                    dcheck_state)) {
-    LOG(ERROR) << "Unable to initialize logging to " << log_path.value();
+    DLOG(ERROR) << "Unable to initialize logging to " << log_path.value();
     RemoveSymlinkAndLog(log_path, target_path);
   } else {
     chrome_logging_redirected_ = true;
   }
 }
 
-
-#endif
+#endif  // OS_CHROMEOS
 
 void InitChromeLogging(const CommandLine& command_line,
                        OldFileDeletionState delete_old_log_file) {
@@ -246,27 +269,37 @@ void InitChromeLogging(const CommandLine& command_line,
 #if defined(OS_POSIX) && defined(IPC_MESSAGE_LOG_ENABLED)
   IPC::Logging::set_log_function_map(&g_log_function_mapping);
 #endif
+  LoggingDestination logging_dest = DetermineLogMode(command_line);
+  FilePath log_path;
+#if defined(OS_CHROMEOS)
+    FilePath target_path;
+#endif
 
-  FilePath log_path = GetLogFileName();
+  // Don't resolve the log path unless we need to. Otherwise we leave an open
+  // ALPC handle after sandbox lockdown on Windows.
+  if (logging_dest == LOG_ONLY_TO_FILE ||
+      logging_dest == LOG_TO_BOTH_FILE_AND_SYSTEM_DEBUG_LOG) {
+    log_path = GetLogFileName();
 
 #if defined(OS_CHROMEOS)
-  // For BWSI (Incognito) logins, we want to put the logs in the user
-  // profile directory that is created for the temporary session instead
-  // of in the system log directory, for privacy reasons.
-  if (command_line.HasSwitch(switches::kGuestSession))
-    log_path = GetSessionLogFile(command_line);
+    // For BWSI (Incognito) logins, we want to put the logs in the user
+    // profile directory that is created for the temporary session instead
+    // of in the system log directory, for privacy reasons.
+    if (command_line.HasSwitch(switches::kGuestSession))
+      log_path = GetSessionLogFile(command_line);
 
-  // On ChromeOS we log to the symlink.  We force creation of a new
-  // symlink if we've been asked to delete the old log, since that
-  // indicates the start of a new session.
-  FilePath target_path = SetUpSymlinkIfNeeded(
-      log_path, delete_old_log_file == logging::DELETE_OLD_LOG_FILE);
+    // On ChromeOS we log to the symlink.  We force creation of a new
+    // symlink if we've been asked to delete the old log, since that
+    // indicates the start of a new session.
+    target_path = SetUpSymlinkIfNeeded(
+        log_path, delete_old_log_file == logging::DELETE_OLD_LOG_FILE);
 
-  // Because ChromeOS manages the move to a new session by redirecting
-  // the link, it shouldn't remove the old file in the logging code,
-  // since that will remove the newly created link instead.
-  delete_old_log_file = logging::APPEND_TO_OLD_LOG_FILE;
+    // Because ChromeOS manages the move to a new session by redirecting
+    // the link, it shouldn't remove the old file in the logging code,
+    // since that will remove the newly created link instead.
+    delete_old_log_file = logging::APPEND_TO_OLD_LOG_FILE;
 #endif
+  }
 
   logging::DcheckState dcheck_state =
       command_line.HasSwitch(switches::kEnableDCHECK) ?
@@ -274,21 +307,23 @@ void InitChromeLogging(const CommandLine& command_line,
       logging::DISABLE_DCHECK_FOR_NON_OFFICIAL_RELEASE_BUILDS;
 
   bool success = InitLogging(log_path.value().c_str(),
-                             DetermineLogMode(command_line),
+                             logging_dest,
                              logging::LOCK_LOG_FILE,
                              delete_old_log_file,
                              dcheck_state);
 
 #if defined(OS_CHROMEOS)
   if (!success) {
-    PLOG(ERROR) << "Unable to initialize logging to " << log_path.value()
+    DPLOG(ERROR) << "Unable to initialize logging to " << log_path.value()
                 << " (which should be a link to " << target_path.value() << ")";
     RemoveSymlinkAndLog(log_path, target_path);
+    chrome_logging_failed_ = true;
     return;
   }
 #else
   if (!success) {
-    PLOG(ERROR) << "Unable to initialize logging to " << log_path.value();
+    DPLOG(ERROR) << "Unable to initialize logging to " << log_path.value();
+    chrome_logging_failed_ = true;
     return;
   }
 #endif
@@ -322,7 +357,7 @@ void InitChromeLogging(const CommandLine& command_line,
         level >= 0 && level < LOG_NUM_SEVERITIES) {
       logging::SetMinLogLevel(level);
     } else {
-      LOG(WARNING) << "Bad log level: " << log_level;
+      DLOG(WARNING) << "Bad log level: " << log_level;
     }
   }
 
@@ -332,12 +367,24 @@ void InitChromeLogging(const CommandLine& command_line,
     logging::LogEventProvider::Initialize(kChromeTraceProviderName);
 #endif
 
+#ifdef NDEBUG
+  if (command_line.HasSwitch(switches::kSilentDumpOnDCHECK) &&
+      command_line.HasSwitch(switches::kEnableDCHECK)) {
+#if defined(OS_WIN)
+    logging::SetLogReportHandler(DumpProcessAssertHandler);
+#endif
+  }
+#endif  // NDEBUG
+
   chrome_logging_initialized_ = true;
 }
 
 // This is a no-op, but we'll keep it around in case
 // we need to do more cleanup in the future.
 void CleanupChromeLogging() {
+  if (chrome_logging_failed_)
+    return;  // We failed to initiailize logging, no cleanup.
+
   DCHECK(chrome_logging_initialized_) <<
     "Attempted to clean up logging when it wasn't initialized.";
 
@@ -352,9 +399,9 @@ FilePath GetLogFileName() {
   scoped_ptr<base::Environment> env(base::Environment::Create());
   if (env->GetVar(env_vars::kLogFileName, &filename) && !filename.empty()) {
 #if defined(OS_WIN)
-    return FilePath(UTF8ToWide(filename).c_str());
+    return FilePath(UTF8ToWide(filename));
 #elif defined(OS_POSIX)
-    return FilePath(filename.c_str());
+    return FilePath(filename);
 #endif
   }
 
@@ -402,4 +449,14 @@ size_t GetFatalAssertions(AssertionList* assertions) {
   return assertion_count;
 }
 
+
+void DumpWithoutCrashing() {
+#if defined(OS_WIN)
+  std::string str;
+  DumpProcessAssertHandler(str);
+#else
+  NOTIMPLEMENTED();
+#endif  // OS_WIN
+}
+
 }  // namespace logging
