@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,24 +15,14 @@ class GoogleServiceAuthError;
 class GaiaAuthConsumer {
  public:
   struct ClientLoginResult {
-    inline ClientLoginResult() : two_factor(false) {}
-    inline ClientLoginResult(const std::string& new_sid,
-                             const std::string& new_lsid,
-                             const std::string& new_token,
-                             const std::string& new_data)
-        : sid(new_sid),
-          lsid(new_lsid),
-          token(new_token),
-          data(new_data),
-          two_factor(false) {}
+    ClientLoginResult();
+    ClientLoginResult(const std::string& new_sid,
+                      const std::string& new_lsid,
+                      const std::string& new_token,
+                      const std::string& new_data);
+    ~ClientLoginResult();
 
-    inline bool operator==(const ClientLoginResult &b) const {
-      return sid == b.sid &&
-             lsid == b.lsid &&
-             token == b.token &&
-             data == b.data &&
-             two_factor == b.two_factor;
-    }
+    bool operator==(const ClientLoginResult &b) const;
 
     std::string sid;
     std::string lsid;
@@ -52,10 +42,21 @@ class GaiaAuthConsumer {
   virtual void OnIssueAuthTokenFailure(const std::string& service,
                                        const GoogleServiceAuthError& error) {}
 
+  virtual void OnOAuthLoginTokenSuccess(const std::string& refresh_token,
+                                        const std::string& access_token,
+                                        int expires_in_secs) {}
+  virtual void OnOAuthLoginTokenFailure(const GoogleServiceAuthError& error) {}
+
   virtual void OnGetUserInfoSuccess(const std::string& key,
                                     const std::string& value) {}
   virtual void OnGetUserInfoKeyNotFound(const std::string& key) {}
   virtual void OnGetUserInfoFailure(const GoogleServiceAuthError& error) {}
+
+  virtual void OnTokenAuthSuccess(const std::string& data) {}
+  virtual void OnTokenAuthFailure(const GoogleServiceAuthError& error) {}
+
+  virtual void OnMergeSessionSuccess(const std::string& data) {}
+  virtual void OnMergeSessionFailure(const GoogleServiceAuthError& error) {}
 };
 
 #endif  // CHROME_COMMON_NET_GAIA_GAIA_AUTH_CONSUMER_H_
