@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,9 +6,7 @@
 #define CHROME_COMMON_NET_NET_RESOURCE_PROVIDER_H_
 #pragma once
 
-namespace base {
-class StringPiece;
-}
+#include "base/string_piece.h"
 
 namespace chrome_common_net {
 
