@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,14 +7,16 @@
 #include <cert.h>
 #include <cms.h>
 #include <hasht.h>
-#include <pk11pub.h>
+#include <keyhi.h>  // SECKEY_DestroyPrivateKey
+#include <keythi.h>  // SECKEYPrivateKey
+#include <pk11pub.h>  // PK11_FindKeyByAnyCert
+#include <seccomon.h>  // SECItem
 #include <sechash.h>
 
-#include <pk11pub.h>
-
 #include "base/logging.h"
-#include "base/nss_util.h"
 #include "base/string_number_conversions.h"
+#include "crypto/nss_util.h"
+#include "crypto/scoped_nss_types.h"
 #include "net/base/x509_certificate.h"
 #include "chrome/third_party/mozilla_security_manager/nsNSSCertHelper.h"
 #include "chrome/third_party/mozilla_security_manager/nsNSSCertificate.h"
@@ -100,7 +102,14 @@ using std::string;
 string GetCertNameOrNickname(X509Certificate::OSCertHandle cert_handle) {
   string name = ProcessIDN(Stringize(CERT_GetCommonName(&cert_handle->subject),
                                      ""));
-  if (name.empty() && cert_handle->nickname) {
+  if (!name.empty())
+    return name;
+  return GetNickname(cert_handle);
+}
+
+string GetNickname(X509Certificate::OSCertHandle cert_handle) {
+  string name;
+  if (cert_handle->nickname) {
     name = cert_handle->nickname;
     // Hack copied from mozilla: Cut off text before first :, which seems to
     // just be the token name.
@@ -190,8 +199,8 @@ bool GetTimes(X509Certificate::OSCertHandle cert_handle,
               base::Time* issued, base::Time* expires) {
   PRTime pr_issued, pr_expires;
   if (CERT_GetCertTimes(cert_handle, &pr_issued, &pr_expires) == SECSuccess) {
-    *issued = base::PRTimeToBaseTime(pr_issued);
-    *expires = base::PRTimeToBaseTime(pr_expires);
+    *issued = crypto::PRTimeToBaseTime(pr_issued);
+    *expires = crypto::PRTimeToBaseTime(pr_expires);
     return true;
   }
   return false;
@@ -249,6 +258,29 @@ void GetNicknameStringsFromCertList(
   CERT_DestroyCertList(cert_list);
 }
 
+// For background see this discussion on dev-tech-crypto.lists.mozilla.org:
+// http://web.archiveorange.com/archive/v/6JJW7E40sypfZGtbkzxX
+//
+// NOTE: This function relies on the convention that the same PKCS#11 ID
+// is shared between a certificate and its associated private and public
+// keys.  I tried to implement this with PK11_GetLowLevelKeyIDForCert(),
+// but that always returns NULL on Chrome OS for me.
+std::string GetPkcs11Id(net::X509Certificate::OSCertHandle cert_handle) {
+  std::string pkcs11_id;
+  SECKEYPrivateKey *priv_key = PK11_FindKeyByAnyCert(cert_handle,
+                                                     NULL /* wincx */);
+  if (priv_key) {
+    // Get the CKA_ID attribute for a key.
+    SECItem* sec_item = PK11_GetLowLevelKeyIDForPrivateKey(priv_key);
+    if (sec_item) {
+      pkcs11_id = base::HexEncode(sec_item->data, sec_item->len);
+      SECITEM_FreeItem(sec_item, PR_TRUE);
+    }
+    SECKEY_DestroyPrivateKey(priv_key);
+  }
+  return pkcs11_id;
+}
+
 void GetExtensions(
     const string& critical_label,
     const string& non_critical_label,
@@ -300,24 +332,24 @@ string GetDerString(X509Certificate::OSCertHandle cert_handle) {
 
 string GetCMSString(const X509Certificate::OSCertHandles& cert_chain,
                     size_t start, size_t end) {
-  ScopedPRArenaPool arena(PORT_NewArena(1024));
-  CHECK(arena.get());
+  crypto::ScopedPLArenaPool arena(PORT_NewArena(1024));
+  DCHECK(arena.get());
 
   ScopedNSSCMSMessage message(NSS_CMSMessage_Create(arena.get()));
-  CHECK(message.get());
+  DCHECK(message.get());
 
   // First, create SignedData with the certificate only (no chain).
   ScopedNSSCMSSignedData signed_data(NSS_CMSSignedData_CreateCertsOnly(
       message.get(), cert_chain[start], PR_FALSE));
   if (!signed_data.get()) {
-    LOG(ERROR) << "NSS_CMSSignedData_Create failed";
+    DLOG(ERROR) << "NSS_CMSSignedData_Create failed";
     return "";
   }
   // Add the rest of the chain (if any).
   for (size_t i = start + 1; i < end; ++i) {
     if (NSS_CMSSignedData_AddCertificate(signed_data.get(), cert_chain[i]) !=
         SECSuccess) {
-      LOG(ERROR) << "NSS_CMSSignedData_AddCertificate failed on " << i;
+      DLOG(ERROR) << "NSS_CMSSignedData_AddCertificate failed on " << i;
       return "";
     }
   }
@@ -327,7 +359,7 @@ string GetCMSString(const X509Certificate::OSCertHandles& cert_chain,
       message.get(), cinfo, signed_data.get()) == SECSuccess) {
     ignore_result(signed_data.release());
   } else {
-    LOG(ERROR) << "NSS_CMSMessage_GetContentInfo failed";
+    DLOG(ERROR) << "NSS_CMSMessage_GetContentInfo failed";
     return "";
   }
 
@@ -337,12 +369,12 @@ string GetCMSString(const X509Certificate::OSCertHandles& cert_chain,
                                                    NULL, NULL, NULL, NULL,
                                                    NULL);
   if (!ecx) {
-    LOG(ERROR) << "NSS_CMSEncoder_Start failed";
+    DLOG(ERROR) << "NSS_CMSEncoder_Start failed";
     return "";
   }
 
   if (NSS_CMSEncoder_Finish(ecx) != SECSuccess) {
-    LOG(ERROR) << "NSS_CMSEncoder_Finish failed";
+    DLOG(ERROR) << "NSS_CMSEncoder_Finish failed";
     return "";
   }
 
@@ -375,6 +407,7 @@ string ProcessRawBitsSignatureWrap(X509Certificate::OSCertHandle cert_handle) {
 }
 
 void RegisterDynamicOids() {
+  psm::RegisterDynamicOids();
 }
 
 }  // namespace x509_certificate_model
