@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,8 +9,11 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/memory/ref_counted.h"
 
+namespace base {
 class Value;
+}
 
 // This is an abstract interface for reading and writing from/to a persistent
 // preference store, used by PrefService. An implementation using a JSON file
@@ -19,7 +22,7 @@ class Value;
 // CommandLinePrefStore, which bridges command line options to preferences and
 // ConfigurationPolicyPrefStore, which is used for hooking up configuration
 // policy with the preference subsystem.
-class PrefStore {
+class PrefStore : public base::RefCounted<PrefStore> {
  public:
   // Observer interface for monitoring PrefStore.
   class Observer {
@@ -29,7 +32,7 @@ class PrefStore {
     // Called when the value for the given |key| in the store changes.
     virtual void OnPrefValueChanged(const std::string& key) = 0;
     // Notification about the PrefStore being fully initialized.
-    virtual void OnInitializationCompleted() = 0;
+    virtual void OnInitializationCompleted(bool succeeded) = 0;
   };
 
   // Return values for GetValue().
@@ -43,20 +46,26 @@ class PrefStore {
   };
 
   PrefStore() {}
-  virtual ~PrefStore() {}
 
   // Add and remove observers.
   virtual void AddObserver(Observer* observer) {}
   virtual void RemoveObserver(Observer* observer) {}
+  virtual size_t NumberOfObservers() const;
 
   // Whether the store has completed all asynchronous initialization.
   virtual bool IsInitializationComplete() const;
 
-  // Get the value for a given preference |key| and stores it in |result|.
-  // |result| is only modified if the return value is READ_OK. Ownership of the
-  // |result| value remains with the PrefStore.
-  virtual ReadResult GetValue(const std::string& key, Value** result) const = 0;
+  // Get the value for a given preference |key| and stores it in |*result|.
+  // |*result| is only modified if the return value is READ_OK and if |result|
+  // is not NULL. Ownership of the |*result| value remains with the PrefStore.
+  virtual ReadResult GetValue(const std::string& key,
+                              const base::Value** result) const = 0;
 
+ protected:
+  friend class base::RefCounted<PrefStore>;
+  virtual ~PrefStore() {}
+
+ private:
   DISALLOW_COPY_AND_ASSIGN(PrefStore);
 };
 
