@@ -8,4 +8,6 @@ namespace google_update {
 const wchar_t kAppGuid[] = L"{8A69D345-D564-463c-AFF1-A69D9E530F96}";
 const wchar_t kSxSAppGuid[] = L"{4ea16ac7-fd5a-47c3-875b-dbf4a2008c20}";
 const wchar_t kChromeFrameAppGuid[] = L"{8BA986DA-5100-405E-AA35-86F34A02ACBF}";
+const wchar_t kMultiInstallAppGuid[] =
+    L"{4DC8B4CA-1BDA-483e-B5FA-D3C12E15B62D}";
 }
