@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -14,18 +14,22 @@
 
 #include "base/command_line.h"
 #include "base/file_path.h"
+#include "base/json/json_value_serializer.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
+#include "base/process_util.h"
 #include "base/rand_util.h"
-#include "base/scoped_ptr.h"
 #include "base/string_number_conversions.h"
+#include "base/string_split.h"
 #include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/registry.h"
 #include "base/win/windows_version.h"
+#include "chrome/common/attrition_experiments.h"
+#include "chrome/common/chrome_result_codes.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/json_value_serializer.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/common/result_codes.h"
 #include "chrome/installer/util/channel_info.h"
 #include "chrome/installer/util/product.h"
 #include "chrome/installer/util/install_util.h"
@@ -46,29 +50,15 @@ const wchar_t kChromeGuid[] = L"{8A69D345-D564-463c-AFF1-A69D9E530F96}";
 const wchar_t kBrowserAppId[] = L"Chrome";
 
 // The following strings are the possible outcomes of the toast experiment
-// as recorded in the  |client| field. Previously the groups used "TSxx" but
-// the data captured is not valid.
-const wchar_t kToastExpControlGroup[] =      L"T%lc01";
-const wchar_t kToastExpCancelGroup[] =       L"T%lc02";
-const wchar_t kToastExpUninstallGroup[] =    L"T%lc04";
-const wchar_t kToastExpTriesOkGroup[] =      L"T%lc18";
-const wchar_t kToastExpTriesErrorGroup[] =   L"T%lc28";
-const wchar_t kToastActiveGroup[] =          L"T%lc40";
-const wchar_t kToastUDDirFailure[] =         L"T%lc40";
-const wchar_t kToastExpBaseGroup[] =         L"T%lc80";
-
-// Generates the actual group string that gets written in the registry.
-// |group| is one of the above kToast* strings and |flavor| is a number
-// from 0 to 3.
-//
-// The big experiment in Dec 2009 used TGxx and THxx.
-// The big experiment in Feb 2010 used TKxx and TLxx.
-// The big experiment in Apr 2010 used TMxx and TNxx.
-// The big experiment in Oct 2010 (current) uses TVxx TWxx TXxx TYxx.
-std::wstring GetExperimentGroup(const wchar_t* group, int flavor) {
-  wchar_t c = flavor < 4 ? L'V' + flavor : L'Z';
-  return StringPrintf(group, c);
-}
+// as recorded in the |client| field.
+const wchar_t kToastExpControlGroup[] =      L"01";
+const wchar_t kToastExpCancelGroup[] =       L"02";
+const wchar_t kToastExpUninstallGroup[] =    L"04";
+const wchar_t kToastExpTriesOkGroup[] =      L"18";
+const wchar_t kToastExpTriesErrorGroup[] =   L"28";
+const wchar_t kToastActiveGroup[] =          L"40";
+const wchar_t kToastUDDirFailure[] =         L"40";
+const wchar_t kToastExpBaseGroup[] =         L"80";
 
 // Substitute the locale parameter in uninstall URL with whatever
 // Google Update tells us is the locale. In case we fail to find
@@ -110,8 +100,12 @@ int GetDirectoryWriteTimeInHours(const wchar_t* path) {
   if (INVALID_HANDLE_VALUE == file)
     return -1;
   FILETIME time;
-  if (!::GetFileTime(file, NULL, NULL, &time))
+  if (!::GetFileTime(file, NULL, NULL, &time)) {
+    ::CloseHandle(file);
     return -1;
+  }
+
+  ::CloseHandle(file);
   return FileTimeToHours(time);
 }
 
@@ -130,18 +124,14 @@ int GetDirectoryWriteAgeInHours(const wchar_t* path) {
   return (now_time - dir_time);
 }
 
-// Launches again this same process with switch --|flag|=|value|.
+// Launches setup.exe (located at |setup_path|) with |cmd_line|.
 // If system_level_toast is true, appends --system-level-toast.
 // If handle to experiment result key was given at startup, re-add it.
 // Does not wait for the process to terminate.
-bool RelaunchSetup(const std::string& flag, int value,
-                   bool system_level_toast) {
-  CommandLine new_cmd_line(CommandLine::ForCurrentProcess()->GetProgram());
-  new_cmd_line.AppendSwitchASCII(flag, base::IntToString(value));
-
+bool LaunchSetup(CommandLine cmd_line, bool system_level_toast) {
   // Re-add the system level toast flag.
   if (system_level_toast) {
-    new_cmd_line.AppendSwitch(installer::switches::kSystemLevelToast);
+    cmd_line.AppendSwitch(installer::switches::kSystemLevelToast);
 
     // Re-add the toast result key. We need to do this because Setup running as
     // system passes the key to Setup running as user, but that child process
@@ -151,17 +141,17 @@ bool RelaunchSetup(const std::string& flag, int value,
     std::string key(installer::switches::kToastResultsKey);
     std::string toast_key = current_cmd_line.GetSwitchValueASCII(key);
     if (!toast_key.empty()) {
-      new_cmd_line.AppendSwitchASCII(key, toast_key);
+      cmd_line.AppendSwitchASCII(key, toast_key);
 
       // Use handle inheritance to make sure the duplicated toast results key
       // gets inherited by the child process.
-      return base::LaunchAppWithHandleInheritance(
-          new_cmd_line.command_line_string(), false, false, NULL);
+      base::LaunchOptions options;
+      options.inherit_handles = true;
+      return base::LaunchProcess(cmd_line, options, NULL);
     }
   }
 
-  return base::LaunchApp(new_cmd_line.command_line_string(),
-                         false, false, NULL);
+  return base::LaunchProcess(cmd_line, base::LaunchOptions(), NULL);
 }
 
 // For System level installs, setup.exe lives in the system temp, which
@@ -170,7 +160,7 @@ bool RelaunchSetup(const std::string& flag, int value,
 // This function changes the permisions so that any authenticated user
 // can launch |exe| later on. This function should only be called if the
 // code is running at the system level.
-bool FixDACLsForExecute(const wchar_t* exe) {
+bool FixDACLsForExecute(const FilePath& exe) {
   // The general strategy to is to add an ACE to the exe DACL the quick
   // and dirty way: a) read the DACL b) convert it to sddl string c) add the
   // new ACE to the string d) convert sddl string back to DACL and finally
@@ -178,8 +168,10 @@ bool FixDACLsForExecute(const wchar_t* exe) {
   char buff[1024];
   DWORD len = sizeof(buff);
   PSECURITY_DESCRIPTOR sd = reinterpret_cast<PSECURITY_DESCRIPTOR>(buff);
-  if (!::GetFileSecurityW(exe, DACL_SECURITY_INFORMATION, sd, len, &len))
+  if (!::GetFileSecurityW(exe.value().c_str(), DACL_SECURITY_INFORMATION,
+                          sd, len, &len)) {
     return false;
+  }
   wchar_t* sddl = 0;
   if (!::ConvertSecurityDescriptorToStringSecurityDescriptorW(sd,
       SDDL_REVISION_1, DACL_SECURITY_INFORMATION, &sddl, NULL))
@@ -205,7 +197,8 @@ bool FixDACLsForExecute(const wchar_t* exe) {
   if (!::ConvertStringSecurityDescriptorToSecurityDescriptorW(new_sddl.c_str(),
       SDDL_REVISION_1, &sd, NULL))
     return false;
-  bool rv = ::SetFileSecurityW(exe, DACL_SECURITY_INFORMATION, sd) == TRUE;
+  bool rv = ::SetFileSecurityW(exe.value().c_str(), DACL_SECURITY_INFORMATION,
+                               sd) == TRUE;
   ::LocalFree(sd);
   return rv;
 }
@@ -218,9 +211,9 @@ bool FixDACLsForExecute(const wchar_t* exe) {
 // the computer is on but nobody has logged in locally.
 // Remote Desktop sessions do not count as interactive sessions; running this
 // method as a user logged in via remote desktop will do nothing.
-bool RelaunchSetupAsConsoleUser(const std::string& flag) {
-  FilePath setup_exe = CommandLine::ForCurrentProcess()->GetProgram();
-  CommandLine cmd_line(setup_exe);
+bool LaunchSetupAsConsoleUser(const FilePath& setup_path,
+                              const std::string& flag) {
+  CommandLine cmd_line(setup_path);
   cmd_line.AppendSwitch(flag);
 
   // Get the Google Update results key, and pass it on the command line to
@@ -232,7 +225,7 @@ bool RelaunchSetupAsConsoleUser(const std::string& flag) {
   if (base::win::GetVersion() > base::win::VERSION_XP) {
     // Make sure that in Vista and Above we have the proper DACLs so
     // the interactive user can launch it.
-    if (!FixDACLsForExecute(setup_exe.ToWStringHack().c_str()))
+    if (!FixDACLsForExecute(setup_path))
       NOTREACHED();
   }
 
@@ -244,18 +237,53 @@ bool RelaunchSetupAsConsoleUser(const std::string& flag) {
     return false;
   // Note: Handle inheritance must be true in order for the child process to be
   // able to use the duplicated handle above (Google Update results).
-  bool launched = base::LaunchAppAsUser(user_token,
-                                        cmd_line.command_line_string(),
-                                        false, NULL, true, true);
+  base::LaunchOptions options;
+  options.as_user = user_token;
+  options.inherit_handles = true;
+  bool launched = base::LaunchProcess(cmd_line, options, NULL);
   ::CloseHandle(user_token);
   return launched;
 }
 
+// The plugin infobar experiment is just setting the client registry value
+// to one of four possible values from 10% of the elegible population, which
+// is defined as active users that have opted-in for sending stats.
+// Chrome reads this value and modifies the plugin blocking and infobar
+// behavior accordingly.
+bool DoInfobarPluginsExperiment(int dir_age_hours) {
+  std::wstring client;
+  if (!GoogleUpdateSettings::GetClient(&client))
+    return false;
+  // Make sure the user is not already in this experiment.
+  if ((client.size() > 3) && (client[0] == L'P') && (client[1] == L'I'))
+    return false;
+  if (!GoogleUpdateSettings::GetCollectStatsConsent())
+    return false;
+  if (dir_age_hours > (24 * 14))
+    return false;
+  if (base::RandInt(0, 9)) {
+    GoogleUpdateSettings::SetClient(
+        attrition_experiments::kNotInPluginExperiment);
+    return false;
+  }
+
+  const wchar_t* buckets[] = {
+    attrition_experiments::kPluginNoBlockNoOOD,
+    attrition_experiments::kPluginNoBlockDoOOD,
+    attrition_experiments::kPluginDoBlockNoOOD,
+    attrition_experiments::kPluginDoBlockDoOOD
+  };
+
+  size_t group = base::RandInt(0, arraysize(buckets)-1);
+  GoogleUpdateSettings::SetClient(buckets[group]);
+  VLOG(1) << "Plugin infobar experiment group: " << group;
+  return true;
+}
 }  // namespace
 
-GoogleChromeDistribution::GoogleChromeDistribution(
-    const installer::MasterPreferences& prefs)
-        : BrowserDistribution(prefs), product_guid_(kChromeGuid) {
+GoogleChromeDistribution::GoogleChromeDistribution()
+    : BrowserDistribution(CHROME_BROWSER),
+      product_guid_(kChromeGuid) {
 }
 
 // The functions below are not used by the 64-bit Windows binary -
@@ -339,14 +367,10 @@ void GoogleChromeDistribution::DoPostUninstallOperations(
   // need to escape the string before using it in a URL.
   const std::wstring kVersionParam = L"crversion";
   const std::wstring kOSParam = L"os";
-  std::wstring os_version = L"na";
-  OSVERSIONINFO version_info;
-  version_info.dwOSVersionInfoSize = sizeof(version_info);
-  if (GetVersionEx(&version_info)) {
-    os_version = StringPrintf(L"%d.%d.%d", version_info.dwMajorVersion,
-                                           version_info.dwMinorVersion,
-                                           version_info.dwBuildNumber);
-  }
+  base::win::OSInfo::VersionNumber version_number =
+      base::win::OSInfo::GetInstance()->version_number();
+  std::wstring os_version = base::StringPrintf(L"%d.%d.%d",
+      version_number.major, version_number.minor, version_number.build);
 
   FilePath iexplore;
   if (!PathService::Get(base::DIR_PROGRAM_FILES, &iexplore))
@@ -383,9 +407,10 @@ std::wstring GoogleChromeDistribution::GetAppGuid() {
 }
 
 std::wstring GoogleChromeDistribution::GetApplicationName() {
-  const std::wstring& product_name =
-      installer::GetLocalizedString(IDS_PRODUCT_NAME_BASE);
-  return product_name;
+  // I'd really like to return L ## PRODUCT_FULLNAME_STRING; but that's no good
+  // since it'd be "Chromium" in a non-Chrome build, which isn't at all what I
+  // want.  Sigh.
+  return L"Google Chrome";
 }
 
 std::wstring GoogleChromeDistribution::GetAlternateApplicationName() {
@@ -439,6 +464,10 @@ std::wstring GoogleChromeDistribution::GetStatsServerURL() {
   return L"https://clients4.google.com/firefox/metrics/collect";
 }
 
+std::string GoogleChromeDistribution::GetNetworkStatsServer() const {
+  return "chrome.googleechotest.com";
+}
+
 std::wstring GoogleChromeDistribution::GetDistributionData(HKEY root_key) {
   std::wstring sub_key(google_update::kRegPathClientState);
   sub_key.append(L"\\");
@@ -502,11 +531,11 @@ std::wstring GoogleChromeDistribution::GetVersionKey() {
 // string (if it is present) regardless of whether installer failed or not.
 // There is no fall-back for full installer :)
 void GoogleChromeDistribution::UpdateInstallStatus(bool system_install,
-    bool incremental_install, bool multi_install,
+    installer::ArchiveType archive_type,
     installer::InstallStatus install_status) {
   GoogleUpdateSettings::UpdateInstallStatus(system_install,
-      incremental_install, multi_install,
-      InstallUtil::GetInstallReturnCode(install_status), product_guid());
+      archive_type, InstallUtil::GetInstallReturnCode(install_status),
+      product_guid());
 }
 
 // The functions below are not used by the 64-bit Windows binary -
@@ -516,7 +545,7 @@ void GoogleChromeDistribution::UpdateInstallStatus(bool system_install,
 // command line, but HKCU otherwise. |experiment_group| is the value to write
 // and |last_write| is used when writing to HKLM to determine whether to close
 // the handle when done.
-void SetClient(std::wstring experiment_group, bool last_write) {
+void SetClient(const std::wstring& experiment_group, bool last_write) {
   static int reg_key_handle = -1;
   if (reg_key_handle == -1) {
     // If a specific Toast Results key handle (presumably to our HKLM key) was
@@ -545,8 +574,104 @@ void SetClient(std::wstring experiment_group, bool last_write) {
   }
 }
 
-// Currently we only have one experiment: the inactive user toast. Which only
-// applies for users doing upgrades.
+bool GoogleChromeDistribution::GetExperimentDetails(
+    UserExperiment* experiment, int flavor) {
+  // Maximum number of experiment flavors we support.
+  const int kMax = 4;
+  // This struct determines which experiment flavors we show for each locale and
+  // brand.
+  //
+  // The big experiment in Dec 2009 used TGxx and THxx.
+  // The big experiment in Feb 2010 used TKxx and TLxx.
+  // The big experiment in Apr 2010 used TMxx and TNxx.
+  // The big experiment in Oct 2010 used TVxx TWxx TXxx TYxx.
+  // The big experiment in Feb 2011 used SJxx SKxx SLxx SMxx.
+  // Note: the plugin infobar experiment uses PIxx codes.
+  using namespace attrition_experiments;
+  static const struct UserExperimentDetails {
+    const wchar_t* locale;  // Locale to show this experiment for (* for all).
+    const wchar_t* brands;  // Brand codes show this experiment for (* for all).
+    int control_group;      // Size of the control group, in percentages.
+    const wchar_t prefix1;  // The first letter for the experiment code.
+    const wchar_t prefix2;  // The second letter for the experiment code. This
+                            // will be incremented by one for each additional
+                            // experiment flavor beyond the first.
+    int flavors;            // Numbers of flavors for this experiment. Should
+                            // always be positive and never exceed the number
+                            // of headings (below).
+    int headings[kMax];     // A list of IDs per experiment. 0 == no heading.
+  } kExperimentFlavors[] = {
+    // This list should be ordered most-specific rule first (catch-all, like all
+    // brands or all locales should be last).
+
+    // The experiment with the more compact bubble. This one is a bit special
+    // because it is split into two: CAxx is regular style bubble and CBxx is
+    // compact style bubble. See |compact_bubble| below.
+    {L"en-US", kBrief, 1, L'C', L'A', 2, { kEnUs3, kEnUs3, 0, 0 } },
+
+    // Catch-all rules.
+    {kAll, kAll, 1, L'B', L'A', 1, {kEnUs3, 0, 0, 0} },
+  };
+
+  std::wstring locale;
+  std::wstring brand;
+
+  if (!GoogleUpdateSettings::GetLanguage(&locale))
+    locale = ASCIIToWide("en-US");
+  if (!GoogleUpdateSettings::GetBrand(&brand))
+    brand = ASCIIToWide("");  // Could still be viable for catch-all rules.
+  if (brand == kEnterprise)
+    return false;
+
+  for (int i = 0; i < arraysize(kExperimentFlavors); ++i) {
+    // A maximum of four flavors are supported at the moment.
+    DCHECK_LE(kExperimentFlavors[i].flavors, kMax);
+    DCHECK_GT(kExperimentFlavors[i].flavors, 0);
+    // Make sure each experiment has valid headings.
+    for (int f = 0; f < kMax; ++f) {
+      if (f < kExperimentFlavors[i].flavors) {
+        DCHECK_GT(kExperimentFlavors[i].headings[f], 0);
+      } else {
+        DCHECK_EQ(kExperimentFlavors[i].headings[f], 0);
+      }
+    }
+    // Make sure we don't overflow on the second letter of the experiment code.
+    DCHECK(kExperimentFlavors[i].prefix2 +
+           kExperimentFlavors[i].flavors - 1 <= 'Z');
+
+    if (kExperimentFlavors[i].locale != locale &&
+        kExperimentFlavors[i].locale != ASCIIToWide("*"))
+      continue;
+
+    std::vector<std::wstring> brand_codes;
+    base::SplitString(kExperimentFlavors[i].brands, L',', &brand_codes);
+    if (brand_codes.empty())
+      return false;
+    for (std::vector<std::wstring>::iterator it = brand_codes.begin();
+         it != brand_codes.end(); ++it) {
+      if (*it != brand && *it != L"*")
+        continue;
+
+      // We have found our match.
+      if (flavor < 0)
+        flavor = base::RandInt(0, kExperimentFlavors[i].flavors - 1);
+      experiment->flavor = flavor;
+      experiment->heading = kExperimentFlavors[i].headings[flavor];
+      experiment->control_group = kExperimentFlavors[i].control_group;
+      experiment->prefix.resize(2);
+      experiment->prefix[0] = kExperimentFlavors[i].prefix1;
+      experiment->prefix[1] = kExperimentFlavors[i].prefix2 + flavor;
+      experiment->compact_bubble = (brand == kBrief) && (flavor == 1);
+      return true;
+    }
+  }
+
+  return false;
+}
+
+// Currently we have two experiments: 1) The inactive user toast. Which only
+// applies to users doing upgrades, and 2) The plugin infobar experiment
+// which only applies for active users.
 //
 // There are three scenarios when this function is called:
 // 1- Is a per-user-install and it updated: perform the experiment
@@ -554,12 +679,14 @@ void SetClient(std::wstring experiment_group, bool last_write) {
 // 3- It has been re-launched from the #2 case. In this case we enter
 //    this function with |system_install| true and a REENTRY_SYS_UPDATE status.
 void GoogleChromeDistribution::LaunchUserExperiment(
-    installer::InstallStatus status, const Version& version,
-    const installer::Product& installation, bool system_level) {
+    const FilePath& setup_path, installer::InstallStatus status,
+    const Version& version, const installer::Product& installation,
+    bool system_level) {
   if (system_level) {
     if (installer::NEW_VERSION_UPDATED == status) {
       // We need to relaunch as the interactive user.
-      RelaunchSetupAsConsoleUser(installer::switches::kSystemLevelToast);
+      LaunchSetupAsConsoleUser(setup_path,
+                               installer::switches::kSystemLevelToast);
       return;
     }
   } else {
@@ -570,19 +697,15 @@ void GoogleChromeDistribution::LaunchUserExperiment(
     }
   }
 
-  // This ends up being processed by ShowTryChromeDialog to show different
-  // experiments.  Only run the experiment in en-US and ja.
-  int flavor = 0;
-  std::wstring language;
-  if (GoogleUpdateSettings::GetLanguage(&language)) {
-    if (language == L"en-US") {
-      // en-US has four different toasts.
-      flavor = base::RandInt(0, 3);
-    } else if (language == L"ja") {
-      // ja has three different toasts.
-      flavor = base::RandInt(0, 2);
-    }
+  // The |flavor| value ends up being processed by TryChromeDialogView to show
+  // different experiments.
+  UserExperiment experiment;
+  if (!GetExperimentDetails(&experiment, -1)) {
+    VLOG(1) << "Failed to get experiment details.";
+    return;
   }
+  int flavor = experiment.flavor;
+  std::wstring base_group = experiment.prefix;
 
   std::wstring brand;
   if (GoogleUpdateSettings::GetBrand(&brand) && (brand == L"CHXX")) {
@@ -593,47 +716,57 @@ void GoogleChromeDistribution::LaunchUserExperiment(
     // chrome user data directory.
     FilePath user_data_dir(installation.GetUserDataPath());
 
-    const bool experiment_enabled = false;
+    const bool toast_experiment_enabled = false;
     const int kThirtyDays = 30 * 24;
 
     int dir_age_hours = GetDirectoryWriteAgeInHours(
         user_data_dir.value().c_str());
-    if (!experiment_enabled) {
-      VLOG(1) << "Toast experiment is disabled.";
+    if (!toast_experiment_enabled) {
+      // Ok, no toast, but what about the plugin infobar experiment?
+      if (!DoInfobarPluginsExperiment(dir_age_hours)) {
+        VLOG(1) << "No infobar experiment";
+      }
       return;
     } else if (dir_age_hours < 0) {
       // This means that we failed to find the user data dir. The most likely
       // cause is that this user has not ever used chrome at all which can
       // happen in a system-level install.
-      SetClient(GetExperimentGroup(kToastUDDirFailure, flavor), true);
+      SetClient(base_group + kToastUDDirFailure, true);
       return;
     } else if (dir_age_hours < kThirtyDays) {
-      // An active user, so it does not qualify.
       VLOG(1) << "Chrome used in last " << dir_age_hours << " hours";
-      SetClient(GetExperimentGroup(kToastActiveGroup, flavor), true);
+      SetClient(base_group + kToastActiveGroup, true);
       return;
     }
-    // 1% are in the control group that qualifies but does not get drafted.
-    if (base::RandDouble() > 0.99) {
-      SetClient(GetExperimentGroup(kToastExpControlGroup, flavor), true);
+    // Check to see if this user belongs to the control group.
+    double control_group = 1.0 * (100 - experiment.control_group) / 100;
+    if (base::RandDouble() > control_group) {
+      SetClient(base_group + kToastExpControlGroup, true);
       VLOG(1) << "User is control group";
       return;
     }
   }
 
   VLOG(1) << "User drafted for toast experiment " << flavor;
-  SetClient(GetExperimentGroup(kToastExpBaseGroup, flavor), false);
+  SetClient(base_group + kToastExpBaseGroup, false);
   // User level: The experiment needs to be performed in a different process
   // because google_update expects the upgrade process to be quick and nimble.
   // System level: We have already been relaunched, so we don't need to be
   // quick, but we relaunch to follow the exact same codepath.
-  RelaunchSetup(installer::switches::kInactiveUserToast, flavor,
-                system_level);
+  CommandLine cmd_line(setup_path);
+  cmd_line.AppendSwitchASCII(installer::switches::kInactiveUserToast,
+                             base::IntToString(flavor));
+  cmd_line.AppendSwitchASCII(installer::switches::kExperimentGroup,
+                             WideToASCII(base_group));
+  LaunchSetup(cmd_line, system_level);
 }
 
-// User qualifies for the experiment. Launch chrome with --try-chrome=flavor.
+// User qualifies for the experiment. To test, use --try-chrome-again=|flavor|
+// as a parameter to chrome.exe.
 void GoogleChromeDistribution::InactiveUserToastExperiment(int flavor,
-    const installer::Product& installation) {
+    const std::wstring& experiment_group,
+    const installer::Product& installation,
+    const FilePath& application_path) {
   bool has_welcome_url = (flavor == 0);
   // Possibly add a url to launch depending on the experiment flavor.
   CommandLine options(CommandLine::NO_PROGRAM);
@@ -647,23 +780,23 @@ void GoogleChromeDistribution::InactiveUserToastExperiment(int flavor,
     // The command line should now have the url added as:
     // "chrome.exe -- <url>"
     DCHECK_NE(std::wstring::npos,
-        options.command_line_string().find(L" -- " + url));
+        options.GetCommandLineString().find(L" -- " + url));
   }
   // Launch chrome now. It will show the toast UI.
   int32 exit_code = 0;
-  if (!installation.LaunchChromeAndWait(options, &exit_code))
+  if (!installation.LaunchChromeAndWait(application_path, options, &exit_code))
     return;
 
   // The chrome process has exited, figure out what happened.
   const wchar_t* outcome = NULL;
   switch (exit_code) {
-    case ResultCodes::NORMAL_EXIT:
+    case content::RESULT_CODE_NORMAL_EXIT:
       outcome = kToastExpTriesOkGroup;
       break;
-    case ResultCodes::NORMAL_EXIT_CANCEL:
+    case chrome::RESULT_CODE_NORMAL_EXIT_CANCEL:
       outcome = kToastExpCancelGroup;
       break;
-    case ResultCodes::NORMAL_EXIT_EXP2:
+    case chrome::RESULT_CODE_NORMAL_EXIT_EXP2:
       outcome = kToastExpUninstallGroup;
       break;
     default:
@@ -671,7 +804,7 @@ void GoogleChromeDistribution::InactiveUserToastExperiment(int flavor,
   };
 
   // Write to the |client| key for the last time.
-  SetClient(GetExperimentGroup(outcome, flavor), true);
+  SetClient(experiment_group + outcome, true);
 
   if (outcome != kToastExpUninstallGroup)
     return;
@@ -682,16 +815,8 @@ void GoogleChromeDistribution::InactiveUserToastExperiment(int flavor,
   bool system_level_toast = CommandLine::ForCurrentProcess()->HasSwitch(
       installer::switches::kSystemLevelToast);
 
-  std::wstring cmd(InstallUtil::GetChromeUninstallCmd(
-      system_level_toast, this));
-
-  base::LaunchApp(cmd, false, false, NULL);
+  CommandLine cmd(InstallUtil::GetChromeUninstallCmd(system_level_toast,
+                                                     GetType()));
+  base::LaunchProcess(cmd, base::LaunchOptions(), NULL);
 }
 #endif
-
-bool GoogleChromeDistribution::SetChannelFlags(
-    bool set,
-    installer::ChannelInfo* channel_info) {
-  DCHECK(channel_info);
-  return channel_info->SetChrome(set);
-}
