@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,17 +9,31 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/string16.h"
+#include "chrome/installer/util/util_constants.h"
+
+class BrowserDistribution;
 
 namespace installer {
 class ChannelInfo;
-class Package;
+class InstallationState;
 }
 
 // This class provides accessors to the Google Update 'ClientState' information
 // that recorded when the user downloads the chrome installer. It is
-// google_update.exe responsability to write the initial values.
+// google_update.exe responsibility to write the initial values.
 class GoogleUpdateSettings {
  public:
+  // Update policy constants defined by Google Update; do not change these.
+  enum UpdatePolicy {
+    UPDATES_DISABLED    = 0,
+    AUTOMATIC_UPDATES   = 1,
+    MANUAL_UPDATES_ONLY = 2,
+  };
+
+  // Returns true if this install is system-wide, false if it is per-user.
+  static bool IsSystemInstall();
+
   // Returns whether the user has given consent to collect UMA data and send
   // crash dumps to Google. This information is collected by the web server
   // used to download the chrome installer.
@@ -29,6 +43,18 @@ class GoogleUpdateSettings {
   // false if the setting could not be recorded.
   static bool SetCollectStatsConsent(bool consented);
 
+#if defined(OS_WIN)
+  // Returns whether the user has given consent to collect UMA data and send
+  // crash dumps to Google. This information is collected by the web server
+  // used to download the chrome installer.
+  static bool GetCollectStatsConsentAtLevel(bool system_install);
+
+  // Sets the user consent to send UMA and crash dumps to Google. Returns
+  // false if the setting could not be recorded.
+  static bool SetCollectStatsConsentAtLevel(bool system_install,
+                                            bool consented);
+#endif
+
   // Returns the metrics id set in the registry (that can be used in crash
   // reports). If none found, returns empty string.
   static bool GetMetricsId(std::wstring* metrics_id);
@@ -38,7 +64,8 @@ class GoogleUpdateSettings {
 
   // Sets the machine-wide EULA consented flag required on OEM installs.
   // Returns false if the setting could not be recorded.
-  static bool SetEULAConsent(const installer::Package& package,
+  static bool SetEULAConsent(const installer::InstallationState& machine_state,
+                             BrowserDistribution* dist,
                              bool consented);
 
   // Returns the last time chrome was run in days. It uses a recorded value
@@ -67,8 +94,20 @@ class GoogleUpdateSettings {
 
   // Returns in |brand| the RLZ brand code or distribution tag that has been
   // assigned to a partner. Returns false if the information is not available.
+  //
+  // NOTE: This function is Windows only.  If the code you are writing is not
+  // specifically for Windows, prefer calling google_util::GetBrand().
   static bool GetBrand(std::wstring* brand);
 
+  // Returns in |brand| the RLZ reactivation brand code or distribution tag
+  // that has been assigned to a partner for reactivating a dormant chrome
+  // install. Returns false if the information is not available.
+  //
+  // NOTE: This function is Windows only.  If the code you are writing is not
+  // specifically for Windows, prefer calling
+  // google_util::GetReactivationBrand().
+  static bool GetReactivationBrand(std::wstring* brand);
+
   // Returns in |client| the google_update client field, which is currently
   // used to track experiments. Returns false if the entry does not exist.
   static bool GetClient(std::wstring* client);
@@ -85,10 +124,22 @@ class GoogleUpdateSettings {
   // true if this operation succeeded.
   static bool ClearReferral();
 
+  // Set did_run "dr" in the client state value. This is used to measure
+  // active users. Returns false if writting to the registry failed.
+  static bool UpdateDidRunState(bool did_run, bool system_level);
+
+  // Returns only the channel name: "" (stable), "dev", "beta", "canary", or
+  // "unknown" if unknown. This value will not be modified by "-m" for a
+  // multi-install. See kChromeChannel* in util_constants.h
+  static std::wstring GetChromeChannel(bool system_install);
+
   // Return a human readable modifier for the version string, e.g.
   // the channel (dev, beta, stable). Returns true if this operation succeeded,
-  // on success, channel contains one of "", "unknown", "dev" or "beta".
-  static bool GetChromeChannel(bool system_install, std::wstring* channel);
+  // on success, channel contains one of "", "unknown", "dev" or "beta" (unless
+  // it is a multi-install product, in which case it will return "m",
+  // "unknown-m", "dev-m", or "beta-m").
+  static bool GetChromeChannelAndModifiers(bool system_install,
+                                           string16* channel);
 
   // This method changes the Google Update "ap" value to move the installation
   // on to or off of one of the recovery channels.
@@ -98,12 +149,10 @@ class GoogleUpdateSettings {
   // - If we are currently running full installer, we remove this magic
   // string (if it is present) regardless of whether installer failed or not.
   // There is no fall-back for full installer :)
-  // - If multi-install fails we append -multifail; otherwise, we remove it
-  // (i.e., success or single-install).
+  // - Unconditionally remove "-multifail" since we haven't crashed.
   // |state_key| should be obtained via InstallerState::state_key().
   static void UpdateInstallStatus(bool system_install,
-                                  bool incremental_install,
-                                  bool multi_install,
+                                  installer::ArchiveType archive_type,
                                   int install_return_code,
                                   const std::wstring& product_guid);
 
@@ -116,12 +165,11 @@ class GoogleUpdateSettings {
   // - If full installer failed, still remove this magic
   //   string (if it is present already).
   //
-  // diff_install: tells whether this is incremental install or not.
+  // archive_type: tells whether this is incremental install or not.
   // install_return_code: if 0, means installation was successful.
   // value: current value of Google Update "ap" key.
   // Returns true if |value| is modified.
-  static bool UpdateGoogleUpdateApKey(bool diff_install,
-                                      bool multi_install,
+  static bool UpdateGoogleUpdateApKey(installer::ArchiveType archive_type,
                                       int install_return_code,
                                       installer::ChannelInfo* value);
 
@@ -142,8 +190,11 @@ class GoogleUpdateSettings {
                                                const std::wstring& key,
                                                const std::wstring& value);
 
-  // True if a build is strictly organic, according to its brand code.
-  static bool IsOrganic(const std::wstring& brand);
+  // Returns the effective update policy for |app_guid| as dictated by
+  // Group Policy settings.  |is_overridden|, if non-NULL, is populated with
+  // true if an app-specific policy override is in force, or false otherwise.
+  static UpdatePolicy GetAppUpdatePolicy(const std::wstring& app_guid,
+                                         bool* is_overridden);
 
  private:
   DISALLOW_IMPLICIT_CONSTRUCTORS(GoogleUpdateSettings);
