@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -16,7 +16,7 @@
 
 #include <string>
 
-namespace installer_util {
+namespace installer {
 
 // Given a string base id, return the localized version of the string based on
 // the system language.  This is used for shortcuts placed on the user's
@@ -27,6 +27,9 @@ std::wstring GetLocalizedString(int base_message_id);
 // The empty string is returned on failure.
 std::wstring GetLocalizedEulaResource();
 
-}  // namespace installer_util.
+// Returns the language identifier of the translation currently in use.
+std::wstring GetCurrentTranslation();
+
+}  // namespace installer.
 
 #endif  // CHROME_INSTALLER_UTIL_L10N_STRING_UTIL_H_
