@@ -1,15 +1,16 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/installer/util/master_preferences.h"
 
+#include "base/environment.h"
 #include "base/file_util.h"
+#include "base/json/json_value_serializer.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/path_service.h"
 #include "base/string_util.h"
-#include "chrome/common/json_value_serializer.h"
 #include "chrome/installer/util/master_preferences_constants.h"
 #include "chrome/installer/util/util_constants.h"
 #include "googleurl/src/gurl.h"
@@ -19,8 +20,8 @@ namespace {
 const char kDistroDict[] = "distribution";
 const char kFirstRunTabs[] = "first_run_tabs";
 
-base::LazyInstance<installer::MasterPreferences> g_master_preferences(
-    base::LINKER_INITIALIZED);
+base::LazyInstance<installer::MasterPreferences> g_master_preferences =
+    LAZY_INSTANCE_INITIALIZER;
 
 bool GetGURLFromValue(const Value* in_value, GURL* out_value) {
   if (!in_value || !out_value)
@@ -79,7 +80,6 @@ namespace installer {
 
 MasterPreferences::MasterPreferences() : distribution_(NULL),
                                          preferences_read_from_file_(false),
-                                         ceee_(false),
                                          chrome_(true),
                                          chrome_frame_(false),
                                          multi_install_(false) {
@@ -89,7 +89,6 @@ MasterPreferences::MasterPreferences() : distribution_(NULL),
 MasterPreferences::MasterPreferences(const CommandLine& cmd_line)
     : distribution_(NULL),
       preferences_read_from_file_(false),
-      ceee_(false),
       chrome_(true),
       chrome_frame_(false),
       multi_install_(false) {
@@ -97,7 +96,7 @@ MasterPreferences::MasterPreferences(const CommandLine& cmd_line)
 }
 
 MasterPreferences::MasterPreferences(const FilePath& prefs_path)
-    : distribution_(NULL), preferences_read_from_file_(false), ceee_(false),
+    : distribution_(NULL), preferences_read_from_file_(false),
       chrome_(true), chrome_frame_(false), multi_install_(false) {
   master_dictionary_.reset(ParseDistributionPreferences(prefs_path));
 
@@ -134,8 +133,8 @@ void MasterPreferences::InitializeFromCommandLine(const CommandLine& cmd_line) {
     const char* cmd_line_switch;
     const char* distribution_switch;
   } translate_switches[] = {
-    { installer::switches::kCeee,
-      installer::master_preferences::kCeee },
+    { installer::switches::kAutoLaunchChrome,
+      installer::master_preferences::kAutoLaunchChrome },
     { installer::switches::kChrome,
       installer::master_preferences::kChrome },
     { installer::switches::kChromeFrame,
@@ -184,6 +183,20 @@ void MasterPreferences::InitializeFromCommandLine(const CommandLine& cmd_line) {
     master_dictionary_->SetString(name, str_value);
   }
 
+  // Handle the special case of --system-level being implied by the presence of
+  // the kGoogleUpdateIsMachineEnvVar environment variable.
+  scoped_ptr<base::Environment> env(base::Environment::Create());
+  if (env != NULL) {
+    std::string is_machine_var;
+    env->GetVar(kGoogleUpdateIsMachineEnvVar, &is_machine_var);
+    if (!is_machine_var.empty() && is_machine_var[0] == '1') {
+      VLOG(1) << "Taking system-level from environment.";
+      name.resize(arraysize(kDistroDict) - 1);
+      name.append(".").append(installer::master_preferences::kSystemLevel);
+      master_dictionary_->SetBoolean(name, true);
+    }
+  }
+
   // Cache a pointer to the distribution dictionary. Ignore errors if any.
   master_dictionary_->GetDictionary(kDistroDict, &distribution_);
 
@@ -195,12 +208,10 @@ void MasterPreferences::InitializeProductFlags() {
   // Make sure we start out with the correct defaults.
   multi_install_ = false;
   chrome_frame_ = false;
-  ceee_ = false;
   chrome_ = true;
 
   GetBool(installer::master_preferences::kMultiInstall, &multi_install_);
   GetBool(installer::master_preferences::kChromeFrame, &chrome_frame_);
-  GetBool(installer::master_preferences::kCeee, &ceee_);
 
   // When multi-install is specified, the checks are pretty simple (in theory):
   // In order to be installed/uninstalled, each product must have its switch
@@ -209,14 +220,7 @@ void MasterPreferences::InitializeProductFlags() {
   // two products, Chrome and Chrome Frame.  For the time being we need to
   // continue to support this mode where multi-install is not set.
   // So, when multi-install is not set, we continue to support mutually
-  // exclusive installation of Chrome and Chrome Frame in addition to supporting
-  // installation of CEEE with Chrome Frame.
-
-  // Regardless of multi install being present, CEEE always needs CF to
-  // be installed.
-  if (ceee_)
-    chrome_frame_ = true;
-
+  // exclusive installation of Chrome and Chrome Frame.
   if (multi_install_) {
     if (!GetBool(installer::master_preferences::kChrome, &chrome_))
       chrome_ = false;
