@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -13,17 +13,22 @@
 #include <vector>
 
 #include "base/command_line.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "googleurl/src/gurl.h"
 
-class DictionaryValue;
 class FilePath;
 
+namespace base {
+class DictionaryValue;
+}
+
 namespace installer {
 
+#if !defined(OS_MACOSX)
 // This is the default name for the master preferences file used to pre-set
 // values in the user profile at first run.
 const char kDefaultMasterPrefs[] = "master_preferences";
+#endif
 
 // The master preferences is a JSON file with the same entries as the
 // 'Default\Preferences' file. This function parses the distribution
@@ -34,7 +39,7 @@ const char kDefaultMasterPrefs[] = "master_preferences";
 // {
 //   "distribution": {
 //      "alternate_shortcut_text": false,
-//      "oem_bubble": false,
+//      "auto_launch_chrome": false,
 //      "chrome_shortcut_icon_index": 0,
 //      "create_all_shortcuts": true,
 //      "import_bookmarks": false,
@@ -149,17 +154,13 @@ class MasterPreferences {
   //     }
   //  }
   //
-  bool GetExtensionsBlock(DictionaryValue** extensions) const;
+  bool GetExtensionsBlock(base::DictionaryValue** extensions) const;
 
   // Returns true iff the master preferences were successfully read from a file.
   bool read_from_file() const {
     return preferences_read_from_file_;
   }
 
-  bool install_ceee() const {
-    return ceee_;
-  }
-
   bool install_chrome() const {
     return chrome_;
   }
@@ -184,10 +185,9 @@ class MasterPreferences {
   void InitializeFromCommandLine(const CommandLine& cmd_line);
 
  protected:
-  scoped_ptr<DictionaryValue> master_dictionary_;
-  DictionaryValue* distribution_;
+  scoped_ptr<base::DictionaryValue> master_dictionary_;
+  base::DictionaryValue* distribution_;
   bool preferences_read_from_file_;
-  bool ceee_;
   bool chrome_;
   bool chrome_frame_;
   bool multi_install_;
