@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -18,6 +18,11 @@
 #include "chrome/installer/util/work_item_list.h"
 
 class BrowserDistribution;
+class FilePath;
+
+namespace base {
+class DictionaryValue;
+}
 
 // This is a utility class that provides common shell integration methods
 // that can be used by installer as well as Chrome.
@@ -29,6 +34,9 @@ class ShellUtil {
     SYSTEM_LEVEL = 0x2   // Make any shell changes only at the system level
   };
 
+  // Relative path of the URL Protocol registry entry (prefixed with '\').
+  static const wchar_t* kRegURLProtocol;
+
   // Relative path of DefaultIcon registry entry (prefixed with '\').
   static const wchar_t* kRegDefaultIcon;
 
@@ -68,8 +76,12 @@ class ShellUtil {
   // File extensions that Chrome registers itself for.
   static const wchar_t* kFileAssociations[];
 
-  // Protocols that Chrome registers itself for.
-  static const wchar_t* kProtocolAssociations[];
+  // Protocols that Chrome registers itself as the default handler for
+  // when the user makes Chrome the default browser.
+  static const wchar_t* kBrowserProtocolAssociations[];
+
+  // Protocols that Chrome registers itself as being capable of handling.
+  static const wchar_t* kPotentialProtocolAssociations[];
 
   // Registry value name that is needed for ChromeHTML ProgId
   static const wchar_t* kRegUrlProtocol;
@@ -79,18 +91,32 @@ class ShellUtil {
   static bool AdminNeededForRegistryCleanup(BrowserDistribution* dist,
                                             const std::wstring& suffix);
 
-  // Create Chrome shortcut on Desktop
-  // If shell_change is CURRENT_USER, the shortcut is created in the
-  // Desktop folder of current user's profile.
-  // If shell_change is SYSTEM_LEVEL, the shortcut is created in the
-  // Desktop folder of "All Users" profile.
-  // If alternate is true, an alternate text for the shortcut is used.
-  // create_new: If false, will only update the shortcut. If true, the function
-  //             will create a new shortcut if it doesn't exist already.
+  // Creates Chrome shortcut on the Desktop.
+  // |dist| gives the type of browser distribution currently in use.
+  // |chrome_exe| provides the target path information.
+  // |description| provides the shortcut's "comment" property.
+  // |appended_name| provides a string to be appended to the distribution name,
+  //     and can be the empty string.
+  // |arguments| gives a set of arguments to be passed to the executable.
+  // |icon_path| provides the path to the icon file to use.
+  // |icon_index| provides the index of the icon within the provided icon file.
+  // If |shell_change| is CURRENT_USER, the shortcut is created in the
+  //     Desktop folder of current user's profile.
+  // If |shell_change| is SYSTEM_LEVEL, the shortcut is created in the
+  //     Desktop folder of the "All Users" profile.
+  // If |alternate| is true, an alternate text for the shortcut is used.
+  // If |create_new| is false, an existing shortcut will be updated, but if
+  //     no shortcut exists, it will not be created.
+  // Returns true iff the method causes a shortcut to be created / updated.
   static bool CreateChromeDesktopShortcut(BrowserDistribution* dist,
                                           const std::wstring& chrome_exe,
                                           const std::wstring& description,
-                                          int shell_change, bool alternate,
+                                          const std::wstring& appended_name,
+                                          const std::wstring& arguments,
+                                          const std::wstring& icon_path,
+                                          int icon_index,
+                                          ShellChange shell_change,
+                                          bool alternate,
                                           bool create_new);
 
   // Create Chrome shortcut on Quick Launch Bar.
@@ -119,16 +145,19 @@ class ShellUtil {
   // chrome_exe: the full path to chrome.exe
   static std::wstring GetChromeShellOpenCmd(const std::wstring& chrome_exe);
 
-  // Returns the localized name of Chrome shortcut. If |alternate| is true
-  // it returns a second localized text that is better suited for certain
-  // scenarios.
+  // Returns the localized name of Chrome shortcut in |shortcut|. If
+  // |appended_name| is not empty, it is included in the shortcut name. If
+  // |alternate| is true, a second localized text that is better suited for
+  // certain scenarios is used.
   static bool GetChromeShortcutName(BrowserDistribution* dist,
-                                    std::wstring* shortcut, bool alternate);
+                                    bool alternate,
+                                    const std::wstring& appended_name,
+                                    std::wstring* shortcut);
 
   // Gets the desktop path for the current user or all users (if system_level
   // is true) and returns it in 'path' argument. Return true if successful,
   // otherwise returns false.
-  static bool GetDesktopPath(bool system_level, std::wstring* path);
+  static bool GetDesktopPath(bool system_level, FilePath* path);
 
   // Gets the Quick Launch shortcuts path for the current user and
   // returns it in 'path' argument. Return true if successful, otherwise
@@ -153,7 +182,18 @@ class ShellUtil {
   static bool GetUserSpecificDefaultBrowserSuffix(BrowserDistribution* dist,
                                                   std::wstring* entry);
 
-  // Make Chrome default browser.
+  // Make Chrome the default browser. This function works by going through
+  // the url protocols and file associations that are related to general
+  // browsing, e.g. http, https, .html etc., and requesting to become the
+  // default handler for each. If any of these fails the operation will return
+  // false to indicate failure, which is consistent with the return value of
+  // ShellIntegration::IsDefaultBrowser.
+  //
+  // In the case of failure any successful changes will be left, however no
+  // more changes will be attempted.
+  // TODO(benwells): Attempt to undo any changes that were successfully made.
+  // http://crbug.com/83970
+  //
   // shell_change: Defined whether to register as default browser at system
   //               level or user level. If value has ShellChange::SYSTEM_LEVEL
   //               we should be running as admin user.
@@ -165,6 +205,13 @@ class ShellUtil {
                                 const std::wstring& chrome_exe,
                                 bool elevate_if_not_admin);
 
+  // Make Chrome the default application for a protocol.
+  // chrome_exe: The chrome.exe path to register as default browser.
+  // protocol: The protocol to register as the default handler for.
+  static bool MakeChromeDefaultProtocolClient(BrowserDistribution* dist,
+                                              const std::wstring& chrome_exe,
+                                              const std::wstring& protocol);
+
   // This method adds Chrome to the list that shows up in Add/Remove Programs->
   // Set Program Access and Defaults and also creates Chrome ProgIds under
   // Software\Classes. This method requires write access to HKLM so is just
@@ -191,6 +238,29 @@ class ShellUtil {
                                     const std::wstring& unique_suffix,
                                     bool elevate_if_not_admin);
 
+  // This method declares to Windows that Chrome is capable of handling the
+  // given protocol. This function will call the RegisterChromeBrowser function
+  // to register with Windows as capable of handling the protocol, if it isn't
+  // currently registered as capable.
+  // Declaring the capability of handling a protocol is necessary to register
+  // as the default handler for the protocol in Vista and later versions of
+  // Windows.
+  //
+  // If called by the browser and elevation is required, it will elevate by
+  // calling setup.exe which will again call this function with elevate false.
+  //
+  // |chrome_exe| full path to chrome.exe.
+  // |unique_suffix| Optional input. If given, this function appends the value
+  // to default browser entries names that it creates in the registry.
+  // |protocol| The protocol to register as being capable of handling.s
+  // |elevate_if_not_admin| if true will make this method try alternate methods
+  // as described above.
+  static bool RegisterChromeForProtocol(BrowserDistribution* dist,
+                                        const std::wstring& chrome_exe,
+                                        const std::wstring& unique_suffix,
+                                        const std::wstring& protocol,
+                                        bool elevate_if_not_admin);
+
   // Remove Chrome shortcut from Desktop.
   // If shell_change is CURRENT_USER, the shortcut is removed from the
   // Desktop folder of current user's profile.
@@ -199,7 +269,14 @@ class ShellUtil {
   // If alternate is true, the shortcut with the alternate name is removed. See
   // CreateChromeDesktopShortcut() for more information.
   static bool RemoveChromeDesktopShortcut(BrowserDistribution* dist,
-                                          int shell_change, bool alternate);
+                                          int shell_change,
+                                          bool alternate);
+
+  // Removes a set of existing Chrome desktop shortcuts. |appended_names| is a
+  // list of shortcut file names as obtained from
+  // ShellUtil::GetChromeShortcutName.
+  static bool RemoveChromeDesktopShortcutsWithAppendedNames(
+      const std::vector<std::wstring>& appended_names);
 
   // Remove Chrome shortcut from Quick Launch Bar.
   // If shell_change is CURRENT_USER, the shortcut is removed from
@@ -210,14 +287,17 @@ class ShellUtil {
                                               int shell_change);
 
   // Updates shortcut (or creates a new shortcut) at destination given by
-  // shortcut to a target given by chrome_exe. The arguments is left NULL
-  // for the target and icon is set as icon at index 0 from exe.
-  // If create_new is set to true, the function will create a new shortcut if
-  // if doesn't exist.
+  // shortcut to a target given by chrome_exe. The arguments are given by
+  // |arguments| for the target and icon is set based on |icon_path| and
+  // |icon_index|. If create_new is set to true, the function will create a new
+  // shortcut if it doesn't exist.
   static bool UpdateChromeShortcut(BrowserDistribution* dist,
                                    const std::wstring& chrome_exe,
                                    const std::wstring& shortcut,
+                                   const std::wstring& arguments,
                                    const std::wstring& description,
+                                   const std::wstring& icon_path,
+                                   int icon_index,
                                    bool create_new);
 
  private:
