@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -9,6 +9,8 @@
 #define CHROME_INSTALLER_UTIL_UTIL_CONSTANTS_H_
 #pragma once
 
+#include "base/basictypes.h"
+
 namespace installer {
 
 // Return status of installer
@@ -33,7 +35,7 @@ enum InstallStatus {
   CHROME_RUNNING,         // 16. Chrome currently running (when trying to
                           // uninstall)
   UNINSTALL_CONFIRMED,    // 17. User has confirmed Chrome uninstall
-  UNINSTALL_DELETE_PROFILE, // 18. User confirmed uninstall and profile deletion
+  UNINSTALL_DELETE_PROFILE,  // 18. User okayed uninstall and profile deletion.
   UNINSTALL_SUCCESSFUL,   // 19. Chrome successfully uninstalled
   UNINSTALL_FAILED,       // 20. Chrome uninstallation failed
   UNINSTALL_CANCELLED,    // 21. User cancelled Chrome uninstallation
@@ -42,7 +44,7 @@ enum InstallStatus {
   RENAME_FAILED,          // 24. Rename of new_chrome.exe failed
   EULA_REJECTED,          // 25. EULA dialog was not accepted by user.
   EULA_ACCEPTED,          // 26. EULA dialog was accepted by user.
-  EULA_ACCEPTED_OPT_IN,   // 27. EULA accepted wtih the crash optin selected.
+  EULA_ACCEPTED_OPT_IN,   // 27. EULA accepted with the crash option selected.
   INSTALL_DIR_IN_USE,     // 28. Installation directory is in use by another
                           // process
   UNINSTALL_REQUIRES_REBOOT,  // 29. Uninstallation required a reboot.
@@ -66,18 +68,71 @@ enum InstallStatus {
                                     // Chrome Frame.
   READY_MODE_END_TEMP_OPT_OUT_FAILED,   // 38. Failed to end temporary opt-out
                                         // of Chrome Frame.
+  CONFLICTING_CHANNEL_EXISTS,  // 39. A multi-install product on a different
+                               // update channel exists.
+  READY_MODE_REQUIRES_CHROME,  // 40. Chrome Frame in ready-mode requires Chrome
+  REQUIRES_MULTI_INSTALL,      // 41. --multi-install was missing from the
+                               // command line.
+  APPLY_DIFF_PATCH_FAILED,     // 42. Failed to apply a diff patch.
+  INCONSISTENT_UPDATE_POLICY,  // 43. Inconsistent update policy GP settings.
 };
 
+
+// If the following compile assert fires it means that the InstallStatus
+// enumeration changed which will break the contract between the old
+// chrome installed and the new setup.exe that is trying to upgrade.
+COMPILE_ASSERT(installer::INCONSISTENT_UPDATE_POLICY == 43,
+               dont_change_enum);
+
+// The type of an update archive.
+enum ArchiveType {
+  UNKNOWN_ARCHIVE_TYPE,     // Unknown or uninitialized.
+  FULL_ARCHIVE_TYPE,        // Full chrome.7z archive.
+  INCREMENTAL_ARCHIVE_TYPE  // Incremental or differential archive.
+};
+
+// Stages of an installation reported through Google Update on failure.
+// The order and value of existing enums must not change. Please add new
+// values to the end (before NUM_STAGES) and update the compile assert below
+// to assert on the last value added.
+enum InstallerStage {
+  NO_STAGE,                    // 0: No stage to report.
+  PRECONDITIONS,               // 1: Evaluating pre-install conditions.
+  UNCOMPRESSING,               // 2: Uncompressing chrome.packed.7z.
+  ENSEMBLE_PATCHING,           // 3: Patching chrome.7z using courgette.
+  BINARY_PATCHING,             // 4: Patching chrome.7z using bspatch.
+  UNPACKING,                   // 5: Unpacking chrome.7z.
+  BUILDING,                    // 6: Building the install work item list.
+  EXECUTING,                   // 7: Executing the install work item list.
+  ROLLINGBACK,                 // 8: Rolling-back the install work item list.
+  REFRESHING_POLICY,           // 9: Refreshing the elevation policy.
+  UPDATING_CHANNELS,           // 10: Updating channel information.
+  COPYING_PREFERENCES_FILE,    // 11: Copying preferences file.
+  CREATING_SHORTCUTS,          // 12: Creating shortcuts.
+  REGISTERING_CHROME,          // 13: Performing Chrome registration.
+  REMOVING_OLD_VERSIONS,       // 14: Deleting old version directories.
+  FINISHING,                   // 15: Finishing the install.
+  CONFIGURE_AUTO_LAUNCH,       // 16: Configuring Chrome to auto-launch.
+  NUM_STAGES                   // 17: The number of stages.
+};
+
+// When we start reporting the numerical values from the enum, the order
+// above MUST be preserved.
+COMPILE_ASSERT(CONFIGURE_AUTO_LAUNCH == 16,
+               never_ever_ever_change_InstallerStage_values_bang);
+
 namespace switches {
-extern const char kCeee[];
+extern const char kAutoLaunchChrome[];
 extern const char kChrome[];
 extern const char kChromeFrame[];
+extern const char kChromeFrameQuickEnable[];
 extern const char kChromeFrameReadyMode[];
 extern const char kChromeFrameReadyModeOptIn[];
 extern const char kChromeFrameReadyModeTempOptOut[];
 extern const char kChromeFrameReadyModeEndTempOptOut[];
 extern const char kChromeSxS[];
 extern const char kCreateAllShortcuts[];
+extern const char kCriticalUpdateVersion[];
 extern const char kDeleteProfile[];
 extern const char kDisableLogging[];
 extern const char kDoNotCreateShortcuts[];
@@ -95,6 +150,7 @@ extern const char kMultiInstall[];
 extern const char kNewSetupExe[];
 extern const char kRegisterChromeBrowser[];
 extern const char kRegisterChromeBrowserSuffix[];
+extern const char kRegisterURLProtocol[];
 extern const char kRenameChromeExe[];
 extern const char kRemoveChromeRegistration[];
 extern const char kRunAsAdmin[];
@@ -106,25 +162,25 @@ extern const char kShowEula[];
 extern const char kAltDesktopShortcut[];
 extern const char kInactiveUserToast[];
 extern const char kSystemLevelToast[];
+extern const char kExperimentGroup[];
 extern const char kToastResultsKey[];
 }  // namespace switches
 
-extern const wchar_t kCeeeBrokerExe[];
-extern const wchar_t kCeeeIeDll[];
-extern const wchar_t kCeeeInstallHelperDll[];
 extern const wchar_t kChromeDll[];
 extern const wchar_t kChromeExe[];
 extern const wchar_t kChromeFrameDll[];
 extern const wchar_t kChromeFrameHelperExe[];
 extern const wchar_t kChromeFrameHelperWndClass[];
 extern const wchar_t kChromeFrameReadyModeField[];
-extern const wchar_t kChromeNaCl64Dll[];
+extern const wchar_t kChromeLauncherExe[];
 extern const wchar_t kChromeOldExe[];
 extern const wchar_t kChromeNewExe[];
+extern const wchar_t kCmdQuickEnableCf[];
 extern const wchar_t kGoogleChromeInstallSubDir1[];
 extern const wchar_t kGoogleChromeInstallSubDir2[];
 extern const wchar_t kInstallBinaryDir[];
 extern const wchar_t kInstallerDir[];
+extern const wchar_t kInstallTempDir[];
 extern const wchar_t kInstallUserDataDir[];
 extern const wchar_t kNaClExe[];
 extern const wchar_t kSetupExe[];
@@ -135,12 +191,30 @@ extern const wchar_t kUninstallInstallationDate[];
 extern const char kUninstallMetricsName[];
 extern const wchar_t kUninstallStringField[];
 
-// Used by InstallUtil::WriteInstallerResult.
-extern const wchar_t kInstallerResult[];
+// Google Update installer result API
 extern const wchar_t kInstallerError[];
+extern const wchar_t kInstallerExtraCode1[];
+extern const wchar_t kInstallerResult[];
 extern const wchar_t kInstallerResultUIString[];
 extern const wchar_t kInstallerSuccessLaunchCmdLine[];
 
+// Google Update named environment variable that implies kSystemLevel.
+extern const char kGoogleUpdateIsMachineEnvVar[];
+
+// Product options.
+extern const wchar_t kOptionMultiInstall[];
+extern const wchar_t kOptionReadyMode[];
+
+// Chrome channel display names.
+// NOTE: Canary is not strictly a 'channel', but rather a separate product
+//     installed side-by-side. However, GoogleUpdateSettings::GetChromeChannel
+//     will return "canary" for that product.
+extern const wchar_t kChromeChannelUnknown[];
+extern const wchar_t kChromeChannelCanary[];
+extern const wchar_t kChromeChannelDev[];
+extern const wchar_t kChromeChannelBeta[];
+extern const wchar_t kChromeChannelStable[];
+
 }  // namespace installer
 
 #endif  // CHROME_INSTALLER_UTIL_UTIL_CONSTANTS_H_
