@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,11 +9,13 @@
 #include "base/file_path.h"
 #include "base/logging.h"
 #include "base/native_library.h"
+#include "chrome/common/chrome_sandbox_type_mac.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/sandbox_mac.h"
+#include "content/common/sandbox_mac.h"
+#include "content/public/common/sandbox_init.h"
 
 NaClMainPlatformDelegate::NaClMainPlatformDelegate(
-    const MainFunctionParams& parameters)
+    const content::MainFunctionParams& parameters)
     : parameters_(parameters), sandbox_test_module_(NULL) {
 }
 
@@ -32,7 +34,7 @@ void NaClMainPlatformDelegate::PlatformUninitialize() {
 }
 
 void NaClMainPlatformDelegate::InitSandboxTests(bool no_sandbox) {
-  const CommandLine& command_line = parameters_.command_line_;
+  const CommandLine& command_line = parameters_.command_line;
 
   DVLOG(1) << "Started NaClLdr with ";
   const std::vector<std::string>& argstrings = command_line.argv();
@@ -47,20 +49,15 @@ void NaClMainPlatformDelegate::InitSandboxTests(bool no_sandbox) {
     FilePath test_dll_name =
       command_line.GetSwitchValuePath(switches::kTestNaClSandbox);
     if (!test_dll_name.empty()) {
-      sandbox_test_module_ = base::LoadNativeLibrary(test_dll_name);
+      sandbox_test_module_ = base::LoadNativeLibrary(test_dll_name, NULL);
       CHECK(sandbox_test_module_);
     }
   }
 }
 
 void NaClMainPlatformDelegate::EnableSandbox() {
-  CommandLine* parsed_command_line = CommandLine::ForCurrentProcess();
-  SandboxInitWrapper sandbox_wrapper;
-  bool sandbox_initialized_ok =
-       sandbox_wrapper.InitializeSandbox(*parsed_command_line,
-                                         switches::kNaClLoaderProcess);
-  CHECK(sandbox_initialized_ok) << "Error initializing sandbox for "
-                                << switches::kNaClLoaderProcess;
+  CHECK(content::InitializeSandbox(CHROME_SANDBOX_TYPE_NACL_LOADER, FilePath()))
+      << "Error initializing sandbox for " << switches::kNaClLoaderProcess;
 }
 
 bool NaClMainPlatformDelegate::RunSandboxTests() {
