@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,22 +6,19 @@
 #define CHROME_RENDERER_EXTERNAL_HOST_BINDINGS_H_
 #pragma once
 
-#include "chrome/renderer/dom_ui_bindings.h"
 #include "ipc/ipc_message.h"
+#include "webkit/glue/cpp_bound_class.h"
 
 // ExternalHostBindings is the class backing the "externalHost" object
 // accessible from Javascript
 //
 // We expose one function, for sending a message to the external host:
 //  postMessage(String message[, String target]);
-class ExternalHostBindings : public DOMBoundBrowserObject {
+class ExternalHostBindings : public CppBoundClass {
  public:
-  ExternalHostBindings();
+  ExternalHostBindings(IPC::Message::Sender* sender, int routing_id);
   virtual ~ExternalHostBindings();
 
-  // The postMessage() function provided to Javascript.
-  void postMessage(const CppArgumentList& args, CppVariant* result);
-
   // Invokes the registered onmessage handler.
   // Returns true on successful invocation.
   bool ForwardMessageFromExternalHost(const std::string& message,
@@ -29,20 +26,21 @@ class ExternalHostBindings : public DOMBoundBrowserObject {
                                       const std::string& target);
 
   // Overridden to hold onto a pointer back to the web frame.
-  void BindToJavascript(WebKit::WebFrame* frame, const std::string& classname) {
-    frame_ = frame;
-    DOMBoundBrowserObject::BindToJavascript(frame, classname);
-  }
+  void BindToJavascript(WebKit::WebFrame* frame, const std::string& classname);
 
- protected:
+ private:
   // Creates an uninitialized instance of a MessageEvent object.
   // This is equivalent to calling window.document.createEvent("MessageEvent")
   // in javascript.
   bool CreateMessageEvent(NPObject** message_event);
 
- private:
+  // The postMessage() function provided to Javascript.
+  void PostMessage(const CppArgumentList& args, CppVariant* result);
+
   CppVariant on_message_handler_;
   WebKit::WebFrame* frame_;
+  IPC::Message::Sender* sender_;
+  int routing_id_;
 
   DISALLOW_COPY_AND_ASSIGN(ExternalHostBindings);
 };
