@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,37 +8,54 @@
 #include "base/logging.h"
 #include "base/string16.h"
 #include "base/string_number_conversions.h"
-#include "base/sys_info.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/common/extensions/extension_set.h"
+#include "content/public/renderer/render_thread.h"
 #include "googleurl/src/gurl.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "net/base/escape.h"
 #include "net/base/net_errors.h"
-#include "third_party/WebKit/Source/WebKit/chromium/public/WebURLError.h"
+#include "third_party/WebKit/Source/WebKit/chromium/public/platform/WebURLError.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "webkit/glue/webkit_glue.h"
 
+#if defined(OS_WIN)
+#include "base/win/windows_version.h"
+#endif
+
 using WebKit::WebURLError;
+using content::RenderThread;
 
 namespace {
 
 static const char kRedirectLoopLearnMoreUrl[] =
-    "http://www.google.com/support/chrome/bin/answer.py?answer=95626";
+    "https://www.google.com/support/chrome/bin/answer.py?answer=95626";
 static const char kWeakDHKeyLearnMoreUrl[] =
-    "http://sites.google.com/a/chromium.org/dev/err_ssl_weak_server_ephemeral_dh_key";
+    "http://sites.google.com/a/chromium.org/dev/"
+    "err_ssl_weak_server_ephemeral_dh_key";
 static const char kESETLearnMoreUrl[] =
     "http://kb.eset.com/esetkb/index?page=content&id=SOLN2588";
+static const char kKasperskyLearnMoreUrl[] =
+    "http://support.kaspersky.com/kav2012/settings/options"
+    "?print=true&qid=208284701";
+#if defined(OS_CHROMEOS)
+static const char kAppWarningLearnMoreUrl[] =
+    "chrome-extension://honijodknafkokifofgiaalefdiedpko/main.html"
+    "?answer=1721911";
+#endif  // defined(OS_CHROMEOS)
 
 enum NAV_SUGGESTIONS {
   SUGGEST_NONE     = 0,
   SUGGEST_RELOAD   = 1 << 0,
   SUGGEST_HOSTNAME = 1 << 1,
-  SUGGEST_FIREWALL_CONFIG = 1 << 2,
-  SUGGEST_PROXY_CONFIG = 1 << 3,
-  SUGGEST_LEARNMORE = 1 << 4,
+  SUGGEST_CHECK_CONNECTION = 1 << 2,
+  SUGGEST_DNS_CONFIG = 1 << 3,
+  SUGGEST_FIREWALL_CONFIG = 1 << 4,
+  SUGGEST_PROXY_CONFIG = 1 << 5,
+  SUGGEST_DISABLE_EXTENSION = 1 << 6,
+  SUGGEST_LEARNMORE = 1 << 7,
 };
 
 struct LocalizedErrorMap {
@@ -54,30 +71,56 @@ const LocalizedErrorMap net_error_options[] = {
   {net::ERR_TIMED_OUT,
    IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
-   IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE,
+   IDS_ERRORPAGES_SUMMARY_TIMED_OUT,
    IDS_ERRORPAGES_DETAILS_TIMED_OUT,
-   SUGGEST_RELOAD,
+   SUGGEST_RELOAD | SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG |
+       SUGGEST_PROXY_CONFIG,
   },
   {net::ERR_CONNECTION_TIMED_OUT,
    IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
-   IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE,
+   IDS_ERRORPAGES_SUMMARY_TIMED_OUT,
    IDS_ERRORPAGES_DETAILS_TIMED_OUT,
+   SUGGEST_RELOAD | SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG |
+       SUGGEST_PROXY_CONFIG,
+  },
+  {net::ERR_CONNECTION_CLOSED,
+   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
+   IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
+   IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE,
+   IDS_ERRORPAGES_DETAILS_CONNECTION_CLOSED,
    SUGGEST_RELOAD,
   },
+  {net::ERR_CONNECTION_RESET,
+   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
+   IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
+   IDS_ERRORPAGES_SUMMARY_CONNECTION_RESET,
+   IDS_ERRORPAGES_DETAILS_CONNECTION_RESET,
+   SUGGEST_RELOAD | SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG |
+       SUGGEST_PROXY_CONFIG,
+  },
+  {net::ERR_CONNECTION_REFUSED,
+   IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
+   IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
+   IDS_ERRORPAGES_SUMMARY_CONNECTION_REFUSED,
+   IDS_ERRORPAGES_DETAILS_CONNECTION_REFUSED,
+   SUGGEST_RELOAD | SUGGEST_CHECK_CONNECTION | SUGGEST_FIREWALL_CONFIG |
+       SUGGEST_PROXY_CONFIG,
+  },
   {net::ERR_CONNECTION_FAILED,
    IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
    IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE,
-   IDS_ERRORPAGES_DETAILS_CONNECT_FAILED,
+   IDS_ERRORPAGES_DETAILS_CONNECTION_FAILED,
    SUGGEST_RELOAD,
   },
   {net::ERR_NAME_NOT_RESOLVED,
    IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
    IDS_ERRORPAGES_HEADING_NOT_AVAILABLE,
-   IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE,
+   IDS_ERRORPAGES_SUMMARY_NAME_NOT_RESOLVED,
    IDS_ERRORPAGES_DETAILS_NAME_NOT_RESOLVED,
-   SUGGEST_RELOAD,
+   SUGGEST_RELOAD | SUGGEST_CHECK_CONNECTION | SUGGEST_DNS_CONFIG |
+       SUGGEST_FIREWALL_CONFIG | SUGGEST_PROXY_CONFIG,
   },
   {net::ERR_ADDRESS_UNREACHABLE,
    IDS_ERRORPAGES_TITLE_NOT_AVAILABLE,
@@ -114,6 +157,13 @@ const LocalizedErrorMap net_error_options[] = {
    IDS_ERRORPAGES_DETAILS_FILE_NOT_FOUND,
    SUGGEST_NONE,
   },
+  {net::ERR_CACHE_MISS,
+   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_CACHE_MISS,
+   IDS_ERRORPAGES_SUMMARY_CACHE_MISS,
+   IDS_ERRORPAGES_DETAILS_CACHE_MISS,
+   SUGGEST_RELOAD,
+  },
   {net::ERR_CACHE_READ_FAILURE,
    IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_CACHE_READ_FAILURE,
@@ -135,6 +185,34 @@ const LocalizedErrorMap net_error_options[] = {
    IDS_ERRORPAGES_DETAILS_TOO_MANY_REDIRECTS,
    SUGGEST_RELOAD | SUGGEST_LEARNMORE,
   },
+  {net::ERR_EMPTY_RESPONSE,
+   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_EMPTY_RESPONSE,
+   IDS_ERRORPAGES_SUMMARY_EMPTY_RESPONSE,
+   IDS_ERRORPAGES_DETAILS_EMPTY_RESPONSE,
+   SUGGEST_RELOAD,
+  },
+  {net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_LENGTH,
+   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_DUPLICATE_HEADERS,
+   IDS_ERRORPAGES_SUMMARY_DUPLICATE_HEADERS,
+   IDS_ERRORPAGES_DETAILS_RESPONSE_HEADERS_MULTIPLE_CONTENT_LENGTH,
+   SUGGEST_NONE,
+  },
+  {net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_DISPOSITION,
+   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_DUPLICATE_HEADERS,
+   IDS_ERRORPAGES_SUMMARY_DUPLICATE_HEADERS,
+   IDS_ERRORPAGES_DETAILS_RESPONSE_HEADERS_MULTIPLE_CONTENT_DISPOSITION,
+   SUGGEST_NONE,
+  },
+  {net::ERR_RESPONSE_HEADERS_MULTIPLE_LOCATION,
+   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_DUPLICATE_HEADERS,
+   IDS_ERRORPAGES_SUMMARY_DUPLICATE_HEADERS,
+   IDS_ERRORPAGES_DETAILS_RESPONSE_HEADERS_MULTIPLE_LOCATION,
+   SUGGEST_NONE,
+  },
   {net::ERR_SSL_PROTOCOL_ERROR,
    IDS_ERRORPAGES_TITLE_LOAD_FAILED,
    IDS_ERRORPAGES_HEADING_SSL_PROTOCOL_ERROR,
@@ -170,6 +248,20 @@ const LocalizedErrorMap net_error_options[] = {
    IDS_ERRORPAGES_DETAILS_SSL_PROTOCOL_ERROR,
    SUGGEST_LEARNMORE,
   },
+  {net::ERR_KASPERSKY_ANTI_VIRUS_SSL_INTERCEPTION,
+   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_KASPERSKY_ANTI_VIRUS_SSL_INTERCEPTION,
+   IDS_ERRORPAGES_SUMMARY_KASPERSKY_ANTI_VIRUS_SSL_INTERCEPTION,
+   IDS_ERRORPAGES_DETAILS_SSL_PROTOCOL_ERROR,
+   SUGGEST_LEARNMORE,
+  },
+  {net::ERR_SSL_PINNED_KEY_NOT_IN_CERT_CHAIN,
+   IDS_ERRORPAGES_TITLE_LOAD_FAILED,
+   IDS_ERRORPAGES_HEADING_PINNING_FAILURE,
+   IDS_ERRORPAGES_SUMMARY_PINNING_FAILURE,
+   IDS_ERRORPAGES_DETAILS_PINNING_FAILURE,
+   SUGGEST_NONE,
+  },
   {net::ERR_TEMPORARILY_THROTTLED,
    IDS_ERRORPAGES_TITLE_ACCESS_DENIED,
    IDS_ERRORPAGES_HEADING_ACCESS_DENIED,
@@ -177,6 +269,13 @@ const LocalizedErrorMap net_error_options[] = {
    IDS_ERRORPAGES_DETAILS_TEMPORARILY_THROTTLED,
    SUGGEST_NONE,
   },
+  {net::ERR_BLOCKED_BY_CLIENT,
+   IDS_ERRORPAGES_TITLE_BLOCKED,
+   IDS_ERRORPAGES_HEADING_BLOCKED,
+   IDS_ERRORPAGES_SUMMARY_BLOCKED,
+   IDS_ERRORPAGES_DETAILS_BLOCKED,
+   SUGGEST_DISABLE_EXTENSION,
+  },
 };
 
 const LocalizedErrorMap http_error_options[] = {
@@ -322,6 +421,17 @@ bool LocaleIsRTL() {
 #endif
 }
 
+// Returns a dictionary containing the strings for the settings menu under the
+// wrench, and the advanced settings button.
+DictionaryValue* GetStandardMenuItemsText() {
+  DictionaryValue* standard_menu_items_text = new DictionaryValue();
+  standard_menu_items_text->SetString("settingsTitle",
+      l10n_util::GetStringUTF16(IDS_SETTINGS_TITLE));
+  standard_menu_items_text->SetString("advancedTitle",
+      l10n_util::GetStringUTF16(IDS_OPTIONS_ADVANCED_TAB_LABEL));
+  return standard_menu_items_text;
+}
+
 }  // namespace
 
 const char LocalizedError::kHttpErrorDomain[] = "http";
@@ -355,20 +465,23 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
         l10n_util::GetStringUTF16(IDS_ERRORPAGES_SUGGESTION_HEADING));
   }
 
-  string16 failed_url(ASCIIToUTF16(error.unreachableURL.spec()));
+  const GURL failed_url = error.unreachableURL;
+  string16 failed_url_string(ASCIIToUTF16(failed_url.spec()));
   // URLs are always LTR.
   if (rtl)
-    base::i18n::WrapStringWithLTRFormatting(&failed_url);
+    base::i18n::WrapStringWithLTRFormatting(&failed_url_string);
   error_strings->SetString("title",
-      l10n_util::GetStringFUTF16(options.title_resource_id, failed_url));
+      l10n_util::GetStringFUTF16(options.title_resource_id, failed_url_string));
   error_strings->SetString("heading",
       l10n_util::GetStringUTF16(options.heading_resource_id));
 
   DictionaryValue* summary = new DictionaryValue;
   summary->SetString("msg",
       l10n_util::GetStringUTF16(options.summary_resource_id));
-  // TODO(tc): we want the unicode url here since it's being displayed
-  summary->SetString("failedUrl", failed_url);
+  // TODO(tc): We want the unicode url and host here since they're being
+  //           displayed.
+  summary->SetString("failedUrl", failed_url_string);
+  summary->SetString("hostName", failed_url.host());
   summary->SetString("productName",
                      l10n_util::GetStringUTF16(IDS_PRODUCT_NAME));
   error_strings->Set("summary", summary);
@@ -386,14 +499,12 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
         IDS_ERRORPAGES_SUMMARY_INTERNET_DISCONNECTED_PLATFORM;
 #if defined(OS_WIN)
     // Different versions of Windows have different instructions.
-    int32 major_version, minor_version, bugfix_version;
-    base::SysInfo::OperatingSystemVersionNumbers(
-        &major_version, &minor_version, &bugfix_version);
-    if (major_version < 6) {
+    base::win::Version windows_version = base::win::GetVersion();
+    if (windows_version < base::win::VERSION_VISTA) {
       // XP, XP64, and Server 2003.
       platform_string_id =
           IDS_ERRORPAGES_SUMMARY_INTERNET_DISCONNECTED_PLATFORM_XP;
-    } else if (major_version == 6 && minor_version == 0) {
+    } else if (windows_version == base::win::VERSION_VISTA) {
       // Vista
       platform_string_id =
           IDS_ERRORPAGES_SUMMARY_INTERNET_DISCONNECTED_PLATFORM_VISTA;
@@ -413,13 +524,12 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
     DictionaryValue* suggest_reload = new DictionaryValue;
     suggest_reload->SetString("msg",
         l10n_util::GetStringUTF16(IDS_ERRORPAGES_SUGGESTION_RELOAD));
-    suggest_reload->SetString("reloadUrl", failed_url);
+    suggest_reload->SetString("reloadUrl", failed_url_string);
     error_strings->Set("suggestionsReload", suggest_reload);
   }
 
   if (options.suggestions & SUGGEST_HOSTNAME) {
     // Only show the "Go to hostname" suggestion if the failed_url has a path.
-    const GURL& failed_url = error.unreachableURL;
     if (std::string() == failed_url.path()) {
       DictionaryValue* suggest_home_page = new DictionaryValue;
       suggest_home_page->SetString("suggestionsHomepageMsg",
@@ -435,6 +545,31 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
     }
   }
 
+  if (options.suggestions & SUGGEST_CHECK_CONNECTION) {
+    DictionaryValue* suggest_check_connection = new DictionaryValue;
+    suggest_check_connection->SetString("msg",
+        l10n_util::GetStringUTF16(IDS_ERRORPAGES_SUGGESTION_CHECK_CONNECTION));
+    error_strings->Set("suggestionsCheckConnection", suggest_check_connection);
+  }
+
+  if (options.suggestions & SUGGEST_DNS_CONFIG) {
+    DictionaryValue* suggest_dns_config = new DictionaryValue;
+    suggest_dns_config->SetString("msg",
+        l10n_util::GetStringUTF16(IDS_ERRORPAGES_SUGGESTION_DNS_CONFIG));
+    error_strings->Set("suggestionsDNSConfig", suggest_dns_config);
+
+    DictionaryValue* suggest_network_prediction = GetStandardMenuItemsText();
+    suggest_network_prediction->SetString("msg",
+        l10n_util::GetStringUTF16(
+            IDS_ERRORPAGES_SUGGESTION_NETWORK_PREDICTION));
+    suggest_network_prediction->SetString(
+        "noNetworkPredictionTitle",
+        l10n_util::GetStringUTF16(
+            IDS_NETWORK_PREDICTION_ENABLED_DESCRIPTION));
+    error_strings->Set("suggestionsDisableNetworkPrediction",
+                       suggest_network_prediction);
+  }
+
   if (options.suggestions & SUGGEST_FIREWALL_CONFIG) {
     DictionaryValue* suggest_firewall_config = new DictionaryValue;
     suggest_firewall_config->SetString("msg",
@@ -445,17 +580,40 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
   }
 
   if (options.suggestions & SUGGEST_PROXY_CONFIG) {
-    DictionaryValue* suggest_proxy_config = new DictionaryValue;
+#if defined(OS_CHROMEOS)
+    DictionaryValue* suggest_proxy_config = new DictionaryValue();
+#else
+    DictionaryValue* suggest_proxy_config = GetStandardMenuItemsText();
+#endif  // defined(OS_CHROMEOS)
     suggest_proxy_config->SetString("msg",
-        l10n_util::GetStringUTF16(IDS_ERRORPAGES_SUGGESTION_PROXY_CONFIG));
-    error_strings->Set("suggestionsProxyConfig", suggest_proxy_config);
-
-    DictionaryValue* suggest_proxy_disable = new DictionaryValue;
-    suggest_proxy_disable->SetString("msg",
-        l10n_util::GetStringFUTF16(IDS_ERRORPAGES_SUGGESTION_PROXY_DISABLE,
+        l10n_util::GetStringFUTF16(IDS_ERRORPAGES_SUGGESTION_PROXY_CONFIG,
             l10n_util::GetStringUTF16(
                 IDS_ERRORPAGES_SUGGESTION_PROXY_DISABLE_PLATFORM)));
-    error_strings->Set("suggestionsProxyDisable", suggest_proxy_disable);
+#if defined(OS_CHROMEOS)
+    suggest_proxy_config->SetString("settingsTitle",
+        l10n_util::GetStringUTF16(IDS_SETTINGS_TITLE));
+    suggest_proxy_config->SetString("internetTitle",
+        l10n_util::GetStringUTF16(IDS_OPTIONS_INTERNET_TAB_LABEL));
+    suggest_proxy_config->SetString("optionsButton",
+        l10n_util::GetStringUTF16(IDS_OPTIONS_SETTINGS_OPTIONS));
+    suggest_proxy_config->SetString("networkTab",
+        l10n_util::GetStringUTF16(IDS_OPTIONS_SETTINGS_INTERNET_TAB_NETWORK));
+    suggest_proxy_config->SetString("proxyButton",
+        l10n_util::GetStringUTF16(
+            IDS_OPTIONS_SETTINGS_INTERNET_OPTIONS_CHANGE_PROXY_BUTTON));
+#else
+    suggest_proxy_config->SetString("proxyTitle",
+        l10n_util::GetStringUTF16(IDS_OPTIONS_PROXIES_CONFIGURE_BUTTON));
+#endif  // defined(OS_CHROMEOS)
+    error_strings->Set("suggestionsProxyConfig", suggest_proxy_config);
+  }
+
+  if (options.suggestions & SUGGEST_DISABLE_EXTENSION) {
+    DictionaryValue* suggestion = new DictionaryValue;
+    suggestion->SetString("msg",
+        l10n_util::GetStringUTF16(IDS_ERRORPAGES_SUGGESTION_DISABLE_EXTENSION));
+    suggestion->SetString("reloadUrl", failed_url_string);
+    error_strings->Set("suggestionsDisableExtension", suggestion);
   }
 
   if (options.suggestions & SUGGEST_LEARNMORE) {
@@ -470,6 +628,9 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
       case net::ERR_ESET_ANTI_VIRUS_SSL_INTERCEPTION:
         learn_more_url = GURL(kESETLearnMoreUrl);
         break;
+      case net::ERR_KASPERSKY_ANTI_VIRUS_SSL_INTERCEPTION:
+        learn_more_url = GURL(kKasperskyLearnMoreUrl);
+        break;
       default:
         break;
     }
@@ -477,7 +638,8 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
     if (learn_more_url.is_valid()) {
       // Add the language parameter to the URL.
       std::string query = learn_more_url.query() + "&hl=" +
-          webkit_glue::GetWebKitLocale();
+          RenderThread::Get()->GetLocale();
+
       GURL::Replacements repl;
       repl.SetQueryStr(query);
       learn_more_url = learn_more_url.ReplaceComponents(repl);
@@ -491,6 +653,15 @@ void LocalizedError::GetStrings(const WebKit::WebURLError& error,
   }
 }
 
+string16 LocalizedError::GetErrorDetails(const WebKit::WebURLError& error) {
+  const LocalizedErrorMap* error_map =
+      LookupErrorMap(error.domain.utf8(), error.reason);
+  if (error_map)
+    return l10n_util::GetStringUTF16(error_map->details_resource_id);
+  else
+    return l10n_util::GetStringUTF16(IDS_ERRORPAGES_DETAILS_UNKNOWN);
+}
+
 bool LocalizedError::HasStrings(const std::string& error_domain,
                                 int error_code) {
   return LookupErrorMap(error_domain, error_code) != NULL;
@@ -541,4 +712,14 @@ void LocalizedError::GetAppErrorStrings(
   error_strings->SetString("name", app->name());
   error_strings->SetString("msg",
       l10n_util::GetStringUTF16(IDS_ERRORPAGES_APP_WARNING));
+
+#if defined(OS_CHROMEOS)
+  GURL learn_more_url(kAppWarningLearnMoreUrl);
+  DictionaryValue* suggest_learn_more = new DictionaryValue();
+  suggest_learn_more->SetString("msg",
+                                l10n_util::GetStringUTF16(
+                                    IDS_ERRORPAGES_SUGGESTION_LEARNMORE));
+  suggest_learn_more->SetString("learnMoreUrl", learn_more_url.spec());
+  error_strings->Set("suggestionsLearnMore", suggest_learn_more);
+#endif  // defined(OS_CHROMEOS)
 }
