@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,13 +10,16 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/string16.h"
 #include "printing/image.h"
+#include "ui/gfx/rect.h"
+#include "ui/gfx/size.h"
 
-struct ViewMsg_Print_Params;
-struct ViewMsg_PrintPages_Params;
-struct ViewHostMsg_DidPrintPage_Params;
+struct PrintMsg_Print_Params;
+struct PrintMsg_PrintPages_Params;
+struct PrintHostMsg_DidPrintPage_Params;
 
 // A class which represents an output page used in the MockPrinter class.
 // The MockPrinter class stores output pages in a vector, so, this class
@@ -66,16 +69,20 @@ class MockPrinter {
 
   // Functions that changes settings of a pseudo printer.
   void ResetPrinter();
-  void SetDefaultPrintSettings(const ViewMsg_Print_Params& params);
+  void SetDefaultPrintSettings(const PrintMsg_Print_Params& params);
+  void UseInvalidSettings();
 
   // Functions that handles IPC events.
-  void GetDefaultPrintSettings(ViewMsg_Print_Params* params);
+  void GetDefaultPrintSettings(PrintMsg_Print_Params* params);
   void ScriptedPrint(int cookie,
                      int expected_pages_count,
                      bool has_selection,
-                     ViewMsg_PrintPages_Params* settings);
+                     PrintMsg_PrintPages_Params* settings);
+  void UpdateSettings(int cookie, PrintMsg_PrintPages_Params* params,
+                      const std::vector<int>& page_range_array,
+                      int margins_type);
   void SetPrintedPagesCount(int cookie, int number_pages);
-  void PrintPage(const ViewHostMsg_DidPrintPage_Params& params);
+  void PrintPage(const PrintHostMsg_DidPrintPage_Params& params);
 
   // Functions that retrieve the output pages.
   Status GetPrinterStatus() const { return printer_status_; }
@@ -95,12 +102,17 @@ class MockPrinter {
 
  protected:
   int CreateDocumentCookie();
-  bool GetChecksum(const void* data, uint32 size, std::string* checksum) const;
 
  private:
+  // Helper function to fill the fields in |params|.
+  void SetPrintParams(PrintMsg_Print_Params* params);
+
   // In pixels according to dpi_x and dpi_y.
-  int printable_width_;
-  int printable_height_;
+  gfx::Size page_size_;
+  gfx::Size content_size_;
+  int margin_left_;
+  int margin_top_;
+  gfx::Rect printable_area_;
 
   // Specifies dots per inch.
   double dpi_;
@@ -123,6 +135,21 @@ class MockPrinter {
   // The output of a printing job.
   int number_pages_;
   int page_number_;
+
+  // Used only in the preview sequence.
+  bool is_first_request_;
+  bool print_to_pdf_;
+  int preview_request_id_;
+
+  // Used for displaying headers and footers.
+  bool display_header_footer_;
+  string16 date_;
+  string16 title_;
+  string16 url_;
+
+  // Used for generating invalid settings.
+  bool use_invalid_settings_;
+
   std::vector<scoped_refptr<MockPrinterPage> > pages_;
 
   DISALLOW_COPY_AND_ASSIGN(MockPrinter);
