@@ -1,16 +1,17 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/renderer/page_click_tracker.h"
 
+#include "chrome/common/render_messages.h"
 #include "chrome/renderer/page_click_listener.h"
-#include "chrome/renderer/render_view.h"
+#include "content/public/renderer/render_view.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebDocument.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebDOMMouseEvent.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebFrame.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebInputElement.h"
-#include "third_party/WebKit/Source/WebKit/chromium/public/WebString.h"
+#include "third_party/WebKit/Source/WebKit/chromium/public/platform/WebString.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebView.h"
 
 using WebKit::WebDOMEvent;
@@ -25,8 +26,39 @@ using WebKit::WebNode;
 using WebKit::WebString;
 using WebKit::WebView;
 
-PageClickTracker::PageClickTracker(RenderView* render_view)
-    : RenderViewObserver(render_view),
+namespace {
+
+// Casts |node| to a WebInputElement.
+// Returns an empty (isNull()) WebInputElement if |node| is not a text
+// WebInputElement.
+const WebInputElement GetTextWebInputElement(const WebNode& node) {
+  if (!node.isElementNode())
+    return WebInputElement();
+  const WebElement element = node.toConst<WebElement>();
+  if (!element.isFormControlElement())
+    return WebInputElement();
+  const WebFormControlElement control =
+      element.toConst<WebFormControlElement>();
+  if (control.formControlType() != WebString::fromUTF8("text"))
+    return WebInputElement();
+  return element.toConst<WebInputElement>();
+}
+
+// Checks to see if a text field was the previously selected node and is now
+// losing its focus.
+bool DidSelectedTextFieldLoseFocus(const WebNode& newly_clicked_node) {
+  WebKit::WebNode focused_node = newly_clicked_node.document().focusedNode();
+
+  if (focused_node.isNull() || GetTextWebInputElement(focused_node).isNull())
+    return false;
+
+  return focused_node != newly_clicked_node;
+}
+
+}  // namespace
+
+PageClickTracker::PageClickTracker(content::RenderView* render_view)
+    : content::RenderViewObserver(render_view),
       was_focused_(false) {
 }
 
@@ -46,27 +78,18 @@ void PageClickTracker::DidHandleMouseEvent(const WebMouseEvent& event) {
   }
 
   // We are only interested in text field clicks.
-  if (!last_node_clicked_.isElementNode())
-    return;
-  const WebElement& element = last_node_clicked_.toConst<WebElement>();
-  if (!element.isFormControlElement())
-    return;
-  const WebFormControlElement& control =
-      element.toConst<WebFormControlElement>();
-  if (control.formControlType() != WebString::fromUTF8("text"))
+  const WebInputElement input_element =
+      GetTextWebInputElement(last_node_clicked_);
+  if (input_element.isNull())
     return;
 
-  const WebInputElement& input_element = element.toConst<WebInputElement>();
-
-  bool is_focused = (last_node_clicked_ == GetFocusedNode());
+  bool is_focused = (last_node_clicked_ == render_view()->GetFocusedNode());
   ObserverListBase<PageClickListener>::Iterator it(listeners_);
   PageClickListener* listener;
   while ((listener = it.GetNext()) != NULL) {
     if (listener->InputElementClicked(input_element, was_focused_, is_focused))
       break;
   }
-
-  last_node_clicked_.reset();
 }
 
 void PageClickTracker::AddListener(PageClickListener* listener) {
@@ -77,21 +100,6 @@ void PageClickTracker::RemoveListener(PageClickListener* listener) {
   listeners_.RemoveObserver(listener);
 }
 
-bool PageClickTracker::OnMessageReceived(const IPC::Message& message) {
-  if (message.type() == ViewMsg_HandleInputEvent::ID) {
-    void* iter = NULL;
-    const char* data;
-    int data_length;
-    if (message.ReadData(&iter, &data, &data_length)) {
-      const WebInputEvent* input_event =
-          reinterpret_cast<const WebInputEvent*>(data);
-      if (WebInputEvent::isMouseEventType(input_event->type))
-        DidHandleMouseEvent(*(static_cast<const WebMouseEvent*>(input_event)));
-    }
-  }
-  return false;
-}
-
 void PageClickTracker::DidFinishDocumentLoad(WebKit::WebFrame* frame) {
   tracked_frames_.push_back(frame);
   frame->document().addEventListener("mousedown", this, false);
@@ -123,18 +131,27 @@ void PageClickTracker::handleEvent(const WebDOMEvent& event) {
   // Remember which node has focus before the click is processed.
   // We'll get a notification once the mouse event has been processed
   // (DidHandleMouseEvent), we'll notify the listener at that point.
-  last_node_clicked_ = mouse_event.target();
-  was_focused_ = (GetFocusedNode() == last_node_clicked_);
+  WebNode node = mouse_event.target();
+
+  HandleTextFieldMaybeLosingFocus(node);
+
+  // We are only interested in text field clicks.
+  if (GetTextWebInputElement(node).isNull())
+    return;
+
+  last_node_clicked_ = node;
+  was_focused_ = (node.document().focusedNode() == last_node_clicked_);
 }
 
-WebNode PageClickTracker::GetFocusedNode() {
-  WebView* web_view = render_view()->webview();
-  if (!web_view)
-    return WebNode();
+void PageClickTracker::HandleTextFieldMaybeLosingFocus(
+    const WebNode& newly_clicked_node) {
+  if (!DidSelectedTextFieldLoseFocus(newly_clicked_node))
+    return;
 
-  WebFrame* web_frame = web_view->focusedFrame();
-  if (!web_frame)
-    return WebNode();
-
-  return web_frame->document().focusedNode();
+  ObserverListBase<PageClickListener>::Iterator it(listeners_);
+  PageClickListener* listener;
+  while ((listener = it.GetNext()) != NULL) {
+    if (listener->InputElementLostFocus())
+      break;
+  }
 }
