@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,21 +6,21 @@
 
 #include <ctype.h>
 
+#include "base/bind.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
 #include "base/metrics/histogram.h"
 #include "chrome/common/render_messages.h"
-#include "chrome/renderer/render_process.h"
-#include "chrome/renderer/render_thread.h"
+#include "content/public/renderer/render_thread.h"
 
 // TODO(raman): Before renderer shuts down send final snapshot lists.
 
 using base::Histogram;
 using base::StatisticsRecorder;
+using content::RenderThread;
 
 RendererHistogramSnapshots::RendererHistogramSnapshots()
-    : ALLOW_THIS_IN_INITIALIZER_LIST(
-          renderer_histogram_snapshots_factory_(this)) {
+    : ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {
 }
 
 RendererHistogramSnapshots::~RendererHistogramSnapshots() {
@@ -28,22 +28,38 @@ RendererHistogramSnapshots::~RendererHistogramSnapshots() {
 
 // Send data quickly!
 void RendererHistogramSnapshots::SendHistograms(int sequence_number) {
-  RenderThread::current()->message_loop()->PostTask(FROM_HERE,
-      renderer_histogram_snapshots_factory_.NewRunnableMethod(
-          &RendererHistogramSnapshots::UploadAllHistrograms, sequence_number));
+  RenderThread::Get()->GetMessageLoop()->PostTask(
+      FROM_HERE, base::Bind(&RendererHistogramSnapshots::UploadAllHistrograms,
+                            weak_factory_.GetWeakPtr(), sequence_number));
+}
+
+bool RendererHistogramSnapshots::OnControlMessageReceived(
+    const IPC::Message& message) {
+  bool handled = true;
+  IPC_BEGIN_MESSAGE_MAP(RendererHistogramSnapshots, message)
+    IPC_MESSAGE_HANDLER(ChromeViewMsg_GetRendererHistograms,
+                        OnGetRendererHistograms)
+    IPC_MESSAGE_UNHANDLED(handled = false)
+  IPC_END_MESSAGE_MAP()
+  return handled;
+}
+
+void RendererHistogramSnapshots::OnGetRendererHistograms(int sequence_number) {
+  SendHistograms(sequence_number);
 }
 
 void RendererHistogramSnapshots::UploadAllHistrograms(int sequence_number) {
   DCHECK_EQ(0u, pickled_histograms_.size());
 
+  StatisticsRecorder::CollectHistogramStats("Renderer");
+
   // Push snapshots into our pickled_histograms_ vector.
   TransmitAllHistograms(Histogram::kIPCSerializationSourceFlag, false);
 
   // Send the sequence number and list of pickled histograms over synchronous
   // IPC, so we can clear pickled_histograms_ afterwards.
-  RenderThread::current()->Send(
-      new ViewHostMsg_RendererHistograms(
-          sequence_number, pickled_histograms_));
+  RenderThread::Get()->Send(new ChromeViewHostMsg_RendererHistograms(
+      sequence_number, pickled_histograms_));
 
   pickled_histograms_.clear();
 }
