@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -11,21 +11,35 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
-#include "base/sha2.h"
+#include "base/bind.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string16.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/renderer/safe_browsing/client_model.pb.h"
+#include "chrome/common/safe_browsing/client_model.pb.h"
+#include "chrome/common/safe_browsing/csd.pb.h"
 #include "chrome/renderer/safe_browsing/features.h"
 #include "chrome/renderer/safe_browsing/mock_feature_extractor_clock.h"
-#include "chrome/renderer/safe_browsing/render_view_fake_resources_test.h"
+#include "chrome/renderer/safe_browsing/murmurhash3_util.h"
 #include "chrome/renderer/safe_browsing/scorer.h"
+#include "content/test/render_view_fake_resources_test.h"
+#include "crypto/sha2.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
+using ::testing::AllOf;
+using ::testing::Contains;
+using ::testing::Not;
+using ::testing::Pair;
+
 namespace safe_browsing {
 
 class PhishingClassifierTest : public RenderViewFakeResourcesTest {
  protected:
+  PhishingClassifierTest()
+      : url_tld_token_net_(features::kUrlTldToken + std::string("net")),
+        page_link_domain_phishing_(features::kPageLinkDomain +
+                                   std::string("phishing.com")),
+        page_term_login_(features::kPageTerm + std::string("login")) {}
+
   virtual void SetUp() {
     // Set up WebKit and the RenderView.
     RenderViewFakeResourcesTest::SetUp();
@@ -33,13 +47,18 @@ class PhishingClassifierTest : public RenderViewFakeResourcesTest {
     // Construct a model to test with.  We include one feature from each of
     // the feature extractors, which allows us to verify that they all ran.
     ClientSideModel model;
-    model.add_hashes(base::SHA256HashString(features::kUrlTldToken +
-                                            std::string("net")));
-    model.add_hashes(base::SHA256HashString(features::kPageLinkDomain +
-                                            std::string("phishing.com")));
-    model.add_hashes(base::SHA256HashString(features::kPageTerm +
-                                            std::string("login")));
-    model.add_hashes(base::SHA256HashString("login"));
+
+    model.add_hashes(crypto::SHA256HashString(url_tld_token_net_));
+    model.add_hashes(crypto::SHA256HashString(page_link_domain_phishing_));
+    model.add_hashes(crypto::SHA256HashString(page_term_login_));
+    model.add_hashes(crypto::SHA256HashString("login"));
+    model.add_hashes(crypto::SHA256HashString(features::kUrlTldToken +
+                                              std::string("net")));
+    model.add_hashes(crypto::SHA256HashString(features::kPageLinkDomain +
+                                              std::string("phishing.com")));
+    model.add_hashes(crypto::SHA256HashString(features::kPageTerm +
+                                              std::string("login")));
+    model.add_hashes(crypto::SHA256HashString("login"));
 
     // Add a default rule with a non-phishy weight.
     ClientSideModel::Rule* rule = model.add_rule();
@@ -55,13 +74,14 @@ class PhishingClassifierTest : public RenderViewFakeResourcesTest {
     rule->set_weight(1.0);
 
     model.add_page_term(3);
-    model.add_page_word(3);
+    model.set_murmur_hash_seed(2777808611U);
+    model.add_page_word(MurmurHash3String("login", model.murmur_hash_seed()));
     model.set_max_words_per_term(1);
 
     clock_ = new MockFeatureExtractorClock;
     scorer_.reset(Scorer::Create(model.SerializeAsString()));
     ASSERT_TRUE(scorer_.get());
-    classifier_.reset(new PhishingClassifier(view_, clock_));
+    classifier_.reset(new PhishingClassifier(view(), clock_));
   }
 
   virtual void TearDown() {
@@ -69,25 +89,32 @@ class PhishingClassifierTest : public RenderViewFakeResourcesTest {
   }
 
   // Helper method to start phishing classification and wait for it to
-  // complete.  Returns the success value from the PhishingClassifier's
-  // DoneCallback, and fills in phishy_score with the score.
-  bool RunPhishingClassifier(const string16* page_text, double* phishy_score) {
-    success_ = false;
+  // complete.  Returns the true if the page is classified as phishy and
+  // false otherwise.
+  bool RunPhishingClassifier(const string16* page_text,
+                             float* phishy_score,
+                             FeatureMap* features) {
+    verdict_.Clear();
     *phishy_score = PhishingClassifier::kInvalidScore;
+    features->Clear();
 
     classifier_->BeginClassification(
         page_text,
-        NewCallback(this, &PhishingClassifierTest::ClassificationFinished));
+        base::Bind(&PhishingClassifierTest::ClassificationFinished,
+                   base::Unretained(this)));
     message_loop_.Run();
 
-    *phishy_score = phishy_score_;
-    return success_;
+    *phishy_score = verdict_.client_score();
+    for (int i = 0; i < verdict_.feature_map_size(); ++i) {
+      features->AddRealFeature(verdict_.feature_map(i).name(),
+                               verdict_.feature_map(i).value());
+    }
+    return verdict_.is_phishing();
   }
 
   // Completion callback for classification.
-  void ClassificationFinished(bool success, double phishy_score) {
-    success_ = success;
-    phishy_score_ = phishy_score;
+  void ClassificationFinished(const ClientPhishingRequest& verdict) {
+    verdict_ = verdict;  // copy the verdict.
     message_loop_.Quit();
   }
 
@@ -95,10 +122,14 @@ class PhishingClassifierTest : public RenderViewFakeResourcesTest {
   scoped_ptr<PhishingClassifier> classifier_;
   MockFeatureExtractorClock* clock_;  // owned by classifier_
 
-  // These members hold the status from the most recent call to the
+  // Features that are in the model.
+  const std::string url_tld_token_net_;
+  const std::string page_link_domain_phishing_;
+  const std::string page_term_login_;
+
+  // This member holds the status from the most recent call to the
   // ClassificationFinished callback.
-  bool success_;
-  double phishy_score_;
+  ClientPhishingRequest verdict_;
 };
 
 TEST_F(PhishingClassifierTest, TestClassification) {
@@ -118,35 +149,63 @@ TEST_F(PhishingClassifierTest, TestClassification) {
   LoadURL("http://host.net/");
 
   string16 page_text = ASCIIToUTF16("login");
-  double phishy_score;
-  EXPECT_TRUE(RunPhishingClassifier(&page_text, &phishy_score));
-  EXPECT_DOUBLE_EQ(0.5, phishy_score);
+  float phishy_score;
+  FeatureMap features;
+  EXPECT_TRUE(RunPhishingClassifier(&page_text, &phishy_score, &features));
+  // Note: features.features() might contain other features that simply aren't
+  // in the model.
+  EXPECT_THAT(features.features(),
+              AllOf(Contains(Pair(url_tld_token_net_, 1.0)),
+                    Contains(Pair(page_link_domain_phishing_, 1.0)),
+                    Contains(Pair(page_term_login_, 1.0))));
+  EXPECT_FLOAT_EQ(0.5, phishy_score);
 
   // Change the link domain to something non-phishy.
   responses_["http://host.net/"] =
       "<html><body><a href=\"http://safe.com/\">login</a></body></html>";
   LoadURL("http://host.net/");
 
-  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score));
+  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score, &features));
+  EXPECT_THAT(features.features(),
+              AllOf(Contains(Pair(url_tld_token_net_, 1.0)),
+                    Contains(Pair(page_term_login_, 1.0))));
+  EXPECT_THAT(features.features(),
+              Not(Contains(Pair(page_link_domain_phishing_, 1.0))));
   EXPECT_GE(phishy_score, 0.0);
   EXPECT_LT(phishy_score, 0.5);
 
   // Extraction should fail for this case, since there is no TLD.
   responses_["http://localhost/"] = "<html><body>content</body></html>";
   LoadURL("http://localhost/");
-  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score));
+  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score, &features));
+  EXPECT_EQ(0U, features.features().size());
   EXPECT_EQ(PhishingClassifier::kInvalidScore, phishy_score);
 
   // Extraction should also fail for this case, because the URL is not http.
   responses_["https://host.net/"] = "<html><body>secure</body></html>";
   LoadURL("https://host.net/");
-  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score));
+  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score, &features));
+  EXPECT_EQ(0U, features.features().size());
   EXPECT_EQ(PhishingClassifier::kInvalidScore, phishy_score);
 
   // Extraction should fail for this case because the URL is a POST request.
   LoadURLWithPost("http://host.net/");
-  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score));
+  EXPECT_FALSE(RunPhishingClassifier(&page_text, &phishy_score, &features));
+  EXPECT_EQ(0U, features.features().size());
   EXPECT_EQ(PhishingClassifier::kInvalidScore, phishy_score);
 }
 
+TEST_F(PhishingClassifierTest, DisableDetection) {
+  // No scorer yet, so the classifier is not ready.
+  EXPECT_FALSE(classifier_->is_ready());
+
+  // Now set the scorer.
+  classifier_->set_phishing_scorer(scorer_.get());
+  EXPECT_TRUE(classifier_->is_ready());
+
+  // Set a NULL scorer, which turns detection back off.
+  classifier_->set_phishing_scorer(NULL);
+  EXPECT_FALSE(classifier_->is_ready());
+}
+
 }  // namespace safe_browsing
