@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,15 +7,18 @@
 #include <list>
 #include <map>
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
+#include "base/i18n/case_conversion.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
-#include "base/sha2.h"
 #include "base/metrics/histogram.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
+#include "crypto/sha2.h"
 #include "chrome/renderer/safe_browsing/feature_extractor_clock.h"
 #include "chrome/renderer/safe_browsing/features.h"
+#include "chrome/renderer/safe_browsing/murmurhash3_util.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "unicode/ubrk.h"
 
@@ -23,20 +26,23 @@ namespace safe_browsing {
 
 // This time should be short enough that it doesn't noticeably disrupt the
 // user's interaction with the page.
-const int PhishingTermFeatureExtractor::kMaxTimePerChunkMs = 50;
+const int PhishingTermFeatureExtractor::kMaxTimePerChunkMs = 20;
 
 // Experimenting shows that we get a reasonable gain in performance by
 // increasing this up to around 10, but there's not much benefit in
 // increasing it past that.
-const int PhishingTermFeatureExtractor::kClockCheckGranularity = 10;
+const int PhishingTermFeatureExtractor::kClockCheckGranularity = 5;
 
 // This should be longer than we expect feature extraction to take on any
 // actual phishing page.
 const int PhishingTermFeatureExtractor::kMaxTotalTimeMs = 500;
 
+// The maximum size of the negative word cache.
+const int PhishingTermFeatureExtractor::kMaxNegativeWordCacheSize = 1000;
+
 // All of the state pertaining to the current feature extraction.
 struct PhishingTermFeatureExtractor::ExtractionState {
-  // Stores up to max_words_per_ngram_ previous words separated by spaces.
+  // Stores up to max_words_per_term_ previous words separated by spaces.
   std::string previous_words;
 
   // Stores the sizes of the words in previous_words.  Note: the size includes
@@ -86,14 +92,17 @@ struct PhishingTermFeatureExtractor::ExtractionState {
 
 PhishingTermFeatureExtractor::PhishingTermFeatureExtractor(
     const base::hash_set<std::string>* page_term_hashes,
-    const base::hash_set<std::string>* page_word_hashes,
+    const base::hash_set<uint32>* page_word_hashes,
     size_t max_words_per_term,
+    uint32 murmurhash3_seed,
     FeatureExtractorClock* clock)
     : page_term_hashes_(page_term_hashes),
       page_word_hashes_(page_word_hashes),
       max_words_per_term_(max_words_per_term),
+      murmurhash3_seed_(murmurhash3_seed),
+      negative_word_cache_(kMaxNegativeWordCacheSize),
       clock_(clock),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {
   Clear();
 }
 
@@ -106,7 +115,7 @@ PhishingTermFeatureExtractor::~PhishingTermFeatureExtractor() {
 void PhishingTermFeatureExtractor::ExtractFeatures(
     const string16* page_text,
     FeatureMap* features,
-    DoneCallback* done_callback) {
+    const DoneCallback& done_callback) {
   // The RenderView should have called CancelPendingExtraction() before
   // starting a new extraction, so DCHECK this.
   CheckNoPendingExtraction();
@@ -116,18 +125,18 @@ void PhishingTermFeatureExtractor::ExtractFeatures(
 
   page_text_ = page_text;
   features_ = features;
-  done_callback_.reset(done_callback);
+  done_callback_ = done_callback;
 
   state_.reset(new ExtractionState(*page_text_, clock_->Now()));
   MessageLoop::current()->PostTask(
       FROM_HERE,
-      method_factory_.NewRunnableMethod(
-          &PhishingTermFeatureExtractor::ExtractFeaturesWithTimeout));
+      base::Bind(&PhishingTermFeatureExtractor::ExtractFeaturesWithTimeout,
+                 weak_factory_.GetWeakPtr()));
 }
 
 void PhishingTermFeatureExtractor::CancelPendingExtraction() {
   // Cancel any pending callbacks, and clear our state.
-  method_factory_.RevokeAll();
+  weak_factory_.InvalidateWeakPtrs();
   Clear();
 }
 
@@ -158,8 +167,8 @@ void PhishingTermFeatureExtractor::ExtractFeaturesWithTimeout() {
        next != UBRK_DONE; next = ubrk_next(state_->iterator)) {
     if (ubrk_getRuleStatus(state_->iterator) != UBRK_WORD_NONE) {
       // next is now positioned at the end of a word.
-      HandleWord(string16(*page_text_, state_->position,
-                          next - state_->position));
+      HandleWord(base::StringPiece16(page_text_->data() + state_->position,
+                                     next - state_->position));
       ++num_words;
     }
     state_->position = next;
@@ -188,8 +197,9 @@ void PhishingTermFeatureExtractor::ExtractFeaturesWithTimeout() {
                             chunk_elapsed);
         MessageLoop::current()->PostTask(
             FROM_HERE,
-            method_factory_.NewRunnableMethod(
-                &PhishingTermFeatureExtractor::ExtractFeaturesWithTimeout));
+            base::Bind(
+                &PhishingTermFeatureExtractor::ExtractFeaturesWithTimeout,
+                weak_factory_.GetWeakPtr()));
         return;
       }
       // Otherwise, continue.
@@ -198,43 +208,47 @@ void PhishingTermFeatureExtractor::ExtractFeaturesWithTimeout() {
   RunCallback(true);
 }
 
-void PhishingTermFeatureExtractor::HandleWord(const string16& word) {
-  std::string word_lower = UTF16ToUTF8(l10n_util::ToLower(word));
-  std::string word_hash = base::SHA256HashString(word_lower);
+void PhishingTermFeatureExtractor::HandleWord(
+    const base::StringPiece16& word) {
+  // Quickest out if we have seen this word before and know that it's not
+  // part of any term. This avoids the lowercasing and UTF conversion, both of
+  // which are relatively expensive.
+  if (negative_word_cache_.Get(word) != negative_word_cache_.end()) {
+    // We know we're no longer in a possible n-gram, so clear the previous word
+    // state.
+    state_->previous_words.clear();
+    state_->previous_word_sizes.clear();
+    return;
+  }
+
+  std::string word_lower = UTF16ToUTF8(base::i18n::ToLower(word));
+  uint32 word_hash = MurmurHash3String(word_lower, murmurhash3_seed_);
 
   // Quick out if the word is not part of any term, which is the common case.
   if (page_word_hashes_->find(word_hash) == page_word_hashes_->end()) {
     // Word doesn't exist in our terms so we can clear the n-gram state.
     state_->previous_words.clear();
     state_->previous_word_sizes.clear();
+    // Insert into negative cache so that we don't try this again.
+    negative_word_cache_.Put(word, true);
     return;
   }
 
-  // Find all of the n-grams that we need to check and compute their hashes.
-  // We already have the hash for word_lower, so we don't compute that again.
+  // Find all of the n-grams that we need to check and compute their SHA-256
+  // hashes.
   std::map<std::string /* hash */, std::string /* plaintext */>
       hashes_to_check;
-  hashes_to_check[word_hash] = word_lower;
+  hashes_to_check[crypto::SHA256HashString(word_lower)] = word_lower;
 
   // Combine the new word with the previous words to find additional n-grams.
   // Note that we don't yet add the new word length to previous_word_sizes,
   // since we don't want to compute the hash for the word by itself again.
   //
-  // TODO(bryner): Use UMA stats to determine whether this is too slow.
-  // If it is, there are a couple of cases that we could optimize:
-  //  - We could cache plaintext words that are not in page_word_hashes_, so
-  //    that we can avoid hashing these again.
-  //  - We could include positional information about words in the n-grams,
-  //    rather than just a list of all of the words.  For example, we could
-  //    change the term format so that each word is hashed separately, or
-  //    we could add extra data to the word list to indicate the position
-  //    at which the word appears in an n-gram, and skip checking the word if
-  //    it's not at that position.
   state_->previous_words.append(word_lower);
   std::string current_term = state_->previous_words;
   for (std::list<size_t>::iterator it = state_->previous_word_sizes.begin();
        it != state_->previous_word_sizes.end(); ++it) {
-    hashes_to_check[base::SHA256HashString(current_term)] = current_term;
+    hashes_to_check[crypto::SHA256HashString(current_term)] = current_term;
     current_term.erase(0, *it);
   }
 
@@ -263,9 +277,9 @@ void PhishingTermFeatureExtractor::HandleWord(const string16& word) {
 }
 
 void PhishingTermFeatureExtractor::CheckNoPendingExtraction() {
-  DCHECK(!done_callback_.get());
+  DCHECK(done_callback_.is_null());
   DCHECK(!state_.get());
-  if (done_callback_.get() || state_.get()) {
+  if (!done_callback_.is_null() || state_.get()) {
     LOG(ERROR) << "Extraction in progress, missing call to "
                << "CancelPendingExtraction";
   }
@@ -280,16 +294,17 @@ void PhishingTermFeatureExtractor::RunCallback(bool success) {
   UMA_HISTOGRAM_TIMES("SBClientPhishing.TermFeatureTotalTime",
                       clock_->Now() - state_->start_time);
 
-  DCHECK(done_callback_.get());
-  done_callback_->Run(success);
+  DCHECK(!done_callback_.is_null());
+  done_callback_.Run(success);
   Clear();
 }
 
 void PhishingTermFeatureExtractor::Clear() {
   page_text_ = NULL;
   features_ = NULL;
-  done_callback_.reset(NULL);
+  done_callback_.Reset();
   state_.reset(NULL);
+  negative_word_cache_.Clear();
 }
 
 }  // namespace safe_browsing
