@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,10 +10,13 @@
 #include "base/utf_string_conversions.h"
 #include "chrome/common/render_messages.h"
 #include "chrome/common/spellcheck_common.h"
-#include "chrome/renderer/render_thread.h"
+#include "chrome/common/spellcheck_messages.h"
+#include "content/public/renderer/render_thread.h"
 #include "third_party/hunspell/src/hunspell/hunspell.hxx"
+#include "third_party/WebKit/Source/WebKit/chromium/public/WebTextCheckingResult.h"
 
 using base::TimeTicks;
+using content::RenderThread;
 
 SpellCheck::SpellCheck()
     : file_(base::kInvalidPlatformFileValue),
@@ -26,9 +29,47 @@ SpellCheck::SpellCheck()
 SpellCheck::~SpellCheck() {
 }
 
+bool SpellCheck::OnControlMessageReceived(const IPC::Message& message) {
+  bool handled = true;
+  IPC_BEGIN_MESSAGE_MAP(SpellCheck, message)
+    IPC_MESSAGE_HANDLER(SpellCheckMsg_Init, OnInit)
+    IPC_MESSAGE_HANDLER(SpellCheckMsg_WordAdded, OnWordAdded)
+    IPC_MESSAGE_HANDLER(SpellCheckMsg_EnableAutoSpellCorrect,
+                        OnEnableAutoSpellCorrect)
+    IPC_MESSAGE_UNHANDLED(handled = false)
+  IPC_END_MESSAGE_MAP()
+
+  return handled;
+}
+
+void SpellCheck::OnInit(IPC::PlatformFileForTransit bdict_file,
+                        const std::vector<std::string>& custom_words,
+                        const std::string& language,
+                        bool auto_spell_correct) {
+  Init(IPC::PlatformFileForTransitToPlatformFile(bdict_file),
+       custom_words, language);
+  auto_spell_correct_turned_on_ = auto_spell_correct;
+}
+
+void SpellCheck::OnWordAdded(const std::string& word) {
+  if (is_using_platform_spelling_engine_)
+    return;
+
+  if (!hunspell_.get()) {
+    // Save it for later---add it when hunspell is initialized.
+    custom_words_.push_back(word);
+  } else {
+    AddWordToHunspell(word);
+  }
+}
+
+void SpellCheck::OnEnableAutoSpellCorrect(bool enable) {
+  auto_spell_correct_turned_on_ = enable;
+}
+
 void SpellCheck::Init(base::PlatformFile file,
                       const std::vector<std::string>& custom_words,
-                      const std::string language) {
+                      const std::string& language) {
   initialized_ = true;
   hunspell_.reset();
   bdict_file_.reset();
@@ -70,12 +111,18 @@ bool SpellCheck::SpellCheckWord(
   if (in_word_len == 0)
     return true;  // No input means always spelled correctly.
 
-  SpellcheckWordIterator word_iterator;
   string16 word;
   int word_start;
   int word_length;
-  word_iterator.Initialize(&character_attributes_, in_word, in_word_len, true);
-  while (word_iterator.GetNextWord(&word, &word_start, &word_length)) {
+  if (!text_iterator_.IsInitialized() &&
+      !text_iterator_.Initialize(&character_attributes_, true)) {
+      // We failed to initialize text_iterator_, return as spelled correctly.
+      VLOG(1) << "Failed to initialize SpellcheckWordIterator";
+      return true;
+  }
+
+  text_iterator_.SetText(in_word, in_word_len);
+  while (text_iterator_.GetNextWord(&word, &word_start, &word_length)) {
     // Found a word (or a contraction) that the spellchecker can check the
     // spelling of.
     if (CheckSpelling(word, tag))
@@ -98,6 +145,50 @@ bool SpellCheck::SpellCheckWord(
   return true;
 }
 
+bool SpellCheck::SpellCheckParagraph(
+    const string16& text,
+    int tag,
+    std::vector<WebKit::WebTextCheckingResult>* results) {
+#if !defined(OS_MACOSX)
+  // Mac has its own spell checker, so this method will not be used.
+
+  DCHECK(results);
+
+  size_t length = text.length();
+  size_t offset = 0;
+
+  // Spellcheck::SpellCheckWord() automatically breaks text into words and
+  // checks the spellings of the extracted words. This function sets the
+  // position and length of the first misspelled word and returns false when
+  // the text includes misspelled words. Therefore, we just repeat calling the
+  // function until it returns true to check the whole text.
+  int misspelling_start = 0;
+  int misspelling_length = 0;
+  while (offset <= length) {
+    if (SpellCheckWord(&text[offset],
+                       length - offset,
+                       0,
+                       &misspelling_start,
+                       &misspelling_length,
+                       NULL)) {
+      return true;
+    }
+
+    if (results) {
+      results->push_back(WebKit::WebTextCheckingResult(
+          WebKit::WebTextCheckingTypeSpelling,
+          misspelling_start + offset,
+          misspelling_length));
+    }
+    offset += misspelling_start + misspelling_length;
+  }
+
+  return false;
+#else
+  return true;
+#endif
+}
+
 string16 SpellCheck::GetAutoCorrectionWord(const string16& word, int tag) {
   string16 autocorrect_word;
   if (!auto_spell_correct_turned_on_)
@@ -147,22 +238,6 @@ string16 SpellCheck::GetAutoCorrectionWord(const string16& word, int tag) {
   return autocorrect_word;
 }
 
-void SpellCheck::EnableAutoSpellCorrect(bool turn_on) {
-  auto_spell_correct_turned_on_ = turn_on;
-}
-
-void SpellCheck::WordAdded(const std::string& word) {
-  if (is_using_platform_spelling_engine_)
-    return;
-
-  if (!hunspell_.get()) {
-    // Save it for later---add it when hunspell is initialized.
-    custom_words_.push_back(word);
-  } else {
-    AddWordToHunspell(word);
-  }
-}
-
 void SpellCheck::InitializeHunspell() {
   if (hunspell_.get())
     return;
@@ -170,7 +245,7 @@ void SpellCheck::InitializeHunspell() {
   bdict_file_.reset(new file_util::MemoryMappedFile);
 
   if (bdict_file_->Initialize(file_)) {
-    TimeTicks start_time = TimeTicks::Now();
+    TimeTicks debug_start_time = base::Histogram::DebugNow();
 
     hunspell_.reset(
         new Hunspell(bdict_file_->data(), bdict_file_->length()));
@@ -182,7 +257,7 @@ void SpellCheck::InitializeHunspell() {
     }
 
     DHISTOGRAM_TIMES("Spellcheck.InitTime",
-                     TimeTicks::Now() - start_time);
+                     base::Histogram::DebugNow() - debug_start_time);
   } else {
     NOTREACHED() << "Could not mmap spellchecker dictionary.";
   }
@@ -198,8 +273,7 @@ bool SpellCheck::InitializeIfNeeded() {
     return false;
 
   if (!initialized_) {
-    RenderThread::current()->Send(
-        new ViewHostMsg_SpellChecker_RequestDictionary);
+    RenderThread::Get()->Send(new SpellCheckHostMsg_RequestDictionary);
     initialized_ = true;
     return true;
   }
@@ -217,9 +291,10 @@ bool SpellCheck::CheckSpelling(const string16& word_to_check, int tag) {
   bool word_correct = false;
 
   if (is_using_platform_spelling_engine_) {
-    RenderThread::current()->Send(
-        new ViewHostMsg_SpellChecker_PlatformCheckSpelling(word_to_check, tag,
-                                                           &word_correct));
+#if defined(OS_MACOSX)
+    RenderThread::Get()->Send(new SpellCheckHostMsg_CheckSpelling(
+        word_to_check, tag, &word_correct));
+#endif
   } else {
     std::string word_to_check_utf8(UTF16ToUTF8(word_to_check));
     // Hunspell shouldn't let us exceed its max, but check just in case
@@ -243,9 +318,10 @@ void SpellCheck::FillSuggestionList(
     const string16& wrong_word,
     std::vector<string16>* optional_suggestions) {
   if (is_using_platform_spelling_engine_) {
-    RenderThread::current()->Send(
-        new ViewHostMsg_SpellChecker_PlatformFillSuggestionList(
-            wrong_word, optional_suggestions));
+#if defined(OS_MACOSX)
+    RenderThread::Get()->Send(new SpellCheckHostMsg_FillSuggestionList(
+        wrong_word, optional_suggestions));
+#endif
     return;
   }
 
@@ -273,14 +349,19 @@ void SpellCheck::FillSuggestionList(
 // returns a concatenated word which is not in the selected dictionary
 // (e.g. "in'n'out") but each word is valid.
 bool SpellCheck::IsValidContraction(const string16& contraction, int tag) {
-  SpellcheckWordIterator word_iterator;
-  word_iterator.Initialize(&character_attributes_, contraction.c_str(),
-                           contraction.length(), false);
+  if (!contraction_iterator_.IsInitialized() &&
+      !contraction_iterator_.Initialize(&character_attributes_, false)) {
+    // We failed to initialize the word iterator, return as spelled correctly.
+    VLOG(1) << "Failed to initialize contraction_iterator_";
+    return true;
+  }
+
+  contraction_iterator_.SetText(contraction.c_str(), contraction.length());
 
   string16 word;
   int word_start;
   int word_length;
-  while (word_iterator.GetNextWord(&word, &word_start, &word_length)) {
+  while (contraction_iterator_.GetNextWord(&word, &word_start, &word_length)) {
     if (!CheckSpelling(word, tag))
       return false;
   }
