@@ -1,17 +1,23 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/renderer/translate_helper.h"
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
+#include "base/message_loop.h"
+#include "base/metrics/histogram.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/common/chrome_constants.h"
-#include "chrome/renderer/render_view.h"
+#include "chrome/common/render_messages.h"
+#include "content/public/renderer/render_view.h"
+#include "third_party/WebKit/Source/WebKit/chromium/public/WebDocument.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebElement.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebFrame.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebScriptSource.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebView.h"
+#include "third_party/cld/encodings/compact_lang_det/win/cld_unicodetext.h"
 #include "v8/include/v8.h"
 #include "webkit/glue/dom_operations.h"
 
@@ -22,7 +28,7 @@ using WebKit::WebScriptSource;
 using WebKit::WebString;
 using WebKit::WebView;
 
-// The delay in millliseconds that we'll wait before checking to see if the
+// The delay in milliseconds that we'll wait before checking to see if the
 // translate library injected in the page is ready.
 static const int kTranslateInitCheckDelayMs = 150;
 
@@ -40,72 +46,35 @@ static const char* const kAutoDetectionLanguage = "auto";
 ////////////////////////////////////////////////////////////////////////////////
 // TranslateHelper, public:
 //
-TranslateHelper::TranslateHelper(RenderView* render_view)
-    : render_view_(render_view),
+TranslateHelper::TranslateHelper(content::RenderView* render_view)
+    : content::RenderViewObserver(render_view),
       translation_pending_(false),
       page_id_(-1),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_method_factory_(this)) {
 }
 
 TranslateHelper::~TranslateHelper() {
+  CancelPendingTranslation();
 }
 
-void TranslateHelper::TranslatePage(int page_id,
-                                    const std::string& source_lang,
-                                    const std::string& target_lang,
-                                    const std::string& translate_script) {
-  if (render_view_->page_id() != page_id)
-    return;  // We navigated away, nothing to do.
-
-  if (translation_pending_ && page_id == page_id_ &&
-      target_lang_ == target_lang) {
-    // A similar translation is already under way, nothing to do.
-    return;
+void TranslateHelper::PageCaptured(const string16& contents) {
+  WebDocument document = render_view()->GetWebView()->mainFrame()->document();
+  // If the page explicitly specifies a language, use it, otherwise we'll
+  // determine it based on the text content using the CLD.
+  std::string language = GetPageLanguageFromMetaTag(&document);
+  if (language.empty()) {
+    base::TimeTicks begin_time = base::TimeTicks::Now();
+    language = DetermineTextLanguage(contents);
+    UMA_HISTOGRAM_MEDIUM_TIMES("Renderer4.LanguageDetection",
+                               base::TimeTicks::Now() - begin_time);
   }
 
-  // Any pending translation is now irrelevant.
-  CancelPendingTranslation();
-
-  // Set our states.
-  translation_pending_ = true;
-  page_id_ = page_id;
-  // If the source language is undetermined, we'll let the translate element
-  // detect it.
-  source_lang_ = (source_lang != chrome::kUnknownLanguageCode) ?
-                  source_lang : kAutoDetectionLanguage;
-  target_lang_ = target_lang;
-
-  if (!IsTranslateLibAvailable()) {
-    // Evaluate the script to add the translation related method to the global
-    // context of the page.
-    ExecuteScript(translate_script);
-    DCHECK(IsTranslateLibAvailable());
-  }
-
-  TranslatePageImpl(0);
-}
-
-void TranslateHelper::RevertTranslation(int page_id) {
-  if (render_view_->page_id() != page_id)
-    return;  // We navigated away, nothing to do.
-
-  if (!IsTranslateLibAvailable()) {
-    NOTREACHED();
-    return;
-  }
-
-  WebFrame* main_frame = GetMainFrame();
-  if (!main_frame)
-    return;
-
-  CancelPendingTranslation();
-
-  main_frame->executeScript(
-      WebScriptSource(ASCIIToUTF16("cr.googleTranslate.revert()")));
+  Send(new ChromeViewHostMsg_TranslateLanguageDetermined(
+      routing_id(), language, IsPageTranslatable(&document)));
 }
 
 void TranslateHelper::CancelPendingTranslation() {
-  method_factory_.RevokeAll();
+  weak_method_factory_.InvalidateWeakPtrs();
   translation_pending_ = false;
   page_id_ = -1;
   source_lang_.clear();
@@ -166,6 +135,31 @@ std::string TranslateHelper::GetPageLanguageFromMetaTag(WebDocument* document) {
   return language;
 }
 
+// static
+std::string TranslateHelper::DetermineTextLanguage(const string16& text) {
+  std::string language = chrome::kUnknownLanguageCode;
+  int num_languages = 0;
+  int text_bytes = 0;
+  bool is_reliable = false;
+  Language cld_language =
+      DetectLanguageOfUnicodeText(NULL, text.c_str(), true, &is_reliable,
+                                  &num_languages, NULL, &text_bytes);
+  // We don't trust the result if the CLD reports that the detection is not
+  // reliable, or if the actual text used to detect the language was less than
+  // 100 bytes (short texts can often lead to wrong results).
+  if (is_reliable && text_bytes >= 100 && cld_language != NUM_LANGUAGES &&
+      cld_language != UNKNOWN_LANGUAGE && cld_language != TG_UNKNOWN_LANGUAGE) {
+    // We should not use LanguageCode_ISO_639_1 because it does not cover all
+    // the languages CLD can detect. As a result, it'll return the invalid
+    // language code for tradtional Chinese among others.
+    // |LanguageCodeWithDialect| will go through ISO 639-1, ISO-639-2 and
+    // 'other' tables to do the 'right' thing. In addition, it'll return zh-CN
+    // for Simplified Chinese.
+    language = LanguageCodeWithDialects(cld_language);
+  }
+  return language;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // TranslateHelper, protected:
 //
@@ -236,9 +230,76 @@ bool TranslateHelper::DontDelayTasks() {
 ////////////////////////////////////////////////////////////////////////////////
 // TranslateHelper, private:
 //
+
+bool TranslateHelper::OnMessageReceived(const IPC::Message& message) {
+  bool handled = true;
+  IPC_BEGIN_MESSAGE_MAP(TranslateHelper, message)
+    IPC_MESSAGE_HANDLER(ChromeViewMsg_TranslatePage, OnTranslatePage)
+    IPC_MESSAGE_HANDLER(ChromeViewMsg_RevertTranslation, OnRevertTranslation)
+    IPC_MESSAGE_UNHANDLED(handled = false)
+  IPC_END_MESSAGE_MAP()
+  return handled;
+}
+
+void TranslateHelper::OnTranslatePage(int page_id,
+                                      const std::string& translate_script,
+                                      const std::string& source_lang,
+                                      const std::string& target_lang) {
+  if (render_view()->GetPageId() != page_id)
+    return;  // We navigated away, nothing to do.
+
+  if (translation_pending_ && page_id == page_id_ &&
+      target_lang_ == target_lang) {
+    // A similar translation is already under way, nothing to do.
+    return;
+  }
+
+  // Any pending translation is now irrelevant.
+  CancelPendingTranslation();
+
+  // Set our states.
+  translation_pending_ = true;
+  page_id_ = page_id;
+  // If the source language is undetermined, we'll let the translate element
+  // detect it.
+  source_lang_ = (source_lang != chrome::kUnknownLanguageCode) ?
+                  source_lang : kAutoDetectionLanguage;
+  target_lang_ = target_lang;
+
+  if (!IsTranslateLibAvailable()) {
+    // Evaluate the script to add the translation related method to the global
+    // context of the page.
+    ExecuteScript(translate_script);
+    DCHECK(IsTranslateLibAvailable());
+  }
+
+  TranslatePageImpl(0);
+}
+
+void TranslateHelper::OnRevertTranslation(int page_id) {
+  if (render_view()->GetPageId() != page_id)
+    return;  // We navigated away, nothing to do.
+
+  if (!IsTranslateLibAvailable()) {
+    NOTREACHED();
+    return;
+  }
+
+  WebFrame* main_frame = GetMainFrame();
+  if (!main_frame)
+    return;
+
+  CancelPendingTranslation();
+
+  main_frame->executeScript(
+      WebScriptSource(ASCIIToUTF16("cr.googleTranslate.revert()")));
+}
+
 void TranslateHelper::CheckTranslateStatus() {
-  if (page_id_ != render_view_->page_id())
-    return;  // This is not the same page, the translation has been canceled.
+  // If this is not the same page, the translation has been canceled.  If the
+  // view is gone, the page is closing.
+  if (page_id_ != render_view()->GetPageId() || !render_view()->GetWebView())
+    return;
 
   // First check if there was an error.
   if (HasTranslationFailed()) {
@@ -270,20 +331,20 @@ void TranslateHelper::CheckTranslateStatus() {
 
     translation_pending_ = false;
 
-    // Notify the renderer we are done.
-    render_view_->OnPageTranslated();
-
     // Notify the browser we are done.
-    render_view_->Send(new ViewHostMsg_PageTranslated(
-        render_view_->routing_id(), render_view_->page_id(),
+    render_view()->Send(new ChromeViewHostMsg_PageTranslated(
+        render_view()->GetRoutingId(), render_view()->GetPageId(),
         actual_source_lang, target_lang_, TranslateErrors::NONE));
     return;
   }
 
   // The translation is still pending, check again later.
-  MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      method_factory_.NewRunnableMethod(&TranslateHelper::CheckTranslateStatus),
-      DontDelayTasks() ? 0 : kTranslateStatusCheckDelayMs);
+  MessageLoop::current()->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(&TranslateHelper::CheckTranslateStatus,
+                 weak_method_factory_.GetWeakPtr()),
+      base::TimeDelta::FromMilliseconds(
+          DontDelayTasks() ? 0 : kTranslateStatusCheckDelayMs));
 }
 
 bool TranslateHelper::ExecuteScript(const std::string& script) {
@@ -332,7 +393,7 @@ bool TranslateHelper::ExecuteScriptAndGetStringResult(const std::string& script,
 
 void TranslateHelper::TranslatePageImpl(int count) {
   DCHECK_LT(count, kMaxTranslateInitCheckAttempts);
-  if (page_id_ != render_view_->page_id())
+  if (page_id_ != render_view()->GetPageId() || !render_view()->GetWebView())
     return;
 
   if (!IsTranslateLibReady()) {
@@ -342,10 +403,12 @@ void TranslateHelper::TranslatePageImpl(int count) {
       NotifyBrowserTranslationFailed(TranslateErrors::INITIALIZATION_ERROR);
       return;
     }
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-        method_factory_.NewRunnableMethod(&TranslateHelper::TranslatePageImpl,
-                                          count),
-        DontDelayTasks() ? 0 : count * kTranslateInitCheckDelayMs);
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&TranslateHelper::TranslatePageImpl,
+                   weak_method_factory_.GetWeakPtr(), count),
+        base::TimeDelta::FromMilliseconds(
+            DontDelayTasks() ? 0 : count * kTranslateInitCheckDelayMs));
     return;
   }
 
@@ -354,21 +417,25 @@ void TranslateHelper::TranslatePageImpl(int count) {
     return;
   }
   // Check the status of the translation.
-  MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      method_factory_.NewRunnableMethod(&TranslateHelper::CheckTranslateStatus),
-      DontDelayTasks() ? 0 : kTranslateStatusCheckDelayMs);
+  MessageLoop::current()->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(&TranslateHelper::CheckTranslateStatus,
+                 weak_method_factory_.GetWeakPtr()),
+      base::TimeDelta::FromMilliseconds(
+          DontDelayTasks() ? 0 : kTranslateStatusCheckDelayMs));
 }
 
 void TranslateHelper::NotifyBrowserTranslationFailed(
     TranslateErrors::Type error) {
   translation_pending_ = false;
   // Notify the browser there was an error.
-  render_view_->Send(new ViewHostMsg_PageTranslated(
-      render_view_->routing_id(), page_id_, source_lang_, target_lang_, error));
+  render_view()->Send(new ChromeViewHostMsg_PageTranslated(
+      render_view()->GetRoutingId(), page_id_, source_lang_,
+      target_lang_, error));
 }
 
 WebFrame* TranslateHelper::GetMainFrame() {
-  WebView* web_view = render_view_->webview();
+  WebView* web_view = render_view()->GetWebView();
   if (!web_view) {
     // When the WebView is going away, the render view should have called
     // CancelPendingTranslation() which should have stopped any pending work, so
