@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,10 +8,10 @@
 
 #include <string>
 
-#include "base/task.h"
+#include "base/memory/weak_ptr.h"
 #include "chrome/common/translate_errors.h"
+#include "content/public/renderer/render_view_observer.h"
 
-class RenderView;
 namespace WebKit {
 class WebDocument;
 class WebFrame;
@@ -20,41 +20,22 @@ class WebFrame;
 // This class deals with page translation.
 // There is one TranslateHelper per RenderView.
 
-class TranslateHelper {
+class TranslateHelper : public content::RenderViewObserver {
  public:
-  explicit TranslateHelper(RenderView* render_view);
+  explicit TranslateHelper(content::RenderView* render_view);
   virtual ~TranslateHelper();
 
-  // Translates the page contents from |source_lang| to |target_lang|.
-  // Does nothing if |page_id| is not the current page id.
-  // If the library is not ready, it will post a task to try again after 50ms.
-  void TranslatePage(int page_id,
-                     const std::string& source_lang,
-                     const std::string& target_lang,
-                     const std::string& translate_script);
-
-  // Reverts the page's text to its original contents.
-  void RevertTranslation(int page_id);
-
-  // Cancels any translation that is currently being performed.  This does not
-  // revert existing translations.
-  void CancelPendingTranslation();
-
-  // Returns whether the page associated with |document| is a candidate for
-  // translation.  Some pages can explictly specify (via a meta-tag) that they
-  // should not be translated.
-  static bool IsPageTranslatable(WebKit::WebDocument* document);
-
-  // Returns the language specified in the language meta tag of |document|, or
-  // an empty string if no such tag was found.
-  // The tag may specify several languages, the first one is returned.
-  // Example of such meta-tag:
-  // <meta http-equiv="content-language" content="en, fr">
-  static std::string GetPageLanguageFromMetaTag(WebKit::WebDocument* document);
+  // Informs us that the page's text has been extracted.
+  void PageCaptured(const string16& contents);
 
  protected:
   // The following methods are protected so they can be overridden in
   // unit-tests.
+  void OnTranslatePage(int page_id,
+                       const std::string& translate_script,
+                       const std::string& source_lang,
+                       const std::string& target_lang);
+  void OnRevertTranslation(int page_id);
 
   // Returns true if the translate library is available, meaning the JavaScript
   // has already been injected in that page.
@@ -85,6 +66,29 @@ class TranslateHelper {
   virtual bool DontDelayTasks();
 
  private:
+  // Returns whether the page associated with |document| is a candidate for
+  // translation.  Some pages can explictly specify (via a meta-tag) that they
+  // should not be translated.
+  static bool IsPageTranslatable(WebKit::WebDocument* document);
+
+  // Returns the language specified in the language meta tag of |document|, or
+  // an empty string if no such tag was found.
+  // The tag may specify several languages, the first one is returned.
+  // Example of such meta-tag:
+  // <meta http-equiv="content-language" content="en, fr">
+  static std::string GetPageLanguageFromMetaTag(WebKit::WebDocument* document);
+
+  // Returns the ISO 639_1 language code of the specified |text|, or 'unknown'
+  // if it failed.
+  static std::string DetermineTextLanguage(const string16& text);
+
+  // RenderViewObserver implementation.
+  virtual bool OnMessageReceived(const IPC::Message& message) OVERRIDE;
+
+  // Cancels any translation that is currently being performed.  This does not
+  // revert existing translations.
+  void CancelPendingTranslation();
+
   // Checks if the current running page translation is finished or errored and
   // notifies the browser accordingly.  If the translation has not terminated,
   // posts a task to check again later.
@@ -120,9 +124,6 @@ class TranslateHelper {
   // if the page is being closed.
   WebKit::WebFrame* GetMainFrame();
 
-  // The RenderView we are performing translations for.
-  RenderView* render_view_;
-
   // The states associated with the current translation.
   bool translation_pending_;
   int page_id_;
@@ -130,7 +131,7 @@ class TranslateHelper {
   std::string target_lang_;
 
   // Method factory used to make calls to TranslatePageImpl.
-  ScopedRunnableMethodFactory<TranslateHelper> method_factory_;
+  base::WeakPtrFactory<TranslateHelper> weak_method_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(TranslateHelper);
 };
