@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -27,15 +27,18 @@ const char kJobListValue[] = "jobs";
 const char kTitleValue[] = "title";
 const char kPrinterCapsHashValue[] = "capsHash";
 const char kTagsValue[] = "tags";
+const char kXMPPJidValue[] = "xmpp_jid";
+const char kOAuthCodeValue[] = "authorization_code";
+
 const char kProxyTagPrefix[] = "__cp__";
 const char kTagsHashTagName[] = "__cp__tagshash";
 const char kTagDryRunFlag[] = "__cp__dry_run";
 
 
 const char kDefaultCloudPrintServerUrl[] = "https://www.google.com/cloudprint";
-const char kGaiaUrl[] = "https://www.google.com/accounts/ClientLogin";
 const char kCloudPrintGaiaServiceId[] = "cloudprint";
 const char kSyncGaiaServiceId[] = "chromiumsync";
+const char kProxyAuthUserAgent[] = "ChromiumBrowser";
 const char kCloudPrintPushNotificationsSource[] = "cloudprint.google.com";
 // The cloud print server expects the X-Google-CloudPrint-Proxy header for
 // certain requests.
@@ -57,4 +60,9 @@ const char kJobFetchReasonQueryMore[] = "querymore";
 // Short message ids for diagnostic messages sent to the server.
 const char kPrintSystemFailedMessageId[] = "printsystemfail";
 const char kGetPrinterCapsFailedMessageId[] = "getprncapsfail";
+const char kEnumPrintersFailedMessageId[] = "enumfail";
+
+const char kDefaultCloudPrintOAuthClientId[] =
+    "551556820943.apps.googleusercontent.com";
+const char kDefaultCloudPrintOAuthClientSecret[] = "u3/mp8CgLFxh4uiX1855/MHe";
 
