@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,15 +6,14 @@
 
 #include "base/json/json_reader.h"
 #include "base/md5.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/rand_util.h"
-#include "base/scoped_ptr.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
-#include "base/task.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/service/cloud_print/cloud_print_consts.h"
-#include "chrome/common/net/url_fetcher.h"
+#include "chrome/service/cloud_print/cloud_print_token_store.h"
 #include "chrome/service/service_process.h"
 
 std::string StringFromJobStatus(cloud_print::PrintJobStatus status) {
@@ -139,7 +138,7 @@ GURL CloudPrintHelpers::GetUrlForJobStatusUpdate(
 
 GURL CloudPrintHelpers::GetUrlForUserMessage(const GURL& cloud_print_server_url,
                                              const std::string& message_id) {
-  std::string path(AppendPathToUrl(cloud_print_server_url, "user/message"));
+  std::string path(AppendPathToUrl(cloud_print_server_url, "message"));
   GURL::Replacements replacements;
   replacements.SetPathStr(path);
   std::string query = StringPrintf("code=%s", message_id.c_str());
@@ -147,6 +146,22 @@ GURL CloudPrintHelpers::GetUrlForUserMessage(const GURL& cloud_print_server_url,
   return cloud_print_server_url.ReplaceComponents(replacements);
 }
 
+GURL CloudPrintHelpers::GetUrlForGetAuthCode(const GURL& cloud_print_server_url,
+                                             const std::string& oauth_client_id,
+                                             const std::string& proxy_id) {
+  // We use the internal API "createrobot" instead of "getauthcode". This API
+  // will add the robot as owner to all the existing printers for this user.
+  std::string path(AppendPathToUrl(cloud_print_server_url, "createrobot"));
+  GURL::Replacements replacements;
+  replacements.SetPathStr(path);
+  std::string query = StringPrintf("oauth_client_id=%s&proxy=%s",
+                                    oauth_client_id.c_str(),
+                                    proxy_id.c_str());
+  replacements.SetQueryStr(query);
+  return cloud_print_server_url.ReplaceComponents(replacements);
+}
+
+
 bool CloudPrintHelpers::ParseResponseJSON(
     const std::string& response_data, bool* succeeded,
     DictionaryValue** response_dict) {
@@ -159,8 +174,10 @@ bool CloudPrintHelpers::ParseResponseJSON(
 
   scoped_ptr<DictionaryValue> response_dict_local(
       static_cast<DictionaryValue*>(message_value.release()));
-  if (succeeded)
-    response_dict_local->GetBoolean(kSuccessValue, succeeded);
+  if (succeeded) {
+    if (!response_dict_local->GetBoolean(kSuccessValue, succeeded))
+      *succeeded = false;
+  }
   if (response_dict)
     *response_dict = response_dict_local.release();
   return true;
@@ -200,7 +217,7 @@ std::string CloudPrintHelpers::GenerateHashOfStringMap(
     values_list.append(it->first);
     values_list.append(it->second);
   }
-  return MD5String(values_list);
+  return base::MD5String(values_list);
 }
 
 void CloudPrintHelpers::GenerateMultipartPostDataForPrinterTags(
@@ -233,7 +250,7 @@ void CloudPrintHelpers::GenerateMultipartPostDataForPrinterTags(
     AddMultipartValueForUpload(kPrinterTagValue, msg, mime_boundary,
                                std::string(), post_data);
   }
-  std::string tags_hash = MD5String(tags_list);
+  std::string tags_hash = base::MD5String(tags_list);
   std::string tags_hash_msg(kTagsHashTagName);
   tags_hash_msg += "=";
   tags_hash_msg += tags_hash;
@@ -249,3 +266,20 @@ bool CloudPrintHelpers::IsDryRunJob(const std::vector<std::string>& tags) {
   }
   return false;
 }
+
+std::string CloudPrintHelpers::GetCloudPrintAuthHeader() {
+  std::string header;
+  CloudPrintTokenStore* token_store = CloudPrintTokenStore::current();
+  if (!token_store || token_store->token().empty()) {
+    // Using LOG here for critical errors. GCP connector may run in the headless
+    // mode and error indication might be useful for user in that case.
+    LOG(ERROR) << "CP_PROXY: Missing OAuth token for request";
+  }
+
+  if (token_store) {
+    header = "Authorization: OAuth ";
+    header += token_store->token();
+  }
+  return header;
+}
+
