@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,13 +8,14 @@
 
 #include <map>
 #include <string>
+#include <vector>
 
 #include "chrome/service/cloud_print/print_system.h"
 #include "googleurl/src/gurl.h"
 
+namespace base {
 class DictionaryValue;
-class Task;
-class URLFetcher;
+}
 
 // Helper methods for the cloud print proxy code.
 class CloudPrintHelpers {
@@ -37,13 +38,18 @@ class CloudPrintHelpers {
       const cloud_print::PrintJobDetails& details);
   static GURL GetUrlForUserMessage(const GURL& cloud_print_server_url,
                                    const std::string& message_id);
+  static GURL GetUrlForGetAuthCode(const GURL& cloud_print_server_url,
+                                   const std::string& oauth_client_id,
+                                   const std::string& proxy_id);
+
 
   // Parses the response data for any cloud print server request. The method
   // returns false if there was an error in parsing the JSON. The succeeded
   // value returns the value of the "success" value in the response JSON.
   // Returns the response as a dictionary value.
   static bool ParseResponseJSON(const std::string& response_data,
-      bool* succeeded, DictionaryValue** response_dict);
+                                bool* succeeded,
+                                base::DictionaryValue** response_dict);
 
   // Prepares one value as part of a multi-part upload request.
   static void AddMultipartValueForUpload(
@@ -62,10 +68,11 @@ class CloudPrintHelpers {
 
   // Returns true is tags indicate a dry run (test) job.
   static bool IsDryRunJob(const std::vector<std::string>& tags);
+
+  static std::string GetCloudPrintAuthHeader();
+
  private:
-  CloudPrintHelpers() {
-  }
+  CloudPrintHelpers() {}
 };
 
 #endif  // CHROME_SERVICE_CLOUD_PRINT_CLOUD_PRINT_HELPERS_H_
-
