@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,9 +11,15 @@
 #include "base/threading/thread.h"
 #include "printing/backend/print_backend.h"
 
-class CloudPrintProxyService;
 class GURL;
+
+namespace base {
 class DictionaryValue;
+}
+
+namespace gaia {
+struct OAuthClientInfo;
+}
 
 // CloudPrintProxyFrontend is the interface used by CloudPrintProxyBackend to
 // communicate with the entity that created it and, presumably, is interested in
@@ -24,14 +30,11 @@ class CloudPrintProxyFrontend {
  public:
   CloudPrintProxyFrontend() {}
 
-  // There is a list of printers available that can be registered.
-  virtual void OnPrinterListAvailable(
-      const printing::PrinterList& printer_list) = 0;
   // We successfully authenticated with the cloud print server. This callback
   // allows the frontend to persist the tokens.
-  virtual void OnAuthenticated(const std::string& cloud_print_token,
-                               const std::string& cloud_print_xmpp_token,
-                               const std::string& email) = 0;
+  virtual void OnAuthenticated(const std::string& robot_oauth_refresh_token,
+                               const std::string& robot_email,
+                               const std::string& user_email) = 0;
   // We have invalid/expired credentials.
   virtual void OnAuthenticationFailed() = 0;
   // The print system could not be initialized.
@@ -49,16 +52,36 @@ class CloudPrintProxyBackend {
  public:
   // It is OK for print_system_settings to be NULL. In this case system should
   // use system default settings.
-  explicit CloudPrintProxyBackend(CloudPrintProxyFrontend* frontend,
-                                  const GURL& cloud_print_server_url,
-                                  const DictionaryValue* print_sys_settings);
+  CloudPrintProxyBackend(
+      CloudPrintProxyFrontend* frontend,
+      const std::string& proxy_id,
+      const GURL& cloud_print_server_url,
+      const base::DictionaryValue* print_sys_settings,
+      const gaia::OAuthClientInfo& oauth_client_info,
+      bool enable_job_poll);
   ~CloudPrintProxyBackend();
 
-  bool InitializeWithLsid(const std::string& lsid, const std::string& proxy_id);
-  bool InitializeWithToken(const std::string cloud_print_token,
-                           const std::string cloud_print_xmpp_token,
-                           const std::string email,
+  // Called when the user enables Google Cloud Print.
+  // |last_robot_refresh_token|, |last_robot_email| and |last_user_email| are
+  // the previously persisted credentials if any. We will use this is the passed
+  // in LSID belongs to the same user as |last_user_email|.
+  bool InitializeWithLsid(const std::string& lsid,
+                          const std::string& proxy_id,
+                          const std::string& last_robot_refresh_token,
+                          const std::string& last_robot_email,
+                          const std::string& last_user_email);
+  // Legacy mechanism when we have saved user credentials but no saved robot
+  // credentials.
+  bool InitializeWithToken(const std::string& cloud_print_token,
                            const std::string& proxy_id);
+  // Called when we have saved robot credentials.
+  bool InitializeWithRobotToken(const std::string& robot_oauth_refresh_token,
+                                const std::string& robot_email,
+                                const std::string& proxy_id);
+  // Called when an external entity passed in the auth code for the robot.
+  bool InitializeWithRobotAuthCode(const std::string& robot_oauth_auth_code,
+                                   const std::string& robot_email,
+                                   const std::string& proxy_id);
   void Shutdown();
   void RegisterPrinters(const printing::PrinterList& printer_list);
 
