@@ -1,12 +1,14 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/service/cloud_print/job_status_updater.h"
 
+#include "base/bind.h"
 #include "base/json/json_reader.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
+#include "base/values.h"
 #include "chrome/common/net/http_return.h"
 #include "chrome/service/cloud_print/cloud_print_consts.h"
 #include "chrome/service/cloud_print/cloud_print_helpers.h"
@@ -15,12 +17,11 @@
 JobStatusUpdater::JobStatusUpdater(const std::string& printer_name,
                            const std::string& job_id,
                            cloud_print::PlatformJobId& local_job_id,
-                           const std::string& auth_token,
                            const GURL& cloud_print_server_url,
                            cloud_print::PrintSystem* print_system,
                            Delegate* delegate)
     : printer_name_(printer_name), job_id_(job_id),
-      local_job_id_(local_job_id), auth_token_(auth_token),
+      local_job_id_(local_job_id),
       cloud_print_server_url_(cloud_print_server_url),
       print_system_(print_system), delegate_(delegate), stopped_(false) {
   DCHECK(delegate_);
@@ -62,7 +63,9 @@ void JobStatusUpdater::UpdateStatus() {
       request_->StartGetRequest(
           CloudPrintHelpers::GetUrlForJobStatusUpdate(
               cloud_print_server_url_, job_id_, last_job_details_),
-          this, auth_token_, kCloudPrintAPIMaxRetryCount);
+          this,
+          kCloudPrintAPIMaxRetryCount,
+          std::string());
     }
   }
 }
@@ -76,18 +79,28 @@ void JobStatusUpdater::Stop() {
 
 // CloudPrintURLFetcher::Delegate implementation.
 CloudPrintURLFetcher::ResponseAction JobStatusUpdater::HandleJSONData(
-      const URLFetcher* source,
+      const content::URLFetcher* source,
       const GURL& url,
       DictionaryValue* json_data,
       bool succeeded) {
   if (last_job_details_.status == cloud_print::PRINT_JOB_STATUS_COMPLETED) {
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &JobStatusUpdater::Stop));
+        FROM_HERE, base::Bind(&JobStatusUpdater::Stop, this));
   }
   return CloudPrintURLFetcher::STOP_PROCESSING;
 }
 
-void JobStatusUpdater::OnRequestAuthError() {
+CloudPrintURLFetcher::ResponseAction JobStatusUpdater::OnRequestAuthError() {
+  // We got an Auth error and have no idea how long it will take to refresh
+  // auth information (may take forever). We'll drop current request and
+  // propagate this error to the upper level. After auth issues will be
+  // resolved, GCP connector will restart.
   if (delegate_)
     delegate_->OnAuthError();
+  return CloudPrintURLFetcher::STOP_PROCESSING;
 }
+
+std::string JobStatusUpdater::GetAuthHeader() {
+  return CloudPrintHelpers::GetCloudPrintAuthHeader();
+}
+
