@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,15 +11,17 @@
 #include <vector>
 
 #include "base/callback.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 
 #include "printing/backend/print_backend.h"
 
-class DictionaryValue;
 class FilePath;
 
+namespace base {
+class DictionaryValue;
+}
+
 namespace printing {
-class PrintBackend;
 struct PrinterBasicInfo;
 struct PrinterCapsAndDefaults;
 }
@@ -145,11 +147,10 @@ class PrintSystem : public base::RefCountedThreadSafe<PrintSystem> {
     PrintSystemResult() { }
   };
 
-  typedef Callback3<
-      bool,
-      const std::string&,
-      const printing::PrinterCapsAndDefaults&>::Type
-          PrinterCapsAndDefaultsCallback;
+  typedef base::Callback<void(bool,
+                              const std::string&,
+                              const printing::PrinterCapsAndDefaults&)>
+      PrinterCapsAndDefaultsCallback;
 
   virtual ~PrintSystem();
 
@@ -158,12 +159,13 @@ class PrintSystem : public base::RefCountedThreadSafe<PrintSystem> {
   virtual PrintSystemResult Init() = 0;
 
   // Enumerates the list of installed local and network printers.
-  virtual void EnumeratePrinters(printing::PrinterList* printer_list) = 0;
+  virtual PrintSystemResult EnumeratePrinters(
+      printing::PrinterList* printer_list) = 0;
 
   // Gets the capabilities and defaults for a specific printer asynchronously.
   virtual void GetPrinterCapsAndDefaults(
       const std::string& printer_name,
-      PrinterCapsAndDefaultsCallback* callback) = 0;
+      const PrinterCapsAndDefaultsCallback& callback) = 0;
 
   // Returns true if printer_name points to a valid printer.
   virtual bool IsValidPrinter(const std::string& printer_name) = 0;
@@ -175,7 +177,7 @@ class PrintSystem : public base::RefCountedThreadSafe<PrintSystem> {
   // Get details for already spooled job.
   virtual bool GetJobDetails(const std::string& printer_name,
                              PlatformJobId job_id,
-                             PrintJobDetails *job_details) = 0;
+                             PrintJobDetails* job_details) = 0;
 
   // Factory methods to create corresponding watcher. Callee is responsible
   // for deleting objects. Return NULL if failed.
@@ -184,6 +186,11 @@ class PrintSystem : public base::RefCountedThreadSafe<PrintSystem> {
       const std::string& printer_name) = 0;
   virtual JobSpooler* CreateJobSpooler() = 0;
 
+  // Returns a comma separated list of mimetypes for print data that are
+  // supported by this print system. The format of this string is the same as
+  // that used for the HTTP Accept: header.
+  virtual std::string GetSupportedMimeTypes() = 0;
+
   // Generate unique for proxy.
   static std::string GenerateProxyId();
 
@@ -191,7 +198,7 @@ class PrintSystem : public base::RefCountedThreadSafe<PrintSystem> {
   // If print settings are NULL, default settings will be used.
   // Return NULL if no print system available.
   static scoped_refptr<PrintSystem> CreateInstance(
-      const DictionaryValue* print_system_settings);
+      const base::DictionaryValue* print_system_settings);
 };
 
 
