@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,38 +13,45 @@
 #include <list>
 #include <map>
 
+#include "base/bind.h"
 #include "base/file_path.h"
 #include "base/json/json_reader.h"
 #include "base/logging.h"
 #include "base/md5.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
 #include "base/rand_util.h"
-#include "base/scoped_ptr.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
-#include "base/task.h"
-#include "base/values.h"
 #include "base/utf_string_conversions.h"
+#include "base/values.h"
 #include "chrome/service/cloud_print/cloud_print_consts.h"
 #include "chrome/service/cloud_print/cloud_print_helpers.h"
 #include "googleurl/src/gurl.h"
+#include "grit/generated_resources.h"
 #include "printing/backend/cups_helper.h"
 #include "printing/backend/print_backend.h"
 #include "printing/backend/print_backend_consts.h"
+#include "ui/base/l10n/l10n_util.h"
 
 namespace {
-static const char kCUPSPrinterInfoOpt[] = "printer-info";
-static const char kCUPSPrinterStateOpt[] = "printer-state";
-static const char kCUPSPrintServerURLs[] = "print_server_urls";
-static const char kCUPSUpdateTimeoutMs[] = "update_timeout_ms";
+
+// CUPS specific options.
+const char kCUPSPrinterInfoOpt[] = "printer-info";
+const char kCUPSPrinterStateOpt[] = "printer-state";
+
+// Print system config options.
+const char kCUPSPrintServerURLs[] = "print_server_urls";
+const char kCUPSUpdateTimeoutMs[] = "update_timeout_ms";
+const char kCUPSNotifyDelete[] = "notify_delete";
 
 // Default port for IPP print servers.
-static const int kDefaultIPPServerPort = 631;
+const int kDefaultIPPServerPort = 631;
 
 // Time interval to check for printer's updates.
-const int kCheckForPrinterUpdatesMs = 6*60*60*1000;
+const int kCheckForPrinterUpdatesMs = 5*60*1000;
 
-// Job update timeput
+// Job update timeout
 const int kJobUpdateTimeoutMs = 5000;
 
 // Job id for dry run (it should not affect CUPS job ids, since 0 job-id is
@@ -71,27 +78,24 @@ class PrintSystemCUPS : public PrintSystem {
   explicit PrintSystemCUPS(const DictionaryValue* print_system_settings);
 
   // PrintSystem implementation.
-  virtual PrintSystemResult Init();
-
-  virtual void EnumeratePrinters(printing::PrinterList* printer_list);
-
+  virtual PrintSystemResult Init() OVERRIDE;
+  virtual PrintSystem::PrintSystemResult EnumeratePrinters(
+      printing::PrinterList* printer_list) OVERRIDE;
   virtual void GetPrinterCapsAndDefaults(
       const std::string& printer_name,
-      PrinterCapsAndDefaultsCallback* callback);
-
-  virtual bool IsValidPrinter(const std::string& printer_name);
-
-  virtual bool ValidatePrintTicket(const std::string& printer_name,
-                                   const std::string& print_ticket_data);
-
+      const PrinterCapsAndDefaultsCallback& callback) OVERRIDE;
+  virtual bool IsValidPrinter(const std::string& printer_name) OVERRIDE;
+  virtual bool ValidatePrintTicket(
+      const std::string& printer_name,
+      const std::string& print_ticket_data) OVERRIDE;
   virtual bool GetJobDetails(const std::string& printer_name,
                              PlatformJobId job_id,
-                             PrintJobDetails *job_details);
-
-  virtual PrintSystem::PrintServerWatcher* CreatePrintServerWatcher();
+                             PrintJobDetails *job_details) OVERRIDE;
+  virtual PrintSystem::PrintServerWatcher* CreatePrintServerWatcher() OVERRIDE;
   virtual PrintSystem::PrinterWatcher* CreatePrinterWatcher(
-      const std::string& printer_name);
-  virtual PrintSystem::JobSpooler* CreateJobSpooler();
+      const std::string& printer_name) OVERRIDE;
+  virtual PrintSystem::JobSpooler* CreateJobSpooler() OVERRIDE;
+  virtual std::string GetSupportedMimeTypes() OVERRIDE;
 
   // Helper functions.
   PlatformJobId SpoolPrintJob(const std::string& print_ticket,
@@ -115,6 +119,12 @@ class PrintSystemCUPS : public PrintSystem {
     return update_timeout_;
   }
 
+  bool NotifyDelete() const {
+    // Notify about deleted printers only when we
+    // fetched printers list without errors.
+    return notify_delete_ && printer_enum_succeeded_;
+  }
+
  private:
   // Following functions are wrappers around corresponding CUPS functions.
   // <functions>2()  are called when print server is specified, and plain
@@ -139,7 +149,7 @@ class PrintSystemCUPS : public PrintSystem {
 
   // Helper method to invoke a PrinterCapsAndDefaultsCallback.
   static void RunCapsCallback(
-      PrinterCapsAndDefaultsCallback* callback,
+      const PrinterCapsAndDefaultsCallback& callback,
       bool succeeded,
       const std::string& printer_name,
       const printing::PrinterCapsAndDefaults& printer_info);
@@ -151,6 +161,8 @@ class PrintSystemCUPS : public PrintSystem {
 
   int update_timeout_;
   bool initialized_;
+  bool printer_enum_succeeded_;
+  bool notify_delete_;
 };
 
 class PrintServerWatcherCUPS
@@ -164,17 +176,19 @@ class PrintServerWatcherCUPS
     StopWatching();
   }
 
-  // PrintSystem::PrintServerWatcher interface
+  // PrintSystem::PrintServerWatcher implementation.
   virtual bool StartWatching(
-      PrintSystem::PrintServerWatcher::Delegate* delegate) {
+      PrintSystem::PrintServerWatcher::Delegate* delegate) OVERRIDE {
     delegate_ = delegate;
     printers_hash_ = GetPrintersHash();
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-        NewRunnableMethod(this, &PrintServerWatcherCUPS::CheckForUpdates),
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&PrintServerWatcherCUPS::CheckForUpdates, this),
         print_system_->GetUpdateTimeoutMs());
     return true;
   }
-  virtual bool StopWatching() {
+
+  virtual bool StopWatching() OVERRIDE {
     delegate_ = NULL;
     return true;
   }
@@ -188,10 +202,12 @@ class PrintServerWatcherCUPS
       printers_hash_ = new_hash;
       delegate_->OnPrinterAdded();
     }
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-        NewRunnableMethod(this, &PrintServerWatcherCUPS::CheckForUpdates),
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&PrintServerWatcherCUPS::CheckForUpdates, this),
         print_system_->GetUpdateTimeoutMs());
   }
+
  private:
   std::string GetPrintersHash() {
     printing::PrinterList printer_list;
@@ -208,50 +224,56 @@ class PrintServerWatcherCUPS
     for (size_t i = 0; i < printers.size(); i++)
       to_hash += printers[i];
 
-    return MD5String(to_hash);
+    return base::MD5String(to_hash);
   }
 
   scoped_refptr<PrintSystemCUPS> print_system_;
   PrintSystem::PrintServerWatcher::Delegate* delegate_;
   std::string printers_hash_;
+
   DISALLOW_COPY_AND_ASSIGN(PrintServerWatcherCUPS);
 };
 
 class PrinterWatcherCUPS
     : public PrintSystem::PrinterWatcher {
  public:
-  explicit PrinterWatcherCUPS(PrintSystemCUPS* print_system,
-                             const std::string& printer_name)
+  PrinterWatcherCUPS(PrintSystemCUPS* print_system,
+                     const std::string& printer_name)
       : printer_name_(printer_name),
     delegate_(NULL),
     print_system_(print_system) {
   }
+
   ~PrinterWatcherCUPS() {
     StopWatching();
   }
 
-  // PrintSystem::PrinterWatcher interface
+  // PrintSystem::PrinterWatcher implementation.
   virtual bool StartWatching(
-      PrintSystem::PrinterWatcher::Delegate* delegate) {
+      PrintSystem::PrinterWatcher::Delegate* delegate) OVERRIDE{
     if (delegate_ != NULL)
       StopWatching();
     delegate_ = delegate;
     settings_hash_ = GetSettingsHash();
     // Schedule next job status update.
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-        NewRunnableMethod(this, &PrinterWatcherCUPS::JobStatusUpdate),
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&PrinterWatcherCUPS::JobStatusUpdate, this),
         kJobUpdateTimeoutMs);
     // Schedule next printer check.
     // TODO(gene): Randomize time for the next printer update.
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-        NewRunnableMethod(this, &PrinterWatcherCUPS::PrinterUpdate),
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&PrinterWatcherCUPS::PrinterUpdate, this),
         print_system_->GetUpdateTimeoutMs());
     return true;
   }
-  virtual bool StopWatching() {
+
+  virtual bool StopWatching() OVERRIDE{
     delegate_ = NULL;
     return true;
   }
+
   bool GetCurrentPrinterInfo(printing::PrinterBasicInfo* printer_info) {
     DCHECK(printer_info);
     return print_system_->GetPrinterInfo(printer_name_, printer_info);
@@ -265,8 +287,9 @@ class PrinterWatcherCUPS
     // jobs for this printer and check their status. If printer has no
     // outstanding jobs, OnJobChanged() will do nothing.
     delegate_->OnJobChanged();
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-        NewRunnableMethod(this, &PrinterWatcherCUPS::JobStatusUpdate),
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&PrinterWatcherCUPS::JobStatusUpdate, this),
         kJobUpdateTimeoutMs);
   }
 
@@ -274,16 +297,24 @@ class PrinterWatcherCUPS
     if (delegate_ == NULL)
       return;  // Orphan call. We have been stopped already.
     VLOG(1) << "CP_CUPS: Checking for printer updates: " << printer_name_;
-    std::string new_hash = GetSettingsHash();
-    if (settings_hash_ != new_hash) {
-      settings_hash_ = new_hash;
-      delegate_->OnPrinterChanged();
-      VLOG(1) << "CP_CUPS: Printer update detected for: " << printer_name_;
+    if (print_system_->NotifyDelete() &&
+        !print_system_->IsValidPrinter(printer_name_)) {
+      delegate_->OnPrinterDeleted();
+      VLOG(1) << "CP_CUPS: Printer deleted: " << printer_name_;
+    } else {
+      std::string new_hash = GetSettingsHash();
+      if (settings_hash_ != new_hash) {
+        settings_hash_ = new_hash;
+        delegate_->OnPrinterChanged();
+        VLOG(1) << "CP_CUPS: Printer update detected for: " << printer_name_;
+      }
     }
-    MessageLoop::current()->PostDelayedTask(FROM_HERE,
-        NewRunnableMethod(this, &PrinterWatcherCUPS::PrinterUpdate),
+    MessageLoop::current()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&PrinterWatcherCUPS::PrinterUpdate, this),
         print_system_->GetUpdateTimeoutMs());
   }
+
  private:
   std::string GetSettingsHash() {
     printing::PrinterBasicInfo info;
@@ -307,13 +338,14 @@ class PrinterWatcherCUPS
     to_hash += caps.printer_defaults;
     to_hash += caps.defaults_mime_type;
 
-    return MD5String(to_hash);
+    return base::MD5String(to_hash);
   }
 
   std::string printer_name_;
   PrintSystem::PrinterWatcher::Delegate* delegate_;
   scoped_refptr<PrintSystemCUPS> print_system_;
   std::string settings_hash_;
+
   DISALLOW_COPY_AND_ASSIGN(PrinterWatcherCUPS);
 };
 
@@ -323,6 +355,7 @@ class JobSpoolerCUPS : public PrintSystem::JobSpooler {
       : print_system_(print_system) {
     DCHECK(print_system_.get());
   }
+
   // PrintSystem::JobSpooler implementation.
   virtual bool Spool(const std::string& print_ticket,
                      const FilePath& print_data_file_path,
@@ -330,18 +363,15 @@ class JobSpoolerCUPS : public PrintSystem::JobSpooler {
                      const std::string& printer_name,
                      const std::string& job_title,
                      const std::vector<std::string>& tags,
-                     JobSpooler::Delegate* delegate) {
+                     JobSpooler::Delegate* delegate) OVERRIDE{
     DCHECK(delegate);
     bool dry_run = false;
     int job_id = print_system_->SpoolPrintJob(
         print_ticket, print_data_file_path, print_data_mime_type,
         printer_name, job_title, tags, &dry_run);
-    MessageLoop::current()->PostTask(FROM_HERE,
-                                     NewRunnableFunction(
-                                         &JobSpoolerCUPS::NotifyDelegate,
-                                         delegate,
-                                         job_id,
-                                         dry_run));
+    MessageLoop::current()->PostTask(
+        FROM_HERE,
+        base::Bind(&JobSpoolerCUPS::NotifyDelegate, delegate, job_id, dry_run));
     return true;
   }
 
@@ -352,17 +382,26 @@ class JobSpoolerCUPS : public PrintSystem::JobSpooler {
     else
       delegate->OnJobSpoolFailed();
   }
+
  private:
   scoped_refptr<PrintSystemCUPS> print_system_;
+
   DISALLOW_COPY_AND_ASSIGN(JobSpoolerCUPS);
 };
 
 PrintSystemCUPS::PrintSystemCUPS(const DictionaryValue* print_system_settings)
-    : update_timeout_(kCheckForPrinterUpdatesMs), initialized_(false) {
+    : update_timeout_(kCheckForPrinterUpdatesMs),
+      initialized_(false),
+      printer_enum_succeeded_(false),
+      notify_delete_(true) {
   if (print_system_settings) {
     int timeout;
     if (print_system_settings->GetInteger(kCUPSUpdateTimeoutMs, &timeout))
       update_timeout_ = timeout;
+
+    bool notify_delete = true;
+    if (print_system_settings->GetBoolean(kCUPSNotifyDelete, &notify_delete))
+      notify_delete_ = notify_delete;
   }
 
   InitPrintBackends(print_system_settings);
@@ -412,8 +451,10 @@ PrintSystem::PrintSystemResult PrintSystemCUPS::Init() {
 
 void PrintSystemCUPS::UpdatePrinters() {
   PrintServerList::iterator it;
+  printer_enum_succeeded_ = true;
   for (it = print_servers_.begin(); it != print_servers_.end(); ++it) {
-    it->backend->EnumeratePrinters(&it->printers);
+    if (!it->backend->EnumeratePrinters(&it->printers))
+      printer_enum_succeeded_ = false;
     it->caps_cache.clear();
     printing::PrinterList::iterator printer_it;
     for (printer_it = it->printers.begin();
@@ -426,12 +467,13 @@ void PrintSystemCUPS::UpdatePrinters() {
   }
 
   // Schedule next update.
-  MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      NewRunnableMethod(this, &PrintSystemCUPS::UpdatePrinters),
-      GetUpdateTimeoutMs());
+  MessageLoop::current()->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(&PrintSystemCUPS::UpdatePrinters, this), GetUpdateTimeoutMs());
 }
 
-void PrintSystemCUPS::EnumeratePrinters(printing::PrinterList* printer_list) {
+PrintSystem::PrintSystemResult PrintSystemCUPS::EnumeratePrinters(
+    printing::PrinterList* printer_list) {
   DCHECK(initialized_);
   printer_list->clear();
   PrintServerList::iterator it;
@@ -440,20 +482,20 @@ void PrintSystemCUPS::EnumeratePrinters(printing::PrinterList* printer_list) {
         it->printers.begin(), it->printers.end());
   }
   VLOG(1) << "CUPS: Total " << printer_list->size() << " printers enumerated.";
+  // TODO(sanjeevr): Maybe some day we want to report the actual server names
+  // for which the enumeration failed.
+  return PrintSystemResult(printer_enum_succeeded_, std::string());
 }
 
 void PrintSystemCUPS::GetPrinterCapsAndDefaults(
     const std::string& printer_name,
-    PrinterCapsAndDefaultsCallback* callback) {
+    const PrinterCapsAndDefaultsCallback& callback) {
   printing::PrinterCapsAndDefaults printer_info;
   bool succeeded = GetPrinterCapsAndDefaults(printer_name, &printer_info);
   MessageLoop::current()->PostTask(
       FROM_HERE,
-      NewRunnableFunction(&PrintSystemCUPS::RunCapsCallback,
-                          callback,
-                          succeeded,
-                          printer_name,
-                          printer_info));
+      base::Bind(&PrintSystemCUPS::RunCapsCallback, callback, succeeded,
+                 printer_name, printer_info));
 }
 
 bool PrintSystemCUPS::IsValidPrinter(const std::string& printer_name) {
@@ -533,19 +575,21 @@ bool PrintSystemCUPS::GetJobDetails(const std::string& printer_name,
   cups_job_t* jobs = NULL;
   int num_jobs = GetJobs(&jobs, server_info->url,
                          short_printer_name.c_str(), 1, -1);
-
+  bool error = (num_jobs == 0) && (cupsLastError() > IPP_OK_EVENTS_COMPLETE);
+  if (error) {
+    VLOG(1) << "CP_CUPS: Error getting jobs from CUPS server. Printer:"
+            << printer_name
+            << " Error: "
+            << static_cast<int>(cupsLastError());
+    return false;
+  }
 
   // Check if the request is for dummy dry run job.
   // We check this after calling GetJobs API to see if this printer is actually
   // accessible through CUPS.
   if (job_id == kDryRunJobId) {
-    if (num_jobs >= 0) {
-      job_details->status = PRINT_JOB_STATUS_COMPLETED;
-      VLOG(1) << "CP_CUPS: Dry run job succeeded for: " << printer_name;
-    } else {
-      job_details->status = PRINT_JOB_STATUS_ERROR;
-      VLOG(1) << "CP_CUPS: Dry run job faield for: " << printer_name;
-    }
+    job_details->status = PRINT_JOB_STATUS_COMPLETED;
+    VLOG(1) << "CP_CUPS: Dry run job succeeded for: " << printer_name;
     return true;
   }
 
@@ -630,6 +674,14 @@ PrintSystem::JobSpooler* PrintSystemCUPS::CreateJobSpooler() {
   return new JobSpoolerCUPS(this);
 }
 
+std::string PrintSystemCUPS::GetSupportedMimeTypes() {
+  // Since we hand off the document to the CUPS server directly, list some types
+  // that we know CUPS supports (http://www.cups.org/articles.php?L205+TFAQ+Q)
+  // TODO(sanjeevr): Determine this dynamically (http://crbug.com/73240).
+  return
+      "application/pdf,application/postscript,image/jpeg,image/png,image/gif";
+}
+
 std::string PrintSystem::GenerateProxyId() {
   // TODO(gene): This code should generate a unique id for proxy. ID should be
   // unique for this user. Rand may return the same number. We'll need to change
@@ -764,12 +816,11 @@ PrintServerInfoCUPS* PrintSystemCUPS::FindServerByFullName(
 }
 
 void PrintSystemCUPS::RunCapsCallback(
-    PrinterCapsAndDefaultsCallback* callback,
+    const PrinterCapsAndDefaultsCallback& callback,
     bool succeeded,
     const std::string& printer_name,
     const printing::PrinterCapsAndDefaults& printer_info) {
-  callback->Run(succeeded, printer_name, printer_info);
-  delete callback;
+  callback.Run(succeeded, printer_name, printer_info);
 }
 
 }  // namespace cloud_print
