@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,28 +6,30 @@
 
 #include <objidl.h>
 #include <winspool.h>
+#include <xpsprint.h>
 
+#include "base/bind.h"
 #include "base/file_path.h"
-#include "base/scoped_ptr.h"
+#include "base/file_util.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/object_watcher.h"
 #include "base/win/scoped_bstr.h"
 #include "base/win/scoped_comptr.h"
 #include "base/win/scoped_hdc.h"
-#include "chrome/service/cloud_print/cloud_print_consts.h"
 #include "chrome/service/service_process.h"
 #include "chrome/service/service_utility_process_host.h"
-#include "gfx/rect.h"
 #include "grit/generated_resources.h"
 #include "printing/backend/print_backend.h"
 #include "printing/backend/print_backend_consts.h"
 #include "printing/backend/win_helper.h"
-#include "printing/native_metafile.h"
+#include "printing/emf_win.h"
 #include "printing/page_range.h"
+#include "printing/pdf_render_settings.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/gfx/rect.h"
 
-using base::win::ScopedBstr;
-using base::win::ScopedComPtr;
+#pragma comment(lib, "rpcrt4.lib")  // for UuidToString & Co.
 
 namespace {
 
@@ -84,7 +86,7 @@ HRESULT PrintTicketToDevMode(const std::string& printer_name,
     return E_FAIL;
   }
 
-  ScopedComPtr<IStream> pt_stream;
+  base::win::ScopedComPtr<IStream> pt_stream;
   HRESULT hr = StreamFromPrintTicket(print_ticket, pt_stream.Receive());
   if (FAILED(hr))
     return hr;
@@ -131,6 +133,7 @@ class PrintSystemWatcherWin : public base::win::ObjectWatcher::Delegate {
 
   class Delegate {
    public:
+    virtual ~Delegate() {}
     virtual void OnPrinterAdded() = 0;
     virtual void OnPrinterDeleted() = 0;
     virtual void OnPrinterChanged() = 0;
@@ -207,7 +210,7 @@ class PrintSystemWatcherWin : public base::win::ObjectWatcher::Delegate {
     bool ret = false;
     GetPrinter(printer_, 2, NULL, 0, &bytes_needed);
     if (0 != bytes_needed) {
-      scoped_ptr<BYTE> printer_info_buffer(new BYTE[bytes_needed]);
+      scoped_array<BYTE> printer_info_buffer(new BYTE[bytes_needed]);
       if (GetPrinter(printer_, 2, printer_info_buffer.get(),
                      bytes_needed, &bytes_needed)) {
         PRINTER_INFO_2* printer_info_win =
@@ -248,22 +251,19 @@ class PrintSystemWin : public PrintSystem {
   PrintSystemWin();
 
   // PrintSystem implementation.
-  virtual PrintSystemResult Init();
-
-  virtual void EnumeratePrinters(printing::PrinterList* printer_list);
-
+  virtual PrintSystemResult Init() OVERRIDE;
+  virtual PrintSystem::PrintSystemResult EnumeratePrinters(
+      printing::PrinterList* printer_list) OVERRIDE;
   virtual void GetPrinterCapsAndDefaults(
       const std::string& printer_name,
-      PrinterCapsAndDefaultsCallback* callback);
-
-  virtual bool IsValidPrinter(const std::string& printer_name);
-
-  virtual bool ValidatePrintTicket(const std::string& printer_name,
-                                   const std::string& print_ticket_data);
-
+      const PrinterCapsAndDefaultsCallback& callback) OVERRIDE;
+  virtual bool IsValidPrinter(const std::string& printer_name) OVERRIDE;
+  virtual bool ValidatePrintTicket(
+      const std::string& printer_name,
+      const std::string& print_ticket_data) OVERRIDE;
   virtual bool GetJobDetails(const std::string& printer_name,
                              PlatformJobId job_id,
-                             PrintJobDetails *job_details);
+                             PrintJobDetails *job_details) OVERRIDE;
 
   class PrintServerWatcherWin
     : public PrintSystem::PrintServerWatcher,
@@ -271,32 +271,31 @@ class PrintSystemWin : public PrintSystem {
    public:
     PrintServerWatcherWin() : delegate_(NULL) {}
 
-    // PrintSystem::PrintServerWatcher interface
+    // PrintSystem::PrintServerWatcher implementation.
     virtual bool StartWatching(
-        PrintSystem::PrintServerWatcher::Delegate* delegate) {
+        PrintSystem::PrintServerWatcher::Delegate* delegate) OVERRIDE{
       delegate_ = delegate;
       return watcher_.Start(std::string(), this);
     }
-    virtual bool StopWatching() {
+
+    virtual bool StopWatching() OVERRIDE{
       bool ret = watcher_.Stop();
       delegate_ = NULL;
       return ret;
     }
 
-    // PrintSystemWatcherWin::Delegate interface
-    virtual void OnPrinterAdded() {
+    // PrintSystemWatcherWin::Delegate implementation.
+    virtual void OnPrinterAdded() OVERRIDE {
       delegate_->OnPrinterAdded();
     }
-    virtual void OnPrinterDeleted() {
-    }
-    virtual void OnPrinterChanged() {
-    }
-    virtual void OnJobChanged() {
-    }
+    virtual void OnPrinterDeleted() OVERRIDE {}
+    virtual void OnPrinterChanged() OVERRIDE {}
+    virtual void OnJobChanged() OVERRIDE {}
 
    private:
     PrintSystem::PrintServerWatcher::Delegate* delegate_;
     PrintSystemWatcherWin watcher_;
+
     DISALLOW_COPY_AND_ASSIGN(PrintServerWatcherWin);
   };
 
@@ -309,33 +308,35 @@ class PrintSystemWin : public PrintSystem {
            delegate_(NULL) {
      }
 
-    // PrintSystem::PrinterWatcher interface
+    // PrintSystem::PrinterWatcher implementation.
     virtual bool StartWatching(
-        PrintSystem::PrinterWatcher::Delegate* delegate) {
+        PrintSystem::PrinterWatcher::Delegate* delegate) OVERRIDE {
       delegate_ = delegate;
       return watcher_.Start(printer_name_, this);
     }
-    virtual bool StopWatching() {
+
+    virtual bool StopWatching() OVERRIDE {
       bool ret = watcher_.Stop();
       delegate_ = NULL;
       return ret;
     }
+
     virtual bool GetCurrentPrinterInfo(
-        printing::PrinterBasicInfo* printer_info) {
+        printing::PrinterBasicInfo* printer_info) OVERRIDE {
       return watcher_.GetCurrentPrinterInfo(printer_info);
     }
 
-    // PrintSystemWatcherWin::Delegate interface
-    virtual void OnPrinterAdded() {
+    // PrintSystemWatcherWin::Delegate implementation.
+    virtual void OnPrinterAdded() OVERRIDE {
       NOTREACHED();
     }
-    virtual void OnPrinterDeleted() {
+    virtual void OnPrinterDeleted() OVERRIDE {
       delegate_->OnPrinterDeleted();
     }
-    virtual void OnPrinterChanged() {
+    virtual void OnPrinterChanged() OVERRIDE {
       delegate_->OnPrinterChanged();
     }
-    virtual void OnJobChanged() {
+    virtual void OnJobChanged() OVERRIDE {
       delegate_->OnJobChanged();
     }
 
@@ -343,12 +344,14 @@ class PrintSystemWin : public PrintSystem {
     std::string printer_name_;
     PrintSystem::PrinterWatcher::Delegate* delegate_;
     PrintSystemWatcherWin watcher_;
+
     DISALLOW_COPY_AND_ASSIGN(PrinterWatcherWin);
   };
 
   class JobSpoolerWin : public PrintSystem::JobSpooler {
    public:
     JobSpoolerWin() : core_(new Core) {}
+
     // PrintSystem::JobSpooler implementation.
     virtual bool Spool(const std::string& print_ticket,
                        const FilePath& print_data_file_path,
@@ -356,22 +359,29 @@ class PrintSystemWin : public PrintSystem {
                        const std::string& printer_name,
                        const std::string& job_title,
                        const std::vector<std::string>& tags,
-                       JobSpooler::Delegate* delegate) {
+                       JobSpooler::Delegate* delegate) OVERRIDE {
       // TODO(gene): add tags handling.
       return core_->Spool(print_ticket, print_data_file_path,
                           print_data_mime_type, printer_name, job_title,
                           delegate);
     }
+
    private:
     // We use a Core class because we want a separate RefCountedThreadSafe
     // implementation for ServiceUtilityProcessHost::Client.
-    class Core : public ServiceUtilityProcessHost::Client {
+    class Core : public ServiceUtilityProcessHost::Client,
+                 public base::win::ObjectWatcher::Delegate {
      public:
       Core()
-          : last_page_printed_(-1), job_id_(-1), delegate_(NULL), saved_dc_(0) {
-      }
-      ~Core() {
+          : last_page_printed_(-1),
+            job_id_(-1),
+            delegate_(NULL),
+            saved_dc_(0),
+            should_couninit_(false) {
       }
+
+      ~Core() {}
+
       bool Spool(const std::string& print_ticket,
                  const FilePath& print_data_file_path,
                  const std::string& print_data_mime_type,
@@ -384,54 +394,72 @@ class PrintSystemWin : public PrintSystem {
           return false;
         }
         last_page_printed_ = -1;
-        // We only support PDFs for now.
-        if (print_data_mime_type != "application/pdf") {
+        // We only support PDF and XPS documents for now.
+        if (print_data_mime_type == "application/pdf") {
+          DevMode pt_dev_mode;
+          HRESULT hr = PrintTicketToDevMode(printer_name, print_ticket,
+                                            &pt_dev_mode);
+          if (FAILED(hr)) {
+            NOTREACHED();
+            return false;
+          }
+
+          HDC dc = CreateDC(L"WINSPOOL", UTF8ToWide(printer_name).c_str(),
+                            NULL, pt_dev_mode.dm_);
+          if (!dc) {
+            NOTREACHED();
+            return false;
+          }
+          hr = E_FAIL;
+          DOCINFO di = {0};
+          di.cbSize = sizeof(DOCINFO);
+          std::wstring doc_name = UTF8ToWide(job_title);
+          di.lpszDocName = doc_name.c_str();
+          job_id_ = StartDoc(dc, &di);
+          if (job_id_ <= 0)
+            return false;
+
+          printer_dc_.Set(dc);
+          saved_dc_ = SaveDC(printer_dc_.Get());
+          print_data_file_path_ = print_data_file_path;
+          delegate_ = delegate;
+          RenderNextPDFPages();
+        } else if (print_data_mime_type == "application/vnd.ms-xpsdocument") {
+          bool ret = PrintXPSDocument(printer_name,
+                                      job_title,
+                                      print_data_file_path,
+                                      print_ticket);
+          if (ret)
+            delegate_ = delegate;
+          return ret;
+        } else {
           NOTREACHED();
           return false;
         }
-
-        DevMode pt_dev_mode;
-        HRESULT hr = PrintTicketToDevMode(printer_name, print_ticket,
-                                          &pt_dev_mode);
-        if (FAILED(hr)) {
-          NOTREACHED();
-          return false;
-        }
-
-        HDC dc = CreateDC(L"WINSPOOL", UTF8ToWide(printer_name).c_str(),
-                          NULL, pt_dev_mode.dm_);
-        if (!dc) {
-          NOTREACHED();
-          return false;
-        }
-        hr = E_FAIL;
-        DOCINFO di = {0};
-        di.cbSize = sizeof(DOCINFO);
-        std::wstring doc_name = UTF8ToWide(job_title);
-        di.lpszDocName = doc_name.c_str();
-        job_id_ = StartDoc(dc, &di);
-        if (job_id_ <= 0)
-          return false;
-
-        printer_dc_.Set(dc);
-
-        int printer_dpi = ::GetDeviceCaps(printer_dc_.Get(), LOGPIXELSX);
-        saved_dc_ = SaveDC(printer_dc_.Get());
-        SetGraphicsMode(printer_dc_.Get(), GM_ADVANCED);
-        XFORM xform = {0};
-        xform.eM11 = xform.eM22 = static_cast<float>(printer_dpi) /
-            static_cast<float>(GetDeviceCaps(GetDC(NULL), LOGPIXELSX));
-        ModifyWorldTransform(printer_dc_.Get(), &xform, MWT_LEFTMULTIPLY);
-        print_data_file_path_ = print_data_file_path;
-        delegate_ = delegate;
-        RenderNextPDFPages();
         return true;
       }
 
+      void PreparePageDCForPrinting(HDC, double scale_factor) {
+        SetGraphicsMode(printer_dc_.Get(), GM_ADVANCED);
+        // Setup the matrix to translate and scale to the right place. Take in
+        // account the scale factor.
+        // Note that the printing output is relative to printable area of
+        // the page. That is 0,0 is offset by PHYSICALOFFSETX/Y from the page.
+        int offset_x = ::GetDeviceCaps(printer_dc_.Get(), PHYSICALOFFSETX);
+        int offset_y = ::GetDeviceCaps(printer_dc_.Get(), PHYSICALOFFSETY);
+        XFORM xform = {0};
+        xform.eDx = static_cast<float>(-offset_x);
+        xform.eDy = static_cast<float>(-offset_y);
+        xform.eM11 = xform.eM22 = 1.0 / scale_factor;
+        SetWorldTransform(printer_dc_.Get(), &xform);
+      }
+
       // ServiceUtilityProcessHost::Client implementation.
       virtual void OnRenderPDFPagesToMetafileSucceeded(
-          const printing::NativeMetafile& metafile,
-          int highest_rendered_page_number) {
+          const printing::Emf& metafile,
+          int highest_rendered_page_number,
+          double scale_factor) OVERRIDE {
+        PreparePageDCForPrinting(printer_dc_.Get(), scale_factor);
         metafile.SafePlayback(printer_dc_.Get());
         bool done_printing = (highest_rendered_page_number !=
             last_page_printed_ + kPageCountPerBatch);
@@ -441,12 +469,46 @@ class PrintSystemWin : public PrintSystem {
         else
           RenderNextPDFPages();
       }
-      virtual void OnRenderPDFPagesToMetafileFailed() {
+
+      // base::win::ObjectWatcher::Delegate implementation.
+      virtual void OnObjectSignaled(HANDLE object) OVERRIDE {
+        DCHECK(xps_print_job_);
+        DCHECK(object == job_progress_event_.Get());
+        ResetEvent(job_progress_event_.Get());
+        if (!delegate_)
+          return;
+        XPS_JOB_STATUS job_status = {0};
+        xps_print_job_->GetJobStatus(&job_status);
+        bool done = false;
+        if ((job_status.completion == XPS_JOB_CANCELLED) ||
+            (job_status.completion == XPS_JOB_FAILED)) {
+          delegate_->OnJobSpoolFailed();
+          done = true;
+        } else if (job_status.jobId ||
+                   (job_status.completion == XPS_JOB_COMPLETED)) {
+          // Note: In the case of the XPS document being printed to the
+          // Microsoft XPS Document Writer, it seems to skip spooling the job
+          // and goes to the completed state without ever assigning a job id.
+          delegate_->OnJobSpoolSucceeded(job_status.jobId);
+          done = true;
+        } else {
+          job_progress_watcher_.StopWatching();
+          job_progress_watcher_.StartWatching(job_progress_event_.Get(), this);
+        }
+        if (done && should_couninit_) {
+          CoUninitialize();
+          should_couninit_ = false;
+        }
+      }
+
+      virtual void OnRenderPDFPagesToMetafileFailed() OVERRIDE {
         PrintJobDone();
       }
-      virtual void OnChildDied() {
+
+      virtual void OnChildDied() OVERRIDE {
         PrintJobDone();
       }
+
      private:
       void PrintJobDone() {
         // If there is no delegate, then there is nothing pending to process.
@@ -461,6 +523,7 @@ class PrintSystemWin : public PrintSystem {
         }
         delegate_ = NULL;
       }
+
       void RenderNextPDFPages() {
         printing::PageRange range;
         // Render 10 pages at a time.
@@ -475,15 +538,11 @@ class PrintSystemWin : public PrintSystem {
         gfx::Rect render_area(0, 0, dc_width, dc_height);
         g_service_process->io_thread()->message_loop_proxy()->PostTask(
             FROM_HERE,
-            NewRunnableMethod(
-                this,
-                &JobSpoolerWin::Core::RenderPDFPagesInSandbox,
-                print_data_file_path_,
-                render_area,
-                printer_dpi,
-                page_ranges,
-                base::MessageLoopProxy::CreateForCurrentThread()));
+            base::Bind(&JobSpoolerWin::Core::RenderPDFPagesInSandbox, this,
+                       print_data_file_path_, render_area, printer_dpi,
+                       page_ranges, base::MessageLoopProxy::current()));
       }
+
       // Called on the service process IO thread.
       void RenderPDFPagesInSandbox(
           const FilePath& pdf_path, const gfx::Rect& render_area,
@@ -494,14 +553,81 @@ class PrintSystemWin : public PrintSystem {
             BelongsToCurrentThread());
         scoped_ptr<ServiceUtilityProcessHost> utility_host(
             new ServiceUtilityProcessHost(this, client_message_loop_proxy));
-        if (utility_host->StartRenderPDFPagesToMetafile(pdf_path,
-                                                        render_area,
-                                                        render_dpi,
-                                                        page_ranges)) {
+        // TODO(gene): For now we disabling autorotation for CloudPrinting.
+        // Landscape/Portrait setting is passed in the print ticket and
+        // server is generating portrait PDF always.
+        // We should enable autorotation once server will be able to generate
+        // PDF that matches paper size and orientation.
+        if (utility_host->StartRenderPDFPagesToMetafile(
+                pdf_path,
+                printing::PdfRenderSettings(render_area, render_dpi, false),
+                page_ranges)) {
           // The object will self-destruct when the child process dies.
           utility_host.release();
         }
       }
+
+      bool PrintXPSDocument(const std::string& printer_name,
+                            const std::string& job_title,
+                            const FilePath& print_data_file_path,
+                            const std::string& print_ticket) {
+        if (!printing::XPSPrintModule::Init())
+          return false;
+        job_progress_event_.Set(CreateEvent(NULL, TRUE, FALSE, NULL));
+        if (!job_progress_event_.Get())
+          return false;
+        should_couninit_ = SUCCEEDED(CoInitializeEx(NULL,
+                                                    COINIT_MULTITHREADED));
+        base::win::ScopedComPtr<IXpsPrintJobStream> doc_stream;
+        base::win::ScopedComPtr<IXpsPrintJobStream> print_ticket_stream;
+        bool ret = false;
+        // Use nested SUCCEEDED checks because we want a common return point.
+        if (SUCCEEDED(printing::XPSPrintModule::StartXpsPrintJob(
+                UTF8ToWide(printer_name).c_str(),
+                UTF8ToWide(job_title).c_str(),
+                NULL,
+                job_progress_event_.Get(),
+                NULL,
+                NULL,
+                NULL,
+                xps_print_job_.Receive(),
+                doc_stream.Receive(),
+                print_ticket_stream.Receive()))) {
+          ULONG bytes_written = 0;
+          if (SUCCEEDED(print_ticket_stream->Write(print_ticket.c_str(),
+                                                   print_ticket.length(),
+                                                   &bytes_written))) {
+            DCHECK(bytes_written == print_ticket.length());
+            if (SUCCEEDED(print_ticket_stream->Close())) {
+              std::string document_data;
+              file_util::ReadFileToString(print_data_file_path, &document_data);
+              bytes_written = 0;
+              if (SUCCEEDED(doc_stream->Write(document_data.c_str(),
+                                              document_data.length(),
+                                              &bytes_written))) {
+                DCHECK(bytes_written == document_data.length());
+                if (SUCCEEDED(doc_stream->Close())) {
+                  job_progress_watcher_.StartWatching(job_progress_event_.Get(),
+                                                      this);
+                  ret = true;
+                }
+              }
+            }
+          }
+        }
+        if (!ret) {
+          if (xps_print_job_) {
+            xps_print_job_->Cancel();
+            xps_print_job_.Release();
+          }
+          if (should_couninit_) {
+            CoUninitialize();
+            should_couninit_ = false;
+          }
+        }
+        return ret;
+      }
+
       // Some Cairo-generated PDFs from Chrome OS result in huge metafiles.
       // So the PageCountPerBatch is set to 1 for now.
       // TODO(sanjeevr): Figure out a smarter way to determine the pages per
@@ -512,11 +638,17 @@ class PrintSystemWin : public PrintSystem {
       PlatformJobId job_id_;
       PrintSystem::JobSpooler::Delegate* delegate_;
       int saved_dc_;
-      base::win::ScopedHDC printer_dc_;
+      base::win::ScopedCreateDC printer_dc_;
       FilePath print_data_file_path_;
+      base::win::ScopedHandle job_progress_event_;
+      base::win::ObjectWatcher job_progress_watcher_;
+      base::win::ScopedComPtr<IXpsPrintJob> xps_print_job_;
+      bool should_couninit_;
+
       DISALLOW_COPY_AND_ASSIGN(JobSpoolerWin::Core);
     };
     scoped_refptr<Core> core_;
+
     DISALLOW_COPY_AND_ASSIGN(JobSpoolerWin);
   };
 
@@ -526,36 +658,38 @@ class PrintSystemWin : public PrintSystem {
    public:
     PrinterCapsHandler(
         const std::string& printer_name,
-        PrinterCapsAndDefaultsCallback* callback)
+        const PrinterCapsAndDefaultsCallback& callback)
             : printer_name_(printer_name), callback_(callback) {
     }
-    virtual void Start() {
-      g_service_process->io_thread()->message_loop_proxy()->PostTask(
-          FROM_HERE,
-          NewRunnableMethod(
-              this,
-              &PrinterCapsHandler::GetPrinterCapsAndDefaultsImpl,
-              base::MessageLoopProxy::CreateForCurrentThread()));
-    }
 
-    virtual void OnChildDied() {
+    // ServiceUtilityProcessHost::Client implementation.
+    virtual void OnChildDied() OVERRIDE {
       OnGetPrinterCapsAndDefaultsFailed(printer_name_);
     }
+
     virtual void OnGetPrinterCapsAndDefaultsSucceeded(
         const std::string& printer_name,
-        const printing::PrinterCapsAndDefaults& caps_and_defaults) {
-      callback_->Run(true, printer_name, caps_and_defaults);
-      callback_.reset();
+        const printing::PrinterCapsAndDefaults& caps_and_defaults) OVERRIDE {
+      callback_.Run(true, printer_name, caps_and_defaults);
+      callback_.Reset();
       Release();
     }
 
     virtual void OnGetPrinterCapsAndDefaultsFailed(
-        const std::string& printer_name) {
+        const std::string& printer_name) OVERRIDE {
       printing::PrinterCapsAndDefaults caps_and_defaults;
-      callback_->Run(false, printer_name, caps_and_defaults);
-      callback_.reset();
+      callback_.Run(false, printer_name, caps_and_defaults);
+      callback_.Reset();
       Release();
     }
+
+    void Start() {
+      g_service_process->io_thread()->message_loop_proxy()->PostTask(
+          FROM_HERE,
+          base::Bind(&PrinterCapsHandler::GetPrinterCapsAndDefaultsImpl, this,
+                     base::MessageLoopProxy::current()));
+    }
+
    private:
       // Called on the service process IO thread.
     void GetPrinterCapsAndDefaultsImpl(
@@ -571,22 +705,21 @@ class PrintSystemWin : public PrintSystem {
       } else {
         client_message_loop_proxy->PostTask(
             FROM_HERE,
-            NewRunnableMethod(
-                this,
-                &PrinterCapsHandler::OnGetPrinterCapsAndDefaultsFailed,
-                printer_name_));
+            base::Bind(&PrinterCapsHandler::OnGetPrinterCapsAndDefaultsFailed,
+                       this, printer_name_));
       }
     }
 
     std::string printer_name_;
-    scoped_ptr<PrinterCapsAndDefaultsCallback> callback_;
+    PrinterCapsAndDefaultsCallback callback_;
   };
 
 
-  virtual PrintSystem::PrintServerWatcher* CreatePrintServerWatcher();
+  virtual PrintSystem::PrintServerWatcher* CreatePrintServerWatcher() OVERRIDE;
   virtual PrintSystem::PrinterWatcher* CreatePrinterWatcher(
-      const std::string& printer_name);
-  virtual PrintSystem::JobSpooler* CreateJobSpooler();
+      const std::string& printer_name) OVERRIDE;
+  virtual PrintSystem::JobSpooler* CreateJobSpooler() OVERRIDE;
+  virtual std::string GetSupportedMimeTypes() OVERRIDE;
 
  private:
   scoped_refptr<printing::PrintBackend> print_backend_;
@@ -598,20 +731,23 @@ PrintSystemWin::PrintSystemWin() {
 
 PrintSystem::PrintSystemResult PrintSystemWin::Init() {
   if (!printing::XPSModule::Init()) {
-    std::string message = l10n_util::GetStringUTF8(
-        IDS_CLOUD_PRINT_XPS_UNAVAILABLE);
+    std::string message = l10n_util::GetStringFUTF8(
+        IDS_CLOUD_PRINT_XPS_UNAVAILABLE,
+        l10n_util::GetStringUTF16(IDS_GOOGLE_CLOUD_PRINT));
     return PrintSystemResult(false, message);
   }
   return PrintSystemResult(true, std::string());
 }
 
-void PrintSystemWin::EnumeratePrinters(printing::PrinterList* printer_list) {
-  print_backend_->EnumeratePrinters(printer_list);
+PrintSystem::PrintSystemResult PrintSystemWin::EnumeratePrinters(
+    printing::PrinterList* printer_list) {
+  bool ret = print_backend_->EnumeratePrinters(printer_list);
+  return PrintSystemResult(ret, std::string());
 }
 
 void PrintSystemWin::GetPrinterCapsAndDefaults(
     const std::string& printer_name,
-    PrinterCapsAndDefaultsCallback* callback) {
+    const PrinterCapsAndDefaultsCallback& callback) {
   // Launch as child process to retrieve the capabilities and defaults because
   // this involves invoking a printer driver DLL and crashes have been known to
   // occur.
@@ -639,7 +775,7 @@ bool PrintSystemWin::ValidatePrintTicket(
                                     1,
                                     &provider);
   if (provider) {
-    ScopedComPtr<IStream> print_ticket_stream;
+    base::win::ScopedComPtr<IStream> print_ticket_stream;
     CreateStreamOnHGlobal(NULL, TRUE, print_ticket_stream.Receive());
     ULONG bytes_written = 0;
     print_ticket_stream->Write(print_ticket_data.c_str(),
@@ -649,8 +785,8 @@ bool PrintSystemWin::ValidatePrintTicket(
     LARGE_INTEGER pos = {0};
     ULARGE_INTEGER new_pos = {0};
     print_ticket_stream->Seek(pos, STREAM_SEEK_SET, &new_pos);
-    ScopedBstr error;
-    ScopedComPtr<IStream> result_ticket_stream;
+    base::win::ScopedBstr error;
+    base::win::ScopedComPtr<IStream> result_ticket_stream;
     CreateStreamOnHGlobal(NULL, TRUE, result_ticket_stream.Receive());
     ret = SUCCEEDED(printing::XPSModule::MergeAndValidatePrintTicket(
         provider,
@@ -665,8 +801,8 @@ bool PrintSystemWin::ValidatePrintTicket(
 }
 
 bool PrintSystemWin::GetJobDetails(const std::string& printer_name,
-                                       PlatformJobId job_id,
-                                       PrintJobDetails *job_details) {
+                                   PlatformJobId job_id,
+                                   PrintJobDetails *job_details) {
   DCHECK(job_details);
   HANDLE printer_handle = NULL;
   std::wstring printer_name_wide = UTF8ToWide(printer_name);
@@ -681,7 +817,7 @@ bool PrintSystemWin::GetJobDetails(const std::string& printer_name,
     if (ERROR_INVALID_PARAMETER != last_error) {
       // ERROR_INVALID_PARAMETER normally means that the job id is not valid.
       DCHECK(last_error == ERROR_INSUFFICIENT_BUFFER);
-      scoped_ptr<BYTE> job_info_buffer(new BYTE[bytes_needed]);
+      scoped_array<BYTE> job_info_buffer(new BYTE[bytes_needed]);
       if (GetJob(printer_handle, job_id, 1, job_info_buffer.get(), bytes_needed,
                 &bytes_needed)) {
         JOB_INFO_1 *job_info =
@@ -725,6 +861,13 @@ PrintSystemWin::CreateJobSpooler() {
   return new JobSpoolerWin();
 }
 
+std::string PrintSystemWin::GetSupportedMimeTypes() {
+  if (printing::XPSPrintModule::Init())
+    return "application/vnd.ms-xpsdocument,application/pdf";
+  return "application/pdf";
+}
+
+
 std::string PrintSystem::GenerateProxyId() {
   GUID proxy_id = {0};
   HRESULT hr = UuidCreate(&proxy_id);
@@ -739,7 +882,7 @@ std::string PrintSystem::GenerateProxyId() {
 }
 
 scoped_refptr<PrintSystem> PrintSystem::CreateInstance(
-    const DictionaryValue* print_system_settings) {
+    const base::DictionaryValue* print_system_settings) {
   return new PrintSystemWin;
 }
 
