@@ -1,13 +1,15 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/service/cloud_print/printer_job_handler.h"
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/file_util.h"
 #include "base/json/json_reader.h"
 #include "base/md5.h"
-#include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/common/net/http_return.h"
@@ -33,14 +35,12 @@ void PrinterJobHandler::JobDetails::Clear() {
 PrinterJobHandler::PrinterJobHandler(
     const printing::PrinterBasicInfo& printer_info,
     const PrinterInfoFromCloud& printer_info_cloud,
-    const std::string& auth_token,
     const GURL& cloud_print_server_url,
     cloud_print::PrintSystem* print_system,
     Delegate* delegate)
     : print_system_(print_system),
       printer_info_(printer_info),
       printer_info_cloud_(printer_info_cloud),
-      auth_token_(auth_token),
       cloud_print_server_url_(cloud_print_server_url),
       delegate_(delegate),
       local_job_id_(-1),
@@ -49,25 +49,21 @@ PrinterJobHandler::PrinterJobHandler(
       server_error_count_(0),
       print_thread_("Chrome_CloudPrintJobPrintThread"),
       job_handler_message_loop_proxy_(
-          base::MessageLoopProxy::CreateForCurrentThread()),
+          base::MessageLoopProxy::current()),
       shutting_down_(false),
       job_check_pending_(false),
       printer_update_pending_(true),
-      printer_delete_pending_(false),
       task_in_progress_(false) {
 }
 
 bool PrinterJobHandler::Initialize() {
-  if (print_system_->IsValidPrinter(
-      printer_info_.printer_name)) {
-    printer_watcher_ = print_system_->CreatePrinterWatcher(
-        printer_info_.printer_name);
-    printer_watcher_->StartWatching(this);
-    CheckForJobs(kJobFetchReasonStartup);
-  } else {
-    // This printer does not exist any more. Delete it from the server.
-    OnPrinterDeleted();
-  }
+  if (!print_system_->IsValidPrinter(printer_info_.printer_name))
+    return false;
+
+  printer_watcher_ = print_system_->CreatePrinterWatcher(
+      printer_info_.printer_name);
+  printer_watcher_->StartWatching(this);
+  CheckForJobs(kJobFetchReasonStartup);
   return true;
 }
 
@@ -83,8 +79,12 @@ void PrinterJobHandler::Reset() {
   print_thread_.Stop();
 }
 
+std::string PrinterJobHandler::GetPrinterName() const {
+  return printer_info_.printer_name;
+}
+
 void PrinterJobHandler::Start() {
-  VLOG(1) << "CP_PROXY: Start printer job handler, id: "
+  VLOG(1) << "CP_CONNECTOR: Start printer job handler, id: "
           << printer_info_cloud_.printer_id
           << ", task in progress: " << task_in_progress_;
   if (task_in_progress_) {
@@ -96,16 +96,6 @@ void PrinterJobHandler::Start() {
   if (!shutting_down_) {
     // Check if we have work to do.
     if (HavePendingTasks()) {
-      if (printer_delete_pending_) {
-        printer_delete_pending_ = false;
-        task_in_progress_ = true;
-        SetNextJSONHandler(&PrinterJobHandler::HandlePrinterDeleteResponse);
-        request_ = new CloudPrintURLFetcher;
-        request_->StartGetRequest(
-            CloudPrintHelpers::GetUrlForPrinterDelete(
-                cloud_print_server_url_, printer_info_cloud_.printer_id),
-            this, auth_token_, kCloudPrintAPIMaxRetryCount);
-      }
       if (!task_in_progress_ && printer_update_pending_) {
         printer_update_pending_ = false;
         task_in_progress_ = UpdatePrinterInfo();
@@ -120,7 +110,9 @@ void PrinterJobHandler::Start() {
             CloudPrintHelpers::GetUrlForJobFetch(
                 cloud_print_server_url_, printer_info_cloud_.printer_id,
                 job_fetch_reason_),
-            this, auth_token_, kCloudPrintAPIMaxRetryCount);
+            this,
+            kCloudPrintAPIMaxRetryCount,
+            std::string());
         last_job_fetch_time_ = base::TimeTicks::Now();
         VLOG(1) << "Last job fetch time for printer "
                 << printer_info_.printer_name.c_str() << " is "
@@ -132,18 +124,18 @@ void PrinterJobHandler::Start() {
 }
 
 void PrinterJobHandler::Stop() {
-  VLOG(1) << "CP_PROXY: Stop printer job handler, id: "
+  VLOG(1) << "CP_CONNECTOR: Stop printer job handler, id: "
           << printer_info_cloud_.printer_id;
   task_in_progress_ = false;
   Reset();
   if (HavePendingTasks()) {
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Start));
+        FROM_HERE, base::Bind(&PrinterJobHandler::Start, this));
   }
 }
 
 void PrinterJobHandler::CheckForJobs(const std::string& reason) {
-  VLOG(1) << "CP_PROXY: CheckForJobs, id: "
+  VLOG(1) << "CP_CONNECTOR: CheckForJobs, id: "
           << printer_info_cloud_.printer_id
           << ", reason: " << reason
           << ", task in progress: " << task_in_progress_;
@@ -151,25 +143,33 @@ void PrinterJobHandler::CheckForJobs(const std::string& reason) {
   job_check_pending_ = true;
   if (!task_in_progress_) {
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Start));
+        FROM_HERE, base::Bind(&PrinterJobHandler::Start, this));
   }
 }
 
 bool PrinterJobHandler::UpdatePrinterInfo() {
-  VLOG(1) << "CP_PROXY: Update printer info, id: "
+  if (!printer_watcher_) {
+    LOG(ERROR) << "CP_CONNECTOR: Printer watcher is missing."
+               << "Check printer server url for printer id: "
+               << printer_info_cloud_.printer_id;
+    return false;
+  }
+
+  VLOG(1) << "CP_CONNECTOR: Update printer info, id: "
           << printer_info_cloud_.printer_id;
   // We need to update the parts of the printer info that have changed
   // (could be printer name, description, status or capabilities).
   // First asynchronously fetch the capabilities.
   printing::PrinterBasicInfo printer_info;
   printer_watcher_->GetCurrentPrinterInfo(&printer_info);
-  cloud_print::PrintSystem::PrinterCapsAndDefaultsCallback* callback =
-       NewCallback(this,
-                   &PrinterJobHandler::OnReceivePrinterCaps);
-  // Asnchronously fetch the printer caps and defaults. The story will
+
+  // Asynchronously fetch the printer caps and defaults. The story will
   // continue in OnReceivePrinterCaps.
   print_system_->GetPrinterCapsAndDefaults(
-      printer_info.printer_name.c_str(), callback);
+      printer_info.printer_name.c_str(),
+      base::Bind(&PrinterJobHandler::OnReceivePrinterCaps,
+                 base::Unretained(this)));
+
   // While we are waiting for the data, pretend we have work to do and return
   // true.
   return true;
@@ -180,14 +180,16 @@ void PrinterJobHandler::OnReceivePrinterCaps(
     const std::string& printer_name,
     const printing::PrinterCapsAndDefaults& caps_and_defaults) {
   printing::PrinterBasicInfo printer_info;
-  printer_watcher_->GetCurrentPrinterInfo(&printer_info);
+  if (printer_watcher_)
+    printer_watcher_->GetCurrentPrinterInfo(&printer_info);
 
   std::string post_data;
   std::string mime_boundary;
   CloudPrintHelpers::CreateMimeBoundaryForUpload(&mime_boundary);
 
   if (succeeded) {
-    std::string caps_hash = MD5String(caps_and_defaults.printer_capabilities);
+    std::string caps_hash =
+        base::MD5String(caps_and_defaults.printer_capabilities);
     if (caps_hash != printer_info_cloud_.caps_hash) {
       // Hashes don't match, we need to upload new capabilities (the defaults
       // go for free along with the capabilities)
@@ -235,8 +237,11 @@ void PrinterJobHandler::OnReceivePrinterCaps(
   }
   if (printer_info.printer_status != printer_info_.printer_status) {
     CloudPrintHelpers::AddMultipartValueForUpload(
-        kPrinterStatusValue, StringPrintf("%d", printer_info.printer_status),
-        mime_boundary, std::string(), &post_data);
+        kPrinterStatusValue,
+        base::StringPrintf("%d", printer_info.printer_status),
+        mime_boundary,
+        std::string(),
+        &post_data);
   }
   printer_info_ = printer_info;
   if (!post_data.empty()) {
@@ -249,17 +254,40 @@ void PrinterJobHandler::OnReceivePrinterCaps(
     request_->StartPostRequest(
         CloudPrintHelpers::GetUrlForPrinterUpdate(
             cloud_print_server_url_, printer_info_cloud_.printer_id),
-        this, auth_token_, kCloudPrintAPIMaxRetryCount, mime_type, post_data);
+        this,
+        kCloudPrintAPIMaxRetryCount,
+        mime_type,
+        post_data,
+        std::string());
   } else {
     // We are done here. Go to the Stop state
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Stop));
+        FROM_HERE, base::Bind(&PrinterJobHandler::Stop, this));
   }
 }
 
 // CloudPrintURLFetcher::Delegate implementation.
+CloudPrintURLFetcher::ResponseAction PrinterJobHandler::HandleRawResponse(
+    const content::URLFetcher* source,
+    const GURL& url,
+    const net::URLRequestStatus& status,
+    int response_code,
+    const net::ResponseCookies& cookies,
+    const std::string& data) {
+  // 415 (Unsupported media type) error while fetching data from the server
+  // means data conversion error. Stop fetching process and mark job as error.
+  if (next_data_handler_ == (&PrinterJobHandler::HandlePrintDataResponse) &&
+      response_code == RC_UNSUPPORTED_MEDIA_TYPE) {
+    MessageLoop::current()->PostTask(
+        FROM_HERE,
+        base::Bind(&PrinterJobHandler::JobFailed, this, JOB_DOWNLOAD_FAILED));
+    return CloudPrintURLFetcher::STOP_PROCESSING;
+  }
+  return CloudPrintURLFetcher::CONTINUE_PROCESSING;
+}
+
 CloudPrintURLFetcher::ResponseAction PrinterJobHandler::HandleRawData(
-    const URLFetcher* source,
+    const content::URLFetcher* source,
     const GURL& url,
     const std::string& data) {
   if (!next_data_handler_)
@@ -268,25 +296,33 @@ CloudPrintURLFetcher::ResponseAction PrinterJobHandler::HandleRawData(
 }
 
 CloudPrintURLFetcher::ResponseAction PrinterJobHandler::HandleJSONData(
-    const URLFetcher* source,
+    const content::URLFetcher* source,
     const GURL& url,
     DictionaryValue* json_data,
     bool succeeded) {
   DCHECK(next_json_data_handler_);
-  return (this->*next_json_data_handler_)(source,
-                                          url,
-                                          json_data,
-                                          succeeded);
+  return (this->*next_json_data_handler_)(source, url, json_data, succeeded);
 }
 
 void PrinterJobHandler::OnRequestGiveUp() {
+  // The only time we call CloudPrintURLFetcher::StartGetRequest() with a
+  // specified number of retries, is when we are trying to fetch print job
+  // data. So, this function will be reached only if we failed to get job data.
+  // In that case, we should make job as error and should not try it later.
   MessageLoop::current()->PostTask(
       FROM_HERE,
-      NewRunnableMethod(this, &PrinterJobHandler::Stop));
+      base::Bind(&PrinterJobHandler::JobFailed, this, JOB_DOWNLOAD_FAILED));
 }
 
-void PrinterJobHandler::OnRequestAuthError() {
+CloudPrintURLFetcher::ResponseAction PrinterJobHandler::OnRequestAuthError() {
+  // TODO(gene): We might consider stop processing if we get auth error here.
   OnAuthError();
+  // Continue processing as a network error.
+  return CloudPrintURLFetcher::CONTINUE_PROCESSING;
+}
+
+std::string PrinterJobHandler::GetAuthHeader() {
+  return CloudPrintHelpers::GetCloudPrintAuthHeader();
 }
 
 // JobStatusUpdater::Delegate implementation
@@ -309,18 +345,15 @@ void PrinterJobHandler::OnAuthError() {
 }
 
 void PrinterJobHandler::OnPrinterDeleted() {
-  printer_delete_pending_ = true;
-  if (!task_in_progress_) {
-    MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Start));
-  }
+  if (delegate_)
+    delegate_->OnPrinterDeleted(printer_info_cloud_.printer_id);
 }
 
 void PrinterJobHandler::OnPrinterChanged() {
   printer_update_pending_ = true;
   if (!task_in_progress_) {
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Start));
+        FROM_HERE, base::Bind(&PrinterJobHandler::Start, this));
   }
 }
 
@@ -330,53 +363,37 @@ void PrinterJobHandler::OnJobChanged() {
   for (JobStatusUpdaterList::iterator index = job_status_updater_list_.begin();
        index != job_status_updater_list_.end(); index++) {
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(index->get(),
-                                     &JobStatusUpdater::UpdateStatus));
+        FROM_HERE, base::Bind(&JobStatusUpdater::UpdateStatus, index->get()));
   }
 }
 
 // Begin Response handlers
 CloudPrintURLFetcher::ResponseAction
 PrinterJobHandler::HandlePrinterUpdateResponse(
-    const URLFetcher* source,
+    const content::URLFetcher* source,
     const GURL& url,
     DictionaryValue* json_data,
     bool succeeded) {
-  VLOG(1) << "CP_PROXY: Handle printer update response, id: "
+  VLOG(1) << "CP_CONNECTOR: Handle printer update response, id: "
           << printer_info_cloud_.printer_id;
   // We are done here. Go to the Stop state
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Stop));
-  return CloudPrintURLFetcher::STOP_PROCESSING;
-}
-
-CloudPrintURLFetcher::ResponseAction
-PrinterJobHandler::HandlePrinterDeleteResponse(
-    const URLFetcher* source,
-    const GURL& url,
-    DictionaryValue* json_data,
-    bool succeeded) {
-  VLOG(1) << "CP_PROXY: Handler printer delete response, id: "
-          << printer_info_cloud_.printer_id;
-  // The printer has been deleted. Shutdown the handler class.
-  MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Shutdown));
+      FROM_HERE, base::Bind(&PrinterJobHandler::Stop, this));
   return CloudPrintURLFetcher::STOP_PROCESSING;
 }
 
 CloudPrintURLFetcher::ResponseAction
 PrinterJobHandler::HandleJobMetadataResponse(
-    const URLFetcher* source,
+    const content::URLFetcher* source,
     const GURL& url,
     DictionaryValue* json_data,
     bool succeeded) {
-  VLOG(1) << "CP_PROXY: Handle job metadata response, id: "
+  VLOG(1) << "CP_CONNECTOR: Handle job metadata response, id: "
           << printer_info_cloud_.printer_id;
   bool job_available = false;
   if (succeeded) {
     ListValue* job_list = NULL;
-    json_data->GetList(kJobListValue, &job_list);
-    if (job_list) {
+    if (json_data->GetList(kJobListValue, &job_list) && job_list) {
       // Even though it is a job list, for now we are only interested in the
       // first job
       DictionaryValue* job_data = NULL;
@@ -402,32 +419,34 @@ PrinterJobHandler::HandleJobMetadataResponse(
         request_ = new CloudPrintURLFetcher;
         request_->StartGetRequest(GURL(print_ticket_url.c_str()),
                                   this,
-                                  auth_token_,
-                                  kCloudPrintAPIMaxRetryCount);
+                                  kCloudPrintAPIMaxRetryCount,
+                                  std::string());
       }
     }
   }
   // If no jobs are available, go to the Stop state.
   if (!job_available)
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Stop));
+        FROM_HERE, base::Bind(&PrinterJobHandler::Stop, this));
   return CloudPrintURLFetcher::STOP_PROCESSING;
 }
 
 CloudPrintURLFetcher::ResponseAction
-PrinterJobHandler::HandlePrintTicketResponse(const URLFetcher* source,
+PrinterJobHandler::HandlePrintTicketResponse(const content::URLFetcher* source,
                                              const GURL& url,
                                              const std::string& data) {
-  VLOG(1) << "CP_PROXY: Handle print ticket response, id: "
+  VLOG(1) << "CP_CONNECTOR: Handle print ticket response, id: "
           << printer_info_cloud_.printer_id;
   if (print_system_->ValidatePrintTicket(printer_info_.printer_name, data)) {
     job_details_.print_ticket_ = data;
     SetNextDataHandler(&PrinterJobHandler::HandlePrintDataResponse);
     request_ = new CloudPrintURLFetcher;
+    std::string accept_headers = "Accept: ";
+    accept_headers += print_system_->GetSupportedMimeTypes();
     request_->StartGetRequest(GURL(print_data_url_.c_str()),
                               this,
-                              auth_token_,
-                              kJobDataMaxRetryCount);
+                              kJobDataMaxRetryCount,
+                              accept_headers);
   } else {
     // The print ticket was not valid. We are done here.
     FailedFetchingJobData();
@@ -436,28 +455,28 @@ PrinterJobHandler::HandlePrintTicketResponse(const URLFetcher* source,
 }
 
 CloudPrintURLFetcher::ResponseAction
-PrinterJobHandler::HandlePrintDataResponse(const URLFetcher* source,
+PrinterJobHandler::HandlePrintDataResponse(const content::URLFetcher* source,
                                            const GURL& url,
                                            const std::string& data) {
-  VLOG(1) << "CP_PROXY: Handle print data response, id: "
+  VLOG(1) << "CP_CONNECTOR: Handle print data response, id: "
           << printer_info_cloud_.printer_id;
-  Task* next_task = NULL;
+  base::Closure next_task;
   if (file_util::CreateTemporaryFile(&job_details_.print_data_file_path_)) {
     int ret = file_util::WriteFile(job_details_.print_data_file_path_,
                                    data.c_str(),
                                    data.length());
-    source->response_headers()->GetMimeType(
+    source->GetResponseHeaders()->GetMimeType(
         &job_details_.print_data_mime_type_);
     DCHECK(ret == static_cast<int>(data.length()));
     if (ret == static_cast<int>(data.length())) {
-      next_task = NewRunnableMethod(this, &PrinterJobHandler::StartPrinting);
+      next_task = base::Bind(&PrinterJobHandler::StartPrinting, this);
     }
   }
   // If there was no task allocated above, then there was an error in
   // saving the print data, bail out here.
-  if (!next_task) {
-    next_task = NewRunnableMethod(this, &PrinterJobHandler::JobFailed,
-                                  JOB_DOWNLOAD_FAILED);
+  if (next_task.is_null()) {
+    next_task = base::Bind(&PrinterJobHandler::JobFailed, this,
+                           JOB_DOWNLOAD_FAILED);
   }
   MessageLoop::current()->PostTask(FROM_HERE, next_task);
   return CloudPrintURLFetcher::STOP_PROCESSING;
@@ -465,47 +484,48 @@ PrinterJobHandler::HandlePrintDataResponse(const URLFetcher* source,
 
 CloudPrintURLFetcher::ResponseAction
 PrinterJobHandler::HandleSuccessStatusUpdateResponse(
-    const URLFetcher* source,
+    const content::URLFetcher* source,
     const GURL& url,
     DictionaryValue* json_data,
     bool succeeded) {
-  VLOG(1) << "CP_PROXY: Handle success status update response, id: "
+  VLOG(1) << "CP_CONNECTOR: Handle success status update response, id: "
           << printer_info_cloud_.printer_id;
   // The print job has been spooled locally. We now need to create an object
   // that monitors the status of the job and updates the server.
   scoped_refptr<JobStatusUpdater> job_status_updater(
       new JobStatusUpdater(printer_info_.printer_name, job_details_.job_id_,
-                           local_job_id_, auth_token_, cloud_print_server_url_,
+                           local_job_id_, cloud_print_server_url_,
                            print_system_.get(), this));
   job_status_updater_list_.push_back(job_status_updater);
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableMethod(job_status_updater.get(),
-                                   &JobStatusUpdater::UpdateStatus));
+      FROM_HERE, base::Bind(&JobStatusUpdater::UpdateStatus,
+                            job_status_updater.get()));
   if (succeeded) {
     // Since we just printed successfully, we want to look for more jobs.
     CheckForJobs(kJobFetchReasonQueryMore);
   }
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Stop));
+      FROM_HERE, base::Bind(&PrinterJobHandler::Stop, this));
   return CloudPrintURLFetcher::STOP_PROCESSING;
 }
 
 CloudPrintURLFetcher::ResponseAction
 PrinterJobHandler::HandleFailureStatusUpdateResponse(
-    const URLFetcher* source,
+    const content::URLFetcher* source,
     const GURL& url,
     DictionaryValue* json_data,
     bool succeeded) {
-  VLOG(1) << "CP_PROXY: Handle failure status update response, id: "
+  VLOG(1) << "CP_CONNECTOR: Handle failure status update response, id: "
           << printer_info_cloud_.printer_id;
   MessageLoop::current()->PostTask(
-      FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::Stop));
+      FROM_HERE, base::Bind(&PrinterJobHandler::Stop, this));
   return CloudPrintURLFetcher::STOP_PROCESSING;
 }
 // End Response handlers
 
 void PrinterJobHandler::StartPrinting() {
-  VLOG(1) << "CP_PROXY: Start printing, id: " << printer_info_cloud_.printer_id;
+  VLOG(1) << "CP_CONNECTOR: Start printing, id: "
+          << printer_info_cloud_.printer_id;
   // We are done with the request object for now.
   request_ = NULL;
   if (!shutting_down_) {
@@ -513,22 +533,21 @@ void PrinterJobHandler::StartPrinting() {
       JobFailed(PRINT_FAILED);
     } else {
       print_thread_.message_loop()->PostTask(
-          FROM_HERE, NewRunnableMethod(this, &PrinterJobHandler::DoPrint,
-                                       job_details_,
-                                       printer_info_.printer_name));
+          FROM_HERE, base::Bind(&PrinterJobHandler::DoPrint, this, job_details_,
+                                printer_info_.printer_name));
     }
   }
 }
 
 void PrinterJobHandler::JobFailed(PrintJobError error) {
-  VLOG(1) << "CP_PROXY: Job failed, id: " << printer_info_cloud_.printer_id;
+  VLOG(1) << "CP_CONNECTOR: Job failed, id: " << printer_info_cloud_.printer_id;
   if (!shutting_down_) {
     UpdateJobStatus(cloud_print::PRINT_JOB_STATUS_ERROR, error);
   }
 }
 
 void PrinterJobHandler::JobSpooled(cloud_print::PlatformJobId local_job_id) {
-  VLOG(1) << "CP_PROXY: Job spooled, printer id: "
+  VLOG(1) << "CP_CONNECTOR: Job spooled, printer id: "
           << printer_info_cloud_.printer_id << ", job id: " << local_job_id;
   if (!shutting_down_) {
     local_job_id_ = local_job_id;
@@ -538,7 +557,7 @@ void PrinterJobHandler::JobSpooled(cloud_print::PlatformJobId local_job_id) {
 }
 
 void PrinterJobHandler::Shutdown() {
-  VLOG(1) << "CP_PROXY: Printer job handler shutdown, id: "
+  VLOG(1) << "CP_CONNECTOR: Printer job handler shutdown, id: "
           << printer_info_cloud_.printer_id;
   Reset();
   shutting_down_ = true;
@@ -547,20 +566,16 @@ void PrinterJobHandler::Shutdown() {
     // remove the updater object from the list.
     job_status_updater_list_.front()->Stop();
   }
-  if (delegate_) {
-    delegate_->OnPrinterJobHandlerShutdown(this,
-                                           printer_info_cloud_.printer_id);
-  }
 }
 
 void PrinterJobHandler::UpdateJobStatus(cloud_print::PrintJobStatus status,
                                         PrintJobError error) {
-  VLOG(1) << "CP_PROXY: Update job status, id: "
+  VLOG(1) << "CP_CONNECTOR: Update job status, id: "
           << printer_info_cloud_.printer_id;
   if (!shutting_down_) {
     if (!job_details_.job_id_.empty()) {
-      VLOG(1) << "CP_PROXY: Updating status, job id: " << job_details_.job_id_
-              << ", status: " << status;
+      VLOG(1) << "CP_CONNECTOR: Updating status, job id: "
+              << job_details_.job_id_ << ", status: " << status;
       if (error == SUCCESS) {
         SetNextJSONHandler(
             &PrinterJobHandler::HandleSuccessStatusUpdateResponse);
@@ -574,8 +589,8 @@ void PrinterJobHandler::UpdateJobStatus(cloud_print::PrintJobStatus status,
                                                       job_details_.job_id_,
                                                       status),
           this,
-          auth_token_,
-          kCloudPrintAPIMaxRetryCount);
+          kCloudPrintAPIMaxRetryCount,
+          std::string());
     }
   }
 }
@@ -591,13 +606,12 @@ void PrinterJobHandler::SetNextDataHandler(DataHandler handler) {
 }
 
 bool PrinterJobHandler::HavePendingTasks() {
-  return (job_check_pending_ || printer_update_pending_ ||
-          printer_delete_pending_);
+  return (job_check_pending_ || printer_update_pending_);
 }
 
 void PrinterJobHandler::FailedFetchingJobData() {
   if (!shutting_down_) {
-    LOG(ERROR) << "CP_PROXY: Failed fetching job data for printer: " <<
+    LOG(ERROR) << "CP_CONNECTOR: Failed fetching job data for printer: " <<
         printer_info_.printer_name << ", job id: " << job_details_.job_id_;
     JobFailed(INVALID_JOB_DATA);
   }
@@ -625,17 +639,13 @@ void PrinterJobHandler::OnJobSpoolSucceeded(
     const cloud_print::PlatformJobId& job_id) {
   DCHECK(MessageLoop::current() == print_thread_.message_loop());
   job_spooler_ = NULL;
-  job_handler_message_loop_proxy_->PostTask(FROM_HERE,
-                                            NewRunnableMethod(this,
-                                                &PrinterJobHandler::JobSpooled,
-                                                job_id));
+  job_handler_message_loop_proxy_->PostTask(
+      FROM_HERE, base::Bind(&PrinterJobHandler::JobSpooled, this, job_id));
 }
 
 void PrinterJobHandler::OnJobSpoolFailed() {
   DCHECK(MessageLoop::current() == print_thread_.message_loop());
   job_spooler_ = NULL;
-  job_handler_message_loop_proxy_->PostTask(FROM_HERE,
-                                            NewRunnableMethod(this,
-                                                &PrinterJobHandler::JobFailed,
-                                                PRINT_FAILED));
+  job_handler_message_loop_proxy_->PostTask(
+      FROM_HERE, base::Bind(&PrinterJobHandler::JobFailed, this, PRINT_FAILED));
 }
