@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,10 +8,11 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
 #include "base/synchronization/waitable_event.h"
-#include "chrome/common/net/url_fetcher.h"
 #include "chrome/common/net/gaia/gaia_authenticator.h"
+#include "content/public/common/url_fetcher_delegate.h"
 
 namespace base {
 class MessageLoopProxy;
@@ -21,28 +22,25 @@ class MessageLoopProxy;
 // we cannot rely on the existence of a Profile)
 class ServiceGaiaAuthenticator
     : public base::RefCountedThreadSafe<ServiceGaiaAuthenticator>,
-      public URLFetcher::Delegate,
+      public content::URLFetcherDelegate,
       public gaia::GaiaAuthenticator {
  public:
   ServiceGaiaAuthenticator(const std::string& user_agent,
                            const std::string& service_id,
                            const std::string& gaia_url,
                            base::MessageLoopProxy* io_message_loop_proxy);
-  ~ServiceGaiaAuthenticator();
+  virtual ~ServiceGaiaAuthenticator();
 
-  // URLFetcher::Delegate implementation.
-  virtual void OnURLFetchComplete(const URLFetcher *source,
-                                  const GURL &url,
-                                  const net::URLRequestStatus &status,
-                                  int response_code,
-                                  const ResponseCookies &cookies,
-                                  const std::string &data);
+  // content::URLFetcherDelegate implementation.
+  virtual void OnURLFetchComplete(const content::URLFetcher* source) OVERRIDE;
 
  protected:
   // GaiaAuthenticator overrides.
-  virtual bool Post(const GURL& url, const std::string& post_body,
-                    unsigned long* response_code, std::string* response_body);
-  virtual int GetBackoffDelaySeconds(int current_backoff_delay);
+  virtual bool Post(const GURL& url,
+                    const std::string& post_body,
+                    unsigned long* response_code,
+                    std::string* response_body) OVERRIDE;
+  virtual int GetBackoffDelaySeconds(int current_backoff_delay) OVERRIDE;
 
  private:
   void DoPost(const GURL& post_url, const std::string& post_body);
@@ -56,4 +54,3 @@ class ServiceGaiaAuthenticator
 };
 
 #endif  // CHROME_SERVICE_GAIA_SERVICE_GAIA_AUTHENTICATOR_H_
-
