@@ -8,21 +8,22 @@
 #include <sys/utsname.h>
 #endif
 
+#include "base/compiler_specific.h"
 #include "base/message_loop_proxy.h"
-#include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "base/sys_info.h"
 #include "chrome/common/chrome_version_info.h"
 #include "chrome/service/service_process.h"
 #include "net/base/cert_verifier.h"
 #include "net/base/cookie_monster.h"
-#include "net/base/cookie_policy.h"
-#include "net/base/dnsrr_resolver.h"
 #include "net/base/host_resolver.h"
 #include "net/base/ssl_config_service_defaults.h"
 #include "net/ftp/ftp_network_layer.h"
 #include "net/http/http_auth_handler_factory.h"
 #include "net/http/http_cache.h"
-#include "net/http/http_network_layer.h"
+#include "net/http/http_network_session.h"
+#include "net/http/http_server_properties_impl.h"
+#include "net/proxy/proxy_config_service.h"
 #include "net/proxy/proxy_service.h"
 
 namespace {
@@ -104,43 +105,41 @@ std::string MakeUserAgentForServiceProcess() {
 }  // namespace
 
 ServiceURLRequestContext::ServiceURLRequestContext(
-    const std::string& user_agent) : user_agent_(user_agent) {
-  host_resolver_ =
+    const std::string& user_agent,
+    net::ProxyConfigService* net_proxy_config_service)
+    : user_agent_(user_agent),
+      ALLOW_THIS_IN_INITIALIZER_LIST(storage_(this)) {
+  storage_.set_host_resolver(
       net::CreateSystemHostResolver(net::HostResolver::kDefaultParallelism,
-                                    NULL, NULL);
-  DCHECK(g_service_process);
-  // TODO(sanjeevr): Change CreateSystemProxyConfigService to accept a
-  // MessageLoopProxy* instead of MessageLoop*.
-  // Also this needs to be created on the UI thread on Linux. Fix this.
-  net::ProxyConfigService * proxy_config_service =
-      net::ProxyService::CreateSystemProxyConfigService(
-          g_service_process->io_thread()->message_loop(),
-          g_service_process->file_thread()->message_loop());
-  proxy_service_ = net::ProxyService::CreateUsingSystemProxyResolver(
-      proxy_config_service, 0u, NULL);
-  cert_verifier_ = new net::CertVerifier;
-  dnsrr_resolver_ = new net::DnsRRResolver;
-  ftp_transaction_factory_ = new net::FtpNetworkLayer(host_resolver_);
-  ssl_config_service_ = new net::SSLConfigServiceDefaults;
-  http_auth_handler_factory_ = net::HttpAuthHandlerFactory::CreateDefault(
-      host_resolver_);
-  http_transaction_factory_ = new net::HttpCache(
-      net::HttpNetworkLayer::CreateFactory(host_resolver_,
-                                           cert_verifier_,
-                                           dnsrr_resolver_,
-                                           NULL /* dns_cert_checker */,
-                                           NULL /* ssl_host_info_factory */,
-                                           proxy_service_,
-                                           ssl_config_service_,
-                                           http_auth_handler_factory_,
-                                           NULL /* network_delegate */,
-                                           NULL /* net_log */),
-      NULL /* net_log */,
-      net::HttpCache::DefaultBackend::InMemory(0));
+                                    net::HostResolver::kDefaultRetryAttempts,
+                                    NULL));
+  storage_.set_proxy_service(net::ProxyService::CreateUsingSystemProxyResolver(
+      net_proxy_config_service, 0u, NULL));
+  storage_.set_cert_verifier(new net::CertVerifier);
+  storage_.set_ftp_transaction_factory(
+      new net::FtpNetworkLayer(host_resolver()));
+  storage_.set_ssl_config_service(new net::SSLConfigServiceDefaults);
+  storage_.set_http_auth_handler_factory(
+      net::HttpAuthHandlerFactory::CreateDefault(host_resolver()));
+  storage_.set_http_server_properties(new net::HttpServerPropertiesImpl);
+
+  net::HttpNetworkSession::Params session_params;
+  session_params.host_resolver = host_resolver();
+  session_params.cert_verifier = cert_verifier();
+  session_params.proxy_service = proxy_service();
+  session_params.ssl_config_service = ssl_config_service();
+  session_params.http_auth_handler_factory = http_auth_handler_factory();
+  session_params.http_server_properties = http_server_properties();
+  scoped_refptr<net::HttpNetworkSession> network_session(
+      new net::HttpNetworkSession(session_params));
+  storage_.set_http_transaction_factory(
+      new net::HttpCache(
+          network_session,
+          net::HttpCache::DefaultBackend::InMemory(0)));
   // In-memory cookie store.
-  cookie_store_ = new net::CookieMonster(NULL, NULL);
-  accept_language_ = "en-us,fr";
-  accept_charset_ = "iso-8859-1,*,utf-8";
+  storage_.set_cookie_store(new net::CookieMonster(NULL, NULL));
+  set_accept_language("en-us,fr");
+  set_accept_charset("iso-8859-1,*,utf-8");
 }
 
 const std::string& ServiceURLRequestContext::GetUserAgent(
@@ -152,11 +151,6 @@ const std::string& ServiceURLRequestContext::GetUserAgent(
 }
 
 ServiceURLRequestContext::~ServiceURLRequestContext() {
-  delete ftp_transaction_factory_;
-  delete http_transaction_factory_;
-  delete http_auth_handler_factory_;
-  delete cert_verifier_;
-  delete dnsrr_resolver_;
 }
 
 ServiceURLRequestContextGetter::ServiceURLRequestContextGetter()
@@ -164,12 +158,22 @@ ServiceURLRequestContextGetter::ServiceURLRequestContextGetter()
           g_service_process->io_thread()->message_loop_proxy()) {
   // Build the default user agent.
   user_agent_ = MakeUserAgentForServiceProcess();
+
+  // TODO(sanjeevr): Change CreateSystemProxyConfigService to accept a
+  // MessageLoopProxy* instead of MessageLoop*.
+  DCHECK(g_service_process);
+  proxy_config_service_.reset(
+      net::ProxyService::CreateSystemProxyConfigService(
+          g_service_process->io_thread()->message_loop(),
+          g_service_process->file_thread()->message_loop()));
 }
 
 net::URLRequestContext*
 ServiceURLRequestContextGetter::GetURLRequestContext() {
   if (!url_request_context_)
-    url_request_context_ = new ServiceURLRequestContext(user_agent_);
+    url_request_context_ =
+        new ServiceURLRequestContext(user_agent_,
+                                     proxy_config_service_.release());
   return url_request_context_;
 }
 
