@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,8 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "ipc/ipc_channel_handle.h"
 #include "ipc/ipc_sync_channel.h"
 #include "ipc/ipc_sync_message_filter.h"
 #include "ipc/ipc_message.h"
@@ -16,13 +17,13 @@
 class ServiceIPCServer : public IPC::Channel::Listener,
                          public IPC::Message::Sender {
  public:
-  explicit ServiceIPCServer(const std::string& channel_name);
+  explicit ServiceIPCServer(const IPC::ChannelHandle& handle);
   virtual ~ServiceIPCServer();
 
   bool Init();
 
   // IPC::Message::Sender implementation.
-  virtual bool Send(IPC::Message* msg);
+  virtual bool Send(IPC::Message* msg) OVERRIDE;
 
   IPC::SyncChannel* channel() { return channel_.get(); }
 
@@ -34,30 +35,23 @@ class ServiceIPCServer : public IPC::Channel::Listener,
 
 
  private:
-  // IPC::Channel::Listener implementation:
-  virtual bool OnMessageReceived(const IPC::Message& msg);
-  virtual void OnChannelConnected(int32 peer_pid);
-  virtual void OnChannelError();
+  friend class MockServiceIPCServer;
+
+  // IPC::Channel::Listener implementation.
+  virtual bool OnMessageReceived(const IPC::Message& msg) OVERRIDE;
+  virtual void OnChannelConnected(int32 peer_pid) OVERRIDE;
+  virtual void OnChannelError() OVERRIDE;
 
   // IPC message handlers.
   void OnEnableCloudPrintProxy(const std::string& lsid);
-  void OnEnableCloudPrintProxyWithTokens(const std::string& cloud_print_token,
-                                         const std::string& talk_token);
-  void OnIsCloudPrintProxyEnabled();
+  void OnEnableCloudPrintProxyWithRobot(
+      const std::string& robot_auth_code,
+      const std::string& robot_email,
+      const std::string& user_email);
+  void OnGetCloudPrintProxyInfo();
   void OnDisableCloudPrintProxy();
-
-#if defined(ENABLE_REMOTING)
-  void OnSetRemotingHostCredentials(const std::string& login,
-                                      const std::string& talk_token);
-  void OnEnableRemotingHost();
-  void OnDisableRemotingHost();
-  void OnGetRemotingHostInfo();
-
-  // Sends HostInfo message to the browser. It must is called when we
-  // receive GetRemotingHostInfo message or when status of the host
-  // is changed.
-  void SendRemotingHostInfo();
-#endif  // defined(ENABLE_REMOTING)
+  void OnEnableVirtualDriver();
+  void OnDisableVirtualDriver();
 
   void OnShutdown();
   void OnUpdateAvailable();
@@ -65,7 +59,7 @@ class ServiceIPCServer : public IPC::Channel::Listener,
   // Helper method to create the sync channel.
   void CreateChannel();
 
-  std::string channel_name_;
+  IPC::ChannelHandle channel_handle_;
   scoped_ptr<IPC::SyncChannel> channel_;
   // Indicates whether a client is currently connected to the channel.
   bool client_connected_;
