@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,15 +9,17 @@
 #include <string>
 
 #include "base/gtest_prod_util.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/threading/thread.h"
 #include "base/synchronization/waitable_event.h"
 #include "chrome/service/cloud_print/cloud_print_proxy.h"
-#include "chrome/service/remoting/chromoting_host_manager.h"
 
 class ServiceProcessPrefs;
 class ServiceIPCServer;
+class CommandLine;
+class ServiceURLRequestContextGetter;
+class ServiceProcessState;
 
 namespace net {
 class NetworkChangeNotifier;
@@ -27,14 +29,23 @@ class CommandLine;
 
 // The ServiceProcess does not inherit from ChildProcess because this
 // process can live independently of the browser process.
-class ServiceProcess : public CloudPrintProxy::Client,
-                       public remoting::ChromotingHostManager::Observer {
+// ServiceProcess Design Notes
+// https://sites.google.com/a/chromium.org/dev/developers/design-documents/service-processes
+class ServiceProcess : public CloudPrintProxy::Client {
  public:
   ServiceProcess();
-  ~ServiceProcess();
+  virtual ~ServiceProcess();
 
   // Initialize the ServiceProcess with the message loop that it should run on.
-  bool Initialize(MessageLoop* message_loop, const CommandLine& command_line);
+  // ServiceProcess takes ownership of |state|.
+  bool Initialize(MessageLoopForUI* message_loop,
+                  const CommandLine& command_line,
+                  ServiceProcessState* state);
+
+  // Functions for Cloud Print virtual driver on Mac.
+  void EnableVirtualPrintDriver();
+  void DisableVirtualPrintDriver();
+
   bool Teardown();
   // TODO(sanjeevr): Change various parts of the code such as
   // net::ProxyService::CreateSystemProxyConfigService to take in
@@ -82,39 +93,45 @@ class ServiceProcess : public CloudPrintProxy::Client,
   CloudPrintProxy* GetCloudPrintProxy();
 
   // CloudPrintProxy::Client implementation.
-  virtual void OnCloudPrintProxyEnabled(bool persist_state);
-  virtual void OnCloudPrintProxyDisabled(bool persist_state);
+  virtual void OnCloudPrintProxyEnabled(bool persist_state) OVERRIDE;
+  virtual void OnCloudPrintProxyDisabled(bool persist_state) OVERRIDE;
 
-  // ChromotingHostManager::Observer interface.
-  virtual void OnRemotingHostEnabled();
-  virtual void OnRemotingHostDisabled();
-
-#if defined(ENABLE_REMOTING)
-  // Return the reference to the chromoting host only if it has started.
-  remoting::ChromotingHostManager* remoting_host_manager() {
-    return remoting_host_manager_;
-  }
-#endif
+  ServiceURLRequestContextGetter* GetServiceURLRequestContextGetter();
 
  private:
+  friend class TestServiceProcess;
+
   // Schedule a call to ShutdownIfNeeded.
   void ScheduleShutdownCheck();
+
   // Shuts down the process if no services are enabled and no clients are
   // connected.
   void ShutdownIfNeeded();
+
+  // Schedule a call to CloudPrintPolicyCheckIfNeeded.
+  void ScheduleCloudPrintPolicyCheck();
+
+  // Launch the browser for a policy check if we're not connected.
+  void CloudPrintPolicyCheckIfNeeded();
+
   // Called exactly ONCE per process instance for each service that gets
   // enabled in this process.
   void OnServiceEnabled();
+
   // Called exactly ONCE per process instance for each service that gets
   // disabled in this process (note that shutdown != disabled).
   void OnServiceDisabled();
 
+  // Terminate forces the service process to quit.
+  void Terminate();
+
   scoped_ptr<net::NetworkChangeNotifier> network_change_notifier_;
   scoped_ptr<base::Thread> io_thread_;
   scoped_ptr<base::Thread> file_thread_;
   scoped_ptr<CloudPrintProxy> cloud_print_proxy_;
   scoped_ptr<ServiceProcessPrefs> service_prefs_;
   scoped_ptr<ServiceIPCServer> ipc_server_;
+  scoped_ptr<ServiceProcessState> service_process_state_;
 
   // An event that will be signalled when we shutdown.
   base::WaitableEvent shutdown_event_;
@@ -128,9 +145,7 @@ class ServiceProcess : public CloudPrintProxy::Client,
   // Speficies whether a product update is available.
   bool update_available_;
 
-#if defined(ENABLE_REMOTING)
-  scoped_refptr<remoting::ChromotingHostManager> remoting_host_manager_;
-#endif
+  scoped_refptr<ServiceURLRequestContextGetter> request_context_getter_;
 
   DISALLOW_COPY_AND_ASSIGN(ServiceProcess);
 };
