@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,22 +11,22 @@
 
 #include "base/basictypes.h"
 #include "base/compiler_specific.h"
-#include "base/threading/platform_thread.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/process_util.h"
-#include "base/scoped_ptr.h"
-#include "base/time.h"
-#include "base/threading/thread.h"
 #include "base/synchronization/waitable_event.h"
+#include "base/threading/platform_thread.h"
+#include "base/threading/thread.h"
+#include "base/time.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/common/automation_constants.h"
 #include "chrome/test/automation/automation_handle_tracker.h"
 #include "chrome/test/automation/browser_proxy.h"
-#include "gfx/native_widget_types.h"
 #include "googleurl/src/gurl.h"
 #include "ipc/ipc_channel_proxy.h"
 #include "ipc/ipc_message.h"
 #include "ipc/ipc_sync_channel.h"
-#include "ui/base/message_box_flags.h"
+#include "ui/base/ui_base_types.h"
+#include "ui/gfx/native_widget_types.h"
 
 class BrowserProxy;
 class ExtensionProxy;
@@ -49,6 +49,7 @@ class AutomationMessageSender : public IPC::Message::Sender {
   //       and the proxy provider might be still working on the previous
   //       request.
   virtual bool Send(IPC::Message* message) = 0;
+  virtual bool Send(IPC::Message* message, int timeout_ms) = 0;
 };
 
 // This is the interface that external processes can use to interact with
@@ -56,7 +57,7 @@ class AutomationMessageSender : public IPC::Message::Sender {
 class AutomationProxy : public IPC::Channel::Listener,
                         public AutomationMessageSender {
  public:
-  AutomationProxy(int command_execution_timeout_ms, bool disconnect_on_failure);
+  AutomationProxy(int action_timeout_ms, bool disconnect_on_failure);
   virtual ~AutomationProxy();
 
   // Creates a previously unused channel id.
@@ -80,13 +81,16 @@ class AutomationProxy : public IPC::Channel::Listener,
   // Waits for the app to launch and the automation provider to say hello
   // (the app isn't fully done loading by this point).
   // Returns SUCCESS if the launch is successful.
-  // Returns TIMEOUT if there was no response by command_execution_timeout_
+  // Returns TIMEOUT if there was no response by action_timeout_
   // Returns VERSION_MISMATCH if the automation protocol version of the
   // automation provider does not match and if perform_version_check_ is set
   // to true. Note that perform_version_check_ defaults to false, call
   // set_perform_version_check() to set it.
   AutomationLaunchResult WaitForAppLaunch();
 
+  // See description in AutomationMsg_WaitForProcessLauncherThreadToGoIdle.
+  bool WaitForProcessLauncherThreadToGoIdle() WARN_UNUSED_RESULT;
+
   // Waits for any initial page loads to complete.
   // NOTE: this only fires once for a run of the application.
   // Returns true if the load is successful
@@ -123,11 +127,10 @@ class AutomationProxy : public IPC::Channel::Listener,
   // Returns whether an app modal dialog window is showing right now (i.e., a
   // javascript alert), and what buttons it contains.
   bool GetShowingAppModalDialog(bool* showing_app_modal_dialog,
-      ui::MessageBoxFlags::DialogButton* button) WARN_UNUSED_RESULT;
+                                ui::DialogButton* button) WARN_UNUSED_RESULT;
 
   // Simulates a click on a dialog button. Synchronous.
-  bool ClickAppModalDialogButton(
-      ui::MessageBoxFlags::DialogButton button) WARN_UNUSED_RESULT;
+  bool ClickAppModalDialogButton(ui::DialogButton button) WARN_UNUSED_RESULT;
 
   // Block the thread until a modal dialog is displayed. Returns true on
   // success.
@@ -150,10 +153,10 @@ class AutomationProxy : public IPC::Channel::Listener,
   scoped_refptr<BrowserProxy> GetBrowserWindow(int window_index);
 
   // Finds the first browser window that is not incognito mode and of type
-  // TYPE_NORMAL, and returns its corresponding BrowserProxy, transferring
+  // TYPE_TABBED, and returns its corresponding BrowserProxy, transferring
   // ownership of the pointer to the caller.
   // On failure, returns NULL.
-  scoped_refptr<BrowserProxy> FindNormalBrowserWindow();
+  scoped_refptr<BrowserProxy> FindTabbedBrowserWindow();
 
   // Returns the BrowserProxy for the browser window which was last active,
   // transferring ownership of the pointer to the caller.
@@ -193,23 +196,25 @@ class AutomationProxy : public IPC::Channel::Listener,
   // sent.
   bool SavePackageShouldPromptUser(bool should_prompt) WARN_UNUSED_RESULT;
 
-  // Installs the extension crx. If |with_ui| is true an install confirmation
+  // Installs the extension. If |with_ui| is true an install confirmation
   // and notification UI is shown, otherwise the install is silent. Returns the
   // ExtensionProxy for the installed extension, or NULL on failure.
   // Note: Overinstalls and downgrades will return NULL.
-  scoped_refptr<ExtensionProxy> InstallExtension(const FilePath& crx_file,
+  scoped_refptr<ExtensionProxy> InstallExtension(const FilePath& extension_path,
                                                  bool with_ui);
 
-  // Asserts that the next extension test result is true.
-  void EnsureExtensionTestResult();
-
-  // Gets a list of all enabled extensions' base directories.
-  // Returns true on success.
-  bool GetEnabledExtensions(std::vector<FilePath>* extension_directories);
+  // Gets the next extension test result in |result|. Returns false if there
+  // was a problem sending the result querying RPC.
+  bool GetExtensionTestResult(bool* result, std::string* message);
 
   // Resets to the default theme. Returns true on success.
   bool ResetToDefaultTheme();
 
+  // Generic pattern for sending automation requests.
+  bool SendJSONRequest(const std::string& request,
+                       int timeout_ms,
+                       std::string* response) WARN_UNUSED_RESULT;
+
 #if defined(OS_CHROMEOS)
   // Logs in through the Chrome OS login wizard with given |username|
   // and |password|.  Returns true on success.
@@ -217,12 +222,28 @@ class AutomationProxy : public IPC::Channel::Listener,
                             const std::string& password) WARN_UNUSED_RESULT;
 #endif
 
-#if defined(OS_POSIX)
-  base::file_handle_mapping_vector fds_to_map() const;
-#endif
+  // Begin tracing specified categories on the browser instance. Blocks until
+  // browser acknowledges that tracing has begun (or failed if false is
+  // returned). |categories| is a comma-delimited list of category wildcards.
+  // A category can have an optional '-' prefix to make it an excluded category.
+  // Either all categories must be included or all must be excluded.
+  //
+  // Example: BeginTracing("test_MyTest*");
+  // Example: BeginTracing("test_MyTest*,test_OtherStuff");
+  // Example: BeginTracing("-excluded_category1,-excluded_category2");
+  //
+  // See base/event_trace.h for documentation of included and excluded
+  // categories.
+  bool BeginTracing(const std::string& categories) WARN_UNUSED_RESULT;
+
+  // End trace and collect the trace output as a json string.
+  bool EndTracing(std::string* json_trace_output) WARN_UNUSED_RESULT;
+
+  IPC::SyncChannel* channel();
 
   // AutomationMessageSender implementation.
   virtual bool Send(IPC::Message* message) WARN_UNUSED_RESULT;
+  virtual bool Send(IPC::Message* message, int timeout_ms) WARN_UNUSED_RESULT;
 
   // Wrapper over AutomationHandleTracker::InvalidateHandle. Receives the
   // message from AutomationProxy, unpacks the messages and routes that call to
@@ -237,15 +258,15 @@ class AutomationProxy : public IPC::Channel::Listener,
       gfx::NativeWindow* external_tab_container,
       gfx::NativeWindow* tab);
 
-  int command_execution_timeout_ms() const {
-    return static_cast<int>(command_execution_timeout_.InMilliseconds());
+  int action_timeout_ms() const {
+    return static_cast<int>(action_timeout_.InMilliseconds());
   }
 
   // Sets the timeout for subsequent automation calls.
-  void set_command_execution_timeout_ms(int timeout_ms) {
+  void set_action_timeout_ms(int timeout_ms) {
     DCHECK(timeout_ms <= 10 * 60 * 1000 ) << "10+ min of automation timeout "
         "can make the test hang and be killed by buildbot";
-    command_execution_timeout_ = base::TimeDelta::FromMilliseconds(timeout_ms);
+    action_timeout_ = base::TimeDelta::FromMilliseconds(timeout_ms);
   }
 
   // Returns the server version of the server connected. You may only call this
@@ -266,6 +287,11 @@ class AutomationProxy : public IPC::Channel::Listener,
   void SetChannel(IPC::Channel* channel);
   void ResetChannel();
 
+  // See description above |channel_disconnected_on_failure_|.
+  bool channel_disconnected_on_failure() const {
+    return channel_disconnected_on_failure_;
+  }
+
  protected:
   template <class T> scoped_refptr<T> ProxyObjectFromHandle(int handle);
   void InitializeThread();
@@ -286,9 +312,6 @@ class AutomationProxy : public IPC::Channel::Listener,
   // The version of the automation provider we are communicating with.
   std::string server_version_;
 
-  // Used to guard against multiple hello messages being received.
-  int app_launch_signaled_;
-
   // Whether to perform a version check between the automation proxy and
   // the automation provider at connection time. Defaults to false, you can
   // set this to true if building the automation proxy into a module with
@@ -299,8 +322,11 @@ class AutomationProxy : public IPC::Channel::Listener,
   // to send an IPC message. This helps avoid long timeouts in tests.
   bool disconnect_on_failure_;
 
+  // Set if disconnect_on_failure_ caused the connection to be dropped.
+  bool channel_disconnected_on_failure_;
+
   // Delay to let the browser execute the command.
-  base::TimeDelta command_execution_timeout_;
+  base::TimeDelta action_timeout_;
 
   base::PlatformThreadId listener_thread_id_;
 
