@@ -1,17 +1,18 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <string>
 
-#include "app/app_switches.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
 #include "base/i18n/rtl.h"
+#include "base/json/json_value_serializer.h"
+#include "base/md5.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/scoped_temp_dir.h"
-#include "base/scoped_ptr.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/sys_info.h"
@@ -19,31 +20,29 @@
 #include "base/utf_string_conversions.h"
 #include "build/build_config.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/net/url_request_slow_http_job.h"
 #include "chrome/browser/ui/view_ids.h"
 #include "chrome/common/automation_messages.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/json_value_serializer.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/automation/autocomplete_edit_proxy.h"
 #include "chrome/test/automation/automation_proxy_uitest.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/proxy_launcher.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/automation/window_proxy.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "chrome/test/ui/ui_test.h"
-#include "gfx/codec/png_codec.h"
-#include "gfx/rect.h"
+#include "content/browser/net/url_request_slow_http_job.h"
+#include "net/base/host_port_pair.h"
 #include "net/base/net_util.h"
 #include "net/test/test_server.h"
 #define GMOCK_MUTANT_INCLUDE_LATE_OBJECT_BINDING
 #include "testing/gmock_mutant.h"
 #include "third_party/skia/include/core/SkBitmap.h"
-#include "ui/base/message_box_flags.h"
 #include "ui/base/ui_base_switches.h"
-#include "views/event.h"
+#include "ui/base/ui_base_types.h"
+#include "ui/gfx/codec/png_codec.h"
+#include "ui/gfx/rect.h"
 
 using ui_test_utils::TimedMessageLoopRunner;
 using testing::CreateFunctor;
@@ -65,9 +64,17 @@ class ExternalTabUITestMockLauncher : public ProxyLauncher {
     return *mock_;
   }
 
-  void InitializeConnection(const LaunchState& state,
-                            bool wait_for_initial_loads) {
-    LaunchBrowserAndServer(state, wait_for_initial_loads);
+  bool InitializeConnection(
+      const LaunchState& state,
+      bool wait_for_initial_loads) OVERRIDE WARN_UNUSED_RESULT {
+    bool launch_browser_and_server =
+        LaunchBrowserAndServer(state, wait_for_initial_loads);
+    EXPECT_TRUE(launch_browser_and_server);
+    return launch_browser_and_server;
+  }
+
+  void TerminateConnection() {
+    CloseBrowserAndServer();
   }
 
   std::string PrefixedChannelID() const {
@@ -91,7 +98,7 @@ TEST_F(AutomationProxyTest, GetBrowserWindowCount) {
   int window_count = 0;
   EXPECT_TRUE(automation()->GetBrowserWindowCount(&window_count));
   EXPECT_EQ(1, window_count);
-#ifdef NDEBUG
+#if defined(NDEBUG) && !defined(DCHECK_ALWAYS_ON)
   ASSERT_FALSE(automation()->GetBrowserWindowCount(NULL));
 #endif
 }
@@ -132,7 +139,7 @@ TEST_F(AutomationProxyVisibleTest, MAYBE_WindowGetViewBounds) {
     ASSERT_TRUE(tab1->GetCurrentURL(&tab1_url));
 
     // Add another tab so we can simulate dragging.
-    ASSERT_TRUE(browser->AppendTab(GURL("about:")));
+    ASSERT_TRUE(browser->AppendTab(GURL(chrome::kChromeUIVersionURL)));
 
     scoped_refptr<TabProxy> tab2(browser->GetTab(1));
     ASSERT_TRUE(tab2.get());
@@ -167,13 +174,18 @@ TEST_F(AutomationProxyVisibleTest, MAYBE_WindowGetViewBounds) {
     }
     EXPECT_EQ(bounds2.y(), bounds.y());
 
-    gfx::Rect urlbar_bounds;
-    ASSERT_TRUE(window->GetViewBounds(VIEW_ID_LOCATION_BAR, &urlbar_bounds,
-                                      false));
-    EXPECT_GT(urlbar_bounds.x(), 0);
-    EXPECT_GT(urlbar_bounds.y(), 0);
-    EXPECT_GT(urlbar_bounds.width(), 0);
-    EXPECT_GT(urlbar_bounds.height(), 0);
+    // Sometimes tests start the browser in full screen mode. Don't check the
+    // location bar in such a case.
+    bool fullscreen = false;
+    if (browser->IsFullscreen(&fullscreen) && !fullscreen) {
+      gfx::Rect urlbar_bounds;
+      ASSERT_TRUE(window->GetViewBounds(VIEW_ID_LOCATION_BAR, &urlbar_bounds,
+                                        false));
+      EXPECT_GT(urlbar_bounds.x(), 0);
+      EXPECT_GT(urlbar_bounds.y(), 0);
+      EXPECT_GT(urlbar_bounds.width(), 0);
+      EXPECT_GT(urlbar_bounds.height(), 0);
+    }
 
     /*
 
@@ -187,8 +199,7 @@ TEST_F(AutomationProxyVisibleTest, MAYBE_WindowGetViewBounds) {
     start.y = bounds.y() + bounds.height() / 2;
     end.x = start.x + 2 * bounds.width() / 3;
     end.y = start.y;
-    ASSERT_TRUE(browser->SimulateDrag(start, end,
-                                      views::Event::EF_LEFT_BUTTON_DOWN));
+    ASSERT_TRUE(browser->SimulateDrag(start, end, ui::EF_LEFT_MOUSE_BUTTON));
 
     // Check to see that the drag event successfully swapped the two tabs.
     tab1 = browser->GetTab(0);
@@ -287,8 +298,7 @@ TEST_F(AutomationProxyTest, GetTab) {
     ASSERT_TRUE(tab.get());
     std::wstring title;
     ASSERT_TRUE(tab->GetTabTitle(&title));
-    // BUG [634097] : expected title should be "about:blank"
-    ASSERT_STREQ(L"", title.c_str());
+    ASSERT_STREQ(L"about:blank", title.c_str());
   }
 
   {
@@ -309,8 +319,7 @@ TEST_F(AutomationProxyTest, NavigateToURL) {
 
   std::wstring title;
   ASSERT_TRUE(tab->GetTabTitle(&title));
-  // BUG [634097] : expected title should be "about:blank"
-  ASSERT_STREQ(L"", title.c_str());
+  ASSERT_STREQ(L"about:blank", title.c_str());
 
   FilePath filename(test_data_directory_);
   filename = filename.AppendASCII("title2.html");
@@ -331,12 +340,11 @@ TEST_F(AutomationProxyTest, GoBackForward) {
 
   std::wstring title;
   ASSERT_TRUE(tab->GetTabTitle(&title));
-  // BUG [634097] : expected title should be "about:blank"
-  ASSERT_STREQ(L"", title.c_str());
+  ASSERT_STREQ(L"about:blank", title.c_str());
 
   ASSERT_FALSE(tab->GoBack());
   ASSERT_TRUE(tab->GetTabTitle(&title));
-  ASSERT_STREQ(L"", title.c_str());
+  ASSERT_STREQ(L"about:blank", title.c_str());
 
   FilePath filename(test_data_directory_);
   filename = filename.AppendASCII("title2.html");
@@ -346,8 +354,7 @@ TEST_F(AutomationProxyTest, GoBackForward) {
 
   ASSERT_TRUE(tab->GoBack());
   ASSERT_TRUE(tab->GetTabTitle(&title));
-  // BUG [634097] : expected title should be "about:blank"
-  ASSERT_STREQ(L"", title.c_str());
+  ASSERT_STREQ(L"about:blank", title.c_str());
 
   ASSERT_TRUE(tab->GoForward());
   ASSERT_TRUE(tab->GetTabTitle(&title));
@@ -404,7 +411,13 @@ TEST_F(AutomationProxyTest2, GetActiveTabIndex) {
   ASSERT_EQ(1, active_tab_index);
 }
 
-TEST_F(AutomationProxyTest2, GetTabTitle) {
+// http://crbug.com/98071
+#if defined(OS_MACOSX)
+#define MAYBE_GetTabTitle FLAKY_GetTabTitle
+#else
+#define MAYBE_GetTabTitle GetTabTitle
+#endif
+TEST_F(AutomationProxyTest2, MAYBE_GetTabTitle) {
   scoped_refptr<BrowserProxy> window(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(window.get());
   scoped_refptr<TabProxy> tab(window->GetTab(0));
@@ -501,14 +514,19 @@ TEST_F(AutomationProxyTest, AcceleratorDownloads) {
   EXPECT_EQ(L"Downloads", GetActiveTabTitle());
 }
 
-TEST_F(AutomationProxyTest, AcceleratorExtensions) {
+// http://crbug.com/109642
+#if defined(OS_MACOSX)
+#define MAYBE_AcceleratorExtensions FLAKY_AcceleratorExtensions
+#else
+#define MAYBE_AcceleratorExtensions AcceleratorExtensions
+#endif
+TEST_F(AutomationProxyTest, MAYBE_AcceleratorExtensions) {
   scoped_refptr<BrowserProxy> window(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(window.get());
 
   ASSERT_TRUE(window->RunCommand(IDC_MANAGE_EXTENSIONS));
 
-  // We expect the RunCommand above to wait until the title is updated.
-  EXPECT_EQ(L"Extensions", GetActiveTabTitle());
+  EXPECT_EQ("chrome://settings/extensions", GetActiveTabURL().spec());
 }
 
 TEST_F(AutomationProxyTest, AcceleratorHistory) {
@@ -638,7 +656,8 @@ TEST_F(AutomationProxyTest3, FrameDocumentCanBeAccessed) {
   ASSERT_EQ(L"DIV", actual);
 }
 
-TEST_F(AutomationProxyTest, BlockedPopupTest) {
+// Flaky, http://crbug.com/70937
+TEST_F(AutomationProxyTest, FLAKY_BlockedPopupTest) {
   scoped_refptr<BrowserProxy> window(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(window.get());
 
@@ -697,8 +716,11 @@ void ExternalTabUITestMockClient::ReplyEnd(const net::URLRequestStatus& status,
 }
 
 void ExternalTabUITestMockClient::Reply404(int tab_handle, int request_id) {
-  const AutomationURLResponse notfound("", "HTTP/1.1 404\r\n\r\n", 0,
-                                       base::Time(), "", 0);
+  AutomationURLResponse notfound;
+  notfound.headers = "HTTP/1.1 404\r\n\r\n";
+  notfound.content_length = 0;
+  notfound.redirect_status = 0;
+
   ReplyStarted(&notfound, tab_handle, request_id);
   ReplyEOF(tab_handle, request_id);
 }
@@ -711,7 +733,7 @@ void ExternalTabUITestMockClient::ServeHTMLData(int tab_handle,
       testing::Field(&AutomationURLRequest::method, StrEq("GET")))))
     .Times(1)
     .WillOnce(testing::WithArgs<0, 0>(testing::Invoke(CreateFunctor(this,
-        &ExternalTabUITestMockClient::ReplyStarted, &http_200))));
+        &ExternalTabUITestMockClient::ReplyStarted, &GetHttp200Response()))));
 
   EXPECT_CALL(*this, OnRequestRead(tab_handle, testing::Gt(0)))
       .Times(2)
@@ -721,7 +743,7 @@ void ExternalTabUITestMockClient::ServeHTMLData(int tab_handle,
           &ExternalTabUITestMockClient::ReplyEOF))));
 }
 
-void ExternalTabUITestMockClient::IgnoreFavIconNetworkRequest() {
+void ExternalTabUITestMockClient::IgnoreFaviconNetworkRequest() {
   // Ignore favicon.ico
   EXPECT_CALL(*this, OnRequestStart(_, testing::AllOf(
           testing::Field(&AutomationURLRequest::url,
@@ -744,26 +766,27 @@ void ExternalTabUITestMockClient::InvalidateHandle(
   HandleClosed(handle);
 }
 
-// Most of the time we need external tab with these settings.
-const ExternalTabSettings ExternalTabUITestMockClient::default_settings(
-    NULL, gfx::Rect(),  // will be replaced by CreateHostWindowAndTab
-    WS_CHILD | WS_VISIBLE,
-    false,   // is_off_the_record
-    true,    // load_requests_via_automation
-    true,    // handle_top_level_requests
-    GURL(),  // initial_url
-    GURL(),  // referrer
-    false,   // infobars_enabled
-    false);  // route_all_top_level_navigations
+// static
+ExternalTabSettings ExternalTabUITestMockClient::GetDefaultSettings() {
+  ExternalTabSettings settings;
+  settings.parent = NULL;
+  settings.style = WS_CHILD | WS_VISIBLE;
+  settings.is_incognito = false;
+  settings.load_requests_via_automation = true;
+  settings.handle_top_level_requests = true;
+  settings.infobars_enabled = false;
+  settings.route_all_top_level_navigations = false;
+  return settings;
+}
 
 // static
-const AutomationURLResponse ExternalTabUITestMockClient::http_200(
-    "",
-    "HTTP/0.9 200\r\n\r\n",
-    0,
-    base::Time(),
-    "",
-    0);
+AutomationURLResponse ExternalTabUITestMockClient::GetHttp200Response() {
+  AutomationURLResponse response;
+  response.headers = "HTTP/0.9 200\r\n\r\n";
+  response.content_length = 0;
+  response.redirect_status = 0;
+  return response;
+}
 
 bool ExternalTabUITestMockClient::OnMessageReceived(const IPC::Message& msg) {
   bool handled = true;
@@ -812,8 +835,7 @@ scoped_refptr<TabProxy> ExternalTabUITestMockClient::CreateHostWindowAndTab(
 
 scoped_refptr<TabProxy> ExternalTabUITestMockClient::CreateTabWithUrl(
     const GURL& initial_url) {
-  ExternalTabSettings settings =
-      ExternalTabUITestMockClient::default_settings;
+  ExternalTabSettings settings = GetDefaultSettings();
   settings.initial_url = initial_url;
   return CreateHostWindowAndTab(settings);
 }
@@ -838,7 +860,7 @@ void ExternalTabUITestMockClient::ConnectToExternalTab(gfx::NativeWindow parent,
 
   RECT rect;
   ::GetClientRect(parent, &rect);
-  Reposition_Params params = {0};
+  Reposition_Params params;
   params.window = tab_container;
   params.flags = SWP_NOZORDER | SWP_SHOWWINDOW;
   params.width = rect.right - rect.left;
@@ -954,8 +976,8 @@ TEST_F(ExternalTabUITest, FLAKY_IncognitoMode) {
   EXPECT_CALL(*mock_, HandleClosed(1)).Times(1);
 
   ExternalTabSettings incognito =
-      ExternalTabUITestMockClient::default_settings;
-  incognito.is_off_the_record = true;
+      ExternalTabUITestMockClient::GetDefaultSettings();
+  incognito.is_incognito = true;
   // SetCookie is a sync call and deadlock can happen if window is visible,
   // since it shares same thread with AutomationProxy.
   mock_->host_window_style_ &= ~WS_VISIBLE;
@@ -1064,15 +1086,15 @@ TEST_F(ExternalTabUITest, FLAKY_PostMessageTarget)  {
 
   EXPECT_CALL(*mock_, HandleClosed(1)).Times(1);
 
-  ExternalTabSettings s = ExternalTabUITestMockClient::default_settings;
+  ExternalTabSettings s = ExternalTabUITestMockClient::GetDefaultSettings();
   s.load_requests_via_automation = false;
   s.initial_url = GURL("http://localhost:1337/files/post_message.html");
   tab = mock_->CreateHostWindowAndTab(s);
   loop.RunFor(TestTimeouts::action_max_timeout_ms());
 }
 
-// Flaky, http://crbug.com/42545.
-TEST_F(ExternalTabUITest, FLAKY_HostNetworkStack) {
+// Disabled, http://crbug.com/42545.
+TEST_F(ExternalTabUITest, DISABLED_HostNetworkStack) {
   scoped_refptr<TabProxy> tab;
   TimedMessageLoopRunner loop(MessageLoop::current());
   ASSERT_THAT(mock_, testing::NotNull());
@@ -1087,11 +1109,12 @@ TEST_F(ExternalTabUITest, FLAKY_HostNetworkStack) {
           testing::Field(&AutomationURLRequest::url, StrEq(url + "/")),
           testing::Field(&AutomationURLRequest::method, StrEq("GET")))))
       .Times(1)
-      // We can simply do CreateFunctor(1, 2, &http_200) since we know the
-      // tab handle and request id, but using WithArgs<> is much more fancy :)
+      // We can simply do CreateFunctor(1, 2, &GetHttp200Response()) since we
+      // know the tab handle and request id, but using WithArgs<> is much more
+      // fancy :)
       .WillOnce(testing::WithArgs<0, 0>(testing::Invoke(CreateFunctor(mock_,
           &ExternalTabUITestMockClient::ReplyStarted,
-          &ExternalTabUITestMockClient::http_200))));
+          &ExternalTabUITestMockClient::GetHttp200Response()))));
 
   // Return some trivial page, that have a link to a "logo.gif" image
   const std::string data = "<!DOCTYPE html><title>Hello</title>"
@@ -1142,8 +1165,8 @@ TEST_F(ExternalTabUITest, FLAKY_HostNetworkStack) {
   loop.RunFor(TestTimeouts::action_max_timeout_ms());
 }
 
-// Flaky, http://crbug.com/61023.
-TEST_F(ExternalTabUITest, FLAKY_HostNetworkStackAbortRequest) {
+// Disabled, http://crbug.com/61023.
+TEST_F(ExternalTabUITest, DISABLED_HostNetworkStackAbortRequest) {
   scoped_refptr<TabProxy> tab;
   TimedMessageLoopRunner loop(MessageLoop::current());
   ASSERT_THAT(mock_, testing::NotNull());
@@ -1157,11 +1180,12 @@ TEST_F(ExternalTabUITest, FLAKY_HostNetworkStackAbortRequest) {
           testing::Field(&AutomationURLRequest::url, StrEq(url + "/")),
           testing::Field(&AutomationURLRequest::method, StrEq("GET")))))
       .Times(1)
-      // We can simply do CreateFunctor(1, 2, &http_200) since we know the
-      // tab handle and request id, but using WithArgs<> is much more fancy :)
+      // We can simply do CreateFunctor(1, 2, &GetHttp200Response()) since we
+      // know the tab handle and request id, but using WithArgs<> is much more
+      // fancy :)
       .WillOnce(testing::WithArgs<0, 0>(testing::Invoke(CreateFunctor(mock_,
           &ExternalTabUITestMockClient::ReplyStarted,
-          &ExternalTabUITestMockClient::http_200))));
+          &ExternalTabUITestMockClient::GetHttp200Response()))));
 
   // Return some trivial page, that have a link to a "logo.gif" image
   const std::string data = "<!DOCTYPE html><title>Hello";
@@ -1184,8 +1208,8 @@ TEST_F(ExternalTabUITest, FLAKY_HostNetworkStackAbortRequest) {
   loop.RunFor(TestTimeouts::action_max_timeout_ms());
 }
 
-// Flaky, http://crbug.com/61023.
-TEST_F(ExternalTabUITest, FLAKY_HostNetworkStackUnresponsiveRenderer) {
+// Disabled, http://crbug.com/61023.
+TEST_F(ExternalTabUITest, DISABLED_HostNetworkStackUnresponsiveRenderer) {
   scoped_refptr<TabProxy> tab;
   TimedMessageLoopRunner loop(MessageLoop::current());
   ASSERT_THAT(mock_, testing::NotNull());
@@ -1204,11 +1228,12 @@ TEST_F(ExternalTabUITest, FLAKY_HostNetworkStackUnresponsiveRenderer) {
           testing::Field(&AutomationURLRequest::url, StrEq(url + "/")),
           testing::Field(&AutomationURLRequest::method, StrEq("GET")))))
       .Times(1)
-      // We can simply do CreateFunctor(1, 2, &http_200) since we know the
-      // tab handle and request id, but using WithArgs<> is much more fancy :)
+      // We can simply do CreateFunctor(1, 2, &GetHttp200Response()) since we
+      // know the tab handle and request id, but using WithArgs<> is much more
+      // fancy :)
       .WillOnce(testing::WithArgs<0, 0>(testing::Invoke(CreateFunctor(mock_,
           &ExternalTabUITestMockClient::ReplyStarted,
-          &ExternalTabUITestMockClient::http_200))));
+          &ExternalTabUITestMockClient::GetHttp200Response()))));
 
   const std::string head = "<html><title>Hello</title><body>";
 
@@ -1263,8 +1288,8 @@ class ExternalTabUITestPopupEnabled : public ExternalTabUITest {
 
 #if defined(OS_WIN)
 // http://crbug.com/61023 - Fails on one popular operating system.
-#define MAYBE_WindowDotOpen FLAKY_WindowDotOpen
-#define MAYBE_UserGestureTargetBlank FLAKY_UserGestureTargetBlank
+#define MAYBE_WindowDotOpen DISABLED_WindowDotOpen
+#define MAYBE_UserGestureTargetBlank DISABLED_UserGestureTargetBlank
 #else
 #define MAYBE_WindowDotOpen WindowDotOpen
 #define MAYBE_UserGestureTargetBlank UserGestureTargetBlank
@@ -1276,7 +1301,7 @@ class ExternalTabUITestPopupEnabled : public ExternalTabUITest {
 TEST_F(ExternalTabUITestPopupEnabled, MAYBE_WindowDotOpen) {
   TimedMessageLoopRunner loop(MessageLoop::current());
   ASSERT_THAT(mock_, testing::NotNull());
-  mock_->IgnoreFavIconNetworkRequest();
+  mock_->IgnoreFaviconNetworkRequest();
   // Ignore navigation state changes.
   EXPECT_CALL(*mock_, OnNavigationStateChanged(_, _))
       .Times(testing::AnyNumber());
@@ -1341,7 +1366,7 @@ TEST_F(ExternalTabUITestPopupEnabled, MAYBE_WindowDotOpen) {
 TEST_F(ExternalTabUITestPopupEnabled, MAYBE_UserGestureTargetBlank) {
   TimedMessageLoopRunner loop(MessageLoop::current());
   ASSERT_THAT(mock_, testing::NotNull());
-  mock_->IgnoreFavIconNetworkRequest();
+  mock_->IgnoreFaviconNetworkRequest();
   // Ignore navigation state changes.
   EXPECT_CALL(*mock_, OnNavigationStateChanged(_, _))
       .Times(testing::AnyNumber());
@@ -1372,82 +1397,8 @@ TEST_F(ExternalTabUITestPopupEnabled, MAYBE_UserGestureTargetBlank) {
   ::DestroyWindow(foo_host);
   mock_->DestroyHostWindow();
 }
-
 #endif  // defined(OS_WIN)
 
-// TODO(port): Need to port autocomplete_edit_proxy.* first.
-#if defined(OS_WIN) || defined(OS_LINUX)
-TEST_F(AutomationProxyTest, AutocompleteGetSetText) {
-  scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
-  ASSERT_TRUE(browser.get());
-  scoped_refptr<AutocompleteEditProxy> edit(
-      browser->GetAutocompleteEdit());
-  ASSERT_TRUE(edit.get());
-  EXPECT_TRUE(edit->is_valid());
-  const std::wstring text_to_set = L"Lollerskates";
-  std::wstring actual_text;
-  EXPECT_TRUE(edit->SetText(text_to_set));
-  EXPECT_TRUE(edit->GetText(&actual_text));
-  EXPECT_EQ(text_to_set, actual_text);
-  scoped_refptr<AutocompleteEditProxy> edit2(
-      browser->GetAutocompleteEdit());
-  EXPECT_TRUE(edit2->GetText(&actual_text));
-  EXPECT_EQ(text_to_set, actual_text);
-}
-
-TEST_F(AutomationProxyTest, AutocompleteParallelProxy) {
-  scoped_refptr<BrowserProxy> browser1(automation()->GetBrowserWindow(0));
-  ASSERT_TRUE(browser1.get());
-  scoped_refptr<AutocompleteEditProxy> edit1(
-      browser1->GetAutocompleteEdit());
-  ASSERT_TRUE(edit1.get());
-  EXPECT_TRUE(browser1->RunCommand(IDC_NEW_WINDOW));
-  scoped_refptr<BrowserProxy> browser2(automation()->GetBrowserWindow(1));
-  ASSERT_TRUE(browser2.get());
-  scoped_refptr<AutocompleteEditProxy> edit2(
-      browser2->GetAutocompleteEdit());
-  ASSERT_TRUE(edit2.get());
-  EXPECT_TRUE(browser2->GetTab(0)->WaitForTabToBeRestored(
-      TestTimeouts::action_max_timeout_ms()));
-  const std::wstring text_to_set1 = L"Lollerskates";
-  const std::wstring text_to_set2 = L"Roflcopter";
-  std::wstring actual_text1, actual_text2;
-  EXPECT_TRUE(edit1->SetText(text_to_set1));
-  EXPECT_TRUE(edit2->SetText(text_to_set2));
-  EXPECT_TRUE(edit1->GetText(&actual_text1));
-  EXPECT_TRUE(edit2->GetText(&actual_text2));
-  EXPECT_EQ(text_to_set1, actual_text1);
-  EXPECT_EQ(text_to_set2, actual_text2);
-}
-
-#endif  // defined(OS_WIN) || defined(OS_LINUX)
-
-#if defined(OS_MACOSX)
-// Disabled, http://crbug.com/48601.
-#define AutocompleteMatchesTest DISABLED_AutocompleteMatchesTest
-#else
-// Flaky, http://crbug.com/19876.
-#define AutocompleteMatchesTest FLAKY_AutocompleteMatchesTest
-#endif
-TEST_F(AutomationProxyVisibleTest, AutocompleteMatchesTest) {
-  scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
-  ASSERT_TRUE(browser.get());
-  scoped_refptr<AutocompleteEditProxy> edit(
-      browser->GetAutocompleteEdit());
-  ASSERT_TRUE(edit.get());
-  EXPECT_TRUE(edit->is_valid());
-  EXPECT_TRUE(browser->ApplyAccelerator(IDC_FOCUS_LOCATION));
-  ASSERT_TRUE(edit->WaitForFocus());
-  EXPECT_TRUE(edit->SetText(L"Roflcopter"));
-  EXPECT_TRUE(edit->WaitForQuery(TestTimeouts::action_max_timeout_ms()));
-  bool query_in_progress;
-  EXPECT_TRUE(edit->IsQueryInProgress(&query_in_progress));
-  EXPECT_FALSE(query_in_progress);
-  std::vector<AutocompleteMatchData> matches;
-  EXPECT_TRUE(edit->GetAutocompleteMatches(&matches));
-  EXPECT_FALSE(matches.empty());
-}
-
 // Flaky especially on Windows. See crbug.com/25039.
 TEST_F(AutomationProxyTest, FLAKY_AppModalDialogTest) {
   scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
@@ -1456,12 +1407,11 @@ TEST_F(AutomationProxyTest, FLAKY_AppModalDialogTest) {
   ASSERT_TRUE(tab.get());
 
   bool modal_dialog_showing = false;
-  ui::MessageBoxFlags::DialogButton button =
-      ui::MessageBoxFlags::DIALOGBUTTON_NONE;
+  ui::DialogButton button = ui::DIALOG_BUTTON_NONE;
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_FALSE(modal_dialog_showing);
-  EXPECT_EQ(ui::MessageBoxFlags::DIALOGBUTTON_NONE, button);
+  EXPECT_EQ(ui::DIALOG_BUTTON_NONE, button);
 
   // Show a simple alert.
   std::string content =
@@ -1474,19 +1424,19 @@ TEST_F(AutomationProxyTest, FLAKY_AppModalDialogTest) {
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_TRUE(modal_dialog_showing);
-  EXPECT_EQ(ui::MessageBoxFlags::DIALOGBUTTON_OK, button);
+  EXPECT_EQ(ui::DIALOG_BUTTON_OK, button);
 
   // Test that clicking missing button fails graciously and does not close the
   // dialog.
   EXPECT_FALSE(automation()->ClickAppModalDialogButton(
-      ui::MessageBoxFlags::DIALOGBUTTON_CANCEL));
+      ui::DIALOG_BUTTON_CANCEL));
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_TRUE(modal_dialog_showing);
 
   // Now click OK, that should close the dialog.
   EXPECT_TRUE(automation()->ClickAppModalDialogButton(
-      ui::MessageBoxFlags::DIALOGBUTTON_OK));
+      ui::DIALOG_BUTTON_OK));
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_FALSE(modal_dialog_showing);
@@ -1502,18 +1452,17 @@ TEST_F(AutomationProxyTest, FLAKY_AppModalDialogTest) {
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_TRUE(modal_dialog_showing);
-  EXPECT_EQ(ui::MessageBoxFlags::DIALOGBUTTON_OK |
-            ui::MessageBoxFlags::DIALOGBUTTON_CANCEL, button);
+  EXPECT_EQ(ui::DIALOG_BUTTON_OK | ui::DIALOG_BUTTON_CANCEL, button);
 
   // Click OK.
-  EXPECT_TRUE(automation()->ClickAppModalDialogButton(
-      ui::MessageBoxFlags::DIALOGBUTTON_OK));
+  EXPECT_TRUE(automation()->ClickAppModalDialogButton(ui::DIALOG_BUTTON_OK));
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_FALSE(modal_dialog_showing);
   int result = -1;
   EXPECT_TRUE(tab->ExecuteAndExtractInt(
-      L"", L"window.domAutomationController.send(result);", &result));
+      std::wstring(),
+      L"window.domAutomationController.send(result);", &result));
   EXPECT_EQ(0, result);
 
   // Try again.
@@ -1523,17 +1472,17 @@ TEST_F(AutomationProxyTest, FLAKY_AppModalDialogTest) {
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_TRUE(modal_dialog_showing);
-  EXPECT_EQ(ui::MessageBoxFlags::DIALOGBUTTON_OK |
-            ui::MessageBoxFlags::DIALOGBUTTON_CANCEL, button);
+  EXPECT_EQ(ui::DIALOG_BUTTON_OK | ui::DIALOG_BUTTON_CANCEL, button);
 
   // Click Cancel this time.
   EXPECT_TRUE(automation()->ClickAppModalDialogButton(
-      ui::MessageBoxFlags::DIALOGBUTTON_CANCEL));
+      ui::DIALOG_BUTTON_CANCEL));
   EXPECT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
                                                      &button));
   EXPECT_FALSE(modal_dialog_showing);
   EXPECT_TRUE(tab->ExecuteAndExtractInt(
-      L"", L"window.domAutomationController.send(result);", &result));
+      std::wstring(),
+      L"window.domAutomationController.send(result);", &result));
   EXPECT_EQ(1, result);
 }
 
@@ -1562,7 +1511,7 @@ TEST_F(AutomationProxyTest5, TestLifetimeOfDomAutomationController) {
             tab->NavigateToURL(net::FilePathToFileURL(filename)));
 
   // Allow some time for the popup to show up and close.
-  base::PlatformThread::Sleep(TestTimeouts::action_timeout_ms());
+  base::PlatformThread::Sleep(TestTimeouts::action_timeout());
 
   std::wstring expected(L"string");
   std::wstring jscript = CreateJSString(L"\"" + expected + L"\"");
@@ -1583,8 +1532,7 @@ class AutomationProxySnapshotTest : public UITest {
 
   // Asserts that the given png file can be read and decoded into the given
   // bitmap.
-  void AssertReadPNG(const FilePath& filename, SkBitmap* bitmap) {
-    DCHECK(bitmap);
+  void AssertReadPNG(const FilePath& filename, std::string* data) {
     ASSERT_TRUE(file_util::PathExists(filename));
 
     int64 size64;
@@ -1594,165 +1542,48 @@ class AutomationProxySnapshotTest : public UITest {
 
     // Read and decode image.
     int size = static_cast<int>(size64);
-    scoped_array<char> data(new char[size]);
-    int bytes_read = file_util::ReadFile(filename, &data[0], size);
+    data->resize(size);
+    int bytes_read = file_util::ReadFile(filename, &(*data)[0], size);
     ASSERT_EQ(size, bytes_read);
-    ASSERT_TRUE(gfx::PNGCodec::Decode(
-        reinterpret_cast<unsigned char*>(&data[0]),
-        bytes_read,
-        bitmap));
   }
 
   // Returns the file path for the directory for these tests appended with
   // the given relative path.
-  FilePath GetTestFilePath(const char* relative_path) {
+  FilePath GetTestFilePath(const std::string& relative_path) {
     FilePath filename(test_data_directory_);
     return filename.AppendASCII("automation_proxy_snapshot")
         .AppendASCII(relative_path);
   }
 
+  GURL GetTestUrl(const std::string& relative_path, const std::string& query) {
+    FilePath file_path = GetTestFilePath(relative_path);
+    return ui_test_utils::GetFileUrlWithQuery(file_path, query);
+  }
+
   FilePath snapshot_path_;
   ScopedTempDir snapshot_dir_;
 };
 
-// See http://crbug.com/63022.
-#if defined(OS_LINUX)
-#define MAYBE_ContentSmallerThanView FAILS_ContentSmallerThanView
-#else
-#define MAYBE_ContentSmallerThanView ContentSmallerThanView
-#endif
-// Tests that taking a snapshot when the content is smaller than the view
-// produces a snapshot equal to the view size.
-TEST_F(AutomationProxySnapshotTest, MAYBE_ContentSmallerThanView) {
-  scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
-  ASSERT_TRUE(browser.get());
-
-  scoped_refptr<WindowProxy> window(browser->GetWindow());
-  ASSERT_TRUE(window.get());
-  ASSERT_TRUE(window->SetBounds(gfx::Rect(300, 400)));
-
-  scoped_refptr<TabProxy> tab(browser->GetTab(0));
-  ASSERT_TRUE(tab.get());
-
-  ASSERT_EQ(AUTOMATION_MSG_NAVIGATION_SUCCESS,
-            tab->NavigateToURL(GURL(chrome::kAboutBlankURL)));
-
-  gfx::Rect view_bounds;
-  ASSERT_TRUE(window->GetViewBounds(VIEW_ID_TAB_CONTAINER, &view_bounds,
-                                    false));
-
-  ASSERT_TRUE(tab->CaptureEntirePageAsPNG(snapshot_path_));
-
-  SkBitmap bitmap;
-  ASSERT_NO_FATAL_FAILURE(AssertReadPNG(snapshot_path_, &bitmap));
-  ASSERT_EQ(view_bounds.width(), bitmap.width());
-  ASSERT_EQ(view_bounds.height(), bitmap.height());
-}
-
-// See http://crbug.com/63022.
-#if defined(OS_LINUX)
-#define MAYBE_ContentLargerThanView FAILS_ContentLargerThanView
-#else
-#define MAYBE_ContentLargerThanView ContentLargerThanView
-#endif
 // Tests that taking a snapshot when the content is larger than the view
 // produces a snapshot equal to the content size.
-TEST_F(AutomationProxySnapshotTest, MAYBE_ContentLargerThanView) {
+TEST_F(AutomationProxySnapshotTest, ContentLargerThanView) {
+  const char kReferenceMd5[] = "3d594850fd25cb116338cb3610afe18e";
   scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(browser.get());
-
-  // Resize the window to guarantee that the content is larger than the view.
-  scoped_refptr<WindowProxy> window(browser->GetWindow());
-  ASSERT_TRUE(window.get());
-  ASSERT_TRUE(window->SetBounds(gfx::Rect(300, 400)));
-
   scoped_refptr<TabProxy> tab(browser->GetTab(0));
   ASSERT_TRUE(tab.get());
 
-  FilePath set_size_page = GetTestFilePath("set_size.html?600,800");
   ASSERT_EQ(AUTOMATION_MSG_NAVIGATION_SUCCESS,
-            tab->NavigateToURL(net::FilePathToFileURL(set_size_page)));
+            tab->NavigateToURL(GetTestUrl("set_size.html", "2000,2500")));
 
   ASSERT_TRUE(tab->CaptureEntirePageAsPNG(snapshot_path_));
 
-  SkBitmap bitmap;
-  ASSERT_NO_FATAL_FAILURE(AssertReadPNG(snapshot_path_, &bitmap));
-  ASSERT_EQ(600, bitmap.width());
-  ASSERT_EQ(800, bitmap.height());
-}
-
-// Tests taking a large snapshot works.
-TEST_F(AutomationProxySnapshotTest, LargeSnapshot) {
-  scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
-  ASSERT_TRUE(browser.get());
-
-  scoped_refptr<TabProxy> tab(browser->GetTab(0));
-  ASSERT_TRUE(tab.get());
-
-  // 2000x2000 creates an approximately 15 MB bitmap.
-  // Don't increase this too much. At least my linux box has SHMMAX set at
-  // 32 MB.
-  FilePath set_size_page = GetTestFilePath("set_size.html?2000,2000");
-  ASSERT_EQ(AUTOMATION_MSG_NAVIGATION_SUCCESS,
-            tab->NavigateToURL(net::FilePathToFileURL(set_size_page)));
-
-  ASSERT_TRUE(tab->CaptureEntirePageAsPNG(snapshot_path_));
-
-  SkBitmap bitmap;
-  ASSERT_NO_FATAL_FAILURE(AssertReadPNG(snapshot_path_, &bitmap));
-  ASSERT_EQ(2000, bitmap.width());
-  ASSERT_EQ(2000, bitmap.height());
-}
-
-#if defined(OS_MACOSX)
-// Most pixels on mac are slightly off.
-#define MAYBE_ContentsCorrect DISABLED_ContentsCorrect
-#elif defined(OS_LINUX)
-// See http://crbug.com/63022.
-#define MAYBE_ContentsCorrect FAILS_ContentsCorrect
-#else
-#define MAYBE_ContentsCorrect ContentsCorrect
-#endif
-
-// Tests that the snapshot contents are correct.
-TEST_F(AutomationProxySnapshotTest, MAYBE_ContentsCorrect) {
-  scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
-  ASSERT_TRUE(browser.get());
-
-  const gfx::Size img_size(400, 300);
-  scoped_refptr<WindowProxy> window(browser->GetWindow());
-  ASSERT_TRUE(window.get());
-  ASSERT_TRUE(window->SetBounds(gfx::Rect(img_size)));
-
-  scoped_refptr<TabProxy> tab(browser->GetTab(0));
-  ASSERT_TRUE(tab.get());
-
-  FilePath set_size_page = GetTestFilePath("just_image.html");
-  ASSERT_EQ(AUTOMATION_MSG_NAVIGATION_SUCCESS,
-            tab->NavigateToURL(net::FilePathToFileURL(set_size_page)));
-
-  ASSERT_TRUE(tab->CaptureEntirePageAsPNG(snapshot_path_));
-
-  SkBitmap snapshot_bmp;
-  ASSERT_NO_FATAL_FAILURE(AssertReadPNG(snapshot_path_, &snapshot_bmp));
-  ASSERT_EQ(img_size.width(), snapshot_bmp.width());
-  ASSERT_EQ(img_size.height(), snapshot_bmp.height());
-
-  SkBitmap reference_bmp;
-  ASSERT_NO_FATAL_FAILURE(AssertReadPNG(GetTestFilePath("image.png"),
-                                        &reference_bmp));
-  ASSERT_EQ(img_size.width(), reference_bmp.width());
-  ASSERT_EQ(img_size.height(), reference_bmp.height());
-
-  SkAutoLockPixels lock_snapshot(snapshot_bmp);
-  SkAutoLockPixels lock_reference(reference_bmp);
-  int diff_pixels_count = 0;
-  for (int x = 0; x < img_size.width(); ++x) {
-    for (int y = 0; y < img_size.height(); ++y) {
-      if (*snapshot_bmp.getAddr32(x, y) != *reference_bmp.getAddr32(x, y)) {
-        ++diff_pixels_count;
-      }
-    }
+  std::string data;
+  ASSERT_NO_FATAL_FAILURE(AssertReadPNG(snapshot_path_, &data));
+  EXPECT_STREQ(kReferenceMd5, base::MD5String(data).c_str());
+  if (CommandLine::ForCurrentProcess()->HasSwitch("dump-test-image")) {
+    FilePath path(FILE_PATH_LITERAL("snapshot.png"));
+    EXPECT_EQ(file_util::WriteFile(path, &data[0], data.length()),
+              static_cast<int>(data.length()));
   }
-  ASSERT_EQ(diff_pixels_count, 0);
 }
