@@ -12,9 +12,9 @@
 #include "base/time.h"
 #include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/ui/ui_test.h"
-#include "gfx/native_widget_types.h"
 #include "googleurl/src/gurl.h"
 #include "testing/gmock/include/gmock/gmock.h"
+#include "ui/gfx/native_widget_types.h"
 
 class TabProxy;
 class ExternalTabUITestMockClient;
@@ -81,7 +81,7 @@ class ExternalTabUITestMockClient : public AutomationProxy {
 
   // Navigation using keyboard (TAB + Enter) inside the page.
   void NavigateThroughUserGesture();
-  void IgnoreFavIconNetworkRequest();
+  void IgnoreFaviconNetworkRequest();
 
   void ConnectToExternalTab(gfx::NativeWindow parent,
       const AttachExternalTabParams& attach_params);
@@ -91,13 +91,15 @@ class ExternalTabUITestMockClient : public AutomationProxy {
   void DestroyHostWindow();
   // Returns true if the host window exists.
   bool HostWindowExists();
+
+  static ExternalTabSettings GetDefaultSettings();
+  static AutomationURLResponse GetHttp200Response();
+
   // It's overlapped and visible by default. For Incognito mode test though we
   // want invisible window since SetCookie is a sync call and a deadlock is
   // very possible.
   unsigned long host_window_style_;
 
-  static const AutomationURLResponse http_200;
-  static const ExternalTabSettings default_settings;
  protected:
   gfx::NativeWindow host_window_;
 
