@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,7 +14,6 @@
 #include "chrome/common/content_settings.h"
 #include "chrome/test/automation/automation_handle_tracker.h"
 
-class AutocompleteEditProxy;
 class GURL;
 class TabProxy;
 class WindowProxy;
@@ -52,6 +51,9 @@ class BrowserProxy : public AutomationResourceProxy {
   // Returns true if successful.
   bool AppendTab(const GURL& tab_url) WARN_UNUSED_RESULT;
 
+  // Appends a new tab in the background (as if middle-clicking).
+  bool AppendBackgroundTab(const GURL& tab_url) WARN_UNUSED_RESULT;
+
   // Gets the (zero-based) index of the currently active tab. Returns true if
   // successful.
   bool GetActiveTabIndex(int* active_tab_index) const WARN_UNUSED_RESULT;
@@ -64,6 +66,10 @@ class BrowserProxy : public AutomationResourceProxy {
   // successful.
   bool GetType(Browser::Type* type) const WARN_UNUSED_RESULT;
 
+  // Sets |is_application| to whether the browser is currently in application
+  // mode.
+  bool IsApplication(bool* is_application) WARN_UNUSED_RESULT;
+
   // Returns the TabProxy for the tab at the given index, transferring
   // ownership of the pointer to the caller. On failure, returns NULL.
   //
@@ -81,11 +87,6 @@ class BrowserProxy : public AutomationResourceProxy {
   // On failure, returns NULL.
   scoped_refptr<WindowProxy> GetWindow() const;
 
-  // Returns an AutocompleteEdit for this browser's window. It can be used to
-  // manipulate the omnibox.  The caller owns the returned pointer.
-  // On failure, returns NULL.
-  scoped_refptr<AutocompleteEditProxy> GetAutocompleteEdit();
-
   // Apply the accelerator with given id (IDC_BACK, IDC_NEWTAB ...)
   // The list can be found at chrome/app/chrome_command_ids.h
   // Returns true if the call was successful.
@@ -140,9 +141,12 @@ class BrowserProxy : public AutomationResourceProxy {
   bool RunCommand(int browser_command) const WARN_UNUSED_RESULT;
 
   // Returns whether the Bookmark bar is visible and whether we are animating
-  // it into position. Returns false on failure.
+  // it into position. Also returns whether it is currently detached from the
+  // location bar, as in the NTP.
+  // Returns false on failure.
   bool GetBookmarkBarVisibility(bool* is_visible,
-                                bool* is_animating) WARN_UNUSED_RESULT;
+                                bool* is_animating,
+                                bool* is_detached) WARN_UNUSED_RESULT;
 
   // Get the bookmarks as a JSON string and put it in |json_string|.
   // Return true on success.
@@ -221,8 +225,9 @@ class BrowserProxy : public AutomationResourceProxy {
   bool StartTrackingPopupMenus() WARN_UNUSED_RESULT;
   bool WaitForPopupMenuToOpen() WARN_UNUSED_RESULT;
 
-  // Experimental generic pattern.
+  // Generic pattern for sending automation requests.
   bool SendJSONRequest(const std::string& request,
+                       int timeout_ms,
                        std::string* response) WARN_UNUSED_RESULT;
 
   // Gets the load times for all tabs started from the command line.
@@ -230,8 +235,11 @@ class BrowserProxy : public AutomationResourceProxy {
   // the time when loading stopped into |max_stop_time| (should be similar to
   // the delay that WaitForInitialLoads waits for), and a list of all
   // finished timestamps into |stop_times|. Returns true on success.
-  bool GetInitialLoadTimes(float* min_start_time, float* max_stop_time,
-                           std::vector<float>* stop_times);
+  bool GetInitialLoadTimes(
+      int timeout_ms,
+      float* min_start_time,
+      float* max_stop_time,
+      std::vector<float>* stop_times);
 
 
  protected:
