@@ -1,14 +1,15 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/memory/ref_counted.h"
 #include "base/path_service.h"
-#include "base/ref_counted.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/extension_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
+#include "chrome/test/base/layout_test_http_server.h"
 #include "chrome/test/ui/ui_test.h"
 
 namespace {
@@ -108,8 +109,10 @@ TEST_F(ExtensionProxyUITest, DISABLED_ExecuteBrowserActionInActiveTabAsync) {
   FilePath path;
   // The root directory for the http server does not matter in this case,
   // but we have to pick something.
-  PathService::Get(chrome::DIR_TEST_DATA, &path);
-  StartHttpServerWithPort(path, 1365);
+  ASSERT_TRUE(PathService::Get(chrome::DIR_TEST_DATA, &path));
+  // TODO(phajdan.jr): Use net/test/test_server instead of layout test server.
+  LayoutTestHttpServer http_server(path, 1365);
+  ASSERT_TRUE(http_server.Start());
   GURL localhost = GURL("http://localhost:1365");
   NavigateToURL(localhost);
 
@@ -117,7 +120,13 @@ TEST_F(ExtensionProxyUITest, DISABLED_ExecuteBrowserActionInActiveTabAsync) {
   // the tab's index.
   ASSERT_TRUE(rename_tab_extension->
               ExecuteActionInActiveTabAsync(browser.get()));
-  ASSERT_NO_FATAL_FAILURE(automation()->EnsureExtensionTestResult());
+
+  bool result;
+  std::string message;
+
+  if (!automation()->GetExtensionTestResult(&result, &message))
+    FAIL() << "Could not send WaitForExtensionTestResult message";
+  ASSERT_TRUE(result) << "Extension test message: " << message;
 
   scoped_refptr<TabProxy> display_tab = browser->GetTab(0);
   ASSERT_TRUE(display_tab);
@@ -131,35 +140,41 @@ TEST_F(ExtensionProxyUITest, DISABLED_ExecuteBrowserActionInActiveTabAsync) {
   ASSERT_TRUE(display_tab);
   ASSERT_TRUE(rename_tab_extension->
               ExecuteActionInActiveTabAsync(browser.get()));
-  ASSERT_NO_FATAL_FAILURE(automation()->EnsureExtensionTestResult());
+  if (!automation()->GetExtensionTestResult(&result, &message))
+    FAIL() << "Could not send WaitForExtensionTestResult message";
+  ASSERT_TRUE(result) << "Extension test message: " << message;
   ASSERT_TRUE(display_tab->GetTabTitle(&title_wstring));
   ASSERT_STREQ(L"1", title_wstring.c_str());
 
   // Do not forget to stop the server.
-  StopHttpServer();
+  ASSERT_TRUE(http_server.Stop());
 }
 
 // Flaky, http://crbug.com/59441.
 TEST_F(ExtensionProxyUITest, FLAKY_MoveBrowserAction) {
+  int action_index;
+
   scoped_refptr<ExtensionProxy> rename_tab_extension =
       InstallRenameTabExtension();
   ASSERT_TRUE(rename_tab_extension.get());
-  ASSERT_NO_FATAL_FAILURE(simple_extension_->
-                          EnsureBrowserActionIndexMatches(0));
-  ASSERT_NO_FATAL_FAILURE(rename_tab_extension->
-                          EnsureBrowserActionIndexMatches(1));
+
+  ASSERT_TRUE(simple_extension_->GetBrowserActionIndex(&action_index));
+  ASSERT_EQ(0, action_index);
+  ASSERT_TRUE(rename_tab_extension->GetBrowserActionIndex(&action_index));
+  ASSERT_EQ(1, action_index);
 
   // Move google translate to the end, then beginning, and verify.
   ASSERT_TRUE(simple_extension_->MoveBrowserAction(1));
-  ASSERT_NO_FATAL_FAILURE(simple_extension_->
-                          EnsureBrowserActionIndexMatches(1));
-  ASSERT_NO_FATAL_FAILURE(rename_tab_extension->
-                          EnsureBrowserActionIndexMatches(0));
+  ASSERT_TRUE(simple_extension_->GetBrowserActionIndex(&action_index));
+  ASSERT_EQ(1, action_index);
+  ASSERT_TRUE(rename_tab_extension->GetBrowserActionIndex(&action_index));
+  ASSERT_EQ(0, action_index);
+
   ASSERT_TRUE(simple_extension_->MoveBrowserAction(0));
-  ASSERT_NO_FATAL_FAILURE(simple_extension_->
-                          EnsureBrowserActionIndexMatches(0));
-  ASSERT_NO_FATAL_FAILURE(rename_tab_extension->
-                          EnsureBrowserActionIndexMatches(1));
+  ASSERT_TRUE(simple_extension_->GetBrowserActionIndex(&action_index));
+  ASSERT_EQ(0, action_index);
+  ASSERT_TRUE(rename_tab_extension->GetBrowserActionIndex(&action_index));
+  ASSERT_EQ(1, action_index);
 
   // Try moving browser action to invalid index.
   ASSERT_FALSE(simple_extension_->MoveBrowserAction(-1));
@@ -168,14 +183,21 @@ TEST_F(ExtensionProxyUITest, FLAKY_MoveBrowserAction) {
 
 // Flaky, http://crbug.com/59440.
 TEST_F(ExtensionProxyUITest, FLAKY_GetProperty) {
-  ASSERT_NO_FATAL_FAILURE(simple_extension_->
-                          EnsureIdMatches("aiglobglfckejlcpcbdokbkbjeemfhno"));
-  ASSERT_NO_FATAL_FAILURE(simple_extension_->
-                          EnsureNameMatches("Browser Action"));
-  ASSERT_NO_FATAL_FAILURE(simple_extension_->
-                          EnsureVersionMatches("0.1.1"));
-  ASSERT_NO_FATAL_FAILURE(simple_extension_->
-                          EnsureBrowserActionIndexMatches(0));
+  std::string id;
+  ASSERT_TRUE(simple_extension_->GetId(&id));
+  ASSERT_EQ("aiglobglfckejlcpcbdokbkbjeemfhno", id);
+
+  std::string name;
+  ASSERT_TRUE(simple_extension_->GetName(&name));
+  ASSERT_EQ("Browser Action", name);
+
+  std::string version;
+  ASSERT_TRUE(simple_extension_->GetVersion(&version));
+  ASSERT_EQ("0.1.1", version);
+
+  int browser_action_index;
+  ASSERT_TRUE(simple_extension_->GetBrowserActionIndex(&browser_action_index));
+  ASSERT_EQ(0, browser_action_index);
 }
 
 }  // namespace
