@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,13 +6,16 @@
 
 #include <algorithm>
 
+#include "base/json/json_value_serializer.h"
 #include "base/logging.h"
 #include "base/string16.h"
 #include "base/threading/platform_thread.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/common/automation_constants.h"
 #include "chrome/common/automation_messages.h"
-#include "chrome/common/json_value_serializer.h"
+#include "chrome/test/automation/automation_json_requests.h"
 #include "chrome/test/automation/automation_proxy.h"
+#include "chrome/test/automation/browser_proxy.h"
 #include "googleurl/src/gurl.h"
 
 TabProxy::TabProxy(AutomationMessageSender* sender,
@@ -21,6 +24,29 @@ TabProxy::TabProxy(AutomationMessageSender* sender,
     : AutomationResourceProxy(tracker, sender, handle) {
 }
 
+scoped_refptr<BrowserProxy> TabProxy::GetParentBrowser() const {
+  if (!is_valid())
+    return NULL;
+
+  int browser_handle = 0;
+  bool success = false;
+  sender_->Send(new AutomationMsg_GetParentBrowserOfTab(
+      handle_, &browser_handle, &success));
+  if (!success)
+    return NULL;
+
+  BrowserProxy* browser = static_cast<BrowserProxy*>(
+      tracker_->GetResource(browser_handle));
+  if (!browser) {
+    browser = new BrowserProxy(sender_, tracker_, browser_handle);
+    browser->AddRef();
+  }
+
+  // Since there is no scoped_refptr::attach.
+  scoped_refptr<BrowserProxy> result;
+  result.swap(&browser);
+  return result;
+}
 
 bool TabProxy::GetTabTitle(std::wstring* title) const {
   if (!is_valid())
@@ -59,7 +85,6 @@ int TabProxy::FindInPage(const std::wstring& search_string,
     return -1;
 
   AutomationMsg_Find_Params params;
-  params.unused = 0;
   params.search_string = WideToUTF16Hack(search_string);
   params.find_next = find_next;
   params.match_case = (match_case == CASE_SENSITIVE);
@@ -236,14 +261,13 @@ bool TabProxy::GetProcessID(int* process_id) const {
 bool TabProxy::ExecuteAndExtractString(const std::wstring& frame_xpath,
                                        const std::wstring& jscript,
                                        std::wstring* string_value) {
-  Value* root = NULL;
-  bool succeeded = ExecuteAndExtractValue(frame_xpath, jscript, &root);
-  if (!succeeded)
+  scoped_ptr<Value> root(ExecuteAndExtractValue(frame_xpath, jscript));
+  if (root == NULL)
     return false;
 
   DCHECK(root->IsType(Value::TYPE_LIST));
   Value* value = NULL;
-  succeeded = static_cast<ListValue*>(root)->Get(0, &value);
+  bool succeeded = static_cast<ListValue*>(root.get())->Get(0, &value);
   if (succeeded) {
     string16 read_value;
     succeeded = value->GetAsString(&read_value);
@@ -252,80 +276,66 @@ bool TabProxy::ExecuteAndExtractString(const std::wstring& frame_xpath,
       *string_value = UTF16ToWideHack(read_value);
     }
   }
-
-  delete root;
   return succeeded;
 }
 
 bool TabProxy::ExecuteAndExtractBool(const std::wstring& frame_xpath,
                                      const std::wstring& jscript,
                                      bool* bool_value) {
-  Value* root = NULL;
-  bool succeeded = ExecuteAndExtractValue(frame_xpath, jscript, &root);
-  if (!succeeded)
+  scoped_ptr<Value> root(ExecuteAndExtractValue(frame_xpath, jscript));
+  if (root == NULL)
     return false;
 
   bool read_value = false;
   DCHECK(root->IsType(Value::TYPE_LIST));
   Value* value = NULL;
-  succeeded = static_cast<ListValue*>(root)->Get(0, &value);
+  bool succeeded = static_cast<ListValue*>(root.get())->Get(0, &value);
   if (succeeded) {
     succeeded = value->GetAsBoolean(&read_value);
     if (succeeded) {
       *bool_value = read_value;
     }
   }
-
-  delete value;
   return succeeded;
 }
 
 bool TabProxy::ExecuteAndExtractInt(const std::wstring& frame_xpath,
                                     const std::wstring& jscript,
                                     int* int_value) {
-  Value* root = NULL;
-  bool succeeded = ExecuteAndExtractValue(frame_xpath, jscript, &root);
-  if (!succeeded)
+  scoped_ptr<Value> root(ExecuteAndExtractValue(frame_xpath, jscript));
+  if (root == NULL)
     return false;
 
   int read_value = 0;
   DCHECK(root->IsType(Value::TYPE_LIST));
   Value* value = NULL;
-  succeeded = static_cast<ListValue*>(root)->Get(0, &value);
+  bool succeeded = static_cast<ListValue*>(root.get())->Get(0, &value);
   if (succeeded) {
     succeeded = value->GetAsInteger(&read_value);
     if (succeeded) {
       *int_value = read_value;
     }
   }
-
-  delete value;
   return succeeded;
 }
 
-bool TabProxy::ExecuteAndExtractValue(const std::wstring& frame_xpath,
-                                      const std::wstring& jscript,
-                                      Value** value) {
+Value* TabProxy::ExecuteAndExtractValue(const std::wstring& frame_xpath,
+                                        const std::wstring& jscript) {
   if (!is_valid())
-    return false;
-
-  if (!value) {
-    NOTREACHED();
-    return false;
-  }
+    return NULL;
 
   std::string json;
   if (!sender_->Send(new AutomationMsg_DomOperation(handle_, frame_xpath,
-                                                    jscript, &json)))
-    return false;
+                                                    jscript, &json))) {
+    return NULL;
+  }
   // Wrap |json| in an array before deserializing because valid JSON has an
   // array or an object as the root.
   json.insert(0, "[");
   json.append("]");
 
   JSONStringValueSerializer deserializer(json);
-  *value = deserializer.Deserialize(NULL, NULL);
-  return *value != NULL;
+  return deserializer.Deserialize(NULL, NULL);
 }
 
 DOMElementProxyRef TabProxy::GetDOMDocument() {
@@ -338,15 +348,6 @@ DOMElementProxyRef TabProxy::GetDOMDocument() {
   return GetObjectProxy<DOMElementProxy>(element_handle);
 }
 
-bool TabProxy::SetEnableExtensionAutomation(
-    const std::vector<std::string>& functions_enabled) {
-  if (!is_valid())
-    return false;
-
-  return sender_->Send(new AutomationMsg_SetEnableExtensionAutomation(
-      handle_, functions_enabled));
-}
-
 bool TabProxy::GetConstrainedWindowCount(int* count) const {
   if (!is_valid())
     return false;
@@ -364,7 +365,8 @@ bool TabProxy::WaitForChildWindowCountToChange(int count, int* new_count,
                                                int wait_timeout) {
   int intervals = std::max(wait_timeout / automation::kSleepTime, 1);
   for (int i = 0; i < intervals; ++i) {
-    base::PlatformThread::Sleep(automation::kSleepTime);
+    base::PlatformThread::Sleep(
+        base::TimeDelta::FromMilliseconds(automation::kSleepTime));
     bool succeeded = GetConstrainedWindowCount(new_count);
     if (!succeeded)
       return false;
@@ -392,7 +394,8 @@ bool TabProxy::WaitForBlockedPopupCountToChangeTo(int target_count,
                                                   int wait_timeout) {
   int intervals = std::max(wait_timeout / automation::kSleepTime, 1);
   for (int i = 0; i < intervals; ++i) {
-    base::PlatformThread::Sleep(automation::kSleepTime);
+    base::PlatformThread::Sleep(
+        base::TimeDelta::FromMilliseconds(automation::kSleepTime));
     int new_count = -1;
     bool succeeded = GetBlockedPopupCount(&new_count);
     if (!succeeded)
@@ -563,11 +566,14 @@ void TabProxy::HandleMessageFromExternalHost(const std::string& message,
 bool TabProxy::WaitForTabToBeRestored(uint32 timeout_ms) {
   if (!is_valid())
     return false;
-  return sender_->Send(new AutomationMsg_WaitForTabToBeRestored(handle_));
+  bool succeeded = false;
+  return sender_->Send(
+      new AutomationMsg_WaitForTabToBeRestored(handle_, &succeeded)) &&
+      succeeded;
 }
 
-bool TabProxy::GetSecurityState(SecurityStyle* security_style,
-                                int* ssl_cert_status,
+bool TabProxy::GetSecurityState(content::SecurityStyle* security_style,
+                                net::CertStatus* ssl_cert_status,
                                 int* insecure_content_status) {
   DCHECK(security_style && ssl_cert_status && insecure_content_status);
 
@@ -583,7 +589,7 @@ bool TabProxy::GetSecurityState(SecurityStyle* security_style,
   return succeeded;
 }
 
-bool TabProxy::GetPageType(PageType* type) {
+bool TabProxy::GetPageType(content::PageType* type) {
   DCHECK(type);
 
   if (!is_valid())
@@ -624,7 +630,7 @@ bool TabProxy::PrintAsync() {
 
 bool TabProxy::SavePage(const FilePath& file_name,
                         const FilePath& dir_path,
-                        SavePackage::SavePackageType type) {
+                        content::SavePageType type) {
   if (!is_valid())
     return false;
 
@@ -635,14 +641,14 @@ bool TabProxy::SavePage(const FilePath& file_name,
   return succeeded;
 }
 
-bool TabProxy::GetInfoBarCount(int* count) {
+bool TabProxy::GetInfoBarCount(size_t* count) {
   if (!is_valid())
     return false;
 
   return sender_->Send(new AutomationMsg_GetInfoBarCount(handle_, count));
 }
 
-bool TabProxy::WaitForInfoBarCount(int target_count) {
+bool TabProxy::WaitForInfoBarCount(size_t target_count) {
   if (!is_valid())
     return false;
 
@@ -651,7 +657,7 @@ bool TabProxy::WaitForInfoBarCount(int target_count) {
       handle_, target_count, &success)) && success;
 }
 
-bool TabProxy::ClickInfoBarAccept(int info_bar_index,
+bool TabProxy::ClickInfoBarAccept(size_t info_bar_index,
                                   bool wait_for_navigation) {
   if (!is_valid())
     return false;
@@ -719,17 +725,23 @@ bool TabProxy::CaptureEntirePageAsPNG(const FilePath& path) {
   if (!is_valid())
     return false;
 
-  bool succeeded = false;
-  sender_->Send(new AutomationMsg_CaptureEntirePageAsPNG(handle_, path,
-                                                         &succeeded));
-  return succeeded;
+  int browser_index, tab_index;
+  automation::Error error;
+  if (!SendGetIndicesFromTabHandleJSONRequest(
+         sender_, handle_, &browser_index, &tab_index, &error)) {
+    return false;
+  }
+
+  return SendCaptureEntirePageJSONRequest(
+      sender_, WebViewLocator::ForIndexPair(browser_index, tab_index),
+      path, &error);
 }
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
 void TabProxy::Reposition(HWND window, HWND window_insert_after, int left,
                           int top, int width, int height, int flags,
                           HWND parent_window) {
-  Reposition_Params params = {0};
+  Reposition_Params params;
   params.window = window;
   params.window_insert_after = window_insert_after;
   params.left = left;
@@ -785,7 +797,7 @@ void TabProxy::JavaScriptStressTestControl(int cmd, int param) {
     return;
 
   sender_->Send(new AutomationMsg_JavaScriptStressTestControl(
-      0, handle_, cmd, param));
+      handle_, cmd, param));
 }
 
 void TabProxy::AddObserver(TabProxyDelegate* observer) {
