@@ -1,5 +1,5 @@
-#!/usr/bin/python
-# Copyright (c) 2010 The Chromium Authors. All rights reserved.
+#!/usr/bin/env python
+# Copyright (c) 2011 The Chromium Authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -21,7 +21,7 @@ class ContentTest(pyauto.PyUITest):
     """String check in local file.
 
        For each local filename, tell the browser to load it as a file
-       UEL from the DataDir.  Ask the browser for the loaded html.
+       URL from the DataDir.  Ask the browser for the loaded html.
        Confirm all strings in have_list are found in it.  Confirm all
        strings in nothave_list are NOT found in it.  Assumes only one
        window/tab is open.
@@ -65,7 +65,7 @@ class ContentTest(pyauto.PyUITest):
                                   ['page has no title'],
                                   ['Awesomeness'])
     test_utils.StringContentCheck(self, self.GetTabContents(0, 1),
-                                  ['Awesomeness'],
+                                  ['Title Of Awesomeness'],
                                   ['page has no title'])
     test_utils.StringContentCheck(self, self.GetTabContents(0, 2),
                                   ['Title Of More Awesomeness'],
@@ -75,7 +75,7 @@ class ContentTest(pyauto.PyUITest):
     """Confirm about:version contains some expected content."""
     self.NavigateToURL('about:version')
     test_utils.StringContentCheck(self, self.GetTabContents(),
-                                  ['User Agent', 'Command Line'],
+                                  ['WebKit', 'os_version', 'js_version'],
                                   ['odmomfodfm disfnodugdzuoufgbn ifdnf fif'])
 
   def testHttpsPage(self):
@@ -84,22 +84,12 @@ class ContentTest(pyauto.PyUITest):
     self.NavigateToURL(url)
     html_regular = self.GetTabContents()
     self.assertTrue('Google Search' in html_regular)
-    self.assertTrue('I&#39;m Feeling Lucky' in html_regular)
+    self.assertTrue('Feeling Lucky' in html_regular)
     self.RunCommand(pyauto.IDC_NEW_INCOGNITO_WINDOW)
     self.NavigateToURL(url, 1, 0)
     html_incognito = self.GetTabContents(0, 1)
     self.assertTrue('Google Search' in html_incognito)
-    self.assertTrue('I&#39;m Feeling Lucky' in html_incognito)
-
-  def testTopSitesContent(self):
-    """Test content in TopSites and Verify chrome is not getting blocked by
-       user-agent test by those websites."""
-    topsites_file = os.path.join(self.DataDir(), 'content', 'topsites_content')
-    topsites = self.EvalDataFrom(topsites_file)
-    for (url, name) in topsites.iteritems():
-      self.NavigateToURL(url)
-      html = self.GetTabContents()
-      self.assertTrue(name in html)
+    self.assertTrue('Feeling Lucky' in html_incognito)
 
 
 if __name__ == '__main__':
