@@ -1,5 +1,5 @@
-#!/usr/bin/python
-# Copyright (c) 2010 The Chromium Authors. All rights reserved.
+#!/usr/bin/env python
+# Copyright (c) 2011 The Chromium Authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -20,10 +20,16 @@ class CrashReporterTest(pyauto.PyUITest):
     Attempts to crash, and then checks that crash dumps get generated.  Does
     not actually test crash reports on the server.
     """
-    # bail out if not a branded build
+    # Bail out if not a branded build
     properties = self.GetBrowserInfo()['properties']
     if properties['branding'] != 'Google Chrome':
       return
+
+    # Make sure Chrome minidumps are enabled on Chrome OS
+    if self.IsChromeOS():
+      minidumps_file = '/mnt/stateful_partition/etc/enable_chromium_minidumps'
+      assert os.path.exists(minidumps_file), 'Chrome minidumps are not enabled.'
+
     breakpad_folder = properties['DIR_CRASH_DUMPS']
     self.assertTrue(breakpad_folder, 'Cannot figure crash dir')
 
@@ -35,4 +41,3 @@ class CrashReporterTest(pyauto.PyUITest):
 
 if __name__ == '__main__':
   pyauto_functional.Main()
-
