@@ -1,5 +1,5 @@
-#!/usr/bin/python
-# Copyright (c) 2010 The Chromium Authors. All rights reserved.
+#!/usr/bin/env python
+# Copyright (c) 2012 The Chromium Authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -64,9 +64,9 @@ class DownloadsTest(pyauto.PyUITest):
   def _GetDangerousDownload(self):
     """Returns the file path for a dangerous download for this OS."""
     sub_path = os.path.join(self.DataDir(), 'downloads', 'dangerous')
-    if self.IsMac():
-      return os.path.join(sub_path, 'invalid-dummy.dmg')
-    return os.path.join(sub_path, 'dangerous.exe')
+    if self.IsWin():
+      return os.path.join(sub_path, 'dangerous.com')
+    return os.path.join(sub_path, 'dangerous.jar')
 
   def _EqualFileContents(self, file1, file2):
     """Determine if 2 given files have the same contents."""
@@ -84,14 +84,22 @@ class DownloadsTest(pyauto.PyUITest):
     return self.GetDownloadsInfo().Downloads()[download_index]['id']
 
   def _MakeFile(self, size):
-    """Make a file on-the-fly with the given size. Returns the path to the
-       file.
+    """Make a file on-the-fly with the given size.
+
+    Note that it's really a 1byte file even though ls -lh will report it as
+    of file |size| (du reports the correct usage on disk), but it's good
+    enough for downloads tests because chrome will treat it as a file of size
+    |size| when downloading.
+
+    Returns:
+        the path to the created file.
     """
     fd, file_path = tempfile.mkstemp(suffix='.zip', prefix='file-downloads-')
     os.lseek(fd, size, 0)
     os.write(fd, 'a')
     os.close(fd)
     logging.debug('Created temporary file %s of size %d' % (file_path, size))
+    self._DeleteAfterShutdown(file_path)
     return file_path
 
   def _GetAllDownloadIDs(self):
@@ -136,8 +144,8 @@ class DownloadsTest(pyauto.PyUITest):
     self.RunCommand(pyauto.IDC_NEW_INCOGNITO_WINDOW)
 
     # Trigger download and wait in new incognito window.
-    self.DownloadAndWaitForStart(file_url, 1)
-    self.WaitForAllDownloadsToComplete(1)
+    self.DownloadAndWaitForStart(file_url, windex=1)
+    self.WaitForAllDownloadsToComplete(windex=1)
     incognito_downloads = self.GetDownloadsInfo(1).Downloads()
 
     # Verify that download info exists in the correct profile.
@@ -157,7 +165,7 @@ class DownloadsTest(pyauto.PyUITest):
     self._TriggerUnsafeDownload(os.path.basename(file_path))
     self.PerformActionOnDownload(self._GetDownloadId(),
                                  'save_dangerous_download')
-    self.WaitForDownloadToComplete(downloaded_pkg)
+    self.WaitForAllDownloadsToComplete()
 
     # Verify that the file was downloaded.
     self.assertTrue(os.path.exists(downloaded_pkg))
@@ -179,14 +187,13 @@ class DownloadsTest(pyauto.PyUITest):
 
   def testRemoveDownload(self):
     """Verify that we can remove a download."""
-    test_dir = os.path.join(os.path.abspath(self.DataDir()), 'downloads')
-    file_path = os.path.join(test_dir, 'a_zip_file.zip')
-    file_url = self.GetFileURLForPath(file_path)
+    file_url = self.GetFileURLForDataPath('downloads', 'a_zip_file.zip')
     downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
                                   'a_zip_file.zip')
     self._ClearLocalDownloadState(downloaded_pkg)
 
     self.DownloadAndWaitForStart(file_url)
+    self.WaitForAllDownloadsToComplete()
     self.PerformActionOnDownload(self._GetDownloadId(), 'remove')
 
     # The download is removed from downloads, but not from the disk.
@@ -205,17 +212,17 @@ class DownloadsTest(pyauto.PyUITest):
     """
     # Create a 1 GB file on the fly
     file_path = self._MakeFile(2**30)
-    self._DeleteAfterShutdown(file_path)
+    # Ensure there's sufficient space remaining to download file.
+    free_space = test_utils.GetFreeSpace(self.GetDownloadDirectory().value())
+    assert free_space >= 2**30, \
+        'Not enough disk space to download. Got %d free' % free_space
     file_url = self.GetFileURLForPath(file_path)
     downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
                                   os.path.basename(file_path))
     self._ClearLocalDownloadState(downloaded_pkg)
     self.DownloadAndWaitForStart(file_url)
     self._DeleteAfterShutdown(downloaded_pkg)
-    # Waiting for big file to download might exceed automation timeout.
-    # Temporarily increase the automation timeout.
-    test_utils.CallFunctionWithNewTimeout(self, 4 * 60 * 1000,  # 4 min.
-                                          self.WaitForAllDownloadsToComplete)
+    self.WaitForAllDownloadsToComplete(timeout=self.large_test_timeout_ms());
     # Verify that the file was correctly downloaded
     self.assertTrue(os.path.exists(downloaded_pkg),
                     'Downloaded file %s missing.' % downloaded_pkg)
@@ -324,21 +331,6 @@ class DownloadsTest(pyauto.PyUITest):
     self.assertTrue(self.WaitUntil(
         lambda: len(self.GetDownloadsInfo().Downloads()) == num_downloads + 1))
 
-  def testNoUnsafeDownloadsOnRestart(self):
-    """Verify that unsafe file should not show up on session restart."""
-    file_path = self._GetDangerousDownload()
-    downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
-                                  os.path.basename(file_path))
-    self._ClearLocalDownloadState(downloaded_pkg)
-    self._TriggerUnsafeDownload(os.path.basename(file_path))
-    self.assertTrue(self.IsDownloadShelfVisible())
-    # Restart the browser and assert that the download was removed.
-    self.RestartBrowser(clear_profile=False)
-    self.assertFalse(os.path.exists(downloaded_pkg))
-    self.assertFalse(self.IsDownloadShelfVisible())
-    self.NavigateToURL("chrome://downloads")
-    self.assertFalse(self.GetDownloadsInfo().Downloads())
-
   def testPauseAndResume(self):
     """Verify that pause and resume work while downloading a file.
 
@@ -347,6 +339,10 @@ class DownloadsTest(pyauto.PyUITest):
     """
     # Create a 250 MB file on the fly
     file_path = self._MakeFile(2**28)
+    # Ensure there's sufficient space remaining to download file.
+    free_space = test_utils.GetFreeSpace(self.GetDownloadDirectory().value())
+    assert free_space >= 2**28, \
+        'Not enough disk space to download. Got %d free' % free_space
 
     file_url = self.GetFileURLForPath(file_path)
     downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
@@ -371,11 +367,7 @@ class DownloadsTest(pyauto.PyUITest):
     resume_dict = self.PerformActionOnDownload(self._GetDownloadId(),
                                                'toggle_pause')
     self.assertFalse(resume_dict['is_paused'])
-
-    # Waiting for big file to download might exceed automation timeout.
-    # Temporarily increase the automation timeout.
-    test_utils.CallFunctionWithNewTimeout(self, 2 * 60 * 1000,  # 2 min.
-                                          self.WaitForAllDownloadsToComplete)
+    self.WaitForAllDownloadsToComplete(timeout=self.large_test_timeout_ms());
 
     # Verify that the file was correctly downloaded after pause and resume.
     self.assertTrue(os.path.exists(downloaded_pkg),
@@ -389,6 +381,10 @@ class DownloadsTest(pyauto.PyUITest):
     # Create a big file (250 MB) on the fly, so that the download won't finish
     # before being cancelled.
     file_path = self._MakeFile(2**28)
+    # Ensure there's sufficient space remaining to download file.
+    free_space = test_utils.GetFreeSpace(self.GetDownloadDirectory().value())
+    assert free_space >= 2**28, \
+        'Not enough disk space to download. Got %d free' % free_space
     file_url = self.GetFileURLForPath(file_path)
     downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
                                   os.path.basename(file_path))
@@ -415,6 +411,7 @@ class DownloadsTest(pyauto.PyUITest):
                                   'a_zip_file.zip')
     self._ClearLocalDownloadState(downloaded_pkg)
     self.DownloadAndWaitForStart(file_url)
+    self.WaitForAllDownloadsToComplete()
     downloads = self.GetDownloadsInfo().Downloads()
     self.assertEqual(1, len(downloads))
     self.assertEqual('a_zip_file.zip', downloads[0]['file_name'])
@@ -448,42 +445,6 @@ class DownloadsTest(pyauto.PyUITest):
     self.assertTrue(self.WaitUntil(lambda path: not os.path.exists(path),
                                    args=[downloaded_pkg]))
 
-  def testAlwaysOpenFileType(self):
-    """Verify "Always Open Files of this Type" download option
-
-    If 'always open' option is set for any filetype, downloading that type of
-    file gets opened always after the download.
-    A cross-platform trick to verify it, by downloading a .zip file and
-    expecting it to get unzipped.  Just check if it got unzipped or not.
-    This way you won't have to worry about which application might 'open'
-    it.
-    """
-    if not self.IsMac():
-      logging.info('Don\'t have a standard way to test when a file "opened"')
-      logging.info('Bailing out')
-      return
-    file_path = os.path.join(self.DataDir(), 'downloads', 'a_zip_file.zip')
-    file_url = self.GetFileURLForPath(file_path)
-    downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
-                                  os.path.basename(file_path))
-    os.path.exists(downloaded_pkg) and os.remove(downloaded_pkg)
-    self.DownloadAndWaitForStart(file_url)
-    self.WaitForAllDownloadsToComplete()
-    id = self._GetDownloadId()
-    self.PerformActionOnDownload(id, 'toggle_open_files_like_this')
-    # Retesting the flag we set
-    file_url2 = self.GetFileURLForDataPath(os.path.join('zip', 'test.zip'))
-    unzip_path = os.path.join(self.GetDownloadDirectory().value(),
-                              'test', 'foo')
-    os.path.exists(unzip_path) and pyauto_utils.RemovePath(unzip_path)
-    self.DownloadAndWaitForStart(file_url2)
-    self.WaitForAllDownloadsToComplete()
-    # When the downloaded zip gets 'opened', a_file.txt will become available.
-    self.assertTrue(self.WaitUntil(lambda: os.path.exists(unzip_path)),
-                    'Did not open the filetype')
-    os.path.exists(downloaded_pkg) and os.remove(downloaded_pkg)
-    os.path.exists(unzip_path) and pyauto_utils.RemovePath(unzip_path)
-
   def testExtendedAttributesOnMac(self):
     """Verify that Chrome sets the extended attributes on a file.
        This test is for mac only.
@@ -501,44 +462,14 @@ class DownloadsTest(pyauto.PyUITest):
     import xattr
     self.assertTrue('com.apple.quarantine' in xattr.listxattr(downloaded_pkg))
 
-  def testOpenWhenDone(self):
-    """Verify "Open When Done" download option.
-
-    Test creates a zip file on the fly and downloads it.
-    Set this option when file is downloading. Once file is downloaded,
-    verify that downloaded zip file is unzipped.
-    """
-    if not self.IsMac():
-      logging.info('Don\'t have a standard way to test when a file "opened"')
-      logging.info('Bailing out')
-      return
-    # Creating a temp zip file.
-    file_path = self._MakeFile(2**24)
-    file_url = self.GetFileURLForPath(file_path)
-    downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
-                                  os.path.basename(file_path))
-    os.path.exists(downloaded_pkg) and os.remove(downloaded_pkg)
-    self.DownloadAndWaitForStart(file_url)
-    id = self._GetDownloadId()
-    self.PerformActionOnDownload(id, 'open')
-    self.WaitForAllDownloadsToComplete()
-    unzip_file_name = downloaded_pkg + '.cpgz'
-    # Verify that the file was correctly downloaded.
-    self.assertTrue(self.WaitUntil(lambda: os.path.exists(unzip_file_name)),
-                    'Unzipped folder %s missing.' % unzip_file_name)
-    self.assertTrue(os.path.exists(downloaded_pkg),
-                    'Downloaded file %s missing.' % downloaded_pkg)
-    self.assertTrue(self._EqualFileContents(file_path, downloaded_pkg),
-                    'Downloaded file %s does not match original' %
-                      downloaded_pkg)
-    os.path.exists(file_path) and os.remove(file_path)
-    os.path.exists(downloaded_pkg) and os.remove(downloaded_pkg)
-    os.path.exists(unzip_file_name) and os.remove(unzip_file_name)
-
   def testDownloadPercentage(self):
     """Verify that during downloading, % values increases,
        and once download is over, % value is 100"""
     file_path = self._MakeFile(2**24)
+    # Ensure there's sufficient space remaining to download file.
+    free_space = test_utils.GetFreeSpace(self.GetDownloadDirectory().value())
+    assert free_space >= 2**24, \
+        'Not enough disk space to download. Got %d free' % free_space
     file_url = self.GetFileURLForPath(file_path)
     downloaded_pkg = os.path.join(self.GetDownloadDirectory().value(),
                                   os.path.basename(file_path))
@@ -577,11 +508,11 @@ class DownloadsTest(pyauto.PyUITest):
     self._ClearLocalDownloadState(downloaded_pkg_incog)
 
     self.DownloadAndWaitForStart(file_url, 0)
-    self.WaitForAllDownloadsToComplete(0)
+    self.WaitForAllDownloadsToComplete(windex=0)
 
     self.RunCommand(pyauto.IDC_NEW_INCOGNITO_WINDOW)
     self.DownloadAndWaitForStart(file_url, 1)
-    self.WaitForAllDownloadsToComplete(1)
+    self.WaitForAllDownloadsToComplete(windex=1)
 
     # Verify download in regular window.
     self.assertTrue(os.path.exists(downloaded_pkg_regul))
