@@ -1,5 +1,5 @@
-#!/usr/bin/python
-# Copyright (c) 2010 The Chromium Authors. All rights reserved.
+#!/usr/bin/env python
+# Copyright (c) 2011 The Chromium Authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -14,7 +14,7 @@ class NotificationsTest(pyauto.PyUITest):
   """Test of HTML5 desktop notifications."""
   def __init__(self, methodName='runTest'):
     super(NotificationsTest, self).__init__(methodName)
-    self.NO_SUCH_URL = 'http://no_such_url_exists/'
+    self.EMPTY_PAGE_URL = self.GetHttpURLForDataPath('empty.html')
     # Content settings for default notification permission.
     self.ALLOW_ALL_SETTING = 1
     self.DENY_ALL_SETTING = 2
@@ -33,22 +33,21 @@ class NotificationsTest(pyauto.PyUITest):
       raw_input('Interact with the browser and hit <enter> to dump notification'
                 'state...')
       print '*' * 20
-      import pprint
-      pp = pprint.PrettyPrinter(indent=2)
-      pp.pprint(self.GetActiveNotifications())
+      self.pprint(self.GetActiveNotifications())
+      self.pprint(self._GetDefaultPermissionSetting())
 
   def _SetDefaultPermissionSetting(self, setting):
     """Sets the default setting for whether sites are allowed to create
     notifications.
     """
-    self.SetPrefs(pyauto.kDesktopNotificationDefaultContentSetting, setting)
+    self.SetPrefs(pyauto.kDefaultContentSettings, {u'notifications': setting})
 
   def _GetDefaultPermissionSetting(self):
     """Gets the default setting for whether sites are allowed to create
     notifications.
     """
     return self.GetPrefsInfo().Prefs(
-        pyauto.kDesktopNotificationDefaultContentSetting)
+        pyauto.kDefaultContentSettings)[u'notifications']
 
   def _GetDeniedOrigins(self):
     """Gets the list of origins that are explicitly denied to create
@@ -160,8 +159,8 @@ class NotificationsTest(pyauto.PyUITest):
                                    tab_index,
                                    windex);
 
-  def _CreateHTMLNotification(self, content_url, replace_id='', tab_index=0,
-                              windex=0):
+  def _CreateHTMLNotification(self, content_url, replace_id='',
+                              wait_for_display=True, tab_index=0, windex=0):
     """Creates an HTML notification.
 
     Returns the id of the notification, which can be used to cancel it later.
@@ -175,11 +174,12 @@ class NotificationsTest(pyauto.PyUITest):
       replace_id: id string to be used for this notification. If another
                   notification is shown with the same replace_id, the former
                   will be replaced.
+      wait_for_display: whether we should wait for the notification to display
       tab_index: index of the tab within the given window
       windex: index of the window
     """
     return self.CallJavascriptFunc('createHTMLNotification',
-                                   [content_url, replace_id],
+                                   [content_url, replace_id, wait_for_display],
                                    tab_index,
                                    windex)
 
@@ -233,10 +233,10 @@ class NotificationsTest(pyauto.PyUITest):
     """Creates an HTML notification using a fake url."""
     self._AllowAllOrigins()
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(1, len(self.GetActiveNotifications()))
     notification = self.GetActiveNotifications()[0]
-    self.assertEquals(self.NO_SUCH_URL, notification['content_url'])
+    self.assertEquals(self.EMPTY_PAGE_URL, notification['content_url'])
     self.assertEquals('', notification['display_source'])
     self.assertEquals('file:///', notification['origin_url'])
 
@@ -244,7 +244,7 @@ class NotificationsTest(pyauto.PyUITest):
     """Creates a notification and closes it."""
     self._AllowAllOrigins()
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.CloseNotification(0)
     self.assertFalse(self.GetActiveNotifications())
 
@@ -252,7 +252,7 @@ class NotificationsTest(pyauto.PyUITest):
     """Creates a notification and cancels it in the origin page."""
     self._AllowAllOrigins()
     self.NavigateToURL(self.TEST_PAGE_URL)
-    note_id = self._CreateHTMLNotification(self.NO_SUCH_URL)
+    note_id = self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertNotEquals(-1, note_id)
     self.WaitForNotificationCount(1)
     self._CancelNotification(note_id)
@@ -270,20 +270,20 @@ class NotificationsTest(pyauto.PyUITest):
     """Tries to create a notification and clicks allow on the infobar."""
     self.NavigateToURL(self.TEST_PAGE_URL)
     # This notification should not be shown because we do not have permission.
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertFalse(self.GetActiveNotifications())
 
     self._RequestPermission()
     self.assertTrue(self.WaitForInfobarCount(1))
     self.PerformActionOnInfobar('accept', 0)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.WaitForNotificationCount(1)
 
   def testOriginPreferencesBasic(self):
     """Tests that we can allow and deny origins."""
-    altavista = 'www.altavista.com'
-    gmail = 'www.gmail.com'
-    yahoo = 'www.yahoo.com'
+    altavista = 'http://www.altavista.com'
+    gmail = 'http://www.gmail.com'
+    yahoo = 'http://www.yahoo.com'
     self._SetDeniedOrigins([altavista, gmail])
     self.assertEquals(altavista, self._GetDeniedOrigins()[0])
     self.assertEquals(gmail, self._GetDeniedOrigins()[1])
@@ -322,7 +322,7 @@ class NotificationsTest(pyauto.PyUITest):
     self._RequestPermission()
     self.assertTrue(self.WaitForInfobarCount(1))
     self.PerformActionOnInfobar('cancel', 0)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertFalse(self.GetActiveNotifications())
     self.assertEquals(['file:///'], self._GetDeniedOrigins())
 
@@ -333,7 +333,7 @@ class NotificationsTest(pyauto.PyUITest):
     self._RequestPermission()
     self.assertTrue(self.WaitForInfobarCount(1))
     self.PerformActionOnInfobar('dismiss', 0)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertFalse(self.GetActiveNotifications())
     self.assertFalse(self._GetDeniedOrigins())
 
@@ -351,7 +351,7 @@ class NotificationsTest(pyauto.PyUITest):
     """Verify that all domains can be allowed to show notifications."""
     self._SetDefaultPermissionSetting(self.ALLOW_ALL_SETTING)
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(1, len(self.GetActiveNotifications()))
     self.assertFalse(self.GetBrowserInfo()['windows'][0]['tabs'][0]['infobars'])
 
@@ -359,7 +359,7 @@ class NotificationsTest(pyauto.PyUITest):
     """Verify that no domain can show notifications."""
     self._SetDefaultPermissionSetting(self.DENY_ALL_SETTING)
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertFalse(self.GetActiveNotifications())
 
   def testDenyDomainAndAllowAll(self):
@@ -368,7 +368,7 @@ class NotificationsTest(pyauto.PyUITest):
     self._DenyOrigin('file:///')
     self._SetDefaultPermissionSetting(self.ALLOW_ALL_SETTING)
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertFalse(self.GetActiveNotifications())
 
   def testAllowDomainAndDenyAll(self):
@@ -377,7 +377,7 @@ class NotificationsTest(pyauto.PyUITest):
     self._AllowOrigin('file:///')
     self._SetDefaultPermissionSetting(self.DENY_ALL_SETTING)
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(1, len(self.GetActiveNotifications()))
     self.assertFalse(self.GetBrowserInfo()['windows'][0]['tabs'][0]['infobars'])
 
@@ -385,10 +385,10 @@ class NotificationsTest(pyauto.PyUITest):
     """Verify that denying and again allowing should show notifications."""
     self._DenyOrigin('file:///')
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(len(self.GetActiveNotifications()), 0)
     self._AllowOrigin('file:///')
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(1, len(self.GetActiveNotifications()))
     self.assertFalse(self.GetBrowserInfo()['windows'][0]['tabs'][0]['infobars'])
 
@@ -396,7 +396,7 @@ class NotificationsTest(pyauto.PyUITest):
     """Verify able to create, deny, and close the notification."""
     self._AllowAllOrigins()
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(1, len(self.GetActiveNotifications()))
     origin = 'file:///'
     self._DenyOrigin(origin)
@@ -412,13 +412,17 @@ class NotificationsTest(pyauto.PyUITest):
     self.assertTrue(self.WaitForInfobarCount(1, windex=1))
     self.PerformActionOnInfobar('cancel', 0, windex=1)
 
+    self.CloseBrowserWindow(1)
+    self.RunCommand(pyauto.IDC_NEW_INCOGNITO_WINDOW)
     self.NavigateToURL(self.TEST_PAGE_URL, 1, 0)
     self._RequestPermission(windex=1)
     self.assertTrue(self.WaitForInfobarCount(1, windex=1))
     self.PerformActionOnInfobar('accept', 0, windex=1)
-    self._CreateHTMLNotification(self.NO_SUCH_URL, windex=1)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL, windex=1)
     self.assertEquals(1, len(self.GetActiveNotifications()))
 
+    self.CloseBrowserWindow(1)
+    self.RunCommand(pyauto.IDC_NEW_INCOGNITO_WINDOW)
     self.NavigateToURL(self.TEST_PAGE_URL, 1, 0)
     self._RequestPermission(windex=1)
     self.assertTrue(self.WaitForInfobarCount(1, windex=1))
@@ -434,21 +438,20 @@ class NotificationsTest(pyauto.PyUITest):
 
   def testCrashTabWithPermissionInfobar(self):
     """Test crashing the tab with permission infobar doesn't crash Chrome."""
-    self.AppendTab(pyauto.GURL(self.NO_SUCH_URL))
+    self.AppendTab(pyauto.GURL(self.EMPTY_PAGE_URL))
     self.assertTrue(self.ActivateTab(0))
     self.NavigateToURL(self.TEST_PAGE_URL)
     self._RequestPermission()
     self.assertTrue(self.WaitForInfobarCount(1))
-    self.Kill(self.GetBrowserInfo()['windows'][0]['tabs'][0]['renderer_pid'])
-    self.assertTrue(self.IsBrowserRunning())
+    self.KillRendererProcess(
+        self.GetBrowserInfo()['windows'][0]['tabs'][0]['renderer_pid'])
 
   def testKillNotificationProcess(self):
     """Test killing a notification doesn't crash Chrome."""
     self._AllowAllOrigins()
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
-    self.Kill(self.GetActiveNotifications()[0]['pid'])
-    self.assertTrue(self.IsBrowserRunning())
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
+    self.KillRendererProcess(self.GetActiveNotifications()[0]['pid'])
     self.WaitForNotificationCount(0)
 
   def testIncognitoNotification(self):
@@ -459,14 +462,15 @@ class NotificationsTest(pyauto.PyUITest):
     self._RequestPermission(windex=1)
     self.assertTrue(self.WaitForInfobarCount(1, windex=1))
     self.PerformActionOnInfobar('accept', infobar_index=0, windex=1)
-    self._CreateHTMLNotification(self.NO_SUCH_URL, windex=1)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL, windex=1)
     self.assertEquals(1, len(self.GetActiveNotifications()))
 
   def testSpecialURLNotification(self):
     """Test a page cannot create a notification to a chrome: url."""
     self._AllowAllOrigins()
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification('chrome://extensions/')
+    self._CreateHTMLNotification('chrome://settings',
+                                 wait_for_display=False);
     self.assertFalse(self.GetActiveNotifications())
 
   def testCloseTabWithPermissionInfobar(self):
@@ -477,7 +481,6 @@ class NotificationsTest(pyauto.PyUITest):
     self._RequestPermission()
     self.assertTrue(self.WaitForInfobarCount(1))
     self.GetBrowserWindow(0).GetTab(0).Close(True)
-    self.assertTrue(self.IsBrowserRunning())
 
   def testNavigateAwayWithPermissionInfobar(self):
     """Test navigating away when an infobar is present, then trying to create a
@@ -491,7 +494,7 @@ class NotificationsTest(pyauto.PyUITest):
     self._RequestPermission()
     self.assertTrue(self.WaitForInfobarCount(1))
     self.PerformActionOnInfobar('accept', 0)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(1, len(self.GetActiveNotifications()))
 
   def testCrashRendererNotificationRemain(self):
@@ -500,28 +503,30 @@ class NotificationsTest(pyauto.PyUITest):
     self.AppendTab(pyauto.GURL('about:blank'))
     self.ActivateTab(0)
     self.NavigateToURL(self.TEST_PAGE_URL)
-    self._CreateHTMLNotification(self.NO_SUCH_URL)
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL)
     self.assertEquals(1, len(self.GetActiveNotifications()))
-    self.Kill(self.GetBrowserInfo()['windows'][0]['tabs'][0]['renderer_pid'])
-    self.assertTrue(self.IsBrowserRunning())
+    self.KillRendererProcess(
+        self.GetBrowserInfo()['windows'][0]['tabs'][0]['renderer_pid'])
     self.assertEquals(1, len(self.GetActiveNotifications()))
 
   def testNotificationOrderAfterClosingOne(self):
     """Tests that closing a notification leaves the rest
     of the notifications in the correct order.
     """
+    if self.IsWin7():
+      return  # crbug.com/66072
     self._AllowAllOrigins()
     self.NavigateToURL(self.TEST_PAGE_URL)
     self._CreateSimpleNotification('', 'Title1', '')
     self._CreateSimpleNotification('', 'Title2', '')
     self._CreateSimpleNotification('', 'Title3', '')
-    self.assertEquals(3, len(self.GetActiveNotifications()))
-    old_notifications = self.GetActiveNotifications()
+    old_notifications = self.GetAllNotifications()
+    self.assertEquals(3, len(old_notifications))
     self.CloseNotification(1)
-    self.assertEquals(2, len(self.GetActiveNotifications()))
-    new_notifications = self.GetActiveNotifications()
-    self.assertEquals(old_notifications[0], new_notifications[0])
-    self.assertEquals(old_notifications[2], new_notifications[1])
+    new_notifications = self.GetAllNotifications()
+    self.assertEquals(2, len(new_notifications))
+    self.assertEquals(old_notifications[0]['id'], new_notifications[0]['id'])
+    self.assertEquals(old_notifications[2]['id'], new_notifications[1]['id'])
 
   def testNotificationReplacement(self):
     """Test that we can replace a notification using the replaceId."""
@@ -531,10 +536,10 @@ class NotificationsTest(pyauto.PyUITest):
     self.WaitForNotificationCount(1)
     # Since this notification has the same replaceId, 'chat', it should replace
     # the first notification.
-    self._CreateHTMLNotification(self.NO_SUCH_URL, 'chat')
+    self._CreateHTMLNotification(self.EMPTY_PAGE_URL, 'chat')
     notifications = self.GetActiveNotifications()
     self.assertEquals(1, len(notifications))
-    self.assertEquals(self.NO_SUCH_URL, notifications[0]['content_url'])
+    self.assertEquals(self.EMPTY_PAGE_URL, notifications[0]['content_url'])
 
 
 if __name__ == '__main__':
