@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,7 +10,8 @@
 #include "chrome/common/chrome_switches.h"
 #include "chrome/installer/util/util_constants.h"
 #include "chrome/test/mini_installer_test/mini_installer_test_constants.h"
-#include "chrome_mini_installer.h"
+#include "chrome/test/mini_installer_test/installer_test_util.h"
+
 
 void BackUpProfile(bool chrome_frame) {
   if (base::GetProcessCount(L"chrome.exe", NULL) > 0) {
@@ -18,10 +19,8 @@ void BackUpProfile(bool chrome_frame) {
            "Please close Chrome and run the tests again.\n");
     exit(1);
   }
-  ChromeMiniInstaller installer(mini_installer_constants::kUserInstall,
-                                chrome_frame);
-  FilePath path =
-      FilePath::FromWStringHack(installer.GetChromeInstallDirectoryLocation());
+  FilePath path;
+  installer_test::GetChromeInstallDirectory(false /* system_level */, &path);
   path = path.Append(mini_installer_constants::kChromeAppDir).DirName();
   FilePath backup_path = path;
   // Will hold User Data path that needs to be backed-up.
@@ -47,19 +46,12 @@ int main(int argc, char** argv) {
   CommandLine::Init(argc, argv);
   const CommandLine& command_line = *CommandLine::ForCurrentProcess();
   base::TestSuite test_suite(argc, argv);
-  if (command_line.HasSwitch(switches::kInstallerTestClean)) {
-    printf("Current version of Chrome will be uninstalled "
-           "from all levels before proceeding with tests.\n");
-  } else if (command_line.HasSwitch(switches::kInstallerTestBackup)) {
-    BackUpProfile(command_line.HasSwitch(
-        installer::switches::kChromeFrame));
-  } else {
+
+  if (command_line.HasSwitch(switches::kInstallerHelp)) {
     printf("This test needs command line arguments.\n");
-    printf("Usage: %ls -{clean|backup} [-build <version>] [-force] \n",
+    printf("Usage: %ls [-backup] [-build <version>] [-force] \n",
            command_line.GetProgram().value().c_str());
-    printf("-clean arg will uninstall your chrome at all levels"
-           " and also delete profile.\n"
-           "-backup arg will make a copy of User Data before uninstalling"
+    printf("-backup arg will make a copy of User Data before uninstalling"
            " your chrome at all levels. The copy will be named as"
            " User Data Copy.\n"
            "-build specifies the build to be tested, e.g., 3.0.195.24."
@@ -70,11 +62,15 @@ int main(int argc, char** argv) {
     return 1;
   }
 
+  if (command_line.HasSwitch(switches::kInstallerTestBackup)) {
+    BackUpProfile(command_line.HasSwitch(
+        installer::switches::kChromeFrame));
+  }
+
   if (base::win::GetVersion() < base::win::VERSION_VISTA ||
       command_line.HasSwitch(switches::kInstallerTestForce)) {
     return test_suite.Run();
-  } else {
-    printf("These tests don't run on this platform.\n");
-    return 0;
   }
+  printf("These tests don't run on this platform.\n");
+  return 0;
 }
