@@ -9,12 +9,16 @@
 #define CHROME_TEST_PYAUTOLIB_PYAUTOLIB_H_
 #pragma once
 
+#include "base/compiler_specific.h"
 #include "base/message_loop.h"
-#include "base/mac/scoped_nsautorelease_pool.h"
 #include "base/test/test_timeouts.h"
 #include "chrome/test/ui/ui_test.h"
 #include "chrome/test/ui/ui_test_suite.h"
 
+#if defined(OS_MACOSX)
+#include "base/mac/scoped_nsautorelease_pool.h"
+#endif
+
 // The C++ style guide forbids using default arguments but I'm taking the
 // liberty of allowing it in this file. The sole purpose of this (and the
 // .cc) is to support the python interface, and default args are allowed in
@@ -26,12 +30,16 @@
 class PyUITestSuiteBase : public UITestSuite {
  public:
   PyUITestSuiteBase(int argc, char** argv);
-  ~PyUITestSuiteBase();
+  virtual ~PyUITestSuiteBase();
 
-  void Initialize(const FilePath& browser_dir);
+  void InitializeWithPath(const FilePath& browser_dir);
+
+  void SetCrSourceRoot(const FilePath& path);
 
  private:
+#if defined(OS_MACOSX)
   base::mac::ScopedNSAutoreleasePool pool_;
+#endif
 };
 
 // The primary class that interfaces with Automation Proxy.
@@ -42,15 +50,22 @@ class PyUITestBase : public UITestBase {
 
   // Constructor. Lookup pyauto.py for doc on these args.
   PyUITestBase(bool clear_profile, std::wstring homepage);
-  ~PyUITestBase();
+  virtual ~PyUITestBase();
 
   // Initialize the setup. Should be called before launching the browser.
   // |browser_dir| is the path to dir containing chromium binaries.
   void Initialize(const FilePath& browser_dir);
 
+  void UseNamedChannelID(const std::string& named_channel_id) {
+    named_channel_id_ = named_channel_id;
+    launcher_.reset(CreateProxyLauncher());
+  }
+
+  virtual ProxyLauncher* CreateProxyLauncher() OVERRIDE;
+
   // SetUp,TearDown is redeclared as public to make it accessible from swig.
-  virtual void SetUp();
-  virtual void TearDown();
+  virtual void SetUp() OVERRIDE;
+  virtual void TearDown() OVERRIDE;
 
   // Navigate to the given URL in the active tab. Blocks until page loaded.
   void NavigateToURL(const char* url_string);
@@ -75,7 +90,7 @@ class PyUITestBase : public UITestBase {
   bool AppendTab(const GURL& tab_url, int window_index = 0);
 
   // Activate the tab at the given zero-based index in the given or first
-  // browser window.
+  // browser window.  Also brings the window to front.
   bool ActivateTab(int tab_index, int window_index = 0);
 
   // Apply the accelerator with given id (IDC_BACK, IDC_NEWTAB ...) to the
@@ -87,6 +102,9 @@ class PyUITestBase : public UITestBase {
   // Like ApplyAccelerator except that it waits for the command to execute.
   bool RunCommand(int browser_command, int window_index = 0);
 
+  // Returns true if the given command id is enabled on the given window.
+  bool IsMenuCommandEnabled(int browser_command, int window_index = 0);
+
   // Shows or hides the download shelf.
   void SetDownloadShelfVisible(bool is_visible, int window_index = 0);
 
@@ -113,13 +131,12 @@ class PyUITestBase : public UITestBase {
   // Fetch the number of browser windows. Includes popups.
   int GetBrowserWindowCount();
 
-  // Installs the extension crx. Returns true only if extension was installed
-  // and loaded successfully. Overinstalls will fail.
-  bool InstallExtension(const FilePath& crx_file, bool with_ui);
-
   // Returns bookmark bar visibility state.
   bool GetBookmarkBarVisibility();
 
+  // Returns true if the bookmark bar is visible in the detached state.
+  bool IsBookmarkBarDetached();
+
   // Returns bookmark bar animation state.  Warning: timing issues may
   // change this return value unexpectedly.
   bool IsBookmarkBarAnimating();
@@ -154,29 +171,14 @@ class PyUITestBase : public UITestBase {
   // Get a handle to browser window at the given index, or NULL on failure.
   scoped_refptr<BrowserProxy> GetBrowserWindow(int window_index);
 
-  // Meta-method.  Experimental pattern of passing args and response as
+  // Meta-methods.  Generic pattern of passing args and response as
   // JSON dict to avoid future use of the SWIG interface and
   // automation proxy additions.  Returns response as JSON dict.
-  std::string _SendJSONRequest(int window_index, std::string& request);
-
-  // Execute javascript in a given tab, and return the response. This is
-  // a low-level method intended for use mostly by GetDOMValue(). Note that
-  // any complicated manipulation of the page should be done by something
-  // like WebDriver, not PyAuto. Also note that in order for the script to
-  // return a value to the calling code, it invoke
-  // window.domAutomationController.send(), passing in the intended return
-  // value.
-  std::wstring ExecuteJavascript(const std::wstring& script,
-                                 int window_index = 0,
-                                 int tab_index = 0,
-                                 const std::wstring& frame_xpath = L"");
-
-  // Evaluate a Javascript expression and return the result as a string. This
-  // method is intended largely to read values out of the frame DOM.
-  std::wstring GetDOMValue(const std::wstring& expr,
-                           int window_index = 0,
-                           int tab_index = 0,
-                           const std::wstring& frame_xpath = L"");
+  // Use -ve window_index for automation calls not targetted at a browser
+  // window.  Example: Login call for chromeos.
+  std::string _SendJSONRequest(int window_index,
+                               const std::string& request,
+                               int timeout);
 
   // Resets to the default theme. Returns true on success.
   bool ResetToDefaultTheme();
@@ -192,11 +194,14 @@ class PyUITestBase : public UITestBase {
     return TestTimeouts::action_max_timeout_ms();
   }
 
-  int command_execution_timeout_ms() const {
-    return TestTimeouts::command_execution_timeout_ms();
+  int large_test_timeout_ms() const {
+    return TestTimeouts::large_test_timeout_ms();
   }
 
  private:
+  // Gets the current state of the bookmark bar. Returns false if it failed.
+  bool GetBookmarkBarState(bool* visible, bool* detached);
+
   // Enables PostTask to main thread.
   // Should be shared across multiple instances of PyUITestBase so that this
   // class is re-entrant and multiple instances can be created.
@@ -204,6 +209,9 @@ class PyUITestBase : public UITestBase {
   // TestCase at load time itself.
   static MessageLoop* GetSharedMessageLoop(MessageLoop::Type msg_loop_type);
   static MessageLoop* message_loop_;
+
+  // Path to named channel id.
+  std::string named_channel_id_;
 };
 
 #endif  // CHROME_TEST_PYAUTOLIB_PYAUTOLIB_H_
