@@ -1,18 +1,16 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "build/build_config.h"
 
 #if defined(OS_WIN)
-// windows headers
 #include <comutil.h>
 #include <shellapi.h>
 #include <shlobj.h>
 #include <windows.h>
 #endif
 
-// runtime headers
 #include <memory.h>
 #include <stdlib.h>
 #include <string.h>
@@ -22,19 +20,32 @@
 #include "base/file_path.h"
 #include "base/string_number_conversions.h"
 #include "base/test/test_timeouts.h"
-#include "chrome/browser/net/url_request_mock_http_job.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/url_constants.h"
+#include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/automation/window_proxy.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "chrome/test/ui/npapi_test_helper.h"
-#include "chrome/test/ui_test_utils.h"
+#include "content/browser/net/url_request_mock_http_job.h"
+#include "content/public/common/content_switches.h"
 
 using npapi_test::kTestCompleteCookie;
 using npapi_test::kTestCompleteSuccess;
 
-static const FilePath::CharType* kTestDir = FILE_PATH_LITERAL("npapi");
+namespace {
+
+const FilePath::CharType* kTestDir = FILE_PATH_LITERAL("npapi");
+
+class NPAPIAutomationEnabledTest : public NPAPIVisiblePluginTester {
+ public:
+  NPAPIAutomationEnabledTest() {
+    dom_automation_enabled_ = true;
+  }
+};
+
+}
 
 // Test passing arguments to a plugin.
 TEST_F(NPAPITesterBase, Arguments) {
@@ -92,7 +103,8 @@ TEST_F(NPAPITesterBase, GetJavaScriptURL2) {
                 kTestCompleteSuccess, TestTimeouts::action_max_timeout_ms());
 }
 
-TEST_F(NPAPITesterBase, GetURLRedirectNotification) {
+// Test is flaky on linux/cros/win builders.  http://crbug.com/71904
+TEST_F(NPAPITesterBase, FLAKY_GetURLRedirectNotification) {
   const FilePath test_case(FILE_PATH_LITERAL("geturl_redirect_notify.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
   ASSERT_NO_FATAL_FAILURE(NavigateToURL(url));
@@ -100,6 +112,16 @@ TEST_F(NPAPITesterBase, GetURLRedirectNotification) {
                 kTestCompleteSuccess, TestTimeouts::action_max_timeout_ms());
 }
 
+// Tests that identity is preserved for NPObjects passed from a plugin
+// into JavaScript.
+TEST_F(NPAPITesterBase, NPObjectIdentity) {
+  const FilePath test_case(FILE_PATH_LITERAL("npobject_identity.html"));
+  GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
+  ASSERT_NO_FATAL_FAILURE(NavigateToURL(url));
+  WaitForFinish("npobject_identity", "1", url, kTestCompleteCookie,
+                kTestCompleteSuccess, TestTimeouts::action_max_timeout_ms());
+}
+
 // Tests that if an NPObject is proxies back to its original process, the
 // original pointer is returned and not a proxy.  If this fails the plugin
 // will crash.
@@ -117,9 +139,6 @@ TEST_F(NPAPITesterBase, NPObjectProxy) {
 // http://crbug.com/44960
 TEST_F(NPAPIVisiblePluginTester,
        FLAKY_SelfDeletePluginInvokeInSynchronousPaint) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   show_window_ = true;
   const FilePath test_case(
       FILE_PATH_LITERAL("execute_script_delete_in_paint.html"));
@@ -132,9 +151,6 @@ TEST_F(NPAPIVisiblePluginTester,
 #endif
 
 TEST_F(NPAPIVisiblePluginTester, SelfDeletePluginInNewStream) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   show_window_ = true;
   const FilePath test_case(FILE_PATH_LITERAL("self_delete_plugin_stream.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
@@ -144,6 +160,26 @@ TEST_F(NPAPIVisiblePluginTester, SelfDeletePluginInNewStream) {
                 TestTimeouts::action_max_timeout_ms());
 }
 
+// http://crbug.com/95558
+// This test fails frequently on Mac and windows, so it is disabled for now.
+#if defined(OS_MACOSX)
+#define MAYBE_DeletePluginInDeallocate DISABLED_DeletePluginInDeallocate
+#elif defined(OS_WIN)
+#define MAYBE_DeletePluginInDeallocate FLAKY_DeletePluginInDeallocate
+#else
+#define MAYBE_DeletePluginInDeallocate DeletePluginInDeallocate
+#endif
+TEST_F(NPAPIVisiblePluginTester, MAYBE_DeletePluginInDeallocate) {
+  show_window_ = true;
+  const FilePath test_case(
+      FILE_PATH_LITERAL("plugin_delete_in_deallocate.html"));
+  GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
+  ASSERT_NO_FATAL_FAILURE(NavigateToURL(url));
+  WaitForFinish("delete_plugin_in_deallocate_test", "signaller", url,
+                kTestCompleteCookie, kTestCompleteSuccess,
+                TestTimeouts::action_max_timeout_ms());
+}
+
 #if defined(OS_WIN)
 // Tests if a plugin has a non zero window rect.
 TEST_F(NPAPIVisiblePluginTester, VerifyPluginWindowRect) {
@@ -174,29 +210,24 @@ TEST_F(NPAPIVisiblePluginTester, AlertInWindowMessage) {
   ASSERT_NO_FATAL_FAILURE(NavigateToURL(url));
 
   bool modal_dialog_showing = false;
-  ui::MessageBoxFlags::DialogButton available_buttons;
+  ui::DialogButton available_buttons;
   ASSERT_TRUE(automation()->WaitForAppModalDialog());
   ASSERT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
       &available_buttons));
   ASSERT_TRUE(modal_dialog_showing);
-  ASSERT_NE((ui::MessageBoxFlags::DIALOGBUTTON_OK & available_buttons), 0);
-  ASSERT_TRUE(automation()->ClickAppModalDialogButton(
-      ui::MessageBoxFlags::DIALOGBUTTON_OK));
+  ASSERT_NE((ui::DIALOG_BUTTON_OK & available_buttons), 0);
+  ASSERT_TRUE(automation()->ClickAppModalDialogButton(ui::DIALOG_BUTTON_OK));
 
   modal_dialog_showing = false;
   ASSERT_TRUE(automation()->WaitForAppModalDialog());
   ASSERT_TRUE(automation()->GetShowingAppModalDialog(&modal_dialog_showing,
       &available_buttons));
   ASSERT_TRUE(modal_dialog_showing);
-  ASSERT_NE((ui::MessageBoxFlags::DIALOGBUTTON_OK & available_buttons), 0);
-  ASSERT_TRUE(automation()->ClickAppModalDialogButton(
-      ui::MessageBoxFlags::DIALOGBUTTON_OK));
+  ASSERT_NE((ui::DIALOG_BUTTON_OK & available_buttons), 0);
+  ASSERT_TRUE(automation()->ClickAppModalDialogButton(ui::DIALOG_BUTTON_OK));
 }
 
 TEST_F(NPAPIVisiblePluginTester, VerifyNPObjectLifetimeTest) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   show_window_ = true;
   const FilePath test_case(FILE_PATH_LITERAL("npobject_lifetime_test.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
@@ -216,9 +247,6 @@ TEST_F(NPAPIVisiblePluginTester, NewFails) {
 }
 
 TEST_F(NPAPIVisiblePluginTester, SelfDeletePluginInNPNEvaluate) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   const FilePath test_case(
       FILE_PATH_LITERAL("execute_script_delete_in_npn_evaluate.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
@@ -229,9 +257,6 @@ TEST_F(NPAPIVisiblePluginTester, SelfDeletePluginInNPNEvaluate) {
 }
 
 TEST_F(NPAPIVisiblePluginTester, SelfDeleteCreatePluginInNPNEvaluate) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   const FilePath test_case(
       FILE_PATH_LITERAL("npn_plugin_delete_create_in_evaluate.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
@@ -243,13 +268,16 @@ TEST_F(NPAPIVisiblePluginTester, SelfDeleteCreatePluginInNPNEvaluate) {
 
 #endif
 
-// FLAKY. See bug http://crbug.com/17645. This bug report indicates that this
-// test is crashy. I could not repro the crash on my local setup. Leaving this
-// marked as FLAKY for now while we watch this on the builders.
-TEST_F(NPAPIVisiblePluginTester, FLAKY_OpenPopupWindowWithPlugin) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
+// http://crbug.com/17645
+// As of 6 July 2011, this test always fails on OS X and is flaky on
+// Windows (perhaps due to timing out).
+#if defined(OS_MACOSX)
+#define MAYBE_OpenPopupWindowWithPlugin DISABLED_OpenPopupWindowWithPlugin
+#else
+#define MAYBE_OpenPopupWindowWithPlugin FLAKY_OpenPopupWindowWithPlugin
+#endif
 
+TEST_F(NPAPIVisiblePluginTester, MAYBE_OpenPopupWindowWithPlugin) {
   const FilePath test_case(
       FILE_PATH_LITERAL("get_javascript_open_popup_with_plugin.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
@@ -261,9 +289,6 @@ TEST_F(NPAPIVisiblePluginTester, FLAKY_OpenPopupWindowWithPlugin) {
 
 // Test checking the privacy mode is off.
 TEST_F(NPAPITesterBase, PrivateDisabled) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   const FilePath test_case(FILE_PATH_LITERAL("private.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
   ASSERT_NO_FATAL_FAILURE(NavigateToURL(url));
@@ -288,12 +313,14 @@ TEST_F(NPAPITesterBase, PluginThreadAsyncCall) {
 }
 
 // Test checking the privacy mode is on.
+#if defined(OS_LINUX)
+// http://crbug.com/104380
+#define PrivateEnabled FLAKY_PrivateEnabled
+#endif
 TEST_F(NPAPIIncognitoTester, PrivateEnabled) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
-  const FilePath test_case(FILE_PATH_LITERAL("private.html?private"));
-  GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
+  const FilePath test_case(FILE_PATH_LITERAL("private.html"));
+  GURL url = ui_test_utils::GetFileUrlWithQuery(
+      ui_test_utils::GetTestFilePath(FilePath(kTestDir), test_case), "private");
   ASSERT_NO_FATAL_FAILURE(NavigateToURL(url));
   WaitForFinish("private", "1", url, kTestCompleteCookie,
                 kTestCompleteSuccess, TestTimeouts::action_max_timeout_ms());
@@ -303,9 +330,6 @@ TEST_F(NPAPIIncognitoTester, PrivateEnabled) {
 // Test a browser hang due to special case of multiple
 // plugin instances indulged in sync calls across renderer.
 TEST_F(NPAPIVisiblePluginTester, MultipleInstancesSyncCalls) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   const FilePath test_case(
       FILE_PATH_LITERAL("multiple_instances_sync_calls.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
@@ -316,9 +340,6 @@ TEST_F(NPAPIVisiblePluginTester, MultipleInstancesSyncCalls) {
 #endif
 
 TEST_F(NPAPIVisiblePluginTester, GetURLRequestFailWrite) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   GURL url(URLRequestMockHTTPJob::GetMockUrl(
                FilePath(FILE_PATH_LITERAL(
                             "npapi/plugin_url_request_fail_write.html"))));
@@ -331,9 +352,6 @@ TEST_F(NPAPIVisiblePluginTester, GetURLRequestFailWrite) {
 
 #if defined(OS_WIN)
 TEST_F(NPAPITesterBase, EnsureScriptingWorksInDestroy) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   const FilePath test_case(
       FILE_PATH_LITERAL("ensure_scripting_works_in_destroy.html"));
   GURL url = ui_test_utils::GetTestUrl(FilePath(kTestDir), test_case);
@@ -346,9 +364,6 @@ TEST_F(NPAPITesterBase, EnsureScriptingWorksInDestroy) {
 // This test uses a Windows Event to signal to the plugin that it should crash
 // on NP_Initialize.
 TEST_F(NPAPITesterBase, NoHangIfInitCrashes) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   // Only Windows implements the crash service for now.
 #if defined(OS_WIN)
   expected_crashes_ = 1;
@@ -367,10 +382,11 @@ TEST_F(NPAPITesterBase, NoHangIfInitCrashes) {
 
 #endif
 
+#if defined(OS_MACOSX)
+// http://crbug.com/111508
+#define PluginReferrerTest FLAKY_PluginReferrerTest
+#endif
 TEST_F(NPAPIVisiblePluginTester, PluginReferrerTest) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   GURL url(URLRequestMockHTTPJob::GetMockUrl(
                FilePath(FILE_PATH_LITERAL(
                             "npapi/plugin_url_request_referrer_test.html"))));
@@ -383,9 +399,6 @@ TEST_F(NPAPIVisiblePluginTester, PluginReferrerTest) {
 
 #if defined(OS_MACOSX)
 TEST_F(NPAPIVisiblePluginTester, PluginConvertPointTest) {
-  if (ProxyLauncher::in_process_renderer())
-    return;
-
   scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
   ASSERT_TRUE(browser.get());
   scoped_refptr<WindowProxy> window(browser->GetWindow());
@@ -422,3 +435,27 @@ TEST_F(NPAPIVisiblePluginTester, ClickToPlay) {
   WaitForFinish("setup", "1", url, kTestCompleteCookie,
                 kTestCompleteSuccess, TestTimeouts::action_max_timeout_ms());
 }
+
+TEST_F(NPAPIAutomationEnabledTest, LoadAllBlockedPlugins) {
+  scoped_refptr<BrowserProxy> browser(automation()->GetBrowserWindow(0));
+  ASSERT_TRUE(browser.get());
+  ASSERT_TRUE(browser->SetDefaultContentSetting(CONTENT_SETTINGS_TYPE_PLUGINS,
+                                                CONTENT_SETTING_BLOCK));
+
+  GURL url(URLRequestMockHTTPJob::GetMockUrl(
+      FilePath(FILE_PATH_LITERAL("npapi/load_all_blocked_plugins.html"))));
+  ASSERT_NO_FATAL_FAILURE(NavigateToURL(url));
+
+  scoped_refptr<TabProxy> tab(browser->GetTab(0));
+  ASSERT_TRUE(tab.get());
+
+  ASSERT_TRUE(tab->LoadBlockedPlugins());
+
+  WaitForFinish("setup", "1", url, kTestCompleteCookie,
+                kTestCompleteSuccess, TestTimeouts::action_max_timeout_ms());
+
+  ASSERT_TRUE(tab->ExecuteJavaScript("window.inject()"));
+
+  WaitForFinish("setup", "2", url, kTestCompleteCookie,
+                kTestCompleteSuccess, TestTimeouts::action_max_timeout_ms());
+}
