@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,21 +12,21 @@
 #include <set>
 #include <vector>
 
-#include "app/app_switches.h"
-#include "app/gfx/gl/gl_implementation.h"
-#include "app/sql/connection.h"
 #include "base/base_switches.h"
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/environment.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
+#include "base/json/json_value_serializer.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
 #include "base/process_util.h"
-#include "base/scoped_ptr.h"
 #include "base/scoped_temp_dir.h"
 #include "base/string_number_conversions.h"
 #include "base/string_split.h"
 #include "base/test/test_file_util.h"
+#include "base/test/test_timeouts.h"
 #include "base/threading/platform_thread.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
@@ -36,9 +36,8 @@
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/debug_flags.h"
 #include "chrome/common/logging_chrome.h"
-#include "chrome/common/json_value_serializer.h"
+#include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/browser_proxy.h"
@@ -46,16 +45,20 @@
 #include "chrome/test/automation/proxy_launcher.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/automation/window_proxy.h"
-#include "chrome/test/chrome_process_util.h"
-#include "chrome/test/test_launcher_utils.h"
-#include "chrome/test/test_switches.h"
+#include "chrome/test/base/chrome_process_util.h"
+#include "chrome/test/base/test_switches.h"
+#include "content/common/debug_flags.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/net_util.h"
+#include "ui/gfx/gl/gl_implementation.h"
 
 #if defined(OS_WIN)
 #include "base/win/windows_version.h"
 #endif
 
+#if defined(USE_AURA)
+#include "ui/gfx/compositor/compositor_switches.h"
+#endif
 
 using base::Time;
 using base::TimeDelta;
@@ -89,8 +92,7 @@ UITestBase::UITestBase()
       clear_profile_(true),
       include_testing_id_(true),
       enable_file_cookies_(true),
-      profile_type_(ProxyLauncher::DEFAULT_THEME),
-      shutdown_type_(ProxyLauncher::WINDOW_CLOSE) {
+      profile_type_(UITestBase::DEFAULT_THEME) {
   PathService::Get(chrome::DIR_APP, &browser_directory_);
   PathService::Get(chrome::DIR_TEST_DATA, &test_data_directory_);
 }
@@ -105,8 +107,7 @@ UITestBase::UITestBase(MessageLoop::Type msg_loop_type)
       clear_profile_(true),
       include_testing_id_(true),
       enable_file_cookies_(true),
-      profile_type_(ProxyLauncher::DEFAULT_THEME),
-      shutdown_type_(ProxyLauncher::WINDOW_CLOSE) {
+      profile_type_(UITestBase::DEFAULT_THEME) {
   PathService::Get(chrome::DIR_APP, &browser_directory_);
   PathService::Get(chrome::DIR_TEST_DATA, &test_data_directory_);
 }
@@ -114,24 +115,26 @@ UITestBase::UITestBase(MessageLoop::Type msg_loop_type)
 UITestBase::~UITestBase() {}
 
 void UITestBase::SetUp() {
-  // Some tests (e.g. SessionRestoreUITest) call SetUp() multiple times,
-  // but the ProxyLauncher should be initialized only once per instance.
+  // Tests that do a session restore (e.g. SessionRestoreUITest, StartupTest)
+  // call SetUp() multiple times because they restart the browser mid-test.
+  // We don't want to reset the ProxyLauncher's state in those cases.
   if (!launcher_.get())
     launcher_.reset(CreateProxyLauncher());
-  launcher_->AssertAppNotRunning(L"Please close any other instances "
-                                 L"of the app before testing.");
+  launcher_->AssertAppNotRunning("Please close any other instances "
+                                 "of the app before testing.");
 
   JavaScriptExecutionController::set_timeout(
       TestTimeouts::action_max_timeout_ms());
   test_start_time_ = Time::NowFromSystemTime();
 
   SetLaunchSwitches();
-  launcher_->InitializeConnection(DefaultLaunchState(),
-                                  wait_for_initial_loads_);
+  ASSERT_TRUE(launcher_->InitializeConnection(DefaultLaunchState(),
+                                              wait_for_initial_loads_));
 }
 
 void UITestBase::TearDown() {
-  CloseBrowserAndServer();
+  if (launcher_.get())
+    launcher_->TerminateConnection();
 
   // Make sure that we didn't encounter any assertion failures
   logging::AssertionList assertions;
@@ -160,17 +163,40 @@ void UITestBase::TearDown() {
   EXPECT_EQ(expected_crashes_, actual_crashes) << error_msg;
 }
 
-// TODO(phajdan.jr): get rid of set_command_execution_timeout_ms.
-void UITestBase::set_command_execution_timeout_ms(int timeout) {
-  automation()->set_command_execution_timeout_ms(timeout);
-  VLOG(1) << "Automation command execution timeout set to " << timeout << " ms";
+int UITestBase::action_timeout_ms() {
+  return automation()->action_timeout_ms();
+}
+
+void UITestBase::set_action_timeout_ms(int timeout) {
+  automation()->set_action_timeout_ms(timeout);
+  VLOG(1) << "Automation action timeout set to " << timeout << " ms";
 }
 
 ProxyLauncher* UITestBase::CreateProxyLauncher() {
   return new AnonymousProxyLauncher(false);
 }
 
+ProxyLauncher::LaunchState UITestBase::DefaultLaunchState() {
+  FilePath browser_executable = browser_directory_.Append(GetExecutablePath());
+  CommandLine command(browser_executable);
+  command.AppendArguments(launch_arguments_, false);
+  base::Closure setup_profile_callback = base::Bind(&UITestBase::SetUpProfile,
+                                                    base::Unretained(this));
+  ProxyLauncher::LaunchState state =
+      { clear_profile_, template_user_data_, setup_profile_callback,
+        command, include_testing_id_, show_window_ };
+  return state;
+}
+
+bool UITestBase::ShouldFilterInet() {
+  return true;
+}
+
 void UITestBase::SetLaunchSwitches() {
+  // All flags added here should also be added in ExtraChromeFlags() in
+  // chrome/test/pyautolib/pyauto.py as well to take effect for all tests
+  // on chromeos.
+
   // We need cookies on file:// for things like the page cycler.
   if (enable_file_cookies_)
     launch_arguments_.AppendSwitch(switches::kEnableFileCookies);
@@ -180,6 +206,15 @@ void UITestBase::SetLaunchSwitches() {
     launch_arguments_.AppendSwitchASCII(switches::kHomePage, homepage_);
   if (!test_name_.empty())
     launch_arguments_.AppendSwitchASCII(switches::kTestName, test_name_);
+#if defined(USE_AURA)
+  if (!CommandLine::ForCurrentProcess()->HasSwitch(
+      switches::kDisableTestCompositor)) {
+    launch_arguments_.AppendSwitch(switches::kTestCompositor);
+  }
+#endif
+}
+
+void UITestBase::SetUpProfile() {
 }
 
 void UITestBase::LaunchBrowser() {
@@ -187,45 +222,37 @@ void UITestBase::LaunchBrowser() {
 }
 
 void UITestBase::LaunchBrowserAndServer() {
-  launcher_->LaunchBrowserAndServer(DefaultLaunchState(),
-                                    wait_for_initial_loads_);
+  ASSERT_TRUE(launcher_->LaunchBrowserAndServer(DefaultLaunchState(),
+                                                wait_for_initial_loads_));
 }
 
 void UITestBase::ConnectToRunningBrowser() {
-  launcher_->ConnectToRunningBrowser(wait_for_initial_loads_);
+  ASSERT_TRUE(launcher_->ConnectToRunningBrowser(wait_for_initial_loads_));
 }
 
 void UITestBase::CloseBrowserAndServer() {
   if (launcher_.get())
-    launcher_->CloseBrowserAndServer(shutdown_type_);
+    launcher_->CloseBrowserAndServer();
 }
 
 void UITestBase::LaunchBrowser(const CommandLine& arguments,
                                bool clear_profile) {
   ProxyLauncher::LaunchState state = DefaultLaunchState();
   state.clear_profile = clear_profile;
-  launcher_->LaunchBrowser(state);
+  ASSERT_TRUE(launcher_->LaunchBrowser(state));
 }
 
 #if !defined(OS_MACOSX)
 bool UITestBase::LaunchAnotherBrowserBlockUntilClosed(
     const CommandLine& cmdline) {
   ProxyLauncher::LaunchState state = DefaultLaunchState();
-  state.arguments = cmdline;
+  state.command.AppendArguments(cmdline, false);
   return launcher_->LaunchAnotherBrowserBlockUntilClosed(state);
 }
 #endif
 
 void UITestBase::QuitBrowser() {
-  launcher_->QuitBrowser(shutdown_type_);
-}
-
-void UITestBase::TerminateBrowser() {
-  launcher_->TerminateBrowser();
-}
-
-void UITestBase::CleanupAppProcesses() {
-  TerminateAllChromeProcesses(browser_process_id());
+  launcher_->QuitBrowser();
 }
 
 scoped_refptr<TabProxy> UITestBase::GetActiveTab(int window_index) {
@@ -263,12 +290,6 @@ scoped_refptr<TabProxy> UITestBase::GetActiveTab() {
   return tab_proxy;
 }
 
-void UITestBase::NavigateToURLAsync(const GURL& url) {
-  scoped_refptr<TabProxy> tab_proxy(GetActiveTab());
-  ASSERT_TRUE(tab_proxy.get());
-  ASSERT_TRUE(tab_proxy->NavigateToURLAsync(url));
-}
-
 void UITestBase::NavigateToURL(const GURL& url) {
   NavigateToURL(url, 0, GetActiveTabIndex(0));
 }
@@ -304,27 +325,21 @@ void UITestBase::NavigateToURLBlockUntilNavigationsComplete(
                 url, number_of_navigations)) << url.spec();
 }
 
-bool UITestBase::WaitForBrowserProcessToQuit(int timeout) {
-  return launcher_->WaitForBrowserProcessToQuit(timeout);
-}
-
 bool UITestBase::WaitForBookmarkBarVisibilityChange(BrowserProxy* browser,
                                                     bool wait_for_open) {
   const int kCycles = 10;
+  const TimeDelta kDelay = TestTimeouts::action_timeout() / kCycles;
   for (int i = 0; i < kCycles; i++) {
     bool visible = false;
     bool animating = true;
-    if (!browser->GetBookmarkBarVisibility(&visible, &animating))
+    bool detached;
+    if (!browser->GetBookmarkBarVisibility(&visible, &animating, &detached))
       return false;  // Some error.
     if (visible == wait_for_open && !animating)
       return true;  // Bookmark bar visibility change complete.
 
     // Give it a chance to catch up.
-    bool browser_survived = CrashAwareSleep(
-        TestTimeouts::action_timeout_ms() / kCycles);
-    EXPECT_TRUE(browser_survived);
-    if (!browser_survived)
-      return false;
+    base::PlatformThread::Sleep(kDelay);
   }
 
   ADD_FAILURE() << "Timeout reached in WaitForBookmarkBarVisibilityChange";
@@ -368,14 +383,6 @@ int UITestBase::GetActiveTabIndex(int window_index) {
   return active_tab_index;
 }
 
-bool UITestBase::IsBrowserRunning() {
-  return launcher_->IsBrowserRunning();
-}
-
-bool UITestBase::CrashAwareSleep(int timeout_ms) {
-  return launcher_->CrashAwareSleep(timeout_ms);
-}
-
 int UITestBase::GetTabCount() {
   return GetTabCount(0);
 }
@@ -395,15 +402,13 @@ int UITestBase::GetTabCount(int window_index) {
 
 void UITestBase::WaitUntilTabCount(int tab_count) {
   const int kMaxIntervals = 10;
-  const int kIntervalMs = TestTimeouts::action_timeout_ms() / kMaxIntervals;
+  const TimeDelta kDelay = TestTimeouts::action_timeout() / kMaxIntervals;
 
   for (int i = 0; i < kMaxIntervals; ++i) {
-    bool browser_survived = CrashAwareSleep(kIntervalMs);
-    EXPECT_TRUE(browser_survived);
-    if (!browser_survived)
-      return;
     if (GetTabCount() == tab_count)
       return;
+
+    base::PlatformThread::Sleep(kDelay);
   }
 
   ADD_FAILURE() << "Timeout reached in WaitUntilTabCount";
@@ -420,6 +425,12 @@ FilePath UITestBase::GetDownloadDirectory() {
   return download_directory;
 }
 
+const FilePath::CharType* UITestBase::GetExecutablePath() {
+  if (launch_arguments_.HasSwitch(switches::kEnableChromiumBranding))
+    return chrome::kBrowserProcessExecutablePathChromium;
+  return chrome::kBrowserProcessExecutablePath;
+}
+
 void UITestBase::CloseBrowserAsync(BrowserProxy* browser) const {
   ASSERT_TRUE(automation()->Send(
       new AutomationMsg_CloseBrowserRequestAsync(browser->handle())));
@@ -440,9 +451,10 @@ bool UITestBase::CloseBrowser(BrowserProxy* browser,
     return false;
 
   if (*application_closed) {
-    // Let's wait until the process dies (if it is not gone already).
-    bool success = base::WaitForSingleProcess(process(), base::kNoTimeout);
-    EXPECT_TRUE(success);
+    int exit_code = -1;
+    EXPECT_TRUE(launcher_->WaitForBrowserProcessToQuit(
+        TestTimeouts::action_max_timeout_ms(), &exit_code));
+    EXPECT_EQ(0, exit_code);  // Expect a clean shutown.
   }
 
   return result;
@@ -450,25 +462,25 @@ bool UITestBase::CloseBrowser(BrowserProxy* browser,
 
 // static
 FilePath UITestBase::ComputeTypicalUserDataSource(
-    ProxyLauncher::ProfileType profile_type) {
+    UITestBase::ProfileType profile_type) {
   FilePath source_history_file;
   EXPECT_TRUE(PathService::Get(chrome::DIR_TEST_DATA,
                                &source_history_file));
   source_history_file = source_history_file.AppendASCII("profiles");
   switch (profile_type) {
-    case ProxyLauncher::DEFAULT_THEME:
+    case UITestBase::DEFAULT_THEME:
       source_history_file = source_history_file.AppendASCII("typical_history");
       break;
-    case ProxyLauncher::COMPLEX_THEME:
+    case UITestBase::COMPLEX_THEME:
       source_history_file = source_history_file.AppendASCII("complex_theme");
       break;
-    case ProxyLauncher::NATIVE_THEME:
+    case UITestBase::NATIVE_THEME:
       source_history_file = source_history_file.AppendASCII("gtk_theme");
       break;
-    case ProxyLauncher::CUSTOM_FRAME:
+    case UITestBase::CUSTOM_FRAME:
       source_history_file = source_history_file.AppendASCII("custom_frame");
       break;
-    case ProxyLauncher::CUSTOM_FRAME_NATIVE_THEME:
+    case UITestBase::CUSTOM_FRAME_NATIVE_THEME:
       source_history_file =
           source_history_file.AppendASCII("custom_frame_gtk_theme");
       break;
@@ -496,6 +508,15 @@ void UITestBase::SetBrowserDirectory(const FilePath& dir) {
   browser_directory_ = dir;
 }
 
+void UITestBase::AppendBrowserLaunchSwitch(const char* name) {
+  launch_arguments_.AppendSwitch(name);
+}
+
+void UITestBase::AppendBrowserLaunchSwitch(const char* name,
+                                           const char* value) {
+  launch_arguments_.AppendSwitchASCII(name, value);
+}
+
 // UITest methods
 
 void UITest::SetUp() {
@@ -508,16 +529,6 @@ void UITest::SetUp() {
                   test_info->name());
   }
 
-  // Force tests to use OSMesa if they launch the GPU process. This is in
-  // UITest::SetUp so that it does not affect pyautolib, which runs tests that
-  // do not work with OSMesa.
-  launch_arguments_.AppendSwitchASCII(switches::kUseGL,
-                                      gfx::kGLImplementationOSMesaName);
-
-  // Mac does not support accelerated compositing with OSMesa. Disable on all
-  // platforms so it is consistent. http://crbug.com/58343
-  launch_arguments_.AppendSwitch(switches::kDisableAcceleratedCompositing);
-
   UITestBase::SetUp();
   PlatformTest::SetUp();
 }
@@ -534,79 +545,12 @@ ProxyLauncher* UITest::CreateProxyLauncher() {
   return new AnonymousProxyLauncher(true);
 }
 
-static CommandLine* CreatePythonCommandLine() {
-  return new CommandLine(FilePath(FILE_PATH_LITERAL("python")));
-}
-
-static CommandLine* CreateHttpServerCommandLine() {
-  FilePath src_path;
-  // Get to 'src' dir.
-  PathService::Get(base::DIR_SOURCE_ROOT, &src_path);
-
-  FilePath script_path(src_path);
-  script_path = script_path.AppendASCII("third_party");
-  script_path = script_path.AppendASCII("WebKit");
-  script_path = script_path.AppendASCII("Tools");
-  script_path = script_path.AppendASCII("Scripts");
-  script_path = script_path.AppendASCII("new-run-webkit-httpd");
-
-  CommandLine* cmd_line = CreatePythonCommandLine();
-  cmd_line->AppendArgPath(script_path);
-  return cmd_line;
-}
-
-void UITest::StartHttpServer(const FilePath& root_directory) {
-  StartHttpServerWithPort(root_directory, 0);
-}
-
-void UITest::StartHttpServerWithPort(const FilePath& root_directory,
-                                     int port) {
-  scoped_ptr<CommandLine> cmd_line(CreateHttpServerCommandLine());
-  ASSERT_TRUE(cmd_line.get());
-  cmd_line->AppendSwitchASCII("server", "start");
-  cmd_line->AppendSwitch("register_cygwin");
-  cmd_line->AppendSwitchPath("root", root_directory);
-
-  // For Windows 7, if we start the lighttpd server on the foreground mode,
-  // it will mess up with the command window and cause conhost.exe to crash. To
-  // work around this, we start the http server on the background mode.
-#if defined(OS_WIN)
-  if (base::win::GetVersion() >= base::win::VERSION_WIN7)
-    cmd_line->AppendSwitch("run_background");
-#endif
-
-  if (port)
-    cmd_line->AppendSwitchASCII("port", base::IntToString(port));
-
-#if defined(OS_WIN)
-  // TODO(phajdan.jr): is this needed?
-  base::LaunchAppWithHandleInheritance(cmd_line->command_line_string(),
-                                       true,
-                                       false,
-                                       NULL);
-#else
-  base::LaunchApp(*cmd_line.get(), true, false, NULL);
-#endif
-}
-
-void UITest::StopHttpServer() {
-  scoped_ptr<CommandLine> cmd_line(CreateHttpServerCommandLine());
-  ASSERT_TRUE(cmd_line.get());
-  cmd_line->AppendSwitchASCII("server", "stop");
-
-#if defined(OS_WIN)
-  // TODO(phajdan.jr): is this needed?
-  base::LaunchAppWithHandleInheritance(cmd_line->command_line_string(),
-                                       true,
-                                       false,
-                                       NULL);
-#else
-  base::LaunchApp(*cmd_line.get(), true, false, NULL);
-#endif
-}
-
-int UITest::GetBrowserProcessCount() {
-  return GetRunningChromeProcesses(browser_process_id()).size();
+bool UITest::GetBrowserProcessCount(int* count) {
+  *count = 0;
+  if (!automation()->WaitForProcessLauncherThreadToGoIdle())
+    return false;
+  *count = GetRunningChromeProcesses(browser_process_id()).size();
+  return true;
 }
 
 static DictionaryValue* LoadDictionaryValueFromPath(const FilePath& path) {
@@ -630,7 +574,7 @@ DictionaryValue* UITest::GetLocalState() {
 DictionaryValue* UITest::GetDefaultProfilePreferences() {
   FilePath path;
   PathService::Get(chrome::DIR_USER_DATA, &path);
-  path = path.AppendASCII(WideToUTF8(chrome::kNotSignedInProfile));
+  path = path.AppendASCII(chrome::kInitialProfile);
   return LoadDictionaryValueFromPath(path.Append(chrome::kPreferencesFilename));
 }
 
@@ -660,10 +604,12 @@ void UITest::WaitForFinish(const std::string &name,
 }
 
 bool UITest::EvictFileFromSystemCacheWrapper(const FilePath& path) {
-  for (int i = 0; i < 10; i++) {
+  const int kCycles = 10;
+  const TimeDelta kDelay = TestTimeouts::action_timeout() / kCycles;
+  for (int i = 0; i < kCycles; i++) {
     if (file_util::EvictFileFromSystemCache(path))
       return true;
-    base::PlatformThread::Sleep(TestTimeouts::action_timeout_ms() / 10);
+    base::PlatformThread::Sleep(kDelay);
   }
   return false;
 }
@@ -678,6 +624,7 @@ void UITest::WaitForGeneratedFileAndCheck(
   base::PlatformFileInfo previous, current;
   bool exist = false;
   const int kCycles = 20;
+  const TimeDelta kDelay = TestTimeouts::action_timeout() / kCycles;
   for (int i = 0; i < kCycles; ++i) {
     if (exist) {
       file_util::GetFileInfo(generated_file, &current);
@@ -688,7 +635,7 @@ void UITest::WaitForGeneratedFileAndCheck(
       file_util::GetFileInfo(generated_file, &previous);
       exist = true;
     }
-    base::PlatformThread::Sleep(TestTimeouts::action_timeout_ms() / kCycles);
+    base::PlatformThread::Sleep(kDelay);
   }
   EXPECT_TRUE(exist);
 
@@ -716,16 +663,11 @@ bool UITest::WaitUntilJavaScriptCondition(TabProxy* tab,
                                           const std::wstring& frame_xpath,
                                           const std::wstring& jscript,
                                           int timeout_ms) {
-  const int kIntervalMs = 250;
-  const int kMaxIntervals = timeout_ms / kIntervalMs;
+  const TimeDelta kDelay = TimeDelta::FromMilliseconds(250);
+  const int kMaxDelays = timeout_ms / kDelay.InMilliseconds();
 
   // Wait until the test signals it has completed.
-  for (int i = 0; i < kMaxIntervals; ++i) {
-    bool browser_survived = CrashAwareSleep(kIntervalMs);
-    EXPECT_TRUE(browser_survived);
-    if (!browser_survived)
-      return false;
-
+  for (int i = 0; i < kMaxDelays; ++i) {
     bool done_value = false;
     bool success = tab->ExecuteAndExtractBool(frame_xpath, jscript,
                                               &done_value);
@@ -734,6 +676,8 @@ bool UITest::WaitUntilJavaScriptCondition(TabProxy* tab,
       return false;
     if (done_value)
       return true;
+
+    base::PlatformThread::Sleep(kDelay);
   }
 
   ADD_FAILURE() << "Timeout reached in WaitUntilJavaScriptCondition";
@@ -745,19 +689,16 @@ bool UITest::WaitUntilCookieValue(TabProxy* tab,
                                   const char* cookie_name,
                                   int timeout_ms,
                                   const char* expected_value) {
-  const int kIntervalMs = 250;
-  const int kMaxIntervals = timeout_ms / kIntervalMs;
+  const TimeDelta kDelay = TimeDelta::FromMilliseconds(250);
+  const int kMaxDelays = timeout_ms / kDelay.InMilliseconds();
 
   std::string cookie_value;
-  for (int i = 0; i < kMaxIntervals; ++i) {
-    bool browser_survived = CrashAwareSleep(kIntervalMs);
-    EXPECT_TRUE(browser_survived);
-    if (!browser_survived)
-      return false;
-
+  for (int i = 0; i < kMaxDelays; ++i) {
     EXPECT_TRUE(tab->GetCookieByName(url, cookie_name, &cookie_value));
     if (cookie_value == expected_value)
       return true;
+
+    base::PlatformThread::Sleep(kDelay);
   }
 
   ADD_FAILURE() << "Timeout reached in WaitUntilCookieValue";
@@ -768,19 +709,16 @@ std::string UITest::WaitUntilCookieNonEmpty(TabProxy* tab,
                                             const GURL& url,
                                             const char* cookie_name,
                                             int timeout_ms) {
-  const int kIntervalMs = 250;
-  const int kMaxIntervals = timeout_ms / kIntervalMs;
-
-  for (int i = 0; i < kMaxIntervals; ++i) {
-    bool browser_survived = CrashAwareSleep(kIntervalMs);
-    EXPECT_TRUE(browser_survived);
-    if (!browser_survived)
-      return std::string();
+  const TimeDelta kDelay = TimeDelta::FromMilliseconds(250);
+  const int kMaxDelays = timeout_ms / kDelay.InMilliseconds();
 
+  for (int i = 0; i < kMaxDelays; ++i) {
     std::string cookie_value;
     EXPECT_TRUE(tab->GetCookieByName(url, cookie_name, &cookie_value));
     if (!cookie_value.empty())
       return cookie_value;
+
+    base::PlatformThread::Sleep(kDelay);
   }
 
   ADD_FAILURE() << "Timeout reached in WaitUntilCookieNonEmpty";
@@ -798,6 +736,7 @@ bool UITest::WaitForDownloadShelfInvisible(BrowserProxy* browser) {
 bool UITest::WaitForFindWindowVisibilityChange(BrowserProxy* browser,
                                                bool wait_for_open) {
   const int kCycles = 10;
+  const TimeDelta kDelay = TestTimeouts::action_timeout() / kCycles;
   for (int i = 0; i < kCycles; i++) {
     bool visible = false;
     if (!browser->IsFindWindowFullyVisible(&visible))
@@ -806,37 +745,52 @@ bool UITest::WaitForFindWindowVisibilityChange(BrowserProxy* browser,
       return true;  // Find window visibility change complete.
 
     // Give it a chance to catch up.
-    bool browser_survived = CrashAwareSleep(
-        TestTimeouts::action_timeout_ms() / kCycles);
-    EXPECT_TRUE(browser_survived);
-    if (!browser_survived)
-      return false;
+    base::PlatformThread::Sleep(kDelay);
   }
 
   ADD_FAILURE() << "Timeout reached in WaitForFindWindowVisibilityChange";
   return false;
 }
 
+void UITest::TerminateBrowser() {
+  launcher_->TerminateBrowser();
+
+  // Make sure the UMA metrics say we didn't crash.
+  scoped_ptr<DictionaryValue> local_prefs(GetLocalState());
+  bool exited_cleanly;
+  ASSERT_TRUE(local_prefs.get());
+  ASSERT_TRUE(local_prefs->GetBoolean(prefs::kStabilityExitedCleanly,
+                                      &exited_cleanly));
+  ASSERT_TRUE(exited_cleanly);
+
+  // And that session end was successful.
+  bool session_end_completed;
+  ASSERT_TRUE(local_prefs->GetBoolean(prefs::kStabilitySessionEndCompleted,
+                                      &session_end_completed));
+  ASSERT_TRUE(session_end_completed);
+
+  // Make sure session restore says we didn't crash.
+  scoped_ptr<DictionaryValue> profile_prefs(GetDefaultProfilePreferences());
+  ASSERT_TRUE(profile_prefs.get());
+  ASSERT_TRUE(profile_prefs->GetBoolean(prefs::kSessionExitedCleanly,
+                                        &exited_cleanly));
+  ASSERT_TRUE(exited_cleanly);
+}
+
+void UITest::NavigateToURLAsync(const GURL& url) {
+  scoped_refptr<TabProxy> tab_proxy(GetActiveTab());
+  ASSERT_TRUE(tab_proxy.get());
+  ASSERT_TRUE(tab_proxy->NavigateToURLAsync(url));
+}
+
 bool UITest::WaitForDownloadShelfVisibilityChange(BrowserProxy* browser,
                                                   bool wait_for_open) {
   const int kCycles = 10;
+  const TimeDelta kDelay = TestTimeouts::action_timeout() / kCycles;
   int fail_count = 0;
   int incorrect_state_count = 0;
   base::Time start = base::Time::Now();
   for (int i = 0; i < kCycles; i++) {
-    // Give it a chance to catch up.
-    bool browser_survived = CrashAwareSleep(
-        TestTimeouts::action_timeout_ms() / kCycles);
-    EXPECT_TRUE(browser_survived);
-    if (!browser_survived) {
-      LOG(INFO) << "Elapsed time: " << (base::Time::Now() - start).InSecondsF()
-                << " seconds"
-                << " call failed " << fail_count << " times"
-                << " state was incorrect " << incorrect_state_count << " times";
-      ADD_FAILURE() << "Browser failed in " << __FUNCTION__;
-      return false;
-    }
-
     bool visible = !wait_for_open;
     if (!browser->IsShelfVisible(&visible)) {
       fail_count++;
@@ -850,6 +804,9 @@ bool UITest::WaitForDownloadShelfVisibilityChange(BrowserProxy* browser,
       return true;  // Got the download shelf.
     }
     incorrect_state_count++;
+
+    // Give it a chance to catch up.
+    base::PlatformThread::Sleep(kDelay);
   }
 
   LOG(INFO) << "Elapsed time: " << (base::Time::Now() - start).InSecondsF()
