@@ -15,7 +15,7 @@
 
 #include <string>
 
-#include "chrome_tab.h"  // NOLINT
+#include "chrome_frame/chrome_tab.h"
 #include "chrome_frame/delete_chrome_history.h"
 #include "chrome_frame/resource.h"
 #include "chrome_frame/urlmon_moniker.h"
@@ -111,7 +111,7 @@ END_SINK_MAP()
   static void ProcessOptInUrls(IWebBrowser2* browser, BSTR url);
 
   // COM_INTERFACE_ENTRY_CACHED_TEAR_OFF manages the raw pointer from CComPtr
-  // which ScopedComPtr doesn't expose.
+  // which base::win::ScopedComPtr doesn't expose.
   CComPtr<IUnknown> delete_chrome_history_;
 
  protected:
