@@ -7,42 +7,24 @@
 #pragma once
 
 #include <windows.h>
+
 #include <map>  // for proxy factory
 #include <vector>
 #include <string>
+
 #include "base/synchronization/lock.h"
 #include "base/time.h"         // for base::TimeDelta
 #include "base/file_path.h"
 #include "chrome/common/automation_constants.h"
-#include "chrome/common/page_zoom.h"
+#include "content/public/common/page_zoom.h"
 #include "ipc/ipc_channel.h"
 
-enum FindInPageDirection { BACK = 0, FWD = 1 };
-enum FindInPageCase { IGNORE_CASE = 0, CASE_SENSITIVE = 1 };
-// Specifies the font size on a page which is requested by an automation
-// client.
-enum AutomationPageFontSize {
-  SMALLEST_FONT = 8,
-  SMALL_FONT = 12,
-  MEDIUM_FONT = 16,
-  LARGE_FONT = 24,
-  LARGEST_FONT = 36
-};
-
 class ChromeProxyDelegate;
 class ChromeProxyFactory;
 class GURL;
-struct AttachExternalTabParams;
-struct AutomationURLRequest;
 struct ExternalTabSettings;
-struct MiniContextMenuParams;
-struct NavigationInfo;
 struct ProxyParams;
 
-namespace net {
-class URLRequestStatus;
-}  // namespace net
-
 // Some callers of synchronous messages wants a context to be passed back
 // in order to identify the call they made. Presumably one can make
 // multiple sync calls of same type (as async) and want to identify what
@@ -53,9 +35,8 @@ struct SyncMessageContext {
 
 
 /*
-[npapi]         UIDelegate (UI_THREAD)
-[activex]    <---------------+
-[activedoc]                  |
+[activex]       UIDelegate (UI_THREAD)
+[activedoc]  <---------------+
                              |
                              |            ChromeProxy (UI_THREAD)
                        +----------------+ -------------->   +-------+
@@ -78,14 +59,6 @@ class ChromeProxy {
  public:
   // General
   virtual void RemoveBrowsingData(int remove_mask) = 0;    // async
-  virtual void InstallExtension(ChromeProxyDelegate* delegate,
-                                const FilePath& crx_path,
-                                SyncMessageContext* ctx) = 0;
-  virtual void LoadExtension(ChromeProxyDelegate* delegate,
-                             const FilePath& path,
-                             SyncMessageContext* ctx) = 0;
-  virtual void GetEnabledExtensions(ChromeProxyDelegate* delegate,
-                                    SyncMessageContext* ctx) = 0;
   virtual void SetProxyConfig(const std::string& json_encoded_settings) = 0;
 
   // Tab management.
@@ -113,7 +86,7 @@ class ChromeProxy {
   virtual void Tab_MenuCommand(int tab, int selected_command) = 0;
 
   // UI
-  virtual void Tab_Zoom(int tab, PageZoom::Function zoom_level) = 0;
+  virtual void Tab_Zoom(int tab, content::PageZoom zoom_level) = 0;
   virtual void Tab_FontSize(int tab, enum AutomationPageFontSize font_size) = 0;
   virtual void Tab_SetInitialFocus(int tab,
       bool reverse, bool restore_focus_to_view) = 0;
@@ -124,8 +97,6 @@ class ChromeProxy {
   // Misc.
   virtual void Tab_OnHostMoved(int tab) = 0;
   virtual void Tab_RunUnloadHandlers(int tab) = 0;
-  virtual void Tab_SetEnableExtensionAutomation(int tab,
-      const std::vector<std::string>& functions_enabled) = 0;
   virtual void Tab_Navigate(int tab, const GURL& url, const GURL& referrer) = 0;
   virtual void Tab_OverrideEncoding(int tab, const char* encoding) = 0;
 
@@ -163,12 +134,6 @@ class ChromeProxyDelegate : public IPC::Channel::Listener {
       HWND tab_window, int tab_handle, int session_id) = 0;
   virtual void Completed_Navigate(bool success,
       enum AutomationMsg_NavigationResponseValues res) = 0;
-  virtual void Completed_InstallExtension(bool success,
-      AutomationMsg_ExtensionResponseValues res, SyncMessageContext* ctx) = 0;
-  virtual void Completed_LoadExpandedExtension(bool success,
-      AutomationMsg_ExtensionResponseValues res, SyncMessageContext* ctx) = 0;
-  virtual void Completed_GetEnabledExtensions(bool success,
-      const std::vector<FilePath>* extensions) = 0;
 
  protected:
   ~ChromeProxyDelegate() {}
