@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -80,7 +80,7 @@ class Interface2IPCMessage : public ChromeProxy {
   virtual void Tab_Paste(int tab);
   virtual void Tab_SelectAll(int tab);
   virtual void Tab_MenuCommand(int tab, int selected_command);
-  virtual void Tab_Zoom(int tab, PageZoom::Function zoom_level);
+  virtual void Tab_Zoom(int tab, content::PageZoom zoom_level);
   virtual void Tab_FontSize(int tab, enum AutomationPageFontSize font_size);
   virtual void Tab_SetInitialFocus(int tab, bool reverse,
                                    bool restore_focus_to_view);
@@ -90,8 +90,6 @@ class Interface2IPCMessage : public ChromeProxy {
 
   // Misc.
   virtual void Tab_OnHostMoved(int tab);
-  virtual void Tab_SetEnableExtensionAutomation(int tab,
-      const std::vector<std::string>& functions_enabled);
  protected:
   ~Interface2IPCMessage() {}
  private:
@@ -145,12 +143,6 @@ class CFProxy : public Interface2IPCMessage,
 
   //////////////////////////////////////////////////////////////////////////
   // Sync messages.
-  virtual void InstallExtension(ChromeProxyDelegate* delegate,
-      const FilePath& crx_path, SyncMessageContext* ctx);
-  virtual void LoadExtension(ChromeProxyDelegate* delegate,
-                             const FilePath& path, SyncMessageContext* ctx);
-  virtual void GetEnabledExtensions(ChromeProxyDelegate* delegate,
-                                    SyncMessageContext* ctx);
   virtual void Tab_Find(int tab, const string16& search_string,
       FindInPageDirection forward, FindInPageCase match_case, bool find_next);
   virtual void Tab_OverrideEncoding(int tab, const char* encoding);
@@ -163,10 +155,10 @@ class CFProxy : public Interface2IPCMessage,
   virtual void Tab_RunUnloadHandlers(int tab);
 
   //////////////////////////////////////////////////////////////////////////
-  // IPC::Channel::Listener
-  virtual bool OnMessageReceived(const IPC::Message& message);
-  virtual void OnChannelConnected(int32 peer_pid);
-  virtual void OnChannelError();
+  // IPC::Channel::Listener implementation.
+  virtual bool OnMessageReceived(const IPC::Message& message) OVERRIDE;
+  virtual void OnChannelConnected(int32 peer_pid) OVERRIDE;
+  virtual void OnChannelError() OVERRIDE;
 
   bool CalledOnIpcThread() const {
     return base::PlatformThread::CurrentId() == ipc_thread_.thread_id();
@@ -180,8 +172,6 @@ class CFProxy : public Interface2IPCMessage,
   bool is_connected_;
 };
 
-DISABLE_RUNNABLE_METHOD_REFCOUNT(CFProxy);
-
 // Support functions.
 std::string GenerateChannelId();
 std::wstring BuildCmdLine(const std::string& channel_id,
