@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #include <vector>
 #include "base/atomic_sequence_num.h"
 #include "base/command_line.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/process_util.h"
 #include "chrome/common/automation_messages.h"
 #include "chrome/common/chrome_switches.h"
@@ -25,10 +26,6 @@ void DispatchReplyFail(uint32 type,
     case AutomationMsg_ConnectExternalTab::ID:
       delegate->Completed_ConnectToTab(false, NULL, NULL, 0, 0);
       break;
-    case AutomationMsg_InstallExtension::ID:
-      delegate->Completed_InstallExtension(false,
-          AUTOMATION_MSG_EXTENSION_INSTALL_FAILED, ctx);
-      break;
   }
 }
 
@@ -58,32 +55,6 @@ bool DispatchReplyOk(const IPC::Message* reply_msg, uint32 type,
       }
       return true;
     }
-
-    case AutomationMsg_InstallExtension::ID: {
-      // Tuple1<AutomationMsg_ExtensionResponseValues> out;
-      TupleTypes<AutomationMsg_InstallExtension::ReplyParam>::ValueTuple out;
-      if (ReadParam(reply_msg, &iter, &out))
-        delegate->Completed_InstallExtension(true, out.a, ctx);
-      return true;
-    }
-
-    case AutomationMsg_LoadExpandedExtension::ID: {
-      // Tuple1<AutomationMsg_ExtensionResponseValues> out;
-      TupleTypes<AutomationMsg_LoadExpandedExtension::ReplyParam>::ValueTuple
-          out;
-      if (ReadParam(reply_msg, &iter, &out))
-        delegate->Completed_LoadExpandedExtension(true, out.a, ctx);
-      break;
-    }
-
-    case AutomationMsg_GetEnabledExtensions::ID: {
-      // Tuple1<std::vector<FilePath> >
-      TupleTypes<AutomationMsg_GetEnabledExtensions::ReplyParam>::ValueTuple
-          out;
-      if (ReadParam(reply_msg, &iter, &out))
-        delegate->Completed_GetEnabledExtensions(true, &out.a);
-      break;
-    }
   }  // switch
 
   return false;
@@ -145,7 +116,7 @@ void Interface2IPCMessage::Tab_MenuCommand(int tab, int selected_command) {
       tab, selected_command));
 }
 
-void Interface2IPCMessage::Tab_Zoom(int tab, PageZoom::Function zoom_level) {
+void Interface2IPCMessage::Tab_Zoom(int tab, content::PageZoom zoom_level) {
   sender_->Send(new AutomationMsg_SetZoomLevel(tab, zoom_level));
 }
 
@@ -179,12 +150,6 @@ void Interface2IPCMessage::Tab_OnHostMoved(int tab) {
   sender_->Send(new AutomationMsg_BrowserMove(tab));
 }
 
-void Interface2IPCMessage::Tab_SetEnableExtensionAutomation(int tab,
-    const std::vector<std::string>& functions_enabled) {
-  sender_->Send(new AutomationMsg_SetEnableExtensionAutomation(tab,
-                functions_enabled));
-}
-
 void DelegateHolder::AddDelegate(ChromeProxyDelegate* p) {
   delegate_list_.insert(p);
 }
@@ -310,35 +275,38 @@ std::string GenerateChannelId() {
 std::wstring BuildCmdLine(const std::string& channel_id,
                           const FilePath& profile_path,
                           const std::wstring& extra_args) {
-  scoped_ptr<CommandLine> command_line(
-      chrome_launcher::CreateLaunchCommandLine());
-  command_line->AppendSwitchASCII(switches::kAutomationClientChannelID,
-      channel_id);
-  // Run Chrome in Chrome Frame mode. In practice, this modifies the paths
-  // and registry keys that Chrome looks in via the BrowserDistribution
-  // mechanism.
-  command_line->AppendSwitch(switches::kChromeFrame);
-  // Chrome Frame never wants Chrome to start up with a First Run UI.
-  command_line->AppendSwitch(switches::kNoFirstRun);
-  command_line->AppendSwitch(switches::kDisablePopupBlocking);
+  std::wstring command_line_string;
+  scoped_ptr<CommandLine> command_line;
+  if (chrome_launcher::CreateLaunchCommandLine(&command_line)) {
+    command_line->AppendSwitchASCII(switches::kAutomationClientChannelID,
+        channel_id);
+    // Run Chrome in Chrome Frame mode. In practice, this modifies the paths
+    // and registry keys that Chrome looks in via the BrowserDistribution
+    // mechanism.
+    command_line->AppendSwitch(switches::kChromeFrame);
+    // Chrome Frame never wants Chrome to start up with a First Run UI.
+    command_line->AppendSwitch(switches::kNoFirstRun);
+    command_line->AppendSwitch(switches::kDisablePopupBlocking);
 
-#ifndef NDEBUG
-  // Disable the "Whoa! Chrome has crashed." dialog, because that isn't very
-  // useful for Chrome Frame users.
-  command_line->AppendSwitch(switches::kNoErrorDialogs);
-#endif
+  #ifndef NDEBUG
+    // Disable the "Whoa! Chrome has crashed." dialog, because that isn't very
+    // useful for Chrome Frame users.
+    command_line->AppendSwitch(switches::kNoErrorDialogs);
+  #endif
 
-  // In headless mode runs like reliability test runs we want full crash dumps
-  // from chrome.
-  if (IsHeadlessMode())
-    command_line->AppendSwitch(switches::kFullMemoryCrashReport);
+    // In headless mode runs like reliability test runs we want full crash dumps
+    // from chrome.
+    if (IsHeadlessMode())
+      command_line->AppendSwitch(switches::kFullMemoryCrashReport);
 
-  command_line->AppendSwitchPath(switches::kUserDataDir, profile_path);
+    command_line->AppendSwitchPath(switches::kUserDataDir, profile_path);
 
-  std::wstring command_line_string(command_line->command_line_string());
-  if (!extra_args.empty()) {
-    command_line_string.append(L" ");
-    command_line_string.append(extra_args);
+    command_line_string = command_line->GetCommandLineString();
+    if (!extra_args.empty()) {
+      command_line_string.append(L" ");
+      command_line_string.append(extra_args);
+    }
   }
+
   return command_line_string;
 }
