@@ -3,7 +3,11 @@
 // found in the LICENSE file.
 
 #include <string>
+
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/file_path.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/synchronization/waitable_event.h"
 #include "chrome/common/automation_messages.h"
 #include "chrome_frame/cfproxy_private.h"
@@ -40,12 +44,6 @@ struct MockChromeProxyDelegate : public ChromeProxyDelegate {
       HWND tab_window, int tab_handle, int session_id));
   MOCK_METHOD2(Completed_Navigate, void(bool success,
       enum AutomationMsg_NavigationResponseValues res));
-  MOCK_METHOD3(Completed_InstallExtension, void(bool success,
-      enum AutomationMsg_ExtensionResponseValues res, SyncMessageContext* ctx));
-  MOCK_METHOD3(Completed_LoadExpandedExtension, void(bool success,
-      enum AutomationMsg_ExtensionResponseValues res, SyncMessageContext* ctx));
-  MOCK_METHOD2(Completed_GetEnabledExtensions, void(bool success,
-      const std::vector<FilePath>* v));
 
   // Network requests from Chrome.
   MOCK_METHOD2(Network_Start, void(int request_id,
@@ -106,20 +104,28 @@ struct MockCFProxyTraits : public CFProxyTraits {
   // You may find API_FIRE_XXXX macros (see below) handy instead.
   void FireConnect(base::TimeDelta t) {
     ASSERT_TRUE(ipc_loop != NULL);
-    ipc_loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(listener,
-        &IPC::Channel::Listener::OnChannelConnected, 0), t.InMilliseconds());
+    ipc_loop->PostDelayedTask(
+        FROM_HERE, base::Bind(&IPC::Channel::Listener::OnChannelConnected,
+                              base::Unretained(listener), 0),
+        t.InMilliseconds());
   }
 
   void FireError(base::TimeDelta t) {
     ASSERT_TRUE(ipc_loop != NULL);
-    ipc_loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(listener,
-        &IPC::Channel::Listener::OnChannelError), t.InMilliseconds());
+    ipc_loop->PostDelayedTask(
+        FROM_HERE, base::Bind(&IPC::Channel::Listener::OnChannelError,
+                              base::Unretained(listener)),
+        t.InMilliseconds());
   }
 
   void FireMessage(const IPC::Message& m, base::TimeDelta t) {
     ASSERT_TRUE(ipc_loop != NULL);
-    ipc_loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(listener,
-        &IPC::Channel::Listener::OnMessageReceived, m), t.InMilliseconds());
+    ipc_loop->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(
+            base::IgnoreResult(&IPC::Channel::Listener::OnMessageReceived),
+            base::Unretained(listener), m),
+        t.InMilliseconds());
   }
 
   MockCFProxyTraits() : ipc_loop(NULL) {}
@@ -129,14 +135,13 @@ struct MockCFProxyTraits : public CFProxyTraits {
   IPC::Channel::Listener* listener;
 };
 
-// Handy macros when we want so similate something on the IPC thread.
+// Handy macros when we want so simulate something on the IPC thread.
 #define API_FIRE_CONNECT(api, t)  InvokeWithoutArgs(CreateFunctor(&api, \
                                        &MockCFProxyTraits::FireConnect, t))
 #define API_FIRE_ERROR(api, t) InvokeWithoutArgs(CreateFunctor(&api, \
                                        &MockCFProxyTraits::FireError, t))
 #define API_FIRE_MESSAGE(api, t) InvokeWithoutArgs(CreateFunctor(&api, \
                                        &MockCFProxyTraits::FireMessage, t))
-DISABLE_RUNNABLE_METHOD_REFCOUNT(IPC::Channel::Listener);
 
 TEST(ChromeProxy, DelegateAddRemove) {
   StrictMock<MockCFProxyTraits> api;
@@ -328,7 +333,6 @@ inline IPC::Message* CreateReply(M* m, const A& a, const B& b, const C& c,
   return r;
 }}  // namespace
 
-DISABLE_RUNNABLE_METHOD_REFCOUNT(SyncMsgSender);
 TEST(SyncMsgSender, Deserialize) {
   // Note the ipc thread is not actually needed, but we try to be close
   // to real-world conditions - that SyncMsgSender works from multiple threads.
@@ -342,28 +346,22 @@ TEST(SyncMsgSender, Deserialize) {
   const int kTabHandle = 6;
   const int kSessionId = 8;
 
-  // Create some sync messages and their replies.
-  AutomationMsg_InstallExtension m1(FilePath(L"c:\\awesome.x"), 0);
-  AutomationMsg_CreateExternalTab m2(ExternalTabSettings(), 0, 0, 0, 0);
-  scoped_ptr<IPC::Message> r1(CreateReply(&m1,
-      AUTOMATION_MSG_EXTENSION_INSTALL_SUCCEEDED));
-  scoped_ptr<IPC::Message> r2(CreateReply(&m2, (HWND)1, (HWND)2, kTabHandle,
-                                          kSessionId));
+  // Create a sync message and its reply.
+  AutomationMsg_CreateExternalTab m(ExternalTabSettings(), 0, 0, 0, 0);
+  scoped_ptr<IPC::Message> r(CreateReply(&m, (HWND)1, (HWND)2, kTabHandle,
+                                         kSessionId));
 
-  queue.QueueSyncMessage(&m1, &d1, NULL);
-  queue.QueueSyncMessage(&m2, &d1, NULL);
+  queue.QueueSyncMessage(&m, &d1, NULL);
 
   testing::InSequence s;
-  EXPECT_CALL(d1, Completed_InstallExtension(true,
-      AUTOMATION_MSG_EXTENSION_INSTALL_SUCCEEDED, NULL));
   EXPECT_CALL(d1, Completed_CreateTab(true, (HWND)1, (HWND)2, kTabHandle,
                                       kSessionId));
 
   // Execute replies in a worker thread.
-  ipc.message_loop()->PostTask(FROM_HERE, NewRunnableMethod(&queue,
-      &SyncMsgSender::OnReplyReceived, r1.get()));
-  ipc.message_loop()->PostTask(FROM_HERE, NewRunnableMethod(&queue,
-      &SyncMsgSender::OnReplyReceived, r2.get()));
+  ipc.message_loop()->PostTask(
+      FROM_HERE,
+      base::Bind(base::IgnoreResult(&SyncMsgSender::OnReplyReceived),
+                 base::Unretained(&queue), r.get()));
   ipc.Stop();
 
   // Expect that tab 6 has been associated with the delegate.
