@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,24 +19,14 @@
 
 #include "base/command_line.h"
 #include "base/debug/trace_event.h"
-#include "base/file_util.h"
 #include "base/logging.h"
-#include "base/path_service.h"
-#include "base/process_util.h"
-#include "base/string_tokenizer.h"
 #include "base/string_util.h"
-#include "base/threading/thread.h"
-#include "base/threading/thread_local.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/scoped_variant.h"
-#include "base/win/win_util.h"
-#include "grit/generated_resources.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/app/chrome_dll_resource.h"
-#include "chrome/browser/tab_contents/tab_contents.h"
+#include "chrome/common/automation_messages.h"
 #include "chrome/common/chrome_constants.h"
-#include "chrome/common/navigation_types.h"
-#include "chrome/common/page_zoom.h"
 #include "chrome/test/automation/browser_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome_frame/bho.h"
@@ -44,12 +34,14 @@
 #include "chrome_frame/buggy_bho_handling.h"
 #include "chrome_frame/crash_reporting/crash_metrics.h"
 #include "chrome_frame/utils.h"
+#include "content/browser/tab_contents/tab_contents.h"
+#include "content/public/browser/invalidate_type.h"
+#include "content/public/common/page_zoom.h"
+#include "grit/generated_resources.h"
 
 DEFINE_GUID(CGID_DocHostCmdPriv, 0x000214D4L, 0, 0, 0xC0, 0, 0, 0, 0, 0, 0,
             0x46);
 
-base::ThreadLocalPointer<ChromeActiveDocument> g_active_doc_cache;
-
 bool g_first_launch_by_process_ = true;
 
 const DWORD kIEEncodingIdArray[] = {
@@ -60,40 +52,26 @@ const DWORD kIEEncodingIdArray[] = {
 };
 
 ChromeActiveDocument::ChromeActiveDocument()
-    : first_navigation_(true),
+    : navigation_info_(new NavigationInfo()),
+      first_navigation_(true),
       is_automation_client_reused_(false),
       popup_allowed_(false),
       accelerator_table_(NULL) {
-  TRACE_EVENT_BEGIN("chromeframe.createactivedocument", this, "");
+  TRACE_EVENT_BEGIN_ETW("chromeframe.createactivedocument", this, "");
 
   url_fetcher_->set_frame_busting(false);
-  memset(&navigation_info_, 0, sizeof(navigation_info_));
+  memset(navigation_info_.get(), 0, sizeof(NavigationInfo));
 }
 
 HRESULT ChromeActiveDocument::FinalConstruct() {
-  // If we have a cached ChromeActiveDocument instance in TLS, then grab
-  // ownership of the cached document's automation client. This is an
-  // optimization to get Chrome active documents to load faster.
-  ChromeActiveDocument* cached_document = g_active_doc_cache.Get();
-  if (cached_document && cached_document->IsValid()) {
-    SetResourceModule();
-    DCHECK(automation_client_.get() == NULL);
-    automation_client_.swap(cached_document->automation_client_);
-    DVLOG(1) << "Reusing automation client instance from " << cached_document;
-    DCHECK(automation_client_.get() != NULL);
-    automation_client_->Reinitialize(this, url_fetcher_.get());
-    is_automation_client_reused_ = true;
-    OnAutomationServerReady();
-  } else {
-    // The FinalConstruct implementation in the ChromeFrameActivexBase class
-    // i.e. Base creates an instance of the ChromeFrameAutomationClient class
-    // and initializes it, which would spawn a new Chrome process, etc.
-    // We don't want to be doing this if we have a cached document, whose
-    // automation client instance can be reused.
-    HRESULT hr = BaseActiveX::FinalConstruct();
-    if (FAILED(hr))
-      return hr;
-  }
+  // The FinalConstruct implementation in the ChromeFrameActivexBase class
+  // i.e. Base creates an instance of the ChromeFrameAutomationClient class
+  // and initializes it, which would spawn a new Chrome process, etc.
+  // We don't want to be doing this if we have a cached document, whose
+  // automation client instance can be reused.
+  HRESULT hr = BaseActiveX::FinalConstruct();
+  if (FAILED(hr))
+    return hr;
 
   InitializeAutomationSettings();
 
@@ -131,7 +109,7 @@ ChromeActiveDocument::~ChromeActiveDocument() {
   // ChromeFramePlugin
   BaseActiveX::Uninitialize();
 
-  TRACE_EVENT_END("chromeframe.createactivedocument", this, "");
+  TRACE_EVENT_END_ETW("chromeframe.createactivedocument", this, "");
 }
 
 // Override DoVerb
@@ -152,7 +130,7 @@ STDMETHODIMP ChromeActiveDocument::DoVerb(LONG verb,
   if (doc_site_) {
     switch (verb) {
     case OLEIVERB_SHOW: {
-      ScopedComPtr<IDocHostUIHandler> doc_host_handler;
+      base::win::ScopedComPtr<IDocHostUIHandler> doc_host_handler;
       doc_host_handler.QueryFrom(doc_site_);
       if (doc_host_handler.get())
         doc_host_handler->ShowUI(DOCHOSTUITYPE_BROWSE, this, this, NULL, NULL);
@@ -213,9 +191,9 @@ STDMETHODIMP ChromeActiveDocument::Load(BOOL fully_avalable,
   if (NULL == moniker_name)
     return E_INVALIDARG;
 
-  ScopedComPtr<IOleClientSite> client_site;
+  base::win::ScopedComPtr<IOleClientSite> client_site;
   if (bind_context) {
-    ScopedComPtr<IUnknown> site;
+    base::win::ScopedComPtr<IUnknown> site;
     bind_context->GetObjectParam(SZ_HTML_CLIENTSITE_OBJECTPARAM,
                                  site.Receive());
     if (site)
@@ -230,7 +208,7 @@ STDMETHODIMP ChromeActiveDocument::Load(BOOL fully_avalable,
     // See if mshtml parsed the html header for us.  If so, we need to
     // clear the browser service flag that we use to indicate that this
     // browser instance is navigating to a CF document.
-    ScopedComPtr<IBrowserService> browser_service;
+    base::win::ScopedComPtr<IBrowserService> browser_service;
     DoQueryService(SID_SShellBrowser, client_site, browser_service.Receive());
     if (browser_service) {
       bool flagged = CheckForCFNavigation(browser_service, true);
@@ -243,7 +221,7 @@ STDMETHODIMP ChromeActiveDocument::Load(BOOL fully_avalable,
 
   std::wstring url;
 
-  ScopedComPtr<BindContextInfo> info;
+  base::win::ScopedComPtr<BindContextInfo> info;
   BindContextInfo::FromBindContext(bind_context, info.Receive());
   DCHECK(info);
   if (info && !info->GetUrl().empty()) {
@@ -312,7 +290,7 @@ STDMETHODIMP ChromeActiveDocument::Load(BOOL fully_avalable,
   // When the code for the browser service patch and for the moniker patch is
   // removed, this conditional can also go away.)
   if (RENDERER_TYPE_UNDETERMINED != renderer_type) {
-    THREAD_SAFE_UMA_LAUNCH_TYPE_COUNT(renderer_type);
+    UMA_LAUNCH_TYPE_COUNT(renderer_type);
   }
 
   return S_OK;
@@ -442,7 +420,7 @@ STDMETHODIMP ChromeActiveDocument::SaveHistory(IStream* stream) {
   LARGE_INTEGER offset = {0};
   ULARGE_INTEGER new_pos = {0};
   DWORD written = 0;
-  std::wstring url = UTF8ToWide(navigation_info_.url.spec());
+  std::wstring url = UTF8ToWide(navigation_info_->url.spec());
   return stream->Write(url.c_str(), (url.length() + 1) * sizeof(wchar_t),
                        &written);
 }
@@ -450,7 +428,7 @@ STDMETHODIMP ChromeActiveDocument::SaveHistory(IStream* stream) {
 STDMETHODIMP ChromeActiveDocument::SetPositionCookie(DWORD position_cookie) {
   if (automation_client_.get()) {
     int index = static_cast<int>(position_cookie);
-    navigation_info_.navigation_index = index;
+    navigation_info_->navigation_index = index;
     automation_client_->NavigateToIndex(index);
   } else {
     DLOG(WARNING) << "Invalid automation client instance";
@@ -462,7 +440,7 @@ STDMETHODIMP ChromeActiveDocument::GetPositionCookie(DWORD* position_cookie) {
   if (!position_cookie)
     return E_INVALIDARG;
 
-  *position_cookie = navigation_info_.navigation_index;
+  *position_cookie = navigation_info_->navigation_index;
   return S_OK;
 }
 
@@ -541,14 +519,7 @@ HRESULT ChromeActiveDocument::GetInPlaceFrame(
 HRESULT ChromeActiveDocument::IOleObject_SetClientSite(
     IOleClientSite* client_site) {
   if (client_site == NULL) {
-    ChromeActiveDocument* cached_document = g_active_doc_cache.Get();
-    if (cached_document) {
-      DCHECK(this == cached_document);
-      g_active_doc_cache.Set(NULL);
-      cached_document->Release();
-    }
-
-    ScopedComPtr<IDocHostUIHandler> doc_host_handler;
+    base::win::ScopedComPtr<IDocHostUIHandler> doc_host_handler;
     if (doc_site_)
       doc_host_handler.QueryFrom(doc_site_);
 
@@ -576,7 +547,7 @@ HRESULT ChromeActiveDocument::ActiveXDocActivate(LONG verb) {
   m_bInPlaceActive = TRUE;
   // get location in the parent window,
   // as well as some information about the parent
-  ScopedComPtr<IOleInPlaceUIWindow> in_place_ui_window;
+  base::win::ScopedComPtr<IOleInPlaceUIWindow> in_place_ui_window;
   frame_info_.cb = sizeof(OLEINPLACEFRAMEINFO);
   HWND parent_window = NULL;
   if (m_spInPlaceSite->GetWindow(&parent_window) == S_OK) {
@@ -593,7 +564,7 @@ HRESULT ChromeActiveDocument::ActiveXDocActivate(LONG verb) {
         ::ShowWindow(m_hWnd, SW_SHOW);
         SetFocus();
       } else {
-        m_hWnd = Create(parent_window, position_rect, 0, 0, WS_EX_CLIENTEDGE);
+        m_hWnd = Create(parent_window, position_rect);
         if (!IsWindow()) {
           // This might happen if the automation server couldn't be
           // instantiated.  If so, a NOTREACHED() will have already been hit.
@@ -601,12 +572,11 @@ HRESULT ChromeActiveDocument::ActiveXDocActivate(LONG verb) {
           return AtlHresultFromLastError();
         }
       }
-      SetWindowDimensions();
     }
     SetObjectRects(&position_rect, &clip_rect);
   }
 
-  ScopedComPtr<IOleInPlaceActiveObject> in_place_active_object(this);
+  base::win::ScopedComPtr<IOleInPlaceActiveObject> in_place_active_object(this);
 
   // Gone active by now, take care of UIACTIVATE
   if (DoesVerbUIActivate(verb)) {
@@ -630,7 +600,7 @@ HRESULT ChromeActiveDocument::ActiveXDocActivate(LONG verb) {
 
 void ChromeActiveDocument::OnNavigationStateChanged(
     int flags, const NavigationInfo& nav_info) {
-  // TODO(joshia): handle INVALIDATE_TAB,INVALIDATE_LOAD etc.
+  // TODO(joshia): handle INVALIDATE_TYPE_TAB,INVALIDATE_TYPE_LOAD etc.
   DVLOG(1) << __FUNCTION__
            << "\n Flags: " << flags
            << ", Url: " << nav_info.url
@@ -698,12 +668,6 @@ void ChromeActiveDocument::OnDidNavigate(const NavigationInfo& nav_info) {
   CrashMetricsReporter::GetInstance()->IncrementMetric(
       CrashMetricsReporter::CHROME_FRAME_NAVIGATION_COUNT);
 
-  // This could be NULL if the active document instance is being destroyed.
-  if (!m_spInPlaceSite) {
-    DVLOG(1) << __FUNCTION__ << "m_spInPlaceSite is NULL. Returning";
-    return;
-  }
-
   UpdateNavigationState(nav_info, 0);
 }
 
@@ -712,7 +676,7 @@ void ChromeActiveDocument::OnCloseTab() {
   BaseActiveX::OnCloseTab();
 
   // Close the container window.
-  ScopedComPtr<IWebBrowser2> web_browser2;
+  base::win::ScopedComPtr<IWebBrowser2> web_browser2;
   DoQueryService(SID_SWebBrowserApp, m_spClientSite, web_browser2.Receive());
   if (web_browser2)
     web_browser2->Quit();
@@ -720,19 +684,27 @@ void ChromeActiveDocument::OnCloseTab() {
 
 void ChromeActiveDocument::UpdateNavigationState(
     const NavigationInfo& new_navigation_info, int flags) {
+  // This could be NULL if the active document instance is being destroyed.
+  if (!m_spInPlaceSite) {
+    DVLOG(1) << __FUNCTION__ << "m_spInPlaceSite is NULL. Returning";
+    return;
+  }
+
   HRESULT hr = S_OK;
-  bool is_title_changed = (navigation_info_.title != new_navigation_info.title);
+  bool is_title_changed =
+      (navigation_info_->title != new_navigation_info.title);
   bool is_ssl_state_changed =
-      (navigation_info_.security_style != new_navigation_info.security_style) ||
-      (navigation_info_.displayed_insecure_content !=
+      (navigation_info_->security_style !=
+          new_navigation_info.security_style) ||
+      (navigation_info_->displayed_insecure_content !=
           new_navigation_info.displayed_insecure_content) ||
-      (navigation_info_.ran_insecure_content !=
+      (navigation_info_->ran_insecure_content !=
           new_navigation_info.ran_insecure_content);
 
   if (is_ssl_state_changed) {
     int lock_status = SECURELOCK_SET_UNSECURE;
     switch (new_navigation_info.security_style) {
-      case SECURITY_STYLE_AUTHENTICATED:
+      case content::SECURITY_STYLE_AUTHENTICATED:
         lock_status = new_navigation_info.displayed_insecure_content ?
             SECURELOCK_SET_MIXED : SECURELOCK_SET_SECUREUNKNOWNBIT;
         break;
@@ -745,6 +717,19 @@ void ChromeActiveDocument::UpdateNavigationState(
            OLECMDEXECOPT_DODEFAULT, secure_lock_status.AsInput(), NULL);
   }
 
+  // A number of poorly written bho's crash in their event sink callbacks if
+  // chrome frame is the currently loaded document. This is because they expect
+  // chrome frame to implement interfaces like IHTMLDocument, etc. We patch the
+  // event sink's of these bho's and don't invoke the event sink if chrome
+  // frame is the currently loaded document.
+  if (GetConfigBool(true, kEnableBuggyBhoIntercept)) {
+    base::win::ScopedComPtr<IWebBrowser2> wb2;
+    DoQueryService(SID_SWebBrowserApp, m_spClientSite, wb2.Receive());
+    if (wb2 && buggy_bho::BuggyBhoTls::GetInstance()) {
+      buggy_bho::BuggyBhoTls::GetInstance()->PatchBuggyBHOs(wb2);
+    }
+  }
+
   // Ideally all navigations should come to Chrome Frame so that we can call
   // BeforeNavigate2 on installed BHOs and give them a chance to cancel the
   // navigation. However, in practice what happens is as below:
@@ -772,17 +757,21 @@ void ChromeActiveDocument::UpdateNavigationState(
     url_.Allocate(UTF8ToWide(new_navigation_info.url.spec()).c_str());
 
   if (is_internal_navigation) {
-    ScopedComPtr<IDocObjectService> doc_object_svc;
-    ScopedComPtr<IWebBrowserEventsService> web_browser_events_svc;
-
-    buggy_bho::BuggyBhoTls bad_bho_tls;
-    if (GetConfigBool(true, kEnableBuggyBhoIntercept)) {
-      ScopedComPtr<IWebBrowser2> wb2;
+    // IE6 does not support tabs. If Chrome sent us a window open request
+    // indicating that the navigation needs to occur in a foreground tab or
+    // a popup window, then we need to ensure that the new window in IE6 is
+    // brought to the foreground.
+    if (GetIEVersion() == IE_6 &&
+        is_attach_external_tab_url &&
+        (cf_url.disposition() == NEW_FOREGROUND_TAB ||
+         cf_url.disposition() == NEW_POPUP)) {
+      base::win::ScopedComPtr<IWebBrowser2> wb2;
       DoQueryService(SID_SWebBrowserApp, m_spClientSite, wb2.Receive());
-      if (wb2) {
-        buggy_bho::BuggyBhoTls::PatchBuggyBHOs(wb2);
-      }
+      if (wb2)
+        BaseActiveX::BringWebBrowserWindowToTop(wb2);
     }
+    base::win::ScopedComPtr<IDocObjectService> doc_object_svc;
+    base::win::ScopedComPtr<IWebBrowserEventsService> web_browser_events_svc;
 
     DoQueryService(__uuidof(web_browser_events_svc), m_spClientSite,
                    web_browser_events_svc.Receive());
@@ -845,7 +834,7 @@ void ChromeActiveDocument::UpdateNavigationState(
   // finalized the travel log. This is because IE will ask for information
   // such as navigation index when the travel log is finalized and we need
   // supply the old index and not the new one.
-  navigation_info_ = new_navigation_info;
+  *navigation_info_ = new_navigation_info;
   // Update the IE zone here. Ideally we would like to do it when the active
   // document is activated. However that does not work at times as the frame we
   // get there is not the actual frame which handles the command.
@@ -863,9 +852,9 @@ void ChromeActiveDocument::OnFindInPage() {
 }
 
 void ChromeActiveDocument::OnViewSource() {
-  DCHECK(navigation_info_.url.is_valid());
+  DCHECK(navigation_info_->url.is_valid());
   HostNavigate(GURL(chrome::kViewSourceScheme + std::string(":") +
-      navigation_info_.url.spec()), GURL(), NEW_WINDOW);
+      navigation_info_->url.spec()), GURL(), NEW_WINDOW);
 }
 
 void ChromeActiveDocument::OnDetermineSecurityZone(const GUID* cmd_group_guid,
@@ -908,9 +897,9 @@ void ChromeActiveDocument::OnSetZoomRange(const GUID* cmd_group_guid,
 
   if (in_args && V_VT(in_args) == VT_I4 && IsValid()) {
     if (in_args->lVal == kZoomIn) {
-      automation_client_->SetZoomLevel(PageZoom::ZOOM_IN);
+      automation_client_->SetZoomLevel(content::PAGE_ZOOM_IN);
     } else if (in_args->lVal == kZoomOut) {
-      automation_client_->SetZoomLevel(PageZoom::ZOOM_OUT);
+      automation_client_->SetZoomLevel(content::PAGE_ZOOM_OUT);
     } else {
       DLOG(WARNING) << "Unsupported zoom level:" << in_args->lVal;
     }
@@ -923,6 +912,43 @@ void ChromeActiveDocument::OnUnload(const GUID* cmd_group_guid,
                                     VARIANT* in_args,
                                     VARIANT* out_args) {
   if (IsValid() && out_args) {
+    // If the navigation is attempted to the url loaded in CF, with the only
+    // difference being the addition of an anchor, IE will attempt to unload
+    // the currently loaded document which basically would end up running
+    // unload handlers on the currently loaded document thus rendering it non
+    // functional. We handle this as below:-
+    // The BHO receives the new url in the BeforeNavigate event.
+    // We compare the non anchor portions of the url in the BHO with the loaded
+    // url and if they match, we initiate a Chrome navigation to the url with
+    // the anchor which works nicely.
+    // We don't want to continue processing the unload in this case.
+    // Note:-
+    // IE handles these navigations by querying the loaded document for
+    // IHTMLDocument which then handles the new navigation. That won't work for
+    // us as we don't implement IHTMLDocument.
+    NavigationManager* mgr = NavigationManager::GetThreadInstance();
+    DLOG_IF(ERROR, !mgr) << "Couldn't get instance of NavigationManager";
+    if (mgr) {
+      ChromeFrameUrl url;
+      url.Parse(mgr->url());
+      if (url.gurl().has_ref()) {
+        url_canon::Replacements<char> replacements;
+        replacements.ClearRef();
+
+        if (url.gurl().ReplaceComponents(replacements) ==
+            GURL(static_cast<BSTR>(url_))) {
+          // We want to reuse the existing automation client and channel for
+          // initiating the new navigation. Setting the
+          // is_automation_client_reused_ flag to true before calling the
+          // LaunchUrl function achieves that.
+          is_automation_client_reused_ = true;
+          LaunchUrl(url, mgr->referrer());
+          out_args->vt = VT_BOOL;
+          out_args->boolVal = VARIANT_FALSE;
+          return;
+        }
+      }
+    }
     bool should_unload = true;
     automation_client_->OnUnload(&should_unload);
     out_args->vt = VT_BOOL;
@@ -930,23 +956,6 @@ void ChromeActiveDocument::OnUnload(const GUID* cmd_group_guid,
   }
 }
 
-void ChromeActiveDocument::OnOpenURL(const GURL& url_to_open,
-                                     const GURL& referrer,
-                                     int open_disposition) {
-  // If the disposition indicates that we should be opening the URL in the
-  // current tab, then we can reuse the ChromeFrameAutomationClient instance
-  // maintained by the current ChromeActiveDocument instance. We cache this
-  // instance so that it can be used by the new ChromeActiveDocument instance
-  // which may be instantiated for handling the new URL.
-  if (open_disposition == CURRENT_TAB) {
-    // Grab a reference to ensure that the document remains valid.
-    AddRef();
-    g_active_doc_cache.Set(this);
-  }
-
-  BaseActiveX::OnOpenURL(url_to_open, referrer, open_disposition);
-}
-
 void ChromeActiveDocument::OnAttachExternalTab(
     const AttachExternalTabParams& params) {
   if (!automation_client_.get()) {
@@ -975,8 +984,8 @@ void ChromeActiveDocument::OnAttachExternalTab(
 }
 
 bool ChromeActiveDocument::PreProcessContextMenu(HMENU menu) {
-  ScopedComPtr<IBrowserService> browser_service;
-  ScopedComPtr<ITravelLog> travel_log;
+  base::win::ScopedComPtr<IBrowserService> browser_service;
+  base::win::ScopedComPtr<ITravelLog> travel_log;
   GetBrowserServiceAndTravelLog(browser_service.Receive(),
                                 travel_log.Receive());
   if (!browser_service || !travel_log)
@@ -996,7 +1005,7 @@ bool ChromeActiveDocument::PreProcessContextMenu(HMENU menu) {
 
 bool ChromeActiveDocument::HandleContextMenuCommand(
     UINT cmd, const MiniContextMenuParams& params) {
-  ScopedComPtr<IWebBrowser2> web_browser2;
+  base::win::ScopedComPtr<IWebBrowser2> web_browser2;
   DoQueryService(SID_SWebBrowserApp, m_spClientSite, web_browser2.Receive());
 
   if (cmd == IDC_BACK)
@@ -1016,9 +1025,9 @@ HRESULT ChromeActiveDocument::IEExec(const GUID* cmd_group_guid,
                                      VARIANT* in_args, VARIANT* out_args) {
   HRESULT hr = E_FAIL;
 
-  ScopedComPtr<IOleCommandTarget> frame_cmd_target;
+  base::win::ScopedComPtr<IOleCommandTarget> frame_cmd_target;
 
-  ScopedComPtr<IOleInPlaceSite> in_place_site(m_spInPlaceSite);
+  base::win::ScopedComPtr<IOleInPlaceSite> in_place_site(m_spInPlaceSite);
   if (!in_place_site.get() && m_spClientSite != NULL)
     in_place_site.QueryFrom(m_spClientSite);
 
@@ -1047,11 +1056,12 @@ bool ChromeActiveDocument::LaunchUrl(const ChromeFrameUrl& cf_url,
     }
   }
 
+  document_url_ = cf_url.gurl().spec();
+
   url_.Allocate(UTF8ToWide(cf_url.gurl().spec()).c_str());
   if (cf_url.attach_to_external_tab()) {
-    dimensions_ = cf_url.dimensions();
     automation_client_->AttachExternalTab(cf_url.cookie());
-    SetWindowDimensions();
+    OnMoveWindow(cf_url.dimensions());
   } else if (!automation_client_->InitiateNavigation(cf_url.gurl().spec(),
                                                      referrer,
                                                      this)) {
@@ -1072,13 +1082,12 @@ bool ChromeActiveDocument::LaunchUrl(const ChromeFrameUrl& cf_url,
     // If no profile was given, then make use of the host process's name.
     if (profile.empty())
       profile = GetHostProcessName(false);
-    return InitializeAutomation(profile, L"", IsIEInPrivate(),
+    return InitializeAutomation(profile, IsIEInPrivate(),
                                 false, cf_url.gurl(), GURL(referrer),
                                 false);
   }
 }
 
-
 HRESULT ChromeActiveDocument::OnRefreshPage(const GUID* cmd_group_guid,
     DWORD command_id, DWORD cmd_exec_opt, VARIANT* in_args, VARIANT* out_args) {
   DVLOG(1) << __FUNCTION__;
@@ -1136,10 +1145,10 @@ HRESULT ChromeActiveDocument::OnRefreshPage(const GUID* cmd_group_guid,
     ResetUrlRequestManager();
     url_fetcher_->set_frame_busting(false);
     // And now launch the current URL again.  This starts a new server process.
-    DCHECK(navigation_info_.url.is_valid());
+    DCHECK(navigation_info_->url.is_valid());
     ChromeFrameUrl cf_url;
-    cf_url.Parse(UTF8ToWide(navigation_info_.url.spec()));
-    LaunchUrl(cf_url, navigation_info_.referrer.spec());
+    cf_url.Parse(UTF8ToWide(navigation_info_->url.spec()));
+    LaunchUrl(cf_url, navigation_info_->referrer.spec());
   }
 
   return S_OK;
@@ -1246,8 +1255,8 @@ HRESULT ChromeActiveDocument::OnEncodingChange(const GUID* cmd_group_guid,
 void ChromeActiveDocument::OnGoToHistoryEntryOffset(int offset) {
   DVLOG(1) <<  __FUNCTION__ << " - offset:" << offset;
 
-  ScopedComPtr<IBrowserService> browser_service;
-  ScopedComPtr<ITravelLog> travel_log;
+  base::win::ScopedComPtr<IBrowserService> browser_service;
+  base::win::ScopedComPtr<ITravelLog> travel_log;
   GetBrowserServiceAndTravelLog(browser_service.Receive(),
                                 travel_log.Receive());
 
@@ -1255,10 +1264,36 @@ void ChromeActiveDocument::OnGoToHistoryEntryOffset(int offset) {
     travel_log->Travel(browser_service, offset);
 }
 
+void ChromeActiveDocument::OnMoveWindow(const gfx::Rect& dimensions) {
+  base::win::ScopedComPtr<IWebBrowser2> web_browser2;
+  DoQueryService(SID_SWebBrowserApp, m_spClientSite,
+                 web_browser2.Receive());
+  if (!web_browser2)
+    return;
+  DVLOG(1) << "this:" << this << "\ndimensions: width:" << dimensions.width()
+           << " height:" << dimensions.height();
+  if (!dimensions.IsEmpty()) {
+    web_browser2->put_MenuBar(VARIANT_FALSE);
+    web_browser2->put_ToolBar(VARIANT_FALSE);
+
+    int width = dimensions.width();
+    int height = dimensions.height();
+    // Compute the size of the browser window given the desired size of the
+    // content area. As per MSDN, the WebBrowser object returns an error from
+    // this method. As a result the code below is best effort.
+    web_browser2->ClientToWindow(&width, &height);
+
+    web_browser2->put_Width(width);
+    web_browser2->put_Height(height);
+    web_browser2->put_Left(dimensions.x());
+    web_browser2->put_Top(dimensions.y());
+  }
+}
+
 HRESULT ChromeActiveDocument::GetBrowserServiceAndTravelLog(
     IBrowserService** browser_service, ITravelLog** travel_log) {
   DCHECK(browser_service || travel_log);
-  ScopedComPtr<IBrowserService> browser_service_local;
+  base::win::ScopedComPtr<IBrowserService> browser_service_local;
   HRESULT hr = DoQueryService(SID_SShellBrowser, m_spClientSite,
                               browser_service_local.Receive());
   if (!browser_service_local) {
@@ -1280,7 +1315,7 @@ HRESULT ChromeActiveDocument::GetBrowserServiceAndTravelLog(
 LRESULT ChromeActiveDocument::OnForward(WORD notify_code, WORD id,
                                         HWND control_window,
                                         BOOL& bHandled) {
-  ScopedComPtr<IWebBrowser2> web_browser2;
+  base::win::ScopedComPtr<IWebBrowser2> web_browser2;
   DoQueryService(SID_SWebBrowserApp, m_spClientSite, web_browser2.Receive());
   DCHECK(web_browser2);
 
@@ -1292,7 +1327,7 @@ LRESULT ChromeActiveDocument::OnForward(WORD notify_code, WORD id,
 LRESULT ChromeActiveDocument::OnBack(WORD notify_code, WORD id,
                                      HWND control_window,
                                      BOOL& bHandled) {
-  ScopedComPtr<IWebBrowser2> web_browser2;
+  base::win::ScopedComPtr<IWebBrowser2> web_browser2;
   DoQueryService(SID_SWebBrowserApp, m_spClientSite, web_browser2.Receive());
   DCHECK(web_browser2);
 
@@ -1307,7 +1342,7 @@ LRESULT ChromeActiveDocument::OnFirePrivacyChange(UINT message, WPARAM wparam,
   if (!m_spClientSite)
     return 0;
 
-  ScopedComPtr<IWebBrowser2> web_browser2;
+  base::win::ScopedComPtr<IWebBrowser2> web_browser2;
   DoQueryService(SID_SWebBrowserApp, m_spClientSite,
                  web_browser2.Receive());
   if (!web_browser2) {
@@ -1315,11 +1350,11 @@ LRESULT ChromeActiveDocument::OnFirePrivacyChange(UINT message, WPARAM wparam,
     return 0;
   }
 
-  ScopedComPtr<IShellBrowser> shell_browser;
+  base::win::ScopedComPtr<IShellBrowser> shell_browser;
   DoQueryService(SID_STopLevelBrowser, web_browser2,
                  shell_browser.Receive());
   DCHECK(shell_browser.get() != NULL);
-  ScopedComPtr<ITridentService2> trident_services;
+  base::win::ScopedComPtr<ITridentService2> trident_services;
   trident_services.QueryFrom(shell_browser);
   if (trident_services)
     trident_services->FirePrivacyImpactedStateChange(wparam);
@@ -1344,66 +1379,35 @@ LRESULT ChromeActiveDocument::OnSetFocus(UINT message, WPARAM wparam,
   return 0;
 }
 
-void ChromeActiveDocument::SetWindowDimensions() {
-  ScopedComPtr<IWebBrowser2> web_browser2;
-  DoQueryService(SID_SWebBrowserApp, m_spClientSite,
-                 web_browser2.Receive());
-  if (!web_browser2)
-    return;
-  DVLOG(1) << "this:" << this << "\ndimensions: width:" << dimensions_.width()
-           << " height:" << dimensions_.height();
-  if (!dimensions_.IsEmpty()) {
-    web_browser2->put_MenuBar(VARIANT_FALSE);
-    web_browser2->put_ToolBar(VARIANT_FALSE);
-
-    int width = dimensions_.width();
-    int height = dimensions_.height();
-    // Compute the size of the browser window given the desired size of the
-    // content area. As per MSDN, the WebBrowser object returns an error from
-    // this method. As a result the code below is best effort.
-    if (SUCCEEDED(web_browser2->ClientToWindow(&width, &height))) {
-      dimensions_.set_width(width);
-      dimensions_.set_height(height);
-    }
-    web_browser2->put_Width(dimensions_.width());
-    web_browser2->put_Height(dimensions_.height());
-    web_browser2->put_Left(dimensions_.x());
-    web_browser2->put_Top(dimensions_.y());
-
-    dimensions_.set_height(0);
-    dimensions_.set_width(0);
-  }
-}
-
 bool ChromeActiveDocument::IsNewNavigation(
     const NavigationInfo& new_navigation_info, int flags) const {
   // A new navigation is typically an internal navigation which is initiated by
   // the renderer(WebKit). Condition 1 below has to be true along with the
   // any of the other conditions below.
   // 1. The navigation notification flags passed in as the flags parameter
-  //    is not INVALIDATE_LOAD which indicates that the loading state of the
-  //    tab changed.
+  //    is not INVALIDATE_TYPE_LOAD which indicates that the loading state of
+  //    the tab changed.
   // 2. The navigation index is greater than 0 which means that a top level
   //    navigation was initiated on the current external tab.
   // 3. The navigation type has changed.
   // 4. The url or the referrer are different.
-  if (flags == TabContents::INVALIDATE_LOAD)
+  if (flags == content::INVALIDATE_TYPE_LOAD)
     return false;
 
   if (new_navigation_info.navigation_index <= 0)
     return false;
 
   if (new_navigation_info.navigation_index ==
-      navigation_info_.navigation_index)
+      navigation_info_->navigation_index)
     return false;
 
-  if (new_navigation_info.navigation_type != navigation_info_.navigation_type)
+  if (new_navigation_info.navigation_type != navigation_info_->navigation_type)
     return true;
 
-  if (new_navigation_info.url != navigation_info_.url)
+  if (new_navigation_info.url != navigation_info_->url)
     return true;
 
-  if (new_navigation_info.referrer != navigation_info_.referrer)
+  if (new_navigation_info.referrer != navigation_info_->referrer)
     return true;
 
   return false;
