@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,12 +14,10 @@
 #include <vector>
 
 #include "chrome_frame/chrome_frame_activex_base.h"
+#include "chrome_frame/chrome_tab.h"
 #include "chrome_frame/com_type_info_holder.h"
 #include "grit/chrome_frame_resources.h"
 
-// Include without path to make GYP build see it.
-#include "chrome_tab.h"  // NOLINT
-
 #define WM_HOST_MOVED_NOTIFICATION (WM_APP + 1)
 
 // ChromeFrameActivex: Implementation of the ActiveX control that is
@@ -97,11 +95,6 @@ END_MSG_MAP()
   virtual void OnLoadFailed(int error_code, const std::string& url);
   virtual void OnAutomationServerLaunchFailed(
       AutomationLaunchResult reason, const std::string& server_version);
-  virtual void OnExtensionInstalled(const FilePath& path,
-      void* user_data, AutomationMsg_ExtensionResponseValues response);
-  virtual void OnGetEnabledExtensionsComplete(
-      void* user_data,
-      const std::vector<FilePath>& extension_directories);
   virtual void OnChannelError();
 
   // Separated to static function for unit testing this logic more easily.
