@@ -8,6 +8,7 @@
 #include <atlbase.h>
 #include <atlcom.h>
 #include <atlctl.h>
+#include <exdisp.h>
 #include <wininet.h>
 #include <shdeprecated.h>  // for IBrowserService2
 #include <shlguid.h>
@@ -24,9 +25,10 @@
 #include "base/win/scoped_comptr.h"
 #include "base/win/scoped_variant.h"
 #include "grit/chrome_frame_resources.h"
-#include "ceee/ie/common/ceee_util.h"
+#include "chrome/app/chrome_command_ids.h"
 #include "chrome/common/url_constants.h"
 #include "chrome_frame/chrome_frame_plugin.h"
+#include "chrome_frame/chrome_tab.h"
 #include "chrome_frame/com_message_event.h"
 #include "chrome_frame/com_type_info_holder.h"
 #include "chrome_frame/simple_resource_loader.h"
@@ -36,9 +38,6 @@
 #include "grit/generated_resources.h"
 #include "net/base/cookie_monster.h"
 
-// Include without path to make GYP build see it.
-#include "chrome_tab.h"  // NOLINT
-
 // Connection point class to support firing IChromeFrameEvents (dispinterface).
 template<class T>
 class ATL_NO_VTABLE ProxyDIChromeFrameEvents
@@ -118,26 +117,6 @@ class ATL_NO_VTABLE ProxyDIChromeFrameEvents
                          arraysize(args));
   }
 
-  void Fire_onextensionready(BSTR path, long response) {  // NOLINT
-    // Arguments in reverse order to the function declaration, because
-    // that's what DISPPARAMS requires.
-    VARIANT args[2] = { { VT_I4, }, { VT_BSTR, } };
-    args[0].lVal = response;
-    args[1].bstrVal = path;
-
-    FireMethodWithParams(CF_EVENT_DISPID_ONEXTENSIONREADY,
-                         args,
-                         arraysize(args));
-  }
-
-  void Fire_ongetenabledextensionscomplete(SAFEARRAY* extension_dirs) {
-    VARIANT args[1] = { { VT_ARRAY | VT_BSTR } };
-    args[0].parray = extension_dirs;
-
-    FireMethodWithParams(CF_EVENT_DISPID_ONGETENABLEDEXTENSIONSCOMPLETE,
-                         args, arraysize(args));
-  }
-
   void Fire_onchannelerror() {  // NOLINT
     FireMethodWithParams(CF_EVENT_DISPID_ONCHANNELERROR, NULL, 0);
   }
@@ -149,6 +128,18 @@ class ATL_NO_VTABLE ProxyDIChromeFrameEvents
 
 extern bool g_first_launch_by_process_;
 
+namespace chrome_frame {
+// Implemented outside this file so that the header doesn't include
+// automation_messages.h.
+std::string ActiveXCreateUrl(const GURL& parsed_url,
+                             const AttachExternalTabParams& params);
+int GetDisposition(const AttachExternalTabParams& params);
+void GetMiniContextMenuData(UINT cmd,
+                            const MiniContextMenuParams& params,
+                            GURL* referrer,
+                            GURL* url);
+}  // namespace chrome_frame
+
 // Common implementation for ActiveX and Active Document
 template <class T, const CLSID& class_id>
 class ATL_NO_VTABLE ChromeFrameActivexBase :  // NOLINT
@@ -163,7 +154,6 @@ class ATL_NO_VTABLE ChromeFrameActivexBase :  // NOLINT
   public com_util::IProvideClassInfo2Impl<class_id,
                                           DIID_DIChromeFrameEvents>,
   public com_util::IDispatchImpl<IChromeFrame>,
-  public IChromeFrameInternal,
   public IConnectionPointContainerImpl<T>,
   public ProxyDIChromeFrameEvents<T>,
   public IPropertyNotifySinkCP<T>,
@@ -197,7 +187,6 @@ DECLARE_NOT_AGGREGATABLE(T)
 BEGIN_COM_MAP(ChromeFrameActivexBase)
   COM_INTERFACE_ENTRY(IChromeFrame)
   COM_INTERFACE_ENTRY(IDispatch)
-  COM_INTERFACE_ENTRY(IChromeFrameInternal)
   COM_INTERFACE_ENTRY(IViewObjectEx)
   COM_INTERFACE_ENTRY(IViewObject2)
   COM_INTERFACE_ENTRY(IViewObject)
@@ -263,11 +252,11 @@ END_MSG_MAP()
     // Used to perform one time tasks.
     if (g_first_launch_by_process_) {
       g_first_launch_by_process_ = false;
-      THREAD_SAFE_UMA_HISTOGRAM_CUSTOM_COUNTS("ChromeFrame.IEVersion",
-                                              GetIEVersion(),
-                                              IE_INVALID,
-                                              IE_9,
-                                              IE_9 + 1);
+      UMA_HISTOGRAM_CUSTOM_COUNTS("ChromeFrame.IEVersion",
+                                  GetIEVersion(),
+                                  IE_INVALID,
+                                  IE_10,
+                                  IE_10 + 1);
     }
 
     return S_OK;
@@ -358,13 +347,16 @@ END_MSG_MAP()
         case IDS_CONTENT_CONTEXT_SAVEVIDEOAS:
         case IDS_CONTENT_CONTEXT_SAVEIMAGEAS:
         case IDS_CONTENT_CONTEXT_SAVELINKAS: {
-          const GURL& referrer = params.frame_url.is_empty() ?
-              params.page_url : params.frame_url;
-          const GURL& url = (cmd == IDS_CONTENT_CONTEXT_SAVELINKAS ?
-              params.link_url : params.src_url);
+          GURL referrer, url;
+          chrome_frame::GetMiniContextMenuData(cmd, params, &referrer, &url);
           DoFileDownloadInIE(UTF8ToWide(url.spec()).c_str());
           return true;
         }
+
+        case IDC_PRINT: {
+          automation_client_->PrintTab();
+          return true;
+        }
       }
     }
 
@@ -391,6 +383,16 @@ END_MSG_MAP()
     return true;
   }
 
+  static void BringWebBrowserWindowToTop(IWebBrowser2* web_browser2) {
+    DCHECK(web_browser2);
+    if (web_browser2) {
+      web_browser2->put_Visible(VARIANT_TRUE);
+      HWND ie_window = NULL;
+      web_browser2->get_HWND(reinterpret_cast<long*>(&ie_window));
+      ::BringWindowToTop(ie_window);
+    }
+  }
+
  protected:
   virtual void GetProfilePath(const std::wstring& profile_name,
                               FilePath* profile_path) {
@@ -398,12 +400,7 @@ END_MSG_MAP()
                  (lstrcmpi(profile_name.c_str(), kRundllProfileName) == 0);
     // Browsers without IDeleteBrowsingHistory in non-priv mode
     // have their profiles moved into "Temporary Internet Files".
-    //
-    // If CEEE is registered, we must have a persistent profile. We
-    // considered checking if e.g. ceee_ie.dll is loaded in the process
-    // but this gets into edge cases when the user enables the CEEE add-on
-    // after CF is first loaded.
-    if (is_IE && GetIEVersion() < IE_8 && !ceee_util::IsIeCeeeRegistered()) {
+    if (is_IE && GetIEVersion() < IE_8) {
       *profile_path = GetIETemporaryFilesFolder();
       *profile_path = profile_path->Append(L"Google Chrome Frame");
     } else {
@@ -460,6 +457,18 @@ END_MSG_MAP()
   // There's room for improvement here and also see todo below.
   LPARAM OnDownloadRequestInHost(UINT message, WPARAM wparam, LPARAM lparam,
                                  BOOL& handled) {
+    ChromeFrameUrl cf_url;
+    cf_url.Parse(UTF8ToWide(GetDocumentUrl()));
+
+    // Always issue the download request in a new window to ensure that the
+    // currently loaded ChromeFrame document does not inadvartently see an
+    // unload request. This runs javascript unload handlers on the page which
+    // renders the page non functional.
+    VARIANT flags = { VT_I4 };
+    V_I4(&flags) = navNoHistory;
+    if (!cf_url.attach_to_external_tab())
+      V_I4(&flags) |= navOpenInNewWindow;
+
     DownloadInHostParams* download_params =
         reinterpret_cast<DownloadInHostParams*>(wparam);
     DCHECK(download_params);
@@ -471,51 +480,25 @@ END_MSG_MAP()
       NavigateBrowserToMoniker(
           doc_site_, download_params->moniker,
           UTF8ToWide(download_params->request_headers).c_str(),
-          download_params->bind_ctx, NULL, download_params->post_data);
+          download_params->bind_ctx, NULL, download_params->post_data,
+          &flags);
     }
     delete download_params;
     return TRUE;
   }
 
   virtual void OnAttachExternalTab(const AttachExternalTabParams& params) {
-    std::wstring wide_url = url_;
-    GURL parsed_url(WideToUTF8(wide_url));
-
-    // If Chrome-Frame is presently navigated to an extension page, navigating
-    // the host to a url with scheme chrome-extension will fail, so we
-    // point the host at http:local_host.  Note that this is NOT the URL
-    // to which the host is directed.  It is only used as a temporary message
-    // passing mechanism between this CF instance, and the BHO that will
-    // be constructed in the new IE tab.
-    if (parsed_url.SchemeIs("chrome-extension") &&
-        is_privileged()) {
-      const char kScheme[] = "http";
-      const char kHost[] = "local_host";
-
-      GURL::Replacements r;
-      r.SetScheme(kScheme, url_parse::Component(0, sizeof(kScheme) -1));
-      r.SetHost(kHost, url_parse::Component(0, sizeof(kHost) - 1));
-      parsed_url = parsed_url.ReplaceComponents(r);
-    }
-
-    std::string url = base::StringPrintf(
-        "%hs?attach_external_tab&%I64u&%d&%d&%d&%d&%d&%hs",
-        parsed_url.GetOrigin().spec().c_str(),
-        params.cookie,
-        params.disposition,
-        params.dimensions.x(),
-        params.dimensions.y(),
-        params.dimensions.width(),
-        params.dimensions.height(),
-        params.profile_name.c_str());
-    HostNavigate(GURL(url), GURL(), params.disposition);
+    GURL current_url(static_cast<BSTR>(url_));
+    std::string url = chrome_frame::ActiveXCreateUrl(current_url, params);
+    // Pass the current document url as the referrer for the new navigation.
+    HostNavigate(GURL(url), current_url, chrome_frame::GetDisposition(params));
   }
 
-  virtual void OnHandleContextMenu(HANDLE menu_handle,
+  virtual void OnHandleContextMenu(const ContextMenuModel& menu_model,
                                    int align_flags,
                                    const MiniContextMenuParams& params) {
     scoped_refptr<BasePlugin> ref(this);
-    ChromeFramePlugin<T>::OnHandleContextMenu(menu_handle, align_flags, params);
+    ChromeFramePlugin<T>::OnHandleContextMenu(menu_model, align_flags, params);
   }
 
   LRESULT OnCreate(UINT message, WPARAM wparam, LPARAM lparam,
@@ -786,7 +769,7 @@ END_MSG_MAP()
       return hr;
 
     DCHECK(handlers != NULL);
-    std::remove(handlers->begin(), handlers->end(), listener);
+    handlers->erase(base::win::ScopedComPtr<IDispatch>(listener));
 
     return hr;
   }
@@ -831,68 +814,18 @@ END_MSG_MAP()
   }
 
   STDMETHOD(installExtension)(BSTR crx_path) {
-    DCHECK(automation_client_.get());
-
-    if (NULL == crx_path) {
-      NOTREACHED();
-      return E_INVALIDARG;
-    }
-
-    if (!is_privileged()) {
-      DLOG(ERROR) << "Attempt to installExtension in non-privileged mode";
-      return E_ACCESSDENIED;
-    }
-
-    FilePath::StringType crx_path_str(crx_path);
-    FilePath crx_file_path(crx_path_str);
-
-    automation_client_->InstallExtension(crx_file_path, NULL);
-    return S_OK;
+    NOTREACHED();  // Deprecated.
+    return E_NOTIMPL;
   }
 
   STDMETHOD(loadExtension)(BSTR path) {
-    DCHECK(automation_client_.get());
-
-    if (NULL == path) {
-      NOTREACHED();
-      return E_INVALIDARG;
-    }
-
-    if (!is_privileged()) {
-      DLOG(ERROR) << "Attempt to loadExtension in non-privileged mode";
-      return E_ACCESSDENIED;
-    }
-
-    FilePath::StringType path_str(path);
-    FilePath file_path(path_str);
-
-    automation_client_->LoadExpandedExtension(file_path, NULL);
-    return S_OK;
+    NOTREACHED();  // Deprecated.
+    return E_NOTIMPL;
   }
 
   STDMETHOD(getEnabledExtensions)() {
-    DCHECK(automation_client_.get());
-
-    if (!is_privileged()) {
-      DLOG(ERROR) << "Attempt to getEnabledExtensions in non-privileged mode";
-      return E_ACCESSDENIED;
-    }
-
-    automation_client_->GetEnabledExtensions(NULL);
-    return S_OK;
-  }
-
-  STDMETHOD(getSessionId)(int* session_id) {
-    DCHECK(automation_client_.get());
-    DCHECK(session_id);
-
-    if (!is_privileged()) {
-      DLOG(ERROR) << "Attempt to getSessionId in non-privileged mode";
-      return E_ACCESSDENIED;
-    }
-
-    *session_id = automation_client_->GetSessionId();
-    return (*session_id) == -1 ? S_FALSE : S_OK;
+    NOTREACHED();  // Deprecated.
+    return E_NOTIMPL;
   }
 
   STDMETHOD(registerBhoIfNeeded)() {
@@ -1035,7 +968,8 @@ END_MSG_MAP()
       RECT dummy_pos_rect = {0};
       RECT dummy_clip_rect = {0};
       OLEINPLACEFRAMEINFO dummy_frame_info = {0};
-      if (FAILED(m_spInPlaceSite->GetWindowContext(in_place_frame_.Receive(),
+      if (!m_spInPlaceSite ||
+          FAILED(m_spInPlaceSite->GetWindowContext(in_place_frame_.Receive(),
                                                    dummy_ui_window.Receive(),
                                                    &dummy_pos_rect,
                                                    &dummy_clip_rect,
@@ -1121,7 +1055,6 @@ END_MSG_MAP()
         return;
       }
     }
-
     // Last chance to handle the keystroke is to pass it to chromium.
     // We do this last partially because there's no way for us to tell if
     // chromium actually handled the keystroke, but also since the browser
@@ -1222,6 +1155,21 @@ END_MSG_MAP()
       http_headers.Set(referrer_header.c_str());
     }
 
+    // IE6 does not support tabs. If Chrome sent us a window open request
+    // indicating that the navigation needs to occur in a foreground tab or
+    // a popup window, then we need to ensure that the new window in IE6 is
+    // brought to the foreground.
+    if (ie_version == IE_6) {
+      ChromeFrameUrl cf_url;
+      cf_url.Parse(static_cast<BSTR>(url_));
+
+      if (cf_url.attach_to_external_tab() &&
+          (cf_url.disposition() == NEW_FOREGROUND_TAB ||
+           cf_url.disposition() == NEW_POPUP)) {
+        BringWebBrowserWindowToTop(web_browser2);
+      }
+    }
+
     HRESULT hr = web_browser2->Navigate2(url.AsInput(), &flags, &empty, &empty,
                                          http_headers.AsInput());
     // If the current window is a popup window then attempting to open a new
@@ -1232,7 +1180,6 @@ END_MSG_MAP()
       V_I4(&flags) = navOpenInNewWindow;
       hr = web_browser2->Navigate2(url.AsInput(), &flags, &empty, &empty,
                                    http_headers.AsInput());
-
       DLOG_IF(ERROR, FAILED(hr))
           << "Navigate2 failed with error: "
           << base::StringPrintf("0x%08X", hr);
