@@ -7,24 +7,22 @@
 
 #include <atlbase.h>
 #include <atlwin.h>
-#include <string>
 #include <map>
+#include <string>
 #include <vector>
 
-#include "base/ref_counted.h"
-#include "base/scoped_handle.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_handle.h"
 #include "base/stack_container.h"
 #include "base/synchronization/lock.h"
-#include "base/task.h"
 #include "base/threading/thread.h"
 #include "base/timer.h"
-#include "chrome/common/page_zoom.h"
 #include "chrome/test/automation/automation_proxy.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome_frame/chrome_frame_delegate.h"
-#include "chrome_frame/chrome_frame_histograms.h"
 #include "chrome_frame/plugin_url_request.h"
 #include "chrome_frame/sync_msg_reply_dispatcher.h"
+#include "content/public/common/page_zoom.h"
 
 // By a convoluated route, this timeout also winds up being the sync automation
 // message timeout. See the ChromeFrameAutomationProxyImpl ctor and the
@@ -112,14 +110,13 @@ class ChromeFrameLaunchParams :  // NOLINT
                           const FilePath& profile_path,
                           const std::wstring& profile_name,
                           const std::wstring& language,
-                          const std::wstring& extra_arguments,
                           bool incognito, bool widget_mode,
                           bool route_all_top_level_navigations)
     : launch_timeout_(kCommandExecutionTimeout), url_(url),
       referrer_(referrer), profile_path_(profile_path),
       profile_name_(profile_name), language_(language),
-      extra_arguments_(extra_arguments), version_check_(true),
-      incognito_mode_(incognito), is_widget_mode_(widget_mode),
+      version_check_(true), incognito_mode_(incognito),
+      is_widget_mode_(widget_mode),
       route_all_top_level_navigations_(route_all_top_level_navigations) {
   }
 
@@ -162,10 +159,6 @@ class ChromeFrameLaunchParams :  // NOLINT
     return language_;
   }
 
-  const std::wstring& extra_arguments() const {
-    return extra_arguments_;
-  }
-
   bool version_check() const {
     return version_check_;
   }
@@ -198,7 +191,6 @@ class ChromeFrameLaunchParams :  // NOLINT
   FilePath profile_path_;
   std::wstring profile_name_;
   std::wstring language_;
-  std::wstring extra_arguments_;
   bool version_check_;
   bool incognito_mode_;
   bool is_widget_mode_;
@@ -209,7 +201,8 @@ class ChromeFrameLaunchParams :  // NOLINT
 };
 
 // Callback when chrome process launch is complete and automation handshake
-// (Hello message) is established.
+// (Hello message) is established.  All methods are invoked on the automation
+// proxy's worker thread.
 struct DECLSPEC_NOVTABLE LaunchDelegate {  // NOLINT
   virtual void LaunchComplete(ChromeFrameAutomationProxy* proxy,
                               AutomationLaunchResult result) = 0;
@@ -231,9 +224,6 @@ class AutomationProxyCacheEntry
   void RemoveDelegate(LaunchDelegate* delegate, base::WaitableEvent* done,
                       bool* was_last_delegate);
 
-  void StartSendUmaInterval(ChromeFrameHistogramSnapshots* snapshots,
-                            int send_interval);
-
   DWORD WaitForThread(DWORD timeout) {  // NOLINT
     DCHECK(thread_.get());
     return ::WaitForSingleObject(thread_->thread_handle(), timeout);
@@ -266,7 +256,6 @@ class AutomationProxyCacheEntry
  protected:
   void CreateProxy(ChromeFrameLaunchParams* params,
                    LaunchDelegate* delegate);
-  void SendUMAData();
 
  protected:
   std::wstring profile_name;
@@ -278,8 +267,6 @@ class AutomationProxyCacheEntry
   // Used for UMA histogram logging to measure the time for the chrome
   // automation server to start;
   base::TimeTicks automation_server_launch_start_time_;
-  ChromeFrameHistogramSnapshots* snapshots_;
-  int uma_send_interval_;
 };
 
 // We must create and destroy automation proxy in a thread with a message loop.
@@ -304,17 +291,10 @@ class ProxyFactory {
   Vector proxies_;
   // Lock if we are going to call GetAutomationServer from more than one thread.
   base::Lock lock_;
-
-  // Gathers histograms to be sent to Chrome.
-  ChromeFrameHistogramSnapshots chrome_frame_histograms_;
-
-  // Interval for sending UMA data
-  int uma_send_interval_;
 };
 
 // Handles all automation requests initiated from the chrome frame objects.
-// These include the chrome tab/chrome frame activex/chrome frame npapi
-// plugin objects.
+// These include the chrome tab/chrome frame activex plugin objects.
 class ChromeFrameAutomationClient
     : public CWindowImpl<ChromeFrameAutomationClient>,
       public TaskMarshallerThroughWindowsMessages<ChromeFrameAutomationClient>,
@@ -343,34 +323,11 @@ class ChromeFrameAutomationClient
                                       const std::string& target);
   bool SetProxySettings(const std::string& json_encoded_proxy_settings);
 
-  virtual void SetEnableExtensionAutomation(
-      const std::vector<std::string>& functions_enabled);
-
   void FindInPage(const std::wstring& search_string,
                   FindInPageDirection forward,
                   FindInPageCase match_case,
                   bool find_next);
 
-  virtual void InstallExtension(const FilePath& crx_path, void* user_data);
-
-  virtual void LoadExpandedExtension(const FilePath& path, void* user_data);
-
-  // Starts a request to get the list of enabled extensions' base directories.
-  // Response comes back as ChromeFrameDelegate::OnEnabledExtensions().
-  virtual void GetEnabledExtensions(void* user_data);
-
-  virtual void InstallExtensionComplete(
-      const FilePath& path,
-      void* user_data,
-      AutomationMsg_ExtensionResponseValues res);
-
-  virtual void GetEnabledExtensionsComplete(
-      void* user_data,
-      std::vector<FilePath>* extension_directories);
-
-  // Returns the session ID used to identify a Tab in Chrome.
-  virtual int GetSessionId() const;
-
   virtual void OnChromeFrameHostMoved();
 
   TabProxy* tab() const { return tab_.get(); }
@@ -426,11 +383,6 @@ class ChromeFrameAutomationClient
   // Url request manager set up.
   void SetUrlFetcher(PluginUrlRequestManager* url_fetcher);
 
-  // Called if the same instance of the ChromeFrameAutomationClient object
-  // is reused.
-  bool Reinitialize(ChromeFrameDelegate* chrome_frame_delegate,
-                    PluginUrlRequestManager* url_fetcher);
-
   // Attaches an existing external tab to this automation client instance.
   void AttachExternalTab(uint64 external_tab_cookie);
   void BlockExternalTab(uint64 cookie);
@@ -441,7 +393,7 @@ class ChromeFrameAutomationClient
   void RemoveBrowsingData(int remove_mask);
 
   // Sets the current zoom level on the tab.
-  void SetZoomLevel(PageZoom::Function zoom_level);
+  void SetZoomLevel(content::PageZoom zoom_level);
 
   // Fires before unload and unload handlers on the page if any. Allows the
   // the website to put up a confirmation dialog on unload.
@@ -498,7 +450,8 @@ class ChromeFrameAutomationClient
   // as parameter and forwards to Chrome via IPC.
   virtual void OnResponseStarted(int request_id, const char* mime_type,
       const char* headers, int size, base::Time last_modified,
-      const std::string& redirect_url, int redirect_status);
+      const std::string& redirect_url, int redirect_status,
+      const net::HostPortPair& socket_address);
   virtual void OnReadComplete(int request_id, const std::string& data);
   virtual void OnResponseEnd(int request_id,
                              const net::URLRequestStatus& status);
@@ -550,7 +503,7 @@ class ChromeFrameAutomationClient
   scoped_refptr<ChromeFrameLaunchParams> chrome_launch_params_;
 
   // Cache security manager for URL zone checking
-  ScopedComPtr<IInternetSecurityManager> security_manager_;
+  base::win::ScopedComPtr<IInternetSecurityManager> security_manager_;
 
   // When host network stack is used, this object is in charge of
   // handling network requests.
