@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,7 @@
 #include <string>
 #include <vector>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/win/win_util.h"
 #include "chrome_frame/chrome_frame_automation.h"
 #include "chrome/common/chrome_paths.h"
@@ -20,8 +20,14 @@
 
 #define IDC_ABOUT_CHROME_FRAME 40018
 
+// Helper so that this file doesn't include the messages header.
+void ChromeFramePluginGetParamsCoordinates(
+    const MiniContextMenuParams& params,
+    int* x,
+    int* y);
+
 // A class to implement common functionality for all types of
-// plugins: NPAPI. ActiveX and ActiveDoc
+// plugins: ActiveX and ActiveDoc
 template <typename T>
 class ChromeFramePlugin
     : public ChromeFrameDelegateImpl,
@@ -60,7 +66,6 @@ END_MSG_MAP()
   }
 
   bool InitializeAutomation(const std::wstring& profile_name,
-                            const std::wstring& extra_chrome_arguments,
                             bool incognito, bool is_widget_mode,
                             const GURL& url, const GURL& referrer,
                             bool route_all_top_level_navigations) {
@@ -77,8 +82,7 @@ END_MSG_MAP()
     FilePath actual_profile_name = profile_path.BaseName();
     launch_params_ = new ChromeFrameLaunchParams(url, referrer, profile_path,
         actual_profile_name.value(), SimpleResourceLoader::GetLanguage(),
-        extra_chrome_arguments, incognito_mode, is_widget_mode,
-        route_all_top_level_navigations);
+        incognito_mode, is_widget_mode, route_all_top_level_navigations);
     return automation_client_->Initialize(this, launch_params_);
   }
 
@@ -98,10 +102,6 @@ END_MSG_MAP()
     return document_url_;
   }
   virtual void OnAutomationServerReady() {
-    // Issue the extension automation request if we're privileged to
-    // allow this control to handle extension requests from Chrome.
-    if (is_privileged() && IsValid())
-      automation_client_->SetEnableExtensionAutomation(functions_enabled_);
   }
 
   virtual bool IsValid() const {
@@ -118,31 +118,29 @@ END_MSG_MAP()
     OnLoadFailed(error_code, gurl.spec());
   }
 
-  virtual void OnHandleContextMenu(HANDLE menu_handle,
+  virtual void OnHandleContextMenu(const ContextMenuModel& menu_model,
                                    int align_flags,
                                    const MiniContextMenuParams& params) {
-    if (!menu_handle || !automation_client_.get()) {
+    if (!automation_client_.get()) {
       NOTREACHED();
       return;
     }
 
-    // TrackPopupMenuEx call will fail on IE on Vista running
-    // in low integrity mode. We DO seem to be able to enumerate the menu
-    // though, so just clone it and show the copy:
-    HMENU copy = UtilCloneContextMenu(static_cast<HMENU>(menu_handle));
-    if (!copy)
+    HMENU menu = BuildContextMenu(menu_model);
+    if (!menu)
       return;
 
     T* self = static_cast<T*>(this);
-    if (self->PreProcessContextMenu(copy)) {
+    if (self->PreProcessContextMenu(menu)) {
       // In order for the context menu to handle keyboard input, give the
       // ActiveX window focus.
       ignore_setfocus_ = true;
       SetFocus(GetWindow());
       ignore_setfocus_ = false;
       UINT flags = align_flags | TPM_LEFTBUTTON | TPM_RETURNCMD | TPM_RECURSE;
-      UINT selected = TrackPopupMenuEx(copy, flags, params.screen_x,
-                                       params.screen_y, GetWindow(), NULL);
+      int x, y;
+      ChromeFramePluginGetParamsCoordinates(params, &x, &y);
+      UINT selected = TrackPopupMenuEx(menu, flags, x, y, GetWindow(), NULL);
       // Menu is over now give focus back to chrome
       GiveFocusToChrome(false);
       if (IsValid() && selected != 0 &&
@@ -151,7 +149,7 @@ END_MSG_MAP()
       }
     }
 
-    DestroyMenu(copy);
+    DestroyMenu(menu);
   }
 
   LRESULT OnSetFocus(UINT message, WPARAM wparam, LPARAM lparam,
@@ -260,12 +258,6 @@ END_MSG_MAP()
   // and notifying the host browser that we're doing so.
   // When the flag is not set, we transfer the focus to chrome.
   bool ignore_setfocus_;
-
-  // List of functions to enable for automation, or a single entry "*" to
-  // enable all functions for automation.  Ignored unless is_privileged_ is
-  // true.  Defaults to the empty list, meaning automation will not be
-  // turned on.
-  std::vector<std::string> functions_enabled_;
 };
 
 #endif  // CHROME_FRAME_CHROME_FRAME_PLUGIN_H_
