@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -21,16 +21,18 @@ const wchar_t* kAllowedSwitches[] = {
   L"automation-channel",
   L"chrome-frame",
   L"chrome-version",
+  L"disable-background-mode",
+  L"disable-popup-blocking",
+  L"disable-print-preview",
   L"disable-renderer-accessibility",
   L"enable-experimental-extension-apis",
   L"force-renderer-accessibility",
+  L"full-memory-crash-report",
   L"lang",
   L"no-default-browser-check",
-  L"noerrdialogs",
   L"no-first-run",
+  L"noerrdialogs",
   L"user-data-dir",
-  L"disable-popup-blocking",
-  L"full-memory-crash-report",
 };
 
 const wchar_t kWhitespaceChars[] = {
@@ -143,9 +145,15 @@ bool SanitizeAndLaunchChrome(const wchar_t* command_line) {
     std::wstring chrome_path;
     if (GetChromeExecutablePath(&chrome_path)) {
       const wchar_t* args = PathGetArgs(command_line);
+
+      // Build the command line string with the quoted path to chrome.exe.
+      std::wstring command_line;
+      command_line.reserve(chrome_path.size() + 2);
+      command_line.append(1, L'\"').append(chrome_path).append(1, L'\"');
+
       if (args != NULL) {
-        chrome_path += L" ";
-        chrome_path += args;
+        command_line += L' ';
+        command_line += args;
       }
 
       STARTUPINFO startup_info = {0};
@@ -153,7 +161,7 @@ bool SanitizeAndLaunchChrome(const wchar_t* command_line) {
       startup_info.dwFlags = STARTF_USESHOWWINDOW;
       startup_info.wShowWindow = SW_SHOW;
       PROCESS_INFORMATION process_info = {0};
-      if (CreateProcess(NULL, &chrome_path[0],
+      if (CreateProcess(&chrome_path[0], &command_line[0],
                         NULL, NULL, FALSE, 0, NULL, NULL,
                         &startup_info, &process_info)) {
         // Close handles.
