@@ -2,21 +2,19 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome_frame/external_tab.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/lazy_instance.h"
-#include "base/tracked.h"
-#include "base/task.h"
+#include "base/location.h"
+#include "chrome_frame/external_tab.h"
 #include "base/synchronization/waitable_event.h"
 #include "chrome/common/automation_messages.h"
 #include "chrome_frame/chrome_frame_delegate.h"
 #include "chrome_frame/utils.h"
 
-DISABLE_RUNNABLE_METHOD_REFCOUNT(ExternalTabProxy);
-DISABLE_RUNNABLE_METHOD_REFCOUNT(UIDelegate);
-
 namespace {
-  static base::LazyInstance<ChromeProxyFactory> g_proxy_factory(
-      base::LINKER_INITIALIZED);
+  static base::LazyInstance<ChromeProxyFactory> g_proxy_factory =
+      LAZY_INSTANCE_INITIALIZER;
 
   struct UserDataHolder : public SyncMessageContext {
     explicit UserDataHolder(void* p) : data(p) {}
@@ -46,6 +44,7 @@ bool ExternalTabProxy::OnMessageReceived(const IPC::Message& message) {
     IPC_MESSAGE_HANDLER(AutomationMsg_NavigationFailed, OnNavigationFailed)
     IPC_MESSAGE_HANDLER(AutomationMsg_DidNavigate, OnDidNavigate)
     IPC_MESSAGE_HANDLER(AutomationMsg_TabLoaded, OnTabLoaded)
+    IPC_MESSAGE_HANDLER(AutomationMsg_MoveWindow, OnMoveWindow)
     IPC_MESSAGE_HANDLER(AutomationMsg_ForwardMessageToExternalHost,
                         OnMessageToHost)
     IPC_MESSAGE_HANDLER(AutomationMsg_ForwardContextMenuToExternalHost,
@@ -114,8 +113,8 @@ void ExternalTabProxy::CreateTab(const CreateTabParams& create_params,
 
 void ExternalTabProxy::Connected(ChromeProxy* proxy) {
   // in ipc thread
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(this,
-      &ExternalTabProxy::UiConnected, proxy));
+  ui_.PostTask(FROM_HERE, base::Bind(&ExternalTabProxy::UiConnected,
+                                     base::Unretained(this), proxy));
 }
 
 void ExternalTabProxy::UiConnected(ChromeProxy* proxy) {
@@ -123,7 +122,7 @@ void ExternalTabProxy::UiConnected(ChromeProxy* proxy) {
   ExternalTabSettings settings;
   settings.parent = m_hWnd;
   settings.style = WS_CHILD;
-  settings.is_off_the_record = tab_params_.is_incognito;
+  settings.is_incognito = tab_params_.is_incognito;
   // TODO(stoyan): FIX this.
   settings.load_requests_via_automation = true;
   // TODO(stoyan): FIX this.
@@ -146,8 +145,8 @@ void ExternalTabProxy::Disconnected() {
 }
 
 void ExternalTabProxy::PeerLost(ChromeProxy* proxy, DisconnectReason reason) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(this, &ExternalTabProxy::UiPeerLost,
-      proxy, reason));
+  ui_.PostTask(FROM_HERE, base::Bind(&ExternalTabProxy::UiPeerLost,
+                                     base::Unretained(this), proxy, reason));
 }
 
 void ExternalTabProxy::UiPeerLost(ChromeProxy* proxy, DisconnectReason reason) {
@@ -190,7 +189,7 @@ void ExternalTabProxy::BlockExternalTab(uint64 cookie) {
   proxy_->BlockTab(cookie);
 }
 
-void ExternalTabProxy::SetZoomLevel(PageZoom::Function zoom_level) {
+void ExternalTabProxy::SetZoomLevel(content::PageZoom zoom_level) {
   proxy_->Tab_Zoom(tab_, zoom_level);
 }
 
@@ -204,25 +203,6 @@ void ExternalTabProxy::ForwardMessageFromExternalHost(
   proxy_->Tab_PostMessage(tab_, message, origin, target);
 }
 
-void ExternalTabProxy::SetEnableExtensionAutomation(
-    const std::vector<std::string>& functions_enabled) {
-  proxy_->Tab_SetEnableExtensionAutomation(tab_, functions_enabled);
-}
-
-void ExternalTabProxy::InstallExtension(const FilePath& crx_path,
-                                        void* user_data) {
-  proxy_->InstallExtension(this, crx_path, new UserDataHolder(user_data));
-}
-
-void ExternalTabProxy::LoadExpandedExtension(const FilePath& path,
-                                             void* user_data) {
-  proxy_->LoadExtension(this, path, new UserDataHolder(user_data));
-}
-
-void ExternalTabProxy::GetEnabledExtensions(void* user_data) {
-  proxy_->GetEnabledExtensions(this, new UserDataHolder(user_data));
-}
-
 void ExternalTabProxy::ChromeFrameHostMoved() {
   proxy_->Tab_OnHostMoved(tab_);
 }
@@ -249,9 +229,10 @@ void ExternalTabProxy::Completed_CreateTab(bool success, HWND chrome_wnd,
                                            HWND tab_window, int tab_handle,
                                            int session_id) {
   // in ipc_thread.
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(this,
-      &ExternalTabProxy::UiCompleted_CreateTab,
-      success, chrome_wnd, tab_window, tab_handle, session_id));
+  ui_.PostTask(
+      FROM_HERE, base::Bind(&ExternalTabProxy::UiCompleted_CreateTab,
+                            base::Unretained(this), success, chrome_wnd,
+                            tab_window, tab_handle, session_id));
 }
 
 void ExternalTabProxy::Completed_ConnectToTab(
@@ -266,73 +247,69 @@ void ExternalTabProxy::Completed_Navigate(
   CHECK(0);
 }
 
-void ExternalTabProxy::Completed_InstallExtension(
-    bool success, enum AutomationMsg_ExtensionResponseValues res,
-    SyncMessageContext* ctx) {
-  CHECK(0);
-}
-
-void ExternalTabProxy::Completed_LoadExpandedExtension(
-    bool success, enum AutomationMsg_ExtensionResponseValues res,
-    SyncMessageContext* ctx) {
-  CHECK(0);
-}
-
-void ExternalTabProxy::Completed_GetEnabledExtensions(
-    bool success, const std::vector<FilePath>* extensions) {
-  CHECK(0);
-}
-
 void ExternalTabProxy::OnNavigationStateChanged(
     int flags, const NavigationInfo& nav_info) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnNavigationStateChanged, flags, nav_info));
+  ui_.PostTask(FROM_HERE,
+               base::Bind(&UIDelegate::OnNavigationStateChanged,
+                          base::Unretained(ui_delegate_), flags, nav_info));
 }
 
 void ExternalTabProxy::OnUpdateTargetUrl(const std::wstring& url) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnUpdateTargetUrl, url));
+  ui_.PostTask(FROM_HERE, base::Bind(&UIDelegate::OnUpdateTargetUrl,
+                                     base::Unretained(ui_delegate_), url));
 }
 
 void ExternalTabProxy::OnTabLoaded(const GURL& url) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnLoad, url));
+  ui_.PostTask(FROM_HERE, base::Bind(&UIDelegate::OnLoad,
+                                     base::Unretained(ui_delegate_), url));
+}
+
+void ExternalTabProxy::OnMoveWindow(const gfx::Rect& pos) {
+  ui_.PostTask(FROM_HERE, base::Bind(&UIDelegate::OnMoveWindow,
+                                     base::Unretained(ui_delegate_), pos));
 }
 
 void ExternalTabProxy::OnMessageToHost(const std::string& message,
                                        const std::string& origin,
                                        const std::string& target) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnMessageFromChromeFrame, message, origin, target));
+  ui_.PostTask(
+      FROM_HERE,
+      base::Bind(&UIDelegate::OnMessageFromChromeFrame,
+                 base::Unretained(ui_delegate_), message, origin, target));
 }
 
 void ExternalTabProxy::OnHandleAccelerator(const MSG& accel_message) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnHandleAccelerator, accel_message));
+  ui_.PostTask(FROM_HERE,
+               base::Bind(&UIDelegate::OnHandleAccelerator,
+                          base::Unretained(ui_delegate_), accel_message));
 }
 
 void ExternalTabProxy::OnHandleContextMenu(
-    HANDLE menu_handle,
+    const ContextMenuModel& context_menu_model,
     int align_flags,
     const MiniContextMenuParams& params) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnHandleContextMenu, menu_handle, align_flags, params));
+  ui_.PostTask(FROM_HERE,
+               base::Bind(&UIDelegate::OnHandleContextMenu,
+                          base::Unretained(ui_delegate_), context_menu_model,
+                          align_flags, params));
 }
 
 void ExternalTabProxy::OnTabbedOut(bool reverse) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnTabbedOut, reverse));
+  ui_.PostTask(FROM_HERE, base::Bind(&UIDelegate::OnTabbedOut,
+                                     base::Unretained(ui_delegate_), reverse));
 }
 
 void ExternalTabProxy::OnGoToHistoryOffset(int offset) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnGoToHistoryOffset, offset));
+  ui_.PostTask(FROM_HERE, base::Bind(&UIDelegate::OnGoToHistoryOffset,
+                                     base::Unretained(ui_delegate_), offset));
 }
 
 void ExternalTabProxy::OnOpenURL(const GURL& url_to_open, const GURL& referrer,
                                  int open_disposition) {
-  ui_.PostTask(FROM_HERE, NewRunnableMethod(ui_delegate_,
-      &UIDelegate::OnOpenURL, url_to_open, referrer, open_disposition));
+  ui_.PostTask(
+      FROM_HERE,
+      base::Bind(&UIDelegate::OnOpenURL, base::Unretained(ui_delegate_),
+                 url_to_open, referrer, open_disposition));
 }
 
 void ExternalTabProxy::OnNavigationFailed(int error_code, const GURL& gurl) {
