@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,28 +9,36 @@
 #include <windows.h>
 #include <atlbase.h>
 #include <atlwin.h>
-#include <vector>
+
 #include <string>
-#include "base/scoped_comptr_win.h"
-#include "base/scoped_ptr.h"
+#include <vector>
+
+#include "base/memory/scoped_ptr.h"
 #include "base/time.h"
+#include "base/win/scoped_comptr.h"
 #include "chrome/common/automation_constants.h"
-#include "chrome/common/page_zoom.h"
 #include "chrome_frame/cfproxy.h"
 #include "chrome_frame/task_marshaller.h"
+#include "content/public/common/page_zoom.h"
 #include "googleurl/src/gurl.h"
 
+struct AttachExternalTabParams;
+struct AutomationURLRequest;
+struct ContextMenuModel;
+struct MiniContextMenuParams;
+struct NavigationInfo;
 class Task;
-class CancelableTask;
 
 namespace base {
-class TimeDelta;
 class WaitableEvent;
 }
 
-namespace IPC {
-struct NavigationInfo;
-struct MiniContextMenuParams;
+namespace gfx {
+class Rect;
+}
+
+namespace net {
+class URLRequestStatus;
 }
 
 // This is the delegate/callback interface that has to be implemented
@@ -40,14 +48,14 @@ class UIDelegate {
   virtual void OnNavigationStateChanged(
       int flags, const NavigationInfo& nav_info) = 0;
   virtual void OnUpdateTargetUrl(const std::wstring& new_target_url) = 0;
-  virtual void OnExtensionInstalled(const FilePath& path, void* user_data,
-      AutomationMsg_ExtensionResponseValues response) = 0;
   virtual void OnLoad(const GURL& url) = 0;
+  virtual void OnMoveWindow(const gfx::Rect& pos) = 0;
+
   virtual void OnMessageFromChromeFrame(
       const std::string& message, const std::string& origin,
       const std::string& target) = 0;
   virtual void OnHandleContextMenu(
-      HANDLE menu_handle, int align_flags,
+      const ContextMenuModel& context_menu_model, int align_flags,
       const MiniContextMenuParams& params) = 0;
   virtual void OnHandleAccelerator(const MSG& accel_message) = 0;
   virtual void OnTabbedOut(bool reverse) = 0;
@@ -69,9 +77,8 @@ struct CreateTabParams {
 class NavigationConstraints;
 
 /////////////////////////////////////////////////////////////////////////
-//  ExternalTabProxy is a mediator between ChromeProxy (which runs mostly in
-//  background IPC-channel thread and the UI object (ActiveX, NPAPI,
-//  ActiveDocument).
+//  ExternalTabProxy is a mediator between ChromeProxy (which runs mostly in the
+//  background IPC-channel thread) and the UI object (ActiveX, ActiveDocument).
 //  The lifetime of ExternalTabProxy is determined by the UI object.
 //
 //  When ExternalTabProxy dies:
@@ -107,17 +114,11 @@ class ExternalTabProxy : public CWindowImpl<ExternalTabProxy>,
       const std::string& origin, const std::string& target);
   virtual void ChromeFrameHostMoved();
 
-  virtual void SetEnableExtensionAutomation(
-      const std::vector<std::string>& functions_enabled);
-  virtual void InstallExtension(const FilePath& crx_path, void* user_data);
-  virtual void LoadExpandedExtension(const FilePath& path, void* user_data);
-  virtual void GetEnabledExtensions(void* user_data);
-
   // Attaches an existing external tab to this automation client instance.
   virtual void ConnectToExternalTab(uint64 external_tab_cookie);
   virtual void BlockExternalTab(uint64 cookie);
 
-  void SetZoomLevel(PageZoom::Function zoom_level);
+  void SetZoomLevel(content::PageZoom zoom_level);
 
  private:
   BEGIN_MSG_MAP(ExternalTabProxy)
@@ -140,12 +141,6 @@ class ExternalTabProxy : public CWindowImpl<ExternalTabProxy>,
       HWND tab_window, int tab_handle, int session_id);
   virtual void Completed_Navigate(bool success,
       enum AutomationMsg_NavigationResponseValues res);
-  virtual void Completed_InstallExtension(bool success,
-      enum AutomationMsg_ExtensionResponseValues res, SyncMessageContext* ctx);
-  virtual void Completed_LoadExpandedExtension(bool success,
-      enum AutomationMsg_ExtensionResponseValues res, SyncMessageContext* ctx);
-  virtual void Completed_GetEnabledExtensions(bool success,
-      const std::vector<FilePath>* extensions);
 
   // Network requests from Chrome.
   virtual void OnNetwork_Start(
@@ -163,6 +158,7 @@ class ExternalTabProxy : public CWindowImpl<ExternalTabProxy>,
   virtual void OnNavigationFailed(int error_code, const GURL& gurl);
   virtual void OnDidNavigate(const NavigationInfo& navigation_info);
   virtual void OnTabLoaded(const GURL& url);
+  virtual void OnMoveWindow(const gfx::Rect& pos);
 
   virtual void OnOpenURL(const GURL& url_to_open, const GURL& referrer,
                          int open_disposition);
@@ -173,7 +169,8 @@ class ExternalTabProxy : public CWindowImpl<ExternalTabProxy>,
 
   // Misc. UI.
   virtual void OnHandleAccelerator(const MSG& accel_message);
-  virtual void OnHandleContextMenu(HANDLE menu_handle, int align_flags,
+  virtual void OnHandleContextMenu(const ContextMenuModel& context_menu_model,
+                                   int align_flags,
                                    const MiniContextMenuParams& params);
   virtual void OnTabbedOut(bool reverse);
 
