@@ -9,15 +9,15 @@
 #include <htiframe.h>
 
 #include "base/logging.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "chrome_frame/bho.h"
 #include "chrome_frame/exception_barrier.h"
 #include "chrome_frame/html_utils.h"
-#include "chrome_frame/urlmon_url_request.h"
 #include "chrome_frame/urlmon_moniker.h"
+#include "chrome_frame/urlmon_url_request.h"
 #include "chrome_frame/utils.h"
 #include "chrome_frame/vtable_patch_manager.h"
 #include "net/http/http_response_headers.h"
@@ -133,7 +133,6 @@ std::string AppendCFUserAgentString(LPCWSTR headers,
 
 std::string ReplaceOrAddUserAgent(LPCWSTR headers,
                                   const std::string& user_agent_value) {
-  DCHECK(headers);
   using net::HttpUtil;
 
   std::string new_headers;
@@ -173,11 +172,11 @@ bool HttpNegotiatePatch::Initialize() {
   // Use our SimpleBindStatusCallback class as we need a temporary object that
   // implements IBindStatusCallback.
   CComObjectStackEx<SimpleBindStatusCallback> request;
-  ScopedComPtr<IBindCtx> bind_ctx;
+  base::win::ScopedComPtr<IBindCtx> bind_ctx;
   HRESULT hr = CreateAsyncBindCtx(0, &request, NULL, bind_ctx.Receive());
   DCHECK(SUCCEEDED(hr)) << "CreateAsyncBindCtx";
   if (bind_ctx) {
-    ScopedComPtr<IUnknown> bscb_holder;
+    base::win::ScopedComPtr<IUnknown> bscb_holder;
     bind_ctx->GetObjectParam(L"_BSCB_Holder_", bscb_holder.Receive());
     if (bscb_holder) {
       hr = PatchHttpNegotiate(bscb_holder);
@@ -201,7 +200,7 @@ HRESULT HttpNegotiatePatch::PatchHttpNegotiate(IUnknown* to_patch) {
   DCHECK(to_patch);
   DCHECK_IS_NOT_PATCHED(IHttpNegotiate);
 
-  ScopedComPtr<IHttpNegotiate> http;
+  base::win::ScopedComPtr<IHttpNegotiate> http;
   HRESULT hr = http.QueryFrom(to_patch);
   if (FAILED(hr)) {
     hr = DoQueryService(IID_IHttpNegotiate, to_patch, http.Receive());
@@ -244,6 +243,9 @@ HRESULT HttpNegotiatePatch::BeginningTransaction(
         *additional_headers,
         (updated_headers.length() + 1) * sizeof(wchar_t)));
     lstrcpyW(*additional_headers, ASCIIToWide(updated_headers).c_str());
+  } else {
+    // TODO(erikwright): Remove the user agent if it is present (i.e., because
+    // of PostPlatform setting in the registry).
   }
   return S_OK;
 }
