@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,12 +13,12 @@
 
 #include "base/basictypes.h"
 #include "base/lazy_instance.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/metrics/histogram.h"
-#include "base/scoped_ptr.h"
 #include "base/synchronization/lock.h"
 #include "base/threading/platform_thread.h"
 #include "base/threading/thread_local.h"
-#include "chrome/common/metrics_helpers.h"
+#include "chrome/common/metrics/metrics_service_base.h"
 
 // TODO(ananta)
 // Refactor more common code from chrome/browser/metrics/metrics_service.h into
@@ -36,9 +36,6 @@ class MetricsService : public MetricsServiceBase {
   void InitializeMetricsState();
 
  private:
-  // To enable the default traits object to create an instance of this class.
-  friend struct base::DefaultLazyInstanceTraits<MetricsService>;
-
   MetricsService();
   virtual ~MetricsService();
   // The MetricsService has a lifecycle that is stored as a state.
@@ -106,9 +103,6 @@ class MetricsService : public MetricsServiceBase {
     return reporting_active_;
   }
 
-  // Convert pending_log_ to XML in pending_log_text_ for transmission.
-  std::string PrepareLogSubmissionString();
-
   // Upload pending data to the server by converting it to XML and compressing
   // it. Returns true on success.
   bool UploadData();
@@ -139,6 +133,9 @@ class MetricsService : public MetricsServiceBase {
   // A number that identifies the how many times the app has been launched.
   int session_id_;
 
+  static base::LazyInstance<base::ThreadLocalPointer<MetricsService> >
+      g_metrics_instance_;
+
   base::PlatformThreadId thread_;
 
   // Indicates if this is the first uma upload from this instance.
