@@ -1,11 +1,10 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome_frame/plugin_url_request.h"
 
 #include "chrome/common/automation_messages.h"
-#include "chrome_frame/np_browser_functions.h"
 
 PluginUrlRequest::PluginUrlRequest()
     : delegate_(NULL),
@@ -13,7 +12,8 @@ PluginUrlRequest::PluginUrlRequest()
       post_data_len_(0),
       enable_frame_busting_(false),
       resource_type_(ResourceType::MAIN_FRAME),
-      load_flags_(0) {
+      load_flags_(0),
+      is_chunked_upload_(false) {
 }
 
 PluginUrlRequest::~PluginUrlRequest() {
@@ -47,6 +47,7 @@ bool PluginUrlRequest::Initialize(PluginUrlRequestDelegate* delegate,
       upload_stream->AddRef();
       upload_stream->Initialize(upload_data);
       upload_data_.Attach(upload_stream);
+      is_chunked_upload_ = upload_data->is_chunked();
     }
   }
 
