@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,20 +7,24 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
-#include "base/weak_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "chrome_frame/infobars/infobar_content.h"
 
 class ReadyModeState;
 class ReadyPromptWindow;
+class UrlLauncher;
 
 // Encapsulates the Ready Mode prompt inviting users to permanently activate
 // Chrome Frame, temporarily disable Ready Mode, or permanently disable Ready
 // Mode.
 class ReadyPromptContent : public InfobarContent {
  public:
-  explicit ReadyPromptContent(ReadyModeState* ready_mode_state);
-  ~ReadyPromptContent();
+  // Takes ownership of the ReadyModeState and UrlLauncher instances, which
+  // will be freed upon destruction of the ReadyPromptContent.
+  ReadyPromptContent(ReadyModeState* ready_mode_state,
+                     UrlLauncher* url_launcher);
+  virtual ~ReadyPromptContent();
 
   // InfobarContent implementation
   virtual bool InstallInFrame(Frame* frame);
@@ -30,6 +34,7 @@ class ReadyPromptContent : public InfobarContent {
  private:
   base::WeakPtr<ReadyPromptWindow> window_;
   scoped_ptr<ReadyModeState> ready_mode_state_;
+  scoped_ptr<UrlLauncher> url_launcher_;
 
   DISALLOW_COPY_AND_ASSIGN(ReadyPromptContent);
 };  // class ReadyPromptContent
