@@ -10,6 +10,7 @@
 #include "base/file_path.h"
 #include "base/path_service.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/common/chrome_switches.h"
 #include "chrome_frame/chrome_frame_automation.h"
 #include "chrome_frame/chrome_frame_plugin.h"
 #include "chrome_frame/navigation_constraints.h"
@@ -26,13 +27,17 @@ class AutomationMockDelegate
       int launch_timeout, bool perform_version_check,
       const std::wstring& profile_name,
       const std::wstring& language,
-      const std::wstring& extra_chrome_arguments, bool incognito,
-      bool is_widget_mode)
+      bool incognito, bool is_widget_mode)
       : caller_message_loop_(caller_message_loop), is_connected_(false),
         navigation_result_(false),
         mock_server_(1337, L"127.0.0.1",
             chrome_frame_test::GetTestDataFolder()) {
 
+    // Endeavour to only kill off Chrome Frame derived Chrome processes.
+    KillAllNamedProcessesWithArgument(
+        UTF8ToWide(chrome_frame_test::kChromeImageName),
+        UTF8ToWide(switches::kChromeFrame));
+
     mock_server_.ExpectAndServeAnyRequests(CFInvocation(CFInvocation::NONE));
 
     FilePath profile_path(
@@ -42,8 +47,7 @@ class AutomationMockDelegate
     GURL empty;
     scoped_refptr<ChromeFrameLaunchParams> clp(
         new ChromeFrameLaunchParams(empty, empty, profile_path, profile_name,
-            language, extra_chrome_arguments, incognito, is_widget_mode,
-            false));
+            language, incognito, is_widget_mode, false));
     clp->set_launch_timeout(launch_timeout);
     clp->set_version_check(perform_version_check);
     automation_client_->Initialize(this, clp);
@@ -131,7 +135,7 @@ class AutomationMockDelegate
   void QuitMessageLoop() {
     // Quit on the caller message loop has to be called on the caller
     // thread.
-    caller_message_loop_->PostTask(FROM_HERE, new MessageLoop::QuitTask);
+    caller_message_loop_->PostTask(FROM_HERE, MessageLoop::QuitClosure());
   }
 
  private:
@@ -148,7 +152,7 @@ class AutomationMockLaunch
   typedef AutomationMockDelegate<AutomationMockLaunch> Base;
   AutomationMockLaunch(MessageLoop* caller_message_loop,
                        int launch_timeout)
-      : Base(caller_message_loop, launch_timeout, true, L"", L"", L"", false,
+      : Base(caller_message_loop, launch_timeout, true, L"", L"", false,
              false) {
   }
   virtual void OnAutomationServerReady() {
@@ -166,7 +170,7 @@ class AutomationMockNavigate
   typedef AutomationMockDelegate<AutomationMockNavigate> Base;
   AutomationMockNavigate(MessageLoop* caller_message_loop,
                          int launch_timeout)
-      : Base(caller_message_loop, launch_timeout, true, L"", L"", L"", false,
+      : Base(caller_message_loop, launch_timeout, true, L"", L"", false,
              false) {
   }
   virtual void OnLoad(const GURL& url) {
@@ -181,7 +185,7 @@ class AutomationMockPostMessage
   typedef AutomationMockDelegate<AutomationMockPostMessage> Base;
   AutomationMockPostMessage(MessageLoop* caller_message_loop,
                             int launch_timeout)
-      : Base(caller_message_loop, launch_timeout, true, L"", L"", L"", false,
+      : Base(caller_message_loop, launch_timeout, true, L"", L"", false,
              false),
         postmessage_result_(false) {}
   bool postmessage_result() const {
@@ -209,7 +213,7 @@ class AutomationMockHostNetworkRequestStart
   typedef AutomationMockDelegate<AutomationMockHostNetworkRequestStart> Base;
   AutomationMockHostNetworkRequestStart(MessageLoop* caller_message_loop,
       int launch_timeout)
-      : Base(caller_message_loop, launch_timeout, true, L"", L"", L"", false,
+      : Base(caller_message_loop, launch_timeout, true, L"", L"", false,
              false),
         request_start_result_(false) {
     if (automation()) {
