@@ -14,19 +14,17 @@
 
 #include "base/basictypes.h"
 #include "base/compiler_specific.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
 #include "base/process_util.h"
-#include "base/scoped_comptr_win.h"
-#include "base/scoped_ptr.h"
+#include "base/test/test_reg_util_win.h"
 #include "base/win/registry.h"
-
-#include "chrome_frame/test_utils.h"
+#include "base/win/scoped_comptr.h"
+#include "chrome_frame/chrome_tab.h"
 #include "chrome_frame/test/simulate_input.h"
+#include "chrome_frame/test_utils.h"
 #include "chrome_frame/utils.h"
 
-// Include without path to make GYP build see it.
-#include "chrome_tab.h"  // NOLINT
-
 #include "gtest/gtest.h"
 
 // Needed for CreateFunctor.
@@ -39,10 +37,7 @@ namespace chrome_frame_test {
 
 int CloseVisibleWindowsOnAllThreads(HANDLE process);
 
-base::ProcessHandle LaunchFirefox(const std::wstring& url);
-base::ProcessHandle LaunchOpera(const std::wstring& url);
 base::ProcessHandle LaunchIE(const std::wstring& url);
-base::ProcessHandle LaunchSafari(const std::wstring& url);
 base::ProcessHandle LaunchChrome(const std::wstring& url);
 
 // Attempts to close all open IE windows.
@@ -54,12 +49,10 @@ int CloseAllIEWindows();
 
 extern const wchar_t kIEImageName[];
 extern const wchar_t kIEBrokerImageName[];
-extern const wchar_t kFirefoxImageName[];
-extern const wchar_t kOperaImageName[];
-extern const wchar_t kSafariImageName[];
 extern const char kChromeImageName[];
 extern const wchar_t kChromeLauncher[];
 extern const int kChromeFrameLongNavigationTimeoutInSeconds;
+extern const int kChromeFrameVeryLongNavigationTimeoutInSeconds;
 
 // Temporarily impersonate the current thread to low integrity for the lifetime
 // of the object. Destructor will automatically revert integrity level.
@@ -117,7 +110,7 @@ class HungCOMCallDetector
   }
 
   void TearDown() {
-    ScopedComPtr<IMessageFilter> prev_filter;
+    base::win::ScopedComPtr<IMessageFilter> prev_filter;
     CoRegisterMessageFilter(prev_filter_.get(), prev_filter.Receive());
     DestroyWindow();
     m_hWnd = NULL;
@@ -140,8 +133,7 @@ class HungCOMCallDetector
                                     DWORD tick_count,
                                     DWORD pending_type) {
     MSG msg = {0};
-    if (PeekMessage(&msg, m_hWnd, WM_TIMER, WM_TIMER, PM_NOREMOVE)) {
-      is_hung_ = true;
+    if (is_hung_) {
       return PENDINGMSG_CANCELCALL;
     }
     return PENDINGMSG_WAITDEFPROCESS;
@@ -152,6 +144,7 @@ class HungCOMCallDetector
   }
 
   LRESULT OnTimer(UINT msg, WPARAM wp, LPARAM lp, BOOL& handled) {  // NOLINT
+    is_hung_ = true;
     return 1;
   }
 
@@ -175,13 +168,13 @@ class HungCOMCallDetector
     }
 
     static const int kHungDetectTimerId = 0x0000baba;
-    SetTimer(kHungDetectTimerId, 1000 * (timeout_seconds + 2), NULL);
+    SetTimer(kHungDetectTimerId, 1000 * (timeout_seconds + 40), NULL);
     return S_OK;
   }
 
   // used to detect if outgoing COM calls hung.
   bool is_hung_;
-  ScopedComPtr<IMessageFilter> prev_filter_;
+  base::win::ScopedComPtr<IMessageFilter> prev_filter_;
 };
 
 // MessageLoopForUI wrapper that runs only for a limited time.
@@ -197,9 +190,14 @@ class TimedMsgLoop {
     loop_.MessageLoop::Run();
   }
 
-  void PostDelayedTask(
-    const tracked_objects::Location& from_here, Task* task, int64 delay_ms) {
-      loop_.PostDelayedTask(from_here, task, delay_ms);
+  void PostTask(const tracked_objects::Location& from_here,
+                const base::Closure& task) {
+    loop_.PostTask(from_here, task);
+  }
+
+  void PostDelayedTask(const tracked_objects::Location& from_here,
+                       const base::Closure& task, int64 delay_ms) {
+    loop_.PostDelayedTask(from_here, task, delay_ms);
   }
 
   void Quit() {
@@ -208,7 +206,8 @@ class TimedMsgLoop {
 
   void QuitAfter(int seconds) {
     quit_loop_invoked_ = true;
-    loop_.PostDelayedTask(FROM_HERE, new MessageLoop::QuitTask, 1000 * seconds);
+    loop_.PostDelayedTask(
+        FROM_HERE, MessageLoop::QuitClosure(), 1000 * seconds);
   }
 
   bool WasTimedOut() const {
@@ -290,21 +289,6 @@ class CloseIeAtEndOfScope {
 // during test runs.
 base::ProcessHandle StartCrashService();
 
-class TempRegKeyOverride {
- public:
-  static const wchar_t kTempTestKeyPath[];
-
-  TempRegKeyOverride(HKEY override, const wchar_t* temp_name);
-  ~TempRegKeyOverride();
-
-  static void DeleteAllTempKeys();
-
- protected:
-  HKEY override_;
-  base::win::RegKey temp_key_;
-  std::wstring temp_name_;
-};
-
 // Used in tests where we reference the registry and don't want to run into
 // problems where existing registry settings might conflict with the
 // expectations of the test.
@@ -313,9 +297,12 @@ class ScopedVirtualizeHklmAndHkcu {
   ScopedVirtualizeHklmAndHkcu();
   ~ScopedVirtualizeHklmAndHkcu();
 
+  // Removes all overrides and deletes all temporary test keys used by the
+  // overrides.
+  void RemoveAllOverrides();
+
  protected:
-  scoped_ptr<TempRegKeyOverride> hklm_;
-  scoped_ptr<TempRegKeyOverride> hkcu_;
+  registry_util::RegistryOverrideManager override_manager_;
 };
 
 // Attempts to kill all the processes on the current machine that were launched
@@ -328,6 +315,12 @@ class ScopedVirtualizeHklmAndHkcu {
 bool KillProcesses(const std::wstring& executable_name, int exit_code,
                    bool wait);
 
+// Returns the type of test bed, PER_USER or SYSTEM_LEVEL.
+ScopedChromeFrameRegistrar::RegistrationType GetTestBedType();
+
+// Clears IE8 session restore history.
+void ClearIESessionHistory();
+
 }  // namespace chrome_frame_test
 
 // TODO(tommi): This is a temporary workaround while we're getting our
