@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,11 @@
 #include <shlguid.h>
 #include <shobjidl.h>
 
+#include <map>
+#include <utility>
+
+#include "base/lazy_instance.h"
+#include "base/string_number_conversions.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
@@ -18,6 +23,114 @@
 
 using base::win::ScopedBstr;
 
+namespace {
+
+// A lookup table from DISPID to DWebBrowserEvents and/or DWebBrowserEvents2
+// method name.
+class DispIdNameTable {
+ public:
+  DispIdNameTable();
+  ~DispIdNameTable();
+
+  // Returns the method name corresponding to |dispid| or, if none is known,
+  // the string "DISPID |dispid|".
+  std::string Lookup(DISPID dispid) const;
+
+ private:
+  std::map<DISPID,const char*> dispid_to_name_;
+  DISALLOW_COPY_AND_ASSIGN(DispIdNameTable);
+};
+
+DispIdNameTable::DispIdNameTable() {
+  static const struct {
+    DISPID dispid;
+    const char* name;
+  } kIdToName[] = {
+    // DWebBrowserEvents
+    { 100, "BeforeNavigate" },
+    { 101, "NavigateComplete" },
+    { 102, "StatusTextChange" },
+    { 108, "ProgressChange" },
+    { 104, "DownloadComplete" },
+    { 105, "CommandStateChange" },
+    { 106, "DownloadBegin" },
+    { 107, "NewWindow" },
+    { 113, "TitleChange" },
+    { 200, "FrameBeforeNavigate" },
+    { 201, "FrameNavigateComplete" },
+    { 204, "FrameNewWindow" },
+    { 103, "Quit" },
+    { 109, "WindowMove" },
+    { 110, "WindowResize" },
+    { 111, "WindowActivate" },
+    { 112, "PropertyChange" },
+    // DWebBrowserEvents2
+    { 250, "BeforeNavigate2" },
+    { 251, "NewWindow2" },
+    { 252, "NavigateComplete2" },
+    { 259, "DocumentComplete" },
+    { 253, "OnQuit" },
+    { 254, "OnVisible" },
+    { 255, "OnToolBar" },
+    { 256, "OnMenuBar" },
+    { 257, "OnStatusBar" },
+    { 258, "OnFullScreen" },
+    { 260, "OnTheaterMode" },
+    { 262, "WindowSetResizable" },
+    { 264, "WindowSetLeft" },
+    { 265, "WindowSetTop" },
+    { 266, "WindowSetWidth" },
+    { 267, "WindowSetHeight" },
+    { 263, "WindowClosing" },
+    { 268, "ClientToHostWindow" },
+    { 269, "SetSecureLockIcon" },
+    { 270, "FileDownload" },
+    { 271, "NavigateError" },
+    { 225, "PrintTemplateInstantiation" },
+    { 226, "PrintTemplateTeardown" },
+    { 227, "UpdatePageStatus" },
+    { 272, "PrivacyImpactedStateChange" },
+    { 273, "NewWindow3" },
+    { 282, "SetPhishingFilterStatus" },
+    { 283, "WindowStateChanged" },
+    { 284, "NewProcess" },
+    { 285, "ThirdPartyUrlBlocked" },
+    { 286, "RedirectXDomainBlocked" },
+    // Present in ExDispid.h but not ExDisp.idl
+    { 114, "TitleIconChange" },
+    { 261, "OnAddressBar" },
+    { 281, "ViewUpdate" },
+  };
+  size_t index_of_duplicate = 0;
+  DISPID duplicate_dispid = 0;
+  for (size_t i = 0; i < arraysize(kIdToName); ++i) {
+    if (!dispid_to_name_.insert(std::make_pair(kIdToName[i].dispid,
+                                               kIdToName[i].name)).second &&
+        index_of_duplicate == 0) {
+      index_of_duplicate = i;
+      duplicate_dispid = kIdToName[i].dispid;
+    }
+  }
+  DCHECK_EQ(static_cast<size_t>(0), index_of_duplicate)
+      << "Duplicate name for DISPID " << duplicate_dispid
+      << " at kIdToName[" << index_of_duplicate << "]";
+}
+
+DispIdNameTable::~DispIdNameTable() {
+}
+
+std::string DispIdNameTable::Lookup(DISPID dispid) const {
+  std::map<DISPID,const char*>::const_iterator it =
+      dispid_to_name_.find(dispid);
+  if (it != dispid_to_name_.end())
+    return it->second;
+  return std::string("DISPID ").append(base::IntToString(dispid));
+}
+
+base::LazyInstance<DispIdNameTable> g_dispIdToName = LAZY_INSTANCE_INITIALIZER;
+
+}  // namespace
+
 namespace chrome_frame_test {
 
 const int kDefaultWaitForIEToTerminateMs = 10 * 1000;
@@ -143,10 +256,8 @@ void IEEventSink::Uninitialize() {
         std::wstring browser_url_wstring;
         browser_url_wstring.assign(browser_url, browser_url.Length());
         std::string browser_url_string = WideToUTF8(browser_url_wstring);
-        EXPECT_TRUE(did_receive_on_quit_) << "OnQuit was not received for "
-                                          << "browser with url "
-                                          << browser_url_string;
-
+        LOG(ERROR) << "OnQuit was not received for browser with url "
+                   << browser_url_string;
         web_browser2_->Quit();
       }
 
@@ -176,13 +287,13 @@ bool IEEventSink::IsCFRendering() {
   DCHECK(web_browser2_);
 
   if (web_browser2_) {
-    ScopedComPtr<IDispatch> doc;
+    base::win::ScopedComPtr<IDispatch> doc;
     web_browser2_->get_Document(doc.Receive());
     if (doc) {
       // Detect if CF is rendering based on whether the document is a
       // ChromeActiveDocument. Detecting based on hwnd is problematic as
       // the CF Active Document window may not have been created yet.
-      ScopedComPtr<IChromeFrame> chrome_frame;
+      base::win::ScopedComPtr<IChromeFrame> chrome_frame;
       chrome_frame.QueryFrom(doc);
       return chrome_frame.get();
     }
@@ -192,6 +303,9 @@ bool IEEventSink::IsCFRendering() {
 
 void IEEventSink::PostMessageToCF(const std::wstring& message,
                                   const std::wstring& target) {
+  EXPECT_TRUE(chrome_frame_ != NULL);
+  if (!chrome_frame_)
+    return;
   ScopedBstr message_bstr(message.c_str());
   base::win::ScopedVariant target_variant(target.c_str());
   EXPECT_HRESULT_SUCCEEDED(
@@ -240,7 +354,7 @@ void IEEventSink::ExpectAddressBarUrl(
 void IEEventSink::Exec(const GUID* cmd_group_guid, DWORD command_id,
                                DWORD cmd_exec_opt, VARIANT* in_args,
                                VARIANT* out_args) {
-  ScopedComPtr<IOleCommandTarget> shell_browser_cmd_target;
+  base::win::ScopedComPtr<IOleCommandTarget> shell_browser_cmd_target;
   DoQueryService(SID_STopLevelBrowser, web_browser2_,
                  shell_browser_cmd_target.Receive());
   ASSERT_TRUE(NULL != shell_browser_cmd_target);
@@ -259,7 +373,7 @@ HWND IEEventSink::GetRendererWindow() {
   HWND renderer_window = NULL;
   if (IsCFRendering()) {
     DCHECK(chrome_frame_);
-    ScopedComPtr<IOleWindow> ole_window;
+    base::win::ScopedComPtr<IOleWindow> ole_window;
     ole_window.QueryFrom(chrome_frame_);
     EXPECT_TRUE(ole_window.get());
 
@@ -283,12 +397,12 @@ HWND IEEventSink::GetRendererWindow() {
     }
   } else {
     DCHECK(web_browser2_);
-    ScopedComPtr<IDispatch> doc;
+    base::win::ScopedComPtr<IDispatch> doc;
     HRESULT hr = web_browser2_->get_Document(doc.Receive());
     EXPECT_HRESULT_SUCCEEDED(hr);
     EXPECT_TRUE(doc);
     if (doc) {
-      ScopedComPtr<IOleWindow> ole_window;
+      base::win::ScopedComPtr<IOleWindow> ole_window;
       ole_window.QueryFrom(doc);
       EXPECT_TRUE(ole_window);
       if (ole_window) {
@@ -305,7 +419,7 @@ HWND IEEventSink::GetRendererWindowSafe() {
   HWND renderer_window = NULL;
   if (IsCFRendering()) {
     DCHECK(chrome_frame_);
-    ScopedComPtr<IOleWindow> ole_window;
+    base::win::ScopedComPtr<IOleWindow> ole_window;
     ole_window.QueryFrom(chrome_frame_);
 
     if (ole_window) {
@@ -324,10 +438,10 @@ HWND IEEventSink::GetRendererWindowSafe() {
     }
   } else {
     DCHECK(web_browser2_);
-    ScopedComPtr<IDispatch> doc;
+    base::win::ScopedComPtr<IDispatch> doc;
     web_browser2_->get_Document(doc.Receive());
     if (doc) {
-      ScopedComPtr<IOleWindow> ole_window;
+      base::win::ScopedComPtr<IOleWindow> ole_window;
       ole_window.QueryFrom(doc);
       if (ole_window) {
         ole_window->GetWindow(&renderer_window);
@@ -339,15 +453,28 @@ HWND IEEventSink::GetRendererWindowSafe() {
   return renderer_window;
 }
 
-HRESULT IEEventSink::LaunchIEAndNavigate(
-    const std::wstring& navigate_url, IEEventListener* listener) {
+HRESULT IEEventSink::LaunchIEAndNavigate(const std::wstring& navigate_url,
+                                         IEEventListener* listener) {
   listener_ = listener;
   HRESULT hr = LaunchIEAsComServer(web_browser2_.Receive());
   if (SUCCEEDED(hr)) {
     web_browser2_->put_Visible(VARIANT_TRUE);
-    Attach(web_browser2_);
-    hr = Navigate(navigate_url);
+    hr = Attach(web_browser2_);
+    if (SUCCEEDED(hr)) {
+      hr = Navigate(navigate_url);
+      if (FAILED(hr)) {
+        LOG(ERROR) << "Failed to navigate IE to " << navigate_url << ", hr = 0x"
+                   << std::hex << hr;
+      }
+    } else {
+      LOG(ERROR) << "Failed to attach to web browser event sink for "
+                 << navigate_url << ", hr = 0x" << std::hex << hr;
+    }
+  } else {
+    LOG(ERROR) << "Failed to Launch IE for " << navigate_url << ", hr = 0x"
+               << std::hex << hr;
   }
+
   return hr;
 }
 
@@ -372,7 +499,7 @@ HRESULT IEEventSink::CloseWebBrowser() {
 }
 
 void IEEventSink::Refresh() {
-  base::win::ScopedVariant refresh_level(REFRESH_NORMAL);
+  base::win::ScopedVariant refresh_level(REFRESH_COMPLETELY);
   web_browser2_->Refresh2(refresh_level.AsInput());
 }
 
@@ -381,12 +508,12 @@ void IEEventSink::ConnectToChromeFrame() {
   DCHECK(web_browser2_);
   if (chrome_frame_.get())
     return;
-  ScopedComPtr<IShellBrowser> shell_browser;
+  base::win::ScopedComPtr<IShellBrowser> shell_browser;
   DoQueryService(SID_STopLevelBrowser, web_browser2_,
                  shell_browser.Receive());
 
   if (shell_browser) {
-    ScopedComPtr<IShellView> shell_view;
+    base::win::ScopedComPtr<IShellView> shell_view;
     shell_browser->QueryActiveShellView(shell_view.Receive());
     if (shell_view) {
       shell_view->GetItemObject(SVGIO_BACKGROUND, __uuidof(IChromeFrame),
@@ -409,7 +536,7 @@ void IEEventSink::DisconnectFromChromeFrame() {
     // Use a local ref counted copy of the IChromeFrame interface as the
     // outgoing calls could cause the interface to be deleted due to a message
     // pump running in the context of the outgoing call.
-    ScopedComPtr<IChromeFrame> chrome_frame(chrome_frame_);
+    base::win::ScopedComPtr<IChromeFrame> chrome_frame(chrome_frame_);
     chrome_frame_.Release();
     base::win::ScopedVariant dummy(static_cast<IDispatch*>(NULL));
     chrome_frame->put_onmessage(dummy);
@@ -433,10 +560,31 @@ STDMETHODIMP_(void) IEEventSink::OnDownloadBegin() {
     listener_->OnDownloadBegin();
 }
 
-STDMETHODIMP_(void) IEEventSink::OnNewWindow2(IDispatch** disp,
+STDMETHODIMP_(void) IEEventSink::OnNewWindow2(IDispatch** dispatch,
                                               VARIANT_BOOL* s) {
+  DVLOG(1) << __FUNCTION__;
+
+  EXPECT_TRUE(dispatch);
+  if (!dispatch)
+    return;
+
   if (listener_)
-    listener_->OnNewWindow2(disp, s);
+    listener_->OnNewWindow2(dispatch, s);
+
+  // Note that |dispatch| is an [in/out] argument. IE is asking listeners if
+  // they want to use a IWebBrowser2 of their choice for the new window.
+  // Since we need to listen on events on the new browser, we create one
+  // if needed.
+  if (!*dispatch) {
+    base::win::ScopedComPtr<IDispatch> new_browser;
+    HRESULT hr = new_browser.CreateInstance(CLSID_InternetExplorer, NULL,
+                                            CLSCTX_LOCAL_SERVER);
+    DCHECK(SUCCEEDED(hr) && new_browser);
+    *dispatch = new_browser.Detach();
+  }
+
+  if (*dispatch && listener_)
+    listener_->OnNewBrowserWindow(*dispatch, ScopedBstr());
 }
 
 STDMETHODIMP_(void) IEEventSink::OnNavigateError(IDispatch* dispatch,
@@ -506,14 +654,14 @@ STDMETHODIMP_(void) IEEventSink::OnNewWindow3(
   // Since we need to listen on events on the new browser, we create one
   // if needed.
   if (!*dispatch) {
-    ScopedComPtr<IDispatch> new_browser;
+    base::win::ScopedComPtr<IDispatch> new_browser;
     HRESULT hr = new_browser.CreateInstance(CLSID_InternetExplorer, NULL,
                                             CLSCTX_LOCAL_SERVER);
     DCHECK(SUCCEEDED(hr) && new_browser);
     *dispatch = new_browser.Detach();
   }
 
-  if (*dispatch)
+  if (*dispatch && listener_)
     listener_->OnNewBrowserWindow(*dispatch, url);
 }
 
@@ -529,6 +677,15 @@ STDMETHODIMP_(void) IEEventSink::OnQuit() {
     listener_->OnQuit();
 }
 
+STDMETHODIMP IEEventSink::Invoke(DISPID dispid, REFIID riid, LCID lcid,
+                                 WORD flags, DISPPARAMS* params,
+                                 VARIANT* result, EXCEPINFO* except_info,
+                                 UINT* arg_error) {
+  VLOG(1) << __FUNCTION__ << L" event: " << g_dispIdToName.Get().Lookup(dispid);
+  return DispEventsImpl::Invoke(dispid, riid, lcid, flags, params, result,
+                                except_info, arg_error);
+}
+
 HRESULT IEEventSink::OnLoad(const VARIANT* param) {
   DVLOG(1) << __FUNCTION__ << " " << param->bstrVal;
   base::win::ScopedVariant stack_object(*param);
