@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,6 +9,7 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/threading/platform_thread.h"
 #include "base/time.h"
 #include "chrome/common/chrome_switches.h"
@@ -17,9 +18,9 @@
 #include "chrome_frame/test/ie_event_sink.h"
 #include "chrome_frame/test/mock_ie_event_sink_test.h"
 #include "chrome_frame/test/simulate_input.h"
-#include "gfx/point.h"
-#include "gfx/rect.h"
 #include "testing/gmock/include/gmock/gmock.h"
+#include "ui/gfx/point.h"
+#include "ui/gfx/rect.h"
 
 namespace chrome_frame_test {
 
@@ -38,8 +39,8 @@ ACTION_P2(Navigate, mock, navigate_url) {
 }
 
 ACTION_P3(DelayNavigateToCurrentUrl, mock, loop, delay) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableFunction(&NavigateToCurrentUrl,
-      mock), delay);
+  loop->PostDelayedTask(FROM_HERE,
+                        base::Bind(&NavigateToCurrentUrl, mock), delay);
 }
 
 ACTION_P(CloseBrowserMock, mock) {
@@ -47,12 +48,15 @@ ACTION_P(CloseBrowserMock, mock) {
 }
 
 ACTION_P3(DelayCloseBrowserMock, loop, delay, mock) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(mock->event_sink(),
-      &IEEventSink::CloseWebBrowser), delay);
+  loop->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(base::IgnoreResult(&IEEventSink::CloseWebBrowser),
+                 mock->event_sink()),
+      delay);
 }
 
 ACTION_P2(ConnectDocPropNotifySink, mock, sink) {
-  ScopedComPtr<IDispatch> document;
+  base::win::ScopedComPtr<IDispatch> document;
   mock->event_sink()->web_browser2()->get_Document(document.Receive());
   EXPECT_TRUE(document != NULL);  // NOLINT
   if (document) {
@@ -66,24 +70,27 @@ ACTION_P(DisconnectDocPropNotifySink, sink) {
 
 ACTION_P8(DelayExecCommand, mock, loop, delay, cmd_group_guid, cmd_id,
           cmd_exec_opt, in_args, out_args) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(mock->event_sink(),
-      &IEEventSink::Exec, cmd_group_guid, cmd_id, cmd_exec_opt, in_args,
-          out_args), delay);
+  loop->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(&IEEventSink::Exec, mock->event_sink(), cmd_group_guid, cmd_id,
+                 cmd_exec_opt, in_args, out_args),
+      delay);
 }
 
 ACTION_P3(DelayGoBack, mock, loop, delay) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(mock->event_sink(),
-      &IEEventSink::GoBack), delay);
+  loop->PostDelayedTask(
+      FROM_HERE, base::Bind(&IEEventSink::GoBack, mock->event_sink()), delay);
 }
 
 ACTION_P3(DelayGoForward, mock, loop, delay) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(mock->event_sink(),
-      &IEEventSink::GoForward), delay);
+  loop->PostDelayedTask(
+      FROM_HERE, base::Bind(&IEEventSink::GoForward, mock->event_sink()),
+      delay);
 }
 
 ACTION_P3(DelayRefresh, mock, loop, delay) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableMethod(mock->event_sink(),
-      &IEEventSink::Refresh), delay);
+  loop->PostDelayedTask(
+      FROM_HERE, base::Bind(&IEEventSink::Refresh, mock->event_sink()), delay);
 }
 
 ACTION_P2(PostMessageToCF, mock, message) {
@@ -269,9 +276,7 @@ ACTION(DoCloseWindow) {
 ACTION_P(DelayDoCloseWindow, delay) {
   DCHECK(MessageLoop::current());
   MessageLoop::current()->PostDelayedTask(
-      FROM_HERE,
-      NewRunnableFunction(DoCloseWindowNow, arg0),
-      delay);
+      FROM_HERE, base::Bind(DoCloseWindowNow, arg0), delay);
 }
 
 ACTION(KillChromeFrameProcesses) {
@@ -347,11 +352,11 @@ ACTION_P(VerifySelectedText, expected_text) {
 ACTION_P3(CloseWhenFileSaved, mock, file, timeout_ms) {
   base::Time start = base::Time::Now();
   while (!file_util::PathExists(file)) {
-    base::PlatformThread::Sleep(200);
     if ((base::Time::Now() - start).InMilliseconds() > timeout_ms) {
       ADD_FAILURE() << "File was not saved within timeout";
       break;
     }
+    base::PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(200));
   }
   mock->event_sink()->CloseWebBrowser();
 }
@@ -359,7 +364,7 @@ ACTION_P3(CloseWhenFileSaved, mock, file, timeout_ms) {
 ACTION_P2(WaitForFileSave, file, timeout_ms) {
   base::Time start = base::Time::Now();
   while (!file_util::PathExists(file)) {
-    base::PlatformThread::Sleep(200);
+    base::PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(200));
     if ((base::Time::Now() - start).InMilliseconds() > timeout_ms) {
       ADD_FAILURE() << "File was not saved within timeout";
       break;
@@ -375,31 +380,32 @@ ACTION_P(SetFocusToRenderer, mock) {
 }
 
 ACTION_P4(DelaySendChar, loop, delay, c, mod) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableFunction(
-      simulate_input::SendCharA, c, mod), delay);
+  loop->PostDelayedTask(
+      FROM_HERE, base::Bind(simulate_input::SendCharA, c, mod), delay);
 }
 
 ACTION_P4(DelaySendScanCode, loop, delay, c, mod) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableFunction(
-        simulate_input::SendScanCode, c, mod), delay);
+  loop->PostDelayedTask(
+      FROM_HERE, base::Bind(simulate_input::SendScanCode, c, mod), delay);
 }
 
 // This function selects the address bar via the Alt+d shortcut.
 ACTION_P3(TypeUrlInAddressBar, loop, url, delay) {
-  loop->PostDelayedTask(FROM_HERE, NewRunnableFunction(
-      simulate_input::SendCharA, 'd', simulate_input::ALT),
-      delay);
+  loop->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(simulate_input::SendCharA, 'd', simulate_input::ALT), delay);
 
   const unsigned int kInterval = 500;
   int next_delay = delay + kInterval;
 
-  loop->PostDelayedTask(FROM_HERE, NewRunnableFunction(
-      simulate_input::SendStringW, url), next_delay);
+  loop->PostDelayedTask(
+      FROM_HERE, base::Bind(simulate_input::SendStringW, url), next_delay);
 
   next_delay = next_delay + kInterval;
 
-  loop->PostDelayedTask(FROM_HERE, NewRunnableFunction(
-      simulate_input::SendCharA, VK_RETURN, simulate_input::NONE),
+  loop->PostDelayedTask(
+      FROM_HERE,
+      base::Bind(simulate_input::SendCharA, VK_RETURN, simulate_input::NONE),
       next_delay);
 }
 
