@@ -1,11 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <string>
 
-#include "base/scoped_comptr_win.h"
 #include "base/test/test_file_util.h"
+#include "base/win/scoped_comptr.h"
 #include "base/win/windows_version.h"
 #include "chrome_frame/test/chrome_frame_test_utils.h"
 #include "chrome_frame/test/chrome_frame_ui_test_utils.h"
@@ -17,9 +17,9 @@
 #define GMOCK_MUTANT_INCLUDE_LATE_OBJECT_BINDING
 #include "testing/gmock_mutant.h"
 
-using testing::_;
 using testing::InSequence;
 using testing::StrEq;
+using testing::_;
 
 namespace chrome_frame_test {
 
@@ -124,7 +124,8 @@ TEST_P(FullTabNavigationTest, Refresh) {
 }
 
 // Test that multiple back and forward requests work.
-TEST_P(FullTabNavigationTest, MultipleBackForward) {
+// TODO(tsepez): http://crbug.com/83133
+TEST_P(FullTabNavigationTest, FLAKY_MultipleBackForward) {
   std::wstring page1 = GetSimplePageUrl();
   std::wstring page2 = GetLinkPageUrl();
   std::wstring page3 = GetAnchorPageUrl(0);
@@ -277,7 +278,7 @@ TEST_P(FullTabNavigationTest, BackForwardAnchor) {
 // dialog appears.
 TEST_P(FullTabNavigationTest, RestrictedSite) {
   // Add the server to restricted sites zone.
-  ScopedComPtr<IInternetSecurityManager> security_manager;
+  base::win::ScopedComPtr<IInternetSecurityManager> security_manager;
   HRESULT hr = security_manager.CreateInstance(CLSID_InternetSecurityManager);
   ASSERT_HRESULT_SUCCEEDED(hr);
   hr = security_manager->SetZoneMapping(URLZONE_UNTRUSTED,
@@ -829,7 +830,7 @@ TEST_F(FullTabDownloadTest, CF_DownloadFileFromPost) {
                               testing::Field(&VARIANT::bstrVal,
                               StrEq(tgt_url)), _, _, _, _, _));
   EXPECT_CALL(download_window_mock, OnLoad(false, _));
-  EXPECT_CALL(download_window_mock, OnQuit());
+  EXPECT_CALL(download_window_mock, OnQuit()).Times(testing::AtMost(1));
 
   FilePath temp_file_path;
   ASSERT_TRUE(file_util::CreateTemporaryFile(&temp_file_path));
@@ -856,7 +857,8 @@ TEST_F(FullTabDownloadTest, CF_DownloadFileFromPost) {
             testing::InvokeWithoutArgs(
                 testing::CreateFunctor(CloseWindow, &owner_window)),
             CloseBrowserMock(&ie_mock_)));
-  LaunchIENavigateAndLoop(src_url, kChromeFrameLongNavigationTimeoutInSeconds);
+  LaunchIENavigateAndLoop(src_url,
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 
   std::string data;
   EXPECT_TRUE(file_util::ReadFileToString(temp_file_path, &data));
@@ -958,13 +960,14 @@ TEST_P(FullTabNavigationTest, RefreshContents) {
                                 _, _, _, _, _));
   EXPECT_CALL(ie_mock_,
               OnNavigateComplete2(_, testing::Field(&VARIANT::bstrVal,
-                                                    StrEq(src_url))))
-      .WillOnce(DelayRefresh(&ie_mock_, &loop_, 2000));
-
+                                                    StrEq(src_url))));
   EXPECT_CALL(ie_mock_, OnLoad(in_cf, StrEq(src_url)))
-      .Times(2);
+      .Times(2)
+      .WillOnce(DelayRefresh(&ie_mock_, &loop_, 50))
+      .WillOnce(testing::Return());
 
-  LaunchIEAndNavigate(src_url);
+  LaunchIENavigateAndLoop(src_url,
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 class FullTabSeleniumTest
@@ -988,8 +991,9 @@ ACTION(VerifySeleniumCoreTestResults) {
       " Actual number of tests run: " << num_tests;
 }
 
+// Crashes flakily: http://crbug.com/109114
 // Tests refreshing causes a page load.
-TEST_F(FullTabSeleniumTest, Core) {
+TEST_F(FullTabSeleniumTest, DISABLED_Core) {
   // Please see http://code.google.com/p/chromium/issues/detail?id=60987
   // for more information on why this test is disabled for Vista with IE7.
   if (base::win::GetVersion() == base::win::VERSION_VISTA &&
@@ -1098,7 +1102,9 @@ TEST_F(FullTabDownloadTest, TopLevelPostReissueFromChromeFramePage) {
   EXPECT_CALL(ie_mock_, OnLoad(false, StrEq(src_url)));
 
   EXPECT_CALL(ie_mock_, OnLoad(true, StrEq(tgt_url)))
-      .Times(2);
+      .Times(testing::Between(1,2))
+      .WillOnce(DelayRefresh(&ie_mock_, &loop_, 50))
+      .WillOnce(testing::Return());
 
   EXPECT_CALL(ie_mock_, OnBeforeNavigate2(_,
                               testing::Field(&VARIANT::bstrVal,
@@ -1109,11 +1115,11 @@ TEST_F(FullTabDownloadTest, TopLevelPostReissueFromChromeFramePage) {
                               testing::Field(&VARIANT::bstrVal,
                               StrEq(tgt_url))))
       .Times(2)
-      .WillOnce(testing::DoAll(DelayRefresh(&ie_mock_, &loop_, 2000),
-                DelayCloseBrowserMock(&loop_, 4000, &ie_mock_)))
+      .WillOnce(DelayCloseBrowserMock(&loop_, 4000, &ie_mock_))
       .WillOnce(testing::Return());
 
-  LaunchIENavigateAndLoop(src_url, kChromeFrameLongNavigationTimeoutInSeconds);
+  LaunchIENavigateAndLoop(src_url,
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 MATCHER_P(UserAgentHeaderMatcher, ua_string, "") {
@@ -1174,13 +1180,14 @@ TEST_P(FullTabNavigationTest, RefreshContentsUATest) {
                                 _, _, _, _, _));
   EXPECT_CALL(ie_mock_,
               OnNavigateComplete2(_, testing::Field(&VARIANT::bstrVal,
-                                                    StrEq(src_url))))
-      .WillOnce(DelayRefresh(&ie_mock_, &loop_, 2000));
-
+                                                    StrEq(src_url))));
   EXPECT_CALL(ie_mock_, OnLoad(in_cf, StrEq(src_url)))
-      .Times(testing::AtMost(2));
+      .Times(testing::Between(1, 2))
+      .WillOnce(DelayRefresh(&ie_mock_, &loop_, 50))
+      .WillOnce(testing::Return());
 
-  LaunchIEAndNavigate(src_url);
+  LaunchIENavigateAndLoop(src_url,
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Link navigations in the same domain specified with the noreferrer flag
@@ -1261,7 +1268,7 @@ TEST_F(FullTabNavigationTest, JavascriptWindowOpenNoReferrerOpensInHost) {
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
   LaunchIENavigateAndLoop(initial_url,
-                          kChromeFrameLongNavigationTimeoutInSeconds);
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 }  // namespace chrome_frame_test
