@@ -6,6 +6,7 @@
 #include <wininet.h>
 
 #include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/path_service.h"
 #include "base/win/scoped_handle.h"
 #include "chrome_frame/test/test_server.h"
@@ -14,9 +15,10 @@
 #include "net/disk_cache/disk_cache.h"
 #include "net/http/http_auth_handler_factory.h"
 #include "net/http/http_cache.h"
+#include "net/http/http_network_session.h"
 #include "net/proxy/proxy_service.h"
 #include "net/url_request/url_request.h"
-#include "net/url_request/url_request_unittest.h"
+#include "net/url_request/url_request_test_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 class TestServerTest: public testing::Test {
@@ -58,48 +60,11 @@ class ScopedInternet {
   HINTERNET h_;
 };
 
-class URLRequestTestContext : public net::URLRequestContext {
- public:
-  URLRequestTestContext() {
-    host_resolver_ =
-        net::CreateSystemHostResolver(net::HostResolver::kDefaultParallelism,
-                                      NULL, NULL);
-    cert_verifier_ = new net::CertVerifier;
-    proxy_service_ = net::ProxyService::CreateDirect();
-    ssl_config_service_ = new net::SSLConfigServiceDefaults;
-    http_auth_handler_factory_ = net::HttpAuthHandlerFactory::CreateDefault(
-        host_resolver_);
-    http_transaction_factory_ = new net::HttpCache(
-        net::HttpNetworkLayer::CreateFactory(
-            host_resolver_,
-            cert_verifier_,
-            NULL /* dnsrr_resolver */,
-            NULL /* dns_cert_checker */,
-            NULL /* ssl_host_info_factory */,
-            proxy_service_,
-            ssl_config_service_,
-            http_auth_handler_factory_,
-            NULL /* network_delegate */,
-            NULL /* net_log */),
-        NULL /* net_log */,
-        net::HttpCache::DefaultBackend::InMemory(0));
-    // In-memory cookie store.
-    cookie_store_ = new net::CookieMonster(NULL, NULL);
-  }
-
-  virtual ~URLRequestTestContext() {
-    delete http_transaction_factory_;
-    delete http_auth_handler_factory_;
-    delete cert_verifier_;
-    delete host_resolver_;
-  }
-};
-
 class TestURLRequest : public net::URLRequest {
  public:
   TestURLRequest(const GURL& url, Delegate* delegate)
       : net::URLRequest(url, delegate) {
-    set_context(new URLRequestTestContext());
+    set_context(new TestURLRequestContext());
   }
 };
 
@@ -159,7 +124,7 @@ struct QuitMessageHit {
 
 void QuitMessageLoop(QuitMessageHit* msg) {
   msg->hit_ = true;
-  msg->loop_->PostTask(FROM_HERE, new MessageLoop::QuitTask);
+  msg->loop_->PostTask(FROM_HERE, MessageLoop::QuitClosure());
 }
 
 }  // end namespace
@@ -181,8 +146,7 @@ TEST_F(TestServerTest, TestServer) {
   // We should never hit this, but it's our way to break out of the test if
   // things start hanging.
   QuitMessageHit quit_msg(&loop);
-  loop.PostDelayedTask(FROM_HERE,
-                       NewRunnableFunction(QuitMessageLoop, &quit_msg),
+  loop.PostDelayedTask(FROM_HERE, base::Bind(QuitMessageLoop, &quit_msg),
                        10 * 1000);
 
   UrlTaskChain quit_task("http://localhost:1337/quit", NULL);
