@@ -13,14 +13,12 @@
 #include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/scoped_handle.h"
+#include "chrome_frame/chrome_tab.h"
 #include "chrome_frame/test/chrome_frame_test_utils.h"
 #include "chrome_frame/test/test_server.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
-// Include without path to make GYP build see it.
-#include "chrome_tab.h"  // NOLINT
-
 // Specifies the invocation method for CF.
 class CFInvocation {
  public:
@@ -140,7 +138,7 @@ class MockWebServer : public test_server::HTTPTestServer {
 
 // Class that:
 // 1) Starts the local webserver,
-// 2) Supports launching browsers - Internet Explorer and Firefox with local url
+// 2) Supports launching browsers - Internet Explorer with local url
 // 3) Wait the webserver to finish. It is supposed the test webpage to shutdown
 //    the server by navigating to "kill" page
 // 4) Supports read the posted results from the test webpage to the "dump"
@@ -150,7 +148,7 @@ class ChromeFrameTestWithWebServer: public testing::Test {
   ChromeFrameTestWithWebServer();
 
  protected:
-  enum BrowserKind { INVALID, IE, FIREFOX, OPERA, SAFARI, CHROME };
+  enum BrowserKind { INVALID, IE, CHROME };
 
   bool LaunchBrowser(BrowserKind browser, const wchar_t* url);
   bool WaitForTestToComplete(int milliseconds);
@@ -167,19 +165,9 @@ class ChromeFrameTestWithWebServer: public testing::Test {
       const wchar_t* page, const char* result);
   void SimpleBrowserTest(BrowserKind browser, const wchar_t* page);
 
-  // Same as SimpleBrowserTest but if the browser isn't installed (LaunchBrowser
-  // fails), the function will print out a warning but not treat the test
-  // as failed.
-  // Currently this is how we run Opera tests.
-  void OptionalBrowserTest(BrowserKind browser, const wchar_t* page);
-
   // Test if chrome frame correctly reports its version.
   void VersionTest(BrowserKind browser, const wchar_t* page);
 
-  // Test if chrome frame correctly reports its session Id.
-  void SessionIdTest(BrowserKind browser, const wchar_t* page,
-                     int privilege_mode, const char* expected_result);
-
   // Closes all browsers in preparation for a test and during cleanup.
   void CloseAllBrowsers();
 
@@ -200,14 +188,8 @@ class ChromeFrameTestWithWebServer: public testing::Test {
     switch (kind) {
       case IE:
         return "IE";
-      case FIREFOX:
-        return "Firefox";
-      case OPERA:
-        return "Opera";
       case CHROME:
         return "Chrome";
-      case SAFARI:
-        return "Safari";
       default:
         NOTREACHED();
         break;
