@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -67,7 +67,7 @@ TEST_P(FullTabUITest, KeyboardInput) {
 }
 
 // Tests keyboard shortcuts for back and forward.
-TEST_P(FullTabUITest, KeyboardBackForward) {
+TEST_P(FullTabUITest, FLAKY_KeyboardBackForward) {
   if (IsWorkstationLocked()) {
     LOG(ERROR) << "This test cannot be run in a locked workstation.";
     return;
@@ -103,7 +103,7 @@ TEST_P(FullTabUITest, KeyboardBackForward) {
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
   LaunchIENavigateAndLoop(page1,
-                          kChromeFrameLongNavigationTimeoutInSeconds * 2);
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Tests new window behavior with ctrl+N.
@@ -133,16 +133,15 @@ TEST_P(FullTabUITest, CtrlN) {
   // Watch for new window. It appears that the window close message cannot be
   // reliably delivered immediately upon receipt of the window open event.
   EXPECT_CALL(win_observer_mock, OnWindowOpen(_))
-      .Times(testing::AtMost(2))
-      .WillOnce(DelayDoCloseWindow(500))
-      .WillOnce(testing::Return());
-
-  EXPECT_CALL(win_observer_mock, OnWindowClose(_))
       .Times(testing::AtMost(2))
       .WillOnce(CloseBrowserMock(&ie_mock_))
       .WillOnce(testing::Return());
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  EXPECT_CALL(win_observer_mock, OnWindowClose(_))
+      .Times(testing::AtMost(2));
+
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test that Ctrl+F opens the Find dialog.
@@ -168,15 +167,11 @@ TEST_P(FullTabUITest, CtrlF) {
           SetFocusToRenderer(&ie_mock_),
           DelaySendChar(&loop_, 1500, 'f', simulate_input::CONTROL)));
 
-  // Watch for find dialog. It appears that the window close message cannot be
-  // reliably delivered immediately upon receipt of the window open event.
   EXPECT_CALL(win_observer_mock, OnWindowOpen(_))
-      .WillOnce(DelayDoCloseWindow(500));
-
-  EXPECT_CALL(win_observer_mock, OnWindowClose(_))
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test that ctrl+r does cause a refresh.
@@ -199,7 +194,8 @@ TEST_P(FullTabUITest, CtrlR) {
           DelayCloseBrowserMock(&loop_, 4000, &ie_mock_)))
       .WillRepeatedly(testing::Return());
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test window close with ctrl+w.
@@ -215,7 +211,8 @@ TEST_P(FullTabUITest, CtrlW) {
           SetFocusToRenderer(&ie_mock_),
           DelaySendChar(&loop_, 1000, 'w', simulate_input::CONTROL)));
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test address bar navigation with Alt+d and URL.
@@ -235,11 +232,13 @@ TEST_P(FullTabUITest, AltD) {
                                StrEq(GetLinkPageUrl())))
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Tests that the renderer has focus after navigation.
-TEST_P(FullTabUITest, RendererHasFocus) {
+// Flaky, see http://crbug.com/90791 .
+TEST_P(FullTabUITest, FLAKY_RendererHasFocus) {
   EXPECT_CALL(ie_mock_, OnLoad(GetParam().invokes_cf(),
                                StrEq(GetSimplePageUrl())))
       .WillOnce(testing::DoAll(
@@ -347,7 +346,9 @@ TEST_P(FullTabUITest, TabCrashReload) {
 }
 
 // Tests if Chrome gets restarted after a crash by just refreshing the document.
-TEST_P(FullTabUITest, TabCrashRefresh) {
+// DISABLED as per bug http://crbug.com/99317 (one of the failures is a
+// timeout, which marking as FLAKY or FAILS won't mask).
+TEST_P(FullTabUITest, DISABLED_TabCrashRefresh) {
   using testing::DoAll;
 
   if (!GetParam().invokes_cf()) {
@@ -379,6 +380,32 @@ TEST_P(FullTabUITest, TabCrashRefresh) {
   LaunchIEAndNavigate(GetSimplePageUrl());
 }
 
+// Test that window.print() on a page results in the native Windows print dialog
+// appearing rather than Chrome's in-page print preview.
+TEST_P(FullTabUITest, WindowPrintOpensNativePrintDialog) {
+  std::wstring window_print_url(GetTestUrl(L"window_print.html"));
+  std::wstring window_print_title(L"window.print");
+
+  const bool is_cf = GetParam().invokes_cf();
+  MockWindowObserver win_observer_mock;
+
+  // When the page is loaded, start watching for the Print dialog to appear.
+  EXPECT_CALL(ie_mock_, OnLoad(is_cf, StrEq(window_print_url)))
+      .WillOnce(WatchWindow(&win_observer_mock, "Print", ""));
+
+  // When the print dialog opens, close it.
+  EXPECT_CALL(win_observer_mock, OnWindowOpen(_))
+      .WillOnce(DoCloseWindow());
+
+  // When the print dialog closes, close the browser.
+  EXPECT_CALL(win_observer_mock, OnWindowClose(_))
+      .WillOnce(CloseBrowserMock(&ie_mock_));
+
+  // Launch IE and navigate to the window_print.html page, which will
+  // window.print() immediately after loading.
+  LaunchIEAndNavigate(window_print_url);
+}
+
 // Test fixture for tests related to the context menu UI. Since the context
 // menus for CF and IE are different, these tests are not parameterized.
 class ContextMenuTest : public MockIEEventSinkTest, public testing::Test {
@@ -426,9 +453,10 @@ class ContextMenuTest : public MockIEEventSinkTest, public testing::Test {
             AccDoDefaultAction(AccObjectMatcher(L"Save", L"push button"))));
 
     EXPECT_CALL(win_observer_mock, OnWindowClose(_))
-        .WillOnce(CloseWhenFileSaved(&ie_mock_, temp_file_path, 5000));
+        .WillOnce(CloseWhenFileSaved(&ie_mock_, temp_file_path, 8000));
 
-    LaunchIEAndNavigate(GetTestUrl(L"save_as_context_menu.html"));
+    LaunchIENavigateAndLoop(GetTestUrl(L"save_as_context_menu.html"),
+                            kChromeFrameVeryLongNavigationTimeoutInSeconds);
     ASSERT_TRUE(file_util::DieFileDie(temp_file_path, false));
   }
 
@@ -501,7 +529,7 @@ TEST_F(ContextMenuTest, CFViewSource) {
   LaunchIEAndNavigate(GetSimplePageUrl());
 }
 
-TEST_F(ContextMenuTest, CFPageInfo) {
+TEST_F(ContextMenuTest, DISABLED_CFPageInfo) {
   server_mock_.ExpectAndServeAnyRequests(CFInvocation::MetaTag());
   MockWindowObserver win_observer_mock;
   InSequence expect_in_sequence_for_scope;
@@ -548,10 +576,11 @@ TEST_F(ContextMenuTest, CFInspector) {
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
   LaunchIENavigateAndLoop(GetSimplePageUrl(),
-                          kChromeFrameLongNavigationTimeoutInSeconds * 2);
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
-TEST_F(ContextMenuTest, CFSavePageAs) {
+// http://code.google.com/p/chromium/issues/detail?id=83114
+TEST_F(ContextMenuTest, FLAKY_CFSavePageAs) {
   // Please see http://code.google.com/p/chromium/issues/detail?id=60987
   // for more information on why this test is disabled for Vista with IE7.
   if (base::win::GetVersion() == base::win::VERSION_VISTA &&
@@ -562,7 +591,8 @@ TEST_F(ContextMenuTest, CFSavePageAs) {
   ASSERT_NO_FATAL_FAILURE(DoSaveAsTest(L"", L"Save as...", L".html"));
 }
 
-TEST_F(ContextMenuTest, CFSaveLinkAs) {
+// http://code.google.com/p/chromium/issues/detail?id=83114
+TEST_F(ContextMenuTest, DISABLED_CFSaveLinkAs) {
   // Please see http://code.google.com/p/chromium/issues/detail?id=60987
   // for more information on why this test is disabled for Vista with IE7.
   if (base::win::GetVersion() == base::win::VERSION_VISTA &&
@@ -630,6 +660,11 @@ TEST_F(ContextMenuTest, IEOpen) {
 }
 
 TEST_F(ContextMenuTest, IEOpenInNewWindow) {
+  // See crbug.com/64794.
+  if (GetInstalledIEVersion() == IE_7) {
+    LOG(INFO) << "Not running test with IE7";
+    return;
+  }
   server_mock_.ExpectAndServeAnyRequests(CFInvocation::None());
   MockIEEventSink new_window_mock;
   new_window_mock.ExpectAnyNavigations();
@@ -688,18 +723,15 @@ TEST_F(ContextMenuTest, IEBackForward) {
 }
 
 // Test CF link context menu - Open link in new window.
-TEST_F(ContextMenuTest, CFOpenLinkInNewWindow) {
-  // See crbug.com/64794.
-  if (GetInstalledIEVersion() == IE_7) {
-    LOG(INFO) << "Not running test with IE7";
-    return;
-  }
+// Failing intermittently on IE6/7. See crbug.com/64794.
+TEST_F(ContextMenuTest, FLAKY_CFOpenLinkInNewWindow) {
   server_mock_.ExpectAndServeAnyRequests(CFInvocation::MetaTag());
   MockIEEventSink new_window_mock;
   new_window_mock.ExpectAnyNavigations();
 
   // Invoke 'Open link in new window' context menu item.
   EXPECT_CALL(acc_observer_, OnAccDocLoad(_))
+      .Times(testing::AtMost(2))
       .WillOnce(AccRightClick(AccObjectMatcher(L"", L"link")))
       .WillOnce(testing::Return());
   EXPECT_CALL(acc_observer_, OnMenuPopup(_))
@@ -778,7 +810,7 @@ TEST_F(ContextMenuTest, CFTxtFieldCopy) {
 }
 
 // Test CF text field context menu - paste.
-TEST_F(ContextMenuTest, CFTxtFieldPaste) {
+TEST_F(ContextMenuTest, DISABLED_CFTxtFieldPaste) {
   server_mock_.ExpectAndServeAnyRequests(CFInvocation::MetaTag());
   AccObjectMatcher txtfield_matcher(L"", L"editable text");
 
@@ -959,7 +991,8 @@ TEST_F(ContextMenuTest, CFBackForward) {
   EXPECT_CALL(ie_mock_, OnLoad(IN_CF, StrEq(page3)))
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
-  LaunchIEAndNavigate(page1);
+  LaunchIENavigateAndLoop(page1,
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 }  // namespace chrome_frame_test
