@@ -1,29 +1,34 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome_frame/urlmon_url_request.h"
 
-#include <wininet.h>
 #include <urlmon.h>
+#include <wininet.h>
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/logging.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/scoped_ptr.h"
 #include "base/string_number_conversions.h"
 #include "base/stringprintf.h"
 #include "base/threading/platform_thread.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/common/automation_messages.h"
 #include "chrome_frame/bind_context_info.h"
 #include "chrome_frame/chrome_frame_activex_base.h"
 #include "chrome_frame/extra_system_apis.h"
 #include "chrome_frame/html_utils.h"
-#include "chrome_frame/urlmon_url_request_private.h"
 #include "chrome_frame/urlmon_upload_data_stream.h"
+#include "chrome_frame/urlmon_url_request_private.h"
 #include "chrome_frame/utils.h"
 #include "net/base/load_flags.h"
-#include "net/http/http_util.h"
 #include "net/http/http_response_headers.h"
+#include "net/http/http_util.h"
+
+#define IS_HTTP_SUCCESS_CODE(code) (code >= 200 && code <= 299)
 
 UrlmonUrlRequest::UrlmonUrlRequest()
     : pending_read_size_(0),
@@ -51,6 +56,10 @@ bool UrlmonUrlRequest::Start() {
   DCHECK(thread_ == 0 || thread_ == base::PlatformThread::CurrentId());
   thread_ = base::PlatformThread::CurrentId();
   status_.Start();
+  // Initialize the net::HostPortPair structure from the url initially. We may
+  // not receive the ip address of the host if the request is satisfied from
+  // the cache.
+  socket_address_ = net::HostPortPair::FromURL(GURL(url()));
   // The UrlmonUrlRequest instance can get destroyed in the context of
   // StartAsyncDownload if BindToStorage finishes synchronously with an error.
   // Grab a reference to protect against this.
@@ -156,21 +165,20 @@ HRESULT UrlmonUrlRequest::InitPending(const GURL& url, IMoniker* moniker,
   return S_OK;
 }
 
-void UrlmonUrlRequest::TerminateBind(TerminateBindCallback* callback) {
+void UrlmonUrlRequest::TerminateBind(const TerminateBindCallback& callback) {
   DCHECK_EQ(thread_, base::PlatformThread::CurrentId());
   DVLOG(1) << __FUNCTION__ << me();
   cleanup_transaction_ = false;
   if (status_.get_state() == Status::DONE) {
     // Binding is stopped. Note result could be an error.
-    callback->Run(moniker_, bind_context_, upload_data_,
-                  request_headers_.c_str());
-    delete callback;
+    callback.Run(moniker_, bind_context_, upload_data_,
+                 request_headers_.c_str());
   } else {
     // WORKING (ABORTING?). Save the callback.
     // Now we will return INET_TERMINATE_BIND from ::OnDataAvailable() and in
     // ::OnStopBinding will invoke the callback passing our moniker and
     // bind context.
-    terminate_bind_callback_.reset(callback);
+    terminate_bind_callback_ = callback;
     if (pending_data_) {
       // For downloads to work correctly, we must induce a call to
       // OnDataAvailable so that we can download INET_E_TERMINATED_BIND and
@@ -180,7 +188,7 @@ void UrlmonUrlRequest::TerminateBind(TerminateBindCallback* callback) {
       // should get E_PENDING back and then later we'll get that call
       // to OnDataAvailable.
       std::string data;
-      ScopedComPtr<IStream> read_stream(pending_data_);
+      base::win::ScopedComPtr<IStream> read_stream(pending_data_);
       HRESULT hr;
       while ((hr = ReadStream(read_stream, 0xffff, &data)) == S_OK) {
         // Just drop the data.
@@ -214,7 +222,7 @@ size_t UrlmonUrlRequest::SendDataToDelegate(size_t bytes_to_read) {
       // where we can get a call to OnDataAvailable while inside Read and
       // in our OnDataAvailable call, we can release the stream object
       // while still using it.
-      ScopedComPtr<IStream> pending(pending_data_);
+      base::win::ScopedComPtr<IStream> pending(pending_data_);
       HRESULT hr = ReadStream(pending, bytes_to_read, &read_data);
       if (read_data.empty())
         pending_read_size_ = pending_data_read_save;
@@ -282,11 +290,18 @@ STDMETHODIMP UrlmonUrlRequest::OnProgress(ULONG progress, ULONG max_progress,
   }
 
   switch (status_code) {
+    case BINDSTATUS_CONNECTING: {
+      if (status_text) {
+        socket_address_.set_host(WideToUTF8(status_text));
+      }
+      break;
+    }
+
     case BINDSTATUS_REDIRECTING: {
       // If we receive a redirect for the initial pending request initiated
       // when our document loads we should stash it away and inform Chrome
       // accordingly when it requests data for the original URL.
-      ScopedComPtr<BindContextInfo> info;
+      base::win::ScopedComPtr<BindContextInfo> info;
       BindContextInfo::FromBindContext(bind_context_, info.Receive());
       DCHECK(info);
       GURL previously_redirected(info ? info->GetUrl() : std::wstring());
@@ -356,8 +371,8 @@ STDMETHODIMP UrlmonUrlRequest::OnStopBinding(HRESULT result, LPCWSTR error) {
 
   if (result == INET_E_TERMINATED_BIND) {
     if (terminate_requested()) {
-      terminate_bind_callback_->Run(moniker_, bind_context_, upload_data_,
-                                    request_headers_.c_str());
+      terminate_bind_callback_.Run(moniker_, bind_context_, upload_data_,
+                                   request_headers_.c_str());
     } else {
       cleanup_transaction_ = true;
     }
@@ -368,13 +383,25 @@ STDMETHODIMP UrlmonUrlRequest::OnStopBinding(HRESULT result, LPCWSTR error) {
   if (state == Status::WORKING) {
     status_.set_result(result);
 
-    // Special case. If the last request was a redirect and the current OS
-    // error value is E_ACCESSDENIED, that means an unsafe redirect was
-    // attempted. In that case, correct the OS error value to be the more
-    // specific ERR_UNSAFE_REDIRECT error value.
-    if (result == E_ACCESSDENIED) {
+    if (FAILED(result)) {
       int http_code = GetHttpResponseStatusFromBinding(binding_);
-      if (300 <= http_code && http_code < 400) {
+      // For certain requests like empty POST requests the server can return
+      // back a HTTP success code in the range 200 to 299. We need to flag
+      // these requests as succeeded.
+      if (IS_HTTP_SUCCESS_CODE(http_code)) {
+        // If this DCHECK fires it means that the server returned a HTTP
+        // success code outside the standard range 200-206. We need to confirm
+        // if the following code path is correct.
+        DCHECK_LE(http_code, 206);
+        status_.set_result(S_OK);
+        std::string headers = GetHttpHeadersFromBinding(binding_);
+        OnResponse(0, UTF8ToWide(headers).c_str(), NULL, NULL);
+      } else if (net::HttpResponseHeaders::IsRedirectResponseCode(http_code) &&
+                 result == E_ACCESSDENIED) {
+        // Special case. If the last request was a redirect and the current OS
+        // error value is E_ACCESSDENIED, that means an unsafe redirect was
+        // attempted. In that case, correct the OS error value to be the more
+        // specific ERR_UNSAFE_REDIRECT error value.
         status_.set_result(net::URLRequestStatus::FAILED,
                            net::ERR_UNSAFE_REDIRECT);
       }
@@ -430,6 +457,9 @@ STDMETHODIMP UrlmonUrlRequest::GetBindInfo(DWORD* bind_flags,
 
   *bind_flags = BINDF_ASYNCHRONOUS | BINDF_ASYNCSTORAGE | BINDF_PULLDATA;
 
+  bind_info->dwOptionsFlags = INTERNET_FLAG_NO_AUTO_REDIRECT;
+  bind_info->dwOptions = BINDINFO_OPTIONS_WININETFLAG;
+
   // TODO(ananta)
   // Look into whether the other load flags need to be supported in chrome
   // frame.
@@ -453,7 +483,9 @@ STDMETHODIMP UrlmonUrlRequest::GetBindInfo(DWORD* bind_flags,
     lstrcpyW(bind_info->szCustomVerb, verb.c_str());
   }
 
-  if (post_data_len()) {
+  if (bind_info->dwBindVerb == BINDVERB_POST ||
+      bind_info->dwBindVerb == BINDVERB_PUT ||
+      post_data_len() > 0) {
     // Bypass caching proxies on upload requests and avoid writing responses to
     // the browser's cache.
     *bind_flags |= BINDF_GETNEWESTVERSION | BINDF_PRAGMA_NO_CACHE;
@@ -470,8 +502,12 @@ STDMETHODIMP UrlmonUrlRequest::GetBindInfo(DWORD* bind_flags,
     if (bind_info->dwBindVerb != BINDVERB_CUSTOM)
       bind_info->szCustomVerb = NULL;
 
-    if (get_upload_data(&bind_info->stgmedData.pstm) == S_OK) {
+    if (post_data_len() &&
+        get_upload_data(&bind_info->stgmedData.pstm) == S_OK) {
       bind_info->stgmedData.tymed = TYMED_ISTREAM;
+#pragma warning(disable:4244)
+      bind_info->cbstgmedData = post_data_len();
+#pragma warning(default:4244)
       DVLOG(1) << __FUNCTION__ << me() << method()
                << " request with " << base::Int64ToString(post_data_len())
                << " bytes. url=" << url();
@@ -479,7 +515,6 @@ STDMETHODIMP UrlmonUrlRequest::GetBindInfo(DWORD* bind_flags,
       DVLOG(1) << __FUNCTION__ << me() << "POST request with no data!";
     }
   }
-
   return S_OK;
 }
 
@@ -568,11 +603,9 @@ STDMETHODIMP UrlmonUrlRequest::BeginningTransaction(const wchar_t* url,
 
   std::string new_headers;
   if (post_data_len() > 0) {
-    // Tack on the Content-Length header since when using an IStream type
-    // STGMEDIUM, it looks like it doesn't get set for us :(
-    new_headers = base::StringPrintf(
-        "Content-Length: %s\r\n",
-        base::Int64ToString(post_data_len()).c_str());
+    if (is_chunked_upload()) {
+      new_headers = base::StringPrintf("Transfer-Encoding: chunked\r\n");
+    }
   }
 
   if (!extra_headers().empty()) {
@@ -616,17 +649,20 @@ STDMETHODIMP UrlmonUrlRequest::OnResponse(DWORD dwResponseCode,
     const wchar_t* response_headers, const wchar_t* request_headers,
     wchar_t** additional_headers) {
   DCHECK_EQ(thread_, base::PlatformThread::CurrentId());
-  DVLOG(1) << __FUNCTION__ << me() << "headers: \n" << response_headers;
+  DVLOG(1) << __FUNCTION__ << me() << "headers: \n"
+           << (response_headers == NULL ? L"EMPTY" : response_headers);
 
   if (!delegate_) {
     DLOG(WARNING) << "Invalid delegate";
     return S_OK;
   }
 
-  std::string raw_headers = WideToUTF8(response_headers);
-
   delegate_->AddPrivacyDataForUrl(url(), "", 0);
 
+  std::string raw_headers;
+  if (response_headers)
+    raw_headers = WideToUTF8(response_headers);
+
   // Security check for frame busting headers. We don't honor the headers
   // as-such, but instead simply kill requests which we've been asked to
   // look for if they specify a value for "X-Frame-Options" other than
@@ -662,7 +698,8 @@ STDMETHODIMP UrlmonUrlRequest::OnResponse(DWORD dwResponseCode,
                     0,                    // size
                     base::Time(),         // last_modified
                     status_.get_redirection().utf8_url,
-                    status_.get_redirection().http_code);
+                    status_.get_redirection().http_code,
+                    socket_address_);
   return S_OK;
 }
 
@@ -791,16 +828,16 @@ HRESULT UrlmonUrlRequest::StartAsyncDownload() {
   }
 
   if (SUCCEEDED(hr)) {
-    ScopedComPtr<IStream> stream;
+    base::win::ScopedComPtr<IStream> stream;
 
     // BindToStorage may complete synchronously.
     // We still get all the callbacks - OnStart/StopBinding, this may result
     // in destruction of our object. It's fine but we access some members
     // below for debug info. :)
-    ScopedComPtr<IHttpSecurity> self(this);
+    base::win::ScopedComPtr<IHttpSecurity> self(this);
 
     // Inform our moniker patch this binding should not be tortured.
-    ScopedComPtr<BindContextInfo> info;
+    base::win::ScopedComPtr<BindContextInfo> info;
     BindContextInfo::FromBindContext(bind_context_, info.Receive());
     DCHECK(info);
     if (info)
@@ -837,7 +874,7 @@ void UrlmonUrlRequest::NotifyDelegateAndDie() {
   delegate_ = NULL;
   ReleaseBindings();
   TerminateTransaction();
-  if (delegate) {
+  if (delegate && id() != -1) {
     net::URLRequestStatus result = status_.get_result();
     delegate->OnResponseEnd(id(), result);
   } else {
@@ -857,7 +894,7 @@ void UrlmonUrlRequest::TerminateTransaction() {
     // on this with the special flags 0x2000000 cleanly releases the
     // transaction.
     static const int kUrlmonTerminateTransactionFlags = 0x2000000;
-    ScopedComPtr<BindContextInfo> info;
+    base::win::ScopedComPtr<BindContextInfo> info;
     BindContextInfo::FromBindContext(bind_context_, info.Receive());
     DCHECK(info);
     if (info && info->protocol()) {
@@ -877,6 +914,7 @@ void UrlmonUrlRequest::ReleaseBindings() {
 }
 
 net::Error UrlmonUrlRequest::HresultToNetError(HRESULT hr) {
+  const int kInvalidHostName = 0x8007007b;
   // Useful reference:
   // http://msdn.microsoft.com/en-us/library/ms775145(VS.85).aspx
 
@@ -912,9 +950,15 @@ net::Error UrlmonUrlRequest::HresultToNetError(HRESULT hr) {
       ret = net::ERR_TUNNEL_CONNECTION_FAILED;
       break;
 
+    // The following error codes can be returned while processing an invalid
+    // url. http://msdn.microsoft.com/en-us/library/bb250493(v=vs.85).aspx
     case INET_E_INVALID_URL:
     case INET_E_UNKNOWN_PROTOCOL:
     case INET_E_REDIRECT_FAILED:
+    case INET_E_SECURITY_PROBLEM:
+    case kInvalidHostName:
+    case E_INVALIDARG:
+    case E_OUTOFMEMORY:
       ret = net::ERR_INVALID_URL;
       break;
 
@@ -950,7 +994,7 @@ void UrlmonUrlRequestManager::SetInfoForUrl(const std::wstring& url,
     DCHECK(start_url.is_valid());
     DCHECK(pending_request_ == NULL);
 
-    ScopedComPtr<BindContextInfo> info;
+    base::win::ScopedComPtr<BindContextInfo> info;
     BindContextInfo::FromBindContext(bind_ctx, info.Receive());
     DCHECK(info);
     IStream* cache = info ? info->cache() : NULL;
@@ -967,7 +1011,6 @@ void UrlmonUrlRequestManager::SetInfoForUrl(const std::wstring& url,
 void UrlmonUrlRequestManager::StartRequest(int request_id,
     const AutomationURLRequest& request_info) {
   DVLOG(1) << __FUNCTION__ << " id: " << request_id;
-  DCHECK_EQ(0, calling_delegate_);
 
   if (stopping_) {
     DLOG(WARNING) << __FUNCTION__ << " request not started (stopping)";
@@ -975,28 +1018,59 @@ void UrlmonUrlRequestManager::StartRequest(int request_id,
   }
 
   DCHECK(request_map_.find(request_id) == request_map_.end());
+#ifndef NDEBUG
+  if (background_worker_thread_enabled_) {
+    base::AutoLock lock(background_resource_map_lock_);
+    DCHECK(background_request_map_.find(request_id) ==
+           background_request_map_.end());
+  }
+#endif  // NDEBUG
   DCHECK(GURL(request_info.url).is_valid());
 
+  // Non frame requests like sub resources, images, etc are handled on the
+  // background thread.
+  if (background_worker_thread_enabled_ &&
+      !ResourceType::IsFrame(
+          static_cast<ResourceType::Type>(request_info.resource_type))) {
+    DLOG(INFO) << "Downloading resource type "
+               << request_info.resource_type
+               << " on background thread";
+    background_thread_->message_loop()->PostTask(
+        FROM_HERE,
+        base::Bind(&UrlmonUrlRequestManager::StartRequestHelper,
+                   base::Unretained(this), request_id, request_info,
+                   &background_request_map_, &background_resource_map_lock_));
+    return;
+  }
+  StartRequestHelper(request_id, request_info, &request_map_, NULL);
+}
+
+void UrlmonUrlRequestManager::StartRequestHelper(
+    int request_id,
+    const AutomationURLRequest& request_info,
+    RequestMap* request_map,
+    base::Lock* request_map_lock) {
+  DCHECK(request_map);
   scoped_refptr<UrlmonUrlRequest> new_request;
   bool is_started = false;
   if (pending_request_) {
     if (pending_request_->url() != request_info.url) {
-      DLOG(WARNING) << __FUNCTION__
-                    << "Received unexpected url request for url:"
-                    << request_info.url
-                    << ".Pending url request for url:"
-                    << pending_request_->url()
-                    << " was expected.";
-      net::URLRequestStatus result;
-      result.set_status(net::URLRequestStatus::FAILED);
-      OnResponseEnd(request_id, result);
-      return;
+      DLOG(INFO) << __FUNCTION__
+                 << "Received url request for url:"
+                 << request_info.url
+                 << ". Stopping pending url request for url:"
+                 << pending_request_->url();
+      pending_request_->Stop();
+      pending_request_ = NULL;
+    } else {
+      new_request.swap(pending_request_);
+      is_started = true;
+      DVLOG(1) << __FUNCTION__ << new_request->me()
+               << " assigned id " << request_id;
     }
-    new_request.swap(pending_request_);
-    is_started = true;
-    DVLOG(1) << __FUNCTION__ << new_request->me()
-             << " assigned id " << request_id;
-  } else {
+  }
+
+  if (!is_started) {
     CComObject<UrlmonUrlRequest>* created_request = NULL;
     CComObject<UrlmonUrlRequest>::CreateInstance(&created_request);
     new_request = created_request;
@@ -1015,7 +1089,13 @@ void UrlmonUrlRequestManager::StartRequest(int request_id,
   new_request->set_parent_window(notification_window_);
   new_request->set_privileged_mode(privileged_mode_);
 
-  request_map_[request_id] = new_request;
+  if (request_map_lock)
+    request_map_lock->Acquire();
+
+  (*request_map)[request_id] = new_request;
+
+  if (request_map_lock)
+    request_map_lock->Release();
 
   if (!is_started) {
     // Freshly created, start now.
@@ -1031,28 +1111,59 @@ void UrlmonUrlRequestManager::StartRequest(int request_id,
 
 void UrlmonUrlRequestManager::ReadRequest(int request_id, int bytes_to_read) {
   DVLOG(1) << __FUNCTION__ << " id: " << request_id;
-  DCHECK_EQ(0, calling_delegate_);
-  scoped_refptr<UrlmonUrlRequest> request = LookupRequest(request_id);
-  // if zero, it may just have had network error.
-  if (request)
+  // if we fail to find the request in the normal map and the background
+  // request map, it may mean that the request could have failed with a
+  // network error.
+  scoped_refptr<UrlmonUrlRequest> request = LookupRequest(request_id,
+                                                          &request_map_);
+  if (request) {
     request->Read(bytes_to_read);
+  } else if (background_worker_thread_enabled_) {
+    base::AutoLock lock(background_resource_map_lock_);
+    request = LookupRequest(request_id, &background_request_map_);
+    if (request) {
+      background_thread_->message_loop()->PostTask(
+          FROM_HERE, base::Bind(base::IgnoreResult(&UrlmonUrlRequest::Read),
+                                request.get(), bytes_to_read));
+    }
+  }
+  if (!request)
+    DLOG(ERROR) << __FUNCTION__ << " no request found for " << request_id;
 }
 
 void UrlmonUrlRequestManager::DownloadRequestInHost(int request_id) {
   DVLOG(1) << __FUNCTION__ << " " << request_id;
-  if (IsWindow(notification_window_)) {
-    scoped_refptr<UrlmonUrlRequest> request(LookupRequest(request_id));
-    if (request) {
-      UrlmonUrlRequest::TerminateBindCallback* callback = NewCallback(this,
-          &UrlmonUrlRequestManager::BindTerminated);
-      request->TerminateBind(callback);
-    } else {
-      NOTREACHED();
-    }
-  } else {
+  if (!IsWindow(notification_window_)) {
     NOTREACHED() << "Cannot handle download if we don't have anyone to hand it "
                     "to.";
+    return;
   }
+
+  scoped_refptr<UrlmonUrlRequest> request(LookupRequest(request_id,
+                                                        &request_map_));
+  if (request) {
+    DownloadRequestInHostHelper(request);
+  } else if (background_worker_thread_enabled_) {
+    base::AutoLock lock(background_resource_map_lock_);
+    request = LookupRequest(request_id, &background_request_map_);
+    if (request) {
+      background_thread_->message_loop()->PostTask(
+          FROM_HERE,
+          base::Bind(&UrlmonUrlRequestManager::DownloadRequestInHostHelper,
+                     base::Unretained(this), request.get()));
+    }
+  }
+  if (!request)
+    DLOG(ERROR) << __FUNCTION__ << " no request found for " << request_id;
+}
+
+void UrlmonUrlRequestManager::DownloadRequestInHostHelper(
+    UrlmonUrlRequest* request) {
+  DCHECK(request);
+  UrlmonUrlRequest::TerminateBindCallback callback =
+      base::Bind(&UrlmonUrlRequestManager::BindTerminated,
+                 base::Unretained(this));
+  request->TerminateBind(callback);
 }
 
 void UrlmonUrlRequestManager::BindTerminated(IMoniker* moniker,
@@ -1126,14 +1237,22 @@ void UrlmonUrlRequestManager::SetCookiesForUrl(const GURL& url,
 
 void UrlmonUrlRequestManager::EndRequest(int request_id) {
   DVLOG(1) << __FUNCTION__ << " id: " << request_id;
-  DCHECK_EQ(0, calling_delegate_);
-  scoped_refptr<UrlmonUrlRequest> request = LookupRequest(request_id);
+  scoped_refptr<UrlmonUrlRequest> request = LookupRequest(request_id,
+                                                          &request_map_);
   if (request) {
     request_map_.erase(request_id);
     request->Stop();
-  } else {
-    DLOG(ERROR) << __FUNCTION__ << " no request found for " << request_id;
+  } else if (background_worker_thread_enabled_) {
+    base::AutoLock lock(background_resource_map_lock_);
+    request = LookupRequest(request_id, &background_request_map_);
+    if (request) {
+      background_request_map_.erase(request_id);
+      background_thread_->message_loop()->PostTask(
+          FROM_HERE, base::Bind(&UrlmonUrlRequest::Stop, request.get()));
+    }
   }
+  if (!request)
+    DLOG(ERROR) << __FUNCTION__ << " no request found for " << request_id;
 }
 
 void UrlmonUrlRequestManager::StopAll() {
@@ -1146,36 +1265,75 @@ void UrlmonUrlRequestManager::StopAll() {
   DVLOG(1) << __FUNCTION__ << " stopping " << request_map_.size()
            << " requests";
 
-  for (RequestMap::iterator it = request_map_.begin();
-       it != request_map_.end(); ++it) {
+  StopAllRequestsHelper(&request_map_, NULL);
+
+  if (background_worker_thread_enabled_) {
+    DCHECK(background_thread_.get());
+    background_thread_->message_loop()->PostTask(
+        FROM_HERE, base::Bind(&UrlmonUrlRequestManager::StopAllRequestsHelper,
+                              base::Unretained(this), &background_request_map_,
+                              &background_resource_map_lock_));
+    background_thread_->Stop();
+    background_thread_.reset();
+  }
+}
+
+void UrlmonUrlRequestManager::StopAllRequestsHelper(
+    RequestMap* request_map,
+    base::Lock* request_map_lock) {
+  DCHECK(request_map);
+
+  DVLOG(1) << __FUNCTION__ << " stopping " << request_map->size()
+           << " requests";
+
+  if (request_map_lock)
+    request_map_lock->Acquire();
+
+  for (RequestMap::iterator it = request_map->begin();
+       it != request_map->end(); ++it) {
     DCHECK(it->second != NULL);
     it->second->Stop();
   }
+  request_map->clear();
 
-  request_map_.clear();
+  if (request_map_lock)
+    request_map_lock->Release();
 }
 
 void UrlmonUrlRequestManager::OnResponseStarted(int request_id,
     const char* mime_type, const char* headers, int size,
     base::Time last_modified, const std::string& redirect_url,
-    int redirect_status) {
+    int redirect_status, const net::HostPortPair& socket_address) {
   DCHECK_NE(request_id, -1);
   DVLOG(1) << __FUNCTION__;
-  DCHECK(LookupRequest(request_id) != NULL);
-  ++calling_delegate_;
+
+#ifndef NDEBUG
+  scoped_refptr<UrlmonUrlRequest> request = LookupRequest(request_id,
+                                                          &request_map_);
+  if (request == NULL && background_worker_thread_enabled_) {
+    base::AutoLock lock(background_resource_map_lock_);
+    request = LookupRequest(request_id, &background_request_map_);
+  }
+  DCHECK(request != NULL);
+#endif  // NDEBUG
   delegate_->OnResponseStarted(request_id, mime_type, headers, size,
-      last_modified, redirect_url, redirect_status);
-  --calling_delegate_;
+      last_modified, redirect_url, redirect_status, socket_address);
 }
 
 void UrlmonUrlRequestManager::OnReadComplete(int request_id,
                                              const std::string& data) {
   DCHECK_NE(request_id, -1);
   DVLOG(1) << __FUNCTION__ << " id: " << request_id;
-  DCHECK(LookupRequest(request_id) != NULL);
-  ++calling_delegate_;
+#ifndef NDEBUG
+  scoped_refptr<UrlmonUrlRequest> request = LookupRequest(request_id,
+                                                          &request_map_);
+  if (request == NULL && background_worker_thread_enabled_) {
+    base::AutoLock lock(background_resource_map_lock_);
+    request = LookupRequest(request_id, &background_request_map_);
+  }
+  DCHECK(request != NULL);
+#endif  // NDEBUG
   delegate_->OnReadComplete(request_id, data);
-  --calling_delegate_;
   DVLOG(1) << __FUNCTION__ << " done id: " << request_id;
 }
 
@@ -1185,15 +1343,17 @@ void UrlmonUrlRequestManager::OnResponseEnd(
   DCHECK_NE(request_id, -1);
   DVLOG(1) << __FUNCTION__;
   DCHECK(status.status() != net::URLRequestStatus::CANCELED);
-  RequestMap::size_type n = request_map_.erase(request_id);
-  if (n != 1u) {
-    DLOG(WARNING) << __FUNCTION__
-                  << " Failed to find request id:"
-                  << request_id;
+  RequestMap::size_type erased_count = request_map_.erase(request_id);
+  if (erased_count != 1u && background_worker_thread_enabled_) {
+    base::AutoLock lock(background_resource_map_lock_);
+    erased_count = background_request_map_.erase(request_id);
+    if (erased_count != 1u) {
+      DLOG(WARNING) << __FUNCTION__
+                    << " Failed to find request id:"
+                    << request_id;
+    }
   }
-  ++calling_delegate_;
   delegate_->OnResponseEnd(request_id, status);
-  --calling_delegate_;
 }
 
 void UrlmonUrlRequestManager::OnCookiesRetrieved(bool success, const GURL& url,
@@ -1203,18 +1363,27 @@ void UrlmonUrlRequestManager::OnCookiesRetrieved(bool success, const GURL& url,
 }
 
 scoped_refptr<UrlmonUrlRequest> UrlmonUrlRequestManager::LookupRequest(
-    int request_id) {
-  RequestMap::iterator it = request_map_.find(request_id);
-  if (request_map_.end() != it)
+    int request_id, RequestMap* request_map) {
+  RequestMap::iterator it = request_map->find(request_id);
+  if (request_map->end() != it)
     return it->second;
-  DLOG(ERROR) << __FUNCTION__ << " no request found for " << request_id;
   return NULL;
 }
 
 UrlmonUrlRequestManager::UrlmonUrlRequestManager()
-    : stopping_(false), calling_delegate_(0), notification_window_(NULL),
+    : stopping_(false), notification_window_(NULL),
       privileged_mode_(false),
-      container_(NULL) {
+      container_(NULL),
+      background_worker_thread_enabled_(true) {
+  background_thread_.reset(new ResourceFetcherThread(
+      "cf_iexplore_background_thread"));
+  background_worker_thread_enabled_ =
+      GetConfigBool(true, kUseBackgroundThreadForSubResources);
+  if (background_worker_thread_enabled_) {
+    base::Thread::Options options;
+    options.message_loop_type = MessageLoop::TYPE_UI;
+    background_thread_->StartWithOptions(options);
+  }
 }
 
 UrlmonUrlRequestManager::~UrlmonUrlRequestManager() {
@@ -1228,7 +1397,7 @@ void UrlmonUrlRequestManager::AddPrivacyDataForUrl(
 
   bool fire_privacy_event = false;
 
-  if (privacy_info_.privacy_records.size() == 0)
+  if (privacy_info_.privacy_records.empty())
     flags |= PRIVACY_URLISTOPLEVEL;
 
   if (!privacy_info_.privacy_impacted) {
@@ -1250,3 +1419,19 @@ void UrlmonUrlRequestManager::AddPrivacyDataForUrl(
                 0);
   }
 }
+
+UrlmonUrlRequestManager::ResourceFetcherThread::ResourceFetcherThread(
+    const char* name) : base::Thread(name) {
+}
+
+UrlmonUrlRequestManager::ResourceFetcherThread::~ResourceFetcherThread() {
+  Stop();
+}
+
+void UrlmonUrlRequestManager::ResourceFetcherThread::Init() {
+  CoInitialize(NULL);
+}
+
+void UrlmonUrlRequestManager::ResourceFetcherThread::CleanUp() {
+  CoUninitialize();
+}
