@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,16 +7,14 @@
 
 #include <atlbase.h>
 #include <atlcom.h>
+
 #include <string>
 
-#include "base/callback.h"
+#include "base/gtest_prod_util.h"
 #include "base/threading/platform_thread.h"
 #include "net/base/net_errors.h"
 #include "net/http/http_response_headers.h"
 #include "net/url_request/url_request_status.h"
-#include "testing/gtest/include/gtest/gtest_prod.h"
-
-class RequestData;
 
 class UrlmonUrlRequest
   : public CComObjectRootEx<CComMultiThreadModel>,
@@ -39,9 +37,9 @@ class UrlmonUrlRequest
   // Used from "DownloadRequestInHost".
   // Callback will be invoked either right away (if operation is finished) or
   // from inside ::OnStopBinding() when it is safe to reuse the bind_context.
-  typedef Callback4<IMoniker*, IBindCtx*, IStream*, const char*>::Type
+  typedef base::Callback<void(IMoniker*, IBindCtx*, IStream*, const char*)>
       TerminateBindCallback;
-  void TerminateBind(TerminateBindCallback* callback);
+  void TerminateBind(const TerminateBindCallback& callback);
 
   // Parent Window for UrlMon error dialogs
   void set_parent_window(HWND parent_window) {
@@ -58,6 +56,10 @@ class UrlmonUrlRequest
   // to identify request objects in the log.
   std::string me() const;
 
+  static bool ImplementsThreadSafeReferenceCounting() {
+    return true;
+  }
+
  protected:
   UrlmonUrlRequest();
   ~UrlmonUrlRequest();
@@ -115,7 +117,7 @@ class UrlmonUrlRequest
   }
 
   bool terminate_requested() const {
-    return terminate_bind_callback_.get() != NULL;
+    return !terminate_bind_callback_.is_null();
   }
 
   std::string response_headers() {
@@ -210,15 +212,15 @@ class UrlmonUrlRequest
       return result_;
     }
 
-    void set_result(net::URLRequestStatus::Status status, int os_error) {
+    void set_result(net::URLRequestStatus::Status status, int error) {
       result_.set_status(status);
-      result_.set_os_error(os_error);
+      result_.set_error(error);
     }
 
     void set_result(HRESULT hr) {
       result_.set_status(FAILED(hr)? net::URLRequestStatus::FAILED:
                                      net::URLRequestStatus::SUCCESS);
-      result_.set_os_error(HresultToNetError(hr));
+      result_.set_error(HresultToNetError(hr));
     }
 
    private:
@@ -228,11 +230,11 @@ class UrlmonUrlRequest
   };
 
   Status status_;
-  ScopedComPtr<IBinding> binding_;
-  ScopedComPtr<IMoniker> moniker_;
-  ScopedComPtr<IBindCtx> bind_context_;
-  ScopedComPtr<IStream> cache_;
-  ScopedComPtr<IStream> pending_data_;
+  base::win::ScopedComPtr<IBinding> binding_;
+  base::win::ScopedComPtr<IMoniker> moniker_;
+  base::win::ScopedComPtr<IBindCtx> bind_context_;
+  base::win::ScopedComPtr<IStream> cache_;
+  base::win::ScopedComPtr<IStream> pending_data_;
 
   size_t pending_read_size_;
   base::PlatformThreadId thread_;
@@ -242,7 +244,7 @@ class UrlmonUrlRequest
   // Set to true if the ChromeFrame instance is running in privileged mode.
   bool privileged_mode_;
   bool pending_;
-  scoped_ptr<TerminateBindCallback> terminate_bind_callback_;
+  TerminateBindCallback terminate_bind_callback_;
   std::string response_headers_;
   // Defaults to true and indicates whether we want to keep the original
   // transaction alive when we receive the last data notification from
