@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -16,6 +16,7 @@
 #include "base/logging.h"
 #include "base/path_service.h"
 #include "base/string_number_conversions.h"
+#include "base/string_piece.h"
 #include "base/string_tokenizer.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
@@ -25,9 +26,11 @@
 #include "base/win/scoped_bstr.h"
 #include "base/win/scoped_comptr.h"
 #include "base/win/scoped_variant.h"
+#include "chrome/common/automation_messages.h"
 #include "chrome/common/chrome_paths_internal.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/installer/util/chrome_frame_distribution.h"
+#include "chrome_frame/chrome_tab.h"
 #include "chrome_frame/extra_system_apis.h"
 #include "chrome_frame/html_utils.h"
 #include "chrome_frame/navigation_constraints.h"
@@ -38,9 +41,9 @@
 #include "grit/chromium_strings.h"
 #include "net/base/escape.h"
 #include "net/http/http_util.h"
+#include "ui/base/models/menu_model.h"
 
 using base::win::RegKey;
-using base::win::ScopedComPtr;
 
 // Note that these values are all lower case and are compared to
 // lower-case-transformed values.
@@ -63,10 +66,7 @@ const wchar_t kRundllProfileName[] = L"rundll32";
 
 const wchar_t kAllowUnsafeURLs[] = L"AllowUnsafeURLs";
 const wchar_t kEnableBuggyBhoIntercept[] = L"EnableBuggyBhoIntercept";
-const wchar_t kEnableFirefoxPrivilegeMode[] = L"EnableFirefoxPrivilegeMode";
 
-static const wchar_t kChromeFrameNPAPIKey[] =
-    L"Software\\MozillaPlugins\\@google.com/ChromeFrame,version=1.0";
 static const wchar_t kChromeFramePersistNPAPIReg[] = L"PersistNPAPIReg";
 
 const char kAttachExternalTabPrefix[] = "attach_external_tab";
@@ -83,6 +83,11 @@ const wchar_t kChromeFrameAccessibleMode[] = L"ChromeFrameAccessibleMode";
 // DLL pinning, such as the perf tests.
 const wchar_t kChromeFrameUnpinnedMode[] = L"kChromeFrameUnpinnedMode";
 
+// Controls whether we download subresources, etc on the chrome frame page in
+// the background worker thread. Defaults to true.
+const wchar_t kUseBackgroundThreadForSubResources[]
+    = L"BackgroundHTTPWorkerThread";
+
 // {1AF32B6C-A3BA-48B9-B24E-8AA9C41F6ECD}
 static const IID IID_IWebBrowserPriv2IE7 = { 0x1AF32B6C, 0xA3BA, 0x48B9,
     { 0xB2, 0x4E, 0x8A, 0xA9, 0xC4, 0x1F, 0x6E, 0xCD } };
@@ -106,7 +111,7 @@ namespace {
 // pointer so it should not be dereferenced and used for comparison against a
 // living instance only.
 base::LazyInstance<base::ThreadLocalPointer<IBrowserService> >
-    g_tls_browser_for_cf_navigation(base::LINKER_INITIALIZED);
+    g_tls_browser_for_cf_navigation = LAZY_INSTANCE_INITIALIZER;
 
 }  // end anonymous namespace
 
@@ -216,54 +221,18 @@ HRESULT UtilUnRegisterTypeLib(ITypeLib* typelib,
   return hr;
 }
 
-bool UtilIsNPAPIPluginRegistered() {
-  std::wstring npapi_key_name(kChromeFrameNPAPIKey);
-  RegKey npapi_key(HKEY_LOCAL_MACHINE, npapi_key_name.c_str(), KEY_QUERY_VALUE);
-  return npapi_key.Valid();
-}
-
-bool UtilChangePersistentNPAPIMarker(bool set) {
+bool UtilRemovePersistentNPAPIMarker() {
   BrowserDistribution* cf_dist = BrowserDistribution::GetDistribution();
   std::wstring cf_state_key_path(cf_dist->GetStateKey());
+  RegKey cf_state_key;
 
-  RegKey cf_state_key(HKEY_LOCAL_MACHINE, cf_state_key_path.c_str(),
-                      KEY_READ | KEY_WRITE);
-
-  bool success = false;
-  if (cf_state_key.Valid()) {
-    if (set) {
-      success = (cf_state_key.WriteValue(kChromeFramePersistNPAPIReg, 1) ==
-          ERROR_SUCCESS);
-    } else {
-      // Unfortunately, DeleteValue returns true only if the value
-      // previously existed, so we do a separate existence check to
-      // validate success.
-      cf_state_key.DeleteValue(kChromeFramePersistNPAPIReg);
-      success = !cf_state_key.ValueExists(kChromeFramePersistNPAPIReg);
-    }
-  }
-  return success;
+  LONG result = cf_state_key.Open(HKEY_LOCAL_MACHINE, cf_state_key_path.c_str(),
+                                  KEY_SET_VALUE);
+  if (result == ERROR_SUCCESS)
+    result = cf_state_key.DeleteValue(kChromeFramePersistNPAPIReg);
+  return (result == ERROR_SUCCESS || result == ERROR_FILE_NOT_FOUND);
 }
 
-bool UtilIsPersistentNPAPIMarkerSet() {
-  BrowserDistribution* cf_dist = BrowserDistribution::GetDistribution();
-  std::wstring cf_state_key_path(cf_dist->GetStateKey());
-
-  RegKey cf_state_key(HKEY_LOCAL_MACHINE, cf_state_key_path.c_str(),
-                      KEY_QUERY_VALUE);
-
-  bool success = false;
-  if (cf_state_key.Valid()) {
-    DWORD val = 0;
-    if (cf_state_key.ReadValueDW(kChromeFramePersistNPAPIReg, &val) ==
-        ERROR_SUCCESS) {
-      success = (val != 0);
-    }
-  }
-  return success;
-}
-
-
 HRESULT UtilGetXUACompatContentValue(const std::wstring& html_string,
                                      std::wstring* content_value) {
   if (!content_value) {
@@ -370,8 +339,6 @@ bool IsChrome(RendererType renderer_type) {
 
 namespace {
 const char kIEImageName[] = "iexplore.exe";
-const char kFirefoxImageName[] = "firefox.exe";
-const char kOperaImageName[] = "opera.exe";
 }  // namespace
 
 std::wstring GetHostProcessName(bool include_extension) {
@@ -394,10 +361,6 @@ BrowserType GetBrowserType() {
       std::wstring::const_iterator end = exe.end();
       if (LowerCaseEqualsASCII(begin, end, kIEImageName)) {
         browser_type = BROWSER_IE;
-      } else if (LowerCaseEqualsASCII(begin, end, kFirefoxImageName)) {
-        browser_type = BROWSER_FIREFOX;
-      } else if (LowerCaseEqualsASCII(begin, end, kOperaImageName)) {
-        browser_type = BROWSER_OPERA;
       } else {
         browser_type = BROWSER_UNKNOWN;
       }
@@ -451,8 +414,11 @@ IEVersion GetIEVersion() {
       case 8:
         ie_version = IE_8;
         break;
+      case 9:
+        ie_version = IE_9;
+        break;
       default:
-        ie_version = major_version >= 9 ? IE_9 : IE_UNSUPPORTED;
+        ie_version = (major_version >= 10) ? IE_10 : IE_UNSUPPORTED;
         break;
     }
   }
@@ -465,7 +431,7 @@ FilePath GetIETemporaryFilesFolder() {
   HRESULT hr = SHGetFolderLocation(NULL, CSIDL_INTERNET_CACHE, NULL,
                                    SHGFP_TYPE_CURRENT, &tif_pidl);
   if (SUCCEEDED(hr) && tif_pidl) {
-    ScopedComPtr<IShellFolder> parent_folder;
+    base::win::ScopedComPtr<IShellFolder> parent_folder;
     LPITEMIDLIST relative_pidl = NULL;
     hr = SHBindToParent(tif_pidl, IID_IShellFolder,
                         reinterpret_cast<void**>(parent_folder.Receive()),
@@ -555,8 +521,8 @@ bool GetModuleVersion(HMODULE module, uint32* high, uint32* low) {
       // Copy data as VerQueryValue tries to modify the data. This causes
       // exceptions and heap corruption errors if debugger is attached.
       scoped_array<char> data(new char[version_resource_size]);
-      memcpy(data.get(), readonly_resource_data, version_resource_size);
       if (data.get()) {
+        memcpy(data.get(), readonly_resource_data, version_resource_size);
         VS_FIXEDFILEINFO* ver_info = NULL;
         UINT info_size = 0;
         if (VerQueryValue(data.get(), L"\\",
@@ -586,61 +552,66 @@ namespace {
 
 const int kMaxSubmenuDepth = 10;
 
-// Copies original_menu and returns the copy. The caller is responsible for
-// closing the returned HMENU. This does not currently copy over bitmaps
-// (e.g. hbmpChecked, hbmpUnchecked or hbmpItem), so checkmarks, radio buttons,
-// and custom icons won't work.
+// Builds a Windows menu from the menu model sent from Chrome.  The
+// caller is responsible for closing the returned HMENU.  This does
+// not currently handle bitmaps (e.g. hbmpChecked, hbmpUnchecked or
+// hbmpItem), so checkmarks, radio buttons, and custom icons won't work.
 // It also copies over submenus up to a maximum depth of kMaxSubMenuDepth.
-//
-// TODO(robertshield): Add support for the bitmap fields if need be.
-HMENU UtilCloneContextMenuImpl(HMENU original_menu, int depth) {
-  DCHECK(IsMenu(original_menu));
-
+HMENU BuildContextMenuImpl(const ContextMenuModel* menu_model, int depth) {
   if (depth >= kMaxSubmenuDepth)
     return NULL;
 
-  HMENU new_menu = CreatePopupMenu();
-  int item_count = GetMenuItemCount(original_menu);
-  if (item_count <= 0) {
-    NOTREACHED();
-  } else {
-    for (int i = 0; i < item_count; i++) {
-      MENUITEMINFO item_info = { 0 };
-      item_info.cbSize = sizeof(MENUITEMINFO);
-      item_info.fMask = MIIM_ID | MIIM_STRING | MIIM_FTYPE |
-                        MIIM_STATE | MIIM_DATA | MIIM_SUBMENU |
-                        MIIM_CHECKMARKS | MIIM_BITMAP;
+  HMENU menu = CreatePopupMenu();
+  for (size_t i = 0; i < menu_model->items.size(); i++) {
+    const ContextMenuModel::Item& item = menu_model->items[i];
 
-      // Call GetMenuItemInfo a first time to obtain the buffer size for
-      // the label.
-      if (GetMenuItemInfo(original_menu, i, TRUE, &item_info)) {
-        item_info.cch++;  // Increment this as per MSDN
-        std::vector<wchar_t> buffer(item_info.cch, 0);
-        item_info.dwTypeData = &buffer[0];
-
-        // Call GetMenuItemInfo a second time with dwTypeData set to a buffer
-        // of a correct size to get the label.
-        GetMenuItemInfo(original_menu, i, TRUE, &item_info);
-
-        // Clone any submenus. Within reason.
-        if (item_info.hSubMenu) {
-          HMENU new_submenu = UtilCloneContextMenuImpl(item_info.hSubMenu,
-                                                       depth + 1);
-          item_info.hSubMenu = new_submenu;
-        }
-
-        // Now insert the item into the new menu.
-        InsertMenuItem(new_menu, i, TRUE, &item_info);
-      }
+    MENUITEMINFO item_info = { 0 };
+    item_info.cbSize = sizeof(MENUITEMINFO);
+    switch (item.type) {
+      case ui::MenuModel::TYPE_COMMAND:
+      case ui::MenuModel::TYPE_CHECK:
+        item_info.fMask = MIIM_FTYPE | MIIM_ID | MIIM_STRING;
+        item_info.fType = MFT_STRING;
+        item_info.wID = item.item_id;
+        item_info.dwTypeData = const_cast<LPWSTR>(item.label.c_str());
+        break;
+      case ui::MenuModel::TYPE_RADIO:
+        item_info.fMask = MIIM_FTYPE | MIIM_ID | MIIM_STRING;
+        item_info.fType = MFT_STRING | MFT_RADIOCHECK;
+        item_info.wID = item.item_id;
+        item_info.dwTypeData = const_cast<LPWSTR>(item.label.c_str());
+        break;
+      case ui::MenuModel::TYPE_SEPARATOR:
+        item_info.fMask = MIIM_FTYPE;
+        item_info.fType = MFT_SEPARATOR;
+        break;
+      case ui::MenuModel::TYPE_SUBMENU:
+        item_info.fMask = MIIM_FTYPE | MIIM_ID | MIIM_STRING | MIIM_SUBMENU;
+        item_info.fType = MFT_STRING;
+        item_info.wID = item.item_id;
+        item_info.dwTypeData = const_cast<LPWSTR>(item.label.c_str());
+        item_info.hSubMenu = BuildContextMenuImpl(item.submenu, depth + 1);
+        break;
+      default:
+        NOTREACHED() << "Unsupported MenuModel::ItemType " << item.type;
+        break;
     }
+
+    item_info.fMask |= MIIM_STATE;
+    item_info.fState =
+        (item.checked ? MFS_CHECKED : MFS_UNCHECKED) |
+        (item.enabled ? MFS_ENABLED : (MFS_DISABLED | MFS_GRAYED));
+
+    InsertMenuItem(menu, i, TRUE, &item_info);
   }
-  return new_menu;
+
+  return menu;
 }
 
 }  // namespace
 
-HMENU UtilCloneContextMenu(HMENU original_menu) {
-  return UtilCloneContextMenuImpl(original_menu, 0);
+HMENU BuildContextMenu(const ContextMenuModel& menu_model) {
+  return BuildContextMenuImpl(&menu_model, 0);
 }
 
 std::string ResolveURL(const std::string& document,
@@ -791,12 +762,13 @@ RendererType RendererTypeForUrl(const std::wstring& url) {
 
 HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
                                  const wchar_t* headers, IBindCtx* bind_ctx,
-                                 const wchar_t* fragment, IStream* post_data) {
+                                 const wchar_t* fragment, IStream* post_data,
+                                 VARIANT* flags) {
   DCHECK(browser);
   DCHECK(moniker);
   DCHECK(bind_ctx);
 
-  ScopedComPtr<IWebBrowser2> web_browser2;
+  base::win::ScopedComPtr<IWebBrowser2> web_browser2;
   HRESULT hr = DoQueryService(SID_SWebBrowserApp, browser,
                               web_browser2.Receive());
   DCHECK(web_browser2);
@@ -805,13 +777,6 @@ HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
   if (FAILED(hr))
     return hr;
 
-  // Always issue the download request in a new window to ensure that the
-  // currently loaded ChromeFrame document does not inadvarently see an unload
-  // request. This runs javascript unload handlers on the page which renders
-  // the page non functional.
-  VARIANT flags = { VT_I4 };
-  V_I4(&flags) = navOpenInNewWindow | navNoHistory;
-
   // If the data to be downloaded was received in response to a post request
   // then we need to reissue the post request.
   base::win::ScopedVariant post_data_variant;
@@ -860,7 +825,7 @@ HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
   // HlinkSimpleNavigateToMoniker(moniker, url, NULL, host, bind_context,
   //                              NULL, 0, 0);
 
-  ScopedComPtr<IUriContainer> uri_container;
+  base::win::ScopedComPtr<IUriContainer> uri_container;
   hr = uri_container.QueryFrom(moniker);
 
   base::win::ScopedVariant headers_var;
@@ -876,7 +841,7 @@ HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
       &IID_IWebBrowserPriv2IE8XPBeta,
     };
 
-    ScopedComPtr<IWebBrowserPriv2Common, NULL> browser_priv2;
+    base::win::ScopedComPtr<IWebBrowserPriv2Common, NULL> browser_priv2;
     for (int i = 0; i < arraysize(interface_ids) && browser_priv2 == NULL;
          ++i) {
       hr = web_browser2.QueryInterface(*interface_ids[i],
@@ -886,20 +851,20 @@ HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
     DCHECK(browser_priv2);
 
     if (browser_priv2) {
-      ScopedComPtr<IUri> uri_obj;
+      base::win::ScopedComPtr<IUri> uri_obj;
       uri_container->GetIUri(uri_obj.Receive());
       DCHECK(uri_obj);
 
       if (GetIEVersion() < IE_9) {
         hr = browser_priv2->NavigateWithBindCtx2(
-                uri_obj, &flags, NULL, post_data_variant.AsInput(),
+                uri_obj, flags, NULL, post_data_variant.AsInput(),
                 headers_var.AsInput(), bind_ctx,
                 const_cast<wchar_t*>(fragment));
       } else {
         IWebBrowserPriv2CommonIE9* browser_priv2_ie9 =
             reinterpret_cast<IWebBrowserPriv2CommonIE9*>(browser_priv2.get());
         hr = browser_priv2_ie9->NavigateWithBindCtx2(
-                uri_obj, &flags, NULL, post_data_variant.AsInput(),
+                uri_obj, flags, NULL, post_data_variant.AsInput(),
                 headers_var.AsInput(), bind_ctx,
                 const_cast<wchar_t*>(fragment), 0);
       }
@@ -911,7 +876,7 @@ HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
     LPOLESTR url = NULL;
     if (SUCCEEDED(hr = moniker->GetDisplayName(bind_ctx, NULL, &url))) {
       DVLOG(1) << __FUNCTION__ << " " << url;
-      ScopedComPtr<IWebBrowserPriv> browser_priv;
+      base::win::ScopedComPtr<IWebBrowserPriv> browser_priv;
       if (SUCCEEDED(hr = browser_priv.QueryFrom(web_browser2))) {
         GURL target_url(url);
         // On IE6 if the original URL has a fragment then the navigation
@@ -929,7 +894,7 @@ HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
         }
 
         base::win::ScopedVariant var_url(UTF8ToWide(target_url.spec()).c_str());
-        hr = browser_priv->NavigateWithBindCtx(var_url.AsInput(), &flags, NULL,
+        hr = browser_priv->NavigateWithBindCtx(var_url.AsInput(), flags, NULL,
                                                post_data_variant.AsInput(),
                                                headers_var.AsInput(), bind_ctx,
                                                const_cast<wchar_t*>(fragment));
@@ -1019,14 +984,14 @@ bool IsSubFrameRequest(IUnknown* service_provider) {
 
   // We need to be able to get at an IWebBrowser2 if we are to decide whether
   // this request originates from a non-top-level frame.
-  ScopedComPtr<IWebBrowser2> web_browser;
+  base::win::ScopedComPtr<IWebBrowser2> web_browser;
   HRESULT hr = DoQueryService(IID_ITargetFrame2, service_provider,
                               web_browser.Receive());
 
   bool is_sub_frame_request = false;
   if (web_browser) {
     // Now check to see if we are in a sub-frame.
-    ScopedComPtr<IHTMLWindow2> current_frame, parent_frame;
+    base::win::ScopedComPtr<IHTMLWindow2> current_frame, parent_frame;
     hr = DoQueryService(IID_IHTMLWindow2, service_provider,
                         current_frame.Receive());
     if (current_frame) {
@@ -1179,7 +1144,7 @@ std::string GetHttpHeadersFromBinding(IBinding* binding) {
     return std::string();
   }
 
-  ScopedComPtr<IWinInetHttpInfo> info;
+  base::win::ScopedComPtr<IWinInetHttpInfo> info;
   if (FAILED(info.QueryFrom(binding))) {
     DLOG(WARNING) << "Failed to QI for IWinInetHttpInfo";
     return std::string();
@@ -1197,7 +1162,7 @@ int GetHttpResponseStatusFromBinding(IBinding* binding) {
 
   int http_status = 0;
 
-  ScopedComPtr<IWinInetHttpInfo> info;
+  base::win::ScopedComPtr<IWinInetHttpInfo> info;
   if (SUCCEEDED(info.QueryFrom(binding))) {
     char status[10] = {0};
     DWORD buf_size = sizeof(status);
@@ -1364,6 +1329,7 @@ std::string Bscf2Str(DWORD flags) {
 // Reads data from a stream into a string.
 HRESULT ReadStream(IStream* stream, size_t size, std::string* data) {
   DCHECK(stream);
+  DCHECK_GT(size, 0u);
   DCHECK(data);
 
   DWORD read = 0;
@@ -1459,8 +1425,8 @@ bool ChromeFrameUrl::ParseAttachExternalTabUrl() {
   if (tokenizer.GetNext()) {
     profile_name_ = tokenizer.token();
     // Escape out special characters like %20, etc.
-    profile_name_ = UnescapeURLComponent(profile_name_,
-        UnescapeRule::SPACES | UnescapeRule::URL_SPECIAL_CHARS);
+    profile_name_ = net::UnescapeURLComponent(profile_name_,
+        net::UnescapeRule::SPACES | net::UnescapeRule::URL_SPECIAL_CHARS);
   } else {
     return false;
   }
@@ -1509,18 +1475,20 @@ bool CanNavigate(const GURL& url,
 void PinModule() {
   static bool s_pinned = false;
   if (!s_pinned && !IsUnpinnedMode()) {
-    FilePath module_path;
-    if (PathService::Get(base::FILE_MODULE, &module_path)) {
+    wchar_t system_buffer[MAX_PATH];
+    HMODULE this_module = reinterpret_cast<HMODULE>(&__ImageBase);
+    system_buffer[0] = 0;
+    if (GetModuleFileName(this_module, system_buffer,
+                          arraysize(system_buffer)) != 0) {
       HMODULE unused;
-      if (!GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_PIN,
-                             module_path.value().c_str(), &unused)) {
-        NOTREACHED() << "Failed to pin module " << module_path.value().c_str()
-                     << " , last error: " << GetLastError();
+      if (!GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_PIN, system_buffer,
+                             &unused)) {
+        DPLOG(FATAL) << "Failed to pin module " << system_buffer;
       } else {
         s_pinned = true;
       }
     } else {
-      NOTREACHED() << "Could not get module path.";
+      DPLOG(FATAL) << "Could not get module path.";
     }
   }
 }
@@ -1589,7 +1557,9 @@ int GetXUaCompatibleDirective(const std::string& directive, char delimiter) {
     }
 
     int header_ie_version = 0;
-    if (!base::StringToInt(filter_begin + 2, filter_end, &header_ie_version) ||
+    if (!base::StringToInt(base::StringPiece(filter_begin + 2,
+                                             filter_end),
+                           &header_ie_version) ||
         header_ie_version == 0) {  // ensure it's not a sequence of 0's
       continue;
     }
@@ -1625,3 +1595,53 @@ std::wstring GetCurrentModuleVersion() {
   DCHECK(module_version_info.get() != NULL);
   return module_version_info->file_version();
 }
+
+bool IsChromeFrameDocument(IWebBrowser2* web_browser) {
+  if (!web_browser)
+    return false;
+
+  base::win::ScopedComPtr<IDispatch> doc;
+  web_browser->get_Document(doc.Receive());
+  if (doc) {
+    // Detect if CF is rendering based on whether the document is a
+    // ChromeActiveDocument. Detecting based on hwnd is problematic as
+    // the CF Active Document window may not have been created yet.
+    base::win::ScopedComPtr<IChromeFrame> chrome_frame;
+    chrome_frame.QueryFrom(doc);
+    return chrome_frame.get() != NULL;
+  }
+  return false;
+}
+
+bool IncreaseWinInetConnections(DWORD connections) {
+  static bool wininet_connection_count_updated = false;
+  if (wininet_connection_count_updated) {
+    return true;
+  }
+
+  static int connection_options[] = {
+    INTERNET_OPTION_MAX_CONNS_PER_SERVER,
+    INTERNET_OPTION_MAX_CONNS_PER_1_0_SERVER,
+  };
+
+  BOOL ret = FALSE;
+
+  for (int option_index = 0; option_index < arraysize(connection_options);
+       ++option_index) {
+    DWORD connection_value_size = sizeof(DWORD);
+    DWORD current_connection_limit = 0;
+    InternetQueryOption(NULL, connection_options[option_index],
+                        &current_connection_limit, &connection_value_size);
+    if (current_connection_limit > connections) {
+      continue;
+    }
+
+    ret = InternetSetOption(NULL, connection_options[option_index],
+                            &connections, connection_value_size);
+    if (!ret) {
+      return false;
+    }
+  }
+  wininet_connection_count_updated = true;
+  return true;
+}
