@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,21 +9,21 @@
 #include <objidl.h>
 #include <windows.h>
 #include <wininet.h>
-
 #include <string>
 #include <vector>
 
 #include "base/basictypes.h"
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
-#include "base/synchronization/lock.h"
 #include "base/threading/thread.h"
 #include "base/win/scoped_comptr.h"
-#include "gfx/rect.h"
 #include "googleurl/src/gurl.h"
+#include "ui/gfx/rect.h"
 
 class FilePath;
 interface IBrowserService;
+interface IWebBrowser2;
+struct ContextMenuModel;
 
 // utils.h : Various utility functions and classes
 
@@ -35,7 +35,6 @@ extern const wchar_t kChromeFrameAccessibleMode[];
 extern const wchar_t kChromeFrameUnpinnedMode[];
 extern const wchar_t kAllowUnsafeURLs[];
 extern const wchar_t kEnableBuggyBhoIntercept[];
-extern const wchar_t kEnableFirefoxPrivilegeMode[];
 extern const wchar_t kChromeMimeType[];
 extern const wchar_t kChromeFrameAttachTabPattern[];
 extern const wchar_t kChromeFrameConfigKey[];
@@ -44,6 +43,7 @@ extern const wchar_t kRenderInHostUrlList[];
 extern const wchar_t kEnableGCFRendererByDefault[];
 extern const wchar_t kIexploreProfileName[];
 extern const wchar_t kRundllProfileName[];
+extern const wchar_t kUseBackgroundThreadForSubResources[];
 
 // This function is very similar to the AtlRegisterTypeLib function except
 // that it takes a parameter that specifies whether to register the typelib
@@ -75,15 +75,9 @@ HRESULT UtilRegisterTypeLib(ITypeLib* typelib,
 HRESULT UtilUnRegisterTypeLib(ITypeLib* typelib,
                               bool for_current_user_only);
 
-// Utility function to tell if the NPAPI plugin is registered.
-bool UtilIsNPAPIPluginRegistered();
-
-// Sets or clears a marker that causes NPAPI registration to persist across
-// updates. The marker is added if set is true and is deleted otherwise.
-bool UtilChangePersistentNPAPIMarker(bool set);
-
-// Returns true if the persistent NPAPI marker is set, false otherwise.
-bool UtilIsPersistentNPAPIMarkerSet();
+// Clears a marker that causes legacy NPAPI registration to persist across
+// updates. Returns false if the marker could not be removed.
+bool UtilRemovePersistentNPAPIMarker();
 
 // Given an HTML fragment, this function looks for the
 // <meta http-equiv="X-UA-Compatible"> tag and extracts the value of the
@@ -157,8 +151,6 @@ typedef enum BrowserType {
   BROWSER_INVALID = -1,
   BROWSER_UNKNOWN,
   BROWSER_IE,
-  BROWSER_FIREFOX,
-  BROWSER_OPERA,
 };
 
 BrowserType GetBrowserType();
@@ -171,6 +163,7 @@ typedef enum IEVersion {
   IE_7,
   IE_8,
   IE_9,
+  IE_10,
 };
 
 // The renderer to be used for a page.  Values for Chrome also convey the
@@ -195,8 +188,8 @@ enum RendererType {
 bool IsChrome(RendererType renderer_type);
 
 // Convenience macro for logging a sample for the launch type metric.
-#define THREAD_SAFE_UMA_LAUNCH_TYPE_COUNT(sample) \
-  THREAD_SAFE_UMA_HISTOGRAM_CUSTOM_COUNTS("ChromeFrame.LaunchType", sample, \
+#define UMA_LAUNCH_TYPE_COUNT(sample) \
+  UMA_HISTOGRAM_CUSTOM_COUNTS("ChromeFrame.LaunchType", sample, \
   RENDERER_TYPE_CHROME_MIN, RENDERER_TYPE_CHROME_MAX, \
   RENDERER_TYPE_CHROME_MAX + 1 - RENDERER_TYPE_CHROME_MIN)
 
@@ -235,9 +228,8 @@ bool IsIEInPrivate();
 // Calls [ieframe|shdocvw]!DoFileDownload to initiate a download.
 HRESULT DoFileDownloadInIE(const wchar_t* url);
 
-// Creates a copy of a menu. We need this when original menu comes from
-// a process with higher integrity.
-HMENU UtilCloneContextMenu(HMENU original_menu);
+// Construct a menu from the model sent from Chrome.
+HMENU BuildContextMenu(const ContextMenuModel& menu_model);
 
 // Uses GURL internally to append 'relative' to 'document'
 std::string ResolveURL(const std::string& document,
@@ -303,7 +295,8 @@ HRESULT DoQueryService(const IID& service_id, IUnknown* unk, T** service) {
 // |headers| can be NULL.
 HRESULT NavigateBrowserToMoniker(IUnknown* browser, IMoniker* moniker,
                                  const wchar_t* headers, IBindCtx* bind_ctx,
-                                 const wchar_t* fragment, IStream* post_data);
+                                 const wchar_t* fragment, IStream* post_data,
+                                 VARIANT* flags);
 
 // Raises a flag on the current thread (using TLS) to indicate that an
 // in-progress navigation should be rendered in chrome frame.
@@ -394,6 +387,9 @@ STDMETHODIMP QueryInterfaceIfDelegateSupports(void* obj, REFIID iid,
 class STAThread : public base::Thread {
  public:
   explicit STAThread(const char *name) : Thread(name) {}
+  ~STAThread() {
+    Stop();
+  }
   bool Start() {
     return StartWithOptions(Options(MessageLoop::TYPE_UI, 0));
   }
@@ -426,28 +422,6 @@ bool IsTopLevelWindow(HWND window);
 // Seeks a stream back to position 0.
 HRESULT RewindStream(IStream* stream);
 
-extern base::Lock g_ChromeFrameHistogramLock;
-
-// Thread safe versions of the UMA histogram macros we use for ChromeFrame.
-// These should be used for histograms in ChromeFrame. If other histogram
-// macros from base/metrics/histogram.h are needed then thread safe versions of
-// those should be defined and used.
-#define THREAD_SAFE_UMA_HISTOGRAM_CUSTOM_COUNTS(name, sample, min, max, \
-                                                bucket_count) { \
-  base::AutoLock lock(g_ChromeFrameHistogramLock); \
-  UMA_HISTOGRAM_CUSTOM_COUNTS(name, sample, min, max, bucket_count); \
-}
-
-#define THREAD_SAFE_UMA_HISTOGRAM_TIMES(name, sample) { \
-  base::AutoLock lock(g_ChromeFrameHistogramLock); \
-  UMA_HISTOGRAM_TIMES(name, sample); \
-}
-
-#define THREAD_SAFE_UMA_HISTOGRAM_COUNTS(name, sample) { \
-  base::AutoLock lock(g_ChromeFrameHistogramLock); \
-  UMA_HISTOGRAM_COUNTS(name, sample); \
-}
-
 // Fired when we want to notify IE about privacy changes.
 #define WM_FIRE_PRIVACY_CHANGE_NOTIFICATION (WM_APP + 1)
 
@@ -625,4 +599,14 @@ bool CheckXUaCompatibleDirective(const std::string& directive,
 // Returns the version of the current module as a string.
 std::wstring GetCurrentModuleVersion();
 
+// Returns true if ChromeFrame is the currently loaded document.
+bool IsChromeFrameDocument(IWebBrowser2* web_browser);
+
+// Increases the wininet connection limit for HTTP 1.0/1.1 connections to the
+// value passed in. This is only done if the existing connection limit is
+// lesser than the connection limit passed in. This function attempts to
+// increase the connection count once per process.
+// Returns true on success.
+bool IncreaseWinInetConnections(DWORD connections);
+
 #endif  // CHROME_FRAME_UTILS_H_
