@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,11 +11,10 @@
 #include "base/logging.h"
 
 #include "courgette/crc.h"
-#include "courgette/image_info.h"
 #include "courgette/region.h"
 #include "courgette/streams.h"
 #include "courgette/simple_delta.h"
-#include "courgette/win32_x86_patcher.h"
+#include "courgette/patcher_x86_32.h"
 
 namespace courgette {
 
@@ -136,13 +135,22 @@ Status EnsemblePatchApplication::ReadInitialParameters(
     if (!transformation_parameters->ReadVarint32(&kind))
       return C_BAD_ENSEMBLE_HEADER;
 
-    if (kind == CourgettePatchFile::T_COURGETTE_WIN32_X86) {
-      TransformationPatcher* patcher =
-          new CourgetteWin32X86Patcher(base_region_);
-      patchers_.push_back(patcher);
-    } else {
-      return C_BAD_ENSEMBLE_HEADER;
+    TransformationPatcher* patcher = NULL;
+
+    switch (kind)
+    {
+      case EXE_WIN_32_X86:
+        patcher = new PatcherX86_32(base_region_);
+        break;
+      case EXE_ELF_32_X86:
+        patcher = new PatcherX86_32(base_region_);
+        break;
     }
+
+    if (patcher)
+      patchers_.push_back(patcher);
+    else
+      return C_BAD_ENSEMBLE_HEADER;
   }
 
   for (size_t i = 0;  i < patchers_.size();  ++i) {
@@ -220,10 +228,13 @@ Status EnsemblePatchApplication::TransformDown(
     SinkStream* basic_elements) {
   // Construct blob of original input followed by reformed elements.
 
-  basic_elements->Reserve(final_patch_input_size_prediction_);
+  if (!basic_elements->Reserve(final_patch_input_size_prediction_)) {
+    return C_STREAM_ERROR;
+  }
 
   // The original input:
-  basic_elements->Write(base_region_.start(), base_region_.length());
+  if (!basic_elements->Write(base_region_.start(), base_region_.length()))
+    return C_STREAM_ERROR;
 
   for (size_t i = 0;  i < patchers_.size();  ++i) {
     SourceStreamSet single_corrected_element;
@@ -393,6 +404,8 @@ Status ApplyEnsemblePatch(const FilePath::CharType* old_file_name,
   SinkStream new_sink_stream;
   status = ApplyEnsemblePatch(&old_source_stream, &patch_source_stream,
                               &new_sink_stream);
+  if (status != C_OK)
+    return status;
 
   // Write the patched data to |new_file_name|.
   FilePath new_file_path(new_file_name);
