@@ -1,20 +1,24 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 // This file contains the tests for the RingBuffer class.
 
 #include "gpu/command_buffer/client/ring_buffer.h"
-#include "base/callback.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/message_loop.h"
-#include "base/mac/scoped_nsautorelease_pool.h"
 #include "gpu/command_buffer/client/cmd_buffer_helper.h"
 #include "gpu/command_buffer/service/cmd_buffer_engine.h"
 #include "gpu/command_buffer/service/mocks.h"
 #include "gpu/command_buffer/service/command_buffer_service.h"
-#include "gpu/command_buffer/service/gpu_processor.h"
+#include "gpu/command_buffer/service/gpu_scheduler.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+#if defined(OS_MACOSX)
+#include "base/mac/scoped_nsautorelease_pool.h"
+#endif
+
 namespace gpu {
 
 using testing::Return;
@@ -30,6 +34,25 @@ class BaseRingBufferTest : public testing::Test {
   static const unsigned int kBaseOffset = 128;
   static const unsigned int kBufferSize = 1024;
 
+  class DoJumpCommand {
+   public:
+    explicit DoJumpCommand(GpuScheduler* gpu_scheduler)
+        : gpu_scheduler_(gpu_scheduler) {
+    }
+
+    error::Error DoCommand(
+        unsigned int command,
+        unsigned int arg_count,
+        const void* cmd_data) {
+      const cmd::Jump* jump_cmd = static_cast<const cmd::Jump*>(cmd_data);
+      gpu_scheduler_->parser()->set_get(jump_cmd->offset);
+      return error::kNoError;
+    };
+
+   private:
+    GpuScheduler* gpu_scheduler_;
+  };
+
   virtual void SetUp() {
     api_mock_.reset(new AsyncAPIMock);
     // ignore noops in the mock - we don't want to inspect the internals of the
@@ -42,22 +65,20 @@ class BaseRingBufferTest : public testing::Test {
                               Return(error::kNoError)));
 
     command_buffer_.reset(new CommandBufferService);
-    command_buffer_->Initialize(kBufferSize);
-    Buffer ring_buffer = command_buffer_->GetRingBuffer();
+    command_buffer_->Initialize();
 
-    parser_ = new CommandParser(ring_buffer.ptr,
-                                ring_buffer.size,
-                                0,
-                                ring_buffer.size,
-                                0,
-                                api_mock_.get());
+    gpu_scheduler_.reset(new GpuScheduler(
+        command_buffer_.get(), api_mock_.get(), NULL));
+    command_buffer_->SetPutOffsetChangeCallback(base::Bind(
+        &GpuScheduler::PutChanged, base::Unretained(gpu_scheduler_.get())));
+    command_buffer_->SetGetBufferChangeCallback(base::Bind(
+        &GpuScheduler::SetGetBuffer, base::Unretained(gpu_scheduler_.get())));
 
-    gpu_processor_.reset(new GPUProcessor(
-        command_buffer_.get(), NULL, parser_, INT_MAX));
-    command_buffer_->SetPutOffsetChangeCallback(NewCallback(
-        gpu_processor_.get(), &GPUProcessor::ProcessCommands));
-
-    api_mock_->set_engine(gpu_processor_.get());
+    api_mock_->set_engine(gpu_scheduler_.get());
+    do_jump_command_.reset(new DoJumpCommand(gpu_scheduler_.get()));
+    EXPECT_CALL(*api_mock_, DoCommand(cmd::kJump, _, _))
+        .WillRepeatedly(
+            Invoke(do_jump_command_.get(), &DoJumpCommand::DoCommand));
 
     helper_.reset(new CommandBufferHelper(command_buffer_.get()));
     helper_->Initialize(kBufferSize);
@@ -67,13 +88,15 @@ class BaseRingBufferTest : public testing::Test {
     return command_buffer_->GetState().token;
   }
 
+#if defined(OS_MACOSX)
   base::mac::ScopedNSAutoreleasePool autorelease_pool_;
+#endif
   MessageLoop message_loop_;
   scoped_ptr<AsyncAPIMock> api_mock_;
   scoped_ptr<CommandBufferService> command_buffer_;
-  scoped_ptr<GPUProcessor> gpu_processor_;
-  CommandParser* parser_;
+  scoped_ptr<GpuScheduler> gpu_scheduler_;
   scoped_ptr<CommandBufferHelper> helper_;
+  scoped_ptr<DoJumpCommand> do_jump_command_;
 };
 
 #ifndef _MSC_VER
@@ -93,7 +116,7 @@ class RingBufferTest : public BaseRingBufferTest {
   }
 
   virtual void TearDown() {
-    // If the GPUProcessor posts any tasks, this forces them to run.
+    // If the GpuScheduler posts any tasks, this forces them to run.
     MessageLoop::current()->RunAllPending();
 
     BaseRingBufferTest::TearDown();
@@ -189,7 +212,7 @@ class RingBufferWrapperTest : public BaseRingBufferTest {
   }
 
   virtual void TearDown() {
-    // If the GPUProcessor posts any tasks, this forces them to run.
+    // If the GpuScheduler posts any tasks, this forces them to run.
     MessageLoop::current()->RunAllPending();
 
     BaseRingBufferTest::TearDown();
@@ -256,6 +279,7 @@ TEST_F(RingBufferWrapperTest, TestFreePendingToken) {
   EXPECT_LE(tokens[0], GetToken());
 
   allocator_->FreePendingToken(pointer1, helper_->InsertToken());
+  EXPECT_LE(command_buffer_->GetState().token, helper_->InsertToken());
 }
 
 }  // namespace gpu
