@@ -1,13 +1,13 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 // This file is here so other GLES2 related files can have a common set of
 // includes where appropriate.
 
+#include <stdio.h>
 #include <GLES2/gl2.h>
 #include <GLES2/gl2ext.h>
-#include <GLES2/gles2_command_buffer.h>
 
 #include "../common/gles2_cmd_utils.h"
 #include "../common/gles2_cmd_format.h"
@@ -22,7 +22,7 @@ enum GLErrorBit {
   kInvalidValue = (1 << 1),
   kInvalidOperation = (1 << 2),
   kOutOfMemory = (1 << 3),
-  kInvalidFrameBufferOperation = (1 << 4),
+  kInvalidFrameBufferOperation = (1 << 4)
 };
 }
 
@@ -123,8 +123,12 @@ int GLES2Util::GLGetNumValuesReturned(int id) const {
       return 2;
     case GL_NUM_COMPRESSED_TEXTURE_FORMATS:
       return 1;
+    case GL_NUM_SHADER_BINARY_FORMATS:
+      return 1;
     case GL_PACK_ALIGNMENT:
       return 1;
+    case GL_PACK_REVERSE_ROW_ORDER_ANGLE:
+      return 1;
     case GL_POLYGON_OFFSET_FACTOR:
       return 1;
     case GL_POLYGON_OFFSET_FILL:
@@ -147,6 +151,8 @@ int GLES2Util::GLGetNumValuesReturned(int id) const {
       return 4;
     case GL_SCISSOR_TEST:
       return 1;
+    case GL_SHADER_BINARY_FORMATS:
+      return num_shader_binary_formats_;
     case GL_SHADER_COMPILER:
       return 1;
     case GL_STENCIL_BACK_FAIL:
@@ -189,10 +195,18 @@ int GLES2Util::GLGetNumValuesReturned(int id) const {
       return 1;
     case GL_TEXTURE_BINDING_CUBE_MAP:
       return 1;
+    case GL_TEXTURE_BINDING_EXTERNAL_OES:
+      return 1;
+    case GL_TEXTURE_BINDING_RECTANGLE_ARB:
+      return 1;
     case GL_UNPACK_ALIGNMENT:
       return 1;
     case GL_VIEWPORT:
       return 4;
+    // -- glGetBooleanv, glGetFloatv, glGetIntergerv with
+    //    GL_CHROMIUM_framebuffer_multisample
+    case GL_MAX_SAMPLES_EXT:
+      return 1;
 
     // -- glGetBufferParameteriv
     case GL_BUFFER_SIZE:
@@ -264,6 +278,8 @@ int GLES2Util::GLGetNumValuesReturned(int id) const {
     //   return 1;
     case GL_SHADER_SOURCE_LENGTH:
       return 1;
+    case GL_TRANSLATED_SHADER_SOURCE_LENGTH_ANGLE:
+      return 1;
 
     // -- glGetTexParameterfv, glGetTexParameteriv
     case GL_TEXTURE_MAG_FILTER:
@@ -319,6 +335,7 @@ int ElementsPerGroup(int format, int type) {
     case GL_RGB:
        return 3;
     case GL_LUMINANCE_ALPHA:
+       return 2;
     case GL_RGBA:
     case GL_BGRA_EXT:
        return 4;
@@ -337,6 +354,7 @@ int BytesPerElement(int type) {
   switch (type) {
     case GL_FLOAT:
     case GL_UNSIGNED_INT_24_8_OES:
+    case GL_UNSIGNED_INT:
       return 4;
     case GL_HALF_FLOAT_OES:
     case GL_UNSIGNED_SHORT:
@@ -387,6 +405,27 @@ bool GLES2Util::ComputeImageDataSize(
   return true;
 }
 
+size_t GLES2Util::RenderbufferBytesPerPixel(int format) {
+  switch (format) {
+    case GL_STENCIL_INDEX8:
+      return 1;
+    case GL_RGBA4:
+    case GL_RGB565:
+    case GL_RGB5_A1:
+    case GL_DEPTH_COMPONENT16:
+      return 2;
+    case GL_RGB:
+    case GL_RGBA:
+    case GL_DEPTH24_STENCIL8_OES:
+    case GL_RGB8_OES:
+    case GL_RGBA8_OES:
+    case GL_DEPTH_COMPONENT24_OES:
+      return 4;
+    default:
+      return 0;
+  }
+}
+
 uint32 GLES2Util::GetGLDataTypeSizeForUniforms(int type) {
   switch (type) {
     case GL_FLOAT:
@@ -423,6 +462,8 @@ uint32 GLES2Util::GetGLDataTypeSizeForUniforms(int type) {
       return sizeof(GLint);                // NOLINT
     case GL_SAMPLER_CUBE:
       return sizeof(GLint);                // NOLINT
+    case GL_SAMPLER_EXTERNAL_OES:
+      return sizeof(GLint);                // NOLINT
     default:
       return 0;
   }
@@ -508,9 +549,12 @@ uint32 GLES2Util::GetChannelsForFormat(int format) {
     case GL_LUMINANCE_ALPHA:
       return 0x000F;
     case GL_RGB:
+    case GL_RGB8_OES:
     case GL_RGB565:
       return 0x0007;
+    case GL_BGRA_EXT:
     case GL_RGBA:
+    case GL_RGBA8_OES:
     case GL_RGBA4:
     case GL_RGB5_A1:
       return 0x000F;
@@ -519,6 +563,43 @@ uint32 GLES2Util::GetChannelsForFormat(int format) {
   }
 }
 
+std::string GLES2Util::GetStringEnum(uint32 value) {
+  const EnumToString* entry = enum_to_string_table_;
+  const EnumToString* end = entry + enum_to_string_table_len_;
+  for (;entry < end; ++entry) {
+    if (value == entry->value) {
+      return entry->name;
+    }
+  }
+  char buffer[20];
+  sprintf(buffer, (value < 0x10000) ? "0x%04x" : "0x%08x", value);
+  return buffer;
+}
+
+std::string GLES2Util::GetStringError(uint32 value) {
+  static EnumToString string_table[] = {
+    { GL_NONE, "GL_NONE" },
+  };
+  return GLES2Util::GetQualifiedEnumString(
+      string_table, arraysize(string_table), value);
+}
+
+std::string GLES2Util::GetStringBool(uint32 value) {
+  return value ? "true" : "false";
+}
+
+std::string GLES2Util::GetQualifiedEnumString(
+    const EnumToString* table, size_t count, uint32 value) {
+  for (const EnumToString* end = table + count; table < end; ++table) {
+    if (table->value == value) {
+      return table->name;
+    }
+  }
+  return GetStringEnum(value);
+}
+
+#include "../common/gles2_cmd_utils_implementation_autogen.h"
+
 }  // namespace gles2
 }  // namespace gpu
 
