@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,23 +10,29 @@
 
 namespace gpu {
 
-CommandParser::CommandParser(void *shm_address,
-                             size_t shm_size,
-                             ptrdiff_t offset,
-                             size_t size,
-                             CommandBufferOffset start_get,
-                             AsyncAPIInterface *handler)
-    : get_(start_get),
-      put_(start_get),
+CommandParser::CommandParser(AsyncAPIInterface* handler)
+    : get_(0),
+      put_(0),
+      buffer_(NULL),
+      entry_count_(0),
       handler_(handler) {
+}
+
+void CommandParser::SetBuffer(
+    void* shm_address,
+    size_t shm_size,
+    ptrdiff_t offset,
+    size_t size) {
   // check proper alignments.
   DCHECK_EQ(0, (reinterpret_cast<intptr_t>(shm_address)) % 4);
   DCHECK_EQ(0, offset % 4);
   DCHECK_EQ(0u, size % 4);
   // check that the command buffer fits into the memory buffer.
   DCHECK_GE(shm_size, offset + size);
-  char * buffer_begin = static_cast<char *>(shm_address) + offset;
-  buffer_ = reinterpret_cast<CommandBufferEntry *>(buffer_begin);
+  get_ = 0;
+  put_ = 0;
+  char* buffer_begin = static_cast<char*>(shm_address) + offset;
+  buffer_ = reinterpret_cast<CommandBufferEntry*>(buffer_begin);
   entry_count_ = size / 4;
 }
 
@@ -55,10 +61,11 @@ error::Error CommandParser::ProcessCommand() {
 
   error::Error result = handler_->DoCommand(
       header.command, header.size - 1, buffer_ + get);
+
   // TODO(gman): If you want to log errors this is the best place to catch them.
   //     It seems like we need an official way to turn on a debug mode and
   //     get these errors.
-  if (result != error::kNoError) {
+  if (error::IsError(result)) {
     ReportError(header.command, result);
   }
 
