@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -95,14 +95,14 @@ class MockCommandBufferEngine : public CommandBufferEngine {
 
   template <typename T>
   T GetSharedMemoryAs(uint32 offset) {
-    DCHECK(offset < kBufferSize);
+    DCHECK_LT(offset, kBufferSize);
     return reinterpret_cast<T>(&buffer_[offset]);
   }
 
   int32 GetSharedMemoryOffset(const void* memory) {
     ptrdiff_t offset = reinterpret_cast<const int8*>(memory) - &buffer_[0];
     DCHECK_GE(offset, 0);
-    DCHECK(static_cast<size_t>(offset) < kBufferSize);
+    DCHECK_LT(static_cast<size_t>(offset), kBufferSize);
     return static_cast<int32>(offset);
   }
 
@@ -115,6 +115,12 @@ class MockCommandBufferEngine : public CommandBufferEngine {
     return token_;
   }
 
+  // Overridden from CommandBufferEngine.
+  virtual bool SetGetBuffer(int32 transfer_buffer_id) {
+    NOTREACHED();
+    return false;
+  }
+
   // Overridden from CommandBufferEngine.
   virtual bool SetGetOffset(int32 offset) {
     if (static_cast<size_t>(offset) < kBufferSize) {
