@@ -1,22 +1,23 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef GPU_COMMAND_BUFFER_SERVICE_CONTEXT_GROUP_H_
 #define GPU_COMMAND_BUFFER_SERVICE_CONTEXT_GROUP_H_
 
-#include <map>
 #include <string>
 #include "base/basictypes.h"
-#include "base/linked_ptr.h"
-#include "base/scoped_ptr.h"
-#include "base/ref_counted.h"
+#include "base/hash_tables.h"
+#include "base/memory/linked_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "gpu/command_buffer/common/gles2_cmd_format.h"
 #include "gpu/command_buffer/service/gles2_cmd_validation.h"
 #include "gpu/command_buffer/service/feature_info.h"
 
 namespace gpu {
 
-class IdAllocator;
+class IdAllocatorInterface;
 
 namespace gles2 {
 
@@ -27,6 +28,7 @@ class RenderbufferManager;
 class ProgramManager;
 class ShaderManager;
 class TextureManager;
+struct DisallowedFeatures;
 
 // A Context Group helps manage multiple GLES2Decoders that share
 // resources.
@@ -34,15 +36,20 @@ class ContextGroup : public base::RefCounted<ContextGroup> {
  public:
   typedef scoped_refptr<ContextGroup> Ref;
 
-  ContextGroup();
+  explicit ContextGroup(bool bind_generates_resource);
   ~ContextGroup();
 
-  // This should only be called by GLES2Decoder.
-  bool Initialize(const char* allowed_features);
+  // This should only be called by GLES2Decoder. This must be paired with a
+  // call to destroy if it succeeds.
+  bool Initialize(const DisallowedFeatures& disallowed_features,
+                  const char* allowed_features);
 
-  // Sets the ContextGroup has having a lost context.
-  void set_have_context(bool have_context) {
-    have_context_ = have_context;
+  // Destroys all the resources when called for the last context in the group.
+  // It should only be called by GLES2Decoder.
+  void Destroy(bool have_context);
+
+  bool bind_generates_resource() {
+    return bind_generates_resource_;
   }
 
   uint32 max_vertex_attribs() const {
@@ -74,7 +81,7 @@ class ContextGroup : public base::RefCounted<ContextGroup> {
   }
 
   FeatureInfo* feature_info() {
-    return &feature_info_;
+    return feature_info_.get();
   }
 
   BufferManager* buffer_manager() const {
@@ -101,15 +108,12 @@ class ContextGroup : public base::RefCounted<ContextGroup> {
     return shader_manager_.get();
   }
 
-  IdAllocator* GetIdAllocator(unsigned namepsace_id);
+  IdAllocatorInterface* GetIdAllocator(unsigned namespace_id);
 
  private:
-  // Destroys all the resources.
-  void Destroy();
-
   // Whether or not this context is initialized.
-  bool initialized_;
-  bool have_context_;
+  int num_contexts_;
+  bool bind_generates_resource_;
 
   uint32 max_vertex_attribs_;
   uint32 max_texture_units_;
@@ -131,10 +135,10 @@ class ContextGroup : public base::RefCounted<ContextGroup> {
 
   scoped_ptr<ShaderManager> shader_manager_;
 
-  typedef std::map<uint32, linked_ptr<IdAllocator> > IdAllocatorMap;
-  IdAllocatorMap id_namespaces_;
+  linked_ptr<IdAllocatorInterface>
+      id_namespaces_[id_namespaces::kNumIdNamespaces];
 
-  FeatureInfo feature_info_;
+  FeatureInfo::Ref feature_info_;
 
   DISALLOW_COPY_AND_ASSIGN(ContextGroup);
 };
