@@ -1,14 +1,14 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef GPU_COMMAND_BUFFER_SERVICE_FRAMEBUFFER_MANAGER_H_
 #define GPU_COMMAND_BUFFER_SERVICE_FRAMEBUFFER_MANAGER_H_
 
-#include <map>
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/hash_tables.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "gpu/command_buffer/service/gl_utils.h"
 #include "gpu/command_buffer/service/renderbuffer_manager.h"
 #include "gpu/command_buffer/service/texture_manager.h"
@@ -35,7 +35,15 @@ class FramebufferManager {
       virtual GLenum internal_format() const = 0;
       virtual GLsizei samples() const = 0;
       virtual bool cleared() const = 0;
-      virtual void set_cleared() = 0;
+      virtual void SetCleared(
+          RenderbufferManager* renderbuffer_manager,
+          TextureManager* texture_manager) = 0;
+      virtual bool IsTexture(TextureManager::TextureInfo* texture) const = 0;
+      virtual bool IsRenderbuffer(
+          RenderbufferManager::RenderbufferInfo* renderbuffer) const = 0;
+      virtual bool CanRenderTo() const = 0;
+      virtual void DetachFromFramebuffer() = 0;
+      virtual bool ValidForAttachmentType(GLenum attachment_type) = 0;
     };
 
     explicit FramebufferInfo(GLuint service_id);
@@ -56,7 +64,13 @@ class FramebufferManager {
         GLenum attachment, TextureManager::TextureInfo* texture, GLenum target,
         GLint level);
 
-    void MarkAttachedRenderbuffersAsCleared();
+    // Unbinds the given renderbuffer if it is bound.
+    void UnbindRenderbuffer(
+        GLenum target, RenderbufferManager::RenderbufferInfo* renderbuffer);
+
+    // Unbinds the given texture if it is bound.
+    void UnbindTexture(
+        GLenum target, TextureManager::TextureInfo* texture);
 
     const Attachment* GetAttachment(GLenum attachment) const;
 
@@ -72,15 +86,40 @@ class FramebufferManager {
       return has_been_bound_ && !IsDeleted();
     }
 
+    bool HasDepthAttachment() const;
+    bool HasStencilAttachment() const;
+    GLenum GetColorAttachmentFormat() const;
+
+    // Verify all the rules in OpenGL ES 2.0.25 4.4.5 are followed.
+    // Returns GL_FRAMEBUFFER_COMPLETE if there are no reasons we know we can't
+    // use this combination of attachments. Otherwise returns the value
+    // that glCheckFramebufferStatus should return for this set of attachments.
+    // Note that receiving GL_FRAMEBUFFER_COMPLETE from this function does
+    // not mean the real OpenGL will consider it framebuffer complete. It just
+    // means it passed our tests.
+    GLenum IsPossiblyComplete() const;
+
+    // Check all attachments are cleared
+    bool IsCleared() const;
+
    private:
     friend class FramebufferManager;
     friend class base::RefCounted<FramebufferInfo>;
 
     ~FramebufferInfo();
 
-    void MarkAsDeleted() {
-      service_id_ = 0;
-      attachments_.clear();
+    void MarkAsDeleted();
+
+    void MarkAttachmentsAsCleared(
+      RenderbufferManager* renderbuffer_manager,
+      TextureManager* texture_manager);
+
+    void MarkAsComplete(unsigned state_id) {
+      framebuffer_complete_state_count_id_ = state_id;
+    }
+
+    unsigned framebuffer_complete_state_count_id() const {
+      return framebuffer_complete_state_count_id_;
     }
 
     // Service side framebuffer id.
@@ -89,8 +128,11 @@ class FramebufferManager {
     // Whether this framebuffer has ever been bound.
     bool has_been_bound_;
 
+    // state count when this framebuffer was last checked for completeness.
+    unsigned framebuffer_complete_state_count_id_;
+
     // A map of attachments.
-    typedef std::map<GLenum, Attachment::Ref> AttachmentMap;
+    typedef base::hash_map<GLenum, Attachment::Ref> AttachmentMap;
     AttachmentMap attachments_;
 
     DISALLOW_COPY_AND_ASSIGN(FramebufferInfo);
@@ -114,12 +156,30 @@ class FramebufferManager {
   // Gets a client id for a given service id.
   bool GetClientId(GLuint service_id, GLuint* client_id) const;
 
+  void MarkAttachmentsAsCleared(
+    FramebufferInfo* framebuffer,
+    RenderbufferManager* renderbuffer_manager,
+    TextureManager* texture_manager);
+
+  void MarkAsComplete(FramebufferInfo* framebuffer);
+
+  bool IsComplete(FramebufferInfo* framebuffer);
+
+  void IncFramebufferStateChangeCount() {
+    // make sure this is never 0.
+    framebuffer_state_change_count_ =
+        (framebuffer_state_change_count_ + 1) | 0x80000000U;
+  }
+
  private:
   // Info for each framebuffer in the system.
-  // TODO(gman): Choose a faster container.
-  typedef std::map<GLuint, FramebufferInfo::Ref> FramebufferInfoMap;
+  typedef base::hash_map<GLuint, FramebufferInfo::Ref> FramebufferInfoMap;
   FramebufferInfoMap framebuffer_infos_;
 
+  // Incremented anytime anything changes that might effect framebuffer
+  // state.
+  unsigned framebuffer_state_change_count_;
+
   DISALLOW_COPY_AND_ASSIGN(FramebufferManager);
 };
 
