@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,25 +11,39 @@
 
 #include "base/callback.h"
 #include "build/build_config.h"
-#include "gfx/size.h"
 #include "gpu/command_buffer/service/common_decoder.h"
+#include "ui/gfx/size.h"
 
 namespace gfx {
 class GLContext;
+class GLSurface;
 }
 
 namespace gpu {
 
+class StreamTextureManager;
+
 namespace gles2 {
 
 class ContextGroup;
 class GLES2Util;
 
+struct DisallowedFeatures {
+  DisallowedFeatures()
+      : multisampling(false),
+        driver_bug_workarounds(false) {
+  }
+
+  bool multisampling;
+  bool driver_bug_workarounds;
+};
+
 // This class implements the AsyncAPIInterface interface, decoding GLES2
 // commands and calling GL.
 class GLES2Decoder : public CommonDecoder {
  public:
   typedef error::Error Error;
+  typedef base::Callback<void(int32 id, const std::string& msg)> MsgCallback;
 
   // Creates a decoder.
   static GLES2Decoder* Create(ContextGroup* group);
@@ -40,49 +54,60 @@ class GLES2Decoder : public CommonDecoder {
     return debug_;
   }
 
+  // Set to true to call glGetError after every command.
   void set_debug(bool debug) {
     debug_ = debug;
   }
 
+  bool log_commands() const {
+    return log_commands_;
+  }
+
+  // Set to true to LOG every command.
+  void set_log_commands(bool log_commands) {
+    log_commands_ = log_commands;
+  }
+
   // Initializes the graphics context. Can create an offscreen
   // decoder with a frame buffer that can be referenced from the parent.
   // Takes ownership of GLContext.
   // Parameters:
+  //  surface: the GL surface to render to.
   //  context: the GL context to render to.
   //  size: the size if the GL context is offscreen.
   //  allowed_extensions: A string in the same format as
   //      glGetString(GL_EXTENSIONS) that lists the extensions this context
   //      should allow. Passing NULL or "*" means allow all extensions.
-  //  parent: the GLES2 decoder that can access this decoder's front buffer
-  //      through a texture ID in its namespace.
-  //  parent_client_texture_id: the texture ID of the front buffer in the
-  //      parent's namespace.
   // Returns:
   //   true if successful.
-  virtual bool Initialize(gfx::GLContext* context,
+  virtual bool Initialize(const scoped_refptr<gfx::GLSurface>& surface,
+                          const scoped_refptr<gfx::GLContext>& context,
                           const gfx::Size& size,
+                          const DisallowedFeatures& disallowed_features,
                           const char* allowed_extensions,
-                          const std::vector<int32>& attribs,
-                          GLES2Decoder* parent,
-                          uint32 parent_client_texture_id) = 0;
+                          const std::vector<int32>& attribs) = 0;
 
   // Destroys the graphics context.
   virtual void Destroy() = 0;
 
-  // Resize an offscreen frame buffer.
-  virtual void ResizeOffscreenFrameBuffer(const gfx::Size& size) = 0;
+  virtual bool SetParent(GLES2Decoder* parent_decoder,
+                         uint32 parent_texture_id) = 0;
 
-  // Force the offscreen frame buffer's size to be updated. This
-  // usually occurs lazily, during SwapBuffers, but on some platforms
-  // (Mac OS X in particular) it must be done immediately.
-  virtual bool UpdateOffscreenFrameBufferSize() = 0;
+  // Resize an offscreen frame buffer.
+  virtual bool ResizeOffscreenFrameBuffer(const gfx::Size& size) = 0;
 
   // Make this decoder's GL context current.
   virtual bool MakeCurrent() = 0;
 
+  // Have the decoder release the context.
+  virtual void ReleaseCurrent() = 0;
+
   // Gets the GLES2 Util which holds info.
   virtual GLES2Util* GetGLES2Util() = 0;
 
+  // Gets the associated GLSurface.
+  virtual gfx::GLSurface* GetGLSurface() = 0;
+
   // Gets the associated GLContext.
   virtual gfx::GLContext* GetGLContext() = 0;
 
@@ -91,21 +116,41 @@ class GLES2Decoder : public CommonDecoder {
 
   // Sets a callback which is called when a glResizeCHROMIUM command
   // is processed.
-  virtual void SetResizeCallback(Callback1<gfx::Size>::Type* callback) = 0;
+  virtual void SetResizeCallback(
+      const base::Callback<void(gfx::Size)>& callback) = 0;
 
-  // Sets a callback which is called when a SwapBuffers command is processed.
-  virtual void SetSwapBuffersCallback(Callback0::Type* callback) = 0;
+  virtual void SetStreamTextureManager(StreamTextureManager* manager) = 0;
 
   // Get the service texture ID corresponding to a client texture ID.
   // If no such record is found then return false.
   virtual bool GetServiceTextureId(uint32 client_texture_id,
                                    uint32* service_texture_id);
 
+  // Provides detail about a lost context if one occurred.
+  virtual error::ContextLostReason GetContextLostReason() = 0;
+
+  // Clears a level of a texture
+  // Returns false if a GL error should be generated.
+  virtual bool ClearLevel(
+      unsigned service_id,
+      unsigned bind_target,
+      unsigned target,
+      int level,
+      unsigned format,
+      unsigned type,
+      int width,
+      int height,
+      bool is_texture_immutable) = 0;
+
+  // A callback for messages from the decoder.
+  virtual void SetMsgCallback(const MsgCallback& callback) = 0;
+
  protected:
   GLES2Decoder();
 
  private:
   bool debug_;
+  bool log_commands_;
 
   DISALLOW_COPY_AND_ASSIGN(GLES2Decoder);
 };
