@@ -1,11 +1,10 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef GPU_COMMAND_BUFFER_SERVICE_GLES2_CMD_DECODER_UNITTEST_BASE_H_
 #define GPU_COMMAND_BUFFER_SERVICE_GLES2_CMD_DECODER_UNITTEST_BASE_H_
 
-#include "app/gfx/gl/gl_context_stub.h"
 #include "gpu/command_buffer/common/gl_mock.h"
 #include "gpu/command_buffer/common/gles2_cmd_format.h"
 #include "gpu/command_buffer/common/gles2_cmd_utils.h"
@@ -18,24 +17,17 @@
 #include "gpu/command_buffer/service/renderbuffer_manager.h"
 #include "gpu/command_buffer/service/shader_manager.h"
 #include "gpu/command_buffer/service/texture_manager.h"
-#include "gpu/GLES2/gles2_command_buffer.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "ui/gfx/gl/gl_context_stub.h"
+#include "ui/gfx/gl/gl_surface_stub.h"
 
 namespace gpu {
 namespace gles2 {
 
 class GLES2DecoderTestBase : public testing::Test {
  public:
-  GLES2DecoderTestBase()
-      : client_buffer_id_(100),
-        client_framebuffer_id_(101),
-        client_program_id_(102),
-        client_renderbuffer_id_(103),
-        client_shader_id_(104),
-        client_texture_id_(106),
-        client_element_buffer_id_(107) {
-    memset(immediate_buffer_, 0xEE, sizeof(immediate_buffer_));
-  }
+  GLES2DecoderTestBase();
+  virtual ~GLES2DecoderTestBase();
 
  protected:
   static const GLint kMaxTextureSize = 2048;
@@ -74,267 +66,6 @@ class GLES2DecoderTestBase : public testing::Test {
   static const int kBackBufferWidth = 128;
   static const int kBackBufferHeight = 64;
 
-  // Template to call glGenXXX functions.
-  template <typename T>
-  void GenHelper(GLuint client_id) {
-    int8 buffer[sizeof(T) + sizeof(client_id)];
-    T& cmd = *reinterpret_cast<T*>(&buffer);
-    cmd.Init(1, &client_id);
-    EXPECT_EQ(error::kNoError,
-              ExecuteImmediateCmd(cmd, sizeof(client_id)));
-  }
-
-  // This template exists solely so we can specialize it for
-  // certain commands.
-  template <typename T, int id>
-  void SpecializedSetup(bool valid) {
-  }
-
-  template <typename T>
-  T* GetImmediateAs() {
-    return reinterpret_cast<T*>(immediate_buffer_);
-  }
-
-  template <typename T, typename Command>
-  T GetImmediateDataAs(Command* cmd) {
-    return reinterpret_cast<T>(ImmediateDataAddress(cmd));
-  }
-
-  void ClearSharedMemory() {
-    engine_->ClearSharedMemory();
-  }
-
-  virtual void SetUp();
-  virtual void TearDown();
-
-  template <typename T>
-  error::Error ExecuteCmd(const T& cmd) {
-    COMPILE_ASSERT(T::kArgFlags == cmd::kFixed, Cmd_kArgFlags_not_kFixed);
-    return decoder_->DoCommand(cmd.kCmdId,
-                               ComputeNumEntries(sizeof(cmd)) - 1,
-                               &cmd);
-  }
-
-  template <typename T>
-  error::Error ExecuteImmediateCmd(const T& cmd, size_t data_size) {
-    COMPILE_ASSERT(T::kArgFlags == cmd::kAtLeastN, Cmd_kArgFlags_not_kAtLeastN);
-    return decoder_->DoCommand(cmd.kCmdId,
-                               ComputeNumEntries(sizeof(cmd) + data_size) - 1,
-                               &cmd);
-  }
-
-  template <typename T>
-  T GetSharedMemoryAs() {
-    return reinterpret_cast<T>(shared_memory_address_);
-  }
-
-  template <typename T>
-  T GetSharedMemoryAsWithOffset(uint32 offset) {
-    void* ptr = reinterpret_cast<int8*>(shared_memory_address_) + offset;
-    return reinterpret_cast<T>(ptr);
-  }
-
-  IdAllocator* GetIdAllocator(GLuint namespace_id) {
-    return group_->GetIdAllocator(namespace_id);
-  }
-
-  BufferManager::BufferInfo* GetBufferInfo(GLuint service_id) {
-    return group_->buffer_manager()->GetBufferInfo(service_id);
-  }
-
-  FramebufferManager::FramebufferInfo* GetFramebufferInfo(GLuint service_id) {
-    return group_->framebuffer_manager()->GetFramebufferInfo(service_id);
-  }
-
-  RenderbufferManager::RenderbufferInfo* GetRenderbufferInfo(
-      GLuint service_id) {
-    return group_->renderbuffer_manager()->GetRenderbufferInfo(service_id);
-  }
-
-  TextureManager::TextureInfo* GetTextureInfo(GLuint service_id) {
-    return group_->texture_manager()->GetTextureInfo(service_id);
-  }
-
-  ShaderManager::ShaderInfo* GetShaderInfo(GLuint service_id) {
-    return group_->shader_manager()->GetShaderInfo(service_id);
-  }
-
-  ProgramManager::ProgramInfo* GetProgramInfo(GLuint service_id) {
-    return group_->program_manager()->GetProgramInfo(service_id);
-  }
-
-  void DoCreateProgram(GLuint client_id, GLuint service_id);
-  void DoCreateShader(GLenum shader_type, GLuint client_id, GLuint service_id);
-
-  void SetBucketAsCString(uint32 bucket_id, const char* str);
-
-  void InitDecoder(const char* extensions);
-
-  const ContextGroup& group() const {
-    return *group_.get();
-  }
-
-  struct AttribInfo {
-    const char* name;
-    GLint size;
-    GLenum type;
-    GLint location;
-  };
-
-  struct UniformInfo {
-    const char* name;
-    GLint size;
-    GLenum type;
-    GLint location;
-  };
-
-  void SetupShader(
-      AttribInfo* attribs, size_t num_attribs,
-      UniformInfo* uniforms, size_t num_uniforms,
-      GLuint client_id, GLuint service_id,
-      GLuint vertex_shader_client_id, GLuint vertex_shader_service_id,
-      GLuint fragment_shader_client_id, GLuint fragment_shader_service_id);
-
-  // Setups up a shader for testing glUniform.
-  void SetupShaderForUniform();
-
-  // Note that the error is returned as GLint instead of GLenum.
-  // This is because there is a mismatch in the types of GLenum and
-  // the error values GL_NO_ERROR, GL_INVALID_ENUM, etc. GLenum is
-  // typedef'd as unsigned int while the error values are defined as
-  // integers. This is problematic for template functions such as
-  // EXPECT_EQ that expect both types to be the same.
-  GLint GetGLError();
-
-  void DoBindBuffer(GLenum target, GLuint client_id, GLuint service_id);
-  void DoBindFramebuffer(GLenum target, GLuint client_id, GLuint service_id);
-  void DoBindRenderbuffer(GLenum target, GLuint client_id, GLuint service_id);
-  void DoBindTexture(GLenum target, GLuint client_id, GLuint service_id);
-
-  bool DoIsBuffer(GLuint client_id);
-  bool DoIsFramebuffer(GLuint client_id);
-  bool DoIsProgram(GLuint client_id);
-  bool DoIsRenderbuffer(GLuint client_id);
-  bool DoIsShader(GLuint client_id);
-  bool DoIsTexture(GLuint client_id);
-
-  void DoDeleteBuffer(GLuint client_id, GLuint service_id);
-  void DoDeleteFramebuffer(GLuint client_id, GLuint service_id);
-  void DoDeleteProgram(GLuint client_id, GLuint service_id);
-  void DoDeleteRenderbuffer(GLuint client_id, GLuint service_id);
-  void DoDeleteShader(GLuint client_id, GLuint service_id);
-  void DoDeleteTexture(GLuint client_id, GLuint service_id);
-
-  void DoTexImage2D(GLenum target, GLint level, GLenum internal_format,
-                    GLsizei width, GLsizei height, GLint border,
-                    GLenum format, GLenum type,
-                    uint32 shared_memory_id, uint32 shared_memory_offset);
-  void DoVertexAttribPointer(
-      GLuint index, GLint size, GLenum type, GLsizei stride, GLuint offset);
-
-  void SetupExpectationsForFramebufferAttachment(
-    GLuint clear_bits,
-    GLclampf restore_red,
-    GLclampf restore_green,
-    GLclampf restore_blue,
-    GLclampf restore_alpha,
-    GLuint restore_color_mask,
-    GLuint restore_stencil,
-    GLuint restore_stencil_front_mask,
-    GLuint restore_stencil_back_mask,
-    GLclampf restore_depth,
-    GLboolean restore_depth_mask,
-    bool restore_scissor_test);
-
-  GLvoid* BufferOffset(unsigned i) {
-    return static_cast<int8 *>(NULL)+(i);
-  }
-
-  template <typename Command, typename Result>
-  bool IsObjectHelper(GLuint client_id) {
-    Result* result = static_cast<Result*>(shared_memory_address_);
-    Command cmd;
-    cmd.Init(client_id, kSharedMemoryId, kSharedMemoryOffset);
-    EXPECT_EQ(error::kNoError, ExecuteCmd(cmd));
-    bool isObject = static_cast<bool>(*result);
-    EXPECT_EQ(GL_NO_ERROR, GetGLError());
-    return isObject;
-  }
-
-  // Use StrictMock to make 100% sure we know how GL will be called.
-  scoped_ptr< ::testing::StrictMock< ::gfx::MockGLInterface> > gl_;
-  gfx::StubGLContext* context_;
-  scoped_ptr<GLES2Decoder> decoder_;
-
-  GLuint client_buffer_id_;
-  GLuint client_framebuffer_id_;
-  GLuint client_program_id_;
-  GLuint client_renderbuffer_id_;
-  GLuint client_shader_id_;
-  GLuint client_texture_id_;
-  GLuint client_element_buffer_id_;
-
-  uint32 shared_memory_id_;
-  uint32 shared_memory_offset_;
-  void* shared_memory_address_;
-
-  int8 immediate_buffer_[256];
-
- private:
-  class MockCommandBufferEngine : public CommandBufferEngine {
-   public:
-    MockCommandBufferEngine() {
-      data_.reset(new int8[kSharedBufferSize]);
-      ClearSharedMemory();
-      valid_buffer_.ptr = data_.get();
-      valid_buffer_.size = kSharedBufferSize;
-    }
-
-    virtual ~MockCommandBufferEngine() {
-    }
-
-    Buffer GetSharedMemoryBuffer(int32 shm_id) {
-      return shm_id == kSharedMemoryId ? valid_buffer_ : invalid_buffer_;
-    }
-
-    void ClearSharedMemory() {
-      memset(data_.get(), kInitialMemoryValue, kSharedBufferSize);
-    }
-
-    void set_token(int32 token) {
-      DCHECK(false);
-    }
-
-    // Overridden from CommandBufferEngine.
-    virtual bool SetGetOffset(int32 offset) {
-      DCHECK(false);
-      return false;
-    }
-
-    // Overridden from CommandBufferEngine.
-    virtual int32 GetGetOffset() {
-      DCHECK(false);
-      return 0;
-    }
-
-   private:
-    scoped_array<int8> data_;
-    Buffer valid_buffer_;
-    Buffer invalid_buffer_;
-  };
-
-  scoped_ptr< ::testing::StrictMock<MockCommandBufferEngine> > engine_;
-  ContextGroup::Ref group_;
-};
-
-class GLES2DecoderWithShaderTestBase : public GLES2DecoderTestBase {
- public:
-  GLES2DecoderWithShaderTestBase()
-      : GLES2DecoderTestBase(),
-        client_vertex_shader_id_(121),
-        client_fragment_shader_id_(122) {
-  }
-
   static const GLuint kServiceVertexShaderId = 321;
   static const GLuint kServiceFragmentShaderId = 322;
 
@@ -377,15 +108,197 @@ class GLES2DecoderWithShaderTestBase : public GLES2DecoderTestBase {
   static const GLenum kUniform1Type = GL_SAMPLER_2D;
   static const GLenum kUniform2Type = GL_INT_VEC2;
   static const GLenum kUniform3Type = GL_FLOAT_VEC3;
+  static const GLenum kUniformCubemapType = GL_SAMPLER_CUBE;
   static const GLint kInvalidUniformLocation = 30;
   static const GLint kBadUniformIndex = 1000;
 
- protected:
-  virtual void SetUp();
-  virtual void TearDown();
+  // Template to call glGenXXX functions.
+  template <typename T>
+  void GenHelper(GLuint client_id) {
+    int8 buffer[sizeof(T) + sizeof(client_id)];
+    T& cmd = *reinterpret_cast<T*>(&buffer);
+    cmd.Init(1, &client_id);
+    EXPECT_EQ(error::kNoError,
+              ExecuteImmediateCmd(cmd, sizeof(client_id)));
+  }
 
+  // This template exists solely so we can specialize it for
+  // certain commands.
+  template <typename T, int id>
+  void SpecializedSetup(bool valid) {
+  }
+
+  template <typename T>
+  T* GetImmediateAs() {
+    return reinterpret_cast<T*>(immediate_buffer_);
+  }
+
+  template <typename T, typename Command>
+  T GetImmediateDataAs(Command* cmd) {
+    return reinterpret_cast<T>(ImmediateDataAddress(cmd));
+  }
+
+  void ClearSharedMemory() {
+    engine_->ClearSharedMemory();
+  }
+
+  virtual void SetUp() OVERRIDE;
+  virtual void TearDown() OVERRIDE;
+
+  template <typename T>
+  error::Error ExecuteCmd(const T& cmd) {
+    COMPILE_ASSERT(T::kArgFlags == cmd::kFixed, Cmd_kArgFlags_not_kFixed);
+    return decoder_->DoCommand(cmd.kCmdId,
+                               ComputeNumEntries(sizeof(cmd)) - 1,
+                               &cmd);
+  }
+
+  template <typename T>
+  error::Error ExecuteImmediateCmd(const T& cmd, size_t data_size) {
+    COMPILE_ASSERT(T::kArgFlags == cmd::kAtLeastN, Cmd_kArgFlags_not_kAtLeastN);
+    return decoder_->DoCommand(cmd.kCmdId,
+                               ComputeNumEntries(sizeof(cmd) + data_size) - 1,
+                               &cmd);
+  }
+
+  template <typename T>
+  T GetSharedMemoryAs() {
+    return reinterpret_cast<T>(shared_memory_address_);
+  }
+
+  template <typename T>
+  T GetSharedMemoryAsWithOffset(uint32 offset) {
+    void* ptr = reinterpret_cast<int8*>(shared_memory_address_) + offset;
+    return reinterpret_cast<T>(ptr);
+  }
+
+  IdAllocatorInterface* GetIdAllocator(GLuint namespace_id) {
+    return group_->GetIdAllocator(namespace_id);
+  }
+
+  BufferManager::BufferInfo* GetBufferInfo(GLuint service_id) {
+    return group_->buffer_manager()->GetBufferInfo(service_id);
+  }
+
+  FramebufferManager::FramebufferInfo* GetFramebufferInfo(GLuint service_id) {
+    return group_->framebuffer_manager()->GetFramebufferInfo(service_id);
+  }
+
+  RenderbufferManager::RenderbufferInfo* GetRenderbufferInfo(
+      GLuint service_id) {
+    return group_->renderbuffer_manager()->GetRenderbufferInfo(service_id);
+  }
+
+  TextureManager::TextureInfo* GetTextureInfo(GLuint service_id) {
+    return group_->texture_manager()->GetTextureInfo(service_id);
+  }
+
+  ShaderManager::ShaderInfo* GetShaderInfo(GLuint service_id) {
+    return group_->shader_manager()->GetShaderInfo(service_id);
+  }
+
+  ProgramManager::ProgramInfo* GetProgramInfo(GLuint service_id) {
+    return group_->program_manager()->GetProgramInfo(service_id);
+  }
+
+  void DoCreateProgram(GLuint client_id, GLuint service_id);
+  void DoCreateShader(GLenum shader_type, GLuint client_id, GLuint service_id);
+
+  void SetBucketAsCString(uint32 bucket_id, const char* str);
+
+  void InitDecoder(
+      const char* extensions,
+      bool has_alpha,
+      bool has_depth,
+      bool has_stencil,
+      bool request_alpha,
+      bool request_depth,
+      bool request_stencil,
+      bool bind_generates_resource);
+
+  const ContextGroup& group() const {
+    return *group_.get();
+  }
+
+  struct AttribInfo {
+    const char* name;
+    GLint size;
+    GLenum type;
+    GLint location;
+  };
+
+  struct UniformInfo {
+    const char* name;
+    GLint size;
+    GLenum type;
+    GLint location;
+  };
+
+  void SetupShader(
+      AttribInfo* attribs, size_t num_attribs,
+      UniformInfo* uniforms, size_t num_uniforms,
+      GLuint client_id, GLuint service_id,
+      GLuint vertex_shader_client_id, GLuint vertex_shader_service_id,
+      GLuint fragment_shader_client_id, GLuint fragment_shader_service_id);
+
+  // Setups up a shader for testing glUniform.
+  void SetupShaderForUniform();
+  void SetupDefaultProgram();
+  void SetupCubemapProgram();
   void SetupTexture();
 
+  // Note that the error is returned as GLint instead of GLenum.
+  // This is because there is a mismatch in the types of GLenum and
+  // the error values GL_NO_ERROR, GL_INVALID_ENUM, etc. GLenum is
+  // typedef'd as unsigned int while the error values are defined as
+  // integers. This is problematic for template functions such as
+  // EXPECT_EQ that expect both types to be the same.
+  GLint GetGLError();
+
+  void DoBindBuffer(GLenum target, GLuint client_id, GLuint service_id);
+  void DoBindFramebuffer(GLenum target, GLuint client_id, GLuint service_id);
+  void DoBindRenderbuffer(GLenum target, GLuint client_id, GLuint service_id);
+  void DoBindTexture(GLenum target, GLuint client_id, GLuint service_id);
+
+  bool DoIsBuffer(GLuint client_id);
+  bool DoIsFramebuffer(GLuint client_id);
+  bool DoIsProgram(GLuint client_id);
+  bool DoIsRenderbuffer(GLuint client_id);
+  bool DoIsShader(GLuint client_id);
+  bool DoIsTexture(GLuint client_id);
+
+  void DoDeleteBuffer(GLuint client_id, GLuint service_id);
+  void DoDeleteFramebuffer(
+      GLuint client_id, GLuint service_id,
+      bool reset_draw, GLenum draw_target, GLuint draw_id,
+      bool reset_read, GLenum read_target, GLuint read_id);
+  void DoDeleteProgram(GLuint client_id, GLuint service_id);
+  void DoDeleteRenderbuffer(GLuint client_id, GLuint service_id);
+  void DoDeleteShader(GLuint client_id, GLuint service_id);
+  void DoDeleteTexture(GLuint client_id, GLuint service_id);
+
+  void DoTexImage2D(
+      GLenum target, GLint level, GLenum internal_format,
+      GLsizei width, GLsizei height, GLint border,
+      GLenum format, GLenum type,
+      uint32 shared_memory_id, uint32 shared_memory_offset);
+  void DoRenderbufferStorage(
+      GLenum target, GLenum internal_format, GLenum actual_format,
+      GLsizei width, GLsizei height, GLenum error);
+  void DoFramebufferRenderbuffer(
+      GLenum target,
+      GLenum attachment,
+      GLenum renderbuffer_target,
+      GLuint renderbuffer_client_id,
+      GLuint renderbuffer_service_id,
+      GLenum error);
+  void DoFramebufferTexture2D(
+      GLenum target, GLenum attachment, GLenum tex_target,
+      GLuint texture_client_id, GLuint texture_service_id,
+      GLint level, GLenum error);
+  void DoVertexAttribPointer(
+      GLuint index, GLint size, GLenum type, GLsizei stride, GLuint offset);
+
   void DoEnableVertexAttribArray(GLint index);
 
   void DoBufferData(GLenum target, GLsizei size);
@@ -394,6 +307,7 @@ class GLES2DecoderWithShaderTestBase : public GLES2DecoderTestBase {
       GLenum target, GLint offset, GLsizei size, const void* data);
 
   void SetupVertexBuffer();
+  void SetupAllNeededVertexBuffers();
 
   void SetupIndexBuffer();
 
@@ -401,8 +315,160 @@ class GLES2DecoderWithShaderTestBase : public GLES2DecoderTestBase {
 
   void DeleteIndexBuffer();
 
+  void SetupClearTextureExpections(
+      GLuint service_id,
+      GLuint old_service_id,
+      GLenum bind_target,
+      GLenum target,
+      GLint level,
+      GLenum format,
+      GLenum type,
+      GLsizei width,
+      GLsizei height);
+
+  void SetupExpectationsForFramebufferClearing(
+      GLenum target,
+      GLuint clear_bits,
+      GLclampf restore_red,
+      GLclampf restore_green,
+      GLclampf restore_blue,
+      GLclampf restore_alpha,
+      GLuint restore_stencil,
+      GLclampf restore_depth,
+      bool restore_scissor_test);
+
+  void SetupExpectationsForFramebufferClearingMulti(
+      GLuint read_framebuffer_service_id,
+      GLuint draw_framebuffer_service_id,
+      GLenum target,
+      GLuint clear_bits,
+      GLclampf restore_red,
+      GLclampf restore_green,
+      GLclampf restore_blue,
+      GLclampf restore_alpha,
+      GLuint restore_stencil,
+      GLclampf restore_depth,
+      bool restore_scissor_test);
+
+  void SetupExpectationsForApplyingDirtyState(
+      bool framebuffer_is_rgb,
+      bool framebuffer_has_depth,
+      bool framebuffer_has_stencil,
+      GLuint color_bits,  // NOTE! bits are 0x1000, 0x0100, 0x0010, and 0x0001
+      bool depth_mask,
+      bool depth_enabled,
+      GLuint front_stencil_mask,
+      GLuint back_stencil_mask,
+      bool stencil_enabled);
+
+  void SetupExpectationsForApplyingDefaultDirtyState();
+
+  void AddExpectationsForSimulatedAttrib0WithError(
+      GLsizei num_vertices, GLuint buffer_id, GLenum error);
+
+  void AddExpectationsForSimulatedAttrib0(
+      GLsizei num_vertices, GLuint buffer_id);
+
+  GLvoid* BufferOffset(unsigned i) {
+    return static_cast<int8 *>(NULL)+(i);
+  }
+
+  template <typename Command, typename Result>
+  bool IsObjectHelper(GLuint client_id) {
+    Result* result = static_cast<Result*>(shared_memory_address_);
+    Command cmd;
+    cmd.Init(client_id, kSharedMemoryId, kSharedMemoryOffset);
+    EXPECT_EQ(error::kNoError, ExecuteCmd(cmd));
+    bool isObject = static_cast<bool>(*result);
+    EXPECT_EQ(GL_NO_ERROR, GetGLError());
+    return isObject;
+  }
+
+  // Use StrictMock to make 100% sure we know how GL will be called.
+  scoped_ptr< ::testing::StrictMock< ::gfx::MockGLInterface> > gl_;
+  scoped_refptr<gfx::GLSurfaceStub> surface_;
+  scoped_refptr<gfx::GLContextStub> context_;
+  scoped_ptr<GLES2Decoder> decoder_;
+
+  GLuint client_buffer_id_;
+  GLuint client_framebuffer_id_;
+  GLuint client_program_id_;
+  GLuint client_renderbuffer_id_;
+  GLuint client_shader_id_;
+  GLuint client_texture_id_;
+  GLuint client_element_buffer_id_;
   GLuint client_vertex_shader_id_;
   GLuint client_fragment_shader_id_;
+
+  uint32 shared_memory_id_;
+  uint32 shared_memory_offset_;
+  void* shared_memory_address_;
+  void* shared_memory_base_;
+
+  int8 immediate_buffer_[256];
+
+ private:
+  class MockCommandBufferEngine : public CommandBufferEngine {
+   public:
+    MockCommandBufferEngine() {
+      data_.reset(new int8[kSharedBufferSize]);
+      ClearSharedMemory();
+      valid_buffer_.ptr = data_.get();
+      valid_buffer_.size = kSharedBufferSize;
+    }
+
+    virtual ~MockCommandBufferEngine() {
+    }
+
+    virtual Buffer GetSharedMemoryBuffer(int32 shm_id) OVERRIDE {
+      return shm_id == kSharedMemoryId ? valid_buffer_ : invalid_buffer_;
+    }
+
+    void ClearSharedMemory() {
+      memset(data_.get(), kInitialMemoryValue, kSharedBufferSize);
+    }
+
+    virtual void set_token(int32 token) OVERRIDE {
+      DCHECK(false);
+    }
+
+    virtual bool SetGetBuffer(int32 /* transfer_buffer_id */) OVERRIDE {
+      DCHECK(false);
+      return false;
+    }
+
+    // Overridden from CommandBufferEngine.
+    virtual bool SetGetOffset(int32 offset) OVERRIDE {
+      DCHECK(false);
+      return false;
+    }
+
+    // Overridden from CommandBufferEngine.
+    virtual int32 GetGetOffset() OVERRIDE {
+      DCHECK(false);
+      return 0;
+    }
+
+   private:
+    scoped_array<int8> data_;
+    Buffer valid_buffer_;
+    Buffer invalid_buffer_;
+  };
+
+  scoped_ptr< ::testing::StrictMock<MockCommandBufferEngine> > engine_;
+  ContextGroup::Ref group_;
+};
+
+class GLES2DecoderWithShaderTestBase : public GLES2DecoderTestBase {
+ public:
+  GLES2DecoderWithShaderTestBase()
+      : GLES2DecoderTestBase() {
+  }
+
+ protected:
+  virtual void SetUp() OVERRIDE;
+  virtual void TearDown() OVERRIDE;
+
 };
 
 }  // namespace gles2
