@@ -1,14 +1,14 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef GPU_COMMAND_BUFFER_SERVICE_RENDERBUFFER_MANAGER_H_
 #define GPU_COMMAND_BUFFER_SERVICE_RENDERBUFFER_MANAGER_H_
 
-#include <map>
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/hash_tables.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "gpu/command_buffer/service/gl_utils.h"
 
 namespace gpu {
@@ -23,9 +23,10 @@ class RenderbufferManager {
    public:
     typedef scoped_refptr<RenderbufferInfo> Ref;
 
-    explicit RenderbufferInfo(GLuint service_id)
-        : service_id_(service_id),
-          cleared_(false),
+    RenderbufferInfo(RenderbufferManager* manager, GLuint service_id)
+        : manager_(manager),
+          service_id_(service_id),
+          cleared_(true),
           has_been_bound_(false),
           samples_(0),
           internal_format_(GL_RGBA4),
@@ -41,10 +42,6 @@ class RenderbufferManager {
       return cleared_;
     }
 
-    void set_cleared() {
-      cleared_ = true;
-    }
-
     GLenum internal_format() const {
       return internal_format_;
     }
@@ -61,15 +58,6 @@ class RenderbufferManager {
       return height_;
     }
 
-    void SetInfo(
-        GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height) {
-      samples_ = samples;
-      internal_format_ = internalformat;
-      width_ = width;
-      height_ = height;
-      cleared_ = false;
-    }
-
     bool IsDeleted() const {
       return service_id_ == 0;
     }
@@ -82,16 +70,34 @@ class RenderbufferManager {
       return has_been_bound_ && !IsDeleted();
     }
 
+    size_t EstimatedSize();
+
    private:
     friend class RenderbufferManager;
     friend class base::RefCounted<RenderbufferInfo>;
 
-    ~RenderbufferInfo() { }
+    ~RenderbufferInfo();
+
+    void set_cleared() {
+      cleared_ = true;
+    }
+
+    void SetInfo(
+        GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height) {
+      samples_ = samples;
+      internal_format_ = internalformat;
+      width_ = width;
+      height_ = height;
+      cleared_ = false;
+    }
 
     void MarkAsDeleted() {
       service_id_ = 0;
     }
 
+    // RenderbufferManager that owns this RenderbufferInfo.
+    RenderbufferManager* manager_;
+
     // Service side renderbuffer id.
     GLuint service_id_;
 
@@ -112,13 +118,27 @@ class RenderbufferManager {
     GLsizei height_;
   };
 
-  explicit RenderbufferManager(GLint max_renderbuffer_size);
+  RenderbufferManager(GLint max_renderbuffer_size, GLint max_samples);
   ~RenderbufferManager();
 
   GLint max_renderbuffer_size() const {
     return max_renderbuffer_size_;
   }
 
+  GLint max_samples() const {
+    return max_samples_;
+  }
+
+  bool HaveUnclearedRenderbuffers() const {
+    return num_uncleared_renderbuffers_ != 0;
+  }
+
+  void SetInfo(
+      RenderbufferInfo* renderbuffer,
+      GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height);
+
+  void SetCleared(RenderbufferInfo* renderbuffer);
+
   // Must call before destruction.
   void Destroy(bool have_context);
 
@@ -135,11 +155,19 @@ class RenderbufferManager {
   bool GetClientId(GLuint service_id, GLuint* client_id) const;
 
  private:
+  void UpdateMemRepresented();
+
+  void StopTracking(RenderbufferInfo* renderbuffer);
+
   GLint max_renderbuffer_size_;
+  GLint max_samples_;
+
+  int num_uncleared_renderbuffers_;
+
+  size_t mem_represented_;
 
   // Info for each renderbuffer in the system.
-  // TODO(gman): Choose a faster container.
-  typedef std::map<GLuint, RenderbufferInfo::Ref> RenderbufferInfoMap;
+  typedef base::hash_map<GLuint, RenderbufferInfo::Ref> RenderbufferInfoMap;
   RenderbufferInfoMap renderbuffer_infos_;
 
   DISALLOW_COPY_AND_ASSIGN(RenderbufferManager);
