@@ -1,15 +1,16 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef GPU_COMMAND_BUFFER_SERVICE_SHADER_MANAGER_H_
 #define GPU_COMMAND_BUFFER_SERVICE_SHADER_MANAGER_H_
 
-#include <map>
 #include <string>
 #include "base/basictypes.h"
+#include "base/hash_tables.h"
 #include "base/logging.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "gpu/command_buffer/service/gl_utils.h"
 #include "gpu/command_buffer/service/shader_translator.h"
 
@@ -31,15 +32,14 @@ class ShaderManager {
     typedef scoped_refptr<ShaderInfo> Ref;
     typedef ShaderTranslator::VariableInfo VariableInfo;
 
-    explicit ShaderInfo(GLuint service_id, GLenum shader_type)
-        : use_count_(0),
-          service_id_(service_id),
-          shader_type_(shader_type),
-          valid_(false) {
+    void UpdateSource(const char* source) {
+      source_.reset(source ? new std::string(source) : NULL);
+      translated_source_.reset(NULL);
     }
 
-    void Update(const std::string& source) {
-      source_ = source;
+    void UpdateTranslatedSource(const char* translated_source) {
+      translated_source_.reset(
+          translated_source ? new std::string(translated_source) : NULL);
     }
 
     GLuint service_id() const {
@@ -50,19 +50,23 @@ class ShaderManager {
       return shader_type_;
     }
 
-    const std::string& source() const {
-      return source_;
+    const std::string* source() const {
+      return source_.get();
+    }
+
+    const std::string* translated_source() const {
+      return translated_source_.get();
     }
 
     void SetStatus(
-        bool valid, const std::string& log,
+        bool valid, const char* log,
         ShaderTranslatorInterface* translator);
 
     const VariableInfo* GetAttribInfo(const std::string& name) const;
     const VariableInfo* GetUniformInfo(const std::string& name) const;
 
-    const std::string& log_info() const {
-      return log_info_;
+    const std::string* log_info() const {
+      return log_info_.get();
     }
 
     bool IsValid() const {
@@ -83,21 +87,13 @@ class ShaderManager {
 
     friend class base::RefCounted<ShaderInfo>;
     friend class ShaderManager;
-    ~ShaderInfo() { }
 
-    void IncUseCount() {
-      ++use_count_;
-    }
+    ShaderInfo(GLuint service_id, GLenum shader_type);
+    ~ShaderInfo();
 
-    void DecUseCount() {
-      --use_count_;
-      DCHECK_GE(use_count_, 0);
-    }
-
-    void MarkAsDeleted() {
-      DCHECK_NE(service_id_, 0u);
-      service_id_ = 0;
-    }
+    void IncUseCount();
+    void DecUseCount();
+    void MarkAsDeleted();
 
     int use_count_;
 
@@ -110,10 +106,13 @@ class ShaderManager {
     bool valid_;
 
     // The shader source as passed to glShaderSource.
-    std::string source_;
+    scoped_ptr<std::string> source_;
+
+    // The translated shader source.
+    scoped_ptr<std::string> translated_source_;
 
     // The shader translation log.
-    std::string log_info_;
+    scoped_ptr<std::string> log_info_;
 
     // The type info when the shader was last compiled.
     VariableMap attrib_map_;
@@ -127,9 +126,10 @@ class ShaderManager {
   void Destroy(bool have_context);
 
   // Creates a shader info for the given shader ID.
-  void CreateShaderInfo(GLuint client_id,
-                        GLuint service_id,
-                        GLenum shader_type);
+  ShaderInfo* CreateShaderInfo(
+      GLuint client_id,
+      GLuint service_id,
+      GLenum shader_type);
 
   // Gets an existing shader info for the given shader ID. Returns NULL if none
   // exists.
@@ -147,9 +147,12 @@ class ShaderManager {
   // then we free the info.
   void UnuseShader(ShaderInfo* info);
 
+  // Check if a ShaderInfo is owned by this ShaderManager.
+  bool IsOwned(ShaderInfo* info);
+
  private:
   // Info for each shader by service side shader Id.
-  typedef std::map<GLuint, ShaderInfo::Ref> ShaderInfoMap;
+  typedef base::hash_map<GLuint, ShaderInfo::Ref> ShaderInfoMap;
   ShaderInfoMap shader_infos_;
 
   void RemoveShaderInfoIfUnused(ShaderInfo* info);
