@@ -1,15 +1,15 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef GPU_COMMAND_BUFFER_SERVICE_SHADER_TRANSLATOR_H_
 #define GPU_COMMAND_BUFFER_SERVICE_SHADER_TRANSLATOR_H_
 
-#include <map>
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/hash_tables.h"
+#include "base/memory/scoped_ptr.h"
 #include "third_party/angle/include/GLSLANG/ShaderLang.h"
 
 namespace gpu {
@@ -19,6 +19,16 @@ namespace gles2 {
 // validates GLSL ES 2.0 shaders on a true GLSL ES implementation.
 class ShaderTranslatorInterface {
  public:
+  enum GlslImplementationType {
+    kGlsl,
+    kGlslES
+  };
+
+  enum GlslBuiltInFunctionBehavior {
+    kGlslBuiltInFunctionOriginal,
+    kGlslBuiltInFunctionEmulated
+  };
+
   virtual ~ShaderTranslatorInterface() { }
 
   // Initializes the translator.
@@ -27,7 +37,8 @@ class ShaderTranslatorInterface {
       ShShaderType shader_type,
       ShShaderSpec shader_spec,
       const ShBuiltInResources* resources,
-      bool implementation_is_glsl_es) = 0;
+      GlslImplementationType glsl_implementation_type,
+      GlslBuiltInFunctionBehavior glsl_built_in_function_behavior) = 0;
 
   // Translates the given shader source.
   // Returns true if translation is successful, false otherwise.
@@ -45,15 +56,17 @@ class ShaderTranslatorInterface {
         : type(0),
           size(0) {
     }
-    VariableInfo(int _type, int _size)
+      VariableInfo(int _type, int _size, std::string _name)
         : type(_type),
-          size(_size) {
+          size(_size),
+          name(_name) {
     }
     int type;
     int size;
+    std::string name;  // name in the original shader source.
   };
   // Mapping between variable name and info.
-  typedef std::map<std::string, VariableInfo> VariableMap;
+  typedef base::hash_map<std::string, VariableInfo> VariableMap;
   virtual const VariableMap& attrib_map() const = 0;
   virtual const VariableMap& uniform_map() const = 0;
 };
@@ -62,25 +75,26 @@ class ShaderTranslatorInterface {
 class ShaderTranslator : public ShaderTranslatorInterface {
  public:
   ShaderTranslator();
-  ~ShaderTranslator();
+  virtual ~ShaderTranslator();
 
   // Overridden from ShaderTranslatorInterface.
   virtual bool Init(
       ShShaderType shader_type,
       ShShaderSpec shader_spec,
       const ShBuiltInResources* resources,
-      bool implementation_is_glsl_es);
+      GlslImplementationType glsl_implementation_type,
+      GlslBuiltInFunctionBehavior glsl_built_in_function_behavior) OVERRIDE;
 
   // Overridden from ShaderTranslatorInterface.
-  virtual bool Translate(const char* shader);
+  virtual bool Translate(const char* shader) OVERRIDE;
 
   // Overridden from ShaderTranslatorInterface.
-  virtual const char* translated_shader() const;
-  virtual const char* info_log() const;
+  virtual const char* translated_shader() const OVERRIDE;
+  virtual const char* info_log() const OVERRIDE;
 
   // Overridden from ShaderTranslatorInterface.
-  virtual const VariableMap& attrib_map() const;
-  virtual const VariableMap& uniform_map() const;
+  virtual const VariableMap& attrib_map() const OVERRIDE;
+  virtual const VariableMap& uniform_map() const OVERRIDE;
 
  private:
   void ClearResults();
@@ -91,6 +105,7 @@ class ShaderTranslator : public ShaderTranslatorInterface {
   VariableMap attrib_map_;
   VariableMap uniform_map_;
   bool implementation_is_glsl_es_;
+  bool needs_built_in_function_emulation_;
 
   DISALLOW_COPY_AND_ASSIGN(ShaderTranslator);
 };
