@@ -1,16 +1,16 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "app/gfx/gl/gl_context.h"
 #include "base/at_exit.h"
 #include "base/command_line.h"
 #include "base/logging.h"
 #include "gpu/demos/framework/window.h"
+#include "ui/gfx/gl/gl_surface.h"
 
-#if defined(OS_LINUX)
+#if defined(TOOLKIT_USES_GTK)
 #include <gtk/gtk.h>
-#endif  // OS_LINUX
+#endif  // TOOLKIT_USES_GTK
 
 namespace {
 static const int kWindowWidth = 512;
@@ -18,9 +18,9 @@ static const int kWindowHeight = 512;
 }  // namespace.
 
 int main(int argc, char** argv) {
-#if defined(OS_LINUX)
+#if defined(TOOLKIT_USES_GTK)
   gtk_init(&argc, &argv);
-#endif  // OS_LINUX
+#endif  // TOOLKIT_USES_GTK
 
   // AtExitManager is used by singleton classes to delete themselves when
   // the program terminates.
@@ -28,7 +28,7 @@ int main(int argc, char** argv) {
 
   CommandLine::Init(argc, argv);
 
-  gfx::GLContext::InitializeOneOff();
+  gfx::GLSurface::InitializeOneOff();
 
   gpu::demos::Window window;
   CHECK(window.Init(kWindowWidth, kWindowHeight));
