@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -29,13 +29,13 @@ namespace IPC {
 // the channel with the mode set to one of the NAMED modes. NAMED modes are
 // currently used by automation and service processes.
 
-class Channel : public Message::Sender {
+class IPC_EXPORT Channel : public Message::Sender {
   // Security tests need access to the pipe handle.
   friend class ChannelTest;
 
  public:
   // Implemented by consumers of a Channel to receive messages.
-  class Listener {
+  class IPC_EXPORT Listener {
    public:
     virtual ~Listener() {}
 
@@ -62,27 +62,44 @@ class Channel : public Message::Sender {
 #endif  // OS_POSIX
   };
 
+  // Flags to test modes
+  enum ModeFlags {
+    MODE_NO_FLAG = 0x0,
+    MODE_SERVER_FLAG = 0x1,
+    MODE_CLIENT_FLAG = 0x2,
+    MODE_NAMED_FLAG = 0x4,
+#if defined(OS_POSIX)
+    MODE_OPEN_ACCESS_FLAG = 0x8, // Don't restrict access based on client UID.
+#endif
+  };
+
+  // Some Standard Modes
   enum Mode {
-    MODE_NONE,
-    MODE_SERVER,
-    MODE_CLIENT,
+    MODE_NONE = MODE_NO_FLAG,
+    MODE_SERVER = MODE_SERVER_FLAG,
+    MODE_CLIENT = MODE_CLIENT_FLAG,
     // Channels on Windows are named by default and accessible from other
     // processes. On POSIX channels are anonymous by default and not accessible
     // from other processes. Named channels work via named unix domain sockets.
-    // On Windows MODE_NAMED_SERVER == MODE_SERVER and
-    // MODE_NAMED_CLIENT == MODE_CLIENT.
-    MODE_NAMED_SERVER,
-    MODE_NAMED_CLIENT,
+    // On Windows MODE_NAMED_SERVER is equivalent to MODE_SERVER and
+    // MODE_NAMED_CLIENT is equivalent to MODE_CLIENT.
+    MODE_NAMED_SERVER = MODE_SERVER_FLAG | MODE_NAMED_FLAG,
+    MODE_NAMED_CLIENT = MODE_CLIENT_FLAG | MODE_NAMED_FLAG,
+#if defined(OS_POSIX)
+    // An "open" named server accepts connections from ANY client.
+    // The caller must then implement their own access-control based on the
+    // client process' user Id.
+    MODE_OPEN_NAMED_SERVER = MODE_OPEN_ACCESS_FLAG | MODE_SERVER_FLAG |
+                             MODE_NAMED_FLAG
+#endif
   };
 
-  enum {
-    // The maximum message size in bytes. Attempting to receive a
-    // message of this size or bigger results in a channel error.
-    kMaximumMessageSize = 128 * 1024 * 1024,
+  // The maximum message size in bytes. Attempting to receive a message of this
+  // size or bigger results in a channel error.
+  static const size_t kMaximumMessageSize = 128 * 1024 * 1024;
 
-    // Ammount of data to read at once from the pipe.
-    kReadBufferSize = 4 * 1024
-  };
+  // Ammount of data to read at once from the pipe.
+  static const size_t kReadBufferSize = 4 * 1024;
 
   // Initialize a Channel.
   //
@@ -100,7 +117,7 @@ class Channel : public Message::Sender {
   Channel(const IPC::ChannelHandle &channel_handle, Mode mode,
           Listener* listener);
 
-  ~Channel();
+  virtual ~Channel();
 
   // Connect the pipe.  On the server side, this will initiate
   // waiting for connections.  On the client, it attempts to
@@ -123,14 +140,20 @@ class Channel : public Message::Sender {
   //
   // |message| must be allocated using operator new.  This object will be
   // deleted once the contents of the Message have been sent.
-  virtual bool Send(Message* message);
+  virtual bool Send(Message* message) OVERRIDE;
 
 #if defined(OS_POSIX) && !defined(OS_NACL)
   // On POSIX an IPC::Channel wraps a socketpair(), this method returns the
   // FD # for the client end of the socket.
   // This method may only be called on the server side of a channel.
+  // This method can be called on any thread.
   int GetClientFileDescriptor() const;
 
+  // Same as GetClientFileDescriptor, but transfers the ownership of the
+  // file descriptor to the caller.
+  // This method can be called on any thread.
+  int TakeClientFileDescriptor();
+
   // On POSIX an IPC::Channel can either wrap an established socket, or it
   // can wrap a socket that is listening for connections. Currently an
   // IPC::Channel that listens for connections can only accept one connection
@@ -143,10 +166,25 @@ class Channel : public Message::Sender {
   // currently connected.
   bool HasAcceptedConnection() const;
 
+  // Returns true if the peer process' effective user id can be determined, in
+  // which case the supplied client_euid is updated with it.
+  bool GetClientEuid(uid_t* client_euid) const;
+
   // Closes any currently connected socket, and returns to a listening state
   // for more connections.
   void ResetToAcceptingConnectionState();
-#endif  // defined(OS_POSIX)
+#endif  // defined(OS_POSIX) && !defined(OS_NACL)
+
+  // Returns true if a named server channel is initialized on the given channel
+  // ID. Even if true, the server may have already accepted a connection.
+  static bool IsNamedServerInitialized(const std::string& channel_id);
+
+#if defined(OS_LINUX)
+  // Sandboxed processes live in a PID namespace, so when sending the IPC hello
+  // message from client to server we need to send the PID from the global
+  // PID namespace.
+  static void SetGlobalPid(int pid);
+#endif
 
  protected:
   // Used in Chrome by the TestSink to provide a dummy channel implementation
