@@ -1,4 +1,4 @@
-// Copyright (c) 2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -25,13 +25,20 @@
 // As we cannot anticipate when the sender will provide us with file
 // descriptors, we have to make the decision about whether we call read() or
 // recvmsg() before we actually make the call. The easiest option is to
-// create a dedicated socketpair() for exchanging file descriptors.
+// create a dedicated socketpair() for exchanging file descriptors. Any file
+// descriptors are split out of a message, with the non-file-descriptor payload
+// going over the normal connection, and the file descriptors being sent
+// separately over the other channel. When read()ing from a channel, we'll
+// notice if the message was supposed to have come with file descriptors and
+// use recvmsg on the other socketpair to retrieve them and combine them
+// back with the rest of the message.
+//
 // Mac can also run in IPC_USES_READWRITE mode if necessary, but at this time
 // doesn't take a performance hit from recvmsg and sendmsg, so it doesn't
 // make sense to waste resources on having the separate dedicated socketpair.
 // It is however useful for debugging between Linux and Mac to be able to turn
 // this switch 'on' on the Mac as well.
-
+//
 // The HELLO message from the client to the server is always sent using
 // sendmsg because it will contain the file descriptor that the server
 // needs to send file descriptors in later messages.
@@ -45,32 +52,38 @@ class Channel::ChannelImpl : public MessageLoopForIO::Watcher {
   // Mirror methods of Channel, see ipc_channel.h for description.
   ChannelImpl(const IPC::ChannelHandle& channel_handle, Mode mode,
               Listener* listener);
-  ~ChannelImpl();
+  virtual ~ChannelImpl();
   bool Connect();
   void Close();
   void set_listener(Listener* listener) { listener_ = listener; }
   bool Send(Message* message);
-  int GetClientFileDescriptor() const;
+  int GetClientFileDescriptor();
+  int TakeClientFileDescriptor();
+  void CloseClientFileDescriptor();
   bool AcceptsConnections() const;
   bool HasAcceptedConnection() const;
+  bool GetClientEuid(uid_t* client_euid) const;
   void ResetToAcceptingConnectionState();
+  static bool IsNamedServerInitialized(const std::string& channel_id);
+#if defined(OS_LINUX)
+  static void SetGlobalPid(int pid);
+#endif  // OS_LINUX
 
  private:
-  bool CreatePipe(const IPC::ChannelHandle& channel_handle,
-                  bool uses_domain_sockets,
-                  bool listening_socket);
+  bool CreatePipe(const IPC::ChannelHandle& channel_handle);
 
   bool ProcessIncomingMessages();
   bool ProcessOutgoingMessages();
 
   bool AcceptConnection();
   void ClosePipeOnError();
+  int GetHelloMessageProcId();
   void QueueHelloMessage();
   bool IsHelloMessage(const Message* m) const;
 
   // MessageLoopForIO::Watcher implementation.
-  virtual void OnFileCanReadWithoutBlocking(int fd);
-  virtual void OnFileCanWriteWithoutBlocking(int fd);
+  virtual void OnFileCanReadWithoutBlocking(int fd) OVERRIDE;
+  virtual void OnFileCanWriteWithoutBlocking(int fd) OVERRIDE;
 
   Mode mode_;
 
@@ -98,6 +111,7 @@ class Channel::ChannelImpl : public MessageLoopForIO::Watcher {
   // For a server, the client end of our socketpair() -- the other end of our
   // pipe_ that is passed to the client.
   int client_pipe_;
+  base::Lock client_pipe_lock_;  // Lock that protects |client_pipe_|.
 
 #if defined(IPC_USES_READWRITE)
   // Linux/BSD use a dedicated socketpair() for passing file descriptors.
@@ -117,19 +131,18 @@ class Channel::ChannelImpl : public MessageLoopForIO::Watcher {
   // We read from the pipe into this buffer
   char input_buf_[Channel::kReadBufferSize];
 
-  enum {
-    // We assume a worst case: kReadBufferSize bytes of messages, where each
-    // message has no payload and a full complement of descriptors.
-    MAX_READ_FDS = (Channel::kReadBufferSize / sizeof(IPC::Message::Header)) *
-                   FileDescriptorSet::MAX_DESCRIPTORS_PER_MESSAGE,
-  };
+  // We assume a worst case: kReadBufferSize bytes of messages, where each
+  // message has no payload and a full complement of descriptors.
+  static const size_t kMaxReadFDs =
+      (Channel::kReadBufferSize / sizeof(IPC::Message::Header)) *
+      FileDescriptorSet::kMaxDescriptorsPerMessage;
 
   // This is a control message buffer large enough to hold kMaxReadFDs
 #if defined(OS_MACOSX)
   // TODO(agl): OSX appears to have non-constant CMSG macros!
   char input_cmsg_buf_[1024];
 #else
-  char input_cmsg_buf_[CMSG_SPACE(sizeof(int) * MAX_READ_FDS)];
+  char input_cmsg_buf_[CMSG_SPACE(sizeof(int) * kMaxReadFDs)];
 #endif
 
   // Large messages that span multiple pipe buffers, get built-up using
@@ -140,9 +153,12 @@ class Channel::ChannelImpl : public MessageLoopForIO::Watcher {
   // True if we are responsible for unlinking the unix domain socket file.
   bool must_unlink_;
 
-  ScopedRunnableMethodFactory<ChannelImpl> factory_;
+#if defined(OS_LINUX)
+  // If non-zero, overrides the process ID sent in the hello message.
+  static int global_pid_;
+#endif  // OS_LINUX
 
-  DISALLOW_COPY_AND_ASSIGN(ChannelImpl);
+  DISALLOW_IMPLICIT_CONSTRUCTORS(ChannelImpl);
 };
 
 // The maximum length of the name of a pipe for MODE_NAMED_SERVER or
