@@ -1,29 +1,29 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "base/message_loop.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/threading/thread.h"
+#include "base/bind.h"
+#include "base/compiler_specific.h"
+#include "base/location.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "ipc/ipc_channel_proxy.h"
 #include "ipc/ipc_logging.h"
 #include "ipc/ipc_message_utils.h"
 
 namespace IPC {
 
-//------------------------------------------------------------------------------
-
-// This task ensures the message is deleted if the task is deleted without
+// This helper ensures the message is deleted if the task is deleted without
 // having been run.
-class SendTask : public Task {
+class SendCallbackHelper
+    : public base::RefCountedThreadSafe<SendCallbackHelper> {
  public:
-  SendTask(ChannelProxy::Context* context, Message* message)
+  SendCallbackHelper(ChannelProxy::Context* context, Message* message)
       : context_(context),
         message_(message) {
   }
 
-  virtual void Run() {
+  void Send() {
     context_->OnSendMessage(message_.release());
   }
 
@@ -31,7 +31,7 @@ class SendTask : public Task {
   scoped_refptr<ChannelProxy::Context> context_;
   scoped_ptr<Message> message_;
 
-  DISALLOW_COPY_AND_ASSIGN(SendTask);
+  DISALLOW_COPY_AND_ASSIGN(SendCallbackHelper);
 };
 
 //------------------------------------------------------------------------------
@@ -61,14 +61,17 @@ void ChannelProxy::MessageFilter::OnDestruct() const {
 //------------------------------------------------------------------------------
 
 ChannelProxy::Context::Context(Channel::Listener* listener,
-                               MessageLoop* ipc_message_loop)
-    : listener_message_loop_(MessageLoop::current()),
+                               base::MessageLoopProxy* ipc_message_loop)
+    : listener_message_loop_(base::MessageLoopProxy::current()),
       listener_(listener),
       ipc_message_loop_(ipc_message_loop),
       peer_pid_(0),
       channel_connected_called_(false) {
 }
 
+ChannelProxy::Context::~Context() {
+}
+
 void ChannelProxy::Context::CreateChannel(const IPC::ChannelHandle& handle,
                                           const Channel::Mode& mode) {
   DCHECK(channel_.get() == NULL);
@@ -109,8 +112,8 @@ bool ChannelProxy::Context::OnMessageReceivedNoFilter(const Message& message) {
   // this thread is active.  That should be a reasonable assumption, but it
   // feels risky.  We may want to invent some more indirect way of referring to
   // a MessageLoop if this becomes a problem.
-  listener_message_loop_->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &Context::OnDispatchMessage, message));
+  listener_message_loop_->PostTask(
+      FROM_HERE, base::Bind(&Context::OnDispatchMessage, this, message));
   return true;
 }
 
@@ -127,8 +130,8 @@ void ChannelProxy::Context::OnChannelConnected(int32 peer_pid) {
     filters_[i]->OnChannelConnected(peer_pid);
 
   // See above comment about using listener_message_loop_ here.
-  listener_message_loop_->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &Context::OnDispatchConnected));
+  listener_message_loop_->PostTask(
+      FROM_HERE, base::Bind(&Context::OnDispatchConnected, this));
 }
 
 // Called on the IPC::Channel thread
@@ -137,8 +140,8 @@ void ChannelProxy::Context::OnChannelError() {
     filters_[i]->OnChannelError();
 
   // See above comment about using listener_message_loop_ here.
-  listener_message_loop_->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &Context::OnDispatchError));
+  listener_message_loop_->PostTask(
+      FROM_HERE, base::Bind(&Context::OnDispatchError, this));
 }
 
 // Called on the IPC::Channel thread
@@ -193,22 +196,22 @@ void ChannelProxy::Context::OnSendMessage(Message* message) {
 
 // Called on the IPC::Channel thread
 void ChannelProxy::Context::OnAddFilter() {
-  std::vector<scoped_refptr<MessageFilter> > filters;
+  std::vector<scoped_refptr<MessageFilter> > new_filters;
   {
     base::AutoLock auto_lock(pending_filters_lock_);
-    filters.swap(pending_filters_);
+    new_filters.swap(pending_filters_);
   }
 
-  for (size_t i = 0; i < filters.size(); ++i) {
-    filters_.push_back(filters[i]);
+  for (size_t i = 0; i < new_filters.size(); ++i) {
+    filters_.push_back(new_filters[i]);
 
     // If the channel has already been created, then we need to send this
     // message so that the filter gets access to the Channel.
     if (channel_.get())
-      filters[i]->OnFilterAdded(channel_.get());
-    // Ditto for the peer process id.
+      new_filters[i]->OnFilterAdded(channel_.get());
+    // Ditto for if the channel has been connected.
     if (peer_pid_)
-      filters[i]->OnChannelConnected(peer_pid_);
+      new_filters[i]->OnChannelConnected(peer_pid_);
   }
 }
 
@@ -230,8 +233,7 @@ void ChannelProxy::Context::AddFilter(MessageFilter* filter) {
   base::AutoLock auto_lock(pending_filters_lock_);
   pending_filters_.push_back(make_scoped_refptr(filter));
   ipc_message_loop_->PostTask(
-      FROM_HERE,
-      NewRunnableMethod(this, &Context::OnAddFilter));
+      FROM_HERE, base::Bind(&Context::OnAddFilter, this));
 }
 
 // Called on the listener's thread
@@ -281,18 +283,17 @@ void ChannelProxy::Context::OnDispatchError() {
 ChannelProxy::ChannelProxy(const IPC::ChannelHandle& channel_handle,
                            Channel::Mode mode,
                            Channel::Listener* listener,
-                           MessageLoop* ipc_thread)
-    : context_(new Context(listener, ipc_thread)) {
-  Init(channel_handle, mode, ipc_thread, true);
+                           base::MessageLoopProxy* ipc_thread)
+    : context_(new Context(listener, ipc_thread)),
+      outgoing_message_filter_(NULL),
+      did_init_(false) {
+  Init(channel_handle, mode, true);
 }
 
-ChannelProxy::ChannelProxy(const IPC::ChannelHandle& channel_handle,
-                           Channel::Mode mode,
-                           MessageLoop* ipc_thread,
-                           Context* context,
-                           bool create_pipe_now)
-    : context_(context) {
-  Init(channel_handle, mode, ipc_thread, create_pipe_now);
+ChannelProxy::ChannelProxy(Context* context)
+    : context_(context),
+      outgoing_message_filter_(NULL),
+      did_init_(false) {
 }
 
 ChannelProxy::~ChannelProxy() {
@@ -300,14 +301,15 @@ ChannelProxy::~ChannelProxy() {
 }
 
 void ChannelProxy::Init(const IPC::ChannelHandle& channel_handle,
-                        Channel::Mode mode, MessageLoop* ipc_thread_loop,
+                        Channel::Mode mode,
                         bool create_pipe_now) {
+  DCHECK(!did_init_);
 #if defined(OS_POSIX)
   // When we are creating a server on POSIX, we need its file descriptor
   // to be created immediately so that it can be accessed and passed
   // to other processes. Forcing it to be created immediately avoids
   // race conditions that may otherwise arise.
-  if (mode == Channel::MODE_SERVER) {
+  if (mode & Channel::MODE_SERVER_FLAG) {
     create_pipe_now = true;
   }
 #endif  // defined(OS_POSIX)
@@ -319,13 +321,16 @@ void ChannelProxy::Init(const IPC::ChannelHandle& channel_handle,
     // to connect and get an error since the pipe doesn't exist yet.
     context_->CreateChannel(channel_handle, mode);
   } else {
-    context_->ipc_message_loop()->PostTask(FROM_HERE, NewRunnableMethod(
-        context_.get(), &Context::CreateChannel, channel_handle, mode));
+    context_->ipc_message_loop()->PostTask(
+        FROM_HERE, base::Bind(&Context::CreateChannel, context_.get(),
+                              channel_handle, mode));
   }
 
   // complete initialization on the background thread
-  context_->ipc_message_loop()->PostTask(FROM_HERE, NewRunnableMethod(
-      context_.get(), &Context::OnChannelOpened));
+  context_->ipc_message_loop()->PostTask(
+      FROM_HERE, base::Bind(&Context::OnChannelOpened, context_.get()));
+
+  did_init_ = true;
 }
 
 void ChannelProxy::Close() {
@@ -335,18 +340,24 @@ void ChannelProxy::Close() {
   context_->Clear();
 
   if (context_->ipc_message_loop()) {
-    context_->ipc_message_loop()->PostTask(FROM_HERE, NewRunnableMethod(
-        context_.get(), &Context::OnChannelClosed));
+    context_->ipc_message_loop()->PostTask(
+        FROM_HERE, base::Bind(&Context::OnChannelClosed, context_.get()));
   }
 }
 
 bool ChannelProxy::Send(Message* message) {
+  DCHECK(did_init_);
+  if (outgoing_message_filter())
+    message = outgoing_message_filter()->Rewrite(message);
+
 #ifdef IPC_MESSAGE_LOG_ENABLED
   Logging::GetInstance()->OnSendMessage(message, context_->channel_id());
 #endif
 
-  context_->ipc_message_loop()->PostTask(FROM_HERE,
-                                         new SendTask(context_.get(), message));
+  context_->ipc_message_loop()->PostTask(
+      FROM_HERE,
+      base::Bind(&SendCallbackHelper::Send,
+                 new SendCallbackHelper(context_.get(), message)));
   return true;
 }
 
@@ -356,10 +367,8 @@ void ChannelProxy::AddFilter(MessageFilter* filter) {
 
 void ChannelProxy::RemoveFilter(MessageFilter* filter) {
   context_->ipc_message_loop()->PostTask(
-      FROM_HERE, NewRunnableMethod(
-          context_.get(),
-          &Context::OnRemoveFilter,
-          make_scoped_refptr(filter)));
+      FROM_HERE, base::Bind(&Context::OnRemoveFilter, context_.get(),
+                            make_scoped_refptr(filter)));
 }
 
 void ChannelProxy::ClearIPCMessageLoop() {
@@ -369,13 +378,26 @@ void ChannelProxy::ClearIPCMessageLoop() {
 #if defined(OS_POSIX) && !defined(OS_NACL)
 // See the TODO regarding lazy initialization of the channel in
 // ChannelProxy::Init().
-// We assume that IPC::Channel::GetClientFileDescriptorMapping() is thread-safe.
-int ChannelProxy::GetClientFileDescriptor() const {
-  Channel *channel = context_.get()->channel_.get();
+int ChannelProxy::GetClientFileDescriptor() {
+  Channel* channel = context_.get()->channel_.get();
   // Channel must have been created first.
   DCHECK(channel) << context_.get()->channel_id_;
   return channel->GetClientFileDescriptor();
 }
+
+int ChannelProxy::TakeClientFileDescriptor() {
+  Channel* channel = context_.get()->channel_.get();
+  // Channel must have been created first.
+  DCHECK(channel) << context_.get()->channel_id_;
+  return channel->TakeClientFileDescriptor();
+}
+
+bool ChannelProxy::GetClientEuid(uid_t* client_euid) const {
+  Channel* channel = context_.get()->channel_.get();
+  // Channel must have been created first.
+  DCHECK(channel) << context_.get()->channel_id_;
+  return channel->GetClientEuid(client_euid);
+}
 #endif
 
 //-----------------------------------------------------------------------------
