@@ -1,24 +1,23 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#ifndef IPC_IPC_CHANNEL_PROXY_H__
-#define IPC_IPC_CHANNEL_PROXY_H__
+#ifndef IPC_IPC_CHANNEL_PROXY_H_
+#define IPC_IPC_CHANNEL_PROXY_H_
 #pragma once
 
 #include <vector>
 
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/message_loop_proxy.h"
 #include "base/synchronization/lock.h"
 #include "ipc/ipc_channel.h"
 #include "ipc/ipc_channel_handle.h"
 
-class MessageLoop;
-
 namespace IPC {
 
-class SendTask;
+class SendCallbackHelper;
 
 //-----------------------------------------------------------------------------
 // IPC::ChannelProxy
@@ -48,14 +47,14 @@ class SendTask;
 // The consumer of IPC::ChannelProxy is responsible for allocating the Thread
 // instance where the IPC::Channel will be created and operated.
 //
-class ChannelProxy : public Message::Sender {
+class IPC_EXPORT ChannelProxy : public Message::Sender {
  public:
 
   struct MessageFilterTraits;
 
   // A class that receives messages on the thread where the IPC channel is
   // running.  It can choose to prevent the default action for an IPC message.
-  class MessageFilter
+  class IPC_EXPORT MessageFilter
       : public base::RefCountedThreadSafe<MessageFilter, MessageFilterTraits> {
    public:
     MessageFilter();
@@ -99,6 +98,15 @@ class ChannelProxy : public Message::Sender {
     }
   };
 
+  // Interface for a filter to be imposed on outgoing messages which can
+  // re-write the message.  Used mainly for testing.
+  class OutgoingMessageFilter {
+   public:
+    // Returns a re-written message, freeing the original, or simply the
+    // original unchanged if no rewrite indicated.
+    virtual Message *Rewrite(Message *message) = 0;
+  };
+
   // Initializes a channel proxy.  The channel_handle and mode parameters are
   // passed directly to the underlying IPC::Channel.  The listener is called on
   // the thread that creates the ChannelProxy.  The filter's OnMessageReceived
@@ -110,10 +118,23 @@ class ChannelProxy : public Message::Sender {
   ChannelProxy(const IPC::ChannelHandle& channel_handle,
                Channel::Mode mode,
                Channel::Listener* listener,
-               MessageLoop* ipc_thread_loop);
+               base::MessageLoopProxy* ipc_thread_loop);
+
+  // Creates an uninitialized channel proxy. Init must be called to receive
+  // or send any messages. This two-step setup allows message filters to be
+  // added before any messages are sent or received.
+  ChannelProxy(Channel::Listener* listener,
+               base::MessageLoopProxy* ipc_thread_loop);
 
   virtual ~ChannelProxy();
 
+  // Initializes the channel proxy. Only call this once to initialize a channel
+  // proxy that was not initialized in its constructor. If create_pipe_now is
+  // true, the pipe is created synchronously. Otherwise it's created on the IO
+  // thread.
+  void Init(const IPC::ChannelHandle& channel_handle, Channel::Mode mode,
+            bool create_pipe_now);
+
   // Close the IPC::Channel.  This operation completes asynchronously, once the
   // background thread processes the command to close the channel.  It is ok to
   // call this method multiple times.  Redundant calls are ignored.
@@ -126,7 +147,7 @@ class ChannelProxy : public Message::Sender {
 
   // Send a message asynchronously.  The message is routed to the background
   // thread where it is passed to the IPC::Channel's Send method.
-  virtual bool Send(Message* message);
+  virtual bool Send(Message* message) OVERRIDE;
 
   // Used to intercept messages as they are received on the background thread.
   //
@@ -140,32 +161,35 @@ class ChannelProxy : public Message::Sender {
   void AddFilter(MessageFilter* filter);
   void RemoveFilter(MessageFilter* filter);
 
+  void set_outgoing_message_filter(OutgoingMessageFilter* filter) {
+    outgoing_message_filter_ = filter;
+  }
+
   // Called to clear the pointer to the IPC message loop when it's going away.
   void ClearIPCMessageLoop();
 
 #if defined(OS_POSIX)
   // Calls through to the underlying channel's methods.
-  int GetClientFileDescriptor() const;
+  int GetClientFileDescriptor();
+  int TakeClientFileDescriptor();
+  bool GetClientEuid(uid_t* client_euid) const;
 #endif  // defined(OS_POSIX)
 
  protected:
   class Context;
   // A subclass uses this constructor if it needs to add more information
-  // to the internal state.  If create_pipe_now is true, the pipe is created
-  // immediately.  Otherwise it's created on the IO thread.
-  ChannelProxy(const IPC::ChannelHandle& channel_handle,
-               Channel::Mode mode,
-               MessageLoop* ipc_thread_loop,
-               Context* context,
-               bool create_pipe_now);
+  // to the internal state.
+  ChannelProxy(Context* context);
 
   // Used internally to hold state that is referenced on the IPC thread.
   class Context : public base::RefCountedThreadSafe<Context>,
                   public Channel::Listener {
    public:
-    Context(Channel::Listener* listener, MessageLoop* ipc_thread);
+    Context(Channel::Listener* listener, base::MessageLoopProxy* ipc_thread);
     void ClearIPCMessageLoop() { ipc_message_loop_ = NULL; }
-    MessageLoop* ipc_message_loop() const { return ipc_message_loop_; }
+    base::MessageLoopProxy* ipc_message_loop() const {
+      return ipc_message_loop_.get();
+    }
     const std::string& channel_id() const { return channel_id_; }
 
     // Dispatches a message on the listener thread.
@@ -173,12 +197,12 @@ class ChannelProxy : public Message::Sender {
 
    protected:
     friend class base::RefCountedThreadSafe<Context>;
-    virtual ~Context() { }
+    virtual ~Context();
 
     // IPC::Channel::Listener methods:
-    virtual bool OnMessageReceived(const Message& message);
-    virtual void OnChannelConnected(int32 peer_pid);
-    virtual void OnChannelError();
+    virtual bool OnMessageReceived(const Message& message) OVERRIDE;
+    virtual void OnChannelConnected(int32 peer_pid) OVERRIDE;
+    virtual void OnChannelError() OVERRIDE;
 
     // Like OnMessageReceived but doesn't try the filters.
     bool OnMessageReceivedNoFilter(const Message& message);
@@ -198,7 +222,7 @@ class ChannelProxy : public Message::Sender {
 
    private:
     friend class ChannelProxy;
-    friend class SendTask;
+    friend class SendCallbackHelper;
 
     // Create the Channel
     void CreateChannel(const IPC::ChannelHandle& channel_handle,
@@ -214,12 +238,12 @@ class ChannelProxy : public Message::Sender {
     void OnDispatchConnected();
     void OnDispatchError();
 
-    MessageLoop* listener_message_loop_;
+    scoped_refptr<base::MessageLoopProxy> listener_message_loop_;
     Channel::Listener* listener_;
 
     // List of filters.  This is only accessed on the IPC thread.
     std::vector<scoped_refptr<MessageFilter> > filters_;
-    MessageLoop* ipc_message_loop_;
+    scoped_refptr<base::MessageLoopProxy> ipc_message_loop_;
     scoped_ptr<Channel> channel_;
     std::string channel_id_;
     int peer_pid_;
@@ -234,18 +258,24 @@ class ChannelProxy : public Message::Sender {
 
   Context* context() { return context_; }
 
- private:
-  friend class SendTask;
+  OutgoingMessageFilter* outgoing_message_filter() {
+    return outgoing_message_filter_;
+  }
 
-  void Init(const IPC::ChannelHandle& channel_handle, Channel::Mode mode,
-            MessageLoop* ipc_thread_loop, bool create_pipe_now);
+ private:
+  friend class SendCallbackHelper;
 
   // By maintaining this indirection (ref-counted) to our internal state, we
   // can safely be destroyed while the background thread continues to do stuff
   // that involves this data.
   scoped_refptr<Context> context_;
+
+  OutgoingMessageFilter* outgoing_message_filter_;
+
+  // Whether the channel has been initialized.
+  bool did_init_;
 };
 
 }  // namespace IPC
 
-#endif  // IPC_IPC_CHANNEL_PROXY_H__
+#endif  // IPC_IPC_CHANNEL_PROXY_H_
