@@ -1,14 +1,13 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "ipc/ipc_channel_win.h"
 
 #include <windows.h>
-#include <sddl.h>
-#include <sstream>
 
 #include "base/auto_reset.h"
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
 #include "base/threading/non_thread_safe.h"
@@ -19,72 +18,6 @@
 
 namespace IPC {
 
-namespace {
-
-// Creates a security descriptor with a DACL that has one ace giving full
-// access to the current logon session.
-// The security descriptor returned must be freed using LocalFree.
-// The function returns true if it succeeds, false otherwise.
-bool GetLogonSessionOnlyDACL(SECURITY_DESCRIPTOR** security_descriptor) {
-  // Get the current token.
-  HANDLE token = NULL;
-  if (!::OpenProcessToken(::GetCurrentProcess(), TOKEN_QUERY, &token))
-    return false;
-  base::win::ScopedHandle token_scoped(token);
-
-  // Get the size of the TokenGroups structure.
-  DWORD size = 0;
-  BOOL result = GetTokenInformation(token, TokenGroups, NULL,  0, &size);
-  if (result != FALSE && GetLastError() != ERROR_INSUFFICIENT_BUFFER)
-    return false;
-
-  // Get the data.
-  scoped_array<char> token_groups_chars(new char[size]);
-  TOKEN_GROUPS* token_groups =
-      reinterpret_cast<TOKEN_GROUPS*>(token_groups_chars.get());
-
-  if (!GetTokenInformation(token, TokenGroups, token_groups, size, &size))
-    return false;
-
-  // Look for the logon sid.
-  SID* logon_sid = NULL;
-  for (unsigned int i = 0; i < token_groups->GroupCount ; ++i) {
-    if ((token_groups->Groups[i].Attributes & SE_GROUP_LOGON_ID) != 0) {
-        logon_sid = static_cast<SID*>(token_groups->Groups[i].Sid);
-        break;
-    }
-  }
-
-  if (!logon_sid)
-    return false;
-
-  // Convert the data to a string.
-  wchar_t* sid_string;
-  if (!ConvertSidToStringSid(logon_sid, &sid_string))
-    return false;
-
-  static const wchar_t dacl_format[] = L"D:(A;OICI;GA;;;%ls)";
-  wchar_t dacl[SECURITY_MAX_SID_SIZE + arraysize(dacl_format) + 1] = {0};
-  wsprintf(dacl, dacl_format, sid_string);
-
-  LocalFree(sid_string);
-
-  // Convert the string to a security descriptor
-  if (!ConvertStringSecurityDescriptorToSecurityDescriptor(
-      dacl,
-      SDDL_REVISION_1,
-      reinterpret_cast<PSECURITY_DESCRIPTOR*>(security_descriptor),
-      NULL)) {
-    return false;
-  }
-
-  return true;
-}
-
-}  // namespace
-
-//------------------------------------------------------------------------------
-
 Channel::ChannelImpl::State::State(ChannelImpl* channel) : is_pending(false) {
   memset(&context.overlapped, 0, sizeof(context.overlapped));
   context.handler = channel;
@@ -95,38 +28,16 @@ Channel::ChannelImpl::State::~State() {
                  starts_with_io_context);
 }
 
-//------------------------------------------------------------------------------
-
 Channel::ChannelImpl::ChannelImpl(const IPC::ChannelHandle &channel_handle,
                                   Mode mode, Listener* listener)
     : ALLOW_THIS_IN_INITIALIZER_LIST(input_state_(this)),
       ALLOW_THIS_IN_INITIALIZER_LIST(output_state_(this)),
       pipe_(INVALID_HANDLE_VALUE),
       listener_(listener),
-      waiting_connect_(mode == MODE_SERVER),
+      waiting_connect_(mode & MODE_SERVER_FLAG),
       processing_incoming_(false),
-      ALLOW_THIS_IN_INITIALIZER_LIST(factory_(this)) {
-  switch(mode) {
-    case MODE_NONE:
-      LOG(FATAL) << "Bad mode for " << channel_handle.name;
-      break;
-    case MODE_SERVER:
-    case MODE_CLIENT:
-      break;
-    case MODE_NAMED_SERVER:
-      mode = MODE_SERVER;
-      break;
-    case MODE_NAMED_CLIENT:
-      mode = MODE_CLIENT;
-      break;
-    // Intentionally no default case here so that the compiler
-    // will check that we handle all the cases in the enum.
-  }
-  if (!CreatePipe(channel_handle, mode)) {
-    // The pipe may have been closed already.
-    LOG(WARNING) << "Unable to create pipe named \"" << channel_handle.name <<
-                    "\" in " << (mode == 0 ? "server" : "client") << " mode.";
-  }
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {
+  CreatePipe(channel_handle, mode);
 }
 
 Channel::ChannelImpl::~ChannelImpl() {
@@ -183,41 +94,68 @@ bool Channel::ChannelImpl::Send(Message* message) {
   return true;
 }
 
+// static
+bool Channel::ChannelImpl::IsNamedServerInitialized(
+    const std::string& channel_id) {
+  if (WaitNamedPipe(PipeName(channel_id).c_str(), 1))
+    return true;
+  // If ERROR_SEM_TIMEOUT occurred, the pipe exists but is handling another
+  // connection.
+  return GetLastError() == ERROR_SEM_TIMEOUT;
+}
+
+// static
 const std::wstring Channel::ChannelImpl::PipeName(
-    const std::string& channel_id) const {
-  std::wostringstream ss;
-  // XXX(darin): get application name from somewhere else
-  ss << L"\\\\.\\pipe\\chrome." << ASCIIToWide(channel_id);
-  return ss.str();
+    const std::string& channel_id) {
+  std::string name("\\\\.\\pipe\\chrome.");
+  return ASCIIToWide(name.append(channel_id));
 }
 
 bool Channel::ChannelImpl::CreatePipe(const IPC::ChannelHandle &channel_handle,
                                       Mode mode) {
-  DCHECK(pipe_ == INVALID_HANDLE_VALUE);
-  const std::wstring pipe_name = PipeName(channel_handle.name);
-  if (mode == MODE_SERVER) {
-    SECURITY_ATTRIBUTES security_attributes = {0};
-    security_attributes.bInheritHandle = FALSE;
-    security_attributes.nLength = sizeof(SECURITY_ATTRIBUTES);
-    if (!GetLogonSessionOnlyDACL(
-        reinterpret_cast<SECURITY_DESCRIPTOR**>(
-            &security_attributes.lpSecurityDescriptor))) {
-      NOTREACHED();
+  DCHECK_EQ(INVALID_HANDLE_VALUE, pipe_);
+  string16 pipe_name;
+  // If we already have a valid pipe for channel just copy it.
+  if (channel_handle.pipe.handle) {
+    DCHECK(channel_handle.name.empty());
+    pipe_name = L"Not Available";  // Just used for LOG
+    // Check that the given pipe confirms to the specified mode.  We can
+    // only check for PIPE_TYPE_MESSAGE & PIPE_SERVER_END flags since the
+    // other flags (PIPE_TYPE_BYTE, and PIPE_CLIENT_END) are defined as 0.
+    DWORD flags = 0;
+    GetNamedPipeInfo(channel_handle.pipe.handle, &flags, NULL, NULL, NULL);
+    DCHECK(!(flags & PIPE_TYPE_MESSAGE));
+    if (((mode & MODE_SERVER_FLAG) && !(flags & PIPE_SERVER_END)) ||
+        ((mode & MODE_CLIENT_FLAG) && (flags & PIPE_SERVER_END))) {
+      LOG(WARNING) << "Inconsistent open mode. Mode :" << mode;
+      return false;
     }
-
+    if (!DuplicateHandle(GetCurrentProcess(),
+                         channel_handle.pipe.handle,
+                         GetCurrentProcess(),
+                         &pipe_,
+                         0,
+                         FALSE,
+                         DUPLICATE_SAME_ACCESS)) {
+      LOG(WARNING) << "DuplicateHandle failed. Error :" << GetLastError();
+      return false;
+    }
+  } else if (mode & MODE_SERVER_FLAG) {
+    DCHECK(!channel_handle.pipe.handle);
+    const DWORD open_mode = PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED |
+                            FILE_FLAG_FIRST_PIPE_INSTANCE;
+    pipe_name = PipeName(channel_handle.name);
     pipe_ = CreateNamedPipeW(pipe_name.c_str(),
-                             PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED |
-                                FILE_FLAG_FIRST_PIPE_INSTANCE,
+                             open_mode,
                              PIPE_TYPE_BYTE | PIPE_READMODE_BYTE,
-                             1,         // number of pipe instances
-                             // output buffer size (XXX tune)
+                             1,
                              Channel::kReadBufferSize,
-                             // input buffer size (XXX tune)
                              Channel::kReadBufferSize,
-                             5000,      // timeout in milliseconds (XXX tune)
-                             &security_attributes);
-    LocalFree(security_attributes.lpSecurityDescriptor);
-  } else {
+                             5000,
+                             NULL);
+  } else if (mode & MODE_CLIENT_FLAG) {
+    DCHECK(!channel_handle.pipe.handle);
+    pipe_name = PipeName(channel_handle.name);
     pipe_ = CreateFileW(pipe_name.c_str(),
                         GENERIC_READ | GENERIC_WRITE,
                         0,
@@ -226,10 +164,15 @@ bool Channel::ChannelImpl::CreatePipe(const IPC::ChannelHandle &channel_handle,
                         SECURITY_SQOS_PRESENT | SECURITY_IDENTIFICATION |
                             FILE_FLAG_OVERLAPPED,
                         NULL);
+  } else {
+    NOTREACHED();
   }
+
   if (pipe_ == INVALID_HANDLE_VALUE) {
     // If this process is being closed, the pipe may be gone already.
-    LOG(WARNING) << "failed to create pipe: " << GetLastError();
+    LOG(WARNING) << "Unable to create pipe \"" << pipe_name <<
+                    "\" in " << (mode == 0 ? "server" : "client")
+                    << " mode. Error :" << GetLastError();
     return false;
   }
 
@@ -266,8 +209,10 @@ bool Channel::ChannelImpl::Connect() {
     // Complete setup asynchronously. By not setting input_state_.is_pending
     // to true, we indicate to OnIOCompleted that this is the special
     // initialization signal.
-    MessageLoopForIO::current()->PostTask(FROM_HERE, factory_.NewRunnableMethod(
-        &Channel::ChannelImpl::OnIOCompleted, &input_state_.context, 0, 0));
+    MessageLoopForIO::current()->PostTask(
+        FROM_HERE, base::Bind(&Channel::ChannelImpl::OnIOCompleted,
+                              weak_factory_.GetWeakPtr(), &input_state_.context,
+                              0, 0));
   }
 
   if (!waiting_connect_)
@@ -427,9 +372,10 @@ bool Channel::ChannelImpl::ProcessOutgoingMessages(
 
   // Write to pipe...
   Message* m = output_queue_.front();
+  DCHECK(m->size() <= INT_MAX);
   BOOL ok = WriteFile(pipe_,
                       m->data(),
-                      m->size(),
+                      static_cast<int>(m->size()),
                       &bytes_written,
                       &output_state_.context.overlapped);
   if (!ok) {
@@ -510,4 +456,9 @@ bool Channel::Send(Message* message) {
   return channel_impl_->Send(message);
 }
 
+// static
+bool Channel::IsNamedServerInitialized(const std::string& channel_id) {
+  return ChannelImpl::IsNamedServerInitialized(channel_id);
+}
+
 }  // namespace IPC
