@@ -1,12 +1,13 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "ipc/ipc_sync_channel.h"
 
+#include "base/bind.h"
 #include "base/lazy_instance.h"
+#include "base/location.h"
 #include "base/logging.h"
-#include "base/message_loop.h"
 #include "base/threading/thread_local.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/synchronization/waitable_event_watcher.h"
@@ -63,12 +64,14 @@ class SyncChannel::ReceivedSyncMsgQueue :
       // We set the event in case the listener thread is blocked (or is about
       // to). In case it's not, the PostTask dispatches the messages.
       message_queue_.push_back(QueuedMessage(new Message(msg), context));
+      message_queue_version_++;
     }
 
     dispatch_event_.Signal();
     if (!was_task_pending) {
-      listener_message_loop_->PostTask(FROM_HERE, NewRunnableMethod(
-          this, &ReceivedSyncMsgQueue::DispatchMessagesTask));
+      listener_message_loop_->PostTask(
+          FROM_HERE, base::Bind(&ReceivedSyncMsgQueue::DispatchMessagesTask,
+                                this, scoped_refptr<SyncContext>(context)));
     }
   }
 
@@ -78,28 +81,42 @@ class SyncChannel::ReceivedSyncMsgQueue :
 
   // Called on the listener's thread to process any queues synchronous
   // messages.
-  void DispatchMessagesTask() {
+  void DispatchMessagesTask(SyncContext* context) {
     {
       base::AutoLock auto_lock(message_lock_);
       task_pending_ = false;
     }
-    DispatchMessages();
+    context->DispatchMessages();
   }
 
-  void DispatchMessages() {
+  void DispatchMessages(SyncContext* dispatching_context) {
+    bool first_time = true;
+    uint32 expected_version = 0;
+    SyncMessageQueue::iterator it;
     while (true) {
-      Message* message;
+      Message* message = NULL;
       scoped_refptr<SyncChannel::SyncContext> context;
       {
         base::AutoLock auto_lock(message_lock_);
-        if (message_queue_.empty())
-          break;
-
-        message = message_queue_.front().message;
-        context = message_queue_.front().context;
-        message_queue_.pop_front();
+        if (first_time || message_queue_version_ != expected_version) {
+          it = message_queue_.begin();
+          first_time = false;
+        }
+        for (; it != message_queue_.end(); it++) {
+          if (!it->context->restrict_dispatch() ||
+              it->context == dispatching_context) {
+            message = it->message;
+            context = it->context;
+            it = message_queue_.erase(it);
+            message_queue_version_++;
+            expected_version = message_queue_version_;
+            break;
+          }
+        }
       }
 
+      if (message == NULL)
+        break;
       context->OnDispatchMessage(*message);
       delete message;
     }
@@ -114,6 +131,7 @@ class SyncChannel::ReceivedSyncMsgQueue :
       if (iter->context == context) {
         delete iter->message;
         iter = message_queue_.erase(iter);
+        message_queue_version_++;
       } else {
         iter++;
       }
@@ -126,7 +144,9 @@ class SyncChannel::ReceivedSyncMsgQueue :
   }
 
   WaitableEvent* dispatch_event() { return &dispatch_event_; }
-  MessageLoop* listener_message_loop() { return listener_message_loop_; }
+  base::MessageLoopProxy* listener_message_loop() {
+    return listener_message_loop_;
+  }
 
   // Holds a pointer to the per-thread ReceivedSyncMsgQueue object.
   static base::LazyInstance<base::ThreadLocalPointer<ReceivedSyncMsgQueue> >
@@ -159,8 +179,9 @@ class SyncChannel::ReceivedSyncMsgQueue :
   // See the comment in SyncChannel::SyncChannel for why this event is created
   // as manual reset.
   ReceivedSyncMsgQueue() :
+      message_queue_version_(0),
       dispatch_event_(true, false),
-      listener_message_loop_(MessageLoop::current()),
+      listener_message_loop_(base::MessageLoopProxy::current()),
       task_pending_(false),
       listener_count_(0),
       top_send_done_watcher_(NULL) {
@@ -175,8 +196,9 @@ class SyncChannel::ReceivedSyncMsgQueue :
     scoped_refptr<SyncChannel::SyncContext> context;
   };
 
-  typedef std::deque<QueuedMessage> SyncMessageQueue;
+  typedef std::list<QueuedMessage> SyncMessageQueue;
   SyncMessageQueue message_queue_;
+  uint32 message_queue_version_;  // Used to signal DispatchMessages to rescan
 
   std::vector<QueuedMessage> received_replies_;
 
@@ -184,7 +206,7 @@ class SyncChannel::ReceivedSyncMsgQueue :
   // sender needs its reply before it can reply to our original synchronous
   // message.
   WaitableEvent dispatch_event_;
-  MessageLoop* listener_message_loop_;
+  scoped_refptr<base::MessageLoopProxy> listener_message_loop_;
   base::Lock message_lock_;
   bool task_pending_;
   int listener_count_;
@@ -196,15 +218,17 @@ class SyncChannel::ReceivedSyncMsgQueue :
 };
 
 base::LazyInstance<base::ThreadLocalPointer<SyncChannel::ReceivedSyncMsgQueue> >
-    SyncChannel::ReceivedSyncMsgQueue::lazy_tls_ptr_(base::LINKER_INITIALIZED);
+    SyncChannel::ReceivedSyncMsgQueue::lazy_tls_ptr_ =
+        LAZY_INSTANCE_INITIALIZER;
 
 SyncChannel::SyncContext::SyncContext(
     Channel::Listener* listener,
-    MessageLoop* ipc_thread,
+    base::MessageLoopProxy* ipc_thread,
     WaitableEvent* shutdown_event)
     : ChannelProxy::Context(listener, ipc_thread),
       received_sync_msgs_(ReceivedSyncMsgQueue::AddContext()),
-      shutdown_event_(shutdown_event) {
+      shutdown_event_(shutdown_event),
+      restrict_dispatch_(false) {
 }
 
 SyncChannel::SyncContext::~SyncContext() {
@@ -216,6 +240,10 @@ SyncChannel::SyncContext::~SyncContext() {
 // we know how to deserialize the reply.  Returns a handle that's set when
 // the reply has arrived.
 void SyncChannel::SyncContext::Push(SyncMessage* sync_msg) {
+  // Create the tracking information for this message. This object is stored
+  // by value since all members are pointers that are cheap to copy. These
+  // pointers are cleaned up in the Pop() function.
+  //
   // The event is created as manual reset because in between Signal and
   // OnObjectSignalled, another Send can happen which would stop the watcher
   // from being called.  The event would get watched later, when the nested
@@ -244,8 +272,9 @@ bool SyncChannel::SyncContext::Pop() {
   // blocking Send() call, whose reply we received after we made this last
   // Send() call.  So check if we have any queued replies available that
   // can now unblock the listener thread.
-  ipc_message_loop()->PostTask(FROM_HERE, NewRunnableMethod(
-      received_sync_msgs_.get(), &ReceivedSyncMsgQueue::DispatchReplies));
+  ipc_message_loop()->PostTask(
+      FROM_HERE, base::Bind(&ReceivedSyncMsgQueue::DispatchReplies,
+                            received_sync_msgs_.get()));
 
   return result;
 }
@@ -260,7 +289,7 @@ WaitableEvent* SyncChannel::SyncContext::GetDispatchEvent() {
 }
 
 void SyncChannel::SyncContext::DispatchMessages() {
-  received_sync_msgs_->DispatchMessages();
+  received_sync_msgs_->DispatchMessages(this);
 }
 
 bool SyncChannel::SyncContext::TryToUnblockListener(const Message* msg) {
@@ -348,7 +377,7 @@ void SyncChannel::SyncContext::OnWaitableEventSignaled(WaitableEvent* event) {
     CancelPendingSends();
   } else {
     // We got the reply, timed out or the process shutdown.
-    DCHECK(event == GetSendDoneEvent());
+    DCHECK_EQ(GetSendDoneEvent(), event);
     MessageLoop::current()->QuitNow();
   }
 }
@@ -358,26 +387,31 @@ SyncChannel::SyncChannel(
     const IPC::ChannelHandle& channel_handle,
     Channel::Mode mode,
     Channel::Listener* listener,
-    MessageLoop* ipc_message_loop,
+    base::MessageLoopProxy* ipc_message_loop,
     bool create_pipe_now,
     WaitableEvent* shutdown_event)
-    : ChannelProxy(
-          channel_handle, mode, ipc_message_loop,
-          new SyncContext(listener, ipc_message_loop, shutdown_event),
-          create_pipe_now),
+    : ChannelProxy(new SyncContext(listener, ipc_message_loop, shutdown_event)),
       sync_messages_with_no_timeout_allowed_(true) {
-  // Ideally we only want to watch this object when running a nested message
-  // loop.  However, we don't know when it exits if there's another nested
-  // message loop running under it or not, so we wouldn't know whether to
-  // stop or keep watching.  So we always watch it, and create the event as
-  // manual reset since the object watcher might otherwise reset the event
-  // when we're doing a WaitMany.
-  dispatch_watcher_.StartWatching(sync_context()->GetDispatchEvent(), this);
+  ChannelProxy::Init(channel_handle, mode, create_pipe_now);
+  StartWatching();
+}
+
+SyncChannel::SyncChannel(
+    Channel::Listener* listener,
+    base::MessageLoopProxy* ipc_message_loop,
+    WaitableEvent* shutdown_event)
+    : ChannelProxy(new SyncContext(listener, ipc_message_loop, shutdown_event)),
+      sync_messages_with_no_timeout_allowed_(true) {
+  StartWatching();
 }
 
 SyncChannel::~SyncChannel() {
 }
 
+void SyncChannel::SetRestrictDispatchToSameChannel(bool value) {
+  sync_context()->set_restrict_dispatch(value);
+}
+
 bool SyncChannel::Send(Message* message) {
   return SendWithTimeout(message, base::kNoTimeout);
 }
@@ -408,9 +442,10 @@ bool SyncChannel::SendWithTimeout(Message* message, int timeout_ms) {
     // We use the sync message id so that when a message times out, we don't
     // confuse it with another send that is either above/below this Send in
     // the call stack.
-    context->ipc_message_loop()->PostDelayedTask(FROM_HERE,
-        NewRunnableMethod(context.get(),
-            &SyncContext::OnSendTimeout, message_id), timeout_ms);
+    context->ipc_message_loop()->PostDelayedTask(
+        FROM_HERE,
+        base::Bind(&SyncContext::OnSendTimeout, context.get(), message_id),
+        timeout_ms);
   }
 
   // Wait for reply, or for any other incoming synchronous messages.
@@ -422,6 +457,7 @@ bool SyncChannel::SendWithTimeout(Message* message, int timeout_ms) {
 
 void SyncChannel::WaitForReply(
     SyncContext* context, WaitableEvent* pump_messages_event) {
+  context->DispatchMessages();
   while (true) {
     WaitableEvent* objects[] = {
       context->GetDispatchEvent(),
@@ -491,4 +527,14 @@ void SyncChannel::OnWaitableEventSignaled(WaitableEvent* event) {
   sync_context()->DispatchMessages();
 }
 
+void SyncChannel::StartWatching() {
+  // Ideally we only want to watch this object when running a nested message
+  // loop.  However, we don't know when it exits if there's another nested
+  // message loop running under it or not, so we wouldn't know whether to
+  // stop or keep watching.  So we always watch it, and create the event as
+  // manual reset since the object watcher might otherwise reset the event
+  // when we're doing a WaitMany.
+  dispatch_watcher_.StartWatching(sync_context()->GetDispatchEvent(), this);
+}
+
 }  // namespace IPC
