@@ -1,16 +1,16 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#ifndef IPC_IPC_SYNC_SENDER_H__
-#define IPC_IPC_SYNC_SENDER_H__
+#ifndef IPC_IPC_SYNC_CHANNEL_H_
+#define IPC_IPC_SYNC_CHANNEL_H_
 #pragma once
 
 #include <string>
 #include <deque>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event_watcher.h"
 #include "ipc/ipc_channel_handle.h"
@@ -24,26 +24,63 @@ class WaitableEvent;
 namespace IPC {
 
 class SyncMessage;
-class MessageReplyDeserializer;
 
 // This is similar to ChannelProxy, with the added feature of supporting sending
 // synchronous messages.
+//
+// Overview of how the sync channel works
+// --------------------------------------
+// When the sending thread sends a synchronous message, we create a bunch
+// of tracking info (created in SendWithTimeout, stored in the PendingSyncMsg
+// structure) associated with the message that we identify by the unique
+// "MessageId" on the SyncMessage. Among the things we save is the
+// "Deserializer" which is provided by the sync message. This object is in
+// charge of reading the parameters from the reply message and putting them in
+// the output variables provided by its caller.
+//
+// The info gets stashed in a queue since we could have a nested stack of sync
+// messages (each side could send sync messages in response to sync messages,
+// so it works like calling a function). The message is sent to the I/O thread
+// for dispatch and the original thread blocks waiting for the reply.
+//
+// SyncContext maintains the queue in a threadsafe way and listens for replies
+// on the I/O thread. When a reply comes in that matches one of the messages
+// it's looking for (using the unique message ID), it will execute the
+// deserializer stashed from before, and unblock the original thread.
+//
+//
+// Significant complexity results from the fact that messages are still coming
+// in while the original thread is blocked. Normal async messages are queued
+// and dispatched after the blocking call is complete. Sync messages must
+// be dispatched in a reentrant manner to avoid deadlock.
+//
+//
 // Note that care must be taken that the lifetime of the ipc_thread argument
 // is more than this object.  If the message loop goes away while this object
 // is running and it's used to send a message, then it will use the invalid
 // message loop pointer to proxy it to the ipc thread.
-class SyncChannel : public ChannelProxy,
-                    public base::WaitableEventWatcher::Delegate {
+class IPC_EXPORT SyncChannel : public ChannelProxy,
+                               public base::WaitableEventWatcher::Delegate {
  public:
+  // Creates and initializes a sync channel. If create_pipe_now is specified,
+  // the channel will be initialized synchronously.
   SyncChannel(const IPC::ChannelHandle& channel_handle,
               Channel::Mode mode,
               Channel::Listener* listener,
-              MessageLoop* ipc_message_loop,
+              base::MessageLoopProxy* ipc_message_loop,
               bool create_pipe_now,
               base::WaitableEvent* shutdown_event);
+
+  // Creates an uninitialized sync channel. Call ChannelProxy::Init to
+  // initialize the channel. This two-step setup allows message filters to be
+  // added before any messages are sent or received.
+  SyncChannel(Channel::Listener* listener,
+              base::MessageLoopProxy* ipc_message_loop,
+              base::WaitableEvent* shutdown_event);
+
   virtual ~SyncChannel();
 
-  virtual bool Send(Message* message);
+  virtual bool Send(Message* message) OVERRIDE;
   virtual bool SendWithTimeout(Message* message, int timeout_ms);
 
   // Whether we allow sending messages with no time-out.
@@ -51,6 +88,17 @@ class SyncChannel : public ChannelProxy,
     sync_messages_with_no_timeout_allowed_ = value;
   }
 
+  // Sets this channel to only dispatch its incoming unblocking messages when it
+  // is itself blocked on sending a sync message, not when other channels are.
+  //
+  // Normally, any unblocking message coming from any channel can be dispatched
+  // when any (possibly other) channel is blocked on sending a message. This is
+  // needed in some cases to unblock certain loops (e.g. necessary when some
+  // processes share a window hierarchy), but may cause re-entrancy issues in
+  // some cases where such loops are not possible. This flags allows the tagging
+  // of some particular channels to not re-enter in such cases.
+  void SetRestrictDispatchToSameChannel(bool value);
+
  protected:
   class ReceivedSyncMsgQueue;
   friend class ReceivedSyncMsgQueue;
@@ -62,7 +110,7 @@ class SyncChannel : public ChannelProxy,
                       public base::WaitableEventWatcher::Delegate {
    public:
     SyncContext(Channel::Listener* listener,
-                MessageLoop* ipc_thread,
+                base::MessageLoopProxy* ipc_thread,
                 base::WaitableEvent* shutdown_event);
 
     // Adds information about an outgoing sync message to the context so that
@@ -98,24 +146,27 @@ class SyncChannel : public ChannelProxy,
       return received_sync_msgs_;
     }
 
+    void set_restrict_dispatch(bool value) { restrict_dispatch_ = value; }
+    bool restrict_dispatch() const { return restrict_dispatch_; }
+
    private:
-    ~SyncContext();
+    virtual ~SyncContext();
     // ChannelProxy methods that we override.
 
     // Called on the listener thread.
-    virtual void Clear();
+    virtual void Clear() OVERRIDE;
 
     // Called on the IPC thread.
-    virtual bool OnMessageReceived(const Message& msg);
-    virtual void OnChannelError();
-    virtual void OnChannelOpened();
-    virtual void OnChannelClosed();
+    virtual bool OnMessageReceived(const Message& msg) OVERRIDE;
+    virtual void OnChannelError() OVERRIDE;
+    virtual void OnChannelOpened() OVERRIDE;
+    virtual void OnChannelClosed() OVERRIDE;
 
     // Cancels all pending Send calls.
     void CancelPendingSends();
 
     // WaitableEventWatcher::Delegate implementation.
-    virtual void OnWaitableEventSignaled(base::WaitableEvent* arg);
+    virtual void OnWaitableEventSignaled(base::WaitableEvent* arg) OVERRIDE;
 
     typedef std::deque<PendingSyncMsg> PendingSyncMessageQueue;
     PendingSyncMessageQueue deserializers_;
@@ -125,11 +176,12 @@ class SyncChannel : public ChannelProxy,
 
     base::WaitableEvent* shutdown_event_;
     base::WaitableEventWatcher shutdown_watcher_;
+    bool restrict_dispatch_;
   };
 
  private:
   // WaitableEventWatcher::Delegate implementation.
-  virtual void OnWaitableEventSignaled(base::WaitableEvent* arg);
+  virtual void OnWaitableEventSignaled(base::WaitableEvent* arg) OVERRIDE;
 
   SyncContext* sync_context() {
     return reinterpret_cast<SyncContext*>(context());
@@ -144,6 +196,9 @@ class SyncChannel : public ChannelProxy,
   // shuts down.
   static void WaitForReplyWithNestedMessageLoop(SyncContext* context);
 
+  // Starts the dispatch watcher.
+  void StartWatching();
+
   bool sync_messages_with_no_timeout_allowed_;
 
   // Used to signal events between the IPC and listener threads.
@@ -154,4 +209,4 @@ class SyncChannel : public ChannelProxy,
 
 }  // namespace IPC
 
-#endif  // IPC_IPC_SYNC_SENDER_H__
+#endif  // IPC_IPC_SYNC_CHANNEL_H_
