@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -36,8 +36,8 @@ class IPCChannelTest : public base::MultiProcessTest {
  protected:
 
   // Create a new MessageLoopForIO For each test.
-  virtual void SetUp();
-  virtual void TearDown();
+  virtual void SetUp() OVERRIDE;
+  virtual void TearDown() OVERRIDE;
 
   // Spawns a child process of the specified type
   base::ProcessHandle SpawnChild(ChildType child_type, IPC::Channel* channel);
