@@ -1,23 +1,19 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "jingle/notifier/base/chrome_async_socket.h"
 
-#if defined(OS_WIN)
-#include <winsock2.h>
-#elif defined(OS_POSIX)
-#include <arpa/inet.h>
-#endif
-
 #include <algorithm>
 #include <cstring>
 #include <cstdlib>
 
 #include "base/basictypes.h"
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
+#include "jingle/notifier/base/resolving_client_socket_factory.h"
 #include "net/base/address_list.h"
 #include "net/base/host_port_pair.h"
 #include "net/base/io_buffer.h"
@@ -25,6 +21,7 @@
 #include "net/base/ssl_config_service.h"
 #include "net/base/sys_addrinfo.h"
 #include "net/socket/client_socket_factory.h"
+#include "net/socket/client_socket_handle.h"
 #include "net/socket/ssl_client_socket.h"
 #include "net/socket/tcp_client_socket.h"
 #include "talk/base/socketaddress.h"
@@ -32,30 +29,14 @@
 namespace notifier {
 
 ChromeAsyncSocket::ChromeAsyncSocket(
-    net::ClientSocketFactory* client_socket_factory,
-    const net::SSLConfig& ssl_config,
-    net::CertVerifier* cert_verifier,
+    ResolvingClientSocketFactory* client_socket_factory,
     size_t read_buf_size,
-    size_t write_buf_size,
-    net::NetLog* net_log)
-    : connect_callback_(ALLOW_THIS_IN_INITIALIZER_LIST(this),
-                        &ChromeAsyncSocket::ProcessConnectDone),
-      read_callback_(ALLOW_THIS_IN_INITIALIZER_LIST(this),
-                     &ChromeAsyncSocket::ProcessReadDone),
-      write_callback_(ALLOW_THIS_IN_INITIALIZER_LIST(this),
-                     &ChromeAsyncSocket::ProcessWriteDone),
-      ssl_connect_callback_(ALLOW_THIS_IN_INITIALIZER_LIST(this),
-                            &ChromeAsyncSocket::ProcessSSLConnectDone),
-      client_socket_factory_(client_socket_factory),
-      ssl_config_(ssl_config),
-      cert_verifier_(cert_verifier),
-      bound_net_log_(
-          net::BoundNetLog::Make(net_log, net::NetLog::SOURCE_SOCKET)),
+    size_t write_buf_size)
+    : client_socket_factory_(client_socket_factory),
       state_(STATE_CLOSED),
       error_(ERROR_NONE),
       net_error_(net::OK),
-      scoped_runnable_method_factory_(
-          ALLOW_THIS_IN_INITIALIZER_LIST(this)),
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)),
       read_state_(IDLE),
       read_buf_(new net::IOBufferWithSize(read_buf_size)),
       read_start_(0U),
@@ -103,26 +84,6 @@ void ChromeAsyncSocket::DoNetErrorFromStatus(int status) {
   DoNetError(static_cast<net::Error>(status));
 }
 
-namespace {
-
-// Takes a 32-bit integer in host byte order and converts it to a
-// net::IPAddressNumber.
-net::IPAddressNumber Uint32ToIPAddressNumber(uint32 ip) {
-  uint32 ip_nbo = htonl(ip);
-  const unsigned char* const ip_start =
-      reinterpret_cast<const unsigned char*>(&ip_nbo);
-  return net::IPAddressNumber(ip_start, ip_start + (sizeof ip_nbo));
-}
-
-net::AddressList SocketAddressToAddressList(
-    const talk_base::SocketAddress& address) {
-  DCHECK_NE(address.ip(), 0U);
-  return net::AddressList(Uint32ToIPAddressNumber(address.ip()),
-                          address.port(), false);
-}
-
-}  // namespace
-
 // STATE_CLOSED -> STATE_CONNECTING
 
 bool ChromeAsyncSocket::Connect(const talk_base::SocketAddress& address) {
@@ -131,7 +92,8 @@ bool ChromeAsyncSocket::Connect(const talk_base::SocketAddress& address) {
     DoNonNetError(ERROR_WRONGSTATE);
     return false;
   }
-  if (address.ip() == 0) {
+  // We can't work with an empty hostname and IP address.
+  if (address.hostname().empty() && (address.ip() == 0)) {
     DoNonNetError(ERROR_DNS);
     return false;
   }
@@ -142,15 +104,16 @@ bool ChromeAsyncSocket::Connect(const talk_base::SocketAddress& address) {
 
   state_ = STATE_CONNECTING;
 
-  DCHECK(scoped_runnable_method_factory_.empty());
-  scoped_runnable_method_factory_.RevokeAll();
+  DCHECK_EQ(false, weak_factory_.HasWeakPtrs());
+
+  net::HostPortPair dest_host_port_pair(address.IPAsString(), address.port());
 
-  net::AddressList address_list = SocketAddressToAddressList(address);
   transport_socket_.reset(
-      client_socket_factory_->CreateTCPClientSocket(address_list,
-                                                    bound_net_log_.net_log(),
-                                                    net::NetLog::Source()));
-  int status = transport_socket_->Connect(&connect_callback_);
+      client_socket_factory_->CreateTransportClientSocket(
+          dest_host_port_pair));
+  int status = transport_socket_->Connect(
+      base::Bind(&ChromeAsyncSocket::ProcessConnectDone,
+                 base::Unretained(this)));
   if (status != net::ERR_IO_PENDING) {
     // We defer execution of ProcessConnectDone instead of calling it
     // directly here as the caller may not expect an error/close to
@@ -158,10 +121,9 @@ bool ChromeAsyncSocket::Connect(const talk_base::SocketAddress& address) {
     // the connect always happens asynchronously.
     MessageLoop* message_loop = MessageLoop::current();
     CHECK(message_loop);
-    message_loop->PostTask(
-        FROM_HERE,
-        scoped_runnable_method_factory_.NewRunnableMethod(
-            &ChromeAsyncSocket::ProcessConnectDone, status));
+    message_loop->PostTask(FROM_HERE,
+                           base::Bind(&ChromeAsyncSocket::ProcessConnectDone,
+                                      weak_factory_.GetWeakPtr(), status));
   }
   return true;
 }
@@ -197,8 +159,7 @@ void ChromeAsyncSocket::PostDoRead() {
   CHECK(message_loop);
   message_loop->PostTask(
       FROM_HERE,
-      scoped_runnable_method_factory_.NewRunnableMethod(
-          &ChromeAsyncSocket::DoRead));
+      base::Bind(&ChromeAsyncSocket::DoRead, weak_factory_.GetWeakPtr()));
   read_state_ = POSTED;
 }
 
@@ -215,7 +176,9 @@ void ChromeAsyncSocket::DoRead() {
   // done).
   int status =
       transport_socket_->Read(
-          read_buf_.get(), read_buf_->size(), &read_callback_);
+          read_buf_.get(), read_buf_->size(),
+          base::Bind(&ChromeAsyncSocket::ProcessReadDone,
+                     base::Unretained(this)));
   read_state_ = PENDING;
   if (status != net::ERR_IO_PENDING) {
     ProcessReadDone(status);
@@ -329,8 +292,7 @@ void ChromeAsyncSocket::PostDoWrite() {
   CHECK(message_loop);
   message_loop->PostTask(
       FROM_HERE,
-      scoped_runnable_method_factory_.NewRunnableMethod(
-          &ChromeAsyncSocket::DoWrite));
+      base::Bind(&ChromeAsyncSocket::DoWrite, weak_factory_.GetWeakPtr()));
   write_state_ = POSTED;
 }
 
@@ -346,7 +308,9 @@ void ChromeAsyncSocket::DoWrite() {
   // before we send the next message.
   int status =
       transport_socket_->Write(
-          write_buf_.get(), write_end_, &write_callback_);
+          write_buf_.get(), write_end_,
+          base::Bind(&ChromeAsyncSocket::ProcessWriteDone,
+                     base::Unretained(this)));
   write_state_ = PENDING;
   if (status != net::ERR_IO_PENDING) {
     ProcessWriteDone(status);
@@ -397,7 +361,7 @@ bool ChromeAsyncSocket::Close() {
 // (not STATE_CLOSED) -> STATE_CLOSED
 
 void ChromeAsyncSocket::DoClose() {
-  scoped_runnable_method_factory_.RevokeAll();
+  weak_factory_.InvalidateWeakPtrs();
   if (transport_socket_.get()) {
     transport_socket_->Disconnect();
   }
@@ -434,22 +398,23 @@ bool ChromeAsyncSocket::StartTls(const std::string& domain_name) {
   DCHECK_EQ(write_end_, 0U);
 
   // Clear out any posted DoRead() tasks.
-  scoped_runnable_method_factory_.RevokeAll();
+  weak_factory_.InvalidateWeakPtrs();
 
   DCHECK(transport_socket_.get());
+  net::ClientSocketHandle* socket_handle = new net::ClientSocketHandle();
+  socket_handle->set_socket(transport_socket_.release());
   transport_socket_.reset(
       client_socket_factory_->CreateSSLClientSocket(
-          transport_socket_.release(), net::HostPortPair(domain_name, 443),
-          ssl_config_, NULL /* ssl_host_info */,
-          cert_verifier_));
-  int status = transport_socket_->Connect(&ssl_connect_callback_);
+          socket_handle, net::HostPortPair(domain_name, 443)));
+  int status = transport_socket_->Connect(
+      base::Bind(&ChromeAsyncSocket::ProcessSSLConnectDone,
+                 base::Unretained(this)));
   if (status != net::ERR_IO_PENDING) {
     MessageLoop* message_loop = MessageLoop::current();
     CHECK(message_loop);
-    message_loop->PostTask(
-        FROM_HERE,
-        scoped_runnable_method_factory_.NewRunnableMethod(
-            &ChromeAsyncSocket::ProcessSSLConnectDone, status));
+    message_loop->PostTask(FROM_HERE,
+                           base::Bind(&ChromeAsyncSocket::ProcessSSLConnectDone,
+                                      weak_factory_.GetWeakPtr(), status));
   }
   return true;
 }
