@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -15,34 +15,29 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/task.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "net/base/completion_callback.h"
 #include "net/base/net_errors.h"
-#include "net/base/net_log.h"
-#include "net/base/ssl_config_service.h"
 #include "talk/xmpp/asyncsocket.h"
 
 namespace net {
-class CertVerifier;
-class ClientSocket;
-class ClientSocketFactory;
 class IOBufferWithSize;
+class StreamSocket;
 }  // namespace net
 
 namespace notifier {
 
+class ResolvingClientSocketFactory;
+
 class ChromeAsyncSocket : public buzz::AsyncSocket {
  public:
-  // Takes ownership of |client_socket_factory| but not |cert_verifier| nor
-  // |net_log|.  |cert_verifier| may not be NULL.  |net_log| may be NULL.
-  ChromeAsyncSocket(net::ClientSocketFactory* client_socket_factory,
-                    const net::SSLConfig& ssl_config,
-                    net::CertVerifier* cert_verifier,
+  // Takes ownership of |client_socket_factory|.
+  ChromeAsyncSocket(ResolvingClientSocketFactory* client_socket_factory,
                     size_t read_buf_size,
-                    size_t write_buf_size,
-                    net::NetLog* net_log);
+                    size_t write_buf_size);
 
   // Does not raise any signals.
   virtual ~ChromeAsyncSocket();
@@ -51,16 +46,16 @@ class ChromeAsyncSocket : public buzz::AsyncSocket {
 
   // The current state (see buzz::AsyncSocket::State; all but
   // STATE_CLOSING is used).
-  virtual State state();
+  virtual State state() OVERRIDE;
 
   // The last generated error.  Errors are generated when the main
   // functions below return false or when SignalClosed is raised due
   // to an asynchronous error.
-  virtual Error error();
+  virtual Error error() OVERRIDE;
 
   // GetError() (which is of type net::Error) != net::OK only when
   // error() == ERROR_WINSOCK.
-  virtual int GetError();
+  virtual int GetError() OVERRIDE;
 
   // Tries to connect to the given address.
   //
@@ -73,7 +68,7 @@ class ChromeAsyncSocket : public buzz::AsyncSocket {
   // Otherwise, starts the connection process and returns true.
   // SignalConnected will be raised when the connection is successful;
   // otherwise, SignalClosed will be raised with a net error set.
-  virtual bool Connect(const talk_base::SocketAddress& address);
+  virtual bool Connect(const talk_base::SocketAddress& address) OVERRIDE;
 
   // Tries to read at most |len| bytes into |data|.
   //
@@ -86,7 +81,7 @@ class ChromeAsyncSocket : public buzz::AsyncSocket {
   // case because StartTls() is called during a slot connected to
   // SignalRead after parsing the final non-TLS reply from the server
   // [see XmppClient::Private::OnSocketRead()].)
-  virtual bool Read(char* data, size_t len, size_t* len_read);
+  virtual bool Read(char* data, size_t len, size_t* len_read) OVERRIDE;
 
   // Queues up |len| bytes of |data| for writing.
   //
@@ -105,11 +100,11 @@ class ChromeAsyncSocket : public buzz::AsyncSocket {
   // Note that there's no guarantee that the data will actually be
   // sent; however, it is guaranteed that the any data sent will be
   // sent in FIFO order.
-  virtual bool Write(const char* data, size_t len);
+  virtual bool Write(const char* data, size_t len) OVERRIDE;
 
   // If the socket is not already closed, closes the socket and raises
   // SignalClosed.  Always returns true.
-  virtual bool Close();
+  virtual bool Close() OVERRIDE;
 
   // Tries to change to a TLS connection with the given domain name.
   //
@@ -122,7 +117,7 @@ class ChromeAsyncSocket : public buzz::AsyncSocket {
   // SignalSSLConnected will be raised when the connection is
   // successful; otherwise, SignalClosed will be raised with a net
   // error set.
-  virtual bool StartTls(const std::string& domain_name);
+  virtual bool StartTls(const std::string& domain_name) OVERRIDE;
 
   // Signal behavior:
   //
@@ -181,16 +176,7 @@ class ChromeAsyncSocket : public buzz::AsyncSocket {
   // Close functions.
   void DoClose();
 
-  // Callbacks passed to |transport_socket_|.
-  net::CompletionCallbackImpl<ChromeAsyncSocket> connect_callback_;
-  net::CompletionCallbackImpl<ChromeAsyncSocket> read_callback_;
-  net::CompletionCallbackImpl<ChromeAsyncSocket> write_callback_;
-  net::CompletionCallbackImpl<ChromeAsyncSocket> ssl_connect_callback_;
-
-  scoped_ptr<net::ClientSocketFactory> client_socket_factory_;
-  const net::SSLConfig ssl_config_;
-  net::CertVerifier* const cert_verifier_;
-  net::BoundNetLog bound_net_log_;
+  scoped_ptr<ResolvingClientSocketFactory> client_socket_factory_;
 
   // buzz::AsyncSocket state.
   buzz::AsyncSocket::State state_;
@@ -198,13 +184,12 @@ class ChromeAsyncSocket : public buzz::AsyncSocket {
   net::Error net_error_;
 
   // Used by read/write loops.
-  ScopedRunnableMethodFactory<ChromeAsyncSocket>
-      scoped_runnable_method_factory_;
+  base::WeakPtrFactory<ChromeAsyncSocket> weak_factory_;
 
   // NULL iff state() == STATE_CLOSED.
   //
   // TODO(akalin): Use ClientSocketPool.
-  scoped_ptr<net::ClientSocket> transport_socket_;
+  scoped_ptr<net::StreamSocket> transport_socket_;
 
   // State for the read loop.  |read_start_| <= |read_end_| <=
   // |read_buf_->size()|.  There's a read in flight (i.e.,
