@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -10,24 +10,25 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/weak_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "jingle/notifier/base/task_pump.h"
 
-namespace talk_base {
-class Task;
-}  // namespace talk_base
+namespace buzz {
+class XmppTaskParentInterface;
+}  // namespace buzz
 
 namespace notifier {
 
 class FakeBaseTask {
  public:
   FakeBaseTask();
+  ~FakeBaseTask();
 
-  base::WeakPtr<talk_base::Task> AsWeakPtr();
+  base::WeakPtr<buzz::XmppTaskParentInterface> AsWeakPtr();
 
  private:
   notifier::TaskPump task_pump_;
-  base::WeakPtr<talk_base::Task> base_task_;
+  base::WeakPtr<buzz::XmppTaskParentInterface> base_task_;
 
   DISALLOW_COPY_AND_ASSIGN(FakeBaseTask);
 };
