@@ -1,8 +1,8 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
-// This ClientSocket implementation is to be used with servers that
+// This StreamSocket implementation is to be used with servers that
 // accept connections on port 443 but don't really use SSL.  For
 // example, the Google Talk servers do this to bypass proxies.  (The
 // connection is upgraded to TLS as part of the XMPP negotiation, so
@@ -10,7 +10,7 @@
 // after connection to fool proxies into thinking that this is a real
 // SSL connection.
 //
-// NOTE: This ClientSocket implementation does *not* do a real SSL
+// NOTE: This StreamSocket implementation does *not* do a real SSL
 // handshake nor does it do any encryption!
 
 #ifndef JINGLE_NOTIFIER_BASE_FAKE_SSL_CLIENT_SOCKET_H_
@@ -20,12 +20,13 @@
 #include <cstddef>
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/string_piece.h"
-#include "base/ref_counted.h"
 #include "net/base/completion_callback.h"
 #include "net/base/net_errors.h"
-#include "net/socket/client_socket.h"
+#include "net/socket/stream_socket.h"
 
 namespace net {
 class DrainableIOBuffer;
@@ -33,10 +34,10 @@ class DrainableIOBuffer;
 
 namespace notifier {
 
-class FakeSSLClientSocket : public net::ClientSocket {
+class FakeSSLClientSocket : public net::StreamSocket {
  public:
   // Takes ownership of |transport_socket|.
-  explicit FakeSSLClientSocket(net::ClientSocket* transport_socket);
+  explicit FakeSSLClientSocket(net::StreamSocket* transport_socket);
 
   virtual ~FakeSSLClientSocket();
 
@@ -44,23 +45,26 @@ class FakeSSLClientSocket : public net::ClientSocket {
   static base::StringPiece GetSslClientHello();
   static base::StringPiece GetSslServerHello();
 
-  // net::ClientSocket implementation.
+  // net::StreamSocket implementation.
   virtual int Read(net::IOBuffer* buf, int buf_len,
-                   net::CompletionCallback* callback);
+                   const net::CompletionCallback& callback) OVERRIDE;
   virtual int Write(net::IOBuffer* buf, int buf_len,
-                    net::CompletionCallback* callback);
-  virtual bool SetReceiveBufferSize(int32 size);
-  virtual bool SetSendBufferSize(int32 size);
-  virtual int Connect(net::CompletionCallback* callback);
-  virtual void Disconnect();
-  virtual bool IsConnected() const;
-  virtual bool IsConnectedAndIdle() const;
-  virtual int GetPeerAddress(net::AddressList* address) const;
-  virtual const net::BoundNetLog& NetLog() const;
-  virtual void SetSubresourceSpeculation();
-  virtual void SetOmniboxSpeculation();
-  virtual bool WasEverUsed() const;
-  virtual bool UsingTCPFastOpen() const;
+                    const net::CompletionCallback& callback) OVERRIDE;
+  virtual bool SetReceiveBufferSize(int32 size) OVERRIDE;
+  virtual bool SetSendBufferSize(int32 size) OVERRIDE;
+  virtual int Connect(const net::CompletionCallback& callback) OVERRIDE;
+  virtual void Disconnect() OVERRIDE;
+  virtual bool IsConnected() const OVERRIDE;
+  virtual bool IsConnectedAndIdle() const OVERRIDE;
+  virtual int GetPeerAddress(net::AddressList* address) const OVERRIDE;
+  virtual int GetLocalAddress(net::IPEndPoint* address) const OVERRIDE;
+  virtual const net::BoundNetLog& NetLog() const OVERRIDE;
+  virtual void SetSubresourceSpeculation() OVERRIDE;
+  virtual void SetOmniboxSpeculation() OVERRIDE;
+  virtual bool WasEverUsed() const OVERRIDE;
+  virtual bool UsingTCPFastOpen() const OVERRIDE;
+  virtual int64 NumBytesRead() const OVERRIDE;
+  virtual base::TimeDelta GetConnectTimeMicros() const OVERRIDE;
 
  private:
   enum HandshakeState {
@@ -86,14 +90,7 @@ class FakeSSLClientSocket : public net::ClientSocket {
   void OnVerifyServerHelloDone(int status);
   net::Error ProcessVerifyServerHelloDone(size_t read);
 
-  // Callbacks passed to |transport_socket_|.
-  net::CompletionCallbackImpl<FakeSSLClientSocket> connect_callback_;
-  net::CompletionCallbackImpl<FakeSSLClientSocket>
-      send_client_hello_callback_;
-  net::CompletionCallbackImpl<FakeSSLClientSocket>
-      verify_server_hello_callback_;
-
-  scoped_ptr<net::ClientSocket> transport_socket_;
+  scoped_ptr<net::StreamSocket> transport_socket_;
 
   // During the handshake process, holds a value from HandshakeState.
   // STATE_NONE otherwise.
@@ -103,7 +100,7 @@ class FakeSSLClientSocket : public net::ClientSocket {
   bool handshake_completed_;
 
   // The callback passed to Connect().
-  net::CompletionCallback* user_connect_callback_;
+  net::CompletionCallback user_connect_callback_;
 
   scoped_refptr<net::DrainableIOBuffer> write_buf_;
   scoped_refptr<net::DrainableIOBuffer> read_buf_;
