@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,15 +8,15 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
 #include "net/base/capturing_net_log.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_log.h"
 #include "net/base/test_completion_callback.h"
-#include "net/socket/client_socket.h"
 #include "net/socket/socket_test_util.h"
+#include "net/socket/stream_socket.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -43,24 +43,29 @@ enum {
 };
 
 // Used by PassThroughMethods test.
-class MockClientSocket : public net::ClientSocket {
+class MockClientSocket : public net::StreamSocket {
  public:
   virtual ~MockClientSocket() {}
 
-  MOCK_METHOD3(Read, int(net::IOBuffer*, int, net::CompletionCallback*));
-  MOCK_METHOD3(Write, int(net::IOBuffer*, int, net::CompletionCallback*));
+  MOCK_METHOD3(Read, int(net::IOBuffer*, int,
+                         const net::CompletionCallback&));
+  MOCK_METHOD3(Write, int(net::IOBuffer*, int,
+                          const net::CompletionCallback&));
   MOCK_METHOD1(SetReceiveBufferSize, bool(int32));
   MOCK_METHOD1(SetSendBufferSize, bool(int32));
-  MOCK_METHOD1(Connect, int(net::CompletionCallback*));
+  MOCK_METHOD1(Connect, int(const net::CompletionCallback&));
   MOCK_METHOD0(Disconnect, void());
   MOCK_CONST_METHOD0(IsConnected, bool());
   MOCK_CONST_METHOD0(IsConnectedAndIdle, bool());
   MOCK_CONST_METHOD1(GetPeerAddress, int(net::AddressList*));
+  MOCK_CONST_METHOD1(GetLocalAddress, int(net::IPEndPoint*));
   MOCK_CONST_METHOD0(NetLog, const net::BoundNetLog&());
   MOCK_METHOD0(SetSubresourceSpeculation, void());
   MOCK_METHOD0(SetOmniboxSpeculation, void());
   MOCK_CONST_METHOD0(WasEverUsed, bool());
   MOCK_CONST_METHOD0(UsingTCPFastOpen, bool());
+  MOCK_CONST_METHOD0(NumBytesRead, int64());
+  MOCK_CONST_METHOD0(GetConnectTimeMicros, base::TimeDelta());
 };
 
 // Break up |data| into a bunch of chunked MockReads/Writes and push
@@ -86,8 +91,8 @@ class FakeSSLClientSocketTest : public testing::Test {
 
   virtual ~FakeSSLClientSocketTest() {}
 
-  net::ClientSocket* MakeClientSocket() {
-    return mock_client_socket_factory_.CreateTCPClientSocket(
+  net::StreamSocket* MakeClientSocket() {
+    return mock_client_socket_factory_.CreateTransportClientSocket(
         net::AddressList(), &capturing_net_log_, net::NetLog::Source());
   }
 
@@ -96,8 +101,8 @@ class FakeSSLClientSocketTest : public testing::Test {
                std::vector<net::MockWrite>* writes) {
     static_socket_data_provider_.reset(
         new net::StaticSocketDataProvider(
-            &*reads->begin(), reads->size(),
-            &*writes->begin(), writes->size()));
+            reads->empty() ? NULL : &*reads->begin(), reads->size(),
+            writes->empty() ? NULL : &*writes->begin(), writes->size()));
     static_socket_data_provider_->set_connect_data(mock_connect);
     mock_client_socket_factory_.AddSocketDataProvider(
         static_socket_data_provider_.get());
@@ -105,7 +110,7 @@ class FakeSSLClientSocketTest : public testing::Test {
 
   void ExpectStatus(
       bool async, int expected_status, int immediate_status,
-      TestCompletionCallback* test_completion_callback) {
+      net::TestCompletionCallback* test_completion_callback) {
     if (async) {
       EXPECT_EQ(net::ERR_IO_PENDING, immediate_status);
       int status = test_completion_callback->WaitForResult();
@@ -146,8 +151,9 @@ class FakeSSLClientSocketTest : public testing::Test {
 
     for (int i = 0; i < num_resets + 1; ++i) {
       SCOPED_TRACE(i);
-      TestCompletionCallback test_completion_callback;
-      int status = fake_ssl_client_socket.Connect(&test_completion_callback);
+      net::TestCompletionCallback test_completion_callback;
+      int status = fake_ssl_client_socket.Connect(
+          test_completion_callback.callback());
       if (async) {
         EXPECT_FALSE(fake_ssl_client_socket.IsConnected());
       }
@@ -158,13 +164,14 @@ class FakeSSLClientSocketTest : public testing::Test {
         scoped_refptr<net::IOBuffer> read_buf(
             new net::IOBuffer(read_buf_len));
         int read_status = fake_ssl_client_socket.Read(
-            read_buf, read_buf_len, &test_completion_callback);
+            read_buf, read_buf_len, test_completion_callback.callback());
         ExpectStatus(async, read_len, read_status, &test_completion_callback);
 
         scoped_refptr<net::IOBuffer> write_buf(
             new net::StringIOBuffer(kWriteTestData));
         int write_status = fake_ssl_client_socket.Write(
-            write_buf, arraysize(kWriteTestData), &test_completion_callback);
+            write_buf, arraysize(kWriteTestData),
+            test_completion_callback.callback());
         ExpectStatus(async, arraysize(kWriteTestData), write_status,
                      &test_completion_callback);
       } else {
@@ -239,8 +246,9 @@ class FakeSSLClientSocketTest : public testing::Test {
          (error == ERR_MALFORMED_SERVER_HELLO)) ?
         net::ERR_UNEXPECTED : error;
 
-    TestCompletionCallback test_completion_callback;
-    int status = fake_ssl_client_socket.Connect(&test_completion_callback);
+    net::TestCompletionCallback test_completion_callback;
+    int status = fake_ssl_client_socket.Connect(
+        test_completion_callback.callback());
     EXPECT_FALSE(fake_ssl_client_socket.IsConnected());
     ExpectStatus(async, expected_status, status, &test_completion_callback);
     EXPECT_FALSE(fake_ssl_client_socket.IsConnected());
