@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -11,8 +11,9 @@
 #pragma once
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
+#include "base/memory/weak_ptr.h"
 #include "base/threading/non_thread_safe.h"
-#include "base/weak_ptr.h"
 #include "talk/xmpp/xmppclient.h"
 
 namespace talk_base {
@@ -40,7 +41,7 @@ class WeakXmppClient : public buzz::XmppClient {
   void Invalidate();
 
  protected:
-  virtual void Stop();
+  virtual void Stop() OVERRIDE;
 
  private:
   base::NonThreadSafe non_thread_safe_;
