@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -9,25 +9,20 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/gtest_prod_util.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/threading/non_thread_safe.h"
-#include "base/weak_ptr.h"
+#include "net/url_request/url_request_context_getter.h"
 #include "talk/base/sigslot.h"
 #include "talk/xmpp/xmppengine.h"
-#include "testing/gtest/include/gtest/gtest_prod.h"
 
 namespace buzz {
 class PreXmppAuth;
 class XmlElement;
 class XmppClientSettings;
-}  // namespace
-
-namespace net {
-class CertVerifier;
-}  // namespace
-
-namespace talk_base {
-class Task;
+class XmppTaskParentInterface;
 }  // namespace
 
 namespace notifier {
@@ -44,7 +39,8 @@ class XmppConnection : public sigslot::has_slots<> {
     // Called (at most once) when a connection has been established.
     // |base_task| can be used by the client as the parent of any Task
     // it creates as long as it is valid (i.e., non-NULL).
-    virtual void OnConnect(base::WeakPtr<talk_base::Task> base_task) = 0;
+    virtual void OnConnect(
+        base::WeakPtr<buzz::XmppTaskParentInterface> base_task) = 0;
 
     // Called if an error has occurred (either before or after a call
     // to OnConnect()).  No calls to the delegate will be made after
@@ -69,13 +65,14 @@ class XmppConnection : public sigslot::has_slots<> {
   //
   // TODO(akalin): Avoid the need for |pre_xmpp_auth|.
   XmppConnection(const buzz::XmppClientSettings& xmpp_client_settings,
-                 net::CertVerifier* cert_verifier,
+                 const scoped_refptr<net::URLRequestContextGetter>&
+                     request_context_getter,
                  Delegate* delegate, buzz::PreXmppAuth* pre_xmpp_auth);
 
   // Invalidates any weak pointers passed to the delegate by
   // OnConnect(), but does not trigger a call to the delegate's
   // OnError() function.
-  ~XmppConnection();
+  virtual ~XmppConnection();
 
  private:
   void OnStateChange(buzz::XmppEngine::State state);
@@ -94,6 +91,7 @@ class XmppConnection : public sigslot::has_slots<> {
   FRIEND_TEST(XmppConnectionTest, Connect);
   FRIEND_TEST(XmppConnectionTest, MultipleConnect);
   FRIEND_TEST(XmppConnectionTest, ConnectThenError);
+  FRIEND_TEST(XmppConnectionTest, TasksDontRunAfterXmppConnectionDestructor);
 
   DISALLOW_COPY_AND_ASSIGN(XmppConnection);
 };
