@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,10 +8,14 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/memory/weak_ptr.h"
 #include "base/message_loop.h"
-#include "base/weak_ptr.h"
+#include "jingle/notifier/base/mock_task.h"
+#include "jingle/notifier/base/task_pump.h"
 #include "jingle/notifier/base/weak_xmpp_client.h"
 #include "net/base/cert_verifier.h"
+#include "net/url_request/url_request_context_getter.h"
+#include "net/url_request/url_request_test_util.h"
 #include "talk/xmpp/prexmppauth.h"
 #include "talk/xmpp/xmppclientsettings.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -28,6 +32,32 @@ class SocketAddress;
 class Task;
 }  // namespace talk_base
 
+namespace {
+// TODO(sanjeevr): Move this to net_test_support.
+// Used to return a dummy context.
+class TestURLRequestContextGetter : public net::URLRequestContextGetter {
+ public:
+  TestURLRequestContextGetter()
+      : message_loop_proxy_(base::MessageLoopProxy::current()) {
+  }
+  virtual ~TestURLRequestContextGetter() { }
+
+  // net::URLRequestContextGetter:
+  virtual net::URLRequestContext* GetURLRequestContext() {
+    if (!context_)
+      context_ = new TestURLRequestContext();
+    return context_.get();
+  }
+  virtual scoped_refptr<base::MessageLoopProxy> GetIOMessageLoopProxy() const {
+    return message_loop_proxy_;
+  }
+
+ private:
+  scoped_refptr<net::URLRequestContext> context_;
+  scoped_refptr<base::MessageLoopProxy> message_loop_proxy_;
+};
+}  // namespace
+
 namespace notifier {
 
 using ::testing::_;
@@ -59,14 +89,16 @@ class MockXmppConnectionDelegate : public XmppConnection::Delegate {
  public:
   virtual ~MockXmppConnectionDelegate() {}
 
-  MOCK_METHOD1(OnConnect, void(base::WeakPtr<talk_base::Task>));
+  MOCK_METHOD1(OnConnect, void(base::WeakPtr<buzz::XmppTaskParentInterface>));
   MOCK_METHOD3(OnError,
                void(buzz::XmppEngine::Error, int, const buzz::XmlElement*));
 };
 
 class XmppConnectionTest : public testing::Test {
  protected:
-  XmppConnectionTest() : mock_pre_xmpp_auth_(new MockPreXmppAuth()) {}
+  XmppConnectionTest()
+      : mock_pre_xmpp_auth_(new MockPreXmppAuth()),
+        url_request_context_getter_(new TestURLRequestContextGetter()) {}
 
   virtual ~XmppConnectionTest() {}
 
@@ -77,13 +109,14 @@ class XmppConnectionTest : public testing::Test {
 
   // Needed by XmppConnection.
   MessageLoop message_loop_;
-  net::CertVerifier cert_verifier_;
   MockXmppConnectionDelegate mock_xmpp_connection_delegate_;
   scoped_ptr<MockPreXmppAuth> mock_pre_xmpp_auth_;
+  scoped_refptr<TestURLRequestContextGetter> url_request_context_getter_;
 };
 
 TEST_F(XmppConnectionTest, CreateDestroy) {
-  XmppConnection xmpp_connection(buzz::XmppClientSettings(), &cert_verifier_,
+  XmppConnection xmpp_connection(buzz::XmppClientSettings(),
+                                 url_request_context_getter_,
                                  &mock_xmpp_connection_delegate_, NULL);
 }
 
@@ -94,7 +127,8 @@ TEST_F(XmppConnectionTest, ImmediateFailure) {
   EXPECT_CALL(mock_xmpp_connection_delegate_,
               OnError(buzz::XmppEngine::ERROR_NONE, 0, NULL));
 
-  XmppConnection xmpp_connection(buzz::XmppClientSettings(), &cert_verifier_,
+  XmppConnection xmpp_connection(buzz::XmppClientSettings(),
+                                 url_request_context_getter_,
                                  &mock_xmpp_connection_delegate_, NULL);
 
   // We need to do this *before* |xmpp_connection| gets destroyed or
@@ -113,7 +147,7 @@ TEST_F(XmppConnectionTest, PreAuthFailure) {
               OnError(buzz::XmppEngine::ERROR_AUTH, 5, NULL));
 
   XmppConnection xmpp_connection(
-      buzz::XmppClientSettings(), &cert_verifier_,
+      buzz::XmppClientSettings(), url_request_context_getter_,
       &mock_xmpp_connection_delegate_, mock_pre_xmpp_auth_.release());
 
   // We need to do this *before* |xmpp_connection| gets destroyed or
@@ -131,7 +165,7 @@ TEST_F(XmppConnectionTest, FailureAfterPreAuth) {
               OnError(buzz::XmppEngine::ERROR_NONE, 0, NULL));
 
   XmppConnection xmpp_connection(
-      buzz::XmppClientSettings(), &cert_verifier_,
+      buzz::XmppClientSettings(), url_request_context_getter_,
       &mock_xmpp_connection_delegate_, mock_pre_xmpp_auth_.release());
 
   // We need to do this *before* |xmpp_connection| gets destroyed or
@@ -143,7 +177,8 @@ TEST_F(XmppConnectionTest, RaisedError) {
   EXPECT_CALL(mock_xmpp_connection_delegate_,
               OnError(buzz::XmppEngine::ERROR_NONE, 0, NULL));
 
-  XmppConnection xmpp_connection(buzz::XmppClientSettings(), &cert_verifier_,
+  XmppConnection xmpp_connection(buzz::XmppClientSettings(),
+                                 url_request_context_getter_,
                                  &mock_xmpp_connection_delegate_, NULL);
 
   xmpp_connection.weak_xmpp_client_->
@@ -156,7 +191,8 @@ TEST_F(XmppConnectionTest, Connect) {
       WillOnce(SaveArg<0>(&weak_ptr));
 
   {
-    XmppConnection xmpp_connection(buzz::XmppClientSettings(), &cert_verifier_,
+    XmppConnection xmpp_connection(buzz::XmppClientSettings(),
+                                   url_request_context_getter_,
                                    &mock_xmpp_connection_delegate_, NULL);
 
     xmpp_connection.weak_xmpp_client_->
@@ -173,7 +209,8 @@ TEST_F(XmppConnectionTest, MultipleConnect) {
     EXPECT_CALL(mock_xmpp_connection_delegate_, OnConnect(_)).
         WillOnce(SaveArg<0>(&weak_ptr));
 
-    XmppConnection xmpp_connection(buzz::XmppClientSettings(), &cert_verifier_,
+    XmppConnection xmpp_connection(buzz::XmppClientSettings(),
+                                   url_request_context_getter_,
                                    &mock_xmpp_connection_delegate_, NULL);
 
     xmpp_connection.weak_xmpp_client_->
@@ -194,7 +231,8 @@ TEST_F(XmppConnectionTest, ConnectThenError) {
   EXPECT_CALL(mock_xmpp_connection_delegate_,
               OnError(buzz::XmppEngine::ERROR_NONE, 0, NULL));
 
-  XmppConnection xmpp_connection(buzz::XmppClientSettings(), &cert_verifier_,
+  XmppConnection xmpp_connection(buzz::XmppClientSettings(),
+                                 url_request_context_getter_,
                                  &mock_xmpp_connection_delegate_, NULL);
 
   xmpp_connection.weak_xmpp_client_->
@@ -206,4 +244,26 @@ TEST_F(XmppConnectionTest, ConnectThenError) {
   EXPECT_EQ(NULL, weak_ptr.get());
 }
 
+// We don't destroy XmppConnection's task pump on destruction, but it
+// should still not run any more tasks.
+TEST_F(XmppConnectionTest, TasksDontRunAfterXmppConnectionDestructor) {
+  {
+    XmppConnection xmpp_connection(buzz::XmppClientSettings(),
+                                   url_request_context_getter_,
+                                   &mock_xmpp_connection_delegate_, NULL);
+
+    MockTask* task = new MockTask(xmpp_connection.task_pump_.get());
+    // We have to do this since the state enum is protected in
+    // talk_base::Task.
+    const int TASK_STATE_ERROR = 3;
+    ON_CALL(*task, ProcessStart())
+        .WillByDefault(Return(TASK_STATE_ERROR));
+    EXPECT_CALL(*task, ProcessStart()).Times(0);
+    task->Start();
+  }
+
+  // This should destroy |task_pump|, but |task| still shouldn't run.
+  message_loop_.RunAllPending();
+}
+
 }  // namespace notifier
