@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -33,26 +33,25 @@ static const int kRedirectTimeoutMinutes = 5;
 Login::Login(Delegate* delegate,
              const buzz::XmppClientSettings& user_settings,
              const ConnectionOptions& options,
-             net::HostResolver* host_resolver,
-             net::CertVerifier* cert_verifier,
-             ServerInformation* server_list,
-             int server_count,
-             bool try_ssltcp_first)
+             const scoped_refptr<net::URLRequestContextGetter>&
+                request_context_getter,
+             const ServerList& servers,
+             bool try_ssltcp_first,
+             const std::string& auth_mechanism)
     : delegate_(delegate),
       login_settings_(new LoginSettings(user_settings,
                                         options,
-                                        host_resolver,
-                                        cert_verifier,
-                                        server_list,
-                                        server_count,
-                                        try_ssltcp_first)),
+                                        request_context_getter,
+                                        servers,
+                                        try_ssltcp_first,
+                                        auth_mechanism)),
       redirect_port_(0) {
-  net::NetworkChangeNotifier::AddObserver(this);
+  net::NetworkChangeNotifier::AddIPAddressObserver(this);
   ResetReconnectState();
 }
 
 Login::~Login() {
-  net::NetworkChangeNotifier::RemoveObserver(this);
+  net::NetworkChangeNotifier::RemoveIPAddressObserver(this);
 }
 
 void Login::StartConnection() {
@@ -73,7 +72,11 @@ void Login::StartConnection() {
   single_attempt_.reset(new SingleLoginAttempt(login_settings_.get(), this));
 }
 
-void Login::OnConnect(base::WeakPtr<talk_base::Task> base_task) {
+void Login::UpdateXmppSettings(const buzz::XmppClientSettings& user_settings) {
+  *(login_settings_->modifiable_user_settings()) = user_settings;
+}
+
+void Login::OnConnect(base::WeakPtr<buzz::XmppTaskParentInterface> base_task) {
   ResetReconnectState();
   delegate_->OnConnect(base_task);
 }
@@ -114,7 +117,7 @@ void Login::TryReconnect() {
   VLOG(1) << "Reconnecting in "
           << reconnect_interval_.InSeconds() << " seconds";
   reconnect_timer_.Start(
-      reconnect_interval_, this, &Login::DoReconnect);
+      FROM_HERE, reconnect_interval_, this, &Login::DoReconnect);
   delegate_->OnDisconnect();
 }
 
