@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,35 +7,31 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/time.h"
 #include "base/timer.h"
-#include "base/weak_ptr.h"
+#include "jingle/notifier/base/server_information.h"
 #include "jingle/notifier/communicator/single_login_attempt.h"
 #include "net/base/network_change_notifier.h"
 #include "talk/xmpp/xmppengine.h"
 
 namespace buzz {
 class XmppClient;
-class XmppEngine;
 class XmppClientSettings;
+class XmppTaskParentInterface;
 }  // namespace buzz
 
 namespace net {
-class CertVerifier;
-class HostResolver;
+class URLRequestContextGetter;
 }  // namespace net
 
-namespace talk_base {
-class Task;
-class TaskParent;
-}  // namespace talk_base
-
 namespace notifier {
 
 class ConnectionOptions;
 class LoginSettings;
-struct ServerInformation;
 
 // Workaround for MSVS 2005 bug that fails to handle inheritance from a nested
 // class properly if it comes directly on a base class list.
@@ -44,14 +40,15 @@ typedef SingleLoginAttempt::Delegate SingleLoginAttemptDelegate;
 // Does the login, keeps it alive (with refreshing cookies and reattempting
 // login when disconnected), figures out what actions to take on the various
 // errors that may occur.
-class Login : public net::NetworkChangeNotifier::Observer,
+class Login : public net::NetworkChangeNotifier::IPAddressObserver,
               public SingleLoginAttemptDelegate {
  public:
   class Delegate {
    public:
     virtual ~Delegate() {}
 
-    virtual void OnConnect(base::WeakPtr<talk_base::Task> base_task) = 0;
+    virtual void OnConnect(
+        base::WeakPtr<buzz::XmppTaskParentInterface> base_task) = 0;
     virtual void OnDisconnect() = 0;
   };
 
@@ -60,23 +57,27 @@ class Login : public net::NetworkChangeNotifier::Observer,
   Login(Delegate* delegate,
         const buzz::XmppClientSettings& user_settings,
         const ConnectionOptions& options,
-        net::HostResolver* host_resolver,
-        net::CertVerifier* cert_verifier,
-        ServerInformation* server_list,
-        int server_count,
-        bool try_ssltcp_first);
+        const scoped_refptr<net::URLRequestContextGetter>&
+            request_context_getter,
+        const ServerList& servers,
+        bool try_ssltcp_first,
+        const std::string& auth_mechanism);
   virtual ~Login();
 
   void StartConnection();
+  // The updated settings only take effect the next time StartConnection
+  // is called.
+  void UpdateXmppSettings(const buzz::XmppClientSettings& user_settings);
 
-  // net::NetworkChangeNotifier::Observer implementation.
-  virtual void OnIPAddressChanged();
+  // net::NetworkChangeNotifier::IPAddressObserver implementation.
+  virtual void OnIPAddressChanged() OVERRIDE;
 
   // SingleLoginAttempt::Delegate implementation.
-  virtual void OnConnect(base::WeakPtr<talk_base::Task> base_task);
-  virtual void OnNeedReconnect();
+  virtual void OnConnect(
+      base::WeakPtr<buzz::XmppTaskParentInterface> base_task) OVERRIDE;
+  virtual void OnNeedReconnect() OVERRIDE;
   virtual void OnRedirect(const std::string& redirect_server,
-                          int redirect_port);
+                          int redirect_port) OVERRIDE;
 
  private:
   void OnLogoff();
@@ -109,6 +110,10 @@ class Login : public net::NetworkChangeNotifier::Observer,
   DISALLOW_COPY_AND_ASSIGN(Login);
 };
 
+// Workaround for MSVS 2005 bug that fails to handle inheritance from a nested
+// class properly if it comes directly on a base class list.
+typedef Login::Delegate LoginDelegate;
+
 }  // namespace notifier
 
 #endif  // JINGLE_NOTIFIER_COMMUNICATOR_LOGIN_H_
