@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #include "jingle/notifier/communicator/login_settings.h"
 
 #include "base/logging.h"
+#include "jingle/notifier/base/server_information.h"
 #include "jingle/notifier/communicator/connection_options.h"
 #include "jingle/notifier/communicator/xmpp_connection_generator.h"
 #include "net/base/cert_verifier.h"
@@ -18,26 +19,18 @@ namespace notifier {
 
 LoginSettings::LoginSettings(const buzz::XmppClientSettings& user_settings,
                              const ConnectionOptions& options,
-                             net::HostResolver* host_resolver,
-                             net::CertVerifier* cert_verifier,
-                             ServerInformation* server_list,
-                             int server_count,
-                             bool try_ssltcp_first)
+                             const scoped_refptr<net::URLRequestContextGetter>&
+                                 request_context_getter,
+                             const ServerList& servers,
+                             bool try_ssltcp_first,
+                             const std::string& auth_mechanism)
     :  try_ssltcp_first_(try_ssltcp_first),
-       host_resolver_(host_resolver),
-       cert_verifier_(cert_verifier),
-       server_list_(new ServerInformation[server_count]),
-       server_count_(server_count),
+       request_context_getter_(request_context_getter),
+       servers_(servers),
        user_settings_(new buzz::XmppClientSettings(user_settings)),
-       connection_options_(new ConnectionOptions(options)) {
-  // Note: firewall may be NULL.
-  DCHECK(server_list);
-  DCHECK(host_resolver);
-  DCHECK(cert_verifier);
-  DCHECK_GT(server_count, 0);
-  for (int i = 0; i < server_count_; ++i) {
-    server_list_[i] = server_list[i];
-  }
+       connection_options_(new ConnectionOptions(options)),
+       auth_mechanism_(auth_mechanism) {
+  DCHECK_GT(servers_.size(), 0u);
 }
 
 // Defined so that the destructors are executed here (and the corresponding
@@ -47,9 +40,8 @@ LoginSettings::~LoginSettings() {
 
 void LoginSettings::set_server_override(
     const net::HostPortPair& server) {
-  server_override_.reset(new ServerInformation());
-  server_override_->server = server;
-  server_override_->special_port_magic = server_list_[0].special_port_magic;
+  server_override_.reset(
+      new ServerInformation(server, servers_[0].special_port_magic));
 }
 
 void LoginSettings::clear_server_override() {
