@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,36 +6,27 @@
 #define JINGLE_NOTIFIER_COMMUNICATOR_LOGIN_SETTINGS_H_
 #include <string>
 
+#include "base/memory/ref_counted.h"
+#include "jingle/notifier/base/server_information.h"
 #include "jingle/notifier/communicator/xmpp_connection_generator.h"
-#include "talk/base/scoped_ptr.h"
+#include "net/url_request/url_request_context_getter.h"
 
 namespace buzz {
 class XmppClientSettings;
 }
 
-namespace net {
-class CertVerifier;
-class HostPortPair;
-class HostResolver;
-}
-
-namespace talk_base {
-class SocketAddress;
-}
-
 namespace notifier {
 class ConnectionOptions;
-struct ServerInformation;
 
 class LoginSettings {
  public:
   LoginSettings(const buzz::XmppClientSettings& user_settings,
                 const ConnectionOptions& options,
-                net::HostResolver* host_resolver,
-                net::CertVerifier* cert_verifier,
-                ServerInformation* server_list,
-                int server_count,
-                bool try_ssltcp_first);
+                const scoped_refptr<net::URLRequestContextGetter>&
+                    request_context_getter,
+                const ServerList& servers,
+                bool try_ssltcp_first,
+                const std::string& auth_mechanism);
 
   ~LoginSettings();
 
@@ -43,20 +34,13 @@ class LoginSettings {
     return try_ssltcp_first_;
   }
 
-  net::HostResolver* host_resolver() {
-    return host_resolver_;
+  scoped_refptr<net::URLRequestContextGetter> request_context_getter() {
+    return request_context_getter_;
   }
 
-  net::CertVerifier* cert_verifier() {
-    return cert_verifier_;
-  }
-
-  const ServerInformation* server_list() const {
-    return server_override_.get() ? server_override_.get() : server_list_.get();
-  }
-
-  int server_count() const {
-    return server_override_.get() ? 1 : server_count_;
+  ServerList servers() const {
+    return
+        server_override_.get() ? ServerList(1, *server_override_) : servers_;
   }
 
   const buzz::XmppClientSettings& user_settings() const {
@@ -74,18 +58,22 @@ class LoginSettings {
   void set_server_override(const net::HostPortPair& server);
   void clear_server_override();
 
+  std::string auth_mechanism() const {
+    return auth_mechanism_;
+  }
+
  private:
   bool try_ssltcp_first_;
 
-  net::HostResolver* const host_resolver_;
-  net::CertVerifier* const cert_verifier_;
-  talk_base::scoped_array<ServerInformation> server_list_;
-  int server_count_;
+  scoped_refptr<net::URLRequestContextGetter> request_context_getter_;
+  const ServerList servers_;
   // Used to handle redirects
   scoped_ptr<ServerInformation> server_override_;
 
   scoped_ptr<buzz::XmppClientSettings> user_settings_;
   scoped_ptr<ConnectionOptions> connection_options_;
+  std::string auth_mechanism_;
+
   DISALLOW_COPY_AND_ASSIGN(LoginSettings);
 };
 }  // namespace notifier
