@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,12 +11,14 @@
 
 #include "base/compiler_specific.h"
 #include "base/logging.h"
+#include "jingle/notifier/base/const_communicator.h"
+#include "jingle/notifier/base/gaia_token_pre_xmpp_auth.h"
 #include "jingle/notifier/communicator/connection_options.h"
 #include "jingle/notifier/communicator/connection_settings.h"
-#include "jingle/notifier/communicator/const_communicator.h"
-#include "jingle/notifier/communicator/gaia_token_pre_xmpp_auth.h"
 #include "jingle/notifier/communicator/login_settings.h"
 #include "jingle/notifier/listener/xml_element_util.h"
+#include "net/url_request/url_request_context.h"
+#include "net/url_request/url_request_context_getter.h"
 #include "talk/xmllite/xmlelement.h"
 #include "talk/xmpp/xmppclient.h"
 #include "talk/xmpp/xmppclientsettings.h"
@@ -34,17 +36,21 @@ SingleLoginAttempt::SingleLoginAttempt(LoginSettings* login_settings,
       delegate_(delegate),
       connection_generator_(
           ALLOW_THIS_IN_INITIALIZER_LIST(this),
-          login_settings_->host_resolver(),
+          login_settings_->request_context_getter()->GetURLRequestContext()->
+              host_resolver(),
           &login_settings_->connection_options(),
           login_settings_->try_ssltcp_first(),
-          login_settings_->server_list(),
-          login_settings_->server_count()) {
+          login_settings_->servers()) {
+  // DNS resolution will happen at a lower layer (we are using the socket
+  // pools).
+  connection_generator_.SetShouldResolveDNS(false);
   connection_generator_.StartGenerating();
 }
 
 SingleLoginAttempt::~SingleLoginAttempt() {}
 
-void SingleLoginAttempt::OnConnect(base::WeakPtr<talk_base::Task> base_task) {
+void SingleLoginAttempt::OnConnect(
+    base::WeakPtr<buzz::XmppTaskParentInterface> base_task) {
   delegate_->OnConnect(base_task);
 }
 
@@ -96,13 +102,6 @@ void SingleLoginAttempt::OnError(buzz::XmppEngine::Error error, int subcode,
 
 void SingleLoginAttempt::OnNewSettings(
     const ConnectionSettings& connection_settings) {
-  // TODO(akalin): Resolve any unresolved IPs, possibly through a
-  // proxy, instead of skipping them.
-  if (connection_settings.server().IsUnresolvedIP()) {
-    connection_generator_.UseNextConnection();
-    return;
-  }
-
   buzz::XmppClientSettings client_settings =
       login_settings_->user_settings();
   // Fill in the rest of the client settings.
@@ -113,9 +112,11 @@ void SingleLoginAttempt::OnNewSettings(
   buzz::PreXmppAuth* pre_xmpp_auth =
       new GaiaTokenPreXmppAuth(
           jid.Str(), client_settings.auth_cookie(),
-          client_settings.token_service());
+          client_settings.token_service(),
+          login_settings_->auth_mechanism());
   xmpp_connection_.reset(
-      new XmppConnection(client_settings, login_settings_->cert_verifier(),
+      new XmppConnection(client_settings,
+                         login_settings_->request_context_getter(),
                          this, pre_xmpp_auth));
 }
 
