@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -26,7 +26,7 @@ class MockTalkMediatorDelegate : public TalkMediator::Delegate {
   MOCK_METHOD1(OnNotificationStateChange,
                void(bool notification_changed));
   MOCK_METHOD1(OnIncomingNotification,
-               void(const IncomingNotificationData& data));
+               void(const Notification& data));
   MOCK_METHOD0(OnOutgoingNotification, void());
 
  private:
@@ -40,11 +40,8 @@ class TalkMediatorImplTest : public testing::Test {
 
   TalkMediatorImpl* NewMockedTalkMediator(
       MockMediatorThread* mock_mediator_thread) {
-    const bool kInvalidateXmppAuthToken = false;
-    const bool kAllowInsecureConnection = false;
     return new TalkMediatorImpl(mock_mediator_thread,
-                                kInvalidateXmppAuthToken,
-                                kAllowInsecureConnection);
+                                NotifierOptions());
   }
 
   int last_message_;
@@ -56,41 +53,24 @@ class TalkMediatorImplTest : public testing::Test {
   DISALLOW_COPY_AND_ASSIGN(TalkMediatorImplTest);
 };
 
-TEST_F(TalkMediatorImplTest, SetAuthTokenWithBadInput) {
+TEST_F(TalkMediatorImplTest, SetAuthToken) {
   scoped_ptr<TalkMediatorImpl> talk1(
       NewMockedTalkMediator(new MockMediatorThread()));
-  EXPECT_FALSE(talk1->SetAuthToken("@missinguser.com", "", "fake_service"));
-  EXPECT_FALSE(talk1->state_.initialized);
-
-  scoped_ptr<TalkMediatorImpl> talk2(
-      NewMockedTalkMediator(new MockMediatorThread()));
-  EXPECT_FALSE(talk2->SetAuthToken("", "1234567890", "fake_service"));
-  EXPECT_FALSE(talk2->state_.initialized);
-
-  scoped_ptr<TalkMediatorImpl> talk3(
-      NewMockedTalkMediator(new MockMediatorThread()));
-  EXPECT_FALSE(talk3->SetAuthToken("missingdomain", "abcde",  "fake_service"));
-  EXPECT_FALSE(talk3->state_.initialized);
-}
-
-TEST_F(TalkMediatorImplTest, SetAuthTokenWithGoodInput) {
-  scoped_ptr<TalkMediatorImpl> talk1(
-      NewMockedTalkMediator(new MockMediatorThread()));
-  EXPECT_TRUE(talk1->SetAuthToken("chromium@gmail.com", "token",
-                                  "fake_service"));
+  talk1->SetAuthToken("chromium@gmail.com", "token", "fake_service");
   EXPECT_TRUE(talk1->state_.initialized);
+  talk1->Logout();
 
   scoped_ptr<TalkMediatorImpl> talk2(
       NewMockedTalkMediator(new MockMediatorThread()));
-  EXPECT_TRUE(talk2->SetAuthToken("chromium@mail.google.com", "token",
-                                  "fake_service"));
+  talk2->SetAuthToken("chromium@mail.google.com", "token", "fake_service");
   EXPECT_TRUE(talk2->state_.initialized);
+  talk2->Logout();
 
   scoped_ptr<TalkMediatorImpl> talk3(
       NewMockedTalkMediator(new MockMediatorThread()));
-  EXPECT_TRUE(talk3->SetAuthToken("chromium@chromium.org", "token",
-                                  "fake_service"));
+  talk3->SetAuthToken("chromium@mail.google.com", "token", "fake_service");
   EXPECT_TRUE(talk3->state_.initialized);
+  talk3->Logout();
 }
 
 TEST_F(TalkMediatorImplTest, LoginWiring) {
@@ -102,11 +82,16 @@ TEST_F(TalkMediatorImplTest, LoginWiring) {
   EXPECT_FALSE(talk1->Login());
   EXPECT_EQ(0, mock->login_calls);
 
-  EXPECT_TRUE(talk1->SetAuthToken("chromium@gmail.com", "token",
-                                  "fake_service"));
+  talk1->SetAuthToken("chromium@gmail.com", "token", "fake_service");
+  EXPECT_EQ(0, mock->update_settings_calls);
+
   EXPECT_TRUE(talk1->Login());
   EXPECT_EQ(1, mock->login_calls);
 
+  // We call SetAuthToken again to update the settings after an update.
+  talk1->SetAuthToken("chromium@gmail.com", "token", "fake_service");
+  EXPECT_EQ(1, mock->update_settings_calls);
+
   // Successive calls to login will fail.  One needs to create a new talk
   // mediator object.
   EXPECT_FALSE(talk1->Login());
@@ -125,30 +110,25 @@ TEST_F(TalkMediatorImplTest, SendNotification) {
   MockMediatorThread* mock = new MockMediatorThread();
   scoped_ptr<TalkMediatorImpl> talk1(NewMockedTalkMediator(mock));
 
-  // Failure due to not being logged in.
-  OutgoingNotificationData data;
-  EXPECT_FALSE(talk1->SendNotification(data));
-  EXPECT_EQ(0, mock->send_calls);
+  Notification data;
+  talk1->SendNotification(data);
+  EXPECT_EQ(1, mock->send_calls);
 
-  EXPECT_TRUE(talk1->SetAuthToken("chromium@gmail.com", "token",
-                                  "fake_service"));
+  talk1->SetAuthToken("chromium@gmail.com", "token", "fake_service");
   EXPECT_TRUE(talk1->Login());
   talk1->OnConnectionStateChange(true);
   EXPECT_EQ(1, mock->login_calls);
 
-  // Should be subscribed now.
-  EXPECT_TRUE(talk1->state_.subscribed);
-  EXPECT_TRUE(talk1->SendNotification(data));
-  EXPECT_EQ(1, mock->send_calls);
-  EXPECT_TRUE(talk1->SendNotification(data));
+  talk1->SendNotification(data);
   EXPECT_EQ(2, mock->send_calls);
+  talk1->SendNotification(data);
+  EXPECT_EQ(3, mock->send_calls);
 
   EXPECT_TRUE(talk1->Logout());
   EXPECT_EQ(1, mock->logout_calls);
 
-  // Failure due to being logged out.
-  EXPECT_FALSE(talk1->SendNotification(data));
-  EXPECT_EQ(2, mock->send_calls);
+  talk1->SendNotification(data);
+  EXPECT_EQ(4, mock->send_calls);
 }
 
 TEST_F(TalkMediatorImplTest, MediatorThreadCallbacks) {
@@ -163,29 +143,28 @@ TEST_F(TalkMediatorImplTest, MediatorThreadCallbacks) {
 
   talk1->SetDelegate(&mock_delegate);
 
-  EXPECT_TRUE(talk1->SetAuthToken("chromium@gmail.com", "token",
-                                  "fake_service"));
+  talk1->SetAuthToken("chromium@gmail.com", "token", "fake_service");
   EXPECT_TRUE(talk1->Login());
   EXPECT_EQ(1, mock->login_calls);
 
   // The message triggers calls to listen and subscribe.
   EXPECT_EQ(1, mock->listen_calls);
   EXPECT_EQ(1, mock->subscribe_calls);
-  EXPECT_TRUE(talk1->state_.subscribed);
 
   // After subscription success is receieved, the talk mediator will allow
   // sending of notifications.
-  OutgoingNotificationData outgoing_data;
-  EXPECT_TRUE(talk1->SendNotification(outgoing_data));
+  Notification outgoing_data;
+  talk1->SendNotification(outgoing_data);
   EXPECT_EQ(1, mock->send_calls);
 
-  IncomingNotificationData incoming_data;
-  incoming_data.service_url = "service_url";
-  incoming_data.service_specific_data = "service_data";
+  Notification incoming_data;
+  incoming_data.channel = "service_url";
+  incoming_data.data = "service_data";
   mock->ReceiveNotification(incoming_data);
 
   // Shouldn't trigger a call to the delegate since we disconnect
-  // it before we logout.
+  // it before we logout the mediator thread.
+  talk1->Logout();
   talk1.reset();
 }
 
