@@ -1,26 +1,48 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef MEDIA_AUDIO_AUDIO_MANAGER_H_
 #define MEDIA_AUDIO_AUDIO_MANAGER_H_
 
+#include <string>
+
 #include "base/basictypes.h"
+#include "base/memory/ref_counted.h"
 #include "base/string16.h"
+#include "media/audio/audio_device_name.h"
 #include "media/audio/audio_parameters.h"
 
 class AudioInputStream;
 class AudioOutputStream;
 class MessageLoop;
-
-// TODO(sergeyu): In this interface and some other places AudioParameters struct
-// is passed by value. It is better to change it to const reference.
+namespace base {
+class MessageLoopProxy;
+}
 
 // Manages all audio resources. In particular it owns the AudioOutputStream
 // objects. Provides some convenience functions that avoid the need to provide
 // iterators over the existing streams.
-class AudioManager {
+// TODO(tommi): Make the manager non-refcounted when it's safe to do so.
+// -> Bug 107087.
+class MEDIA_EXPORT AudioManager
+    : public base::RefCountedThreadSafe<AudioManager> {
  public:
+  AudioManager();
+
+#ifndef NDEBUG
+  // Allow base classes in debug builds to override the reference counting
+  // functions.  This allows us to protect against regressions and enforce
+  // correct usage.  The default implementation just calls the base class.
+  virtual void AddRef() const;
+  virtual void Release() const;
+#endif
+
+  // Use to construct the audio manager.
+  // NOTE: There should only be one instance.  If you try to create more than
+  // one instance, it will hit a CHECK().
+  static scoped_refptr<AudioManager> Create();
+
   // Returns true if the OS reports existence of audio devices. This does not
   // guarantee that the existing devices support all formats and sample rates.
   virtual bool HasAudioOutputDevices() = 0;
@@ -34,6 +56,22 @@ class AudioManager {
   // input device for this computer.
   virtual string16 GetAudioInputDeviceModel() = 0;
 
+  // Returns true if the platform specific audio input settings UI is known
+  // and can be shown.
+  virtual bool CanShowAudioInputSettings() = 0;
+
+  // Opens the platform default audio input settings UI.
+  // Note: This could invoke an external application/preferences pane, so
+  // ideally must not be called from the UI thread or other time sensitive
+  // threads to avoid blocking the rest of the application.
+  virtual void ShowAudioInputSettings() = 0;
+
+  // Appends a list of available input devices. It is not guaranteed that
+  // all the devices in the list support all formats and sample rates for
+  // recording.
+  virtual void GetAudioInputDeviceNames(
+      media::AudioDeviceNames* device_names) = 0;
+
   // Factory for all the supported stream formats. |params| defines parameters
   // of the audio stream to be created.
   //
@@ -52,7 +90,8 @@ class AudioManager {
   //    available.
   //
   // Do not free the returned AudioOutputStream. It is owned by AudioManager.
-  virtual AudioOutputStream* MakeAudioOutputStream(AudioParameters params) = 0;
+  virtual AudioOutputStream* MakeAudioOutputStream(
+      const AudioParameters& params) = 0;
 
   // Creates new audio output proxy. A proxy implements
   // AudioOutputStream interface, but unlike regular output stream
@@ -73,34 +112,26 @@ class AudioManager {
   //
   // Do not free the returned AudioInputStream. It is owned by AudioManager.
   // When you are done with it, call |Stop()| and |Close()| to release it.
-  virtual AudioInputStream* MakeAudioInputStream(AudioParameters params) = 0;
+  virtual AudioInputStream* MakeAudioInputStream(
+      const AudioParameters& params, const std::string& device_id) = 0;
 
   // Muting continues playback but effectively the volume is set to zero.
   // Un-muting returns the volume to the previous level.
   virtual void MuteAll() = 0;
   virtual void UnMuteAll() = 0;
 
-  // Returns message loop used for audio IO.
-  virtual MessageLoop* GetMessageLoop() = 0;
+  // Used to determine if something else is currently making use of audio input.
+  virtual bool IsRecordingInProcess() = 0;
 
-  // Get AudioManager singleton.
-  // TODO(cpu): Define threading requirements for interacting with AudioManager.
-  static AudioManager* GetAudioManager();
+  // Returns message loop used for audio IO.
+  virtual scoped_refptr<base::MessageLoopProxy> GetMessageLoop() = 0;
 
  protected:
-  virtual ~AudioManager() {}
-
-  // Called from GetAudioManager() to initialiaze the instance.
+  // Called from Create() to initialize the instance.
   virtual void Init() = 0;
 
-  // Called by Destroy() to cleanup the instance before destruction.
-  virtual void Cleanup() = 0;
-
- private:
-  static void Destroy(void*);
-
-  // Called by GetAudioManager() to create platform-specific audio manager.
-  static AudioManager* CreateAudioManager();
+  friend class base::RefCountedThreadSafe<AudioManager>;
+  virtual ~AudioManager();
 };
 
 #endif  // MEDIA_AUDIO_AUDIO_MANAGER_H_
