@@ -1,11 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/bind.h"
 #include "base/environment.h"
 #include "base/basictypes.h"
 #include "base/logging.h"
-#include "base/task.h"
 #include "base/synchronization/waitable_event.h"
 #include "media/audio/audio_output_controller.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -13,6 +13,7 @@
 
 using ::testing::_;
 using ::testing::AtLeast;
+using ::testing::DoAll;
 using ::testing::Exactly;
 using ::testing::InvokeWithoutArgs;
 using ::testing::NotNull;
@@ -20,10 +21,10 @@ using ::testing::Return;
 
 static const int kSampleRate = AudioParameters::kAudioCDSampleRate;
 static const int kBitsPerSample = 16;
-static const int kChannels = 2;
+static const ChannelLayout kChannelLayout = CHANNEL_LAYOUT_STEREO;
 static const int kSamplesPerPacket = kSampleRate / 10;
-static const int kHardwareBufferSize = kSamplesPerPacket * kChannels *
-    kBitsPerSample / 8;
+static const int kHardwareBufferSize = kSamplesPerPacket *
+    ChannelLayoutToChannelCount(kChannelLayout) * kBitsPerSample / 8;
 static const int kBufferCapacity = 3 * kHardwareBufferSize;
 
 namespace media {
@@ -53,24 +54,12 @@ class MockAudioOutputControllerSyncReader
   MOCK_METHOD1(UpdatePendingBytes, void(uint32 bytes));
   MOCK_METHOD2(Read, uint32(void* data, uint32 size));
   MOCK_METHOD0(Close, void());
+  MOCK_METHOD0(DataReady, bool());
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MockAudioOutputControllerSyncReader);
 };
 
-static bool HasAudioOutputDevices() {
-  AudioManager* audio_man = AudioManager::GetAudioManager();
-  CHECK(audio_man);
-  return audio_man->HasAudioOutputDevices();
-}
-
-static bool IsRunningHeadless() {
-  scoped_ptr<base::Environment> env(base::Environment::Create());
-  if (env->HasVar("CHROME_HEADLESS"))
-    return true;
-  return false;
-}
-
 ACTION_P(SignalEvent, event) {
   event->Signal();
 }
@@ -83,12 +72,13 @@ static void SignalClosedEvent(base::WaitableEvent* event) {
 // Closes AudioOutputController synchronously.
 static void CloseAudioController(AudioOutputController* controller) {
   base::WaitableEvent closed_event(true, false);
-  controller->Close(NewRunnableFunction(&SignalClosedEvent, &closed_event));
+  controller->Close(base::Bind(&SignalClosedEvent, &closed_event));
   closed_event.Wait();
 }
 
 TEST(AudioOutputControllerTest, CreateAndClose) {
-  if (!HasAudioOutputDevices() || IsRunningHeadless())
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
     return;
 
   MockAudioOutputControllerEventHandler event_handler;
@@ -97,10 +87,11 @@ TEST(AudioOutputControllerTest, CreateAndClose) {
       .Times(1);
   EXPECT_CALL(event_handler, OnMoreData(NotNull(), _));
 
-  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannels,
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
   scoped_refptr<AudioOutputController> controller =
-      AudioOutputController::Create(&event_handler, params, kBufferCapacity);
+      AudioOutputController::Create(audio_manager, &event_handler, params,
+                                    kBufferCapacity);
   ASSERT_TRUE(controller.get());
 
   // Close the controller immediately.
@@ -108,7 +99,8 @@ TEST(AudioOutputControllerTest, CreateAndClose) {
 }
 
 TEST(AudioOutputControllerTest, PlayAndClose) {
-  if (!HasAudioOutputDevices() || IsRunningHeadless())
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
     return;
 
   MockAudioOutputControllerEventHandler event_handler;
@@ -127,10 +119,65 @@ TEST(AudioOutputControllerTest, PlayAndClose) {
       .Times(AtLeast(10))
       .WillRepeatedly(SignalEvent(&event));
 
-  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannels,
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
   scoped_refptr<AudioOutputController> controller =
-      AudioOutputController::Create(&event_handler, params, kBufferCapacity);
+      AudioOutputController::Create(audio_manager, &event_handler, params,
+                                    kBufferCapacity);
+  ASSERT_TRUE(controller.get());
+
+  // Wait for OnCreated() to be called.
+  event.Wait();
+
+  controller->Play();
+
+  // Wait until the date is requested at least 10 times.
+  for (int i = 0; i < 10; i++) {
+    event.Wait();
+    uint8 buf[1];
+    controller->EnqueueData(buf, 0);
+  }
+
+  // Now stop the controller.
+  CloseAudioController(controller);
+}
+
+TEST(AudioOutputControllerTest, PlayAndCloseLowLatency) {
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
+    return;
+
+  MockAudioOutputControllerEventHandler event_handler;
+  base::WaitableEvent event(false, false);
+
+  // If OnCreated is called then signal the event.
+  EXPECT_CALL(event_handler, OnCreated(NotNull()))
+      .WillOnce(SignalEvent(&event));
+
+  // OnPlaying() will be called only once.
+  EXPECT_CALL(event_handler, OnPlaying(NotNull()))
+      .Times(Exactly(1));
+
+  MockAudioOutputControllerSyncReader sync_reader;
+  EXPECT_CALL(sync_reader, UpdatePendingBytes(_))
+      .Times(AtLeast(10));
+  EXPECT_CALL(sync_reader, DataReady())
+      .WillOnce(Return(false))
+      .WillOnce(Return(false))
+      .WillRepeatedly(Return(true));
+  EXPECT_CALL(sync_reader, Read(_, kHardwareBufferSize))
+      .Times(AtLeast(10))
+      .WillRepeatedly(DoAll(SignalEvent(&event),
+                            Return(4)));
+  EXPECT_CALL(sync_reader, Close());
+
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
+                         kSampleRate, kBitsPerSample, kSamplesPerPacket);
+  scoped_refptr<AudioOutputController> controller =
+      AudioOutputController::CreateLowLatency(audio_manager,
+                                              &event_handler,
+                                              params,
+                                              &sync_reader);
   ASSERT_TRUE(controller.get());
 
   // Wait for OnCreated() to be called.
@@ -150,7 +197,8 @@ TEST(AudioOutputControllerTest, PlayAndClose) {
 }
 
 TEST(AudioOutputControllerTest, PlayPauseClose) {
-  if (!HasAudioOutputDevices() || IsRunningHeadless())
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
     return;
 
   MockAudioOutputControllerEventHandler event_handler;
@@ -176,10 +224,11 @@ TEST(AudioOutputControllerTest, PlayPauseClose) {
       .Times(Exactly(1))
       .WillOnce(InvokeWithoutArgs(&pause_event, &base::WaitableEvent::Signal));
 
-  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannels,
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
   scoped_refptr<AudioOutputController> controller =
-      AudioOutputController::Create(&event_handler, params, kBufferCapacity);
+      AudioOutputController::Create(audio_manager, &event_handler, params,
+                                    kBufferCapacity);
   ASSERT_TRUE(controller.get());
 
   // Wait for OnCreated() to be called.
@@ -203,8 +252,56 @@ TEST(AudioOutputControllerTest, PlayPauseClose) {
   CloseAudioController(controller);
 }
 
+TEST(AudioOutputControllerTest, PlayPauseCloseLowLatency) {
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
+    return;
+
+  MockAudioOutputControllerEventHandler event_handler;
+  base::WaitableEvent event(false, false);
+  base::WaitableEvent pause_event(false, false);
+
+  // If OnCreated is called then signal the event.
+  EXPECT_CALL(event_handler, OnCreated(NotNull()))
+      .WillOnce(InvokeWithoutArgs(&event, &base::WaitableEvent::Signal));
+
+  // OnPlaying() will be called only once.
+  EXPECT_CALL(event_handler, OnPlaying(NotNull()));
+
+  MockAudioOutputControllerSyncReader sync_reader;
+  EXPECT_CALL(sync_reader, UpdatePendingBytes(_))
+      .Times(AtLeast(2));
+  EXPECT_CALL(sync_reader, Read(_, kHardwareBufferSize))
+      .WillRepeatedly(DoAll(SignalEvent(&event),
+                            Return(4)));
+  EXPECT_CALL(event_handler, OnPaused(NotNull()))
+      .WillOnce(InvokeWithoutArgs(&pause_event, &base::WaitableEvent::Signal));
+  EXPECT_CALL(sync_reader, Close());
+
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
+                         kSampleRate, kBitsPerSample, kSamplesPerPacket);
+  scoped_refptr<AudioOutputController> controller =
+      AudioOutputController::CreateLowLatency(audio_manager,
+                                              &event_handler,
+                                              params,
+                                              &sync_reader);
+  ASSERT_TRUE(controller.get());
+
+  // Wait for OnCreated() to be called.
+  event.Wait();
+
+  ASSERT_FALSE(pause_event.IsSignaled());
+  controller->Play();
+  controller->Pause();
+  pause_event.Wait();
+
+  // Now stop the controller.
+  CloseAudioController(controller);
+}
+
 TEST(AudioOutputControllerTest, PlayPausePlay) {
-  if (!HasAudioOutputDevices() || IsRunningHeadless())
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
     return;
 
   MockAudioOutputControllerEventHandler event_handler;
@@ -236,10 +333,11 @@ TEST(AudioOutputControllerTest, PlayPausePlay) {
     .Times(Exactly(1))
     .RetiresOnSaturation();
 
-  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannels,
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
   scoped_refptr<AudioOutputController> controller =
-      AudioOutputController::Create(&event_handler, params, kBufferCapacity);
+      AudioOutputController::Create(audio_manager, &event_handler, params,
+                                    kBufferCapacity);
   ASSERT_TRUE(controller.get());
 
   // Wait for OnCreated() to be called.
@@ -274,16 +372,17 @@ TEST(AudioOutputControllerTest, PlayPausePlay) {
 }
 
 TEST(AudioOutputControllerTest, HardwareBufferTooLarge) {
-  if (!HasAudioOutputDevices() || IsRunningHeadless())
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
     return;
 
   // Create an audio device with a very large hardware buffer size.
   MockAudioOutputControllerEventHandler event_handler;
-  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannels,
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
                          kSampleRate, kBitsPerSample,
                          kSamplesPerPacket * 1000);
   scoped_refptr<AudioOutputController> controller =
-      AudioOutputController::Create(&event_handler, params,
+      AudioOutputController::Create(audio_manager, &event_handler, params,
                                     kBufferCapacity);
 
   // Use assert because we don't stop the device and assume we can't
@@ -292,7 +391,8 @@ TEST(AudioOutputControllerTest, HardwareBufferTooLarge) {
 }
 
 TEST(AudioOutputControllerTest, CloseTwice) {
-  if (!HasAudioOutputDevices() || IsRunningHeadless())
+  scoped_refptr<AudioManager> audio_manager(AudioManager::Create());
+  if (!audio_manager->HasAudioOutputDevices())
     return;
 
   MockAudioOutputControllerEventHandler event_handler;
@@ -307,10 +407,11 @@ TEST(AudioOutputControllerTest, CloseTwice) {
       .Times(AtLeast(1))
       .WillRepeatedly(SignalEvent(&event));
 
-  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannels,
+  AudioParameters params(AudioParameters::AUDIO_PCM_LINEAR, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
   scoped_refptr<AudioOutputController> controller =
-      AudioOutputController::Create(&event_handler, params, kBufferCapacity);
+      AudioOutputController::Create(audio_manager, &event_handler, params,
+                                    kBufferCapacity);
   ASSERT_TRUE(controller.get());
 
   // Wait for OnCreated() to be called.
@@ -320,10 +421,10 @@ TEST(AudioOutputControllerTest, CloseTwice) {
   event.Wait();
 
   base::WaitableEvent closed_event_1(true, false);
-  controller->Close(NewRunnableFunction(&SignalClosedEvent, &closed_event_1));
+  controller->Close(base::Bind(&SignalClosedEvent, &closed_event_1));
 
   base::WaitableEvent closed_event_2(true, false);
-  controller->Close(NewRunnableFunction(&SignalClosedEvent, &closed_event_2));
+  controller->Close(base::Bind(&SignalClosedEvent, &closed_event_2));
 
   closed_event_1.Wait();
   closed_event_2.Wait();
