@@ -1,72 +1,31 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
 // THREAD SAFETY
 //
-// The AlsaPcmOutputStream object's internal state is accessed by two threads:
+// AlsaPcmOutputStream object is *not* thread-safe and should only be used
+// from the audio thread.  We DCHECK on this assumption whenever we can.
 //
-//   client thread - creates the object and calls the public APIs.
-//   message loop thread - executes all the internal tasks including querying
-//      the data source for more data, writing to the alsa device, and closing
-//      the alsa device.  It does *not* handle opening the device though.
+// SEMANTICS OF Close()
 //
-// The class is designed so that most operations that read/modify the object's
-// internal state are done on the message loop thread.  The exception is data
-// conatined in the |shared_data_| structure.  Data in this structure needs to
-// be accessed by both threads, and should only be accessed when the
-// |shared_data_.lock_| is held.
-//
-// All member variables that are not in |shared_data_| are created/destroyed on
-// the |message_loop_|.  This allows safe access to them from any task posted to
-// |message_loop_|.  The values in |shared_data_| are considered to be read-only
-// signals by tasks posted to |message_loop_| (see the SEMANTICS of
-// |shared_data_| section below).  Because of these two constraints, tasks can,
-// and must, be coded to be safe in the face of a changing |shared_data_|.
-//
-//
-// SEMANTICS OF |shared_data_|
-//
-// Though |shared_data_| is accessable by both threads, the code has been
-// structured so that all mutations to |shared_data_| are only done in the
-// client thread.  The message loop thread only ever reads the shared data.
-//
-// This reduces the need for critical sections because the public API code can
-// assume that no mutations occur to the |shared_data_| between queries.
-//
-// On the message loop side, tasks have been coded such that they can
-// operate safely regardless of when state changes happen to |shared_data_|.
-// Code that is sensitive to the timing of state changes are delegated to the
-// |shared_data_| object so they can executed while holding
-// |shared_data_.lock_|.
-//
-//
-// SEMANTICS OF CloseTask()
-//
-// The CloseTask() is responsible for cleaning up any resources that were
-// acquired after a successful Open().  After a CloseTask() has executed,
-// scheduling of reads should stop.  Currently scheduled tasks may run, but
-// they should not attempt to access any of the internal structures beyond
-// querying the |stop_stream_| flag and no-oping themselves.  This will
-// guarantee that eventually no more tasks will be posted into the message
-// loop, and the AlsaPcmOutputStream will be able to delete itself.
+// Close() is responsible for cleaning up any resources that were acquired after
+// a successful Open().  Close() will nullify any scheduled outstanding runnable
+// methods.
 //
 //
 // SEMANTICS OF ERROR STATES
 //
-// The object has two distinct error states: |shared_data_.state_| == kInError
-// and |stop_stream_|.  The |shared_data_.state_| state is only settable
-// by the client thread, and thus cannot be used to signal when the ALSA device
-// fails due to a hardware (or other low-level) event.  The |stop_stream_|
-// variable is only accessed by the message loop thread; it is used to indicate
+// The object has two distinct error states: |state_| == kInError
+// and |stop_stream_|.  The |stop_stream_| variable is used to indicate
 // that the playback_handle should no longer be used either because of a
-// hardware/low-level event, or because the CloseTask() has been run.
+// hardware/low-level event.
 //
-// When |shared_data_.state_| == kInError, all public API functions will fail
-// with an error (Start() will call the OnError() function on the callback
-// immediately), or no-op themselves with the exception of Close().  Even if an
-// error state has been entered, if Open() has previously returned successfully,
-// Close() must be called to cleanup the ALSA devices and release resources.
+// When |state_| == kInError, all public API functions will fail with an error
+// (Start() will call the OnError() function on the callback immediately), or
+// no-op themselves with the exception of Close().  Even if an error state has
+// been entered, if Open() has previously returned successfully, Close() must be
+// called to cleanup the ALSA devices and release resources.
 //
 // When |stop_stream_| is set, no more commands will be made against the
 // ALSA device, and playback will effectively stop.  From the client's point of
@@ -77,9 +36,11 @@
 
 #include <algorithm>
 
+#include "base/bind.h"
+#include "base/debug/trace_event.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/time.h"
 #include "media/audio/audio_util.h"
 #include "media/audio/linux/alsa_util.h"
@@ -92,6 +53,9 @@
 // busy looping.
 static const uint32 kNoDataSleepMilliseconds = 10;
 
+// Mininum interval between OnMoreData() calls.
+const uint32 kMinIntervalBetweenOnMoreDataCallsInMs = 5;
+
 // According to the linux nanosleep manpage, nanosleep on linux can miss the
 // deadline by up to 10ms because the kernel timeslice is 10ms.  Give a 2x
 // buffer to compensate for the timeslice, and any additional slowdowns.
@@ -105,20 +69,9 @@ static const int kPcmRecoverIsSilent = 1;
 static const int kPcmRecoverIsSilent = 0;
 #endif
 
-const char AlsaPcmOutputStream::kDefaultDevice[] = "default";
-const char AlsaPcmOutputStream::kAutoSelectDevice[] = "";
-const char AlsaPcmOutputStream::kPlugPrefix[] = "plug:";
-
-// Since we expect to only be able to wake up with a resolution of
-// kSleepErrorMilliseconds, double that for our minimum required latency.
-const uint32 AlsaPcmOutputStream::kMinLatencyMicros =
-    kSleepErrorMilliseconds * 2 * 1000;
-
-namespace {
-
-// ALSA is currently limited to 48Khz.
+// ALSA is currently limited to 48kHz.
 // TODO(fbarchard): Resample audio from higher frequency to 48000.
-const int kAlsaMaxSampleRate = 48000;
+static const int kAlsaMaxSampleRate = 48000;
 
 // While the "default" device may support multi-channel audio, in Alsa, only
 // the device names surround40, surround41, surround50, etc, have a defined
@@ -137,7 +90,7 @@ const int kAlsaMaxSampleRate = 48000;
 // TODO(ajwong): The source data should have enough info to tell us if we want
 // surround41 versus surround51, etc., instead of needing us to guess base don
 // channel number.  Fix API to pass that data down.
-const char* GuessSpecificDeviceName(uint32 channels) {
+static const char* GuessSpecificDeviceName(uint32 channels) {
   switch (channels) {
     case 8:
       return "surround71";
@@ -190,10 +143,6 @@ static void Swizzle51Layout(Format* b, uint32 filled) {
   }
 }
 
-}  // namespace
-
-// Not in an anonymous namespace so that it can be a friend to
-// AlsaPcmOutputStream.
 std::ostream& operator<<(std::ostream& os,
                          AlsaPcmOutputStream::InternalState state) {
   switch (state) {
@@ -219,13 +168,20 @@ std::ostream& operator<<(std::ostream& os,
   return os;
 }
 
+const char AlsaPcmOutputStream::kDefaultDevice[] = "default";
+const char AlsaPcmOutputStream::kAutoSelectDevice[] = "";
+const char AlsaPcmOutputStream::kPlugPrefix[] = "plug:";
+
+// Since we expect to only be able to wake up with a resolution of
+// kSleepErrorMilliseconds, double that for our minimum required latency.
+const uint32 AlsaPcmOutputStream::kMinLatencyMicros =
+    kSleepErrorMilliseconds * 2 * 1000;
+
 AlsaPcmOutputStream::AlsaPcmOutputStream(const std::string& device_name,
-                                         AudioParameters params,
+                                         const AudioParameters& params,
                                          AlsaWrapper* wrapper,
-                                         AudioManagerLinux* manager,
-                                         MessageLoop* message_loop)
-    : shared_data_(MessageLoop::current()),
-      requested_device_name_(device_name),
+                                         AudioManagerLinux* manager)
+    : requested_device_name_(device_name),
       pcm_format_(alsa_util::BitsToFormat(params.bits_per_sample)),
       channels_(params.channels),
       sample_rate_(params.sample_rate),
@@ -244,45 +200,46 @@ AlsaPcmOutputStream::AlsaPcmOutputStream(const std::string& device_name,
       manager_(manager),
       playback_handle_(NULL),
       frames_per_packet_(packet_size_ / bytes_per_frame_),
-      client_thread_loop_(MessageLoop::current()),
-      message_loop_(message_loop) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)),
+      state_(kCreated),
+      volume_(1.0f),
+      source_callback_(NULL) {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
   // Sanity check input values.
   if ((params.sample_rate > kAlsaMaxSampleRate) || (params.sample_rate <= 0)) {
     LOG(WARNING) << "Unsupported audio frequency.";
-    shared_data_.TransitionTo(kInError);
+    TransitionTo(kInError);
   }
 
-  if (AudioParameters::AUDIO_PCM_LINEAR != params.format) {
-    LOG(WARNING) << "Only linear PCM supported.";
-    shared_data_.TransitionTo(kInError);
+  if (AudioParameters::AUDIO_PCM_LINEAR != params.format &&
+      AudioParameters::AUDIO_PCM_LOW_LATENCY != params.format) {
+    LOG(WARNING) << "Unsupported audio format";
+    TransitionTo(kInError);
   }
 
   if (pcm_format_ == SND_PCM_FORMAT_UNKNOWN) {
     LOG(WARNING) << "Unsupported bits per sample: " << params.bits_per_sample;
-    shared_data_.TransitionTo(kInError);
+    TransitionTo(kInError);
   }
 }
 
 AlsaPcmOutputStream::~AlsaPcmOutputStream() {
-  InternalState state = shared_data_.state();
-  DCHECK(state == kCreated || state == kIsClosed || state == kInError);
-
-  // TODO(ajwong): Ensure that CloseTask has been called and the
-  // playback handle released by DCHECKing that playback_handle_ is NULL.
-  // Currently, because of Bug 18217, there is a race condition on destruction
-  // where the stream is not always stopped and closed, causing this to fail.
+  InternalState current_state = state();
+  DCHECK(current_state == kCreated ||
+         current_state == kIsClosed ||
+         current_state == kInError);
+  DCHECK(!playback_handle_);
 }
 
 bool AlsaPcmOutputStream::Open() {
-  DCHECK_EQ(MessageLoop::current(), client_thread_loop_);
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  if (shared_data_.state() == kInError) {
+  if (state() == kInError)
     return false;
-  }
 
-  if (!shared_data_.CanTransitionTo(kIsOpened)) {
-    NOTREACHED() << "Invalid state: " << shared_data_.state();
+  if (!CanTransitionTo(kIsOpened)) {
+    NOTREACHED() << "Invalid state: " << state();
     return false;
   }
 
@@ -290,77 +247,13 @@ bool AlsaPcmOutputStream::Open() {
   // CanTransitionTo() was checked above, and it is assumed that this
   // object's public API is only called on one thread so the state cannot
   // transition out from under us.
-  shared_data_.TransitionTo(kIsOpened);
-  message_loop_->PostTask(
-      FROM_HERE,
-      NewRunnableMethod(this, &AlsaPcmOutputStream::OpenTask));
-
-  return true;
-}
-
-void AlsaPcmOutputStream::Close() {
-  DCHECK_EQ(MessageLoop::current(), client_thread_loop_);
-
-  // Sanity check that the transition occurs correctly.  It is safe to
-  // continue anyways because all operations for closing are idempotent.
-  if (shared_data_.TransitionTo(kIsClosed) != kIsClosed) {
-    NOTREACHED() << "Unable to transition Closed.";
-  }
-
-  message_loop_->PostTask(
-      FROM_HERE,
-      NewRunnableMethod(this, &AlsaPcmOutputStream::CloseTask));
-
-  // Signal to the manager that we're closed and can be removed.  Since
-  // we just posted a CloseTask to the message loop, we won't be deleted
-  // immediately, but it will happen soon afterwards.
-  manager()->ReleaseOutputStream(this);
-}
-
-void AlsaPcmOutputStream::Start(AudioSourceCallback* callback) {
-  DCHECK_EQ(MessageLoop::current(), client_thread_loop_);
-
-  CHECK(callback);
-
-  shared_data_.set_source_callback(callback);
-
-  // Only post the task if we can enter the playing state.
-  if (shared_data_.TransitionTo(kIsPlaying) == kIsPlaying) {
-    message_loop_->PostTask(
-        FROM_HERE,
-        NewRunnableMethod(this, &AlsaPcmOutputStream::StartTask));
-  }
-}
-
-void AlsaPcmOutputStream::Stop() {
-  DCHECK_EQ(MessageLoop::current(), client_thread_loop_);
-
-  // Reset the callback, so that it is not called anymore.
-  shared_data_.set_source_callback(NULL);
-
-  shared_data_.TransitionTo(kIsStopped);
-}
-
-void AlsaPcmOutputStream::SetVolume(double volume) {
-  DCHECK_EQ(MessageLoop::current(), client_thread_loop_);
-
-  shared_data_.set_volume(static_cast<float>(volume));
-}
-
-void AlsaPcmOutputStream::GetVolume(double* volume) {
-  DCHECK_EQ(MessageLoop::current(), client_thread_loop_);
-
-  *volume = shared_data_.volume();
-}
-
-void AlsaPcmOutputStream::OpenTask() {
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+  TransitionTo(kIsOpened);
 
   // Try to open the device.
   if (requested_device_name_ == kAutoSelectDevice) {
     playback_handle_ = AutoSelectDevice(latency_micros_);
     if (playback_handle_)
-      VLOG(1) << "Auto-selected device: " << device_name_;
+      DVLOG(1) << "Auto-selected device: " << device_name_;
   } else {
     device_name_ = requested_device_name_;
     playback_handle_ = alsa_util::OpenPlaybackDevice(wrapper_,
@@ -373,6 +266,8 @@ void AlsaPcmOutputStream::OpenTask() {
   // Finish initializing the stream if the device was opened successfully.
   if (playback_handle_ == NULL) {
     stop_stream_ = true;
+    TransitionTo(kInError);
+    return false;
   } else {
     bytes_per_output_frame_ = should_downmix_ ? 2 * bytes_per_sample_ :
         bytes_per_frame_;
@@ -387,66 +282,109 @@ void AlsaPcmOutputStream::OpenTask() {
     if (error < 0) {
       LOG(ERROR) << "Failed to get playback buffer size from ALSA: "
                  << wrapper_->StrError(error);
-      alsa_buffer_frames_ = frames_per_packet_;
+      // Buffer size is at least twice of packet size.
+      alsa_buffer_frames_ = frames_per_packet_ * 2;
     } else {
       alsa_buffer_frames_ = buffer_size;
     }
   }
+
+  return true;
 }
 
-void AlsaPcmOutputStream::StartTask() {
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+void AlsaPcmOutputStream::Close() {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  if (stop_stream_) {
-    return;
+  // Sanity check that the transition occurs correctly.  It is safe to
+  // continue anyways because all operations for closing are idempotent.
+  if (TransitionTo(kIsClosed) != kIsClosed) {
+    NOTREACHED() << "Unable to transition Closed.";
+  } else {
+    // Shutdown the audio device.
+    if (playback_handle_ &&
+        alsa_util::CloseDevice(wrapper_, playback_handle_) < 0) {
+      LOG(WARNING) << "Unable to close audio device. Leaking handle.";
+    }
+    playback_handle_ = NULL;
+
+    // Release the buffer.
+    buffer_.reset();
+
+    // Signal anything that might already be scheduled to stop.
+    stop_stream_ = true;  // Not necessary in production, but unit tests
+                          // uses the flag to verify that stream was closed.
+    weak_factory_.InvalidateWeakPtrs();
+
+    // Signal to the manager that we're closed and can be removed.
+    // Should be last call in the method as it deletes "this".
+    manager_->ReleaseOutputStream(this);
   }
-
-  // When starting again, drop all packets in the device and prepare it again
-  // incase we are restarting from a pause state and need to flush old data.
-  int error = wrapper_->PcmDrop(playback_handle_);
-  if (error < 0 && error != -EAGAIN) {
-    LOG(ERROR) << "Failure clearing playback device ("
-               << wrapper_->PcmName(playback_handle_) << "): "
-               << wrapper_->StrError(error);
-    stop_stream_ = true;
-    return;
-  }
-
-  error = wrapper_->PcmPrepare(playback_handle_);
-  if (error < 0 && error != -EAGAIN) {
-    LOG(ERROR) << "Failure preparing stream ("
-               << wrapper_->PcmName(playback_handle_) << "): "
-               << wrapper_->StrError(error);
-    stop_stream_ = true;
-    return;
-  }
-
-  ScheduleNextWrite(false);
 }
 
-void AlsaPcmOutputStream::CloseTask() {
-  // NOTE: Keep this function idempotent to handle errors that might cause
-  // multiple CloseTasks to be posted.
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+void AlsaPcmOutputStream::Start(AudioSourceCallback* callback) {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  // Shutdown the audio device.
-  if (playback_handle_ &&
-      alsa_util::CloseDevice(wrapper_, playback_handle_) < 0) {
-    LOG(WARNING) << "Unable to close audio device. Leaking handle.";
+  CHECK(callback);
+
+  if (stop_stream_)
+    return;
+
+  set_source_callback(callback);
+
+  // Only post the task if we can enter the playing state.
+  if (TransitionTo(kIsPlaying) == kIsPlaying) {
+    // Before starting, the buffer might have audio from previous user of this
+    // device.
+    buffer_->Clear();
+
+    // When starting again, drop all packets in the device and prepare it again
+    // in case we are restarting from a pause state and need to flush old data.
+    int error = wrapper_->PcmDrop(playback_handle_);
+    if (error < 0 && error != -EAGAIN) {
+      LOG(ERROR) << "Failure clearing playback device ("
+                 << wrapper_->PcmName(playback_handle_) << "): "
+                 << wrapper_->StrError(error);
+      stop_stream_ = true;
+    } else {
+      error = wrapper_->PcmPrepare(playback_handle_);
+      if (error < 0 && error != -EAGAIN) {
+        LOG(ERROR) << "Failure preparing stream ("
+                   << wrapper_->PcmName(playback_handle_) << "): "
+                   << wrapper_->StrError(error);
+        stop_stream_ = true;
+      }
+    }
+
+    if (!stop_stream_)
+      ScheduleNextWrite(false);
   }
-  playback_handle_ = NULL;
+}
 
-  // Release the buffer.
-  buffer_.reset();
+void AlsaPcmOutputStream::Stop() {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  // Signal anything that might already be scheduled to stop.
-  stop_stream_ = true;
+  // Reset the callback, so that it is not called anymore.
+  set_source_callback(NULL);
+
+  TransitionTo(kIsStopped);
+}
+
+void AlsaPcmOutputStream::SetVolume(double volume) {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
+
+  volume_ = static_cast<float>(volume);
+}
+
+void AlsaPcmOutputStream::GetVolume(double* volume) {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
+
+  *volume = volume_;
 }
 
 void AlsaPcmOutputStream::BufferPacket(bool* source_exhausted) {
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  // If stopped, simulate a 0-lengthed packet.
+  // If stopped, simulate a 0-length packet.
   if (stop_stream_) {
     buffer_->Clear();
     *source_exhausted = true;
@@ -455,8 +393,9 @@ void AlsaPcmOutputStream::BufferPacket(bool* source_exhausted) {
 
   *source_exhausted = false;
 
-  // Request more data if we have capacity.
-  if (buffer_->forward_capacity() > buffer_->forward_bytes()) {
+  // Request more data only when we run out of data in the buffer, because
+  // WritePacket() comsumes only the current chunk of data.
+  if (!buffer_->forward_bytes()) {
     // Before making a request to source for data we need to determine the
     // delay (in bytes) for the requested data to be played.
 
@@ -467,12 +406,11 @@ void AlsaPcmOutputStream::BufferPacket(bool* source_exhausted) {
 
     scoped_refptr<media::DataBuffer> packet =
         new media::DataBuffer(packet_size_);
-    size_t packet_size =
-        shared_data_.OnMoreData(
-            this, packet->GetWritableData(), packet->GetBufferSize(),
-            AudioBuffersState(buffer_delay, hardware_delay));
-    CHECK(packet_size <= packet->GetBufferSize()) <<
-        "Data source overran buffer.";
+    size_t packet_size = RunDataCallback(packet->GetWritableData(),
+                                         packet->GetBufferSize(),
+                                         AudioBuffersState(buffer_delay,
+                                                           hardware_delay));
+    CHECK(packet_size <= packet->GetBufferSize());
 
     // This should not happen, but in case it does, drop any trailing bytes
     // that aren't large enough to make a frame.  Without this, packet writing
@@ -486,10 +424,9 @@ void AlsaPcmOutputStream::BufferPacket(bool* source_exhausted) {
                               packet_size,
                               channels_,
                               bytes_per_sample_,
-                              shared_data_.volume())) {
+                              volume_)) {
         // Adjust packet size for downmix.
-        packet_size =
-            packet_size / bytes_per_frame_ * bytes_per_output_frame_;
+        packet_size = packet_size / bytes_per_frame_ * bytes_per_output_frame_;
       } else {
         LOG(ERROR) << "Folding failed";
       }
@@ -522,7 +459,7 @@ void AlsaPcmOutputStream::BufferPacket(bool* source_exhausted) {
                           packet_size,
                           channels_,
                           bytes_per_sample_,
-                          shared_data_.volume());
+                          volume_);
     }
 
     if (packet_size > 0) {
@@ -536,7 +473,7 @@ void AlsaPcmOutputStream::BufferPacket(bool* source_exhausted) {
 }
 
 void AlsaPcmOutputStream::WritePacket() {
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
   // If the device is in error, just eat the bytes.
   if (stop_stream_) {
@@ -544,6 +481,9 @@ void AlsaPcmOutputStream::WritePacket() {
     return;
   }
 
+  if (state() == kIsStopped)
+    return;
+
   CHECK_EQ(buffer_->forward_bytes() % bytes_per_output_frame_, 0u);
 
   const uint8* buffer_data;
@@ -572,7 +512,7 @@ void AlsaPcmOutputStream::WritePacket() {
       if (frames_written != -EAGAIN) {
         LOG(ERROR) << "Failed to write to pcm device: "
                    << wrapper_->StrError(frames_written);
-        shared_data_.OnError(this, frames_written);
+        RunErrorCallback(frames_written);
         stop_stream_ = true;
       }
     } else {
@@ -585,15 +525,25 @@ void AlsaPcmOutputStream::WritePacket() {
       // Seek forward in the buffer after we've written some data to ALSA.
       buffer_->Seek(frames_written * bytes_per_output_frame_);
     }
+  } else {
+    // If nothing left to write and playback hasn't started yet, start it now.
+    // This ensures that shorter sounds will still play.
+    if (playback_handle_ &&
+        (wrapper_->PcmState(playback_handle_) == SND_PCM_STATE_PREPARED) &&
+        GetCurrentDelay() > 0) {
+      wrapper_->PcmStart(playback_handle_);
+    }
   }
 }
 
 void AlsaPcmOutputStream::WriteTask() {
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  if (stop_stream_) {
+  if (stop_stream_)
+    return;
+
+  if (state() == kIsStopped)
     return;
-  }
 
   bool source_exhausted;
   BufferPacket(&source_exhausted);
@@ -603,50 +553,60 @@ void AlsaPcmOutputStream::WriteTask() {
 }
 
 void AlsaPcmOutputStream::ScheduleNextWrite(bool source_exhausted) {
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  if (stop_stream_) {
+  if (stop_stream_)
     return;
-  }
 
-  // Next write is scheduled for the moment when half of the buffer is
-  // available.
   uint32 frames_avail_wanted = alsa_buffer_frames_ / 2;
   uint32 available_frames = GetAvailableFrames();
-  uint32 next_fill_time_ms = 0;
+  uint32 frames_in_buffer = buffer_->forward_bytes() / bytes_per_output_frame_;
 
-  // It's possible to have more frames available than what we want, in which
-  // case we'll leave our |next_fill_time_ms| at 0ms.
-  if (available_frames < frames_avail_wanted) {
-    uint32 frames_until_empty_enough = frames_avail_wanted - available_frames;
-    next_fill_time_ms =
-        FramesToMillis(frames_until_empty_enough, sample_rate_);
-  }
+  // Next write is initially scheduled for the moment when half of a packet
+  // has been played out.
+  uint32 next_fill_time_ms =
+      FramesToMillis(frames_per_packet_ / 2, sample_rate_);
 
-  // Adjust for timer resolution issues.
-  if (next_fill_time_ms < kSleepErrorMilliseconds) {
+  if (frames_in_buffer && (frames_in_buffer <= available_frames)) {
+    // There is data in the current buffer, consume them immediately if we have
+    // enough space in the soundcard.
     next_fill_time_ms = 0;
   } else {
-    next_fill_time_ms -= kSleepErrorMilliseconds;
+    // Otherwise schedule the next write for the moment when half of the alsa
+    // buffer becomes available.
+    if (available_frames < frames_avail_wanted) {
+      uint32 frames_until_empty_enough = frames_avail_wanted - available_frames;
+      next_fill_time_ms =
+          FramesToMillis(frames_until_empty_enough, sample_rate_);
+
+      // Adjust for time resolution.
+      if (next_fill_time_ms > kNoDataSleepMilliseconds)
+        next_fill_time_ms -= kNoDataSleepMilliseconds;
+
+      // Avoid back-to-back writing.
+      if (next_fill_time_ms < kMinIntervalBetweenOnMoreDataCallsInMs)
+        next_fill_time_ms = kMinIntervalBetweenOnMoreDataCallsInMs;
+    } else if (available_frames == alsa_buffer_frames_) {
+      // Buffer is empty, invoke next write immediately.
+      next_fill_time_ms = 0;
+    }
   }
 
   // Avoid busy looping if the data source is exhausted.
-  if (source_exhausted) {
+  if (source_exhausted)
     next_fill_time_ms = std::max(next_fill_time_ms, kNoDataSleepMilliseconds);
-  }
 
   // Only schedule more reads/writes if we are still in the playing state.
-  if (shared_data_.state() == kIsPlaying) {
+  if (state() == kIsPlaying) {
     if (next_fill_time_ms == 0) {
-      message_loop_->PostTask(
-          FROM_HERE,
-          NewRunnableMethod(this, &AlsaPcmOutputStream::WriteTask));
+      manager_->GetMessageLoop()->PostTask(FROM_HERE, base::Bind(
+          &AlsaPcmOutputStream::WriteTask, weak_factory_.GetWeakPtr()));
     } else {
       // TODO(ajwong): Measure the reliability of the delay interval.  Use
       // base/metrics/histogram.h.
-      message_loop_->PostDelayedTask(
-          FROM_HERE,
-          NewRunnableMethod(this, &AlsaPcmOutputStream::WriteTask),
+      manager_->GetMessageLoop()->PostDelayedTask(FROM_HERE,
+          base::Bind(&AlsaPcmOutputStream::WriteTask,
+                     weak_factory_.GetWeakPtr()),
           next_fill_time_ms);
     }
   }
@@ -668,9 +628,8 @@ std::string AlsaPcmOutputStream::FindDeviceForChannels(uint32 channels) {
   static const char kNameHintName[] = "NAME";
 
   const char* wanted_device = GuessSpecificDeviceName(channels);
-  if (!wanted_device) {
+  if (!wanted_device)
     return "";
-  }
 
   std::string guessed_device;
   void** hints = NULL;
@@ -697,7 +656,7 @@ std::string AlsaPcmOutputStream::FindDeviceForChannels(uint32 channels) {
       }
     }
 
-    // Destory the hint now that we're done with it.
+    // Destroy the hint now that we're done with it.
     wrapper_->DeviceNameFreeHint(hints);
     hints = NULL;
   } else {
@@ -736,11 +695,10 @@ snd_pcm_sframes_t AlsaPcmOutputStream::GetCurrentDelay() {
 }
 
 snd_pcm_sframes_t AlsaPcmOutputStream::GetAvailableFrames() {
-  DCHECK_EQ(message_loop_, MessageLoop::current());
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  if (stop_stream_) {
+  if (stop_stream_)
     return 0;
-  }
 
   // Find the number of frames queued in the sound device.
   snd_pcm_sframes_t available_frames =
@@ -763,7 +721,7 @@ snd_pcm_t* AlsaPcmOutputStream::AutoSelectDevice(unsigned int latency) {
   // For auto-selection:
   //   1) Attempt to open a device that best matches the number of channels
   //      requested.
-  //   2) If that fails, attempt the "plug:" version of it incase ALSA can
+  //   2) If that fails, attempt the "plug:" version of it in case ALSA can
   //      remap do some software conversion to make it work.
   //   3) Fallback to kDefaultDevice.
   //   4) If that fails too, try the "plug:" version of kDefaultDevice.
@@ -823,28 +781,7 @@ snd_pcm_t* AlsaPcmOutputStream::AutoSelectDevice(unsigned int latency) {
   return NULL;
 }
 
-AudioManagerLinux* AlsaPcmOutputStream::manager() {
-  DCHECK_EQ(MessageLoop::current(), client_thread_loop_);
-  return manager_;
-}
-
-AlsaPcmOutputStream::SharedData::SharedData(
-    MessageLoop* state_transition_loop)
-    : state_(kCreated),
-      volume_(1.0f),
-      source_callback_(NULL),
-      state_transition_loop_(state_transition_loop) {
-}
-
-bool AlsaPcmOutputStream::SharedData::CanTransitionTo(InternalState to) {
-  base::AutoLock l(lock_);
-  return CanTransitionTo_Locked(to);
-}
-
-bool AlsaPcmOutputStream::SharedData::CanTransitionTo_Locked(
-    InternalState to) {
-  lock_.AssertAcquired();
-
+bool AlsaPcmOutputStream::CanTransitionTo(InternalState to) {
   switch (state_) {
     case kCreated:
       return to == kIsOpened || to == kIsClosed || to == kInError;
@@ -871,11 +808,10 @@ bool AlsaPcmOutputStream::SharedData::CanTransitionTo_Locked(
 }
 
 AlsaPcmOutputStream::InternalState
-AlsaPcmOutputStream::SharedData::TransitionTo(InternalState to) {
-  DCHECK_EQ(MessageLoop::current(), state_transition_loop_);
+AlsaPcmOutputStream::TransitionTo(InternalState to) {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
 
-  base::AutoLock l(lock_);
-  if (!CanTransitionTo_Locked(to)) {
+  if (!CanTransitionTo(to)) {
     NOTREACHED() << "Cannot transition from: " << state_ << " to: " << to;
     state_ = kInError;
   } else {
@@ -884,45 +820,29 @@ AlsaPcmOutputStream::SharedData::TransitionTo(InternalState to) {
   return state_;
 }
 
-AlsaPcmOutputStream::InternalState AlsaPcmOutputStream::SharedData::state() {
-  base::AutoLock l(lock_);
+AlsaPcmOutputStream::InternalState AlsaPcmOutputStream::state() {
   return state_;
 }
 
-float AlsaPcmOutputStream::SharedData::volume() {
-  base::AutoLock l(lock_);
-  return volume_;
-}
+uint32 AlsaPcmOutputStream::RunDataCallback(uint8* dest,
+                                            uint32 max_size,
+                                            AudioBuffersState buffers_state) {
+  TRACE_EVENT0("audio", "AlsaPcmOutputStream::RunDataCallback");
 
-void AlsaPcmOutputStream::SharedData::set_volume(float v) {
-  base::AutoLock l(lock_);
-  volume_ = v;
-}
-
-uint32 AlsaPcmOutputStream::SharedData::OnMoreData(
-    AudioOutputStream* stream, uint8* dest, uint32 max_size,
-    AudioBuffersState buffers_state) {
-  base::AutoLock l(lock_);
-  if (source_callback_) {
-    return source_callback_->OnMoreData(stream, dest, max_size, buffers_state);
-  }
+  if (source_callback_)
+    return source_callback_->OnMoreData(this, dest, max_size, buffers_state);
 
   return 0;
 }
 
-void AlsaPcmOutputStream::SharedData::OnError(AudioOutputStream* stream,
-                                              int code) {
-  base::AutoLock l(lock_);
-  if (source_callback_) {
-    source_callback_->OnError(stream, code);
-  }
+void AlsaPcmOutputStream::RunErrorCallback(int code) {
+  if (source_callback_)
+    source_callback_->OnError(this, code);
 }
 
 // Changes the AudioSourceCallback to proxy calls to.  Pass in NULL to
 // release ownership of the currently registered callback.
-void AlsaPcmOutputStream::SharedData::set_source_callback(
-    AudioSourceCallback* callback) {
-  DCHECK_EQ(MessageLoop::current(), state_transition_loop_);
-  base::AutoLock l(lock_);
+void AlsaPcmOutputStream::set_source_callback(AudioSourceCallback* callback) {
+  DCHECK(manager_->GetMessageLoop()->BelongsToCurrentThread());
   source_callback_ = callback;
 }
