@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,11 +6,9 @@
 #define MEDIA_AUDIO_MAC_AUDIO_MANAGER_MAC_H_
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "media/audio/audio_manager_base.h"
 
-class PCMQueueInAudioInputStream;
-class PCMQueueOutAudioOutputStream;
-
 // Mac OS X implementation of the AudioManager singleton. This class is internal
 // to the audio output and only internal users can call methods not exposed by
 // the AudioManager class.
@@ -19,21 +17,27 @@ class AudioManagerMac : public AudioManagerBase {
   AudioManagerMac();
 
   // Implementation of AudioManager.
-  virtual bool HasAudioOutputDevices();
-  virtual bool HasAudioInputDevices();
-  virtual AudioOutputStream* MakeAudioOutputStream(AudioParameters params);
-  virtual AudioInputStream* MakeAudioInputStream(AudioParameters params);
-  virtual void MuteAll();
-  virtual void UnMuteAll();
+  virtual bool HasAudioOutputDevices() OVERRIDE;
+  virtual bool HasAudioInputDevices() OVERRIDE;
+  virtual void GetAudioInputDeviceNames(media::AudioDeviceNames* device_names)
+      OVERRIDE;
+  virtual AudioOutputStream* MakeAudioOutputStream(
+      const AudioParameters& params) OVERRIDE;
+  virtual AudioInputStream* MakeAudioInputStream(
+      const AudioParameters& params, const std::string& device_id) OVERRIDE;
+  virtual void MuteAll() OVERRIDE;
+  virtual void UnMuteAll() OVERRIDE;
 
   // Mac-only method to free the streams created by above facoty methods.
   // They are called internally by the respective audio stream when it has
   // been closed.
-  void ReleaseOutputStream(PCMQueueOutAudioOutputStream* stream);
-  void ReleaseInputStream(PCMQueueInAudioInputStream* stream);
+  void ReleaseOutputStream(AudioOutputStream* stream);
+  void ReleaseInputStream(AudioInputStream* stream);
+
+ protected:
+  virtual ~AudioManagerMac();
 
  private:
-  virtual ~AudioManagerMac();
 
   // Number of currently open output streams.
   size_t num_output_streams_;
