@@ -1,8 +1,9 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/basictypes.h"
+#include "base/memory/scoped_ptr.h"
 #include "media/audio/audio_io.h"
 #include "media/audio/audio_manager.h"
 #include "media/audio/simple_sources.h"
@@ -47,29 +48,27 @@ TEST(MacAudioTest, SineWaveAudio16MonoTest) {
 // ===========================================================================
 // Validation of AudioParameters::AUDIO_PCM_LINEAR
 //
-// Unlike windows, the tests can reliably detect the existense of real
-// audio devices on the bots thus no need for 'headless' detection.
 
 // Test that can it be created and closed.
 TEST(MacAudioTest, PCMWaveStreamGetAndClose) {
-  AudioManager* audio_man = AudioManager::GetAudioManager();
-  ASSERT_TRUE(NULL != audio_man);
+  scoped_refptr<AudioManager> audio_man(AudioManager::Create());
   if (!audio_man->HasAudioOutputDevices())
     return;
   AudioOutputStream* oas = audio_man->MakeAudioOutputStream(
-      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, 2, 8000, 16, 1024));
+      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, CHANNEL_LAYOUT_STEREO,
+                      8000, 16, 1024));
   ASSERT_TRUE(NULL != oas);
   oas->Close();
 }
 
 // Test that it can be opened and closed.
 TEST(MacAudioTest, PCMWaveStreamOpenAndClose) {
-  AudioManager* audio_man = AudioManager::GetAudioManager();
-  ASSERT_TRUE(NULL != audio_man);
+  scoped_refptr<AudioManager> audio_man(AudioManager::Create());
   if (!audio_man->HasAudioOutputDevices())
     return;
   AudioOutputStream* oas = audio_man->MakeAudioOutputStream(
-      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, 2, 8000, 16, 1024));
+      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, CHANNEL_LAYOUT_STEREO,
+                      8000, 16, 1024));
   ASSERT_TRUE(NULL != oas);
   EXPECT_TRUE(oas->Open());
   oas->Close();
@@ -80,13 +79,12 @@ TEST(MacAudioTest, PCMWaveStreamOpenAndClose) {
 // pops or noises while the sound is playing. The sound must also be identical
 // to the sound of PCMWaveStreamPlay200HzTone22KssMono test.
 TEST(MacAudioTest, PCMWaveStreamPlay200HzTone44KssMono) {
-  AudioManager* audio_man = AudioManager::GetAudioManager();
-  ASSERT_TRUE(NULL != audio_man);
+  scoped_refptr<AudioManager> audio_man(AudioManager::Create());
   if (!audio_man->HasAudioOutputDevices())
     return;
   uint32 frames_100_ms = AudioParameters::kAudioCDSampleRate / 10;
   AudioOutputStream* oas = audio_man->MakeAudioOutputStream(
-      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, 1,
+      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, CHANNEL_LAYOUT_MONO,
                       AudioParameters::kAudioCDSampleRate, 16, frames_100_ms));
   ASSERT_TRUE(NULL != oas);
   EXPECT_TRUE(oas->Open());
@@ -111,13 +109,12 @@ TEST(MacAudioTest, PCMWaveStreamPlay200HzTone44KssMono) {
 // or noises while the sound is playing. The sound must also be identical to the
 // sound of PCMWaveStreamPlay200HzTone44KssMono test.
 TEST(MacAudioTest, PCMWaveStreamPlay200HzTone22KssMono) {
-  AudioManager* audio_man = AudioManager::GetAudioManager();
-  ASSERT_TRUE(NULL != audio_man);
+  scoped_refptr<AudioManager> audio_man(AudioManager::Create());
   if (!audio_man->HasAudioOutputDevices())
     return;
   uint32 frames_100_ms = AudioParameters::kAudioCDSampleRate / 10;
   AudioOutputStream* oas = audio_man->MakeAudioOutputStream(
-      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, 1,
+      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, CHANNEL_LAYOUT_MONO,
                       AudioParameters::kAudioCDSampleRate / 2, 16,
                       frames_100_ms));
   ASSERT_TRUE(NULL != oas);
@@ -138,14 +135,13 @@ static void ClearBuffer(AudioOutputStream* stream, uint8* dest,
 }
 
 TEST(MacAudioTest, PCMWaveStreamPendingBytes) {
-  AudioManager* audio_man = AudioManager::GetAudioManager();
-  ASSERT_TRUE(NULL != audio_man);
+  scoped_refptr<AudioManager> audio_man(AudioManager::Create());
   if (!audio_man->HasAudioOutputDevices())
     return;
 
   uint32 frames_100_ms = AudioParameters::kAudioCDSampleRate / 10;
   AudioOutputStream* oas = audio_man->MakeAudioOutputStream(
-      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, 1,
+      AudioParameters(AudioParameters::AUDIO_PCM_LINEAR, CHANNEL_LAYOUT_MONO,
                       AudioParameters::kAudioCDSampleRate, 16, frames_100_ms));
   ASSERT_TRUE(NULL != oas);
 
