@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,8 +8,11 @@
 namespace media {
 
 TestAudioInputController::TestAudioInputController(
-    TestAudioInputControllerFactory* factory, EventHandler* event_handler)
-    : AudioInputController(event_handler),
+    TestAudioInputControllerFactory* factory,
+    AudioManager* audio_manager,
+    EventHandler* event_handler,
+    SyncWriter* sync_writer)
+    : AudioInputController(audio_manager, event_handler, sync_writer),
       factory_(factory),
       event_handler_(event_handler) {
 }
@@ -24,16 +27,18 @@ TestAudioInputControllerFactory::TestAudioInputControllerFactory()
 }
 
 AudioInputController* TestAudioInputControllerFactory::Create(
+    AudioManager* audio_manager,
     AudioInputController::EventHandler* event_handler,
     AudioParameters params) {
   DCHECK(!controller_);  // Only one test instance managed at a time.
-  controller_ = new TestAudioInputController(this, event_handler);
+  controller_ = new TestAudioInputController(this, audio_manager,
+      event_handler, NULL);
   return controller_;
 }
 
 void TestAudioInputControllerFactory::OnTestAudioInputControllerDestroyed(
     TestAudioInputController* controller) {
-  DCHECK(controller_ == controller);
+  DCHECK_EQ(controller_, controller);
   controller_ = NULL;
 }
 
