@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -45,7 +45,9 @@ class TestAudioInputControllerFactory;
 class TestAudioInputController : public AudioInputController {
  public:
   TestAudioInputController(TestAudioInputControllerFactory* factory,
-                           EventHandler* event_handler);
+                           AudioManager* audio_manager,
+                           EventHandler* event_handler,
+                           SyncWriter* sync_writer);
   virtual ~TestAudioInputController();
 
   // Returns the event handler installed on the AudioInputController.
@@ -53,8 +55,8 @@ class TestAudioInputController : public AudioInputController {
 
   // Overriden to do nothing. It is assumed the caller will notify the event
   // handler with recorded data and other events.
-  virtual void Record() {}
-  virtual void Close() {}
+  virtual void Record() OVERRIDE {}
+  virtual void Close() OVERRIDE {}
 
  private:
   // These are not owned by us and expected to be valid for this object's
@@ -72,9 +74,10 @@ class TestAudioInputControllerFactory : public AudioInputController::Factory {
   TestAudioInputControllerFactory();
 
   // AudioInputController::Factory methods.
-  AudioInputController* Create(
+  virtual AudioInputController* Create(
+      AudioManager* audio_manager,
       AudioInputController::EventHandler* event_handler,
-      AudioParameters params);
+      AudioParameters params) OVERRIDE;
 
   TestAudioInputController* controller() const { return controller_; }
 
