@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,12 +11,20 @@
 #include <setupapi.h>
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/command_line.h"
+#include "base/file_path.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/path_service.h"
+#include "base/process_util.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
+#include "media/audio/audio_util.h"
 #include "media/audio/fake_audio_input_stream.h"
 #include "media/audio/fake_audio_output_stream.h"
+#include "media/audio/win/audio_low_latency_input_win.h"
+#include "media/audio/win/audio_low_latency_output_win.h"
 #include "media/audio/win/audio_manager_win.h"
+#include "media/audio/win/device_enumeration_win.h"
 #include "media/audio/win/wavein_input_win.h"
 #include "media/audio/win/waveout_output_win.h"
 #include "media/base/limits.h"
@@ -32,32 +40,30 @@
 DEFINE_GUID(AM_KSCATEGORY_AUDIO, 0x6994ad04, 0x93ef, 0x11d0,
             0xa3, 0xcc, 0x00, 0xa0, 0xc9, 0x22, 0x31, 0x96);
 
-namespace {
-
 // Maximum number of output streams that can be open simultaneously.
-const size_t kMaxOutputStreams = 50;
+static const size_t kMaxOutputStreams = 50;
 
 // Up to 8 channels can be passed to the driver.
 // This should work, given the right drivers, but graceful error handling is
 // needed.
-const int kWinMaxChannels = 8;
+static const int kWinMaxChannels = 8;
 
-const int kWinMaxInputChannels = 2;
+static const int kWinMaxInputChannels = 2;
 // We use 3 buffers for recording audio so that if a recording callback takes
 // some time to return we won't lose audio. More buffers while recording are
 // ok because they don't introduce any delay in recording, unlike in playback
 // where you first need to fill in that number of buffers before starting to
 // play.
-const int kNumInputBuffers = 3;
+static const int kNumInputBuffers = 3;
 
-int GetVersionPartAsInt(DWORDLONG num) {
+static int GetVersionPartAsInt(DWORDLONG num) {
   return static_cast<int>(num & 0xffff);
 }
 
 // Returns a string containing the given device's description and installed
 // driver version.
-string16 GetDeviceAndDriverInfo(HDEVINFO device_info,
-                                SP_DEVINFO_DATA* device_data) {
+static string16 GetDeviceAndDriverInfo(HDEVINFO device_info,
+                                       SP_DEVINFO_DATA* device_data) {
   // Save the old install params setting and set a flag for the
   // SetupDiBuildDriverInfoList below to return only the installed drivers.
   SP_DEVINSTALL_PARAMS old_device_install_params;
@@ -92,13 +98,20 @@ string16 GetDeviceAndDriverInfo(HDEVINFO device_info,
   return device_and_driver_info;
 }
 
-}  // namespace
-
 AudioManagerWin::AudioManagerWin()
     : num_output_streams_(0) {
+  if (!media::IsWASAPISupported()) {
+    // Use the Wave API for device enumeration if XP or lower.
+    enumeration_type_ = kWaveEnumeration;
+  } else {
+    // Use the MMDevice API for device enumeration if Vista or higher.
+    enumeration_type_ = kMMDeviceEnumeration;
+  }
 }
 
 AudioManagerWin::~AudioManagerWin() {
+  // All output streams should be released upon termination.
+  DCHECK_EQ(0, num_output_streams_);
 }
 
 bool AudioManagerWin::HasAudioOutputDevices() {
@@ -111,10 +124,10 @@ bool AudioManagerWin::HasAudioInputDevices() {
 
 // Factory for the implementations of AudioOutputStream. Two implementations
 // should suffice most windows user's needs.
-// - PCMWaveOutAudioOutputStream: Based on the waveOutWrite API (in progress)
-// - PCMDXSoundAudioOutputStream: Based on DirectSound or XAudio (future work).
+// - PCMWaveOutAudioOutputStream: Based on the waveOut API.
+// - WASAPIAudioOutputStream: Based on Core Audio (WASAPI) API.
 AudioOutputStream* AudioManagerWin::MakeAudioOutputStream(
-    AudioParameters params) {
+    const AudioParameters& params) {
   if (!params.IsValid() || (params.channels > kWinMaxChannels))
     return NULL;
 
@@ -130,34 +143,51 @@ AudioOutputStream* AudioManagerWin::MakeAudioOutputStream(
     return new PCMWaveOutAudioOutputStream(this, params, 3, WAVE_MAPPER);
   } else if (params.format == AudioParameters::AUDIO_PCM_LOW_LATENCY) {
     num_output_streams_++;
-    // TODO(cpu): waveout cannot hit 20ms latency. Use other method.
-    return new PCMWaveOutAudioOutputStream(this, params, 2, WAVE_MAPPER);
+    if (!media::IsWASAPISupported()) {
+      // Fall back to Windows Wave implementation on Windows XP or lower.
+      DLOG(INFO) << "Using WaveOut since WASAPI requires at least Vista.";
+      return new PCMWaveOutAudioOutputStream(this, params, 2, WAVE_MAPPER);
+    } else {
+      // TODO(henrika): improve possibility to specify audio endpoint.
+      // Use the default device (same as for Wave) for now to be compatible.
+      return new WASAPIAudioOutputStream(this, params, eConsole);
+    }
   }
   return NULL;
 }
 
 // Factory for the implementations of AudioInputStream.
 AudioInputStream* AudioManagerWin::MakeAudioInputStream(
-    AudioParameters params) {
-  if (!params.IsValid() || (params.channels > kWinMaxInputChannels))
+    const AudioParameters& params, const std::string& device_id) {
+  if (!params.IsValid() || (params.channels > kWinMaxInputChannels) ||
+      device_id.empty())
     return NULL;
 
   if (params.format == AudioParameters::AUDIO_MOCK) {
     return FakeAudioInputStream::MakeFakeStream(params);
   } else if (params.format == AudioParameters::AUDIO_PCM_LINEAR) {
     return new PCMWaveInAudioInputStream(this, params, kNumInputBuffers,
-                                         WAVE_MAPPER);
+                                         AudioManagerBase::kDefaultDeviceId);
+  } else if (params.format == AudioParameters::AUDIO_PCM_LOW_LATENCY) {
+    if (!media::IsWASAPISupported()) {
+      // Fall back to Windows Wave implementation on Windows XP or lower.
+      DLOG(INFO) << "Using WaveIn since WASAPI requires at least Vista.";
+      return new PCMWaveInAudioInputStream(this, params, kNumInputBuffers,
+                                           device_id);
+    } else {
+      return new WASAPIAudioInputStream(this, params, device_id);
+    }
   }
   return NULL;
 }
 
-void AudioManagerWin::ReleaseOutputStream(PCMWaveOutAudioOutputStream* stream) {
+void AudioManagerWin::ReleaseOutputStream(AudioOutputStream* stream) {
   DCHECK(stream);
   num_output_streams_--;
   delete stream;
 }
 
-void AudioManagerWin::ReleaseInputStream(PCMWaveInAudioInputStream* stream) {
+void AudioManagerWin::ReleaseInputStream(AudioInputStream* stream) {
   delete stream;
 }
 
@@ -177,12 +207,14 @@ string16 AudioManagerWin::GetAudioInputDeviceModel() {
   waveInMessage(reinterpret_cast<HWAVEIN>(device_id),
                 DRV_QUERYDEVICEINTERFACESIZE,
                 reinterpret_cast<DWORD_PTR>(&device_interface_name_size), 0);
-  if (device_interface_name_size == 0)  // No audio capture device?
-    return string16();
+  size_t bytes_in_char16 = sizeof(string16::value_type);
+  DCHECK_EQ(0u, device_interface_name_size % bytes_in_char16);
+  if (device_interface_name_size <= bytes_in_char16)
+    return string16();  // No audio capture device.
 
   string16 device_interface_name;
   string16::value_type* name_ptr = WriteInto(&device_interface_name,
-      device_interface_name_size / sizeof(string16::value_type));
+      device_interface_name_size / bytes_in_char16);
   waveInMessage(reinterpret_cast<HWAVEIN>(device_id),
                 DRV_QUERYDEVICEINTERFACE,
                 reinterpret_cast<DWORD_PTR>(name_ptr),
@@ -229,7 +261,51 @@ string16 AudioManagerWin::GetAudioInputDeviceModel() {
   return string16();
 }
 
-// static
-AudioManager* AudioManager::CreateAudioManager() {
+bool AudioManagerWin::CanShowAudioInputSettings() {
+  return true;
+}
+
+void AudioManagerWin::ShowAudioInputSettings() {
+  std::wstring program;
+  std::string argument;
+  if (!media::IsWASAPISupported()) {
+    program = L"sndvol32.exe";
+    argument = "-R";
+  } else {
+    program = L"control.exe";
+    argument = "mmsys.cpl,,1";
+  }
+
+  FilePath path;
+  PathService::Get(base::DIR_SYSTEM, &path);
+  path = path.Append(program);
+  CommandLine command_line(path);
+  command_line.AppendArg(argument);
+  base::LaunchProcess(command_line, base::LaunchOptions(), NULL);
+}
+
+void AudioManagerWin::GetAudioInputDeviceNames(
+    media::AudioDeviceNames* device_names) {
+  DCHECK(enumeration_type() !=  kUninitializedEnumeration);
+  // Enumerate all active audio-endpoint capture devices.
+  if (enumeration_type() == kWaveEnumeration) {
+    // Utilize the Wave API for Windows XP.
+    GetInputDeviceNamesWinXP(device_names);
+  } else {
+    // Utilize the MMDevice API (part of Core Audio) for Vista and higher.
+    GetInputDeviceNamesWin(device_names);
+  }
+
+  // Always add default device parameters as first element.
+  if (!device_names->empty()) {
+    media::AudioDeviceName name;
+    name.device_name = AudioManagerBase::kDefaultDeviceName;
+    name.unique_id = AudioManagerBase::kDefaultDeviceId;
+    device_names->push_front(name);
+  }
+}
+
+/// static
+AudioManager* CreateAudioManager() {
   return new AudioManagerWin();
 }
