@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,25 +8,27 @@
 #include <mmsystem.h>
 #pragma comment(lib, "winmm.lib")
 
-#include "base/basictypes.h"
 #include "base/logging.h"
 #include "media/audio/audio_io.h"
 #include "media/audio/audio_util.h"
 #include "media/audio/win/audio_manager_win.h"
+#include "media/audio/win/device_enumeration_win.h"
 
 namespace {
+const int kStopInputStreamCallbackTimeout = 3000; // Three seconds.
+}
+
+using media::AudioDeviceNames;
 
 // Our sound buffers are allocated once and kept in a linked list using the
 // the WAVEHDR::dwUser variable. The last buffer points to the first buffer.
-WAVEHDR* GetNextBuffer(WAVEHDR* current) {
+static WAVEHDR* GetNextBuffer(WAVEHDR* current) {
   return reinterpret_cast<WAVEHDR*>(current->dwUser);
 }
 
-}  // namespace
-
 PCMWaveInAudioInputStream::PCMWaveInAudioInputStream(
-    AudioManagerWin* manager, AudioParameters params, int num_buffers,
-    UINT device_id)
+    AudioManagerWin* manager, const AudioParameters& params, int num_buffers,
+    const std::string& device_id)
     : state_(kStateEmpty),
       manager_(manager),
       device_id_(device_id),
@@ -59,10 +61,20 @@ bool PCMWaveInAudioInputStream::Open() {
     return false;
   if (num_buffers_ < 2 || num_buffers_ > 10)
     return false;
-  MMRESULT result = ::waveInOpen(&wavein_, device_id_, &format_,
-                                 reinterpret_cast<DWORD_PTR>(WaveCallback),
-                                 reinterpret_cast<DWORD_PTR>(this),
-                                 CALLBACK_FUNCTION);
+
+  // Convert the stored device id string into an unsigned integer
+  // corresponding to the selected device.
+  UINT device_id = WAVE_MAPPER;
+  if (!GetDeviceId(&device_id)) {
+    return false;
+  }
+
+  // Open the specified input device for recording.
+  MMRESULT result = MMSYSERR_NOERROR;
+  result = ::waveInOpen(&wavein_, device_id, &format_,
+                        reinterpret_cast<DWORD_PTR>(WaveCallback),
+                        reinterpret_cast<DWORD_PTR>(this),
+                        CALLBACK_FUNCTION);
   if (result != MMSYSERR_NOERROR)
     return false;
 
@@ -122,6 +134,8 @@ void PCMWaveInAudioInputStream::Start(AudioInputCallback* callback) {
   if (result != MMSYSERR_NOERROR) {
     HandleError(result);
     state_ = kStateReady;
+  } else {
+    manager_->IncreaseActiveInputStreamCount();
   }
 }
 
@@ -134,10 +148,15 @@ void PCMWaveInAudioInputStream::Stop() {
     return;
   state_ = kStateStopping;
   // Wait for the callback to finish, it will signal us when ready to be reset.
-  if (WAIT_OBJECT_0 != ::WaitForSingleObject(stopped_event_, INFINITE)) {
+  if (WAIT_OBJECT_0 !=
+      ::WaitForSingleObject(stopped_event_, kStopInputStreamCallbackTimeout)) {
     HandleError(::GetLastError());
     return;
   }
+  // Stop is always called before Close. In case of error, this will be
+  // also called when closing the input controller.
+  manager_->DecreaseActiveInputStreamCount();
+
   state_ = kStateStopped;
   MMRESULT res = ::waveInReset(wavein_);
   if (res != MMSYSERR_NOERROR) {
@@ -179,6 +198,41 @@ void PCMWaveInAudioInputStream::QueueNextPacket(WAVEHDR *buffer) {
     HandleError(res);
 }
 
+bool PCMWaveInAudioInputStream::GetDeviceId(UINT* device_index) {
+  // Deliver the default input device id (WAVE_MAPPER) if the default
+  // device has been selected.
+  if (device_id_ == AudioManagerBase::kDefaultDeviceId) {
+    *device_index = WAVE_MAPPER;
+    return true;
+  }
+
+  // Get list of all available and active devices.
+  AudioDeviceNames device_names;
+  if (!GetInputDeviceNamesWinXP(&device_names))
+    return false;
+
+  if (device_names.empty())
+    return false;
+
+  // Search the full list of devices and compare with the specified
+  // device id which was specified in the constructor. Stop comparing
+  // when a match is found and return the corresponding index.
+  UINT index = 0;
+  bool found_device = false;
+  AudioDeviceNames::const_iterator it = device_names.begin();
+  while (it != device_names.end()) {
+    if (it->unique_id.compare(device_id_) == 0) {
+      *device_index = index;
+      found_device = true;
+      break;
+    }
+    ++index;
+    ++it;
+  }
+
+  return found_device;
+}
+
 // Windows calls us back in this function when some events happen. Most notably
 // when it has an audio buffer with recorded data.
 void PCMWaveInAudioInputStream::WaveCallback(HWAVEIN hwi, UINT msg,
@@ -190,8 +244,11 @@ void PCMWaveInAudioInputStream::WaveCallback(HWAVEIN hwi, UINT msg,
   if (msg == WIM_DATA) {
     // WIM_DONE indicates that the driver is done with our buffer. We pass it
     // to the callback and check if we need to stop playing.
+    // It should be OK to assume the data in the buffer is what has been
+    // recorded in the soundcard.
     WAVEHDR* buffer = reinterpret_cast<WAVEHDR*>(param1);
     obj->callback_->OnData(obj, reinterpret_cast<const uint8*>(buffer->lpData),
+                           buffer->dwBytesRecorded,
                            buffer->dwBytesRecorded);
 
     if (obj->state_ == kStateStopping) {
