@@ -2,6 +2,7 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/bind.h"
 #include "base/message_loop.h"
 #include "media/base/composite_filter.h"
 #include "media/base/mock_callback.h"
@@ -53,9 +54,15 @@ class CompositeFilterTest : public testing::Test {
   // |seek_time| - The time to pass to the Seek() call if |method_to_call|
   //               equals SEEK.
   // |callback| - The callback object to pass to the method.
+  // |expected_status| - Some filter methods use a FilterStatusCB instead of
+  //                     a Closure. For these methods this function
+  //                     creates a FilterStatusCB that makes sure the status
+  //                     passed to the callback matches |expected_status| and
+  //                     then calls |callback|.
   void DoFilterCall(MethodToCall method_to_call, Filter* filter,
                     base::TimeDelta seek_time,
-                    FilterCallback* callback);
+                    const base::Closure& callback,
+                    PipelineStatus expected_status);
 
   // Creates an expectation sequence based on the value of method_to_call.
   //
@@ -82,10 +89,16 @@ class CompositeFilterTest : public testing::Test {
   void ExpectInvalidStateFail(MethodToCall method_to_call,
                   base::TimeDelta seek_time = base::TimeDelta());
 
-  // Run the callback stored in |filter_1_callback_|.
+  // Returns whether |filter_1_callback_| or |filter_1_status_cb_| is set.
+  bool HasFilter1Callback() const;
+
+  // Run the callback stored in |filter_1_callback_| or |filter_2_status_cb_|.
   void RunFilter1Callback();
 
-  // Run the callback stored in |filter_2_callback_|.
+  // Returns whether |filter_2_callback_| or |filter_2_status_cb_| is set.
+  bool HasFilter2Callback() const;
+
+  // Run the callback stored in |filter_2_callback_| or |filter_2_status_cb_|.
   void RunFilter2Callback();
 
  protected:
@@ -98,15 +111,27 @@ class CompositeFilterTest : public testing::Test {
   scoped_refptr<StrictMock<MockFilter> > filter_1_;
 
   // Callback passed to |filter_1_| during last Play(), Pause(), Flush(),
-  // Stop(), or Seek() call.
-  FilterCallback* filter_1_callback_;
+  // or Stop() call.
+  base::Closure filter_1_callback_;
+
+  // Status to pass to |filter_1_status_cb_|.
+  PipelineStatus filter_1_status_;
+
+  // Callback passed to |filter_1_| during last Seek() call.
+  FilterStatusCB filter_1_status_cb_;
 
   // Second filter added to the composite.
   scoped_refptr<StrictMock<MockFilter> > filter_2_;
 
   // Callback passed to |filter_2_| during last Play(), Pause(), Flush(),
   // Stop(), or Seek() call.
-  FilterCallback* filter_2_callback_;
+  base::Closure filter_2_callback_;
+
+  // Status to pass to |filter_2_status_cb_|.
+  PipelineStatus filter_2_status_;
+
+  // Callback passed to |filter_2_| during last Seek() call.
+  FilterStatusCB filter_2_status_cb_;
 
   // FilterHost implementation passed to |composite_| via set_host().
   scoped_ptr<StrictMock<MockFilterHost> > mock_filter_host_;
@@ -116,8 +141,8 @@ class CompositeFilterTest : public testing::Test {
 
 CompositeFilterTest::CompositeFilterTest() :
     composite_(new CompositeFilter(&message_loop_)),
-    filter_1_callback_(NULL),
-    filter_2_callback_(NULL),
+    filter_1_status_(PIPELINE_OK),
+    filter_2_status_(PIPELINE_OK),
     mock_filter_host_(new StrictMock<MockFilterHost>()) {
 }
 
@@ -131,7 +156,9 @@ void CompositeFilterTest::SetupAndAdd2Filters() {
   // Setup |filter_1_| and arrange for methods to set
   // |filter_1_callback_| when they are called.
   filter_1_ = new StrictMock<MockFilter>();
-  filter_1_callback_ = NULL;
+  filter_1_callback_.Reset();
+  filter_1_status_ = PIPELINE_OK;
+  filter_1_status_cb_.Reset();
   ON_CALL(*filter_1_, Play(_))
       .WillByDefault(SaveArg<0>(&filter_1_callback_));
   ON_CALL(*filter_1_, Pause(_))
@@ -141,12 +168,14 @@ void CompositeFilterTest::SetupAndAdd2Filters() {
   ON_CALL(*filter_1_, Stop(_))
       .WillByDefault(SaveArg<0>(&filter_1_callback_));
   ON_CALL(*filter_1_, Seek(_,_))
-      .WillByDefault(SaveArg<1>(&filter_1_callback_));
+      .WillByDefault(SaveArg<1>(&filter_1_status_cb_));
 
   // Setup |filter_2_| and arrange for methods to set
   // |filter_2_callback_| when they are called.
   filter_2_ = new StrictMock<MockFilter>();
-  filter_2_callback_ = NULL;
+  filter_2_callback_.Reset();
+  filter_2_status_ = PIPELINE_OK;
+  filter_2_status_cb_.Reset();
   ON_CALL(*filter_2_, Play(_))
       .WillByDefault(SaveArg<0>(&filter_2_callback_));
   ON_CALL(*filter_2_, Pause(_))
@@ -156,7 +185,7 @@ void CompositeFilterTest::SetupAndAdd2Filters() {
   ON_CALL(*filter_2_, Stop(_))
       .WillByDefault(SaveArg<0>(&filter_2_callback_));
   ON_CALL(*filter_2_, Seek(_,_))
-      .WillByDefault(SaveArg<1>(&filter_2_callback_));
+      .WillByDefault(SaveArg<1>(&filter_2_status_cb_));
 
   composite_->AddFilter(filter_1_);
   composite_->AddFilter(filter_2_);
@@ -184,10 +213,20 @@ void CompositeFilterTest::ExpectFilterCall(MethodToCall method_to_call,
   };
 }
 
+void OnStatusCB(PipelineStatus expected_status, const base::Closure& callback,
+                PipelineStatus status) {
+  EXPECT_EQ(status, expected_status);
+  callback.Run();
+}
+
 void CompositeFilterTest::DoFilterCall(MethodToCall method_to_call,
                                        Filter* filter,
                                        base::TimeDelta seek_time,
-                                       FilterCallback* callback) {
+                                       const base::Closure& callback,
+                                       PipelineStatus expected_status) {
+  filter_1_status_ = expected_status;
+  filter_2_status_ = expected_status;
+
   switch(method_to_call) {
     case PLAY:
       filter->Play(callback);
@@ -202,7 +241,8 @@ void CompositeFilterTest::DoFilterCall(MethodToCall method_to_call,
       filter->Stop(callback);
       break;
     case SEEK:
-      filter->Seek(seek_time, callback);
+      filter->Seek(seek_time, base::Bind(&OnStatusCB, expected_status,
+                                         callback));
       break;
   };
 }
@@ -220,29 +260,31 @@ void CompositeFilterTest::ExpectSuccess(MethodToCall method_to_call,
   }
 
   // Make method call on the composite.
-  StrictMock<MockCallback>* callback = new StrictMock<MockCallback>();
-  DoFilterCall(method_to_call, composite_.get(), seek_time, callback);
+  StrictMock<MockClosure>* callback = new StrictMock<MockClosure>();
+  DoFilterCall(method_to_call, composite_.get(), seek_time,
+               base::Bind(&MockClosure::Run, callback),
+               PIPELINE_OK);
 
   if (is_parallel_call) {
     // Make sure both filters have their callbacks set.
-    EXPECT_TRUE(filter_1_callback_ != NULL);
-    EXPECT_TRUE(filter_2_callback_ != NULL);
+    EXPECT_TRUE(HasFilter1Callback());
+    EXPECT_TRUE(HasFilter2Callback());
 
     RunFilter1Callback();
   } else {
     // Make sure that only |filter_1_| has its callback set.
-    EXPECT_TRUE(filter_1_callback_ != NULL);
-    EXPECT_EQ((FilterCallback*)NULL, filter_2_callback_);
+    EXPECT_TRUE(HasFilter1Callback());
+    EXPECT_FALSE(HasFilter2Callback());
 
     ExpectFilterCall(method_to_call, filter_2_.get(), seek_time);
 
     RunFilter1Callback();
 
     // Verify that |filter_2_| was called by checking the callback pointer.
-    EXPECT_TRUE(filter_2_callback_ != NULL);
+    EXPECT_TRUE(HasFilter2Callback());
   }
 
-  callback->ExpectRunAndDelete();
+  EXPECT_CALL(*callback, Run());
 
   RunFilter2Callback();
 }
@@ -271,31 +313,58 @@ void CompositeFilterTest::ExpectInvalidStateFail(MethodToCall method_to_call,
                                      base::TimeDelta seek_time) {
   InSequence seq;
 
-  EXPECT_CALL(*mock_filter_host_, SetError(PIPELINE_ERROR_INVALID_STATE))
-      .WillOnce(Return());
+  if (method_to_call != SEEK) {
+    EXPECT_CALL(*mock_filter_host_, SetError(PIPELINE_ERROR_INVALID_STATE))
+        .WillOnce(Return());
+  }
 
-  DoFilterCall(method_to_call, composite_, seek_time, NewExpectedCallback());
+  DoFilterCall(method_to_call, composite_, seek_time, NewExpectedClosure(),
+               PIPELINE_ERROR_INVALID_STATE);
 
   // Make sure that neither of the filters were called by
   // verifying that the callback pointers weren't set.
-  EXPECT_EQ((FilterCallback*)NULL, filter_1_callback_);
-  EXPECT_EQ((FilterCallback*)NULL, filter_2_callback_);
+  EXPECT_FALSE(HasFilter1Callback());
+  EXPECT_FALSE(HasFilter2Callback());
+}
+
+bool CompositeFilterTest::HasFilter1Callback() const {
+  CHECK(filter_1_callback_.is_null() || filter_1_status_cb_.is_null());
+  return !filter_1_callback_.is_null() || !filter_1_status_cb_.is_null();
 }
 
 void CompositeFilterTest::RunFilter1Callback() {
-  EXPECT_TRUE(filter_1_callback_ != NULL);
-  FilterCallback* callback = filter_1_callback_;
-  filter_1_callback_ = NULL;
-  callback->Run();
-  delete callback;
+  EXPECT_TRUE(HasFilter1Callback());
+
+  if (!filter_1_status_cb_.is_null()) {
+    ResetAndRunCB(&filter_1_status_cb_, filter_1_status_);
+    filter_1_status_ = PIPELINE_OK;
+    return;
+  }
+
+  EXPECT_TRUE(!filter_1_callback_.is_null());
+  base::Closure callback = filter_1_callback_;
+  filter_1_callback_.Reset();
+  callback.Run();
+}
+
+bool CompositeFilterTest::HasFilter2Callback() const {
+  CHECK(filter_2_callback_.is_null() || filter_2_status_cb_.is_null());
+  return !filter_2_callback_.is_null() || !filter_2_status_cb_.is_null();
 }
 
 void CompositeFilterTest::RunFilter2Callback() {
-  EXPECT_TRUE(filter_2_callback_ != NULL);
-  FilterCallback* callback = filter_2_callback_;
-  filter_2_callback_ = NULL;
-  callback->Run();
-  delete callback;
+  EXPECT_TRUE(HasFilter2Callback());
+
+  if (!filter_2_status_cb_.is_null()) {
+    ResetAndRunCB(&filter_2_status_cb_, filter_2_status_);
+    filter_2_status_ = PIPELINE_OK;
+    return;
+  }
+
+  EXPECT_FALSE(filter_2_callback_.is_null());
+  base::Closure callback = filter_2_callback_;
+  filter_2_callback_.Reset();
+  callback.Run();
 }
 
 // Test AddFilter() failure cases.
@@ -310,8 +379,8 @@ TEST_F(CompositeFilterTest, TestAddFilterFailCases) {
   EXPECT_FALSE(composite_->AddFilter(filter));
 }
 
-// Test successful AddFilter() cases.
-TEST_F(CompositeFilterTest, TestAddFilter) {
+// Test successful {Add,Remove}Filter() cases.
+TEST_F(CompositeFilterTest, TestAddRemoveFilter) {
   composite_->set_host(mock_filter_host_.get());
 
   // Add a filter.
@@ -319,8 +388,19 @@ TEST_F(CompositeFilterTest, TestAddFilter) {
   EXPECT_EQ(NULL, filter->host());
 
   EXPECT_TRUE(composite_->AddFilter(filter));
-
   EXPECT_TRUE(filter->host() != NULL);
+  composite_->RemoveFilter(filter);
+  EXPECT_TRUE(filter->host() == NULL);
+}
+
+class CompositeFilterDeathTest : public CompositeFilterTest {};
+
+// Test failure of RemoveFilter() on an unknown filter.
+TEST_F(CompositeFilterDeathTest, TestRemoveUnknownFilter) {
+  composite_->set_host(mock_filter_host_.get());
+  // Remove unknown filter.
+  scoped_refptr<StrictMock<MockFilter> > filter = new StrictMock<MockFilter>();
+  EXPECT_DEATH(composite_->RemoveFilter(filter), "");
 }
 
 TEST_F(CompositeFilterTest, TestPlay) {
@@ -336,11 +416,11 @@ TEST_F(CompositeFilterTest, TestPlay) {
   // Try calling Play() again to make sure that we simply get a callback.
   // We are already in the Play() state so there is no point calling the
   // filters.
-  composite_->Play(NewExpectedCallback());
+  composite_->Play(NewExpectedClosure());
 
   // Verify that neither of the filter callbacks were set.
-  EXPECT_EQ((FilterCallback*)NULL, filter_1_callback_);
-  EXPECT_EQ((FilterCallback*)NULL, filter_2_callback_);
+  EXPECT_FALSE(HasFilter1Callback());
+  EXPECT_FALSE(HasFilter2Callback());
 
   // Stop playback.
   DoStop();
@@ -348,11 +428,11 @@ TEST_F(CompositeFilterTest, TestPlay) {
   // At this point we should be in the kStopped state.
 
   // Try calling Stop() again to make sure neither filter is called.
-  composite_->Stop(NewExpectedCallback());
+  composite_->Stop(NewExpectedClosure());
 
   // Verify that neither of the filter callbacks were set.
-  EXPECT_EQ((FilterCallback*)NULL, filter_1_callback_);
-  EXPECT_EQ((FilterCallback*)NULL, filter_2_callback_);
+  EXPECT_FALSE(HasFilter1Callback());
+  EXPECT_FALSE(HasFilter2Callback());
 
   // Try calling Play() again to make sure we get an error.
   ExpectInvalidStateFail(PLAY);
@@ -367,8 +447,8 @@ TEST_F(CompositeFilterTest, TestPlayErrors) {
   EXPECT_CALL(*filter_1_, Play(_));
 
   // Call Play() on the composite.
-  StrictMock<MockCallback>* callback = new StrictMock<MockCallback>();
-  composite_->Play(callback);
+  StrictMock<MockClosure>* callback = new StrictMock<MockClosure>();
+  composite_->Play(base::Bind(&MockClosure::Run, callback));
 
   EXPECT_CALL(*filter_2_, Play(_));
 
@@ -381,7 +461,7 @@ TEST_F(CompositeFilterTest, TestPlayErrors) {
 
   // Expect error to be reported and "play done" callback to be called.
   EXPECT_CALL(*mock_filter_host_, SetError(PIPELINE_ERROR_OUT_OF_MEMORY));
-  callback->ExpectRunAndDelete();
+  EXPECT_CALL(*callback, Run());
 
   // Run callback to indicate that |filter_2_|'s Play() has completed.
   RunFilter2Callback();
@@ -415,11 +495,11 @@ TEST_F(CompositeFilterTest, TestPause) {
 
   // Try calling Pause() again to make sure that the filters aren't called
   // because we are already in the paused state.
-  composite_->Pause(NewExpectedCallback());
+  composite_->Pause(NewExpectedClosure());
 
   // Verify that neither of the filter callbacks were set.
-  EXPECT_EQ((FilterCallback*)NULL, filter_1_callback_);
-  EXPECT_EQ((FilterCallback*)NULL, filter_2_callback_);
+  EXPECT_FALSE(HasFilter1Callback());
+  EXPECT_FALSE(HasFilter2Callback());
 
   // Verify that we can transition pack to the play state.
   DoPlay();
@@ -446,8 +526,8 @@ TEST_F(CompositeFilterTest, TestPauseErrors) {
   EXPECT_CALL(*filter_1_, Pause(_));
 
   // Call Pause() on the composite.
-  StrictMock<MockCallback>* callback = new StrictMock<MockCallback>();
-  composite_->Pause(callback);
+  StrictMock<MockClosure>* callback = new StrictMock<MockClosure>();
+  composite_->Pause(base::Bind(&MockClosure::Run, callback));
 
   // Simulate an error by calling SetError() on |filter_1_|'s FilterHost
   // interface.
@@ -455,12 +535,12 @@ TEST_F(CompositeFilterTest, TestPauseErrors) {
 
   // Expect error to be reported and "pause done" callback to be called.
   EXPECT_CALL(*mock_filter_host_, SetError(PIPELINE_ERROR_OUT_OF_MEMORY));
-  callback->ExpectRunAndDelete();
+  EXPECT_CALL(*callback, Run());
 
   RunFilter1Callback();
 
   // Make sure |filter_2_callback_| was not set.
-  EXPECT_EQ((FilterCallback*)NULL, filter_2_callback_);
+  EXPECT_FALSE(HasFilter2Callback());
 
   // Verify that Play/Pause/Flush/Seek fail now that an error occured.
   ExpectInvalidStateFail(PLAY);
@@ -513,8 +593,8 @@ TEST_F(CompositeFilterTest, TestFlushErrors) {
   EXPECT_CALL(*filter_2_, Flush(_));
 
   // Call Flush() on the composite.
-  StrictMock<MockCallback>* callback = new StrictMock<MockCallback>();
-  composite_->Flush(callback);
+  StrictMock<MockClosure>* callback = new StrictMock<MockClosure>();
+  composite_->Flush(base::Bind(&MockClosure::Run, callback));
 
   // Simulate an error by calling SetError() on |filter_1_|'s FilterHost
   // interface.
@@ -524,7 +604,7 @@ TEST_F(CompositeFilterTest, TestFlushErrors) {
 
   // Expect error to be reported and "pause done" callback to be called.
   EXPECT_CALL(*mock_filter_host_, SetError(PIPELINE_ERROR_OUT_OF_MEMORY));
-  callback->ExpectRunAndDelete();
+  EXPECT_CALL(*callback, Run());
 
   RunFilter2Callback();
 
@@ -580,8 +660,8 @@ TEST_F(CompositeFilterTest, TestStop) {
 
   EXPECT_CALL(*filter_1_, Stop(_));
 
-  StrictMock<MockCallback>* callback = new StrictMock<MockCallback>();
-  composite_->Stop(callback);
+  StrictMock<MockClosure>* callback = new StrictMock<MockClosure>();
+  composite_->Stop(base::Bind(&MockClosure::Run, callback));
 
   // Have |filter_1_| signal an error.
   filter_1_->host()->SetError(PIPELINE_ERROR_READ);
@@ -590,7 +670,7 @@ TEST_F(CompositeFilterTest, TestStop) {
 
   RunFilter1Callback();
 
-  callback->ExpectRunAndDelete();
+  EXPECT_CALL(*callback, Run());
 
   RunFilter2Callback();
 }
@@ -603,15 +683,13 @@ TEST_F(CompositeFilterTest, TestStopWhilePlayPending) {
 
   EXPECT_CALL(*filter_1_, Play(_));
 
-  StrictMock<MockCallback>* callback = new StrictMock<MockCallback>();
-  composite_->Play(callback);
+  StrictMock<MockClosure>* callback = new StrictMock<MockClosure>();
+  composite_->Play(base::Bind(&MockClosure::Run, callback));
 
   // Note: Play() is pending on |filter_1_| right now.
 
-  EXPECT_CALL(*callback, Destructor());
-
-  callback = new StrictMock<MockCallback>();
-  composite_->Stop(callback);
+  callback = new StrictMock<MockClosure>();
+  composite_->Stop(base::Bind(&MockClosure::Run, callback));
 
   EXPECT_CALL(*filter_1_, Stop(_));
 
@@ -623,7 +701,7 @@ TEST_F(CompositeFilterTest, TestStopWhilePlayPending) {
   // Run |filter_1_|'s callback again to indicate Stop() has completed.
   RunFilter1Callback();
 
-  callback->ExpectRunAndDelete();
+  EXPECT_CALL(*callback, Run());
 
   // Run |filter_2_|'s callback to indicate Stop() has completed.
   RunFilter2Callback();
@@ -638,15 +716,13 @@ TEST_F(CompositeFilterTest, TestStopWhileFlushPending) {
   EXPECT_CALL(*filter_1_, Flush(_));
   EXPECT_CALL(*filter_2_, Flush(_));
 
-  StrictMock<MockCallback>* callback = new StrictMock<MockCallback>();
-  composite_->Flush(callback);
+  StrictMock<MockClosure>* callback = new StrictMock<MockClosure>();
+  composite_->Flush(base::Bind(&MockClosure::Run, callback));
 
   // Note: |filter_1_| and |filter_2_| have pending Flush() calls at this point.
 
-  EXPECT_CALL(*callback, Destructor());
-
-  callback = new StrictMock<MockCallback>();
-  composite_->Stop(callback);
+  callback = new StrictMock<MockClosure>();
+  composite_->Stop(base::Bind(&MockClosure::Run, callback));
 
   // Run callback to indicate that |filter_1_|'s Flush() has completed.
   RunFilter1Callback();
@@ -661,7 +737,7 @@ TEST_F(CompositeFilterTest, TestStopWhileFlushPending) {
   // Run callback to indicate that |filter_1_|'s Stop() has completed.
   RunFilter1Callback();
 
-  callback->ExpectRunAndDelete();
+  EXPECT_CALL(*callback, Run());
 
   // Run callback to indicate that |filter_2_|'s Stop() has completed.
   RunFilter2Callback();
@@ -704,23 +780,23 @@ TEST_F(CompositeFilterTest, TestEmptyComposite) {
   composite_->set_host(mock_filter_host_.get());
 
   // Issue a Play() and expect no errors.
-  composite_->Play(NewExpectedCallback());
+  composite_->Play(NewExpectedClosure());
 
   // Issue a Pause() and expect no errors.
-  composite_->Pause(NewExpectedCallback());
+  composite_->Pause(NewExpectedClosure());
 
   // Issue a Flush() and expect no errors.
-  composite_->Flush(NewExpectedCallback());
+  composite_->Flush(NewExpectedClosure());
 
   // Issue a Seek() and expect no errors.
   composite_->Seek(base::TimeDelta::FromSeconds(5),
-                   NewExpectedCallback());
+                   NewExpectedStatusCB(PIPELINE_OK));
 
   // Issue a Play() and expect no errors.
-  composite_->Play(NewExpectedCallback());
+  composite_->Play(NewExpectedClosure());
 
   // Issue a Stop() and expect no errors.
-  composite_->Stop(NewExpectedCallback());
+  composite_->Stop(NewExpectedClosure());
 }
 
 }  // namespace media
