@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,24 +10,23 @@
 #ifndef MEDIA_BASE_DATA_BUFFER_H_
 #define MEDIA_BASE_DATA_BUFFER_H_
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "media/base/buffers.h"
 
 namespace media {
 
-class DataBuffer : public Buffer {
+class MEDIA_EXPORT DataBuffer : public Buffer {
  public:
-  // Takes ownership of the passed |buffer|, assumes valid data of size
-  // |buffer_size|.
-  DataBuffer(uint8* buffer, size_t buffer_size);
+  // Assumes valid data of size |buffer_size|.
+  DataBuffer(scoped_array<uint8> buffer, size_t buffer_size);
 
   // Allocates buffer of size |buffer_size|. If |buffer_size| is 0, |data_| is
   // set to a NULL ptr.
   explicit DataBuffer(size_t buffer_size);
 
   // Buffer implementation.
-  virtual const uint8* GetData() const;
-  virtual size_t GetDataSize() const;
+  virtual const uint8* GetData() const OVERRIDE;
+  virtual size_t GetDataSize() const OVERRIDE;
 
   // Returns a read-write pointer to the buffer data.
   virtual uint8* GetWritableData();
