@@ -13,12 +13,14 @@
 
 namespace media {
 
-class MessageLoopFactoryImpl : public MessageLoopFactory {
+class MEDIA_EXPORT MessageLoopFactoryImpl : public MessageLoopFactory {
  public:
   MessageLoopFactoryImpl();
 
   // MessageLoopFactory methods.
-  virtual MessageLoop* GetMessageLoop(const std::string& name);
+  virtual MessageLoop* GetMessageLoop(const std::string& name) OVERRIDE;
+  virtual scoped_refptr<base::MessageLoopProxy> GetMessageLoopProxy(
+      const std::string& name) OVERRIDE;
 
  protected:
   virtual ~MessageLoopFactoryImpl();
