@@ -11,7 +11,7 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "media/base/filter_host.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
@@ -24,7 +24,7 @@ class MockFilterHost : public FilterHost {
 
   // FilterHost implementation.
   MOCK_METHOD0(InitializationComplete, void());
-  MOCK_METHOD1(SetError, void(PipelineError error));
+  MOCK_METHOD1(SetError, void(PipelineStatus error));
   MOCK_CONST_METHOD0(GetDuration, base::TimeDelta());
   MOCK_CONST_METHOD0(GetTime, base::TimeDelta());
   MOCK_METHOD1(SetTime, void(base::TimeDelta time));
@@ -32,9 +32,7 @@ class MockFilterHost : public FilterHost {
   MOCK_METHOD1(SetBufferedTime, void(base::TimeDelta buffered_time));
   MOCK_METHOD1(SetTotalBytes, void(int64 total_bytes));
   MOCK_METHOD1(SetBufferedBytes, void(int64 buffered_bytes));
-  MOCK_METHOD2(SetVideoSize, void(size_t width, size_t height));
-  MOCK_METHOD1(SetStreaming, void(bool streamed));
-  MOCK_METHOD1(SetLoaded, void(bool loaded));
+  MOCK_METHOD1(SetNaturalVideoSize, void(const gfx::Size& size));
   MOCK_METHOD1(SetNetworkActivity, void(bool network_activity));
   MOCK_METHOD0(NotifyEnded, void());
   MOCK_METHOD0(DisableAudioRenderer, void());
