@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -15,8 +15,13 @@
 
 #include <string>
 
+#include "base/callback.h"
+#include "media/base/audio_decoder_config.h"
+#include "media/base/demuxer.h"
 #include "media/base/filters.h"
 #include "media/base/filter_collection.h"
+#include "media/base/pipeline.h"
+#include "media/base/video_decoder_config.h"
 #include "media/base/video_frame.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
@@ -48,12 +53,12 @@ class MockFilter : public Filter {
   MockFilter();
 
   // Filter implementation.
-  MOCK_METHOD1(Play, void(FilterCallback* callback));
-  MOCK_METHOD1(Pause, void(FilterCallback* callback));
-  MOCK_METHOD1(Flush, void(FilterCallback* callback));
-  MOCK_METHOD1(Stop, void(FilterCallback* callback));
+  MOCK_METHOD1(Play, void(const base::Closure& callback));
+  MOCK_METHOD1(Pause, void(const base::Closure& callback));
+  MOCK_METHOD1(Flush, void(const base::Closure& callback));
+  MOCK_METHOD1(Stop, void(const base::Closure& callback));
   MOCK_METHOD1(SetPlaybackRate, void(float playback_rate));
-  MOCK_METHOD2(Seek, void(base::TimeDelta time, FilterCallback* callback));
+  MOCK_METHOD2(Seek, void(base::TimeDelta time, const FilterStatusCB& cb));
   MOCK_METHOD0(OnAudioRendererDisabled, void());
 
  protected:
@@ -67,26 +72,32 @@ class MockDataSource : public DataSource {
  public:
   MockDataSource();
 
-  // Filter implementation.
-  MOCK_METHOD1(Stop, void(FilterCallback* callback));
+  virtual void set_host(DataSourceHost* data_source_host);
+
+  MOCK_METHOD1(Stop, void(const base::Closure& callback));
   MOCK_METHOD1(SetPlaybackRate, void(float playback_rate));
-  MOCK_METHOD2(Seek, void(base::TimeDelta time, FilterCallback* callback));
+  MOCK_METHOD2(Seek, void(base::TimeDelta time, const FilterStatusCB& cb));
   MOCK_METHOD0(OnAudioRendererDisabled, void());
 
   // DataSource implementation.
-  MOCK_METHOD1(IsUrlSupported, bool(const std::string& url));
-  MOCK_METHOD2(Initialize, void(const std::string& url,
-                                FilterCallback* callback));
-  MOCK_METHOD0(media_format, const MediaFormat&());
   MOCK_METHOD4(Read, void(int64 position, size_t size, uint8* data,
-                          DataSource::ReadCallback* callback));
+                          const DataSource::ReadCallback& callback));
   MOCK_METHOD1(GetSize, bool(int64* size_out));
+  MOCK_METHOD1(SetPreload, void(Preload preload));
+  MOCK_METHOD1(SetBitrate, void(int bitrate));
   MOCK_METHOD0(IsStreaming, bool());
 
+  // Sets the TotalBytes & BufferedBytes values to be sent to host() when
+  // the set_host() is called.
+  void SetTotalAndBufferedBytes(int64 total_bytes, int64 buffered_bytes);
+
  protected:
   virtual ~MockDataSource();
 
  private:
+  int64 total_bytes_;
+  int64 buffered_bytes_;
+
   DISALLOW_COPY_AND_ASSIGN(MockDataSource);
 };
 
@@ -94,41 +105,72 @@ class MockDemuxer : public Demuxer {
  public:
   MockDemuxer();
 
-  // Filter implementation.
-  MOCK_METHOD1(Stop, void(FilterCallback* callback));
+  virtual void set_host(DemuxerHost* demuxer_host);
+  MOCK_METHOD1(Stop, void(const base::Closure& callback));
   MOCK_METHOD1(SetPlaybackRate, void(float playback_rate));
-  MOCK_METHOD2(Seek, void(base::TimeDelta time, FilterCallback* callback));
+  MOCK_METHOD1(SetPreload, void(Preload preload));
+  MOCK_METHOD2(Seek, void(base::TimeDelta time, const FilterStatusCB& cb));
   MOCK_METHOD0(OnAudioRendererDisabled, void());
+  MOCK_METHOD0(GetBitrate, int());
+  MOCK_METHOD0(IsLocalSource, bool());
+  MOCK_METHOD0(IsSeekable, bool());
 
   // Demuxer implementation.
   MOCK_METHOD2(Initialize, void(DataSource* data_source,
-                                FilterCallback* callback));
-  MOCK_METHOD0(GetNumberOfStreams, size_t());
-  MOCK_METHOD1(GetStream, scoped_refptr<DemuxerStream>(int stream_id));
+                                const base::Closure& callback));
+  MOCK_METHOD1(GetStream, scoped_refptr<DemuxerStream>(DemuxerStream::Type));
+  MOCK_CONST_METHOD0(GetStartTime, base::TimeDelta());
+
+  // Sets the TotalBytes, BufferedBytes, & Duration values to be sent to host()
+  // when set_host() is called.
+  void SetTotalAndBufferedBytesAndDuration(
+      int64 total_bytes, int64 buffered_bytes, const base::TimeDelta& duration);
 
  protected:
   virtual ~MockDemuxer();
 
  private:
+  int64 total_bytes_;
+  int64 buffered_bytes_;
+  base::TimeDelta duration_;
+
   DISALLOW_COPY_AND_ASSIGN(MockDemuxer);
 };
 
+class MockDemuxerFactory : public DemuxerFactory {
+ public:
+  explicit MockDemuxerFactory(MockDemuxer* demuxer);
+  virtual ~MockDemuxerFactory();
+
+  void SetError(PipelineStatus error);
+  void RunBuildCallback(const std::string& url, const BuildCallback& callback);
+
+  // DemuxerFactory methods.
+  MOCK_METHOD2(Build, void(const std::string& url,
+                           const BuildCallback& callback));
+
+ private:
+  scoped_refptr<MockDemuxer> demuxer_;
+  PipelineStatus status_;
+
+  DISALLOW_COPY_AND_ASSIGN(MockDemuxerFactory);
+};
+
 class MockDemuxerStream : public DemuxerStream {
  public:
   MockDemuxerStream();
 
   // DemuxerStream implementation.
-  MOCK_METHOD0(media_format, const MediaFormat&());
-  MOCK_METHOD1(Read, void(Callback1<Buffer*>::Type* read_callback));
-  MOCK_METHOD1(QueryInterface, void*(const char* interface_id));
+  MOCK_METHOD0(type, Type());
+  MOCK_METHOD1(Read, void(const ReadCallback& read_callback));
+  MOCK_METHOD0(audio_decoder_config, const AudioDecoderConfig&());
+  MOCK_METHOD0(video_decoder_config, const VideoDecoderConfig&());
   MOCK_METHOD0(EnableBitstreamConverter, void());
 
  protected:
   virtual ~MockDemuxerStream();
 
  private:
-  MediaFormat media_format_;
-
   DISALLOW_COPY_AND_ASSIGN(MockDemuxerStream);
 };
 
@@ -137,22 +179,18 @@ class MockVideoDecoder : public VideoDecoder {
   MockVideoDecoder();
 
   // Filter implementation.
-  MOCK_METHOD1(Stop, void(FilterCallback* callback));
+  MOCK_METHOD1(Stop, void(const base::Closure& callback));
   MOCK_METHOD1(SetPlaybackRate, void(float playback_rate));
-  MOCK_METHOD2(Seek, void(base::TimeDelta time, FilterCallback* callback));
+  MOCK_METHOD2(Seek, void(base::TimeDelta time, const FilterStatusCB& cb));
   MOCK_METHOD0(OnAudioRendererDisabled, void());
 
   // VideoDecoder implementation.
-  MOCK_METHOD2(Initialize, void(DemuxerStream* stream,
-                                FilterCallback* callback));
-  MOCK_METHOD0(media_format, const MediaFormat&());
-  MOCK_METHOD1(ProduceVideoFrame, void(scoped_refptr<VideoFrame>));
-  MOCK_METHOD0(ProvidesBuffer, bool());
-
-  // Make this method public so that tests can make use of it.
-  void VideoFrameReady(scoped_refptr<VideoFrame> frame) {
-    VideoDecoder::VideoFrameReady(frame);
-  }
+  MOCK_METHOD3(Initialize, void(DemuxerStream* stream,
+                                const PipelineStatusCB& callback,
+                                const StatisticsCallback& stats_callback));
+  MOCK_METHOD1(Read, void(const ReadCB& callback));
+  MOCK_METHOD0(natural_size, const gfx::Size&());
+  MOCK_CONST_METHOD0(HasAlpha, bool());
 
  protected:
   virtual ~MockVideoDecoder();
@@ -166,21 +204,20 @@ class MockAudioDecoder : public AudioDecoder {
   MockAudioDecoder();
 
   // Filter implementation.
-  MOCK_METHOD1(Stop, void(FilterCallback* callback));
+  MOCK_METHOD1(Stop, void(const base::Closure& callback));
   MOCK_METHOD1(SetPlaybackRate, void(float playback_rate));
-  MOCK_METHOD2(Seek, void(base::TimeDelta time, FilterCallback* callback));
+  MOCK_METHOD2(Seek, void(base::TimeDelta time, const FilterStatusCB& cb));
   MOCK_METHOD0(OnAudioRendererDisabled, void());
 
   // AudioDecoder implementation.
-  MOCK_METHOD2(Initialize, void(DemuxerStream* stream,
-                                FilterCallback* callback));
-  MOCK_METHOD0(media_format, const MediaFormat&());
+  MOCK_METHOD3(Initialize, void(DemuxerStream* stream,
+                                const base::Closure& callback,
+                                const StatisticsCallback& stats_callback));
+  MOCK_METHOD1(Read, void(const ReadCB& callback));
   MOCK_METHOD1(ProduceAudioSamples, void(scoped_refptr<Buffer>));
-
-  // change to public to allow unittest for access;
-  ConsumeAudioSamplesCallback* consume_audio_samples_callback() {
-    return AudioDecoder::consume_audio_samples_callback();
-  }
+  MOCK_METHOD0(bits_per_channel, int(void));
+  MOCK_METHOD0(channel_layout, ChannelLayout(void));
+  MOCK_METHOD0(samples_per_second, int(void));
 
  protected:
   virtual ~MockAudioDecoder();
@@ -194,15 +231,19 @@ class MockVideoRenderer : public VideoRenderer {
   MockVideoRenderer();
 
   // Filter implementation.
-  MOCK_METHOD1(Stop, void(FilterCallback* callback));
+  MOCK_METHOD1(Stop, void(const base::Closure& callback));
   MOCK_METHOD1(SetPlaybackRate, void(float playback_rate));
-  MOCK_METHOD2(Seek, void(base::TimeDelta time, FilterCallback* callback));
+  MOCK_METHOD2(Seek, void(base::TimeDelta time, const FilterStatusCB& cb));
   MOCK_METHOD0(OnAudioRendererDisabled, void());
 
   // VideoRenderer implementation.
-  MOCK_METHOD2(Initialize, void(VideoDecoder* decoder,
-                                FilterCallback* callback));
+  MOCK_METHOD3(Initialize, void(VideoDecoder* decoder,
+                                const base::Closure& callback,
+                                const StatisticsCallback& stats_callback));
   MOCK_METHOD0(HasEnded, bool());
+
+  // TODO(scherkus): although VideoRendererBase defines this method, this really
+  // shouldn't be here OR should be renamed.
   MOCK_METHOD1(ConsumeVideoFrame, void(scoped_refptr<VideoFrame> frame));
 
  protected:
@@ -217,17 +258,20 @@ class MockAudioRenderer : public AudioRenderer {
   MockAudioRenderer();
 
   // Filter implementation.
-  MOCK_METHOD1(Stop, void(FilterCallback* callback));
+  MOCK_METHOD1(Stop, void(const base::Closure& callback));
   MOCK_METHOD1(SetPlaybackRate, void(float playback_rate));
-  MOCK_METHOD2(Seek, void(base::TimeDelta time, FilterCallback* callback));
+  MOCK_METHOD2(Seek, void(base::TimeDelta time, const FilterStatusCB& cb));
   MOCK_METHOD0(OnAudioRendererDisabled, void());
 
   // AudioRenderer implementation.
-  MOCK_METHOD2(Initialize, void(AudioDecoder* decoder,
-                                FilterCallback* callback));
+  MOCK_METHOD3(Initialize, void(AudioDecoder* decoder,
+                                const base::Closure& init_callback,
+                                const base::Closure& underflow_callback));
   MOCK_METHOD0(HasEnded, bool());
   MOCK_METHOD1(SetVolume, void(float volume));
 
+  MOCK_METHOD1(ResumeAfterUnderflow, void(bool buffer_more_audio));
+
  protected:
   virtual ~MockAudioRenderer();
 
@@ -243,21 +287,21 @@ class MockFilterCollection {
   virtual ~MockFilterCollection();
 
   // Mock accessors.
-  MockDataSource* data_source() const { return data_source_; }
   MockDemuxer* demuxer() const { return demuxer_; }
   MockVideoDecoder* video_decoder() const { return video_decoder_; }
   MockAudioDecoder* audio_decoder() const { return audio_decoder_; }
   MockVideoRenderer* video_renderer() const { return video_renderer_; }
   MockAudioRenderer* audio_renderer() const { return audio_renderer_; }
 
-  FilterCollection* filter_collection() const {
-    return filter_collection(true);
+  scoped_ptr<FilterCollection> filter_collection() const {
+    return filter_collection(true, true, true, PIPELINE_OK).Pass();
   }
 
-  FilterCollection* filter_collection(bool include_data_source) const;
+  scoped_ptr<FilterCollection> filter_collection(
+      bool include_demuxer, bool run_build_callback, bool run_build,
+      PipelineStatus build_status) const;
 
  private:
-  scoped_refptr<MockDataSource> data_source_;
   scoped_refptr<MockDemuxer> demuxer_;
   scoped_refptr<MockVideoDecoder> video_decoder_;
   scoped_refptr<MockAudioDecoder> audio_decoder_;
@@ -267,20 +311,18 @@ class MockFilterCollection {
   DISALLOW_COPY_AND_ASSIGN(MockFilterCollection);
 };
 
-// Helper gmock function that immediately executes and destroys the
-// FilterCallback on behalf of the provided filter.  Can be used when mocking
+// Helper gmock functions that immediately executes and destroys the
+// Closure on behalf of the provided filter.  Can be used when mocking
 // the Initialize() and Seek() methods.
-void RunFilterCallback(::testing::Unused, FilterCallback* callback);
+void RunFilterCallback(::testing::Unused, const base::Closure& callback);
+void RunFilterStatusCB(::testing::Unused, const FilterStatusCB& cb);
+void RunPipelineStatusCB(PipelineStatus status, const PipelineStatusCB& cb);
+void RunFilterCallback3(::testing::Unused, const base::Closure& callback,
+                        ::testing::Unused);
 
-// Helper gmock function that immediately destroys the FilterCallback on behalf
-// of the provided filter.  Can be used when mocking the Initialize() and Seek()
-// methods.
-void DestroyFilterCallback(::testing::Unused, FilterCallback* callback);
-
-// Helper gmock function that immediately executes and destroys the
-// FilterCallback on behalf of the provided filter.  Can be used when mocking
-// the Stop() method.
-void RunStopFilterCallback(FilterCallback* callback);
+// Helper gmock function that immediately executes the Closure on behalf of the
+// provided filter.  Can be used when mocking the Stop() method.
+void RunStopFilterCallback(const base::Closure& callback);
 
 // Helper gmock action that calls SetError() on behalf of the provided filter.
 ACTION_P2(SetError, filter, error) {
@@ -293,24 +335,21 @@ ACTION_P2(SetDuration, filter, duration) {
   filter->host()->SetDuration(duration);
 }
 
-// Helper gmock action that calls SetTotalBytes() on behalf of the provided
-// filter.
-ACTION_P2(SetTotalBytes, filter, bytes) {
-  filter->host()->SetTotalBytes(bytes);
-}
-
-// Helper gmock action that calls SetBufferedBytes() on behalf of the provided
-// filter.
-ACTION_P2(SetBufferedBytes, filter, bytes) {
-  filter->host()->SetBufferedBytes(bytes);
-}
-
 // Helper gmock action that calls DisableAudioRenderer() on behalf of the
 // provided filter.
 ACTION_P(DisableAudioRenderer, filter) {
   filter->host()->DisableAudioRenderer();
 }
 
+// Helper mock statistics callback.
+class MockStatisticsCallback {
+ public:
+  MockStatisticsCallback();
+  ~MockStatisticsCallback();
+
+  MOCK_METHOD1(OnStatistics, void(const media::PipelineStatistics& statistics));
+};
+
 }  // namespace media
 
 #endif  // MEDIA_BASE_MOCK_FILTERS_H_
