@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -37,7 +37,6 @@
 #include "base/test/perf_test_suite.h"
 #include "media/base/media.h"
 #include "media/ffmpeg/ffmpeg_common.h"
-#include "media/ffmpeg/ffmpeg_util.h"
 #include "media/ffmpeg/file_protocol.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -144,7 +143,7 @@ class FFmpegTest : public testing::TestWithParam<const char*> {
     // Determine duration by picking max stream duration.
     for (unsigned int i = 0; i < av_format_context_->nb_streams; ++i) {
       AVStream* av_stream = av_format_context_->streams[i];
-      int64 duration = ConvertTimestamp(av_stream->time_base,
+      int64 duration = ConvertFromTimeBase(av_stream->time_base,
                                         av_stream->duration).InMicroseconds();
       duration_ = std::max(duration_, duration);
     }
@@ -152,7 +151,7 @@ class FFmpegTest : public testing::TestWithParam<const char*> {
     // Final check to see if the container itself specifies a duration.
     AVRational av_time_base = {1, AV_TIME_BASE};
     int64 duration =
-        ConvertTimestamp(av_time_base,
+        ConvertFromTimeBase(av_time_base,
                          av_format_context_->duration).InMicroseconds();
     duration_ = std::max(duration_, duration);
   }
@@ -174,10 +173,10 @@ class FFmpegTest : public testing::TestWithParam<const char*> {
           << "Could not open AVCodecContext with CodecID "
           << av_codec_context->codec_id;
 
-      if (av_codec->type == CODEC_TYPE_AUDIO) {
+      if (av_codec->type == AVMEDIA_TYPE_AUDIO) {
         EXPECT_EQ(-1, audio_stream_index_) << "Found multiple audio streams.";
         audio_stream_index_ = static_cast<int>(i);
-      } else if (av_codec->type == CODEC_TYPE_VIDEO) {
+      } else if (av_codec->type == AVMEDIA_TYPE_VIDEO) {
         EXPECT_EQ(-1, video_stream_index_) << "Found multiple video streams.";
         video_stream_index_ = static_cast<int>(i);
       } else {
@@ -216,12 +215,12 @@ class FFmpegTest : public testing::TestWithParam<const char*> {
       int64 packet_time = AV_NOPTS_VALUE;
       if (stream_index == audio_stream_index_) {
         packet_time =
-            ConvertTimestamp(av_audio_stream()->time_base, packet->pts)
+            ConvertFromTimeBase(av_audio_stream()->time_base, packet->pts)
                 .InMicroseconds();
         audio_packets_.push(packet.release());
       } else if (stream_index == video_stream_index_) {
         packet_time =
-            ConvertTimestamp(av_video_stream()->time_base, packet->pts)
+            ConvertFromTimeBase(av_video_stream()->time_base, packet->pts)
                 .InMicroseconds();
         video_packets_.push(packet.release());
       } else {
@@ -270,7 +269,7 @@ class FFmpegTest : public testing::TestWithParam<const char*> {
       }
 
       if (result > 0) {
-        // TODO(scherkus): move this to ffmpeg_util.h and dedup.
+        // TODO(scherkus): move this to ffmpeg_common.h and dedup.
         int64 denominator = av_audio_context()->channels *
             av_get_bits_per_sample_fmt(av_audio_context()->sample_fmt) / 8 *
             av_audio_context()->sample_rate;
@@ -286,7 +285,7 @@ class FFmpegTest : public testing::TestWithParam<const char*> {
           decoded_audio_time_ += decoded_audio_duration_;
         } else {
           decoded_audio_time_ =
-              ConvertTimestamp(av_audio_stream()->time_base, packet.pts)
+              ConvertFromTimeBase(av_audio_stream()->time_base, packet.pts)
                   .InMicroseconds();
         }
         return true;
@@ -334,11 +333,11 @@ class FFmpegTest : public testing::TestWithParam<const char*> {
         doubled_time_base.den *= 2;
 
         decoded_video_time_ =
-            ConvertTimestamp(av_video_stream()->time_base,
+            ConvertFromTimeBase(av_video_stream()->time_base,
                              video_buffer_->reordered_opaque)
                 .InMicroseconds();
         decoded_video_duration_ =
-            ConvertTimestamp(doubled_time_base,
+            ConvertFromTimeBase(doubled_time_base,
                              2 + video_buffer_->repeat_pict)
                 .InMicroseconds();
         return true;
