@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,7 @@
 // must also implement the following methods:
 //   OnInitialized
 //   OnStop
+//   OnRenderEndOfStream
 //
 // The general assumption is that subclasses start a callback-based audio thread
 // which needs to be filled with decoded audio data.  AudioDecoderBase provides
@@ -27,35 +28,46 @@
 
 namespace media {
 
-class AudioRendererBase : public AudioRenderer {
+class MEDIA_EXPORT AudioRendererBase : public AudioRenderer {
  public:
   AudioRendererBase();
   virtual ~AudioRendererBase();
 
   // Filter implementation.
-  virtual void Play(FilterCallback* callback);
-  virtual void Pause(FilterCallback* callback);
-  virtual void Stop(FilterCallback* callback);
+  virtual void Play(const base::Closure& callback) OVERRIDE;
+  virtual void Pause(const base::Closure& callback) OVERRIDE;
+  virtual void Stop(const base::Closure& callback) OVERRIDE;
 
-  virtual void Seek(base::TimeDelta time, FilterCallback* callback);
+  virtual void Seek(base::TimeDelta time, const FilterStatusCB& cb) OVERRIDE;
 
   // AudioRenderer implementation.
-  virtual void Initialize(AudioDecoder* decoder, FilterCallback* callback);
-  virtual bool HasEnded();
+  virtual void Initialize(AudioDecoder* decoder,
+                          const base::Closure& init_callback,
+                          const base::Closure& underflow_callback) OVERRIDE;
+  virtual bool HasEnded() OVERRIDE;
+  virtual void ResumeAfterUnderflow(bool buffer_more_audio) OVERRIDE;
 
  protected:
-  // Called by Initialize().  |media_format| is the format of the AudioDecoder.
+  FRIEND_TEST_ALL_PREFIXES(AudioRendererBaseTest, EndOfStream);
+  FRIEND_TEST_ALL_PREFIXES(AudioRendererBaseTest, Underflow_EndOfStream);
+
   // Subclasses should return true if they were able to initialize, false
   // otherwise.
-  virtual bool OnInitialize(const MediaFormat& media_format) = 0;
+  virtual bool OnInitialize(int bits_per_channel,
+                            ChannelLayout channel_layout,
+                            int sample_rate) = 0;
 
   // Called by Stop().  Subclasses should perform any necessary cleanup during
   // this time, such as stopping any running threads.
   virtual void OnStop() = 0;
 
-  // Called when a AudioDecoder completes decoding and decrements
-  // |pending_reads_|.
-  virtual void ConsumeAudioSamples(scoped_refptr<Buffer> buffer_in);
+  // Method called by FillBuffer() when it finds that it reached end of stream.
+  // FillBuffer() cannot immediately signal end of stream event because browser
+  // may have buffered data.
+  virtual void OnRenderEndOfStream() = 0;
+
+  // Callback from the audio decoder delivering decoded audio samples.
+  void DecodedAudioReady(scoped_refptr<Buffer> buffer);
 
   // Fills the given buffer with audio data by delegating to its |algorithm_|.
   // FillBuffer() also takes care of updating the clock. Returns the number of
@@ -73,29 +85,39 @@ class AudioRendererBase : public AudioRenderer {
   // should the filled buffer be played. If FillBuffer() is called as the audio
   // hardware plays the buffer, then |playback_delay| should be zero.
   //
+  // FillBuffer() calls OnRenderEndOfStream() when it reaches end of stream.
+  // It is responsibility of derived class to provide implementation of
+  // OnRenderEndOfStream() that calls SignalEndOfStream() when all the hardware
+  // buffers become empty (i.e. when all the data written to the device has
+  // been played).
+  //
   // Safe to call on any thread.
   uint32 FillBuffer(uint8* dest,
                     uint32 len,
-                    const base::TimeDelta& playback_delay,
-                    bool buffers_empty);
+                    const base::TimeDelta& playback_delay);
 
-  // Helper to parse a media format and return whether we were successful
-  // retrieving all the information we care about.
-  static bool ParseMediaFormat(const MediaFormat& media_format,
-                               int* channels_out, int* sample_rate_out,
-                               int* sample_bits_out);
+  // Called by OnRenderEndOfStream() or some callback scheduled by derived class
+  // to signal end of stream.
+  void SignalEndOfStream();
 
   // Get/Set the playback rate of |algorithm_|.
-  virtual void SetPlaybackRate(float playback_rate);
+  virtual void SetPlaybackRate(float playback_rate) OVERRIDE;
   virtual float GetPlaybackRate();
 
  private:
+  friend class AudioRendererBaseTest;
+
   // Helper method that schedules an asynchronous read from the decoder and
   // increments |pending_reads_|.
   //
   // Safe to call from any thread.
   void ScheduleRead_Locked();
 
+  // Returns true if the data in the buffer is all before
+  // |seek_timestamp_|. This can only return true while
+  // in the kSeeking state.
+  bool IsBeforeSeekTime(const scoped_refptr<Buffer>& buffer);
+
   // Audio decoder.
   scoped_refptr<AudioDecoder> decoder_;
 
@@ -111,31 +133,32 @@ class AudioRendererBase : public AudioRenderer {
     kSeeking,
     kPlaying,
     kStopped,
-    kError,
+    kUnderflow,
+    kRebuffering,
   };
   State state_;
 
+  // Keep track of our outstanding read to |decoder_|.
+  bool pending_read_;
+
   // Keeps track of whether we received and rendered the end of stream buffer.
   bool recieved_end_of_stream_;
   bool rendered_end_of_stream_;
 
-  // Keeps track of our pending reads.  We *must* have no pending reads before
-  // executing the pause callback, otherwise we breach the contract that all
-  // filters are idling.
-  //
-  // We use size_t since we compare against std::deque::size().
-  size_t pending_reads_;
-
   // Audio time at end of last call to FillBuffer().
   // TODO(ralphl): Update this value after seeking.
   base::TimeDelta last_fill_buffer_time_;
 
   // Filter callbacks.
-  scoped_ptr<FilterCallback> pause_callback_;
-  scoped_ptr<FilterCallback> seek_callback_;
+  base::Closure pause_callback_;
+  FilterStatusCB seek_cb_;
+
+  base::Closure underflow_callback_;
 
   base::TimeDelta seek_timestamp_;
 
+  AudioDecoder::ReadCB read_cb_;
+
   DISALLOW_COPY_AND_ASSIGN(AudioRendererBase);
 };
 
