@@ -1,22 +1,23 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/bind.h"
 #include "base/callback.h"
 #include "base/command_line.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/scoped_ptr.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
 #include "base/time.h"
-#include "media/base/filter_host.h"
+#include "media/base/data_buffer.h"
 #include "media/base/limits.h"
 #include "media/base/media_switches.h"
 #include "media/ffmpeg/ffmpeg_common.h"
-#include "media/ffmpeg/ffmpeg_util.h"
 #include "media/filters/bitstream_converter.h"
 #include "media/filters/ffmpeg_demuxer.h"
 #include "media/filters/ffmpeg_glue.h"
+#include "media/filters/ffmpeg_h264_bitstream_converter.h"
 
 namespace media {
 
@@ -25,9 +26,10 @@ namespace media {
 //
 class AVPacketBuffer : public Buffer {
  public:
-  AVPacketBuffer(AVPacket* packet, const base::TimeDelta& timestamp,
+  AVPacketBuffer(scoped_ptr_malloc<AVPacket, ScopedPtrAVFreePacket> packet,
+                 const base::TimeDelta& timestamp,
                  const base::TimeDelta& duration)
-      : packet_(packet) {
+      : packet_(packet.Pass()) {
     SetTimestamp(timestamp);
     SetDuration(duration);
   }
@@ -58,22 +60,20 @@ FFmpegDemuxerStream::FFmpegDemuxerStream(FFmpegDemuxer* demuxer,
                                          AVStream* stream)
     : demuxer_(demuxer),
       stream_(stream),
+      type_(UNKNOWN),
+      discontinuous_(false),
       stopped_(false) {
   DCHECK(demuxer_);
 
   // Determine our media format.
   switch (stream->codec->codec_type) {
-    case CODEC_TYPE_AUDIO:
-      media_format_.SetAsString(MediaFormat::kMimeType,
-                                mime_type::kFFmpegAudio);
-      media_format_.SetAsInteger(MediaFormat::kFFmpegCodecID,
-                                 stream->codec->codec_id);
+    case AVMEDIA_TYPE_AUDIO:
+      type_ = AUDIO;
+      AVCodecContextToAudioDecoderConfig(stream->codec, &audio_config_);
       break;
-    case CODEC_TYPE_VIDEO:
-      media_format_.SetAsString(MediaFormat::kMimeType,
-                                mime_type::kFFmpegVideo);
-      media_format_.SetAsInteger(MediaFormat::kFFmpegCodecID,
-                                 stream->codec->codec_id);
+    case AVMEDIA_TYPE_VIDEO:
+      type_ = VIDEO;
+      AVStreamToVideoDecoderConfig(stream, &video_config_);
       break;
     default:
       NOTREACHED();
@@ -85,34 +85,29 @@ FFmpegDemuxerStream::FFmpegDemuxerStream(FFmpegDemuxer* demuxer,
 }
 
 FFmpegDemuxerStream::~FFmpegDemuxerStream() {
+  base::AutoLock auto_lock(lock_);
   DCHECK(stopped_);
   DCHECK(read_queue_.empty());
   DCHECK(buffer_queue_.empty());
 }
 
-void* FFmpegDemuxerStream::QueryInterface(const char* id) {
-  DCHECK(id);
-  AVStreamProvider* interface_ptr = NULL;
-  if (0 == strcmp(id, AVStreamProvider::interface_id())) {
-    interface_ptr = this;
-  }
-  return interface_ptr;
-}
-
 bool FFmpegDemuxerStream::HasPendingReads() {
   DCHECK_EQ(MessageLoop::current(), demuxer_->message_loop());
+  base::AutoLock auto_lock(lock_);
   DCHECK(!stopped_ || read_queue_.empty())
       << "Read queue should have been emptied if demuxing stream is stopped";
   return !read_queue_.empty();
 }
 
-void FFmpegDemuxerStream::EnqueuePacket(AVPacket* packet) {
+void FFmpegDemuxerStream::EnqueuePacket(
+    scoped_ptr_malloc<AVPacket, ScopedPtrAVFreePacket> packet) {
   DCHECK_EQ(MessageLoop::current(), demuxer_->message_loop());
   base::TimeDelta timestamp =
       ConvertStreamTimestamp(stream_->time_base, packet->pts);
   base::TimeDelta duration =
       ConvertStreamTimestamp(stream_->time_base, packet->duration);
 
+  base::AutoLock auto_lock(lock_);
   if (stopped_) {
     NOTREACHED() << "Attempted to enqueue packet on a stopped stream";
     return;
@@ -120,13 +115,13 @@ void FFmpegDemuxerStream::EnqueuePacket(AVPacket* packet) {
 
   // Convert if the packet if there is bitstream filter.
   if (packet->data && bitstream_converter_.get() &&
-      !bitstream_converter_->ConvertPacket(packet)) {
+      !bitstream_converter_->ConvertPacket(packet.get())) {
     LOG(ERROR) << "Format converstion failed.";
   }
 
   // Enqueue the callback and attempt to satisfy a read immediately.
   scoped_refptr<Buffer> buffer(
-      new AVPacketBuffer(packet, timestamp, duration));
+      new AVPacketBuffer(packet.Pass(), timestamp, duration));
   if (!buffer) {
     NOTREACHED() << "Unable to allocate AVPacketBuffer";
     return;
@@ -138,14 +133,20 @@ void FFmpegDemuxerStream::EnqueuePacket(AVPacket* packet) {
 
 void FFmpegDemuxerStream::FlushBuffers() {
   DCHECK_EQ(MessageLoop::current(), demuxer_->message_loop());
+  base::AutoLock auto_lock(lock_);
   DCHECK(read_queue_.empty()) << "Read requests should be empty";
   buffer_queue_.clear();
 }
 
 void FFmpegDemuxerStream::Stop() {
   DCHECK_EQ(MessageLoop::current(), demuxer_->message_loop());
+  base::AutoLock auto_lock(lock_);
   buffer_queue_.clear();
-  STLDeleteElements(&read_queue_);
+  for (ReadQueue::iterator it = read_queue_.begin();
+       it != read_queue_.end(); ++it) {
+    it->Run(scoped_refptr<Buffer>(new DataBuffer(0)));
+  }
+  read_queue_.clear();
   stopped_ = true;
 }
 
@@ -153,24 +154,49 @@ base::TimeDelta FFmpegDemuxerStream::duration() {
   return duration_;
 }
 
-const MediaFormat& FFmpegDemuxerStream::media_format() {
-  return media_format_;
+DemuxerStream::Type FFmpegDemuxerStream::type() {
+  return type_;
 }
 
-void FFmpegDemuxerStream::Read(Callback1<Buffer*>::Type* read_callback) {
-  DCHECK(read_callback);
-  demuxer_->message_loop()->PostTask(FROM_HERE,
-      NewRunnableMethod(this, &FFmpegDemuxerStream::ReadTask, read_callback));
+void FFmpegDemuxerStream::Read(const ReadCallback& read_callback) {
+  DCHECK(!read_callback.is_null());
+
+  base::AutoLock auto_lock(lock_);
+  // Don't accept any additional reads if we've been told to stop.
+  // The demuxer_ may have been destroyed in the pipleine thread.
+  //
+  // TODO(scherkus): it would be cleaner if we replied with an error message.
+  if (stopped_) {
+    read_callback.Run(scoped_refptr<Buffer>(new DataBuffer(0)));
+    return;
+  }
+
+  if (!buffer_queue_.empty()) {
+    // Dequeue a buffer send back.
+    scoped_refptr<Buffer> buffer = buffer_queue_.front();
+    buffer_queue_.pop_front();
+
+    // Execute the callback.
+    read_callback.Run(buffer);
+
+    if (!read_queue_.empty())
+      demuxer_->PostDemuxTask();
+
+  } else {
+    demuxer_->message_loop()->PostTask(FROM_HERE, base::Bind(
+        &FFmpegDemuxerStream::ReadTask, this, read_callback));
+  }
 }
 
-void FFmpegDemuxerStream::ReadTask(Callback1<Buffer*>::Type* read_callback) {
+void FFmpegDemuxerStream::ReadTask(const ReadCallback& read_callback) {
   DCHECK_EQ(MessageLoop::current(), demuxer_->message_loop());
 
+  base::AutoLock auto_lock(lock_);
   // Don't accept any additional reads if we've been told to stop.
   //
   // TODO(scherkus): it would be cleaner if we replied with an error message.
   if (stopped_) {
-    delete read_callback;
+    read_callback.Run(scoped_refptr<Buffer>(new DataBuffer(0)));
     return;
   }
 
@@ -178,26 +204,27 @@ void FFmpegDemuxerStream::ReadTask(Callback1<Buffer*>::Type* read_callback) {
   read_queue_.push_back(read_callback);
   FulfillPendingRead();
 
-  // There are still pending reads, demux some more.
-  if (HasPendingReads()) {
+  // Check if there are still pending reads, demux some more.
+  if (!read_queue_.empty()) {
     demuxer_->PostDemuxTask();
   }
 }
 
 void FFmpegDemuxerStream::FulfillPendingRead() {
   DCHECK_EQ(MessageLoop::current(), demuxer_->message_loop());
+  lock_.AssertAcquired();
   if (buffer_queue_.empty() || read_queue_.empty()) {
     return;
   }
 
   // Dequeue a buffer and pending read pair.
   scoped_refptr<Buffer> buffer = buffer_queue_.front();
-  scoped_ptr<Callback1<Buffer*>::Type> read_callback(read_queue_.front());
+  ReadCallback read_callback(read_queue_.front());
   buffer_queue_.pop_front();
   read_queue_.pop_front();
 
   // Execute the callback.
-  read_callback->Run(buffer);
+  read_callback.Run(buffer);
 }
 
 void FFmpegDemuxerStream::EnableBitstreamConverter() {
@@ -208,7 +235,13 @@ void FFmpegDemuxerStream::EnableBitstreamConverter() {
   const char* filter_name = NULL;
   if (stream_->codec->codec_id == CODEC_ID_H264) {
     filter_name = "h264_mp4toannexb";
-  } else if (stream_->codec->codec_id == CODEC_ID_MPEG4) {
+    // Use Chromium bitstream converter in case of H.264
+    bitstream_converter_.reset(
+        new FFmpegH264BitstreamConverter(stream_->codec));
+    CHECK(bitstream_converter_->Initialize());
+    return;
+  }
+  if (stream_->codec->codec_id == CODEC_ID_MPEG4) {
     filter_name = "mpeg4video_es";
   } else if (stream_->codec->codec_id == CODEC_ID_WMV3) {
     filter_name = "vc1_asftorcv";
@@ -223,29 +256,41 @@ void FFmpegDemuxerStream::EnableBitstreamConverter() {
   }
 }
 
-AVStream* FFmpegDemuxerStream::GetAVStream() {
-  return stream_;
+const AudioDecoderConfig& FFmpegDemuxerStream::audio_decoder_config() {
+  CHECK_EQ(type_, AUDIO);
+  return audio_config_;
+}
+
+const VideoDecoderConfig& FFmpegDemuxerStream::video_decoder_config() {
+  CHECK_EQ(type_, VIDEO);
+  return video_config_;
 }
 
 // static
 base::TimeDelta FFmpegDemuxerStream::ConvertStreamTimestamp(
     const AVRational& time_base, int64 timestamp) {
   if (timestamp == static_cast<int64>(AV_NOPTS_VALUE))
-    return kNoTimestamp;
+    return kNoTimestamp();
 
-  return ConvertTimestamp(time_base, timestamp);
+  return ConvertFromTimeBase(time_base, timestamp);
 }
 
 //
 // FFmpegDemuxer
 //
-FFmpegDemuxer::FFmpegDemuxer(MessageLoop* message_loop)
+FFmpegDemuxer::FFmpegDemuxer(MessageLoop* message_loop, bool local_source)
     : message_loop_(message_loop),
+      local_source_(local_source),
       format_context_(NULL),
       read_event_(false, false),
       read_has_failed_(false),
       last_read_bytes_(0),
-      read_position_(0) {
+      read_position_(0),
+      max_duration_(base::TimeDelta::FromMicroseconds(-1)),
+      deferred_status_(PIPELINE_OK),
+      first_seek_hack_(true),
+      start_time_(kNoTimestamp()),
+      audio_disabled_(false) {
   DCHECK(message_loop_);
 }
 
@@ -258,83 +303,87 @@ FFmpegDemuxer::~FFmpegDemuxer() {
   if (!format_context_)
     return;
 
-  // Iterate each stream and destroy each one of them.
-  int streams = format_context_->nb_streams;
-  for (int i = 0; i < streams; ++i) {
-    AVStream* stream = format_context_->streams[i];
-
-    // The conditions for calling avcodec_close():
-    // 1. AVStream is alive.
-    // 2. AVCodecContext in AVStream is alive.
-    // 3. AVCodec in AVCodecContext is alive.
-    // Notice that closing a codec context without prior avcodec_open() will
-    // result in a crash in FFmpeg.
-    if (stream && stream->codec && stream->codec->codec) {
-      stream->discard = AVDISCARD_ALL;
-      avcodec_close(stream->codec);
-    }
-  }
-
-  // Then finally cleanup the format context.
-  av_close_input_file(format_context_);
+  DestroyAVFormatContext(format_context_);
   format_context_ = NULL;
 }
 
 void FFmpegDemuxer::PostDemuxTask() {
   message_loop_->PostTask(FROM_HERE,
-      NewRunnableMethod(this, &FFmpegDemuxer::DemuxTask));
+                          base::Bind(&FFmpegDemuxer::DemuxTask, this));
 }
 
-void FFmpegDemuxer::Stop(FilterCallback* callback) {
+void FFmpegDemuxer::Stop(const base::Closure& callback) {
   // Post a task to notify the streams to stop as well.
   message_loop_->PostTask(FROM_HERE,
-      NewRunnableMethod(this, &FFmpegDemuxer::StopTask, callback));
+                          base::Bind(&FFmpegDemuxer::StopTask, this, callback));
 
   // Then wakes up the thread from reading.
   SignalReadCompleted(DataSource::kReadError);
 }
 
-void FFmpegDemuxer::Seek(base::TimeDelta time, FilterCallback* callback) {
-  // TODO(hclam): by returning from this method, it is assumed that the seek
-  // operation is completed and filters behind the demuxer is good to issue
-  // more reads, but we are posting a task here, which makes the seek operation
-  // asynchronous, should change how seek works to make it fully asynchronous.
+void FFmpegDemuxer::Seek(base::TimeDelta time, const PipelineStatusCB& cb) {
   message_loop_->PostTask(FROM_HERE,
-      NewRunnableMethod(this, &FFmpegDemuxer::SeekTask, time, callback));
+                          base::Bind(&FFmpegDemuxer::SeekTask, this, time, cb));
+}
+
+void FFmpegDemuxer::SetPlaybackRate(float playback_rate) {
+  DCHECK(data_source_.get());
+  data_source_->SetPlaybackRate(playback_rate);
+}
+
+void FFmpegDemuxer::SetPreload(Preload preload) {
+  DCHECK(data_source_.get());
+  data_source_->SetPreload(preload);
 }
 
 void FFmpegDemuxer::OnAudioRendererDisabled() {
-  message_loop_->PostTask(FROM_HERE,
-      NewRunnableMethod(this, &FFmpegDemuxer::DisableAudioStreamTask));
+  message_loop_->PostTask(FROM_HERE, base::Bind(
+      &FFmpegDemuxer::DisableAudioStreamTask, this));
+}
+
+void FFmpegDemuxer::set_host(DemuxerHost* demuxer_host) {
+  Demuxer::set_host(demuxer_host);
+  if (data_source_)
+    data_source_->set_host(demuxer_host);
+  if (max_duration_.InMicroseconds() >= 0)
+    host()->SetDuration(max_duration_);
+  if (read_position_ > 0)
+    host()->SetCurrentReadPosition(read_position_);
+  if (deferred_status_ != PIPELINE_OK)
+    host()->OnDemuxerError(deferred_status_);
 }
 
 void FFmpegDemuxer::Initialize(DataSource* data_source,
-                               FilterCallback* callback) {
+                               const PipelineStatusCB& callback) {
   message_loop_->PostTask(
       FROM_HERE,
-      NewRunnableMethod(this,
-                        &FFmpegDemuxer::InitializeTask,
-                        make_scoped_refptr(data_source),
-                        callback));
+      base::Bind(&FFmpegDemuxer::InitializeTask, this,
+                 make_scoped_refptr(data_source),
+                 callback));
 }
 
-size_t FFmpegDemuxer::GetNumberOfStreams() {
-  return streams_.size();
+scoped_refptr<DemuxerStream> FFmpegDemuxer::GetStream(
+    DemuxerStream::Type type) {
+  StreamVector::iterator iter;
+  for (iter = streams_.begin(); iter != streams_.end(); ++iter) {
+    if (*iter && (*iter)->type() == type) {
+      return *iter;
+    }
+  }
+  return NULL;
 }
 
-scoped_refptr<DemuxerStream> FFmpegDemuxer::GetStream(int stream) {
-  DCHECK_GE(stream, 0);
-  DCHECK_LT(stream, static_cast<int>(streams_.size()));
-  return streams_[stream].get();
+base::TimeDelta FFmpegDemuxer::GetStartTime() const {
+  return start_time_;
 }
 
-int FFmpegDemuxer::Read(int size, uint8* data) {
+size_t FFmpegDemuxer::Read(size_t size, uint8* data) {
   DCHECK(data_source_);
 
   // If read has ever failed, return with an error.
   // TODO(hclam): use a more meaningful constant as error.
   if (read_has_failed_)
-    return AVERROR_IO;
+    return AVERROR(EIO);
 
   // Even though FFmpeg defines AVERROR_EOF, it's not to be used with I/O
   // routines.  Instead return 0 for any read at or past EOF.
@@ -344,22 +393,26 @@ int FFmpegDemuxer::Read(int size, uint8* data) {
 
   // Asynchronous read from data source.
   data_source_->Read(read_position_, size, data,
-                     NewCallback(this, &FFmpegDemuxer::OnReadCompleted));
+                     base::Bind(&FFmpegDemuxer::OnReadCompleted, this));
 
   // TODO(hclam): The method is called on the demuxer thread and this method
   // call will block the thread. We need to implemented an additional thread to
   // let FFmpeg demuxer methods to run on.
   size_t last_read_bytes = WaitForRead();
   if (last_read_bytes == DataSource::kReadError) {
-    host()->SetError(PIPELINE_ERROR_READ);
+    if (host())
+      host()->OnDemuxerError(PIPELINE_ERROR_READ);
+    else
+      deferred_status_ = PIPELINE_ERROR_READ;
 
     // Returns with a negative number to signal an error to FFmpeg.
     read_has_failed_ = true;
-    return AVERROR_IO;
+    return AVERROR(EIO);
   }
   read_position_ += last_read_bytes;
 
-  host()->SetCurrentReadPosition(read_position_);
+  if (host())
+    host()->SetCurrentReadPosition(read_position_);
 
   return last_read_bytes;
 }
@@ -373,9 +426,10 @@ bool FFmpegDemuxer::SetPosition(int64 position) {
   DCHECK(data_source_);
 
   int64 file_size;
-  if (!data_source_->GetSize(&file_size) || position >= file_size ||
-      position < 0)
+  if ((data_source_->GetSize(&file_size) && position >= file_size) ||
+      position < 0) {
     return false;
+  }
 
   read_position_ = position;
   return true;
@@ -398,11 +452,12 @@ MessageLoop* FFmpegDemuxer::message_loop() {
 }
 
 void FFmpegDemuxer::InitializeTask(DataSource* data_source,
-                                   FilterCallback* callback) {
+                                   const PipelineStatusCB& callback) {
   DCHECK_EQ(MessageLoop::current(), message_loop_);
-  scoped_ptr<FilterCallback> c(callback);
 
   data_source_ = data_source;
+  if (host())
+    data_source_->set_host(host());
 
   // Add ourself to Protocol list and get our unique key.
   std::string key = FFmpegGlue::GetInstance()->AddProtocol(this);
@@ -416,8 +471,7 @@ void FFmpegDemuxer::InitializeTask(DataSource* data_source,
   FFmpegGlue::GetInstance()->RemoveProtocol(this);
 
   if (result < 0) {
-    host()->SetError(DEMUXER_ERROR_COULD_NOT_OPEN);
-    callback->Run();
+    callback.Run(DEMUXER_ERROR_COULD_NOT_OPEN);
     return;
   }
 
@@ -427,69 +481,141 @@ void FFmpegDemuxer::InitializeTask(DataSource* data_source,
   // Fully initialize AVFormatContext by parsing the stream a little.
   result = av_find_stream_info(format_context_);
   if (result < 0) {
-    host()->SetError(DEMUXER_ERROR_COULD_NOT_PARSE);
-    callback->Run();
+    callback.Run(DEMUXER_ERROR_COULD_NOT_PARSE);
     return;
   }
 
-  // Create demuxer streams for all supported streams.
+  // Create demuxer stream entries for each possible AVStream.
+  streams_.resize(format_context_->nb_streams);
+  bool found_audio_stream = false;
+  bool found_video_stream = false;
+
   base::TimeDelta max_duration;
-  const bool kDemuxerIsWebm = !strcmp("webm", format_context_->iformat->name);
   for (size_t i = 0; i < format_context_->nb_streams; ++i) {
     AVCodecContext* codec_context = format_context_->streams[i]->codec;
-    CodecType codec_type = codec_context->codec_type;
-    if (codec_type == CODEC_TYPE_AUDIO || codec_type == CODEC_TYPE_VIDEO) {
-      AVStream* stream = format_context_->streams[i];
-      // WebM is currently strictly VP8 and Vorbis.
-      if (kDemuxerIsWebm && (stream->codec->codec_id != CODEC_ID_VP8 &&
-        stream->codec->codec_id != CODEC_ID_VORBIS)) {
-        packet_streams_.push_back(NULL);
+    AVMediaType codec_type = codec_context->codec_type;
+
+    if (codec_type == AVMEDIA_TYPE_AUDIO) {
+      if (found_audio_stream)
         continue;
-      }
-
-      FFmpegDemuxerStream* demuxer_stream
-          = new FFmpegDemuxerStream(this, stream);
-
-      DCHECK(demuxer_stream);
-      streams_.push_back(make_scoped_refptr(demuxer_stream));
-      packet_streams_.push_back(make_scoped_refptr(demuxer_stream));
-      max_duration = std::max(max_duration, demuxer_stream->duration());
+      found_audio_stream = true;
+    } else if (codec_type == AVMEDIA_TYPE_VIDEO) {
+      if (found_video_stream)
+        continue;
+      found_video_stream = true;
     } else {
-      packet_streams_.push_back(NULL);
+      continue;
+    }
+
+    AVStream* stream = format_context_->streams[i];
+    scoped_refptr<FFmpegDemuxerStream> demuxer_stream(
+        new FFmpegDemuxerStream(this, stream));
+
+    streams_[i] = demuxer_stream;
+    max_duration = std::max(max_duration, demuxer_stream->duration());
+
+    if (stream->first_dts != static_cast<int64_t>(AV_NOPTS_VALUE)) {
+      const base::TimeDelta first_dts = ConvertFromTimeBase(
+          stream->time_base, stream->first_dts);
+      if (start_time_ == kNoTimestamp() || first_dts < start_time_)
+        start_time_ = first_dts;
     }
   }
-  if (streams_.empty()) {
-    host()->SetError(DEMUXER_ERROR_NO_SUPPORTED_STREAMS);
-    callback->Run();
+
+  if (!found_audio_stream && !found_video_stream) {
+    callback.Run(DEMUXER_ERROR_NO_SUPPORTED_STREAMS);
     return;
   }
+
   if (format_context_->duration != static_cast<int64_t>(AV_NOPTS_VALUE)) {
     // If there is a duration value in the container use that to find the
     // maximum between it and the duration from A/V streams.
     const AVRational av_time_base = {1, AV_TIME_BASE};
     max_duration =
         std::max(max_duration,
-                 ConvertTimestamp(av_time_base, format_context_->duration));
+                 ConvertFromTimeBase(av_time_base, format_context_->duration));
   } else {
-    // If the duration is not a valid value. Assume that this is a live stream
-    // and we set duration to the maximum int64 number to represent infinity.
-    max_duration = base::TimeDelta::FromMicroseconds(
-        Limits::kMaxTimeInMicroseconds);
+    // The duration is unknown, in which case this is likely a live stream.
+    max_duration = kInfiniteDuration();
   }
 
-  // Good to go: set the duration and notify we're done initializing.
-  host()->SetDuration(max_duration);
-  callback->Run();
+  // Some demuxers, like WAV, do not put timestamps on their frames. We
+  // assume the the start time is 0.
+  if (start_time_ == kNoTimestamp())
+    start_time_ = base::TimeDelta();
+
+  // Good to go: set the duration and bitrate and notify we're done
+  // initializing.
+  if (host())
+    host()->SetDuration(max_duration);
+  max_duration_ = max_duration;
+
+  int bitrate = GetBitrate();
+  if (bitrate > 0)
+    data_source_->SetBitrate(bitrate);
+
+  callback.Run(PIPELINE_OK);
 }
 
-void FFmpegDemuxer::SeekTask(base::TimeDelta time, FilterCallback* callback) {
+int FFmpegDemuxer::GetBitrate() {
+  DCHECK(format_context_);
+
+  // If there is a bitrate set on the container, use it.
+  if (format_context_->bit_rate > 0)
+    return format_context_->bit_rate;
+
+  // Then try to sum the bitrates individually per stream.
+  int bitrate = 0;
+  for (size_t i = 0; i < format_context_->nb_streams; ++i) {
+    AVCodecContext* codec_context = format_context_->streams[i]->codec;
+    bitrate += codec_context->bit_rate;
+  }
+  if (bitrate > 0)
+    return bitrate;
+
+  // See if we can approximate the bitrate as long as we have a filesize and
+  // valid duration.
+  int64 filesize_in_bytes;
+  if (max_duration_.InMicroseconds() <= 0 ||
+      max_duration_ == kInfiniteDuration() ||
+      !GetSize(&filesize_in_bytes)) {
+    return 0;
+  }
+
+  // Do math in floating point as we'd overflow an int64 if the filesize was
+  // larger than ~1073GB.
+  double bytes = filesize_in_bytes;
+  double duration = max_duration_.InMicroseconds();
+  return bytes * 8000000.0 / duration;
+}
+
+bool FFmpegDemuxer::IsLocalSource() {
+  return local_source_;
+}
+
+bool FFmpegDemuxer::IsSeekable() {
+  return !IsStreaming();
+}
+
+void FFmpegDemuxer::SeekTask(base::TimeDelta time, const PipelineStatusCB& cb) {
   DCHECK_EQ(MessageLoop::current(), message_loop_);
-  scoped_ptr<FilterCallback> c(callback);
+
+  // TODO(scherkus): remove this by separating Seek() from Flush() from
+  // Preroll() states (i.e., the implicit Seek(0) should really be a Preroll()).
+  if (first_seek_hack_) {
+    first_seek_hack_ = false;
+
+    if (time == start_time_) {
+      cb.Run(PIPELINE_OK);
+      return;
+    }
+  }
 
   // Tell streams to flush buffers due to seeking.
   StreamVector::iterator iter;
   for (iter = streams_.begin(); iter != streams_.end(); ++iter) {
-    (*iter)->FlushBuffers();
+    if (*iter)
+      (*iter)->FlushBuffers();
   }
 
   // Always seek to a timestamp less than or equal to the desired timestamp.
@@ -506,7 +632,7 @@ void FFmpegDemuxer::SeekTask(base::TimeDelta time, FilterCallback* callback) {
   }
 
   // Notify we're finished seeking.
-  callback->Run();
+  cb.Run(PIPELINE_OK);
 }
 
 void FFmpegDemuxer::DemuxTask() {
@@ -532,26 +658,25 @@ void FFmpegDemuxer::DemuxTask() {
   // worried about downstream filters (i.e., decoders) executing on this
   // thread.
   DCHECK_GE(packet->stream_index, 0);
-  DCHECK_LT(packet->stream_index, static_cast<int>(packet_streams_.size()));
-  FFmpegDemuxerStream* demuxer_stream = NULL;
-  size_t i = packet->stream_index;
+  DCHECK_LT(packet->stream_index, static_cast<int>(streams_.size()));
+
   // Defend against ffmpeg giving us a bad stream index.
-  if (i < packet_streams_.size()) {
-    demuxer_stream = packet_streams_[i];
-  }
-  if (demuxer_stream) {
-    // Queue the packet with the appropriate stream.  The stream takes
-    // ownership of the AVPacket.
-    if (packet.get()) {
-      // If a packet is returned by FFmpeg's av_parser_parse2()
-      // the packet will reference an inner memory of FFmpeg.
-      // In this case, the packet's "destruct" member is NULL,
-      // and it MUST be duplicated. This fixes issue with MP3 and possibly
-      // other codecs.  It is safe to call this function even if the packet does
-      // not refer to inner memory from FFmpeg.
-      av_dup_packet(packet.get());
-      demuxer_stream->EnqueuePacket(packet.release());
-    }
+  if (packet->stream_index >= 0 &&
+      packet->stream_index < static_cast<int>(streams_.size()) &&
+      streams_[packet->stream_index] &&
+      (!audio_disabled_ ||
+       streams_[packet->stream_index]->type() != DemuxerStream::AUDIO)) {
+    FFmpegDemuxerStream* demuxer_stream = streams_[packet->stream_index];
+
+    // If a packet is returned by FFmpeg's av_parser_parse2()
+    // the packet will reference an inner memory of FFmpeg.
+    // In this case, the packet's "destruct" member is NULL,
+    // and it MUST be duplicated. This fixes issue with MP3 and possibly
+    // other codecs.  It is safe to call this function even if the packet does
+    // not refer to inner memory from FFmpeg.
+    av_dup_packet(packet.get());
+
+    demuxer_stream->EnqueuePacket(packet.Pass());
   }
 
   // Create a loop by posting another task.  This allows seek and message loop
@@ -561,32 +686,27 @@ void FFmpegDemuxer::DemuxTask() {
   }
 }
 
-void FFmpegDemuxer::StopTask(FilterCallback* callback) {
+void FFmpegDemuxer::StopTask(const base::Closure& callback) {
   DCHECK_EQ(MessageLoop::current(), message_loop_);
   StreamVector::iterator iter;
   for (iter = streams_.begin(); iter != streams_.end(); ++iter) {
-    (*iter)->Stop();
+    if (*iter)
+      (*iter)->Stop();
   }
-  if (callback) {
-    callback->Run();
-    delete callback;
+  if (data_source_) {
+    data_source_->Stop(callback);
+  } else {
+    callback.Run();
   }
 }
 
 void FFmpegDemuxer::DisableAudioStreamTask() {
   DCHECK_EQ(MessageLoop::current(), message_loop_);
-
+  audio_disabled_ = true;
   StreamVector::iterator iter;
-  for (size_t i = 0; i < packet_streams_.size(); ++i) {
-    if (!packet_streams_[i])
-      continue;
-
-    // If the codec type is audio, remove the reference. DemuxTask() will
-    // look for such reference, and this will result in deleting the
-    // audio packets after they are demuxed.
-    if (packet_streams_[i]->GetAVStream()->codec->codec_type ==
-        CODEC_TYPE_AUDIO) {
-      packet_streams_[i] = NULL;
+  for (iter = streams_.begin(); iter != streams_.end(); ++iter) {
+    if (*iter && (*iter)->type() == DemuxerStream::AUDIO) {
+      (*iter)->Stop();
     }
   }
 }
@@ -595,7 +715,7 @@ bool FFmpegDemuxer::StreamsHavePendingReads() {
   DCHECK_EQ(MessageLoop::current(), message_loop_);
   StreamVector::iterator iter;
   for (iter = streams_.begin(); iter != streams_.end(); ++iter) {
-    if ((*iter)->HasPendingReads()) {
+    if (*iter && (*iter)->HasPendingReads()) {
       return true;
     }
   }
@@ -606,9 +726,13 @@ void FFmpegDemuxer::StreamHasEnded() {
   DCHECK_EQ(MessageLoop::current(), message_loop_);
   StreamVector::iterator iter;
   for (iter = streams_.begin(); iter != streams_.end(); ++iter) {
-    AVPacket* packet = new AVPacket();
-    memset(packet, 0, sizeof(*packet));
-    (*iter)->EnqueuePacket(packet);
+    if (!*iter ||
+        (audio_disabled_ && (*iter)->type() == DemuxerStream::AUDIO)) {
+      continue;
+    }
+    scoped_ptr_malloc<AVPacket, ScopedPtrAVFreePacket> packet(new AVPacket());
+    memset(packet.get(), 0, sizeof(*packet.get()));
+    (*iter)->EnqueuePacket(packet.Pass());
   }
 }
 
