@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,27 +9,29 @@
 
 #include "base/gtest_prod_util.h"
 #include "base/synchronization/lock.h"
-#include "media/base/filters.h"
+#include "media/base/data_source.h"
 
 namespace media {
 
 // Basic data source that treats the URL as a file path, and uses the file
 // system to read data for a media pipeline.
-class FileDataSource : public DataSource {
+class MEDIA_EXPORT FileDataSource : public DataSource {
  public:
   FileDataSource();
+  FileDataSource(bool disable_file_size);
   virtual ~FileDataSource();
 
-  // Implementation of Filter.
-  virtual void Stop(FilterCallback* callback);
+  PipelineStatus Initialize(const std::string& url);
 
   // Implementation of DataSource.
-  virtual void Initialize(const std::string& url, FilterCallback* callback);
-  virtual const MediaFormat& media_format();
+  virtual void set_host(DataSourceHost* host) OVERRIDE;
+  virtual void Stop(const base::Closure& callback) OVERRIDE;
   virtual void Read(int64 position, size_t size, uint8* data,
-                    ReadCallback* read_callback);
-  virtual bool GetSize(int64* size_out);
-  virtual bool IsStreaming();
+                    const DataSource::ReadCallback& read_callback) OVERRIDE;
+  virtual bool GetSize(int64* size_out) OVERRIDE;
+  virtual bool IsStreaming() OVERRIDE;
+  virtual void SetPreload(Preload preload) OVERRIDE;
+  virtual void SetBitrate(int bitrate) OVERRIDE;
 
  private:
   // Only allow factories and tests to create this object.
@@ -40,14 +42,18 @@ class FileDataSource : public DataSource {
   FRIEND_TEST_ALL_PREFIXES(FileDataSourceTest, ReadData);
   FRIEND_TEST_ALL_PREFIXES(FileDataSourceTest, Seek);
 
+  // Informs the host of changes in total and buffered bytes.
+  void UpdateHostBytes();
+
   // File handle.  NULL if not initialized or an error occurs.
   FILE* file_;
 
   // Size of the file in bytes.
   int64 file_size_;
 
-  // Media format handed out by the DataSource::GetMediaFormat method.
-  MediaFormat media_format_;
+  // True if the FileDataSource should ignore its set file size, false
+  // otherwise.
+  bool disable_file_size_;
 
   // Critical section that protects all of the DataSource methods to prevent
   // a Stop from happening while in the middle of a file I/O operation.
