@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,12 +9,13 @@
 #include <process.h>
 #include <string.h>
 
+#include "base/bind.h"
 #include "base/logging.h"
 #include "media/base/video_frame.h"
 #include "media/base/yuv_convert.h"
+#include "media/filters/video_renderer_base.h"
 #include "media/tools/player_wtl/movie.h"
 #include "media/tools/player_wtl/player_wtl.h"
-#include "media/tools/player_wtl/wtl_renderer.h"
 
 // Fetchs current time as milliseconds.
 // Returns as double for high duration and precision.
@@ -42,11 +43,22 @@ class WtlVideoWindow : public CScrollWindowImpl<WtlVideoWindow> {
     view_size_ = 2;  // Normal size.
     view_rotate_ = media::ROTATE_0;
     view_filter_ = media::FILTER_NONE;
-    renderer_ = new WtlVideoRenderer(this);
+    renderer_ = new media::VideoRendererBase(base::Bind(
+        &WtlVideoWindow::InvalidateWrapper, base::Unretained(this)),
+                                             base::Bind(
+        &WtlVideoWindow::SetOpaque, base::Unretained(this)));
     last_frame_ = NULL;
     hbmp_ = NULL;
   }
 
+  // Drops the bool return so we can use base::Bind().
+  void InvalidateWrapper() {
+    Invalidate();
+  }
+
+  void SetOpaque(bool /*opaque*/) {
+  }
+
   BOOL PreTranslateMessage(MSG* /*msg*/)  {
     return FALSE;
   }
@@ -195,7 +207,7 @@ class WtlVideoWindow : public CScrollWindowImpl<WtlVideoWindow> {
 
     bool enable_draw = media::Movie::GetInstance()->GetDrawEnable();
     if (enable_draw) {
-      DCHECK(bm.bmBitsPixel == 32);
+      DCHECK_EQ(32, bm.bmBitsPixel);
       DrawYUV(video_frame,
               movie_dib_bits,
               dibrowbytes,
@@ -228,14 +240,17 @@ class WtlVideoWindow : public CScrollWindowImpl<WtlVideoWindow> {
     if (!bmp_.IsNull()) {
       scoped_refptr<media::VideoFrame> frame;
       renderer_->GetCurrentFrame(&frame);
-      if (frame.get()) {
+      if (frame) {
+        // Size the window the first time we get a frame.
+        if (!last_frame_)
+          SetSize(frame->width(), frame->height());
+
         base::TimeDelta frame_timestamp = frame->GetTimestamp();
         if (frame != last_frame_ || frame_timestamp != last_timestamp_) {
           last_frame_ = frame;
           last_timestamp_ = frame_timestamp;
           ConvertFrame(frame);
         }
-        frame = NULL;
       }
       renderer_->PutCurrentFrame(frame);
 
@@ -357,9 +372,10 @@ class WtlVideoWindow : public CScrollWindowImpl<WtlVideoWindow> {
     hbmp_ = hbmp;
   }
 
-  CBitmap bmp_;  // Used by mainfrm.h.
-  SIZE size_;  // Used by WtlVideoWindow.
-  scoped_refptr<WtlVideoRenderer> renderer_;  // Used by WtlVideoWindow.
+  // Following member variables are accessed by CMainFrame.
+  CBitmap bmp_;
+  SIZE size_;
+  scoped_refptr<media::VideoRendererBase> renderer_;
 
  private:
   HBITMAP hbmp_;  // For Images
