@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,33 +10,31 @@
 #include <vector>
 
 #include "base/command_line.h"
-#include "base/scoped_vector.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/scoped_vector.h"
 #include "base/string_number_conversions.h"
 #include "base/time.h"
 #include "media/base/video_frame.h"
 #include "media/base/yuv_convert.h"
-#include "skia/ext/platform_canvas.h"
+#include "third_party/skia/include/core/SkCanvas.h"
 
 using base::TimeDelta;
 using base::TimeTicks;
 using media::VideoFrame;
 
-namespace {
+static int source_width = 1280;
+static int source_height = 720;
+static int dest_width = 1366;
+static int dest_height = 768;
+static int num_frames = 500;
+static int num_buffers = 50;
 
-int source_width = 1280;
-int source_height = 720;
-int dest_width = 1366;
-int dest_height = 768;
-int num_frames = 500;
-int num_buffers = 50;
-
-double BenchmarkSkia() {
+static double BenchmarkSkia() {
   std::vector<scoped_refptr<VideoFrame> > source_frames;
   ScopedVector<SkBitmap> dest_frames;
   for (int i = 0; i < num_buffers; i++) {
-    scoped_refptr<VideoFrame> source_frame;
-    VideoFrame::CreateBlackFrame(source_width, source_height, &source_frame);
-    source_frames.push_back(source_frame);
+    source_frames.push_back(
+        VideoFrame::CreateBlackFrame(source_width, source_height));
 
     SkBitmap* bitmap = new SkBitmap();
     bitmap->setConfig(SkBitmap::kARGB_8888_Config,
@@ -86,23 +84,48 @@ double BenchmarkSkia() {
   return static_cast<double>((end - start).InMilliseconds()) / num_frames;
 }
 
-double BenchmarkFilter(media::ScaleFilter filter) {
+static double BenchmarkRGBToYUV() {
+  int rgb_stride = source_width * 4;
+  scoped_array<uint8> rgb_frame(new uint8[rgb_stride * source_height]);
+
+  int y_stride = source_width;
+  int uv_stride = source_width / 2;
+  scoped_array<uint8> y_plane(new uint8[y_stride * source_height]);
+  scoped_array<uint8> u_plane(new uint8[uv_stride * source_height / 2]);
+  scoped_array<uint8> v_plane(new uint8[uv_stride * source_height / 2]);
+
+  TimeTicks start = TimeTicks::HighResNow();
+
+  for (int i = 0; i < num_frames; ++i) {
+    media::ConvertRGB32ToYUV(rgb_frame.get(),
+                             y_plane.get(),
+                             u_plane.get(),
+                             v_plane.get(),
+                             source_width,
+                             source_height,
+                             rgb_stride,
+                             y_stride,
+                             uv_stride);
+  }
+
+  TimeTicks end = TimeTicks::HighResNow();
+  return static_cast<double>((end - start).InMilliseconds()) / num_frames;
+}
+
+static double BenchmarkFilter(media::ScaleFilter filter) {
   std::vector<scoped_refptr<VideoFrame> > source_frames;
   std::vector<scoped_refptr<VideoFrame> > dest_frames;
 
   for (int i = 0; i < num_buffers; i++) {
-    scoped_refptr<VideoFrame> source_frame;
-    VideoFrame::CreateBlackFrame(source_width, source_height, &source_frame);
-    source_frames.push_back(source_frame);
+    source_frames.push_back(
+        VideoFrame::CreateBlackFrame(source_width, source_height));
 
-    scoped_refptr<VideoFrame> dest_frame;
-    VideoFrame::CreateFrame(VideoFrame::RGB32,
-                            dest_width,
-                            dest_height,
-                            TimeDelta::FromSeconds(0),
-                            TimeDelta::FromSeconds(0),
-                            &dest_frame);
-    dest_frames.push_back(dest_frame);
+    dest_frames.push_back(
+        VideoFrame::CreateFrame(VideoFrame::RGB32,
+                                dest_width,
+                                dest_height,
+                                TimeDelta::FromSeconds(0),
+                                TimeDelta::FromSeconds(0)));
   }
 
   TimeTicks start = TimeTicks::HighResNow();
@@ -129,13 +152,52 @@ double BenchmarkFilter(media::ScaleFilter filter) {
   return static_cast<double>((end - start).InMilliseconds()) / num_frames;
 }
 
-}  // namespace
+static double BenchmarkScaleWithRect() {
+  std::vector<scoped_refptr<VideoFrame> > source_frames;
+  std::vector<scoped_refptr<VideoFrame> > dest_frames;
+
+  for (int i = 0; i < num_buffers; i++) {
+    source_frames.push_back(
+        VideoFrame::CreateBlackFrame(source_width, source_height));
+
+    dest_frames.push_back(
+        VideoFrame::CreateFrame(VideoFrame::RGB32,
+                                dest_width,
+                                dest_height,
+                                TimeDelta::FromSeconds(0),
+                                TimeDelta::FromSeconds(0)));
+  }
+
+  TimeTicks start = TimeTicks::HighResNow();
+  for (int i = 0; i < num_frames; i++) {
+    scoped_refptr<VideoFrame> source_frame = source_frames[i % num_buffers];
+    scoped_refptr<VideoFrame> dest_frame = dest_frames[i % num_buffers];
+
+    media::ScaleYUVToRGB32WithRect(
+        source_frame->data(VideoFrame::kYPlane),
+        source_frame->data(VideoFrame::kUPlane),
+        source_frame->data(VideoFrame::kVPlane),
+        dest_frame->data(0),
+        source_width,
+        source_height,
+        dest_width,
+        dest_height,
+        0, 0,
+        dest_width,
+        dest_height,
+        source_frame->stride(VideoFrame::kYPlane),
+        source_frame->stride(VideoFrame::kUPlane),
+        dest_frame->stride(0));
+  }
+  TimeTicks end = TimeTicks::HighResNow();
+  return static_cast<double>((end - start).InMilliseconds()) / num_frames;
+}
 
 int main(int argc, const char** argv) {
   CommandLine::Init(argc, argv);
   const CommandLine* cmd_line = CommandLine::ForCurrentProcess();
 
-  if (!cmd_line->args().empty()) {
+  if (!cmd_line->GetArgs().empty()) {
     std::cerr << "Usage: " << argv[0] << " [OPTIONS]\n"
               << "  --frames=N                      "
               << "Number of frames\n"
@@ -198,6 +260,8 @@ int main(int argc, const char** argv) {
 
   std::cout << "Skia: " << BenchmarkSkia()
             << "ms/frame" << std::endl;
+  std::cout << "RGB To YUV: " << BenchmarkRGBToYUV()
+            << "ms/frame" << std::endl;
   std::cout << "No filtering: " << BenchmarkFilter(media::FILTER_NONE)
             << "ms/frame" << std::endl;
   std::cout << "Bilinear Vertical: "
@@ -208,6 +272,8 @@ int main(int argc, const char** argv) {
             << "ms/frame" << std::endl;
   std::cout << "Bilinear: " << BenchmarkFilter(media::FILTER_BILINEAR)
             << "ms/frame" << std::endl;
+  std::cout << "Bilinear with rect: " << BenchmarkScaleWithRect()
+            << "ms/frame" << std::endl;
 
   return 0;
 }
