@@ -1,6 +1,7 @@
 #!/usr/bin/python2.4
-# Copyright 2009, Google Inc.
-# All rights reserved.
+# Copyright (c) 2011 The Native Client Authors. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -32,7 +33,7 @@
 
 
 import SCons
-
+import library_deps
 
 __component_list = {}
 
@@ -124,6 +125,22 @@ def _ComponentPlatformSetup(env, builder_name, **kwargs):
   for h in env['INCLUDES']:
     env.Append(CCFLAGS=['${CCFLAG_INCLUDE}%s' % h])
 
+  # This supports a NaCl convention that was previously supported with a
+  # modification to SCons.  Previously, EXTRA_LIBS was interpolated into LIBS
+  # using the ${EXTRA_LIBS} syntax.  It appears, however, that SCons naturally
+  # computes library dependencies before interpolation, so EXTRA_LIBS will not
+  # be correctly depended upon if interpolated.  In the past, SCons was modified
+  # to force interpolation before library dependencies were computed.  This new
+  # approach allows us to use an unmodified version of SCons.
+  # In general, the use of EXTRA_LIBS is discouraged.
+  if 'EXTRA_LIBS' in env:
+    # The SubstList2 method expands and flattens so that scons will
+    # correctly know about the library dependencies in cases like
+    # EXTRA_LIBS=['${FOO_LIBS}', 'bar'].
+    env['LIBS'] = (library_deps.AddLibDeps(env['TARGET_FULLARCH'],
+                                           env.SubstList2('${EXTRA_LIBS}')) +
+                   env.SubstList2('${LIBS}'))
+
   # Call platform-specific component setup function, if any
   if env.get('COMPONENT_PLATFORM_SETUP'):
     env['COMPONENT_PLATFORM_SETUP'](env, builder_name)
@@ -249,6 +266,10 @@ def ComponentLibrary(self, lib_name, *args, **kwargs):
     lib_outputs = env.StaticLibrary(lib_name, *args, **kwargs)
   else:
     lib_outputs = env.SharedLibrary(lib_name, *args, **kwargs)
+    # TODO(robertm): arm hack, figure out a better way to do this
+    #                we should not be modifying the env as a side-effect
+    # BUG: http://code.google.com/p/nativeclient/issues/detail?id=2424
+    env.FilterOut(LINKFLAGS=['-static'])
 
   # Add dependencies on includes
   env.Depends(lib_outputs, env['INCLUDES'])
@@ -451,12 +472,26 @@ def ComponentProgram(self, prog_name, *args, **kwargs):
 
   env['PROGRAM_BASENAME'] = prog_name
 
+  if env['PROGSUFFIX'] and env.subst(prog_name).endswith(env['PROGSUFFIX']):
+    # Temporary hack: If there's already an extension, remove it.
+    # Because PPAPI is revision locked, and expects to be able to use .nexe
+    # TODO: When PPAPI deps is rolled, replace with this:
+    # raise Exception("Program name shouldn't have a suffix")
+    prog_name = env.subst(prog_name)
+    prog_name = prog_name[:-len(env['PROGSUFFIX'])]
+
   # Call env.Program()
   out_nodes = env.Program(prog_name, *args, **kwargs)
 
   # Add dependencies on includes
   env.Depends(out_nodes, env['INCLUDES'])
 
+  # Add dependencies on libraries marked as implicitly included in the link.
+  # These are libraries that are not passed on the command line, but are
+  # always linked in by the toolchain, i.e. startup files and -lc and such.
+  if 'IMPLICIT_LIBS' in env:
+    env.Depends(out_nodes, env['IMPLICIT_LIBS'])
+
   # Publish output
   env.Publish(prog_name, 'run', out_nodes[0])
   env.Publish(prog_name, 'debug', out_nodes[1:])
@@ -616,4 +651,3 @@ def generate(env):
   AddTargetGroup('run_small_tests', 'small tests can be run')
   AddTargetGroup('run_medium_tests', 'medium tests can be run')
   AddTargetGroup('run_large_tests', 'large tests can be run')
-
