@@ -175,8 +175,12 @@ def ComponentPlatformSetup(env, builder_name):
     )
 
   # Make sure link methods are lists, so we can append to them below
-  env['LINKCOM'] = [env['LINKCOM']]
-  env['SHLINKCOM'] = [env['SHLINKCOM']]
+  def MakeList(env, name):
+    if name in env:
+      env[name] = [env[name]]
+
+  MakeList(env, 'LINKCOM')
+  MakeList(env, 'SHLINKCOM')
 
   # Support manifest file generation and consumption
   if env.get('MANIFEST_FILE'):
@@ -261,9 +265,12 @@ def generate(env):
   # NOTE: SCons requires the use of this name, which fails gpylint.
   """SCons entry point for this tool."""
 
-  # TODO: Several sections here are gated out on the mac to prevent failure.
-  # This appears to be SCons issue 1720 manifesting itself on Mac when using
-  # windows tools like msvs or msvc.
+  # TODO(ncbray): Several sections here are gated out on to prevent failure on
+  # non-Windows platforms.  This appears to be SCons issue 1720 manifesting
+  # itself.  A more principled fix would be nice.
+
+  use_msvc_tools = (env['PLATFORM'] in ('win32', 'cygwin')
+                    and not env.Bit('built_elsewhere'))
 
   # Preserve some variables that get blown away by the tools.
   saved = dict()
@@ -278,17 +285,28 @@ def generate(env):
     env.AppendENVPath('LIB', os.environ.get('LIB', '[]'))
 
   # Load various Visual Studio related tools.
-  if env['PLATFORM'] != 'darwin':
+  if use_msvc_tools:
     env.Tool('as')
     env.Tool('msvs')
     env.Tool('windows_hard_link')
 
   pre_msvc_env = env['ENV'].copy()
 
-  if env['PLATFORM'] != 'darwin':
+  if use_msvc_tools:
     env.Tool('msvc')
     env.Tool('mslib')
     env.Tool('mslink')
+  else:
+    # Make sure we have all the builders even when MSVC is not available.
+    # Without these (fake) builders, SCons cannot be run on a Windows bot
+    # that does not have MSVC installed - even if MSVC is never invoked.
+    env.Tool('cc')
+    env.Tool('c++')
+    env.Tool('ar')
+    env.Tool('link')
+    def RES(*argv, **karg):
+      return []
+    env.AddMethod(RES)
 
   # Find VC80_DIR if it isn't already set.
   if not env.get('VC80_DIR'):
@@ -324,6 +342,7 @@ def generate(env):
       PDB='${TARGET.base}.pdb',
 
       # Code coverage related.
+      COVERAGE_CCFLAGS='-DCOVERAGE',
       COVERAGE_LINKFLAGS='/PROFILE',  # Requires vc_80 or higher.
       COVERAGE_SHLINKFLAGS='$COVERAGE_LINKFLAGS',
       # Change install step for coverage to cause instrumentation.
@@ -385,8 +404,6 @@ def generate(env):
   )
 
   env.Append(
-      HOST_PLATFORMS=['WINDOWS'],
-
       # Turn up the warning level
       CCFLAGS=['/W3'],
 
@@ -416,7 +433,7 @@ def generate(env):
   # TODO: mslink.py creates a shlibLinkAction which doesn't specify
   # '$SHLINKCOMSTR' as its command string.  This breaks --brief.  For now,
   # hack into the existing action and override its command string.
-  if env['PLATFORM'] != 'darwin':
+  if use_msvc_tools:
     env['SHLINKCOM'].list[0].cmdstr = '$SHLINKCOMSTR'
 
   # Restore saved flags.
