@@ -1,35 +1,9 @@
 /*
- * Copyright 2008, Google Inc.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- *     * Redistributions in binary form must reproduce the above
- * copyright notice, this list of conditions and the following disclaimer
- * in the documentation and/or other materials provided with the
- * distribution.
- *     * Neither the name of Google Inc. nor the names of its
- * contributors may be used to endorse or promote products derived from
- * this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
-
 /*
  * This file should be at the top of the #include group, followed by
  * standard system #include files, then by native client specific
@@ -51,6 +25,8 @@
 #include "native_client/src/trusted/service_runtime/include/bits/wordsize.h"
 #endif
 
+#include "native_client/src/include/nacl_compiler_annotations.h"
+
 #if NACL_WINDOWS
 /* disable warnings for deprecated functions like getenv, etc. */
 #pragma warning(disable : 4996)
@@ -64,55 +40,6 @@
 # include <sys/time.h>
 #endif  /*NACL_WINDOWS*/
 
-/* MSVC supports "inline" only in C++ */
-#if NACL_WINDOWS
-# define INLINE __forceinline
-#else
-# define INLINE __inline__
-#endif
-
-#if NACL_WINDOWS
-# define DLLEXPORT __declspec(dllexport)
-#elif defined(NACL_LINUX) || defined(NACL_OSX)
-# define DLLEXPORT __attribute__ ((visibility("default")))
-#elif defined(__native_client__)
-/* do nothing */
-#else
-# error "what platform?"
-#endif
-
-#if NACL_WINDOWS
-# define ATTRIBUTE_FORMAT_PRINTF(m, n)
-#else
-# define ATTRIBUTE_FORMAT_PRINTF(m, n) __attribute__((format(printf, m, n)))
-#endif
-
-#if NACL_WINDOWS
-# define UNREFERENCED_PARAMETER(P) (P)
-#else
-# define UNREFERENCED_PARAMETER(P) do { (void) P; } while (0)
-#endif
-
-#if NACL_WINDOWS
-# define NORETURN __declspec(noreturn)
-#else
-# define NORETURN __attribute__((noreturn))  /* assumes gcc */
-# define _cdecl /* empty */
-#endif
-
-#if NACL_WINDOWS
-# define THREAD __declspec(thread)
-#else
-# define THREAD __thread
-# define WINAPI
-#endif
-
-#if NACL_WINDOWS
-# define NACL_WUR
-#else
-# define NACL_WUR __attribute__((__warn_unused_result__))
-#endif
-
 /*
  * Per C99 7.8.14, define __STDC_CONSTANT_MACROS before including <stdint.h>
  * to get the INTn_C and UINTn_C macros for integer constants.  It's difficult
@@ -180,6 +107,8 @@ struct timezone {
 #define NACL_ARCH_CPU_32_BITS 1
 #define NACL_HOST_WORDSIZE 32
 #define NACL_WCHAR_T_IS_UNSIGNED 1
+#elif defined(__pnacl__)
+#define NACL_HOST_WORDSIZE 32
 #else
 #error Unrecognized host architecture
 #endif
@@ -212,7 +141,7 @@ struct timezone {
 #elif NACL_OSX
 # define  NACL___PRIS_PREFIX "l" /* -pedantic C++ programs w/ xcode */
 #elif defined(__native_client__)
-# define NACL___PRIS_PREFIX
+# define NACL___PRIS_PREFIX "z"
 #elif __WORDSIZE == 64
 # define NACL___PRIS_PREFIX "l"
 #else
@@ -241,55 +170,7 @@ struct timezone {
 /*
  * printf macros for intptr_t and uintptr_t, int{8,16,32,64}
  */
-#if !NACL_WINDOWS
-#ifndef __STDC_FORMAT_MACROS
-# define __STDC_FORMAT_MACROS  /* C++ */
-#endif
-# include <inttypes.h>
-
-#define NACL_PRIxPTR PRIxPTR
-#define NACL_PRIXPTR PRIXPTR
-#define NACL_PRIdPTR PRIdPTR
-
-#define NACL_PRId64 PRId64
-#define NACL_PRIu64 PRIu64
-#define NACL_PRIx64 PRIx64
-#define NACL_PRIX64 PRIX64
-
-#define NACL_PRIx32 PRIx32
-#define NACL_PRIX32 PRIX32
-#define NACL_PRId32 PRId32
-#define NACL_PRIu32 PRIu32
-
-#define NACL_PRId16 PRId16
-#define NACL_PRIu16 PRIu16
-#define NACL_PRIx16 PRIx16
-
-#define NACL_PRId8 PRId8
-#define NACL_PRIu8 PRIu8
-#define NACL_PRIx8 PRIx8
-
-
-# if NACL_OSX
-/*
- * OSX defines "hh" prefix for int8_t etc, but that's not standards
- * compliant -- --std=c++98 -Wall -Werror rejects it.
- */
-#  undef NACL_PRId8
-#  undef NACL_PRIi8
-#  undef NACL_PRIo8
-#  undef NACL_PRIu8
-#  undef NACL_PRIx8
-#  undef NACL_PRIX8
-#  define NACL_PRId8  "d"
-#  define NACL_PRIi8  "i"
-#  define NACL_PRIo8  "o"
-#  define NACL_PRIu8  "u"
-#  define NACL_PRIx8  "x"
-#  define NACL_PRIX8  "X"
-# endif
-#else
-/* NACL_WINDOWS */
+#if NACL_WINDOWS
 # if defined(_WIN64)
 #  define NACL___PRIPTR_PREFIX "I64"
 # else
@@ -346,14 +227,85 @@ struct timezone {
 # define NACL_PRIX64 NACL___PRI64_PREFIX "X"
 #endif
 
-#endif
+#else  /* NACL_LINUX, NACL_OSX, __native_client__ */
+
+# ifndef __STDC_FORMAT_MACROS
+#  define __STDC_FORMAT_MACROS  /* C++ */
+# endif
+
+# include <inttypes.h>
+
+# if __native_client__
+#  define NACL_PRIdPTR PRId32
+#  define NACL_PRIiPTR PRIi32
+#  define NACL_PRIoPTR PRIo32
+#  define NACL_PRIuPTR PRIu32
+#  define NACL_PRIxPTR PRIx32
+#  define NACL_PRIXPTR PRIX32
+# else
+#  define NACL_PRIdPTR PRIdPTR
+#  define NACL_PRIiPTR PRIiPTR
+#  define NACL_PRIoPTR PRIoPTR
+#  define NACL_PRIuPTR PRIuPTR
+#  define NACL_PRIxPTR PRIxPTR
+#  define NACL_PRIXPTR PRIXPTR
+# endif
+
+# define NACL_PRId64 PRId64
+# define NACL_PRIi64 PRIi64
+# define NACL_PRIo64 PRIo64
+# define NACL_PRIu64 PRIu64
+# define NACL_PRIx64 PRIx64
+# define NACL_PRIX64 PRIX64
+
+# define NACL_PRId32 PRId32
+# define NACL_PRIi32 PRIi32
+# define NACL_PRIo32 PRIo32
+# define NACL_PRIu32 PRIu32
+# define NACL_PRIx32 PRIx32
+# define NACL_PRIX32 PRIX32
+
+# define NACL_PRId16 PRId16
+# define NACL_PRIi16 PRIi16
+# define NACL_PRIo16 PRIo16
+# define NACL_PRIu16 PRIu16
+# define NACL_PRIx16 PRIx16
+# define NACL_PRIX16 PRIX16
+
+# define NACL_PRId8 PRId8
+# define NACL_PRIi8 PRIi8
+# define NACL_PRIo8 PRIo8
+# define NACL_PRIu8 PRIu8
+# define NACL_PRIx8 PRIx8
+# define NACL_PRIX8 PRIX8
+
+# if NACL_OSX
+/*
+ * OSX defines "hh" prefix for int8_t etc, but that's not standards
+ * compliant -- --std=c++98 -Wall -Werror rejects it.
+ */
+#  undef NACL_PRId8
+#  undef NACL_PRIi8
+#  undef NACL_PRIo8
+#  undef NACL_PRIu8
+#  undef NACL_PRIx8
+#  undef NACL_PRIX8
+#  define NACL_PRId8  "d"
+#  define NACL_PRIi8  "i"
+#  define NACL_PRIo8  "o"
+#  define NACL_PRIu8  "u"
+#  define NACL_PRIx8  "x"
+#  define NACL_PRIX8  "X"
+# endif  /* NACL_OSX */
+
+#endif  /* NACL_LINUX, NACL_OSX, __native_client__ */
 
 /*
  * macros for run-time error detectors (such as Valgrind/Memcheck).
  */
 #if defined(_DEBUG) && NACL_LINUX
 #include "native_client/src/third_party/valgrind/memcheck.h"
-#define NACL_MAKE_MEM_UNDEFINED(a, b) VALGRIND_MAKE_MEM_UNDEFINED(a, b)
+#define NACL_MAKE_MEM_UNDEFINED(a, b) (void) VALGRIND_MAKE_MEM_UNDEFINED(a, b)
 #else
 #define NACL_MAKE_MEM_UNDEFINED(a, b)
 #endif
