@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 The Native Client Authors. All rights reserved.
+ * Copyright (c) 2012 The Native Client Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -7,18 +7,19 @@
 #include "native_client/src/shared/srpc/nacl_srpc.h"
 #include "native_client/src/shared/srpc/nacl_srpc_internal.h"
 
-nacl_abi_size_t kNaClSrpcMaxImcSendmsgSize = 0;
+nacl_abi_size_t NaClSrpcMaxImcSendmsgSize = 0;
 
 int NaClSrpcModuleInit() {
 #ifdef __native_client__
   /* TODO(sehr): add call to sysconf. */
-  kNaClSrpcMaxImcSendmsgSize = 1 << 16;
+  NaClSrpcMaxImcSendmsgSize = 1 << 16;
 #else
   /* TODO(sehr): add call to trusted API. */
-  kNaClSrpcMaxImcSendmsgSize = 1 << 16;
+  NaClSrpcMaxImcSendmsgSize = 1 << 16;
 #endif  /* __native_client__ */
-  return 1;
+  return NaClSrpcLogInit();
 }
 
 void NaClSrpcModuleFini() {
+  NaClSrpcLogFini();
 }
