@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -86,7 +86,9 @@ enum NaClDescTypeTag {
   NACL_DESC_SYNC_SOCKET,
   NACL_DESC_TRANSFERABLE_DATA_SOCKET,
   NACL_DESC_IMC_SOCKET,
-  NACL_DESC_QUOTA
+  NACL_DESC_QUOTA,
+  NACL_DESC_DEVICE_RNG,
+  NACL_DESC_DEVICE_POSTMESSAGE
   /*
    * Add new NaClDesc subclasses here.
    *
@@ -94,7 +96,7 @@ enum NaClDescTypeTag {
    * also be updated to add new internalization functions.
    */
 };
-#define NACL_DESC_TYPE_MAX      (NACL_DESC_QUOTA + 1)
+#define NACL_DESC_TYPE_MAX      (NACL_DESC_DEVICE_POSTMESSAGE + 1)
 #define NACL_DESC_TYPE_END_TAG  (0xff)
 
 struct NaClInternalRealHeader {
@@ -321,7 +323,7 @@ struct NaClDescVtbl {
 };
 
 struct NaClDesc {
-  struct NaClRefCount base;
+  struct NaClRefCount base NACL_IS_REFCOUNT_SUBCLASS;
 };
 
 /*
