@@ -1,8 +1,8 @@
 
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -26,7 +26,7 @@ struct NaClDescXferState;
  */
 
 struct NaClDescDirDesc {
-  struct NaClDesc           base;
+  struct NaClDesc           base NACL_IS_REFCOUNT_SUBCLASS;
   struct NaClHostDir        *hd;
 };
 
