@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -25,11 +25,11 @@
 #include "native_client/src/shared/platform/nacl_log.h"
 #include "native_client/src/shared/platform/nacl_sync_checked.h"
 
-#include "native_client/src/trusted/service_runtime/internal_errno.h"
-#include "native_client/src/trusted/service_runtime/nacl_config.h"
 #include "native_client/src/trusted/service_runtime/include/sys/errno.h"
 #include "native_client/src/trusted/service_runtime/include/sys/mman.h"
 #include "native_client/src/trusted/service_runtime/include/sys/stat.h"
+#include "native_client/src/trusted/service_runtime/internal_errno.h"
+#include "native_client/src/trusted/service_runtime/nacl_config.h"
 
 #ifndef SIZE_T_MAX
 # define SIZE_T_MAX   (~(size_t) 0)
@@ -143,12 +143,8 @@ static uintptr_t NaClDescImcShmMap(struct NaClDesc         *vself,
   /* post-condition: if NULL == start_addr, then NACL_ABI_MAP_FIXED not set */
 
   /*
-   * prot must be not be PROT_NONE nor contain other than PROT_{READ|WRITE}
+   * prot must not contain bits other than PROT_{READ|WRITE|EXEC}.
    */
-  if (NACL_ABI_PROT_NONE == prot) {
-    NaClLog(LOG_INFO, "NaClDescImcShmMap: PROT_NONE not supported\n");
-    return -NACL_ABI_EINVAL;
-  }
   if (0 != (~(NACL_ABI_PROT_READ | NACL_ABI_PROT_WRITE | NACL_ABI_PROT_EXEC)
             & prot)) {
     NaClLog(LOG_INFO,
