@@ -1,7 +1,7 @@
 /*
- * Copyright 2009 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -74,10 +74,6 @@ static struct NaClDescVtbl const kNaClDescInvalidVtbl = {
   NaClDescGetValueNotImplemented,
 };
 
-struct NaClDescInvalid {
-  struct NaClDesc base;
-};
-
 static struct NaClMutex *mutex = NULL;
 static struct NaClDescInvalid *singleton;
 
