@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -23,7 +23,7 @@ struct NaClDescEffector;
 struct NaClDescXferState;
 
 struct NaClDescSemaphore {
-  struct NaClDesc      base;
+  struct NaClDesc      base NACL_IS_REFCOUNT_SUBCLASS;
   struct NaClSemaphore sem;
 };
 
