@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 The Native Client Authors. All rights reserved.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -29,7 +29,7 @@ struct NaClDescXferState;
 struct NaClMessageHeader;
 
 struct NaClDescSyncSocket {
-  struct NaClDesc           base;
+  struct NaClDesc           base NACL_IS_REFCOUNT_SUBCLASS;
   NaClHandle                h;
 };
 
