@@ -1,11 +1,11 @@
-// Copyright (c) 2009 The Native Client Authors. All rights reserved.
+// Copyright (c) 2011 The Native Client Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef NATIVE_CLIENT_SRC_TRUSTED_DESC_NACL_DESC_WRAPPER_H_
 #define NATIVE_CLIENT_SRC_TRUSTED_DESC_NACL_DESC_WRAPPER_H_
 
-#include "native_client/src/shared/platform/nacl_log.h"
+#include "native_client/src/include/nacl_base.h"
 #include "native_client/src/trusted/desc/nacl_desc_base.h"
 #include "native_client/src/trusted/desc/nrd_xfer.h"
 
@@ -33,6 +33,8 @@ class DescWrapperFactory {
   DescWrapper* MakeFileDesc(int host_os_desc, int mode);
   // Create a DescWrapper from opening a host file.
   DescWrapper* OpenHostFile(const char* fname, int flags, int mode);
+  // Create a DescWrapper for a random number generator.
+  DescWrapper* OpenRng();
   // Create a DescWrapper for the designated invalid descriptor
   DescWrapper* MakeInvalid();
 
@@ -46,7 +48,8 @@ class DescWrapperFactory {
   // We will doubtless want more specific factory methods.  For now,
   // we provide a wide-open method.
   DescWrapper* MakeGeneric(struct NaClDesc* desc);
-
+  // Same as above but unrefs desc in case of failure
+  DescWrapper* MakeGenericCleanup(struct NaClDesc* desc);
   // Utility routine for importing sync socket
   DescWrapper* ImportSyncSocketHandle(NaClHandle handle);
   // Utility routine for importing Linux/Mac (posix) and Windows shared memory.
