@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 
@@ -17,8 +17,10 @@
 #include <sys/param.h>
 #include <unistd.h>
 
+#include "native_client/src/include/nacl_macros.h"
 #include "native_client/src/include/nacl_string.h"
 #include "native_client/src/shared/platform/nacl_check.h"
+#include "native_client/src/shared/platform/nacl_exit.h"
 #include "native_client/src/shared/platform/nacl_log.h"
 #include "native_client/src/trusted/nonnacl_util/sel_ldr_launcher.h"
 #include "native_client/src/trusted/desc/nacl_desc_base.h"
@@ -30,15 +32,22 @@ namespace nacl {
 
 SelLdrLauncher::~SelLdrLauncher() {
   CloseHandlesAfterLaunch();
-  NaClDescSafeUnref(socket_address_);
   if (kInvalidHandle != child_process_) {
     int status;
+    // Ensure child process (service runtime) is kaput.  NB: we might
+    // close the command channel (or use the hard_shutdown RPC) rather
+    // than killing the process to allow the service runtime to do
+    // clean up, but the plugin should be responsible for that and we
+    // shouldn't introduce any timeout wait in a dtor.  Currently,
+    // ServiceRuntime::Shutdown kills the subprocess before closing
+    // the command channel, so we aren't providing the opportunity for
+    // a more graceful shutdown.
+    KillChildProcess();
     waitpid(child_process_, &status, 0);
   }
   if (kInvalidHandle != channel_) {
     Close(channel_);
   }
-  delete sel_ldr_locator_;
 }
 
 
@@ -48,6 +57,16 @@ nacl::string SelLdrLauncher::GetSelLdrPathName() {
   return nacl::string(buffer) + "/sel_ldr";
 }
 
+nacl::string SelLdrLauncher::GetSelLdrBootstrapPathName() {
+#if NACL_LINUX
+  char buffer[FILENAME_MAX];
+  GetPluginDirectory(buffer, sizeof(buffer));
+  return nacl::string(buffer) + "/nacl_helper_bootstrap";
+#else
+  return nacl::string(NACL_NO_FILE_PATH);
+#endif
+}
+
 Handle SelLdrLauncher::ExportImcFD(int dest_fd) {
   Handle pair[2];
   if (SocketPair(pair) == -1) {
@@ -66,11 +85,6 @@ Handle SelLdrLauncher::ExportImcFD(int dest_fd) {
 const size_t kMaxExecArgs = 64;
 
 bool SelLdrLauncher::LaunchFromCommandLine() {
-  // Uncomment to turn on the sandbox, or set this in your environment
-  // TODO(neha):  Turn this on by default.
-  //
-  // setenv("NACL_ENABLE_OUTER_SANDBOX");
-
   if (channel_number_ != -1) {
     channel_ = ExportImcFD(channel_number_);
   }
@@ -99,21 +113,29 @@ bool SelLdrLauncher::LaunchFromCommandLine() {
     for (size_t i = 0; i < command.size(); ++i) {
       argv[i] = command[i].c_str();
     }
-    argv[command.size()] = 0;
+    argv[command.size()] = NULL;
 
-    execv(sel_ldr_.c_str(), const_cast<char**>(argv));
+    execv(argv[0], const_cast<char**>(argv));
     NaClLog(LOG_ERROR, "execv failed, args were:\n");
     for (size_t i = 0; i < command.size(); ++i) {
       NaClLog(LOG_ERROR, "%s\n", argv[i]);
     }
     perror("execv");
-    _exit(EXIT_FAILURE);
+    NaClExit(EXIT_FAILURE);
   }
   CloseHandlesAfterLaunch();
   return true;
 }
 
 bool SelLdrLauncher::KillChildProcess() {
+  if (kInvalidHandle == child_process_) {
+    // It is incorrect to use the kill syscall on kInvalidHandle as
+    // the pid, since using -1 as pid is defined by POSIX.1-2001 to
+    // send the signal (SIGKILL) to every process that the calling
+    // process may send signals to (except for init), which is
+    // Definitely Not What Was Intended for this.
+    return true;
+  }
   return 0 == kill(child_process_, SIGKILL);
   // We cannot set child_process_ to kInvalidHandle since we will want to wait
   // on its exit status.
