@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 
@@ -12,15 +12,17 @@
 
 #include <vector>
 
+#include "native_client/src/include/nacl_scoped_ptr.h"
 #include "native_client/src/include/nacl_string.h"
 #include "native_client/src/include/portability.h"
 #include "native_client/src/shared/imc/nacl_imc.h"
+#include "native_client/src/trusted/desc/nacl_desc_wrapper.h"
+#include "native_client/src/trusted/service_runtime/nacl_error_code.h"
 
 struct NaClSrpcChannel;
 struct NaClDesc;
 
 namespace nacl {
-
 // TODO(robertm): Move this to new header if it becomes more popular.
 template <class T> nacl::string ToString(const T& t) {
   nacl::stringstream ss;
@@ -74,7 +76,7 @@ struct SelLdrLauncher {
   Handle child_process() const { return child_process_; }
 
   /////////////////////////////////////////////////////////////////////////////
-  // Command line start-up:
+  // Command line start-up: (Only used by sel_universal.)
   //
   // The command line must include a file path for the nexe application or an
   // indicator that a reference will be supplied after the launch over RPC.
@@ -85,9 +87,11 @@ struct SelLdrLauncher {
   // Returns the second socket. Returns kInvalidHandle on failure.
   Handle ExportImcFD(int dest_fd);
 
-  // |application_file| is empty if nexe reference will be passed over RPC.
-  void InitCommandLine(const nacl::string& application_file,
-                       int imc_fd,
+  // Sets up the command line to start a sel_ldr.  Specifies |imc_fd| to
+  // use for bootstrapping. |sel_ldr_argv| specifies the arguments to be
+  // passed to sel_ldr itself, while |application_argv| specifies the arguments
+  // to be passed to the nexe.
+  void InitCommandLine(int imc_fd,
                        const std::vector<nacl::string>& sel_ldr_argv,
                        const std::vector<nacl::string>& application_argv);
 
@@ -96,23 +100,21 @@ struct SelLdrLauncher {
   // You must call InitCommandLine() before calling this function.
   bool LaunchFromCommandLine();
 
-  // |application_file| is empty if nexe reference will be passed over RPC.
-  bool StartFromCommandLine(const nacl::string& application_file,
-                            int imc_fd,
-                            const std::vector<nacl::string>& sel_ldr_argv,
-                            const std::vector<nacl::string>& app_argv) {
-    InitCommandLine(application_file, imc_fd, sel_ldr_argv, app_argv);
-    return LaunchFromCommandLine();
-  }
-
   // Builds a command line out of the prepopulated args.
   void BuildCommandLine(std::vector<nacl::string>* command);
 
-  // OpenSrpcChannels essentially is a pair Ctor for the two
-  // NaClSrpcChannel objects; if it returns true (success), both were
-  // constructed; if it returns false, neither was constructed (and thus
-  // neither needs to be Dtor'd).
-  bool OpenSrpcChannels(NaClSrpcChannel* command, NaClSrpcChannel* untrusted);
+  // Sets up the command channel |command| and sends the SRPC to load |nexe|.
+  bool SetupCommandAndLoad(NaClSrpcChannel* command,
+                           DescWrapper* nexe);
+
+  // Sends the SRPC to start the nexe over |command| and sets up the application
+  // SRPC chanel |out_app_chan|.
+  bool StartModuleAndSetupAppChannel(NaClSrpcChannel* command,
+                                     NaClSrpcChannel* out_app_chan);
+
+  // Add a prefix shell program, like 'time', to run the sel_ldr in
+  // This is primarily intended to provide a hook for qemu emulation
+  void SetCommandPrefix(const nacl::string& prefix);
 
   // Kill the child process.  The channel() remains valid, but nobody
   // is talking on the other end.  Returns true if successful.
@@ -122,26 +124,51 @@ struct SelLdrLauncher {
   // ownership of the handle prior to the Dtor firing.
   Handle channel() const { return channel_; }
 
-  // Returns the file path of the NaCl module to be run within the sel_ldr.
-  nacl::string application_file() const { return application_file_; }
-
   // Returns the socket address used to connect to the sel_ldr.
-  struct NaClDesc* socket_address() const { return socket_address_; }
+  DescWrapper* socket_addr() const { return socket_addr_.get(); }
+
+  // Wraps a raw NaClDesc descriptor.  If NULL is returned, caller retains
+  // ownership of the reference.
+  DescWrapper* Wrap(NaClDesc* raw_desc);
+
+  // As above, but raw_desc is Unref'd on failure.
+  DescWrapper* WrapCleanup(NaClDesc* raw_desc);
 
   /////////////////////////////////////////////////////////////////////////////
-  // Browser-based start-up (Chrome only):
-  //
-  // A message is sent to the browser process to launch the sel_ldr process.
-  // No nexe information is passed at this point. It will be supplied over
-  // RPC after start-up.
+  // Start the sel_ldr process.  No nexe information is passed at this point.
+  // It will be supplied over RPC after start-up.
   /////////////////////////////////////////////////////////////////////////////
 
-  bool StartFromBrowser(const char* url, int socket_count,
-                        Handle* result_sockets);
+  bool Start(int socket_count, Handle* result_sockets);
 
  private:
+  // OpenSrpcChannels is essentially the following sequence of
+  // (lower-level) operations.
+
+  // BEGIN EQUIVALENT SEQUENCE
+
+  bool SetupBootstrapChannel();
+  bool GetLdrSocketAddress();
+  bool SetupCommandChannel(NaClSrpcChannel* command);
+
+  // LoadModule supplies, via the |command| channel, the |nexe| file
+  // for the service runtime to load.
+  bool LoadModule(NaClSrpcChannel* command, DescWrapper* nexe);
+
+  // ----
+
+  // Tell the service runtime to start the NaCl module via the
+  // |command| channel.  If |error| is non-NULL, any failures will
+  // cause the error code to be written there.
+  bool StartModule(NaClSrpcChannel* command, NaClErrorCode* error);
+
+  bool SetupApplicationChannel(NaClSrpcChannel* app_channel);
+
+  // END EQUIVALENT SEQUENCE
+
   void GetPluginDirectory(char* buffer, size_t len);
   nacl::string GetSelLdrPathName();
+  nacl::string GetSelLdrBootstrapPathName();
   void CloseHandlesAfterLaunch();
 
   Handle child_process_;
@@ -149,17 +176,26 @@ struct SelLdrLauncher {
   int channel_number_;  // IMC file descriptor.
 
   // The following members are used to initialize and build the command line.
+  // The detailed magic is in BuildCommandLine() but roughly we run
+  // <prefix> <sel_ldr> <extra stuff> -f <nexe> <sel_ldr_argv> -- <nexe_args>
+  // Path to prefix tool or empty if not used
+  nacl::string command_prefix_;
+  nacl::string sel_ldr_bootstrap_;
+  // Path to the sel_ldr executable
   nacl::string sel_ldr_;
-  nacl::string application_file_;
+  // arguments to sel_ldr
   std::vector<nacl::string> sel_ldr_argv_;
+  // arguments to the nexe
   std::vector<nacl::string> application_argv_;
 
   std::vector<Handle> close_after_launch_;
 
+  // lifetime of bootstrap_socket_ must be at least that of factory_
+  scoped_ptr<DescWrapperFactory> factory_;
+  scoped_ptr<DescWrapper> bootstrap_socket_;
   // The socket address returned from sel_ldr for connects.
-  struct NaClDesc* socket_address_;
-
-  SelLdrLocator* sel_ldr_locator_;
+  scoped_ptr<DescWrapper> socket_addr_;
+  scoped_ptr<SelLdrLocator> sel_ldr_locator_;
 };
 
 }  // namespace nacl
