@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 #include <string.h>
@@ -25,14 +25,21 @@ namespace nacl {
 
 SelLdrLauncher::~SelLdrLauncher() {
   CloseHandlesAfterLaunch();
-  NaClDescSafeUnref(socket_address_);
   if (kInvalidHandle != child_process_) {
+    // Ensure child process (service runtime) is kaput.  NB: we might
+    // close the command channel (or use the hard_shutdown RPC) rather
+    // than killing the process to allow the service runtime to do
+    // clean up, but the plugin should be responsible for that and we
+    // shouldn't introduce any timeout wait in a dtor.  Currently,
+    // ServiceRuntime::Shutdown kills the subprocess before closing
+    // the command channel, so we aren't providing the opportunity for
+    // a more graceful shutdown.
+    KillChildProcess();
     CloseHandle(child_process_);
   }
   if (kInvalidHandle != channel_) {
     Close(channel_);
   }
-  delete sel_ldr_locator_;
 }
 
 nacl::string SelLdrLauncher::GetSelLdrPathName() {
@@ -45,6 +52,10 @@ nacl::string SelLdrLauncher::GetSelLdrPathName() {
   return nacl::string(buffer) + kSelLdrBasename;
 }
 
+nacl::string SelLdrLauncher::GetSelLdrBootstrapPathName() {
+  return nacl::string(NACL_NO_FILE_PATH);
+}
+
 // TODO(sehr): document what this is supposed to do exactly
 // NOTE: this may be buggy, e.g. how is \\ handled?
 static nacl::string Escape(nacl::string s) {
