@@ -1,7 +1,7 @@
 /*
- * Copyright 2010 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -16,6 +16,13 @@
 #include "native_client/src/trusted/platform_qualify/nacl_dep_qualify.h"
 #include "native_client/src/trusted/platform_qualify/linux/sysv_shm_and_mmap.h"
 
+#if (NACL_OSX && NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && \
+     NACL_BUILD_SUBARCH == 64)
+# define EXPECTED_SIGNAL SIGBUS
+#else
+# define EXPECTED_SIGNAL SIGSEGV
+#endif
+
 static struct sigaction previous_sigaction;
 static struct sigaction try_sigaction;
 static sigjmp_buf try_state;
@@ -29,11 +36,11 @@ static void setup_signals() {
   sigemptyset(&try_sigaction.sa_mask);
   try_sigaction.sa_flags = SA_RESETHAND;
 
-  (void) sigaction(SIGSEGV, &try_sigaction, &previous_sigaction);
+  (void) sigaction(EXPECTED_SIGNAL, &try_sigaction, &previous_sigaction);
 }
 
 static void restore_signals() {
-  (void) sigaction(SIGSEGV, &previous_sigaction, 0);
+  (void) sigaction(EXPECTED_SIGNAL, &previous_sigaction, 0);
 }
 
 /*
@@ -42,7 +49,11 @@ static void restore_signals() {
 int NaClAttemptToExecuteData() {
   int result;
   char *thunk_buffer = malloc(64);
-  nacl_void_thunk thunk = NaClGenerateThunk(thunk_buffer, 64);
+  nacl_void_thunk thunk;
+  if (NULL == thunk_buffer) {
+    return 0;
+  }
+  thunk = NaClGenerateThunk(thunk_buffer, 64);
 
   setup_signals();
 
@@ -54,5 +65,6 @@ int NaClAttemptToExecuteData() {
   }
 
   restore_signals();
+  free(thunk_buffer);
   return result;
 }
