@@ -1,7 +1,7 @@
 /*
- * Copyright 2009 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -15,14 +15,23 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "native_client/src/include/elf_constants.h"
-#include "native_client/src/include/nacl_elf.h"
-#include "native_client/src/include/nacl_macros.h"
+#define NACL_LOG_MODULE_NAME  "elf_util"
 
+#include "native_client/src/include/elf_constants.h"
+#include "native_client/src/include/elf.h"
+#include "native_client/src/include/nacl_macros.h"
+#include "native_client/src/include/nacl_platform.h"
+
+#include "native_client/src/shared/gio/gio.h"
 #include "native_client/src/shared/platform/nacl_log.h"
 
 #include "native_client/src/trusted/service_runtime/elf_util.h"
+#include "native_client/src/trusted/service_runtime/include/bits/mman.h"
 #include "native_client/src/trusted/service_runtime/nacl_config.h"
+#include "native_client/src/trusted/service_runtime/nacl_syscall_common.h"
+#include "native_client/src/trusted/service_runtime/nacl_text.h"
+#include "native_client/src/trusted/service_runtime/nacl_valgrind_hooks.h"
+#include "native_client/src/trusted/service_runtime/sel_memory.h"
 
 /* private */
 struct NaClElfImage {
@@ -85,6 +94,20 @@ static const struct NaClPhdrChecks nacl_phdr_check_data[] = {
 };
 
 
+#if NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86
+# if NACL_BUILD_SUBARCH == 32
+#  define EM_EXPECTED_BY_NACL EM_386
+# elif NACL_BUILD_SUBARCH == 64
+#  define EM_EXPECTED_BY_NACL EM_X86_64
+# else
+#  error "No NACL_BUILD_SUBARCH for x86 -- are we on x86-128?"
+# endif
+#elif NACL_ARCH(NACL_BUILD_ARCH) == NACL_arm
+# define EM_EXPECTED_BY_NACL EM_ARM
+#else
+# error "Unknown platform!"
+#endif
+
 
 static void NaClDumpElfHeader(int loglevel, Elf_Ehdr *elf_hdr) {
 
@@ -138,40 +161,6 @@ static void NaClDumpElfProgramHeader(int     loglevel,
 }
 
 
-NaClErrorCode NaClElfImageValidateAbi(struct NaClElfImage *image) {
-  const Elf_Ehdr *hdr = &image->ehdr;
-
-  if (ELFOSABI_NACL != hdr->e_ident[EI_OSABI]) {
-    NaClLog(LOG_ERROR, "Expected OSABI %d, got %d\n",
-            ELFOSABI_NACL,
-            hdr->e_ident[EI_OSABI]);
-    return LOAD_BAD_ABI;
-  }
-
-  /*
-   * TODO(bsy,khim): e_ident[EI_ABIVERSION] for nacl64 should match nacl32.
-   */
-#if NACL_TARGET_SUBARCH == 64
-  if (EF_NACL_ABIVERSION != hdr->e_ident[EI_ABIVERSION]
-      && 0 != hdr->e_ident[EI_ABIVERSION]) {
-    NaClLog(LOG_ERROR, "Expected ABIVERSION %d, got %d\n",
-            EF_NACL_ABIVERSION,
-            hdr->e_ident[EI_ABIVERSION]);
-      return LOAD_BAD_ABI;
-  }
-#else
-  if (EF_NACL_ABIVERSION != hdr->e_ident[EI_ABIVERSION]) {
-    NaClLog(LOG_ERROR, "Expected ABIVERSION %d, got %d\n",
-            EF_NACL_ABIVERSION,
-            hdr->e_ident[EI_ABIVERSION]);
-      return LOAD_BAD_ABI;
-  }
-#endif
-
-  return LOAD_OK;
-}
-
-
 NaClErrorCode NaClElfImageValidateElfHeader(struct NaClElfImage *image) {
   const Elf_Ehdr *hdr = &image->ehdr;
 
@@ -180,17 +169,10 @@ NaClErrorCode NaClElfImageValidateElfHeader(struct NaClElfImage *image) {
     return LOAD_BAD_ELF_MAGIC;
   }
 
-#if NACL_TARGET_SUBARCH == 64
-  if (ELFCLASS64 != hdr->e_ident[EI_CLASS]) {
-    NaClLog(LOG_ERROR, "bad elf class\n");
-    return LOAD_NOT_64_BIT;
-  }
-#else
   if (ELFCLASS32 != hdr->e_ident[EI_CLASS]) {
     NaClLog(LOG_ERROR, "bad elf class\n");
     return LOAD_NOT_32_BIT;
   }
-#endif
 
   if (ET_EXEC != hdr->e_type) {
     NaClLog(LOG_ERROR, "non executable\n");
@@ -306,21 +288,18 @@ NaClErrorCode NaClElfImageValidateProgramHeaders(
               php->p_vaddr);
       return LOAD_SEGMENT_OUTSIDE_ADDRSPACE;
     }
-    /*
-     * integer overflow?  Elf_Addr and Elf_Word are uint32_t or
-     * uint64_t, so the addition/comparison is well defined.
-     */
-    if (php->p_vaddr + php->p_memsz < php->p_vaddr) {
-      NaClLog(2,
-              "Segment %d: p_memsz caused integer overflow\n",
-              segnum);
-      return LOAD_SEGMENT_OUTSIDE_ADDRSPACE;
-    }
-    if (php->p_vaddr + php->p_memsz >= ((Elf_Addr) 1U << addr_bits)) {
-      NaClLog(2,
-              "Segment %d: too large, ends at 0x%08"NACL_PRIxElf_Addr"\n",
-              segnum,
-              php->p_vaddr + php->p_memsz);
+    if (php->p_vaddr >= ((uint64_t) 1U << addr_bits) ||
+        ((uint64_t) 1U << addr_bits) - php->p_vaddr < php->p_memsz) {
+      if (php->p_vaddr + php->p_memsz < php->p_vaddr) {
+        NaClLog(2,
+                "Segment %d: p_memsz caused integer overflow\n",
+                segnum);
+      } else {
+        NaClLog(2,
+                "Segment %d: too large, ends at 0x%08"NACL_PRIxElf_Addr"\n",
+                segnum,
+                php->p_vaddr + php->p_memsz);
+      }
       return LOAD_SEGMENT_OUTSIDE_ADDRSPACE;
     }
     if (php->p_filesz > php->p_memsz) {
@@ -388,17 +367,32 @@ struct NaClElfImage *NaClElfImageNew(struct Gio     *gp,
                                      NaClErrorCode  *err_code) {
   struct NaClElfImage *result;
   struct NaClElfImage image;
+  union {
+    Elf32_Ehdr ehdr32;
+#if NACL_TARGET_SUBARCH == 64
+    Elf64_Ehdr ehdr64;
+#endif
+  } ehdr;
   int                 cur_ph;
 
   memset(image.loadable, 0, sizeof image.loadable);
   if (-1 == (*gp->vtbl->Seek)(gp, 0, 0)) {
     NaClLog(2, "could not seek to beginning of Gio object containing nexe\n");
+    if (NULL != err_code) {
+      *err_code = LOAD_READ_ERROR;
+    }
     return 0;
   }
+
+  /*
+   * We read the larger size of an ELFCLASS64 header even if it turns out
+   * we're reading an ELFCLASS32 file.  No usable ELFCLASS32 binary could
+   * be so small that it's not larger than Elf64_Ehdr anyway.
+   */
   if ((*gp->vtbl->Read)(gp,
-                        &image.ehdr,
-                        sizeof image.ehdr)
-      != sizeof image.ehdr) {
+                        &ehdr,
+                        sizeof ehdr)
+      != sizeof ehdr) {
     if (NULL != err_code) {
       *err_code = LOAD_READ_ERROR;
     }
@@ -406,6 +400,49 @@ struct NaClElfImage *NaClElfImageNew(struct Gio     *gp,
     return 0;
   }
 
+#if NACL_TARGET_SUBARCH == 64
+  if (ELFCLASS64 == ehdr.ehdr64.e_ident[EI_CLASS]) {
+    /*
+     * Convert ELFCLASS64 format to ELFCLASS32 format.
+     * The initial four fields are the same in both classes.
+     */
+    memcpy(image.ehdr.e_ident, ehdr.ehdr64.e_ident, EI_NIDENT);
+    image.ehdr.e_ident[EI_CLASS] = ELFCLASS32;
+    image.ehdr.e_type = ehdr.ehdr64.e_type;
+    image.ehdr.e_machine = ehdr.ehdr64.e_machine;
+    image.ehdr.e_version = ehdr.ehdr64.e_version;
+    if (ehdr.ehdr64.e_entry > 0xffffffffU ||
+        ehdr.ehdr64.e_phoff > 0xffffffffU ||
+        ehdr.ehdr64.e_shoff > 0xffffffffU) {
+      if (NULL != err_code) {
+        *err_code = LOAD_EHDR_OVERFLOW;
+      }
+      NaClLog(2, "ELFCLASS64 file header fields overflow 32 bits\n");
+      return 0;
+    }
+    image.ehdr.e_entry = (Elf32_Addr) ehdr.ehdr64.e_entry;
+    image.ehdr.e_phoff = (Elf32_Off) ehdr.ehdr64.e_phoff;
+    image.ehdr.e_shoff = (Elf32_Off) ehdr.ehdr64.e_shoff;
+    image.ehdr.e_flags = ehdr.ehdr64.e_flags;
+    if (ehdr.ehdr64.e_ehsize != sizeof(ehdr.ehdr64)) {
+      if (NULL != err_code) {
+        *err_code = LOAD_BAD_EHSIZE;
+      }
+      NaClLog(2, "ELFCLASS64 file e_ehsize != %d\n", (int) sizeof(ehdr.ehdr64));
+      return 0;
+    }
+    image.ehdr.e_ehsize = sizeof(image.ehdr);
+    image.ehdr.e_phentsize = sizeof(image.phdrs[0]);
+    image.ehdr.e_phnum = ehdr.ehdr64.e_phnum;
+    image.ehdr.e_shentsize = ehdr.ehdr64.e_shentsize;
+    image.ehdr.e_shnum = ehdr.ehdr64.e_shnum;
+    image.ehdr.e_shstrndx = ehdr.ehdr64.e_shstrndx;
+  } else
+#endif
+  {
+    image.ehdr = ehdr.ehdr32;
+  }
+
   NaClDumpElfHeader(2, &image.ehdr);
 
   /* read program headers */
@@ -416,23 +453,6 @@ struct NaClElfImage *NaClElfImageNew(struct Gio     *gp,
     return 0;
   }
 
-  if (image.ehdr.e_phentsize < sizeof image.phdrs[0]) {
-    if (NULL != err_code) {
-      *err_code = LOAD_PROG_HDR_SIZE_TOO_SMALL;
-    }
-    NaClLog(2, "bad prog headers size\n");
-    NaClLog(2, " image.ehdr.e_phentsize = 0x%"NACL_PRIxElf_Half"\n",
-            image.ehdr.e_phentsize);
-    NaClLog(2, "  sizeof image.phdrs[0] = 0x%"NACL_PRIxS"\n",
-            sizeof image.phdrs[0]);
-    return 0;
-  }
-
-  /*
-   * NB: cast from e_phoff to off_t may not be valid, since off_t can be
-   * smaller than Elf64_off, but since invalid values will be rejected
-   * by Seek() the cast is safe (cf bsy)
-   */
   if ((*gp->vtbl->Seek)(gp,
                         (off_t) image.ehdr.e_phoff,
                         SEEK_SET) == (off_t) -1) {
@@ -443,15 +463,87 @@ struct NaClElfImage *NaClElfImageNew(struct Gio     *gp,
     return 0;
   }
 
-  if ((size_t) (*gp->vtbl->Read)(gp,
-                                 &image.phdrs[0],
-                                 image.ehdr.e_phnum * sizeof image.phdrs[0])
-      != (image.ehdr.e_phnum * sizeof image.phdrs[0])) {
-    if (NULL != err_code) {
-      *err_code = LOAD_READ_ERROR;
+#if NACL_TARGET_SUBARCH == 64
+  if (ELFCLASS64 == ehdr.ehdr64.e_ident[EI_CLASS]) {
+    /*
+     * We'll load the 64-bit phdrs and convert them to 32-bit format.
+     */
+    Elf64_Phdr phdr64[NACL_MAX_PROGRAM_HEADERS];
+
+    if (ehdr.ehdr64.e_phentsize != sizeof(Elf64_Phdr)) {
+      if (NULL != err_code) {
+        *err_code = LOAD_BAD_PHENTSIZE;
+      }
+      NaClLog(2, "bad prog headers size\n");
+      NaClLog(2, " ehdr64.e_phentsize = 0x%"NACL_PRIxElf_Half"\n",
+              ehdr.ehdr64.e_phentsize);
+      NaClLog(2, "  sizeof(Elf64_Phdr) = 0x%"NACL_PRIxS"\n",
+              sizeof(Elf64_Phdr));
+      return 0;
+    }
+
+    /*
+     * We know the multiplication won't overflow since we rejected
+     * e_phnum values larger than the small constant NACL_MAX_PROGRAM_HEADERS.
+     */
+    if ((size_t) (*gp->vtbl->Read)(gp,
+                                   &phdr64[0],
+                                   image.ehdr.e_phnum * sizeof phdr64[0])
+        != (image.ehdr.e_phnum * sizeof phdr64[0])) {
+      if (NULL != err_code) {
+        *err_code = LOAD_READ_ERROR;
+      }
+      NaClLog(2, "cannot load tp prog headers\n");
+      return 0;
+    }
+
+    for (cur_ph = 0; cur_ph < image.ehdr.e_phnum; ++cur_ph) {
+      if (phdr64[cur_ph].p_offset > 0xffffffffU ||
+          phdr64[cur_ph].p_vaddr > 0xffffffffU ||
+          phdr64[cur_ph].p_paddr > 0xffffffffU ||
+          phdr64[cur_ph].p_filesz > 0xffffffffU ||
+          phdr64[cur_ph].p_memsz > 0xffffffffU ||
+          phdr64[cur_ph].p_align > 0xffffffffU) {
+        if (NULL != err_code) {
+          *err_code = LOAD_PHDR_OVERFLOW;
+        }
+        NaClLog(2, "ELFCLASS64 program header fields overflow 32 bits\n");
+        return 0;
+      }
+      image.phdrs[cur_ph].p_type = phdr64[cur_ph].p_type;
+      image.phdrs[cur_ph].p_offset = (Elf32_Off) phdr64[cur_ph].p_offset;
+      image.phdrs[cur_ph].p_vaddr = (Elf32_Addr) phdr64[cur_ph].p_vaddr;
+      image.phdrs[cur_ph].p_paddr = (Elf32_Addr) phdr64[cur_ph].p_paddr;
+      image.phdrs[cur_ph].p_filesz = (Elf32_Word) phdr64[cur_ph].p_filesz;
+      image.phdrs[cur_ph].p_memsz = (Elf32_Word) phdr64[cur_ph].p_memsz;
+      image.phdrs[cur_ph].p_flags = phdr64[cur_ph].p_flags;
+      image.phdrs[cur_ph].p_align = (Elf32_Word) phdr64[cur_ph].p_align;
+    }
+  } else
+#endif
+  {
+    if (image.ehdr.e_phentsize != sizeof image.phdrs[0]) {
+      if (NULL != err_code) {
+        *err_code = LOAD_BAD_PHENTSIZE;
+      }
+      NaClLog(2, "bad prog headers size\n");
+      NaClLog(2, " image.ehdr.e_phentsize = 0x%"NACL_PRIxElf_Half"\n",
+              image.ehdr.e_phentsize);
+      NaClLog(2, "  sizeof image.phdrs[0] = 0x%"NACL_PRIxS"\n",
+              sizeof image.phdrs[0]);
+      return 0;
+    }
+
+    if ((size_t) (*gp->vtbl->Read)(gp,
+                                   &image.phdrs[0],
+                                   image.ehdr.e_phnum * sizeof image.phdrs[0])
+        != (image.ehdr.e_phnum * sizeof image.phdrs[0])) {
+      if (NULL != err_code) {
+        *err_code = LOAD_READ_ERROR;
+      }
+      NaClLog(2, "cannot load tp prog headers\n");
+      return 0;
     }
-    NaClLog(2, "cannot load tp prog headers\n");
-    return 0;
   }
 
   NaClLog(2, "=================================================\n");
@@ -492,6 +584,12 @@ NaClErrorCode NaClElfImageLoad(struct NaClElfImage *image,
     }
 
     NaClLog(2, "loading segment %d\n", segnum);
+
+    if (0 == php->p_filesz) {
+      NaClLog(4, "zero-sized segment.  ignoring...\n");
+      continue;
+    }
+
     end_vaddr = php->p_vaddr + php->p_filesz;
     /* integer overflow? */
     if (end_vaddr < php->p_vaddr) {
@@ -527,18 +625,146 @@ NaClErrorCode NaClElfImageLoad(struct NaClElfImage *image,
             php->p_filesz,
             php->p_filesz,
             paddr);
+
+    /*
+     * Tell valgrind that this memory is accessible and undefined. For more
+     * details see
+     * http://code.google.com/p/nativeclient/wiki/ValgrindMemcheck#Implementation_details
+     */
+    NACL_MAKE_MEM_UNDEFINED((void *) paddr, php->p_filesz);
+
     if ((Elf_Word) (*gp->vtbl->Read)(gp, (void *) paddr, php->p_filesz)
         != php->p_filesz) {
       NaClLog(LOG_ERROR, "load failure segment %d", segnum);
       return LOAD_SEGMENT_BAD_PARAM;
     }
     /* region from p_filesz to p_memsz should already be zero filled */
+
+    /* Tell Valgrind that we've mapped a segment of nacl_file. */
+    NaClFileMappingForValgrind(paddr, php->p_filesz, php->p_offset);
   }
 
   return LOAD_OK;
 }
 
 
+NaClErrorCode NaClElfImageLoadDynamically(struct NaClElfImage *image,
+                                          struct NaClApp      *nap,
+                                          struct Gio          *gfile) {
+  int segnum;
+  for (segnum = 0; segnum < image->ehdr.e_phnum; ++segnum) {
+    const Elf_Phdr *php = &image->phdrs[segnum];
+    int32_t result;
+
+    /*
+     * We check for PT_LOAD directly rather than using the "loadable"
+     * array because we are not using NaClElfImageValidateProgramHeaders()
+     * to fill out the "loadable" array for this ELF object.  This ELF
+     * object does not have to fit such strict constraints (such as
+     * having code at 0x20000), and safety checks are applied by
+     * NaClTextDyncodeCreate() and NaClCommonSysMmapIntern().
+     */
+    if (PT_LOAD != php->p_type) {
+      continue;
+    }
+
+    /*
+     * Ideally, Gio would have a Pread() method which we would use
+     * instead of Seek().  In practice, though, there is no
+     * Seek()/Read() race condition here because both
+     * GioMemoryFileSnapshot and NaClGioShm use a seek position that
+     * is local and not shared between processes.
+     */
+    if ((*gfile->vtbl->Seek)(gfile, (off_t) php->p_offset,
+                             SEEK_SET) == (off_t) -1) {
+      NaClLog(1, "NaClElfImageLoadDynamically: seek failed\n");
+      return LOAD_READ_ERROR;
+    }
+
+    if (0 != (php->p_flags & PF_X)) {
+      /* Load code segment. */
+      /*
+       * We make a copy of the code.  This is not ideal given that
+       * GioMemoryFileSnapshot and NaClGioShm already have a copy of
+       * the file in memory or mmapped.
+       * TODO(mseaborn): Reduce the amount of copying here.
+       */
+      char *code_copy = malloc(php->p_filesz);
+      if (NULL == code_copy) {
+        NaClLog(1, "NaClElfImageLoadDynamically: malloc failed\n");
+        return LOAD_NO_MEMORY;
+      }
+      if ((Elf_Word) (*gfile->vtbl->Read)(gfile, code_copy, php->p_filesz)
+          != php->p_filesz) {
+        free(code_copy);
+        NaClLog(1, "NaClElfImageLoadDynamically: "
+                "failed to read code segment\n");
+        return LOAD_READ_ERROR;
+      }
+      result = NaClTextDyncodeCreate(nap, (uint32_t) php->p_vaddr,
+                                     code_copy, (uint32_t) php->p_filesz);
+      free(code_copy);
+      if (0 != result) {
+        NaClLog(1, "NaClElfImageLoadDynamically: "
+                "failed to load code segment\n");
+        return LOAD_UNLOADABLE;
+      }
+    } else {
+      /* Load data segment. */
+      void *paddr = (void *) NaClUserToSys(nap, php->p_vaddr);
+      size_t mapping_size = NaClRoundAllocPage(php->p_memsz);
+      /*
+       * Note that we do not used NACL_ABI_MAP_FIXED because we do not
+       * want to silently overwrite any existing mappings, such as the
+       * user app's data segment or the stack.  We detect overmapping
+       * when mmap chooses not to use the preferred address we supply.
+       * (Ideally mmap would provide a MAP_EXCL option for this
+       * instead.)
+       */
+      result = NaClCommonSysMmapIntern(
+          nap, (void *) (uintptr_t) php->p_vaddr, mapping_size,
+          NACL_ABI_PROT_READ | NACL_ABI_PROT_WRITE,
+          NACL_ABI_MAP_ANONYMOUS | NACL_ABI_MAP_PRIVATE,
+          -1, 0);
+      if ((int32_t) php->p_vaddr != result) {
+        NaClLog(1, "NaClElfImageLoadDynamically: failed to map data segment\n");
+        return LOAD_UNLOADABLE;
+      }
+      if ((Elf_Word) (*gfile->vtbl->Read)(gfile, paddr, php->p_filesz)
+          != php->p_filesz) {
+        NaClLog(1, "NaClElfImageLoadDynamically: "
+                "failed to read data segment\n");
+        return LOAD_READ_ERROR;
+      }
+      /*
+       * Note that we do not need to zero the BSS (the region from
+       * p_filesz to p_memsz) because it should already be zero
+       * filled.  This would not be the case if we were mapping the
+       * data segment from the file.
+       */
+
+      if (0 == (php->p_flags & PF_W)) {
+        /* Handle read-only data segment. */
+        int rc = NaCl_mprotect(paddr, mapping_size, NACL_ABI_PROT_READ);
+        if (0 != rc) {
+          NaClLog(1, "NaClElfImageLoadDynamically: "
+                  "failed to mprotect read-only data segment\n");
+          return LOAD_MPROTECT_FAIL;
+        }
+
+        NaClVmmapUpdate(&nap->mem_map,
+                        php->p_vaddr >> NACL_PAGESHIFT,
+                        mapping_size >> NACL_PAGESHIFT,
+                        PROT_READ,
+                        NULL,
+                        0  /* remove: false */);
+      }
+    }
+  }
+  return LOAD_OK;
+}
+
+
 void NaClElfImageDelete(struct NaClElfImage *image) {
   free(image);
 }
@@ -547,18 +773,3 @@ void NaClElfImageDelete(struct NaClElfImage *image) {
 uintptr_t NaClElfImageGetEntryPoint(struct NaClElfImage *image) {
   return image->ehdr.e_entry;
 }
-
-
-int NaClElfImageGetBundleSize(struct NaClElfImage *image) {
-  switch (image->ehdr.e_flags & EF_NACL_ALIGN_MASK) {
-   case  EF_NACL_ALIGN_16:
-    return 16;
-   case EF_NACL_ALIGN_32:
-    return 32;
-   default:
-#if 0 == NACL_DANGEROUS_DEBUG_MODE_DISABLE_INNER_SANDBOX
-    NaClLog(LOG_FATAL, "strange alignment");
-#endif
-    return 0;
-  }
-}
