@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -18,7 +18,10 @@
  */
 
 #define NACL_sys_null                    1
+#define NACL_sys_nameservice             2
 
+#define NACL_sys_dup                     8
+#define NACL_sys_dup2                    9
 #define NACL_sys_open                   10
 #define NACL_sys_close                  11
 #define NACL_sys_read                   12
@@ -70,6 +73,11 @@
 #define NACL_sys_tls_init               82
 #define NACL_sys_thread_nice            83
 #define NACL_sys_tls_get                84
+#define NACL_sys_second_tls_set         85
+#define NACL_sys_second_tls_get         86
+#define NACL_sys_exception_handler      87
+#define NACL_sys_exception_stack        88
+#define NACL_sys_exception_clear_flag   89
 
 #define NACL_sys_sem_create             100
 #define NACL_sys_sem_wait               101
@@ -85,6 +93,8 @@
 #define NACL_sys_dyncode_modify         105
 #define NACL_sys_dyncode_delete         106
 
+#define NACL_sys_test_infoleak          109
+
 #define NACL_MAX_SYSCALLS               110
 
 #endif
