@@ -1,7 +1,7 @@
 /*
- * Copyright 2008  The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2012 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 
@@ -19,6 +19,7 @@
 
 #include "native_client/src/include/portability.h"
 #include "native_client/src/include/nacl_platform.h"
+#include "native_client/src/include/nacl_macros.h"
 
 #include "native_client/src/shared/platform/nacl_check.h"
 #include "native_client/src/shared/platform/nacl_log.h"
@@ -33,7 +34,6 @@
 #include "native_client/src/trusted/desc/nacl_desc_io.h"
 
 #include "native_client/src/trusted/service_runtime/nacl_app_thread.h"
-#include "native_client/src/trusted/service_runtime/nacl_closure.h"
 #include "native_client/src/trusted/service_runtime/nacl_config.h"
 #include "native_client/src/trusted/service_runtime/nacl_globals.h"
 #include "native_client/src/trusted/service_runtime/nacl_memory_object.h"
@@ -79,81 +79,6 @@ int32_t NaClSysNull(struct NaClAppThread *natp) {
   return 0;
 }
 
-int32_t NaClSysOpen(struct NaClAppThread  *natp,
-                    char                  *pathname,
-                    int                   flags,
-                    int                   mode) {
-  return NaClCommonSysOpen(natp, pathname, flags, mode);
-}
-
-int32_t NaClSysClose(struct NaClAppThread *natp,
-                     int                  d) {
-  return NaClCommonSysClose(natp, d);
-}
-
-int32_t NaClSysRead(struct NaClAppThread  *natp,
-                    int                   d,
-                    void                  *buf,
-                    size_t                count) {
-  return NaClCommonSysRead(natp, d, buf, count);
-}
-
-int32_t NaClSysWrite(struct NaClAppThread *natp,
-                     int                  d,
-                     void                 *buf,
-                     size_t               count) {
-  return NaClCommonSysWrite(natp, d, buf, count);
-}
-
-/* Warning: sizeof(nacl_abi_off_t)!=sizeof(off_t) on OSX */
-int32_t NaClSysLseek(struct NaClAppThread *natp,
-                     int                  d,
-                     nacl_abi_off_t       *offp,
-                     int                  whence) {
-  return NaClCommonSysLseek(natp, d, offp, whence);
-}
-
-int32_t NaClSysIoctl(struct NaClAppThread *natp,
-                     int                  d,
-                     int                  request,
-                     void                 *arg) {
-  return NaClCommonSysIoctl(natp, d, request, arg);
-}
-
-int32_t NaClSysFstat(struct NaClAppThread *natp,
-                     int                  d,
-                     struct nacl_abi_stat *nasp) {
-  return NaClCommonSysFstat(natp, d, nasp);
-}
-
-int32_t NaClSysStat(struct NaClAppThread *natp,
-                    const char           *path,
-                    struct nacl_abi_stat *nasp) {
-  return NaClCommonSysStat(natp, path, nasp);
-}
-
-int32_t NaClSysGetdents(struct NaClAppThread  *natp,
-                        int                   d,
-                        void                  *buf,
-                        size_t                count) {
-  return NaClCommonSysGetdents(natp, d, buf, count);
-}
-
-int32_t NaClSysSysbrk(struct NaClAppThread  *natp,
-                      void                  *new_break) {
-  return NaClSetBreak(natp, (uintptr_t) new_break);
-}
-
-int32_t NaClSysMmap(struct NaClAppThread  *natp,
-                    void                  *start,
-                    size_t                length,
-                    int                   prot,
-                    int                   flags,
-                    int                   d,
-                    nacl_abi_off_t        *offp) {
-  return NaClCommonSysMmap(natp, start, length, prot, flags, d, offp);
-}
-
 int32_t NaClSysMunmap(struct NaClAppThread  *natp,
                       void                  *start,
                       size_t                length) {
@@ -162,7 +87,7 @@ int32_t NaClSysMunmap(struct NaClAppThread  *natp,
   int       holding_app_lock = 0;
   size_t    alloc_rounded_length;
 
-  NaClLog(3, "NaClSysMunmap(0x%08"NACL_PRIxPTR", "
+  NaClLog(3, "Entered NaClSysMunmap(0x%08"NACL_PRIxPTR", "
           "0x%08"NACL_PRIxPTR", 0x%"NACL_PRIxS")\n",
           (uintptr_t) natp, (uintptr_t) start, length);
 
@@ -195,6 +120,14 @@ int32_t NaClSysMunmap(struct NaClAppThread  *natp,
   }
 
   NaClXMutexLock(&natp->nap->mu);
+
+  while (0 != natp->nap->threads_launching) {
+    NaClXCondVarWait(&natp->nap->cv, &natp->nap->mu);
+  }
+  natp->nap->vm_hole_may_exist = 1;
+  /* This call is not needed on Unix, but is included for consistency. */
+  NaClUntrustedThreadsSuspend(natp->nap);
+
   holding_app_lock = 1;
   /*
    * NB: windows (or generic) version would use Munmap virtual
@@ -223,7 +156,7 @@ int32_t NaClSysMunmap(struct NaClAppThread  *natp,
    * zero-filled pages, which should be copy-on-write and thus
    * relatively cheap.  Do not open up an address space hole.
    */
-  NaClLog(3,
+  NaClLog(4,
           ("NaClSysMunmap: mmap(0x%08"NACL_PRIxPTR", 0x%"NACL_PRIxS","
            " 0x%x, 0x%x, -1, 0)\n"),
           sysaddr, length, PROT_NONE,
@@ -234,7 +167,7 @@ int32_t NaClSysMunmap(struct NaClAppThread  *natp,
                          MAP_PRIVATE | MAP_ANONYMOUS | MAP_FIXED,
                          -1,
                          (off_t) 0)) {
-    NaClLog(3, "mmap to put in anonymous memory failed, errno = %d\n", errno);
+    NaClLog(4, "mmap to put in anonymous memory failed, errno = %d\n", errno);
     retval = -NaClXlateErrno(errno);
     goto cleanup;
   }
@@ -248,17 +181,17 @@ int32_t NaClSysMunmap(struct NaClAppThread  *natp,
   retval = 0;
 cleanup:
   if (holding_app_lock) {
+    natp->nap->vm_hole_may_exist = 0;
+    NaClXCondVarBroadcast(&natp->nap->cv);
     NaClXMutexUnlock(&natp->nap->mu);
+
+    /* This call is not needed on Unix, but is included for consistency. */
+    NaClUntrustedThreadsResume(natp->nap);
   }
   NaClSysCommonThreadSyscallLeave(natp);
   return retval;
 }
 
-int32_t NaClSysExit(struct NaClAppThread *natp,
-                    int                  status) {
-  return NaClCommonSysExit(natp, status);
-}
-
 int32_t NaClSysGetpid(struct NaClAppThread *natp) {
   int32_t pid;
 
@@ -271,11 +204,6 @@ int32_t NaClSysGetpid(struct NaClAppThread *natp) {
   return pid;
 }
 
-int32_t NaClSysThread_Exit(struct NaClAppThread  *natp,
-                           int32_t               *stack_flag) {
-  return NaClCommonSysThreadExit(natp, stack_flag);
-}
-
 int32_t NaClSysGetTimeOfDay(struct NaClAppThread      *natp,
                             struct nacl_abi_timeval   *tv,
                             struct nacl_abi_timezone  *tz) {
@@ -284,6 +212,11 @@ int32_t NaClSysGetTimeOfDay(struct NaClAppThread      *natp,
   struct nacl_abi_timeval now;
 
   UNREFERENCED_PARAMETER(tz);
+
+  NaClLog(3,
+          ("Entered NaClSysGetTimeOfDay(%08"NACL_PRIxPTR
+           ", 0x%08"NACL_PRIxPTR", 0x%08"NACL_PRIxPTR")\n"),
+          (uintptr_t) natp, (uintptr_t) tv, (uintptr_t) tz);
   NaClSysCommonThreadSyscallEnter(natp);
 
   sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) tv, sizeof tv);
@@ -304,13 +237,14 @@ int32_t NaClSysGetTimeOfDay(struct NaClAppThread      *natp,
   retval = NaClGetTimeOfDay(&now);
   if (0 == retval) {
     /*
-     * To make it harder to distinguish Linux platforms from Windows,
-     * coarsen the time to the same level we get on Windows -
-     * milliseconds, unless in "debug" mode.
+     * Coarsen the time to the same level we get on Windows -
+     * 10 microseconds.
      */
     if (!NaClHighResolutionTimerEnabled()) {
-      now.nacl_abi_tv_usec = (now.nacl_abi_tv_usec / 1000) * 1000;
+      now.nacl_abi_tv_usec = (now.nacl_abi_tv_usec / 10) * 10;
     }
+    CHECK(now.nacl_abi_tv_usec >= 0);
+    CHECK(now.nacl_abi_tv_usec < NACL_MICROS_PER_UNIT);
     *(struct nacl_abi_timeval *) sysaddr = now;
   }
 cleanup:
@@ -326,6 +260,10 @@ cleanup:
 int32_t NaClSysClock(struct NaClAppThread *natp) {
   int32_t retval;
 
+  NaClLog(3,
+          ("Entered NaClSysClock(%08"NACL_PRIxPTR")\n"),
+          (uintptr_t) natp);
+
   NaClSysCommonThreadSyscallEnter(natp);
   retval = clock();
   NaClSysCommonThreadSyscallLeave(natp);
@@ -342,7 +280,10 @@ int32_t NaClSysNanosleep(struct NaClAppThread     *natp,
   struct nacl_abi_timespec  *remptr;
   int                       retval = -NACL_ABI_EINVAL;
 
-  NaClLog(4, "NaClSysNanosleep(%08"NACL_PRIxPTR"x)\n", (uintptr_t) req);
+  NaClLog(3,
+          ("Entered NaClSysNanosleep(0x%08"NACL_PRIxPTR
+           ", 0x%08"NACL_PRIxPTR", 0x%08"NACL_PRIxPTR"x)\n"),
+          (uintptr_t) natp, (uintptr_t) req, (uintptr_t) rem);
 
   NaClSysCommonThreadSyscallEnter(natp);
 
@@ -397,140 +338,6 @@ cleanup:
   return retval;
 }
 
-int32_t NaClSysImc_MakeBoundSock(struct NaClAppThread *natp,
-                                 int32_t              *sap) {
-  return NaClCommonSysImc_MakeBoundSock(natp, sap);
-}
-
-int32_t NaClSysImc_Accept(struct NaClAppThread  *natp,
-                          int                   d) {
-  return NaClCommonSysImc_Accept(natp, d);
-}
-
-int32_t NaClSysImc_Connect(struct NaClAppThread *natp,
-                           int                  d) {
-  return NaClCommonSysImc_Connect(natp, d);
-}
-
-int32_t NaClSysImc_Sendmsg(struct NaClAppThread         *natp,
-                           int                          d,
-                           struct NaClAbiNaClImcMsgHdr  *nanimhp,
-                           int                          flags) {
-  return NaClCommonSysImc_Sendmsg(natp, d, nanimhp, flags);
-}
-
-int32_t NaClSysImc_Recvmsg(struct NaClAppThread         *natp,
-                           int                          d,
-                           struct NaClAbiNaClImcMsgHdr  *nimhp,
-                           int                          flags) {
-  return NaClCommonSysImc_Recvmsg(natp, d, nimhp, flags);
-}
-
-int32_t NaClSysImc_Mem_Obj_Create(struct NaClAppThread  *natp,
-                                  nacl_abi_size_t       size) {
-  return NaClCommonSysImc_Mem_Obj_Create(natp, size);
-}
-
-int32_t NaClSysTls_Init(struct NaClAppThread  *natp,
-                        void                  *tdb,
-                        size_t                size) {
-  return NaClCommonSysTls_Init(natp, tdb, size);
-}
-
-int32_t NaClSysThread_Create(struct NaClAppThread *natp,
-                             void                 *prog_ctr,
-                             void                 *stack_ptr,
-                             void                 *tdb,
-                             size_t               tdb_size) {
-  return NaClCommonSysThread_Create(natp, prog_ctr, stack_ptr, tdb, tdb_size);
-}
-
-int32_t NaClSysTls_Get(struct NaClAppThread *natp) {
-  return NaClCommonSysTdbGet(natp);
-}
-
-int32_t NaClSysThread_Nice(struct NaClAppThread *natp, const int nice) {
-  return NaClCommonSysThread_Nice(natp, nice);
-}
-
-/* mutex */
-
-int32_t NaClSysMutex_Create(struct NaClAppThread *natp) {
-  return NaClCommonSysMutex_Create(natp);
-}
-
-int32_t NaClSysMutex_Lock(struct NaClAppThread *natp,
-                          int32_t              mutex_handle) {
-  return NaClCommonSysMutex_Lock(natp, mutex_handle);
-}
-
-int32_t NaClSysMutex_Unlock(struct NaClAppThread *natp,
-                            int32_t              mutex_handle) {
-  return NaClCommonSysMutex_Unlock(natp, mutex_handle);
-}
-
-int32_t NaClSysMutex_Trylock(struct NaClAppThread *natp,
-                             int32_t              mutex_handle) {
-  return NaClCommonSysMutex_Trylock(natp, mutex_handle);
-}
-
-
-/* condition variable */
-
-int32_t NaClSysCond_Create(struct NaClAppThread *natp) {
-  return NaClCommonSysCond_Create(natp);
-}
-
-int32_t NaClSysCond_Wait(struct NaClAppThread *natp,
-                         int32_t              cond_handle,
-                         int32_t              mutex_handle) {
-  return NaClCommonSysCond_Wait(natp, cond_handle, mutex_handle);
-}
-
-int32_t NaClSysCond_Signal(struct NaClAppThread *natp,
-                           int32_t              cond_handle) {
-  return NaClCommonSysCond_Signal(natp, cond_handle);
-}
-
-int32_t NaClSysCond_Broadcast(struct NaClAppThread *natp,
-                              int32_t              cond_handle) {
-  return NaClCommonSysCond_Broadcast(natp, cond_handle);
-}
-
-int32_t NaClSysCond_Timed_Wait_Abs(struct NaClAppThread     *natp,
-                                   int32_t                  cond_handle,
-                                   int32_t                  mutex_handle,
-                                   struct nacl_abi_timespec *ts) {
-  return NaClCommonSysCond_Timed_Wait_Abs(natp,
-                                          cond_handle,
-                                          mutex_handle,
-                                          ts);
-}
-
-int32_t NaClSysImc_SocketPair(struct NaClAppThread  *natp,
-                              int32_t               *d_out) {
-  return NaClCommonSysImc_SocketPair(natp, d_out);
-}
-int32_t NaClSysSem_Create(struct NaClAppThread *natp,
-                          int32_t              init_value) {
-  return NaClCommonSysSem_Create(natp, init_value);
-}
-
-int32_t NaClSysSem_Wait(struct NaClAppThread *natp,
-                        int32_t              sem_handle) {
-  return NaClCommonSysSem_Wait(natp, sem_handle);
-}
-
-int32_t NaClSysSem_Post(struct NaClAppThread *natp,
-                        int32_t              sem_handle) {
-  return NaClCommonSysSem_Post(natp, sem_handle);
-}
-
-int32_t NaClSysSem_Get_Value(struct NaClAppThread *natp,
-                             int32_t              sem_handle) {
-  return NaClCommonSysSem_Get_Value(natp, sem_handle);
-}
-
 int32_t NaClSysSched_Yield(struct NaClAppThread *natp) {
   UNREFERENCED_PARAMETER(natp);
   sched_yield();
@@ -544,6 +351,11 @@ int32_t NaClSysSysconf(struct NaClAppThread *natp,
   static int32_t  number_of_workers = -1;
   uintptr_t       sysaddr;
 
+  NaClLog(3,
+          ("Entered NaClSysSysconf(%08"NACL_PRIxPTR
+           "x, %d, 0x%08"NACL_PRIxPTR")\n"),
+          (uintptr_t) natp, name, (uintptr_t) result);
+
   NaClSysCommonThreadSyscallEnter(natp);
 
   sysaddr = NaClUserToSysAddrRange(natp->nap,
@@ -584,23 +396,3 @@ cleanup:
   NaClSysCommonThreadSyscallLeave(natp);
   return retval;
 }
-
-int32_t NaClSysDyncode_Create(struct NaClAppThread *natp,
-                            uint32_t             dest,
-                            uint32_t             src,
-                            uint32_t             size) {
-  return NaClTextSysDyncode_Create(natp, dest, src, size);
-}
-
-int32_t NaClSysDyncode_Modify(struct NaClAppThread *natp,
-                            uint32_t             dest,
-                            uint32_t             src,
-                            uint32_t             size) {
-  return NaClTextSysDyncode_Modify(natp, dest, src, size);
-}
-
-int32_t NaClSysDyncode_Delete(struct NaClAppThread *natp,
-                            uint32_t             dest,
-                            uint32_t             size) {
-  return NaClTextSysDyncode_Delete(natp, dest, size);
-}
