@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -11,8 +11,8 @@
 #ifndef NATIVE_CLIENT_SERVICE_RUNTIME_NACL_APP_THREAD_H__
 #define NATIVE_CLIENT_SERVICE_RUNTIME_NACL_APP_THREAD_H__ 1
 
+#include "native_client/src/shared/platform/nacl_sync.h"
 #include "native_client/src/shared/platform/nacl_threads.h"
-#include "native_client/src/trusted/service_runtime/nacl_bottom_half.h"
 #include "native_client/src/trusted/service_runtime/nacl_signal.h"
 #include "native_client/src/trusted/service_runtime/sel_rt.h"
 
@@ -21,6 +21,25 @@ EXTERN_C_BEGIN
 
 struct NaClApp;
 
+/*
+ * The thread hosting the NaClAppThread may change suspend_state
+ * between NACL_APP_THREAD_TRUSTED and NACL_APP_THREAD_UNTRUSTED using
+ * NaClAppThreadSetSuspendState().
+ *
+ * Another thread may change this from:
+ *   NACL_APP_THREAD_UNTRUSTED
+ *   -> NACL_APP_THREAD_UNTRUSTED | NACL_APP_THREAD_SUSPENDING
+ * or
+ *   NACL_APP_THREAD_TRUSTED
+ *   -> NACL_APP_THREAD_TRUSTED | NACL_APP_THREAD_SUSPENDING
+ * and back again.
+ */
+enum NaClSuspendState {
+  NACL_APP_THREAD_UNTRUSTED = 1,
+  NACL_APP_THREAD_TRUSTED = 2,
+  NACL_APP_THREAD_SUSPENDING = 4
+};
+
 enum NaClThreadState {
   NACL_APP_THREAD_ALIVE,
   /* NACL_APP_THREAD_INTERRUPTIBLE_MUTEX, etc */
@@ -38,8 +57,6 @@ struct NaClAppThread {
   struct NaClMutex          mu;
   struct NaClCondVar        cv;
 
-  struct NaClClosureResult  result;
-
   uint32_t                  sysret;
 
   /*
@@ -48,62 +65,28 @@ struct NaClAppThread {
    */
   struct NaClApp            *nap;
 
-  /*
-   * The effector interface object used to manipulate NaCl apps by the
-   * objects in the NaClDesc class hierarchy.  Used by this thread when
-   * making NaClDesc method calls from syscall handlers.
-   */
-  struct NaClDescEffector   *effp;
-
   int                       thread_num;  /* index into nap->threads */
-  uintptr_t                 sys_tdb;  /* saved tdb ptr */
+  /*
+   * sys_tls and tls2 are TLS values used by user code and the
+   * integrated runtime (IRT) respectively.  The first TLS area may be
+   * accessed via the %gs segment register on x86-32 so must point
+   * into untrusted address space; we store it as a system pointer.
+   * The second TLS may be an arbitrary value.
+   */
+  uintptr_t                 sys_tls;  /* first saved TLS ptr */
+  uint32_t                  *usr_tlsp;
+  uint32_t                  tls2;     /* second saved TLS value */
 
   struct NaClThread         thread;  /* low level thread representation */
 
   /*
-   * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
-   *
-   * The locking behavior described below is not yet implemented.
-   *
-   * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
-   *
-   * When a thread invokes a service call from the NaCl application,
-   * it must first grab its own lock (mu) before it executes any code
-   * that can grab other service runtime locks/resources.  The thread
-   * clears the holding_sr_locks flag as it is about to return from
-   * the service call.  Short duration service calls can just hold the
-   * lock throughout; potentially blocking service calls must drop the
-   * thread lock and reacquire when it can unblock.  Condition
-   * variables are used to allow the thread to wake up.  If a thread
-   * is blocked waiting for I/O, a central epoll thread is responsible
-   * for waking up the thread.  (The epoll thread serves the same
-   * purpose as hardware interrupt handlers.)
-   *
-   * To summarily kill a thread from elsewhere, we check the
-   * holding_sr_locks flag after acquiring the target thread's lock.
-   * If it is clear, then the thread is running in the application
-   * space (or at least it has not yet touched any service runtime
-   * resources), and we can directly pthread_kill it before we release
-   * the thread lock, using a signal handler in the trampoline region,
-   * which can deal with segment registers (in the x86-32 case), prior
-   * to invoking pthread_exit.  If it is set, we set the state flag to
-   * NACL_APP_THREAD_SUICIDE_PENDING and release the thread lock,
-   * possibly waking the blocked thread (which should be using
-   * interruptible mutexes/condvars).
-   *
-   * When the thread is about to leave service runtime code and return
-   * to the NaCl application, it should have released all locks to
-   * service-runtime resources.  Next, the thread grabs its own lock
-   * to clear the holding_sr_locks flag, at which point it examines
-   * the suicide flag; if it finds that set, it should gracefully
-   * exit.
-   *
-   * Similarly, if a thread's synchronization operation is
-   * interrupted, it knows that it should abort.  It can fail up
-   * through the normal execution path, freeing up locks and other
-   * resources on the way, and handle thread death as above.
+   * Thread suspension is currently only needed and implemented for
+   * Windows, for preventing race conditions when opening up holes in
+   * untrusted address space during mmap/munmap.
    */
-  int                       holding_sr_locks;
+#if NACL_WINDOWS
+  enum NaClSuspendState     suspend_state;
+#endif
 
   /*
    * a thread cannot free up its own mutex lock and commit suicide,
@@ -141,32 +124,44 @@ struct NaClAppThread {
    * The last generation this thread reported into the service runtime
    * Protected by mu
    */
-  int dynamic_delete_generation;
+  int                       dynamic_delete_generation;
 };
 
+void NaClAppThreadTeardown(struct NaClAppThread *natp);
+
 int NaClAppThreadCtor(struct NaClAppThread  *natp,
                       struct NaClApp        *nap,
                       uintptr_t             entry,
                       uintptr_t             stack_ptr,
                       uint32_t              tls_idx,
-                      uintptr_t             sys_tdb) NACL_WUR;
+                      uintptr_t             sys_tls,
+                      uint32_t              usr_tls2) NACL_WUR;
 
 void NaClAppThreadDtor(struct NaClAppThread *natp);
 
 /*
  * Low level initialization of thread, with validated values.  The
  * usr_entry and usr_stack_ptr values are directly used to initialize the
- * user register values; the sys_tdb_base is the system address for
+ * user register values; the sys_tls_base is the system address for
  * allocating a %gs thread descriptor block base.  The caller is
  * responsible for error checking: usr_entry is a valid entry point (0
- * mod N) and sys_tdb_base is in the NaClApp's address space.
+ * mod N) and sys_tls_base is in the NaClApp's address space.
  */
 int NaClAppThreadAllocSegCtor(struct NaClAppThread  *natp,
                               struct NaClApp        *nap,
                               uintptr_t             usr_entry,
                               uintptr_t             usr_stack_ptr,
-                              uintptr_t             sys_tdb_base,
-                              size_t                tdb_size) NACL_WUR;
+                              uintptr_t             sys_tls_base,
+                              uint32_t              usr_tls2) NACL_WUR;
+
+/*
+ * This function can be called from the thread hosting the
+ * NaClAppThread.  It can be used to switch between the states
+ * NACL_APP_THREAD_TRUSTED and NACL_APP_THREAD_UNTRUSTED.
+ */
+void NaClAppThreadSetSuspendState(struct NaClAppThread *natp,
+                                  enum NaClSuspendState old_state,
+                                  enum NaClSuspendState new_state);
 
 EXTERN_C_END
 
