@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2012 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -72,6 +72,16 @@ extern int NaClAclBypassChecks;
 
 void NaClInsecurelyBypassAllAclChecks(void);
 
+int32_t NaClCommonSysNameService(struct NaClAppThread *natp,
+                                 int32_t              *desc_addr);
+
+int32_t NaClCommonSysDup(struct NaClAppThread *natp,
+                         int                  oldfd);
+
+int32_t NaClCommonSysDup2(struct NaClAppThread  *natp,
+                          int                   oldfd,
+                          int                   newfd);
+
 int32_t NaClCommonSysOpen(struct NaClAppThread  *natp,
                           char                  *pathname,
                           int                   flags,
@@ -108,7 +118,7 @@ int32_t NaClCommonSysStat(struct NaClAppThread *natp,
                           const char           *path,
                           struct nacl_abi_stat *nasp);
 
-void NaClCommonUtilUpdateAddrMap(struct NaClAppThread *natp,
+void NaClCommonUtilUpdateAddrMap(struct NaClApp       *nap,
                                  uintptr_t            sysaddr,
                                  size_t               nbytes,
                                  int                  sysprot,
@@ -130,7 +140,7 @@ int32_t NaClCommonSysMmap(struct NaClAppThread  *natp,
                           int                   d,
                           nacl_abi_off_t        *offp);
 
-int32_t NaClCommonSysMmapIntern(struct NaClAppThread  *natp,
+int32_t NaClCommonSysMmapIntern(struct NaClApp        *nap,
                                 void                  *start,
                                 size_t                length,
                                 int                   prot,
@@ -178,16 +188,20 @@ int32_t NaClCommonSysImc_Mem_Obj_Create(struct NaClAppThread  *natp,
                                         size_t                size);
 
 int32_t NaClCommonSysTls_Init(struct NaClAppThread  *natp,
-                              void                  *tdb,
-                              size_t                size);
+                              void                  *thread_ptr);
 
 int32_t NaClCommonSysThread_Create(struct NaClAppThread *natp,
                                    void                 *eip,
                                    void                 *stack_ptr,
-                                   void                 *tdb,
-                                   size_t               tdb_size);
+                                   void                 *thread_ptr,
+                                   void                 *second_thread_ptr);
 
-int32_t NaClCommonSysTdbGet(struct NaClAppThread *natp);
+int32_t NaClCommonSysTlsGet(struct NaClAppThread *natp);
+
+int32_t NaClSysSecond_Tls_Set(struct NaClAppThread *natp,
+                              uint32_t             new_value);
+
+int32_t NaClSysSecond_Tls_Get(struct NaClAppThread *natp);
 
 int32_t NaClCommonSysThread_Nice(struct NaClAppThread *natp,
                                  const int nice);
@@ -232,7 +246,7 @@ int32_t NaClCommonSysCond_Timed_Wait_Abs(struct NaClAppThread     *natp,
 int32_t NaClCommonDescSocketPair(struct NaClDesc      **pair);
 
 int32_t NaClCommonSysImc_SocketPair(struct NaClAppThread *natp,
-                                    int32_t              *d_out);
+                                    uint32_t             descs_out);
 /* Semaphores */
 int32_t NaClCommonSysSem_Create(struct NaClAppThread *natp,
                                 int32_t              init_value);
@@ -246,6 +260,18 @@ int32_t NaClCommonSysSem_Post(struct NaClAppThread *natp,
 int32_t NaClCommonSysSem_Get_Value(struct NaClAppThread *natp,
                                    int32_t              sem_handle);
 
+int32_t NaClCommonSysException_Handler(struct NaClAppThread *natp,
+                                       uint32_t             handler_addr,
+                                       uint32_t             old_handler);
+
+int32_t NaClCommonSysException_Stack(struct NaClAppThread *natp,
+                                     uint32_t             stack_addr,
+                                     uint32_t             stack_size);
+
+int32_t NaClCommonSysException_Clear_Flag(struct NaClAppThread *natp);
+
+int32_t NaClCommonSysTest_InfoLeak(struct NaClAppThread *natp);
+
 EXTERN_C_END
 
 #endif  /* NATIVE_CLIENT_SERVICE_RUNTIME_NACL_SYSCALL_COMMON_H__ */
