@@ -1,19 +1,23 @@
 /*
- * Copyright 2009 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 #include <string.h>
 
 #include "native_client/src/include/concurrency_ops.h"
+#include "native_client/src/include/nacl_platform.h"
 #include "native_client/src/include/portability.h"
 #include "native_client/src/shared/platform/nacl_check.h"
 #include "native_client/src/shared/platform/nacl_log.h"
+#include "native_client/src/shared/platform/nacl_sync.h"
+#include "native_client/src/shared/platform/nacl_sync_checked.h"
 #include "native_client/src/trusted/desc/nacl_desc_base.h"
 #include "native_client/src/trusted/desc/nacl_desc_effector.h"
 #include "native_client/src/trusted/desc/nacl_desc_effector.h"
 #include "native_client/src/trusted/desc/nacl_desc_imc_shm.h"
+#include "native_client/src/trusted/perf_counter/nacl_perf_counter.h"
 #include "native_client/src/trusted/service_runtime/arch/sel_ldr_arch.h"
 #include "native_client/src/trusted/service_runtime/include/sys/errno.h"
 #include "native_client/src/trusted/service_runtime/include/sys/mman.h"
@@ -27,6 +31,25 @@
 /* initial size of the malloced buffer for dynamic regions */
 static const int kMinDynamicRegionsAllocated = 32;
 
+static const int kBitsPerByte = 8;
+
+static uint8_t *BitmapAllocate(uint32_t indexes) {
+  uint32_t byte_count = (indexes + kBitsPerByte - 1) / kBitsPerByte;
+  uint8_t *bitmap = malloc(byte_count);
+  if (bitmap != NULL) {
+    memset(bitmap, 0, byte_count);
+  }
+  return bitmap;
+}
+
+static int BitmapIsBitSet(uint8_t *bitmap, uint32_t index) {
+  return (bitmap[index / kBitsPerByte] & (1 << (index % kBitsPerByte))) != 0;
+}
+
+static void BitmapSetBit(uint8_t *bitmap, uint32_t index) {
+  bitmap[index / kBitsPerByte] |= 1 << (index % kBitsPerByte);
+}
+
 /*
  * Private subclass of NaClDescEffector, used only in this file.
  */
@@ -48,14 +71,8 @@ int NaClDescEffectorShmUnmapMemory(struct NaClDescEffector  *vself,
                                    uintptr_t                sysaddr,
                                    size_t                   nbytes) {
   UNREFERENCED_PARAMETER(vself);
-  /*
-   * Existing memory is anonymous paging file backed.
-   */
-  NaClLog(4, "NaClDescEffectorShmUnmapMemory called\n");
-  NaClLog(4, " sysaddr 0x%08"NACL_PRIxPTR", "
-          "0x%08"NACL_PRIxS" (%"NACL_PRIdS")\n",
-          sysaddr, nbytes, nbytes);
-  NaCl_page_free((void *) sysaddr, nbytes);
+  UNREFERENCED_PARAMETER(sysaddr);
+  UNREFERENCED_PARAMETER(nbytes);
   return 0;
 }
 
@@ -93,10 +110,11 @@ NaClErrorCode NaClMakeDynamicTextShared(struct NaClApp *nap) {
   struct NaClDescEffectorShm  shm_effector;
   int                         shm_effector_initialized = 0;
   uintptr_t                   shm_vaddr_base;
-  uintptr_t                   shm_offset;
+  int                         mmap_protections;
   uintptr_t                   mmap_ret;
 
   uintptr_t                   shm_upper_bound;
+  uintptr_t                   text_sysaddr;
 
   shm_vaddr_base = NaClEndOfStaticText(nap);
   NaClLog(4,
@@ -166,73 +184,72 @@ NaClErrorCode NaClMakeDynamicTextShared(struct NaClApp *nap) {
   }
   shm_effector_initialized = 1;
 
-  /*
-   * Map shm in place of text.  We currently do this eagerly, which
-   * can result in excess memory/swap traffic.
-   *
-   * TODO(bsy): consider using NX and doing this lazily, or mapping a
-   * canonical HLT-filled 64K shm repeatedly, and only remapping with
-   * a "real" shm text as needed.
-   */
-  for (shm_offset = 0;
-       shm_offset < dynamic_text_size;
-       shm_offset += NACL_MAP_PAGESIZE) {
-    uintptr_t text_vaddr = shm_vaddr_base + shm_offset;
-    uintptr_t text_sysaddr = NaClUserToSys(nap, text_vaddr);
+  text_sysaddr = NaClUserToSys(nap, shm_vaddr_base);
 
-    NaClLog(4,
-            "NaClMakeDynamicTextShared: Map(,,0x%"NACL_PRIxPTR",size = 0x%x,"
-            " prot=0x%x, flags=0x%x, offset=0x%"NACL_PRIxPTR"\n",
-            text_sysaddr,
-            NACL_MAP_PAGESIZE,
-            NACL_ABI_PROT_READ | NACL_ABI_PROT_WRITE,
-            NACL_ABI_MAP_SHARED | NACL_ABI_MAP_FIXED,
-            shm_offset);
-    mmap_ret = (*((struct NaClDescVtbl const *)
-                  shm->base.base.vtbl)->
-                Map)((struct NaClDesc *) shm,
-                     (struct NaClDescEffector *) &shm_effector,
-                     (void *) text_sysaddr,
-                     NACL_MAP_PAGESIZE,
-                     NACL_ABI_PROT_READ | NACL_ABI_PROT_WRITE,
-                     NACL_ABI_MAP_SHARED | NACL_ABI_MAP_FIXED,
-                     shm_offset);
-    if (text_sysaddr != mmap_ret) {
-      NaClLog(LOG_FATAL,
-              "Could not map in shm for dynamic text region, HLT filling.\n");
-    }
-    NaClFillMemoryRegionWithHalt((void *) text_sysaddr, NACL_MAP_PAGESIZE);
-    if (-1 == (*((struct NaClDescVtbl const *)
-                 shm->base.base.vtbl)->
-               UnmapUnsafe)((struct NaClDesc *) shm,
-                            ((struct NaClDescEffector *)
-                             &shm_effector),
-                            (void *) text_sysaddr,
-                            NACL_MAP_PAGESIZE)) {
-      NaClLog(LOG_FATAL,
-              "Could not unmap shm for dynamic text region, post HLT fill.\n");
-    }
-    NaClLog(4,
-            "NaClMakeDynamicTextShared: Map(,,0x%"NACL_PRIxPTR",size = 0x%x,"
-            " prot=0x%x, flags=0x%x, offset=0x%"NACL_PRIxPTR"\n",
-            text_sysaddr,
-            NACL_MAP_PAGESIZE,
-            NACL_ABI_PROT_READ | NACL_ABI_PROT_EXEC,
-            NACL_ABI_MAP_SHARED | NACL_ABI_MAP_FIXED,
-            shm_offset);
-    mmap_ret = (*((struct NaClDescVtbl const *)
-                  shm->base.base.vtbl)->
-                Map)((struct NaClDesc *) shm,
-                     (struct NaClDescEffector *) &shm_effector,
-                     (void *) text_sysaddr,
-                     NACL_MAP_PAGESIZE,
-                     NACL_ABI_PROT_READ | NACL_ABI_PROT_EXEC,
-                     NACL_ABI_MAP_SHARED | NACL_ABI_MAP_FIXED,
-                     shm_offset);
-    if (text_sysaddr != mmap_ret) {
-      NaClLog(LOG_FATAL, "Could not map in shm for dynamic text region\n");
+  /* Existing memory is anonymous paging file backed. */
+  NaCl_page_free((void *) text_sysaddr, dynamic_text_size);
+
+  /*
+   * Unix allows us to map pages with PROT_NONE initially and later
+   * increase the mapping permissions with mprotect().
+   *
+   * Windows does not allow this, however: the initial permissions are
+   * an upper bound on what the permissions may later be changed to
+   * with VirtualProtect().  Given this, using PROT_NONE at this point
+   * does not even make sense.  So we map with read+exec and
+   * immediately turn down the permissions, so that we can later
+   * re-enable read+exec page by page.
+   */
+#if NACL_WINDOWS
+  mmap_protections = NACL_ABI_PROT_READ | NACL_ABI_PROT_EXEC;
+#else
+  mmap_protections = NACL_ABI_PROT_NONE;
+#endif
+  NaClLog(4,
+          "NaClMakeDynamicTextShared: Map(,,0x%"NACL_PRIxPTR",size = 0x%x,"
+          " prot=0x%x, flags=0x%x, offset=0)\n",
+          text_sysaddr,
+          (int) dynamic_text_size,
+          mmap_protections,
+          NACL_ABI_MAP_SHARED | NACL_ABI_MAP_FIXED);
+  mmap_ret = (*((struct NaClDescVtbl const *) shm->base.base.vtbl)->
+              Map)((struct NaClDesc *) shm,
+                   (struct NaClDescEffector *) &shm_effector,
+                   (void *) text_sysaddr,
+                   dynamic_text_size,
+                   mmap_protections,
+                   NACL_ABI_MAP_SHARED | NACL_ABI_MAP_FIXED,
+                   0);
+  if (text_sysaddr != mmap_ret) {
+    NaClLog(LOG_FATAL, "Could not map in shm for dynamic text region\n");
+  }
+
+#if NACL_WINDOWS
+  {
+    /*
+     * We need a loop here because the Map() call above creates one
+     * mapping per page.  However, there is no need for it to do that
+     * for the dynamic code area.
+     * TODO(mseaborn): Create a single mapping here.
+     */
+    uintptr_t offset;
+    for (offset = 0; offset < dynamic_text_size; offset += NACL_MAP_PAGESIZE) {
+      DWORD old_prot;
+      if (!VirtualProtect((void *) (text_sysaddr + offset), NACL_MAP_PAGESIZE,
+                          PAGE_NOACCESS, &old_prot)) {
+        NaClLog(LOG_FATAL,
+                "NaClMakeDynamicTextShared: VirtualProtect() failed to "
+                "set page permissions to PAGE_NOACCESS\n");
+      }
     }
   }
+#endif
+
+  nap->dynamic_page_bitmap =
+    BitmapAllocate((uint32_t) (dynamic_text_size / NACL_MAP_PAGESIZE));
+  if (NULL == nap->dynamic_page_bitmap) {
+    NaClLog(LOG_FATAL, "NaClMakeDynamicTextShared: BitmapAllocate() failed\n");
+  }
 
   nap->text_shm = &shm->base;
   retval = LOAD_OK;
@@ -387,25 +404,28 @@ void NaClSetThreadGeneration(struct NaClAppThread *natp, int generation) {
    * since threads only set their own generation it is safe
    */
   if (natp->dynamic_delete_generation != generation)  {
-    NaClMutexLock(&natp->mu);
+    NaClXMutexLock(&natp->mu);
     CHECK(natp->dynamic_delete_generation <= generation);
     natp->dynamic_delete_generation = generation;
-    NaClMutexUnlock(&natp->mu);
+    NaClXMutexUnlock(&natp->mu);
   }
 }
 
 int NaClMinimumThreadGeneration(struct NaClApp *nap) {
-  int i, rv = 0;
-  NaClMutexLock(&nap->threads_mu);
-  for (i = 0; i < nap->num_threads; ++i) {
-    struct NaClAppThread *thread = NaClGetThreadMu(nap, i);
-    NaClMutexLock(&thread->mu);
-    if (i == 0 || rv > thread->dynamic_delete_generation) {
-      rv = thread->dynamic_delete_generation;
+  size_t index;
+  int rv = INT_MAX;
+  NaClXMutexLock(&nap->threads_mu);
+  for (index = 0; index < nap->threads.num_entries; ++index) {
+    struct NaClAppThread *thread = NaClGetThreadMu(nap, (int) index);
+    if (thread != NULL) {
+      NaClXMutexLock(&thread->mu);
+      if (rv > thread->dynamic_delete_generation) {
+        rv = thread->dynamic_delete_generation;
+      }
+      NaClXMutexUnlock(&thread->mu);
     }
-    NaClMutexUnlock(&thread->mu);
   }
-  NaClMutexUnlock(&nap->threads_mu);
+  NaClXMutexUnlock(&nap->threads_mu);
   return rv;
 }
 
@@ -484,14 +504,39 @@ static INLINE void CopyCodeSafelyInitial(uint8_t  *dest,
   CopyBundleTails(dest, src, size, bundle_size);
   NaClWriteMemoryBarrier();
   CopyBundleHeads(dest, src, size, bundle_size);
+
   /*
-   * For security, we are fine; for correctness, another memory
-   * barrier is needed here.  We assume that there will be one later
-   * in the syscall processing and/or in the inter-thread
-   * communication needed for the code inserting thread to tell other
-   * threads that the newly inserted code is ready to use (and at what
-   * address it lives).
+   * Flush the processor's instruction cache.  This is not necessary
+   * for security, because any old cached instructions will just be
+   * safe halt instructions.  It is only necessary to ensure that
+   * untrusted code runs correctly when it tries to execute the
+   * dynamically-loaded code.
    */
+  NaClClearInstructionCache(dest, dest + size);
+}
+
+static void MakeDynamicCodePageVisible(struct NaClApp *nap,
+                                       uint32_t page_index,
+                                       uint8_t *writable_addr) {
+  void *user_addr;
+
+  if (BitmapIsBitSet(nap->dynamic_page_bitmap, page_index)) {
+    /* The page is already visible: nothing to do. */
+    return;
+  }
+  user_addr = (void *) NaClUserToSys(nap, nap->dynamic_text_start
+                                     + page_index * NACL_MAP_PAGESIZE);
+
+  /* Sanity check:  Ensure the page is not already in use. */
+  CHECK(*writable_addr == 0);
+
+  NaClFillMemoryRegionWithHalt(writable_addr, NACL_MAP_PAGESIZE);
+
+  if (NaCl_mprotect(user_addr, NACL_MAP_PAGESIZE, PROT_READ | PROT_EXEC) != 0) {
+    NaClLog(LOG_FATAL, "MakeDynamicCodePageVisible: NaCl_mprotect() failed\n");
+  }
+
+  BitmapSetBit(nap->dynamic_page_bitmap, page_index);
 }
 
 /*
@@ -504,28 +549,19 @@ static uintptr_t CachedMapWritableText(struct NaClApp *nap,
                                        uint32_t offset,
                                        uint32_t size) {
   /*
-   * The nap->* variables used in this function can be in 3 states:
+   * The nap->* variables used in this function can be in two states:
    *
    * 1)
-   * nap->dynamic_mapcache_offset = nap->dynamic_mapcache_size
-   *                                            = nap->dynamic_mapcache_ret = 0
+   * nap->dynamic_mapcache_size == 0
+   * nap->dynamic_mapcache_ret == 0
    *
    * Initial state, nothing is cached.
    *
    * 2)
-   * nap->dynamic_mapcache_offset != 0
    * nap->dynamic_mapcache_size != 0
-   * !NaClPtrIsNegError(nap->dynamic_mapcache_ret)
+   * nap->dynamic_mapcache_ret != 0
    *
    * We have a cached mmap result stored, that must be unmapped.
-   *
-   * 3)
-   * nap->dynamic_mapcache_offset != 0
-   * nap->dynamic_mapcache_size != 0
-   * NaClPtrIsNegError(nap->dynamic_mapcache_ret)
-   *
-   * The last mmap was an error, cache the error but don't unmap on next call.
-   *
    */
   struct NaClDescEffectorShm shm_effector;
   struct NaClDesc            *shm = nap->text_shm;
@@ -542,8 +578,7 @@ static uintptr_t CachedMapWritableText(struct NaClApp *nap,
     /*
      * cache miss, first clear the old cache if needed
      */
-    if (nap->dynamic_mapcache_size > 0
-               && !NaClPtrIsNegErrno(&nap->dynamic_mapcache_ret)) {
+    if (nap->dynamic_mapcache_size > 0) {
       if (0 != (*((struct NaClDescVtbl const *) shm->base.vtbl)->
             UnmapUnsafe)(shm,
                          (struct NaClDescEffector*) &shm_effector,
@@ -552,16 +587,20 @@ static uintptr_t CachedMapWritableText(struct NaClApp *nap,
         NaClLog(LOG_FATAL, "CachedMapWritableText: Failed to unmap\n");
         return -NACL_ABI_EFAULT;
       }
+      nap->dynamic_mapcache_offset = 0;
+      nap->dynamic_mapcache_size = 0;
+      nap->dynamic_mapcache_ret = 0;
     }
 
     /*
      * update that cached version
      */
-    nap->dynamic_mapcache_offset = offset;
-    nap->dynamic_mapcache_size = size;
     if (size > 0) {
-      CHECK(offset > 0);
-      nap->dynamic_mapcache_ret = (*((struct NaClDescVtbl const *)
+      uint32_t page_index;
+      uint32_t end_page_index;
+      uint8_t *writable_addr;
+
+      uintptr_t mapping = (*((struct NaClDescVtbl const *)
             shm->base.vtbl)->
               Map)(shm,
                    (struct NaClDescEffector*) &shm_effector,
@@ -570,12 +609,22 @@ static uintptr_t CachedMapWritableText(struct NaClApp *nap,
                    NACL_ABI_PROT_READ | NACL_ABI_PROT_WRITE,
                    NACL_ABI_MAP_SHARED,
                    offset);
-    } else {
-      /*
-       * if size is 0, just clear cache
-       */
-      CHECK(0 == offset);
-      nap->dynamic_mapcache_ret = 0;
+      if (NaClPtrIsNegErrno(&mapping)) {
+        return 0;
+      }
+
+      writable_addr = (uint8_t *) mapping;
+      end_page_index = (offset + size) / NACL_MAP_PAGESIZE;
+      for (page_index = offset / NACL_MAP_PAGESIZE;
+           page_index < end_page_index;
+           page_index++) {
+        MakeDynamicCodePageVisible(nap, page_index, writable_addr);
+        writable_addr += NACL_MAP_PAGESIZE;
+      }
+
+      nap->dynamic_mapcache_offset = offset;
+      nap->dynamic_mapcache_size = size;
+      nap->dynamic_mapcache_ret = mapping;
     }
   }
   return nap->dynamic_mapcache_ret;
@@ -610,7 +659,7 @@ static INLINE int NaclTextMapWrapper(struct NaClApp *nap,
   mmap_ret = CachedMapWritableText(nap,
                                    shm_map_offset,
                                    shm_map_size);
-  if (NaClPtrIsNegErrno(&mmap_ret)) {
+  if (0 == mmap_ret) {
     return 0;
   }
   mmap_result = (uint8_t *) mmap_ret;
@@ -640,17 +689,16 @@ static INLINE void NaclTextMapClearCacheIfNeeded(struct NaClApp *nap,
   }
 }
 
-int32_t NaClTextSysDyncode_Create(struct NaClAppThread *natp,
-                                uint32_t             dest,
-                                uint32_t             src,
-                                uint32_t             size) {
-  struct NaClApp              *nap = natp->nap;
+int32_t NaClTextDyncodeCreate(struct NaClApp *nap,
+                              uint32_t       dest,
+                              void           *code_copy,
+                              uint32_t       size) {
   uintptr_t                   dest_addr;
-  uintptr_t                   src_addr;
-  uint8_t                     *code_copy;
   uint8_t                     *mapped_addr;
   int32_t                     retval = -NACL_ABI_EINVAL;
   int                         validator_result;
+  struct NaClPerfCounter      time_dyncode_create;
+  NaClPerfCounterCtor(&time_dyncode_create, "NaClTextDyncodeCreate");
 
   if (NULL == nap->text_shm) {
     NaClLog(1, "NaClTextSysDyncode_Copy: Dynamic loading not enabled\n");
@@ -662,10 +710,8 @@ int32_t NaClTextSysDyncode_Create(struct NaClAppThread *natp,
     return -NACL_ABI_EINVAL;
   }
   dest_addr = NaClUserToSysAddrRange(nap, dest, size);
-  src_addr = NaClUserToSysAddrRange(nap, src, size);
-  if (kNaClBadAddress == dest_addr ||
-      kNaClBadAddress == src_addr) {
-    NaClLog(1, "NaClTextSysDyncode_Copy: Address out of range\n");
+  if (kNaClBadAddress == dest_addr) {
+    NaClLog(1, "NaClTextSysDyncode_Copy: Dest address out of range\n");
     return -NACL_ABI_EFAULT;
   }
   if (dest < nap->dynamic_text_start) {
@@ -685,22 +731,20 @@ int32_t NaClTextSysDyncode_Create(struct NaClAppThread *natp,
     return 0;
   }
 
-  /*
-   * Make a private copy of the code, so that we can validate it
-   * without a TOCTTOU race condition.
-   */
-  code_copy = malloc(size);
-  if (NULL == code_copy) {
-    retval = -NACL_ABI_ENOMEM;
-    goto cleanup_free;
-  }
-  memcpy(code_copy, (uint8_t*) src_addr, size);
-
-  NaClMutexLock(&nap->dynamic_load_mutex);
+  NaClXMutexLock(&nap->dynamic_load_mutex);
 
   if (NaClDynamicRegionCreate(nap, dest_addr, size) == 1) {
     /* target memory region is free */
-    validator_result = NaClValidateCode(nap, dest, code_copy, size);
+    if (!nap->skip_validator) {
+      validator_result = NaClValidateCode(nap, dest, code_copy, size);
+    } else {
+      NaClLog(LOG_ERROR, "VALIDATION SKIPPED.\n");
+      validator_result = LOAD_OK;
+    }
+
+    NaClPerfCounterMark(&time_dyncode_create,
+                        NACL_PERF_IMPORTANT_PREFIX "DynRegionValidate");
+    NaClPerfCounterIntervalLast(&time_dyncode_create);
   } else {
     /* target addr is in use */
     NaClLog(1, "NaClTextSysDyncode_Copy: Code range already allocated\n");
@@ -733,11 +777,38 @@ int32_t NaClTextSysDyncode_Create(struct NaClAppThread *natp,
   NaclTextMapClearCacheIfNeeded(nap, dest, size);
 
  cleanup_unlock:
-  NaClMutexUnlock(&nap->dynamic_load_mutex);
+  NaClXMutexUnlock(&nap->dynamic_load_mutex);
+  return retval;
+}
+
+int32_t NaClTextSysDyncode_Create(struct NaClAppThread *natp,
+                                  uint32_t             dest,
+                                  uint32_t             src,
+                                  uint32_t             size) {
+  struct NaClApp              *nap = natp->nap;
+  uintptr_t                   src_addr;
+  uint8_t                     *code_copy;
+  int32_t                     retval = -NACL_ABI_EINVAL;
+
+  src_addr = NaClUserToSysAddrRange(nap, src, size);
+  if (kNaClBadAddress == src_addr) {
+    NaClLog(1, "NaClTextSysDyncode_Copy: Source address out of range\n");
+    return -NACL_ABI_EFAULT;
+  }
+
+  /*
+   * Make a private copy of the code, so that we can validate it
+   * without a TOCTTOU race condition.
+   */
+  code_copy = malloc(size);
+  if (NULL == code_copy) {
+    return -NACL_ABI_ENOMEM;
+  }
+  memcpy(code_copy, (uint8_t*) src_addr, size);
+
+  retval = NaClTextDyncodeCreate(nap, dest, code_copy, size);
 
- cleanup_free:
   free(code_copy);
-
   return retval;
 }
 
@@ -763,12 +834,6 @@ int32_t NaClTextSysDyncode_Modify(struct NaClAppThread *natp,
     return -NACL_ABI_EINVAL;
   }
 
-  if (!nap->allow_dyncode_replacement) {
-    NaClLog(1, "NaClTextSysDyncode_Modify:"
-               "Dynamic code replacement not enabled\n");
-    return -NACL_ABI_EINVAL;
-  }
-
   if (0 == size) {
     /* Nothing to modify.  Succeed trivially. */
     return 0;
@@ -781,7 +846,7 @@ int32_t NaClTextSysDyncode_Modify(struct NaClAppThread *natp,
     return -NACL_ABI_EFAULT;
   }
 
-  NaClMutexLock(&nap->dynamic_load_mutex);
+  NaClXMutexLock(&nap->dynamic_load_mutex);
 
   region = NaClDynamicRegionFind(nap, dest_addr, size);
   if (NULL == region || region->start > dest_addr
@@ -861,7 +926,7 @@ int32_t NaClTextSysDyncode_Modify(struct NaClAppThread *natp,
   NaclTextMapClearCacheIfNeeded(nap, dest, size);
 
  cleanup_unlock:
-  NaClMutexUnlock(&nap->dynamic_load_mutex);
+  NaClXMutexUnlock(&nap->dynamic_load_mutex);
 
   if (code_copy != code_copy_buf) {
     free(code_copy);
@@ -884,12 +949,6 @@ int32_t NaClTextSysDyncode_Delete(struct NaClAppThread *natp,
     return -NACL_ABI_EINVAL;
   }
 
-  if (!nap->allow_dyncode_replacement) {
-    NaClLog(1, "NaClTextSysDyncode_Delete:"
-               "Dynamic code deletion not enabled\n");
-    return -NACL_ABI_EINVAL;
-  }
-
   dest_addr = NaClUserToSysAddrRange(nap, dest, size);
   if (kNaClBadAddress == dest_addr) {
     NaClLog(1, "NaClTextSysDyncode_Delete: Address out of range\n");
@@ -900,15 +959,15 @@ int32_t NaClTextSysDyncode_Delete(struct NaClAppThread *natp,
     /* Nothing to delete.  Just update our generation. */
     int gen;
     /* fetch current generation */
-    NaClMutexLock(&nap->dynamic_load_mutex);
+    NaClXMutexLock(&nap->dynamic_load_mutex);
     gen = nap->dynamic_delete_generation;
-    NaClMutexUnlock(&nap->dynamic_load_mutex);
+    NaClXMutexUnlock(&nap->dynamic_load_mutex);
     /* set our generation */
     NaClSetThreadGeneration(natp, gen);
     return 0;
   }
 
-  NaClMutexLock(&nap->dynamic_load_mutex);
+  NaClXMutexLock(&nap->dynamic_load_mutex);
 
   /*
    * this check ensures the to-be-deleted region is identical to a
@@ -967,6 +1026,6 @@ int32_t NaClTextSysDyncode_Delete(struct NaClAppThread *natp,
   }
 
  cleanup_unlock:
-  NaClMutexUnlock(&nap->dynamic_load_mutex);
+  NaClXMutexUnlock(&nap->dynamic_load_mutex);
   return retval;
 }
