@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -36,58 +36,55 @@
 #include "native_client/src/shared/platform/nacl_log.h"
 #include "native_client/src/shared/platform/nacl_threads.h"
 
+#include "native_client/src/shared/srpc/nacl_srpc.h"
+
 #include "native_client/src/trusted/service_runtime/dyn_array.h"
 #include "native_client/src/trusted/service_runtime/nacl_config_dangerous.h"
 #include "native_client/src/trusted/service_runtime/nacl_error_code.h"
-#include "native_client/src/trusted/service_runtime/nacl_sync_queue.h"
+#include "native_client/src/trusted/service_runtime/nacl_kern_services.h"
+#include "native_client/src/trusted/service_runtime/nacl_resource.h"
+
 #include "native_client/src/trusted/service_runtime/sel_mem.h"
 #include "native_client/src/trusted/service_runtime/sel_util.h"
 #include "native_client/src/trusted/service_runtime/sel_rt.h"
 
+#include "native_client/src/trusted/service_runtime/name_service/name_service.h"
 
 EXTERN_C_BEGIN
 
 #define NACL_SERVICE_PORT_DESCRIPTOR    3
 #define NACL_SERVICE_ADDRESS_DESCRIPTOR 4
 
-#define NACL_DEFAULT_ALLOC_MAX  (32 << 20)  /* total brk and mmap allocs */
 #define NACL_DEFAULT_STACK_MAX  (16 << 20)  /* main thread stack */
 
 #define NACL_SANDBOX_CHROOT_FD  "SBX_D"
 
-#if NACL_WINDOWS
-#define WINDOWS_EXCEPTION_TRY do { __try {
-#define WINDOWS_EXCEPTION_CATCH } __except(EXCEPTION_EXECUTE_HANDLER) { \
-                                  NaClLog(LOG_ERROR, \
-                                      "Unhandled Windows exception\n"); \
-                                  exit(1); \
-                                } \
-                              } while (0)
-#else
-#define WINDOWS_EXCEPTION_TRY do {
-#define WINDOWS_EXCEPTION_CATCH } while (0)
-#endif
-
 struct NaClAppThread;
 struct NaClDesc;  /* see native_client/src/trusted/desc/nacl_desc_base.h */
 struct NaClDynamicRegion;
+struct NaClManifestProxy;
+struct NaClSecureService;
+struct NaClSecureReverseService;
+struct NaClThreadInterface;  /* see sel_ldr_thread_interface.h */
+
+struct NaClDebugCallbacks {
+  void (*thread_create_hook)(struct NaClAppThread *natp);
+  void (*thread_exit_hook)(struct NaClAppThread *natp);
+  void (*process_exit_hook)(int exit_status);
+};
+
+enum NaClResourcePhase {
+  NACL_RESOURCE_PHASE_START,
+  NACL_RESOURCE_PHASE_REV_CHAN
+};
 
 struct NaClApp {
   /*
    * public, user settable prior to app start.
    */
   uint8_t                   addr_bits;
-  uintptr_t                 max_data_alloc;
   uintptr_t                 stack_size;
   /*
-   * max_data_alloc controls how much total data memory can be
-   * allocated to the NaCl process; this is initialized data,
-   * uninitialized data, and heap and affects the brk system call.
-   * the text size and rodata size are not included, even though in
-   * NaCl the text and rodata pages are also backed by the pager
-   * since due to relocation the text pages and rodata contents
-   * cannot simply be memory mapped from the executable.
-   *
    * stack_size is the maximum size of the (main) stack.  The stack
    * memory is eager allocated (mapped in w/o MAP_NORESERVE) so
    * there must be enough swap space; page table entries are not
@@ -109,9 +106,15 @@ struct NaClApp {
    */
   uintptr_t                 mem_start;
 
-#if NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && NACL_BUILD_SUBARCH == 32 && __PIC__
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 32 && __PIC__)
   uintptr_t                 pcrel_thunk;
 #endif
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 64)
+  uintptr_t                 dispatch_thunk;
+  uintptr_t                 get_tls_fast_path;
+#endif
 
   /* only used for ET_EXEC:  for CS restriction */
   uintptr_t                 static_text_end;  /* relative to mem_start */
@@ -144,7 +147,15 @@ struct NaClApp {
   uintptr_t                 data_end;
   /* see break_addr below */
 
-  uintptr_t                 entry_pt;
+  /*
+   * initial_entry_pt is the first address in untrusted code to jump
+   * to.  When using the IRT (integrated runtime), this is provided by
+   * the IRT library, and user_entry_pt is the entry point in the user
+   * executable.  Otherwise, initial_entry_pt is in the user
+   * executable and user_entry_pt is zero.
+   */
+  uintptr_t                 initial_entry_pt;
+  uintptr_t                 user_entry_pt;
 
   /*
    * bundle_size is the bundle alignment boundary for validation (16
@@ -168,11 +179,53 @@ struct NaClApp {
   struct NaClDesc           *service_port;
   struct NaClDesc           *service_address;
 
-  struct NaClDesc           *secure_channel;
-  struct NaClThread         secure_channel_thr;  /* valid iff secure_channel */
-
   struct NaClMutex          mu;
   struct NaClCondVar        cv;
+
+  /*
+   * invariant: !(vm_hole_may_exist && threads_launching != 0).
+   * vm_hole_may_exist is set while mmap/munmap manipulates the memory
+   * map, and threads_launching is set while a thread is launching
+   * (and a trusted thread stack is being allocated).
+   *
+   * strictly speaking, vm_hole_may_exist need not be present, since
+   * the vm code ensures that 0 == threads_launching and then holds
+   * the lock for the duration of the VM operation.  it is safer this
+   * way, in case we later introduce code that might want to
+   * temporarily drop the process lock.
+   */
+  int                       vm_hole_may_exist;
+  int                       threads_launching;
+
+  /*
+   * An array of NaCl syscall handlers. The length of the array must be
+   * at least NACL_MAX_SYSCALLS.
+   */
+  struct NaClSyscallTableEntry *syscall_table;
+
+  /*
+   * Name service must launch after mu, cv, vm_hole_may_exit,
+   * threads_launching are initialized.
+   */
+  struct NaClNameService    *name_service;  /* default name server */
+  struct NaClDesc           *name_service_conn_cap;
+
+  struct NaClSecureService        *secure_service;
+  struct NaClManifestProxy        *manifest_proxy;
+  struct NaClKernService          *kern_service;
+
+  struct NaClResourceNaClApp      resources;
+  enum NaClResourcePhase          resource_phase;
+
+  struct NaClSecureReverseClient  *reverse_client;
+  enum NaClReverseChannelInitializationState {
+    NACL_REVERSE_CHANNEL_UNINITIALIZED,
+    NACL_REVERSE_CHANNEL_INITIALIZATION_STARTED,
+    NACL_REVERSE_CHANNEL_INITIALIZED
+  }                               reverse_channel_initialization_state;
+  struct NaClSrpcChannel          reverse_channel;
+
+
   NaClErrorCode             module_load_status;
   int                       module_may_start;
 
@@ -191,12 +244,26 @@ struct NaClApp {
    */
   struct NaClVmmap          mem_map;
 
+  /*
+   * This is the effector interface object that is used to manipulate
+   * NaCl apps by the objects in the NaClDesc class hierarchy.  This
+   * is used by this NaClApp when making NaClDesc method calls from
+   * syscall handlers.
+   */
+  struct NaClDescEffector   *effp;
+
   /*
    * may reject nexes that are incompatible w/ dynamic-text in the near future
    */
   int                       use_shm_for_dynamic_text;
   struct NaClDesc           *text_shm;
   struct NaClMutex          dynamic_load_mutex;
+  /*
+   * This records which pages in text_shm have been allocated.  When a
+   * page is allocated, it is filled with halt instructions and then
+   * made executable by untrusted code.
+   */
+  uint8_t                   *dynamic_page_bitmap;
 
   /*
    * The array of dynamic_regions is maintained in sorted order
@@ -226,10 +293,8 @@ struct NaClApp {
   int                       exit_status;
 
   int                       ignore_validator_result;
+  int                       skip_validator;
   int                       validator_stub_out_mode;
-  int                       allow_dyncode_replacement;
-  /* all threads enqueue the "special" syscalls to the work queue */
-  struct NaClSyncQueue      work_queue;
 
 #if NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && NACL_BUILD_SUBARCH == 32
   uint16_t                  code_seg_sel;
@@ -251,12 +316,32 @@ struct NaClApp {
 
   struct NaClMutex          desc_mu;
   struct DynArray           desc_tbl;  /* NaClDesc pointers */
+
+  int                       enable_debug_stub;
+  struct NaClDebugCallbacks *debug_stub_callbacks;
+  struct NaClMutex          exception_mu;
+  uint32_t                  exception_handler;
 };
 
 
 
 void  NaClAppIncrVerbosity(void);
 
+/*
+ * Initializes a NaCl application with the default parameters
+ * and the specified syscall table.
+ *
+ * nap is a pointer to the NaCl object that is being filled in.
+ *
+ * table is the NaCl syscall table. The syscall table must contain at least
+ * NACL_MAX_SYSCALLS valid entries.
+ *
+ * Caution! Syscall handlers must be extremely careful with respect to
+ * argument validation, including time-of-check vs time-of-use defense, etc.
+ */
+int NaClAppWithSyscallTableCtor(struct NaClApp               *nap,
+                                struct NaClSyscallTableEntry *table) NACL_WUR;
+
 int   NaClAppCtor(struct NaClApp  *nap) NACL_WUR;
 
 /*
@@ -282,14 +367,11 @@ int   NaClAppCtor(struct NaClApp  *nap) NACL_WUR;
  * self-modifying code / data writes and automatically invalidate the
  * cache lines.
  */
-enum NaClAbiCheckOption {
-  NACL_ABI_CHECK_OPTION_SKIP,
-  NACL_ABI_CHECK_OPTION_CHECK
-};
-
 NaClErrorCode NaClAppLoadFile(struct Gio      *gp,
-                              struct NaClApp  *nap,
-                              enum NaClAbiCheckOption check_abi) NACL_WUR;
+                              struct NaClApp  *nap) NACL_WUR;
+
+NaClErrorCode NaClAppLoadFileDynamically(struct NaClApp *nap,
+                                         struct Gio     *gio_file) NACL_WUR;
 
 size_t  NaClAlignPad(size_t val,
                      size_t align);
@@ -343,6 +425,30 @@ void NaClAddImcHandle(struct NaClApp  *nap,
                       NaClHandle      h,
                       int             nacl_desc);
 
+/*
+ * Launch system-level service threads.  After this, access to the
+ * NaClApp object must be done in a thread-safe manner, using nap->mu
+ * etc, or access only read-only data.
+ *
+ * NB: the "secure command channel" thread should have already started
+ * (if enabled); that thread must take care to not race with the main
+ * thread that is continuing to set up the NaCl module as well.
+ */
+int NaClAppLaunchServiceThreads(struct NaClApp *nap);
+
+/*
+ * Report the low eight bits of |exit_status| via the reverse channel
+ * in |nap|, if one exists, to whomever is interested.  This usually
+ * involves an RPC.  Returns true if successfully reported.
+ *
+ * Also mark nap's exit_status and running member variables, announce
+ * via condvar that the nexe should be considered no longer running.
+ *
+ * Returns true (non-zero) if exit status was reported via the reverse
+ * channel, and false (0) otherwise.
+ */
+int NaClReportExitStatus(struct NaClApp *nap, int exit_status);
+
 /*
  * Used to launch the main thread.  NB: calling thread may in the
  * future become the main NaCl app thread, and this function will
@@ -357,7 +463,16 @@ int NaClCreateMainThread(struct NaClApp     *nap,
 
 int NaClWaitForMainThreadToExit(struct NaClApp  *nap);
 
-int NaClMainForChromium(int handle_count, const NaClHandle *handles, int debug);
+/*
+ * Register the integrated runtime (IRT) library file for use by
+ * NaClMainForChromium().  This takes a file descriptor, even on
+ * Windows (where file descriptors are emulated by the C runtime
+ * library).
+ */
+void NaClSetIrtFileDesc(int fd);
+
+void NaClMainForChromium(int handle_count, const NaClHandle *handles,
+                         int debug);
 
 /*
  * Used by syscall code.
@@ -365,8 +480,8 @@ int NaClMainForChromium(int handle_count, const NaClHandle *handles, int debug);
 int32_t NaClCreateAdditionalThread(struct NaClApp *nap,
                                    uintptr_t      prog_ctr,
                                    uintptr_t      stack_ptr,
-                                   uintptr_t      sys_tdb,
-                                   size_t         tdb_size) NACL_WUR;
+                                   uintptr_t      sys_tls,
+                                   uint32_t       user_tls2) NACL_WUR;
 
 void NaClLoadTrampoline(struct NaClApp *nap);
 
@@ -374,9 +489,7 @@ void NaClLoadSpringboard(struct NaClApp  *nap);
 
 static const uintptr_t kNaClBadAddress = (uintptr_t) -1;
 
-#ifndef NACL_NO_INLINE
 #include "native_client/src/trusted/service_runtime/sel_ldr-inl.h"
-#endif
 
 /*
  * Looks up a descriptor in the open-file table.  An additional
@@ -427,6 +540,8 @@ void NaClRemoveThreadMu(struct NaClApp  *nap,
 struct NaClAppThread *NaClGetThreadMu(struct NaClApp  *nap,
                                       int             thread_num);
 
+void NaClAppInitialDescriptorHookup(struct NaClApp  *nap);
+
 void NaClAppVmmapUpdate(struct NaClApp    *nap,
                         uintptr_t         page_num,
                         size_t            npages,
@@ -447,22 +562,66 @@ void NaClSendServiceAddressTo(struct NaClApp  *nap,
 
 void NaClSecureCommandChannel(struct NaClApp  *nap);
 
+int NaClSecureReverseClientInsertHandler(
+    struct NaClSecureReverseClient  *self,
+    void                            (*handler)(
+        void                                   *handler_state,
+        struct NaClThreadInterface             *thread_if,
+        struct NaClDesc                        *new_conn),
+    void                            *handler_state) NACL_WUR;
+
 NaClErrorCode NaClWaitForLoadModuleStatus(struct NaClApp *nap) NACL_WUR;
 
 NaClErrorCode NaClWaitForStartModuleCommand(struct NaClApp *nap) NACL_WUR;
 
+/*
+ * NaClBlockIfCommandChannelExists is used during error exit.  If
+ * there is a secure command channel, we sent an RPC reply with the
+ * reason that the nexe was rejected.  If we exit immediately, that
+ * reply may still be in-flight and the various channel closure (esp
+ * reverse channels, if those were set up) may be detected first by
+ * the controlling process on the other end of the command channel or
+ * reverse channel.  When channel closure wins the race against the
+ * RPC reply, it would result in a crash being reported, rather than
+ * the error code carried in the RPC reply.  We want to ensure that
+ * the RPC reply to get processed.  Instead of allowing the service
+ * runtime process to exit, we block the main thread and wait for the
+ * hard-shutdown on the command channel or command channel closure.
+ *
+ * If there is no command channel, NaClBlockIfCommandChannelExists
+ * just returns immediately.
+ */
+void NaClBlockIfCommandChannelExists(struct NaClApp *nap);
+
 void NaClFillMemoryRegionWithHalt(void *start, size_t size);
 
 void NaClFillTrampolineRegion(struct NaClApp *nap);
 
 void NaClFillEndOfTextRegion(struct NaClApp *nap);
 
-#if NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && NACL_BUILD_SUBARCH == 32 && __PIC__
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 32 && __PIC__)
 
 int NaClMakePcrelThunk(struct NaClApp *nap);
 
 #endif
 
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 64)
+
+int NaClMakeDispatchThunk(struct NaClApp *nap);
+void NaClPatchOneTrampolineCall(uintptr_t call_target_addr,
+                                uintptr_t target_addr);
+
+#endif
+
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 32)
+
+void NaClPatchDebuggerInfo(struct NaClApp *nap);
+
+#endif
+
 void NaClPatchOneTrampoline(struct NaClApp *nap,
                             uintptr_t target_addr);
 /*
@@ -512,26 +671,38 @@ int NaClThreadContextCtor(struct NaClThreadContext  *ntcp,
 
 void NaClThreadContextDtor(struct NaClThreadContext *ntcp);
 
+void NaClVmHoleWaitToStartThread(struct NaClApp *nap);
 
-/*
- * Used in mmap code to prepare the untrusted module for mmap.
- * Specifically, on *x this is a no-op, but on Windows where mmap does
- * not atomically replace mappings and we have to unmap (VirtualFree
- * or UnmapViewOfFile) before installing new mappings, we stop all
- * other untrusted threads using SuspendThread in
- * NaClAppBeginMapRaceProtection and then ResumeThread in
- * NaClAppEndMapRaceProtection.  These functions must be called after
- * all the necessary locks for the bracketed intervening operations,
- * since no other thread that might otherwise release locks is able to
- * run.
- */
-void NaClAppBeginMapRaceProtection_AppMu(struct NaClAppThread *natp);
-
-void NaClAppEndMapRaceProtection_AppMu(struct NaClAppThread *natp);
-
+void NaClVmHoleThreadStackIsSafe(struct NaClApp *nap);
 
 void NaClGdbHook(struct NaClApp const *nap);
 
+#if NACL_WINDOWS
+
+void NaClUntrustedThreadsSuspend(struct NaClApp *nap);
+void NaClUntrustedThreadsResume(struct NaClApp *nap);
+
+#else
+
+/*
+ * Suspending untrusted threads is only needed for preventing mmap
+ * races on Windows, so these are no-ops on other platforms.
+ */
+
+static INLINE void NaClUntrustedThreadsSuspend(struct NaClApp *nap) {
+  UNREFERENCED_PARAMETER(nap);
+}
+
+static INLINE void NaClUntrustedThreadsResume(struct NaClApp *nap) {
+  UNREFERENCED_PARAMETER(nap);
+}
+
+#endif
+
+#if NACL_LINUX
+void handle_r_debug(const char *switch_value, char *argv0);
+#endif
+
 EXTERN_C_END
 
 #endif  /* NATIVE_CLIENT_SRC_TRUSTED_SERVICE_RUNTIME_SEL_LDR_H_ */
