@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -10,6 +10,14 @@
 #include "native_client/src/include/portability.h"
 #include "native_client/src/include/portability_io.h"
 
+#if NACL_OSX
+#include <crt_externs.h>
+#endif
+
+#if NACL_LINUX
+#include <getopt.h>
+#endif
+
 #include <errno.h>
 #include <limits.h>
 #include <stdio.h>
@@ -24,23 +32,30 @@
 #include "native_client/src/shared/gio/gio.h"
 #include "native_client/src/shared/imc/nacl_imc_c.h"
 #include "native_client/src/shared/platform/nacl_check.h"
+#include "native_client/src/shared/platform/nacl_exit.h"
 #include "native_client/src/shared/platform/nacl_log.h"
 #include "native_client/src/shared/platform/nacl_sync.h"
 #include "native_client/src/shared/platform/nacl_sync_checked.h"
 #include "native_client/src/shared/srpc/nacl_srpc.h"
 
-#ifdef NACL_BREAKPAD
-#include "native_client/src/trusted/nacl_breakpad/nacl_breakpad.h"
-#endif
+#include "native_client/src/trusted/fault_injection/fault_injection.h"
+#include "native_client/src/trusted/perf_counter/nacl_perf_counter.h"
+#include "native_client/src/trusted/service_runtime/env_cleanser.h"
+#include "native_client/src/trusted/service_runtime/include/sys/fcntl.h"
 #include "native_client/src/trusted/service_runtime/nacl_app.h"
 #include "native_client/src/trusted/service_runtime/nacl_all_modules.h"
 #include "native_client/src/trusted/service_runtime/nacl_config_dangerous.h"
-#include "native_client/src/trusted/service_runtime/nacl_debug.h"
 #include "native_client/src/trusted/service_runtime/nacl_globals.h"
+#include "native_client/src/trusted/service_runtime/nacl_signal.h"
 #include "native_client/src/trusted/service_runtime/nacl_syscall_common.h"
+#include "native_client/src/trusted/service_runtime/nacl_valgrind_hooks.h"
 #include "native_client/src/trusted/service_runtime/outer_sandbox.h"
 #include "native_client/src/trusted/service_runtime/sel_ldr.h"
 #include "native_client/src/trusted/service_runtime/sel_qualify.h"
+#include "native_client/src/trusted/service_runtime/win/exception_patch/ntdll_patch.h"
+#if NACL_WINDOWS
+#include "native_client/src/trusted/service_runtime/win/debug_exception_handler.h"
+#endif
 
 static void VmentryPrinter(void           *state,
                     struct NaClVmmapEntry *vmep) {
@@ -94,17 +109,6 @@ int ImportModeMap(char opt) {
   /* NOTREACHED */
 }
 
-int NaClDup2(int d_old, int d_new) {
-  if (d_old == d_new) {
-    return 0;
-  }
-  if (DUP2(d_old, d_new) == -1) {
-    fprintf(stderr, "sel_ldr: dup2 for log file failed\n");
-    exit(1);
-  }
-  return 1;
-}
-
 
 static void PrintUsage() {
   /* NOTE: this is broken up into multiple statements to work around
@@ -123,6 +127,7 @@ static void PrintUsage() {
           "    respectively\n"
           " -i associates an IMC handle D with app desc d\n"
           " -f file to load; if omitted, 1st arg after \"--\" is loaded\n"
+          " -B additional ELF file to load as a blob library\n"
           " -v increases verbosity\n"
           " -X create a bound socket and export the address via an\n"
           "    IMC message to a corresponding NaCl app descriptor\n"
@@ -135,21 +140,31 @@ static void PrintUsage() {
           "\n"
           " (testing flags)\n"
           " -a allow file access! dangerous!\n"
-          " -c ignore validator! dangerous!\n"
+          " -c ignore validator! dangerous! Repeating this option twice skips\n"
+          "    validation completely.\n"
           " -F fuzz testing; quit after loading NaCl app\n"
-          " -g enable gdb debug stub.\n"
-          " -I disable ELF ABI version number check (safe)\n"
+          " -S enable signal handling.  Not secure on x86-64 Windows.\n"
+          " -g enable gdb debug stub.  Not secure on x86-64 Windows.\n"
           " -l <file>  write log output to the given file\n"
           " -s safely stub out non-validating instructions\n"
           " -Q disable platform qualification (dangerous!)\n"
+          " -E <name=value>|<name> set an environment variable\n"
           );  /* easier to add new flags/lines */
 }
 
-/*
- * NB: do NOT return from main until our dependency on SDL is removed.
- * This is because the OSX version of SDL ignores main's return value
- * and always exits with an exit status of 0, breaking tests.
- */
+#if NACL_LINUX
+static const struct option longopts[] = {
+  { "r_debug", required_argument, NULL, 'D' },
+  { NULL, 0, NULL, 0 }
+};
+
+static int my_getopt(int argc, char *const *argv, const char *shortopts) {
+  return getopt_long(argc, argv, shortopts, longopts, NULL);
+}
+#else
+#define my_getopt getopt
+#endif
+
 int main(int  argc,
          char **argv) {
   int                           opt;
@@ -161,33 +176,46 @@ int main(int  argc,
 
   struct NaClApp                state;
   char                          *nacl_file = NULL;
+  char                          *blob_library_file = NULL;
   int                           rpc_supplies_nexe = 0;
   int                           export_addr_to = -2;
-  enum NaClAbiCheckOption       check_abi = NACL_ABI_CHECK_OPTION_CHECK;
 
   struct NaClApp                *nap;
 
   struct GioFile                gout;
-  NaClErrorCode                 errcode;
-  struct GioMemoryFileSnapshot  gf;
-
+  NaClErrorCode                 errcode = LOAD_INTERNAL;
+  struct GioMemoryFileSnapshot  blob_file;
 
   int                           ret_code;
   struct DynArray               env_vars;
 
   char                          *log_file = NULL;
-  struct GioFile                *log_gio;
-  int                           log_desc;
   int                           verbosity = 0;
   int                           fuzzing_quit_after_load = 0;
   int                           debug_mode_bypass_acl_checks = 0;
   int                           debug_mode_ignore_validator = 0;
   int                           stub_out_mode = 0;
   int                           skip_qualification = 0;
-  int                           start_broken = 0;
+  int                           enable_debug_stub = 0;
+  int                           handle_signals = 0;
+  int                           exception_handling = 0;
+  struct NaClPerfCounter        time_all_main;
+  const char                    **envp;
+  struct NaClEnvCleanser        env_cleanser;
+
 
   const char* sandbox_fd_string;
 
+#if NACL_OSX
+  /* Mac dynamic libraries cannot access the environ variable directly. */
+  envp = (const char **) *_NSGetEnviron();
+#else
+  /* Overzealous code style check is overzealous. */
+  /* @IGNORE_LINES_FOR_CODE_HYGIENE[1] */
+  extern char **environ;
+  envp = (const char **) environ;
+#endif
+
 #if NACL_ARCH(NACL_BUILD_ARCH) == NACL_arm || NACL_SANDBOX_FIXED_AT_ZERO == 1
   /*
    * Set malloc not to use mmap even for large allocations.  This is currently
@@ -214,16 +242,12 @@ int main(int  argc,
   redir_queue = NULL;
   redir_qend = &redir_queue;
 
-  /*
-   * Set an exception handler so Windows won't show a message box if
-   * an exception occurs
-   */
-  WINDOWS_EXCEPTION_TRY;
-
   NaClAllModulesInit();
 
   verbosity = NaClLogGetVerbosity();
 
+  NaClPerfCounterCtor(&time_all_main, "SelMain");
+
   fflush((FILE *) NULL);
 
   if (!GioFileRefCtor(&gout, stdout)) {
@@ -234,28 +258,58 @@ int main(int  argc,
   if (!DynArrayCtor(&env_vars, 0)) {
     NaClLog(LOG_FATAL, "Failed to allocate env var array\n");
   }
-  while ((opt = getopt(argc, argv, "abcE:f:Fgh:i:Il:Qr:Rsvw:X:")) != -1) {
+  /*
+   * On platforms with glibc getopt, require POSIXLY_CORRECT behavior,
+   * viz, no reordering of the arglist -- stop argument processing as
+   * soon as an unrecognized argument is encountered, so that, for
+   * example, in the invocation
+   *
+   *   sel_ldr foo.nexe -vvv
+   *
+   * the -vvv flags are made available to the nexe, rather than being
+   * consumed by getopt.  This makes the behavior of the Linux build
+   * of sel_ldr consistent with the Windows and OSX builds.
+   */
+  while ((opt = my_getopt(argc, argv,
+#if NACL_LINUX
+                       "+D:"
+#endif
+                       "aB:ceE:f:Fgh:i:Il:Qr:RsSvw:X:")) != -1) {
     switch (opt) {
+      case 'e':
+        exception_handling = 1;
+        break;
+#if NACL_LINUX
+      case 'D':
+        handle_r_debug(optarg, argv[0]);
+        break;
+#endif
       case 'c':
-        fprintf(stderr, "DEBUG MODE ENABLED (ignore validator)\n");
-        debug_mode_ignore_validator = 1;
+        ++debug_mode_ignore_validator;
         break;
       case 'a':
         fprintf(stderr, "DEBUG MODE ENABLED (bypass acl)\n");
         debug_mode_bypass_acl_checks = 1;
         break;
-      case 'b':
-        start_broken = 1;
-        break;
       case 'f':
         nacl_file = optarg;
         break;
+      case 'B':
+        blob_library_file = optarg;
+        break;
       case 'F':
         fuzzing_quit_after_load = 1;
         break;
+
       case 'g':
-        NaClDebugSetAllow(1);
+        handle_signals = 1;
+        enable_debug_stub = 1;
         break;
+
+      case 'S':
+        handle_signals = 1;
+        break;
+
       case 'h':
       case 'r':
       case 'w':
@@ -287,9 +341,6 @@ int main(int  argc,
         *redir_qend = entry;
         redir_qend = &entry->next;
         break;
-      case 'I':
-        check_abi = NACL_ABI_CHECK_OPTION_SKIP;
-        break;
       case 'l':
         log_file = optarg;
         break;
@@ -338,6 +389,32 @@ int main(int  argc,
     }
   }
 
+  if (exception_handling) {
+#if NACL_WINDOWS
+    int status;
+    DWORD exit_code;
+    status = NaClLaunchAndDebugItself(argv[0], &exit_code);
+    if (status == DEBUG_EXCEPTION_HANDLER_NOT_SUPPORTED) {
+      exception_handling = 0;
+    } else if (status == DEBUG_EXCEPTION_HANDLER_SUCCESS) {
+      return exit_code;
+    } else if (status == DEBUG_EXCEPTION_HANDLER_ERROR) {
+      fprintf(stderr, "ERROR in debug exception handling %d\n", GetLastError());
+      return -1;
+    }
+#else
+    exception_handling = 0;
+#endif
+    if (!exception_handling) {
+      fprintf(stderr, "WARNING: exception handling is not supported\n");
+    }
+  }
+
+  if (debug_mode_ignore_validator == 1)
+    fprintf(stderr, "DEBUG MODE ENABLED (ignore validator)\n");
+  else if (debug_mode_ignore_validator > 1)
+    fprintf(stderr, "DEBUG MODE ENABLED (skip validator)\n");
+
   if (verbosity) {
     int         ix;
     char const  *separator = "";
@@ -350,9 +427,6 @@ int main(int  argc,
     putc('\n', stderr);
   }
 
-  /* Check if we should start broken */
-  if (start_broken) NaClDebugSetStartBroken(1);
-
   if (NACL_DANGEROUS_STUFF_ENABLED) {
     fprintf(stderr,
             "WARNING WARNING WARNING WARNING"
@@ -391,20 +465,6 @@ int main(int  argc,
     NaClLogSetFile(log_file);
   }
 
-  /*
-   * NB: the following cast is okay since we only ever permit GioFile
-   * objects to be used -- NaClLogModuleInit and NaClLogSetFile both
-   * can only assign the log output to a file.  If neither were
-   * called, logging goes to stderr.
-   */
-  log_gio = (struct GioFile *) NaClLogGetGio();
-  /*
-   * By default, the logging module logs to stderr, or descriptor 2.
-   * If NaClLogSetFile was performed above, then log_desc will have
-   * the non-default value.
-   */
-  log_desc = fileno(log_gio->iop);
-
   if (rpc_supplies_nexe) {
     if (NULL != nacl_file) {
       fprintf(stderr,
@@ -447,8 +507,10 @@ int main(int  argc,
     goto done_file_dtor;
   }
 
-  state.ignore_validator_result = debug_mode_ignore_validator;
+  state.ignore_validator_result = (debug_mode_ignore_validator > 0);
+  state.skip_validator = (debug_mode_ignore_validator > 1);
   state.validator_stub_out_mode = stub_out_mode;
+  state.enable_debug_stub = enable_debug_stub;
 
   nap = &state;
   errcode = LOAD_OK;
@@ -463,9 +525,22 @@ int main(int  argc,
 
   /*
    * Ensure the platform qualification checks pass.
+   *
+   * NACL_DANGEROUS_SKIP_QUALIFICATION_TEST is used by tsan / memcheck
+   * (see src/third_party/valgrind/).
    */
+  if (!skip_qualification &&
+      getenv("NACL_DANGEROUS_SKIP_QUALIFICATION_TEST") != NULL) {
+    fprintf(stderr, "PLATFORM QUALIFICATION DISABLED BY ENVIRONMENT - "
+            "Native Client's sandbox will be unreliable!\n");
+    skip_qualification = 1;
+  }
+
+  /* We use the signal handler to verify a signal took place. */
+  NaClSignalHandlerInit();
   if (!skip_qualification) {
-    NaClErrorCode pq_error = NaClRunSelQualificationTests();
+    NaClErrorCode pq_error = NACL_FI_VAL("pq", NaClErrorCode,
+                                         NaClRunSelQualificationTests());
     if (LOAD_OK != pq_error) {
       errcode = pq_error;
       nap->module_load_status = pq_error;
@@ -476,15 +551,55 @@ int main(int  argc,
     }
   }
 
+  /* Remove the signal handler if we are not using it. */
+  if (!handle_signals) {
+    NaClSignalHandlerFini();
+    /* Sanity check. */
+    NaClSignalAssertNoHandlers();
+
+    /*
+     * Patch the Windows exception dispatcher to be safe in the case
+     * of faults inside x86-64 sandboxed code.  The sandbox is not
+     * secure on 64-bit Windows without this.
+     */
+#if (NACL_WINDOWS && NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && \
+     NACL_BUILD_SUBARCH == 64)
+    NaClPatchWindowsExceptionDispatcher();
+#endif
+  }
+
+  /*
+   * Open both files first because (on Mac OS X at least)
+   * NaClAppLoadFile() enables an outer sandbox.
+   */
+  if (NULL != blob_library_file) {
+    NaClFileNameForValgrind(blob_library_file);
+    if (0 == GioMemoryFileSnapshotCtor(&blob_file, blob_library_file)) {
+      perror("sel_main");
+      fprintf(stderr, "Cannot open \"%s\".\n", blob_library_file);
+      exit(1);
+    }
+    NaClPerfCounterMark(&time_all_main, "SnapshotBlob");
+    NaClPerfCounterIntervalLast(&time_all_main);
+  }
+
+  NaClAppInitialDescriptorHookup(nap);
+
   if (!rpc_supplies_nexe) {
-    if (0 == GioMemoryFileSnapshotCtor(&gf, nacl_file)) {
+    struct GioMemoryFileSnapshot main_file;
+
+    NaClFileNameForValgrind(nacl_file);
+    if (0 == GioMemoryFileSnapshotCtor(&main_file, nacl_file)) {
       perror("sel_main");
       fprintf(stderr, "Cannot open \"%s\".\n", nacl_file);
       exit(1);
     }
+    NaClPerfCounterMark(&time_all_main, "SnapshotNaclFile");
+    NaClPerfCounterIntervalLast(&time_all_main);
 
     if (LOAD_OK == errcode) {
-      errcode = NaClAppLoadFile((struct Gio *) &gf, nap, check_abi);
+      NaClLog(2, "Loading nacl file %s (non-RPC)\n", nacl_file);
+      errcode = NaClAppLoadFile((struct Gio *) &main_file, nap);
       if (LOAD_OK != errcode) {
         fprintf(stderr, "Error while loading \"%s\": %s\n",
                 nacl_file,
@@ -495,6 +610,8 @@ int main(int  argc,
                  "or a corrupt nexe file may be"
                  " responsible for this error.\n"));
       }
+      NaClPerfCounterMark(&time_all_main, "AppLoadEnd");
+      NaClPerfCounterIntervalLast(&time_all_main);
 
       NaClXMutexLock(&nap->mu);
       nap->module_load_status = errcode;
@@ -502,6 +619,12 @@ int main(int  argc,
       NaClXMutexUnlock(&nap->mu);
     }
 
+    if (-1 == (*((struct Gio *) &main_file)->vtbl->Close)((struct Gio *)
+                                                          &main_file)) {
+      fprintf(stderr, "Error while closing \"%s\".\n", nacl_file);
+    }
+    (*((struct Gio *) &main_file)->vtbl->Dtor)((struct Gio *) &main_file);
+
     if (fuzzing_quit_after_load) {
       exit(0);
     }
@@ -515,7 +638,7 @@ int main(int  argc,
    * close descriptors, since the underlying host OS will do so
    * as part of service runtime exit.
    */
-  NaClLog(4, "Processing I/O redirection/inheritance from environment\n");
+  NaClLog(4, "Processing I/O redirection/inheritance from command line\n");
   for (entry = redir_queue; NULL != entry; entry = entry->next) {
     switch (entry->tag) {
       case HOST_DESC:
@@ -555,6 +678,7 @@ int main(int  argc,
       NaClSecureCommandChannel(nap);
     }
   }
+
   /*
    * May have created a thread, so need to synchronize uses of nap
    * contents henceforth.
@@ -562,6 +686,8 @@ int main(int  argc,
 
   if (rpc_supplies_nexe) {
     errcode = NaClWaitForLoadModuleStatus(nap);
+    NaClPerfCounterMark(&time_all_main, "WaitForLoad");
+    NaClPerfCounterIntervalLast(&time_all_main);
   } else {
     /**************************************************************************
      * TODO(bsy): This else block should be made unconditional and
@@ -578,40 +704,48 @@ int main(int  argc,
       }
 
       /*
-       * Finish setting up the NaCl App.  This includes dup'ing
-       * descriptors 0-2 and making them available to the NaCl App.
-       *
-       * If a log file were specified at the command-line and the
-       * logging module was set to use a log file instead of standard
-       * error, then we redirect standard output of the NaCl module to
-       * that log file too.  Standard input is inherited, and could
-       * result in a NaCl module competing for input from the terminal;
-       * for graphical / browser plugin environments, this never is
-       * allowed to happen, and having this is useful for debugging, and
-       * for potential standalone text-mode applications of NaCl.
-       *
-       * TODO(bsy): consider whether inheriting stdin should occur only
-       * in debug mode.
+       * Finish setting up the NaCl App.  On x86-32, this means
+       * allocating segment selectors.  On x86-64 and ARM, this is
+       * (currently) a no-op.
        */
-      errcode = NaClAppPrepareToLaunch(nap,
-                                       0,
-                                       (2 == log_desc) ? 1 : log_desc,
-                                       log_desc);
+      errcode = NaClAppPrepareToLaunch(nap);
       if (LOAD_OK != errcode) {
         nap->module_load_status = errcode;
         fprintf(stderr, "NaClAppPrepareToLaunch returned %d", errcode);
       }
+      NaClPerfCounterMark(&time_all_main, "AppPrepLaunch");
+      NaClPerfCounterIntervalLast(&time_all_main);
     }
 
-    if (-1 == (*((struct Gio *) &gf)->vtbl->Close)((struct Gio *) &gf)) {
-      fprintf(stderr, "Error while closing \"%s\".\n", argv[optind]);
-    }
-    (*((struct Gio *) &gf)->vtbl->Dtor)((struct Gio *) &gf);
-
     /* Give debuggers a well known point at which xlate_base is known.  */
     NaClGdbHook(&state);
   }
 
+
+  if (NULL != blob_library_file) {
+    if (LOAD_OK == errcode) {
+      NaClLog(2, "Loading blob file %s\n", blob_library_file);
+      errcode = NaClAppLoadFileDynamically(nap, (struct Gio *) &blob_file);
+      if (LOAD_OK != errcode) {
+        fprintf(stderr, "Error while loading \"%s\": %s\n",
+                blob_library_file,
+                NaClErrorString(errcode));
+      }
+      NaClPerfCounterMark(&time_all_main, "BlobLoaded");
+      NaClPerfCounterIntervalLast(&time_all_main);
+    }
+
+    if (-1 == (*((struct Gio *) &blob_file)->vtbl->Close)((struct Gio *)
+                                                          &blob_file)) {
+      fprintf(stderr, "Error while closing \"%s\".\n", blob_library_file);
+    }
+    (*((struct Gio *) &blob_file)->vtbl->Dtor)((struct Gio *) &blob_file);
+    if (verbosity) {
+      gprintf((struct Gio *) &gout, "printing post-IRT NaClApp details\n");
+      NaClAppPrintDetails(nap, (struct Gio *) &gout);
+    }
+  }
+
   /*
    * Print out a marker for scripts to use to mark the start of app
    * output.
@@ -638,7 +772,7 @@ int main(int  argc,
     errno = 0;  /* To distinguish between success/failure after call */
     fd_long = strtol(sandbox_fd_string, &endptr, 10);
 
-    fprintf(stdout, "Chrooting the NaCl module\n");
+    NaClLog(1, "Chrooting the NaCl module\n");
     if ((ERANGE == errno && (LONG_MAX == fd_long || LONG_MIN == fd_long)) ||
         (0 != errno && 0 == fd_long)) {
       perror("strtol");
@@ -682,11 +816,17 @@ int main(int  argc,
    */
   fflush((FILE *) NULL);
 
-  if (NULL != nap->secure_channel && LOAD_OK == errcode) {
+  if (NULL != nap->secure_service) {
+    NaClErrorCode start_result;
     /*
      * wait for start_module RPC call on secure channel thread.
      */
-    errcode = NaClWaitForStartModuleCommand(nap);
+    start_result = NaClWaitForStartModuleCommand(nap);
+    NaClPerfCounterMark(&time_all_main, "WaitedForStartModuleCommand");
+    NaClPerfCounterIntervalLast(&time_all_main);
+    if (LOAD_OK == errcode) {
+      errcode = start_result;
+    }
   }
 
   /*
@@ -703,27 +843,44 @@ int main(int  argc,
   if (!DynArraySet(&env_vars, env_vars.num_entries, NULL)) {
     NaClLog(LOG_FATAL, "Adding env_vars NULL terminator failed\n");
   }
-  /*
-   * only nap->ehdrs.e_entry is usable, no symbol table is
-   * available.
-   */
+
+  NaClEnvCleanserCtor(&env_cleanser, 0);
+  if (!NaClEnvCleanserInit(&env_cleanser, envp,
+          (char const *const *)env_vars.ptr_array)) {
+    NaClLog(LOG_FATAL, "Failed to initialise env cleanser\n");
+  }
+
+  if (!NaClAppLaunchServiceThreads(nap)) {
+    fprintf(stderr, "Launch service threads failed\n");
+    goto done;
+  }
   if (!NaClCreateMainThread(nap,
                             argc - optind,
                             argv + optind,
-                            (const char **) env_vars.ptr_array)) {
+                            NaClEnvCleanserEnvironment(&env_cleanser))) {
     fprintf(stderr, "creating main thread failed\n");
     goto done;
   }
+
+  NaClEnvCleanserDtor(&env_cleanser);
+
+  NaClPerfCounterMark(&time_all_main, "CreateMainThread");
+  NaClPerfCounterIntervalLast(&time_all_main);
   DynArrayDtor(&env_vars);
 
   ret_code = NaClWaitForMainThreadToExit(nap);
+  NaClPerfCounterMark(&time_all_main, "WaitForMainThread");
+  NaClPerfCounterIntervalLast(&time_all_main);
+
+  NaClPerfCounterMark(&time_all_main, "SelMainEnd");
+  NaClPerfCounterIntervalTotal(&time_all_main);
 
   /*
    * exit_group or equiv kills any still running threads while module
    * addr space is still valid.  otherwise we'd have to kill threads
    * before we clean up the address space.
    */
-  _exit(ret_code);
+  NaClExit(ret_code);
 
  done:
   fflush(stdout);
@@ -736,6 +893,17 @@ int main(int  argc,
     PrintVmmap(nap);
     fflush(stdout);
   }
+  /*
+   * If there is a secure command channel, we sent an RPC reply with
+   * the reason that the nexe was rejected.  If we exit now, that
+   * reply may still be in-flight and the various channel closure (esp
+   * reverse channel) may be detected first.  This would result in a
+   * crash being reported, rather than the error in the RPC reply.
+   * Instead, we wait for the hard-shutdown on the command channel.
+   */
+  if (LOAD_OK != errcode) {
+    NaClBlockIfCommandChannelExists(nap);
+  }
 
  done_file_dtor:
   if (verbosity > 0) {
@@ -743,9 +911,11 @@ int main(int  argc,
   }
   fflush(stdout);
 
+  if (handle_signals) NaClSignalHandlerFini();
   NaClAllModulesFini();
 
-  WINDOWS_EXCEPTION_CATCH;
+  NaClExit(ret_code);
 
-  _exit(ret_code);
+  /* Unreachable, but having the return prevents a compiler error. */
+  return ret_code;
 }
