@@ -1,8 +1,7 @@
 /*
- * Copyright 2009 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
- * Copyright 2009, Google Inc.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 #ifndef NATIVE_CLIENT_SRC_TRUSTED_VALIDATOR_ARM_V2_INST_CLASSES_H
@@ -120,17 +119,27 @@ class ClassDecoder {
   }
 
   /*
-   * For instructions that can write memory, gets the register used as the base
-   * for generating the effective address.
+   * For instructions that can read or write memory, gets the register used as
+   * the base for generating the effective address.
    *
-   * The result is useful only for safe instructions where writes_memory() is
-   * true.  It is stubbed to return nonsense.
+   * It is stubbed to return nonsense.
    */
   virtual Register base_address_register(Instruction i) const {
     UNREFERENCED_PARAMETER(i);
     return kRegisterNone;
   }
 
+  /*
+   * Checks whether the instruction computes its read or write address as
+   * base address + immediate.
+   *
+   * It is stubbed to return false.
+   */
+  virtual bool offset_is_immediate(Instruction i) const {
+    UNREFERENCED_PARAMETER(i);
+    return false;
+  }
+
   /*
    * For indirect branch instructions, returns the register being moved into
    * r15.  Otherwise, reports kRegisterNone.
@@ -450,17 +459,6 @@ class LongMultiply : public Multiply {
   virtual RegisterList defs(Instruction i) const;
 };
 
-/*
- * Models a weird variation on double-word multiply instructions, where
- * the destination registers are identified in 15:12 and 11:8.
- */
-class LongMultiply15_8 : public Multiply {
- public:
-  virtual ~LongMultiply15_8() {}
-
-  virtual RegisterList defs(Instruction i) const;
-};
-
 /*
  * Saturating adds and subtracts.  Conceptually equivalent to DataProc,
  * except for the use of the S bit (bit 20) -- they always set flags.
@@ -571,7 +569,9 @@ class LoadRegister : public AbstractLoad {
  public:
   virtual ~LoadRegister() {}
 
+  virtual SafetyLevel safety(Instruction i) const;
   virtual RegisterList defs(Instruction i) const;
+  virtual Register base_address_register(Instruction i) const;
 };
 
 /*
@@ -586,6 +586,8 @@ class LoadImmediate : public AbstractLoad {
   virtual ~LoadImmediate() {}
 
   virtual RegisterList immediate_addressing_defs(Instruction i) const;
+  virtual Register base_address_register(Instruction i) const;
+  virtual bool offset_is_immediate(Instruction i) const;
 };
 
 /*
@@ -596,6 +598,8 @@ class LoadDoubleI : public LoadImmediate {
   virtual ~LoadDoubleI() {}
 
   virtual RegisterList defs(Instruction i) const;
+  virtual Register base_address_register(Instruction i) const;
+  virtual bool offset_is_immediate(Instruction i) const;
 };
 
 /*
@@ -605,7 +609,9 @@ class LoadDoubleR : public LoadRegister {
  public:
   virtual ~LoadDoubleR() {}
 
+  virtual SafetyLevel safety(Instruction i) const;
   virtual RegisterList defs(Instruction i) const;
+  virtual Register base_address_register(Instruction i) const;
 };
 
 /*
@@ -614,6 +620,7 @@ class LoadDoubleR : public LoadRegister {
 class LoadExclusive : public AbstractLoad {
  public:
   virtual ~LoadExclusive() {}
+  virtual Register base_address_register(Instruction i) const;
 };
 
 /*
@@ -624,6 +631,7 @@ class LoadDoubleExclusive : public LoadExclusive {
   virtual ~LoadDoubleExclusive() {}
 
   virtual RegisterList defs(Instruction i) const;
+  virtual Register base_address_register(Instruction i) const;
 };
 
 /*
@@ -637,6 +645,7 @@ class LoadMultiple : public ClassDecoder {
   virtual SafetyLevel safety(Instruction i) const;
   virtual RegisterList defs(Instruction i) const;
   virtual RegisterList immediate_addressing_defs(Instruction i) const;
+  virtual Register base_address_register(Instruction i) const;
 };
 
 /*
@@ -650,6 +659,7 @@ class VectorLoad : public ClassDecoder {
   virtual SafetyLevel safety(Instruction i) const;
   virtual RegisterList defs(Instruction i) const;
   virtual RegisterList immediate_addressing_defs(Instruction i) const;
+  virtual Register base_address_register(Instruction i) const;
 };
 
 /*
