@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -13,7 +13,10 @@
 #endif
 
 #include <stdio.h>
-#include "native_client/src/trusted/validator_x86/ncdecode.h"
+#include "native_client/src/shared/gio/gio.h"
+#include "native_client/src/shared/platform/nacl_log.h"
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncdecode.h"
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncdecode_verbose.h"
 
 struct NCDecodeTestCase {
   char *description;
@@ -37,7 +40,7 @@ struct NCDecodeTestCase NCDecoderTests[] = {
   },
   {
     "test2",
-    342,
+    341,
     (uint8_t *)"\x8d\x4c\x24\x04"
     "\x83\xe4\xf0"
     "\xff\x71\xfc"
@@ -328,18 +331,20 @@ void ncdecode_unittests() {
     printf("%s\n", NCDecoderTests[i].description);
     NCDecodeSegment(NCDecoderTests[i].testbytes,
                     (uint32_t)(NCDecoderTests[i].testbytes),
-                    NCDecoderTests[i].testsize,
-                    NULL);
+                    NCDecoderTests[i].testsize);
   }
 }
 
-/* TODO: move this into header */
-extern void PrintInstStdout(const struct NCDecoderState *mstate);
-
-
 int main() {
-  NCDecodeRegisterCallbacks(PrintInstStdout, NULL, NULL, NULL);
+  struct GioFile gio_out_stream;
+  struct Gio *gout = (struct Gio*) &gio_out_stream;
+  if (!GioFileRefCtor(&gio_out_stream, stdout)) {
+    fprintf(stderr, "Unable to create gio file for stdout!\n");
+    return 1;
+  }
+  NaClLogModuleInitExtended(LOG_INFO, gout);
   ncdecode_unittests();
   printf("PASSED\n");
+  GioFileDtor(gout);
   return 0;
 }
