@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2012 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -12,18 +12,21 @@
 #error("This file is not meant for use in the TCB")
 #endif
 
-#include "native_client/src/include/portability.h"
 #include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "native_client/src/trusted/validator_x86/ncvalidate.h"
-#include "native_client/src/trusted/validator_x86/ncvalidate_internaltypes.h"
+#include "native_client/src/include/nacl_macros.h"
+#include "native_client/src/include/portability.h"
+#include "native_client/src/shared/gio/gio.h"
+#include "native_client/src/shared/platform/nacl_log.h"
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncdecode_verbose.h"
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncvalidate.h"
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncvalidate_internaltypes.h"
 
-#if !defined(ARRAYSIZE)
-#define ARRAYSIZE(a) sizeof(a)/sizeof(a[0])
-#endif
+/* Define the set of CPU features to use while validating. */
+static CPUFeatures g_ncval_cpu_features;
 
 void Info(const char *fmt, ...)
 {
@@ -49,6 +52,11 @@ struct NCValTestCase {
 };
 
 struct NCValTestCase NCValTests[] = {
+  /* NOTE: Many of these tests are now in the textual testing structure in
+   * native_client/src/trusted/validator_x86/testdata/32 using
+   * files "test-n.hex", "test-n.ndis", "test-n.nvals", and
+   * "test-n.nvals16".
+   */
   {
     "test 1",
     "a first very simple test with an illegal inst.",
@@ -328,7 +336,7 @@ struct NCValTestCase NCValTests[] = {
   {
     "test 14",
     "test 14: ldmxcsr, stmxcsr",
-    /* sawfailure= */ 1, /* illegalinst= */ 2,
+    /* sawfailure= */ 0, /* illegalinst= */ 0,
     /* instructions= */ 10,
     /* vaddr= */ 0x80000000,
     "90 0f ae 10 90 0f ae 18 \n"
@@ -822,7 +830,8 @@ struct NCValTestCase NCValTests[] = {
     /* sawfailure= */ 0, /* illegalinst= */ 0,
     /* instructions= */ 4,
     /* vaddr= */ 0x80000000,
-    "0f 60 00 90 90 90 f4 \n"
+    "0f 60 00 \n" /* punpcklbw (%eax),%mm0 */
+    "90 90 90 f4 \n"
   },
   {
     "test 69",
@@ -991,6 +1000,22 @@ struct NCValTestCase NCValTests[] = {
     /* sawfailure= */ 1, /* illegalinst= */ 1,
     /* instructions= */ 3,
     /* vaddr= */ 0x80000000,
+    /* Note: Fixed so that this is a legal instruction,
+     * except for the prefix! (karl)
+     */
+    "2e 0f 3a 0f bb ab 00 00 00 00 \n"
+    "90 90 \n"
+    "f4 \n"                             /* hlt                             */
+  },
+  {
+    "test 87a",
+    "test 87a: bad NACLi_3BYTE instruction (not really an instruction)",
+    /* sawfailure= */ 1, /* illegalinst= */ 2,
+    /* instructions= */ 2,
+    /* vaddr= */ 0x80000000,
+    /* Note: Fixed so that this is a legal instruction,
+     * except for the prefix! (karl)
+     */
     "2e 0f 3a 7d bb ab 00 00 00 00 \n"
     "90 90 \n"
     "f4 \n"                             /* hlt                             */
@@ -1114,7 +1139,7 @@ static void DecodeHexString(const char *input, uint8_t **result_data,
   *result_size = output - buf;
 }
 
-static void TestValidator(struct NCValTestCase *vtest) {
+static void TestValidator(struct NCValTestCase *vtest, int didstubout) {
   struct NCValidatorState *vstate;
   uint8_t *byte0;
   size_t data_size;
@@ -1128,20 +1153,27 @@ static void TestValidator(struct NCValTestCase *vtest) {
    */
   assert(byte0[data_size - 1] == 0xf4 /* HLT */);
 
-  vstate = NCValidateInit(vtest->vaddr, vtest->vaddr + data_size, 16);
+  vstate = NCValidateInit(vtest->vaddr, data_size - 1, 16);
   assert (vstate != NULL);
+  NCValidateSetErrorReporter(vstate, &kNCVerboseErrorReporter);
   NCValidateSegment(byte0, (uint32_t)vtest->vaddr, data_size - 1, vstate);
   free(byte0);
   rc = NCValidateFinish(vstate);
+
   do {
+    printf("vtest->sawfailure = %d, vstate->stats.sawfailure = %d\n",
+           vtest->sawfailure, vstate->stats.sawfailure);
+    NCStatsPrint(vstate);
+    if (vtest->sawfailure != rc) break;
     if (vtest->sawfailure ^ vstate->stats.sawfailure) break;
+    if (didstubout != vstate->stats.didstubout) break;
     if (vtest->instructions != vstate->stats.instructions) break;
     if (vtest->illegalinst != vstate->stats.illegalinst) break;
     Info("*** %s passed (%s)\n", vtest->name, vtest->description);
     NCValidateFreeState(&vstate);
     return;
   } while (0);
-  Stats_Print(stderr, vstate);
+  NCStatsPrint(vstate);
   NCValidateFreeState(&vstate);
   Info("*** %s failed (%s)\n", vtest->name, vtest->description);
   exit(-1);
@@ -1150,33 +1182,64 @@ static void TestValidator(struct NCValTestCase *vtest) {
 void test_fail_on_bad_alignment() {
   struct NCValidatorState *vstate;
 
-  vstate = NCValidateInit(0x80000000, 0x80001000, 16);
+  vstate = NCValidateInit(0x80000000, 0x1000, 16);
   assert (vstate != NULL);
   NCValidateFreeState(&vstate);
 
   /* Unaligned start addresses are not allowed. */
-  vstate = NCValidateInit(0x80000001, 0x80001000, 16);
+  vstate = NCValidateInit(0x80000001, 0x1000, 16);
   assert (vstate == NULL);
 
   /* Only alignments of 32 and 64 bytes are supported. */
-  vstate = NCValidateInit(0x80000000, 0x80001000, 64);
+  vstate = NCValidateInit(0x80000000, 0x1000, 64);
   assert (vstate == NULL);
 }
 
+void test_stubout() {
+  /* Similar to test 68 */
+  struct NCValTestCase test = {
+    "test stubout",
+    "test stubout: NACLi_MMX",
+    /* sawfailure= */ 0, /* illegalinst= */ 0,
+    /* instructions= */ 1,
+    /* vaddr= */ 0x80000000,
+    "0f 60 00 f4 \n" /* punpcklbw (%eax),%mm0 */
+  };
+
+  /* If MMX instructions are not allowed, stubout will occur. */
+  g_ncval_cpu_features.f_MMX = FALSE;
+  TestValidator(&test, TRUE);
+  g_ncval_cpu_features.f_MMX = TRUE;
+}
+
 void ncvalidate_unittests() {
   size_t i;
 
-  for (i = 0; i < ARRAYSIZE(NCValTests); i++) {
-    TestValidator(&NCValTests[i]);
+  /* Default to stubbing out nothing. */
+  NaClSetAllCPUFeatures(&g_ncval_cpu_features);
+  NCValidateSetCPUFeatures(&g_ncval_cpu_features);
+
+  for (i = 0; i < NACL_ARRAY_SIZE(NCValTests); i++) {
+    TestValidator(&NCValTests[i], FALSE);
   }
 
   test_fail_on_bad_alignment();
+  test_stubout();
 
   Info("\nAll tests passed.\n\n");
 }
 
 
 int main() {
+  struct GioFile gio_out_stream;
+  struct Gio *gout = (struct Gio*) &gio_out_stream;
+  if (!GioFileRefCtor(&gio_out_stream, stdout)) {
+    fprintf(stderr, "Unable to create gio file for stdout!\n");
+    return 1;
+  }
+
+  NaClLogModuleInitExtended(LOG_INFO, gout);
   ncvalidate_unittests();
+  GioFileDtor(gout);
   return 0;
 }
