@@ -1,11 +1,12 @@
 /*
- * Copyright 2009 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
+/* TODO(bradchen): figure out where to move this include file and then move it */
 
-#ifndef NATIVE_CLIENT_SRC_UNTRUSTED_SYSCALL_BINDINGS_TRAMPOLINE_H
-#define NATIVE_CLIENT_SRC_UNTRUSTED_SYSCALL_BINDINGS_TRAMPOLINE_H
+#ifndef NATIVE_CLIENT_SRC_UNTRUSTED_NACL_SYSCALL_BINDINGS_TRAMPOLINE_H
+#define NATIVE_CLIENT_SRC_UNTRUSTED_NACL_SYSCALL_BINDINGS_TRAMPOLINE_H
 
 #if __cplusplus
 extern "C" {
@@ -21,7 +22,6 @@ struct NaClImcMsgHdr;
 struct stat;
 struct timeval;
 struct timespec;
-union NaClMultimediaEvent;
 
 #define NACL_SYSCALL(s) ((TYPE_nacl_ ## s) NACL_SYSCALL_ADDR(NACL_sys_ ## s))
 
@@ -33,15 +33,15 @@ union NaClMultimediaEvent;
  * parking instrumentation (tls related calls).
  */
 
-extern void nacl_pre_syscall_hook();
-extern void nacl_post_syscall_hook();
+extern void IRT_pre_irtcall_hook(void);
+extern void IRT_post_irtcall_hook(void);
 
 #define NACL_GC_WRAP_SYSCALL(_expr) \
   ({                                \
     __typeof__(_expr) __sysret;     \
-    nacl_pre_syscall_hook();        \
+    IRT_pre_irtcall_hook();        \
     __sysret = _expr;               \
-    nacl_post_syscall_hook();       \
+    IRT_post_irtcall_hook();       \
     __sysret;                       \
   })
 
@@ -49,6 +49,12 @@ extern void nacl_post_syscall_hook();
 /* files */
 /* ============================================================ */
 
+typedef int (*TYPE_nacl_nameservice)(int *desc_in_out);
+
+typedef int (*TYPE_nacl_dup)(int oldfd);
+
+typedef int (*TYPE_nacl_dup2)(int oldfd, int newfd);
+
 typedef int (*TYPE_nacl_read) (int desc, void *buf, size_t count);
 
 typedef int (*TYPE_nacl_close) (int desc);
@@ -105,15 +111,15 @@ typedef int (*TYPE_nacl_munmap) (void *start, size_t length);
 typedef void (*TYPE_nacl_thread_exit) (int32_t *stack_flag);
 typedef int (*TYPE_nacl_thread_create) (void *start_user_address,
                                         void *stack,
-                                        void *tdb,
-                                        size_t tdb_size);
+                                        void *thread_ptr,
+                                        void *second_thread_ptr);
 typedef int (*TYPE_nacl_thread_nice) (const int nice);
 
 /* ============================================================ */
 /* mutex */
 /* ============================================================ */
 
-typedef int (*TYPE_nacl_mutex_create) ();
+typedef int (*TYPE_nacl_mutex_create) (void);
 typedef int (*TYPE_nacl_mutex_lock) (int mutex);
 typedef int (*TYPE_nacl_mutex_unlock) (int mutex);
 typedef int (*TYPE_nacl_mutex_trylock) (int mutex);
@@ -122,13 +128,13 @@ typedef int (*TYPE_nacl_mutex_trylock) (int mutex);
 /* condvar */
 /* ============================================================ */
 
-typedef int (*TYPE_nacl_cond_create) ();
+typedef int (*TYPE_nacl_cond_create) (void);
 typedef int (*TYPE_nacl_cond_wait) (int cv, int mutex);
 typedef int (*TYPE_nacl_cond_signal) (int cv);
 typedef int (*TYPE_nacl_cond_broadcast) (int cv);
 typedef int (*TYPE_nacl_cond_timed_wait_abs) (int condvar,
                                               int mutex,
-                                              struct timespec *abstime);
+                                              const struct timespec *abstime);
 
 /* ============================================================ */
 /* semaphore */
@@ -138,23 +144,6 @@ typedef int (*TYPE_nacl_sem_create) (int32_t value);
 typedef int (*TYPE_nacl_sem_wait) (int sem);
 typedef int (*TYPE_nacl_sem_post) (int sem);
 
-/* ============================================================ */
-/* multimedia */
-/* ============================================================ */
-
-typedef int (*TYPE_nacl_multimedia_init)(int subsystems);
-typedef int (*TYPE_nacl_multimedia_shutdown)();
-typedef int (*TYPE_nacl_video_init)(int width, int height);
-typedef int (*TYPE_nacl_video_shutdown)();
-typedef int (*TYPE_nacl_video_update)(const void* data);
-typedef int (*TYPE_nacl_video_poll_event)(union NaClMultimediaEvent *event);
-/* NOTE: we cannot forward declare enums in C */
-typedef int (*TYPE_nacl_audio_init)(/*enum NaClAudioFormat*/ int format,
-                                    int desired_samples,
-                                    int *obtained_samples);
-typedef int (*TYPE_nacl_audio_shutdown)();
-typedef int (*TYPE_nacl_audio_stream)(const void *data, size_t *size);
-
 /* ============================================================ */
 /* misc */
 /* ============================================================ */
@@ -163,7 +152,7 @@ typedef int (*TYPE_nacl_getdents) (int desc, void *dirp, size_t count);
 
 typedef int (*TYPE_nacl_gettimeofday) (struct timeval *tv, void *tz);
 
-typedef int (*TYPE_nacl_sched_yield) ();
+typedef int (*TYPE_nacl_sched_yield) (void);
 
 typedef int (*TYPE_nacl_sysconf) (int name, int *res);
 
@@ -184,12 +173,13 @@ typedef void (*TYPE_nacl_exit) (int status);
 
 typedef void (*TYPE_nacl_null) (void);
 
-typedef int (*TYPE_nacl_tls_init) (void *tdb, int size);
+typedef int (*TYPE_nacl_tls_init) (void *thread_ptr);
 
-typedef void *(*TYPE_nacl_tls_get) ();
+typedef void *(*TYPE_nacl_tls_get) (void);
 
-typedef int (*TYPE_nacl_dyncode_copy) (void *dest, const void *src,
-                                       size_t size);
+typedef int (*TYPE_nacl_second_tls_set) (void *new_value);
+
+typedef void *(*TYPE_nacl_second_tls_get) (void);
 
 typedef int (*TYPE_nacl_dyncode_create) (void *dest, const void *src,
                                        size_t size);
@@ -199,8 +189,18 @@ typedef int (*TYPE_nacl_dyncode_modify) (void *dest, const void *src,
 
 typedef int (*TYPE_nacl_dyncode_delete) (void *dest, size_t size);
 
+typedef int (*TYPE_nacl_exception_handler) (
+                  void (*handler) (int eip, int esp),
+                  void (**old_handler) (int eip, int esp));
+
+typedef int (*TYPE_nacl_exception_stack) (void* stack, size_t size);
+
+typedef int (*TYPE_nacl_exception_clear_flag) (void);
+
+typedef int (*TYPE_nacl_test_infoleak) (void);
+
 #if __cplusplus
 }
 #endif
 
-#endif  /*  NATIVE_CLIENT_SRC_UNTRUSTED_SYSCALL_BINDINGS_TRAMPOLINE_H */
+#endif  /*  NATIVE_CLIENT_SRC_UNTRUSTED_NACL_SYSCALL_BINDINGS_TRAMPOLINE_H */
