@@ -1,10 +1,9 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
-
 /*
  * Native Client threads library
  */
@@ -19,19 +18,30 @@
 
 #include "native_client/src/include/nacl_base.h"
 
+#include "native_client/src/untrusted/irt/irt_interfaces.h"
+#include "native_client/src/untrusted/nacl/nacl_irt.h"
 #include "native_client/src/untrusted/nacl/tls.h"
-#include "native_client/src/untrusted/nacl/syscall_bindings_trampoline.h"
+#include "native_client/src/untrusted/nacl/tls_params.h"
 
 #include "native_client/src/untrusted/pthread/nc_hash.h"
 #include "native_client/src/untrusted/pthread/pthread.h"
+#include "native_client/src/untrusted/pthread/pthread_internal.h"
 #include "native_client/src/untrusted/pthread/pthread_types.h"
 
 #include "native_client/src/untrusted/valgrind/dynamic_annotations.h"
 
 #define FUN_TO_VOID_PTR(a) ((void*)((uintptr_t) a))
 
+/*
+ * ABI tables for underyling NaCl thread interfaces.
+ */
+static struct nacl_irt_thread irt_thread;
+
 static const uint32_t kStackAlignment = 32;
 
+#define TDB_SIZE \
+  (sizeof(nc_thread_descriptor_t) + sizeof(nc_basic_thread_data_t))
+
 static inline char* align(uint32_t offset, uint32_t alignment) {
   return (char*) ((offset + alignment - 1) & ~(alignment - 1));
 }
@@ -56,10 +66,6 @@ pthread_cond_t __nc_last_thread_cond;
 /* number of threads currently running in this NaCl module */
 int __nc_running_threads_counter;
 
-/*
- * Array for TSD keys status management. */
-tsd_status_t __nc_thread_specific_storage[PTHREAD_KEYS_MAX];
-
 /* we have two queues of memory blocks - one for each type */
 STAILQ_HEAD(tailhead, entry) __nc_thread_memory_blocks[2];
 /* We need a counter for each queue to keep track of number of blocks */
@@ -70,8 +76,13 @@ int __nc_memory_block_counter[2];
 
 /* Internal functions */
 
-static inline nc_thread_descriptor_t *nc_get_tdb() {
-  return NACL_SYSCALL(tls_get)();
+static inline nc_thread_descriptor_t *nc_get_tdb(void) {
+  /*
+   * Fetch the thread-specific data pointer.  This is usually just
+   * a wrapper around __libnacl_irt_tls.tls_get() but we don't use
+   * that here so that the IRT build can override the definition.
+   */
+  return (void *) ((char *) __nacl_read_tp() + __nacl_tp_tdb_offset(TDB_SIZE));
 }
 
 static int nc_allocate_thread_id_mu(nc_basic_thread_data_t *basic_data) {
@@ -101,7 +112,7 @@ static int nc_allocate_thread_id_mu(nc_basic_thread_data_t *basic_data) {
     }
     if (!found) {
       /* all the IDs are in use?! */
-      return -EAGAIN;
+      return EAGAIN;
     }
   }
 
@@ -125,7 +136,6 @@ void nc_thread_starter(nc_thread_function func,
   __newlib_thread_init();
   retval = tdb->start_func(tdb->state);
   /* if the function returns, terminate the thread */
-  tdb->exiting_without_returning = 0;
   pthread_exit(retval);
   /* NOTREACHED */
   /* TODO(gregoryd) - add assert */
@@ -162,40 +172,50 @@ static nc_thread_memory_block_t* nc_allocate_memory_block_mu(
      * as they are added to the queue, therefore there is no need to check the
      * next queue entries if the first one is still in use.
      */
-    if (0 == node->is_used) {
+    if (0 == node->is_used && node->size >= required_size) {
+      /* This will only re-use the first node possibly, and could be
+       * improved to provide the stack with a best-fit algorithm if needed.
+       * TODO: we should scan all nodes to see if there is one that fits
+       *   before allocating another.
+       *   http://code.google.com/p/nativeclient/issues/detail?id=1569
+       */
+      int size = node->size;
       STAILQ_REMOVE_HEAD(head, entries);
       --__nc_memory_block_counter[type];
-      while (__nc_memory_block_counter[type] > __nc_kMaxCachedMemoryBlocks) {
-        /*
-         * We have too many blocks in the queue - try to release some.
-         * The maximum number of memory blocks to keep in the queue
-         * is almost arbitrary and can be tuned.
-         * The main limitation is that if we keep too many
-         * blocks in the queue, the NaCl app will run out of memory,
-         * since the default thread stack size is 512K.
-         * TODO(gregoryd): we might give up reusing stack entries once we
-         * support variable stack size.
-         */
-        nc_thread_memory_block_t *tmp = STAILQ_FIRST(head);
-        if (0 == tmp->is_used) {
-          STAILQ_REMOVE_HEAD(head, entries);
-          --__nc_memory_block_counter[type];
-          free(tmp);
-        } else {
-          /*
-           * Stop once we find a block that is still in use,
-           * since probably there is no point to continue
-           */
-          break;
-        }
-      }
+
       memset(node, 0,sizeof(*node));
-      node->size = required_size;
+      node->size = size;
       node->is_used = 1;
       return node;
     }
+
+    while (__nc_memory_block_counter[type] > __nc_kMaxCachedMemoryBlocks) {
+      /*
+       * We have too many blocks in the queue - try to release some.
+       * The maximum number of memory blocks to keep in the queue
+       * is almost arbitrary and can be tuned.
+       * The main limitation is that if we keep too many
+       * blocks in the queue, the NaCl app will run out of memory,
+       * since the default thread stack size is 512K.
+       * TODO(gregoryd): we might give up reusing stack entries once we
+       * support variable stack size.
+       */
+      nc_thread_memory_block_t *tmp = STAILQ_FIRST(head);
+      if (0 == tmp->is_used) {
+        STAILQ_REMOVE_HEAD(head, entries);
+        --__nc_memory_block_counter[type];
+        free(tmp);
+      } else {
+        /*
+         * Stop once we find a block that is still in use,
+         * since probably there is no point to continue
+         */
+        break;
+      }
+    }
+
   }
-  /* no available blocks of the required type - allocate one */
+  /* no available blocks of the required type/size - allocate one */
   node = malloc(MEMORY_BLOCK_ALLOCATION_SIZE(required_size));
   if (NULL != node) {
     memset(node, 0, sizeof(*node));
@@ -215,10 +235,10 @@ static void nc_free_memory_block_mu(nc_thread_memory_block_type_t type,
 
 static void nc_release_basic_data_mu(nc_basic_thread_data_t *basic_data) {
   if (NACL_PTHREAD_ILLEGAL_THREAD_ID != basic_data->thread_id) {
-    HASH_REMOVE(&__nc_thread_threads,
-                basic_data->thread_id,
-                nc_basic_thread_data,
-                hash_entry);
+    (void)HASH_REMOVE(&__nc_thread_threads,
+                      basic_data->thread_id,
+                      nc_basic_thread_data,
+                      hash_entry);
   }
   /* join_condvar can be initialized only if tls_node exists */
   pthread_cond_destroy(&basic_data->join_condvar);
@@ -228,7 +248,8 @@ static void nc_release_basic_data_mu(nc_basic_thread_data_t *basic_data) {
 static void nc_release_tls_node(nc_thread_memory_block_t *block) {
   if (block) {
     nc_thread_descriptor_t* tdb =
-      (nc_thread_descriptor_t *) __nacl_tls_tdb_start(NODE_TO_PAYLOAD(block));
+        (nc_thread_descriptor_t *) __nacl_tls_tdb_start(NODE_TO_PAYLOAD(block),
+                                                        TDB_SIZE);
 
     tdb->basic_data->tdb = NULL;
     block->is_used = 0;
@@ -236,34 +257,6 @@ static void nc_release_tls_node(nc_thread_memory_block_t *block) {
   }
 }
 
-/* Implements this, atomically:
- *  old = *ptr;
- *  if (*ptr == old_value)
- *    *ptr = new_value;
- *  return old;
- */
-static AtomicInt32 CompareAndSwap(volatile AtomicInt32* ptr,
-                                  AtomicInt32 old_value,
-                                  AtomicInt32 new_value) {
-  /* GCC intrinsic; see:
-   * http://gcc.gnu.org/onlinedocs/gcc/Atomic-Builtins.html.
-   * The x86-{32,64} compilers generate inline code.  The ARM
-   * implementation is external: instrincs/crt1_arm.S.
-   */
-  return __sync_val_compare_and_swap(ptr, old_value, new_value);
-}
-
-void nc_spinlock_lock(volatile int *lock) {
-  uint32_t val;
-  do {
-    val = CompareAndSwap(lock, 0, 1);
-  } while (val != 0);
-}
-
-void nc_spinlock_unlock(volatile int *lock) {
-  *lock = 0;
-}
-
 uint32_t __nacl_tdb_id_function(nc_hash_entry_t *entry) {
   nc_basic_thread_data_t *basic_data =
       HASH_ENTRY_TO_ENTRY_TYPE(entry, nc_basic_thread_data, hash_entry);
@@ -285,17 +278,11 @@ static int nc_tdb_init(nc_thread_descriptor_t *tdb,
   tdb->joinable = PTHREAD_CREATE_JOINABLE;
   tdb->join_waiting = 0;
 
-  tdb->thread_specific_data = NULL;
   tdb->tls_node = NULL;
   tdb->stack_node = NULL;
 
   tdb->start_func = NULL;
   tdb->state = NULL;
-  /*
-   * A call to pthread_exit without returning from the thread function
-   * should be recognized
-   */
-  tdb->exiting_without_returning = 1;
 
   /* Imitate PTHREAD_COND_INITIALIZER - we cannot use it directly here,
    * since this is not variable initialization.
@@ -304,18 +291,21 @@ static int nc_tdb_init(nc_thread_descriptor_t *tdb,
   return 0;
 }
 
-
 /* Will be called from the library startup code,
  * which always happens on the application's main thread
  */
-int __pthread_initialize() {
+int __pthread_initialize(void) {
   int retval = 0;
   nc_thread_descriptor_t *tdb;
   nc_basic_thread_data_t *basic_data;
   /* allocate TLS+TDB area */
   /* We allocate the basic data immediately after TDB */
-  __pthread_initialize_minimal(sizeof(nc_thread_descriptor_t) +
-                               sizeof(nc_basic_thread_data_t));
+  __pthread_initialize_minimal(TDB_SIZE);
+
+  /*
+   * Fetch the ABI tables from the IRT.  If we don't have these, all is lost.
+   */
+  __nc_initialize_interfaces(&irt_thread);
 
   /* At this point GS is already initialized */
   tdb = nc_get_tdb();
@@ -359,14 +349,13 @@ int __pthread_initialize() {
 }
 
 
-
 /* pthread functions */
 
 int pthread_create(pthread_t *thread_id,
-                   pthread_attr_t *attr,
+                   const pthread_attr_t *attr,
                    void *(*start_routine) (void *),
                    void *arg) {
-  int retval = -EAGAIN;
+  int retval = EAGAIN;
   void *esp;
   /* declare the variables outside of the while scope */
   nc_thread_memory_block_t *stack_node = NULL;
@@ -375,6 +364,7 @@ int pthread_create(pthread_t *thread_id,
   nc_basic_thread_data_t *new_basic_data = NULL;
   nc_thread_memory_block_t *tls_node = NULL;
   size_t stacksize = PTHREAD_STACK_DEFAULT;
+  void *new_tp;
 
   /* TODO(gregoryd) - right now a single lock is used, try to optimize? */
   pthread_mutex_lock(&__nc_thread_management_lock);
@@ -382,16 +372,16 @@ int pthread_create(pthread_t *thread_id,
   do {
     /* Allocate the combined TLS + TDB block---see tls.h for explanation. */
 
-    tls_node = nc_allocate_memory_block_mu(
-        TLS_AND_TDB_MEMORY,
-         __nacl_tls_combined_size(sizeof(nc_thread_descriptor_t)));
+    tls_node = nc_allocate_memory_block_mu(TLS_AND_TDB_MEMORY,
+                                           __nacl_tls_combined_size(TDB_SIZE));
     if (NULL == tls_node)
       break;
 
-    __nacl_tls_data_bss_initialize_from_template(NODE_TO_PAYLOAD(tls_node));
+    __nacl_tls_data_bss_initialize_from_template(NODE_TO_PAYLOAD(tls_node),
+                                                 TDB_SIZE);
 
     new_tdb = (nc_thread_descriptor_t *)
-              __nacl_tls_tdb_start(NODE_TO_PAYLOAD(tls_node));
+              __nacl_tls_tdb_start(NODE_TO_PAYLOAD(tls_node), TDB_SIZE);
     /* TODO(gregoryd): consider creating a pool of basic_data structs,
      * similar to stack and TLS+TDB (probably when adding the support for
      * variable stack size).
@@ -422,7 +412,7 @@ int pthread_create(pthread_t *thread_id,
     /* Allocate the stack for the thread */
     stack_node = nc_allocate_memory_block_mu(THREAD_STACK_MEMORY, stacksize);
     if (NULL == stack_node) {
-      retval = -EAGAIN;
+      retval = EAGAIN;
       break;
     }
     thread_stack = align((uint32_t) NODE_TO_PAYLOAD(stack_node),
@@ -450,22 +440,22 @@ int pthread_create(pthread_t *thread_id,
   pthread_mutex_unlock(&__nc_thread_management_lock);
 
   /*
-   * Calculate the top-of-stack location.  The very first location, is
-   * a zero address of architecture-dependent width, needed to satisfy
-   * the alignment requirement when we call nacl's thread_exit syscall
-   * when the thread terminates.
+   * Calculate the top-of-stack location.  The very first location is a
+   * zero address of architecture-dependent width, needed to satisfy the
+   * normal ABI alignment requirements for the stack.  (On some machines
+   * this is the dummy return address of the thread-start function.)
    *
    * Both thread_stack and stacksize are multiples of 16.
    */
-  size_t return_addr_size = __nacl_return_address_size();
-  esp = (void*) (thread_stack + stacksize - return_addr_size);
-  memset(esp, 0, return_addr_size);  /* NULL/0x00 pun is not strictly legal. */
+  size_t stack_padding = __nacl_thread_stack_padding();
+  esp = (void *) (thread_stack + stacksize - stack_padding);
+  memset(esp, 0, stack_padding);
+
+  new_tp = (char *) new_tdb - __nacl_tp_tdb_offset(TDB_SIZE);
 
   /* start the thread */
-  retval = NACL_SYSCALL(thread_create)(FUN_TO_VOID_PTR(nc_thread_starter),
-                                       esp,
-                                       new_tdb,
-                                       sizeof(nc_thread_descriptor_t));
+  retval = irt_thread.thread_create(
+      FUN_TO_VOID_PTR(nc_thread_starter), esp, new_tp);
   if (0 != retval) {
     pthread_mutex_lock(&__nc_thread_management_lock);
     /* TODO(gregoryd) : replace with atomic decrement? */
@@ -494,11 +484,10 @@ ret:
     *thread_id = -1;
   }
 
-  /* pthread_create returns errno, not syscall ABI convention */
-  return -retval;
+  return retval;
 }
 
-int __pthread_shutdown() {
+static int wait_for_threads(void) {
   pthread_mutex_lock(&__nc_thread_management_lock);
 
   while (1 != __nc_running_threads_counter) {
@@ -528,36 +517,9 @@ void pthread_exit (void* retval) {
   nc_basic_thread_data_t    *basic_data = tdb->basic_data;
   pthread_t                 thread_id = basic_data->thread_id;
   int                       joinable = tdb->joinable;
-  unsigned int              generation;
 
   /* call the destruction functions for TSD */
-  if (tdb->exiting_without_returning) {
-    int destruction_iterations;
-    for (destruction_iterations = 0;
-        destruction_iterations < PTHREAD_DESTRUCTOR_ITERATIONS;
-        destruction_iterations++) {
-      int i = 0;
-      int nothing_to_do = 1;
-      for (i = 0; i < PTHREAD_KEYS_MAX; i++) {
-        void (*destructor)(void*) =
-            __nc_thread_specific_storage[i].destruction_callback;
-        if ((KEY_ALLOCATED(generation =
-                __nc_thread_specific_storage[i].generation)) &&
-            (tdb->thread_specific_data != NULL) &&
-            (tdb->thread_specific_data[i].generation == generation) &&
-            (tdb->thread_specific_data[i].ptr != NULL) &&
-            (NULL != destructor)) {
-          void *tmp = tdb->thread_specific_data[i].ptr;
-          tdb->thread_specific_data[i].ptr = NULL;
-          destructor(tmp);
-          nothing_to_do = 0;
-        }
-      }
-      if (nothing_to_do) {
-        break;
-      }
-    }
-  }
+  __nc_tsd_exit();
 
   __newlib_thread_exit();
 
@@ -573,8 +535,8 @@ void pthread_exit (void* retval) {
 
   if (0 == thread_id) {
     /* This is the main thread - wait for other threads to complete */
-    __pthread_shutdown();
-    NACL_SYSCALL(exit)(0);
+    wait_for_threads();
+    exit(0);
   }
 
   pthread_mutex_lock(&__nc_thread_management_lock);
@@ -604,7 +566,8 @@ void pthread_exit (void* retval) {
   }
 
   pthread_mutex_unlock(&__nc_thread_management_lock);
-  NACL_SYSCALL(thread_exit)(is_used);
+  irt_thread.thread_exit(is_used);
+  while (1) *(volatile int *) 0 = 0;  /* Crash.  */
 }
 
 int pthread_join(pthread_t thread_id, void **thread_return) {
@@ -690,10 +653,6 @@ ret:
   return retval;
 }
 
-int nacl_thread_nice(const int nice) {
-  return NACL_SYSCALL(thread_nice)(nice);
-}
-
 int pthread_kill(pthread_t thread_id,
                  int sig) {
   /* This function is currently unimplemented. */
@@ -701,7 +660,7 @@ int pthread_kill(pthread_t thread_id,
   return -1;
 }
 
-pthread_t pthread_self() {
+pthread_t pthread_self(void) {
   /* get the tdb pointer from gs and use it to return the thread handle*/
   nc_thread_descriptor_t *tdb = nc_get_tdb();
   return tdb->basic_data->thread_id;
@@ -711,87 +670,14 @@ int pthread_equal (pthread_t thread1, pthread_t thread2) {
   return (thread1 == thread2);
 }
 
-/* Thread-specific data functions */
-
-int pthread_key_create (pthread_key_t *key,
-                        void (*destr_function) (void *)) {
-  int i;
-  unsigned int generation;
-  for (i=0; i<PTHREAD_KEYS_MAX; ++i) {
-    if (!KEY_ALLOCATED(generation =
-            __nc_thread_specific_storage[i].generation) &&
-        /* A key is not reusable after UINT_MAX-1 generations -
-         * the last even generation */
-        (__nc_thread_specific_storage[i].generation != UINT_MAX - 1) &&
-        (generation == CompareAndSwap(
-            (AtomicInt32*) &__nc_thread_specific_storage[i].generation,
-            generation,
-            generation + 1))) {
-      *key = i;
-      __nc_thread_specific_storage[i].destruction_callback = destr_function;
-      return 0;
-    }
-  }
-
-  return EAGAIN;
-}
-
-int pthread_key_delete (pthread_key_t key) {
-  unsigned int generation;
-  if ((key > PTHREAD_KEYS_MAX - 1) ||
-      !KEY_ALLOCATED(generation =
-          __nc_thread_specific_storage[key].generation)) {
-    return EINVAL;
-  }
-
-  if (generation !=
-      CompareAndSwap(
-          (AtomicInt32*) &__nc_thread_specific_storage[key].generation,
-          generation,
-          generation + 1)) {
-    /* Somebody incremented the generation counter before we did, i.e. the key
-     * has already been deleted.
+int pthread_setschedprio(pthread_t thread_id, int prio) {
+  if (thread_id != pthread_self()) {
+    /*
+     * We can only support changing our own priority.
      */
-    return EINVAL;
+    return EPERM;
   }
-
-  __nc_thread_specific_storage[key].destruction_callback = NULL;
-  return 0;
-}
-
-int pthread_setspecific (pthread_key_t key,
-                         const void *pointer) {
-  nc_thread_descriptor_t *tdb = nc_get_tdb();
-  unsigned int generation;
-
-  if ((key > PTHREAD_KEYS_MAX - 1) ||
-      !KEY_ALLOCATED(generation =
-          __nc_thread_specific_storage[key].generation)) {
-      return EINVAL;
-  }
-
-  /* Allocate the memory for this thread's TSD - we actually need it */
-  if (NULL == tdb->thread_specific_data) {
-    tdb->thread_specific_data = calloc(PTHREAD_KEYS_MAX, sizeof(tsd_t));
-    if (tdb->thread_specific_data == NULL) {
-      return EINVAL;
-    }
-  }
-
-  tdb->thread_specific_data[key].generation = generation;
-  tdb->thread_specific_data[key].ptr = (void*) pointer;
-  return 0;
-}
-
-void *pthread_getspecific (pthread_key_t key) {
-  nc_thread_descriptor_t *tdb = nc_get_tdb();
-  if (!tdb->thread_specific_data ||
-      (key > PTHREAD_KEYS_MAX - 1) ||
-      (tdb->thread_specific_data[key].generation !=
-          __nc_thread_specific_storage[key].generation)) {
-    return NULL;
-  }
-  return tdb->thread_specific_data[key].ptr;
+  return irt_thread.thread_nice(prio);
 }
 
 int pthread_attr_init (pthread_attr_t *attr) {
@@ -846,7 +732,8 @@ int pthread_attr_getstacksize(pthread_attr_t *attr,
   if (NULL == attr) {
     return EINVAL;
   }
-  return attr->stacksize;
+  *stacksize = attr->stacksize;
+  return 0;
 }
 
 void __local_lock_init(_LOCK_T* lock);
@@ -948,3 +835,12 @@ void __local_lock_release(_LOCK_T* lock) {
 void __local_lock_release_recursive(_LOCK_T* lock) {
   __local_lock_release(lock);
 }
+
+/*
+ * We include this directly in this file rather than compiling it
+ * separately because there is some code (e.g. libstdc++) that uses weak
+ * references to all pthread functions, but conditionalizes its calls only
+ * on one symbol.  So if these functions are in another file in a library
+ * archive, they might not be linked in by static linking.
+ */
+#include "native_client/src/untrusted/pthread/nc_tsd.c"
