@@ -1,10 +1,9 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
-
 /** @file
  * Defines the API in the
  * <a href="group___pthread.html">Pthread library</a>
@@ -53,7 +52,6 @@ enum {
   PTHREAD_MUTEX_DEFAULT = PTHREAD_MUTEX_NORMAL
 };
 
-
 /*
  * The layout of pthread_mutex_t and the static initializers are redefined
  * in newlib's sys/lock.h file (including this file from sys/lock.h will
@@ -65,8 +63,8 @@ enum {
  * opaque record; the names of the fields can change anytime.
  */
 typedef struct {
-  /** Initialization spinlock */
-  int spinlock;
+  /** Initialization token **/
+  int token;
 
   /**
    * The kind of mutex:
@@ -103,8 +101,11 @@ typedef struct {
  * opaque record; the names of the fields can change anytime.
  */
 typedef struct {
-  int spinlock; /**< Initialization spinlock */
-  int handle; /**< Handle to the system-side condition variable */
+  /** Initialization token **/
+  int token;
+
+  /**< Handle to the system-side condition variable */
+  int handle;
 } pthread_cond_t;
 
 /**
@@ -155,7 +156,7 @@ typedef struct {
  * @return 0 upon success, 1 otherwise
  */
 extern int pthread_mutex_init(pthread_mutex_t *mutex,
-                              pthread_mutexattr_t *mutex_attr);
+                              const pthread_mutexattr_t *mutex_attr);
 
 /** @nqPosix
 * Destroys a mutex.
@@ -276,7 +277,7 @@ extern int pthread_mutexattr_gettype(const pthread_mutexattr_t *attr,
 * @return 0 for success, 1 otherwise.
 */
 extern int pthread_cond_init(pthread_cond_t *cond,
-                             pthread_condattr_t *cond_attr);
+                             const pthread_condattr_t *cond_attr);
 
 /** @nqPosix
 * Destroys a condition variable.
@@ -341,7 +342,7 @@ extern int pthread_cond_wait(pthread_cond_t *cond,
 */
 int pthread_cond_timedwait_abs(pthread_cond_t *cond,
                                pthread_mutex_t *mutex,
-                               struct timespec *abstime);
+                               const struct timespec *abstime);
 
  /** @nqPosix
 * Waits for condition variable cond to be signaled or broadcast; wait time is
@@ -358,7 +359,7 @@ int pthread_cond_timedwait_abs(pthread_cond_t *cond,
 */
 int pthread_cond_timedwait_rel(pthread_cond_t *cond,
                                pthread_mutex_t *mutex,
-                               struct timespec *reltime);
+                               const struct timespec *reltime);
 
 /**
  * Defined for POSIX compatibility; pthread_cond_timedwait() is actually
@@ -407,7 +408,7 @@ typedef struct {
 * @return 0 for success, non-zero error code otherwise.
 */
 extern int pthread_create(pthread_t *thread_id,
-                          pthread_attr_t *attr,
+                          const pthread_attr_t *attr,
                           void *(*start_routine) (void *p),
                           void *arg);
 
@@ -470,22 +471,6 @@ extern int pthread_join(pthread_t th, void **thread_return);
 */
 extern int pthread_detach(pthread_t th);
 
-/** @nqPosix
-* Indicates that the calling thread is to receive special consideration
-* by the scheduler as indicated by parameter nice. Suggested values are
-* NICE_REALTIME, NICE_NORMAL, NICE_BACKGROUND, defined above. The
-* implementation of this subroutine is platform-specific. Implementations
-* should respect the sign of the nice parameter, and may respect the
-* magnitude.
-*
-* @linkPthread
-*
-* @param nice Nice value
-*
-* @return 0 on success, non-zero error code otherwise.
-*/
-extern int nacl_thread_nice(const int nice);
-
 /** @nqPosix
 * Sends a signal to a thread.  (Currently only a stub implementation.)
 *
@@ -670,6 +655,17 @@ typedef struct {
 extern int pthread_once(pthread_once_t *__once_control,
                         void (*__init_routine)(void));
 
+/** @nqPosix
+* Sets the scheduling priority of a thread.
+*
+* @linkPthread
+*
+* @param thread_id Identifies the thread to operate on.
+* @param prio Scheduling priority to apply to that thread.
+*
+* @return 0 on success, non-zero error code otherwise.
+*/
+extern int pthread_setschedprio(pthread_t thread_id, int prio);
 
 /*
  * NOTE: this is only declared here to shut up
