@@ -1,7 +1,7 @@
 /*
- * Copyright 2010 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /* This file contains valgrind interceptors for NaCl's untrusted library
@@ -26,11 +26,11 @@
 
 #include <stdio.h>
 #include <assert.h>
-#include <pthread.h>
 #include <errno.h>
 #include <sched.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "native_client/src/third_party/valgrind/nacl_valgrind.h"
 #include "native_client/src/third_party/valgrind/nacl_memcheck.h"
@@ -42,8 +42,20 @@
 
 #include "native_client/src/third_party/valgrind/ts_valgrind_client_requests.h"
 
+#ifdef __GLIBC__
+#include <pthread.h>
+#else
+/*
+ * Get pthread.h from the source tree rather than using the installed one
+ * in the newlib build.  It might not be installed yet when we're building.
+ * Since we're about to include its private header here anyway, we might
+ * as well consistently refer to the source rather than what's installed.
+ */
+#include "native_client/src/untrusted/pthread/pthread.h"
+
 /* For sizeof(nc_thread_memory_block_t) */
 #include "native_client/src/untrusted/pthread/pthread_types.h"
+#endif
 
 /* This variable needs to be referenced by a program (either in sources,
   or using the linker flag -u) to which this library is linked.
@@ -53,10 +65,25 @@ int have_nacl_valgrind_interceptors;
 
 /* TSan interceptors. */
 
-#define VG_NACL_FUNC(f) I_WRAP_SONAME_FNNAME_ZZ(NaCl, f)
-#define VG_NACL_ANN(f) \
-  I_WRAP_SONAME_FNNAME_ZZ(NaCl, DYNAMIC_ANNOTATIONS_NAME(f))
+#ifdef __GLIBC__
+#define VG_NACL_Z_LIBC_SONAME  NaClZulibcZdsoZdZa
+#define VG_NACL_Z_LIBPTHREAD_SONAME NaClZulibpthreadZdsoZdZa
+#define VG_NACL_Z_NONE_SONAME NaClZuNONE
+#define VG_NACL_Z_ANY_SONAME NaClZuZa
+#else
+#define VG_NACL_Z_LIBC_SONAME NaClZuNONE
+#define VG_NACL_Z_LIBPTHREAD_SONAME NaClZuNONE
+#define VG_NACL_Z_NONE_SONAME NaClZuNONE
+#define VG_NACL_Z_ANY_SONAME NaClZuNONE
+#endif
 
+#define VG_NACL_NONE_FUNC(f) I_WRAP_SONAME_FNNAME_ZZ(VG_NACL_Z_NONE_SONAME, f)
+#define VG_NACL_LIBC_FUNC(f) I_WRAP_SONAME_FNNAME_ZZ(VG_NACL_Z_LIBC_SONAME, f)
+#define VG_NACL_LIBPTHREAD_FUNC(f) \
+  I_WRAP_SONAME_FNNAME_ZZ(VG_NACL_Z_LIBPTHREAD_SONAME, f)
+#define VG_NACL_ANY_FUNC(f) I_WRAP_SONAME_FNNAME_ZZ(VG_NACL_Z_ANY_SONAME, f)
+#define VG_NACL_ANN(f) \
+  I_WRAP_SONAME_FNNAME_ZZ(VG_NACL_Z_ANY_SONAME, DYNAMIC_ANNOTATIONS_NAME(f))
 
 #define VG_CREQ_v_v(_req)                                               \
   do {                                                                  \
@@ -186,9 +213,9 @@ INLINE static size_t handle_free_before(size_t ptr) {
   uint64_t base;
   size_t size, old_ptr;
   size_t orig_ptr = ptr;
+  start_ignore_all_accesses_and_sync();
   if (!ptr)
     return 0;
-  start_ignore_all_accesses_and_sync();
   /* Get the size of allocated region, check sanity. */
   ptr -= kRedZoneSize;
   base = VALGRIND_SANDBOX_PTR(ptr);
@@ -227,37 +254,46 @@ INLINE static void handle_free_after() {
   stop_ignore_all_accesses_and_sync();
 }
 
-/* _malloc_r() */
-size_t VG_NACL_FUNC(_malloc_r)(size_t reent, size_t size) {
+static __thread int inside_malloc = 0;
+
+/* malloc() */
+size_t VG_NACL_LIBC_FUNC(malloc)(size_t size) {
   OrigFn fn;
   size_t ptr;
+  int nested;
   VALGRIND_GET_ORIG_FN(fn);
-  size_t allocSize = handle_malloc_before(size);
-  CALL_FN_W_WW(ptr, fn, reent, allocSize);
-  return handle_malloc_after(ptr, size);
+  nested = inside_malloc++;
+  if (nested) {
+    CALL_FN_W_W(ptr, fn, size);
+  } else {
+    size_t allocSize = handle_malloc_before(size);
+    CALL_FN_W_W(ptr, fn, allocSize);
+    ptr = handle_malloc_after(ptr, size);
+  }
+  inside_malloc--;
+  return ptr;
 }
 
-/* _calloc_r() */
-size_t VG_NACL_FUNC(_calloc_r)(size_t reent, size_t nmemb, size_t size) {
+/* calloc() */
+size_t VG_NACL_LIBC_FUNC(calloc)(size_t nmemb, size_t size) {
   size_t totalSize = nmemb * size;
-  void* ptr = _malloc_r((void*)reent, totalSize);
+  void* ptr = malloc(totalSize);
   if (ptr)
     memset(ptr, 0, totalSize);
   return (size_t)ptr;
 }
 
-/* _realloc_r() */
-size_t VG_NACL_FUNC(_realloc_r)(void* reent, size_t origPtr, size_t size) {
+/* realloc() */
+size_t VG_NACL_LIBC_FUNC(realloc)(size_t origPtr, size_t size) {
   if (!origPtr) {
-    return (size_t)_malloc_r(reent, size);
+    return (size_t)malloc(size);
   }
   if (!size) {
     free((void*)origPtr);
     return 0;
   }
-  size_t newPtr = (size_t)_malloc_r(reent, size);
+  size_t newPtr = (size_t)malloc(size);
   if (!newPtr) {
-    free((void*)origPtr);
     return 0;
   }
 
@@ -267,24 +303,24 @@ size_t VG_NACL_FUNC(_realloc_r)(void* reent, size_t origPtr, size_t size) {
   size_t copySize = size < origSize ? size : origSize;
 
   memcpy((void*)newPtr, (void*)origPtr, copySize);
-  _free_r(reent, (void*)origPtr);
+  free((void*)origPtr);
   return newPtr;
 }
 
-/* _free_r() */
-void VG_NACL_FUNC(_free_r)(size_t reent, size_t ptr) {
+/* free() */
+void VG_NACL_LIBC_FUNC(free)(size_t ptr) {
   OrigFn fn;
   VALGRIND_GET_ORIG_FN(fn);
   size_t old_ptr = handle_free_before(ptr);
   if (old_ptr)
-    CALL_FN_v_WW(fn, reent, old_ptr);
+    CALL_FN_v_W(fn, old_ptr);
   handle_free_after();
 }
 
 /* Unoptimized string functions. Optimized versions often read several bytes
    beyond the end of the string. This makes Memcheck sad. */
 /* strlen() */
-size_t VG_NACL_FUNC(strlen)(char* ptr) {
+size_t VG_NACL_ANY_FUNC(strlen)(char* ptr) {
   size_t i = 0;
   while (ptr[i])
     ++i;
@@ -292,7 +328,7 @@ size_t VG_NACL_FUNC(strlen)(char* ptr) {
 }
 
 /* strchr() */
-char* VG_NACL_FUNC(strchr)(char* s, int c) {
+char* VG_NACL_ANY_FUNC(strchr)(char* s, int c) {
   size_t i;
   char *ret = 0;
   for (i = 0; ; i++) {
@@ -305,8 +341,109 @@ char* VG_NACL_FUNC(strchr)(char* s, int c) {
   return ret;
 }
 
+/* strcmp() */
+int VG_NACL_ANY_FUNC(strcmp)(char* s1, char* s2) {
+  while (*s1 && *s1 == *s2)
+    ++s1, ++s2;
+  if (*s1 < *s2)
+    return -1;
+  if (*s1 > *s2)
+    return 1;
+  return 0;
+}
+
+#ifdef __GLIBC__
+
+static void* tsan_start_thread(void* arg) {
+  size_t* args;
+  void* (*start_routine)(void*);
+  size_t real_arg;
+
+  start_ignore_all_accesses();
+  VG_CREQ_v_W(TSREQ_SET_MY_PTHREAD_T, pthread_self());
+
+  args = (size_t*)arg;
+  start_routine = (void* (*)(void*))args[0];
+  real_arg = args[1];
+
+  /* Let the tool guess where the stack starts. */
+  VG_CREQ_v_W(TSREQ_THR_STACK_TOP,
+      VALGRIND_SANDBOX_PTR((size_t)&args));
+
+  __sync_synchronize();
+  args[2] = 0; /* The parent may continue. */
+  stop_ignore_all_accesses();
+
+  return start_routine((void*)real_arg);
+}
+
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZucreateZAZa)(size_t thread, size_t attr,
+    size_t start_routine, size_t arg) {
+  OrigFn fn;
+  int ret;
+  volatile size_t args[3];
+  VALGRIND_GET_ORIG_FN(fn);
+
+  args[0] = start_routine;
+  args[1] = arg;
+  args[2] = 1;
+
+  CALL_FN_W_WWWW(ret, fn, thread, attr, (size_t)tsan_start_thread,
+      (size_t)&args);
+
+  start_ignore_all_accesses();
+  if (!ret)
+    while (args[2])
+      sched_yield();
+
+  __sync_synchronize();
+  stop_ignore_all_accesses();
+
+  return ret;
+}
+
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZujoin)(size_t thread, size_t value_ptr) {
+  OrigFn fn;
+  int ret;
+  VALGRIND_GET_ORIG_FN(fn);
+  CALL_FN_W_WW(ret, fn, thread, value_ptr);
+
+  /* *(int*)0=0; */
+
+  if (!ret) /* success */
+    VG_CREQ_v_W(TSREQ_PTHREAD_JOIN_POST, thread);
+
+  return ret;
+}
+
+void VG_NACL_LIBPTHREAD_FUNC(ZuZupthreadZuinitializeZuminimal)(void) {
+  OrigFn fn;
+  int local_var;
+  VALGRIND_GET_ORIG_FN(fn);
+  CALL_FN_v_v(fn);
+
+  VG_CREQ_v_W(TSREQ_SET_MY_PTHREAD_T, pthread_self());
+  /* Let the tool guess where the stack starts. */
+  VG_CREQ_v_W(TSREQ_THR_STACK_TOP,
+      VALGRIND_SANDBOX_PTR((size_t)&local_var));
+}
+
+#else
+
+void VG_NACL_NONE_FUNC(ncZuthreadZustarter)(size_t func, size_t state) {
+  OrigFn fn;
+  int local_stack_var = 0;
+  VALGRIND_GET_ORIG_FN(fn);
+
+  /* Let the tool guess where the stack starts. */
+  VG_CREQ_v_W(TSREQ_THR_STACK_TOP, (size_t)&local_stack_var);
+
+  CALL_FN_v_WW(fn, func, state);
+}
+
 /* nc_allocate_memory_block_mu() - a cached malloc for thread stack & tls. */
-size_t VG_NACL_FUNC(nc_allocate_memory_block_mu)(int type, size_t size) {
+size_t VG_NACL_NONE_FUNC(ncZuallocateZumemoryZublockZumu)(int type,
+    size_t size) {
   OrigFn fn;
   size_t ret;
   VALGRIND_GET_ORIG_FN(fn);
@@ -319,27 +456,14 @@ size_t VG_NACL_FUNC(nc_allocate_memory_block_mu)(int type, size_t size) {
   }
   return ret;
 }
-
-/* Tell the tool that the stack has moved.
-   This is the first untrusted function of any NaCl thread and the first
-   place after the context switch that we can intercept. */
-void VG_NACL_FUNC(nc_thread_starter)(size_t func, size_t state) {
-  OrigFn fn;
-  int local_stack_var = 0;
-
-  /* Let the tool guess where the stack starts. */
-  VG_CREQ_v_W(TSREQ_THR_STACK_TOP, (size_t)&local_stack_var);
-
-  VALGRIND_GET_ORIG_FN(fn);
-  CALL_FN_v_WW(fn, func, state);
-}
+#endif
 
 
 /* ------------------------------------------------------------------*/
 /*                        PThread mutex.                             */
 
 /* pthread_mutex_init */
-int VG_NACL_FUNC(pthreadZumutexZuinit)(pthread_mutex_t *mutex,
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZumutexZuinit)(pthread_mutex_t *mutex,
     pthread_mutexattr_t* attr) {
   int    ret;
   OrigFn fn;
@@ -356,7 +480,7 @@ int VG_NACL_FUNC(pthreadZumutexZuinit)(pthread_mutex_t *mutex,
 }
 
 /* pthread_mutex_destroy */
-int VG_NACL_FUNC(pthreadZumutexZudestroy)(pthread_mutex_t *mutex) {
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZumutexZudestroy)(pthread_mutex_t *mutex) {
   int    ret;
   OrigFn fn;
   VALGRIND_GET_ORIG_FN(fn);
@@ -370,7 +494,7 @@ int VG_NACL_FUNC(pthreadZumutexZudestroy)(pthread_mutex_t *mutex) {
 }
 
 /* pthread_mutex_lock */
-int VG_NACL_FUNC(pthreadZumutexZulock)(pthread_mutex_t *mutex) {
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZumutexZulock)(pthread_mutex_t *mutex) {
   int    ret;
   OrigFn fn;
   VALGRIND_GET_ORIG_FN(fn);
@@ -388,7 +512,7 @@ int VG_NACL_FUNC(pthreadZumutexZulock)(pthread_mutex_t *mutex) {
 }
 
 /* pthread_mutex_trylock. */
-int VG_NACL_FUNC(pthreadZumutexZutrylock)(pthread_mutex_t *mutex) {
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZumutexZutrylock)(pthread_mutex_t *mutex) {
   int    ret;
   OrigFn fn;
   VALGRIND_GET_ORIG_FN(fn);
@@ -406,7 +530,7 @@ int VG_NACL_FUNC(pthreadZumutexZutrylock)(pthread_mutex_t *mutex) {
 }
 
 /* pthread_mutex_timedlock. */
-int VG_NACL_FUNC(pthreadZumutexZutimedlock)(pthread_mutex_t *mutex,
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZumutexZutimedlock)(pthread_mutex_t *mutex,
     void* timeout) {
   int    ret;
   OrigFn fn;
@@ -425,7 +549,7 @@ int VG_NACL_FUNC(pthreadZumutexZutimedlock)(pthread_mutex_t *mutex,
 }
 
 /* pthread_mutex_unlock */
-int VG_NACL_FUNC(pthreadZumutexZuunlock)(pthread_mutex_t *mutex) {
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZumutexZuunlock)(pthread_mutex_t *mutex) {
   int    ret;
   OrigFn fn;
   VALGRIND_GET_ORIG_FN(fn);
@@ -443,8 +567,7 @@ int VG_NACL_FUNC(pthreadZumutexZuunlock)(pthread_mutex_t *mutex) {
 /* ------------------------------------------------------------------*/
 /*                     Conditional variables.                        */
 
-/* pthread_cond_wait */
-int VG_NACL_FUNC(pthreadZucondZuwait)(size_t cond, size_t mutex) {
+static int handle_cond_wait(size_t cond, size_t mutex) {
   int ret;
   OrigFn fn;
   VALGRIND_GET_ORIG_FN(fn);
@@ -461,8 +584,7 @@ int VG_NACL_FUNC(pthreadZucondZuwait)(size_t cond, size_t mutex) {
   return ret;
 }
 
-/* pthread_cond_signal */
-int VG_NACL_FUNC(pthreadZucondZusignal)(size_t cond) {
+static int handle_cond_signal(size_t cond) {
   int ret;
   OrigFn fn;
   VALGRIND_GET_ORIG_FN(fn);
@@ -476,6 +598,33 @@ int VG_NACL_FUNC(pthreadZucondZusignal)(size_t cond) {
   return ret;
 }
 
+#ifdef __GLIBC__
+
+/* pthread_cond_wait@* */
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZucondZuwaitZAZa)(size_t cond,
+    size_t mutex) {
+  return handle_cond_wait(cond, mutex);
+}
+
+/* pthread_cond_signal@* */
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZucondZusignalZAZa)(size_t cond) {
+  return handle_cond_signal(cond);
+}
+
+#else
+
+/* pthread_cond_wait */
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZucondZuwait)(size_t cond, size_t mutex) {
+  return handle_cond_wait(cond, mutex);
+}
+
+/* pthread_cond_signal */
+int VG_NACL_LIBPTHREAD_FUNC(pthreadZucondZusignal)(size_t cond) {
+  return handle_cond_signal(cond);
+}
+
+#endif
+
 /* ------------------------------------------------------------------*/
 /*                       POSIX semaphores.                           */
 
@@ -494,18 +643,7 @@ static int handle_sem_wait(void* sem) {
   return ret;
 }
 
-/* sem_wait */
-int VG_NACL_FUNC(semZuwait)(void* sem) {
-  return handle_sem_wait(sem);
-}
-
-/* sem_trywait */
-int VG_NACL_FUNC(semZutrywait)(void* sem) {
-  return handle_sem_wait(sem);
-}
-
-/* sem_timedwait */
-int VG_NACL_FUNC(semZutimedwait)(void* sem, void* abs_timeout) {
+static int handle_sem_timedwait(void* sem, void* abs_timeout) {
   OrigFn fn;
   int    ret;
   VALGRIND_GET_ORIG_FN(fn);
@@ -520,8 +658,7 @@ int VG_NACL_FUNC(semZutimedwait)(void* sem, void* abs_timeout) {
   return ret;
 }
 
-/* sem_post */
-int VG_NACL_FUNC(semZupost)(void* sem) {
+static int handle_sem_post(void* sem) {
   OrigFn fn;
   int    ret;
   VALGRIND_GET_ORIG_FN(fn);
@@ -535,6 +672,51 @@ int VG_NACL_FUNC(semZupost)(void* sem) {
   return ret;
 }
 
+#ifdef __GLIBC__
+
+/* sem_wait@* */
+int VG_NACL_LIBPTHREAD_FUNC(semZuwaitZAZa)(void* sem) {
+  return handle_sem_wait(sem);
+}
+
+/* sem_trywait@* */
+int VG_NACL_LIBPTHREAD_FUNC(semZutrywaitZAZa)(void* sem) {
+  return handle_sem_wait(sem);
+}
+
+/* sem_timedwait@* */
+int VG_NACL_LIBPTHREAD_FUNC(semZutimedwaitZAZa)(void* sem, void* abs_timeout) {
+  return handle_sem_timedwait(sem, abs_timeout);
+}
+
+/* sem_post@* */
+int VG_NACL_LIBPTHREAD_FUNC(semZupostZAZa)(void* sem) {
+  return handle_sem_post(sem);
+}
+
+#else
+
+/* sem_wait */
+int VG_NACL_LIBPTHREAD_FUNC(semZuwait)(void* sem) {
+  return handle_sem_wait(sem);
+}
+
+/* sem_trywait */
+int VG_NACL_LIBPTHREAD_FUNC(semZutrywait)(void* sem) {
+  return handle_sem_wait(sem);
+}
+
+/* sem_timedwait */
+int VG_NACL_LIBPTHREAD_FUNC(semZutimedwait)(void* sem, void* abs_timeout) {
+  return handle_sem_timedwait(sem, abs_timeout);
+}
+
+/* sem_post */
+int VG_NACL_LIBPTHREAD_FUNC(semZupost)(void* sem) {
+  return handle_sem_post(sem);
+}
+#endif
+
 /* ------------------------------------------------------------------*/
 /*             Limited support for dynamic annotations.              */
 
@@ -548,7 +730,7 @@ void VG_NACL_ANN(AnnotateMutexIsNotPHB)(char *file, int line, void *mu) {
 
 void VG_NACL_ANN(AnnotateExpectRace)(char *file, int line, void *addr,
     void* desc) {
-  VG_CREQ_v_WWW(TSREQ_EXPECT_RACE, VALGRIND_SANDBOX_PTR((size_t)addr), 1,
+  VG_CREQ_v_WW(TSREQ_EXPECT_RACE, VALGRIND_SANDBOX_PTR((size_t)addr),
       VALGRIND_SANDBOX_PTR((size_t)desc));
 }
 
