@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -18,7 +18,7 @@ class AddressListNetLogParam : public NetLog::EventParameters {
  public:
   explicit AddressListNetLogParam(const AddressList& address_list);
 
-  virtual Value* ToValue() const;
+  virtual base::Value* ToValue() const OVERRIDE;
 
  private:
   AddressList address_list_;
