@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,13 +10,15 @@
 #include <vector>
 
 #include "base/basictypes.h"
+#include "base/memory/ref_counted.h"
 #include "base/string16.h"
-#include "base/ref_counted.h"
 #include "net/base/cert_type.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
 class CryptoModule;
+typedef std::vector<scoped_refptr<CryptoModule> > CryptoModuleList;
 class X509Certificate;
 typedef std::vector<scoped_refptr<X509Certificate> > CertificateList;
 
@@ -28,10 +30,40 @@ typedef std::vector<scoped_refptr<X509Certificate> > CertificateList;
 // for all operations that manipulate the underlying system
 // certificate store.
 
-class CertDatabase {
+class NET_EXPORT CertDatabase {
  public:
+
+  // A CertDatabase::Observer will be notified on certificate database changes.
+  // The change could be either a new user certificate is added or trust on
+  // a certificate is changed.  Observers can register themselves
+  // via CertDatabase::AddObserver, and can un-register with
+  // CertDatabase::RemoveObserver.
+  class NET_EXPORT Observer {
+   public:
+    virtual ~Observer() {}
+
+    // Will be called when a new user certificate is added.
+    // Called with |cert| == NULL after importing a list of certificates
+    // in ImportFromPKCS12().
+    virtual void OnUserCertAdded(const X509Certificate* cert) {}
+
+    // Will be called when a user certificate is removed.
+    virtual void OnUserCertRemoved(const X509Certificate* cert) {}
+
+    // Will be called when a certificate's trust is changed.
+    // Called with |cert| == NULL after importing a list of certificates
+    // in ImportCACerts().
+    virtual void OnCertTrustChanged(const X509Certificate* cert) {}
+
+   protected:
+    Observer() {}
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(Observer);
+  };
+
   // Stores per-certificate error codes for import failures.
-  struct ImportCertFailure {
+  struct NET_EXPORT ImportCertFailure {
    public:
     ImportCertFailure(X509Certificate* cert, int err);
     ~ImportCertFailure();
@@ -50,6 +82,9 @@ class CertDatabase {
   // trusted as a server.
   // For EMAIL_CERT, only TRUSTED_EMAIL makes sense, and specifies the cert is
   // trusted for email.
+  // NOTE: The actual constants are defined using an enum instead of static
+  // consts due to compilation/linkage constraints with template functions.
+  typedef uint32 TrustBits;
   enum {
     UNTRUSTED        =      0,
     TRUSTED_SSL      = 1 << 0,
@@ -69,20 +104,33 @@ class CertDatabase {
   int AddUserCert(X509Certificate* cert);
 
 #if defined(USE_NSS) || defined(USE_OPENSSL)
-  // Get a list of unique certificates in the certificate database.  (One
-  // instance of all certificates.)
+  // Get a list of unique certificates in the certificate database (one
+  // instance of all certificates).
   void ListCerts(CertificateList* certs);
 
-  // Get the default module.
+  // Get the default module for public key data.
   // The returned pointer must be stored in a scoped_refptr<CryptoModule>.
-  CryptoModule* GetDefaultModule() const;
+  CryptoModule* GetPublicModule() const;
+
+  // Get the default module for private key or mixed private/public key data.
+  // The returned pointer must be stored in a scoped_refptr<CryptoModule>.
+  CryptoModule* GetPrivateModule() const;
+
+  // Get all modules.
+  // If |need_rw| is true, only writable modules will be returned.
+  void ListModules(CryptoModuleList* modules, bool need_rw) const;
 
   // Import certificates and private keys from PKCS #12 blob into the module.
+  // If |is_extractable| is false, mark the private key as being unextractable
+  // from the module.
   // Returns OK or a network error code such as ERR_PKCS12_IMPORT_BAD_PASSWORD
-  // or ERR_PKCS12_IMPORT_ERROR.
-  int ImportFromPKCS12(net::CryptoModule* module,
+  // or ERR_PKCS12_IMPORT_ERROR. |imported_certs|, if non-NULL, returns a list
+  // of certs that were imported.
+  int ImportFromPKCS12(CryptoModule* module,
                        const std::string& data,
-                       const string16& password);
+                       const string16& password,
+                       bool is_extractable,
+                       CertificateList* imported_certs);
 
   // Export the given certificates and private keys into a PKCS #12 blob,
   // storing into |output|.
@@ -104,7 +152,7 @@ class CertDatabase {
   // |not_imported| should be checked for any certificates that were not
   // imported.
   bool ImportCACerts(const CertificateList& certificates,
-                     unsigned int trust_bits,
+                     TrustBits trust_bits,
                      ImportCertFailureList* not_imported);
 
   // Import server certificate.  The first cert should be the server cert.  Any
@@ -119,24 +167,43 @@ class CertDatabase {
                         ImportCertFailureList* not_imported);
 
   // Get trust bits for certificate.
-  unsigned int GetCertTrust(const X509Certificate* cert, CertType type) const;
+  TrustBits GetCertTrust(const X509Certificate* cert, CertType type) const;
+
+  // IsUntrusted returns true if |cert| is specifically untrusted. These
+  // certificates are stored in the database for the specific purpose of
+  // rejecting them.
+  bool IsUntrusted(const X509Certificate* cert) const;
 
   // Set trust values for certificate.
   // Returns true on success or false on failure.
   bool SetCertTrust(const X509Certificate* cert,
                     CertType type,
-                    unsigned int trust_bits);
+                    TrustBits trust_bits);
 
   // Delete certificate and associated private key (if one exists).
-  // Returns true on success or false on failure.
-  // |cert| is still valid when this function returns.
+  // |cert| is still valid when this function returns. Returns true on
+  // success.
   bool DeleteCertAndKey(const X509Certificate* cert);
 
   // Check whether cert is stored in a readonly slot.
   bool IsReadOnly(const X509Certificate* cert) const;
 #endif
 
+  // Registers |observer| to receive notifications of certificate changes.  The
+  // thread on which this is called is the thread on which |observer| will be
+  // called back with notifications.
+  static void AddObserver(Observer* observer);
+
+  // Unregisters |observer| from receiving notifications.  This must be called
+  // on the same thread on which AddObserver() was called.
+  static void RemoveObserver(Observer* observer);
+
  private:
+  // Broadcasts notifications to all registered observers.
+  static void NotifyObserversOfUserCertAdded(const X509Certificate* cert);
+  static void NotifyObserversOfUserCertRemoved(const X509Certificate* cert);
+  static void NotifyObserversOfCertTrustChanged(const X509Certificate* cert);
+
   DISALLOW_COPY_AND_ASSIGN(CertDatabase);
 };
 
