@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,9 +11,9 @@
 #include <secmod.h>
 
 #include "base/logging.h"
-#include "base/nss_util.h"
-#include "base/nss_util_internal.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "crypto/nss_util.h"
+#include "crypto/nss_util_internal.h"
 #include "net/base/crypto_module.h"
 #include "net/base/net_errors.h"
 #include "net/base/x509_certificate.h"
@@ -21,13 +21,19 @@
 #include "net/third_party/mozilla_security_manager/nsNSSCertTrust.h"
 #include "net/third_party/mozilla_security_manager/nsPKCS12Blob.h"
 
+// In NSS 3.13, CERTDB_VALID_PEER was renamed CERTDB_TERMINAL_RECORD. So we use
+// the new name of the macro.
+#if !defined(CERTDB_TERMINAL_RECORD)
+#define CERTDB_TERMINAL_RECORD CERTDB_VALID_PEER
+#endif
+
 // PSM = Mozilla's Personal Security Manager.
 namespace psm = mozilla_security_manager;
 
 namespace net {
 
 CertDatabase::CertDatabase() {
-  base::EnsureNSSInit();
+  crypto::EnsureNSSInit();
   psm::EnsurePKCS12Init();
 }
 
@@ -58,30 +64,13 @@ int CertDatabase::CheckUserCert(X509Certificate* cert_obj) {
 int CertDatabase::AddUserCert(X509Certificate* cert_obj) {
   CERTCertificate* cert = cert_obj->os_cert_handle();
   PK11SlotInfo* slot = NULL;
-  std::string nickname;
-
-  // Create a nickname for this certificate.
-  // We use the scheme used by Firefox:
-  // --> <subject's common name>'s <issuer's common name> ID.
-
-  std::string username, ca_name;
-  char* temp_username = CERT_GetCommonName(&cert->subject);
-  char* temp_ca_name = CERT_GetCommonName(&cert->issuer);
-  if (temp_username) {
-    username = temp_username;
-    PORT_Free(temp_username);
-  }
-  if (temp_ca_name) {
-    ca_name = temp_ca_name;
-    PORT_Free(temp_ca_name);
-  }
-  nickname = username + "'s " + ca_name + " ID";
 
   {
-    base::AutoNSSWriteLock lock;
-    slot = PK11_ImportCertForKey(cert,
-                                 const_cast<char*>(nickname.c_str()),
-                                 NULL);
+    crypto::AutoNSSWriteLock lock;
+    slot = PK11_ImportCertForKey(
+        cert,
+        cert_obj->GetDefaultNickname(net::USER_CERT).c_str(),
+        NULL);
   }
 
   if (!slot) {
@@ -89,6 +78,7 @@ int CertDatabase::AddUserCert(X509Certificate* cert_obj) {
     return ERR_ADD_USER_CERT_FAILED;
   }
   PK11_FreeSlot(slot);
+  CertDatabase::NotifyObserversOfUserCertAdded(cert_obj);
   return OK;
 }
 
@@ -101,30 +91,70 @@ void CertDatabase::ListCerts(CertificateList* certs) {
        !CERT_LIST_END(node, cert_list);
        node = CERT_LIST_NEXT(node)) {
     certs->push_back(X509Certificate::CreateFromHandle(
-        node->cert,
-        X509Certificate::SOURCE_LONE_CERT_IMPORT,
-        X509Certificate::OSCertHandles()));
+        node->cert, X509Certificate::OSCertHandles()));
   }
   CERT_DestroyCertList(cert_list);
 }
 
-CryptoModule* CertDatabase::GetDefaultModule() const {
+CryptoModule* CertDatabase::GetPublicModule() const {
   CryptoModule* module =
-      CryptoModule::CreateFromHandle(base::GetDefaultNSSKeySlot());
-  // The module is already referenced when returned from GetDefaultNSSKeymodule,
-  // so we need to deref it once.
+      CryptoModule::CreateFromHandle(crypto::GetPublicNSSKeySlot());
+  // The module is already referenced when returned from
+  // GetPublicNSSKeySlot, so we need to deref it once.
   PK11_FreeSlot(module->os_module_handle());
 
   return module;
 }
 
+CryptoModule* CertDatabase::GetPrivateModule() const {
+  CryptoModule* module =
+      CryptoModule::CreateFromHandle(crypto::GetPrivateNSSKeySlot());
+  // The module is already referenced when returned from
+  // GetPrivateNSSKeySlot, so we need to deref it once.
+  PK11_FreeSlot(module->os_module_handle());
+
+  return module;
+}
+
+void CertDatabase::ListModules(CryptoModuleList* modules, bool need_rw) const {
+  modules->clear();
+
+  PK11SlotList* slot_list = NULL;
+  // The wincx arg is unused since we don't call PK11_SetIsLoggedInFunc.
+  slot_list = PK11_GetAllTokens(CKM_INVALID_MECHANISM,
+                                need_rw ? PR_TRUE : PR_FALSE,  // needRW
+                                PR_TRUE,  // loadCerts (unused)
+                                NULL);  // wincx
+  if (!slot_list) {
+    LOG(ERROR) << "PK11_GetAllTokens failed: " << PORT_GetError();
+    return;
+  }
+
+  PK11SlotListElement* slot_element = PK11_GetFirstSafe(slot_list);
+  while (slot_element) {
+    modules->push_back(CryptoModule::CreateFromHandle(slot_element->slot));
+    slot_element = PK11_GetNextSafe(slot_list, slot_element,
+                                    PR_FALSE);  // restart
+  }
+
+  PK11_FreeSlotList(slot_list);
+}
+
 int CertDatabase::ImportFromPKCS12(
-    net::CryptoModule* module,
+    CryptoModule* module,
     const std::string& data,
-    const string16& password) {
-  return psm::nsPKCS12Blob_Import(module->os_module_handle(),
-                                  data.data(), data.size(),
-                                  password);
+    const string16& password,
+    bool is_extractable,
+    net::CertificateList* imported_certs) {
+  int result = psm::nsPKCS12Blob_Import(module->os_module_handle(),
+                                        data.data(), data.size(),
+                                        password,
+                                        is_extractable,
+                                        imported_certs);
+  if (result == net::OK)
+    CertDatabase::NotifyObserversOfUserCertAdded(NULL);
+
+  return result;
 }
 
 int CertDatabase::ExportToPKCS12(
@@ -158,10 +188,15 @@ X509Certificate* CertDatabase::FindRootInList(
 }
 
 bool CertDatabase::ImportCACerts(const CertificateList& certificates,
-                                 unsigned int trust_bits,
+                                 TrustBits trust_bits,
                                  ImportCertFailureList* not_imported) {
   X509Certificate* root = FindRootInList(certificates);
-  return psm::ImportCACerts(certificates, root, trust_bits, not_imported);
+  bool success = psm::ImportCACerts(certificates, root, trust_bits,
+                                    not_imported);
+  if (success)
+    CertDatabase::NotifyObserversOfCertTrustChanged(NULL);
+
+  return success;
 }
 
 bool CertDatabase::ImportServerCert(const CertificateList& certificates,
@@ -169,8 +204,8 @@ bool CertDatabase::ImportServerCert(const CertificateList& certificates,
   return psm::ImportServerCert(certificates, not_imported);
 }
 
-unsigned int CertDatabase::GetCertTrust(
-    const X509Certificate* cert, CertType type) const {
+CertDatabase::TrustBits CertDatabase::GetCertTrust(const X509Certificate* cert,
+                                                   CertType type) const {
   CERTCertTrust nsstrust;
   SECStatus srv = CERT_GetCertTrust(cert->os_cert_handle(), &nsstrust);
   if (srv != SECSuccess) {
@@ -192,10 +227,67 @@ unsigned int CertDatabase::GetCertTrust(
   }
 }
 
+bool CertDatabase::IsUntrusted(const X509Certificate* cert) const {
+  CERTCertTrust nsstrust;
+  SECStatus rv = CERT_GetCertTrust(cert->os_cert_handle(), &nsstrust);
+  if (rv != SECSuccess) {
+    LOG(ERROR) << "CERT_GetCertTrust failed with error " << PORT_GetError();
+    return false;
+  }
+
+  // The CERTCertTrust structure contains three trust records:
+  // sslFlags, emailFlags, and objectSigningFlags.  The three
+  // trust records are independent of each other.
+  //
+  // If the CERTDB_TERMINAL_RECORD bit in a trust record is set,
+  // then that trust record is a terminal record.  A terminal
+  // record is used for explicit trust and distrust of an
+  // end-entity or intermediate CA cert.
+  //
+  // In a terminal record, if neither CERTDB_TRUSTED_CA nor
+  // CERTDB_TRUSTED is set, then the terminal record means
+  // explicit distrust.  On the other hand, if the terminal
+  // record has either CERTDB_TRUSTED_CA or CERTDB_TRUSTED bit
+  // set, then the terminal record means explicit trust.
+  //
+  // For a root CA, the trust record does not have
+  // the CERTDB_TERMINAL_RECORD bit set.
+
+  static const unsigned int kTrusted = CERTDB_TRUSTED_CA | CERTDB_TRUSTED;
+  if ((nsstrust.sslFlags & CERTDB_TERMINAL_RECORD) != 0 &&
+      (nsstrust.sslFlags & kTrusted) == 0) {
+    return true;
+  }
+  if ((nsstrust.emailFlags & CERTDB_TERMINAL_RECORD) != 0 &&
+      (nsstrust.emailFlags & kTrusted) == 0) {
+    return true;
+  }
+  if ((nsstrust.objectSigningFlags & CERTDB_TERMINAL_RECORD) != 0 &&
+      (nsstrust.objectSigningFlags & kTrusted) == 0) {
+    return true;
+  }
+
+  // Self-signed certificates that don't have any trust bits set are untrusted.
+  // Other certificates that don't have any trust bits set may still be trusted
+  // if they chain up to a trust anchor.
+  if (CERT_CompareName(&cert->os_cert_handle()->issuer,
+                       &cert->os_cert_handle()->subject) == SECEqual) {
+    return (nsstrust.sslFlags & kTrusted) == 0 &&
+           (nsstrust.emailFlags & kTrusted) == 0 &&
+           (nsstrust.objectSigningFlags & kTrusted) == 0;
+  }
+
+  return false;
+}
+
 bool CertDatabase::SetCertTrust(const X509Certificate* cert,
                                 CertType type,
-                                unsigned int trusted) {
-  return psm::SetCertTrust(cert, type, trusted);
+                                TrustBits trust_bits) {
+  bool success = psm::SetCertTrust(cert, type, trust_bits);
+  if (success)
+    CertDatabase::NotifyObserversOfCertTrustChanged(cert);
+
+  return success;
 }
 
 bool CertDatabase::DeleteCertAndKey(const X509Certificate* cert) {
@@ -217,6 +309,9 @@ bool CertDatabase::DeleteCertAndKey(const X509Certificate* cert) {
       return false;
     }
   }
+
+  CertDatabase::NotifyObserversOfUserCertRemoved(cert);
+
   return true;
 }
 
