@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #include <openssl/x509.h>
 
 #include "base/logging.h"
+#include "net/base/crypto_module.h"
 #include "net/base/net_errors.h"
 #include "net/base/openssl_private_key_store.h"
 #include "net/base/x509_certificate.h"
@@ -40,15 +41,29 @@ void CertDatabase::ListCerts(CertificateList* certs) {
   NOTIMPLEMENTED();
 }
 
-CryptoModule* CertDatabase::GetDefaultModule() const {
+CryptoModule* CertDatabase::GetPublicModule() const {
   // TODO(bulach): implement me.
   NOTIMPLEMENTED();
   return NULL;
 }
 
-int CertDatabase::ImportFromPKCS12(net::CryptoModule* module,
+CryptoModule* CertDatabase::GetPrivateModule() const {
+  // TODO(bulach): implement me.
+  NOTIMPLEMENTED();
+  return NULL;
+}
+
+void CertDatabase::ListModules(CryptoModuleList* modules, bool need_rw) const {
+  // TODO(bulach): implement me.
+  NOTIMPLEMENTED();
+  modules->clear();
+}
+
+int CertDatabase::ImportFromPKCS12(CryptoModule* module,
                                    const std::string& data,
-                                   const string16& password) {
+                                   const string16& password,
+                                   bool is_extractable,
+                                   CertificateList* imported_certs) {
   // TODO(bulach): implement me.
   NOTIMPLEMENTED();
   return ERR_NOT_IMPLEMENTED;
@@ -68,8 +83,8 @@ bool CertDatabase::DeleteCertAndKey(const X509Certificate* cert) {
   return false;
 }
 
-unsigned int CertDatabase::GetCertTrust(const X509Certificate* cert,
-                                        CertType type) const {
+CertDatabase::TrustBits CertDatabase::GetCertTrust(const X509Certificate* cert,
+                                                   CertType type) const {
   // TODO(bulach): implement me.
   NOTIMPLEMENTED();
   return 0;
@@ -77,7 +92,7 @@ unsigned int CertDatabase::GetCertTrust(const X509Certificate* cert,
 
 bool CertDatabase::SetCertTrust(const X509Certificate* cert,
                                 CertType type,
-                                unsigned int trust_bits) {
+                                TrustBits trust_bits) {
   // TODO(bulach): implement me.
   NOTIMPLEMENTED();
   return false;
