@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,22 +8,29 @@
 #include "base/logging.h"
 #include "base/message_loop.h"
 #include "base/path_service.h"
+#include "base/scoped_temp_dir.h"
 #include "net/base/directory_lister.h"
 #include "net/base/net_errors.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-namespace {
-class DirectoryListerTest : public testing::Test {};
-}
+namespace net {
 
-class ListerDelegate : public net::DirectoryLister::DirectoryListerDelegate {
+class ListerDelegate : public DirectoryLister::DirectoryListerDelegate {
  public:
-  explicit ListerDelegate(bool recursive) : error_(-1), recursive_(recursive) {
+  ListerDelegate(bool recursive,
+                 bool quit_loop_after_each_file)
+      : error_(-1),
+        recursive_(recursive),
+        quit_loop_after_each_file_(quit_loop_after_each_file) {
   }
-  void OnListFile(const net::DirectoryLister::DirectoryListerData& data) {
+
+  void OnListFile(const DirectoryLister::DirectoryListerData& data) {
     file_list_.push_back(data.info);
     paths_.push_back(data.path);
+    if (quit_loop_after_each_file_)
+      MessageLoop::current()->Quit();
   }
+
   void OnListDone(int error) {
     error_ = error;
     MessageLoop::current()->Quit();
@@ -32,6 +39,7 @@ class ListerDelegate : public net::DirectoryLister::DirectoryListerDelegate {
     else
       CheckSort();
   }
+
   void CheckRecursiveSort() {
     // Check that we got files in the right order.
     if (!file_list_.empty()) {
@@ -43,6 +51,7 @@ class ListerDelegate : public net::DirectoryLister::DirectoryListerDelegate {
       }
     }
   }
+
   void CheckSort() {
     // Check that we got files in the right order.
     if (!file_list_.empty()) {
@@ -64,10 +73,15 @@ class ListerDelegate : public net::DirectoryLister::DirectoryListerDelegate {
       }
     }
   }
+
   int error() const { return error_; }
+
+  int num_files() const { return file_list_.size(); }
+
  private:
   int error_;
   bool recursive_;
+  bool quit_loop_after_each_file_;
   std::vector<file_util::FileEnumerator::FindInfo> file_list_;
   std::vector<FilePath> paths_;
 };
@@ -76,47 +90,62 @@ TEST(DirectoryListerTest, BigDirTest) {
   FilePath path;
   ASSERT_TRUE(PathService::Get(base::DIR_SOURCE_ROOT, &path));
 
-  ListerDelegate delegate(false);
-  scoped_refptr<net::DirectoryLister> lister(
-      new net::DirectoryLister(path, &delegate));
-
-  lister->Start();
+  ListerDelegate delegate(false, false);
+  DirectoryLister lister(path, &delegate);
+  lister.Start();
 
   MessageLoop::current()->Run();
 
-  EXPECT_EQ(delegate.error(), net::OK);
+  EXPECT_EQ(OK, delegate.error());
 }
 
 TEST(DirectoryListerTest, BigDirRecursiveTest) {
   FilePath path;
   ASSERT_TRUE(PathService::Get(base::DIR_EXE, &path));
 
-  ListerDelegate delegate(true);
-  scoped_refptr<net::DirectoryLister> lister(
-      new net::DirectoryLister(path,
-                               true,
-                               net::DirectoryLister::FULL_PATH,
-                               &delegate));
-
-  lister->Start();
+  ListerDelegate delegate(true, false);
+  DirectoryLister lister(path, true, DirectoryLister::FULL_PATH, &delegate);
+  lister.Start();
 
   MessageLoop::current()->Run();
 
-  EXPECT_EQ(delegate.error(), net::OK);
+  EXPECT_EQ(OK, delegate.error());
 }
 
 TEST(DirectoryListerTest, CancelTest) {
   FilePath path;
   ASSERT_TRUE(PathService::Get(base::DIR_SOURCE_ROOT, &path));
 
-  ListerDelegate delegate(false);
-  scoped_refptr<net::DirectoryLister> lister(
-      new net::DirectoryLister(path, &delegate));
-
-  lister->Start();
-  lister->Cancel();
+  ListerDelegate delegate(false, true);
+  DirectoryLister lister(path, &delegate);
+  lister.Start();
 
   MessageLoop::current()->Run();
 
-  EXPECT_EQ(delegate.error(), net::ERR_ABORTED);
+  int num_files = delegate.num_files();
+
+  lister.Cancel();
+
+  MessageLoop::current()->RunAllPending();
+
+  EXPECT_EQ(num_files, delegate.num_files());
 }
+
+TEST(DirectoryListerTest, EmptyDirTest) {
+  ScopedTempDir tempDir;
+  EXPECT_TRUE(tempDir.CreateUniqueTempDir());
+
+  bool kRecursive = false;
+  bool kQuitLoopAfterEachFile = false;
+  ListerDelegate delegate(kRecursive, kQuitLoopAfterEachFile);
+  DirectoryLister lister(tempDir.path(), &delegate);
+  lister.Start();
+
+  MessageLoop::current()->Run();
+
+  // Contains only the parent directory ("..")
+  EXPECT_EQ(1, delegate.num_files());
+  EXPECT_EQ(OK, delegate.error());
+}
+
+}  // namespace net
