@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,6 +9,8 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/string_piece.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
@@ -17,29 +19,36 @@ namespace net {
 //
 //   dotted: a string in dotted form: "www.google.com"
 //   out: a result in DNS form: "\x03www\x06google\x03com\x00"
-bool DNSDomainFromDot(const std::string& dotted, std::string* out);
+NET_EXPORT_PRIVATE bool DNSDomainFromDot(const base::StringPiece& dotted,
+                                         std::string* out);
+
+// DNSDomainToString converts a domain in DNS format to a dotted string.
+// Excludes the dot at the end.
+NET_EXPORT_PRIVATE std::string DNSDomainToString(
+    const base::StringPiece& domain);
 
 // Returns true iff the given character is in the set of valid DNS label
 // characters as given in RFC 3490, 4.1, 3(a)
-bool IsSTD3ASCIIValidCharacter(char c);
+NET_EXPORT_PRIVATE bool IsSTD3ASCIIValidCharacter(char c);
 
 // Returns the hostname by trimming the ending dot, if one exists.
-std::string TrimEndingDot(const std::string& host);
+NET_EXPORT std::string TrimEndingDot(const base::StringPiece& host);
+
+// TODO(szym): remove all definitions below once DnsRRResolver migrates to
+// DnsClient
+
+// DNS class types.
+static const uint16 kClassIN = 1;
 
 // DNS resource record types. See
 // http://www.iana.org/assignments/dns-parameters
-// WARNING: if you're adding any new values here you may need to add them to
-// dnsrr_resolver.cc:DnsRRIsParsedByWindows.
-
 static const uint16 kDNS_CNAME = 5;
 static const uint16 kDNS_TXT = 16;
 static const uint16 kDNS_CERT = 37;
 static const uint16 kDNS_DS = 43;
 static const uint16 kDNS_RRSIG = 46;
 static const uint16 kDNS_DNSKEY = 48;
-static const uint16 kDNS_ANY = 0xff;
-static const uint16 kDNS_CAA = 13172;  // temporary, not IANA
-static const uint16 kDNS_TESTING = 0xfffe;  // in private use area.
+static const uint16 kDNS_CAA = 257;
 
 // http://www.iana.org/assignments/dns-sec-alg-numbers/dns-sec-alg-numbers.xhtml
 static const uint8 kDNSSEC_RSA_SHA1 = 5;
