@@ -7,37 +7,43 @@
 #pragma once
 
 #include <string>
+#include <vector>
 
 #include "base/basictypes.h"
 #include "base/string16.h"
+#include "net/base/net_export.h"
+
+namespace net {
 
 // Escaping --------------------------------------------------------------------
 
-// Escape a file.  This includes:
+// Escapes a file.  This includes:
 // non-printable, non-7bit, and (including space)  "#%:<>?[\]^`{|}
-std::string EscapePath(const std::string& path);
+NET_EXPORT std::string EscapePath(const std::string& path);
 
-// Escape application/x-www-form-urlencoded content.  This includes:
+// Escapes application/x-www-form-urlencoded content.  This includes:
 // non-printable, non-7bit, and (including space)  ?>=<;+'&%$#"![\]^`{|}
-// Space is escaped as + and other special characters as %XX (hex).
-std::string EscapeUrlEncodedData(const std::string& path);
+// Space is escaped as + (if use_plus is true) and other special characters
+// as %XX (hex).
+NET_EXPORT std::string EscapeUrlEncodedData(const std::string& path,
+                                            bool use_plus);
 
-// Escape all non-ASCII input.
-std::string EscapeNonASCII(const std::string& input);
+// Escapes all non-ASCII input.
+NET_EXPORT std::string EscapeNonASCII(const std::string& input);
 
 // Escapes characters in text suitable for use as an external protocol handler
 // command.
 // We %XX everything except alphanumerics and %-_.!~*'() and the restricted
 // chracters (;/?:@&=+$,).
-std::string EscapeExternalHandlerValue(const std::string& text);
+NET_EXPORT std::string EscapeExternalHandlerValue(const std::string& text);
 
-// Append the given character to the output string, escaping the character if
+// Appends the given character to the output string, escaping the character if
 // the character would be interpretted as an HTML delimiter.
-void AppendEscapedCharForHTML(char c, std::string* output);
+NET_EXPORT void AppendEscapedCharForHTML(char c, std::string* output);
 
-// Escape chars that might cause this text to be interpretted as HTML tags.
-std::string EscapeForHTML(const std::string& text);
-string16 EscapeForHTML(const string16& text);
+// Escapes chars that might cause this text to be interpretted as HTML tags.
+NET_EXPORT std::string EscapeForHTML(const std::string& text);
+NET_EXPORT string16 EscapeForHTML(const string16& text);
 
 // Unescaping ------------------------------------------------------------------
 
@@ -89,29 +95,34 @@ class UnescapeRule {
 // Watch out: this doesn't necessarily result in the correct final result,
 // because the encoding may be unknown. For example, the input might be ASCII,
 // which, after unescaping, is supposed to be interpreted as UTF-8, and then
-// converted into full wide chars. This function won't tell you if any
+// converted into full UTF-16 chars. This function won't tell you if any
 // conversions need to take place, it only unescapes.
-std::string UnescapeURLComponent(const std::string& escaped_text,
-                                 UnescapeRule::Type rules);
-string16 UnescapeURLComponent(const string16& escaped_text,
-                              UnescapeRule::Type rules);
+NET_EXPORT std::string UnescapeURLComponent(const std::string& escaped_text,
+                                            UnescapeRule::Type rules);
+NET_EXPORT string16 UnescapeURLComponent(const string16& escaped_text,
+                                         UnescapeRule::Type rules);
 
 // Unescapes the given substring as a URL, and then tries to interpret the
 // result as being encoded as UTF-8. If the result is convertable into UTF-8, it
 // will be returned as converted. If it is not, the original escaped string will
-// be converted into a string16 and returned.
-//
-// |offset_for_adjustment| may be NULL; if not, it is an offset into |text| that
-// will be adjusted to point at the same logical place in the result string.  If
-// this isn't possible because it points into the middle of an escape sequence
-// or past the end of the string, it will be set to string16::npos.
-string16 UnescapeAndDecodeUTF8URLComponent(const std::string& text,
-                                           UnescapeRule::Type rules,
-                                           size_t* offset_for_adjustment);
+// be converted into a string16 and returned. (|offset[s]_for_adjustment|)
+// specifies one or more offsets into the source strings; each offset will be
+// adjusted to point at the same logical place in the result strings during
+// decoding.  If this isn't possible because an offset points past the end of
+// the source strings or into the middle of a multibyte sequence, the offending
+// offset will be set to string16::npos. |offset[s]_for_adjustment| may be NULL.
+NET_EXPORT string16 UnescapeAndDecodeUTF8URLComponent(
+    const std::string& text,
+    UnescapeRule::Type rules,
+    size_t* offset_for_adjustment);
+NET_EXPORT string16 UnescapeAndDecodeUTF8URLComponentWithOffsets(
+    const std::string& text,
+    UnescapeRule::Type rules,
+    std::vector<size_t>* offsets_for_adjustment);
 
-// Unescape the following ampersand character codes from |text|:
+// Unescapes the following ampersand character codes from |text|:
 // &lt; &gt; &amp; &quot; &#39;
-string16 UnescapeForHTML(const string16& text);
+NET_EXPORT string16 UnescapeForHTML(const string16& text);
 
 // Deprecated ------------------------------------------------------------------
 
@@ -121,12 +132,35 @@ string16 UnescapeForHTML(const string16& text);
 // This is basically the same as encodeURIComponent in javascript.
 // For the string16 version, we do a conversion to charset before encoding the
 // string.  If the charset doesn't exist, we return false.
-std::string EscapeQueryParamValue(const std::string& text, bool use_plus);
-bool EscapeQueryParamValue(const string16& text, const char* codepage,
-                           bool use_plus, string16* escaped);
+NET_EXPORT std::string EscapeQueryParamValue(const std::string& text,
+                                             bool use_plus);
+NET_EXPORT bool EscapeQueryParamValue(const string16& text,
+                                      const char* codepage,
+                                      bool use_plus,
+                                      string16* escaped);
 
 // A specialized version of EscapeQueryParamValue for string16s that
 // assumes the codepage is UTF8.  This is provided as a convenience.
-string16 EscapeQueryParamValueUTF8(const string16& text, bool use_plus);
+NET_EXPORT string16 EscapeQueryParamValueUTF8(const string16& text,
+                                              bool use_plus);
+
+namespace internal {
+
+// Private Functions (Exposed for Unit Testing) --------------------------------
+
+// A function called by std::for_each that will adjust any offset which occurs
+// after one or more encoded characters.
+struct NET_EXPORT_PRIVATE AdjustEncodingOffset {
+  typedef std::vector<size_t> Adjustments;
+
+  explicit AdjustEncodingOffset(const Adjustments& adjustments);
+  void operator()(size_t& offset);
+
+  const Adjustments& adjustments;
+};
+
+}  // namespace internal
+
+}  // namespace net
 
 #endif  // NET_BASE_ESCAPE_H_
