@@ -1,23 +1,29 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "net/base/file_stream.h"
+
+#include "base/bind.h"
 #include "base/callback.h"
 #include "base/file_util.h"
 #include "base/message_loop.h"
 #include "base/path_service.h"
 #include "base/platform_file.h"
-#include "net/base/file_stream.h"
 #include "net/base/net_errors.h"
 #include "net/base/test_completion_callback.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
 
+namespace net {
+
 namespace {
 
 const char kTestData[] = "0123456789";
 const int kTestDataSize = arraysize(kTestData) - 1;
 
+}  // namespace
+
 class FileStreamTest : public PlatformTest {
  public:
   virtual void SetUp() {
@@ -31,16 +37,55 @@ class FileStreamTest : public PlatformTest {
 
     PlatformTest::TearDown();
   }
+
   const FilePath temp_file_path() const { return temp_file_path_; }
+
+  static base::PlatformFile GetFile(const FileStream& stream) {
+    return stream.file_;
+  }
+
  private:
   FilePath temp_file_path_;
 };
 
+namespace {
+
 TEST_F(FileStreamTest, BasicOpenClose) {
-  net::FileStream stream;
-  int rv = stream.Open(temp_file_path(),
-      base::PLATFORM_FILE_OPEN | base::PLATFORM_FILE_READ);
-  EXPECT_EQ(net::OK, rv);
+  base::PlatformFile file = base::kInvalidPlatformFileValue;
+  {
+    FileStream stream;
+    int rv = stream.Open(temp_file_path(),
+        base::PLATFORM_FILE_OPEN | base::PLATFORM_FILE_READ);
+    EXPECT_EQ(OK, rv);
+    EXPECT_TRUE(stream.IsOpen());
+    file = GetFile(stream);
+  }
+  EXPECT_NE(base::kInvalidPlatformFileValue, file);
+  base::PlatformFileInfo info;
+  // The file should be closed.
+  EXPECT_FALSE(base::GetPlatformFileInfo(file, &info));
+}
+
+TEST_F(FileStreamTest, FileHandleLeftOpen) {
+  bool created = false;
+  ASSERT_EQ(kTestDataSize,
+      file_util::WriteFile(temp_file_path(), kTestData, kTestDataSize));
+  int flags = base::PLATFORM_FILE_OPEN_ALWAYS | base::PLATFORM_FILE_READ;
+  base::PlatformFile file = base::CreatePlatformFile(
+      temp_file_path(), flags, &created, NULL);
+
+  {
+    // Seek to the beginning of the file and read.
+    FileStream read_stream(file, flags);
+    EXPECT_TRUE(read_stream.IsOpen());
+  }
+
+  EXPECT_NE(base::kInvalidPlatformFileValue, file);
+  base::PlatformFileInfo info;
+  // The file should still be open.
+  EXPECT_TRUE(base::GetPlatformFileInfo(file, &info));
+  // Clean up.
+  EXPECT_TRUE(base::ClosePlatformFile(file));
 }
 
 // Test the use of FileStream with a file handle provided at construction.
@@ -55,12 +100,13 @@ TEST_F(FileStreamTest, UseFileHandle) {
       temp_file_path(), flags, &created, NULL);
 
   // Seek to the beginning of the file and read.
-  net::FileStream read_stream(file, flags);
-  ASSERT_EQ(0, read_stream.Seek(net::FROM_BEGIN, 0));
+  FileStream read_stream(file, flags);
+  ASSERT_EQ(0, read_stream.Seek(FROM_BEGIN, 0));
   ASSERT_EQ(kTestDataSize, read_stream.Available());
   // Read into buffer and compare.
   char buffer[kTestDataSize];
-  ASSERT_EQ(kTestDataSize, read_stream.Read(buffer, kTestDataSize, NULL));
+  ASSERT_EQ(kTestDataSize,
+            read_stream.Read(buffer, kTestDataSize, CompletionCallback()));
   ASSERT_EQ(0, memcmp(kTestData, buffer, kTestDataSize));
   read_stream.Close();
 
@@ -69,9 +115,10 @@ TEST_F(FileStreamTest, UseFileHandle) {
   flags = base::PLATFORM_FILE_OPEN_ALWAYS | base::PLATFORM_FILE_WRITE;
   file = base::CreatePlatformFile(temp_file_path(), flags, &created, NULL);
 
-  net::FileStream write_stream(file, flags);
-  ASSERT_EQ(0, write_stream.Seek(net::FROM_BEGIN, 0));
-  ASSERT_EQ(kTestDataSize, write_stream.Write(kTestData, kTestDataSize, NULL));
+  FileStream write_stream(file, flags);
+  ASSERT_EQ(0, write_stream.Seek(FROM_BEGIN, 0));
+  ASSERT_EQ(kTestDataSize,
+            write_stream.Write(kTestData, kTestDataSize, CompletionCallback()));
   write_stream.Close();
 
   // Read into buffer and compare to make sure the handle worked fine.
@@ -81,22 +128,22 @@ TEST_F(FileStreamTest, UseFileHandle) {
 }
 
 TEST_F(FileStreamTest, UseClosedStream) {
-  net::FileStream stream;
+  FileStream stream;
 
   EXPECT_FALSE(stream.IsOpen());
 
   // Try seeking...
-  int64 new_offset = stream.Seek(net::FROM_BEGIN, 5);
-  EXPECT_EQ(net::ERR_UNEXPECTED, new_offset);
+  int64 new_offset = stream.Seek(FROM_BEGIN, 5);
+  EXPECT_EQ(ERR_UNEXPECTED, new_offset);
 
   // Try available...
   int64 avail = stream.Available();
-  EXPECT_EQ(net::ERR_UNEXPECTED, avail);
+  EXPECT_EQ(ERR_UNEXPECTED, avail);
 
   // Try reading...
   char buf[10];
-  int rv = stream.Read(buf, arraysize(buf), NULL);
-  EXPECT_EQ(net::ERR_UNEXPECTED, rv);
+  int rv = stream.Read(buf, arraysize(buf), CompletionCallback());
+  EXPECT_EQ(ERR_UNEXPECTED, rv);
 }
 
 TEST_F(FileStreamTest, BasicRead) {
@@ -104,11 +151,11 @@ TEST_F(FileStreamTest, BasicRead) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
@@ -118,7 +165,7 @@ TEST_F(FileStreamTest, BasicRead) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), NULL);
+    rv = stream.Read(buf, arraysize(buf), CompletionCallback());
     EXPECT_LE(0, rv);
     if (rv <= 0)
       break;
@@ -134,12 +181,12 @@ TEST_F(FileStreamTest, AsyncRead) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
@@ -151,8 +198,8 @@ TEST_F(FileStreamTest, AsyncRead) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), &callback);
-    if (rv == net::ERR_IO_PENDING)
+    rv = stream.Read(buf, arraysize(buf), callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LE(0, rv);
     if (rv <= 0)
@@ -169,12 +216,12 @@ TEST_F(FileStreamTest, AsyncRead_EarlyClose) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
@@ -182,10 +229,10 @@ TEST_F(FileStreamTest, AsyncRead_EarlyClose) {
   TestCompletionCallback callback;
 
   char buf[4];
-  rv = stream.Read(buf, arraysize(buf), &callback);
+  rv = stream.Read(buf, arraysize(buf), callback.callback());
   stream.Close();
   if (rv < 0) {
-    EXPECT_EQ(net::ERR_IO_PENDING, rv);
+    EXPECT_EQ(ERR_IO_PENDING, rv);
     // The callback should not be called if the request is cancelled.
     MessageLoop::current()->RunAllPending();
     EXPECT_FALSE(callback.have_result());
@@ -199,14 +246,14 @@ TEST_F(FileStreamTest, BasicRead_FromOffset) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   const int64 kOffset = 3;
-  int64 new_offset = stream.Seek(net::FROM_BEGIN, kOffset);
+  int64 new_offset = stream.Seek(FROM_BEGIN, kOffset);
   EXPECT_EQ(kOffset, new_offset);
 
   int64 total_bytes_avail = stream.Available();
@@ -217,7 +264,7 @@ TEST_F(FileStreamTest, BasicRead_FromOffset) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), NULL);
+    rv = stream.Read(buf, arraysize(buf), CompletionCallback());
     EXPECT_LE(0, rv);
     if (rv <= 0)
       break;
@@ -234,15 +281,15 @@ TEST_F(FileStreamTest, AsyncRead_FromOffset) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   const int64 kOffset = 3;
-  int64 new_offset = stream.Seek(net::FROM_BEGIN, kOffset);
+  int64 new_offset = stream.Seek(FROM_BEGIN, kOffset);
   EXPECT_EQ(kOffset, new_offset);
 
   int64 total_bytes_avail = stream.Available();
@@ -255,8 +302,8 @@ TEST_F(FileStreamTest, AsyncRead_FromOffset) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), &callback);
-    if (rv == net::ERR_IO_PENDING)
+    rv = stream.Read(buf, arraysize(buf), callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LE(0, rv);
     if (rv <= 0)
@@ -269,41 +316,41 @@ TEST_F(FileStreamTest, AsyncRead_FromOffset) {
 }
 
 TEST_F(FileStreamTest, SeekAround) {
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   const int64 kOffset = 3;
-  int64 new_offset = stream.Seek(net::FROM_BEGIN, kOffset);
+  int64 new_offset = stream.Seek(FROM_BEGIN, kOffset);
   EXPECT_EQ(kOffset, new_offset);
 
-  new_offset = stream.Seek(net::FROM_CURRENT, kOffset);
+  new_offset = stream.Seek(FROM_CURRENT, kOffset);
   EXPECT_EQ(2 * kOffset, new_offset);
 
-  new_offset = stream.Seek(net::FROM_CURRENT, -kOffset);
+  new_offset = stream.Seek(FROM_CURRENT, -kOffset);
   EXPECT_EQ(kOffset, new_offset);
 
   const int kTestDataLen = arraysize(kTestData) - 1;
 
-  new_offset = stream.Seek(net::FROM_END, -kTestDataLen);
+  new_offset = stream.Seek(FROM_END, -kTestDataLen);
   EXPECT_EQ(0, new_offset);
 }
 
 TEST_F(FileStreamTest, BasicWrite) {
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_CREATE_ALWAYS |
               base::PLATFORM_FILE_WRITE;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 file_size;
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
   EXPECT_EQ(0, file_size);
 
-  rv = stream.Write(kTestData, kTestDataSize, NULL);
+  rv = stream.Write(kTestData, kTestDataSize, CompletionCallback());
   EXPECT_EQ(kTestDataSize, rv);
   stream.Close();
 
@@ -313,12 +360,12 @@ TEST_F(FileStreamTest, BasicWrite) {
 }
 
 TEST_F(FileStreamTest, AsyncWrite) {
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_CREATE_ALWAYS |
               base::PLATFORM_FILE_WRITE |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 file_size;
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
@@ -331,8 +378,8 @@ TEST_F(FileStreamTest, AsyncWrite) {
   while (total_bytes_written != kTestDataSize) {
     rv = stream.Write(kTestData + total_bytes_written,
                       kTestDataSize - total_bytes_written,
-                      &callback);
-    if (rv == net::ERR_IO_PENDING)
+                      callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LT(0, rv);
     if (rv <= 0)
@@ -345,12 +392,12 @@ TEST_F(FileStreamTest, AsyncWrite) {
 }
 
 TEST_F(FileStreamTest, AsyncWrite_EarlyClose) {
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_CREATE_ALWAYS |
               base::PLATFORM_FILE_WRITE |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 file_size;
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
@@ -362,10 +409,10 @@ TEST_F(FileStreamTest, AsyncWrite_EarlyClose) {
 
   rv = stream.Write(kTestData + total_bytes_written,
                     kTestDataSize - total_bytes_written,
-                    &callback);
+                    callback.callback());
   stream.Close();
   if (rv < 0) {
-    EXPECT_EQ(net::ERR_IO_PENDING, rv);
+    EXPECT_EQ(ERR_IO_PENDING, rv);
     // The callback should not be called if the request is cancelled.
     MessageLoop::current()->RunAllPending();
     EXPECT_FALSE(callback.have_result());
@@ -377,11 +424,11 @@ TEST_F(FileStreamTest, AsyncWrite_EarlyClose) {
 }
 
 TEST_F(FileStreamTest, BasicWrite_FromOffset) {
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_WRITE;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 file_size;
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
@@ -389,10 +436,10 @@ TEST_F(FileStreamTest, BasicWrite_FromOffset) {
   EXPECT_EQ(kTestDataSize, file_size);
 
   const int64 kOffset = 0;
-  int64 new_offset = stream.Seek(net::FROM_END, kOffset);
+  int64 new_offset = stream.Seek(FROM_END, kOffset);
   EXPECT_EQ(kTestDataSize, new_offset);
 
-  rv = stream.Write(kTestData, kTestDataSize, NULL);
+  rv = stream.Write(kTestData, kTestDataSize, CompletionCallback());
   EXPECT_EQ(kTestDataSize, rv);
   stream.Close();
 
@@ -406,15 +453,15 @@ TEST_F(FileStreamTest, AsyncWrite_FromOffset) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_WRITE |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   const int64 kOffset = 0;
-  int64 new_offset = stream.Seek(net::FROM_END, kOffset);
+  int64 new_offset = stream.Seek(FROM_END, kOffset);
   EXPECT_EQ(kTestDataSize, new_offset);
 
   TestCompletionCallback callback;
@@ -423,8 +470,8 @@ TEST_F(FileStreamTest, AsyncWrite_FromOffset) {
   while (total_bytes_written != kTestDataSize) {
     rv = stream.Write(kTestData + total_bytes_written,
                       kTestDataSize - total_bytes_written,
-                      &callback);
-    if (rv == net::ERR_IO_PENDING)
+                      callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LT(0, rv);
     if (rv <= 0)
@@ -441,12 +488,12 @@ TEST_F(FileStreamTest, BasicReadWrite) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_WRITE;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
@@ -456,7 +503,7 @@ TEST_F(FileStreamTest, BasicReadWrite) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), NULL);
+    rv = stream.Read(buf, arraysize(buf), CompletionCallback());
     EXPECT_LE(0, rv);
     if (rv <= 0)
       break;
@@ -466,7 +513,7 @@ TEST_F(FileStreamTest, BasicReadWrite) {
   EXPECT_EQ(file_size, total_bytes_read);
   EXPECT_TRUE(data_read == kTestData);
 
-  rv = stream.Write(kTestData, kTestDataSize, NULL);
+  rv = stream.Write(kTestData, kTestDataSize, CompletionCallback());
   EXPECT_EQ(kTestDataSize, rv);
   stream.Close();
 
@@ -480,23 +527,23 @@ TEST_F(FileStreamTest, BasicWriteRead) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_WRITE;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
 
-  int64 offset = stream.Seek(net::FROM_END, 0);
+  int64 offset = stream.Seek(FROM_END, 0);
   EXPECT_EQ(offset, file_size);
 
-  rv = stream.Write(kTestData, kTestDataSize, NULL);
+  rv = stream.Write(kTestData, kTestDataSize, CompletionCallback());
   EXPECT_EQ(kTestDataSize, rv);
 
-  offset = stream.Seek(net::FROM_BEGIN, 0);
+  offset = stream.Seek(FROM_BEGIN, 0);
   EXPECT_EQ(0, offset);
 
   int64 total_bytes_read = 0;
@@ -504,7 +551,7 @@ TEST_F(FileStreamTest, BasicWriteRead) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), NULL);
+    rv = stream.Read(buf, arraysize(buf), CompletionCallback());
     EXPECT_LE(0, rv);
     if (rv <= 0)
       break;
@@ -528,13 +575,13 @@ TEST_F(FileStreamTest, BasicAsyncReadWrite) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_WRITE |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
@@ -545,8 +592,8 @@ TEST_F(FileStreamTest, BasicAsyncReadWrite) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), &callback);
-    if (rv == net::ERR_IO_PENDING)
+    rv = stream.Read(buf, arraysize(buf), callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LE(0, rv);
     if (rv <= 0)
@@ -562,8 +609,8 @@ TEST_F(FileStreamTest, BasicAsyncReadWrite) {
   while (total_bytes_written != kTestDataSize) {
     rv = stream.Write(kTestData + total_bytes_written,
                       kTestDataSize - total_bytes_written,
-                      &callback);
-    if (rv == net::ERR_IO_PENDING)
+                      callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LT(0, rv);
     if (rv <= 0)
@@ -583,18 +630,18 @@ TEST_F(FileStreamTest, BasicAsyncWriteRead) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_WRITE |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
 
-  int64 offset = stream.Seek(net::FROM_END, 0);
+  int64 offset = stream.Seek(FROM_END, 0);
   EXPECT_EQ(offset, file_size);
 
   TestCompletionCallback callback;
@@ -603,8 +650,8 @@ TEST_F(FileStreamTest, BasicAsyncWriteRead) {
   while (total_bytes_written != kTestDataSize) {
     rv = stream.Write(kTestData + total_bytes_written,
                       kTestDataSize - total_bytes_written,
-                      &callback);
-    if (rv == net::ERR_IO_PENDING)
+                      callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LT(0, rv);
     if (rv <= 0)
@@ -614,7 +661,7 @@ TEST_F(FileStreamTest, BasicAsyncWriteRead) {
 
   EXPECT_EQ(kTestDataSize, total_bytes_written);
 
-  offset = stream.Seek(net::FROM_BEGIN, 0);
+  offset = stream.Seek(FROM_BEGIN, 0);
   EXPECT_EQ(0, offset);
 
   int total_bytes_read = 0;
@@ -622,8 +669,8 @@ TEST_F(FileStreamTest, BasicAsyncWriteRead) {
   std::string data_read;
   for (;;) {
     char buf[4];
-    rv = stream.Read(buf, arraysize(buf), &callback);
-    if (rv == net::ERR_IO_PENDING)
+    rv = stream.Read(buf, arraysize(buf), callback.callback());
+    if (rv == ERR_IO_PENDING)
       rv = callback.WaitForResult();
     EXPECT_LE(0, rv);
     if (rv <= 0)
@@ -643,10 +690,10 @@ TEST_F(FileStreamTest, BasicAsyncWriteRead) {
   EXPECT_EQ(kExpectedFileData, data_read);
 }
 
-class TestWriteReadCompletionCallback : public Callback1<int>::Type {
+class TestWriteReadCompletionCallback {
  public:
-  explicit TestWriteReadCompletionCallback(
-      net::FileStream* stream,
+  TestWriteReadCompletionCallback(
+      FileStream* stream,
       int* total_bytes_written,
       int* total_bytes_read,
       std::string* data_read)
@@ -656,7 +703,9 @@ class TestWriteReadCompletionCallback : public Callback1<int>::Type {
         stream_(stream),
         total_bytes_written_(total_bytes_written),
         total_bytes_read_(total_bytes_read),
-        data_read_(data_read) {}
+        data_read_(data_read),
+        callback_(base::Bind(&TestWriteReadCompletionCallback::OnComplete,
+                             base::Unretained(this))) {}
 
   int WaitForResult() {
     DCHECK(!waiting_for_result_);
@@ -669,10 +718,12 @@ class TestWriteReadCompletionCallback : public Callback1<int>::Type {
     return result_;
   }
 
+  const CompletionCallback& callback() const { return callback_; }
+
  private:
-  virtual void RunWithParams(const Tuple1<int>& params) {
-    DCHECK_LT(0, params.a);
-    *total_bytes_written_ += params.a;
+  void OnComplete(int result) {
+    DCHECK_LT(0, result);
+    *total_bytes_written_ += result;
 
     int rv;
 
@@ -684,20 +735,20 @@ class TestWriteReadCompletionCallback : public Callback1<int>::Type {
           stream_, &total_bytes_written, &total_bytes_read, &data_read);
       rv = stream_->Write(kTestData + *total_bytes_written_,
                           kTestDataSize - *total_bytes_written_,
-                          &callback);
-      DCHECK_EQ(net::ERR_IO_PENDING, rv);
+                          callback.callback());
+      DCHECK_EQ(ERR_IO_PENDING, rv);
       rv = callback.WaitForResult();
       *total_bytes_written_ += total_bytes_written;
       *total_bytes_read_ += total_bytes_read;
       *data_read_ += data_read;
     } else {  // We're done writing all data.  Start reading the data.
-      stream_->Seek(net::FROM_BEGIN, 0);
+      stream_->Seek(FROM_BEGIN, 0);
 
       TestCompletionCallback callback;
       for (;;) {
         char buf[4];
-        rv = stream_->Read(buf, arraysize(buf), &callback);
-        if (rv == net::ERR_IO_PENDING) {
+        rv = stream_->Read(buf, arraysize(buf), callback.callback());
+        if (rv == ERR_IO_PENDING) {
           bool old_state = MessageLoop::current()->NestableTasksAllowed();
           MessageLoop::current()->SetNestableTasksAllowed(true);
           rv = callback.WaitForResult();
@@ -720,10 +771,11 @@ class TestWriteReadCompletionCallback : public Callback1<int>::Type {
   int result_;
   bool have_result_;
   bool waiting_for_result_;
-  net::FileStream* stream_;
+  FileStream* stream_;
   int* total_bytes_written_;
   int* total_bytes_read_;
   std::string* data_read_;
+  const CompletionCallback callback_;
 
   DISALLOW_COPY_AND_ASSIGN(TestWriteReadCompletionCallback);
 };
@@ -733,18 +785,18 @@ TEST_F(FileStreamTest, AsyncWriteRead) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_WRITE |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
 
-  int64 offset = stream.Seek(net::FROM_END, 0);
+  int64 offset = stream.Seek(FROM_END, 0);
   EXPECT_EQ(offset, file_size);
 
   int total_bytes_written = 0;
@@ -755,8 +807,8 @@ TEST_F(FileStreamTest, AsyncWriteRead) {
 
   rv = stream.Write(kTestData + total_bytes_written,
                     kTestDataSize - static_cast<int>(total_bytes_written),
-                    &callback);
-  if (rv == net::ERR_IO_PENDING)
+                    callback.callback());
+  if (rv == ERR_IO_PENDING)
     rv = callback.WaitForResult();
   EXPECT_LT(0, rv);
   EXPECT_EQ(kTestDataSize, total_bytes_written);
@@ -773,15 +825,16 @@ TEST_F(FileStreamTest, AsyncWriteRead) {
   EXPECT_EQ(kExpectedFileData, data_read);
 }
 
-class TestWriteCloseCompletionCallback : public Callback1<int>::Type {
+class TestWriteCloseCompletionCallback {
  public:
-  explicit TestWriteCloseCompletionCallback(net::FileStream* stream,
-                                            int* total_bytes_written)
+  TestWriteCloseCompletionCallback(FileStream* stream, int* total_bytes_written)
       : result_(0),
         have_result_(false),
         waiting_for_result_(false),
         stream_(stream),
-        total_bytes_written_(total_bytes_written) {}
+        total_bytes_written_(total_bytes_written),
+        callback_(base::Bind(&TestWriteCloseCompletionCallback::OnComplete,
+                             base::Unretained(this))) {}
 
   int WaitForResult() {
     DCHECK(!waiting_for_result_);
@@ -794,10 +847,12 @@ class TestWriteCloseCompletionCallback : public Callback1<int>::Type {
     return result_;
   }
 
+  const CompletionCallback& callback() const { return callback_; }
+
  private:
-  virtual void RunWithParams(const Tuple1<int>& params) {
-    DCHECK_LT(0, params.a);
-    *total_bytes_written_ += params.a;
+  void OnComplete(int result) {
+    DCHECK_LT(0, result);
+    *total_bytes_written_ += result;
 
     int rv;
 
@@ -807,8 +862,8 @@ class TestWriteCloseCompletionCallback : public Callback1<int>::Type {
       TestWriteCloseCompletionCallback callback(stream_, &total_bytes_written);
       rv = stream_->Write(kTestData + *total_bytes_written_,
                           kTestDataSize - *total_bytes_written_,
-                          &callback);
-      DCHECK_EQ(net::ERR_IO_PENDING, rv);
+                          callback.callback());
+      DCHECK_EQ(ERR_IO_PENDING, rv);
       rv = callback.WaitForResult();
       *total_bytes_written_ += total_bytes_written;
     } else {  // We're done writing all data.  Close the file.
@@ -824,8 +879,9 @@ class TestWriteCloseCompletionCallback : public Callback1<int>::Type {
   int result_;
   bool have_result_;
   bool waiting_for_result_;
-  net::FileStream* stream_;
+  FileStream* stream_;
   int* total_bytes_written_;
+  const CompletionCallback callback_;
 
   DISALLOW_COPY_AND_ASSIGN(TestWriteCloseCompletionCallback);
 };
@@ -835,25 +891,25 @@ TEST_F(FileStreamTest, AsyncWriteClose) {
   bool ok = file_util::GetFileSize(temp_file_path(), &file_size);
   EXPECT_TRUE(ok);
 
-  net::FileStream stream;
+  FileStream stream;
   int flags = base::PLATFORM_FILE_OPEN |
               base::PLATFORM_FILE_READ |
               base::PLATFORM_FILE_WRITE |
               base::PLATFORM_FILE_ASYNC;
   int rv = stream.Open(temp_file_path(), flags);
-  EXPECT_EQ(net::OK, rv);
+  EXPECT_EQ(OK, rv);
 
   int64 total_bytes_avail = stream.Available();
   EXPECT_EQ(file_size, total_bytes_avail);
 
-  int64 offset = stream.Seek(net::FROM_END, 0);
+  int64 offset = stream.Seek(FROM_END, 0);
   EXPECT_EQ(offset, file_size);
 
   int total_bytes_written = 0;
   TestWriteCloseCompletionCallback callback(&stream, &total_bytes_written);
 
-  rv = stream.Write(kTestData, kTestDataSize, &callback);
-  if (rv == net::ERR_IO_PENDING)
+  rv = stream.Write(kTestData, kTestDataSize, callback.callback());
+  if (rv == ERR_IO_PENDING)
     total_bytes_written = callback.WaitForResult();
   EXPECT_LT(0, total_bytes_written);
   EXPECT_EQ(kTestDataSize, total_bytes_written);
@@ -867,18 +923,18 @@ TEST_F(FileStreamTest, AsyncWriteClose) {
 TEST_F(FileStreamTest, Truncate) {
   int flags = base::PLATFORM_FILE_CREATE_ALWAYS | base::PLATFORM_FILE_WRITE;
 
-  net::FileStream write_stream;
-  ASSERT_EQ(net::OK, write_stream.Open(temp_file_path(), flags));
+  FileStream write_stream;
+  ASSERT_EQ(OK, write_stream.Open(temp_file_path(), flags));
 
   // Write some data to the file.
   const char test_data[] = "0123456789";
-  write_stream.Write(test_data, arraysize(test_data), NULL);
+  write_stream.Write(test_data, arraysize(test_data), CompletionCallback());
 
   // Truncate the file.
   ASSERT_EQ(4, write_stream.Truncate(4));
 
   // Write again.
-  write_stream.Write(test_data, 4, NULL);
+  write_stream.Write(test_data, 4, CompletionCallback());
 
   // Close the stream.
   write_stream.Close();
@@ -891,3 +947,5 @@ TEST_F(FileStreamTest, Truncate) {
 }
 
 }  // namespace
+
+}  // namespace net
