@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,16 +10,17 @@
 #include <string>
 
 #include "base/gtest_prod_util.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/threading/non_thread_safe.h"
 #include "base/time.h"
 #include "net/base/address_family.h"
 #include "net/base/address_list.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
 // Cache used by HostResolver to map hostnames to their resolved result.
-class HostCache : public base::NonThreadSafe {
+class NET_EXPORT HostCache : NON_EXPORTED_BASE(public base::NonThreadSafe) {
  public:
   // Stores the latest address list that was looked up for a hostname.
   struct Entry : public base::RefCounted<Entry> {
@@ -72,12 +73,8 @@ class HostCache : public base::NonThreadSafe {
 
   typedef std::map<Key, scoped_refptr<Entry> > EntryMap;
 
-  // Constructs a HostCache that caches successful host resolves for
-  // |success_entry_ttl| time, and failed host resolves for
-  // |failure_entry_ttl|. The cache will store up to |max_entries|.
-  HostCache(size_t max_entries,
-            base::TimeDelta success_entry_ttl,
-            base::TimeDelta failure_entry_ttl);
+  // Constructs a HostCache that stores up to |max_entries|.
+  explicit HostCache(size_t max_entries);
 
   ~HostCache();
 
@@ -87,12 +84,13 @@ class HostCache : public base::NonThreadSafe {
 
   // Overwrites or creates an entry for |key|. Returns the pointer to the
   // entry, or NULL on failure (fails if caching is disabled).
-  // (|error|, |addrlist|) is the value to set, and |now| is the current
-  // timestamp.
+  // (|error|, |addrlist|) is the value to set, |now| is the current time
+  // |ttl| is the "time to live".
   Entry* Set(const Key& key,
              int error,
              const AddressList& addrlist,
-             base::TimeTicks now);
+             base::TimeTicks now,
+             base::TimeDelta ttl);
 
   // Empties the cache
   void clear();
@@ -103,13 +101,12 @@ class HostCache : public base::NonThreadSafe {
   // Following are used by net_internals UI.
   size_t max_entries() const;
 
-  base::TimeDelta success_entry_ttl() const;
-
-  base::TimeDelta failure_entry_ttl() const;
-
   // Note that this map may contain expired entries.
   const EntryMap& entries() const;
 
+  // Creates a default cache.
+  static HostCache* CreateDefaultCache();
+
  private:
   FRIEND_TEST_ALL_PREFIXES(HostCacheTest, Compact);
   FRIEND_TEST_ALL_PREFIXES(HostCacheTest, NoCache);
@@ -129,10 +126,6 @@ class HostCache : public base::NonThreadSafe {
   // Bound on total size of the cache.
   size_t max_entries_;
 
-  // Time to live for cache entries.
-  base::TimeDelta success_entry_ttl_;
-  base::TimeDelta failure_entry_ttl_;
-
   // Map from hostname (presumably in lowercase canonicalized format) to
   // a resolved result entry.
   EntryMap entries_;
