@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,12 +8,16 @@
 
 #include <vector>
 
-#include "base/scoped_ptr.h"
+#include "base/basictypes.h"
+#include "base/gtest_prod_util.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/threading/non_thread_safe.h"
+#include "base/time.h"
 #include "net/base/capturing_net_log.h"
 #include "net/base/host_cache.h"
 #include "net/base/host_resolver.h"
 #include "net/base/host_resolver_proc.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/base/network_change_notifier.h"
 
@@ -48,10 +52,18 @@ namespace net {
 // threads.
 //
 // Requests are ordered in the queue based on their priority.
+//
+// Whenever we try to resolve the host, we post a delayed task to check if host
+// resolution (OnLookupComplete) is completed or not. If the original attempt
+// hasn't completed, then we start another attempt for host resolution. We take
+// the results from the first attempt that finishes and ignore the results from
+// all other attempts.
 
-class HostResolverImpl : public HostResolver,
-                         public base::NonThreadSafe,
-                         public NetworkChangeNotifier::Observer {
+class NET_EXPORT HostResolverImpl
+    : public HostResolver,
+      NON_EXPORTED_BASE(public base::NonThreadSafe),
+      public NetworkChangeNotifier::IPAddressObserver,
+      public NetworkChangeNotifier::DNSObserver {
  public:
   // The index into |job_pools_| for the various job pools. Pools with a higher
   // index have lower priority.
@@ -74,12 +86,24 @@ class HostResolverImpl : public HostResolver,
   // |resolver_proc| is NULL then the default host resolver procedure is
   // used (which is SystemHostResolverProc except if overridden).
   // |max_jobs| specifies the maximum number of threads that the host resolver
-  // will use. Use SetPoolConstraints() to specify finer-grain settings.
+  // will use (not counting potential duplicate attempts). Use
+  // SetPoolConstraints() to specify finer-grain settings.
+  // |max_retry_attempts| is the maximum number of times we will retry for host
+  // resolution. Pass HostResolver::kDefaultRetryAttempts to choose a default
+  // value.
+  //
+  // For each attempt, we could start another attempt if host is not resolved
+  // within unresponsive_delay_ time. We keep attempting to resolve the host
+  // for max_retry_attempts. For every retry attempt, we grow the
+  // unresponsive_delay_ by the retry_factor_ amount (that is retry interval is
+  // multiplied by the retry factor each time). Once we have retried
+  // max_retry_attempts, we give up on additional attempts.
   //
   // |net_log| must remain valid for the life of the HostResolverImpl.
   HostResolverImpl(HostResolverProc* resolver_proc,
                    HostCache* cache,
                    size_t max_jobs,
+                   size_t max_retry_attempts,
                    NetLog* net_log);
 
   // If any completion callbacks are pending when the resolver is destroyed,
@@ -87,13 +111,6 @@ class HostResolverImpl : public HostResolver,
   // be called.
   virtual ~HostResolverImpl();
 
-  // Continuously observe whether IPv6 is supported, and set the allowable
-  // address family to IPv4 iff IPv6 is not supported.
-  void ProbeIPv6Support();
-
-  // Returns the cache this resolver uses, or NULL if caching is disabled.
-  HostCache* cache() { return cache_.get(); }
-
   // Applies a set of constraints for requests that belong to the specified
   // pool. NOTE: Don't call this after requests have been already been started.
   //
@@ -112,23 +129,25 @@ class HostResolverImpl : public HostResolver,
   // HostResolver methods:
   virtual int Resolve(const RequestInfo& info,
                       AddressList* addresses,
-                      CompletionCallback* callback,
+                      const CompletionCallback& callback,
                       RequestHandle* out_req,
-                      const BoundNetLog& source_net_log);
-  virtual void CancelRequest(RequestHandle req);
-  virtual void AddObserver(HostResolver::Observer* observer);
-  virtual void RemoveObserver(HostResolver::Observer* observer);
-
-  // Set address family, and disable IPv6 probe support.
-  virtual void SetDefaultAddressFamily(AddressFamily address_family);
-  virtual AddressFamily GetDefaultAddressFamily() const;
-
-  virtual HostResolverImpl* GetAsHostResolverImpl();
-
-  // TODO(eroman): hack for http://crbug.com/15513
-  virtual void Shutdown();
+                      const BoundNetLog& source_net_log) OVERRIDE;
+  virtual int ResolveFromCache(const RequestInfo& info,
+                               AddressList* addresses,
+                               const BoundNetLog& source_net_log) OVERRIDE;
+  virtual void CancelRequest(RequestHandle req) OVERRIDE;
+  virtual void SetDefaultAddressFamily(AddressFamily address_family) OVERRIDE;
+  virtual AddressFamily GetDefaultAddressFamily() const OVERRIDE;
+  virtual void ProbeIPv6Support() OVERRIDE;
+  virtual HostCache* GetHostCache() OVERRIDE;
 
  private:
+  // Allow tests to access our innards for testing purposes.
+  friend class LookupAttemptHostResolverProc;
+
+  // Allow tests to access our innards for testing purposes.
+  FRIEND_TEST_ALL_PREFIXES(HostResolverImplTest, MultipleAttempts);
+
   class Job;
   class JobPool;
   class IPv6ProbeJob;
@@ -136,7 +155,31 @@ class HostResolverImpl : public HostResolver,
   typedef std::vector<Request*> RequestsList;
   typedef HostCache::Key Key;
   typedef std::map<Key, scoped_refptr<Job> > JobMap;
-  typedef std::vector<HostResolver::Observer*> ObserversList;
+
+  // Helper used by |Resolve()| and |ResolveFromCache()|.  Performs IP
+  // literal and cache lookup, returns OK if successful,
+  // ERR_NAME_NOT_RESOLVED if either hostname is invalid or IP literal is
+  // incompatible, ERR_DNS_CACHE_MISS if entry was not found in cache.
+  int ResolveHelper(const Key& key,
+                    const RequestInfo& info,
+                    AddressList* addresses,
+                    const BoundNetLog& request_net_log);
+
+  // Tries to resolve |key| as an IP, returns true and sets |net_error| if
+  // succeeds, returns false otherwise.
+  bool ResolveAsIP(const Key& key,
+                   const RequestInfo& info,
+                   int* net_error,
+                   AddressList* addresses);
+
+  // If |key| is not found in cache returns false, otherwise returns
+  // true, sets |net_error| to the cached error code and fills |addresses|
+  // if it is a positive entry.
+  bool ServeFromCache(const Key& key,
+                      const RequestInfo& info,
+                      const BoundNetLog& request_net_log,
+                      int* net_error,
+                      AddressList* addresses);
 
   // Returns the HostResolverProc to use for this instance.
   HostResolverProc* effective_resolver_proc() const {
@@ -168,13 +211,11 @@ class HostResolverImpl : public HostResolver,
   // Called when a request has just been started.
   void OnStartRequest(const BoundNetLog& source_net_log,
                       const BoundNetLog& request_net_log,
-                      int request_id,
                       const RequestInfo& info);
 
   // Called when a request has just completed (before its callback is run).
   void OnFinishRequest(const BoundNetLog& source_net_log,
                        const BoundNetLog& request_net_log,
-                       int request_id,
                        const RequestInfo& info,
                        int net_error,
                        int os_error);
@@ -182,7 +223,6 @@ class HostResolverImpl : public HostResolver,
   // Called when a request has been cancelled.
   void OnCancelRequest(const BoundNetLog& source_net_log,
                        const BoundNetLog& request_net_log,
-                       int request_id,
                        const RequestInfo& info);
 
   // Notify IPv6ProbeJob not to call back, and discard reference to the job.
@@ -223,8 +263,33 @@ class HostResolverImpl : public HostResolver,
   // Aborts all in progress jobs (but might start new ones).
   void AbortAllInProgressJobs();
 
-  // NetworkChangeNotifier::Observer methods:
-  virtual void OnIPAddressChanged();
+  // NetworkChangeNotifier::IPAddressObserver methods:
+  virtual void OnIPAddressChanged() OVERRIDE;
+
+  // Helper methods to get and set max_retry_attempts_.
+  size_t max_retry_attempts() const {
+    return max_retry_attempts_;
+  }
+  void set_max_retry_attempts(const size_t max_retry_attempts) {
+    max_retry_attempts_ = max_retry_attempts;
+  }
+
+  // Helper methods for unit tests to get and set unresponsive_delay_.
+  base::TimeDelta unresponsive_delay() const { return unresponsive_delay_; }
+  void set_unresponsive_delay(const base::TimeDelta& unresponsive_delay) {
+    unresponsive_delay_ = unresponsive_delay;
+  }
+
+  // Helper methods to get and set retry_factor_.
+  uint32 retry_factor() const {
+    return retry_factor_;
+  }
+  void set_retry_factor(const uint32 retry_factor) {
+    retry_factor_ = retry_factor;
+  }
+
+  // NetworkChangeNotifier::OnDNSChanged methods:
+  virtual void OnDNSChanged() OVERRIDE;
 
   // Cache of host resolution results.
   scoped_ptr<HostCache> cache_;
@@ -232,9 +297,22 @@ class HostResolverImpl : public HostResolver,
   // Map from hostname to outstanding job.
   JobMap jobs_;
 
-  // Maximum number of concurrent jobs allowed, across all pools.
+  // Maximum number of concurrent jobs allowed, across all pools. Each job may
+  // create multiple concurrent resolve attempts for the hostname.
   size_t max_jobs_;
 
+  // Maximum number retry attempts to resolve the hostname.
+  size_t max_retry_attempts_;
+
+  // This is the limit after which we make another attempt to resolve the host
+  // if the worker thread has not responded yet. Allow unit tests to change the
+  // value.
+  base::TimeDelta unresponsive_delay_;
+
+  // Factor to grow unresponsive_delay_ when we re-re-try. Allow unit tests to
+  // change the value.
+  uint32 retry_factor_;
+
   // The information to track pending requests for a JobPool, as well as
   // how many outstanding jobs the pool already has, and its constraints.
   JobPool* job_pools_[POOL_COUNT];
@@ -243,13 +321,6 @@ class HostResolverImpl : public HostResolver,
   // HostResolver gets deleted from within the callback).
   scoped_refptr<Job> cur_completing_job_;
 
-  // The observers to notify when a request starts/ends.
-  ObserversList observers_;
-
-  // Monotonically increasing ID number to assign to the next request.
-  // Observers are the only consumers of this ID number.
-  int next_request_id_;
-
   // Monotonically increasing ID number to assign to the next job.
   // The only consumer of this ID is the requests tracing code.
   int next_job_id_;
@@ -261,9 +332,6 @@ class HostResolverImpl : public HostResolver,
   // Address family to use when the request doesn't specify one.
   AddressFamily default_address_family_;
 
-  // TODO(eroman): hack for http://crbug.com/15513
-  bool shutdown_;
-
   // Indicate if probing is done after each network change event to set address
   // family.
   // When false, explicit setting of address family is used.
