@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,13 @@
 #include <fcntl.h>
 #include <sys/types.h>
 
+#include "base/bind.h"
 #include "base/eintr_wrapper.h"
 #include "net/base/net_util.h"
 #include "testing/platform_test.h"
 
+namespace net {
+
 const int ListenSocketTester::kTestPort = 9999;
 
 static const int kReadBufSize = 1024;
@@ -19,8 +22,12 @@ static const int kMaxQueueSize = 20;
 static const char kLoopback[] = "127.0.0.1";
 static const int kDefaultTimeoutMs = 5000;
 
-ListenSocket* ListenSocketTester::DoListen() {
-  return ListenSocket::Listen(kLoopback, kTestPort, this);
+ListenSocketTester::ListenSocketTester()
+    : thread_(NULL),
+      loop_(NULL),
+      server_(NULL),
+      connection_(NULL),
+      cv_(&lock_) {
 }
 
 void ListenSocketTester::SetUp() {
@@ -30,8 +37,7 @@ void ListenSocketTester::SetUp() {
   thread_->StartWithOptions(options);
   loop_ = reinterpret_cast<MessageLoopForIO*>(thread_->message_loop());
 
-  loop_->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &ListenSocketTester::Listen));
+  loop_->PostTask(FROM_HERE, base::Bind(&ListenSocketTester::Listen, this));
 
   // verify Listen succeeded
   NextAction();
@@ -63,8 +69,7 @@ void ListenSocketTester::TearDown() {
   NextAction();
   ASSERT_EQ(ACTION_CLOSE, last_action_.type());
 
-  loop_->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &ListenSocketTester::Shutdown));
+  loop_->PostTask(FROM_HERE, base::Bind(&ListenSocketTester::Shutdown, this));
   NextAction();
   ASSERT_EQ(ACTION_SHUTDOWN, last_action_.type());
 
@@ -121,36 +126,6 @@ void ListenSocketTester::SendFromTester() {
   ReportAction(ListenSocketTestAction(ACTION_SEND));
 }
 
-void ListenSocketTester::DidAccept(ListenSocket *server,
-                                   ListenSocket *connection) {
-  connection_ = connection;
-  connection_->AddRef();
-  ReportAction(ListenSocketTestAction(ACTION_ACCEPT));
-}
-
-void ListenSocketTester::DidRead(ListenSocket *connection,
-                                 const char* data,
-                                 int len) {
-  std::string str(data, len);
-  ReportAction(ListenSocketTestAction(ACTION_READ, str));
-}
-
-void ListenSocketTester::DidClose(ListenSocket *sock) {
-  ReportAction(ListenSocketTestAction(ACTION_CLOSE));
-}
-
-bool ListenSocketTester::Send(SOCKET sock, const std::string& str) {
-  int len = static_cast<int>(str.length());
-  int send_len = HANDLE_EINTR(send(sock, str.data(), len, 0));
-  if (send_len == SOCKET_ERROR) {
-    LOG(ERROR) << "send failed: " << errno;
-    return false;
-  } else if (send_len != len) {
-    return false;
-  }
-  return true;
-}
-
 void ListenSocketTester::TestClientSend() {
   ASSERT_TRUE(Send(test_socket_, kHelloWorld));
   NextAction();
@@ -182,8 +157,8 @@ void ListenSocketTester::TestClientSendLong() {
 }
 
 void ListenSocketTester::TestServerSend() {
-  loop_->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &ListenSocketTester::SendFromTester));
+  loop_->PostTask(FROM_HERE, base::Bind(
+      &ListenSocketTester::SendFromTester, this));
   NextAction();
   ASSERT_EQ(ACTION_SEND, last_action_.type());
   const int buf_len = 200;
@@ -200,6 +175,41 @@ void ListenSocketTester::TestServerSend() {
   ASSERT_STREQ(buf, kHelloWorld);
 }
 
+bool ListenSocketTester::Send(SOCKET sock, const std::string& str) {
+  int len = static_cast<int>(str.length());
+  int send_len = HANDLE_EINTR(send(sock, str.data(), len, 0));
+  if (send_len == SOCKET_ERROR) {
+    LOG(ERROR) << "send failed: " << errno;
+    return false;
+  } else if (send_len != len) {
+    return false;
+  }
+  return true;
+}
+
+void ListenSocketTester::DidAccept(ListenSocket *server,
+                                   ListenSocket *connection) {
+  connection_ = connection;
+  connection_->AddRef();
+  ReportAction(ListenSocketTestAction(ACTION_ACCEPT));
+}
+
+void ListenSocketTester::DidRead(ListenSocket *connection,
+                                 const char* data,
+                                 int len) {
+  std::string str(data, len);
+  ReportAction(ListenSocketTestAction(ACTION_READ, str));
+}
+
+void ListenSocketTester::DidClose(ListenSocket *sock) {
+  ReportAction(ListenSocketTestAction(ACTION_CLOSE));
+}
+
+ListenSocketTester::~ListenSocketTester() {}
+
+ListenSocket* ListenSocketTester::DoListen() {
+  return ListenSocket::Listen(kLoopback, kTestPort, this);
+}
 
 class ListenSocketTest: public PlatformTest {
  public:
@@ -233,3 +243,5 @@ TEST_F(ListenSocketTest, ClientSendLong) {
 TEST_F(ListenSocketTest, ServerSend) {
   tester_->TestServerSend();
 }
+
+}  // namespace net
