@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,8 +17,8 @@
 #endif
 
 #include "base/basictypes.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/scoped_ptr.h"
 #include "base/string_util.h"
 #include "base/synchronization/condition_variable.h"
 #include "base/synchronization/lock.h"
@@ -29,12 +29,14 @@
 #include "testing/gtest/include/gtest/gtest.h"
 
 #if defined(OS_POSIX)
-// Used same name as in Windows to avoid #ifdef where refrenced
+// Used same name as in Windows to avoid #ifdef where referenced
 #define SOCKET int
 const int INVALID_SOCKET = -1;
 const int SOCKET_ERROR = -1;
 #endif
 
+namespace net {
+
 enum ActionType {
   ACTION_NONE = 0,
   ACTION_LISTEN = 1,
@@ -63,29 +65,16 @@ class ListenSocketTestAction {
 
 
 // This had to be split out into a separate class because I couldn't
-// make a the testing::Test class refcounted.
+// make the testing::Test class refcounted.
 class ListenSocketTester :
     public ListenSocket::ListenSocketDelegate,
     public base::RefCountedThreadSafe<ListenSocketTester> {
 
- protected:
-  friend class base::RefCountedThreadSafe<ListenSocketTester>;
-
-  virtual ~ListenSocketTester() {}
-
-  virtual ListenSocket* DoListen();
-
  public:
-  ListenSocketTester()
-      : thread_(NULL),
-        loop_(NULL),
-        server_(NULL),
-        connection_(NULL),
-        cv_(&lock_) {
-  }
+  ListenSocketTester();
 
-  virtual void SetUp();
-  virtual void TearDown();
+  void SetUp();
+  void TearDown();
 
   void ReportAction(const ListenSocketTestAction& action);
   void NextAction();
@@ -96,10 +85,6 @@ class ListenSocketTester :
   void Shutdown();
   void Listen();
   void SendFromTester();
-  virtual void DidAccept(ListenSocket *server, ListenSocket *connection);
-  virtual void DidRead(ListenSocket *connection, const char* data, int len);
-  virtual void DidClose(ListenSocket *sock);
-  virtual bool Send(SOCKET sock, const std::string& str);
   // verify the send/read from client to server
   void TestClientSend();
   // verify send/read of a longer string
@@ -107,6 +92,16 @@ class ListenSocketTester :
   // verify a send/read from server to client
   void TestServerSend();
 
+  virtual bool Send(SOCKET sock, const std::string& str);
+
+  // ListenSocket::ListenSocketDelegate:
+  virtual void DidAccept(ListenSocket *server,
+                         ListenSocket *connection) OVERRIDE;
+  virtual void DidRead(ListenSocket *connection,
+                       const char* data,
+                       int len) OVERRIDE;
+  virtual void DidClose(ListenSocket *sock) OVERRIDE;
+
   scoped_ptr<base::Thread> thread_;
   MessageLoopForIO* loop_;
   ListenSocket* server_;
@@ -119,6 +114,15 @@ class ListenSocketTester :
   base::Lock lock_;  // protects |queue_| and wraps |cv_|
   base::ConditionVariable cv_;
   std::deque<ListenSocketTestAction> queue_;
+
+ protected:
+  friend class base::RefCountedThreadSafe<ListenSocketTester>;
+
+  virtual ~ListenSocketTester();
+
+  virtual ListenSocket* DoListen();
 };
 
+}  // namespace net
+
 #endif  // NET_BASE_LISTEN_SOCKET_UNITTEST_H_
