@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,8 @@
 #define NET_BASE_LOAD_STATES_H__
 #pragma once
 
+#include "base/string16.h"
+
 namespace net {
 
 // These states correspond to the lengthy periods of time that a resource load
@@ -13,10 +15,17 @@ namespace net {
 enum LoadState {
   // This is the default state.  It corresponds to a resource load that has
   // either not yet begun or is idle waiting for the consumer to do something
-  // to move things along (e.g., the consumer of an net::URLRequest may not have
+  // to move things along (e.g., the consumer of an URLRequest may not have
   // called Read yet).
   LOAD_STATE_IDLE,
 
+  // This state indicates that the URLRequest delegate has chosen to block this
+  // request before it was sent over the network. When in this state, the
+  // delegate should set a load state parameter on the URLRequest describing
+  // the nature of the delay (i.e. "Waiting for <description given by
+  // delegate>").
+  LOAD_STATE_WAITING_FOR_DELEGATE,
+
   // This state corresponds to a resource load that is blocked waiting for
   // access to a resource in the cache.  If multiple requests are made for the
   // same resource, the first request will be responsible for writing (or
@@ -24,12 +33,21 @@ enum LoadState {
   // the first completes.  This may be done to optimize for cache reuse.
   LOAD_STATE_WAITING_FOR_CACHE,
 
+  // This state corresponds to a resource load that is blocked waiting for
+  // access to a resource in the AppCache.
+  // Note: This is a layering violation, but being the only one it's not that
+  // bad. TODO(rvargas): Reconsider what to do if we need to add more.
+  LOAD_STATE_WAITING_FOR_APPCACHE,
+
   // This state corresponds to a resource load that is blocked waiting for a
-  // proxy autoconfig script to return a proxy server to use.  This state may
-  // take a while if the proxy script needs to resolve the IP address of the
-  // host before deciding what proxy to use.
+  // proxy autoconfig script to return a proxy server to use.
   LOAD_STATE_RESOLVING_PROXY_FOR_URL,
 
+  // This state corresponds to a resource load that is blocked waiting for a
+  // proxy autoconfig script to return a proxy server to use, but that proxy
+  // script is busy resolving the IP address of a host.
+  LOAD_STATE_RESOLVING_HOST_IN_PROXY_SCRIPT,
+
   // This state indicates that we're in the process of establishing a tunnel
   // through the proxy server.
   LOAD_STATE_ESTABLISHING_PROXY_TUNNEL,
@@ -65,10 +83,21 @@ enum LoadState {
   // read to complete.  In the case of a HTTP transaction, this corresponds to
   // the period after the response headers have been received and before all of
   // the response body has been downloaded.  (NOTE: This state only applies for
-  // an net::URLRequest while there is an outstanding Read operation.)
+  // an URLRequest while there is an outstanding Read operation.)
   LOAD_STATE_READING_RESPONSE,
 };
 
+// Some states, like LOAD_STATE_WAITING_FOR_DELEGATE, are associated with extra
+// data that describes more precisely what the delegate (for example) is doing.
+// This class provides an easy way to hold a load state with an extra parameter.
+struct LoadStateWithParam {
+  LoadState state;
+  string16 param;
+  LoadStateWithParam() : state(LOAD_STATE_IDLE) {}
+  LoadStateWithParam(LoadState state, const string16& param)
+      : state(state), param(param) {}
+};
+
 }  // namespace net
 
 #endif  // NET_BASE_LOAD_STATES_H__
