@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -28,6 +28,9 @@ class MimeUtil : public PlatformMimeUtil {
   bool GetMimeTypeFromFile(const FilePath& file_path,
                            std::string* mime_type) const;
 
+  bool GetWellKnownMimeTypeFromExtension(const FilePath::StringType& ext,
+                                         std::string* mime_type) const;
+
   bool IsSupportedImageMimeType(const char* mime_type) const;
   bool IsSupportedMediaMimeType(const char* mime_type) const;
   bool IsSupportedNonImageMimeType(const char* mime_type) const;
@@ -59,6 +62,10 @@ class MimeUtil : public PlatformMimeUtil {
   // For faster lookup, keep hash sets.
   void InitializeMimeTypeMaps();
 
+  bool GetMimeTypeFromExtensionHelper(
+      const FilePath::StringType& ext, bool include_platform_types,
+      std::string* mime_type) const;
+
   typedef base::hash_set<std::string> MimeMappings;
   MimeMappings image_map_;
   MimeMappings media_map_;
@@ -71,7 +78,7 @@ class MimeUtil : public PlatformMimeUtil {
   StrictMappings strict_format_map_;
 };  // class MimeUtil
 
-static base::LazyInstance<MimeUtil> g_mime_util(base::LINKER_INITIALIZED);
+static base::LazyInstance<MimeUtil> g_mime_util = LAZY_INSTANCE_INITIALIZER;
 
 struct MimeInfo {
   const char* mime_type;
@@ -95,7 +102,8 @@ static const MimeInfo primary_mappings[] = {
   { "audio/webm", "webm" },
   { "audio/wav", "wav" },
   { "application/xhtml+xml", "xhtml,xht" },
-  { "application/x-chrome-extension", "crx" }
+  { "application/x-chrome-extension", "crx" },
+  { "multipart/related", "mhtml,mht" }
 };
 
 static const MimeInfo secondary_mappings[] = {
@@ -104,6 +112,7 @@ static const MimeInfo secondary_mappings[] = {
   { "application/pdf", "pdf" },
   { "application/postscript", "ps,eps,ai" },
   { "application/x-javascript", "js" },
+  { "application/x-woff", "woff" },
   { "image/bmp", "bmp" },
   { "image/x-icon", "ico" },
   { "image/jpeg", "jfif,pjpeg,pjp" },
@@ -117,7 +126,9 @@ static const MimeInfo secondary_mappings[] = {
   { "application/rdf+xml", "rdf" },
   { "text/xml", "xsl,xbl" },
   { "application/vnd.mozilla.xul+xml", "xul" },
-  { "application/x-shockwave-flash", "swf,swl" }
+  { "application/x-shockwave-flash", "swf,swl" },
+  { "application/pkcs7-mime", "p7m,p7c,p7z" },
+  { "application/pkcs7-signature", "p7s" }
 };
 
 static const char* FindMimeType(const MimeInfo* mappings,
@@ -143,6 +154,25 @@ static const char* FindMimeType(const MimeInfo* mappings,
 
 bool MimeUtil::GetMimeTypeFromExtension(const FilePath::StringType& ext,
                                         string* result) const {
+  return GetMimeTypeFromExtensionHelper(ext, true, result);
+}
+
+bool MimeUtil::GetWellKnownMimeTypeFromExtension(
+    const FilePath::StringType& ext, string* result) const {
+  return GetMimeTypeFromExtensionHelper(ext, false, result);
+}
+
+bool MimeUtil::GetMimeTypeFromFile(const FilePath& file_path,
+                                   string* result) const {
+  FilePath::StringType file_name_str = file_path.Extension();
+  if (file_name_str.empty())
+    return false;
+  return GetMimeTypeFromExtension(file_name_str.substr(1), result);
+}
+
+bool MimeUtil::GetMimeTypeFromExtensionHelper(
+    const FilePath::StringType& ext, bool include_platform_types,
+    string* result) const {
   // Avoids crash when unable to handle a long file path. See crbug.com/48733.
   const unsigned kMaxFilePathSize = 65536;
   if (ext.length() > kMaxFilePathSize)
@@ -168,7 +198,7 @@ bool MimeUtil::GetMimeTypeFromExtension(const FilePath::StringType& ext,
     return true;
   }
 
-  if (GetPlatformMimeTypeFromExtension(ext, result))
+  if (include_platform_types && GetPlatformMimeTypeFromExtension(ext, result))
     return true;
 
   mime_type = FindMimeType(secondary_mappings, arraysize(secondary_mappings),
@@ -181,14 +211,6 @@ bool MimeUtil::GetMimeTypeFromExtension(const FilePath::StringType& ext,
   return false;
 }
 
-bool MimeUtil::GetMimeTypeFromFile(const FilePath& file_path,
-                                   string* result) const {
-  FilePath::StringType file_name_str = file_path.Extension();
-  if (file_name_str.empty())
-    return false;
-  return GetMimeTypeFromExtension(file_name_str.substr(1), result);
-}
-
 // From WebKit's WebCore/platform/MIMETypeRegistry.cpp:
 
 static const char* const supported_image_types[] = {
@@ -206,10 +228,14 @@ static const char* const supported_image_types[] = {
 // A list of media types: http://en.wikipedia.org/wiki/Internet_media_type
 // A comprehensive mime type list: http://plugindoc.mozdev.org/winmime.php
 static const char* const supported_media_types[] = {
+#if defined(ENABLE_MEDIA_TYPE_OGG)
   // Ogg.
   "video/ogg",
   "audio/ogg",
   "application/ogg",
+#endif
+
+  // WebM.
   "video/webm",
   "audio/webm",
   "audio/wav",
@@ -233,15 +259,20 @@ static const char* const supported_media_types[] = {
 //
 // Refer to http://wiki.whatwg.org/wiki/Video_type_parameters#Browser_Support
 // for more information.
+//
+// The codecs for WAV are integers as defined in Appendix A of RFC2361:
+// http://tools.ietf.org/html/rfc2361
 static const char* const supported_media_codecs[] = {
 #if defined(GOOGLE_CHROME_BUILD) || defined(USE_PROPRIETARY_CODECS)
   "avc1",
   "mp4a",
 #endif
+#if defined(ENABLE_MEDIA_CODEC_THEORA)
   "theora",
+#endif
   "vorbis",
   "vp8",
-  "1"  // PCM for WAV.
+  "1"  // WAVE_FORMAT_PCM.
 };
 
 // Note: does not include javascript types list (see supported_javascript_types)
@@ -266,12 +297,11 @@ static const char* const supported_non_image_types[] = {
   "application/atom+xml",
   "application/json",
   "application/x-x509-user-cert",
+  "multipart/related",  // For MHTML support.
   "multipart/x-mixed-replace"
   // Note: ADDING a new type here will probably render it AS HTML. This can
   // result in cross site scripting.
 };
-COMPILE_ASSERT(arraysize(supported_non_image_types) == 16,
-               supported_non_images_types_must_equal_16);
 
 //  Mozilla 1.8 and WinIE 7 both accept text/javascript and text/ecmascript.
 //  Mozilla 1.8 accepts application/javascript, application/ecmascript, and
@@ -385,8 +415,8 @@ bool MimeUtil::IsSupportedMimeType(const std::string& mime_type) const {
 bool MimeUtil::MatchesMimeType(const std::string &mime_type_pattern,
                                const std::string &mime_type) const {
   // verify caller is passing lowercase
-  DCHECK(mime_type_pattern == StringToLowerASCII(mime_type_pattern));
-  DCHECK(mime_type == StringToLowerASCII(mime_type));
+  DCHECK_EQ(StringToLowerASCII(mime_type_pattern), mime_type_pattern);
+  DCHECK_EQ(StringToLowerASCII(mime_type), mime_type);
 
   // This comparison handles absolute maching and also basic
   // wildcards.  The plugin mime types could be:
@@ -484,6 +514,11 @@ bool GetMimeTypeFromFile(const FilePath& file_path, std::string* mime_type) {
   return g_mime_util.Get().GetMimeTypeFromFile(file_path, mime_type);
 }
 
+bool GetWellKnownMimeTypeFromExtension(const FilePath::StringType& ext,
+                                       std::string* mime_type) {
+  return g_mime_util.Get().GetWellKnownMimeTypeFromExtension(ext, mime_type);
+}
+
 bool GetPreferredExtensionForMimeType(const std::string& mime_type,
                                       FilePath::StringType* extension) {
   return g_mime_util.Get().GetPreferredExtensionForMimeType(mime_type,
@@ -633,7 +668,7 @@ void GetExtensionsHelper(
     base::hash_set<FilePath::StringType>* extensions) {
   FilePath::StringType extension;
   for (size_t i = 0; i < standard_types_len; ++i) {
-    if (net::GetPreferredExtensionForMimeType(standard_types[i], &extension))
+    if (GetPreferredExtensionForMimeType(standard_types[i], &extension))
       extensions->insert(extension);
   }
 
@@ -695,7 +730,7 @@ void GetExtensionsForMimeType(const std::string& mime_type,
                               std::vector<FilePath::StringType>* extensions) {
   base::hash_set<FilePath::StringType> unique_extensions;
   FilePath::StringType extension;
-  if (net::GetPreferredExtensionForMimeType(mime_type, &extension))
+  if (GetPreferredExtensionForMimeType(mime_type, &extension))
     unique_extensions.insert(extension);
 
   // Also look up the extensions from hard-coded mappings in case that some
