@@ -1,20 +1,32 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/base/mock_host_resolver.h"
 
+#include "base/bind.h"
+#include "base/location.h"
+#include "base/memory/ref_counted.h"
+#include "base/message_loop.h"
+#include "base/stl_util.h"
+#include "base/string_split.h"
 #include "base/string_util.h"
-#include "base/ref_counted.h"
 #include "base/threading/platform_thread.h"
+#include "net/base/host_cache.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_util.h"
 #include "net/base/sys_addrinfo.h"
+#include "net/base/test_completion_callback.h"
 
 namespace net {
 
 namespace {
 
+// Cache size for the MockCachingHostResolver.
+const unsigned kMaxCacheEntries = 100;
+// TTL for the successful resolutions. Failures are not cached.
+const unsigned kCacheEntryTTLSeconds = 60;
+
 char* do_strdup(const char* src) {
 #if defined(OS_WIN)
   return _strdup(src);
@@ -23,88 +35,167 @@ char* do_strdup(const char* src) {
 #endif
 }
 
-// Fills |*addrlist| with a socket address for |host| which should be an
-// IPv4 or IPv6 literal without enclosing brackets. If |canonical_name| is
-// non-empty it is used as the DNS canonical name for the host. Returns OK on
-// success, ERR_UNEXPECTED otherwise.
-int CreateIPAddress(const std::string& host,
-                    const std::string& canonical_name,
-                    AddressList* addrlist) {
-  IPAddressNumber ip_number;
-  if (!ParseIPLiteralToNumber(host, &ip_number)) {
-    LOG(WARNING) << "Not a supported IP literal: " << host;
-    return ERR_UNEXPECTED;
-  }
+}  // namespace
 
-  AddressList result(ip_number, -1, false);
-  struct addrinfo* ai = const_cast<struct addrinfo*>(result.head());
-  ai->ai_canonname = do_strdup(canonical_name.c_str());
-  *addrlist = result;
+int ParseAddressList(const std::string& host_list,
+                     const std::string& canonical_name,
+                     AddressList* addrlist) {
+  *addrlist = AddressList();
+  std::vector<std::string> addresses;
+  base::SplitString(host_list, ',', &addresses);
+  for (size_t index = 0; index < addresses.size(); ++index) {
+    IPAddressNumber ip_number;
+    if (!ParseIPLiteralToNumber(addresses[index], &ip_number)) {
+      LOG(WARNING) << "Not a supported IP literal: " << addresses[index];
+      return ERR_UNEXPECTED;
+    }
+
+    AddressList result = AddressList::CreateFromIPAddress(ip_number, -1);
+    struct addrinfo* ai = const_cast<struct addrinfo*>(result.head());
+    if (index == 0)
+      ai->ai_canonname = do_strdup(canonical_name.c_str());
+    if (!addrlist->head())
+      *addrlist = AddressList::CreateByCopyingFirstAddress(result.head());
+    else
+      addrlist->Append(result.head());
+  }
   return OK;
 }
 
-}  // namespace
+struct MockHostResolverBase::Request {
+  Request(const RequestInfo& req_info,
+          AddressList* addr,
+          const CompletionCallback& cb)
+      : info(req_info), addresses(addr), callback(cb) {}
+  RequestInfo info;
+  AddressList* addresses;
+  CompletionCallback callback;
+};
 
-MockHostResolverBase::~MockHostResolverBase() {}
-
-void MockHostResolverBase::Reset(HostResolverProc* interceptor) {
-  synchronous_mode_ = false;
-
-  // At the root of the chain, map everything to localhost.
-  scoped_refptr<RuleBasedHostResolverProc> catchall(
-      new RuleBasedHostResolverProc(NULL));
-  catchall->AddRule("*", "127.0.0.1");
-
-  // Next add a rules-based layer the use controls.
-  rules_ = new RuleBasedHostResolverProc(catchall);
-
-  HostResolverProc* proc = rules_;
-
-  // Lastly add the provided interceptor to the front of the chain.
-  if (interceptor) {
-    interceptor->SetPreviousProc(proc);
-    proc = interceptor;
-  }
-
-  HostCache* cache = NULL;
-
-  if (use_caching_) {
-    cache = new HostCache(
-        100,  // max entries.
-        base::TimeDelta::FromMinutes(1),
-        base::TimeDelta::FromSeconds(0));
-  }
-
-  impl_.reset(new HostResolverImpl(proc, cache, 50u, NULL));
+MockHostResolverBase::~MockHostResolverBase() {
+  STLDeleteValues(&requests_);
 }
 
 int MockHostResolverBase::Resolve(const RequestInfo& info,
                                   AddressList* addresses,
-                                  CompletionCallback* callback,
-                                  RequestHandle* out_req,
+                                  const CompletionCallback& callback,
+                                  RequestHandle* handle,
                                   const BoundNetLog& net_log) {
-  if (synchronous_mode_) {
-    callback = NULL;
-    out_req = NULL;
+  DCHECK(CalledOnValidThread());
+  size_t id = next_request_id_++;
+  int rv = ResolveFromIPLiteralOrCache(info, addresses);
+  if (rv != ERR_DNS_CACHE_MISS) {
+    return rv;
   }
-  return impl_->Resolve(info, addresses, callback, out_req, net_log);
+  if (synchronous_mode_) {
+    return ResolveProc(id, info, addresses);
+  }
+  // Store the request for asynchronous resolution
+  Request* req = new Request(info, addresses, callback);
+  requests_[id] = req;
+  if (handle)
+    *handle = reinterpret_cast<RequestHandle>(id);
+  MessageLoop::current()->PostTask(FROM_HERE,
+                                   base::Bind(&MockHostResolverBase::ResolveNow,
+                                              AsWeakPtr(),
+                                              id));
+  return ERR_IO_PENDING;
 }
 
-void MockHostResolverBase::CancelRequest(RequestHandle req) {
-  impl_->CancelRequest(req);
+int MockHostResolverBase::ResolveFromCache(const RequestInfo& info,
+                                           AddressList* addresses,
+                                           const BoundNetLog& net_log) {
+  DCHECK(CalledOnValidThread());
+  next_request_id_++;
+  int rv = ResolveFromIPLiteralOrCache(info, addresses);
+  return rv;
 }
 
-void MockHostResolverBase::AddObserver(Observer* observer) {
-  impl_->AddObserver(observer);
+void MockHostResolverBase::CancelRequest(RequestHandle handle) {
+  DCHECK(CalledOnValidThread());
+  size_t id = reinterpret_cast<size_t>(handle);
+  RequestMap::iterator it = requests_.find(id);
+  if (it != requests_.end()) {
+    scoped_ptr<Request> req(it->second);
+    requests_.erase(it);
+  } else {
+    NOTREACHED() << "CancelRequest must NOT be called after request is "
+        "complete or canceled.";
+  }
 }
 
-void MockHostResolverBase::RemoveObserver(Observer* observer) {
-  impl_->RemoveObserver(observer);
+HostCache* MockHostResolverBase::GetHostCache() {
+  return cache_.get();
 }
 
+// start id from 1 to distinguish from NULL RequestHandle
 MockHostResolverBase::MockHostResolverBase(bool use_caching)
-    : use_caching_(use_caching) {
-  Reset(NULL);
+    : synchronous_mode_(false), next_request_id_(1) {
+  rules_ = CreateCatchAllHostResolverProc();
+  proc_ = rules_;
+
+  if (use_caching) {
+    cache_.reset(new HostCache(kMaxCacheEntries));
+  }
+}
+
+int MockHostResolverBase::ResolveFromIPLiteralOrCache(const RequestInfo& info,
+                                                      AddressList* addresses) {
+  IPAddressNumber ip;
+  if (ParseIPLiteralToNumber(info.hostname(), &ip)) {
+    *addresses = AddressList::CreateFromIPAddressWithCname(
+        ip, info.port(), info.host_resolver_flags() & HOST_RESOLVER_CANONNAME);
+    return OK;
+  }
+  int rv = ERR_DNS_CACHE_MISS;
+  if (cache_.get() && info.allow_cached_response()) {
+    HostCache::Key key(info.hostname(),
+                       info.address_family(),
+                       info.host_resolver_flags());
+    const HostCache::Entry* entry = cache_->Lookup(key, base::TimeTicks::Now());
+    if (entry) {
+      rv = entry->error;
+      if (rv == OK)
+        *addresses = CreateAddressListUsingPort(entry->addrlist, info.port());
+    }
+  }
+  return rv;
+}
+
+int MockHostResolverBase::ResolveProc(size_t id,
+                                      const RequestInfo& info,
+                                      AddressList* addresses) {
+  AddressList addr;
+  int rv = proc_->Resolve(info.hostname(),
+                          info.address_family(),
+                          info.host_resolver_flags(),
+                          &addr,
+                          NULL);
+  if (cache_.get()) {
+    HostCache::Key key(info.hostname(),
+                       info.address_family(),
+                       info.host_resolver_flags());
+    // Storing a failure with TTL 0 so that it overwrites previous value.
+    base::TimeDelta ttl;
+    if (rv == OK)
+      ttl = base::TimeDelta::FromSeconds(kCacheEntryTTLSeconds);
+    cache_->Set(key, rv, addr, base::TimeTicks::Now(), ttl);
+  }
+  if (rv == OK)
+    *addresses = CreateAddressListUsingPort(addr, info.port());
+  return rv;
+}
+
+void MockHostResolverBase::ResolveNow(size_t id) {
+  RequestMap::iterator it = requests_.find(id);
+  if (it == requests_.end())
+    return;  // was canceled
+
+  scoped_ptr<Request> req(it->second);
+  requests_.erase(it);
+  int rv = ResolveProc(id, req->info, req->addresses);
+  if (!req->callback.is_null())
+    req->callback.Run(rv);
 }
 
 //-----------------------------------------------------------------------------
@@ -175,7 +266,8 @@ void RuleBasedHostResolverProc::AddIPLiteralRule(
   if (!canonical_name.empty())
     flags |= HOST_RESOLVER_CANONNAME;
   Rule rule(Rule::kResolverTypeIPLiteral, host_pattern,
-            ADDRESS_FAMILY_UNSPECIFIED, flags, ip_literal, canonical_name, 0);
+            ADDRESS_FAMILY_UNSPECIFIED, flags, ip_literal, canonical_name,
+            0);
   rules_.push_back(rule);
 }
 
@@ -227,8 +319,10 @@ int RuleBasedHostResolverProc::Resolve(const std::string& host,
         host_resolver_flags;
     if (matches_flags && matches_address_family &&
         MatchPattern(host, r->host_pattern)) {
-      if (r->latency_ms != 0)
-        base::PlatformThread::Sleep(r->latency_ms);
+      if (r->latency_ms != 0) {
+        base::PlatformThread::Sleep(
+            base::TimeDelta::FromMilliseconds(r->latency_ms));
+      }
 
       // Remap to a new host.
       const std::string& effective_host =
@@ -244,7 +338,9 @@ int RuleBasedHostResolverProc::Resolve(const std::string& host,
                                         host_resolver_flags,
                                         addrlist, os_error);
         case Rule::kResolverTypeIPLiteral:
-          return CreateIPAddress(effective_host, r->canonical_name, addrlist);
+          return ParseAddressList(effective_host,
+                                  r->canonical_name,
+                                  addrlist);
         default:
           NOTREACHED();
           return ERR_UNEXPECTED;
@@ -258,27 +354,36 @@ int RuleBasedHostResolverProc::Resolve(const std::string& host,
 RuleBasedHostResolverProc::~RuleBasedHostResolverProc() {
 }
 
+RuleBasedHostResolverProc* CreateCatchAllHostResolverProc() {
+  RuleBasedHostResolverProc* catchall = new RuleBasedHostResolverProc(NULL);
+#if defined(OS_ANDROID)
+  // In Android emulator, the development machine's '127.0.0.1' is mapped to
+  // '10.0.2.2'.
+  catchall->AddIPLiteralRule("*", "10.0.2.2", "localhost");
+#else
+  catchall->AddIPLiteralRule("*", "127.0.0.1", "localhost");
+#endif
+
+  // Next add a rules-based layer the use controls.
+  return new RuleBasedHostResolverProc(catchall);
+}
+
 //-----------------------------------------------------------------------------
 
-WaitingHostResolverProc::WaitingHostResolverProc(HostResolverProc* previous)
-    : HostResolverProc(previous), event_(false, false) {}
-
-void WaitingHostResolverProc::Signal() {
-  event_.Signal();
+int HangingHostResolver::Resolve(const RequestInfo& info,
+                                 AddressList* addresses,
+                                 const CompletionCallback& callback,
+                                 RequestHandle* out_req,
+                                 const BoundNetLog& net_log) {
+  return ERR_IO_PENDING;
 }
 
-int WaitingHostResolverProc::Resolve(const std::string& host,
-                                     AddressFamily address_family,
-                                     HostResolverFlags host_resolver_flags,
-                                     AddressList* addrlist,
-                                     int* os_error) {
-  event_.Wait();
-  return ResolveUsingPrevious(host, address_family, host_resolver_flags,
-                              addrlist, os_error);
+int HangingHostResolver::ResolveFromCache(const RequestInfo& info,
+                                          AddressList* addresses,
+                                          const BoundNetLog& net_log) {
+  return ERR_DNS_CACHE_MISS;
 }
 
-WaitingHostResolverProc::~WaitingHostResolverProc() {}
-
 //-----------------------------------------------------------------------------
 
 ScopedDefaultHostResolverProc::ScopedDefaultHostResolverProc() {}
