@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,6 +13,42 @@
 
 namespace net {
 
+namespace {
+
+// Parameters for logging data transferred events. Includes bytes transferred
+// and, if |bytes| is not NULL, the bytes themselves.
+class NetLogBytesTransferredParameter : public NetLog::EventParameters {
+ public:
+  NetLogBytesTransferredParameter(int byte_count, const char* bytes);
+
+  virtual Value* ToValue() const;
+
+ private:
+  const int byte_count_;
+  std::string hex_encoded_bytes_;
+  bool has_bytes_;
+};
+
+NetLogBytesTransferredParameter::NetLogBytesTransferredParameter(
+    int byte_count, const char* transferred_bytes)
+    : byte_count_(byte_count),
+      has_bytes_(false) {
+  if (transferred_bytes) {
+    hex_encoded_bytes_ = base::HexEncode(transferred_bytes, byte_count);
+    has_bytes_ = true;
+  }
+}
+
+Value* NetLogBytesTransferredParameter::ToValue() const {
+  DictionaryValue* dict = new DictionaryValue();
+  dict->SetInteger("byte_count", byte_count_);
+  if (has_bytes_ && byte_count_ > 0)
+    dict->SetString("hex_encoded_bytes", hex_encoded_bytes_);
+  return dict;
+}
+
+}  // namespace
+
 Value* NetLog::Source::ToValue() const {
   DictionaryValue* dict = new DictionaryValue();
   dict->SetInteger("type", static_cast<int>(type));
@@ -20,6 +56,17 @@ Value* NetLog::Source::ToValue() const {
   return dict;
 }
 
+NetLog::ThreadSafeObserver::ThreadSafeObserver(LogLevel log_level)
+    : log_level_(log_level) {
+}
+
+NetLog::ThreadSafeObserver::~ThreadSafeObserver() {
+}
+
+NetLog::LogLevel NetLog::ThreadSafeObserver::log_level() const {
+  return log_level_;
+}
+
 // static
 std::string NetLog::TickCountToString(const base::TimeTicks& time) {
   int64 delta_time = (time - base::TimeTicks()).InMilliseconds();
@@ -71,11 +118,11 @@ const char* NetLog::EventPhaseToString(EventPhase phase) {
 }
 
 // static
-Value* NetLog::EntryToDictionaryValue(net::NetLog::EventType type,
+Value* NetLog::EntryToDictionaryValue(NetLog::EventType type,
                                       const base::TimeTicks& time,
-                                      const net::NetLog::Source& source,
-                                      net::NetLog::EventPhase phase,
-                                      net::NetLog::EventParameters* params,
+                                      const NetLog::Source& source,
+                                      NetLog::EventPhase phase,
+                                      NetLog::EventParameters* params,
                                       bool use_strings) {
   DictionaryValue* entry_dict = new DictionaryValue();
 
@@ -88,7 +135,7 @@ Value* NetLog::EntryToDictionaryValue(net::NetLog::EventType type,
     source_dict->SetInteger("type", static_cast<int>(source.type));
   } else {
     source_dict->SetString("type",
-                           net::NetLog::SourceTypeToString(source.type));
+                           NetLog::SourceTypeToString(source.type));
   }
   entry_dict->Set("source", source_dict);
 
@@ -97,8 +144,8 @@ Value* NetLog::EntryToDictionaryValue(net::NetLog::EventType type,
     entry_dict->SetInteger("type", static_cast<int>(type));
     entry_dict->SetInteger("phase", static_cast<int>(phase));
   } else {
-    entry_dict->SetString("type", net::NetLog::EventTypeToString(type));
-    entry_dict->SetString("phase", net::NetLog::EventPhaseToString(phase));
+    entry_dict->SetString("type", NetLog::EventTypeToString(type));
+    entry_dict->SetString("phase", NetLog::EventPhaseToString(phase));
   }
 
   // Set the event-specific parameters.
@@ -145,9 +192,18 @@ void BoundNetLog::EndEvent(
   AddEntry(event_type, NetLog::PHASE_END, params);
 }
 
+void BoundNetLog::AddEventWithNetErrorCode(NetLog::EventType event_type,
+                                           int net_error) const {
+  DCHECK_GT(0, net_error);
+  DCHECK_NE(ERR_IO_PENDING, net_error);
+  AddEvent(
+      event_type,
+      make_scoped_refptr(new NetLogIntegerParameter("net_error", net_error)));
+}
+
 void BoundNetLog::EndEventWithNetErrorCode(NetLog::EventType event_type,
                                            int net_error) const {
-  DCHECK_NE(net_error, net::ERR_IO_PENDING);
+  DCHECK_NE(ERR_IO_PENDING, net_error);
   if (net_error >= 0) {
     EndEvent(event_type, NULL);
   } else {
@@ -157,6 +213,18 @@ void BoundNetLog::EndEventWithNetErrorCode(NetLog::EventType event_type,
   }
 }
 
+void BoundNetLog::AddByteTransferEvent(NetLog::EventType event_type,
+                                       int byte_count,
+                                       const char* bytes) const {
+  scoped_refptr<NetLog::EventParameters> params;
+  if (IsLoggingBytes()) {
+    params = new NetLogBytesTransferredParameter(byte_count, bytes);
+  } else {
+    params = new NetLogBytesTransferredParameter(byte_count, NULL);
+  }
+  AddEvent(event_type, params);
+}
+
 NetLog::LogLevel BoundNetLog::GetLogLevel() const {
   if (net_log_)
     return net_log_->GetLogLevel();
@@ -203,8 +271,8 @@ Value* NetLogStringParameter::ToValue() const {
 
 Value* NetLogSourceParameter::ToValue() const {
   DictionaryValue* dict = new DictionaryValue();
-
-  dict->Set(name_, value_.ToValue());
+  if (value_.is_valid())
+    dict->Set(name_, value_.ToValue());
   return dict;
 }
 
