@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,22 +7,33 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/compiler_specific.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/net_export.h"
 #include "net/base/network_change_notifier.h"
 
+namespace dbus {
+class Bus;
+}
+
 namespace net {
 
-class NetworkChangeNotifierLinux : public NetworkChangeNotifier {
+class NET_EXPORT_PRIVATE NetworkChangeNotifierLinux
+    : public NetworkChangeNotifier {
  public:
-  NetworkChangeNotifierLinux();
+  static NetworkChangeNotifierLinux* Create();
+
+  // Unittests inject a mock bus.
+  static NetworkChangeNotifierLinux* CreateForTest(dbus::Bus* bus);
 
  private:
   class Thread;
 
+  explicit NetworkChangeNotifierLinux(dbus::Bus* bus);
   virtual ~NetworkChangeNotifierLinux();
 
   // NetworkChangeNotifier:
-  virtual bool IsCurrentlyOffline() const;
+  virtual bool IsCurrentlyOffline() const OVERRIDE;
 
   // The thread used to listen for notifications.  This relays the notification
   // to the registered observers without posting back to the thread the object
