@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,14 +7,13 @@
 #include "base/base64.h"
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
-#include "base/sha2.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
+#include "crypto/sha2.h"
 #include "net/base/registry_controlled_domain.h"
 #include "net/url_request/url_request_http_job.h"
 
-using base::Time;
-using base::TimeDelta;
+namespace net {
 
 //------------------------------------------------------------------------------
 // static
@@ -24,20 +23,27 @@ const size_t SdchManager::kMaxDictionarySize = 1000000;
 const size_t SdchManager::kMaxDictionaryCount = 20;
 
 // static
-SdchManager* SdchManager::global_;
+SdchManager* SdchManager::global_ = NULL;
+
+// static
+bool SdchManager::g_sdch_enabled_ = true;
 
 //------------------------------------------------------------------------------
 SdchManager::Dictionary::Dictionary(const std::string& dictionary_text,
-    size_t offset, const std::string& client_hash, const GURL& gurl,
-    const std::string& domain, const std::string& path, const Time& expiration,
-    const std::set<int> ports)
-      : text_(dictionary_text, offset),
-        client_hash_(client_hash),
-        url_(gurl),
-        domain_(domain),
-        path_(path),
-        expiration_(expiration),
-        ports_(ports) {
+                                    size_t offset,
+                                    const std::string& client_hash,
+                                    const GURL& gurl,
+                                    const std::string& domain,
+                                    const std::string& path,
+                                    const base::Time& expiration,
+                                    const std::set<int>& ports)
+    : text_(dictionary_text, offset),
+      client_hash_(client_hash),
+      url_(gurl),
+      domain_(domain),
+      path_(path),
+      expiration_(expiration),
+      ports_(ports) {
 }
 
 SdchManager::Dictionary::~Dictionary() {
@@ -66,7 +72,7 @@ bool SdchManager::Dictionary::CanAdvertise(const GURL& target_url) {
     return false;
   if (target_url.SchemeIsSecure())
     return false;
-  if (Time::Now() > expiration_)
+  if (base::Time::Now() > expiration_)
     return false;
   return true;
 }
@@ -77,7 +83,7 @@ bool SdchManager::Dictionary::CanAdvertise(const GURL& target_url) {
 // static
 bool SdchManager::Dictionary::CanSet(const std::string& domain,
                                      const std::string& path,
-                                     const std::set<int> ports,
+                                     const std::set<int>& ports,
                                      const GURL& dictionary_url) {
   if (!SdchManager::Global()->IsInSupportedDomain(dictionary_url))
     return false;
@@ -103,8 +109,7 @@ bool SdchManager::Dictionary::CanSet(const std::string& domain,
     SdchErrorRecovery(DICTIONARY_MISSING_DOMAIN_SPECIFIER);
     return false;  // Domain is required.
   }
-  if (net::RegistryControlledDomainService::GetDomainAndRegistry(domain).size()
-      == 0) {
+  if (RegistryControlledDomainService::GetDomainAndRegistry(domain).empty()) {
     SdchErrorRecovery(DICTIONARY_SPECIFIES_TOP_LEVEL_DOMAIN);
     return false;  // domain was a TLD.
   }
@@ -199,13 +204,15 @@ bool SdchManager::Dictionary::DomainMatch(const GURL& gurl,
 }
 
 //------------------------------------------------------------------------------
-SdchManager::SdchManager() : sdch_enabled_(false) {
+SdchManager::SdchManager() {
   DCHECK(!global_);
+  DCHECK(CalledOnValidThread());
   global_ = this;
 }
 
 SdchManager::~SdchManager() {
-  DCHECK(global_ == this);
+  DCHECK_EQ(this, global_);
+  DCHECK(CalledOnValidThread());
   while (!dictionaries_.empty()) {
     DictionaryMap::iterator it = dictionaries_.begin();
     it->second->Release();
@@ -216,9 +223,10 @@ SdchManager::~SdchManager() {
 
 // static
 void SdchManager::Shutdown() {
+  EnableSdchSupport(false);
   if (!global_ )
     return;
-  global_->fetcher_.reset(NULL);
+  global_->set_sdch_fetcher(NULL);
 }
 
 // static
@@ -231,10 +239,14 @@ void SdchManager::SdchErrorRecovery(ProblemCodes problem) {
   UMA_HISTOGRAM_ENUMERATION("Sdch3.ProblemCodes_4", problem, MAX_PROBLEM_CODE);
 }
 
-void SdchManager::EnableSdchSupport(const std::string& domain) {
-  // We presume that there is a SDCH manager instance.
-  global_->supported_domain_ = domain;
-  global_->sdch_enabled_ = true;
+void SdchManager::set_sdch_fetcher(SdchFetcher* fetcher) {
+  DCHECK(CalledOnValidThread());
+  fetcher_.reset(fetcher);
+}
+
+// static
+void SdchManager::EnableSdchSupport(bool enabled) {
+  g_sdch_enabled_ = enabled;
 }
 
 // static
@@ -296,11 +308,9 @@ int SdchManager::BlacklistDomainExponential(const std::string& domain) {
 }
 
 bool SdchManager::IsInSupportedDomain(const GURL& url) {
-  if (!sdch_enabled_ )
+  DCHECK(CalledOnValidThread());
+  if (!g_sdch_enabled_ )
     return false;
-  if (!supported_domain_.empty() &&
-      !url.DomainIs(supported_domain_.data(), supported_domain_.size()))
-     return false;  // It is not the singular supported domain.
 
   if (blacklisted_domains_.empty())
     return true;
@@ -321,6 +331,7 @@ bool SdchManager::IsInSupportedDomain(const GURL& url) {
 
 void SdchManager::FetchDictionary(const GURL& request_url,
                                   const GURL& dictionary_url) {
+  DCHECK(CalledOnValidThread());
   if (SdchManager::Global()->CanFetchDictionary(request_url, dictionary_url) &&
       fetcher_.get())
     fetcher_->Schedule(dictionary_url);
@@ -328,6 +339,7 @@ void SdchManager::FetchDictionary(const GURL& request_url,
 
 bool SdchManager::CanFetchDictionary(const GURL& referring_url,
                                      const GURL& dictionary_url) const {
+  DCHECK(CalledOnValidThread());
   /* The user agent may retrieve a dictionary from the dictionary URL if all of
      the following are true:
        1 The dictionary URL host name matches the referrer URL host name
@@ -360,6 +372,7 @@ bool SdchManager::CanFetchDictionary(const GURL& referring_url,
 
 bool SdchManager::AddSdchDictionary(const std::string& dictionary_text,
     const GURL& dictionary_url) {
+  DCHECK(CalledOnValidThread());
   std::string client_hash;
   std::string server_hash;
   GenerateHash(dictionary_text, &client_hash, &server_hash);
@@ -370,7 +383,7 @@ bool SdchManager::AddSdchDictionary(const std::string& dictionary_text,
 
   std::string domain, path;
   std::set<int> ports;
-  Time expiration(Time::Now() + TimeDelta::FromDays(30));
+  base::Time expiration(base::Time::Now() + base::TimeDelta::FromDays(30));
 
   if (dictionary_text.empty()) {
     SdchErrorRecovery(DICTIONARY_HAS_NO_TEXT);
@@ -386,7 +399,7 @@ bool SdchManager::AddSdchDictionary(const std::string& dictionary_text,
   while (1) {
     size_t line_end = dictionary_text.find('\n', line_start);
     DCHECK(std::string::npos != line_end);
-    DCHECK(line_end <= header_end);
+    DCHECK_LE(line_end, header_end);
 
     size_t colon_index = dictionary_text.find(':', line_start);
     if (std::string::npos == colon_index) {
@@ -415,7 +428,7 @@ bool SdchManager::AddSdchDictionary(const std::string& dictionary_text,
       } else if (name == "max-age") {
         int64 seconds;
         base::StringToInt64(value, &seconds);
-        expiration = Time::Now() + TimeDelta::FromSeconds(seconds);
+        expiration = base::Time::Now() + base::TimeDelta::FromSeconds(seconds);
       } else if (name == "port") {
         int port;
         base::StringToInt(value, &port);
@@ -458,6 +471,7 @@ bool SdchManager::AddSdchDictionary(const std::string& dictionary_text,
 
 void SdchManager::GetVcdiffDictionary(const std::string& server_hash,
     const GURL& referring_url, Dictionary** dictionary) {
+  DCHECK(CalledOnValidThread());
   *dictionary = NULL;
   DictionaryMap::iterator it = dictionaries_.find(server_hash);
   if (it == dictionaries_.end()) {
@@ -474,6 +488,7 @@ void SdchManager::GetVcdiffDictionary(const std::string& server_hash,
 // instances that can be used if/when a server specifies one.
 void SdchManager::GetAvailDictionaryList(const GURL& target_url,
                                          std::string* list) {
+  DCHECK(CalledOnValidThread());
   int count = 0;
   for (DictionaryMap::iterator it = dictionaries_.begin();
        it != dictionaries_.end(); ++it) {
@@ -493,7 +508,7 @@ void SdchManager::GetAvailDictionaryList(const GURL& target_url,
 void SdchManager::GenerateHash(const std::string& dictionary_text,
     std::string* client_hash, std::string* server_hash) {
   char binary_hash[32];
-  base::SHA256HashString(dictionary_text, binary_hash, sizeof(binary_hash));
+  crypto::SHA256HashString(dictionary_text, binary_hash, sizeof(binary_hash));
 
   std::string first_48_bits(&binary_hash[0], 6);
   std::string second_48_bits(&binary_hash[6], 6);
@@ -508,11 +523,13 @@ void SdchManager::GenerateHash(const std::string& dictionary_text,
 // Methods for supporting latency experiments.
 
 bool SdchManager::AllowLatencyExperiment(const GURL& url) const {
+  DCHECK(CalledOnValidThread());
   return allow_latency_experiment_.end() !=
       allow_latency_experiment_.find(url.host());
 }
 
 void SdchManager::SetAllowLatencyExperiment(const GURL& url, bool enable) {
+  DCHECK(CalledOnValidThread());
   if (enable) {
     allow_latency_experiment_.insert(url.host());
     return;
@@ -543,3 +560,5 @@ void SdchManager::UrlSafeBase64Encode(const std::string& input,
     }
   }
 }
+
+}  // namespace net
