@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,8 @@
 #include <string>
 #include <vector>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
@@ -17,7 +18,7 @@ class X509Certificate;
 
 // The SSLCertRequestInfo class contains the info that allows a user to
 // select a certificate to send to the SSL server for client authentication.
-class SSLCertRequestInfo
+class NET_EXPORT SSLCertRequestInfo
     : public base::RefCountedThreadSafe<SSLCertRequestInfo> {
  public:
   SSLCertRequestInfo();
@@ -27,6 +28,10 @@ class SSLCertRequestInfo
   // The host and port of the SSL server that requested client authentication.
   std::string host_and_port;
 
+  // True if the server that issues this request was the HTTPS proxy used in
+  // the request.  False, if the server was the origin server.
+  bool is_proxy;
+
   // A list of client certificates that match the server's criteria in the
   // SSL CertificateRequest message.  In TLS 1.0, the CertificateRequest
   // message is defined as:
