@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,10 @@
 #include <string>
 #include <map>
 
-#include "base/ref_counted.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
+#include "net/base/cert_database.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
@@ -22,10 +25,10 @@ class X509Certificate;
 //
 // TODO(wtc): This class is based on FtpAuthCache.  We can extract the common
 // code to a template class.
-class SSLClientAuthCache {
+class NET_EXPORT_PRIVATE SSLClientAuthCache : public CertDatabase::Observer {
  public:
   SSLClientAuthCache();
-  ~SSLClientAuthCache();
+  virtual ~SSLClientAuthCache();
 
   // Checks for a client certificate preference for SSL server at |server|.
   // Returns true if a preference is found, and sets |*certificate| to the
@@ -44,6 +47,9 @@ class SSLClientAuthCache {
   // Remove the client certificate for |server| from the cache, if one exists.
   void Remove(const std::string& server);
 
+  // CertDatabase::Observer methods:
+  virtual void OnUserCertAdded(const X509Certificate* cert) OVERRIDE;
+
  private:
   typedef std::string AuthCacheKey;
   typedef scoped_refptr<X509Certificate> AuthCacheValue;
