@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,7 @@
 #define NET_BASE_SSL_CONFIG_SERVICE_DEFAULTS_H_
 #pragma once
 
+#include "net/base/net_export.h"
 #include "net/base/ssl_config_service.h"
 
 namespace net {
@@ -13,12 +14,12 @@ namespace net {
 // This SSLConfigService always returns the default SSLConfig settings.  It is
 // mainly useful for unittests, or for platforms that do not have a native
 // implementation of SSLConfigService yet.
-class SSLConfigServiceDefaults : public SSLConfigService {
+class NET_EXPORT SSLConfigServiceDefaults : public SSLConfigService {
  public:
   SSLConfigServiceDefaults();
 
   // Store default SSL config settings in |config|.
-  virtual void GetSSLConfig(SSLConfig* config);
+  virtual void GetSSLConfig(SSLConfig* config) OVERRIDE;
 
  private:
   virtual ~SSLConfigServiceDefaults();
