@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,17 +9,12 @@
 
 namespace net {
 
-SSLInfo::SSLInfo()
-    : cert_status(0),
-      security_bits(-1),
-      connection_status(0) {
+SSLInfo::SSLInfo() {
+  Reset();
 }
 
-SSLInfo::SSLInfo(const SSLInfo& info)
-    : cert(info.cert),
-      cert_status(info.cert_status),
-      security_bits(info.security_bits),
-      connection_status(info.connection_status) {
+SSLInfo::SSLInfo(const SSLInfo& info) {
+  *this = info;
 }
 
 SSLInfo::~SSLInfo() {
@@ -30,6 +25,10 @@ SSLInfo& SSLInfo::operator=(const SSLInfo& info) {
   cert_status = info.cert_status;
   security_bits = info.security_bits;
   connection_status = info.connection_status;
+  is_issued_by_known_root = info.is_issued_by_known_root;
+  client_cert_sent = info.client_cert_sent;
+  handshake_type = info.handshake_type;
+  public_key_hashes = info.public_key_hashes;
   return *this;
 }
 
@@ -38,6 +37,10 @@ void SSLInfo::Reset() {
   cert_status = 0;
   security_bits = -1;
   connection_status = 0;
+  is_issued_by_known_root = false;
+  client_cert_sent = false;
+  handshake_type = HANDSHAKE_UNKNOWN;
+  public_key_hashes.clear();
 }
 
 void SSLInfo::SetCertError(int error) {
