@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,8 @@
 #define NET_BASE_STATIC_COOKIE_POLICY_H_
 #pragma once
 
-#include <string>
-
 #include "base/basictypes.h"
-#include "net/base/cookie_policy.h"
+#include "net/base/net_export.h"
 
 class GURL;
 
@@ -17,11 +15,7 @@ namespace net {
 
 // The StaticCookiePolicy class implements a static cookie policy that supports
 // three modes: allow all, deny all, or block third-party cookies.
-//
-// NOTE: This CookiePolicy implementation always completes synchronously.  The
-// callback parameter will be ignored if specified.  Just pass NULL.
-//
-class StaticCookiePolicy : public CookiePolicy {
+class NET_EXPORT StaticCookiePolicy {
  public:
   // Do not change the order of these types as they are persisted in
   // preferences.
@@ -49,20 +43,14 @@ class StaticCookiePolicy : public CookiePolicy {
   void set_type(Type type) { type_ = type; }
   Type type() const { return type_; }
 
-  // CookiePolicy methods:
-
   // Consults the user's third-party cookie blocking preferences to determine
   // whether the URL's cookies can be read.
-  virtual int CanGetCookies(const GURL& url,
-                            const GURL& first_party_for_cookies,
-                            CompletionCallback* callback);
+  int CanGetCookies(const GURL& url, const GURL& first_party_for_cookies) const;
 
   // Consults the user's third-party cookie blocking preferences to determine
   // whether the URL's cookies can be set.
-  virtual int CanSetCookie(const GURL& url,
-                           const GURL& first_party_for_cookies,
-                           const std::string& cookie_line,
-                           CompletionCallback* callback);
+  int CanSetCookie(const GURL& url,
+                   const GURL& first_party_for_cookies) const;
 
  private:
   Type type_;
