@@ -1,11 +1,15 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/base/upload_data.h"
 
+#include <algorithm>
+
 #include "base/file_util.h"
 #include "base/logging.h"
+#include "base/string_util.h"
+#include "base/threading/thread_restrictions.h"
 #include "net/base/file_stream.h"
 #include "net/base/net_errors.h"
 
@@ -15,6 +19,7 @@ UploadData::Element::Element()
     : type_(TYPE_BYTES),
       file_range_offset_(0),
       file_range_length_(kuint64max),
+      is_last_chunk_(false),
       override_content_length_(false),
       content_length_computed_(false),
       content_length_(-1),
@@ -26,11 +31,20 @@ UploadData::Element::~Element() {
   delete file_stream_;
 }
 
+void UploadData::Element::SetToChunk(const char* bytes,
+                                     int bytes_len,
+                                     bool is_last_chunk) {
+  bytes_.clear();
+  bytes_.insert(bytes_.end(), bytes, bytes + bytes_len);
+  type_ = TYPE_CHUNK;
+  is_last_chunk_ = is_last_chunk;
+}
+
 uint64 UploadData::Element::GetContentLength() {
   if (override_content_length_ || content_length_computed_)
     return content_length_;
 
-  if (type_ == TYPE_BYTES)
+  if (type_ == TYPE_BYTES || type_ == TYPE_CHUNK)
     return static_cast<uint64>(bytes_.size());
   else if (type_ == TYPE_BLOB)
     // The blob reference will be resolved later.
@@ -54,8 +68,15 @@ uint64 UploadData::Element::GetContentLength() {
     return 0;
 
   int64 length = 0;
-  if (!file_util::GetFileSize(file_path_, &length))
-    return 0;
+
+  {
+    // TODO(tzik):
+    // file_util::GetFileSize may cause blocking IO.
+    // Temporary allow until fix: http://crbug.com/72001.
+    base::ThreadRestrictions::ScopedAllowIO allow_io;
+    if (!file_util::GetFileSize(file_path_, &length))
+      return 0;
+  }
 
   if (file_range_offset_ >= static_cast<uint64>(length))
     return 0;  // range is beyond eof
@@ -75,6 +96,11 @@ FileStream* UploadData::Element::NewFileStreamForReading() {
     return file;
   }
 
+  // TODO(tzik):
+  // FileStream::Open and FileStream::Seek may cause blocking IO.
+  // Temporary allow until fix: http://crbug.com/72001.
+  base::ThreadRestrictions::ScopedAllowIO allow_io;
+
   scoped_ptr<FileStream> file(new FileStream());
   int64 rv = file->Open(file_path_,
                       base::PLATFORM_FILE_OPEN | base::PLATFORM_FILE_READ);
@@ -97,35 +123,53 @@ FileStream* UploadData::Element::NewFileStreamForReading() {
   return file.release();
 }
 
-UploadData::UploadData() : identifier_(0) {
+UploadData::UploadData()
+    : identifier_(0),
+      chunk_callback_(NULL),
+      is_chunked_(false) {
 }
 
 void UploadData::AppendBytes(const char* bytes, int bytes_len) {
+  DCHECK(!is_chunked_);
   if (bytes_len > 0) {
     elements_.push_back(Element());
     elements_.back().SetToBytes(bytes, bytes_len);
   }
 }
 
-void UploadData::AppendFile(const FilePath& file_path) {
-  elements_.push_back(Element());
-  elements_.back().SetToFilePath(file_path);
-}
-
 void UploadData::AppendFileRange(const FilePath& file_path,
                                  uint64 offset, uint64 length,
                                  const base::Time& expected_modification_time) {
+  DCHECK(!is_chunked_);
   elements_.push_back(Element());
   elements_.back().SetToFilePathRange(file_path, offset, length,
                                       expected_modification_time);
 }
 
 void UploadData::AppendBlob(const GURL& blob_url) {
+  DCHECK(!is_chunked_);
   elements_.push_back(Element());
   elements_.back().SetToBlobUrl(blob_url);
 }
 
+void UploadData::AppendChunk(const char* bytes,
+                             int bytes_len,
+                             bool is_last_chunk) {
+  DCHECK(is_chunked_);
+  elements_.push_back(Element());
+  elements_.back().SetToChunk(bytes, bytes_len, is_last_chunk);
+  if (chunk_callback_)
+    chunk_callback_->OnChunkAvailable();
+}
+
+void UploadData::set_chunk_callback(ChunkCallback* callback) {
+  chunk_callback_ = callback;
+}
+
 uint64 UploadData::GetContentLength() {
+  if (is_chunked_)
+    return 0;
+
   uint64 len = 0;
   std::vector<Element>::iterator it = elements_.begin();
   for (; it != elements_.end(); ++it)
@@ -133,6 +177,22 @@ uint64 UploadData::GetContentLength() {
   return len;
 }
 
+bool UploadData::IsInMemory() const {
+  // Chunks are in memory, but UploadData does not have all the chunks at
+  // once. Chunks are provided progressively with AppendChunk() as chunks
+  // are ready. Check is_chunked_ here, rather than relying on the loop
+  // below, as there is a case that is_chunked_ is set to true, but the
+  // first chunk is not yet delivered.
+  if (is_chunked_)
+    return false;
+
+  for (size_t i = 0; i < elements_.size(); ++i) {
+    if (elements_[i].type() != TYPE_BYTES)
+      return false;
+  }
+  return true;
+}
+
 void UploadData::SetElements(const std::vector<Element>& elements) {
   elements_ = elements;
 }
