@@ -1,18 +1,19 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 // See net/disk_cache/disk_cache.h for the public interface.
 
-#ifndef NET_DISK_CACHE_BLOCK_FILES_H__
-#define NET_DISK_CACHE_BLOCK_FILES_H__
+#ifndef NET_DISK_CACHE_BLOCK_FILES_H_
+#define NET_DISK_CACHE_BLOCK_FILES_H_
 #pragma once
 
 #include <vector>
 
 #include "base/file_path.h"
 #include "base/gtest_prod_util.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/net_export.h"
 #include "net/disk_cache/addr.h"
 #include "net/disk_cache/mapped_file.h"
 
@@ -23,7 +24,7 @@ class ThreadChecker;
 namespace disk_cache {
 
 // This class handles the set of block-files open by the disk cache.
-class BlockFiles {
+class NET_EXPORT_PRIVATE BlockFiles {
  public:
   explicit BlockFiles(const FilePath& path);
   ~BlockFiles();
@@ -74,7 +75,7 @@ class BlockFiles {
   int CreateNextBlockFile(FileType block_type);
 
   // Removes a chained block file that is now empty.
-  void RemoveEmptyFile(FileType block_type);
+  bool RemoveEmptyFile(FileType block_type);
 
   // Restores the header of a potentially inconsistent file.
   bool FixBlockFileHeader(MappedFile* file);
@@ -92,6 +93,7 @@ class BlockFiles {
   scoped_ptr<base::ThreadChecker> thread_checker_;
 
   FRIEND_TEST_ALL_PREFIXES(DiskCacheTest, BlockFiles_ZeroSizeFile);
+  FRIEND_TEST_ALL_PREFIXES(DiskCacheTest, BlockFiles_TruncatedFile);
   FRIEND_TEST_ALL_PREFIXES(DiskCacheTest, BlockFiles_InvalidFile);
   FRIEND_TEST_ALL_PREFIXES(DiskCacheTest, BlockFiles_Stats);
 
@@ -100,4 +102,4 @@ class BlockFiles {
 
 }  // namespace disk_cache
 
-#endif  // NET_DISK_CACHE_BLOCK_FILES_H__
+#endif  // NET_DISK_CACHE_BLOCK_FILES_H_
