@@ -1,9 +1,11 @@
-// Copyright (c) 2006-2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/disk_cache/disk_cache_test_base.h"
 
+#include "base/file_util.h"
+#include "base/path_service.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
 #include "net/base/test_completion_callback.h"
@@ -11,6 +13,32 @@
 #include "net/disk_cache/disk_cache_test_util.h"
 #include "net/disk_cache/mem_backend_impl.h"
 
+DiskCacheTest::DiskCacheTest() {
+  cache_path_ = GetCacheFilePath();
+  if (!MessageLoop::current())
+    message_loop_.reset(new MessageLoopForIO());
+}
+
+DiskCacheTest::~DiskCacheTest() {
+}
+
+bool DiskCacheTest::CopyTestCache(const std::string& name) {
+  FilePath path;
+  PathService::Get(base::DIR_SOURCE_ROOT, &path);
+  path = path.AppendASCII("net");
+  path = path.AppendASCII("data");
+  path = path.AppendASCII("cache_tests");
+  path = path.AppendASCII(name);
+
+  if (!CleanupCacheDir())
+    return false;
+  return file_util::CopyDirectory(path, cache_path_, false);
+}
+
+bool DiskCacheTest::CleanupCacheDir() {
+  return DeleteCache(cache_path_);
+}
+
 void DiskCacheTest::TearDown() {
   MessageLoop::current()->RunAllPending();
 }
@@ -51,16 +79,15 @@ void DiskCacheTestWithCache::InitCache() {
 // We are expected to leak memory when simulating crashes.
 void DiskCacheTestWithCache::SimulateCrash() {
   ASSERT_TRUE(implementation_ && !memory_only_);
-  TestCompletionCallback cb;
-  int rv = cache_impl_->FlushQueueForTest(&cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_impl_->FlushQueueForTest(cb.callback());
   ASSERT_EQ(net::OK, cb.GetResult(rv));
   cache_impl_->ClearRefCountForTest();
 
   delete cache_impl_;
-  FilePath path = GetCacheFilePath();
-  EXPECT_TRUE(CheckCacheIntegrity(path, new_eviction_));
+  EXPECT_TRUE(CheckCacheIntegrity(cache_path_, new_eviction_, mask_));
 
-  InitDiskCacheImpl(path);
+  InitDiskCacheImpl();
 }
 
 void DiskCacheTestWithCache::SetTestMode() {
@@ -79,47 +106,47 @@ void DiskCacheTestWithCache::SetMaxSize(int size) {
 
 int DiskCacheTestWithCache::OpenEntry(const std::string& key,
                                       disk_cache::Entry** entry) {
-  TestCompletionCallback cb;
-  int rv = cache_->OpenEntry(key, entry, &cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_->OpenEntry(key, entry, cb.callback());
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::CreateEntry(const std::string& key,
                                         disk_cache::Entry** entry) {
-  TestCompletionCallback cb;
-  int rv = cache_->CreateEntry(key, entry, &cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_->CreateEntry(key, entry, cb.callback());
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::DoomEntry(const std::string& key) {
-  TestCompletionCallback cb;
-  int rv = cache_->DoomEntry(key, &cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_->DoomEntry(key, cb.callback());
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::DoomAllEntries() {
-  TestCompletionCallback cb;
-  int rv = cache_->DoomAllEntries(&cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_->DoomAllEntries(cb.callback());
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::DoomEntriesBetween(const base::Time initial_time,
                                                const base::Time end_time) {
-  TestCompletionCallback cb;
-  int rv = cache_->DoomEntriesBetween(initial_time, end_time, &cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_->DoomEntriesBetween(initial_time, end_time, cb.callback());
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::DoomEntriesSince(const base::Time initial_time) {
-  TestCompletionCallback cb;
-  int rv = cache_->DoomEntriesSince(initial_time, &cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_->DoomEntriesSince(initial_time, cb.callback());
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::OpenNextEntry(void** iter,
                                           disk_cache::Entry** next_entry) {
-  TestCompletionCallback cb;
-  int rv = cache_->OpenNextEntry(iter, next_entry, &cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_->OpenNextEntry(iter, next_entry, cb.callback());
   return cb.GetResult(rv);
 }
 
@@ -127,82 +154,63 @@ void DiskCacheTestWithCache::FlushQueueForTest() {
   if (memory_only_ || !cache_impl_)
     return;
 
-  TestCompletionCallback cb;
-  int rv = cache_impl_->FlushQueueForTest(&cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_impl_->FlushQueueForTest(cb.callback());
   EXPECT_EQ(net::OK, cb.GetResult(rv));
 }
 
-void DiskCacheTestWithCache::RunTaskForTest(Task* task) {
+void DiskCacheTestWithCache::RunTaskForTest(const base::Closure& closure) {
   if (memory_only_ || !cache_impl_) {
-    task->Run();
-    delete task;
+    closure.Run();
     return;
   }
 
-  TestCompletionCallback cb;
-  int rv = cache_impl_->RunTaskForTest(task, &cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_impl_->RunTaskForTest(closure, cb.callback());
   EXPECT_EQ(net::OK, cb.GetResult(rv));
 }
 
 int DiskCacheTestWithCache::ReadData(disk_cache::Entry* entry, int index,
                                      int offset, net::IOBuffer* buf, int len) {
-  TestCompletionCallback cb;
-  int rv = entry->ReadData(index, offset, buf, len, &cb);
+  net::TestCompletionCallback cb;
+  int rv = entry->ReadData(index, offset, buf, len, cb.callback());
   return cb.GetResult(rv);
 }
 
-
 int DiskCacheTestWithCache::WriteData(disk_cache::Entry* entry, int index,
                                       int offset, net::IOBuffer* buf, int len,
                                       bool truncate) {
-  TestCompletionCallback cb;
-  int rv = entry->WriteData(index, offset, buf, len, &cb, truncate);
+  net::TestCompletionCallback cb;
+  int rv = entry->WriteData(index, offset, buf, len, cb.callback(), truncate);
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::ReadSparseData(disk_cache::Entry* entry,
                                            int64 offset, net::IOBuffer* buf,
                                            int len) {
-  TestCompletionCallback cb;
-  int rv = entry->ReadSparseData(offset, buf, len, &cb);
+  net::TestCompletionCallback cb;
+  int rv = entry->ReadSparseData(offset, buf, len, cb.callback());
   return cb.GetResult(rv);
 }
 
 int DiskCacheTestWithCache::WriteSparseData(disk_cache::Entry* entry,
                                             int64 offset,
                                             net::IOBuffer* buf, int len) {
-  TestCompletionCallback cb;
-  int rv = entry->WriteSparseData(offset, buf, len, &cb);
+  net::TestCompletionCallback cb;
+  int rv = entry->WriteSparseData(offset, buf, len, cb.callback());
   return cb.GetResult(rv);
 }
 
-// Simple task to run part of a test from the cache thread.
-class TrimTask : public Task {
- public:
-  TrimTask(disk_cache::BackendImpl* backend, bool deleted, bool empty)
-      : backend_(backend),
-        deleted_(deleted),
-        empty_(empty) {}
-
-  virtual void Run() {
-    if (deleted_)
-      backend_->TrimDeletedListForTest(empty_);
-    else
-      backend_->TrimForTest(empty_);
-  }
-
- protected:
-  disk_cache::BackendImpl* backend_;
-  bool deleted_;
-  bool empty_;
-};
-
 void DiskCacheTestWithCache::TrimForTest(bool empty) {
-  RunTaskForTest(new TrimTask(cache_impl_, false, empty));
+  RunTaskForTest(base::Bind(&disk_cache::BackendImpl::TrimForTest,
+                            base::Unretained(cache_impl_),
+                            empty));
 }
 
 void DiskCacheTestWithCache::TrimDeletedListForTest(bool empty) {
-  RunTaskForTest(new TrimTask(cache_impl_, true, empty));
+  RunTaskForTest(base::Bind(&disk_cache::BackendImpl::TrimDeletedListForTest,
+                            base::Unretained(cache_impl_),
+                            empty));
 }
 
 void DiskCacheTestWithCache::TearDown() {
@@ -212,8 +220,7 @@ void DiskCacheTestWithCache::TearDown() {
     cache_thread_.Stop();
 
   if (!memory_only_ && integrity_) {
-    FilePath path = GetCacheFilePath();
-    EXPECT_TRUE(CheckCacheIntegrity(path, new_eviction_));
+    EXPECT_TRUE(CheckCacheIntegrity(cache_path_, new_eviction_, mask_));
   }
 
   PlatformTest::TearDown();
@@ -221,11 +228,11 @@ void DiskCacheTestWithCache::TearDown() {
 
 void DiskCacheTestWithCache::InitMemoryCache() {
   if (!implementation_) {
-    cache_ = disk_cache::MemBackendImpl::CreateBackend(size_);
+    cache_ = disk_cache::MemBackendImpl::CreateBackend(size_, NULL);
     return;
   }
 
-  mem_cache_ = new disk_cache::MemBackendImpl();
+  mem_cache_ = new disk_cache::MemBackendImpl(NULL);
   cache_ = mem_cache_;
   ASSERT_TRUE(NULL != cache_);
 
@@ -236,9 +243,8 @@ void DiskCacheTestWithCache::InitMemoryCache() {
 }
 
 void DiskCacheTestWithCache::InitDiskCache() {
-  FilePath path = GetCacheFilePath();
   if (first_cleanup_)
-    ASSERT_TRUE(DeleteCache(path));
+    ASSERT_TRUE(CleanupCacheDir());
 
   if (!cache_thread_.IsRunning()) {
     EXPECT_TRUE(cache_thread_.StartWithOptions(
@@ -247,27 +253,27 @@ void DiskCacheTestWithCache::InitDiskCache() {
   ASSERT_TRUE(cache_thread_.message_loop() != NULL);
 
   if (implementation_)
-    return InitDiskCacheImpl(path);
+    return InitDiskCacheImpl();
 
   scoped_refptr<base::MessageLoopProxy> thread =
-      use_current_thread_ ? base::MessageLoopProxy::CreateForCurrentThread() :
+      use_current_thread_ ? base::MessageLoopProxy::current() :
                             cache_thread_.message_loop_proxy();
 
-  TestCompletionCallback cb;
+  net::TestCompletionCallback cb;
   int rv = disk_cache::BackendImpl::CreateBackend(
-               path, force_creation_, size_, type_,
-               disk_cache::kNoRandom, thread, NULL, &cache_, &cb);
+               cache_path_, force_creation_, size_, type_,
+               disk_cache::kNoRandom, thread, NULL, &cache_, cb.callback());
   ASSERT_EQ(net::OK, cb.GetResult(rv));
 }
 
-void DiskCacheTestWithCache::InitDiskCacheImpl(const FilePath& path) {
+void DiskCacheTestWithCache::InitDiskCacheImpl() {
   scoped_refptr<base::MessageLoopProxy> thread =
-      use_current_thread_ ? base::MessageLoopProxy::CreateForCurrentThread() :
+      use_current_thread_ ? base::MessageLoopProxy::current() :
                             cache_thread_.message_loop_proxy();
   if (mask_)
-    cache_impl_ = new disk_cache::BackendImpl(path, mask_, thread, NULL);
+    cache_impl_ = new disk_cache::BackendImpl(cache_path_, mask_, thread, NULL);
   else
-    cache_impl_ = new disk_cache::BackendImpl(path, thread, NULL);
+    cache_impl_ = new disk_cache::BackendImpl(cache_path_, thread, NULL);
 
   cache_ = cache_impl_;
   ASSERT_TRUE(NULL != cache_);
@@ -280,7 +286,7 @@ void DiskCacheTestWithCache::InitDiskCacheImpl(const FilePath& path) {
 
   cache_impl_->SetType(type_);
   cache_impl_->SetFlags(disk_cache::kNoRandom);
-  TestCompletionCallback cb;
-  int rv = cache_impl_->Init(&cb);
+  net::TestCompletionCallback cb;
+  int rv = cache_impl_->Init(cb.callback());
   ASSERT_EQ(net::OK, cb.GetResult(rv));
 }
