@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,13 +7,13 @@
 #pragma once
 
 #include "base/basictypes.h"
+#include "base/file_path.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/threading/thread.h"
 #include "net/base/cache_type.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
 
-class FilePath;
-
 namespace net {
 
 class IOBuffer;
@@ -34,7 +34,22 @@ class MemBackendImpl;
 // cache (and that do not need to be a DiskCacheTestWithCache) are susceptible
 // to this problem; all such tests should use TEST_F(DiskCacheTest, ...).
 class DiskCacheTest : public PlatformTest {
-  virtual void TearDown();
+ protected:
+  DiskCacheTest();
+  virtual ~DiskCacheTest();
+
+  // Copies a set of cache files from the data folder to the test folder.
+  bool CopyTestCache(const std::string& name);
+
+  // Deletes the contents of |cache_path_|.
+  bool CleanupCacheDir();
+
+  virtual void TearDown() OVERRIDE;
+
+  FilePath cache_path_;
+
+ private:
+  scoped_ptr<MessageLoop> message_loop_;
 };
 
 // Provides basic support for cache related tests.
@@ -97,7 +112,7 @@ class DiskCacheTestWithCache : public DiskCacheTest {
   int DoomEntriesSince(const base::Time initial_time);
   int OpenNextEntry(void** iter, disk_cache::Entry** next_entry);
   void FlushQueueForTest();
-  void RunTaskForTest(Task* task);
+  void RunTaskForTest(const base::Closure& closure);
   int ReadData(disk_cache::Entry* entry, int index, int offset,
                net::IOBuffer* buf, int len);
   int WriteData(disk_cache::Entry* entry, int index, int offset,
@@ -107,16 +122,16 @@ class DiskCacheTestWithCache : public DiskCacheTest {
   int WriteSparseData(disk_cache::Entry* entry, int64 offset,
                       net::IOBuffer* buf, int len);
 
-  // Asks the cache to trim a an entry. If |empty| is true, the whole entry is
+  // Asks the cache to trim an entry. If |empty| is true, the whole cache is
   // deleted.
   void TrimForTest(bool empty);
 
-  // Asks the cache to trim a an entry from the deleted list. If |empty| is
-  // true, the whole entry is deleted.
+  // Asks the cache to trim an entry from the deleted list. If |empty| is
+  // true, the whole list is deleted.
   void TrimDeletedListForTest(bool empty);
 
   // DiskCacheTest:
-  virtual void TearDown();
+  virtual void TearDown() OVERRIDE;
 
   // cache_ will always have a valid object, regardless of how the cache was
   // initialized. The implementation pointers can be NULL.
@@ -140,7 +155,7 @@ class DiskCacheTestWithCache : public DiskCacheTest {
  private:
   void InitMemoryCache();
   void InitDiskCache();
-  void InitDiskCacheImpl(const FilePath& path);
+  void InitDiskCacheImpl();
 
   base::Thread cache_thread_;
   DISALLOW_COPY_AND_ASSIGN(DiskCacheTestWithCache);
