@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,7 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/task.h"
+#include "base/memory/weak_ptr.h"
 #include "net/disk_cache/disk_format.h"
 #include "net/disk_cache/rankings.h"
 
@@ -48,9 +48,10 @@ class Eviction {
   void PostDelayedTrim();
   void DelayedTrim();
   bool ShouldTrim();
+  bool ShouldTrimDeleted();
   void ReportTrimTimes(EntryImpl* entry);
   Rankings::List GetListForEntry(EntryImpl* entry);
-  bool EvictEntry(CacheRankingsBlock* node, bool empty);
+  bool EvictEntry(CacheRankingsBlock* node, bool empty, Rankings::List list);
 
   // We'll just keep for a while a separate set of methods that implement the
   // new eviction algorithm. This code will replace the original methods when
@@ -66,7 +67,7 @@ class Eviction {
   bool RemoveDeletedNode(CacheRankingsBlock* node);
 
   bool NodeIsOldEnough(CacheRankingsBlock* node, int list);
-  int SelectListByLenght();
+  int SelectListByLength(Rankings::ScopedRankingsBlock* next);
   void ReportListStats();
 
   BackendImpl* backend_;
@@ -81,7 +82,7 @@ class Eviction {
   bool init_;
   bool test_mode_;
   bool in_experiment_;
-  ScopedRunnableMethodFactory<Eviction> factory_;
+  base::WeakPtrFactory<Eviction> ptr_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(Eviction);
 };
