@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,8 +8,9 @@
 #define NET_DISK_CACHE_FILE_H_
 #pragma once
 
+#include "base/memory/ref_counted.h"
 #include "base/platform_file.h"
-#include "base/ref_counted.h"
+#include "net/base/net_export.h"
 
 class FilePath;
 
@@ -26,7 +27,7 @@ class FileIOCallback {
 };
 
 // Simple wrapper around a file that allows asynchronous operations.
-class File : public base::RefCounted<File> {
+class NET_EXPORT_PRIVATE File : public base::RefCounted<File> {
   friend class base::RefCounted<File>;
  public:
   File();
