@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,11 @@
 
 #include <fcntl.h>
 
+#include "base/bind.h"
+#include "base/location.h"
 #include "base/logging.h"
 #include "base/threading/worker_pool.h"
+#include "net/base/net_errors.h"
 #include "net/disk_cache/disk_cache.h"
 #include "net/disk_cache/in_flight_io.h"
 
@@ -92,7 +95,7 @@ void FileBackgroundIO::Read() {
   if (file_->Read(const_cast<void*>(buf_), buf_len_, offset_)) {
     result_ = static_cast<int>(buf_len_);
   } else {
-    result_ = -1;
+    result_ = net::ERR_CACHE_READ_FAILURE;
   }
   controller_->OnIOComplete(this);
 }
@@ -101,7 +104,7 @@ void FileBackgroundIO::Read() {
 void FileBackgroundIO::Write() {
   bool rv = file_->Write(buf_, buf_len_, offset_);
 
-  result_ = rv ? static_cast<int>(buf_len_) : -1;
+  result_ = rv ? static_cast<int>(buf_len_) : net::ERR_CACHE_WRITE_FAILURE;
   controller_->OnIOComplete(this);
 }
 
@@ -114,7 +117,7 @@ void FileInFlightIO::PostRead(disk_cache::File *file, void* buf, size_t buf_len,
   file->AddRef();  // Balanced on OnOperationComplete()
 
   base::WorkerPool::PostTask(FROM_HERE,
-      NewRunnableMethod(operation.get(), &FileBackgroundIO::Read), true);
+      base::Bind(&FileBackgroundIO::Read, operation.get()), true);
   OnOperationPosted(operation);
 }
 
@@ -126,7 +129,7 @@ void FileInFlightIO::PostWrite(disk_cache::File* file, const void* buf,
   file->AddRef();  // Balanced on OnOperationComplete()
 
   base::WorkerPool::PostTask(FROM_HERE,
-      NewRunnableMethod(operation.get(), &FileBackgroundIO::Write), true);
+      base::Bind(&FileBackgroundIO::Write, operation.get()), true);
   OnOperationPosted(operation);
 }
 
@@ -189,11 +192,6 @@ bool File::Init(const FilePath& name) {
   return true;
 }
 
-File::~File() {
-  if (platform_file_)
-    close(platform_file_);
-}
-
 base::PlatformFile File::platform_file() const {
   return platform_file_;
 }
@@ -206,19 +204,24 @@ bool File::IsValid() const {
 
 bool File::Read(void* buffer, size_t buffer_len, size_t offset) {
   DCHECK(init_);
-  if (buffer_len > ULONG_MAX || offset > LONG_MAX)
+  if (buffer_len > static_cast<size_t>(kint32max) ||
+      offset > static_cast<size_t>(kint32max))
     return false;
 
-  int ret = pread(platform_file_, buffer, buffer_len, offset);
+  int ret = base::ReadPlatformFile(platform_file_, offset,
+                                   static_cast<char*>(buffer), buffer_len);
   return (static_cast<size_t>(ret) == buffer_len);
 }
 
 bool File::Write(const void* buffer, size_t buffer_len, size_t offset) {
   DCHECK(init_);
-  if (buffer_len > ULONG_MAX || offset > ULONG_MAX)
+  if (buffer_len > static_cast<size_t>(kint32max) ||
+      offset > static_cast<size_t>(kint32max))
     return false;
 
-  int ret = pwrite(platform_file_, buffer, buffer_len, offset);
+  int ret = base::WritePlatformFile(platform_file_, offset,
+                                    static_cast<const char*>(buffer),
+                                    buffer_len);
   return (static_cast<size_t>(ret) == buffer_len);
 }
 
@@ -255,6 +258,35 @@ bool File::Write(const void* buffer, size_t buffer_len, size_t offset,
   return AsyncWrite(buffer, buffer_len, offset, callback, completed);
 }
 
+bool File::SetLength(size_t length) {
+  DCHECK(init_);
+  if (length > ULONG_MAX)
+    return false;
+
+  return base::TruncatePlatformFile(platform_file_, length);
+}
+
+size_t File::GetLength() {
+  DCHECK(init_);
+  off_t ret = lseek(platform_file_, 0, SEEK_END);
+  if (ret < 0)
+    return 0;
+  return ret;
+}
+
+// Static.
+void File::WaitForPendingIO(int* num_pending_io) {
+  // We may be running unit tests so we should allow be able to reset the
+  // message loop.
+  GetFileInFlightIO()->WaitForPendingIO();
+  DeleteFileInFlightIO();
+}
+
+File::~File() {
+  if (IsValid())
+    base::ClosePlatformFile(platform_file_);
+}
+
 bool File::AsyncWrite(const void* buffer, size_t buffer_len, size_t offset,
                       FileIOCallback* callback, bool* completed) {
   DCHECK(init_);
@@ -268,26 +300,4 @@ bool File::AsyncWrite(const void* buffer, size_t buffer_len, size_t offset,
   return true;
 }
 
-bool File::SetLength(size_t length) {
-  DCHECK(init_);
-  if (length > ULONG_MAX)
-    return false;
-
-  return 0 == ftruncate(platform_file_, length);
-}
-
-size_t File::GetLength() {
-  DCHECK(init_);
-  size_t ret = lseek(platform_file_, 0, SEEK_END);
-  return ret;
-}
-
-// Static.
-void File::WaitForPendingIO(int* num_pending_io) {
-  // We may be running unit tests so we should allow be able to reset the
-  // message loop.
-  GetFileInFlightIO()->WaitForPendingIO();
-  DeleteFileInFlightIO();
-}
-
 }  // namespace disk_cache
