@@ -1,9 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/disk_cache/in_flight_backend_io.h"
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
 #include "net/base/net_errors.h"
@@ -14,11 +16,16 @@
 namespace disk_cache {
 
 BackendIO::BackendIO(InFlightIO* controller, BackendImpl* backend,
-                     net::CompletionCallback* callback)
-    : BackgroundIO(controller), backend_(backend), callback_(callback),
+                     const net::CompletionCallback& callback)
+    : BackgroundIO(controller),
+      backend_(backend),
+      callback_(callback),
       operation_(OP_NONE),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          my_callback_(this, &BackendIO::OnIOComplete)) {
+      entry_ptr_(NULL),
+      iter_ptr_(NULL),
+      iter_(NULL),
+      entry_(NULL),
+      start_(NULL) {
   start_time_ = base::TimeTicks::Now();
 }
 
@@ -105,6 +112,11 @@ void BackendIO::EndEnumeration(void* iterator) {
   iter_ = iterator;
 }
 
+void BackendIO::OnExternalCacheHit(const std::string& key) {
+  operation_ = OP_ON_EXTERNAL_CACHE_HIT;
+  key_ = key;
+}
+
 void BackendIO::CloseEntryImpl(EntryImpl* entry) {
   operation_ = OP_CLOSE_ENTRY;
   entry_ = entry;
@@ -119,7 +131,7 @@ void BackendIO::FlushQueue() {
   operation_ = OP_FLUSH_QUEUE;
 }
 
-void BackendIO::RunTask(Task* task) {
+void BackendIO::RunTask(const base::Closure& task) {
   operation_ = OP_RUN_TASK;
   task_ = task;
 }
@@ -218,6 +230,10 @@ void BackendIO::ExecuteBackendOperation() {
       backend_->SyncEndEnumeration(iter_);
       result_ = net::OK;
       break;
+    case OP_ON_EXTERNAL_CACHE_HIT:
+      backend_->SyncOnExternalCacheHit(key_);
+      result_ = net::OK;
+      break;
     case OP_CLOSE_ENTRY:
       entry_->Release();
       result_ = net::OK;
@@ -230,8 +246,7 @@ void BackendIO::ExecuteBackendOperation() {
       result_ = net::OK;
       break;
     case OP_RUN_TASK:
-      task_->Run();
-      delete task_;
+      task_.Run();
       result_ = net::OK;
       break;
     default:
@@ -246,20 +261,25 @@ void BackendIO::ExecuteBackendOperation() {
 void BackendIO::ExecuteEntryOperation() {
   switch (operation_) {
     case OP_READ:
-      result_ = entry_->ReadDataImpl(index_, offset_, buf_, buf_len_,
-                                     &my_callback_);
+      result_ = entry_->ReadDataImpl(
+          index_, offset_, buf_, buf_len_,
+          base::Bind(&BackendIO::OnIOComplete, base::Unretained(this)));
       break;
     case OP_WRITE:
-      result_ = entry_->WriteDataImpl(index_, offset_, buf_, buf_len_,
-                                      &my_callback_, truncate_);
+      result_ = entry_->WriteDataImpl(
+          index_, offset_, buf_, buf_len_,
+          base::Bind(&BackendIO::OnIOComplete, base::Unretained(this)),
+          truncate_);
       break;
     case OP_READ_SPARSE:
-      result_ = entry_->ReadSparseDataImpl(offset64_, buf_, buf_len_,
-                                           &my_callback_);
+      result_ = entry_->ReadSparseDataImpl(
+          offset64_, buf_, buf_len_,
+          base::Bind(&BackendIO::OnIOComplete, base::Unretained(this)));
       break;
     case OP_WRITE_SPARSE:
-      result_ = entry_->WriteSparseDataImpl(offset64_, buf_, buf_len_,
-                                            &my_callback_);
+      result_ = entry_->WriteSparseDataImpl(
+          offset64_, buf_, buf_len_,
+          base::Bind(&BackendIO::OnIOComplete, base::Unretained(this)));
       break;
     case OP_GET_RANGE:
       result_ = entry_->GetAvailableRangeImpl(offset64_, buf_len_, start_);
@@ -269,267 +289,205 @@ void BackendIO::ExecuteEntryOperation() {
       result_ = net::OK;
       break;
     case OP_IS_READY:
-      result_ = entry_->ReadyForSparseIOImpl(&my_callback_);
+      result_ = entry_->ReadyForSparseIOImpl(
+          base::Bind(&BackendIO::OnIOComplete, base::Unretained(this)));
       break;
     default:
       NOTREACHED() << "Invalid Operation";
       result_ = net::ERR_UNEXPECTED;
   }
-  // We added a reference to protect the queued operation.
-  entry_->Release();
   if (result_ != net::ERR_IO_PENDING)
     controller_->OnIOComplete(this);
 }
 
-// ---------------------------------------------------------------------------
-
 InFlightBackendIO::InFlightBackendIO(BackendImpl* backend,
                     base::MessageLoopProxy* background_thread)
     : backend_(backend),
-      background_thread_(background_thread),
-      max_queue_len_(0),
-      queue_entry_ops_(false) {
+      background_thread_(background_thread) {
 }
 
 InFlightBackendIO::~InFlightBackendIO() {
 }
 
-void InFlightBackendIO::Init(CompletionCallback* callback) {
+void InFlightBackendIO::Init(const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->Init();
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::OpenEntry(const std::string& key, Entry** entry,
-                                  CompletionCallback* callback) {
+                                  const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->OpenEntry(key, entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::CreateEntry(const std::string& key, Entry** entry,
-                                    CompletionCallback* callback) {
+                                    const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->CreateEntry(key, entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::DoomEntry(const std::string& key,
-                                  CompletionCallback* callback) {
+                                  const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->DoomEntry(key);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::DoomAllEntries(CompletionCallback* callback) {
+void InFlightBackendIO::DoomAllEntries(
+    const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->DoomAllEntries();
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::DoomEntriesBetween(const base::Time initial_time,
                         const base::Time end_time,
-                        CompletionCallback* callback) {
+                        const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->DoomEntriesBetween(initial_time, end_time);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::DoomEntriesSince(const base::Time initial_time,
-                                         CompletionCallback* callback) {
+void InFlightBackendIO::DoomEntriesSince(
+    const base::Time initial_time, const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->DoomEntriesSince(initial_time);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::OpenNextEntry(void** iter, Entry** next_entry,
-                                      CompletionCallback* callback) {
+                                      const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->OpenNextEntry(iter, next_entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::OpenPrevEntry(void** iter, Entry** prev_entry,
-                                      CompletionCallback* callback) {
+                                      const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->OpenPrevEntry(iter, prev_entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::EndEnumeration(void* iterator) {
-  scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, NULL));
+  scoped_refptr<BackendIO> operation(
+      new BackendIO(this, backend_, net::CompletionCallback()));
   operation->EndEnumeration(iterator);
-  QueueOperation(operation);
+  PostOperation(operation);
+}
+
+void InFlightBackendIO::OnExternalCacheHit(const std::string& key) {
+  scoped_refptr<BackendIO> operation(
+      new BackendIO(this, backend_, net::CompletionCallback()));
+  operation->OnExternalCacheHit(key);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::CloseEntryImpl(EntryImpl* entry) {
-  scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, NULL));
+  scoped_refptr<BackendIO> operation(
+      new BackendIO(this, backend_, net::CompletionCallback()));
   operation->CloseEntryImpl(entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::DoomEntryImpl(EntryImpl* entry) {
-  scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, NULL));
+  scoped_refptr<BackendIO> operation(
+      new BackendIO(this, backend_, net::CompletionCallback()));
   operation->DoomEntryImpl(entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::FlushQueue(net::CompletionCallback* callback) {
+void InFlightBackendIO::FlushQueue(const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->FlushQueue();
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::RunTask(Task* task, net::CompletionCallback* callback) {
+void InFlightBackendIO::RunTask(
+    const base::Closure& task, const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->RunTask(task);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::ReadData(EntryImpl* entry, int index, int offset,
                                  net::IOBuffer* buf, int buf_len,
-                                 CompletionCallback* callback) {
+                                 const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->ReadData(entry, index, offset, buf, buf_len);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::WriteData(EntryImpl* entry, int index, int offset,
                                   net::IOBuffer* buf, int buf_len,
                                   bool truncate,
-                                  CompletionCallback* callback) {
+                                  const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->WriteData(entry, index, offset, buf, buf_len, truncate);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::ReadSparseData(EntryImpl* entry, int64 offset,
-                                       net::IOBuffer* buf, int buf_len,
-                                       CompletionCallback* callback) {
+void InFlightBackendIO::ReadSparseData(
+    EntryImpl* entry, int64 offset, net::IOBuffer* buf, int buf_len,
+    const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->ReadSparseData(entry, offset, buf, buf_len);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::WriteSparseData(EntryImpl* entry, int64 offset,
-                                        net::IOBuffer* buf, int buf_len,
-                                        CompletionCallback* callback) {
+void InFlightBackendIO::WriteSparseData(
+    EntryImpl* entry, int64 offset, net::IOBuffer* buf, int buf_len,
+    const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->WriteSparseData(entry, offset, buf, buf_len);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::GetAvailableRange(EntryImpl* entry, int64 offset,
-                                          int len, int64* start,
-                                          CompletionCallback* callback) {
+void InFlightBackendIO::GetAvailableRange(
+    EntryImpl* entry, int64 offset, int len, int64* start,
+    const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->GetAvailableRange(entry, offset, len, start);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::CancelSparseIO(EntryImpl* entry) {
-  scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, NULL));
+  scoped_refptr<BackendIO> operation(
+      new BackendIO(this, backend_, net::CompletionCallback()));
   operation->CancelSparseIO(entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
-void InFlightBackendIO::ReadyForSparseIO(EntryImpl* entry,
-                                         CompletionCallback* callback) {
+void InFlightBackendIO::ReadyForSparseIO(
+    EntryImpl* entry, const net::CompletionCallback& callback) {
   scoped_refptr<BackendIO> operation(new BackendIO(this, backend_, callback));
   operation->ReadyForSparseIO(entry);
-  QueueOperation(operation);
+  PostOperation(operation);
 }
 
 void InFlightBackendIO::WaitForPendingIO() {
   InFlightIO::WaitForPendingIO();
 }
 
-void InFlightBackendIO::StartQueingOperations() {
-  queue_entry_ops_ = true;
-  CACHE_UMA(COUNTS_10000, "InitialQueuedOperations", 0, pending_ops_.size());
-  if (!pending_ops_.size()) {
-    queueing_start_ = base::TimeTicks::Now();
-    max_queue_len_ = 0;
-  }
-}
-
-void InFlightBackendIO::StopQueingOperations() {
-  queue_entry_ops_ = false;
-  CACHE_UMA(COUNTS_10000, "FinalQueuedOperations", 0, pending_ops_.size());
-}
-
 void InFlightBackendIO::OnOperationComplete(BackgroundIO* operation,
                                             bool cancel) {
   BackendIO* op = static_cast<BackendIO*>(operation);
 
   if (op->IsEntryOperation()) {
-    backend_->OnOperationCompleted(op->ElapsedTime());
-    if (!pending_ops_.empty() && RemoveFirstQueuedOperation(op)) {
-      // Process the next request. Note that invoking the callback may result
-      // in the backend destruction (and with it this object), so we should deal
-      // with the next operation before invoking the callback.
-      PostQueuedOperation();
-      if (!pending_ops_.size() && !queue_entry_ops_) {
-        CACHE_UMA(AGE_MS, "ThrottleTime", 0, queueing_start_);
-        CACHE_UMA(COUNTS_10000, "MaxQueuedOperations", 0, max_queue_len_);
-      }
-    }
+    CACHE_UMA(TIMES, "TotalIOTime", 0, op->ElapsedTime());
   }
 
-  if (op->callback() && (!cancel || op->IsEntryOperation()))
-    op->callback()->Run(op->result());
-}
-
-void InFlightBackendIO::QueueOperation(BackendIO* operation) {
-  if (!operation->IsEntryOperation())
-    return PostOperation(operation);
-
-  // We have to protect the entry from deletion while it is on the queue.
-  // If the caller closes the entry right after writing to it, and the write is
-  // waiting on the queue, we could end up deleting the entry before the write
-  // operation is actually posted. Sending a task to reference the entry we make
-  // sure that there is an extra reference before the caller can post a task to
-  // release its reference.
-  background_thread_->PostTask(FROM_HERE,
-      NewRunnableMethod(operation, &BackendIO::ReferenceEntry));
-
-  bool empty_list = pending_ops_.empty();
-  if (!queue_entry_ops_ && empty_list)
-    return PostOperation(operation);
-
-  CACHE_UMA(COUNTS_10000, "QueuedOperations", 0, pending_ops_.size());
-
-  // We keep the operation that we are executing in the list so that we know
-  // when it completes.
-  pending_ops_.push_back(operation);
-  max_queue_len_ = std::max(max_queue_len_, pending_ops_.size());
-  if (empty_list)
-    PostOperation(operation);
+  if (!op->callback().is_null() && (!cancel || op->IsEntryOperation()))
+    op->callback().Run(op->result());
 }
 
 void InFlightBackendIO::PostOperation(BackendIO* operation) {
   background_thread_->PostTask(FROM_HERE,
-      NewRunnableMethod(operation, &BackendIO::ExecuteOperation));
+      base::Bind(&BackendIO::ExecuteOperation, operation));
   OnOperationPosted(operation);
 }
 
-void InFlightBackendIO::PostQueuedOperation() {
-  if (pending_ops_.empty())
-    return;
-
-  // Keep it in the list until it's done.
-  scoped_refptr<BackendIO> next_op = pending_ops_.front();
-  PostOperation(next_op);
-}
-
-bool InFlightBackendIO::RemoveFirstQueuedOperation(BackendIO* operation) {
-  DCHECK(!pending_ops_.empty());
-  scoped_refptr<BackendIO> next_op = pending_ops_.front();
-  if (operation != next_op)
-    return false;
-
-  pending_ops_.pop_front();
-  return true;
-}
-
 }  // namespace
