@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,21 +10,23 @@
 #include <string>
 
 #include "base/message_loop_proxy.h"
+#include "base/time.h"
 #include "net/base/completion_callback.h"
 #include "net/base/io_buffer.h"
-#include "net/disk_cache/entry_impl.h"
 #include "net/disk_cache/in_flight_io.h"
 
 namespace disk_cache {
 
 class BackendImpl;
+class Entry;
+class EntryImpl;
 
 // This class represents a single asynchronous disk cache IO operation while it
 // is being bounced between threads.
 class BackendIO : public BackgroundIO {
  public:
   BackendIO(InFlightIO* controller, BackendImpl* backend,
-            net::CompletionCallback* callback);
+            const net::CompletionCallback& callback);
 
   // Runs the actual operation on the background thread.
   void ExecuteOperation();
@@ -35,7 +37,7 @@ class BackendIO : public BackgroundIO {
   // Returns true if this operation is directed to an entry (vs. the backend).
   bool IsEntryOperation();
 
-  net::CompletionCallback* callback() { return callback_; }
+  net::CompletionCallback callback() const { return callback_; }
 
   // Grabs an extra reference of entry_.
   void ReferenceEntry();
@@ -55,10 +57,11 @@ class BackendIO : public BackgroundIO {
   void OpenNextEntry(void** iter, Entry** next_entry);
   void OpenPrevEntry(void** iter, Entry** prev_entry);
   void EndEnumeration(void* iterator);
+  void OnExternalCacheHit(const std::string& key);
   void CloseEntryImpl(EntryImpl* entry);
   void DoomEntryImpl(EntryImpl* entry);
   void FlushQueue();  // Dummy operation.
-  void RunTask(Task* task);
+  void RunTask(const base::Closure& task);
   void ReadData(EntryImpl* entry, int index, int offset, net::IOBuffer* buf,
                 int buf_len);
   void WriteData(EntryImpl* entry, int index, int offset, net::IOBuffer* buf,
@@ -89,6 +92,7 @@ class BackendIO : public BackgroundIO {
     OP_OPEN_NEXT,
     OP_OPEN_PREV,
     OP_END_ENUMERATION,
+    OP_ON_EXTERNAL_CACHE_HIT,
     OP_CLOSE_ENTRY,
     OP_DOOM_ENTRY,
     OP_FLUSH_QUEUE,
@@ -103,15 +107,14 @@ class BackendIO : public BackgroundIO {
     OP_IS_READY
   };
 
-  ~BackendIO();
+  virtual ~BackendIO();
 
   void ExecuteBackendOperation();
   void ExecuteEntryOperation();
 
   BackendImpl* backend_;
-  net::CompletionCallback* callback_;
+  net::CompletionCallback callback_;
   Operation operation_;
-  net::CompletionCallbackImpl<BackendIO> my_callback_;
 
   // The arguments of all the operations we proxy:
   std::string key_;
@@ -129,7 +132,7 @@ class BackendIO : public BackgroundIO {
   int64 offset64_;
   int64* start_;
   base::TimeTicks start_time_;
-  Task* task_;
+  base::Closure task_;
 
   DISALLOW_COPY_AND_ASSIGN(BackendIO);
 };
@@ -139,42 +142,47 @@ class InFlightBackendIO : public InFlightIO {
  public:
   InFlightBackendIO(BackendImpl* backend,
                     base::MessageLoopProxy* background_thread);
-  ~InFlightBackendIO();
+  virtual ~InFlightBackendIO();
 
-  // The operations we proxy:
-  void Init(net::CompletionCallback* callback);
+  // Proxied operations.
+  void Init(const net::CompletionCallback& callback);
   void OpenEntry(const std::string& key, Entry** entry,
-                 net::CompletionCallback* callback);
+                 const net::CompletionCallback& callback);
   void CreateEntry(const std::string& key, Entry** entry,
-                   net::CompletionCallback* callback);
-  void DoomEntry(const std::string& key, net::CompletionCallback* callback);
-  void DoomAllEntries(net::CompletionCallback* callback);
+                   const net::CompletionCallback& callback);
+  void DoomEntry(const std::string& key,
+                 const net::CompletionCallback& callback);
+  void DoomAllEntries(const net::CompletionCallback& callback);
   void DoomEntriesBetween(const base::Time initial_time,
                           const base::Time end_time,
-                          net::CompletionCallback* callback);
+                          const net::CompletionCallback& callback);
   void DoomEntriesSince(const base::Time initial_time,
-                        net::CompletionCallback* callback);
+                        const net::CompletionCallback& callback);
   void OpenNextEntry(void** iter, Entry** next_entry,
-                     net::CompletionCallback* callback);
+                     const net::CompletionCallback& callback);
   void OpenPrevEntry(void** iter, Entry** prev_entry,
-                     net::CompletionCallback* callback);
+                     const net::CompletionCallback& callback);
   void EndEnumeration(void* iterator);
+  void OnExternalCacheHit(const std::string& key);
   void CloseEntryImpl(EntryImpl* entry);
   void DoomEntryImpl(EntryImpl* entry);
-  void FlushQueue(net::CompletionCallback* callback);
-  void RunTask(Task* task, net::CompletionCallback* callback);
+  void FlushQueue(const net::CompletionCallback& callback);
+  void RunTask(const base::Closure& task,
+               const net::CompletionCallback& callback);
   void ReadData(EntryImpl* entry, int index, int offset, net::IOBuffer* buf,
-                int buf_len, net::CompletionCallback* callback);
-  void WriteData(EntryImpl* entry, int index, int offset, net::IOBuffer* buf,
-                 int buf_len, bool truncate, net::CompletionCallback* callback);
+                int buf_len, const net::CompletionCallback& callback);
+  void WriteData(
+      EntryImpl* entry, int index, int offset, net::IOBuffer* buf,
+      int buf_len, bool truncate, const net::CompletionCallback& callback);
   void ReadSparseData(EntryImpl* entry, int64 offset, net::IOBuffer* buf,
-                      int buf_len, net::CompletionCallback* callback);
+                      int buf_len, const net::CompletionCallback& callback);
   void WriteSparseData(EntryImpl* entry, int64 offset, net::IOBuffer* buf,
-                       int buf_len, net::CompletionCallback* callback);
+                       int buf_len, const net::CompletionCallback& callback);
   void GetAvailableRange(EntryImpl* entry, int64 offset, int len, int64* start,
-                         net::CompletionCallback* callback);
+                         const net::CompletionCallback& callback);
   void CancelSparseIO(EntryImpl* entry);
-  void ReadyForSparseIO(EntryImpl* entry, net::CompletionCallback* callback);
+  void ReadyForSparseIO(EntryImpl* entry,
+                        const net::CompletionCallback& callback);
 
   // Blocks until all operations are cancelled or completed.
   void WaitForPendingIO();
@@ -188,26 +196,15 @@ class InFlightBackendIO : public InFlightIO {
     return background_thread_->BelongsToCurrentThread();
   }
 
-  // Controls the queing of entry (async) operations.
-  void StartQueingOperations();
-  void StopQueingOperations();
-
  protected:
-  virtual void OnOperationComplete(BackgroundIO* operation, bool cancel);
+  virtual void OnOperationComplete(BackgroundIO* operation,
+                                   bool cancel) OVERRIDE;
 
  private:
-  typedef std::list<scoped_refptr<BackendIO> > OperationList;
-  void QueueOperation(BackendIO* operation);
   void PostOperation(BackendIO* operation);
-  void PostQueuedOperation();
-  bool RemoveFirstQueuedOperation(BackendIO* operation);
 
   BackendImpl* backend_;
   scoped_refptr<base::MessageLoopProxy> background_thread_;
-  OperationList pending_ops_;  // Entry (async) operations to be posted.
-  base::TimeTicks queueing_start_;
-  size_t max_queue_len_;
-  bool queue_entry_ops_;  // True if we are queuing entry (async) operations.
 
   DISALLOW_COPY_AND_ASSIGN(InFlightBackendIO);
 };
