@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,11 @@
 #define NET_DISK_CACHE_MEM_ENTRY_IMPL_H_
 #pragma once
 
+#include "base/gtest_prod_util.h"
 #include "base/hash_tables.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/net_log.h"
 #include "net/disk_cache/disk_cache.h"
-#include "testing/gtest/include/gtest/gtest_prod.h"
 
 namespace disk_cache {
 
@@ -54,7 +55,7 @@ class MemEntryImpl : public Entry {
 
   // Performs the initialization of a EntryImpl that will be added to the
   // cache.
-  bool CreateEntry(const std::string& key);
+  bool CreateEntry(const std::string& key, net::NetLog* net_log);
 
   // Permanently destroys this entry.
   void InternalDoom();
@@ -82,27 +83,40 @@ class MemEntryImpl : public Entry {
     return parent_ ? kChildEntry : kParentEntry;
   }
 
+  std::string& key() {
+    return key_;
+  }
+
+  net::BoundNetLog& net_log() {
+    return net_log_;
+  }
+
   // Entry interface.
-  virtual void Doom();
-  virtual void Close();
-  virtual std::string GetKey() const;
-  virtual base::Time GetLastUsed() const;
-  virtual base::Time GetLastModified() const;
-  virtual int32 GetDataSize(int index) const;
-  virtual int ReadData(int index, int offset, net::IOBuffer* buf, int buf_len,
-                       net::CompletionCallback* completion_callback);
-  virtual int WriteData(int index, int offset, net::IOBuffer* buf, int buf_len,
-                        net::CompletionCallback* completion_callback,
-                        bool truncate);
-  virtual int ReadSparseData(int64 offset, net::IOBuffer* buf, int buf_len,
-                             net::CompletionCallback* completion_callback);
-  virtual int WriteSparseData(int64 offset, net::IOBuffer* buf, int buf_len,
-                              net::CompletionCallback* completion_callback);
-  virtual int GetAvailableRange(int64 offset, int len, int64* start,
-                                CompletionCallback* callback);
-  virtual bool CouldBeSparse() const;
-  virtual void CancelSparseIO() {}
-  virtual int ReadyForSparseIO(net::CompletionCallback* completion_callback);
+  virtual void Doom() OVERRIDE;
+  virtual void Close() OVERRIDE;
+  virtual std::string GetKey() const OVERRIDE;
+  virtual base::Time GetLastUsed() const OVERRIDE;
+  virtual base::Time GetLastModified() const OVERRIDE;
+  virtual int32 GetDataSize(int index) const OVERRIDE;
+  virtual int ReadData(
+      int index, int offset, net::IOBuffer* buf, int buf_len,
+      const net::CompletionCallback& callback) OVERRIDE;
+  virtual int WriteData(
+      int index, int offset, net::IOBuffer* buf, int buf_len,
+      const net::CompletionCallback& callback, bool truncate) OVERRIDE;
+  virtual int ReadSparseData(
+      int64 offset, net::IOBuffer* buf, int buf_len,
+      const net::CompletionCallback& callback) OVERRIDE;
+  virtual int WriteSparseData(
+      int64 offset, net::IOBuffer* buf, int buf_len,
+      const net::CompletionCallback& callback) OVERRIDE;
+  virtual int GetAvailableRange(
+      int64 offset, int len, int64* start,
+      const net::CompletionCallback& callback) OVERRIDE;
+  virtual bool CouldBeSparse() const OVERRIDE;
+  virtual void CancelSparseIO() OVERRIDE {}
+  virtual int ReadyForSparseIO(
+      const net::CompletionCallback& callback) OVERRIDE;
 
  private:
   typedef base::hash_map<int, MemEntryImpl*> EntryMap;
@@ -111,7 +125,15 @@ class MemEntryImpl : public Entry {
     NUM_STREAMS = 3
   };
 
-  ~MemEntryImpl();
+  virtual ~MemEntryImpl();
+
+  // Do all the work for corresponding public functions.  Implemented as
+  // separate functions to make logging of results simpler.
+  int InternalReadData(int index, int offset, net::IOBuffer* buf, int buf_len);
+  int InternalWriteData(int index, int offset, net::IOBuffer* buf, int buf_len,
+                        bool truncate);
+  int InternalReadSparseData(int64 offset, net::IOBuffer* buf, int buf_len);
+  int InternalWriteSparseData(int64 offset, net::IOBuffer* buf, int buf_len);
 
   // Old Entry interface.
   int GetAvailableRange(int64 offset, int len, int64* start);
@@ -129,7 +151,7 @@ class MemEntryImpl : public Entry {
   // Performs the initialization of a MemEntryImpl as a child entry.
   // |parent| is the pointer to the parent entry. |child_id| is the ID of
   // the new child.
-  bool InitChildEntry(MemEntryImpl* parent, int child_id);
+  bool InitChildEntry(MemEntryImpl* parent, int child_id, net::NetLog* net_log);
 
   // Returns an entry responsible for |offset|. The returned entry can be a
   // child entry or this entry itself if |offset| points to the first range.
@@ -163,6 +185,8 @@ class MemEntryImpl : public Entry {
   MemBackendImpl* backend_;   // Back pointer to the cache.
   bool doomed_;               // True if this entry was removed from the cache.
 
+  net::BoundNetLog net_log_;
+
   DISALLOW_COPY_AND_ASSIGN(MemEntryImpl);
 };
 
