@@ -1,9 +1,10 @@
-// Copyright (c) 2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/disk_cache/stats_histogram.h"
 
+#include "base/debug/leak_annotations.h"
 #include "base/logging.h"
 #include "net/disk_cache/stats.h"
 
@@ -21,23 +22,22 @@ StatsHistogram::~StatsHistogram() {
     stats_ = NULL;
 }
 
-scoped_refptr<StatsHistogram> StatsHistogram::StatsHistogramFactoryGet(
-    const std::string& name) {
-  scoped_refptr<Histogram> histogram(NULL);
+StatsHistogram* StatsHistogram::FactoryGet(const std::string& name) {
+  Histogram* histogram(NULL);
 
   Sample minimum = 1;
   Sample maximum = disk_cache::Stats::kDataSizesLength - 1;
   size_t bucket_count = disk_cache::Stats::kDataSizesLength;
 
   if (StatisticsRecorder::FindHistogram(name, &histogram)) {
-    DCHECK(histogram.get() != NULL);
+    DCHECK(histogram != NULL);
   } else {
-    histogram = new StatsHistogram(name, minimum, maximum, bucket_count);
-    scoped_refptr<Histogram> registered_histogram(NULL);
-    StatisticsRecorder::FindHistogram(name, &registered_histogram);
-    if (registered_histogram.get() != NULL &&
-        registered_histogram.get() != histogram.get())
-      histogram = registered_histogram;
+    // To avoid racy destruction at shutdown, the following will be leaked.
+    StatsHistogram* stats_histogram =
+        new StatsHistogram(name, minimum, maximum, bucket_count);
+    stats_histogram->InitializeBucketRange();
+    stats_histogram->SetFlags(kUmaTargetedHistogramFlag);
+    histogram = StatisticsRecorder::RegisterOrDeleteDuplicate(stats_histogram);
   }
 
   DCHECK(HISTOGRAM == histogram->histogram_type());
@@ -45,10 +45,10 @@ scoped_refptr<StatsHistogram> StatsHistogram::StatsHistogramFactoryGet(
 
   // We're preparing for an otherwise unsafe upcast by ensuring we have the
   // proper class type.
-  Histogram* temp_histogram = histogram.get();
-  StatsHistogram* temp_stats_histogram =
-      static_cast<StatsHistogram*>(temp_histogram);
-  scoped_refptr<StatsHistogram> return_histogram(temp_stats_histogram);
+  StatsHistogram* return_histogram = static_cast<StatsHistogram*>(histogram);
+  // Validate upcast by seeing that we're probably providing the checksum.
+  CHECK_EQ(return_histogram->StatsHistogram::CalculateRangeChecksum(),
+           return_histogram->CalculateRangeChecksum());
   return return_histogram;
 }
 
@@ -57,8 +57,6 @@ bool StatsHistogram::Init(const Stats* stats) {
   if (stats_)
     return false;
 
-  SetFlags(kUmaTargetedHistogramFlag);
-
   // We support statistics report for only one cache.
   init_ = true;
   stats_ = stats;
@@ -91,5 +89,10 @@ Histogram::Inconsistencies StatsHistogram::FindCorruption(
   return NO_INCONSISTENCIES;  // This class won't monitor inconsistencies.
 }
 
+uint32 StatsHistogram::CalculateRangeChecksum() const {
+  // We don't calculate checksums, so at least establish a unique constant.
+  const uint32 kStatsHistogramChecksum = 0x0cecce;
+  return kStatsHistogramChecksum;
+}
 
 }  // namespace disk_cache
