@@ -1,4 +1,4 @@
-// Copyright (c) 2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -30,21 +30,22 @@ class StatsHistogram : public base::Histogram {
     }
   };
 
-  explicit StatsHistogram(const std::string& name, Sample minimum,
-                          Sample maximum, size_t bucket_count)
+  StatsHistogram(const std::string& name, Sample minimum,
+                 Sample maximum, size_t bucket_count)
       : Histogram(name, minimum, maximum, bucket_count), init_(false) {}
-  ~StatsHistogram();
+  virtual ~StatsHistogram();
 
-  static scoped_refptr<StatsHistogram>
-      StatsHistogramFactoryGet(const std::string& name);
+  static StatsHistogram* FactoryGet(const std::string& name);
 
   // We'll be reporting data from the given set of cache stats.
   bool Init(const Stats* stats);
 
-  virtual Sample ranges(size_t i) const;
-  virtual size_t bucket_count() const;
-  virtual void SnapshotSample(SampleSet* sample) const;
-  virtual Inconsistencies FindCorruption(const SampleSet& snapshot) const;
+  virtual Sample ranges(size_t i) const OVERRIDE;
+  virtual size_t bucket_count() const OVERRIDE;
+  virtual void SnapshotSample(SampleSet* sample) const OVERRIDE;
+  virtual Inconsistencies FindCorruption(
+      const SampleSet& snapshot) const OVERRIDE;
+  virtual uint32 CalculateRangeChecksum() const OVERRIDE;
 
  private:
   bool init_;
