@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,7 +11,8 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "net/base/net_export.h"
 
 namespace disk_cache {
 
@@ -25,6 +26,7 @@ class TraceObject : public base::RefCounted<TraceObject> {
   friend class base::RefCounted<TraceObject>;
  public:
   static TraceObject* GetTraceObject();
+  void EnableTracing(bool enable);
 
  private:
   TraceObject();
@@ -33,7 +35,7 @@ class TraceObject : public base::RefCounted<TraceObject> {
 };
 
 // Traces to the internal buffer.
-void Trace(const char* format, ...);
+NET_EXPORT_PRIVATE void Trace(const char* format, ...);
 
 }  // namespace disk_cache
 
