@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,85 +9,124 @@
 #include "base/stringprintf.h"
 #include "net/ftp/ftp_directory_listing_parser_ls.h"
 
+namespace net {
+
 namespace {
 
-typedef net::FtpDirectoryListingParserTest FtpDirectoryListingParserLsTest;
+typedef FtpDirectoryListingParserTest FtpDirectoryListingParserLsTest;
 
 TEST_F(FtpDirectoryListingParserLsTest, Good) {
   const struct SingleLineTestData good_cases[] = {
     { "-rw-r--r--    1 ftp      ftp           528 Nov 01  2007 README",
-      net::FtpDirectoryListingEntry::FILE, "README", 528,
+      FtpDirectoryListingEntry::FILE, "README", 528,
       2007, 11, 1, 0, 0 },
     { "drwxr-xr-x    3 ftp      ftp          4096 May 15 18:11 directory",
-      net::FtpDirectoryListingEntry::DIRECTORY, "directory", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "directory", -1,
       1994, 5, 15, 18, 11 },
     { "lrwxrwxrwx 1 0  0 26 Sep 18 2008 pub -> vol/1/.CLUSTER/var_ftp/pub",
-      net::FtpDirectoryListingEntry::SYMLINK, "pub", -1,
+      FtpDirectoryListingEntry::SYMLINK, "pub", -1,
       2008, 9, 18, 0, 0 },
     { "lrwxrwxrwx 1 0  0 3 Oct 12 13:37 mirror -> pub",
-      net::FtpDirectoryListingEntry::SYMLINK, "mirror", -1,
+      FtpDirectoryListingEntry::SYMLINK, "mirror", -1,
       1994, 10, 12, 13, 37 },
     { "drwxrwsr-x    4 501      501          4096 Feb 20  2007 pub",
-      net::FtpDirectoryListingEntry::DIRECTORY, "pub", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "pub", -1,
       2007, 2, 20, 0, 0 },
     { "drwxr-xr-x   4 (?)      (?)          4096 Apr  8  2007 jigdo",
-      net::FtpDirectoryListingEntry::DIRECTORY, "jigdo", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "jigdo", -1,
       2007, 4, 8, 0, 0 },
     { "drwx-wx-wt  2 root  wheel  512 Jul  1 02:15 incoming",
-      net::FtpDirectoryListingEntry::DIRECTORY, "incoming", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "incoming", -1,
       1994, 7, 1, 2, 15 },
     { "-rw-r--r-- 1 2 3 3447432 May 18  2009 Foo - Manual.pdf",
-      net::FtpDirectoryListingEntry::FILE, "Foo - Manual.pdf", 3447432,
+      FtpDirectoryListingEntry::FILE, "Foo - Manual.pdf", 3447432,
       2009, 5, 18, 0, 0 },
     { "d-wx-wx-wt+  4 ftp      989          512 Dec  8 15:54 incoming",
-      net::FtpDirectoryListingEntry::DIRECTORY, "incoming", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "incoming", -1,
       1993, 12, 8, 15, 54 },
-    { "drwxrwxrwx   1 owner    group               0 Sep 13  0:30 audio",
-      net::FtpDirectoryListingEntry::DIRECTORY, "audio", -1,
+    { "drwxrwxrwx   1 owner    group               1024 Sep 13  0:30 audio",
+      FtpDirectoryListingEntry::DIRECTORY, "audio", -1,
       1994, 9, 13, 0, 30 },
     { "lrwxrwxrwx 1 0  0 26 Sep 18 2008 pub",
-      net::FtpDirectoryListingEntry::SYMLINK, "pub", -1,
+      FtpDirectoryListingEntry::SYMLINK, "pub", -1,
       2008, 9, 18, 0, 0 },
+    { "-rw-r--r--    1 ftp      ftp           -528 Nov 01  2007 README",
+      FtpDirectoryListingEntry::FILE, "README", -1,
+      2007, 11, 1, 0, 0 },
 
     // Tests for the wu-ftpd variant:
     { "drwxr-xr-x   2 sys          512 Mar 27  2009 pub",
-      net::FtpDirectoryListingEntry::DIRECTORY, "pub", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "pub", -1,
       2009, 3, 27, 0, 0 },
     { "lrwxrwxrwx 0  0 26 Sep 18 2008 pub -> vol/1/.CLUSTER/var_ftp/pub",
-      net::FtpDirectoryListingEntry::SYMLINK, "pub", -1,
+      FtpDirectoryListingEntry::SYMLINK, "pub", -1,
       2008, 9, 18, 0, 0 },
     { "drwxr-xr-x   (?)      (?)          4096 Apr  8  2007 jigdo",
-      net::FtpDirectoryListingEntry::DIRECTORY, "jigdo", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "jigdo", -1,
       2007, 4, 8, 0, 0 },
     { "-rw-r--r-- 2 3 3447432 May 18  2009 Foo - Manual.pdf",
-      net::FtpDirectoryListingEntry::FILE, "Foo - Manual.pdf", 3447432,
+      FtpDirectoryListingEntry::FILE, "Foo - Manual.pdf", 3447432,
       2009, 5, 18, 0, 0 },
 
     // Tests for "ls -l" style listings sent by an OS/2 server (FtpServer):
     { "-r--r--r--  1 ftp      -A---       13274 Mar  1  2006 UpTime.exe",
-      net::FtpDirectoryListingEntry::FILE, "UpTime.exe", 13274,
+      FtpDirectoryListingEntry::FILE, "UpTime.exe", 13274,
       2006, 3, 1, 0, 0 },
     { "dr--r--r--  1 ftp      -----           0 Nov 17 17:08 kernels",
-      net::FtpDirectoryListingEntry::DIRECTORY, "kernels", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "kernels", -1,
       1993, 11, 17, 17, 8 },
 
     // Tests for "ls -l" style listing sent by Xplain FTP Server.
     { "drwxr-xr-x               folder        0 Jul 17  2006 online",
-      net::FtpDirectoryListingEntry::DIRECTORY, "online", -1,
+      FtpDirectoryListingEntry::DIRECTORY, "online", -1,
       2006, 7, 17, 0, 0 },
 
     // Tests for "ls -l" style listing with owning group name
     // not separated from file size (http://crbug.com/58963).
     { "-rw-r--r-- 1 ftpadmin ftpadmin125435904 Apr  9  2008 .pureftpd-upload",
-      net::FtpDirectoryListingEntry::FILE, ".pureftpd-upload", 0,
+      FtpDirectoryListingEntry::FILE, ".pureftpd-upload", 0,
       2008, 4, 9, 0, 0 },
+
+    // Tests for "ls -l" style listing with number of links
+    // not separated from permission listing (http://crbug.com/70394).
+    { "drwxr-xr-x1732 266      111        90112 Jun 21  2001 .rda_2",
+      FtpDirectoryListingEntry::DIRECTORY, ".rda_2", -1,
+      2001, 6, 21, 0, 0 },
+
+    // Tests for "ls -l" style listing with group name containing spaces.
+    { "drwxrwxr-x   3 %%%%     Domain Users     4096 Dec  9  2009 %%%%%",
+      FtpDirectoryListingEntry::DIRECTORY, "%%%%%", -1,
+      2009, 12, 9, 0, 0 },
+
+    // Tests for "ls -l" style listing in Russian locale (note the swapped
+    // parts order: the day of month is the first, before month).
+    { "-rwxrwxr-x 1 ftp ftp 123 23 \xd0\xbc\xd0\xb0\xd0\xb9 2011 test",
+      FtpDirectoryListingEntry::FILE, "test", 123,
+      2011, 5, 23, 0, 0 },
+    { "drwxrwxr-x 1 ftp ftp 4096 19 \xd0\xbe\xd0\xba\xd1\x82 2011 dir",
+      FtpDirectoryListingEntry::DIRECTORY, "dir", -1,
+      2011, 10, 19, 0, 0 },
+
+    // Plan9 sends entry type "a" for append-only files.
+    { "ar-xr-xr-x   2 none     none         512 Apr 26 17:52 plan9",
+      FtpDirectoryListingEntry::FILE, "plan9", 512,
+      1994, 4, 26, 17, 52 },
+
+    // Hylafax sends a shorter permission listing.
+    { "drwxrwx   2       10     4096 Jul 28 02:41 tmp",
+      FtpDirectoryListingEntry::DIRECTORY, "tmp", -1,
+      1994, 7, 28, 2, 41 },
   };
   for (size_t i = 0; i < arraysize(good_cases); i++) {
     SCOPED_TRACE(base::StringPrintf("Test[%" PRIuS "]: %s", i,
                                     good_cases[i].input));
 
-    net::FtpDirectoryListingParserLs parser(GetMockCurrentTime());
-    RunSingleLineTestCase(&parser, good_cases[i]);
+    std::vector<FtpDirectoryListingEntry> entries;
+    EXPECT_TRUE(ParseFtpDirectoryListingLs(
+        GetSingleLineTestCase(good_cases[i].input),
+        GetMockCurrentTime(),
+        &entries));
+    VerifySingleLineTestCase(good_cases[i], entries);
   }
 }
 
@@ -97,6 +136,7 @@ TEST_F(FtpDirectoryListingParserLsTest, Ignored) {
 
     "ftpd: .: Permission denied",
     "ftpd-BSD: .: Permission denied",
+    "ls: .: EDC5111I Permission denied.",
 
     // Tests important for security: verify that after we detect the column
     // offset we don't try to access invalid memory on malformed input.
@@ -108,11 +148,12 @@ TEST_F(FtpDirectoryListingParserLsTest, Ignored) {
     SCOPED_TRACE(base::StringPrintf("Test[%" PRIuS "]: %s", i,
                                     ignored_cases[i]));
 
-    net::FtpDirectoryListingParserLs parser(GetMockCurrentTime());
-    EXPECT_TRUE(parser.ConsumeLine(UTF8ToUTF16(ignored_cases[i])));
-    EXPECT_FALSE(parser.EntryAvailable());
-    EXPECT_TRUE(parser.OnEndOfInput());
-    EXPECT_FALSE(parser.EntryAvailable());
+    std::vector<FtpDirectoryListingEntry> entries;
+    EXPECT_TRUE(ParseFtpDirectoryListingLs(
+                    GetSingleLineTestCase(ignored_cases[i]),
+                    GetMockCurrentTime(),
+                    &entries));
+    EXPECT_EQ(0U, entries.size());
   }
 }
 
@@ -121,27 +162,28 @@ TEST_F(FtpDirectoryListingParserLsTest, Bad) {
     " foo",
     "garbage",
     "-rw-r--r-- ftp ftp",
-    "-rw-r--rgb ftp ftp 528 Nov 01 2007 README",
     "-rw-rgbr-- ftp ftp 528 Nov 01 2007 README",
     "qrwwr--r-- ftp ftp 528 Nov 01 2007 README",
-    "-rw-r--r-- ftp ftp -528 Nov 01 2007 README",
     "-rw-r--r-- ftp ftp 528 Foo 01 2007 README",
     "-rw-r--r-- 1 ftp ftp",
-    "-rw-r--rgb 1 ftp ftp 528 Nov 01 2007 README",
     "-rw-rgbr-- 1 ftp ftp 528 Nov 01 2007 README",
     "qrwwr--r-- 1 ftp ftp 528 Nov 01 2007 README",
-    "-rw-r--r-- 1 ftp ftp -528 Nov 01 2007 README",
     "-rw-r--r-- 1 ftp ftp 528 Foo 01 2007 README",
-    "drwxrwxrwx   1 owner    group               0 Sep 13  0:3 audio",
+    "drwxrwxrwx   1 owner    group               1024 Sep 13  0:3 audio",
 
-    "d-wx-wx-wt++  4 ftp 989 512 Dec  8 15:54 incoming",
-    "d-wx-wx-wt$  4 ftp 989 512 Dec  8 15:54 incoming",
     "-qqqqqqqqq+  2 sys          512 Mar 27  2009 pub",
   };
   for (size_t i = 0; i < arraysize(bad_cases); i++) {
-    net::FtpDirectoryListingParserLs parser(GetMockCurrentTime());
-    EXPECT_FALSE(parser.ConsumeLine(UTF8ToUTF16(bad_cases[i]))) << bad_cases[i];
+    SCOPED_TRACE(base::StringPrintf("Test[%" PRIuS "]: %s", i,
+                                    bad_cases[i]));
+
+    std::vector<FtpDirectoryListingEntry> entries;
+    EXPECT_FALSE(ParseFtpDirectoryListingLs(GetSingleLineTestCase(bad_cases[i]),
+                                            GetMockCurrentTime(),
+                                            &entries));
   }
 }
 
 }  // namespace
+
+}  // namespace net
