@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,43 +9,49 @@
 #include <string>
 #include <utility>
 
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/string16.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "net/base/address_list.h"
+#include "net/base/auth.h"
 #include "net/base/host_resolver.h"
 #include "net/base/net_log.h"
+#include "net/base/single_request_host_resolver.h"
 #include "net/ftp/ftp_ctrl_response_buffer.h"
 #include "net/ftp/ftp_response_info.h"
 #include "net/ftp/ftp_transaction.h"
 
 namespace net {
 
-class ClientSocket;
 class ClientSocketFactory;
 class FtpNetworkSession;
+class StreamSocket;
 
-class FtpNetworkTransaction : public FtpTransaction {
+class NET_EXPORT_PRIVATE FtpNetworkTransaction : public FtpTransaction {
  public:
   FtpNetworkTransaction(FtpNetworkSession* session,
                         ClientSocketFactory* socket_factory);
   virtual ~FtpNetworkTransaction();
 
+  virtual int Stop(int error);
+  virtual int RestartIgnoringLastError(const CompletionCallback& callback);
+
   // FtpTransaction methods:
   virtual int Start(const FtpRequestInfo* request_info,
-                    CompletionCallback* callback,
-                    const BoundNetLog& net_log);
-  virtual int Stop(int error);
-  virtual int RestartWithAuth(const string16& username,
-                              const string16& password,
-                              CompletionCallback* callback);
-  virtual int RestartIgnoringLastError(CompletionCallback* callback);
-  virtual int Read(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual const FtpResponseInfo* GetResponseInfo() const;
-  virtual LoadState GetLoadState() const;
-  virtual uint64 GetUploadProgress() const;
+                    const CompletionCallback& callback,
+                    const BoundNetLog& net_log) OVERRIDE;
+  virtual int RestartWithAuth(const AuthCredentials& credentials,
+                              const CompletionCallback& callback) OVERRIDE;
+  virtual int Read(IOBuffer* buf, int buf_len,
+                   const CompletionCallback& callback) OVERRIDE;
+  virtual const FtpResponseInfo* GetResponseInfo() const OVERRIDE;
+  virtual LoadState GetLoadState() const OVERRIDE;
+  virtual uint64 GetUploadProgress() const OVERRIDE;
 
  private:
+  FRIEND_TEST_ALL_PREFIXES(FtpNetworkTransactionTest,
+                           DownloadTransactionEvilPasvUnsafeHost);
+
   enum Command {
     COMMAND_NONE,
     COMMAND_USER,
@@ -87,6 +93,36 @@ class FtpNetworkTransaction : public FtpTransaction {
     RESOURCE_TYPE_DIRECTORY,
   };
 
+  enum State {
+    // Control connection states:
+    STATE_CTRL_RESOLVE_HOST,
+    STATE_CTRL_RESOLVE_HOST_COMPLETE,
+    STATE_CTRL_CONNECT,
+    STATE_CTRL_CONNECT_COMPLETE,
+    STATE_CTRL_READ,
+    STATE_CTRL_READ_COMPLETE,
+    STATE_CTRL_WRITE,
+    STATE_CTRL_WRITE_COMPLETE,
+    STATE_CTRL_WRITE_USER,
+    STATE_CTRL_WRITE_PASS,
+    STATE_CTRL_WRITE_SYST,
+    STATE_CTRL_WRITE_TYPE,
+    STATE_CTRL_WRITE_EPSV,
+    STATE_CTRL_WRITE_PASV,
+    STATE_CTRL_WRITE_PWD,
+    STATE_CTRL_WRITE_RETR,
+    STATE_CTRL_WRITE_SIZE,
+    STATE_CTRL_WRITE_CWD,
+    STATE_CTRL_WRITE_LIST,
+    STATE_CTRL_WRITE_QUIT,
+    // Data connection states:
+    STATE_DATA_CONNECT,
+    STATE_DATA_CONNECT_COMPLETE,
+    STATE_DATA_READ,
+    STATE_DATA_READ_COMPLETE,
+    STATE_NONE
+  };
+
   // Resets the members of the transaction so it can be restarted.
   void ResetStateForRestart();
 
@@ -141,6 +177,7 @@ class FtpNetworkTransaction : public FtpTransaction {
   int ProcessResponseSIZE(const FtpCtrlResponse& response);
   int DoCtrlWriteCWD();
   int ProcessResponseCWD(const FtpCtrlResponse& response);
+  int ProcessResponseCWDNotADirectory();
   int DoCtrlWriteLIST();
   int ProcessResponseLIST(const FtpCtrlResponse& response);
   int DoCtrlWriteQUIT();
@@ -155,8 +192,8 @@ class FtpNetworkTransaction : public FtpTransaction {
 
   Command command_sent_;
 
-  CompletionCallbackImpl<FtpNetworkTransaction> io_callback_;
-  CompletionCallback* user_callback_;
+  CompletionCallback io_callback_;
+  CompletionCallback user_callback_;
 
   scoped_refptr<FtpNetworkSession> session_;
 
@@ -197,8 +234,7 @@ class FtpNetworkTransaction : public FtpTransaction {
   // EPSV fail, we fall back to PASV for the duration of connection.
   bool use_epsv_;
 
-  string16 username_;
-  string16 password_;
+  AuthCredentials credentials_;
 
   // Current directory on the remote server, as returned by last PWD command,
   // with any trailing slash removed.
@@ -208,38 +244,9 @@ class FtpNetworkTransaction : public FtpTransaction {
 
   ClientSocketFactory* socket_factory_;
 
-  scoped_ptr<ClientSocket> ctrl_socket_;
-  scoped_ptr<ClientSocket> data_socket_;
+  scoped_ptr<StreamSocket> ctrl_socket_;
+  scoped_ptr<StreamSocket> data_socket_;
 
-  enum State {
-    // Control connection states:
-    STATE_CTRL_RESOLVE_HOST,
-    STATE_CTRL_RESOLVE_HOST_COMPLETE,
-    STATE_CTRL_CONNECT,
-    STATE_CTRL_CONNECT_COMPLETE,
-    STATE_CTRL_READ,
-    STATE_CTRL_READ_COMPLETE,
-    STATE_CTRL_WRITE,
-    STATE_CTRL_WRITE_COMPLETE,
-    STATE_CTRL_WRITE_USER,
-    STATE_CTRL_WRITE_PASS,
-    STATE_CTRL_WRITE_SYST,
-    STATE_CTRL_WRITE_TYPE,
-    STATE_CTRL_WRITE_EPSV,
-    STATE_CTRL_WRITE_PASV,
-    STATE_CTRL_WRITE_PWD,
-    STATE_CTRL_WRITE_RETR,
-    STATE_CTRL_WRITE_SIZE,
-    STATE_CTRL_WRITE_CWD,
-    STATE_CTRL_WRITE_LIST,
-    STATE_CTRL_WRITE_QUIT,
-    // Data connection states:
-    STATE_DATA_CONNECT,
-    STATE_DATA_CONNECT_COMPLETE,
-    STATE_DATA_READ,
-    STATE_DATA_READ_COMPLETE,
-    STATE_NONE
-  };
   State next_state_;
 };
 
