@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,9 +6,11 @@
 
 #include "build/build_config.h"
 
-#include "base/ref_counted.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
+#include "net/base/host_port_pair.h"
 #include "net/base/io_buffer.h"
 #include "net/base/mock_host_resolver.h"
 #include "net/base/net_util.h"
@@ -56,7 +58,7 @@ class FtpSocketDataProvider : public DynamicSocketDataProvider {
     Init();
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -108,7 +110,7 @@ class FtpSocketDataProvider : public DynamicSocketDataProvider {
     return state_;
   }
 
-  virtual void Reset() {
+  virtual void Reset() OVERRIDE {
     DynamicSocketDataProvider::Reset();
     Init();
   }
@@ -180,7 +182,7 @@ class FtpSocketDataProviderDirectoryListing : public FtpSocketDataProvider {
   FtpSocketDataProviderDirectoryListing() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -206,7 +208,7 @@ class FtpSocketDataProviderDirectoryListingWithPasvFallback
   FtpSocketDataProviderDirectoryListingWithPasvFallback() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -232,7 +234,7 @@ class FtpSocketDataProviderDirectoryListingZeroSize
   FtpSocketDataProviderDirectoryListingZeroSize() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -252,7 +254,7 @@ class FtpSocketDataProviderVMSDirectoryListing : public FtpSocketDataProvider {
   FtpSocketDataProviderVMSDirectoryListing() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -289,7 +291,7 @@ class FtpSocketDataProviderVMSDirectoryListingRootDirectory
   FtpSocketDataProviderVMSDirectoryListingRootDirectory() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -328,7 +330,7 @@ class FtpSocketDataProviderFileDownloadWithFileTypecode
   FtpSocketDataProviderFileDownloadWithFileTypecode() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -351,7 +353,7 @@ class FtpSocketDataProviderFileDownload : public FtpSocketDataProvider {
   FtpSocketDataProviderFileDownload() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -377,7 +379,7 @@ class FtpSocketDataProviderFileNotFound : public FtpSocketDataProvider {
   FtpSocketDataProviderFileNotFound() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -405,7 +407,7 @@ class FtpSocketDataProviderFileDownloadWithPasvFallback
   FtpSocketDataProviderFileDownloadWithPasvFallback() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -430,7 +432,7 @@ class FtpSocketDataProviderFileDownloadZeroSize
   FtpSocketDataProviderFileDownloadZeroSize() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -449,12 +451,34 @@ class FtpSocketDataProviderFileDownloadZeroSize
   DISALLOW_COPY_AND_ASSIGN(FtpSocketDataProviderFileDownloadZeroSize);
 };
 
+class FtpSocketDataProviderFileDownloadCWD451
+    : public FtpSocketDataProviderFileDownload {
+ public:
+  FtpSocketDataProviderFileDownloadCWD451() {
+  }
+
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
+    if (InjectFault())
+      return MockWriteResult(true, data.length());
+    switch (state()) {
+      case PRE_CWD:
+        return Verify("CWD /file\r\n", data, PRE_RETR,
+                      "451 not a directory\r\n");
+      default:
+        return FtpSocketDataProviderFileDownload::OnWrite(data);
+    }
+  }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(FtpSocketDataProviderFileDownloadCWD451);
+};
+
 class FtpSocketDataProviderVMSFileDownload : public FtpSocketDataProvider {
  public:
   FtpSocketDataProviderVMSFileDownload() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -492,7 +516,7 @@ class FtpSocketDataProviderEscaping : public FtpSocketDataProviderFileDownload {
   FtpSocketDataProviderEscaping() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -520,7 +544,7 @@ class FtpSocketDataProviderFileDownloadTransferStarting
   FtpSocketDataProviderFileDownloadTransferStarting() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -544,7 +568,7 @@ class FtpSocketDataProviderDirectoryListingTransferStarting
   FtpSocketDataProviderDirectoryListingTransferStarting() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -569,7 +593,7 @@ class FtpSocketDataProviderFileDownloadInvalidResponse
   FtpSocketDataProviderFileDownloadInvalidResponse() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -603,7 +627,7 @@ class FtpSocketDataProviderEvilEpsv : public FtpSocketDataProviderFileDownload {
         epsv_response_length_(epsv_response_length),
         expected_state_(expected_state) {}
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -626,13 +650,12 @@ class FtpSocketDataProviderEvilEpsv : public FtpSocketDataProviderFileDownload {
 class FtpSocketDataProviderEvilPasv
     : public FtpSocketDataProviderFileDownloadWithPasvFallback {
  public:
-  explicit FtpSocketDataProviderEvilPasv(const char* pasv_response,
-                                         State expected_state)
+  FtpSocketDataProviderEvilPasv(const char* pasv_response, State expected_state)
       : pasv_response_(pasv_response),
         expected_state_(expected_state) {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -657,7 +680,7 @@ class FtpSocketDataProviderEvilSize : public FtpSocketDataProviderFileDownload {
         expected_state_(expected_state) {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -684,7 +707,7 @@ class FtpSocketDataProviderEvilLogin
         expected_password_(expected_password) {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -711,7 +734,7 @@ class FtpSocketDataProviderCloseConnection : public FtpSocketDataProvider {
   FtpSocketDataProviderCloseConnection() {
   }
 
-  virtual MockWriteResult OnWrite(const std::string& data) {
+  virtual MockWriteResult OnWrite(const std::string& data) OVERRIDE {
     if (InjectFault())
       return MockWriteResult(true, data.length());
     switch (state()) {
@@ -759,20 +782,23 @@ class FtpNetworkTransactionTest : public PlatformTest {
     FtpRequestInfo request_info = GetRequestInfo(request);
     EXPECT_EQ(LOAD_STATE_IDLE, transaction_.GetLoadState());
     ASSERT_EQ(ERR_IO_PENDING,
-              transaction_.Start(&request_info, &callback_, BoundNetLog()));
+              transaction_.Start(&request_info, callback_.callback(),
+                                 BoundNetLog()));
     EXPECT_NE(LOAD_STATE_IDLE, transaction_.GetLoadState());
     ASSERT_EQ(expected_result, callback_.WaitForResult());
     if (expected_result == OK) {
       scoped_refptr<IOBuffer> io_buffer(new IOBuffer(kBufferSize));
       memset(io_buffer->data(), 0, kBufferSize);
       ASSERT_EQ(ERR_IO_PENDING,
-                transaction_.Read(io_buffer.get(), kBufferSize, &callback_));
+                transaction_.Read(io_buffer.get(), kBufferSize,
+                                  callback_.callback()));
       ASSERT_EQ(static_cast<int>(mock_data.length()),
                 callback_.WaitForResult());
       EXPECT_EQ(mock_data, std::string(io_buffer->data(), mock_data.length()));
 
       // Do another Read to detect that the data socket is now closed.
-      int rv = transaction_.Read(io_buffer.get(), kBufferSize, &callback_);
+      int rv = transaction_.Read(io_buffer.get(), kBufferSize,
+                                 callback_.callback());
       if (rv == ERR_IO_PENDING) {
         EXPECT_EQ(0, callback_.WaitForResult());
       } else {
@@ -805,7 +831,8 @@ TEST_F(FtpNetworkTransactionTest, FailedLookup) {
   host_resolver_->rules()->AddSimulatedFailure("badhost");
   EXPECT_EQ(LOAD_STATE_IDLE, transaction_.GetLoadState());
   ASSERT_EQ(ERR_IO_PENDING,
-            transaction_.Start(&request_info, &callback_, BoundNetLog()));
+            transaction_.Start(&request_info, callback_.callback(),
+                               BoundNetLog()));
   ASSERT_EQ(ERR_NAME_NOT_RESOLVED, callback_.WaitForResult());
   EXPECT_EQ(LOAD_STATE_IDLE, transaction_.GetLoadState());
 }
@@ -831,6 +858,9 @@ TEST_F(FtpNetworkTransactionTest, DirectoryTransaction) {
 
   EXPECT_TRUE(transaction_.GetResponseInfo()->is_directory_listing);
   EXPECT_EQ(-1, transaction_.GetResponseInfo()->expected_content_size);
+  EXPECT_EQ("192.0.2.33",
+            transaction_.GetResponseInfo()->socket_address.host());
+  EXPECT_EQ(0, transaction_.GetResponseInfo()->socket_address.port());
 }
 
 TEST_F(FtpNetworkTransactionTest, DirectoryTransactionWithPasvFallback) {
@@ -904,6 +934,9 @@ TEST_F(FtpNetworkTransactionTest, DownloadTransaction) {
 
   // We pass an artificial value of 18 as a response to the SIZE command.
   EXPECT_EQ(18, transaction_.GetResponseInfo()->expected_content_size);
+  EXPECT_EQ("192.0.2.33",
+            transaction_.GetResponseInfo()->socket_address.host());
+  EXPECT_EQ(0, transaction_.GetResponseInfo()->socket_address.port());
 }
 
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionWithPasvFallback) {
@@ -954,6 +987,11 @@ TEST_F(FtpNetworkTransactionTest, DownloadTransactionZeroSize) {
   ExecuteTransaction(&ctrl_socket, "ftp://host/file", OK);
 }
 
+TEST_F(FtpNetworkTransactionTest, DownloadTransactionCWD451) {
+  FtpSocketDataProviderFileDownloadCWD451 ctrl_socket;
+  ExecuteTransaction(&ctrl_socket, "ftp://host/file", OK);
+}
+
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionVMS) {
   FtpSocketDataProviderVMSFileDownload ctrl_socket;
   ExecuteTransaction(&ctrl_socket, "ftp://host/file", OK);
@@ -971,40 +1009,40 @@ TEST_F(FtpNetworkTransactionTest, DownloadTransactionInvalidResponse) {
 
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvReallyBadFormat) {
   FtpSocketDataProviderEvilPasv ctrl_socket("227 Portscan (127,0,0,\r\n",
-                                           FtpSocketDataProvider::PRE_QUIT);
+                                            FtpSocketDataProvider::PRE_QUIT);
   ExecuteTransaction(&ctrl_socket, "ftp://host/file", ERR_INVALID_RESPONSE);
 }
 
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvUnsafePort1) {
   FtpSocketDataProviderEvilPasv ctrl_socket("227 Portscan (127,0,0,1,0,22)\r\n",
-                                           FtpSocketDataProvider::PRE_QUIT);
+                                            FtpSocketDataProvider::PRE_QUIT);
   ExecuteTransaction(&ctrl_socket, "ftp://host/file", ERR_UNSAFE_PORT);
 }
 
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvUnsafePort2) {
   // Still unsafe. 1 * 256 + 2 = 258, which is < 1024.
   FtpSocketDataProviderEvilPasv ctrl_socket("227 Portscan (127,0,0,1,1,2)\r\n",
-                                           FtpSocketDataProvider::PRE_QUIT);
+                                            FtpSocketDataProvider::PRE_QUIT);
   ExecuteTransaction(&ctrl_socket, "ftp://host/file", ERR_UNSAFE_PORT);
 }
 
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvUnsafePort3) {
   // Still unsafe. 3 * 256 + 4 = 772, which is < 1024.
   FtpSocketDataProviderEvilPasv ctrl_socket("227 Portscan (127,0,0,1,3,4)\r\n",
-                                           FtpSocketDataProvider::PRE_QUIT);
+                                            FtpSocketDataProvider::PRE_QUIT);
   ExecuteTransaction(&ctrl_socket, "ftp://host/file", ERR_UNSAFE_PORT);
 }
 
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvUnsafePort4) {
   // Unsafe. 8 * 256 + 1 = 2049, which is used by nfs.
   FtpSocketDataProviderEvilPasv ctrl_socket("227 Portscan (127,0,0,1,8,1)\r\n",
-                                           FtpSocketDataProvider::PRE_QUIT);
+                                            FtpSocketDataProvider::PRE_QUIT);
   ExecuteTransaction(&ctrl_socket, "ftp://host/file", ERR_UNSAFE_PORT);
 }
 
 TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvUnsafeHost) {
   FtpSocketDataProviderEvilPasv ctrl_socket(
-      "227 Portscan (10,1,2,3,4,123,456)\r\n", FtpSocketDataProvider::PRE_SIZE);
+      "227 Portscan (10,1,2,3,123,456)\r\n", FtpSocketDataProvider::PRE_SIZE);
   std::string mock_data("mock-data");
   MockRead data_reads[] = {
     MockRead(mock_data.c_str()),
@@ -1017,13 +1055,14 @@ TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvUnsafeHost) {
 
   // Start the transaction.
   ASSERT_EQ(ERR_IO_PENDING,
-            transaction_.Start(&request_info, &callback_, BoundNetLog()));
+            transaction_.Start(&request_info, callback_.callback(),
+                               BoundNetLog()));
   ASSERT_EQ(OK, callback_.WaitForResult());
 
   // The transaction fires the callback when we can start reading data. That
   // means that the data socket should be open.
   MockTCPClientSocket* data_socket =
-      mock_socket_factory_.GetMockTCPClientSocket(1);
+      static_cast<MockTCPClientSocket*>(transaction_.data_socket_.get());
   ASSERT_TRUE(data_socket);
   ASSERT_TRUE(data_socket->IsConnected());
 
@@ -1032,7 +1071,7 @@ TEST_F(FtpNetworkTransactionTest, DownloadTransactionEvilPasvUnsafeHost) {
   // or ::1 depending on whether we use IPv6).
   const struct addrinfo* addrinfo = data_socket->addresses().head();
   while (addrinfo) {
-    EXPECT_NE("1.2.3.4", NetAddressToString(addrinfo));
+    EXPECT_NE("10.1.2.3", NetAddressToString(addrinfo));
     addrinfo = addrinfo->ai_next;
   }
 }
@@ -1141,7 +1180,8 @@ TEST_F(FtpNetworkTransactionTest, EvilRestartUser) {
   FtpRequestInfo request_info = GetRequestInfo("ftp://host/file");
 
   ASSERT_EQ(ERR_IO_PENDING,
-            transaction_.Start(&request_info, &callback_, BoundNetLog()));
+            transaction_.Start(&request_info, callback_.callback(),
+                               BoundNetLog()));
   ASSERT_EQ(ERR_FTP_FAILED, callback_.WaitForResult());
 
   MockRead ctrl_reads[] = {
@@ -1156,9 +1196,11 @@ TEST_F(FtpNetworkTransactionTest, EvilRestartUser) {
                                         ctrl_writes, arraysize(ctrl_writes));
   mock_socket_factory_.AddSocketDataProvider(&ctrl_socket2);
   ASSERT_EQ(ERR_IO_PENDING,
-            transaction_.RestartWithAuth(ASCIIToUTF16("foo\nownz0red"),
-                                         ASCIIToUTF16("innocent"),
-                                         &callback_));
+            transaction_.RestartWithAuth(
+                AuthCredentials(
+                    ASCIIToUTF16("foo\nownz0red"),
+                    ASCIIToUTF16("innocent")),
+                callback_.callback()));
   EXPECT_EQ(ERR_MALFORMED_IDENTITY, callback_.WaitForResult());
 }
 
@@ -1172,7 +1214,8 @@ TEST_F(FtpNetworkTransactionTest, EvilRestartPassword) {
   FtpRequestInfo request_info = GetRequestInfo("ftp://host/file");
 
   ASSERT_EQ(ERR_IO_PENDING,
-            transaction_.Start(&request_info, &callback_, BoundNetLog()));
+            transaction_.Start(&request_info, callback_.callback(),
+                               BoundNetLog()));
   ASSERT_EQ(ERR_FTP_FAILED, callback_.WaitForResult());
 
   MockRead ctrl_reads[] = {
@@ -1189,9 +1232,10 @@ TEST_F(FtpNetworkTransactionTest, EvilRestartPassword) {
                                         ctrl_writes, arraysize(ctrl_writes));
   mock_socket_factory_.AddSocketDataProvider(&ctrl_socket2);
   ASSERT_EQ(ERR_IO_PENDING,
-            transaction_.RestartWithAuth(ASCIIToUTF16("innocent"),
-                                         ASCIIToUTF16("foo\nownz0red"),
-                                         &callback_));
+            transaction_.RestartWithAuth(
+                AuthCredentials(ASCIIToUTF16("innocent"),
+                                ASCIIToUTF16("foo\nownz0red")),
+                callback_.callback()));
   EXPECT_EQ(ERR_MALFORMED_IDENTITY, callback_.WaitForResult());
 }
 
