@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,8 +9,9 @@
 #include <set>
 #include <string>
 
-#include "base/scoped_ptr.h"
-#include "base/string16.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/auth.h"
+#include "net/base/net_export.h"
 #include "net/http/http_util.h"
 
 template <class T> class scoped_refptr;
@@ -23,7 +24,7 @@ class HttpAuthHandlerFactory;
 class HttpResponseHeaders;
 
 // Utility class for http authentication.
-class HttpAuth {
+class NET_EXPORT_PRIVATE HttpAuth {
  public:
   // Http authentication can be done the the proxy server, origin server,
   // or both. This enum tracks who the target is.
@@ -52,6 +53,13 @@ class HttpAuth {
     AUTHORIZATION_RESULT_INVALID,  // The authentication challenge headers are
                                    // poorly formed (the authorization attempt
                                    // itself may have been fine).
+
+    AUTHORIZATION_RESULT_DIFFERENT_REALM,  // The authorization
+                                           // attempt was rejected,
+                                           // but the realm associated
+                                           // with the new challenge
+                                           // is different from the
+                                           // previous attempt.
   };
 
   // Describes where the identity used for authentication came from.
@@ -96,8 +104,7 @@ class HttpAuth {
 
     IdentitySource source;
     bool invalid;
-    string16 username;
-    string16 password;
+    AuthCredentials credentials;
   };
 
   // Get the name of the header containing the auth challenge
@@ -169,7 +176,7 @@ class HttpAuth {
   // parameters as either a base-64 encoded string or a comma-delimited list
   // of name-value pairs. param_pairs() and base64_param() methods are provided
   // to support either usage.
-  class ChallengeTokenizer {
+  class NET_EXPORT_PRIVATE ChallengeTokenizer {
    public:
     ChallengeTokenizer(std::string::const_iterator begin,
                        std::string::const_iterator end)
