@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,9 +10,9 @@
 #include <string>
 
 #include "base/gtest_prod_util.h"
-#include "base/ref_counted.h"
-#include "base/string16.h"
+#include "base/memory/ref_counted.h"
 #include "googleurl/src/gurl.h"
+#include "net/base/net_export.h"
 #include "net/http/http_auth.h"
 
 namespace net {
@@ -25,7 +25,7 @@ namespace net {
 //   - the last auth handler used (contains realm and authentication scheme)
 //   - the list of paths which used this realm
 // Entries can be looked up by either (origin, realm, scheme) or (origin, path).
-class HttpAuthCache {
+class NET_EXPORT_PRIVATE HttpAuthCache {
  public:
   class Entry;
 
@@ -65,8 +65,7 @@ class HttpAuthCache {
   //   |origin|   - the {scheme, host, port} of the server.
   //   |realm|    - the auth realm for the challenge.
   //   |scheme|   - the authentication scheme (i.e. basic, negotiate).
-  //   |username| - login information for the realm.
-  //   |password| - login information for the realm.
+  //   |credentials| - login information for the realm.
   //   |path|     - absolute path for a resource contained in the protection
   //                space; this will be added to the list of known paths.
   //   returns    - the entry that was just added/updated.
@@ -74,23 +73,20 @@ class HttpAuthCache {
              const std::string& realm,
              HttpAuth::Scheme scheme,
              const std::string& auth_challenge,
-             const string16& username,
-             const string16& password,
+             const AuthCredentials& credentials,
              const std::string& path);
 
   // Remove entry on server |origin| for realm |realm| and scheme |scheme|
-  // if one exists AND if the cached identity matches (|username|, |password|).
+  // if one exists AND if the cached credentials matches |credentials|.
   //   |origin|   - the {scheme, host, port} of the server.
   //   |realm|    - case sensitive realm string.
   //   |scheme|   - the authentication scheme (i.e. basic, negotiate).
-  //   |username| - condition to match.
-  //   |password| - condition to match.
+  //   |credentials| - the credentials to match.
   //   returns    - true if an entry was removed.
   bool Remove(const GURL& origin,
               const std::string& realm,
               HttpAuth::Scheme scheme,
-              const string16& username,
-              const string16& password);
+              const AuthCredentials& credentials);
 
   // Updates a stale digest entry on server |origin| for realm |realm| and
   // scheme |scheme|. The cached auth challenge is replaced with
@@ -102,13 +98,16 @@ class HttpAuthCache {
                             HttpAuth::Scheme scheme,
                             const std::string& auth_challenge);
 
+  // Copies all entries from |other| cache.
+  void UpdateAllFrom(const HttpAuthCache& other);
+
  private:
   typedef std::list<Entry> EntryList;
   EntryList entries_;
 };
 
 // An authentication realm entry.
-class HttpAuthCache::Entry {
+class NET_EXPORT_PRIVATE HttpAuthCache::Entry {
  public:
   ~Entry();
 
@@ -131,14 +130,9 @@ class HttpAuthCache::Entry {
     return auth_challenge_;
   }
 
-  // The login username.
-  const string16 username() const {
-    return username_;
-  }
-
-  // The login password.
-  const string16 password() const {
-    return password_;
+  // The login credentials.
+  const AuthCredentials& credentials() const {
+    return credentials_;
   }
 
   int IncrementNonceCount() {
@@ -160,8 +154,15 @@ class HttpAuthCache::Entry {
   // already contained in the protection space, is a no-op.
   void AddPath(const std::string& path);
 
-  // Returns true if |dir| is contained within the realm's protection space.
-  bool HasEnclosingPath(const std::string& dir);
+  // Returns true if |dir| is contained within the realm's protection
+  // space.  |*path_len| is set to the length of the enclosing path if
+  // such a path exists and |path_len| is non-NULL.  If no enclosing
+  // path is found, |*path_len| is left unmodified.
+  //
+  // Note that proxy auth cache entries are associated with empty
+  // paths.  Therefore it is possible for HasEnclosingPath() to return
+  // true and set |*path_len| to 0.
+  bool HasEnclosingPath(const std::string& dir, size_t* path_len);
 
   // |origin_| contains the {protocol, host, port} of the server.
   GURL origin_;
@@ -170,8 +171,7 @@ class HttpAuthCache::Entry {
 
   // Identity.
   std::string auth_challenge_;
-  string16 username_;
-  string16 password_;
+  AuthCredentials credentials_;
 
   int nonce_count_;
 
