@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,26 +10,28 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/string16.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/threading/non_thread_safe.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/completion_callback.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/http/http_auth.h"
 
 namespace net {
 
 class AuthChallengeInfo;
+class AuthCredentials;
 class HttpAuthHandler;
 class HttpAuthHandlerFactory;
 class HttpAuthCache;
 class HttpRequestHeaders;
 struct HttpRequestInfo;
 
-class HttpAuthController : public base::RefCounted<HttpAuthController>,
-                           public base::NonThreadSafe {
+class NET_EXPORT_PRIVATE HttpAuthController
+    : public base::RefCounted<HttpAuthController>,
+      NON_EXPORTED_BASE(public base::NonThreadSafe) {
  public:
   // The arguments are self explanatory except possibly for |auth_url|, which
   // should be both the auth target and auth path in a single url argument.
@@ -43,7 +45,7 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
   // a token is correctly generated synchronously, as well as when no tokens
   // were necessary.
   virtual int MaybeGenerateAuthToken(const HttpRequestInfo* request,
-                                     CompletionCallback* callback,
+                                     const CompletionCallback& callback,
                                      const BoundNetLog& net_log);
 
   // Adds either the proxy auth header, or the origin server auth header,
@@ -60,8 +62,7 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
                                   const BoundNetLog& net_log);
 
   // Store the supplied credentials and prepare to restart the auth.
-  virtual void ResetAuth(const string16& username,
-                         const string16& password);
+  virtual void ResetAuth(const AuthCredentials& credentials);
 
   virtual bool HaveAuthHandler() const;
 
@@ -73,6 +74,13 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
   virtual void DisableAuthScheme(HttpAuth::Scheme scheme);
 
  private:
+  // Actions for InvalidateCurrentHandler()
+  enum InvalidateHandlerAction {
+    INVALIDATE_HANDLER_AND_CACHED_CREDENTIALS,
+    INVALIDATE_HANDLER_AND_DISABLE_SCHEME,
+    INVALIDATE_HANDLER
+  };
+
   // So that we can mock this object.
   friend class base::RefCounted<HttpAuthController>;
 
@@ -83,8 +91,10 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
   // cache entry's data and returns true.
   bool SelectPreemptiveAuth(const BoundNetLog& net_log);
 
-  // Invalidates the current handler, including cache.
-  void InvalidateCurrentHandler();
+  // Invalidates the current handler.  If |action| is
+  // INVALIDATE_HANDLER_AND_CACHED_CREDENTIALS, then also invalidate
+  // the cached credentials used by the handler.
+  void InvalidateCurrentHandler(InvalidateHandlerAction action);
 
   // Invalidates any auth cache entries after authentication has failed.
   // The identity that was rejected is |identity_|.
@@ -96,9 +106,14 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
   bool SelectNextAuthIdentityToTry();
 
   // Populates auth_info_ with the challenge information, so that
-  // URLRequestHttpJob can prompt for a username/password.
+  // URLRequestHttpJob can prompt for credentials.
   void PopulateAuthChallenge();
 
+  // If |result| indicates a permanent failure, disables the current
+  // auth scheme for this controller and returns true.  Returns false
+  // otherwise.
+  bool DisableOnAuthHandlerResult(int result);
+
   void OnIOComplete(int result);
 
   // Indicates if this handler is for Proxy auth or Server auth.
@@ -119,8 +134,8 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
   // associated auth handler.
   scoped_ptr<HttpAuthHandler> handler_;
 
-  // |identity_| holds the (username/password) that should be used by
-  // the handler_ to generate credentials. This identity can come from
+  // |identity_| holds the credentials that should be used by
+  // the handler_ to generate challenge responses. This identity can come from
   // a number of places (url, cache, prompt).
   HttpAuth::Identity identity_;
 
@@ -131,7 +146,7 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
   // Contains information about the auth challenge.
   scoped_refptr<AuthChallengeInfo> auth_info_;
 
-  // True if we've used the username/password embedded in the URL.  This
+  // True if we've used the username:password embedded in the URL.  This
   // makes sure we use the embedded identity only once for the transaction,
   // preventing an infinite auth restart loop.
   bool embedded_identity_used_;
@@ -148,8 +163,7 @@ class HttpAuthController : public base::RefCounted<HttpAuthController>,
 
   std::set<HttpAuth::Scheme> disabled_schemes_;
 
-  CompletionCallbackImpl<HttpAuthController> io_callback_;
-  CompletionCallback* user_callback_;
+  CompletionCallback callback_;
 };
 
 }  // namespace net
