@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,8 @@
 
 #include "base/basictypes.h"
 #include "base/logging.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/native_library.h"
-#include "base/scoped_ptr.h"
 #include "net/base/net_errors.h"
 #include "net/http/mock_gssapi_library_posix.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -51,13 +51,13 @@ const char kInitialAuthResponse[] = "Mary had a little lamb";
 
 void EstablishInitialContext(test::MockGSSAPILibrary* library) {
   test::GssContextMockImpl context_info(
-      "localhost",                    // Source name
-      "example.com",                  // Target name
-      23,                             // Lifetime
-      *GSS_C_NT_HOSTBASED_SERVICE,    // Mechanism
-      0,                              // Context flags
-      1,                              // Locally initiated
-      0);                             // Open
+      "localhost",                         // Source name
+      "example.com",                       // Target name
+      23,                                  // Lifetime
+      *CHROME_GSS_C_NT_HOSTBASED_SERVICE,  // Mechanism
+      0,                                   // Context flags
+      1,                                   // Locally initiated
+      0);                                  // Open
   gss_buffer_desc in_buffer = {0, NULL};
   gss_buffer_desc out_buffer = {arraysize(kInitialAuthResponse),
                                 const_cast<char*>(kInitialAuthResponse)};
@@ -78,14 +78,16 @@ TEST(HttpAuthGSSAPIPOSIXTest, GSSAPIStartup) {
   // functions we want.
   scoped_ptr<GSSAPILibrary> gssapi(new GSSAPISharedLibrary(""));
   DCHECK(gssapi.get());
-  DCHECK(gssapi.get()->Init());
+  EXPECT_TRUE(gssapi.get()->Init());
 }
 
+#if defined(DLOPEN_KERBEROS)
 TEST(HttpAuthGSSAPIPOSIXTest, GSSAPILoadCustomLibrary) {
   scoped_ptr<GSSAPILibrary> gssapi(
       new GSSAPISharedLibrary("/this/library/does/not/exist"));
-  DCHECK(!gssapi.get()->Init());
+  EXPECT_FALSE(gssapi.get()->Init());
 }
+#endif  // defined(DLOPEN_KERBEROS)
 
 TEST(HttpAuthGSSAPIPOSIXTest, GSSAPICycle) {
   scoped_ptr<test::MockGSSAPILibrary> mock_library(new test::MockGSSAPILibrary);
@@ -93,39 +95,36 @@ TEST(HttpAuthGSSAPIPOSIXTest, GSSAPICycle) {
   mock_library->Init();
   const char kAuthResponse[] = "Mary had a little lamb";
   test::GssContextMockImpl context1(
-      "localhost",                    // Source name
-      "example.com",                  // Target name
-      23,                             // Lifetime
-      *GSS_C_NT_HOSTBASED_SERVICE,    // Mechanism
-      0,                              // Context flags
-      1,                              // Locally initiated
-      0);                             // Open
+      "localhost",                         // Source name
+      "example.com",                       // Target name
+      23,                                  // Lifetime
+      *CHROME_GSS_C_NT_HOSTBASED_SERVICE,  // Mechanism
+      0,                                   // Context flags
+      1,                                   // Locally initiated
+      0);                                  // Open
   test::GssContextMockImpl context2(
-      "localhost",                    // Source name
-      "example.com",                  // Target name
-      23,                             // Lifetime
-      *GSS_C_NT_HOSTBASED_SERVICE,    // Mechanism
-      0,                              // Context flags
-      1,                              // Locally initiated
-      1);                             // Open
+      "localhost",                         // Source name
+      "example.com",                       // Target name
+      23,                                  // Lifetime
+      *CHROME_GSS_C_NT_HOSTBASED_SERVICE,  // Mechanism
+      0,                                   // Context flags
+      1,                                   // Locally initiated
+      1);                                  // Open
   test::MockGSSAPILibrary::SecurityContextQuery queries[] = {
-    { "Negotiate",                    // Package name
-      GSS_S_CONTINUE_NEEDED,          // Major response code
-      0,                              // Minor response code
-      context1,                       // Context
-      { 0, NULL },                           // Expected input token
-      { arraysize(kAuthResponse),
-        const_cast<char*>(kAuthResponse) }   // Output token
-    },
-    { "Negotiate",                    // Package name
-      GSS_S_COMPLETE,                 // Major response code
-      0,                              // Minor response code
-      context2,                       // Context
-      { arraysize(kAuthResponse),
-        const_cast<char*>(kAuthResponse) },  // Expected input token
-      { arraysize(kAuthResponse),
-        const_cast<char*>(kAuthResponse) }   // Output token
-    },
+    test::MockGSSAPILibrary::SecurityContextQuery(
+        "Negotiate",            // Package name
+        GSS_S_CONTINUE_NEEDED,  // Major response code
+        0,                      // Minor response code
+        context1,               // Context
+        NULL,                   // Expected input token
+        kAuthResponse),         // Output token
+    test::MockGSSAPILibrary::SecurityContextQuery(
+        "Negotiate",            // Package name
+        GSS_S_COMPLETE,         // Major response code
+        0,                      // Minor response code
+        context2,               // Context
+        kAuthResponse,          // Expected input token
+        kAuthResponse)          // Output token
   };
 
   for (size_t i = 0; i < arraysize(queries); ++i) {
@@ -165,6 +164,7 @@ TEST(HttpAuthGSSAPIPOSIXTest, GSSAPICycle) {
                                                   &output_token,
                                                   &ret_flags,
                                                   &time_rec);
+    EXPECT_EQ(queries[i].response_code, major_status);
     CopyBuffer(&input_token, &output_token);
     ClearBuffer(&output_token);
   }
@@ -172,6 +172,7 @@ TEST(HttpAuthGSSAPIPOSIXTest, GSSAPICycle) {
   major_status = mock_library->delete_sec_context(&minor_status,
                                                   &context_handle,
                                                   GSS_C_NO_BUFFER);
+  EXPECT_EQ(static_cast<OM_uint32>(GSS_S_COMPLETE), major_status);
 }
 
 TEST(HttpAuthGSSAPITest, ParseChallenge_FirstRound) {
@@ -201,8 +202,7 @@ TEST(HttpAuthGSSAPITest, ParseChallenge_TwoRounds) {
   // Generate an auth token and create another thing.
   EstablishInitialContext(&mock_library);
   std::string auth_token;
-  EXPECT_EQ(OK, auth_gssapi.GenerateAuthToken(NULL, NULL,
-                                              L"HTTP/intranet.google.com",
+  EXPECT_EQ(OK, auth_gssapi.GenerateAuthToken(NULL, L"HTTP/intranet.google.com",
                                               &auth_token));
 
   std::string second_challenge_text = "Negotiate Zm9vYmFy";
@@ -239,8 +239,7 @@ TEST(HttpAuthGSSAPITest, ParseChallenge_MissingTokenSecondRound) {
 
   EstablishInitialContext(&mock_library);
   std::string auth_token;
-  EXPECT_EQ(OK, auth_gssapi.GenerateAuthToken(NULL, NULL,
-                                              L"HTTP/intranet.google.com",
+  EXPECT_EQ(OK, auth_gssapi.GenerateAuthToken(NULL, L"HTTP/intranet.google.com",
                                               &auth_token));
   std::string second_challenge_text = "Negotiate";
   HttpAuth::ChallengeTokenizer second_challenge(second_challenge_text.begin(),
@@ -263,8 +262,7 @@ TEST(HttpAuthGSSAPITest, ParseChallenge_NonBase64EncodedToken) {
 
   EstablishInitialContext(&mock_library);
   std::string auth_token;
-  EXPECT_EQ(OK, auth_gssapi.GenerateAuthToken(NULL, NULL,
-                                              L"HTTP/intranet.google.com",
+  EXPECT_EQ(OK, auth_gssapi.GenerateAuthToken(NULL, L"HTTP/intranet.google.com",
                                               &auth_token));
   std::string second_challenge_text = "Negotiate =happyjoy=";
   HttpAuth::ChallengeTokenizer second_challenge(second_challenge_text.begin(),
