@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,7 +10,8 @@
 #include <string>
 #include <vector>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/net_export.h"
 #include "net/http/http_auth.h"
 #include "net/http/url_security_manager.h"
 
@@ -26,7 +27,7 @@ class HttpAuthHandlerRegistryFactory;
 // An HttpAuthHandlerFactory is used to create HttpAuthHandler objects.
 // The HttpAuthHandlerFactory object _must_ outlive any of the HttpAuthHandler
 // objects that it creates.
-class HttpAuthHandlerFactory {
+class NET_EXPORT HttpAuthHandlerFactory {
  public:
   enum CreateReason {
     CREATE_CHALLENGE,     // Create a handler in response to a challenge.
@@ -125,7 +126,8 @@ class HttpAuthHandlerFactory {
 
 // The HttpAuthHandlerRegistryFactory dispatches create requests out
 // to other factories based on the auth scheme.
-class HttpAuthHandlerRegistryFactory : public HttpAuthHandlerFactory {
+class NET_EXPORT HttpAuthHandlerRegistryFactory
+    : public HttpAuthHandlerFactory {
  public:
   HttpAuthHandlerRegistryFactory();
   virtual ~HttpAuthHandlerRegistryFactory();
@@ -187,7 +189,7 @@ class HttpAuthHandlerRegistryFactory : public HttpAuthHandlerFactory {
                                 CreateReason reason,
                                 int digest_nonce_count,
                                 const BoundNetLog& net_log,
-                                scoped_ptr<HttpAuthHandler>* handler);
+                                scoped_ptr<HttpAuthHandler>* handler) OVERRIDE;
 
  private:
   typedef std::map<std::string, HttpAuthHandlerFactory*> FactoryMap;
