@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,11 +10,11 @@
 #include "net/base/net_errors.h"
 #include "net/base/test_completion_callback.h"
 #include "net/http/http_request_info.h"
+#include "net/http/mock_allow_url_security_manager.h"
 #if defined(OS_WIN)
 #include "net/http/mock_sspi_library_win.h"
 #elif defined(OS_POSIX)
 #include "net/http/mock_gssapi_library_posix.h"
-#include "net/third_party/gssapi/gssapi.h"
 #endif
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
@@ -25,7 +25,6 @@ typedef net::MockSSPILibrary MockAuthLibrary;
 typedef net::test::MockGSSAPILibrary MockAuthLibrary;
 #endif
 
-
 namespace net {
 
 class HttpAuthHandlerNegotiateTest : public PlatformTest {
@@ -36,7 +35,7 @@ class HttpAuthHandlerNegotiateTest : public PlatformTest {
     resolver_->rules()->AddIPLiteralRule("alias", "10.0.0.2",
                                            "canonical.example.com");
 
-    url_security_manager_.reset(new URLSecurityManagerAllow());
+    url_security_manager_.reset(new MockAllowURLSecurityManager());
     factory_.reset(new HttpAuthHandlerNegotiate::Factory());
     factory_->set_url_security_manager(url_security_manager_.get());
     factory_->set_library(auth_library_);
@@ -99,39 +98,36 @@ class HttpAuthHandlerNegotiateTest : public PlatformTest {
         "\x69\xE3\x55\xF9\x30\xD3\xD4\x08\xC8\xCA\x62\xF8\x64\xEC\x9B\x92"
         "\x1A\xF1\x03\x2E\xCC\xDC\xEB\x17\xDE\x09\xAC\xA9\x58\x86";
     test::GssContextMockImpl context1(
-        "localhost",                    // Source name
-        "example.com",                  // Target name
-        23,                             // Lifetime
-        *GSS_C_NT_HOSTBASED_SERVICE,    // Mechanism
-        0,                              // Context flags
-        1,                              // Locally initiated
-        0);                             // Open
+        "localhost",                         // Source name
+        "example.com",                       // Target name
+        23,                                  // Lifetime
+        *CHROME_GSS_C_NT_HOSTBASED_SERVICE,  // Mechanism
+        0,                                   // Context flags
+        1,                                   // Locally initiated
+        0);                                  // Open
     test::GssContextMockImpl context2(
-        "localhost",                    // Source name
-        "example.com",                  // Target name
-        23,                             // Lifetime
-        *GSS_C_NT_HOSTBASED_SERVICE,    // Mechanism
-        0,                              // Context flags
-        1,                              // Locally initiated
-        1);                             // Open
+        "localhost",                         // Source name
+        "example.com",                       // Target name
+        23,                                  // Lifetime
+        *CHROME_GSS_C_NT_HOSTBASED_SERVICE,  // Mechanism
+        0,                                   // Context flags
+        1,                                   // Locally initiated
+        1);                                  // Open
     test::MockGSSAPILibrary::SecurityContextQuery queries[] = {
-      { "Negotiate",                    // Package name
+    test::MockGSSAPILibrary::SecurityContextQuery(
+        "Negotiate",                    // Package name
         GSS_S_CONTINUE_NEEDED,          // Major response code
         0,                              // Minor response code
         context1,                       // Context
-        { 0, NULL },                           // Expected input token
-        { arraysize(kAuthResponse),
-          const_cast<char*>(kAuthResponse) }   // Output token
-      },
-      { "Negotiate",                    // Package name
+        NULL,                           // Expected input token
+        kAuthResponse),                 // Output token
+    test::MockGSSAPILibrary::SecurityContextQuery(
+        "Negotiate",                    // Package name
         GSS_S_COMPLETE,                 // Major response code
         0,                              // Minor response code
         context2,                       // Context
-        { arraysize(kAuthResponse),
-          const_cast<char*>(kAuthResponse) },  // Expected input token
-        { arraysize(kAuthResponse),
-          const_cast<char*>(kAuthResponse) }   // Output token
-      },
+        kAuthResponse,                  // Expected input token
+        kAuthResponse)                  // Output token
     };
 
     for (size_t i = 0; i < arraysize(queries); ++i) {
@@ -158,14 +154,13 @@ class HttpAuthHandlerNegotiateTest : public PlatformTest {
         0,                              // Context flags
         1,                              // Locally initiated
         0);                             // Open
-    test::MockGSSAPILibrary::SecurityContextQuery query = {
-      "Negotiate",                    // Package name
-      major_status,                   // Major response code
-      minor_status,                   // Minor response code
-      context,                        // Context
-      { 0, NULL },                    // Expected input token
-      { 0, NULL }                     // Output token
-    };
+    test::MockGSSAPILibrary::SecurityContextQuery query(
+        "Negotiate",                    // Package name
+        major_status,                   // Major response code
+        minor_status,                   // Minor response code
+        context,                        // Context
+        NULL,                           // Expected input token
+        NULL);                          // Output token
 
     mock_library->ExpectSecurityContext(query.expected_package,
                                         query.response_code,
@@ -228,9 +223,8 @@ TEST_F(HttpAuthHandlerNegotiateTest, DisableCname) {
   TestCompletionCallback callback;
   HttpRequestInfo request_info;
   std::string token;
-  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, NULL,
-                                                &request_info,
-                                                &callback, &token));
+  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, &request_info,
+                                                callback.callback(), &token));
 #if defined(OS_WIN)
   EXPECT_EQ(L"HTTP/alias", auth_handler->spn());
 #elif defined(OS_POSIX)
@@ -247,9 +241,8 @@ TEST_F(HttpAuthHandlerNegotiateTest, DisableCnameStandardPort) {
   TestCompletionCallback callback;
   HttpRequestInfo request_info;
   std::string token;
-  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, NULL,
-                                                &request_info,
-                                                &callback, &token));
+  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, &request_info,
+                                                callback.callback(), &token));
 #if defined(OS_WIN)
   EXPECT_EQ(L"HTTP/alias", auth_handler->spn());
 #elif defined(OS_POSIX)
@@ -266,9 +259,8 @@ TEST_F(HttpAuthHandlerNegotiateTest, DisableCnameNonstandardPort) {
   TestCompletionCallback callback;
   HttpRequestInfo request_info;
   std::string token;
-  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, NULL,
-                                                &request_info,
-                                                &callback, &token));
+  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, &request_info,
+                                                callback.callback(), &token));
 #if defined(OS_WIN)
   EXPECT_EQ(L"HTTP/alias:500", auth_handler->spn());
 #elif defined(OS_POSIX)
@@ -285,9 +277,8 @@ TEST_F(HttpAuthHandlerNegotiateTest, CnameSync) {
   TestCompletionCallback callback;
   HttpRequestInfo request_info;
   std::string token;
-  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, NULL,
-                                                &request_info,
-                                                &callback, &token));
+  EXPECT_EQ(OK, auth_handler->GenerateAuthToken(NULL, &request_info,
+                                                callback.callback(), &token));
 #if defined(OS_WIN)
   EXPECT_EQ(L"HTTP/canonical.example.com", auth_handler->spn());
 #elif defined(OS_POSIX)
@@ -305,7 +296,7 @@ TEST_F(HttpAuthHandlerNegotiateTest, CnameAsync) {
   HttpRequestInfo request_info;
   std::string token;
   EXPECT_EQ(ERR_IO_PENDING, auth_handler->GenerateAuthToken(
-      NULL, NULL, &request_info, &callback, &token));
+      NULL, &request_info, callback.callback(), &token));
   EXPECT_EQ(OK, callback.WaitForResult());
 #if defined(OS_WIN)
   EXPECT_EQ(L"HTTP/canonical.example.com", auth_handler->spn());
@@ -316,9 +307,8 @@ TEST_F(HttpAuthHandlerNegotiateTest, CnameAsync) {
 
 #if defined(OS_POSIX)
 
-// These tests are only for GSSAPI, as we can't use explicit credentials with
+// This test is only for GSSAPI, as we can't use explicit credentials with
 // that library.
-
 TEST_F(HttpAuthHandlerNegotiateTest, ServerNotInKerberosDatabase) {
   SetupErrorMocks(AuthLibrary(), GSS_S_FAILURE, 0x96C73A07);  // No server
   scoped_ptr<HttpAuthHandlerNegotiate> auth_handler;
@@ -329,10 +319,12 @@ TEST_F(HttpAuthHandlerNegotiateTest, ServerNotInKerberosDatabase) {
   HttpRequestInfo request_info;
   std::string token;
   EXPECT_EQ(ERR_IO_PENDING, auth_handler->GenerateAuthToken(
-      NULL, NULL, &request_info, &callback, &token));
+      NULL, &request_info, callback.callback(), &token));
   EXPECT_EQ(ERR_MISSING_AUTH_CREDENTIALS, callback.WaitForResult());
 }
 
+// This test is only for GSSAPI, as we can't use explicit credentials with
+// that library.
 TEST_F(HttpAuthHandlerNegotiateTest, NoKerberosCredentials) {
   SetupErrorMocks(AuthLibrary(), GSS_S_FAILURE, 0x96C73AC3);  // No credentials
   scoped_ptr<HttpAuthHandlerNegotiate> auth_handler;
@@ -343,10 +335,34 @@ TEST_F(HttpAuthHandlerNegotiateTest, NoKerberosCredentials) {
   HttpRequestInfo request_info;
   std::string token;
   EXPECT_EQ(ERR_IO_PENDING, auth_handler->GenerateAuthToken(
-      NULL, NULL, &request_info, &callback, &token));
+      NULL, &request_info, callback.callback(), &token));
   EXPECT_EQ(ERR_MISSING_AUTH_CREDENTIALS, callback.WaitForResult());
 }
 
+#if defined(DLOPEN_KERBEROS)
+TEST_F(HttpAuthHandlerNegotiateTest, MissingGSSAPI) {
+  scoped_ptr<HostResolver> host_resolver(new MockHostResolver());
+  MockAllowURLSecurityManager url_security_manager;
+  scoped_ptr<HttpAuthHandlerNegotiate::Factory> negotiate_factory(
+      new HttpAuthHandlerNegotiate::Factory());
+  negotiate_factory->set_host_resolver(host_resolver.get());
+  negotiate_factory->set_url_security_manager(&url_security_manager);
+  negotiate_factory->set_library(
+      new GSSAPISharedLibrary("/this/library/does/not/exist"));
+
+  GURL gurl("http://www.example.com");
+  scoped_ptr<HttpAuthHandler> generic_handler;
+  int rv = negotiate_factory->CreateAuthHandlerFromString(
+      "Negotiate",
+      HttpAuth::AUTH_SERVER,
+      gurl,
+      BoundNetLog(),
+      &generic_handler);
+  EXPECT_EQ(ERR_UNSUPPORTED_AUTH_SCHEME, rv);
+  EXPECT_TRUE(generic_handler.get() == NULL);
+}
+#endif  // defined(DLOPEN_KERBEROS)
+
 #endif  // defined(OS_POSIX)
 
 }  // namespace net
