@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,21 +15,30 @@
 
 namespace net {
 
+HttpAuth::AuthorizationResult HttpAuthHandlerNTLM::HandleAnotherChallenge(
+    HttpAuth::ChallengeTokenizer* challenge) {
+  return ParseChallenge(challenge, false);
+}
+
+bool HttpAuthHandlerNTLM::Init(HttpAuth::ChallengeTokenizer* tok) {
+  auth_scheme_ = HttpAuth::AUTH_SCHEME_NTLM;
+  score_ = 3;
+  properties_ = ENCRYPTS_IDENTITY | IS_CONNECTION_BASED;
+
+  return ParseChallenge(tok, true) == HttpAuth::AUTHORIZATION_RESULT_ACCEPT;
+}
+
 int HttpAuthHandlerNTLM::GenerateAuthTokenImpl(
-    const string16* username,
-    const string16* password,
-    const HttpRequestInfo* request,
-    CompletionCallback* callback,
-    std::string* auth_token) {
+    const AuthCredentials* credentials, const HttpRequestInfo* request,
+    const CompletionCallback& callback, std::string* auth_token) {
 #if defined(NTLM_SSPI)
   return auth_sspi_.GenerateAuthToken(
-      username,
-      password,
+      credentials,
       CreateSPN(origin_),
       auth_token);
 #else  // !defined(NTLM_SSPI)
   // TODO(cbentzel): Shouldn't be hitting this case.
-  if (!username || !password) {
+  if (!credentials) {
     LOG(ERROR) << "Username and password are expected to be non-NULL.";
     return ERR_MISSING_AUTH_CREDENTIALS;
   }
@@ -41,21 +50,21 @@ int HttpAuthHandlerNTLM::GenerateAuthTokenImpl(
   uint32 in_buf_len, out_buf_len;
   std::string decoded_auth_data;
 
-  // |username| may be in the form "DOMAIN\user".  Parse it into the two
+  // The username may be in the form "DOMAIN\user".  Parse it into the two
   // components.
   string16 domain;
   string16 user;
+  const string16& username = credentials->username();
   const char16 backslash_character = '\\';
-  size_t backslash_idx = username->find(backslash_character);
+  size_t backslash_idx = username.find(backslash_character);
   if (backslash_idx == string16::npos) {
-    user = *username;
+    user = username;
   } else {
-    domain = username->substr(0, backslash_idx);
-    user = username->substr(backslash_idx + 1);
+    domain = username.substr(0, backslash_idx);
+    user = username.substr(backslash_idx + 1);
   }
   domain_ = domain;
-  username_ = user;
-  password_ = *password;
+  credentials_.Set(user, credentials->password());
 
   // Initial challenge.
   if (auth_data_.empty()) {
@@ -92,19 +101,6 @@ int HttpAuthHandlerNTLM::GenerateAuthTokenImpl(
 #endif
 }
 
-bool HttpAuthHandlerNTLM::Init(HttpAuth::ChallengeTokenizer* tok) {
-  auth_scheme_ = HttpAuth::AUTH_SCHEME_NTLM;
-  score_ = 3;
-  properties_ = ENCRYPTS_IDENTITY | IS_CONNECTION_BASED;
-
-  return ParseChallenge(tok, true) == HttpAuth::AUTHORIZATION_RESULT_ACCEPT;
-}
-
-HttpAuth::AuthorizationResult HttpAuthHandlerNTLM::HandleAnotherChallenge(
-    HttpAuth::ChallengeTokenizer* challenge) {
-  return ParseChallenge(challenge, false);
-}
-
 // The NTLM challenge header looks like:
 //   WWW-Authenticate: NTLM auth-data
 HttpAuth::AuthorizationResult HttpAuthHandlerNTLM::ParseChallenge(
