@@ -1,17 +1,21 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/http/http_basic_stream.h"
 
+#include "base/format_macros.h"
+#include "base/metrics/histogram.h"
 #include "base/stringprintf.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
 #include "net/http/http_request_headers.h"
 #include "net/http/http_request_info.h"
+#include "net/http/http_response_body_drainer.h"
 #include "net/http/http_stream_parser.h"
 #include "net/http/http_util.h"
 #include "net/socket/client_socket_handle.h"
+#include "net/socket/client_socket_pool_base.h"
 
 namespace net {
 
@@ -22,16 +26,21 @@ HttpBasicStream::HttpBasicStream(ClientSocketHandle* connection,
       parser_(parser),
       connection_(connection),
       using_proxy_(using_proxy),
-      request_info_(NULL) {
+      request_info_(NULL),
+      response_(NULL),
+      bytes_read_offset_(0) {
 }
 
-int HttpBasicStream::InitializeStream(const HttpRequestInfo* request_info,
-                                      const BoundNetLog& net_log,
-                                      CompletionCallback* callback) {
+HttpBasicStream::~HttpBasicStream() {}
+
+int HttpBasicStream::InitializeStream(
+    const HttpRequestInfo* request_info, const BoundNetLog& net_log,
+    const CompletionCallback& callback) {
   DCHECK(!parser_.get());
   request_info_ = request_info;
   parser_.reset(new HttpStreamParser(connection_.get(), request_info,
                                      read_buf_, net_log));
+  bytes_read_offset_ = connection_->socket()->NumBytesRead();
   return OK;
 }
 
@@ -39,7 +48,7 @@ int HttpBasicStream::InitializeStream(const HttpRequestInfo* request_info,
 int HttpBasicStream::SendRequest(const HttpRequestHeaders& headers,
                                  UploadDataStream* request_body,
                                  HttpResponseInfo* response,
-                                 CompletionCallback* callback) {
+                                 const CompletionCallback& callback) {
   DCHECK(parser_.get());
   DCHECK(request_info_);
   const std::string path = using_proxy_ ?
@@ -48,17 +57,16 @@ int HttpBasicStream::SendRequest(const HttpRequestHeaders& headers,
   request_line_ = base::StringPrintf("%s %s HTTP/1.1\r\n",
                                      request_info_->method.c_str(),
                                      path.c_str());
+  response_ = response;
   return parser_->SendRequest(request_line_, headers, request_body, response,
                               callback);
 }
 
-HttpBasicStream::~HttpBasicStream() {}
-
 uint64 HttpBasicStream::GetUploadProgress() const {
   return parser_->GetUploadProgress();
 }
 
-int HttpBasicStream::ReadResponseHeaders(CompletionCallback* callback) {
+int HttpBasicStream::ReadResponseHeaders(const CompletionCallback& callback) {
   return parser_->ReadResponseHeaders(callback);
 }
 
@@ -67,7 +75,7 @@ const HttpResponseInfo* HttpBasicStream::GetResponseInfo() const {
 }
 
 int HttpBasicStream::ReadResponseBody(IOBuffer* buf, int buf_len,
-                                      CompletionCallback* callback) {
+                                      const CompletionCallback& callback) {
   return parser_->ReadResponseBody(buf, buf_len, callback);
 }
 
@@ -102,6 +110,10 @@ void HttpBasicStream::SetConnectionReused() {
   parser_->SetConnectionReused();
 }
 
+bool HttpBasicStream::IsConnectionReusable() const {
+  return parser_->IsConnectionReusable();
+}
+
 void HttpBasicStream::GetSSLInfo(SSLInfo* ssl_info) {
   parser_->GetSSLInfo(ssl_info);
 }
@@ -111,4 +123,18 @@ void HttpBasicStream::GetSSLCertRequestInfo(
   parser_->GetSSLCertRequestInfo(cert_request_info);
 }
 
+bool HttpBasicStream::IsSpdyHttpStream() const {
+  return false;
+}
+
+void HttpBasicStream::LogNumRttVsBytesMetrics() const {
+  // Log rtt metrics here.
+}
+
+void HttpBasicStream::Drain(HttpNetworkSession* session) {
+  HttpResponseBodyDrainer* drainer = new HttpResponseBodyDrainer(this);
+  drainer->Start(session);
+  // |drainer| will delete itself.
+}
+
 }  // namespace net
