@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -13,7 +13,7 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "net/http/http_stream.h"
 
 namespace net {
@@ -42,39 +42,48 @@ class HttpBasicStream : public HttpStream {
   // HttpStream methods:
   virtual int InitializeStream(const HttpRequestInfo* request_info,
                                const BoundNetLog& net_log,
-                               CompletionCallback* callback);
+                               const CompletionCallback& callback) OVERRIDE;
 
   virtual int SendRequest(const HttpRequestHeaders& headers,
                           UploadDataStream* request_body,
                           HttpResponseInfo* response,
-                          CompletionCallback* callback);
+                          const CompletionCallback& callback) OVERRIDE;
 
-  virtual uint64 GetUploadProgress() const;
+  virtual uint64 GetUploadProgress() const OVERRIDE;
 
-  virtual int ReadResponseHeaders(CompletionCallback* callback);
+  virtual int ReadResponseHeaders(const CompletionCallback& callback) OVERRIDE;
 
-  virtual const HttpResponseInfo* GetResponseInfo() const;
+  virtual const HttpResponseInfo* GetResponseInfo() const OVERRIDE;
 
   virtual int ReadResponseBody(IOBuffer* buf, int buf_len,
-                               CompletionCallback* callback);
+                               const CompletionCallback& callback) OVERRIDE;
 
-  virtual void Close(bool not_reusable);
+  virtual void Close(bool not_reusable) OVERRIDE;
 
-  virtual HttpStream* RenewStreamForAuth();
+  virtual HttpStream* RenewStreamForAuth() OVERRIDE;
 
-  virtual bool IsResponseBodyComplete() const;
+  virtual bool IsResponseBodyComplete() const OVERRIDE;
 
-  virtual bool CanFindEndOfResponse() const;
+  virtual bool CanFindEndOfResponse() const OVERRIDE;
 
-  virtual bool IsMoreDataBuffered() const;
+  virtual bool IsMoreDataBuffered() const OVERRIDE;
 
-  virtual bool IsConnectionReused() const;
+  virtual bool IsConnectionReused() const OVERRIDE;
 
-  virtual void SetConnectionReused();
+  virtual void SetConnectionReused() OVERRIDE;
 
-  virtual void GetSSLInfo(SSLInfo* ssl_info);
+  virtual bool IsConnectionReusable() const OVERRIDE;
 
-  virtual void GetSSLCertRequestInfo(SSLCertRequestInfo* cert_request_info);
+  virtual void GetSSLInfo(SSLInfo* ssl_info) OVERRIDE;
+
+  virtual void GetSSLCertRequestInfo(
+      SSLCertRequestInfo* cert_request_info) OVERRIDE;
+
+  virtual bool IsSpdyHttpStream() const OVERRIDE;
+
+  virtual void LogNumRttVsBytesMetrics() const OVERRIDE;
+
+  virtual void Drain(HttpNetworkSession* session) OVERRIDE;
 
  private:
   scoped_refptr<GrowableIOBuffer> read_buf_;
@@ -89,6 +98,10 @@ class HttpBasicStream : public HttpStream {
 
   const HttpRequestInfo* request_info_;
 
+  const HttpResponseInfo* response_;
+
+  int64 bytes_read_offset_;
+
   DISALLOW_COPY_AND_ASSIGN(HttpBasicStream);
 };
 
