@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,16 +11,16 @@
 
 #include <string>
 
-#include "base/string16.h"
 #include "base/time.h"
+#include "net/base/completion_callback.h"
 #include "net/base/net_log.h"
 #include "net/http/http_cache.h"
 #include "net/http/http_response_info.h"
+#include "net/http/http_request_headers.h"
 #include "net/http/http_transaction.h"
 
 namespace net {
 
-class HttpResponseHeaders;
 class PartialData;
 struct HttpRequestInfo;
 
@@ -28,25 +28,6 @@ struct HttpRequestInfo;
 // factory.
 class HttpCache::Transaction : public HttpTransaction {
  public:
-  Transaction(HttpCache* cache);
-  virtual ~Transaction();
-
-  // HttpTransaction methods:
-  virtual int Start(const HttpRequestInfo*, CompletionCallback*,
-                    const BoundNetLog&);
-  virtual int RestartIgnoringLastError(CompletionCallback* callback);
-  virtual int RestartWithCertificate(X509Certificate* client_cert,
-                                     CompletionCallback* callback);
-  virtual int RestartWithAuth(const string16& username,
-                              const string16& password,
-                              CompletionCallback* callback);
-  virtual bool IsReadyToRestartForAuth();
-  virtual int Read(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual void StopCaching();
-  virtual const HttpResponseInfo* GetResponseInfo() const;
-  virtual LoadState GetLoadState() const;
-  virtual uint64 GetUploadProgress(void) const;
-
   // The transaction has the following modes, which apply to how it may access
   // its cache entry.
   //
@@ -76,6 +57,9 @@ class HttpCache::Transaction : public HttpTransaction {
     UPDATE          = READ_META | WRITE,  // READ_WRITE & ~READ_DATA
   };
 
+  explicit Transaction(HttpCache* cache);
+  virtual ~Transaction();
+
   Mode mode() const { return mode_; }
 
   const std::string& key() const { return cache_key_; }
@@ -85,7 +69,7 @@ class HttpCache::Transaction : public HttpTransaction {
   // Returns the number of bytes actually written, or a net error code. If the
   // operation cannot complete immediately, returns ERR_IO_PENDING, grabs a
   // reference to the buffer (until completion), and notifies the caller using
-  // the provided |callback| when the operatiopn finishes.
+  // the provided |callback| when the operation finishes.
   //
   // The first time this method is called for a given transaction, previous
   // meta-data will be overwritten with the provided data, and subsequent
@@ -95,7 +79,9 @@ class HttpCache::Transaction : public HttpTransaction {
   // response (or response info) must be evaluated by the caller, for instance
   // to make sure that the response_time is as expected, before calling this
   // method.
-  int WriteMetadata(IOBuffer* buf, int buf_len, CompletionCallback* callback);
+  int WriteMetadata(IOBuffer* buf,
+                    int buf_len,
+                    const CompletionCallback& callback);
 
   // This transaction is being deleted and we are not done writing to the cache.
   // We need to indicate that the response data was truncated.  Returns true on
@@ -108,10 +94,30 @@ class HttpCache::Transaction : public HttpTransaction {
   // to the cache entry.
   LoadState GetWriterLoadState() const;
 
-  CompletionCallback* io_callback() { return &io_callback_; }
+  const CompletionCallback& io_callback() { return io_callback_; }
 
   const BoundNetLog& net_log() const;
 
+  // HttpTransaction methods:
+  virtual int Start(const HttpRequestInfo*, const CompletionCallback&,
+                    const BoundNetLog&) OVERRIDE;
+  virtual int RestartIgnoringLastError(
+      const CompletionCallback& callback) OVERRIDE;
+  virtual int RestartWithCertificate(
+      X509Certificate* client_cert,
+      const CompletionCallback& callback) OVERRIDE;
+  virtual int RestartWithAuth(const AuthCredentials& credentials,
+                              const CompletionCallback& callback) OVERRIDE;
+  virtual bool IsReadyToRestartForAuth() OVERRIDE;
+  virtual int Read(IOBuffer* buf,
+                   int buf_len,
+                   const CompletionCallback& callback) OVERRIDE;
+  virtual void StopCaching() OVERRIDE;
+  virtual void DoneReading() OVERRIDE;
+  virtual const HttpResponseInfo* GetResponseInfo() const OVERRIDE;
+  virtual LoadState GetLoadState() const OVERRIDE;
+  virtual uint64 GetUploadProgress(void) const OVERRIDE;
+
  private:
   static const size_t kNumValidationHeaders = 2;
   // Helper struct to pair a header name with its value, for
@@ -238,7 +244,7 @@ class HttpCache::Transaction : public HttpTransaction {
 
   // Validates the entry headers against the requested range and continues with
   // the validation of the rest of the entry.  Returns a network error code.
-  int ValidateEntryHeadersAndContinue(bool byte_range_requested);
+  int ValidateEntryHeadersAndContinue();
 
   // Called to start requests which were given an "if-modified-since" or
   // "if-none-match" validation header by the caller (NOT when the request was
@@ -256,8 +262,7 @@ class HttpCache::Transaction : public HttpTransaction {
 
   // Called to restart a network transaction with authentication credentials.
   // Returns network error code.
-  int RestartNetworkRequestWithAuth(const string16& username,
-                                    const string16& password);
+  int RestartNetworkRequestWithAuth(const AuthCredentials& credentials);
 
   // Called to determine if we need to validate the cache entry before using it.
   bool RequiresValidation();
@@ -267,9 +272,9 @@ class HttpCache::Transaction : public HttpTransaction {
   bool ConditionalizeRequest();
 
   // Makes sure that a 206 response is expected.  Returns true on success.
-  // On success, |partial_content| will be set to true if we are processing a
+  // On success, handling_206_ will be set to true if we are processing a
   // partial entry.
-  bool ValidatePartialResponse(bool* partial_content);
+  bool ValidatePartialResponse();
 
   // Handles a response validation error by bypassing the cache.
   void IgnoreRangeRequest();
@@ -288,7 +293,7 @@ class HttpCache::Transaction : public HttpTransaction {
   // cache entry is destroyed.  Future calls to this function will just do
   // nothing without side-effect.  Returns a network error code.
   int WriteToEntry(int index, int offset, IOBuffer* data, int data_len,
-                   CompletionCallback* callback);
+                   const CompletionCallback& callback);
 
   // Called to write response_ to the cache entry. |truncated| indicates if the
   // entry should be marked as incomplete.
@@ -297,7 +302,7 @@ class HttpCache::Transaction : public HttpTransaction {
   // Called to append response data to the cache entry.  Returns a network error
   // code.
   int AppendResponseDataToEntry(IOBuffer* data, int data_len,
-                                CompletionCallback* callback);
+                                const CompletionCallback& callback);
 
   // Called when we are done writing to the cache entry.
   void DoneWritingToEntry(bool success);
@@ -314,6 +319,11 @@ class HttpCache::Transaction : public HttpTransaction {
   // working with range requests.
   int DoPartialCacheReadCompleted(int result);
 
+  // Returns true if we should bother attempting to resume this request if it
+  // is aborted while in progress. If |has_data| is true, the size of the stored
+  // data is considered for the result.
+  bool CanResume(bool has_data);
+
   // Called to signal completion of asynchronous IO.
   void OnIOComplete(int result);
 
@@ -321,6 +331,7 @@ class HttpCache::Transaction : public HttpTransaction {
   const HttpRequestInfo* request_;
   BoundNetLog net_log_;
   scoped_ptr<HttpRequestInfo> custom_request_;
+  HttpRequestHeaders request_headers_copy_;
   // If extra_headers specified a "if-modified-since" or "if-none-match",
   // |external_validation_| contains the value of those headers.
   ValidationHeaders external_validation_;
@@ -329,7 +340,7 @@ class HttpCache::Transaction : public HttpTransaction {
   base::TimeTicks entry_lock_waiting_since_;
   HttpCache::ActiveEntry* new_entry_;
   scoped_ptr<HttpTransaction> network_trans_;
-  CompletionCallback* callback_;  // Consumer's callback.
+  CompletionCallback callback_;  // Consumer's callback.
   HttpResponseInfo response_;
   HttpResponseInfo auth_response_;
   const HttpResponseInfo* new_response_;
@@ -340,8 +351,10 @@ class HttpCache::Transaction : public HttpTransaction {
   bool invalid_range_;  // We may bypass the cache for this request.
   bool truncated_;  // We don't have all the response data.
   bool is_sparse_;  // The data is stored in sparse byte ranges.
-  bool server_responded_206_;
+  bool range_requested_;  // The user requested a byte range.
+  bool handling_206_;  // We must deal with this 206 response.
   bool cache_pending_;  // We are waiting for the HttpCache.
+  bool done_reading_;
   scoped_refptr<IOBuffer> read_buf_;
   int io_buf_len_;
   int read_offset_;
@@ -349,10 +362,8 @@ class HttpCache::Transaction : public HttpTransaction {
   int write_len_;
   scoped_ptr<PartialData> partial_;  // We are dealing with range requests.
   uint64 final_upload_progress_;
-  CompletionCallbackImpl<Transaction> io_callback_;
-  scoped_refptr<CancelableCompletionCallback<Transaction> > cache_callback_;
-  scoped_refptr<CancelableCompletionCallback<Transaction> >
-      write_headers_callback_;
+  base::WeakPtrFactory<Transaction> weak_factory_;
+  CompletionCallback io_callback_;
 };
 
 }  // namespace net
