@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,11 +9,13 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "net/base/net_log.h"
 #include "net/http/http_request_headers.h"
 
+namespace base {
 class Value;
+}
 
 namespace net {
 
@@ -33,7 +35,7 @@ class NetLogHttpRequestParameter : public NetLog::EventParameters {
   }
 
   // NetLog::EventParameters
-  virtual Value* ToValue() const;
+  virtual base::Value* ToValue() const OVERRIDE;
 
  private:
   virtual ~NetLogHttpRequestParameter();
@@ -54,7 +56,7 @@ class NetLogHttpResponseParameter : public NetLog::EventParameters {
   }
 
   // NetLog::EventParameters
-  virtual Value* ToValue() const;
+  virtual base::Value* ToValue() const OVERRIDE;
 
  private:
   virtual ~NetLogHttpResponseParameter();
@@ -67,4 +69,3 @@ class NetLogHttpResponseParameter : public NetLog::EventParameters {
 }  // namespace net
 
 #endif  // NET_HTTP_HTTP_NET_LOG_PARAMS_H_
-
