@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,8 +10,8 @@
 
 #include "base/basictypes.h"
 #include "base/gtest_prod_util.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/time.h"
 #include "net/base/net_log.h"
 #include "net/base/request_priority.h"
@@ -19,8 +19,8 @@
 #include "net/http/http_auth.h"
 #include "net/http/http_request_headers.h"
 #include "net/http/http_response_info.h"
+#include "net/http/http_stream_factory.h"
 #include "net/http/http_transaction.h"
-#include "net/http/stream_factory.h"
 #include "net/proxy/proxy_service.h"
 
 namespace net {
@@ -30,10 +30,12 @@ class HttpNetworkSession;
 class HttpStream;
 class HttpStreamRequest;
 class IOBuffer;
+class UploadDataStream;
 struct HttpRequestInfo;
 
-class HttpNetworkTransaction : public HttpTransaction,
-                               public StreamRequest::Delegate {
+class NET_EXPORT_PRIVATE HttpNetworkTransaction
+    : public HttpTransaction,
+      public HttpStreamRequest::Delegate {
  public:
   explicit HttpNetworkTransaction(HttpNetworkSession* session);
 
@@ -41,32 +43,46 @@ class HttpNetworkTransaction : public HttpTransaction,
 
   // HttpTransaction methods:
   virtual int Start(const HttpRequestInfo* request_info,
-                    CompletionCallback* callback,
-                    const BoundNetLog& net_log);
-  virtual int RestartIgnoringLastError(CompletionCallback* callback);
-  virtual int RestartWithCertificate(X509Certificate* client_cert,
-                                     CompletionCallback* callback);
-  virtual int RestartWithAuth(const string16& username,
-                              const string16& password,
-                              CompletionCallback* callback);
-  virtual bool IsReadyToRestartForAuth();
+                    const CompletionCallback& callback,
+                    const BoundNetLog& net_log) OVERRIDE;
+  virtual int RestartIgnoringLastError(
+      const CompletionCallback& callback) OVERRIDE;
+  virtual int RestartWithCertificate(
+      X509Certificate* client_cert,
+      const CompletionCallback& callback) OVERRIDE;
+  virtual int RestartWithAuth(const AuthCredentials& credentials,
+                              const CompletionCallback& callback) OVERRIDE;
+  virtual bool IsReadyToRestartForAuth() OVERRIDE;
 
-  virtual int Read(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual void StopCaching() {}
-  virtual const HttpResponseInfo* GetResponseInfo() const;
-  virtual LoadState GetLoadState() const;
-  virtual uint64 GetUploadProgress() const;
+  virtual int Read(IOBuffer* buf,
+                   int buf_len,
+                   const CompletionCallback& callback) OVERRIDE;
+  virtual void StopCaching() OVERRIDE {}
+  virtual void DoneReading() OVERRIDE {}
+  virtual const HttpResponseInfo* GetResponseInfo() const OVERRIDE;
+  virtual LoadState GetLoadState() const OVERRIDE;
+  virtual uint64 GetUploadProgress() const OVERRIDE;
 
-  // StreamRequest::Delegate methods:
-  virtual void OnStreamReady(HttpStream* stream);
-  virtual void OnStreamFailed(int status);
-  virtual void OnCertificateError(int status, const SSLInfo& ssl_info);
+  // HttpStreamRequest::Delegate methods:
+  virtual void OnStreamReady(const SSLConfig& used_ssl_config,
+                             const ProxyInfo& used_proxy_info,
+                             HttpStream* stream) OVERRIDE;
+  virtual void OnStreamFailed(int status,
+                              const SSLConfig& used_ssl_config) OVERRIDE;
+  virtual void OnCertificateError(int status,
+                                  const SSLConfig& used_ssl_config,
+                                  const SSLInfo& ssl_info) OVERRIDE;
   virtual void OnNeedsProxyAuth(
       const HttpResponseInfo& response_info,
-      HttpAuthController* auth_controller);
-  virtual void OnNeedsClientAuth(SSLCertRequestInfo* cert_info);
+      const SSLConfig& used_ssl_config,
+      const ProxyInfo& used_proxy_info,
+      HttpAuthController* auth_controller) OVERRIDE;
+  virtual void OnNeedsClientAuth(const SSLConfig& used_ssl_config,
+                                 SSLCertRequestInfo* cert_info) OVERRIDE;
   virtual void OnHttpsProxyTunnelResponse(const HttpResponseInfo& response_info,
-                                          HttpStream* stream);
+                                          const SSLConfig& used_ssl_config,
+                                          const ProxyInfo& used_proxy_info,
+                                          HttpStream* stream) OVERRIDE;
 
  private:
   FRIEND_TEST_ALL_PREFIXES(HttpNetworkTransactionTest, ResetStateForRestart);
@@ -84,6 +100,8 @@ class HttpNetworkTransaction : public HttpTransaction,
     STATE_GENERATE_PROXY_AUTH_TOKEN_COMPLETE,
     STATE_GENERATE_SERVER_AUTH_TOKEN,
     STATE_GENERATE_SERVER_AUTH_TOKEN_COMPLETE,
+    STATE_BUILD_REQUEST,
+    STATE_BUILD_REQUEST_COMPLETE,
     STATE_SEND_REQUEST,
     STATE_SEND_REQUEST_COMPLETE,
     STATE_READ_HEADERS,
@@ -115,6 +133,8 @@ class HttpNetworkTransaction : public HttpTransaction,
   int DoGenerateProxyAuthTokenComplete(int result);
   int DoGenerateServerAuthToken();
   int DoGenerateServerAuthTokenComplete(int result);
+  int DoBuildRequest();
+  int DoBuildRequestComplete(int result);
   int DoSendRequest();
   int DoSendRequestComplete(int result);
   int DoReadHeaders();
@@ -124,6 +144,8 @@ class HttpNetworkTransaction : public HttpTransaction,
   int DoDrainBodyForAuthRestart();
   int DoDrainBodyForAuthRestartComplete(int result);
 
+  void BuildRequestHeaders(bool using_proxy);
+
   // Record histogram of time until first byte of header is received.
   void LogTransactionConnectedMetrics();
 
@@ -206,8 +228,9 @@ class HttpNetworkTransaction : public HttpTransaction,
   // cleared by RestartWithAuth().
   HttpAuth::Target pending_auth_target_;
 
-  CompletionCallbackImpl<HttpNetworkTransaction> io_callback_;
-  CompletionCallback* user_callback_;
+  CompletionCallback io_callback_;
+  CompletionCallback callback_;
+  scoped_ptr<UploadDataStream> request_body_;
 
   scoped_refptr<HttpNetworkSession> session_;
 
@@ -215,9 +238,10 @@ class HttpNetworkTransaction : public HttpTransaction,
   const HttpRequestInfo* request_;
   HttpResponseInfo response_;
 
+  // |proxy_info_| is the ProxyInfo used by the HttpStreamRequest.
   ProxyInfo proxy_info_;
 
-  scoped_ptr<StreamRequest> stream_request_;
+  scoped_ptr<HttpStreamRequest> stream_request_;
   scoped_ptr<HttpStream> stream_;
 
   // True if we've validated the headers that the stream parser has returned.
@@ -228,7 +252,8 @@ class HttpNetworkTransaction : public HttpTransaction,
   // responses.
   bool logged_response_time_;
 
-  SSLConfig ssl_config_;
+  SSLConfig server_ssl_config_;
+  SSLConfig proxy_ssl_config_;
 
   HttpRequestHeaders request_headers_;
 
@@ -254,8 +279,6 @@ class HttpNetworkTransaction : public HttpTransaction,
   DISALLOW_COPY_AND_ASSIGN(HttpNetworkTransaction);
 };
 
-int ConvertRequestPriorityToSpdyPriority(RequestPriority priority);
-
 }  // namespace net
 
 #endif  // NET_HTTP_HTTP_NETWORK_TRANSACTION_H_
