@@ -1,10 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/http/http_request_headers.h"
 
 #include "base/logging.h"
+#include "base/stringprintf.h"
 #include "base/string_split.h"
 #include "base/string_util.h"
 #include "net/http/http_util.h"
@@ -15,6 +16,7 @@ const char HttpRequestHeaders::kGetMethod[] = "GET";
 const char HttpRequestHeaders::kAcceptCharset[] = "Accept-Charset";
 const char HttpRequestHeaders::kAcceptEncoding[] = "Accept-Encoding";
 const char HttpRequestHeaders::kAcceptLanguage[] = "Accept-Language";
+const char HttpRequestHeaders::kAuthorization[] = "Authorization";
 const char HttpRequestHeaders::kCacheControl[] = "Cache-Control";
 const char HttpRequestHeaders::kConnection[] = "Connection";
 const char HttpRequestHeaders::kContentLength[] = "Content-Length";
@@ -26,10 +28,12 @@ const char HttpRequestHeaders::kIfNoneMatch[] = "If-None-Match";
 const char HttpRequestHeaders::kIfRange[] = "If-Range";
 const char HttpRequestHeaders::kOrigin[] = "Origin";
 const char HttpRequestHeaders::kPragma[] = "Pragma";
+const char HttpRequestHeaders::kProxyAuthorization[] = "Proxy-Authorization";
 const char HttpRequestHeaders::kProxyConnection[] = "Proxy-Connection";
 const char HttpRequestHeaders::kRange[] = "Range";
 const char HttpRequestHeaders::kReferer[] = "Referer";
 const char HttpRequestHeaders::kUserAgent[] = "User-Agent";
+const char HttpRequestHeaders::kTransferEncoding[] = "Transfer-Encoding";
 
 HttpRequestHeaders::HeaderKeyValuePair::HeaderKeyValuePair() {
 }
@@ -76,13 +80,6 @@ void HttpRequestHeaders::Clear() {
   headers_.clear();
 }
 
-void HttpRequestHeaders::SetHeaderIfMissing(const base::StringPiece& key,
-                                            const base::StringPiece& value) {
-  HeaderVector::iterator it = FindHeader(key);
-  if (it == headers_.end())
-    headers_.push_back(HeaderKeyValuePair(key.as_string(), value.as_string()));
-}
-
 void HttpRequestHeaders::SetHeader(const base::StringPiece& key,
                                    const base::StringPiece& value) {
   HeaderVector::iterator it = FindHeader(key);
@@ -92,6 +89,13 @@ void HttpRequestHeaders::SetHeader(const base::StringPiece& key,
     headers_.push_back(HeaderKeyValuePair(key.as_string(), value.as_string()));
 }
 
+void HttpRequestHeaders::SetHeaderIfMissing(const base::StringPiece& key,
+                                            const base::StringPiece& value) {
+  HeaderVector::iterator it = FindHeader(key);
+  if (it == headers_.end())
+    headers_.push_back(HeaderKeyValuePair(key.as_string(), value.as_string()));
+}
+
 void HttpRequestHeaders::RemoveHeader(const base::StringPiece& key) {
   HeaderVector::iterator it = FindHeader(key);
   if (it != headers_.end())
