@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,7 +11,8 @@
 
 #include "base/basictypes.h"
 #include "base/hash_tables.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "net/base/net_export.h"
 #include "net/http/http_version.h"
 
 class Pickle;
@@ -24,7 +25,7 @@ class TimeDelta;
 namespace net {
 
 // HttpResponseHeaders: parses and holds HTTP response headers.
-class HttpResponseHeaders
+class NET_EXPORT HttpResponseHeaders
     : public base::RefCountedThreadSafe<HttpResponseHeaders> {
  public:
   // Persist options.
@@ -43,9 +44,6 @@ class HttpResponseHeaders
   // (Note that line continuations should have already been joined;
   // see HttpUtil::AssembleRawHeaders)
   //
-  // NOTE: For now, raw_headers is not really 'raw' in that this constructor is
-  // called with a 'NativeMB' string on Windows because WinHTTP does not allow
-  // us to access the raw byte sequence as sent by a web server.  In any case,
   // HttpResponseHeaders does not perform any encoding changes on the input.
   //
   explicit HttpResponseHeaders(const std::string& raw_headers);
@@ -65,6 +63,10 @@ class HttpResponseHeaders
   // Removes all instances of a particular header.
   void RemoveHeader(const std::string& name);
 
+  // Removes a particular header. The header name is compared
+  // case-insensitively.
+  void RemoveHeaderWithValue(const std::string& name, const std::string& value);
+
   // Adds a particular header.  |header| has to be a single header without any
   // EOL termination, just [<header-name>: <header-values>]
   // If a header with the same name is already stored, the two headers are not
@@ -100,9 +102,6 @@ class HttpResponseHeaders
   //
   void GetNormalizedHeaders(std::string* output) const;
 
-  // Gets the raw stored headers, in human-readable form.
-  void GetRawHeaders(std::string* output) const;
-
   // Fetch the "normalized" value of a single header, where all values for the
   // header name are separated by commas.  See the GetNormalizedHeaders for
   // format details.  Returns false if this header wasn't found.
@@ -240,6 +239,9 @@ class HttpResponseHeaders
                        int64* last_byte_position,
                        int64* instance_length) const;
 
+  // Returns true if the response is chunk-encoded.
+  bool IsChunkEncoded() const;
+
   // Returns the HTTP response code.  This is 0 if the response code text seems
   // to exist but could not be parsed.  Otherwise, it defaults to 200 if the
   // response code is not found in the raw headers.
@@ -276,7 +278,7 @@ class HttpResponseHeaders
   // construct a valid one.  Example input:
   //    HTTP/1.1 200 OK
   // with line_begin and end pointing at the begin and end of this line.
-  // Output will be a normalized version of this, with a trailing \n.
+  // Output will be a normalized version of this.
   void ParseStatusLine(std::string::const_iterator line_begin,
                        std::string::const_iterator line_end,
                        bool has_headers);
@@ -305,6 +307,16 @@ class HttpResponseHeaders
   void MergeWithHeaders(const std::string& raw_headers,
                         const HeaderSet& headers_to_remove);
 
+  // Replaces the current headers with the merged version of |raw_headers| and
+  // the current headers with out the header consisting of
+  // |header_to_remove_name| and |header_to_remove_value|. Note that
+  // |header_to_remove_name| is compared case-insensitively.
+  // Note that the header to remove is removed from the current headers (before
+  // the merge), not after the merge.
+  void MergeWithHeadersWithValue(const std::string& raw_headers,
+                                 const std::string& header_to_remove_name,
+                                 const std::string& header_to_remove_value);
+
   // Adds the values from any 'cache-control: no-cache="foo,bar"' headers.
   void AddNonCacheableHeaders(HeaderSet* header_names) const;
 
