@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -22,12 +22,17 @@ namespace net {
 // serialized HttpResponseInfo.
 enum {
   // The version of the response info used when persisting response info.
-  RESPONSE_INFO_VERSION = 1,
+  RESPONSE_INFO_VERSION = 2,
+
+  // The minimum version supported for deserializing response info.
+  RESPONSE_INFO_MINIMUM_VERSION = 1,
 
   // We reserve up to 8 bits for the version number.
   RESPONSE_INFO_VERSION_MASK = 0xFF,
 
   // This bit is set if the response info has a cert at the end.
+  // Version 1 serialized only the end-entity certificate, while subsequent
+  // versions include the available certificate chain.
   RESPONSE_INFO_HAS_CERT = 1 << 8,
 
   // This bit is set if the response info has a security-bits field (security
@@ -52,9 +57,13 @@ enum {
   // This bit is set if the request was fetched via an explicit proxy.
   RESPONSE_INFO_WAS_PROXY = 1 << 15,
 
-  // This bit is set if response could use alternate protocol. However, browser
-  // will ingore the alternate protocol if spdy is not enabled.
-  RESPONSE_INFO_WAS_ALTERNATE_PROTOCOL_AVAILABLE = 1 << 16,
+  // This bit is set if the response info has an SSL connection status field.
+  // This contains the ciphersuite used to fetch the resource as well as the
+  // protocol version, compression method and whether SSLv3 fallback was used.
+  RESPONSE_INFO_HAS_SSL_CONNECTION_STATUS = 1 << 16,
+
+  // This bit is set if the response info has protocol version.
+  RESPONSE_INFO_HAS_NPN_NEGOTIATED_PROTOCOL = 1 << 17,
 
   // TODO(darin): Add other bits to indicate alternate request methods.
   // For now, we don't support storing those.
@@ -64,7 +73,6 @@ HttpResponseInfo::HttpResponseInfo()
     : was_cached(false),
       was_fetched_via_spdy(false),
       was_npn_negotiated(false),
-      was_alternate_protocol_available(false),
       was_fetched_via_proxy(false) {
 }
 
@@ -72,8 +80,9 @@ HttpResponseInfo::HttpResponseInfo(const HttpResponseInfo& rhs)
     : was_cached(rhs.was_cached),
       was_fetched_via_spdy(rhs.was_fetched_via_spdy),
       was_npn_negotiated(rhs.was_npn_negotiated),
-      was_alternate_protocol_available(rhs.was_alternate_protocol_available),
       was_fetched_via_proxy(rhs.was_fetched_via_proxy),
+      socket_address(rhs.socket_address),
+      npn_negotiated_protocol(rhs.npn_negotiated_protocol),
       request_time(rhs.request_time),
       response_time(rhs.response_time),
       auth_challenge(rhs.auth_challenge),
@@ -91,8 +100,9 @@ HttpResponseInfo& HttpResponseInfo::operator=(const HttpResponseInfo& rhs) {
   was_cached = rhs.was_cached;
   was_fetched_via_spdy = rhs.was_fetched_via_spdy;
   was_npn_negotiated = rhs.was_npn_negotiated;
-  was_alternate_protocol_available = rhs.was_alternate_protocol_available;
   was_fetched_via_proxy = rhs.was_fetched_via_proxy;
+  socket_address = rhs.socket_address;
+  npn_negotiated_protocol = rhs.npn_negotiated_protocol;
   request_time = rhs.request_time;
   response_time = rhs.response_time;
   auth_challenge = rhs.auth_challenge;
@@ -113,7 +123,8 @@ bool HttpResponseInfo::InitFromPickle(const Pickle& pickle,
   if (!pickle.ReadInt(&iter, &flags))
     return false;
   int version = flags & RESPONSE_INFO_VERSION_MASK;
-  if (version != RESPONSE_INFO_VERSION) {
+  if (version < RESPONSE_INFO_MINIMUM_VERSION ||
+      version > RESPONSE_INFO_VERSION) {
     DLOG(ERROR) << "unexpected response info version: " << version;
     return false;
   }
@@ -132,16 +143,21 @@ bool HttpResponseInfo::InitFromPickle(const Pickle& pickle,
 
   // read response-headers
   headers = new HttpResponseHeaders(pickle, &iter);
-  DCHECK_NE(headers->response_code(), -1);
+  if (headers->response_code() == -1)
+    return false;
 
   // read ssl-info
   if (flags & RESPONSE_INFO_HAS_CERT) {
-    ssl_info.cert =
-        X509Certificate::CreateFromPickle(pickle, &iter);
+    X509Certificate::PickleType type = (version == 1) ?
+        X509Certificate::PICKLETYPE_SINGLE_CERTIFICATE :
+        X509Certificate::PICKLETYPE_CERTIFICATE_CHAIN;
+    ssl_info.cert = X509Certificate::CreateFromPickle(pickle, &iter, type);
+    if (!ssl_info.cert)
+      return false;
   }
   if (flags & RESPONSE_INFO_HAS_CERT_STATUS) {
-    int cert_status;
-    if (!pickle.ReadInt(&iter, &cert_status))
+    CertStatus cert_status;
+    if (!pickle.ReadUInt32(&iter, &cert_status))
       return false;
     ssl_info.cert_status = cert_status;
   }
@@ -152,19 +168,43 @@ bool HttpResponseInfo::InitFromPickle(const Pickle& pickle,
     ssl_info.security_bits = security_bits;
   }
 
+  if (flags & RESPONSE_INFO_HAS_SSL_CONNECTION_STATUS) {
+    int connection_status;
+    if (!pickle.ReadInt(&iter, &connection_status))
+      return false;
+    ssl_info.connection_status = connection_status;
+  }
+
   // read vary-data
   if (flags & RESPONSE_INFO_HAS_VARY_DATA) {
     if (!vary_data.InitFromPickle(pickle, &iter))
       return false;
   }
 
+  // Read socket_address.
+  std::string socket_address_host;
+  if (pickle.ReadString(&iter, &socket_address_host)) {
+    // If the host was written, we always expect the port to follow.
+    uint16 socket_address_port;
+    if (!pickle.ReadUInt16(&iter, &socket_address_port))
+      return false;
+    socket_address = HostPortPair(socket_address_host, socket_address_port);
+  } else if (version > 1) {
+    // socket_address was not always present in version 1 of the response
+    // info, so we don't fail if it can't be read.
+    return false;
+  }
+
+  // read protocol-version.
+  if (flags & RESPONSE_INFO_HAS_NPN_NEGOTIATED_PROTOCOL) {
+    if (!pickle.ReadString(&iter, &npn_negotiated_protocol))
+      return false;
+  }
+
   was_fetched_via_spdy = (flags & RESPONSE_INFO_WAS_SPDY) != 0;
 
   was_npn_negotiated = (flags & RESPONSE_INFO_WAS_NPN) != 0;
 
-  was_alternate_protocol_available =
-      (flags & RESPONSE_INFO_WAS_ALTERNATE_PROTOCOL_AVAILABLE) != 0;
-
   was_fetched_via_proxy = (flags & RESPONSE_INFO_WAS_PROXY) != 0;
 
   *response_truncated = (flags & RESPONSE_INFO_TRUNCATED) ? true : false;
@@ -181,7 +221,8 @@ void HttpResponseInfo::Persist(Pickle* pickle,
     flags |= RESPONSE_INFO_HAS_CERT_STATUS;
     if (ssl_info.security_bits != -1)
       flags |= RESPONSE_INFO_HAS_SECURITY_BITS;
-    // TODO(wtc): we should persist ssl_info.connection_status.
+    if (ssl_info.connection_status != 0)
+      flags |= RESPONSE_INFO_HAS_SSL_CONNECTION_STATUS;
   }
   if (vary_data.is_valid())
     flags |= RESPONSE_INFO_HAS_VARY_DATA;
@@ -189,10 +230,10 @@ void HttpResponseInfo::Persist(Pickle* pickle,
     flags |= RESPONSE_INFO_TRUNCATED;
   if (was_fetched_via_spdy)
     flags |= RESPONSE_INFO_WAS_SPDY;
-  if (was_npn_negotiated)
+  if (was_npn_negotiated) {
     flags |= RESPONSE_INFO_WAS_NPN;
-  if (was_alternate_protocol_available)
-    flags |= RESPONSE_INFO_WAS_ALTERNATE_PROTOCOL_AVAILABLE;
+    flags |= RESPONSE_INFO_HAS_NPN_NEGOTIATED_PROTOCOL;
+  }
   if (was_fetched_via_proxy)
     flags |= RESPONSE_INFO_WAS_PROXY;
 
@@ -216,13 +257,21 @@ void HttpResponseInfo::Persist(Pickle* pickle,
 
   if (ssl_info.is_valid()) {
     ssl_info.cert->Persist(pickle);
-    pickle->WriteInt(ssl_info.cert_status);
+    pickle->WriteUInt32(ssl_info.cert_status);
     if (ssl_info.security_bits != -1)
       pickle->WriteInt(ssl_info.security_bits);
+    if (ssl_info.connection_status != 0)
+      pickle->WriteInt(ssl_info.connection_status);
   }
 
   if (vary_data.is_valid())
     vary_data.Persist(pickle);
+
+  pickle->WriteString(socket_address.host());
+  pickle->WriteUInt16(socket_address.port());
+
+  if (was_npn_negotiated)
+    pickle->WriteString(npn_negotiated_protocol);
 }
 
 }  // namespace net
