@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,11 @@
 #define NET_HTTP_HTTP_RESPONSE_INFO_H_
 #pragma once
 
+#include <string>
+
 #include "base/time.h"
+#include "net/base/host_port_pair.h"
+#include "net/base/net_export.h"
 #include "net/base/ssl_info.h"
 #include "net/http/http_vary_data.h"
 
@@ -19,7 +23,7 @@ class HttpResponseHeaders;
 class IOBufferWithSize;
 class SSLCertRequestInfo;
 
-class HttpResponseInfo {
+class NET_EXPORT HttpResponseInfo {
  public:
   HttpResponseInfo();
   HttpResponseInfo(const HttpResponseInfo& rhs);
@@ -51,15 +55,23 @@ class HttpResponseInfo {
   // True if the npn was negotiated for this request.
   bool was_npn_negotiated;
 
-  // True if response could use alternate protocol. However, browser
-  // will ingore the alternate protocol if spdy is not enabled.
-  bool was_alternate_protocol_available;
-
   // True if the request was fetched via an explicit proxy.  The proxy could
   // be any type of proxy, HTTP or SOCKS.  Note, we do not know if a
   // transparent proxy may have been involved.
   bool was_fetched_via_proxy;
 
+  // Remote address of the socket which fetched this resource.
+  //
+  // NOTE: If the response was served from the cache (was_cached is true),
+  // the socket address will be set to the address that the content came from
+  // originally.  This is true even if the response was re-validated using a
+  // different remote address, or if some of the content came from a byte-range
+  // request to a different address.
+  HostPortPair socket_address;
+
+  // Protocol negotiated with the server.
+  std::string npn_negotiated_protocol;
+
   // The time at which the request was made that resulted in this response.
   // For cached responses, this is the last time the cache entry was validated.
   base::Time request_time;
