@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,8 @@
 #define NET_HTTP_HTTP_TRANSACTION_FACTORY_H__
 #pragma once
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
@@ -16,7 +17,7 @@ class HttpTransaction;
 
 
 // An interface to a class that can create HttpTransaction objects.
-class HttpTransactionFactory {
+class NET_EXPORT HttpTransactionFactory {
  public:
   virtual ~HttpTransactionFactory() {}
 
@@ -29,10 +30,6 @@ class HttpTransactionFactory {
 
   // Returns the associated HttpNetworkSession used by new transactions.
   virtual HttpNetworkSession* GetSession() = 0;
-
-  // Suspends the creation of new transactions. If |suspend| is false, creation
-  // of new transactions is resumed.
-  virtual void Suspend(bool suspend) = 0;
 };
 
 }  // namespace net
