@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,9 +9,10 @@
 #include <string>
 #include <vector>
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/string_tokenizer.h"
 #include "googleurl/src/gurl.h"
+#include "net/base/net_export.h"
 #include "net/http/http_byte_range.h"
 
 // This is a macro to support extending this string literal at compile time.
@@ -20,13 +21,7 @@
 
 namespace net {
 
-class HttpAuthController;
-struct HttpRequestInfo;
-class HttpRequestHeaders;
-class HttpStream;
-class UploadDataStream;
-
-class HttpUtil {
+class NET_EXPORT HttpUtil {
  public:
   // Returns the absolute path of the URL, to be used for the http request.
   // The absolute path starts with a '/' and may contain a query.
@@ -48,11 +43,14 @@ class HttpUtil {
   // Parses the value of a Content-Type header.  The resulting mime_type and
   // charset values are normalized to lowercase.  The mime_type and charset
   // output values are only modified if the content_type_str contains a mime
-  // type and charset value, respectively.
+  // type and charset value, respectively.  The boundary output value is
+  // optional and will be assigned the (quoted) value of the boundary
+  // paramter, if any.
   static void ParseContentType(const std::string& content_type_str,
                                std::string* mime_type,
                                std::string* charset,
-                               bool *had_charset);
+                               bool* had_charset,
+                               std::string* boundary);
 
   // Scans the headers and look for the first "Range" header in |headers|,
   // if "Range" exists and the first one of it is well formatted then returns
@@ -73,6 +71,10 @@ class HttpUtil {
   // TODO(darin): kill this
   static bool HasHeader(const std::string& headers, const char* name);
 
+  // Returns true if it is safe to allow users and scripts to specify the header
+  // named |name|.
+  static bool IsSafeHeader(const std::string& name);
+
   // Strips all header lines from |headers| whose name matches
   // |headers_to_remove|. |headers_to_remove| is a list of null-terminated
   // lower-case header names, with array length |headers_to_remove_len|.
@@ -102,6 +104,13 @@ class HttpUtil {
   // Whether the character is the start of a quotation mark.
   static bool IsQuote(char c);
 
+  // Whether the string is a valid |token| as defined in RFC 2616 Sec 2.2.
+  static bool IsToken(std::string::const_iterator begin,
+                      std::string::const_iterator end);
+  static bool IsToken(const std::string& str) {
+    return IsToken(str.begin(), str.end());
+  }
+
   // RFC 2616 Sec 2.2:
   // quoted-string = ( <"> *(qdtext | quoted-pair ) <"> )
   // Unquote() strips the surrounding quotemarks off a string, and unescapes
@@ -134,8 +143,22 @@ class HttpUtil {
   // handling HTTP line continuations (i.e., lines starting with LWS are
   // continuations of the previous line).  |buf_len| indicates the position of
   // the end-of-headers marker as defined by LocateEndOfHeaders.
+  // If a \0 appears within the headers themselves, it will be stripped. This
+  // is a workaround to avoid later code from incorrectly interpreting it as
+  // a line terminator.
+  //
+  // TODO(eroman): we should use \n as the canonical line separator rather than
+  //               \0 to avoid this problem. Unfortunately the persistence layer
+  //               is already dependent on newlines being replaced by NULL so
+  //               this is hard to change without breaking things.
   static std::string AssembleRawHeaders(const char* buf, int buf_len);
 
+  // Converts assembled "raw headers" back to the HTTP response format. That is
+  // convert each \0 occurence to CRLF. This is used by DevTools.
+  // Since all line continuations info is already lost at this point, the result
+  // consists of status line and then one line for each header.
+  static std::string ConvertHeadersBackToHTTPResponse(const std::string& str);
+
   // Given a comma separated ordered list of language codes, return
   // the list with a qvalue appended to each language.
   // The way qvalues are assigned is rather simple. The qvalue
@@ -164,24 +187,11 @@ class HttpUtil {
                                     const std::string& header_value,
                                     std::string* headers);
 
-  // Constructs |request_headers| from the information contained in
-  // |request_info|.  The correct server and proxy auth headers will
-  // be populated from |auth_controllers| if |enable_server_auth| or
-  // |enable_proxy_auth| is true.
-  static void BuildRequestHeaders(const HttpRequestInfo* request_info,
-                                  const UploadDataStream* upload_data_stream,
-                                  const scoped_refptr<HttpAuthController>
-                                      auth_controllers[],
-                                  bool enable_server_auth,
-                                  bool enable_proxy_auth,
-                                  bool enable_full_url,
-                                  HttpRequestHeaders* request_headers);
-
   // Used to iterate over the name/value pairs of HTTP headers.  To iterate
   // over the values in a multi-value header, use ValuesIterator.
   // See AssembleRawHeaders for joining line continuations (this iterator
   // does not expect any).
-  class HeadersIterator {
+  class NET_EXPORT HeadersIterator {
    public:
     HeadersIterator(std::string::const_iterator headers_begin,
                     std::string::const_iterator headers_end,
@@ -244,7 +254,7 @@ class HttpUtil {
   // This iterator is careful to skip over delimiters found inside an HTTP
   // quoted string.
   //
-  class ValuesIterator {
+  class NET_EXPORT_PRIVATE ValuesIterator {
    public:
     ValuesIterator(std::string::const_iterator values_begin,
                    std::string::const_iterator values_end,
@@ -278,7 +288,7 @@ class HttpUtil {
   //
   // String iterators returned from this class' methods may be invalidated upon
   // calls to GetNext() or after the NameValuePairsIterator is destroyed.
-  class NameValuePairsIterator {
+  class NET_EXPORT NameValuePairsIterator {
    public:
     NameValuePairsIterator(std::string::const_iterator begin,
                            std::string::const_iterator end,
@@ -307,9 +317,13 @@ class HttpUtil {
     }
     std::string value() const {
       return value_is_quoted_ ? unquoted_value_ : std::string(value_begin_,
-                                                               value_end_);
+                                                              value_end_);
     }
 
+    // The value before unquoting (if any).
+    std::string raw_value() const { return std::string(value_begin_,
+                                                       value_end_); }
+
    private:
     HttpUtil::ValuesIterator props_;
     bool valid_;
