@@ -1,10 +1,11 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <algorithm>
 
 #include "base/basictypes.h"
+#include "base/string_util.h"
 #include "net/http/http_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -14,6 +15,84 @@ namespace {
 class HttpUtilTest : public testing::Test {};
 }
 
+TEST(HttpUtilTest, IsSafeHeader) {
+  static const char* unsafe_headers[] = {
+    "sec-",
+    "sEc-",
+    "sec-foo",
+    "sEc-FoO",
+    "proxy-",
+    "pRoXy-",
+    "proxy-foo",
+    "pRoXy-FoO",
+    "accept-charset",
+    "accept-encoding",
+    "connection",
+    "content-length",
+    "cookie",
+    "cookie2",
+    "content-transfer-encoding",
+    "date",
+    "expect",
+    "host",
+    "keep-alive",
+    "origin",
+    "referer",
+    "te",
+    "trailer",
+    "transfer-encoding",
+    "upgrade",
+    "user-agent",
+    "via",
+  };
+  for (size_t i = 0; i < arraysize(unsafe_headers); ++i) {
+    EXPECT_FALSE(HttpUtil::IsSafeHeader(unsafe_headers[i]))
+      << unsafe_headers[i];
+    EXPECT_FALSE(HttpUtil::IsSafeHeader(StringToUpperASCII(std::string(
+        unsafe_headers[i])))) << unsafe_headers[i];
+  }
+  static const char* safe_headers[] = {
+    "foo",
+    "x-",
+    "x-foo",
+    "content-disposition",
+    "update",
+    "accept-charseta",
+    "accept_charset",
+    "accept-encodinga",
+    "accept_encoding",
+    "connectiona",
+    "content-lengtha",
+    "content_length",
+    "cookiea",
+    "cookie2a",
+    "cookie3",
+    "content-transfer-encodinga",
+    "content_transfer_encoding",
+    "datea",
+    "expecta",
+    "hosta",
+    "keep-alivea",
+    "keep_alive",
+    "origina",
+    "referera",
+    "referrer",
+    "tea",
+    "trailera",
+    "transfer-encodinga",
+    "transfer_encoding",
+    "upgradea",
+    "user-agenta",
+    "user_agent",
+    "viaa",
+  };
+  for (size_t i = 0; i < arraysize(safe_headers); ++i) {
+    EXPECT_TRUE(HttpUtil::IsSafeHeader(safe_headers[i])) << safe_headers[i];
+    EXPECT_TRUE(HttpUtil::IsSafeHeader(StringToUpperASCII(std::string(
+        safe_headers[i])))) << safe_headers[i];
+  }
+}
+
 TEST(HttpUtilTest, HasHeader) {
   static const struct {
     const char* headers;
@@ -195,7 +274,7 @@ TEST(HttpUtilTest, LocateEndOfHeaders) {
 
 TEST(HttpUtilTest, AssembleRawHeaders) {
   struct {
-    const char* input;
+    const char* input;  // with '|' representing '\0'
     const char* expected_result;  // with '\0' changed to '|'
   } tests[] = {
     { "HTTP/1.0 200 OK\r\nFoo: 1\r\nBar: 2\r\n\r\n",
@@ -482,12 +561,26 @@ TEST(HttpUtilTest, AssembleRawHeaders) {
       "Bar: 2||",
     },
 
+    // Embed NULLs in the status line. They should not be understood
+    // as line separators.
+    {
+      "HTTP/1.0 200 OK|Bar2:0|Baz2:1\r\nFoo: 1\r\nBar: 2\r\n\r\n",
+      "HTTP/1.0 200 OKBar2:0Baz2:1|Foo: 1|Bar: 2||"
+    },
+
+    // Embed NULLs in a header line. They should not be understood as
+    // line separators.
+    {
+      "HTTP/1.0 200 OK\nFoo: 1|Foo2: 3\nBar: 2\n\n",
+      "HTTP/1.0 200 OK|Foo: 1Foo2: 3|Bar: 2||"
+    },
   };
   for (size_t i = 0; i < ARRAYSIZE_UNSAFE(tests); ++i) {
-    int input_len = static_cast<int>(strlen(tests[i].input));
-    std::string raw = HttpUtil::AssembleRawHeaders(tests[i].input, input_len);
+    std::string input = tests[i].input;
+    std::replace(input.begin(), input.end(), '|', '\0');
+    std::string raw = HttpUtil::AssembleRawHeaders(input.data(), input.size());
     std::replace(raw.begin(), raw.end(), '\0', '|');
-    EXPECT_TRUE(raw == tests[i].expected_result);
+    EXPECT_EQ(tests[i].expected_result, raw);
   }
 }
 
@@ -539,6 +632,91 @@ TEST(HttpUtilTest, GenerateAcceptCharsetHeader) {
             HttpUtil::GenerateAcceptCharsetHeader("EUC-JP"));
 }
 
+// HttpResponseHeadersTest.GetMimeType also tests ParseContentType.
+TEST(HttpUtilTest, ParseContentType) {
+  const struct {
+    const char* content_type;
+    const char* expected_mime_type;
+    const char* expected_charset;
+    const bool expected_had_charset;
+    const char* expected_boundary;
+  } tests[] = {
+    { "text/html; charset=utf-8",
+      "text/html",
+      "utf-8",
+      true,
+      ""
+    },
+    { "text/html; charset =utf-8",
+      "text/html",
+      "utf-8",
+      true,
+      ""
+    },
+    { "text/html; charset= utf-8",
+      "text/html",
+      "utf-8",
+      true,
+      ""
+    },
+    { "text/html; charset=utf-8 ",
+      "text/html",
+      "utf-8",
+      true,
+      ""
+    },
+    { "text/html; boundary=\"WebKit-ada-df-dsf-adsfadsfs\"",
+      "text/html",
+      "",
+      false,
+      "\"WebKit-ada-df-dsf-adsfadsfs\""
+    },
+    { "text/html; boundary =\"WebKit-ada-df-dsf-adsfadsfs\"",
+      "text/html",
+      "",
+      false,
+      "\"WebKit-ada-df-dsf-adsfadsfs\""
+    },
+    { "text/html; boundary= \"WebKit-ada-df-dsf-adsfadsfs\"",
+      "text/html",
+      "",
+      false,
+      "\"WebKit-ada-df-dsf-adsfadsfs\""
+    },
+    { "text/html; boundary= \"WebKit-ada-df-dsf-adsfadsfs\"   ",
+      "text/html",
+      "",
+      false,
+      "\"WebKit-ada-df-dsf-adsfadsfs\""
+    },
+    { "text/html; boundary=\"WebKit-ada-df-dsf-adsfadsfs  \"",
+      "text/html",
+      "",
+      false,
+      "\"WebKit-ada-df-dsf-adsfadsfs  \""
+    },
+    { "text/html; boundary=WebKit-ada-df-dsf-adsfadsfs",
+      "text/html",
+      "",
+      false,
+      "WebKit-ada-df-dsf-adsfadsfs"
+    },
+    // TODO(abarth): Add more interesting test cases.
+  };
+  for (size_t i = 0; i < ARRAYSIZE_UNSAFE(tests); ++i) {
+    std::string mime_type;
+    std::string charset;
+    bool had_charset = false;
+    std::string boundary;
+    net::HttpUtil::ParseContentType(tests[i].content_type, &mime_type,
+                                    &charset, &had_charset, &boundary);
+    EXPECT_EQ(tests[i].expected_mime_type, mime_type) << "i=" << i;
+    EXPECT_EQ(tests[i].expected_charset, charset) << "i=" << i;
+    EXPECT_EQ(tests[i].expected_had_charset, had_charset) << "i=" << i;
+    EXPECT_EQ(tests[i].expected_boundary, boundary) << "i=" << i;
+  }
+}
+
 TEST(HttpUtilTest, ParseRanges) {
   const struct {
     const char* headers;
@@ -673,27 +851,48 @@ TEST(HttpUtilTest, ParseRanges) {
 }
 
 namespace {
-
-void CheckNameValuePair(HttpUtil::NameValuePairsIterator* parser,
-                        bool expect_next,
-                        bool expect_valid,
-                        std::string expected_name,
-                        std::string expected_value,
-                        std::string expected_unquoted_value) {
-  ASSERT_EQ(expect_next, parser->GetNext());
+void CheckCurrentNameValuePair(HttpUtil::NameValuePairsIterator* parser,
+                               bool expect_valid,
+                               std::string expected_name,
+                               std::string expected_value) {
   ASSERT_EQ(expect_valid, parser->valid());
-  if (!expect_next || !expect_valid) {
+  if (!expect_valid) {
     return;
   }
+
+  // Let's make sure that these never change (i.e., when a quoted value is
+  // unquoted, it should be cached on the first calls and not regenerated
+  // later).
+  std::string::const_iterator first_value_begin = parser->value_begin();
+  std::string::const_iterator first_value_end = parser->value_end();
+
   ASSERT_EQ(expected_name, std::string(parser->name_begin(),
                                        parser->name_end()));
   ASSERT_EQ(expected_name, parser->name());
   ASSERT_EQ(expected_value, std::string(parser->value_begin(),
                                         parser->value_end()));
   ASSERT_EQ(expected_value, parser->value());
-  ASSERT_EQ(expected_unquoted_value, parser->unquoted_value());
-  ASSERT_EQ(expected_value != expected_unquoted_value,
-            parser->value_is_quoted());
+
+  // Make sure they didn't/don't change.
+  ASSERT_TRUE(first_value_begin == parser->value_begin());
+  ASSERT_TRUE(first_value_end == parser->value_end());
+}
+
+void CheckNextNameValuePair(HttpUtil::NameValuePairsIterator* parser,
+                            bool expect_next,
+                            bool expect_valid,
+                            std::string expected_name,
+                            std::string expected_value) {
+  ASSERT_EQ(expect_next, parser->GetNext());
+  ASSERT_EQ(expect_valid, parser->valid());
+  if (!expect_next || !expect_valid) {
+    return;
+  }
+
+  CheckCurrentNameValuePair(parser,
+                            expect_valid,
+                            expected_name,
+                            expected_value);
 }
 
 void CheckInvalidNameValuePair(std::string valid_part,
@@ -731,35 +930,78 @@ void CheckInvalidNameValuePair(std::string valid_part,
 
 }  // anonymous namespace
 
+TEST(HttpUtilTest, NameValuePairsIteratorCopyAndAssign) {
+  std::string data = "alpha='\\'a\\''; beta=\" b \"; cappa='c;'; delta=\"d\"";
+  HttpUtil::NameValuePairsIterator parser_a(data.begin(), data.end(), ';');
+
+  EXPECT_TRUE(parser_a.valid());
+  ASSERT_NO_FATAL_FAILURE(
+      CheckNextNameValuePair(&parser_a, true, true, "alpha", "'a'"));
+
+  HttpUtil::NameValuePairsIterator parser_b(parser_a);
+  // a and b now point to same location
+  ASSERT_NO_FATAL_FAILURE(
+      CheckCurrentNameValuePair(&parser_b, true, "alpha", "'a'"));
+  ASSERT_NO_FATAL_FAILURE(
+      CheckCurrentNameValuePair(&parser_a, true, "alpha", "'a'"));
+
+  // advance a, no effect on b
+  ASSERT_NO_FATAL_FAILURE(
+      CheckNextNameValuePair(&parser_a, true, true, "beta", " b "));
+  ASSERT_NO_FATAL_FAILURE(
+      CheckCurrentNameValuePair(&parser_b, true, "alpha", "'a'"));
+
+  // assign b the current state of a, no effect on a
+  parser_b = parser_a;
+  ASSERT_NO_FATAL_FAILURE(
+      CheckCurrentNameValuePair(&parser_b, true, "beta", " b "));
+  ASSERT_NO_FATAL_FAILURE(
+      CheckCurrentNameValuePair(&parser_a, true, "beta", " b "));
+
+  // advance b, no effect on a
+  ASSERT_NO_FATAL_FAILURE(
+      CheckNextNameValuePair(&parser_b, true, true, "cappa", "c;"));
+  ASSERT_NO_FATAL_FAILURE(
+      CheckCurrentNameValuePair(&parser_a, true, "beta", " b "));
+}
+
 TEST(HttpUtilTest, NameValuePairsIteratorEmptyInput) {
   std::string data = "";
   HttpUtil::NameValuePairsIterator parser(data.begin(), data.end(), ';');
 
   EXPECT_TRUE(parser.valid());
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, false, true, "", "", ""));
+      CheckNextNameValuePair(&parser, false, true, "", ""));
 }
 
 TEST(HttpUtilTest, NameValuePairsIterator) {
-  std::string data = "alpha=1; beta= 2 ;cappa =' 3 ';"
-                     "delta= \" 4 \"; e= \" '5'\"; e=6";
+  std::string data = "alpha=1; beta= 2 ;cappa =' 3; ';"
+                     "delta= \" \\\"4\\\" \"; e= \" '5'\"; e=6;"
+                     "f='\\'\\h\\e\\l\\l\\o\\ \\w\\o\\r\\l\\d\\'';"
+                     "g=''; h='hello'";
   HttpUtil::NameValuePairsIterator parser(data.begin(), data.end(), ';');
   EXPECT_TRUE(parser.valid());
 
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "alpha", "1", "1"));
+      CheckNextNameValuePair(&parser, true, true, "alpha", "1"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "beta", "2", "2"));
+      CheckNextNameValuePair(&parser, true, true, "beta", "2"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "cappa", "' 3 '", " 3 "));
+      CheckNextNameValuePair(&parser, true, true, "cappa", " 3; "));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "delta", "\" 4 \"", " 4 "));
+      CheckNextNameValuePair(&parser, true, true, "delta", " \"4\" "));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "e", "\" '5'\"", " '5'"));
+      CheckNextNameValuePair(&parser, true, true, "e", " '5'"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "e", "6", "6"));
+      CheckNextNameValuePair(&parser, true, true, "e", "6"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, false, true, "", "", ""));
+      CheckNextNameValuePair(&parser, true, true, "f", "'hello world'"));
+  ASSERT_NO_FATAL_FAILURE(
+      CheckNextNameValuePair(&parser, true, true, "g", ""));
+  ASSERT_NO_FATAL_FAILURE(
+      CheckNextNameValuePair(&parser, true, true, "h", "hello"));
+  ASSERT_NO_FATAL_FAILURE(
+      CheckNextNameValuePair(&parser, false, true, "", ""));
 }
 
 TEST(HttpUtilTest, NameValuePairsIteratorIllegalInputs) {
@@ -768,6 +1010,9 @@ TEST(HttpUtilTest, NameValuePairsIteratorIllegalInputs) {
 
   ASSERT_NO_FATAL_FAILURE(CheckInvalidNameValuePair("alpha=1", "; 'beta'=2"));
   ASSERT_NO_FATAL_FAILURE(CheckInvalidNameValuePair("", "'beta'=2"));
+  ASSERT_NO_FATAL_FAILURE(CheckInvalidNameValuePair("alpha=1", ";beta="));
+  ASSERT_NO_FATAL_FAILURE(CheckInvalidNameValuePair("alpha=1",
+                                                    ";beta=;cappa=2"));
 
   // According to the spec this is an error, but it doesn't seem appropriate to
   // change our behaviour to be less permissive at this time.
@@ -783,13 +1028,13 @@ TEST(HttpUtilTest, NameValuePairsIteratorExtraSeparators) {
   EXPECT_TRUE(parser.valid());
 
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "alpha", "1", "1"));
+      CheckNextNameValuePair(&parser, true, true, "alpha", "1"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "beta", "2", "2"));
+      CheckNextNameValuePair(&parser, true, true, "beta", "2"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "cappa", "3", "3"));
+      CheckNextNameValuePair(&parser, true, true, "cappa", "3"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, false, true, "", "", ""));
+      CheckNextNameValuePair(&parser, false, true, "", ""));
 }
 
 // See comments on the implementation of NameValuePairsIterator::GetNext
@@ -800,7 +1045,7 @@ TEST(HttpUtilTest, NameValuePairsIteratorMissingEndQuote) {
   EXPECT_TRUE(parser.valid());
 
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, true, true, "name", "value", "value"));
+      CheckNextNameValuePair(&parser, true, true, "name", "value"));
   ASSERT_NO_FATAL_FAILURE(
-    CheckNameValuePair(&parser, false, true, "", "", ""));
+      CheckNextNameValuePair(&parser, false, true, "", ""));
 }
