@@ -1,9 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/http/partial_data.h"
 
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/format_macros.h"
 #include "base/logging.h"
 #include "base/string_number_conversions.h"
@@ -66,13 +68,12 @@ class PartialData::Core {
 
   PartialData* owner_;
   int64 start_;
-  net::CompletionCallbackImpl<Core> callback_;
+
   DISALLOW_COPY_AND_ASSIGN(Core);
 };
 
 PartialData::Core::Core(PartialData* owner)
-    : owner_(owner),
-      ALLOW_THIS_IN_INITIALIZER_LIST(callback_(this, &Core::OnIOComplete)) {
+    : owner_(owner), start_(0) {
   DCHECK(!owner_->core_);
   owner_->core_ = this;
 }
@@ -89,7 +90,9 @@ void PartialData::Core::Cancel() {
 
 int PartialData::Core::GetAvailableRange(disk_cache::Entry* entry, int64 offset,
                                          int len, int64* start) {
-  int rv = entry->GetAvailableRange(offset, len, &start_, &callback_);
+  int rv = entry->GetAvailableRange(
+      offset, len, &start_, base::Bind(&PartialData::Core::OnIOComplete,
+                                       base::Unretained(this)));
   if (rv != net::ERR_IO_PENDING) {
     // The callback will not be invoked. Lets cleanup.
     *start = start_;
@@ -111,8 +114,8 @@ PartialData::PartialData()
       final_range_(false),
       sparse_entry_(true),
       truncated_(false),
-      core_(NULL),
-      callback_(NULL) {
+      initial_validation_(false),
+      core_(NULL) {
 }
 
 PartialData::~PartialData() {
@@ -153,12 +156,12 @@ void PartialData::RestoreHeaders(HttpRequestHeaders* headers) const {
               byte_range_.suffix_length() : byte_range_.last_byte_position();
 
   headers->CopyFrom(extra_headers_);
-  if (byte_range_.IsValid())
+  if (!truncated_ && byte_range_.IsValid())
     AddRangeHeader(current_range_start_, end, headers);
 }
 
 int PartialData::ShouldValidateCache(disk_cache::Entry* entry,
-                                     CompletionCallback* callback) {
+                                     const CompletionCallback& callback) {
   DCHECK_GE(current_range_start_, 0);
 
   // Scan the disk cache for the first cached portion within this range.
@@ -169,7 +172,7 @@ int PartialData::ShouldValidateCache(disk_cache::Entry* entry,
   DVLOG(3) << "ShouldValidateCache len: " << len;
 
   if (sparse_entry_) {
-    DCHECK(!callback_);
+    DCHECK(callback_.is_null());
     Core* core = Core::CreateCore(this);
     cached_min_len_ = core->GetAvailableRange(entry, current_range_start_, len,
                                               &cached_start_);
@@ -178,13 +181,7 @@ int PartialData::ShouldValidateCache(disk_cache::Entry* entry,
       callback_ = callback;
       return ERR_IO_PENDING;
     }
-  } else if (truncated_) {
-    if (!current_range_start_) {
-      // Update the cached range only the first time.
-      cached_min_len_ = static_cast<int32>(byte_range_.first_byte_position());
-      cached_start_ = 0;
-    }
-  } else {
+  } else if (!truncated_) {
     if (byte_range_.HasFirstBytePosition() &&
         byte_range_.first_byte_position() >= resource_size_) {
       // The caller should take care of this condition because we should have
@@ -245,6 +242,9 @@ bool PartialData::UpdateFromStoredHeaders(const HttpResponseHeaders* headers,
                                           disk_cache::Entry* entry,
                                           bool truncated) {
   resource_size_ = 0;
+  if (!headers->HasStrongValidators())
+    return false;
+
   if (truncated) {
     DCHECK_EQ(headers->response_code(), 200);
     // We don't have the real length and the user may be trying to create a
@@ -255,14 +255,18 @@ bool PartialData::UpdateFromStoredHeaders(const HttpResponseHeaders* headers,
     // Now we avoid resume if there is no content length, but that was not
     // always the case so double check here.
     int64 total_length = headers->GetContentLength();
-    if (total_length <= 0 || !headers->HasStrongValidators())
+    if (total_length <= 0)
       return false;
 
     truncated_ = true;
+    initial_validation_ = true;
     sparse_entry_ = false;
-    byte_range_.set_first_byte_position(entry->GetDataSize(kDataStream));
+    int current_len = entry->GetDataSize(kDataStream);
+    byte_range_.set_first_byte_position(current_len);
     resource_size_ = total_length;
-    current_range_start_ = 0;
+    current_range_start_ = current_len;
+    cached_min_len_ = current_len;
+    cached_start_ = current_len + 1;
     return true;
   }
 
@@ -274,6 +278,9 @@ bool PartialData::UpdateFromStoredHeaders(const HttpResponseHeaders* headers,
     return true;
   }
 
+  if (!headers->HasStrongValidators())
+    return false;
+
   int64 length_value = headers->GetContentLength();
   if (length_value <= 0)
     return false;  // We must have stored the resource length.
@@ -284,12 +291,20 @@ bool PartialData::UpdateFromStoredHeaders(const HttpResponseHeaders* headers,
   return entry->CouldBeSparse();
 }
 
+void PartialData::SetRangeToStartDownload() {
+  DCHECK(truncated_);
+  DCHECK(!sparse_entry_);
+  current_range_start_ = 0;
+  cached_start_ = 0;
+  initial_validation_ = false;
+}
+
 bool PartialData::IsRequestedRangeOK() {
   if (byte_range_.IsValid()) {
-    if (truncated_)
-      return true;
     if (!byte_range_.ComputeBounds(resource_size_))
       return false;
+    if (truncated_)
+      return true;
 
     if (current_range_start_ < 0)
       current_range_start_ = byte_range_.first_byte_position();
@@ -403,8 +418,9 @@ void PartialData::FixContentLength(HttpResponseHeaders* headers) {
                                         resource_size_));
 }
 
-int PartialData::CacheRead(disk_cache::Entry* entry, IOBuffer* data,
-                           int data_len, CompletionCallback* callback) {
+int PartialData::CacheRead(
+    disk_cache::Entry* entry, IOBuffer* data, int data_len,
+    const net::CompletionCallback& callback) {
   int read_len = std::min(data_len, cached_min_len_);
   if (!read_len)
     return 0;
@@ -423,12 +439,13 @@ int PartialData::CacheRead(disk_cache::Entry* entry, IOBuffer* data,
   return rv;
 }
 
-int PartialData::CacheWrite(disk_cache::Entry* entry, IOBuffer* data,
-                            int data_len, CompletionCallback* callback) {
+int PartialData::CacheWrite(
+    disk_cache::Entry* entry, IOBuffer* data, int data_len,
+    const net::CompletionCallback& callback) {
   DVLOG(3) << "To write: " << data_len;
   if (sparse_entry_) {
-    return entry->WriteSparseData(current_range_start_, data, data_len,
-                                  callback);
+    return entry->WriteSparseData(
+        current_range_start_, data, data_len, callback);
   } else  {
     if (current_range_start_ > kint32max)
       return ERR_INVALID_ARGUMENT;
@@ -463,7 +480,7 @@ int PartialData::GetNextRangeLen() {
 }
 
 void PartialData::GetAvailableRangeCompleted(int result, int64 start) {
-  DCHECK(callback_);
+  DCHECK(!callback_.is_null());
   DCHECK_NE(ERR_IO_PENDING, result);
 
   cached_start_ = start;
@@ -471,9 +488,9 @@ void PartialData::GetAvailableRangeCompleted(int result, int64 start) {
   if (result >= 0)
     result = 1;  // Return success, go ahead and validate the entry.
 
-  CompletionCallback* cb = callback_;
-  callback_ = NULL;
-  cb->Run(result);
+  CompletionCallback cb = callback_;
+  callback_.Reset();
+  cb.Run(result);
 }
 
 }  // namespace net
