@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -55,7 +55,7 @@ class PartialData {
   // error code. If this method returns ERR_IO_PENDING, the |callback| will be
   // notified when the result is ready.
   int ShouldValidateCache(disk_cache::Entry* entry,
-                          CompletionCallback* callback);
+                          const CompletionCallback& callback);
 
   // Builds the required |headers| to perform the proper cache validation for
   // the next range to be fetched.
@@ -75,6 +75,9 @@ class PartialData {
   bool UpdateFromStoredHeaders(const HttpResponseHeaders* headers,
                                disk_cache::Entry* entry, bool truncated);
 
+  // Sets the byte current range to start again at zero (for a truncated entry).
+  void SetRangeToStartDownload();
+
   // Returns true if the requested range is valid given the stored data.
   bool IsRequestedRangeOK();
 
@@ -95,12 +98,12 @@ class PartialData {
   // operation completes, OnCacheReadCompleted() must be called with the result
   // of the operation.
   int CacheRead(disk_cache::Entry* entry, IOBuffer* data, int data_len,
-                CompletionCallback* callback);
+                const net::CompletionCallback& callback);
 
   // Writes |data_len| bytes to cache. This is basically a wrapper around the
   // API of the cache that provides the right arguments for the current range.
   int CacheWrite(disk_cache::Entry* entry, IOBuffer* data, int data_len,
-                 CompletionCallback* callback);
+                 const net::CompletionCallback& callback);
 
   // This method should be called when CacheRead() finishes the read, to update
   // the internal state about the current range.
@@ -110,6 +113,8 @@ class PartialData {
   // update the internal state about the current range.
   void OnNetworkReadCompleted(int result);
 
+  bool initial_validation() const { return initial_validation_; }
+
  private:
   class Core;
   // Returns the length to use when scanning the cache.
@@ -129,8 +134,9 @@ class PartialData {
   bool final_range_;
   bool sparse_entry_;
   bool truncated_;  // We have an incomplete 200 stored.
+  bool initial_validation_;  // Only used for truncated entries.
   Core* core_;
-  CompletionCallback* callback_;
+  CompletionCallback callback_;
 
   DISALLOW_COPY_AND_ASSIGN(PartialData);
 };
