@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,14 +6,14 @@
 #define NET_HTTP_PROXY_CLIENT_SOCKET_H_
 #pragma once
 
-#include "net/socket/client_socket.h"
+#include "net/socket/stream_socket.h"
 
 namespace net {
 
 class HttpStream;
 class HttpResponseInfo;
 
-class ProxyClientSocket : public ClientSocket {
+class NET_EXPORT_PRIVATE ProxyClientSocket : public StreamSocket {
  public:
   ProxyClientSocket() {}
   virtual ~ProxyClientSocket() {}
