@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,9 @@
 #define NET_HTTP_URL_SECURITY_MANAGER_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
 #include "base/basictypes.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/net_export.h"
 
 class GURL;
 
@@ -17,7 +18,7 @@ class HttpAuthFilter;
 
 // The URL security manager controls the policies (allow, deny, prompt user)
 // regarding URL actions (e.g., sending the default credentials to a server).
-class URLSecurityManager {
+class NET_EXPORT URLSecurityManager {
  public:
   URLSecurityManager() {}
   virtual ~URLSecurityManager() {}
@@ -64,8 +65,8 @@ class URLSecurityManagerWhitelist : public URLSecurityManager {
   virtual ~URLSecurityManagerWhitelist();
 
   // URLSecurityManager methods.
-  virtual bool CanUseDefaultCredentials(const GURL& auth_origin) const;
-  virtual bool CanDelegate(const GURL& auth_origin) const;
+  virtual bool CanUseDefaultCredentials(const GURL& auth_origin) const OVERRIDE;
+  virtual bool CanDelegate(const GURL& auth_origin) const OVERRIDE;
 
  private:
   scoped_ptr<const HttpAuthFilter> whitelist_default_;
@@ -74,25 +75,6 @@ class URLSecurityManagerWhitelist : public URLSecurityManager {
   DISALLOW_COPY_AND_ASSIGN(URLSecurityManagerWhitelist);
 };
 
-#if defined(UNIT_TEST)
-// An URLSecurityManager which is very permissive.
-class URLSecurityManagerAllow : public URLSecurityManager {
- public:
-  URLSecurityManagerAllow() {}
-  virtual ~URLSecurityManagerAllow() {}
-
-  virtual bool CanUseDefaultCredentials(const GURL& auth_origin) const {
-    return true;
-  }
-  virtual bool CanDelegate(const GURL& auth_origin) const {
-    return true;
-  }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(URLSecurityManagerAllow);
-};
-#endif  // defined(UNIT_TEST)
-
 }  // namespace net
 
 #endif  // NET_HTTP_URL_SECURITY_MANAGER_H_
