@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,12 +15,13 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/callback.h"
 #include "base/compiler_specific.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
 #include "base/metrics/histogram.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/synchronization/condition_variable.h"
@@ -59,7 +60,8 @@ class OCSPIOLoop {
   }
 
   // Called from worker thread.
-  void PostTaskToIOLoop(const tracked_objects::Location& from_here, Task* task);
+  void PostTaskToIOLoop(const tracked_objects::Location& from_here,
+                        const base::Closure& task);
 
   void EnsureIOLoop();
 
@@ -85,8 +87,8 @@ class OCSPIOLoop {
   DISALLOW_COPY_AND_ASSIGN(OCSPIOLoop);
 };
 
-base::LazyInstance<OCSPIOLoop, base::LeakyLazyInstanceTraits<OCSPIOLoop> >
-    g_ocsp_io_loop(base::LINKER_INITIALIZED);
+base::LazyInstance<OCSPIOLoop>::Leaky
+    g_ocsp_io_loop = LAZY_INSTANCE_INITIALIZER;
 
 const int kRecvBufferSize = 4096;
 
@@ -136,8 +138,8 @@ class OCSPNSSInitialization {
   DISALLOW_COPY_AND_ASSIGN(OCSPNSSInitialization);
 };
 
-base::LazyInstance<OCSPNSSInitialization> g_ocsp_nss_initialization(
-    base::LINKER_INITIALIZED);
+base::LazyInstance<OCSPNSSInitialization> g_ocsp_nss_initialization =
+    LAZY_INSTANCE_INITIALIZER;
 
 // Concrete class for SEC_HTTP_REQUEST_SESSION.
 // Public methods except virtual methods of net::URLRequest::Delegate
@@ -179,7 +181,7 @@ class OCSPRequestSession
     DCHECK(!io_loop_);
     g_ocsp_io_loop.Get().PostTaskToIOLoop(
         FROM_HERE,
-        NewRunnableMethod(this, &OCSPRequestSession::StartURLRequest));
+        base::Bind(&OCSPRequestSession::StartURLRequest, this));
   }
 
   bool Started() const {
@@ -340,7 +342,7 @@ class OCSPRequestSession
     if (io_loop_) {
       io_loop_->PostTask(
           FROM_HERE,
-          NewRunnableMethod(this, &OCSPRequestSession::CancelURLRequest));
+          base::Bind(&OCSPRequestSession::CancelURLRequest, this));
     }
   }
 
@@ -364,7 +366,7 @@ class OCSPRequestSession
 
     request_ = new net::URLRequest(url_, this);
     request_->set_context(url_request_context);
-    // To meet the privacy requirements of off-the-record mode.
+    // To meet the privacy requirements of incognito mode.
     request_->set_load_flags(
         net::LOAD_DISABLE_CACHE | net::LOAD_DO_NOT_SAVE_COOKIES |
         net::LOAD_DO_NOT_SEND_COOKIES);
@@ -493,7 +495,7 @@ void OCSPIOLoop::Shutdown() {
 }
 
 void OCSPIOLoop::PostTaskToIOLoop(
-    const tracked_objects::Location& from_here, Task* task) {
+    const tracked_objects::Location& from_here, const base::Closure& task) {
   base::AutoLock autolock(lock_);
   if (io_loop_)
     io_loop_->PostTask(from_here, task);
@@ -566,7 +568,12 @@ OCSPNSSInitialization::OCSPNSSInitialization() {
   }
 }
 
-OCSPNSSInitialization::~OCSPNSSInitialization() {}
+OCSPNSSInitialization::~OCSPNSSInitialization() {
+  SECStatus status = CERT_RegisterAlternateOCSPAIAInfoCallBack(NULL, NULL);
+  if (status != SECSuccess) {
+    LOG(ERROR) << "Error unregistering OCSP: " << PR_GetError();
+  }
+}
 
 
 // OCSP Http Client functions.
@@ -718,57 +725,88 @@ SECStatus OCSPTrySendAndReceive(SEC_HTTP_REQUEST_SESSION request,
   }
 
   const base::Time start_time = base::Time::Now();
+  bool request_ok = true;
   req->Start();
   if (!req->Wait() || req->http_response_code() == static_cast<PRUint16>(-1)) {
     // If the response code is -1, the request failed and there is no response.
-    PORT_SetError(SEC_ERROR_BAD_HTTP_RESPONSE);  // Simple approximation.
-    return SECFailure;
+    request_ok = false;
   }
   const base::TimeDelta duration = base::Time::Now() - start_time;
 
+  // For metrics, we want to know if the request was 'successful' or not.
+  // |request_ok| determines if we'll pass the response back to NSS and |ok|
+  // keep track of if we think the response was good.
+  bool ok = true;
+  if (!request_ok ||
+      (req->http_response_code() >= 400 && req->http_response_code() < 600) ||
+      req->http_response_data().size() == 0 ||
+      // 0x30 is the ASN.1 DER encoding of a SEQUENCE. All valid OCSP/CRL/CRT
+      // responses must start with this. If we didn't check for this then a
+      // captive portal could provide an HTML reply that we would count as a
+      // 'success' (although it wouldn't count in NSS, of course).
+      req->http_response_data().data()[0] != 0x30) {
+    ok = false;
+  }
+
   // We want to know if this was:
   //   1) An OCSP request
   //   2) A CRL request
   //   3) A request for a missing intermediate certificate
   // There's no sure way to do this, so we use heuristics like MIME type and
   // URL.
-  const char* mime_type = req->http_response_content_type().c_str();
-  bool is_ocsp_resp =
+  const char* mime_type = "";
+  if (ok)
+    mime_type = req->http_response_content_type().c_str();
+  bool is_ocsp =
       strcasecmp(mime_type, "application/ocsp-response") == 0;
-  bool is_crl_resp = strcasecmp(mime_type, "application/x-pkcs7-crl") == 0 ||
-                     strcasecmp(mime_type, "application/x-x509-crl") == 0 ||
-                     strcasecmp(mime_type, "application/pkix-crl") == 0;
-  bool is_crt_resp =
+  bool is_crl = strcasecmp(mime_type, "application/x-pkcs7-crl") == 0 ||
+                strcasecmp(mime_type, "application/x-x509-crl") == 0 ||
+                strcasecmp(mime_type, "application/pkix-crl") == 0;
+  bool is_crt =
       strcasecmp(mime_type, "application/x-x509-ca-cert") == 0 ||
       strcasecmp(mime_type, "application/x-x509-server-cert") == 0 ||
       strcasecmp(mime_type, "application/pkix-cert") == 0 ||
       strcasecmp(mime_type, "application/pkcs7-mime") == 0;
-  bool known_resp_type = is_crt_resp || is_crt_resp || is_ocsp_resp;
 
-  bool crl_in_url = false, crt_in_url = false, ocsp_in_url = false,
-       have_url_hint = false;
-  if (!known_resp_type) {
+  if (!is_crt && !is_crt && !is_ocsp) {
+    // We didn't get a hint from the MIME type, so do the best that we can.
     const std::string path = req->url().path();
     const std::string host = req->url().host();
-    crl_in_url = strcasestr(path.c_str(), ".crl") != NULL;
-    crt_in_url = strcasestr(path.c_str(), ".crt") != NULL ||
-                 strcasestr(path.c_str(), ".p7c") != NULL ||
-                 strcasestr(path.c_str(), ".cer") != NULL;
-    ocsp_in_url = strcasestr(host.c_str(), "ocsp") != NULL;
-    have_url_hint = crl_in_url || crt_in_url || ocsp_in_url;
+    is_crl = strcasestr(path.c_str(), ".crl") != NULL;
+    is_crt = strcasestr(path.c_str(), ".crt") != NULL ||
+             strcasestr(path.c_str(), ".p7c") != NULL ||
+             strcasestr(path.c_str(), ".cer") != NULL;
+    is_ocsp = strcasestr(host.c_str(), "ocsp") != NULL ||
+              req->http_request_method() == "POST";
   }
 
-  if (is_ocsp_resp ||
-      (!known_resp_type && (ocsp_in_url ||
-                            (!have_url_hint &&
-                             req->http_request_method() == "POST")))) {
-    UMA_HISTOGRAM_TIMES("Net.OCSPRequestTimeMs", duration);
-  } else if (is_crl_resp || (!known_resp_type && crl_in_url)) {
-    UMA_HISTOGRAM_TIMES("Net.CRLRequestTimeMs", duration);
-  } else if (is_crt_resp || (!known_resp_type && crt_in_url)) {
-    UMA_HISTOGRAM_TIMES("Net.CRTRequestTimeMs", duration);
+  if (is_ocsp) {
+    if (ok) {
+      UMA_HISTOGRAM_TIMES("Net.OCSPRequestTimeMs", duration);
+      UMA_HISTOGRAM_BOOLEAN("Net.OCSPRequestSuccess", true);
+    } else {
+      UMA_HISTOGRAM_TIMES("Net.OCSPRequestFailedTimeMs", duration);
+      UMA_HISTOGRAM_BOOLEAN("Net.OCSPRequestSuccess", false);
+    }
+  } else if (is_crl) {
+    if (ok) {
+      UMA_HISTOGRAM_TIMES("Net.CRLRequestTimeMs", duration);
+      UMA_HISTOGRAM_BOOLEAN("Net.CRLRequestSuccess", true);
+    } else {
+      UMA_HISTOGRAM_TIMES("Net.CRLRequestFailedTimeMs", duration);
+      UMA_HISTOGRAM_BOOLEAN("Net.CRLRequestSuccess", false);
+    }
+  } else if (is_crt) {
+    if (ok)
+      UMA_HISTOGRAM_TIMES("Net.CRTRequestTimeMs", duration);
   } else {
-    UMA_HISTOGRAM_TIMES("Net.UnknownTypeRequestTimeMs", duration);
+    if (ok)
+      UMA_HISTOGRAM_TIMES("Net.UnknownTypeRequestTimeMs", duration);
+  }
+
+  if (!request_ok) {
+    PORT_SetError(SEC_ERROR_BAD_HTTP_RESPONSE);  // Simple approximation.
+    return SECFailure;
   }
 
   return OCSPSetResponse(
@@ -900,19 +938,10 @@ void ShutdownOCSP() {
 void SetURLRequestContextForOCSP(URLRequestContext* request_context) {
   pthread_mutex_lock(&g_request_context_lock);
   if (request_context) {
-    DCHECK(request_context->is_main());
     DCHECK(!g_request_context);
   }
   g_request_context = request_context;
   pthread_mutex_unlock(&g_request_context_lock);
 }
 
-URLRequestContext* GetURLRequestContextForOCSP() {
-  pthread_mutex_lock(&g_request_context_lock);
-  URLRequestContext* request_context = g_request_context;
-  pthread_mutex_unlock(&g_request_context_lock);
-  DCHECK(!request_context || request_context->is_main());
-  return request_context;
-}
-
 }  // namespace net
