@@ -1,15 +1,17 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/proxy/multi_threaded_proxy_resolver.h"
 
-#include "base/message_loop.h"
+#include "base/bind.h"
+#include "base/message_loop_proxy.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
-#include "base/thread.h"
-#include "net/base/capturing_net_log.h"
+#include "base/threading/thread.h"
+#include "base/threading/thread_restrictions.h"
 #include "net/base/net_errors.h"
+#include "net/base/net_log.h"
 #include "net/proxy/proxy_info.h"
 
 // TODO(eroman): Have the MultiThreadedProxyResolver clear its PAC script
@@ -99,9 +101,9 @@ class MultiThreadedProxyResolver::Job
     TYPE_SET_PAC_SCRIPT_INTERNAL,
   };
 
-  Job(Type type, CompletionCallback* user_callback)
+  Job(Type type, const CompletionCallback& callback)
       : type_(type),
-        user_callback_(user_callback),
+        callback_(callback),
         executor_(NULL),
         was_cancelled_(false) {
   }
@@ -132,8 +134,8 @@ class MultiThreadedProxyResolver::Job
   // scheduled internally (for example TYPE_SET_PAC_SCRIPT_INTERNAL).
   //
   // Otherwise jobs that correspond with user-initiated work will
-  // have a non-NULL callback up until the callback is run.
-  bool has_user_callback() const { return user_callback_ != NULL; }
+  // have a non-null callback up until the callback is run.
+  bool has_user_callback() const { return !callback_.is_null(); }
 
   // This method is called when the job is inserted into a wait queue
   // because no executors were ready to accept it.
@@ -145,7 +147,7 @@ class MultiThreadedProxyResolver::Job
   // This method is called on the worker thread to do the job's work. On
   // completion, implementors are expected to call OnJobCompleted() on
   // |origin_loop|.
-  virtual void Run(MessageLoop* origin_loop) = 0;
+  virtual void Run(scoped_refptr<base::MessageLoopProxy> origin_loop) = 0;
 
  protected:
   void OnJobCompleted() {
@@ -156,10 +158,10 @@ class MultiThreadedProxyResolver::Job
 
   void RunUserCallback(int result) {
     DCHECK(has_user_callback());
-    CompletionCallback* callback = user_callback_;
-    // Null the callback so has_user_callback() will now return false.
-    user_callback_ = NULL;
-    callback->Run(result);
+    CompletionCallback callback = callback_;
+    // Reset the callback so has_user_callback() will now return false.
+    callback_.Reset();
+    callback.Run(result);
   }
 
   friend class base::RefCountedThreadSafe<MultiThreadedProxyResolver::Job>;
@@ -168,7 +170,7 @@ class MultiThreadedProxyResolver::Job
 
  private:
   const Type type_;
-  CompletionCallback* user_callback_;
+  CompletionCallback callback_;
   Executor* executor_;
   bool was_cancelled_;
 };
@@ -180,21 +182,22 @@ class MultiThreadedProxyResolver::SetPacScriptJob
     : public MultiThreadedProxyResolver::Job {
  public:
   SetPacScriptJob(const scoped_refptr<ProxyResolverScriptData>& script_data,
-                  CompletionCallback* callback)
-    : Job(callback ? TYPE_SET_PAC_SCRIPT : TYPE_SET_PAC_SCRIPT_INTERNAL,
+                  const CompletionCallback& callback)
+    : Job(!callback.is_null() ? TYPE_SET_PAC_SCRIPT :
+                                TYPE_SET_PAC_SCRIPT_INTERNAL,
           callback),
       script_data_(script_data) {
   }
 
   // Runs on the worker thread.
-  virtual void Run(MessageLoop* origin_loop) {
+  virtual void Run(scoped_refptr<base::MessageLoopProxy> origin_loop) OVERRIDE {
     ProxyResolver* resolver = executor()->resolver();
-    int rv = resolver->SetPacScript(script_data_, NULL);
+    int rv = resolver->SetPacScript(script_data_, CompletionCallback());
 
     DCHECK_NE(rv, ERR_IO_PENDING);
     origin_loop->PostTask(
         FROM_HERE,
-        NewRunnableMethod(this, &SetPacScriptJob::RequestComplete, rv));
+        base::Bind(&SetPacScriptJob::RequestComplete, this, rv));
   }
 
  private:
@@ -219,25 +222,25 @@ class MultiThreadedProxyResolver::GetProxyForURLJob
   // |results|     -- the structure to fill with proxy resolve results.
   GetProxyForURLJob(const GURL& url,
                     ProxyInfo* results,
-                    CompletionCallback* callback,
+                    const CompletionCallback& callback,
                     const BoundNetLog& net_log)
       : Job(TYPE_GET_PROXY_FOR_URL, callback),
         results_(results),
         net_log_(net_log),
         url_(url),
         was_waiting_for_thread_(false) {
-    DCHECK(callback);
+    DCHECK(!callback.is_null());
   }
 
   BoundNetLog* net_log() { return &net_log_; }
 
-  virtual void WaitingForThread() {
+  virtual void WaitingForThread() OVERRIDE {
     was_waiting_for_thread_ = true;
     net_log_.BeginEvent(
         NetLog::TYPE_WAITING_FOR_PROXY_RESOLVER_THREAD, NULL);
   }
 
-  virtual void FinishedWaitingForThread() {
+  virtual void FinishedWaitingForThread() OVERRIDE {
     DCHECK(executor());
 
     if (was_waiting_for_thread_) {
@@ -247,24 +250,20 @@ class MultiThreadedProxyResolver::GetProxyForURLJob
 
     net_log_.AddEvent(
         NetLog::TYPE_SUBMITTED_TO_RESOLVER_THREAD,
-        new NetLogIntegerParameter(
-            "thread_number", executor()->thread_number()));
+        make_scoped_refptr(new NetLogIntegerParameter(
+            "thread_number", executor()->thread_number())));
   }
 
   // Runs on the worker thread.
-  virtual void Run(MessageLoop* origin_loop) {
-    const size_t kNetLogBound = 50u;
-    worker_log_.reset(new CapturingNetLog(kNetLogBound));
-    BoundNetLog bound_worker_log(NetLog::Source(), worker_log_.get());
-
+  virtual void Run(scoped_refptr<base::MessageLoopProxy> origin_loop) OVERRIDE {
     ProxyResolver* resolver = executor()->resolver();
     int rv = resolver->GetProxyForURL(
-        url_, &results_buf_, NULL, NULL, bound_worker_log);
+        url_, &results_buf_, CompletionCallback(), NULL, net_log_);
     DCHECK_NE(rv, ERR_IO_PENDING);
 
     origin_loop->PostTask(
         FROM_HERE,
-        NewRunnableMethod(this, &GetProxyForURLJob::QueryComplete, rv));
+        base::Bind(&GetProxyForURLJob::QueryComplete, this, rv));
   }
 
  private:
@@ -272,12 +271,6 @@ class MultiThreadedProxyResolver::GetProxyForURLJob
   void QueryComplete(int result_code) {
     // The Job may have been cancelled after it was started.
     if (!was_cancelled()) {
-      // Merge the load log that was generated on the worker thread, into the
-      // main log.
-      CapturingBoundNetLog bound_worker_log(NetLog::Source(),
-                                            worker_log_.release());
-      bound_worker_log.AppendTo(net_log_);
-
       if (result_code >= OK) {  // Note: unit-tests use values > 0.
         results_->Use(results_buf_);
       }
@@ -288,16 +281,14 @@ class MultiThreadedProxyResolver::GetProxyForURLJob
 
   // Must only be used on the "origin" thread.
   ProxyInfo* results_;
+
+  // Can be used on either "origin" or worker thread.
   BoundNetLog net_log_;
   const GURL url_;
 
   // Usable from within DoQuery on the worker thread.
   ProxyInfo results_buf_;
 
-  // Used to pass the captured events between DoQuery [worker thread] and
-  // QueryComplete [origin thread].
-  scoped_ptr<CapturingNetLog> worker_log_;
-
   bool was_waiting_for_thread_;
 };
 
@@ -318,7 +309,7 @@ MultiThreadedProxyResolver::Executor::Executor(
   std::string thread_name =
       base::StringPrintf("PAC thread #%d", thread_number);
   thread_.reset(new base::Thread(thread_name.c_str()));
-  thread_->Start();
+  CHECK(thread_->Start());
 }
 
 void MultiThreadedProxyResolver::Executor::StartJob(Job* job) {
@@ -331,7 +322,7 @@ void MultiThreadedProxyResolver::Executor::StartJob(Job* job) {
   job->FinishedWaitingForThread();
   thread_->message_loop()->PostTask(
       FROM_HERE,
-      NewRunnableMethod(job, &Job::Run, MessageLoop::current()));
+      base::Bind(&Job::Run, job, base::MessageLoopProxy::current()));
 }
 
 void MultiThreadedProxyResolver::Executor::OnJobCompleted(Job* job) {
@@ -348,8 +339,13 @@ void MultiThreadedProxyResolver::Executor::Destroy() {
   // to avoid deadlocks.
   resolver_->Shutdown();
 
-  // Join the worker thread.
-  thread_.reset();
+  {
+    // See http://crbug.com/69710.
+    base::ThreadRestrictions::ScopedAllowIO allow_io;
+
+    // Join the worker thread.
+    thread_.reset();
+  }
 
   // Cancel any outstanding job.
   if (outstanding_job_) {
@@ -371,7 +367,7 @@ void MultiThreadedProxyResolver::Executor::PurgeMemory() {
   scoped_refptr<PurgeMemoryTask> helper(new PurgeMemoryTask(resolver_.get()));
   thread_->message_loop()->PostTask(
       FROM_HERE,
-      NewRunnableMethod(helper.get(), &PurgeMemoryTask::PurgeMemory));
+      base::Bind(&PurgeMemoryTask::PurgeMemory, helper.get()));
 }
 
 MultiThreadedProxyResolver::Executor::~Executor() {
@@ -400,18 +396,16 @@ MultiThreadedProxyResolver::~MultiThreadedProxyResolver() {
   ReleaseAllExecutors();
 }
 
-int MultiThreadedProxyResolver::GetProxyForURL(const GURL& url,
-                                               ProxyInfo* results,
-                                               CompletionCallback* callback,
-                                               RequestHandle* request,
-                                               const BoundNetLog& net_log) {
+int MultiThreadedProxyResolver::GetProxyForURL(
+    const GURL& url, ProxyInfo* results, const CompletionCallback& callback,
+    RequestHandle* request, const BoundNetLog& net_log) {
   DCHECK(CalledOnValidThread());
-  DCHECK(callback);
+  DCHECK(!callback.is_null());
   DCHECK(current_script_data_.get())
       << "Resolver is un-initialized. Must call SetPacScript() first!";
 
-  scoped_refptr<GetProxyForURLJob> job =
-      new GetProxyForURLJob(url, results, callback, net_log);
+  scoped_refptr<GetProxyForURLJob> job(
+      new GetProxyForURLJob(url, results, callback, net_log));
 
   // Completion will be notified through |callback|, unless the caller cancels
   // the request using |request|.
@@ -436,7 +430,7 @@ int MultiThreadedProxyResolver::GetProxyForURL(const GURL& url,
   if (executors_.size() < max_num_threads_) {
     executor = AddNewExecutor();
     executor->StartJob(
-        new SetPacScriptJob(current_script_data_, NULL));
+        new SetPacScriptJob(current_script_data_, CompletionCallback()));
   }
 
   return ERR_IO_PENDING;
@@ -462,6 +456,22 @@ void MultiThreadedProxyResolver::CancelRequest(RequestHandle req) {
   }
 }
 
+LoadState MultiThreadedProxyResolver::GetLoadState(RequestHandle req) const {
+  DCHECK(CalledOnValidThread());
+  DCHECK(req);
+
+  Job* job = reinterpret_cast<Job*>(req);
+  if (job->executor())
+    return job->executor()->resolver()->GetLoadStateThreadSafe(NULL);
+  return LOAD_STATE_RESOLVING_PROXY_FOR_URL;
+}
+
+LoadState MultiThreadedProxyResolver::GetLoadStateThreadSafe(
+    RequestHandle req) const {
+  NOTIMPLEMENTED();
+  return LOAD_STATE_IDLE;
+}
+
 void MultiThreadedProxyResolver::CancelSetPacScript() {
   DCHECK(CalledOnValidThread());
   DCHECK_EQ(0u, pending_jobs_.size());
@@ -487,9 +497,9 @@ void MultiThreadedProxyResolver::PurgeMemory() {
 
 int MultiThreadedProxyResolver::SetPacScript(
     const scoped_refptr<ProxyResolverScriptData>& script_data,
-    CompletionCallback* callback) {
+    const CompletionCallback&callback) {
   DCHECK(CalledOnValidThread());
-  DCHECK(callback);
+  DCHECK(!callback.is_null());
 
   // Save the script details, so we can provision new executors later.
   current_script_data_ = script_data;
@@ -557,7 +567,7 @@ MultiThreadedProxyResolver::AddNewExecutor() {
   ProxyResolver* resolver = resolver_factory_->CreateProxyResolver();
   Executor* executor = new Executor(
       this, resolver, thread_number);
-  executors_.push_back(executor);
+  executors_.push_back(make_scoped_refptr(executor));
   return executor;
 }
 
