@@ -1,12 +1,14 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/proxy/polling_proxy_config_service.h"
 
+#include "base/bind.h"
+#include "base/location.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop_proxy.h"
 #include "base/observer_list.h"
-#include "base/scoped_ptr.h"
 #include "base/synchronization/lock.h"
 #include "base/threading/worker_pool.h"
 #include "net/proxy/proxy_config.h"
@@ -90,7 +92,7 @@ class PollingProxyConfigService::Core
     poll_task_queued_ = false;
     base::WorkerPool::PostTask(
         FROM_HERE,
-        NewRunnableMethod(this, &Core::PollOnWorkerThread, get_config_func_),
+        base::Bind(&Core::PollOnWorkerThread, this, get_config_func_),
         true);
   }
 
@@ -103,7 +105,7 @@ class PollingProxyConfigService::Core
     if (origin_loop_proxy_) {
       origin_loop_proxy_->PostTask(
           FROM_HERE,
-          NewRunnableMethod(this, &Core::GetConfigCompleted, config));
+          base::Bind(&Core::GetConfigCompleted, this, config));
     }
   }
 
@@ -121,7 +123,9 @@ class PollingProxyConfigService::Core
       // If the configuration has changed, notify the observers.
       has_config_ = true;
       last_config_ = config;
-      FOR_EACH_OBSERVER(Observer, observers_, OnProxyConfigChanged(config));
+      FOR_EACH_OBSERVER(Observer, observers_,
+                        OnProxyConfigChanged(config,
+                                             ProxyConfigService::CONFIG_VALID));
     }
 
     if (poll_task_queued_)
@@ -134,7 +138,7 @@ class PollingProxyConfigService::Core
     //               UI thread so we can't cache the IO thread for the purpose
     //               of DCHECKs until the first call is made.
     if (!have_initialized_origin_loop_) {
-      origin_loop_proxy_ = base::MessageLoopProxy::CreateForCurrentThread();
+      origin_loop_proxy_ = base::MessageLoopProxy::current();
       have_initialized_origin_loop_ = true;
     }
   }
@@ -162,8 +166,9 @@ void PollingProxyConfigService::RemoveObserver(Observer* observer) {
   core_->RemoveObserver(observer);
 }
 
-bool PollingProxyConfigService::GetLatestProxyConfig(ProxyConfig* config) {
-  return core_->GetLatestProxyConfig(config);
+ProxyConfigService::ConfigAvailability
+    PollingProxyConfigService::GetLatestProxyConfig(ProxyConfig* config) {
+  return core_->GetLatestProxyConfig(config) ? CONFIG_VALID : CONFIG_PENDING;
 }
 
 void PollingProxyConfigService::OnLazyPoll() {
