@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,16 +10,17 @@
 #include <vector>
 
 #include "googleurl/src/gurl.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
 // ProxyBypassRules describes the set of URLs that should bypass the proxy
 // settings, as a list of rules. A URL is said to match the bypass rules
 // if it matches any one of these rules.
-class ProxyBypassRules {
+class NET_EXPORT ProxyBypassRules {
  public:
   // Interface for an individual proxy bypass rule.
-  class Rule {
+  class NET_EXPORT Rule {
    public:
     Rule();
     virtual ~Rule();
@@ -153,6 +154,10 @@ class ProxyBypassRules {
   // NOTE: Use AddRuleFromString() unless you truly need this behavior.
   bool AddRuleFromStringUsingSuffixMatching(const std::string& raw);
 
+  // Converts the rules to string representation. Inverse operation to
+  // ParseFromString().
+  std::string ToString() const;
+
   // Removes all the rules.
   void Clear();
 
