@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,15 +8,16 @@
 #include <string>
 #include <vector>
 
+#include "base/bind.h"
+#include "base/compiler_specific.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
 #include "base/format_macros.h"
 #include "base/logging.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
-#include "base/task.h"
-#include "base/threading/thread.h"
 #include "base/synchronization/waitable_event.h"
+#include "base/threading/thread.h"
 #include "net/proxy/proxy_config.h"
 #include "net/proxy/proxy_config_service_common_unittest.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -63,12 +64,12 @@ struct GConfValues {
 
 // Mapping from a setting name to the location of the corresponding
 // value (inside a EnvVarValues or GConfValues struct).
-template<typename value_type>
+template<typename key_type, typename value_type>
 struct SettingsTable {
-  typedef std::map<std::string, value_type*> map_type;
+  typedef std::map<key_type, value_type*> map_type;
 
   // Gets the value from its location
-  value_type Get(const std::string& key) {
+  value_type Get(key_type key) {
     typename map_type::const_iterator it = settings.find(key);
     // In case there's a typo or the unittest becomes out of sync.
     CHECK(it != settings.end()) << "key " << key << " not found";
@@ -82,7 +83,7 @@ struct SettingsTable {
 class MockEnvironment : public base::Environment {
  public:
   MockEnvironment() {
-#define ENTRY(x) table.settings[#x] = &values.x
+#define ENTRY(x) table[#x] = &values.x
     ENTRY(DESKTOP_SESSION);
     ENTRY(HOME);
     ENTRY(KDEHOME);
@@ -99,29 +100,31 @@ class MockEnvironment : public base::Environment {
     Reset();
   }
 
-  // Zeros all environment values.
+  // Zeroes all environment values.
   void Reset() {
     EnvVarValues zero_values = { 0 };
     values = zero_values;
   }
 
   // Begin base::Environment implementation.
-  virtual bool GetVar(const char* variable_name, std::string* result) {
-    const char* env_value = table.Get(variable_name);
-    if (env_value) {
+  virtual bool GetVar(const char* variable_name, std::string* result) OVERRIDE {
+    std::map<std::string, const char**>::iterator it =
+        table.find(variable_name);
+    if (it != table.end() && *(it->second) != NULL) {
       // Note that the variable may be defined but empty.
-      *result = env_value;
+      *result = *(it->second);
       return true;
     }
     return false;
   }
 
-  virtual bool SetVar(const char* variable_name, const std::string& new_value) {
+  virtual bool SetVar(const char* variable_name, const std::string& new_value)
+      OVERRIDE {
     ADD_FAILURE();
     return false;
   }
 
-  virtual bool UnSetVar(const char* variable_name) {
+  virtual bool UnSetVar(const char* variable_name) OVERRIDE {
     ADD_FAILURE();
     return false;
   }
@@ -131,37 +134,38 @@ class MockEnvironment : public base::Environment {
   EnvVarValues values;
 
  private:
-  SettingsTable<const char*> table;
+  std::map<std::string, const char**> table;
 };
 
-class MockGConfSettingGetter
-    : public ProxyConfigServiceLinux::GConfSettingGetter {
+class MockSettingGetter
+    : public ProxyConfigServiceLinux::SettingGetter {
  public:
-  MockGConfSettingGetter() {
+  typedef ProxyConfigServiceLinux::SettingGetter SettingGetter;
+  MockSettingGetter() {
 #define ENTRY(key, field) \
-      strings_table.settings["/system/" key] = &values.field
-    ENTRY("proxy/mode", mode);
-    ENTRY("proxy/autoconfig_url", autoconfig_url);
-    ENTRY("http_proxy/host", http_host);
-    ENTRY("proxy/secure_host", secure_host);
-    ENTRY("proxy/ftp_host", ftp_host);
-    ENTRY("proxy/socks_host", socks_host);
+      strings_table.settings[SettingGetter::key] = &values.field
+    ENTRY(PROXY_MODE, mode);
+    ENTRY(PROXY_AUTOCONF_URL, autoconfig_url);
+    ENTRY(PROXY_HTTP_HOST, http_host);
+    ENTRY(PROXY_HTTPS_HOST, secure_host);
+    ENTRY(PROXY_FTP_HOST, ftp_host);
+    ENTRY(PROXY_SOCKS_HOST, socks_host);
 #undef ENTRY
 #define ENTRY(key, field) \
-      ints_table.settings["/system/" key] = &values.field
-    ENTRY("http_proxy/port", http_port);
-    ENTRY("proxy/secure_port", secure_port);
-    ENTRY("proxy/ftp_port", ftp_port);
-    ENTRY("proxy/socks_port", socks_port);
+      ints_table.settings[SettingGetter::key] = &values.field
+    ENTRY(PROXY_HTTP_PORT, http_port);
+    ENTRY(PROXY_HTTPS_PORT, secure_port);
+    ENTRY(PROXY_FTP_PORT, ftp_port);
+    ENTRY(PROXY_SOCKS_PORT, socks_port);
 #undef ENTRY
 #define ENTRY(key, field) \
-      bools_table.settings["/system/" key] = &values.field
-    ENTRY("http_proxy/use_http_proxy", use_proxy);
-    ENTRY("http_proxy/use_same_proxy", same_proxy);
-    ENTRY("http_proxy/use_authentication", use_auth);
+      bools_table.settings[SettingGetter::key] = &values.field
+    ENTRY(PROXY_USE_HTTP_PROXY, use_proxy);
+    ENTRY(PROXY_USE_SAME_PROXY, same_proxy);
+    ENTRY(PROXY_USE_AUTHENTICATION, use_auth);
 #undef ENTRY
-    string_lists_table.settings["/system/http_proxy/ignore_hosts"] =
-      &values.ignore_hosts;
+    string_lists_table.settings[SettingGetter::PROXY_IGNORE_HOSTS] =
+        &values.ignore_hosts;
     Reset();
   }
 
@@ -172,25 +176,26 @@ class MockGConfSettingGetter
   }
 
   virtual bool Init(MessageLoop* glib_default_loop,
-                    MessageLoopForIO* file_loop) {
+                    MessageLoopForIO* file_loop) OVERRIDE {
     return true;
   }
 
-  virtual void Shutdown() {}
+  virtual void ShutDown() OVERRIDE {}
 
-  virtual bool SetupNotification(ProxyConfigServiceLinux::Delegate* delegate) {
+  virtual bool SetUpNotifications(ProxyConfigServiceLinux::Delegate* delegate)
+      OVERRIDE {
     return true;
   }
 
-  virtual MessageLoop* GetNotificationLoop() {
+  virtual MessageLoop* GetNotificationLoop() OVERRIDE {
     return NULL;
   }
 
-  virtual const char* GetDataSource() {
+  virtual const char* GetDataSource() OVERRIDE {
     return "test";
   }
 
-  virtual bool GetString(const char* key, std::string* result) {
+  virtual bool GetString(StringSetting key, std::string* result) OVERRIDE {
     const char* value = strings_table.Get(key);
     if (value) {
       *result = value;
@@ -199,13 +204,7 @@ class MockGConfSettingGetter
     return false;
   }
 
-  virtual bool GetInt(const char* key, int* result) {
-    // We don't bother to distinguish unset keys from 0 values.
-    *result = ints_table.Get(key);
-    return true;
-  }
-
-  virtual bool GetBoolean(const char* key, bool* result) {
+  virtual bool GetBool(BoolSetting key, bool* result) OVERRIDE {
     BoolSettingValue value = bools_table.Get(key);
     switch (value) {
     case UNSET:
@@ -219,18 +218,24 @@ class MockGConfSettingGetter
     return true;
   }
 
-  virtual bool GetStringList(const char* key,
-                             std::vector<std::string>* result) {
+  virtual bool GetInt(IntSetting key, int* result) OVERRIDE {
+    // We don't bother to distinguish unset keys from 0 values.
+    *result = ints_table.Get(key);
+    return true;
+  }
+
+  virtual bool GetStringList(StringListSetting key,
+                             std::vector<std::string>* result) OVERRIDE {
     *result = string_lists_table.Get(key);
     // We don't bother to distinguish unset keys from empty lists.
     return !result->empty();
   }
 
-  virtual bool BypassListIsReversed() {
+  virtual bool BypassListIsReversed() OVERRIDE {
     return false;
   }
 
-  virtual bool MatchHostsUsingSuffixMatching() {
+  virtual bool MatchHostsUsingSuffixMatching() OVERRIDE {
     return false;
   }
 
@@ -238,10 +243,11 @@ class MockGConfSettingGetter
   GConfValues values;
 
  private:
-  SettingsTable<const char*> strings_table;
-  SettingsTable<int> ints_table;
-  SettingsTable<BoolSettingValue> bools_table;
-  SettingsTable<std::vector<std::string> > string_lists_table;
+  SettingsTable<StringSetting, const char*> strings_table;
+  SettingsTable<BoolSetting, BoolSettingValue> bools_table;
+  SettingsTable<IntSetting, int> ints_table;
+  SettingsTable<StringListSetting,
+                std::vector<std::string> > string_lists_table;
 };
 
 }  // namespace
@@ -263,8 +269,8 @@ class SynchConfigGetter {
     io_thread_.StartWithOptions(options);
 
     // Make sure the thread started.
-    io_thread_.message_loop()->PostTask(FROM_HERE, NewRunnableMethod(
-        this, &SynchConfigGetter::Init));
+    io_thread_.message_loop()->PostTask(FROM_HERE,
+        base::Bind(&SynchConfigGetter::Init, base::Unretained(this)));
     Wait();
   }
 
@@ -273,8 +279,8 @@ class SynchConfigGetter {
     // before cleaning up that thread.
     delete config_service_;
     // Clean up the IO thread.
-    io_thread_.message_loop()->PostTask(FROM_HERE, NewRunnableMethod(
-        this, &SynchConfigGetter::Cleanup));
+    io_thread_.message_loop()->PostTask(FROM_HERE,
+        base::Bind(&SynchConfigGetter::CleanUp, base::Unretained(this)));
     Wait();
   }
 
@@ -290,9 +296,11 @@ class SynchConfigGetter {
         static_cast<MessageLoopForIO*>(file_loop));
   }
   // Synchronously gets the proxy config.
-  bool SyncGetLatestProxyConfig(net::ProxyConfig* config) {
-    io_thread_.message_loop()->PostTask(FROM_HERE, NewRunnableMethod(
-        this, &SynchConfigGetter::GetLatestConfigOnIOThread));
+  net::ProxyConfigService::ConfigAvailability SyncGetLatestProxyConfig(
+      net::ProxyConfig* config) {
+    io_thread_.message_loop()->PostTask(FROM_HERE,
+        base::Bind(&SynchConfigGetter::GetLatestConfigOnIOThread,
+                   base::Unretained(this)));
     Wait();
     *config = proxy_config_;
     return get_latest_config_result_;
@@ -313,7 +321,7 @@ class SynchConfigGetter {
   }
 
   // [Runs on |io_thread_|] Signals |event_| on cleanup completion.
-  void Cleanup() {
+  void CleanUp() {
     MessageLoop::current()->RunAllPending();
     event_.Signal();
   }
@@ -331,10 +339,10 @@ class SynchConfigGetter {
   // The config obtained by |io_thread_| and read back by the main
   // thread.
   net::ProxyConfig proxy_config_;
-  bool get_latest_config_result_;  // Return value from GetLatestProxyConfig().
-};
 
-DISABLE_RUNNABLE_METHOD_REFCOUNT(SynchConfigGetter);
+  // Return value from GetLatestProxyConfig().
+  net::ProxyConfigService::ConfigAvailability get_latest_config_result_;
+};
 
 namespace net {
 
@@ -343,7 +351,7 @@ namespace net {
 // must use the same test fixture class (also "ProxyConfigServiceLinuxTest").
 class ProxyConfigServiceLinuxTest : public PlatformTest {
  protected:
-  virtual void SetUp() {
+  virtual void SetUp() OVERRIDE {
     PlatformTest::SetUp();
     // Set up a temporary KDE home directory.
     std::string prefix("ProxyConfigServiceLinuxTest_user_home");
@@ -360,7 +368,7 @@ class ProxyConfigServiceLinuxTest : public PlatformTest {
     kioslaverc4_ = kde4_config_.Append(FILE_PATH_LITERAL("kioslaverc"));
   }
 
-  virtual void TearDown() {
+  virtual void TearDown() OVERRIDE {
     // Delete the temporary KDE home directory.
     file_util::Delete(user_home_, true);
     PlatformTest::TearDown();
@@ -394,7 +402,8 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
     // Input.
     GConfValues values;
 
-    // Expected outputs (fields of the ProxyConfig).
+    // Expected outputs (availability and fields of ProxyConfig).
+    ProxyConfigService::ConfigAvailability availability;
     bool auto_detect;
     GURL pac_url;
     ProxyRulesExpectation proxy_rules;
@@ -411,6 +420,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                      // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -428,6 +438,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       true,                       // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -445,6 +456,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                         // auto_detect
       GURL("http://wpad/wpad.dat"),  // pac_url
       ProxyRulesExpectation::Empty(),
@@ -462,7 +474,8 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
-      false,                         // auto_detect
+      ProxyConfigService::CONFIG_VALID,
+      false,                          // auto_detect
       GURL(),                        // pac_url
       ProxyRulesExpectation::Empty(),
     },
@@ -479,6 +492,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Single(
@@ -498,6 +512,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Empty(),
@@ -515,6 +530,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                          // auto_detect
       GURL(),                                         // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -536,6 +552,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                          // auto_detect
       GURL(),                                         // pac_url
       ProxyRulesExpectation::Single(
@@ -558,6 +575,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                          // auto_detect
       GURL(),                                         // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -579,6 +597,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                          // auto_detect
       GURL(),                                         // pac_url
       ProxyRulesExpectation::Single(
@@ -586,6 +605,58 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
           "")                       // bypass rules
     },
 
+    {
+      TEST_DESC("Per-scheme proxy rules with fallback to SOCKS"),
+      { // Input.
+        "manual",                                     // mode
+        "",                                           // autoconfig_url
+        "www.google.com",                             // http_host
+        "www.foo.com",                                // secure_host
+        "ftp.foo.com",                                // ftp
+        "foobar.net",                                 // socks
+        88, 110, 121, 99,                             // ports
+        TRUE, FALSE, FALSE,                           // use, same, auth
+        empty_ignores,                                // ignore_hosts
+      },
+
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
+      false,                                          // auto_detect
+      GURL(),                                         // pac_url
+      ProxyRulesExpectation::PerSchemeWithSocks(
+          "www.google.com:88",      // http
+          "www.foo.com:110",        // https
+          "ftp.foo.com:121",        // ftp
+          "socks5://foobar.net:99", // socks
+          ""),                      // bypass rules
+    },
+
+    {
+      TEST_DESC("Per-scheme proxy rules (just HTTP) with fallback to SOCKS"),
+      { // Input.
+        "manual",                                     // mode
+        "",                                           // autoconfig_url
+        "www.google.com",                             // http_host
+        "",                                           // secure_host
+        "",                                           // ftp
+        "foobar.net",                                 // socks
+        88, 0, 0, 99,                                 // ports
+        TRUE, FALSE, FALSE,                           // use, same, auth
+        empty_ignores,                                // ignore_hosts
+      },
+
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
+      false,                                          // auto_detect
+      GURL(),                                         // pac_url
+      ProxyRulesExpectation::PerSchemeWithSocks(
+          "www.google.com:88",      // http
+          "",                       // https
+          "",                       // ftp
+          "socks5://foobar.net:99", // socks
+          ""),                      // bypass rules
+    },
+
     {
       TEST_DESC("Bypass *.google.com"),
       { // Input.
@@ -597,6 +668,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
         google_ignores,                               // ignore_hosts
       },
 
+      ProxyConfigService::CONFIG_VALID,
       false,                                          // auto_detect
       GURL(),                                         // pac_url
       ProxyRulesExpectation::Single(
@@ -609,17 +681,21 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicGConfTest) {
     SCOPED_TRACE(base::StringPrintf("Test[%" PRIuS "] %s", i,
                                     tests[i].description.c_str()));
     MockEnvironment* env = new MockEnvironment;
-    MockGConfSettingGetter* gconf_getter = new MockGConfSettingGetter;
+    MockSettingGetter* setting_getter = new MockSettingGetter;
     SynchConfigGetter sync_config_getter(
-        new ProxyConfigServiceLinux(env, gconf_getter));
+        new ProxyConfigServiceLinux(env, setting_getter));
     ProxyConfig config;
-    gconf_getter->values = tests[i].values;
+    setting_getter->values = tests[i].values;
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    ProxyConfigService::ConfigAvailability availability =
+        sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(tests[i].availability, availability);
 
-    EXPECT_EQ(tests[i].auto_detect, config.auto_detect());
-    EXPECT_EQ(tests[i].pac_url, config.pac_url());
-    EXPECT_TRUE(tests[i].proxy_rules.Matches(config.proxy_rules()));
+    if (availability == ProxyConfigService::CONFIG_VALID) {
+      EXPECT_EQ(tests[i].auto_detect, config.auto_detect());
+      EXPECT_EQ(tests[i].pac_url, config.pac_url());
+      EXPECT_TRUE(tests[i].proxy_rules.Matches(config.proxy_rules()));
+    }
   }
 }
 
@@ -632,7 +708,8 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
     // Input.
     EnvVarValues values;
 
-    // Expected outputs (fields of the ProxyConfig).
+    // Expected outputs (availability and fields of ProxyConfig).
+    ProxyConfigService::ConfigAvailability availability;
     bool auto_detect;
     GURL pac_url;
     ProxyRulesExpectation proxy_rules;
@@ -652,6 +729,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                      // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -672,6 +750,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       true,                       // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -692,6 +771,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                         // auto_detect
       GURL("http://wpad/wpad.dat"),  // pac_url
       ProxyRulesExpectation::Empty(),
@@ -712,6 +792,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                       // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -732,6 +813,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Single(
@@ -754,6 +836,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Single(
@@ -776,6 +859,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Single(
@@ -798,6 +882,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -822,6 +907,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Single(
@@ -844,6 +930,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Single(
@@ -866,6 +953,7 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
       },
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Single(
@@ -887,6 +975,8 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
         ".google.com, foo.com:99, 1.2.3.4:22, 127.0.0.1/8",  // no_proxy
       },
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                      // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Single(
@@ -899,39 +989,45 @@ TEST_F(ProxyConfigServiceLinuxTest, BasicEnvTest) {
     SCOPED_TRACE(base::StringPrintf("Test[%" PRIuS "] %s", i,
                                     tests[i].description.c_str()));
     MockEnvironment* env = new MockEnvironment;
-    MockGConfSettingGetter* gconf_getter = new MockGConfSettingGetter;
+    MockSettingGetter* setting_getter = new MockSettingGetter;
     SynchConfigGetter sync_config_getter(
-        new ProxyConfigServiceLinux(env, gconf_getter));
+        new ProxyConfigServiceLinux(env, setting_getter));
     ProxyConfig config;
     env->values = tests[i].values;
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    ProxyConfigService::ConfigAvailability availability =
+        sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(tests[i].availability, availability);
 
-    EXPECT_EQ(tests[i].auto_detect, config.auto_detect());
-    EXPECT_EQ(tests[i].pac_url, config.pac_url());
-    EXPECT_TRUE(tests[i].proxy_rules.Matches(config.proxy_rules()));
+    if (availability == ProxyConfigService::CONFIG_VALID) {
+      EXPECT_EQ(tests[i].auto_detect, config.auto_detect());
+      EXPECT_EQ(tests[i].pac_url, config.pac_url());
+      EXPECT_TRUE(tests[i].proxy_rules.Matches(config.proxy_rules()));
+    }
   }
 }
 
 TEST_F(ProxyConfigServiceLinuxTest, GconfNotification) {
   MockEnvironment* env = new MockEnvironment;
-  MockGConfSettingGetter* gconf_getter = new MockGConfSettingGetter;
+  MockSettingGetter* setting_getter = new MockSettingGetter;
   ProxyConfigServiceLinux* service =
-      new ProxyConfigServiceLinux(env, gconf_getter);
+      new ProxyConfigServiceLinux(env, setting_getter);
   SynchConfigGetter sync_config_getter(service);
   ProxyConfig config;
 
   // Start with no proxy.
-  gconf_getter->values.mode = "none";
+  setting_getter->values.mode = "none";
   sync_config_getter.SetupAndInitialFetch();
-  sync_config_getter.SyncGetLatestProxyConfig(&config);
+  EXPECT_EQ(ProxyConfigService::CONFIG_VALID,
+            sync_config_getter.SyncGetLatestProxyConfig(&config));
   EXPECT_FALSE(config.auto_detect());
 
   // Now set to auto-detect.
-  gconf_getter->values.mode = "auto";
-  // Simulate gconf notification callback.
+  setting_getter->values.mode = "auto";
+  // Simulate setting change notification callback.
   service->OnCheckProxyConfigSettings();
-  sync_config_getter.SyncGetLatestProxyConfig(&config);
+  EXPECT_EQ(ProxyConfigService::CONFIG_VALID,
+            sync_config_getter.SyncGetLatestProxyConfig(&config));
   EXPECT_TRUE(config.auto_detect());
 }
 
@@ -939,7 +1035,7 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
   // One of the tests below needs a worst-case long line prefix. We build it
   // programmatically so that it will always be the right size.
   std::string long_line;
-  size_t limit = ProxyConfigServiceLinux::GConfSettingGetter::BUFFER_SIZE - 1;
+  size_t limit = ProxyConfigServiceLinux::SettingGetter::BUFFER_SIZE - 1;
   for (size_t i = 0; i < limit; ++i)
     long_line += "-";
 
@@ -952,7 +1048,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
     std::string kioslaverc;
     EnvVarValues env_values;
 
-    // Expected outputs (fields of the ProxyConfig).
+    // Expected outputs (availability and fields of ProxyConfig).
+    ProxyConfigService::ConfigAvailability availability;
     bool auto_detect;
     GURL pac_url;
     ProxyRulesExpectation proxy_rules;
@@ -965,6 +1062,7 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       {},                                      // env_values
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                      // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -978,6 +1076,7 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       {},                                      // env_values
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       true,                       // auto_detect
       GURL(),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -992,6 +1091,7 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       {},                                      // env_values
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                         // auto_detect
       GURL("http://wpad/wpad.dat"),  // pac_url
       ProxyRulesExpectation::Empty(),
@@ -1006,6 +1106,7 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       {},                                      // env_values
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1024,6 +1125,7 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       {},                                      // env_values
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1042,6 +1144,7 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       {},                                      // env_values
 
       // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1059,6 +1162,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "NoProxyFor=.google.com\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1076,6 +1181,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "NoProxyFor=.google.com,.kde.org\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1093,6 +1200,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "NoProxyFor=.google.com\nReversedException=true\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerSchemeWithBypassReversed(
@@ -1110,6 +1219,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "NoProxyFor=google.com,kde.org,<local>\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1127,6 +1238,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "NoProxyFor=.google.com\nReversedException=true  \n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerSchemeWithBypassReversed(
@@ -1144,6 +1257,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "httpProxy=www.google.com\n[Other Section]\nftpProxy=ftp.foo.com\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1160,6 +1275,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       "[Proxy Settings]\r\nProxyType=1\r\nhttpProxy=www.google.com\r\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1176,6 +1293,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       "[Proxy Settings]\r\n\nProxyType=1\n\r\nhttpProxy=www.google.com\n\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1192,6 +1311,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
       "[Proxy Settings]\nProxyType[$e]=1\nhttpProxy[$e]=www.google.com\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1209,6 +1330,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "httpsProxy$e]=www.foo.com\nftpProxy=ftp.foo.com\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1226,6 +1349,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "  Proxy Config Script =  http:// foo\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL("http:// foo"),                     // pac_url
       ProxyRulesExpectation::Empty(),
@@ -1239,6 +1364,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           long_line + "httpsProxy=www.foo.com\nhttpProxy=www.google.com\n",
       {},                                          // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                       // auto_detect
       GURL(),                                      // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1256,6 +1383,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
           "httpsProxy=https_proxy\nftpProxy=ftp_proxy\nNoProxyFor=no_proxy\n",
       {},                                      // env_values
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::Empty(),
@@ -1281,6 +1410,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
         ".google.com, .kde.org",  // no_proxy
       },
 
+      // Expected result.
+      ProxyConfigService::CONFIG_VALID,
       false,                                   // auto_detect
       GURL(),                                  // pac_url
       ProxyRulesExpectation::PerScheme(
@@ -1307,11 +1438,15 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEConfigParser) {
     file_util::WriteFile(kioslaverc_, tests[i].kioslaverc.c_str(),
                          tests[i].kioslaverc.length());
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    ProxyConfigService::ConfigAvailability availability =
+        sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(tests[i].availability, availability);
 
-    EXPECT_EQ(tests[i].auto_detect, config.auto_detect());
-    EXPECT_EQ(tests[i].pac_url, config.pac_url());
-    EXPECT_TRUE(tests[i].proxy_rules.Matches(config.proxy_rules()));
+    if (availability == ProxyConfigService::CONFIG_VALID) {
+      EXPECT_EQ(tests[i].auto_detect, config.auto_detect());
+      EXPECT_EQ(tests[i].pac_url, config.pac_url());
+      EXPECT_TRUE(tests[i].proxy_rules.Matches(config.proxy_rules()));
+    }
   }
 }
 
@@ -1338,7 +1473,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEHomePicker) {
         new ProxyConfigServiceLinux(env));
     ProxyConfig config;
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(ProxyConfigService::CONFIG_VALID,
+              sync_config_getter.SyncGetLatestProxyConfig(&config));
     EXPECT_TRUE(config.auto_detect());
     EXPECT_EQ(GURL(), config.pac_url());
   }
@@ -1357,7 +1493,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEHomePicker) {
         new ProxyConfigServiceLinux(env));
     ProxyConfig config;
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(ProxyConfigService::CONFIG_VALID,
+              sync_config_getter.SyncGetLatestProxyConfig(&config));
     EXPECT_FALSE(config.auto_detect());
     EXPECT_EQ(slaverc4_pac_url, config.pac_url());
   }
@@ -1370,7 +1507,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEHomePicker) {
         new ProxyConfigServiceLinux(env));
     ProxyConfig config;
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(ProxyConfigService::CONFIG_VALID,
+              sync_config_getter.SyncGetLatestProxyConfig(&config));
     EXPECT_TRUE(config.auto_detect());
     EXPECT_EQ(GURL(), config.pac_url());
   }
@@ -1384,7 +1522,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEHomePicker) {
         new ProxyConfigServiceLinux(env));
     ProxyConfig config;
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(ProxyConfigService::CONFIG_VALID,
+              sync_config_getter.SyncGetLatestProxyConfig(&config));
     EXPECT_TRUE(config.auto_detect());
     EXPECT_EQ(GURL(), config.pac_url());
   }
@@ -1401,7 +1540,8 @@ TEST_F(ProxyConfigServiceLinuxTest, KDEHomePicker) {
         new ProxyConfigServiceLinux(env));
     ProxyConfig config;
     sync_config_getter.SetupAndInitialFetch();
-    sync_config_getter.SyncGetLatestProxyConfig(&config);
+    EXPECT_EQ(ProxyConfigService::CONFIG_VALID,
+              sync_config_getter.SyncGetLatestProxyConfig(&config));
     EXPECT_TRUE(config.auto_detect());
     EXPECT_EQ(GURL(), config.pac_url());
   }
