@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,7 @@
 
 namespace net {
 
-ProxyInfo::ProxyInfo() : config_id_(ProxyConfig::INVALID_ID) {
+ProxyInfo::ProxyInfo() : config_id_(ProxyConfig::kInvalidConfigID) {
 }
 
 ProxyInfo::~ProxyInfo() {
@@ -16,26 +16,30 @@ ProxyInfo::~ProxyInfo() {
 
 void ProxyInfo::Use(const ProxyInfo& other) {
   proxy_list_ = other.proxy_list_;
+  proxy_retry_info_ = other.proxy_retry_info_;
 }
 
 void ProxyInfo::UseDirect() {
   proxy_list_.SetSingleProxyServer(ProxyServer::Direct());
+  proxy_retry_info_.clear();
 }
 
 void ProxyInfo::UseNamedProxy(const std::string& proxy_uri_list) {
   proxy_list_.Set(proxy_uri_list);
+  proxy_retry_info_.clear();
 }
 
 void ProxyInfo::UseProxyServer(const ProxyServer& proxy_server) {
   proxy_list_.SetSingleProxyServer(proxy_server);
+  proxy_retry_info_.clear();
 }
 
 std::string ProxyInfo::ToPacString() const {
   return proxy_list_.ToPacString();
 }
 
-bool ProxyInfo::Fallback(ProxyRetryInfoMap* proxy_retry_info) {
-  return proxy_list_.Fallback(proxy_retry_info);
+bool ProxyInfo::Fallback(const BoundNetLog& net_log) {
+  return proxy_list_.Fallback(&proxy_retry_info_, net_log);
 }
 
 void ProxyInfo::DeprioritizeBadProxies(
