@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,12 @@
 #pragma once
 
 #include "base/logging.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/string16.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/completion_callback.h"
+#include "net/base/load_states.h"
+#include "net/base/net_export.h"
 #include "net/proxy/proxy_resolver_script_data.h"
 
 namespace net {
@@ -22,7 +24,7 @@ class ProxyInfo;
 // to use for a particular URL. Generally the backend for a ProxyResolver is
 // a PAC script, but it doesn't need to be. ProxyResolver can service multiple
 // requests at a time.
-class ProxyResolver {
+class NET_EXPORT_PRIVATE ProxyResolver {
  public:
   // Opaque pointer type, to return a handle to cancel outstanding requests.
   typedef void* RequestHandle;
@@ -41,13 +43,19 @@ class ProxyResolver {
   // |*request| is written to, and can be passed to CancelRequest().
   virtual int GetProxyForURL(const GURL& url,
                              ProxyInfo* results,
-                             CompletionCallback* callback,
+                             const net::CompletionCallback& callback,
                              RequestHandle* request,
                              const BoundNetLog& net_log) = 0;
 
   // Cancels |request|.
   virtual void CancelRequest(RequestHandle request) = 0;
 
+  // Gets the LoadState for |request|.
+  virtual LoadState GetLoadState(RequestHandle request) const = 0;
+
+  // Gets the LoadState for |request|. May be called from another thread.
+  virtual LoadState GetLoadStateThreadSafe(RequestHandle request) const = 0;
+
   // The PAC script backend can be specified to the ProxyResolver either via
   // URL, or via the javascript text itself.  If |expects_pac_bytes| is true,
   // then the ProxyResolverScriptData passed to SetPacScript() should
@@ -66,7 +74,7 @@ class ProxyResolver {
   // the result through |callback|.
   virtual int SetPacScript(
       const scoped_refptr<ProxyResolverScriptData>& pac_script,
-      CompletionCallback* callback) = 0;
+      const net::CompletionCallback& callback) = 0;
 
   // Optional shutdown code to be run before destruction. This is only used
   // by the multithreaded runner to signal cleanup from origin thread
