@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,9 @@
 #define NET_PROXY_PROXY_RESOLVER_MAC_H_
 #pragma once
 
+#include "base/compiler_specific.h"
 #include "googleurl/src/gurl.h"
+#include "net/base/net_export.h"
 #include "net/base/net_errors.h"
 #include "net/proxy/proxy_resolver.h"
 
@@ -14,27 +16,30 @@ namespace net {
 
 // Implementation of ProxyResolver that uses the Mac CFProxySupport to implement
 // proxies.
-class ProxyResolverMac : public ProxyResolver {
+class NET_EXPORT ProxyResolverMac : public ProxyResolver {
  public:
-  ProxyResolverMac() : ProxyResolver(false /*expects_pac_bytes*/) {}
+  ProxyResolverMac();
+  virtual ~ProxyResolverMac();
 
   // ProxyResolver methods:
   virtual int GetProxyForURL(const GURL& url,
                              ProxyInfo* results,
-                             CompletionCallback* callback,
+                             const net::CompletionCallback& callback,
                              RequestHandle* request,
-                             const BoundNetLog& net_log);
+                             const BoundNetLog& net_log) OVERRIDE;
 
-  virtual void CancelRequest(RequestHandle request) {
-    NOTREACHED();
-  }
+  virtual void CancelRequest(RequestHandle request) OVERRIDE;
+
+  virtual LoadState GetLoadState(RequestHandle request) const OVERRIDE;
+
+  virtual LoadState GetLoadStateThreadSafe(
+      RequestHandle request) const OVERRIDE;
+
+  virtual void CancelSetPacScript() OVERRIDE;
 
   virtual int SetPacScript(
       const scoped_refptr<ProxyResolverScriptData>& script_data,
-      CompletionCallback* /*callback*/) {
-    script_data_ = script_data;
-    return OK;
-  }
+      const net::CompletionCallback& /*callback*/) OVERRIDE;
 
  private:
   scoped_refptr<ProxyResolverScriptData> script_data_;
