@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,9 @@
 #define NET_PROXY_PROXY_RESOLVER_V8_H_
 #pragma once
 
-#include "base/scoped_ptr.h"
+#include "base/compiler_specific.h"
+#include "base/memory/scoped_ptr.h"
+#include "net/base/net_export.h"
 #include "net/proxy/proxy_resolver.h"
 
 namespace net {
@@ -24,14 +26,14 @@ class ProxyResolverJSBindings;
 // since only one will be running inside V8 at a time.
 //
 // It is important that *ALL* instances of V8 in the process be using
-// v8::Locker. If not there can be race conditions beween the non-locked V8
+// v8::Locker. If not there can be race conditions between the non-locked V8
 // instances and the locked V8 instances used by ProxyResolverV8 (assuming they
 // run on different threads).
 //
 // This is the case with the V8 instance used by chromium's renderer -- it runs
 // on a different thread from ProxyResolver (renderer thread vs PAC thread),
 // and does not use locking since it expects to be alone.
-class ProxyResolverV8 : public ProxyResolver {
+class NET_EXPORT_PRIVATE ProxyResolverV8 : public ProxyResolver {
  public:
   // Constructs a ProxyResolverV8 with custom bindings. ProxyResolverV8 takes
   // ownership of |custom_js_bindings| and deletes it when ProxyResolverV8
@@ -45,16 +47,19 @@ class ProxyResolverV8 : public ProxyResolver {
   // ProxyResolver implementation:
   virtual int GetProxyForURL(const GURL& url,
                              ProxyInfo* results,
-                             CompletionCallback* /*callback*/,
+                             const net::CompletionCallback& /*callback*/,
                              RequestHandle* /*request*/,
-                             const BoundNetLog& net_log);
-  virtual void CancelRequest(RequestHandle request);
-  virtual void CancelSetPacScript();
-  virtual void PurgeMemory();
-  virtual void Shutdown();
+                             const BoundNetLog& net_log) OVERRIDE;
+  virtual void CancelRequest(RequestHandle request) OVERRIDE;
+  virtual LoadState GetLoadState(RequestHandle request) const OVERRIDE;
+  virtual LoadState GetLoadStateThreadSafe(
+      RequestHandle request) const OVERRIDE;
+  virtual void CancelSetPacScript() OVERRIDE;
+  virtual void PurgeMemory() OVERRIDE;
+  virtual void Shutdown() OVERRIDE;
   virtual int SetPacScript(
       const scoped_refptr<ProxyResolverScriptData>& script_data,
-      CompletionCallback* /*callback*/);
+      const net::CompletionCallback& /*callback*/) OVERRIDE;
 
  private:
   // Context holds the Javascript state for the most recently loaded PAC
