@@ -12,6 +12,7 @@
 
 #include "base/string16.h"
 #include "net/base/completion_callback.h"
+#include "net/base/net_export.h"
 
 class GURL;
 
@@ -21,14 +22,17 @@ class URLRequestContext;
 
 // Interface for downloading a PAC script. Implementations can enforce
 // timeouts, maximum size constraints, content encoding, etc..
-class ProxyScriptFetcher {
+class NET_EXPORT_PRIVATE ProxyScriptFetcher {
  public:
   // Destruction should cancel any outstanding requests.
   virtual ~ProxyScriptFetcher() {}
 
   // Downloads the given PAC URL, and invokes |callback| on completion.
-  // On success |callback| is executed with a result code of OK, |*utf16_text|
-  // is filled with the response. On failure, the result text is
+  // Returns OK on success, otherwise the error code. If the return code is
+  // ERR_IO_PENDING, then the request completes asynchronously, and |callback|
+  // will be invoked later with the final error code.
+  // After synchronous or asynchronous completion with a result code of OK,
+  // |*utf16_text| is filled with the response. On failure, the result text is
   // an empty string, and the result code is a network error. Some special
   // network errors that may occur are:
   //
@@ -42,14 +46,14 @@ class ProxyScriptFetcher {
   //
   // Only one fetch is allowed to be outstanding at a time.
   virtual int Fetch(const GURL& url, string16* utf16_text,
-                    CompletionCallback* callback) = 0;
+                    const net::CompletionCallback& callback) = 0;
 
   // Aborts the in-progress fetch (if any).
   virtual void Cancel() = 0;
 
   // Returns the request context that this fetcher uses to issue downloads,
   // or NULL.
-  virtual URLRequestContext* GetRequestContext() = 0;
+  virtual URLRequestContext* GetRequestContext() const = 0;
 };
 
 }  // namespace net
