@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,7 @@
 #include <string>
 
 #include "base/logging.h"
-#include "base/mac/mac_util.h"
+#include "base/mac/foundation_util.h"
 #include "base/sys_string_conversions.h"
 
 namespace net {
@@ -25,8 +25,7 @@ ProxyServer ProxyServer::FromDictionary(Scheme scheme,
   }
 
   CFStringRef host_ref =
-      (CFStringRef)base::mac::GetValueFromDictionary(dict, host_key,
-                                                    CFStringGetTypeID());
+      base::mac::GetValueFromDictionary<CFStringRef>(dict, host_key);
   if (!host_ref) {
     LOG(WARNING) << "Could not find expected key "
                  << base::SysCFStringRefToUTF8(host_key)
@@ -36,8 +35,7 @@ ProxyServer ProxyServer::FromDictionary(Scheme scheme,
   std::string host = base::SysCFStringRefToUTF8(host_ref);
 
   CFNumberRef port_ref =
-      (CFNumberRef)base::mac::GetValueFromDictionary(dict, port_key,
-                                                    CFNumberGetTypeID());
+      base::mac::GetValueFromDictionary<CFNumberRef>(dict, port_key);
   int port;
   if (port_ref) {
     CFNumberGetValue(port_ref, kCFNumberIntType, &port);
