@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,37 +6,84 @@
 #define NET_PROXY_PROXY_SERVICE_H_
 #pragma once
 
+#include <string>
 #include <vector>
 
 #include "base/gtest_prod_util.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/synchronization/waitable_event.h"
+#include "base/threading/non_thread_safe.h"
 #include "net/base/completion_callback.h"
-#include "net/base/network_change_notifier.h"
+#include "net/base/load_states.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
+#include "net/base/network_change_notifier.h"
 #include "net/proxy/proxy_config_service.h"
-#include "net/proxy/proxy_server.h"
 #include "net/proxy/proxy_info.h"
+#include "net/proxy/proxy_server.h"
 
 class GURL;
 class MessageLoop;
 
 namespace net {
 
+class DhcpProxyScriptFetcher;
 class HostResolver;
-class InitProxyResolver;
+class NetworkDelegate;
 class ProxyResolver;
+class ProxyResolverScriptData;
+class ProxyScriptDecider;
 class ProxyScriptFetcher;
-class URLRequestContext;
 
 // This class can be used to resolve the proxy server to use when loading a
 // HTTP(S) URL.  It uses the given ProxyResolver to handle the actual proxy
 // resolution.  See ProxyResolverV8 for example.
-class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
-                     public NetworkChangeNotifier::Observer,
-                     public ProxyConfigService::Observer {
+class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
+                                public ProxyConfigService::Observer,
+                                NON_EXPORTED_BASE(public base::NonThreadSafe) {
  public:
+  // This interface defines the set of policies for when to poll the PAC
+  // script for changes.
+  //
+  // The polling policy decides what the next poll delay should be in
+  // milliseconds. It also decides how to wait for this delay -- either
+  // by starting a timer to do the poll at exactly |next_delay_ms|
+  // (MODE_USE_TIMER) or by waiting for the first network request issued after
+  // |next_delay_ms| (MODE_START_AFTER_ACTIVITY).
+  //
+  // The timer method is more precise and guarantees that polling happens when
+  // it was requested. However it has the disadvantage of causing spurious CPU
+  // and network activity. It is a reasonable choice to use for short poll
+  // intervals which only happen a couple times.
+  //
+  // However for repeated timers this will prevent the browser from going
+  // idle. MODE_START_AFTER_ACTIVITY solves this problem by only polling in
+  // direct response to network activity. The drawback to
+  // MODE_START_AFTER_ACTIVITY is since the poll is initiated only after the
+  // request is received, the first couple requests initiated after a long
+  // period of inactivity will likely see a stale version of the PAC script
+  // until the background polling gets a chance to update things.
+  class NET_EXPORT_PRIVATE PacPollPolicy {
+   public:
+    enum Mode {
+      MODE_USE_TIMER,
+      MODE_START_AFTER_ACTIVITY,
+    };
+
+    virtual ~PacPollPolicy() {}
+
+    // Decides the next poll delay. |current_delay| is the delay used
+    // by the preceding poll, or a negative TimeDelta value if determining
+    // the delay for the initial poll. |initial_error| is the network error
+    // code that the last PAC fetch (or WPAD initialization) failed with,
+    // or OK if it completed successfully. Implementations must set
+    // |next_delay| to a non-negative value.
+    virtual Mode GetNextDelay(int initial_error,
+                              base::TimeDelta current_delay,
+                              base::TimeDelta* next_delay) const = 0;
+  };
+
   // The instance takes ownership of |config_service| and |resolver|.
   // |net_log| is a possibly NULL destination to send log events to. It must
   // remain alive for the lifetime of this ProxyService.
@@ -44,6 +91,8 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
                ProxyResolver* resolver,
                NetLog* net_log);
 
+  virtual ~ProxyService();
+
   // Used internally to handle PAC queries.
   // TODO(eroman): consider naming this simply "Request".
   class PacRequest;
@@ -60,7 +109,7 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   // the caller will not need to cancel the request.
   //
   // We use the three possible proxy access types in the following order,
-  // doing fallback if one doesn't work.  See "init_proxy_resolver.h"
+  // doing fallback if one doesn't work.  See "pac_script_decider.h"
   // for the specifics.
   //   1.  WPAD auto-detection
   //   2.  PAC URL
@@ -69,7 +118,7 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   // Profiling information for the request is saved to |net_log| if non-NULL.
   int ResolveProxy(const GURL& url,
                    ProxyInfo* results,
-                   CompletionCallback* callback,
+                   const net::CompletionCallback& callback,
                    PacRequest** pac_request,
                    const BoundNetLog& net_log);
 
@@ -87,17 +136,27 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   // Profiling information for the request is saved to |net_log| if non-NULL.
   int ReconsiderProxyAfterError(const GURL& url,
                                 ProxyInfo* results,
-                                CompletionCallback* callback,
+                                const CompletionCallback& callback,
                                 PacRequest** pac_request,
                                 const BoundNetLog& net_log);
 
+  // Called to report that the last proxy connection succeeded.  If |proxy_info|
+  // has a non empty proxy_retry_info map, the proxies that have been tried (and
+  // failed) for this request will be marked as bad.
+  void ReportSuccess(const ProxyInfo& proxy_info);
+
   // Call this method with a non-null |pac_request| to cancel the PAC request.
   void CancelPacRequest(PacRequest* pac_request);
 
-  // Sets the ProxyScriptFetcher dependency. This is needed if the ProxyResolver
-  // is of type ProxyResolverWithoutFetch. ProxyService takes ownership of
-  // |proxy_script_fetcher|.
-  void SetProxyScriptFetcher(ProxyScriptFetcher* proxy_script_fetcher);
+  // Returns the LoadState for this |pac_request| which must be non-NULL.
+  LoadState GetLoadState(const PacRequest* pac_request) const;
+
+  // Sets the ProxyScriptFetcher and DhcpProxyScriptFetcher dependencies. This
+  // is needed if the ProxyResolver is of type ProxyResolverWithoutFetch.
+  // ProxyService takes ownership of both objects.
+  void SetProxyScriptFetchers(
+      ProxyScriptFetcher* proxy_script_fetcher,
+      DhcpProxyScriptFetcher* dhcp_proxy_script_fetcher);
   ProxyScriptFetcher* GetProxyScriptFetcher() const;
 
   // Tells this ProxyService to start using a new ProxyConfigService to
@@ -157,6 +216,10 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   // |proxy_script_fetcher| specifies the dependency to use for downloading
   // any PAC scripts. The resulting ProxyService will take ownership of it.
   //
+  // |dhcp_proxy_script_fetcher| specifies the dependency to use for attempting
+  // to retrieve the most appropriate PAC script configured in DHCP. The
+  // resulting ProxyService will take ownership of it.
+  //
   // |host_resolver| points to the host resolving dependency the PAC script
   // should use for any DNS queries. It must remain valid throughout the
   // lifetime of the ProxyService.
@@ -170,8 +233,10 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
       ProxyConfigService* proxy_config_service,
       size_t num_pac_threads,
       ProxyScriptFetcher* proxy_script_fetcher,
+      DhcpProxyScriptFetcher* dhcp_proxy_script_fetcher,
       HostResolver* host_resolver,
-      NetLog* net_log);
+      NetLog* net_log,
+      NetworkDelegate* network_delegate);
 
   // Same as CreateUsingV8ProxyResolver, except it uses system libraries
   // for evaluating the PAC script if available, otherwise skips
@@ -208,17 +273,26 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   static ProxyConfigService* CreateSystemProxyConfigService(
       MessageLoop* io_loop, MessageLoop* file_loop);
 
-#if UNIT_TEST
+  // This method should only be used by unit tests.
   void set_stall_proxy_auto_config_delay(base::TimeDelta delay) {
     stall_proxy_auto_config_delay_ = delay;
   }
-#endif
+
+  // This method should only be used by unit tests. Returns the previously
+  // active policy.
+  static const PacPollPolicy* set_pac_script_poll_policy(
+      const PacPollPolicy* policy);
+
+  // This method should only be used by unit tests. Creates an instance
+  // of the default internal PacPollPolicy used by ProxyService.
+  static scoped_ptr<PacPollPolicy> CreateDefaultPacPollPolicy();
 
  private:
-  friend class base::RefCountedThreadSafe<ProxyService>;
   FRIEND_TEST_ALL_PREFIXES(ProxyServiceTest, UpdateConfigAfterFailedAutodetect);
   FRIEND_TEST_ALL_PREFIXES(ProxyServiceTest, UpdateConfigFromPACToDirect);
   friend class PacRequest;
+  class InitProxyResolver;
+  class ProxyScriptDeciderPoller;
 
   // TODO(eroman): change this to a std::set. Note that this requires updating
   // some tests in proxy_service_unittest.cc such as:
@@ -233,8 +307,6 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
     STATE_READY,
   };
 
-  virtual ~ProxyService();
-
   // Resets all the variables associated with the current proxy configuration,
   // and rewinds the current state to |STATE_NONE|. Returns the previous value
   // of |current_state_|.  If |reset_fetched_config| is true then
@@ -280,12 +352,20 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   // Start initialization using |fetched_config_|.
   void InitializeUsingLastFetchedConfig();
 
-  // NetworkChangeNotifier::Observer
+  // Start the initialization skipping past the "decision" phase.
+  void InitializeUsingDecidedConfig(
+      int decider_result,
+      ProxyResolverScriptData* script_data,
+      const ProxyConfig& effective_config);
+
+  // NetworkChangeNotifier::IPAddressObserver
   // When this is called, we re-fetch PAC scripts and re-run WPAD.
-  virtual void OnIPAddressChanged();
+  virtual void OnIPAddressChanged() OVERRIDE;
 
   // ProxyConfigService::Observer
-  virtual void OnProxyConfigChanged(const ProxyConfig& config);
+  virtual void OnProxyConfigChanged(
+      const ProxyConfig& config,
+      ProxyConfigService::ConfigAvailability availability) OVERRIDE;
 
   scoped_ptr<ProxyConfigService> config_service_;
   scoped_ptr<ProxyResolver> resolver_;
@@ -315,8 +395,10 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   // external PAC script fetching.
   scoped_ptr<ProxyScriptFetcher> proxy_script_fetcher_;
 
-  // Callback for when |init_proxy_resolver_| is done.
-  CompletionCallbackImpl<ProxyService> init_proxy_resolver_callback_;
+  // The fetcher to use when attempting to download the most appropriate PAC
+  // script configured in DHCP, if any. Can be NULL if the ProxyResolver has
+  // no need for DHCP PAC script fetching.
+  scoped_ptr<DhcpProxyScriptFetcher> dhcp_proxy_script_fetcher_;
 
   // Helper to download the PAC script (wpad + custom) and apply fallback rules.
   //
@@ -324,8 +406,15 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
   // |proxy_resolver_| must outlive |init_proxy_resolver_|.
   scoped_ptr<InitProxyResolver> init_proxy_resolver_;
 
+  // Helper to poll the PAC script for changes.
+  scoped_ptr<ProxyScriptDeciderPoller> script_poller_;
+
   State current_state_;
 
+  // Either OK or an ERR_* value indicating that a permanent error (e.g.
+  // failed to fetch the PAC script) prevents proxy resolution.
+  int permanent_error_;
+
   // This is the log where any events generated by |init_proxy_resolver_| are
   // sent to.
   NetLog* net_log_;
@@ -341,7 +430,7 @@ class ProxyService : public base::RefCountedThreadSafe<ProxyService>,
 };
 
 // Wrapper for invoking methods on a ProxyService synchronously.
-class SyncProxyServiceHelper
+class NET_EXPORT SyncProxyServiceHelper
     : public base::RefCountedThreadSafe<SyncProxyServiceHelper> {
  public:
   SyncProxyServiceHelper(MessageLoop* io_message_loop,
@@ -368,7 +457,7 @@ class SyncProxyServiceHelper
   ProxyService* proxy_service_;
 
   base::WaitableEvent event_;
-  CompletionCallbackImpl<SyncProxyServiceHelper> callback_;
+  CompletionCallback callback_;
   ProxyInfo proxy_info_;
   int result_;
 };
