@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,8 +6,10 @@
 #define NET_SERVER_HTTP_SERVER_REQUEST_INFO_H_
 #pragma once
 
-#include <string>
 #include <map>
+#include <string>
+
+namespace net {
 
 // Meta information about an HTTP request.
 // This is geared toward servers in that it keeps a map of the headers and
@@ -18,6 +20,9 @@ class HttpServerRequestInfo {
   HttpServerRequestInfo();
   ~HttpServerRequestInfo();
 
+  // Returns header value for given header name.
+  std::string GetHeaderValue(const std::string& header_name) const;
+
   // Request method.
   std::string method;
 
@@ -32,4 +37,6 @@ class HttpServerRequestInfo {
   mutable HeadersMap headers;
 };
 
+}  // namespace net
+
 #endif  // NET_SERVER_HTTP_SERVER_REQUEST_INFO_H_
