@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -30,31 +30,39 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/task.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/time.h"
 #include "base/timer.h"
 #include "net/base/address_list.h"
 #include "net/base/completion_callback.h"
 #include "net/base/load_states.h"
 #include "net/base/net_errors.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/base/network_change_notifier.h"
 #include "net/base/request_priority.h"
-#include "net/socket/client_socket.h"
 #include "net/socket/client_socket_pool.h"
+#include "net/socket/stream_socket.h"
 
 namespace net {
 
 class ClientSocketHandle;
 
+// Returns the client socket reuse policy.
+NET_EXPORT_PRIVATE int GetSocketReusePolicy();
+
+// Sets the client socket reuse policy.
+// NOTE: 'policy' should be a valid ClientSocketReusePolicy enum value.
+NET_EXPORT void SetSocketReusePolicy(int policy);
+
 // ConnectJob provides an abstract interface for "connecting" a socket.
 // The connection may involve host resolution, tcp connection, ssl connection,
 // etc.
-class ConnectJob {
+class NET_EXPORT_PRIVATE ConnectJob {
  public:
-  class Delegate {
+  class NET_EXPORT_PRIVATE Delegate {
    public:
     Delegate() {}
     virtual ~Delegate() {}
@@ -89,7 +97,7 @@ class ConnectJob {
 
   // Releases |socket_| to the client.  On connection error, this should return
   // NULL.
-  ClientSocket* ReleaseSocket() { return socket_.release(); }
+  StreamSocket* ReleaseSocket() { return socket_.release(); }
 
   // Begins connecting the socket.  Returns OK on success, ERR_IO_PENDING if it
   // cannot complete synchronously without blocking, or another net error code
@@ -113,8 +121,8 @@ class ConnectJob {
   const BoundNetLog& net_log() const { return net_log_; }
 
  protected:
-  void set_socket(ClientSocket* socket);
-  ClientSocket* socket() { return socket_.get(); }
+  void set_socket(StreamSocket* socket);
+  StreamSocket* socket() { return socket_.get(); }
   void NotifyDelegateOfCompletion(int rv);
   void ResetTimer(base::TimeDelta remainingTime);
 
@@ -138,7 +146,7 @@ class ConnectJob {
   // Timer to abort jobs that take too long.
   base::OneShotTimer<ConnectJob> timer_;
   Delegate* delegate_;
-  scoped_ptr<ClientSocket> socket_;
+  scoped_ptr<StreamSocket> socket_;
   BoundNetLog net_log_;
   // A ConnectJob is idle until Connect() has been called.
   bool idle_;
@@ -154,9 +162,9 @@ namespace internal {
 // ClientSocketPoolBase adds templated definitions built on top of
 // ClientSocketPoolBaseHelper.  This class is not for external use, please use
 // ClientSocketPoolBase instead.
-class ClientSocketPoolBaseHelper
+class NET_EXPORT_PRIVATE ClientSocketPoolBaseHelper
     : public ConnectJob::Delegate,
-      public NetworkChangeNotifier::Observer {
+      public NetworkChangeNotifier::IPAddressObserver {
  public:
   typedef uint32 Flags;
 
@@ -166,26 +174,40 @@ class ClientSocketPoolBaseHelper
     NO_IDLE_SOCKETS = 0x1,  // Do not return an idle socket. Create a new one.
   };
 
-  class Request {
+  enum ClientSocketReusePolicy {
+    // Socket with largest amount of bytes transferred.
+    USE_WARMEST_SOCKET = 0,
+
+    // Socket which scores highest on large bytes transferred and low idle time.
+    USE_WARM_SOCKET = 1,
+
+    // Socket which was most recently used.
+    USE_LAST_ACCESSED_SOCKET = 2,
+  };
+
+  class NET_EXPORT_PRIVATE Request {
    public:
     Request(ClientSocketHandle* handle,
-            CompletionCallback* callback,
+            const CompletionCallback& callback,
             RequestPriority priority,
+            bool ignore_limits,
             Flags flags,
             const BoundNetLog& net_log);
 
     virtual ~Request();
 
     ClientSocketHandle* handle() const { return handle_; }
-    CompletionCallback* callback() const { return callback_; }
+    const CompletionCallback& callback() const { return callback_; }
     RequestPriority priority() const { return priority_; }
+    bool ignore_limits() const { return ignore_limits_; }
     Flags flags() const { return flags_; }
     const BoundNetLog& net_log() const { return net_log_; }
 
    private:
     ClientSocketHandle* const handle_;
-    CompletionCallback* const callback_;
+    CompletionCallback callback_;
     const RequestPriority priority_;
+    bool ignore_limits_;
     const Flags flags_;
     BoundNetLog net_log_;
 
@@ -215,7 +237,7 @@ class ClientSocketPoolBaseHelper
       base::TimeDelta used_idle_socket_timeout,
       ConnectJobFactory* connect_job_factory);
 
-  ~ClientSocketPoolBaseHelper();
+  virtual ~ClientSocketPoolBaseHelper();
 
   // See ClientSocketPool::RequestSocket for documentation on this function.
   // ClientSocketPoolBaseHelper takes ownership of |request|, which must be
@@ -233,7 +255,7 @@ class ClientSocketPoolBaseHelper
 
   // See ClientSocketPool::ReleaseSocket for documentation on this function.
   void ReleaseSocket(const std::string& group_name,
-                     ClientSocket* socket,
+                     StreamSocket* socket,
                      int id);
 
   // See ClientSocketPool::Flush for documentation on this function.
@@ -255,10 +277,11 @@ class ClientSocketPoolBaseHelper
   LoadState GetLoadState(const std::string& group_name,
                          const ClientSocketHandle* handle) const;
 
-  int ConnectRetryIntervalMs() const {
+  base::TimeDelta ConnectRetryInterval() const {
     // TODO(mbelshe): Make this tuned dynamically based on measured RTT.
     //                For now, just use the max retry interval.
-    return ClientSocketPool::kMaxConnectRetryIntervalMs;
+    return base::TimeDelta::FromMilliseconds(
+        ClientSocketPool::kMaxConnectRetryIntervalMs);
   }
 
   int NumConnectJobsInGroup(const std::string& group_name) const {
@@ -271,26 +294,36 @@ class ClientSocketPoolBaseHelper
 
   bool HasGroup(const std::string& group_name) const;
 
+  // Called to enable/disable cleaning up idle sockets. When enabled,
+  // idle sockets that have been around for longer than a period defined
+  // by kCleanupInterval are cleaned up using a timer. Otherwise they are
+  // closed next time client makes a request. This may reduce network
+  // activity and power consumption.
+  static bool cleanup_timer_enabled();
+  static bool set_cleanup_timer_enabled(bool enabled);
+
   // Closes all idle sockets if |force| is true.  Else, only closes idle
   // sockets that timed out or can't be reused.  Made public for testing.
   void CleanupIdleSockets(bool force);
 
   // See ClientSocketPool::GetInfoAsValue for documentation on this function.
-  DictionaryValue* GetInfoAsValue(const std::string& name,
-                                  const std::string& type) const;
+  base::DictionaryValue* GetInfoAsValue(const std::string& name,
+                                        const std::string& type) const;
 
   base::TimeDelta ConnectionTimeout() const {
     return connect_job_factory_->ConnectionTimeout();
   }
 
-  static void set_connect_backup_jobs_enabled(bool enabled);
+  static bool connect_backup_jobs_enabled();
+  static bool set_connect_backup_jobs_enabled(bool enabled);
+
   void EnableConnectBackupJobs();
 
   // ConnectJob::Delegate methods:
-  virtual void OnConnectJobComplete(int result, ConnectJob* job);
+  virtual void OnConnectJobComplete(int result, ConnectJob* job) OVERRIDE;
 
-  // NetworkChangeNotifier::Observer methods:
-  virtual void OnIPAddressChanged();
+  // NetworkChangeNotifier::IPAddressObserver methods:
+  virtual void OnIPAddressChanged() OVERRIDE;
 
  private:
   friend class base::RefCounted<ClientSocketPoolBaseHelper>;
@@ -309,7 +342,7 @@ class ClientSocketPoolBaseHelper
     // socket for a new request.
     bool ShouldCleanup(base::TimeTicks now, base::TimeDelta timeout) const;
 
-    ClientSocket* socket;
+    StreamSocket* socket;
     base::TimeTicks start_time;
   };
 
@@ -347,10 +380,10 @@ class ClientSocketPoolBaseHelper
       return pending_requests_.front()->priority();
     }
 
-    bool HasBackupJob() const { return !method_factory_.empty(); }
+    bool HasBackupJob() const { return weak_factory_.HasWeakPtrs(); }
 
     void CleanupBackupJob() {
-      method_factory_.RevokeAll();
+      weak_factory_.InvalidateWeakPtrs();
     }
 
     // Set a timer to create a backup socket if it takes too long to create one.
@@ -386,7 +419,7 @@ class ClientSocketPoolBaseHelper
     RequestQueue pending_requests_;
     int active_socket_count_;  // number of active sockets used by clients
     // A factory to pin the backup_job tasks.
-    ScopedRunnableMethodFactory<Group> method_factory_;
+    base::WeakPtrFactory<Group> weak_factory_;
   };
 
   typedef std::map<std::string, Group*> GroupMap;
@@ -394,11 +427,12 @@ class ClientSocketPoolBaseHelper
   typedef std::set<ConnectJob*> ConnectJobSet;
 
   struct CallbackResultPair {
-    CallbackResultPair() : callback(NULL), result(OK) {}
-    CallbackResultPair(CompletionCallback* callback_in, int result_in)
+    CallbackResultPair() : result(OK) {}
+    CallbackResultPair(const CompletionCallback& callback_in, int result_in)
         : callback(callback_in), result(result_in) {}
+    ~CallbackResultPair();
 
-    CompletionCallback* callback;
+    CompletionCallback callback;
     int result;
   };
 
@@ -407,7 +441,7 @@ class ClientSocketPoolBaseHelper
 
   static void InsertRequestIntoQueue(const Request* r,
                                      RequestQueue* pending_requests);
-  static const Request* RemoveRequestFromQueue(RequestQueue::iterator it,
+  static const Request* RemoveRequestFromQueue(const RequestQueue::iterator& it,
                                                Group* group);
 
   Group* GetOrCreateGroup(const std::string& group_name);
@@ -418,6 +452,9 @@ class ClientSocketPoolBaseHelper
   void IncrementIdleCount();
   void DecrementIdleCount();
 
+  // Start cleanup timer for idle sockets.
+  void StartIdleSocketTimer();
+
   // Scans the group map for groups which have an available socket slot and
   // at least one pending request. Returns true if any groups are stalled, and
   // if so, fills |group| and |group_name| with data of the stalled group
@@ -440,7 +477,7 @@ class ClientSocketPoolBaseHelper
   void ProcessPendingRequest(const std::string& group_name, Group* group);
 
   // Assigns |socket| to |handle| and updates |group|'s counters appropriately.
-  void HandOutSocket(ClientSocket* socket,
+  void HandOutSocket(StreamSocket* socket,
                      bool reused,
                      ClientSocketHandle* handle,
                      base::TimeDelta time_idle,
@@ -448,7 +485,7 @@ class ClientSocketPoolBaseHelper
                      const BoundNetLog& net_log);
 
   // Adds |socket| to the list of idle sockets for |group|.
-  void AddIdleSocket(ClientSocket* socket, Group* group);
+  void AddIdleSocket(StreamSocket* socket, Group* group);
 
   // Iterates through |group_map_|, canceling all ConnectJobs and deleting
   // groups if they are no longer needed.
@@ -494,7 +531,7 @@ class ClientSocketPoolBaseHelper
   // current message loop.  Inserts |callback| into |pending_callback_map_|,
   // keyed by |handle|.
   void InvokeUserCallbackLater(
-      ClientSocketHandle* handle, CompletionCallback* callback, int rv);
+      ClientSocketHandle* handle, const CompletionCallback& callback, int rv);
 
   // Invokes the user callback for |handle|.  By the time this task has run,
   // it's possible that the request has been cancelled, so |handle| may not
@@ -528,13 +565,16 @@ class ClientSocketPoolBaseHelper
   // The maximum number of sockets kept per group.
   const int max_sockets_per_group_;
 
+  // Whether to use timer to cleanup idle sockets.
+  bool use_cleanup_timer_;
+
   // The time to wait until closing idle sockets.
   const base::TimeDelta unused_idle_socket_timeout_;
   const base::TimeDelta used_idle_socket_timeout_;
 
   const scoped_ptr<ConnectJobFactory> connect_job_factory_;
 
-  // TODO(vandebo) Remove when backup jobs move to TCPClientSocketPool
+  // TODO(vandebo) Remove when backup jobs move to TransportClientSocketPool
   bool connect_backup_jobs_enabled_;
 
   // A unique id for the pool.  It gets incremented every time we Flush() the
@@ -542,29 +582,27 @@ class ClientSocketPoolBaseHelper
   // make sure that they are discarded rather than reused.
   int pool_generation_number_;
 
-  ScopedRunnableMethodFactory<ClientSocketPoolBaseHelper> method_factory_;
+  base::WeakPtrFactory<ClientSocketPoolBaseHelper> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(ClientSocketPoolBaseHelper);
 };
 
 }  // namespace internal
 
-// The maximum duration, in seconds, to keep used idle persistent sockets alive.
-static const int kUsedIdleSocketTimeout = 300;  // 5 minutes
-
 template <typename SocketParams>
 class ClientSocketPoolBase {
  public:
   class Request : public internal::ClientSocketPoolBaseHelper::Request {
    public:
     Request(ClientSocketHandle* handle,
-            CompletionCallback* callback,
+            const CompletionCallback& callback,
             RequestPriority priority,
             internal::ClientSocketPoolBaseHelper::Flags flags,
+            bool ignore_limits,
             const scoped_refptr<SocketParams>& params,
             const BoundNetLog& net_log)
         : internal::ClientSocketPoolBaseHelper::Request(
-              handle, callback, priority, flags, net_log),
+              handle, callback, priority, ignore_limits, flags, net_log),
           params_(params) {}
 
     const scoped_refptr<SocketParams>& params() const { return params_; }
@@ -617,11 +655,12 @@ class ClientSocketPoolBase {
                     const scoped_refptr<SocketParams>& params,
                     RequestPriority priority,
                     ClientSocketHandle* handle,
-                    CompletionCallback* callback,
+                    const CompletionCallback& callback,
                     const BoundNetLog& net_log) {
     Request* request =
         new Request(handle, callback, priority,
                     internal::ClientSocketPoolBaseHelper::NORMAL,
+                    params->ignore_limits(),
                     params, net_log);
     return helper_.RequestSocket(group_name, request);
   }
@@ -634,9 +673,10 @@ class ClientSocketPoolBase {
                       int num_sockets,
                       const BoundNetLog& net_log) {
     const Request request(NULL /* no handle */,
-                          NULL /* no callback */,
+                          CompletionCallback(),
                           LOWEST,
                           internal::ClientSocketPoolBaseHelper::NO_IDLE_SOCKETS,
+                          params->ignore_limits(),
                           params,
                           net_log);
     helper_.RequestSockets(group_name, request, num_sockets);
@@ -647,7 +687,7 @@ class ClientSocketPoolBase {
     return helper_.CancelRequest(group_name, handle);
   }
 
-  void ReleaseSocket(const std::string& group_name, ClientSocket* socket,
+  void ReleaseSocket(const std::string& group_name, StreamSocket* socket,
                      int id) {
     return helper_.ReleaseSocket(group_name, socket, id);
   }
@@ -685,8 +725,8 @@ class ClientSocketPoolBase {
     return helper_.CleanupIdleSockets(force);
   }
 
-  DictionaryValue* GetInfoAsValue(const std::string& name,
-                                  const std::string& type) const {
+  base::DictionaryValue* GetInfoAsValue(const std::string& name,
+                                        const std::string& type) const {
     return helper_.GetInfoAsValue(name, type);
   }
 
