@@ -2,11 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// This file is only inclued in ssl_client_socket_nss.cc and
+// This file is only included in ssl_client_socket_nss.cc and
 // ssl_server_socket_nss.cc to share common functions of NSS.
 
 #ifndef NET_SOCKET_NSS_SSL_UTIL_H_
-#define NEt_SOCKET_NSS_SSL_UTIL_H_
+#define NET_SOCKET_NSS_SSL_UTIL_H_
 
 #include <prerror.h>
 
@@ -28,7 +28,7 @@ PRErrorCode MapErrorToNSS(int result);
 // Map NSS error code to network error code.
 int MapNSSError(PRErrorCode err);
 
-// Map NSS handshake error to network error code.
+// Map NSS error code from the first SSL handshake to network error code.
 int MapNSSHandshakeError(PRErrorCode err);
 
 }  // namespace net
