@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,15 +10,15 @@
 
 #include "base/basictypes.h"
 #include "base/gtest_prod_util.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "googleurl/src/gurl.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "net/base/address_list.h"
 #include "net/base/completion_callback.h"
 #include "net/base/host_resolver.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_log.h"
-#include "net/socket/client_socket.h"
+#include "net/base/single_request_host_resolver.h"
+#include "net/socket/stream_socket.h"
 
 namespace net {
 
@@ -26,7 +26,7 @@ class ClientSocketHandle;
 class BoundNetLog;
 
 // The SOCKS client socket implementation
-class SOCKSClientSocket : public ClientSocket {
+class NET_EXPORT_PRIVATE SOCKSClientSocket : public StreamSocket {
  public:
   // Takes ownership of the |transport_socket|, which should already be
   // connected by the time Connect() is called.
@@ -37,35 +37,42 @@ class SOCKSClientSocket : public ClientSocket {
                     const HostResolver::RequestInfo& req_info,
                     HostResolver* host_resolver);
 
-  // Deprecated constructor (http://crbug.com/37810) that takes a ClientSocket.
-  SOCKSClientSocket(ClientSocket* transport_socket,
+  // Deprecated constructor (http://crbug.com/37810) that takes a StreamSocket.
+  SOCKSClientSocket(StreamSocket* transport_socket,
                     const HostResolver::RequestInfo& req_info,
                     HostResolver* host_resolver);
 
   // On destruction Disconnect() is called.
   virtual ~SOCKSClientSocket();
 
-  // ClientSocket methods:
+  // StreamSocket implementation.
 
   // Does the SOCKS handshake and completes the protocol.
-  virtual int Connect(CompletionCallback* callback);
-  virtual void Disconnect();
-  virtual bool IsConnected() const;
-  virtual bool IsConnectedAndIdle() const;
-  virtual const BoundNetLog& NetLog() const;
-  virtual void SetSubresourceSpeculation();
-  virtual void SetOmniboxSpeculation();
-  virtual bool WasEverUsed() const;
-  virtual bool UsingTCPFastOpen() const;
+  virtual int Connect(const CompletionCallback& callback) OVERRIDE;
+  virtual void Disconnect() OVERRIDE;
+  virtual bool IsConnected() const OVERRIDE;
+  virtual bool IsConnectedAndIdle() const OVERRIDE;
+  virtual const BoundNetLog& NetLog() const OVERRIDE;
+  virtual void SetSubresourceSpeculation() OVERRIDE;
+  virtual void SetOmniboxSpeculation() OVERRIDE;
+  virtual bool WasEverUsed() const OVERRIDE;
+  virtual bool UsingTCPFastOpen() const OVERRIDE;
+  virtual int64 NumBytesRead() const OVERRIDE;
+  virtual base::TimeDelta GetConnectTimeMicros() const OVERRIDE;
 
-  // Socket methods:
-  virtual int Read(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual int Write(IOBuffer* buf, int buf_len, CompletionCallback* callback);
+  // Socket implementation.
+  virtual int Read(IOBuffer* buf,
+                   int buf_len,
+                   const CompletionCallback& callback) OVERRIDE;
+  virtual int Write(IOBuffer* buf,
+                    int buf_len,
+                    const CompletionCallback& callback) OVERRIDE;
 
-  virtual bool SetReceiveBufferSize(int32 size);
-  virtual bool SetSendBufferSize(int32 size);
+  virtual bool SetReceiveBufferSize(int32 size) OVERRIDE;
+  virtual bool SetSendBufferSize(int32 size) OVERRIDE;
 
-  virtual int GetPeerAddress(AddressList* address) const;
+  virtual int GetPeerAddress(AddressList* address) const OVERRIDE;
+  virtual int GetLocalAddress(IPEndPoint* address) const OVERRIDE;
 
  private:
   FRIEND_TEST_ALL_PREFIXES(SOCKSClientSocketTest, CompleteHandshake);
@@ -82,16 +89,6 @@ class SOCKSClientSocket : public ClientSocket {
     STATE_NONE,
   };
 
-  // The SOCKS proxy connection either has the hostname resolved via the
-  // client or via the server. This enum stores the state of the SOCKS
-  // connection. If the client can resolve the hostname, the connection is
-  // SOCKS4, otherwise it is SOCKS4A.
-  enum SocksVersion {
-    kSOCKS4Unresolved,
-    kSOCKS4,
-    kSOCKS4a,
-  };
-
   void DoCallback(int result);
   void OnIOComplete(int result);
 
@@ -105,16 +102,13 @@ class SOCKSClientSocket : public ClientSocket {
 
   const std::string BuildHandshakeWriteBuffer() const;
 
-  CompletionCallbackImpl<SOCKSClientSocket> io_callback_;
-
   // Stores the underlying socket.
   scoped_ptr<ClientSocketHandle> transport_;
 
   State next_state_;
-  SocksVersion socks_version_;
 
   // Stores the callback to the layer above, called on completing Connect().
-  CompletionCallback* user_callback_;
+  CompletionCallback user_callback_;
 
   // This IOBuffer is used by the class to read and write
   // SOCKS handshake data. The length contains the expected size to
