@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,16 +19,13 @@
 #include "base/metrics/stats_counters.h"
 #include "base/string_util.h"
 #include "net/base/address_list_net_log_param.h"
+#include "net/base/connection_type_histograms.h"
 #include "net/base/io_buffer.h"
+#include "net/base/ip_endpoint.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_log.h"
 #include "net/base/net_util.h"
 #include "net/base/network_change_notifier.h"
-#if defined(USE_SYSTEM_LIBEVENT)
-#include <event.h>
-#else
-#include "third_party/libevent/event.h"
-#endif
 
 namespace net {
 
@@ -68,43 +65,34 @@ void SetTCPKeepAlive(int fd) {
 #endif
 }
 
-// Convert values from <errno.h> to values from "net/base/net_errors.h"
-int MapPosixError(int os_error) {
-  // There are numerous posix error codes, but these are the ones we thus far
-  // find interesting.
-  switch (os_error) {
-    case EAGAIN:
-#if EWOULDBLOCK != EAGAIN
-    case EWOULDBLOCK:
-#endif
-      return ERR_IO_PENDING;
-    case EACCES:
-      return ERR_ACCESS_DENIED;
-    case ENETDOWN:
-      return ERR_INTERNET_DISCONNECTED;
-    case ETIMEDOUT:
-      return ERR_TIMED_OUT;
-    case ECONNRESET:
-    case ENETRESET:  // Related to keep-alive
-    case EPIPE:
-      return ERR_CONNECTION_RESET;
-    case ECONNABORTED:
-      return ERR_CONNECTION_ABORTED;
-    case ECONNREFUSED:
-      return ERR_CONNECTION_REFUSED;
-    case EHOSTUNREACH:
-    case EHOSTDOWN:
-    case ENETUNREACH:
-      return ERR_ADDRESS_UNREACHABLE;
-    case EADDRNOTAVAIL:
-      return ERR_ADDRESS_INVALID;
-    case 0:
-      return OK;
-    default:
-      LOG(WARNING) << "Unknown error " << os_error
-                   << " mapped to net::ERR_FAILED";
-      return ERR_FAILED;
+// Sets socket parameters. Returns the OS error code (or 0 on
+// success).
+int SetupSocket(int socket) {
+  if (SetNonBlocking(socket))
+    return errno;
+
+  // This mirrors the behaviour on Windows. See the comment in
+  // tcp_client_socket_win.cc after searching for "NODELAY".
+  DisableNagle(socket);  // If DisableNagle fails, we don't care.
+  SetTCPKeepAlive(socket);
+
+  return 0;
+}
+
+// Creates a new socket and sets default parameters for it. Returns
+// the OS error code (or 0 on success).
+int CreateSocket(int family, int* socket) {
+  *socket = ::socket(family, SOCK_STREAM, IPPROTO_TCP);
+  if (*socket == kInvalidSocket)
+    return errno;
+  int error = SetupSocket(*socket);
+  if (error) {
+    if (HANDLE_EINTR(close(*socket)) < 0)
+      PLOG(ERROR) << "close";
+    *socket = kInvalidSocket;
+    return error;
   }
+  return 0;
 }
 
 int MapConnectError(int os_error) {
@@ -114,7 +102,7 @@ int MapConnectError(int os_error) {
     case ETIMEDOUT:
       return ERR_CONNECTION_TIMED_OUT;
     default: {
-      int net_error = MapPosixError(os_error);
+      int net_error = MapSystemError(os_error);
       if (net_error == ERR_FAILED)
         return ERR_CONNECTION_FAILED;  // More specific than ERR_FAILED.
 
@@ -137,18 +125,18 @@ TCPClientSocketLibevent::TCPClientSocketLibevent(
     net::NetLog* net_log,
     const net::NetLog::Source& source)
     : socket_(kInvalidSocket),
+      bound_socket_(kInvalidSocket),
       addresses_(addresses),
       current_ai_(NULL),
       read_watcher_(this),
       write_watcher_(this),
-      read_callback_(NULL),
-      write_callback_(NULL),
       next_connect_state_(CONNECT_STATE_NONE),
       connect_os_error_(0),
       net_log_(BoundNetLog::Make(net_log, NetLog::SOURCE_SOCKET)),
       previously_disconnected_(false),
       use_tcp_fastopen_(false),
-      tcp_fastopen_connected_(false) {
+      tcp_fastopen_connected_(false),
+      num_bytes_read_(0) {
   scoped_refptr<NetLog::EventParameters> params;
   if (source.is_valid())
     params = new NetLogSourceParameter("source_dependency", source);
@@ -163,26 +151,63 @@ TCPClientSocketLibevent::~TCPClientSocketLibevent() {
   net_log_.EndEvent(NetLog::TYPE_SOCKET_ALIVE, NULL);
 }
 
-void TCPClientSocketLibevent::AdoptSocket(int socket) {
+int TCPClientSocketLibevent::AdoptSocket(int socket) {
   DCHECK_EQ(socket_, kInvalidSocket);
+
+  int error = SetupSocket(socket);
+  if (error)
+    return MapSystemError(error);
+
   socket_ = socket;
-  int error = SetupSocket();
-  DCHECK_EQ(0, error);
-  // This is to make GetPeerAddress work. It's up to the test that is calling
-  // this function to ensure that address_ contains a reasonable address for
-  // this socket. (i.e. at least match IPv4 vs IPv6!).
+
+  // This is to make GetPeerAddress() work. It's up to the caller ensure
+  // that |address_| contains a reasonable address for this
+  // socket. (i.e. at least match IPv4 vs IPv6!).
   current_ai_ = addresses_.head();
   use_history_.set_was_ever_connected();
+
+  return OK;
 }
 
-int TCPClientSocketLibevent::Connect(CompletionCallback* callback) {
+int TCPClientSocketLibevent::Bind(const IPEndPoint& address) {
+  if (current_ai_ != NULL || bind_address_.get()) {
+    // Cannot bind the socket if we are already bound connected or
+    // connecting.
+    return ERR_UNEXPECTED;
+  }
+
+  sockaddr_storage addr_storage;
+  sockaddr* addr = reinterpret_cast<struct sockaddr*>(&addr_storage);
+  size_t addr_len = sizeof(addr_storage);
+  if (!address.ToSockAddr(addr, &addr_len))
+    return ERR_INVALID_ARGUMENT;
+
+  // Create |bound_socket_| and try to bound it to |address|.
+  int error = CreateSocket(address.GetFamily(), &bound_socket_);
+  if (error)
+    return MapSystemError(error);
+
+  if (HANDLE_EINTR(bind(bound_socket_, addr, addr_len))) {
+    error = errno;
+    if (HANDLE_EINTR(close(bound_socket_)) < 0)
+      PLOG(ERROR) << "close";
+    bound_socket_ = kInvalidSocket;
+    return MapSystemError(error);
+  }
+
+  bind_address_.reset(new IPEndPoint(address));
+
+  return 0;
+}
+
+int TCPClientSocketLibevent::Connect(const CompletionCallback& callback) {
   DCHECK(CalledOnValidThread());
 
   // If already connected, then just return OK.
   if (socket_ != kInvalidSocket)
     return OK;
 
-  static base::StatsCounter connects("tcp.connect");
+  base::StatsCounter connects("tcp.connect");
   connects.Increment();
 
   DCHECK(!waiting_connect());
@@ -199,7 +224,7 @@ int TCPClientSocketLibevent::Connect(CompletionCallback* callback) {
   int rv = DoConnectLoop(OK);
   if (rv == ERR_IO_PENDING) {
     // Synchronous operation not supported.
-    DCHECK(callback);
+    DCHECK(!callback.is_null());
     write_callback_ = callback;
   } else {
     LogConnectCompletion(rv);
@@ -249,13 +274,30 @@ int TCPClientSocketLibevent::DoConnect() {
 
   next_connect_state_ = CONNECT_STATE_CONNECT_COMPLETE;
 
-  // Create a non-blocking socket.
-  connect_os_error_ = CreateSocket(current_ai_);
-  if (connect_os_error_)
-    return MapPosixError(connect_os_error_);
+  if (bound_socket_ != kInvalidSocket) {
+    DCHECK(bind_address_.get());
+    socket_ = bound_socket_;
+    bound_socket_ = kInvalidSocket;
+  } else {
+    // Create a non-blocking socket.
+    connect_os_error_ = CreateSocket(current_ai_->ai_family, &socket_);
+    if (connect_os_error_)
+      return MapSystemError(connect_os_error_);
+
+    if (bind_address_.get()) {
+      sockaddr_storage addr_storage;
+      sockaddr* addr = reinterpret_cast<struct sockaddr*>(&addr_storage);
+      size_t addr_len = sizeof(addr_storage);
+      if (!bind_address_->ToSockAddr(addr, &addr_len))
+        return ERR_INVALID_ARGUMENT;
+      if (HANDLE_EINTR(bind(socket_, addr, addr_len)))
+        return MapSystemError(errno);
+    }
+  }
 
   // Connect the socket.
   if (!use_tcp_fastopen_) {
+    connect_start_time_ = base::TimeTicks::Now();
     if (!HANDLE_EINTR(connect(socket_, current_ai_->ai_addr,
                               static_cast<int>(current_ai_->ai_addrlen)))) {
       // Connected without waiting!
@@ -279,7 +321,7 @@ int TCPClientSocketLibevent::DoConnect() {
           &write_watcher_)) {
     connect_os_error_ = errno;
     DVLOG(1) << "WatchFileDescriptor failed: " << connect_os_error_;
-    return MapPosixError(connect_os_error_);
+    return MapSystemError(connect_os_error_);
   }
 
   return ERR_IO_PENDING;
@@ -294,9 +336,9 @@ int TCPClientSocketLibevent::DoConnectComplete(int result) {
     params = new NetLogIntegerParameter("os_error", os_error);
   net_log_.EndEvent(NetLog::TYPE_TCP_CONNECT_ATTEMPT, params);
 
-  write_socket_watcher_.StopWatchingFileDescriptor();
-
   if (result == OK) {
+    connect_time_micros_ = base::TimeTicks::Now() - connect_start_time_;
+    write_socket_watcher_.StopWatchingFileDescriptor();
     use_history_.set_was_ever_connected();
     return OK;  // Done!
   }
@@ -342,6 +384,15 @@ bool TCPClientSocketLibevent::IsConnected() const {
   if (socket_ == kInvalidSocket || waiting_connect())
     return false;
 
+  if (use_tcp_fastopen_ && !tcp_fastopen_connected_) {
+    // With TCP FastOpen, we pretend that the socket is connected.
+    // This allows GetPeerAddress() to return current_ai_ as the peer
+    // address.  Since we don't fail over to the next address if
+    // sendto() fails, current_ai_ is the only possible peer address.
+    CHECK(current_ai_);
+    return true;
+  }
+
   // Check if connection is alive.
   char c;
   int rv = HANDLE_EINTR(recv(socket_, &c, 1, MSG_PEEK));
@@ -359,6 +410,9 @@ bool TCPClientSocketLibevent::IsConnectedAndIdle() const {
   if (socket_ == kInvalidSocket || waiting_connect())
     return false;
 
+  // TODO(wtc): should we also handle the TCP FastOpen case here,
+  // as we do in IsConnected()?
+
   // Check if connection is alive and we haven't received any data
   // unexpectedly.
   char c;
@@ -373,35 +427,36 @@ bool TCPClientSocketLibevent::IsConnectedAndIdle() const {
 
 int TCPClientSocketLibevent::Read(IOBuffer* buf,
                                   int buf_len,
-                                  CompletionCallback* callback) {
+                                  const CompletionCallback& callback) {
   DCHECK(CalledOnValidThread());
   DCHECK_NE(kInvalidSocket, socket_);
   DCHECK(!waiting_connect());
-  DCHECK(!read_callback_);
+  DCHECK(read_callback_.is_null());
   // Synchronous operation not supported
-  DCHECK(callback);
+  DCHECK(!callback.is_null());
   DCHECK_GT(buf_len, 0);
 
   int nread = HANDLE_EINTR(read(socket_, buf->data(), buf_len));
   if (nread >= 0) {
-    static base::StatsCounter read_bytes("tcp.read_bytes");
+    base::StatsCounter read_bytes("tcp.read_bytes");
     read_bytes.Add(nread);
+    num_bytes_read_ += static_cast<int64>(nread);
     if (nread > 0)
       use_history_.set_was_used_to_convey_data();
-    LogByteTransfer(
-        net_log_, NetLog::TYPE_SOCKET_BYTES_RECEIVED, nread, buf->data());
+    net_log_.AddByteTransferEvent(NetLog::TYPE_SOCKET_BYTES_RECEIVED, nread,
+                                  buf->data());
     return nread;
   }
   if (errno != EAGAIN && errno != EWOULDBLOCK) {
     DVLOG(1) << "read failed, errno " << errno;
-    return MapPosixError(errno);
+    return MapSystemError(errno);
   }
 
   if (!MessageLoopForIO::current()->WatchFileDescriptor(
           socket_, true, MessageLoopForIO::WATCH_READ,
           &read_socket_watcher_, &read_watcher_)) {
     DVLOG(1) << "WatchFileDescriptor failed on read, errno " << errno;
-    return MapPosixError(errno);
+    return MapSystemError(errno);
   }
 
   read_buf_ = buf;
@@ -412,33 +467,33 @@ int TCPClientSocketLibevent::Read(IOBuffer* buf,
 
 int TCPClientSocketLibevent::Write(IOBuffer* buf,
                                    int buf_len,
-                                   CompletionCallback* callback) {
+                                   const CompletionCallback& callback) {
   DCHECK(CalledOnValidThread());
   DCHECK_NE(kInvalidSocket, socket_);
   DCHECK(!waiting_connect());
-  DCHECK(!write_callback_);
+  DCHECK(write_callback_.is_null());
   // Synchronous operation not supported
-  DCHECK(callback);
+  DCHECK(!callback.is_null());
   DCHECK_GT(buf_len, 0);
 
   int nwrite = InternalWrite(buf, buf_len);
   if (nwrite >= 0) {
-    static base::StatsCounter write_bytes("tcp.write_bytes");
+    base::StatsCounter write_bytes("tcp.write_bytes");
     write_bytes.Add(nwrite);
     if (nwrite > 0)
       use_history_.set_was_used_to_convey_data();
-    LogByteTransfer(
-        net_log_, NetLog::TYPE_SOCKET_BYTES_SENT, nwrite, buf->data());
+    net_log_.AddByteTransferEvent(NetLog::TYPE_SOCKET_BYTES_SENT, nwrite,
+                                  buf->data());
     return nwrite;
   }
   if (errno != EAGAIN && errno != EWOULDBLOCK)
-    return MapPosixError(errno);
+    return MapSystemError(errno);
 
   if (!MessageLoopForIO::current()->WatchFileDescriptor(
           socket_, true, MessageLoopForIO::WATCH_WRITE,
           &write_socket_watcher_, &write_watcher_)) {
     DVLOG(1) << "WatchFileDescriptor failed on write, errno " << errno;
-    return MapPosixError(errno);
+    return MapSystemError(errno);
   }
 
   write_buf_ = buf;
@@ -497,55 +552,54 @@ bool TCPClientSocketLibevent::SetSendBufferSize(int32 size) {
   return rv == 0;
 }
 
+void TCPClientSocketLibevent::LogConnectCompletion(int net_error) {
+  if (net_error == OK)
+    UpdateConnectionTypeHistograms(CONNECTION_ANY);
 
-int TCPClientSocketLibevent::CreateSocket(const addrinfo* ai) {
-  socket_ = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
-  if (socket_ == kInvalidSocket)
-    return errno;
-  return SetupSocket();
-}
-
-int TCPClientSocketLibevent::SetupSocket() {
-  if (SetNonBlocking(socket_)) {
-    const int err = errno;
-    close(socket_);
-    socket_ = kInvalidSocket;
-    return err;
+  if (net_error != OK) {
+    net_log_.EndEventWithNetErrorCode(NetLog::TYPE_TCP_CONNECT, net_error);
+    return;
   }
 
-  // This mirrors the behaviour on Windows. See the comment in
-  // tcp_client_socket_win.cc after searching for "NODELAY".
-  DisableNagle(socket_);  // If DisableNagle fails, we don't care.
-  SetTCPKeepAlive(socket_);
+  struct sockaddr_storage source_address;
+  socklen_t addrlen = sizeof(source_address);
+  int rv = getsockname(
+      socket_, reinterpret_cast<struct sockaddr*>(&source_address), &addrlen);
+  if (rv != 0) {
+    PLOG(ERROR) << "getsockname() [rv: " << rv << "] error: ";
+    NOTREACHED();
+    net_log_.EndEventWithNetErrorCode(NetLog::TYPE_TCP_CONNECT, rv);
+    return;
+  }
 
-  return 0;
-}
-
-void TCPClientSocketLibevent::LogConnectCompletion(int net_error) {
-  scoped_refptr<NetLog::EventParameters> params;
-  if (net_error != OK)
-    params = new NetLogIntegerParameter("net_error", net_error);
-  net_log_.EndEvent(NetLog::TYPE_TCP_CONNECT, params);
+  const std::string source_address_str =
+      NetAddressToStringWithPort(
+          reinterpret_cast<const struct sockaddr*>(&source_address),
+          sizeof(source_address));
+  net_log_.EndEvent(NetLog::TYPE_TCP_CONNECT,
+                    make_scoped_refptr(new NetLogStringParameter(
+                        "source address",
+                        source_address_str)));
 }
 
 void TCPClientSocketLibevent::DoReadCallback(int rv) {
   DCHECK_NE(rv, ERR_IO_PENDING);
-  DCHECK(read_callback_);
+  DCHECK(!read_callback_.is_null());
 
   // since Run may result in Read being called, clear read_callback_ up front.
-  CompletionCallback* c = read_callback_;
-  read_callback_ = NULL;
-  c->Run(rv);
+  CompletionCallback c = read_callback_;
+  read_callback_.Reset();
+  c.Run(rv);
 }
 
 void TCPClientSocketLibevent::DoWriteCallback(int rv) {
   DCHECK_NE(rv, ERR_IO_PENDING);
-  DCHECK(write_callback_);
+  DCHECK(!write_callback_.is_null());
 
   // since Run may result in Write being called, clear write_callback_ up front.
-  CompletionCallback* c = write_callback_;
-  write_callback_ = NULL;
-  c->Run(rv);
+  CompletionCallback c = write_callback_;
+  write_callback_.Reset();
+  c.Run(rv);
 }
 
 void TCPClientSocketLibevent::DidCompleteConnect() {
@@ -579,14 +633,15 @@ void TCPClientSocketLibevent::DidCompleteRead() {
   int result;
   if (bytes_transferred >= 0) {
     result = bytes_transferred;
-    static base::StatsCounter read_bytes("tcp.read_bytes");
+    base::StatsCounter read_bytes("tcp.read_bytes");
     read_bytes.Add(bytes_transferred);
+    num_bytes_read_ += static_cast<int64>(bytes_transferred);
     if (bytes_transferred > 0)
       use_history_.set_was_used_to_convey_data();
-    LogByteTransfer(net_log_, NetLog::TYPE_SOCKET_BYTES_RECEIVED, result,
-                    read_buf_->data());
+    net_log_.AddByteTransferEvent(NetLog::TYPE_SOCKET_BYTES_RECEIVED, result,
+                                  read_buf_->data());
   } else {
-    result = MapPosixError(errno);
+    result = MapSystemError(errno);
   }
 
   if (result != ERR_IO_PENDING) {
@@ -606,14 +661,14 @@ void TCPClientSocketLibevent::DidCompleteWrite() {
   int result;
   if (bytes_transferred >= 0) {
     result = bytes_transferred;
-    static base::StatsCounter write_bytes("tcp.write_bytes");
+    base::StatsCounter write_bytes("tcp.write_bytes");
     write_bytes.Add(bytes_transferred);
     if (bytes_transferred > 0)
       use_history_.set_was_used_to_convey_data();
-    LogByteTransfer(net_log_, NetLog::TYPE_SOCKET_BYTES_SENT, result,
-                    write_buf_->data());
+    net_log_.AddByteTransferEvent(NetLog::TYPE_SOCKET_BYTES_SENT, result,
+                                  write_buf_->data());
   } else {
-    result = MapPosixError(errno);
+    result = MapSystemError(errno);
   }
 
   if (result != ERR_IO_PENDING) {
@@ -629,7 +684,24 @@ int TCPClientSocketLibevent::GetPeerAddress(AddressList* address) const {
   DCHECK(address);
   if (!IsConnected())
     return ERR_SOCKET_NOT_CONNECTED;
-  address->Copy(current_ai_, false);
+  *address = AddressList::CreateByCopyingFirstAddress(current_ai_);
+  return OK;
+}
+
+int TCPClientSocketLibevent::GetLocalAddress(IPEndPoint* address) const {
+  DCHECK(CalledOnValidThread());
+  DCHECK(address);
+  if (!IsConnected())
+    return ERR_SOCKET_NOT_CONNECTED;
+
+  struct sockaddr_storage addr_storage;
+  socklen_t addr_len = sizeof(addr_storage);
+  struct sockaddr* addr = reinterpret_cast<struct sockaddr*>(&addr_storage);
+  if (getsockname(socket_, addr, &addr_len))
+    return MapSystemError(errno);
+  if (!address->FromSockAddr(addr, addr_len))
+    return ERR_FAILED;
+
   return OK;
 }
 
@@ -653,4 +725,12 @@ bool TCPClientSocketLibevent::UsingTCPFastOpen() const {
   return use_tcp_fastopen_;
 }
 
+int64 TCPClientSocketLibevent::NumBytesRead() const {
+  return num_bytes_read_;
+}
+
+base::TimeDelta TCPClientSocketLibevent::GetConnectTimeMicros() const {
+  return connect_time_micros_;
+}
+
 }  // namespace net
