@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,23 +6,22 @@
 #define NET_SOCKET_TCP_CLIENT_SOCKET_LIBEVENT_H_
 #pragma once
 
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
 #include "base/threading/non_thread_safe.h"
 #include "net/base/address_list.h"
 #include "net/base/completion_callback.h"
 #include "net/base/net_log.h"
-#include "net/socket/client_socket.h"
-
-struct event;  // From libevent
+#include "net/socket/stream_socket.h"
 
 namespace net {
 
 class BoundNetLog;
 
 // A client socket that uses TCP as the transport layer.
-class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
+class NET_EXPORT_PRIVATE TCPClientSocketLibevent : public StreamSocket,
+                                                   public base::NonThreadSafe {
  public:
   // The IP address(es) and port number to connect to.  The TCP socket will try
   // each IP address in the list until it succeeds in establishing a
@@ -36,28 +35,39 @@ class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
   // AdoptSocket causes the given, connected socket to be adopted as a TCP
   // socket. This object must not be connected. This object takes ownership of
   // the given socket and then acts as if Connect() had been called. This
-  // function is intended for testing only.
-  void AdoptSocket(int socket);
+  // function is used by TCPServerSocket() to adopt accepted connections
+  // and for testing.
+  int AdoptSocket(int socket);
 
-  // ClientSocket methods:
-  virtual int Connect(CompletionCallback* callback);
-  virtual void Disconnect();
-  virtual bool IsConnected() const;
-  virtual bool IsConnectedAndIdle() const;
-  virtual int GetPeerAddress(AddressList* address) const;
-  virtual const BoundNetLog& NetLog() const;
-  virtual void SetSubresourceSpeculation();
-  virtual void SetOmniboxSpeculation();
-  virtual bool WasEverUsed() const;
-  virtual bool UsingTCPFastOpen() const;
+  // Binds the socket to a local IP address and port.
+  int Bind(const IPEndPoint& address);
 
-  // Socket methods:
+  // StreamSocket implementation.
+  virtual int Connect(const CompletionCallback& callback) OVERRIDE;
+  virtual void Disconnect() OVERRIDE;
+  virtual bool IsConnected() const OVERRIDE;
+  virtual bool IsConnectedAndIdle() const OVERRIDE;
+  virtual int GetPeerAddress(AddressList* address) const OVERRIDE;
+  virtual int GetLocalAddress(IPEndPoint* address) const OVERRIDE;
+  virtual const BoundNetLog& NetLog() const OVERRIDE;
+  virtual void SetSubresourceSpeculation() OVERRIDE;
+  virtual void SetOmniboxSpeculation() OVERRIDE;
+  virtual bool WasEverUsed() const OVERRIDE;
+  virtual bool UsingTCPFastOpen() const OVERRIDE;
+  virtual int64 NumBytesRead() const OVERRIDE;
+  virtual base::TimeDelta GetConnectTimeMicros() const OVERRIDE;
+
+  // Socket implementation.
   // Multiple outstanding requests are not supported.
   // Full duplex mode (reading and writing at the same time) is supported
-  virtual int Read(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual int Write(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual bool SetReceiveBufferSize(int32 size);
-  virtual bool SetSendBufferSize(int32 size);
+  virtual int Read(IOBuffer* buf,
+                   int buf_len,
+                   const CompletionCallback& callback) OVERRIDE;
+  virtual int Write(IOBuffer* buf,
+                    int buf_len,
+                    const CompletionCallback& callback) OVERRIDE;
+  virtual bool SetReceiveBufferSize(int32 size) OVERRIDE;
+  virtual bool SetSendBufferSize(int32 size) OVERRIDE;
 
  private:
   // State machine for connecting the socket.
@@ -73,12 +83,12 @@ class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
 
     // MessageLoopForIO::Watcher methods
 
-    virtual void OnFileCanReadWithoutBlocking(int /* fd */) {
-      if (socket_->read_callback_)
+    virtual void OnFileCanReadWithoutBlocking(int /* fd */) OVERRIDE {
+      if (!socket_->read_callback_.is_null())
         socket_->DidCompleteRead();
     }
 
-    virtual void OnFileCanWriteWithoutBlocking(int /* fd */) {}
+    virtual void OnFileCanWriteWithoutBlocking(int /* fd */) OVERRIDE {}
 
    private:
     TCPClientSocketLibevent* const socket_;
@@ -90,14 +100,12 @@ class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
    public:
     explicit WriteWatcher(TCPClientSocketLibevent* socket) : socket_(socket) {}
 
-    // MessageLoopForIO::Watcher methods
-
-    virtual void OnFileCanReadWithoutBlocking(int /* fd */) {}
-
-    virtual void OnFileCanWriteWithoutBlocking(int /* fd */) {
+    // MessageLoopForIO::Watcher implementation.
+    virtual void OnFileCanReadWithoutBlocking(int /* fd */) OVERRIDE {}
+    virtual void OnFileCanWriteWithoutBlocking(int /* fd */) OVERRIDE {
       if (socket_->waiting_connect()) {
         socket_->DidCompleteConnect();
-      } else if (socket_->write_callback_) {
+      } else if (!socket_->write_callback_.is_null()) {
         socket_->DidCompleteWrite();
       }
     }
@@ -128,12 +136,6 @@ class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
     return next_connect_state_ != CONNECT_STATE_NONE;
   }
 
-  // Returns the OS error code (or 0 on success).
-  int CreateSocket(const struct addrinfo* ai);
-
-  // Returns the OS error code (or 0 on success).
-  int SetupSocket();
-
   // Helper to add a TCP_CONNECT (end) event to the NetLog.
   void LogConnectCompletion(int net_error);
 
@@ -142,6 +144,13 @@ class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
 
   int socket_;
 
+  // Local IP address and port we are bound to. Set to NULL if Bind()
+  // was't called (in that cases OS chooses address/port).
+  scoped_ptr<IPEndPoint> bind_address_;
+
+  // Stores bound socket between Bind() and Connect() calls.
+  int bound_socket_;
+
   // The list of addresses we should try in order to establish a connection.
   AddressList addresses_;
 
@@ -165,10 +174,10 @@ class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
   int write_buf_len_;
 
   // External callback; called when read is complete.
-  CompletionCallback* read_callback_;
+  CompletionCallback read_callback_;
 
   // External callback; called when write is complete.
-  CompletionCallback* write_callback_;
+  CompletionCallback write_callback_;
 
   // The next state for the Connect() state machine.
   ConnectState next_connect_state_;
@@ -191,6 +200,10 @@ class TCPClientSocketLibevent : public ClientSocket, base::NonThreadSafe {
   // True when TCP FastOpen is in use and we have done the connect.
   bool tcp_fastopen_connected_;
 
+  base::TimeTicks connect_start_time_;
+  base::TimeDelta connect_time_micros_;
+  int64 num_bytes_read_;
+
   DISALLOW_COPY_AND_ASSIGN(TCPClientSocketLibevent);
 };
 
