@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,17 +8,19 @@
 
 #include <winsock2.h>
 
+#include "base/memory/scoped_ptr.h"
 #include "base/threading/non_thread_safe.h"
 #include "net/base/address_list.h"
 #include "net/base/completion_callback.h"
 #include "net/base/net_log.h"
-#include "net/socket/client_socket.h"
+#include "net/socket/stream_socket.h"
 
 namespace net {
 
 class BoundNetLog;
 
-class TCPClientSocketWin : public ClientSocket, base::NonThreadSafe {
+class NET_EXPORT TCPClientSocketWin : public StreamSocket,
+                                      NON_EXPORTED_BASE(base::NonThreadSafe) {
  public:
   // The IP address(es) and port number to connect to.  The TCP socket will try
   // each IP address in the list until it succeeds in establishing a
@@ -32,26 +34,35 @@ class TCPClientSocketWin : public ClientSocket, base::NonThreadSafe {
   // AdoptSocket causes the given, connected socket to be adopted as a TCP
   // socket. This object must not be connected. This object takes ownership of
   // the given socket and then acts as if Connect() had been called. This
-  // function is intended for testing only.
-  void AdoptSocket(SOCKET socket);
+  // function is used by TCPServerSocket() to adopt accepted connections
+  // and for testing.
+  int AdoptSocket(SOCKET socket);
 
-  // ClientSocket methods:
-  virtual int Connect(CompletionCallback* callback);
+  // Binds the socket to a local IP address and port.
+  int Bind(const IPEndPoint& address);
+
+  // StreamSocket implementation.
+  virtual int Connect(const CompletionCallback& callback);
   virtual void Disconnect();
   virtual bool IsConnected() const;
   virtual bool IsConnectedAndIdle() const;
   virtual int GetPeerAddress(AddressList* address) const;
+  virtual int GetLocalAddress(IPEndPoint* address) const;
   virtual const BoundNetLog& NetLog() const { return net_log_; }
   virtual void SetSubresourceSpeculation();
   virtual void SetOmniboxSpeculation();
   virtual bool WasEverUsed() const;
   virtual bool UsingTCPFastOpen() const;
+  virtual int64 NumBytesRead() const;
+  virtual base::TimeDelta GetConnectTimeMicros() const;
 
-  // Socket methods:
+  // Socket implementation.
   // Multiple outstanding requests are not supported.
   // Full duplex mode (reading and writing at the same time) is supported
-  virtual int Read(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual int Write(IOBuffer* buf, int buf_len, CompletionCallback* callback);
+  virtual int Read(IOBuffer* buf, int buf_len,
+                   const CompletionCallback& callback);
+  virtual int Write(IOBuffer* buf, int buf_len,
+                    const CompletionCallback& callback);
 
   virtual bool SetReceiveBufferSize(int32 size);
   virtual bool SetSendBufferSize(int32 size);
@@ -80,12 +91,6 @@ class TCPClientSocketWin : public ClientSocket, base::NonThreadSafe {
     return next_connect_state_ != CONNECT_STATE_NONE;
   }
 
-  // Returns the OS error code (or 0 on success).
-  int CreateSocket(const struct addrinfo* ai);
-
-  // Returns the OS error code (or 0 on success).
-  int SetupSocket();
-
   // Called after Connect() has completed with |net_error|.
   void LogConnectCompletion(int net_error);
 
@@ -97,6 +102,13 @@ class TCPClientSocketWin : public ClientSocket, base::NonThreadSafe {
 
   SOCKET socket_;
 
+  // Local IP address and port we are bound to. Set to NULL if Bind()
+  // was't called (in that cases OS chooses address/port).
+  scoped_ptr<IPEndPoint> bind_address_;
+
+  // Stores bound socket between Bind() and Connect() calls.
+  SOCKET bound_socket_;
+
   // The list of addresses we should try in order to establish a connection.
   AddressList addresses_;
 
@@ -113,10 +125,10 @@ class TCPClientSocketWin : public ClientSocket, base::NonThreadSafe {
   scoped_refptr<Core> core_;
 
   // External callback; called when connect or read is complete.
-  CompletionCallback* read_callback_;
+  CompletionCallback read_callback_;
 
   // External callback; called when write is complete.
-  CompletionCallback* write_callback_;
+  CompletionCallback write_callback_;
 
   // The next state for the Connect() state machine.
   ConnectState next_connect_state_;
@@ -133,6 +145,10 @@ class TCPClientSocketWin : public ClientSocket, base::NonThreadSafe {
   // histograms.
   UseHistory use_history_;
 
+  base::TimeTicks connect_start_time_;
+  base::TimeDelta connect_time_micros_;
+  int64 num_bytes_read_;
+
   DISALLOW_COPY_AND_ASSIGN(TCPClientSocketWin);
 };
 
