@@ -10,14 +10,13 @@
 #include <map>
 #include <string>
 
-#include "base/linked_ptr.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/string16.h"
-#include "base/task.h"
+#include "base/memory/linked_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "net/base/address_list.h"
 #include "net/base/completion_callback.h"
 #include "net/base/io_buffer.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/base/net_errors.h"
 #include "net/base/ssl_config_service.h"
@@ -32,6 +31,7 @@ namespace net {
 
 class AuthChallengeInfo;
 class ClientSocketFactory;
+class CookieOptions;
 class HostResolver;
 class HttpAuthHandlerFactory;
 class SSLConfigService;
@@ -44,7 +44,8 @@ class SocketStreamMetrics;
 // authentication identity for proxy URL first.  If server requires proxy
 // authentication, it will try authentication identity for realm that server
 // requests.
-class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
+class NET_EXPORT SocketStream
+    : public base::RefCountedThreadSafe<SocketStream> {
  public:
   // Derive from this class and add your own data members to associate extra
   // information with a SocketStream.  Use GetUserData(key) and
@@ -55,12 +56,12 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
     virtual ~UserData() {}
   };
 
-  class Delegate {
+  class NET_EXPORT Delegate {
    public:
     virtual ~Delegate() {}
 
     virtual int OnStartOpenConnection(SocketStream* socket,
-                                      CompletionCallback* callback) {
+                                      const CompletionCallback& callback) {
       return OK;
     }
 
@@ -95,6 +96,21 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
     // This is only for error reporting to the delegate.
     // |error| is net::Error.
     virtual void OnError(const SocketStream* socket, int error) {}
+
+    // Called when reading cookies to allow the delegate to block access to the
+    // cookie.
+    virtual bool CanGetCookies(SocketStream* socket, const GURL& url) {
+      return true;
+    }
+
+    // Called when a cookie is set to allow the delegate to block access to the
+    // cookie.
+    virtual bool CanSetCookie(SocketStream* request,
+                              const GURL& url,
+                              const std::string& cookie_line,
+                              CookieOptions* options) {
+      return true;
+    }
   };
 
   SocketStream(const GURL& url, Delegate* delegate);
@@ -133,15 +149,15 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
 
   // Restarts with authentication info.
   // Should be used for response of OnAuthRequired.
-  virtual void RestartWithAuth(
-      const string16& username,
-      const string16& password);
+  virtual void RestartWithAuth(const AuthCredentials& credentials);
 
   // Detach delegate.  Call before delegate is deleted.
   // Once delegate is detached, close the socket stream and never call delegate
   // back.
   virtual void DetachDelegate();
 
+  const ProxyServer& proxy_server() const;
+
   // Sets an alternative HostResolver. For testing purposes only.
   void SetHostResolver(HostResolver* host_resolver);
 
@@ -156,6 +172,9 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
   Delegate* delegate_;
 
  private:
+  FRIEND_TEST_ALL_PREFIXES(SocketStreamTest, IOPending);
+  FRIEND_TEST_ALL_PREFIXES(SocketStreamTest, SwitchAfterPending);
+
   friend class WebSocketThrottleTest;
 
   typedef std::map<const void*, linked_ptr<UserData> > UserDataMap;
@@ -172,7 +191,7 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
     std::string headers_;
 
     private:
-     ~RequestHeaders() { data_ = NULL; }
+     virtual ~RequestHeaders() { data_ = NULL; }
   };
 
   class ResponseHeaders : public IOBuffer {
@@ -185,7 +204,7 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
     void Realloc(size_t new_size);
 
    private:
-     ~ResponseHeaders();
+     virtual ~ResponseHeaders();
 
     scoped_ptr_malloc<char> headers_;
   };
@@ -196,6 +215,8 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
     STATE_RESOLVE_PROXY_COMPLETE,
     STATE_RESOLVE_HOST,
     STATE_RESOLVE_HOST_COMPLETE,
+    STATE_RESOLVE_PROTOCOL,
+    STATE_RESOLVE_PROTOCOL_COMPLETE,
     STATE_TCP_CONNECT,
     STATE_TCP_CONNECT_COMPLETE,
     STATE_WRITE_TUNNEL_HEADERS,
@@ -204,6 +225,8 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
     STATE_READ_TUNNEL_HEADERS_COMPLETE,
     STATE_SOCKS_CONNECT,
     STATE_SOCKS_CONNECT_COMPLETE,
+    STATE_SECURE_PROXY_CONNECT,
+    STATE_SECURE_PROXY_CONNECT_COMPLETE,
     STATE_SSL_CONNECT,
     STATE_SSL_CONNECT_COMPLETE,
     STATE_READ_WRITE,
@@ -231,6 +254,7 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
   // notifications will be sent to delegate.
   void Finish(int result);
 
+  int DidEstablishSSL(int result, SSLConfig* ssl_config);
   int DidEstablishConnection();
   int DidReceiveData(int result);
   int DidSendData(int result);
@@ -245,6 +269,8 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
   int DoResolveProxyComplete(int result);
   int DoResolveHost();
   int DoResolveHostComplete(int result);
+  int DoResolveProtocol(int result);
+  int DoResolveProtocolComplete(int result);
   int DoTcpConnect(int result);
   int DoTcpConnectComplete(int result);
   int DoWriteTunnelHeaders();
@@ -253,12 +279,15 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
   int DoReadTunnelHeadersComplete(int result);
   int DoSOCKSConnect();
   int DoSOCKSConnectComplete(int result);
+  int DoSecureProxyConnect();
+  int DoSecureProxyConnectComplete(int result);
   int DoSSLConnect();
   int DoSSLConnectComplete(int result);
   int DoReadWrite(int result);
 
   GURL ProxyAuthOrigin() const;
   int HandleAuthChallenge(const HttpResponseHeaders* headers);
+  int HandleCertificateRequest(int result);
   void DoAuthRequired();
   void DoRestartWithAuth();
 
@@ -278,6 +307,7 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
   State next_state_;
   HostResolver* host_resolver_;
   CertVerifier* cert_verifier_;
+  OriginBoundCertService* origin_bound_cert_service_;
   HttpAuthHandlerFactory* http_auth_handler_factory_;
   ClientSocketFactory* factory_;
 
@@ -300,13 +330,12 @@ class SocketStream : public base::RefCountedThreadSafe<SocketStream> {
 
   scoped_ptr<SingleRequestHostResolver> resolver_;
   AddressList addresses_;
-  scoped_ptr<ClientSocket> socket_;
+  scoped_ptr<StreamSocket> socket_;
 
-  SSLConfig ssl_config_;
+  SSLConfig server_ssl_config_;
+  SSLConfig proxy_ssl_config_;
 
-  CompletionCallbackImpl<SocketStream> io_callback_;
-  CompletionCallbackImpl<SocketStream> read_callback_;
-  CompletionCallbackImpl<SocketStream> write_callback_;
+  CompletionCallback io_callback_;
 
   scoped_refptr<IOBuffer> read_buf_;
   int read_buf_size_;
