@@ -1,11 +1,14 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/socket_stream/socket_stream_job.h"
 
-#include "base/singleton.h"
+#include "base/memory/singleton.h"
+#include "net/base/ssl_config_service.h"
+#include "net/base/transport_security_state.h"
 #include "net/socket_stream/socket_stream_job_manager.h"
+#include "net/url_request/url_request_context.h"
 
 namespace net {
 
@@ -18,8 +21,22 @@ SocketStreamJob::ProtocolFactory* SocketStreamJob::RegisterProtocolFactory(
 
 // static
 SocketStreamJob* SocketStreamJob::CreateSocketStreamJob(
-    const GURL& url, SocketStream::Delegate* delegate) {
-  return SocketStreamJobManager::GetInstance()->CreateJob(url, delegate);
+    const GURL& url,
+    SocketStream::Delegate* delegate,
+    TransportSecurityState* sts,
+    SSLConfigService* ssl) {
+  GURL socket_url(url);
+  TransportSecurityState::DomainState domain_state;
+  if (url.scheme() == "ws" && sts && sts->GetDomainState(
+          &domain_state, url.host(), SSLConfigService::IsSNIAvailable(ssl)) &&
+      domain_state.ShouldRedirectHTTPToHTTPS()) {
+    url_canon::Replacements<char> replacements;
+    static const char kNewScheme[] = "wss";
+    replacements.SetScheme(kNewScheme,
+                           url_parse::Component(0, strlen(kNewScheme)));
+    socket_url = url.ReplaceComponents(replacements);
+  }
+  return SocketStreamJobManager::GetInstance()->CreateJob(socket_url, delegate);
 }
 
 SocketStreamJob::SocketStreamJob() {}
@@ -45,9 +62,8 @@ void SocketStreamJob::Close() {
   socket_->Close();
 }
 
-void SocketStreamJob::RestartWithAuth(const string16& username,
-                                      const string16& password) {
-  socket_->RestartWithAuth(username, password);
+void SocketStreamJob::RestartWithAuth(const AuthCredentials& credentials) {
+  socket_->RestartWithAuth(credentials);
 }
 
 void SocketStreamJob::DetachDelegate() {
