@@ -8,21 +8,25 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
-#include "base/string16.h"
+#include "base/memory/ref_counted.h"
+#include "net/base/net_export.h"
 #include "net/socket_stream/socket_stream.h"
 
 class GURL;
 
 namespace net {
 
+class SSLConfigService;
+class TransportSecurityState;
+
 // SocketStreamJob represents full-duplex communication over SocketStream.
 // If a protocol (e.g. WebSocket protocol) needs to inspect/modify data
 // over SocketStream, you can implement protocol specific job (e.g.
 // WebSocketJob) to do some work on data over SocketStream.
 // Registers the protocol specific SocketStreamJob by RegisterProtocolFactory
 // and call CreateSocketStreamJob to create SocketStreamJob for the URL.
-class SocketStreamJob : public base::RefCountedThreadSafe<SocketStreamJob> {
+class NET_EXPORT SocketStreamJob
+    : public base::RefCountedThreadSafe<SocketStreamJob> {
  public:
   // Callback function implemented by protocol handlers to create new jobs.
   typedef SocketStreamJob* (ProtocolFactory)(const GURL& url,
@@ -32,7 +36,10 @@ class SocketStreamJob : public base::RefCountedThreadSafe<SocketStreamJob> {
                                                   ProtocolFactory* factory);
 
   static SocketStreamJob* CreateSocketStreamJob(
-      const GURL& url, SocketStream::Delegate* delegate);
+      const GURL& url,
+      SocketStream::Delegate* delegate,
+      TransportSecurityState* sts,
+      SSLConfigService* ssl);
 
   SocketStreamJob();
   void InitSocketStream(SocketStream* socket) {
@@ -55,12 +62,12 @@ class SocketStreamJob : public base::RefCountedThreadSafe<SocketStreamJob> {
 
   virtual void Close();
 
-  virtual void RestartWithAuth(const string16& username,
-                               const string16& password);
+  virtual void RestartWithAuth(const AuthCredentials& credentials);
 
   virtual void DetachDelegate();
 
  protected:
+  friend class WebSocketJobTest;
   friend class base::RefCountedThreadSafe<SocketStreamJob>;
   virtual ~SocketStreamJob();
 
