@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,8 +10,8 @@
 #include <string>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
-#include "base/task.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/weak_ptr.h"
 #include "net/base/completion_callback.h"
 #include "net/base/net_log.h"
 #include "net/http/http_request_info.h"
@@ -29,7 +29,8 @@ class UploadData;
 class UploadDataStream;
 
 // The SpdyHttpStream is a HTTP-specific type of stream known to a SpdySession.
-class SpdyHttpStream : public SpdyStream::Delegate, public HttpStream {
+class NET_EXPORT_PRIVATE SpdyHttpStream : public SpdyStream::Delegate,
+                                          public HttpStream {
  public:
   SpdyHttpStream(SpdySession* spdy_session, bool direct);
   virtual ~SpdyHttpStream();
@@ -42,40 +43,46 @@ class SpdyHttpStream : public SpdyStream::Delegate, public HttpStream {
   // Cancels any callbacks from being invoked and deletes the stream.
   void Cancel();
 
-  // HttpStream methods:
+  // HttpStream implementation.
   virtual int InitializeStream(const HttpRequestInfo* request_info,
                                const BoundNetLog& net_log,
-                               CompletionCallback* callback);
+                               const CompletionCallback& callback) OVERRIDE;
   virtual int SendRequest(const HttpRequestHeaders& headers,
                           UploadDataStream* request_body,
                           HttpResponseInfo* response,
-                          CompletionCallback* callback);
-  virtual uint64 GetUploadProgress() const;
-  virtual int ReadResponseHeaders(CompletionCallback* callback);
-  virtual const HttpResponseInfo* GetResponseInfo() const;
+                          const CompletionCallback& callback) OVERRIDE;
+  virtual uint64 GetUploadProgress() const OVERRIDE;
+  virtual int ReadResponseHeaders(const CompletionCallback& callback) OVERRIDE;
+  virtual const HttpResponseInfo* GetResponseInfo() const OVERRIDE;
   virtual int ReadResponseBody(IOBuffer* buf,
                                int buf_len,
-                               CompletionCallback* callback);
-  virtual void Close(bool not_reusable);
-  virtual HttpStream* RenewStreamForAuth();
-  virtual bool IsResponseBodyComplete() const;
-  virtual bool CanFindEndOfResponse() const;
-  virtual bool IsMoreDataBuffered() const;
-  virtual bool IsConnectionReused() const;
-  virtual void SetConnectionReused();
-  virtual void GetSSLInfo(SSLInfo* ssl_info);
-  virtual void GetSSLCertRequestInfo(SSLCertRequestInfo* cert_request_info);
+                               const CompletionCallback& callback) OVERRIDE;
+  virtual void Close(bool not_reusable) OVERRIDE;
+  virtual HttpStream* RenewStreamForAuth() OVERRIDE;
+  virtual bool IsResponseBodyComplete() const OVERRIDE;
+  virtual bool CanFindEndOfResponse() const OVERRIDE;
+  virtual bool IsMoreDataBuffered() const OVERRIDE;
+  virtual bool IsConnectionReused() const OVERRIDE;
+  virtual void SetConnectionReused() OVERRIDE;
+  virtual bool IsConnectionReusable() const OVERRIDE;
+  virtual void GetSSLInfo(SSLInfo* ssl_info) OVERRIDE;
+  virtual void GetSSLCertRequestInfo(
+      SSLCertRequestInfo* cert_request_info) OVERRIDE;
+  virtual bool IsSpdyHttpStream() const OVERRIDE;
+  virtual void LogNumRttVsBytesMetrics() const OVERRIDE {}
+  virtual void Drain(HttpNetworkSession* session) OVERRIDE;
 
-  // SpdyStream::Delegate methods:
-  virtual bool OnSendHeadersComplete(int status);
-  virtual int OnSendBody();
-  virtual bool OnSendBodyComplete(int status);
+  // SpdyStream::Delegate implementation.
+  virtual bool OnSendHeadersComplete(int status) OVERRIDE;
+  virtual int OnSendBody() OVERRIDE;
+  virtual int OnSendBodyComplete(int status, bool* eof) OVERRIDE;
   virtual int OnResponseReceived(const spdy::SpdyHeaderBlock& response,
                                  base::Time response_time,
-                                 int status);
-  virtual void OnDataReceived(const char* buffer, int bytes);
-  virtual void OnDataSent(int length);
-  virtual void OnClose(int status);
+                                 int status) OVERRIDE;
+  virtual void OnDataReceived(const char* buffer, int bytes) OVERRIDE;
+  virtual void OnDataSent(int length) OVERRIDE;
+  virtual void OnClose(int status) OVERRIDE;
+  virtual void set_chunk_callback(ChunkCallback* callback) OVERRIDE;
 
  private:
   FRIEND_TEST_ALL_PREFIXES(SpdyNetworkTransactionTest, FlowControlStallResume);
@@ -89,7 +96,7 @@ class SpdyHttpStream : public SpdyStream::Delegate, public HttpStream {
   bool DoBufferedReadCallback();
   bool ShouldWaitForMoreBufferedData() const;
 
-  ScopedRunnableMethodFactory<SpdyHttpStream> read_callback_factory_;
+  base::WeakPtrFactory<SpdyHttpStream> weak_factory_;
   scoped_refptr<SpdyStream> stream_;
   scoped_refptr<SpdySession> spdy_session_;
 
@@ -112,7 +119,7 @@ class SpdyHttpStream : public SpdyStream::Delegate, public HttpStream {
   // TODO(mbelshe):  is this infinite buffering?
   std::list<scoped_refptr<IOBufferWithSize> > response_body_;
 
-  CompletionCallback* user_callback_;
+  CompletionCallback callback_;
 
   // User provided buffer for the ReadResponseBody() response.
   scoped_refptr<IOBuffer> user_buffer_;
@@ -127,6 +134,8 @@ class SpdyHttpStream : public SpdyStream::Delegate, public HttpStream {
   // Is this spdy stream direct to the origin server (or to a proxy).
   bool direct_;
 
+  bool send_last_chunk_;
+
   DISALLOW_COPY_AND_ASSIGN(SpdyHttpStream);
 };
 
