@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,7 +10,8 @@
 #include <list>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/weak_ptr.h"
 #include "net/base/completion_callback.h"
 #include "net/base/host_port_pair.h"
 #include "net/base/net_log.h"
@@ -30,14 +31,12 @@ class GURL;
 namespace net {
 
 class AddressList;
-class ClientSocketHandle;
 class HttpStream;
 class IOBuffer;
-class SpdySession;
 class SpdyStream;
 
-class SpdyProxyClientSocket : public ProxyClientSocket,
-                              public SpdyStream::Delegate {
+class NET_EXPORT_PRIVATE SpdyProxyClientSocket : public ProxyClientSocket,
+                                                 public SpdyStream::Delegate {
  public:
   // Create a socket on top of the |spdy_stream| by sending a SYN_STREAM
   // CONNECT frame for |endpoint|.  After the SYN_REPLY is received,
@@ -60,41 +59,49 @@ class SpdyProxyClientSocket : public ProxyClientSocket,
   }
 
   // ProxyClientSocket methods:
-  virtual const HttpResponseInfo* GetConnectResponseInfo() const;
+  virtual const HttpResponseInfo* GetConnectResponseInfo() const OVERRIDE;
 
   // In the event of a non-200 response to the CONNECT request, this
   // method may be called to return an HttpStream in order to read
   // the response body.
-  virtual HttpStream* CreateConnectResponseStream();
+  virtual HttpStream* CreateConnectResponseStream() OVERRIDE;
 
-  // ClientSocket methods:
-  virtual int Connect(CompletionCallback* callback);
-  virtual void Disconnect();
-  virtual bool IsConnected() const;
-  virtual bool IsConnectedAndIdle() const;
-  virtual const BoundNetLog& NetLog() const;
-  virtual void SetSubresourceSpeculation();
-  virtual void SetOmniboxSpeculation();
-  virtual bool WasEverUsed() const;
-  virtual bool UsingTCPFastOpen() const;
+  // StreamSocket implementation.
+  virtual int Connect(const CompletionCallback& callback) OVERRIDE;
+  virtual void Disconnect() OVERRIDE;
+  virtual bool IsConnected() const OVERRIDE;
+  virtual bool IsConnectedAndIdle() const OVERRIDE;
+  virtual const BoundNetLog& NetLog() const OVERRIDE;
+  virtual void SetSubresourceSpeculation() OVERRIDE;
+  virtual void SetOmniboxSpeculation() OVERRIDE;
+  virtual bool WasEverUsed() const OVERRIDE;
+  virtual bool UsingTCPFastOpen() const OVERRIDE;
+  virtual int64 NumBytesRead() const OVERRIDE;
+  virtual base::TimeDelta GetConnectTimeMicros() const OVERRIDE;
 
-  // Socket methods:
-  virtual int Read(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual int Write(IOBuffer* buf, int buf_len, CompletionCallback* callback);
-  virtual bool SetReceiveBufferSize(int32 size);
-  virtual bool SetSendBufferSize(int32 size);
-  virtual int GetPeerAddress(AddressList* address) const;
+  // Socket implementation.
+  virtual int Read(IOBuffer* buf,
+                   int buf_len,
+                   const CompletionCallback& callback) OVERRIDE;
+  virtual int Write(IOBuffer* buf,
+                    int buf_len,
+                    const CompletionCallback& callback) OVERRIDE;
+  virtual bool SetReceiveBufferSize(int32 size) OVERRIDE;
+  virtual bool SetSendBufferSize(int32 size) OVERRIDE;
+  virtual int GetPeerAddress(AddressList* address) const OVERRIDE;
+  virtual int GetLocalAddress(IPEndPoint* address) const OVERRIDE;
 
-  // SpdyStream::Delegate methods:
-  virtual bool OnSendHeadersComplete(int status);
-  virtual int OnSendBody();
-  virtual bool OnSendBodyComplete(int status);
+  // SpdyStream::Delegate implementation.
+  virtual bool OnSendHeadersComplete(int status) OVERRIDE;
+  virtual int OnSendBody() OVERRIDE;
+  virtual int OnSendBodyComplete(int status, bool* eof) OVERRIDE;
   virtual int OnResponseReceived(const spdy::SpdyHeaderBlock& response,
                                  base::Time response_time,
-                                 int status);
-  virtual void OnDataReceived(const char* data, int length);
-  virtual void OnDataSent(int length);
-  virtual void OnClose(int status);
+                                 int status) OVERRIDE;
+  virtual void OnDataReceived(const char* data, int length) OVERRIDE;
+  virtual void OnDataSent(int length) OVERRIDE;
+  virtual void OnClose(int status) OVERRIDE;
+  virtual void set_chunk_callback(ChunkCallback* /*callback*/) OVERRIDE;
 
  private:
   enum State {
@@ -121,7 +128,6 @@ class SpdyProxyClientSocket : public ProxyClientSocket,
   // and returns the number of bytes read.
   int PopulateUserReadBuffer();
 
-  CompletionCallbackImpl<SpdyProxyClientSocket> io_callback_;
   State next_state_;
 
   // Pointer to the SPDY Stream that this sits on top of.
@@ -129,9 +135,9 @@ class SpdyProxyClientSocket : public ProxyClientSocket,
 
   // Stores the callback to the layer above, called on completing Read() or
   // Connect().
-  CompletionCallback* read_callback_;
+  CompletionCallback read_callback_;
   // Stores the callback to the layer above, called on completing Write().
-  CompletionCallback* write_callback_;
+  CompletionCallback write_callback_;
 
   // CONNECT request and response.
   HttpRequestInfo request_;
@@ -153,13 +159,13 @@ class SpdyProxyClientSocket : public ProxyClientSocket,
   // Number of bytes written which have not been confirmed
   int write_bytes_outstanding_;
 
-  // True if read has ever returned zero for eof.
-  bool eof_has_been_read_;
   // True if the transport socket has ever sent data.
   bool was_ever_used_;
 
   scoped_ptr<SpdyHttpStream> response_stream_;
 
+  base::WeakPtrFactory<SpdyProxyClientSocket> weak_factory_;
+
   const BoundNetLog net_log_;
 
   DISALLOW_COPY_AND_ASSIGN(SpdyProxyClientSocket);
