@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -18,7 +18,7 @@ const spdy::SpdySettings& SpdySettingsStorage::Get(
     const HostPortPair& host_port_pair) const {
   SettingsMap::const_iterator it = settings_map_.find(host_port_pair);
   if (it == settings_map_.end()) {
-    static const spdy::SpdySettings kEmpty;
+    CR_DEFINE_STATIC_LOCAL(spdy::SpdySettings, kEmpty, ());
     return kEmpty;
   }
   return it->second;
@@ -46,5 +46,9 @@ void SpdySettingsStorage::Set(const HostPortPair& host_port_pair,
   settings_map_[host_port_pair] = persistent_settings;
 }
 
+void SpdySettingsStorage::Clear() {
+  settings_map_.clear();
+}
+
 }  // namespace net
 
