@@ -1,9 +1,10 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/spdy/spdy_stream.h"
 
+#include "base/bind.h"
 #include "base/logging.h"
 #include "base/message_loop.h"
 #include "base/values.h"
@@ -71,8 +72,7 @@ void SpdyStream::SetDelegate(Delegate* delegate) {
   if (pushed_) {
     CHECK(response_received());
     MessageLoop::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this,
-                                     &SpdyStream::PushedStreamReplayData));
+        FROM_HERE, base::Bind(&SpdyStream::PushedStreamReplayData, this));
   } else {
     continue_buffering_data_ = false;
   }
@@ -112,6 +112,8 @@ void SpdyStream::PushedStreamReplayData() {
 }
 
 void SpdyStream::DetachDelegate() {
+  if (delegate_)
+    delegate_->set_chunk_callback(NULL);
   delegate_ = NULL;
   if (!closed())
     Cancel();
@@ -126,6 +128,10 @@ void SpdyStream::set_spdy_headers(
   request_ = headers;
 }
 
+void SpdyStream::AdjustSendWindowSize(int delta_window_size) {
+  send_window_size_ = send_window_size_ + delta_window_size;
+}
+
 void SpdyStream::IncreaseSendWindowSize(int delta_window_size) {
   DCHECK_GE(delta_window_size, 1);
   int new_window_size = send_window_size_ + delta_window_size;
@@ -155,7 +161,7 @@ void SpdyStream::IncreaseSendWindowSize(int delta_window_size) {
       NetLog::TYPE_SPDY_STREAM_SEND_WINDOW_UPDATE,
       make_scoped_refptr(new NetLogSpdyStreamWindowUpdateParameter(
           stream_id_, delta_window_size, send_window_size_)));
-  if (stalled_by_flow_control_) {
+  if (send_window_size_ > 0 && stalled_by_flow_control_) {
     stalled_by_flow_control_ = false;
     io_state_ = STATE_SEND_BODY;
     DoLoop(OK);
@@ -216,6 +222,10 @@ int SpdyStream::GetPeerAddress(AddressList* address) const {
   return session_->GetPeerAddress(address);
 }
 
+int SpdyStream::GetLocalAddress(IPEndPoint* address) const {
+  return session_->GetLocalAddress(address);
+}
+
 bool SpdyStream::WasEverUsed() const {
   return session_->WasEverUsed();
 }
@@ -319,7 +329,7 @@ void SpdyStream::OnDataReceived(const char* data, int length) {
     return;
   }
 
-  if (session_->flow_control())
+  if (session_->is_flow_control_enabled())
     DecreaseRecvWindowSize(length);
 
   // Track our bandwidth.
@@ -348,13 +358,22 @@ void SpdyStream::OnWriteComplete(int bytes) {
   DoLoop(bytes);
 }
 
+void SpdyStream::OnChunkAvailable() {
+  DCHECK(io_state_ == STATE_SEND_HEADERS || io_state_ == STATE_SEND_BODY ||
+         io_state_ == STATE_SEND_BODY_COMPLETE);
+  if (io_state_ == STATE_SEND_BODY)
+    OnWriteComplete(0);
+}
+
 void SpdyStream::OnClose(int status) {
   io_state_ = STATE_DONE;
   response_status_ = status;
   Delegate* delegate = delegate_;
   delegate_ = NULL;
-  if (delegate)
+  if (delegate) {
+    delegate->set_chunk_callback(NULL);
     delegate->OnClose(status);
+  }
 }
 
 void SpdyStream::Cancel() {
@@ -366,7 +385,14 @@ void SpdyStream::Cancel() {
     session_->ResetStream(stream_id_, spdy::CANCEL);
 }
 
+void SpdyStream::Close() {
+  session_->CloseStream(stream_id_, net::OK);
+}
+
 int SpdyStream::SendRequest(bool has_upload_data) {
+  if (delegate_)
+    delegate_->set_chunk_callback(this);
+
   // Pushed streams do not send any data, and should always be in STATE_OPEN or
   // STATE_DONE. However, we still want to return IO_PENDING to mimic non-push
   // behavior.
@@ -387,8 +413,11 @@ int SpdyStream::WriteStreamData(IOBuffer* data, int length,
   return session_->WriteStreamData(stream_id_, data, length, flags);
 }
 
-bool SpdyStream::GetSSLInfo(SSLInfo* ssl_info, bool* was_npn_negotiated) {
-  return session_->GetSSLInfo(ssl_info, was_npn_negotiated);
+bool SpdyStream::GetSSLInfo(SSLInfo* ssl_info,
+                            bool* was_npn_negotiated,
+                            SSLClientSocket::NextProto* protocol_negotiated) {
+  return session_->GetSSLInfo(
+      ssl_info, was_npn_negotiated, protocol_negotiated);
 }
 
 bool SpdyStream::GetSSLCertRequestInfo(SSLCertRequestInfo* cert_request_info) {
@@ -545,17 +574,17 @@ int SpdyStream::DoSendBodyComplete(int result) {
   if (result < 0)
     return result;
 
-  CHECK_NE(result, 0);
-
   if (!delegate_)
     return ERR_UNEXPECTED;
 
-  if (!delegate_->OnSendBodyComplete(result))
+  bool eof = false;
+  result = delegate_->OnSendBodyComplete(result, &eof);
+  if (!eof)
     io_state_ = STATE_SEND_BODY;
   else
     io_state_ = STATE_WAITING_FOR_RESPONSE;
 
-  return OK;
+  return result;
 }
 
 int SpdyStream::DoOpen(int result) {
