@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,19 +10,23 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/linked_ptr.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/linked_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/bandwidth_metrics.h"
 #include "net/base/io_buffer.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
+#include "net/base/upload_data.h"
+#include "net/socket/ssl_client_socket.h"
 #include "net/spdy/spdy_framer.h"
 #include "net/spdy/spdy_protocol.h"
 
 namespace net {
 
 class AddressList;
+class IPEndPoint;
 class SpdySession;
 class SSLCertRequestInfo;
 class SSLInfo;
@@ -34,10 +38,12 @@ class SSLInfo;
 // a SpdyNetworkTransaction) will maintain a reference to the stream.  When
 // initiated by the server, only the SpdySession will maintain any reference,
 // until such a time as a client object requests a stream for the path.
-class SpdyStream : public base::RefCounted<SpdyStream> {
+class NET_EXPORT_PRIVATE SpdyStream
+    : public base::RefCounted<SpdyStream>,
+      public ChunkCallback {
  public:
   // Delegate handles protocol specific behavior of spdy stream.
-  class Delegate {
+  class NET_EXPORT_PRIVATE Delegate {
    public:
     Delegate() {}
 
@@ -50,9 +56,10 @@ class SpdyStream : public base::RefCounted<SpdyStream> {
     virtual int OnSendBody() = 0;
 
     // Called when data has been sent. |status| indicates network error
-    // or number of bytes has been sent.
-    // Returns true if no more data to be sent.
-    virtual bool OnSendBodyComplete(int status) = 0;
+    // or number of bytes that has been sent. On return, |eof| is set to true
+    // if no more data is available to send in the request body.
+    // Returns network error code. OK when it successfully sent data.
+    virtual int OnSendBodyComplete(int status, bool* eof) = 0;
 
     // Called when the SYN_STREAM, SYN_REPLY, or HEADERS frames are received.
     // Normal streams will receive a SYN_REPLY and optional HEADERS frames.
@@ -73,6 +80,9 @@ class SpdyStream : public base::RefCounted<SpdyStream> {
     // Called when SpdyStream is closed.
     virtual void OnClose(int status) = 0;
 
+    // Sets the callback to be invoked when a new chunk is available to upload.
+    virtual void set_chunk_callback(ChunkCallback* callback) = 0;
+
    protected:
     friend class base::RefCounted<Delegate>;
     virtual ~Delegate() {}
@@ -127,6 +137,9 @@ class SpdyStream : public base::RefCounted<SpdyStream> {
     stalled_by_flow_control_ = stalled;
   }
 
+  // Adjust the |send_window_size_| by |delta_window_size|.
+  void AdjustSendWindowSize(int delta_window_size);
+
   // Increases |send_window_size_| with delta extracted from a WINDOW_UPDATE
   // frame; sends a RST_STREAM if delta overflows |send_window_size_| and
   // removes the stream from the session.
@@ -136,6 +149,7 @@ class SpdyStream : public base::RefCounted<SpdyStream> {
   void DecreaseSendWindowSize(int delta_window_size);
 
   int GetPeerAddress(AddressList* address) const;
+  int GetLocalAddress(IPEndPoint* address) const;
 
   // Returns true if the underlying transport socket ever had any reads or
   // writes.
@@ -189,6 +203,7 @@ class SpdyStream : public base::RefCounted<SpdyStream> {
   void OnClose(int status);
 
   void Cancel();
+  void Close();
   bool cancelled() const { return cancelled_; }
   bool closed() const { return io_state_ == STATE_DONE; }
 
@@ -203,7 +218,9 @@ class SpdyStream : public base::RefCounted<SpdyStream> {
                       spdy::SpdyDataFlags flags);
 
   // Fills SSL info in |ssl_info| and returns true when SSL is in use.
-  bool GetSSLInfo(SSLInfo* ssl_info, bool* was_npn_negotiated);
+  bool GetSSLInfo(SSLInfo* ssl_info,
+                  bool* was_npn_negotiated,
+                  SSLClientSocket::NextProto* protocol_negotiated);
 
   // Fills SSL Certificate Request info |cert_request_info| and returns
   // true when SSL is in use.
@@ -222,6 +239,9 @@ class SpdyStream : public base::RefCounted<SpdyStream> {
   // true.
   GURL GetUrl() const;
 
+  // ChunkCallback methods.
+  virtual void OnChunkAvailable() OVERRIDE;
+
  private:
   enum State {
     STATE_NONE,
