@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,14 @@
 #include <string>
 
 #include "base/basictypes.h"
+#include "base/compiler_specific.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
+#include "net/http/http_network_session.h"
 #include "net/http/http_network_transaction.h"
+#include "net/http/http_server_properties_impl.h"
 #include "net/spdy/spdy_framer.h"
+#include "net/spdy/spdy_http_utils.h"
 
 namespace net {
 
@@ -35,7 +39,7 @@ MockWrite* ChopWriteFrame(const char* data, int length, int num_chunks) {
 // |num_chunks| is the number of chunks to create.
 MockWrite* ChopWriteFrame(const spdy::SpdyFrame& frame, int num_chunks) {
   return ChopWriteFrame(frame.data(),
-                        frame.length() + spdy::SpdyFrame::size(),
+                        frame.length() + spdy::SpdyFrame::kHeaderSize,
                         num_chunks);
 }
 
@@ -60,7 +64,7 @@ MockRead* ChopReadFrame(const char* data, int length, int num_chunks) {
 // |num_chunks| is the number of chunks to create.
 MockRead* ChopReadFrame(const spdy::SpdyFrame& frame, int num_chunks) {
   return ChopReadFrame(frame.data(),
-                       frame.length() + spdy::SpdyFrame::size(),
+                       frame.length() + spdy::SpdyFrame::kHeaderSize,
                        num_chunks);
 }
 
@@ -192,6 +196,13 @@ spdy::SpdyFrame* ConstructSpdySettings(spdy::SpdySettings settings) {
   return framer.CreateSettings(settings);
 }
 
+// Construct a SPDY PING frame.
+// Returns the constructed frame.  The caller takes ownership of the frame.
+spdy::SpdyFrame* ConstructSpdyPing() {
+  spdy::SpdyFramer framer;
+  return framer.CreatePingFrame(1);
+}
+
 // Construct a SPDY GOAWAY frame.
 // Returns the constructed frame.  The caller takes ownership of the frame.
 spdy::SpdyFrame* ConstructSpdyGoAway() {
@@ -290,7 +301,7 @@ spdy::SpdyFrame* ConstructSpdyControlFrame(const char* const extra_headers[],
     type,                         // Kind = Syn
     stream_id,                    // Stream ID
     associated_stream_id,         // Associated stream ID
-    net::ConvertRequestPriorityToSpdyPriority(request_priority),
+    ConvertRequestPriorityToSpdyPriority(request_priority),
                                   // Priority
     flags,                        // Control Flags
     compressed,                   // Compressed
@@ -670,6 +681,35 @@ spdy::SpdyFrame* ConstructSpdyPost(int64 content_length,
                                    arraysize(post_headers));
 }
 
+// Constructs a chunked transfer SPDY POST SYN packet.
+// |extra_headers| are the extra header-value pairs, which typically
+// will vary the most between calls.
+// Returns a SpdyFrame.
+spdy::SpdyFrame* ConstructChunkedSpdyPost(const char* const extra_headers[],
+                                          int extra_header_count) {
+  const char* post_headers[] = {
+    "method",
+    "POST",
+    "url",
+    "/",
+    "host",
+    "www.google.com",
+    "scheme",
+    "http",
+    "version",
+    "HTTP/1.1"
+  };
+  return ConstructSpdyControlFrame(extra_headers,
+                                   extra_header_count,
+                                   false,
+                                   1,
+                                   LOWEST,
+                                   spdy::SYN_STREAM,
+                                   spdy::CONTROL_FLAG_NONE,
+                                   post_headers,
+                                   arraysize(post_headers));
+}
+
 // Constructs a standard SPDY SYN_REPLY packet to match the SPDY POST.
 // |extra_headers| are the extra header-value pairs, which typically
 // will vary the most between calls.
@@ -718,7 +758,7 @@ spdy::SpdyFrame* ConstructWrappedSpdyFrame(
     const scoped_ptr<spdy::SpdyFrame>& frame,
     int stream_id) {
   return ConstructSpdyBodyFrame(stream_id, frame->data(),
-                                frame->length() + spdy::SpdyFrame::size(),
+                                frame->length() + spdy::SpdyFrame::kHeaderSize,
                                 false);
 }
 
@@ -732,7 +772,6 @@ int ConstructSpdyReplyString(const char* const extra_headers[],
                              char* buffer,
                              int buffer_length) {
   int packet_size = 0;
-  int header_count = 0;
   char* buffer_write = buffer;
   int buffer_left = buffer_length;
   spdy::SpdyHeaderBlock headers;
@@ -740,7 +779,6 @@ int ConstructSpdyReplyString(const char* const extra_headers[],
     return 0;
   // Copy in the extra headers.
   AppendHeadersToSpdyFrame(extra_headers, extra_header_count, &headers);
-  header_count = headers.size();
   // The iterator gets us the list of header/value pairs in sorted order.
   spdy::SpdyHeaderBlock::iterator next = headers.begin();
   spdy::SpdyHeaderBlock::iterator last = headers.end();
@@ -803,7 +841,7 @@ int ConstructSpdyReplyString(const char* const extra_headers[],
 // Create a MockWrite from the given SpdyFrame.
 MockWrite CreateMockWrite(const spdy::SpdyFrame& req) {
   return MockWrite(
-      true, req.data(), req.length() + spdy::SpdyFrame::size());
+      true, req.data(), req.length() + spdy::SpdyFrame::kHeaderSize);
 }
 
 // Create a MockWrite from the given SpdyFrame and sequence number.
@@ -814,13 +852,13 @@ MockWrite CreateMockWrite(const spdy::SpdyFrame& req, int seq) {
 // Create a MockWrite from the given SpdyFrame and sequence number.
 MockWrite CreateMockWrite(const spdy::SpdyFrame& req, int seq, bool async) {
   return MockWrite(
-      async, req.data(), req.length() + spdy::SpdyFrame::size(), seq);
+      async, req.data(), req.length() + spdy::SpdyFrame::kHeaderSize, seq);
 }
 
 // Create a MockRead from the given SpdyFrame.
 MockRead CreateMockRead(const spdy::SpdyFrame& resp) {
   return MockRead(
-      true, resp.data(), resp.length() + spdy::SpdyFrame::size());
+      true, resp.data(), resp.length() + spdy::SpdyFrame::kHeaderSize);
 }
 
 // Create a MockRead from the given SpdyFrame and sequence number.
@@ -831,7 +869,7 @@ MockRead CreateMockRead(const spdy::SpdyFrame& resp, int seq) {
 // Create a MockRead from the given SpdyFrame and sequence number.
 MockRead CreateMockRead(const spdy::SpdyFrame& resp, int seq, bool async) {
   return MockRead(
-      async, resp.data(), resp.length() + spdy::SpdyFrame::size(), seq);
+      async, resp.data(), resp.length() + spdy::SpdyFrame::kHeaderSize, seq);
 }
 
 // Combines the given SpdyFrames into the given char array and returns
@@ -840,12 +878,12 @@ int CombineFrames(const spdy::SpdyFrame** frames, int num_frames,
                   char* buff, int buff_len) {
   int total_len = 0;
   for (int i = 0; i < num_frames; ++i) {
-    total_len += frames[i]->length() + spdy::SpdyFrame::size();
+    total_len += frames[i]->length() + spdy::SpdyFrame::kHeaderSize;
   }
   DCHECK_LE(total_len, buff_len);
   char* ptr = buff;
   for (int i = 0; i < num_frames; ++i) {
-    int len = frames[i]->length() + spdy::SpdyFrame::size();
+    int len = frames[i]->length() + spdy::SpdyFrame::kHeaderSize;
     memcpy(ptr, frames[i]->data(), len);
     ptr += len;
   }
@@ -853,7 +891,7 @@ int CombineFrames(const spdy::SpdyFrame** frames, int num_frames,
 }
 
 SpdySessionDependencies::SpdySessionDependencies()
-    : host_resolver(new MockHostResolver),
+    : host_resolver(new MockCachingHostResolver),
       cert_verifier(new CertVerifier),
       proxy_service(ProxyService::CreateDirect()),
       ssl_config_service(new SSLConfigServiceDefaults),
@@ -885,70 +923,63 @@ SpdySessionDependencies::~SpdySessionDependencies() {}
 // static
 HttpNetworkSession* SpdySessionDependencies::SpdyCreateSession(
     SpdySessionDependencies* session_deps) {
-  return new HttpNetworkSession(session_deps->host_resolver.get(),
-                                session_deps->cert_verifier.get(),
-                                NULL /* dnsrr_resolver */,
-                                NULL /* dns_cert_checker */,
-                                NULL /* ssl_host_info_factory */,
-                                session_deps->proxy_service,
-                                session_deps->socket_factory.get(),
-                                session_deps->ssl_config_service,
-                                new SpdySessionPool(NULL),
-                                session_deps->http_auth_handler_factory.get(),
-                                NULL,
-                                NULL);
+  net::HttpNetworkSession::Params params;
+  params.client_socket_factory = session_deps->socket_factory.get();
+  params.host_resolver = session_deps->host_resolver.get();
+  params.cert_verifier = session_deps->cert_verifier.get();
+  params.proxy_service = session_deps->proxy_service.get();
+  params.ssl_config_service = session_deps->ssl_config_service;
+  params.http_auth_handler_factory =
+      session_deps->http_auth_handler_factory.get();
+  params.http_server_properties = &session_deps->http_server_properties;
+  return new HttpNetworkSession(params);
 }
 
 // static
 HttpNetworkSession* SpdySessionDependencies::SpdyCreateSessionDeterministic(
     SpdySessionDependencies* session_deps) {
-  return new HttpNetworkSession(session_deps->host_resolver.get(),
-                                session_deps->cert_verifier.get(),
-                                NULL /* dnsrr_resolver */,
-                                NULL /* dns_cert_checker */,
-                                NULL /* ssl_host_info_factory */,
-                                session_deps->proxy_service,
-                                session_deps->
-                                deterministic_socket_factory.get(),
-                                session_deps->ssl_config_service,
-                                new SpdySessionPool(NULL),
-                                session_deps->http_auth_handler_factory.get(),
-                                NULL,
-                                NULL);
+  net::HttpNetworkSession::Params params;
+  params.client_socket_factory =
+      session_deps->deterministic_socket_factory.get();
+  params.host_resolver = session_deps->host_resolver.get();
+  params.cert_verifier = session_deps->cert_verifier.get();
+  params.proxy_service = session_deps->proxy_service.get();
+  params.ssl_config_service = session_deps->ssl_config_service;
+  params.http_auth_handler_factory =
+      session_deps->http_auth_handler_factory.get();
+  params.http_server_properties = &session_deps->http_server_properties;
+  return new HttpNetworkSession(params);
 }
 
-SpdyURLRequestContext::SpdyURLRequestContext() {
-  host_resolver_ = new MockHostResolver();
-  cert_verifier_ = new CertVerifier;
-  proxy_service_ = ProxyService::CreateDirect();
-  ssl_config_service_ = new SSLConfigServiceDefaults;
-  http_auth_handler_factory_ = HttpAuthHandlerFactory::CreateDefault(
-      host_resolver_);
-  http_transaction_factory_ = new HttpCache(
-      new HttpNetworkLayer(&socket_factory_,
-                           host_resolver_,
-                           cert_verifier_,
-                           NULL /* dnsrr_resolver */,
-                           NULL /* dns_cert_checker */,
-                           NULL /* ssl_host_info_factory */,
-                           proxy_service_,
-                           ssl_config_service_,
-                           new SpdySessionPool(NULL),
-                           http_auth_handler_factory_,
-                           network_delegate_,
-                           NULL),
-      NULL /* net_log */,
-      HttpCache::DefaultBackend::InMemory(0));
+SpdyURLRequestContext::SpdyURLRequestContext()
+    : ALLOW_THIS_IN_INITIALIZER_LIST(storage_(this)) {
+  storage_.set_host_resolver(new MockHostResolver());
+  storage_.set_cert_verifier(new CertVerifier);
+  storage_.set_proxy_service(ProxyService::CreateDirect());
+  storage_.set_ssl_config_service(new SSLConfigServiceDefaults);
+  storage_.set_http_auth_handler_factory(HttpAuthHandlerFactory::CreateDefault(
+      host_resolver()));
+  storage_.set_http_server_properties(new HttpServerPropertiesImpl);
+  net::HttpNetworkSession::Params params;
+  params.client_socket_factory = &socket_factory_;
+  params.host_resolver = host_resolver();
+  params.cert_verifier = cert_verifier();
+  params.proxy_service = proxy_service();
+  params.ssl_config_service = ssl_config_service();
+  params.http_auth_handler_factory = http_auth_handler_factory();
+  params.network_delegate = network_delegate();
+  params.http_server_properties = http_server_properties();
+  scoped_refptr<HttpNetworkSession> network_session(
+      new HttpNetworkSession(params));
+  storage_.set_http_transaction_factory(new HttpCache(
+      network_session,
+      HttpCache::DefaultBackend::InMemory(0)));
 }
 
 SpdyURLRequestContext::~SpdyURLRequestContext() {
-  delete http_transaction_factory_;
-  delete http_auth_handler_factory_;
-  delete cert_verifier_;
-  delete host_resolver_;
 }
 
-const SpdyHeaderInfo make_spdy_header(spdy::SpdyControlType type) {
+const SpdyHeaderInfo MakeSpdyHeader(spdy::SpdyControlType type) {
   const SpdyHeaderInfo kHeader = {
     type,                         // Kind = Syn
     1,                            // Stream ID
