@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,11 +17,11 @@
 #include "base/base64.h"
 #include "base/command_line.h"
 #include "base/debug/leak_annotations.h"
-#include "base/json/json_reader.h"
 #include "base/file_util.h"
+#include "base/json/json_reader.h"
 #include "base/logging.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
-#include "base/scoped_ptr.h"
 #include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
@@ -39,12 +39,6 @@ namespace net {
 
 namespace {
 
-// Number of connection attempts for tests.
-const int kServerConnectionAttempts = 10;
-
-// Connection timeout in milliseconds for tests.
-const int kServerConnectionTimeoutMs = 1000;
-
 std::string GetHostname(TestServer::Type type,
                         const TestServer::HTTPSOptions& options) {
   if (type == TestServer::TYPE_HTTPS &&
@@ -62,13 +56,15 @@ std::string GetHostname(TestServer::Type type,
 TestServer::HTTPSOptions::HTTPSOptions()
     : server_certificate(CERT_OK),
       request_client_certificate(false),
-      bulk_ciphers(HTTPSOptions::BULK_CIPHER_ANY) {}
+      bulk_ciphers(HTTPSOptions::BULK_CIPHER_ANY),
+      record_resume(false) {}
 
 TestServer::HTTPSOptions::HTTPSOptions(
     TestServer::HTTPSOptions::ServerCertificate cert)
     : server_certificate(cert),
       request_client_certificate(false),
-      bulk_ciphers(HTTPSOptions::BULK_CIPHER_ANY) {}
+      bulk_ciphers(HTTPSOptions::BULK_CIPHER_ANY),
+      record_resume(false) {}
 
 TestServer::HTTPSOptions::~HTTPSOptions() {}
 
@@ -87,7 +83,8 @@ FilePath TestServer::HTTPSOptions::GetCertificateFile() const {
 
 TestServer::TestServer(Type type, const FilePath& document_root)
     : type_(type),
-      started_(false) {
+      started_(false),
+      log_to_console_(false) {
   Init(document_root);
 }
 
@@ -95,7 +92,8 @@ TestServer::TestServer(const HTTPSOptions& https_options,
                        const FilePath& document_root)
     : https_options_(https_options),
       type_(TYPE_HTTPS),
-      started_(false) {
+      started_(false),
+      log_to_console_(false) {
   Init(document_root);
 }
 
@@ -105,25 +103,6 @@ TestServer::~TestServer() {
   Stop();
 }
 
-void TestServer::Init(const FilePath& document_root) {
-  // At this point, the port that the testserver will listen on is unknown.
-  // The testserver will listen on an ephemeral port, and write the port
-  // number out over a pipe that this TestServer object will read from. Once
-  // that is complete, the host_port_pair_ will contain the actual port.
-  host_port_pair_ = HostPortPair(GetHostname(type_, https_options_), 0);
-  process_handle_ = base::kNullProcessHandle;
-
-  FilePath src_dir;
-  PathService::Get(base::DIR_SOURCE_ROOT, &src_dir);
-
-  document_root_ = src_dir.Append(document_root);
-
-  certificates_dir_ = src_dir.Append(FILE_PATH_LITERAL("net"))
-                       .Append(FILE_PATH_LITERAL("data"))
-                       .Append(FILE_PATH_LITERAL("ssl"))
-                       .Append(FILE_PATH_LITERAL("certificates"));
-}
-
 bool TestServer::Start() {
   if (type_ == TYPE_HTTPS) {
     if (!LoadTestRootCert())
@@ -201,6 +180,10 @@ std::string TestServer::GetScheme() const {
       return "http";
     case TYPE_HTTPS:
       return "https";
+    case TYPE_TCP_ECHO:
+      NOTREACHED();
+    case TYPE_UDP_ECHO:
+      NOTREACHED();
     default:
       NOTREACHED();
   }
@@ -211,9 +194,15 @@ bool TestServer::GetAddressList(AddressList* address_list) const {
   DCHECK(address_list);
 
   scoped_ptr<HostResolver> resolver(
-      CreateSystemHostResolver(HostResolver::kDefaultParallelism, NULL, NULL));
+      CreateSystemHostResolver(HostResolver::kDefaultParallelism,
+                               HostResolver::kDefaultRetryAttempts,
+                               NULL));
   HostResolver::RequestInfo info(host_port_pair_);
-  int rv = resolver->Resolve(info, address_list, NULL, NULL, BoundNetLog());
+  TestCompletionCallback callback;
+  int rv = resolver->Resolve(info, address_list, callback.callback(), NULL,
+                             BoundNetLog());
+  if (rv == ERR_IO_PENDING)
+    rv = callback.WaitForResult();
   if (rv != net::OK) {
     LOG(ERROR) << "Failed to resolve hostname: " << host_port_pair_.host();
     return false;
@@ -276,91 +265,60 @@ bool TestServer::GetFilePathWithReplacements(
   return true;
 }
 
+void TestServer::Init(const FilePath& document_root) {
+  // At this point, the port that the testserver will listen on is unknown.
+  // The testserver will listen on an ephemeral port, and write the port
+  // number out over a pipe that this TestServer object will read from. Once
+  // that is complete, the host_port_pair_ will contain the actual port.
+  host_port_pair_ = HostPortPair(GetHostname(type_, https_options_), 0);
+  process_handle_ = base::kNullProcessHandle;
+
+  FilePath src_dir;
+  PathService::Get(base::DIR_SOURCE_ROOT, &src_dir);
+
+  document_root_ = src_dir.Append(document_root);
+
+  certificates_dir_ = src_dir.Append(FILE_PATH_LITERAL("net"))
+                       .Append(FILE_PATH_LITERAL("data"))
+                       .Append(FILE_PATH_LITERAL("ssl"))
+                       .Append(FILE_PATH_LITERAL("certificates"));
+
+  // TODO(battre) Remove this after figuring out why the TestServer is flaky.
+  // http://crbug.com/96594
+  log_to_console_ = true;
+}
+
 bool TestServer::SetPythonPath() {
   FilePath third_party_dir;
   if (!PathService::Get(base::DIR_SOURCE_ROOT, &third_party_dir)) {
     LOG(ERROR) << "Failed to get DIR_SOURCE_ROOT";
     return false;
   }
-  third_party_dir = third_party_dir.Append(FILE_PATH_LITERAL("third_party"));
+  third_party_dir = third_party_dir.AppendASCII("third_party");
 
   // For simplejson. (simplejson, unlike all the other python modules
   // we include, doesn't have an extra 'simplejson' directory, so we
   // need to include its parent directory, i.e. third_party_dir).
   AppendToPythonPath(third_party_dir);
 
-  AppendToPythonPath(third_party_dir.Append(FILE_PATH_LITERAL("tlslite")));
-  AppendToPythonPath(third_party_dir.Append(FILE_PATH_LITERAL("pyftpdlib")));
+  AppendToPythonPath(third_party_dir.AppendASCII("tlslite"));
+  AppendToPythonPath(
+      third_party_dir.AppendASCII("pyftpdlib").AppendASCII("src"));
 
   // Locate the Python code generated by the protocol buffers compiler.
-  FilePath pyproto_code_dir;
-  if (!GetPyProtoPath(&pyproto_code_dir)) {
-    LOG(ERROR) << "Failed to get python dir for generated code.";
-    return false;
+  FilePath pyproto_dir;
+  if (!GetPyProtoPath(&pyproto_dir)) {
+    LOG(WARNING) << "Cannot find pyproto dir for generated code. "
+                 << "Testserver features that rely on it will not work";
+    return true;
   }
 
-  AppendToPythonPath(pyproto_code_dir);
-  AppendToPythonPath(pyproto_code_dir.Append(FILE_PATH_LITERAL("sync_pb")));
-  AppendToPythonPath(pyproto_code_dir.Append(
-      FILE_PATH_LITERAL("device_management_pb")));
-
-  return true;
-}
-
-FilePath TestServer::GetRootCertificatePath() {
-  return certificates_dir_.AppendASCII("root_ca_cert.crt");
-}
-
-bool TestServer::LoadTestRootCert() {
-  TestRootCerts* root_certs = TestRootCerts::GetInstance();
-  return root_certs->AddFromFile(GetRootCertificatePath());
-}
-
-bool TestServer::AddCommandLineArguments(CommandLine* command_line) const {
-  command_line->AppendSwitchASCII("port",
-                                  base::IntToString(host_port_pair_.port()));
-  command_line->AppendSwitchPath("data-dir", document_root_);
-
-  if (type_ == TYPE_FTP) {
-    command_line->AppendArg("-f");
-  } else if (type_ == TYPE_SYNC) {
-    command_line->AppendArg("--sync");
-  } else if (type_ == TYPE_HTTPS) {
-    FilePath certificate_path(certificates_dir_);
-    certificate_path = certificate_path.Append(
-        https_options_.GetCertificateFile());
-    if (!file_util::PathExists(certificate_path)) {
-      LOG(ERROR) << "Certificate path " << certificate_path.value()
-                 << " doesn't exist. Can't launch https server.";
-      return false;
-    }
-    command_line->AppendSwitchPath("https", certificate_path);
-
-    if (https_options_.request_client_certificate)
-      command_line->AppendSwitch("ssl-client-auth");
-
-    for (std::vector<FilePath>::const_iterator it =
-             https_options_.client_authorities.begin();
-         it != https_options_.client_authorities.end(); ++it) {
-      if (!file_util::PathExists(*it)) {
-        LOG(ERROR) << "Client authority path " << it->value()
-                   << " doesn't exist. Can't launch https server.";
-        return false;
-      }
-
-      command_line->AppendSwitchPath("ssl-client-ca", *it);
-    }
-
-    const char kBulkCipherSwitch[] = "ssl-bulk-cipher";
-    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_RC4)
-      command_line->AppendSwitchASCII(kBulkCipherSwitch, "rc4");
-    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_AES128)
-      command_line->AppendSwitchASCII(kBulkCipherSwitch, "aes128");
-    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_AES256)
-      command_line->AppendSwitchASCII(kBulkCipherSwitch, "aes256");
-    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_3DES)
-      command_line->AppendSwitchASCII(kBulkCipherSwitch, "3des");
-  }
+  AppendToPythonPath(pyproto_dir);
+  AppendToPythonPath(pyproto_dir.AppendASCII("chrome")
+                       .AppendASCII("browser")
+                       .AppendASCII("sync")
+                       .AppendASCII("protocol"));
+  AppendToPythonPath(pyproto_dir.AppendASCII("device_management_pb"));
 
   return true;
 }
@@ -389,4 +347,76 @@ bool TestServer::ParseServerData(const std::string& server_data) {
   return true;
 }
 
+FilePath TestServer::GetRootCertificatePath() const {
+  return certificates_dir_.AppendASCII("root_ca_cert.crt");
+}
+
+bool TestServer::LoadTestRootCert() {
+  TestRootCerts* root_certs = TestRootCerts::GetInstance();
+  return root_certs->AddFromFile(GetRootCertificatePath());
+}
+
+bool TestServer::AddCommandLineArguments(CommandLine* command_line) const {
+  command_line->AppendArg("--port=" +
+                          base::IntToString(host_port_pair_.port()));
+  command_line->AppendArgNative(FILE_PATH_LITERAL("--data-dir=") +
+                                document_root_.value());
+
+  if (VLOG_IS_ON(1) || log_to_console_) {
+    command_line->AppendArg("--log-to-console");
+  }
+
+  if (type_ == TYPE_FTP) {
+    command_line->AppendArg("-f");
+  } else if (type_ == TYPE_SYNC) {
+    command_line->AppendArg("--sync");
+  } else if (type_ == TYPE_TCP_ECHO) {
+    command_line->AppendArg("--tcp-echo");
+  } else if (type_ == TYPE_UDP_ECHO) {
+    command_line->AppendArg("--udp-echo");
+  } else if (type_ == TYPE_HTTPS) {
+    FilePath certificate_path(certificates_dir_);
+    certificate_path = certificate_path.Append(
+        https_options_.GetCertificateFile());
+    if (!file_util::PathExists(certificate_path)) {
+      LOG(ERROR) << "Certificate path " << certificate_path.value()
+                 << " doesn't exist. Can't launch https server.";
+      return false;
+    }
+    command_line->AppendArgNative(FILE_PATH_LITERAL("--https=") +
+                                  certificate_path.value());
+
+    if (https_options_.request_client_certificate)
+      command_line->AppendArg("--ssl-client-auth");
+
+    for (std::vector<FilePath>::const_iterator it =
+             https_options_.client_authorities.begin();
+         it != https_options_.client_authorities.end(); ++it) {
+      if (!file_util::PathExists(*it)) {
+        LOG(ERROR) << "Client authority path " << it->value()
+                   << " doesn't exist. Can't launch https server.";
+        return false;
+      }
+
+      command_line->AppendArgNative(FILE_PATH_LITERAL("--ssl-client-ca=") +
+                                                      it->value());
+    }
+
+    const std::string kBulkCipherSwitch("--ssl-bulk-cipher");
+    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_RC4)
+      command_line->AppendArg(kBulkCipherSwitch + "=rc4");
+    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_AES128)
+      command_line->AppendArg(kBulkCipherSwitch + "=aes128");
+    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_AES256)
+      command_line->AppendArg(kBulkCipherSwitch + "=aes256");
+    if (https_options_.bulk_ciphers & HTTPSOptions::BULK_CIPHER_3DES)
+      command_line->AppendArg(kBulkCipherSwitch + "=3des");
+
+    if (https_options_.record_resume)
+      command_line->AppendArg("--https-record-resume");
+  }
+
+  return true;
+}
+
 }  // namespace net
