@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,6 +17,7 @@
 #include "net/disk_cache/disk_format.h"
 #include "net/disk_cache/mapped_file.h"
 #include "net/disk_cache/storage_block.h"
+#include "net/disk_cache/storage_block-inl.h"
 
 namespace {
 
@@ -33,7 +34,7 @@ bool ReadHeader(const std::wstring& name, char* header, int header_size) {
     return false;
   }
 
-  int read = file.Read(header, header_size, NULL);
+  int read = file.Read(header, header_size, net::CompletionCallback());
   if (read != header_size) {
     printf("Unable to read file %ls\n", name.c_str());
     return false;
@@ -256,7 +257,7 @@ void DumpRankings(const disk_cache::RankingsNode& rankings) {
   printf("prev: 0x%x\n", rankings.prev);
   printf("entry: 0x%x\n", rankings.contents);
   printf("dirty: %d\n", rankings.dirty);
-  printf("pointer: 0x%x\n", rankings.dummy);
+  printf("hash: 0x%x\n", rankings.self_hash);
   printf("----------\n\n");
 }
 
@@ -293,10 +294,10 @@ int DumpHeaders(const std::wstring& input_path) {
 
   std::wstring pattern(kDataPrefix);
   pattern.append(L"*");
-  file_util::FileEnumerator iter(FilePath::FromWStringHack(input_path), false,
+  file_util::FileEnumerator iter(FilePath(input_path), false,
                                  file_util::FileEnumerator::FILES, pattern);
-  for (std::wstring file = iter.Next().ToWStringHack(); !file.empty();
-       file = iter.Next().ToWStringHack()) {
+  for (std::wstring file = iter.Next().value(); !file.empty();
+       file = iter.Next().value()) {
     DumpBlockHeader(file);
   }
 
