@@ -1,10 +1,12 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "build/build_config.h"
 
 #include "base/at_exit.h"
+#include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/command_line.h"
 #include "base/lazy_instance.h"
 #include "base/message_loop.h"
@@ -16,14 +18,15 @@
 #include "net/base/host_resolver.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
-#include "net/base/ssl_config_service.h"
+#include "net/base/ssl_config_service_defaults.h"
 #include "net/http/http_auth_handler_factory.h"
 #include "net/http/http_cache.h"
 #include "net/http/http_network_layer.h"
+#include "net/http/http_network_session.h"
 #include "net/http/http_request_info.h"
+#include "net/http/http_server_properties_impl.h"
 #include "net/http/http_transaction.h"
 #include "net/proxy/proxy_service.h"
-#include "net/socket/client_socket_factory.h"
 
 void usage(const char* program_name) {
   printf("usage: %s --url=<url>  [--n=<clients>] [--stats] [--use_cache]\n",
@@ -48,18 +51,14 @@ class Driver {
   int clients_;
 };
 
-static base::LazyInstance<Driver> g_driver(base::LINKER_INITIALIZED);
+static base::LazyInstance<Driver> g_driver = LAZY_INSTANCE_INITIALIZER;
 
 // A network client
 class Client {
  public:
   Client(net::HttpTransactionFactory* factory, const std::string& url) :
       url_(url),
-      buffer_(new net::IOBuffer(kBufferSize)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          connect_callback_(this, &Client::OnConnectComplete)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          read_callback_(this, &Client::OnReadComplete)) {
+      buffer_(new net::IOBuffer(kBufferSize)) {
     int rv = factory->CreateTransaction(&transaction_);
     DCHECK_EQ(net::OK, rv);
     buffer_->AddRef();
@@ -67,14 +66,18 @@ class Client {
     request_info_.url = url_;
     request_info_.method = "GET";
     int state = transaction_->Start(
-        &request_info_, &connect_callback_, net::BoundNetLog());
+        &request_info_,
+        base::Bind(&Client::OnConnectComplete, base::Unretained(this)),
+        net::BoundNetLog());
     DCHECK(state == net::ERR_IO_PENDING);
   };
 
  private:
   void OnConnectComplete(int result) {
     // Do work here.
-    int state = transaction_->Read(buffer_.get(), kBufferSize, &read_callback_);
+    int state = transaction_->Read(
+        buffer_.get(), kBufferSize,
+        base::Bind(&Client::OnReadComplete, base::Unretained(this)));
     if (state == net::ERR_IO_PENDING)
       return;  // IO has started.
     if (state < 0)
@@ -89,11 +92,13 @@ class Client {
     }
 
     // Deal with received data here.
-    static base::StatsCounter bytes_read("FetchClient.bytes_read");
+    base::StatsCounter bytes_read("FetchClient.bytes_read");
     bytes_read.Add(result);
 
     // Issue a read for more data.
-    int state = transaction_->Read(buffer_.get(), kBufferSize, &read_callback_);
+    int state = transaction_->Read(
+        buffer_.get(), kBufferSize,
+        base::Bind(&Client::OnReadComplete, base::Unretained(this)));
     if (state == net::ERR_IO_PENDING)
       return;  // IO has started.
     if (state < 0)
@@ -102,7 +107,7 @@ class Client {
   }
 
   void OnRequestComplete(int result) {
-    static base::StatsCounter requests("FetchClient.requests");
+    base::StatsCounter requests("FetchClient.requests");
     requests.Increment();
     g_driver.Get().ClientStopped();
     printf(".");
@@ -113,11 +118,9 @@ class Client {
   net::HttpRequestInfo request_info_;
   scoped_ptr<net::HttpTransaction> transaction_;
   scoped_refptr<net::IOBuffer> buffer_;
-  net::CompletionCallbackImpl<Client> connect_callback_;
-  net::CompletionCallbackImpl<Client> read_callback_;
 };
 
-int main(int argc, char**argv) {
+int main(int argc, char** argv) {
   base::AtExitManager exit;
   base::StatsTable table("fetchclient", 50, 1000);
   table.set_current(&table);
@@ -139,34 +142,34 @@ int main(int argc, char**argv) {
 
   scoped_ptr<net::HostResolver> host_resolver(
       net::CreateSystemHostResolver(net::HostResolver::kDefaultParallelism,
-                                    NULL, NULL));
+                                    net::HostResolver::kDefaultRetryAttempts,
+                                    NULL));
 
   scoped_ptr<net::CertVerifier> cert_verifier(new net::CertVerifier);
-  scoped_refptr<net::ProxyService> proxy_service(
+  scoped_ptr<net::ProxyService> proxy_service(
       net::ProxyService::CreateDirect());
   scoped_refptr<net::SSLConfigService> ssl_config_service(
-      net::SSLConfigService::CreateSystemSSLConfigService());
+      new net::SSLConfigServiceDefaults);
   net::HttpTransactionFactory* factory = NULL;
   scoped_ptr<net::HttpAuthHandlerFactory> http_auth_handler_factory(
       net::HttpAuthHandlerFactory::CreateDefault(host_resolver.get()));
+  net::HttpServerPropertiesImpl http_server_properties;
+
+  net::HttpNetworkSession::Params session_params;
+  session_params.host_resolver = host_resolver.get();
+  session_params.cert_verifier = cert_verifier.get();
+  session_params.proxy_service = proxy_service.get();
+  session_params.http_auth_handler_factory = http_auth_handler_factory.get();
+  session_params.http_server_properties = &http_server_properties;
+  session_params.ssl_config_service = ssl_config_service;
+
+  scoped_refptr<net::HttpNetworkSession> network_session(
+      new net::HttpNetworkSession(session_params));
   if (use_cache) {
-    factory = new net::HttpCache(host_resolver.get(), cert_verifier.get(),
-        NULL, NULL, proxy_service, ssl_config_service,
-        http_auth_handler_factory.get(), NULL, NULL,
-        net::HttpCache::DefaultBackend::InMemory(0));
+    factory = new net::HttpCache(network_session,
+                                 net::HttpCache::DefaultBackend::InMemory(0));
   } else {
-    factory = new net::HttpNetworkLayer(
-        net::ClientSocketFactory::GetDefaultFactory(),
-        host_resolver.get(),
-        cert_verifier.get(),
-        NULL /* dnsrr_resolver */,
-        NULL /* dns_cert_checker */,
-        NULL /* ssl_host_info_factory */,
-        proxy_service,
-        ssl_config_service,
-        http_auth_handler_factory.get(),
-        NULL,
-        NULL);
+    factory = new net::HttpNetworkLayer(network_session);
   }
 
   {
