@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,7 +17,8 @@
 // does).
 class HttpServerRequestInfo : public net::HttpRequestInfo {
  public:
-  HttpServerRequestInfo() : net::HttpRequestInfo() {}
+  HttpServerRequestInfo();
+  virtual ~HttpServerRequestInfo();
 
   // A map of the names -> values for HTTP headers.
   std::map<std::string, std::string> headers;
