@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,7 +17,7 @@ class HttpSession : HttpListenSocket::Delegate {
   virtual ~HttpSession();
 
   virtual void OnRequest(HttpListenSocket* connection,
-                         HttpServerRequestInfo* info);
+                         HttpServerRequestInfo* info) OVERRIDE;
 
  private:
   scoped_refptr<HttpListenSocket> socket_;
