@@ -1,5 +1,4 @@
-#!/usr/bin/python2.4
-# Copyright (c) 2010 The Chromium Authors. All rights reserved.
+# Copyright (c) 2012 The Chromium Authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -12,19 +11,27 @@ buffer definition at chrome/browser/sync/protocol/sync.proto.
 import cgi
 import copy
 import operator
+import pickle
 import random
 import sys
 import threading
+import time
+import urlparse
 
+import app_notification_specifics_pb2
+import app_setting_specifics_pb2
 import app_specifics_pb2
 import autofill_specifics_pb2
 import bookmark_specifics_pb2
+import extension_setting_specifics_pb2
 import extension_specifics_pb2
 import nigori_specifics_pb2
 import password_specifics_pb2
 import preference_specifics_pb2
+import search_engine_specifics_pb2
 import session_specifics_pb2
 import sync_pb2
+import sync_enums_pb2
 import theme_specifics_pb2
 import typed_url_specifics_pb2
 
@@ -35,29 +42,39 @@ import typed_url_specifics_pb2
 ALL_TYPES = (
     TOP_LEVEL,  # The type of the 'Google Chrome' folder.
     APPS,
+    APP_NOTIFICATION,
+    APP_SETTINGS,
     AUTOFILL,
+    AUTOFILL_PROFILE,
     BOOKMARK,
     EXTENSIONS,
     NIGORI,
     PASSWORD,
     PREFERENCE,
+    SEARCH_ENGINE,
     SESSION,
     THEME,
-    TYPED_URL) = range(11)
+    TYPED_URL,
+    EXTENSION_SETTINGS) = range(16)
 
-# Well-known server tag of the top level "Google Chrome" folder.
+# Well-known server tag of the top level 'Google Chrome' folder.
 TOP_LEVEL_FOLDER_TAG = 'google_chrome'
 
 # Given a sync type from ALL_TYPES, find the extension token corresponding
 # to that datatype.  Note that TOP_LEVEL has no such token.
 SYNC_TYPE_TO_EXTENSION = {
+    APP_NOTIFICATION: app_notification_specifics_pb2.app_notification,
+    APP_SETTINGS: app_setting_specifics_pb2.app_setting,
     APPS: app_specifics_pb2.app,
     AUTOFILL: autofill_specifics_pb2.autofill,
+    AUTOFILL_PROFILE: autofill_specifics_pb2.autofill_profile,
     BOOKMARK: bookmark_specifics_pb2.bookmark,
+    EXTENSION_SETTINGS: extension_setting_specifics_pb2.extension_setting,
     EXTENSIONS: extension_specifics_pb2.extension,
     NIGORI: nigori_specifics_pb2.nigori,
     PASSWORD: password_specifics_pb2.password,
     PREFERENCE: preference_specifics_pb2.preference,
+    SEARCH_ENGINE: search_engine_specifics_pb2.search_engine,
     SESSION: session_specifics_pb2.session,
     THEME: theme_specifics_pb2.theme,
     TYPED_URL: typed_url_specifics_pb2.typed_url,
@@ -66,6 +83,9 @@ SYNC_TYPE_TO_EXTENSION = {
 # The parent ID used to indicate a top-level node.
 ROOT_ID = '0'
 
+# Unix time epoch in struct_time format. The tuple corresponds to UTC Wednesday
+# Jan 1 1970, 00:00:00, non-dst.
+UNIX_TIME_EPOCH = (1970, 1, 1, 0, 0, 0, 3, 1, 0)
 
 class Error(Exception):
   """Error class for this module."""
@@ -79,13 +99,36 @@ class DataTypeIdNotRecognized(Error):
   """The requested data type is not recognized."""
 
 
+class MigrationDoneError(Error):
+  """A server-side migration occurred; clients must re-sync some datatypes.
+
+  Attributes:
+    datatypes: a list of the datatypes (python enum) needing migration.
+  """
+
+  def __init__(self, datatypes):
+    self.datatypes = datatypes
+
+
+class StoreBirthdayError(Error):
+  """The client sent a birthday that doesn't correspond to this server."""
+
+
+class TransientError(Error):
+  """The client would be sent a transient error."""
+
+
+class SyncInducedError(Error):
+  """The client would be sent an error."""
+
+
 def GetEntryType(entry):
   """Extract the sync type from a SyncEntry.
 
   Args:
     entry: A SyncEntity protobuf object whose type to determine.
   Returns:
-    A value from ALL_TYPES if the entry's type can be determined, or None
+    An enum value from ALL_TYPES if the entry's type can be determined, or None
     if the type cannot be determined.
   Raises:
     ProtobufExtensionNotUnique: More than one type was indicated by the entry.
@@ -115,7 +158,7 @@ def GetEntryTypesFromSpecifics(specifics):
     specifics: A EntitySpecifics protobuf message whose extensions to
       enumerate.
   Returns:
-    A list of the sync types (values from ALL_TYPES) assocated with each
+    A list of the sync types (values from ALL_TYPES) associated with each
     recognized extension of the specifics message.
   """
   return [data_type for data_type, extension
@@ -136,6 +179,55 @@ def ProtocolDataTypeIdToSyncType(protocol_data_type_id):
   raise DataTypeIdNotRecognized
 
 
+def DataTypeStringToSyncTypeLoose(data_type_string):
+  """Converts a human-readable string to a sync type (python enum).
+
+  Capitalization and pluralization don't matter; this function is appropriate
+  for values that might have been typed by a human being; e.g., command-line
+  flags or query parameters.
+  """
+  if data_type_string.isdigit():
+    return ProtocolDataTypeIdToSyncType(int(data_type_string))
+  name = data_type_string.lower().rstrip('s')
+  for data_type, protocol_extension in SYNC_TYPE_TO_EXTENSION.iteritems():
+    if protocol_extension.name.lower().rstrip('s') == name:
+      return data_type
+  raise DataTypeIdNotRecognized
+
+
+def SyncTypeToString(data_type):
+  """Formats a sync type enum (from ALL_TYPES) to a human-readable string."""
+  return SYNC_TYPE_TO_EXTENSION[data_type].name
+
+
+def CallerInfoToString(caller_info_source):
+  """Formats a GetUpdatesSource enum value to a readable string."""
+  return sync_pb2.GetUpdatesCallerInfo.DESCRIPTOR.enum_types_by_name[
+      'GetUpdatesSource'].values_by_number[caller_info_source].name
+
+
+def ShortDatatypeListSummary(data_types):
+  """Formats compactly a list of sync types (python enums) for human eyes.
+
+  This function is intended for use by logging.  If the list of datatypes
+  contains almost all of the values, the return value will be expressed
+  in terms of the datatypes that aren't set.
+  """
+  included = set(data_types) - set([TOP_LEVEL])
+  if not included:
+    return 'nothing'
+  excluded = set(ALL_TYPES) - included - set([TOP_LEVEL])
+  if not excluded:
+    return 'everything'
+  simple_text = '+'.join(sorted([SyncTypeToString(x) for x in included]))
+  all_but_text = 'all except %s' % (
+      '+'.join(sorted([SyncTypeToString(x) for x in excluded])))
+  if len(included) < len(excluded) or len(simple_text) <= len(all_but_text):
+    return simple_text
+  else:
+    return all_but_text
+
+
 def GetDefaultEntitySpecifics(data_type):
   """Get an EntitySpecifics having a sync type's default extension value."""
   specifics = sync_pb2.EntitySpecifics()
@@ -145,13 +237,6 @@ def GetDefaultEntitySpecifics(data_type):
   return specifics
 
 
-def DeepCopyOfProto(proto):
-  """Return a deep copy of a protocol buffer."""
-  new_proto = type(proto)()
-  new_proto.MergeFrom(proto)
-  return new_proto
-
-
 class PermanentItem(object):
   """A specification of one server-created permanent item.
 
@@ -174,29 +259,93 @@ class PermanentItem(object):
     self.sync_type = sync_type
 
 
+class MigrationHistory(object):
+  """A record of the migration events associated with an account.
+
+  Each migration event invalidates one or more datatypes on all clients
+  that had synced the datatype before the event.  Such clients will continue
+  to receive MigrationDone errors until they throw away their progress and
+  re-sync that datatype from the beginning.
+  """
+  def __init__(self):
+    self._migrations = {}
+    for datatype in ALL_TYPES:
+      self._migrations[datatype] = [1]
+    self._next_migration_version = 2
+
+  def GetLatestVersion(self, datatype):
+    return self._migrations[datatype][-1]
+
+  def CheckAllCurrent(self, versions_map):
+    """Raises an error if any the provided versions are out of date.
+
+    This function intentionally returns migrations in the order that they were
+    triggered.  Doing it this way allows the client to queue up two migrations
+    in a row, so the second one is received while responding to the first.
+
+    Arguments:
+      version_map: a map whose keys are datatypes and whose values are versions.
+
+    Raises:
+      MigrationDoneError: if a mismatch is found.
+    """
+    problems = {}
+    for datatype, client_migration in versions_map.iteritems():
+      for server_migration in self._migrations[datatype]:
+        if client_migration < server_migration:
+          problems.setdefault(server_migration, []).append(datatype)
+    if problems:
+      raise MigrationDoneError(problems[min(problems.keys())])
+
+  def Bump(self, datatypes):
+    """Add a record of a migration, to cause errors on future requests."""
+    for idx, datatype in enumerate(datatypes):
+      self._migrations[datatype].append(self._next_migration_version)
+    self._next_migration_version += 1
+
+
 class UpdateSieve(object):
   """A filter to remove items the client has already seen."""
-  def __init__(self, request):
+  def __init__(self, request, migration_history=None):
     self._original_request = request
     self._state = {}
+    self._migration_history = migration_history or MigrationHistory()
+    self._migration_versions_to_check = {}
     if request.from_progress_marker:
       for marker in request.from_progress_marker:
-        if marker.HasField("timestamp_token_for_migration"):
+        data_type = ProtocolDataTypeIdToSyncType(marker.data_type_id)
+        if marker.HasField('timestamp_token_for_migration'):
           timestamp = marker.timestamp_token_for_migration
+          if timestamp:
+            self._migration_versions_to_check[data_type] = 1
         elif marker.token:
-          timestamp = int(marker.token)
-        elif marker.HasField("token"):
+          (timestamp, version) = pickle.loads(marker.token)
+          self._migration_versions_to_check[data_type] = version
+        elif marker.HasField('token'):
           timestamp = 0
         else:
-          raise ValueError("No timestamp information in progress marker.")
+          raise ValueError('No timestamp information in progress marker.')
         data_type = ProtocolDataTypeIdToSyncType(marker.data_type_id)
         self._state[data_type] = timestamp
-    elif request.HasField("from_timestamp"):
+    elif request.HasField('from_timestamp'):
       for data_type in GetEntryTypesFromSpecifics(request.requested_types):
         self._state[data_type] = request.from_timestamp
+        self._migration_versions_to_check[data_type] = 1
     if self._state:
       self._state[TOP_LEVEL] = min(self._state.itervalues())
 
+  def SummarizeRequest(self):
+    timestamps = {}
+    for data_type, timestamp in self._state.iteritems():
+      if data_type == TOP_LEVEL:
+        continue
+      timestamps.setdefault(timestamp, []).append(data_type)
+    return ', '.join('<%s>@%d' % (ShortDatatypeListSummary(types), stamp)
+                     for stamp, types in sorted(timestamps.iteritems()))
+
+  def CheckMigrationState(self):
+    self._migration_history.CheckAllCurrent(self._migration_versions_to_check)
+
   def ClientWantsItem(self, item):
     """Return true if the client hasn't already seen an item."""
     return self._state.get(GetEntryType(item), sys.maxint) < item.version
@@ -222,10 +371,12 @@ class UpdateSieve(object):
           continue
         new_marker = sync_pb2.DataTypeProgressMarker()
         new_marker.data_type_id = SyncTypeToProtocolDataTypeId(data_type)
-        new_marker.token = str(max(old_timestamp, new_timestamp))
+        final_stamp = max(old_timestamp, new_timestamp)
+        final_migration = self._migration_history.GetLatestVersion(data_type)
+        new_marker.token = pickle.dumps((final_stamp, final_migration))
         if new_marker not in self._original_request.from_progress_marker:
           get_updates_response.new_progress_marker.add().MergeFrom(new_marker)
-    elif self._original_request.HasField("from_timestamp"):
+    elif self._original_request.HasField('from_timestamp'):
       if self._original_request.from_timestamp < new_timestamp:
         get_updates_response.new_timestamp = new_timestamp
 
@@ -248,10 +399,20 @@ class SyncDataModel(object):
                     parent_tag='google_chrome', sync_type=PREFERENCE),
       PermanentItem('google_chrome_autofill', name='Autofill',
                     parent_tag='google_chrome', sync_type=AUTOFILL),
+      PermanentItem('google_chrome_autofill_profiles', name='Autofill Profiles',
+                    parent_tag='google_chrome', sync_type=AUTOFILL_PROFILE),
+      PermanentItem('google_chrome_app_settings',
+                    name='App Settings',
+                    parent_tag='google_chrome', sync_type=APP_SETTINGS),
+      PermanentItem('google_chrome_extension_settings',
+                    name='Extension Settings',
+                    parent_tag='google_chrome', sync_type=EXTENSION_SETTINGS),
       PermanentItem('google_chrome_extensions', name='Extensions',
                     parent_tag='google_chrome', sync_type=EXTENSIONS),
       PermanentItem('google_chrome_passwords', name='Passwords',
                     parent_tag='google_chrome', sync_type=PASSWORD),
+      PermanentItem('google_chrome_search_engines', name='Search Engines',
+                    parent_tag='google_chrome', sync_type=SEARCH_ENGINE),
       PermanentItem('google_chrome_sessions', name='Sessions',
                     parent_tag='google_chrome', sync_type=SESSION),
       PermanentItem('google_chrome_themes', name='Themes',
@@ -262,6 +423,8 @@ class SyncDataModel(object):
                     parent_tag='google_chrome', sync_type=NIGORI),
       PermanentItem('google_chrome_apps', name='Apps',
                     parent_tag='google_chrome', sync_type=APPS),
+      PermanentItem('google_chrome_app_notifications', name='App Notifications',
+                    parent_tag='google_chrome', sync_type=APP_NOTIFICATION),
       ]
 
   def __init__(self):
@@ -273,8 +436,11 @@ class SyncDataModel(object):
     # SyncEntity protocol buffer.
     self._entries = {}
 
-    # TODO(nick): uuid.uuid1() is better, but python 2.5 only.
-    self.store_birthday = '%0.30f' % random.random()
+    self.ResetStoreBirthday()
+
+    self.migration_history = MigrationHistory()
+
+    self.induced_error = sync_pb2.ClientToServerResponse.Error()
 
   def _SaveEntry(self, entry):
     """Insert or update an entry in the change log, and give it a new version.
@@ -299,7 +465,10 @@ class SyncDataModel(object):
       entry.originator_cache_guid = base_entry.originator_cache_guid
       entry.originator_client_item_id = base_entry.originator_client_item_id
 
-    self._entries[entry.id_string] = DeepCopyOfProto(entry)
+    self._entries[entry.id_string] = copy.deepcopy(entry)
+    # Store the current time since the Unix epoch in milliseconds.
+    self._entries[entry.id_string].mtime = (int((time.mktime(time.gmtime()) -
+        time.mktime(UNIX_TIME_EPOCH))*1000))
 
   def _ServerTagToId(self, tag):
     """Determine the server ID from a server-unique tag.
@@ -308,35 +477,38 @@ class SyncDataModel(object):
     generation methods.
 
     Args:
+      datatype: The sync type (python enum) of the identified object.
       tag: The unique, known-to-the-client tag of a server-generated item.
     Returns:
       The string value of the computed server ID.
     """
-    if tag and tag != ROOT_ID:
-      return '<server tag>%s' % tag
-    else:
+    if not tag or tag == ROOT_ID:
       return tag
+    spec = [x for x in self._PERMANENT_ITEM_SPECS if x.tag == tag][0]
+    return self._MakeCurrentId(spec.sync_type, '<server tag>%s' % tag)
 
-  def _ClientTagToId(self, tag):
+  def _ClientTagToId(self, datatype, tag):
     """Determine the server ID from a client-unique tag.
 
     The resulting value is guaranteed not to collide with the other ID
     generation methods.
 
     Args:
+      datatype: The sync type (python enum) of the identified object.
       tag: The unique, opaque-to-the-server tag of a client-tagged item.
     Returns:
       The string value of the computed server ID.
     """
-    return '<client tag>%s' % tag
+    return self._MakeCurrentId(datatype, '<client tag>%s' % tag)
 
-  def _ClientIdToId(self, client_guid, client_item_id):
+  def _ClientIdToId(self, datatype, client_guid, client_item_id):
     """Compute a unique server ID from a client-local ID tag.
 
     The resulting value is guaranteed not to collide with the other ID
     generation methods.
 
     Args:
+      datatype: The sync type (python enum) of the identified object.
       client_guid: A globally unique ID that identifies the client which
         created this item.
       client_item_id: An ID that uniquely identifies this item on the client
@@ -346,68 +518,41 @@ class SyncDataModel(object):
     """
     # Using the client ID info is not required here (we could instead generate
     # a random ID), but it's useful for debugging.
-    return '<server ID originally>%s/%s' % (client_guid, client_item_id)
+    return self._MakeCurrentId(datatype,
+        '<server ID originally>%s/%s' % (client_guid, client_item_id))
 
-  def _WritePosition(self, entry, parent_id, prev_id=None):
-    """Convert from a relative position into an absolute, numeric position.
+  def _MakeCurrentId(self, datatype, inner_id):
+    return '%d^%d^%s' % (datatype,
+                         self.migration_history.GetLatestVersion(datatype),
+                         inner_id)
 
-    Clients specify positions using the predecessor-based references; the
-    server stores and reports item positions using sparse integer values.
-    This method converts from the former to the latter.
+  def _ExtractIdInfo(self, id_string):
+    if not id_string or id_string == ROOT_ID:
+      return None
+    datatype_string, separator, remainder = id_string.partition('^')
+    migration_version_string, separator, inner_id = remainder.partition('^')
+    return (int(datatype_string), int(migration_version_string), inner_id)
+
+  def _WritePosition(self, entry, parent_id):
+    """Ensure the entry has an absolute, numeric position and parent_id.
+
+    Historically, clients would specify positions using the predecessor-based
+    references in the insert_after_item_id field; starting July 2011, this
+    was changed and Chrome now sends up the absolute position.  The server
+    must store a position_in_parent value and must not maintain
+    insert_after_item_id.
 
     Args:
-      entry: The entry for which to compute a position.  Its ID field are
+      entry: The entry for which to write a position.  Its ID field are
         assumed to be server IDs.  This entry will have its parent_id_string
         and position_in_parent fields updated; its insert_after_item_id field
         will be cleared.
       parent_id: The ID of the entry intended as the new parent.
-      prev_id: The ID of the entry intended as the new predecessor.  If this
-        is None, or an ID of an object which is not a child of the new parent,
-        the entry will be positioned at the end (right) of the ordering.  If
-        the empty ID (''), this will be positioned at the front (left) of the
-        ordering.  Otherwise, the entry will be given a position_in_parent
-        value placing it just after (to the right of) the new predecessor.
     """
-    preferred_gap = 2 ** 20
-
-    def ExtendRange(current_limit_entry, sign_multiplier):
-      """Compute values at the beginning or end."""
-      if current_limit_entry.id_string == entry.id_string:
-        step = 0
-      else:
-        step = sign_multiplier * preferred_gap
-      return current_limit_entry.position_in_parent + step
-
-    siblings = [x for x in self._entries.values()
-                if x.parent_id_string == parent_id and not x.deleted]
-    siblings = sorted(siblings, key=operator.attrgetter('position_in_parent'))
-    if prev_id == entry.id_string:
-      prev_id = ''
-    if not siblings:
-      # First item in this container; start in the middle.
-      entry.position_in_parent = 0
-    elif not prev_id:
-      # A special value in the protocol.  Insert at first position.
-      entry.position_in_parent = ExtendRange(siblings[0], -1)
-    else:
-      # Handle mid-insertion; consider items along with their successors.
-      for item, successor in zip(siblings, siblings[1:]):
-        if item.id_string != prev_id:
-          continue
-        elif successor.id_string == entry.id_string:
-          # We're already in place; don't change anything.
-          entry.position_in_parent = successor.position_in_parent
-        else:
-          # Interpolate new position between the previous item and its
-          # existing successor.
-          entry.position_in_parent = (item.position_in_parent * 7 +
-                                      successor.position_in_parent) / 8
-        break
-      else:
-        # Insert at end. Includes the case where prev_id is None.
-        entry.position_in_parent = ExtendRange(siblings[-1], +1)
 
     entry.parent_id_string = parent_id
+    if not entry.HasField('position_in_parent'):
+      entry.position_in_parent = 1337  # A debuggable, distinctive default.
     entry.ClearField('insert_after_item_id')
 
   def _ItemExists(self, id_string):
@@ -448,6 +593,15 @@ class SyncDataModel(object):
       if spec.sync_type in requested_types:
         self._CreatePermanentItem(spec)
 
+  def ResetStoreBirthday(self):
+    """Resets the store birthday to a random value."""
+    # TODO(nick): uuid.uuid1() is better, but python 2.5 only.
+    self.store_birthday = '%0.30f' % random.random()
+
+  def StoreBirthday(self):
+    """Gets the store birthday."""
+    return self.store_birthday
+
   def GetChanges(self, sieve):
     """Get entries which have changed, oldest first.
 
@@ -480,13 +634,25 @@ class SyncDataModel(object):
 
     # Restrict batch to requested types.  Tombstones are untyped
     # and will always get included.
-    filtered = [DeepCopyOfProto(item) for item in batch
+    filtered = [copy.deepcopy(item) for item in batch
                 if item.deleted or sieve.ClientWantsItem(item)]
 
     # The new client timestamp is the timestamp of the last item in the
     # batch, even if that item was filtered out.
     return (batch[-1].version, filtered, len(new_changes) - len(batch))
 
+  def _CopyOverImmutableFields(self, entry):
+    """Preserve immutable fields by copying pre-commit state.
+
+    Args:
+      entry: A sync entity from the client.
+    """
+    if entry.id_string in self._entries:
+      if self._entries[entry.id_string].HasField(
+          'server_defined_unique_tag'):
+        entry.server_defined_unique_tag = (
+            self._entries[entry.id_string].server_defined_unique_tag)
+
   def _CheckVersionForCommit(self, entry):
     """Perform an optimistic concurrency check on the version number.
 
@@ -552,11 +718,12 @@ class SyncDataModel(object):
         IDs, for any items committed earlier in the batch.
     """
     if entry.version == 0:
+      data_type = GetEntryType(entry)
       if entry.HasField('client_defined_unique_tag'):
         # When present, this should determine the item's ID.
-        new_id = self._ClientTagToId(entry.client_defined_unique_tag)
+        new_id = self._ClientTagToId(data_type, entry.client_defined_unique_tag)
       else:
-        new_id = self._ClientIdToId(cache_guid, entry.id_string)
+        new_id = self._ClientIdToId(data_type, cache_guid, entry.id_string)
         entry.originator_cache_guid = cache_guid
         entry.originator_client_item_id = entry.id_string
       commit_session[entry.id_string] = new_id  # Remember the remapping.
@@ -566,6 +733,37 @@ class SyncDataModel(object):
     if entry.insert_after_item_id in commit_session:
       entry.insert_after_item_id = commit_session[entry.insert_after_item_id]
 
+  def ValidateCommitEntries(self, entries):
+    """Raise an exception if a commit batch contains any global errors.
+
+    Arguments:
+      entries: an iterable containing commit-form SyncEntity protocol buffers.
+
+    Raises:
+      MigrationDoneError: if any of the entries reference a recently-migrated
+        datatype.
+    """
+    server_ids_in_commit = set()
+    local_ids_in_commit = set()
+    for entry in entries:
+      if entry.version:
+        server_ids_in_commit.add(entry.id_string)
+      else:
+        local_ids_in_commit.add(entry.id_string)
+      if entry.HasField('parent_id_string'):
+        if entry.parent_id_string not in local_ids_in_commit:
+          server_ids_in_commit.add(entry.parent_id_string)
+
+    versions_present = {}
+    for server_id in server_ids_in_commit:
+      parsed = self._ExtractIdInfo(server_id)
+      if parsed:
+        datatype, version, _ = parsed
+        versions_present.setdefault(datatype, []).append(version)
+
+    self.migration_history.CheckAllCurrent(
+         dict((k, min(v)) for k, v in versions_present.iteritems()))
+
   def CommitEntry(self, entry, cache_guid, commit_session):
     """Attempt to commit one entry to the user's account.
 
@@ -581,7 +779,7 @@ class SyncDataModel(object):
       A SyncEntity reflecting the post-commit value of the entry, or None
       if the entry was not committed due to an error.
     """
-    entry = DeepCopyOfProto(entry)
+    entry = copy.deepcopy(entry)
 
     # Generate server IDs for this entry, and write generated server IDs
     # from earlier entries into the message's fields, as appropriate.  The
@@ -599,6 +797,8 @@ class SyncDataModel(object):
     if not self._CheckParentIdForCommit(entry):
       return None
 
+    self._CopyOverImmutableFields(entry);
+
     # At this point, the commit is definitely going to happen.
 
     # Deletion works by storing a limited record for an entry, called a
@@ -649,19 +849,13 @@ class SyncDataModel(object):
       entry = MakeTombstone(entry.id_string)
     else:
       # Comments in sync.proto detail how the representation of positional
-      # ordering works: the 'insert_after_item_id' field specifies a
-      # predecessor during Commit operations, but the 'position_in_parent'
-      # field provides an absolute ordering in GetUpdates contexts.  Here
-      # we convert from the former to the latter.  Specifically, we'll
-      # generate a numeric position placing the item just after the object
-      # identified by 'insert_after_item_id', and then clear the
-      # 'insert_after_item_id' field so that it's not sent back to the client
-      # during later GetUpdates requests.
-      if entry.HasField('insert_after_item_id'):
-        self._WritePosition(entry, entry.parent_id_string,
-                            entry.insert_after_item_id)
-      else:
-        self._WritePosition(entry, entry.parent_id_string)
+      # ordering works: either the 'insert_after_item_id' field or the
+      # 'position_in_parent' field may determine the sibling order during
+      # Commit operations.  The 'position_in_parent' field provides an absolute
+      # ordering in GetUpdates contexts.  Here we assume the client will
+      # always send a valid position_in_parent (this is the newer style), and
+      # we ignore insert_after_item_id (an older style).
+      self._WritePosition(entry, entry.parent_id_string)
 
     # Preserve the originator info, which the client is not required to send
     # when updating.
@@ -674,6 +868,54 @@ class SyncDataModel(object):
     self._SaveEntry(entry)
     return entry
 
+  def _RewriteVersionInId(self, id_string):
+    """Rewrites an ID so that its migration version becomes current."""
+    parsed_id = self._ExtractIdInfo(id_string)
+    if not parsed_id:
+      return id_string
+    datatype, old_migration_version, inner_id = parsed_id
+    return self._MakeCurrentId(datatype, inner_id)
+
+  def TriggerMigration(self, datatypes):
+    """Cause a migration to occur for a set of datatypes on this account.
+
+    Clients will see the MIGRATION_DONE error for these datatypes until they
+    resync them.
+    """
+    versions_to_remap = self.migration_history.Bump(datatypes)
+    all_entries = self._entries.values()
+    self._entries.clear()
+    for entry in all_entries:
+      new_id = self._RewriteVersionInId(entry.id_string)
+      entry.id_string = new_id
+      if entry.HasField('parent_id_string'):
+        entry.parent_id_string = self._RewriteVersionInId(
+            entry.parent_id_string)
+      self._entries[entry.id_string] = entry
+
+  def TriggerSyncTabs(self):
+    """Set the 'sync_tabs' field to this account's nigori node.
+
+    If the field is not currently set, will write a new nigori node entry
+    with the field set. Else does nothing.
+    """
+
+    nigori_tag = "google_chrome_nigori"
+    nigori_original = self._entries.get(self._ServerTagToId(nigori_tag))
+    if (nigori_original.specifics.Extensions[nigori_specifics_pb2.nigori].
+        sync_tabs):
+      return
+    nigori_new = copy.deepcopy(nigori_original)
+    nigori_new.specifics.Extensions[nigori_specifics_pb2.nigori].sync_tabs = (
+        True)
+    self._SaveEntry(nigori_new)
+
+  def SetInducedError(self, error):
+    self.induced_error = error
+
+  def GetInducedError(self):
+    return self.induced_error
+
 
 class TestServer(object):
   """An object to handle requests for one (and only one) Chrome Sync account.
@@ -691,7 +933,8 @@ class TestServer(object):
     # to its nickname.
     self.clients = {}
     self.client_name_generator = ('+' * times + chr(c)
-        for times in xrange(0, sys.maxint) for c in xrange(ord('A'),ord('Z')))
+        for times in xrange(0, sys.maxint) for c in xrange(ord('A'), ord('Z')))
+    self.transient_error = False
 
   def GetShortClientName(self, query):
     parsed = cgi.parse_qs(query[query.find('?')+1:])
@@ -703,6 +946,98 @@ class TestServer(object):
       self.clients[client_id] = self.client_name_generator.next()
     return self.clients[client_id]
 
+  def CheckStoreBirthday(self, request):
+    """Raises StoreBirthdayError if the request's birthday is a mismatch."""
+    if not request.HasField('store_birthday'):
+      return
+    if self.account.StoreBirthday() != request.store_birthday:
+      raise StoreBirthdayError
+
+  def CheckTransientError(self):
+    """Raises TransientError if transient_error variable is set."""
+    if self.transient_error:
+      raise TransientError
+
+  def CheckSendError(self):
+     """Raises SyncInducedError if needed."""
+     if (self.account.induced_error.error_type !=
+         sync_enums_pb2.SyncEnums.UNKNOWN):
+       raise SyncInducedError
+
+  def HandleMigrate(self, path):
+    query = urlparse.urlparse(path)[4]
+    code = 200
+    self.account_lock.acquire()
+    try:
+      datatypes = [DataTypeStringToSyncTypeLoose(x)
+                   for x in urlparse.parse_qs(query).get('type',[])]
+      if datatypes:
+        self.account.TriggerMigration(datatypes)
+        response = 'Migrated datatypes %s' % (
+            ' and '.join(SyncTypeToString(x).upper() for x in datatypes))
+      else:
+        response = 'Please specify one or more <i>type=name</i> parameters'
+        code = 400
+    except DataTypeIdNotRecognized, error:
+      response = 'Could not interpret datatype name'
+      code = 400
+    finally:
+      self.account_lock.release()
+    return (code, '<html><title>Migration: %d</title><H1>%d %s</H1></html>' %
+                (code, code, response))
+
+  def HandleSetInducedError(self, path):
+     query = urlparse.urlparse(path)[4]
+     self.account_lock.acquire()
+     code = 200;
+     response = 'Success'
+     error = sync_pb2.ClientToServerResponse.Error()
+     try:
+       error_type = urlparse.parse_qs(query)['error']
+       action = urlparse.parse_qs(query)['action']
+       error.error_type = int(error_type[0])
+       error.action = int(action[0])
+       try:
+         error.url = (urlparse.parse_qs(query)['url'])[0]
+       except KeyError:
+         error.url = ''
+       try:
+         error.error_description =(
+         (urlparse.parse_qs(query)['error_description'])[0])
+       except KeyError:
+         error.error_description = ''
+       self.account.SetInducedError(error)
+       response = ('Error = %d, action = %d, url = %s, description = %s' %
+                   (error.error_type, error.action,
+                    error.url,
+                    error.error_description))
+     except error:
+       response = 'Could not parse url'
+       code = 400
+     finally:
+       self.account_lock.release()
+     return (code, '<html><title>SetError: %d</title><H1>%d %s</H1></html>' %
+                (code, code, response))
+
+  def HandleCreateBirthdayError(self):
+    self.account.ResetStoreBirthday()
+    return (
+        200,
+        '<html><title>Birthday error</title><H1>Birthday error</H1></html>')
+
+  def HandleSetTransientError(self):
+    self.transient_error = True
+    return (
+        200,
+        '<html><title>Transient error</title><H1>Transient error</H1></html>')
+
+  def HandleSetSyncTabs(self):
+    """Set the 'sync_tab' field of the nigori node for this account."""
+    self.account.TriggerSyncTabs()
+    return (
+        200,
+        '<html><title>Sync Tabs</title><H1>Sync Tabs</H1></html>')
+
   def HandleCommand(self, query, raw_request):
     """Decode and handle a sync command from a raw input of bytes.
 
@@ -718,30 +1053,74 @@ class TestServer(object):
       serialized reply to the command.
     """
     self.account_lock.acquire()
+    def print_context(direction):
+      print '[Client %s %s %s.py]' % (self.GetShortClientName(query), direction,
+                                      __name__),
+
     try:
       request = sync_pb2.ClientToServerMessage()
       request.MergeFromString(raw_request)
       contents = request.message_contents
 
       response = sync_pb2.ClientToServerResponse()
-      response.error_code = sync_pb2.ClientToServerResponse.SUCCESS
+      response.error_code = sync_enums_pb2.SyncEnums.SUCCESS
+      self.CheckStoreBirthday(request)
       response.store_birthday = self.account.store_birthday
-      log_context = "[Client %s -> %s.py]" % (self.GetShortClientName(query),
-                                              __name__)
+      self.CheckTransientError();
+      self.CheckSendError();
+
+      print_context('->')
 
       if contents == sync_pb2.ClientToServerMessage.AUTHENTICATE:
-        print '%s Authenticate' % log_context
+        print 'Authenticate'
         # We accept any authentication token, and support only one account.
         # TODO(nick): Mock out the GAIA authentication as well; hook up here.
         response.authenticate.user.email = 'syncjuser@chromium'
         response.authenticate.user.display_name = 'Sync J User'
       elif contents == sync_pb2.ClientToServerMessage.COMMIT:
-        print '%s Commit' % log_context
+        print 'Commit %d item(s)' % len(request.commit.entries)
         self.HandleCommit(request.commit, response.commit)
       elif contents == sync_pb2.ClientToServerMessage.GET_UPDATES:
-        print ('%s GetUpdates from timestamp %d' %
-               (log_context, request.get_updates.from_timestamp))
+        print 'GetUpdates',
         self.HandleGetUpdates(request.get_updates, response.get_updates)
+        print_context('<-')
+        print '%d update(s)' % len(response.get_updates.entries)
+      else:
+        print 'Unrecognizable sync request!'
+        return (400, None)  # Bad request.
+      return (200, response.SerializeToString())
+    except MigrationDoneError, error:
+      print_context('<-')
+      print 'MIGRATION_DONE: <%s>' % (ShortDatatypeListSummary(error.datatypes))
+      response = sync_pb2.ClientToServerResponse()
+      response.store_birthday = self.account.store_birthday
+      response.error_code = sync_enums_pb2.SyncEnums.MIGRATION_DONE
+      response.migrated_data_type_id[:] = [
+          SyncTypeToProtocolDataTypeId(x) for x in error.datatypes]
+      return (200, response.SerializeToString())
+    except StoreBirthdayError, error:
+      print_context('<-')
+      print 'NOT_MY_BIRTHDAY'
+      response = sync_pb2.ClientToServerResponse()
+      response.store_birthday = self.account.store_birthday
+      response.error_code = sync_enums_pb2.SyncEnums.NOT_MY_BIRTHDAY
+      return (200, response.SerializeToString())
+    except TransientError, error:
+      ### This is deprecated now. Would be removed once test cases are removed.
+      print_context('<-')
+      print 'TRANSIENT_ERROR'
+      response.store_birthday = self.account.store_birthday
+      response.error_code = sync_enums_pb2.SyncEnums.TRANSIENT_ERROR
+      return (200, response.SerializeToString())
+    except SyncInducedError, error:
+      print_context('<-')
+      print 'INDUCED_ERROR'
+      response.store_birthday = self.account.store_birthday
+      error = self.account.GetInducedError()
+      response.error.error_type = error.error_type
+      response.error.url = error.url
+      response.error.error_description = error.error_description
+      response.error.action = error.action
       return (200, response.SerializeToString())
     finally:
       self.account_lock.release()
@@ -762,6 +1141,9 @@ class TestServer(object):
     batch_failure = False
     session = {}  # Tracks ID renaming during the commit operation.
     guid = commit_message.cache_guid
+
+    self.account.ValidateCommitEntries(commit_message.entries)
+
     for entry in commit_message.entries:
       server_entry = None
       if not batch_failure:
@@ -800,7 +1182,13 @@ class TestServer(object):
         to the client request will be written.
     """
     update_response.SetInParent()
-    update_sieve = UpdateSieve(update_request)
+    update_sieve = UpdateSieve(update_request, self.account.migration_history)
+
+    print CallerInfoToString(update_request.caller_info.source),
+    print update_sieve.SummarizeRequest()
+
+    update_sieve.CheckMigrationState()
+
     new_timestamp, entries, remaining = self.account.GetChanges(update_sieve)
 
     update_response.changes_remaining = remaining
