@@ -1,13 +1,15 @@
-#!/usr/bin/python2.4
-# Copyright (c) 2010 The Chromium Authors. All rights reserved.
+#!/usr/bin/env python
+# Copyright (c) 2011 The Chromium Authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
 """Tests exercising chromiumsync and SyncDataModel."""
 
+import pickle
 import unittest
 
 import autofill_specifics_pb2
+import bookmark_specifics_pb2
 import chromiumsync
 import sync_pb2
 import theme_specifics_pb2
@@ -25,7 +27,15 @@ class SyncDataModelTest(unittest.TestCase):
     for data_type in requested_types:
       message.requested_types.Extensions[
         chromiumsync.SYNC_TYPE_TO_EXTENSION[data_type]].SetInParent()
-    return self.model.GetChanges(chromiumsync.UpdateSieve(message))
+    return self.model.GetChanges(
+        chromiumsync.UpdateSieve(message, self.model.migration_history))
+
+  def FindMarkerByNumber(self, markers, datatype):
+    """Search a list of progress markers and find the one for a datatype."""
+    for marker in markers:
+      if marker.data_type_id == datatype.number:
+        return marker
+    self.fail('Required marker not found: %s' % datatype.name)
 
   def testPermanentItemSpecs(self):
     specs = chromiumsync.SyncDataModel._PERMANENT_ITEM_SPECS
@@ -55,59 +65,6 @@ class SyncDataModelTest(unittest.TestCase):
     self.assertTrue(self.model._ItemExists(proto.id_string))
     self.assertEqual(2, self.model._entries[proto.id_string].version)
 
-  def testWritePosition(self):
-    def MakeProto(id_string, parent, position):
-      proto = sync_pb2.SyncEntity()
-      proto.id_string = id_string
-      proto.position_in_parent = position
-      proto.parent_id_string = parent
-      self.AddToModel(proto)
-
-    MakeProto('a', 'X', 1000)
-    MakeProto('b', 'X', 1800)
-    MakeProto('c', 'X', 2600)
-    MakeProto('a1', 'Z', 1007)
-    MakeProto('a2', 'Z', 1807)
-    MakeProto('a3', 'Z', 2607)
-    MakeProto('s', 'Y', 10000)
-
-    def AssertPositionResult(my_id, parent_id, prev_id, expected_position):
-      entry = sync_pb2.SyncEntity()
-      entry.id_string = my_id
-      self.model._WritePosition(entry, parent_id, prev_id)
-      self.assertEqual(expected_position, entry.position_in_parent)
-      self.assertEqual(parent_id, entry.parent_id_string)
-      self.assertFalse(entry.HasField('insert_after_item_id'))
-
-    AssertPositionResult('new', 'new_parent', '', 0)
-    AssertPositionResult('new', 'Y', '', 10000 - (2 ** 20))
-    AssertPositionResult('new', 'Y', 's', 10000 + (2 ** 20))
-    AssertPositionResult('s', 'Y', '', 10000)
-    AssertPositionResult('s', 'Y', 's', 10000)
-    AssertPositionResult('a1', 'Z', '', 1007)
-
-    AssertPositionResult('new', 'X', '', 1000 - (2 ** 20))
-    AssertPositionResult('new', 'X', 'a', 1100)
-    AssertPositionResult('new', 'X', 'b', 1900)
-    AssertPositionResult('new', 'X', 'c', 2600 + (2 ** 20))
-
-    AssertPositionResult('a1', 'X', '', 1000 - (2 ** 20))
-    AssertPositionResult('a1', 'X', 'a', 1100)
-    AssertPositionResult('a1', 'X', 'b', 1900)
-    AssertPositionResult('a1', 'X', 'c', 2600 + (2 ** 20))
-
-    AssertPositionResult('a', 'X', '', 1000)
-    AssertPositionResult('a', 'X', 'b', 1900)
-    AssertPositionResult('a', 'X', 'c', 2600 + (2 ** 20))
-
-    AssertPositionResult('b', 'X', '', 1000 - (2 ** 20))
-    AssertPositionResult('b', 'X', 'a', 1800)
-    AssertPositionResult('b', 'X', 'c', 2600 + (2 ** 20))
-
-    AssertPositionResult('c', 'X', '', 1000 - (2 ** 20))
-    AssertPositionResult('c', 'X', 'a', 1100)
-    AssertPositionResult('c', 'X', 'b', 2600)
-
   def testCreatePermanentItems(self):
     self.model._CreatePermanentItems(chromiumsync.ALL_TYPES)
     self.assertEqual(len(chromiumsync.ALL_TYPES) + 2,
@@ -227,7 +184,7 @@ class SyncDataModelTest(unittest.TestCase):
           self.GetChangesFromTimestamp([sync_type], 0))
 
       def DoCommit(original=None, id_string='', name=None, parent=None,
-                   prev=None):
+                   position=0):
         proto = sync_pb2.SyncEntity()
         if original is not None:
           proto.version = original.version
@@ -242,10 +199,8 @@ class SyncDataModelTest(unittest.TestCase):
           proto.name = name
         if parent:
           proto.parent_id_string = parent.id_string
-        if prev:
-          proto.insert_after_item_id = prev.id_string
-        else:
-          proto.insert_after_item_id = ''
+        proto.insert_after_item_id = 'please discard'
+        proto.position_in_parent = position
         proto.folder = True
         proto.deleted = False
         result = self.model.CommitEntry(proto, my_cache_guid, commit_session)
@@ -254,14 +209,14 @@ class SyncDataModelTest(unittest.TestCase):
 
       # Commit a new item.
       proto1, result1 = DoCommit(name='namae', id_string='Foo',
-                                 parent=original_changes[-1])
+                                 parent=original_changes[-1], position=100)
       # Commit an item whose parent is another item (referenced via the
       # pre-commit ID).
       proto2, result2 = DoCommit(name='Secondo', id_string='Bar',
-                                 parent=proto1)
+                                 parent=proto1, position=-100)
         # Commit a sibling of the second item.
       proto3, result3 = DoCommit(name='Third!', id_string='Baz',
-                                 parent=proto1, prev=proto2)
+                                 parent=proto1, position=-50)
 
       self.assertEqual(3, len(commit_session))
       for p, r in [(proto1, result1), (proto2, result2), (proto3, result3)]:
@@ -286,7 +241,7 @@ class SyncDataModelTest(unittest.TestCase):
         self.assertTrue(c is not self.model._entries[c.id_string],
                         "GetChanges didn't make a defensive copy.")
       self.assertTrue(result2.position_in_parent < result3.position_in_parent)
-      self.assertEqual(0, result2.position_in_parent)
+      self.assertEqual(-100, result2.position_in_parent)
 
       # Now update the items so that the second item is the parent of the
       # first; with the first sandwiched between two new items (4 and 5).
@@ -298,11 +253,11 @@ class SyncDataModelTest(unittest.TestCase):
       proto2b, result2b = DoCommit(original=result2,
                                    parent=original_changes[-1])
       proto4, result4 = DoCommit(id_string='ID4', name='Four',
-                                 parent=result2, prev=None)
+                                 parent=result2, position=-200)
       proto1b, result1b = DoCommit(original=result1,
-                                   parent=result2, prev=proto4)
+                                   parent=result2, position=-150)
       proto5, result5 = DoCommit(id_string='ID5', name='Five', parent=result2,
-                                 prev=result1)
+                                 position=150)
 
       self.assertEqual(2, len(commit_session), 'Only new items in second '
                        'batch should be in the session')
@@ -371,10 +326,10 @@ class SyncDataModelTest(unittest.TestCase):
     msg = sync_pb2.GetUpdatesMessage()
     marker = msg.from_progress_marker.add()
     marker.data_type_id = autofill.number
-    marker.token = '15412'
+    marker.token = pickle.dumps((15412, 1))
     marker = msg.from_progress_marker.add()
     marker.data_type_id = theme.number
-    marker.token = '15413'
+    marker.token = pickle.dumps((15413, 1))
     sieve = chromiumsync.UpdateSieve(msg)
     self.assertEqual(sieve._state,
         {chromiumsync.TOP_LEVEL: 15412,
@@ -387,14 +342,14 @@ class SyncDataModelTest(unittest.TestCase):
     self.assertFalse(response.HasField('new_timestamp'))
     marker = response.new_progress_marker[0]
     self.assertEqual(marker.data_type_id, autofill.number)
-    self.assertEqual(marker.token, '15413')
+    self.assertEqual(pickle.loads(marker.token), (15413, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
 
     # Empty tokens indicating from timestamp = 0
     msg = sync_pb2.GetUpdatesMessage()
     marker = msg.from_progress_marker.add()
     marker.data_type_id = autofill.number
-    marker.token = '412'
+    marker.token = pickle.dumps((412, 1))
     marker = msg.from_progress_marker.add()
     marker.data_type_id = theme.number
     marker.token = ''
@@ -409,7 +364,7 @@ class SyncDataModelTest(unittest.TestCase):
     self.assertFalse(response.HasField('new_timestamp'))
     marker = response.new_progress_marker[0]
     self.assertEqual(marker.data_type_id, theme.number)
-    self.assertEqual(marker.token, '1')
+    self.assertEqual(pickle.loads(marker.token), (1, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
 
     response = sync_pb2.GetUpdatesResponse()
@@ -418,23 +373,24 @@ class SyncDataModelTest(unittest.TestCase):
     self.assertFalse(response.HasField('new_timestamp'))
     marker = response.new_progress_marker[0]
     self.assertEqual(marker.data_type_id, theme.number)
-    self.assertEqual(marker.token, '412')
+    self.assertEqual(pickle.loads(marker.token), (412, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
 
     response = sync_pb2.GetUpdatesResponse()
     sieve.SaveProgress(413, response)
     self.assertEqual(2, len(response.new_progress_marker))
     self.assertFalse(response.HasField('new_timestamp'))
-    marker = response.new_progress_marker[0]
-    self.assertEqual(marker.data_type_id, theme.number)
-    self.assertEqual(marker.token, '413')
+    marker = self.FindMarkerByNumber(response.new_progress_marker, theme)
+    self.assertEqual(pickle.loads(marker.token), (413, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
-    marker = response.new_progress_marker[1]
-    self.assertEqual(marker.data_type_id, autofill.number)
-    self.assertEqual(marker.token, '413')
+    marker = self.FindMarkerByNumber(response.new_progress_marker, autofill)
+    self.assertEqual(pickle.loads(marker.token), (413, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
 
     # Migration token timestamps (client gives timestamp, server returns token)
+    # These are for migrating from the old 'timestamp' protocol to the
+    # progressmarker protocol, and have nothing to do with the MIGRATION_DONE
+    # error code.
     msg = sync_pb2.GetUpdatesMessage()
     marker = msg.from_progress_marker.add()
     marker.data_type_id = autofill.number
@@ -451,13 +407,11 @@ class SyncDataModelTest(unittest.TestCase):
     sieve.SaveProgress(16000, response)  # There were updates
     self.assertEqual(2, len(response.new_progress_marker))
     self.assertFalse(response.HasField('new_timestamp'))
-    marker = response.new_progress_marker[0]
-    self.assertEqual(marker.data_type_id, theme.number)
-    self.assertEqual(marker.token, '16000')
+    marker = self.FindMarkerByNumber(response.new_progress_marker, theme)
+    self.assertEqual(pickle.loads(marker.token), (16000, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
-    marker = response.new_progress_marker[1]
-    self.assertEqual(marker.data_type_id, autofill.number)
-    self.assertEqual(marker.token, '16000')
+    marker = self.FindMarkerByNumber(response.new_progress_marker, autofill)
+    self.assertEqual(pickle.loads(marker.token), (16000, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
 
     msg = sync_pb2.GetUpdatesMessage()
@@ -476,15 +430,136 @@ class SyncDataModelTest(unittest.TestCase):
     sieve.SaveProgress(3000, response)  # Already up to date
     self.assertEqual(2, len(response.new_progress_marker))
     self.assertFalse(response.HasField('new_timestamp'))
-    marker = response.new_progress_marker[0]
-    self.assertEqual(marker.data_type_id, theme.number)
-    self.assertEqual(marker.token, '3000')
+    marker = self.FindMarkerByNumber(response.new_progress_marker, theme)
+    self.assertEqual(pickle.loads(marker.token), (3000, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
-    marker = response.new_progress_marker[1]
-    self.assertEqual(marker.data_type_id, autofill.number)
-    self.assertEqual(marker.token, '3000')
+    marker = self.FindMarkerByNumber(response.new_progress_marker, autofill)
+    self.assertEqual(pickle.loads(marker.token), (3000, 1))
     self.assertFalse(marker.HasField('timestamp_token_for_migration'))
 
+  def testCheckRaiseTransientError(self):
+    testserver = chromiumsync.TestServer()
+    http_code, raw_respon = testserver.HandleSetTransientError()
+    self.assertEqual(http_code, 200)
+    try:
+      testserver.CheckTransientError()
+      self.fail('Should have raised transient error exception')
+    except chromiumsync.TransientError:
+      self.assertTrue(testserver.transient_error)
+
+  def testUpdateSieveStoreMigration(self):
+    autofill = autofill_specifics_pb2.autofill
+    theme = theme_specifics_pb2.theme
+    migrator = chromiumsync.MigrationHistory()
+    msg = sync_pb2.GetUpdatesMessage()
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = autofill.number
+    marker.token = pickle.dumps((15412, 1))
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = theme.number
+    marker.token = pickle.dumps((15413, 1))
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    sieve.CheckMigrationState()
+
+    migrator.Bump([chromiumsync.BOOKMARK, chromiumsync.PASSWORD])  # v=2
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    sieve.CheckMigrationState()
+    self.assertEqual(sieve._state,
+        {chromiumsync.TOP_LEVEL: 15412,
+         chromiumsync.AUTOFILL: 15412,
+         chromiumsync.THEME: 15413})
+
+    migrator.Bump([chromiumsync.AUTOFILL, chromiumsync.PASSWORD])  # v=3
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    try:
+      sieve.CheckMigrationState()
+      self.fail('Should have raised.')
+    except chromiumsync.MigrationDoneError, error:
+      # We want this to happen.
+      self.assertEqual([chromiumsync.AUTOFILL], error.datatypes)
+
+    msg = sync_pb2.GetUpdatesMessage()
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = autofill.number
+    marker.token = ''
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = theme.number
+    marker.token = pickle.dumps((15413, 1))
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    sieve.CheckMigrationState()
+    response = sync_pb2.GetUpdatesResponse()
+    sieve.SaveProgress(15412, response)  # There were updates
+    self.assertEqual(1, len(response.new_progress_marker))
+    self.assertFalse(response.HasField('new_timestamp'))
+    self.assertFalse(marker.HasField('timestamp_token_for_migration'))
+    marker = self.FindMarkerByNumber(response.new_progress_marker, autofill)
+    self.assertEqual(pickle.loads(marker.token), (15412, 3))
+    self.assertFalse(marker.HasField('timestamp_token_for_migration'))
+    msg = sync_pb2.GetUpdatesMessage()
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = autofill.number
+    marker.token = pickle.dumps((15412, 3))
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = theme.number
+    marker.token = pickle.dumps((15413, 1))
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    sieve.CheckMigrationState()
+
+    migrator.Bump([chromiumsync.THEME, chromiumsync.AUTOFILL])  # v=4
+    migrator.Bump([chromiumsync.AUTOFILL])                      # v=5
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    try:
+      sieve.CheckMigrationState()
+      self.fail("Should have raised.")
+    except chromiumsync.MigrationDoneError, error:
+      # We want this to happen.
+      self.assertEqual(set([chromiumsync.THEME, chromiumsync.AUTOFILL]),
+                       set(error.datatypes))
+    msg = sync_pb2.GetUpdatesMessage()
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = autofill.number
+    marker.token = ''
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = theme.number
+    marker.token = pickle.dumps((15413, 1))
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    try:
+      sieve.CheckMigrationState()
+      self.fail("Should have raised.")
+    except chromiumsync.MigrationDoneError, error:
+      # We want this to happen.
+      self.assertEqual([chromiumsync.THEME], error.datatypes)
+
+    msg = sync_pb2.GetUpdatesMessage()
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = autofill.number
+    marker.token = ''
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = theme.number
+    marker.token = ''
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    sieve.CheckMigrationState()
+    response = sync_pb2.GetUpdatesResponse()
+    sieve.SaveProgress(15412, response)  # There were updates
+    self.assertEqual(2, len(response.new_progress_marker))
+    self.assertFalse(response.HasField('new_timestamp'))
+    self.assertFalse(marker.HasField('timestamp_token_for_migration'))
+    marker = self.FindMarkerByNumber(response.new_progress_marker, autofill)
+    self.assertEqual(pickle.loads(marker.token), (15412, 5))
+    self.assertFalse(marker.HasField('timestamp_token_for_migration'))
+    marker = self.FindMarkerByNumber(response.new_progress_marker, theme)
+    self.assertEqual(pickle.loads(marker.token), (15412, 4))
+    self.assertFalse(marker.HasField('timestamp_token_for_migration'))
+    msg = sync_pb2.GetUpdatesMessage()
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = autofill.number
+    marker.token = pickle.dumps((15412, 5))
+    marker = msg.from_progress_marker.add()
+    marker.data_type_id = theme.number
+    marker.token = pickle.dumps((15413, 4))
+    sieve = chromiumsync.UpdateSieve(msg, migrator)
+    sieve.CheckMigrationState()
+
 
 if __name__ == '__main__':
   unittest.main()
