@@ -1,5 +1,4 @@
-#!/usr/bin/python2.4
-# Copyright (c) 2010 The Chromium Authors. All rights reserved.
+# Copyright (c) 2011 The Chromium Authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -243,9 +242,9 @@ class HandshakeTask(object):
   # The id attribute is filled in later.
   _IQ_RESPONSE_STANZA = ParseXml('<iq id="" type="result"/>')
 
-  def __init__(self, connection, id_generator, resource_prefix):
+  def __init__(self, connection, resource_prefix):
     self._connection = connection
-    self._id_generator = id_generator
+    self._id_generator = IdGenerator(resource_prefix)
     self._username = ''
     self._domain = ''
     self._jid = None
@@ -357,18 +356,10 @@ class XmppConnection(asynchat.async_chat):
   Google notification protocol.
   """
 
-  # We use this XML template for subscription responses as well as
-  # notifications (conveniently enough, the same template works
-  # for both).
+  # Used for acknowledgements to the client.
   #
-  # The from, to, id, and type attributes are filled in later.
-  _NOTIFIER_STANZA = ParseXml(
-    """<iq from="" to="" id="" type="">
-         <not:getAll xmlns:not="google:notifier">
-           <Result xmlns=""/>
-         </not:getAll>
-       </iq>
-    """)
+  # The from and id attributes are filled in later.
+  _IQ_RESPONSE_STANZA = ParseXml('<iq from="" id="" type="result"/>')
 
   def __init__(self, sock, socket_map, delegate, addr):
     """Starts up the xmpp connection.
@@ -398,9 +389,7 @@ class XmppConnection(asynchat.async_chat):
 
     self._addr = addr
     addr_str = AddrString(self._addr)
-    self._id_generator = IdGenerator(addr_str)
-    self._handshake_task = (
-      HandshakeTask(self, self._id_generator, addr_str))
+    self._handshake_task = HandshakeTask(self, addr_str)
     print 'Starting connection to %s' % self
 
   def __str__(self):
@@ -427,8 +416,12 @@ class XmppConnection(asynchat.async_chat):
   def FeedStanza(self, stanza):
     if self._handshake_task:
       self._handshake_task.FeedStanza(stanza)
-    elif stanza.tagName == 'iq':
-      self._HandleIq(stanza)
+    elif stanza.tagName == 'iq' and stanza.getAttribute('type') == 'result':
+      # Ignore all client acks.
+      pass
+    elif (stanza.firstChild and
+          stanza.firstChild.namespaceURI == 'google:push'):
+      self._HandlePushCommand(stanza)
     else:
       raise UnexpectedXml(stanza)
 
@@ -439,36 +432,20 @@ class XmppConnection(asynchat.async_chat):
     self._delegate.OnXmppHandshakeDone(self)
     print "Handshake done for %s" % self
 
-  def _HandleIq(self, iq):
-    if (iq.firstChild and
-        iq.firstChild.namespaceURI == 'google:notifier'):
-      iq_id = iq.getAttribute('id')
-      self._HandleNotifierCommand(iq_id, iq.firstChild)
-    elif iq.getAttribute('type') == 'result':
-      # Ignore all client acks.
-      pass
-    else:
-      raise UnexpectedXml(iq)
-
-  def _HandleNotifierCommand(self, id, command_xml):
-    command = command_xml.tagName
-    if command == 'getAll':
+  def _HandlePushCommand(self, stanza):
+    if stanza.tagName == 'iq' and stanza.firstChild.tagName == 'subscribe':
       # Subscription request.
-      if not command_xml.getElementsByTagName('SubscribedServiceUrl'):
-        raise UnexpectedXml(command_xml)
-      self._SendNotifierStanza(id, 'result')
-    elif command == 'set':
+      self._SendIqResponseStanza(stanza)
+    elif stanza.tagName == 'message' and stanza.firstChild.tagName == 'push':
       # Send notification request.
-      self._delegate.SendNotification(self)
+      self._delegate.ForwardNotification(self, stanza)
     else:
       raise UnexpectedXml(command_xml)
 
-  def _SendNotifierStanza(self, id, type):
-    stanza = CloneXml(self._NOTIFIER_STANZA)
+  def _SendIqResponseStanza(self, iq):
+    stanza = CloneXml(self._IQ_RESPONSE_STANZA)
     stanza.setAttribute('from', str(self._jid.GetBareJid()))
-    stanza.setAttribute('to', str(self._jid))
-    stanza.setAttribute('id', id)
-    stanza.setAttribute('type', type)
+    stanza.setAttribute('id', iq.getAttribute('id'))
     self.SendStanza(stanza)
 
   def SendStanza(self, stanza, unlink=True):
@@ -490,10 +467,11 @@ class XmppConnection(asynchat.async_chat):
     # (some minidom library functions return unicode strings).
     self.push(data.encode('ascii'))
 
-  def SendNotification(self):
-    """Sends a notification to the client."""
-    next_id = self._id_generator.GetNextId()
-    self._SendNotifierStanza(next_id, 'set')
+  def ForwardNotification(self, notification_stanza):
+    """Forwards a notification to the client."""
+    notification_stanza.setAttribute('from', str(self._jid.GetBareJid()))
+    notification_stanza.setAttribute('to', str(self._jid))
+    self.SendStanza(notification_stanza, False)
 
 
 class XmppServer(asyncore.dispatcher):
@@ -509,6 +487,14 @@ class XmppServer(asyncore.dispatcher):
     asyncore.loop(30.0, False, socket_map)
   """
 
+  # Used when sending a notification.
+  _NOTIFICATION_STANZA = ParseXml(
+    '<message>'
+    '  <push xmlns="google:push">'
+    '    <data/>'
+    '  </push>'
+    '</message>')
+
   def __init__(self, socket_map, addr):
     asyncore.dispatcher.__init__(self, None, socket_map)
     self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -518,11 +504,14 @@ class XmppServer(asyncore.dispatcher):
     self._socket_map = socket_map
     self._connections = set()
     self._handshake_done_connections = set()
+    self._notifications_enabled = True
 
   def handle_accept(self):
     (sock, addr) = self.accept()
     xmpp_connection = XmppConnection(sock, self._socket_map, self, addr)
     self._connections.add(xmpp_connection)
+    # Return the new XmppConnection for testing.
+    return xmpp_connection
 
   def close(self):
     # A copy is necessary since calling close on each connection
@@ -531,6 +520,39 @@ class XmppServer(asyncore.dispatcher):
       connection.close()
     asyncore.dispatcher.close(self)
 
+  def EnableNotifications(self):
+    self._notifications_enabled = True
+
+  def DisableNotifications(self):
+    self._notifications_enabled = False
+
+  def MakeNotification(self, channel, data):
+    """Makes a notification from the given channel and encoded data.
+
+    Args:
+      channel: The channel on which to send the notification.
+      data: The notification payload.
+    """
+    notification_stanza = CloneXml(self._NOTIFICATION_STANZA)
+    push_element = notification_stanza.getElementsByTagName('push')[0]
+    push_element.setAttribute('channel', channel)
+    data_element = push_element.getElementsByTagName('data')[0]
+    encoded_data = base64.b64encode(data)
+    data_text = notification_stanza.parentNode.createTextNode(encoded_data)
+    data_element.appendChild(data_text)
+    return notification_stanza
+
+  def SendNotification(self, channel, data):
+    """Sends a notification to all connections.
+
+    Args:
+      channel: The channel on which to send the notification.
+      data: The notification payload.
+    """
+    notification_stanza = self.MakeNotification(channel, data)
+    self.ForwardNotification(None, notification_stanza)
+    notification_stanza.unlink()
+
   # XmppConnection delegate methods.
   def OnXmppHandshakeDone(self, xmpp_connection):
     self._handshake_done_connections.add(xmpp_connection)
@@ -539,7 +561,10 @@ class XmppServer(asyncore.dispatcher):
     self._connections.discard(xmpp_connection)
     self._handshake_done_connections.discard(xmpp_connection)
 
-  def SendNotification(self, unused_xmpp_connection):
-    for connection in self._handshake_done_connections:
-      print 'Sending notification to %s' % connection
-      connection.SendNotification()
+  def ForwardNotification(self, unused_xmpp_connection, notification_stanza):
+    if self._notifications_enabled:
+      for connection in self._handshake_done_connections:
+        print 'Sending notification to %s' % connection
+        connection.ForwardNotification(notification_stanza)
+    else:
+      print 'Notifications disabled; dropping notification'
