@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,24 +8,28 @@
 
 #include <string>
 
+#include "base/memory/weak_ptr.h"
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_job.h"
 
 namespace net {
 
-class URLRequestAboutJob : public URLRequestJob {
+class NET_EXPORT URLRequestAboutJob : public URLRequestJob {
  public:
   explicit URLRequestAboutJob(URLRequest* request);
 
-  virtual void Start();
-  virtual bool GetMimeType(std::string* mime_type) const;
-
   static URLRequest::ProtocolFactory Factory;
 
+  // URLRequestJob:
+  virtual void Start() OVERRIDE;
+  virtual bool GetMimeType(std::string* mime_type) const OVERRIDE;
+
  private:
-  ~URLRequestAboutJob();
+  virtual ~URLRequestAboutJob();
 
   void StartAsync();
+
+  base::WeakPtrFactory<URLRequestAboutJob> weak_factory_;
 };
 
 }  // namespace net
