@@ -11,37 +11,48 @@
 #define NET_URL_REQUEST_URL_REQUEST_CONTEXT_H_
 #pragma once
 
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
 #include "base/threading/non_thread_safe.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/base/ssl_config_service.h"
 #include "net/base/transport_security_state.h"
+#include "net/http/http_server_properties.h"
 #include "net/ftp/ftp_auth_cache.h"
-#include "net/proxy/proxy_service.h"
-#include "net/socket/dns_cert_provenance_checker.h"
 
 namespace net {
 class CertVerifier;
-class CookiePolicy;
 class CookieStore;
-class DnsCertProvenanceChecker;
-class DnsRRResolver;
+class FraudulentCertificateReporter;
 class FtpTransactionFactory;
 class HostResolver;
 class HttpAuthHandlerFactory;
-class HttpNetworkDelegate;
 class HttpTransactionFactory;
-class SSLConfigService;
+class NetworkDelegate;
+class OriginBoundCertService;
+class ProxyService;
 class URLRequest;
+class URLRequestJobFactory;
 
 // Subclass to provide application-specific context for URLRequest
-// instances.
-class URLRequestContext
+// instances. Note that URLRequestContext typically does not provide storage for
+// these member variables, since they may be shared. For the ones that aren't
+// shared, URLRequestContextStorage can be helpful in defining their storage.
+class NET_EXPORT URLRequestContext
     : public base::RefCountedThreadSafe<URLRequestContext>,
-      public base::NonThreadSafe {
+      NON_EXPORTED_BASE(public base::NonThreadSafe) {
  public:
   URLRequestContext();
 
+  base::WeakPtr<URLRequestContext> GetWeakPtr() {
+    return weak_factory_.GetWeakPtr();
+  }
+
+  // Copies the state from |other| into this context.
+  void CopyFrom(URLRequestContext* other);
+
   NetLog* net_log() const {
     return net_log_;
   }
@@ -66,30 +77,33 @@ class URLRequestContext
     cert_verifier_ = cert_verifier;
   }
 
-  DnsRRResolver* dnsrr_resolver() const {
-    return dnsrr_resolver_;
+  OriginBoundCertService* origin_bound_cert_service() const {
+    return origin_bound_cert_service_;
   }
 
-  void set_dnsrr_resolver(DnsRRResolver* dnsrr_resolver) {
-    dnsrr_resolver_ = dnsrr_resolver;
+  void set_origin_bound_cert_service(
+      OriginBoundCertService* origin_bound_cert_service) {
+    origin_bound_cert_service_ = origin_bound_cert_service;
   }
 
-  DnsCertProvenanceChecker* dns_cert_checker() const {
-    return dns_cert_checker_.get();
+  FraudulentCertificateReporter* fraudulent_certificate_reporter() const {
+    return fraudulent_certificate_reporter_;
+  }
+  void set_fraudulent_certificate_reporter(
+      FraudulentCertificateReporter* fraudulent_certificate_reporter) {
+    fraudulent_certificate_reporter_ = fraudulent_certificate_reporter;
   }
 
   // Get the proxy service for this context.
-  ProxyService* proxy_service() const {
-    return proxy_service_;
-  }
-
+  ProxyService* proxy_service() const { return proxy_service_; }
   void set_proxy_service(ProxyService* proxy_service) {
     proxy_service_ = proxy_service;
   }
 
   // Get the ssl config service for this context.
-  SSLConfigService* ssl_config_service() const {
-    return ssl_config_service_;
+  SSLConfigService* ssl_config_service() const { return ssl_config_service_; }
+  void set_ssl_config_service(SSLConfigService* service) {
+    ssl_config_service_ = service;
   }
 
   // Gets the HTTP Authentication Handler Factory for this context.
@@ -105,37 +119,58 @@ class URLRequestContext
   HttpTransactionFactory* http_transaction_factory() const {
     return http_transaction_factory_;
   }
-
   void set_http_transaction_factory(HttpTransactionFactory* factory) {
     http_transaction_factory_ = factory;
   }
 
   // Gets the ftp transaction factory for this context.
-  FtpTransactionFactory* ftp_transaction_factory() {
+  FtpTransactionFactory* ftp_transaction_factory() const {
     return ftp_transaction_factory_;
   }
+  void set_ftp_transaction_factory(FtpTransactionFactory* factory) {
+    ftp_transaction_factory_ = factory;
+  }
+
+  void set_network_delegate(NetworkDelegate* network_delegate) {
+    network_delegate_ = network_delegate;
+  }
+  NetworkDelegate* network_delegate() const { return network_delegate_; }
+
+  void set_http_server_properties(
+      HttpServerProperties* http_server_properties) {
+    http_server_properties_ = http_server_properties;
+  }
+  HttpServerProperties* http_server_properties() const {
+    return http_server_properties_;
+  }
 
   // Gets the cookie store for this context (may be null, in which case
   // cookies are not stored).
-  CookieStore* cookie_store() { return cookie_store_.get(); }
-
+  CookieStore* cookie_store() const { return cookie_store_.get(); }
   void set_cookie_store(CookieStore* cookie_store);
 
-  // Gets the cookie policy for this context (may be null, in which case
-  // cookies are allowed).
-  CookiePolicy* cookie_policy() { return cookie_policy_; }
-
-  TransportSecurityState* transport_security_state() {
-      return transport_security_state_; }
+  TransportSecurityState* transport_security_state() const {
+      return transport_security_state_;
+  }
+  void set_transport_security_state(
+      TransportSecurityState* state) {
+    transport_security_state_ = state;
+  }
 
   // Gets the FTP authentication cache for this context.
-  FtpAuthCache* ftp_auth_cache() { return &ftp_auth_cache_; }
+  FtpAuthCache* ftp_auth_cache() const { return ftp_auth_cache_.get(); }
 
   // Gets the value of 'Accept-Charset' header field.
   const std::string& accept_charset() const { return accept_charset_; }
+  void set_accept_charset(const std::string& accept_charset) {
+    accept_charset_ = accept_charset;
+  }
 
   // Gets the value of 'Accept-Language' header field.
   const std::string& accept_language() const { return accept_language_; }
+  void set_accept_language(const std::string& accept_language) {
+    accept_language_ = accept_language;
+  }
 
   // Gets the UA string to use for the given URL.  Pass an invalid URL (such as
   // GURL()) to get the default UA string.  Subclasses should override this
@@ -149,43 +184,53 @@ class URLRequestContext
     referrer_charset_ = charset;
   }
 
-  // Controls whether or not the URLRequestContext considers itself to be the
-  // "main" URLRequestContext.
-  bool is_main() const { return is_main_; }
-  void set_is_main(bool is_main) { is_main_ = is_main; }
+  const URLRequestJobFactory* job_factory() const { return job_factory_; }
+  void set_job_factory(const URLRequestJobFactory* job_factory) {
+    job_factory_ = job_factory;
+  }
 
  protected:
   friend class base::RefCountedThreadSafe<URLRequestContext>;
 
   virtual ~URLRequestContext();
 
-  // The following members are expected to be initialized and owned by
-  // subclasses.
+ private:
+  base::WeakPtrFactory<URLRequestContext> weak_factory_;
+
+  // ---------------------------------------------------------------------------
+  // Important: When adding any new members below, consider whether they need to
+  // be added to CopyFrom.
+  // ---------------------------------------------------------------------------
+
+  // Ownership for these members are not defined here. Clients should either
+  // provide storage elsewhere or have a subclass take ownership.
   NetLog* net_log_;
   HostResolver* host_resolver_;
   CertVerifier* cert_verifier_;
-  DnsRRResolver* dnsrr_resolver_;
-  scoped_ptr<DnsCertProvenanceChecker> dns_cert_checker_;
-  scoped_refptr<ProxyService> proxy_service_;
-  scoped_refptr<SSLConfigService> ssl_config_service_;
-  HttpTransactionFactory* http_transaction_factory_;
-  FtpTransactionFactory* ftp_transaction_factory_;
+  OriginBoundCertService* origin_bound_cert_service_;
+  FraudulentCertificateReporter* fraudulent_certificate_reporter_;
   HttpAuthHandlerFactory* http_auth_handler_factory_;
-  HttpNetworkDelegate* network_delegate_;
+  ProxyService* proxy_service_;
+  scoped_refptr<SSLConfigService> ssl_config_service_;
+  NetworkDelegate* network_delegate_;
+  HttpServerProperties* http_server_properties_;
   scoped_refptr<CookieStore> cookie_store_;
-  CookiePolicy* cookie_policy_;
-  scoped_refptr<TransportSecurityState> transport_security_state_;
-  FtpAuthCache ftp_auth_cache_;
+  TransportSecurityState* transport_security_state_;
+  scoped_ptr<FtpAuthCache> ftp_auth_cache_;
   std::string accept_language_;
   std::string accept_charset_;
   // The charset of the referrer where this request comes from. It's not
   // used in communication with a server but is used to construct a suggested
   // filename for file download.
   std::string referrer_charset_;
+  HttpTransactionFactory* http_transaction_factory_;
+  FtpTransactionFactory* ftp_transaction_factory_;
+  const URLRequestJobFactory* job_factory_;
 
- private:
-  // Indicates whether or not this is the main URLRequestContext.
-  bool is_main_;
+  // ---------------------------------------------------------------------------
+  // Important: When adding any new members below, consider whether they need to
+  // be added to CopyFrom.
+  // ---------------------------------------------------------------------------
 
   DISALLOW_COPY_AND_ASSIGN(URLRequestContext);
 };
