@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,23 +11,27 @@
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_simple_job.h"
 
+namespace net {
+
 class URLRequest;
 
 class URLRequestDataJob : public URLRequestSimpleJob {
  public:
   explicit URLRequestDataJob(URLRequest* request);
 
-  virtual bool GetData(std::string* mime_type,
-                       std::string* charset,
-                       std::string* data) const;
-
   static URLRequest::ProtocolFactory Factory;
 
+  // URLRequestSimpleJob
+  virtual bool GetData(std::string* mime_type,
+                       std::string* charset,
+                       std::string* data) const OVERRIDE;
+
  private:
-  ~URLRequestDataJob();
+  virtual ~URLRequestDataJob();
 
   DISALLOW_COPY_AND_ASSIGN(URLRequestDataJob);
 };
 
-#endif  // NET_URL_REQUEST_URL_REQUEST_DATA_JOB_H_
+}  // namespace net
 
+#endif  // NET_URL_REQUEST_URL_REQUEST_DATA_JOB_H_
