@@ -1,9 +1,10 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/url_request/url_request_file_dir_job.h"
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/file_util.h"
 #include "base/message_loop.h"
@@ -12,6 +13,7 @@
 #include "base/time.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/io_buffer.h"
+#include "net/base/net_errors.h"
 #include "net/base/net_util.h"
 #include "net/url_request/url_request.h"
 
@@ -24,18 +26,20 @@ namespace net {
 URLRequestFileDirJob::URLRequestFileDirJob(URLRequest* request,
                                            const FilePath& dir_path)
     : URLRequestJob(request),
+      ALLOW_THIS_IN_INITIALIZER_LIST(lister_(dir_path, this)),
       dir_path_(dir_path),
       canceled_(false),
       list_complete_(false),
       wrote_header_(false),
       read_pending_(false),
       read_buffer_length_(0),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {
 }
 
-URLRequestFileDirJob::~URLRequestFileDirJob() {
-  DCHECK(read_pending_ == false);
-  DCHECK(lister_ == NULL);
+void URLRequestFileDirJob::StartAsync() {
+  lister_.Start();
+
+  NotifyHeadersComplete();
 }
 
 void URLRequestFileDirJob::Start() {
@@ -43,24 +47,8 @@ void URLRequestFileDirJob::Start() {
   // callbacks happen as they would for network requests.
   MessageLoop::current()->PostTask(
       FROM_HERE,
-      method_factory_.NewRunnableMethod(
-          &URLRequestFileDirJob::StartAsync));
-}
-
-void URLRequestFileDirJob::StartAsync() {
-  DCHECK(!lister_);
-
-  // TODO(willchan): This is stupid.  We should tell |lister_| not to call us
-  // back.  Fix this stupidity.
-
-  // AddRef so that *this* cannot be destroyed while the lister_
-  // is trying to feed us data.
-
-  AddRef();
-  lister_ = new DirectoryLister(dir_path_, this);
-  lister_->Start();
-
-  NotifyHeadersComplete();
+      base::Bind(&URLRequestFileDirJob::StartAsync,
+                 weak_factory_.GetWeakPtr()));
 }
 
 void URLRequestFileDirJob::Kill() {
@@ -69,15 +57,12 @@ void URLRequestFileDirJob::Kill() {
 
   canceled_ = true;
 
-  // Don't call CloseLister or dispatch an error to the URLRequest because
-  // we want OnListDone to be called to also write the error to the output
-  // stream. OnListDone will notify the URLRequest at this time.
-  if (lister_)
-    lister_->Cancel();
+  if (!list_complete_)
+    lister_.Cancel();
 
   URLRequestJob::Kill();
 
-  method_factory_.RevokeAll();
+  weak_factory_.InvalidateWeakPtrs();
 }
 
 bool URLRequestFileDirJob::ReadRawData(IOBuffer* buf, int buf_size,
@@ -158,27 +143,35 @@ void URLRequestFileDirJob::OnListFile(
 }
 
 void URLRequestFileDirJob::OnListDone(int error) {
-  CloseLister();
-
-  if (canceled_) {
-    read_pending_ = false;
-    // No need for NotifyCanceled() since canceled_ is set inside Kill().
-  } else if (error) {
+  DCHECK(!canceled_);
+  if (error != OK) {
     read_pending_ = false;
     NotifyDone(URLRequestStatus(URLRequestStatus::FAILED, error));
   } else {
     list_complete_ = true;
     CompleteRead();
   }
-
-  Release();  // The Lister is finished; may delete *this*
 }
 
-void URLRequestFileDirJob::CloseLister() {
-  if (lister_) {
-    lister_->Cancel();
-    lister_->set_delegate(NULL);
-    lister_ = NULL;
+URLRequestFileDirJob::~URLRequestFileDirJob() {}
+
+void URLRequestFileDirJob::CompleteRead() {
+  if (read_pending_) {
+    int bytes_read;
+    if (FillReadBuffer(read_buffer_->data(), read_buffer_length_,
+                       &bytes_read)) {
+      // We completed the read, so reset the read buffer.
+      read_pending_ = false;
+      read_buffer_ = NULL;
+      read_buffer_length_ = 0;
+
+      SetStatus(URLRequestStatus());
+      NotifyReadComplete(bytes_read);
+    } else {
+      NOTREACHED();
+      // TODO: Better error code.
+      NotifyDone(URLRequestStatus(URLRequestStatus::FAILED, 0));
+    }
   }
 }
 
@@ -201,24 +194,4 @@ bool URLRequestFileDirJob::FillReadBuffer(char *buf, int buf_size,
   return false;
 }
 
-void URLRequestFileDirJob::CompleteRead() {
-  if (read_pending_) {
-    int bytes_read;
-    if (FillReadBuffer(read_buffer_->data(), read_buffer_length_,
-                       &bytes_read)) {
-      // We completed the read, so reset the read buffer.
-      read_pending_ = false;
-      read_buffer_ = NULL;
-      read_buffer_length_ = 0;
-
-      SetStatus(URLRequestStatus());
-      NotifyReadComplete(bytes_read);
-    } else {
-      NOTREACHED();
-      // TODO: Better error code.
-      NotifyDone(URLRequestStatus(URLRequestStatus::FAILED, 0));
-    }
-  }
-}
-
 }  // namespace net
