@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,7 +10,7 @@
 
 #include "base/file_path.h"
 #include "base/file_util.h"
-#include "base/task.h"
+#include "base/memory/weak_ptr.h"
 #include "net/base/directory_lister.h"
 #include "net/url_request/url_request_job.h"
 
@@ -22,25 +22,29 @@ class URLRequestFileDirJob
  public:
   URLRequestFileDirJob(URLRequest* request, const FilePath& dir_path);
 
-  // Overridden from URLRequestJob:
-  virtual void Start();
+  bool list_complete() const { return list_complete_; }
+
   virtual void StartAsync();
-  virtual void Kill();
-  virtual bool ReadRawData(IOBuffer* buf, int buf_size, int *bytes_read);
-  virtual bool GetMimeType(std::string* mime_type) const;
-  virtual bool GetCharset(std::string* charset);
+
+  // Overridden from URLRequestJob:
+  virtual void Start() OVERRIDE;
+  virtual void Kill() OVERRIDE;
+  virtual bool ReadRawData(IOBuffer* buf,
+                           int buf_size,
+                           int *bytes_read) OVERRIDE;
+  virtual bool GetMimeType(std::string* mime_type) const OVERRIDE;
+  virtual bool GetCharset(std::string* charset) OVERRIDE;
 
   // Overridden from DirectoryLister::DirectoryListerDelegate:
   virtual void OnListFile(
-      const DirectoryLister::DirectoryListerData& data);
-  virtual void OnListDone(int error);
-
-  bool list_complete() const { return list_complete_; }
+      const DirectoryLister::DirectoryListerData& data) OVERRIDE;
+  virtual void OnListDone(int error) OVERRIDE;
 
  private:
   virtual ~URLRequestFileDirJob();
 
   void CloseLister();
+
   // When we have data and a read has been pending, this function
   // will fill the response buffer and notify the request
   // appropriately.
@@ -49,7 +53,7 @@ class URLRequestFileDirJob
   // Fills a buffer with the output.
   bool FillReadBuffer(char *buf, int buf_size, int *bytes_read);
 
-  scoped_refptr<DirectoryLister> lister_;
+  DirectoryLister lister_;
   FilePath dir_path_;
   std::string data_;
   bool canceled_;
@@ -66,7 +70,7 @@ class URLRequestFileDirJob
   bool read_pending_;
   scoped_refptr<IOBuffer> read_buffer_;
   int read_buffer_length_;
-  ScopedRunnableMethodFactory<URLRequestFileDirJob> method_factory_;
+  base::WeakPtrFactory<URLRequestFileDirJob> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(URLRequestFileDirJob);
 };
