@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,6 +19,7 @@
 
 #include "net/url_request/url_request_file_job.h"
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/message_loop.h"
 #include "base/platform_file.h"
@@ -40,7 +41,6 @@
 
 namespace net {
 
-#if defined(OS_WIN)
 class URLRequestFileJob::AsyncResolver
     : public base::RefCountedThreadSafe<URLRequestFileJob::AsyncResolver> {
  public:
@@ -53,8 +53,9 @@ class URLRequestFileJob::AsyncResolver
     bool exists = file_util::GetFileInfo(file_path, &file_info);
     base::AutoLock locked(lock_);
     if (owner_loop_) {
-      owner_loop_->PostTask(FROM_HERE, NewRunnableMethod(
-          this, &AsyncResolver::ReturnResults, exists, file_info));
+      owner_loop_->PostTask(
+          FROM_HERE,
+          base::Bind(&AsyncResolver::ReturnResults, this, exists, file_info));
     }
   }
 
@@ -80,7 +81,14 @@ class URLRequestFileJob::AsyncResolver
   base::Lock lock_;
   MessageLoop* owner_loop_;
 };
-#endif
+
+URLRequestFileJob::URLRequestFileJob(URLRequest* request,
+                                     const FilePath& file_path)
+    : URLRequestJob(request),
+      file_path_(file_path),
+      is_directory_(false),
+      remaining_bytes_(0) {
+}
 
 // static
 URLRequestJob* URLRequestFileJob::Factory(URLRequest* request,
@@ -111,63 +119,52 @@ URLRequestJob* URLRequestFileJob::Factory(URLRequest* request,
   return new URLRequestFileJob(request, file_path);
 }
 
-URLRequestFileJob::URLRequestFileJob(URLRequest* request,
-                                     const FilePath& file_path)
-    : URLRequestJob(request),
-      file_path_(file_path),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          io_callback_(this, &URLRequestFileJob::DidRead)),
-      is_directory_(false),
-      remaining_bytes_(0),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
-}
+#if defined(OS_CHROMEOS)
+static const char* const kLocalAccessWhiteList[] = {
+  "/home/chronos/user/Downloads",
+  "/media",
+  "/opt/oem",
+  "/usr/share/chromeos-assets",
+  "/tmp",
+  "/var/log",
+};
 
-URLRequestFileJob::~URLRequestFileJob() {
-#if defined(OS_WIN)
-  DCHECK(!async_resolver_);
-#endif
+// static
+bool URLRequestFileJob::AccessDisabled(const FilePath& file_path) {
+  if (URLRequest::IsFileAccessAllowed()) {  // for tests.
+    return false;
+  }
+
+  for (size_t i = 0; i < arraysize(kLocalAccessWhiteList); ++i) {
+    const FilePath white_listed_path(kLocalAccessWhiteList[i]);
+    // FilePath::operator== should probably handle trailing seperators.
+    if (white_listed_path == file_path.StripTrailingSeparators() ||
+        white_listed_path.IsParent(file_path)) {
+      return false;
+    }
+  }
+  return true;
 }
+#endif  // OS_CHROMEOS
 
 void URLRequestFileJob::Start() {
-#if defined(OS_WIN)
-  // Resolve UNC paths on a background thread.
-  if (!file_path_.value().compare(0, 2, L"\\\\")) {
-    DCHECK(!async_resolver_);
-    async_resolver_ = new AsyncResolver(this);
-    base::WorkerPool::PostTask(FROM_HERE, NewRunnableMethod(
-        async_resolver_.get(), &AsyncResolver::Resolve, file_path_), true);
-    return;
-  }
-#endif
-
-  // URL requests should not block on the disk!
-  //   http://code.google.com/p/chromium/issues/detail?id=59849
-  bool exists;
-  base::PlatformFileInfo file_info;
-  {
-    base::ThreadRestrictions::ScopedAllowIO allow_io;
-    exists = file_util::GetFileInfo(file_path_, &file_info);
-  }
-
-  // Continue asynchronously.
-  MessageLoop::current()->PostTask(
+  DCHECK(!async_resolver_);
+  async_resolver_ = new AsyncResolver(this);
+  base::WorkerPool::PostTask(
       FROM_HERE,
-      method_factory_.NewRunnableMethod(
-          &URLRequestFileJob::DidResolve, exists, file_info));
+      base::Bind(&AsyncResolver::Resolve, async_resolver_.get(), file_path_),
+      true);
 }
 
 void URLRequestFileJob::Kill() {
   stream_.Close();
 
-#if defined(OS_WIN)
   if (async_resolver_) {
     async_resolver_->Cancel();
     async_resolver_ = NULL;
   }
-#endif
 
   URLRequestJob::Kill();
-  method_factory_.RevokeAll();
 }
 
 bool URLRequestFileJob::ReadRawData(IOBuffer* dest, int dest_size,
@@ -186,7 +183,9 @@ bool URLRequestFileJob::ReadRawData(IOBuffer* dest, int dest_size,
     return true;
   }
 
-  int rv = stream_.Read(dest->data(), dest_size, &io_callback_);
+  int rv = stream_.Read(dest->data(), dest_size,
+                        base::Bind(&URLRequestFileJob::DidRead,
+                                   base::Unretained(this)));
   if (rv >= 0) {
     // Data is immediately available.
     *bytes_read = rv;
@@ -204,15 +203,47 @@ bool URLRequestFileJob::ReadRawData(IOBuffer* dest, int dest_size,
   return false;
 }
 
-bool URLRequestFileJob::GetContentEncodings(
-    std::vector<Filter::FilterType>* encoding_types) {
-  DCHECK(encoding_types->empty());
+bool URLRequestFileJob::IsRedirectResponse(GURL* location,
+                                           int* http_status_code) {
+  if (is_directory_) {
+    // This happens when we discovered the file is a directory, so needs a
+    // slash at the end of the path.
+    std::string new_path = request_->url().path();
+    new_path.push_back('/');
+    GURL::Replacements replacements;
+    replacements.SetPathStr(new_path);
 
+    *location = request_->url().ReplaceComponents(replacements);
+    *http_status_code = 301;  // simulate a permanent redirect
+    return true;
+  }
+
+#if defined(OS_WIN)
+  // Follow a Windows shortcut.
+  // We just resolve .lnk file, ignore others.
+  if (!LowerCaseEqualsASCII(file_path_.Extension(), ".lnk"))
+    return false;
+
+  FilePath new_path = file_path_;
+  bool resolved;
+  resolved = file_util::ResolveShortcut(&new_path);
+
+  // If shortcut is not resolved succesfully, do not redirect.
+  if (!resolved)
+    return false;
+
+  *location = FilePathToFileURL(new_path);
+  *http_status_code = 301;
+  return true;
+#else
+  return false;
+#endif
+}
+
+Filter* URLRequestFileJob::SetupFilter() const {
   // Bug 9936 - .svgz files needs to be decompressed.
-  if (LowerCaseEqualsASCII(file_path_.Extension(), ".svgz"))
-    encoding_types->push_back(Filter::FILTER_TYPE_GZIP);
-
-  return !encoding_types->empty();
+  return LowerCaseEqualsASCII(file_path_.Extension(), ".svgz")
+      ? Filter::GZipFactory() : NULL;
 }
 
 bool URLRequestFileJob::GetMimeType(std::string* mime_type) const {
@@ -245,11 +276,13 @@ void URLRequestFileJob::SetExtraRequestHeaders(
   }
 }
 
+URLRequestFileJob::~URLRequestFileJob() {
+  DCHECK(!async_resolver_);
+}
+
 void URLRequestFileJob::DidResolve(
     bool exists, const base::PlatformFileInfo& file_info) {
-#if defined(OS_WIN)
   async_resolver_ = NULL;
-#endif
 
   // We may have been orphaned...
   if (!request_)
@@ -294,14 +327,19 @@ void URLRequestFileJob::DidResolve(
                      byte_range_.first_byte_position() + 1;
   DCHECK_GE(remaining_bytes_, 0);
 
-  // Do the seek at the beginning of the request.
-  if (remaining_bytes_ > 0 &&
-      byte_range_.first_byte_position() != 0 &&
-      byte_range_.first_byte_position() !=
-          stream_.Seek(FROM_BEGIN, byte_range_.first_byte_position())) {
-    NotifyDone(URLRequestStatus(URLRequestStatus::FAILED,
-               ERR_REQUEST_RANGE_NOT_SATISFIABLE));
-    return;
+  // URL requests should not block on the disk!
+  //   http://code.google.com/p/chromium/issues/detail?id=59849
+  {
+    base::ThreadRestrictions::ScopedAllowIO allow_io;
+    // Do the seek at the beginning of the request.
+    if (remaining_bytes_ > 0 &&
+        byte_range_.first_byte_position() != 0 &&
+        byte_range_.first_byte_position() !=
+        stream_.Seek(FROM_BEGIN, byte_range_.first_byte_position())) {
+      NotifyDone(URLRequestStatus(URLRequestStatus::FAILED,
+                                  ERR_REQUEST_RANGE_NOT_SATISFIABLE));
+      return;
+    }
   }
 
   set_expected_content_size(remaining_bytes_);
@@ -323,69 +361,4 @@ void URLRequestFileJob::DidRead(int result) {
   NotifyReadComplete(result);
 }
 
-bool URLRequestFileJob::IsRedirectResponse(GURL* location,
-                                           int* http_status_code) {
-  if (is_directory_) {
-    // This happens when we discovered the file is a directory, so needs a
-    // slash at the end of the path.
-    std::string new_path = request_->url().path();
-    new_path.push_back('/');
-    GURL::Replacements replacements;
-    replacements.SetPathStr(new_path);
-
-    *location = request_->url().ReplaceComponents(replacements);
-    *http_status_code = 301;  // simulate a permanent redirect
-    return true;
-  }
-
-#if defined(OS_WIN)
-  // Follow a Windows shortcut.
-  // We just resolve .lnk file, ignore others.
-  if (!LowerCaseEqualsASCII(file_path_.Extension(), ".lnk"))
-    return false;
-
-  FilePath new_path = file_path_;
-  bool resolved;
-  resolved = file_util::ResolveShortcut(&new_path);
-
-  // If shortcut is not resolved succesfully, do not redirect.
-  if (!resolved)
-    return false;
-
-  *location = FilePathToFileURL(new_path);
-  *http_status_code = 301;
-  return true;
-#else
-  return false;
-#endif
-}
-
-#if defined(OS_CHROMEOS)
-static const char* const kLocalAccessWhiteList[] = {
-  "/home/chronos/user/Downloads",
-  "/media",
-  "/mnt/partner_partition",
-  "/usr/share/chromeos-assets",
-  "/tmp",
-  "/var/log",
-};
-
-// static
-bool URLRequestFileJob::AccessDisabled(const FilePath& file_path) {
-  if (URLRequest::IsFileAccessAllowed()) {  // for tests.
-    return false;
-  }
-
-  for (size_t i = 0; i < arraysize(kLocalAccessWhiteList); ++i) {
-    const FilePath white_listed_path(kLocalAccessWhiteList[i]);
-    // FilePath::operator== should probably handle trailing seperators.
-    if (white_listed_path == file_path.StripTrailingSeparators() ||
-        white_listed_path.IsParent(file_path)) {
-      return false;
-    }
-  }
-  return true;
-}
-#endif
-
 }  // namespace net
