@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,7 @@
 #include "base/message_loop.h"
 #include "base/utf_string_conversions.h"
 #include "net/base/auth.h"
+#include "net/base/host_port_pair.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_util.h"
 #include "net/ftp/ftp_response_info.h"
@@ -20,13 +21,9 @@ namespace net {
 
 URLRequestFtpJob::URLRequestFtpJob(URLRequest* request)
     : URLRequestJob(request),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          start_callback_(this, &URLRequestFtpJob::OnStartCompleted)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          read_callback_(this, &URLRequestFtpJob::OnReadCompleted)),
       read_in_progress_(false),
       context_(request->context()),
-      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {
 }
 
 // static
@@ -52,6 +49,13 @@ bool URLRequestFtpJob::GetMimeType(std::string* mime_type) const {
   return false;
 }
 
+HostPortPair URLRequestFtpJob::GetSocketAddress() const {
+  if (!transaction_.get()) {
+    return HostPortPair();
+  }
+  return transaction_->GetResponseInfo()->socket_address;
+}
+
 URLRequestFtpJob::~URLRequestFtpJob() {
 }
 
@@ -70,7 +74,10 @@ void URLRequestFtpJob::StartTransaction() {
   int rv;
   if (transaction_.get()) {
     rv = transaction_->Start(
-        &request_info_, &start_callback_, request_->net_log());
+        &request_info_,
+        base::Bind(&URLRequestFtpJob::OnStartCompleted,
+                   base::Unretained(this)),
+        request_->net_log());
     if (rv == ERR_IO_PENDING)
       return;
   } else {
@@ -80,8 +87,8 @@ void URLRequestFtpJob::StartTransaction() {
   // URLRequest delegate via the message loop.
   MessageLoop::current()->PostTask(
       FROM_HERE,
-      method_factory_.NewRunnableMethod(
-          &URLRequestFtpJob::OnStartCompleted, rv));
+      base::Bind(&URLRequestFtpJob::OnStartCompleted,
+                 weak_factory_.GetWeakPtr(), rv));
 }
 
 void URLRequestFtpJob::OnStartCompleted(int result) {
@@ -98,9 +105,8 @@ void URLRequestFtpJob::OnStartCompleted(int result) {
   } else if (transaction_->GetResponseInfo()->needs_auth) {
     GURL origin = request_->url().GetOrigin();
     if (server_auth_ && server_auth_->state == AUTH_STATE_HAVE_AUTH) {
-      request_->context()->ftp_auth_cache()->Remove(origin,
-                                                    server_auth_->username,
-                                                    server_auth_->password);
+      request_->context()->ftp_auth_cache()->Remove(
+          origin, server_auth_->credentials);
     } else if (!server_auth_) {
       server_auth_ = new AuthData();
     }
@@ -111,7 +117,7 @@ void URLRequestFtpJob::OnStartCompleted(int result) {
 
     if (cached_auth) {
       // Retry using cached auth data.
-      SetAuth(cached_auth->username, cached_auth->password);
+      SetAuth(cached_auth->credentials);
     } else {
       // Prompt for a username/password.
       NotifyHeadersComplete();
@@ -141,14 +147,17 @@ void URLRequestFtpJob::RestartTransactionWithAuth() {
   // be notifying our consumer asynchronously via OnStartCompleted.
   SetStatus(URLRequestStatus(URLRequestStatus::IO_PENDING, 0));
 
-  int rv = transaction_->RestartWithAuth(server_auth_->username,
-                                         server_auth_->password,
-                                         &start_callback_);
+  int rv = transaction_->RestartWithAuth(
+      server_auth_->credentials,
+      base::Bind(&URLRequestFtpJob::OnStartCompleted,
+                 base::Unretained(this)));
   if (rv == ERR_IO_PENDING)
     return;
 
-  MessageLoop::current()->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &URLRequestFtpJob::OnStartCompleted, rv));
+  MessageLoop::current()->PostTask(
+      FROM_HERE,
+      base::Bind(&URLRequestFtpJob::OnStartCompleted,
+                 weak_factory_.GetWeakPtr(), rv));
 }
 
 void URLRequestFtpJob::Start() {
@@ -162,7 +171,7 @@ void URLRequestFtpJob::Kill() {
     return;
   transaction_.reset();
   URLRequestJob::Kill();
-  method_factory_.RevokeAll();
+  weak_factory_.InvalidateWeakPtrs();
 }
 
 LoadState URLRequestFtpJob::GetLoadState() const {
@@ -184,22 +193,20 @@ void URLRequestFtpJob::GetAuthChallengeInfo(
          (server_auth_->state == AUTH_STATE_NEED_AUTH));
   scoped_refptr<AuthChallengeInfo> auth_info(new AuthChallengeInfo);
   auth_info->is_proxy = false;
-  auth_info->host_and_port = ASCIIToWide(
-      GetHostAndPort(request_->url()));
-  auth_info->scheme = L"";
-  auth_info->realm = L"";
+  auth_info->challenger = HostPortPair::FromURL(request_->url());
+  // scheme and realm are kept empty.
+  DCHECK(auth_info->scheme.empty());
+  DCHECK(auth_info->realm.empty());
   result->swap(auth_info);
 }
 
-void URLRequestFtpJob::SetAuth(const string16& username,
-                               const string16& password) {
+void URLRequestFtpJob::SetAuth(const AuthCredentials& credentials) {
   DCHECK(NeedsAuth());
   server_auth_->state = AUTH_STATE_HAVE_AUTH;
-  server_auth_->username = username;
-  server_auth_->password = password;
+  server_auth_->credentials = credentials;
 
   request_->context()->ftp_auth_cache()->Add(request_->url().GetOrigin(),
-                                             username, password);
+                                             server_auth_->credentials);
 
   RestartTransactionWithAuth();
 }
@@ -213,8 +220,8 @@ void URLRequestFtpJob::CancelAuth() {
   // any recursing into the caller as a result of this call.
   MessageLoop::current()->PostTask(
       FROM_HERE,
-      method_factory_.NewRunnableMethod(
-          &URLRequestFtpJob::OnStartCompleted, OK));
+      base::Bind(&URLRequestFtpJob::OnStartCompleted,
+                 weak_factory_.GetWeakPtr(), OK));
 }
 
 uint64 URLRequestFtpJob::GetUploadProgress() const {
@@ -228,7 +235,9 @@ bool URLRequestFtpJob::ReadRawData(IOBuffer* buf,
   DCHECK(bytes_read);
   DCHECK(!read_in_progress_);
 
-  int rv = transaction_->Read(buf, buf_size, &read_callback_);
+  int rv = transaction_->Read(buf, buf_size,
+                              base::Bind(&URLRequestFtpJob::OnReadCompleted,
+                                         base::Unretained(this)));
   if (rv >= 0) {
     *bytes_read = rv;
     return true;
