@@ -8,8 +8,7 @@
 
 #include <string>
 
-#include "base/string16.h"
-#include "base/task.h"
+#include "base/memory/weak_ptr.h"
 #include "net/base/auth.h"
 #include "net/base/completion_callback.h"
 #include "net/ftp/ftp_request_info.h"
@@ -30,7 +29,8 @@ class URLRequestFtpJob : public URLRequestJob {
                                 const std::string& scheme);
 
   // Overridden from URLRequestJob:
-  virtual bool GetMimeType(std::string* mime_type) const;
+  virtual bool GetMimeType(std::string* mime_type) const OVERRIDE;
+  virtual HostPortPair GetSocketAddress() const OVERRIDE;
 
  private:
   virtual ~URLRequestFtpJob();
@@ -45,35 +45,33 @@ class URLRequestFtpJob : public URLRequestJob {
   void LogFtpServerType(char server_type);
 
   // Overridden from URLRequestJob:
-  virtual void Start();
-  virtual void Kill();
-  virtual LoadState GetLoadState() const;
-  virtual bool NeedsAuth();
+  virtual void Start() OVERRIDE;
+  virtual void Kill() OVERRIDE;
+  virtual LoadState GetLoadState() const OVERRIDE;
+  virtual bool NeedsAuth() OVERRIDE;
   virtual void GetAuthChallengeInfo(
-      scoped_refptr<AuthChallengeInfo>* auth_info);
-  virtual void SetAuth(const string16& username,
-                       const string16& password);
-  virtual void CancelAuth();
+      scoped_refptr<AuthChallengeInfo>* auth_info) OVERRIDE;
+  virtual void SetAuth(const AuthCredentials& credentials) OVERRIDE;
+  virtual void CancelAuth() OVERRIDE;
 
   // TODO(ibrar):  Yet to give another look at this function.
-  virtual uint64 GetUploadProgress() const;
-  virtual bool ReadRawData(IOBuffer* buf, int buf_size, int *bytes_read);
+  virtual uint64 GetUploadProgress() const OVERRIDE;
+  virtual bool ReadRawData(IOBuffer* buf,
+                           int buf_size,
+                           int *bytes_read) OVERRIDE;
 
   FtpRequestInfo request_info_;
   scoped_ptr<FtpTransaction> transaction_;
 
-  CompletionCallbackImpl<URLRequestFtpJob> start_callback_;
-  CompletionCallbackImpl<URLRequestFtpJob> read_callback_;
-
   bool read_in_progress_;
 
   scoped_refptr<AuthData> server_auth_;
 
   // Keep a reference to the url request context to be sure it's not deleted
   // before us.
-  scoped_refptr<URLRequestContext> context_;
+  scoped_refptr<const URLRequestContext> context_;
 
-  ScopedRunnableMethodFactory<URLRequestFtpJob> method_factory_;
+  base::WeakPtrFactory<URLRequestFtpJob> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(URLRequestFtpJob);
 };
