@@ -1,66 +1,51 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "net/url_request/url_request_job.h"
 
+#include "base/bind.h"
+#include "base/compiler_specific.h"
 #include "base/message_loop.h"
-#include "base/metrics/histogram.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
 #include "net/base/auth.h"
+#include "net/base/host_port_pair.h"
 #include "net/base/io_buffer.h"
-#include "net/base/load_flags.h"
-#include "net/base/mime_util.h"
+#include "net/base/load_states.h"
 #include "net/base/net_errors.h"
+#include "net/base/network_delegate.h"
 #include "net/http/http_response_headers.h"
 #include "net/url_request/url_request.h"
-#include "net/url_request/url_request_job_metrics.h"
-#include "net/url_request/url_request_job_tracker.h"
-
-using base::Time;
-using base::TimeTicks;
+#include "net/url_request/url_request_context.h"
 
 namespace net {
 
-// Buffer size allocated when de-compressing data.
-// static
-const int URLRequestJob::kFilterBufSize = 32 * 1024;
-
 URLRequestJob::URLRequestJob(URLRequest* request)
     : request_(request),
+      done_(false),
       prefilter_bytes_read_(0),
       postfilter_bytes_read_(0),
-      is_compressible_content_(false),
-      is_compressed_(false),
-      done_(false),
+      filter_input_byte_count_(0),
       filter_needs_more_output_space_(false),
       filtered_read_buffer_len_(0),
       has_handled_response_(false),
       expected_content_size_(-1),
       deferred_redirect_status_code_(-1),
-      packet_timing_enabled_(false),
-      filter_input_byte_count_(0),
-      bytes_observed_in_packets_(0),
-      max_packets_timed_(0),
-      observed_packet_count_(0) {
-  load_flags_ = request_->load_flags();
-  is_profiling_ = request->enable_profiling();
-  if (is_profiling()) {
-    metrics_.reset(new URLRequestJobMetrics());
-    metrics_->start_time_ = TimeTicks::Now();
-  }
-  g_url_request_job_tracker.AddNewJob(this);
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_factory_(this)) {
+  base::SystemMonitor* system_monitor = base::SystemMonitor::Get();
+  if (system_monitor)
+    base::SystemMonitor::Get()->AddPowerObserver(this);
 }
 
-void URLRequestJob::SetUpload(net::UploadData* upload) {
+void URLRequestJob::SetUpload(UploadData* upload) {
 }
 
-void URLRequestJob::SetExtraRequestHeaders(
-    const net::HttpRequestHeaders& headers) {
+void URLRequestJob::SetExtraRequestHeaders(const HttpRequestHeaders& headers) {
 }
 
 void URLRequestJob::Kill() {
+  weak_factory_.InvalidateWeakPtrs();
   // Make sure the request is notified that we are done.  We assume that the
   // request took care of setting its error status before calling Kill.
   if (request_)
@@ -74,10 +59,10 @@ void URLRequestJob::DetachRequest() {
 // This function calls ReadData to get stream data. If a filter exists, passes
 // the data to the attached filter. Then returns the output from filter back to
 // the caller.
-bool URLRequestJob::Read(net::IOBuffer* buf, int buf_size, int *bytes_read) {
+bool URLRequestJob::Read(IOBuffer* buf, int buf_size, int *bytes_read) {
   bool rv = false;
 
-  DCHECK_LT(buf_size, 1000000);  // sanity check
+  DCHECK_LT(buf_size, 1000000);  // Sanity check.
   DCHECK(buf);
   DCHECK(bytes_read);
   DCHECK(filtered_read_buffer_ == NULL);
@@ -85,7 +70,7 @@ bool URLRequestJob::Read(net::IOBuffer* buf, int buf_size, int *bytes_read) {
 
   *bytes_read = 0;
 
-  // Skip Filter if not present
+  // Skip Filter if not present.
   if (!filter_.get()) {
     rv = ReadRawDataHelper(buf, buf_size, bytes_read);
   } else {
@@ -95,9 +80,15 @@ bool URLRequestJob::Read(net::IOBuffer* buf, int buf_size, int *bytes_read) {
     filtered_read_buffer_len_ = buf_size;
 
     if (ReadFilteredData(bytes_read)) {
-      rv = true;   // we have data to return
+      rv = true;   // We have data to return.
+
+      // It is fine to call DoneReading even if ReadFilteredData receives 0
+      // bytes from the net, but we avoid making that call if we know for
+      // sure that's the case (ReadRawDataHelper path).
+      if (*bytes_read == 0)
+        DoneReading();
     } else {
-      rv = false;  // error, or a new IO is pending
+      rv = false;  // Error, or a new IO is pending.
     }
   }
   if (rv && *bytes_read == 0)
@@ -109,8 +100,8 @@ void URLRequestJob::StopCaching() {
   // Nothing to do here.
 }
 
-net::LoadState URLRequestJob::GetLoadState() const {
-  return net::LOAD_STATE_IDLE;
+LoadState URLRequestJob::GetLoadState() const {
+  return LOAD_STATE_IDLE;
 }
 
 uint64 URLRequestJob::GetUploadProgress() const {
@@ -121,29 +112,21 @@ bool URLRequestJob::GetCharset(std::string* charset) {
   return false;
 }
 
-void URLRequestJob::GetResponseInfo(net::HttpResponseInfo* info) {
+void URLRequestJob::GetResponseInfo(HttpResponseInfo* info) {
 }
 
 bool URLRequestJob::GetResponseCookies(std::vector<std::string>* cookies) {
   return false;
 }
 
-bool URLRequestJob::GetContentEncodings(
-    std::vector<Filter::FilterType>* encoding_types) {
-  return false;
-}
-
-void URLRequestJob::SetupFilter() {
-  std::vector<Filter::FilterType> encoding_types;
-  if (GetContentEncodings(&encoding_types)) {
-    filter_.reset(Filter::Factory(encoding_types, *this));
-  }
+Filter* URLRequestJob::SetupFilter() const {
+  return NULL;
 }
 
 bool URLRequestJob::IsRedirectResponse(GURL* location,
                                        int* http_status_code) {
   // For non-HTTP jobs, headers will be null.
-  net::HttpResponseHeaders* headers = request_->response_headers();
+  HttpResponseHeaders* headers = request_->response_headers();
   if (!headers)
     return false;
 
@@ -165,14 +148,13 @@ bool URLRequestJob::NeedsAuth() {
 }
 
 void URLRequestJob::GetAuthChallengeInfo(
-    scoped_refptr<net::AuthChallengeInfo>* auth_info) {
+    scoped_refptr<AuthChallengeInfo>* auth_info) {
   // This will only be called if NeedsAuth() returns true, in which
   // case the derived class should implement this!
   NOTREACHED();
 }
 
-void URLRequestJob::SetAuth(const string16& username,
-                            const string16& password) {
+void URLRequestJob::SetAuth(const AuthCredentials& credentials) {
   // This will only be called if NeedsAuth() returns true, in which
   // case the derived class should implement this!
   NOTREACHED();
@@ -185,7 +167,7 @@ void URLRequestJob::CancelAuth() {
 }
 
 void URLRequestJob::ContinueWithCertificate(
-    net::X509Certificate* client_cert) {
+    X509Certificate* client_cert) {
   // The derived class should implement this!
   NOTREACHED();
 }
@@ -216,194 +198,64 @@ void URLRequestJob::FollowDeferredRedirect() {
   FollowRedirect(redirect_url, redirect_status_code);
 }
 
-URLRequestJobMetrics* URLRequestJob::RetrieveMetrics() {
-  if (is_profiling())
-    return metrics_.release();
-  else
-    return NULL;
-}
-
 bool URLRequestJob::GetMimeType(std::string* mime_type) const {
   return false;
 }
 
-bool URLRequestJob::GetURL(GURL* gurl) const {
-  if (!request_)
-    return false;
-  *gurl = request_->url();
-  return true;
-}
-
-base::Time URLRequestJob::GetRequestTime() const {
-  if (!request_)
-    return base::Time();
-  return request_->request_time();
-}
-
-bool URLRequestJob::IsDownload() const {
-  return (load_flags_ & net::LOAD_IS_DOWNLOAD) != 0;
-}
-
-bool URLRequestJob::IsSdchResponse() const {
-  return false;
-}
-
-bool URLRequestJob::IsCachedContent() const {
-  return false;
-}
-
-int64 URLRequestJob::GetByteReadCount() const {
-  return filter_input_byte_count_;
-}
-
 int URLRequestJob::GetResponseCode() const {
   return -1;
 }
 
-int URLRequestJob::GetInputStreamBufferSize() const {
-  return kFilterBufSize;
+HostPortPair URLRequestJob::GetSocketAddress() const {
+  return HostPortPair();
 }
 
-void URLRequestJob::RecordPacketStats(StatisticSelector statistic) const {
-  if (!packet_timing_enabled_ || (final_packet_time_ == base::Time()))
-    return;
+void URLRequestJob::OnSuspend() {
+  Kill();
+}
 
-  // Caller should verify that we're not cached content, but we can't always
-  // really check for it here because we may (at destruction time) call our own
-  // class method and get a bogus const answer of false. This DCHECK only helps
-  // when this method has a valid overridden definition.
-  DCHECK(!IsCachedContent());
-
-  base::TimeDelta duration = final_packet_time_ - request_time_snapshot_;
-  switch (statistic) {
-    case SDCH_DECODE: {
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Decode_Latency_F_a", duration,
-                                  base::TimeDelta::FromMilliseconds(20),
-                                  base::TimeDelta::FromMinutes(10), 100);
-      UMA_HISTOGRAM_COUNTS_100("Sdch3.Network_Decode_Packets_b",
-                               static_cast<int>(observed_packet_count_));
-      UMA_HISTOGRAM_CUSTOM_COUNTS("Sdch3.Network_Decode_Bytes_Processed_b",
-          static_cast<int>(bytes_observed_in_packets_), 500, 100000, 100);
-      if (packet_times_.empty())
-        return;
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Decode_1st_To_Last_a",
-                                  final_packet_time_ - packet_times_[0],
-                                  base::TimeDelta::FromMilliseconds(20),
-                                  base::TimeDelta::FromMinutes(10), 100);
-
-      DCHECK(max_packets_timed_ >= kSdchPacketHistogramCount);
-      DCHECK(kSdchPacketHistogramCount > 4);
-      if (packet_times_.size() <= 4)
-        return;
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Decode_1st_To_2nd_c",
-                                  packet_times_[1] - packet_times_[0],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Decode_2nd_To_3rd_c",
-                                  packet_times_[2] - packet_times_[1],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Decode_3rd_To_4th_c",
-                                  packet_times_[3] - packet_times_[2],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Decode_4th_To_5th_c",
-                                  packet_times_[4] - packet_times_[3],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      return;
-    }
-    case SDCH_PASSTHROUGH: {
-      // Despite advertising a dictionary, we handled non-sdch compressed
-      // content.
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Pass-through_Latency_F_a",
-                                  duration,
-                                  base::TimeDelta::FromMilliseconds(20),
-                                  base::TimeDelta::FromMinutes(10), 100);
-      UMA_HISTOGRAM_COUNTS_100("Sdch3.Network_Pass-through_Packets_b",
-                               observed_packet_count_);
-      if (packet_times_.empty())
-        return;
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Pass-through_1st_To_Last_a",
-                                  final_packet_time_ - packet_times_[0],
-                                  base::TimeDelta::FromMilliseconds(20),
-                                  base::TimeDelta::FromMinutes(10), 100);
-      DCHECK(max_packets_timed_ >= kSdchPacketHistogramCount);
-      DCHECK(kSdchPacketHistogramCount > 4);
-      if (packet_times_.size() <= 4)
-        return;
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Pass-through_1st_To_2nd_c",
-                                  packet_times_[1] - packet_times_[0],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Pass-through_2nd_To_3rd_c",
-                                  packet_times_[2] - packet_times_[1],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Pass-through_3rd_To_4th_c",
-                                  packet_times_[3] - packet_times_[2],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Network_Pass-through_4th_To_5th_c",
-                                  packet_times_[4] - packet_times_[3],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      return;
-    }
-
-    case SDCH_EXPERIMENT_DECODE: {
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Experiment_Decode",
-                                  duration,
-                                  base::TimeDelta::FromMilliseconds(20),
-                                  base::TimeDelta::FromMinutes(10), 100);
-      // We already provided interpacket histograms above in the SDCH_DECODE
-      // case, so we don't need them here.
-      return;
-    }
-    case SDCH_EXPERIMENT_HOLDBACK: {
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Experiment_Holdback",
-                                  duration,
-                                  base::TimeDelta::FromMilliseconds(20),
-                                  base::TimeDelta::FromMinutes(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Experiment_Holdback_1st_To_Last_a",
-                                  final_packet_time_ - packet_times_[0],
-                                  base::TimeDelta::FromMilliseconds(20),
-                                  base::TimeDelta::FromMinutes(10), 100);
-
-      DCHECK(max_packets_timed_ >= kSdchPacketHistogramCount);
-      DCHECK(kSdchPacketHistogramCount > 4);
-      if (packet_times_.size() <= 4)
-        return;
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Experiment_Holdback_1st_To_2nd_c",
-                                  packet_times_[1] - packet_times_[0],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Experiment_Holdback_2nd_To_3rd_c",
-                                  packet_times_[2] - packet_times_[1],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Experiment_Holdback_3rd_To_4th_c",
-                                  packet_times_[3] - packet_times_[2],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      UMA_HISTOGRAM_CLIPPED_TIMES("Sdch3.Experiment_Holdback_4th_To_5th_c",
-                                  packet_times_[4] - packet_times_[3],
-                                  base::TimeDelta::FromMilliseconds(1),
-                                  base::TimeDelta::FromSeconds(10), 100);
-      return;
-    }
-    default:
-      NOTREACHED();
-      return;
-  }
+void URLRequestJob::NotifyURLRequestDestroyed() {
 }
 
 URLRequestJob::~URLRequestJob() {
-  g_url_request_job_tracker.RemoveJob(this);
+  base::SystemMonitor* system_monitor = base::SystemMonitor::Get();
+  if (system_monitor)
+    base::SystemMonitor::Get()->RemovePowerObserver(this);
+}
+
+void URLRequestJob::NotifyCertificateRequested(
+    SSLCertRequestInfo* cert_request_info) {
+  if (!request_)
+    return;  // The request was destroyed, so there is no more work to do.
+
+  request_->NotifyCertificateRequested(cert_request_info);
+}
+
+void URLRequestJob::NotifySSLCertificateError(const SSLInfo& ssl_info,
+                                              bool fatal) {
+  if (!request_)
+    return;  // The request was destroyed, so there is no more work to do.
+
+  request_->NotifySSLCertificateError(ssl_info, fatal);
+}
+
+bool URLRequestJob::CanGetCookies(const CookieList& cookie_list) const {
+  if (!request_)
+    return false;  // The request was destroyed, so there is no more work to do.
+
+  return request_->CanGetCookies(cookie_list);
+}
+
+bool URLRequestJob::CanSetCookie(const std::string& cookie_line,
+                                 CookieOptions* options) const {
+  if (!request_)
+    return false;  // The request was destroyed, so there is no more work to do.
+
+  return request_->CanSetCookie(cookie_line, options);
 }
 
 void URLRequestJob::NotifyHeadersComplete() {
-  if (!request_ || !request_->delegate())
+  if (!request_ || !request_->has_delegate())
     return;  // The request was destroyed, so there is no more work to do.
 
   if (has_handled_response_)
@@ -413,8 +265,8 @@ void URLRequestJob::NotifyHeadersComplete() {
 
   // Initialize to the current time, and let the subclass optionally override
   // the time stamps if it has that information.  The default request_time is
-  // set by net::URLRequest before it calls our Start method.
-  request_->response_info_.response_time = Time::Now();
+  // set by URLRequest before it calls our Start method.
+  request_->response_info_.response_time = base::Time::Now();
   GetResponseInfo(&request_->response_info_);
 
   // When notifying the delegate, the delegate can release the request
@@ -441,10 +293,11 @@ void URLRequestJob::NotifyHeadersComplete() {
     }
 
     bool defer_redirect = false;
-    request_->ReceivedRedirect(new_location, &defer_redirect);
+    request_->NotifyReceivedRedirect(new_location, &defer_redirect);
 
-    // Ensure that the request wasn't detached or destroyed in ReceivedRedirect
-    if (!request_ || !request_->delegate())
+    // Ensure that the request wasn't detached or destroyed in
+    // NotifyReceivedRedirect
+    if (!request_ || !request_->has_delegate())
       return;
 
     // If we were not cancelled, then maybe follow the redirect.
@@ -458,47 +311,33 @@ void URLRequestJob::NotifyHeadersComplete() {
       return;
     }
   } else if (NeedsAuth()) {
-    scoped_refptr<net::AuthChallengeInfo> auth_info;
+    scoped_refptr<AuthChallengeInfo> auth_info;
     GetAuthChallengeInfo(&auth_info);
     // Need to check for a NULL auth_info because the server may have failed
     // to send a challenge with the 401 response.
     if (auth_info) {
-      request_->delegate()->OnAuthRequired(request_, auth_info);
+      request_->NotifyAuthRequired(auth_info);
       // Wait for SetAuth or CancelAuth to be called.
       return;
     }
   }
 
   has_handled_response_ = true;
-  if (request_->status().is_success()) {
-    SetupFilter();
-
-    // Check if this content appears to be compressible.
-    std::string mime_type;
-    if (GetMimeType(&mime_type) &&
-        (net::IsSupportedJavascriptMimeType(mime_type.c_str()) ||
-        net::IsSupportedNonImageMimeType(mime_type.c_str()))) {
-      is_compressible_content_ = true;
-    }
-  }
+  if (request_->status().is_success())
+    filter_.reset(SetupFilter());
 
   if (!filter_.get()) {
     std::string content_length;
     request_->GetResponseHeaderByName("content-length", &content_length);
     if (!content_length.empty())
       base::StringToInt64(content_length, &expected_content_size_);
-  } else {
-    // Chrome today only sends "Accept-Encoding" for compression schemes.
-    // So, if there is a filter on the response, we know that the content
-    // was compressed.
-    is_compressed_ = true;
   }
 
-  request_->ResponseStarted();
+  request_->NotifyResponseStarted();
 }
 
 void URLRequestJob::NotifyReadComplete(int bytes_read) {
-  if (!request_ || !request_->delegate())
+  if (!request_ || !request_->has_delegate())
     return;  // The request was destroyed, so there is no more work to do.
 
   // TODO(darin): Bug 1004233. Re-enable this test once all of the chrome
@@ -515,13 +354,12 @@ void URLRequestJob::NotifyReadComplete(int bytes_read) {
     return;
 
   // When notifying the delegate, the delegate can release the request
-  // (and thus release 'this').  After calling to the delgate, we must
+  // (and thus release 'this').  After calling to the delegate, we must
   // check the request pointer to see if it still exists, and return
   // immediately if it has been destroyed.  self_preservation ensures our
   // survival until we can get out of this method.
   scoped_refptr<URLRequestJob> self_preservation(this);
 
-  prefilter_bytes_read_ += bytes_read;
   if (filter_.get()) {
     // Tell the filter that it has more data
     FilteredDataRead(bytes_read);
@@ -529,13 +367,18 @@ void URLRequestJob::NotifyReadComplete(int bytes_read) {
     // Filter the data.
     int filter_bytes_read = 0;
     if (ReadFilteredData(&filter_bytes_read)) {
-      postfilter_bytes_read_ += filter_bytes_read;
-      request_->delegate()->OnReadCompleted(request_, filter_bytes_read);
+      if (!filter_bytes_read)
+        DoneReading();
+      request_->NotifyReadCompleted(filter_bytes_read);
     }
   } else {
-    postfilter_bytes_read_ += bytes_read;
-    request_->delegate()->OnReadCompleted(request_, bytes_read);
+    request_->NotifyReadCompleted(bytes_read);
   }
+  DVLOG(1) << __FUNCTION__ << "() "
+           << "\"" << (request_ ? request_->url().spec() : "???") << "\""
+           << " pre bytes read = " << bytes_read
+           << " pre total = " << prefilter_bytes_read_
+           << " post total = " << postfilter_bytes_read_;
 }
 
 void URLRequestJob::NotifyStartError(const URLRequestStatus &status) {
@@ -543,7 +386,7 @@ void URLRequestJob::NotifyStartError(const URLRequestStatus &status) {
   has_handled_response_ = true;
   if (request_) {
     request_->set_status(status);
-    request_->ResponseStarted();
+    request_->NotifyResponseStarted();
   }
 }
 
@@ -553,23 +396,6 @@ void URLRequestJob::NotifyDone(const URLRequestStatus &status) {
     return;
   done_ = true;
 
-  RecordCompressionHistograms();
-
-  if (is_profiling() && metrics_->total_bytes_read_ > 0) {
-    // There are valid IO statistics. Fill in other fields of metrics for
-    // profiling consumers to retrieve information.
-    metrics_->original_url_.reset(new GURL(request_->original_url()));
-    metrics_->end_time_ = TimeTicks::Now();
-    metrics_->success_ = status.is_success();
-
-    if (!(request_->original_url() == request_->url())) {
-      metrics_->url_.reset(new GURL(request_->url()));
-    }
-  } else {
-    metrics_.reset();
-  }
-
-
   // Unless there was an error, we should have at least tried to handle
   // the response before getting here.
   DCHECK(has_handled_response_ || !status.is_success());
@@ -584,39 +410,45 @@ void URLRequestJob::NotifyDone(const URLRequestStatus &status) {
     // an error, we do not change the status back to success.  To
     // enforce this, only set the status if the job is so far
     // successful.
-    if (request_->status().is_success())
+    if (request_->status().is_success()) {
+      if (status.status() == URLRequestStatus::FAILED) {
+        request_->net_log().AddEvent(
+            NetLog::TYPE_FAILED,
+            make_scoped_refptr(new NetLogIntegerParameter("net_error",
+                                                          status.error())));
+      }
       request_->set_status(status);
+    }
   }
 
-  g_url_request_job_tracker.OnJobDone(this, status);
-
   // Complete this notification later.  This prevents us from re-entering the
   // delegate if we're done because of a synchronous call.
-  MessageLoop::current()->PostTask(FROM_HERE, NewRunnableMethod(
-      this, &URLRequestJob::CompleteNotifyDone));
+  MessageLoop::current()->PostTask(
+      FROM_HERE,
+      base::Bind(&URLRequestJob::CompleteNotifyDone,
+                 weak_factory_.GetWeakPtr()));
 }
 
 void URLRequestJob::CompleteNotifyDone() {
   // Check if we should notify the delegate that we're done because of an error.
   if (request_ &&
       !request_->status().is_success() &&
-      request_->delegate()) {
+      request_->has_delegate()) {
     // We report the error differently depending on whether we've called
     // OnResponseStarted yet.
     if (has_handled_response_) {
       // We signal the error by calling OnReadComplete with a bytes_read of -1.
-      request_->delegate()->OnReadCompleted(request_, -1);
+      request_->NotifyReadCompleted(-1);
     } else {
       has_handled_response_ = true;
-      request_->ResponseStarted();
+      request_->NotifyResponseStarted();
     }
   }
 }
 
 void URLRequestJob::NotifyCanceled() {
   if (!done_) {
-    NotifyDone(URLRequestStatus(URLRequestStatus::CANCELED,
-                                net::ERR_ABORTED));
+    NotifyDone(URLRequestStatus(URLRequestStatus::CANCELED, ERR_ABORTED));
   }
 }
 
@@ -626,12 +458,23 @@ void URLRequestJob::NotifyRestartRequired() {
     request_->Restart();
 }
 
-bool URLRequestJob::ReadRawData(net::IOBuffer* buf, int buf_size,
+void URLRequestJob::SetBlockedOnDelegate() {
+  request_->SetBlockedOnDelegate();
+}
+
+void URLRequestJob::SetUnblockedOnDelegate() {
+  request_->SetUnblockedOnDelegate();
+}
+
+bool URLRequestJob::ReadRawData(IOBuffer* buf, int buf_size,
                                 int *bytes_read) {
   DCHECK(bytes_read);
   *bytes_read = 0;
-  NotifyDone(URLRequestStatus());
-  return false;
+  return true;
+}
+
+void URLRequestJob::DoneReading() {
+  // Do nothing.
 }
 
 void URLRequestJob::FilteredDataRead(int bytes_read) {
@@ -688,6 +531,7 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
       case Filter::FILTER_DONE: {
         filter_needs_more_output_space_ = false;
         *bytes_read = filtered_data_len;
+        postfilter_bytes_read_ += filtered_data_len;
         rv = true;
         break;
       }
@@ -702,6 +546,7 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
         // We can revisit this issue if there is a real perf need.
         if (filtered_data_len > 0) {
           *bytes_read = filtered_data_len;
+          postfilter_bytes_read_ += filtered_data_len;
           rv = true;
         } else {
           // Read again since we haven't received enough data yet (e.g., we may
@@ -714,13 +559,17 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
         filter_needs_more_output_space_ =
             (filtered_data_len == output_buffer_size);
         *bytes_read = filtered_data_len;
+        postfilter_bytes_read_ += filtered_data_len;
         rv = true;
         break;
       }
       case Filter::FILTER_ERROR: {
+        DVLOG(1) << __FUNCTION__ << "() "
+                 << "\"" << (request_ ? request_->url().spec() : "???") << "\""
+                 << " Filter Error";
         filter_needs_more_output_space_ = false;
         NotifyDone(URLRequestStatus(URLRequestStatus::FAILED,
-                   net::ERR_CONTENT_DECODING_FAILED));
+                   ERR_CONTENT_DECODING_FAILED));
         rv = false;
         break;
       }
@@ -731,6 +580,20 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
         break;
       }
     }
+    DVLOG(2) << __FUNCTION__ << "() "
+             << "\"" << (request_ ? request_->url().spec() : "???") << "\""
+             << " rv = " << rv
+             << " post bytes read = " << filtered_data_len
+             << " pre total = " << prefilter_bytes_read_
+             << " post total = "
+             << postfilter_bytes_read_;
+    // If logging all bytes is enabled, log the filtered bytes read.
+    if (rv && request() && request()->net_log().IsLoggingBytes() &&
+        filtered_data_len > 0) {
+      request()->net_log().AddByteTransferEvent(
+          NetLog::TYPE_URL_REQUEST_JOB_FILTERED_BYTES_READ,
+          filtered_data_len, filtered_read_buffer_->data());
+    }
   } else {
     // we are done, or there is no data left.
     rv = true;
@@ -745,18 +608,12 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
   return rv;
 }
 
-void URLRequestJob::EnablePacketCounting(size_t max_packets_timed) {
-  if (max_packets_timed_ < max_packets_timed)
-    max_packets_timed_ = max_packets_timed;
-  packet_timing_enabled_ = true;
-}
-
 const URLRequestStatus URLRequestJob::GetStatus() {
   if (request_)
     return request_->status();
   // If the request is gone, we must be cancelled.
   return URLRequestStatus(URLRequestStatus::CANCELED,
-                          net::ERR_ABORTED);
+                          ERR_ABORTED);
 }
 
 void URLRequestJob::SetStatus(const URLRequestStatus &status) {
@@ -776,14 +633,14 @@ bool URLRequestJob::ReadRawDataForFilter(int* bytes_read) {
   // TODO(mbelshe): is it possible that the filter needs *MORE* data
   //    when there is some data already in the buffer?
   if (!filter_->stream_data_len() && !is_done()) {
-    net::IOBuffer* stream_buffer = filter_->stream_buffer();
+    IOBuffer* stream_buffer = filter_->stream_buffer();
     int stream_buffer_size = filter_->stream_buffer_size();
     rv = ReadRawDataHelper(stream_buffer, stream_buffer_size, bytes_read);
   }
   return rv;
 }
 
-bool URLRequestJob::ReadRawDataHelper(net::IOBuffer* buf, int buf_size,
+bool URLRequestJob::ReadRawDataHelper(IOBuffer* buf, int buf_size,
                                       int* bytes_read) {
   DCHECK(!request_->status().is_io_pending());
   DCHECK(raw_read_buffer_ == NULL);
@@ -795,6 +652,15 @@ bool URLRequestJob::ReadRawDataHelper(net::IOBuffer* buf, int buf_size,
   bool rv = ReadRawData(buf, buf_size, bytes_read);
 
   if (!request_->status().is_io_pending()) {
+    // If |filter_| is NULL, and logging all bytes is enabled, log the raw
+    // bytes read.
+    if (!filter_.get() && request() && request()->net_log().IsLoggingBytes() &&
+        *bytes_read > 0) {
+      request()->net_log().AddByteTransferEvent(
+          NetLog::TYPE_URL_REQUEST_JOB_BYTES_READ,
+          *bytes_read, raw_read_buffer_->data());
+    }
+
     // If the read completes synchronously, either success or failure,
     // invoke the OnRawReadComplete callback so we can account for the
     // completed read.
@@ -804,10 +670,8 @@ bool URLRequestJob::ReadRawDataHelper(net::IOBuffer* buf, int buf_size,
 }
 
 void URLRequestJob::FollowRedirect(const GURL& location, int http_status_code) {
-  g_url_request_job_tracker.OnJobRedirect(this, location, http_status_code);
-
   int rv = request_->Redirect(location, http_status_code);
-  if (rv != net::OK)
+  if (rv != OK)
     NotifyDone(URLRequestStatus(URLRequestStatus::FAILED, rv));
 }
 
@@ -820,14 +684,19 @@ void URLRequestJob::OnRawReadComplete(int bytes_read) {
 }
 
 void URLRequestJob::RecordBytesRead(int bytes_read) {
-  if (is_profiling()) {
-    ++(metrics_->number_of_read_IO_);
-    metrics_->total_bytes_read_ += bytes_read;
-  }
   filter_input_byte_count_ += bytes_read;
+  prefilter_bytes_read_ += bytes_read;
+  if (!filter_.get())
+    postfilter_bytes_read_ += bytes_read;
+  DVLOG(2) << __FUNCTION__ << "() "
+           << "\"" << (request_ ? request_->url().spec() : "???") << "\""
+           << " pre bytes read = " << bytes_read
+           << " pre total = " << prefilter_bytes_read_
+           << " post total = " << postfilter_bytes_read_;
   UpdatePacketReadTimes();  // Facilitate stats recording if it is active.
-  g_url_request_job_tracker.OnBytesRead(this, raw_read_buffer_->data(),
-                                        bytes_read);
+  const URLRequestContext* context = request_->context();
+  if (context && context->network_delegate())
+    context->network_delegate()->NotifyRawBytesRead(*request_, bytes_read);
 }
 
 bool URLRequestJob::FilterHasData() {
@@ -835,102 +704,6 @@ bool URLRequestJob::FilterHasData() {
 }
 
 void URLRequestJob::UpdatePacketReadTimes() {
-  if (!packet_timing_enabled_)
-    return;
-
-  if (filter_input_byte_count_ <= bytes_observed_in_packets_) {
-    DCHECK(filter_input_byte_count_ == bytes_observed_in_packets_);
-    return;  // No new bytes have arrived.
-  }
-
-  if (!bytes_observed_in_packets_)
-    request_time_snapshot_ = GetRequestTime();
-
-  final_packet_time_ = base::Time::Now();
-  const size_t kTypicalPacketSize = 1430;
-  while (filter_input_byte_count_ > bytes_observed_in_packets_) {
-    ++observed_packet_count_;
-    if (max_packets_timed_ > packet_times_.size()) {
-      packet_times_.push_back(final_packet_time_);
-      DCHECK(static_cast<size_t>(observed_packet_count_) ==
-             packet_times_.size());
-    }
-    bytes_observed_in_packets_ += kTypicalPacketSize;
-  }
-  // Since packets may not be full, we'll remember the number of bytes we've
-  // accounted for in packets thus far.
-  bytes_observed_in_packets_ = filter_input_byte_count_;
-}
-
-// The common type of histogram we use for all compression-tracking histograms.
-#define COMPRESSION_HISTOGRAM(name, sample) \
-    do { \
-      UMA_HISTOGRAM_CUSTOM_COUNTS("Net.Compress." name, sample, \
-                                  500, 1000000, 100); \
-    } while(0)
-
-void URLRequestJob::RecordCompressionHistograms() {
-  if (IsCachedContent() ||          // Don't record cached content
-      !GetStatus().is_success() ||  // Don't record failed content
-      !is_compressible_content_ ||  // Only record compressible content
-      !prefilter_bytes_read_)       // Zero-byte responses aren't useful.
-    return;
-
-  // Miniature requests aren't really compressible.  Don't count them.
-  const int kMinSize = 16;
-  if (prefilter_bytes_read_ < kMinSize)
-    return;
-
-  // Only record for http or https urls.
-  bool is_http = request_->url().SchemeIs("http");
-  bool is_https = request_->url().SchemeIs("https");
-  if (!is_http && !is_https)
-    return;
-
-  const net::HttpResponseInfo& response = request_->response_info_;
-  int compressed_B = prefilter_bytes_read_;
-  int decompressed_B = postfilter_bytes_read_;
-
-  // We want to record how often downloaded resources are compressed.
-  // But, we recognize that different protocols may have different
-  // properties.  So, for each request, we'll put it into one of 3
-  // groups:
-  //      a) SSL resources
-  //         Proxies cannot tamper with compression headers with SSL.
-  //      b) Non-SSL, loaded-via-proxy resources
-  //         In this case, we know a proxy might have interfered.
-  //      c) Non-SSL, loaded-without-proxy resources
-  //         In this case, we know there was no explicit proxy.  However,
-  //         it is possible that a transparent proxy was still interfering.
-  //
-  // For each group, we record the same 3 histograms.
-
-  if (is_https) {
-    if (is_compressed_) {
-      COMPRESSION_HISTOGRAM("SSL.BytesBeforeCompression", compressed_B);
-      COMPRESSION_HISTOGRAM("SSL.BytesAfterCompression", decompressed_B);
-    } else {
-      COMPRESSION_HISTOGRAM("SSL.ShouldHaveBeenCompressed", decompressed_B);
-    }
-    return;
-  }
-
-  if (response.was_fetched_via_proxy) {
-    if (is_compressed_) {
-      COMPRESSION_HISTOGRAM("Proxy.BytesBeforeCompression", compressed_B);
-      COMPRESSION_HISTOGRAM("Proxy.BytesAfterCompression", decompressed_B);
-    } else {
-      COMPRESSION_HISTOGRAM("Proxy.ShouldHaveBeenCompressed", decompressed_B);
-    }
-    return;
-  }
-
-  if (is_compressed_) {
-    COMPRESSION_HISTOGRAM("NoProxy.BytesBeforeCompression", compressed_B);
-    COMPRESSION_HISTOGRAM("NoProxy.BytesAfterCompression", decompressed_B);
-  } else {
-    COMPRESSION_HISTOGRAM("NoProxy.ShouldHaveBeenCompressed", decompressed_B);
-  }
 }
 
 }  // namespace net
