@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,16 +10,20 @@
 
 #include "base/basictypes.h"
 #include "googleurl/src/gurl.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/base/request_priority.h"
 
+namespace net {
+
 // Holds the parameters to emit to the NetLog when starting a URLRequest.
-class URLRequestStartEventParameters : public net::NetLog::EventParameters {
+class NET_EXPORT URLRequestStartEventParameters
+    : public NetLog::EventParameters {
  public:
   URLRequestStartEventParameters(const GURL& url,
                                  const std::string& method,
                                  int load_flags,
-                                 net::RequestPriority priority);
+                                 RequestPriority priority);
 
   const GURL& url() const {
     return url_;
@@ -29,15 +33,17 @@ class URLRequestStartEventParameters : public net::NetLog::EventParameters {
     return load_flags_;
   }
 
-  virtual Value* ToValue() const;
+  virtual base::Value* ToValue() const OVERRIDE;
 
  private:
   const GURL url_;
   const std::string method_;
   const int load_flags_;
-  const net::RequestPriority priority_;
+  const RequestPriority priority_;
 
   DISALLOW_COPY_AND_ASSIGN(URLRequestStartEventParameters);
 };
 
+}  // namespace net
+
 #endif  // NET_URL_REQUEST_URL_REQUEST_NETLOG_PARAMS_H_
