@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,23 +8,27 @@
 
 #include <string>
 
+#include "base/memory/weak_ptr.h"
+#include "net/base/net_export.h"
 #include "net/url_request/url_request_job.h"
 
 namespace net {
 
 class URLRequest;
 
-class URLRequestSimpleJob : public URLRequestJob {
+class NET_EXPORT URLRequestSimpleJob : public URLRequestJob {
  public:
   explicit URLRequestSimpleJob(URLRequest* request);
 
-  virtual void Start();
-  virtual bool ReadRawData(IOBuffer* buf, int buf_size, int *bytes_read);
-  virtual bool GetMimeType(std::string* mime_type) const;
-  virtual bool GetCharset(std::string* charset);
+  virtual void Start() OVERRIDE;
+  virtual bool ReadRawData(IOBuffer* buf,
+                           int buf_size,
+                           int *bytes_read) OVERRIDE;
+  virtual bool GetMimeType(std::string* mime_type) const OVERRIDE;
+  virtual bool GetCharset(std::string* charset) OVERRIDE;
 
  protected:
-  ~URLRequestSimpleJob() {}
+  virtual ~URLRequestSimpleJob();
 
   // subclasses must override the way response data is determined.
   virtual bool GetData(std::string* mime_type,
@@ -39,6 +43,7 @@ class URLRequestSimpleJob : public URLRequestJob {
   std::string charset_;
   std::string data_;
   int data_offset_;
+  base::WeakPtrFactory<URLRequestSimpleJob> weak_factory_;
 };
 
 }  // namespace net
