@@ -8,6 +8,7 @@
 
 #include <string>
 
+#include "base/memory/weak_ptr.h"
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_job.h"
 
@@ -35,7 +36,7 @@ namespace net {
 //
 // Optionally, you can also construct test jobs that advance automatically
 // without having to call ProcessOnePendingMessage.
-class URLRequestTestJob : public URLRequestJob {
+class NET_EXPORT_PRIVATE URLRequestTestJob : public URLRequestJob {
  public:
   // Constructs a job to return one of the canned responses depending on the
   // request url, with auto advance disabled.
@@ -43,15 +44,15 @@ class URLRequestTestJob : public URLRequestJob {
 
   // Constructs a job to return one of the canned responses depending on the
   // request url, optionally with auto advance enabled.
-  explicit URLRequestTestJob(URLRequest* request, bool auto_advance);
+  URLRequestTestJob(URLRequest* request, bool auto_advance);
 
   // Constructs a job to return the given response regardless of the request
   // url. The headers should include the HTTP status line and be formatted as
   // expected by HttpResponseHeaders.
-  explicit URLRequestTestJob(URLRequest* request,
-                             const std::string& response_headers,
-                             const std::string& response_data,
-                             bool auto_advance);
+  URLRequestTestJob(URLRequest* request,
+                    const std::string& response_headers,
+                    const std::string& response_data,
+                    bool auto_advance);
 
   // The three canned URLs this handler will respond to without having been
   // explicitly initialized with response headers and data.
@@ -91,13 +92,16 @@ class URLRequestTestJob : public URLRequestJob {
   static URLRequest::ProtocolFactory Factory;
 
   // Job functions
-  virtual void Start();
-  virtual bool ReadRawData(IOBuffer* buf, int buf_size, int *bytes_read);
-  virtual void Kill();
-  virtual bool GetMimeType(std::string* mime_type) const;
-  virtual void GetResponseInfo(HttpResponseInfo* info);
-  virtual int GetResponseCode() const;
-  virtual bool IsRedirectResponse(GURL* location, int* http_status_code);
+  virtual void Start() OVERRIDE;
+  virtual bool ReadRawData(IOBuffer* buf,
+                           int buf_size,
+                           int *bytes_read) OVERRIDE;
+  virtual void Kill() OVERRIDE;
+  virtual bool GetMimeType(std::string* mime_type) const OVERRIDE;
+  virtual void GetResponseInfo(HttpResponseInfo* info) OVERRIDE;
+  virtual int GetResponseCode() const OVERRIDE;
+  virtual bool IsRedirectResponse(GURL* location,
+                                  int* http_status_code) OVERRIDE;
 
  protected:
   // This is what operation we are going to do next when this job is handled.
@@ -134,6 +138,8 @@ class URLRequestTestJob : public URLRequestJob {
   // Holds the buffer for an asynchronous ReadRawData call
   IOBuffer* async_buf_;
   int async_buf_size_;
+
+  base::WeakPtrFactory<URLRequestTestJob> weak_factory_;
 };
 
 }  // namespace net
