@@ -1,13 +1,15 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef NET_URL_REQUEST_URL_REQUEST_THROTTLER_HEADER_ADAPTER_H_
 #define NET_URL_REQUEST_URL_REQUEST_THROTTLER_HEADER_ADAPTER_H_
+#pragma once
 
 #include <string>
 
-#include "base/ref_counted.h"
+#include "base/compiler_specific.h"
+#include "base/memory/ref_counted.h"
 #include "net/url_request/url_request_throttler_header_interface.h"
 
 namespace net {
@@ -18,15 +20,15 @@ class HttpResponseHeaders;
 class URLRequestThrottlerHeaderAdapter
     : public URLRequestThrottlerHeaderInterface {
  public:
-  explicit URLRequestThrottlerHeaderAdapter(net::HttpResponseHeaders* headers);
+  explicit URLRequestThrottlerHeaderAdapter(HttpResponseHeaders* headers);
   virtual ~URLRequestThrottlerHeaderAdapter();
 
   // Implementation of URLRequestThrottlerHeaderInterface
-  virtual std::string GetNormalizedValue(const std::string& key) const;
-  virtual int GetResponseCode() const;
+  virtual std::string GetNormalizedValue(const std::string& key) const OVERRIDE;
+  virtual int GetResponseCode() const OVERRIDE;
 
  private:
-  const scoped_refptr<net::HttpResponseHeaders> response_header_;
+  const scoped_refptr<HttpResponseHeaders> response_header_;
 };
 
 }  // namespace net
