@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,21 +6,27 @@
 
 #include <algorithm>
 
+#include "base/bind.h"
 #include "base/lazy_instance.h"
 #include "base/string_tokenizer.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_log.h"
-#include "net/base/cookie_policy.h"
 #include "net/base/cookie_store.h"
 #include "net/base/io_buffer.h"
+#include "net/http/http_network_session.h"
+#include "net/http/http_transaction_factory.h"
 #include "net/http/http_util.h"
+#include "net/spdy/spdy_session.h"
+#include "net/spdy/spdy_session_pool.h"
 #include "net/url_request/url_request_context.h"
 #include "net/websockets/websocket_frame_handler.h"
 #include "net/websockets/websocket_handshake_handler.h"
 #include "net/websockets/websocket_net_log_params.h"
 #include "net/websockets/websocket_throttle.h"
 
+static const int kMaxPendingSendAllowed = 32768;  // 32 kilobytes.
+
 namespace {
 
 // lower-case header names.
@@ -47,33 +53,38 @@ class WebSocketJobInitSingleton {
   }
 };
 
-static base::LazyInstance<WebSocketJobInitSingleton> g_websocket_job_init(
-    base::LINKER_INITIALIZED);
+static base::LazyInstance<WebSocketJobInitSingleton> g_websocket_job_init =
+    LAZY_INSTANCE_INITIALIZER;
 
 }  // anonymous namespace
 
 namespace net {
 
+bool WebSocketJob::websocket_over_spdy_enabled_ = false;
+
 // static
 void WebSocketJob::EnsureInit() {
   g_websocket_job_init.Get();
 }
 
+// static
+void WebSocketJob::set_websocket_over_spdy_enabled(bool enabled) {
+  websocket_over_spdy_enabled_ = enabled;
+}
+
 WebSocketJob::WebSocketJob(SocketStream::Delegate* delegate)
     : delegate_(delegate),
       state_(INITIALIZED),
       waiting_(false),
-      callback_(NULL),
       handshake_request_(new WebSocketHandshakeRequestHandler),
       handshake_response_(new WebSocketHandshakeResponseHandler),
+      started_to_send_handshake_request_(false),
       handshake_request_sent_(0),
       response_cookies_save_index_(0),
-      ALLOW_THIS_IN_INITIALIZER_LIST(can_get_cookies_callback_(
-          this, &WebSocketJob::OnCanGetCookiesCompleted)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(can_set_cookie_callback_(
-          this, &WebSocketJob::OnCanSetCookieCompleted)),
       send_frame_handler_(new WebSocketFrameHandler),
-      receive_frame_handler_(new WebSocketFrameHandler) {
+      receive_frame_handler_(new WebSocketFrameHandler),
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_ptr_factory_(this)),
+      ALLOW_THIS_IN_INITIALIZER_LIST(weak_ptr_factory_for_send_pending_(this)) {
 }
 
 WebSocketJob::~WebSocketJob() {
@@ -107,7 +118,8 @@ bool WebSocketJob::SendData(const char* data, int len) {
         // pending bytes less than max_pending_send_allowed, so when sending
         // larger message than max_pending_send_allowed should not be buffered.
         // If we don't call OnSentData, WebCore::SocketStreamHandle would stop
-        // sending more data when pending data reaches max_pending_send_allowed.
+        // sending more data when pending data reaches
+        // max_pending_send_allowed.
         // TODO(ukai): Fix this to support compression for larger message.
         int err = 0;
         if (!send_frame_handler_->GetCurrentBuffer() &&
@@ -116,7 +128,7 @@ bool WebSocketJob::SendData(const char* data, int len) {
           current_buffer_ = new DrainableIOBuffer(
               send_frame_handler_->GetCurrentBuffer(),
               send_frame_handler_->GetCurrentBufferSize());
-          return socket_->SendData(
+          return SendDataInternal(
               current_buffer_->data(), current_buffer_->BytesRemaining());
         }
         return err >= 0;
@@ -130,20 +142,21 @@ bool WebSocketJob::SendData(const char* data, int len) {
 }
 
 void WebSocketJob::Close() {
+  if (state_ == CLOSED)
+    return;
+
   state_ = CLOSING;
   if (current_buffer_) {
     // Will close in SendPending.
     return;
   }
   state_ = CLOSED;
-  socket_->Close();
+  CloseInternal();
 }
 
-void WebSocketJob::RestartWithAuth(
-    const string16& username,
-    const string16& password) {
+void WebSocketJob::RestartWithAuth(const AuthCredentials& credentials) {
   state_ = CONNECTING;
-  socket_->RestartWithAuth(username, password);
+  socket_->RestartWithAuth(credentials);
 }
 
 void WebSocketJob::DetachDelegate() {
@@ -152,29 +165,38 @@ void WebSocketJob::DetachDelegate() {
   WebSocketThrottle::GetInstance()->WakeupSocketIfNecessary();
 
   scoped_refptr<WebSocketJob> protect(this);
+  weak_ptr_factory_.InvalidateWeakPtrs();
+  weak_ptr_factory_for_send_pending_.InvalidateWeakPtrs();
 
   delegate_ = NULL;
   if (socket_)
     socket_->DetachDelegate();
   socket_ = NULL;
-  if (callback_) {
+  if (!callback_.is_null()) {
     waiting_ = false;
-    callback_ = NULL;
+    callback_.Reset();
     Release();  // Balanced with OnStartOpenConnection().
   }
 }
 
 int WebSocketJob::OnStartOpenConnection(
-    SocketStream* socket, CompletionCallback* callback) {
-  DCHECK(!callback_);
+    SocketStream* socket, const CompletionCallback& callback) {
+  DCHECK(callback_.is_null());
   state_ = CONNECTING;
-  addresses_.Copy(socket->address_list().head(), true);
+  addresses_ = socket->address_list();
   WebSocketThrottle::GetInstance()->PutInQueue(this);
-  if (!waiting_)
-    return OK;
-  callback_ = callback;
-  AddRef();  // Balanced when callback_ becomes NULL.
-  return ERR_IO_PENDING;
+  if (delegate_) {
+    int result = delegate_->OnStartOpenConnection(socket, callback);
+    DCHECK_EQ(OK, result);
+  }
+  if (waiting_) {
+    // PutInQueue() may set |waiting_| true for throttling. In this case,
+    // Wakeup() will be called later.
+    callback_ = callback;
+    AddRef();  // Balanced when callback_ is cleared.
+    return ERR_IO_PENDING;
+  }
+  return TrySpdyStream();
 }
 
 void WebSocketJob::OnConnected(
@@ -188,6 +210,7 @@ void WebSocketJob::OnConnected(
 
 void WebSocketJob::OnSentData(SocketStream* socket, int amount_sent) {
   DCHECK_NE(INITIALIZED, state_);
+  DCHECK_GT(amount_sent, 0);
   if (state_ == CLOSED)
     return;
   if (state_ == CONNECTING) {
@@ -196,8 +219,10 @@ void WebSocketJob::OnSentData(SocketStream* socket, int amount_sent) {
   }
   if (delegate_) {
     DCHECK(state_ == OPEN || state_ == CLOSING);
-    DCHECK_GT(amount_sent, 0);
-    DCHECK(current_buffer_);
+    if (current_buffer_ == NULL) {
+      VLOG(1) << "OnSentData current_buffer=NULL amount_sent=" << amount_sent;
+      return;
+    }
     current_buffer_->DidConsume(amount_sent);
     if (current_buffer_->BytesRemaining() > 0)
       return;
@@ -208,9 +233,13 @@ void WebSocketJob::OnSentData(SocketStream* socket, int amount_sent) {
     DCHECK_GT(amount_sent, 0);
     current_buffer_ = NULL;
     send_frame_handler_->ReleaseCurrentBuffer();
+    if (!weak_ptr_factory_for_send_pending_.HasWeakPtrs()) {
+      MessageLoopForIO::current()->PostTask(
+          FROM_HERE,
+          base::Bind(&WebSocketJob::SendPending,
+                     weak_ptr_factory_for_send_pending_.GetWeakPtr()));
+    }
     delegate_->OnSentData(socket, amount_sent);
-    MessageLoopForIO::current()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &WebSocketJob::SendPending));
   }
 }
 
@@ -234,7 +263,7 @@ void WebSocketJob::OnReceivedData(
                     receive_frame_handler_->GetCurrentBufferSize());
     receive_frame_handler_->ReleaseCurrentBuffer();
   }
-  if (delegate_ && received_data.size() > 0)
+  if (delegate_ && !received_data.empty())
       delegate_->OnReceivedData(
           socket, received_data.data(), received_data.size());
 }
@@ -245,13 +274,14 @@ void WebSocketJob::OnClose(SocketStream* socket) {
   WebSocketThrottle::GetInstance()->WakeupSocketIfNecessary();
 
   scoped_refptr<WebSocketJob> protect(this);
+  weak_ptr_factory_.InvalidateWeakPtrs();
 
   SocketStream::Delegate* delegate = delegate_;
   delegate_ = NULL;
   socket_ = NULL;
-  if (callback_) {
+  if (!callback_.is_null()) {
     waiting_ = false;
-    callback_ = NULL;
+    callback_.Reset();
     Release();  // Balanced with OnStartOpenConnection().
   }
   if (delegate)
@@ -265,55 +295,126 @@ void WebSocketJob::OnAuthRequired(
 }
 
 void WebSocketJob::OnError(const SocketStream* socket, int error) {
-  if (delegate_)
+  if (delegate_ && error != ERR_PROTOCOL_SWITCHED)
     delegate_->OnError(socket, error);
 }
 
+void WebSocketJob::OnCreatedSpdyStream(int result) {
+  DCHECK(spdy_websocket_stream_.get());
+  DCHECK(socket_.get());
+  DCHECK_NE(ERR_IO_PENDING, result);
+
+  if (state_ == CLOSED) {
+    result = ERR_ABORTED;
+  } else if (result == OK) {
+    state_ = CONNECTING;
+    result = ERR_PROTOCOL_SWITCHED;
+  } else {
+    spdy_websocket_stream_.reset();
+  }
+
+  CompleteIO(result);
+}
+
+void WebSocketJob::OnSentSpdyHeaders(int result) {
+  DCHECK_NE(INITIALIZED, state_);
+  if (state_ != CONNECTING)
+    return;
+  if (delegate_)
+    delegate_->OnSentData(socket_, handshake_request_->original_length());
+  handshake_request_.reset();
+}
+
+int WebSocketJob::OnReceivedSpdyResponseHeader(
+    const spdy::SpdyHeaderBlock& headers, int status) {
+  DCHECK_NE(INITIALIZED, state_);
+  if (state_ != CONNECTING)
+    return status;
+  if (status != OK)
+    return status;
+  // TODO(toyoshim): Fallback to non-spdy connection?
+  handshake_response_->ParseResponseHeaderBlock(headers, challenge_);
+
+  SaveCookiesAndNotifyHeaderComplete();
+  return OK;
+}
+
+void WebSocketJob::OnSentSpdyData(int amount_sent) {
+  DCHECK_NE(INITIALIZED, state_);
+  DCHECK_NE(CONNECTING, state_);
+  if (state_ == CLOSED)
+    return;
+  if (!spdy_websocket_stream_.get())
+    return;
+  OnSentData(socket_, amount_sent);
+}
+
+void WebSocketJob::OnReceivedSpdyData(const char* data, int length) {
+  DCHECK_NE(INITIALIZED, state_);
+  DCHECK_NE(CONNECTING, state_);
+  if (state_ == CLOSED)
+    return;
+  if (!spdy_websocket_stream_.get())
+    return;
+  OnReceivedData(socket_, data, length);
+}
+
+void WebSocketJob::OnCloseSpdyStream() {
+  spdy_websocket_stream_.reset();
+  OnClose(socket_);
+}
+
 bool WebSocketJob::SendHandshakeRequest(const char* data, int len) {
   DCHECK_EQ(state_, CONNECTING);
+  if (started_to_send_handshake_request_)
+    return false;
   if (!handshake_request_->ParseRequest(data, len))
     return false;
 
   // handshake message is completed.
+  handshake_response_->set_protocol_version(
+      handshake_request_->protocol_version());
   AddCookieHeaderAndSend();
-  // Just buffered in |handshake_request_|.
   return true;
 }
 
 void WebSocketJob::AddCookieHeaderAndSend() {
-  AddRef();  // Balanced in OnCanGetCookiesCompleted
+  bool allow = true;
+  if (delegate_ && !delegate_->CanGetCookies(socket_, GetURLForCookies()))
+    allow = false;
 
-  int policy = OK;
-  if (socket_->context()->cookie_policy()) {
-    GURL url_for_cookies = GetURLForCookies();
-    policy = socket_->context()->cookie_policy()->CanGetCookies(
-        url_for_cookies,
-        url_for_cookies,
-        &can_get_cookies_callback_);
-    if (policy == ERR_IO_PENDING)
-      return;  // Wait for completion callback
-  }
-  OnCanGetCookiesCompleted(policy);
-}
-
-void WebSocketJob::OnCanGetCookiesCompleted(int policy) {
   if (socket_ && delegate_ && state_ == CONNECTING) {
     handshake_request_->RemoveHeaders(
         kCookieHeaders, arraysize(kCookieHeaders));
-    if (policy == OK) {
+    if (allow && socket_->context()->cookie_store()) {
       // Add cookies, including HttpOnly cookies.
-      if (socket_->context()->cookie_store()) {
-        CookieOptions cookie_options;
-        cookie_options.set_include_httponly();
-        std::string cookie =
-            socket_->context()->cookie_store()->GetCookiesWithOptions(
-                GetURLForCookies(), cookie_options);
-        if (!cookie.empty())
-          handshake_request_->AppendHeaderIfMissing("Cookie", cookie);
-      }
+      CookieOptions cookie_options;
+      cookie_options.set_include_httponly();
+      socket_->context()->cookie_store()->GetCookiesWithOptionsAsync(
+          GetURLForCookies(), cookie_options,
+          base::Bind(&WebSocketJob::LoadCookieCallback,
+                     weak_ptr_factory_.GetWeakPtr()));
+    } else {
+      DoSendData();
     }
+  }
+}
 
-    const std::string& handshake_request = handshake_request_->GetRawRequest();
+void WebSocketJob::LoadCookieCallback(const std::string& cookie) {
+  if (!cookie.empty())
+    handshake_request_->AppendHeaderIfMissing("Cookie", cookie);
+  DoSendData();
+}
+
+void WebSocketJob::DoSendData() {
+  if (spdy_websocket_stream_.get()) {
+    linked_ptr<spdy::SpdyHeaderBlock> headers(new spdy::SpdyHeaderBlock);
+    handshake_request_->GetRequestHeaderBlock(
+        socket_->url(), headers.get(), &challenge_);
+    spdy_websocket_stream_->SendRequest(headers);
+  } else {
+    const std::string& handshake_request =
+        handshake_request_->GetRawRequest();
     handshake_request_sent_ = 0;
     socket_->net_log()->AddEvent(
         NetLog::TYPE_WEB_SOCKET_SEND_REQUEST_HEADERS,
@@ -322,7 +423,8 @@ void WebSocketJob::OnCanGetCookiesCompleted(int policy) {
     socket_->SendData(handshake_request.data(),
                       handshake_request.size());
   }
-  Release();  // Balance AddRef taken in AddCookieHeaderAndSend
+  // Just buffered in |handshake_request_|.
+  started_to_send_handshake_request_ = true;
 }
 
 void WebSocketJob::OnSentHandshakeRequest(
@@ -414,40 +516,29 @@ void WebSocketJob::SaveNextCookie() {
     return;
   }
 
-  AddRef();  // Balanced in OnCanSetCookieCompleted
+  bool allow = true;
+  CookieOptions options;
+  GURL url = GetURLForCookies();
+  std::string cookie = response_cookies_[response_cookies_save_index_];
+  if (delegate_ && !delegate_->CanSetCookie(socket_, url, cookie, &options))
+    allow = false;
 
-  int policy = OK;
-  if (socket_->context()->cookie_policy()) {
-    GURL url_for_cookies = GetURLForCookies();
-    policy = socket_->context()->cookie_policy()->CanSetCookie(
-        url_for_cookies,
-        url_for_cookies,
-        response_cookies_[response_cookies_save_index_],
-        &can_set_cookie_callback_);
-    if (policy == ERR_IO_PENDING)
-      return;  // Wait for completion callback
+  if (socket_ && delegate_ && state_ == CONNECTING) {
+    response_cookies_save_index_++;
+    if (allow && socket_->context()->cookie_store()) {
+      options.set_include_httponly();
+      socket_->context()->cookie_store()->SetCookieWithOptionsAsync(
+          url, cookie, options,
+          base::Bind(&WebSocketJob::SaveCookieCallback,
+                     weak_ptr_factory_.GetWeakPtr()));
+    } else {
+      SaveNextCookie();
+    }
   }
-
-  OnCanSetCookieCompleted(policy);
 }
 
-void WebSocketJob::OnCanSetCookieCompleted(int policy) {
-  if (socket_ && delegate_ && state_ == CONNECTING) {
-    if ((policy == OK || policy == OK_FOR_SESSION_ONLY) &&
-        socket_->context()->cookie_store()) {
-      CookieOptions options;
-      options.set_include_httponly();
-      if (policy == OK_FOR_SESSION_ONLY)
-        options.set_force_session();
-      GURL url_for_cookies = GetURLForCookies();
-      socket_->context()->cookie_store()->SetCookieWithOptions(
-          url_for_cookies, response_cookies_[response_cookies_save_index_],
-          options);
-    }
-    response_cookies_save_index_++;
-    SaveNextCookie();
-  }
-  Release();  // Balance AddRef taken in SaveNextCookie
+void WebSocketJob::SaveCookieCallback(bool cookie_status) {
+  SaveNextCookie();
 }
 
 GURL WebSocketJob::GetURLForCookies() const {
@@ -463,6 +554,57 @@ const AddressList& WebSocketJob::address_list() const {
   return addresses_;
 }
 
+int WebSocketJob::TrySpdyStream() {
+  if (!socket_.get())
+    return ERR_FAILED;
+
+  if (!websocket_over_spdy_enabled_)
+    return OK;
+
+  // Check if we have a SPDY session available.
+  HttpTransactionFactory* factory =
+      socket_->context()->http_transaction_factory();
+  if (!factory)
+    return OK;
+  scoped_refptr<HttpNetworkSession> session = factory->GetSession();
+  if (!session.get())
+    return OK;
+  SpdySessionPool* spdy_pool = session->spdy_session_pool();
+  const HostPortProxyPair pair(HostPortPair::FromURL(socket_->url()),
+                               socket_->proxy_server());
+  if (!spdy_pool->HasSession(pair))
+    return OK;
+
+  // Forbid wss downgrade to SPDY without SSL.
+  // TODO(toyoshim): Does it realize the same policy with HTTP?
+  scoped_refptr<SpdySession> spdy_session =
+      spdy_pool->Get(pair, *socket_->net_log());
+  SSLInfo ssl_info;
+  bool was_npn_negotiated;
+  SSLClientSocket::NextProto protocol_negotiated =
+      SSLClientSocket::kProtoUnknown;
+  bool use_ssl = spdy_session->GetSSLInfo(
+      &ssl_info, &was_npn_negotiated, &protocol_negotiated);
+  if (socket_->is_secure() && !use_ssl)
+    return OK;
+
+  // Create SpdyWebSocketStream.
+  spdy_websocket_stream_.reset(new SpdyWebSocketStream(spdy_session, this));
+
+  int result = spdy_websocket_stream_->InitializeStream(
+      socket_->url(), MEDIUM, *socket_->net_log());
+  if (result == OK) {
+    OnConnected(socket_, kMaxPendingSendAllowed);
+    return ERR_PROTOCOL_SWITCHED;
+  }
+  if (result != ERR_IO_PENDING) {
+    spdy_websocket_stream_.reset();
+    return OK;
+  }
+
+  return ERR_IO_PENDING;
+}
+
 void WebSocketJob::SetWaiting() {
   waiting_ = true;
 }
@@ -475,23 +617,47 @@ void WebSocketJob::Wakeup() {
   if (!waiting_)
     return;
   waiting_ = false;
-  DCHECK(callback_);
+  DCHECK(!callback_.is_null());
   MessageLoopForIO::current()->PostTask(
       FROM_HERE,
-      NewRunnableMethod(this,
-                        &WebSocketJob::DoCallback));
+      base::Bind(&WebSocketJob::RetryPendingIO,
+                 weak_ptr_factory_.GetWeakPtr()));
 }
 
-void WebSocketJob::DoCallback() {
-  // |callback_| may be NULL if OnClose() or DetachDelegate() was called.
-  if (callback_) {
-    net::CompletionCallback* callback = callback_;
-    callback_ = NULL;
-    callback->Run(net::OK);
+void WebSocketJob::RetryPendingIO() {
+  int result = TrySpdyStream();
+
+  // In the case of ERR_IO_PENDING, CompleteIO() will be called from
+  // OnCreatedSpdyStream().
+  if (result != ERR_IO_PENDING)
+    CompleteIO(result);
+}
+
+void WebSocketJob::CompleteIO(int result) {
+  // |callback_| may be null if OnClose() or DetachDelegate() was called.
+  if (!callback_.is_null()) {
+    CompletionCallback callback = callback_;
+    callback_.Reset();
+    callback.Run(result);
     Release();  // Balanced with OnStartOpenConnection().
   }
 }
 
+bool WebSocketJob::SendDataInternal(const char* data, int length) {
+  if (spdy_websocket_stream_.get())
+    return ERR_IO_PENDING == spdy_websocket_stream_->SendData(data, length);
+  if (socket_.get())
+    return socket_->SendData(data, length);
+  return false;
+}
+
+void WebSocketJob::CloseInternal() {
+  if (spdy_websocket_stream_.get())
+    spdy_websocket_stream_->Close();
+  if (socket_.get())
+    socket_->Close();
+}
+
 void WebSocketJob::SendPending() {
   if (current_buffer_)
     return;
@@ -500,13 +666,13 @@ void WebSocketJob::SendPending() {
   if (send_frame_handler_->UpdateCurrentBuffer(false) <= 0) {
     // No more data to send.
     if (state_ == CLOSING)
-      socket_->Close();
+      CloseInternal();
     return;
   }
   current_buffer_ = new DrainableIOBuffer(
       send_frame_handler_->GetCurrentBuffer(),
       send_frame_handler_->GetCurrentBufferSize());
-  socket_->SendData(current_buffer_->data(), current_buffer_->BytesRemaining());
+  SendDataInternal(current_buffer_->data(), current_buffer_->BytesRemaining());
 }
 
 }  // namespace net
