@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,7 +10,7 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "base/string_split.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
@@ -19,11 +19,12 @@
 
 namespace net {
 
-class NetLogWebSocketHandshakeParameter : public NetLog::EventParameters {
+class NET_EXPORT_PRIVATE NetLogWebSocketHandshakeParameter
+    : public NetLog::EventParameters {
  public:
   explicit NetLogWebSocketHandshakeParameter(const std::string& headers);
 
-  virtual Value* ToValue() const;
+  virtual Value* ToValue() const OVERRIDE;
 
  private:
   virtual ~NetLogWebSocketHandshakeParameter();
