@@ -1,7 +1,9 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "net/websockets/websocket_throttle.h"
+
 #include <string>
 
 #include "base/message_loop.h"
@@ -10,8 +12,8 @@
 #include "net/base/sys_addrinfo.h"
 #include "net/base/test_completion_callback.h"
 #include "net/socket_stream/socket_stream.h"
+#include "net/url_request/url_request_test_util.h"
 #include "net/websockets/websocket_job.h"
-#include "net/websockets/websocket_throttle.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/platform_test.h"
 
@@ -64,6 +66,8 @@ class WebSocketThrottleTest : public PlatformTest {
   static void MockSocketStreamConnect(
       SocketStream* socket, struct addrinfo* head) {
     socket->CopyAddrInfo(head);
+    // TODO(toyoshim): We should introduce additional tests on cases via proxy.
+    socket->proxy_info_.UseDirect();
     // In SocketStream::Connect(), it adds reference to socket, which is
     // balanced with SocketStream::Finish() that is finally called from
     // SocketStream::Close() or SocketStream::DetachDelegate(), when
@@ -77,7 +81,11 @@ class WebSocketThrottleTest : public PlatformTest {
 };
 
 TEST_F(WebSocketThrottleTest, Throttle) {
+  scoped_refptr<URLRequestContext> context(new TestURLRequestContext);
   DummySocketStreamDelegate delegate;
+  // TODO(toyoshim): We need to consider both spdy-enabled and spdy-disabled
+  // configuration.
+  WebSocketJob::set_websocket_over_spdy_enabled(true);
 
   // For host1: 1.2.3.4, 1.2.3.5, 1.2.3.6
   struct addrinfo* addr = AddAddr(1, 2, 3, 4, NULL);
@@ -86,6 +94,7 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   scoped_refptr<WebSocketJob> w1(new WebSocketJob(&delegate));
   scoped_refptr<SocketStream> s1(
       new SocketStream(GURL("ws://host1/"), w1.get()));
+  s1->set_context(context.get());
   w1->InitSocketStream(s1.get());
   WebSocketThrottleTest::MockSocketStreamConnect(s1, addr);
   DeleteAddrInfo(addr);
@@ -93,7 +102,7 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   DVLOG(1) << "socket1";
   TestCompletionCallback callback_s1;
   // Trying to open connection to host1 will start without wait.
-  EXPECT_EQ(OK, w1->OnStartOpenConnection(s1, &callback_s1));
+  EXPECT_EQ(OK, w1->OnStartOpenConnection(s1, callback_s1.callback()));
 
   // Now connecting to host1, so waiting queue looks like
   // Address | head -> tail
@@ -106,6 +115,7 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   scoped_refptr<WebSocketJob> w2(new WebSocketJob(&delegate));
   scoped_refptr<SocketStream> s2(
       new SocketStream(GURL("ws://host2/"), w2.get()));
+  s2->set_context(context.get());
   w2->InitSocketStream(s2.get());
   WebSocketThrottleTest::MockSocketStreamConnect(s2, addr);
   DeleteAddrInfo(addr);
@@ -113,7 +123,8 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   DVLOG(1) << "socket2";
   TestCompletionCallback callback_s2;
   // Trying to open connection to host2 will wait for w1.
-  EXPECT_EQ(ERR_IO_PENDING, w2->OnStartOpenConnection(s2, &callback_s2));
+  EXPECT_EQ(ERR_IO_PENDING,
+            w2->OnStartOpenConnection(s2, callback_s2.callback()));
   // Now waiting queue looks like
   // Address | head -> tail
   // 1.2.3.4 | w1 w2
@@ -125,6 +136,7 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   scoped_refptr<WebSocketJob> w3(new WebSocketJob(&delegate));
   scoped_refptr<SocketStream> s3(
       new SocketStream(GURL("ws://host3/"), w3.get()));
+  s3->set_context(context.get());
   w3->InitSocketStream(s3.get());
   WebSocketThrottleTest::MockSocketStreamConnect(s3, addr);
   DeleteAddrInfo(addr);
@@ -132,7 +144,8 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   DVLOG(1) << "socket3";
   TestCompletionCallback callback_s3;
   // Trying to open connection to host3 will wait for w1.
-  EXPECT_EQ(ERR_IO_PENDING, w3->OnStartOpenConnection(s3, &callback_s3));
+  EXPECT_EQ(ERR_IO_PENDING,
+            w3->OnStartOpenConnection(s3, callback_s3.callback()));
   // Address | head -> tail
   // 1.2.3.4 | w1 w2
   // 1.2.3.5 | w1    w3
@@ -144,6 +157,7 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   scoped_refptr<WebSocketJob> w4(new WebSocketJob(&delegate));
   scoped_refptr<SocketStream> s4(
       new SocketStream(GURL("ws://host4/"), w4.get()));
+  s4->set_context(context.get());
   w4->InitSocketStream(s4.get());
   WebSocketThrottleTest::MockSocketStreamConnect(s4, addr);
   DeleteAddrInfo(addr);
@@ -151,7 +165,8 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   DVLOG(1) << "socket4";
   TestCompletionCallback callback_s4;
   // Trying to open connection to host4 will wait for w1, w2.
-  EXPECT_EQ(ERR_IO_PENDING, w4->OnStartOpenConnection(s4, &callback_s4));
+  EXPECT_EQ(ERR_IO_PENDING,
+            w4->OnStartOpenConnection(s4, callback_s4.callback()));
   // Address | head -> tail
   // 1.2.3.4 | w1 w2    w4
   // 1.2.3.5 | w1    w3
@@ -162,6 +177,7 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   scoped_refptr<WebSocketJob> w5(new WebSocketJob(&delegate));
   scoped_refptr<SocketStream> s5(
       new SocketStream(GURL("ws://host5/"), w5.get()));
+  s5->set_context(context.get());
   w5->InitSocketStream(s5.get());
   WebSocketThrottleTest::MockSocketStreamConnect(s5, addr);
   DeleteAddrInfo(addr);
@@ -169,7 +185,8 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   DVLOG(1) << "socket5";
   TestCompletionCallback callback_s5;
   // Trying to open connection to host5 will wait for w1, w4
-  EXPECT_EQ(ERR_IO_PENDING, w5->OnStartOpenConnection(s5, &callback_s5));
+  EXPECT_EQ(ERR_IO_PENDING,
+            w5->OnStartOpenConnection(s5, callback_s5.callback()));
   // Address | head -> tail
   // 1.2.3.4 | w1 w2    w4
   // 1.2.3.5 | w1    w3
@@ -180,6 +197,7 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   scoped_refptr<WebSocketJob> w6(new WebSocketJob(&delegate));
   scoped_refptr<SocketStream> s6(
       new SocketStream(GURL("ws://host6/"), w6.get()));
+  s6->set_context(context.get());
   w6->InitSocketStream(s6.get());
   WebSocketThrottleTest::MockSocketStreamConnect(s6, addr);
   DeleteAddrInfo(addr);
@@ -187,7 +205,8 @@ TEST_F(WebSocketThrottleTest, Throttle) {
   DVLOG(1) << "socket6";
   TestCompletionCallback callback_s6;
   // Trying to open connection to host6 will wait for w1, w4, w5
-  EXPECT_EQ(ERR_IO_PENDING, w6->OnStartOpenConnection(s6, &callback_s6));
+  EXPECT_EQ(ERR_IO_PENDING,
+            w6->OnStartOpenConnection(s6, callback_s6.callback()));
   // Address | head -> tail
   // 1.2.3.4 | w1 w2    w4
   // 1.2.3.5 | w1    w3
@@ -281,7 +300,9 @@ TEST_F(WebSocketThrottleTest, Throttle) {
 }
 
 TEST_F(WebSocketThrottleTest, NoThrottleForDuplicateAddress) {
+  scoped_refptr<URLRequestContext> context(new TestURLRequestContext);
   DummySocketStreamDelegate delegate;
+  WebSocketJob::set_websocket_over_spdy_enabled(true);
 
   // For localhost: 127.0.0.1, 127.0.0.1
   struct addrinfo* addr = AddAddr(127, 0, 0, 1, NULL);
@@ -289,6 +310,7 @@ TEST_F(WebSocketThrottleTest, NoThrottleForDuplicateAddress) {
   scoped_refptr<WebSocketJob> w1(new WebSocketJob(&delegate));
   scoped_refptr<SocketStream> s1(
       new SocketStream(GURL("ws://localhost/"), w1.get()));
+  s1->set_context(context.get());
   w1->InitSocketStream(s1.get());
   WebSocketThrottleTest::MockSocketStreamConnect(s1, addr);
   DeleteAddrInfo(addr);
@@ -296,7 +318,7 @@ TEST_F(WebSocketThrottleTest, NoThrottleForDuplicateAddress) {
   DVLOG(1) << "socket1";
   TestCompletionCallback callback_s1;
   // Trying to open connection to localhost will start without wait.
-  EXPECT_EQ(OK, w1->OnStartOpenConnection(s1, &callback_s1));
+  EXPECT_EQ(OK, w1->OnStartOpenConnection(s1, callback_s1.callback()));
 
   DVLOG(1) << "socket1 close";
   w1->OnClose(s1.get());
