@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,26 +7,23 @@
 
 #include "ppapi/c/pp_module.h"
 #include "ppapi/proxy/interface_proxy.h"
+#include "ppapi/shared_impl/host_resource.h"
 
 struct PPB_PDF;
 
-namespace pp {
+namespace ppapi {
 namespace proxy {
 
 struct SerializedFontDescription;
 
 class PPB_PDF_Proxy : public InterfaceProxy {
  public:
-  PPB_PDF_Proxy(Dispatcher* dispatcher, const void* target_interface);
+  PPB_PDF_Proxy(Dispatcher* dispatcher);
   virtual ~PPB_PDF_Proxy();
 
-  const PPB_PDF* ppb_pdf_target() const {
-    return static_cast<const PPB_PDF*>(target_interface());
-  }
+  static const Info* GetInfo();
 
   // InterfaceProxy implementation.
-  virtual const void* GetSourceInterface() const;
-  virtual InterfaceID GetInterfaceId() const;
   virtual bool OnMessageReceived(const IPC::Message& msg);
 
  private:
@@ -34,13 +31,18 @@ class PPB_PDF_Proxy : public InterfaceProxy {
   void OnMsgGetFontFileWithFallback(PP_Module module,
                                     const SerializedFontDescription& desc,
                                     int32_t charset,
-                                    PP_Resource* result);
-  void OnMsgGetFontTableForPrivateFontFile(PP_Resource font_file,
+                                    ppapi::HostResource* result);
+  void OnMsgGetFontTableForPrivateFontFile(const ppapi::HostResource& font_file,
                                            uint32_t table,
                                            std::string* result);
+
+  // When this proxy is in the host side, this value caches the interface
+  // pointer so we don't have to retrieve it from the dispatcher each time.
+  // In the plugin, this value is always NULL.
+  const PPB_PDF* ppb_pdf_impl_;
 };
 
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
 
 #endif  // PPAPI_PPB_PDF_PROXY_H_
