@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,7 +11,7 @@
 #include "ppapi/proxy/ppapi_param_traits.h"
 #include "ppapi/proxy/var_serialization_rules.h"
 
-namespace pp {
+namespace ppapi {
 namespace proxy {
 
 // SerializedVar::Inner --------------------------------------------------------
@@ -19,7 +19,8 @@ namespace proxy {
 SerializedVar::Inner::Inner()
     : serialization_rules_(NULL),
       var_(PP_MakeUndefined()),
-      cleanup_mode_(CLEANUP_NONE) {
+      cleanup_mode_(CLEANUP_NONE),
+      dispatcher_for_end_send_pass_ref_(NULL) {
 #ifndef NDEBUG
   has_been_serialized_ = false;
   has_been_deserialized_ = false;
@@ -29,7 +30,8 @@ SerializedVar::Inner::Inner()
 SerializedVar::Inner::Inner(VarSerializationRules* serialization_rules)
     : serialization_rules_(serialization_rules),
       var_(PP_MakeUndefined()),
-      cleanup_mode_(CLEANUP_NONE) {
+      cleanup_mode_(CLEANUP_NONE),
+      dispatcher_for_end_send_pass_ref_(NULL) {
 #ifndef NDEBUG
   has_been_serialized_ = false;
   has_been_deserialized_ = false;
@@ -40,7 +42,8 @@ SerializedVar::Inner::Inner(VarSerializationRules* serialization_rules,
                             const PP_Var& var)
     : serialization_rules_(serialization_rules),
       var_(var),
-      cleanup_mode_(CLEANUP_NONE) {
+      cleanup_mode_(CLEANUP_NONE),
+      dispatcher_for_end_send_pass_ref_(NULL) {
 #ifndef NDEBUG
   has_been_serialized_ = false;
   has_been_deserialized_ = false;
@@ -50,7 +53,9 @@ SerializedVar::Inner::Inner(VarSerializationRules* serialization_rules,
 SerializedVar::Inner::~Inner() {
   switch (cleanup_mode_) {
     case END_SEND_PASS_REF:
-      serialization_rules_->EndSendPassRef(var_);
+      DCHECK(dispatcher_for_end_send_pass_ref_);
+      serialization_rules_->EndSendPassRef(var_,
+                                           dispatcher_for_end_send_pass_ref_);
       break;
     case END_RECEIVE_CALLER_OWNED:
       serialization_rules_->EndReceiveCallerOwned(var_);
@@ -91,6 +96,14 @@ std::string* SerializedVar::Inner::GetStringPtr() {
   return &string_value_;
 }
 
+void SerializedVar::Inner::ForceSetVarValueForTest(PP_Var value) {
+  var_ = value;
+}
+
+void SerializedVar::Inner::ForceSetStringValueForTest(const std::string& str) {
+  string_value_ = str;
+}
+
 void SerializedVar::Inner::WriteToMessage(IPC::Message* m) const {
   // When writing to the IPC messages, a serization rules handler should
   // always have been set.
@@ -123,7 +136,7 @@ void SerializedVar::Inner::WriteToMessage(IPC::Message* m) const {
       // just serialized.
       break;
     case PP_VARTYPE_BOOL:
-      m->WriteBool(PPBoolToBool(var_.value.as_bool));
+      m->WriteBool(PP_ToBool(var_.value.as_bool));
       break;
     case PP_VARTYPE_INT32:
       m->WriteInt(var_.value.as_int);
@@ -139,9 +152,18 @@ void SerializedVar::Inner::WriteToMessage(IPC::Message* m) const {
       // what looks like a valid empty string.
       m->WriteString(string_value_);
       break;
+    case PP_VARTYPE_ARRAY_BUFFER:
+      // TODO(dmichael): Proxy ArrayBuffer.
+      NOTIMPLEMENTED();
+      break;
     case PP_VARTYPE_OBJECT:
       m->WriteInt64(var_.value.as_id);
       break;
+    case PP_VARTYPE_ARRAY:
+    case PP_VARTYPE_DICTIONARY:
+      // TODO(brettw) when these are supported, implement this.
+      NOTIMPLEMENTED();
+      break;
   }
 }
 
@@ -175,7 +197,7 @@ bool SerializedVar::Inner::ReadFromMessage(const IPC::Message* m, void** iter) {
     case PP_VARTYPE_BOOL: {
       bool bool_value;
       success = m->ReadBool(iter, &bool_value);
-      var_.value.as_bool = BoolToPPBool(bool_value);
+      var_.value.as_bool = PP_FromBool(bool_value);
       break;
     }
     case PP_VARTYPE_INT32:
@@ -191,6 +213,11 @@ bool SerializedVar::Inner::ReadFromMessage(const IPC::Message* m, void** iter) {
     case PP_VARTYPE_OBJECT:
       success = m->ReadInt64(iter, &var_.value.as_id);
       break;
+    case PP_VARTYPE_ARRAY:
+    case PP_VARTYPE_DICTIONARY:
+      // TODO(brettw) when these types are supported, implement this.
+      NOTIMPLEMENTED();
+      break;
     default:
       // Leave success as false.
       break;
@@ -204,6 +231,18 @@ bool SerializedVar::Inner::ReadFromMessage(const IPC::Message* m, void** iter) {
   return success;
 }
 
+void SerializedVar::Inner::SetCleanupModeToEndSendPassRef(
+    Dispatcher* dispatcher) {
+  DCHECK(dispatcher);
+  DCHECK(!dispatcher_for_end_send_pass_ref_);
+  dispatcher_for_end_send_pass_ref_ = dispatcher;
+  cleanup_mode_ = END_SEND_PASS_REF;
+}
+
+void SerializedVar::Inner::SetCleanupModeToEndReceiveCallerOwned() {
+  cleanup_mode_ = END_RECEIVE_CALLER_OWNED;
+}
+
 // SerializedVar ---------------------------------------------------------------
 
 SerializedVar::SerializedVar() : inner_(new Inner) {
@@ -249,6 +288,11 @@ void SerializedVarSendInput::ConvertVector(Dispatcher* dispatcher,
 ReceiveSerializedVarReturnValue::ReceiveSerializedVarReturnValue() {
 }
 
+ReceiveSerializedVarReturnValue::ReceiveSerializedVarReturnValue(
+    const SerializedVar& serialized)
+    : SerializedVar(serialized) {
+}
+
 PP_Var ReceiveSerializedVarReturnValue::Return(Dispatcher* dispatcher) {
   inner_->set_serialization_rules(dispatcher->serialization_rules());
   inner_->SetVar(inner_->serialization_rules()->ReceivePassRef(
@@ -339,7 +383,7 @@ PP_Var SerializedVarReceiveInput::Get(Dispatcher* dispatcher) {
 
   // Ensure that when the serialized var goes out of scope it cleans up the
   // stuff we're making in BeginReceiveCallerOwned.
-  serialized_.inner_->set_cleanup_mode(SerializedVar::END_RECEIVE_CALLER_OWNED);
+  serialized_.inner_->SetCleanupModeToEndReceiveCallerOwned();
 
   serialized_.inner_->SetVar(
       serialized_.inner_->serialization_rules()->BeginReceiveCallerOwned(
@@ -367,7 +411,7 @@ PP_Var* SerializedVarVectorReceiveInput::Get(Dispatcher* dispatcher,
                                              uint32_t* array_size) {
   deserialized_.resize(serialized_.size());
   for (size_t i = 0; i < serialized_.size(); i++) {
-    // The vector must be able to clean themselves up after this call is
+    // The vectors must be able to clean themselves up after this call is
     // torn down.
     serialized_[i].inner_->set_serialization_rules(
         dispatcher->serialization_rules());
@@ -381,7 +425,7 @@ PP_Var* SerializedVarVectorReceiveInput::Get(Dispatcher* dispatcher,
   }
 
   *array_size = static_cast<uint32_t>(serialized_.size());
-  return deserialized_.size() > 0 ? &deserialized_[0] : NULL;
+  return deserialized_.empty() ? NULL : &deserialized_[0];
 }
 
 // SerializedVarReturnValue ----------------------------------------------------
@@ -396,7 +440,7 @@ void SerializedVarReturnValue::Return(Dispatcher* dispatcher,
       dispatcher->serialization_rules());
 
   // Var must clean up after our BeginSendPassRef call.
-  serialized_->inner_->set_cleanup_mode(SerializedVar::END_SEND_PASS_REF);
+  serialized_->inner_->SetCleanupModeToEndSendPassRef(dispatcher);
 
   serialized_->inner_->SetVar(
       dispatcher->serialization_rules()->BeginSendPassRef(
@@ -404,11 +448,22 @@ void SerializedVarReturnValue::Return(Dispatcher* dispatcher,
           serialized_->inner_->GetStringPtr()));
 }
 
+// static
+SerializedVar SerializedVarReturnValue::Convert(Dispatcher* dispatcher,
+                                                const PP_Var& var) {
+  // Mimic what happens in the normal case.
+  SerializedVar result;
+  SerializedVarReturnValue retvalue(&result);
+  retvalue.Return(dispatcher, var);
+  return result;
+}
+
 // SerializedVarOutParam -------------------------------------------------------
 
 SerializedVarOutParam::SerializedVarOutParam(SerializedVar* serialized)
     : serialized_(serialized),
-      writable_var_(PP_MakeUndefined()) {
+      writable_var_(PP_MakeUndefined()),
+      dispatcher_(NULL) {
 }
 
 SerializedVarOutParam::~SerializedVarOutParam() {
@@ -422,11 +477,12 @@ SerializedVarOutParam::~SerializedVarOutParam() {
     // Normally the current object will be created on the stack to wrap a
     // SerializedVar and won't have a scope around the actual IPC send. So we
     // need to tell the SerializedVar to do the begin/end send pass ref calls.
-    serialized_->inner_->set_cleanup_mode(SerializedVar::END_SEND_PASS_REF);
+    serialized_->inner_->SetCleanupModeToEndSendPassRef(dispatcher_);
   }
 }
 
 PP_Var* SerializedVarOutParam::OutParam(Dispatcher* dispatcher) {
+  dispatcher_ = dispatcher;
   serialized_->inner_->set_serialization_rules(
       dispatcher->serialization_rules());
   return &writable_var_;
@@ -468,6 +524,25 @@ PP_Var** SerializedVarVectorOutParam::ArrayOutParam(Dispatcher* dispatcher) {
   return &array_;
 }
 
+SerializedVarTestConstructor::SerializedVarTestConstructor(
+    const PP_Var& pod_var) {
+  DCHECK(pod_var.type != PP_VARTYPE_STRING);
+  inner_->ForceSetVarValueForTest(pod_var);
+}
+
+SerializedVarTestConstructor::SerializedVarTestConstructor(
+    const std::string& str) {
+  PP_Var string_var = {};
+  string_var.type = PP_VARTYPE_STRING;
+  string_var.value.as_id = 0;
+  inner_->ForceSetVarValueForTest(string_var);
+  inner_->ForceSetStringValueForTest(str);
+}
+
+SerializedVarTestReader::SerializedVarTestReader(const SerializedVar& var)
+    : SerializedVar(var) {
+}
+
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
 
