@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,16 @@
 
 namespace printing {
 
-PrinterBasicInfo::PrinterBasicInfo() : printer_status(0) {}
+PrinterBasicInfo::PrinterBasicInfo()
+    : printer_status(0),
+      is_default(false) {}
 
 PrinterBasicInfo::~PrinterBasicInfo() {}
 
+PrinterCapsAndDefaults::PrinterCapsAndDefaults() {}
+
+PrinterCapsAndDefaults::~PrinterCapsAndDefaults() {}
+
 PrintBackend::~PrintBackend() {}
 
 }  // namespace printing
