@@ -8,15 +8,19 @@
 #include <wingdi.h>
 #include <winspool.h>
 
+#include <string>
+
 #include "base/basictypes.h"
 #include "base/file_path.h"
 #include "base/file_util.h"
+#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
-#include "base/scoped_ptr.h"
 #include "base/scoped_temp_dir.h"
 #include "base/win/scoped_hdc.h"
 #include "printing/printing_context.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "ui/gfx/point.h"
+#include "ui/gfx/size.h"
 
 namespace {
 
@@ -39,28 +43,30 @@ const uint32 EMF_HEADER_SIZE = 128;
 
 }  // namespace
 
+namespace printing {
+
 TEST(EmfTest, DC) {
   // Simplest use case.
-  printing::Emf emf;
-  RECT rect = {100, 100, 200, 200};
-  HDC hdc = CreateCompatibleDC(NULL);
-  EXPECT_TRUE(hdc != NULL);
-  EXPECT_TRUE(emf.CreateDc(hdc, &rect));
-  EXPECT_TRUE(emf.hdc() != NULL);
-  // In theory, you'd use the HDC with GDI functions here.
-  EXPECT_TRUE(emf.CloseDc());
-  uint32 size = emf.GetDataSize();
-  EXPECT_EQ(size, EMF_HEADER_SIZE);
+  uint32 size;
   std::vector<BYTE> data;
-  EXPECT_TRUE(emf.GetData(&data));
-  EXPECT_EQ(data.size(), size);
-  emf.CloseEmf();
-  EXPECT_TRUE(DeleteDC(hdc));
+  {
+    Emf emf;
+    EXPECT_TRUE(emf.Init());
+    EXPECT_TRUE(emf.context() != NULL);
+    // An empty EMF is invalid, so we put at least a rectangle in it.
+    ::Rectangle(emf.context(), 10, 10, 190, 190);
+    EXPECT_TRUE(emf.FinishDocument());
+    size = emf.GetDataSize();
+    EXPECT_GT(size, EMF_HEADER_SIZE);
+    EXPECT_TRUE(emf.GetDataAsVector(&data));
+    EXPECT_EQ(data.size(), size);
+  }
 
   // Playback the data.
-  hdc = CreateCompatibleDC(NULL);
+  Emf emf;
+  EXPECT_TRUE(emf.InitFromData(&data.front(), size));
+  HDC hdc = CreateCompatibleDC(NULL);
   EXPECT_TRUE(hdc);
-  EXPECT_TRUE(emf.Init(&data.front(), size));
   RECT output_rect = {0, 0, 10, 10};
   EXPECT_TRUE(emf.Playback(hdc, &output_rect));
   EXPECT_TRUE(DeleteDC(hdc));
@@ -71,15 +77,14 @@ TEST_F(EmfPrintingTest, Enumerate) {
   if (IsTestCaseDisabled())
     return;
 
-  printing::PrintSettings settings;
+  PrintSettings settings;
 
   // My test case is a HP Color LaserJet 4550 PCL.
   settings.set_device_name(L"UnitTest Printer");
 
   // Initialize it.
-  scoped_ptr<printing::PrintingContext> context(
-      printing::PrintingContext::Create(std::string()));
-  EXPECT_EQ(context->InitWithSettings(settings), printing::PrintingContext::OK);
+  scoped_ptr<PrintingContext> context(PrintingContext::Create(std::string()));
+  EXPECT_EQ(context->InitWithSettings(settings), PrintingContext::OK);
 
   FilePath emf_file;
   EXPECT_TRUE(PathService::Get(base::DIR_SOURCE_ROOT, &emf_file));
@@ -88,22 +93,22 @@ TEST_F(EmfPrintingTest, Enumerate) {
                      .Append(FILE_PATH_LITERAL("data"))
                      .Append(FILE_PATH_LITERAL("test4.emf"));
   // Load any EMF with an image.
-  printing::Emf emf;
+  Emf emf;
   std::string emf_data;
   file_util::ReadFileToString(emf_file, &emf_data);
   ASSERT_TRUE(emf_data.size());
-  EXPECT_TRUE(emf.Init(&emf_data[0], emf_data.size()));
+  EXPECT_TRUE(emf.InitFromData(&emf_data[0], emf_data.size()));
 
   // This will print to file. The reason is that when running inside a
-  // unit_test, printing::PrintingContext automatically dumps its files to the
+  // unit_test, PrintingContext automatically dumps its files to the
   // current directory.
   // TODO(maruel):  Clean the .PRN file generated in current directory.
   context->NewDocument(L"EmfTest.Enumerate");
   context->NewPage();
   // Process one at a time.
-  printing::Emf::Enumerator emf_enum(emf, context->context(),
-                                &emf.GetBounds().ToRECT());
-  for (printing::Emf::Enumerator::const_iterator itr = emf_enum.begin();
+  Emf::Enumerator emf_enum(emf, context->context(),
+                           &emf.GetPageBounds(1).ToRECT());
+  for (Emf::Enumerator::const_iterator itr = emf_enum.begin();
        itr != emf_enum.end();
        ++itr) {
     // To help debugging.
@@ -120,33 +125,37 @@ TEST_F(EmfPrintingTest, Enumerate) {
 
 // Disabled if no "UnitTest printer" exists.
 TEST_F(EmfPrintingTest, PageBreak) {
-  base::win::ScopedHDC dc(
+  base::win::ScopedCreateDC dc(
       CreateDC(L"WINSPOOL", L"UnitTest Printer", NULL, NULL));
   if (!dc.Get())
     return;
-  printing::Emf emf;
-  EXPECT_TRUE(emf.CreateDc(dc.Get(), NULL));
-  EXPECT_TRUE(emf.hdc() != NULL);
-  int pages = 3;
-  while (pages) {
-    EXPECT_TRUE(emf.StartPage());
-    ::Rectangle(emf.hdc(), 10, 10, 190, 190);
-    EXPECT_TRUE(emf.EndPage());
-    --pages;
-  }
-  EXPECT_TRUE(emf.CloseDc());
-  uint32 size = emf.GetDataSize();
+  uint32 size;
   std::vector<BYTE> data;
-  EXPECT_TRUE(emf.GetData(&data));
-  EXPECT_EQ(data.size(), size);
-  emf.CloseEmf();
+  {
+    Emf emf;
+    EXPECT_TRUE(emf.Init());
+    EXPECT_TRUE(emf.context() != NULL);
+    int pages = 3;
+    while (pages) {
+      EXPECT_TRUE(emf.StartPage(gfx::Size(), gfx::Rect(), 1));
+      ::Rectangle(emf.context(), 10, 10, 190, 190);
+      EXPECT_TRUE(emf.FinishPage());
+      --pages;
+    }
+    EXPECT_EQ(3U, emf.GetPageCount());
+    EXPECT_TRUE(emf.FinishDocument());
+    size = emf.GetDataSize();
+    EXPECT_TRUE(emf.GetDataAsVector(&data));
+    EXPECT_EQ(data.size(), size);
+  }
 
   // Playback the data.
   DOCINFO di = {0};
   di.cbSize = sizeof(DOCINFO);
   di.lpszDocName = L"Test Job";
   int job_id = ::StartDoc(dc.Get(), &di);
-  EXPECT_TRUE(emf.Init(&data.front(), size));
+  Emf emf;
+  EXPECT_TRUE(emf.InitFromData(&data.front(), size));
   EXPECT_TRUE(emf.SafePlayback(dc.Get()));
   ::EndDoc(dc.Get());
   // Since presumably the printer is not real, let us just delete the job from
@@ -158,40 +167,39 @@ TEST_F(EmfPrintingTest, PageBreak) {
   }
 }
 
-TEST(EmfTest, FileBackedDC) {
+TEST(EmfTest, FileBackedEmf) {
   // Simplest use case.
-  printing::Emf emf;
-  RECT rect = {100, 100, 200, 200};
-  HDC hdc = CreateCompatibleDC(NULL);
-  EXPECT_TRUE(hdc != NULL);
   ScopedTempDir scratch_metafile_dir;
   ASSERT_TRUE(scratch_metafile_dir.CreateUniqueTempDir());
   FilePath metafile_path;
   EXPECT_TRUE(file_util::CreateTemporaryFileInDir(scratch_metafile_dir.path(),
                                                   &metafile_path));
-  EXPECT_TRUE(emf.CreateFileBackedDc(hdc, &rect, metafile_path));
-  EXPECT_TRUE(emf.hdc() != NULL);
-  // In theory, you'd use the HDC with GDI functions here.
-  EXPECT_TRUE(emf.CloseDc());
-
-  uint32 size = emf.GetDataSize();
-  EXPECT_EQ(size, EMF_HEADER_SIZE);
+  uint32 size;
   std::vector<BYTE> data;
-  EXPECT_TRUE(emf.GetData(&data));
-  EXPECT_EQ(data.size(), size);
-  emf.CloseEmf();
+  {
+    Emf emf;
+    EXPECT_TRUE(emf.InitToFile(metafile_path));
+    EXPECT_TRUE(emf.context() != NULL);
+    // An empty EMF is invalid, so we put at least a rectangle in it.
+    ::Rectangle(emf.context(), 10, 10, 190, 190);
+    EXPECT_TRUE(emf.FinishDocument());
+    size = emf.GetDataSize();
+    EXPECT_GT(size, EMF_HEADER_SIZE);
+    EXPECT_TRUE(emf.GetDataAsVector(&data));
+    EXPECT_EQ(data.size(), size);
+  }
   int64 file_size = 0;
   file_util::GetFileSize(metafile_path, &file_size);
   EXPECT_EQ(size, file_size);
-  EXPECT_TRUE(DeleteDC(hdc));
 
   // Playback the data.
-  hdc = CreateCompatibleDC(NULL);
+  HDC hdc = CreateCompatibleDC(NULL);
   EXPECT_TRUE(hdc);
-  EXPECT_TRUE(emf.CreateFromFile(metafile_path));
+  Emf emf;
+  EXPECT_TRUE(emf.InitFromFile(metafile_path));
   RECT output_rect = {0, 0, 10, 10};
   EXPECT_TRUE(emf.Playback(hdc, &output_rect));
   EXPECT_TRUE(DeleteDC(hdc));
-  emf.CloseEmf();
 }
 
+}  // namespace printing
