@@ -1,14 +1,18 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "printing/image.h"
 
+#include <algorithm>
+
 #include "base/file_util.h"
 #include "base/md5.h"
 #include "base/string_number_conversions.h"
-#include "gfx/codec/png_codec.h"
+#include "printing/metafile.h"
+#include "printing/metafile_impl.h"
 #include "third_party/skia/include/core/SkColor.h"
+#include "ui/gfx/codec/png_codec.h"
 
 namespace printing {
 
@@ -32,7 +36,7 @@ Image::Image(const FilePath& path)
   }
 }
 
-Image::Image(const NativeMetafile& metafile)
+Image::Image(const Metafile& metafile)
     : row_length_(0),
       ignore_alpha_(true) {
   LoadMetafile(metafile);
@@ -48,8 +52,8 @@ Image::Image(const Image& image)
 Image::~Image() {}
 
 std::string Image::checksum() const {
-  MD5Digest digest;
-  MD5Sum(&data_[0], data_.size(), &digest);
+  base::MD5Digest digest;
+  base::MD5Sum(&data_[0], data_.size(), &digest);
   return base::HexEncode(&digest, sizeof(digest));
 }
 
@@ -58,10 +62,10 @@ bool Image::SaveToPng(const FilePath& filepath) const {
   std::vector<unsigned char> compressed;
   bool success = gfx::PNGCodec::Encode(&*data_.begin(),
                                        gfx::PNGCodec::FORMAT_BGRA,
-                                       size_.width(),
-                                       size_.height(),
+                                       size_,
                                        row_length_,
                                        true,
+                                       std::vector<gfx::PNGCodec::Comment>(),
                                        &compressed);
   DCHECK(success && compressed.size());
   if (success) {
@@ -145,7 +149,8 @@ bool Image::LoadPng(const std::string& compressed) {
 bool Image::LoadMetafile(const std::string& data) {
   DCHECK(!data.empty());
   NativeMetafile metafile;
-  metafile.Init(data.data(), data.size());
+  if (!metafile.InitFromData(data.data(), data.size()))
+    return false;
   return LoadMetafile(metafile);
 }
 
