@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,17 +14,17 @@
 #include "base/i18n/file_util_icu.h"
 #include "base/lazy_instance.h"
 #include "base/message_loop.h"
+#include "base/stringprintf.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
 #include "base/i18n/time_formatting.h"
-#include "gfx/font.h"
 #include "printing/page_number.h"
-#include "printing/page_overlays.h"
 #include "printing/printed_pages_source.h"
 #include "printing/printed_page.h"
 #include "printing/units.h"
 #include "skia/ext/platform_device.h"
 #include "ui/base/text/text_elider.h"
+#include "ui/gfx/font.h"
 
 namespace {
 
@@ -37,8 +37,8 @@ struct PrintDebugDumpPath {
   FilePath debug_dump_path;
 };
 
-static base::LazyInstance<PrintDebugDumpPath> g_debug_dump_info(
-    base::LINKER_INITIALIZED);
+base::LazyInstance<PrintDebugDumpPath> g_debug_dump_info =
+    LAZY_INSTANCE_INITIALIZER;
 
 }  // namespace
 
@@ -64,11 +64,10 @@ PrintedDocument::~PrintedDocument() {
 }
 
 void PrintedDocument::SetPage(int page_number,
-                              NativeMetafile* metafile,
+                              Metafile* metafile,
                               double shrink,
                               const gfx::Size& paper_size,
-                              const gfx::Rect& page_rect,
-                              bool has_visible_overlays) {
+                              const gfx::Rect& page_rect) {
   // Notice the page_number + 1, the reason is that this is the value that will
   // be shown. Users dislike 0-based counting.
   scoped_refptr<PrintedPage> page(
@@ -76,15 +75,15 @@ void PrintedDocument::SetPage(int page_number,
                       metafile,
                       paper_size,
                       page_rect,
-                      has_visible_overlays));
+                      shrink));
   {
     base::AutoLock lock(lock_);
     mutable_.pages_[page_number] = page;
-    if (mutable_.shrink_factor == 0) {
-      mutable_.shrink_factor = shrink;
-    } else {
-      DCHECK_EQ(mutable_.shrink_factor, shrink);
-    }
+
+#if defined(OS_POSIX) && !defined(OS_MACOSX)
+    if (page_number < mutable_.first_page)
+      mutable_.first_page = page_number;
+#endif
   }
   DebugDump(*page);
 }
@@ -109,10 +108,17 @@ bool PrintedDocument::IsComplete() const {
   PageNumber page(immutable_.settings_, mutable_.page_count_);
   if (page == PageNumber::npos())
     return false;
+
   for (; page != PageNumber::npos(); ++page) {
+#if defined(OS_WIN) || defined(OS_MACOSX)
+    const bool metafile_must_be_valid = true;
+#elif defined(OS_POSIX)
+    const bool metafile_must_be_valid = (page.ToInt() == mutable_.first_page);
+#endif
     PrintedPages::const_iterator itr = mutable_.pages_.find(page.ToInt());
-    if (itr == mutable_.pages_.end() || !itr->second.get() ||
-        !itr->second->native_metafile())
+    if (itr == mutable_.pages_.end() || !itr->second.get())
+      return false;
+    if (metafile_must_be_valid && !itr->second->metafile())
       return false;
   }
   return true;
@@ -138,7 +144,7 @@ uint32 PrintedDocument::MemoryUsage() const {
   }
   uint32 total = 0;
   for (size_t i = 0; i < pages_copy.size(); ++i) {
-    total += pages_copy[i]->native_metafile()->GetDataSize();
+    total += pages_copy[i]->metafile()->GetDataSize();
   }
   return total;
 }
@@ -166,94 +172,22 @@ int PrintedDocument::expected_page_count() const {
   return mutable_.expected_page_count_;
 }
 
-void PrintedDocument::PrintHeaderFooter(gfx::NativeDrawingContext context,
-                                        const PrintedPage& page,
-                                        PageOverlays::HorizontalPosition x,
-                                        PageOverlays::VerticalPosition y,
-                                        const gfx::Font& font) const {
-  const PrintSettings& settings = immutable_.settings_;
-  if (!settings.use_overlays || !page.has_visible_overlays()) {
-    return;
-  }
-  const std::wstring& line = settings.overlays.GetOverlay(x, y);
-  if (line.empty()) {
-    return;
-  }
-  std::wstring output(PageOverlays::ReplaceVariables(line, *this, page));
-  if (output.empty()) {
-    // May happen if document name or url is empty.
-    return;
-  }
-  const gfx::Size string_size(font.GetStringWidth(WideToUTF16Hack(output)),
-                              font.GetHeight());
-  gfx::Rect bounding;
-  bounding.set_height(string_size.height());
-  const gfx::Rect& overlay_area(
-      settings.page_setup_device_units().overlay_area());
-  // Hard code .25 cm interstice between overlays. Make sure that some space is
-  // kept between each headers.
-  const int interstice = ConvertUnit(250, kHundrethsMMPerInch,
-                                     settings.device_units_per_inch());
-  const int max_width = overlay_area.width() / 3 - interstice;
-  const int actual_width = std::min(string_size.width(), max_width);
-  switch (x) {
-    case PageOverlays::LEFT:
-      bounding.set_x(overlay_area.x());
-      bounding.set_width(max_width);
-      break;
-    case PageOverlays::CENTER:
-      bounding.set_x(overlay_area.x() +
-                     (overlay_area.width() - actual_width) / 2);
-      bounding.set_width(actual_width);
-      break;
-    case PageOverlays::RIGHT:
-      bounding.set_x(overlay_area.right() - actual_width);
-      bounding.set_width(actual_width);
-      break;
-  }
-
-  DCHECK_LE(bounding.right(), overlay_area.right());
-
-  switch (y) {
-    case PageOverlays::BOTTOM:
-      bounding.set_y(overlay_area.bottom() - string_size.height());
-      break;
-    case PageOverlays::TOP:
-      bounding.set_y(overlay_area.y());
-      break;
-  }
-
-  if (string_size.width() > bounding.width()) {
-    if (line == PageOverlays::kUrl) {
-      output = UTF16ToWideHack(ui::ElideUrl(url(), font, bounding.width(),
-                                            std::wstring()));
-    } else {
-      output = UTF16ToWideHack(ui::ElideText(WideToUTF16Hack(output),
-          font, bounding.width(), false));
-    }
-  }
-
-  DrawHeaderFooter(context, output, bounding);
-}
-
 void PrintedDocument::DebugDump(const PrintedPage& page) {
   if (!g_debug_dump_info.Get().enabled)
     return;
 
   string16 filename;
-  filename += date();
-  filename += ASCIIToUTF16("_");
-  filename += time();
-  filename += ASCIIToUTF16("_");
   filename += name();
   filename += ASCIIToUTF16("_");
   filename += ASCIIToUTF16(StringPrintf("%02d", page.page_number()));
-  filename += ASCIIToUTF16("_.emf");
 #if defined(OS_WIN)
-  page.native_metafile()->SaveTo(
-      g_debug_dump_info.Get().debug_dump_path.Append(filename).ToWStringHack());
+  filename += ASCIIToUTF16("_.emf");
+  page.metafile()->SaveTo(
+      g_debug_dump_info.Get().debug_dump_path.Append(filename));
 #else  // OS_WIN
-  NOTIMPLEMENTED();
+  filename += ASCIIToUTF16("_.pdf");
+  page.metafile()->SaveTo(
+      g_debug_dump_info.Get().debug_dump_path.Append(UTF16ToUTF8(filename)));
 #endif  // OS_WIN
 }
 
@@ -269,8 +203,10 @@ const FilePath& PrintedDocument::debug_dump_path() {
 PrintedDocument::Mutable::Mutable(PrintedPagesSource* source)
     : source_(source),
       expected_page_count_(0),
-      page_count_(0),
-      shrink_factor(0) {
+      page_count_(0) {
+#if defined(OS_POSIX) && !defined(OS_MACOSX)
+  first_page = INT_MAX;
+#endif
 }
 
 PrintedDocument::Mutable::~Mutable() {
@@ -282,12 +218,17 @@ PrintedDocument::Immutable::Immutable(const PrintSettings& settings,
     : settings_(settings),
       source_message_loop_(MessageLoop::current()),
       name_(source->RenderSourceName()),
-      url_(source->RenderSourceUrl()),
       cookie_(cookie) {
-  SetDocumentDate();
 }
 
 PrintedDocument::Immutable::~Immutable() {
 }
 
+#if defined(OS_POSIX) && defined(USE_AURA)
+// This function is not used on aura linux/chromeos.
+void PrintedDocument::RenderPrintedPage(const PrintedPage& page,
+                                        PrintingContext* context) const {
+}
+#endif
+
 }  // namespace printing
