@@ -6,17 +6,94 @@
 
 #include <winspool.h>
 
+#include <algorithm>
+
 #include "base/i18n/file_util_icu.h"
 #include "base/i18n/time_formatting.h"
 #include "base/message_loop.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
+#include "base/values.h"
+#include "printing/print_job_constants.h"
 #include "printing/print_settings_initializer_win.h"
 #include "printing/printed_document.h"
-#include "skia/ext/platform_device_win.h"
+#include "printing/units.h"
+#include "skia/ext/platform_device.h"
 
 using base::Time;
 
+namespace {
+
+// Constants for setting default PDF settings.
+const int kPDFDpi = 300;  // 300 dpi
+// LETTER: 8.5 x 11 inches
+const int kPDFLetterWidth = 8.5 * kPDFDpi;
+const int kPDFLetterHeight = 11 * kPDFDpi;
+// LEGAL: 8.5 x 14 inches
+const int kPDFLegalWidth = 8.5 * kPDFDpi;
+const int kPDFLegalHeight = 14 * kPDFDpi;
+// A4: 8.27 x 11.69 inches
+const int kPDFA4Width = 8.27 * kPDFDpi;
+const int kPDFA4Height = 11.69 * kPDFDpi;
+// A3: 11.69 x 16.54 inches
+const int kPDFA3Width = 11.69 * kPDFDpi;
+const int kPDFA3Height = 16.54 * kPDFDpi;
+
+// Retrieves the printer's PRINTER_INFO_* structure.
+// Output |level| can be 9 (user-default), 8 (admin-default), or 2
+// (printer-default).
+// |devmode| is a pointer points to the start of DEVMODE structure in
+// |buffer|.
+bool GetPrinterInfo(HANDLE printer,
+                    const std::wstring &device_name,
+                    int* level,
+                    scoped_array<uint8>* buffer,
+                    DEVMODE** dev_mode) {
+  DCHECK(buffer);
+
+  // A PRINTER_INFO_9 structure specifying the per-user default printer
+  // settings.
+  printing::PrintingContextWin::GetPrinterHelper(printer, 9, buffer);
+  if (buffer->get()) {
+    PRINTER_INFO_9* info_9 = reinterpret_cast<PRINTER_INFO_9*>(buffer->get());
+    if (info_9->pDevMode != NULL) {
+      *level = 9;
+      *dev_mode = info_9->pDevMode;
+      return true;
+    }
+    buffer->reset();
+  }
+
+  // A PRINTER_INFO_8 structure specifying the global default printer settings.
+  printing::PrintingContextWin::GetPrinterHelper(printer, 8, buffer);
+  if (buffer->get()) {
+    PRINTER_INFO_8* info_8 = reinterpret_cast<PRINTER_INFO_8*>(buffer->get());
+    if (info_8->pDevMode != NULL) {
+      *level = 8;
+      *dev_mode = info_8->pDevMode;
+      return true;
+    }
+    buffer->reset();
+  }
+
+  // A PRINTER_INFO_2 structure specifying the driver's default printer
+  // settings.
+  printing::PrintingContextWin::GetPrinterHelper(printer, 2, buffer);
+  if (buffer->get()) {
+    PRINTER_INFO_2* info_2 = reinterpret_cast<PRINTER_INFO_2*>(buffer->get());
+    if (info_2->pDevMode != NULL) {
+      *level = 2;
+      *dev_mode = info_2->pDevMode;
+      return true;
+    }
+    buffer->reset();
+  }
+
+  return false;
+}
+
+}  // anonymous namespace
+
 namespace printing {
 
 class PrintingContextWin::CallbackHandler : public IPrintDialogCallback,
@@ -136,10 +213,10 @@ PrintingContextWin::~PrintingContextWin() {
   ReleaseContext();
 }
 
-void PrintingContextWin::AskUserForSettings(HWND view,
-                                            int max_pages,
-                                            bool has_selection,
-                                            PrintSettingsCallback* callback) {
+void PrintingContextWin::AskUserForSettings(
+    gfx::NativeView view, int max_pages, bool has_selection,
+    const PrintSettingsCallback& callback) {
+#if !defined(USE_AURA)
   DCHECK(!in_print_job_);
   dialog_box_dismissed_ = false;
 
@@ -189,11 +266,12 @@ void PrintingContextWin::AskUserForSettings(HWND view,
 
   if ((*print_dialog_func_)(&dialog_options) != S_OK) {
     ResetSettings();
-    callback->Run(FAILED);
+    callback.Run(FAILED);
   }
 
   // TODO(maruel):  Support PD_PRINTTOFILE.
-  callback->Run(ParseDialogResultEx(dialog_options));
+  callback.Run(ParseDialogResultEx(dialog_options));
+#endif
 }
 
 PrintingContext::Result PrintingContextWin::UseDefaultSettings() {
@@ -201,11 +279,174 @@ PrintingContext::Result PrintingContextWin::UseDefaultSettings() {
 
   PRINTDLG dialog_options = { sizeof(PRINTDLG) };
   dialog_options.Flags = PD_RETURNDC | PD_RETURNDEFAULT;
-  if (PrintDlg(&dialog_options) == 0) {
-    ResetSettings();
-    return FAILED;
+  if (PrintDlg(&dialog_options))
+    return ParseDialogResult(dialog_options);
+
+  // No default printer configured, do we have any printers at all?
+  DWORD bytes_needed = 0;
+  DWORD count_returned = 0;
+  (void)::EnumPrinters(PRINTER_ENUM_LOCAL|PRINTER_ENUM_CONNECTIONS,
+                       NULL, 2, NULL, 0, &bytes_needed, &count_returned);
+  if (bytes_needed) {
+    DCHECK(bytes_needed >= count_returned * sizeof(PRINTER_INFO_2));
+    scoped_array<BYTE> printer_info_buffer(new BYTE[bytes_needed]);
+    BOOL ret = ::EnumPrinters(PRINTER_ENUM_LOCAL|PRINTER_ENUM_CONNECTIONS,
+                              NULL, 2, printer_info_buffer.get(),
+                              bytes_needed, &bytes_needed,
+                              &count_returned);
+    if (ret && count_returned) {  // have printers
+      // Open the first successfully found printer.
+      for (DWORD count = 0; count < count_returned; ++count) {
+        PRINTER_INFO_2* info_2 = reinterpret_cast<PRINTER_INFO_2*>(
+            printer_info_buffer.get() + count * sizeof(PRINTER_INFO_2));
+        std::wstring printer_name = info_2->pPrinterName;
+        if (info_2->pDevMode == NULL || printer_name.length() == 0)
+          continue;
+        if (!AllocateContext(printer_name, info_2->pDevMode, &context_))
+          break;
+        if (InitializeSettings(*info_2->pDevMode, printer_name,
+                               NULL, 0, false)) {
+          break;
+        }
+        ReleaseContext();
+      }
+      if (context_)
+        return OK;
+    }
   }
-  return ParseDialogResult(dialog_options);
+
+  ResetSettings();
+  return FAILED;
+}
+
+PrintingContext::Result PrintingContextWin::UpdatePrinterSettings(
+    const DictionaryValue& job_settings,
+    const PageRanges& ranges) {
+  DCHECK(!in_print_job_);
+
+  bool collate;
+  int color;
+  bool landscape;
+  bool print_to_pdf;
+  bool is_cloud_dialog;
+  int copies;
+  int duplex_mode;
+  string16 device_name;
+
+  if (!job_settings.GetBoolean(kSettingLandscape, &landscape) ||
+      !job_settings.GetBoolean(kSettingCollate, &collate) ||
+      !job_settings.GetInteger(kSettingColor, &color) ||
+      !job_settings.GetBoolean(kSettingPrintToPDF, &print_to_pdf) ||
+      !job_settings.GetInteger(kSettingDuplexMode, &duplex_mode) ||
+      !job_settings.GetInteger(kSettingCopies, &copies) ||
+      !job_settings.GetString(kSettingDeviceName, &device_name) ||
+      !job_settings.GetBoolean(kSettingCloudPrintDialog, &is_cloud_dialog)) {
+    return OnError();
+  }
+
+  bool print_to_cloud = job_settings.HasKey(kSettingCloudPrintId);
+
+  if (print_to_pdf || print_to_cloud || is_cloud_dialog) {
+    // Default fallback to Letter size.
+    gfx::Size paper_size;
+    gfx::Rect paper_rect;
+    paper_size.SetSize(kPDFLetterWidth, kPDFLetterHeight);
+
+    // Get settings from locale. Paper type buffer length is at most 4.
+    const int paper_type_buffer_len = 4;
+    wchar_t paper_type_buffer[paper_type_buffer_len] = {0};
+    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IPAPERSIZE, paper_type_buffer,
+                  paper_type_buffer_len);
+    if (wcslen(paper_type_buffer)) {  // The call succeeded.
+      int paper_code = _wtoi(paper_type_buffer);
+      switch (paper_code) {
+        case DMPAPER_LEGAL:
+          paper_size.SetSize(kPDFLegalWidth, kPDFLegalHeight);
+          break;
+        case DMPAPER_A4:
+          paper_size.SetSize(kPDFA4Width, kPDFA4Height);
+          break;
+        case DMPAPER_A3:
+          paper_size.SetSize(kPDFA3Width, kPDFA3Height);
+          break;
+        default:  // DMPAPER_LETTER is used for default fallback.
+          break;
+      }
+    }
+    paper_rect.SetRect(0, 0, paper_size.width(), paper_size.height());
+    settings_.SetPrinterPrintableArea(paper_size, paper_rect, kPDFDpi);
+    settings_.set_dpi(kPDFDpi);
+    settings_.SetOrientation(landscape);
+    settings_.ranges = ranges;
+    return OK;
+  }
+
+  HANDLE printer;
+  LPWSTR device_name_wide = const_cast<wchar_t*>(device_name.c_str());
+  if (!OpenPrinter(device_name_wide, &printer, NULL))
+    return OnError();
+
+  // Make printer changes local to Chrome.
+  // See MSDN documentation regarding DocumentProperties.
+  scoped_array<uint8> buffer;
+  DEVMODE* dev_mode = NULL;
+  LONG buffer_size = DocumentProperties(NULL, printer, device_name_wide,
+                                        NULL, NULL, 0);
+  if (buffer_size > 0) {
+    buffer.reset(new uint8[buffer_size]);
+    memset(buffer.get(), 0, buffer_size);
+    if (DocumentProperties(NULL, printer, device_name_wide,
+                           reinterpret_cast<PDEVMODE>(buffer.get()), NULL,
+                           DM_OUT_BUFFER) == IDOK) {
+      dev_mode = reinterpret_cast<PDEVMODE>(buffer.get());
+    }
+  }
+  if (dev_mode == NULL) {
+    buffer.reset();
+    ClosePrinter(printer);
+    return OnError();
+  }
+
+  if (color == GRAY)
+    dev_mode->dmColor = DMCOLOR_MONOCHROME;
+  else
+    dev_mode->dmColor = DMCOLOR_COLOR;
+
+  dev_mode->dmCopies = std::max(copies, 1);
+  if (dev_mode->dmCopies > 1)  // do not change collate unless multiple copies
+    dev_mode->dmCollate = collate ? DMCOLLATE_TRUE : DMCOLLATE_FALSE;
+  switch (duplex_mode) {
+    case LONG_EDGE:
+      dev_mode->dmDuplex = DMDUP_VERTICAL;
+      break;
+    case SHORT_EDGE:
+      dev_mode->dmDuplex = DMDUP_HORIZONTAL;
+      break;
+    case SIMPLEX:
+      dev_mode->dmDuplex = DMDUP_SIMPLEX;
+      break;
+    default:  // UNKNOWN_DUPLEX_MODE
+      break;
+  }
+  dev_mode->dmOrientation = landscape ? DMORIENT_LANDSCAPE : DMORIENT_PORTRAIT;
+
+  // Update data using DocumentProperties.
+  if (DocumentProperties(NULL, printer, device_name_wide, dev_mode, dev_mode,
+                         DM_IN_BUFFER | DM_OUT_BUFFER) != IDOK) {
+    ClosePrinter(printer);
+    return OnError();
+  }
+
+  // Set printer then refresh printer settings.
+  if (!AllocateContext(device_name, dev_mode, &context_)) {
+    ClosePrinter(printer);
+    return OnError();
+  }
+  PrintSettingsInitializerWin::InitPrintSettings(context_, *dev_mode,
+                                                 ranges, device_name,
+                                                 false, &settings_);
+  ClosePrinter(printer);
+  return OK;
 }
 
 PrintingContext::Result PrintingContextWin::InitWithSettings(
@@ -287,13 +528,11 @@ PrintingContext::Result PrintingContextWin::NewDocument(
 PrintingContext::Result PrintingContextWin::NewPage() {
   if (abort_printing_)
     return CANCEL;
-
   DCHECK(context_);
   DCHECK(in_print_job_);
 
-  // Inform the driver that the application is about to begin sending data.
-  if (StartPage(context_) <= 0)
-    return OnError();
+  // Intentional No-op. NativeMetafile::SafePlayback takes care of calling
+  // ::StartPage().
 
   return OK;
 }
@@ -303,8 +542,9 @@ PrintingContext::Result PrintingContextWin::PageDone() {
     return CANCEL;
   DCHECK(in_print_job_);
 
-  if (EndPage(context_) <= 0)
-    return OnError();
+  // Intentional No-op. NativeMetafile::SafePlayback takes care of calling
+  // ::EndPage().
+
   return OK;
 }
 
@@ -327,10 +567,6 @@ void PrintingContextWin::Cancel() {
   in_print_job_ = false;
   if (context_)
     CancelDC(context_);
-  DismissDialog();
-}
-
-void PrintingContextWin::DismissDialog() {
   if (dialog_box_) {
     DestroyWindow(dialog_box_);
     dialog_box_dismissed_ = true;
@@ -363,7 +599,7 @@ bool PrintingContextWin::InitializeSettings(const DEVMODE& dev_mode,
                                             const PRINTPAGERANGE* ranges,
                                             int number_ranges,
                                             bool selection_only) {
-  skia::PlatformDevice::InitializeDC(context_);
+  skia::InitializeDC(context_);
   DCHECK(GetDeviceCaps(context_, CLIPCAPS));
   DCHECK(GetDeviceCaps(context_, RASTERCAPS) & RC_STRETCHDIB);
   DCHECK(GetDeviceCaps(context_, RASTERCAPS) & RC_BITMAP64);
@@ -409,60 +645,24 @@ bool PrintingContextWin::GetPrinterSettings(HANDLE printer,
                                             const std::wstring& device_name) {
   DCHECK(!in_print_job_);
   scoped_array<uint8> buffer;
+  int level = 0;
+  DEVMODE* dev_mode = NULL;
 
-  // A PRINTER_INFO_9 structure specifying the per-user default printer
-  // settings.
-  GetPrinterHelper(printer, 9, &buffer);
-  if (buffer.get()) {
-    PRINTER_INFO_9* info_9 = reinterpret_cast<PRINTER_INFO_9*>(buffer.get());
-    if (info_9->pDevMode != NULL) {
-      if (!AllocateContext(device_name, info_9->pDevMode, &context_)) {
-        ResetSettings();
-        return false;
-      }
-      return InitializeSettings(*info_9->pDevMode, device_name, NULL, 0, false);
-    }
-    buffer.reset();
+  if (GetPrinterInfo(printer, device_name, &level, &buffer, &dev_mode) &&
+      AllocateContext(device_name, dev_mode, &context_)) {
+    return InitializeSettings(*dev_mode, device_name, NULL, 0, false);
   }
 
-  // A PRINTER_INFO_8 structure specifying the global default printer settings.
-  GetPrinterHelper(printer, 8, &buffer);
-  if (buffer.get()) {
-    PRINTER_INFO_8* info_8 = reinterpret_cast<PRINTER_INFO_8*>(buffer.get());
-    if (info_8->pDevMode != NULL) {
-      if (!AllocateContext(device_name, info_8->pDevMode, &context_)) {
-        ResetSettings();
-        return false;
-      }
-      return InitializeSettings(*info_8->pDevMode, device_name, NULL, 0, false);
-    }
-    buffer.reset();
-  }
-
-  // A PRINTER_INFO_2 structure specifying the driver's default printer
-  // settings.
-  GetPrinterHelper(printer, 2, &buffer);
-  if (buffer.get()) {
-    PRINTER_INFO_2* info_2 = reinterpret_cast<PRINTER_INFO_2*>(buffer.get());
-    if (info_2->pDevMode != NULL) {
-      if (!AllocateContext(device_name, info_2->pDevMode, &context_)) {
-        ResetSettings();
-        return false;
-      }
-      return InitializeSettings(*info_2->pDevMode, device_name, NULL, 0, false);
-    }
-    buffer.reset();
-  }
-  // Failed to retrieve the printer settings.
+  buffer.reset();
   ResetSettings();
   return false;
 }
 
 // static
-bool PrintingContextWin::AllocateContext(const std::wstring& printer_name,
+bool PrintingContextWin::AllocateContext(const std::wstring& device_name,
                                          const DEVMODE* dev_mode,
                                          gfx::NativeDrawingContext* context) {
-  *context = CreateDC(L"WINSPOOL", printer_name.c_str(), NULL, dev_mode);
+  *context = CreateDC(L"WINSPOOL", device_name.c_str(), NULL, dev_mode);
   DCHECK(*context);
   return *context != NULL;
 }
