@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,35 +10,38 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "build/build_config.h"
-#include "gfx/native_widget_types.h"
 #include "printing/printing_context.h"
+#include "ui/gfx/native_widget_types.h"
 
 namespace printing {
 
-class PrintingContextWin : public PrintingContext {
+class PRINTING_EXPORT PrintingContextWin : public PrintingContext {
  public:
   explicit PrintingContextWin(const std::string& app_locale);
   ~PrintingContextWin();
 
   // PrintingContext implementation.
-  virtual void AskUserForSettings(gfx::NativeView parent_view,
-                                  int max_pages,
-                                  bool has_selection,
-                                  PrintSettingsCallback* callback);
-  virtual Result UseDefaultSettings();
-  virtual Result InitWithSettings(const PrintSettings& settings);
-  virtual Result NewDocument(const string16& document_name);
-  virtual Result NewPage();
-  virtual Result PageDone();
-  virtual Result DocumentDone();
-  virtual void Cancel();
-  virtual void DismissDialog();
-  virtual void ReleaseContext();
-  virtual gfx::NativeDrawingContext context() const;
+  virtual void AskUserForSettings(
+      gfx::NativeView parent_view,
+      int max_pages,
+      bool has_selection,
+      const PrintSettingsCallback& callback) OVERRIDE;
+  virtual Result UseDefaultSettings() OVERRIDE;
+  virtual Result UpdatePrinterSettings(
+      const base::DictionaryValue& job_settings,
+      const PageRanges& ranges) OVERRIDE;
+  virtual Result InitWithSettings(const PrintSettings& settings) OVERRIDE;
+  virtual Result NewDocument(const string16& document_name) OVERRIDE;
+  virtual Result NewPage() OVERRIDE;
+  virtual Result PageDone() OVERRIDE;
+  virtual Result DocumentDone() OVERRIDE;
+  virtual void Cancel() OVERRIDE;
+  virtual void ReleaseContext() OVERRIDE;
+  virtual gfx::NativeDrawingContext context() const OVERRIDE;
 
-#if defined(UNIT_TEST)
+#if defined(UNIT_TEST) || defined(PRINTING_IMPLEMENTATION)
   // Sets a fake PrintDlgEx function pointer in tests.
   void SetPrintDialog(HRESULT (__stdcall *print_dialog_func)(LPPRINTDLGEX)) {
     print_dialog_func_ = print_dialog_func;
