@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,11 +14,13 @@ DataPlanes::DataPlanes() {
 }
 
 CaptureData::CaptureData(const DataPlanes &data_planes,
-                         int width,
-                         int height,
-                         media::VideoFrame::Format format) :
-    data_planes_(data_planes), dirty_rects_(),
-    width_(width), height_(height), pixel_format_(format) {
+                         const SkISize& size,
+                         media::VideoFrame::Format format)
+    : data_planes_(data_planes),
+      size_(size),
+      pixel_format_(format),
+      capture_time_ms_(0),
+      client_sequence_number_(0) {
 }
 
 CaptureData::~CaptureData() {}
