@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,9 +8,9 @@
 #include <vector>
 
 #include "base/basictypes.h"
-#include "base/ref_counted.h"
+#include "base/memory/ref_counted.h"
 #include "media/base/video_frame.h"
-#include "remoting/base/types.h"
+#include "third_party/skia/include/core/SkRegion.h"
 
 namespace remoting {
 
@@ -27,36 +27,50 @@ struct DataPlanes {
 class CaptureData : public base::RefCountedThreadSafe<CaptureData> {
  public:
   CaptureData(const DataPlanes &data_planes,
-              int width,
-              int height,
+              const SkISize& size,
               media::VideoFrame::Format format);
 
-  // Get the data_planes data of the last capture.
+  // Get the data_planes data of the previous capture.
   const DataPlanes& data_planes() const { return data_planes_; }
 
-  // Get the list of updated rectangles in the last capture. The result is
-  // written into |rects|.
-  const InvalidRects& dirty_rects() const { return dirty_rects_; }
+  // Get the dirty region from the previous capture.
+  const SkRegion& dirty_region() const { return dirty_region_; }
 
-  // Get the width of the image captured.
-  int width() const { return width_; }
-
-  // Get the height of the image captured.
-  int height() const { return height_; }
+  // Return the size of the image captured.
+  SkISize size() const { return size_; }
 
   // Get the pixel format of the image captured.
   media::VideoFrame::Format pixel_format() const { return pixel_format_; }
 
   // Mutating methods.
-  InvalidRects& mutable_dirty_rects() { return dirty_rects_; }
+  SkRegion& mutable_dirty_region() { return dirty_region_; }
+
+  // Return the time spent on capturing.
+  int capture_time_ms() const { return capture_time_ms_; }
+
+  // Set the time spent on capturing.
+  void set_capture_time_ms(int capture_time_ms) {
+    capture_time_ms_ = capture_time_ms;
+  }
+
+  int64 client_sequence_number() const { return client_sequence_number_; }
+
+  void set_client_sequence_number(int64 client_sequence_number) {
+    client_sequence_number_ = client_sequence_number;
+  }
 
  private:
   const DataPlanes data_planes_;
-  InvalidRects dirty_rects_;
-  int width_;
-  int height_;
+  SkRegion dirty_region_;
+  SkISize size_;
   media::VideoFrame::Format pixel_format_;
 
+  // Time spent in capture. Unit is in milliseconds.
+  int capture_time_ms_;
+
+  // Sequence number supplied by client for performance tracking.
+  int64 client_sequence_number_;
+
   friend class base::RefCountedThreadSafe<CaptureData>;
   virtual ~CaptureData();
 };
