@@ -1,16 +1,18 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <deque>
 #include <stdlib.h>
 
-#include "gfx/rect.h"
+#include "base/bind.h"
+#include "base/logging.h"
+#include "base/memory/scoped_ptr.h"
 #include "media/base/video_frame.h"
+#include "remoting/base/base_mock_objects.h"
 #include "remoting/base/codec_test.h"
 #include "remoting/base/decoder.h"
 #include "remoting/base/encoder.h"
-#include "remoting/base/mock_objects.h"
 #include "remoting/base/util.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -19,12 +21,12 @@ static const int kHeight = 240;
 static const int kBytesPerPixel = 4;
 
 // Some sample rects for testing.
-static const gfx::Rect kTestRects[] = {
-  gfx::Rect(0, 0, kWidth, kHeight),
-  gfx::Rect(0, 0, kWidth / 2, kHeight / 2),
-  gfx::Rect(kWidth / 2, kHeight / 2, kWidth / 2, kHeight / 2),
-  gfx::Rect(16, 16, 16, 16),
-  gfx::Rect(128, 64, 32, 32),
+static const SkIRect kTestRects[] = {
+  SkIRect::MakeXYWH(0, 0, kWidth, kHeight),
+  SkIRect::MakeXYWH(0, 0, kWidth / 2, kHeight / 2),
+  SkIRect::MakeXYWH(kWidth / 2, kHeight / 2, kWidth / 2, kHeight / 2),
+  SkIRect::MakeXYWH(16, 16, 16, 16),
+  SkIRect::MakeXYWH(128, 64, 32, 32),
 };
 
 namespace remoting {
@@ -58,10 +60,10 @@ class EncoderMessageTester {
       ++begin_rect_;
 
       if (strict_) {
-        gfx::Rect rect = rects_.front();
+        SkIRect rect = rects_.front();
         rects_.pop_front();
-        EXPECT_EQ(rect.x(), packet->format().x());
-        EXPECT_EQ(rect.y(), packet->format().y());
+        EXPECT_EQ(rect.fLeft, packet->format().x());
+        EXPECT_EQ(rect.fTop, packet->format().y());
         EXPECT_EQ(rect.width(), packet->format().width());
         EXPECT_EQ(rect.height(), packet->format().height());
       }
@@ -93,7 +95,7 @@ class EncoderMessageTester {
     strict_ = strict;
   }
 
-  void AddRects(const gfx::Rect* rects, int count) {
+  void AddRects(const SkIRect* rects, int count) {
     rects_.insert(rects_.begin() + rects_.size(), rects, rects + count);
     added_rects_ += count;
   }
@@ -111,7 +113,7 @@ class EncoderMessageTester {
   State state_;
   bool strict_;
 
-  std::deque<gfx::Rect> rects_;
+  std::deque<SkIRect> rects_;
 
   DISALLOW_COPY_AND_ASSIGN(EncoderMessageTester);
 };
@@ -121,17 +123,17 @@ class DecoderTester {
   DecoderTester(Decoder* decoder)
       : strict_(false),
         decoder_(decoder) {
-    media::VideoFrame::CreateFrame(media::VideoFrame::RGB32,
-                                   kWidth, kHeight,
-                                   base::TimeDelta(),
-                                   base::TimeDelta(), &frame_);
+    frame_ = media::VideoFrame::CreateFrame(media::VideoFrame::RGB32,
+                                            kWidth, kHeight,
+                                            base::TimeDelta(),
+                                            base::TimeDelta());
     EXPECT_TRUE(frame_.get());
     decoder_->Initialize(frame_);
   }
 
   void Reset() {
-    rects_.clear();
-    update_rects_.clear();
+    expected_region_.setEmpty();
+    update_region_.setEmpty();
   }
 
   void ReceivedPacket(VideoPacket* packet) {
@@ -140,7 +142,7 @@ class DecoderTester {
     ASSERT_NE(Decoder::DECODE_ERROR, result);
 
     if (result == Decoder::DECODE_DONE) {
-      decoder_->GetUpdatedRects(&update_rects_);
+      decoder_->GetUpdatedRegion(&update_region_);
     }
   }
 
@@ -152,8 +154,10 @@ class DecoderTester {
     capture_data_ = data;
   }
 
-  void AddRects(const gfx::Rect* rects, int count) {
-    rects_.insert(rects_.begin() + rects_.size(), rects, rects + count);
+  void AddRects(const SkIRect* rects, int count) {
+    SkRegion new_rects;
+    new_rects.setRects(rects, count);
+    expected_region_.op(new_rects, SkRegion::kUnion_Op);
   }
 
   void VerifyResults() {
@@ -162,20 +166,17 @@ class DecoderTester {
 
     ASSERT_TRUE(capture_data_.get());
 
-    // Test the content of the update rect.
-    ASSERT_EQ(rects_.size(), update_rects_.size());
-    for (size_t i = 0; i < update_rects_.size(); ++i) {
-      gfx::Rect rect = rects_[i];
-      EXPECT_EQ(rect, update_rects_[i]);
-
+    // Test the content of the update region.
+    EXPECT_EQ(expected_region_, update_region_);
+    for (SkRegion::Iterator i(update_region_); !i.done(); i.next()) {
       EXPECT_EQ(frame_->stride(0), capture_data_->data_planes().strides[0]);
       const int stride = frame_->stride(0);
-      const int offset =  stride * update_rects_[i].y() +
-          kBytesPerPixel * update_rects_[i].x();
+      const int offset =  stride * i.rect().top() +
+          kBytesPerPixel * i.rect().left();
       const uint8* original = capture_data_->data_planes().data[0] + offset;
       const uint8* decoded = frame_->data(0) + offset;
-      const int row_size = kBytesPerPixel * update_rects_[i].width();
-      for (int y = 0; y < update_rects_[i].height(); ++y) {
+      const int row_size = kBytesPerPixel * i.rect().width();
+      for (int y = 0; y < i.rect().height(); ++y) {
         EXPECT_EQ(0, memcmp(original, decoded, row_size))
             << "Row " << y << " is different";
         original += stride;
@@ -186,8 +187,8 @@ class DecoderTester {
 
  private:
   bool strict_;
-  std::deque<gfx::Rect> rects_;
-  UpdatedRects update_rects_;
+  SkRegion expected_region_;
+  SkRegion update_region_;
   Decoder* decoder_;
   scoped_refptr<media::VideoFrame> frame_;
   scoped_refptr<CaptureData> capture_data_;
@@ -221,7 +222,7 @@ class EncoderTester {
     delete packet;
   }
 
-  void AddRects(const gfx::Rect* rects, int count) {
+  void AddRects(const SkIRect* rects, int count) {
     message_tester_->AddRects(rects, count);
   }
 
@@ -256,22 +257,22 @@ scoped_refptr<CaptureData> PrepareEncodeData(media::VideoFrame::Format format,
   planes.strides[0] = kWidth * kBytesPerPixel;
 
   scoped_refptr<CaptureData> data =
-      new CaptureData(planes, kWidth, kHeight, format);
+      new CaptureData(planes, SkISize::Make(kWidth, kHeight), format);
   return data;
 }
 
 static void TestEncodingRects(Encoder* encoder,
                               EncoderTester* tester,
                               scoped_refptr<CaptureData> data,
-                              const gfx::Rect* rects, int count) {
-  data->mutable_dirty_rects().clear();
+                              const SkIRect* rects, int count) {
+  data->mutable_dirty_region().setEmpty();
   for (int i = 0; i < count; ++i) {
-    data->mutable_dirty_rects().insert(rects[i]);
+    data->mutable_dirty_region().op(rects[i], SkRegion::kUnion_Op);
   }
   tester->AddRects(rects, count);
 
-  encoder->Encode(data, true,
-                  NewCallback(tester, &EncoderTester::DataAvailable));
+  encoder->Encode(data, true, base::Bind(
+      &EncoderTester::DataAvailable, base::Unretained(tester)));
 }
 
 void TestEncoder(Encoder* encoder, bool strict) {
@@ -283,44 +284,40 @@ void TestEncoder(Encoder* encoder, bool strict) {
   uint8* memory;
   scoped_refptr<CaptureData> data =
       PrepareEncodeData(media::VideoFrame::RGB32, &memory);
+  scoped_array<uint8> memory_wrapper(memory);
 
   TestEncodingRects(encoder, &tester, data, kTestRects, 1);
   TestEncodingRects(encoder, &tester, data, kTestRects + 1, 1);
   TestEncodingRects(encoder, &tester, data, kTestRects + 2, 1);
   TestEncodingRects(encoder, &tester, data, kTestRects + 3, 2);
-  delete [] memory;
 }
 
-static void TestEncodingRects(Encoder* encoder,
-                              EncoderTester* encoder_tester,
-                              DecoderTester* decoder_tester,
-                              scoped_refptr<CaptureData> data,
-                              const gfx::Rect* rects, int count) {
-  data->mutable_dirty_rects().clear();
-  for (int i = 0; i < count; ++i) {
-    data->mutable_dirty_rects().insert(rects[i]);
-  }
+static void TestEncodeDecodeRects(Encoder* encoder,
+                                  EncoderTester* encoder_tester,
+                                  DecoderTester* decoder_tester,
+                                  scoped_refptr<CaptureData> data,
+                                  const SkIRect* rects, int count) {
+  data->mutable_dirty_region().setRects(rects, count);
   encoder_tester->AddRects(rects, count);
   decoder_tester->AddRects(rects, count);
 
-  // Generate random data for the updated rects.
+  // Generate random data for the updated region.
   srand(0);
   for (int i = 0; i < count; ++i) {
-    const gfx::Rect rect = rects[i];
     const int bytes_per_pixel = GetBytesPerPixel(data->pixel_format());
-    const int row_size = bytes_per_pixel * rect.width();
+    const int row_size = bytes_per_pixel * rects[i].width();
     uint8* memory = data->data_planes().data[0] +
-                    data->data_planes().strides[0] * rect.y() +
-                    bytes_per_pixel * rect.x();
-    for (int y = 0; y < rect.height(); ++y) {
+      data->data_planes().strides[0] * rects[i].top() +
+      bytes_per_pixel * rects[i].left();
+    for (int y = 0; y < rects[i].height(); ++y) {
       for (int x = 0; x < row_size; ++x)
         memory[x] = rand() % 256;
       memory += data->data_planes().strides[0];
     }
   }
 
-  encoder->Encode(data, true,
-                  NewCallback(encoder_tester, &EncoderTester::DataAvailable));
+  encoder->Encode(data, true, base::Bind(&EncoderTester::DataAvailable,
+                                         base::Unretained(encoder_tester)));
   decoder_tester->VerifyResults();
   decoder_tester->Reset();
 }
@@ -334,22 +331,21 @@ void TestEncoderDecoder(Encoder* encoder, Decoder* decoder, bool strict) {
   uint8* memory;
   scoped_refptr<CaptureData> data =
       PrepareEncodeData(media::VideoFrame::RGB32, &memory);
+  scoped_array<uint8> memory_wrapper(memory);
+
   DecoderTester decoder_tester(decoder);
   decoder_tester.set_strict(strict);
   decoder_tester.set_capture_data(data);
   encoder_tester.set_decoder_tester(&decoder_tester);
 
-  TestEncodingRects(encoder, &encoder_tester, &decoder_tester, data,
-                    kTestRects, 1);
-  TestEncodingRects(encoder, &encoder_tester, &decoder_tester, data,
-                    kTestRects + 1, 1);
-  TestEncodingRects(encoder, &encoder_tester, &decoder_tester, data,
-                    kTestRects + 2, 1);
-  TestEncodingRects(encoder, &encoder_tester, &decoder_tester, data,
-                    kTestRects + 3, 2);
-  delete [] memory;
+  TestEncodeDecodeRects(encoder, &encoder_tester, &decoder_tester, data,
+                        kTestRects, 1);
+  TestEncodeDecodeRects(encoder, &encoder_tester, &decoder_tester, data,
+                        kTestRects + 1, 1);
+  TestEncodeDecodeRects(encoder, &encoder_tester, &decoder_tester, data,
+                        kTestRects + 2, 1);
+  TestEncodeDecodeRects(encoder, &encoder_tester, &decoder_tester, data,
+                        kTestRects + 3, 2);
 }
 
 }  // namespace remoting
-
-DISABLE_RUNNABLE_METHOD_REFCOUNT(remoting::DecoderTester);
