@@ -1,9 +1,10 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "remoting/base/decoder_row_based.h"
 
+#include "base/logging.h"
 #include "remoting/base/decompressor.h"
 #include "remoting/base/decompressor_zlib.h"
 #include "remoting/base/decompressor_verbatim.h"
@@ -29,14 +30,11 @@ DecoderRowBased* DecoderRowBased::CreateVerbatimDecoder() {
 DecoderRowBased::DecoderRowBased(Decompressor* decompressor,
                                  VideoPacketFormat::Encoding encoding)
     : state_(kUninitialized),
+      clip_(SkIRect::MakeEmpty()),
       decompressor_(decompressor),
       encoding_(encoding),
       row_pos_(0),
-      row_y_(0),
-      // TODO(hclam): We should use the information from the update stream
-      // to determine whether we should reverse the rows or not.
-      // But for simplicity we set to be always true.
-      reverse_rows_(true) {
+      row_y_(0) {
 }
 
 DecoderRowBased::~DecoderRowBased() {
@@ -46,7 +44,7 @@ void DecoderRowBased::Reset() {
   frame_ = NULL;
   decompressor_->Reset();
   state_ = kUninitialized;
-  updated_rects_.clear();
+  updated_region_.setEmpty();
 }
 
 bool DecoderRowBased::IsReadyForData() {
@@ -92,16 +90,8 @@ Decoder::DecodeResult DecoderRowBased::DecodePacket(const VideoPacket* packet) {
   int stride = frame_->stride(media::VideoFrame::kRGBPlane);
   uint8* rect_begin = frame_->data(media::VideoFrame::kRGBPlane);
 
-  if (reverse_rows_) {
-    // Advance the pointer to the last row.
-    rect_begin += (frame_->height() - 1) * stride;
-
-    // And then make the stride negative.
-    stride = -stride;
-  }
-
-  uint8* out = rect_begin + stride * (clip_.y() + row_y_) +
-      kBytesPerPixel * clip_.x();
+  uint8* out = rect_begin + stride * (clip_.fTop + row_y_) +
+      kBytesPerPixel * clip_.fLeft;
 
   // Consume all the data in the message.
   bool decompress_again = true;
@@ -138,7 +128,7 @@ Decoder::DecodeResult DecoderRowBased::DecodePacket(const VideoPacket* packet) {
       return DECODE_ERROR;
     }
 
-    updated_rects_.push_back(clip_);
+    updated_region_.op(clip_, SkRegion::kUnion_Op);
     decompressor_->Reset();
   }
 
@@ -163,7 +153,7 @@ void DecoderRowBased::UpdateStateForPacket(const VideoPacket* packet) {
     state_ = kProcessing;
 
     // Reset the buffer location status variables on the first packet.
-    clip_.SetRect(packet->format().x(), packet->format().y(),
+    clip_.setXYWH(packet->format().x(), packet->format().y(),
                   packet->format().width(), packet->format().height());
     row_pos_ = 0;
     row_y_ = 0;
@@ -196,9 +186,9 @@ void DecoderRowBased::UpdateStateForPacket(const VideoPacket* packet) {
   return;
 }
 
-void DecoderRowBased::GetUpdatedRects(UpdatedRects* rects) {
-  rects->swap(updated_rects_);
-  updated_rects_.clear();
+void DecoderRowBased::GetUpdatedRegion(SkRegion* region) {
+  region->swap(updated_region_);
+  updated_region_.setEmpty();
 }
 
 VideoPacketFormat::Encoding DecoderRowBased::Encoding() {
