@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,17 +19,12 @@ class DecoderRowBased : public Decoder {
   static DecoderRowBased* CreateVerbatimDecoder();
 
   // Decoder implementation.
-  virtual bool IsReadyForData();
-  virtual void Initialize(scoped_refptr<media::VideoFrame> frame);
-  virtual DecodeResult DecodePacket(const VideoPacket* packet);
-  virtual void GetUpdatedRects(UpdatedRects* rects);
-  virtual void Reset();
-  virtual VideoPacketFormat::Encoding Encoding();
-
-  // TODO(hclam): Should make this into the Decoder interface.
-  // TODO(ajwong): Before putting into the interface, we should decide if the
-  // Host should normalize the coordinate system.
-  void set_reverse_rows(bool reverse) { reverse_rows_ = reverse; }
+  virtual bool IsReadyForData() OVERRIDE;
+  virtual void Initialize(scoped_refptr<media::VideoFrame> frame) OVERRIDE;
+  virtual DecodeResult DecodePacket(const VideoPacket* packet) OVERRIDE;
+  virtual void GetUpdatedRegion(SkRegion* region) OVERRIDE;
+  virtual void Reset() OVERRIDE;
+  virtual VideoPacketFormat::Encoding Encoding() OVERRIDE;
 
  private:
   enum State {
@@ -51,7 +46,7 @@ class DecoderRowBased : public Decoder {
   State state_;
 
   // Keeps track of the updating rect.
-  gfx::Rect clip_;
+  SkIRect clip_;
 
   // The video frame to write to.
   scoped_refptr<media::VideoFrame> frame_;
@@ -68,10 +63,7 @@ class DecoderRowBased : public Decoder {
   // The current row in the rect that we are updaing.
   int row_y_;
 
-  // True if we should decode the image upside down.
-  bool reverse_rows_;
-
-  UpdatedRects updated_rects_;
+  SkRegion updated_region_;
 
   DISALLOW_COPY_AND_ASSIGN(DecoderRowBased);
 };
