@@ -1,11 +1,11 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef REMOTING_BASE_DECOMPRESSOR_VERBATIM_H_
 #define REMOTING_BASE_DECOMPRESSOR_VERBATIM_H_
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "remoting/base/decompressor.h"
 
 namespace remoting {
@@ -16,12 +16,12 @@ class DecompressorVerbatim : public Decompressor {
   DecompressorVerbatim();
   virtual ~DecompressorVerbatim();
 
-  virtual void Reset();
+  virtual void Reset() OVERRIDE;
 
   // Decompressor implementations.
   virtual bool Process(const uint8* input_data, int input_size,
                        uint8* output_data, int output_size,
-                       int* consumed, int* written);
+                       int* consumed, int* written) OVERRIDE;
 };
 
 }  // namespace remoting
