@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -44,6 +44,9 @@ int32_t PPP_InitializeModule(PP_Module module_id,
     return PP_ERROR_FAILED;
   }
 
+  // Register a global log handler.
+  ChromotingInstance::RegisterLogMessageHandler();
+
   g_module_singleton = module;
   return PP_OK;
 }
@@ -59,4 +62,10 @@ const void* PPP_GetInterface(const char* interface_name) {
   return pp::Module::Get()->GetPluginInterface(interface_name);
 }
 
+const void* PPP_GetBrowserInterface(const char* interface_name) {
+  if (!pp::Module::Get())
+    return NULL;
+  return pp::Module::Get()->GetBrowserInterface(interface_name);
+}
+
 }  // namespace remoting
