@@ -1,19 +1,18 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "remoting/host/capturer_fake_ascii.h"
 
-#include "gfx/rect.h"
-
 namespace remoting {
 
 static const int kWidth = 32;
 static const int kHeight = 20;
 static const int kBytesPerPixel = 1;
 
-CapturerFakeAscii::CapturerFakeAscii(MessageLoop* message_loop)
-    : Capturer(message_loop) {
+CapturerFakeAscii::CapturerFakeAscii()
+    : current_buffer_(0),
+      pixel_format_(media::VideoFrame::ASCII) {
 }
 
 CapturerFakeAscii::~CapturerFakeAscii() {
@@ -22,8 +21,8 @@ CapturerFakeAscii::~CapturerFakeAscii() {
 void CapturerFakeAscii::ScreenConfigurationChanged() {
   width_ = kWidth;
   height_ = kHeight;
-  pixel_format_ = media::VideoFrame::ASCII;
   bytes_per_row_ = width_ * kBytesPerPixel;
+  pixel_format_ = media::VideoFrame::ASCII;
 
   // Create memory for the buffers.
   int buffer_size = height_ * bytes_per_row_;
@@ -32,24 +31,43 @@ void CapturerFakeAscii::ScreenConfigurationChanged() {
   }
 }
 
-void CapturerFakeAscii::CalculateInvalidRects() {
-  // Capture and invalidate the entire screen.
-  // Performing the capture here is modelled on the Windows
-  // GDI capturer.
-  GenerateImage();
-  InvalidateFullScreen();
+media::VideoFrame::Format CapturerFakeAscii::pixel_format() const {
+  return pixel_format_;
 }
 
-void CapturerFakeAscii::CaptureRects(const InvalidRects& rects,
-                                     CaptureCompletedCallback* callback) {
+void CapturerFakeAscii::ClearInvalidRegion() {
+  helper_.ClearInvalidRegion();
+}
+
+void CapturerFakeAscii::InvalidateRegion(const SkRegion& invalid_region) {
+  helper_.InvalidateRegion(invalid_region);
+}
+
+void CapturerFakeAscii::InvalidateScreen(const SkISize& size) {
+  helper_.InvalidateScreen(size);
+}
+
+void CapturerFakeAscii::InvalidateFullScreen() {
+  helper_.InvalidateFullScreen();
+}
+
+void CapturerFakeAscii::CaptureInvalidRegion(
+    const CaptureCompletedCallback& callback) {
+  GenerateImage();
   DataPlanes planes;
   planes.data[0] = buffers_[current_buffer_].get();
+  current_buffer_ = (current_buffer_ + 1) % kNumBuffers;
   planes.strides[0] = bytes_per_row_;
-  scoped_refptr<CaptureData> capture_data(new CaptureData(planes,
-                                                          width_,
-                                                          height_,
-                                                          pixel_format_));
-  FinishCapture(capture_data, callback);
+  scoped_refptr<CaptureData> capture_data(new CaptureData(
+      planes, SkISize::Make(width_, height_), pixel_format_));
+
+  helper_.set_size_most_recent(capture_data->size());
+
+  callback.Run(capture_data);
+}
+
+const SkISize& CapturerFakeAscii::size_most_recent() const {
+  return helper_.size_most_recent();
 }
 
 void CapturerFakeAscii::GenerateImage() {
