@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,14 +6,18 @@
 
 #include <string>
 
+#include "base/bind.h"
 #include "base/threading/thread.h"
 #include "remoting/jingle_glue/jingle_thread.h"
 
 namespace remoting {
 
-ChromotingHostContext::ChromotingHostContext()
+ChromotingHostContext::ChromotingHostContext(
+    base::MessageLoopProxy* ui_message_loop)
     : main_thread_("ChromotingMainThread"),
-      encode_thread_("ChromotingEncodeThread") {
+      encode_thread_("ChromotingEncodeThread"),
+      desktop_thread_("ChromotingDesktopThread"),
+      ui_message_loop_(ui_message_loop) {
 }
 
 ChromotingHostContext::~ChromotingHostContext() {
@@ -24,6 +28,7 @@ void ChromotingHostContext::Start() {
   main_thread_.Start();
   encode_thread_.Start();
   jingle_thread_.Start();
+  desktop_thread_.Start();
 }
 
 void ChromotingHostContext::Stop() {
@@ -31,12 +36,17 @@ void ChromotingHostContext::Stop() {
   jingle_thread_.Stop();
   encode_thread_.Stop();
   main_thread_.Stop();
+  desktop_thread_.Stop();
 }
 
 JingleThread* ChromotingHostContext::jingle_thread() {
   return &jingle_thread_;
 }
 
+base::MessageLoopProxy* ChromotingHostContext::ui_message_loop() {
+  return ui_message_loop_;
+}
+
 MessageLoop* ChromotingHostContext::main_message_loop() {
   return main_thread_.message_loop();
 }
@@ -45,8 +55,12 @@ MessageLoop* ChromotingHostContext::encode_message_loop() {
   return encode_thread_.message_loop();
 }
 
-MessageLoop* ChromotingHostContext::network_message_loop() {
-  return jingle_thread_.message_loop();
+base::MessageLoopProxy* ChromotingHostContext::network_message_loop() {
+  return jingle_thread_.message_loop_proxy();
+}
+
+MessageLoop* ChromotingHostContext::desktop_message_loop() {
+  return desktop_thread_.message_loop();
 }
 
 }  // namespace remoting
