@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,16 +8,18 @@
 #include <string>
 
 #include "base/gtest_prod_util.h"
+#include "base/threading/platform_thread.h"
 #include "base/threading/thread.h"
 #include "remoting/jingle_glue/jingle_thread.h"
 
 namespace remoting {
 
 // A class that manages threads and running context for the chromoting host
-// process.
+// process.  This class is virtual only for testing purposes (see below).
 class ChromotingHostContext {
  public:
-  ChromotingHostContext();
+  // Create a context.
+  explicit ChromotingHostContext(base::MessageLoopProxy* ui_message_loop);
   virtual ~ChromotingHostContext();
 
   // TODO(ajwong): Move the Start/Stop methods out of this class. Then
@@ -29,10 +31,11 @@ class ChromotingHostContext {
 
   virtual JingleThread* jingle_thread();
 
-  // TODO(hclam): Change these all to MessageLoopProxy.
+  virtual base::MessageLoopProxy* ui_message_loop();
   virtual MessageLoop* main_message_loop();
   virtual MessageLoop* encode_message_loop();
-  virtual MessageLoop* network_message_loop();
+  virtual base::MessageLoopProxy* network_message_loop();
+  virtual MessageLoop* desktop_message_loop();
 
  private:
   FRIEND_TEST_ALL_PREFIXES(ChromotingHostContextTest, StartAndStop);
@@ -46,11 +49,15 @@ class ChromotingHostContext {
   // A thread that hosts all encode operations.
   base::Thread encode_thread_;
 
+  // A thread that hosts desktop integration (capture, input injection, etc)
+  // This is NOT a Chrome-style UI thread.
+  base::Thread desktop_thread_;
+
+  scoped_refptr<base::MessageLoopProxy> ui_message_loop_;
+
   DISALLOW_COPY_AND_ASSIGN(ChromotingHostContext);
 };
 
 }  // namespace remoting
 
-DISABLE_RUNNABLE_METHOD_REFCOUNT(remoting::ChromotingHostContext);
-
 #endif  // REMOTING_HOST_CHROMOTING_HOST_CONTEXT_H_
