@@ -1,18 +1,18 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "remoting/host/heartbeat_sender.h"
 
+#include "base/bind.h"
 #include "base/logging.h"
-#include "base/message_loop.h"
+#include "base/message_loop_proxy.h"
 #include "base/string_number_conversions.h"
 #include "base/time.h"
 #include "remoting/base/constants.h"
-#include "remoting/host/host_config.h"
-#include "remoting/jingle_glue/iq_request.h"
-#include "remoting/jingle_glue/jingle_client.h"
+#include "remoting/jingle_glue/iq_sender.h"
 #include "remoting/jingle_glue/jingle_thread.h"
+#include "remoting/jingle_glue/signal_strategy.h"
 #include "third_party/libjingle/source/talk/xmllite/xmlelement.h"
 #include "third_party/libjingle/source/talk/xmpp/constants.h"
 
@@ -22,6 +22,7 @@ using buzz::XmlElement;
 namespace remoting {
 
 namespace {
+
 const char kHeartbeatQueryTag[] = "heartbeat";
 const char kHostIdAttr[] = "hostid";
 const char kHeartbeatSignatureTag[] = "signature";
@@ -31,86 +32,49 @@ const char kHeartbeatResultTag[] = "heartbeat-result";
 const char kSetIntervalTag[] = "set-interval";
 
 const int64 kDefaultHeartbeatIntervalMs = 5 * 60 * 1000;  // 5 minutes.
-}
 
-HeartbeatSender::HeartbeatSender()
-    : state_(CREATED),
+}  // namespace
+
+HeartbeatSender::HeartbeatSender(
+    const std::string& host_id,
+    SignalStrategy* signal_strategy,
+    HostKeyPair* key_pair)
+    : host_id_(host_id),
+      signal_strategy_(signal_strategy),
+      key_pair_(key_pair),
       interval_ms_(kDefaultHeartbeatIntervalMs) {
+  DCHECK(signal_strategy_);
+  DCHECK(key_pair_);
+
+  signal_strategy_->AddListener(this);
+
+  // Start heartbeats if the |signal_strategy_| is already connected.
+  OnSignalStrategyStateChange(signal_strategy_->GetState());
 }
 
 HeartbeatSender::~HeartbeatSender() {
-  DCHECK(state_ == CREATED || state_ == INITIALIZED || state_ == STOPPED);
+  signal_strategy_->RemoveListener(this);
 }
 
-bool HeartbeatSender::Init(MutableHostConfig* config,
-                           JingleClient* jingle_client) {
-  DCHECK(jingle_client);
-  DCHECK(state_ == CREATED);
-
-  jingle_client_ = jingle_client;
-  config_ = config;
-
-  if (!config_->GetString(kHostIdConfigPath, &host_id_)) {
-    LOG(ERROR) << "host_id is not defined in the config.";
-    return false;
+void HeartbeatSender::OnSignalStrategyStateChange(SignalStrategy::State state) {
+  if (state == SignalStrategy::CONNECTED) {
+    iq_sender_.reset(new IqSender(signal_strategy_));
+    DoSendStanza();
+    timer_.Start(FROM_HERE, base::TimeDelta::FromMilliseconds(interval_ms_),
+                 this, &HeartbeatSender::DoSendStanza);
+  } else if (state == SignalStrategy::DISCONNECTED) {
+    request_.reset();
+    iq_sender_.reset();
+    timer_.Stop();
   }
-
-  if (!key_pair_.Load(config)) {
-    return false;
-  }
-
-  state_ = INITIALIZED;
-
-  return true;
-}
-
-void HeartbeatSender::Start() {
-  if (MessageLoop::current() != jingle_client_->message_loop()) {
-    jingle_client_->message_loop()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &HeartbeatSender::Start));
-    return;
-  }
-
-  DCHECK_EQ(INITIALIZED, state_);
-  state_ = STARTED;
-
-  request_.reset(jingle_client_->CreateIqRequest());
-  request_->set_callback(NewCallback(this, &HeartbeatSender::ProcessResponse));
-
-  jingle_client_->message_loop()->PostTask(
-      FROM_HERE, NewRunnableMethod(this, &HeartbeatSender::DoSendStanza));
-}
-
-void HeartbeatSender::Stop() {
-  if (MessageLoop::current() != jingle_client_->message_loop()) {
-    jingle_client_->message_loop()->PostTask(
-        FROM_HERE, NewRunnableMethod(this, &HeartbeatSender::Stop));
-    return;
-  }
-
-  // We may call Stop() even if we have not started.
-  if (state_ != STARTED)
-    return;
-  state_ = STOPPED;
-  request_.reset(NULL);
 }
 
 void HeartbeatSender::DoSendStanza() {
-  if (state_ == STARTED) {
-    // |jingle_client_| may be already destroyed if |state_| is set to
-    // |STOPPED|, so don't touch it here unless we are in |STARTED| state.
-    DCHECK(MessageLoop::current() == jingle_client_->message_loop());
-
-    VLOG(1) << "Sending heartbeat stanza to " << kChromotingBotJid;
-
-    request_->SendIq(buzz::STR_SET, kChromotingBotJid,
-                     CreateHeartbeatMessage());
-
-    // Schedule next heartbeat.
-    jingle_client_->message_loop()->PostDelayedTask(
-        FROM_HERE, NewRunnableMethod(this, &HeartbeatSender::DoSendStanza),
-        interval_ms_);
-  }
+  VLOG(1) << "Sending heartbeat stanza to " << kChromotingBotJid;
+  request_.reset(iq_sender_->SendIq(
+      buzz::STR_SET, kChromotingBotJid, CreateHeartbeatMessage(),
+      base::Bind(&HeartbeatSender::ProcessResponse,
+                 base::Unretained(this))));
 }
 
 void HeartbeatSender::ProcessResponse(const XmlElement* response) {
@@ -122,7 +86,7 @@ void HeartbeatSender::ProcessResponse(const XmlElement* response) {
   }
 
   // This method must only be called for error or result stanzas.
-  DCHECK_EQ(buzz::STR_RESULT, type);
+  DCHECK_EQ(std::string(buzz::STR_RESULT), type);
 
   const XmlElement* result_element =
       response->FirstNamed(QName(kChromotingXmlNamespace, kHeartbeatResultTag));
@@ -137,12 +101,25 @@ void HeartbeatSender::ProcessResponse(const XmlElement* response) {
         LOG(ERROR) << "Received invalid set-interval: "
                    << set_interval_element->Str();
       } else {
-        interval_ms_ = interval * base::Time::kMillisecondsPerSecond;
+        SetInterval(interval * base::Time::kMillisecondsPerSecond);
       }
     }
   }
 }
 
+void HeartbeatSender::SetInterval(int interval) {
+  if (interval != interval_ms_) {
+    interval_ms_ = interval;
+
+    // Restart the timer with the new interval.
+    if (timer_.IsRunning()) {
+      timer_.Stop();
+      timer_.Start(FROM_HERE, base::TimeDelta::FromMilliseconds(interval_ms_),
+                   this, &HeartbeatSender::DoSendStanza);
+    }
+  }
+}
+
 XmlElement* HeartbeatSender::CreateHeartbeatMessage() {
   XmlElement* query = new XmlElement(
       QName(kChromotingXmlNamespace, kHeartbeatQueryTag));
@@ -160,8 +137,8 @@ XmlElement* HeartbeatSender::CreateSignature() {
   signature_tag->AddAttr(
       QName(kChromotingXmlNamespace, kSignatureTimeAttr), time_str);
 
-  std::string message = jingle_client_->GetFullJid() + ' ' + time_str;
-  std::string signature(key_pair_.GetSignature(message));
+  std::string message = signal_strategy_->GetLocalJid() + ' ' + time_str;
+  std::string signature(key_pair_->GetSignature(message));
   signature_tag->AddText(signature);
 
   return signature_tag;
