@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,18 +7,27 @@
 
 #include <string>
 
-#include "base/scoped_ptr.h"
-#include "base/ref_counted.h"
+#include "base/compiler_specific.h"
+#include "base/gtest_prod_util.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
+#include "base/timer.h"
 #include "remoting/host/host_key_pair.h"
-#include "remoting/jingle_glue/iq_request.h"
-#include "testing/gtest/include/gtest/gtest_prod.h"
+#include "remoting/jingle_glue/signal_strategy.h"
+
+namespace base {
+class MessageLoopProxy;
+}  // namespace base
+
+namespace buzz {
+class XmlElement;
+}  // namespace buzz
 
 namespace remoting {
 
-class IqRequest;
 class HostKeyPair;
-class JingleClient;
-class MutableHostConfig;
+class IqRequest;
+class IqSender;
 
 // HeartbeatSender periodically sends heartbeat stanzas to the Chromoting Bot.
 // Each heartbeat stanza looks as follows:
@@ -52,56 +61,41 @@ class MutableHostConfig;
 // optional. Host uses default heartbeat interval if it doesn't find
 // set-interval tag in the result Iq stanza it receives from the
 // server.
-//
-// TODO(sergeyu): Is it enough to sign JID and nothing else?
-class HeartbeatSender : public base::RefCountedThreadSafe<HeartbeatSender> {
+class HeartbeatSender : public SignalStrategy::Listener {
  public:
-  HeartbeatSender();
+  // Doesn't take ownership of |signal_strategy| or |key_pair|. Both
+  // parameters must outlive this object. Heartbeats will start when
+  // the supplied SignalStrategy enters the CONNECTED state.
+  HeartbeatSender(const std::string& host_id,
+                  SignalStrategy* signal_strategy,
+                  HostKeyPair* key_pair);
   virtual ~HeartbeatSender();
 
-  // Initializes heart-beating for |jingle_client| with the specified
-  // config. Returns false if the config is invalid (e.g. private key
-  // cannot be parsed).
-  bool Init(MutableHostConfig* config, JingleClient* jingle_client);
-
-  // Starts heart-beating. Must be called after init.
-  void Start();
-
-  // Stops heart-beating. Must be called before corresponding JingleClient
-  // is destroyed. This object will not be deleted until Stop() is called,
-  // and it may (and will) crash after JingleClient is destroyed. Heartbeating
-  // cannot be restarted after it has been stopped, A new sender must be created
-  // instead.
-  void Stop();
+  // SignalStrategy::Listener interface.
+  virtual void OnSignalStrategyStateChange(
+      SignalStrategy::State state) OVERRIDE;
 
  private:
   FRIEND_TEST_ALL_PREFIXES(HeartbeatSenderTest, DoSendStanza);
   FRIEND_TEST_ALL_PREFIXES(HeartbeatSenderTest, CreateHeartbeatMessage);
   FRIEND_TEST_ALL_PREFIXES(HeartbeatSenderTest, ProcessResponse);
 
-  enum State {
-    CREATED,
-    INITIALIZED,
-    STARTED,
-    STOPPED,
-  };
-
   void DoSendStanza();
+  void ProcessResponse(const buzz::XmlElement* response);
+  void SetInterval(int interval);
 
   // Helper methods used by DoSendStanza() to generate heartbeat stanzas.
   // Caller owns the result.
   buzz::XmlElement* CreateHeartbeatMessage();
   buzz::XmlElement* CreateSignature();
 
-  void ProcessResponse(const buzz::XmlElement* response);
-
-  State state_;
-  scoped_refptr<MutableHostConfig> config_;
-  JingleClient* jingle_client_;
-  scoped_ptr<IqRequest> request_;
   std::string host_id_;
-  HostKeyPair key_pair_;
+  SignalStrategy* signal_strategy_;
+  HostKeyPair* key_pair_;
+  scoped_ptr<IqSender> iq_sender_;
+  scoped_ptr<IqRequest> request_;
   int interval_ms_;
+  base::RepeatingTimer<HeartbeatSender> timer_;
 
   DISALLOW_COPY_AND_ASSIGN(HeartbeatSender);
 };
