@@ -1,19 +1,20 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "remoting/host/heartbeat_sender.h"
+
+#include <set>
+
+#include "base/memory/ref_counted.h"
 #include "base/message_loop.h"
 #include "base/message_loop_proxy.h"
-#include "base/ref_counted.h"
 #include "base/string_number_conversions.h"
 #include "remoting/base/constants.h"
-#include "remoting/host/heartbeat_sender.h"
 #include "remoting/host/host_key_pair.h"
-#include "remoting/host/in_memory_host_config.h"
 #include "remoting/host/test_key_pair.h"
-#include "remoting/jingle_glue/iq_request.h"
-#include "remoting/jingle_glue/jingle_client.h"
-#include "remoting/jingle_glue/jingle_thread.h"
+#include "remoting/jingle_glue/iq_sender.h"
+#include "remoting/jingle_glue/mock_objects.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/libjingle/source/talk/xmllite/xmlelement.h"
@@ -25,8 +26,10 @@ using buzz::XmlElement;
 using testing::_;
 using testing::DeleteArg;
 using testing::DoAll;
+using testing::Invoke;
 using testing::NotNull;
 using testing::Return;
+using testing::SaveArg;
 
 namespace remoting {
 
@@ -34,74 +37,77 @@ namespace {
 const char kHostId[] = "0";
 const char kTestJid[] = "user@gmail.com/chromoting123";
 const int64 kTestTime = 123123123;
+const char kStanzaId[] = "123";
 }  // namespace
 
-class MockJingleClient : public JingleClient {
- public:
-  explicit MockJingleClient(JingleThread* thread) : JingleClient(thread) { }
-  MOCK_METHOD0(CreateIqRequest, IqRequest*());
-};
-
-class MockIqRequest : public IqRequest {
- public:
-  explicit MockIqRequest(JingleClient* jingle_client)
-      : IqRequest(jingle_client) {
-  }
-  MOCK_METHOD3(SendIq, void(const std::string& type,
-                            const std::string& addressee,
-                            XmlElement* iq_body));
-};
+ACTION_P(AddListener, list) {
+  list->insert(arg0);
+}
+ACTION_P(RemoveListener, list) {
+  EXPECT_TRUE(list->find(arg0) != list->end());
+  list->erase(arg0);
+}
 
 class HeartbeatSenderTest : public testing::Test {
  protected:
-  virtual void SetUp() {
-    config_ = new InMemoryHostConfig();
-    config_->SetString(kHostIdConfigPath, kHostId);
-    config_->SetString(kPrivateKeyConfigPath, kTestHostKeyPair);
+  virtual void SetUp() OVERRIDE {
+    ASSERT_TRUE(key_pair_.LoadFromString(kTestHostKeyPair));
 
-    jingle_thread_.message_loop_ = &message_loop_;
+    EXPECT_CALL(signal_strategy_, GetState())
+        .WillOnce(Return(SignalStrategy::DISCONNECTED));
+    EXPECT_CALL(signal_strategy_, AddListener(NotNull()))
+        .WillRepeatedly(AddListener(&signal_strategy_listeners_));
+    EXPECT_CALL(signal_strategy_, RemoveListener(NotNull()))
+        .WillRepeatedly(RemoveListener(&signal_strategy_listeners_));
+    EXPECT_CALL(signal_strategy_, GetLocalJid())
+        .WillRepeatedly(Return(kTestJid));
 
-    jingle_client_ = new MockJingleClient(&jingle_thread_);
-    jingle_client_->full_jid_ = kTestJid;
+    heartbeat_sender_.reset(
+        new HeartbeatSender(kHostId, &signal_strategy_, &key_pair_));
+  }
+
+  virtual void TearDown() OVERRIDE {
+    heartbeat_sender_.reset();
+    EXPECT_TRUE(signal_strategy_listeners_.empty());
   }
 
-  JingleThread jingle_thread_;
-  scoped_refptr<MockJingleClient> jingle_client_;
   MessageLoop message_loop_;
-  scoped_refptr<InMemoryHostConfig> config_;
+  MockSignalStrategy signal_strategy_;
+  std::set<SignalStrategy::Listener*> signal_strategy_listeners_;
+  HostKeyPair key_pair_;
+  scoped_ptr<HeartbeatSender> heartbeat_sender_;
 };
 
 // Call Start() followed by Stop(), and makes sure an Iq stanza is
-// being send.
+// being sent.
 TEST_F(HeartbeatSenderTest, DoSendStanza) {
-  // |iq_request| is freed by HeartbeatSender.
-  MockIqRequest* iq_request = new MockIqRequest(jingle_client_);
+  XmlElement* sent_iq = NULL;
+  EXPECT_CALL(signal_strategy_, GetLocalJid())
+      .WillRepeatedly(Return(kTestJid));
+  EXPECT_CALL(signal_strategy_, GetNextId())
+      .WillOnce(Return(kStanzaId));
+  EXPECT_CALL(signal_strategy_, SendStanza(NotNull()))
+      .WillOnce(DoAll(SaveArg<0>(&sent_iq), Return(true)));
 
-  scoped_refptr<HeartbeatSender> heartbeat_sender(new HeartbeatSender());
-  ASSERT_TRUE(heartbeat_sender->Init(config_, jingle_client_));
-
-  EXPECT_CALL(*jingle_client_, CreateIqRequest())
-      .WillOnce(Return(iq_request));
-
-  EXPECT_CALL(*iq_request, SendIq(buzz::STR_SET, kChromotingBotJid, NotNull()))
-      .WillOnce(DoAll(DeleteArg<2>(), Return()));
-
-  heartbeat_sender->Start();
+  heartbeat_sender_->OnSignalStrategyStateChange(SignalStrategy::CONNECTED);
   message_loop_.RunAllPending();
 
-  heartbeat_sender->Stop();
+  scoped_ptr<XmlElement> stanza(sent_iq);
+  ASSERT_TRUE(stanza != NULL);
+
+  EXPECT_EQ(stanza->Attr(buzz::QName("", "to")),
+            std::string(kChromotingBotJid));
+  EXPECT_EQ(stanza->Attr(buzz::QName("", "type")), "set");
+
+  heartbeat_sender_->OnSignalStrategyStateChange(SignalStrategy::DISCONNECTED);
   message_loop_.RunAllPending();
 }
 
 // Validate format of the heartbeat stanza.
 TEST_F(HeartbeatSenderTest, CreateHeartbeatMessage) {
-  scoped_refptr<HeartbeatSender> heartbeat_sender(new HeartbeatSender());
-  ASSERT_TRUE(heartbeat_sender->Init(config_, jingle_client_));
-
   int64 start_time = static_cast<int64>(base::Time::Now().ToDoubleT());
 
-  scoped_ptr<XmlElement> stanza(
-      heartbeat_sender->CreateHeartbeatMessage());
+  scoped_ptr<XmlElement> stanza(heartbeat_sender_->CreateHeartbeatMessage());
   ASSERT_TRUE(stanza.get() != NULL);
 
   EXPECT_TRUE(QName(kChromotingXmlNamespace, "heartbeat") ==
@@ -131,7 +137,7 @@ TEST_F(HeartbeatSenderTest, CreateHeartbeatMessage) {
 
 // Verify that ProcessResponse parses set-interval result.
 TEST_F(HeartbeatSenderTest, ProcessResponse) {
-  scoped_ptr<XmlElement> response(new XmlElement(QName("", "iq")));
+  scoped_ptr<XmlElement> response(new XmlElement(buzz::QN_IQ));
   response->AddAttr(QName("", "type"), "result");
 
   XmlElement* result = new XmlElement(
@@ -145,10 +151,9 @@ TEST_F(HeartbeatSenderTest, ProcessResponse) {
   const int kTestInterval = 123;
   set_interval->AddText(base::IntToString(kTestInterval));
 
-  scoped_refptr<HeartbeatSender> heartbeat_sender(new HeartbeatSender());
-  heartbeat_sender->ProcessResponse(response.get());
+  heartbeat_sender_->ProcessResponse(response.get());
 
-  EXPECT_EQ(kTestInterval * 1000, heartbeat_sender->interval_ms_);
+  EXPECT_EQ(kTestInterval * 1000, heartbeat_sender_->interval_ms_);
 }
 
 }  // namespace remoting
