@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2012 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,9 +7,7 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
-
-class Task;
+#include "base/memory/ref_counted.h"
 
 namespace remoting {
 
@@ -19,21 +17,25 @@ namespace remoting {
 extern const char kHostEnabledConfigPath[];
 // Login used to authenticate in XMPP network.
 extern const char kXmppLoginConfigPath[];
-// Auth token used to authenticate in XMPP network.
+// Auth token used to authenticate to XMPP network.
 extern const char kXmppAuthTokenConfigPath[];
+// Auth service used to authenticate to XMPP network.
+extern const char kXmppAuthServiceConfigPath[];
 // Unique identifier of the host used to register the host in directory.
 // Normally a random UUID.
 extern const char kHostIdConfigPath[];
 // Readable host name.
 extern const char kHostNameConfigPath[];
+// Hash of the host secret used for authentication.
+extern const char kHostSecretHashConfigPath[];
 // Private keys used for host authentication.
 extern const char kPrivateKeyConfigPath[];
 
 // HostConfig interace provides read-only access to host configuration.
 class HostConfig : public base::RefCountedThreadSafe<HostConfig> {
  public:
-  HostConfig() { };
-  virtual ~HostConfig() { }
+  HostConfig() {}
+  virtual ~HostConfig() {}
 
   virtual bool GetString(const std::string& path, std::string* out_value) = 0;
   virtual bool GetBoolean(const std::string& path, bool* out_value) = 0;
@@ -44,7 +46,7 @@ class HostConfig : public base::RefCountedThreadSafe<HostConfig> {
 // MutableHostConfig extends HostConfig for mutability.
 class MutableHostConfig : public HostConfig {
  public:
-  MutableHostConfig() { };
+  MutableHostConfig() {}
 
   // SetString() updates specified config value. Save() must be called to save
   // the changes on the disk.
