@@ -1,17 +1,20 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "remoting/host/host_key_pair.h"
 
+#include <limits>
 #include <string>
 #include <vector>
 
 #include "base/base64.h"
-#include "base/crypto/rsa_private_key.h"
-#include "base/crypto/signature_creator.h"
 #include "base/logging.h"
-#include "base/task.h"
+#include "base/rand_util.h"
+#include "base/time.h"
+#include "crypto/rsa_private_key.h"
+#include "crypto/signature_creator.h"
+#include "net/base/x509_certificate.h"
 #include "remoting/host/host_config.h"
 
 namespace remoting {
@@ -21,7 +24,7 @@ HostKeyPair::HostKeyPair() { }
 HostKeyPair::~HostKeyPair() { }
 
 void HostKeyPair::Generate() {
-  key_.reset(base::RSAPrivateKey::Create(2048));
+  key_.reset(crypto::RSAPrivateKey::Create(2048));
 }
 
 bool HostKeyPair::LoadFromString(const std::string& key_base64) {
@@ -32,7 +35,7 @@ bool HostKeyPair::LoadFromString(const std::string& key_base64) {
   }
 
   std::vector<uint8> key_buf(key_str.begin(), key_str.end());
-  key_.reset(base::RSAPrivateKey::CreateFromPrivateKeyInfo(key_buf));
+  key_.reset(crypto::RSAPrivateKey::CreateFromPrivateKeyInfo(key_buf));
   if (key_.get() == NULL) {
     LOG(ERROR) << "Invalid private key.";
     return false;
@@ -72,8 +75,8 @@ std::string HostKeyPair::GetPublicKey() const {
 }
 
 std::string HostKeyPair::GetSignature(const std::string& message) const {
-  scoped_ptr<base::SignatureCreator> signature_creator(
-      base::SignatureCreator::Create(key_.get()));
+  scoped_ptr<crypto::SignatureCreator> signature_creator(
+      crypto::SignatureCreator::Create(key_.get()));
   signature_creator->Update(reinterpret_cast<const uint8*>(message.c_str()),
                             message.length());
   std::vector<uint8> signature_buf;
@@ -84,4 +87,23 @@ std::string HostKeyPair::GetSignature(const std::string& message) const {
   return signature_base64;
 }
 
+crypto::RSAPrivateKey* HostKeyPair::CopyPrivateKey() const {
+  std::vector<uint8> key_bytes;
+  CHECK(key_->ExportPrivateKey(&key_bytes));
+  return crypto::RSAPrivateKey::CreateFromPrivateKeyInfo(key_bytes);
+}
+
+std::string HostKeyPair::GenerateCertificate() const {
+  scoped_refptr<net::X509Certificate> cert =
+      net::X509Certificate::CreateSelfSigned(
+          key_.get(), "CN=chromoting",
+          base::RandInt(1, std::numeric_limits<int>::max()),
+          base::TimeDelta::FromDays(1));
+  std::string encoded;
+  bool result = net::X509Certificate::GetDEREncoded(cert->os_cert_handle(),
+                                                    &encoded);
+  CHECK(result);
+  return encoded;
+}
+
 }  // namespace remoting
