@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,11 +7,10 @@
 
 #include <string>
 
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
-#include "base/task.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 
-namespace base {
+namespace crypto {
 class RSAPrivateKey;
 }  // namespace base
 
@@ -30,15 +29,19 @@ class HostKeyPair {
   bool Load(HostConfig* host_config);
   void Save(MutableHostConfig* host_config);
 
+  crypto::RSAPrivateKey* private_key() { return key_.get(); }
+
   std::string GetPublicKey() const;
   std::string GetSignature(const std::string& message) const;
 
+  // Make a new copy of private key. Caller will own the generated private key.
+  crypto::RSAPrivateKey* CopyPrivateKey() const;
+  std::string GenerateCertificate() const;
+
  private:
-  scoped_ptr<base::RSAPrivateKey> key_;
+  scoped_ptr<crypto::RSAPrivateKey> key_;
 };
 
 }  // namespace remoting
 
-DISABLE_RUNNABLE_METHOD_REFCOUNT(remoting::HostKeyPair);
-
 #endif  // REMOTING_HOST_HOST_KEY_PAIR_H_
