@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,13 +8,10 @@
 #include <string>
 
 #include "base/file_path.h"
-#include "base/ref_counted.h"
-#include "base/scoped_ptr.h"
+#include "base/memory/ref_counted.h"
+#include "base/memory/scoped_ptr.h"
 #include "remoting/host/in_memory_host_config.h"
 
-class DictionaryValue;
-class Task;
-
 namespace base {
 class MessageLoopProxy;
 }  // namespace base
@@ -31,7 +28,7 @@ class JsonHostConfig : public InMemoryHostConfig {
   virtual bool Read();
 
   // MutableHostConfig interface.
-  virtual void Save();
+  virtual void Save() OVERRIDE;
 
  private:
   void DoWrite();
