@@ -1,12 +1,12 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/file_util.h"
+#include "base/memory/ref_counted.h"
 #include "base/message_loop.h"
 #include "base/message_loop_proxy.h"
 #include "base/path_service.h"
-#include "base/ref_counted.h"
 #include "base/scoped_temp_dir.h"
 #include "remoting/host/json_host_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -27,7 +27,7 @@ const char* kTestConfig =
 class JsonHostConfigTest : public testing::Test {
  protected:
   virtual void SetUp() {
-    message_loop_proxy_ = base::MessageLoopProxy::CreateForCurrentThread();
+    message_loop_proxy_ = base::MessageLoopProxy::current();
   }
 
   static void WriteTestFile(const FilePath& filename) {
