@@ -1,11 +1,13 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "remoting/jingle_glue/ssl_socket_adapter.h"
 
+#include "base/base64.h"
 #include "base/compiler_specific.h"
 #include "base/message_loop.h"
+#include "jingle/glue/utils.h"
 #include "net/base/address_list.h"
 #include "net/base/cert_verifier.h"
 #include "net/base/host_port_pair.h"
@@ -14,7 +16,6 @@
 #include "net/base/sys_addrinfo.h"
 #include "net/socket/client_socket_factory.h"
 #include "net/url_request/url_request_context.h"
-#include "remoting/jingle_glue/utils.h"
 
 namespace remoting {
 
@@ -26,15 +27,10 @@ SSLSocketAdapter::SSLSocketAdapter(AsyncSocket* socket)
     : SSLAdapter(socket),
       ignore_bad_cert_(false),
       cert_verifier_(new net::CertVerifier()),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          connected_callback_(this, &SSLSocketAdapter::OnConnected)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          read_callback_(this, &SSLSocketAdapter::OnRead)),
-      ALLOW_THIS_IN_INITIALIZER_LIST(
-          write_callback_(this, &SSLSocketAdapter::OnWrite)),
       ssl_state_(SSLSTATE_NONE),
       read_state_(IOSTATE_NONE),
-      write_state_(IOSTATE_NONE) {
+      write_state_(IOSTATE_NONE),
+      data_transferred_(0) {
   transport_socket_ = new TransportSocket(socket, this);
 }
 
@@ -67,14 +63,17 @@ int SSLSocketAdapter::BeginSSL() {
   // are correct for us, so we don't use the config service to initialize this
   // object.
   net::SSLConfig ssl_config;
+  net::SSLClientSocketContext context;
+  context.cert_verifier = cert_verifier_.get();
+
   transport_socket_->set_addr(talk_base::SocketAddress(hostname_, 0));
   ssl_socket_.reset(
       net::ClientSocketFactory::GetDefaultFactory()->CreateSSLClientSocket(
           transport_socket_, net::HostPortPair(hostname_, 443), ssl_config,
-          NULL /* ssl_host_info */,
-          cert_verifier_.get()));
+          NULL /* ssl_host_info */, context));
 
-  int result = ssl_socket_->Connect(&connected_callback_);
+  int result = ssl_socket_->Connect(
+      base::Bind(&SSLSocketAdapter::OnConnected, base::Unretained(this)));
 
   if (result == net::ERR_IO_PENDING || result == net::OK) {
     return 0;
@@ -91,7 +90,8 @@ int SSLSocketAdapter::Send(const void* buf, size_t len) {
     scoped_refptr<net::IOBuffer> transport_buf(new net::IOBuffer(len));
     memcpy(transport_buf->data(), buf, len);
 
-    int result = ssl_socket_->Write(transport_buf, len, NULL);
+    int result = ssl_socket_->Write(transport_buf, len,
+                                    net::CompletionCallback());
     if (result == net::ERR_IO_PENDING) {
       SetError(EWOULDBLOCK);
     }
@@ -113,7 +113,9 @@ int SSLSocketAdapter::Recv(void* buf, size_t len) {
       switch (read_state_) {
         case IOSTATE_NONE: {
           transport_buf_ = new net::IOBuffer(len);
-          int result = ssl_socket_->Read(transport_buf_, len, &read_callback_);
+          int result = ssl_socket_->Read(
+              transport_buf_, len,
+              base::Bind(&SSLSocketAdapter::OnRead, base::Unretained(this)));
           if (result >= 0) {
             memcpy(buf, transport_buf_->data(), len);
           }
@@ -184,9 +186,7 @@ void SSLSocketAdapter::OnConnectEvent(talk_base::AsyncSocket* socket) {
 
 TransportSocket::TransportSocket(talk_base::AsyncSocket* socket,
                                  SSLSocketAdapter *ssl_adapter)
-    : read_callback_(NULL),
-      write_callback_(NULL),
-      read_buffer_len_(0),
+    : read_buffer_len_(0),
       write_buffer_len_(0),
       socket_(socket),
       was_used_to_convey_data_(false) {
@@ -197,7 +197,7 @@ TransportSocket::TransportSocket(talk_base::AsyncSocket* socket,
 TransportSocket::~TransportSocket() {
 }
 
-int TransportSocket::Connect(net::CompletionCallback* callback) {
+int TransportSocket::Connect(const net::CompletionCallback& callback) {
   // Connect is never called by SSLClientSocket, instead SSLSocketAdapter
   // calls Connect() on socket_ directly.
   NOTREACHED();
@@ -233,10 +233,19 @@ int TransportSocket::GetPeerAddress(net::AddressList* address) const {
   ai.ai_addr = reinterpret_cast<struct sockaddr*>(&ipv4addr);
   ai.ai_addrlen = sizeof(ipv4addr);
 
-  address->Copy(&ai, false);
+  *address = net::AddressList::CreateByCopyingFirstAddress(&ai);
   return net::OK;
 }
 
+int TransportSocket::GetLocalAddress(net::IPEndPoint* address) const {
+  talk_base::SocketAddress socket_address = socket_->GetLocalAddress();
+  if (jingle_glue::SocketAddressToIPEndPoint(socket_address, address)) {
+    return net::OK;
+  } else {
+    return net::ERR_FAILED;
+  }
+}
+
 const net::BoundNetLog& TransportSocket::NetLog() const {
   return net_log_;
 }
@@ -259,14 +268,24 @@ bool TransportSocket::UsingTCPFastOpen() const {
   return false;
 }
 
+int64 TransportSocket::NumBytesRead() const {
+  NOTREACHED();
+  return -1;
+}
+
+base::TimeDelta TransportSocket::GetConnectTimeMicros() const {
+  NOTREACHED();
+  return base::TimeDelta::FromMicroseconds(-1);
+}
+
 int TransportSocket::Read(net::IOBuffer* buf, int buf_len,
-                          net::CompletionCallback* callback) {
+                          const net::CompletionCallback& callback) {
   DCHECK(buf);
-  DCHECK(!read_callback_);
+  DCHECK(read_callback_.is_null());
   DCHECK(!read_buffer_.get());
   int result = socket_->Recv(buf->data(), buf_len);
   if (result < 0) {
-    result = MapPosixToChromeError(socket_->GetError());
+    result = net::MapSystemError(socket_->GetError());
     if (result == net::ERR_IO_PENDING) {
       read_callback_ = callback;
       read_buffer_ = buf;
@@ -279,13 +298,13 @@ int TransportSocket::Read(net::IOBuffer* buf, int buf_len,
 }
 
 int TransportSocket::Write(net::IOBuffer* buf, int buf_len,
-                           net::CompletionCallback* callback) {
+                           const net::CompletionCallback& callback) {
   DCHECK(buf);
-  DCHECK(!write_callback_);
+  DCHECK(write_callback_.is_null());
   DCHECK(!write_buffer_.get());
   int result = socket_->Send(buf->data(), buf_len);
   if (result < 0) {
-    result = MapPosixToChromeError(socket_->GetError());
+    result = net::MapSystemError(socket_->GetError());
     if (result == net::ERR_IO_PENDING) {
       write_callback_ = callback;
       write_buffer_ = buf;
@@ -308,19 +327,19 @@ bool TransportSocket::SetSendBufferSize(int32 size) {
 }
 
 void TransportSocket::OnReadEvent(talk_base::AsyncSocket* socket) {
-  if (read_callback_) {
+  if (!read_callback_.is_null()) {
     DCHECK(read_buffer_.get());
-    net::CompletionCallback* callback = read_callback_;
+    net::CompletionCallback callback = read_callback_;
     scoped_refptr<net::IOBuffer> buffer = read_buffer_;
     int buffer_len = read_buffer_len_;
 
-    read_callback_ = NULL;
+    read_callback_.Reset();
     read_buffer_ = NULL;
     read_buffer_len_ = 0;
 
     int result = socket_->Recv(buffer->data(), buffer_len);
     if (result < 0) {
-      result = MapPosixToChromeError(socket_->GetError());
+      result = net::MapSystemError(socket_->GetError());
       if (result == net::ERR_IO_PENDING) {
         read_callback_ = callback;
         read_buffer_ = buffer;
@@ -329,24 +348,24 @@ void TransportSocket::OnReadEvent(talk_base::AsyncSocket* socket) {
       }
     }
     was_used_to_convey_data_ = true;
-    callback->RunWithParams(Tuple1<int>(result));
+    callback.Run(result);
   }
 }
 
 void TransportSocket::OnWriteEvent(talk_base::AsyncSocket* socket) {
-  if (write_callback_) {
+  if (!write_callback_.is_null()) {
     DCHECK(write_buffer_.get());
-    net::CompletionCallback* callback = write_callback_;
+    net::CompletionCallback callback = write_callback_;
     scoped_refptr<net::IOBuffer> buffer = write_buffer_;
     int buffer_len = write_buffer_len_;
 
-    write_callback_ = NULL;
+    write_callback_.Reset();
     write_buffer_ = NULL;
     write_buffer_len_ = 0;
 
     int result = socket_->Send(buffer->data(), buffer_len);
     if (result < 0) {
-      result = MapPosixToChromeError(socket_->GetError());
+      result = net::MapSystemError(socket_->GetError());
       if (result == net::ERR_IO_PENDING) {
         write_callback_ = callback;
         write_buffer_ = buffer;
@@ -355,7 +374,7 @@ void TransportSocket::OnWriteEvent(talk_base::AsyncSocket* socket) {
       }
     }
     was_used_to_convey_data_ = true;
-    callback->RunWithParams(Tuple1<int>(result));
+    callback.Run(result);
   }
 }
 
