@@ -1,17 +1,17 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef REMOTING_JINGLE_GLUE_SSL_SOCKET_ADAPTER_H_
 #define REMOTING_JINGLE_GLUE_SSL_SOCKET_ADAPTER_H_
 
-#include "base/scoped_ptr.h"
+#include "base/memory/scoped_ptr.h"
 #include "net/base/completion_callback.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_log.h"
-#include "net/socket/client_socket.h"
 #include "net/socket/ssl_client_socket.h"
+#include "net/socket/stream_socket.h"
 #include "third_party/libjingle/source/talk/base/asyncsocket.h"
 #include "third_party/libjingle/source/talk/base/ssladapter.h"
 
@@ -26,40 +26,41 @@ class SSLSocketAdapter;
 // TODO(sergeyu): Write unittests for this code!
 
 // This class provides a wrapper to libjingle's talk_base::AsyncSocket that
-// implements Chromium's net::ClientSocket interface.  It's used by
+// implements Chromium's net::StreamSocket interface.  It's used by
 // SSLSocketAdapter to enable Chromium's SSL implementation to work over
 // libjingle's socket class.
-class TransportSocket : public net::ClientSocket, public sigslot::has_slots<> {
+class TransportSocket : public net::StreamSocket, public sigslot::has_slots<> {
  public:
   TransportSocket(talk_base::AsyncSocket* socket,
                   SSLSocketAdapter *ssl_adapter);
-  ~TransportSocket();
+  virtual ~TransportSocket();
 
   void set_addr(const talk_base::SocketAddress& addr) {
     addr_ = addr;
   }
 
-  // net::ClientSocket implementation
-
-  virtual int Connect(net::CompletionCallback* callback);
-  virtual void Disconnect();
-  virtual bool IsConnected() const;
-  virtual bool IsConnectedAndIdle() const;
-  virtual int GetPeerAddress(net::AddressList* address) const;
-  virtual const net::BoundNetLog& NetLog() const;
-  virtual void SetSubresourceSpeculation();
-  virtual void SetOmniboxSpeculation();
-  virtual bool WasEverUsed() const;
-  virtual bool UsingTCPFastOpen() const;
-
-  // net::Socket implementation
+  // net::StreamSocket implementation.
+  virtual int Connect(const net::CompletionCallback& callback) OVERRIDE;
+  virtual void Disconnect() OVERRIDE;
+  virtual bool IsConnected() const OVERRIDE;
+  virtual bool IsConnectedAndIdle() const OVERRIDE;
+  virtual int GetPeerAddress(net::AddressList* address) const OVERRIDE;
+  virtual int GetLocalAddress(net::IPEndPoint* address) const OVERRIDE;
+  virtual const net::BoundNetLog& NetLog() const OVERRIDE;
+  virtual void SetSubresourceSpeculation() OVERRIDE;
+  virtual void SetOmniboxSpeculation() OVERRIDE;
+  virtual bool WasEverUsed() const OVERRIDE;
+  virtual bool UsingTCPFastOpen() const OVERRIDE;
+  virtual int64 NumBytesRead() const OVERRIDE;
+  virtual base::TimeDelta GetConnectTimeMicros() const OVERRIDE;
 
+  // net::Socket implementation.
   virtual int Read(net::IOBuffer* buf, int buf_len,
-                   net::CompletionCallback* callback);
+                   const net::CompletionCallback& callback) OVERRIDE;
   virtual int Write(net::IOBuffer* buf, int buf_len,
-                    net::CompletionCallback* callback);
-  virtual bool SetReceiveBufferSize(int32 size);
-  virtual bool SetSendBufferSize(int32 size);
+                    const net::CompletionCallback& callback) OVERRIDE;
+  virtual bool SetReceiveBufferSize(int32 size) OVERRIDE;
+  virtual bool SetSendBufferSize(int32 size) OVERRIDE;
 
  private:
   friend class SSLSocketAdapter;
@@ -67,8 +68,9 @@ class TransportSocket : public net::ClientSocket, public sigslot::has_slots<> {
   void OnReadEvent(talk_base::AsyncSocket* socket);
   void OnWriteEvent(talk_base::AsyncSocket* socket);
 
-  net::CompletionCallback* read_callback_;
-  net::CompletionCallback* write_callback_;
+  // Holds the user's completion callback when Write and Read are called.
+  net::CompletionCallback read_callback_;
+  net::CompletionCallback write_callback_;
 
   scoped_refptr<net::IOBuffer> read_buffer_;
   int read_buffer_len_;
@@ -91,20 +93,20 @@ class TransportSocket : public net::ClientSocket, public sigslot::has_slots<> {
 class SSLSocketAdapter : public talk_base::SSLAdapter {
  public:
   explicit SSLSocketAdapter(talk_base::AsyncSocket* socket);
-  ~SSLSocketAdapter();
+  virtual ~SSLSocketAdapter();
 
   // StartSSL returns 0 if successful, or non-zero on failure.
   // If StartSSL is called while the socket is closed or connecting, the SSL
   // negotiation will begin as soon as the socket connects.
   //
   // restartable is not implemented, and must be set to false.
-  virtual int StartSSL(const char* hostname, bool restartable);
+  virtual int StartSSL(const char* hostname, bool restartable) OVERRIDE;
 
   // Create the default SSL adapter for this platform.
   static SSLSocketAdapter* Create(AsyncSocket* socket);
 
-  virtual int Send(const void* pv, size_t cb);
-  virtual int Recv(void* pv, size_t cb);
+  virtual int Send(const void* pv, size_t cb) OVERRIDE;
+  virtual int Recv(void* pv, size_t cb) OVERRIDE;
 
  private:
   friend class TransportSocket;
@@ -125,18 +127,19 @@ class SSLSocketAdapter : public talk_base::SSLAdapter {
   void OnRead(int result);
   void OnWrite(int result);
 
-  virtual void OnConnectEvent(talk_base::AsyncSocket* socket);
+  virtual void OnConnectEvent(talk_base::AsyncSocket* socket) OVERRIDE;
 
   int BeginSSL();
 
   bool ignore_bad_cert_;
   std::string hostname_;
   TransportSocket* transport_socket_;
-  scoped_ptr<net::SSLClientSocket> ssl_socket_;
+
+  // |cert_verifier_| must be defined before |ssl_socket_|, so that
+  // it's destroyed after |ssl_socket_|.
   scoped_ptr<net::CertVerifier> cert_verifier_;
-  net::CompletionCallbackImpl<SSLSocketAdapter> connected_callback_;
-  net::CompletionCallbackImpl<SSLSocketAdapter> read_callback_;
-  net::CompletionCallbackImpl<SSLSocketAdapter> write_callback_;
+  scoped_ptr<net::SSLClientSocket> ssl_socket_;
+
   SSLState ssl_state_;
   IOState read_state_;
   IOState write_state_;
