@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,23 +12,16 @@
 
 #include "base/basictypes.h"
 
-class Task;
-
 namespace remoting {
 namespace protocol {
 
-class LocalLoginStatus;
-class NotifyResolutionRequest;
-
 class ClientStub {
  public:
   ClientStub() {}
   virtual ~ClientStub() {}
 
-  virtual void NotifyResolution(const NotifyResolutionRequest* msg,
-                                Task* done) = 0;
-  virtual void BeginSessionResponse(const LocalLoginStatus* msg,
-                                    Task* done) = 0;
+  // Currently we don't use the control channel for anything. Add new
+  // message handlers here when necessary.
 
  private:
   DISALLOW_COPY_AND_ASSIGN(ClientStub);
